// This code conforms with the UFC specification version 1.0
// and was automatically generated by FFC version 0.4.5.
//
// Warning: This code was generated with the option '-l dolfin'
// and contains DOLFIN-specific wrappers that depend on DOLFIN.

#ifndef __POISSON3D_5_H
#define __POISSON3D_5_H

#include <cmath>
#include <stdexcept>
#include <fstream>
#include <ufc.h>

/// This class defines the interface for a finite element.

class UFC_Poisson3D_5BilinearForm_finite_element_0: public ufc::finite_element
{
public:

  /// Constructor
  UFC_Poisson3D_5BilinearForm_finite_element_0() : ufc::finite_element()
  {
    // Do nothing
  }

  /// Destructor
  virtual ~UFC_Poisson3D_5BilinearForm_finite_element_0()
  {
    // Do nothing
  }

  /// Return a string identifying the finite element
  virtual const char* signature() const
  {
    return "Lagrange finite element of degree 5 on a tetrahedron";
  }

  /// Return the cell shape
  virtual ufc::shape cell_shape() const
  {
    return ufc::tetrahedron;
  }

  /// Return the dimension of the finite element function space
  virtual unsigned int space_dimension() const
  {
    return 56;
  }

  /// Return the rank of the value space
  virtual unsigned int value_rank() const
  {
    return 0;
  }

  /// Return the dimension of the value space for axis i
  virtual unsigned int value_dimension(unsigned int i) const
  {
    return 1;
  }

  /// Evaluate basis function i at given point in cell
  virtual void evaluate_basis(unsigned int i,
                              double* values,
                              const double* coordinates,
                              const ufc::cell& c) const
  {
    // Extract vertex coordinates
    const double * const * element_coordinates = c.coordinates;
    
    // Compute Jacobian of affine map from reference cell
    const double J_00 = element_coordinates[1][0] - element_coordinates[0][0];
    const double J_01 = element_coordinates[2][0] - element_coordinates[0][0];
    const double J_02 = element_coordinates[3][0] - element_coordinates[0][0];
    const double J_10 = element_coordinates[1][1] - element_coordinates[0][1];
    const double J_11 = element_coordinates[2][1] - element_coordinates[0][1];
    const double J_12 = element_coordinates[3][1] - element_coordinates[0][1];
    const double J_20 = element_coordinates[1][2] - element_coordinates[0][2];
    const double J_21 = element_coordinates[2][2] - element_coordinates[0][2];
    const double J_22 = element_coordinates[3][2] - element_coordinates[0][2];
      
    // Compute sub determinants
    const double d00 = J_11*J_22 - J_12*J_21;
    const double d01 = J_12*J_20 - J_10*J_22;
    const double d02 = J_10*J_21 - J_11*J_20;
    
    const double d10 = J_02*J_21 - J_01*J_22;
    const double d11 = J_00*J_22 - J_02*J_20;
    const double d12 = J_01*J_20 - J_00*J_21;
    
    const double d20 = J_01*J_12 - J_02*J_11;
    const double d21 = J_02*J_10 - J_00*J_12;
    const double d22 = J_00*J_11 - J_01*J_10;
      
    // Compute determinant of Jacobian
    double detJ = J_00*d00 + J_10*d10 + J_20*d20;
    
    // Compute inverse of Jacobian
    
    // Compute constants
    const double C0 = d00*(element_coordinates[0][0] - element_coordinates[2][0] - element_coordinates[3][0]) \
                    + d10*(element_coordinates[0][1] - element_coordinates[2][1] - element_coordinates[3][1]) \
                    + d20*(element_coordinates[0][2] - element_coordinates[2][2] - element_coordinates[3][2]);
    
    const double C1 = d01*(element_coordinates[0][0] - element_coordinates[1][0] - element_coordinates[3][0]) \
                    + d11*(element_coordinates[0][1] - element_coordinates[1][1] - element_coordinates[3][1]) \
                    + d21*(element_coordinates[0][2] - element_coordinates[1][2] - element_coordinates[3][2]);
    
    const double C2 = d02*(element_coordinates[0][0] - element_coordinates[1][0] - element_coordinates[2][0]) \
                    + d12*(element_coordinates[0][1] - element_coordinates[1][1] - element_coordinates[2][1]) \
                    + d22*(element_coordinates[0][2] - element_coordinates[1][2] - element_coordinates[2][2]);
    
    // Get coordinates and map to the UFC reference element
    double x = (C0 + d00*coordinates[0] + d10*coordinates[1] + d20*coordinates[2]) / detJ;
    double y = (C1 + d01*coordinates[0] + d11*coordinates[1] + d21*coordinates[2]) / detJ;
    double z = (C2 + d02*coordinates[0] + d12*coordinates[1] + d22*coordinates[2]) / detJ;
    
    // Map coordinates to the reference cube
    if (std::abs(y + z - 1.0) < 1e-14)
      x = 1.0;
    else
      x = -2.0 * x/(y + z - 1.0) - 1.0;
    if (std::abs(z - 1.0) < 1e-14)
      y = -1.0;
    else
      y = 2.0 * y/(1.0 - z) - 1.0;
    z = 2.0 * z - 1.0;
    
    // Reset values
    *values = 0;
    
    // Map degree of freedom to element degree of freedom
    const unsigned int dof = i;
    
    // Generate scalings
    const double scalings_y_0 = 1;
    const double scalings_y_1 = scalings_y_0*(0.5 - 0.5*y);
    const double scalings_y_2 = scalings_y_1*(0.5 - 0.5*y);
    const double scalings_y_3 = scalings_y_2*(0.5 - 0.5*y);
    const double scalings_y_4 = scalings_y_3*(0.5 - 0.5*y);
    const double scalings_y_5 = scalings_y_4*(0.5 - 0.5*y);
    const double scalings_z_0 = 1;
    const double scalings_z_1 = scalings_z_0*(0.5 - 0.5*z);
    const double scalings_z_2 = scalings_z_1*(0.5 - 0.5*z);
    const double scalings_z_3 = scalings_z_2*(0.5 - 0.5*z);
    const double scalings_z_4 = scalings_z_3*(0.5 - 0.5*z);
    const double scalings_z_5 = scalings_z_4*(0.5 - 0.5*z);
    
    // Compute psitilde_a
    const double psitilde_a_0 = 1;
    const double psitilde_a_1 = x;
    const double psitilde_a_2 = 1.5*x*psitilde_a_1 - 0.5*psitilde_a_0;
    const double psitilde_a_3 = 1.66666666666667*x*psitilde_a_2 - 0.666666666666667*psitilde_a_1;
    const double psitilde_a_4 = 1.75*x*psitilde_a_3 - 0.75*psitilde_a_2;
    const double psitilde_a_5 = 1.8*x*psitilde_a_4 - 0.8*psitilde_a_3;
    
    // Compute psitilde_bs
    const double psitilde_bs_0_0 = 1;
    const double psitilde_bs_0_1 = 1.5*y + 0.5;
    const double psitilde_bs_0_2 = 0.111111111111111*psitilde_bs_0_1 + 1.66666666666667*y*psitilde_bs_0_1 - 0.555555555555556*psitilde_bs_0_0;
    const double psitilde_bs_0_3 = 0.05*psitilde_bs_0_2 + 1.75*y*psitilde_bs_0_2 - 0.7*psitilde_bs_0_1;
    const double psitilde_bs_0_4 = 0.0285714285714286*psitilde_bs_0_3 + 1.8*y*psitilde_bs_0_3 - 0.771428571428571*psitilde_bs_0_2;
    const double psitilde_bs_0_5 = 0.0185185185185185*psitilde_bs_0_4 + 1.83333333333333*y*psitilde_bs_0_4 - 0.814814814814815*psitilde_bs_0_3;
    const double psitilde_bs_1_0 = 1;
    const double psitilde_bs_1_1 = 2.5*y + 1.5;
    const double psitilde_bs_1_2 = 0.54*psitilde_bs_1_1 + 2.1*y*psitilde_bs_1_1 - 0.56*psitilde_bs_1_0;
    const double psitilde_bs_1_3 = 0.285714285714286*psitilde_bs_1_2 + 2*y*psitilde_bs_1_2 - 0.714285714285714*psitilde_bs_1_1;
    const double psitilde_bs_1_4 = 0.178571428571429*psitilde_bs_1_3 + 1.96428571428571*y*psitilde_bs_1_3 - 0.785714285714286*psitilde_bs_1_2;
    const double psitilde_bs_2_0 = 1;
    const double psitilde_bs_2_1 = 3.5*y + 2.5;
    const double psitilde_bs_2_2 = 1.02040816326531*psitilde_bs_2_1 + 2.57142857142857*y*psitilde_bs_2_1 - 0.551020408163265*psitilde_bs_2_0;
    const double psitilde_bs_2_3 = 0.578703703703704*psitilde_bs_2_2 + 2.29166666666667*y*psitilde_bs_2_2 - 0.712962962962963*psitilde_bs_2_1;
    const double psitilde_bs_3_0 = 1;
    const double psitilde_bs_3_1 = 4.5*y + 3.5;
    const double psitilde_bs_3_2 = 1.51234567901235*psitilde_bs_3_1 + 3.05555555555556*y*psitilde_bs_3_1 - 0.54320987654321*psitilde_bs_3_0;
    const double psitilde_bs_4_0 = 1;
    const double psitilde_bs_4_1 = 5.5*y + 4.5;
    const double psitilde_bs_5_0 = 1;
    
    // Compute psitilde_cs
    const double psitilde_cs_00_0 = 1;
    const double psitilde_cs_00_1 = 2*z + 1;
    const double psitilde_cs_00_2 = 0.3125*psitilde_cs_00_1 + 1.875*z*psitilde_cs_00_1 - 0.5625*psitilde_cs_00_0;
    const double psitilde_cs_00_3 = 0.155555555555556*psitilde_cs_00_2 + 1.86666666666667*z*psitilde_cs_00_2 - 0.711111111111111*psitilde_cs_00_1;
    const double psitilde_cs_00_4 = 0.09375*psitilde_cs_00_3 + 1.875*z*psitilde_cs_00_3 - 0.78125*psitilde_cs_00_2;
    const double psitilde_cs_00_5 = 0.0628571428571429*psitilde_cs_00_4 + 1.88571428571429*z*psitilde_cs_00_4 - 0.822857142857143*psitilde_cs_00_3;
    const double psitilde_cs_01_0 = 1;
    const double psitilde_cs_01_1 = 3*z + 2;
    const double psitilde_cs_01_2 = 0.777777777777778*psitilde_cs_01_1 + 2.33333333333333*z*psitilde_cs_01_1 - 0.555555555555556*psitilde_cs_01_0;
    const double psitilde_cs_01_3 = 0.428571428571429*psitilde_cs_01_2 + 2.14285714285714*z*psitilde_cs_01_2 - 0.714285714285714*psitilde_cs_01_1;
    const double psitilde_cs_01_4 = 0.275*psitilde_cs_01_3 + 2.0625*z*psitilde_cs_01_3 - 0.7875*psitilde_cs_01_2;
    const double psitilde_cs_02_0 = 1;
    const double psitilde_cs_02_1 = 4*z + 3;
    const double psitilde_cs_02_2 = 1.265625*psitilde_cs_02_1 + 2.8125*z*psitilde_cs_02_1 - 0.546875*psitilde_cs_02_0;
    const double psitilde_cs_02_3 = 0.733333333333333*psitilde_cs_02_2 + 2.44444444444444*z*psitilde_cs_02_2 - 0.711111111111111*psitilde_cs_02_1;
    const double psitilde_cs_03_0 = 1;
    const double psitilde_cs_03_1 = 5*z + 4;
    const double psitilde_cs_03_2 = 1.76*psitilde_cs_03_1 + 3.3*z*psitilde_cs_03_1 - 0.54*psitilde_cs_03_0;
    const double psitilde_cs_04_0 = 1;
    const double psitilde_cs_04_1 = 6*z + 5;
    const double psitilde_cs_05_0 = 1;
    const double psitilde_cs_10_0 = 1;
    const double psitilde_cs_10_1 = 3*z + 2;
    const double psitilde_cs_10_2 = 0.777777777777778*psitilde_cs_10_1 + 2.33333333333333*z*psitilde_cs_10_1 - 0.555555555555556*psitilde_cs_10_0;
    const double psitilde_cs_10_3 = 0.428571428571429*psitilde_cs_10_2 + 2.14285714285714*z*psitilde_cs_10_2 - 0.714285714285714*psitilde_cs_10_1;
    const double psitilde_cs_10_4 = 0.275*psitilde_cs_10_3 + 2.0625*z*psitilde_cs_10_3 - 0.7875*psitilde_cs_10_2;
    const double psitilde_cs_11_0 = 1;
    const double psitilde_cs_11_1 = 4*z + 3;
    const double psitilde_cs_11_2 = 1.265625*psitilde_cs_11_1 + 2.8125*z*psitilde_cs_11_1 - 0.546875*psitilde_cs_11_0;
    const double psitilde_cs_11_3 = 0.733333333333333*psitilde_cs_11_2 + 2.44444444444444*z*psitilde_cs_11_2 - 0.711111111111111*psitilde_cs_11_1;
    const double psitilde_cs_12_0 = 1;
    const double psitilde_cs_12_1 = 5*z + 4;
    const double psitilde_cs_12_2 = 1.76*psitilde_cs_12_1 + 3.3*z*psitilde_cs_12_1 - 0.54*psitilde_cs_12_0;
    const double psitilde_cs_13_0 = 1;
    const double psitilde_cs_13_1 = 6*z + 5;
    const double psitilde_cs_14_0 = 1;
    const double psitilde_cs_20_0 = 1;
    const double psitilde_cs_20_1 = 4*z + 3;
    const double psitilde_cs_20_2 = 1.265625*psitilde_cs_20_1 + 2.8125*z*psitilde_cs_20_1 - 0.546875*psitilde_cs_20_0;
    const double psitilde_cs_20_3 = 0.733333333333333*psitilde_cs_20_2 + 2.44444444444444*z*psitilde_cs_20_2 - 0.711111111111111*psitilde_cs_20_1;
    const double psitilde_cs_21_0 = 1;
    const double psitilde_cs_21_1 = 5*z + 4;
    const double psitilde_cs_21_2 = 1.76*psitilde_cs_21_1 + 3.3*z*psitilde_cs_21_1 - 0.54*psitilde_cs_21_0;
    const double psitilde_cs_22_0 = 1;
    const double psitilde_cs_22_1 = 6*z + 5;
    const double psitilde_cs_23_0 = 1;
    const double psitilde_cs_30_0 = 1;
    const double psitilde_cs_30_1 = 5*z + 4;
    const double psitilde_cs_30_2 = 1.76*psitilde_cs_30_1 + 3.3*z*psitilde_cs_30_1 - 0.54*psitilde_cs_30_0;
    const double psitilde_cs_31_0 = 1;
    const double psitilde_cs_31_1 = 6*z + 5;
    const double psitilde_cs_32_0 = 1;
    const double psitilde_cs_40_0 = 1;
    const double psitilde_cs_40_1 = 6*z + 5;
    const double psitilde_cs_41_0 = 1;
    const double psitilde_cs_50_0 = 1;
    
    // Compute basisvalues
    const double basisvalue0 = 0.866025403784439*psitilde_a_0*scalings_y_0*psitilde_bs_0_0*scalings_z_0*psitilde_cs_00_0;
    const double basisvalue1 = 2.73861278752583*psitilde_a_1*scalings_y_1*psitilde_bs_1_0*scalings_z_1*psitilde_cs_10_0;
    const double basisvalue2 = 1.58113883008419*psitilde_a_0*scalings_y_0*psitilde_bs_0_1*scalings_z_1*psitilde_cs_01_0;
    const double basisvalue3 = 1.11803398874989*psitilde_a_0*scalings_y_0*psitilde_bs_0_0*scalings_z_0*psitilde_cs_00_1;
    const double basisvalue4 = 5.1234753829798*psitilde_a_2*scalings_y_2*psitilde_bs_2_0*scalings_z_2*psitilde_cs_20_0;
    const double basisvalue5 = 3.96862696659689*psitilde_a_1*scalings_y_1*psitilde_bs_1_1*scalings_z_2*psitilde_cs_11_0;
    const double basisvalue6 = 2.29128784747792*psitilde_a_0*scalings_y_0*psitilde_bs_0_2*scalings_z_2*psitilde_cs_02_0;
    const double basisvalue7 = 3.24037034920393*psitilde_a_1*scalings_y_1*psitilde_bs_1_0*scalings_z_1*psitilde_cs_10_1;
    const double basisvalue8 = 1.87082869338697*psitilde_a_0*scalings_y_0*psitilde_bs_0_1*scalings_z_1*psitilde_cs_01_1;
    const double basisvalue9 = 1.3228756555323*psitilde_a_0*scalings_y_0*psitilde_bs_0_0*scalings_z_0*psitilde_cs_00_2;
    const double basisvalue10 = 7.93725393319377*psitilde_a_3*scalings_y_3*psitilde_bs_3_0*scalings_z_3*psitilde_cs_30_0;
    const double basisvalue11 = 6.70820393249937*psitilde_a_2*scalings_y_2*psitilde_bs_2_1*scalings_z_3*psitilde_cs_21_0;
    const double basisvalue12 = 5.19615242270663*psitilde_a_1*scalings_y_1*psitilde_bs_1_2*scalings_z_3*psitilde_cs_12_0;
    const double basisvalue13 = 3*psitilde_a_0*scalings_y_0*psitilde_bs_0_3*scalings_z_3*psitilde_cs_03_0;
    const double basisvalue14 = 5.80947501931113*psitilde_a_2*scalings_y_2*psitilde_bs_2_0*scalings_z_2*psitilde_cs_20_1;
    const double basisvalue15 = 4.5*psitilde_a_1*scalings_y_1*psitilde_bs_1_1*scalings_z_2*psitilde_cs_11_1;
    const double basisvalue16 = 2.59807621135332*psitilde_a_0*scalings_y_0*psitilde_bs_0_2*scalings_z_2*psitilde_cs_02_1;
    const double basisvalue17 = 3.67423461417477*psitilde_a_1*scalings_y_1*psitilde_bs_1_0*scalings_z_1*psitilde_cs_10_2;
    const double basisvalue18 = 2.12132034355964*psitilde_a_0*scalings_y_0*psitilde_bs_0_1*scalings_z_1*psitilde_cs_01_2;
    const double basisvalue19 = 1.5*psitilde_a_0*scalings_y_0*psitilde_bs_0_0*scalings_z_0*psitilde_cs_00_3;
    const double basisvalue20 = 11.1242977306435*psitilde_a_4*scalings_y_4*psitilde_bs_4_0*scalings_z_4*psitilde_cs_40_0;
    const double basisvalue21 = 9.81070843517429*psitilde_a_3*scalings_y_3*psitilde_bs_3_1*scalings_z_4*psitilde_cs_31_0;
    const double basisvalue22 = 8.2915619758885*psitilde_a_2*scalings_y_2*psitilde_bs_2_2*scalings_z_4*psitilde_cs_22_0;
    const double basisvalue23 = 6.42261628933256*psitilde_a_1*scalings_y_1*psitilde_bs_1_3*scalings_z_4*psitilde_cs_13_0;
    const double basisvalue24 = 3.70809924354783*psitilde_a_0*scalings_y_0*psitilde_bs_0_4*scalings_z_4*psitilde_cs_04_0;
    const double basisvalue25 = 8.77496438739212*psitilde_a_3*scalings_y_3*psitilde_bs_3_0*scalings_z_3*psitilde_cs_30_1;
    const double basisvalue26 = 7.41619848709566*psitilde_a_2*scalings_y_2*psitilde_bs_2_1*scalings_z_3*psitilde_cs_21_1;
    const double basisvalue27 = 5.74456264653803*psitilde_a_1*scalings_y_1*psitilde_bs_1_2*scalings_z_3*psitilde_cs_12_1;
    const double basisvalue28 = 3.3166247903554*psitilde_a_0*scalings_y_0*psitilde_bs_0_3*scalings_z_3*psitilde_cs_03_1;
    const double basisvalue29 = 6.42261628933256*psitilde_a_2*scalings_y_2*psitilde_bs_2_0*scalings_z_2*psitilde_cs_20_2;
    const double basisvalue30 = 4.9749371855331*psitilde_a_1*scalings_y_1*psitilde_bs_1_1*scalings_z_2*psitilde_cs_11_2;
    const double basisvalue31 = 2.87228132326901*psitilde_a_0*scalings_y_0*psitilde_bs_0_2*scalings_z_2*psitilde_cs_02_2;
    const double basisvalue32 = 4.06201920231798*psitilde_a_1*scalings_y_1*psitilde_bs_1_0*scalings_z_1*psitilde_cs_10_3;
    const double basisvalue33 = 2.34520787991171*psitilde_a_0*scalings_y_0*psitilde_bs_0_1*scalings_z_1*psitilde_cs_01_3;
    const double basisvalue34 = 1.6583123951777*psitilde_a_0*scalings_y_0*psitilde_bs_0_0*scalings_z_0*psitilde_cs_00_4;
    const double basisvalue35 = 14.6458185158768*psitilde_a_5*scalings_y_5*psitilde_bs_5_0*scalings_z_5*psitilde_cs_50_0;
    const double basisvalue36 = 13.2476412994918*psitilde_a_4*scalings_y_4*psitilde_bs_4_1*scalings_z_5*psitilde_cs_41_0;
    const double basisvalue37 = 11.6833214455479*psitilde_a_3*scalings_y_3*psitilde_bs_3_2*scalings_z_5*psitilde_cs_32_0;
    const double basisvalue38 = 9.87420882906575*psitilde_a_2*scalings_y_2*psitilde_bs_2_3*scalings_z_5*psitilde_cs_23_0;
    const double basisvalue39 = 7.64852927038918*psitilde_a_1*scalings_y_1*psitilde_bs_1_4*scalings_z_5*psitilde_cs_14_0;
    const double basisvalue40 = 4.41588043316392*psitilde_a_0*scalings_y_0*psitilde_bs_0_5*scalings_z_5*psitilde_cs_05_0;
    const double basisvalue41 = 12.0933866224478*psitilde_a_4*scalings_y_4*psitilde_bs_4_0*scalings_z_4*psitilde_cs_40_1;
    const double basisvalue42 = 10.6653645038508*psitilde_a_3*scalings_y_3*psitilde_bs_3_1*scalings_z_4*psitilde_cs_31_1;
    const double basisvalue43 = 9.01387818865997*psitilde_a_2*scalings_y_2*psitilde_bs_2_2*scalings_z_4*psitilde_cs_22_1;
    const double basisvalue44 = 6.98212002188447*psitilde_a_1*scalings_y_1*psitilde_bs_1_3*scalings_z_4*psitilde_cs_13_1;
    const double basisvalue45 = 4.03112887414927*psitilde_a_0*scalings_y_0*psitilde_bs_0_4*scalings_z_4*psitilde_cs_04_1;
    const double basisvalue46 = 9.53939201416946*psitilde_a_3*scalings_y_3*psitilde_bs_3_0*scalings_z_3*psitilde_cs_30_2;
    const double basisvalue47 = 8.06225774829855*psitilde_a_2*scalings_y_2*psitilde_bs_2_1*scalings_z_3*psitilde_cs_21_2;
    const double basisvalue48 = 6.2449979983984*psitilde_a_1*scalings_y_1*psitilde_bs_1_2*scalings_z_3*psitilde_cs_12_2;
    const double basisvalue49 = 3.60555127546399*psitilde_a_0*scalings_y_0*psitilde_bs_0_3*scalings_z_3*psitilde_cs_03_2;
    const double basisvalue50 = 6.98212002188447*psitilde_a_2*scalings_y_2*psitilde_bs_2_0*scalings_z_2*psitilde_cs_20_3;
    const double basisvalue51 = 5.40832691319598*psitilde_a_1*scalings_y_1*psitilde_bs_1_1*scalings_z_2*psitilde_cs_11_3;
    const double basisvalue52 = 3.1224989991992*psitilde_a_0*scalings_y_0*psitilde_bs_0_2*scalings_z_2*psitilde_cs_02_3;
    const double basisvalue53 = 4.41588043316392*psitilde_a_1*scalings_y_1*psitilde_bs_1_0*scalings_z_1*psitilde_cs_10_4;
    const double basisvalue54 = 2.54950975679639*psitilde_a_0*scalings_y_0*psitilde_bs_0_1*scalings_z_1*psitilde_cs_01_4;
    const double basisvalue55 = 1.80277563773199*psitilde_a_0*scalings_y_0*psitilde_bs_0_0*scalings_z_0*psitilde_cs_00_5;
    
    // Table(s) of coefficients
    const static double coefficients0[56][56] = \
    {{0.00945067404923504, 0.00609788120679252, 0.003520613356228, 0.00248944957812473, 0.00705946233325038, 0.00546823600997147, 0.00315708753234945, 0.00446479600584757, 0.00257775117585287, 0.00182274533665719, -0.00298267418725718, -0.0025208197798293, -0.00195261860521383, -0.0011273448773449, -0.00218309396769446, -0.00169101731601731, -0.000976309302606924, -0.00138070985681772, -0.000797153207506492, -0.000563672438672437, 0.00278687112460006, 0.00245778931055954, 0.00207721109304565, 0.00160900079398463, 0.000928957041533348, 0.00219831358911339, 0.00185791408306671, 0.00143913406048031, 0.000830884437218262, 0.00160900079398463, 0.00124632665582739, 0.000719567030240153, 0.00101762145320216, 0.000587524019939396, 0.000415442218609115, -0.00705593524717797, -0.0063823335708908, -0.00562868913761196, -0.00475711057319694, -0.00368484200521179, -0.00212744452363026, -0.00582624677716564, -0.00513826668309063, -0.00434262794914383, -0.00336378514516178, -0.00194208225905521, -0.00459580543596519, -0.00388416451811043, -0.00300866089851429, -0.00173705117965752, -0.00336378514516179, -0.00260557676948628, -0.00150433044925714, -0.00212744452363026, -0.00122828066840392, -0.000868525589828755},
    {0.00945067404923484, -0.0060978812067926, 0.00352061335622807, 0.00248944957812473, 0.00705946233325039, -0.00546823600997146, 0.00315708753234942, -0.00446479600584756, 0.00257775117585286, 0.00182274533665715, 0.00298267418725716, -0.00252081977982931, 0.00195261860521383, -0.00112734487734489, -0.00218309396769446, 0.00169101731601731, -0.000976309302606908, 0.00138070985681772, -0.000797153207506487, -0.000563672438672423, 0.00278687112460003, -0.00245778931055954, 0.00207721109304565, -0.00160900079398463, 0.000928957041533354, -0.00219831358911339, 0.00185791408306669, -0.00143913406048031, 0.000830884437218255, 0.00160900079398463, -0.0012463266558274, 0.000719567030240159, -0.00101762145320216, 0.000587524019939409, 0.000415442218609113, 0.00705593524717798, -0.00638233357089081, 0.00562868913761196, -0.00475711057319694, 0.00368484200521178, -0.00212744452363027, -0.00582624677716566, 0.00513826668309062, -0.00434262794914383, 0.00336378514516178, -0.00194208225905522, 0.0045958054359652, -0.00388416451811043, 0.00300866089851429, -0.00173705117965753, -0.00336378514516178, 0.00260557676948629, -0.00150433044925715, 0.00212744452363026, -0.00122828066840393, -0.000868525589828756},
    {0.00945067404923496, 0, -0.00704122671245609, 0.00248944957812476, 0, 0, 0.00947126259704828, 0, -0.00515550235170569, 0.00182274533665715, 0, 0, 0, 0.0045093795093795, 0, 0, -0.00292892790782073, 0, 0.00159430641501296, -0.000563672438672431, 0, 0, 0, 0, 0.00464478520766673, 0, 0, 0, -0.00332353774887306, 0, 0, 0.00215870109072046, 0, -0.00117504803987882, 0.000415442218609132, 0, 0, 0, 0, 0, 0.0127646671417816, 0, 0, 0, 0, -0.00971041129527609, 0, 0, 0, 0.0069482047186301, 0, 0, -0.00451299134777144, 0, 0.00245656133680787, -0.000868525589828766},
    {0.00945067404923495, 0, 0, -0.00746834873437432, 0, 0, 0, 0, 0, 0.0109364720199429, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0.00563672438672439, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0.00623163327913698, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0.0182390373864041},
    {-0.0100234421734309, 0, 0.0374718748311398, -0.0289614183593723, 0, 0, 0.0284137877911449, 0, 0.00515550235170565, -0.00619733414463432, 0, 0, 0, 0.0315656565656566, 0, 0, -0.0113902751970807, 0, 0.00265717735835495, -0.000187890812890785, 0, 0, 0, 0, 0.0232239260383336, 0, 0, 0, -0.00166176887443649, 0, 0, -0.00647610327216139, 0, 0.00646276421933346, -0.00290809553026393, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0.0213629048496073, 0, 0, 0, -0.0277928188745204, 0, 0, 0.0243701532779658, 0, -0.0157219925555703, 0.00607967912880134},
    {0.0100234421734308, 0, -0.0287574853355261, 0.05360943398437, 0, 0, -0.0189425251940965, 0, 0.0154665070551172, 0.0153110608279201, 0, 0, 0, -0.00450937950937953, 0, 0, 0.0370997534990628, 0, -0.0281660799985623, 0.0108976671476671, 0, 0, 0, 0, 0, 0, 0, 0, 0.0299118397398575, 0, 0, -0.00431740218144094, 0, -0.0123380044187275, 0.00872428659079179, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0.0312669212338355, 0, 0, -0.0487403065559315, 0, 0.0412702304583721, -0.018239037386404},
    {0.0100234421734309, 0, 0.0409576306293854, -0.0449826285156208, 0, 0, 0.0189425251940966, 0, -0.00515550235170581, -0.021143845905223, 0, 0, 0, 0, 0, 0, 0.00455611007883228, 0, 0.0356061766019561, -0.0319414381914382, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0.0302218152700864, 0, 0.00411266813957589, -0.0145404776513196, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0.0379091273212801, 0, -0.0550269739444961, 0.0303983956440067},
    {-0.0100234421734309, 0, -0.0148144621425437, 0.0449826285156208, 0, 0, 0, 0, 0.0206220094068229, 0.021143845905223, 0, 0, 0, 0, 0, 0, 0, 0, 0.0106287094334197, 0.0319414381914382, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0.0205633406978791, 0.0145404776513196, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0.03439185871531, -0.0303983956440067},
    {-0.0100234421734309, 0.0324515955311978, -0.01873593741557, -0.0289614183593723, 0.0211783869997511, -0.0164047080299144, 0.00947126259704827, 0.00446479600584751, -0.00257775117585286, -0.00619733414463429, 0.0208787193108001, -0.0176457384588051, 0.0136683302364968, -0.00789141414141412, -0.00848980987436741, 0.00657617845117844, -0.00379675839902691, 0.0023011830946962, -0.00132858867917746, -0.000187890812890819, 0.0139343556230002, -0.0122889465527977, 0.0103860554652283, -0.00804500396992315, 0.00464478520766672, -0.00109915679455667, 0.000928957041533349, -0.000719567030240153, 0.000415442218609154, -0.00482700238195389, 0.00373897996748218, -0.00215870109072046, 0.00559691799261187, -0.00323138210966673, -0.00290809553026394, 0, 0, 0, 0, 0, 0, 0.0128177429097644, -0.0113041867027994, 0.00955378148811641, -0.00740032731935592, 0.00427258096992148, -0.0183832217438608, 0.0155366580724417, -0.0120346435940572, 0.00694820471863012, 0.0181644397838736, -0.014070114555226, 0.00812338442598859, -0.0136156449512337, 0.00786099627778517, 0.00607967912880136},
    {0.0100234421734311, -0.024904712849524, 0.014378742667763, 0.0536094339843701, -0.0141189246665008, 0.0109364720199429, -0.00631417506469887, 0.0133943880175428, -0.00773325352755854, 0.01531106082792, -0.00298267418725714, 0.0025208197798293, -0.00195261860521383, 0.00112734487734485, 0.0276525235907967, -0.0214195526695527, 0.0123665844996876, -0.0243925408037797, 0.0140830399992812, 0.0108976671476671, 0, 0, 0, 0, 0, 0.0197848223020205, -0.0167212267476003, 0.0129522065443228, -0.0074779599349644, -0.00321800158796925, 0.00249265331165481, -0.00143913406048031, -0.0106850252586226, 0.00616900220936375, 0.00872428659079183, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0.0206811244618434, -0.017478740331497, 0.0135389740433143, -0.00781673030845889, -0.0363288795677472, 0.028140229110452, -0.0162467688519772, 0.0357410679969885, -0.0206351152291861, -0.0182390373864041},
    {0.0100234421734311, 0.0354703486038675, -0.0204788153146928, -0.0449826285156207, 0.0141189246665009, -0.0109364720199429, 0.00631417506469889, -0.0044647960058475, 0.00257775117585279, -0.021143845905223, 0, 0, 0, 0, 0.00339592394974698, -0.00263047138047136, 0.00151870335961076, 0.0308358534689291, -0.0178030883009781, -0.0319414381914382, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0.0225260111157848, -0.0174485731815835, 0.0100739384233621, 0.00356167508620756, -0.00205633406978792, -0.0145404776513196, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0.028255795219359, -0.0218868448636849, 0.01263637577376, -0.047654757329318, 0.0275134869722481, 0.0303983956440068},
    {-0.0100234421734311, -0.0128297005588457, 0.00740723107127181, 0.0449826285156207, 0, 0, 0, 0.0178591840233903, -0.0103110047034114, 0.0211438459052229, 0, 0, 0, 0, 0, 0, 0, 0.0092047323787848, -0.00531435471670985, 0.0319414381914382, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0.0178083754310377, -0.0102816703489396, 0.0145404776513196, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0.0297842233308237, -0.017195929357655, -0.0303983956440068},
    {-0.0100234421734311, 0.0324515955311977, -0.0335503995581136, -0.00801060507812428, 0.0211783869997512, 0.00109364720199428, -0.00568275755822898, -0.0169662248222208, 0.00567105258687634, 0.0083846285486229, 0.0208787193108001, -0.0109235523792603, 0.00499002532443531, -0.00187890812890811, -0.0162519217595033, 0.00958243145743145, -0.00466458889023307, 0.0108922666482287, -0.00558007245254534, -0.00469727032227035, 0.0139343556230002, -0.00245778931055952, -0.00415442218609131, 0.0064360031759385, -0.00464478520766676, -0.0120907247401236, 0.00464478520766674, -0.000719567030240154, -0.000415442218609135, 0.00965400476390777, -0.00498530662330959, 0.00215870109072046, -0.00661453944581401, 0.0032313821096667, 0.00290809553026394, 0, 0.0127646671417816, -0.0202632808954031, 0.0228341307513453, -0.020635115229186, 0.0127646671417816, -0.00116524935543312, -0.00719357335632687, 0.0112908326677739, -0.0114368694935501, 0.00737991258440983, 0.00183832217438608, 0.00310733161448834, -0.00481385743762287, 0.00347410235931505, -0.00201827108709706, -0.000521115353897253, 0.000902598269554298, 0.00170195561890421, -0.000491312267361565, -0.000868525589828764},
    {0.0100234421734313, -0.0249047128495239, 0.0553363732971484, -0.00431340273437463, -0.0141189246665008, 0.0153110608279201, 0.00883984509057839, 0.00803663281052565, -0.0159820572902878, 0.000729098134662871, -0.00298267418725714, 0.0277290175781224, -0.0301571095694136, 0.0191648629148629, -0.00145539597846295, -0.0116492303992304, 0.00976309302606917, 0.00352848074520082, 0.00132858867917745, -0.00263047138047135, 0, 0.0196623144844763, -0.00727023882565982, -0.00724050357293083, 0.00928957041533347, -0.00219831358911338, -0.0148633126645336, 0.0129522065443228, -0.00664707549774609, 0.00402250198496158, 0.00685479660705067, -0.00575653624192123, -0.00457929653940971, -0.000293762009969677, 0.0029080955302639, 0, 0, 0.0202632808954031, -0.0399597288148543, 0.0464290092656686, -0.031911667854454, 0, -0.00411061334647249, -0.00303983956440069, 0.00908221989193683, -0.00776832903622088, 0.000459580543596524, 0.00427258096992149, -0.00270779480866288, 0.000868525589828788, -0.00100913554354855, -0.00234501909253767, 0.00180519653910856, 0.00127646671417817, 0.000245656133680783, -0.000868525589828766},
    {0.0100234421734309, 0.0354703486038674, -0.0492363006502187, -0.00431340273437457, 0.0141189246665008, -0.00656188321196574, -0.016416855168217, -0.00982255121286468, 0.0149509568199466, 0.00072909813466289, 0, 0.00336109303977242, 0.0282044909641998, -0.0353234728234728, -0.000485131992821028, -0.0124007936507936, 0.00889526253486298, 0.00291483191994853, 0.00239145962251946, -0.0026304713804714, 0, 0, 0.0218107164769794, -0.00402250198496158, -0.00928957041533346, 0, -0.00557374224920008, -0.0143913406048031, 0.0141250354327105, 0.000804500396992295, 0.00934744991870546, -0.00287826812096058, -0.00254405363300538, -0.00381890612960614, 0.00290809553026392, 0, 0, 0, 0.0266398192099029, -0.0515877880729651, 0.0425488904726054, 0, 0, -0.00911951869320201, 0.0084094628629044, -0.0038841645181104, 0, 0.00233049871086625, 0.00300866089851431, -0.00347410235931508, -0.000336378514516169, -0.00286613444643491, 0.00120346435940571, 0.000850977809452102, 0.000982624534723146, -0.000868525589828757},
    {-0.0100234421734311, -0.0128297005588456, 0.0448791059024117, -0.00801060507812423, 0, 0.0174983552319087, 0.0132597676358676, -0.00357183680467805, -0.0175287079957994, 0.00838462854862291, 0, 0, 0.00867830491206144, 0.0225468975468975, 0, -0.00300625300625305, -0.019200749617936, 0.000613648825252313, 0.0122230158484327, -0.00469727032227032, 0, 0, 0, 0.0160900079398463, 0.00464478520766673, 0, 0, -0.00719567030240156, -0.0108014976838374, 0, 0.00249265331165479, 0.0107935054536023, -0.000508810726601082, -0.00734405024924251, 0.00290809553026393, 0, 0, 0, 0, 0.0257938940364825, -0.031911667854454, 0, 0, 0, -0.0134551405806471, 0.0135945758133865, 0, 0, 0.00601732179702858, -0.00347410235931504, 0, -0.00208446141558903, -0.000902598269554288, 0.000425488904726059, 0.0017195929357655, -0.000868525589828767},
    {-0.010023442173431, -0.0324515955311979, -0.0187359374155701, -0.0289614183593722, 0.0211783869997511, 0.0164047080299144, 0.00947126259704825, -0.0044647960058475, -0.00257775117585286, -0.00619733414463431, -0.0208787193108001, -0.0176457384588052, -0.0136683302364968, -0.00789141414141413, -0.00848980987436743, -0.00657617845117846, -0.00379675839902691, -0.00230118309469614, -0.00132858867917743, -0.000187890812890788, 0.0139343556230002, 0.0122889465527977, 0.0103860554652283, 0.00804500396992312, 0.00464478520766673, 0.00109915679455666, 0.00092895704153335, 0.000719567030240155, 0.000415442218609149, -0.0048270023819539, -0.00373897996748222, -0.00215870109072046, -0.00559691799261192, -0.00323138210966673, -0.00290809553026393, 0, 0, 0, 0, 0, 0, 0.0128177429097644, 0.0113041867027994, 0.00955378148811642, 0.00740032731935589, 0.00427258096992148, 0.0183832217438608, 0.0155366580724417, 0.0120346435940572, 0.00694820471863011, 0.0181644397838736, 0.014070114555226, 0.00812338442598858, 0.0136156449512337, 0.00786099627778515, 0.00607967912880134},
    {0.0100234421734311, 0.024904712849524, 0.014378742667763, 0.05360943398437, -0.0141189246665008, -0.0109364720199429, -0.00631417506469881, -0.0133943880175428, -0.00773325352755853, 0.01531106082792, 0.00298267418725717, 0.0025208197798293, 0.00195261860521382, 0.00112734487734485, 0.0276525235907967, 0.0214195526695527, 0.0123665844996876, 0.0243925408037797, 0.0140830399992811, 0.0108976671476671, 0, 0, 0, 0, 0, -0.0197848223020205, -0.0167212267476003, -0.0129522065443227, -0.0074779599349644, -0.00321800158796927, -0.00249265331165477, -0.00143913406048032, 0.0106850252586227, 0.00616900220936373, 0.00872428659079179, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, -0.0206811244618434, -0.017478740331497, -0.0135389740433143, -0.00781673030845888, -0.0363288795677472, -0.028140229110452, -0.0162467688519772, -0.0357410679969885, -0.020635115229186, -0.018239037386404},
    {0.0100234421734307, -0.0354703486038675, -0.0204788153146927, -0.0449826285156207, 0.0141189246665008, 0.0109364720199429, 0.00631417506469886, 0.00446479600584751, 0.00257775117585285, -0.0211438459052229, 0, 0, 0, 0, 0.00339592394974697, 0.00263047138047138, 0.00151870335961076, -0.0308358534689291, -0.017803088300978, -0.0319414381914382, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0.0225260111157848, 0.0174485731815835, 0.0100739384233621, -0.00356167508620757, -0.00205633406978789, -0.0145404776513196, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0.0282557952193589, 0.0218868448636849, 0.01263637577376, 0.047654757329318, 0.027513486972248, 0.0303983956440067},
    {-0.010023442173431, 0.0128297005588457, 0.00740723107127182, 0.0449826285156208, 0, 0, 0, -0.0178591840233902, -0.0103110047034114, 0.021143845905223, 0, 0, 0, 0, 0, 0, 0, -0.00920473237878481, -0.00531435471670985, 0.0319414381914381, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, -0.0178083754310377, -0.0102816703489396, 0.0145404776513196, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, -0.0297842233308237, -0.017195929357655, -0.0303983956440068},
    {-0.0100234421734306, -0.0324515955311977, -0.0335503995581136, -0.00801060507812422, 0.0211783869997512, -0.00109364720199428, -0.00568275755822895, 0.0169662248222208, 0.00567105258687631, 0.00838462854862288, -0.0208787193108001, -0.0109235523792604, -0.00499002532443533, -0.00187890812890813, -0.0162519217595033, -0.00958243145743147, -0.00466458889023305, -0.0108922666482287, -0.00558007245254534, -0.00469727032227036, 0.0139343556230002, 0.00245778931055952, -0.00415442218609133, -0.00643600317593853, -0.00464478520766674, 0.0120907247401236, 0.00464478520766673, 0.000719567030240151, -0.000415442218609145, 0.00965400476390776, 0.00498530662330958, 0.00215870109072046, 0.006614539445814, 0.00323138210966669, 0.00290809553026394, 0, 0.0127646671417816, 0.0202632808954031, 0.0228341307513453, 0.020635115229186, 0.0127646671417816, -0.00116524935543314, 0.00719357335632687, 0.0112908326677739, 0.01143686949355, 0.00737991258440983, -0.00183832217438609, 0.00310733161448835, 0.00481385743762286, 0.00347410235931504, -0.00201827108709706, 0.00052111535389726, 0.000902598269554298, -0.00170195561890421, -0.000491312267361563, -0.000868525589828763},
    {0.0100234421734315, 0.024904712849524, 0.0553363732971485, -0.00431340273437464, -0.0141189246665008, -0.0153110608279201, 0.00883984509057836, -0.00803663281052558, -0.0159820572902878, 0.000729098134662864, 0.00298267418725715, 0.0277290175781224, 0.0301571095694136, 0.0191648629148629, -0.00145539597846298, 0.0116492303992304, 0.00976309302606916, -0.00352848074520084, 0.00132858867917744, -0.00263047138047133, 0, -0.0196623144844763, -0.0072702388256598, 0.00724050357293084, 0.0092895704153335, 0.00219831358911338, -0.0148633126645336, -0.0129522065443228, -0.0066470754977461, 0.00402250198496159, -0.00685479660705068, -0.00575653624192122, 0.0045792965394097, -0.000293762009969664, 0.00290809553026391, 0, 0, -0.0202632808954031, -0.0399597288148543, -0.0464290092656686, -0.031911667854454, 0, 0.00411061334647249, -0.00303983956440066, -0.0090822198919368, -0.00776832903622086, -0.000459580543596519, 0.0042725809699215, 0.00270779480866287, 0.000868525589828796, -0.00100913554354854, 0.00234501909253766, 0.00180519653910856, -0.00127646671417816, 0.00024565613368078, -0.00086852558982877},
    {0.0100234421734312, -0.0354703486038675, -0.0492363006502186, -0.0043134027343746, 0.0141189246665008, 0.00656188321196572, -0.016416855168217, 0.0098225512128646, 0.0149509568199467, 0.000729098134662864, 0, 0.0033610930397724, -0.0282044909641998, -0.0353234728234728, -0.000485131992820996, 0.0124007936507936, 0.00889526253486298, -0.00291483191994851, 0.00239145962251946, -0.00263047138047141, 0, 0, 0.0218107164769794, 0.00402250198496156, -0.00928957041533348, 0, -0.0055737422492001, 0.0143913406048031, 0.0141250354327105, 0.00080450039699229, -0.00934744991870546, -0.00287826812096059, 0.00254405363300538, -0.00381890612960615, 0.00290809553026392, 0, 0, 0, 0.0266398192099029, 0.0515877880729651, 0.0425488904726054, 0, 0, -0.00911951869320204, -0.00840946286290444, -0.00388416451811041, 0, 0.00233049871086624, -0.0030086608985143, -0.00347410235931509, -0.000336378514516169, 0.00286613444643491, 0.00120346435940572, -0.000850977809452107, 0.000982624534723155, -0.000868525589828754},
    {-0.010023442173431, 0.0128297005588457, 0.0448791059024117, -0.00801060507812423, 0, -0.0174983552319087, 0.0132597676358676, 0.0035718368046781, -0.0175287079957995, 0.00838462854862291, 0, 0, -0.00867830491206147, 0.0225468975468975, 0, 0.00300625300625302, -0.019200749617936, -0.000613648825252327, 0.0122230158484327, -0.00469727032227032, 0, 0, 0, -0.0160900079398463, 0.00464478520766674, 0, 0, 0.00719567030240152, -0.0108014976838374, 0, -0.00249265331165479, 0.0107935054536023, 0.000508810726601078, -0.00734405024924251, 0.00290809553026392, 0, 0, 0, 0, -0.0257938940364826, -0.031911667854454, 0, 0, 0, 0.0134551405806471, 0.0135945758133865, 0, 0, -0.00601732179702859, -0.00347410235931504, 0, 0.00208446141558904, -0.000902598269554294, -0.000425488904726048, 0.0017195929357655, -0.000868525589828768},
    {-0.0100234421734308, -0.0452812960900435, -0.0113287063442981, -0.00801060507812425, -0.00141189246665006, 0.0164047080299144, 0.0145226026488073, 0.0133943880175427, 0.0118576554089232, 0.0083846285486229, -0.00894802256177148, -0.0142846454190327, -0.014536160727703, -0.00939454064454062, -0.0123708658169354, -0.0125886844636845, -0.00813591085505763, -0.0102786178229763, -0.00664294339588729, -0.00469727032227033, -0.00836061337380013, 0.00245778931055953, 0.00830884437218264, 0.00965400476390776, 0.00650269929073342, 0.00219831358911336, 0.00743165633226678, 0.00863480436288185, 0.00581619106052787, 0.00643600317593851, 0.00747795993496438, 0.00503696921168107, 0.00610572871921291, 0.00411266813957582, 0.00290809553026393, 0.0352796762358899, 0.0191470007126724, 0.00788016479265675, 0.000951422114639387, -0.00221090520312706, -0.00212744452363027, 0.017478740331497, 0.00719357335632688, 0.000868525589828766, -0.00201827108709706, -0.00194208225905523, 0.00643412761035128, 0.00077683290362208, -0.00180519653910858, -0.00173705117965754, 0.000672757029032357, -0.00156334606169177, -0.00150433044925715, -0.00127646671417815, -0.00122828066840394, -0.000868525589828763},
    {0.010023442173431, 0.0603750614533912, -0.00610007264692973, -0.0043134027343746, -0.00564756986660029, -0.0218729440398858, 0.00126283501293976, -0.0178591840233903, 0.00103110047034112, 0.00072909813466285, 0.0417574386216002, 0.0168054651988621, -0.000867830491206138, -0.00526094276094278, 0.0145539597846298, -0.000751563251563223, -0.00455611007883228, -0.000613648825252315, -0.00372004830169692, -0.00263047138047137, 0.00557374224920009, -0.0196623144844763, -0.0103860554652283, 0.00321800158796927, 0.00650269929073345, -0.0175865087129071, -0.00928957041533348, 0.00287826812096062, 0.00581619106052783, -0.00804500396992314, 0.0024926533116548, 0.00503696921168107, 0.00203524290640432, 0.00411266813957582, 0.00290809553026392, -0.0705593524717798, -0.0127646671417816, 0.00450295131008957, 0.00380568845855756, -0.000736968401042374, -0.00212744452363027, -0.0116524935543313, 0.0041106133464725, 0.00347410235931506, -0.000672757029032359, -0.0019420822590552, 0.00367664434877216, 0.00310733161448835, -0.000601732179702866, -0.00173705117965752, 0.00269102811612943, -0.000521115353897279, -0.00150433044925714, -0.000425488904726059, -0.00122828066840393, -0.000868525589828762},
    {0.010023442173431, -0.0603750614533913, -0.00610007264692972, -0.00431340273437459, -0.00564756986660034, 0.0218729440398858, 0.00126283501293975, 0.0178591840233902, 0.00103110047034114, 0.000729098134662857, -0.0417574386216002, 0.016805465198862, 0.000867830491206142, -0.00526094276094274, 0.0145539597846298, 0.000751563251563242, -0.00455611007883226, 0.000613648825252326, -0.0037200483016969, -0.00263047138047136, 0.0055737422492001, 0.0196623144844763, -0.0103860554652283, -0.00321800158796928, 0.00650269929073344, 0.0175865087129071, -0.00928957041533347, -0.00287826812096062, 0.00581619106052785, -0.00804500396992314, -0.0024926533116548, 0.00503696921168109, -0.00203524290640433, 0.00411266813957582, 0.00290809553026392, 0.0705593524717798, -0.0127646671417816, -0.00450295131008957, 0.00380568845855755, 0.000736968401042381, -0.00212744452363028, -0.0116524935543313, -0.00411061334647249, 0.00347410235931505, 0.00067275702903236, -0.00194208225905522, -0.00367664434877216, 0.00310733161448835, 0.000601732179702862, -0.00173705117965754, 0.00269102811612941, 0.000521115353897267, -0.00150433044925715, 0.000425488904726055, -0.00122828066840394, -0.00086852558982877},
    {-0.0100234421734311, 0.0452812960900435, -0.0113287063442981, -0.00801060507812424, -0.00141189246665007, -0.0164047080299144, 0.0145226026488074, -0.0133943880175427, 0.0118576554089232, 0.00838462854862294, 0.00894802256177145, -0.0142846454190327, 0.014536160727703, -0.00939454064454065, -0.0123708658169354, 0.0125886844636845, -0.00813591085505765, 0.0102786178229763, -0.00664294339588732, -0.00469727032227034, -0.00836061337380011, -0.00245778931055953, 0.00830884437218264, -0.00965400476390776, 0.00650269929073342, -0.00219831358911338, 0.00743165633226677, -0.00863480436288184, 0.00581619106052786, 0.00643600317593851, -0.00747795993496437, 0.00503696921168106, -0.0061057287192129, 0.00411266813957581, 0.00290809553026391, -0.0352796762358899, 0.0191470007126724, -0.00788016479265674, 0.000951422114639392, 0.00221090520312706, -0.00212744452363026, 0.017478740331497, -0.00719357335632687, 0.000868525589828777, 0.00201827108709706, -0.00194208225905521, -0.00643412761035126, 0.000776832903622074, 0.00180519653910858, -0.00173705117965752, 0.00067275702903235, 0.00156334606169178, -0.00150433044925714, 0.00127646671417815, -0.00122828066840393, -0.000868525589828756},
    {0.0787556170769578, 0.0679219441350652, -0.00435719474780692, -0.00308100195312471, 0.0705946233325039, -0.0328094160598287, 0.0126283501293978, -0.0267887760350854, 0.03608851646194, -0.00729098134662863, 0.0298267418725716, 0.0151249186789758, -0.0325436434202305, 0.0248015873015872, 0.0130985638061668, -0.0229226791726792, 0.0169226945785199, -0.0248527774227189, 0.0164744996218005, 0.0142797017797018, 0, 0.0245778931055953, -0.0363511941282991, 0.0362025178646542, -0.0232239260383337, 0.0219831358911338, -0.00743165633226679, 0, 0.00166176887443652, -0.0345935170706695, 0.0130864298861876, -0.00359783515120077, 0.0335815079556711, -0.014688100498485, -0.0178640154001927, 0, 0, 0, 0, 0, 0, 0, 0.0226083734055987, -0.0334382352084074, 0.0333014729371016, -0.0213629048496074, -0.00459580543596519, -0.0194208225905522, 0.0270779480866286, -0.0191075629762328, 0.0090822198919368, 0.00703505727761299, -0.00812338442598857, -0.0102117337134253, 0.00196524906944628, 0.00607967912880137},
    {-0.0214788046573525, 0.00377344134083702, 0.00217859737390337, -0.021567013671873, 0, 0.0984282481794862, -0.0852413633734345, -0.0312535720409329, -0.0180442582309699, 0.0309866707231716, 0, 0.0453747560369276, 0, -0.0202922077922077, -0.0116431678277039, 0.0236742424242425, -0.0335199527228374, -0.00138070985681772, -0.000797153207506455, 0.00394570707070704, 0, 0, 0.0363511941282991, -0.0603375297744235, 0.0464478520766673, 0, 0.0241528830798671, 0, -0.0108014976838374, -0.00724050357293081, -0.0392592896585629, 0.0259044130886455, 0.0188259968842399, 0.0108691943688789, -0.0178640154001926, 0, 0, 0, 0, 0, 0, 0, 0, 0.0334382352084074, -0.0555024548951693, 0.0427258096992148, 0, -0.0155366580724417, 0, 0.00694820471863008, 0.00302740663064561, 0.0164151336477636, -0.0108311792346514, -0.00680782247561684, -0.00393049813889257, 0.00607967912880131},
    {0.0787556170769581, 0.0301875307266957, 0.0610007264692977, -0.00308100195312485, 0, 0.0218729440398858, 0.0757701007763862, 0.0178591840233902, -0.0412440188136458, -0.00729098134662867, 0, 0, 0.0520698294723689, -0.00901875901875909, 0, -0.0127765752765753, 0.0286384062098029, 0.00184094647575695, -0.0297603864135753, 0.0142797017797018, 0, 0, 0, 0.0402250198496157, -0.0464478520766674, 0, 0, 0.014391340604803, 0.0182794576188018, 0, -0.0137095932141014, -0.0345392174515274, 0.00407048581280862, 0.036426489236243, -0.0178640154001927, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0.0370016365967796, -0.0427258096992148, 0, 0, -0.030086608985143, 0.0243187165152054, 0, 0.014070114555226, 0, -0.00340391123780844, -0.00982624534723146, 0.00607967912880138},
    {-0.0214788046573519, 0.00377344134083692, -0.0196073763651313, 0.00924300585937408, 0, -0.0109364720199429, 0.00947126259704824, 0.102690308134494, -0.0695992817480272, -0.060150596109686, 0, -0.00504163955965862, 0.0065087286840461, -0.00450937950937948, 0.0465726713108155, 0.0214195526695527, -0.0289638426440052, -0.00782402252196701, -0.0263060558477138, 0.000563672438672498, 0, 0, 0, 0, 0, 0, 0.0334424534952005, -0.0431740218144092, 0.0299118397398575, 0.0289620142917233, -0.00249265331165477, -0.00431740218144094, -0.0605484764655283, 0.0185070066280912, 0.0461140862656136, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0.0349574806629939, -0.0451299134777144, 0.0312669212338355, -0.0121096265225824, -0.028140229110452, 0.0270779480866286, 0.023827378664659, 0, -0.0182390373864041},
    {-0.0214788046573522, -0.0150937653633478, 0.0130715842434209, 0.00924300585937409, 0, -0.0109364720199429, 0.00947126259704823, -0.00892959201169517, 0.123732056440937, -0.060150596109686, 0, 0, -0.00650872868404615, 0.00676406926406924, 0, 0.0574945887445888, 0.0126920209338899, -0.026693723898476, 0.00637722566005184, 0.000563672438672442, 0, 0, 0, 0, 0, 0, 0, 0.0431740218144092, -0.0448677596097863, 0, 0.0199412264932383, 0.0215870109072046, -0.0142467003448301, -0.0616900220936373, 0.0461140862656136, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0.0451299134777144, -0.0469003818507533, 0, -0.0375203054806026, 0.0162467688519772, 0.0119136893323295, 0.0206351152291861, -0.0182390373864041},
    {0.078755617076958, 0.0301875307266957, 0.0174287789912279, 0.0585390371093694, 0, 0.0218729440398858, -0.0189425251940966, 0.0178591840233902, 0.0103110047034113, 0.083846285486229, 0, 0, 0, 0, 0, 0.00526094276094275, -0.00455611007883227, 0.0533874477969518, 0.0308232573569172, -0.0319414381914382, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0.0348971463631671, -0.0302218152700865, 0.0284934006896603, 0.0164506725583033, -0.0643935438844155, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0.0437736897273697, -0.0379091273212801, -0.023827378664659, -0.0137567434861241, 0.0303983956440068},
    {0.0787556170769583, -0.0679219441350652, -0.00435719474780694, -0.00308100195312467, 0.0705946233325039, 0.0328094160598287, 0.0126283501293977, 0.0267887760350853, 0.03608851646194, -0.00729098134662864, -0.0298267418725716, 0.0151249186789759, 0.0325436434202305, 0.0248015873015873, 0.0130985638061668, 0.0229226791726792, 0.0169226945785199, 0.0248527774227189, 0.0164744996218005, 0.0142797017797018, 0, -0.0245778931055954, -0.036351194128299, -0.0362025178646541, -0.0232239260383336, -0.0219831358911338, -0.00743165633226678, 0, 0.00166176887443654, -0.0345935170706695, -0.0130864298861876, -0.00359783515120077, -0.0335815079556711, -0.014688100498485, -0.0178640154001927, 0, 0, 0, 0, 0, 0, 0, -0.0226083734055987, -0.0334382352084074, -0.0333014729371016, -0.0213629048496074, 0.00459580543596522, -0.0194208225905522, -0.0270779480866286, -0.0191075629762328, 0.0090822198919368, -0.007035057277613, -0.00812338442598858, 0.0102117337134253, 0.00196524906944627, 0.00607967912880136},
    {-0.0214788046573522, -0.00377344134083698, 0.00217859737390352, -0.0215670136718729, 0, -0.0984282481794863, -0.0852413633734345, 0.031253572040933, -0.0180442582309699, 0.0309866707231716, 0, 0.0453747560369276, 0, -0.0202922077922078, -0.0116431678277039, -0.0236742424242425, -0.0335199527228375, 0.00138070985681774, -0.000797153207506485, 0.00394570707070704, 0, 0, 0.0363511941282991, 0.0603375297744236, 0.0464478520766674, 0, 0.024152883079867, 0, -0.0108014976838374, -0.0072405035729308, 0.0392592896585629, 0.0259044130886455, -0.0188259968842399, 0.0108691943688789, -0.0178640154001927, 0, 0, 0, 0, 0, 0, 0, 0, 0.0334382352084074, 0.0555024548951693, 0.0427258096992148, 0, -0.0155366580724417, 0, 0.0069482047186301, 0.0030274066306456, -0.0164151336477636, -0.0108311792346515, 0.00680782247561683, -0.00393049813889256, 0.00607967912880134},
    {0.078755617076958, -0.0301875307266958, 0.0610007264692976, -0.00308100195312472, 0, -0.0218729440398859, 0.0757701007763862, -0.0178591840233903, -0.0412440188136458, -0.00729098134662865, 0, 0, -0.0520698294723688, -0.00901875901875904, 0, 0.0127765752765752, 0.0286384062098029, -0.00184094647575696, -0.0297603864135752, 0.0142797017797018, 0, 0, 0, -0.0402250198496157, -0.0464478520766673, 0, 0, -0.0143913406048031, 0.0182794576188018, 0, 0.0137095932141014, -0.0345392174515273, -0.00407048581280864, 0.036426489236243, -0.0178640154001927, 0, 0, 0, 0, 0, 0, 0, 0, 0, -0.0370016365967795, -0.0427258096992148, 0, 0, 0.0300866089851429, 0.0243187165152054, 0, -0.014070114555226, 0, 0.00340391123780844, -0.00982624534723146, 0.00607967912880137},
    {-0.0214788046573522, -0.00377344134083677, -0.0196073763651313, 0.00924300585937403, 0, 0.0109364720199429, 0.00947126259704829, -0.102690308134494, -0.0695992817480271, -0.0601505961096861, 0, -0.00504163955965865, -0.00650872868404618, -0.00450937950937947, 0.0465726713108155, -0.0214195526695527, -0.0289638426440052, 0.00782402252196705, -0.0263060558477139, 0.000563672438672498, 0, 0, 0, 0, 0, 0, 0.0334424534952005, 0.0431740218144092, 0.0299118397398575, 0.0289620142917233, 0.00249265331165484, -0.00431740218144093, 0.0605484764655283, 0.0185070066280911, 0.0461140862656136, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0.0349574806629939, 0.0451299134777144, 0.0312669212338355, -0.0121096265225824, 0.028140229110452, 0.0270779480866286, -0.023827378664659, 0, -0.018239037386404},
    {-0.0214788046573521, 0.0150937653633479, 0.0130715842434208, 0.00924300585937423, 0, 0.010936472019943, 0.00947126259704825, 0.00892959201169522, 0.123732056440937, -0.060150596109686, 0, 0, 0.00650872868404613, 0.00676406926406924, 0, -0.0574945887445887, 0.0126920209338899, 0.0266937238984759, 0.00637722566005184, 0.000563672438672452, 0, 0, 0, 0, 0, 0, 0, -0.0431740218144092, -0.0448677596097863, 0, -0.0199412264932383, 0.0215870109072046, 0.0142467003448301, -0.0616900220936373, 0.0461140862656136, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, -0.0451299134777144, -0.0469003818507533, 0, 0.0375203054806026, 0.0162467688519772, -0.0119136893323295, 0.0206351152291861, -0.0182390373864041},
    {0.0787556170769579, -0.0301875307266956, 0.017428778991228, 0.0585390371093696, 0, -0.0218729440398858, -0.0189425251940965, -0.0178591840233903, 0.0103110047034114, 0.083846285486229, 0, 0, 0, 0, 0, -0.00526094276094277, -0.00455611007883227, -0.0533874477969518, 0.0308232573569172, -0.0319414381914383, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, -0.0348971463631671, -0.0302218152700865, -0.0284934006896603, 0.0164506725583033, -0.0643935438844154, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, -0.0437736897273697, -0.0379091273212801, 0.023827378664659, -0.0137567434861241, 0.0303983956440068},
    {0.078755617076958, -0.0377344134083697, -0.0566435317214906, -0.00308100195312479, 0.0423567739995023, 0.0546823600997145, 0.0378850503881931, 0.0446479600584756, 0.00515550235170578, -0.00729098134662856, 0.0417574386216002, -0.00504163955965862, -0.0247331689993752, -0.0202922077922078, 0.0295930515620807, 0.0101461038961039, 0.00216957622801541, 0.0266937238984759, 0.0132858867917747, 0.0142797017797018, -0.0557374224920008, -0.0245778931055953, -0.00519302773261415, 0.00402250198496156, 0.00464478520766674, -0.00219831358911332, -0.0130053985814669, -0.0158304746652834, -0.0108014976838375, -0.0168945083368386, -0.026796023100289, -0.0194283098164841, -0.0295110221428625, -0.0217383887377579, -0.0178640154001927, 0, 0, 0, 0, 0, 0, -0.0512709716390577, -0.0226083734055987, -0.00477689074405822, 0.00370016365967794, 0.00427258096992147, -0.0367664434877216, -0.00776832903622086, 0.00601732179702859, 0.00694820471863011, -0.00908221989193683, 0.00703505727761299, 0.00812338442598859, 0.00680782247561683, 0.00786099627778515, 0.00607967912880135},
    {-0.0214788046573522, 0, -0.00435719474780689, -0.0215670136718729, -0.127070321998507, 0, 0.0284137877911447, 0, 0.0360885164619399, 0.0309866707231715, 0, 0.0453747560369275, 0, -0.0202922077922078, -0.0422064833754266, 0, -0.00618329224984384, 0, 0.0015943064150129, 0.00394570707070703, 0.0836061337380012, 0, -0.0103860554652283, 0, 0.00464478520766673, 0, 0.024152883079867, 0, -0.0108014976838374, 0.043443021437585, 0, -0.0194283098164841, 0, -0.0217383887377578, -0.0178640154001926, 0, 0, 0, 0, 0, 0, 0.0769064574585866, 0, -0.00955378148811639, 0, 0.00427258096992146, 0, -0.0155366580724417, 0, 0.00694820471863015, -0.0181644397838736, 0, 0.00812338442598858, 0, 0.00786099627778515, 0.00607967912880134},
    {0.0787556170769581, 0.0377344134083697, -0.0566435317214905, -0.00308100195312477, 0.0423567739995024, -0.0546823600997146, 0.0378850503881931, -0.0446479600584756, 0.00515550235170579, -0.00729098134662863, -0.0417574386216002, -0.00504163955965864, 0.0247331689993752, -0.0202922077922078, 0.0295930515620807, -0.0101461038961039, 0.00216957622801542, -0.0266937238984759, 0.0132858867917747, 0.0142797017797018, -0.0557374224920008, 0.0245778931055954, -0.00519302773261416, -0.00402250198496156, 0.00464478520766676, 0.00219831358911337, -0.0130053985814669, 0.0158304746652834, -0.0108014976838375, -0.0168945083368386, 0.026796023100289, -0.0194283098164841, 0.0295110221428625, -0.0217383887377579, -0.0178640154001927, 0, 0, 0, 0, 0, 0, -0.0512709716390577, 0.0226083734055987, -0.00477689074405822, -0.00370016365967795, 0.00427258096992149, 0.0367664434877216, -0.00776832903622086, -0.00601732179702859, 0.00694820471863009, -0.00908221989193678, -0.00703505727761301, 0.00812338442598859, -0.00680782247561686, 0.00786099627778517, 0.00607967912880135},
    {-0.021478804657352, -0.0188672067041847, 0.00653579212171052, 0.00924300585937401, 0.0141189246665008, 0, -0.00315708753234946, -0.111619900146189, -0.05413277469291, -0.0601505961096861, -0.00894802256177153, -0.00252081977982931, 0.000650872868404653, 0.00112734487734485, -0.0276525235907967, 0.0360750360750361, 0.0374251899332651, -0.0188697013765088, 0.019928830187662, 0.000563672438672417, 0, 0, 0, 0, 0, 0.0593544669060614, 0.0167212267476003, -0.00431740218144091, -0.0074779599349644, 0.00321800158796925, 0.0224338798048931, 0.018708742786244, 0.0463017761206981, 0.0431830154655461, 0.0461140862656137, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0.0620433733855302, 0.017478740331497, -0.00451299134777146, -0.0078167303084589, 0.0363288795677472, -0.00938007637015066, -0.0162467688519772, -0.0119136893323295, -0.020635115229186, -0.018239037386404},
    {-0.021478804657352, 0.0188672067041848, 0.00653579212171056, 0.00924300585937413, 0.0141189246665008, 0, -0.00315708753234944, 0.111619900146189, -0.05413277469291, -0.060150596109686, 0.00894802256177151, -0.00252081977982929, -0.000650872868404627, 0.00112734487734488, -0.0276525235907967, -0.0360750360750361, 0.0374251899332651, 0.0188697013765088, 0.0199288301876619, 0.000563672438672459, 0, 0, 0, 0, 0, -0.0593544669060614, 0.0167212267476003, 0.00431740218144093, -0.0074779599349644, 0.00321800158796927, -0.0224338798048931, 0.018708742786244, -0.046301776120698, 0.0431830154655461, 0.0461140862656136, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, -0.0620433733855302, 0.017478740331497, 0.00451299134777145, -0.00781673030845891, 0.0363288795677472, 0.00938007637015065, -0.0162467688519772, 0.0119136893323295, -0.020635115229186, -0.0182390373864041},
    {0.0787556170769582, 0, -0.0348575579824558, 0.0585390371093696, -0.0282378493330016, 0, 0.00631417506469889, 0, -0.0206220094068229, 0.083846285486229, 0, 0, 0, 0, -0.00679184789949391, 0, 0.00151870335961076, 0, -0.0616465147138344, -0.0319414381914382, 0, 0, 0, 0, 0, 0, 0, 0, 0, -0.0450520222315696, 0, 0.0100739384233621, 0, -0.0329013451166065, -0.0643935438844154, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, -0.0565115904387179, 0, 0.01263637577376, 0, 0.027513486972248, 0.0303983956440067},
    {0.0787556170769578, -0.0377344134083695, -0.0217859737390347, -0.0523770332031201, 0.0423567739995023, 0.0546823600997146, 0, 0.0446479600584757, 0.0257775117585286, 0.0291639253865144, 0.0417574386216002, 0.0285692908380655, 0.0273366604729936, 0.0187890812890813, -0.00921750786359888, -0.00789141414141413, -0.00347132196482459, -0.0248527774227189, -0.0143487577351166, -0.015031265031265, -0.0557374224920008, -0.00491557862111907, -0.0197335053839337, -0.0394205194526234, -0.0325134964536672, -0.0241814494802472, -0.00928957041533347, -0.0158304746652834, -0.0141250354327105, -0.00241350119097692, -0.000623163327913714, -0.00215870109072046, 0.00712335017241511, 0.00411266813957583, 0.00540074884191872, 0, -0.0510586685671264, -0.0405265617908061, -0.0114170653756727, 0.010317557614593, 0.0127646671417816, 0.00466099742173251, -0.0143871467126538, -0.00564541633388696, 0.00571843474677503, 0.00737991258440982, 0.00367664434877214, -0.00155366580724416, 0.00240692871881144, 0.00347410235931507, 0.00100913554354852, 0.000260557676948644, 0.000902598269554298, -0.000850977809452118, -0.000491312267361587, -0.000868525589828769},
    {-0.0214788046573518, 0, -0.0217859737390347, 0.00308100195312481, -0.127070321998507, 0, 0.0473563129852414, 0, 0.0257775117585285, 0.0127592173566, 0, -0.0352914769176103, 0, 0.00225468975468975, 0.0509388592462044, 0, -0.00943765659186686, 0, -0.0143487577351166, -0.0129644660894661, 0.0836061337380013, 0, 0.0477758551400502, 0, -0.0325134964536671, 0, 0.00928957041533347, 0, -0.0141250354327104, -0.0144810071458617, 0, -0.00215870109072047, 0, 0.00411266813957582, 0.00540074884191876, 0, 0.0765880028506897, 0, -0.0228341307513453, 0, 0.0127646671417816, -0.00699149613259876, 0, -0.0112908326677739, 0, 0.00737991258440979, 0, -0.00310733161448836, 0, 0.00347410235931506, 0.00201827108709705, 0, 0.000902598269554284, 0, -0.000491312267361592, -0.000868525589828792},
    {0.0787556170769577, 0.0377344134083696, -0.0217859737390349, -0.05237703320312, 0.0423567739995023, -0.0546823600997146, 0, -0.0446479600584756, 0.0257775117585286, 0.0291639253865144, -0.0417574386216002, 0.0285692908380655, -0.0273366604729936, 0.0187890812890813, -0.0092175078635989, 0.00789141414141413, -0.00347132196482458, 0.0248527774227189, -0.0143487577351166, -0.015031265031265, -0.0557374224920009, 0.00491557862111903, -0.0197335053839338, 0.0394205194526234, -0.0325134964536671, 0.0241814494802472, -0.00928957041533348, 0.0158304746652834, -0.0141250354327105, -0.00241350119097695, 0.000623163327913703, -0.00215870109072045, -0.00712335017241511, 0.00411266813957582, 0.00540074884191871, 0, -0.0510586685671264, 0.0405265617908061, -0.0114170653756726, -0.010317557614593, 0.0127646671417816, 0.0046609974217325, 0.0143871467126537, -0.00564541633388698, -0.00571843474677501, 0.00737991258440984, -0.00367664434877218, -0.00155366580724417, -0.00240692871881142, 0.00347410235931505, 0.00100913554354854, -0.000260557676948633, 0.000902598269554291, 0.000850977809452119, -0.000491312267361577, -0.000868525589828757},
    {-0.0214788046573522, -0.0188672067041847, 0.0108929868695173, 0.00308100195312474, 0.0141189246665008, -0.109364720199429, -0.0789271883087357, 0.0223239800292378, -0.0128887558792642, 0.0127592173566001, -0.00894802256177151, -0.0277290175781224, -0.00585785581564148, 0.0191648629148629, 0.00145539597846299, 0.0383297258297258, 0.0348216984596467, -0.0124263887113595, 0.00717437886755833, -0.0129644660894661, 0, 0.0589869434534289, 0.00727023882565978, 0.0506835250105158, 0.0650269929073344, -0.00659494076734013, 0.0148633126645336, -0.00431740218144089, -0.00166176887443655, -0.0040225019849616, -0.00810112326287806, -0.0115130724838424, 0.00356167508620753, -0.00205633406978792, 0.00540074884191869, 0, 0, 0.0607898426862092, 0.0399597288148543, -0.0154763364218895, -0.031911667854454, 0, -0.0123318400394175, 0.00303983956440066, -0.00302740663064559, -0.00776832903622087, 0.00137874163078958, -0.0042725809699215, 0.000902598269554286, 0.000868525589828745, 0.00100913554354854, 0.000781673030845884, 0.00180519653910853, -0.000425488904726043, 0.0002456561336808, -0.000868525589828759},
    {-0.0214788046573523, 0.0188672067041848, 0.0108929868695173, 0.00308100195312473, 0.0141189246665008, 0.109364720199429, -0.0789271883087356, -0.0223239800292378, -0.0128887558792643, 0.0127592173566, 0.00894802256177147, -0.0277290175781224, 0.00585785581564147, 0.0191648629148629, 0.00145539597846298, -0.0383297258297258, 0.0348216984596467, 0.0124263887113595, 0.00717437886755829, -0.0129644660894661, 0, -0.0589869434534289, 0.0072702388256598, -0.0506835250105158, 0.0650269929073343, 0.00659494076734013, 0.0148633126645336, 0.0043174021814409, -0.00166176887443651, -0.00402250198496155, 0.00810112326287803, -0.0115130724838425, -0.00356167508620755, -0.0020563340697879, 0.00540074884191873, 0, 0, -0.0607898426862092, 0.0399597288148542, 0.0154763364218895, -0.031911667854454, 0, 0.0123318400394175, 0.00303983956440069, 0.0030274066306456, -0.00776832903622087, -0.00137874163078958, -0.00427258096992146, -0.000902598269554311, 0.000868525589828782, 0.00100913554354855, -0.000781673030845884, 0.00180519653910857, 0.000425488904726043, 0.000245656133680809, -0.000868525589828767},
    {0.0787556170769581, 0, 0.0435719474780695, -0.0523770332031201, -0.0282378493330016, 0, 0.0631417506469885, 0, -0.0515550235170572, 0.0291639253865144, 0, -0.00672218607954481, 0, -0.0601250601250601, 0.000970263985641982, 0, -0.0125835421224891, 0, 0.0286975154702333, -0.015031265031265, 0, 0, -0.0436214329539588, 0, -0.0650269929073343, 0, 0.0111474844984002, 0, 0.031573608614294, -0.00160900079398462, 0, -0.0028782681209606, 0, -0.00822533627915164, 0.00540074884191871, 0, 0, 0, -0.0532796384198057, 0, 0.0425488904726053, 0, 0, 0.018239037386404, 0, -0.00388416451811043, 0, -0.00466099742173252, 0, -0.00347410235931507, 0.00067275702903234, 0, 0.00120346435940573, 0, 0.000982624534723144, -0.000868525589828764},
    {0.10739402328676, -0.188672067041848, -0.108929868695174, -0.0770250488281177, -0.0705946233325038, -0.0546823600997147, -0.0315708753234941, -0.0446479600584757, -0.0257775117585286, -0.0182274533665716, 0.0894802256177148, -0.0151249186789758, 0.058578558156415, 0.0744047619047618, -0.0130985638061668, -0.0281836219336219, 0.00976309302606916, 0.069035492840886, 0.0398576603753241, 0.0597492784992785, 0, 0.0737336793167861, 0.036351194128299, -0.0120675059548847, -0.0232239260383337, 0.0659494076734016, 0.00743165633226678, 0.0431740218144092, 0.0465295284842228, 0.0345935170706695, 0.0155790831978425, 0.0251848460584054, -0.0254405363300539, -0.0146881004984851, -0.0328199352701214, 0, 0, 0, 0, 0, 0, 0, 0.0678251202167962, 0.0334382352084074, -0.0111004909790339, -0.0213629048496074, -0.0137874163078956, 0.0194208225905522, -0.0090259826955429, -0.0191075629762328, -0.00908221989193678, -0.00234501909253769, -0.00812338442598858, 0.00340391123780841, 0.00196524906944627, 0.00607967912880134},
    {0.107394023286761, 0.188672067041848, -0.108929868695174, -0.0770250488281178, -0.0705946233325039, 0.0546823600997145, -0.0315708753234942, 0.0446479600584757, -0.0257775117585287, -0.0182274533665715, -0.0894802256177147, -0.0151249186789758, -0.0585785581564149, 0.0744047619047619, -0.0130985638061668, 0.0281836219336219, 0.00976309302606916, -0.0690354928408859, 0.039857660375324, 0.0597492784992785, 0, -0.0737336793167861, 0.036351194128299, 0.0120675059548847, -0.0232239260383338, -0.0659494076734015, 0.00743165633226678, -0.0431740218144092, 0.0465295284842228, 0.0345935170706695, -0.0155790831978424, 0.0251848460584053, 0.0254405363300539, -0.0146881004984851, -0.0328199352701214, 0, 0, 0, 0, 0, 0, 0, -0.0678251202167962, 0.0334382352084074, 0.0111004909790339, -0.0213629048496074, 0.0137874163078956, 0.0194208225905522, 0.00902598269554287, -0.0191075629762328, -0.0090822198919368, 0.00234501909253767, -0.00812338442598857, -0.0034039112378084, 0.00196524906944627, 0.00607967912880135},
    {0.107394023286761, 0, 0.217859737390348, -0.0770250488281178, 0, 0, -0.094712625970483, 0, 0.0515550235170572, -0.0182274533665716, 0, -0.0907495120738551, 0, -0.0946969696969696, 0.0232863356554077, 0, -0.0227805503941613, 0, -0.0797153207506479, 0.0597492784992786, 0, 0, -0.0727023882565981, 0, 0.0464478520766674, 0, -0.0483057661597341, 0, -0.0781031370985168, 0.0144810071458616, 0, 0.0431740218144092, 0, 0.0293762009969701, -0.0328199352701215, 0, 0, 0, 0, 0, 0, 0, 0, -0.0668764704168148, 0, 0.0427258096992148, 0, 0.0310733161448835, 0, 0.00694820471863007, -0.00605481326129118, 0, -0.0108311792346515, 0, -0.00393049813889257, 0.00607967912880135},
    {0.107394023286761, 0, 0, 0.231075146484353, 0, 0, 0, 0, 0, -0.109364720199429, 0, 0.0100832791193172, 0, -0.00450937950937963, -0.093145342621631, 0, -0.104139658944738, 0, 0, -0.09244227994228, 0, 0, 0, 0, 0, 0, -0.066884906990401, 0, 0.0299118397398574, -0.0579240285834467, 0, -0.0647610327216138, 0, 0, 0.0835038859404354, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, -0.0699149613259879, 0, 0.0312669212338356, 0.0242192530451648, 0, 0.0270779480866286, 0, 0, -0.018239037386404}};
    
    // Extract relevant coefficients
    const double coeff0_0 = coefficients0[dof][0];
    const double coeff0_1 = coefficients0[dof][1];
    const double coeff0_2 = coefficients0[dof][2];
    const double coeff0_3 = coefficients0[dof][3];
    const double coeff0_4 = coefficients0[dof][4];
    const double coeff0_5 = coefficients0[dof][5];
    const double coeff0_6 = coefficients0[dof][6];
    const double coeff0_7 = coefficients0[dof][7];
    const double coeff0_8 = coefficients0[dof][8];
    const double coeff0_9 = coefficients0[dof][9];
    const double coeff0_10 = coefficients0[dof][10];
    const double coeff0_11 = coefficients0[dof][11];
    const double coeff0_12 = coefficients0[dof][12];
    const double coeff0_13 = coefficients0[dof][13];
    const double coeff0_14 = coefficients0[dof][14];
    const double coeff0_15 = coefficients0[dof][15];
    const double coeff0_16 = coefficients0[dof][16];
    const double coeff0_17 = coefficients0[dof][17];
    const double coeff0_18 = coefficients0[dof][18];
    const double coeff0_19 = coefficients0[dof][19];
    const double coeff0_20 = coefficients0[dof][20];
    const double coeff0_21 = coefficients0[dof][21];
    const double coeff0_22 = coefficients0[dof][22];
    const double coeff0_23 = coefficients0[dof][23];
    const double coeff0_24 = coefficients0[dof][24];
    const double coeff0_25 = coefficients0[dof][25];
    const double coeff0_26 = coefficients0[dof][26];
    const double coeff0_27 = coefficients0[dof][27];
    const double coeff0_28 = coefficients0[dof][28];
    const double coeff0_29 = coefficients0[dof][29];
    const double coeff0_30 = coefficients0[dof][30];
    const double coeff0_31 = coefficients0[dof][31];
    const double coeff0_32 = coefficients0[dof][32];
    const double coeff0_33 = coefficients0[dof][33];
    const double coeff0_34 = coefficients0[dof][34];
    const double coeff0_35 = coefficients0[dof][35];
    const double coeff0_36 = coefficients0[dof][36];
    const double coeff0_37 = coefficients0[dof][37];
    const double coeff0_38 = coefficients0[dof][38];
    const double coeff0_39 = coefficients0[dof][39];
    const double coeff0_40 = coefficients0[dof][40];
    const double coeff0_41 = coefficients0[dof][41];
    const double coeff0_42 = coefficients0[dof][42];
    const double coeff0_43 = coefficients0[dof][43];
    const double coeff0_44 = coefficients0[dof][44];
    const double coeff0_45 = coefficients0[dof][45];
    const double coeff0_46 = coefficients0[dof][46];
    const double coeff0_47 = coefficients0[dof][47];
    const double coeff0_48 = coefficients0[dof][48];
    const double coeff0_49 = coefficients0[dof][49];
    const double coeff0_50 = coefficients0[dof][50];
    const double coeff0_51 = coefficients0[dof][51];
    const double coeff0_52 = coefficients0[dof][52];
    const double coeff0_53 = coefficients0[dof][53];
    const double coeff0_54 = coefficients0[dof][54];
    const double coeff0_55 = coefficients0[dof][55];
    
    // Compute value(s)
    *values = coeff0_0*basisvalue0 + coeff0_1*basisvalue1 + coeff0_2*basisvalue2 + coeff0_3*basisvalue3 + coeff0_4*basisvalue4 + coeff0_5*basisvalue5 + coeff0_6*basisvalue6 + coeff0_7*basisvalue7 + coeff0_8*basisvalue8 + coeff0_9*basisvalue9 + coeff0_10*basisvalue10 + coeff0_11*basisvalue11 + coeff0_12*basisvalue12 + coeff0_13*basisvalue13 + coeff0_14*basisvalue14 + coeff0_15*basisvalue15 + coeff0_16*basisvalue16 + coeff0_17*basisvalue17 + coeff0_18*basisvalue18 + coeff0_19*basisvalue19 + coeff0_20*basisvalue20 + coeff0_21*basisvalue21 + coeff0_22*basisvalue22 + coeff0_23*basisvalue23 + coeff0_24*basisvalue24 + coeff0_25*basisvalue25 + coeff0_26*basisvalue26 + coeff0_27*basisvalue27 + coeff0_28*basisvalue28 + coeff0_29*basisvalue29 + coeff0_30*basisvalue30 + coeff0_31*basisvalue31 + coeff0_32*basisvalue32 + coeff0_33*basisvalue33 + coeff0_34*basisvalue34 + coeff0_35*basisvalue35 + coeff0_36*basisvalue36 + coeff0_37*basisvalue37 + coeff0_38*basisvalue38 + coeff0_39*basisvalue39 + coeff0_40*basisvalue40 + coeff0_41*basisvalue41 + coeff0_42*basisvalue42 + coeff0_43*basisvalue43 + coeff0_44*basisvalue44 + coeff0_45*basisvalue45 + coeff0_46*basisvalue46 + coeff0_47*basisvalue47 + coeff0_48*basisvalue48 + coeff0_49*basisvalue49 + coeff0_50*basisvalue50 + coeff0_51*basisvalue51 + coeff0_52*basisvalue52 + coeff0_53*basisvalue53 + coeff0_54*basisvalue54 + coeff0_55*basisvalue55;
  }

  /// Evaluate all basis functions at given point in cell
  virtual void evaluate_basis_all(double* values,
                                  const double* coordinates,
                                  const ufc::cell& c) const
  {
    throw std::runtime_error("The vectorised version of evaluate_basis() is not yet implemented.");
  }

  /// Evaluate order n derivatives of basis function i at given point in cell
  virtual void evaluate_basis_derivatives(unsigned int i,
                                          unsigned int n,
                                          double* values,
                                          const double* coordinates,
                                          const ufc::cell& c) const
  {
    // Extract vertex coordinates
    const double * const * element_coordinates = c.coordinates;
    
    // Compute Jacobian of affine map from reference cell
    const double J_00 = element_coordinates[1][0] - element_coordinates[0][0];
    const double J_01 = element_coordinates[2][0] - element_coordinates[0][0];
    const double J_02 = element_coordinates[3][0] - element_coordinates[0][0];
    const double J_10 = element_coordinates[1][1] - element_coordinates[0][1];
    const double J_11 = element_coordinates[2][1] - element_coordinates[0][1];
    const double J_12 = element_coordinates[3][1] - element_coordinates[0][1];
    const double J_20 = element_coordinates[1][2] - element_coordinates[0][2];
    const double J_21 = element_coordinates[2][2] - element_coordinates[0][2];
    const double J_22 = element_coordinates[3][2] - element_coordinates[0][2];
      
    // Compute sub determinants
    const double d00 = J_11*J_22 - J_12*J_21;
    const double d01 = J_12*J_20 - J_10*J_22;
    const double d02 = J_10*J_21 - J_11*J_20;
    
    const double d10 = J_02*J_21 - J_01*J_22;
    const double d11 = J_00*J_22 - J_02*J_20;
    const double d12 = J_01*J_20 - J_00*J_21;
    
    const double d20 = J_01*J_12 - J_02*J_11;
    const double d21 = J_02*J_10 - J_00*J_12;
    const double d22 = J_00*J_11 - J_01*J_10;
      
    // Compute determinant of Jacobian
    double detJ = J_00*d00 + J_10*d10 + J_20*d20;
    
    // Compute inverse of Jacobian
    
    // Compute constants
    const double C0 = d00*(element_coordinates[0][0] - element_coordinates[2][0] - element_coordinates[3][0]) \
                    + d10*(element_coordinates[0][1] - element_coordinates[2][1] - element_coordinates[3][1]) \
                    + d20*(element_coordinates[0][2] - element_coordinates[2][2] - element_coordinates[3][2]);
    
    const double C1 = d01*(element_coordinates[0][0] - element_coordinates[1][0] - element_coordinates[3][0]) \
                    + d11*(element_coordinates[0][1] - element_coordinates[1][1] - element_coordinates[3][1]) \
                    + d21*(element_coordinates[0][2] - element_coordinates[1][2] - element_coordinates[3][2]);
    
    const double C2 = d02*(element_coordinates[0][0] - element_coordinates[1][0] - element_coordinates[2][0]) \
                    + d12*(element_coordinates[0][1] - element_coordinates[1][1] - element_coordinates[2][1]) \
                    + d22*(element_coordinates[0][2] - element_coordinates[1][2] - element_coordinates[2][2]);
    
    // Get coordinates and map to the UFC reference element
    double x = (C0 + d00*coordinates[0] + d10*coordinates[1] + d20*coordinates[2]) / detJ;
    double y = (C1 + d01*coordinates[0] + d11*coordinates[1] + d21*coordinates[2]) / detJ;
    double z = (C2 + d02*coordinates[0] + d12*coordinates[1] + d22*coordinates[2]) / detJ;
    
    // Map coordinates to the reference cube
    if (std::abs(y + z - 1.0) < 1e-14)
      x = 1.0;
    else
      x = -2.0 * x/(y + z - 1.0) - 1.0;
    if (std::abs(z - 1.0) < 1e-14)
      y = -1.0;
    else
      y = 2.0 * y/(1.0 - z) - 1.0;
    z = 2.0 * z - 1.0;
    
    // Compute number of derivatives
    unsigned int num_derivatives = 1;
    
    for (unsigned int j = 0; j < n; j++)
      num_derivatives *= 3;
    
    
    // Declare pointer to two dimensional array that holds combinations of derivatives and initialise
    unsigned int **combinations = new unsigned int *[num_derivatives];
        
    for (unsigned int j = 0; j < num_derivatives; j++)
    {
      combinations[j] = new unsigned int [n];
      for (unsigned int k = 0; k < n; k++)
        combinations[j][k] = 0;
    }
        
    // Generate combinations of derivatives
    for (unsigned int row = 1; row < num_derivatives; row++)
    {
      for (unsigned int num = 0; num < row; num++)
      {
        for (unsigned int col = n-1; col+1 > 0; col--)
        {
          if (combinations[row][col] + 1 > 2)
            combinations[row][col] = 0;
          else
          {
            combinations[row][col] += 1;
            break;
          }
        }
      }
    }
    
    // Compute inverse of Jacobian
    const double Jinv[3][3] ={{d00 / detJ, d10 / detJ, d20 / detJ}, {d01 / detJ, d11 / detJ, d21 / detJ}, {d02 / detJ, d12 / detJ, d22 / detJ}};
    
    // Declare transformation matrix
    // Declare pointer to two dimensional array and initialise
    double **transform = new double *[num_derivatives];
        
    for (unsigned int j = 0; j < num_derivatives; j++)
    {
      transform[j] = new double [num_derivatives];
      for (unsigned int k = 0; k < num_derivatives; k++)
        transform[j][k] = 1;
    }
    
    // Construct transformation matrix
    for (unsigned int row = 0; row < num_derivatives; row++)
    {
      for (unsigned int col = 0; col < num_derivatives; col++)
      {
        for (unsigned int k = 0; k < n; k++)
          transform[row][col] *= Jinv[combinations[col][k]][combinations[row][k]];
      }
    }
    
    // Reset values
    for (unsigned int j = 0; j < 1*num_derivatives; j++)
      values[j] = 0;
    
    // Map degree of freedom to element degree of freedom
    const unsigned int dof = i;
    
    // Generate scalings
    const double scalings_y_0 = 1;
    const double scalings_y_1 = scalings_y_0*(0.5 - 0.5*y);
    const double scalings_y_2 = scalings_y_1*(0.5 - 0.5*y);
    const double scalings_y_3 = scalings_y_2*(0.5 - 0.5*y);
    const double scalings_y_4 = scalings_y_3*(0.5 - 0.5*y);
    const double scalings_y_5 = scalings_y_4*(0.5 - 0.5*y);
    const double scalings_z_0 = 1;
    const double scalings_z_1 = scalings_z_0*(0.5 - 0.5*z);
    const double scalings_z_2 = scalings_z_1*(0.5 - 0.5*z);
    const double scalings_z_3 = scalings_z_2*(0.5 - 0.5*z);
    const double scalings_z_4 = scalings_z_3*(0.5 - 0.5*z);
    const double scalings_z_5 = scalings_z_4*(0.5 - 0.5*z);
    
    // Compute psitilde_a
    const double psitilde_a_0 = 1;
    const double psitilde_a_1 = x;
    const double psitilde_a_2 = 1.5*x*psitilde_a_1 - 0.5*psitilde_a_0;
    const double psitilde_a_3 = 1.66666666666667*x*psitilde_a_2 - 0.666666666666667*psitilde_a_1;
    const double psitilde_a_4 = 1.75*x*psitilde_a_3 - 0.75*psitilde_a_2;
    const double psitilde_a_5 = 1.8*x*psitilde_a_4 - 0.8*psitilde_a_3;
    
    // Compute psitilde_bs
    const double psitilde_bs_0_0 = 1;
    const double psitilde_bs_0_1 = 1.5*y + 0.5;
    const double psitilde_bs_0_2 = 0.111111111111111*psitilde_bs_0_1 + 1.66666666666667*y*psitilde_bs_0_1 - 0.555555555555556*psitilde_bs_0_0;
    const double psitilde_bs_0_3 = 0.05*psitilde_bs_0_2 + 1.75*y*psitilde_bs_0_2 - 0.7*psitilde_bs_0_1;
    const double psitilde_bs_0_4 = 0.0285714285714286*psitilde_bs_0_3 + 1.8*y*psitilde_bs_0_3 - 0.771428571428571*psitilde_bs_0_2;
    const double psitilde_bs_0_5 = 0.0185185185185185*psitilde_bs_0_4 + 1.83333333333333*y*psitilde_bs_0_4 - 0.814814814814815*psitilde_bs_0_3;
    const double psitilde_bs_1_0 = 1;
    const double psitilde_bs_1_1 = 2.5*y + 1.5;
    const double psitilde_bs_1_2 = 0.54*psitilde_bs_1_1 + 2.1*y*psitilde_bs_1_1 - 0.56*psitilde_bs_1_0;
    const double psitilde_bs_1_3 = 0.285714285714286*psitilde_bs_1_2 + 2*y*psitilde_bs_1_2 - 0.714285714285714*psitilde_bs_1_1;
    const double psitilde_bs_1_4 = 0.178571428571429*psitilde_bs_1_3 + 1.96428571428571*y*psitilde_bs_1_3 - 0.785714285714286*psitilde_bs_1_2;
    const double psitilde_bs_2_0 = 1;
    const double psitilde_bs_2_1 = 3.5*y + 2.5;
    const double psitilde_bs_2_2 = 1.02040816326531*psitilde_bs_2_1 + 2.57142857142857*y*psitilde_bs_2_1 - 0.551020408163265*psitilde_bs_2_0;
    const double psitilde_bs_2_3 = 0.578703703703704*psitilde_bs_2_2 + 2.29166666666667*y*psitilde_bs_2_2 - 0.712962962962963*psitilde_bs_2_1;
    const double psitilde_bs_3_0 = 1;
    const double psitilde_bs_3_1 = 4.5*y + 3.5;
    const double psitilde_bs_3_2 = 1.51234567901235*psitilde_bs_3_1 + 3.05555555555556*y*psitilde_bs_3_1 - 0.54320987654321*psitilde_bs_3_0;
    const double psitilde_bs_4_0 = 1;
    const double psitilde_bs_4_1 = 5.5*y + 4.5;
    const double psitilde_bs_5_0 = 1;
    
    // Compute psitilde_cs
    const double psitilde_cs_00_0 = 1;
    const double psitilde_cs_00_1 = 2*z + 1;
    const double psitilde_cs_00_2 = 0.3125*psitilde_cs_00_1 + 1.875*z*psitilde_cs_00_1 - 0.5625*psitilde_cs_00_0;
    const double psitilde_cs_00_3 = 0.155555555555556*psitilde_cs_00_2 + 1.86666666666667*z*psitilde_cs_00_2 - 0.711111111111111*psitilde_cs_00_1;
    const double psitilde_cs_00_4 = 0.09375*psitilde_cs_00_3 + 1.875*z*psitilde_cs_00_3 - 0.78125*psitilde_cs_00_2;
    const double psitilde_cs_00_5 = 0.0628571428571429*psitilde_cs_00_4 + 1.88571428571429*z*psitilde_cs_00_4 - 0.822857142857143*psitilde_cs_00_3;
    const double psitilde_cs_01_0 = 1;
    const double psitilde_cs_01_1 = 3*z + 2;
    const double psitilde_cs_01_2 = 0.777777777777778*psitilde_cs_01_1 + 2.33333333333333*z*psitilde_cs_01_1 - 0.555555555555556*psitilde_cs_01_0;
    const double psitilde_cs_01_3 = 0.428571428571429*psitilde_cs_01_2 + 2.14285714285714*z*psitilde_cs_01_2 - 0.714285714285714*psitilde_cs_01_1;
    const double psitilde_cs_01_4 = 0.275*psitilde_cs_01_3 + 2.0625*z*psitilde_cs_01_3 - 0.7875*psitilde_cs_01_2;
    const double psitilde_cs_02_0 = 1;
    const double psitilde_cs_02_1 = 4*z + 3;
    const double psitilde_cs_02_2 = 1.265625*psitilde_cs_02_1 + 2.8125*z*psitilde_cs_02_1 - 0.546875*psitilde_cs_02_0;
    const double psitilde_cs_02_3 = 0.733333333333333*psitilde_cs_02_2 + 2.44444444444444*z*psitilde_cs_02_2 - 0.711111111111111*psitilde_cs_02_1;
    const double psitilde_cs_03_0 = 1;
    const double psitilde_cs_03_1 = 5*z + 4;
    const double psitilde_cs_03_2 = 1.76*psitilde_cs_03_1 + 3.3*z*psitilde_cs_03_1 - 0.54*psitilde_cs_03_0;
    const double psitilde_cs_04_0 = 1;
    const double psitilde_cs_04_1 = 6*z + 5;
    const double psitilde_cs_05_0 = 1;
    const double psitilde_cs_10_0 = 1;
    const double psitilde_cs_10_1 = 3*z + 2;
    const double psitilde_cs_10_2 = 0.777777777777778*psitilde_cs_10_1 + 2.33333333333333*z*psitilde_cs_10_1 - 0.555555555555556*psitilde_cs_10_0;
    const double psitilde_cs_10_3 = 0.428571428571429*psitilde_cs_10_2 + 2.14285714285714*z*psitilde_cs_10_2 - 0.714285714285714*psitilde_cs_10_1;
    const double psitilde_cs_10_4 = 0.275*psitilde_cs_10_3 + 2.0625*z*psitilde_cs_10_3 - 0.7875*psitilde_cs_10_2;
    const double psitilde_cs_11_0 = 1;
    const double psitilde_cs_11_1 = 4*z + 3;
    const double psitilde_cs_11_2 = 1.265625*psitilde_cs_11_1 + 2.8125*z*psitilde_cs_11_1 - 0.546875*psitilde_cs_11_0;
    const double psitilde_cs_11_3 = 0.733333333333333*psitilde_cs_11_2 + 2.44444444444444*z*psitilde_cs_11_2 - 0.711111111111111*psitilde_cs_11_1;
    const double psitilde_cs_12_0 = 1;
    const double psitilde_cs_12_1 = 5*z + 4;
    const double psitilde_cs_12_2 = 1.76*psitilde_cs_12_1 + 3.3*z*psitilde_cs_12_1 - 0.54*psitilde_cs_12_0;
    const double psitilde_cs_13_0 = 1;
    const double psitilde_cs_13_1 = 6*z + 5;
    const double psitilde_cs_14_0 = 1;
    const double psitilde_cs_20_0 = 1;
    const double psitilde_cs_20_1 = 4*z + 3;
    const double psitilde_cs_20_2 = 1.265625*psitilde_cs_20_1 + 2.8125*z*psitilde_cs_20_1 - 0.546875*psitilde_cs_20_0;
    const double psitilde_cs_20_3 = 0.733333333333333*psitilde_cs_20_2 + 2.44444444444444*z*psitilde_cs_20_2 - 0.711111111111111*psitilde_cs_20_1;
    const double psitilde_cs_21_0 = 1;
    const double psitilde_cs_21_1 = 5*z + 4;
    const double psitilde_cs_21_2 = 1.76*psitilde_cs_21_1 + 3.3*z*psitilde_cs_21_1 - 0.54*psitilde_cs_21_0;
    const double psitilde_cs_22_0 = 1;
    const double psitilde_cs_22_1 = 6*z + 5;
    const double psitilde_cs_23_0 = 1;
    const double psitilde_cs_30_0 = 1;
    const double psitilde_cs_30_1 = 5*z + 4;
    const double psitilde_cs_30_2 = 1.76*psitilde_cs_30_1 + 3.3*z*psitilde_cs_30_1 - 0.54*psitilde_cs_30_0;
    const double psitilde_cs_31_0 = 1;
    const double psitilde_cs_31_1 = 6*z + 5;
    const double psitilde_cs_32_0 = 1;
    const double psitilde_cs_40_0 = 1;
    const double psitilde_cs_40_1 = 6*z + 5;
    const double psitilde_cs_41_0 = 1;
    const double psitilde_cs_50_0 = 1;
    
    // Compute basisvalues
    const double basisvalue0 = 0.866025403784439*psitilde_a_0*scalings_y_0*psitilde_bs_0_0*scalings_z_0*psitilde_cs_00_0;
    const double basisvalue1 = 2.73861278752583*psitilde_a_1*scalings_y_1*psitilde_bs_1_0*scalings_z_1*psitilde_cs_10_0;
    const double basisvalue2 = 1.58113883008419*psitilde_a_0*scalings_y_0*psitilde_bs_0_1*scalings_z_1*psitilde_cs_01_0;
    const double basisvalue3 = 1.11803398874989*psitilde_a_0*scalings_y_0*psitilde_bs_0_0*scalings_z_0*psitilde_cs_00_1;
    const double basisvalue4 = 5.1234753829798*psitilde_a_2*scalings_y_2*psitilde_bs_2_0*scalings_z_2*psitilde_cs_20_0;
    const double basisvalue5 = 3.96862696659689*psitilde_a_1*scalings_y_1*psitilde_bs_1_1*scalings_z_2*psitilde_cs_11_0;
    const double basisvalue6 = 2.29128784747792*psitilde_a_0*scalings_y_0*psitilde_bs_0_2*scalings_z_2*psitilde_cs_02_0;
    const double basisvalue7 = 3.24037034920393*psitilde_a_1*scalings_y_1*psitilde_bs_1_0*scalings_z_1*psitilde_cs_10_1;
    const double basisvalue8 = 1.87082869338697*psitilde_a_0*scalings_y_0*psitilde_bs_0_1*scalings_z_1*psitilde_cs_01_1;
    const double basisvalue9 = 1.3228756555323*psitilde_a_0*scalings_y_0*psitilde_bs_0_0*scalings_z_0*psitilde_cs_00_2;
    const double basisvalue10 = 7.93725393319377*psitilde_a_3*scalings_y_3*psitilde_bs_3_0*scalings_z_3*psitilde_cs_30_0;
    const double basisvalue11 = 6.70820393249937*psitilde_a_2*scalings_y_2*psitilde_bs_2_1*scalings_z_3*psitilde_cs_21_0;
    const double basisvalue12 = 5.19615242270663*psitilde_a_1*scalings_y_1*psitilde_bs_1_2*scalings_z_3*psitilde_cs_12_0;
    const double basisvalue13 = 3*psitilde_a_0*scalings_y_0*psitilde_bs_0_3*scalings_z_3*psitilde_cs_03_0;
    const double basisvalue14 = 5.80947501931113*psitilde_a_2*scalings_y_2*psitilde_bs_2_0*scalings_z_2*psitilde_cs_20_1;
    const double basisvalue15 = 4.5*psitilde_a_1*scalings_y_1*psitilde_bs_1_1*scalings_z_2*psitilde_cs_11_1;
    const double basisvalue16 = 2.59807621135332*psitilde_a_0*scalings_y_0*psitilde_bs_0_2*scalings_z_2*psitilde_cs_02_1;
    const double basisvalue17 = 3.67423461417477*psitilde_a_1*scalings_y_1*psitilde_bs_1_0*scalings_z_1*psitilde_cs_10_2;
    const double basisvalue18 = 2.12132034355964*psitilde_a_0*scalings_y_0*psitilde_bs_0_1*scalings_z_1*psitilde_cs_01_2;
    const double basisvalue19 = 1.5*psitilde_a_0*scalings_y_0*psitilde_bs_0_0*scalings_z_0*psitilde_cs_00_3;
    const double basisvalue20 = 11.1242977306435*psitilde_a_4*scalings_y_4*psitilde_bs_4_0*scalings_z_4*psitilde_cs_40_0;
    const double basisvalue21 = 9.81070843517429*psitilde_a_3*scalings_y_3*psitilde_bs_3_1*scalings_z_4*psitilde_cs_31_0;
    const double basisvalue22 = 8.2915619758885*psitilde_a_2*scalings_y_2*psitilde_bs_2_2*scalings_z_4*psitilde_cs_22_0;
    const double basisvalue23 = 6.42261628933256*psitilde_a_1*scalings_y_1*psitilde_bs_1_3*scalings_z_4*psitilde_cs_13_0;
    const double basisvalue24 = 3.70809924354783*psitilde_a_0*scalings_y_0*psitilde_bs_0_4*scalings_z_4*psitilde_cs_04_0;
    const double basisvalue25 = 8.77496438739212*psitilde_a_3*scalings_y_3*psitilde_bs_3_0*scalings_z_3*psitilde_cs_30_1;
    const double basisvalue26 = 7.41619848709566*psitilde_a_2*scalings_y_2*psitilde_bs_2_1*scalings_z_3*psitilde_cs_21_1;
    const double basisvalue27 = 5.74456264653803*psitilde_a_1*scalings_y_1*psitilde_bs_1_2*scalings_z_3*psitilde_cs_12_1;
    const double basisvalue28 = 3.3166247903554*psitilde_a_0*scalings_y_0*psitilde_bs_0_3*scalings_z_3*psitilde_cs_03_1;
    const double basisvalue29 = 6.42261628933256*psitilde_a_2*scalings_y_2*psitilde_bs_2_0*scalings_z_2*psitilde_cs_20_2;
    const double basisvalue30 = 4.9749371855331*psitilde_a_1*scalings_y_1*psitilde_bs_1_1*scalings_z_2*psitilde_cs_11_2;
    const double basisvalue31 = 2.87228132326901*psitilde_a_0*scalings_y_0*psitilde_bs_0_2*scalings_z_2*psitilde_cs_02_2;
    const double basisvalue32 = 4.06201920231798*psitilde_a_1*scalings_y_1*psitilde_bs_1_0*scalings_z_1*psitilde_cs_10_3;
    const double basisvalue33 = 2.34520787991171*psitilde_a_0*scalings_y_0*psitilde_bs_0_1*scalings_z_1*psitilde_cs_01_3;
    const double basisvalue34 = 1.6583123951777*psitilde_a_0*scalings_y_0*psitilde_bs_0_0*scalings_z_0*psitilde_cs_00_4;
    const double basisvalue35 = 14.6458185158768*psitilde_a_5*scalings_y_5*psitilde_bs_5_0*scalings_z_5*psitilde_cs_50_0;
    const double basisvalue36 = 13.2476412994918*psitilde_a_4*scalings_y_4*psitilde_bs_4_1*scalings_z_5*psitilde_cs_41_0;
    const double basisvalue37 = 11.6833214455479*psitilde_a_3*scalings_y_3*psitilde_bs_3_2*scalings_z_5*psitilde_cs_32_0;
    const double basisvalue38 = 9.87420882906575*psitilde_a_2*scalings_y_2*psitilde_bs_2_3*scalings_z_5*psitilde_cs_23_0;
    const double basisvalue39 = 7.64852927038918*psitilde_a_1*scalings_y_1*psitilde_bs_1_4*scalings_z_5*psitilde_cs_14_0;
    const double basisvalue40 = 4.41588043316392*psitilde_a_0*scalings_y_0*psitilde_bs_0_5*scalings_z_5*psitilde_cs_05_0;
    const double basisvalue41 = 12.0933866224478*psitilde_a_4*scalings_y_4*psitilde_bs_4_0*scalings_z_4*psitilde_cs_40_1;
    const double basisvalue42 = 10.6653645038508*psitilde_a_3*scalings_y_3*psitilde_bs_3_1*scalings_z_4*psitilde_cs_31_1;
    const double basisvalue43 = 9.01387818865997*psitilde_a_2*scalings_y_2*psitilde_bs_2_2*scalings_z_4*psitilde_cs_22_1;
    const double basisvalue44 = 6.98212002188447*psitilde_a_1*scalings_y_1*psitilde_bs_1_3*scalings_z_4*psitilde_cs_13_1;
    const double basisvalue45 = 4.03112887414927*psitilde_a_0*scalings_y_0*psitilde_bs_0_4*scalings_z_4*psitilde_cs_04_1;
    const double basisvalue46 = 9.53939201416946*psitilde_a_3*scalings_y_3*psitilde_bs_3_0*scalings_z_3*psitilde_cs_30_2;
    const double basisvalue47 = 8.06225774829855*psitilde_a_2*scalings_y_2*psitilde_bs_2_1*scalings_z_3*psitilde_cs_21_2;
    const double basisvalue48 = 6.2449979983984*psitilde_a_1*scalings_y_1*psitilde_bs_1_2*scalings_z_3*psitilde_cs_12_2;
    const double basisvalue49 = 3.60555127546399*psitilde_a_0*scalings_y_0*psitilde_bs_0_3*scalings_z_3*psitilde_cs_03_2;
    const double basisvalue50 = 6.98212002188447*psitilde_a_2*scalings_y_2*psitilde_bs_2_0*scalings_z_2*psitilde_cs_20_3;
    const double basisvalue51 = 5.40832691319598*psitilde_a_1*scalings_y_1*psitilde_bs_1_1*scalings_z_2*psitilde_cs_11_3;
    const double basisvalue52 = 3.1224989991992*psitilde_a_0*scalings_y_0*psitilde_bs_0_2*scalings_z_2*psitilde_cs_02_3;
    const double basisvalue53 = 4.41588043316392*psitilde_a_1*scalings_y_1*psitilde_bs_1_0*scalings_z_1*psitilde_cs_10_4;
    const double basisvalue54 = 2.54950975679639*psitilde_a_0*scalings_y_0*psitilde_bs_0_1*scalings_z_1*psitilde_cs_01_4;
    const double basisvalue55 = 1.80277563773199*psitilde_a_0*scalings_y_0*psitilde_bs_0_0*scalings_z_0*psitilde_cs_00_5;
    
    // Table(s) of coefficients
    const static double coefficients0[56][56] = \
    {{0.00945067404923504, 0.00609788120679252, 0.003520613356228, 0.00248944957812473, 0.00705946233325038, 0.00546823600997147, 0.00315708753234945, 0.00446479600584757, 0.00257775117585287, 0.00182274533665719, -0.00298267418725718, -0.0025208197798293, -0.00195261860521383, -0.0011273448773449, -0.00218309396769446, -0.00169101731601731, -0.000976309302606924, -0.00138070985681772, -0.000797153207506492, -0.000563672438672437, 0.00278687112460006, 0.00245778931055954, 0.00207721109304565, 0.00160900079398463, 0.000928957041533348, 0.00219831358911339, 0.00185791408306671, 0.00143913406048031, 0.000830884437218262, 0.00160900079398463, 0.00124632665582739, 0.000719567030240153, 0.00101762145320216, 0.000587524019939396, 0.000415442218609115, -0.00705593524717797, -0.0063823335708908, -0.00562868913761196, -0.00475711057319694, -0.00368484200521179, -0.00212744452363026, -0.00582624677716564, -0.00513826668309063, -0.00434262794914383, -0.00336378514516178, -0.00194208225905521, -0.00459580543596519, -0.00388416451811043, -0.00300866089851429, -0.00173705117965752, -0.00336378514516179, -0.00260557676948628, -0.00150433044925714, -0.00212744452363026, -0.00122828066840392, -0.000868525589828755},
    {0.00945067404923484, -0.0060978812067926, 0.00352061335622807, 0.00248944957812473, 0.00705946233325039, -0.00546823600997146, 0.00315708753234942, -0.00446479600584756, 0.00257775117585286, 0.00182274533665715, 0.00298267418725716, -0.00252081977982931, 0.00195261860521383, -0.00112734487734489, -0.00218309396769446, 0.00169101731601731, -0.000976309302606908, 0.00138070985681772, -0.000797153207506487, -0.000563672438672423, 0.00278687112460003, -0.00245778931055954, 0.00207721109304565, -0.00160900079398463, 0.000928957041533354, -0.00219831358911339, 0.00185791408306669, -0.00143913406048031, 0.000830884437218255, 0.00160900079398463, -0.0012463266558274, 0.000719567030240159, -0.00101762145320216, 0.000587524019939409, 0.000415442218609113, 0.00705593524717798, -0.00638233357089081, 0.00562868913761196, -0.00475711057319694, 0.00368484200521178, -0.00212744452363027, -0.00582624677716566, 0.00513826668309062, -0.00434262794914383, 0.00336378514516178, -0.00194208225905522, 0.0045958054359652, -0.00388416451811043, 0.00300866089851429, -0.00173705117965753, -0.00336378514516178, 0.00260557676948629, -0.00150433044925715, 0.00212744452363026, -0.00122828066840393, -0.000868525589828756},
    {0.00945067404923496, 0, -0.00704122671245609, 0.00248944957812476, 0, 0, 0.00947126259704828, 0, -0.00515550235170569, 0.00182274533665715, 0, 0, 0, 0.0045093795093795, 0, 0, -0.00292892790782073, 0, 0.00159430641501296, -0.000563672438672431, 0, 0, 0, 0, 0.00464478520766673, 0, 0, 0, -0.00332353774887306, 0, 0, 0.00215870109072046, 0, -0.00117504803987882, 0.000415442218609132, 0, 0, 0, 0, 0, 0.0127646671417816, 0, 0, 0, 0, -0.00971041129527609, 0, 0, 0, 0.0069482047186301, 0, 0, -0.00451299134777144, 0, 0.00245656133680787, -0.000868525589828766},
    {0.00945067404923495, 0, 0, -0.00746834873437432, 0, 0, 0, 0, 0, 0.0109364720199429, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0.00563672438672439, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0.00623163327913698, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0.0182390373864041},
    {-0.0100234421734309, 0, 0.0374718748311398, -0.0289614183593723, 0, 0, 0.0284137877911449, 0, 0.00515550235170565, -0.00619733414463432, 0, 0, 0, 0.0315656565656566, 0, 0, -0.0113902751970807, 0, 0.00265717735835495, -0.000187890812890785, 0, 0, 0, 0, 0.0232239260383336, 0, 0, 0, -0.00166176887443649, 0, 0, -0.00647610327216139, 0, 0.00646276421933346, -0.00290809553026393, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0.0213629048496073, 0, 0, 0, -0.0277928188745204, 0, 0, 0.0243701532779658, 0, -0.0157219925555703, 0.00607967912880134},
    {0.0100234421734308, 0, -0.0287574853355261, 0.05360943398437, 0, 0, -0.0189425251940965, 0, 0.0154665070551172, 0.0153110608279201, 0, 0, 0, -0.00450937950937953, 0, 0, 0.0370997534990628, 0, -0.0281660799985623, 0.0108976671476671, 0, 0, 0, 0, 0, 0, 0, 0, 0.0299118397398575, 0, 0, -0.00431740218144094, 0, -0.0123380044187275, 0.00872428659079179, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0.0312669212338355, 0, 0, -0.0487403065559315, 0, 0.0412702304583721, -0.018239037386404},
    {0.0100234421734309, 0, 0.0409576306293854, -0.0449826285156208, 0, 0, 0.0189425251940966, 0, -0.00515550235170581, -0.021143845905223, 0, 0, 0, 0, 0, 0, 0.00455611007883228, 0, 0.0356061766019561, -0.0319414381914382, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0.0302218152700864, 0, 0.00411266813957589, -0.0145404776513196, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0.0379091273212801, 0, -0.0550269739444961, 0.0303983956440067},
    {-0.0100234421734309, 0, -0.0148144621425437, 0.0449826285156208, 0, 0, 0, 0, 0.0206220094068229, 0.021143845905223, 0, 0, 0, 0, 0, 0, 0, 0, 0.0106287094334197, 0.0319414381914382, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0.0205633406978791, 0.0145404776513196, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0.03439185871531, -0.0303983956440067},
    {-0.0100234421734309, 0.0324515955311978, -0.01873593741557, -0.0289614183593723, 0.0211783869997511, -0.0164047080299144, 0.00947126259704827, 0.00446479600584751, -0.00257775117585286, -0.00619733414463429, 0.0208787193108001, -0.0176457384588051, 0.0136683302364968, -0.00789141414141412, -0.00848980987436741, 0.00657617845117844, -0.00379675839902691, 0.0023011830946962, -0.00132858867917746, -0.000187890812890819, 0.0139343556230002, -0.0122889465527977, 0.0103860554652283, -0.00804500396992315, 0.00464478520766672, -0.00109915679455667, 0.000928957041533349, -0.000719567030240153, 0.000415442218609154, -0.00482700238195389, 0.00373897996748218, -0.00215870109072046, 0.00559691799261187, -0.00323138210966673, -0.00290809553026394, 0, 0, 0, 0, 0, 0, 0.0128177429097644, -0.0113041867027994, 0.00955378148811641, -0.00740032731935592, 0.00427258096992148, -0.0183832217438608, 0.0155366580724417, -0.0120346435940572, 0.00694820471863012, 0.0181644397838736, -0.014070114555226, 0.00812338442598859, -0.0136156449512337, 0.00786099627778517, 0.00607967912880136},
    {0.0100234421734311, -0.024904712849524, 0.014378742667763, 0.0536094339843701, -0.0141189246665008, 0.0109364720199429, -0.00631417506469887, 0.0133943880175428, -0.00773325352755854, 0.01531106082792, -0.00298267418725714, 0.0025208197798293, -0.00195261860521383, 0.00112734487734485, 0.0276525235907967, -0.0214195526695527, 0.0123665844996876, -0.0243925408037797, 0.0140830399992812, 0.0108976671476671, 0, 0, 0, 0, 0, 0.0197848223020205, -0.0167212267476003, 0.0129522065443228, -0.0074779599349644, -0.00321800158796925, 0.00249265331165481, -0.00143913406048031, -0.0106850252586226, 0.00616900220936375, 0.00872428659079183, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0.0206811244618434, -0.017478740331497, 0.0135389740433143, -0.00781673030845889, -0.0363288795677472, 0.028140229110452, -0.0162467688519772, 0.0357410679969885, -0.0206351152291861, -0.0182390373864041},
    {0.0100234421734311, 0.0354703486038675, -0.0204788153146928, -0.0449826285156207, 0.0141189246665009, -0.0109364720199429, 0.00631417506469889, -0.0044647960058475, 0.00257775117585279, -0.021143845905223, 0, 0, 0, 0, 0.00339592394974698, -0.00263047138047136, 0.00151870335961076, 0.0308358534689291, -0.0178030883009781, -0.0319414381914382, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0.0225260111157848, -0.0174485731815835, 0.0100739384233621, 0.00356167508620756, -0.00205633406978792, -0.0145404776513196, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0.028255795219359, -0.0218868448636849, 0.01263637577376, -0.047654757329318, 0.0275134869722481, 0.0303983956440068},
    {-0.0100234421734311, -0.0128297005588457, 0.00740723107127181, 0.0449826285156207, 0, 0, 0, 0.0178591840233903, -0.0103110047034114, 0.0211438459052229, 0, 0, 0, 0, 0, 0, 0, 0.0092047323787848, -0.00531435471670985, 0.0319414381914382, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0.0178083754310377, -0.0102816703489396, 0.0145404776513196, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0.0297842233308237, -0.017195929357655, -0.0303983956440068},
    {-0.0100234421734311, 0.0324515955311977, -0.0335503995581136, -0.00801060507812428, 0.0211783869997512, 0.00109364720199428, -0.00568275755822898, -0.0169662248222208, 0.00567105258687634, 0.0083846285486229, 0.0208787193108001, -0.0109235523792603, 0.00499002532443531, -0.00187890812890811, -0.0162519217595033, 0.00958243145743145, -0.00466458889023307, 0.0108922666482287, -0.00558007245254534, -0.00469727032227035, 0.0139343556230002, -0.00245778931055952, -0.00415442218609131, 0.0064360031759385, -0.00464478520766676, -0.0120907247401236, 0.00464478520766674, -0.000719567030240154, -0.000415442218609135, 0.00965400476390777, -0.00498530662330959, 0.00215870109072046, -0.00661453944581401, 0.0032313821096667, 0.00290809553026394, 0, 0.0127646671417816, -0.0202632808954031, 0.0228341307513453, -0.020635115229186, 0.0127646671417816, -0.00116524935543312, -0.00719357335632687, 0.0112908326677739, -0.0114368694935501, 0.00737991258440983, 0.00183832217438608, 0.00310733161448834, -0.00481385743762287, 0.00347410235931505, -0.00201827108709706, -0.000521115353897253, 0.000902598269554298, 0.00170195561890421, -0.000491312267361565, -0.000868525589828764},
    {0.0100234421734313, -0.0249047128495239, 0.0553363732971484, -0.00431340273437463, -0.0141189246665008, 0.0153110608279201, 0.00883984509057839, 0.00803663281052565, -0.0159820572902878, 0.000729098134662871, -0.00298267418725714, 0.0277290175781224, -0.0301571095694136, 0.0191648629148629, -0.00145539597846295, -0.0116492303992304, 0.00976309302606917, 0.00352848074520082, 0.00132858867917745, -0.00263047138047135, 0, 0.0196623144844763, -0.00727023882565982, -0.00724050357293083, 0.00928957041533347, -0.00219831358911338, -0.0148633126645336, 0.0129522065443228, -0.00664707549774609, 0.00402250198496158, 0.00685479660705067, -0.00575653624192123, -0.00457929653940971, -0.000293762009969677, 0.0029080955302639, 0, 0, 0.0202632808954031, -0.0399597288148543, 0.0464290092656686, -0.031911667854454, 0, -0.00411061334647249, -0.00303983956440069, 0.00908221989193683, -0.00776832903622088, 0.000459580543596524, 0.00427258096992149, -0.00270779480866288, 0.000868525589828788, -0.00100913554354855, -0.00234501909253767, 0.00180519653910856, 0.00127646671417817, 0.000245656133680783, -0.000868525589828766},
    {0.0100234421734309, 0.0354703486038674, -0.0492363006502187, -0.00431340273437457, 0.0141189246665008, -0.00656188321196574, -0.016416855168217, -0.00982255121286468, 0.0149509568199466, 0.00072909813466289, 0, 0.00336109303977242, 0.0282044909641998, -0.0353234728234728, -0.000485131992821028, -0.0124007936507936, 0.00889526253486298, 0.00291483191994853, 0.00239145962251946, -0.0026304713804714, 0, 0, 0.0218107164769794, -0.00402250198496158, -0.00928957041533346, 0, -0.00557374224920008, -0.0143913406048031, 0.0141250354327105, 0.000804500396992295, 0.00934744991870546, -0.00287826812096058, -0.00254405363300538, -0.00381890612960614, 0.00290809553026392, 0, 0, 0, 0.0266398192099029, -0.0515877880729651, 0.0425488904726054, 0, 0, -0.00911951869320201, 0.0084094628629044, -0.0038841645181104, 0, 0.00233049871086625, 0.00300866089851431, -0.00347410235931508, -0.000336378514516169, -0.00286613444643491, 0.00120346435940571, 0.000850977809452102, 0.000982624534723146, -0.000868525589828757},
    {-0.0100234421734311, -0.0128297005588456, 0.0448791059024117, -0.00801060507812423, 0, 0.0174983552319087, 0.0132597676358676, -0.00357183680467805, -0.0175287079957994, 0.00838462854862291, 0, 0, 0.00867830491206144, 0.0225468975468975, 0, -0.00300625300625305, -0.019200749617936, 0.000613648825252313, 0.0122230158484327, -0.00469727032227032, 0, 0, 0, 0.0160900079398463, 0.00464478520766673, 0, 0, -0.00719567030240156, -0.0108014976838374, 0, 0.00249265331165479, 0.0107935054536023, -0.000508810726601082, -0.00734405024924251, 0.00290809553026393, 0, 0, 0, 0, 0.0257938940364825, -0.031911667854454, 0, 0, 0, -0.0134551405806471, 0.0135945758133865, 0, 0, 0.00601732179702858, -0.00347410235931504, 0, -0.00208446141558903, -0.000902598269554288, 0.000425488904726059, 0.0017195929357655, -0.000868525589828767},
    {-0.010023442173431, -0.0324515955311979, -0.0187359374155701, -0.0289614183593722, 0.0211783869997511, 0.0164047080299144, 0.00947126259704825, -0.0044647960058475, -0.00257775117585286, -0.00619733414463431, -0.0208787193108001, -0.0176457384588052, -0.0136683302364968, -0.00789141414141413, -0.00848980987436743, -0.00657617845117846, -0.00379675839902691, -0.00230118309469614, -0.00132858867917743, -0.000187890812890788, 0.0139343556230002, 0.0122889465527977, 0.0103860554652283, 0.00804500396992312, 0.00464478520766673, 0.00109915679455666, 0.00092895704153335, 0.000719567030240155, 0.000415442218609149, -0.0048270023819539, -0.00373897996748222, -0.00215870109072046, -0.00559691799261192, -0.00323138210966673, -0.00290809553026393, 0, 0, 0, 0, 0, 0, 0.0128177429097644, 0.0113041867027994, 0.00955378148811642, 0.00740032731935589, 0.00427258096992148, 0.0183832217438608, 0.0155366580724417, 0.0120346435940572, 0.00694820471863011, 0.0181644397838736, 0.014070114555226, 0.00812338442598858, 0.0136156449512337, 0.00786099627778515, 0.00607967912880134},
    {0.0100234421734311, 0.024904712849524, 0.014378742667763, 0.05360943398437, -0.0141189246665008, -0.0109364720199429, -0.00631417506469881, -0.0133943880175428, -0.00773325352755853, 0.01531106082792, 0.00298267418725717, 0.0025208197798293, 0.00195261860521382, 0.00112734487734485, 0.0276525235907967, 0.0214195526695527, 0.0123665844996876, 0.0243925408037797, 0.0140830399992811, 0.0108976671476671, 0, 0, 0, 0, 0, -0.0197848223020205, -0.0167212267476003, -0.0129522065443227, -0.0074779599349644, -0.00321800158796927, -0.00249265331165477, -0.00143913406048032, 0.0106850252586227, 0.00616900220936373, 0.00872428659079179, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, -0.0206811244618434, -0.017478740331497, -0.0135389740433143, -0.00781673030845888, -0.0363288795677472, -0.028140229110452, -0.0162467688519772, -0.0357410679969885, -0.020635115229186, -0.018239037386404},
    {0.0100234421734307, -0.0354703486038675, -0.0204788153146927, -0.0449826285156207, 0.0141189246665008, 0.0109364720199429, 0.00631417506469886, 0.00446479600584751, 0.00257775117585285, -0.0211438459052229, 0, 0, 0, 0, 0.00339592394974697, 0.00263047138047138, 0.00151870335961076, -0.0308358534689291, -0.017803088300978, -0.0319414381914382, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0.0225260111157848, 0.0174485731815835, 0.0100739384233621, -0.00356167508620757, -0.00205633406978789, -0.0145404776513196, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0.0282557952193589, 0.0218868448636849, 0.01263637577376, 0.047654757329318, 0.027513486972248, 0.0303983956440067},
    {-0.010023442173431, 0.0128297005588457, 0.00740723107127182, 0.0449826285156208, 0, 0, 0, -0.0178591840233902, -0.0103110047034114, 0.021143845905223, 0, 0, 0, 0, 0, 0, 0, -0.00920473237878481, -0.00531435471670985, 0.0319414381914381, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, -0.0178083754310377, -0.0102816703489396, 0.0145404776513196, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, -0.0297842233308237, -0.017195929357655, -0.0303983956440068},
    {-0.0100234421734306, -0.0324515955311977, -0.0335503995581136, -0.00801060507812422, 0.0211783869997512, -0.00109364720199428, -0.00568275755822895, 0.0169662248222208, 0.00567105258687631, 0.00838462854862288, -0.0208787193108001, -0.0109235523792604, -0.00499002532443533, -0.00187890812890813, -0.0162519217595033, -0.00958243145743147, -0.00466458889023305, -0.0108922666482287, -0.00558007245254534, -0.00469727032227036, 0.0139343556230002, 0.00245778931055952, -0.00415442218609133, -0.00643600317593853, -0.00464478520766674, 0.0120907247401236, 0.00464478520766673, 0.000719567030240151, -0.000415442218609145, 0.00965400476390776, 0.00498530662330958, 0.00215870109072046, 0.006614539445814, 0.00323138210966669, 0.00290809553026394, 0, 0.0127646671417816, 0.0202632808954031, 0.0228341307513453, 0.020635115229186, 0.0127646671417816, -0.00116524935543314, 0.00719357335632687, 0.0112908326677739, 0.01143686949355, 0.00737991258440983, -0.00183832217438609, 0.00310733161448835, 0.00481385743762286, 0.00347410235931504, -0.00201827108709706, 0.00052111535389726, 0.000902598269554298, -0.00170195561890421, -0.000491312267361563, -0.000868525589828763},
    {0.0100234421734315, 0.024904712849524, 0.0553363732971485, -0.00431340273437464, -0.0141189246665008, -0.0153110608279201, 0.00883984509057836, -0.00803663281052558, -0.0159820572902878, 0.000729098134662864, 0.00298267418725715, 0.0277290175781224, 0.0301571095694136, 0.0191648629148629, -0.00145539597846298, 0.0116492303992304, 0.00976309302606916, -0.00352848074520084, 0.00132858867917744, -0.00263047138047133, 0, -0.0196623144844763, -0.0072702388256598, 0.00724050357293084, 0.0092895704153335, 0.00219831358911338, -0.0148633126645336, -0.0129522065443228, -0.0066470754977461, 0.00402250198496159, -0.00685479660705068, -0.00575653624192122, 0.0045792965394097, -0.000293762009969664, 0.00290809553026391, 0, 0, -0.0202632808954031, -0.0399597288148543, -0.0464290092656686, -0.031911667854454, 0, 0.00411061334647249, -0.00303983956440066, -0.0090822198919368, -0.00776832903622086, -0.000459580543596519, 0.0042725809699215, 0.00270779480866287, 0.000868525589828796, -0.00100913554354854, 0.00234501909253766, 0.00180519653910856, -0.00127646671417816, 0.00024565613368078, -0.00086852558982877},
    {0.0100234421734312, -0.0354703486038675, -0.0492363006502186, -0.0043134027343746, 0.0141189246665008, 0.00656188321196572, -0.016416855168217, 0.0098225512128646, 0.0149509568199467, 0.000729098134662864, 0, 0.0033610930397724, -0.0282044909641998, -0.0353234728234728, -0.000485131992820996, 0.0124007936507936, 0.00889526253486298, -0.00291483191994851, 0.00239145962251946, -0.00263047138047141, 0, 0, 0.0218107164769794, 0.00402250198496156, -0.00928957041533348, 0, -0.0055737422492001, 0.0143913406048031, 0.0141250354327105, 0.00080450039699229, -0.00934744991870546, -0.00287826812096059, 0.00254405363300538, -0.00381890612960615, 0.00290809553026392, 0, 0, 0, 0.0266398192099029, 0.0515877880729651, 0.0425488904726054, 0, 0, -0.00911951869320204, -0.00840946286290444, -0.00388416451811041, 0, 0.00233049871086624, -0.0030086608985143, -0.00347410235931509, -0.000336378514516169, 0.00286613444643491, 0.00120346435940572, -0.000850977809452107, 0.000982624534723155, -0.000868525589828754},
    {-0.010023442173431, 0.0128297005588457, 0.0448791059024117, -0.00801060507812423, 0, -0.0174983552319087, 0.0132597676358676, 0.0035718368046781, -0.0175287079957995, 0.00838462854862291, 0, 0, -0.00867830491206147, 0.0225468975468975, 0, 0.00300625300625302, -0.019200749617936, -0.000613648825252327, 0.0122230158484327, -0.00469727032227032, 0, 0, 0, -0.0160900079398463, 0.00464478520766674, 0, 0, 0.00719567030240152, -0.0108014976838374, 0, -0.00249265331165479, 0.0107935054536023, 0.000508810726601078, -0.00734405024924251, 0.00290809553026392, 0, 0, 0, 0, -0.0257938940364826, -0.031911667854454, 0, 0, 0, 0.0134551405806471, 0.0135945758133865, 0, 0, -0.00601732179702859, -0.00347410235931504, 0, 0.00208446141558904, -0.000902598269554294, -0.000425488904726048, 0.0017195929357655, -0.000868525589828768},
    {-0.0100234421734308, -0.0452812960900435, -0.0113287063442981, -0.00801060507812425, -0.00141189246665006, 0.0164047080299144, 0.0145226026488073, 0.0133943880175427, 0.0118576554089232, 0.0083846285486229, -0.00894802256177148, -0.0142846454190327, -0.014536160727703, -0.00939454064454062, -0.0123708658169354, -0.0125886844636845, -0.00813591085505763, -0.0102786178229763, -0.00664294339588729, -0.00469727032227033, -0.00836061337380013, 0.00245778931055953, 0.00830884437218264, 0.00965400476390776, 0.00650269929073342, 0.00219831358911336, 0.00743165633226678, 0.00863480436288185, 0.00581619106052787, 0.00643600317593851, 0.00747795993496438, 0.00503696921168107, 0.00610572871921291, 0.00411266813957582, 0.00290809553026393, 0.0352796762358899, 0.0191470007126724, 0.00788016479265675, 0.000951422114639387, -0.00221090520312706, -0.00212744452363027, 0.017478740331497, 0.00719357335632688, 0.000868525589828766, -0.00201827108709706, -0.00194208225905523, 0.00643412761035128, 0.00077683290362208, -0.00180519653910858, -0.00173705117965754, 0.000672757029032357, -0.00156334606169177, -0.00150433044925715, -0.00127646671417815, -0.00122828066840394, -0.000868525589828763},
    {0.010023442173431, 0.0603750614533912, -0.00610007264692973, -0.0043134027343746, -0.00564756986660029, -0.0218729440398858, 0.00126283501293976, -0.0178591840233903, 0.00103110047034112, 0.00072909813466285, 0.0417574386216002, 0.0168054651988621, -0.000867830491206138, -0.00526094276094278, 0.0145539597846298, -0.000751563251563223, -0.00455611007883228, -0.000613648825252315, -0.00372004830169692, -0.00263047138047137, 0.00557374224920009, -0.0196623144844763, -0.0103860554652283, 0.00321800158796927, 0.00650269929073345, -0.0175865087129071, -0.00928957041533348, 0.00287826812096062, 0.00581619106052783, -0.00804500396992314, 0.0024926533116548, 0.00503696921168107, 0.00203524290640432, 0.00411266813957582, 0.00290809553026392, -0.0705593524717798, -0.0127646671417816, 0.00450295131008957, 0.00380568845855756, -0.000736968401042374, -0.00212744452363027, -0.0116524935543313, 0.0041106133464725, 0.00347410235931506, -0.000672757029032359, -0.0019420822590552, 0.00367664434877216, 0.00310733161448835, -0.000601732179702866, -0.00173705117965752, 0.00269102811612943, -0.000521115353897279, -0.00150433044925714, -0.000425488904726059, -0.00122828066840393, -0.000868525589828762},
    {0.010023442173431, -0.0603750614533913, -0.00610007264692972, -0.00431340273437459, -0.00564756986660034, 0.0218729440398858, 0.00126283501293975, 0.0178591840233902, 0.00103110047034114, 0.000729098134662857, -0.0417574386216002, 0.016805465198862, 0.000867830491206142, -0.00526094276094274, 0.0145539597846298, 0.000751563251563242, -0.00455611007883226, 0.000613648825252326, -0.0037200483016969, -0.00263047138047136, 0.0055737422492001, 0.0196623144844763, -0.0103860554652283, -0.00321800158796928, 0.00650269929073344, 0.0175865087129071, -0.00928957041533347, -0.00287826812096062, 0.00581619106052785, -0.00804500396992314, -0.0024926533116548, 0.00503696921168109, -0.00203524290640433, 0.00411266813957582, 0.00290809553026392, 0.0705593524717798, -0.0127646671417816, -0.00450295131008957, 0.00380568845855755, 0.000736968401042381, -0.00212744452363028, -0.0116524935543313, -0.00411061334647249, 0.00347410235931505, 0.00067275702903236, -0.00194208225905522, -0.00367664434877216, 0.00310733161448835, 0.000601732179702862, -0.00173705117965754, 0.00269102811612941, 0.000521115353897267, -0.00150433044925715, 0.000425488904726055, -0.00122828066840394, -0.00086852558982877},
    {-0.0100234421734311, 0.0452812960900435, -0.0113287063442981, -0.00801060507812424, -0.00141189246665007, -0.0164047080299144, 0.0145226026488074, -0.0133943880175427, 0.0118576554089232, 0.00838462854862294, 0.00894802256177145, -0.0142846454190327, 0.014536160727703, -0.00939454064454065, -0.0123708658169354, 0.0125886844636845, -0.00813591085505765, 0.0102786178229763, -0.00664294339588732, -0.00469727032227034, -0.00836061337380011, -0.00245778931055953, 0.00830884437218264, -0.00965400476390776, 0.00650269929073342, -0.00219831358911338, 0.00743165633226677, -0.00863480436288184, 0.00581619106052786, 0.00643600317593851, -0.00747795993496437, 0.00503696921168106, -0.0061057287192129, 0.00411266813957581, 0.00290809553026391, -0.0352796762358899, 0.0191470007126724, -0.00788016479265674, 0.000951422114639392, 0.00221090520312706, -0.00212744452363026, 0.017478740331497, -0.00719357335632687, 0.000868525589828777, 0.00201827108709706, -0.00194208225905521, -0.00643412761035126, 0.000776832903622074, 0.00180519653910858, -0.00173705117965752, 0.00067275702903235, 0.00156334606169178, -0.00150433044925714, 0.00127646671417815, -0.00122828066840393, -0.000868525589828756},
    {0.0787556170769578, 0.0679219441350652, -0.00435719474780692, -0.00308100195312471, 0.0705946233325039, -0.0328094160598287, 0.0126283501293978, -0.0267887760350854, 0.03608851646194, -0.00729098134662863, 0.0298267418725716, 0.0151249186789758, -0.0325436434202305, 0.0248015873015872, 0.0130985638061668, -0.0229226791726792, 0.0169226945785199, -0.0248527774227189, 0.0164744996218005, 0.0142797017797018, 0, 0.0245778931055953, -0.0363511941282991, 0.0362025178646542, -0.0232239260383337, 0.0219831358911338, -0.00743165633226679, 0, 0.00166176887443652, -0.0345935170706695, 0.0130864298861876, -0.00359783515120077, 0.0335815079556711, -0.014688100498485, -0.0178640154001927, 0, 0, 0, 0, 0, 0, 0, 0.0226083734055987, -0.0334382352084074, 0.0333014729371016, -0.0213629048496074, -0.00459580543596519, -0.0194208225905522, 0.0270779480866286, -0.0191075629762328, 0.0090822198919368, 0.00703505727761299, -0.00812338442598857, -0.0102117337134253, 0.00196524906944628, 0.00607967912880137},
    {-0.0214788046573525, 0.00377344134083702, 0.00217859737390337, -0.021567013671873, 0, 0.0984282481794862, -0.0852413633734345, -0.0312535720409329, -0.0180442582309699, 0.0309866707231716, 0, 0.0453747560369276, 0, -0.0202922077922077, -0.0116431678277039, 0.0236742424242425, -0.0335199527228374, -0.00138070985681772, -0.000797153207506455, 0.00394570707070704, 0, 0, 0.0363511941282991, -0.0603375297744235, 0.0464478520766673, 0, 0.0241528830798671, 0, -0.0108014976838374, -0.00724050357293081, -0.0392592896585629, 0.0259044130886455, 0.0188259968842399, 0.0108691943688789, -0.0178640154001926, 0, 0, 0, 0, 0, 0, 0, 0, 0.0334382352084074, -0.0555024548951693, 0.0427258096992148, 0, -0.0155366580724417, 0, 0.00694820471863008, 0.00302740663064561, 0.0164151336477636, -0.0108311792346514, -0.00680782247561684, -0.00393049813889257, 0.00607967912880131},
    {0.0787556170769581, 0.0301875307266957, 0.0610007264692977, -0.00308100195312485, 0, 0.0218729440398858, 0.0757701007763862, 0.0178591840233902, -0.0412440188136458, -0.00729098134662867, 0, 0, 0.0520698294723689, -0.00901875901875909, 0, -0.0127765752765753, 0.0286384062098029, 0.00184094647575695, -0.0297603864135753, 0.0142797017797018, 0, 0, 0, 0.0402250198496157, -0.0464478520766674, 0, 0, 0.014391340604803, 0.0182794576188018, 0, -0.0137095932141014, -0.0345392174515274, 0.00407048581280862, 0.036426489236243, -0.0178640154001927, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0.0370016365967796, -0.0427258096992148, 0, 0, -0.030086608985143, 0.0243187165152054, 0, 0.014070114555226, 0, -0.00340391123780844, -0.00982624534723146, 0.00607967912880138},
    {-0.0214788046573519, 0.00377344134083692, -0.0196073763651313, 0.00924300585937408, 0, -0.0109364720199429, 0.00947126259704824, 0.102690308134494, -0.0695992817480272, -0.060150596109686, 0, -0.00504163955965862, 0.0065087286840461, -0.00450937950937948, 0.0465726713108155, 0.0214195526695527, -0.0289638426440052, -0.00782402252196701, -0.0263060558477138, 0.000563672438672498, 0, 0, 0, 0, 0, 0, 0.0334424534952005, -0.0431740218144092, 0.0299118397398575, 0.0289620142917233, -0.00249265331165477, -0.00431740218144094, -0.0605484764655283, 0.0185070066280912, 0.0461140862656136, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0.0349574806629939, -0.0451299134777144, 0.0312669212338355, -0.0121096265225824, -0.028140229110452, 0.0270779480866286, 0.023827378664659, 0, -0.0182390373864041},
    {-0.0214788046573522, -0.0150937653633478, 0.0130715842434209, 0.00924300585937409, 0, -0.0109364720199429, 0.00947126259704823, -0.00892959201169517, 0.123732056440937, -0.060150596109686, 0, 0, -0.00650872868404615, 0.00676406926406924, 0, 0.0574945887445888, 0.0126920209338899, -0.026693723898476, 0.00637722566005184, 0.000563672438672442, 0, 0, 0, 0, 0, 0, 0, 0.0431740218144092, -0.0448677596097863, 0, 0.0199412264932383, 0.0215870109072046, -0.0142467003448301, -0.0616900220936373, 0.0461140862656136, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0.0451299134777144, -0.0469003818507533, 0, -0.0375203054806026, 0.0162467688519772, 0.0119136893323295, 0.0206351152291861, -0.0182390373864041},
    {0.078755617076958, 0.0301875307266957, 0.0174287789912279, 0.0585390371093694, 0, 0.0218729440398858, -0.0189425251940966, 0.0178591840233902, 0.0103110047034113, 0.083846285486229, 0, 0, 0, 0, 0, 0.00526094276094275, -0.00455611007883227, 0.0533874477969518, 0.0308232573569172, -0.0319414381914382, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0.0348971463631671, -0.0302218152700865, 0.0284934006896603, 0.0164506725583033, -0.0643935438844155, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0.0437736897273697, -0.0379091273212801, -0.023827378664659, -0.0137567434861241, 0.0303983956440068},
    {0.0787556170769583, -0.0679219441350652, -0.00435719474780694, -0.00308100195312467, 0.0705946233325039, 0.0328094160598287, 0.0126283501293977, 0.0267887760350853, 0.03608851646194, -0.00729098134662864, -0.0298267418725716, 0.0151249186789759, 0.0325436434202305, 0.0248015873015873, 0.0130985638061668, 0.0229226791726792, 0.0169226945785199, 0.0248527774227189, 0.0164744996218005, 0.0142797017797018, 0, -0.0245778931055954, -0.036351194128299, -0.0362025178646541, -0.0232239260383336, -0.0219831358911338, -0.00743165633226678, 0, 0.00166176887443654, -0.0345935170706695, -0.0130864298861876, -0.00359783515120077, -0.0335815079556711, -0.014688100498485, -0.0178640154001927, 0, 0, 0, 0, 0, 0, 0, -0.0226083734055987, -0.0334382352084074, -0.0333014729371016, -0.0213629048496074, 0.00459580543596522, -0.0194208225905522, -0.0270779480866286, -0.0191075629762328, 0.0090822198919368, -0.007035057277613, -0.00812338442598858, 0.0102117337134253, 0.00196524906944627, 0.00607967912880136},
    {-0.0214788046573522, -0.00377344134083698, 0.00217859737390352, -0.0215670136718729, 0, -0.0984282481794863, -0.0852413633734345, 0.031253572040933, -0.0180442582309699, 0.0309866707231716, 0, 0.0453747560369276, 0, -0.0202922077922078, -0.0116431678277039, -0.0236742424242425, -0.0335199527228375, 0.00138070985681774, -0.000797153207506485, 0.00394570707070704, 0, 0, 0.0363511941282991, 0.0603375297744236, 0.0464478520766674, 0, 0.024152883079867, 0, -0.0108014976838374, -0.0072405035729308, 0.0392592896585629, 0.0259044130886455, -0.0188259968842399, 0.0108691943688789, -0.0178640154001927, 0, 0, 0, 0, 0, 0, 0, 0, 0.0334382352084074, 0.0555024548951693, 0.0427258096992148, 0, -0.0155366580724417, 0, 0.0069482047186301, 0.0030274066306456, -0.0164151336477636, -0.0108311792346515, 0.00680782247561683, -0.00393049813889256, 0.00607967912880134},
    {0.078755617076958, -0.0301875307266958, 0.0610007264692976, -0.00308100195312472, 0, -0.0218729440398859, 0.0757701007763862, -0.0178591840233903, -0.0412440188136458, -0.00729098134662865, 0, 0, -0.0520698294723688, -0.00901875901875904, 0, 0.0127765752765752, 0.0286384062098029, -0.00184094647575696, -0.0297603864135752, 0.0142797017797018, 0, 0, 0, -0.0402250198496157, -0.0464478520766673, 0, 0, -0.0143913406048031, 0.0182794576188018, 0, 0.0137095932141014, -0.0345392174515273, -0.00407048581280864, 0.036426489236243, -0.0178640154001927, 0, 0, 0, 0, 0, 0, 0, 0, 0, -0.0370016365967795, -0.0427258096992148, 0, 0, 0.0300866089851429, 0.0243187165152054, 0, -0.014070114555226, 0, 0.00340391123780844, -0.00982624534723146, 0.00607967912880137},
    {-0.0214788046573522, -0.00377344134083677, -0.0196073763651313, 0.00924300585937403, 0, 0.0109364720199429, 0.00947126259704829, -0.102690308134494, -0.0695992817480271, -0.0601505961096861, 0, -0.00504163955965865, -0.00650872868404618, -0.00450937950937947, 0.0465726713108155, -0.0214195526695527, -0.0289638426440052, 0.00782402252196705, -0.0263060558477139, 0.000563672438672498, 0, 0, 0, 0, 0, 0, 0.0334424534952005, 0.0431740218144092, 0.0299118397398575, 0.0289620142917233, 0.00249265331165484, -0.00431740218144093, 0.0605484764655283, 0.0185070066280911, 0.0461140862656136, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0.0349574806629939, 0.0451299134777144, 0.0312669212338355, -0.0121096265225824, 0.028140229110452, 0.0270779480866286, -0.023827378664659, 0, -0.018239037386404},
    {-0.0214788046573521, 0.0150937653633479, 0.0130715842434208, 0.00924300585937423, 0, 0.010936472019943, 0.00947126259704825, 0.00892959201169522, 0.123732056440937, -0.060150596109686, 0, 0, 0.00650872868404613, 0.00676406926406924, 0, -0.0574945887445887, 0.0126920209338899, 0.0266937238984759, 0.00637722566005184, 0.000563672438672452, 0, 0, 0, 0, 0, 0, 0, -0.0431740218144092, -0.0448677596097863, 0, -0.0199412264932383, 0.0215870109072046, 0.0142467003448301, -0.0616900220936373, 0.0461140862656136, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, -0.0451299134777144, -0.0469003818507533, 0, 0.0375203054806026, 0.0162467688519772, -0.0119136893323295, 0.0206351152291861, -0.0182390373864041},
    {0.0787556170769579, -0.0301875307266956, 0.017428778991228, 0.0585390371093696, 0, -0.0218729440398858, -0.0189425251940965, -0.0178591840233903, 0.0103110047034114, 0.083846285486229, 0, 0, 0, 0, 0, -0.00526094276094277, -0.00455611007883227, -0.0533874477969518, 0.0308232573569172, -0.0319414381914383, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, -0.0348971463631671, -0.0302218152700865, -0.0284934006896603, 0.0164506725583033, -0.0643935438844154, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, -0.0437736897273697, -0.0379091273212801, 0.023827378664659, -0.0137567434861241, 0.0303983956440068},
    {0.078755617076958, -0.0377344134083697, -0.0566435317214906, -0.00308100195312479, 0.0423567739995023, 0.0546823600997145, 0.0378850503881931, 0.0446479600584756, 0.00515550235170578, -0.00729098134662856, 0.0417574386216002, -0.00504163955965862, -0.0247331689993752, -0.0202922077922078, 0.0295930515620807, 0.0101461038961039, 0.00216957622801541, 0.0266937238984759, 0.0132858867917747, 0.0142797017797018, -0.0557374224920008, -0.0245778931055953, -0.00519302773261415, 0.00402250198496156, 0.00464478520766674, -0.00219831358911332, -0.0130053985814669, -0.0158304746652834, -0.0108014976838375, -0.0168945083368386, -0.026796023100289, -0.0194283098164841, -0.0295110221428625, -0.0217383887377579, -0.0178640154001927, 0, 0, 0, 0, 0, 0, -0.0512709716390577, -0.0226083734055987, -0.00477689074405822, 0.00370016365967794, 0.00427258096992147, -0.0367664434877216, -0.00776832903622086, 0.00601732179702859, 0.00694820471863011, -0.00908221989193683, 0.00703505727761299, 0.00812338442598859, 0.00680782247561683, 0.00786099627778515, 0.00607967912880135},
    {-0.0214788046573522, 0, -0.00435719474780689, -0.0215670136718729, -0.127070321998507, 0, 0.0284137877911447, 0, 0.0360885164619399, 0.0309866707231715, 0, 0.0453747560369275, 0, -0.0202922077922078, -0.0422064833754266, 0, -0.00618329224984384, 0, 0.0015943064150129, 0.00394570707070703, 0.0836061337380012, 0, -0.0103860554652283, 0, 0.00464478520766673, 0, 0.024152883079867, 0, -0.0108014976838374, 0.043443021437585, 0, -0.0194283098164841, 0, -0.0217383887377578, -0.0178640154001926, 0, 0, 0, 0, 0, 0, 0.0769064574585866, 0, -0.00955378148811639, 0, 0.00427258096992146, 0, -0.0155366580724417, 0, 0.00694820471863015, -0.0181644397838736, 0, 0.00812338442598858, 0, 0.00786099627778515, 0.00607967912880134},
    {0.0787556170769581, 0.0377344134083697, -0.0566435317214905, -0.00308100195312477, 0.0423567739995024, -0.0546823600997146, 0.0378850503881931, -0.0446479600584756, 0.00515550235170579, -0.00729098134662863, -0.0417574386216002, -0.00504163955965864, 0.0247331689993752, -0.0202922077922078, 0.0295930515620807, -0.0101461038961039, 0.00216957622801542, -0.0266937238984759, 0.0132858867917747, 0.0142797017797018, -0.0557374224920008, 0.0245778931055954, -0.00519302773261416, -0.00402250198496156, 0.00464478520766676, 0.00219831358911337, -0.0130053985814669, 0.0158304746652834, -0.0108014976838375, -0.0168945083368386, 0.026796023100289, -0.0194283098164841, 0.0295110221428625, -0.0217383887377579, -0.0178640154001927, 0, 0, 0, 0, 0, 0, -0.0512709716390577, 0.0226083734055987, -0.00477689074405822, -0.00370016365967795, 0.00427258096992149, 0.0367664434877216, -0.00776832903622086, -0.00601732179702859, 0.00694820471863009, -0.00908221989193678, -0.00703505727761301, 0.00812338442598859, -0.00680782247561686, 0.00786099627778517, 0.00607967912880135},
    {-0.021478804657352, -0.0188672067041847, 0.00653579212171052, 0.00924300585937401, 0.0141189246665008, 0, -0.00315708753234946, -0.111619900146189, -0.05413277469291, -0.0601505961096861, -0.00894802256177153, -0.00252081977982931, 0.000650872868404653, 0.00112734487734485, -0.0276525235907967, 0.0360750360750361, 0.0374251899332651, -0.0188697013765088, 0.019928830187662, 0.000563672438672417, 0, 0, 0, 0, 0, 0.0593544669060614, 0.0167212267476003, -0.00431740218144091, -0.0074779599349644, 0.00321800158796925, 0.0224338798048931, 0.018708742786244, 0.0463017761206981, 0.0431830154655461, 0.0461140862656137, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0.0620433733855302, 0.017478740331497, -0.00451299134777146, -0.0078167303084589, 0.0363288795677472, -0.00938007637015066, -0.0162467688519772, -0.0119136893323295, -0.020635115229186, -0.018239037386404},
    {-0.021478804657352, 0.0188672067041848, 0.00653579212171056, 0.00924300585937413, 0.0141189246665008, 0, -0.00315708753234944, 0.111619900146189, -0.05413277469291, -0.060150596109686, 0.00894802256177151, -0.00252081977982929, -0.000650872868404627, 0.00112734487734488, -0.0276525235907967, -0.0360750360750361, 0.0374251899332651, 0.0188697013765088, 0.0199288301876619, 0.000563672438672459, 0, 0, 0, 0, 0, -0.0593544669060614, 0.0167212267476003, 0.00431740218144093, -0.0074779599349644, 0.00321800158796927, -0.0224338798048931, 0.018708742786244, -0.046301776120698, 0.0431830154655461, 0.0461140862656136, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, -0.0620433733855302, 0.017478740331497, 0.00451299134777145, -0.00781673030845891, 0.0363288795677472, 0.00938007637015065, -0.0162467688519772, 0.0119136893323295, -0.020635115229186, -0.0182390373864041},
    {0.0787556170769582, 0, -0.0348575579824558, 0.0585390371093696, -0.0282378493330016, 0, 0.00631417506469889, 0, -0.0206220094068229, 0.083846285486229, 0, 0, 0, 0, -0.00679184789949391, 0, 0.00151870335961076, 0, -0.0616465147138344, -0.0319414381914382, 0, 0, 0, 0, 0, 0, 0, 0, 0, -0.0450520222315696, 0, 0.0100739384233621, 0, -0.0329013451166065, -0.0643935438844154, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, -0.0565115904387179, 0, 0.01263637577376, 0, 0.027513486972248, 0.0303983956440067},
    {0.0787556170769578, -0.0377344134083695, -0.0217859737390347, -0.0523770332031201, 0.0423567739995023, 0.0546823600997146, 0, 0.0446479600584757, 0.0257775117585286, 0.0291639253865144, 0.0417574386216002, 0.0285692908380655, 0.0273366604729936, 0.0187890812890813, -0.00921750786359888, -0.00789141414141413, -0.00347132196482459, -0.0248527774227189, -0.0143487577351166, -0.015031265031265, -0.0557374224920008, -0.00491557862111907, -0.0197335053839337, -0.0394205194526234, -0.0325134964536672, -0.0241814494802472, -0.00928957041533347, -0.0158304746652834, -0.0141250354327105, -0.00241350119097692, -0.000623163327913714, -0.00215870109072046, 0.00712335017241511, 0.00411266813957583, 0.00540074884191872, 0, -0.0510586685671264, -0.0405265617908061, -0.0114170653756727, 0.010317557614593, 0.0127646671417816, 0.00466099742173251, -0.0143871467126538, -0.00564541633388696, 0.00571843474677503, 0.00737991258440982, 0.00367664434877214, -0.00155366580724416, 0.00240692871881144, 0.00347410235931507, 0.00100913554354852, 0.000260557676948644, 0.000902598269554298, -0.000850977809452118, -0.000491312267361587, -0.000868525589828769},
    {-0.0214788046573518, 0, -0.0217859737390347, 0.00308100195312481, -0.127070321998507, 0, 0.0473563129852414, 0, 0.0257775117585285, 0.0127592173566, 0, -0.0352914769176103, 0, 0.00225468975468975, 0.0509388592462044, 0, -0.00943765659186686, 0, -0.0143487577351166, -0.0129644660894661, 0.0836061337380013, 0, 0.0477758551400502, 0, -0.0325134964536671, 0, 0.00928957041533347, 0, -0.0141250354327104, -0.0144810071458617, 0, -0.00215870109072047, 0, 0.00411266813957582, 0.00540074884191876, 0, 0.0765880028506897, 0, -0.0228341307513453, 0, 0.0127646671417816, -0.00699149613259876, 0, -0.0112908326677739, 0, 0.00737991258440979, 0, -0.00310733161448836, 0, 0.00347410235931506, 0.00201827108709705, 0, 0.000902598269554284, 0, -0.000491312267361592, -0.000868525589828792},
    {0.0787556170769577, 0.0377344134083696, -0.0217859737390349, -0.05237703320312, 0.0423567739995023, -0.0546823600997146, 0, -0.0446479600584756, 0.0257775117585286, 0.0291639253865144, -0.0417574386216002, 0.0285692908380655, -0.0273366604729936, 0.0187890812890813, -0.0092175078635989, 0.00789141414141413, -0.00347132196482458, 0.0248527774227189, -0.0143487577351166, -0.015031265031265, -0.0557374224920009, 0.00491557862111903, -0.0197335053839338, 0.0394205194526234, -0.0325134964536671, 0.0241814494802472, -0.00928957041533348, 0.0158304746652834, -0.0141250354327105, -0.00241350119097695, 0.000623163327913703, -0.00215870109072045, -0.00712335017241511, 0.00411266813957582, 0.00540074884191871, 0, -0.0510586685671264, 0.0405265617908061, -0.0114170653756726, -0.010317557614593, 0.0127646671417816, 0.0046609974217325, 0.0143871467126537, -0.00564541633388698, -0.00571843474677501, 0.00737991258440984, -0.00367664434877218, -0.00155366580724417, -0.00240692871881142, 0.00347410235931505, 0.00100913554354854, -0.000260557676948633, 0.000902598269554291, 0.000850977809452119, -0.000491312267361577, -0.000868525589828757},
    {-0.0214788046573522, -0.0188672067041847, 0.0108929868695173, 0.00308100195312474, 0.0141189246665008, -0.109364720199429, -0.0789271883087357, 0.0223239800292378, -0.0128887558792642, 0.0127592173566001, -0.00894802256177151, -0.0277290175781224, -0.00585785581564148, 0.0191648629148629, 0.00145539597846299, 0.0383297258297258, 0.0348216984596467, -0.0124263887113595, 0.00717437886755833, -0.0129644660894661, 0, 0.0589869434534289, 0.00727023882565978, 0.0506835250105158, 0.0650269929073344, -0.00659494076734013, 0.0148633126645336, -0.00431740218144089, -0.00166176887443655, -0.0040225019849616, -0.00810112326287806, -0.0115130724838424, 0.00356167508620753, -0.00205633406978792, 0.00540074884191869, 0, 0, 0.0607898426862092, 0.0399597288148543, -0.0154763364218895, -0.031911667854454, 0, -0.0123318400394175, 0.00303983956440066, -0.00302740663064559, -0.00776832903622087, 0.00137874163078958, -0.0042725809699215, 0.000902598269554286, 0.000868525589828745, 0.00100913554354854, 0.000781673030845884, 0.00180519653910853, -0.000425488904726043, 0.0002456561336808, -0.000868525589828759},
    {-0.0214788046573523, 0.0188672067041848, 0.0108929868695173, 0.00308100195312473, 0.0141189246665008, 0.109364720199429, -0.0789271883087356, -0.0223239800292378, -0.0128887558792643, 0.0127592173566, 0.00894802256177147, -0.0277290175781224, 0.00585785581564147, 0.0191648629148629, 0.00145539597846298, -0.0383297258297258, 0.0348216984596467, 0.0124263887113595, 0.00717437886755829, -0.0129644660894661, 0, -0.0589869434534289, 0.0072702388256598, -0.0506835250105158, 0.0650269929073343, 0.00659494076734013, 0.0148633126645336, 0.0043174021814409, -0.00166176887443651, -0.00402250198496155, 0.00810112326287803, -0.0115130724838425, -0.00356167508620755, -0.0020563340697879, 0.00540074884191873, 0, 0, -0.0607898426862092, 0.0399597288148542, 0.0154763364218895, -0.031911667854454, 0, 0.0123318400394175, 0.00303983956440069, 0.0030274066306456, -0.00776832903622087, -0.00137874163078958, -0.00427258096992146, -0.000902598269554311, 0.000868525589828782, 0.00100913554354855, -0.000781673030845884, 0.00180519653910857, 0.000425488904726043, 0.000245656133680809, -0.000868525589828767},
    {0.0787556170769581, 0, 0.0435719474780695, -0.0523770332031201, -0.0282378493330016, 0, 0.0631417506469885, 0, -0.0515550235170572, 0.0291639253865144, 0, -0.00672218607954481, 0, -0.0601250601250601, 0.000970263985641982, 0, -0.0125835421224891, 0, 0.0286975154702333, -0.015031265031265, 0, 0, -0.0436214329539588, 0, -0.0650269929073343, 0, 0.0111474844984002, 0, 0.031573608614294, -0.00160900079398462, 0, -0.0028782681209606, 0, -0.00822533627915164, 0.00540074884191871, 0, 0, 0, -0.0532796384198057, 0, 0.0425488904726053, 0, 0, 0.018239037386404, 0, -0.00388416451811043, 0, -0.00466099742173252, 0, -0.00347410235931507, 0.00067275702903234, 0, 0.00120346435940573, 0, 0.000982624534723144, -0.000868525589828764},
    {0.10739402328676, -0.188672067041848, -0.108929868695174, -0.0770250488281177, -0.0705946233325038, -0.0546823600997147, -0.0315708753234941, -0.0446479600584757, -0.0257775117585286, -0.0182274533665716, 0.0894802256177148, -0.0151249186789758, 0.058578558156415, 0.0744047619047618, -0.0130985638061668, -0.0281836219336219, 0.00976309302606916, 0.069035492840886, 0.0398576603753241, 0.0597492784992785, 0, 0.0737336793167861, 0.036351194128299, -0.0120675059548847, -0.0232239260383337, 0.0659494076734016, 0.00743165633226678, 0.0431740218144092, 0.0465295284842228, 0.0345935170706695, 0.0155790831978425, 0.0251848460584054, -0.0254405363300539, -0.0146881004984851, -0.0328199352701214, 0, 0, 0, 0, 0, 0, 0, 0.0678251202167962, 0.0334382352084074, -0.0111004909790339, -0.0213629048496074, -0.0137874163078956, 0.0194208225905522, -0.0090259826955429, -0.0191075629762328, -0.00908221989193678, -0.00234501909253769, -0.00812338442598858, 0.00340391123780841, 0.00196524906944627, 0.00607967912880134},
    {0.107394023286761, 0.188672067041848, -0.108929868695174, -0.0770250488281178, -0.0705946233325039, 0.0546823600997145, -0.0315708753234942, 0.0446479600584757, -0.0257775117585287, -0.0182274533665715, -0.0894802256177147, -0.0151249186789758, -0.0585785581564149, 0.0744047619047619, -0.0130985638061668, 0.0281836219336219, 0.00976309302606916, -0.0690354928408859, 0.039857660375324, 0.0597492784992785, 0, -0.0737336793167861, 0.036351194128299, 0.0120675059548847, -0.0232239260383338, -0.0659494076734015, 0.00743165633226678, -0.0431740218144092, 0.0465295284842228, 0.0345935170706695, -0.0155790831978424, 0.0251848460584053, 0.0254405363300539, -0.0146881004984851, -0.0328199352701214, 0, 0, 0, 0, 0, 0, 0, -0.0678251202167962, 0.0334382352084074, 0.0111004909790339, -0.0213629048496074, 0.0137874163078956, 0.0194208225905522, 0.00902598269554287, -0.0191075629762328, -0.0090822198919368, 0.00234501909253767, -0.00812338442598857, -0.0034039112378084, 0.00196524906944627, 0.00607967912880135},
    {0.107394023286761, 0, 0.217859737390348, -0.0770250488281178, 0, 0, -0.094712625970483, 0, 0.0515550235170572, -0.0182274533665716, 0, -0.0907495120738551, 0, -0.0946969696969696, 0.0232863356554077, 0, -0.0227805503941613, 0, -0.0797153207506479, 0.0597492784992786, 0, 0, -0.0727023882565981, 0, 0.0464478520766674, 0, -0.0483057661597341, 0, -0.0781031370985168, 0.0144810071458616, 0, 0.0431740218144092, 0, 0.0293762009969701, -0.0328199352701215, 0, 0, 0, 0, 0, 0, 0, 0, -0.0668764704168148, 0, 0.0427258096992148, 0, 0.0310733161448835, 0, 0.00694820471863007, -0.00605481326129118, 0, -0.0108311792346515, 0, -0.00393049813889257, 0.00607967912880135},
    {0.107394023286761, 0, 0, 0.231075146484353, 0, 0, 0, 0, 0, -0.109364720199429, 0, 0.0100832791193172, 0, -0.00450937950937963, -0.093145342621631, 0, -0.104139658944738, 0, 0, -0.09244227994228, 0, 0, 0, 0, 0, 0, -0.066884906990401, 0, 0.0299118397398574, -0.0579240285834467, 0, -0.0647610327216138, 0, 0, 0.0835038859404354, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, -0.0699149613259879, 0, 0.0312669212338356, 0.0242192530451648, 0, 0.0270779480866286, 0, 0, -0.018239037386404}};
    
    // Interesting (new) part
    // Tables of derivatives of the polynomial base (transpose)
    const static double dmats0[56][56] = \
    {{0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {6.32455532033671, 0, 0, 0, 0, 2.16505338810566e-14, 0, 2.05845987409731e-14, -1.22512335797976e-14, 0, 0, 0, 0, 0, -1.08130730953582e-14, 0, 1.22483177886604e-14, 0, 1.21041660596838e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {-1.33727360005496e-14, 11.2249721603218, 0, 0, 0, 2.8506046608589e-14, 0, 0, 0, 1.00951115474313e-14, 0, -1.20188905885721e-14, 1.28545373789999e-14, 0, 0, -1.03476262582923e-14, -1.93999491268998e-14, 0, 1.33226762955019e-14, 0, 0, 0, 0, 0, 1.05444038416428e-14, 0, 0, 0, -1.00620217474173e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1.11065094110678e-14, 0, 0, -1.90621990730363e-14, 0, 0, 1.25288297745861e-14, 0, 0, 0, 0, 0, 0, 0},
    {4.58257569495582, -1.0968053520467e-14, 8.36660026534075, -1.18321595661994, 0, 0, -1.51387532776558e-14, -1.08221538572235e-14, -1.3598597955937e-14, 0, 0, 1.9215300364784e-14, -1.53957147424772e-14, 0, 0, 1.50088321802921e-14, 0, 0, 0, -1.15463194561016e-14, 0, 0, 0, 0, 0, 0, 0, -1.00882470358913e-14, 0, 0, 0, 0, 0, 0, 0, 0, 1.03041185216246e-14, 0, 0, 0, 0, 0, 0, 0, 1.67984122495906e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {3.74165738677394, 2.85608062261247e-14, 0, 8.69482604771366, 0, 1.32677370521961e-14, -1.0621406876971e-14, 4.08006961549745e-14, -2.2818496382273e-14, -2.1316282072803e-14, 0, 0, 0, 0, 0, -1.21576837027352e-14, 0, 1.41223042954593e-14, 1.22383070012588e-14, -1.4210854715202e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, -1.00375396573897e-14, -1.10000775654917e-14, 0, -1.24344978758018e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, -1.27154464053485e-14, 0, -1.33226762955019e-14},
    {0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {5.49909083394704, -2.12464983341981e-14, -3.34664010613631, -2.36643191323983, 15.4919333848297, 0, 0.692820323027548, 0, 0.565685424949223, 0.400000000000019, -2.45617804595055e-14, 0, -2.50285594161414e-14, -1.12752625494543e-14, 0, -2.06682744450517e-14, 0, -1.87825158914691e-14, 0, 0, 0, 1.67613011937654e-14, 0, 0, 0, 1.28202275780632e-14, 0, 1.30009839200864e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, -1.30459382160744e-14, 0, 0, 0, 0, -1.95146235819213e-14, 1.32110885228657e-14, 1.12246955104063e-14, 0, -1.24476276034392e-14, 0, 0, 0, 1.14879018579048e-14, 0, 0, 0, 0, 0},
    {1.79012076899773e-14, 4.89897948556635, 0, 0, 0, 14.1985914794391, -1.56346770721582e-14, -0.828078671210841, 1.85962356624714e-14, 1.44765989981636e-14, 0, -3.14433996145488e-14, 0, -1.75618818945982e-14, -1.31955099699921e-14, 1.85934501523347e-14, -2.7953864447905e-14, 1.28785870856518e-14, 0, 0, 0, 0, 1.3985009962376e-14, 0, 1.27337992225105e-14, 0, 1.07957902261976e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, -1.64679520687763e-14, 0, -1.14163879478899e-14, 0, 0, -1.9487535909084e-14, 1.99307532306351e-14, -2.46558752706505e-14, 0, 0, 0, 0, 0, 1.11086317443674e-14, 0, 0, 0, 0},
    {3.60000000000003, -1.17043800208557e-14, 8.76356092008264, -1.54919333848296, 0, 0, 9.52470471983252, -1.08725201100446e-14, -1.48131215963609, 0.261861468283195, 0, 0, -2.19166032081878e-14, -1.15727480921906e-14, 1.19782412678522e-14, 0, 2.00140531387981e-14, -1.08284340452512e-14, -2.62934736398987e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1.25371390349952e-14, 0, 0, 0, 0, 0, -1.02110492017178e-14, 0, 0, 0, 0, 0, 0},
    {0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {-2.24915431235802e-14, 4.24264068711927, -1.64313007644523e-14, 0, 0, 0, 0, 14.3427433120127, 0, 1.59620734906468e-14, 0, 0, 2.06494753825246e-14, 1.24642807480246e-14, 1.31854538806711e-14, 0, -2.70659809367255e-14, 0, 0, -1.23240404410758e-14, 0, 0, 0, 0, 0, 0, 0, 0, -1.22125558154546e-14, 1.05003199257295e-14, 1.39537903134404e-14, 1.71734205981969e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, -2.01310662662892e-14, 0, 0, 2.30351452905449e-14, 0, 0, 0, 0, 0, 1.24344978758018e-14, 0},
    {3.11769145362398, 0, 3.1622776601684, 4.91934955049952, 0, 0, 0, 0, 10.690449676497, -2.41897262725906, 0, 0, 0, 1.08320679531897e-14, 0, 0, 0, 1.61780317385165e-14, 0, -3.01980662698043e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, -1.01163085491832e-14, 0, -1.4210854715202e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1.90254351599586e-14, 0, 0, 0, 1.4669610635169e-14, 0, 0, 0, 1.20972007395846e-14, 0, 0, 0},
    {0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {2.54558441227159, 0, 0, 7.66811580507233, 0, 0, 0, 2.00435576705123e-14, 0, 10.3691851174526, 0, 0, 0, 0, 0, 0, -1.4355744756654e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, -1.31237403589518e-14, 0, 1.19255832274785e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, -1.21989789544854e-14, 0, 0, 0, 1.08631431175004e-14, 0, 0, 0, 0, 0, 0},
    {0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {-3.7592379574637e-14, 11.60576914948, 1.02695629777827e-14, 0, 0, -4.20458932936044, 1.40755215458979e-14, -3.43303281162799, 0, 0, 19.6214168703486, 2.18203847862408e-14, 0.611677741841185, 1.46622784386203e-14, 0, 0.529728463363973, 1.12642251693005e-14, 0.4325214791568, 0, 0, 0, -1.06703789507439e-14, 0, 0, 0, -1.60649692254054e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1.42508255988556e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {4.53137212479104, -2.04494956755329e-14, 3.54562104171167, -2.50713268211203, 4.78713553878169, 0, -5.56626745075489, 0, -1.04880884817014, 0.741619848709575, -1.23815345194237e-14, 18.8034951158403, -1.86020833228553e-14, 1.68183573174416, -0.603122387046451, -1.92746782669555e-14, 0.701283781217653, 0, 0.132137494528672, -0.0934353184302359, 0, 1.50506010183261e-14, 0, 0, 0, 1.11149238093515e-14, 0, 1.50741577923534e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, -1.5968916436806e-14, 0, 0, 0, 0, -2.36522650283687e-14, 1.26860679795035e-14, 2.21112666692775e-14, -1.04107559329747e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {3.86209880120533e-14, 2.5951288749407, 1.92068583260152e-14, 0, 0, 10.0285307284481, -1.22612314755764e-14, -0.767649473578743, 0, 0, 0, 0, 16.4130361329658, 0, 0, -1.2634761106116, 3.26893402596665e-14, 0.0967147429123276, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1.32878527298855e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, -1.0672045061505e-14, -1.16637535122366e-14, 0, -1.46741880066399e-14, 0, 1.32138836425176e-14, 0, 0, 0, -1.00831659427246e-14, 0, 0, 0, 0, 0, 0, 0},
    {2.96647939483827, 0, 8.12403840463593, -1.64130361329658, 0, 0, 11.7728501222092, 0, -2.40312296813959, 0.485504156227621, 0, 0, -1.71460988515286e-14, 10.2761860629321, 1.14956049951209e-14, -4.33332801490326e-14, -1.48323969741912, -1.03124879870092e-14, 0.302765035409724, -0.0611677741841183, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, -1.45169999216629e-14, 1.16636407607716e-14, -2.16593088290316e-14, 0, 0, 0, 0, -1.3706959798735e-14, 0, 0, 0, 0, 0, 0},
    {0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {4.05298244095222, -1.17512299762369e-14, -1.58564993434419, 4.48489528465113, 4.28174419288836, 0, 0.191485421551272, 0, -3.75233260785876, -4.311612227462, 0, 0, -1.13529814871025e-14, -1.07967706476489e-14, 18.8807171866293, 0, 0.844371341865059, -1.30185990386282e-14, 1.18187368057055, 1.04463861754669, 0, 1.00351859283595e-14, 0, 0, 0, 1.08982318008379e-14, 1.16650614658273e-14, 1.78540699708776e-14, 0, 1.48568586111212e-14, 1.2997651195193e-14, 0, 1.19564095870861e-14, 0, 0, 0, 0, 0, 0, 0, 0, -1.90031310112405e-14, -2.21313572206035e-14, 0, 2.31096376853888e-14, 1.25147787012743e-14, -1.31451704746157e-14, 0, 0, 0, 2.34657400816165e-14, 0, 0, 0, 1.19833135941395e-14, 0},
    {1.03681424251779e-14, 2.321153829896, -1.04360964314765e-14, 0, 0, 3.9242833740697, 0, 5.49285249860475, 0, 1.4249489099847e-14, 0, 0, 2.63684141763982e-14, 1.12010640489931e-14, 0, 17.3044631365566, -5.0266260224736e-14, -1.73008591662709, 0, 0, 0, 0, 0, 0, 1.32329702395699e-14, 0, 1.22894929912907e-14, -1.21668635551694e-14, -1.35421297848238e-14, 1.0253781445641e-14, 0, 1.86415806813024e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 2.41268605947696e-14, -3.37685959257577e-14, 0, 0, 3.2419745025707e-14, 1.21885930822672e-14, 0, 1.77958086184916e-14, 0, 1.4210854715202e-14, 0, 1.40814441510978e-14},
    {2.65329983228435, -2.32098024377698e-14, 4.15220619990512, 2.93605316083775, 0, -1.72381743916861e-14, 2.63248931621763, -4.00364084983131e-14, 9.82591326180784, -2.8226127713774, 0, 0, -1.33474731826653e-14, -1.06959103616322e-14, 1.26654039377864e-14, 1.8991099966876e-14, 11.6081867662439, -1.19927215290095e-14, -3.0948717731947, 0.683876505540241, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1.61148409137901e-14, 1.47100077738277e-14, 0, 0, 0, 0, 0, 0, 0, 1.3422167174583e-14, 0, -2.15269669668842e-14, 0},
    {0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {4.0410983709019e-14, 2.01017818278147, 0, 1.34362847627492e-14, 0, -2.17732253774783e-14, 0, 8.91928088068604, -1.37667655053519e-14, 1.38253977761529e-14, -1.02564879604251e-14, 0, 0, 0, -1.34085149628415e-14, 0, 0, 16.8558564884489, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1.06581410364015e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, -1.48981943277725e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1.77635683940025e-14, 0, 0},
    {2.29782505861523, 1.22923050669652e-14, 1.49829835452879, 5.50917601898536, 0, 0, 0, 1.92682707206214e-14, 6.64803945320939, 4.13676892548485, 0, 0, 1.46016900962614e-14, 2.50030852312528e-14, 0, -1.88410641965856e-14, -3.22541831300375e-14, -1.10413917361675e-14, 12.5636136423842, -3.55352656109507, 0, 0, 0, 0, 0, 0, 0, 0, -2.00805899834693e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, -1.13841034187579e-14, -2.03312427847762e-14, 0, 0, 0, 1.35488987469781e-14, 0, 0, 0, 0, -1.41455019594718e-14, 0},
    {0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {1.87616630392938, -2.83080569710049e-14, 0, 6.22830929985772, 0, 0, 0, -1.88930188353776e-14, 0, 11.0541524195339, 0, 1.07055222640856e-14, 0, 0, 0, 1.48136724622646e-14, 1.71418057121951e-14, -2.40309133366388e-14, 0, 11.6057691494799, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1.22846189228214e-14, 1.32604072649903e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1.33226762955019e-14},
    {0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {4.83186700722507, 0, -4.41087092453692, -3.11895674167863, 16.6749979177077, -1.13866638474088e-14, 2.13065307820926, -2.61564103865851e-14, 1.73967095350101, 1.23013312825381, 0, -4.36653943835008, 1.40983965277648e-14, -0.557935943461962, -3.78153408023779, 2.08111552908862e-14, -0.483186700722529, 1.18208720943676e-14, -0.394520289089675, -0.278967971730977, 23.6981011897578, 0, 0.490653381462656, 0, 0.0626933893908963, -1.13263908790947e-14, 0.438853725736244, 0, 0.0560746721671562, 0.380058475033041, 0, 0.0485620906056454, 0, 0.0396507809421992, 0.0280373360835894, 0, 0, 0, 0, 0, 0, 1.56357143454011e-14, 0, 0, 0, 0, 1.0335662317795e-14, 1.60193852711159e-14, 0, 0, 0, 1.11786220690221e-14, -1.25974506359966e-14, 0, -1.04950117655873e-14, 0},
    {-2.48352929483769e-14, 6.91050664139874, 0, -1.11694286112743e-14, 0, 10.5706241650195, 1.37569619238853e-14, -2.72554057547698, -1.5404344466674e-14, 0, 4.67332857821917, 1.98201416293709e-14, -7.86705867811461, 0, 0, -2.39719223514615, 0, 0.61809450436526, 0, 0, 0, 23.10555488766, 0, 1.62065746295907, 0, -0.469687194424188, 0, 0.790669146636838, 1.33915447641953e-14, 0, 0.240926884676765, 0, -0.0621208350291002, 0, 0, 0, 1.5285431037408e-14, -1.09568490725226e-14, 0, 0, 0, 0, 0, -1.60712929692335e-14, 0, 1.01513003014644e-14, 0, 0, -1.13906592977489e-14, 0, 0, 0, 0, 0, 0, 0},
    {3.85449644663769, 0, 5.98171817966647, -2.48806675764062, 1.90029237516522, 0, -0.339934634239515, 0, -2.35922146606158, 0.981306762925306, -1.25696577846526e-14, 9.9522670305624, 1.59597845699183e-14, -6.45364422706461, -0.430945803685653, 1.41669885314633e-14, 0.0770899289327383, 0, 0.535021138879222, -0.222539456105695, 0, 0, 21.5273297444417, 0, 2.75066129614998, 0, -1.00024047132728, 0, 0.648615649442928, 0.0433116829031283, 0, -0.00774782937530501, 0, -0.053771647653677, 0.0223660568773448, 0, 0, 0, 0, 0, -1.57026876352916e-14, 0, 0, 0, 0, 0, 0, 1.17596913617043e-14, 0, 0, 0, 0, -1.17877079382012e-14, 0, 0, 0},
    {0, 1.54523626091315, 0, 0, 0, 6.71778024562961, 1.24434270987286e-14, -0.609449400220049, -2.44890913103646e-14, 0, 0, 1.86912392652466e-14, 14.6593983227032, 0, 1.30441346035566e-14, -1.52344936219876, 0, 0.138210132827974, 0, 0, 0, 0, 0, 18.119501277094, 0, 0, 0, -1.47332496632642, 0, 0, 0.153112421864233, 0, -0.0138906409944119, 0, 0, 0, 0, 0, 1.22183321677886e-14, 0, 0, 0, -1.08769537630915e-14, 0, -1.31873092152821e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {2.52336024752224, 0, 7.37120708415863, -1.62882203585592, 0, 0, 12.0171306297064, 0, -2.90724328050878, 0.642416074439641, 0, 0, 1.07383829658909e-14, 13.9858820944259, 0, -2.32400745727179e-14, -2.72522906732405, 0, 0.659300804656265, -0.145686271816941, 0, 0, 0, 0, 10.8043830863937, 0, 0, 0, -1.40563403846548, 0, 0, 0.273895826790425, 0, -0.0662622240310308, 0.0146420212340205, 0, 0, 0, 0, 0, -2.09114976580726e-14, 0, 0, 0, 0, -2.35127496508513e-14, 0, 0, 0, -1.14219479838847e-14, 0, 0, 0, 0, 0, 0},
    {0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0, 6.30840061880561, 0, 0, 0, -1.52362350055011, 0, 11.1963004093827, -2.55351295663786e-14, 0, 4.26614580154032, 0, 0.132992762321598, 0, 2.56609067489954e-14, -4.72217953828914, 1.17310134406447e-14, -6.67684597013019, 0, 1.60758884410055e-14, 0, 0, 0, 0, 0, 23.1532444221696, 0, 0.72177887856059, 1.28783366735277e-14, -1.53368636742207e-14, 1.1112512795529, -1.22624479997289e-14, 1.19087439227729, 0, 0, 0, 0, 0, 0, 0, 0, 2.28291451277892e-14, 0, 0, 0, 1.34418865877235e-14, 2.02478204822475e-14, 0, -1.70814325121815e-14, -1.2090896650498e-14, 0, 0, 0, -2.48689957516035e-14, 0, 0},
    {3.518657752745, 0, 1.92724822331884, 2.72554057547703, 1.73472166622177, 0, -2.01705689521107, 0, 3.42052627529744, -4.03112887414927, 0, 4.08831086321546, -1.50711171565645e-14, 0.365669640132009, 5.37643791520789, -1.10002031399277e-14, -6.25148194071022, 0, -2.03981013744553, 1.8080332206528, 0, 1.12311783689995e-14, -1.2495355609558e-14, 0, 0, 0, 22.1880978975596, 2.59644251000699e-14, 1.98456380761453, -1.2652152389098, 0, 1.47113578579291, 0, 0.363818136386349, -0.294009452979948, 0, 0, 0, 0, 0, 0, 0, -2.14091784397362e-14, 3.96979729470341e-14, 2.95273442389967e-14, 0, -1.02898405303999e-14, -1.55944604137083e-14, 0, -1.2756671985334e-14, 2.67341313377078e-14, 0, 0, 0, 0, 0},
    {5.00588290781391e-14, 1.4106012612951, 1.86517468137026e-14, 1.57048767420779e-14, 1.14431516135412e-14, 3.63405410063598, 0, 2.50356888118883, 0, 0, 0, 0, 3.56857028479905, 0, 0, 11.263055412878, 3.38127349244425e-14, -1.49298814645067, 0, 2.47950570155456e-14, 0, 0, 0, 0, 0, 0, 0, 19.3673596462205, 0, 0, -2.65048896124153, 0, 0.266287609379583, 0, 0, 0, 0, 0, 0, 0, 0, 0, -1.24805331057479e-14, 0, 2.67117428927039e-14, 0, 0, 0, -2.83074697144157e-14, 0, 0, 0, 0, 0, 0, 0},
    {2.30350221379958, 0, 4.4158804331639, 1.78428514239955, 0, 0, 4.2661458015403, 1.92100483859941e-14, 7.83741028656788, -2.63899331455873, 0, 0, 0, 2.23427840700302, 1.30548943777412e-14, -2.38078955158898e-14, 13.2221027072096, -1.23603801475692e-14, -4.67379217909115, 1.18363558466232, 0, 0, 0, 0, 0, 0, -1.58444179603095e-14, 0, 12.1258851598178, 0, 1.08652403506101e-14, -3.11150358274815, 0, 0.833612074594099, -0.192474367616158, 0, 0, 0, 0, 0, 0, 0, 0, 1.49231173587483e-14, -1.98974312579798e-14, -1.74186427463722e-14, 0, 0, 0, 0, 0, 0, 0, 0, -2.0574377947513e-14, 0},
    {0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {3.14718316987773, 0, -0.861891607371334, 6.09449400220044, 1.55158222708543, 0, 0.0693888666488801, 0, -3.05941170815564, 1.80277563773202, 0, 0, 0, 0, 9.03122156537873, 2.80112084875395e-14, 0.403888506800968, -1.0607742830904e-14, -3.67461994579463, -5.49650682721189, 0, 0, 0, 0, 0, 0, 1.10205726024203e-14, 0, 1.65064789881475e-14, 21.7841260588564, 0, 0.974215733960566, 1.13989530288554e-14, 1.78974858569577, 1.80791908794933, 0, 0, 0, 0, 0, 0, -1.4994119558265e-14, 0, -2.17157199142971e-14, 0, 1.16195530358102e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 2.22044604925031e-14},
    {0, 1.26168012376113, 1.95399252334028e-14, 0, -1.17816950171735e-14, 1.42204860051341, 1.74453095862365e-14, 4.47852016375308, -2.08721928629529e-14, 1.0744670813999e-14, -1.34464719541228e-14, 3.78456620442729e-14, 0, 2.80676208590537e-14, 0, 8.27725128825316, 0, 5.37909280973871, 1.24344978758017e-14, -1.95594693216656e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 19.9654813226339, 0, -2.61992366301004, 1.06581410364015e-14, 0, 0, 0, 0, 1.3455473305103e-14, 0, 1.46585553191507e-14, 0, -1.27103118437955e-14, 0, -2.2308537695773e-14, 0, 0, 1.05662688412451e-14, 2.42300772936765e-14, 0, 0, 0, 1.19298522441569e-14, 2.66453525910038e-14, 1.50990331349021e-14, 0},
    {2.06031501455086, -3.07345773477637e-14, 2.25696201807219, 3.98978286964828, 0, -1.17529718472799e-14, 0.953939201416948, -4.35480723877723e-14, 8.01142041980431, 1.18019368870419, 0, 0, 0, 1.31328347545057e-14, 0, 1.03323004232013e-14, 5.5525489642145, -2.26382415357737e-14, 9.62241374388481, -3.59830837050572, 0, 0, 0, 0, 0, 0, 1.02408289475646e-14, 0, 0, 0, 0, 13.3932520322736, 0, -4.6866619250806, 1.18356086726904, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1.27725097955809e-14, -1.27832588361236e-14, 0, 0, -1.04546715347572e-14, 1.45339591718248e-14, 0, 0, 0, 0, -1.59580006645491e-14, 1.4210854715202e-14},
    {0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {3.93184363124296e-14, 1.09264703862703, 1.28785870856518e-14, 0, 0, 1.22575380004522e-14, -1.14864973879958e-14, 5.38682254607081, 0, 0, 0, -1.44121099608086e-14, 0, -1.75698029368367e-14, -1.80160878442005e-14, 2.57528022630009e-14, 2.69054202848798e-14, 13.4377817958113, 0, 0, 0, 0, 1.06823510383984e-14, 0, 0, 0, 0, 0, 1.89056142369167e-14, 0, -1.31345967659526e-14, -1.51823905359486e-14, 18.907670401189, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, -1.07391608525697e-14, 0, 1.46384409866762e-14, 0, 0, -1.63612658717074e-14, 0, 0, 0, 0, 0, 0, 0},
    {1.78428514239956, -1.14226462513753e-14, 0.814411017927918, 5.06770487035906, 0, 1.13885635970811e-14, 0, 1.47667974881489e-14, 4.01510046524763, 6.58672305740271, 0, 2.01242961116133e-14, 0, -1.5623829602503e-14, 0, 0, 0, -3.27887259670989e-14, 10.0159311874144, 2.83293314500738, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1.60502464055099e-14, 0, 1.06885013954928e-14, 14.0929454377398, -4.55552790213387, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, -2.19027884045748e-14, -1.24344978758018e-14},
    {0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {1.4568627181694, 0, 0, 5.07816454066245, -1.07582259106397e-14, 0, 0, -2.20867693944166e-14, 0, 10.0142755247554, 0, 0, 0, 0, 0, 1.02210862188786e-14, 3.51018877774324e-14, 0, 0, 13.8784813613724, 0, 0, 0, 0, 0, 0, 0, 0, 1.67232492370418e-14, 0, 0, 0, 0, 0, 12.5535587278944, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1.55927253999833e-14, 1.90609040202255e-14, 0, 0, 0, -1.79253699586741e-14, 0, 0, 1.08479262389015e-14, 0, 0, 3.10862446895044e-14},
    {0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0}};
    
    const static double dmats1[56][56] = \
    {{0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {3.16227766016836, 0, 0, 0, 0, 1.08252669405283e-14, 0, 1.02922993704865e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {5.47722557505162, 0, 0, 0, 0, 1.92068165630412e-14, 0, 1.80834223416197e-14, -1.14152204219919e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {2.95803989154978, 5.6124860801609, -1.08012344973465, -0.763762615825971, 0, 2.58464835607474e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, -1.40850097772758e-14, 0, 0, 1.16248598628335e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {2.2912878474779, 7.24568837309471, 4.18330013267037, -0.591607978309959, 0, 2.03703972918374e-14, 0, 0, 0, 1.08801856413265e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1.16401775975232e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {-2.64575131106454, 0, 9.66091783079294, 0.683130051063963, 0, -1.49625671831933e-14, 0, -3.2502596093762e-14, 0, 0, 0, 1.34670915888159e-14, 0, 0, 0, 2.90440444432224e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {1.87082869338697, 1.42804031130623e-14, 0, 4.34741302385683, 0, 0, 0, 2.04003480774873e-14, -1.14092481911365e-14, -1.06581410364015e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {3.24037034920393, 2.38080669966159e-14, 0, 7.52994023880667, 0, 1.13478631598507e-14, 0, 3.45590001304086e-14, -1.69864122767649e-14, -1.4210854715202e-14, 0, 0, 0, 0, 0, -1.32400798627181e-14, 0, 1.22473609554424e-14, 1.33269883629769e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {2.74954541697351, 5.79655069847575, -1.67332005306816, -1.18321595661991, 7.74596669241483, -1.19999999999998, 0.346410161513775, -0.979795897113269, 0.282842712474611, 0.200000000000014, -1.0696429695597e-14, -1.26705171178379e-14, -1.2948158065601e-14, 0, 0, -1.45874734860756e-14, 0, -1.16672151462161e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, -1.07102102118129e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {2.32379000772449, 2.44948974278316, 2.82842712474617, -0.999999999999999, 9.16515138991168, 7.09929573971955, -2.04939015319193, -0.414039335605429, -0.478091443733758, 0.169030850945723, 0, -1.28646947787458e-14, -1.8460920120059e-14, -1.81736908030475e-14, 0, 1.37951452580958e-14, -1.77890663211312e-14, 0, 0, 0, 0, 1.08892803896054e-14, 0, 0, 0, 0, 1.05640513109647e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, -1.22044403129704e-14, 0, 0, 0, 2.27334526137921e-14, -1.76454983335969e-14, -1.05574887489127e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {1.80000000000004, -5.69209978830308, 4.38178046004131, -0.774596669241497, 1.24189930203385e-14, 10.998181667894, 4.76235235991625, 0.962140470884722, -0.740656079818038, 0.130930734141603, 0, -1.20430917197879e-14, -1.1981685754404e-14, -2.88797787541631e-14, -1.3779450272627e-14, 0, 0, 0, -2.22505656218757e-14, -1.14908083048704e-14, 0, 0, 0, 0, 0, 0, 0, 0, 1.22673594366302e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, -1.12284484669662e-14, 0, 0, 0, 0, 0, 0, 0, 0, -1.1387566167118e-14, 0, 1.48982410846523e-14, 0, 0, 0, 0},
    {5.19615242270664, 0, -3.16227766016842, -2.23606797749978, 0, 0, 13.7477270848675, 1.78325020862166e-14, 0.534522483824845, 0.377964473009226, 0, -2.80136022320966e-14, -1.36437984192736e-14, -2.9552592280819e-14, 1.3605842011612e-14, -4.49783579557011e-14, 2.03714412277317e-14, -1.61597054173618e-14, -2.47182474765566e-14, 0, 0, 0, 0, 0, 0, 0, 0, 1.7014378114654e-14, 0, 0, 0, -1.14827159581245e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, -1.57668143745762e-14, 2.14083883902157e-14, -2.68877125810315e-14, 0, 0, 0, -1.36900091929644e-14, -2.0687516582275e-14, 0, 0, 0, 0, 0, 0},
    {2.01246117974979, 2.12132034355965, -0.408248290463867, 3.17542648054294, 0, 1.00616573615107e-14, 0, 7.17137165600639, -1.38013111868472, -1.56144011671766, 0, 0, 1.10848947143279e-14, 0, 0, -1.41970084791766e-14, -1.32242728950118e-14, 1.59365405521834e-14, 0, -1.4210854715202e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, -1.214689797411e-14, 0, 0, 1.24931114721564e-14, 0, 0, 0, 0, 0, 0, 0},
    {1.55884572681198, 2.73861278752583, 1.58113883008418, 2.45967477524978, 0, 0, 0, 9.25820099772551, 5.34522483824847, -1.20948631362952, 0, 0, 1.84632408273169e-14, 0, 0, 0, -1.50085060064379e-14, 1.49723986219342e-14, 0, -1.33226762955019e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, -1.46277209591922e-14, 0, 0, 2.16306109608827e-14, 0, 0, 0, 1.38795388762322e-14, 0, 0, 0},
    {-1.79999999999998, -2.72689985024876e-14, 3.65148371670112, -2.84018778721877, 0, -2.17150305878754e-14, 0, -5.90005603264198e-14, 12.3442679969674, 1.39659449751037, 0, 0, 0, 0, 0, 2.68747109317333e-14, 0, 0, -1.90845800418516e-14, 0, 0, 0, 0, 0, 0, -1.23984917003451e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1.36024520914242e-14, 0, 0, 0, 0, 0, 1.0365125502094e-14, 0, 0, 1.00866023900427e-14, -1.51206012498905e-14, 0},
    {1.27279220613579, 0, 0, 3.83405790253616, 0, 0, 0, 1.00217788352562e-14, 0, 5.18459255872629, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {2.20454076850487, 0, 0, 6.6407830863536, 0, 0, 0, 1.6954568423569e-14, -1.04207941733572e-14, 8.97997772825745, 0, 0, 0, 0, 0, 0, -1.07399115598352e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, -1.06281916397066e-14, 0, 1.0771947239792e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {2.56904651573302, 5.80288457473997, -2.01017818278147, -1.42141062443802, 8.14215222512715, -2.10229466468021, 0.728256234341424, -1.71651640581399, 0.594618725379061, 0.420458932936053, 9.8107084351743, -1.18450885369836, 0.30583887092058, -0.105945692672794, -1.02581475831037, 0.26486423168198, -0.0917516612761728, 0.216260739578395, -0.0749149177264333, -0.0529728463363882, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {2.26568606239551, 3.07059789431495, 1.77281052085584, -1.25356634105602, 2.39356776939086, 6.3037687140313, -2.78313372537745, -0.908295106229251, -0.524404424085067, 0.370809924354793, 11.1242977306435, 9.40174755792013, -2.42752078113808, 0.840917865872078, -0.301561193523236, -0.794200206656967, 0.350641890608821, 0.114434427054281, 0.0660687472643356, -0.0467176592151211, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1.01015959200066e-14, 0, 0, 0, 0, 0, 0, 0, 1.84814770714563e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {1.91485421551268, 1.29756443747034, 3.74574588632198, -1.05945692672795, -8.09173593712686, 5.01426536422406, 1.80936716113936, -0.383824736789375, -1.10800657553489, 0.313391585263996, 0, 12.7134831207354, 8.20651806648289, -2.84282124887607, 1.01946290306867, -0.631738055305808, -0.227958835180056, 0.0483573714561744, 0.139595707137592, -0.039483628456617, 0, 0, 0, 0, 0, 0, 0, 1.2257456641157e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, -1.29080203530852e-14, 1.16595669138782e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {1.48323969741914, 7.03562363973513, 4.062019202318, -0.820651806648271, 0, -6.79705818718658, 5.88642506110458, -2.08116553882674, -1.20156148406981, 0.242752078113807, 0, 0, 14.8323969741913, 5.13809303146607, 1.63714487212955e-14, 0.856348838577624, -0.741619848709526, 0.262202212042521, 0.151382517704875, -0.030583887092048, 0, 0, 0, 0, 0, 0, -1.18952346076914e-14, 1.30853433842222e-14, -1.16925429925565e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, -1.88029658615293e-14, 1.91748164115319e-14, -1.45977163230575e-14, 1.09655150319724e-14, 0, 0, -1.89024735595586e-14, 0, 0, 0, 0, 0, 0, 0},
    {-3.42539535431071, 0, 9.38083151964686, 1.89521416591735, 0, 0, -3.39852909359329, 0, -2.77488738510232, -0.560611910581379, 0, 0, 0, 17.7988763690296, 0, -1.94078776694002e-14, 0.42817441928881, 0, 0.349602949390047, 0.0706304617818554, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, -1.20061774357534e-14, 0, 0, 0, 0, 0, -1.00079856173773e-14, 0, -2.26674794489762e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {2.02649122047609, 2.74642624930236, -0.792824967172097, 2.24244764232558, 2.14087209644418, -0.331662479035527, 0.0957427107756351, 6.49923072370878, -1.87616630392938, -2.15580611373099, 0, 0, 0, 0, 9.44035859331468, -1.46249406456536, 0.422185670932516, -2.04706526287666, 0.590936840285292, 0.52231930877335, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1.66277980866635e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1.68132514445692e-14, 0, 0, -1.30320642762071e-14, 1.5317530203921e-14, 0},
    {1.71269767715538, 1.16057691494799, 1.34011878852097, 1.89521416591739, 2.5331140255951, 1.96214168703485, -0.566421515598877, 2.74642624930236, 3.17129986868837, -1.82198870938948, 0, 0, 1.26698604015865e-14, 0, 11.1699829238239, 8.6522315682783, -2.49768411251824, -0.86504295831355, -0.998865569685873, 0.441440386136653, 0, 0, 0, 0, 0, 0, 1.98384517908788e-14, 0, 0, 1.09458095686894e-14, 0, 1.6523812455151e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, -1.03226253428986e-14, 3.45055971392177e-14, -1.0963026379453e-14, 0, 0, 1.98669435916771e-14, 0, 2.24143595655932e-14, 1.21246267628695e-14, 0, 1.40825657142122e-14, 0, 0},
    {1.32664991614221, -2.69693703815181, 2.07610309995257, 1.46802658041887, 0, 3.03973683071411, 1.31624465810881, -6.38211787508104, 4.91295663090393, -1.4113063856887, 0, 0, 0, 0, -1.12046668488404e-14, 13.4039795085888, 5.80409338312195, 2.01017818278149, -1.54743588659735, 0.341938252770126, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1.99188479486906e-14, 0, 0, 0, 0, 0, 0, 1.9439663850877e-14, 0, 2.04972151503089e-14, -2.09775353099935e-14, 0},
    {3.82970843102538, -1.35375539472083e-14, -1.49829835452878, 4.23782770691181, 0, 0, 3.79967103839267, 1.23996592074953e-14, -3.54562104171168, -4.07409060843206, 0, 0, -1.7600753095073e-14, -1.0822450122319e-14, 1.7784123181026e-14, -1.50271421719475e-14, 16.7549743857359, -2.83790269242822e-14, 1.11676565710081, 0.987090711415297, 0, 0, 0, 0, 0, 1.14492917376685e-14, -1.371929590441e-14, 2.1952750214123e-14, 0, 0, 1.18235696338831e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, -1.24727047960437e-14, 2.54018340570979e-14, -1.39065024647839e-14, 0, 0, 0, -2.41394818807004e-14, 0, 0, 0, 0, 0, 0, 0},
    {1.48323969741916, 1.00508909139075, -0.193429485824665, 3.55615782880926, 0, -1.09316735690911e-14, 0, 4.45964044034304, -0.85825820290701, 2.67027285925186, 0, 0, 0, 0, 0, 0, 0, 8.42792824422444, -1.62195554683794, -2.29379153190449, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {1.14891252930764, 1.29756443747036, 0.749149177264399, 2.75458800949268, 0, -1.56850849701631e-14, 0, 5.75737105184055, 3.32401972660469, 2.06838446274243, 0, 0, 0, 1.55284895654593e-14, 0, 0, -1.28699742148281e-14, 10.8804085776374, 6.28180682119208, -1.77676328054755, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, -1.17679879087453e-14, 0, 0, 0, 0, 0, 0, 0, 0, 1.29340213107974e-14, 0, 0, 0, 0},
    {-1.32664991614215, -2.47030321919216e-14, 1.73008591662716, -3.18072425757422, 0, 0, 1.13119318909818e-14, -3.04199753195453e-14, 7.6764947357874, -2.38836465270395, 0, 0, 0, 2.05269241282193e-14, 0, -1.45662197301565e-14, -1.85676576002613e-14, -2.8126806197496e-14, 14.5072114368499, 2.05162951662072, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, -1.09052206632751e-14, 0, 0, 0, 0, 0, 0, 0, 0, -1.073521706443e-14, 0, 0, 0, 0, 0, 0, 0, 1.02157480437484e-14, 0, 0, 0, 0, 1.06581410364015e-14},
    {0.938083151964688, -1.67075414421403e-14, 0, 3.11415464992886, 0, 0, 0, 0, 0, 5.52707620976693, 0, 0, 0, 0, 0, 0, 0, -1.44579473224948e-14, 0, 5.80288457473998, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {1.6248076809272, -2.71174079765265e-14, 0, 5.39387407630366, 0, 0, 0, -1.76761006483537e-14, 0, 9.57317681262153, 0, 0, 0, 0, 0, 1.62541936715167e-14, 1.29794840192237e-14, -2.09070339378817e-14, 0, 10.0508909139074, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1.07896695788578e-14, 1.01165706770544e-14, 0, 1.24344978758018e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1.24344978758018e-14},
    {0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {2.41593350361252, 5.72988941019469, -2.20543546226845, -1.55947837083931, 8.33749895885385, -2.7677995385711, 1.06532653910463, -2.25989885993664, 0.869835476750505, 0.615066564126907, 10.333118277332, -2.18326971917503, 0.724780051083774, -0.278967971730974, -1.8907670401189, 0.627677936394735, -0.241593350361259, 0.512496888990078, -0.197260144544833, -0.139483985865487, 11.8490505948789, -1.16109782023229, 0.245326690731328, -0.0814411017928031, 0.0313466946954498, -1.03851746182649, 0.219426862868121, -0.0728431359084709, 0.0280373360835783, 0.190029237516519, -0.063084006188059, 0.024281045302823, -0.0515078753637739, 0.019825390471097, 0.0140186680417963, 0, 0, 1.00108896213852e-14, 0, 0, 0, 0, 0, 0, 0, 0, 1.21150877964365e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {2.18529407725402, 3.45525332069937, 0.997445717412081, -1.41060126129511, 3.23209352764251, 5.28531208250976, -2.89087233497829, -1.36277028773851, -0.393397896234712, 0.556348640264183, 2.3366642891096, 9.02784807228871, -3.93352933905729, 1.64018416088948, -0.73296991613515, -1.19859611757306, 0.655588223176202, 0.309047252182647, 0.0892142571199726, -0.12616801237611, 13.0996183150502, 11.55277744383, -2.44096975264614, 0.810328731479527, -0.311895674167859, -0.234843597212102, -0.907332870306517, 0.395334573318419, -0.164844710568308, 0.0736662483163133, 0.120463442338377, -0.0658890955530689, -0.031060417514551, -0.00896637020659282, 0.012680362351413, 0, 2.00524187044473e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1.35703519068716e-14, 0, 0, 0, 0, 0, 0, 0, -1.01417984314538e-14, 0},
    {1.92724822331887, 1.82834820066011, 2.99085908983326, -1.24403337882031, 0.950146187582608, 5.74064456311309, -0.169967317119755, -0.721110255092798, -1.1796107330308, 0.490653381462651, -10.3037210107158, 4.97613351528121, 5.78174466995658, -3.22682211353229, -0.215472901842825, -1.30185581821821, 0.0385449644663842, 0.163532434528608, 0.267510569439612, -0.111269728052846, 0, 14.555131461167, 10.7636648722209, -3.57321383950879, 1.37533064807499, 1.03556292536504, -0.500120235663637, -0.581087203147965, 0.324307824721464, 0.0216558414515625, 0.13084143273245, -0.00387391468765269, -0.0164356280722631, -0.0268858238268406, 0.0111830284386733, 0, 0, 0, 0, 0, 0, 0, -1.24894083304515e-14, 0, 0, 0, 0, 0, -1.79685973736065e-14, 0, 0, 0, 0, 0, 0, 0},
    {1.62882203585591, 0.77261813045657, 3.86595114186567, -1.05140010313427, 9.1544290607646, 3.35889012281481, 3.87851223317102, -0.304724700110015, -1.52475169285617, 0.414677792940093, -1.50838999194959e-14, -9.46260092820841, 7.3296991613516, 0.564240504518021, -2.07602937339133, -0.761724681099385, -0.879563899362177, 0.0691050664139764, 0.345781182036188, -0.0940400840863524, 0, 0, 16.3745228938128, 9.05975063854702, -3.48709964663735, 0, 0.951027176355628, -0.736662483163206, -0.0567083043941531, 0.208648802584714, 0.076556210932118, 0.0883994979795754, -0.00694532049720325, -0.0347523163751444, 0.00945138406569365, 0, -1.34204874150947e-14, 0, 0, 0, 0, 0, 0, 0, -2.42374492692268e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {1.26168012376109, -6.78263087840205, 3.68560354207928, -0.814411017927976, 0, 10.0216907612581, 6.00856531485322, 2.67510569439611, -1.4536216402544, 0.321208037219824, 0, 0, -7.57008074256671, 6.99294104721291, 0, -2.27270584034422, -1.36261453366206, -0.606656948415834, 0.329650402328115, -0.0728431359084831, 0, 0, 0, 18.7137404500718, 5.40219154319688, 0, 0, 0.760821741084502, -0.702817019232737, 0, 0.228415531250642, 0.136947913395211, 0.0609713174047674, -0.033131112015512, 0.00732101061700773, 0, 0, 0, 0, 0, -1.30324950839724e-14, 0, 0, 0, -1.36428396125789e-14, -2.70392224640622e-14, 0, 0, 0, -1.78921270332478e-14, 0, 0, 0, 0, 0, 0},
    {5.09901951359278, 1.47405589080242e-14, -4.65474668125635, -3.29140294302192, 0, -1.75121952403549e-14, 13.490737563232, -1.88659898491929e-14, 1.83585684909538, 1.29814682728313, 1.74929961328223e-14, 0, 3.18637490243895e-14, -3.53270434653118, 0, 2.70955022774169e-14, -3.05941170815572, 0, -0.416333199893208, -0.294392028877594, 0, 0, -1.00417365518154e-14, -2.77992935601067e-14, 21.8326971917504, 0, 0, 0, 0.355050145839428, 0, 0, 0.307482445914326, 0, 0.0418430609640593, 0.0295875121532926, 0, 0, 1.53618763757974e-14, -1.4536552951721e-14, 0, -2.34517773777192e-14, 0, 0, 0, 1.75720087957737e-14, -2.91662766570223e-14, 0, 1.76059530106093e-14, 0, 0, 0, 0, 1.07470318048245e-14, 0, 0, 0},
    {1.99489143482416, 3.15420030940281, -1.09264703862702, 1.54523626091316, 2.95048422176041, -0.761811750275054, 0.263899331455877, 5.59815020469135, -1.93925611658553, -2.28543525082516, 2.13307290077015, -0.257539376818855, 0.0664963811607952, -0.0230350221380054, 9.14446135479766, -2.36108976914457, 0.817905488277913, -3.33842298506509, 1.1564636454577, 1.02505848514083, 0, 0, 0, 0, 0, 11.5766222110848, -1.39771878815471, 0.3608894392803, -0.125015768949695, -2.15192884958005, 0.555625639776457, -0.192474367616166, 0.595437196138648, -0.206265495285699, -0.166687691932932, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1.62916528370584e-14, 1.08185730701625e-14, 0, 0, 0, -1.29677564833166e-14, 0, 0, 0, 0, 0, 0},
    {1.75932887637252, 1.66904592079257, 0.963624111659422, 1.36277028773852, 0.867360833110887, 2.28430636201791, -1.00852844760554, 2.96226264871973, 1.71026313764871, -2.01556443707464, 2.41867732448957, 2.04415543160773, -0.527798662911753, 0.182834820065996, 2.68821895760395, 7.07977053255619, -3.12574097035512, -1.76652739792481, -1.01990506872276, 0.904016610326408, 0, 0, 0, 0, 0, 13.1266557397187, 11.0940489487798, -2.8644711213756, 0.992281903807275, -0.632607619454911, -1.66605356688629, 0.735567892896454, 0.315075748468089, 0.18190906819317, -0.147004726489971, 0, 0, 0, 0, 0, 0, 0, 0, 0, 3.50533057667082e-14, 0, 0, 0, 0, 0, 1.4911169345421e-14, 0, 0, 0, -1.04402756430621e-14, 0},
    {1.48690428533297, 0.70530063064754, 2.03602754481989, 1.1517511068998, -2.93221479395414, 1.81702705031798, 0.655663160391202, 1.2517844405944, 3.61359041872289, -1.70346285967313, 0, 2.76420265655949, 1.78428514239953, -0.618094504365263, -9.08783875864323, 5.63152770643898, 2.03210252328839, -0.746494073225332, -2.15494277062554, 0.764033484562606, 0, 0, 0, 0, 0, 0, 15.0018922739643, 9.68367982311024, -3.35452509157131, 2.13860408462396, -1.32524448062077, -0.478206411017787, 0.133143804689794, 0.384353057392899, -0.124241670058204, 0, 0, 0, 0, 0, 0, 0, -1.2184958691979e-14, 4.08565188434445e-14, 2.21302457056835e-14, 0, 0, 0, -1.0903058801995e-14, 0, 0, 0, 0, 0, 0, 0},
    {1.15175110689978, 3.82426463519456, 2.20794021658196, 0.892142571199804, 0, -2.46306042692148, 2.13307290077015, 6.78739640804928, 3.91870514328392, -1.31949665727937, 0, 0, 3.22490309931938, 1.11713920350151, 2.5966242421383e-14, -7.6337845572604, 6.61105135360486, -4.04762275910199, -2.33689608954557, 0.591817792331191, 0, 0, 0, 0, 0, 0, -2.33944627292341e-14, 17.5022076529583, 6.06294257990891, 0, 1.79642743108414, -1.55575179137408, 0.721929233499939, 0.416806037297049, -0.0962371838080711, 0, 0, 0, 0, 0, 0, 0, -1.06500367370775e-14, 2.55596393426003e-14, -1.78278849535737e-14, 1.24660473149391e-14, 0, 0, -3.33471800635493e-14, 0, 0, 0, 0, -2.74739380889122e-14, 0, 0},
    {-2.65985524643224, 3.04548590059106e-14, 5.09901951359276, -2.06031501455087, 0, 1.27314974860562e-14, -1.23153021346075, 4.54600263645348e-14, 9.04986187739903, 3.04724700110024, 0, 0, 1.37797484823791e-14, 3.8698837191833, 0, -2.28548550248364e-14, -3.81689227863023, 1.32945293178207e-14, -5.39683034546927, -1.36674464685441, 0, 0, 0, 1.36272283216478e-14, 1.00072372011083e-14, 0, 0, -1.24367890909937e-14, 21.00264918355, 0, 0, 0.898213715542077, 0, 0.962572311333257, 0.22225025591058, 0, 0, 0, 0, 0, 0, 0, 1.43237887055327e-14, -2.25958210828995e-14, -2.54351954443607e-14, -4.42292868735092e-14, 0, 0, 1.18670146344888e-14, 0, 0, 0, 0, 0, -2.23325753829243e-14, 0},
    {1.57359158493888, 1.49284005458436, -0.43094580368567, 3.04724700110021, 0.775791113542718, -0.120185042515478, 0.0346944333244453, 5.29905651979672, -1.52970585407783, 0.901387818866015, 0, 0, 1.54345206564601e-14, 0, 4.51561078268936, -0.69955541437241, 0.201944253400477, 6.36462844462229, -1.83730997289732, -2.74825341360596, 0, 0, 0, 0, 0, 0, 0, 0, 0, 10.8920630294282, -1.68739114875268, 0.487107866980291, -3.0999354831996, 0.894874292847886, 0.903959543974661, 0, 0, 0, 0, 0, 0, 0, 0, -1.35315801444742e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {1.32992762321608, 0.630840061880569, 0.728431359084703, 2.57539376818858, 0.917928424547677, 0.711024300256696, -0.205255035576777, 2.23926008187653, 2.58567482211402, 0.761811750275079, 0, 1.50972141321262e-14, 1.92432056571699e-14, 2.94270416460811e-14, 5.34294273196304, 4.13862564412658, -1.1947183148558, 2.68954640486935, 3.10562068169861, -2.32269806558124, 0, 0, 0, 0, 0, 0, 0, 0, 0, 12.8876627769294, 9.98274066131697, -2.88176900403077, -1.30996183150502, -1.51261363209512, 0.763985254692642, 0, 0, 0, 0, 0, 1.07852658462479e-14, 0, -1.13464763436474e-14, -1.56492117088446e-14, -1.2428153133211e-14, 0, 0, 0, 1.57186842569255e-14, 1.23293837842049e-14, 0, 0, 0, 1.39814305552841e-14, 0, 1.99840144432528e-14},
    {1.03015750727542, -1.46593983227033, 1.12848100903611, 1.99489143482414, 0, 1.10151410945721, 0.476969600708479, -5.20357020296097, 4.00571020990215, 0.590096844352093, 0, 1.61268443608728e-14, 0, 2.17744761210991e-14, 0, 6.41153127835569, 2.77627448210725, -6.24994106094659, 4.81120687194241, -1.79915418525286, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 15.4651953323153, 6.69662601613678, 3.04407621455179, -2.3433309625403, 0.59178043363452, 0, 0, 0, 1.05482318770498e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {2.9738085706659, -2.31628665433557e-14, -0.814411017927956, 5.75875553449895, 0, 0, 1.37689263682152, -1.87511781470446e-14, -2.89087233497829, 1.70346285967313, 0, 1.53767244373726e-14, -2.60164507598573e-14, -3.0606105648312e-14, 0, 3.91555026057662e-14, 8.01441409794458, 0, -3.47218947830366, -5.19371076584692, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 19.3314941653941, 0, 1.69115345252876, 1.70832296330022, 0, 0, 0, 0, 0, -1.47710207017114e-14, 0, 0, 0, 2.39374069194048e-14, 0, -1.16737795196999e-14, 0, -2.35947835773664e-14, 0, 0, 0, -2.14656473740582e-14, 0, 0, 0},
    {1.15175110689981, 0.546323519313508, -0.105140010313419, 3.27118942773249, 0, 0, 0, 2.69341127303541, -0.518347241175128, 4.25171145123352, 0, 0, 0, 0, 0, 1.89132326876168e-14, 1.07703482219709e-14, 6.71889089790563, -1.29305115618717, 1.82865048192212, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 9.45383520059452, -1.81939143264587, -2.94058061635795, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0.892142571199805, 0.705300630647555, 0.40720550896397, 2.53385243517954, 0, 1.39813392926345e-14, 0, 3.47717900165116, 2.00755023262382, 3.29336152870135, 0, 0, 0, -1.4230588162391e-14, 0, 2.12522836138748e-14, 1.52618648424287e-14, 8.67405085085769, 5.00796559370718, 1.4164665725037, 0, 0, 0, 0, 0, 0, 0, 0, 1.13147703328239e-14, 0, 0, 0, 12.2048487632307, 7.0464727188699, -2.27776395106694, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1.14999053942145e-14, 0, 0},
    {-1.03015750727539, 0, 0.940400840863377, -2.92584077107543, 0, 0, 0, 1.11293406173924e-14, 4.63623866886823, -3.80284633026899, 0, 0, 0, -2.94156668790987e-14, 0, 0, 0, -1.33922800433549e-14, 11.5654011344769, -1.63559471386628, 0, 0, 0, 0, 0, 0, 0, 0, 1.04604926852949e-14, 0, 0, -1.05559372926886e-14, 0, 16.2731316843076, 2.63013526059784, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, -1.36105120406189e-14, 0, 0, -1.64162100411904e-14, 0, 0, -1.43624641143248e-14, 0, 0, -1.06581410364015e-14},
    {0.728431359084702, 0, 0, 2.53908227033122, 0, 0, 0, -1.12654293021334e-14, 0, 5.00713776237769, 0, 0, 0, 0, 0, 0, 1.75509438887162e-14, 0, 0, 6.9392406806862, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 6.27677936394721, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1.55431223447522e-14},
    {1.26168012376115, 0, 0, 4.39781949681102, 0, 0, 0, -1.94061175819238e-14, 0, 8.6726170049349, 0, 0, 0, 0, 0, 1.01179304274323e-14, 2.87309305235289e-14, 0, -1.04523229052332e-14, 12.0191174248973, 0, 0, 0, 0, 0, 0, 0, 0, 1.49130416320287e-14, 0, 0, 0, 0, 0, 10.8717007662564, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1.39042309332655e-14, 1.68148173406132e-14, 0, 0, 0, -1.39259589347659e-14, 0, 0, 0, 0, 0, 2.57571741713036e-14},
    {0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0}};
    
    const static double dmats2[56][56] = \
    {{0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {3.16227766016836, 0, 0, 0, 0, 1.11583338479159e-14, 0, 1.02922993704865e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {1.82574185835054, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {5.16397779494319, 0, 0, 0, 0, 1.73479189776187e-14, 0, 1.65773933526039e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {2.95803989154978, 5.6124860801609, -1.08012344973464, -0.763762615825969, 0, 2.58464835607474e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, -1.45290989871259e-14, 0, 0, 1.07366814431334e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {2.29128784747791, 1.44913767461894, 4.18330013267037, -0.591607978309966, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1.02908502069211e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {1.32287565553229, 0, 3.86436713231718, -0.341565025531991, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {1.87082869338696, 7.09929573971954, 0, 4.34741302385683, 0, 2.31412979990705e-14, 0, 1.79578574233119e-14, -1.09651589812864e-14, 0, 0, 0, 1.0275456662804e-14, 1.11253951863901e-14, 0, 0, -1.23516279655211e-14, 0, 1.67772945370309e-14, -1.06581410364015e-14, 0, 0, 0, 0, 0, 0, 0, 0, -1.04810894589995e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, -1.45492836924348e-14, 0, 0, 0, 0, 0, 0, 0, -1.07986153011749e-14, 0, 0},
    {1.08012344973466, 0, 7.09929573971953, 2.50998007960221, 0, 0, 0, 0, 0, 0, 0, 1.10981825134126e-14, 0, 0, 0, 1.49126177831961e-14, 0, 0, -1.09527631538099e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1.11294528477952e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {-3.81881307912981, 2.63805095952449e-14, 0, 8.87411967464941, 0, -1.08587953265362e-14, 0, 1.60284200435428e-14, 0, -2.30926389122033e-14, 0, 0, 2.0594465710441e-14, 1.14859138885232e-14, 1.07233040380981e-14, 0, -1.40594294599945e-14, 0, 0, -2.1316282072803e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, -1.15463194561016e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {2.74954541697351, 5.79655069847576, -1.67332005306816, -1.18321595661991, 7.74596669241483, -1.19999999999999, 0.346410161513776, -0.979795897113272, 0.282842712474611, 0.200000000000011, 0, -1.42248293523131e-14, -1.16991571628977e-14, 0, 0, -1.34772504614504e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, -1.05991879093504e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {2.32379000772445, 2.44948974278317, 2.82842712474618, -1.00000000000001, 1.30930734141596, 7.09929573971955, -2.04939015319193, -0.414039335605422, -0.478091443733756, 0.169030850945715, 0, 0, -1.38137027369175e-14, 0, -1.23135650746216e-14, 1.4971941277217e-14, -1.68198040008164e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, -1.41869149998573e-14, 1.99142275743244e-14, -1.52384110702395e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {1.80000000000002, 0.632455532033669, 4.38178046004131, -0.774596669241481, 0, 3.14233761939829, 4.76235235991625, -0.106904496764978, -0.740656079818043, 0.130930734141599, 0, 0, 0, -1.70268155420246e-14, 0, 0, 0, 0, -1.65329170450561e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {1.03923048454135, 0, 3.16227766016836, -0.447213595499958, 0, 0, 5.89188303637179, 0, -0.53452248382485, 0.0755928946018462, 0, 0, 0, -1.1578427571356e-14, 0, 0, 0, 0, -1.31656139489963e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {2.01246117974984, 2.12132034355964, -0.40824829046387, 3.17542648054295, 9.07114735222145, 0, 0, 7.17137165600638, -1.38013111868473, -1.56144011671764, -1.28131039656276e-14, -1.05722626181333e-14, 0, 0, 0, -1.43560570277939e-14, -2.27050760178409e-14, 0, 0, 0, 0, 0, 0, 0, 0, 1.20645452546378e-14, 0, 0, -1.39975315007857e-14, 0, 1.33944298105448e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, -1.79370968861081e-14, 0, 0, 0, 0, 0, 0, 0, 0, 1.10225378253926e-14, 0},
    {1.558845726812, 0.547722557505167, 1.58113883008418, 2.45967477524977, 0, 9.07114735222146, -1.85760141295822e-14, 1.85164019954509, 5.34522483824848, -1.20948631362952, 0, -1.79269712993067e-14, 0, 0, -1.21834179081657e-14, 0, -1.08231757343816e-14, 1.58605770416344e-14, 0, -1.77635683940025e-14, 0, 0, 0, 0, 0, 0, 1.20070279998216e-14, 0, 0, 0, 0, 0, 0, 0, -1.11022302462516e-14, 0, 0, 0, 0, 0, 0, 0, 0, -1.5994754107584e-14, 1.5836245903059e-14, -1.59336158038543e-14, 0, 0, 1.24090681928578e-14, 0, 0, 0, 0, 0, 0, 0},
    {0.900000000000037, 0, 1.46059348668045, 1.42009389360939, 0, -1.21399527118824e-14, 9.07114735222146, 0, 4.93770719878694, -0.698297248755176, 0, -1.69782446707204e-14, 0, 0, 1.38222655524042e-14, -1.41725217571765e-14, 0, 0, -1.92181403448241e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1.52483803956785e-14, -1.04387107607342e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {1.27279220613579, -6.2609903369994, 0, 3.83405790253616, 0, -2.66966858276427e-14, 0, 10.5830052442584, -1.29525898283173e-14, 5.18459255872629, 0, 1.15955623710069e-14, 2.12136539532159e-14, 0, 0, 0, -1.1187497199729e-14, 0, 0, -2.1316282072803e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1.1791882435372e-14, 1.34567031689284e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1.11941676856894e-14, 0, 0, 1.28246579327207e-14, 0, 0, 0, 0},
    {0.734846922834948, 0, -6.26099033699939, 2.21359436211787, 0, 0, 0, 0, 10.5830052442584, 2.99332590941916, 0, -1.26338784464357e-14, 1.51937191134705e-14, 2.01241989119956e-14, 0, 0, -1.07773281838078e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {5.71576766497725, -3.5255606739263e-14, -2.0846313271379e-14, -4.69574275274955, 0, 0, 1.14621342066822e-14, -1.32075361813795e-14, 0, 12.6996062931101, 0, 1.1579866837032e-14, -1.17571210377054e-14, 0, 0, 1.11781775592062e-14, 0, -2.03916735577721e-14, 0, 1.24344978758018e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1.77352530249445e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1.34841270690221e-14, 0, 0, 0, 0, 0, 1.33226762955019e-14},
    {2.56904651573302, 5.80288457473997, -2.01017818278147, -1.42141062443802, 8.14215222512715, -2.10229466468021, 0.728256234341424, -1.71651640581399, 0.59461872537906, 0.420458932936052, 9.8107084351743, -1.18450885369835, 0.305838870920579, -0.105945692672794, -1.02581475831037, 0.264864231681979, -0.0917516612761747, 0.216260739578394, -0.0749149177264344, -0.0529728463363901, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {2.26568606239552, 3.07059789431494, 1.77281052085583, -1.25356634105602, 2.39356776939085, 6.30376871403133, -2.78313372537745, -0.908295106229254, -0.524404424085065, 0.370809924354795, 1.23603308118261, 9.40174755792012, -2.42752078113806, 0.84091786587207, -0.301561193523234, -0.794200206656967, 0.350641890608815, 0.114434427054271, 0.0660687472643394, -0.0467176592151202, 0, 0, 0, 0, 1.20766335966975e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, -1.26955258141599e-14, 0, 0, 0, 0, 0, 1.65501530620379e-14, -1.82242579771102e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {1.91485421551271, 1.29756443747034, 3.74574588632197, -1.05945692672795, 0.404586796856347, 5.01426536422407, 1.80936716113936, -0.383824736789378, -1.1080065755349, 0.313391585264007, 0, 2.82521847127454, 8.20651806648288, -2.84282124887606, -0.0509731451534317, -0.631738055305793, -0.227958835180061, 0.0483573714561631, 0.139595707137598, -0.0394836284566152, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {1.48323969741914, 0.335029697130245, 4.06201920231797, -0.820651806648286, 0, 1.69926454679664, 5.88642506110457, -0.0991031208965069, -1.20156148406979, 0.24275207811381, 0, 0, 4.94413232473043, 5.13809303146604, 0, -0.214087209644449, -0.741619848709543, 0.0124858196210661, 0.151382517704864, -0.0305838870920592, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, -1.236469996079e-14, 0, -1.11803154979493e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0.85634883857767, 0, 2.68023757704195, -0.473803541479343, 0, 0, 5.09779364038993, 1.03628894560728e-14, -0.792824967172095, 0.140152977645353, 0, 0, 0, 7.9106117195687, 0, -3.2024651826757e-14, -0.642261628933259, 0, 0.0998865569685812, -0.0176576154454647, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, -1.41874267782629e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {2.02649122047608, 2.74642624930236, -0.79282496717209, 2.24244764232558, 2.14087209644418, -0.331662479035553, 0.0957427107756432, 6.49923072370878, -1.87616630392938, -2.15580611373099, 11.0554159678513, 0, 0, 0, 9.44035859331468, -1.46249406456537, 0.42218567093253, -2.04706526287665, 0.590936840285286, 0.522319308773345, 0, 0, 0, 0, 0, 0, 0, 1.07417485753862e-14, 0, 0, 1.36630425513329e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, -1.23701834881004e-14, 0, 1.02417490692638e-14, 0, 0, 0, 0, 0, 1.80291351901461e-14, 0, 0, -1.01316066243738e-14, 0, 0},
    {1.71269767715536, 1.16057691494801, 1.34011878852098, 1.89521416591738, 0.361873432227874, 1.96214168703485, -0.56642151559888, 2.74642624930238, 3.17129986868838, -1.82198870938951, 0, 11.0554159678513, 1.45760542423516e-14, 1.47166563795188e-14, 1.59571184626056, 8.65223156827826, -2.49768411251825, -0.865042958313534, -0.998865569685866, 0.44144038613664, 0, 0, 0, 0, 1.29892244953406e-14, 0, 1.19356062430186e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 2.55080325568313e-14, -1.99643912911818e-14, 0, 0, 2.52250943240554e-14, 0, 0, 0, 0, 0, 0, 0},
    {1.32664991614219, 0.299659670905743, 2.07610309995258, 1.46802658041888, 0, 0.868496237346862, 1.31624465810882, 0.709124208342315, 4.91295663090392, -1.4113063856887, 0, 1.04153151561804e-14, 11.0554159678513, 0, 0, 3.82970843102535, 5.80409338312197, -0.223353131420174, -1.54743588659735, 0.341938252770125, 0, 0, 0, -1.22971310029744e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, -1.3965710429727e-14, 1.20613126934675e-14, 1.38979281389624e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, -1.63145986065679e-14, 0},
    {0.765941686205059, -1.0369529147852e-14, 1.49829835452879, 0.847565541382363, 0, 0, 1.62843044502544, 0, 3.54562104171167, -0.814818121686403, 0, 0, 0, 11.0554159678513, 0, -2.54103636481183e-14, 7.18070330817254, 0, -1.11676565710084, 0.197418142283058, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, -1.21467689198221e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {1.48323969741914, 1.00508909139075, -0.193429485824672, 3.55615782880927, -8.46157280212813, 0, 1.04737699623147e-14, 4.45964044034302, -0.858258202906999, 2.67027285925188, 0, 0, 1.43424333847387e-14, 0, 12.4373429638327, 2.02429672213922e-14, 1.33492152068286e-14, 8.42792824422444, -1.62195554683795, -2.29379153190448, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1.03917274103107e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, -1.08180915476963e-14, 0, 0, 0, 1.39785916269036e-14, 0, 0, 0, 0, 0, 1.03080706885695e-14, 0, 0, 0, 0},
    {1.14891252930762, 0.259512887494088, 0.749149177264413, 2.75458800949269, 0, -8.46157280212813, 1.97736713455776e-14, 1.15147421036812, 3.32401972660468, 2.06838446274242, 0, 0, 1.76595565092679e-14, 3.53657180389623e-14, 0, 12.4373429638327, -2.12998980019174e-14, 2.1760817155275, 6.28180682119209, -1.77676328054753, 0, 0, 0, 0, 0, 0, 0, 0, -2.02381918388612e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1.29298824184789e-14, 0, 0, 0, 0, -1.20899537342362e-14, 0, 0, 1.49778288260513e-14, 1.57856876457792e-14, 0, 0, 0, 0, 0, 0},
    {0.663324958071077, -2.03917269773753e-14, 0.692034366650878, 1.59036212878712, 0, 0, -8.46157280212812, -1.91967284198321e-14, 3.07059789431496, 1.194182326352, 0, 1.63277888554017e-14, 0, 2.18186738788674e-14, 0, 2.30908333865784e-14, 12.4373429638327, -2.11081152198351e-14, 5.80288457473997, -1.02581475831036, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1.31364686077023e-14, 0, -1.38856086370832e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1.88129856179495e-14, 0, 0, 0, 0, 0, 0},
    {0.938083151964734, 7.62808987244123, 0, 3.11415464992886, 0, 0, 0, -7.52139804633612, 0, 5.52707620976692, 0, 0, -1.777231263474e-14, 0, -1.62387587846106e-14, 1.69100848543788e-14, 2.98390950438128e-14, 14.2141062443803, 0, 5.80288457473998, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1.39576755950212e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1.84182164821129e-14, 0, 1.05827350607878e-14, 0, 0, 0, 1.04237510536707e-14, 0, 1.26392642221179e-14, 0, 0},
    {0.541602560309088, -1.13099822653614e-14, 7.62808987244125, 1.79795802543456, 0, 0, 0, 0, -7.5213980463361, 3.19105893754052, 0, 1.03213108228181e-14, 0, 0, 0, -1.02355531327919e-14, 0, -3.31702746937809e-14, 14.2141062443803, 3.35029697130246, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, -1.166062958575e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {-5.36159180343545, -1.34228950103456e-14, 1.18705474084944e-14, 8.89943818451483, 0, 0, 0, -2.89831441233684e-14, 0, -5.26497863243526, 0, 0, 0, 0, -1.29782943052458e-14, 0, 2.98133166946067e-14, 0, 0, 16.583123951777, 0, 0, 0, 0, 0, 0, 0, 0, 1.67376776564845e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1.61338757506847e-14, 1.94557000035003e-14, 0, 0, 0, -1.11267253048686e-14, 0, 0, 0, 0, 1.32728374914348e-14, 1.68753899743024e-14},
    {2.41593350361252, 5.72988941019469, -2.20543546226845, -1.55947837083931, 8.33749895885385, -2.7677995385711, 1.06532653910463, -2.25989885993664, 0.869835476750504, 0.615066564126906, 10.333118277332, -2.18326971917503, 0.724780051083775, -0.278967971730974, -1.8907670401189, 0.627677936394735, -0.241593350361257, 0.51249688899008, -0.197260144544834, -0.139483985865487, 11.8490505948789, -1.16109782023229, 0.245326690731328, -0.0814411017928033, 0.0313466946954489, -1.03851746182649, 0.219426862868122, -0.0728431359084709, 0.0280373360835781, 0.190029237516519, -0.063084006188059, 0.0242810453028233, -0.0515078753637743, 0.0198253904710968, 0.014018668041796, 0, 0, 1.04549788312352e-14, 0, 0, 0, 0, 0, 0, 0, 0, 1.07828201668863e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {2.18529407725403, 3.45525332069937, 0.997445717412063, -1.41060126129511, 3.23209352764252, 5.28531208250976, -2.89087233497829, -1.36277028773849, -0.393397896234727, 0.556348640264195, 2.33666428910958, 9.02784807228869, -3.93352933905731, 1.64018416088945, -0.732969916135171, -1.19859611757308, 0.655588223176205, 0.309047252182632, 0.0892142571199684, -0.126168012376111, 1.19087439227729, 11.55277744383, -2.44096975264613, 0.810328731479535, -0.311895674167857, -0.234843597212088, -0.907332870306508, 0.395334573318422, -0.164844710568306, 0.0736662483163242, 0.120463442338383, -0.0658890955530685, -0.0310604175145507, -0.00896637020658553, 0.0126803623514123, 0, 0, 0, 0, 0, 0, 0, -1.66112853067542e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {1.92724822331887, 1.82834820066012, 2.99085908983324, -1.24403337882031, 0.950146187582616, 5.74064456311309, -0.169967317119763, -0.721110255092806, -1.17961073303079, 0.490653381462656, 0.294392028877589, 4.9761335152812, 5.78174466995658, -3.2268221135323, -0.215472901842837, -1.30185581821819, 0.0385449644663795, 0.163532434528623, 0.267510569439614, -0.11126972805285, 0, 2.64638753839399, 10.7636648722209, -3.57321383950879, 1.37533064807499, -0.0295875121532857, -0.500120235663636, -0.581087203147967, 0.324307824721465, 0.0216558414515637, 0.130841432732446, -0.00387391468765088, -0.0164356280722617, -0.0268858238268399, 0.0111830284386723, 0, 0, 0, 0, 0, -1.09166603315028e-14, 0, 0, 0, 1.42437445527895e-14, 0, 0, 0, -1.02438819301777e-14, 0, 0, 0, 0, 0, 0, 0},
    {1.62882203585591, 0.772618130456562, 3.86595114186567, -1.05140010313427, 0.160604018609904, 3.35889012281481, 3.87851223317101, -0.304724700110016, -1.52475169285617, 0.414677792940092, 0, 1.13551211138501, 7.32969916135159, 0.564240504518027, -0.0364215679542207, -0.761724681099395, -0.879563899362167, 0.0691050664139829, 0.345781182036178, -0.0940400840863483, 0, 0, 4.46577897103986, 9.05975063854702, -3.48709964663734, 0, -0.114123261162682, -0.736662483163202, -0.0567083043941542, 0.0036605053085017, 0.0765562109321201, 0.0883994979795774, -0.00694532049720645, -0.0347523163751393, 0.00945138406569343, 0, 0, 0, 0, 0, 0, 0, 0, 1.06468345288818e-14, -1.93401297016719e-14, 1.26134404858539e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {1.26168012376111, 0.199489143482416, 3.68560354207931, -0.814411017927961, 0, 1.02786571910339, 6.00856531485322, -0.0786795792469436, -1.4536216402544, 0.321208037219821, 0, 0, 3.0280322970267, 6.99294104721294, 0, -0.23309803490712, -1.36261453366203, 0.0178428514239939, 0.32965040232813, -0.0728431359084719, 0, 0, 0, 6.80499652729883, 5.40219154319687, 0, 0, -0.304328696433803, -0.702817019232747, 0, 0.0234272339744286, 0.136947913395211, -0.00179327404131641, -0.0331311120155151, 0.00732101061701076, 0, 0, 0, 0, 0, 0, 0, 0, 0, -1.10019056033674e-14, -1.37033343408735e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0.728431359084681, 0, 2.32737334062814, -0.470200420431712, 0, 0, 4.49691252107734, 0, -0.917928424547679, 0.185449546754743, 0, 0, 1.80505771145776e-14, 7.06540869306227, 0, 0, -1.01980390271859, 0, 0.208166599946618, -0.042056004125377, 0, 0, 0, 0, 9.92395326897748, 0, 0, 0, -0.710100291678884, 0, 0, 0.102494148638114, 0, -0.0209215304820281, 0.00422678745047274, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, -2.50163969655578e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {1.99489143482414, 3.15420030940281, -1.09264703862701, 1.54523626091315, 2.9504842217604, -0.761811750275062, 0.263899331455876, 5.59815020469134, -1.93925611658551, -2.28543525082518, 2.13307290077017, -0.257539376818833, 0.0664963811608233, -0.0230350221379902, 9.14446135479768, -2.36108976914455, 0.817905488277909, -3.33842298506508, 1.1564636454577, 1.02505848514083, 13.0453753561106, -1.2103323911658e-14, 0, -1.46406519642548e-14, 0, 11.5766222110848, -1.39771878815472, 0.360889439280293, -0.125015768949695, -2.15192884958006, 0.555625639776451, -0.192474367616164, 0.595437196138645, -0.206265495285706, -0.166687691932928, 0, 0, 1.33850600318309e-14, 0, 0, 0, 2.24412658009199e-14, 0, 0, 0, 1.66496943714949e-14, 1.3113069716231e-14, 0, -1.15323982020285e-14, 0, 0, 0, -1.02677636674246e-14, 0, 0, 0},
    {1.7593288763725, 1.66904592079256, 0.96362411165943, 1.36277028773851, 0.86736083311088, 2.28430636201791, -1.00852844760552, 2.96226264871973, 1.71026313764871, -2.01556443707463, 0.268741924943271, 2.04415543160776, -0.527798662911752, 0.18283482006601, 2.68821895760395, 7.07977053255616, -3.12574097035513, -1.76652739792481, -1.01990506872277, 0.904016610326402, 0, 13.0453753561106, 0, 0, 0, 1.4585173044132, 11.0940489487798, -2.86447112137559, 0.992281903807266, -0.632607619454896, -1.66605356688627, 0.735567892896463, 0.315075748468093, 0.181909068193174, -0.147004726489972, 0, 0, -1.03074285928854e-14, 0, 0, 0, 0, -1.72927567727304e-14, 1.04819480446677e-14, 1.76046313417165e-14, 0, 0, 0, 0, 0, 1.43579679437173e-14, 0, 0, 0, 0, 0},
    {1.48690428533297, 0.705300630647553, 2.03602754481992, 1.1517511068998, 0.14661073969771, 1.81702705031798, 0.65566316039122, 1.2517844405944, 3.61359041872288, -1.70346285967312, 0, 0.614267257013221, 1.78428514239955, -0.618094504365253, 0.45439193793218, 5.631527706439, 2.03210252328837, -0.746494073225325, -2.15494277062552, 0.764033484562605, 0, 0, 13.0453753561106, 0, 0, 0, 3.33375383865873, 9.68367982311024, -3.3545250915713, -0.1069302042312, -1.32524448062077, -0.478206411017785, 0.133143804689794, 0.384353057392897, -0.124241670058195, 0, 0, 0, 0, 0, 0, 0, 0, 1.35190501684936e-14, 2.13313974056602e-14, 0, 0, 0, -1.39901730746776e-14, 0, 0, 0, 0, 0, 0, 0},
    {1.15175110689978, 0.18210783977117, 2.20794021658194, 0.892142571199777, 0, 0.615765106730376, 2.13307290077017, 0.323209352764267, 3.91870514328392, -1.31949665727937, -1.1268544360518e-14, 0, 1.07496769977313, 1.11713920350151, 2.42487345110409e-14, 1.90844613931506, 6.61105135360481, -0.192743940909627, -2.33689608954558, 0.591817792331174, 0, 0, 0, 13.0453753561106, 0, 0, -1.3756522835902e-14, 5.83406921765278, 6.06294257990891, 0, -0.449106857771022, -1.55575179137408, 0.0343775825476135, 0.416806037297048, -0.0962371838080746, 0, 0, 0, 0, 0, 0, 0, 0, 1.42803357596901e-14, 0, 0, 0, 0, -1.21198190345942e-14, 0, 0, 0, 0, 0, -1.3839902652557e-14, 0},
    {0.664963811608031, 1.90472468148085e-14, 1.45686271816934, 0.515078753637701, 0, 0, 1.8472953201911, 0, 2.58567482211402, -0.761811750275038, 0, 1.08037159720826e-14, 2.26487951659612e-14, 1.71994831963699, 0, 1.52639255690414e-14, 5.72533841794521, 0, -1.54195152727693, 0.3416861617136, 0, 0, -1.00168576142402e-14, 0, 13.0453753561106, 0, 0, 0, 9.33451074824442, 0, 0, -1.34732057331308, 0, 0.275020660380927, -0.0555625639776386, 0, 0, 0, 0, 0, -2.03433658742105e-14, 0, 0, 0, 0, -3.84929350411527e-14, 0, 1.299574464293e-14, 0, 0, 0, 0, 0, 0, 0, 0},
    {1.57359158493892, 1.49284005458436, -0.43094580368567, 3.04724700110022, 0.775791113542717, -0.120185042515459, 0.0346944333244422, 5.29905651979673, -1.52970585407784, 0.901387818866001, -10.576283741361, 0, 0, -1.14454236590933e-14, 4.51561078268938, -0.699555414372402, 0.201944253400485, 6.36462844462229, -1.83730997289732, -2.74825341360594, 0, 0, 0, -1.03903584730172e-14, 0, 14.3499128917217, 0, 0, 1.48731679939335e-14, 10.8920630294282, -1.68739114875269, 0.487107866980279, -3.0999354831996, 0.894874292847882, 0.903959543974665, 0, 0, 0, 0, 0, 0, 0, 0, -1.69818485661453e-14, 1.01890667576586e-14, 1.15185313992138e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1.11022302462516e-14},
    {1.3299276232161, 0.630840061880575, 0.728431359084684, 2.57539376818859, 0.131132632078234, 0.711024300256703, -0.205255035576781, 2.23926008187655, 2.585674822114, 0.761811750275056, 0, -10.576283741361, 1.57303341744021e-14, 1.70725378740663e-14, 0.763277533137587, 4.13862564412658, -1.19471831485577, 2.68954640486935, 3.10562068169859, -2.32269806558125, 0, 0, 0, 0, 0, 0, 14.3499128917217, 0, 0, 1.84109468241849, 9.98274066131698, -2.88176900403078, -1.30996183150502, -1.51261363209512, 0.763985254692618, 0, 0, 0, 0, 0, 1.04560213685676e-14, 0, 0, 1.2776556039649e-14, -1.21474404387607e-14, 0, 0, 0, 1.05952301335811e-14, 0, 0, 1.33062123785088e-14, 1.01731376449221e-14, 1.65118907931275e-14, 0, 0},
    {1.03015750727544, 0.162882203585583, 1.12848100903609, 1.99489143482414, 0, 0.314718316987774, 0.476969600708478, 0.578174466995633, 4.00571020990216, 0.590096844352089, 0, 0, -10.5762837413611, 0, 0, 1.83186607953021, 2.77627448210726, 0.694437895660724, 4.8112068719424, -1.79915418525285, 0, 0, 0, 0, 0, 0, 0, 14.3499128917217, 0, 0, 4.41862723780437, 6.69662601613678, -0.338230690505753, -2.3433309625403, 0.591780433634519, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, -1.48270629766312e-14, 0, 0, 0, 0, 0, -1.15317140110751e-14, 0},
    {0.59476171413317, 0, 0.814411017927954, 1.15175110689979, 0, 0, 0.590096844352077, 1.28927856428244e-14, 2.89087233497831, 0.340692571934635, 0, 0, 0, -10.576283741361, 0, 0, 3.4347488991191, -1.66893313716044e-14, 3.47218947830366, -1.03874215316937, 0, 0, 0, 0, 0, 0, 0, 0, 14.3499128917217, 0, 0, 8.2849260708832, 0, -1.69115345252879, 0.341664592660046, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, -1.19245108709972e-14, -1.0049703503871e-14, 0, -1.01285720807823e-14, 1.46934267294508e-14, 0, 0, 0, 0, -1.33537813474028e-14, 1.11022302462516e-14},
    {1.15175110689978, 0.546323519313492, -0.105140010313412, 3.27118942773249, 9.65295620481433, 0, 0, 2.69341127303539, -0.518347241175101, 4.25171145123355, 0, 0, -1.03926949302858e-14, 0, -9.915266007526, 2.32678428089351e-14, -1.46605852864906e-14, 6.71889089790561, -1.29305115618716, 1.82865048192215, 0, 0, 1.09752665623958e-14, 0, 0, 0, 0, 0, 0, 15.9443476574685, -1.29058000479168e-14, 0, 9.45383520059453, -1.81939143264587, -2.94058061635793, 0, 0, 0, 0, 0, 0, 0, 0, -1.22633667590031e-14, 0, 0, 0, 1.71762874424691e-14, 0, 0, 0, 0, 0, 0, 0, 2.15938378289593e-14},
    {0.89214257119978, 0.141060126129492, 0.407205508963968, 2.53385243517952, 0, 9.65295620481434, 0, 0.695435800330236, 2.00755023262381, 3.29336152870136, 0, 1.78864735610198e-14, -1.45456143897546e-14, -1.59407140440659e-14, -1.27256502403865e-14, -9.91526600752597, 1.74986266317689e-14, 1.73481017017152, 5.00796559370718, 1.41646657250367, 0, 0, 0, 0, 0, 0, 0, 0, 1.67839306981372e-14, 0, 15.9443476574685, 0, 2.44096975264614, 7.04647271886991, -2.27776395106694, 0, 0, 0, 0, 0, 0, 0, 0, -1.25723596525151e-14, -1.13140881999668e-14, 1.41799567998805e-14, 0, 1.21347241236521e-14, 0, 0, 0, 0, 0, 0, 0, -1.24344978758018e-14},
    {0.515078753637724, -2.3619485326075e-14, 0.376160336345333, 1.4629203855377, 0, 1.01804649671178e-14, 9.65295620481433, -2.11069547491802e-14, 1.8544954675473, 1.90142316513451, 0, 0, -2.14622752906569e-14, -4.34603909418763e-14, -1.24631731336805e-14, 0, -9.91526600752596, -1.51138454398741e-14, 4.62616045379076, 0.817797356933143, 0, 0, 0, 0, -1.03207533752511e-14, 0, 0, 0, 2.46817376593445e-14, 0, 0, 15.9443476574685, 1.00089905485244e-14, 6.50925267372302, -1.3150676302989, 0, 0, 0, 0, 0, -1.6390734838769e-14, 0, 0, -1.2109391798167e-14, 0, 1.09260746948048e-14, 0, 0, 0, -1.16429829249248e-14, 0, 0, 0, 0, 0, 1.24344978758018e-14},
    {0.728431359084676, -7.94708263760856, 0, 2.53908227033122, 0, 0, -1.27415633137685e-14, 10.2207771580387, 0, 5.0071377623777, 2.28612268350337e-14, -1.8719154526289e-14, 1.31033584296945e-14, -2.53050332009669e-14, 0, 2.83491039062371e-14, 1.64090158394923e-14, -8.49879943502228, 0, 6.93924068068621, 0, -1.52898531740193e-14, 0, 0, 0, -1.42660700700925e-14, 0, 0, 1.84038304706445e-14, 0, -1.43708740249419e-14, 0, 17.9373911146521, 0, 6.27677936394721, 0, 0, 0, 0, 0, 0, 0, 1.74801884942513e-14, 0, 1.54335530678735e-14, 0, 0, 0, 0, -1.82587310754697e-14, 0, 0, 0, -1.41280724493113e-14, 0, 1.68753899743024e-14},
    {0.420560041253715, 0, -7.94708263760862, 1.46593983227031, 0, 0, 0, 0, 10.2207771580387, 2.89087233497828, 0, 0, 0, -4.72854335057725e-14, 0, 2.90947883710161e-14, 2.94685265414151e-14, 0, -8.49879943502229, 4.00637247496575, 0, 0, 0, -1.04474568447982e-14, -1.49316504487054e-14, 0, 0, 0, 2.10839214785391e-14, 0, 0, -1.06464761849486e-14, 0, 17.9373911146521, 3.62390025541881, 0, 0, 0, 0, 0, -1.23540168908514e-14, 0, 0, 0, 2.09495610608972e-14, 1.91267628119894e-14, 0, 0, 0, -2.42410393446135e-14, 0, 0, 0, 0, 0, 0},
    {6.542378855465, 2.78160733626274e-14, 0, -6.91050664139872, 0, 0, -1.56111299636533e-14, 1.45917324702114e-14, 0, 12.2649325896465, 0, -1.97411729296339e-14, 0, 0, 0, 0, 1.23535778391715e-14, 2.43644083996432e-14, -1.70040757261543e-14, -5.66586629001476, 0, 0, 0, 0, 0, 0, 1.69114983860161e-14, 0, 0, 0, 0, 0, 0, 0, 20.4998755596024, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, -1.54091873433635e-14, 0, -2.20095446006121e-14, 0, 0, 1.1817028848966e-14, -1.05566456925328e-14, -1.23638867682084e-14, 2.1316282072803e-14}};
    
    // Compute reference derivatives
    // Declare pointer to array of derivatives on FIAT element
    double *derivatives = new double [num_derivatives];
    
    // Declare coefficients
    double coeff0_0 = 0;
    double coeff0_1 = 0;
    double coeff0_2 = 0;
    double coeff0_3 = 0;
    double coeff0_4 = 0;
    double coeff0_5 = 0;
    double coeff0_6 = 0;
    double coeff0_7 = 0;
    double coeff0_8 = 0;
    double coeff0_9 = 0;
    double coeff0_10 = 0;
    double coeff0_11 = 0;
    double coeff0_12 = 0;
    double coeff0_13 = 0;
    double coeff0_14 = 0;
    double coeff0_15 = 0;
    double coeff0_16 = 0;
    double coeff0_17 = 0;
    double coeff0_18 = 0;
    double coeff0_19 = 0;
    double coeff0_20 = 0;
    double coeff0_21 = 0;
    double coeff0_22 = 0;
    double coeff0_23 = 0;
    double coeff0_24 = 0;
    double coeff0_25 = 0;
    double coeff0_26 = 0;
    double coeff0_27 = 0;
    double coeff0_28 = 0;
    double coeff0_29 = 0;
    double coeff0_30 = 0;
    double coeff0_31 = 0;
    double coeff0_32 = 0;
    double coeff0_33 = 0;
    double coeff0_34 = 0;
    double coeff0_35 = 0;
    double coeff0_36 = 0;
    double coeff0_37 = 0;
    double coeff0_38 = 0;
    double coeff0_39 = 0;
    double coeff0_40 = 0;
    double coeff0_41 = 0;
    double coeff0_42 = 0;
    double coeff0_43 = 0;
    double coeff0_44 = 0;
    double coeff0_45 = 0;
    double coeff0_46 = 0;
    double coeff0_47 = 0;
    double coeff0_48 = 0;
    double coeff0_49 = 0;
    double coeff0_50 = 0;
    double coeff0_51 = 0;
    double coeff0_52 = 0;
    double coeff0_53 = 0;
    double coeff0_54 = 0;
    double coeff0_55 = 0;
    
    // Declare new coefficients
    double new_coeff0_0 = 0;
    double new_coeff0_1 = 0;
    double new_coeff0_2 = 0;
    double new_coeff0_3 = 0;
    double new_coeff0_4 = 0;
    double new_coeff0_5 = 0;
    double new_coeff0_6 = 0;
    double new_coeff0_7 = 0;
    double new_coeff0_8 = 0;
    double new_coeff0_9 = 0;
    double new_coeff0_10 = 0;
    double new_coeff0_11 = 0;
    double new_coeff0_12 = 0;
    double new_coeff0_13 = 0;
    double new_coeff0_14 = 0;
    double new_coeff0_15 = 0;
    double new_coeff0_16 = 0;
    double new_coeff0_17 = 0;
    double new_coeff0_18 = 0;
    double new_coeff0_19 = 0;
    double new_coeff0_20 = 0;
    double new_coeff0_21 = 0;
    double new_coeff0_22 = 0;
    double new_coeff0_23 = 0;
    double new_coeff0_24 = 0;
    double new_coeff0_25 = 0;
    double new_coeff0_26 = 0;
    double new_coeff0_27 = 0;
    double new_coeff0_28 = 0;
    double new_coeff0_29 = 0;
    double new_coeff0_30 = 0;
    double new_coeff0_31 = 0;
    double new_coeff0_32 = 0;
    double new_coeff0_33 = 0;
    double new_coeff0_34 = 0;
    double new_coeff0_35 = 0;
    double new_coeff0_36 = 0;
    double new_coeff0_37 = 0;
    double new_coeff0_38 = 0;
    double new_coeff0_39 = 0;
    double new_coeff0_40 = 0;
    double new_coeff0_41 = 0;
    double new_coeff0_42 = 0;
    double new_coeff0_43 = 0;
    double new_coeff0_44 = 0;
    double new_coeff0_45 = 0;
    double new_coeff0_46 = 0;
    double new_coeff0_47 = 0;
    double new_coeff0_48 = 0;
    double new_coeff0_49 = 0;
    double new_coeff0_50 = 0;
    double new_coeff0_51 = 0;
    double new_coeff0_52 = 0;
    double new_coeff0_53 = 0;
    double new_coeff0_54 = 0;
    double new_coeff0_55 = 0;
    
    // Loop possible derivatives
    for (unsigned int deriv_num = 0; deriv_num < num_derivatives; deriv_num++)
    {
      // Get values from coefficients array
      new_coeff0_0 = coefficients0[dof][0];
      new_coeff0_1 = coefficients0[dof][1];
      new_coeff0_2 = coefficients0[dof][2];
      new_coeff0_3 = coefficients0[dof][3];
      new_coeff0_4 = coefficients0[dof][4];
      new_coeff0_5 = coefficients0[dof][5];
      new_coeff0_6 = coefficients0[dof][6];
      new_coeff0_7 = coefficients0[dof][7];
      new_coeff0_8 = coefficients0[dof][8];
      new_coeff0_9 = coefficients0[dof][9];
      new_coeff0_10 = coefficients0[dof][10];
      new_coeff0_11 = coefficients0[dof][11];
      new_coeff0_12 = coefficients0[dof][12];
      new_coeff0_13 = coefficients0[dof][13];
      new_coeff0_14 = coefficients0[dof][14];
      new_coeff0_15 = coefficients0[dof][15];
      new_coeff0_16 = coefficients0[dof][16];
      new_coeff0_17 = coefficients0[dof][17];
      new_coeff0_18 = coefficients0[dof][18];
      new_coeff0_19 = coefficients0[dof][19];
      new_coeff0_20 = coefficients0[dof][20];
      new_coeff0_21 = coefficients0[dof][21];
      new_coeff0_22 = coefficients0[dof][22];
      new_coeff0_23 = coefficients0[dof][23];
      new_coeff0_24 = coefficients0[dof][24];
      new_coeff0_25 = coefficients0[dof][25];
      new_coeff0_26 = coefficients0[dof][26];
      new_coeff0_27 = coefficients0[dof][27];
      new_coeff0_28 = coefficients0[dof][28];
      new_coeff0_29 = coefficients0[dof][29];
      new_coeff0_30 = coefficients0[dof][30];
      new_coeff0_31 = coefficients0[dof][31];
      new_coeff0_32 = coefficients0[dof][32];
      new_coeff0_33 = coefficients0[dof][33];
      new_coeff0_34 = coefficients0[dof][34];
      new_coeff0_35 = coefficients0[dof][35];
      new_coeff0_36 = coefficients0[dof][36];
      new_coeff0_37 = coefficients0[dof][37];
      new_coeff0_38 = coefficients0[dof][38];
      new_coeff0_39 = coefficients0[dof][39];
      new_coeff0_40 = coefficients0[dof][40];
      new_coeff0_41 = coefficients0[dof][41];
      new_coeff0_42 = coefficients0[dof][42];
      new_coeff0_43 = coefficients0[dof][43];
      new_coeff0_44 = coefficients0[dof][44];
      new_coeff0_45 = coefficients0[dof][45];
      new_coeff0_46 = coefficients0[dof][46];
      new_coeff0_47 = coefficients0[dof][47];
      new_coeff0_48 = coefficients0[dof][48];
      new_coeff0_49 = coefficients0[dof][49];
      new_coeff0_50 = coefficients0[dof][50];
      new_coeff0_51 = coefficients0[dof][51];
      new_coeff0_52 = coefficients0[dof][52];
      new_coeff0_53 = coefficients0[dof][53];
      new_coeff0_54 = coefficients0[dof][54];
      new_coeff0_55 = coefficients0[dof][55];
    
      // Loop derivative order
      for (unsigned int j = 0; j < n; j++)
      {
        // Update old coefficients
        coeff0_0 = new_coeff0_0;
        coeff0_1 = new_coeff0_1;
        coeff0_2 = new_coeff0_2;
        coeff0_3 = new_coeff0_3;
        coeff0_4 = new_coeff0_4;
        coeff0_5 = new_coeff0_5;
        coeff0_6 = new_coeff0_6;
        coeff0_7 = new_coeff0_7;
        coeff0_8 = new_coeff0_8;
        coeff0_9 = new_coeff0_9;
        coeff0_10 = new_coeff0_10;
        coeff0_11 = new_coeff0_11;
        coeff0_12 = new_coeff0_12;
        coeff0_13 = new_coeff0_13;
        coeff0_14 = new_coeff0_14;
        coeff0_15 = new_coeff0_15;
        coeff0_16 = new_coeff0_16;
        coeff0_17 = new_coeff0_17;
        coeff0_18 = new_coeff0_18;
        coeff0_19 = new_coeff0_19;
        coeff0_20 = new_coeff0_20;
        coeff0_21 = new_coeff0_21;
        coeff0_22 = new_coeff0_22;
        coeff0_23 = new_coeff0_23;
        coeff0_24 = new_coeff0_24;
        coeff0_25 = new_coeff0_25;
        coeff0_26 = new_coeff0_26;
        coeff0_27 = new_coeff0_27;
        coeff0_28 = new_coeff0_28;
        coeff0_29 = new_coeff0_29;
        coeff0_30 = new_coeff0_30;
        coeff0_31 = new_coeff0_31;
        coeff0_32 = new_coeff0_32;
        coeff0_33 = new_coeff0_33;
        coeff0_34 = new_coeff0_34;
        coeff0_35 = new_coeff0_35;
        coeff0_36 = new_coeff0_36;
        coeff0_37 = new_coeff0_37;
        coeff0_38 = new_coeff0_38;
        coeff0_39 = new_coeff0_39;
        coeff0_40 = new_coeff0_40;
        coeff0_41 = new_coeff0_41;
        coeff0_42 = new_coeff0_42;
        coeff0_43 = new_coeff0_43;
        coeff0_44 = new_coeff0_44;
        coeff0_45 = new_coeff0_45;
        coeff0_46 = new_coeff0_46;
        coeff0_47 = new_coeff0_47;
        coeff0_48 = new_coeff0_48;
        coeff0_49 = new_coeff0_49;
        coeff0_50 = new_coeff0_50;
        coeff0_51 = new_coeff0_51;
        coeff0_52 = new_coeff0_52;
        coeff0_53 = new_coeff0_53;
        coeff0_54 = new_coeff0_54;
        coeff0_55 = new_coeff0_55;
    
        if(combinations[deriv_num][j] == 0)
        {
          new_coeff0_0 = coeff0_0*dmats0[0][0] + coeff0_1*dmats0[1][0] + coeff0_2*dmats0[2][0] + coeff0_3*dmats0[3][0] + coeff0_4*dmats0[4][0] + coeff0_5*dmats0[5][0] + coeff0_6*dmats0[6][0] + coeff0_7*dmats0[7][0] + coeff0_8*dmats0[8][0] + coeff0_9*dmats0[9][0] + coeff0_10*dmats0[10][0] + coeff0_11*dmats0[11][0] + coeff0_12*dmats0[12][0] + coeff0_13*dmats0[13][0] + coeff0_14*dmats0[14][0] + coeff0_15*dmats0[15][0] + coeff0_16*dmats0[16][0] + coeff0_17*dmats0[17][0] + coeff0_18*dmats0[18][0] + coeff0_19*dmats0[19][0] + coeff0_20*dmats0[20][0] + coeff0_21*dmats0[21][0] + coeff0_22*dmats0[22][0] + coeff0_23*dmats0[23][0] + coeff0_24*dmats0[24][0] + coeff0_25*dmats0[25][0] + coeff0_26*dmats0[26][0] + coeff0_27*dmats0[27][0] + coeff0_28*dmats0[28][0] + coeff0_29*dmats0[29][0] + coeff0_30*dmats0[30][0] + coeff0_31*dmats0[31][0] + coeff0_32*dmats0[32][0] + coeff0_33*dmats0[33][0] + coeff0_34*dmats0[34][0] + coeff0_35*dmats0[35][0] + coeff0_36*dmats0[36][0] + coeff0_37*dmats0[37][0] + coeff0_38*dmats0[38][0] + coeff0_39*dmats0[39][0] + coeff0_40*dmats0[40][0] + coeff0_41*dmats0[41][0] + coeff0_42*dmats0[42][0] + coeff0_43*dmats0[43][0] + coeff0_44*dmats0[44][0] + coeff0_45*dmats0[45][0] + coeff0_46*dmats0[46][0] + coeff0_47*dmats0[47][0] + coeff0_48*dmats0[48][0] + coeff0_49*dmats0[49][0] + coeff0_50*dmats0[50][0] + coeff0_51*dmats0[51][0] + coeff0_52*dmats0[52][0] + coeff0_53*dmats0[53][0] + coeff0_54*dmats0[54][0] + coeff0_55*dmats0[55][0];
          new_coeff0_1 = coeff0_0*dmats0[0][1] + coeff0_1*dmats0[1][1] + coeff0_2*dmats0[2][1] + coeff0_3*dmats0[3][1] + coeff0_4*dmats0[4][1] + coeff0_5*dmats0[5][1] + coeff0_6*dmats0[6][1] + coeff0_7*dmats0[7][1] + coeff0_8*dmats0[8][1] + coeff0_9*dmats0[9][1] + coeff0_10*dmats0[10][1] + coeff0_11*dmats0[11][1] + coeff0_12*dmats0[12][1] + coeff0_13*dmats0[13][1] + coeff0_14*dmats0[14][1] + coeff0_15*dmats0[15][1] + coeff0_16*dmats0[16][1] + coeff0_17*dmats0[17][1] + coeff0_18*dmats0[18][1] + coeff0_19*dmats0[19][1] + coeff0_20*dmats0[20][1] + coeff0_21*dmats0[21][1] + coeff0_22*dmats0[22][1] + coeff0_23*dmats0[23][1] + coeff0_24*dmats0[24][1] + coeff0_25*dmats0[25][1] + coeff0_26*dmats0[26][1] + coeff0_27*dmats0[27][1] + coeff0_28*dmats0[28][1] + coeff0_29*dmats0[29][1] + coeff0_30*dmats0[30][1] + coeff0_31*dmats0[31][1] + coeff0_32*dmats0[32][1] + coeff0_33*dmats0[33][1] + coeff0_34*dmats0[34][1] + coeff0_35*dmats0[35][1] + coeff0_36*dmats0[36][1] + coeff0_37*dmats0[37][1] + coeff0_38*dmats0[38][1] + coeff0_39*dmats0[39][1] + coeff0_40*dmats0[40][1] + coeff0_41*dmats0[41][1] + coeff0_42*dmats0[42][1] + coeff0_43*dmats0[43][1] + coeff0_44*dmats0[44][1] + coeff0_45*dmats0[45][1] + coeff0_46*dmats0[46][1] + coeff0_47*dmats0[47][1] + coeff0_48*dmats0[48][1] + coeff0_49*dmats0[49][1] + coeff0_50*dmats0[50][1] + coeff0_51*dmats0[51][1] + coeff0_52*dmats0[52][1] + coeff0_53*dmats0[53][1] + coeff0_54*dmats0[54][1] + coeff0_55*dmats0[55][1];
          new_coeff0_2 = coeff0_0*dmats0[0][2] + coeff0_1*dmats0[1][2] + coeff0_2*dmats0[2][2] + coeff0_3*dmats0[3][2] + coeff0_4*dmats0[4][2] + coeff0_5*dmats0[5][2] + coeff0_6*dmats0[6][2] + coeff0_7*dmats0[7][2] + coeff0_8*dmats0[8][2] + coeff0_9*dmats0[9][2] + coeff0_10*dmats0[10][2] + coeff0_11*dmats0[11][2] + coeff0_12*dmats0[12][2] + coeff0_13*dmats0[13][2] + coeff0_14*dmats0[14][2] + coeff0_15*dmats0[15][2] + coeff0_16*dmats0[16][2] + coeff0_17*dmats0[17][2] + coeff0_18*dmats0[18][2] + coeff0_19*dmats0[19][2] + coeff0_20*dmats0[20][2] + coeff0_21*dmats0[21][2] + coeff0_22*dmats0[22][2] + coeff0_23*dmats0[23][2] + coeff0_24*dmats0[24][2] + coeff0_25*dmats0[25][2] + coeff0_26*dmats0[26][2] + coeff0_27*dmats0[27][2] + coeff0_28*dmats0[28][2] + coeff0_29*dmats0[29][2] + coeff0_30*dmats0[30][2] + coeff0_31*dmats0[31][2] + coeff0_32*dmats0[32][2] + coeff0_33*dmats0[33][2] + coeff0_34*dmats0[34][2] + coeff0_35*dmats0[35][2] + coeff0_36*dmats0[36][2] + coeff0_37*dmats0[37][2] + coeff0_38*dmats0[38][2] + coeff0_39*dmats0[39][2] + coeff0_40*dmats0[40][2] + coeff0_41*dmats0[41][2] + coeff0_42*dmats0[42][2] + coeff0_43*dmats0[43][2] + coeff0_44*dmats0[44][2] + coeff0_45*dmats0[45][2] + coeff0_46*dmats0[46][2] + coeff0_47*dmats0[47][2] + coeff0_48*dmats0[48][2] + coeff0_49*dmats0[49][2] + coeff0_50*dmats0[50][2] + coeff0_51*dmats0[51][2] + coeff0_52*dmats0[52][2] + coeff0_53*dmats0[53][2] + coeff0_54*dmats0[54][2] + coeff0_55*dmats0[55][2];
          new_coeff0_3 = coeff0_0*dmats0[0][3] + coeff0_1*dmats0[1][3] + coeff0_2*dmats0[2][3] + coeff0_3*dmats0[3][3] + coeff0_4*dmats0[4][3] + coeff0_5*dmats0[5][3] + coeff0_6*dmats0[6][3] + coeff0_7*dmats0[7][3] + coeff0_8*dmats0[8][3] + coeff0_9*dmats0[9][3] + coeff0_10*dmats0[10][3] + coeff0_11*dmats0[11][3] + coeff0_12*dmats0[12][3] + coeff0_13*dmats0[13][3] + coeff0_14*dmats0[14][3] + coeff0_15*dmats0[15][3] + coeff0_16*dmats0[16][3] + coeff0_17*dmats0[17][3] + coeff0_18*dmats0[18][3] + coeff0_19*dmats0[19][3] + coeff0_20*dmats0[20][3] + coeff0_21*dmats0[21][3] + coeff0_22*dmats0[22][3] + coeff0_23*dmats0[23][3] + coeff0_24*dmats0[24][3] + coeff0_25*dmats0[25][3] + coeff0_26*dmats0[26][3] + coeff0_27*dmats0[27][3] + coeff0_28*dmats0[28][3] + coeff0_29*dmats0[29][3] + coeff0_30*dmats0[30][3] + coeff0_31*dmats0[31][3] + coeff0_32*dmats0[32][3] + coeff0_33*dmats0[33][3] + coeff0_34*dmats0[34][3] + coeff0_35*dmats0[35][3] + coeff0_36*dmats0[36][3] + coeff0_37*dmats0[37][3] + coeff0_38*dmats0[38][3] + coeff0_39*dmats0[39][3] + coeff0_40*dmats0[40][3] + coeff0_41*dmats0[41][3] + coeff0_42*dmats0[42][3] + coeff0_43*dmats0[43][3] + coeff0_44*dmats0[44][3] + coeff0_45*dmats0[45][3] + coeff0_46*dmats0[46][3] + coeff0_47*dmats0[47][3] + coeff0_48*dmats0[48][3] + coeff0_49*dmats0[49][3] + coeff0_50*dmats0[50][3] + coeff0_51*dmats0[51][3] + coeff0_52*dmats0[52][3] + coeff0_53*dmats0[53][3] + coeff0_54*dmats0[54][3] + coeff0_55*dmats0[55][3];
          new_coeff0_4 = coeff0_0*dmats0[0][4] + coeff0_1*dmats0[1][4] + coeff0_2*dmats0[2][4] + coeff0_3*dmats0[3][4] + coeff0_4*dmats0[4][4] + coeff0_5*dmats0[5][4] + coeff0_6*dmats0[6][4] + coeff0_7*dmats0[7][4] + coeff0_8*dmats0[8][4] + coeff0_9*dmats0[9][4] + coeff0_10*dmats0[10][4] + coeff0_11*dmats0[11][4] + coeff0_12*dmats0[12][4] + coeff0_13*dmats0[13][4] + coeff0_14*dmats0[14][4] + coeff0_15*dmats0[15][4] + coeff0_16*dmats0[16][4] + coeff0_17*dmats0[17][4] + coeff0_18*dmats0[18][4] + coeff0_19*dmats0[19][4] + coeff0_20*dmats0[20][4] + coeff0_21*dmats0[21][4] + coeff0_22*dmats0[22][4] + coeff0_23*dmats0[23][4] + coeff0_24*dmats0[24][4] + coeff0_25*dmats0[25][4] + coeff0_26*dmats0[26][4] + coeff0_27*dmats0[27][4] + coeff0_28*dmats0[28][4] + coeff0_29*dmats0[29][4] + coeff0_30*dmats0[30][4] + coeff0_31*dmats0[31][4] + coeff0_32*dmats0[32][4] + coeff0_33*dmats0[33][4] + coeff0_34*dmats0[34][4] + coeff0_35*dmats0[35][4] + coeff0_36*dmats0[36][4] + coeff0_37*dmats0[37][4] + coeff0_38*dmats0[38][4] + coeff0_39*dmats0[39][4] + coeff0_40*dmats0[40][4] + coeff0_41*dmats0[41][4] + coeff0_42*dmats0[42][4] + coeff0_43*dmats0[43][4] + coeff0_44*dmats0[44][4] + coeff0_45*dmats0[45][4] + coeff0_46*dmats0[46][4] + coeff0_47*dmats0[47][4] + coeff0_48*dmats0[48][4] + coeff0_49*dmats0[49][4] + coeff0_50*dmats0[50][4] + coeff0_51*dmats0[51][4] + coeff0_52*dmats0[52][4] + coeff0_53*dmats0[53][4] + coeff0_54*dmats0[54][4] + coeff0_55*dmats0[55][4];
          new_coeff0_5 = coeff0_0*dmats0[0][5] + coeff0_1*dmats0[1][5] + coeff0_2*dmats0[2][5] + coeff0_3*dmats0[3][5] + coeff0_4*dmats0[4][5] + coeff0_5*dmats0[5][5] + coeff0_6*dmats0[6][5] + coeff0_7*dmats0[7][5] + coeff0_8*dmats0[8][5] + coeff0_9*dmats0[9][5] + coeff0_10*dmats0[10][5] + coeff0_11*dmats0[11][5] + coeff0_12*dmats0[12][5] + coeff0_13*dmats0[13][5] + coeff0_14*dmats0[14][5] + coeff0_15*dmats0[15][5] + coeff0_16*dmats0[16][5] + coeff0_17*dmats0[17][5] + coeff0_18*dmats0[18][5] + coeff0_19*dmats0[19][5] + coeff0_20*dmats0[20][5] + coeff0_21*dmats0[21][5] + coeff0_22*dmats0[22][5] + coeff0_23*dmats0[23][5] + coeff0_24*dmats0[24][5] + coeff0_25*dmats0[25][5] + coeff0_26*dmats0[26][5] + coeff0_27*dmats0[27][5] + coeff0_28*dmats0[28][5] + coeff0_29*dmats0[29][5] + coeff0_30*dmats0[30][5] + coeff0_31*dmats0[31][5] + coeff0_32*dmats0[32][5] + coeff0_33*dmats0[33][5] + coeff0_34*dmats0[34][5] + coeff0_35*dmats0[35][5] + coeff0_36*dmats0[36][5] + coeff0_37*dmats0[37][5] + coeff0_38*dmats0[38][5] + coeff0_39*dmats0[39][5] + coeff0_40*dmats0[40][5] + coeff0_41*dmats0[41][5] + coeff0_42*dmats0[42][5] + coeff0_43*dmats0[43][5] + coeff0_44*dmats0[44][5] + coeff0_45*dmats0[45][5] + coeff0_46*dmats0[46][5] + coeff0_47*dmats0[47][5] + coeff0_48*dmats0[48][5] + coeff0_49*dmats0[49][5] + coeff0_50*dmats0[50][5] + coeff0_51*dmats0[51][5] + coeff0_52*dmats0[52][5] + coeff0_53*dmats0[53][5] + coeff0_54*dmats0[54][5] + coeff0_55*dmats0[55][5];
          new_coeff0_6 = coeff0_0*dmats0[0][6] + coeff0_1*dmats0[1][6] + coeff0_2*dmats0[2][6] + coeff0_3*dmats0[3][6] + coeff0_4*dmats0[4][6] + coeff0_5*dmats0[5][6] + coeff0_6*dmats0[6][6] + coeff0_7*dmats0[7][6] + coeff0_8*dmats0[8][6] + coeff0_9*dmats0[9][6] + coeff0_10*dmats0[10][6] + coeff0_11*dmats0[11][6] + coeff0_12*dmats0[12][6] + coeff0_13*dmats0[13][6] + coeff0_14*dmats0[14][6] + coeff0_15*dmats0[15][6] + coeff0_16*dmats0[16][6] + coeff0_17*dmats0[17][6] + coeff0_18*dmats0[18][6] + coeff0_19*dmats0[19][6] + coeff0_20*dmats0[20][6] + coeff0_21*dmats0[21][6] + coeff0_22*dmats0[22][6] + coeff0_23*dmats0[23][6] + coeff0_24*dmats0[24][6] + coeff0_25*dmats0[25][6] + coeff0_26*dmats0[26][6] + coeff0_27*dmats0[27][6] + coeff0_28*dmats0[28][6] + coeff0_29*dmats0[29][6] + coeff0_30*dmats0[30][6] + coeff0_31*dmats0[31][6] + coeff0_32*dmats0[32][6] + coeff0_33*dmats0[33][6] + coeff0_34*dmats0[34][6] + coeff0_35*dmats0[35][6] + coeff0_36*dmats0[36][6] + coeff0_37*dmats0[37][6] + coeff0_38*dmats0[38][6] + coeff0_39*dmats0[39][6] + coeff0_40*dmats0[40][6] + coeff0_41*dmats0[41][6] + coeff0_42*dmats0[42][6] + coeff0_43*dmats0[43][6] + coeff0_44*dmats0[44][6] + coeff0_45*dmats0[45][6] + coeff0_46*dmats0[46][6] + coeff0_47*dmats0[47][6] + coeff0_48*dmats0[48][6] + coeff0_49*dmats0[49][6] + coeff0_50*dmats0[50][6] + coeff0_51*dmats0[51][6] + coeff0_52*dmats0[52][6] + coeff0_53*dmats0[53][6] + coeff0_54*dmats0[54][6] + coeff0_55*dmats0[55][6];
          new_coeff0_7 = coeff0_0*dmats0[0][7] + coeff0_1*dmats0[1][7] + coeff0_2*dmats0[2][7] + coeff0_3*dmats0[3][7] + coeff0_4*dmats0[4][7] + coeff0_5*dmats0[5][7] + coeff0_6*dmats0[6][7] + coeff0_7*dmats0[7][7] + coeff0_8*dmats0[8][7] + coeff0_9*dmats0[9][7] + coeff0_10*dmats0[10][7] + coeff0_11*dmats0[11][7] + coeff0_12*dmats0[12][7] + coeff0_13*dmats0[13][7] + coeff0_14*dmats0[14][7] + coeff0_15*dmats0[15][7] + coeff0_16*dmats0[16][7] + coeff0_17*dmats0[17][7] + coeff0_18*dmats0[18][7] + coeff0_19*dmats0[19][7] + coeff0_20*dmats0[20][7] + coeff0_21*dmats0[21][7] + coeff0_22*dmats0[22][7] + coeff0_23*dmats0[23][7] + coeff0_24*dmats0[24][7] + coeff0_25*dmats0[25][7] + coeff0_26*dmats0[26][7] + coeff0_27*dmats0[27][7] + coeff0_28*dmats0[28][7] + coeff0_29*dmats0[29][7] + coeff0_30*dmats0[30][7] + coeff0_31*dmats0[31][7] + coeff0_32*dmats0[32][7] + coeff0_33*dmats0[33][7] + coeff0_34*dmats0[34][7] + coeff0_35*dmats0[35][7] + coeff0_36*dmats0[36][7] + coeff0_37*dmats0[37][7] + coeff0_38*dmats0[38][7] + coeff0_39*dmats0[39][7] + coeff0_40*dmats0[40][7] + coeff0_41*dmats0[41][7] + coeff0_42*dmats0[42][7] + coeff0_43*dmats0[43][7] + coeff0_44*dmats0[44][7] + coeff0_45*dmats0[45][7] + coeff0_46*dmats0[46][7] + coeff0_47*dmats0[47][7] + coeff0_48*dmats0[48][7] + coeff0_49*dmats0[49][7] + coeff0_50*dmats0[50][7] + coeff0_51*dmats0[51][7] + coeff0_52*dmats0[52][7] + coeff0_53*dmats0[53][7] + coeff0_54*dmats0[54][7] + coeff0_55*dmats0[55][7];
          new_coeff0_8 = coeff0_0*dmats0[0][8] + coeff0_1*dmats0[1][8] + coeff0_2*dmats0[2][8] + coeff0_3*dmats0[3][8] + coeff0_4*dmats0[4][8] + coeff0_5*dmats0[5][8] + coeff0_6*dmats0[6][8] + coeff0_7*dmats0[7][8] + coeff0_8*dmats0[8][8] + coeff0_9*dmats0[9][8] + coeff0_10*dmats0[10][8] + coeff0_11*dmats0[11][8] + coeff0_12*dmats0[12][8] + coeff0_13*dmats0[13][8] + coeff0_14*dmats0[14][8] + coeff0_15*dmats0[15][8] + coeff0_16*dmats0[16][8] + coeff0_17*dmats0[17][8] + coeff0_18*dmats0[18][8] + coeff0_19*dmats0[19][8] + coeff0_20*dmats0[20][8] + coeff0_21*dmats0[21][8] + coeff0_22*dmats0[22][8] + coeff0_23*dmats0[23][8] + coeff0_24*dmats0[24][8] + coeff0_25*dmats0[25][8] + coeff0_26*dmats0[26][8] + coeff0_27*dmats0[27][8] + coeff0_28*dmats0[28][8] + coeff0_29*dmats0[29][8] + coeff0_30*dmats0[30][8] + coeff0_31*dmats0[31][8] + coeff0_32*dmats0[32][8] + coeff0_33*dmats0[33][8] + coeff0_34*dmats0[34][8] + coeff0_35*dmats0[35][8] + coeff0_36*dmats0[36][8] + coeff0_37*dmats0[37][8] + coeff0_38*dmats0[38][8] + coeff0_39*dmats0[39][8] + coeff0_40*dmats0[40][8] + coeff0_41*dmats0[41][8] + coeff0_42*dmats0[42][8] + coeff0_43*dmats0[43][8] + coeff0_44*dmats0[44][8] + coeff0_45*dmats0[45][8] + coeff0_46*dmats0[46][8] + coeff0_47*dmats0[47][8] + coeff0_48*dmats0[48][8] + coeff0_49*dmats0[49][8] + coeff0_50*dmats0[50][8] + coeff0_51*dmats0[51][8] + coeff0_52*dmats0[52][8] + coeff0_53*dmats0[53][8] + coeff0_54*dmats0[54][8] + coeff0_55*dmats0[55][8];
          new_coeff0_9 = coeff0_0*dmats0[0][9] + coeff0_1*dmats0[1][9] + coeff0_2*dmats0[2][9] + coeff0_3*dmats0[3][9] + coeff0_4*dmats0[4][9] + coeff0_5*dmats0[5][9] + coeff0_6*dmats0[6][9] + coeff0_7*dmats0[7][9] + coeff0_8*dmats0[8][9] + coeff0_9*dmats0[9][9] + coeff0_10*dmats0[10][9] + coeff0_11*dmats0[11][9] + coeff0_12*dmats0[12][9] + coeff0_13*dmats0[13][9] + coeff0_14*dmats0[14][9] + coeff0_15*dmats0[15][9] + coeff0_16*dmats0[16][9] + coeff0_17*dmats0[17][9] + coeff0_18*dmats0[18][9] + coeff0_19*dmats0[19][9] + coeff0_20*dmats0[20][9] + coeff0_21*dmats0[21][9] + coeff0_22*dmats0[22][9] + coeff0_23*dmats0[23][9] + coeff0_24*dmats0[24][9] + coeff0_25*dmats0[25][9] + coeff0_26*dmats0[26][9] + coeff0_27*dmats0[27][9] + coeff0_28*dmats0[28][9] + coeff0_29*dmats0[29][9] + coeff0_30*dmats0[30][9] + coeff0_31*dmats0[31][9] + coeff0_32*dmats0[32][9] + coeff0_33*dmats0[33][9] + coeff0_34*dmats0[34][9] + coeff0_35*dmats0[35][9] + coeff0_36*dmats0[36][9] + coeff0_37*dmats0[37][9] + coeff0_38*dmats0[38][9] + coeff0_39*dmats0[39][9] + coeff0_40*dmats0[40][9] + coeff0_41*dmats0[41][9] + coeff0_42*dmats0[42][9] + coeff0_43*dmats0[43][9] + coeff0_44*dmats0[44][9] + coeff0_45*dmats0[45][9] + coeff0_46*dmats0[46][9] + coeff0_47*dmats0[47][9] + coeff0_48*dmats0[48][9] + coeff0_49*dmats0[49][9] + coeff0_50*dmats0[50][9] + coeff0_51*dmats0[51][9] + coeff0_52*dmats0[52][9] + coeff0_53*dmats0[53][9] + coeff0_54*dmats0[54][9] + coeff0_55*dmats0[55][9];
          new_coeff0_10 = coeff0_0*dmats0[0][10] + coeff0_1*dmats0[1][10] + coeff0_2*dmats0[2][10] + coeff0_3*dmats0[3][10] + coeff0_4*dmats0[4][10] + coeff0_5*dmats0[5][10] + coeff0_6*dmats0[6][10] + coeff0_7*dmats0[7][10] + coeff0_8*dmats0[8][10] + coeff0_9*dmats0[9][10] + coeff0_10*dmats0[10][10] + coeff0_11*dmats0[11][10] + coeff0_12*dmats0[12][10] + coeff0_13*dmats0[13][10] + coeff0_14*dmats0[14][10] + coeff0_15*dmats0[15][10] + coeff0_16*dmats0[16][10] + coeff0_17*dmats0[17][10] + coeff0_18*dmats0[18][10] + coeff0_19*dmats0[19][10] + coeff0_20*dmats0[20][10] + coeff0_21*dmats0[21][10] + coeff0_22*dmats0[22][10] + coeff0_23*dmats0[23][10] + coeff0_24*dmats0[24][10] + coeff0_25*dmats0[25][10] + coeff0_26*dmats0[26][10] + coeff0_27*dmats0[27][10] + coeff0_28*dmats0[28][10] + coeff0_29*dmats0[29][10] + coeff0_30*dmats0[30][10] + coeff0_31*dmats0[31][10] + coeff0_32*dmats0[32][10] + coeff0_33*dmats0[33][10] + coeff0_34*dmats0[34][10] + coeff0_35*dmats0[35][10] + coeff0_36*dmats0[36][10] + coeff0_37*dmats0[37][10] + coeff0_38*dmats0[38][10] + coeff0_39*dmats0[39][10] + coeff0_40*dmats0[40][10] + coeff0_41*dmats0[41][10] + coeff0_42*dmats0[42][10] + coeff0_43*dmats0[43][10] + coeff0_44*dmats0[44][10] + coeff0_45*dmats0[45][10] + coeff0_46*dmats0[46][10] + coeff0_47*dmats0[47][10] + coeff0_48*dmats0[48][10] + coeff0_49*dmats0[49][10] + coeff0_50*dmats0[50][10] + coeff0_51*dmats0[51][10] + coeff0_52*dmats0[52][10] + coeff0_53*dmats0[53][10] + coeff0_54*dmats0[54][10] + coeff0_55*dmats0[55][10];
          new_coeff0_11 = coeff0_0*dmats0[0][11] + coeff0_1*dmats0[1][11] + coeff0_2*dmats0[2][11] + coeff0_3*dmats0[3][11] + coeff0_4*dmats0[4][11] + coeff0_5*dmats0[5][11] + coeff0_6*dmats0[6][11] + coeff0_7*dmats0[7][11] + coeff0_8*dmats0[8][11] + coeff0_9*dmats0[9][11] + coeff0_10*dmats0[10][11] + coeff0_11*dmats0[11][11] + coeff0_12*dmats0[12][11] + coeff0_13*dmats0[13][11] + coeff0_14*dmats0[14][11] + coeff0_15*dmats0[15][11] + coeff0_16*dmats0[16][11] + coeff0_17*dmats0[17][11] + coeff0_18*dmats0[18][11] + coeff0_19*dmats0[19][11] + coeff0_20*dmats0[20][11] + coeff0_21*dmats0[21][11] + coeff0_22*dmats0[22][11] + coeff0_23*dmats0[23][11] + coeff0_24*dmats0[24][11] + coeff0_25*dmats0[25][11] + coeff0_26*dmats0[26][11] + coeff0_27*dmats0[27][11] + coeff0_28*dmats0[28][11] + coeff0_29*dmats0[29][11] + coeff0_30*dmats0[30][11] + coeff0_31*dmats0[31][11] + coeff0_32*dmats0[32][11] + coeff0_33*dmats0[33][11] + coeff0_34*dmats0[34][11] + coeff0_35*dmats0[35][11] + coeff0_36*dmats0[36][11] + coeff0_37*dmats0[37][11] + coeff0_38*dmats0[38][11] + coeff0_39*dmats0[39][11] + coeff0_40*dmats0[40][11] + coeff0_41*dmats0[41][11] + coeff0_42*dmats0[42][11] + coeff0_43*dmats0[43][11] + coeff0_44*dmats0[44][11] + coeff0_45*dmats0[45][11] + coeff0_46*dmats0[46][11] + coeff0_47*dmats0[47][11] + coeff0_48*dmats0[48][11] + coeff0_49*dmats0[49][11] + coeff0_50*dmats0[50][11] + coeff0_51*dmats0[51][11] + coeff0_52*dmats0[52][11] + coeff0_53*dmats0[53][11] + coeff0_54*dmats0[54][11] + coeff0_55*dmats0[55][11];
          new_coeff0_12 = coeff0_0*dmats0[0][12] + coeff0_1*dmats0[1][12] + coeff0_2*dmats0[2][12] + coeff0_3*dmats0[3][12] + coeff0_4*dmats0[4][12] + coeff0_5*dmats0[5][12] + coeff0_6*dmats0[6][12] + coeff0_7*dmats0[7][12] + coeff0_8*dmats0[8][12] + coeff0_9*dmats0[9][12] + coeff0_10*dmats0[10][12] + coeff0_11*dmats0[11][12] + coeff0_12*dmats0[12][12] + coeff0_13*dmats0[13][12] + coeff0_14*dmats0[14][12] + coeff0_15*dmats0[15][12] + coeff0_16*dmats0[16][12] + coeff0_17*dmats0[17][12] + coeff0_18*dmats0[18][12] + coeff0_19*dmats0[19][12] + coeff0_20*dmats0[20][12] + coeff0_21*dmats0[21][12] + coeff0_22*dmats0[22][12] + coeff0_23*dmats0[23][12] + coeff0_24*dmats0[24][12] + coeff0_25*dmats0[25][12] + coeff0_26*dmats0[26][12] + coeff0_27*dmats0[27][12] + coeff0_28*dmats0[28][12] + coeff0_29*dmats0[29][12] + coeff0_30*dmats0[30][12] + coeff0_31*dmats0[31][12] + coeff0_32*dmats0[32][12] + coeff0_33*dmats0[33][12] + coeff0_34*dmats0[34][12] + coeff0_35*dmats0[35][12] + coeff0_36*dmats0[36][12] + coeff0_37*dmats0[37][12] + coeff0_38*dmats0[38][12] + coeff0_39*dmats0[39][12] + coeff0_40*dmats0[40][12] + coeff0_41*dmats0[41][12] + coeff0_42*dmats0[42][12] + coeff0_43*dmats0[43][12] + coeff0_44*dmats0[44][12] + coeff0_45*dmats0[45][12] + coeff0_46*dmats0[46][12] + coeff0_47*dmats0[47][12] + coeff0_48*dmats0[48][12] + coeff0_49*dmats0[49][12] + coeff0_50*dmats0[50][12] + coeff0_51*dmats0[51][12] + coeff0_52*dmats0[52][12] + coeff0_53*dmats0[53][12] + coeff0_54*dmats0[54][12] + coeff0_55*dmats0[55][12];
          new_coeff0_13 = coeff0_0*dmats0[0][13] + coeff0_1*dmats0[1][13] + coeff0_2*dmats0[2][13] + coeff0_3*dmats0[3][13] + coeff0_4*dmats0[4][13] + coeff0_5*dmats0[5][13] + coeff0_6*dmats0[6][13] + coeff0_7*dmats0[7][13] + coeff0_8*dmats0[8][13] + coeff0_9*dmats0[9][13] + coeff0_10*dmats0[10][13] + coeff0_11*dmats0[11][13] + coeff0_12*dmats0[12][13] + coeff0_13*dmats0[13][13] + coeff0_14*dmats0[14][13] + coeff0_15*dmats0[15][13] + coeff0_16*dmats0[16][13] + coeff0_17*dmats0[17][13] + coeff0_18*dmats0[18][13] + coeff0_19*dmats0[19][13] + coeff0_20*dmats0[20][13] + coeff0_21*dmats0[21][13] + coeff0_22*dmats0[22][13] + coeff0_23*dmats0[23][13] + coeff0_24*dmats0[24][13] + coeff0_25*dmats0[25][13] + coeff0_26*dmats0[26][13] + coeff0_27*dmats0[27][13] + coeff0_28*dmats0[28][13] + coeff0_29*dmats0[29][13] + coeff0_30*dmats0[30][13] + coeff0_31*dmats0[31][13] + coeff0_32*dmats0[32][13] + coeff0_33*dmats0[33][13] + coeff0_34*dmats0[34][13] + coeff0_35*dmats0[35][13] + coeff0_36*dmats0[36][13] + coeff0_37*dmats0[37][13] + coeff0_38*dmats0[38][13] + coeff0_39*dmats0[39][13] + coeff0_40*dmats0[40][13] + coeff0_41*dmats0[41][13] + coeff0_42*dmats0[42][13] + coeff0_43*dmats0[43][13] + coeff0_44*dmats0[44][13] + coeff0_45*dmats0[45][13] + coeff0_46*dmats0[46][13] + coeff0_47*dmats0[47][13] + coeff0_48*dmats0[48][13] + coeff0_49*dmats0[49][13] + coeff0_50*dmats0[50][13] + coeff0_51*dmats0[51][13] + coeff0_52*dmats0[52][13] + coeff0_53*dmats0[53][13] + coeff0_54*dmats0[54][13] + coeff0_55*dmats0[55][13];
          new_coeff0_14 = coeff0_0*dmats0[0][14] + coeff0_1*dmats0[1][14] + coeff0_2*dmats0[2][14] + coeff0_3*dmats0[3][14] + coeff0_4*dmats0[4][14] + coeff0_5*dmats0[5][14] + coeff0_6*dmats0[6][14] + coeff0_7*dmats0[7][14] + coeff0_8*dmats0[8][14] + coeff0_9*dmats0[9][14] + coeff0_10*dmats0[10][14] + coeff0_11*dmats0[11][14] + coeff0_12*dmats0[12][14] + coeff0_13*dmats0[13][14] + coeff0_14*dmats0[14][14] + coeff0_15*dmats0[15][14] + coeff0_16*dmats0[16][14] + coeff0_17*dmats0[17][14] + coeff0_18*dmats0[18][14] + coeff0_19*dmats0[19][14] + coeff0_20*dmats0[20][14] + coeff0_21*dmats0[21][14] + coeff0_22*dmats0[22][14] + coeff0_23*dmats0[23][14] + coeff0_24*dmats0[24][14] + coeff0_25*dmats0[25][14] + coeff0_26*dmats0[26][14] + coeff0_27*dmats0[27][14] + coeff0_28*dmats0[28][14] + coeff0_29*dmats0[29][14] + coeff0_30*dmats0[30][14] + coeff0_31*dmats0[31][14] + coeff0_32*dmats0[32][14] + coeff0_33*dmats0[33][14] + coeff0_34*dmats0[34][14] + coeff0_35*dmats0[35][14] + coeff0_36*dmats0[36][14] + coeff0_37*dmats0[37][14] + coeff0_38*dmats0[38][14] + coeff0_39*dmats0[39][14] + coeff0_40*dmats0[40][14] + coeff0_41*dmats0[41][14] + coeff0_42*dmats0[42][14] + coeff0_43*dmats0[43][14] + coeff0_44*dmats0[44][14] + coeff0_45*dmats0[45][14] + coeff0_46*dmats0[46][14] + coeff0_47*dmats0[47][14] + coeff0_48*dmats0[48][14] + coeff0_49*dmats0[49][14] + coeff0_50*dmats0[50][14] + coeff0_51*dmats0[51][14] + coeff0_52*dmats0[52][14] + coeff0_53*dmats0[53][14] + coeff0_54*dmats0[54][14] + coeff0_55*dmats0[55][14];
          new_coeff0_15 = coeff0_0*dmats0[0][15] + coeff0_1*dmats0[1][15] + coeff0_2*dmats0[2][15] + coeff0_3*dmats0[3][15] + coeff0_4*dmats0[4][15] + coeff0_5*dmats0[5][15] + coeff0_6*dmats0[6][15] + coeff0_7*dmats0[7][15] + coeff0_8*dmats0[8][15] + coeff0_9*dmats0[9][15] + coeff0_10*dmats0[10][15] + coeff0_11*dmats0[11][15] + coeff0_12*dmats0[12][15] + coeff0_13*dmats0[13][15] + coeff0_14*dmats0[14][15] + coeff0_15*dmats0[15][15] + coeff0_16*dmats0[16][15] + coeff0_17*dmats0[17][15] + coeff0_18*dmats0[18][15] + coeff0_19*dmats0[19][15] + coeff0_20*dmats0[20][15] + coeff0_21*dmats0[21][15] + coeff0_22*dmats0[22][15] + coeff0_23*dmats0[23][15] + coeff0_24*dmats0[24][15] + coeff0_25*dmats0[25][15] + coeff0_26*dmats0[26][15] + coeff0_27*dmats0[27][15] + coeff0_28*dmats0[28][15] + coeff0_29*dmats0[29][15] + coeff0_30*dmats0[30][15] + coeff0_31*dmats0[31][15] + coeff0_32*dmats0[32][15] + coeff0_33*dmats0[33][15] + coeff0_34*dmats0[34][15] + coeff0_35*dmats0[35][15] + coeff0_36*dmats0[36][15] + coeff0_37*dmats0[37][15] + coeff0_38*dmats0[38][15] + coeff0_39*dmats0[39][15] + coeff0_40*dmats0[40][15] + coeff0_41*dmats0[41][15] + coeff0_42*dmats0[42][15] + coeff0_43*dmats0[43][15] + coeff0_44*dmats0[44][15] + coeff0_45*dmats0[45][15] + coeff0_46*dmats0[46][15] + coeff0_47*dmats0[47][15] + coeff0_48*dmats0[48][15] + coeff0_49*dmats0[49][15] + coeff0_50*dmats0[50][15] + coeff0_51*dmats0[51][15] + coeff0_52*dmats0[52][15] + coeff0_53*dmats0[53][15] + coeff0_54*dmats0[54][15] + coeff0_55*dmats0[55][15];
          new_coeff0_16 = coeff0_0*dmats0[0][16] + coeff0_1*dmats0[1][16] + coeff0_2*dmats0[2][16] + coeff0_3*dmats0[3][16] + coeff0_4*dmats0[4][16] + coeff0_5*dmats0[5][16] + coeff0_6*dmats0[6][16] + coeff0_7*dmats0[7][16] + coeff0_8*dmats0[8][16] + coeff0_9*dmats0[9][16] + coeff0_10*dmats0[10][16] + coeff0_11*dmats0[11][16] + coeff0_12*dmats0[12][16] + coeff0_13*dmats0[13][16] + coeff0_14*dmats0[14][16] + coeff0_15*dmats0[15][16] + coeff0_16*dmats0[16][16] + coeff0_17*dmats0[17][16] + coeff0_18*dmats0[18][16] + coeff0_19*dmats0[19][16] + coeff0_20*dmats0[20][16] + coeff0_21*dmats0[21][16] + coeff0_22*dmats0[22][16] + coeff0_23*dmats0[23][16] + coeff0_24*dmats0[24][16] + coeff0_25*dmats0[25][16] + coeff0_26*dmats0[26][16] + coeff0_27*dmats0[27][16] + coeff0_28*dmats0[28][16] + coeff0_29*dmats0[29][16] + coeff0_30*dmats0[30][16] + coeff0_31*dmats0[31][16] + coeff0_32*dmats0[32][16] + coeff0_33*dmats0[33][16] + coeff0_34*dmats0[34][16] + coeff0_35*dmats0[35][16] + coeff0_36*dmats0[36][16] + coeff0_37*dmats0[37][16] + coeff0_38*dmats0[38][16] + coeff0_39*dmats0[39][16] + coeff0_40*dmats0[40][16] + coeff0_41*dmats0[41][16] + coeff0_42*dmats0[42][16] + coeff0_43*dmats0[43][16] + coeff0_44*dmats0[44][16] + coeff0_45*dmats0[45][16] + coeff0_46*dmats0[46][16] + coeff0_47*dmats0[47][16] + coeff0_48*dmats0[48][16] + coeff0_49*dmats0[49][16] + coeff0_50*dmats0[50][16] + coeff0_51*dmats0[51][16] + coeff0_52*dmats0[52][16] + coeff0_53*dmats0[53][16] + coeff0_54*dmats0[54][16] + coeff0_55*dmats0[55][16];
          new_coeff0_17 = coeff0_0*dmats0[0][17] + coeff0_1*dmats0[1][17] + coeff0_2*dmats0[2][17] + coeff0_3*dmats0[3][17] + coeff0_4*dmats0[4][17] + coeff0_5*dmats0[5][17] + coeff0_6*dmats0[6][17] + coeff0_7*dmats0[7][17] + coeff0_8*dmats0[8][17] + coeff0_9*dmats0[9][17] + coeff0_10*dmats0[10][17] + coeff0_11*dmats0[11][17] + coeff0_12*dmats0[12][17] + coeff0_13*dmats0[13][17] + coeff0_14*dmats0[14][17] + coeff0_15*dmats0[15][17] + coeff0_16*dmats0[16][17] + coeff0_17*dmats0[17][17] + coeff0_18*dmats0[18][17] + coeff0_19*dmats0[19][17] + coeff0_20*dmats0[20][17] + coeff0_21*dmats0[21][17] + coeff0_22*dmats0[22][17] + coeff0_23*dmats0[23][17] + coeff0_24*dmats0[24][17] + coeff0_25*dmats0[25][17] + coeff0_26*dmats0[26][17] + coeff0_27*dmats0[27][17] + coeff0_28*dmats0[28][17] + coeff0_29*dmats0[29][17] + coeff0_30*dmats0[30][17] + coeff0_31*dmats0[31][17] + coeff0_32*dmats0[32][17] + coeff0_33*dmats0[33][17] + coeff0_34*dmats0[34][17] + coeff0_35*dmats0[35][17] + coeff0_36*dmats0[36][17] + coeff0_37*dmats0[37][17] + coeff0_38*dmats0[38][17] + coeff0_39*dmats0[39][17] + coeff0_40*dmats0[40][17] + coeff0_41*dmats0[41][17] + coeff0_42*dmats0[42][17] + coeff0_43*dmats0[43][17] + coeff0_44*dmats0[44][17] + coeff0_45*dmats0[45][17] + coeff0_46*dmats0[46][17] + coeff0_47*dmats0[47][17] + coeff0_48*dmats0[48][17] + coeff0_49*dmats0[49][17] + coeff0_50*dmats0[50][17] + coeff0_51*dmats0[51][17] + coeff0_52*dmats0[52][17] + coeff0_53*dmats0[53][17] + coeff0_54*dmats0[54][17] + coeff0_55*dmats0[55][17];
          new_coeff0_18 = coeff0_0*dmats0[0][18] + coeff0_1*dmats0[1][18] + coeff0_2*dmats0[2][18] + coeff0_3*dmats0[3][18] + coeff0_4*dmats0[4][18] + coeff0_5*dmats0[5][18] + coeff0_6*dmats0[6][18] + coeff0_7*dmats0[7][18] + coeff0_8*dmats0[8][18] + coeff0_9*dmats0[9][18] + coeff0_10*dmats0[10][18] + coeff0_11*dmats0[11][18] + coeff0_12*dmats0[12][18] + coeff0_13*dmats0[13][18] + coeff0_14*dmats0[14][18] + coeff0_15*dmats0[15][18] + coeff0_16*dmats0[16][18] + coeff0_17*dmats0[17][18] + coeff0_18*dmats0[18][18] + coeff0_19*dmats0[19][18] + coeff0_20*dmats0[20][18] + coeff0_21*dmats0[21][18] + coeff0_22*dmats0[22][18] + coeff0_23*dmats0[23][18] + coeff0_24*dmats0[24][18] + coeff0_25*dmats0[25][18] + coeff0_26*dmats0[26][18] + coeff0_27*dmats0[27][18] + coeff0_28*dmats0[28][18] + coeff0_29*dmats0[29][18] + coeff0_30*dmats0[30][18] + coeff0_31*dmats0[31][18] + coeff0_32*dmats0[32][18] + coeff0_33*dmats0[33][18] + coeff0_34*dmats0[34][18] + coeff0_35*dmats0[35][18] + coeff0_36*dmats0[36][18] + coeff0_37*dmats0[37][18] + coeff0_38*dmats0[38][18] + coeff0_39*dmats0[39][18] + coeff0_40*dmats0[40][18] + coeff0_41*dmats0[41][18] + coeff0_42*dmats0[42][18] + coeff0_43*dmats0[43][18] + coeff0_44*dmats0[44][18] + coeff0_45*dmats0[45][18] + coeff0_46*dmats0[46][18] + coeff0_47*dmats0[47][18] + coeff0_48*dmats0[48][18] + coeff0_49*dmats0[49][18] + coeff0_50*dmats0[50][18] + coeff0_51*dmats0[51][18] + coeff0_52*dmats0[52][18] + coeff0_53*dmats0[53][18] + coeff0_54*dmats0[54][18] + coeff0_55*dmats0[55][18];
          new_coeff0_19 = coeff0_0*dmats0[0][19] + coeff0_1*dmats0[1][19] + coeff0_2*dmats0[2][19] + coeff0_3*dmats0[3][19] + coeff0_4*dmats0[4][19] + coeff0_5*dmats0[5][19] + coeff0_6*dmats0[6][19] + coeff0_7*dmats0[7][19] + coeff0_8*dmats0[8][19] + coeff0_9*dmats0[9][19] + coeff0_10*dmats0[10][19] + coeff0_11*dmats0[11][19] + coeff0_12*dmats0[12][19] + coeff0_13*dmats0[13][19] + coeff0_14*dmats0[14][19] + coeff0_15*dmats0[15][19] + coeff0_16*dmats0[16][19] + coeff0_17*dmats0[17][19] + coeff0_18*dmats0[18][19] + coeff0_19*dmats0[19][19] + coeff0_20*dmats0[20][19] + coeff0_21*dmats0[21][19] + coeff0_22*dmats0[22][19] + coeff0_23*dmats0[23][19] + coeff0_24*dmats0[24][19] + coeff0_25*dmats0[25][19] + coeff0_26*dmats0[26][19] + coeff0_27*dmats0[27][19] + coeff0_28*dmats0[28][19] + coeff0_29*dmats0[29][19] + coeff0_30*dmats0[30][19] + coeff0_31*dmats0[31][19] + coeff0_32*dmats0[32][19] + coeff0_33*dmats0[33][19] + coeff0_34*dmats0[34][19] + coeff0_35*dmats0[35][19] + coeff0_36*dmats0[36][19] + coeff0_37*dmats0[37][19] + coeff0_38*dmats0[38][19] + coeff0_39*dmats0[39][19] + coeff0_40*dmats0[40][19] + coeff0_41*dmats0[41][19] + coeff0_42*dmats0[42][19] + coeff0_43*dmats0[43][19] + coeff0_44*dmats0[44][19] + coeff0_45*dmats0[45][19] + coeff0_46*dmats0[46][19] + coeff0_47*dmats0[47][19] + coeff0_48*dmats0[48][19] + coeff0_49*dmats0[49][19] + coeff0_50*dmats0[50][19] + coeff0_51*dmats0[51][19] + coeff0_52*dmats0[52][19] + coeff0_53*dmats0[53][19] + coeff0_54*dmats0[54][19] + coeff0_55*dmats0[55][19];
          new_coeff0_20 = coeff0_0*dmats0[0][20] + coeff0_1*dmats0[1][20] + coeff0_2*dmats0[2][20] + coeff0_3*dmats0[3][20] + coeff0_4*dmats0[4][20] + coeff0_5*dmats0[5][20] + coeff0_6*dmats0[6][20] + coeff0_7*dmats0[7][20] + coeff0_8*dmats0[8][20] + coeff0_9*dmats0[9][20] + coeff0_10*dmats0[10][20] + coeff0_11*dmats0[11][20] + coeff0_12*dmats0[12][20] + coeff0_13*dmats0[13][20] + coeff0_14*dmats0[14][20] + coeff0_15*dmats0[15][20] + coeff0_16*dmats0[16][20] + coeff0_17*dmats0[17][20] + coeff0_18*dmats0[18][20] + coeff0_19*dmats0[19][20] + coeff0_20*dmats0[20][20] + coeff0_21*dmats0[21][20] + coeff0_22*dmats0[22][20] + coeff0_23*dmats0[23][20] + coeff0_24*dmats0[24][20] + coeff0_25*dmats0[25][20] + coeff0_26*dmats0[26][20] + coeff0_27*dmats0[27][20] + coeff0_28*dmats0[28][20] + coeff0_29*dmats0[29][20] + coeff0_30*dmats0[30][20] + coeff0_31*dmats0[31][20] + coeff0_32*dmats0[32][20] + coeff0_33*dmats0[33][20] + coeff0_34*dmats0[34][20] + coeff0_35*dmats0[35][20] + coeff0_36*dmats0[36][20] + coeff0_37*dmats0[37][20] + coeff0_38*dmats0[38][20] + coeff0_39*dmats0[39][20] + coeff0_40*dmats0[40][20] + coeff0_41*dmats0[41][20] + coeff0_42*dmats0[42][20] + coeff0_43*dmats0[43][20] + coeff0_44*dmats0[44][20] + coeff0_45*dmats0[45][20] + coeff0_46*dmats0[46][20] + coeff0_47*dmats0[47][20] + coeff0_48*dmats0[48][20] + coeff0_49*dmats0[49][20] + coeff0_50*dmats0[50][20] + coeff0_51*dmats0[51][20] + coeff0_52*dmats0[52][20] + coeff0_53*dmats0[53][20] + coeff0_54*dmats0[54][20] + coeff0_55*dmats0[55][20];
          new_coeff0_21 = coeff0_0*dmats0[0][21] + coeff0_1*dmats0[1][21] + coeff0_2*dmats0[2][21] + coeff0_3*dmats0[3][21] + coeff0_4*dmats0[4][21] + coeff0_5*dmats0[5][21] + coeff0_6*dmats0[6][21] + coeff0_7*dmats0[7][21] + coeff0_8*dmats0[8][21] + coeff0_9*dmats0[9][21] + coeff0_10*dmats0[10][21] + coeff0_11*dmats0[11][21] + coeff0_12*dmats0[12][21] + coeff0_13*dmats0[13][21] + coeff0_14*dmats0[14][21] + coeff0_15*dmats0[15][21] + coeff0_16*dmats0[16][21] + coeff0_17*dmats0[17][21] + coeff0_18*dmats0[18][21] + coeff0_19*dmats0[19][21] + coeff0_20*dmats0[20][21] + coeff0_21*dmats0[21][21] + coeff0_22*dmats0[22][21] + coeff0_23*dmats0[23][21] + coeff0_24*dmats0[24][21] + coeff0_25*dmats0[25][21] + coeff0_26*dmats0[26][21] + coeff0_27*dmats0[27][21] + coeff0_28*dmats0[28][21] + coeff0_29*dmats0[29][21] + coeff0_30*dmats0[30][21] + coeff0_31*dmats0[31][21] + coeff0_32*dmats0[32][21] + coeff0_33*dmats0[33][21] + coeff0_34*dmats0[34][21] + coeff0_35*dmats0[35][21] + coeff0_36*dmats0[36][21] + coeff0_37*dmats0[37][21] + coeff0_38*dmats0[38][21] + coeff0_39*dmats0[39][21] + coeff0_40*dmats0[40][21] + coeff0_41*dmats0[41][21] + coeff0_42*dmats0[42][21] + coeff0_43*dmats0[43][21] + coeff0_44*dmats0[44][21] + coeff0_45*dmats0[45][21] + coeff0_46*dmats0[46][21] + coeff0_47*dmats0[47][21] + coeff0_48*dmats0[48][21] + coeff0_49*dmats0[49][21] + coeff0_50*dmats0[50][21] + coeff0_51*dmats0[51][21] + coeff0_52*dmats0[52][21] + coeff0_53*dmats0[53][21] + coeff0_54*dmats0[54][21] + coeff0_55*dmats0[55][21];
          new_coeff0_22 = coeff0_0*dmats0[0][22] + coeff0_1*dmats0[1][22] + coeff0_2*dmats0[2][22] + coeff0_3*dmats0[3][22] + coeff0_4*dmats0[4][22] + coeff0_5*dmats0[5][22] + coeff0_6*dmats0[6][22] + coeff0_7*dmats0[7][22] + coeff0_8*dmats0[8][22] + coeff0_9*dmats0[9][22] + coeff0_10*dmats0[10][22] + coeff0_11*dmats0[11][22] + coeff0_12*dmats0[12][22] + coeff0_13*dmats0[13][22] + coeff0_14*dmats0[14][22] + coeff0_15*dmats0[15][22] + coeff0_16*dmats0[16][22] + coeff0_17*dmats0[17][22] + coeff0_18*dmats0[18][22] + coeff0_19*dmats0[19][22] + coeff0_20*dmats0[20][22] + coeff0_21*dmats0[21][22] + coeff0_22*dmats0[22][22] + coeff0_23*dmats0[23][22] + coeff0_24*dmats0[24][22] + coeff0_25*dmats0[25][22] + coeff0_26*dmats0[26][22] + coeff0_27*dmats0[27][22] + coeff0_28*dmats0[28][22] + coeff0_29*dmats0[29][22] + coeff0_30*dmats0[30][22] + coeff0_31*dmats0[31][22] + coeff0_32*dmats0[32][22] + coeff0_33*dmats0[33][22] + coeff0_34*dmats0[34][22] + coeff0_35*dmats0[35][22] + coeff0_36*dmats0[36][22] + coeff0_37*dmats0[37][22] + coeff0_38*dmats0[38][22] + coeff0_39*dmats0[39][22] + coeff0_40*dmats0[40][22] + coeff0_41*dmats0[41][22] + coeff0_42*dmats0[42][22] + coeff0_43*dmats0[43][22] + coeff0_44*dmats0[44][22] + coeff0_45*dmats0[45][22] + coeff0_46*dmats0[46][22] + coeff0_47*dmats0[47][22] + coeff0_48*dmats0[48][22] + coeff0_49*dmats0[49][22] + coeff0_50*dmats0[50][22] + coeff0_51*dmats0[51][22] + coeff0_52*dmats0[52][22] + coeff0_53*dmats0[53][22] + coeff0_54*dmats0[54][22] + coeff0_55*dmats0[55][22];
          new_coeff0_23 = coeff0_0*dmats0[0][23] + coeff0_1*dmats0[1][23] + coeff0_2*dmats0[2][23] + coeff0_3*dmats0[3][23] + coeff0_4*dmats0[4][23] + coeff0_5*dmats0[5][23] + coeff0_6*dmats0[6][23] + coeff0_7*dmats0[7][23] + coeff0_8*dmats0[8][23] + coeff0_9*dmats0[9][23] + coeff0_10*dmats0[10][23] + coeff0_11*dmats0[11][23] + coeff0_12*dmats0[12][23] + coeff0_13*dmats0[13][23] + coeff0_14*dmats0[14][23] + coeff0_15*dmats0[15][23] + coeff0_16*dmats0[16][23] + coeff0_17*dmats0[17][23] + coeff0_18*dmats0[18][23] + coeff0_19*dmats0[19][23] + coeff0_20*dmats0[20][23] + coeff0_21*dmats0[21][23] + coeff0_22*dmats0[22][23] + coeff0_23*dmats0[23][23] + coeff0_24*dmats0[24][23] + coeff0_25*dmats0[25][23] + coeff0_26*dmats0[26][23] + coeff0_27*dmats0[27][23] + coeff0_28*dmats0[28][23] + coeff0_29*dmats0[29][23] + coeff0_30*dmats0[30][23] + coeff0_31*dmats0[31][23] + coeff0_32*dmats0[32][23] + coeff0_33*dmats0[33][23] + coeff0_34*dmats0[34][23] + coeff0_35*dmats0[35][23] + coeff0_36*dmats0[36][23] + coeff0_37*dmats0[37][23] + coeff0_38*dmats0[38][23] + coeff0_39*dmats0[39][23] + coeff0_40*dmats0[40][23] + coeff0_41*dmats0[41][23] + coeff0_42*dmats0[42][23] + coeff0_43*dmats0[43][23] + coeff0_44*dmats0[44][23] + coeff0_45*dmats0[45][23] + coeff0_46*dmats0[46][23] + coeff0_47*dmats0[47][23] + coeff0_48*dmats0[48][23] + coeff0_49*dmats0[49][23] + coeff0_50*dmats0[50][23] + coeff0_51*dmats0[51][23] + coeff0_52*dmats0[52][23] + coeff0_53*dmats0[53][23] + coeff0_54*dmats0[54][23] + coeff0_55*dmats0[55][23];
          new_coeff0_24 = coeff0_0*dmats0[0][24] + coeff0_1*dmats0[1][24] + coeff0_2*dmats0[2][24] + coeff0_3*dmats0[3][24] + coeff0_4*dmats0[4][24] + coeff0_5*dmats0[5][24] + coeff0_6*dmats0[6][24] + coeff0_7*dmats0[7][24] + coeff0_8*dmats0[8][24] + coeff0_9*dmats0[9][24] + coeff0_10*dmats0[10][24] + coeff0_11*dmats0[11][24] + coeff0_12*dmats0[12][24] + coeff0_13*dmats0[13][24] + coeff0_14*dmats0[14][24] + coeff0_15*dmats0[15][24] + coeff0_16*dmats0[16][24] + coeff0_17*dmats0[17][24] + coeff0_18*dmats0[18][24] + coeff0_19*dmats0[19][24] + coeff0_20*dmats0[20][24] + coeff0_21*dmats0[21][24] + coeff0_22*dmats0[22][24] + coeff0_23*dmats0[23][24] + coeff0_24*dmats0[24][24] + coeff0_25*dmats0[25][24] + coeff0_26*dmats0[26][24] + coeff0_27*dmats0[27][24] + coeff0_28*dmats0[28][24] + coeff0_29*dmats0[29][24] + coeff0_30*dmats0[30][24] + coeff0_31*dmats0[31][24] + coeff0_32*dmats0[32][24] + coeff0_33*dmats0[33][24] + coeff0_34*dmats0[34][24] + coeff0_35*dmats0[35][24] + coeff0_36*dmats0[36][24] + coeff0_37*dmats0[37][24] + coeff0_38*dmats0[38][24] + coeff0_39*dmats0[39][24] + coeff0_40*dmats0[40][24] + coeff0_41*dmats0[41][24] + coeff0_42*dmats0[42][24] + coeff0_43*dmats0[43][24] + coeff0_44*dmats0[44][24] + coeff0_45*dmats0[45][24] + coeff0_46*dmats0[46][24] + coeff0_47*dmats0[47][24] + coeff0_48*dmats0[48][24] + coeff0_49*dmats0[49][24] + coeff0_50*dmats0[50][24] + coeff0_51*dmats0[51][24] + coeff0_52*dmats0[52][24] + coeff0_53*dmats0[53][24] + coeff0_54*dmats0[54][24] + coeff0_55*dmats0[55][24];
          new_coeff0_25 = coeff0_0*dmats0[0][25] + coeff0_1*dmats0[1][25] + coeff0_2*dmats0[2][25] + coeff0_3*dmats0[3][25] + coeff0_4*dmats0[4][25] + coeff0_5*dmats0[5][25] + coeff0_6*dmats0[6][25] + coeff0_7*dmats0[7][25] + coeff0_8*dmats0[8][25] + coeff0_9*dmats0[9][25] + coeff0_10*dmats0[10][25] + coeff0_11*dmats0[11][25] + coeff0_12*dmats0[12][25] + coeff0_13*dmats0[13][25] + coeff0_14*dmats0[14][25] + coeff0_15*dmats0[15][25] + coeff0_16*dmats0[16][25] + coeff0_17*dmats0[17][25] + coeff0_18*dmats0[18][25] + coeff0_19*dmats0[19][25] + coeff0_20*dmats0[20][25] + coeff0_21*dmats0[21][25] + coeff0_22*dmats0[22][25] + coeff0_23*dmats0[23][25] + coeff0_24*dmats0[24][25] + coeff0_25*dmats0[25][25] + coeff0_26*dmats0[26][25] + coeff0_27*dmats0[27][25] + coeff0_28*dmats0[28][25] + coeff0_29*dmats0[29][25] + coeff0_30*dmats0[30][25] + coeff0_31*dmats0[31][25] + coeff0_32*dmats0[32][25] + coeff0_33*dmats0[33][25] + coeff0_34*dmats0[34][25] + coeff0_35*dmats0[35][25] + coeff0_36*dmats0[36][25] + coeff0_37*dmats0[37][25] + coeff0_38*dmats0[38][25] + coeff0_39*dmats0[39][25] + coeff0_40*dmats0[40][25] + coeff0_41*dmats0[41][25] + coeff0_42*dmats0[42][25] + coeff0_43*dmats0[43][25] + coeff0_44*dmats0[44][25] + coeff0_45*dmats0[45][25] + coeff0_46*dmats0[46][25] + coeff0_47*dmats0[47][25] + coeff0_48*dmats0[48][25] + coeff0_49*dmats0[49][25] + coeff0_50*dmats0[50][25] + coeff0_51*dmats0[51][25] + coeff0_52*dmats0[52][25] + coeff0_53*dmats0[53][25] + coeff0_54*dmats0[54][25] + coeff0_55*dmats0[55][25];
          new_coeff0_26 = coeff0_0*dmats0[0][26] + coeff0_1*dmats0[1][26] + coeff0_2*dmats0[2][26] + coeff0_3*dmats0[3][26] + coeff0_4*dmats0[4][26] + coeff0_5*dmats0[5][26] + coeff0_6*dmats0[6][26] + coeff0_7*dmats0[7][26] + coeff0_8*dmats0[8][26] + coeff0_9*dmats0[9][26] + coeff0_10*dmats0[10][26] + coeff0_11*dmats0[11][26] + coeff0_12*dmats0[12][26] + coeff0_13*dmats0[13][26] + coeff0_14*dmats0[14][26] + coeff0_15*dmats0[15][26] + coeff0_16*dmats0[16][26] + coeff0_17*dmats0[17][26] + coeff0_18*dmats0[18][26] + coeff0_19*dmats0[19][26] + coeff0_20*dmats0[20][26] + coeff0_21*dmats0[21][26] + coeff0_22*dmats0[22][26] + coeff0_23*dmats0[23][26] + coeff0_24*dmats0[24][26] + coeff0_25*dmats0[25][26] + coeff0_26*dmats0[26][26] + coeff0_27*dmats0[27][26] + coeff0_28*dmats0[28][26] + coeff0_29*dmats0[29][26] + coeff0_30*dmats0[30][26] + coeff0_31*dmats0[31][26] + coeff0_32*dmats0[32][26] + coeff0_33*dmats0[33][26] + coeff0_34*dmats0[34][26] + coeff0_35*dmats0[35][26] + coeff0_36*dmats0[36][26] + coeff0_37*dmats0[37][26] + coeff0_38*dmats0[38][26] + coeff0_39*dmats0[39][26] + coeff0_40*dmats0[40][26] + coeff0_41*dmats0[41][26] + coeff0_42*dmats0[42][26] + coeff0_43*dmats0[43][26] + coeff0_44*dmats0[44][26] + coeff0_45*dmats0[45][26] + coeff0_46*dmats0[46][26] + coeff0_47*dmats0[47][26] + coeff0_48*dmats0[48][26] + coeff0_49*dmats0[49][26] + coeff0_50*dmats0[50][26] + coeff0_51*dmats0[51][26] + coeff0_52*dmats0[52][26] + coeff0_53*dmats0[53][26] + coeff0_54*dmats0[54][26] + coeff0_55*dmats0[55][26];
          new_coeff0_27 = coeff0_0*dmats0[0][27] + coeff0_1*dmats0[1][27] + coeff0_2*dmats0[2][27] + coeff0_3*dmats0[3][27] + coeff0_4*dmats0[4][27] + coeff0_5*dmats0[5][27] + coeff0_6*dmats0[6][27] + coeff0_7*dmats0[7][27] + coeff0_8*dmats0[8][27] + coeff0_9*dmats0[9][27] + coeff0_10*dmats0[10][27] + coeff0_11*dmats0[11][27] + coeff0_12*dmats0[12][27] + coeff0_13*dmats0[13][27] + coeff0_14*dmats0[14][27] + coeff0_15*dmats0[15][27] + coeff0_16*dmats0[16][27] + coeff0_17*dmats0[17][27] + coeff0_18*dmats0[18][27] + coeff0_19*dmats0[19][27] + coeff0_20*dmats0[20][27] + coeff0_21*dmats0[21][27] + coeff0_22*dmats0[22][27] + coeff0_23*dmats0[23][27] + coeff0_24*dmats0[24][27] + coeff0_25*dmats0[25][27] + coeff0_26*dmats0[26][27] + coeff0_27*dmats0[27][27] + coeff0_28*dmats0[28][27] + coeff0_29*dmats0[29][27] + coeff0_30*dmats0[30][27] + coeff0_31*dmats0[31][27] + coeff0_32*dmats0[32][27] + coeff0_33*dmats0[33][27] + coeff0_34*dmats0[34][27] + coeff0_35*dmats0[35][27] + coeff0_36*dmats0[36][27] + coeff0_37*dmats0[37][27] + coeff0_38*dmats0[38][27] + coeff0_39*dmats0[39][27] + coeff0_40*dmats0[40][27] + coeff0_41*dmats0[41][27] + coeff0_42*dmats0[42][27] + coeff0_43*dmats0[43][27] + coeff0_44*dmats0[44][27] + coeff0_45*dmats0[45][27] + coeff0_46*dmats0[46][27] + coeff0_47*dmats0[47][27] + coeff0_48*dmats0[48][27] + coeff0_49*dmats0[49][27] + coeff0_50*dmats0[50][27] + coeff0_51*dmats0[51][27] + coeff0_52*dmats0[52][27] + coeff0_53*dmats0[53][27] + coeff0_54*dmats0[54][27] + coeff0_55*dmats0[55][27];
          new_coeff0_28 = coeff0_0*dmats0[0][28] + coeff0_1*dmats0[1][28] + coeff0_2*dmats0[2][28] + coeff0_3*dmats0[3][28] + coeff0_4*dmats0[4][28] + coeff0_5*dmats0[5][28] + coeff0_6*dmats0[6][28] + coeff0_7*dmats0[7][28] + coeff0_8*dmats0[8][28] + coeff0_9*dmats0[9][28] + coeff0_10*dmats0[10][28] + coeff0_11*dmats0[11][28] + coeff0_12*dmats0[12][28] + coeff0_13*dmats0[13][28] + coeff0_14*dmats0[14][28] + coeff0_15*dmats0[15][28] + coeff0_16*dmats0[16][28] + coeff0_17*dmats0[17][28] + coeff0_18*dmats0[18][28] + coeff0_19*dmats0[19][28] + coeff0_20*dmats0[20][28] + coeff0_21*dmats0[21][28] + coeff0_22*dmats0[22][28] + coeff0_23*dmats0[23][28] + coeff0_24*dmats0[24][28] + coeff0_25*dmats0[25][28] + coeff0_26*dmats0[26][28] + coeff0_27*dmats0[27][28] + coeff0_28*dmats0[28][28] + coeff0_29*dmats0[29][28] + coeff0_30*dmats0[30][28] + coeff0_31*dmats0[31][28] + coeff0_32*dmats0[32][28] + coeff0_33*dmats0[33][28] + coeff0_34*dmats0[34][28] + coeff0_35*dmats0[35][28] + coeff0_36*dmats0[36][28] + coeff0_37*dmats0[37][28] + coeff0_38*dmats0[38][28] + coeff0_39*dmats0[39][28] + coeff0_40*dmats0[40][28] + coeff0_41*dmats0[41][28] + coeff0_42*dmats0[42][28] + coeff0_43*dmats0[43][28] + coeff0_44*dmats0[44][28] + coeff0_45*dmats0[45][28] + coeff0_46*dmats0[46][28] + coeff0_47*dmats0[47][28] + coeff0_48*dmats0[48][28] + coeff0_49*dmats0[49][28] + coeff0_50*dmats0[50][28] + coeff0_51*dmats0[51][28] + coeff0_52*dmats0[52][28] + coeff0_53*dmats0[53][28] + coeff0_54*dmats0[54][28] + coeff0_55*dmats0[55][28];
          new_coeff0_29 = coeff0_0*dmats0[0][29] + coeff0_1*dmats0[1][29] + coeff0_2*dmats0[2][29] + coeff0_3*dmats0[3][29] + coeff0_4*dmats0[4][29] + coeff0_5*dmats0[5][29] + coeff0_6*dmats0[6][29] + coeff0_7*dmats0[7][29] + coeff0_8*dmats0[8][29] + coeff0_9*dmats0[9][29] + coeff0_10*dmats0[10][29] + coeff0_11*dmats0[11][29] + coeff0_12*dmats0[12][29] + coeff0_13*dmats0[13][29] + coeff0_14*dmats0[14][29] + coeff0_15*dmats0[15][29] + coeff0_16*dmats0[16][29] + coeff0_17*dmats0[17][29] + coeff0_18*dmats0[18][29] + coeff0_19*dmats0[19][29] + coeff0_20*dmats0[20][29] + coeff0_21*dmats0[21][29] + coeff0_22*dmats0[22][29] + coeff0_23*dmats0[23][29] + coeff0_24*dmats0[24][29] + coeff0_25*dmats0[25][29] + coeff0_26*dmats0[26][29] + coeff0_27*dmats0[27][29] + coeff0_28*dmats0[28][29] + coeff0_29*dmats0[29][29] + coeff0_30*dmats0[30][29] + coeff0_31*dmats0[31][29] + coeff0_32*dmats0[32][29] + coeff0_33*dmats0[33][29] + coeff0_34*dmats0[34][29] + coeff0_35*dmats0[35][29] + coeff0_36*dmats0[36][29] + coeff0_37*dmats0[37][29] + coeff0_38*dmats0[38][29] + coeff0_39*dmats0[39][29] + coeff0_40*dmats0[40][29] + coeff0_41*dmats0[41][29] + coeff0_42*dmats0[42][29] + coeff0_43*dmats0[43][29] + coeff0_44*dmats0[44][29] + coeff0_45*dmats0[45][29] + coeff0_46*dmats0[46][29] + coeff0_47*dmats0[47][29] + coeff0_48*dmats0[48][29] + coeff0_49*dmats0[49][29] + coeff0_50*dmats0[50][29] + coeff0_51*dmats0[51][29] + coeff0_52*dmats0[52][29] + coeff0_53*dmats0[53][29] + coeff0_54*dmats0[54][29] + coeff0_55*dmats0[55][29];
          new_coeff0_30 = coeff0_0*dmats0[0][30] + coeff0_1*dmats0[1][30] + coeff0_2*dmats0[2][30] + coeff0_3*dmats0[3][30] + coeff0_4*dmats0[4][30] + coeff0_5*dmats0[5][30] + coeff0_6*dmats0[6][30] + coeff0_7*dmats0[7][30] + coeff0_8*dmats0[8][30] + coeff0_9*dmats0[9][30] + coeff0_10*dmats0[10][30] + coeff0_11*dmats0[11][30] + coeff0_12*dmats0[12][30] + coeff0_13*dmats0[13][30] + coeff0_14*dmats0[14][30] + coeff0_15*dmats0[15][30] + coeff0_16*dmats0[16][30] + coeff0_17*dmats0[17][30] + coeff0_18*dmats0[18][30] + coeff0_19*dmats0[19][30] + coeff0_20*dmats0[20][30] + coeff0_21*dmats0[21][30] + coeff0_22*dmats0[22][30] + coeff0_23*dmats0[23][30] + coeff0_24*dmats0[24][30] + coeff0_25*dmats0[25][30] + coeff0_26*dmats0[26][30] + coeff0_27*dmats0[27][30] + coeff0_28*dmats0[28][30] + coeff0_29*dmats0[29][30] + coeff0_30*dmats0[30][30] + coeff0_31*dmats0[31][30] + coeff0_32*dmats0[32][30] + coeff0_33*dmats0[33][30] + coeff0_34*dmats0[34][30] + coeff0_35*dmats0[35][30] + coeff0_36*dmats0[36][30] + coeff0_37*dmats0[37][30] + coeff0_38*dmats0[38][30] + coeff0_39*dmats0[39][30] + coeff0_40*dmats0[40][30] + coeff0_41*dmats0[41][30] + coeff0_42*dmats0[42][30] + coeff0_43*dmats0[43][30] + coeff0_44*dmats0[44][30] + coeff0_45*dmats0[45][30] + coeff0_46*dmats0[46][30] + coeff0_47*dmats0[47][30] + coeff0_48*dmats0[48][30] + coeff0_49*dmats0[49][30] + coeff0_50*dmats0[50][30] + coeff0_51*dmats0[51][30] + coeff0_52*dmats0[52][30] + coeff0_53*dmats0[53][30] + coeff0_54*dmats0[54][30] + coeff0_55*dmats0[55][30];
          new_coeff0_31 = coeff0_0*dmats0[0][31] + coeff0_1*dmats0[1][31] + coeff0_2*dmats0[2][31] + coeff0_3*dmats0[3][31] + coeff0_4*dmats0[4][31] + coeff0_5*dmats0[5][31] + coeff0_6*dmats0[6][31] + coeff0_7*dmats0[7][31] + coeff0_8*dmats0[8][31] + coeff0_9*dmats0[9][31] + coeff0_10*dmats0[10][31] + coeff0_11*dmats0[11][31] + coeff0_12*dmats0[12][31] + coeff0_13*dmats0[13][31] + coeff0_14*dmats0[14][31] + coeff0_15*dmats0[15][31] + coeff0_16*dmats0[16][31] + coeff0_17*dmats0[17][31] + coeff0_18*dmats0[18][31] + coeff0_19*dmats0[19][31] + coeff0_20*dmats0[20][31] + coeff0_21*dmats0[21][31] + coeff0_22*dmats0[22][31] + coeff0_23*dmats0[23][31] + coeff0_24*dmats0[24][31] + coeff0_25*dmats0[25][31] + coeff0_26*dmats0[26][31] + coeff0_27*dmats0[27][31] + coeff0_28*dmats0[28][31] + coeff0_29*dmats0[29][31] + coeff0_30*dmats0[30][31] + coeff0_31*dmats0[31][31] + coeff0_32*dmats0[32][31] + coeff0_33*dmats0[33][31] + coeff0_34*dmats0[34][31] + coeff0_35*dmats0[35][31] + coeff0_36*dmats0[36][31] + coeff0_37*dmats0[37][31] + coeff0_38*dmats0[38][31] + coeff0_39*dmats0[39][31] + coeff0_40*dmats0[40][31] + coeff0_41*dmats0[41][31] + coeff0_42*dmats0[42][31] + coeff0_43*dmats0[43][31] + coeff0_44*dmats0[44][31] + coeff0_45*dmats0[45][31] + coeff0_46*dmats0[46][31] + coeff0_47*dmats0[47][31] + coeff0_48*dmats0[48][31] + coeff0_49*dmats0[49][31] + coeff0_50*dmats0[50][31] + coeff0_51*dmats0[51][31] + coeff0_52*dmats0[52][31] + coeff0_53*dmats0[53][31] + coeff0_54*dmats0[54][31] + coeff0_55*dmats0[55][31];
          new_coeff0_32 = coeff0_0*dmats0[0][32] + coeff0_1*dmats0[1][32] + coeff0_2*dmats0[2][32] + coeff0_3*dmats0[3][32] + coeff0_4*dmats0[4][32] + coeff0_5*dmats0[5][32] + coeff0_6*dmats0[6][32] + coeff0_7*dmats0[7][32] + coeff0_8*dmats0[8][32] + coeff0_9*dmats0[9][32] + coeff0_10*dmats0[10][32] + coeff0_11*dmats0[11][32] + coeff0_12*dmats0[12][32] + coeff0_13*dmats0[13][32] + coeff0_14*dmats0[14][32] + coeff0_15*dmats0[15][32] + coeff0_16*dmats0[16][32] + coeff0_17*dmats0[17][32] + coeff0_18*dmats0[18][32] + coeff0_19*dmats0[19][32] + coeff0_20*dmats0[20][32] + coeff0_21*dmats0[21][32] + coeff0_22*dmats0[22][32] + coeff0_23*dmats0[23][32] + coeff0_24*dmats0[24][32] + coeff0_25*dmats0[25][32] + coeff0_26*dmats0[26][32] + coeff0_27*dmats0[27][32] + coeff0_28*dmats0[28][32] + coeff0_29*dmats0[29][32] + coeff0_30*dmats0[30][32] + coeff0_31*dmats0[31][32] + coeff0_32*dmats0[32][32] + coeff0_33*dmats0[33][32] + coeff0_34*dmats0[34][32] + coeff0_35*dmats0[35][32] + coeff0_36*dmats0[36][32] + coeff0_37*dmats0[37][32] + coeff0_38*dmats0[38][32] + coeff0_39*dmats0[39][32] + coeff0_40*dmats0[40][32] + coeff0_41*dmats0[41][32] + coeff0_42*dmats0[42][32] + coeff0_43*dmats0[43][32] + coeff0_44*dmats0[44][32] + coeff0_45*dmats0[45][32] + coeff0_46*dmats0[46][32] + coeff0_47*dmats0[47][32] + coeff0_48*dmats0[48][32] + coeff0_49*dmats0[49][32] + coeff0_50*dmats0[50][32] + coeff0_51*dmats0[51][32] + coeff0_52*dmats0[52][32] + coeff0_53*dmats0[53][32] + coeff0_54*dmats0[54][32] + coeff0_55*dmats0[55][32];
          new_coeff0_33 = coeff0_0*dmats0[0][33] + coeff0_1*dmats0[1][33] + coeff0_2*dmats0[2][33] + coeff0_3*dmats0[3][33] + coeff0_4*dmats0[4][33] + coeff0_5*dmats0[5][33] + coeff0_6*dmats0[6][33] + coeff0_7*dmats0[7][33] + coeff0_8*dmats0[8][33] + coeff0_9*dmats0[9][33] + coeff0_10*dmats0[10][33] + coeff0_11*dmats0[11][33] + coeff0_12*dmats0[12][33] + coeff0_13*dmats0[13][33] + coeff0_14*dmats0[14][33] + coeff0_15*dmats0[15][33] + coeff0_16*dmats0[16][33] + coeff0_17*dmats0[17][33] + coeff0_18*dmats0[18][33] + coeff0_19*dmats0[19][33] + coeff0_20*dmats0[20][33] + coeff0_21*dmats0[21][33] + coeff0_22*dmats0[22][33] + coeff0_23*dmats0[23][33] + coeff0_24*dmats0[24][33] + coeff0_25*dmats0[25][33] + coeff0_26*dmats0[26][33] + coeff0_27*dmats0[27][33] + coeff0_28*dmats0[28][33] + coeff0_29*dmats0[29][33] + coeff0_30*dmats0[30][33] + coeff0_31*dmats0[31][33] + coeff0_32*dmats0[32][33] + coeff0_33*dmats0[33][33] + coeff0_34*dmats0[34][33] + coeff0_35*dmats0[35][33] + coeff0_36*dmats0[36][33] + coeff0_37*dmats0[37][33] + coeff0_38*dmats0[38][33] + coeff0_39*dmats0[39][33] + coeff0_40*dmats0[40][33] + coeff0_41*dmats0[41][33] + coeff0_42*dmats0[42][33] + coeff0_43*dmats0[43][33] + coeff0_44*dmats0[44][33] + coeff0_45*dmats0[45][33] + coeff0_46*dmats0[46][33] + coeff0_47*dmats0[47][33] + coeff0_48*dmats0[48][33] + coeff0_49*dmats0[49][33] + coeff0_50*dmats0[50][33] + coeff0_51*dmats0[51][33] + coeff0_52*dmats0[52][33] + coeff0_53*dmats0[53][33] + coeff0_54*dmats0[54][33] + coeff0_55*dmats0[55][33];
          new_coeff0_34 = coeff0_0*dmats0[0][34] + coeff0_1*dmats0[1][34] + coeff0_2*dmats0[2][34] + coeff0_3*dmats0[3][34] + coeff0_4*dmats0[4][34] + coeff0_5*dmats0[5][34] + coeff0_6*dmats0[6][34] + coeff0_7*dmats0[7][34] + coeff0_8*dmats0[8][34] + coeff0_9*dmats0[9][34] + coeff0_10*dmats0[10][34] + coeff0_11*dmats0[11][34] + coeff0_12*dmats0[12][34] + coeff0_13*dmats0[13][34] + coeff0_14*dmats0[14][34] + coeff0_15*dmats0[15][34] + coeff0_16*dmats0[16][34] + coeff0_17*dmats0[17][34] + coeff0_18*dmats0[18][34] + coeff0_19*dmats0[19][34] + coeff0_20*dmats0[20][34] + coeff0_21*dmats0[21][34] + coeff0_22*dmats0[22][34] + coeff0_23*dmats0[23][34] + coeff0_24*dmats0[24][34] + coeff0_25*dmats0[25][34] + coeff0_26*dmats0[26][34] + coeff0_27*dmats0[27][34] + coeff0_28*dmats0[28][34] + coeff0_29*dmats0[29][34] + coeff0_30*dmats0[30][34] + coeff0_31*dmats0[31][34] + coeff0_32*dmats0[32][34] + coeff0_33*dmats0[33][34] + coeff0_34*dmats0[34][34] + coeff0_35*dmats0[35][34] + coeff0_36*dmats0[36][34] + coeff0_37*dmats0[37][34] + coeff0_38*dmats0[38][34] + coeff0_39*dmats0[39][34] + coeff0_40*dmats0[40][34] + coeff0_41*dmats0[41][34] + coeff0_42*dmats0[42][34] + coeff0_43*dmats0[43][34] + coeff0_44*dmats0[44][34] + coeff0_45*dmats0[45][34] + coeff0_46*dmats0[46][34] + coeff0_47*dmats0[47][34] + coeff0_48*dmats0[48][34] + coeff0_49*dmats0[49][34] + coeff0_50*dmats0[50][34] + coeff0_51*dmats0[51][34] + coeff0_52*dmats0[52][34] + coeff0_53*dmats0[53][34] + coeff0_54*dmats0[54][34] + coeff0_55*dmats0[55][34];
          new_coeff0_35 = coeff0_0*dmats0[0][35] + coeff0_1*dmats0[1][35] + coeff0_2*dmats0[2][35] + coeff0_3*dmats0[3][35] + coeff0_4*dmats0[4][35] + coeff0_5*dmats0[5][35] + coeff0_6*dmats0[6][35] + coeff0_7*dmats0[7][35] + coeff0_8*dmats0[8][35] + coeff0_9*dmats0[9][35] + coeff0_10*dmats0[10][35] + coeff0_11*dmats0[11][35] + coeff0_12*dmats0[12][35] + coeff0_13*dmats0[13][35] + coeff0_14*dmats0[14][35] + coeff0_15*dmats0[15][35] + coeff0_16*dmats0[16][35] + coeff0_17*dmats0[17][35] + coeff0_18*dmats0[18][35] + coeff0_19*dmats0[19][35] + coeff0_20*dmats0[20][35] + coeff0_21*dmats0[21][35] + coeff0_22*dmats0[22][35] + coeff0_23*dmats0[23][35] + coeff0_24*dmats0[24][35] + coeff0_25*dmats0[25][35] + coeff0_26*dmats0[26][35] + coeff0_27*dmats0[27][35] + coeff0_28*dmats0[28][35] + coeff0_29*dmats0[29][35] + coeff0_30*dmats0[30][35] + coeff0_31*dmats0[31][35] + coeff0_32*dmats0[32][35] + coeff0_33*dmats0[33][35] + coeff0_34*dmats0[34][35] + coeff0_35*dmats0[35][35] + coeff0_36*dmats0[36][35] + coeff0_37*dmats0[37][35] + coeff0_38*dmats0[38][35] + coeff0_39*dmats0[39][35] + coeff0_40*dmats0[40][35] + coeff0_41*dmats0[41][35] + coeff0_42*dmats0[42][35] + coeff0_43*dmats0[43][35] + coeff0_44*dmats0[44][35] + coeff0_45*dmats0[45][35] + coeff0_46*dmats0[46][35] + coeff0_47*dmats0[47][35] + coeff0_48*dmats0[48][35] + coeff0_49*dmats0[49][35] + coeff0_50*dmats0[50][35] + coeff0_51*dmats0[51][35] + coeff0_52*dmats0[52][35] + coeff0_53*dmats0[53][35] + coeff0_54*dmats0[54][35] + coeff0_55*dmats0[55][35];
          new_coeff0_36 = coeff0_0*dmats0[0][36] + coeff0_1*dmats0[1][36] + coeff0_2*dmats0[2][36] + coeff0_3*dmats0[3][36] + coeff0_4*dmats0[4][36] + coeff0_5*dmats0[5][36] + coeff0_6*dmats0[6][36] + coeff0_7*dmats0[7][36] + coeff0_8*dmats0[8][36] + coeff0_9*dmats0[9][36] + coeff0_10*dmats0[10][36] + coeff0_11*dmats0[11][36] + coeff0_12*dmats0[12][36] + coeff0_13*dmats0[13][36] + coeff0_14*dmats0[14][36] + coeff0_15*dmats0[15][36] + coeff0_16*dmats0[16][36] + coeff0_17*dmats0[17][36] + coeff0_18*dmats0[18][36] + coeff0_19*dmats0[19][36] + coeff0_20*dmats0[20][36] + coeff0_21*dmats0[21][36] + coeff0_22*dmats0[22][36] + coeff0_23*dmats0[23][36] + coeff0_24*dmats0[24][36] + coeff0_25*dmats0[25][36] + coeff0_26*dmats0[26][36] + coeff0_27*dmats0[27][36] + coeff0_28*dmats0[28][36] + coeff0_29*dmats0[29][36] + coeff0_30*dmats0[30][36] + coeff0_31*dmats0[31][36] + coeff0_32*dmats0[32][36] + coeff0_33*dmats0[33][36] + coeff0_34*dmats0[34][36] + coeff0_35*dmats0[35][36] + coeff0_36*dmats0[36][36] + coeff0_37*dmats0[37][36] + coeff0_38*dmats0[38][36] + coeff0_39*dmats0[39][36] + coeff0_40*dmats0[40][36] + coeff0_41*dmats0[41][36] + coeff0_42*dmats0[42][36] + coeff0_43*dmats0[43][36] + coeff0_44*dmats0[44][36] + coeff0_45*dmats0[45][36] + coeff0_46*dmats0[46][36] + coeff0_47*dmats0[47][36] + coeff0_48*dmats0[48][36] + coeff0_49*dmats0[49][36] + coeff0_50*dmats0[50][36] + coeff0_51*dmats0[51][36] + coeff0_52*dmats0[52][36] + coeff0_53*dmats0[53][36] + coeff0_54*dmats0[54][36] + coeff0_55*dmats0[55][36];
          new_coeff0_37 = coeff0_0*dmats0[0][37] + coeff0_1*dmats0[1][37] + coeff0_2*dmats0[2][37] + coeff0_3*dmats0[3][37] + coeff0_4*dmats0[4][37] + coeff0_5*dmats0[5][37] + coeff0_6*dmats0[6][37] + coeff0_7*dmats0[7][37] + coeff0_8*dmats0[8][37] + coeff0_9*dmats0[9][37] + coeff0_10*dmats0[10][37] + coeff0_11*dmats0[11][37] + coeff0_12*dmats0[12][37] + coeff0_13*dmats0[13][37] + coeff0_14*dmats0[14][37] + coeff0_15*dmats0[15][37] + coeff0_16*dmats0[16][37] + coeff0_17*dmats0[17][37] + coeff0_18*dmats0[18][37] + coeff0_19*dmats0[19][37] + coeff0_20*dmats0[20][37] + coeff0_21*dmats0[21][37] + coeff0_22*dmats0[22][37] + coeff0_23*dmats0[23][37] + coeff0_24*dmats0[24][37] + coeff0_25*dmats0[25][37] + coeff0_26*dmats0[26][37] + coeff0_27*dmats0[27][37] + coeff0_28*dmats0[28][37] + coeff0_29*dmats0[29][37] + coeff0_30*dmats0[30][37] + coeff0_31*dmats0[31][37] + coeff0_32*dmats0[32][37] + coeff0_33*dmats0[33][37] + coeff0_34*dmats0[34][37] + coeff0_35*dmats0[35][37] + coeff0_36*dmats0[36][37] + coeff0_37*dmats0[37][37] + coeff0_38*dmats0[38][37] + coeff0_39*dmats0[39][37] + coeff0_40*dmats0[40][37] + coeff0_41*dmats0[41][37] + coeff0_42*dmats0[42][37] + coeff0_43*dmats0[43][37] + coeff0_44*dmats0[44][37] + coeff0_45*dmats0[45][37] + coeff0_46*dmats0[46][37] + coeff0_47*dmats0[47][37] + coeff0_48*dmats0[48][37] + coeff0_49*dmats0[49][37] + coeff0_50*dmats0[50][37] + coeff0_51*dmats0[51][37] + coeff0_52*dmats0[52][37] + coeff0_53*dmats0[53][37] + coeff0_54*dmats0[54][37] + coeff0_55*dmats0[55][37];
          new_coeff0_38 = coeff0_0*dmats0[0][38] + coeff0_1*dmats0[1][38] + coeff0_2*dmats0[2][38] + coeff0_3*dmats0[3][38] + coeff0_4*dmats0[4][38] + coeff0_5*dmats0[5][38] + coeff0_6*dmats0[6][38] + coeff0_7*dmats0[7][38] + coeff0_8*dmats0[8][38] + coeff0_9*dmats0[9][38] + coeff0_10*dmats0[10][38] + coeff0_11*dmats0[11][38] + coeff0_12*dmats0[12][38] + coeff0_13*dmats0[13][38] + coeff0_14*dmats0[14][38] + coeff0_15*dmats0[15][38] + coeff0_16*dmats0[16][38] + coeff0_17*dmats0[17][38] + coeff0_18*dmats0[18][38] + coeff0_19*dmats0[19][38] + coeff0_20*dmats0[20][38] + coeff0_21*dmats0[21][38] + coeff0_22*dmats0[22][38] + coeff0_23*dmats0[23][38] + coeff0_24*dmats0[24][38] + coeff0_25*dmats0[25][38] + coeff0_26*dmats0[26][38] + coeff0_27*dmats0[27][38] + coeff0_28*dmats0[28][38] + coeff0_29*dmats0[29][38] + coeff0_30*dmats0[30][38] + coeff0_31*dmats0[31][38] + coeff0_32*dmats0[32][38] + coeff0_33*dmats0[33][38] + coeff0_34*dmats0[34][38] + coeff0_35*dmats0[35][38] + coeff0_36*dmats0[36][38] + coeff0_37*dmats0[37][38] + coeff0_38*dmats0[38][38] + coeff0_39*dmats0[39][38] + coeff0_40*dmats0[40][38] + coeff0_41*dmats0[41][38] + coeff0_42*dmats0[42][38] + coeff0_43*dmats0[43][38] + coeff0_44*dmats0[44][38] + coeff0_45*dmats0[45][38] + coeff0_46*dmats0[46][38] + coeff0_47*dmats0[47][38] + coeff0_48*dmats0[48][38] + coeff0_49*dmats0[49][38] + coeff0_50*dmats0[50][38] + coeff0_51*dmats0[51][38] + coeff0_52*dmats0[52][38] + coeff0_53*dmats0[53][38] + coeff0_54*dmats0[54][38] + coeff0_55*dmats0[55][38];
          new_coeff0_39 = coeff0_0*dmats0[0][39] + coeff0_1*dmats0[1][39] + coeff0_2*dmats0[2][39] + coeff0_3*dmats0[3][39] + coeff0_4*dmats0[4][39] + coeff0_5*dmats0[5][39] + coeff0_6*dmats0[6][39] + coeff0_7*dmats0[7][39] + coeff0_8*dmats0[8][39] + coeff0_9*dmats0[9][39] + coeff0_10*dmats0[10][39] + coeff0_11*dmats0[11][39] + coeff0_12*dmats0[12][39] + coeff0_13*dmats0[13][39] + coeff0_14*dmats0[14][39] + coeff0_15*dmats0[15][39] + coeff0_16*dmats0[16][39] + coeff0_17*dmats0[17][39] + coeff0_18*dmats0[18][39] + coeff0_19*dmats0[19][39] + coeff0_20*dmats0[20][39] + coeff0_21*dmats0[21][39] + coeff0_22*dmats0[22][39] + coeff0_23*dmats0[23][39] + coeff0_24*dmats0[24][39] + coeff0_25*dmats0[25][39] + coeff0_26*dmats0[26][39] + coeff0_27*dmats0[27][39] + coeff0_28*dmats0[28][39] + coeff0_29*dmats0[29][39] + coeff0_30*dmats0[30][39] + coeff0_31*dmats0[31][39] + coeff0_32*dmats0[32][39] + coeff0_33*dmats0[33][39] + coeff0_34*dmats0[34][39] + coeff0_35*dmats0[35][39] + coeff0_36*dmats0[36][39] + coeff0_37*dmats0[37][39] + coeff0_38*dmats0[38][39] + coeff0_39*dmats0[39][39] + coeff0_40*dmats0[40][39] + coeff0_41*dmats0[41][39] + coeff0_42*dmats0[42][39] + coeff0_43*dmats0[43][39] + coeff0_44*dmats0[44][39] + coeff0_45*dmats0[45][39] + coeff0_46*dmats0[46][39] + coeff0_47*dmats0[47][39] + coeff0_48*dmats0[48][39] + coeff0_49*dmats0[49][39] + coeff0_50*dmats0[50][39] + coeff0_51*dmats0[51][39] + coeff0_52*dmats0[52][39] + coeff0_53*dmats0[53][39] + coeff0_54*dmats0[54][39] + coeff0_55*dmats0[55][39];
          new_coeff0_40 = coeff0_0*dmats0[0][40] + coeff0_1*dmats0[1][40] + coeff0_2*dmats0[2][40] + coeff0_3*dmats0[3][40] + coeff0_4*dmats0[4][40] + coeff0_5*dmats0[5][40] + coeff0_6*dmats0[6][40] + coeff0_7*dmats0[7][40] + coeff0_8*dmats0[8][40] + coeff0_9*dmats0[9][40] + coeff0_10*dmats0[10][40] + coeff0_11*dmats0[11][40] + coeff0_12*dmats0[12][40] + coeff0_13*dmats0[13][40] + coeff0_14*dmats0[14][40] + coeff0_15*dmats0[15][40] + coeff0_16*dmats0[16][40] + coeff0_17*dmats0[17][40] + coeff0_18*dmats0[18][40] + coeff0_19*dmats0[19][40] + coeff0_20*dmats0[20][40] + coeff0_21*dmats0[21][40] + coeff0_22*dmats0[22][40] + coeff0_23*dmats0[23][40] + coeff0_24*dmats0[24][40] + coeff0_25*dmats0[25][40] + coeff0_26*dmats0[26][40] + coeff0_27*dmats0[27][40] + coeff0_28*dmats0[28][40] + coeff0_29*dmats0[29][40] + coeff0_30*dmats0[30][40] + coeff0_31*dmats0[31][40] + coeff0_32*dmats0[32][40] + coeff0_33*dmats0[33][40] + coeff0_34*dmats0[34][40] + coeff0_35*dmats0[35][40] + coeff0_36*dmats0[36][40] + coeff0_37*dmats0[37][40] + coeff0_38*dmats0[38][40] + coeff0_39*dmats0[39][40] + coeff0_40*dmats0[40][40] + coeff0_41*dmats0[41][40] + coeff0_42*dmats0[42][40] + coeff0_43*dmats0[43][40] + coeff0_44*dmats0[44][40] + coeff0_45*dmats0[45][40] + coeff0_46*dmats0[46][40] + coeff0_47*dmats0[47][40] + coeff0_48*dmats0[48][40] + coeff0_49*dmats0[49][40] + coeff0_50*dmats0[50][40] + coeff0_51*dmats0[51][40] + coeff0_52*dmats0[52][40] + coeff0_53*dmats0[53][40] + coeff0_54*dmats0[54][40] + coeff0_55*dmats0[55][40];
          new_coeff0_41 = coeff0_0*dmats0[0][41] + coeff0_1*dmats0[1][41] + coeff0_2*dmats0[2][41] + coeff0_3*dmats0[3][41] + coeff0_4*dmats0[4][41] + coeff0_5*dmats0[5][41] + coeff0_6*dmats0[6][41] + coeff0_7*dmats0[7][41] + coeff0_8*dmats0[8][41] + coeff0_9*dmats0[9][41] + coeff0_10*dmats0[10][41] + coeff0_11*dmats0[11][41] + coeff0_12*dmats0[12][41] + coeff0_13*dmats0[13][41] + coeff0_14*dmats0[14][41] + coeff0_15*dmats0[15][41] + coeff0_16*dmats0[16][41] + coeff0_17*dmats0[17][41] + coeff0_18*dmats0[18][41] + coeff0_19*dmats0[19][41] + coeff0_20*dmats0[20][41] + coeff0_21*dmats0[21][41] + coeff0_22*dmats0[22][41] + coeff0_23*dmats0[23][41] + coeff0_24*dmats0[24][41] + coeff0_25*dmats0[25][41] + coeff0_26*dmats0[26][41] + coeff0_27*dmats0[27][41] + coeff0_28*dmats0[28][41] + coeff0_29*dmats0[29][41] + coeff0_30*dmats0[30][41] + coeff0_31*dmats0[31][41] + coeff0_32*dmats0[32][41] + coeff0_33*dmats0[33][41] + coeff0_34*dmats0[34][41] + coeff0_35*dmats0[35][41] + coeff0_36*dmats0[36][41] + coeff0_37*dmats0[37][41] + coeff0_38*dmats0[38][41] + coeff0_39*dmats0[39][41] + coeff0_40*dmats0[40][41] + coeff0_41*dmats0[41][41] + coeff0_42*dmats0[42][41] + coeff0_43*dmats0[43][41] + coeff0_44*dmats0[44][41] + coeff0_45*dmats0[45][41] + coeff0_46*dmats0[46][41] + coeff0_47*dmats0[47][41] + coeff0_48*dmats0[48][41] + coeff0_49*dmats0[49][41] + coeff0_50*dmats0[50][41] + coeff0_51*dmats0[51][41] + coeff0_52*dmats0[52][41] + coeff0_53*dmats0[53][41] + coeff0_54*dmats0[54][41] + coeff0_55*dmats0[55][41];
          new_coeff0_42 = coeff0_0*dmats0[0][42] + coeff0_1*dmats0[1][42] + coeff0_2*dmats0[2][42] + coeff0_3*dmats0[3][42] + coeff0_4*dmats0[4][42] + coeff0_5*dmats0[5][42] + coeff0_6*dmats0[6][42] + coeff0_7*dmats0[7][42] + coeff0_8*dmats0[8][42] + coeff0_9*dmats0[9][42] + coeff0_10*dmats0[10][42] + coeff0_11*dmats0[11][42] + coeff0_12*dmats0[12][42] + coeff0_13*dmats0[13][42] + coeff0_14*dmats0[14][42] + coeff0_15*dmats0[15][42] + coeff0_16*dmats0[16][42] + coeff0_17*dmats0[17][42] + coeff0_18*dmats0[18][42] + coeff0_19*dmats0[19][42] + coeff0_20*dmats0[20][42] + coeff0_21*dmats0[21][42] + coeff0_22*dmats0[22][42] + coeff0_23*dmats0[23][42] + coeff0_24*dmats0[24][42] + coeff0_25*dmats0[25][42] + coeff0_26*dmats0[26][42] + coeff0_27*dmats0[27][42] + coeff0_28*dmats0[28][42] + coeff0_29*dmats0[29][42] + coeff0_30*dmats0[30][42] + coeff0_31*dmats0[31][42] + coeff0_32*dmats0[32][42] + coeff0_33*dmats0[33][42] + coeff0_34*dmats0[34][42] + coeff0_35*dmats0[35][42] + coeff0_36*dmats0[36][42] + coeff0_37*dmats0[37][42] + coeff0_38*dmats0[38][42] + coeff0_39*dmats0[39][42] + coeff0_40*dmats0[40][42] + coeff0_41*dmats0[41][42] + coeff0_42*dmats0[42][42] + coeff0_43*dmats0[43][42] + coeff0_44*dmats0[44][42] + coeff0_45*dmats0[45][42] + coeff0_46*dmats0[46][42] + coeff0_47*dmats0[47][42] + coeff0_48*dmats0[48][42] + coeff0_49*dmats0[49][42] + coeff0_50*dmats0[50][42] + coeff0_51*dmats0[51][42] + coeff0_52*dmats0[52][42] + coeff0_53*dmats0[53][42] + coeff0_54*dmats0[54][42] + coeff0_55*dmats0[55][42];
          new_coeff0_43 = coeff0_0*dmats0[0][43] + coeff0_1*dmats0[1][43] + coeff0_2*dmats0[2][43] + coeff0_3*dmats0[3][43] + coeff0_4*dmats0[4][43] + coeff0_5*dmats0[5][43] + coeff0_6*dmats0[6][43] + coeff0_7*dmats0[7][43] + coeff0_8*dmats0[8][43] + coeff0_9*dmats0[9][43] + coeff0_10*dmats0[10][43] + coeff0_11*dmats0[11][43] + coeff0_12*dmats0[12][43] + coeff0_13*dmats0[13][43] + coeff0_14*dmats0[14][43] + coeff0_15*dmats0[15][43] + coeff0_16*dmats0[16][43] + coeff0_17*dmats0[17][43] + coeff0_18*dmats0[18][43] + coeff0_19*dmats0[19][43] + coeff0_20*dmats0[20][43] + coeff0_21*dmats0[21][43] + coeff0_22*dmats0[22][43] + coeff0_23*dmats0[23][43] + coeff0_24*dmats0[24][43] + coeff0_25*dmats0[25][43] + coeff0_26*dmats0[26][43] + coeff0_27*dmats0[27][43] + coeff0_28*dmats0[28][43] + coeff0_29*dmats0[29][43] + coeff0_30*dmats0[30][43] + coeff0_31*dmats0[31][43] + coeff0_32*dmats0[32][43] + coeff0_33*dmats0[33][43] + coeff0_34*dmats0[34][43] + coeff0_35*dmats0[35][43] + coeff0_36*dmats0[36][43] + coeff0_37*dmats0[37][43] + coeff0_38*dmats0[38][43] + coeff0_39*dmats0[39][43] + coeff0_40*dmats0[40][43] + coeff0_41*dmats0[41][43] + coeff0_42*dmats0[42][43] + coeff0_43*dmats0[43][43] + coeff0_44*dmats0[44][43] + coeff0_45*dmats0[45][43] + coeff0_46*dmats0[46][43] + coeff0_47*dmats0[47][43] + coeff0_48*dmats0[48][43] + coeff0_49*dmats0[49][43] + coeff0_50*dmats0[50][43] + coeff0_51*dmats0[51][43] + coeff0_52*dmats0[52][43] + coeff0_53*dmats0[53][43] + coeff0_54*dmats0[54][43] + coeff0_55*dmats0[55][43];
          new_coeff0_44 = coeff0_0*dmats0[0][44] + coeff0_1*dmats0[1][44] + coeff0_2*dmats0[2][44] + coeff0_3*dmats0[3][44] + coeff0_4*dmats0[4][44] + coeff0_5*dmats0[5][44] + coeff0_6*dmats0[6][44] + coeff0_7*dmats0[7][44] + coeff0_8*dmats0[8][44] + coeff0_9*dmats0[9][44] + coeff0_10*dmats0[10][44] + coeff0_11*dmats0[11][44] + coeff0_12*dmats0[12][44] + coeff0_13*dmats0[13][44] + coeff0_14*dmats0[14][44] + coeff0_15*dmats0[15][44] + coeff0_16*dmats0[16][44] + coeff0_17*dmats0[17][44] + coeff0_18*dmats0[18][44] + coeff0_19*dmats0[19][44] + coeff0_20*dmats0[20][44] + coeff0_21*dmats0[21][44] + coeff0_22*dmats0[22][44] + coeff0_23*dmats0[23][44] + coeff0_24*dmats0[24][44] + coeff0_25*dmats0[25][44] + coeff0_26*dmats0[26][44] + coeff0_27*dmats0[27][44] + coeff0_28*dmats0[28][44] + coeff0_29*dmats0[29][44] + coeff0_30*dmats0[30][44] + coeff0_31*dmats0[31][44] + coeff0_32*dmats0[32][44] + coeff0_33*dmats0[33][44] + coeff0_34*dmats0[34][44] + coeff0_35*dmats0[35][44] + coeff0_36*dmats0[36][44] + coeff0_37*dmats0[37][44] + coeff0_38*dmats0[38][44] + coeff0_39*dmats0[39][44] + coeff0_40*dmats0[40][44] + coeff0_41*dmats0[41][44] + coeff0_42*dmats0[42][44] + coeff0_43*dmats0[43][44] + coeff0_44*dmats0[44][44] + coeff0_45*dmats0[45][44] + coeff0_46*dmats0[46][44] + coeff0_47*dmats0[47][44] + coeff0_48*dmats0[48][44] + coeff0_49*dmats0[49][44] + coeff0_50*dmats0[50][44] + coeff0_51*dmats0[51][44] + coeff0_52*dmats0[52][44] + coeff0_53*dmats0[53][44] + coeff0_54*dmats0[54][44] + coeff0_55*dmats0[55][44];
          new_coeff0_45 = coeff0_0*dmats0[0][45] + coeff0_1*dmats0[1][45] + coeff0_2*dmats0[2][45] + coeff0_3*dmats0[3][45] + coeff0_4*dmats0[4][45] + coeff0_5*dmats0[5][45] + coeff0_6*dmats0[6][45] + coeff0_7*dmats0[7][45] + coeff0_8*dmats0[8][45] + coeff0_9*dmats0[9][45] + coeff0_10*dmats0[10][45] + coeff0_11*dmats0[11][45] + coeff0_12*dmats0[12][45] + coeff0_13*dmats0[13][45] + coeff0_14*dmats0[14][45] + coeff0_15*dmats0[15][45] + coeff0_16*dmats0[16][45] + coeff0_17*dmats0[17][45] + coeff0_18*dmats0[18][45] + coeff0_19*dmats0[19][45] + coeff0_20*dmats0[20][45] + coeff0_21*dmats0[21][45] + coeff0_22*dmats0[22][45] + coeff0_23*dmats0[23][45] + coeff0_24*dmats0[24][45] + coeff0_25*dmats0[25][45] + coeff0_26*dmats0[26][45] + coeff0_27*dmats0[27][45] + coeff0_28*dmats0[28][45] + coeff0_29*dmats0[29][45] + coeff0_30*dmats0[30][45] + coeff0_31*dmats0[31][45] + coeff0_32*dmats0[32][45] + coeff0_33*dmats0[33][45] + coeff0_34*dmats0[34][45] + coeff0_35*dmats0[35][45] + coeff0_36*dmats0[36][45] + coeff0_37*dmats0[37][45] + coeff0_38*dmats0[38][45] + coeff0_39*dmats0[39][45] + coeff0_40*dmats0[40][45] + coeff0_41*dmats0[41][45] + coeff0_42*dmats0[42][45] + coeff0_43*dmats0[43][45] + coeff0_44*dmats0[44][45] + coeff0_45*dmats0[45][45] + coeff0_46*dmats0[46][45] + coeff0_47*dmats0[47][45] + coeff0_48*dmats0[48][45] + coeff0_49*dmats0[49][45] + coeff0_50*dmats0[50][45] + coeff0_51*dmats0[51][45] + coeff0_52*dmats0[52][45] + coeff0_53*dmats0[53][45] + coeff0_54*dmats0[54][45] + coeff0_55*dmats0[55][45];
          new_coeff0_46 = coeff0_0*dmats0[0][46] + coeff0_1*dmats0[1][46] + coeff0_2*dmats0[2][46] + coeff0_3*dmats0[3][46] + coeff0_4*dmats0[4][46] + coeff0_5*dmats0[5][46] + coeff0_6*dmats0[6][46] + coeff0_7*dmats0[7][46] + coeff0_8*dmats0[8][46] + coeff0_9*dmats0[9][46] + coeff0_10*dmats0[10][46] + coeff0_11*dmats0[11][46] + coeff0_12*dmats0[12][46] + coeff0_13*dmats0[13][46] + coeff0_14*dmats0[14][46] + coeff0_15*dmats0[15][46] + coeff0_16*dmats0[16][46] + coeff0_17*dmats0[17][46] + coeff0_18*dmats0[18][46] + coeff0_19*dmats0[19][46] + coeff0_20*dmats0[20][46] + coeff0_21*dmats0[21][46] + coeff0_22*dmats0[22][46] + coeff0_23*dmats0[23][46] + coeff0_24*dmats0[24][46] + coeff0_25*dmats0[25][46] + coeff0_26*dmats0[26][46] + coeff0_27*dmats0[27][46] + coeff0_28*dmats0[28][46] + coeff0_29*dmats0[29][46] + coeff0_30*dmats0[30][46] + coeff0_31*dmats0[31][46] + coeff0_32*dmats0[32][46] + coeff0_33*dmats0[33][46] + coeff0_34*dmats0[34][46] + coeff0_35*dmats0[35][46] + coeff0_36*dmats0[36][46] + coeff0_37*dmats0[37][46] + coeff0_38*dmats0[38][46] + coeff0_39*dmats0[39][46] + coeff0_40*dmats0[40][46] + coeff0_41*dmats0[41][46] + coeff0_42*dmats0[42][46] + coeff0_43*dmats0[43][46] + coeff0_44*dmats0[44][46] + coeff0_45*dmats0[45][46] + coeff0_46*dmats0[46][46] + coeff0_47*dmats0[47][46] + coeff0_48*dmats0[48][46] + coeff0_49*dmats0[49][46] + coeff0_50*dmats0[50][46] + coeff0_51*dmats0[51][46] + coeff0_52*dmats0[52][46] + coeff0_53*dmats0[53][46] + coeff0_54*dmats0[54][46] + coeff0_55*dmats0[55][46];
          new_coeff0_47 = coeff0_0*dmats0[0][47] + coeff0_1*dmats0[1][47] + coeff0_2*dmats0[2][47] + coeff0_3*dmats0[3][47] + coeff0_4*dmats0[4][47] + coeff0_5*dmats0[5][47] + coeff0_6*dmats0[6][47] + coeff0_7*dmats0[7][47] + coeff0_8*dmats0[8][47] + coeff0_9*dmats0[9][47] + coeff0_10*dmats0[10][47] + coeff0_11*dmats0[11][47] + coeff0_12*dmats0[12][47] + coeff0_13*dmats0[13][47] + coeff0_14*dmats0[14][47] + coeff0_15*dmats0[15][47] + coeff0_16*dmats0[16][47] + coeff0_17*dmats0[17][47] + coeff0_18*dmats0[18][47] + coeff0_19*dmats0[19][47] + coeff0_20*dmats0[20][47] + coeff0_21*dmats0[21][47] + coeff0_22*dmats0[22][47] + coeff0_23*dmats0[23][47] + coeff0_24*dmats0[24][47] + coeff0_25*dmats0[25][47] + coeff0_26*dmats0[26][47] + coeff0_27*dmats0[27][47] + coeff0_28*dmats0[28][47] + coeff0_29*dmats0[29][47] + coeff0_30*dmats0[30][47] + coeff0_31*dmats0[31][47] + coeff0_32*dmats0[32][47] + coeff0_33*dmats0[33][47] + coeff0_34*dmats0[34][47] + coeff0_35*dmats0[35][47] + coeff0_36*dmats0[36][47] + coeff0_37*dmats0[37][47] + coeff0_38*dmats0[38][47] + coeff0_39*dmats0[39][47] + coeff0_40*dmats0[40][47] + coeff0_41*dmats0[41][47] + coeff0_42*dmats0[42][47] + coeff0_43*dmats0[43][47] + coeff0_44*dmats0[44][47] + coeff0_45*dmats0[45][47] + coeff0_46*dmats0[46][47] + coeff0_47*dmats0[47][47] + coeff0_48*dmats0[48][47] + coeff0_49*dmats0[49][47] + coeff0_50*dmats0[50][47] + coeff0_51*dmats0[51][47] + coeff0_52*dmats0[52][47] + coeff0_53*dmats0[53][47] + coeff0_54*dmats0[54][47] + coeff0_55*dmats0[55][47];
          new_coeff0_48 = coeff0_0*dmats0[0][48] + coeff0_1*dmats0[1][48] + coeff0_2*dmats0[2][48] + coeff0_3*dmats0[3][48] + coeff0_4*dmats0[4][48] + coeff0_5*dmats0[5][48] + coeff0_6*dmats0[6][48] + coeff0_7*dmats0[7][48] + coeff0_8*dmats0[8][48] + coeff0_9*dmats0[9][48] + coeff0_10*dmats0[10][48] + coeff0_11*dmats0[11][48] + coeff0_12*dmats0[12][48] + coeff0_13*dmats0[13][48] + coeff0_14*dmats0[14][48] + coeff0_15*dmats0[15][48] + coeff0_16*dmats0[16][48] + coeff0_17*dmats0[17][48] + coeff0_18*dmats0[18][48] + coeff0_19*dmats0[19][48] + coeff0_20*dmats0[20][48] + coeff0_21*dmats0[21][48] + coeff0_22*dmats0[22][48] + coeff0_23*dmats0[23][48] + coeff0_24*dmats0[24][48] + coeff0_25*dmats0[25][48] + coeff0_26*dmats0[26][48] + coeff0_27*dmats0[27][48] + coeff0_28*dmats0[28][48] + coeff0_29*dmats0[29][48] + coeff0_30*dmats0[30][48] + coeff0_31*dmats0[31][48] + coeff0_32*dmats0[32][48] + coeff0_33*dmats0[33][48] + coeff0_34*dmats0[34][48] + coeff0_35*dmats0[35][48] + coeff0_36*dmats0[36][48] + coeff0_37*dmats0[37][48] + coeff0_38*dmats0[38][48] + coeff0_39*dmats0[39][48] + coeff0_40*dmats0[40][48] + coeff0_41*dmats0[41][48] + coeff0_42*dmats0[42][48] + coeff0_43*dmats0[43][48] + coeff0_44*dmats0[44][48] + coeff0_45*dmats0[45][48] + coeff0_46*dmats0[46][48] + coeff0_47*dmats0[47][48] + coeff0_48*dmats0[48][48] + coeff0_49*dmats0[49][48] + coeff0_50*dmats0[50][48] + coeff0_51*dmats0[51][48] + coeff0_52*dmats0[52][48] + coeff0_53*dmats0[53][48] + coeff0_54*dmats0[54][48] + coeff0_55*dmats0[55][48];
          new_coeff0_49 = coeff0_0*dmats0[0][49] + coeff0_1*dmats0[1][49] + coeff0_2*dmats0[2][49] + coeff0_3*dmats0[3][49] + coeff0_4*dmats0[4][49] + coeff0_5*dmats0[5][49] + coeff0_6*dmats0[6][49] + coeff0_7*dmats0[7][49] + coeff0_8*dmats0[8][49] + coeff0_9*dmats0[9][49] + coeff0_10*dmats0[10][49] + coeff0_11*dmats0[11][49] + coeff0_12*dmats0[12][49] + coeff0_13*dmats0[13][49] + coeff0_14*dmats0[14][49] + coeff0_15*dmats0[15][49] + coeff0_16*dmats0[16][49] + coeff0_17*dmats0[17][49] + coeff0_18*dmats0[18][49] + coeff0_19*dmats0[19][49] + coeff0_20*dmats0[20][49] + coeff0_21*dmats0[21][49] + coeff0_22*dmats0[22][49] + coeff0_23*dmats0[23][49] + coeff0_24*dmats0[24][49] + coeff0_25*dmats0[25][49] + coeff0_26*dmats0[26][49] + coeff0_27*dmats0[27][49] + coeff0_28*dmats0[28][49] + coeff0_29*dmats0[29][49] + coeff0_30*dmats0[30][49] + coeff0_31*dmats0[31][49] + coeff0_32*dmats0[32][49] + coeff0_33*dmats0[33][49] + coeff0_34*dmats0[34][49] + coeff0_35*dmats0[35][49] + coeff0_36*dmats0[36][49] + coeff0_37*dmats0[37][49] + coeff0_38*dmats0[38][49] + coeff0_39*dmats0[39][49] + coeff0_40*dmats0[40][49] + coeff0_41*dmats0[41][49] + coeff0_42*dmats0[42][49] + coeff0_43*dmats0[43][49] + coeff0_44*dmats0[44][49] + coeff0_45*dmats0[45][49] + coeff0_46*dmats0[46][49] + coeff0_47*dmats0[47][49] + coeff0_48*dmats0[48][49] + coeff0_49*dmats0[49][49] + coeff0_50*dmats0[50][49] + coeff0_51*dmats0[51][49] + coeff0_52*dmats0[52][49] + coeff0_53*dmats0[53][49] + coeff0_54*dmats0[54][49] + coeff0_55*dmats0[55][49];
          new_coeff0_50 = coeff0_0*dmats0[0][50] + coeff0_1*dmats0[1][50] + coeff0_2*dmats0[2][50] + coeff0_3*dmats0[3][50] + coeff0_4*dmats0[4][50] + coeff0_5*dmats0[5][50] + coeff0_6*dmats0[6][50] + coeff0_7*dmats0[7][50] + coeff0_8*dmats0[8][50] + coeff0_9*dmats0[9][50] + coeff0_10*dmats0[10][50] + coeff0_11*dmats0[11][50] + coeff0_12*dmats0[12][50] + coeff0_13*dmats0[13][50] + coeff0_14*dmats0[14][50] + coeff0_15*dmats0[15][50] + coeff0_16*dmats0[16][50] + coeff0_17*dmats0[17][50] + coeff0_18*dmats0[18][50] + coeff0_19*dmats0[19][50] + coeff0_20*dmats0[20][50] + coeff0_21*dmats0[21][50] + coeff0_22*dmats0[22][50] + coeff0_23*dmats0[23][50] + coeff0_24*dmats0[24][50] + coeff0_25*dmats0[25][50] + coeff0_26*dmats0[26][50] + coeff0_27*dmats0[27][50] + coeff0_28*dmats0[28][50] + coeff0_29*dmats0[29][50] + coeff0_30*dmats0[30][50] + coeff0_31*dmats0[31][50] + coeff0_32*dmats0[32][50] + coeff0_33*dmats0[33][50] + coeff0_34*dmats0[34][50] + coeff0_35*dmats0[35][50] + coeff0_36*dmats0[36][50] + coeff0_37*dmats0[37][50] + coeff0_38*dmats0[38][50] + coeff0_39*dmats0[39][50] + coeff0_40*dmats0[40][50] + coeff0_41*dmats0[41][50] + coeff0_42*dmats0[42][50] + coeff0_43*dmats0[43][50] + coeff0_44*dmats0[44][50] + coeff0_45*dmats0[45][50] + coeff0_46*dmats0[46][50] + coeff0_47*dmats0[47][50] + coeff0_48*dmats0[48][50] + coeff0_49*dmats0[49][50] + coeff0_50*dmats0[50][50] + coeff0_51*dmats0[51][50] + coeff0_52*dmats0[52][50] + coeff0_53*dmats0[53][50] + coeff0_54*dmats0[54][50] + coeff0_55*dmats0[55][50];
          new_coeff0_51 = coeff0_0*dmats0[0][51] + coeff0_1*dmats0[1][51] + coeff0_2*dmats0[2][51] + coeff0_3*dmats0[3][51] + coeff0_4*dmats0[4][51] + coeff0_5*dmats0[5][51] + coeff0_6*dmats0[6][51] + coeff0_7*dmats0[7][51] + coeff0_8*dmats0[8][51] + coeff0_9*dmats0[9][51] + coeff0_10*dmats0[10][51] + coeff0_11*dmats0[11][51] + coeff0_12*dmats0[12][51] + coeff0_13*dmats0[13][51] + coeff0_14*dmats0[14][51] + coeff0_15*dmats0[15][51] + coeff0_16*dmats0[16][51] + coeff0_17*dmats0[17][51] + coeff0_18*dmats0[18][51] + coeff0_19*dmats0[19][51] + coeff0_20*dmats0[20][51] + coeff0_21*dmats0[21][51] + coeff0_22*dmats0[22][51] + coeff0_23*dmats0[23][51] + coeff0_24*dmats0[24][51] + coeff0_25*dmats0[25][51] + coeff0_26*dmats0[26][51] + coeff0_27*dmats0[27][51] + coeff0_28*dmats0[28][51] + coeff0_29*dmats0[29][51] + coeff0_30*dmats0[30][51] + coeff0_31*dmats0[31][51] + coeff0_32*dmats0[32][51] + coeff0_33*dmats0[33][51] + coeff0_34*dmats0[34][51] + coeff0_35*dmats0[35][51] + coeff0_36*dmats0[36][51] + coeff0_37*dmats0[37][51] + coeff0_38*dmats0[38][51] + coeff0_39*dmats0[39][51] + coeff0_40*dmats0[40][51] + coeff0_41*dmats0[41][51] + coeff0_42*dmats0[42][51] + coeff0_43*dmats0[43][51] + coeff0_44*dmats0[44][51] + coeff0_45*dmats0[45][51] + coeff0_46*dmats0[46][51] + coeff0_47*dmats0[47][51] + coeff0_48*dmats0[48][51] + coeff0_49*dmats0[49][51] + coeff0_50*dmats0[50][51] + coeff0_51*dmats0[51][51] + coeff0_52*dmats0[52][51] + coeff0_53*dmats0[53][51] + coeff0_54*dmats0[54][51] + coeff0_55*dmats0[55][51];
          new_coeff0_52 = coeff0_0*dmats0[0][52] + coeff0_1*dmats0[1][52] + coeff0_2*dmats0[2][52] + coeff0_3*dmats0[3][52] + coeff0_4*dmats0[4][52] + coeff0_5*dmats0[5][52] + coeff0_6*dmats0[6][52] + coeff0_7*dmats0[7][52] + coeff0_8*dmats0[8][52] + coeff0_9*dmats0[9][52] + coeff0_10*dmats0[10][52] + coeff0_11*dmats0[11][52] + coeff0_12*dmats0[12][52] + coeff0_13*dmats0[13][52] + coeff0_14*dmats0[14][52] + coeff0_15*dmats0[15][52] + coeff0_16*dmats0[16][52] + coeff0_17*dmats0[17][52] + coeff0_18*dmats0[18][52] + coeff0_19*dmats0[19][52] + coeff0_20*dmats0[20][52] + coeff0_21*dmats0[21][52] + coeff0_22*dmats0[22][52] + coeff0_23*dmats0[23][52] + coeff0_24*dmats0[24][52] + coeff0_25*dmats0[25][52] + coeff0_26*dmats0[26][52] + coeff0_27*dmats0[27][52] + coeff0_28*dmats0[28][52] + coeff0_29*dmats0[29][52] + coeff0_30*dmats0[30][52] + coeff0_31*dmats0[31][52] + coeff0_32*dmats0[32][52] + coeff0_33*dmats0[33][52] + coeff0_34*dmats0[34][52] + coeff0_35*dmats0[35][52] + coeff0_36*dmats0[36][52] + coeff0_37*dmats0[37][52] + coeff0_38*dmats0[38][52] + coeff0_39*dmats0[39][52] + coeff0_40*dmats0[40][52] + coeff0_41*dmats0[41][52] + coeff0_42*dmats0[42][52] + coeff0_43*dmats0[43][52] + coeff0_44*dmats0[44][52] + coeff0_45*dmats0[45][52] + coeff0_46*dmats0[46][52] + coeff0_47*dmats0[47][52] + coeff0_48*dmats0[48][52] + coeff0_49*dmats0[49][52] + coeff0_50*dmats0[50][52] + coeff0_51*dmats0[51][52] + coeff0_52*dmats0[52][52] + coeff0_53*dmats0[53][52] + coeff0_54*dmats0[54][52] + coeff0_55*dmats0[55][52];
          new_coeff0_53 = coeff0_0*dmats0[0][53] + coeff0_1*dmats0[1][53] + coeff0_2*dmats0[2][53] + coeff0_3*dmats0[3][53] + coeff0_4*dmats0[4][53] + coeff0_5*dmats0[5][53] + coeff0_6*dmats0[6][53] + coeff0_7*dmats0[7][53] + coeff0_8*dmats0[8][53] + coeff0_9*dmats0[9][53] + coeff0_10*dmats0[10][53] + coeff0_11*dmats0[11][53] + coeff0_12*dmats0[12][53] + coeff0_13*dmats0[13][53] + coeff0_14*dmats0[14][53] + coeff0_15*dmats0[15][53] + coeff0_16*dmats0[16][53] + coeff0_17*dmats0[17][53] + coeff0_18*dmats0[18][53] + coeff0_19*dmats0[19][53] + coeff0_20*dmats0[20][53] + coeff0_21*dmats0[21][53] + coeff0_22*dmats0[22][53] + coeff0_23*dmats0[23][53] + coeff0_24*dmats0[24][53] + coeff0_25*dmats0[25][53] + coeff0_26*dmats0[26][53] + coeff0_27*dmats0[27][53] + coeff0_28*dmats0[28][53] + coeff0_29*dmats0[29][53] + coeff0_30*dmats0[30][53] + coeff0_31*dmats0[31][53] + coeff0_32*dmats0[32][53] + coeff0_33*dmats0[33][53] + coeff0_34*dmats0[34][53] + coeff0_35*dmats0[35][53] + coeff0_36*dmats0[36][53] + coeff0_37*dmats0[37][53] + coeff0_38*dmats0[38][53] + coeff0_39*dmats0[39][53] + coeff0_40*dmats0[40][53] + coeff0_41*dmats0[41][53] + coeff0_42*dmats0[42][53] + coeff0_43*dmats0[43][53] + coeff0_44*dmats0[44][53] + coeff0_45*dmats0[45][53] + coeff0_46*dmats0[46][53] + coeff0_47*dmats0[47][53] + coeff0_48*dmats0[48][53] + coeff0_49*dmats0[49][53] + coeff0_50*dmats0[50][53] + coeff0_51*dmats0[51][53] + coeff0_52*dmats0[52][53] + coeff0_53*dmats0[53][53] + coeff0_54*dmats0[54][53] + coeff0_55*dmats0[55][53];
          new_coeff0_54 = coeff0_0*dmats0[0][54] + coeff0_1*dmats0[1][54] + coeff0_2*dmats0[2][54] + coeff0_3*dmats0[3][54] + coeff0_4*dmats0[4][54] + coeff0_5*dmats0[5][54] + coeff0_6*dmats0[6][54] + coeff0_7*dmats0[7][54] + coeff0_8*dmats0[8][54] + coeff0_9*dmats0[9][54] + coeff0_10*dmats0[10][54] + coeff0_11*dmats0[11][54] + coeff0_12*dmats0[12][54] + coeff0_13*dmats0[13][54] + coeff0_14*dmats0[14][54] + coeff0_15*dmats0[15][54] + coeff0_16*dmats0[16][54] + coeff0_17*dmats0[17][54] + coeff0_18*dmats0[18][54] + coeff0_19*dmats0[19][54] + coeff0_20*dmats0[20][54] + coeff0_21*dmats0[21][54] + coeff0_22*dmats0[22][54] + coeff0_23*dmats0[23][54] + coeff0_24*dmats0[24][54] + coeff0_25*dmats0[25][54] + coeff0_26*dmats0[26][54] + coeff0_27*dmats0[27][54] + coeff0_28*dmats0[28][54] + coeff0_29*dmats0[29][54] + coeff0_30*dmats0[30][54] + coeff0_31*dmats0[31][54] + coeff0_32*dmats0[32][54] + coeff0_33*dmats0[33][54] + coeff0_34*dmats0[34][54] + coeff0_35*dmats0[35][54] + coeff0_36*dmats0[36][54] + coeff0_37*dmats0[37][54] + coeff0_38*dmats0[38][54] + coeff0_39*dmats0[39][54] + coeff0_40*dmats0[40][54] + coeff0_41*dmats0[41][54] + coeff0_42*dmats0[42][54] + coeff0_43*dmats0[43][54] + coeff0_44*dmats0[44][54] + coeff0_45*dmats0[45][54] + coeff0_46*dmats0[46][54] + coeff0_47*dmats0[47][54] + coeff0_48*dmats0[48][54] + coeff0_49*dmats0[49][54] + coeff0_50*dmats0[50][54] + coeff0_51*dmats0[51][54] + coeff0_52*dmats0[52][54] + coeff0_53*dmats0[53][54] + coeff0_54*dmats0[54][54] + coeff0_55*dmats0[55][54];
          new_coeff0_55 = coeff0_0*dmats0[0][55] + coeff0_1*dmats0[1][55] + coeff0_2*dmats0[2][55] + coeff0_3*dmats0[3][55] + coeff0_4*dmats0[4][55] + coeff0_5*dmats0[5][55] + coeff0_6*dmats0[6][55] + coeff0_7*dmats0[7][55] + coeff0_8*dmats0[8][55] + coeff0_9*dmats0[9][55] + coeff0_10*dmats0[10][55] + coeff0_11*dmats0[11][55] + coeff0_12*dmats0[12][55] + coeff0_13*dmats0[13][55] + coeff0_14*dmats0[14][55] + coeff0_15*dmats0[15][55] + coeff0_16*dmats0[16][55] + coeff0_17*dmats0[17][55] + coeff0_18*dmats0[18][55] + coeff0_19*dmats0[19][55] + coeff0_20*dmats0[20][55] + coeff0_21*dmats0[21][55] + coeff0_22*dmats0[22][55] + coeff0_23*dmats0[23][55] + coeff0_24*dmats0[24][55] + coeff0_25*dmats0[25][55] + coeff0_26*dmats0[26][55] + coeff0_27*dmats0[27][55] + coeff0_28*dmats0[28][55] + coeff0_29*dmats0[29][55] + coeff0_30*dmats0[30][55] + coeff0_31*dmats0[31][55] + coeff0_32*dmats0[32][55] + coeff0_33*dmats0[33][55] + coeff0_34*dmats0[34][55] + coeff0_35*dmats0[35][55] + coeff0_36*dmats0[36][55] + coeff0_37*dmats0[37][55] + coeff0_38*dmats0[38][55] + coeff0_39*dmats0[39][55] + coeff0_40*dmats0[40][55] + coeff0_41*dmats0[41][55] + coeff0_42*dmats0[42][55] + coeff0_43*dmats0[43][55] + coeff0_44*dmats0[44][55] + coeff0_45*dmats0[45][55] + coeff0_46*dmats0[46][55] + coeff0_47*dmats0[47][55] + coeff0_48*dmats0[48][55] + coeff0_49*dmats0[49][55] + coeff0_50*dmats0[50][55] + coeff0_51*dmats0[51][55] + coeff0_52*dmats0[52][55] + coeff0_53*dmats0[53][55] + coeff0_54*dmats0[54][55] + coeff0_55*dmats0[55][55];
        }
        if(combinations[deriv_num][j] == 1)
        {
          new_coeff0_0 = coeff0_0*dmats1[0][0] + coeff0_1*dmats1[1][0] + coeff0_2*dmats1[2][0] + coeff0_3*dmats1[3][0] + coeff0_4*dmats1[4][0] + coeff0_5*dmats1[5][0] + coeff0_6*dmats1[6][0] + coeff0_7*dmats1[7][0] + coeff0_8*dmats1[8][0] + coeff0_9*dmats1[9][0] + coeff0_10*dmats1[10][0] + coeff0_11*dmats1[11][0] + coeff0_12*dmats1[12][0] + coeff0_13*dmats1[13][0] + coeff0_14*dmats1[14][0] + coeff0_15*dmats1[15][0] + coeff0_16*dmats1[16][0] + coeff0_17*dmats1[17][0] + coeff0_18*dmats1[18][0] + coeff0_19*dmats1[19][0] + coeff0_20*dmats1[20][0] + coeff0_21*dmats1[21][0] + coeff0_22*dmats1[22][0] + coeff0_23*dmats1[23][0] + coeff0_24*dmats1[24][0] + coeff0_25*dmats1[25][0] + coeff0_26*dmats1[26][0] + coeff0_27*dmats1[27][0] + coeff0_28*dmats1[28][0] + coeff0_29*dmats1[29][0] + coeff0_30*dmats1[30][0] + coeff0_31*dmats1[31][0] + coeff0_32*dmats1[32][0] + coeff0_33*dmats1[33][0] + coeff0_34*dmats1[34][0] + coeff0_35*dmats1[35][0] + coeff0_36*dmats1[36][0] + coeff0_37*dmats1[37][0] + coeff0_38*dmats1[38][0] + coeff0_39*dmats1[39][0] + coeff0_40*dmats1[40][0] + coeff0_41*dmats1[41][0] + coeff0_42*dmats1[42][0] + coeff0_43*dmats1[43][0] + coeff0_44*dmats1[44][0] + coeff0_45*dmats1[45][0] + coeff0_46*dmats1[46][0] + coeff0_47*dmats1[47][0] + coeff0_48*dmats1[48][0] + coeff0_49*dmats1[49][0] + coeff0_50*dmats1[50][0] + coeff0_51*dmats1[51][0] + coeff0_52*dmats1[52][0] + coeff0_53*dmats1[53][0] + coeff0_54*dmats1[54][0] + coeff0_55*dmats1[55][0];
          new_coeff0_1 = coeff0_0*dmats1[0][1] + coeff0_1*dmats1[1][1] + coeff0_2*dmats1[2][1] + coeff0_3*dmats1[3][1] + coeff0_4*dmats1[4][1] + coeff0_5*dmats1[5][1] + coeff0_6*dmats1[6][1] + coeff0_7*dmats1[7][1] + coeff0_8*dmats1[8][1] + coeff0_9*dmats1[9][1] + coeff0_10*dmats1[10][1] + coeff0_11*dmats1[11][1] + coeff0_12*dmats1[12][1] + coeff0_13*dmats1[13][1] + coeff0_14*dmats1[14][1] + coeff0_15*dmats1[15][1] + coeff0_16*dmats1[16][1] + coeff0_17*dmats1[17][1] + coeff0_18*dmats1[18][1] + coeff0_19*dmats1[19][1] + coeff0_20*dmats1[20][1] + coeff0_21*dmats1[21][1] + coeff0_22*dmats1[22][1] + coeff0_23*dmats1[23][1] + coeff0_24*dmats1[24][1] + coeff0_25*dmats1[25][1] + coeff0_26*dmats1[26][1] + coeff0_27*dmats1[27][1] + coeff0_28*dmats1[28][1] + coeff0_29*dmats1[29][1] + coeff0_30*dmats1[30][1] + coeff0_31*dmats1[31][1] + coeff0_32*dmats1[32][1] + coeff0_33*dmats1[33][1] + coeff0_34*dmats1[34][1] + coeff0_35*dmats1[35][1] + coeff0_36*dmats1[36][1] + coeff0_37*dmats1[37][1] + coeff0_38*dmats1[38][1] + coeff0_39*dmats1[39][1] + coeff0_40*dmats1[40][1] + coeff0_41*dmats1[41][1] + coeff0_42*dmats1[42][1] + coeff0_43*dmats1[43][1] + coeff0_44*dmats1[44][1] + coeff0_45*dmats1[45][1] + coeff0_46*dmats1[46][1] + coeff0_47*dmats1[47][1] + coeff0_48*dmats1[48][1] + coeff0_49*dmats1[49][1] + coeff0_50*dmats1[50][1] + coeff0_51*dmats1[51][1] + coeff0_52*dmats1[52][1] + coeff0_53*dmats1[53][1] + coeff0_54*dmats1[54][1] + coeff0_55*dmats1[55][1];
          new_coeff0_2 = coeff0_0*dmats1[0][2] + coeff0_1*dmats1[1][2] + coeff0_2*dmats1[2][2] + coeff0_3*dmats1[3][2] + coeff0_4*dmats1[4][2] + coeff0_5*dmats1[5][2] + coeff0_6*dmats1[6][2] + coeff0_7*dmats1[7][2] + coeff0_8*dmats1[8][2] + coeff0_9*dmats1[9][2] + coeff0_10*dmats1[10][2] + coeff0_11*dmats1[11][2] + coeff0_12*dmats1[12][2] + coeff0_13*dmats1[13][2] + coeff0_14*dmats1[14][2] + coeff0_15*dmats1[15][2] + coeff0_16*dmats1[16][2] + coeff0_17*dmats1[17][2] + coeff0_18*dmats1[18][2] + coeff0_19*dmats1[19][2] + coeff0_20*dmats1[20][2] + coeff0_21*dmats1[21][2] + coeff0_22*dmats1[22][2] + coeff0_23*dmats1[23][2] + coeff0_24*dmats1[24][2] + coeff0_25*dmats1[25][2] + coeff0_26*dmats1[26][2] + coeff0_27*dmats1[27][2] + coeff0_28*dmats1[28][2] + coeff0_29*dmats1[29][2] + coeff0_30*dmats1[30][2] + coeff0_31*dmats1[31][2] + coeff0_32*dmats1[32][2] + coeff0_33*dmats1[33][2] + coeff0_34*dmats1[34][2] + coeff0_35*dmats1[35][2] + coeff0_36*dmats1[36][2] + coeff0_37*dmats1[37][2] + coeff0_38*dmats1[38][2] + coeff0_39*dmats1[39][2] + coeff0_40*dmats1[40][2] + coeff0_41*dmats1[41][2] + coeff0_42*dmats1[42][2] + coeff0_43*dmats1[43][2] + coeff0_44*dmats1[44][2] + coeff0_45*dmats1[45][2] + coeff0_46*dmats1[46][2] + coeff0_47*dmats1[47][2] + coeff0_48*dmats1[48][2] + coeff0_49*dmats1[49][2] + coeff0_50*dmats1[50][2] + coeff0_51*dmats1[51][2] + coeff0_52*dmats1[52][2] + coeff0_53*dmats1[53][2] + coeff0_54*dmats1[54][2] + coeff0_55*dmats1[55][2];
          new_coeff0_3 = coeff0_0*dmats1[0][3] + coeff0_1*dmats1[1][3] + coeff0_2*dmats1[2][3] + coeff0_3*dmats1[3][3] + coeff0_4*dmats1[4][3] + coeff0_5*dmats1[5][3] + coeff0_6*dmats1[6][3] + coeff0_7*dmats1[7][3] + coeff0_8*dmats1[8][3] + coeff0_9*dmats1[9][3] + coeff0_10*dmats1[10][3] + coeff0_11*dmats1[11][3] + coeff0_12*dmats1[12][3] + coeff0_13*dmats1[13][3] + coeff0_14*dmats1[14][3] + coeff0_15*dmats1[15][3] + coeff0_16*dmats1[16][3] + coeff0_17*dmats1[17][3] + coeff0_18*dmats1[18][3] + coeff0_19*dmats1[19][3] + coeff0_20*dmats1[20][3] + coeff0_21*dmats1[21][3] + coeff0_22*dmats1[22][3] + coeff0_23*dmats1[23][3] + coeff0_24*dmats1[24][3] + coeff0_25*dmats1[25][3] + coeff0_26*dmats1[26][3] + coeff0_27*dmats1[27][3] + coeff0_28*dmats1[28][3] + coeff0_29*dmats1[29][3] + coeff0_30*dmats1[30][3] + coeff0_31*dmats1[31][3] + coeff0_32*dmats1[32][3] + coeff0_33*dmats1[33][3] + coeff0_34*dmats1[34][3] + coeff0_35*dmats1[35][3] + coeff0_36*dmats1[36][3] + coeff0_37*dmats1[37][3] + coeff0_38*dmats1[38][3] + coeff0_39*dmats1[39][3] + coeff0_40*dmats1[40][3] + coeff0_41*dmats1[41][3] + coeff0_42*dmats1[42][3] + coeff0_43*dmats1[43][3] + coeff0_44*dmats1[44][3] + coeff0_45*dmats1[45][3] + coeff0_46*dmats1[46][3] + coeff0_47*dmats1[47][3] + coeff0_48*dmats1[48][3] + coeff0_49*dmats1[49][3] + coeff0_50*dmats1[50][3] + coeff0_51*dmats1[51][3] + coeff0_52*dmats1[52][3] + coeff0_53*dmats1[53][3] + coeff0_54*dmats1[54][3] + coeff0_55*dmats1[55][3];
          new_coeff0_4 = coeff0_0*dmats1[0][4] + coeff0_1*dmats1[1][4] + coeff0_2*dmats1[2][4] + coeff0_3*dmats1[3][4] + coeff0_4*dmats1[4][4] + coeff0_5*dmats1[5][4] + coeff0_6*dmats1[6][4] + coeff0_7*dmats1[7][4] + coeff0_8*dmats1[8][4] + coeff0_9*dmats1[9][4] + coeff0_10*dmats1[10][4] + coeff0_11*dmats1[11][4] + coeff0_12*dmats1[12][4] + coeff0_13*dmats1[13][4] + coeff0_14*dmats1[14][4] + coeff0_15*dmats1[15][4] + coeff0_16*dmats1[16][4] + coeff0_17*dmats1[17][4] + coeff0_18*dmats1[18][4] + coeff0_19*dmats1[19][4] + coeff0_20*dmats1[20][4] + coeff0_21*dmats1[21][4] + coeff0_22*dmats1[22][4] + coeff0_23*dmats1[23][4] + coeff0_24*dmats1[24][4] + coeff0_25*dmats1[25][4] + coeff0_26*dmats1[26][4] + coeff0_27*dmats1[27][4] + coeff0_28*dmats1[28][4] + coeff0_29*dmats1[29][4] + coeff0_30*dmats1[30][4] + coeff0_31*dmats1[31][4] + coeff0_32*dmats1[32][4] + coeff0_33*dmats1[33][4] + coeff0_34*dmats1[34][4] + coeff0_35*dmats1[35][4] + coeff0_36*dmats1[36][4] + coeff0_37*dmats1[37][4] + coeff0_38*dmats1[38][4] + coeff0_39*dmats1[39][4] + coeff0_40*dmats1[40][4] + coeff0_41*dmats1[41][4] + coeff0_42*dmats1[42][4] + coeff0_43*dmats1[43][4] + coeff0_44*dmats1[44][4] + coeff0_45*dmats1[45][4] + coeff0_46*dmats1[46][4] + coeff0_47*dmats1[47][4] + coeff0_48*dmats1[48][4] + coeff0_49*dmats1[49][4] + coeff0_50*dmats1[50][4] + coeff0_51*dmats1[51][4] + coeff0_52*dmats1[52][4] + coeff0_53*dmats1[53][4] + coeff0_54*dmats1[54][4] + coeff0_55*dmats1[55][4];
          new_coeff0_5 = coeff0_0*dmats1[0][5] + coeff0_1*dmats1[1][5] + coeff0_2*dmats1[2][5] + coeff0_3*dmats1[3][5] + coeff0_4*dmats1[4][5] + coeff0_5*dmats1[5][5] + coeff0_6*dmats1[6][5] + coeff0_7*dmats1[7][5] + coeff0_8*dmats1[8][5] + coeff0_9*dmats1[9][5] + coeff0_10*dmats1[10][5] + coeff0_11*dmats1[11][5] + coeff0_12*dmats1[12][5] + coeff0_13*dmats1[13][5] + coeff0_14*dmats1[14][5] + coeff0_15*dmats1[15][5] + coeff0_16*dmats1[16][5] + coeff0_17*dmats1[17][5] + coeff0_18*dmats1[18][5] + coeff0_19*dmats1[19][5] + coeff0_20*dmats1[20][5] + coeff0_21*dmats1[21][5] + coeff0_22*dmats1[22][5] + coeff0_23*dmats1[23][5] + coeff0_24*dmats1[24][5] + coeff0_25*dmats1[25][5] + coeff0_26*dmats1[26][5] + coeff0_27*dmats1[27][5] + coeff0_28*dmats1[28][5] + coeff0_29*dmats1[29][5] + coeff0_30*dmats1[30][5] + coeff0_31*dmats1[31][5] + coeff0_32*dmats1[32][5] + coeff0_33*dmats1[33][5] + coeff0_34*dmats1[34][5] + coeff0_35*dmats1[35][5] + coeff0_36*dmats1[36][5] + coeff0_37*dmats1[37][5] + coeff0_38*dmats1[38][5] + coeff0_39*dmats1[39][5] + coeff0_40*dmats1[40][5] + coeff0_41*dmats1[41][5] + coeff0_42*dmats1[42][5] + coeff0_43*dmats1[43][5] + coeff0_44*dmats1[44][5] + coeff0_45*dmats1[45][5] + coeff0_46*dmats1[46][5] + coeff0_47*dmats1[47][5] + coeff0_48*dmats1[48][5] + coeff0_49*dmats1[49][5] + coeff0_50*dmats1[50][5] + coeff0_51*dmats1[51][5] + coeff0_52*dmats1[52][5] + coeff0_53*dmats1[53][5] + coeff0_54*dmats1[54][5] + coeff0_55*dmats1[55][5];
          new_coeff0_6 = coeff0_0*dmats1[0][6] + coeff0_1*dmats1[1][6] + coeff0_2*dmats1[2][6] + coeff0_3*dmats1[3][6] + coeff0_4*dmats1[4][6] + coeff0_5*dmats1[5][6] + coeff0_6*dmats1[6][6] + coeff0_7*dmats1[7][6] + coeff0_8*dmats1[8][6] + coeff0_9*dmats1[9][6] + coeff0_10*dmats1[10][6] + coeff0_11*dmats1[11][6] + coeff0_12*dmats1[12][6] + coeff0_13*dmats1[13][6] + coeff0_14*dmats1[14][6] + coeff0_15*dmats1[15][6] + coeff0_16*dmats1[16][6] + coeff0_17*dmats1[17][6] + coeff0_18*dmats1[18][6] + coeff0_19*dmats1[19][6] + coeff0_20*dmats1[20][6] + coeff0_21*dmats1[21][6] + coeff0_22*dmats1[22][6] + coeff0_23*dmats1[23][6] + coeff0_24*dmats1[24][6] + coeff0_25*dmats1[25][6] + coeff0_26*dmats1[26][6] + coeff0_27*dmats1[27][6] + coeff0_28*dmats1[28][6] + coeff0_29*dmats1[29][6] + coeff0_30*dmats1[30][6] + coeff0_31*dmats1[31][6] + coeff0_32*dmats1[32][6] + coeff0_33*dmats1[33][6] + coeff0_34*dmats1[34][6] + coeff0_35*dmats1[35][6] + coeff0_36*dmats1[36][6] + coeff0_37*dmats1[37][6] + coeff0_38*dmats1[38][6] + coeff0_39*dmats1[39][6] + coeff0_40*dmats1[40][6] + coeff0_41*dmats1[41][6] + coeff0_42*dmats1[42][6] + coeff0_43*dmats1[43][6] + coeff0_44*dmats1[44][6] + coeff0_45*dmats1[45][6] + coeff0_46*dmats1[46][6] + coeff0_47*dmats1[47][6] + coeff0_48*dmats1[48][6] + coeff0_49*dmats1[49][6] + coeff0_50*dmats1[50][6] + coeff0_51*dmats1[51][6] + coeff0_52*dmats1[52][6] + coeff0_53*dmats1[53][6] + coeff0_54*dmats1[54][6] + coeff0_55*dmats1[55][6];
          new_coeff0_7 = coeff0_0*dmats1[0][7] + coeff0_1*dmats1[1][7] + coeff0_2*dmats1[2][7] + coeff0_3*dmats1[3][7] + coeff0_4*dmats1[4][7] + coeff0_5*dmats1[5][7] + coeff0_6*dmats1[6][7] + coeff0_7*dmats1[7][7] + coeff0_8*dmats1[8][7] + coeff0_9*dmats1[9][7] + coeff0_10*dmats1[10][7] + coeff0_11*dmats1[11][7] + coeff0_12*dmats1[12][7] + coeff0_13*dmats1[13][7] + coeff0_14*dmats1[14][7] + coeff0_15*dmats1[15][7] + coeff0_16*dmats1[16][7] + coeff0_17*dmats1[17][7] + coeff0_18*dmats1[18][7] + coeff0_19*dmats1[19][7] + coeff0_20*dmats1[20][7] + coeff0_21*dmats1[21][7] + coeff0_22*dmats1[22][7] + coeff0_23*dmats1[23][7] + coeff0_24*dmats1[24][7] + coeff0_25*dmats1[25][7] + coeff0_26*dmats1[26][7] + coeff0_27*dmats1[27][7] + coeff0_28*dmats1[28][7] + coeff0_29*dmats1[29][7] + coeff0_30*dmats1[30][7] + coeff0_31*dmats1[31][7] + coeff0_32*dmats1[32][7] + coeff0_33*dmats1[33][7] + coeff0_34*dmats1[34][7] + coeff0_35*dmats1[35][7] + coeff0_36*dmats1[36][7] + coeff0_37*dmats1[37][7] + coeff0_38*dmats1[38][7] + coeff0_39*dmats1[39][7] + coeff0_40*dmats1[40][7] + coeff0_41*dmats1[41][7] + coeff0_42*dmats1[42][7] + coeff0_43*dmats1[43][7] + coeff0_44*dmats1[44][7] + coeff0_45*dmats1[45][7] + coeff0_46*dmats1[46][7] + coeff0_47*dmats1[47][7] + coeff0_48*dmats1[48][7] + coeff0_49*dmats1[49][7] + coeff0_50*dmats1[50][7] + coeff0_51*dmats1[51][7] + coeff0_52*dmats1[52][7] + coeff0_53*dmats1[53][7] + coeff0_54*dmats1[54][7] + coeff0_55*dmats1[55][7];
          new_coeff0_8 = coeff0_0*dmats1[0][8] + coeff0_1*dmats1[1][8] + coeff0_2*dmats1[2][8] + coeff0_3*dmats1[3][8] + coeff0_4*dmats1[4][8] + coeff0_5*dmats1[5][8] + coeff0_6*dmats1[6][8] + coeff0_7*dmats1[7][8] + coeff0_8*dmats1[8][8] + coeff0_9*dmats1[9][8] + coeff0_10*dmats1[10][8] + coeff0_11*dmats1[11][8] + coeff0_12*dmats1[12][8] + coeff0_13*dmats1[13][8] + coeff0_14*dmats1[14][8] + coeff0_15*dmats1[15][8] + coeff0_16*dmats1[16][8] + coeff0_17*dmats1[17][8] + coeff0_18*dmats1[18][8] + coeff0_19*dmats1[19][8] + coeff0_20*dmats1[20][8] + coeff0_21*dmats1[21][8] + coeff0_22*dmats1[22][8] + coeff0_23*dmats1[23][8] + coeff0_24*dmats1[24][8] + coeff0_25*dmats1[25][8] + coeff0_26*dmats1[26][8] + coeff0_27*dmats1[27][8] + coeff0_28*dmats1[28][8] + coeff0_29*dmats1[29][8] + coeff0_30*dmats1[30][8] + coeff0_31*dmats1[31][8] + coeff0_32*dmats1[32][8] + coeff0_33*dmats1[33][8] + coeff0_34*dmats1[34][8] + coeff0_35*dmats1[35][8] + coeff0_36*dmats1[36][8] + coeff0_37*dmats1[37][8] + coeff0_38*dmats1[38][8] + coeff0_39*dmats1[39][8] + coeff0_40*dmats1[40][8] + coeff0_41*dmats1[41][8] + coeff0_42*dmats1[42][8] + coeff0_43*dmats1[43][8] + coeff0_44*dmats1[44][8] + coeff0_45*dmats1[45][8] + coeff0_46*dmats1[46][8] + coeff0_47*dmats1[47][8] + coeff0_48*dmats1[48][8] + coeff0_49*dmats1[49][8] + coeff0_50*dmats1[50][8] + coeff0_51*dmats1[51][8] + coeff0_52*dmats1[52][8] + coeff0_53*dmats1[53][8] + coeff0_54*dmats1[54][8] + coeff0_55*dmats1[55][8];
          new_coeff0_9 = coeff0_0*dmats1[0][9] + coeff0_1*dmats1[1][9] + coeff0_2*dmats1[2][9] + coeff0_3*dmats1[3][9] + coeff0_4*dmats1[4][9] + coeff0_5*dmats1[5][9] + coeff0_6*dmats1[6][9] + coeff0_7*dmats1[7][9] + coeff0_8*dmats1[8][9] + coeff0_9*dmats1[9][9] + coeff0_10*dmats1[10][9] + coeff0_11*dmats1[11][9] + coeff0_12*dmats1[12][9] + coeff0_13*dmats1[13][9] + coeff0_14*dmats1[14][9] + coeff0_15*dmats1[15][9] + coeff0_16*dmats1[16][9] + coeff0_17*dmats1[17][9] + coeff0_18*dmats1[18][9] + coeff0_19*dmats1[19][9] + coeff0_20*dmats1[20][9] + coeff0_21*dmats1[21][9] + coeff0_22*dmats1[22][9] + coeff0_23*dmats1[23][9] + coeff0_24*dmats1[24][9] + coeff0_25*dmats1[25][9] + coeff0_26*dmats1[26][9] + coeff0_27*dmats1[27][9] + coeff0_28*dmats1[28][9] + coeff0_29*dmats1[29][9] + coeff0_30*dmats1[30][9] + coeff0_31*dmats1[31][9] + coeff0_32*dmats1[32][9] + coeff0_33*dmats1[33][9] + coeff0_34*dmats1[34][9] + coeff0_35*dmats1[35][9] + coeff0_36*dmats1[36][9] + coeff0_37*dmats1[37][9] + coeff0_38*dmats1[38][9] + coeff0_39*dmats1[39][9] + coeff0_40*dmats1[40][9] + coeff0_41*dmats1[41][9] + coeff0_42*dmats1[42][9] + coeff0_43*dmats1[43][9] + coeff0_44*dmats1[44][9] + coeff0_45*dmats1[45][9] + coeff0_46*dmats1[46][9] + coeff0_47*dmats1[47][9] + coeff0_48*dmats1[48][9] + coeff0_49*dmats1[49][9] + coeff0_50*dmats1[50][9] + coeff0_51*dmats1[51][9] + coeff0_52*dmats1[52][9] + coeff0_53*dmats1[53][9] + coeff0_54*dmats1[54][9] + coeff0_55*dmats1[55][9];
          new_coeff0_10 = coeff0_0*dmats1[0][10] + coeff0_1*dmats1[1][10] + coeff0_2*dmats1[2][10] + coeff0_3*dmats1[3][10] + coeff0_4*dmats1[4][10] + coeff0_5*dmats1[5][10] + coeff0_6*dmats1[6][10] + coeff0_7*dmats1[7][10] + coeff0_8*dmats1[8][10] + coeff0_9*dmats1[9][10] + coeff0_10*dmats1[10][10] + coeff0_11*dmats1[11][10] + coeff0_12*dmats1[12][10] + coeff0_13*dmats1[13][10] + coeff0_14*dmats1[14][10] + coeff0_15*dmats1[15][10] + coeff0_16*dmats1[16][10] + coeff0_17*dmats1[17][10] + coeff0_18*dmats1[18][10] + coeff0_19*dmats1[19][10] + coeff0_20*dmats1[20][10] + coeff0_21*dmats1[21][10] + coeff0_22*dmats1[22][10] + coeff0_23*dmats1[23][10] + coeff0_24*dmats1[24][10] + coeff0_25*dmats1[25][10] + coeff0_26*dmats1[26][10] + coeff0_27*dmats1[27][10] + coeff0_28*dmats1[28][10] + coeff0_29*dmats1[29][10] + coeff0_30*dmats1[30][10] + coeff0_31*dmats1[31][10] + coeff0_32*dmats1[32][10] + coeff0_33*dmats1[33][10] + coeff0_34*dmats1[34][10] + coeff0_35*dmats1[35][10] + coeff0_36*dmats1[36][10] + coeff0_37*dmats1[37][10] + coeff0_38*dmats1[38][10] + coeff0_39*dmats1[39][10] + coeff0_40*dmats1[40][10] + coeff0_41*dmats1[41][10] + coeff0_42*dmats1[42][10] + coeff0_43*dmats1[43][10] + coeff0_44*dmats1[44][10] + coeff0_45*dmats1[45][10] + coeff0_46*dmats1[46][10] + coeff0_47*dmats1[47][10] + coeff0_48*dmats1[48][10] + coeff0_49*dmats1[49][10] + coeff0_50*dmats1[50][10] + coeff0_51*dmats1[51][10] + coeff0_52*dmats1[52][10] + coeff0_53*dmats1[53][10] + coeff0_54*dmats1[54][10] + coeff0_55*dmats1[55][10];
          new_coeff0_11 = coeff0_0*dmats1[0][11] + coeff0_1*dmats1[1][11] + coeff0_2*dmats1[2][11] + coeff0_3*dmats1[3][11] + coeff0_4*dmats1[4][11] + coeff0_5*dmats1[5][11] + coeff0_6*dmats1[6][11] + coeff0_7*dmats1[7][11] + coeff0_8*dmats1[8][11] + coeff0_9*dmats1[9][11] + coeff0_10*dmats1[10][11] + coeff0_11*dmats1[11][11] + coeff0_12*dmats1[12][11] + coeff0_13*dmats1[13][11] + coeff0_14*dmats1[14][11] + coeff0_15*dmats1[15][11] + coeff0_16*dmats1[16][11] + coeff0_17*dmats1[17][11] + coeff0_18*dmats1[18][11] + coeff0_19*dmats1[19][11] + coeff0_20*dmats1[20][11] + coeff0_21*dmats1[21][11] + coeff0_22*dmats1[22][11] + coeff0_23*dmats1[23][11] + coeff0_24*dmats1[24][11] + coeff0_25*dmats1[25][11] + coeff0_26*dmats1[26][11] + coeff0_27*dmats1[27][11] + coeff0_28*dmats1[28][11] + coeff0_29*dmats1[29][11] + coeff0_30*dmats1[30][11] + coeff0_31*dmats1[31][11] + coeff0_32*dmats1[32][11] + coeff0_33*dmats1[33][11] + coeff0_34*dmats1[34][11] + coeff0_35*dmats1[35][11] + coeff0_36*dmats1[36][11] + coeff0_37*dmats1[37][11] + coeff0_38*dmats1[38][11] + coeff0_39*dmats1[39][11] + coeff0_40*dmats1[40][11] + coeff0_41*dmats1[41][11] + coeff0_42*dmats1[42][11] + coeff0_43*dmats1[43][11] + coeff0_44*dmats1[44][11] + coeff0_45*dmats1[45][11] + coeff0_46*dmats1[46][11] + coeff0_47*dmats1[47][11] + coeff0_48*dmats1[48][11] + coeff0_49*dmats1[49][11] + coeff0_50*dmats1[50][11] + coeff0_51*dmats1[51][11] + coeff0_52*dmats1[52][11] + coeff0_53*dmats1[53][11] + coeff0_54*dmats1[54][11] + coeff0_55*dmats1[55][11];
          new_coeff0_12 = coeff0_0*dmats1[0][12] + coeff0_1*dmats1[1][12] + coeff0_2*dmats1[2][12] + coeff0_3*dmats1[3][12] + coeff0_4*dmats1[4][12] + coeff0_5*dmats1[5][12] + coeff0_6*dmats1[6][12] + coeff0_7*dmats1[7][12] + coeff0_8*dmats1[8][12] + coeff0_9*dmats1[9][12] + coeff0_10*dmats1[10][12] + coeff0_11*dmats1[11][12] + coeff0_12*dmats1[12][12] + coeff0_13*dmats1[13][12] + coeff0_14*dmats1[14][12] + coeff0_15*dmats1[15][12] + coeff0_16*dmats1[16][12] + coeff0_17*dmats1[17][12] + coeff0_18*dmats1[18][12] + coeff0_19*dmats1[19][12] + coeff0_20*dmats1[20][12] + coeff0_21*dmats1[21][12] + coeff0_22*dmats1[22][12] + coeff0_23*dmats1[23][12] + coeff0_24*dmats1[24][12] + coeff0_25*dmats1[25][12] + coeff0_26*dmats1[26][12] + coeff0_27*dmats1[27][12] + coeff0_28*dmats1[28][12] + coeff0_29*dmats1[29][12] + coeff0_30*dmats1[30][12] + coeff0_31*dmats1[31][12] + coeff0_32*dmats1[32][12] + coeff0_33*dmats1[33][12] + coeff0_34*dmats1[34][12] + coeff0_35*dmats1[35][12] + coeff0_36*dmats1[36][12] + coeff0_37*dmats1[37][12] + coeff0_38*dmats1[38][12] + coeff0_39*dmats1[39][12] + coeff0_40*dmats1[40][12] + coeff0_41*dmats1[41][12] + coeff0_42*dmats1[42][12] + coeff0_43*dmats1[43][12] + coeff0_44*dmats1[44][12] + coeff0_45*dmats1[45][12] + coeff0_46*dmats1[46][12] + coeff0_47*dmats1[47][12] + coeff0_48*dmats1[48][12] + coeff0_49*dmats1[49][12] + coeff0_50*dmats1[50][12] + coeff0_51*dmats1[51][12] + coeff0_52*dmats1[52][12] + coeff0_53*dmats1[53][12] + coeff0_54*dmats1[54][12] + coeff0_55*dmats1[55][12];
          new_coeff0_13 = coeff0_0*dmats1[0][13] + coeff0_1*dmats1[1][13] + coeff0_2*dmats1[2][13] + coeff0_3*dmats1[3][13] + coeff0_4*dmats1[4][13] + coeff0_5*dmats1[5][13] + coeff0_6*dmats1[6][13] + coeff0_7*dmats1[7][13] + coeff0_8*dmats1[8][13] + coeff0_9*dmats1[9][13] + coeff0_10*dmats1[10][13] + coeff0_11*dmats1[11][13] + coeff0_12*dmats1[12][13] + coeff0_13*dmats1[13][13] + coeff0_14*dmats1[14][13] + coeff0_15*dmats1[15][13] + coeff0_16*dmats1[16][13] + coeff0_17*dmats1[17][13] + coeff0_18*dmats1[18][13] + coeff0_19*dmats1[19][13] + coeff0_20*dmats1[20][13] + coeff0_21*dmats1[21][13] + coeff0_22*dmats1[22][13] + coeff0_23*dmats1[23][13] + coeff0_24*dmats1[24][13] + coeff0_25*dmats1[25][13] + coeff0_26*dmats1[26][13] + coeff0_27*dmats1[27][13] + coeff0_28*dmats1[28][13] + coeff0_29*dmats1[29][13] + coeff0_30*dmats1[30][13] + coeff0_31*dmats1[31][13] + coeff0_32*dmats1[32][13] + coeff0_33*dmats1[33][13] + coeff0_34*dmats1[34][13] + coeff0_35*dmats1[35][13] + coeff0_36*dmats1[36][13] + coeff0_37*dmats1[37][13] + coeff0_38*dmats1[38][13] + coeff0_39*dmats1[39][13] + coeff0_40*dmats1[40][13] + coeff0_41*dmats1[41][13] + coeff0_42*dmats1[42][13] + coeff0_43*dmats1[43][13] + coeff0_44*dmats1[44][13] + coeff0_45*dmats1[45][13] + coeff0_46*dmats1[46][13] + coeff0_47*dmats1[47][13] + coeff0_48*dmats1[48][13] + coeff0_49*dmats1[49][13] + coeff0_50*dmats1[50][13] + coeff0_51*dmats1[51][13] + coeff0_52*dmats1[52][13] + coeff0_53*dmats1[53][13] + coeff0_54*dmats1[54][13] + coeff0_55*dmats1[55][13];
          new_coeff0_14 = coeff0_0*dmats1[0][14] + coeff0_1*dmats1[1][14] + coeff0_2*dmats1[2][14] + coeff0_3*dmats1[3][14] + coeff0_4*dmats1[4][14] + coeff0_5*dmats1[5][14] + coeff0_6*dmats1[6][14] + coeff0_7*dmats1[7][14] + coeff0_8*dmats1[8][14] + coeff0_9*dmats1[9][14] + coeff0_10*dmats1[10][14] + coeff0_11*dmats1[11][14] + coeff0_12*dmats1[12][14] + coeff0_13*dmats1[13][14] + coeff0_14*dmats1[14][14] + coeff0_15*dmats1[15][14] + coeff0_16*dmats1[16][14] + coeff0_17*dmats1[17][14] + coeff0_18*dmats1[18][14] + coeff0_19*dmats1[19][14] + coeff0_20*dmats1[20][14] + coeff0_21*dmats1[21][14] + coeff0_22*dmats1[22][14] + coeff0_23*dmats1[23][14] + coeff0_24*dmats1[24][14] + coeff0_25*dmats1[25][14] + coeff0_26*dmats1[26][14] + coeff0_27*dmats1[27][14] + coeff0_28*dmats1[28][14] + coeff0_29*dmats1[29][14] + coeff0_30*dmats1[30][14] + coeff0_31*dmats1[31][14] + coeff0_32*dmats1[32][14] + coeff0_33*dmats1[33][14] + coeff0_34*dmats1[34][14] + coeff0_35*dmats1[35][14] + coeff0_36*dmats1[36][14] + coeff0_37*dmats1[37][14] + coeff0_38*dmats1[38][14] + coeff0_39*dmats1[39][14] + coeff0_40*dmats1[40][14] + coeff0_41*dmats1[41][14] + coeff0_42*dmats1[42][14] + coeff0_43*dmats1[43][14] + coeff0_44*dmats1[44][14] + coeff0_45*dmats1[45][14] + coeff0_46*dmats1[46][14] + coeff0_47*dmats1[47][14] + coeff0_48*dmats1[48][14] + coeff0_49*dmats1[49][14] + coeff0_50*dmats1[50][14] + coeff0_51*dmats1[51][14] + coeff0_52*dmats1[52][14] + coeff0_53*dmats1[53][14] + coeff0_54*dmats1[54][14] + coeff0_55*dmats1[55][14];
          new_coeff0_15 = coeff0_0*dmats1[0][15] + coeff0_1*dmats1[1][15] + coeff0_2*dmats1[2][15] + coeff0_3*dmats1[3][15] + coeff0_4*dmats1[4][15] + coeff0_5*dmats1[5][15] + coeff0_6*dmats1[6][15] + coeff0_7*dmats1[7][15] + coeff0_8*dmats1[8][15] + coeff0_9*dmats1[9][15] + coeff0_10*dmats1[10][15] + coeff0_11*dmats1[11][15] + coeff0_12*dmats1[12][15] + coeff0_13*dmats1[13][15] + coeff0_14*dmats1[14][15] + coeff0_15*dmats1[15][15] + coeff0_16*dmats1[16][15] + coeff0_17*dmats1[17][15] + coeff0_18*dmats1[18][15] + coeff0_19*dmats1[19][15] + coeff0_20*dmats1[20][15] + coeff0_21*dmats1[21][15] + coeff0_22*dmats1[22][15] + coeff0_23*dmats1[23][15] + coeff0_24*dmats1[24][15] + coeff0_25*dmats1[25][15] + coeff0_26*dmats1[26][15] + coeff0_27*dmats1[27][15] + coeff0_28*dmats1[28][15] + coeff0_29*dmats1[29][15] + coeff0_30*dmats1[30][15] + coeff0_31*dmats1[31][15] + coeff0_32*dmats1[32][15] + coeff0_33*dmats1[33][15] + coeff0_34*dmats1[34][15] + coeff0_35*dmats1[35][15] + coeff0_36*dmats1[36][15] + coeff0_37*dmats1[37][15] + coeff0_38*dmats1[38][15] + coeff0_39*dmats1[39][15] + coeff0_40*dmats1[40][15] + coeff0_41*dmats1[41][15] + coeff0_42*dmats1[42][15] + coeff0_43*dmats1[43][15] + coeff0_44*dmats1[44][15] + coeff0_45*dmats1[45][15] + coeff0_46*dmats1[46][15] + coeff0_47*dmats1[47][15] + coeff0_48*dmats1[48][15] + coeff0_49*dmats1[49][15] + coeff0_50*dmats1[50][15] + coeff0_51*dmats1[51][15] + coeff0_52*dmats1[52][15] + coeff0_53*dmats1[53][15] + coeff0_54*dmats1[54][15] + coeff0_55*dmats1[55][15];
          new_coeff0_16 = coeff0_0*dmats1[0][16] + coeff0_1*dmats1[1][16] + coeff0_2*dmats1[2][16] + coeff0_3*dmats1[3][16] + coeff0_4*dmats1[4][16] + coeff0_5*dmats1[5][16] + coeff0_6*dmats1[6][16] + coeff0_7*dmats1[7][16] + coeff0_8*dmats1[8][16] + coeff0_9*dmats1[9][16] + coeff0_10*dmats1[10][16] + coeff0_11*dmats1[11][16] + coeff0_12*dmats1[12][16] + coeff0_13*dmats1[13][16] + coeff0_14*dmats1[14][16] + coeff0_15*dmats1[15][16] + coeff0_16*dmats1[16][16] + coeff0_17*dmats1[17][16] + coeff0_18*dmats1[18][16] + coeff0_19*dmats1[19][16] + coeff0_20*dmats1[20][16] + coeff0_21*dmats1[21][16] + coeff0_22*dmats1[22][16] + coeff0_23*dmats1[23][16] + coeff0_24*dmats1[24][16] + coeff0_25*dmats1[25][16] + coeff0_26*dmats1[26][16] + coeff0_27*dmats1[27][16] + coeff0_28*dmats1[28][16] + coeff0_29*dmats1[29][16] + coeff0_30*dmats1[30][16] + coeff0_31*dmats1[31][16] + coeff0_32*dmats1[32][16] + coeff0_33*dmats1[33][16] + coeff0_34*dmats1[34][16] + coeff0_35*dmats1[35][16] + coeff0_36*dmats1[36][16] + coeff0_37*dmats1[37][16] + coeff0_38*dmats1[38][16] + coeff0_39*dmats1[39][16] + coeff0_40*dmats1[40][16] + coeff0_41*dmats1[41][16] + coeff0_42*dmats1[42][16] + coeff0_43*dmats1[43][16] + coeff0_44*dmats1[44][16] + coeff0_45*dmats1[45][16] + coeff0_46*dmats1[46][16] + coeff0_47*dmats1[47][16] + coeff0_48*dmats1[48][16] + coeff0_49*dmats1[49][16] + coeff0_50*dmats1[50][16] + coeff0_51*dmats1[51][16] + coeff0_52*dmats1[52][16] + coeff0_53*dmats1[53][16] + coeff0_54*dmats1[54][16] + coeff0_55*dmats1[55][16];
          new_coeff0_17 = coeff0_0*dmats1[0][17] + coeff0_1*dmats1[1][17] + coeff0_2*dmats1[2][17] + coeff0_3*dmats1[3][17] + coeff0_4*dmats1[4][17] + coeff0_5*dmats1[5][17] + coeff0_6*dmats1[6][17] + coeff0_7*dmats1[7][17] + coeff0_8*dmats1[8][17] + coeff0_9*dmats1[9][17] + coeff0_10*dmats1[10][17] + coeff0_11*dmats1[11][17] + coeff0_12*dmats1[12][17] + coeff0_13*dmats1[13][17] + coeff0_14*dmats1[14][17] + coeff0_15*dmats1[15][17] + coeff0_16*dmats1[16][17] + coeff0_17*dmats1[17][17] + coeff0_18*dmats1[18][17] + coeff0_19*dmats1[19][17] + coeff0_20*dmats1[20][17] + coeff0_21*dmats1[21][17] + coeff0_22*dmats1[22][17] + coeff0_23*dmats1[23][17] + coeff0_24*dmats1[24][17] + coeff0_25*dmats1[25][17] + coeff0_26*dmats1[26][17] + coeff0_27*dmats1[27][17] + coeff0_28*dmats1[28][17] + coeff0_29*dmats1[29][17] + coeff0_30*dmats1[30][17] + coeff0_31*dmats1[31][17] + coeff0_32*dmats1[32][17] + coeff0_33*dmats1[33][17] + coeff0_34*dmats1[34][17] + coeff0_35*dmats1[35][17] + coeff0_36*dmats1[36][17] + coeff0_37*dmats1[37][17] + coeff0_38*dmats1[38][17] + coeff0_39*dmats1[39][17] + coeff0_40*dmats1[40][17] + coeff0_41*dmats1[41][17] + coeff0_42*dmats1[42][17] + coeff0_43*dmats1[43][17] + coeff0_44*dmats1[44][17] + coeff0_45*dmats1[45][17] + coeff0_46*dmats1[46][17] + coeff0_47*dmats1[47][17] + coeff0_48*dmats1[48][17] + coeff0_49*dmats1[49][17] + coeff0_50*dmats1[50][17] + coeff0_51*dmats1[51][17] + coeff0_52*dmats1[52][17] + coeff0_53*dmats1[53][17] + coeff0_54*dmats1[54][17] + coeff0_55*dmats1[55][17];
          new_coeff0_18 = coeff0_0*dmats1[0][18] + coeff0_1*dmats1[1][18] + coeff0_2*dmats1[2][18] + coeff0_3*dmats1[3][18] + coeff0_4*dmats1[4][18] + coeff0_5*dmats1[5][18] + coeff0_6*dmats1[6][18] + coeff0_7*dmats1[7][18] + coeff0_8*dmats1[8][18] + coeff0_9*dmats1[9][18] + coeff0_10*dmats1[10][18] + coeff0_11*dmats1[11][18] + coeff0_12*dmats1[12][18] + coeff0_13*dmats1[13][18] + coeff0_14*dmats1[14][18] + coeff0_15*dmats1[15][18] + coeff0_16*dmats1[16][18] + coeff0_17*dmats1[17][18] + coeff0_18*dmats1[18][18] + coeff0_19*dmats1[19][18] + coeff0_20*dmats1[20][18] + coeff0_21*dmats1[21][18] + coeff0_22*dmats1[22][18] + coeff0_23*dmats1[23][18] + coeff0_24*dmats1[24][18] + coeff0_25*dmats1[25][18] + coeff0_26*dmats1[26][18] + coeff0_27*dmats1[27][18] + coeff0_28*dmats1[28][18] + coeff0_29*dmats1[29][18] + coeff0_30*dmats1[30][18] + coeff0_31*dmats1[31][18] + coeff0_32*dmats1[32][18] + coeff0_33*dmats1[33][18] + coeff0_34*dmats1[34][18] + coeff0_35*dmats1[35][18] + coeff0_36*dmats1[36][18] + coeff0_37*dmats1[37][18] + coeff0_38*dmats1[38][18] + coeff0_39*dmats1[39][18] + coeff0_40*dmats1[40][18] + coeff0_41*dmats1[41][18] + coeff0_42*dmats1[42][18] + coeff0_43*dmats1[43][18] + coeff0_44*dmats1[44][18] + coeff0_45*dmats1[45][18] + coeff0_46*dmats1[46][18] + coeff0_47*dmats1[47][18] + coeff0_48*dmats1[48][18] + coeff0_49*dmats1[49][18] + coeff0_50*dmats1[50][18] + coeff0_51*dmats1[51][18] + coeff0_52*dmats1[52][18] + coeff0_53*dmats1[53][18] + coeff0_54*dmats1[54][18] + coeff0_55*dmats1[55][18];
          new_coeff0_19 = coeff0_0*dmats1[0][19] + coeff0_1*dmats1[1][19] + coeff0_2*dmats1[2][19] + coeff0_3*dmats1[3][19] + coeff0_4*dmats1[4][19] + coeff0_5*dmats1[5][19] + coeff0_6*dmats1[6][19] + coeff0_7*dmats1[7][19] + coeff0_8*dmats1[8][19] + coeff0_9*dmats1[9][19] + coeff0_10*dmats1[10][19] + coeff0_11*dmats1[11][19] + coeff0_12*dmats1[12][19] + coeff0_13*dmats1[13][19] + coeff0_14*dmats1[14][19] + coeff0_15*dmats1[15][19] + coeff0_16*dmats1[16][19] + coeff0_17*dmats1[17][19] + coeff0_18*dmats1[18][19] + coeff0_19*dmats1[19][19] + coeff0_20*dmats1[20][19] + coeff0_21*dmats1[21][19] + coeff0_22*dmats1[22][19] + coeff0_23*dmats1[23][19] + coeff0_24*dmats1[24][19] + coeff0_25*dmats1[25][19] + coeff0_26*dmats1[26][19] + coeff0_27*dmats1[27][19] + coeff0_28*dmats1[28][19] + coeff0_29*dmats1[29][19] + coeff0_30*dmats1[30][19] + coeff0_31*dmats1[31][19] + coeff0_32*dmats1[32][19] + coeff0_33*dmats1[33][19] + coeff0_34*dmats1[34][19] + coeff0_35*dmats1[35][19] + coeff0_36*dmats1[36][19] + coeff0_37*dmats1[37][19] + coeff0_38*dmats1[38][19] + coeff0_39*dmats1[39][19] + coeff0_40*dmats1[40][19] + coeff0_41*dmats1[41][19] + coeff0_42*dmats1[42][19] + coeff0_43*dmats1[43][19] + coeff0_44*dmats1[44][19] + coeff0_45*dmats1[45][19] + coeff0_46*dmats1[46][19] + coeff0_47*dmats1[47][19] + coeff0_48*dmats1[48][19] + coeff0_49*dmats1[49][19] + coeff0_50*dmats1[50][19] + coeff0_51*dmats1[51][19] + coeff0_52*dmats1[52][19] + coeff0_53*dmats1[53][19] + coeff0_54*dmats1[54][19] + coeff0_55*dmats1[55][19];
          new_coeff0_20 = coeff0_0*dmats1[0][20] + coeff0_1*dmats1[1][20] + coeff0_2*dmats1[2][20] + coeff0_3*dmats1[3][20] + coeff0_4*dmats1[4][20] + coeff0_5*dmats1[5][20] + coeff0_6*dmats1[6][20] + coeff0_7*dmats1[7][20] + coeff0_8*dmats1[8][20] + coeff0_9*dmats1[9][20] + coeff0_10*dmats1[10][20] + coeff0_11*dmats1[11][20] + coeff0_12*dmats1[12][20] + coeff0_13*dmats1[13][20] + coeff0_14*dmats1[14][20] + coeff0_15*dmats1[15][20] + coeff0_16*dmats1[16][20] + coeff0_17*dmats1[17][20] + coeff0_18*dmats1[18][20] + coeff0_19*dmats1[19][20] + coeff0_20*dmats1[20][20] + coeff0_21*dmats1[21][20] + coeff0_22*dmats1[22][20] + coeff0_23*dmats1[23][20] + coeff0_24*dmats1[24][20] + coeff0_25*dmats1[25][20] + coeff0_26*dmats1[26][20] + coeff0_27*dmats1[27][20] + coeff0_28*dmats1[28][20] + coeff0_29*dmats1[29][20] + coeff0_30*dmats1[30][20] + coeff0_31*dmats1[31][20] + coeff0_32*dmats1[32][20] + coeff0_33*dmats1[33][20] + coeff0_34*dmats1[34][20] + coeff0_35*dmats1[35][20] + coeff0_36*dmats1[36][20] + coeff0_37*dmats1[37][20] + coeff0_38*dmats1[38][20] + coeff0_39*dmats1[39][20] + coeff0_40*dmats1[40][20] + coeff0_41*dmats1[41][20] + coeff0_42*dmats1[42][20] + coeff0_43*dmats1[43][20] + coeff0_44*dmats1[44][20] + coeff0_45*dmats1[45][20] + coeff0_46*dmats1[46][20] + coeff0_47*dmats1[47][20] + coeff0_48*dmats1[48][20] + coeff0_49*dmats1[49][20] + coeff0_50*dmats1[50][20] + coeff0_51*dmats1[51][20] + coeff0_52*dmats1[52][20] + coeff0_53*dmats1[53][20] + coeff0_54*dmats1[54][20] + coeff0_55*dmats1[55][20];
          new_coeff0_21 = coeff0_0*dmats1[0][21] + coeff0_1*dmats1[1][21] + coeff0_2*dmats1[2][21] + coeff0_3*dmats1[3][21] + coeff0_4*dmats1[4][21] + coeff0_5*dmats1[5][21] + coeff0_6*dmats1[6][21] + coeff0_7*dmats1[7][21] + coeff0_8*dmats1[8][21] + coeff0_9*dmats1[9][21] + coeff0_10*dmats1[10][21] + coeff0_11*dmats1[11][21] + coeff0_12*dmats1[12][21] + coeff0_13*dmats1[13][21] + coeff0_14*dmats1[14][21] + coeff0_15*dmats1[15][21] + coeff0_16*dmats1[16][21] + coeff0_17*dmats1[17][21] + coeff0_18*dmats1[18][21] + coeff0_19*dmats1[19][21] + coeff0_20*dmats1[20][21] + coeff0_21*dmats1[21][21] + coeff0_22*dmats1[22][21] + coeff0_23*dmats1[23][21] + coeff0_24*dmats1[24][21] + coeff0_25*dmats1[25][21] + coeff0_26*dmats1[26][21] + coeff0_27*dmats1[27][21] + coeff0_28*dmats1[28][21] + coeff0_29*dmats1[29][21] + coeff0_30*dmats1[30][21] + coeff0_31*dmats1[31][21] + coeff0_32*dmats1[32][21] + coeff0_33*dmats1[33][21] + coeff0_34*dmats1[34][21] + coeff0_35*dmats1[35][21] + coeff0_36*dmats1[36][21] + coeff0_37*dmats1[37][21] + coeff0_38*dmats1[38][21] + coeff0_39*dmats1[39][21] + coeff0_40*dmats1[40][21] + coeff0_41*dmats1[41][21] + coeff0_42*dmats1[42][21] + coeff0_43*dmats1[43][21] + coeff0_44*dmats1[44][21] + coeff0_45*dmats1[45][21] + coeff0_46*dmats1[46][21] + coeff0_47*dmats1[47][21] + coeff0_48*dmats1[48][21] + coeff0_49*dmats1[49][21] + coeff0_50*dmats1[50][21] + coeff0_51*dmats1[51][21] + coeff0_52*dmats1[52][21] + coeff0_53*dmats1[53][21] + coeff0_54*dmats1[54][21] + coeff0_55*dmats1[55][21];
          new_coeff0_22 = coeff0_0*dmats1[0][22] + coeff0_1*dmats1[1][22] + coeff0_2*dmats1[2][22] + coeff0_3*dmats1[3][22] + coeff0_4*dmats1[4][22] + coeff0_5*dmats1[5][22] + coeff0_6*dmats1[6][22] + coeff0_7*dmats1[7][22] + coeff0_8*dmats1[8][22] + coeff0_9*dmats1[9][22] + coeff0_10*dmats1[10][22] + coeff0_11*dmats1[11][22] + coeff0_12*dmats1[12][22] + coeff0_13*dmats1[13][22] + coeff0_14*dmats1[14][22] + coeff0_15*dmats1[15][22] + coeff0_16*dmats1[16][22] + coeff0_17*dmats1[17][22] + coeff0_18*dmats1[18][22] + coeff0_19*dmats1[19][22] + coeff0_20*dmats1[20][22] + coeff0_21*dmats1[21][22] + coeff0_22*dmats1[22][22] + coeff0_23*dmats1[23][22] + coeff0_24*dmats1[24][22] + coeff0_25*dmats1[25][22] + coeff0_26*dmats1[26][22] + coeff0_27*dmats1[27][22] + coeff0_28*dmats1[28][22] + coeff0_29*dmats1[29][22] + coeff0_30*dmats1[30][22] + coeff0_31*dmats1[31][22] + coeff0_32*dmats1[32][22] + coeff0_33*dmats1[33][22] + coeff0_34*dmats1[34][22] + coeff0_35*dmats1[35][22] + coeff0_36*dmats1[36][22] + coeff0_37*dmats1[37][22] + coeff0_38*dmats1[38][22] + coeff0_39*dmats1[39][22] + coeff0_40*dmats1[40][22] + coeff0_41*dmats1[41][22] + coeff0_42*dmats1[42][22] + coeff0_43*dmats1[43][22] + coeff0_44*dmats1[44][22] + coeff0_45*dmats1[45][22] + coeff0_46*dmats1[46][22] + coeff0_47*dmats1[47][22] + coeff0_48*dmats1[48][22] + coeff0_49*dmats1[49][22] + coeff0_50*dmats1[50][22] + coeff0_51*dmats1[51][22] + coeff0_52*dmats1[52][22] + coeff0_53*dmats1[53][22] + coeff0_54*dmats1[54][22] + coeff0_55*dmats1[55][22];
          new_coeff0_23 = coeff0_0*dmats1[0][23] + coeff0_1*dmats1[1][23] + coeff0_2*dmats1[2][23] + coeff0_3*dmats1[3][23] + coeff0_4*dmats1[4][23] + coeff0_5*dmats1[5][23] + coeff0_6*dmats1[6][23] + coeff0_7*dmats1[7][23] + coeff0_8*dmats1[8][23] + coeff0_9*dmats1[9][23] + coeff0_10*dmats1[10][23] + coeff0_11*dmats1[11][23] + coeff0_12*dmats1[12][23] + coeff0_13*dmats1[13][23] + coeff0_14*dmats1[14][23] + coeff0_15*dmats1[15][23] + coeff0_16*dmats1[16][23] + coeff0_17*dmats1[17][23] + coeff0_18*dmats1[18][23] + coeff0_19*dmats1[19][23] + coeff0_20*dmats1[20][23] + coeff0_21*dmats1[21][23] + coeff0_22*dmats1[22][23] + coeff0_23*dmats1[23][23] + coeff0_24*dmats1[24][23] + coeff0_25*dmats1[25][23] + coeff0_26*dmats1[26][23] + coeff0_27*dmats1[27][23] + coeff0_28*dmats1[28][23] + coeff0_29*dmats1[29][23] + coeff0_30*dmats1[30][23] + coeff0_31*dmats1[31][23] + coeff0_32*dmats1[32][23] + coeff0_33*dmats1[33][23] + coeff0_34*dmats1[34][23] + coeff0_35*dmats1[35][23] + coeff0_36*dmats1[36][23] + coeff0_37*dmats1[37][23] + coeff0_38*dmats1[38][23] + coeff0_39*dmats1[39][23] + coeff0_40*dmats1[40][23] + coeff0_41*dmats1[41][23] + coeff0_42*dmats1[42][23] + coeff0_43*dmats1[43][23] + coeff0_44*dmats1[44][23] + coeff0_45*dmats1[45][23] + coeff0_46*dmats1[46][23] + coeff0_47*dmats1[47][23] + coeff0_48*dmats1[48][23] + coeff0_49*dmats1[49][23] + coeff0_50*dmats1[50][23] + coeff0_51*dmats1[51][23] + coeff0_52*dmats1[52][23] + coeff0_53*dmats1[53][23] + coeff0_54*dmats1[54][23] + coeff0_55*dmats1[55][23];
          new_coeff0_24 = coeff0_0*dmats1[0][24] + coeff0_1*dmats1[1][24] + coeff0_2*dmats1[2][24] + coeff0_3*dmats1[3][24] + coeff0_4*dmats1[4][24] + coeff0_5*dmats1[5][24] + coeff0_6*dmats1[6][24] + coeff0_7*dmats1[7][24] + coeff0_8*dmats1[8][24] + coeff0_9*dmats1[9][24] + coeff0_10*dmats1[10][24] + coeff0_11*dmats1[11][24] + coeff0_12*dmats1[12][24] + coeff0_13*dmats1[13][24] + coeff0_14*dmats1[14][24] + coeff0_15*dmats1[15][24] + coeff0_16*dmats1[16][24] + coeff0_17*dmats1[17][24] + coeff0_18*dmats1[18][24] + coeff0_19*dmats1[19][24] + coeff0_20*dmats1[20][24] + coeff0_21*dmats1[21][24] + coeff0_22*dmats1[22][24] + coeff0_23*dmats1[23][24] + coeff0_24*dmats1[24][24] + coeff0_25*dmats1[25][24] + coeff0_26*dmats1[26][24] + coeff0_27*dmats1[27][24] + coeff0_28*dmats1[28][24] + coeff0_29*dmats1[29][24] + coeff0_30*dmats1[30][24] + coeff0_31*dmats1[31][24] + coeff0_32*dmats1[32][24] + coeff0_33*dmats1[33][24] + coeff0_34*dmats1[34][24] + coeff0_35*dmats1[35][24] + coeff0_36*dmats1[36][24] + coeff0_37*dmats1[37][24] + coeff0_38*dmats1[38][24] + coeff0_39*dmats1[39][24] + coeff0_40*dmats1[40][24] + coeff0_41*dmats1[41][24] + coeff0_42*dmats1[42][24] + coeff0_43*dmats1[43][24] + coeff0_44*dmats1[44][24] + coeff0_45*dmats1[45][24] + coeff0_46*dmats1[46][24] + coeff0_47*dmats1[47][24] + coeff0_48*dmats1[48][24] + coeff0_49*dmats1[49][24] + coeff0_50*dmats1[50][24] + coeff0_51*dmats1[51][24] + coeff0_52*dmats1[52][24] + coeff0_53*dmats1[53][24] + coeff0_54*dmats1[54][24] + coeff0_55*dmats1[55][24];
          new_coeff0_25 = coeff0_0*dmats1[0][25] + coeff0_1*dmats1[1][25] + coeff0_2*dmats1[2][25] + coeff0_3*dmats1[3][25] + coeff0_4*dmats1[4][25] + coeff0_5*dmats1[5][25] + coeff0_6*dmats1[6][25] + coeff0_7*dmats1[7][25] + coeff0_8*dmats1[8][25] + coeff0_9*dmats1[9][25] + coeff0_10*dmats1[10][25] + coeff0_11*dmats1[11][25] + coeff0_12*dmats1[12][25] + coeff0_13*dmats1[13][25] + coeff0_14*dmats1[14][25] + coeff0_15*dmats1[15][25] + coeff0_16*dmats1[16][25] + coeff0_17*dmats1[17][25] + coeff0_18*dmats1[18][25] + coeff0_19*dmats1[19][25] + coeff0_20*dmats1[20][25] + coeff0_21*dmats1[21][25] + coeff0_22*dmats1[22][25] + coeff0_23*dmats1[23][25] + coeff0_24*dmats1[24][25] + coeff0_25*dmats1[25][25] + coeff0_26*dmats1[26][25] + coeff0_27*dmats1[27][25] + coeff0_28*dmats1[28][25] + coeff0_29*dmats1[29][25] + coeff0_30*dmats1[30][25] + coeff0_31*dmats1[31][25] + coeff0_32*dmats1[32][25] + coeff0_33*dmats1[33][25] + coeff0_34*dmats1[34][25] + coeff0_35*dmats1[35][25] + coeff0_36*dmats1[36][25] + coeff0_37*dmats1[37][25] + coeff0_38*dmats1[38][25] + coeff0_39*dmats1[39][25] + coeff0_40*dmats1[40][25] + coeff0_41*dmats1[41][25] + coeff0_42*dmats1[42][25] + coeff0_43*dmats1[43][25] + coeff0_44*dmats1[44][25] + coeff0_45*dmats1[45][25] + coeff0_46*dmats1[46][25] + coeff0_47*dmats1[47][25] + coeff0_48*dmats1[48][25] + coeff0_49*dmats1[49][25] + coeff0_50*dmats1[50][25] + coeff0_51*dmats1[51][25] + coeff0_52*dmats1[52][25] + coeff0_53*dmats1[53][25] + coeff0_54*dmats1[54][25] + coeff0_55*dmats1[55][25];
          new_coeff0_26 = coeff0_0*dmats1[0][26] + coeff0_1*dmats1[1][26] + coeff0_2*dmats1[2][26] + coeff0_3*dmats1[3][26] + coeff0_4*dmats1[4][26] + coeff0_5*dmats1[5][26] + coeff0_6*dmats1[6][26] + coeff0_7*dmats1[7][26] + coeff0_8*dmats1[8][26] + coeff0_9*dmats1[9][26] + coeff0_10*dmats1[10][26] + coeff0_11*dmats1[11][26] + coeff0_12*dmats1[12][26] + coeff0_13*dmats1[13][26] + coeff0_14*dmats1[14][26] + coeff0_15*dmats1[15][26] + coeff0_16*dmats1[16][26] + coeff0_17*dmats1[17][26] + coeff0_18*dmats1[18][26] + coeff0_19*dmats1[19][26] + coeff0_20*dmats1[20][26] + coeff0_21*dmats1[21][26] + coeff0_22*dmats1[22][26] + coeff0_23*dmats1[23][26] + coeff0_24*dmats1[24][26] + coeff0_25*dmats1[25][26] + coeff0_26*dmats1[26][26] + coeff0_27*dmats1[27][26] + coeff0_28*dmats1[28][26] + coeff0_29*dmats1[29][26] + coeff0_30*dmats1[30][26] + coeff0_31*dmats1[31][26] + coeff0_32*dmats1[32][26] + coeff0_33*dmats1[33][26] + coeff0_34*dmats1[34][26] + coeff0_35*dmats1[35][26] + coeff0_36*dmats1[36][26] + coeff0_37*dmats1[37][26] + coeff0_38*dmats1[38][26] + coeff0_39*dmats1[39][26] + coeff0_40*dmats1[40][26] + coeff0_41*dmats1[41][26] + coeff0_42*dmats1[42][26] + coeff0_43*dmats1[43][26] + coeff0_44*dmats1[44][26] + coeff0_45*dmats1[45][26] + coeff0_46*dmats1[46][26] + coeff0_47*dmats1[47][26] + coeff0_48*dmats1[48][26] + coeff0_49*dmats1[49][26] + coeff0_50*dmats1[50][26] + coeff0_51*dmats1[51][26] + coeff0_52*dmats1[52][26] + coeff0_53*dmats1[53][26] + coeff0_54*dmats1[54][26] + coeff0_55*dmats1[55][26];
          new_coeff0_27 = coeff0_0*dmats1[0][27] + coeff0_1*dmats1[1][27] + coeff0_2*dmats1[2][27] + coeff0_3*dmats1[3][27] + coeff0_4*dmats1[4][27] + coeff0_5*dmats1[5][27] + coeff0_6*dmats1[6][27] + coeff0_7*dmats1[7][27] + coeff0_8*dmats1[8][27] + coeff0_9*dmats1[9][27] + coeff0_10*dmats1[10][27] + coeff0_11*dmats1[11][27] + coeff0_12*dmats1[12][27] + coeff0_13*dmats1[13][27] + coeff0_14*dmats1[14][27] + coeff0_15*dmats1[15][27] + coeff0_16*dmats1[16][27] + coeff0_17*dmats1[17][27] + coeff0_18*dmats1[18][27] + coeff0_19*dmats1[19][27] + coeff0_20*dmats1[20][27] + coeff0_21*dmats1[21][27] + coeff0_22*dmats1[22][27] + coeff0_23*dmats1[23][27] + coeff0_24*dmats1[24][27] + coeff0_25*dmats1[25][27] + coeff0_26*dmats1[26][27] + coeff0_27*dmats1[27][27] + coeff0_28*dmats1[28][27] + coeff0_29*dmats1[29][27] + coeff0_30*dmats1[30][27] + coeff0_31*dmats1[31][27] + coeff0_32*dmats1[32][27] + coeff0_33*dmats1[33][27] + coeff0_34*dmats1[34][27] + coeff0_35*dmats1[35][27] + coeff0_36*dmats1[36][27] + coeff0_37*dmats1[37][27] + coeff0_38*dmats1[38][27] + coeff0_39*dmats1[39][27] + coeff0_40*dmats1[40][27] + coeff0_41*dmats1[41][27] + coeff0_42*dmats1[42][27] + coeff0_43*dmats1[43][27] + coeff0_44*dmats1[44][27] + coeff0_45*dmats1[45][27] + coeff0_46*dmats1[46][27] + coeff0_47*dmats1[47][27] + coeff0_48*dmats1[48][27] + coeff0_49*dmats1[49][27] + coeff0_50*dmats1[50][27] + coeff0_51*dmats1[51][27] + coeff0_52*dmats1[52][27] + coeff0_53*dmats1[53][27] + coeff0_54*dmats1[54][27] + coeff0_55*dmats1[55][27];
          new_coeff0_28 = coeff0_0*dmats1[0][28] + coeff0_1*dmats1[1][28] + coeff0_2*dmats1[2][28] + coeff0_3*dmats1[3][28] + coeff0_4*dmats1[4][28] + coeff0_5*dmats1[5][28] + coeff0_6*dmats1[6][28] + coeff0_7*dmats1[7][28] + coeff0_8*dmats1[8][28] + coeff0_9*dmats1[9][28] + coeff0_10*dmats1[10][28] + coeff0_11*dmats1[11][28] + coeff0_12*dmats1[12][28] + coeff0_13*dmats1[13][28] + coeff0_14*dmats1[14][28] + coeff0_15*dmats1[15][28] + coeff0_16*dmats1[16][28] + coeff0_17*dmats1[17][28] + coeff0_18*dmats1[18][28] + coeff0_19*dmats1[19][28] + coeff0_20*dmats1[20][28] + coeff0_21*dmats1[21][28] + coeff0_22*dmats1[22][28] + coeff0_23*dmats1[23][28] + coeff0_24*dmats1[24][28] + coeff0_25*dmats1[25][28] + coeff0_26*dmats1[26][28] + coeff0_27*dmats1[27][28] + coeff0_28*dmats1[28][28] + coeff0_29*dmats1[29][28] + coeff0_30*dmats1[30][28] + coeff0_31*dmats1[31][28] + coeff0_32*dmats1[32][28] + coeff0_33*dmats1[33][28] + coeff0_34*dmats1[34][28] + coeff0_35*dmats1[35][28] + coeff0_36*dmats1[36][28] + coeff0_37*dmats1[37][28] + coeff0_38*dmats1[38][28] + coeff0_39*dmats1[39][28] + coeff0_40*dmats1[40][28] + coeff0_41*dmats1[41][28] + coeff0_42*dmats1[42][28] + coeff0_43*dmats1[43][28] + coeff0_44*dmats1[44][28] + coeff0_45*dmats1[45][28] + coeff0_46*dmats1[46][28] + coeff0_47*dmats1[47][28] + coeff0_48*dmats1[48][28] + coeff0_49*dmats1[49][28] + coeff0_50*dmats1[50][28] + coeff0_51*dmats1[51][28] + coeff0_52*dmats1[52][28] + coeff0_53*dmats1[53][28] + coeff0_54*dmats1[54][28] + coeff0_55*dmats1[55][28];
          new_coeff0_29 = coeff0_0*dmats1[0][29] + coeff0_1*dmats1[1][29] + coeff0_2*dmats1[2][29] + coeff0_3*dmats1[3][29] + coeff0_4*dmats1[4][29] + coeff0_5*dmats1[5][29] + coeff0_6*dmats1[6][29] + coeff0_7*dmats1[7][29] + coeff0_8*dmats1[8][29] + coeff0_9*dmats1[9][29] + coeff0_10*dmats1[10][29] + coeff0_11*dmats1[11][29] + coeff0_12*dmats1[12][29] + coeff0_13*dmats1[13][29] + coeff0_14*dmats1[14][29] + coeff0_15*dmats1[15][29] + coeff0_16*dmats1[16][29] + coeff0_17*dmats1[17][29] + coeff0_18*dmats1[18][29] + coeff0_19*dmats1[19][29] + coeff0_20*dmats1[20][29] + coeff0_21*dmats1[21][29] + coeff0_22*dmats1[22][29] + coeff0_23*dmats1[23][29] + coeff0_24*dmats1[24][29] + coeff0_25*dmats1[25][29] + coeff0_26*dmats1[26][29] + coeff0_27*dmats1[27][29] + coeff0_28*dmats1[28][29] + coeff0_29*dmats1[29][29] + coeff0_30*dmats1[30][29] + coeff0_31*dmats1[31][29] + coeff0_32*dmats1[32][29] + coeff0_33*dmats1[33][29] + coeff0_34*dmats1[34][29] + coeff0_35*dmats1[35][29] + coeff0_36*dmats1[36][29] + coeff0_37*dmats1[37][29] + coeff0_38*dmats1[38][29] + coeff0_39*dmats1[39][29] + coeff0_40*dmats1[40][29] + coeff0_41*dmats1[41][29] + coeff0_42*dmats1[42][29] + coeff0_43*dmats1[43][29] + coeff0_44*dmats1[44][29] + coeff0_45*dmats1[45][29] + coeff0_46*dmats1[46][29] + coeff0_47*dmats1[47][29] + coeff0_48*dmats1[48][29] + coeff0_49*dmats1[49][29] + coeff0_50*dmats1[50][29] + coeff0_51*dmats1[51][29] + coeff0_52*dmats1[52][29] + coeff0_53*dmats1[53][29] + coeff0_54*dmats1[54][29] + coeff0_55*dmats1[55][29];
          new_coeff0_30 = coeff0_0*dmats1[0][30] + coeff0_1*dmats1[1][30] + coeff0_2*dmats1[2][30] + coeff0_3*dmats1[3][30] + coeff0_4*dmats1[4][30] + coeff0_5*dmats1[5][30] + coeff0_6*dmats1[6][30] + coeff0_7*dmats1[7][30] + coeff0_8*dmats1[8][30] + coeff0_9*dmats1[9][30] + coeff0_10*dmats1[10][30] + coeff0_11*dmats1[11][30] + coeff0_12*dmats1[12][30] + coeff0_13*dmats1[13][30] + coeff0_14*dmats1[14][30] + coeff0_15*dmats1[15][30] + coeff0_16*dmats1[16][30] + coeff0_17*dmats1[17][30] + coeff0_18*dmats1[18][30] + coeff0_19*dmats1[19][30] + coeff0_20*dmats1[20][30] + coeff0_21*dmats1[21][30] + coeff0_22*dmats1[22][30] + coeff0_23*dmats1[23][30] + coeff0_24*dmats1[24][30] + coeff0_25*dmats1[25][30] + coeff0_26*dmats1[26][30] + coeff0_27*dmats1[27][30] + coeff0_28*dmats1[28][30] + coeff0_29*dmats1[29][30] + coeff0_30*dmats1[30][30] + coeff0_31*dmats1[31][30] + coeff0_32*dmats1[32][30] + coeff0_33*dmats1[33][30] + coeff0_34*dmats1[34][30] + coeff0_35*dmats1[35][30] + coeff0_36*dmats1[36][30] + coeff0_37*dmats1[37][30] + coeff0_38*dmats1[38][30] + coeff0_39*dmats1[39][30] + coeff0_40*dmats1[40][30] + coeff0_41*dmats1[41][30] + coeff0_42*dmats1[42][30] + coeff0_43*dmats1[43][30] + coeff0_44*dmats1[44][30] + coeff0_45*dmats1[45][30] + coeff0_46*dmats1[46][30] + coeff0_47*dmats1[47][30] + coeff0_48*dmats1[48][30] + coeff0_49*dmats1[49][30] + coeff0_50*dmats1[50][30] + coeff0_51*dmats1[51][30] + coeff0_52*dmats1[52][30] + coeff0_53*dmats1[53][30] + coeff0_54*dmats1[54][30] + coeff0_55*dmats1[55][30];
          new_coeff0_31 = coeff0_0*dmats1[0][31] + coeff0_1*dmats1[1][31] + coeff0_2*dmats1[2][31] + coeff0_3*dmats1[3][31] + coeff0_4*dmats1[4][31] + coeff0_5*dmats1[5][31] + coeff0_6*dmats1[6][31] + coeff0_7*dmats1[7][31] + coeff0_8*dmats1[8][31] + coeff0_9*dmats1[9][31] + coeff0_10*dmats1[10][31] + coeff0_11*dmats1[11][31] + coeff0_12*dmats1[12][31] + coeff0_13*dmats1[13][31] + coeff0_14*dmats1[14][31] + coeff0_15*dmats1[15][31] + coeff0_16*dmats1[16][31] + coeff0_17*dmats1[17][31] + coeff0_18*dmats1[18][31] + coeff0_19*dmats1[19][31] + coeff0_20*dmats1[20][31] + coeff0_21*dmats1[21][31] + coeff0_22*dmats1[22][31] + coeff0_23*dmats1[23][31] + coeff0_24*dmats1[24][31] + coeff0_25*dmats1[25][31] + coeff0_26*dmats1[26][31] + coeff0_27*dmats1[27][31] + coeff0_28*dmats1[28][31] + coeff0_29*dmats1[29][31] + coeff0_30*dmats1[30][31] + coeff0_31*dmats1[31][31] + coeff0_32*dmats1[32][31] + coeff0_33*dmats1[33][31] + coeff0_34*dmats1[34][31] + coeff0_35*dmats1[35][31] + coeff0_36*dmats1[36][31] + coeff0_37*dmats1[37][31] + coeff0_38*dmats1[38][31] + coeff0_39*dmats1[39][31] + coeff0_40*dmats1[40][31] + coeff0_41*dmats1[41][31] + coeff0_42*dmats1[42][31] + coeff0_43*dmats1[43][31] + coeff0_44*dmats1[44][31] + coeff0_45*dmats1[45][31] + coeff0_46*dmats1[46][31] + coeff0_47*dmats1[47][31] + coeff0_48*dmats1[48][31] + coeff0_49*dmats1[49][31] + coeff0_50*dmats1[50][31] + coeff0_51*dmats1[51][31] + coeff0_52*dmats1[52][31] + coeff0_53*dmats1[53][31] + coeff0_54*dmats1[54][31] + coeff0_55*dmats1[55][31];
          new_coeff0_32 = coeff0_0*dmats1[0][32] + coeff0_1*dmats1[1][32] + coeff0_2*dmats1[2][32] + coeff0_3*dmats1[3][32] + coeff0_4*dmats1[4][32] + coeff0_5*dmats1[5][32] + coeff0_6*dmats1[6][32] + coeff0_7*dmats1[7][32] + coeff0_8*dmats1[8][32] + coeff0_9*dmats1[9][32] + coeff0_10*dmats1[10][32] + coeff0_11*dmats1[11][32] + coeff0_12*dmats1[12][32] + coeff0_13*dmats1[13][32] + coeff0_14*dmats1[14][32] + coeff0_15*dmats1[15][32] + coeff0_16*dmats1[16][32] + coeff0_17*dmats1[17][32] + coeff0_18*dmats1[18][32] + coeff0_19*dmats1[19][32] + coeff0_20*dmats1[20][32] + coeff0_21*dmats1[21][32] + coeff0_22*dmats1[22][32] + coeff0_23*dmats1[23][32] + coeff0_24*dmats1[24][32] + coeff0_25*dmats1[25][32] + coeff0_26*dmats1[26][32] + coeff0_27*dmats1[27][32] + coeff0_28*dmats1[28][32] + coeff0_29*dmats1[29][32] + coeff0_30*dmats1[30][32] + coeff0_31*dmats1[31][32] + coeff0_32*dmats1[32][32] + coeff0_33*dmats1[33][32] + coeff0_34*dmats1[34][32] + coeff0_35*dmats1[35][32] + coeff0_36*dmats1[36][32] + coeff0_37*dmats1[37][32] + coeff0_38*dmats1[38][32] + coeff0_39*dmats1[39][32] + coeff0_40*dmats1[40][32] + coeff0_41*dmats1[41][32] + coeff0_42*dmats1[42][32] + coeff0_43*dmats1[43][32] + coeff0_44*dmats1[44][32] + coeff0_45*dmats1[45][32] + coeff0_46*dmats1[46][32] + coeff0_47*dmats1[47][32] + coeff0_48*dmats1[48][32] + coeff0_49*dmats1[49][32] + coeff0_50*dmats1[50][32] + coeff0_51*dmats1[51][32] + coeff0_52*dmats1[52][32] + coeff0_53*dmats1[53][32] + coeff0_54*dmats1[54][32] + coeff0_55*dmats1[55][32];
          new_coeff0_33 = coeff0_0*dmats1[0][33] + coeff0_1*dmats1[1][33] + coeff0_2*dmats1[2][33] + coeff0_3*dmats1[3][33] + coeff0_4*dmats1[4][33] + coeff0_5*dmats1[5][33] + coeff0_6*dmats1[6][33] + coeff0_7*dmats1[7][33] + coeff0_8*dmats1[8][33] + coeff0_9*dmats1[9][33] + coeff0_10*dmats1[10][33] + coeff0_11*dmats1[11][33] + coeff0_12*dmats1[12][33] + coeff0_13*dmats1[13][33] + coeff0_14*dmats1[14][33] + coeff0_15*dmats1[15][33] + coeff0_16*dmats1[16][33] + coeff0_17*dmats1[17][33] + coeff0_18*dmats1[18][33] + coeff0_19*dmats1[19][33] + coeff0_20*dmats1[20][33] + coeff0_21*dmats1[21][33] + coeff0_22*dmats1[22][33] + coeff0_23*dmats1[23][33] + coeff0_24*dmats1[24][33] + coeff0_25*dmats1[25][33] + coeff0_26*dmats1[26][33] + coeff0_27*dmats1[27][33] + coeff0_28*dmats1[28][33] + coeff0_29*dmats1[29][33] + coeff0_30*dmats1[30][33] + coeff0_31*dmats1[31][33] + coeff0_32*dmats1[32][33] + coeff0_33*dmats1[33][33] + coeff0_34*dmats1[34][33] + coeff0_35*dmats1[35][33] + coeff0_36*dmats1[36][33] + coeff0_37*dmats1[37][33] + coeff0_38*dmats1[38][33] + coeff0_39*dmats1[39][33] + coeff0_40*dmats1[40][33] + coeff0_41*dmats1[41][33] + coeff0_42*dmats1[42][33] + coeff0_43*dmats1[43][33] + coeff0_44*dmats1[44][33] + coeff0_45*dmats1[45][33] + coeff0_46*dmats1[46][33] + coeff0_47*dmats1[47][33] + coeff0_48*dmats1[48][33] + coeff0_49*dmats1[49][33] + coeff0_50*dmats1[50][33] + coeff0_51*dmats1[51][33] + coeff0_52*dmats1[52][33] + coeff0_53*dmats1[53][33] + coeff0_54*dmats1[54][33] + coeff0_55*dmats1[55][33];
          new_coeff0_34 = coeff0_0*dmats1[0][34] + coeff0_1*dmats1[1][34] + coeff0_2*dmats1[2][34] + coeff0_3*dmats1[3][34] + coeff0_4*dmats1[4][34] + coeff0_5*dmats1[5][34] + coeff0_6*dmats1[6][34] + coeff0_7*dmats1[7][34] + coeff0_8*dmats1[8][34] + coeff0_9*dmats1[9][34] + coeff0_10*dmats1[10][34] + coeff0_11*dmats1[11][34] + coeff0_12*dmats1[12][34] + coeff0_13*dmats1[13][34] + coeff0_14*dmats1[14][34] + coeff0_15*dmats1[15][34] + coeff0_16*dmats1[16][34] + coeff0_17*dmats1[17][34] + coeff0_18*dmats1[18][34] + coeff0_19*dmats1[19][34] + coeff0_20*dmats1[20][34] + coeff0_21*dmats1[21][34] + coeff0_22*dmats1[22][34] + coeff0_23*dmats1[23][34] + coeff0_24*dmats1[24][34] + coeff0_25*dmats1[25][34] + coeff0_26*dmats1[26][34] + coeff0_27*dmats1[27][34] + coeff0_28*dmats1[28][34] + coeff0_29*dmats1[29][34] + coeff0_30*dmats1[30][34] + coeff0_31*dmats1[31][34] + coeff0_32*dmats1[32][34] + coeff0_33*dmats1[33][34] + coeff0_34*dmats1[34][34] + coeff0_35*dmats1[35][34] + coeff0_36*dmats1[36][34] + coeff0_37*dmats1[37][34] + coeff0_38*dmats1[38][34] + coeff0_39*dmats1[39][34] + coeff0_40*dmats1[40][34] + coeff0_41*dmats1[41][34] + coeff0_42*dmats1[42][34] + coeff0_43*dmats1[43][34] + coeff0_44*dmats1[44][34] + coeff0_45*dmats1[45][34] + coeff0_46*dmats1[46][34] + coeff0_47*dmats1[47][34] + coeff0_48*dmats1[48][34] + coeff0_49*dmats1[49][34] + coeff0_50*dmats1[50][34] + coeff0_51*dmats1[51][34] + coeff0_52*dmats1[52][34] + coeff0_53*dmats1[53][34] + coeff0_54*dmats1[54][34] + coeff0_55*dmats1[55][34];
          new_coeff0_35 = coeff0_0*dmats1[0][35] + coeff0_1*dmats1[1][35] + coeff0_2*dmats1[2][35] + coeff0_3*dmats1[3][35] + coeff0_4*dmats1[4][35] + coeff0_5*dmats1[5][35] + coeff0_6*dmats1[6][35] + coeff0_7*dmats1[7][35] + coeff0_8*dmats1[8][35] + coeff0_9*dmats1[9][35] + coeff0_10*dmats1[10][35] + coeff0_11*dmats1[11][35] + coeff0_12*dmats1[12][35] + coeff0_13*dmats1[13][35] + coeff0_14*dmats1[14][35] + coeff0_15*dmats1[15][35] + coeff0_16*dmats1[16][35] + coeff0_17*dmats1[17][35] + coeff0_18*dmats1[18][35] + coeff0_19*dmats1[19][35] + coeff0_20*dmats1[20][35] + coeff0_21*dmats1[21][35] + coeff0_22*dmats1[22][35] + coeff0_23*dmats1[23][35] + coeff0_24*dmats1[24][35] + coeff0_25*dmats1[25][35] + coeff0_26*dmats1[26][35] + coeff0_27*dmats1[27][35] + coeff0_28*dmats1[28][35] + coeff0_29*dmats1[29][35] + coeff0_30*dmats1[30][35] + coeff0_31*dmats1[31][35] + coeff0_32*dmats1[32][35] + coeff0_33*dmats1[33][35] + coeff0_34*dmats1[34][35] + coeff0_35*dmats1[35][35] + coeff0_36*dmats1[36][35] + coeff0_37*dmats1[37][35] + coeff0_38*dmats1[38][35] + coeff0_39*dmats1[39][35] + coeff0_40*dmats1[40][35] + coeff0_41*dmats1[41][35] + coeff0_42*dmats1[42][35] + coeff0_43*dmats1[43][35] + coeff0_44*dmats1[44][35] + coeff0_45*dmats1[45][35] + coeff0_46*dmats1[46][35] + coeff0_47*dmats1[47][35] + coeff0_48*dmats1[48][35] + coeff0_49*dmats1[49][35] + coeff0_50*dmats1[50][35] + coeff0_51*dmats1[51][35] + coeff0_52*dmats1[52][35] + coeff0_53*dmats1[53][35] + coeff0_54*dmats1[54][35] + coeff0_55*dmats1[55][35];
          new_coeff0_36 = coeff0_0*dmats1[0][36] + coeff0_1*dmats1[1][36] + coeff0_2*dmats1[2][36] + coeff0_3*dmats1[3][36] + coeff0_4*dmats1[4][36] + coeff0_5*dmats1[5][36] + coeff0_6*dmats1[6][36] + coeff0_7*dmats1[7][36] + coeff0_8*dmats1[8][36] + coeff0_9*dmats1[9][36] + coeff0_10*dmats1[10][36] + coeff0_11*dmats1[11][36] + coeff0_12*dmats1[12][36] + coeff0_13*dmats1[13][36] + coeff0_14*dmats1[14][36] + coeff0_15*dmats1[15][36] + coeff0_16*dmats1[16][36] + coeff0_17*dmats1[17][36] + coeff0_18*dmats1[18][36] + coeff0_19*dmats1[19][36] + coeff0_20*dmats1[20][36] + coeff0_21*dmats1[21][36] + coeff0_22*dmats1[22][36] + coeff0_23*dmats1[23][36] + coeff0_24*dmats1[24][36] + coeff0_25*dmats1[25][36] + coeff0_26*dmats1[26][36] + coeff0_27*dmats1[27][36] + coeff0_28*dmats1[28][36] + coeff0_29*dmats1[29][36] + coeff0_30*dmats1[30][36] + coeff0_31*dmats1[31][36] + coeff0_32*dmats1[32][36] + coeff0_33*dmats1[33][36] + coeff0_34*dmats1[34][36] + coeff0_35*dmats1[35][36] + coeff0_36*dmats1[36][36] + coeff0_37*dmats1[37][36] + coeff0_38*dmats1[38][36] + coeff0_39*dmats1[39][36] + coeff0_40*dmats1[40][36] + coeff0_41*dmats1[41][36] + coeff0_42*dmats1[42][36] + coeff0_43*dmats1[43][36] + coeff0_44*dmats1[44][36] + coeff0_45*dmats1[45][36] + coeff0_46*dmats1[46][36] + coeff0_47*dmats1[47][36] + coeff0_48*dmats1[48][36] + coeff0_49*dmats1[49][36] + coeff0_50*dmats1[50][36] + coeff0_51*dmats1[51][36] + coeff0_52*dmats1[52][36] + coeff0_53*dmats1[53][36] + coeff0_54*dmats1[54][36] + coeff0_55*dmats1[55][36];
          new_coeff0_37 = coeff0_0*dmats1[0][37] + coeff0_1*dmats1[1][37] + coeff0_2*dmats1[2][37] + coeff0_3*dmats1[3][37] + coeff0_4*dmats1[4][37] + coeff0_5*dmats1[5][37] + coeff0_6*dmats1[6][37] + coeff0_7*dmats1[7][37] + coeff0_8*dmats1[8][37] + coeff0_9*dmats1[9][37] + coeff0_10*dmats1[10][37] + coeff0_11*dmats1[11][37] + coeff0_12*dmats1[12][37] + coeff0_13*dmats1[13][37] + coeff0_14*dmats1[14][37] + coeff0_15*dmats1[15][37] + coeff0_16*dmats1[16][37] + coeff0_17*dmats1[17][37] + coeff0_18*dmats1[18][37] + coeff0_19*dmats1[19][37] + coeff0_20*dmats1[20][37] + coeff0_21*dmats1[21][37] + coeff0_22*dmats1[22][37] + coeff0_23*dmats1[23][37] + coeff0_24*dmats1[24][37] + coeff0_25*dmats1[25][37] + coeff0_26*dmats1[26][37] + coeff0_27*dmats1[27][37] + coeff0_28*dmats1[28][37] + coeff0_29*dmats1[29][37] + coeff0_30*dmats1[30][37] + coeff0_31*dmats1[31][37] + coeff0_32*dmats1[32][37] + coeff0_33*dmats1[33][37] + coeff0_34*dmats1[34][37] + coeff0_35*dmats1[35][37] + coeff0_36*dmats1[36][37] + coeff0_37*dmats1[37][37] + coeff0_38*dmats1[38][37] + coeff0_39*dmats1[39][37] + coeff0_40*dmats1[40][37] + coeff0_41*dmats1[41][37] + coeff0_42*dmats1[42][37] + coeff0_43*dmats1[43][37] + coeff0_44*dmats1[44][37] + coeff0_45*dmats1[45][37] + coeff0_46*dmats1[46][37] + coeff0_47*dmats1[47][37] + coeff0_48*dmats1[48][37] + coeff0_49*dmats1[49][37] + coeff0_50*dmats1[50][37] + coeff0_51*dmats1[51][37] + coeff0_52*dmats1[52][37] + coeff0_53*dmats1[53][37] + coeff0_54*dmats1[54][37] + coeff0_55*dmats1[55][37];
          new_coeff0_38 = coeff0_0*dmats1[0][38] + coeff0_1*dmats1[1][38] + coeff0_2*dmats1[2][38] + coeff0_3*dmats1[3][38] + coeff0_4*dmats1[4][38] + coeff0_5*dmats1[5][38] + coeff0_6*dmats1[6][38] + coeff0_7*dmats1[7][38] + coeff0_8*dmats1[8][38] + coeff0_9*dmats1[9][38] + coeff0_10*dmats1[10][38] + coeff0_11*dmats1[11][38] + coeff0_12*dmats1[12][38] + coeff0_13*dmats1[13][38] + coeff0_14*dmats1[14][38] + coeff0_15*dmats1[15][38] + coeff0_16*dmats1[16][38] + coeff0_17*dmats1[17][38] + coeff0_18*dmats1[18][38] + coeff0_19*dmats1[19][38] + coeff0_20*dmats1[20][38] + coeff0_21*dmats1[21][38] + coeff0_22*dmats1[22][38] + coeff0_23*dmats1[23][38] + coeff0_24*dmats1[24][38] + coeff0_25*dmats1[25][38] + coeff0_26*dmats1[26][38] + coeff0_27*dmats1[27][38] + coeff0_28*dmats1[28][38] + coeff0_29*dmats1[29][38] + coeff0_30*dmats1[30][38] + coeff0_31*dmats1[31][38] + coeff0_32*dmats1[32][38] + coeff0_33*dmats1[33][38] + coeff0_34*dmats1[34][38] + coeff0_35*dmats1[35][38] + coeff0_36*dmats1[36][38] + coeff0_37*dmats1[37][38] + coeff0_38*dmats1[38][38] + coeff0_39*dmats1[39][38] + coeff0_40*dmats1[40][38] + coeff0_41*dmats1[41][38] + coeff0_42*dmats1[42][38] + coeff0_43*dmats1[43][38] + coeff0_44*dmats1[44][38] + coeff0_45*dmats1[45][38] + coeff0_46*dmats1[46][38] + coeff0_47*dmats1[47][38] + coeff0_48*dmats1[48][38] + coeff0_49*dmats1[49][38] + coeff0_50*dmats1[50][38] + coeff0_51*dmats1[51][38] + coeff0_52*dmats1[52][38] + coeff0_53*dmats1[53][38] + coeff0_54*dmats1[54][38] + coeff0_55*dmats1[55][38];
          new_coeff0_39 = coeff0_0*dmats1[0][39] + coeff0_1*dmats1[1][39] + coeff0_2*dmats1[2][39] + coeff0_3*dmats1[3][39] + coeff0_4*dmats1[4][39] + coeff0_5*dmats1[5][39] + coeff0_6*dmats1[6][39] + coeff0_7*dmats1[7][39] + coeff0_8*dmats1[8][39] + coeff0_9*dmats1[9][39] + coeff0_10*dmats1[10][39] + coeff0_11*dmats1[11][39] + coeff0_12*dmats1[12][39] + coeff0_13*dmats1[13][39] + coeff0_14*dmats1[14][39] + coeff0_15*dmats1[15][39] + coeff0_16*dmats1[16][39] + coeff0_17*dmats1[17][39] + coeff0_18*dmats1[18][39] + coeff0_19*dmats1[19][39] + coeff0_20*dmats1[20][39] + coeff0_21*dmats1[21][39] + coeff0_22*dmats1[22][39] + coeff0_23*dmats1[23][39] + coeff0_24*dmats1[24][39] + coeff0_25*dmats1[25][39] + coeff0_26*dmats1[26][39] + coeff0_27*dmats1[27][39] + coeff0_28*dmats1[28][39] + coeff0_29*dmats1[29][39] + coeff0_30*dmats1[30][39] + coeff0_31*dmats1[31][39] + coeff0_32*dmats1[32][39] + coeff0_33*dmats1[33][39] + coeff0_34*dmats1[34][39] + coeff0_35*dmats1[35][39] + coeff0_36*dmats1[36][39] + coeff0_37*dmats1[37][39] + coeff0_38*dmats1[38][39] + coeff0_39*dmats1[39][39] + coeff0_40*dmats1[40][39] + coeff0_41*dmats1[41][39] + coeff0_42*dmats1[42][39] + coeff0_43*dmats1[43][39] + coeff0_44*dmats1[44][39] + coeff0_45*dmats1[45][39] + coeff0_46*dmats1[46][39] + coeff0_47*dmats1[47][39] + coeff0_48*dmats1[48][39] + coeff0_49*dmats1[49][39] + coeff0_50*dmats1[50][39] + coeff0_51*dmats1[51][39] + coeff0_52*dmats1[52][39] + coeff0_53*dmats1[53][39] + coeff0_54*dmats1[54][39] + coeff0_55*dmats1[55][39];
          new_coeff0_40 = coeff0_0*dmats1[0][40] + coeff0_1*dmats1[1][40] + coeff0_2*dmats1[2][40] + coeff0_3*dmats1[3][40] + coeff0_4*dmats1[4][40] + coeff0_5*dmats1[5][40] + coeff0_6*dmats1[6][40] + coeff0_7*dmats1[7][40] + coeff0_8*dmats1[8][40] + coeff0_9*dmats1[9][40] + coeff0_10*dmats1[10][40] + coeff0_11*dmats1[11][40] + coeff0_12*dmats1[12][40] + coeff0_13*dmats1[13][40] + coeff0_14*dmats1[14][40] + coeff0_15*dmats1[15][40] + coeff0_16*dmats1[16][40] + coeff0_17*dmats1[17][40] + coeff0_18*dmats1[18][40] + coeff0_19*dmats1[19][40] + coeff0_20*dmats1[20][40] + coeff0_21*dmats1[21][40] + coeff0_22*dmats1[22][40] + coeff0_23*dmats1[23][40] + coeff0_24*dmats1[24][40] + coeff0_25*dmats1[25][40] + coeff0_26*dmats1[26][40] + coeff0_27*dmats1[27][40] + coeff0_28*dmats1[28][40] + coeff0_29*dmats1[29][40] + coeff0_30*dmats1[30][40] + coeff0_31*dmats1[31][40] + coeff0_32*dmats1[32][40] + coeff0_33*dmats1[33][40] + coeff0_34*dmats1[34][40] + coeff0_35*dmats1[35][40] + coeff0_36*dmats1[36][40] + coeff0_37*dmats1[37][40] + coeff0_38*dmats1[38][40] + coeff0_39*dmats1[39][40] + coeff0_40*dmats1[40][40] + coeff0_41*dmats1[41][40] + coeff0_42*dmats1[42][40] + coeff0_43*dmats1[43][40] + coeff0_44*dmats1[44][40] + coeff0_45*dmats1[45][40] + coeff0_46*dmats1[46][40] + coeff0_47*dmats1[47][40] + coeff0_48*dmats1[48][40] + coeff0_49*dmats1[49][40] + coeff0_50*dmats1[50][40] + coeff0_51*dmats1[51][40] + coeff0_52*dmats1[52][40] + coeff0_53*dmats1[53][40] + coeff0_54*dmats1[54][40] + coeff0_55*dmats1[55][40];
          new_coeff0_41 = coeff0_0*dmats1[0][41] + coeff0_1*dmats1[1][41] + coeff0_2*dmats1[2][41] + coeff0_3*dmats1[3][41] + coeff0_4*dmats1[4][41] + coeff0_5*dmats1[5][41] + coeff0_6*dmats1[6][41] + coeff0_7*dmats1[7][41] + coeff0_8*dmats1[8][41] + coeff0_9*dmats1[9][41] + coeff0_10*dmats1[10][41] + coeff0_11*dmats1[11][41] + coeff0_12*dmats1[12][41] + coeff0_13*dmats1[13][41] + coeff0_14*dmats1[14][41] + coeff0_15*dmats1[15][41] + coeff0_16*dmats1[16][41] + coeff0_17*dmats1[17][41] + coeff0_18*dmats1[18][41] + coeff0_19*dmats1[19][41] + coeff0_20*dmats1[20][41] + coeff0_21*dmats1[21][41] + coeff0_22*dmats1[22][41] + coeff0_23*dmats1[23][41] + coeff0_24*dmats1[24][41] + coeff0_25*dmats1[25][41] + coeff0_26*dmats1[26][41] + coeff0_27*dmats1[27][41] + coeff0_28*dmats1[28][41] + coeff0_29*dmats1[29][41] + coeff0_30*dmats1[30][41] + coeff0_31*dmats1[31][41] + coeff0_32*dmats1[32][41] + coeff0_33*dmats1[33][41] + coeff0_34*dmats1[34][41] + coeff0_35*dmats1[35][41] + coeff0_36*dmats1[36][41] + coeff0_37*dmats1[37][41] + coeff0_38*dmats1[38][41] + coeff0_39*dmats1[39][41] + coeff0_40*dmats1[40][41] + coeff0_41*dmats1[41][41] + coeff0_42*dmats1[42][41] + coeff0_43*dmats1[43][41] + coeff0_44*dmats1[44][41] + coeff0_45*dmats1[45][41] + coeff0_46*dmats1[46][41] + coeff0_47*dmats1[47][41] + coeff0_48*dmats1[48][41] + coeff0_49*dmats1[49][41] + coeff0_50*dmats1[50][41] + coeff0_51*dmats1[51][41] + coeff0_52*dmats1[52][41] + coeff0_53*dmats1[53][41] + coeff0_54*dmats1[54][41] + coeff0_55*dmats1[55][41];
          new_coeff0_42 = coeff0_0*dmats1[0][42] + coeff0_1*dmats1[1][42] + coeff0_2*dmats1[2][42] + coeff0_3*dmats1[3][42] + coeff0_4*dmats1[4][42] + coeff0_5*dmats1[5][42] + coeff0_6*dmats1[6][42] + coeff0_7*dmats1[7][42] + coeff0_8*dmats1[8][42] + coeff0_9*dmats1[9][42] + coeff0_10*dmats1[10][42] + coeff0_11*dmats1[11][42] + coeff0_12*dmats1[12][42] + coeff0_13*dmats1[13][42] + coeff0_14*dmats1[14][42] + coeff0_15*dmats1[15][42] + coeff0_16*dmats1[16][42] + coeff0_17*dmats1[17][42] + coeff0_18*dmats1[18][42] + coeff0_19*dmats1[19][42] + coeff0_20*dmats1[20][42] + coeff0_21*dmats1[21][42] + coeff0_22*dmats1[22][42] + coeff0_23*dmats1[23][42] + coeff0_24*dmats1[24][42] + coeff0_25*dmats1[25][42] + coeff0_26*dmats1[26][42] + coeff0_27*dmats1[27][42] + coeff0_28*dmats1[28][42] + coeff0_29*dmats1[29][42] + coeff0_30*dmats1[30][42] + coeff0_31*dmats1[31][42] + coeff0_32*dmats1[32][42] + coeff0_33*dmats1[33][42] + coeff0_34*dmats1[34][42] + coeff0_35*dmats1[35][42] + coeff0_36*dmats1[36][42] + coeff0_37*dmats1[37][42] + coeff0_38*dmats1[38][42] + coeff0_39*dmats1[39][42] + coeff0_40*dmats1[40][42] + coeff0_41*dmats1[41][42] + coeff0_42*dmats1[42][42] + coeff0_43*dmats1[43][42] + coeff0_44*dmats1[44][42] + coeff0_45*dmats1[45][42] + coeff0_46*dmats1[46][42] + coeff0_47*dmats1[47][42] + coeff0_48*dmats1[48][42] + coeff0_49*dmats1[49][42] + coeff0_50*dmats1[50][42] + coeff0_51*dmats1[51][42] + coeff0_52*dmats1[52][42] + coeff0_53*dmats1[53][42] + coeff0_54*dmats1[54][42] + coeff0_55*dmats1[55][42];
          new_coeff0_43 = coeff0_0*dmats1[0][43] + coeff0_1*dmats1[1][43] + coeff0_2*dmats1[2][43] + coeff0_3*dmats1[3][43] + coeff0_4*dmats1[4][43] + coeff0_5*dmats1[5][43] + coeff0_6*dmats1[6][43] + coeff0_7*dmats1[7][43] + coeff0_8*dmats1[8][43] + coeff0_9*dmats1[9][43] + coeff0_10*dmats1[10][43] + coeff0_11*dmats1[11][43] + coeff0_12*dmats1[12][43] + coeff0_13*dmats1[13][43] + coeff0_14*dmats1[14][43] + coeff0_15*dmats1[15][43] + coeff0_16*dmats1[16][43] + coeff0_17*dmats1[17][43] + coeff0_18*dmats1[18][43] + coeff0_19*dmats1[19][43] + coeff0_20*dmats1[20][43] + coeff0_21*dmats1[21][43] + coeff0_22*dmats1[22][43] + coeff0_23*dmats1[23][43] + coeff0_24*dmats1[24][43] + coeff0_25*dmats1[25][43] + coeff0_26*dmats1[26][43] + coeff0_27*dmats1[27][43] + coeff0_28*dmats1[28][43] + coeff0_29*dmats1[29][43] + coeff0_30*dmats1[30][43] + coeff0_31*dmats1[31][43] + coeff0_32*dmats1[32][43] + coeff0_33*dmats1[33][43] + coeff0_34*dmats1[34][43] + coeff0_35*dmats1[35][43] + coeff0_36*dmats1[36][43] + coeff0_37*dmats1[37][43] + coeff0_38*dmats1[38][43] + coeff0_39*dmats1[39][43] + coeff0_40*dmats1[40][43] + coeff0_41*dmats1[41][43] + coeff0_42*dmats1[42][43] + coeff0_43*dmats1[43][43] + coeff0_44*dmats1[44][43] + coeff0_45*dmats1[45][43] + coeff0_46*dmats1[46][43] + coeff0_47*dmats1[47][43] + coeff0_48*dmats1[48][43] + coeff0_49*dmats1[49][43] + coeff0_50*dmats1[50][43] + coeff0_51*dmats1[51][43] + coeff0_52*dmats1[52][43] + coeff0_53*dmats1[53][43] + coeff0_54*dmats1[54][43] + coeff0_55*dmats1[55][43];
          new_coeff0_44 = coeff0_0*dmats1[0][44] + coeff0_1*dmats1[1][44] + coeff0_2*dmats1[2][44] + coeff0_3*dmats1[3][44] + coeff0_4*dmats1[4][44] + coeff0_5*dmats1[5][44] + coeff0_6*dmats1[6][44] + coeff0_7*dmats1[7][44] + coeff0_8*dmats1[8][44] + coeff0_9*dmats1[9][44] + coeff0_10*dmats1[10][44] + coeff0_11*dmats1[11][44] + coeff0_12*dmats1[12][44] + coeff0_13*dmats1[13][44] + coeff0_14*dmats1[14][44] + coeff0_15*dmats1[15][44] + coeff0_16*dmats1[16][44] + coeff0_17*dmats1[17][44] + coeff0_18*dmats1[18][44] + coeff0_19*dmats1[19][44] + coeff0_20*dmats1[20][44] + coeff0_21*dmats1[21][44] + coeff0_22*dmats1[22][44] + coeff0_23*dmats1[23][44] + coeff0_24*dmats1[24][44] + coeff0_25*dmats1[25][44] + coeff0_26*dmats1[26][44] + coeff0_27*dmats1[27][44] + coeff0_28*dmats1[28][44] + coeff0_29*dmats1[29][44] + coeff0_30*dmats1[30][44] + coeff0_31*dmats1[31][44] + coeff0_32*dmats1[32][44] + coeff0_33*dmats1[33][44] + coeff0_34*dmats1[34][44] + coeff0_35*dmats1[35][44] + coeff0_36*dmats1[36][44] + coeff0_37*dmats1[37][44] + coeff0_38*dmats1[38][44] + coeff0_39*dmats1[39][44] + coeff0_40*dmats1[40][44] + coeff0_41*dmats1[41][44] + coeff0_42*dmats1[42][44] + coeff0_43*dmats1[43][44] + coeff0_44*dmats1[44][44] + coeff0_45*dmats1[45][44] + coeff0_46*dmats1[46][44] + coeff0_47*dmats1[47][44] + coeff0_48*dmats1[48][44] + coeff0_49*dmats1[49][44] + coeff0_50*dmats1[50][44] + coeff0_51*dmats1[51][44] + coeff0_52*dmats1[52][44] + coeff0_53*dmats1[53][44] + coeff0_54*dmats1[54][44] + coeff0_55*dmats1[55][44];
          new_coeff0_45 = coeff0_0*dmats1[0][45] + coeff0_1*dmats1[1][45] + coeff0_2*dmats1[2][45] + coeff0_3*dmats1[3][45] + coeff0_4*dmats1[4][45] + coeff0_5*dmats1[5][45] + coeff0_6*dmats1[6][45] + coeff0_7*dmats1[7][45] + coeff0_8*dmats1[8][45] + coeff0_9*dmats1[9][45] + coeff0_10*dmats1[10][45] + coeff0_11*dmats1[11][45] + coeff0_12*dmats1[12][45] + coeff0_13*dmats1[13][45] + coeff0_14*dmats1[14][45] + coeff0_15*dmats1[15][45] + coeff0_16*dmats1[16][45] + coeff0_17*dmats1[17][45] + coeff0_18*dmats1[18][45] + coeff0_19*dmats1[19][45] + coeff0_20*dmats1[20][45] + coeff0_21*dmats1[21][45] + coeff0_22*dmats1[22][45] + coeff0_23*dmats1[23][45] + coeff0_24*dmats1[24][45] + coeff0_25*dmats1[25][45] + coeff0_26*dmats1[26][45] + coeff0_27*dmats1[27][45] + coeff0_28*dmats1[28][45] + coeff0_29*dmats1[29][45] + coeff0_30*dmats1[30][45] + coeff0_31*dmats1[31][45] + coeff0_32*dmats1[32][45] + coeff0_33*dmats1[33][45] + coeff0_34*dmats1[34][45] + coeff0_35*dmats1[35][45] + coeff0_36*dmats1[36][45] + coeff0_37*dmats1[37][45] + coeff0_38*dmats1[38][45] + coeff0_39*dmats1[39][45] + coeff0_40*dmats1[40][45] + coeff0_41*dmats1[41][45] + coeff0_42*dmats1[42][45] + coeff0_43*dmats1[43][45] + coeff0_44*dmats1[44][45] + coeff0_45*dmats1[45][45] + coeff0_46*dmats1[46][45] + coeff0_47*dmats1[47][45] + coeff0_48*dmats1[48][45] + coeff0_49*dmats1[49][45] + coeff0_50*dmats1[50][45] + coeff0_51*dmats1[51][45] + coeff0_52*dmats1[52][45] + coeff0_53*dmats1[53][45] + coeff0_54*dmats1[54][45] + coeff0_55*dmats1[55][45];
          new_coeff0_46 = coeff0_0*dmats1[0][46] + coeff0_1*dmats1[1][46] + coeff0_2*dmats1[2][46] + coeff0_3*dmats1[3][46] + coeff0_4*dmats1[4][46] + coeff0_5*dmats1[5][46] + coeff0_6*dmats1[6][46] + coeff0_7*dmats1[7][46] + coeff0_8*dmats1[8][46] + coeff0_9*dmats1[9][46] + coeff0_10*dmats1[10][46] + coeff0_11*dmats1[11][46] + coeff0_12*dmats1[12][46] + coeff0_13*dmats1[13][46] + coeff0_14*dmats1[14][46] + coeff0_15*dmats1[15][46] + coeff0_16*dmats1[16][46] + coeff0_17*dmats1[17][46] + coeff0_18*dmats1[18][46] + coeff0_19*dmats1[19][46] + coeff0_20*dmats1[20][46] + coeff0_21*dmats1[21][46] + coeff0_22*dmats1[22][46] + coeff0_23*dmats1[23][46] + coeff0_24*dmats1[24][46] + coeff0_25*dmats1[25][46] + coeff0_26*dmats1[26][46] + coeff0_27*dmats1[27][46] + coeff0_28*dmats1[28][46] + coeff0_29*dmats1[29][46] + coeff0_30*dmats1[30][46] + coeff0_31*dmats1[31][46] + coeff0_32*dmats1[32][46] + coeff0_33*dmats1[33][46] + coeff0_34*dmats1[34][46] + coeff0_35*dmats1[35][46] + coeff0_36*dmats1[36][46] + coeff0_37*dmats1[37][46] + coeff0_38*dmats1[38][46] + coeff0_39*dmats1[39][46] + coeff0_40*dmats1[40][46] + coeff0_41*dmats1[41][46] + coeff0_42*dmats1[42][46] + coeff0_43*dmats1[43][46] + coeff0_44*dmats1[44][46] + coeff0_45*dmats1[45][46] + coeff0_46*dmats1[46][46] + coeff0_47*dmats1[47][46] + coeff0_48*dmats1[48][46] + coeff0_49*dmats1[49][46] + coeff0_50*dmats1[50][46] + coeff0_51*dmats1[51][46] + coeff0_52*dmats1[52][46] + coeff0_53*dmats1[53][46] + coeff0_54*dmats1[54][46] + coeff0_55*dmats1[55][46];
          new_coeff0_47 = coeff0_0*dmats1[0][47] + coeff0_1*dmats1[1][47] + coeff0_2*dmats1[2][47] + coeff0_3*dmats1[3][47] + coeff0_4*dmats1[4][47] + coeff0_5*dmats1[5][47] + coeff0_6*dmats1[6][47] + coeff0_7*dmats1[7][47] + coeff0_8*dmats1[8][47] + coeff0_9*dmats1[9][47] + coeff0_10*dmats1[10][47] + coeff0_11*dmats1[11][47] + coeff0_12*dmats1[12][47] + coeff0_13*dmats1[13][47] + coeff0_14*dmats1[14][47] + coeff0_15*dmats1[15][47] + coeff0_16*dmats1[16][47] + coeff0_17*dmats1[17][47] + coeff0_18*dmats1[18][47] + coeff0_19*dmats1[19][47] + coeff0_20*dmats1[20][47] + coeff0_21*dmats1[21][47] + coeff0_22*dmats1[22][47] + coeff0_23*dmats1[23][47] + coeff0_24*dmats1[24][47] + coeff0_25*dmats1[25][47] + coeff0_26*dmats1[26][47] + coeff0_27*dmats1[27][47] + coeff0_28*dmats1[28][47] + coeff0_29*dmats1[29][47] + coeff0_30*dmats1[30][47] + coeff0_31*dmats1[31][47] + coeff0_32*dmats1[32][47] + coeff0_33*dmats1[33][47] + coeff0_34*dmats1[34][47] + coeff0_35*dmats1[35][47] + coeff0_36*dmats1[36][47] + coeff0_37*dmats1[37][47] + coeff0_38*dmats1[38][47] + coeff0_39*dmats1[39][47] + coeff0_40*dmats1[40][47] + coeff0_41*dmats1[41][47] + coeff0_42*dmats1[42][47] + coeff0_43*dmats1[43][47] + coeff0_44*dmats1[44][47] + coeff0_45*dmats1[45][47] + coeff0_46*dmats1[46][47] + coeff0_47*dmats1[47][47] + coeff0_48*dmats1[48][47] + coeff0_49*dmats1[49][47] + coeff0_50*dmats1[50][47] + coeff0_51*dmats1[51][47] + coeff0_52*dmats1[52][47] + coeff0_53*dmats1[53][47] + coeff0_54*dmats1[54][47] + coeff0_55*dmats1[55][47];
          new_coeff0_48 = coeff0_0*dmats1[0][48] + coeff0_1*dmats1[1][48] + coeff0_2*dmats1[2][48] + coeff0_3*dmats1[3][48] + coeff0_4*dmats1[4][48] + coeff0_5*dmats1[5][48] + coeff0_6*dmats1[6][48] + coeff0_7*dmats1[7][48] + coeff0_8*dmats1[8][48] + coeff0_9*dmats1[9][48] + coeff0_10*dmats1[10][48] + coeff0_11*dmats1[11][48] + coeff0_12*dmats1[12][48] + coeff0_13*dmats1[13][48] + coeff0_14*dmats1[14][48] + coeff0_15*dmats1[15][48] + coeff0_16*dmats1[16][48] + coeff0_17*dmats1[17][48] + coeff0_18*dmats1[18][48] + coeff0_19*dmats1[19][48] + coeff0_20*dmats1[20][48] + coeff0_21*dmats1[21][48] + coeff0_22*dmats1[22][48] + coeff0_23*dmats1[23][48] + coeff0_24*dmats1[24][48] + coeff0_25*dmats1[25][48] + coeff0_26*dmats1[26][48] + coeff0_27*dmats1[27][48] + coeff0_28*dmats1[28][48] + coeff0_29*dmats1[29][48] + coeff0_30*dmats1[30][48] + coeff0_31*dmats1[31][48] + coeff0_32*dmats1[32][48] + coeff0_33*dmats1[33][48] + coeff0_34*dmats1[34][48] + coeff0_35*dmats1[35][48] + coeff0_36*dmats1[36][48] + coeff0_37*dmats1[37][48] + coeff0_38*dmats1[38][48] + coeff0_39*dmats1[39][48] + coeff0_40*dmats1[40][48] + coeff0_41*dmats1[41][48] + coeff0_42*dmats1[42][48] + coeff0_43*dmats1[43][48] + coeff0_44*dmats1[44][48] + coeff0_45*dmats1[45][48] + coeff0_46*dmats1[46][48] + coeff0_47*dmats1[47][48] + coeff0_48*dmats1[48][48] + coeff0_49*dmats1[49][48] + coeff0_50*dmats1[50][48] + coeff0_51*dmats1[51][48] + coeff0_52*dmats1[52][48] + coeff0_53*dmats1[53][48] + coeff0_54*dmats1[54][48] + coeff0_55*dmats1[55][48];
          new_coeff0_49 = coeff0_0*dmats1[0][49] + coeff0_1*dmats1[1][49] + coeff0_2*dmats1[2][49] + coeff0_3*dmats1[3][49] + coeff0_4*dmats1[4][49] + coeff0_5*dmats1[5][49] + coeff0_6*dmats1[6][49] + coeff0_7*dmats1[7][49] + coeff0_8*dmats1[8][49] + coeff0_9*dmats1[9][49] + coeff0_10*dmats1[10][49] + coeff0_11*dmats1[11][49] + coeff0_12*dmats1[12][49] + coeff0_13*dmats1[13][49] + coeff0_14*dmats1[14][49] + coeff0_15*dmats1[15][49] + coeff0_16*dmats1[16][49] + coeff0_17*dmats1[17][49] + coeff0_18*dmats1[18][49] + coeff0_19*dmats1[19][49] + coeff0_20*dmats1[20][49] + coeff0_21*dmats1[21][49] + coeff0_22*dmats1[22][49] + coeff0_23*dmats1[23][49] + coeff0_24*dmats1[24][49] + coeff0_25*dmats1[25][49] + coeff0_26*dmats1[26][49] + coeff0_27*dmats1[27][49] + coeff0_28*dmats1[28][49] + coeff0_29*dmats1[29][49] + coeff0_30*dmats1[30][49] + coeff0_31*dmats1[31][49] + coeff0_32*dmats1[32][49] + coeff0_33*dmats1[33][49] + coeff0_34*dmats1[34][49] + coeff0_35*dmats1[35][49] + coeff0_36*dmats1[36][49] + coeff0_37*dmats1[37][49] + coeff0_38*dmats1[38][49] + coeff0_39*dmats1[39][49] + coeff0_40*dmats1[40][49] + coeff0_41*dmats1[41][49] + coeff0_42*dmats1[42][49] + coeff0_43*dmats1[43][49] + coeff0_44*dmats1[44][49] + coeff0_45*dmats1[45][49] + coeff0_46*dmats1[46][49] + coeff0_47*dmats1[47][49] + coeff0_48*dmats1[48][49] + coeff0_49*dmats1[49][49] + coeff0_50*dmats1[50][49] + coeff0_51*dmats1[51][49] + coeff0_52*dmats1[52][49] + coeff0_53*dmats1[53][49] + coeff0_54*dmats1[54][49] + coeff0_55*dmats1[55][49];
          new_coeff0_50 = coeff0_0*dmats1[0][50] + coeff0_1*dmats1[1][50] + coeff0_2*dmats1[2][50] + coeff0_3*dmats1[3][50] + coeff0_4*dmats1[4][50] + coeff0_5*dmats1[5][50] + coeff0_6*dmats1[6][50] + coeff0_7*dmats1[7][50] + coeff0_8*dmats1[8][50] + coeff0_9*dmats1[9][50] + coeff0_10*dmats1[10][50] + coeff0_11*dmats1[11][50] + coeff0_12*dmats1[12][50] + coeff0_13*dmats1[13][50] + coeff0_14*dmats1[14][50] + coeff0_15*dmats1[15][50] + coeff0_16*dmats1[16][50] + coeff0_17*dmats1[17][50] + coeff0_18*dmats1[18][50] + coeff0_19*dmats1[19][50] + coeff0_20*dmats1[20][50] + coeff0_21*dmats1[21][50] + coeff0_22*dmats1[22][50] + coeff0_23*dmats1[23][50] + coeff0_24*dmats1[24][50] + coeff0_25*dmats1[25][50] + coeff0_26*dmats1[26][50] + coeff0_27*dmats1[27][50] + coeff0_28*dmats1[28][50] + coeff0_29*dmats1[29][50] + coeff0_30*dmats1[30][50] + coeff0_31*dmats1[31][50] + coeff0_32*dmats1[32][50] + coeff0_33*dmats1[33][50] + coeff0_34*dmats1[34][50] + coeff0_35*dmats1[35][50] + coeff0_36*dmats1[36][50] + coeff0_37*dmats1[37][50] + coeff0_38*dmats1[38][50] + coeff0_39*dmats1[39][50] + coeff0_40*dmats1[40][50] + coeff0_41*dmats1[41][50] + coeff0_42*dmats1[42][50] + coeff0_43*dmats1[43][50] + coeff0_44*dmats1[44][50] + coeff0_45*dmats1[45][50] + coeff0_46*dmats1[46][50] + coeff0_47*dmats1[47][50] + coeff0_48*dmats1[48][50] + coeff0_49*dmats1[49][50] + coeff0_50*dmats1[50][50] + coeff0_51*dmats1[51][50] + coeff0_52*dmats1[52][50] + coeff0_53*dmats1[53][50] + coeff0_54*dmats1[54][50] + coeff0_55*dmats1[55][50];
          new_coeff0_51 = coeff0_0*dmats1[0][51] + coeff0_1*dmats1[1][51] + coeff0_2*dmats1[2][51] + coeff0_3*dmats1[3][51] + coeff0_4*dmats1[4][51] + coeff0_5*dmats1[5][51] + coeff0_6*dmats1[6][51] + coeff0_7*dmats1[7][51] + coeff0_8*dmats1[8][51] + coeff0_9*dmats1[9][51] + coeff0_10*dmats1[10][51] + coeff0_11*dmats1[11][51] + coeff0_12*dmats1[12][51] + coeff0_13*dmats1[13][51] + coeff0_14*dmats1[14][51] + coeff0_15*dmats1[15][51] + coeff0_16*dmats1[16][51] + coeff0_17*dmats1[17][51] + coeff0_18*dmats1[18][51] + coeff0_19*dmats1[19][51] + coeff0_20*dmats1[20][51] + coeff0_21*dmats1[21][51] + coeff0_22*dmats1[22][51] + coeff0_23*dmats1[23][51] + coeff0_24*dmats1[24][51] + coeff0_25*dmats1[25][51] + coeff0_26*dmats1[26][51] + coeff0_27*dmats1[27][51] + coeff0_28*dmats1[28][51] + coeff0_29*dmats1[29][51] + coeff0_30*dmats1[30][51] + coeff0_31*dmats1[31][51] + coeff0_32*dmats1[32][51] + coeff0_33*dmats1[33][51] + coeff0_34*dmats1[34][51] + coeff0_35*dmats1[35][51] + coeff0_36*dmats1[36][51] + coeff0_37*dmats1[37][51] + coeff0_38*dmats1[38][51] + coeff0_39*dmats1[39][51] + coeff0_40*dmats1[40][51] + coeff0_41*dmats1[41][51] + coeff0_42*dmats1[42][51] + coeff0_43*dmats1[43][51] + coeff0_44*dmats1[44][51] + coeff0_45*dmats1[45][51] + coeff0_46*dmats1[46][51] + coeff0_47*dmats1[47][51] + coeff0_48*dmats1[48][51] + coeff0_49*dmats1[49][51] + coeff0_50*dmats1[50][51] + coeff0_51*dmats1[51][51] + coeff0_52*dmats1[52][51] + coeff0_53*dmats1[53][51] + coeff0_54*dmats1[54][51] + coeff0_55*dmats1[55][51];
          new_coeff0_52 = coeff0_0*dmats1[0][52] + coeff0_1*dmats1[1][52] + coeff0_2*dmats1[2][52] + coeff0_3*dmats1[3][52] + coeff0_4*dmats1[4][52] + coeff0_5*dmats1[5][52] + coeff0_6*dmats1[6][52] + coeff0_7*dmats1[7][52] + coeff0_8*dmats1[8][52] + coeff0_9*dmats1[9][52] + coeff0_10*dmats1[10][52] + coeff0_11*dmats1[11][52] + coeff0_12*dmats1[12][52] + coeff0_13*dmats1[13][52] + coeff0_14*dmats1[14][52] + coeff0_15*dmats1[15][52] + coeff0_16*dmats1[16][52] + coeff0_17*dmats1[17][52] + coeff0_18*dmats1[18][52] + coeff0_19*dmats1[19][52] + coeff0_20*dmats1[20][52] + coeff0_21*dmats1[21][52] + coeff0_22*dmats1[22][52] + coeff0_23*dmats1[23][52] + coeff0_24*dmats1[24][52] + coeff0_25*dmats1[25][52] + coeff0_26*dmats1[26][52] + coeff0_27*dmats1[27][52] + coeff0_28*dmats1[28][52] + coeff0_29*dmats1[29][52] + coeff0_30*dmats1[30][52] + coeff0_31*dmats1[31][52] + coeff0_32*dmats1[32][52] + coeff0_33*dmats1[33][52] + coeff0_34*dmats1[34][52] + coeff0_35*dmats1[35][52] + coeff0_36*dmats1[36][52] + coeff0_37*dmats1[37][52] + coeff0_38*dmats1[38][52] + coeff0_39*dmats1[39][52] + coeff0_40*dmats1[40][52] + coeff0_41*dmats1[41][52] + coeff0_42*dmats1[42][52] + coeff0_43*dmats1[43][52] + coeff0_44*dmats1[44][52] + coeff0_45*dmats1[45][52] + coeff0_46*dmats1[46][52] + coeff0_47*dmats1[47][52] + coeff0_48*dmats1[48][52] + coeff0_49*dmats1[49][52] + coeff0_50*dmats1[50][52] + coeff0_51*dmats1[51][52] + coeff0_52*dmats1[52][52] + coeff0_53*dmats1[53][52] + coeff0_54*dmats1[54][52] + coeff0_55*dmats1[55][52];
          new_coeff0_53 = coeff0_0*dmats1[0][53] + coeff0_1*dmats1[1][53] + coeff0_2*dmats1[2][53] + coeff0_3*dmats1[3][53] + coeff0_4*dmats1[4][53] + coeff0_5*dmats1[5][53] + coeff0_6*dmats1[6][53] + coeff0_7*dmats1[7][53] + coeff0_8*dmats1[8][53] + coeff0_9*dmats1[9][53] + coeff0_10*dmats1[10][53] + coeff0_11*dmats1[11][53] + coeff0_12*dmats1[12][53] + coeff0_13*dmats1[13][53] + coeff0_14*dmats1[14][53] + coeff0_15*dmats1[15][53] + coeff0_16*dmats1[16][53] + coeff0_17*dmats1[17][53] + coeff0_18*dmats1[18][53] + coeff0_19*dmats1[19][53] + coeff0_20*dmats1[20][53] + coeff0_21*dmats1[21][53] + coeff0_22*dmats1[22][53] + coeff0_23*dmats1[23][53] + coeff0_24*dmats1[24][53] + coeff0_25*dmats1[25][53] + coeff0_26*dmats1[26][53] + coeff0_27*dmats1[27][53] + coeff0_28*dmats1[28][53] + coeff0_29*dmats1[29][53] + coeff0_30*dmats1[30][53] + coeff0_31*dmats1[31][53] + coeff0_32*dmats1[32][53] + coeff0_33*dmats1[33][53] + coeff0_34*dmats1[34][53] + coeff0_35*dmats1[35][53] + coeff0_36*dmats1[36][53] + coeff0_37*dmats1[37][53] + coeff0_38*dmats1[38][53] + coeff0_39*dmats1[39][53] + coeff0_40*dmats1[40][53] + coeff0_41*dmats1[41][53] + coeff0_42*dmats1[42][53] + coeff0_43*dmats1[43][53] + coeff0_44*dmats1[44][53] + coeff0_45*dmats1[45][53] + coeff0_46*dmats1[46][53] + coeff0_47*dmats1[47][53] + coeff0_48*dmats1[48][53] + coeff0_49*dmats1[49][53] + coeff0_50*dmats1[50][53] + coeff0_51*dmats1[51][53] + coeff0_52*dmats1[52][53] + coeff0_53*dmats1[53][53] + coeff0_54*dmats1[54][53] + coeff0_55*dmats1[55][53];
          new_coeff0_54 = coeff0_0*dmats1[0][54] + coeff0_1*dmats1[1][54] + coeff0_2*dmats1[2][54] + coeff0_3*dmats1[3][54] + coeff0_4*dmats1[4][54] + coeff0_5*dmats1[5][54] + coeff0_6*dmats1[6][54] + coeff0_7*dmats1[7][54] + coeff0_8*dmats1[8][54] + coeff0_9*dmats1[9][54] + coeff0_10*dmats1[10][54] + coeff0_11*dmats1[11][54] + coeff0_12*dmats1[12][54] + coeff0_13*dmats1[13][54] + coeff0_14*dmats1[14][54] + coeff0_15*dmats1[15][54] + coeff0_16*dmats1[16][54] + coeff0_17*dmats1[17][54] + coeff0_18*dmats1[18][54] + coeff0_19*dmats1[19][54] + coeff0_20*dmats1[20][54] + coeff0_21*dmats1[21][54] + coeff0_22*dmats1[22][54] + coeff0_23*dmats1[23][54] + coeff0_24*dmats1[24][54] + coeff0_25*dmats1[25][54] + coeff0_26*dmats1[26][54] + coeff0_27*dmats1[27][54] + coeff0_28*dmats1[28][54] + coeff0_29*dmats1[29][54] + coeff0_30*dmats1[30][54] + coeff0_31*dmats1[31][54] + coeff0_32*dmats1[32][54] + coeff0_33*dmats1[33][54] + coeff0_34*dmats1[34][54] + coeff0_35*dmats1[35][54] + coeff0_36*dmats1[36][54] + coeff0_37*dmats1[37][54] + coeff0_38*dmats1[38][54] + coeff0_39*dmats1[39][54] + coeff0_40*dmats1[40][54] + coeff0_41*dmats1[41][54] + coeff0_42*dmats1[42][54] + coeff0_43*dmats1[43][54] + coeff0_44*dmats1[44][54] + coeff0_45*dmats1[45][54] + coeff0_46*dmats1[46][54] + coeff0_47*dmats1[47][54] + coeff0_48*dmats1[48][54] + coeff0_49*dmats1[49][54] + coeff0_50*dmats1[50][54] + coeff0_51*dmats1[51][54] + coeff0_52*dmats1[52][54] + coeff0_53*dmats1[53][54] + coeff0_54*dmats1[54][54] + coeff0_55*dmats1[55][54];
          new_coeff0_55 = coeff0_0*dmats1[0][55] + coeff0_1*dmats1[1][55] + coeff0_2*dmats1[2][55] + coeff0_3*dmats1[3][55] + coeff0_4*dmats1[4][55] + coeff0_5*dmats1[5][55] + coeff0_6*dmats1[6][55] + coeff0_7*dmats1[7][55] + coeff0_8*dmats1[8][55] + coeff0_9*dmats1[9][55] + coeff0_10*dmats1[10][55] + coeff0_11*dmats1[11][55] + coeff0_12*dmats1[12][55] + coeff0_13*dmats1[13][55] + coeff0_14*dmats1[14][55] + coeff0_15*dmats1[15][55] + coeff0_16*dmats1[16][55] + coeff0_17*dmats1[17][55] + coeff0_18*dmats1[18][55] + coeff0_19*dmats1[19][55] + coeff0_20*dmats1[20][55] + coeff0_21*dmats1[21][55] + coeff0_22*dmats1[22][55] + coeff0_23*dmats1[23][55] + coeff0_24*dmats1[24][55] + coeff0_25*dmats1[25][55] + coeff0_26*dmats1[26][55] + coeff0_27*dmats1[27][55] + coeff0_28*dmats1[28][55] + coeff0_29*dmats1[29][55] + coeff0_30*dmats1[30][55] + coeff0_31*dmats1[31][55] + coeff0_32*dmats1[32][55] + coeff0_33*dmats1[33][55] + coeff0_34*dmats1[34][55] + coeff0_35*dmats1[35][55] + coeff0_36*dmats1[36][55] + coeff0_37*dmats1[37][55] + coeff0_38*dmats1[38][55] + coeff0_39*dmats1[39][55] + coeff0_40*dmats1[40][55] + coeff0_41*dmats1[41][55] + coeff0_42*dmats1[42][55] + coeff0_43*dmats1[43][55] + coeff0_44*dmats1[44][55] + coeff0_45*dmats1[45][55] + coeff0_46*dmats1[46][55] + coeff0_47*dmats1[47][55] + coeff0_48*dmats1[48][55] + coeff0_49*dmats1[49][55] + coeff0_50*dmats1[50][55] + coeff0_51*dmats1[51][55] + coeff0_52*dmats1[52][55] + coeff0_53*dmats1[53][55] + coeff0_54*dmats1[54][55] + coeff0_55*dmats1[55][55];
        }
        if(combinations[deriv_num][j] == 2)
        {
          new_coeff0_0 = coeff0_0*dmats2[0][0] + coeff0_1*dmats2[1][0] + coeff0_2*dmats2[2][0] + coeff0_3*dmats2[3][0] + coeff0_4*dmats2[4][0] + coeff0_5*dmats2[5][0] + coeff0_6*dmats2[6][0] + coeff0_7*dmats2[7][0] + coeff0_8*dmats2[8][0] + coeff0_9*dmats2[9][0] + coeff0_10*dmats2[10][0] + coeff0_11*dmats2[11][0] + coeff0_12*dmats2[12][0] + coeff0_13*dmats2[13][0] + coeff0_14*dmats2[14][0] + coeff0_15*dmats2[15][0] + coeff0_16*dmats2[16][0] + coeff0_17*dmats2[17][0] + coeff0_18*dmats2[18][0] + coeff0_19*dmats2[19][0] + coeff0_20*dmats2[20][0] + coeff0_21*dmats2[21][0] + coeff0_22*dmats2[22][0] + coeff0_23*dmats2[23][0] + coeff0_24*dmats2[24][0] + coeff0_25*dmats2[25][0] + coeff0_26*dmats2[26][0] + coeff0_27*dmats2[27][0] + coeff0_28*dmats2[28][0] + coeff0_29*dmats2[29][0] + coeff0_30*dmats2[30][0] + coeff0_31*dmats2[31][0] + coeff0_32*dmats2[32][0] + coeff0_33*dmats2[33][0] + coeff0_34*dmats2[34][0] + coeff0_35*dmats2[35][0] + coeff0_36*dmats2[36][0] + coeff0_37*dmats2[37][0] + coeff0_38*dmats2[38][0] + coeff0_39*dmats2[39][0] + coeff0_40*dmats2[40][0] + coeff0_41*dmats2[41][0] + coeff0_42*dmats2[42][0] + coeff0_43*dmats2[43][0] + coeff0_44*dmats2[44][0] + coeff0_45*dmats2[45][0] + coeff0_46*dmats2[46][0] + coeff0_47*dmats2[47][0] + coeff0_48*dmats2[48][0] + coeff0_49*dmats2[49][0] + coeff0_50*dmats2[50][0] + coeff0_51*dmats2[51][0] + coeff0_52*dmats2[52][0] + coeff0_53*dmats2[53][0] + coeff0_54*dmats2[54][0] + coeff0_55*dmats2[55][0];
          new_coeff0_1 = coeff0_0*dmats2[0][1] + coeff0_1*dmats2[1][1] + coeff0_2*dmats2[2][1] + coeff0_3*dmats2[3][1] + coeff0_4*dmats2[4][1] + coeff0_5*dmats2[5][1] + coeff0_6*dmats2[6][1] + coeff0_7*dmats2[7][1] + coeff0_8*dmats2[8][1] + coeff0_9*dmats2[9][1] + coeff0_10*dmats2[10][1] + coeff0_11*dmats2[11][1] + coeff0_12*dmats2[12][1] + coeff0_13*dmats2[13][1] + coeff0_14*dmats2[14][1] + coeff0_15*dmats2[15][1] + coeff0_16*dmats2[16][1] + coeff0_17*dmats2[17][1] + coeff0_18*dmats2[18][1] + coeff0_19*dmats2[19][1] + coeff0_20*dmats2[20][1] + coeff0_21*dmats2[21][1] + coeff0_22*dmats2[22][1] + coeff0_23*dmats2[23][1] + coeff0_24*dmats2[24][1] + coeff0_25*dmats2[25][1] + coeff0_26*dmats2[26][1] + coeff0_27*dmats2[27][1] + coeff0_28*dmats2[28][1] + coeff0_29*dmats2[29][1] + coeff0_30*dmats2[30][1] + coeff0_31*dmats2[31][1] + coeff0_32*dmats2[32][1] + coeff0_33*dmats2[33][1] + coeff0_34*dmats2[34][1] + coeff0_35*dmats2[35][1] + coeff0_36*dmats2[36][1] + coeff0_37*dmats2[37][1] + coeff0_38*dmats2[38][1] + coeff0_39*dmats2[39][1] + coeff0_40*dmats2[40][1] + coeff0_41*dmats2[41][1] + coeff0_42*dmats2[42][1] + coeff0_43*dmats2[43][1] + coeff0_44*dmats2[44][1] + coeff0_45*dmats2[45][1] + coeff0_46*dmats2[46][1] + coeff0_47*dmats2[47][1] + coeff0_48*dmats2[48][1] + coeff0_49*dmats2[49][1] + coeff0_50*dmats2[50][1] + coeff0_51*dmats2[51][1] + coeff0_52*dmats2[52][1] + coeff0_53*dmats2[53][1] + coeff0_54*dmats2[54][1] + coeff0_55*dmats2[55][1];
          new_coeff0_2 = coeff0_0*dmats2[0][2] + coeff0_1*dmats2[1][2] + coeff0_2*dmats2[2][2] + coeff0_3*dmats2[3][2] + coeff0_4*dmats2[4][2] + coeff0_5*dmats2[5][2] + coeff0_6*dmats2[6][2] + coeff0_7*dmats2[7][2] + coeff0_8*dmats2[8][2] + coeff0_9*dmats2[9][2] + coeff0_10*dmats2[10][2] + coeff0_11*dmats2[11][2] + coeff0_12*dmats2[12][2] + coeff0_13*dmats2[13][2] + coeff0_14*dmats2[14][2] + coeff0_15*dmats2[15][2] + coeff0_16*dmats2[16][2] + coeff0_17*dmats2[17][2] + coeff0_18*dmats2[18][2] + coeff0_19*dmats2[19][2] + coeff0_20*dmats2[20][2] + coeff0_21*dmats2[21][2] + coeff0_22*dmats2[22][2] + coeff0_23*dmats2[23][2] + coeff0_24*dmats2[24][2] + coeff0_25*dmats2[25][2] + coeff0_26*dmats2[26][2] + coeff0_27*dmats2[27][2] + coeff0_28*dmats2[28][2] + coeff0_29*dmats2[29][2] + coeff0_30*dmats2[30][2] + coeff0_31*dmats2[31][2] + coeff0_32*dmats2[32][2] + coeff0_33*dmats2[33][2] + coeff0_34*dmats2[34][2] + coeff0_35*dmats2[35][2] + coeff0_36*dmats2[36][2] + coeff0_37*dmats2[37][2] + coeff0_38*dmats2[38][2] + coeff0_39*dmats2[39][2] + coeff0_40*dmats2[40][2] + coeff0_41*dmats2[41][2] + coeff0_42*dmats2[42][2] + coeff0_43*dmats2[43][2] + coeff0_44*dmats2[44][2] + coeff0_45*dmats2[45][2] + coeff0_46*dmats2[46][2] + coeff0_47*dmats2[47][2] + coeff0_48*dmats2[48][2] + coeff0_49*dmats2[49][2] + coeff0_50*dmats2[50][2] + coeff0_51*dmats2[51][2] + coeff0_52*dmats2[52][2] + coeff0_53*dmats2[53][2] + coeff0_54*dmats2[54][2] + coeff0_55*dmats2[55][2];
          new_coeff0_3 = coeff0_0*dmats2[0][3] + coeff0_1*dmats2[1][3] + coeff0_2*dmats2[2][3] + coeff0_3*dmats2[3][3] + coeff0_4*dmats2[4][3] + coeff0_5*dmats2[5][3] + coeff0_6*dmats2[6][3] + coeff0_7*dmats2[7][3] + coeff0_8*dmats2[8][3] + coeff0_9*dmats2[9][3] + coeff0_10*dmats2[10][3] + coeff0_11*dmats2[11][3] + coeff0_12*dmats2[12][3] + coeff0_13*dmats2[13][3] + coeff0_14*dmats2[14][3] + coeff0_15*dmats2[15][3] + coeff0_16*dmats2[16][3] + coeff0_17*dmats2[17][3] + coeff0_18*dmats2[18][3] + coeff0_19*dmats2[19][3] + coeff0_20*dmats2[20][3] + coeff0_21*dmats2[21][3] + coeff0_22*dmats2[22][3] + coeff0_23*dmats2[23][3] + coeff0_24*dmats2[24][3] + coeff0_25*dmats2[25][3] + coeff0_26*dmats2[26][3] + coeff0_27*dmats2[27][3] + coeff0_28*dmats2[28][3] + coeff0_29*dmats2[29][3] + coeff0_30*dmats2[30][3] + coeff0_31*dmats2[31][3] + coeff0_32*dmats2[32][3] + coeff0_33*dmats2[33][3] + coeff0_34*dmats2[34][3] + coeff0_35*dmats2[35][3] + coeff0_36*dmats2[36][3] + coeff0_37*dmats2[37][3] + coeff0_38*dmats2[38][3] + coeff0_39*dmats2[39][3] + coeff0_40*dmats2[40][3] + coeff0_41*dmats2[41][3] + coeff0_42*dmats2[42][3] + coeff0_43*dmats2[43][3] + coeff0_44*dmats2[44][3] + coeff0_45*dmats2[45][3] + coeff0_46*dmats2[46][3] + coeff0_47*dmats2[47][3] + coeff0_48*dmats2[48][3] + coeff0_49*dmats2[49][3] + coeff0_50*dmats2[50][3] + coeff0_51*dmats2[51][3] + coeff0_52*dmats2[52][3] + coeff0_53*dmats2[53][3] + coeff0_54*dmats2[54][3] + coeff0_55*dmats2[55][3];
          new_coeff0_4 = coeff0_0*dmats2[0][4] + coeff0_1*dmats2[1][4] + coeff0_2*dmats2[2][4] + coeff0_3*dmats2[3][4] + coeff0_4*dmats2[4][4] + coeff0_5*dmats2[5][4] + coeff0_6*dmats2[6][4] + coeff0_7*dmats2[7][4] + coeff0_8*dmats2[8][4] + coeff0_9*dmats2[9][4] + coeff0_10*dmats2[10][4] + coeff0_11*dmats2[11][4] + coeff0_12*dmats2[12][4] + coeff0_13*dmats2[13][4] + coeff0_14*dmats2[14][4] + coeff0_15*dmats2[15][4] + coeff0_16*dmats2[16][4] + coeff0_17*dmats2[17][4] + coeff0_18*dmats2[18][4] + coeff0_19*dmats2[19][4] + coeff0_20*dmats2[20][4] + coeff0_21*dmats2[21][4] + coeff0_22*dmats2[22][4] + coeff0_23*dmats2[23][4] + coeff0_24*dmats2[24][4] + coeff0_25*dmats2[25][4] + coeff0_26*dmats2[26][4] + coeff0_27*dmats2[27][4] + coeff0_28*dmats2[28][4] + coeff0_29*dmats2[29][4] + coeff0_30*dmats2[30][4] + coeff0_31*dmats2[31][4] + coeff0_32*dmats2[32][4] + coeff0_33*dmats2[33][4] + coeff0_34*dmats2[34][4] + coeff0_35*dmats2[35][4] + coeff0_36*dmats2[36][4] + coeff0_37*dmats2[37][4] + coeff0_38*dmats2[38][4] + coeff0_39*dmats2[39][4] + coeff0_40*dmats2[40][4] + coeff0_41*dmats2[41][4] + coeff0_42*dmats2[42][4] + coeff0_43*dmats2[43][4] + coeff0_44*dmats2[44][4] + coeff0_45*dmats2[45][4] + coeff0_46*dmats2[46][4] + coeff0_47*dmats2[47][4] + coeff0_48*dmats2[48][4] + coeff0_49*dmats2[49][4] + coeff0_50*dmats2[50][4] + coeff0_51*dmats2[51][4] + coeff0_52*dmats2[52][4] + coeff0_53*dmats2[53][4] + coeff0_54*dmats2[54][4] + coeff0_55*dmats2[55][4];
          new_coeff0_5 = coeff0_0*dmats2[0][5] + coeff0_1*dmats2[1][5] + coeff0_2*dmats2[2][5] + coeff0_3*dmats2[3][5] + coeff0_4*dmats2[4][5] + coeff0_5*dmats2[5][5] + coeff0_6*dmats2[6][5] + coeff0_7*dmats2[7][5] + coeff0_8*dmats2[8][5] + coeff0_9*dmats2[9][5] + coeff0_10*dmats2[10][5] + coeff0_11*dmats2[11][5] + coeff0_12*dmats2[12][5] + coeff0_13*dmats2[13][5] + coeff0_14*dmats2[14][5] + coeff0_15*dmats2[15][5] + coeff0_16*dmats2[16][5] + coeff0_17*dmats2[17][5] + coeff0_18*dmats2[18][5] + coeff0_19*dmats2[19][5] + coeff0_20*dmats2[20][5] + coeff0_21*dmats2[21][5] + coeff0_22*dmats2[22][5] + coeff0_23*dmats2[23][5] + coeff0_24*dmats2[24][5] + coeff0_25*dmats2[25][5] + coeff0_26*dmats2[26][5] + coeff0_27*dmats2[27][5] + coeff0_28*dmats2[28][5] + coeff0_29*dmats2[29][5] + coeff0_30*dmats2[30][5] + coeff0_31*dmats2[31][5] + coeff0_32*dmats2[32][5] + coeff0_33*dmats2[33][5] + coeff0_34*dmats2[34][5] + coeff0_35*dmats2[35][5] + coeff0_36*dmats2[36][5] + coeff0_37*dmats2[37][5] + coeff0_38*dmats2[38][5] + coeff0_39*dmats2[39][5] + coeff0_40*dmats2[40][5] + coeff0_41*dmats2[41][5] + coeff0_42*dmats2[42][5] + coeff0_43*dmats2[43][5] + coeff0_44*dmats2[44][5] + coeff0_45*dmats2[45][5] + coeff0_46*dmats2[46][5] + coeff0_47*dmats2[47][5] + coeff0_48*dmats2[48][5] + coeff0_49*dmats2[49][5] + coeff0_50*dmats2[50][5] + coeff0_51*dmats2[51][5] + coeff0_52*dmats2[52][5] + coeff0_53*dmats2[53][5] + coeff0_54*dmats2[54][5] + coeff0_55*dmats2[55][5];
          new_coeff0_6 = coeff0_0*dmats2[0][6] + coeff0_1*dmats2[1][6] + coeff0_2*dmats2[2][6] + coeff0_3*dmats2[3][6] + coeff0_4*dmats2[4][6] + coeff0_5*dmats2[5][6] + coeff0_6*dmats2[6][6] + coeff0_7*dmats2[7][6] + coeff0_8*dmats2[8][6] + coeff0_9*dmats2[9][6] + coeff0_10*dmats2[10][6] + coeff0_11*dmats2[11][6] + coeff0_12*dmats2[12][6] + coeff0_13*dmats2[13][6] + coeff0_14*dmats2[14][6] + coeff0_15*dmats2[15][6] + coeff0_16*dmats2[16][6] + coeff0_17*dmats2[17][6] + coeff0_18*dmats2[18][6] + coeff0_19*dmats2[19][6] + coeff0_20*dmats2[20][6] + coeff0_21*dmats2[21][6] + coeff0_22*dmats2[22][6] + coeff0_23*dmats2[23][6] + coeff0_24*dmats2[24][6] + coeff0_25*dmats2[25][6] + coeff0_26*dmats2[26][6] + coeff0_27*dmats2[27][6] + coeff0_28*dmats2[28][6] + coeff0_29*dmats2[29][6] + coeff0_30*dmats2[30][6] + coeff0_31*dmats2[31][6] + coeff0_32*dmats2[32][6] + coeff0_33*dmats2[33][6] + coeff0_34*dmats2[34][6] + coeff0_35*dmats2[35][6] + coeff0_36*dmats2[36][6] + coeff0_37*dmats2[37][6] + coeff0_38*dmats2[38][6] + coeff0_39*dmats2[39][6] + coeff0_40*dmats2[40][6] + coeff0_41*dmats2[41][6] + coeff0_42*dmats2[42][6] + coeff0_43*dmats2[43][6] + coeff0_44*dmats2[44][6] + coeff0_45*dmats2[45][6] + coeff0_46*dmats2[46][6] + coeff0_47*dmats2[47][6] + coeff0_48*dmats2[48][6] + coeff0_49*dmats2[49][6] + coeff0_50*dmats2[50][6] + coeff0_51*dmats2[51][6] + coeff0_52*dmats2[52][6] + coeff0_53*dmats2[53][6] + coeff0_54*dmats2[54][6] + coeff0_55*dmats2[55][6];
          new_coeff0_7 = coeff0_0*dmats2[0][7] + coeff0_1*dmats2[1][7] + coeff0_2*dmats2[2][7] + coeff0_3*dmats2[3][7] + coeff0_4*dmats2[4][7] + coeff0_5*dmats2[5][7] + coeff0_6*dmats2[6][7] + coeff0_7*dmats2[7][7] + coeff0_8*dmats2[8][7] + coeff0_9*dmats2[9][7] + coeff0_10*dmats2[10][7] + coeff0_11*dmats2[11][7] + coeff0_12*dmats2[12][7] + coeff0_13*dmats2[13][7] + coeff0_14*dmats2[14][7] + coeff0_15*dmats2[15][7] + coeff0_16*dmats2[16][7] + coeff0_17*dmats2[17][7] + coeff0_18*dmats2[18][7] + coeff0_19*dmats2[19][7] + coeff0_20*dmats2[20][7] + coeff0_21*dmats2[21][7] + coeff0_22*dmats2[22][7] + coeff0_23*dmats2[23][7] + coeff0_24*dmats2[24][7] + coeff0_25*dmats2[25][7] + coeff0_26*dmats2[26][7] + coeff0_27*dmats2[27][7] + coeff0_28*dmats2[28][7] + coeff0_29*dmats2[29][7] + coeff0_30*dmats2[30][7] + coeff0_31*dmats2[31][7] + coeff0_32*dmats2[32][7] + coeff0_33*dmats2[33][7] + coeff0_34*dmats2[34][7] + coeff0_35*dmats2[35][7] + coeff0_36*dmats2[36][7] + coeff0_37*dmats2[37][7] + coeff0_38*dmats2[38][7] + coeff0_39*dmats2[39][7] + coeff0_40*dmats2[40][7] + coeff0_41*dmats2[41][7] + coeff0_42*dmats2[42][7] + coeff0_43*dmats2[43][7] + coeff0_44*dmats2[44][7] + coeff0_45*dmats2[45][7] + coeff0_46*dmats2[46][7] + coeff0_47*dmats2[47][7] + coeff0_48*dmats2[48][7] + coeff0_49*dmats2[49][7] + coeff0_50*dmats2[50][7] + coeff0_51*dmats2[51][7] + coeff0_52*dmats2[52][7] + coeff0_53*dmats2[53][7] + coeff0_54*dmats2[54][7] + coeff0_55*dmats2[55][7];
          new_coeff0_8 = coeff0_0*dmats2[0][8] + coeff0_1*dmats2[1][8] + coeff0_2*dmats2[2][8] + coeff0_3*dmats2[3][8] + coeff0_4*dmats2[4][8] + coeff0_5*dmats2[5][8] + coeff0_6*dmats2[6][8] + coeff0_7*dmats2[7][8] + coeff0_8*dmats2[8][8] + coeff0_9*dmats2[9][8] + coeff0_10*dmats2[10][8] + coeff0_11*dmats2[11][8] + coeff0_12*dmats2[12][8] + coeff0_13*dmats2[13][8] + coeff0_14*dmats2[14][8] + coeff0_15*dmats2[15][8] + coeff0_16*dmats2[16][8] + coeff0_17*dmats2[17][8] + coeff0_18*dmats2[18][8] + coeff0_19*dmats2[19][8] + coeff0_20*dmats2[20][8] + coeff0_21*dmats2[21][8] + coeff0_22*dmats2[22][8] + coeff0_23*dmats2[23][8] + coeff0_24*dmats2[24][8] + coeff0_25*dmats2[25][8] + coeff0_26*dmats2[26][8] + coeff0_27*dmats2[27][8] + coeff0_28*dmats2[28][8] + coeff0_29*dmats2[29][8] + coeff0_30*dmats2[30][8] + coeff0_31*dmats2[31][8] + coeff0_32*dmats2[32][8] + coeff0_33*dmats2[33][8] + coeff0_34*dmats2[34][8] + coeff0_35*dmats2[35][8] + coeff0_36*dmats2[36][8] + coeff0_37*dmats2[37][8] + coeff0_38*dmats2[38][8] + coeff0_39*dmats2[39][8] + coeff0_40*dmats2[40][8] + coeff0_41*dmats2[41][8] + coeff0_42*dmats2[42][8] + coeff0_43*dmats2[43][8] + coeff0_44*dmats2[44][8] + coeff0_45*dmats2[45][8] + coeff0_46*dmats2[46][8] + coeff0_47*dmats2[47][8] + coeff0_48*dmats2[48][8] + coeff0_49*dmats2[49][8] + coeff0_50*dmats2[50][8] + coeff0_51*dmats2[51][8] + coeff0_52*dmats2[52][8] + coeff0_53*dmats2[53][8] + coeff0_54*dmats2[54][8] + coeff0_55*dmats2[55][8];
          new_coeff0_9 = coeff0_0*dmats2[0][9] + coeff0_1*dmats2[1][9] + coeff0_2*dmats2[2][9] + coeff0_3*dmats2[3][9] + coeff0_4*dmats2[4][9] + coeff0_5*dmats2[5][9] + coeff0_6*dmats2[6][9] + coeff0_7*dmats2[7][9] + coeff0_8*dmats2[8][9] + coeff0_9*dmats2[9][9] + coeff0_10*dmats2[10][9] + coeff0_11*dmats2[11][9] + coeff0_12*dmats2[12][9] + coeff0_13*dmats2[13][9] + coeff0_14*dmats2[14][9] + coeff0_15*dmats2[15][9] + coeff0_16*dmats2[16][9] + coeff0_17*dmats2[17][9] + coeff0_18*dmats2[18][9] + coeff0_19*dmats2[19][9] + coeff0_20*dmats2[20][9] + coeff0_21*dmats2[21][9] + coeff0_22*dmats2[22][9] + coeff0_23*dmats2[23][9] + coeff0_24*dmats2[24][9] + coeff0_25*dmats2[25][9] + coeff0_26*dmats2[26][9] + coeff0_27*dmats2[27][9] + coeff0_28*dmats2[28][9] + coeff0_29*dmats2[29][9] + coeff0_30*dmats2[30][9] + coeff0_31*dmats2[31][9] + coeff0_32*dmats2[32][9] + coeff0_33*dmats2[33][9] + coeff0_34*dmats2[34][9] + coeff0_35*dmats2[35][9] + coeff0_36*dmats2[36][9] + coeff0_37*dmats2[37][9] + coeff0_38*dmats2[38][9] + coeff0_39*dmats2[39][9] + coeff0_40*dmats2[40][9] + coeff0_41*dmats2[41][9] + coeff0_42*dmats2[42][9] + coeff0_43*dmats2[43][9] + coeff0_44*dmats2[44][9] + coeff0_45*dmats2[45][9] + coeff0_46*dmats2[46][9] + coeff0_47*dmats2[47][9] + coeff0_48*dmats2[48][9] + coeff0_49*dmats2[49][9] + coeff0_50*dmats2[50][9] + coeff0_51*dmats2[51][9] + coeff0_52*dmats2[52][9] + coeff0_53*dmats2[53][9] + coeff0_54*dmats2[54][9] + coeff0_55*dmats2[55][9];
          new_coeff0_10 = coeff0_0*dmats2[0][10] + coeff0_1*dmats2[1][10] + coeff0_2*dmats2[2][10] + coeff0_3*dmats2[3][10] + coeff0_4*dmats2[4][10] + coeff0_5*dmats2[5][10] + coeff0_6*dmats2[6][10] + coeff0_7*dmats2[7][10] + coeff0_8*dmats2[8][10] + coeff0_9*dmats2[9][10] + coeff0_10*dmats2[10][10] + coeff0_11*dmats2[11][10] + coeff0_12*dmats2[12][10] + coeff0_13*dmats2[13][10] + coeff0_14*dmats2[14][10] + coeff0_15*dmats2[15][10] + coeff0_16*dmats2[16][10] + coeff0_17*dmats2[17][10] + coeff0_18*dmats2[18][10] + coeff0_19*dmats2[19][10] + coeff0_20*dmats2[20][10] + coeff0_21*dmats2[21][10] + coeff0_22*dmats2[22][10] + coeff0_23*dmats2[23][10] + coeff0_24*dmats2[24][10] + coeff0_25*dmats2[25][10] + coeff0_26*dmats2[26][10] + coeff0_27*dmats2[27][10] + coeff0_28*dmats2[28][10] + coeff0_29*dmats2[29][10] + coeff0_30*dmats2[30][10] + coeff0_31*dmats2[31][10] + coeff0_32*dmats2[32][10] + coeff0_33*dmats2[33][10] + coeff0_34*dmats2[34][10] + coeff0_35*dmats2[35][10] + coeff0_36*dmats2[36][10] + coeff0_37*dmats2[37][10] + coeff0_38*dmats2[38][10] + coeff0_39*dmats2[39][10] + coeff0_40*dmats2[40][10] + coeff0_41*dmats2[41][10] + coeff0_42*dmats2[42][10] + coeff0_43*dmats2[43][10] + coeff0_44*dmats2[44][10] + coeff0_45*dmats2[45][10] + coeff0_46*dmats2[46][10] + coeff0_47*dmats2[47][10] + coeff0_48*dmats2[48][10] + coeff0_49*dmats2[49][10] + coeff0_50*dmats2[50][10] + coeff0_51*dmats2[51][10] + coeff0_52*dmats2[52][10] + coeff0_53*dmats2[53][10] + coeff0_54*dmats2[54][10] + coeff0_55*dmats2[55][10];
          new_coeff0_11 = coeff0_0*dmats2[0][11] + coeff0_1*dmats2[1][11] + coeff0_2*dmats2[2][11] + coeff0_3*dmats2[3][11] + coeff0_4*dmats2[4][11] + coeff0_5*dmats2[5][11] + coeff0_6*dmats2[6][11] + coeff0_7*dmats2[7][11] + coeff0_8*dmats2[8][11] + coeff0_9*dmats2[9][11] + coeff0_10*dmats2[10][11] + coeff0_11*dmats2[11][11] + coeff0_12*dmats2[12][11] + coeff0_13*dmats2[13][11] + coeff0_14*dmats2[14][11] + coeff0_15*dmats2[15][11] + coeff0_16*dmats2[16][11] + coeff0_17*dmats2[17][11] + coeff0_18*dmats2[18][11] + coeff0_19*dmats2[19][11] + coeff0_20*dmats2[20][11] + coeff0_21*dmats2[21][11] + coeff0_22*dmats2[22][11] + coeff0_23*dmats2[23][11] + coeff0_24*dmats2[24][11] + coeff0_25*dmats2[25][11] + coeff0_26*dmats2[26][11] + coeff0_27*dmats2[27][11] + coeff0_28*dmats2[28][11] + coeff0_29*dmats2[29][11] + coeff0_30*dmats2[30][11] + coeff0_31*dmats2[31][11] + coeff0_32*dmats2[32][11] + coeff0_33*dmats2[33][11] + coeff0_34*dmats2[34][11] + coeff0_35*dmats2[35][11] + coeff0_36*dmats2[36][11] + coeff0_37*dmats2[37][11] + coeff0_38*dmats2[38][11] + coeff0_39*dmats2[39][11] + coeff0_40*dmats2[40][11] + coeff0_41*dmats2[41][11] + coeff0_42*dmats2[42][11] + coeff0_43*dmats2[43][11] + coeff0_44*dmats2[44][11] + coeff0_45*dmats2[45][11] + coeff0_46*dmats2[46][11] + coeff0_47*dmats2[47][11] + coeff0_48*dmats2[48][11] + coeff0_49*dmats2[49][11] + coeff0_50*dmats2[50][11] + coeff0_51*dmats2[51][11] + coeff0_52*dmats2[52][11] + coeff0_53*dmats2[53][11] + coeff0_54*dmats2[54][11] + coeff0_55*dmats2[55][11];
          new_coeff0_12 = coeff0_0*dmats2[0][12] + coeff0_1*dmats2[1][12] + coeff0_2*dmats2[2][12] + coeff0_3*dmats2[3][12] + coeff0_4*dmats2[4][12] + coeff0_5*dmats2[5][12] + coeff0_6*dmats2[6][12] + coeff0_7*dmats2[7][12] + coeff0_8*dmats2[8][12] + coeff0_9*dmats2[9][12] + coeff0_10*dmats2[10][12] + coeff0_11*dmats2[11][12] + coeff0_12*dmats2[12][12] + coeff0_13*dmats2[13][12] + coeff0_14*dmats2[14][12] + coeff0_15*dmats2[15][12] + coeff0_16*dmats2[16][12] + coeff0_17*dmats2[17][12] + coeff0_18*dmats2[18][12] + coeff0_19*dmats2[19][12] + coeff0_20*dmats2[20][12] + coeff0_21*dmats2[21][12] + coeff0_22*dmats2[22][12] + coeff0_23*dmats2[23][12] + coeff0_24*dmats2[24][12] + coeff0_25*dmats2[25][12] + coeff0_26*dmats2[26][12] + coeff0_27*dmats2[27][12] + coeff0_28*dmats2[28][12] + coeff0_29*dmats2[29][12] + coeff0_30*dmats2[30][12] + coeff0_31*dmats2[31][12] + coeff0_32*dmats2[32][12] + coeff0_33*dmats2[33][12] + coeff0_34*dmats2[34][12] + coeff0_35*dmats2[35][12] + coeff0_36*dmats2[36][12] + coeff0_37*dmats2[37][12] + coeff0_38*dmats2[38][12] + coeff0_39*dmats2[39][12] + coeff0_40*dmats2[40][12] + coeff0_41*dmats2[41][12] + coeff0_42*dmats2[42][12] + coeff0_43*dmats2[43][12] + coeff0_44*dmats2[44][12] + coeff0_45*dmats2[45][12] + coeff0_46*dmats2[46][12] + coeff0_47*dmats2[47][12] + coeff0_48*dmats2[48][12] + coeff0_49*dmats2[49][12] + coeff0_50*dmats2[50][12] + coeff0_51*dmats2[51][12] + coeff0_52*dmats2[52][12] + coeff0_53*dmats2[53][12] + coeff0_54*dmats2[54][12] + coeff0_55*dmats2[55][12];
          new_coeff0_13 = coeff0_0*dmats2[0][13] + coeff0_1*dmats2[1][13] + coeff0_2*dmats2[2][13] + coeff0_3*dmats2[3][13] + coeff0_4*dmats2[4][13] + coeff0_5*dmats2[5][13] + coeff0_6*dmats2[6][13] + coeff0_7*dmats2[7][13] + coeff0_8*dmats2[8][13] + coeff0_9*dmats2[9][13] + coeff0_10*dmats2[10][13] + coeff0_11*dmats2[11][13] + coeff0_12*dmats2[12][13] + coeff0_13*dmats2[13][13] + coeff0_14*dmats2[14][13] + coeff0_15*dmats2[15][13] + coeff0_16*dmats2[16][13] + coeff0_17*dmats2[17][13] + coeff0_18*dmats2[18][13] + coeff0_19*dmats2[19][13] + coeff0_20*dmats2[20][13] + coeff0_21*dmats2[21][13] + coeff0_22*dmats2[22][13] + coeff0_23*dmats2[23][13] + coeff0_24*dmats2[24][13] + coeff0_25*dmats2[25][13] + coeff0_26*dmats2[26][13] + coeff0_27*dmats2[27][13] + coeff0_28*dmats2[28][13] + coeff0_29*dmats2[29][13] + coeff0_30*dmats2[30][13] + coeff0_31*dmats2[31][13] + coeff0_32*dmats2[32][13] + coeff0_33*dmats2[33][13] + coeff0_34*dmats2[34][13] + coeff0_35*dmats2[35][13] + coeff0_36*dmats2[36][13] + coeff0_37*dmats2[37][13] + coeff0_38*dmats2[38][13] + coeff0_39*dmats2[39][13] + coeff0_40*dmats2[40][13] + coeff0_41*dmats2[41][13] + coeff0_42*dmats2[42][13] + coeff0_43*dmats2[43][13] + coeff0_44*dmats2[44][13] + coeff0_45*dmats2[45][13] + coeff0_46*dmats2[46][13] + coeff0_47*dmats2[47][13] + coeff0_48*dmats2[48][13] + coeff0_49*dmats2[49][13] + coeff0_50*dmats2[50][13] + coeff0_51*dmats2[51][13] + coeff0_52*dmats2[52][13] + coeff0_53*dmats2[53][13] + coeff0_54*dmats2[54][13] + coeff0_55*dmats2[55][13];
          new_coeff0_14 = coeff0_0*dmats2[0][14] + coeff0_1*dmats2[1][14] + coeff0_2*dmats2[2][14] + coeff0_3*dmats2[3][14] + coeff0_4*dmats2[4][14] + coeff0_5*dmats2[5][14] + coeff0_6*dmats2[6][14] + coeff0_7*dmats2[7][14] + coeff0_8*dmats2[8][14] + coeff0_9*dmats2[9][14] + coeff0_10*dmats2[10][14] + coeff0_11*dmats2[11][14] + coeff0_12*dmats2[12][14] + coeff0_13*dmats2[13][14] + coeff0_14*dmats2[14][14] + coeff0_15*dmats2[15][14] + coeff0_16*dmats2[16][14] + coeff0_17*dmats2[17][14] + coeff0_18*dmats2[18][14] + coeff0_19*dmats2[19][14] + coeff0_20*dmats2[20][14] + coeff0_21*dmats2[21][14] + coeff0_22*dmats2[22][14] + coeff0_23*dmats2[23][14] + coeff0_24*dmats2[24][14] + coeff0_25*dmats2[25][14] + coeff0_26*dmats2[26][14] + coeff0_27*dmats2[27][14] + coeff0_28*dmats2[28][14] + coeff0_29*dmats2[29][14] + coeff0_30*dmats2[30][14] + coeff0_31*dmats2[31][14] + coeff0_32*dmats2[32][14] + coeff0_33*dmats2[33][14] + coeff0_34*dmats2[34][14] + coeff0_35*dmats2[35][14] + coeff0_36*dmats2[36][14] + coeff0_37*dmats2[37][14] + coeff0_38*dmats2[38][14] + coeff0_39*dmats2[39][14] + coeff0_40*dmats2[40][14] + coeff0_41*dmats2[41][14] + coeff0_42*dmats2[42][14] + coeff0_43*dmats2[43][14] + coeff0_44*dmats2[44][14] + coeff0_45*dmats2[45][14] + coeff0_46*dmats2[46][14] + coeff0_47*dmats2[47][14] + coeff0_48*dmats2[48][14] + coeff0_49*dmats2[49][14] + coeff0_50*dmats2[50][14] + coeff0_51*dmats2[51][14] + coeff0_52*dmats2[52][14] + coeff0_53*dmats2[53][14] + coeff0_54*dmats2[54][14] + coeff0_55*dmats2[55][14];
          new_coeff0_15 = coeff0_0*dmats2[0][15] + coeff0_1*dmats2[1][15] + coeff0_2*dmats2[2][15] + coeff0_3*dmats2[3][15] + coeff0_4*dmats2[4][15] + coeff0_5*dmats2[5][15] + coeff0_6*dmats2[6][15] + coeff0_7*dmats2[7][15] + coeff0_8*dmats2[8][15] + coeff0_9*dmats2[9][15] + coeff0_10*dmats2[10][15] + coeff0_11*dmats2[11][15] + coeff0_12*dmats2[12][15] + coeff0_13*dmats2[13][15] + coeff0_14*dmats2[14][15] + coeff0_15*dmats2[15][15] + coeff0_16*dmats2[16][15] + coeff0_17*dmats2[17][15] + coeff0_18*dmats2[18][15] + coeff0_19*dmats2[19][15] + coeff0_20*dmats2[20][15] + coeff0_21*dmats2[21][15] + coeff0_22*dmats2[22][15] + coeff0_23*dmats2[23][15] + coeff0_24*dmats2[24][15] + coeff0_25*dmats2[25][15] + coeff0_26*dmats2[26][15] + coeff0_27*dmats2[27][15] + coeff0_28*dmats2[28][15] + coeff0_29*dmats2[29][15] + coeff0_30*dmats2[30][15] + coeff0_31*dmats2[31][15] + coeff0_32*dmats2[32][15] + coeff0_33*dmats2[33][15] + coeff0_34*dmats2[34][15] + coeff0_35*dmats2[35][15] + coeff0_36*dmats2[36][15] + coeff0_37*dmats2[37][15] + coeff0_38*dmats2[38][15] + coeff0_39*dmats2[39][15] + coeff0_40*dmats2[40][15] + coeff0_41*dmats2[41][15] + coeff0_42*dmats2[42][15] + coeff0_43*dmats2[43][15] + coeff0_44*dmats2[44][15] + coeff0_45*dmats2[45][15] + coeff0_46*dmats2[46][15] + coeff0_47*dmats2[47][15] + coeff0_48*dmats2[48][15] + coeff0_49*dmats2[49][15] + coeff0_50*dmats2[50][15] + coeff0_51*dmats2[51][15] + coeff0_52*dmats2[52][15] + coeff0_53*dmats2[53][15] + coeff0_54*dmats2[54][15] + coeff0_55*dmats2[55][15];
          new_coeff0_16 = coeff0_0*dmats2[0][16] + coeff0_1*dmats2[1][16] + coeff0_2*dmats2[2][16] + coeff0_3*dmats2[3][16] + coeff0_4*dmats2[4][16] + coeff0_5*dmats2[5][16] + coeff0_6*dmats2[6][16] + coeff0_7*dmats2[7][16] + coeff0_8*dmats2[8][16] + coeff0_9*dmats2[9][16] + coeff0_10*dmats2[10][16] + coeff0_11*dmats2[11][16] + coeff0_12*dmats2[12][16] + coeff0_13*dmats2[13][16] + coeff0_14*dmats2[14][16] + coeff0_15*dmats2[15][16] + coeff0_16*dmats2[16][16] + coeff0_17*dmats2[17][16] + coeff0_18*dmats2[18][16] + coeff0_19*dmats2[19][16] + coeff0_20*dmats2[20][16] + coeff0_21*dmats2[21][16] + coeff0_22*dmats2[22][16] + coeff0_23*dmats2[23][16] + coeff0_24*dmats2[24][16] + coeff0_25*dmats2[25][16] + coeff0_26*dmats2[26][16] + coeff0_27*dmats2[27][16] + coeff0_28*dmats2[28][16] + coeff0_29*dmats2[29][16] + coeff0_30*dmats2[30][16] + coeff0_31*dmats2[31][16] + coeff0_32*dmats2[32][16] + coeff0_33*dmats2[33][16] + coeff0_34*dmats2[34][16] + coeff0_35*dmats2[35][16] + coeff0_36*dmats2[36][16] + coeff0_37*dmats2[37][16] + coeff0_38*dmats2[38][16] + coeff0_39*dmats2[39][16] + coeff0_40*dmats2[40][16] + coeff0_41*dmats2[41][16] + coeff0_42*dmats2[42][16] + coeff0_43*dmats2[43][16] + coeff0_44*dmats2[44][16] + coeff0_45*dmats2[45][16] + coeff0_46*dmats2[46][16] + coeff0_47*dmats2[47][16] + coeff0_48*dmats2[48][16] + coeff0_49*dmats2[49][16] + coeff0_50*dmats2[50][16] + coeff0_51*dmats2[51][16] + coeff0_52*dmats2[52][16] + coeff0_53*dmats2[53][16] + coeff0_54*dmats2[54][16] + coeff0_55*dmats2[55][16];
          new_coeff0_17 = coeff0_0*dmats2[0][17] + coeff0_1*dmats2[1][17] + coeff0_2*dmats2[2][17] + coeff0_3*dmats2[3][17] + coeff0_4*dmats2[4][17] + coeff0_5*dmats2[5][17] + coeff0_6*dmats2[6][17] + coeff0_7*dmats2[7][17] + coeff0_8*dmats2[8][17] + coeff0_9*dmats2[9][17] + coeff0_10*dmats2[10][17] + coeff0_11*dmats2[11][17] + coeff0_12*dmats2[12][17] + coeff0_13*dmats2[13][17] + coeff0_14*dmats2[14][17] + coeff0_15*dmats2[15][17] + coeff0_16*dmats2[16][17] + coeff0_17*dmats2[17][17] + coeff0_18*dmats2[18][17] + coeff0_19*dmats2[19][17] + coeff0_20*dmats2[20][17] + coeff0_21*dmats2[21][17] + coeff0_22*dmats2[22][17] + coeff0_23*dmats2[23][17] + coeff0_24*dmats2[24][17] + coeff0_25*dmats2[25][17] + coeff0_26*dmats2[26][17] + coeff0_27*dmats2[27][17] + coeff0_28*dmats2[28][17] + coeff0_29*dmats2[29][17] + coeff0_30*dmats2[30][17] + coeff0_31*dmats2[31][17] + coeff0_32*dmats2[32][17] + coeff0_33*dmats2[33][17] + coeff0_34*dmats2[34][17] + coeff0_35*dmats2[35][17] + coeff0_36*dmats2[36][17] + coeff0_37*dmats2[37][17] + coeff0_38*dmats2[38][17] + coeff0_39*dmats2[39][17] + coeff0_40*dmats2[40][17] + coeff0_41*dmats2[41][17] + coeff0_42*dmats2[42][17] + coeff0_43*dmats2[43][17] + coeff0_44*dmats2[44][17] + coeff0_45*dmats2[45][17] + coeff0_46*dmats2[46][17] + coeff0_47*dmats2[47][17] + coeff0_48*dmats2[48][17] + coeff0_49*dmats2[49][17] + coeff0_50*dmats2[50][17] + coeff0_51*dmats2[51][17] + coeff0_52*dmats2[52][17] + coeff0_53*dmats2[53][17] + coeff0_54*dmats2[54][17] + coeff0_55*dmats2[55][17];
          new_coeff0_18 = coeff0_0*dmats2[0][18] + coeff0_1*dmats2[1][18] + coeff0_2*dmats2[2][18] + coeff0_3*dmats2[3][18] + coeff0_4*dmats2[4][18] + coeff0_5*dmats2[5][18] + coeff0_6*dmats2[6][18] + coeff0_7*dmats2[7][18] + coeff0_8*dmats2[8][18] + coeff0_9*dmats2[9][18] + coeff0_10*dmats2[10][18] + coeff0_11*dmats2[11][18] + coeff0_12*dmats2[12][18] + coeff0_13*dmats2[13][18] + coeff0_14*dmats2[14][18] + coeff0_15*dmats2[15][18] + coeff0_16*dmats2[16][18] + coeff0_17*dmats2[17][18] + coeff0_18*dmats2[18][18] + coeff0_19*dmats2[19][18] + coeff0_20*dmats2[20][18] + coeff0_21*dmats2[21][18] + coeff0_22*dmats2[22][18] + coeff0_23*dmats2[23][18] + coeff0_24*dmats2[24][18] + coeff0_25*dmats2[25][18] + coeff0_26*dmats2[26][18] + coeff0_27*dmats2[27][18] + coeff0_28*dmats2[28][18] + coeff0_29*dmats2[29][18] + coeff0_30*dmats2[30][18] + coeff0_31*dmats2[31][18] + coeff0_32*dmats2[32][18] + coeff0_33*dmats2[33][18] + coeff0_34*dmats2[34][18] + coeff0_35*dmats2[35][18] + coeff0_36*dmats2[36][18] + coeff0_37*dmats2[37][18] + coeff0_38*dmats2[38][18] + coeff0_39*dmats2[39][18] + coeff0_40*dmats2[40][18] + coeff0_41*dmats2[41][18] + coeff0_42*dmats2[42][18] + coeff0_43*dmats2[43][18] + coeff0_44*dmats2[44][18] + coeff0_45*dmats2[45][18] + coeff0_46*dmats2[46][18] + coeff0_47*dmats2[47][18] + coeff0_48*dmats2[48][18] + coeff0_49*dmats2[49][18] + coeff0_50*dmats2[50][18] + coeff0_51*dmats2[51][18] + coeff0_52*dmats2[52][18] + coeff0_53*dmats2[53][18] + coeff0_54*dmats2[54][18] + coeff0_55*dmats2[55][18];
          new_coeff0_19 = coeff0_0*dmats2[0][19] + coeff0_1*dmats2[1][19] + coeff0_2*dmats2[2][19] + coeff0_3*dmats2[3][19] + coeff0_4*dmats2[4][19] + coeff0_5*dmats2[5][19] + coeff0_6*dmats2[6][19] + coeff0_7*dmats2[7][19] + coeff0_8*dmats2[8][19] + coeff0_9*dmats2[9][19] + coeff0_10*dmats2[10][19] + coeff0_11*dmats2[11][19] + coeff0_12*dmats2[12][19] + coeff0_13*dmats2[13][19] + coeff0_14*dmats2[14][19] + coeff0_15*dmats2[15][19] + coeff0_16*dmats2[16][19] + coeff0_17*dmats2[17][19] + coeff0_18*dmats2[18][19] + coeff0_19*dmats2[19][19] + coeff0_20*dmats2[20][19] + coeff0_21*dmats2[21][19] + coeff0_22*dmats2[22][19] + coeff0_23*dmats2[23][19] + coeff0_24*dmats2[24][19] + coeff0_25*dmats2[25][19] + coeff0_26*dmats2[26][19] + coeff0_27*dmats2[27][19] + coeff0_28*dmats2[28][19] + coeff0_29*dmats2[29][19] + coeff0_30*dmats2[30][19] + coeff0_31*dmats2[31][19] + coeff0_32*dmats2[32][19] + coeff0_33*dmats2[33][19] + coeff0_34*dmats2[34][19] + coeff0_35*dmats2[35][19] + coeff0_36*dmats2[36][19] + coeff0_37*dmats2[37][19] + coeff0_38*dmats2[38][19] + coeff0_39*dmats2[39][19] + coeff0_40*dmats2[40][19] + coeff0_41*dmats2[41][19] + coeff0_42*dmats2[42][19] + coeff0_43*dmats2[43][19] + coeff0_44*dmats2[44][19] + coeff0_45*dmats2[45][19] + coeff0_46*dmats2[46][19] + coeff0_47*dmats2[47][19] + coeff0_48*dmats2[48][19] + coeff0_49*dmats2[49][19] + coeff0_50*dmats2[50][19] + coeff0_51*dmats2[51][19] + coeff0_52*dmats2[52][19] + coeff0_53*dmats2[53][19] + coeff0_54*dmats2[54][19] + coeff0_55*dmats2[55][19];
          new_coeff0_20 = coeff0_0*dmats2[0][20] + coeff0_1*dmats2[1][20] + coeff0_2*dmats2[2][20] + coeff0_3*dmats2[3][20] + coeff0_4*dmats2[4][20] + coeff0_5*dmats2[5][20] + coeff0_6*dmats2[6][20] + coeff0_7*dmats2[7][20] + coeff0_8*dmats2[8][20] + coeff0_9*dmats2[9][20] + coeff0_10*dmats2[10][20] + coeff0_11*dmats2[11][20] + coeff0_12*dmats2[12][20] + coeff0_13*dmats2[13][20] + coeff0_14*dmats2[14][20] + coeff0_15*dmats2[15][20] + coeff0_16*dmats2[16][20] + coeff0_17*dmats2[17][20] + coeff0_18*dmats2[18][20] + coeff0_19*dmats2[19][20] + coeff0_20*dmats2[20][20] + coeff0_21*dmats2[21][20] + coeff0_22*dmats2[22][20] + coeff0_23*dmats2[23][20] + coeff0_24*dmats2[24][20] + coeff0_25*dmats2[25][20] + coeff0_26*dmats2[26][20] + coeff0_27*dmats2[27][20] + coeff0_28*dmats2[28][20] + coeff0_29*dmats2[29][20] + coeff0_30*dmats2[30][20] + coeff0_31*dmats2[31][20] + coeff0_32*dmats2[32][20] + coeff0_33*dmats2[33][20] + coeff0_34*dmats2[34][20] + coeff0_35*dmats2[35][20] + coeff0_36*dmats2[36][20] + coeff0_37*dmats2[37][20] + coeff0_38*dmats2[38][20] + coeff0_39*dmats2[39][20] + coeff0_40*dmats2[40][20] + coeff0_41*dmats2[41][20] + coeff0_42*dmats2[42][20] + coeff0_43*dmats2[43][20] + coeff0_44*dmats2[44][20] + coeff0_45*dmats2[45][20] + coeff0_46*dmats2[46][20] + coeff0_47*dmats2[47][20] + coeff0_48*dmats2[48][20] + coeff0_49*dmats2[49][20] + coeff0_50*dmats2[50][20] + coeff0_51*dmats2[51][20] + coeff0_52*dmats2[52][20] + coeff0_53*dmats2[53][20] + coeff0_54*dmats2[54][20] + coeff0_55*dmats2[55][20];
          new_coeff0_21 = coeff0_0*dmats2[0][21] + coeff0_1*dmats2[1][21] + coeff0_2*dmats2[2][21] + coeff0_3*dmats2[3][21] + coeff0_4*dmats2[4][21] + coeff0_5*dmats2[5][21] + coeff0_6*dmats2[6][21] + coeff0_7*dmats2[7][21] + coeff0_8*dmats2[8][21] + coeff0_9*dmats2[9][21] + coeff0_10*dmats2[10][21] + coeff0_11*dmats2[11][21] + coeff0_12*dmats2[12][21] + coeff0_13*dmats2[13][21] + coeff0_14*dmats2[14][21] + coeff0_15*dmats2[15][21] + coeff0_16*dmats2[16][21] + coeff0_17*dmats2[17][21] + coeff0_18*dmats2[18][21] + coeff0_19*dmats2[19][21] + coeff0_20*dmats2[20][21] + coeff0_21*dmats2[21][21] + coeff0_22*dmats2[22][21] + coeff0_23*dmats2[23][21] + coeff0_24*dmats2[24][21] + coeff0_25*dmats2[25][21] + coeff0_26*dmats2[26][21] + coeff0_27*dmats2[27][21] + coeff0_28*dmats2[28][21] + coeff0_29*dmats2[29][21] + coeff0_30*dmats2[30][21] + coeff0_31*dmats2[31][21] + coeff0_32*dmats2[32][21] + coeff0_33*dmats2[33][21] + coeff0_34*dmats2[34][21] + coeff0_35*dmats2[35][21] + coeff0_36*dmats2[36][21] + coeff0_37*dmats2[37][21] + coeff0_38*dmats2[38][21] + coeff0_39*dmats2[39][21] + coeff0_40*dmats2[40][21] + coeff0_41*dmats2[41][21] + coeff0_42*dmats2[42][21] + coeff0_43*dmats2[43][21] + coeff0_44*dmats2[44][21] + coeff0_45*dmats2[45][21] + coeff0_46*dmats2[46][21] + coeff0_47*dmats2[47][21] + coeff0_48*dmats2[48][21] + coeff0_49*dmats2[49][21] + coeff0_50*dmats2[50][21] + coeff0_51*dmats2[51][21] + coeff0_52*dmats2[52][21] + coeff0_53*dmats2[53][21] + coeff0_54*dmats2[54][21] + coeff0_55*dmats2[55][21];
          new_coeff0_22 = coeff0_0*dmats2[0][22] + coeff0_1*dmats2[1][22] + coeff0_2*dmats2[2][22] + coeff0_3*dmats2[3][22] + coeff0_4*dmats2[4][22] + coeff0_5*dmats2[5][22] + coeff0_6*dmats2[6][22] + coeff0_7*dmats2[7][22] + coeff0_8*dmats2[8][22] + coeff0_9*dmats2[9][22] + coeff0_10*dmats2[10][22] + coeff0_11*dmats2[11][22] + coeff0_12*dmats2[12][22] + coeff0_13*dmats2[13][22] + coeff0_14*dmats2[14][22] + coeff0_15*dmats2[15][22] + coeff0_16*dmats2[16][22] + coeff0_17*dmats2[17][22] + coeff0_18*dmats2[18][22] + coeff0_19*dmats2[19][22] + coeff0_20*dmats2[20][22] + coeff0_21*dmats2[21][22] + coeff0_22*dmats2[22][22] + coeff0_23*dmats2[23][22] + coeff0_24*dmats2[24][22] + coeff0_25*dmats2[25][22] + coeff0_26*dmats2[26][22] + coeff0_27*dmats2[27][22] + coeff0_28*dmats2[28][22] + coeff0_29*dmats2[29][22] + coeff0_30*dmats2[30][22] + coeff0_31*dmats2[31][22] + coeff0_32*dmats2[32][22] + coeff0_33*dmats2[33][22] + coeff0_34*dmats2[34][22] + coeff0_35*dmats2[35][22] + coeff0_36*dmats2[36][22] + coeff0_37*dmats2[37][22] + coeff0_38*dmats2[38][22] + coeff0_39*dmats2[39][22] + coeff0_40*dmats2[40][22] + coeff0_41*dmats2[41][22] + coeff0_42*dmats2[42][22] + coeff0_43*dmats2[43][22] + coeff0_44*dmats2[44][22] + coeff0_45*dmats2[45][22] + coeff0_46*dmats2[46][22] + coeff0_47*dmats2[47][22] + coeff0_48*dmats2[48][22] + coeff0_49*dmats2[49][22] + coeff0_50*dmats2[50][22] + coeff0_51*dmats2[51][22] + coeff0_52*dmats2[52][22] + coeff0_53*dmats2[53][22] + coeff0_54*dmats2[54][22] + coeff0_55*dmats2[55][22];
          new_coeff0_23 = coeff0_0*dmats2[0][23] + coeff0_1*dmats2[1][23] + coeff0_2*dmats2[2][23] + coeff0_3*dmats2[3][23] + coeff0_4*dmats2[4][23] + coeff0_5*dmats2[5][23] + coeff0_6*dmats2[6][23] + coeff0_7*dmats2[7][23] + coeff0_8*dmats2[8][23] + coeff0_9*dmats2[9][23] + coeff0_10*dmats2[10][23] + coeff0_11*dmats2[11][23] + coeff0_12*dmats2[12][23] + coeff0_13*dmats2[13][23] + coeff0_14*dmats2[14][23] + coeff0_15*dmats2[15][23] + coeff0_16*dmats2[16][23] + coeff0_17*dmats2[17][23] + coeff0_18*dmats2[18][23] + coeff0_19*dmats2[19][23] + coeff0_20*dmats2[20][23] + coeff0_21*dmats2[21][23] + coeff0_22*dmats2[22][23] + coeff0_23*dmats2[23][23] + coeff0_24*dmats2[24][23] + coeff0_25*dmats2[25][23] + coeff0_26*dmats2[26][23] + coeff0_27*dmats2[27][23] + coeff0_28*dmats2[28][23] + coeff0_29*dmats2[29][23] + coeff0_30*dmats2[30][23] + coeff0_31*dmats2[31][23] + coeff0_32*dmats2[32][23] + coeff0_33*dmats2[33][23] + coeff0_34*dmats2[34][23] + coeff0_35*dmats2[35][23] + coeff0_36*dmats2[36][23] + coeff0_37*dmats2[37][23] + coeff0_38*dmats2[38][23] + coeff0_39*dmats2[39][23] + coeff0_40*dmats2[40][23] + coeff0_41*dmats2[41][23] + coeff0_42*dmats2[42][23] + coeff0_43*dmats2[43][23] + coeff0_44*dmats2[44][23] + coeff0_45*dmats2[45][23] + coeff0_46*dmats2[46][23] + coeff0_47*dmats2[47][23] + coeff0_48*dmats2[48][23] + coeff0_49*dmats2[49][23] + coeff0_50*dmats2[50][23] + coeff0_51*dmats2[51][23] + coeff0_52*dmats2[52][23] + coeff0_53*dmats2[53][23] + coeff0_54*dmats2[54][23] + coeff0_55*dmats2[55][23];
          new_coeff0_24 = coeff0_0*dmats2[0][24] + coeff0_1*dmats2[1][24] + coeff0_2*dmats2[2][24] + coeff0_3*dmats2[3][24] + coeff0_4*dmats2[4][24] + coeff0_5*dmats2[5][24] + coeff0_6*dmats2[6][24] + coeff0_7*dmats2[7][24] + coeff0_8*dmats2[8][24] + coeff0_9*dmats2[9][24] + coeff0_10*dmats2[10][24] + coeff0_11*dmats2[11][24] + coeff0_12*dmats2[12][24] + coeff0_13*dmats2[13][24] + coeff0_14*dmats2[14][24] + coeff0_15*dmats2[15][24] + coeff0_16*dmats2[16][24] + coeff0_17*dmats2[17][24] + coeff0_18*dmats2[18][24] + coeff0_19*dmats2[19][24] + coeff0_20*dmats2[20][24] + coeff0_21*dmats2[21][24] + coeff0_22*dmats2[22][24] + coeff0_23*dmats2[23][24] + coeff0_24*dmats2[24][24] + coeff0_25*dmats2[25][24] + coeff0_26*dmats2[26][24] + coeff0_27*dmats2[27][24] + coeff0_28*dmats2[28][24] + coeff0_29*dmats2[29][24] + coeff0_30*dmats2[30][24] + coeff0_31*dmats2[31][24] + coeff0_32*dmats2[32][24] + coeff0_33*dmats2[33][24] + coeff0_34*dmats2[34][24] + coeff0_35*dmats2[35][24] + coeff0_36*dmats2[36][24] + coeff0_37*dmats2[37][24] + coeff0_38*dmats2[38][24] + coeff0_39*dmats2[39][24] + coeff0_40*dmats2[40][24] + coeff0_41*dmats2[41][24] + coeff0_42*dmats2[42][24] + coeff0_43*dmats2[43][24] + coeff0_44*dmats2[44][24] + coeff0_45*dmats2[45][24] + coeff0_46*dmats2[46][24] + coeff0_47*dmats2[47][24] + coeff0_48*dmats2[48][24] + coeff0_49*dmats2[49][24] + coeff0_50*dmats2[50][24] + coeff0_51*dmats2[51][24] + coeff0_52*dmats2[52][24] + coeff0_53*dmats2[53][24] + coeff0_54*dmats2[54][24] + coeff0_55*dmats2[55][24];
          new_coeff0_25 = coeff0_0*dmats2[0][25] + coeff0_1*dmats2[1][25] + coeff0_2*dmats2[2][25] + coeff0_3*dmats2[3][25] + coeff0_4*dmats2[4][25] + coeff0_5*dmats2[5][25] + coeff0_6*dmats2[6][25] + coeff0_7*dmats2[7][25] + coeff0_8*dmats2[8][25] + coeff0_9*dmats2[9][25] + coeff0_10*dmats2[10][25] + coeff0_11*dmats2[11][25] + coeff0_12*dmats2[12][25] + coeff0_13*dmats2[13][25] + coeff0_14*dmats2[14][25] + coeff0_15*dmats2[15][25] + coeff0_16*dmats2[16][25] + coeff0_17*dmats2[17][25] + coeff0_18*dmats2[18][25] + coeff0_19*dmats2[19][25] + coeff0_20*dmats2[20][25] + coeff0_21*dmats2[21][25] + coeff0_22*dmats2[22][25] + coeff0_23*dmats2[23][25] + coeff0_24*dmats2[24][25] + coeff0_25*dmats2[25][25] + coeff0_26*dmats2[26][25] + coeff0_27*dmats2[27][25] + coeff0_28*dmats2[28][25] + coeff0_29*dmats2[29][25] + coeff0_30*dmats2[30][25] + coeff0_31*dmats2[31][25] + coeff0_32*dmats2[32][25] + coeff0_33*dmats2[33][25] + coeff0_34*dmats2[34][25] + coeff0_35*dmats2[35][25] + coeff0_36*dmats2[36][25] + coeff0_37*dmats2[37][25] + coeff0_38*dmats2[38][25] + coeff0_39*dmats2[39][25] + coeff0_40*dmats2[40][25] + coeff0_41*dmats2[41][25] + coeff0_42*dmats2[42][25] + coeff0_43*dmats2[43][25] + coeff0_44*dmats2[44][25] + coeff0_45*dmats2[45][25] + coeff0_46*dmats2[46][25] + coeff0_47*dmats2[47][25] + coeff0_48*dmats2[48][25] + coeff0_49*dmats2[49][25] + coeff0_50*dmats2[50][25] + coeff0_51*dmats2[51][25] + coeff0_52*dmats2[52][25] + coeff0_53*dmats2[53][25] + coeff0_54*dmats2[54][25] + coeff0_55*dmats2[55][25];
          new_coeff0_26 = coeff0_0*dmats2[0][26] + coeff0_1*dmats2[1][26] + coeff0_2*dmats2[2][26] + coeff0_3*dmats2[3][26] + coeff0_4*dmats2[4][26] + coeff0_5*dmats2[5][26] + coeff0_6*dmats2[6][26] + coeff0_7*dmats2[7][26] + coeff0_8*dmats2[8][26] + coeff0_9*dmats2[9][26] + coeff0_10*dmats2[10][26] + coeff0_11*dmats2[11][26] + coeff0_12*dmats2[12][26] + coeff0_13*dmats2[13][26] + coeff0_14*dmats2[14][26] + coeff0_15*dmats2[15][26] + coeff0_16*dmats2[16][26] + coeff0_17*dmats2[17][26] + coeff0_18*dmats2[18][26] + coeff0_19*dmats2[19][26] + coeff0_20*dmats2[20][26] + coeff0_21*dmats2[21][26] + coeff0_22*dmats2[22][26] + coeff0_23*dmats2[23][26] + coeff0_24*dmats2[24][26] + coeff0_25*dmats2[25][26] + coeff0_26*dmats2[26][26] + coeff0_27*dmats2[27][26] + coeff0_28*dmats2[28][26] + coeff0_29*dmats2[29][26] + coeff0_30*dmats2[30][26] + coeff0_31*dmats2[31][26] + coeff0_32*dmats2[32][26] + coeff0_33*dmats2[33][26] + coeff0_34*dmats2[34][26] + coeff0_35*dmats2[35][26] + coeff0_36*dmats2[36][26] + coeff0_37*dmats2[37][26] + coeff0_38*dmats2[38][26] + coeff0_39*dmats2[39][26] + coeff0_40*dmats2[40][26] + coeff0_41*dmats2[41][26] + coeff0_42*dmats2[42][26] + coeff0_43*dmats2[43][26] + coeff0_44*dmats2[44][26] + coeff0_45*dmats2[45][26] + coeff0_46*dmats2[46][26] + coeff0_47*dmats2[47][26] + coeff0_48*dmats2[48][26] + coeff0_49*dmats2[49][26] + coeff0_50*dmats2[50][26] + coeff0_51*dmats2[51][26] + coeff0_52*dmats2[52][26] + coeff0_53*dmats2[53][26] + coeff0_54*dmats2[54][26] + coeff0_55*dmats2[55][26];
          new_coeff0_27 = coeff0_0*dmats2[0][27] + coeff0_1*dmats2[1][27] + coeff0_2*dmats2[2][27] + coeff0_3*dmats2[3][27] + coeff0_4*dmats2[4][27] + coeff0_5*dmats2[5][27] + coeff0_6*dmats2[6][27] + coeff0_7*dmats2[7][27] + coeff0_8*dmats2[8][27] + coeff0_9*dmats2[9][27] + coeff0_10*dmats2[10][27] + coeff0_11*dmats2[11][27] + coeff0_12*dmats2[12][27] + coeff0_13*dmats2[13][27] + coeff0_14*dmats2[14][27] + coeff0_15*dmats2[15][27] + coeff0_16*dmats2[16][27] + coeff0_17*dmats2[17][27] + coeff0_18*dmats2[18][27] + coeff0_19*dmats2[19][27] + coeff0_20*dmats2[20][27] + coeff0_21*dmats2[21][27] + coeff0_22*dmats2[22][27] + coeff0_23*dmats2[23][27] + coeff0_24*dmats2[24][27] + coeff0_25*dmats2[25][27] + coeff0_26*dmats2[26][27] + coeff0_27*dmats2[27][27] + coeff0_28*dmats2[28][27] + coeff0_29*dmats2[29][27] + coeff0_30*dmats2[30][27] + coeff0_31*dmats2[31][27] + coeff0_32*dmats2[32][27] + coeff0_33*dmats2[33][27] + coeff0_34*dmats2[34][27] + coeff0_35*dmats2[35][27] + coeff0_36*dmats2[36][27] + coeff0_37*dmats2[37][27] + coeff0_38*dmats2[38][27] + coeff0_39*dmats2[39][27] + coeff0_40*dmats2[40][27] + coeff0_41*dmats2[41][27] + coeff0_42*dmats2[42][27] + coeff0_43*dmats2[43][27] + coeff0_44*dmats2[44][27] + coeff0_45*dmats2[45][27] + coeff0_46*dmats2[46][27] + coeff0_47*dmats2[47][27] + coeff0_48*dmats2[48][27] + coeff0_49*dmats2[49][27] + coeff0_50*dmats2[50][27] + coeff0_51*dmats2[51][27] + coeff0_52*dmats2[52][27] + coeff0_53*dmats2[53][27] + coeff0_54*dmats2[54][27] + coeff0_55*dmats2[55][27];
          new_coeff0_28 = coeff0_0*dmats2[0][28] + coeff0_1*dmats2[1][28] + coeff0_2*dmats2[2][28] + coeff0_3*dmats2[3][28] + coeff0_4*dmats2[4][28] + coeff0_5*dmats2[5][28] + coeff0_6*dmats2[6][28] + coeff0_7*dmats2[7][28] + coeff0_8*dmats2[8][28] + coeff0_9*dmats2[9][28] + coeff0_10*dmats2[10][28] + coeff0_11*dmats2[11][28] + coeff0_12*dmats2[12][28] + coeff0_13*dmats2[13][28] + coeff0_14*dmats2[14][28] + coeff0_15*dmats2[15][28] + coeff0_16*dmats2[16][28] + coeff0_17*dmats2[17][28] + coeff0_18*dmats2[18][28] + coeff0_19*dmats2[19][28] + coeff0_20*dmats2[20][28] + coeff0_21*dmats2[21][28] + coeff0_22*dmats2[22][28] + coeff0_23*dmats2[23][28] + coeff0_24*dmats2[24][28] + coeff0_25*dmats2[25][28] + coeff0_26*dmats2[26][28] + coeff0_27*dmats2[27][28] + coeff0_28*dmats2[28][28] + coeff0_29*dmats2[29][28] + coeff0_30*dmats2[30][28] + coeff0_31*dmats2[31][28] + coeff0_32*dmats2[32][28] + coeff0_33*dmats2[33][28] + coeff0_34*dmats2[34][28] + coeff0_35*dmats2[35][28] + coeff0_36*dmats2[36][28] + coeff0_37*dmats2[37][28] + coeff0_38*dmats2[38][28] + coeff0_39*dmats2[39][28] + coeff0_40*dmats2[40][28] + coeff0_41*dmats2[41][28] + coeff0_42*dmats2[42][28] + coeff0_43*dmats2[43][28] + coeff0_44*dmats2[44][28] + coeff0_45*dmats2[45][28] + coeff0_46*dmats2[46][28] + coeff0_47*dmats2[47][28] + coeff0_48*dmats2[48][28] + coeff0_49*dmats2[49][28] + coeff0_50*dmats2[50][28] + coeff0_51*dmats2[51][28] + coeff0_52*dmats2[52][28] + coeff0_53*dmats2[53][28] + coeff0_54*dmats2[54][28] + coeff0_55*dmats2[55][28];
          new_coeff0_29 = coeff0_0*dmats2[0][29] + coeff0_1*dmats2[1][29] + coeff0_2*dmats2[2][29] + coeff0_3*dmats2[3][29] + coeff0_4*dmats2[4][29] + coeff0_5*dmats2[5][29] + coeff0_6*dmats2[6][29] + coeff0_7*dmats2[7][29] + coeff0_8*dmats2[8][29] + coeff0_9*dmats2[9][29] + coeff0_10*dmats2[10][29] + coeff0_11*dmats2[11][29] + coeff0_12*dmats2[12][29] + coeff0_13*dmats2[13][29] + coeff0_14*dmats2[14][29] + coeff0_15*dmats2[15][29] + coeff0_16*dmats2[16][29] + coeff0_17*dmats2[17][29] + coeff0_18*dmats2[18][29] + coeff0_19*dmats2[19][29] + coeff0_20*dmats2[20][29] + coeff0_21*dmats2[21][29] + coeff0_22*dmats2[22][29] + coeff0_23*dmats2[23][29] + coeff0_24*dmats2[24][29] + coeff0_25*dmats2[25][29] + coeff0_26*dmats2[26][29] + coeff0_27*dmats2[27][29] + coeff0_28*dmats2[28][29] + coeff0_29*dmats2[29][29] + coeff0_30*dmats2[30][29] + coeff0_31*dmats2[31][29] + coeff0_32*dmats2[32][29] + coeff0_33*dmats2[33][29] + coeff0_34*dmats2[34][29] + coeff0_35*dmats2[35][29] + coeff0_36*dmats2[36][29] + coeff0_37*dmats2[37][29] + coeff0_38*dmats2[38][29] + coeff0_39*dmats2[39][29] + coeff0_40*dmats2[40][29] + coeff0_41*dmats2[41][29] + coeff0_42*dmats2[42][29] + coeff0_43*dmats2[43][29] + coeff0_44*dmats2[44][29] + coeff0_45*dmats2[45][29] + coeff0_46*dmats2[46][29] + coeff0_47*dmats2[47][29] + coeff0_48*dmats2[48][29] + coeff0_49*dmats2[49][29] + coeff0_50*dmats2[50][29] + coeff0_51*dmats2[51][29] + coeff0_52*dmats2[52][29] + coeff0_53*dmats2[53][29] + coeff0_54*dmats2[54][29] + coeff0_55*dmats2[55][29];
          new_coeff0_30 = coeff0_0*dmats2[0][30] + coeff0_1*dmats2[1][30] + coeff0_2*dmats2[2][30] + coeff0_3*dmats2[3][30] + coeff0_4*dmats2[4][30] + coeff0_5*dmats2[5][30] + coeff0_6*dmats2[6][30] + coeff0_7*dmats2[7][30] + coeff0_8*dmats2[8][30] + coeff0_9*dmats2[9][30] + coeff0_10*dmats2[10][30] + coeff0_11*dmats2[11][30] + coeff0_12*dmats2[12][30] + coeff0_13*dmats2[13][30] + coeff0_14*dmats2[14][30] + coeff0_15*dmats2[15][30] + coeff0_16*dmats2[16][30] + coeff0_17*dmats2[17][30] + coeff0_18*dmats2[18][30] + coeff0_19*dmats2[19][30] + coeff0_20*dmats2[20][30] + coeff0_21*dmats2[21][30] + coeff0_22*dmats2[22][30] + coeff0_23*dmats2[23][30] + coeff0_24*dmats2[24][30] + coeff0_25*dmats2[25][30] + coeff0_26*dmats2[26][30] + coeff0_27*dmats2[27][30] + coeff0_28*dmats2[28][30] + coeff0_29*dmats2[29][30] + coeff0_30*dmats2[30][30] + coeff0_31*dmats2[31][30] + coeff0_32*dmats2[32][30] + coeff0_33*dmats2[33][30] + coeff0_34*dmats2[34][30] + coeff0_35*dmats2[35][30] + coeff0_36*dmats2[36][30] + coeff0_37*dmats2[37][30] + coeff0_38*dmats2[38][30] + coeff0_39*dmats2[39][30] + coeff0_40*dmats2[40][30] + coeff0_41*dmats2[41][30] + coeff0_42*dmats2[42][30] + coeff0_43*dmats2[43][30] + coeff0_44*dmats2[44][30] + coeff0_45*dmats2[45][30] + coeff0_46*dmats2[46][30] + coeff0_47*dmats2[47][30] + coeff0_48*dmats2[48][30] + coeff0_49*dmats2[49][30] + coeff0_50*dmats2[50][30] + coeff0_51*dmats2[51][30] + coeff0_52*dmats2[52][30] + coeff0_53*dmats2[53][30] + coeff0_54*dmats2[54][30] + coeff0_55*dmats2[55][30];
          new_coeff0_31 = coeff0_0*dmats2[0][31] + coeff0_1*dmats2[1][31] + coeff0_2*dmats2[2][31] + coeff0_3*dmats2[3][31] + coeff0_4*dmats2[4][31] + coeff0_5*dmats2[5][31] + coeff0_6*dmats2[6][31] + coeff0_7*dmats2[7][31] + coeff0_8*dmats2[8][31] + coeff0_9*dmats2[9][31] + coeff0_10*dmats2[10][31] + coeff0_11*dmats2[11][31] + coeff0_12*dmats2[12][31] + coeff0_13*dmats2[13][31] + coeff0_14*dmats2[14][31] + coeff0_15*dmats2[15][31] + coeff0_16*dmats2[16][31] + coeff0_17*dmats2[17][31] + coeff0_18*dmats2[18][31] + coeff0_19*dmats2[19][31] + coeff0_20*dmats2[20][31] + coeff0_21*dmats2[21][31] + coeff0_22*dmats2[22][31] + coeff0_23*dmats2[23][31] + coeff0_24*dmats2[24][31] + coeff0_25*dmats2[25][31] + coeff0_26*dmats2[26][31] + coeff0_27*dmats2[27][31] + coeff0_28*dmats2[28][31] + coeff0_29*dmats2[29][31] + coeff0_30*dmats2[30][31] + coeff0_31*dmats2[31][31] + coeff0_32*dmats2[32][31] + coeff0_33*dmats2[33][31] + coeff0_34*dmats2[34][31] + coeff0_35*dmats2[35][31] + coeff0_36*dmats2[36][31] + coeff0_37*dmats2[37][31] + coeff0_38*dmats2[38][31] + coeff0_39*dmats2[39][31] + coeff0_40*dmats2[40][31] + coeff0_41*dmats2[41][31] + coeff0_42*dmats2[42][31] + coeff0_43*dmats2[43][31] + coeff0_44*dmats2[44][31] + coeff0_45*dmats2[45][31] + coeff0_46*dmats2[46][31] + coeff0_47*dmats2[47][31] + coeff0_48*dmats2[48][31] + coeff0_49*dmats2[49][31] + coeff0_50*dmats2[50][31] + coeff0_51*dmats2[51][31] + coeff0_52*dmats2[52][31] + coeff0_53*dmats2[53][31] + coeff0_54*dmats2[54][31] + coeff0_55*dmats2[55][31];
          new_coeff0_32 = coeff0_0*dmats2[0][32] + coeff0_1*dmats2[1][32] + coeff0_2*dmats2[2][32] + coeff0_3*dmats2[3][32] + coeff0_4*dmats2[4][32] + coeff0_5*dmats2[5][32] + coeff0_6*dmats2[6][32] + coeff0_7*dmats2[7][32] + coeff0_8*dmats2[8][32] + coeff0_9*dmats2[9][32] + coeff0_10*dmats2[10][32] + coeff0_11*dmats2[11][32] + coeff0_12*dmats2[12][32] + coeff0_13*dmats2[13][32] + coeff0_14*dmats2[14][32] + coeff0_15*dmats2[15][32] + coeff0_16*dmats2[16][32] + coeff0_17*dmats2[17][32] + coeff0_18*dmats2[18][32] + coeff0_19*dmats2[19][32] + coeff0_20*dmats2[20][32] + coeff0_21*dmats2[21][32] + coeff0_22*dmats2[22][32] + coeff0_23*dmats2[23][32] + coeff0_24*dmats2[24][32] + coeff0_25*dmats2[25][32] + coeff0_26*dmats2[26][32] + coeff0_27*dmats2[27][32] + coeff0_28*dmats2[28][32] + coeff0_29*dmats2[29][32] + coeff0_30*dmats2[30][32] + coeff0_31*dmats2[31][32] + coeff0_32*dmats2[32][32] + coeff0_33*dmats2[33][32] + coeff0_34*dmats2[34][32] + coeff0_35*dmats2[35][32] + coeff0_36*dmats2[36][32] + coeff0_37*dmats2[37][32] + coeff0_38*dmats2[38][32] + coeff0_39*dmats2[39][32] + coeff0_40*dmats2[40][32] + coeff0_41*dmats2[41][32] + coeff0_42*dmats2[42][32] + coeff0_43*dmats2[43][32] + coeff0_44*dmats2[44][32] + coeff0_45*dmats2[45][32] + coeff0_46*dmats2[46][32] + coeff0_47*dmats2[47][32] + coeff0_48*dmats2[48][32] + coeff0_49*dmats2[49][32] + coeff0_50*dmats2[50][32] + coeff0_51*dmats2[51][32] + coeff0_52*dmats2[52][32] + coeff0_53*dmats2[53][32] + coeff0_54*dmats2[54][32] + coeff0_55*dmats2[55][32];
          new_coeff0_33 = coeff0_0*dmats2[0][33] + coeff0_1*dmats2[1][33] + coeff0_2*dmats2[2][33] + coeff0_3*dmats2[3][33] + coeff0_4*dmats2[4][33] + coeff0_5*dmats2[5][33] + coeff0_6*dmats2[6][33] + coeff0_7*dmats2[7][33] + coeff0_8*dmats2[8][33] + coeff0_9*dmats2[9][33] + coeff0_10*dmats2[10][33] + coeff0_11*dmats2[11][33] + coeff0_12*dmats2[12][33] + coeff0_13*dmats2[13][33] + coeff0_14*dmats2[14][33] + coeff0_15*dmats2[15][33] + coeff0_16*dmats2[16][33] + coeff0_17*dmats2[17][33] + coeff0_18*dmats2[18][33] + coeff0_19*dmats2[19][33] + coeff0_20*dmats2[20][33] + coeff0_21*dmats2[21][33] + coeff0_22*dmats2[22][33] + coeff0_23*dmats2[23][33] + coeff0_24*dmats2[24][33] + coeff0_25*dmats2[25][33] + coeff0_26*dmats2[26][33] + coeff0_27*dmats2[27][33] + coeff0_28*dmats2[28][33] + coeff0_29*dmats2[29][33] + coeff0_30*dmats2[30][33] + coeff0_31*dmats2[31][33] + coeff0_32*dmats2[32][33] + coeff0_33*dmats2[33][33] + coeff0_34*dmats2[34][33] + coeff0_35*dmats2[35][33] + coeff0_36*dmats2[36][33] + coeff0_37*dmats2[37][33] + coeff0_38*dmats2[38][33] + coeff0_39*dmats2[39][33] + coeff0_40*dmats2[40][33] + coeff0_41*dmats2[41][33] + coeff0_42*dmats2[42][33] + coeff0_43*dmats2[43][33] + coeff0_44*dmats2[44][33] + coeff0_45*dmats2[45][33] + coeff0_46*dmats2[46][33] + coeff0_47*dmats2[47][33] + coeff0_48*dmats2[48][33] + coeff0_49*dmats2[49][33] + coeff0_50*dmats2[50][33] + coeff0_51*dmats2[51][33] + coeff0_52*dmats2[52][33] + coeff0_53*dmats2[53][33] + coeff0_54*dmats2[54][33] + coeff0_55*dmats2[55][33];
          new_coeff0_34 = coeff0_0*dmats2[0][34] + coeff0_1*dmats2[1][34] + coeff0_2*dmats2[2][34] + coeff0_3*dmats2[3][34] + coeff0_4*dmats2[4][34] + coeff0_5*dmats2[5][34] + coeff0_6*dmats2[6][34] + coeff0_7*dmats2[7][34] + coeff0_8*dmats2[8][34] + coeff0_9*dmats2[9][34] + coeff0_10*dmats2[10][34] + coeff0_11*dmats2[11][34] + coeff0_12*dmats2[12][34] + coeff0_13*dmats2[13][34] + coeff0_14*dmats2[14][34] + coeff0_15*dmats2[15][34] + coeff0_16*dmats2[16][34] + coeff0_17*dmats2[17][34] + coeff0_18*dmats2[18][34] + coeff0_19*dmats2[19][34] + coeff0_20*dmats2[20][34] + coeff0_21*dmats2[21][34] + coeff0_22*dmats2[22][34] + coeff0_23*dmats2[23][34] + coeff0_24*dmats2[24][34] + coeff0_25*dmats2[25][34] + coeff0_26*dmats2[26][34] + coeff0_27*dmats2[27][34] + coeff0_28*dmats2[28][34] + coeff0_29*dmats2[29][34] + coeff0_30*dmats2[30][34] + coeff0_31*dmats2[31][34] + coeff0_32*dmats2[32][34] + coeff0_33*dmats2[33][34] + coeff0_34*dmats2[34][34] + coeff0_35*dmats2[35][34] + coeff0_36*dmats2[36][34] + coeff0_37*dmats2[37][34] + coeff0_38*dmats2[38][34] + coeff0_39*dmats2[39][34] + coeff0_40*dmats2[40][34] + coeff0_41*dmats2[41][34] + coeff0_42*dmats2[42][34] + coeff0_43*dmats2[43][34] + coeff0_44*dmats2[44][34] + coeff0_45*dmats2[45][34] + coeff0_46*dmats2[46][34] + coeff0_47*dmats2[47][34] + coeff0_48*dmats2[48][34] + coeff0_49*dmats2[49][34] + coeff0_50*dmats2[50][34] + coeff0_51*dmats2[51][34] + coeff0_52*dmats2[52][34] + coeff0_53*dmats2[53][34] + coeff0_54*dmats2[54][34] + coeff0_55*dmats2[55][34];
          new_coeff0_35 = coeff0_0*dmats2[0][35] + coeff0_1*dmats2[1][35] + coeff0_2*dmats2[2][35] + coeff0_3*dmats2[3][35] + coeff0_4*dmats2[4][35] + coeff0_5*dmats2[5][35] + coeff0_6*dmats2[6][35] + coeff0_7*dmats2[7][35] + coeff0_8*dmats2[8][35] + coeff0_9*dmats2[9][35] + coeff0_10*dmats2[10][35] + coeff0_11*dmats2[11][35] + coeff0_12*dmats2[12][35] + coeff0_13*dmats2[13][35] + coeff0_14*dmats2[14][35] + coeff0_15*dmats2[15][35] + coeff0_16*dmats2[16][35] + coeff0_17*dmats2[17][35] + coeff0_18*dmats2[18][35] + coeff0_19*dmats2[19][35] + coeff0_20*dmats2[20][35] + coeff0_21*dmats2[21][35] + coeff0_22*dmats2[22][35] + coeff0_23*dmats2[23][35] + coeff0_24*dmats2[24][35] + coeff0_25*dmats2[25][35] + coeff0_26*dmats2[26][35] + coeff0_27*dmats2[27][35] + coeff0_28*dmats2[28][35] + coeff0_29*dmats2[29][35] + coeff0_30*dmats2[30][35] + coeff0_31*dmats2[31][35] + coeff0_32*dmats2[32][35] + coeff0_33*dmats2[33][35] + coeff0_34*dmats2[34][35] + coeff0_35*dmats2[35][35] + coeff0_36*dmats2[36][35] + coeff0_37*dmats2[37][35] + coeff0_38*dmats2[38][35] + coeff0_39*dmats2[39][35] + coeff0_40*dmats2[40][35] + coeff0_41*dmats2[41][35] + coeff0_42*dmats2[42][35] + coeff0_43*dmats2[43][35] + coeff0_44*dmats2[44][35] + coeff0_45*dmats2[45][35] + coeff0_46*dmats2[46][35] + coeff0_47*dmats2[47][35] + coeff0_48*dmats2[48][35] + coeff0_49*dmats2[49][35] + coeff0_50*dmats2[50][35] + coeff0_51*dmats2[51][35] + coeff0_52*dmats2[52][35] + coeff0_53*dmats2[53][35] + coeff0_54*dmats2[54][35] + coeff0_55*dmats2[55][35];
          new_coeff0_36 = coeff0_0*dmats2[0][36] + coeff0_1*dmats2[1][36] + coeff0_2*dmats2[2][36] + coeff0_3*dmats2[3][36] + coeff0_4*dmats2[4][36] + coeff0_5*dmats2[5][36] + coeff0_6*dmats2[6][36] + coeff0_7*dmats2[7][36] + coeff0_8*dmats2[8][36] + coeff0_9*dmats2[9][36] + coeff0_10*dmats2[10][36] + coeff0_11*dmats2[11][36] + coeff0_12*dmats2[12][36] + coeff0_13*dmats2[13][36] + coeff0_14*dmats2[14][36] + coeff0_15*dmats2[15][36] + coeff0_16*dmats2[16][36] + coeff0_17*dmats2[17][36] + coeff0_18*dmats2[18][36] + coeff0_19*dmats2[19][36] + coeff0_20*dmats2[20][36] + coeff0_21*dmats2[21][36] + coeff0_22*dmats2[22][36] + coeff0_23*dmats2[23][36] + coeff0_24*dmats2[24][36] + coeff0_25*dmats2[25][36] + coeff0_26*dmats2[26][36] + coeff0_27*dmats2[27][36] + coeff0_28*dmats2[28][36] + coeff0_29*dmats2[29][36] + coeff0_30*dmats2[30][36] + coeff0_31*dmats2[31][36] + coeff0_32*dmats2[32][36] + coeff0_33*dmats2[33][36] + coeff0_34*dmats2[34][36] + coeff0_35*dmats2[35][36] + coeff0_36*dmats2[36][36] + coeff0_37*dmats2[37][36] + coeff0_38*dmats2[38][36] + coeff0_39*dmats2[39][36] + coeff0_40*dmats2[40][36] + coeff0_41*dmats2[41][36] + coeff0_42*dmats2[42][36] + coeff0_43*dmats2[43][36] + coeff0_44*dmats2[44][36] + coeff0_45*dmats2[45][36] + coeff0_46*dmats2[46][36] + coeff0_47*dmats2[47][36] + coeff0_48*dmats2[48][36] + coeff0_49*dmats2[49][36] + coeff0_50*dmats2[50][36] + coeff0_51*dmats2[51][36] + coeff0_52*dmats2[52][36] + coeff0_53*dmats2[53][36] + coeff0_54*dmats2[54][36] + coeff0_55*dmats2[55][36];
          new_coeff0_37 = coeff0_0*dmats2[0][37] + coeff0_1*dmats2[1][37] + coeff0_2*dmats2[2][37] + coeff0_3*dmats2[3][37] + coeff0_4*dmats2[4][37] + coeff0_5*dmats2[5][37] + coeff0_6*dmats2[6][37] + coeff0_7*dmats2[7][37] + coeff0_8*dmats2[8][37] + coeff0_9*dmats2[9][37] + coeff0_10*dmats2[10][37] + coeff0_11*dmats2[11][37] + coeff0_12*dmats2[12][37] + coeff0_13*dmats2[13][37] + coeff0_14*dmats2[14][37] + coeff0_15*dmats2[15][37] + coeff0_16*dmats2[16][37] + coeff0_17*dmats2[17][37] + coeff0_18*dmats2[18][37] + coeff0_19*dmats2[19][37] + coeff0_20*dmats2[20][37] + coeff0_21*dmats2[21][37] + coeff0_22*dmats2[22][37] + coeff0_23*dmats2[23][37] + coeff0_24*dmats2[24][37] + coeff0_25*dmats2[25][37] + coeff0_26*dmats2[26][37] + coeff0_27*dmats2[27][37] + coeff0_28*dmats2[28][37] + coeff0_29*dmats2[29][37] + coeff0_30*dmats2[30][37] + coeff0_31*dmats2[31][37] + coeff0_32*dmats2[32][37] + coeff0_33*dmats2[33][37] + coeff0_34*dmats2[34][37] + coeff0_35*dmats2[35][37] + coeff0_36*dmats2[36][37] + coeff0_37*dmats2[37][37] + coeff0_38*dmats2[38][37] + coeff0_39*dmats2[39][37] + coeff0_40*dmats2[40][37] + coeff0_41*dmats2[41][37] + coeff0_42*dmats2[42][37] + coeff0_43*dmats2[43][37] + coeff0_44*dmats2[44][37] + coeff0_45*dmats2[45][37] + coeff0_46*dmats2[46][37] + coeff0_47*dmats2[47][37] + coeff0_48*dmats2[48][37] + coeff0_49*dmats2[49][37] + coeff0_50*dmats2[50][37] + coeff0_51*dmats2[51][37] + coeff0_52*dmats2[52][37] + coeff0_53*dmats2[53][37] + coeff0_54*dmats2[54][37] + coeff0_55*dmats2[55][37];
          new_coeff0_38 = coeff0_0*dmats2[0][38] + coeff0_1*dmats2[1][38] + coeff0_2*dmats2[2][38] + coeff0_3*dmats2[3][38] + coeff0_4*dmats2[4][38] + coeff0_5*dmats2[5][38] + coeff0_6*dmats2[6][38] + coeff0_7*dmats2[7][38] + coeff0_8*dmats2[8][38] + coeff0_9*dmats2[9][38] + coeff0_10*dmats2[10][38] + coeff0_11*dmats2[11][38] + coeff0_12*dmats2[12][38] + coeff0_13*dmats2[13][38] + coeff0_14*dmats2[14][38] + coeff0_15*dmats2[15][38] + coeff0_16*dmats2[16][38] + coeff0_17*dmats2[17][38] + coeff0_18*dmats2[18][38] + coeff0_19*dmats2[19][38] + coeff0_20*dmats2[20][38] + coeff0_21*dmats2[21][38] + coeff0_22*dmats2[22][38] + coeff0_23*dmats2[23][38] + coeff0_24*dmats2[24][38] + coeff0_25*dmats2[25][38] + coeff0_26*dmats2[26][38] + coeff0_27*dmats2[27][38] + coeff0_28*dmats2[28][38] + coeff0_29*dmats2[29][38] + coeff0_30*dmats2[30][38] + coeff0_31*dmats2[31][38] + coeff0_32*dmats2[32][38] + coeff0_33*dmats2[33][38] + coeff0_34*dmats2[34][38] + coeff0_35*dmats2[35][38] + coeff0_36*dmats2[36][38] + coeff0_37*dmats2[37][38] + coeff0_38*dmats2[38][38] + coeff0_39*dmats2[39][38] + coeff0_40*dmats2[40][38] + coeff0_41*dmats2[41][38] + coeff0_42*dmats2[42][38] + coeff0_43*dmats2[43][38] + coeff0_44*dmats2[44][38] + coeff0_45*dmats2[45][38] + coeff0_46*dmats2[46][38] + coeff0_47*dmats2[47][38] + coeff0_48*dmats2[48][38] + coeff0_49*dmats2[49][38] + coeff0_50*dmats2[50][38] + coeff0_51*dmats2[51][38] + coeff0_52*dmats2[52][38] + coeff0_53*dmats2[53][38] + coeff0_54*dmats2[54][38] + coeff0_55*dmats2[55][38];
          new_coeff0_39 = coeff0_0*dmats2[0][39] + coeff0_1*dmats2[1][39] + coeff0_2*dmats2[2][39] + coeff0_3*dmats2[3][39] + coeff0_4*dmats2[4][39] + coeff0_5*dmats2[5][39] + coeff0_6*dmats2[6][39] + coeff0_7*dmats2[7][39] + coeff0_8*dmats2[8][39] + coeff0_9*dmats2[9][39] + coeff0_10*dmats2[10][39] + coeff0_11*dmats2[11][39] + coeff0_12*dmats2[12][39] + coeff0_13*dmats2[13][39] + coeff0_14*dmats2[14][39] + coeff0_15*dmats2[15][39] + coeff0_16*dmats2[16][39] + coeff0_17*dmats2[17][39] + coeff0_18*dmats2[18][39] + coeff0_19*dmats2[19][39] + coeff0_20*dmats2[20][39] + coeff0_21*dmats2[21][39] + coeff0_22*dmats2[22][39] + coeff0_23*dmats2[23][39] + coeff0_24*dmats2[24][39] + coeff0_25*dmats2[25][39] + coeff0_26*dmats2[26][39] + coeff0_27*dmats2[27][39] + coeff0_28*dmats2[28][39] + coeff0_29*dmats2[29][39] + coeff0_30*dmats2[30][39] + coeff0_31*dmats2[31][39] + coeff0_32*dmats2[32][39] + coeff0_33*dmats2[33][39] + coeff0_34*dmats2[34][39] + coeff0_35*dmats2[35][39] + coeff0_36*dmats2[36][39] + coeff0_37*dmats2[37][39] + coeff0_38*dmats2[38][39] + coeff0_39*dmats2[39][39] + coeff0_40*dmats2[40][39] + coeff0_41*dmats2[41][39] + coeff0_42*dmats2[42][39] + coeff0_43*dmats2[43][39] + coeff0_44*dmats2[44][39] + coeff0_45*dmats2[45][39] + coeff0_46*dmats2[46][39] + coeff0_47*dmats2[47][39] + coeff0_48*dmats2[48][39] + coeff0_49*dmats2[49][39] + coeff0_50*dmats2[50][39] + coeff0_51*dmats2[51][39] + coeff0_52*dmats2[52][39] + coeff0_53*dmats2[53][39] + coeff0_54*dmats2[54][39] + coeff0_55*dmats2[55][39];
          new_coeff0_40 = coeff0_0*dmats2[0][40] + coeff0_1*dmats2[1][40] + coeff0_2*dmats2[2][40] + coeff0_3*dmats2[3][40] + coeff0_4*dmats2[4][40] + coeff0_5*dmats2[5][40] + coeff0_6*dmats2[6][40] + coeff0_7*dmats2[7][40] + coeff0_8*dmats2[8][40] + coeff0_9*dmats2[9][40] + coeff0_10*dmats2[10][40] + coeff0_11*dmats2[11][40] + coeff0_12*dmats2[12][40] + coeff0_13*dmats2[13][40] + coeff0_14*dmats2[14][40] + coeff0_15*dmats2[15][40] + coeff0_16*dmats2[16][40] + coeff0_17*dmats2[17][40] + coeff0_18*dmats2[18][40] + coeff0_19*dmats2[19][40] + coeff0_20*dmats2[20][40] + coeff0_21*dmats2[21][40] + coeff0_22*dmats2[22][40] + coeff0_23*dmats2[23][40] + coeff0_24*dmats2[24][40] + coeff0_25*dmats2[25][40] + coeff0_26*dmats2[26][40] + coeff0_27*dmats2[27][40] + coeff0_28*dmats2[28][40] + coeff0_29*dmats2[29][40] + coeff0_30*dmats2[30][40] + coeff0_31*dmats2[31][40] + coeff0_32*dmats2[32][40] + coeff0_33*dmats2[33][40] + coeff0_34*dmats2[34][40] + coeff0_35*dmats2[35][40] + coeff0_36*dmats2[36][40] + coeff0_37*dmats2[37][40] + coeff0_38*dmats2[38][40] + coeff0_39*dmats2[39][40] + coeff0_40*dmats2[40][40] + coeff0_41*dmats2[41][40] + coeff0_42*dmats2[42][40] + coeff0_43*dmats2[43][40] + coeff0_44*dmats2[44][40] + coeff0_45*dmats2[45][40] + coeff0_46*dmats2[46][40] + coeff0_47*dmats2[47][40] + coeff0_48*dmats2[48][40] + coeff0_49*dmats2[49][40] + coeff0_50*dmats2[50][40] + coeff0_51*dmats2[51][40] + coeff0_52*dmats2[52][40] + coeff0_53*dmats2[53][40] + coeff0_54*dmats2[54][40] + coeff0_55*dmats2[55][40];
          new_coeff0_41 = coeff0_0*dmats2[0][41] + coeff0_1*dmats2[1][41] + coeff0_2*dmats2[2][41] + coeff0_3*dmats2[3][41] + coeff0_4*dmats2[4][41] + coeff0_5*dmats2[5][41] + coeff0_6*dmats2[6][41] + coeff0_7*dmats2[7][41] + coeff0_8*dmats2[8][41] + coeff0_9*dmats2[9][41] + coeff0_10*dmats2[10][41] + coeff0_11*dmats2[11][41] + coeff0_12*dmats2[12][41] + coeff0_13*dmats2[13][41] + coeff0_14*dmats2[14][41] + coeff0_15*dmats2[15][41] + coeff0_16*dmats2[16][41] + coeff0_17*dmats2[17][41] + coeff0_18*dmats2[18][41] + coeff0_19*dmats2[19][41] + coeff0_20*dmats2[20][41] + coeff0_21*dmats2[21][41] + coeff0_22*dmats2[22][41] + coeff0_23*dmats2[23][41] + coeff0_24*dmats2[24][41] + coeff0_25*dmats2[25][41] + coeff0_26*dmats2[26][41] + coeff0_27*dmats2[27][41] + coeff0_28*dmats2[28][41] + coeff0_29*dmats2[29][41] + coeff0_30*dmats2[30][41] + coeff0_31*dmats2[31][41] + coeff0_32*dmats2[32][41] + coeff0_33*dmats2[33][41] + coeff0_34*dmats2[34][41] + coeff0_35*dmats2[35][41] + coeff0_36*dmats2[36][41] + coeff0_37*dmats2[37][41] + coeff0_38*dmats2[38][41] + coeff0_39*dmats2[39][41] + coeff0_40*dmats2[40][41] + coeff0_41*dmats2[41][41] + coeff0_42*dmats2[42][41] + coeff0_43*dmats2[43][41] + coeff0_44*dmats2[44][41] + coeff0_45*dmats2[45][41] + coeff0_46*dmats2[46][41] + coeff0_47*dmats2[47][41] + coeff0_48*dmats2[48][41] + coeff0_49*dmats2[49][41] + coeff0_50*dmats2[50][41] + coeff0_51*dmats2[51][41] + coeff0_52*dmats2[52][41] + coeff0_53*dmats2[53][41] + coeff0_54*dmats2[54][41] + coeff0_55*dmats2[55][41];
          new_coeff0_42 = coeff0_0*dmats2[0][42] + coeff0_1*dmats2[1][42] + coeff0_2*dmats2[2][42] + coeff0_3*dmats2[3][42] + coeff0_4*dmats2[4][42] + coeff0_5*dmats2[5][42] + coeff0_6*dmats2[6][42] + coeff0_7*dmats2[7][42] + coeff0_8*dmats2[8][42] + coeff0_9*dmats2[9][42] + coeff0_10*dmats2[10][42] + coeff0_11*dmats2[11][42] + coeff0_12*dmats2[12][42] + coeff0_13*dmats2[13][42] + coeff0_14*dmats2[14][42] + coeff0_15*dmats2[15][42] + coeff0_16*dmats2[16][42] + coeff0_17*dmats2[17][42] + coeff0_18*dmats2[18][42] + coeff0_19*dmats2[19][42] + coeff0_20*dmats2[20][42] + coeff0_21*dmats2[21][42] + coeff0_22*dmats2[22][42] + coeff0_23*dmats2[23][42] + coeff0_24*dmats2[24][42] + coeff0_25*dmats2[25][42] + coeff0_26*dmats2[26][42] + coeff0_27*dmats2[27][42] + coeff0_28*dmats2[28][42] + coeff0_29*dmats2[29][42] + coeff0_30*dmats2[30][42] + coeff0_31*dmats2[31][42] + coeff0_32*dmats2[32][42] + coeff0_33*dmats2[33][42] + coeff0_34*dmats2[34][42] + coeff0_35*dmats2[35][42] + coeff0_36*dmats2[36][42] + coeff0_37*dmats2[37][42] + coeff0_38*dmats2[38][42] + coeff0_39*dmats2[39][42] + coeff0_40*dmats2[40][42] + coeff0_41*dmats2[41][42] + coeff0_42*dmats2[42][42] + coeff0_43*dmats2[43][42] + coeff0_44*dmats2[44][42] + coeff0_45*dmats2[45][42] + coeff0_46*dmats2[46][42] + coeff0_47*dmats2[47][42] + coeff0_48*dmats2[48][42] + coeff0_49*dmats2[49][42] + coeff0_50*dmats2[50][42] + coeff0_51*dmats2[51][42] + coeff0_52*dmats2[52][42] + coeff0_53*dmats2[53][42] + coeff0_54*dmats2[54][42] + coeff0_55*dmats2[55][42];
          new_coeff0_43 = coeff0_0*dmats2[0][43] + coeff0_1*dmats2[1][43] + coeff0_2*dmats2[2][43] + coeff0_3*dmats2[3][43] + coeff0_4*dmats2[4][43] + coeff0_5*dmats2[5][43] + coeff0_6*dmats2[6][43] + coeff0_7*dmats2[7][43] + coeff0_8*dmats2[8][43] + coeff0_9*dmats2[9][43] + coeff0_10*dmats2[10][43] + coeff0_11*dmats2[11][43] + coeff0_12*dmats2[12][43] + coeff0_13*dmats2[13][43] + coeff0_14*dmats2[14][43] + coeff0_15*dmats2[15][43] + coeff0_16*dmats2[16][43] + coeff0_17*dmats2[17][43] + coeff0_18*dmats2[18][43] + coeff0_19*dmats2[19][43] + coeff0_20*dmats2[20][43] + coeff0_21*dmats2[21][43] + coeff0_22*dmats2[22][43] + coeff0_23*dmats2[23][43] + coeff0_24*dmats2[24][43] + coeff0_25*dmats2[25][43] + coeff0_26*dmats2[26][43] + coeff0_27*dmats2[27][43] + coeff0_28*dmats2[28][43] + coeff0_29*dmats2[29][43] + coeff0_30*dmats2[30][43] + coeff0_31*dmats2[31][43] + coeff0_32*dmats2[32][43] + coeff0_33*dmats2[33][43] + coeff0_34*dmats2[34][43] + coeff0_35*dmats2[35][43] + coeff0_36*dmats2[36][43] + coeff0_37*dmats2[37][43] + coeff0_38*dmats2[38][43] + coeff0_39*dmats2[39][43] + coeff0_40*dmats2[40][43] + coeff0_41*dmats2[41][43] + coeff0_42*dmats2[42][43] + coeff0_43*dmats2[43][43] + coeff0_44*dmats2[44][43] + coeff0_45*dmats2[45][43] + coeff0_46*dmats2[46][43] + coeff0_47*dmats2[47][43] + coeff0_48*dmats2[48][43] + coeff0_49*dmats2[49][43] + coeff0_50*dmats2[50][43] + coeff0_51*dmats2[51][43] + coeff0_52*dmats2[52][43] + coeff0_53*dmats2[53][43] + coeff0_54*dmats2[54][43] + coeff0_55*dmats2[55][43];
          new_coeff0_44 = coeff0_0*dmats2[0][44] + coeff0_1*dmats2[1][44] + coeff0_2*dmats2[2][44] + coeff0_3*dmats2[3][44] + coeff0_4*dmats2[4][44] + coeff0_5*dmats2[5][44] + coeff0_6*dmats2[6][44] + coeff0_7*dmats2[7][44] + coeff0_8*dmats2[8][44] + coeff0_9*dmats2[9][44] + coeff0_10*dmats2[10][44] + coeff0_11*dmats2[11][44] + coeff0_12*dmats2[12][44] + coeff0_13*dmats2[13][44] + coeff0_14*dmats2[14][44] + coeff0_15*dmats2[15][44] + coeff0_16*dmats2[16][44] + coeff0_17*dmats2[17][44] + coeff0_18*dmats2[18][44] + coeff0_19*dmats2[19][44] + coeff0_20*dmats2[20][44] + coeff0_21*dmats2[21][44] + coeff0_22*dmats2[22][44] + coeff0_23*dmats2[23][44] + coeff0_24*dmats2[24][44] + coeff0_25*dmats2[25][44] + coeff0_26*dmats2[26][44] + coeff0_27*dmats2[27][44] + coeff0_28*dmats2[28][44] + coeff0_29*dmats2[29][44] + coeff0_30*dmats2[30][44] + coeff0_31*dmats2[31][44] + coeff0_32*dmats2[32][44] + coeff0_33*dmats2[33][44] + coeff0_34*dmats2[34][44] + coeff0_35*dmats2[35][44] + coeff0_36*dmats2[36][44] + coeff0_37*dmats2[37][44] + coeff0_38*dmats2[38][44] + coeff0_39*dmats2[39][44] + coeff0_40*dmats2[40][44] + coeff0_41*dmats2[41][44] + coeff0_42*dmats2[42][44] + coeff0_43*dmats2[43][44] + coeff0_44*dmats2[44][44] + coeff0_45*dmats2[45][44] + coeff0_46*dmats2[46][44] + coeff0_47*dmats2[47][44] + coeff0_48*dmats2[48][44] + coeff0_49*dmats2[49][44] + coeff0_50*dmats2[50][44] + coeff0_51*dmats2[51][44] + coeff0_52*dmats2[52][44] + coeff0_53*dmats2[53][44] + coeff0_54*dmats2[54][44] + coeff0_55*dmats2[55][44];
          new_coeff0_45 = coeff0_0*dmats2[0][45] + coeff0_1*dmats2[1][45] + coeff0_2*dmats2[2][45] + coeff0_3*dmats2[3][45] + coeff0_4*dmats2[4][45] + coeff0_5*dmats2[5][45] + coeff0_6*dmats2[6][45] + coeff0_7*dmats2[7][45] + coeff0_8*dmats2[8][45] + coeff0_9*dmats2[9][45] + coeff0_10*dmats2[10][45] + coeff0_11*dmats2[11][45] + coeff0_12*dmats2[12][45] + coeff0_13*dmats2[13][45] + coeff0_14*dmats2[14][45] + coeff0_15*dmats2[15][45] + coeff0_16*dmats2[16][45] + coeff0_17*dmats2[17][45] + coeff0_18*dmats2[18][45] + coeff0_19*dmats2[19][45] + coeff0_20*dmats2[20][45] + coeff0_21*dmats2[21][45] + coeff0_22*dmats2[22][45] + coeff0_23*dmats2[23][45] + coeff0_24*dmats2[24][45] + coeff0_25*dmats2[25][45] + coeff0_26*dmats2[26][45] + coeff0_27*dmats2[27][45] + coeff0_28*dmats2[28][45] + coeff0_29*dmats2[29][45] + coeff0_30*dmats2[30][45] + coeff0_31*dmats2[31][45] + coeff0_32*dmats2[32][45] + coeff0_33*dmats2[33][45] + coeff0_34*dmats2[34][45] + coeff0_35*dmats2[35][45] + coeff0_36*dmats2[36][45] + coeff0_37*dmats2[37][45] + coeff0_38*dmats2[38][45] + coeff0_39*dmats2[39][45] + coeff0_40*dmats2[40][45] + coeff0_41*dmats2[41][45] + coeff0_42*dmats2[42][45] + coeff0_43*dmats2[43][45] + coeff0_44*dmats2[44][45] + coeff0_45*dmats2[45][45] + coeff0_46*dmats2[46][45] + coeff0_47*dmats2[47][45] + coeff0_48*dmats2[48][45] + coeff0_49*dmats2[49][45] + coeff0_50*dmats2[50][45] + coeff0_51*dmats2[51][45] + coeff0_52*dmats2[52][45] + coeff0_53*dmats2[53][45] + coeff0_54*dmats2[54][45] + coeff0_55*dmats2[55][45];
          new_coeff0_46 = coeff0_0*dmats2[0][46] + coeff0_1*dmats2[1][46] + coeff0_2*dmats2[2][46] + coeff0_3*dmats2[3][46] + coeff0_4*dmats2[4][46] + coeff0_5*dmats2[5][46] + coeff0_6*dmats2[6][46] + coeff0_7*dmats2[7][46] + coeff0_8*dmats2[8][46] + coeff0_9*dmats2[9][46] + coeff0_10*dmats2[10][46] + coeff0_11*dmats2[11][46] + coeff0_12*dmats2[12][46] + coeff0_13*dmats2[13][46] + coeff0_14*dmats2[14][46] + coeff0_15*dmats2[15][46] + coeff0_16*dmats2[16][46] + coeff0_17*dmats2[17][46] + coeff0_18*dmats2[18][46] + coeff0_19*dmats2[19][46] + coeff0_20*dmats2[20][46] + coeff0_21*dmats2[21][46] + coeff0_22*dmats2[22][46] + coeff0_23*dmats2[23][46] + coeff0_24*dmats2[24][46] + coeff0_25*dmats2[25][46] + coeff0_26*dmats2[26][46] + coeff0_27*dmats2[27][46] + coeff0_28*dmats2[28][46] + coeff0_29*dmats2[29][46] + coeff0_30*dmats2[30][46] + coeff0_31*dmats2[31][46] + coeff0_32*dmats2[32][46] + coeff0_33*dmats2[33][46] + coeff0_34*dmats2[34][46] + coeff0_35*dmats2[35][46] + coeff0_36*dmats2[36][46] + coeff0_37*dmats2[37][46] + coeff0_38*dmats2[38][46] + coeff0_39*dmats2[39][46] + coeff0_40*dmats2[40][46] + coeff0_41*dmats2[41][46] + coeff0_42*dmats2[42][46] + coeff0_43*dmats2[43][46] + coeff0_44*dmats2[44][46] + coeff0_45*dmats2[45][46] + coeff0_46*dmats2[46][46] + coeff0_47*dmats2[47][46] + coeff0_48*dmats2[48][46] + coeff0_49*dmats2[49][46] + coeff0_50*dmats2[50][46] + coeff0_51*dmats2[51][46] + coeff0_52*dmats2[52][46] + coeff0_53*dmats2[53][46] + coeff0_54*dmats2[54][46] + coeff0_55*dmats2[55][46];
          new_coeff0_47 = coeff0_0*dmats2[0][47] + coeff0_1*dmats2[1][47] + coeff0_2*dmats2[2][47] + coeff0_3*dmats2[3][47] + coeff0_4*dmats2[4][47] + coeff0_5*dmats2[5][47] + coeff0_6*dmats2[6][47] + coeff0_7*dmats2[7][47] + coeff0_8*dmats2[8][47] + coeff0_9*dmats2[9][47] + coeff0_10*dmats2[10][47] + coeff0_11*dmats2[11][47] + coeff0_12*dmats2[12][47] + coeff0_13*dmats2[13][47] + coeff0_14*dmats2[14][47] + coeff0_15*dmats2[15][47] + coeff0_16*dmats2[16][47] + coeff0_17*dmats2[17][47] + coeff0_18*dmats2[18][47] + coeff0_19*dmats2[19][47] + coeff0_20*dmats2[20][47] + coeff0_21*dmats2[21][47] + coeff0_22*dmats2[22][47] + coeff0_23*dmats2[23][47] + coeff0_24*dmats2[24][47] + coeff0_25*dmats2[25][47] + coeff0_26*dmats2[26][47] + coeff0_27*dmats2[27][47] + coeff0_28*dmats2[28][47] + coeff0_29*dmats2[29][47] + coeff0_30*dmats2[30][47] + coeff0_31*dmats2[31][47] + coeff0_32*dmats2[32][47] + coeff0_33*dmats2[33][47] + coeff0_34*dmats2[34][47] + coeff0_35*dmats2[35][47] + coeff0_36*dmats2[36][47] + coeff0_37*dmats2[37][47] + coeff0_38*dmats2[38][47] + coeff0_39*dmats2[39][47] + coeff0_40*dmats2[40][47] + coeff0_41*dmats2[41][47] + coeff0_42*dmats2[42][47] + coeff0_43*dmats2[43][47] + coeff0_44*dmats2[44][47] + coeff0_45*dmats2[45][47] + coeff0_46*dmats2[46][47] + coeff0_47*dmats2[47][47] + coeff0_48*dmats2[48][47] + coeff0_49*dmats2[49][47] + coeff0_50*dmats2[50][47] + coeff0_51*dmats2[51][47] + coeff0_52*dmats2[52][47] + coeff0_53*dmats2[53][47] + coeff0_54*dmats2[54][47] + coeff0_55*dmats2[55][47];
          new_coeff0_48 = coeff0_0*dmats2[0][48] + coeff0_1*dmats2[1][48] + coeff0_2*dmats2[2][48] + coeff0_3*dmats2[3][48] + coeff0_4*dmats2[4][48] + coeff0_5*dmats2[5][48] + coeff0_6*dmats2[6][48] + coeff0_7*dmats2[7][48] + coeff0_8*dmats2[8][48] + coeff0_9*dmats2[9][48] + coeff0_10*dmats2[10][48] + coeff0_11*dmats2[11][48] + coeff0_12*dmats2[12][48] + coeff0_13*dmats2[13][48] + coeff0_14*dmats2[14][48] + coeff0_15*dmats2[15][48] + coeff0_16*dmats2[16][48] + coeff0_17*dmats2[17][48] + coeff0_18*dmats2[18][48] + coeff0_19*dmats2[19][48] + coeff0_20*dmats2[20][48] + coeff0_21*dmats2[21][48] + coeff0_22*dmats2[22][48] + coeff0_23*dmats2[23][48] + coeff0_24*dmats2[24][48] + coeff0_25*dmats2[25][48] + coeff0_26*dmats2[26][48] + coeff0_27*dmats2[27][48] + coeff0_28*dmats2[28][48] + coeff0_29*dmats2[29][48] + coeff0_30*dmats2[30][48] + coeff0_31*dmats2[31][48] + coeff0_32*dmats2[32][48] + coeff0_33*dmats2[33][48] + coeff0_34*dmats2[34][48] + coeff0_35*dmats2[35][48] + coeff0_36*dmats2[36][48] + coeff0_37*dmats2[37][48] + coeff0_38*dmats2[38][48] + coeff0_39*dmats2[39][48] + coeff0_40*dmats2[40][48] + coeff0_41*dmats2[41][48] + coeff0_42*dmats2[42][48] + coeff0_43*dmats2[43][48] + coeff0_44*dmats2[44][48] + coeff0_45*dmats2[45][48] + coeff0_46*dmats2[46][48] + coeff0_47*dmats2[47][48] + coeff0_48*dmats2[48][48] + coeff0_49*dmats2[49][48] + coeff0_50*dmats2[50][48] + coeff0_51*dmats2[51][48] + coeff0_52*dmats2[52][48] + coeff0_53*dmats2[53][48] + coeff0_54*dmats2[54][48] + coeff0_55*dmats2[55][48];
          new_coeff0_49 = coeff0_0*dmats2[0][49] + coeff0_1*dmats2[1][49] + coeff0_2*dmats2[2][49] + coeff0_3*dmats2[3][49] + coeff0_4*dmats2[4][49] + coeff0_5*dmats2[5][49] + coeff0_6*dmats2[6][49] + coeff0_7*dmats2[7][49] + coeff0_8*dmats2[8][49] + coeff0_9*dmats2[9][49] + coeff0_10*dmats2[10][49] + coeff0_11*dmats2[11][49] + coeff0_12*dmats2[12][49] + coeff0_13*dmats2[13][49] + coeff0_14*dmats2[14][49] + coeff0_15*dmats2[15][49] + coeff0_16*dmats2[16][49] + coeff0_17*dmats2[17][49] + coeff0_18*dmats2[18][49] + coeff0_19*dmats2[19][49] + coeff0_20*dmats2[20][49] + coeff0_21*dmats2[21][49] + coeff0_22*dmats2[22][49] + coeff0_23*dmats2[23][49] + coeff0_24*dmats2[24][49] + coeff0_25*dmats2[25][49] + coeff0_26*dmats2[26][49] + coeff0_27*dmats2[27][49] + coeff0_28*dmats2[28][49] + coeff0_29*dmats2[29][49] + coeff0_30*dmats2[30][49] + coeff0_31*dmats2[31][49] + coeff0_32*dmats2[32][49] + coeff0_33*dmats2[33][49] + coeff0_34*dmats2[34][49] + coeff0_35*dmats2[35][49] + coeff0_36*dmats2[36][49] + coeff0_37*dmats2[37][49] + coeff0_38*dmats2[38][49] + coeff0_39*dmats2[39][49] + coeff0_40*dmats2[40][49] + coeff0_41*dmats2[41][49] + coeff0_42*dmats2[42][49] + coeff0_43*dmats2[43][49] + coeff0_44*dmats2[44][49] + coeff0_45*dmats2[45][49] + coeff0_46*dmats2[46][49] + coeff0_47*dmats2[47][49] + coeff0_48*dmats2[48][49] + coeff0_49*dmats2[49][49] + coeff0_50*dmats2[50][49] + coeff0_51*dmats2[51][49] + coeff0_52*dmats2[52][49] + coeff0_53*dmats2[53][49] + coeff0_54*dmats2[54][49] + coeff0_55*dmats2[55][49];
          new_coeff0_50 = coeff0_0*dmats2[0][50] + coeff0_1*dmats2[1][50] + coeff0_2*dmats2[2][50] + coeff0_3*dmats2[3][50] + coeff0_4*dmats2[4][50] + coeff0_5*dmats2[5][50] + coeff0_6*dmats2[6][50] + coeff0_7*dmats2[7][50] + coeff0_8*dmats2[8][50] + coeff0_9*dmats2[9][50] + coeff0_10*dmats2[10][50] + coeff0_11*dmats2[11][50] + coeff0_12*dmats2[12][50] + coeff0_13*dmats2[13][50] + coeff0_14*dmats2[14][50] + coeff0_15*dmats2[15][50] + coeff0_16*dmats2[16][50] + coeff0_17*dmats2[17][50] + coeff0_18*dmats2[18][50] + coeff0_19*dmats2[19][50] + coeff0_20*dmats2[20][50] + coeff0_21*dmats2[21][50] + coeff0_22*dmats2[22][50] + coeff0_23*dmats2[23][50] + coeff0_24*dmats2[24][50] + coeff0_25*dmats2[25][50] + coeff0_26*dmats2[26][50] + coeff0_27*dmats2[27][50] + coeff0_28*dmats2[28][50] + coeff0_29*dmats2[29][50] + coeff0_30*dmats2[30][50] + coeff0_31*dmats2[31][50] + coeff0_32*dmats2[32][50] + coeff0_33*dmats2[33][50] + coeff0_34*dmats2[34][50] + coeff0_35*dmats2[35][50] + coeff0_36*dmats2[36][50] + coeff0_37*dmats2[37][50] + coeff0_38*dmats2[38][50] + coeff0_39*dmats2[39][50] + coeff0_40*dmats2[40][50] + coeff0_41*dmats2[41][50] + coeff0_42*dmats2[42][50] + coeff0_43*dmats2[43][50] + coeff0_44*dmats2[44][50] + coeff0_45*dmats2[45][50] + coeff0_46*dmats2[46][50] + coeff0_47*dmats2[47][50] + coeff0_48*dmats2[48][50] + coeff0_49*dmats2[49][50] + coeff0_50*dmats2[50][50] + coeff0_51*dmats2[51][50] + coeff0_52*dmats2[52][50] + coeff0_53*dmats2[53][50] + coeff0_54*dmats2[54][50] + coeff0_55*dmats2[55][50];
          new_coeff0_51 = coeff0_0*dmats2[0][51] + coeff0_1*dmats2[1][51] + coeff0_2*dmats2[2][51] + coeff0_3*dmats2[3][51] + coeff0_4*dmats2[4][51] + coeff0_5*dmats2[5][51] + coeff0_6*dmats2[6][51] + coeff0_7*dmats2[7][51] + coeff0_8*dmats2[8][51] + coeff0_9*dmats2[9][51] + coeff0_10*dmats2[10][51] + coeff0_11*dmats2[11][51] + coeff0_12*dmats2[12][51] + coeff0_13*dmats2[13][51] + coeff0_14*dmats2[14][51] + coeff0_15*dmats2[15][51] + coeff0_16*dmats2[16][51] + coeff0_17*dmats2[17][51] + coeff0_18*dmats2[18][51] + coeff0_19*dmats2[19][51] + coeff0_20*dmats2[20][51] + coeff0_21*dmats2[21][51] + coeff0_22*dmats2[22][51] + coeff0_23*dmats2[23][51] + coeff0_24*dmats2[24][51] + coeff0_25*dmats2[25][51] + coeff0_26*dmats2[26][51] + coeff0_27*dmats2[27][51] + coeff0_28*dmats2[28][51] + coeff0_29*dmats2[29][51] + coeff0_30*dmats2[30][51] + coeff0_31*dmats2[31][51] + coeff0_32*dmats2[32][51] + coeff0_33*dmats2[33][51] + coeff0_34*dmats2[34][51] + coeff0_35*dmats2[35][51] + coeff0_36*dmats2[36][51] + coeff0_37*dmats2[37][51] + coeff0_38*dmats2[38][51] + coeff0_39*dmats2[39][51] + coeff0_40*dmats2[40][51] + coeff0_41*dmats2[41][51] + coeff0_42*dmats2[42][51] + coeff0_43*dmats2[43][51] + coeff0_44*dmats2[44][51] + coeff0_45*dmats2[45][51] + coeff0_46*dmats2[46][51] + coeff0_47*dmats2[47][51] + coeff0_48*dmats2[48][51] + coeff0_49*dmats2[49][51] + coeff0_50*dmats2[50][51] + coeff0_51*dmats2[51][51] + coeff0_52*dmats2[52][51] + coeff0_53*dmats2[53][51] + coeff0_54*dmats2[54][51] + coeff0_55*dmats2[55][51];
          new_coeff0_52 = coeff0_0*dmats2[0][52] + coeff0_1*dmats2[1][52] + coeff0_2*dmats2[2][52] + coeff0_3*dmats2[3][52] + coeff0_4*dmats2[4][52] + coeff0_5*dmats2[5][52] + coeff0_6*dmats2[6][52] + coeff0_7*dmats2[7][52] + coeff0_8*dmats2[8][52] + coeff0_9*dmats2[9][52] + coeff0_10*dmats2[10][52] + coeff0_11*dmats2[11][52] + coeff0_12*dmats2[12][52] + coeff0_13*dmats2[13][52] + coeff0_14*dmats2[14][52] + coeff0_15*dmats2[15][52] + coeff0_16*dmats2[16][52] + coeff0_17*dmats2[17][52] + coeff0_18*dmats2[18][52] + coeff0_19*dmats2[19][52] + coeff0_20*dmats2[20][52] + coeff0_21*dmats2[21][52] + coeff0_22*dmats2[22][52] + coeff0_23*dmats2[23][52] + coeff0_24*dmats2[24][52] + coeff0_25*dmats2[25][52] + coeff0_26*dmats2[26][52] + coeff0_27*dmats2[27][52] + coeff0_28*dmats2[28][52] + coeff0_29*dmats2[29][52] + coeff0_30*dmats2[30][52] + coeff0_31*dmats2[31][52] + coeff0_32*dmats2[32][52] + coeff0_33*dmats2[33][52] + coeff0_34*dmats2[34][52] + coeff0_35*dmats2[35][52] + coeff0_36*dmats2[36][52] + coeff0_37*dmats2[37][52] + coeff0_38*dmats2[38][52] + coeff0_39*dmats2[39][52] + coeff0_40*dmats2[40][52] + coeff0_41*dmats2[41][52] + coeff0_42*dmats2[42][52] + coeff0_43*dmats2[43][52] + coeff0_44*dmats2[44][52] + coeff0_45*dmats2[45][52] + coeff0_46*dmats2[46][52] + coeff0_47*dmats2[47][52] + coeff0_48*dmats2[48][52] + coeff0_49*dmats2[49][52] + coeff0_50*dmats2[50][52] + coeff0_51*dmats2[51][52] + coeff0_52*dmats2[52][52] + coeff0_53*dmats2[53][52] + coeff0_54*dmats2[54][52] + coeff0_55*dmats2[55][52];
          new_coeff0_53 = coeff0_0*dmats2[0][53] + coeff0_1*dmats2[1][53] + coeff0_2*dmats2[2][53] + coeff0_3*dmats2[3][53] + coeff0_4*dmats2[4][53] + coeff0_5*dmats2[5][53] + coeff0_6*dmats2[6][53] + coeff0_7*dmats2[7][53] + coeff0_8*dmats2[8][53] + coeff0_9*dmats2[9][53] + coeff0_10*dmats2[10][53] + coeff0_11*dmats2[11][53] + coeff0_12*dmats2[12][53] + coeff0_13*dmats2[13][53] + coeff0_14*dmats2[14][53] + coeff0_15*dmats2[15][53] + coeff0_16*dmats2[16][53] + coeff0_17*dmats2[17][53] + coeff0_18*dmats2[18][53] + coeff0_19*dmats2[19][53] + coeff0_20*dmats2[20][53] + coeff0_21*dmats2[21][53] + coeff0_22*dmats2[22][53] + coeff0_23*dmats2[23][53] + coeff0_24*dmats2[24][53] + coeff0_25*dmats2[25][53] + coeff0_26*dmats2[26][53] + coeff0_27*dmats2[27][53] + coeff0_28*dmats2[28][53] + coeff0_29*dmats2[29][53] + coeff0_30*dmats2[30][53] + coeff0_31*dmats2[31][53] + coeff0_32*dmats2[32][53] + coeff0_33*dmats2[33][53] + coeff0_34*dmats2[34][53] + coeff0_35*dmats2[35][53] + coeff0_36*dmats2[36][53] + coeff0_37*dmats2[37][53] + coeff0_38*dmats2[38][53] + coeff0_39*dmats2[39][53] + coeff0_40*dmats2[40][53] + coeff0_41*dmats2[41][53] + coeff0_42*dmats2[42][53] + coeff0_43*dmats2[43][53] + coeff0_44*dmats2[44][53] + coeff0_45*dmats2[45][53] + coeff0_46*dmats2[46][53] + coeff0_47*dmats2[47][53] + coeff0_48*dmats2[48][53] + coeff0_49*dmats2[49][53] + coeff0_50*dmats2[50][53] + coeff0_51*dmats2[51][53] + coeff0_52*dmats2[52][53] + coeff0_53*dmats2[53][53] + coeff0_54*dmats2[54][53] + coeff0_55*dmats2[55][53];
          new_coeff0_54 = coeff0_0*dmats2[0][54] + coeff0_1*dmats2[1][54] + coeff0_2*dmats2[2][54] + coeff0_3*dmats2[3][54] + coeff0_4*dmats2[4][54] + coeff0_5*dmats2[5][54] + coeff0_6*dmats2[6][54] + coeff0_7*dmats2[7][54] + coeff0_8*dmats2[8][54] + coeff0_9*dmats2[9][54] + coeff0_10*dmats2[10][54] + coeff0_11*dmats2[11][54] + coeff0_12*dmats2[12][54] + coeff0_13*dmats2[13][54] + coeff0_14*dmats2[14][54] + coeff0_15*dmats2[15][54] + coeff0_16*dmats2[16][54] + coeff0_17*dmats2[17][54] + coeff0_18*dmats2[18][54] + coeff0_19*dmats2[19][54] + coeff0_20*dmats2[20][54] + coeff0_21*dmats2[21][54] + coeff0_22*dmats2[22][54] + coeff0_23*dmats2[23][54] + coeff0_24*dmats2[24][54] + coeff0_25*dmats2[25][54] + coeff0_26*dmats2[26][54] + coeff0_27*dmats2[27][54] + coeff0_28*dmats2[28][54] + coeff0_29*dmats2[29][54] + coeff0_30*dmats2[30][54] + coeff0_31*dmats2[31][54] + coeff0_32*dmats2[32][54] + coeff0_33*dmats2[33][54] + coeff0_34*dmats2[34][54] + coeff0_35*dmats2[35][54] + coeff0_36*dmats2[36][54] + coeff0_37*dmats2[37][54] + coeff0_38*dmats2[38][54] + coeff0_39*dmats2[39][54] + coeff0_40*dmats2[40][54] + coeff0_41*dmats2[41][54] + coeff0_42*dmats2[42][54] + coeff0_43*dmats2[43][54] + coeff0_44*dmats2[44][54] + coeff0_45*dmats2[45][54] + coeff0_46*dmats2[46][54] + coeff0_47*dmats2[47][54] + coeff0_48*dmats2[48][54] + coeff0_49*dmats2[49][54] + coeff0_50*dmats2[50][54] + coeff0_51*dmats2[51][54] + coeff0_52*dmats2[52][54] + coeff0_53*dmats2[53][54] + coeff0_54*dmats2[54][54] + coeff0_55*dmats2[55][54];
          new_coeff0_55 = coeff0_0*dmats2[0][55] + coeff0_1*dmats2[1][55] + coeff0_2*dmats2[2][55] + coeff0_3*dmats2[3][55] + coeff0_4*dmats2[4][55] + coeff0_5*dmats2[5][55] + coeff0_6*dmats2[6][55] + coeff0_7*dmats2[7][55] + coeff0_8*dmats2[8][55] + coeff0_9*dmats2[9][55] + coeff0_10*dmats2[10][55] + coeff0_11*dmats2[11][55] + coeff0_12*dmats2[12][55] + coeff0_13*dmats2[13][55] + coeff0_14*dmats2[14][55] + coeff0_15*dmats2[15][55] + coeff0_16*dmats2[16][55] + coeff0_17*dmats2[17][55] + coeff0_18*dmats2[18][55] + coeff0_19*dmats2[19][55] + coeff0_20*dmats2[20][55] + coeff0_21*dmats2[21][55] + coeff0_22*dmats2[22][55] + coeff0_23*dmats2[23][55] + coeff0_24*dmats2[24][55] + coeff0_25*dmats2[25][55] + coeff0_26*dmats2[26][55] + coeff0_27*dmats2[27][55] + coeff0_28*dmats2[28][55] + coeff0_29*dmats2[29][55] + coeff0_30*dmats2[30][55] + coeff0_31*dmats2[31][55] + coeff0_32*dmats2[32][55] + coeff0_33*dmats2[33][55] + coeff0_34*dmats2[34][55] + coeff0_35*dmats2[35][55] + coeff0_36*dmats2[36][55] + coeff0_37*dmats2[37][55] + coeff0_38*dmats2[38][55] + coeff0_39*dmats2[39][55] + coeff0_40*dmats2[40][55] + coeff0_41*dmats2[41][55] + coeff0_42*dmats2[42][55] + coeff0_43*dmats2[43][55] + coeff0_44*dmats2[44][55] + coeff0_45*dmats2[45][55] + coeff0_46*dmats2[46][55] + coeff0_47*dmats2[47][55] + coeff0_48*dmats2[48][55] + coeff0_49*dmats2[49][55] + coeff0_50*dmats2[50][55] + coeff0_51*dmats2[51][55] + coeff0_52*dmats2[52][55] + coeff0_53*dmats2[53][55] + coeff0_54*dmats2[54][55] + coeff0_55*dmats2[55][55];
        }
    
      }
      // Compute derivatives on reference element as dot product of coefficients and basisvalues
      derivatives[deriv_num] = new_coeff0_0*basisvalue0 + new_coeff0_1*basisvalue1 + new_coeff0_2*basisvalue2 + new_coeff0_3*basisvalue3 + new_coeff0_4*basisvalue4 + new_coeff0_5*basisvalue5 + new_coeff0_6*basisvalue6 + new_coeff0_7*basisvalue7 + new_coeff0_8*basisvalue8 + new_coeff0_9*basisvalue9 + new_coeff0_10*basisvalue10 + new_coeff0_11*basisvalue11 + new_coeff0_12*basisvalue12 + new_coeff0_13*basisvalue13 + new_coeff0_14*basisvalue14 + new_coeff0_15*basisvalue15 + new_coeff0_16*basisvalue16 + new_coeff0_17*basisvalue17 + new_coeff0_18*basisvalue18 + new_coeff0_19*basisvalue19 + new_coeff0_20*basisvalue20 + new_coeff0_21*basisvalue21 + new_coeff0_22*basisvalue22 + new_coeff0_23*basisvalue23 + new_coeff0_24*basisvalue24 + new_coeff0_25*basisvalue25 + new_coeff0_26*basisvalue26 + new_coeff0_27*basisvalue27 + new_coeff0_28*basisvalue28 + new_coeff0_29*basisvalue29 + new_coeff0_30*basisvalue30 + new_coeff0_31*basisvalue31 + new_coeff0_32*basisvalue32 + new_coeff0_33*basisvalue33 + new_coeff0_34*basisvalue34 + new_coeff0_35*basisvalue35 + new_coeff0_36*basisvalue36 + new_coeff0_37*basisvalue37 + new_coeff0_38*basisvalue38 + new_coeff0_39*basisvalue39 + new_coeff0_40*basisvalue40 + new_coeff0_41*basisvalue41 + new_coeff0_42*basisvalue42 + new_coeff0_43*basisvalue43 + new_coeff0_44*basisvalue44 + new_coeff0_45*basisvalue45 + new_coeff0_46*basisvalue46 + new_coeff0_47*basisvalue47 + new_coeff0_48*basisvalue48 + new_coeff0_49*basisvalue49 + new_coeff0_50*basisvalue50 + new_coeff0_51*basisvalue51 + new_coeff0_52*basisvalue52 + new_coeff0_53*basisvalue53 + new_coeff0_54*basisvalue54 + new_coeff0_55*basisvalue55;
    }
    
    // Transform derivatives back to physical element
    for (unsigned int row = 0; row < num_derivatives; row++)
    {
      for (unsigned int col = 0; col < num_derivatives; col++)
      {
        values[row] += transform[row][col]*derivatives[col];
      }
    }
    // Delete pointer to array of derivatives on FIAT element
    delete [] derivatives;
    
    // Delete pointer to array of combinations of derivatives and transform
    for (unsigned int row = 0; row < num_derivatives; row++)
    {
      delete [] combinations[row];
      delete [] transform[row];
    }
    
    delete [] combinations;
    delete [] transform;
  }

  /// Evaluate order n derivatives of all basis functions at given point in cell
  virtual void evaluate_basis_derivatives_all(unsigned int n,
                                              double* values,
                                              const double* coordinates,
                                              const ufc::cell& c) const
  {
    throw std::runtime_error("The vectorised version of evaluate_basis_derivatives() is not yet implemented.");
  }

  /// Evaluate linear functional for dof i on the function f
  virtual double evaluate_dof(unsigned int i,
                              const ufc::function& f,
                              const ufc::cell& c) const
  {
    // The reference points, direction and weights:
    const static double X[56][1][3] = {{{0, 0, 0}}, {{1, 0, 0}}, {{0, 1, 0}}, {{0, 0, 1}}, {{0, 0.8, 0.2}}, {{0, 0.6, 0.4}}, {{0, 0.4, 0.6}}, {{0, 0.2, 0.8}}, {{0.8, 0, 0.2}}, {{0.6, 0, 0.4}}, {{0.4, 0, 0.6}}, {{0.2, 0, 0.8}}, {{0.8, 0.2, 0}}, {{0.6, 0.4, 0}}, {{0.4, 0.6, 0}}, {{0.2, 0.8, 0}}, {{0, 0, 0.2}}, {{0, 0, 0.4}}, {{0, 0, 0.6}}, {{0, 0, 0.8}}, {{0, 0.2, 0}}, {{0, 0.4, 0}}, {{0, 0.6, 0}}, {{0, 0.8, 0}}, {{0.2, 0, 0}}, {{0.4, 0, 0}}, {{0.6, 0, 0}}, {{0.8, 0, 0}}, {{0.6, 0.2, 0.2}}, {{0.4, 0.4, 0.2}}, {{0.2, 0.6, 0.2}}, {{0.4, 0.2, 0.4}}, {{0.2, 0.4, 0.4}}, {{0.2, 0.2, 0.6}}, {{0, 0.2, 0.2}}, {{0, 0.4, 0.2}}, {{0, 0.6, 0.2}}, {{0, 0.2, 0.4}}, {{0, 0.4, 0.4}}, {{0, 0.2, 0.6}}, {{0.2, 0, 0.2}}, {{0.4, 0, 0.2}}, {{0.6, 0, 0.2}}, {{0.2, 0, 0.4}}, {{0.4, 0, 0.4}}, {{0.2, 0, 0.6}}, {{0.2, 0.2, 0}}, {{0.4, 0.2, 0}}, {{0.6, 0.2, 0}}, {{0.2, 0.4, 0}}, {{0.4, 0.4, 0}}, {{0.2, 0.6, 0}}, {{0.2, 0.2, 0.2}}, {{0.4, 0.2, 0.2}}, {{0.2, 0.4, 0.2}}, {{0.2, 0.2, 0.4}}};
    const static double W[56][1] = {{1}, {1}, {1}, {1}, {1}, {1}, {1}, {1}, {1}, {1}, {1}, {1}, {1}, {1}, {1}, {1}, {1}, {1}, {1}, {1}, {1}, {1}, {1}, {1}, {1}, {1}, {1}, {1}, {1}, {1}, {1}, {1}, {1}, {1}, {1}, {1}, {1}, {1}, {1}, {1}, {1}, {1}, {1}, {1}, {1}, {1}, {1}, {1}, {1}, {1}, {1}, {1}, {1}, {1}, {1}, {1}};
    const static double D[56][1][1] = {{{1}}, {{1}}, {{1}}, {{1}}, {{1}}, {{1}}, {{1}}, {{1}}, {{1}}, {{1}}, {{1}}, {{1}}, {{1}}, {{1}}, {{1}}, {{1}}, {{1}}, {{1}}, {{1}}, {{1}}, {{1}}, {{1}}, {{1}}, {{1}}, {{1}}, {{1}}, {{1}}, {{1}}, {{1}}, {{1}}, {{1}}, {{1}}, {{1}}, {{1}}, {{1}}, {{1}}, {{1}}, {{1}}, {{1}}, {{1}}, {{1}}, {{1}}, {{1}}, {{1}}, {{1}}, {{1}}, {{1}}, {{1}}, {{1}}, {{1}}, {{1}}, {{1}}, {{1}}, {{1}}, {{1}}, {{1}}};
    
    const double * const * x = c.coordinates;
    double result = 0.0;
    // Iterate over the points:
    // Evaluate basis functions for affine mapping
    const double w0 = 1.0 - X[i][0][0] - X[i][0][1] - X[i][0][2];
    const double w1 = X[i][0][0];
    const double w2 = X[i][0][1];
    const double w3 = X[i][0][2];
    
    // Compute affine mapping y = F(X)
    double y[3];
    y[0] = w0*x[0][0] + w1*x[1][0] + w2*x[2][0] + w3*x[3][0];
    y[1] = w0*x[0][1] + w1*x[1][1] + w2*x[2][1] + w3*x[3][1];
    y[2] = w0*x[0][2] + w1*x[1][2] + w2*x[2][2] + w3*x[3][2];
    
    // Evaluate function at physical points
    double values[1];
    f.evaluate(values, y, c);
    
    // Map function values using appropriate mapping
    // Affine map: Do nothing
    
    // Note that we do not map the weights (yet).
    
    // Take directional components
    for(int k = 0; k < 1; k++)
      result += values[k]*D[i][0][k];
    // Multiply by weights 
    result *= W[i][0];
    
    return result;
  }

  /// Evaluate linear functionals for all dofs on the function f
  virtual void evaluate_dofs(double* values,
                             const ufc::function& f,
                             const ufc::cell& c) const
  {
    throw std::runtime_error("Not implemented (introduced in UFC v1.1).");
  }

  /// Interpolate vertex values from dof values
  virtual void interpolate_vertex_values(double* vertex_values,
                                         const double* dof_values,
                                         const ufc::cell& c) const
  {
    // Evaluate at vertices and use affine mapping
    vertex_values[0] = dof_values[0];
    vertex_values[1] = dof_values[1];
    vertex_values[2] = dof_values[2];
    vertex_values[3] = dof_values[3];
  }

  /// Return the number of sub elements (for a mixed element)
  virtual unsigned int num_sub_elements() const
  {
    return 1;
  }

  /// Create a new finite element for sub element i (for a mixed element)
  virtual ufc::finite_element* create_sub_element(unsigned int i) const
  {
    return new UFC_Poisson3D_5BilinearForm_finite_element_0();
  }

};

/// This class defines the interface for a finite element.

class UFC_Poisson3D_5BilinearForm_finite_element_1: public ufc::finite_element
{
public:

  /// Constructor
  UFC_Poisson3D_5BilinearForm_finite_element_1() : ufc::finite_element()
  {
    // Do nothing
  }

  /// Destructor
  virtual ~UFC_Poisson3D_5BilinearForm_finite_element_1()
  {
    // Do nothing
  }

  /// Return a string identifying the finite element
  virtual const char* signature() const
  {
    return "Lagrange finite element of degree 5 on a tetrahedron";
  }

  /// Return the cell shape
  virtual ufc::shape cell_shape() const
  {
    return ufc::tetrahedron;
  }

  /// Return the dimension of the finite element function space
  virtual unsigned int space_dimension() const
  {
    return 56;
  }

  /// Return the rank of the value space
  virtual unsigned int value_rank() const
  {
    return 0;
  }

  /// Return the dimension of the value space for axis i
  virtual unsigned int value_dimension(unsigned int i) const
  {
    return 1;
  }

  /// Evaluate basis function i at given point in cell
  virtual void evaluate_basis(unsigned int i,
                              double* values,
                              const double* coordinates,
                              const ufc::cell& c) const
  {
    // Extract vertex coordinates
    const double * const * element_coordinates = c.coordinates;
    
    // Compute Jacobian of affine map from reference cell
    const double J_00 = element_coordinates[1][0] - element_coordinates[0][0];
    const double J_01 = element_coordinates[2][0] - element_coordinates[0][0];
    const double J_02 = element_coordinates[3][0] - element_coordinates[0][0];
    const double J_10 = element_coordinates[1][1] - element_coordinates[0][1];
    const double J_11 = element_coordinates[2][1] - element_coordinates[0][1];
    const double J_12 = element_coordinates[3][1] - element_coordinates[0][1];
    const double J_20 = element_coordinates[1][2] - element_coordinates[0][2];
    const double J_21 = element_coordinates[2][2] - element_coordinates[0][2];
    const double J_22 = element_coordinates[3][2] - element_coordinates[0][2];
      
    // Compute sub determinants
    const double d00 = J_11*J_22 - J_12*J_21;
    const double d01 = J_12*J_20 - J_10*J_22;
    const double d02 = J_10*J_21 - J_11*J_20;
    
    const double d10 = J_02*J_21 - J_01*J_22;
    const double d11 = J_00*J_22 - J_02*J_20;
    const double d12 = J_01*J_20 - J_00*J_21;
    
    const double d20 = J_01*J_12 - J_02*J_11;
    const double d21 = J_02*J_10 - J_00*J_12;
    const double d22 = J_00*J_11 - J_01*J_10;
      
    // Compute determinant of Jacobian
    double detJ = J_00*d00 + J_10*d10 + J_20*d20;
    
    // Compute inverse of Jacobian
    
    // Compute constants
    const double C0 = d00*(element_coordinates[0][0] - element_coordinates[2][0] - element_coordinates[3][0]) \
                    + d10*(element_coordinates[0][1] - element_coordinates[2][1] - element_coordinates[3][1]) \
                    + d20*(element_coordinates[0][2] - element_coordinates[2][2] - element_coordinates[3][2]);
    
    const double C1 = d01*(element_coordinates[0][0] - element_coordinates[1][0] - element_coordinates[3][0]) \
                    + d11*(element_coordinates[0][1] - element_coordinates[1][1] - element_coordinates[3][1]) \
                    + d21*(element_coordinates[0][2] - element_coordinates[1][2] - element_coordinates[3][2]);
    
    const double C2 = d02*(element_coordinates[0][0] - element_coordinates[1][0] - element_coordinates[2][0]) \
                    + d12*(element_coordinates[0][1] - element_coordinates[1][1] - element_coordinates[2][1]) \
                    + d22*(element_coordinates[0][2] - element_coordinates[1][2] - element_coordinates[2][2]);
    
    // Get coordinates and map to the UFC reference element
    double x = (C0 + d00*coordinates[0] + d10*coordinates[1] + d20*coordinates[2]) / detJ;
    double y = (C1 + d01*coordinates[0] + d11*coordinates[1] + d21*coordinates[2]) / detJ;
    double z = (C2 + d02*coordinates[0] + d12*coordinates[1] + d22*coordinates[2]) / detJ;
    
    // Map coordinates to the reference cube
    if (std::abs(y + z - 1.0) < 1e-14)
      x = 1.0;
    else
      x = -2.0 * x/(y + z - 1.0) - 1.0;
    if (std::abs(z - 1.0) < 1e-14)
      y = -1.0;
    else
      y = 2.0 * y/(1.0 - z) - 1.0;
    z = 2.0 * z - 1.0;
    
    // Reset values
    *values = 0;
    
    // Map degree of freedom to element degree of freedom
    const unsigned int dof = i;
    
    // Generate scalings
    const double scalings_y_0 = 1;
    const double scalings_y_1 = scalings_y_0*(0.5 - 0.5*y);
    const double scalings_y_2 = scalings_y_1*(0.5 - 0.5*y);
    const double scalings_y_3 = scalings_y_2*(0.5 - 0.5*y);
    const double scalings_y_4 = scalings_y_3*(0.5 - 0.5*y);
    const double scalings_y_5 = scalings_y_4*(0.5 - 0.5*y);
    const double scalings_z_0 = 1;
    const double scalings_z_1 = scalings_z_0*(0.5 - 0.5*z);
    const double scalings_z_2 = scalings_z_1*(0.5 - 0.5*z);
    const double scalings_z_3 = scalings_z_2*(0.5 - 0.5*z);
    const double scalings_z_4 = scalings_z_3*(0.5 - 0.5*z);
    const double scalings_z_5 = scalings_z_4*(0.5 - 0.5*z);
    
    // Compute psitilde_a
    const double psitilde_a_0 = 1;
    const double psitilde_a_1 = x;
    const double psitilde_a_2 = 1.5*x*psitilde_a_1 - 0.5*psitilde_a_0;
    const double psitilde_a_3 = 1.66666666666667*x*psitilde_a_2 - 0.666666666666667*psitilde_a_1;
    const double psitilde_a_4 = 1.75*x*psitilde_a_3 - 0.75*psitilde_a_2;
    const double psitilde_a_5 = 1.8*x*psitilde_a_4 - 0.8*psitilde_a_3;
    
    // Compute psitilde_bs
    const double psitilde_bs_0_0 = 1;
    const double psitilde_bs_0_1 = 1.5*y + 0.5;
    const double psitilde_bs_0_2 = 0.111111111111111*psitilde_bs_0_1 + 1.66666666666667*y*psitilde_bs_0_1 - 0.555555555555556*psitilde_bs_0_0;
    const double psitilde_bs_0_3 = 0.05*psitilde_bs_0_2 + 1.75*y*psitilde_bs_0_2 - 0.7*psitilde_bs_0_1;
    const double psitilde_bs_0_4 = 0.0285714285714286*psitilde_bs_0_3 + 1.8*y*psitilde_bs_0_3 - 0.771428571428571*psitilde_bs_0_2;
    const double psitilde_bs_0_5 = 0.0185185185185185*psitilde_bs_0_4 + 1.83333333333333*y*psitilde_bs_0_4 - 0.814814814814815*psitilde_bs_0_3;
    const double psitilde_bs_1_0 = 1;
    const double psitilde_bs_1_1 = 2.5*y + 1.5;
    const double psitilde_bs_1_2 = 0.54*psitilde_bs_1_1 + 2.1*y*psitilde_bs_1_1 - 0.56*psitilde_bs_1_0;
    const double psitilde_bs_1_3 = 0.285714285714286*psitilde_bs_1_2 + 2*y*psitilde_bs_1_2 - 0.714285714285714*psitilde_bs_1_1;
    const double psitilde_bs_1_4 = 0.178571428571429*psitilde_bs_1_3 + 1.96428571428571*y*psitilde_bs_1_3 - 0.785714285714286*psitilde_bs_1_2;
    const double psitilde_bs_2_0 = 1;
    const double psitilde_bs_2_1 = 3.5*y + 2.5;
    const double psitilde_bs_2_2 = 1.02040816326531*psitilde_bs_2_1 + 2.57142857142857*y*psitilde_bs_2_1 - 0.551020408163265*psitilde_bs_2_0;
    const double psitilde_bs_2_3 = 0.578703703703704*psitilde_bs_2_2 + 2.29166666666667*y*psitilde_bs_2_2 - 0.712962962962963*psitilde_bs_2_1;
    const double psitilde_bs_3_0 = 1;
    const double psitilde_bs_3_1 = 4.5*y + 3.5;
    const double psitilde_bs_3_2 = 1.51234567901235*psitilde_bs_3_1 + 3.05555555555556*y*psitilde_bs_3_1 - 0.54320987654321*psitilde_bs_3_0;
    const double psitilde_bs_4_0 = 1;
    const double psitilde_bs_4_1 = 5.5*y + 4.5;
    const double psitilde_bs_5_0 = 1;
    
    // Compute psitilde_cs
    const double psitilde_cs_00_0 = 1;
    const double psitilde_cs_00_1 = 2*z + 1;
    const double psitilde_cs_00_2 = 0.3125*psitilde_cs_00_1 + 1.875*z*psitilde_cs_00_1 - 0.5625*psitilde_cs_00_0;
    const double psitilde_cs_00_3 = 0.155555555555556*psitilde_cs_00_2 + 1.86666666666667*z*psitilde_cs_00_2 - 0.711111111111111*psitilde_cs_00_1;
    const double psitilde_cs_00_4 = 0.09375*psitilde_cs_00_3 + 1.875*z*psitilde_cs_00_3 - 0.78125*psitilde_cs_00_2;
    const double psitilde_cs_00_5 = 0.0628571428571429*psitilde_cs_00_4 + 1.88571428571429*z*psitilde_cs_00_4 - 0.822857142857143*psitilde_cs_00_3;
    const double psitilde_cs_01_0 = 1;
    const double psitilde_cs_01_1 = 3*z + 2;
    const double psitilde_cs_01_2 = 0.777777777777778*psitilde_cs_01_1 + 2.33333333333333*z*psitilde_cs_01_1 - 0.555555555555556*psitilde_cs_01_0;
    const double psitilde_cs_01_3 = 0.428571428571429*psitilde_cs_01_2 + 2.14285714285714*z*psitilde_cs_01_2 - 0.714285714285714*psitilde_cs_01_1;
    const double psitilde_cs_01_4 = 0.275*psitilde_cs_01_3 + 2.0625*z*psitilde_cs_01_3 - 0.7875*psitilde_cs_01_2;
    const double psitilde_cs_02_0 = 1;
    const double psitilde_cs_02_1 = 4*z + 3;
    const double psitilde_cs_02_2 = 1.265625*psitilde_cs_02_1 + 2.8125*z*psitilde_cs_02_1 - 0.546875*psitilde_cs_02_0;
    const double psitilde_cs_02_3 = 0.733333333333333*psitilde_cs_02_2 + 2.44444444444444*z*psitilde_cs_02_2 - 0.711111111111111*psitilde_cs_02_1;
    const double psitilde_cs_03_0 = 1;
    const double psitilde_cs_03_1 = 5*z + 4;
    const double psitilde_cs_03_2 = 1.76*psitilde_cs_03_1 + 3.3*z*psitilde_cs_03_1 - 0.54*psitilde_cs_03_0;
    const double psitilde_cs_04_0 = 1;
    const double psitilde_cs_04_1 = 6*z + 5;
    const double psitilde_cs_05_0 = 1;
    const double psitilde_cs_10_0 = 1;
    const double psitilde_cs_10_1 = 3*z + 2;
    const double psitilde_cs_10_2 = 0.777777777777778*psitilde_cs_10_1 + 2.33333333333333*z*psitilde_cs_10_1 - 0.555555555555556*psitilde_cs_10_0;
    const double psitilde_cs_10_3 = 0.428571428571429*psitilde_cs_10_2 + 2.14285714285714*z*psitilde_cs_10_2 - 0.714285714285714*psitilde_cs_10_1;
    const double psitilde_cs_10_4 = 0.275*psitilde_cs_10_3 + 2.0625*z*psitilde_cs_10_3 - 0.7875*psitilde_cs_10_2;
    const double psitilde_cs_11_0 = 1;
    const double psitilde_cs_11_1 = 4*z + 3;
    const double psitilde_cs_11_2 = 1.265625*psitilde_cs_11_1 + 2.8125*z*psitilde_cs_11_1 - 0.546875*psitilde_cs_11_0;
    const double psitilde_cs_11_3 = 0.733333333333333*psitilde_cs_11_2 + 2.44444444444444*z*psitilde_cs_11_2 - 0.711111111111111*psitilde_cs_11_1;
    const double psitilde_cs_12_0 = 1;
    const double psitilde_cs_12_1 = 5*z + 4;
    const double psitilde_cs_12_2 = 1.76*psitilde_cs_12_1 + 3.3*z*psitilde_cs_12_1 - 0.54*psitilde_cs_12_0;
    const double psitilde_cs_13_0 = 1;
    const double psitilde_cs_13_1 = 6*z + 5;
    const double psitilde_cs_14_0 = 1;
    const double psitilde_cs_20_0 = 1;
    const double psitilde_cs_20_1 = 4*z + 3;
    const double psitilde_cs_20_2 = 1.265625*psitilde_cs_20_1 + 2.8125*z*psitilde_cs_20_1 - 0.546875*psitilde_cs_20_0;
    const double psitilde_cs_20_3 = 0.733333333333333*psitilde_cs_20_2 + 2.44444444444444*z*psitilde_cs_20_2 - 0.711111111111111*psitilde_cs_20_1;
    const double psitilde_cs_21_0 = 1;
    const double psitilde_cs_21_1 = 5*z + 4;
    const double psitilde_cs_21_2 = 1.76*psitilde_cs_21_1 + 3.3*z*psitilde_cs_21_1 - 0.54*psitilde_cs_21_0;
    const double psitilde_cs_22_0 = 1;
    const double psitilde_cs_22_1 = 6*z + 5;
    const double psitilde_cs_23_0 = 1;
    const double psitilde_cs_30_0 = 1;
    const double psitilde_cs_30_1 = 5*z + 4;
    const double psitilde_cs_30_2 = 1.76*psitilde_cs_30_1 + 3.3*z*psitilde_cs_30_1 - 0.54*psitilde_cs_30_0;
    const double psitilde_cs_31_0 = 1;
    const double psitilde_cs_31_1 = 6*z + 5;
    const double psitilde_cs_32_0 = 1;
    const double psitilde_cs_40_0 = 1;
    const double psitilde_cs_40_1 = 6*z + 5;
    const double psitilde_cs_41_0 = 1;
    const double psitilde_cs_50_0 = 1;
    
    // Compute basisvalues
    const double basisvalue0 = 0.866025403784439*psitilde_a_0*scalings_y_0*psitilde_bs_0_0*scalings_z_0*psitilde_cs_00_0;
    const double basisvalue1 = 2.73861278752583*psitilde_a_1*scalings_y_1*psitilde_bs_1_0*scalings_z_1*psitilde_cs_10_0;
    const double basisvalue2 = 1.58113883008419*psitilde_a_0*scalings_y_0*psitilde_bs_0_1*scalings_z_1*psitilde_cs_01_0;
    const double basisvalue3 = 1.11803398874989*psitilde_a_0*scalings_y_0*psitilde_bs_0_0*scalings_z_0*psitilde_cs_00_1;
    const double basisvalue4 = 5.1234753829798*psitilde_a_2*scalings_y_2*psitilde_bs_2_0*scalings_z_2*psitilde_cs_20_0;
    const double basisvalue5 = 3.96862696659689*psitilde_a_1*scalings_y_1*psitilde_bs_1_1*scalings_z_2*psitilde_cs_11_0;
    const double basisvalue6 = 2.29128784747792*psitilde_a_0*scalings_y_0*psitilde_bs_0_2*scalings_z_2*psitilde_cs_02_0;
    const double basisvalue7 = 3.24037034920393*psitilde_a_1*scalings_y_1*psitilde_bs_1_0*scalings_z_1*psitilde_cs_10_1;
    const double basisvalue8 = 1.87082869338697*psitilde_a_0*scalings_y_0*psitilde_bs_0_1*scalings_z_1*psitilde_cs_01_1;
    const double basisvalue9 = 1.3228756555323*psitilde_a_0*scalings_y_0*psitilde_bs_0_0*scalings_z_0*psitilde_cs_00_2;
    const double basisvalue10 = 7.93725393319377*psitilde_a_3*scalings_y_3*psitilde_bs_3_0*scalings_z_3*psitilde_cs_30_0;
    const double basisvalue11 = 6.70820393249937*psitilde_a_2*scalings_y_2*psitilde_bs_2_1*scalings_z_3*psitilde_cs_21_0;
    const double basisvalue12 = 5.19615242270663*psitilde_a_1*scalings_y_1*psitilde_bs_1_2*scalings_z_3*psitilde_cs_12_0;
    const double basisvalue13 = 3*psitilde_a_0*scalings_y_0*psitilde_bs_0_3*scalings_z_3*psitilde_cs_03_0;
    const double basisvalue14 = 5.80947501931113*psitilde_a_2*scalings_y_2*psitilde_bs_2_0*scalings_z_2*psitilde_cs_20_1;
    const double basisvalue15 = 4.5*psitilde_a_1*scalings_y_1*psitilde_bs_1_1*scalings_z_2*psitilde_cs_11_1;
    const double basisvalue16 = 2.59807621135332*psitilde_a_0*scalings_y_0*psitilde_bs_0_2*scalings_z_2*psitilde_cs_02_1;
    const double basisvalue17 = 3.67423461417477*psitilde_a_1*scalings_y_1*psitilde_bs_1_0*scalings_z_1*psitilde_cs_10_2;
    const double basisvalue18 = 2.12132034355964*psitilde_a_0*scalings_y_0*psitilde_bs_0_1*scalings_z_1*psitilde_cs_01_2;
    const double basisvalue19 = 1.5*psitilde_a_0*scalings_y_0*psitilde_bs_0_0*scalings_z_0*psitilde_cs_00_3;
    const double basisvalue20 = 11.1242977306435*psitilde_a_4*scalings_y_4*psitilde_bs_4_0*scalings_z_4*psitilde_cs_40_0;
    const double basisvalue21 = 9.81070843517429*psitilde_a_3*scalings_y_3*psitilde_bs_3_1*scalings_z_4*psitilde_cs_31_0;
    const double basisvalue22 = 8.2915619758885*psitilde_a_2*scalings_y_2*psitilde_bs_2_2*scalings_z_4*psitilde_cs_22_0;
    const double basisvalue23 = 6.42261628933256*psitilde_a_1*scalings_y_1*psitilde_bs_1_3*scalings_z_4*psitilde_cs_13_0;
    const double basisvalue24 = 3.70809924354783*psitilde_a_0*scalings_y_0*psitilde_bs_0_4*scalings_z_4*psitilde_cs_04_0;
    const double basisvalue25 = 8.77496438739212*psitilde_a_3*scalings_y_3*psitilde_bs_3_0*scalings_z_3*psitilde_cs_30_1;
    const double basisvalue26 = 7.41619848709566*psitilde_a_2*scalings_y_2*psitilde_bs_2_1*scalings_z_3*psitilde_cs_21_1;
    const double basisvalue27 = 5.74456264653803*psitilde_a_1*scalings_y_1*psitilde_bs_1_2*scalings_z_3*psitilde_cs_12_1;
    const double basisvalue28 = 3.3166247903554*psitilde_a_0*scalings_y_0*psitilde_bs_0_3*scalings_z_3*psitilde_cs_03_1;
    const double basisvalue29 = 6.42261628933256*psitilde_a_2*scalings_y_2*psitilde_bs_2_0*scalings_z_2*psitilde_cs_20_2;
    const double basisvalue30 = 4.9749371855331*psitilde_a_1*scalings_y_1*psitilde_bs_1_1*scalings_z_2*psitilde_cs_11_2;
    const double basisvalue31 = 2.87228132326901*psitilde_a_0*scalings_y_0*psitilde_bs_0_2*scalings_z_2*psitilde_cs_02_2;
    const double basisvalue32 = 4.06201920231798*psitilde_a_1*scalings_y_1*psitilde_bs_1_0*scalings_z_1*psitilde_cs_10_3;
    const double basisvalue33 = 2.34520787991171*psitilde_a_0*scalings_y_0*psitilde_bs_0_1*scalings_z_1*psitilde_cs_01_3;
    const double basisvalue34 = 1.6583123951777*psitilde_a_0*scalings_y_0*psitilde_bs_0_0*scalings_z_0*psitilde_cs_00_4;
    const double basisvalue35 = 14.6458185158768*psitilde_a_5*scalings_y_5*psitilde_bs_5_0*scalings_z_5*psitilde_cs_50_0;
    const double basisvalue36 = 13.2476412994918*psitilde_a_4*scalings_y_4*psitilde_bs_4_1*scalings_z_5*psitilde_cs_41_0;
    const double basisvalue37 = 11.6833214455479*psitilde_a_3*scalings_y_3*psitilde_bs_3_2*scalings_z_5*psitilde_cs_32_0;
    const double basisvalue38 = 9.87420882906575*psitilde_a_2*scalings_y_2*psitilde_bs_2_3*scalings_z_5*psitilde_cs_23_0;
    const double basisvalue39 = 7.64852927038918*psitilde_a_1*scalings_y_1*psitilde_bs_1_4*scalings_z_5*psitilde_cs_14_0;
    const double basisvalue40 = 4.41588043316392*psitilde_a_0*scalings_y_0*psitilde_bs_0_5*scalings_z_5*psitilde_cs_05_0;
    const double basisvalue41 = 12.0933866224478*psitilde_a_4*scalings_y_4*psitilde_bs_4_0*scalings_z_4*psitilde_cs_40_1;
    const double basisvalue42 = 10.6653645038508*psitilde_a_3*scalings_y_3*psitilde_bs_3_1*scalings_z_4*psitilde_cs_31_1;
    const double basisvalue43 = 9.01387818865997*psitilde_a_2*scalings_y_2*psitilde_bs_2_2*scalings_z_4*psitilde_cs_22_1;
    const double basisvalue44 = 6.98212002188447*psitilde_a_1*scalings_y_1*psitilde_bs_1_3*scalings_z_4*psitilde_cs_13_1;
    const double basisvalue45 = 4.03112887414927*psitilde_a_0*scalings_y_0*psitilde_bs_0_4*scalings_z_4*psitilde_cs_04_1;
    const double basisvalue46 = 9.53939201416946*psitilde_a_3*scalings_y_3*psitilde_bs_3_0*scalings_z_3*psitilde_cs_30_2;
    const double basisvalue47 = 8.06225774829855*psitilde_a_2*scalings_y_2*psitilde_bs_2_1*scalings_z_3*psitilde_cs_21_2;
    const double basisvalue48 = 6.2449979983984*psitilde_a_1*scalings_y_1*psitilde_bs_1_2*scalings_z_3*psitilde_cs_12_2;
    const double basisvalue49 = 3.60555127546399*psitilde_a_0*scalings_y_0*psitilde_bs_0_3*scalings_z_3*psitilde_cs_03_2;
    const double basisvalue50 = 6.98212002188447*psitilde_a_2*scalings_y_2*psitilde_bs_2_0*scalings_z_2*psitilde_cs_20_3;
    const double basisvalue51 = 5.40832691319598*psitilde_a_1*scalings_y_1*psitilde_bs_1_1*scalings_z_2*psitilde_cs_11_3;
    const double basisvalue52 = 3.1224989991992*psitilde_a_0*scalings_y_0*psitilde_bs_0_2*scalings_z_2*psitilde_cs_02_3;
    const double basisvalue53 = 4.41588043316392*psitilde_a_1*scalings_y_1*psitilde_bs_1_0*scalings_z_1*psitilde_cs_10_4;
    const double basisvalue54 = 2.54950975679639*psitilde_a_0*scalings_y_0*psitilde_bs_0_1*scalings_z_1*psitilde_cs_01_4;
    const double basisvalue55 = 1.80277563773199*psitilde_a_0*scalings_y_0*psitilde_bs_0_0*scalings_z_0*psitilde_cs_00_5;
    
    // Table(s) of coefficients
    const static double coefficients0[56][56] = \
    {{0.00945067404923504, 0.00609788120679252, 0.003520613356228, 0.00248944957812473, 0.00705946233325038, 0.00546823600997147, 0.00315708753234945, 0.00446479600584757, 0.00257775117585287, 0.00182274533665719, -0.00298267418725718, -0.0025208197798293, -0.00195261860521383, -0.0011273448773449, -0.00218309396769446, -0.00169101731601731, -0.000976309302606924, -0.00138070985681772, -0.000797153207506492, -0.000563672438672437, 0.00278687112460006, 0.00245778931055954, 0.00207721109304565, 0.00160900079398463, 0.000928957041533348, 0.00219831358911339, 0.00185791408306671, 0.00143913406048031, 0.000830884437218262, 0.00160900079398463, 0.00124632665582739, 0.000719567030240153, 0.00101762145320216, 0.000587524019939396, 0.000415442218609115, -0.00705593524717797, -0.0063823335708908, -0.00562868913761196, -0.00475711057319694, -0.00368484200521179, -0.00212744452363026, -0.00582624677716564, -0.00513826668309063, -0.00434262794914383, -0.00336378514516178, -0.00194208225905521, -0.00459580543596519, -0.00388416451811043, -0.00300866089851429, -0.00173705117965752, -0.00336378514516179, -0.00260557676948628, -0.00150433044925714, -0.00212744452363026, -0.00122828066840392, -0.000868525589828755},
    {0.00945067404923484, -0.0060978812067926, 0.00352061335622807, 0.00248944957812473, 0.00705946233325039, -0.00546823600997146, 0.00315708753234942, -0.00446479600584756, 0.00257775117585286, 0.00182274533665715, 0.00298267418725716, -0.00252081977982931, 0.00195261860521383, -0.00112734487734489, -0.00218309396769446, 0.00169101731601731, -0.000976309302606908, 0.00138070985681772, -0.000797153207506487, -0.000563672438672423, 0.00278687112460003, -0.00245778931055954, 0.00207721109304565, -0.00160900079398463, 0.000928957041533354, -0.00219831358911339, 0.00185791408306669, -0.00143913406048031, 0.000830884437218255, 0.00160900079398463, -0.0012463266558274, 0.000719567030240159, -0.00101762145320216, 0.000587524019939409, 0.000415442218609113, 0.00705593524717798, -0.00638233357089081, 0.00562868913761196, -0.00475711057319694, 0.00368484200521178, -0.00212744452363027, -0.00582624677716566, 0.00513826668309062, -0.00434262794914383, 0.00336378514516178, -0.00194208225905522, 0.0045958054359652, -0.00388416451811043, 0.00300866089851429, -0.00173705117965753, -0.00336378514516178, 0.00260557676948629, -0.00150433044925715, 0.00212744452363026, -0.00122828066840393, -0.000868525589828756},
    {0.00945067404923496, 0, -0.00704122671245609, 0.00248944957812476, 0, 0, 0.00947126259704828, 0, -0.00515550235170569, 0.00182274533665715, 0, 0, 0, 0.0045093795093795, 0, 0, -0.00292892790782073, 0, 0.00159430641501296, -0.000563672438672431, 0, 0, 0, 0, 0.00464478520766673, 0, 0, 0, -0.00332353774887306, 0, 0, 0.00215870109072046, 0, -0.00117504803987882, 0.000415442218609132, 0, 0, 0, 0, 0, 0.0127646671417816, 0, 0, 0, 0, -0.00971041129527609, 0, 0, 0, 0.0069482047186301, 0, 0, -0.00451299134777144, 0, 0.00245656133680787, -0.000868525589828766},
    {0.00945067404923495, 0, 0, -0.00746834873437432, 0, 0, 0, 0, 0, 0.0109364720199429, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0.00563672438672439, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0.00623163327913698, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0.0182390373864041},
    {-0.0100234421734309, 0, 0.0374718748311398, -0.0289614183593723, 0, 0, 0.0284137877911449, 0, 0.00515550235170565, -0.00619733414463432, 0, 0, 0, 0.0315656565656566, 0, 0, -0.0113902751970807, 0, 0.00265717735835495, -0.000187890812890785, 0, 0, 0, 0, 0.0232239260383336, 0, 0, 0, -0.00166176887443649, 0, 0, -0.00647610327216139, 0, 0.00646276421933346, -0.00290809553026393, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0.0213629048496073, 0, 0, 0, -0.0277928188745204, 0, 0, 0.0243701532779658, 0, -0.0157219925555703, 0.00607967912880134},
    {0.0100234421734308, 0, -0.0287574853355261, 0.05360943398437, 0, 0, -0.0189425251940965, 0, 0.0154665070551172, 0.0153110608279201, 0, 0, 0, -0.00450937950937953, 0, 0, 0.0370997534990628, 0, -0.0281660799985623, 0.0108976671476671, 0, 0, 0, 0, 0, 0, 0, 0, 0.0299118397398575, 0, 0, -0.00431740218144094, 0, -0.0123380044187275, 0.00872428659079179, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0.0312669212338355, 0, 0, -0.0487403065559315, 0, 0.0412702304583721, -0.018239037386404},
    {0.0100234421734309, 0, 0.0409576306293854, -0.0449826285156208, 0, 0, 0.0189425251940966, 0, -0.00515550235170581, -0.021143845905223, 0, 0, 0, 0, 0, 0, 0.00455611007883228, 0, 0.0356061766019561, -0.0319414381914382, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0.0302218152700864, 0, 0.00411266813957589, -0.0145404776513196, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0.0379091273212801, 0, -0.0550269739444961, 0.0303983956440067},
    {-0.0100234421734309, 0, -0.0148144621425437, 0.0449826285156208, 0, 0, 0, 0, 0.0206220094068229, 0.021143845905223, 0, 0, 0, 0, 0, 0, 0, 0, 0.0106287094334197, 0.0319414381914382, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0.0205633406978791, 0.0145404776513196, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0.03439185871531, -0.0303983956440067},
    {-0.0100234421734309, 0.0324515955311978, -0.01873593741557, -0.0289614183593723, 0.0211783869997511, -0.0164047080299144, 0.00947126259704827, 0.00446479600584751, -0.00257775117585286, -0.00619733414463429, 0.0208787193108001, -0.0176457384588051, 0.0136683302364968, -0.00789141414141412, -0.00848980987436741, 0.00657617845117844, -0.00379675839902691, 0.0023011830946962, -0.00132858867917746, -0.000187890812890819, 0.0139343556230002, -0.0122889465527977, 0.0103860554652283, -0.00804500396992315, 0.00464478520766672, -0.00109915679455667, 0.000928957041533349, -0.000719567030240153, 0.000415442218609154, -0.00482700238195389, 0.00373897996748218, -0.00215870109072046, 0.00559691799261187, -0.00323138210966673, -0.00290809553026394, 0, 0, 0, 0, 0, 0, 0.0128177429097644, -0.0113041867027994, 0.00955378148811641, -0.00740032731935592, 0.00427258096992148, -0.0183832217438608, 0.0155366580724417, -0.0120346435940572, 0.00694820471863012, 0.0181644397838736, -0.014070114555226, 0.00812338442598859, -0.0136156449512337, 0.00786099627778517, 0.00607967912880136},
    {0.0100234421734311, -0.024904712849524, 0.014378742667763, 0.0536094339843701, -0.0141189246665008, 0.0109364720199429, -0.00631417506469887, 0.0133943880175428, -0.00773325352755854, 0.01531106082792, -0.00298267418725714, 0.0025208197798293, -0.00195261860521383, 0.00112734487734485, 0.0276525235907967, -0.0214195526695527, 0.0123665844996876, -0.0243925408037797, 0.0140830399992812, 0.0108976671476671, 0, 0, 0, 0, 0, 0.0197848223020205, -0.0167212267476003, 0.0129522065443228, -0.0074779599349644, -0.00321800158796925, 0.00249265331165481, -0.00143913406048031, -0.0106850252586226, 0.00616900220936375, 0.00872428659079183, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0.0206811244618434, -0.017478740331497, 0.0135389740433143, -0.00781673030845889, -0.0363288795677472, 0.028140229110452, -0.0162467688519772, 0.0357410679969885, -0.0206351152291861, -0.0182390373864041},
    {0.0100234421734311, 0.0354703486038675, -0.0204788153146928, -0.0449826285156207, 0.0141189246665009, -0.0109364720199429, 0.00631417506469889, -0.0044647960058475, 0.00257775117585279, -0.021143845905223, 0, 0, 0, 0, 0.00339592394974698, -0.00263047138047136, 0.00151870335961076, 0.0308358534689291, -0.0178030883009781, -0.0319414381914382, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0.0225260111157848, -0.0174485731815835, 0.0100739384233621, 0.00356167508620756, -0.00205633406978792, -0.0145404776513196, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0.028255795219359, -0.0218868448636849, 0.01263637577376, -0.047654757329318, 0.0275134869722481, 0.0303983956440068},
    {-0.0100234421734311, -0.0128297005588457, 0.00740723107127181, 0.0449826285156207, 0, 0, 0, 0.0178591840233903, -0.0103110047034114, 0.0211438459052229, 0, 0, 0, 0, 0, 0, 0, 0.0092047323787848, -0.00531435471670985, 0.0319414381914382, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0.0178083754310377, -0.0102816703489396, 0.0145404776513196, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0.0297842233308237, -0.017195929357655, -0.0303983956440068},
    {-0.0100234421734311, 0.0324515955311977, -0.0335503995581136, -0.00801060507812428, 0.0211783869997512, 0.00109364720199428, -0.00568275755822898, -0.0169662248222208, 0.00567105258687634, 0.0083846285486229, 0.0208787193108001, -0.0109235523792603, 0.00499002532443531, -0.00187890812890811, -0.0162519217595033, 0.00958243145743145, -0.00466458889023307, 0.0108922666482287, -0.00558007245254534, -0.00469727032227035, 0.0139343556230002, -0.00245778931055952, -0.00415442218609131, 0.0064360031759385, -0.00464478520766676, -0.0120907247401236, 0.00464478520766674, -0.000719567030240154, -0.000415442218609135, 0.00965400476390777, -0.00498530662330959, 0.00215870109072046, -0.00661453944581401, 0.0032313821096667, 0.00290809553026394, 0, 0.0127646671417816, -0.0202632808954031, 0.0228341307513453, -0.020635115229186, 0.0127646671417816, -0.00116524935543312, -0.00719357335632687, 0.0112908326677739, -0.0114368694935501, 0.00737991258440983, 0.00183832217438608, 0.00310733161448834, -0.00481385743762287, 0.00347410235931505, -0.00201827108709706, -0.000521115353897253, 0.000902598269554298, 0.00170195561890421, -0.000491312267361565, -0.000868525589828764},
    {0.0100234421734313, -0.0249047128495239, 0.0553363732971484, -0.00431340273437463, -0.0141189246665008, 0.0153110608279201, 0.00883984509057839, 0.00803663281052565, -0.0159820572902878, 0.000729098134662871, -0.00298267418725714, 0.0277290175781224, -0.0301571095694136, 0.0191648629148629, -0.00145539597846295, -0.0116492303992304, 0.00976309302606917, 0.00352848074520082, 0.00132858867917745, -0.00263047138047135, 0, 0.0196623144844763, -0.00727023882565982, -0.00724050357293083, 0.00928957041533347, -0.00219831358911338, -0.0148633126645336, 0.0129522065443228, -0.00664707549774609, 0.00402250198496158, 0.00685479660705067, -0.00575653624192123, -0.00457929653940971, -0.000293762009969677, 0.0029080955302639, 0, 0, 0.0202632808954031, -0.0399597288148543, 0.0464290092656686, -0.031911667854454, 0, -0.00411061334647249, -0.00303983956440069, 0.00908221989193683, -0.00776832903622088, 0.000459580543596524, 0.00427258096992149, -0.00270779480866288, 0.000868525589828788, -0.00100913554354855, -0.00234501909253767, 0.00180519653910856, 0.00127646671417817, 0.000245656133680783, -0.000868525589828766},
    {0.0100234421734309, 0.0354703486038674, -0.0492363006502187, -0.00431340273437457, 0.0141189246665008, -0.00656188321196574, -0.016416855168217, -0.00982255121286468, 0.0149509568199466, 0.00072909813466289, 0, 0.00336109303977242, 0.0282044909641998, -0.0353234728234728, -0.000485131992821028, -0.0124007936507936, 0.00889526253486298, 0.00291483191994853, 0.00239145962251946, -0.0026304713804714, 0, 0, 0.0218107164769794, -0.00402250198496158, -0.00928957041533346, 0, -0.00557374224920008, -0.0143913406048031, 0.0141250354327105, 0.000804500396992295, 0.00934744991870546, -0.00287826812096058, -0.00254405363300538, -0.00381890612960614, 0.00290809553026392, 0, 0, 0, 0.0266398192099029, -0.0515877880729651, 0.0425488904726054, 0, 0, -0.00911951869320201, 0.0084094628629044, -0.0038841645181104, 0, 0.00233049871086625, 0.00300866089851431, -0.00347410235931508, -0.000336378514516169, -0.00286613444643491, 0.00120346435940571, 0.000850977809452102, 0.000982624534723146, -0.000868525589828757},
    {-0.0100234421734311, -0.0128297005588456, 0.0448791059024117, -0.00801060507812423, 0, 0.0174983552319087, 0.0132597676358676, -0.00357183680467805, -0.0175287079957994, 0.00838462854862291, 0, 0, 0.00867830491206144, 0.0225468975468975, 0, -0.00300625300625305, -0.019200749617936, 0.000613648825252313, 0.0122230158484327, -0.00469727032227032, 0, 0, 0, 0.0160900079398463, 0.00464478520766673, 0, 0, -0.00719567030240156, -0.0108014976838374, 0, 0.00249265331165479, 0.0107935054536023, -0.000508810726601082, -0.00734405024924251, 0.00290809553026393, 0, 0, 0, 0, 0.0257938940364825, -0.031911667854454, 0, 0, 0, -0.0134551405806471, 0.0135945758133865, 0, 0, 0.00601732179702858, -0.00347410235931504, 0, -0.00208446141558903, -0.000902598269554288, 0.000425488904726059, 0.0017195929357655, -0.000868525589828767},
    {-0.010023442173431, -0.0324515955311979, -0.0187359374155701, -0.0289614183593722, 0.0211783869997511, 0.0164047080299144, 0.00947126259704825, -0.0044647960058475, -0.00257775117585286, -0.00619733414463431, -0.0208787193108001, -0.0176457384588052, -0.0136683302364968, -0.00789141414141413, -0.00848980987436743, -0.00657617845117846, -0.00379675839902691, -0.00230118309469614, -0.00132858867917743, -0.000187890812890788, 0.0139343556230002, 0.0122889465527977, 0.0103860554652283, 0.00804500396992312, 0.00464478520766673, 0.00109915679455666, 0.00092895704153335, 0.000719567030240155, 0.000415442218609149, -0.0048270023819539, -0.00373897996748222, -0.00215870109072046, -0.00559691799261192, -0.00323138210966673, -0.00290809553026393, 0, 0, 0, 0, 0, 0, 0.0128177429097644, 0.0113041867027994, 0.00955378148811642, 0.00740032731935589, 0.00427258096992148, 0.0183832217438608, 0.0155366580724417, 0.0120346435940572, 0.00694820471863011, 0.0181644397838736, 0.014070114555226, 0.00812338442598858, 0.0136156449512337, 0.00786099627778515, 0.00607967912880134},
    {0.0100234421734311, 0.024904712849524, 0.014378742667763, 0.05360943398437, -0.0141189246665008, -0.0109364720199429, -0.00631417506469881, -0.0133943880175428, -0.00773325352755853, 0.01531106082792, 0.00298267418725717, 0.0025208197798293, 0.00195261860521382, 0.00112734487734485, 0.0276525235907967, 0.0214195526695527, 0.0123665844996876, 0.0243925408037797, 0.0140830399992811, 0.0108976671476671, 0, 0, 0, 0, 0, -0.0197848223020205, -0.0167212267476003, -0.0129522065443227, -0.0074779599349644, -0.00321800158796927, -0.00249265331165477, -0.00143913406048032, 0.0106850252586227, 0.00616900220936373, 0.00872428659079179, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, -0.0206811244618434, -0.017478740331497, -0.0135389740433143, -0.00781673030845888, -0.0363288795677472, -0.028140229110452, -0.0162467688519772, -0.0357410679969885, -0.020635115229186, -0.018239037386404},
    {0.0100234421734307, -0.0354703486038675, -0.0204788153146927, -0.0449826285156207, 0.0141189246665008, 0.0109364720199429, 0.00631417506469886, 0.00446479600584751, 0.00257775117585285, -0.0211438459052229, 0, 0, 0, 0, 0.00339592394974697, 0.00263047138047138, 0.00151870335961076, -0.0308358534689291, -0.017803088300978, -0.0319414381914382, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0.0225260111157848, 0.0174485731815835, 0.0100739384233621, -0.00356167508620757, -0.00205633406978789, -0.0145404776513196, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0.0282557952193589, 0.0218868448636849, 0.01263637577376, 0.047654757329318, 0.027513486972248, 0.0303983956440067},
    {-0.010023442173431, 0.0128297005588457, 0.00740723107127182, 0.0449826285156208, 0, 0, 0, -0.0178591840233902, -0.0103110047034114, 0.021143845905223, 0, 0, 0, 0, 0, 0, 0, -0.00920473237878481, -0.00531435471670985, 0.0319414381914381, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, -0.0178083754310377, -0.0102816703489396, 0.0145404776513196, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, -0.0297842233308237, -0.017195929357655, -0.0303983956440068},
    {-0.0100234421734306, -0.0324515955311977, -0.0335503995581136, -0.00801060507812422, 0.0211783869997512, -0.00109364720199428, -0.00568275755822895, 0.0169662248222208, 0.00567105258687631, 0.00838462854862288, -0.0208787193108001, -0.0109235523792604, -0.00499002532443533, -0.00187890812890813, -0.0162519217595033, -0.00958243145743147, -0.00466458889023305, -0.0108922666482287, -0.00558007245254534, -0.00469727032227036, 0.0139343556230002, 0.00245778931055952, -0.00415442218609133, -0.00643600317593853, -0.00464478520766674, 0.0120907247401236, 0.00464478520766673, 0.000719567030240151, -0.000415442218609145, 0.00965400476390776, 0.00498530662330958, 0.00215870109072046, 0.006614539445814, 0.00323138210966669, 0.00290809553026394, 0, 0.0127646671417816, 0.0202632808954031, 0.0228341307513453, 0.020635115229186, 0.0127646671417816, -0.00116524935543314, 0.00719357335632687, 0.0112908326677739, 0.01143686949355, 0.00737991258440983, -0.00183832217438609, 0.00310733161448835, 0.00481385743762286, 0.00347410235931504, -0.00201827108709706, 0.00052111535389726, 0.000902598269554298, -0.00170195561890421, -0.000491312267361563, -0.000868525589828763},
    {0.0100234421734315, 0.024904712849524, 0.0553363732971485, -0.00431340273437464, -0.0141189246665008, -0.0153110608279201, 0.00883984509057836, -0.00803663281052558, -0.0159820572902878, 0.000729098134662864, 0.00298267418725715, 0.0277290175781224, 0.0301571095694136, 0.0191648629148629, -0.00145539597846298, 0.0116492303992304, 0.00976309302606916, -0.00352848074520084, 0.00132858867917744, -0.00263047138047133, 0, -0.0196623144844763, -0.0072702388256598, 0.00724050357293084, 0.0092895704153335, 0.00219831358911338, -0.0148633126645336, -0.0129522065443228, -0.0066470754977461, 0.00402250198496159, -0.00685479660705068, -0.00575653624192122, 0.0045792965394097, -0.000293762009969664, 0.00290809553026391, 0, 0, -0.0202632808954031, -0.0399597288148543, -0.0464290092656686, -0.031911667854454, 0, 0.00411061334647249, -0.00303983956440066, -0.0090822198919368, -0.00776832903622086, -0.000459580543596519, 0.0042725809699215, 0.00270779480866287, 0.000868525589828796, -0.00100913554354854, 0.00234501909253766, 0.00180519653910856, -0.00127646671417816, 0.00024565613368078, -0.00086852558982877},
    {0.0100234421734312, -0.0354703486038675, -0.0492363006502186, -0.0043134027343746, 0.0141189246665008, 0.00656188321196572, -0.016416855168217, 0.0098225512128646, 0.0149509568199467, 0.000729098134662864, 0, 0.0033610930397724, -0.0282044909641998, -0.0353234728234728, -0.000485131992820996, 0.0124007936507936, 0.00889526253486298, -0.00291483191994851, 0.00239145962251946, -0.00263047138047141, 0, 0, 0.0218107164769794, 0.00402250198496156, -0.00928957041533348, 0, -0.0055737422492001, 0.0143913406048031, 0.0141250354327105, 0.00080450039699229, -0.00934744991870546, -0.00287826812096059, 0.00254405363300538, -0.00381890612960615, 0.00290809553026392, 0, 0, 0, 0.0266398192099029, 0.0515877880729651, 0.0425488904726054, 0, 0, -0.00911951869320204, -0.00840946286290444, -0.00388416451811041, 0, 0.00233049871086624, -0.0030086608985143, -0.00347410235931509, -0.000336378514516169, 0.00286613444643491, 0.00120346435940572, -0.000850977809452107, 0.000982624534723155, -0.000868525589828754},
    {-0.010023442173431, 0.0128297005588457, 0.0448791059024117, -0.00801060507812423, 0, -0.0174983552319087, 0.0132597676358676, 0.0035718368046781, -0.0175287079957995, 0.00838462854862291, 0, 0, -0.00867830491206147, 0.0225468975468975, 0, 0.00300625300625302, -0.019200749617936, -0.000613648825252327, 0.0122230158484327, -0.00469727032227032, 0, 0, 0, -0.0160900079398463, 0.00464478520766674, 0, 0, 0.00719567030240152, -0.0108014976838374, 0, -0.00249265331165479, 0.0107935054536023, 0.000508810726601078, -0.00734405024924251, 0.00290809553026392, 0, 0, 0, 0, -0.0257938940364826, -0.031911667854454, 0, 0, 0, 0.0134551405806471, 0.0135945758133865, 0, 0, -0.00601732179702859, -0.00347410235931504, 0, 0.00208446141558904, -0.000902598269554294, -0.000425488904726048, 0.0017195929357655, -0.000868525589828768},
    {-0.0100234421734308, -0.0452812960900435, -0.0113287063442981, -0.00801060507812425, -0.00141189246665006, 0.0164047080299144, 0.0145226026488073, 0.0133943880175427, 0.0118576554089232, 0.0083846285486229, -0.00894802256177148, -0.0142846454190327, -0.014536160727703, -0.00939454064454062, -0.0123708658169354, -0.0125886844636845, -0.00813591085505763, -0.0102786178229763, -0.00664294339588729, -0.00469727032227033, -0.00836061337380013, 0.00245778931055953, 0.00830884437218264, 0.00965400476390776, 0.00650269929073342, 0.00219831358911336, 0.00743165633226678, 0.00863480436288185, 0.00581619106052787, 0.00643600317593851, 0.00747795993496438, 0.00503696921168107, 0.00610572871921291, 0.00411266813957582, 0.00290809553026393, 0.0352796762358899, 0.0191470007126724, 0.00788016479265675, 0.000951422114639387, -0.00221090520312706, -0.00212744452363027, 0.017478740331497, 0.00719357335632688, 0.000868525589828766, -0.00201827108709706, -0.00194208225905523, 0.00643412761035128, 0.00077683290362208, -0.00180519653910858, -0.00173705117965754, 0.000672757029032357, -0.00156334606169177, -0.00150433044925715, -0.00127646671417815, -0.00122828066840394, -0.000868525589828763},
    {0.010023442173431, 0.0603750614533912, -0.00610007264692973, -0.0043134027343746, -0.00564756986660029, -0.0218729440398858, 0.00126283501293976, -0.0178591840233903, 0.00103110047034112, 0.00072909813466285, 0.0417574386216002, 0.0168054651988621, -0.000867830491206138, -0.00526094276094278, 0.0145539597846298, -0.000751563251563223, -0.00455611007883228, -0.000613648825252315, -0.00372004830169692, -0.00263047138047137, 0.00557374224920009, -0.0196623144844763, -0.0103860554652283, 0.00321800158796927, 0.00650269929073345, -0.0175865087129071, -0.00928957041533348, 0.00287826812096062, 0.00581619106052783, -0.00804500396992314, 0.0024926533116548, 0.00503696921168107, 0.00203524290640432, 0.00411266813957582, 0.00290809553026392, -0.0705593524717798, -0.0127646671417816, 0.00450295131008957, 0.00380568845855756, -0.000736968401042374, -0.00212744452363027, -0.0116524935543313, 0.0041106133464725, 0.00347410235931506, -0.000672757029032359, -0.0019420822590552, 0.00367664434877216, 0.00310733161448835, -0.000601732179702866, -0.00173705117965752, 0.00269102811612943, -0.000521115353897279, -0.00150433044925714, -0.000425488904726059, -0.00122828066840393, -0.000868525589828762},
    {0.010023442173431, -0.0603750614533913, -0.00610007264692972, -0.00431340273437459, -0.00564756986660034, 0.0218729440398858, 0.00126283501293975, 0.0178591840233902, 0.00103110047034114, 0.000729098134662857, -0.0417574386216002, 0.016805465198862, 0.000867830491206142, -0.00526094276094274, 0.0145539597846298, 0.000751563251563242, -0.00455611007883226, 0.000613648825252326, -0.0037200483016969, -0.00263047138047136, 0.0055737422492001, 0.0196623144844763, -0.0103860554652283, -0.00321800158796928, 0.00650269929073344, 0.0175865087129071, -0.00928957041533347, -0.00287826812096062, 0.00581619106052785, -0.00804500396992314, -0.0024926533116548, 0.00503696921168109, -0.00203524290640433, 0.00411266813957582, 0.00290809553026392, 0.0705593524717798, -0.0127646671417816, -0.00450295131008957, 0.00380568845855755, 0.000736968401042381, -0.00212744452363028, -0.0116524935543313, -0.00411061334647249, 0.00347410235931505, 0.00067275702903236, -0.00194208225905522, -0.00367664434877216, 0.00310733161448835, 0.000601732179702862, -0.00173705117965754, 0.00269102811612941, 0.000521115353897267, -0.00150433044925715, 0.000425488904726055, -0.00122828066840394, -0.00086852558982877},
    {-0.0100234421734311, 0.0452812960900435, -0.0113287063442981, -0.00801060507812424, -0.00141189246665007, -0.0164047080299144, 0.0145226026488074, -0.0133943880175427, 0.0118576554089232, 0.00838462854862294, 0.00894802256177145, -0.0142846454190327, 0.014536160727703, -0.00939454064454065, -0.0123708658169354, 0.0125886844636845, -0.00813591085505765, 0.0102786178229763, -0.00664294339588732, -0.00469727032227034, -0.00836061337380011, -0.00245778931055953, 0.00830884437218264, -0.00965400476390776, 0.00650269929073342, -0.00219831358911338, 0.00743165633226677, -0.00863480436288184, 0.00581619106052786, 0.00643600317593851, -0.00747795993496437, 0.00503696921168106, -0.0061057287192129, 0.00411266813957581, 0.00290809553026391, -0.0352796762358899, 0.0191470007126724, -0.00788016479265674, 0.000951422114639392, 0.00221090520312706, -0.00212744452363026, 0.017478740331497, -0.00719357335632687, 0.000868525589828777, 0.00201827108709706, -0.00194208225905521, -0.00643412761035126, 0.000776832903622074, 0.00180519653910858, -0.00173705117965752, 0.00067275702903235, 0.00156334606169178, -0.00150433044925714, 0.00127646671417815, -0.00122828066840393, -0.000868525589828756},
    {0.0787556170769578, 0.0679219441350652, -0.00435719474780692, -0.00308100195312471, 0.0705946233325039, -0.0328094160598287, 0.0126283501293978, -0.0267887760350854, 0.03608851646194, -0.00729098134662863, 0.0298267418725716, 0.0151249186789758, -0.0325436434202305, 0.0248015873015872, 0.0130985638061668, -0.0229226791726792, 0.0169226945785199, -0.0248527774227189, 0.0164744996218005, 0.0142797017797018, 0, 0.0245778931055953, -0.0363511941282991, 0.0362025178646542, -0.0232239260383337, 0.0219831358911338, -0.00743165633226679, 0, 0.00166176887443652, -0.0345935170706695, 0.0130864298861876, -0.00359783515120077, 0.0335815079556711, -0.014688100498485, -0.0178640154001927, 0, 0, 0, 0, 0, 0, 0, 0.0226083734055987, -0.0334382352084074, 0.0333014729371016, -0.0213629048496074, -0.00459580543596519, -0.0194208225905522, 0.0270779480866286, -0.0191075629762328, 0.0090822198919368, 0.00703505727761299, -0.00812338442598857, -0.0102117337134253, 0.00196524906944628, 0.00607967912880137},
    {-0.0214788046573525, 0.00377344134083702, 0.00217859737390337, -0.021567013671873, 0, 0.0984282481794862, -0.0852413633734345, -0.0312535720409329, -0.0180442582309699, 0.0309866707231716, 0, 0.0453747560369276, 0, -0.0202922077922077, -0.0116431678277039, 0.0236742424242425, -0.0335199527228374, -0.00138070985681772, -0.000797153207506455, 0.00394570707070704, 0, 0, 0.0363511941282991, -0.0603375297744235, 0.0464478520766673, 0, 0.0241528830798671, 0, -0.0108014976838374, -0.00724050357293081, -0.0392592896585629, 0.0259044130886455, 0.0188259968842399, 0.0108691943688789, -0.0178640154001926, 0, 0, 0, 0, 0, 0, 0, 0, 0.0334382352084074, -0.0555024548951693, 0.0427258096992148, 0, -0.0155366580724417, 0, 0.00694820471863008, 0.00302740663064561, 0.0164151336477636, -0.0108311792346514, -0.00680782247561684, -0.00393049813889257, 0.00607967912880131},
    {0.0787556170769581, 0.0301875307266957, 0.0610007264692977, -0.00308100195312485, 0, 0.0218729440398858, 0.0757701007763862, 0.0178591840233902, -0.0412440188136458, -0.00729098134662867, 0, 0, 0.0520698294723689, -0.00901875901875909, 0, -0.0127765752765753, 0.0286384062098029, 0.00184094647575695, -0.0297603864135753, 0.0142797017797018, 0, 0, 0, 0.0402250198496157, -0.0464478520766674, 0, 0, 0.014391340604803, 0.0182794576188018, 0, -0.0137095932141014, -0.0345392174515274, 0.00407048581280862, 0.036426489236243, -0.0178640154001927, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0.0370016365967796, -0.0427258096992148, 0, 0, -0.030086608985143, 0.0243187165152054, 0, 0.014070114555226, 0, -0.00340391123780844, -0.00982624534723146, 0.00607967912880138},
    {-0.0214788046573519, 0.00377344134083692, -0.0196073763651313, 0.00924300585937408, 0, -0.0109364720199429, 0.00947126259704824, 0.102690308134494, -0.0695992817480272, -0.060150596109686, 0, -0.00504163955965862, 0.0065087286840461, -0.00450937950937948, 0.0465726713108155, 0.0214195526695527, -0.0289638426440052, -0.00782402252196701, -0.0263060558477138, 0.000563672438672498, 0, 0, 0, 0, 0, 0, 0.0334424534952005, -0.0431740218144092, 0.0299118397398575, 0.0289620142917233, -0.00249265331165477, -0.00431740218144094, -0.0605484764655283, 0.0185070066280912, 0.0461140862656136, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0.0349574806629939, -0.0451299134777144, 0.0312669212338355, -0.0121096265225824, -0.028140229110452, 0.0270779480866286, 0.023827378664659, 0, -0.0182390373864041},
    {-0.0214788046573522, -0.0150937653633478, 0.0130715842434209, 0.00924300585937409, 0, -0.0109364720199429, 0.00947126259704823, -0.00892959201169517, 0.123732056440937, -0.060150596109686, 0, 0, -0.00650872868404615, 0.00676406926406924, 0, 0.0574945887445888, 0.0126920209338899, -0.026693723898476, 0.00637722566005184, 0.000563672438672442, 0, 0, 0, 0, 0, 0, 0, 0.0431740218144092, -0.0448677596097863, 0, 0.0199412264932383, 0.0215870109072046, -0.0142467003448301, -0.0616900220936373, 0.0461140862656136, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0.0451299134777144, -0.0469003818507533, 0, -0.0375203054806026, 0.0162467688519772, 0.0119136893323295, 0.0206351152291861, -0.0182390373864041},
    {0.078755617076958, 0.0301875307266957, 0.0174287789912279, 0.0585390371093694, 0, 0.0218729440398858, -0.0189425251940966, 0.0178591840233902, 0.0103110047034113, 0.083846285486229, 0, 0, 0, 0, 0, 0.00526094276094275, -0.00455611007883227, 0.0533874477969518, 0.0308232573569172, -0.0319414381914382, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0.0348971463631671, -0.0302218152700865, 0.0284934006896603, 0.0164506725583033, -0.0643935438844155, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0.0437736897273697, -0.0379091273212801, -0.023827378664659, -0.0137567434861241, 0.0303983956440068},
    {0.0787556170769583, -0.0679219441350652, -0.00435719474780694, -0.00308100195312467, 0.0705946233325039, 0.0328094160598287, 0.0126283501293977, 0.0267887760350853, 0.03608851646194, -0.00729098134662864, -0.0298267418725716, 0.0151249186789759, 0.0325436434202305, 0.0248015873015873, 0.0130985638061668, 0.0229226791726792, 0.0169226945785199, 0.0248527774227189, 0.0164744996218005, 0.0142797017797018, 0, -0.0245778931055954, -0.036351194128299, -0.0362025178646541, -0.0232239260383336, -0.0219831358911338, -0.00743165633226678, 0, 0.00166176887443654, -0.0345935170706695, -0.0130864298861876, -0.00359783515120077, -0.0335815079556711, -0.014688100498485, -0.0178640154001927, 0, 0, 0, 0, 0, 0, 0, -0.0226083734055987, -0.0334382352084074, -0.0333014729371016, -0.0213629048496074, 0.00459580543596522, -0.0194208225905522, -0.0270779480866286, -0.0191075629762328, 0.0090822198919368, -0.007035057277613, -0.00812338442598858, 0.0102117337134253, 0.00196524906944627, 0.00607967912880136},
    {-0.0214788046573522, -0.00377344134083698, 0.00217859737390352, -0.0215670136718729, 0, -0.0984282481794863, -0.0852413633734345, 0.031253572040933, -0.0180442582309699, 0.0309866707231716, 0, 0.0453747560369276, 0, -0.0202922077922078, -0.0116431678277039, -0.0236742424242425, -0.0335199527228375, 0.00138070985681774, -0.000797153207506485, 0.00394570707070704, 0, 0, 0.0363511941282991, 0.0603375297744236, 0.0464478520766674, 0, 0.024152883079867, 0, -0.0108014976838374, -0.0072405035729308, 0.0392592896585629, 0.0259044130886455, -0.0188259968842399, 0.0108691943688789, -0.0178640154001927, 0, 0, 0, 0, 0, 0, 0, 0, 0.0334382352084074, 0.0555024548951693, 0.0427258096992148, 0, -0.0155366580724417, 0, 0.0069482047186301, 0.0030274066306456, -0.0164151336477636, -0.0108311792346515, 0.00680782247561683, -0.00393049813889256, 0.00607967912880134},
    {0.078755617076958, -0.0301875307266958, 0.0610007264692976, -0.00308100195312472, 0, -0.0218729440398859, 0.0757701007763862, -0.0178591840233903, -0.0412440188136458, -0.00729098134662865, 0, 0, -0.0520698294723688, -0.00901875901875904, 0, 0.0127765752765752, 0.0286384062098029, -0.00184094647575696, -0.0297603864135752, 0.0142797017797018, 0, 0, 0, -0.0402250198496157, -0.0464478520766673, 0, 0, -0.0143913406048031, 0.0182794576188018, 0, 0.0137095932141014, -0.0345392174515273, -0.00407048581280864, 0.036426489236243, -0.0178640154001927, 0, 0, 0, 0, 0, 0, 0, 0, 0, -0.0370016365967795, -0.0427258096992148, 0, 0, 0.0300866089851429, 0.0243187165152054, 0, -0.014070114555226, 0, 0.00340391123780844, -0.00982624534723146, 0.00607967912880137},
    {-0.0214788046573522, -0.00377344134083677, -0.0196073763651313, 0.00924300585937403, 0, 0.0109364720199429, 0.00947126259704829, -0.102690308134494, -0.0695992817480271, -0.0601505961096861, 0, -0.00504163955965865, -0.00650872868404618, -0.00450937950937947, 0.0465726713108155, -0.0214195526695527, -0.0289638426440052, 0.00782402252196705, -0.0263060558477139, 0.000563672438672498, 0, 0, 0, 0, 0, 0, 0.0334424534952005, 0.0431740218144092, 0.0299118397398575, 0.0289620142917233, 0.00249265331165484, -0.00431740218144093, 0.0605484764655283, 0.0185070066280911, 0.0461140862656136, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0.0349574806629939, 0.0451299134777144, 0.0312669212338355, -0.0121096265225824, 0.028140229110452, 0.0270779480866286, -0.023827378664659, 0, -0.018239037386404},
    {-0.0214788046573521, 0.0150937653633479, 0.0130715842434208, 0.00924300585937423, 0, 0.010936472019943, 0.00947126259704825, 0.00892959201169522, 0.123732056440937, -0.060150596109686, 0, 0, 0.00650872868404613, 0.00676406926406924, 0, -0.0574945887445887, 0.0126920209338899, 0.0266937238984759, 0.00637722566005184, 0.000563672438672452, 0, 0, 0, 0, 0, 0, 0, -0.0431740218144092, -0.0448677596097863, 0, -0.0199412264932383, 0.0215870109072046, 0.0142467003448301, -0.0616900220936373, 0.0461140862656136, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, -0.0451299134777144, -0.0469003818507533, 0, 0.0375203054806026, 0.0162467688519772, -0.0119136893323295, 0.0206351152291861, -0.0182390373864041},
    {0.0787556170769579, -0.0301875307266956, 0.017428778991228, 0.0585390371093696, 0, -0.0218729440398858, -0.0189425251940965, -0.0178591840233903, 0.0103110047034114, 0.083846285486229, 0, 0, 0, 0, 0, -0.00526094276094277, -0.00455611007883227, -0.0533874477969518, 0.0308232573569172, -0.0319414381914383, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, -0.0348971463631671, -0.0302218152700865, -0.0284934006896603, 0.0164506725583033, -0.0643935438844154, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, -0.0437736897273697, -0.0379091273212801, 0.023827378664659, -0.0137567434861241, 0.0303983956440068},
    {0.078755617076958, -0.0377344134083697, -0.0566435317214906, -0.00308100195312479, 0.0423567739995023, 0.0546823600997145, 0.0378850503881931, 0.0446479600584756, 0.00515550235170578, -0.00729098134662856, 0.0417574386216002, -0.00504163955965862, -0.0247331689993752, -0.0202922077922078, 0.0295930515620807, 0.0101461038961039, 0.00216957622801541, 0.0266937238984759, 0.0132858867917747, 0.0142797017797018, -0.0557374224920008, -0.0245778931055953, -0.00519302773261415, 0.00402250198496156, 0.00464478520766674, -0.00219831358911332, -0.0130053985814669, -0.0158304746652834, -0.0108014976838375, -0.0168945083368386, -0.026796023100289, -0.0194283098164841, -0.0295110221428625, -0.0217383887377579, -0.0178640154001927, 0, 0, 0, 0, 0, 0, -0.0512709716390577, -0.0226083734055987, -0.00477689074405822, 0.00370016365967794, 0.00427258096992147, -0.0367664434877216, -0.00776832903622086, 0.00601732179702859, 0.00694820471863011, -0.00908221989193683, 0.00703505727761299, 0.00812338442598859, 0.00680782247561683, 0.00786099627778515, 0.00607967912880135},
    {-0.0214788046573522, 0, -0.00435719474780689, -0.0215670136718729, -0.127070321998507, 0, 0.0284137877911447, 0, 0.0360885164619399, 0.0309866707231715, 0, 0.0453747560369275, 0, -0.0202922077922078, -0.0422064833754266, 0, -0.00618329224984384, 0, 0.0015943064150129, 0.00394570707070703, 0.0836061337380012, 0, -0.0103860554652283, 0, 0.00464478520766673, 0, 0.024152883079867, 0, -0.0108014976838374, 0.043443021437585, 0, -0.0194283098164841, 0, -0.0217383887377578, -0.0178640154001926, 0, 0, 0, 0, 0, 0, 0.0769064574585866, 0, -0.00955378148811639, 0, 0.00427258096992146, 0, -0.0155366580724417, 0, 0.00694820471863015, -0.0181644397838736, 0, 0.00812338442598858, 0, 0.00786099627778515, 0.00607967912880134},
    {0.0787556170769581, 0.0377344134083697, -0.0566435317214905, -0.00308100195312477, 0.0423567739995024, -0.0546823600997146, 0.0378850503881931, -0.0446479600584756, 0.00515550235170579, -0.00729098134662863, -0.0417574386216002, -0.00504163955965864, 0.0247331689993752, -0.0202922077922078, 0.0295930515620807, -0.0101461038961039, 0.00216957622801542, -0.0266937238984759, 0.0132858867917747, 0.0142797017797018, -0.0557374224920008, 0.0245778931055954, -0.00519302773261416, -0.00402250198496156, 0.00464478520766676, 0.00219831358911337, -0.0130053985814669, 0.0158304746652834, -0.0108014976838375, -0.0168945083368386, 0.026796023100289, -0.0194283098164841, 0.0295110221428625, -0.0217383887377579, -0.0178640154001927, 0, 0, 0, 0, 0, 0, -0.0512709716390577, 0.0226083734055987, -0.00477689074405822, -0.00370016365967795, 0.00427258096992149, 0.0367664434877216, -0.00776832903622086, -0.00601732179702859, 0.00694820471863009, -0.00908221989193678, -0.00703505727761301, 0.00812338442598859, -0.00680782247561686, 0.00786099627778517, 0.00607967912880135},
    {-0.021478804657352, -0.0188672067041847, 0.00653579212171052, 0.00924300585937401, 0.0141189246665008, 0, -0.00315708753234946, -0.111619900146189, -0.05413277469291, -0.0601505961096861, -0.00894802256177153, -0.00252081977982931, 0.000650872868404653, 0.00112734487734485, -0.0276525235907967, 0.0360750360750361, 0.0374251899332651, -0.0188697013765088, 0.019928830187662, 0.000563672438672417, 0, 0, 0, 0, 0, 0.0593544669060614, 0.0167212267476003, -0.00431740218144091, -0.0074779599349644, 0.00321800158796925, 0.0224338798048931, 0.018708742786244, 0.0463017761206981, 0.0431830154655461, 0.0461140862656137, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0.0620433733855302, 0.017478740331497, -0.00451299134777146, -0.0078167303084589, 0.0363288795677472, -0.00938007637015066, -0.0162467688519772, -0.0119136893323295, -0.020635115229186, -0.018239037386404},
    {-0.021478804657352, 0.0188672067041848, 0.00653579212171056, 0.00924300585937413, 0.0141189246665008, 0, -0.00315708753234944, 0.111619900146189, -0.05413277469291, -0.060150596109686, 0.00894802256177151, -0.00252081977982929, -0.000650872868404627, 0.00112734487734488, -0.0276525235907967, -0.0360750360750361, 0.0374251899332651, 0.0188697013765088, 0.0199288301876619, 0.000563672438672459, 0, 0, 0, 0, 0, -0.0593544669060614, 0.0167212267476003, 0.00431740218144093, -0.0074779599349644, 0.00321800158796927, -0.0224338798048931, 0.018708742786244, -0.046301776120698, 0.0431830154655461, 0.0461140862656136, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, -0.0620433733855302, 0.017478740331497, 0.00451299134777145, -0.00781673030845891, 0.0363288795677472, 0.00938007637015065, -0.0162467688519772, 0.0119136893323295, -0.020635115229186, -0.0182390373864041},
    {0.0787556170769582, 0, -0.0348575579824558, 0.0585390371093696, -0.0282378493330016, 0, 0.00631417506469889, 0, -0.0206220094068229, 0.083846285486229, 0, 0, 0, 0, -0.00679184789949391, 0, 0.00151870335961076, 0, -0.0616465147138344, -0.0319414381914382, 0, 0, 0, 0, 0, 0, 0, 0, 0, -0.0450520222315696, 0, 0.0100739384233621, 0, -0.0329013451166065, -0.0643935438844154, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, -0.0565115904387179, 0, 0.01263637577376, 0, 0.027513486972248, 0.0303983956440067},
    {0.0787556170769578, -0.0377344134083695, -0.0217859737390347, -0.0523770332031201, 0.0423567739995023, 0.0546823600997146, 0, 0.0446479600584757, 0.0257775117585286, 0.0291639253865144, 0.0417574386216002, 0.0285692908380655, 0.0273366604729936, 0.0187890812890813, -0.00921750786359888, -0.00789141414141413, -0.00347132196482459, -0.0248527774227189, -0.0143487577351166, -0.015031265031265, -0.0557374224920008, -0.00491557862111907, -0.0197335053839337, -0.0394205194526234, -0.0325134964536672, -0.0241814494802472, -0.00928957041533347, -0.0158304746652834, -0.0141250354327105, -0.00241350119097692, -0.000623163327913714, -0.00215870109072046, 0.00712335017241511, 0.00411266813957583, 0.00540074884191872, 0, -0.0510586685671264, -0.0405265617908061, -0.0114170653756727, 0.010317557614593, 0.0127646671417816, 0.00466099742173251, -0.0143871467126538, -0.00564541633388696, 0.00571843474677503, 0.00737991258440982, 0.00367664434877214, -0.00155366580724416, 0.00240692871881144, 0.00347410235931507, 0.00100913554354852, 0.000260557676948644, 0.000902598269554298, -0.000850977809452118, -0.000491312267361587, -0.000868525589828769},
    {-0.0214788046573518, 0, -0.0217859737390347, 0.00308100195312481, -0.127070321998507, 0, 0.0473563129852414, 0, 0.0257775117585285, 0.0127592173566, 0, -0.0352914769176103, 0, 0.00225468975468975, 0.0509388592462044, 0, -0.00943765659186686, 0, -0.0143487577351166, -0.0129644660894661, 0.0836061337380013, 0, 0.0477758551400502, 0, -0.0325134964536671, 0, 0.00928957041533347, 0, -0.0141250354327104, -0.0144810071458617, 0, -0.00215870109072047, 0, 0.00411266813957582, 0.00540074884191876, 0, 0.0765880028506897, 0, -0.0228341307513453, 0, 0.0127646671417816, -0.00699149613259876, 0, -0.0112908326677739, 0, 0.00737991258440979, 0, -0.00310733161448836, 0, 0.00347410235931506, 0.00201827108709705, 0, 0.000902598269554284, 0, -0.000491312267361592, -0.000868525589828792},
    {0.0787556170769577, 0.0377344134083696, -0.0217859737390349, -0.05237703320312, 0.0423567739995023, -0.0546823600997146, 0, -0.0446479600584756, 0.0257775117585286, 0.0291639253865144, -0.0417574386216002, 0.0285692908380655, -0.0273366604729936, 0.0187890812890813, -0.0092175078635989, 0.00789141414141413, -0.00347132196482458, 0.0248527774227189, -0.0143487577351166, -0.015031265031265, -0.0557374224920009, 0.00491557862111903, -0.0197335053839338, 0.0394205194526234, -0.0325134964536671, 0.0241814494802472, -0.00928957041533348, 0.0158304746652834, -0.0141250354327105, -0.00241350119097695, 0.000623163327913703, -0.00215870109072045, -0.00712335017241511, 0.00411266813957582, 0.00540074884191871, 0, -0.0510586685671264, 0.0405265617908061, -0.0114170653756726, -0.010317557614593, 0.0127646671417816, 0.0046609974217325, 0.0143871467126537, -0.00564541633388698, -0.00571843474677501, 0.00737991258440984, -0.00367664434877218, -0.00155366580724417, -0.00240692871881142, 0.00347410235931505, 0.00100913554354854, -0.000260557676948633, 0.000902598269554291, 0.000850977809452119, -0.000491312267361577, -0.000868525589828757},
    {-0.0214788046573522, -0.0188672067041847, 0.0108929868695173, 0.00308100195312474, 0.0141189246665008, -0.109364720199429, -0.0789271883087357, 0.0223239800292378, -0.0128887558792642, 0.0127592173566001, -0.00894802256177151, -0.0277290175781224, -0.00585785581564148, 0.0191648629148629, 0.00145539597846299, 0.0383297258297258, 0.0348216984596467, -0.0124263887113595, 0.00717437886755833, -0.0129644660894661, 0, 0.0589869434534289, 0.00727023882565978, 0.0506835250105158, 0.0650269929073344, -0.00659494076734013, 0.0148633126645336, -0.00431740218144089, -0.00166176887443655, -0.0040225019849616, -0.00810112326287806, -0.0115130724838424, 0.00356167508620753, -0.00205633406978792, 0.00540074884191869, 0, 0, 0.0607898426862092, 0.0399597288148543, -0.0154763364218895, -0.031911667854454, 0, -0.0123318400394175, 0.00303983956440066, -0.00302740663064559, -0.00776832903622087, 0.00137874163078958, -0.0042725809699215, 0.000902598269554286, 0.000868525589828745, 0.00100913554354854, 0.000781673030845884, 0.00180519653910853, -0.000425488904726043, 0.0002456561336808, -0.000868525589828759},
    {-0.0214788046573523, 0.0188672067041848, 0.0108929868695173, 0.00308100195312473, 0.0141189246665008, 0.109364720199429, -0.0789271883087356, -0.0223239800292378, -0.0128887558792643, 0.0127592173566, 0.00894802256177147, -0.0277290175781224, 0.00585785581564147, 0.0191648629148629, 0.00145539597846298, -0.0383297258297258, 0.0348216984596467, 0.0124263887113595, 0.00717437886755829, -0.0129644660894661, 0, -0.0589869434534289, 0.0072702388256598, -0.0506835250105158, 0.0650269929073343, 0.00659494076734013, 0.0148633126645336, 0.0043174021814409, -0.00166176887443651, -0.00402250198496155, 0.00810112326287803, -0.0115130724838425, -0.00356167508620755, -0.0020563340697879, 0.00540074884191873, 0, 0, -0.0607898426862092, 0.0399597288148542, 0.0154763364218895, -0.031911667854454, 0, 0.0123318400394175, 0.00303983956440069, 0.0030274066306456, -0.00776832903622087, -0.00137874163078958, -0.00427258096992146, -0.000902598269554311, 0.000868525589828782, 0.00100913554354855, -0.000781673030845884, 0.00180519653910857, 0.000425488904726043, 0.000245656133680809, -0.000868525589828767},
    {0.0787556170769581, 0, 0.0435719474780695, -0.0523770332031201, -0.0282378493330016, 0, 0.0631417506469885, 0, -0.0515550235170572, 0.0291639253865144, 0, -0.00672218607954481, 0, -0.0601250601250601, 0.000970263985641982, 0, -0.0125835421224891, 0, 0.0286975154702333, -0.015031265031265, 0, 0, -0.0436214329539588, 0, -0.0650269929073343, 0, 0.0111474844984002, 0, 0.031573608614294, -0.00160900079398462, 0, -0.0028782681209606, 0, -0.00822533627915164, 0.00540074884191871, 0, 0, 0, -0.0532796384198057, 0, 0.0425488904726053, 0, 0, 0.018239037386404, 0, -0.00388416451811043, 0, -0.00466099742173252, 0, -0.00347410235931507, 0.00067275702903234, 0, 0.00120346435940573, 0, 0.000982624534723144, -0.000868525589828764},
    {0.10739402328676, -0.188672067041848, -0.108929868695174, -0.0770250488281177, -0.0705946233325038, -0.0546823600997147, -0.0315708753234941, -0.0446479600584757, -0.0257775117585286, -0.0182274533665716, 0.0894802256177148, -0.0151249186789758, 0.058578558156415, 0.0744047619047618, -0.0130985638061668, -0.0281836219336219, 0.00976309302606916, 0.069035492840886, 0.0398576603753241, 0.0597492784992785, 0, 0.0737336793167861, 0.036351194128299, -0.0120675059548847, -0.0232239260383337, 0.0659494076734016, 0.00743165633226678, 0.0431740218144092, 0.0465295284842228, 0.0345935170706695, 0.0155790831978425, 0.0251848460584054, -0.0254405363300539, -0.0146881004984851, -0.0328199352701214, 0, 0, 0, 0, 0, 0, 0, 0.0678251202167962, 0.0334382352084074, -0.0111004909790339, -0.0213629048496074, -0.0137874163078956, 0.0194208225905522, -0.0090259826955429, -0.0191075629762328, -0.00908221989193678, -0.00234501909253769, -0.00812338442598858, 0.00340391123780841, 0.00196524906944627, 0.00607967912880134},
    {0.107394023286761, 0.188672067041848, -0.108929868695174, -0.0770250488281178, -0.0705946233325039, 0.0546823600997145, -0.0315708753234942, 0.0446479600584757, -0.0257775117585287, -0.0182274533665715, -0.0894802256177147, -0.0151249186789758, -0.0585785581564149, 0.0744047619047619, -0.0130985638061668, 0.0281836219336219, 0.00976309302606916, -0.0690354928408859, 0.039857660375324, 0.0597492784992785, 0, -0.0737336793167861, 0.036351194128299, 0.0120675059548847, -0.0232239260383338, -0.0659494076734015, 0.00743165633226678, -0.0431740218144092, 0.0465295284842228, 0.0345935170706695, -0.0155790831978424, 0.0251848460584053, 0.0254405363300539, -0.0146881004984851, -0.0328199352701214, 0, 0, 0, 0, 0, 0, 0, -0.0678251202167962, 0.0334382352084074, 0.0111004909790339, -0.0213629048496074, 0.0137874163078956, 0.0194208225905522, 0.00902598269554287, -0.0191075629762328, -0.0090822198919368, 0.00234501909253767, -0.00812338442598857, -0.0034039112378084, 0.00196524906944627, 0.00607967912880135},
    {0.107394023286761, 0, 0.217859737390348, -0.0770250488281178, 0, 0, -0.094712625970483, 0, 0.0515550235170572, -0.0182274533665716, 0, -0.0907495120738551, 0, -0.0946969696969696, 0.0232863356554077, 0, -0.0227805503941613, 0, -0.0797153207506479, 0.0597492784992786, 0, 0, -0.0727023882565981, 0, 0.0464478520766674, 0, -0.0483057661597341, 0, -0.0781031370985168, 0.0144810071458616, 0, 0.0431740218144092, 0, 0.0293762009969701, -0.0328199352701215, 0, 0, 0, 0, 0, 0, 0, 0, -0.0668764704168148, 0, 0.0427258096992148, 0, 0.0310733161448835, 0, 0.00694820471863007, -0.00605481326129118, 0, -0.0108311792346515, 0, -0.00393049813889257, 0.00607967912880135},
    {0.107394023286761, 0, 0, 0.231075146484353, 0, 0, 0, 0, 0, -0.109364720199429, 0, 0.0100832791193172, 0, -0.00450937950937963, -0.093145342621631, 0, -0.104139658944738, 0, 0, -0.09244227994228, 0, 0, 0, 0, 0, 0, -0.066884906990401, 0, 0.0299118397398574, -0.0579240285834467, 0, -0.0647610327216138, 0, 0, 0.0835038859404354, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, -0.0699149613259879, 0, 0.0312669212338356, 0.0242192530451648, 0, 0.0270779480866286, 0, 0, -0.018239037386404}};
    
    // Extract relevant coefficients
    const double coeff0_0 = coefficients0[dof][0];
    const double coeff0_1 = coefficients0[dof][1];
    const double coeff0_2 = coefficients0[dof][2];
    const double coeff0_3 = coefficients0[dof][3];
    const double coeff0_4 = coefficients0[dof][4];
    const double coeff0_5 = coefficients0[dof][5];
    const double coeff0_6 = coefficients0[dof][6];
    const double coeff0_7 = coefficients0[dof][7];
    const double coeff0_8 = coefficients0[dof][8];
    const double coeff0_9 = coefficients0[dof][9];
    const double coeff0_10 = coefficients0[dof][10];
    const double coeff0_11 = coefficients0[dof][11];
    const double coeff0_12 = coefficients0[dof][12];
    const double coeff0_13 = coefficients0[dof][13];
    const double coeff0_14 = coefficients0[dof][14];
    const double coeff0_15 = coefficients0[dof][15];
    const double coeff0_16 = coefficients0[dof][16];
    const double coeff0_17 = coefficients0[dof][17];
    const double coeff0_18 = coefficients0[dof][18];
    const double coeff0_19 = coefficients0[dof][19];
    const double coeff0_20 = coefficients0[dof][20];
    const double coeff0_21 = coefficients0[dof][21];
    const double coeff0_22 = coefficients0[dof][22];
    const double coeff0_23 = coefficients0[dof][23];
    const double coeff0_24 = coefficients0[dof][24];
    const double coeff0_25 = coefficients0[dof][25];
    const double coeff0_26 = coefficients0[dof][26];
    const double coeff0_27 = coefficients0[dof][27];
    const double coeff0_28 = coefficients0[dof][28];
    const double coeff0_29 = coefficients0[dof][29];
    const double coeff0_30 = coefficients0[dof][30];
    const double coeff0_31 = coefficients0[dof][31];
    const double coeff0_32 = coefficients0[dof][32];
    const double coeff0_33 = coefficients0[dof][33];
    const double coeff0_34 = coefficients0[dof][34];
    const double coeff0_35 = coefficients0[dof][35];
    const double coeff0_36 = coefficients0[dof][36];
    const double coeff0_37 = coefficients0[dof][37];
    const double coeff0_38 = coefficients0[dof][38];
    const double coeff0_39 = coefficients0[dof][39];
    const double coeff0_40 = coefficients0[dof][40];
    const double coeff0_41 = coefficients0[dof][41];
    const double coeff0_42 = coefficients0[dof][42];
    const double coeff0_43 = coefficients0[dof][43];
    const double coeff0_44 = coefficients0[dof][44];
    const double coeff0_45 = coefficients0[dof][45];
    const double coeff0_46 = coefficients0[dof][46];
    const double coeff0_47 = coefficients0[dof][47];
    const double coeff0_48 = coefficients0[dof][48];
    const double coeff0_49 = coefficients0[dof][49];
    const double coeff0_50 = coefficients0[dof][50];
    const double coeff0_51 = coefficients0[dof][51];
    const double coeff0_52 = coefficients0[dof][52];
    const double coeff0_53 = coefficients0[dof][53];
    const double coeff0_54 = coefficients0[dof][54];
    const double coeff0_55 = coefficients0[dof][55];
    
    // Compute value(s)
    *values = coeff0_0*basisvalue0 + coeff0_1*basisvalue1 + coeff0_2*basisvalue2 + coeff0_3*basisvalue3 + coeff0_4*basisvalue4 + coeff0_5*basisvalue5 + coeff0_6*basisvalue6 + coeff0_7*basisvalue7 + coeff0_8*basisvalue8 + coeff0_9*basisvalue9 + coeff0_10*basisvalue10 + coeff0_11*basisvalue11 + coeff0_12*basisvalue12 + coeff0_13*basisvalue13 + coeff0_14*basisvalue14 + coeff0_15*basisvalue15 + coeff0_16*basisvalue16 + coeff0_17*basisvalue17 + coeff0_18*basisvalue18 + coeff0_19*basisvalue19 + coeff0_20*basisvalue20 + coeff0_21*basisvalue21 + coeff0_22*basisvalue22 + coeff0_23*basisvalue23 + coeff0_24*basisvalue24 + coeff0_25*basisvalue25 + coeff0_26*basisvalue26 + coeff0_27*basisvalue27 + coeff0_28*basisvalue28 + coeff0_29*basisvalue29 + coeff0_30*basisvalue30 + coeff0_31*basisvalue31 + coeff0_32*basisvalue32 + coeff0_33*basisvalue33 + coeff0_34*basisvalue34 + coeff0_35*basisvalue35 + coeff0_36*basisvalue36 + coeff0_37*basisvalue37 + coeff0_38*basisvalue38 + coeff0_39*basisvalue39 + coeff0_40*basisvalue40 + coeff0_41*basisvalue41 + coeff0_42*basisvalue42 + coeff0_43*basisvalue43 + coeff0_44*basisvalue44 + coeff0_45*basisvalue45 + coeff0_46*basisvalue46 + coeff0_47*basisvalue47 + coeff0_48*basisvalue48 + coeff0_49*basisvalue49 + coeff0_50*basisvalue50 + coeff0_51*basisvalue51 + coeff0_52*basisvalue52 + coeff0_53*basisvalue53 + coeff0_54*basisvalue54 + coeff0_55*basisvalue55;
  }

  /// Evaluate all basis functions at given point in cell
  virtual void evaluate_basis_all(double* values,
                                  const double* coordinates,
                                  const ufc::cell& c) const
  {
    throw std::runtime_error("The vectorised version of evaluate_basis() is not yet implemented.");
  }

  /// Evaluate order n derivatives of basis function i at given point in cell
  virtual void evaluate_basis_derivatives(unsigned int i,
                                          unsigned int n,
                                          double* values,
                                          const double* coordinates,
                                          const ufc::cell& c) const
  {
    // Extract vertex coordinates
    const double * const * element_coordinates = c.coordinates;
    
    // Compute Jacobian of affine map from reference cell
    const double J_00 = element_coordinates[1][0] - element_coordinates[0][0];
    const double J_01 = element_coordinates[2][0] - element_coordinates[0][0];
    const double J_02 = element_coordinates[3][0] - element_coordinates[0][0];
    const double J_10 = element_coordinates[1][1] - element_coordinates[0][1];
    const double J_11 = element_coordinates[2][1] - element_coordinates[0][1];
    const double J_12 = element_coordinates[3][1] - element_coordinates[0][1];
    const double J_20 = element_coordinates[1][2] - element_coordinates[0][2];
    const double J_21 = element_coordinates[2][2] - element_coordinates[0][2];
    const double J_22 = element_coordinates[3][2] - element_coordinates[0][2];
      
    // Compute sub determinants
    const double d00 = J_11*J_22 - J_12*J_21;
    const double d01 = J_12*J_20 - J_10*J_22;
    const double d02 = J_10*J_21 - J_11*J_20;
    
    const double d10 = J_02*J_21 - J_01*J_22;
    const double d11 = J_00*J_22 - J_02*J_20;
    const double d12 = J_01*J_20 - J_00*J_21;
    
    const double d20 = J_01*J_12 - J_02*J_11;
    const double d21 = J_02*J_10 - J_00*J_12;
    const double d22 = J_00*J_11 - J_01*J_10;
      
    // Compute determinant of Jacobian
    double detJ = J_00*d00 + J_10*d10 + J_20*d20;
    
    // Compute inverse of Jacobian
    
    // Compute constants
    const double C0 = d00*(element_coordinates[0][0] - element_coordinates[2][0] - element_coordinates[3][0]) \
                    + d10*(element_coordinates[0][1] - element_coordinates[2][1] - element_coordinates[3][1]) \
                    + d20*(element_coordinates[0][2] - element_coordinates[2][2] - element_coordinates[3][2]);
    
    const double C1 = d01*(element_coordinates[0][0] - element_coordinates[1][0] - element_coordinates[3][0]) \
                    + d11*(element_coordinates[0][1] - element_coordinates[1][1] - element_coordinates[3][1]) \
                    + d21*(element_coordinates[0][2] - element_coordinates[1][2] - element_coordinates[3][2]);
    
    const double C2 = d02*(element_coordinates[0][0] - element_coordinates[1][0] - element_coordinates[2][0]) \
                    + d12*(element_coordinates[0][1] - element_coordinates[1][1] - element_coordinates[2][1]) \
                    + d22*(element_coordinates[0][2] - element_coordinates[1][2] - element_coordinates[2][2]);
    
    // Get coordinates and map to the UFC reference element
    double x = (C0 + d00*coordinates[0] + d10*coordinates[1] + d20*coordinates[2]) / detJ;
    double y = (C1 + d01*coordinates[0] + d11*coordinates[1] + d21*coordinates[2]) / detJ;
    double z = (C2 + d02*coordinates[0] + d12*coordinates[1] + d22*coordinates[2]) / detJ;
    
    // Map coordinates to the reference cube
    if (std::abs(y + z - 1.0) < 1e-14)
      x = 1.0;
    else
      x = -2.0 * x/(y + z - 1.0) - 1.0;
    if (std::abs(z - 1.0) < 1e-14)
      y = -1.0;
    else
      y = 2.0 * y/(1.0 - z) - 1.0;
    z = 2.0 * z - 1.0;
    
    // Compute number of derivatives
    unsigned int num_derivatives = 1;
    
    for (unsigned int j = 0; j < n; j++)
      num_derivatives *= 3;
    
    
    // Declare pointer to two dimensional array that holds combinations of derivatives and initialise
    unsigned int **combinations = new unsigned int *[num_derivatives];
        
    for (unsigned int j = 0; j < num_derivatives; j++)
    {
      combinations[j] = new unsigned int [n];
      for (unsigned int k = 0; k < n; k++)
        combinations[j][k] = 0;
    }
        
    // Generate combinations of derivatives
    for (unsigned int row = 1; row < num_derivatives; row++)
    {
      for (unsigned int num = 0; num < row; num++)
      {
        for (unsigned int col = n-1; col+1 > 0; col--)
        {
          if (combinations[row][col] + 1 > 2)
            combinations[row][col] = 0;
          else
          {
            combinations[row][col] += 1;
            break;
          }
        }
      }
    }
    
    // Compute inverse of Jacobian
    const double Jinv[3][3] ={{d00 / detJ, d10 / detJ, d20 / detJ}, {d01 / detJ, d11 / detJ, d21 / detJ}, {d02 / detJ, d12 / detJ, d22 / detJ}};
    
    // Declare transformation matrix
    // Declare pointer to two dimensional array and initialise
    double **transform = new double *[num_derivatives];
        
    for (unsigned int j = 0; j < num_derivatives; j++)
    {
      transform[j] = new double [num_derivatives];
      for (unsigned int k = 0; k < num_derivatives; k++)
        transform[j][k] = 1;
    }
    
    // Construct transformation matrix
    for (unsigned int row = 0; row < num_derivatives; row++)
    {
      for (unsigned int col = 0; col < num_derivatives; col++)
      {
        for (unsigned int k = 0; k < n; k++)
          transform[row][col] *= Jinv[combinations[col][k]][combinations[row][k]];
      }
    }
    
    // Reset values
    for (unsigned int j = 0; j < 1*num_derivatives; j++)
      values[j] = 0;
    
    // Map degree of freedom to element degree of freedom
    const unsigned int dof = i;
    
    // Generate scalings
    const double scalings_y_0 = 1;
    const double scalings_y_1 = scalings_y_0*(0.5 - 0.5*y);
    const double scalings_y_2 = scalings_y_1*(0.5 - 0.5*y);
    const double scalings_y_3 = scalings_y_2*(0.5 - 0.5*y);
    const double scalings_y_4 = scalings_y_3*(0.5 - 0.5*y);
    const double scalings_y_5 = scalings_y_4*(0.5 - 0.5*y);
    const double scalings_z_0 = 1;
    const double scalings_z_1 = scalings_z_0*(0.5 - 0.5*z);
    const double scalings_z_2 = scalings_z_1*(0.5 - 0.5*z);
    const double scalings_z_3 = scalings_z_2*(0.5 - 0.5*z);
    const double scalings_z_4 = scalings_z_3*(0.5 - 0.5*z);
    const double scalings_z_5 = scalings_z_4*(0.5 - 0.5*z);
    
    // Compute psitilde_a
    const double psitilde_a_0 = 1;
    const double psitilde_a_1 = x;
    const double psitilde_a_2 = 1.5*x*psitilde_a_1 - 0.5*psitilde_a_0;
    const double psitilde_a_3 = 1.66666666666667*x*psitilde_a_2 - 0.666666666666667*psitilde_a_1;
    const double psitilde_a_4 = 1.75*x*psitilde_a_3 - 0.75*psitilde_a_2;
    const double psitilde_a_5 = 1.8*x*psitilde_a_4 - 0.8*psitilde_a_3;
    
    // Compute psitilde_bs
    const double psitilde_bs_0_0 = 1;
    const double psitilde_bs_0_1 = 1.5*y + 0.5;
    const double psitilde_bs_0_2 = 0.111111111111111*psitilde_bs_0_1 + 1.66666666666667*y*psitilde_bs_0_1 - 0.555555555555556*psitilde_bs_0_0;
    const double psitilde_bs_0_3 = 0.05*psitilde_bs_0_2 + 1.75*y*psitilde_bs_0_2 - 0.7*psitilde_bs_0_1;
    const double psitilde_bs_0_4 = 0.0285714285714286*psitilde_bs_0_3 + 1.8*y*psitilde_bs_0_3 - 0.771428571428571*psitilde_bs_0_2;
    const double psitilde_bs_0_5 = 0.0185185185185185*psitilde_bs_0_4 + 1.83333333333333*y*psitilde_bs_0_4 - 0.814814814814815*psitilde_bs_0_3;
    const double psitilde_bs_1_0 = 1;
    const double psitilde_bs_1_1 = 2.5*y + 1.5;
    const double psitilde_bs_1_2 = 0.54*psitilde_bs_1_1 + 2.1*y*psitilde_bs_1_1 - 0.56*psitilde_bs_1_0;
    const double psitilde_bs_1_3 = 0.285714285714286*psitilde_bs_1_2 + 2*y*psitilde_bs_1_2 - 0.714285714285714*psitilde_bs_1_1;
    const double psitilde_bs_1_4 = 0.178571428571429*psitilde_bs_1_3 + 1.96428571428571*y*psitilde_bs_1_3 - 0.785714285714286*psitilde_bs_1_2;
    const double psitilde_bs_2_0 = 1;
    const double psitilde_bs_2_1 = 3.5*y + 2.5;
    const double psitilde_bs_2_2 = 1.02040816326531*psitilde_bs_2_1 + 2.57142857142857*y*psitilde_bs_2_1 - 0.551020408163265*psitilde_bs_2_0;
    const double psitilde_bs_2_3 = 0.578703703703704*psitilde_bs_2_2 + 2.29166666666667*y*psitilde_bs_2_2 - 0.712962962962963*psitilde_bs_2_1;
    const double psitilde_bs_3_0 = 1;
    const double psitilde_bs_3_1 = 4.5*y + 3.5;
    const double psitilde_bs_3_2 = 1.51234567901235*psitilde_bs_3_1 + 3.05555555555556*y*psitilde_bs_3_1 - 0.54320987654321*psitilde_bs_3_0;
    const double psitilde_bs_4_0 = 1;
    const double psitilde_bs_4_1 = 5.5*y + 4.5;
    const double psitilde_bs_5_0 = 1;
    
    // Compute psitilde_cs
    const double psitilde_cs_00_0 = 1;
    const double psitilde_cs_00_1 = 2*z + 1;
    const double psitilde_cs_00_2 = 0.3125*psitilde_cs_00_1 + 1.875*z*psitilde_cs_00_1 - 0.5625*psitilde_cs_00_0;
    const double psitilde_cs_00_3 = 0.155555555555556*psitilde_cs_00_2 + 1.86666666666667*z*psitilde_cs_00_2 - 0.711111111111111*psitilde_cs_00_1;
    const double psitilde_cs_00_4 = 0.09375*psitilde_cs_00_3 + 1.875*z*psitilde_cs_00_3 - 0.78125*psitilde_cs_00_2;
    const double psitilde_cs_00_5 = 0.0628571428571429*psitilde_cs_00_4 + 1.88571428571429*z*psitilde_cs_00_4 - 0.822857142857143*psitilde_cs_00_3;
    const double psitilde_cs_01_0 = 1;
    const double psitilde_cs_01_1 = 3*z + 2;
    const double psitilde_cs_01_2 = 0.777777777777778*psitilde_cs_01_1 + 2.33333333333333*z*psitilde_cs_01_1 - 0.555555555555556*psitilde_cs_01_0;
    const double psitilde_cs_01_3 = 0.428571428571429*psitilde_cs_01_2 + 2.14285714285714*z*psitilde_cs_01_2 - 0.714285714285714*psitilde_cs_01_1;
    const double psitilde_cs_01_4 = 0.275*psitilde_cs_01_3 + 2.0625*z*psitilde_cs_01_3 - 0.7875*psitilde_cs_01_2;
    const double psitilde_cs_02_0 = 1;
    const double psitilde_cs_02_1 = 4*z + 3;
    const double psitilde_cs_02_2 = 1.265625*psitilde_cs_02_1 + 2.8125*z*psitilde_cs_02_1 - 0.546875*psitilde_cs_02_0;
    const double psitilde_cs_02_3 = 0.733333333333333*psitilde_cs_02_2 + 2.44444444444444*z*psitilde_cs_02_2 - 0.711111111111111*psitilde_cs_02_1;
    const double psitilde_cs_03_0 = 1;
    const double psitilde_cs_03_1 = 5*z + 4;
    const double psitilde_cs_03_2 = 1.76*psitilde_cs_03_1 + 3.3*z*psitilde_cs_03_1 - 0.54*psitilde_cs_03_0;
    const double psitilde_cs_04_0 = 1;
    const double psitilde_cs_04_1 = 6*z + 5;
    const double psitilde_cs_05_0 = 1;
    const double psitilde_cs_10_0 = 1;
    const double psitilde_cs_10_1 = 3*z + 2;
    const double psitilde_cs_10_2 = 0.777777777777778*psitilde_cs_10_1 + 2.33333333333333*z*psitilde_cs_10_1 - 0.555555555555556*psitilde_cs_10_0;
    const double psitilde_cs_10_3 = 0.428571428571429*psitilde_cs_10_2 + 2.14285714285714*z*psitilde_cs_10_2 - 0.714285714285714*psitilde_cs_10_1;
    const double psitilde_cs_10_4 = 0.275*psitilde_cs_10_3 + 2.0625*z*psitilde_cs_10_3 - 0.7875*psitilde_cs_10_2;
    const double psitilde_cs_11_0 = 1;
    const double psitilde_cs_11_1 = 4*z + 3;
    const double psitilde_cs_11_2 = 1.265625*psitilde_cs_11_1 + 2.8125*z*psitilde_cs_11_1 - 0.546875*psitilde_cs_11_0;
    const double psitilde_cs_11_3 = 0.733333333333333*psitilde_cs_11_2 + 2.44444444444444*z*psitilde_cs_11_2 - 0.711111111111111*psitilde_cs_11_1;
    const double psitilde_cs_12_0 = 1;
    const double psitilde_cs_12_1 = 5*z + 4;
    const double psitilde_cs_12_2 = 1.76*psitilde_cs_12_1 + 3.3*z*psitilde_cs_12_1 - 0.54*psitilde_cs_12_0;
    const double psitilde_cs_13_0 = 1;
    const double psitilde_cs_13_1 = 6*z + 5;
    const double psitilde_cs_14_0 = 1;
    const double psitilde_cs_20_0 = 1;
    const double psitilde_cs_20_1 = 4*z + 3;
    const double psitilde_cs_20_2 = 1.265625*psitilde_cs_20_1 + 2.8125*z*psitilde_cs_20_1 - 0.546875*psitilde_cs_20_0;
    const double psitilde_cs_20_3 = 0.733333333333333*psitilde_cs_20_2 + 2.44444444444444*z*psitilde_cs_20_2 - 0.711111111111111*psitilde_cs_20_1;
    const double psitilde_cs_21_0 = 1;
    const double psitilde_cs_21_1 = 5*z + 4;
    const double psitilde_cs_21_2 = 1.76*psitilde_cs_21_1 + 3.3*z*psitilde_cs_21_1 - 0.54*psitilde_cs_21_0;
    const double psitilde_cs_22_0 = 1;
    const double psitilde_cs_22_1 = 6*z + 5;
    const double psitilde_cs_23_0 = 1;
    const double psitilde_cs_30_0 = 1;
    const double psitilde_cs_30_1 = 5*z + 4;
    const double psitilde_cs_30_2 = 1.76*psitilde_cs_30_1 + 3.3*z*psitilde_cs_30_1 - 0.54*psitilde_cs_30_0;
    const double psitilde_cs_31_0 = 1;
    const double psitilde_cs_31_1 = 6*z + 5;
    const double psitilde_cs_32_0 = 1;
    const double psitilde_cs_40_0 = 1;
    const double psitilde_cs_40_1 = 6*z + 5;
    const double psitilde_cs_41_0 = 1;
    const double psitilde_cs_50_0 = 1;
    
    // Compute basisvalues
    const double basisvalue0 = 0.866025403784439*psitilde_a_0*scalings_y_0*psitilde_bs_0_0*scalings_z_0*psitilde_cs_00_0;
    const double basisvalue1 = 2.73861278752583*psitilde_a_1*scalings_y_1*psitilde_bs_1_0*scalings_z_1*psitilde_cs_10_0;
    const double basisvalue2 = 1.58113883008419*psitilde_a_0*scalings_y_0*psitilde_bs_0_1*scalings_z_1*psitilde_cs_01_0;
    const double basisvalue3 = 1.11803398874989*psitilde_a_0*scalings_y_0*psitilde_bs_0_0*scalings_z_0*psitilde_cs_00_1;
    const double basisvalue4 = 5.1234753829798*psitilde_a_2*scalings_y_2*psitilde_bs_2_0*scalings_z_2*psitilde_cs_20_0;
    const double basisvalue5 = 3.96862696659689*psitilde_a_1*scalings_y_1*psitilde_bs_1_1*scalings_z_2*psitilde_cs_11_0;
    const double basisvalue6 = 2.29128784747792*psitilde_a_0*scalings_y_0*psitilde_bs_0_2*scalings_z_2*psitilde_cs_02_0;
    const double basisvalue7 = 3.24037034920393*psitilde_a_1*scalings_y_1*psitilde_bs_1_0*scalings_z_1*psitilde_cs_10_1;
    const double basisvalue8 = 1.87082869338697*psitilde_a_0*scalings_y_0*psitilde_bs_0_1*scalings_z_1*psitilde_cs_01_1;
    const double basisvalue9 = 1.3228756555323*psitilde_a_0*scalings_y_0*psitilde_bs_0_0*scalings_z_0*psitilde_cs_00_2;
    const double basisvalue10 = 7.93725393319377*psitilde_a_3*scalings_y_3*psitilde_bs_3_0*scalings_z_3*psitilde_cs_30_0;
    const double basisvalue11 = 6.70820393249937*psitilde_a_2*scalings_y_2*psitilde_bs_2_1*scalings_z_3*psitilde_cs_21_0;
    const double basisvalue12 = 5.19615242270663*psitilde_a_1*scalings_y_1*psitilde_bs_1_2*scalings_z_3*psitilde_cs_12_0;
    const double basisvalue13 = 3*psitilde_a_0*scalings_y_0*psitilde_bs_0_3*scalings_z_3*psitilde_cs_03_0;
    const double basisvalue14 = 5.80947501931113*psitilde_a_2*scalings_y_2*psitilde_bs_2_0*scalings_z_2*psitilde_cs_20_1;
    const double basisvalue15 = 4.5*psitilde_a_1*scalings_y_1*psitilde_bs_1_1*scalings_z_2*psitilde_cs_11_1;
    const double basisvalue16 = 2.59807621135332*psitilde_a_0*scalings_y_0*psitilde_bs_0_2*scalings_z_2*psitilde_cs_02_1;
    const double basisvalue17 = 3.67423461417477*psitilde_a_1*scalings_y_1*psitilde_bs_1_0*scalings_z_1*psitilde_cs_10_2;
    const double basisvalue18 = 2.12132034355964*psitilde_a_0*scalings_y_0*psitilde_bs_0_1*scalings_z_1*psitilde_cs_01_2;
    const double basisvalue19 = 1.5*psitilde_a_0*scalings_y_0*psitilde_bs_0_0*scalings_z_0*psitilde_cs_00_3;
    const double basisvalue20 = 11.1242977306435*psitilde_a_4*scalings_y_4*psitilde_bs_4_0*scalings_z_4*psitilde_cs_40_0;
    const double basisvalue21 = 9.81070843517429*psitilde_a_3*scalings_y_3*psitilde_bs_3_1*scalings_z_4*psitilde_cs_31_0;
    const double basisvalue22 = 8.2915619758885*psitilde_a_2*scalings_y_2*psitilde_bs_2_2*scalings_z_4*psitilde_cs_22_0;
    const double basisvalue23 = 6.42261628933256*psitilde_a_1*scalings_y_1*psitilde_bs_1_3*scalings_z_4*psitilde_cs_13_0;
    const double basisvalue24 = 3.70809924354783*psitilde_a_0*scalings_y_0*psitilde_bs_0_4*scalings_z_4*psitilde_cs_04_0;
    const double basisvalue25 = 8.77496438739212*psitilde_a_3*scalings_y_3*psitilde_bs_3_0*scalings_z_3*psitilde_cs_30_1;
    const double basisvalue26 = 7.41619848709566*psitilde_a_2*scalings_y_2*psitilde_bs_2_1*scalings_z_3*psitilde_cs_21_1;
    const double basisvalue27 = 5.74456264653803*psitilde_a_1*scalings_y_1*psitilde_bs_1_2*scalings_z_3*psitilde_cs_12_1;
    const double basisvalue28 = 3.3166247903554*psitilde_a_0*scalings_y_0*psitilde_bs_0_3*scalings_z_3*psitilde_cs_03_1;
    const double basisvalue29 = 6.42261628933256*psitilde_a_2*scalings_y_2*psitilde_bs_2_0*scalings_z_2*psitilde_cs_20_2;
    const double basisvalue30 = 4.9749371855331*psitilde_a_1*scalings_y_1*psitilde_bs_1_1*scalings_z_2*psitilde_cs_11_2;
    const double basisvalue31 = 2.87228132326901*psitilde_a_0*scalings_y_0*psitilde_bs_0_2*scalings_z_2*psitilde_cs_02_2;
    const double basisvalue32 = 4.06201920231798*psitilde_a_1*scalings_y_1*psitilde_bs_1_0*scalings_z_1*psitilde_cs_10_3;
    const double basisvalue33 = 2.34520787991171*psitilde_a_0*scalings_y_0*psitilde_bs_0_1*scalings_z_1*psitilde_cs_01_3;
    const double basisvalue34 = 1.6583123951777*psitilde_a_0*scalings_y_0*psitilde_bs_0_0*scalings_z_0*psitilde_cs_00_4;
    const double basisvalue35 = 14.6458185158768*psitilde_a_5*scalings_y_5*psitilde_bs_5_0*scalings_z_5*psitilde_cs_50_0;
    const double basisvalue36 = 13.2476412994918*psitilde_a_4*scalings_y_4*psitilde_bs_4_1*scalings_z_5*psitilde_cs_41_0;
    const double basisvalue37 = 11.6833214455479*psitilde_a_3*scalings_y_3*psitilde_bs_3_2*scalings_z_5*psitilde_cs_32_0;
    const double basisvalue38 = 9.87420882906575*psitilde_a_2*scalings_y_2*psitilde_bs_2_3*scalings_z_5*psitilde_cs_23_0;
    const double basisvalue39 = 7.64852927038918*psitilde_a_1*scalings_y_1*psitilde_bs_1_4*scalings_z_5*psitilde_cs_14_0;
    const double basisvalue40 = 4.41588043316392*psitilde_a_0*scalings_y_0*psitilde_bs_0_5*scalings_z_5*psitilde_cs_05_0;
    const double basisvalue41 = 12.0933866224478*psitilde_a_4*scalings_y_4*psitilde_bs_4_0*scalings_z_4*psitilde_cs_40_1;
    const double basisvalue42 = 10.6653645038508*psitilde_a_3*scalings_y_3*psitilde_bs_3_1*scalings_z_4*psitilde_cs_31_1;
    const double basisvalue43 = 9.01387818865997*psitilde_a_2*scalings_y_2*psitilde_bs_2_2*scalings_z_4*psitilde_cs_22_1;
    const double basisvalue44 = 6.98212002188447*psitilde_a_1*scalings_y_1*psitilde_bs_1_3*scalings_z_4*psitilde_cs_13_1;
    const double basisvalue45 = 4.03112887414927*psitilde_a_0*scalings_y_0*psitilde_bs_0_4*scalings_z_4*psitilde_cs_04_1;
    const double basisvalue46 = 9.53939201416946*psitilde_a_3*scalings_y_3*psitilde_bs_3_0*scalings_z_3*psitilde_cs_30_2;
    const double basisvalue47 = 8.06225774829855*psitilde_a_2*scalings_y_2*psitilde_bs_2_1*scalings_z_3*psitilde_cs_21_2;
    const double basisvalue48 = 6.2449979983984*psitilde_a_1*scalings_y_1*psitilde_bs_1_2*scalings_z_3*psitilde_cs_12_2;
    const double basisvalue49 = 3.60555127546399*psitilde_a_0*scalings_y_0*psitilde_bs_0_3*scalings_z_3*psitilde_cs_03_2;
    const double basisvalue50 = 6.98212002188447*psitilde_a_2*scalings_y_2*psitilde_bs_2_0*scalings_z_2*psitilde_cs_20_3;
    const double basisvalue51 = 5.40832691319598*psitilde_a_1*scalings_y_1*psitilde_bs_1_1*scalings_z_2*psitilde_cs_11_3;
    const double basisvalue52 = 3.1224989991992*psitilde_a_0*scalings_y_0*psitilde_bs_0_2*scalings_z_2*psitilde_cs_02_3;
    const double basisvalue53 = 4.41588043316392*psitilde_a_1*scalings_y_1*psitilde_bs_1_0*scalings_z_1*psitilde_cs_10_4;
    const double basisvalue54 = 2.54950975679639*psitilde_a_0*scalings_y_0*psitilde_bs_0_1*scalings_z_1*psitilde_cs_01_4;
    const double basisvalue55 = 1.80277563773199*psitilde_a_0*scalings_y_0*psitilde_bs_0_0*scalings_z_0*psitilde_cs_00_5;
    
    // Table(s) of coefficients
    const static double coefficients0[56][56] = \
    {{0.00945067404923504, 0.00609788120679252, 0.003520613356228, 0.00248944957812473, 0.00705946233325038, 0.00546823600997147, 0.00315708753234945, 0.00446479600584757, 0.00257775117585287, 0.00182274533665719, -0.00298267418725718, -0.0025208197798293, -0.00195261860521383, -0.0011273448773449, -0.00218309396769446, -0.00169101731601731, -0.000976309302606924, -0.00138070985681772, -0.000797153207506492, -0.000563672438672437, 0.00278687112460006, 0.00245778931055954, 0.00207721109304565, 0.00160900079398463, 0.000928957041533348, 0.00219831358911339, 0.00185791408306671, 0.00143913406048031, 0.000830884437218262, 0.00160900079398463, 0.00124632665582739, 0.000719567030240153, 0.00101762145320216, 0.000587524019939396, 0.000415442218609115, -0.00705593524717797, -0.0063823335708908, -0.00562868913761196, -0.00475711057319694, -0.00368484200521179, -0.00212744452363026, -0.00582624677716564, -0.00513826668309063, -0.00434262794914383, -0.00336378514516178, -0.00194208225905521, -0.00459580543596519, -0.00388416451811043, -0.00300866089851429, -0.00173705117965752, -0.00336378514516179, -0.00260557676948628, -0.00150433044925714, -0.00212744452363026, -0.00122828066840392, -0.000868525589828755},
    {0.00945067404923484, -0.0060978812067926, 0.00352061335622807, 0.00248944957812473, 0.00705946233325039, -0.00546823600997146, 0.00315708753234942, -0.00446479600584756, 0.00257775117585286, 0.00182274533665715, 0.00298267418725716, -0.00252081977982931, 0.00195261860521383, -0.00112734487734489, -0.00218309396769446, 0.00169101731601731, -0.000976309302606908, 0.00138070985681772, -0.000797153207506487, -0.000563672438672423, 0.00278687112460003, -0.00245778931055954, 0.00207721109304565, -0.00160900079398463, 0.000928957041533354, -0.00219831358911339, 0.00185791408306669, -0.00143913406048031, 0.000830884437218255, 0.00160900079398463, -0.0012463266558274, 0.000719567030240159, -0.00101762145320216, 0.000587524019939409, 0.000415442218609113, 0.00705593524717798, -0.00638233357089081, 0.00562868913761196, -0.00475711057319694, 0.00368484200521178, -0.00212744452363027, -0.00582624677716566, 0.00513826668309062, -0.00434262794914383, 0.00336378514516178, -0.00194208225905522, 0.0045958054359652, -0.00388416451811043, 0.00300866089851429, -0.00173705117965753, -0.00336378514516178, 0.00260557676948629, -0.00150433044925715, 0.00212744452363026, -0.00122828066840393, -0.000868525589828756},
    {0.00945067404923496, 0, -0.00704122671245609, 0.00248944957812476, 0, 0, 0.00947126259704828, 0, -0.00515550235170569, 0.00182274533665715, 0, 0, 0, 0.0045093795093795, 0, 0, -0.00292892790782073, 0, 0.00159430641501296, -0.000563672438672431, 0, 0, 0, 0, 0.00464478520766673, 0, 0, 0, -0.00332353774887306, 0, 0, 0.00215870109072046, 0, -0.00117504803987882, 0.000415442218609132, 0, 0, 0, 0, 0, 0.0127646671417816, 0, 0, 0, 0, -0.00971041129527609, 0, 0, 0, 0.0069482047186301, 0, 0, -0.00451299134777144, 0, 0.00245656133680787, -0.000868525589828766},
    {0.00945067404923495, 0, 0, -0.00746834873437432, 0, 0, 0, 0, 0, 0.0109364720199429, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0.00563672438672439, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0.00623163327913698, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0.0182390373864041},
    {-0.0100234421734309, 0, 0.0374718748311398, -0.0289614183593723, 0, 0, 0.0284137877911449, 0, 0.00515550235170565, -0.00619733414463432, 0, 0, 0, 0.0315656565656566, 0, 0, -0.0113902751970807, 0, 0.00265717735835495, -0.000187890812890785, 0, 0, 0, 0, 0.0232239260383336, 0, 0, 0, -0.00166176887443649, 0, 0, -0.00647610327216139, 0, 0.00646276421933346, -0.00290809553026393, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0.0213629048496073, 0, 0, 0, -0.0277928188745204, 0, 0, 0.0243701532779658, 0, -0.0157219925555703, 0.00607967912880134},
    {0.0100234421734308, 0, -0.0287574853355261, 0.05360943398437, 0, 0, -0.0189425251940965, 0, 0.0154665070551172, 0.0153110608279201, 0, 0, 0, -0.00450937950937953, 0, 0, 0.0370997534990628, 0, -0.0281660799985623, 0.0108976671476671, 0, 0, 0, 0, 0, 0, 0, 0, 0.0299118397398575, 0, 0, -0.00431740218144094, 0, -0.0123380044187275, 0.00872428659079179, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0.0312669212338355, 0, 0, -0.0487403065559315, 0, 0.0412702304583721, -0.018239037386404},
    {0.0100234421734309, 0, 0.0409576306293854, -0.0449826285156208, 0, 0, 0.0189425251940966, 0, -0.00515550235170581, -0.021143845905223, 0, 0, 0, 0, 0, 0, 0.00455611007883228, 0, 0.0356061766019561, -0.0319414381914382, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0.0302218152700864, 0, 0.00411266813957589, -0.0145404776513196, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0.0379091273212801, 0, -0.0550269739444961, 0.0303983956440067},
    {-0.0100234421734309, 0, -0.0148144621425437, 0.0449826285156208, 0, 0, 0, 0, 0.0206220094068229, 0.021143845905223, 0, 0, 0, 0, 0, 0, 0, 0, 0.0106287094334197, 0.0319414381914382, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0.0205633406978791, 0.0145404776513196, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0.03439185871531, -0.0303983956440067},
    {-0.0100234421734309, 0.0324515955311978, -0.01873593741557, -0.0289614183593723, 0.0211783869997511, -0.0164047080299144, 0.00947126259704827, 0.00446479600584751, -0.00257775117585286, -0.00619733414463429, 0.0208787193108001, -0.0176457384588051, 0.0136683302364968, -0.00789141414141412, -0.00848980987436741, 0.00657617845117844, -0.00379675839902691, 0.0023011830946962, -0.00132858867917746, -0.000187890812890819, 0.0139343556230002, -0.0122889465527977, 0.0103860554652283, -0.00804500396992315, 0.00464478520766672, -0.00109915679455667, 0.000928957041533349, -0.000719567030240153, 0.000415442218609154, -0.00482700238195389, 0.00373897996748218, -0.00215870109072046, 0.00559691799261187, -0.00323138210966673, -0.00290809553026394, 0, 0, 0, 0, 0, 0, 0.0128177429097644, -0.0113041867027994, 0.00955378148811641, -0.00740032731935592, 0.00427258096992148, -0.0183832217438608, 0.0155366580724417, -0.0120346435940572, 0.00694820471863012, 0.0181644397838736, -0.014070114555226, 0.00812338442598859, -0.0136156449512337, 0.00786099627778517, 0.00607967912880136},
    {0.0100234421734311, -0.024904712849524, 0.014378742667763, 0.0536094339843701, -0.0141189246665008, 0.0109364720199429, -0.00631417506469887, 0.0133943880175428, -0.00773325352755854, 0.01531106082792, -0.00298267418725714, 0.0025208197798293, -0.00195261860521383, 0.00112734487734485, 0.0276525235907967, -0.0214195526695527, 0.0123665844996876, -0.0243925408037797, 0.0140830399992812, 0.0108976671476671, 0, 0, 0, 0, 0, 0.0197848223020205, -0.0167212267476003, 0.0129522065443228, -0.0074779599349644, -0.00321800158796925, 0.00249265331165481, -0.00143913406048031, -0.0106850252586226, 0.00616900220936375, 0.00872428659079183, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0.0206811244618434, -0.017478740331497, 0.0135389740433143, -0.00781673030845889, -0.0363288795677472, 0.028140229110452, -0.0162467688519772, 0.0357410679969885, -0.0206351152291861, -0.0182390373864041},
    {0.0100234421734311, 0.0354703486038675, -0.0204788153146928, -0.0449826285156207, 0.0141189246665009, -0.0109364720199429, 0.00631417506469889, -0.0044647960058475, 0.00257775117585279, -0.021143845905223, 0, 0, 0, 0, 0.00339592394974698, -0.00263047138047136, 0.00151870335961076, 0.0308358534689291, -0.0178030883009781, -0.0319414381914382, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0.0225260111157848, -0.0174485731815835, 0.0100739384233621, 0.00356167508620756, -0.00205633406978792, -0.0145404776513196, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0.028255795219359, -0.0218868448636849, 0.01263637577376, -0.047654757329318, 0.0275134869722481, 0.0303983956440068},
    {-0.0100234421734311, -0.0128297005588457, 0.00740723107127181, 0.0449826285156207, 0, 0, 0, 0.0178591840233903, -0.0103110047034114, 0.0211438459052229, 0, 0, 0, 0, 0, 0, 0, 0.0092047323787848, -0.00531435471670985, 0.0319414381914382, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0.0178083754310377, -0.0102816703489396, 0.0145404776513196, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0.0297842233308237, -0.017195929357655, -0.0303983956440068},
    {-0.0100234421734311, 0.0324515955311977, -0.0335503995581136, -0.00801060507812428, 0.0211783869997512, 0.00109364720199428, -0.00568275755822898, -0.0169662248222208, 0.00567105258687634, 0.0083846285486229, 0.0208787193108001, -0.0109235523792603, 0.00499002532443531, -0.00187890812890811, -0.0162519217595033, 0.00958243145743145, -0.00466458889023307, 0.0108922666482287, -0.00558007245254534, -0.00469727032227035, 0.0139343556230002, -0.00245778931055952, -0.00415442218609131, 0.0064360031759385, -0.00464478520766676, -0.0120907247401236, 0.00464478520766674, -0.000719567030240154, -0.000415442218609135, 0.00965400476390777, -0.00498530662330959, 0.00215870109072046, -0.00661453944581401, 0.0032313821096667, 0.00290809553026394, 0, 0.0127646671417816, -0.0202632808954031, 0.0228341307513453, -0.020635115229186, 0.0127646671417816, -0.00116524935543312, -0.00719357335632687, 0.0112908326677739, -0.0114368694935501, 0.00737991258440983, 0.00183832217438608, 0.00310733161448834, -0.00481385743762287, 0.00347410235931505, -0.00201827108709706, -0.000521115353897253, 0.000902598269554298, 0.00170195561890421, -0.000491312267361565, -0.000868525589828764},
    {0.0100234421734313, -0.0249047128495239, 0.0553363732971484, -0.00431340273437463, -0.0141189246665008, 0.0153110608279201, 0.00883984509057839, 0.00803663281052565, -0.0159820572902878, 0.000729098134662871, -0.00298267418725714, 0.0277290175781224, -0.0301571095694136, 0.0191648629148629, -0.00145539597846295, -0.0116492303992304, 0.00976309302606917, 0.00352848074520082, 0.00132858867917745, -0.00263047138047135, 0, 0.0196623144844763, -0.00727023882565982, -0.00724050357293083, 0.00928957041533347, -0.00219831358911338, -0.0148633126645336, 0.0129522065443228, -0.00664707549774609, 0.00402250198496158, 0.00685479660705067, -0.00575653624192123, -0.00457929653940971, -0.000293762009969677, 0.0029080955302639, 0, 0, 0.0202632808954031, -0.0399597288148543, 0.0464290092656686, -0.031911667854454, 0, -0.00411061334647249, -0.00303983956440069, 0.00908221989193683, -0.00776832903622088, 0.000459580543596524, 0.00427258096992149, -0.00270779480866288, 0.000868525589828788, -0.00100913554354855, -0.00234501909253767, 0.00180519653910856, 0.00127646671417817, 0.000245656133680783, -0.000868525589828766},
    {0.0100234421734309, 0.0354703486038674, -0.0492363006502187, -0.00431340273437457, 0.0141189246665008, -0.00656188321196574, -0.016416855168217, -0.00982255121286468, 0.0149509568199466, 0.00072909813466289, 0, 0.00336109303977242, 0.0282044909641998, -0.0353234728234728, -0.000485131992821028, -0.0124007936507936, 0.00889526253486298, 0.00291483191994853, 0.00239145962251946, -0.0026304713804714, 0, 0, 0.0218107164769794, -0.00402250198496158, -0.00928957041533346, 0, -0.00557374224920008, -0.0143913406048031, 0.0141250354327105, 0.000804500396992295, 0.00934744991870546, -0.00287826812096058, -0.00254405363300538, -0.00381890612960614, 0.00290809553026392, 0, 0, 0, 0.0266398192099029, -0.0515877880729651, 0.0425488904726054, 0, 0, -0.00911951869320201, 0.0084094628629044, -0.0038841645181104, 0, 0.00233049871086625, 0.00300866089851431, -0.00347410235931508, -0.000336378514516169, -0.00286613444643491, 0.00120346435940571, 0.000850977809452102, 0.000982624534723146, -0.000868525589828757},
    {-0.0100234421734311, -0.0128297005588456, 0.0448791059024117, -0.00801060507812423, 0, 0.0174983552319087, 0.0132597676358676, -0.00357183680467805, -0.0175287079957994, 0.00838462854862291, 0, 0, 0.00867830491206144, 0.0225468975468975, 0, -0.00300625300625305, -0.019200749617936, 0.000613648825252313, 0.0122230158484327, -0.00469727032227032, 0, 0, 0, 0.0160900079398463, 0.00464478520766673, 0, 0, -0.00719567030240156, -0.0108014976838374, 0, 0.00249265331165479, 0.0107935054536023, -0.000508810726601082, -0.00734405024924251, 0.00290809553026393, 0, 0, 0, 0, 0.0257938940364825, -0.031911667854454, 0, 0, 0, -0.0134551405806471, 0.0135945758133865, 0, 0, 0.00601732179702858, -0.00347410235931504, 0, -0.00208446141558903, -0.000902598269554288, 0.000425488904726059, 0.0017195929357655, -0.000868525589828767},
    {-0.010023442173431, -0.0324515955311979, -0.0187359374155701, -0.0289614183593722, 0.0211783869997511, 0.0164047080299144, 0.00947126259704825, -0.0044647960058475, -0.00257775117585286, -0.00619733414463431, -0.0208787193108001, -0.0176457384588052, -0.0136683302364968, -0.00789141414141413, -0.00848980987436743, -0.00657617845117846, -0.00379675839902691, -0.00230118309469614, -0.00132858867917743, -0.000187890812890788, 0.0139343556230002, 0.0122889465527977, 0.0103860554652283, 0.00804500396992312, 0.00464478520766673, 0.00109915679455666, 0.00092895704153335, 0.000719567030240155, 0.000415442218609149, -0.0048270023819539, -0.00373897996748222, -0.00215870109072046, -0.00559691799261192, -0.00323138210966673, -0.00290809553026393, 0, 0, 0, 0, 0, 0, 0.0128177429097644, 0.0113041867027994, 0.00955378148811642, 0.00740032731935589, 0.00427258096992148, 0.0183832217438608, 0.0155366580724417, 0.0120346435940572, 0.00694820471863011, 0.0181644397838736, 0.014070114555226, 0.00812338442598858, 0.0136156449512337, 0.00786099627778515, 0.00607967912880134},
    {0.0100234421734311, 0.024904712849524, 0.014378742667763, 0.05360943398437, -0.0141189246665008, -0.0109364720199429, -0.00631417506469881, -0.0133943880175428, -0.00773325352755853, 0.01531106082792, 0.00298267418725717, 0.0025208197798293, 0.00195261860521382, 0.00112734487734485, 0.0276525235907967, 0.0214195526695527, 0.0123665844996876, 0.0243925408037797, 0.0140830399992811, 0.0108976671476671, 0, 0, 0, 0, 0, -0.0197848223020205, -0.0167212267476003, -0.0129522065443227, -0.0074779599349644, -0.00321800158796927, -0.00249265331165477, -0.00143913406048032, 0.0106850252586227, 0.00616900220936373, 0.00872428659079179, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, -0.0206811244618434, -0.017478740331497, -0.0135389740433143, -0.00781673030845888, -0.0363288795677472, -0.028140229110452, -0.0162467688519772, -0.0357410679969885, -0.020635115229186, -0.018239037386404},
    {0.0100234421734307, -0.0354703486038675, -0.0204788153146927, -0.0449826285156207, 0.0141189246665008, 0.0109364720199429, 0.00631417506469886, 0.00446479600584751, 0.00257775117585285, -0.0211438459052229, 0, 0, 0, 0, 0.00339592394974697, 0.00263047138047138, 0.00151870335961076, -0.0308358534689291, -0.017803088300978, -0.0319414381914382, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0.0225260111157848, 0.0174485731815835, 0.0100739384233621, -0.00356167508620757, -0.00205633406978789, -0.0145404776513196, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0.0282557952193589, 0.0218868448636849, 0.01263637577376, 0.047654757329318, 0.027513486972248, 0.0303983956440067},
    {-0.010023442173431, 0.0128297005588457, 0.00740723107127182, 0.0449826285156208, 0, 0, 0, -0.0178591840233902, -0.0103110047034114, 0.021143845905223, 0, 0, 0, 0, 0, 0, 0, -0.00920473237878481, -0.00531435471670985, 0.0319414381914381, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, -0.0178083754310377, -0.0102816703489396, 0.0145404776513196, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, -0.0297842233308237, -0.017195929357655, -0.0303983956440068},
    {-0.0100234421734306, -0.0324515955311977, -0.0335503995581136, -0.00801060507812422, 0.0211783869997512, -0.00109364720199428, -0.00568275755822895, 0.0169662248222208, 0.00567105258687631, 0.00838462854862288, -0.0208787193108001, -0.0109235523792604, -0.00499002532443533, -0.00187890812890813, -0.0162519217595033, -0.00958243145743147, -0.00466458889023305, -0.0108922666482287, -0.00558007245254534, -0.00469727032227036, 0.0139343556230002, 0.00245778931055952, -0.00415442218609133, -0.00643600317593853, -0.00464478520766674, 0.0120907247401236, 0.00464478520766673, 0.000719567030240151, -0.000415442218609145, 0.00965400476390776, 0.00498530662330958, 0.00215870109072046, 0.006614539445814, 0.00323138210966669, 0.00290809553026394, 0, 0.0127646671417816, 0.0202632808954031, 0.0228341307513453, 0.020635115229186, 0.0127646671417816, -0.00116524935543314, 0.00719357335632687, 0.0112908326677739, 0.01143686949355, 0.00737991258440983, -0.00183832217438609, 0.00310733161448835, 0.00481385743762286, 0.00347410235931504, -0.00201827108709706, 0.00052111535389726, 0.000902598269554298, -0.00170195561890421, -0.000491312267361563, -0.000868525589828763},
    {0.0100234421734315, 0.024904712849524, 0.0553363732971485, -0.00431340273437464, -0.0141189246665008, -0.0153110608279201, 0.00883984509057836, -0.00803663281052558, -0.0159820572902878, 0.000729098134662864, 0.00298267418725715, 0.0277290175781224, 0.0301571095694136, 0.0191648629148629, -0.00145539597846298, 0.0116492303992304, 0.00976309302606916, -0.00352848074520084, 0.00132858867917744, -0.00263047138047133, 0, -0.0196623144844763, -0.0072702388256598, 0.00724050357293084, 0.0092895704153335, 0.00219831358911338, -0.0148633126645336, -0.0129522065443228, -0.0066470754977461, 0.00402250198496159, -0.00685479660705068, -0.00575653624192122, 0.0045792965394097, -0.000293762009969664, 0.00290809553026391, 0, 0, -0.0202632808954031, -0.0399597288148543, -0.0464290092656686, -0.031911667854454, 0, 0.00411061334647249, -0.00303983956440066, -0.0090822198919368, -0.00776832903622086, -0.000459580543596519, 0.0042725809699215, 0.00270779480866287, 0.000868525589828796, -0.00100913554354854, 0.00234501909253766, 0.00180519653910856, -0.00127646671417816, 0.00024565613368078, -0.00086852558982877},
    {0.0100234421734312, -0.0354703486038675, -0.0492363006502186, -0.0043134027343746, 0.0141189246665008, 0.00656188321196572, -0.016416855168217, 0.0098225512128646, 0.0149509568199467, 0.000729098134662864, 0, 0.0033610930397724, -0.0282044909641998, -0.0353234728234728, -0.000485131992820996, 0.0124007936507936, 0.00889526253486298, -0.00291483191994851, 0.00239145962251946, -0.00263047138047141, 0, 0, 0.0218107164769794, 0.00402250198496156, -0.00928957041533348, 0, -0.0055737422492001, 0.0143913406048031, 0.0141250354327105, 0.00080450039699229, -0.00934744991870546, -0.00287826812096059, 0.00254405363300538, -0.00381890612960615, 0.00290809553026392, 0, 0, 0, 0.0266398192099029, 0.0515877880729651, 0.0425488904726054, 0, 0, -0.00911951869320204, -0.00840946286290444, -0.00388416451811041, 0, 0.00233049871086624, -0.0030086608985143, -0.00347410235931509, -0.000336378514516169, 0.00286613444643491, 0.00120346435940572, -0.000850977809452107, 0.000982624534723155, -0.000868525589828754},
    {-0.010023442173431, 0.0128297005588457, 0.0448791059024117, -0.00801060507812423, 0, -0.0174983552319087, 0.0132597676358676, 0.0035718368046781, -0.0175287079957995, 0.00838462854862291, 0, 0, -0.00867830491206147, 0.0225468975468975, 0, 0.00300625300625302, -0.019200749617936, -0.000613648825252327, 0.0122230158484327, -0.00469727032227032, 0, 0, 0, -0.0160900079398463, 0.00464478520766674, 0, 0, 0.00719567030240152, -0.0108014976838374, 0, -0.00249265331165479, 0.0107935054536023, 0.000508810726601078, -0.00734405024924251, 0.00290809553026392, 0, 0, 0, 0, -0.0257938940364826, -0.031911667854454, 0, 0, 0, 0.0134551405806471, 0.0135945758133865, 0, 0, -0.00601732179702859, -0.00347410235931504, 0, 0.00208446141558904, -0.000902598269554294, -0.000425488904726048, 0.0017195929357655, -0.000868525589828768},
    {-0.0100234421734308, -0.0452812960900435, -0.0113287063442981, -0.00801060507812425, -0.00141189246665006, 0.0164047080299144, 0.0145226026488073, 0.0133943880175427, 0.0118576554089232, 0.0083846285486229, -0.00894802256177148, -0.0142846454190327, -0.014536160727703, -0.00939454064454062, -0.0123708658169354, -0.0125886844636845, -0.00813591085505763, -0.0102786178229763, -0.00664294339588729, -0.00469727032227033, -0.00836061337380013, 0.00245778931055953, 0.00830884437218264, 0.00965400476390776, 0.00650269929073342, 0.00219831358911336, 0.00743165633226678, 0.00863480436288185, 0.00581619106052787, 0.00643600317593851, 0.00747795993496438, 0.00503696921168107, 0.00610572871921291, 0.00411266813957582, 0.00290809553026393, 0.0352796762358899, 0.0191470007126724, 0.00788016479265675, 0.000951422114639387, -0.00221090520312706, -0.00212744452363027, 0.017478740331497, 0.00719357335632688, 0.000868525589828766, -0.00201827108709706, -0.00194208225905523, 0.00643412761035128, 0.00077683290362208, -0.00180519653910858, -0.00173705117965754, 0.000672757029032357, -0.00156334606169177, -0.00150433044925715, -0.00127646671417815, -0.00122828066840394, -0.000868525589828763},
    {0.010023442173431, 0.0603750614533912, -0.00610007264692973, -0.0043134027343746, -0.00564756986660029, -0.0218729440398858, 0.00126283501293976, -0.0178591840233903, 0.00103110047034112, 0.00072909813466285, 0.0417574386216002, 0.0168054651988621, -0.000867830491206138, -0.00526094276094278, 0.0145539597846298, -0.000751563251563223, -0.00455611007883228, -0.000613648825252315, -0.00372004830169692, -0.00263047138047137, 0.00557374224920009, -0.0196623144844763, -0.0103860554652283, 0.00321800158796927, 0.00650269929073345, -0.0175865087129071, -0.00928957041533348, 0.00287826812096062, 0.00581619106052783, -0.00804500396992314, 0.0024926533116548, 0.00503696921168107, 0.00203524290640432, 0.00411266813957582, 0.00290809553026392, -0.0705593524717798, -0.0127646671417816, 0.00450295131008957, 0.00380568845855756, -0.000736968401042374, -0.00212744452363027, -0.0116524935543313, 0.0041106133464725, 0.00347410235931506, -0.000672757029032359, -0.0019420822590552, 0.00367664434877216, 0.00310733161448835, -0.000601732179702866, -0.00173705117965752, 0.00269102811612943, -0.000521115353897279, -0.00150433044925714, -0.000425488904726059, -0.00122828066840393, -0.000868525589828762},
    {0.010023442173431, -0.0603750614533913, -0.00610007264692972, -0.00431340273437459, -0.00564756986660034, 0.0218729440398858, 0.00126283501293975, 0.0178591840233902, 0.00103110047034114, 0.000729098134662857, -0.0417574386216002, 0.016805465198862, 0.000867830491206142, -0.00526094276094274, 0.0145539597846298, 0.000751563251563242, -0.00455611007883226, 0.000613648825252326, -0.0037200483016969, -0.00263047138047136, 0.0055737422492001, 0.0196623144844763, -0.0103860554652283, -0.00321800158796928, 0.00650269929073344, 0.0175865087129071, -0.00928957041533347, -0.00287826812096062, 0.00581619106052785, -0.00804500396992314, -0.0024926533116548, 0.00503696921168109, -0.00203524290640433, 0.00411266813957582, 0.00290809553026392, 0.0705593524717798, -0.0127646671417816, -0.00450295131008957, 0.00380568845855755, 0.000736968401042381, -0.00212744452363028, -0.0116524935543313, -0.00411061334647249, 0.00347410235931505, 0.00067275702903236, -0.00194208225905522, -0.00367664434877216, 0.00310733161448835, 0.000601732179702862, -0.00173705117965754, 0.00269102811612941, 0.000521115353897267, -0.00150433044925715, 0.000425488904726055, -0.00122828066840394, -0.00086852558982877},
    {-0.0100234421734311, 0.0452812960900435, -0.0113287063442981, -0.00801060507812424, -0.00141189246665007, -0.0164047080299144, 0.0145226026488074, -0.0133943880175427, 0.0118576554089232, 0.00838462854862294, 0.00894802256177145, -0.0142846454190327, 0.014536160727703, -0.00939454064454065, -0.0123708658169354, 0.0125886844636845, -0.00813591085505765, 0.0102786178229763, -0.00664294339588732, -0.00469727032227034, -0.00836061337380011, -0.00245778931055953, 0.00830884437218264, -0.00965400476390776, 0.00650269929073342, -0.00219831358911338, 0.00743165633226677, -0.00863480436288184, 0.00581619106052786, 0.00643600317593851, -0.00747795993496437, 0.00503696921168106, -0.0061057287192129, 0.00411266813957581, 0.00290809553026391, -0.0352796762358899, 0.0191470007126724, -0.00788016479265674, 0.000951422114639392, 0.00221090520312706, -0.00212744452363026, 0.017478740331497, -0.00719357335632687, 0.000868525589828777, 0.00201827108709706, -0.00194208225905521, -0.00643412761035126, 0.000776832903622074, 0.00180519653910858, -0.00173705117965752, 0.00067275702903235, 0.00156334606169178, -0.00150433044925714, 0.00127646671417815, -0.00122828066840393, -0.000868525589828756},
    {0.0787556170769578, 0.0679219441350652, -0.00435719474780692, -0.00308100195312471, 0.0705946233325039, -0.0328094160598287, 0.0126283501293978, -0.0267887760350854, 0.03608851646194, -0.00729098134662863, 0.0298267418725716, 0.0151249186789758, -0.0325436434202305, 0.0248015873015872, 0.0130985638061668, -0.0229226791726792, 0.0169226945785199, -0.0248527774227189, 0.0164744996218005, 0.0142797017797018, 0, 0.0245778931055953, -0.0363511941282991, 0.0362025178646542, -0.0232239260383337, 0.0219831358911338, -0.00743165633226679, 0, 0.00166176887443652, -0.0345935170706695, 0.0130864298861876, -0.00359783515120077, 0.0335815079556711, -0.014688100498485, -0.0178640154001927, 0, 0, 0, 0, 0, 0, 0, 0.0226083734055987, -0.0334382352084074, 0.0333014729371016, -0.0213629048496074, -0.00459580543596519, -0.0194208225905522, 0.0270779480866286, -0.0191075629762328, 0.0090822198919368, 0.00703505727761299, -0.00812338442598857, -0.0102117337134253, 0.00196524906944628, 0.00607967912880137},
    {-0.0214788046573525, 0.00377344134083702, 0.00217859737390337, -0.021567013671873, 0, 0.0984282481794862, -0.0852413633734345, -0.0312535720409329, -0.0180442582309699, 0.0309866707231716, 0, 0.0453747560369276, 0, -0.0202922077922077, -0.0116431678277039, 0.0236742424242425, -0.0335199527228374, -0.00138070985681772, -0.000797153207506455, 0.00394570707070704, 0, 0, 0.0363511941282991, -0.0603375297744235, 0.0464478520766673, 0, 0.0241528830798671, 0, -0.0108014976838374, -0.00724050357293081, -0.0392592896585629, 0.0259044130886455, 0.0188259968842399, 0.0108691943688789, -0.0178640154001926, 0, 0, 0, 0, 0, 0, 0, 0, 0.0334382352084074, -0.0555024548951693, 0.0427258096992148, 0, -0.0155366580724417, 0, 0.00694820471863008, 0.00302740663064561, 0.0164151336477636, -0.0108311792346514, -0.00680782247561684, -0.00393049813889257, 0.00607967912880131},
    {0.0787556170769581, 0.0301875307266957, 0.0610007264692977, -0.00308100195312485, 0, 0.0218729440398858, 0.0757701007763862, 0.0178591840233902, -0.0412440188136458, -0.00729098134662867, 0, 0, 0.0520698294723689, -0.00901875901875909, 0, -0.0127765752765753, 0.0286384062098029, 0.00184094647575695, -0.0297603864135753, 0.0142797017797018, 0, 0, 0, 0.0402250198496157, -0.0464478520766674, 0, 0, 0.014391340604803, 0.0182794576188018, 0, -0.0137095932141014, -0.0345392174515274, 0.00407048581280862, 0.036426489236243, -0.0178640154001927, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0.0370016365967796, -0.0427258096992148, 0, 0, -0.030086608985143, 0.0243187165152054, 0, 0.014070114555226, 0, -0.00340391123780844, -0.00982624534723146, 0.00607967912880138},
    {-0.0214788046573519, 0.00377344134083692, -0.0196073763651313, 0.00924300585937408, 0, -0.0109364720199429, 0.00947126259704824, 0.102690308134494, -0.0695992817480272, -0.060150596109686, 0, -0.00504163955965862, 0.0065087286840461, -0.00450937950937948, 0.0465726713108155, 0.0214195526695527, -0.0289638426440052, -0.00782402252196701, -0.0263060558477138, 0.000563672438672498, 0, 0, 0, 0, 0, 0, 0.0334424534952005, -0.0431740218144092, 0.0299118397398575, 0.0289620142917233, -0.00249265331165477, -0.00431740218144094, -0.0605484764655283, 0.0185070066280912, 0.0461140862656136, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0.0349574806629939, -0.0451299134777144, 0.0312669212338355, -0.0121096265225824, -0.028140229110452, 0.0270779480866286, 0.023827378664659, 0, -0.0182390373864041},
    {-0.0214788046573522, -0.0150937653633478, 0.0130715842434209, 0.00924300585937409, 0, -0.0109364720199429, 0.00947126259704823, -0.00892959201169517, 0.123732056440937, -0.060150596109686, 0, 0, -0.00650872868404615, 0.00676406926406924, 0, 0.0574945887445888, 0.0126920209338899, -0.026693723898476, 0.00637722566005184, 0.000563672438672442, 0, 0, 0, 0, 0, 0, 0, 0.0431740218144092, -0.0448677596097863, 0, 0.0199412264932383, 0.0215870109072046, -0.0142467003448301, -0.0616900220936373, 0.0461140862656136, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0.0451299134777144, -0.0469003818507533, 0, -0.0375203054806026, 0.0162467688519772, 0.0119136893323295, 0.0206351152291861, -0.0182390373864041},
    {0.078755617076958, 0.0301875307266957, 0.0174287789912279, 0.0585390371093694, 0, 0.0218729440398858, -0.0189425251940966, 0.0178591840233902, 0.0103110047034113, 0.083846285486229, 0, 0, 0, 0, 0, 0.00526094276094275, -0.00455611007883227, 0.0533874477969518, 0.0308232573569172, -0.0319414381914382, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0.0348971463631671, -0.0302218152700865, 0.0284934006896603, 0.0164506725583033, -0.0643935438844155, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0.0437736897273697, -0.0379091273212801, -0.023827378664659, -0.0137567434861241, 0.0303983956440068},
    {0.0787556170769583, -0.0679219441350652, -0.00435719474780694, -0.00308100195312467, 0.0705946233325039, 0.0328094160598287, 0.0126283501293977, 0.0267887760350853, 0.03608851646194, -0.00729098134662864, -0.0298267418725716, 0.0151249186789759, 0.0325436434202305, 0.0248015873015873, 0.0130985638061668, 0.0229226791726792, 0.0169226945785199, 0.0248527774227189, 0.0164744996218005, 0.0142797017797018, 0, -0.0245778931055954, -0.036351194128299, -0.0362025178646541, -0.0232239260383336, -0.0219831358911338, -0.00743165633226678, 0, 0.00166176887443654, -0.0345935170706695, -0.0130864298861876, -0.00359783515120077, -0.0335815079556711, -0.014688100498485, -0.0178640154001927, 0, 0, 0, 0, 0, 0, 0, -0.0226083734055987, -0.0334382352084074, -0.0333014729371016, -0.0213629048496074, 0.00459580543596522, -0.0194208225905522, -0.0270779480866286, -0.0191075629762328, 0.0090822198919368, -0.007035057277613, -0.00812338442598858, 0.0102117337134253, 0.00196524906944627, 0.00607967912880136},
    {-0.0214788046573522, -0.00377344134083698, 0.00217859737390352, -0.0215670136718729, 0, -0.0984282481794863, -0.0852413633734345, 0.031253572040933, -0.0180442582309699, 0.0309866707231716, 0, 0.0453747560369276, 0, -0.0202922077922078, -0.0116431678277039, -0.0236742424242425, -0.0335199527228375, 0.00138070985681774, -0.000797153207506485, 0.00394570707070704, 0, 0, 0.0363511941282991, 0.0603375297744236, 0.0464478520766674, 0, 0.024152883079867, 0, -0.0108014976838374, -0.0072405035729308, 0.0392592896585629, 0.0259044130886455, -0.0188259968842399, 0.0108691943688789, -0.0178640154001927, 0, 0, 0, 0, 0, 0, 0, 0, 0.0334382352084074, 0.0555024548951693, 0.0427258096992148, 0, -0.0155366580724417, 0, 0.0069482047186301, 0.0030274066306456, -0.0164151336477636, -0.0108311792346515, 0.00680782247561683, -0.00393049813889256, 0.00607967912880134},
    {0.078755617076958, -0.0301875307266958, 0.0610007264692976, -0.00308100195312472, 0, -0.0218729440398859, 0.0757701007763862, -0.0178591840233903, -0.0412440188136458, -0.00729098134662865, 0, 0, -0.0520698294723688, -0.00901875901875904, 0, 0.0127765752765752, 0.0286384062098029, -0.00184094647575696, -0.0297603864135752, 0.0142797017797018, 0, 0, 0, -0.0402250198496157, -0.0464478520766673, 0, 0, -0.0143913406048031, 0.0182794576188018, 0, 0.0137095932141014, -0.0345392174515273, -0.00407048581280864, 0.036426489236243, -0.0178640154001927, 0, 0, 0, 0, 0, 0, 0, 0, 0, -0.0370016365967795, -0.0427258096992148, 0, 0, 0.0300866089851429, 0.0243187165152054, 0, -0.014070114555226, 0, 0.00340391123780844, -0.00982624534723146, 0.00607967912880137},
    {-0.0214788046573522, -0.00377344134083677, -0.0196073763651313, 0.00924300585937403, 0, 0.0109364720199429, 0.00947126259704829, -0.102690308134494, -0.0695992817480271, -0.0601505961096861, 0, -0.00504163955965865, -0.00650872868404618, -0.00450937950937947, 0.0465726713108155, -0.0214195526695527, -0.0289638426440052, 0.00782402252196705, -0.0263060558477139, 0.000563672438672498, 0, 0, 0, 0, 0, 0, 0.0334424534952005, 0.0431740218144092, 0.0299118397398575, 0.0289620142917233, 0.00249265331165484, -0.00431740218144093, 0.0605484764655283, 0.0185070066280911, 0.0461140862656136, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0.0349574806629939, 0.0451299134777144, 0.0312669212338355, -0.0121096265225824, 0.028140229110452, 0.0270779480866286, -0.023827378664659, 0, -0.018239037386404},
    {-0.0214788046573521, 0.0150937653633479, 0.0130715842434208, 0.00924300585937423, 0, 0.010936472019943, 0.00947126259704825, 0.00892959201169522, 0.123732056440937, -0.060150596109686, 0, 0, 0.00650872868404613, 0.00676406926406924, 0, -0.0574945887445887, 0.0126920209338899, 0.0266937238984759, 0.00637722566005184, 0.000563672438672452, 0, 0, 0, 0, 0, 0, 0, -0.0431740218144092, -0.0448677596097863, 0, -0.0199412264932383, 0.0215870109072046, 0.0142467003448301, -0.0616900220936373, 0.0461140862656136, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, -0.0451299134777144, -0.0469003818507533, 0, 0.0375203054806026, 0.0162467688519772, -0.0119136893323295, 0.0206351152291861, -0.0182390373864041},
    {0.0787556170769579, -0.0301875307266956, 0.017428778991228, 0.0585390371093696, 0, -0.0218729440398858, -0.0189425251940965, -0.0178591840233903, 0.0103110047034114, 0.083846285486229, 0, 0, 0, 0, 0, -0.00526094276094277, -0.00455611007883227, -0.0533874477969518, 0.0308232573569172, -0.0319414381914383, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, -0.0348971463631671, -0.0302218152700865, -0.0284934006896603, 0.0164506725583033, -0.0643935438844154, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, -0.0437736897273697, -0.0379091273212801, 0.023827378664659, -0.0137567434861241, 0.0303983956440068},
    {0.078755617076958, -0.0377344134083697, -0.0566435317214906, -0.00308100195312479, 0.0423567739995023, 0.0546823600997145, 0.0378850503881931, 0.0446479600584756, 0.00515550235170578, -0.00729098134662856, 0.0417574386216002, -0.00504163955965862, -0.0247331689993752, -0.0202922077922078, 0.0295930515620807, 0.0101461038961039, 0.00216957622801541, 0.0266937238984759, 0.0132858867917747, 0.0142797017797018, -0.0557374224920008, -0.0245778931055953, -0.00519302773261415, 0.00402250198496156, 0.00464478520766674, -0.00219831358911332, -0.0130053985814669, -0.0158304746652834, -0.0108014976838375, -0.0168945083368386, -0.026796023100289, -0.0194283098164841, -0.0295110221428625, -0.0217383887377579, -0.0178640154001927, 0, 0, 0, 0, 0, 0, -0.0512709716390577, -0.0226083734055987, -0.00477689074405822, 0.00370016365967794, 0.00427258096992147, -0.0367664434877216, -0.00776832903622086, 0.00601732179702859, 0.00694820471863011, -0.00908221989193683, 0.00703505727761299, 0.00812338442598859, 0.00680782247561683, 0.00786099627778515, 0.00607967912880135},
    {-0.0214788046573522, 0, -0.00435719474780689, -0.0215670136718729, -0.127070321998507, 0, 0.0284137877911447, 0, 0.0360885164619399, 0.0309866707231715, 0, 0.0453747560369275, 0, -0.0202922077922078, -0.0422064833754266, 0, -0.00618329224984384, 0, 0.0015943064150129, 0.00394570707070703, 0.0836061337380012, 0, -0.0103860554652283, 0, 0.00464478520766673, 0, 0.024152883079867, 0, -0.0108014976838374, 0.043443021437585, 0, -0.0194283098164841, 0, -0.0217383887377578, -0.0178640154001926, 0, 0, 0, 0, 0, 0, 0.0769064574585866, 0, -0.00955378148811639, 0, 0.00427258096992146, 0, -0.0155366580724417, 0, 0.00694820471863015, -0.0181644397838736, 0, 0.00812338442598858, 0, 0.00786099627778515, 0.00607967912880134},
    {0.0787556170769581, 0.0377344134083697, -0.0566435317214905, -0.00308100195312477, 0.0423567739995024, -0.0546823600997146, 0.0378850503881931, -0.0446479600584756, 0.00515550235170579, -0.00729098134662863, -0.0417574386216002, -0.00504163955965864, 0.0247331689993752, -0.0202922077922078, 0.0295930515620807, -0.0101461038961039, 0.00216957622801542, -0.0266937238984759, 0.0132858867917747, 0.0142797017797018, -0.0557374224920008, 0.0245778931055954, -0.00519302773261416, -0.00402250198496156, 0.00464478520766676, 0.00219831358911337, -0.0130053985814669, 0.0158304746652834, -0.0108014976838375, -0.0168945083368386, 0.026796023100289, -0.0194283098164841, 0.0295110221428625, -0.0217383887377579, -0.0178640154001927, 0, 0, 0, 0, 0, 0, -0.0512709716390577, 0.0226083734055987, -0.00477689074405822, -0.00370016365967795, 0.00427258096992149, 0.0367664434877216, -0.00776832903622086, -0.00601732179702859, 0.00694820471863009, -0.00908221989193678, -0.00703505727761301, 0.00812338442598859, -0.00680782247561686, 0.00786099627778517, 0.00607967912880135},
    {-0.021478804657352, -0.0188672067041847, 0.00653579212171052, 0.00924300585937401, 0.0141189246665008, 0, -0.00315708753234946, -0.111619900146189, -0.05413277469291, -0.0601505961096861, -0.00894802256177153, -0.00252081977982931, 0.000650872868404653, 0.00112734487734485, -0.0276525235907967, 0.0360750360750361, 0.0374251899332651, -0.0188697013765088, 0.019928830187662, 0.000563672438672417, 0, 0, 0, 0, 0, 0.0593544669060614, 0.0167212267476003, -0.00431740218144091, -0.0074779599349644, 0.00321800158796925, 0.0224338798048931, 0.018708742786244, 0.0463017761206981, 0.0431830154655461, 0.0461140862656137, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0.0620433733855302, 0.017478740331497, -0.00451299134777146, -0.0078167303084589, 0.0363288795677472, -0.00938007637015066, -0.0162467688519772, -0.0119136893323295, -0.020635115229186, -0.018239037386404},
    {-0.021478804657352, 0.0188672067041848, 0.00653579212171056, 0.00924300585937413, 0.0141189246665008, 0, -0.00315708753234944, 0.111619900146189, -0.05413277469291, -0.060150596109686, 0.00894802256177151, -0.00252081977982929, -0.000650872868404627, 0.00112734487734488, -0.0276525235907967, -0.0360750360750361, 0.0374251899332651, 0.0188697013765088, 0.0199288301876619, 0.000563672438672459, 0, 0, 0, 0, 0, -0.0593544669060614, 0.0167212267476003, 0.00431740218144093, -0.0074779599349644, 0.00321800158796927, -0.0224338798048931, 0.018708742786244, -0.046301776120698, 0.0431830154655461, 0.0461140862656136, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, -0.0620433733855302, 0.017478740331497, 0.00451299134777145, -0.00781673030845891, 0.0363288795677472, 0.00938007637015065, -0.0162467688519772, 0.0119136893323295, -0.020635115229186, -0.0182390373864041},
    {0.0787556170769582, 0, -0.0348575579824558, 0.0585390371093696, -0.0282378493330016, 0, 0.00631417506469889, 0, -0.0206220094068229, 0.083846285486229, 0, 0, 0, 0, -0.00679184789949391, 0, 0.00151870335961076, 0, -0.0616465147138344, -0.0319414381914382, 0, 0, 0, 0, 0, 0, 0, 0, 0, -0.0450520222315696, 0, 0.0100739384233621, 0, -0.0329013451166065, -0.0643935438844154, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, -0.0565115904387179, 0, 0.01263637577376, 0, 0.027513486972248, 0.0303983956440067},
    {0.0787556170769578, -0.0377344134083695, -0.0217859737390347, -0.0523770332031201, 0.0423567739995023, 0.0546823600997146, 0, 0.0446479600584757, 0.0257775117585286, 0.0291639253865144, 0.0417574386216002, 0.0285692908380655, 0.0273366604729936, 0.0187890812890813, -0.00921750786359888, -0.00789141414141413, -0.00347132196482459, -0.0248527774227189, -0.0143487577351166, -0.015031265031265, -0.0557374224920008, -0.00491557862111907, -0.0197335053839337, -0.0394205194526234, -0.0325134964536672, -0.0241814494802472, -0.00928957041533347, -0.0158304746652834, -0.0141250354327105, -0.00241350119097692, -0.000623163327913714, -0.00215870109072046, 0.00712335017241511, 0.00411266813957583, 0.00540074884191872, 0, -0.0510586685671264, -0.0405265617908061, -0.0114170653756727, 0.010317557614593, 0.0127646671417816, 0.00466099742173251, -0.0143871467126538, -0.00564541633388696, 0.00571843474677503, 0.00737991258440982, 0.00367664434877214, -0.00155366580724416, 0.00240692871881144, 0.00347410235931507, 0.00100913554354852, 0.000260557676948644, 0.000902598269554298, -0.000850977809452118, -0.000491312267361587, -0.000868525589828769},
    {-0.0214788046573518, 0, -0.0217859737390347, 0.00308100195312481, -0.127070321998507, 0, 0.0473563129852414, 0, 0.0257775117585285, 0.0127592173566, 0, -0.0352914769176103, 0, 0.00225468975468975, 0.0509388592462044, 0, -0.00943765659186686, 0, -0.0143487577351166, -0.0129644660894661, 0.0836061337380013, 0, 0.0477758551400502, 0, -0.0325134964536671, 0, 0.00928957041533347, 0, -0.0141250354327104, -0.0144810071458617, 0, -0.00215870109072047, 0, 0.00411266813957582, 0.00540074884191876, 0, 0.0765880028506897, 0, -0.0228341307513453, 0, 0.0127646671417816, -0.00699149613259876, 0, -0.0112908326677739, 0, 0.00737991258440979, 0, -0.00310733161448836, 0, 0.00347410235931506, 0.00201827108709705, 0, 0.000902598269554284, 0, -0.000491312267361592, -0.000868525589828792},
    {0.0787556170769577, 0.0377344134083696, -0.0217859737390349, -0.05237703320312, 0.0423567739995023, -0.0546823600997146, 0, -0.0446479600584756, 0.0257775117585286, 0.0291639253865144, -0.0417574386216002, 0.0285692908380655, -0.0273366604729936, 0.0187890812890813, -0.0092175078635989, 0.00789141414141413, -0.00347132196482458, 0.0248527774227189, -0.0143487577351166, -0.015031265031265, -0.0557374224920009, 0.00491557862111903, -0.0197335053839338, 0.0394205194526234, -0.0325134964536671, 0.0241814494802472, -0.00928957041533348, 0.0158304746652834, -0.0141250354327105, -0.00241350119097695, 0.000623163327913703, -0.00215870109072045, -0.00712335017241511, 0.00411266813957582, 0.00540074884191871, 0, -0.0510586685671264, 0.0405265617908061, -0.0114170653756726, -0.010317557614593, 0.0127646671417816, 0.0046609974217325, 0.0143871467126537, -0.00564541633388698, -0.00571843474677501, 0.00737991258440984, -0.00367664434877218, -0.00155366580724417, -0.00240692871881142, 0.00347410235931505, 0.00100913554354854, -0.000260557676948633, 0.000902598269554291, 0.000850977809452119, -0.000491312267361577, -0.000868525589828757},
    {-0.0214788046573522, -0.0188672067041847, 0.0108929868695173, 0.00308100195312474, 0.0141189246665008, -0.109364720199429, -0.0789271883087357, 0.0223239800292378, -0.0128887558792642, 0.0127592173566001, -0.00894802256177151, -0.0277290175781224, -0.00585785581564148, 0.0191648629148629, 0.00145539597846299, 0.0383297258297258, 0.0348216984596467, -0.0124263887113595, 0.00717437886755833, -0.0129644660894661, 0, 0.0589869434534289, 0.00727023882565978, 0.0506835250105158, 0.0650269929073344, -0.00659494076734013, 0.0148633126645336, -0.00431740218144089, -0.00166176887443655, -0.0040225019849616, -0.00810112326287806, -0.0115130724838424, 0.00356167508620753, -0.00205633406978792, 0.00540074884191869, 0, 0, 0.0607898426862092, 0.0399597288148543, -0.0154763364218895, -0.031911667854454, 0, -0.0123318400394175, 0.00303983956440066, -0.00302740663064559, -0.00776832903622087, 0.00137874163078958, -0.0042725809699215, 0.000902598269554286, 0.000868525589828745, 0.00100913554354854, 0.000781673030845884, 0.00180519653910853, -0.000425488904726043, 0.0002456561336808, -0.000868525589828759},
    {-0.0214788046573523, 0.0188672067041848, 0.0108929868695173, 0.00308100195312473, 0.0141189246665008, 0.109364720199429, -0.0789271883087356, -0.0223239800292378, -0.0128887558792643, 0.0127592173566, 0.00894802256177147, -0.0277290175781224, 0.00585785581564147, 0.0191648629148629, 0.00145539597846298, -0.0383297258297258, 0.0348216984596467, 0.0124263887113595, 0.00717437886755829, -0.0129644660894661, 0, -0.0589869434534289, 0.0072702388256598, -0.0506835250105158, 0.0650269929073343, 0.00659494076734013, 0.0148633126645336, 0.0043174021814409, -0.00166176887443651, -0.00402250198496155, 0.00810112326287803, -0.0115130724838425, -0.00356167508620755, -0.0020563340697879, 0.00540074884191873, 0, 0, -0.0607898426862092, 0.0399597288148542, 0.0154763364218895, -0.031911667854454, 0, 0.0123318400394175, 0.00303983956440069, 0.0030274066306456, -0.00776832903622087, -0.00137874163078958, -0.00427258096992146, -0.000902598269554311, 0.000868525589828782, 0.00100913554354855, -0.000781673030845884, 0.00180519653910857, 0.000425488904726043, 0.000245656133680809, -0.000868525589828767},
    {0.0787556170769581, 0, 0.0435719474780695, -0.0523770332031201, -0.0282378493330016, 0, 0.0631417506469885, 0, -0.0515550235170572, 0.0291639253865144, 0, -0.00672218607954481, 0, -0.0601250601250601, 0.000970263985641982, 0, -0.0125835421224891, 0, 0.0286975154702333, -0.015031265031265, 0, 0, -0.0436214329539588, 0, -0.0650269929073343, 0, 0.0111474844984002, 0, 0.031573608614294, -0.00160900079398462, 0, -0.0028782681209606, 0, -0.00822533627915164, 0.00540074884191871, 0, 0, 0, -0.0532796384198057, 0, 0.0425488904726053, 0, 0, 0.018239037386404, 0, -0.00388416451811043, 0, -0.00466099742173252, 0, -0.00347410235931507, 0.00067275702903234, 0, 0.00120346435940573, 0, 0.000982624534723144, -0.000868525589828764},
    {0.10739402328676, -0.188672067041848, -0.108929868695174, -0.0770250488281177, -0.0705946233325038, -0.0546823600997147, -0.0315708753234941, -0.0446479600584757, -0.0257775117585286, -0.0182274533665716, 0.0894802256177148, -0.0151249186789758, 0.058578558156415, 0.0744047619047618, -0.0130985638061668, -0.0281836219336219, 0.00976309302606916, 0.069035492840886, 0.0398576603753241, 0.0597492784992785, 0, 0.0737336793167861, 0.036351194128299, -0.0120675059548847, -0.0232239260383337, 0.0659494076734016, 0.00743165633226678, 0.0431740218144092, 0.0465295284842228, 0.0345935170706695, 0.0155790831978425, 0.0251848460584054, -0.0254405363300539, -0.0146881004984851, -0.0328199352701214, 0, 0, 0, 0, 0, 0, 0, 0.0678251202167962, 0.0334382352084074, -0.0111004909790339, -0.0213629048496074, -0.0137874163078956, 0.0194208225905522, -0.0090259826955429, -0.0191075629762328, -0.00908221989193678, -0.00234501909253769, -0.00812338442598858, 0.00340391123780841, 0.00196524906944627, 0.00607967912880134},
    {0.107394023286761, 0.188672067041848, -0.108929868695174, -0.0770250488281178, -0.0705946233325039, 0.0546823600997145, -0.0315708753234942, 0.0446479600584757, -0.0257775117585287, -0.0182274533665715, -0.0894802256177147, -0.0151249186789758, -0.0585785581564149, 0.0744047619047619, -0.0130985638061668, 0.0281836219336219, 0.00976309302606916, -0.0690354928408859, 0.039857660375324, 0.0597492784992785, 0, -0.0737336793167861, 0.036351194128299, 0.0120675059548847, -0.0232239260383338, -0.0659494076734015, 0.00743165633226678, -0.0431740218144092, 0.0465295284842228, 0.0345935170706695, -0.0155790831978424, 0.0251848460584053, 0.0254405363300539, -0.0146881004984851, -0.0328199352701214, 0, 0, 0, 0, 0, 0, 0, -0.0678251202167962, 0.0334382352084074, 0.0111004909790339, -0.0213629048496074, 0.0137874163078956, 0.0194208225905522, 0.00902598269554287, -0.0191075629762328, -0.0090822198919368, 0.00234501909253767, -0.00812338442598857, -0.0034039112378084, 0.00196524906944627, 0.00607967912880135},
    {0.107394023286761, 0, 0.217859737390348, -0.0770250488281178, 0, 0, -0.094712625970483, 0, 0.0515550235170572, -0.0182274533665716, 0, -0.0907495120738551, 0, -0.0946969696969696, 0.0232863356554077, 0, -0.0227805503941613, 0, -0.0797153207506479, 0.0597492784992786, 0, 0, -0.0727023882565981, 0, 0.0464478520766674, 0, -0.0483057661597341, 0, -0.0781031370985168, 0.0144810071458616, 0, 0.0431740218144092, 0, 0.0293762009969701, -0.0328199352701215, 0, 0, 0, 0, 0, 0, 0, 0, -0.0668764704168148, 0, 0.0427258096992148, 0, 0.0310733161448835, 0, 0.00694820471863007, -0.00605481326129118, 0, -0.0108311792346515, 0, -0.00393049813889257, 0.00607967912880135},
    {0.107394023286761, 0, 0, 0.231075146484353, 0, 0, 0, 0, 0, -0.109364720199429, 0, 0.0100832791193172, 0, -0.00450937950937963, -0.093145342621631, 0, -0.104139658944738, 0, 0, -0.09244227994228, 0, 0, 0, 0, 0, 0, -0.066884906990401, 0, 0.0299118397398574, -0.0579240285834467, 0, -0.0647610327216138, 0, 0, 0.0835038859404354, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, -0.0699149613259879, 0, 0.0312669212338356, 0.0242192530451648, 0, 0.0270779480866286, 0, 0, -0.018239037386404}};
    
    // Interesting (new) part
    // Tables of derivatives of the polynomial base (transpose)
    const static double dmats0[56][56] = \
    {{0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {6.32455532033671, 0, 0, 0, 0, 2.16505338810566e-14, 0, 2.05845987409731e-14, -1.22512335797976e-14, 0, 0, 0, 0, 0, -1.08130730953582e-14, 0, 1.22483177886604e-14, 0, 1.21041660596838e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {-1.33727360005496e-14, 11.2249721603218, 0, 0, 0, 2.8506046608589e-14, 0, 0, 0, 1.00951115474313e-14, 0, -1.20188905885721e-14, 1.28545373789999e-14, 0, 0, -1.03476262582923e-14, -1.93999491268998e-14, 0, 1.33226762955019e-14, 0, 0, 0, 0, 0, 1.05444038416428e-14, 0, 0, 0, -1.00620217474173e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1.11065094110678e-14, 0, 0, -1.90621990730363e-14, 0, 0, 1.25288297745861e-14, 0, 0, 0, 0, 0, 0, 0},
    {4.58257569495582, -1.0968053520467e-14, 8.36660026534075, -1.18321595661994, 0, 0, -1.51387532776558e-14, -1.08221538572235e-14, -1.3598597955937e-14, 0, 0, 1.9215300364784e-14, -1.53957147424772e-14, 0, 0, 1.50088321802921e-14, 0, 0, 0, -1.15463194561016e-14, 0, 0, 0, 0, 0, 0, 0, -1.00882470358913e-14, 0, 0, 0, 0, 0, 0, 0, 0, 1.03041185216246e-14, 0, 0, 0, 0, 0, 0, 0, 1.67984122495906e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {3.74165738677394, 2.85608062261247e-14, 0, 8.69482604771366, 0, 1.32677370521961e-14, -1.0621406876971e-14, 4.08006961549745e-14, -2.2818496382273e-14, -2.1316282072803e-14, 0, 0, 0, 0, 0, -1.21576837027352e-14, 0, 1.41223042954593e-14, 1.22383070012588e-14, -1.4210854715202e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, -1.00375396573897e-14, -1.10000775654917e-14, 0, -1.24344978758018e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, -1.27154464053485e-14, 0, -1.33226762955019e-14},
    {0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {5.49909083394704, -2.12464983341981e-14, -3.34664010613631, -2.36643191323983, 15.4919333848297, 0, 0.692820323027548, 0, 0.565685424949223, 0.400000000000019, -2.45617804595055e-14, 0, -2.50285594161414e-14, -1.12752625494543e-14, 0, -2.06682744450517e-14, 0, -1.87825158914691e-14, 0, 0, 0, 1.67613011937654e-14, 0, 0, 0, 1.28202275780632e-14, 0, 1.30009839200864e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, -1.30459382160744e-14, 0, 0, 0, 0, -1.95146235819213e-14, 1.32110885228657e-14, 1.12246955104063e-14, 0, -1.24476276034392e-14, 0, 0, 0, 1.14879018579048e-14, 0, 0, 0, 0, 0},
    {1.79012076899773e-14, 4.89897948556635, 0, 0, 0, 14.1985914794391, -1.56346770721582e-14, -0.828078671210841, 1.85962356624714e-14, 1.44765989981636e-14, 0, -3.14433996145488e-14, 0, -1.75618818945982e-14, -1.31955099699921e-14, 1.85934501523347e-14, -2.7953864447905e-14, 1.28785870856518e-14, 0, 0, 0, 0, 1.3985009962376e-14, 0, 1.27337992225105e-14, 0, 1.07957902261976e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, -1.64679520687763e-14, 0, -1.14163879478899e-14, 0, 0, -1.9487535909084e-14, 1.99307532306351e-14, -2.46558752706505e-14, 0, 0, 0, 0, 0, 1.11086317443674e-14, 0, 0, 0, 0},
    {3.60000000000003, -1.17043800208557e-14, 8.76356092008264, -1.54919333848296, 0, 0, 9.52470471983252, -1.08725201100446e-14, -1.48131215963609, 0.261861468283195, 0, 0, -2.19166032081878e-14, -1.15727480921906e-14, 1.19782412678522e-14, 0, 2.00140531387981e-14, -1.08284340452512e-14, -2.62934736398987e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1.25371390349952e-14, 0, 0, 0, 0, 0, -1.02110492017178e-14, 0, 0, 0, 0, 0, 0},
    {0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {-2.24915431235802e-14, 4.24264068711927, -1.64313007644523e-14, 0, 0, 0, 0, 14.3427433120127, 0, 1.59620734906468e-14, 0, 0, 2.06494753825246e-14, 1.24642807480246e-14, 1.31854538806711e-14, 0, -2.70659809367255e-14, 0, 0, -1.23240404410758e-14, 0, 0, 0, 0, 0, 0, 0, 0, -1.22125558154546e-14, 1.05003199257295e-14, 1.39537903134404e-14, 1.71734205981969e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, -2.01310662662892e-14, 0, 0, 2.30351452905449e-14, 0, 0, 0, 0, 0, 1.24344978758018e-14, 0},
    {3.11769145362398, 0, 3.1622776601684, 4.91934955049952, 0, 0, 0, 0, 10.690449676497, -2.41897262725906, 0, 0, 0, 1.08320679531897e-14, 0, 0, 0, 1.61780317385165e-14, 0, -3.01980662698043e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, -1.01163085491832e-14, 0, -1.4210854715202e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1.90254351599586e-14, 0, 0, 0, 1.4669610635169e-14, 0, 0, 0, 1.20972007395846e-14, 0, 0, 0},
    {0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {2.54558441227159, 0, 0, 7.66811580507233, 0, 0, 0, 2.00435576705123e-14, 0, 10.3691851174526, 0, 0, 0, 0, 0, 0, -1.4355744756654e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, -1.31237403589518e-14, 0, 1.19255832274785e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, -1.21989789544854e-14, 0, 0, 0, 1.08631431175004e-14, 0, 0, 0, 0, 0, 0},
    {0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {-3.7592379574637e-14, 11.60576914948, 1.02695629777827e-14, 0, 0, -4.20458932936044, 1.40755215458979e-14, -3.43303281162799, 0, 0, 19.6214168703486, 2.18203847862408e-14, 0.611677741841185, 1.46622784386203e-14, 0, 0.529728463363973, 1.12642251693005e-14, 0.4325214791568, 0, 0, 0, -1.06703789507439e-14, 0, 0, 0, -1.60649692254054e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1.42508255988556e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {4.53137212479104, -2.04494956755329e-14, 3.54562104171167, -2.50713268211203, 4.78713553878169, 0, -5.56626745075489, 0, -1.04880884817014, 0.741619848709575, -1.23815345194237e-14, 18.8034951158403, -1.86020833228553e-14, 1.68183573174416, -0.603122387046451, -1.92746782669555e-14, 0.701283781217653, 0, 0.132137494528672, -0.0934353184302359, 0, 1.50506010183261e-14, 0, 0, 0, 1.11149238093515e-14, 0, 1.50741577923534e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, -1.5968916436806e-14, 0, 0, 0, 0, -2.36522650283687e-14, 1.26860679795035e-14, 2.21112666692775e-14, -1.04107559329747e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {3.86209880120533e-14, 2.5951288749407, 1.92068583260152e-14, 0, 0, 10.0285307284481, -1.22612314755764e-14, -0.767649473578743, 0, 0, 0, 0, 16.4130361329658, 0, 0, -1.2634761106116, 3.26893402596665e-14, 0.0967147429123276, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1.32878527298855e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, -1.0672045061505e-14, -1.16637535122366e-14, 0, -1.46741880066399e-14, 0, 1.32138836425176e-14, 0, 0, 0, -1.00831659427246e-14, 0, 0, 0, 0, 0, 0, 0},
    {2.96647939483827, 0, 8.12403840463593, -1.64130361329658, 0, 0, 11.7728501222092, 0, -2.40312296813959, 0.485504156227621, 0, 0, -1.71460988515286e-14, 10.2761860629321, 1.14956049951209e-14, -4.33332801490326e-14, -1.48323969741912, -1.03124879870092e-14, 0.302765035409724, -0.0611677741841183, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, -1.45169999216629e-14, 1.16636407607716e-14, -2.16593088290316e-14, 0, 0, 0, 0, -1.3706959798735e-14, 0, 0, 0, 0, 0, 0},
    {0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {4.05298244095222, -1.17512299762369e-14, -1.58564993434419, 4.48489528465113, 4.28174419288836, 0, 0.191485421551272, 0, -3.75233260785876, -4.311612227462, 0, 0, -1.13529814871025e-14, -1.07967706476489e-14, 18.8807171866293, 0, 0.844371341865059, -1.30185990386282e-14, 1.18187368057055, 1.04463861754669, 0, 1.00351859283595e-14, 0, 0, 0, 1.08982318008379e-14, 1.16650614658273e-14, 1.78540699708776e-14, 0, 1.48568586111212e-14, 1.2997651195193e-14, 0, 1.19564095870861e-14, 0, 0, 0, 0, 0, 0, 0, 0, -1.90031310112405e-14, -2.21313572206035e-14, 0, 2.31096376853888e-14, 1.25147787012743e-14, -1.31451704746157e-14, 0, 0, 0, 2.34657400816165e-14, 0, 0, 0, 1.19833135941395e-14, 0},
    {1.03681424251779e-14, 2.321153829896, -1.04360964314765e-14, 0, 0, 3.9242833740697, 0, 5.49285249860475, 0, 1.4249489099847e-14, 0, 0, 2.63684141763982e-14, 1.12010640489931e-14, 0, 17.3044631365566, -5.0266260224736e-14, -1.73008591662709, 0, 0, 0, 0, 0, 0, 1.32329702395699e-14, 0, 1.22894929912907e-14, -1.21668635551694e-14, -1.35421297848238e-14, 1.0253781445641e-14, 0, 1.86415806813024e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 2.41268605947696e-14, -3.37685959257577e-14, 0, 0, 3.2419745025707e-14, 1.21885930822672e-14, 0, 1.77958086184916e-14, 0, 1.4210854715202e-14, 0, 1.40814441510978e-14},
    {2.65329983228435, -2.32098024377698e-14, 4.15220619990512, 2.93605316083775, 0, -1.72381743916861e-14, 2.63248931621763, -4.00364084983131e-14, 9.82591326180784, -2.8226127713774, 0, 0, -1.33474731826653e-14, -1.06959103616322e-14, 1.26654039377864e-14, 1.8991099966876e-14, 11.6081867662439, -1.19927215290095e-14, -3.0948717731947, 0.683876505540241, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1.61148409137901e-14, 1.47100077738277e-14, 0, 0, 0, 0, 0, 0, 0, 1.3422167174583e-14, 0, -2.15269669668842e-14, 0},
    {0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {4.0410983709019e-14, 2.01017818278147, 0, 1.34362847627492e-14, 0, -2.17732253774783e-14, 0, 8.91928088068604, -1.37667655053519e-14, 1.38253977761529e-14, -1.02564879604251e-14, 0, 0, 0, -1.34085149628415e-14, 0, 0, 16.8558564884489, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1.06581410364015e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, -1.48981943277725e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1.77635683940025e-14, 0, 0},
    {2.29782505861523, 1.22923050669652e-14, 1.49829835452879, 5.50917601898536, 0, 0, 0, 1.92682707206214e-14, 6.64803945320939, 4.13676892548485, 0, 0, 1.46016900962614e-14, 2.50030852312528e-14, 0, -1.88410641965856e-14, -3.22541831300375e-14, -1.10413917361675e-14, 12.5636136423842, -3.55352656109507, 0, 0, 0, 0, 0, 0, 0, 0, -2.00805899834693e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, -1.13841034187579e-14, -2.03312427847762e-14, 0, 0, 0, 1.35488987469781e-14, 0, 0, 0, 0, -1.41455019594718e-14, 0},
    {0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {1.87616630392938, -2.83080569710049e-14, 0, 6.22830929985772, 0, 0, 0, -1.88930188353776e-14, 0, 11.0541524195339, 0, 1.07055222640856e-14, 0, 0, 0, 1.48136724622646e-14, 1.71418057121951e-14, -2.40309133366388e-14, 0, 11.6057691494799, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1.22846189228214e-14, 1.32604072649903e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1.33226762955019e-14},
    {0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {4.83186700722507, 0, -4.41087092453692, -3.11895674167863, 16.6749979177077, -1.13866638474088e-14, 2.13065307820926, -2.61564103865851e-14, 1.73967095350101, 1.23013312825381, 0, -4.36653943835008, 1.40983965277648e-14, -0.557935943461962, -3.78153408023779, 2.08111552908862e-14, -0.483186700722529, 1.18208720943676e-14, -0.394520289089675, -0.278967971730977, 23.6981011897578, 0, 0.490653381462656, 0, 0.0626933893908963, -1.13263908790947e-14, 0.438853725736244, 0, 0.0560746721671562, 0.380058475033041, 0, 0.0485620906056454, 0, 0.0396507809421992, 0.0280373360835894, 0, 0, 0, 0, 0, 0, 1.56357143454011e-14, 0, 0, 0, 0, 1.0335662317795e-14, 1.60193852711159e-14, 0, 0, 0, 1.11786220690221e-14, -1.25974506359966e-14, 0, -1.04950117655873e-14, 0},
    {-2.48352929483769e-14, 6.91050664139874, 0, -1.11694286112743e-14, 0, 10.5706241650195, 1.37569619238853e-14, -2.72554057547698, -1.5404344466674e-14, 0, 4.67332857821917, 1.98201416293709e-14, -7.86705867811461, 0, 0, -2.39719223514615, 0, 0.61809450436526, 0, 0, 0, 23.10555488766, 0, 1.62065746295907, 0, -0.469687194424188, 0, 0.790669146636838, 1.33915447641953e-14, 0, 0.240926884676765, 0, -0.0621208350291002, 0, 0, 0, 1.5285431037408e-14, -1.09568490725226e-14, 0, 0, 0, 0, 0, -1.60712929692335e-14, 0, 1.01513003014644e-14, 0, 0, -1.13906592977489e-14, 0, 0, 0, 0, 0, 0, 0},
    {3.85449644663769, 0, 5.98171817966647, -2.48806675764062, 1.90029237516522, 0, -0.339934634239515, 0, -2.35922146606158, 0.981306762925306, -1.25696577846526e-14, 9.9522670305624, 1.59597845699183e-14, -6.45364422706461, -0.430945803685653, 1.41669885314633e-14, 0.0770899289327383, 0, 0.535021138879222, -0.222539456105695, 0, 0, 21.5273297444417, 0, 2.75066129614998, 0, -1.00024047132728, 0, 0.648615649442928, 0.0433116829031283, 0, -0.00774782937530501, 0, -0.053771647653677, 0.0223660568773448, 0, 0, 0, 0, 0, -1.57026876352916e-14, 0, 0, 0, 0, 0, 0, 1.17596913617043e-14, 0, 0, 0, 0, -1.17877079382012e-14, 0, 0, 0},
    {0, 1.54523626091315, 0, 0, 0, 6.71778024562961, 1.24434270987286e-14, -0.609449400220049, -2.44890913103646e-14, 0, 0, 1.86912392652466e-14, 14.6593983227032, 0, 1.30441346035566e-14, -1.52344936219876, 0, 0.138210132827974, 0, 0, 0, 0, 0, 18.119501277094, 0, 0, 0, -1.47332496632642, 0, 0, 0.153112421864233, 0, -0.0138906409944119, 0, 0, 0, 0, 0, 1.22183321677886e-14, 0, 0, 0, -1.08769537630915e-14, 0, -1.31873092152821e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {2.52336024752224, 0, 7.37120708415863, -1.62882203585592, 0, 0, 12.0171306297064, 0, -2.90724328050878, 0.642416074439641, 0, 0, 1.07383829658909e-14, 13.9858820944259, 0, -2.32400745727179e-14, -2.72522906732405, 0, 0.659300804656265, -0.145686271816941, 0, 0, 0, 0, 10.8043830863937, 0, 0, 0, -1.40563403846548, 0, 0, 0.273895826790425, 0, -0.0662622240310308, 0.0146420212340205, 0, 0, 0, 0, 0, -2.09114976580726e-14, 0, 0, 0, 0, -2.35127496508513e-14, 0, 0, 0, -1.14219479838847e-14, 0, 0, 0, 0, 0, 0},
    {0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0, 6.30840061880561, 0, 0, 0, -1.52362350055011, 0, 11.1963004093827, -2.55351295663786e-14, 0, 4.26614580154032, 0, 0.132992762321598, 0, 2.56609067489954e-14, -4.72217953828914, 1.17310134406447e-14, -6.67684597013019, 0, 1.60758884410055e-14, 0, 0, 0, 0, 0, 23.1532444221696, 0, 0.72177887856059, 1.28783366735277e-14, -1.53368636742207e-14, 1.1112512795529, -1.22624479997289e-14, 1.19087439227729, 0, 0, 0, 0, 0, 0, 0, 0, 2.28291451277892e-14, 0, 0, 0, 1.34418865877235e-14, 2.02478204822475e-14, 0, -1.70814325121815e-14, -1.2090896650498e-14, 0, 0, 0, -2.48689957516035e-14, 0, 0},
    {3.518657752745, 0, 1.92724822331884, 2.72554057547703, 1.73472166622177, 0, -2.01705689521107, 0, 3.42052627529744, -4.03112887414927, 0, 4.08831086321546, -1.50711171565645e-14, 0.365669640132009, 5.37643791520789, -1.10002031399277e-14, -6.25148194071022, 0, -2.03981013744553, 1.8080332206528, 0, 1.12311783689995e-14, -1.2495355609558e-14, 0, 0, 0, 22.1880978975596, 2.59644251000699e-14, 1.98456380761453, -1.2652152389098, 0, 1.47113578579291, 0, 0.363818136386349, -0.294009452979948, 0, 0, 0, 0, 0, 0, 0, -2.14091784397362e-14, 3.96979729470341e-14, 2.95273442389967e-14, 0, -1.02898405303999e-14, -1.55944604137083e-14, 0, -1.2756671985334e-14, 2.67341313377078e-14, 0, 0, 0, 0, 0},
    {5.00588290781391e-14, 1.4106012612951, 1.86517468137026e-14, 1.57048767420779e-14, 1.14431516135412e-14, 3.63405410063598, 0, 2.50356888118883, 0, 0, 0, 0, 3.56857028479905, 0, 0, 11.263055412878, 3.38127349244425e-14, -1.49298814645067, 0, 2.47950570155456e-14, 0, 0, 0, 0, 0, 0, 0, 19.3673596462205, 0, 0, -2.65048896124153, 0, 0.266287609379583, 0, 0, 0, 0, 0, 0, 0, 0, 0, -1.24805331057479e-14, 0, 2.67117428927039e-14, 0, 0, 0, -2.83074697144157e-14, 0, 0, 0, 0, 0, 0, 0},
    {2.30350221379958, 0, 4.4158804331639, 1.78428514239955, 0, 0, 4.2661458015403, 1.92100483859941e-14, 7.83741028656788, -2.63899331455873, 0, 0, 0, 2.23427840700302, 1.30548943777412e-14, -2.38078955158898e-14, 13.2221027072096, -1.23603801475692e-14, -4.67379217909115, 1.18363558466232, 0, 0, 0, 0, 0, 0, -1.58444179603095e-14, 0, 12.1258851598178, 0, 1.08652403506101e-14, -3.11150358274815, 0, 0.833612074594099, -0.192474367616158, 0, 0, 0, 0, 0, 0, 0, 0, 1.49231173587483e-14, -1.98974312579798e-14, -1.74186427463722e-14, 0, 0, 0, 0, 0, 0, 0, 0, -2.0574377947513e-14, 0},
    {0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {3.14718316987773, 0, -0.861891607371334, 6.09449400220044, 1.55158222708543, 0, 0.0693888666488801, 0, -3.05941170815564, 1.80277563773202, 0, 0, 0, 0, 9.03122156537873, 2.80112084875395e-14, 0.403888506800968, -1.0607742830904e-14, -3.67461994579463, -5.49650682721189, 0, 0, 0, 0, 0, 0, 1.10205726024203e-14, 0, 1.65064789881475e-14, 21.7841260588564, 0, 0.974215733960566, 1.13989530288554e-14, 1.78974858569577, 1.80791908794933, 0, 0, 0, 0, 0, 0, -1.4994119558265e-14, 0, -2.17157199142971e-14, 0, 1.16195530358102e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 2.22044604925031e-14},
    {0, 1.26168012376113, 1.95399252334028e-14, 0, -1.17816950171735e-14, 1.42204860051341, 1.74453095862365e-14, 4.47852016375308, -2.08721928629529e-14, 1.0744670813999e-14, -1.34464719541228e-14, 3.78456620442729e-14, 0, 2.80676208590537e-14, 0, 8.27725128825316, 0, 5.37909280973871, 1.24344978758017e-14, -1.95594693216656e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 19.9654813226339, 0, -2.61992366301004, 1.06581410364015e-14, 0, 0, 0, 0, 1.3455473305103e-14, 0, 1.46585553191507e-14, 0, -1.27103118437955e-14, 0, -2.2308537695773e-14, 0, 0, 1.05662688412451e-14, 2.42300772936765e-14, 0, 0, 0, 1.19298522441569e-14, 2.66453525910038e-14, 1.50990331349021e-14, 0},
    {2.06031501455086, -3.07345773477637e-14, 2.25696201807219, 3.98978286964828, 0, -1.17529718472799e-14, 0.953939201416948, -4.35480723877723e-14, 8.01142041980431, 1.18019368870419, 0, 0, 0, 1.31328347545057e-14, 0, 1.03323004232013e-14, 5.5525489642145, -2.26382415357737e-14, 9.62241374388481, -3.59830837050572, 0, 0, 0, 0, 0, 0, 1.02408289475646e-14, 0, 0, 0, 0, 13.3932520322736, 0, -4.6866619250806, 1.18356086726904, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1.27725097955809e-14, -1.27832588361236e-14, 0, 0, -1.04546715347572e-14, 1.45339591718248e-14, 0, 0, 0, 0, -1.59580006645491e-14, 1.4210854715202e-14},
    {0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {3.93184363124296e-14, 1.09264703862703, 1.28785870856518e-14, 0, 0, 1.22575380004522e-14, -1.14864973879958e-14, 5.38682254607081, 0, 0, 0, -1.44121099608086e-14, 0, -1.75698029368367e-14, -1.80160878442005e-14, 2.57528022630009e-14, 2.69054202848798e-14, 13.4377817958113, 0, 0, 0, 0, 1.06823510383984e-14, 0, 0, 0, 0, 0, 1.89056142369167e-14, 0, -1.31345967659526e-14, -1.51823905359486e-14, 18.907670401189, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, -1.07391608525697e-14, 0, 1.46384409866762e-14, 0, 0, -1.63612658717074e-14, 0, 0, 0, 0, 0, 0, 0},
    {1.78428514239956, -1.14226462513753e-14, 0.814411017927918, 5.06770487035906, 0, 1.13885635970811e-14, 0, 1.47667974881489e-14, 4.01510046524763, 6.58672305740271, 0, 2.01242961116133e-14, 0, -1.5623829602503e-14, 0, 0, 0, -3.27887259670989e-14, 10.0159311874144, 2.83293314500738, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1.60502464055099e-14, 0, 1.06885013954928e-14, 14.0929454377398, -4.55552790213387, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, -2.19027884045748e-14, -1.24344978758018e-14},
    {0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {1.4568627181694, 0, 0, 5.07816454066245, -1.07582259106397e-14, 0, 0, -2.20867693944166e-14, 0, 10.0142755247554, 0, 0, 0, 0, 0, 1.02210862188786e-14, 3.51018877774324e-14, 0, 0, 13.8784813613724, 0, 0, 0, 0, 0, 0, 0, 0, 1.67232492370418e-14, 0, 0, 0, 0, 0, 12.5535587278944, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1.55927253999833e-14, 1.90609040202255e-14, 0, 0, 0, -1.79253699586741e-14, 0, 0, 1.08479262389015e-14, 0, 0, 3.10862446895044e-14},
    {0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0}};
    
    const static double dmats1[56][56] = \
    {{0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {3.16227766016836, 0, 0, 0, 0, 1.08252669405283e-14, 0, 1.02922993704865e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {5.47722557505162, 0, 0, 0, 0, 1.92068165630412e-14, 0, 1.80834223416197e-14, -1.14152204219919e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {2.95803989154978, 5.6124860801609, -1.08012344973465, -0.763762615825971, 0, 2.58464835607474e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, -1.40850097772758e-14, 0, 0, 1.16248598628335e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {2.2912878474779, 7.24568837309471, 4.18330013267037, -0.591607978309959, 0, 2.03703972918374e-14, 0, 0, 0, 1.08801856413265e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1.16401775975232e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {-2.64575131106454, 0, 9.66091783079294, 0.683130051063963, 0, -1.49625671831933e-14, 0, -3.2502596093762e-14, 0, 0, 0, 1.34670915888159e-14, 0, 0, 0, 2.90440444432224e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {1.87082869338697, 1.42804031130623e-14, 0, 4.34741302385683, 0, 0, 0, 2.04003480774873e-14, -1.14092481911365e-14, -1.06581410364015e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {3.24037034920393, 2.38080669966159e-14, 0, 7.52994023880667, 0, 1.13478631598507e-14, 0, 3.45590001304086e-14, -1.69864122767649e-14, -1.4210854715202e-14, 0, 0, 0, 0, 0, -1.32400798627181e-14, 0, 1.22473609554424e-14, 1.33269883629769e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {2.74954541697351, 5.79655069847575, -1.67332005306816, -1.18321595661991, 7.74596669241483, -1.19999999999998, 0.346410161513775, -0.979795897113269, 0.282842712474611, 0.200000000000014, -1.0696429695597e-14, -1.26705171178379e-14, -1.2948158065601e-14, 0, 0, -1.45874734860756e-14, 0, -1.16672151462161e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, -1.07102102118129e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {2.32379000772449, 2.44948974278316, 2.82842712474617, -0.999999999999999, 9.16515138991168, 7.09929573971955, -2.04939015319193, -0.414039335605429, -0.478091443733758, 0.169030850945723, 0, -1.28646947787458e-14, -1.8460920120059e-14, -1.81736908030475e-14, 0, 1.37951452580958e-14, -1.77890663211312e-14, 0, 0, 0, 0, 1.08892803896054e-14, 0, 0, 0, 0, 1.05640513109647e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, -1.22044403129704e-14, 0, 0, 0, 2.27334526137921e-14, -1.76454983335969e-14, -1.05574887489127e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {1.80000000000004, -5.69209978830308, 4.38178046004131, -0.774596669241497, 1.24189930203385e-14, 10.998181667894, 4.76235235991625, 0.962140470884722, -0.740656079818038, 0.130930734141603, 0, -1.20430917197879e-14, -1.1981685754404e-14, -2.88797787541631e-14, -1.3779450272627e-14, 0, 0, 0, -2.22505656218757e-14, -1.14908083048704e-14, 0, 0, 0, 0, 0, 0, 0, 0, 1.22673594366302e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, -1.12284484669662e-14, 0, 0, 0, 0, 0, 0, 0, 0, -1.1387566167118e-14, 0, 1.48982410846523e-14, 0, 0, 0, 0},
    {5.19615242270664, 0, -3.16227766016842, -2.23606797749978, 0, 0, 13.7477270848675, 1.78325020862166e-14, 0.534522483824845, 0.377964473009226, 0, -2.80136022320966e-14, -1.36437984192736e-14, -2.9552592280819e-14, 1.3605842011612e-14, -4.49783579557011e-14, 2.03714412277317e-14, -1.61597054173618e-14, -2.47182474765566e-14, 0, 0, 0, 0, 0, 0, 0, 0, 1.7014378114654e-14, 0, 0, 0, -1.14827159581245e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, -1.57668143745762e-14, 2.14083883902157e-14, -2.68877125810315e-14, 0, 0, 0, -1.36900091929644e-14, -2.0687516582275e-14, 0, 0, 0, 0, 0, 0},
    {2.01246117974979, 2.12132034355965, -0.408248290463867, 3.17542648054294, 0, 1.00616573615107e-14, 0, 7.17137165600639, -1.38013111868472, -1.56144011671766, 0, 0, 1.10848947143279e-14, 0, 0, -1.41970084791766e-14, -1.32242728950118e-14, 1.59365405521834e-14, 0, -1.4210854715202e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, -1.214689797411e-14, 0, 0, 1.24931114721564e-14, 0, 0, 0, 0, 0, 0, 0},
    {1.55884572681198, 2.73861278752583, 1.58113883008418, 2.45967477524978, 0, 0, 0, 9.25820099772551, 5.34522483824847, -1.20948631362952, 0, 0, 1.84632408273169e-14, 0, 0, 0, -1.50085060064379e-14, 1.49723986219342e-14, 0, -1.33226762955019e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, -1.46277209591922e-14, 0, 0, 2.16306109608827e-14, 0, 0, 0, 1.38795388762322e-14, 0, 0, 0},
    {-1.79999999999998, -2.72689985024876e-14, 3.65148371670112, -2.84018778721877, 0, -2.17150305878754e-14, 0, -5.90005603264198e-14, 12.3442679969674, 1.39659449751037, 0, 0, 0, 0, 0, 2.68747109317333e-14, 0, 0, -1.90845800418516e-14, 0, 0, 0, 0, 0, 0, -1.23984917003451e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1.36024520914242e-14, 0, 0, 0, 0, 0, 1.0365125502094e-14, 0, 0, 1.00866023900427e-14, -1.51206012498905e-14, 0},
    {1.27279220613579, 0, 0, 3.83405790253616, 0, 0, 0, 1.00217788352562e-14, 0, 5.18459255872629, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {2.20454076850487, 0, 0, 6.6407830863536, 0, 0, 0, 1.6954568423569e-14, -1.04207941733572e-14, 8.97997772825745, 0, 0, 0, 0, 0, 0, -1.07399115598352e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, -1.06281916397066e-14, 0, 1.0771947239792e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {2.56904651573302, 5.80288457473997, -2.01017818278147, -1.42141062443802, 8.14215222512715, -2.10229466468021, 0.728256234341424, -1.71651640581399, 0.594618725379061, 0.420458932936053, 9.8107084351743, -1.18450885369836, 0.30583887092058, -0.105945692672794, -1.02581475831037, 0.26486423168198, -0.0917516612761728, 0.216260739578395, -0.0749149177264333, -0.0529728463363882, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {2.26568606239551, 3.07059789431495, 1.77281052085584, -1.25356634105602, 2.39356776939086, 6.3037687140313, -2.78313372537745, -0.908295106229251, -0.524404424085067, 0.370809924354793, 11.1242977306435, 9.40174755792013, -2.42752078113808, 0.840917865872078, -0.301561193523236, -0.794200206656967, 0.350641890608821, 0.114434427054281, 0.0660687472643356, -0.0467176592151211, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1.01015959200066e-14, 0, 0, 0, 0, 0, 0, 0, 1.84814770714563e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {1.91485421551268, 1.29756443747034, 3.74574588632198, -1.05945692672795, -8.09173593712686, 5.01426536422406, 1.80936716113936, -0.383824736789375, -1.10800657553489, 0.313391585263996, 0, 12.7134831207354, 8.20651806648289, -2.84282124887607, 1.01946290306867, -0.631738055305808, -0.227958835180056, 0.0483573714561744, 0.139595707137592, -0.039483628456617, 0, 0, 0, 0, 0, 0, 0, 1.2257456641157e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, -1.29080203530852e-14, 1.16595669138782e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {1.48323969741914, 7.03562363973513, 4.062019202318, -0.820651806648271, 0, -6.79705818718658, 5.88642506110458, -2.08116553882674, -1.20156148406981, 0.242752078113807, 0, 0, 14.8323969741913, 5.13809303146607, 1.63714487212955e-14, 0.856348838577624, -0.741619848709526, 0.262202212042521, 0.151382517704875, -0.030583887092048, 0, 0, 0, 0, 0, 0, -1.18952346076914e-14, 1.30853433842222e-14, -1.16925429925565e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, -1.88029658615293e-14, 1.91748164115319e-14, -1.45977163230575e-14, 1.09655150319724e-14, 0, 0, -1.89024735595586e-14, 0, 0, 0, 0, 0, 0, 0},
    {-3.42539535431071, 0, 9.38083151964686, 1.89521416591735, 0, 0, -3.39852909359329, 0, -2.77488738510232, -0.560611910581379, 0, 0, 0, 17.7988763690296, 0, -1.94078776694002e-14, 0.42817441928881, 0, 0.349602949390047, 0.0706304617818554, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, -1.20061774357534e-14, 0, 0, 0, 0, 0, -1.00079856173773e-14, 0, -2.26674794489762e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {2.02649122047609, 2.74642624930236, -0.792824967172097, 2.24244764232558, 2.14087209644418, -0.331662479035527, 0.0957427107756351, 6.49923072370878, -1.87616630392938, -2.15580611373099, 0, 0, 0, 0, 9.44035859331468, -1.46249406456536, 0.422185670932516, -2.04706526287666, 0.590936840285292, 0.52231930877335, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1.66277980866635e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1.68132514445692e-14, 0, 0, -1.30320642762071e-14, 1.5317530203921e-14, 0},
    {1.71269767715538, 1.16057691494799, 1.34011878852097, 1.89521416591739, 2.5331140255951, 1.96214168703485, -0.566421515598877, 2.74642624930236, 3.17129986868837, -1.82198870938948, 0, 0, 1.26698604015865e-14, 0, 11.1699829238239, 8.6522315682783, -2.49768411251824, -0.86504295831355, -0.998865569685873, 0.441440386136653, 0, 0, 0, 0, 0, 0, 1.98384517908788e-14, 0, 0, 1.09458095686894e-14, 0, 1.6523812455151e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, -1.03226253428986e-14, 3.45055971392177e-14, -1.0963026379453e-14, 0, 0, 1.98669435916771e-14, 0, 2.24143595655932e-14, 1.21246267628695e-14, 0, 1.40825657142122e-14, 0, 0},
    {1.32664991614221, -2.69693703815181, 2.07610309995257, 1.46802658041887, 0, 3.03973683071411, 1.31624465810881, -6.38211787508104, 4.91295663090393, -1.4113063856887, 0, 0, 0, 0, -1.12046668488404e-14, 13.4039795085888, 5.80409338312195, 2.01017818278149, -1.54743588659735, 0.341938252770126, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1.99188479486906e-14, 0, 0, 0, 0, 0, 0, 1.9439663850877e-14, 0, 2.04972151503089e-14, -2.09775353099935e-14, 0},
    {3.82970843102538, -1.35375539472083e-14, -1.49829835452878, 4.23782770691181, 0, 0, 3.79967103839267, 1.23996592074953e-14, -3.54562104171168, -4.07409060843206, 0, 0, -1.7600753095073e-14, -1.0822450122319e-14, 1.7784123181026e-14, -1.50271421719475e-14, 16.7549743857359, -2.83790269242822e-14, 1.11676565710081, 0.987090711415297, 0, 0, 0, 0, 0, 1.14492917376685e-14, -1.371929590441e-14, 2.1952750214123e-14, 0, 0, 1.18235696338831e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, -1.24727047960437e-14, 2.54018340570979e-14, -1.39065024647839e-14, 0, 0, 0, -2.41394818807004e-14, 0, 0, 0, 0, 0, 0, 0},
    {1.48323969741916, 1.00508909139075, -0.193429485824665, 3.55615782880926, 0, -1.09316735690911e-14, 0, 4.45964044034304, -0.85825820290701, 2.67027285925186, 0, 0, 0, 0, 0, 0, 0, 8.42792824422444, -1.62195554683794, -2.29379153190449, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {1.14891252930764, 1.29756443747036, 0.749149177264399, 2.75458800949268, 0, -1.56850849701631e-14, 0, 5.75737105184055, 3.32401972660469, 2.06838446274243, 0, 0, 0, 1.55284895654593e-14, 0, 0, -1.28699742148281e-14, 10.8804085776374, 6.28180682119208, -1.77676328054755, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, -1.17679879087453e-14, 0, 0, 0, 0, 0, 0, 0, 0, 1.29340213107974e-14, 0, 0, 0, 0},
    {-1.32664991614215, -2.47030321919216e-14, 1.73008591662716, -3.18072425757422, 0, 0, 1.13119318909818e-14, -3.04199753195453e-14, 7.6764947357874, -2.38836465270395, 0, 0, 0, 2.05269241282193e-14, 0, -1.45662197301565e-14, -1.85676576002613e-14, -2.8126806197496e-14, 14.5072114368499, 2.05162951662072, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, -1.09052206632751e-14, 0, 0, 0, 0, 0, 0, 0, 0, -1.073521706443e-14, 0, 0, 0, 0, 0, 0, 0, 1.02157480437484e-14, 0, 0, 0, 0, 1.06581410364015e-14},
    {0.938083151964688, -1.67075414421403e-14, 0, 3.11415464992886, 0, 0, 0, 0, 0, 5.52707620976693, 0, 0, 0, 0, 0, 0, 0, -1.44579473224948e-14, 0, 5.80288457473998, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {1.6248076809272, -2.71174079765265e-14, 0, 5.39387407630366, 0, 0, 0, -1.76761006483537e-14, 0, 9.57317681262153, 0, 0, 0, 0, 0, 1.62541936715167e-14, 1.29794840192237e-14, -2.09070339378817e-14, 0, 10.0508909139074, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1.07896695788578e-14, 1.01165706770544e-14, 0, 1.24344978758018e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1.24344978758018e-14},
    {0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {2.41593350361252, 5.72988941019469, -2.20543546226845, -1.55947837083931, 8.33749895885385, -2.7677995385711, 1.06532653910463, -2.25989885993664, 0.869835476750505, 0.615066564126907, 10.333118277332, -2.18326971917503, 0.724780051083774, -0.278967971730974, -1.8907670401189, 0.627677936394735, -0.241593350361259, 0.512496888990078, -0.197260144544833, -0.139483985865487, 11.8490505948789, -1.16109782023229, 0.245326690731328, -0.0814411017928031, 0.0313466946954498, -1.03851746182649, 0.219426862868121, -0.0728431359084709, 0.0280373360835783, 0.190029237516519, -0.063084006188059, 0.024281045302823, -0.0515078753637739, 0.019825390471097, 0.0140186680417963, 0, 0, 1.00108896213852e-14, 0, 0, 0, 0, 0, 0, 0, 0, 1.21150877964365e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {2.18529407725402, 3.45525332069937, 0.997445717412081, -1.41060126129511, 3.23209352764251, 5.28531208250976, -2.89087233497829, -1.36277028773851, -0.393397896234712, 0.556348640264183, 2.3366642891096, 9.02784807228871, -3.93352933905729, 1.64018416088948, -0.73296991613515, -1.19859611757306, 0.655588223176202, 0.309047252182647, 0.0892142571199726, -0.12616801237611, 13.0996183150502, 11.55277744383, -2.44096975264614, 0.810328731479527, -0.311895674167859, -0.234843597212102, -0.907332870306517, 0.395334573318419, -0.164844710568308, 0.0736662483163133, 0.120463442338377, -0.0658890955530689, -0.031060417514551, -0.00896637020659282, 0.012680362351413, 0, 2.00524187044473e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1.35703519068716e-14, 0, 0, 0, 0, 0, 0, 0, -1.01417984314538e-14, 0},
    {1.92724822331887, 1.82834820066011, 2.99085908983326, -1.24403337882031, 0.950146187582608, 5.74064456311309, -0.169967317119755, -0.721110255092798, -1.1796107330308, 0.490653381462651, -10.3037210107158, 4.97613351528121, 5.78174466995658, -3.22682211353229, -0.215472901842825, -1.30185581821821, 0.0385449644663842, 0.163532434528608, 0.267510569439612, -0.111269728052846, 0, 14.555131461167, 10.7636648722209, -3.57321383950879, 1.37533064807499, 1.03556292536504, -0.500120235663637, -0.581087203147965, 0.324307824721464, 0.0216558414515625, 0.13084143273245, -0.00387391468765269, -0.0164356280722631, -0.0268858238268406, 0.0111830284386733, 0, 0, 0, 0, 0, 0, 0, -1.24894083304515e-14, 0, 0, 0, 0, 0, -1.79685973736065e-14, 0, 0, 0, 0, 0, 0, 0},
    {1.62882203585591, 0.77261813045657, 3.86595114186567, -1.05140010313427, 9.1544290607646, 3.35889012281481, 3.87851223317102, -0.304724700110015, -1.52475169285617, 0.414677792940093, -1.50838999194959e-14, -9.46260092820841, 7.3296991613516, 0.564240504518021, -2.07602937339133, -0.761724681099385, -0.879563899362177, 0.0691050664139764, 0.345781182036188, -0.0940400840863524, 0, 0, 16.3745228938128, 9.05975063854702, -3.48709964663735, 0, 0.951027176355628, -0.736662483163206, -0.0567083043941531, 0.208648802584714, 0.076556210932118, 0.0883994979795754, -0.00694532049720325, -0.0347523163751444, 0.00945138406569365, 0, -1.34204874150947e-14, 0, 0, 0, 0, 0, 0, 0, -2.42374492692268e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {1.26168012376109, -6.78263087840205, 3.68560354207928, -0.814411017927976, 0, 10.0216907612581, 6.00856531485322, 2.67510569439611, -1.4536216402544, 0.321208037219824, 0, 0, -7.57008074256671, 6.99294104721291, 0, -2.27270584034422, -1.36261453366206, -0.606656948415834, 0.329650402328115, -0.0728431359084831, 0, 0, 0, 18.7137404500718, 5.40219154319688, 0, 0, 0.760821741084502, -0.702817019232737, 0, 0.228415531250642, 0.136947913395211, 0.0609713174047674, -0.033131112015512, 0.00732101061700773, 0, 0, 0, 0, 0, -1.30324950839724e-14, 0, 0, 0, -1.36428396125789e-14, -2.70392224640622e-14, 0, 0, 0, -1.78921270332478e-14, 0, 0, 0, 0, 0, 0},
    {5.09901951359278, 1.47405589080242e-14, -4.65474668125635, -3.29140294302192, 0, -1.75121952403549e-14, 13.490737563232, -1.88659898491929e-14, 1.83585684909538, 1.29814682728313, 1.74929961328223e-14, 0, 3.18637490243895e-14, -3.53270434653118, 0, 2.70955022774169e-14, -3.05941170815572, 0, -0.416333199893208, -0.294392028877594, 0, 0, -1.00417365518154e-14, -2.77992935601067e-14, 21.8326971917504, 0, 0, 0, 0.355050145839428, 0, 0, 0.307482445914326, 0, 0.0418430609640593, 0.0295875121532926, 0, 0, 1.53618763757974e-14, -1.4536552951721e-14, 0, -2.34517773777192e-14, 0, 0, 0, 1.75720087957737e-14, -2.91662766570223e-14, 0, 1.76059530106093e-14, 0, 0, 0, 0, 1.07470318048245e-14, 0, 0, 0},
    {1.99489143482416, 3.15420030940281, -1.09264703862702, 1.54523626091316, 2.95048422176041, -0.761811750275054, 0.263899331455877, 5.59815020469135, -1.93925611658553, -2.28543525082516, 2.13307290077015, -0.257539376818855, 0.0664963811607952, -0.0230350221380054, 9.14446135479766, -2.36108976914457, 0.817905488277913, -3.33842298506509, 1.1564636454577, 1.02505848514083, 0, 0, 0, 0, 0, 11.5766222110848, -1.39771878815471, 0.3608894392803, -0.125015768949695, -2.15192884958005, 0.555625639776457, -0.192474367616166, 0.595437196138648, -0.206265495285699, -0.166687691932932, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1.62916528370584e-14, 1.08185730701625e-14, 0, 0, 0, -1.29677564833166e-14, 0, 0, 0, 0, 0, 0},
    {1.75932887637252, 1.66904592079257, 0.963624111659422, 1.36277028773852, 0.867360833110887, 2.28430636201791, -1.00852844760554, 2.96226264871973, 1.71026313764871, -2.01556443707464, 2.41867732448957, 2.04415543160773, -0.527798662911753, 0.182834820065996, 2.68821895760395, 7.07977053255619, -3.12574097035512, -1.76652739792481, -1.01990506872276, 0.904016610326408, 0, 0, 0, 0, 0, 13.1266557397187, 11.0940489487798, -2.8644711213756, 0.992281903807275, -0.632607619454911, -1.66605356688629, 0.735567892896454, 0.315075748468089, 0.18190906819317, -0.147004726489971, 0, 0, 0, 0, 0, 0, 0, 0, 0, 3.50533057667082e-14, 0, 0, 0, 0, 0, 1.4911169345421e-14, 0, 0, 0, -1.04402756430621e-14, 0},
    {1.48690428533297, 0.70530063064754, 2.03602754481989, 1.1517511068998, -2.93221479395414, 1.81702705031798, 0.655663160391202, 1.2517844405944, 3.61359041872289, -1.70346285967313, 0, 2.76420265655949, 1.78428514239953, -0.618094504365263, -9.08783875864323, 5.63152770643898, 2.03210252328839, -0.746494073225332, -2.15494277062554, 0.764033484562606, 0, 0, 0, 0, 0, 0, 15.0018922739643, 9.68367982311024, -3.35452509157131, 2.13860408462396, -1.32524448062077, -0.478206411017787, 0.133143804689794, 0.384353057392899, -0.124241670058204, 0, 0, 0, 0, 0, 0, 0, -1.2184958691979e-14, 4.08565188434445e-14, 2.21302457056835e-14, 0, 0, 0, -1.0903058801995e-14, 0, 0, 0, 0, 0, 0, 0},
    {1.15175110689978, 3.82426463519456, 2.20794021658196, 0.892142571199804, 0, -2.46306042692148, 2.13307290077015, 6.78739640804928, 3.91870514328392, -1.31949665727937, 0, 0, 3.22490309931938, 1.11713920350151, 2.5966242421383e-14, -7.6337845572604, 6.61105135360486, -4.04762275910199, -2.33689608954557, 0.591817792331191, 0, 0, 0, 0, 0, 0, -2.33944627292341e-14, 17.5022076529583, 6.06294257990891, 0, 1.79642743108414, -1.55575179137408, 0.721929233499939, 0.416806037297049, -0.0962371838080711, 0, 0, 0, 0, 0, 0, 0, -1.06500367370775e-14, 2.55596393426003e-14, -1.78278849535737e-14, 1.24660473149391e-14, 0, 0, -3.33471800635493e-14, 0, 0, 0, 0, -2.74739380889122e-14, 0, 0},
    {-2.65985524643224, 3.04548590059106e-14, 5.09901951359276, -2.06031501455087, 0, 1.27314974860562e-14, -1.23153021346075, 4.54600263645348e-14, 9.04986187739903, 3.04724700110024, 0, 0, 1.37797484823791e-14, 3.8698837191833, 0, -2.28548550248364e-14, -3.81689227863023, 1.32945293178207e-14, -5.39683034546927, -1.36674464685441, 0, 0, 0, 1.36272283216478e-14, 1.00072372011083e-14, 0, 0, -1.24367890909937e-14, 21.00264918355, 0, 0, 0.898213715542077, 0, 0.962572311333257, 0.22225025591058, 0, 0, 0, 0, 0, 0, 0, 1.43237887055327e-14, -2.25958210828995e-14, -2.54351954443607e-14, -4.42292868735092e-14, 0, 0, 1.18670146344888e-14, 0, 0, 0, 0, 0, -2.23325753829243e-14, 0},
    {1.57359158493888, 1.49284005458436, -0.43094580368567, 3.04724700110021, 0.775791113542718, -0.120185042515478, 0.0346944333244453, 5.29905651979672, -1.52970585407783, 0.901387818866015, 0, 0, 1.54345206564601e-14, 0, 4.51561078268936, -0.69955541437241, 0.201944253400477, 6.36462844462229, -1.83730997289732, -2.74825341360596, 0, 0, 0, 0, 0, 0, 0, 0, 0, 10.8920630294282, -1.68739114875268, 0.487107866980291, -3.0999354831996, 0.894874292847886, 0.903959543974661, 0, 0, 0, 0, 0, 0, 0, 0, -1.35315801444742e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {1.32992762321608, 0.630840061880569, 0.728431359084703, 2.57539376818858, 0.917928424547677, 0.711024300256696, -0.205255035576777, 2.23926008187653, 2.58567482211402, 0.761811750275079, 0, 1.50972141321262e-14, 1.92432056571699e-14, 2.94270416460811e-14, 5.34294273196304, 4.13862564412658, -1.1947183148558, 2.68954640486935, 3.10562068169861, -2.32269806558124, 0, 0, 0, 0, 0, 0, 0, 0, 0, 12.8876627769294, 9.98274066131697, -2.88176900403077, -1.30996183150502, -1.51261363209512, 0.763985254692642, 0, 0, 0, 0, 0, 1.07852658462479e-14, 0, -1.13464763436474e-14, -1.56492117088446e-14, -1.2428153133211e-14, 0, 0, 0, 1.57186842569255e-14, 1.23293837842049e-14, 0, 0, 0, 1.39814305552841e-14, 0, 1.99840144432528e-14},
    {1.03015750727542, -1.46593983227033, 1.12848100903611, 1.99489143482414, 0, 1.10151410945721, 0.476969600708479, -5.20357020296097, 4.00571020990215, 0.590096844352093, 0, 1.61268443608728e-14, 0, 2.17744761210991e-14, 0, 6.41153127835569, 2.77627448210725, -6.24994106094659, 4.81120687194241, -1.79915418525286, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 15.4651953323153, 6.69662601613678, 3.04407621455179, -2.3433309625403, 0.59178043363452, 0, 0, 0, 1.05482318770498e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {2.9738085706659, -2.31628665433557e-14, -0.814411017927956, 5.75875553449895, 0, 0, 1.37689263682152, -1.87511781470446e-14, -2.89087233497829, 1.70346285967313, 0, 1.53767244373726e-14, -2.60164507598573e-14, -3.0606105648312e-14, 0, 3.91555026057662e-14, 8.01441409794458, 0, -3.47218947830366, -5.19371076584692, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 19.3314941653941, 0, 1.69115345252876, 1.70832296330022, 0, 0, 0, 0, 0, -1.47710207017114e-14, 0, 0, 0, 2.39374069194048e-14, 0, -1.16737795196999e-14, 0, -2.35947835773664e-14, 0, 0, 0, -2.14656473740582e-14, 0, 0, 0},
    {1.15175110689981, 0.546323519313508, -0.105140010313419, 3.27118942773249, 0, 0, 0, 2.69341127303541, -0.518347241175128, 4.25171145123352, 0, 0, 0, 0, 0, 1.89132326876168e-14, 1.07703482219709e-14, 6.71889089790563, -1.29305115618717, 1.82865048192212, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 9.45383520059452, -1.81939143264587, -2.94058061635795, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0.892142571199805, 0.705300630647555, 0.40720550896397, 2.53385243517954, 0, 1.39813392926345e-14, 0, 3.47717900165116, 2.00755023262382, 3.29336152870135, 0, 0, 0, -1.4230588162391e-14, 0, 2.12522836138748e-14, 1.52618648424287e-14, 8.67405085085769, 5.00796559370718, 1.4164665725037, 0, 0, 0, 0, 0, 0, 0, 0, 1.13147703328239e-14, 0, 0, 0, 12.2048487632307, 7.0464727188699, -2.27776395106694, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1.14999053942145e-14, 0, 0},
    {-1.03015750727539, 0, 0.940400840863377, -2.92584077107543, 0, 0, 0, 1.11293406173924e-14, 4.63623866886823, -3.80284633026899, 0, 0, 0, -2.94156668790987e-14, 0, 0, 0, -1.33922800433549e-14, 11.5654011344769, -1.63559471386628, 0, 0, 0, 0, 0, 0, 0, 0, 1.04604926852949e-14, 0, 0, -1.05559372926886e-14, 0, 16.2731316843076, 2.63013526059784, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, -1.36105120406189e-14, 0, 0, -1.64162100411904e-14, 0, 0, -1.43624641143248e-14, 0, 0, -1.06581410364015e-14},
    {0.728431359084702, 0, 0, 2.53908227033122, 0, 0, 0, -1.12654293021334e-14, 0, 5.00713776237769, 0, 0, 0, 0, 0, 0, 1.75509438887162e-14, 0, 0, 6.9392406806862, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 6.27677936394721, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1.55431223447522e-14},
    {1.26168012376115, 0, 0, 4.39781949681102, 0, 0, 0, -1.94061175819238e-14, 0, 8.6726170049349, 0, 0, 0, 0, 0, 1.01179304274323e-14, 2.87309305235289e-14, 0, -1.04523229052332e-14, 12.0191174248973, 0, 0, 0, 0, 0, 0, 0, 0, 1.49130416320287e-14, 0, 0, 0, 0, 0, 10.8717007662564, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1.39042309332655e-14, 1.68148173406132e-14, 0, 0, 0, -1.39259589347659e-14, 0, 0, 0, 0, 0, 2.57571741713036e-14},
    {0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0}};
    
    const static double dmats2[56][56] = \
    {{0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {3.16227766016836, 0, 0, 0, 0, 1.11583338479159e-14, 0, 1.02922993704865e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {1.82574185835054, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {5.16397779494319, 0, 0, 0, 0, 1.73479189776187e-14, 0, 1.65773933526039e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {2.95803989154978, 5.6124860801609, -1.08012344973464, -0.763762615825969, 0, 2.58464835607474e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, -1.45290989871259e-14, 0, 0, 1.07366814431334e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {2.29128784747791, 1.44913767461894, 4.18330013267037, -0.591607978309966, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1.02908502069211e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {1.32287565553229, 0, 3.86436713231718, -0.341565025531991, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {1.87082869338696, 7.09929573971954, 0, 4.34741302385683, 0, 2.31412979990705e-14, 0, 1.79578574233119e-14, -1.09651589812864e-14, 0, 0, 0, 1.0275456662804e-14, 1.11253951863901e-14, 0, 0, -1.23516279655211e-14, 0, 1.67772945370309e-14, -1.06581410364015e-14, 0, 0, 0, 0, 0, 0, 0, 0, -1.04810894589995e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, -1.45492836924348e-14, 0, 0, 0, 0, 0, 0, 0, -1.07986153011749e-14, 0, 0},
    {1.08012344973466, 0, 7.09929573971953, 2.50998007960221, 0, 0, 0, 0, 0, 0, 0, 1.10981825134126e-14, 0, 0, 0, 1.49126177831961e-14, 0, 0, -1.09527631538099e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1.11294528477952e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {-3.81881307912981, 2.63805095952449e-14, 0, 8.87411967464941, 0, -1.08587953265362e-14, 0, 1.60284200435428e-14, 0, -2.30926389122033e-14, 0, 0, 2.0594465710441e-14, 1.14859138885232e-14, 1.07233040380981e-14, 0, -1.40594294599945e-14, 0, 0, -2.1316282072803e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, -1.15463194561016e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {2.74954541697351, 5.79655069847576, -1.67332005306816, -1.18321595661991, 7.74596669241483, -1.19999999999999, 0.346410161513776, -0.979795897113272, 0.282842712474611, 0.200000000000011, 0, -1.42248293523131e-14, -1.16991571628977e-14, 0, 0, -1.34772504614504e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, -1.05991879093504e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {2.32379000772445, 2.44948974278317, 2.82842712474618, -1.00000000000001, 1.30930734141596, 7.09929573971955, -2.04939015319193, -0.414039335605422, -0.478091443733756, 0.169030850945715, 0, 0, -1.38137027369175e-14, 0, -1.23135650746216e-14, 1.4971941277217e-14, -1.68198040008164e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, -1.41869149998573e-14, 1.99142275743244e-14, -1.52384110702395e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {1.80000000000002, 0.632455532033669, 4.38178046004131, -0.774596669241481, 0, 3.14233761939829, 4.76235235991625, -0.106904496764978, -0.740656079818043, 0.130930734141599, 0, 0, 0, -1.70268155420246e-14, 0, 0, 0, 0, -1.65329170450561e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {1.03923048454135, 0, 3.16227766016836, -0.447213595499958, 0, 0, 5.89188303637179, 0, -0.53452248382485, 0.0755928946018462, 0, 0, 0, -1.1578427571356e-14, 0, 0, 0, 0, -1.31656139489963e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {2.01246117974984, 2.12132034355964, -0.40824829046387, 3.17542648054295, 9.07114735222145, 0, 0, 7.17137165600638, -1.38013111868473, -1.56144011671764, -1.28131039656276e-14, -1.05722626181333e-14, 0, 0, 0, -1.43560570277939e-14, -2.27050760178409e-14, 0, 0, 0, 0, 0, 0, 0, 0, 1.20645452546378e-14, 0, 0, -1.39975315007857e-14, 0, 1.33944298105448e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, -1.79370968861081e-14, 0, 0, 0, 0, 0, 0, 0, 0, 1.10225378253926e-14, 0},
    {1.558845726812, 0.547722557505167, 1.58113883008418, 2.45967477524977, 0, 9.07114735222146, -1.85760141295822e-14, 1.85164019954509, 5.34522483824848, -1.20948631362952, 0, -1.79269712993067e-14, 0, 0, -1.21834179081657e-14, 0, -1.08231757343816e-14, 1.58605770416344e-14, 0, -1.77635683940025e-14, 0, 0, 0, 0, 0, 0, 1.20070279998216e-14, 0, 0, 0, 0, 0, 0, 0, -1.11022302462516e-14, 0, 0, 0, 0, 0, 0, 0, 0, -1.5994754107584e-14, 1.5836245903059e-14, -1.59336158038543e-14, 0, 0, 1.24090681928578e-14, 0, 0, 0, 0, 0, 0, 0},
    {0.900000000000037, 0, 1.46059348668045, 1.42009389360939, 0, -1.21399527118824e-14, 9.07114735222146, 0, 4.93770719878694, -0.698297248755176, 0, -1.69782446707204e-14, 0, 0, 1.38222655524042e-14, -1.41725217571765e-14, 0, 0, -1.92181403448241e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1.52483803956785e-14, -1.04387107607342e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {1.27279220613579, -6.2609903369994, 0, 3.83405790253616, 0, -2.66966858276427e-14, 0, 10.5830052442584, -1.29525898283173e-14, 5.18459255872629, 0, 1.15955623710069e-14, 2.12136539532159e-14, 0, 0, 0, -1.1187497199729e-14, 0, 0, -2.1316282072803e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1.1791882435372e-14, 1.34567031689284e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1.11941676856894e-14, 0, 0, 1.28246579327207e-14, 0, 0, 0, 0},
    {0.734846922834948, 0, -6.26099033699939, 2.21359436211787, 0, 0, 0, 0, 10.5830052442584, 2.99332590941916, 0, -1.26338784464357e-14, 1.51937191134705e-14, 2.01241989119956e-14, 0, 0, -1.07773281838078e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {5.71576766497725, -3.5255606739263e-14, -2.0846313271379e-14, -4.69574275274955, 0, 0, 1.14621342066822e-14, -1.32075361813795e-14, 0, 12.6996062931101, 0, 1.1579866837032e-14, -1.17571210377054e-14, 0, 0, 1.11781775592062e-14, 0, -2.03916735577721e-14, 0, 1.24344978758018e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1.77352530249445e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1.34841270690221e-14, 0, 0, 0, 0, 0, 1.33226762955019e-14},
    {2.56904651573302, 5.80288457473997, -2.01017818278147, -1.42141062443802, 8.14215222512715, -2.10229466468021, 0.728256234341424, -1.71651640581399, 0.59461872537906, 0.420458932936052, 9.8107084351743, -1.18450885369835, 0.305838870920579, -0.105945692672794, -1.02581475831037, 0.264864231681979, -0.0917516612761747, 0.216260739578394, -0.0749149177264344, -0.0529728463363901, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {2.26568606239552, 3.07059789431494, 1.77281052085583, -1.25356634105602, 2.39356776939085, 6.30376871403133, -2.78313372537745, -0.908295106229254, -0.524404424085065, 0.370809924354795, 1.23603308118261, 9.40174755792012, -2.42752078113806, 0.84091786587207, -0.301561193523234, -0.794200206656967, 0.350641890608815, 0.114434427054271, 0.0660687472643394, -0.0467176592151202, 0, 0, 0, 0, 1.20766335966975e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, -1.26955258141599e-14, 0, 0, 0, 0, 0, 1.65501530620379e-14, -1.82242579771102e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {1.91485421551271, 1.29756443747034, 3.74574588632197, -1.05945692672795, 0.404586796856347, 5.01426536422407, 1.80936716113936, -0.383824736789378, -1.1080065755349, 0.313391585264007, 0, 2.82521847127454, 8.20651806648288, -2.84282124887606, -0.0509731451534317, -0.631738055305793, -0.227958835180061, 0.0483573714561631, 0.139595707137598, -0.0394836284566152, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {1.48323969741914, 0.335029697130245, 4.06201920231797, -0.820651806648286, 0, 1.69926454679664, 5.88642506110457, -0.0991031208965069, -1.20156148406979, 0.24275207811381, 0, 0, 4.94413232473043, 5.13809303146604, 0, -0.214087209644449, -0.741619848709543, 0.0124858196210661, 0.151382517704864, -0.0305838870920592, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, -1.236469996079e-14, 0, -1.11803154979493e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0.85634883857767, 0, 2.68023757704195, -0.473803541479343, 0, 0, 5.09779364038993, 1.03628894560728e-14, -0.792824967172095, 0.140152977645353, 0, 0, 0, 7.9106117195687, 0, -3.2024651826757e-14, -0.642261628933259, 0, 0.0998865569685812, -0.0176576154454647, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, -1.41874267782629e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {2.02649122047608, 2.74642624930236, -0.79282496717209, 2.24244764232558, 2.14087209644418, -0.331662479035553, 0.0957427107756432, 6.49923072370878, -1.87616630392938, -2.15580611373099, 11.0554159678513, 0, 0, 0, 9.44035859331468, -1.46249406456537, 0.42218567093253, -2.04706526287665, 0.590936840285286, 0.522319308773345, 0, 0, 0, 0, 0, 0, 0, 1.07417485753862e-14, 0, 0, 1.36630425513329e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, -1.23701834881004e-14, 0, 1.02417490692638e-14, 0, 0, 0, 0, 0, 1.80291351901461e-14, 0, 0, -1.01316066243738e-14, 0, 0},
    {1.71269767715536, 1.16057691494801, 1.34011878852098, 1.89521416591738, 0.361873432227874, 1.96214168703485, -0.56642151559888, 2.74642624930238, 3.17129986868838, -1.82198870938951, 0, 11.0554159678513, 1.45760542423516e-14, 1.47166563795188e-14, 1.59571184626056, 8.65223156827826, -2.49768411251825, -0.865042958313534, -0.998865569685866, 0.44144038613664, 0, 0, 0, 0, 1.29892244953406e-14, 0, 1.19356062430186e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 2.55080325568313e-14, -1.99643912911818e-14, 0, 0, 2.52250943240554e-14, 0, 0, 0, 0, 0, 0, 0},
    {1.32664991614219, 0.299659670905743, 2.07610309995258, 1.46802658041888, 0, 0.868496237346862, 1.31624465810882, 0.709124208342315, 4.91295663090392, -1.4113063856887, 0, 1.04153151561804e-14, 11.0554159678513, 0, 0, 3.82970843102535, 5.80409338312197, -0.223353131420174, -1.54743588659735, 0.341938252770125, 0, 0, 0, -1.22971310029744e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, -1.3965710429727e-14, 1.20613126934675e-14, 1.38979281389624e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, -1.63145986065679e-14, 0},
    {0.765941686205059, -1.0369529147852e-14, 1.49829835452879, 0.847565541382363, 0, 0, 1.62843044502544, 0, 3.54562104171167, -0.814818121686403, 0, 0, 0, 11.0554159678513, 0, -2.54103636481183e-14, 7.18070330817254, 0, -1.11676565710084, 0.197418142283058, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, -1.21467689198221e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {1.48323969741914, 1.00508909139075, -0.193429485824672, 3.55615782880927, -8.46157280212813, 0, 1.04737699623147e-14, 4.45964044034302, -0.858258202906999, 2.67027285925188, 0, 0, 1.43424333847387e-14, 0, 12.4373429638327, 2.02429672213922e-14, 1.33492152068286e-14, 8.42792824422444, -1.62195554683795, -2.29379153190448, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1.03917274103107e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, -1.08180915476963e-14, 0, 0, 0, 1.39785916269036e-14, 0, 0, 0, 0, 0, 1.03080706885695e-14, 0, 0, 0, 0},
    {1.14891252930762, 0.259512887494088, 0.749149177264413, 2.75458800949269, 0, -8.46157280212813, 1.97736713455776e-14, 1.15147421036812, 3.32401972660468, 2.06838446274242, 0, 0, 1.76595565092679e-14, 3.53657180389623e-14, 0, 12.4373429638327, -2.12998980019174e-14, 2.1760817155275, 6.28180682119209, -1.77676328054753, 0, 0, 0, 0, 0, 0, 0, 0, -2.02381918388612e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1.29298824184789e-14, 0, 0, 0, 0, -1.20899537342362e-14, 0, 0, 1.49778288260513e-14, 1.57856876457792e-14, 0, 0, 0, 0, 0, 0},
    {0.663324958071077, -2.03917269773753e-14, 0.692034366650878, 1.59036212878712, 0, 0, -8.46157280212812, -1.91967284198321e-14, 3.07059789431496, 1.194182326352, 0, 1.63277888554017e-14, 0, 2.18186738788674e-14, 0, 2.30908333865784e-14, 12.4373429638327, -2.11081152198351e-14, 5.80288457473997, -1.02581475831036, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1.31364686077023e-14, 0, -1.38856086370832e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1.88129856179495e-14, 0, 0, 0, 0, 0, 0},
    {0.938083151964734, 7.62808987244123, 0, 3.11415464992886, 0, 0, 0, -7.52139804633612, 0, 5.52707620976692, 0, 0, -1.777231263474e-14, 0, -1.62387587846106e-14, 1.69100848543788e-14, 2.98390950438128e-14, 14.2141062443803, 0, 5.80288457473998, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1.39576755950212e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1.84182164821129e-14, 0, 1.05827350607878e-14, 0, 0, 0, 1.04237510536707e-14, 0, 1.26392642221179e-14, 0, 0},
    {0.541602560309088, -1.13099822653614e-14, 7.62808987244125, 1.79795802543456, 0, 0, 0, 0, -7.5213980463361, 3.19105893754052, 0, 1.03213108228181e-14, 0, 0, 0, -1.02355531327919e-14, 0, -3.31702746937809e-14, 14.2141062443803, 3.35029697130246, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, -1.166062958575e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {-5.36159180343545, -1.34228950103456e-14, 1.18705474084944e-14, 8.89943818451483, 0, 0, 0, -2.89831441233684e-14, 0, -5.26497863243526, 0, 0, 0, 0, -1.29782943052458e-14, 0, 2.98133166946067e-14, 0, 0, 16.583123951777, 0, 0, 0, 0, 0, 0, 0, 0, 1.67376776564845e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1.61338757506847e-14, 1.94557000035003e-14, 0, 0, 0, -1.11267253048686e-14, 0, 0, 0, 0, 1.32728374914348e-14, 1.68753899743024e-14},
    {2.41593350361252, 5.72988941019469, -2.20543546226845, -1.55947837083931, 8.33749895885385, -2.7677995385711, 1.06532653910463, -2.25989885993664, 0.869835476750504, 0.615066564126906, 10.333118277332, -2.18326971917503, 0.724780051083775, -0.278967971730974, -1.8907670401189, 0.627677936394735, -0.241593350361257, 0.51249688899008, -0.197260144544834, -0.139483985865487, 11.8490505948789, -1.16109782023229, 0.245326690731328, -0.0814411017928033, 0.0313466946954489, -1.03851746182649, 0.219426862868122, -0.0728431359084709, 0.0280373360835781, 0.190029237516519, -0.063084006188059, 0.0242810453028233, -0.0515078753637743, 0.0198253904710968, 0.014018668041796, 0, 0, 1.04549788312352e-14, 0, 0, 0, 0, 0, 0, 0, 0, 1.07828201668863e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {2.18529407725403, 3.45525332069937, 0.997445717412063, -1.41060126129511, 3.23209352764252, 5.28531208250976, -2.89087233497829, -1.36277028773849, -0.393397896234727, 0.556348640264195, 2.33666428910958, 9.02784807228869, -3.93352933905731, 1.64018416088945, -0.732969916135171, -1.19859611757308, 0.655588223176205, 0.309047252182632, 0.0892142571199684, -0.126168012376111, 1.19087439227729, 11.55277744383, -2.44096975264613, 0.810328731479535, -0.311895674167857, -0.234843597212088, -0.907332870306508, 0.395334573318422, -0.164844710568306, 0.0736662483163242, 0.120463442338383, -0.0658890955530685, -0.0310604175145507, -0.00896637020658553, 0.0126803623514123, 0, 0, 0, 0, 0, 0, 0, -1.66112853067542e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {1.92724822331887, 1.82834820066012, 2.99085908983324, -1.24403337882031, 0.950146187582616, 5.74064456311309, -0.169967317119763, -0.721110255092806, -1.17961073303079, 0.490653381462656, 0.294392028877589, 4.9761335152812, 5.78174466995658, -3.2268221135323, -0.215472901842837, -1.30185581821819, 0.0385449644663795, 0.163532434528623, 0.267510569439614, -0.11126972805285, 0, 2.64638753839399, 10.7636648722209, -3.57321383950879, 1.37533064807499, -0.0295875121532857, -0.500120235663636, -0.581087203147967, 0.324307824721465, 0.0216558414515637, 0.130841432732446, -0.00387391468765088, -0.0164356280722617, -0.0268858238268399, 0.0111830284386723, 0, 0, 0, 0, 0, -1.09166603315028e-14, 0, 0, 0, 1.42437445527895e-14, 0, 0, 0, -1.02438819301777e-14, 0, 0, 0, 0, 0, 0, 0},
    {1.62882203585591, 0.772618130456562, 3.86595114186567, -1.05140010313427, 0.160604018609904, 3.35889012281481, 3.87851223317101, -0.304724700110016, -1.52475169285617, 0.414677792940092, 0, 1.13551211138501, 7.32969916135159, 0.564240504518027, -0.0364215679542207, -0.761724681099395, -0.879563899362167, 0.0691050664139829, 0.345781182036178, -0.0940400840863483, 0, 0, 4.46577897103986, 9.05975063854702, -3.48709964663734, 0, -0.114123261162682, -0.736662483163202, -0.0567083043941542, 0.0036605053085017, 0.0765562109321201, 0.0883994979795774, -0.00694532049720645, -0.0347523163751393, 0.00945138406569343, 0, 0, 0, 0, 0, 0, 0, 0, 1.06468345288818e-14, -1.93401297016719e-14, 1.26134404858539e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {1.26168012376111, 0.199489143482416, 3.68560354207931, -0.814411017927961, 0, 1.02786571910339, 6.00856531485322, -0.0786795792469436, -1.4536216402544, 0.321208037219821, 0, 0, 3.0280322970267, 6.99294104721294, 0, -0.23309803490712, -1.36261453366203, 0.0178428514239939, 0.32965040232813, -0.0728431359084719, 0, 0, 0, 6.80499652729883, 5.40219154319687, 0, 0, -0.304328696433803, -0.702817019232747, 0, 0.0234272339744286, 0.136947913395211, -0.00179327404131641, -0.0331311120155151, 0.00732101061701076, 0, 0, 0, 0, 0, 0, 0, 0, 0, -1.10019056033674e-14, -1.37033343408735e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0.728431359084681, 0, 2.32737334062814, -0.470200420431712, 0, 0, 4.49691252107734, 0, -0.917928424547679, 0.185449546754743, 0, 0, 1.80505771145776e-14, 7.06540869306227, 0, 0, -1.01980390271859, 0, 0.208166599946618, -0.042056004125377, 0, 0, 0, 0, 9.92395326897748, 0, 0, 0, -0.710100291678884, 0, 0, 0.102494148638114, 0, -0.0209215304820281, 0.00422678745047274, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, -2.50163969655578e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {1.99489143482414, 3.15420030940281, -1.09264703862701, 1.54523626091315, 2.9504842217604, -0.761811750275062, 0.263899331455876, 5.59815020469134, -1.93925611658551, -2.28543525082518, 2.13307290077017, -0.257539376818833, 0.0664963811608233, -0.0230350221379902, 9.14446135479768, -2.36108976914455, 0.817905488277909, -3.33842298506508, 1.1564636454577, 1.02505848514083, 13.0453753561106, -1.2103323911658e-14, 0, -1.46406519642548e-14, 0, 11.5766222110848, -1.39771878815472, 0.360889439280293, -0.125015768949695, -2.15192884958006, 0.555625639776451, -0.192474367616164, 0.595437196138645, -0.206265495285706, -0.166687691932928, 0, 0, 1.33850600318309e-14, 0, 0, 0, 2.24412658009199e-14, 0, 0, 0, 1.66496943714949e-14, 1.3113069716231e-14, 0, -1.15323982020285e-14, 0, 0, 0, -1.02677636674246e-14, 0, 0, 0},
    {1.7593288763725, 1.66904592079256, 0.96362411165943, 1.36277028773851, 0.86736083311088, 2.28430636201791, -1.00852844760552, 2.96226264871973, 1.71026313764871, -2.01556443707463, 0.268741924943271, 2.04415543160776, -0.527798662911752, 0.18283482006601, 2.68821895760395, 7.07977053255616, -3.12574097035513, -1.76652739792481, -1.01990506872277, 0.904016610326402, 0, 13.0453753561106, 0, 0, 0, 1.4585173044132, 11.0940489487798, -2.86447112137559, 0.992281903807266, -0.632607619454896, -1.66605356688627, 0.735567892896463, 0.315075748468093, 0.181909068193174, -0.147004726489972, 0, 0, -1.03074285928854e-14, 0, 0, 0, 0, -1.72927567727304e-14, 1.04819480446677e-14, 1.76046313417165e-14, 0, 0, 0, 0, 0, 1.43579679437173e-14, 0, 0, 0, 0, 0},
    {1.48690428533297, 0.705300630647553, 2.03602754481992, 1.1517511068998, 0.14661073969771, 1.81702705031798, 0.65566316039122, 1.2517844405944, 3.61359041872288, -1.70346285967312, 0, 0.614267257013221, 1.78428514239955, -0.618094504365253, 0.45439193793218, 5.631527706439, 2.03210252328837, -0.746494073225325, -2.15494277062552, 0.764033484562605, 0, 0, 13.0453753561106, 0, 0, 0, 3.33375383865873, 9.68367982311024, -3.3545250915713, -0.1069302042312, -1.32524448062077, -0.478206411017785, 0.133143804689794, 0.384353057392897, -0.124241670058195, 0, 0, 0, 0, 0, 0, 0, 0, 1.35190501684936e-14, 2.13313974056602e-14, 0, 0, 0, -1.39901730746776e-14, 0, 0, 0, 0, 0, 0, 0},
    {1.15175110689978, 0.18210783977117, 2.20794021658194, 0.892142571199777, 0, 0.615765106730376, 2.13307290077017, 0.323209352764267, 3.91870514328392, -1.31949665727937, -1.1268544360518e-14, 0, 1.07496769977313, 1.11713920350151, 2.42487345110409e-14, 1.90844613931506, 6.61105135360481, -0.192743940909627, -2.33689608954558, 0.591817792331174, 0, 0, 0, 13.0453753561106, 0, 0, -1.3756522835902e-14, 5.83406921765278, 6.06294257990891, 0, -0.449106857771022, -1.55575179137408, 0.0343775825476135, 0.416806037297048, -0.0962371838080746, 0, 0, 0, 0, 0, 0, 0, 0, 1.42803357596901e-14, 0, 0, 0, 0, -1.21198190345942e-14, 0, 0, 0, 0, 0, -1.3839902652557e-14, 0},
    {0.664963811608031, 1.90472468148085e-14, 1.45686271816934, 0.515078753637701, 0, 0, 1.8472953201911, 0, 2.58567482211402, -0.761811750275038, 0, 1.08037159720826e-14, 2.26487951659612e-14, 1.71994831963699, 0, 1.52639255690414e-14, 5.72533841794521, 0, -1.54195152727693, 0.3416861617136, 0, 0, -1.00168576142402e-14, 0, 13.0453753561106, 0, 0, 0, 9.33451074824442, 0, 0, -1.34732057331308, 0, 0.275020660380927, -0.0555625639776386, 0, 0, 0, 0, 0, -2.03433658742105e-14, 0, 0, 0, 0, -3.84929350411527e-14, 0, 1.299574464293e-14, 0, 0, 0, 0, 0, 0, 0, 0},
    {1.57359158493892, 1.49284005458436, -0.43094580368567, 3.04724700110022, 0.775791113542717, -0.120185042515459, 0.0346944333244422, 5.29905651979673, -1.52970585407784, 0.901387818866001, -10.576283741361, 0, 0, -1.14454236590933e-14, 4.51561078268938, -0.699555414372402, 0.201944253400485, 6.36462844462229, -1.83730997289732, -2.74825341360594, 0, 0, 0, -1.03903584730172e-14, 0, 14.3499128917217, 0, 0, 1.48731679939335e-14, 10.8920630294282, -1.68739114875269, 0.487107866980279, -3.0999354831996, 0.894874292847882, 0.903959543974665, 0, 0, 0, 0, 0, 0, 0, 0, -1.69818485661453e-14, 1.01890667576586e-14, 1.15185313992138e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1.11022302462516e-14},
    {1.3299276232161, 0.630840061880575, 0.728431359084684, 2.57539376818859, 0.131132632078234, 0.711024300256703, -0.205255035576781, 2.23926008187655, 2.585674822114, 0.761811750275056, 0, -10.576283741361, 1.57303341744021e-14, 1.70725378740663e-14, 0.763277533137587, 4.13862564412658, -1.19471831485577, 2.68954640486935, 3.10562068169859, -2.32269806558125, 0, 0, 0, 0, 0, 0, 14.3499128917217, 0, 0, 1.84109468241849, 9.98274066131698, -2.88176900403078, -1.30996183150502, -1.51261363209512, 0.763985254692618, 0, 0, 0, 0, 0, 1.04560213685676e-14, 0, 0, 1.2776556039649e-14, -1.21474404387607e-14, 0, 0, 0, 1.05952301335811e-14, 0, 0, 1.33062123785088e-14, 1.01731376449221e-14, 1.65118907931275e-14, 0, 0},
    {1.03015750727544, 0.162882203585583, 1.12848100903609, 1.99489143482414, 0, 0.314718316987774, 0.476969600708478, 0.578174466995633, 4.00571020990216, 0.590096844352089, 0, 0, -10.5762837413611, 0, 0, 1.83186607953021, 2.77627448210726, 0.694437895660724, 4.8112068719424, -1.79915418525285, 0, 0, 0, 0, 0, 0, 0, 14.3499128917217, 0, 0, 4.41862723780437, 6.69662601613678, -0.338230690505753, -2.3433309625403, 0.591780433634519, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, -1.48270629766312e-14, 0, 0, 0, 0, 0, -1.15317140110751e-14, 0},
    {0.59476171413317, 0, 0.814411017927954, 1.15175110689979, 0, 0, 0.590096844352077, 1.28927856428244e-14, 2.89087233497831, 0.340692571934635, 0, 0, 0, -10.576283741361, 0, 0, 3.4347488991191, -1.66893313716044e-14, 3.47218947830366, -1.03874215316937, 0, 0, 0, 0, 0, 0, 0, 0, 14.3499128917217, 0, 0, 8.2849260708832, 0, -1.69115345252879, 0.341664592660046, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, -1.19245108709972e-14, -1.0049703503871e-14, 0, -1.01285720807823e-14, 1.46934267294508e-14, 0, 0, 0, 0, -1.33537813474028e-14, 1.11022302462516e-14},
    {1.15175110689978, 0.546323519313492, -0.105140010313412, 3.27118942773249, 9.65295620481433, 0, 0, 2.69341127303539, -0.518347241175101, 4.25171145123355, 0, 0, -1.03926949302858e-14, 0, -9.915266007526, 2.32678428089351e-14, -1.46605852864906e-14, 6.71889089790561, -1.29305115618716, 1.82865048192215, 0, 0, 1.09752665623958e-14, 0, 0, 0, 0, 0, 0, 15.9443476574685, -1.29058000479168e-14, 0, 9.45383520059453, -1.81939143264587, -2.94058061635793, 0, 0, 0, 0, 0, 0, 0, 0, -1.22633667590031e-14, 0, 0, 0, 1.71762874424691e-14, 0, 0, 0, 0, 0, 0, 0, 2.15938378289593e-14},
    {0.89214257119978, 0.141060126129492, 0.407205508963968, 2.53385243517952, 0, 9.65295620481434, 0, 0.695435800330236, 2.00755023262381, 3.29336152870136, 0, 1.78864735610198e-14, -1.45456143897546e-14, -1.59407140440659e-14, -1.27256502403865e-14, -9.91526600752597, 1.74986266317689e-14, 1.73481017017152, 5.00796559370718, 1.41646657250367, 0, 0, 0, 0, 0, 0, 0, 0, 1.67839306981372e-14, 0, 15.9443476574685, 0, 2.44096975264614, 7.04647271886991, -2.27776395106694, 0, 0, 0, 0, 0, 0, 0, 0, -1.25723596525151e-14, -1.13140881999668e-14, 1.41799567998805e-14, 0, 1.21347241236521e-14, 0, 0, 0, 0, 0, 0, 0, -1.24344978758018e-14},
    {0.515078753637724, -2.3619485326075e-14, 0.376160336345333, 1.4629203855377, 0, 1.01804649671178e-14, 9.65295620481433, -2.11069547491802e-14, 1.8544954675473, 1.90142316513451, 0, 0, -2.14622752906569e-14, -4.34603909418763e-14, -1.24631731336805e-14, 0, -9.91526600752596, -1.51138454398741e-14, 4.62616045379076, 0.817797356933143, 0, 0, 0, 0, -1.03207533752511e-14, 0, 0, 0, 2.46817376593445e-14, 0, 0, 15.9443476574685, 1.00089905485244e-14, 6.50925267372302, -1.3150676302989, 0, 0, 0, 0, 0, -1.6390734838769e-14, 0, 0, -1.2109391798167e-14, 0, 1.09260746948048e-14, 0, 0, 0, -1.16429829249248e-14, 0, 0, 0, 0, 0, 1.24344978758018e-14},
    {0.728431359084676, -7.94708263760856, 0, 2.53908227033122, 0, 0, -1.27415633137685e-14, 10.2207771580387, 0, 5.0071377623777, 2.28612268350337e-14, -1.8719154526289e-14, 1.31033584296945e-14, -2.53050332009669e-14, 0, 2.83491039062371e-14, 1.64090158394923e-14, -8.49879943502228, 0, 6.93924068068621, 0, -1.52898531740193e-14, 0, 0, 0, -1.42660700700925e-14, 0, 0, 1.84038304706445e-14, 0, -1.43708740249419e-14, 0, 17.9373911146521, 0, 6.27677936394721, 0, 0, 0, 0, 0, 0, 0, 1.74801884942513e-14, 0, 1.54335530678735e-14, 0, 0, 0, 0, -1.82587310754697e-14, 0, 0, 0, -1.41280724493113e-14, 0, 1.68753899743024e-14},
    {0.420560041253715, 0, -7.94708263760862, 1.46593983227031, 0, 0, 0, 0, 10.2207771580387, 2.89087233497828, 0, 0, 0, -4.72854335057725e-14, 0, 2.90947883710161e-14, 2.94685265414151e-14, 0, -8.49879943502229, 4.00637247496575, 0, 0, 0, -1.04474568447982e-14, -1.49316504487054e-14, 0, 0, 0, 2.10839214785391e-14, 0, 0, -1.06464761849486e-14, 0, 17.9373911146521, 3.62390025541881, 0, 0, 0, 0, 0, -1.23540168908514e-14, 0, 0, 0, 2.09495610608972e-14, 1.91267628119894e-14, 0, 0, 0, -2.42410393446135e-14, 0, 0, 0, 0, 0, 0},
    {6.542378855465, 2.78160733626274e-14, 0, -6.91050664139872, 0, 0, -1.56111299636533e-14, 1.45917324702114e-14, 0, 12.2649325896465, 0, -1.97411729296339e-14, 0, 0, 0, 0, 1.23535778391715e-14, 2.43644083996432e-14, -1.70040757261543e-14, -5.66586629001476, 0, 0, 0, 0, 0, 0, 1.69114983860161e-14, 0, 0, 0, 0, 0, 0, 0, 20.4998755596024, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, -1.54091873433635e-14, 0, -2.20095446006121e-14, 0, 0, 1.1817028848966e-14, -1.05566456925328e-14, -1.23638867682084e-14, 2.1316282072803e-14}};
    
    // Compute reference derivatives
    // Declare pointer to array of derivatives on FIAT element
    double *derivatives = new double [num_derivatives];
    
    // Declare coefficients
    double coeff0_0 = 0;
    double coeff0_1 = 0;
    double coeff0_2 = 0;
    double coeff0_3 = 0;
    double coeff0_4 = 0;
    double coeff0_5 = 0;
    double coeff0_6 = 0;
    double coeff0_7 = 0;
    double coeff0_8 = 0;
    double coeff0_9 = 0;
    double coeff0_10 = 0;
    double coeff0_11 = 0;
    double coeff0_12 = 0;
    double coeff0_13 = 0;
    double coeff0_14 = 0;
    double coeff0_15 = 0;
    double coeff0_16 = 0;
    double coeff0_17 = 0;
    double coeff0_18 = 0;
    double coeff0_19 = 0;
    double coeff0_20 = 0;
    double coeff0_21 = 0;
    double coeff0_22 = 0;
    double coeff0_23 = 0;
    double coeff0_24 = 0;
    double coeff0_25 = 0;
    double coeff0_26 = 0;
    double coeff0_27 = 0;
    double coeff0_28 = 0;
    double coeff0_29 = 0;
    double coeff0_30 = 0;
    double coeff0_31 = 0;
    double coeff0_32 = 0;
    double coeff0_33 = 0;
    double coeff0_34 = 0;
    double coeff0_35 = 0;
    double coeff0_36 = 0;
    double coeff0_37 = 0;
    double coeff0_38 = 0;
    double coeff0_39 = 0;
    double coeff0_40 = 0;
    double coeff0_41 = 0;
    double coeff0_42 = 0;
    double coeff0_43 = 0;
    double coeff0_44 = 0;
    double coeff0_45 = 0;
    double coeff0_46 = 0;
    double coeff0_47 = 0;
    double coeff0_48 = 0;
    double coeff0_49 = 0;
    double coeff0_50 = 0;
    double coeff0_51 = 0;
    double coeff0_52 = 0;
    double coeff0_53 = 0;
    double coeff0_54 = 0;
    double coeff0_55 = 0;
    
    // Declare new coefficients
    double new_coeff0_0 = 0;
    double new_coeff0_1 = 0;
    double new_coeff0_2 = 0;
    double new_coeff0_3 = 0;
    double new_coeff0_4 = 0;
    double new_coeff0_5 = 0;
    double new_coeff0_6 = 0;
    double new_coeff0_7 = 0;
    double new_coeff0_8 = 0;
    double new_coeff0_9 = 0;
    double new_coeff0_10 = 0;
    double new_coeff0_11 = 0;
    double new_coeff0_12 = 0;
    double new_coeff0_13 = 0;
    double new_coeff0_14 = 0;
    double new_coeff0_15 = 0;
    double new_coeff0_16 = 0;
    double new_coeff0_17 = 0;
    double new_coeff0_18 = 0;
    double new_coeff0_19 = 0;
    double new_coeff0_20 = 0;
    double new_coeff0_21 = 0;
    double new_coeff0_22 = 0;
    double new_coeff0_23 = 0;
    double new_coeff0_24 = 0;
    double new_coeff0_25 = 0;
    double new_coeff0_26 = 0;
    double new_coeff0_27 = 0;
    double new_coeff0_28 = 0;
    double new_coeff0_29 = 0;
    double new_coeff0_30 = 0;
    double new_coeff0_31 = 0;
    double new_coeff0_32 = 0;
    double new_coeff0_33 = 0;
    double new_coeff0_34 = 0;
    double new_coeff0_35 = 0;
    double new_coeff0_36 = 0;
    double new_coeff0_37 = 0;
    double new_coeff0_38 = 0;
    double new_coeff0_39 = 0;
    double new_coeff0_40 = 0;
    double new_coeff0_41 = 0;
    double new_coeff0_42 = 0;
    double new_coeff0_43 = 0;
    double new_coeff0_44 = 0;
    double new_coeff0_45 = 0;
    double new_coeff0_46 = 0;
    double new_coeff0_47 = 0;
    double new_coeff0_48 = 0;
    double new_coeff0_49 = 0;
    double new_coeff0_50 = 0;
    double new_coeff0_51 = 0;
    double new_coeff0_52 = 0;
    double new_coeff0_53 = 0;
    double new_coeff0_54 = 0;
    double new_coeff0_55 = 0;
    
    // Loop possible derivatives
    for (unsigned int deriv_num = 0; deriv_num < num_derivatives; deriv_num++)
    {
      // Get values from coefficients array
      new_coeff0_0 = coefficients0[dof][0];
      new_coeff0_1 = coefficients0[dof][1];
      new_coeff0_2 = coefficients0[dof][2];
      new_coeff0_3 = coefficients0[dof][3];
      new_coeff0_4 = coefficients0[dof][4];
      new_coeff0_5 = coefficients0[dof][5];
      new_coeff0_6 = coefficients0[dof][6];
      new_coeff0_7 = coefficients0[dof][7];
      new_coeff0_8 = coefficients0[dof][8];
      new_coeff0_9 = coefficients0[dof][9];
      new_coeff0_10 = coefficients0[dof][10];
      new_coeff0_11 = coefficients0[dof][11];
      new_coeff0_12 = coefficients0[dof][12];
      new_coeff0_13 = coefficients0[dof][13];
      new_coeff0_14 = coefficients0[dof][14];
      new_coeff0_15 = coefficients0[dof][15];
      new_coeff0_16 = coefficients0[dof][16];
      new_coeff0_17 = coefficients0[dof][17];
      new_coeff0_18 = coefficients0[dof][18];
      new_coeff0_19 = coefficients0[dof][19];
      new_coeff0_20 = coefficients0[dof][20];
      new_coeff0_21 = coefficients0[dof][21];
      new_coeff0_22 = coefficients0[dof][22];
      new_coeff0_23 = coefficients0[dof][23];
      new_coeff0_24 = coefficients0[dof][24];
      new_coeff0_25 = coefficients0[dof][25];
      new_coeff0_26 = coefficients0[dof][26];
      new_coeff0_27 = coefficients0[dof][27];
      new_coeff0_28 = coefficients0[dof][28];
      new_coeff0_29 = coefficients0[dof][29];
      new_coeff0_30 = coefficients0[dof][30];
      new_coeff0_31 = coefficients0[dof][31];
      new_coeff0_32 = coefficients0[dof][32];
      new_coeff0_33 = coefficients0[dof][33];
      new_coeff0_34 = coefficients0[dof][34];
      new_coeff0_35 = coefficients0[dof][35];
      new_coeff0_36 = coefficients0[dof][36];
      new_coeff0_37 = coefficients0[dof][37];
      new_coeff0_38 = coefficients0[dof][38];
      new_coeff0_39 = coefficients0[dof][39];
      new_coeff0_40 = coefficients0[dof][40];
      new_coeff0_41 = coefficients0[dof][41];
      new_coeff0_42 = coefficients0[dof][42];
      new_coeff0_43 = coefficients0[dof][43];
      new_coeff0_44 = coefficients0[dof][44];
      new_coeff0_45 = coefficients0[dof][45];
      new_coeff0_46 = coefficients0[dof][46];
      new_coeff0_47 = coefficients0[dof][47];
      new_coeff0_48 = coefficients0[dof][48];
      new_coeff0_49 = coefficients0[dof][49];
      new_coeff0_50 = coefficients0[dof][50];
      new_coeff0_51 = coefficients0[dof][51];
      new_coeff0_52 = coefficients0[dof][52];
      new_coeff0_53 = coefficients0[dof][53];
      new_coeff0_54 = coefficients0[dof][54];
      new_coeff0_55 = coefficients0[dof][55];
    
      // Loop derivative order
      for (unsigned int j = 0; j < n; j++)
      {
        // Update old coefficients
        coeff0_0 = new_coeff0_0;
        coeff0_1 = new_coeff0_1;
        coeff0_2 = new_coeff0_2;
        coeff0_3 = new_coeff0_3;
        coeff0_4 = new_coeff0_4;
        coeff0_5 = new_coeff0_5;
        coeff0_6 = new_coeff0_6;
        coeff0_7 = new_coeff0_7;
        coeff0_8 = new_coeff0_8;
        coeff0_9 = new_coeff0_9;
        coeff0_10 = new_coeff0_10;
        coeff0_11 = new_coeff0_11;
        coeff0_12 = new_coeff0_12;
        coeff0_13 = new_coeff0_13;
        coeff0_14 = new_coeff0_14;
        coeff0_15 = new_coeff0_15;
        coeff0_16 = new_coeff0_16;
        coeff0_17 = new_coeff0_17;
        coeff0_18 = new_coeff0_18;
        coeff0_19 = new_coeff0_19;
        coeff0_20 = new_coeff0_20;
        coeff0_21 = new_coeff0_21;
        coeff0_22 = new_coeff0_22;
        coeff0_23 = new_coeff0_23;
        coeff0_24 = new_coeff0_24;
        coeff0_25 = new_coeff0_25;
        coeff0_26 = new_coeff0_26;
        coeff0_27 = new_coeff0_27;
        coeff0_28 = new_coeff0_28;
        coeff0_29 = new_coeff0_29;
        coeff0_30 = new_coeff0_30;
        coeff0_31 = new_coeff0_31;
        coeff0_32 = new_coeff0_32;
        coeff0_33 = new_coeff0_33;
        coeff0_34 = new_coeff0_34;
        coeff0_35 = new_coeff0_35;
        coeff0_36 = new_coeff0_36;
        coeff0_37 = new_coeff0_37;
        coeff0_38 = new_coeff0_38;
        coeff0_39 = new_coeff0_39;
        coeff0_40 = new_coeff0_40;
        coeff0_41 = new_coeff0_41;
        coeff0_42 = new_coeff0_42;
        coeff0_43 = new_coeff0_43;
        coeff0_44 = new_coeff0_44;
        coeff0_45 = new_coeff0_45;
        coeff0_46 = new_coeff0_46;
        coeff0_47 = new_coeff0_47;
        coeff0_48 = new_coeff0_48;
        coeff0_49 = new_coeff0_49;
        coeff0_50 = new_coeff0_50;
        coeff0_51 = new_coeff0_51;
        coeff0_52 = new_coeff0_52;
        coeff0_53 = new_coeff0_53;
        coeff0_54 = new_coeff0_54;
        coeff0_55 = new_coeff0_55;
    
        if(combinations[deriv_num][j] == 0)
        {
          new_coeff0_0 = coeff0_0*dmats0[0][0] + coeff0_1*dmats0[1][0] + coeff0_2*dmats0[2][0] + coeff0_3*dmats0[3][0] + coeff0_4*dmats0[4][0] + coeff0_5*dmats0[5][0] + coeff0_6*dmats0[6][0] + coeff0_7*dmats0[7][0] + coeff0_8*dmats0[8][0] + coeff0_9*dmats0[9][0] + coeff0_10*dmats0[10][0] + coeff0_11*dmats0[11][0] + coeff0_12*dmats0[12][0] + coeff0_13*dmats0[13][0] + coeff0_14*dmats0[14][0] + coeff0_15*dmats0[15][0] + coeff0_16*dmats0[16][0] + coeff0_17*dmats0[17][0] + coeff0_18*dmats0[18][0] + coeff0_19*dmats0[19][0] + coeff0_20*dmats0[20][0] + coeff0_21*dmats0[21][0] + coeff0_22*dmats0[22][0] + coeff0_23*dmats0[23][0] + coeff0_24*dmats0[24][0] + coeff0_25*dmats0[25][0] + coeff0_26*dmats0[26][0] + coeff0_27*dmats0[27][0] + coeff0_28*dmats0[28][0] + coeff0_29*dmats0[29][0] + coeff0_30*dmats0[30][0] + coeff0_31*dmats0[31][0] + coeff0_32*dmats0[32][0] + coeff0_33*dmats0[33][0] + coeff0_34*dmats0[34][0] + coeff0_35*dmats0[35][0] + coeff0_36*dmats0[36][0] + coeff0_37*dmats0[37][0] + coeff0_38*dmats0[38][0] + coeff0_39*dmats0[39][0] + coeff0_40*dmats0[40][0] + coeff0_41*dmats0[41][0] + coeff0_42*dmats0[42][0] + coeff0_43*dmats0[43][0] + coeff0_44*dmats0[44][0] + coeff0_45*dmats0[45][0] + coeff0_46*dmats0[46][0] + coeff0_47*dmats0[47][0] + coeff0_48*dmats0[48][0] + coeff0_49*dmats0[49][0] + coeff0_50*dmats0[50][0] + coeff0_51*dmats0[51][0] + coeff0_52*dmats0[52][0] + coeff0_53*dmats0[53][0] + coeff0_54*dmats0[54][0] + coeff0_55*dmats0[55][0];
          new_coeff0_1 = coeff0_0*dmats0[0][1] + coeff0_1*dmats0[1][1] + coeff0_2*dmats0[2][1] + coeff0_3*dmats0[3][1] + coeff0_4*dmats0[4][1] + coeff0_5*dmats0[5][1] + coeff0_6*dmats0[6][1] + coeff0_7*dmats0[7][1] + coeff0_8*dmats0[8][1] + coeff0_9*dmats0[9][1] + coeff0_10*dmats0[10][1] + coeff0_11*dmats0[11][1] + coeff0_12*dmats0[12][1] + coeff0_13*dmats0[13][1] + coeff0_14*dmats0[14][1] + coeff0_15*dmats0[15][1] + coeff0_16*dmats0[16][1] + coeff0_17*dmats0[17][1] + coeff0_18*dmats0[18][1] + coeff0_19*dmats0[19][1] + coeff0_20*dmats0[20][1] + coeff0_21*dmats0[21][1] + coeff0_22*dmats0[22][1] + coeff0_23*dmats0[23][1] + coeff0_24*dmats0[24][1] + coeff0_25*dmats0[25][1] + coeff0_26*dmats0[26][1] + coeff0_27*dmats0[27][1] + coeff0_28*dmats0[28][1] + coeff0_29*dmats0[29][1] + coeff0_30*dmats0[30][1] + coeff0_31*dmats0[31][1] + coeff0_32*dmats0[32][1] + coeff0_33*dmats0[33][1] + coeff0_34*dmats0[34][1] + coeff0_35*dmats0[35][1] + coeff0_36*dmats0[36][1] + coeff0_37*dmats0[37][1] + coeff0_38*dmats0[38][1] + coeff0_39*dmats0[39][1] + coeff0_40*dmats0[40][1] + coeff0_41*dmats0[41][1] + coeff0_42*dmats0[42][1] + coeff0_43*dmats0[43][1] + coeff0_44*dmats0[44][1] + coeff0_45*dmats0[45][1] + coeff0_46*dmats0[46][1] + coeff0_47*dmats0[47][1] + coeff0_48*dmats0[48][1] + coeff0_49*dmats0[49][1] + coeff0_50*dmats0[50][1] + coeff0_51*dmats0[51][1] + coeff0_52*dmats0[52][1] + coeff0_53*dmats0[53][1] + coeff0_54*dmats0[54][1] + coeff0_55*dmats0[55][1];
          new_coeff0_2 = coeff0_0*dmats0[0][2] + coeff0_1*dmats0[1][2] + coeff0_2*dmats0[2][2] + coeff0_3*dmats0[3][2] + coeff0_4*dmats0[4][2] + coeff0_5*dmats0[5][2] + coeff0_6*dmats0[6][2] + coeff0_7*dmats0[7][2] + coeff0_8*dmats0[8][2] + coeff0_9*dmats0[9][2] + coeff0_10*dmats0[10][2] + coeff0_11*dmats0[11][2] + coeff0_12*dmats0[12][2] + coeff0_13*dmats0[13][2] + coeff0_14*dmats0[14][2] + coeff0_15*dmats0[15][2] + coeff0_16*dmats0[16][2] + coeff0_17*dmats0[17][2] + coeff0_18*dmats0[18][2] + coeff0_19*dmats0[19][2] + coeff0_20*dmats0[20][2] + coeff0_21*dmats0[21][2] + coeff0_22*dmats0[22][2] + coeff0_23*dmats0[23][2] + coeff0_24*dmats0[24][2] + coeff0_25*dmats0[25][2] + coeff0_26*dmats0[26][2] + coeff0_27*dmats0[27][2] + coeff0_28*dmats0[28][2] + coeff0_29*dmats0[29][2] + coeff0_30*dmats0[30][2] + coeff0_31*dmats0[31][2] + coeff0_32*dmats0[32][2] + coeff0_33*dmats0[33][2] + coeff0_34*dmats0[34][2] + coeff0_35*dmats0[35][2] + coeff0_36*dmats0[36][2] + coeff0_37*dmats0[37][2] + coeff0_38*dmats0[38][2] + coeff0_39*dmats0[39][2] + coeff0_40*dmats0[40][2] + coeff0_41*dmats0[41][2] + coeff0_42*dmats0[42][2] + coeff0_43*dmats0[43][2] + coeff0_44*dmats0[44][2] + coeff0_45*dmats0[45][2] + coeff0_46*dmats0[46][2] + coeff0_47*dmats0[47][2] + coeff0_48*dmats0[48][2] + coeff0_49*dmats0[49][2] + coeff0_50*dmats0[50][2] + coeff0_51*dmats0[51][2] + coeff0_52*dmats0[52][2] + coeff0_53*dmats0[53][2] + coeff0_54*dmats0[54][2] + coeff0_55*dmats0[55][2];
          new_coeff0_3 = coeff0_0*dmats0[0][3] + coeff0_1*dmats0[1][3] + coeff0_2*dmats0[2][3] + coeff0_3*dmats0[3][3] + coeff0_4*dmats0[4][3] + coeff0_5*dmats0[5][3] + coeff0_6*dmats0[6][3] + coeff0_7*dmats0[7][3] + coeff0_8*dmats0[8][3] + coeff0_9*dmats0[9][3] + coeff0_10*dmats0[10][3] + coeff0_11*dmats0[11][3] + coeff0_12*dmats0[12][3] + coeff0_13*dmats0[13][3] + coeff0_14*dmats0[14][3] + coeff0_15*dmats0[15][3] + coeff0_16*dmats0[16][3] + coeff0_17*dmats0[17][3] + coeff0_18*dmats0[18][3] + coeff0_19*dmats0[19][3] + coeff0_20*dmats0[20][3] + coeff0_21*dmats0[21][3] + coeff0_22*dmats0[22][3] + coeff0_23*dmats0[23][3] + coeff0_24*dmats0[24][3] + coeff0_25*dmats0[25][3] + coeff0_26*dmats0[26][3] + coeff0_27*dmats0[27][3] + coeff0_28*dmats0[28][3] + coeff0_29*dmats0[29][3] + coeff0_30*dmats0[30][3] + coeff0_31*dmats0[31][3] + coeff0_32*dmats0[32][3] + coeff0_33*dmats0[33][3] + coeff0_34*dmats0[34][3] + coeff0_35*dmats0[35][3] + coeff0_36*dmats0[36][3] + coeff0_37*dmats0[37][3] + coeff0_38*dmats0[38][3] + coeff0_39*dmats0[39][3] + coeff0_40*dmats0[40][3] + coeff0_41*dmats0[41][3] + coeff0_42*dmats0[42][3] + coeff0_43*dmats0[43][3] + coeff0_44*dmats0[44][3] + coeff0_45*dmats0[45][3] + coeff0_46*dmats0[46][3] + coeff0_47*dmats0[47][3] + coeff0_48*dmats0[48][3] + coeff0_49*dmats0[49][3] + coeff0_50*dmats0[50][3] + coeff0_51*dmats0[51][3] + coeff0_52*dmats0[52][3] + coeff0_53*dmats0[53][3] + coeff0_54*dmats0[54][3] + coeff0_55*dmats0[55][3];
          new_coeff0_4 = coeff0_0*dmats0[0][4] + coeff0_1*dmats0[1][4] + coeff0_2*dmats0[2][4] + coeff0_3*dmats0[3][4] + coeff0_4*dmats0[4][4] + coeff0_5*dmats0[5][4] + coeff0_6*dmats0[6][4] + coeff0_7*dmats0[7][4] + coeff0_8*dmats0[8][4] + coeff0_9*dmats0[9][4] + coeff0_10*dmats0[10][4] + coeff0_11*dmats0[11][4] + coeff0_12*dmats0[12][4] + coeff0_13*dmats0[13][4] + coeff0_14*dmats0[14][4] + coeff0_15*dmats0[15][4] + coeff0_16*dmats0[16][4] + coeff0_17*dmats0[17][4] + coeff0_18*dmats0[18][4] + coeff0_19*dmats0[19][4] + coeff0_20*dmats0[20][4] + coeff0_21*dmats0[21][4] + coeff0_22*dmats0[22][4] + coeff0_23*dmats0[23][4] + coeff0_24*dmats0[24][4] + coeff0_25*dmats0[25][4] + coeff0_26*dmats0[26][4] + coeff0_27*dmats0[27][4] + coeff0_28*dmats0[28][4] + coeff0_29*dmats0[29][4] + coeff0_30*dmats0[30][4] + coeff0_31*dmats0[31][4] + coeff0_32*dmats0[32][4] + coeff0_33*dmats0[33][4] + coeff0_34*dmats0[34][4] + coeff0_35*dmats0[35][4] + coeff0_36*dmats0[36][4] + coeff0_37*dmats0[37][4] + coeff0_38*dmats0[38][4] + coeff0_39*dmats0[39][4] + coeff0_40*dmats0[40][4] + coeff0_41*dmats0[41][4] + coeff0_42*dmats0[42][4] + coeff0_43*dmats0[43][4] + coeff0_44*dmats0[44][4] + coeff0_45*dmats0[45][4] + coeff0_46*dmats0[46][4] + coeff0_47*dmats0[47][4] + coeff0_48*dmats0[48][4] + coeff0_49*dmats0[49][4] + coeff0_50*dmats0[50][4] + coeff0_51*dmats0[51][4] + coeff0_52*dmats0[52][4] + coeff0_53*dmats0[53][4] + coeff0_54*dmats0[54][4] + coeff0_55*dmats0[55][4];
          new_coeff0_5 = coeff0_0*dmats0[0][5] + coeff0_1*dmats0[1][5] + coeff0_2*dmats0[2][5] + coeff0_3*dmats0[3][5] + coeff0_4*dmats0[4][5] + coeff0_5*dmats0[5][5] + coeff0_6*dmats0[6][5] + coeff0_7*dmats0[7][5] + coeff0_8*dmats0[8][5] + coeff0_9*dmats0[9][5] + coeff0_10*dmats0[10][5] + coeff0_11*dmats0[11][5] + coeff0_12*dmats0[12][5] + coeff0_13*dmats0[13][5] + coeff0_14*dmats0[14][5] + coeff0_15*dmats0[15][5] + coeff0_16*dmats0[16][5] + coeff0_17*dmats0[17][5] + coeff0_18*dmats0[18][5] + coeff0_19*dmats0[19][5] + coeff0_20*dmats0[20][5] + coeff0_21*dmats0[21][5] + coeff0_22*dmats0[22][5] + coeff0_23*dmats0[23][5] + coeff0_24*dmats0[24][5] + coeff0_25*dmats0[25][5] + coeff0_26*dmats0[26][5] + coeff0_27*dmats0[27][5] + coeff0_28*dmats0[28][5] + coeff0_29*dmats0[29][5] + coeff0_30*dmats0[30][5] + coeff0_31*dmats0[31][5] + coeff0_32*dmats0[32][5] + coeff0_33*dmats0[33][5] + coeff0_34*dmats0[34][5] + coeff0_35*dmats0[35][5] + coeff0_36*dmats0[36][5] + coeff0_37*dmats0[37][5] + coeff0_38*dmats0[38][5] + coeff0_39*dmats0[39][5] + coeff0_40*dmats0[40][5] + coeff0_41*dmats0[41][5] + coeff0_42*dmats0[42][5] + coeff0_43*dmats0[43][5] + coeff0_44*dmats0[44][5] + coeff0_45*dmats0[45][5] + coeff0_46*dmats0[46][5] + coeff0_47*dmats0[47][5] + coeff0_48*dmats0[48][5] + coeff0_49*dmats0[49][5] + coeff0_50*dmats0[50][5] + coeff0_51*dmats0[51][5] + coeff0_52*dmats0[52][5] + coeff0_53*dmats0[53][5] + coeff0_54*dmats0[54][5] + coeff0_55*dmats0[55][5];
          new_coeff0_6 = coeff0_0*dmats0[0][6] + coeff0_1*dmats0[1][6] + coeff0_2*dmats0[2][6] + coeff0_3*dmats0[3][6] + coeff0_4*dmats0[4][6] + coeff0_5*dmats0[5][6] + coeff0_6*dmats0[6][6] + coeff0_7*dmats0[7][6] + coeff0_8*dmats0[8][6] + coeff0_9*dmats0[9][6] + coeff0_10*dmats0[10][6] + coeff0_11*dmats0[11][6] + coeff0_12*dmats0[12][6] + coeff0_13*dmats0[13][6] + coeff0_14*dmats0[14][6] + coeff0_15*dmats0[15][6] + coeff0_16*dmats0[16][6] + coeff0_17*dmats0[17][6] + coeff0_18*dmats0[18][6] + coeff0_19*dmats0[19][6] + coeff0_20*dmats0[20][6] + coeff0_21*dmats0[21][6] + coeff0_22*dmats0[22][6] + coeff0_23*dmats0[23][6] + coeff0_24*dmats0[24][6] + coeff0_25*dmats0[25][6] + coeff0_26*dmats0[26][6] + coeff0_27*dmats0[27][6] + coeff0_28*dmats0[28][6] + coeff0_29*dmats0[29][6] + coeff0_30*dmats0[30][6] + coeff0_31*dmats0[31][6] + coeff0_32*dmats0[32][6] + coeff0_33*dmats0[33][6] + coeff0_34*dmats0[34][6] + coeff0_35*dmats0[35][6] + coeff0_36*dmats0[36][6] + coeff0_37*dmats0[37][6] + coeff0_38*dmats0[38][6] + coeff0_39*dmats0[39][6] + coeff0_40*dmats0[40][6] + coeff0_41*dmats0[41][6] + coeff0_42*dmats0[42][6] + coeff0_43*dmats0[43][6] + coeff0_44*dmats0[44][6] + coeff0_45*dmats0[45][6] + coeff0_46*dmats0[46][6] + coeff0_47*dmats0[47][6] + coeff0_48*dmats0[48][6] + coeff0_49*dmats0[49][6] + coeff0_50*dmats0[50][6] + coeff0_51*dmats0[51][6] + coeff0_52*dmats0[52][6] + coeff0_53*dmats0[53][6] + coeff0_54*dmats0[54][6] + coeff0_55*dmats0[55][6];
          new_coeff0_7 = coeff0_0*dmats0[0][7] + coeff0_1*dmats0[1][7] + coeff0_2*dmats0[2][7] + coeff0_3*dmats0[3][7] + coeff0_4*dmats0[4][7] + coeff0_5*dmats0[5][7] + coeff0_6*dmats0[6][7] + coeff0_7*dmats0[7][7] + coeff0_8*dmats0[8][7] + coeff0_9*dmats0[9][7] + coeff0_10*dmats0[10][7] + coeff0_11*dmats0[11][7] + coeff0_12*dmats0[12][7] + coeff0_13*dmats0[13][7] + coeff0_14*dmats0[14][7] + coeff0_15*dmats0[15][7] + coeff0_16*dmats0[16][7] + coeff0_17*dmats0[17][7] + coeff0_18*dmats0[18][7] + coeff0_19*dmats0[19][7] + coeff0_20*dmats0[20][7] + coeff0_21*dmats0[21][7] + coeff0_22*dmats0[22][7] + coeff0_23*dmats0[23][7] + coeff0_24*dmats0[24][7] + coeff0_25*dmats0[25][7] + coeff0_26*dmats0[26][7] + coeff0_27*dmats0[27][7] + coeff0_28*dmats0[28][7] + coeff0_29*dmats0[29][7] + coeff0_30*dmats0[30][7] + coeff0_31*dmats0[31][7] + coeff0_32*dmats0[32][7] + coeff0_33*dmats0[33][7] + coeff0_34*dmats0[34][7] + coeff0_35*dmats0[35][7] + coeff0_36*dmats0[36][7] + coeff0_37*dmats0[37][7] + coeff0_38*dmats0[38][7] + coeff0_39*dmats0[39][7] + coeff0_40*dmats0[40][7] + coeff0_41*dmats0[41][7] + coeff0_42*dmats0[42][7] + coeff0_43*dmats0[43][7] + coeff0_44*dmats0[44][7] + coeff0_45*dmats0[45][7] + coeff0_46*dmats0[46][7] + coeff0_47*dmats0[47][7] + coeff0_48*dmats0[48][7] + coeff0_49*dmats0[49][7] + coeff0_50*dmats0[50][7] + coeff0_51*dmats0[51][7] + coeff0_52*dmats0[52][7] + coeff0_53*dmats0[53][7] + coeff0_54*dmats0[54][7] + coeff0_55*dmats0[55][7];
          new_coeff0_8 = coeff0_0*dmats0[0][8] + coeff0_1*dmats0[1][8] + coeff0_2*dmats0[2][8] + coeff0_3*dmats0[3][8] + coeff0_4*dmats0[4][8] + coeff0_5*dmats0[5][8] + coeff0_6*dmats0[6][8] + coeff0_7*dmats0[7][8] + coeff0_8*dmats0[8][8] + coeff0_9*dmats0[9][8] + coeff0_10*dmats0[10][8] + coeff0_11*dmats0[11][8] + coeff0_12*dmats0[12][8] + coeff0_13*dmats0[13][8] + coeff0_14*dmats0[14][8] + coeff0_15*dmats0[15][8] + coeff0_16*dmats0[16][8] + coeff0_17*dmats0[17][8] + coeff0_18*dmats0[18][8] + coeff0_19*dmats0[19][8] + coeff0_20*dmats0[20][8] + coeff0_21*dmats0[21][8] + coeff0_22*dmats0[22][8] + coeff0_23*dmats0[23][8] + coeff0_24*dmats0[24][8] + coeff0_25*dmats0[25][8] + coeff0_26*dmats0[26][8] + coeff0_27*dmats0[27][8] + coeff0_28*dmats0[28][8] + coeff0_29*dmats0[29][8] + coeff0_30*dmats0[30][8] + coeff0_31*dmats0[31][8] + coeff0_32*dmats0[32][8] + coeff0_33*dmats0[33][8] + coeff0_34*dmats0[34][8] + coeff0_35*dmats0[35][8] + coeff0_36*dmats0[36][8] + coeff0_37*dmats0[37][8] + coeff0_38*dmats0[38][8] + coeff0_39*dmats0[39][8] + coeff0_40*dmats0[40][8] + coeff0_41*dmats0[41][8] + coeff0_42*dmats0[42][8] + coeff0_43*dmats0[43][8] + coeff0_44*dmats0[44][8] + coeff0_45*dmats0[45][8] + coeff0_46*dmats0[46][8] + coeff0_47*dmats0[47][8] + coeff0_48*dmats0[48][8] + coeff0_49*dmats0[49][8] + coeff0_50*dmats0[50][8] + coeff0_51*dmats0[51][8] + coeff0_52*dmats0[52][8] + coeff0_53*dmats0[53][8] + coeff0_54*dmats0[54][8] + coeff0_55*dmats0[55][8];
          new_coeff0_9 = coeff0_0*dmats0[0][9] + coeff0_1*dmats0[1][9] + coeff0_2*dmats0[2][9] + coeff0_3*dmats0[3][9] + coeff0_4*dmats0[4][9] + coeff0_5*dmats0[5][9] + coeff0_6*dmats0[6][9] + coeff0_7*dmats0[7][9] + coeff0_8*dmats0[8][9] + coeff0_9*dmats0[9][9] + coeff0_10*dmats0[10][9] + coeff0_11*dmats0[11][9] + coeff0_12*dmats0[12][9] + coeff0_13*dmats0[13][9] + coeff0_14*dmats0[14][9] + coeff0_15*dmats0[15][9] + coeff0_16*dmats0[16][9] + coeff0_17*dmats0[17][9] + coeff0_18*dmats0[18][9] + coeff0_19*dmats0[19][9] + coeff0_20*dmats0[20][9] + coeff0_21*dmats0[21][9] + coeff0_22*dmats0[22][9] + coeff0_23*dmats0[23][9] + coeff0_24*dmats0[24][9] + coeff0_25*dmats0[25][9] + coeff0_26*dmats0[26][9] + coeff0_27*dmats0[27][9] + coeff0_28*dmats0[28][9] + coeff0_29*dmats0[29][9] + coeff0_30*dmats0[30][9] + coeff0_31*dmats0[31][9] + coeff0_32*dmats0[32][9] + coeff0_33*dmats0[33][9] + coeff0_34*dmats0[34][9] + coeff0_35*dmats0[35][9] + coeff0_36*dmats0[36][9] + coeff0_37*dmats0[37][9] + coeff0_38*dmats0[38][9] + coeff0_39*dmats0[39][9] + coeff0_40*dmats0[40][9] + coeff0_41*dmats0[41][9] + coeff0_42*dmats0[42][9] + coeff0_43*dmats0[43][9] + coeff0_44*dmats0[44][9] + coeff0_45*dmats0[45][9] + coeff0_46*dmats0[46][9] + coeff0_47*dmats0[47][9] + coeff0_48*dmats0[48][9] + coeff0_49*dmats0[49][9] + coeff0_50*dmats0[50][9] + coeff0_51*dmats0[51][9] + coeff0_52*dmats0[52][9] + coeff0_53*dmats0[53][9] + coeff0_54*dmats0[54][9] + coeff0_55*dmats0[55][9];
          new_coeff0_10 = coeff0_0*dmats0[0][10] + coeff0_1*dmats0[1][10] + coeff0_2*dmats0[2][10] + coeff0_3*dmats0[3][10] + coeff0_4*dmats0[4][10] + coeff0_5*dmats0[5][10] + coeff0_6*dmats0[6][10] + coeff0_7*dmats0[7][10] + coeff0_8*dmats0[8][10] + coeff0_9*dmats0[9][10] + coeff0_10*dmats0[10][10] + coeff0_11*dmats0[11][10] + coeff0_12*dmats0[12][10] + coeff0_13*dmats0[13][10] + coeff0_14*dmats0[14][10] + coeff0_15*dmats0[15][10] + coeff0_16*dmats0[16][10] + coeff0_17*dmats0[17][10] + coeff0_18*dmats0[18][10] + coeff0_19*dmats0[19][10] + coeff0_20*dmats0[20][10] + coeff0_21*dmats0[21][10] + coeff0_22*dmats0[22][10] + coeff0_23*dmats0[23][10] + coeff0_24*dmats0[24][10] + coeff0_25*dmats0[25][10] + coeff0_26*dmats0[26][10] + coeff0_27*dmats0[27][10] + coeff0_28*dmats0[28][10] + coeff0_29*dmats0[29][10] + coeff0_30*dmats0[30][10] + coeff0_31*dmats0[31][10] + coeff0_32*dmats0[32][10] + coeff0_33*dmats0[33][10] + coeff0_34*dmats0[34][10] + coeff0_35*dmats0[35][10] + coeff0_36*dmats0[36][10] + coeff0_37*dmats0[37][10] + coeff0_38*dmats0[38][10] + coeff0_39*dmats0[39][10] + coeff0_40*dmats0[40][10] + coeff0_41*dmats0[41][10] + coeff0_42*dmats0[42][10] + coeff0_43*dmats0[43][10] + coeff0_44*dmats0[44][10] + coeff0_45*dmats0[45][10] + coeff0_46*dmats0[46][10] + coeff0_47*dmats0[47][10] + coeff0_48*dmats0[48][10] + coeff0_49*dmats0[49][10] + coeff0_50*dmats0[50][10] + coeff0_51*dmats0[51][10] + coeff0_52*dmats0[52][10] + coeff0_53*dmats0[53][10] + coeff0_54*dmats0[54][10] + coeff0_55*dmats0[55][10];
          new_coeff0_11 = coeff0_0*dmats0[0][11] + coeff0_1*dmats0[1][11] + coeff0_2*dmats0[2][11] + coeff0_3*dmats0[3][11] + coeff0_4*dmats0[4][11] + coeff0_5*dmats0[5][11] + coeff0_6*dmats0[6][11] + coeff0_7*dmats0[7][11] + coeff0_8*dmats0[8][11] + coeff0_9*dmats0[9][11] + coeff0_10*dmats0[10][11] + coeff0_11*dmats0[11][11] + coeff0_12*dmats0[12][11] + coeff0_13*dmats0[13][11] + coeff0_14*dmats0[14][11] + coeff0_15*dmats0[15][11] + coeff0_16*dmats0[16][11] + coeff0_17*dmats0[17][11] + coeff0_18*dmats0[18][11] + coeff0_19*dmats0[19][11] + coeff0_20*dmats0[20][11] + coeff0_21*dmats0[21][11] + coeff0_22*dmats0[22][11] + coeff0_23*dmats0[23][11] + coeff0_24*dmats0[24][11] + coeff0_25*dmats0[25][11] + coeff0_26*dmats0[26][11] + coeff0_27*dmats0[27][11] + coeff0_28*dmats0[28][11] + coeff0_29*dmats0[29][11] + coeff0_30*dmats0[30][11] + coeff0_31*dmats0[31][11] + coeff0_32*dmats0[32][11] + coeff0_33*dmats0[33][11] + coeff0_34*dmats0[34][11] + coeff0_35*dmats0[35][11] + coeff0_36*dmats0[36][11] + coeff0_37*dmats0[37][11] + coeff0_38*dmats0[38][11] + coeff0_39*dmats0[39][11] + coeff0_40*dmats0[40][11] + coeff0_41*dmats0[41][11] + coeff0_42*dmats0[42][11] + coeff0_43*dmats0[43][11] + coeff0_44*dmats0[44][11] + coeff0_45*dmats0[45][11] + coeff0_46*dmats0[46][11] + coeff0_47*dmats0[47][11] + coeff0_48*dmats0[48][11] + coeff0_49*dmats0[49][11] + coeff0_50*dmats0[50][11] + coeff0_51*dmats0[51][11] + coeff0_52*dmats0[52][11] + coeff0_53*dmats0[53][11] + coeff0_54*dmats0[54][11] + coeff0_55*dmats0[55][11];
          new_coeff0_12 = coeff0_0*dmats0[0][12] + coeff0_1*dmats0[1][12] + coeff0_2*dmats0[2][12] + coeff0_3*dmats0[3][12] + coeff0_4*dmats0[4][12] + coeff0_5*dmats0[5][12] + coeff0_6*dmats0[6][12] + coeff0_7*dmats0[7][12] + coeff0_8*dmats0[8][12] + coeff0_9*dmats0[9][12] + coeff0_10*dmats0[10][12] + coeff0_11*dmats0[11][12] + coeff0_12*dmats0[12][12] + coeff0_13*dmats0[13][12] + coeff0_14*dmats0[14][12] + coeff0_15*dmats0[15][12] + coeff0_16*dmats0[16][12] + coeff0_17*dmats0[17][12] + coeff0_18*dmats0[18][12] + coeff0_19*dmats0[19][12] + coeff0_20*dmats0[20][12] + coeff0_21*dmats0[21][12] + coeff0_22*dmats0[22][12] + coeff0_23*dmats0[23][12] + coeff0_24*dmats0[24][12] + coeff0_25*dmats0[25][12] + coeff0_26*dmats0[26][12] + coeff0_27*dmats0[27][12] + coeff0_28*dmats0[28][12] + coeff0_29*dmats0[29][12] + coeff0_30*dmats0[30][12] + coeff0_31*dmats0[31][12] + coeff0_32*dmats0[32][12] + coeff0_33*dmats0[33][12] + coeff0_34*dmats0[34][12] + coeff0_35*dmats0[35][12] + coeff0_36*dmats0[36][12] + coeff0_37*dmats0[37][12] + coeff0_38*dmats0[38][12] + coeff0_39*dmats0[39][12] + coeff0_40*dmats0[40][12] + coeff0_41*dmats0[41][12] + coeff0_42*dmats0[42][12] + coeff0_43*dmats0[43][12] + coeff0_44*dmats0[44][12] + coeff0_45*dmats0[45][12] + coeff0_46*dmats0[46][12] + coeff0_47*dmats0[47][12] + coeff0_48*dmats0[48][12] + coeff0_49*dmats0[49][12] + coeff0_50*dmats0[50][12] + coeff0_51*dmats0[51][12] + coeff0_52*dmats0[52][12] + coeff0_53*dmats0[53][12] + coeff0_54*dmats0[54][12] + coeff0_55*dmats0[55][12];
          new_coeff0_13 = coeff0_0*dmats0[0][13] + coeff0_1*dmats0[1][13] + coeff0_2*dmats0[2][13] + coeff0_3*dmats0[3][13] + coeff0_4*dmats0[4][13] + coeff0_5*dmats0[5][13] + coeff0_6*dmats0[6][13] + coeff0_7*dmats0[7][13] + coeff0_8*dmats0[8][13] + coeff0_9*dmats0[9][13] + coeff0_10*dmats0[10][13] + coeff0_11*dmats0[11][13] + coeff0_12*dmats0[12][13] + coeff0_13*dmats0[13][13] + coeff0_14*dmats0[14][13] + coeff0_15*dmats0[15][13] + coeff0_16*dmats0[16][13] + coeff0_17*dmats0[17][13] + coeff0_18*dmats0[18][13] + coeff0_19*dmats0[19][13] + coeff0_20*dmats0[20][13] + coeff0_21*dmats0[21][13] + coeff0_22*dmats0[22][13] + coeff0_23*dmats0[23][13] + coeff0_24*dmats0[24][13] + coeff0_25*dmats0[25][13] + coeff0_26*dmats0[26][13] + coeff0_27*dmats0[27][13] + coeff0_28*dmats0[28][13] + coeff0_29*dmats0[29][13] + coeff0_30*dmats0[30][13] + coeff0_31*dmats0[31][13] + coeff0_32*dmats0[32][13] + coeff0_33*dmats0[33][13] + coeff0_34*dmats0[34][13] + coeff0_35*dmats0[35][13] + coeff0_36*dmats0[36][13] + coeff0_37*dmats0[37][13] + coeff0_38*dmats0[38][13] + coeff0_39*dmats0[39][13] + coeff0_40*dmats0[40][13] + coeff0_41*dmats0[41][13] + coeff0_42*dmats0[42][13] + coeff0_43*dmats0[43][13] + coeff0_44*dmats0[44][13] + coeff0_45*dmats0[45][13] + coeff0_46*dmats0[46][13] + coeff0_47*dmats0[47][13] + coeff0_48*dmats0[48][13] + coeff0_49*dmats0[49][13] + coeff0_50*dmats0[50][13] + coeff0_51*dmats0[51][13] + coeff0_52*dmats0[52][13] + coeff0_53*dmats0[53][13] + coeff0_54*dmats0[54][13] + coeff0_55*dmats0[55][13];
          new_coeff0_14 = coeff0_0*dmats0[0][14] + coeff0_1*dmats0[1][14] + coeff0_2*dmats0[2][14] + coeff0_3*dmats0[3][14] + coeff0_4*dmats0[4][14] + coeff0_5*dmats0[5][14] + coeff0_6*dmats0[6][14] + coeff0_7*dmats0[7][14] + coeff0_8*dmats0[8][14] + coeff0_9*dmats0[9][14] + coeff0_10*dmats0[10][14] + coeff0_11*dmats0[11][14] + coeff0_12*dmats0[12][14] + coeff0_13*dmats0[13][14] + coeff0_14*dmats0[14][14] + coeff0_15*dmats0[15][14] + coeff0_16*dmats0[16][14] + coeff0_17*dmats0[17][14] + coeff0_18*dmats0[18][14] + coeff0_19*dmats0[19][14] + coeff0_20*dmats0[20][14] + coeff0_21*dmats0[21][14] + coeff0_22*dmats0[22][14] + coeff0_23*dmats0[23][14] + coeff0_24*dmats0[24][14] + coeff0_25*dmats0[25][14] + coeff0_26*dmats0[26][14] + coeff0_27*dmats0[27][14] + coeff0_28*dmats0[28][14] + coeff0_29*dmats0[29][14] + coeff0_30*dmats0[30][14] + coeff0_31*dmats0[31][14] + coeff0_32*dmats0[32][14] + coeff0_33*dmats0[33][14] + coeff0_34*dmats0[34][14] + coeff0_35*dmats0[35][14] + coeff0_36*dmats0[36][14] + coeff0_37*dmats0[37][14] + coeff0_38*dmats0[38][14] + coeff0_39*dmats0[39][14] + coeff0_40*dmats0[40][14] + coeff0_41*dmats0[41][14] + coeff0_42*dmats0[42][14] + coeff0_43*dmats0[43][14] + coeff0_44*dmats0[44][14] + coeff0_45*dmats0[45][14] + coeff0_46*dmats0[46][14] + coeff0_47*dmats0[47][14] + coeff0_48*dmats0[48][14] + coeff0_49*dmats0[49][14] + coeff0_50*dmats0[50][14] + coeff0_51*dmats0[51][14] + coeff0_52*dmats0[52][14] + coeff0_53*dmats0[53][14] + coeff0_54*dmats0[54][14] + coeff0_55*dmats0[55][14];
          new_coeff0_15 = coeff0_0*dmats0[0][15] + coeff0_1*dmats0[1][15] + coeff0_2*dmats0[2][15] + coeff0_3*dmats0[3][15] + coeff0_4*dmats0[4][15] + coeff0_5*dmats0[5][15] + coeff0_6*dmats0[6][15] + coeff0_7*dmats0[7][15] + coeff0_8*dmats0[8][15] + coeff0_9*dmats0[9][15] + coeff0_10*dmats0[10][15] + coeff0_11*dmats0[11][15] + coeff0_12*dmats0[12][15] + coeff0_13*dmats0[13][15] + coeff0_14*dmats0[14][15] + coeff0_15*dmats0[15][15] + coeff0_16*dmats0[16][15] + coeff0_17*dmats0[17][15] + coeff0_18*dmats0[18][15] + coeff0_19*dmats0[19][15] + coeff0_20*dmats0[20][15] + coeff0_21*dmats0[21][15] + coeff0_22*dmats0[22][15] + coeff0_23*dmats0[23][15] + coeff0_24*dmats0[24][15] + coeff0_25*dmats0[25][15] + coeff0_26*dmats0[26][15] + coeff0_27*dmats0[27][15] + coeff0_28*dmats0[28][15] + coeff0_29*dmats0[29][15] + coeff0_30*dmats0[30][15] + coeff0_31*dmats0[31][15] + coeff0_32*dmats0[32][15] + coeff0_33*dmats0[33][15] + coeff0_34*dmats0[34][15] + coeff0_35*dmats0[35][15] + coeff0_36*dmats0[36][15] + coeff0_37*dmats0[37][15] + coeff0_38*dmats0[38][15] + coeff0_39*dmats0[39][15] + coeff0_40*dmats0[40][15] + coeff0_41*dmats0[41][15] + coeff0_42*dmats0[42][15] + coeff0_43*dmats0[43][15] + coeff0_44*dmats0[44][15] + coeff0_45*dmats0[45][15] + coeff0_46*dmats0[46][15] + coeff0_47*dmats0[47][15] + coeff0_48*dmats0[48][15] + coeff0_49*dmats0[49][15] + coeff0_50*dmats0[50][15] + coeff0_51*dmats0[51][15] + coeff0_52*dmats0[52][15] + coeff0_53*dmats0[53][15] + coeff0_54*dmats0[54][15] + coeff0_55*dmats0[55][15];
          new_coeff0_16 = coeff0_0*dmats0[0][16] + coeff0_1*dmats0[1][16] + coeff0_2*dmats0[2][16] + coeff0_3*dmats0[3][16] + coeff0_4*dmats0[4][16] + coeff0_5*dmats0[5][16] + coeff0_6*dmats0[6][16] + coeff0_7*dmats0[7][16] + coeff0_8*dmats0[8][16] + coeff0_9*dmats0[9][16] + coeff0_10*dmats0[10][16] + coeff0_11*dmats0[11][16] + coeff0_12*dmats0[12][16] + coeff0_13*dmats0[13][16] + coeff0_14*dmats0[14][16] + coeff0_15*dmats0[15][16] + coeff0_16*dmats0[16][16] + coeff0_17*dmats0[17][16] + coeff0_18*dmats0[18][16] + coeff0_19*dmats0[19][16] + coeff0_20*dmats0[20][16] + coeff0_21*dmats0[21][16] + coeff0_22*dmats0[22][16] + coeff0_23*dmats0[23][16] + coeff0_24*dmats0[24][16] + coeff0_25*dmats0[25][16] + coeff0_26*dmats0[26][16] + coeff0_27*dmats0[27][16] + coeff0_28*dmats0[28][16] + coeff0_29*dmats0[29][16] + coeff0_30*dmats0[30][16] + coeff0_31*dmats0[31][16] + coeff0_32*dmats0[32][16] + coeff0_33*dmats0[33][16] + coeff0_34*dmats0[34][16] + coeff0_35*dmats0[35][16] + coeff0_36*dmats0[36][16] + coeff0_37*dmats0[37][16] + coeff0_38*dmats0[38][16] + coeff0_39*dmats0[39][16] + coeff0_40*dmats0[40][16] + coeff0_41*dmats0[41][16] + coeff0_42*dmats0[42][16] + coeff0_43*dmats0[43][16] + coeff0_44*dmats0[44][16] + coeff0_45*dmats0[45][16] + coeff0_46*dmats0[46][16] + coeff0_47*dmats0[47][16] + coeff0_48*dmats0[48][16] + coeff0_49*dmats0[49][16] + coeff0_50*dmats0[50][16] + coeff0_51*dmats0[51][16] + coeff0_52*dmats0[52][16] + coeff0_53*dmats0[53][16] + coeff0_54*dmats0[54][16] + coeff0_55*dmats0[55][16];
          new_coeff0_17 = coeff0_0*dmats0[0][17] + coeff0_1*dmats0[1][17] + coeff0_2*dmats0[2][17] + coeff0_3*dmats0[3][17] + coeff0_4*dmats0[4][17] + coeff0_5*dmats0[5][17] + coeff0_6*dmats0[6][17] + coeff0_7*dmats0[7][17] + coeff0_8*dmats0[8][17] + coeff0_9*dmats0[9][17] + coeff0_10*dmats0[10][17] + coeff0_11*dmats0[11][17] + coeff0_12*dmats0[12][17] + coeff0_13*dmats0[13][17] + coeff0_14*dmats0[14][17] + coeff0_15*dmats0[15][17] + coeff0_16*dmats0[16][17] + coeff0_17*dmats0[17][17] + coeff0_18*dmats0[18][17] + coeff0_19*dmats0[19][17] + coeff0_20*dmats0[20][17] + coeff0_21*dmats0[21][17] + coeff0_22*dmats0[22][17] + coeff0_23*dmats0[23][17] + coeff0_24*dmats0[24][17] + coeff0_25*dmats0[25][17] + coeff0_26*dmats0[26][17] + coeff0_27*dmats0[27][17] + coeff0_28*dmats0[28][17] + coeff0_29*dmats0[29][17] + coeff0_30*dmats0[30][17] + coeff0_31*dmats0[31][17] + coeff0_32*dmats0[32][17] + coeff0_33*dmats0[33][17] + coeff0_34*dmats0[34][17] + coeff0_35*dmats0[35][17] + coeff0_36*dmats0[36][17] + coeff0_37*dmats0[37][17] + coeff0_38*dmats0[38][17] + coeff0_39*dmats0[39][17] + coeff0_40*dmats0[40][17] + coeff0_41*dmats0[41][17] + coeff0_42*dmats0[42][17] + coeff0_43*dmats0[43][17] + coeff0_44*dmats0[44][17] + coeff0_45*dmats0[45][17] + coeff0_46*dmats0[46][17] + coeff0_47*dmats0[47][17] + coeff0_48*dmats0[48][17] + coeff0_49*dmats0[49][17] + coeff0_50*dmats0[50][17] + coeff0_51*dmats0[51][17] + coeff0_52*dmats0[52][17] + coeff0_53*dmats0[53][17] + coeff0_54*dmats0[54][17] + coeff0_55*dmats0[55][17];
          new_coeff0_18 = coeff0_0*dmats0[0][18] + coeff0_1*dmats0[1][18] + coeff0_2*dmats0[2][18] + coeff0_3*dmats0[3][18] + coeff0_4*dmats0[4][18] + coeff0_5*dmats0[5][18] + coeff0_6*dmats0[6][18] + coeff0_7*dmats0[7][18] + coeff0_8*dmats0[8][18] + coeff0_9*dmats0[9][18] + coeff0_10*dmats0[10][18] + coeff0_11*dmats0[11][18] + coeff0_12*dmats0[12][18] + coeff0_13*dmats0[13][18] + coeff0_14*dmats0[14][18] + coeff0_15*dmats0[15][18] + coeff0_16*dmats0[16][18] + coeff0_17*dmats0[17][18] + coeff0_18*dmats0[18][18] + coeff0_19*dmats0[19][18] + coeff0_20*dmats0[20][18] + coeff0_21*dmats0[21][18] + coeff0_22*dmats0[22][18] + coeff0_23*dmats0[23][18] + coeff0_24*dmats0[24][18] + coeff0_25*dmats0[25][18] + coeff0_26*dmats0[26][18] + coeff0_27*dmats0[27][18] + coeff0_28*dmats0[28][18] + coeff0_29*dmats0[29][18] + coeff0_30*dmats0[30][18] + coeff0_31*dmats0[31][18] + coeff0_32*dmats0[32][18] + coeff0_33*dmats0[33][18] + coeff0_34*dmats0[34][18] + coeff0_35*dmats0[35][18] + coeff0_36*dmats0[36][18] + coeff0_37*dmats0[37][18] + coeff0_38*dmats0[38][18] + coeff0_39*dmats0[39][18] + coeff0_40*dmats0[40][18] + coeff0_41*dmats0[41][18] + coeff0_42*dmats0[42][18] + coeff0_43*dmats0[43][18] + coeff0_44*dmats0[44][18] + coeff0_45*dmats0[45][18] + coeff0_46*dmats0[46][18] + coeff0_47*dmats0[47][18] + coeff0_48*dmats0[48][18] + coeff0_49*dmats0[49][18] + coeff0_50*dmats0[50][18] + coeff0_51*dmats0[51][18] + coeff0_52*dmats0[52][18] + coeff0_53*dmats0[53][18] + coeff0_54*dmats0[54][18] + coeff0_55*dmats0[55][18];
          new_coeff0_19 = coeff0_0*dmats0[0][19] + coeff0_1*dmats0[1][19] + coeff0_2*dmats0[2][19] + coeff0_3*dmats0[3][19] + coeff0_4*dmats0[4][19] + coeff0_5*dmats0[5][19] + coeff0_6*dmats0[6][19] + coeff0_7*dmats0[7][19] + coeff0_8*dmats0[8][19] + coeff0_9*dmats0[9][19] + coeff0_10*dmats0[10][19] + coeff0_11*dmats0[11][19] + coeff0_12*dmats0[12][19] + coeff0_13*dmats0[13][19] + coeff0_14*dmats0[14][19] + coeff0_15*dmats0[15][19] + coeff0_16*dmats0[16][19] + coeff0_17*dmats0[17][19] + coeff0_18*dmats0[18][19] + coeff0_19*dmats0[19][19] + coeff0_20*dmats0[20][19] + coeff0_21*dmats0[21][19] + coeff0_22*dmats0[22][19] + coeff0_23*dmats0[23][19] + coeff0_24*dmats0[24][19] + coeff0_25*dmats0[25][19] + coeff0_26*dmats0[26][19] + coeff0_27*dmats0[27][19] + coeff0_28*dmats0[28][19] + coeff0_29*dmats0[29][19] + coeff0_30*dmats0[30][19] + coeff0_31*dmats0[31][19] + coeff0_32*dmats0[32][19] + coeff0_33*dmats0[33][19] + coeff0_34*dmats0[34][19] + coeff0_35*dmats0[35][19] + coeff0_36*dmats0[36][19] + coeff0_37*dmats0[37][19] + coeff0_38*dmats0[38][19] + coeff0_39*dmats0[39][19] + coeff0_40*dmats0[40][19] + coeff0_41*dmats0[41][19] + coeff0_42*dmats0[42][19] + coeff0_43*dmats0[43][19] + coeff0_44*dmats0[44][19] + coeff0_45*dmats0[45][19] + coeff0_46*dmats0[46][19] + coeff0_47*dmats0[47][19] + coeff0_48*dmats0[48][19] + coeff0_49*dmats0[49][19] + coeff0_50*dmats0[50][19] + coeff0_51*dmats0[51][19] + coeff0_52*dmats0[52][19] + coeff0_53*dmats0[53][19] + coeff0_54*dmats0[54][19] + coeff0_55*dmats0[55][19];
          new_coeff0_20 = coeff0_0*dmats0[0][20] + coeff0_1*dmats0[1][20] + coeff0_2*dmats0[2][20] + coeff0_3*dmats0[3][20] + coeff0_4*dmats0[4][20] + coeff0_5*dmats0[5][20] + coeff0_6*dmats0[6][20] + coeff0_7*dmats0[7][20] + coeff0_8*dmats0[8][20] + coeff0_9*dmats0[9][20] + coeff0_10*dmats0[10][20] + coeff0_11*dmats0[11][20] + coeff0_12*dmats0[12][20] + coeff0_13*dmats0[13][20] + coeff0_14*dmats0[14][20] + coeff0_15*dmats0[15][20] + coeff0_16*dmats0[16][20] + coeff0_17*dmats0[17][20] + coeff0_18*dmats0[18][20] + coeff0_19*dmats0[19][20] + coeff0_20*dmats0[20][20] + coeff0_21*dmats0[21][20] + coeff0_22*dmats0[22][20] + coeff0_23*dmats0[23][20] + coeff0_24*dmats0[24][20] + coeff0_25*dmats0[25][20] + coeff0_26*dmats0[26][20] + coeff0_27*dmats0[27][20] + coeff0_28*dmats0[28][20] + coeff0_29*dmats0[29][20] + coeff0_30*dmats0[30][20] + coeff0_31*dmats0[31][20] + coeff0_32*dmats0[32][20] + coeff0_33*dmats0[33][20] + coeff0_34*dmats0[34][20] + coeff0_35*dmats0[35][20] + coeff0_36*dmats0[36][20] + coeff0_37*dmats0[37][20] + coeff0_38*dmats0[38][20] + coeff0_39*dmats0[39][20] + coeff0_40*dmats0[40][20] + coeff0_41*dmats0[41][20] + coeff0_42*dmats0[42][20] + coeff0_43*dmats0[43][20] + coeff0_44*dmats0[44][20] + coeff0_45*dmats0[45][20] + coeff0_46*dmats0[46][20] + coeff0_47*dmats0[47][20] + coeff0_48*dmats0[48][20] + coeff0_49*dmats0[49][20] + coeff0_50*dmats0[50][20] + coeff0_51*dmats0[51][20] + coeff0_52*dmats0[52][20] + coeff0_53*dmats0[53][20] + coeff0_54*dmats0[54][20] + coeff0_55*dmats0[55][20];
          new_coeff0_21 = coeff0_0*dmats0[0][21] + coeff0_1*dmats0[1][21] + coeff0_2*dmats0[2][21] + coeff0_3*dmats0[3][21] + coeff0_4*dmats0[4][21] + coeff0_5*dmats0[5][21] + coeff0_6*dmats0[6][21] + coeff0_7*dmats0[7][21] + coeff0_8*dmats0[8][21] + coeff0_9*dmats0[9][21] + coeff0_10*dmats0[10][21] + coeff0_11*dmats0[11][21] + coeff0_12*dmats0[12][21] + coeff0_13*dmats0[13][21] + coeff0_14*dmats0[14][21] + coeff0_15*dmats0[15][21] + coeff0_16*dmats0[16][21] + coeff0_17*dmats0[17][21] + coeff0_18*dmats0[18][21] + coeff0_19*dmats0[19][21] + coeff0_20*dmats0[20][21] + coeff0_21*dmats0[21][21] + coeff0_22*dmats0[22][21] + coeff0_23*dmats0[23][21] + coeff0_24*dmats0[24][21] + coeff0_25*dmats0[25][21] + coeff0_26*dmats0[26][21] + coeff0_27*dmats0[27][21] + coeff0_28*dmats0[28][21] + coeff0_29*dmats0[29][21] + coeff0_30*dmats0[30][21] + coeff0_31*dmats0[31][21] + coeff0_32*dmats0[32][21] + coeff0_33*dmats0[33][21] + coeff0_34*dmats0[34][21] + coeff0_35*dmats0[35][21] + coeff0_36*dmats0[36][21] + coeff0_37*dmats0[37][21] + coeff0_38*dmats0[38][21] + coeff0_39*dmats0[39][21] + coeff0_40*dmats0[40][21] + coeff0_41*dmats0[41][21] + coeff0_42*dmats0[42][21] + coeff0_43*dmats0[43][21] + coeff0_44*dmats0[44][21] + coeff0_45*dmats0[45][21] + coeff0_46*dmats0[46][21] + coeff0_47*dmats0[47][21] + coeff0_48*dmats0[48][21] + coeff0_49*dmats0[49][21] + coeff0_50*dmats0[50][21] + coeff0_51*dmats0[51][21] + coeff0_52*dmats0[52][21] + coeff0_53*dmats0[53][21] + coeff0_54*dmats0[54][21] + coeff0_55*dmats0[55][21];
          new_coeff0_22 = coeff0_0*dmats0[0][22] + coeff0_1*dmats0[1][22] + coeff0_2*dmats0[2][22] + coeff0_3*dmats0[3][22] + coeff0_4*dmats0[4][22] + coeff0_5*dmats0[5][22] + coeff0_6*dmats0[6][22] + coeff0_7*dmats0[7][22] + coeff0_8*dmats0[8][22] + coeff0_9*dmats0[9][22] + coeff0_10*dmats0[10][22] + coeff0_11*dmats0[11][22] + coeff0_12*dmats0[12][22] + coeff0_13*dmats0[13][22] + coeff0_14*dmats0[14][22] + coeff0_15*dmats0[15][22] + coeff0_16*dmats0[16][22] + coeff0_17*dmats0[17][22] + coeff0_18*dmats0[18][22] + coeff0_19*dmats0[19][22] + coeff0_20*dmats0[20][22] + coeff0_21*dmats0[21][22] + coeff0_22*dmats0[22][22] + coeff0_23*dmats0[23][22] + coeff0_24*dmats0[24][22] + coeff0_25*dmats0[25][22] + coeff0_26*dmats0[26][22] + coeff0_27*dmats0[27][22] + coeff0_28*dmats0[28][22] + coeff0_29*dmats0[29][22] + coeff0_30*dmats0[30][22] + coeff0_31*dmats0[31][22] + coeff0_32*dmats0[32][22] + coeff0_33*dmats0[33][22] + coeff0_34*dmats0[34][22] + coeff0_35*dmats0[35][22] + coeff0_36*dmats0[36][22] + coeff0_37*dmats0[37][22] + coeff0_38*dmats0[38][22] + coeff0_39*dmats0[39][22] + coeff0_40*dmats0[40][22] + coeff0_41*dmats0[41][22] + coeff0_42*dmats0[42][22] + coeff0_43*dmats0[43][22] + coeff0_44*dmats0[44][22] + coeff0_45*dmats0[45][22] + coeff0_46*dmats0[46][22] + coeff0_47*dmats0[47][22] + coeff0_48*dmats0[48][22] + coeff0_49*dmats0[49][22] + coeff0_50*dmats0[50][22] + coeff0_51*dmats0[51][22] + coeff0_52*dmats0[52][22] + coeff0_53*dmats0[53][22] + coeff0_54*dmats0[54][22] + coeff0_55*dmats0[55][22];
          new_coeff0_23 = coeff0_0*dmats0[0][23] + coeff0_1*dmats0[1][23] + coeff0_2*dmats0[2][23] + coeff0_3*dmats0[3][23] + coeff0_4*dmats0[4][23] + coeff0_5*dmats0[5][23] + coeff0_6*dmats0[6][23] + coeff0_7*dmats0[7][23] + coeff0_8*dmats0[8][23] + coeff0_9*dmats0[9][23] + coeff0_10*dmats0[10][23] + coeff0_11*dmats0[11][23] + coeff0_12*dmats0[12][23] + coeff0_13*dmats0[13][23] + coeff0_14*dmats0[14][23] + coeff0_15*dmats0[15][23] + coeff0_16*dmats0[16][23] + coeff0_17*dmats0[17][23] + coeff0_18*dmats0[18][23] + coeff0_19*dmats0[19][23] + coeff0_20*dmats0[20][23] + coeff0_21*dmats0[21][23] + coeff0_22*dmats0[22][23] + coeff0_23*dmats0[23][23] + coeff0_24*dmats0[24][23] + coeff0_25*dmats0[25][23] + coeff0_26*dmats0[26][23] + coeff0_27*dmats0[27][23] + coeff0_28*dmats0[28][23] + coeff0_29*dmats0[29][23] + coeff0_30*dmats0[30][23] + coeff0_31*dmats0[31][23] + coeff0_32*dmats0[32][23] + coeff0_33*dmats0[33][23] + coeff0_34*dmats0[34][23] + coeff0_35*dmats0[35][23] + coeff0_36*dmats0[36][23] + coeff0_37*dmats0[37][23] + coeff0_38*dmats0[38][23] + coeff0_39*dmats0[39][23] + coeff0_40*dmats0[40][23] + coeff0_41*dmats0[41][23] + coeff0_42*dmats0[42][23] + coeff0_43*dmats0[43][23] + coeff0_44*dmats0[44][23] + coeff0_45*dmats0[45][23] + coeff0_46*dmats0[46][23] + coeff0_47*dmats0[47][23] + coeff0_48*dmats0[48][23] + coeff0_49*dmats0[49][23] + coeff0_50*dmats0[50][23] + coeff0_51*dmats0[51][23] + coeff0_52*dmats0[52][23] + coeff0_53*dmats0[53][23] + coeff0_54*dmats0[54][23] + coeff0_55*dmats0[55][23];
          new_coeff0_24 = coeff0_0*dmats0[0][24] + coeff0_1*dmats0[1][24] + coeff0_2*dmats0[2][24] + coeff0_3*dmats0[3][24] + coeff0_4*dmats0[4][24] + coeff0_5*dmats0[5][24] + coeff0_6*dmats0[6][24] + coeff0_7*dmats0[7][24] + coeff0_8*dmats0[8][24] + coeff0_9*dmats0[9][24] + coeff0_10*dmats0[10][24] + coeff0_11*dmats0[11][24] + coeff0_12*dmats0[12][24] + coeff0_13*dmats0[13][24] + coeff0_14*dmats0[14][24] + coeff0_15*dmats0[15][24] + coeff0_16*dmats0[16][24] + coeff0_17*dmats0[17][24] + coeff0_18*dmats0[18][24] + coeff0_19*dmats0[19][24] + coeff0_20*dmats0[20][24] + coeff0_21*dmats0[21][24] + coeff0_22*dmats0[22][24] + coeff0_23*dmats0[23][24] + coeff0_24*dmats0[24][24] + coeff0_25*dmats0[25][24] + coeff0_26*dmats0[26][24] + coeff0_27*dmats0[27][24] + coeff0_28*dmats0[28][24] + coeff0_29*dmats0[29][24] + coeff0_30*dmats0[30][24] + coeff0_31*dmats0[31][24] + coeff0_32*dmats0[32][24] + coeff0_33*dmats0[33][24] + coeff0_34*dmats0[34][24] + coeff0_35*dmats0[35][24] + coeff0_36*dmats0[36][24] + coeff0_37*dmats0[37][24] + coeff0_38*dmats0[38][24] + coeff0_39*dmats0[39][24] + coeff0_40*dmats0[40][24] + coeff0_41*dmats0[41][24] + coeff0_42*dmats0[42][24] + coeff0_43*dmats0[43][24] + coeff0_44*dmats0[44][24] + coeff0_45*dmats0[45][24] + coeff0_46*dmats0[46][24] + coeff0_47*dmats0[47][24] + coeff0_48*dmats0[48][24] + coeff0_49*dmats0[49][24] + coeff0_50*dmats0[50][24] + coeff0_51*dmats0[51][24] + coeff0_52*dmats0[52][24] + coeff0_53*dmats0[53][24] + coeff0_54*dmats0[54][24] + coeff0_55*dmats0[55][24];
          new_coeff0_25 = coeff0_0*dmats0[0][25] + coeff0_1*dmats0[1][25] + coeff0_2*dmats0[2][25] + coeff0_3*dmats0[3][25] + coeff0_4*dmats0[4][25] + coeff0_5*dmats0[5][25] + coeff0_6*dmats0[6][25] + coeff0_7*dmats0[7][25] + coeff0_8*dmats0[8][25] + coeff0_9*dmats0[9][25] + coeff0_10*dmats0[10][25] + coeff0_11*dmats0[11][25] + coeff0_12*dmats0[12][25] + coeff0_13*dmats0[13][25] + coeff0_14*dmats0[14][25] + coeff0_15*dmats0[15][25] + coeff0_16*dmats0[16][25] + coeff0_17*dmats0[17][25] + coeff0_18*dmats0[18][25] + coeff0_19*dmats0[19][25] + coeff0_20*dmats0[20][25] + coeff0_21*dmats0[21][25] + coeff0_22*dmats0[22][25] + coeff0_23*dmats0[23][25] + coeff0_24*dmats0[24][25] + coeff0_25*dmats0[25][25] + coeff0_26*dmats0[26][25] + coeff0_27*dmats0[27][25] + coeff0_28*dmats0[28][25] + coeff0_29*dmats0[29][25] + coeff0_30*dmats0[30][25] + coeff0_31*dmats0[31][25] + coeff0_32*dmats0[32][25] + coeff0_33*dmats0[33][25] + coeff0_34*dmats0[34][25] + coeff0_35*dmats0[35][25] + coeff0_36*dmats0[36][25] + coeff0_37*dmats0[37][25] + coeff0_38*dmats0[38][25] + coeff0_39*dmats0[39][25] + coeff0_40*dmats0[40][25] + coeff0_41*dmats0[41][25] + coeff0_42*dmats0[42][25] + coeff0_43*dmats0[43][25] + coeff0_44*dmats0[44][25] + coeff0_45*dmats0[45][25] + coeff0_46*dmats0[46][25] + coeff0_47*dmats0[47][25] + coeff0_48*dmats0[48][25] + coeff0_49*dmats0[49][25] + coeff0_50*dmats0[50][25] + coeff0_51*dmats0[51][25] + coeff0_52*dmats0[52][25] + coeff0_53*dmats0[53][25] + coeff0_54*dmats0[54][25] + coeff0_55*dmats0[55][25];
          new_coeff0_26 = coeff0_0*dmats0[0][26] + coeff0_1*dmats0[1][26] + coeff0_2*dmats0[2][26] + coeff0_3*dmats0[3][26] + coeff0_4*dmats0[4][26] + coeff0_5*dmats0[5][26] + coeff0_6*dmats0[6][26] + coeff0_7*dmats0[7][26] + coeff0_8*dmats0[8][26] + coeff0_9*dmats0[9][26] + coeff0_10*dmats0[10][26] + coeff0_11*dmats0[11][26] + coeff0_12*dmats0[12][26] + coeff0_13*dmats0[13][26] + coeff0_14*dmats0[14][26] + coeff0_15*dmats0[15][26] + coeff0_16*dmats0[16][26] + coeff0_17*dmats0[17][26] + coeff0_18*dmats0[18][26] + coeff0_19*dmats0[19][26] + coeff0_20*dmats0[20][26] + coeff0_21*dmats0[21][26] + coeff0_22*dmats0[22][26] + coeff0_23*dmats0[23][26] + coeff0_24*dmats0[24][26] + coeff0_25*dmats0[25][26] + coeff0_26*dmats0[26][26] + coeff0_27*dmats0[27][26] + coeff0_28*dmats0[28][26] + coeff0_29*dmats0[29][26] + coeff0_30*dmats0[30][26] + coeff0_31*dmats0[31][26] + coeff0_32*dmats0[32][26] + coeff0_33*dmats0[33][26] + coeff0_34*dmats0[34][26] + coeff0_35*dmats0[35][26] + coeff0_36*dmats0[36][26] + coeff0_37*dmats0[37][26] + coeff0_38*dmats0[38][26] + coeff0_39*dmats0[39][26] + coeff0_40*dmats0[40][26] + coeff0_41*dmats0[41][26] + coeff0_42*dmats0[42][26] + coeff0_43*dmats0[43][26] + coeff0_44*dmats0[44][26] + coeff0_45*dmats0[45][26] + coeff0_46*dmats0[46][26] + coeff0_47*dmats0[47][26] + coeff0_48*dmats0[48][26] + coeff0_49*dmats0[49][26] + coeff0_50*dmats0[50][26] + coeff0_51*dmats0[51][26] + coeff0_52*dmats0[52][26] + coeff0_53*dmats0[53][26] + coeff0_54*dmats0[54][26] + coeff0_55*dmats0[55][26];
          new_coeff0_27 = coeff0_0*dmats0[0][27] + coeff0_1*dmats0[1][27] + coeff0_2*dmats0[2][27] + coeff0_3*dmats0[3][27] + coeff0_4*dmats0[4][27] + coeff0_5*dmats0[5][27] + coeff0_6*dmats0[6][27] + coeff0_7*dmats0[7][27] + coeff0_8*dmats0[8][27] + coeff0_9*dmats0[9][27] + coeff0_10*dmats0[10][27] + coeff0_11*dmats0[11][27] + coeff0_12*dmats0[12][27] + coeff0_13*dmats0[13][27] + coeff0_14*dmats0[14][27] + coeff0_15*dmats0[15][27] + coeff0_16*dmats0[16][27] + coeff0_17*dmats0[17][27] + coeff0_18*dmats0[18][27] + coeff0_19*dmats0[19][27] + coeff0_20*dmats0[20][27] + coeff0_21*dmats0[21][27] + coeff0_22*dmats0[22][27] + coeff0_23*dmats0[23][27] + coeff0_24*dmats0[24][27] + coeff0_25*dmats0[25][27] + coeff0_26*dmats0[26][27] + coeff0_27*dmats0[27][27] + coeff0_28*dmats0[28][27] + coeff0_29*dmats0[29][27] + coeff0_30*dmats0[30][27] + coeff0_31*dmats0[31][27] + coeff0_32*dmats0[32][27] + coeff0_33*dmats0[33][27] + coeff0_34*dmats0[34][27] + coeff0_35*dmats0[35][27] + coeff0_36*dmats0[36][27] + coeff0_37*dmats0[37][27] + coeff0_38*dmats0[38][27] + coeff0_39*dmats0[39][27] + coeff0_40*dmats0[40][27] + coeff0_41*dmats0[41][27] + coeff0_42*dmats0[42][27] + coeff0_43*dmats0[43][27] + coeff0_44*dmats0[44][27] + coeff0_45*dmats0[45][27] + coeff0_46*dmats0[46][27] + coeff0_47*dmats0[47][27] + coeff0_48*dmats0[48][27] + coeff0_49*dmats0[49][27] + coeff0_50*dmats0[50][27] + coeff0_51*dmats0[51][27] + coeff0_52*dmats0[52][27] + coeff0_53*dmats0[53][27] + coeff0_54*dmats0[54][27] + coeff0_55*dmats0[55][27];
          new_coeff0_28 = coeff0_0*dmats0[0][28] + coeff0_1*dmats0[1][28] + coeff0_2*dmats0[2][28] + coeff0_3*dmats0[3][28] + coeff0_4*dmats0[4][28] + coeff0_5*dmats0[5][28] + coeff0_6*dmats0[6][28] + coeff0_7*dmats0[7][28] + coeff0_8*dmats0[8][28] + coeff0_9*dmats0[9][28] + coeff0_10*dmats0[10][28] + coeff0_11*dmats0[11][28] + coeff0_12*dmats0[12][28] + coeff0_13*dmats0[13][28] + coeff0_14*dmats0[14][28] + coeff0_15*dmats0[15][28] + coeff0_16*dmats0[16][28] + coeff0_17*dmats0[17][28] + coeff0_18*dmats0[18][28] + coeff0_19*dmats0[19][28] + coeff0_20*dmats0[20][28] + coeff0_21*dmats0[21][28] + coeff0_22*dmats0[22][28] + coeff0_23*dmats0[23][28] + coeff0_24*dmats0[24][28] + coeff0_25*dmats0[25][28] + coeff0_26*dmats0[26][28] + coeff0_27*dmats0[27][28] + coeff0_28*dmats0[28][28] + coeff0_29*dmats0[29][28] + coeff0_30*dmats0[30][28] + coeff0_31*dmats0[31][28] + coeff0_32*dmats0[32][28] + coeff0_33*dmats0[33][28] + coeff0_34*dmats0[34][28] + coeff0_35*dmats0[35][28] + coeff0_36*dmats0[36][28] + coeff0_37*dmats0[37][28] + coeff0_38*dmats0[38][28] + coeff0_39*dmats0[39][28] + coeff0_40*dmats0[40][28] + coeff0_41*dmats0[41][28] + coeff0_42*dmats0[42][28] + coeff0_43*dmats0[43][28] + coeff0_44*dmats0[44][28] + coeff0_45*dmats0[45][28] + coeff0_46*dmats0[46][28] + coeff0_47*dmats0[47][28] + coeff0_48*dmats0[48][28] + coeff0_49*dmats0[49][28] + coeff0_50*dmats0[50][28] + coeff0_51*dmats0[51][28] + coeff0_52*dmats0[52][28] + coeff0_53*dmats0[53][28] + coeff0_54*dmats0[54][28] + coeff0_55*dmats0[55][28];
          new_coeff0_29 = coeff0_0*dmats0[0][29] + coeff0_1*dmats0[1][29] + coeff0_2*dmats0[2][29] + coeff0_3*dmats0[3][29] + coeff0_4*dmats0[4][29] + coeff0_5*dmats0[5][29] + coeff0_6*dmats0[6][29] + coeff0_7*dmats0[7][29] + coeff0_8*dmats0[8][29] + coeff0_9*dmats0[9][29] + coeff0_10*dmats0[10][29] + coeff0_11*dmats0[11][29] + coeff0_12*dmats0[12][29] + coeff0_13*dmats0[13][29] + coeff0_14*dmats0[14][29] + coeff0_15*dmats0[15][29] + coeff0_16*dmats0[16][29] + coeff0_17*dmats0[17][29] + coeff0_18*dmats0[18][29] + coeff0_19*dmats0[19][29] + coeff0_20*dmats0[20][29] + coeff0_21*dmats0[21][29] + coeff0_22*dmats0[22][29] + coeff0_23*dmats0[23][29] + coeff0_24*dmats0[24][29] + coeff0_25*dmats0[25][29] + coeff0_26*dmats0[26][29] + coeff0_27*dmats0[27][29] + coeff0_28*dmats0[28][29] + coeff0_29*dmats0[29][29] + coeff0_30*dmats0[30][29] + coeff0_31*dmats0[31][29] + coeff0_32*dmats0[32][29] + coeff0_33*dmats0[33][29] + coeff0_34*dmats0[34][29] + coeff0_35*dmats0[35][29] + coeff0_36*dmats0[36][29] + coeff0_37*dmats0[37][29] + coeff0_38*dmats0[38][29] + coeff0_39*dmats0[39][29] + coeff0_40*dmats0[40][29] + coeff0_41*dmats0[41][29] + coeff0_42*dmats0[42][29] + coeff0_43*dmats0[43][29] + coeff0_44*dmats0[44][29] + coeff0_45*dmats0[45][29] + coeff0_46*dmats0[46][29] + coeff0_47*dmats0[47][29] + coeff0_48*dmats0[48][29] + coeff0_49*dmats0[49][29] + coeff0_50*dmats0[50][29] + coeff0_51*dmats0[51][29] + coeff0_52*dmats0[52][29] + coeff0_53*dmats0[53][29] + coeff0_54*dmats0[54][29] + coeff0_55*dmats0[55][29];
          new_coeff0_30 = coeff0_0*dmats0[0][30] + coeff0_1*dmats0[1][30] + coeff0_2*dmats0[2][30] + coeff0_3*dmats0[3][30] + coeff0_4*dmats0[4][30] + coeff0_5*dmats0[5][30] + coeff0_6*dmats0[6][30] + coeff0_7*dmats0[7][30] + coeff0_8*dmats0[8][30] + coeff0_9*dmats0[9][30] + coeff0_10*dmats0[10][30] + coeff0_11*dmats0[11][30] + coeff0_12*dmats0[12][30] + coeff0_13*dmats0[13][30] + coeff0_14*dmats0[14][30] + coeff0_15*dmats0[15][30] + coeff0_16*dmats0[16][30] + coeff0_17*dmats0[17][30] + coeff0_18*dmats0[18][30] + coeff0_19*dmats0[19][30] + coeff0_20*dmats0[20][30] + coeff0_21*dmats0[21][30] + coeff0_22*dmats0[22][30] + coeff0_23*dmats0[23][30] + coeff0_24*dmats0[24][30] + coeff0_25*dmats0[25][30] + coeff0_26*dmats0[26][30] + coeff0_27*dmats0[27][30] + coeff0_28*dmats0[28][30] + coeff0_29*dmats0[29][30] + coeff0_30*dmats0[30][30] + coeff0_31*dmats0[31][30] + coeff0_32*dmats0[32][30] + coeff0_33*dmats0[33][30] + coeff0_34*dmats0[34][30] + coeff0_35*dmats0[35][30] + coeff0_36*dmats0[36][30] + coeff0_37*dmats0[37][30] + coeff0_38*dmats0[38][30] + coeff0_39*dmats0[39][30] + coeff0_40*dmats0[40][30] + coeff0_41*dmats0[41][30] + coeff0_42*dmats0[42][30] + coeff0_43*dmats0[43][30] + coeff0_44*dmats0[44][30] + coeff0_45*dmats0[45][30] + coeff0_46*dmats0[46][30] + coeff0_47*dmats0[47][30] + coeff0_48*dmats0[48][30] + coeff0_49*dmats0[49][30] + coeff0_50*dmats0[50][30] + coeff0_51*dmats0[51][30] + coeff0_52*dmats0[52][30] + coeff0_53*dmats0[53][30] + coeff0_54*dmats0[54][30] + coeff0_55*dmats0[55][30];
          new_coeff0_31 = coeff0_0*dmats0[0][31] + coeff0_1*dmats0[1][31] + coeff0_2*dmats0[2][31] + coeff0_3*dmats0[3][31] + coeff0_4*dmats0[4][31] + coeff0_5*dmats0[5][31] + coeff0_6*dmats0[6][31] + coeff0_7*dmats0[7][31] + coeff0_8*dmats0[8][31] + coeff0_9*dmats0[9][31] + coeff0_10*dmats0[10][31] + coeff0_11*dmats0[11][31] + coeff0_12*dmats0[12][31] + coeff0_13*dmats0[13][31] + coeff0_14*dmats0[14][31] + coeff0_15*dmats0[15][31] + coeff0_16*dmats0[16][31] + coeff0_17*dmats0[17][31] + coeff0_18*dmats0[18][31] + coeff0_19*dmats0[19][31] + coeff0_20*dmats0[20][31] + coeff0_21*dmats0[21][31] + coeff0_22*dmats0[22][31] + coeff0_23*dmats0[23][31] + coeff0_24*dmats0[24][31] + coeff0_25*dmats0[25][31] + coeff0_26*dmats0[26][31] + coeff0_27*dmats0[27][31] + coeff0_28*dmats0[28][31] + coeff0_29*dmats0[29][31] + coeff0_30*dmats0[30][31] + coeff0_31*dmats0[31][31] + coeff0_32*dmats0[32][31] + coeff0_33*dmats0[33][31] + coeff0_34*dmats0[34][31] + coeff0_35*dmats0[35][31] + coeff0_36*dmats0[36][31] + coeff0_37*dmats0[37][31] + coeff0_38*dmats0[38][31] + coeff0_39*dmats0[39][31] + coeff0_40*dmats0[40][31] + coeff0_41*dmats0[41][31] + coeff0_42*dmats0[42][31] + coeff0_43*dmats0[43][31] + coeff0_44*dmats0[44][31] + coeff0_45*dmats0[45][31] + coeff0_46*dmats0[46][31] + coeff0_47*dmats0[47][31] + coeff0_48*dmats0[48][31] + coeff0_49*dmats0[49][31] + coeff0_50*dmats0[50][31] + coeff0_51*dmats0[51][31] + coeff0_52*dmats0[52][31] + coeff0_53*dmats0[53][31] + coeff0_54*dmats0[54][31] + coeff0_55*dmats0[55][31];
          new_coeff0_32 = coeff0_0*dmats0[0][32] + coeff0_1*dmats0[1][32] + coeff0_2*dmats0[2][32] + coeff0_3*dmats0[3][32] + coeff0_4*dmats0[4][32] + coeff0_5*dmats0[5][32] + coeff0_6*dmats0[6][32] + coeff0_7*dmats0[7][32] + coeff0_8*dmats0[8][32] + coeff0_9*dmats0[9][32] + coeff0_10*dmats0[10][32] + coeff0_11*dmats0[11][32] + coeff0_12*dmats0[12][32] + coeff0_13*dmats0[13][32] + coeff0_14*dmats0[14][32] + coeff0_15*dmats0[15][32] + coeff0_16*dmats0[16][32] + coeff0_17*dmats0[17][32] + coeff0_18*dmats0[18][32] + coeff0_19*dmats0[19][32] + coeff0_20*dmats0[20][32] + coeff0_21*dmats0[21][32] + coeff0_22*dmats0[22][32] + coeff0_23*dmats0[23][32] + coeff0_24*dmats0[24][32] + coeff0_25*dmats0[25][32] + coeff0_26*dmats0[26][32] + coeff0_27*dmats0[27][32] + coeff0_28*dmats0[28][32] + coeff0_29*dmats0[29][32] + coeff0_30*dmats0[30][32] + coeff0_31*dmats0[31][32] + coeff0_32*dmats0[32][32] + coeff0_33*dmats0[33][32] + coeff0_34*dmats0[34][32] + coeff0_35*dmats0[35][32] + coeff0_36*dmats0[36][32] + coeff0_37*dmats0[37][32] + coeff0_38*dmats0[38][32] + coeff0_39*dmats0[39][32] + coeff0_40*dmats0[40][32] + coeff0_41*dmats0[41][32] + coeff0_42*dmats0[42][32] + coeff0_43*dmats0[43][32] + coeff0_44*dmats0[44][32] + coeff0_45*dmats0[45][32] + coeff0_46*dmats0[46][32] + coeff0_47*dmats0[47][32] + coeff0_48*dmats0[48][32] + coeff0_49*dmats0[49][32] + coeff0_50*dmats0[50][32] + coeff0_51*dmats0[51][32] + coeff0_52*dmats0[52][32] + coeff0_53*dmats0[53][32] + coeff0_54*dmats0[54][32] + coeff0_55*dmats0[55][32];
          new_coeff0_33 = coeff0_0*dmats0[0][33] + coeff0_1*dmats0[1][33] + coeff0_2*dmats0[2][33] + coeff0_3*dmats0[3][33] + coeff0_4*dmats0[4][33] + coeff0_5*dmats0[5][33] + coeff0_6*dmats0[6][33] + coeff0_7*dmats0[7][33] + coeff0_8*dmats0[8][33] + coeff0_9*dmats0[9][33] + coeff0_10*dmats0[10][33] + coeff0_11*dmats0[11][33] + coeff0_12*dmats0[12][33] + coeff0_13*dmats0[13][33] + coeff0_14*dmats0[14][33] + coeff0_15*dmats0[15][33] + coeff0_16*dmats0[16][33] + coeff0_17*dmats0[17][33] + coeff0_18*dmats0[18][33] + coeff0_19*dmats0[19][33] + coeff0_20*dmats0[20][33] + coeff0_21*dmats0[21][33] + coeff0_22*dmats0[22][33] + coeff0_23*dmats0[23][33] + coeff0_24*dmats0[24][33] + coeff0_25*dmats0[25][33] + coeff0_26*dmats0[26][33] + coeff0_27*dmats0[27][33] + coeff0_28*dmats0[28][33] + coeff0_29*dmats0[29][33] + coeff0_30*dmats0[30][33] + coeff0_31*dmats0[31][33] + coeff0_32*dmats0[32][33] + coeff0_33*dmats0[33][33] + coeff0_34*dmats0[34][33] + coeff0_35*dmats0[35][33] + coeff0_36*dmats0[36][33] + coeff0_37*dmats0[37][33] + coeff0_38*dmats0[38][33] + coeff0_39*dmats0[39][33] + coeff0_40*dmats0[40][33] + coeff0_41*dmats0[41][33] + coeff0_42*dmats0[42][33] + coeff0_43*dmats0[43][33] + coeff0_44*dmats0[44][33] + coeff0_45*dmats0[45][33] + coeff0_46*dmats0[46][33] + coeff0_47*dmats0[47][33] + coeff0_48*dmats0[48][33] + coeff0_49*dmats0[49][33] + coeff0_50*dmats0[50][33] + coeff0_51*dmats0[51][33] + coeff0_52*dmats0[52][33] + coeff0_53*dmats0[53][33] + coeff0_54*dmats0[54][33] + coeff0_55*dmats0[55][33];
          new_coeff0_34 = coeff0_0*dmats0[0][34] + coeff0_1*dmats0[1][34] + coeff0_2*dmats0[2][34] + coeff0_3*dmats0[3][34] + coeff0_4*dmats0[4][34] + coeff0_5*dmats0[5][34] + coeff0_6*dmats0[6][34] + coeff0_7*dmats0[7][34] + coeff0_8*dmats0[8][34] + coeff0_9*dmats0[9][34] + coeff0_10*dmats0[10][34] + coeff0_11*dmats0[11][34] + coeff0_12*dmats0[12][34] + coeff0_13*dmats0[13][34] + coeff0_14*dmats0[14][34] + coeff0_15*dmats0[15][34] + coeff0_16*dmats0[16][34] + coeff0_17*dmats0[17][34] + coeff0_18*dmats0[18][34] + coeff0_19*dmats0[19][34] + coeff0_20*dmats0[20][34] + coeff0_21*dmats0[21][34] + coeff0_22*dmats0[22][34] + coeff0_23*dmats0[23][34] + coeff0_24*dmats0[24][34] + coeff0_25*dmats0[25][34] + coeff0_26*dmats0[26][34] + coeff0_27*dmats0[27][34] + coeff0_28*dmats0[28][34] + coeff0_29*dmats0[29][34] + coeff0_30*dmats0[30][34] + coeff0_31*dmats0[31][34] + coeff0_32*dmats0[32][34] + coeff0_33*dmats0[33][34] + coeff0_34*dmats0[34][34] + coeff0_35*dmats0[35][34] + coeff0_36*dmats0[36][34] + coeff0_37*dmats0[37][34] + coeff0_38*dmats0[38][34] + coeff0_39*dmats0[39][34] + coeff0_40*dmats0[40][34] + coeff0_41*dmats0[41][34] + coeff0_42*dmats0[42][34] + coeff0_43*dmats0[43][34] + coeff0_44*dmats0[44][34] + coeff0_45*dmats0[45][34] + coeff0_46*dmats0[46][34] + coeff0_47*dmats0[47][34] + coeff0_48*dmats0[48][34] + coeff0_49*dmats0[49][34] + coeff0_50*dmats0[50][34] + coeff0_51*dmats0[51][34] + coeff0_52*dmats0[52][34] + coeff0_53*dmats0[53][34] + coeff0_54*dmats0[54][34] + coeff0_55*dmats0[55][34];
          new_coeff0_35 = coeff0_0*dmats0[0][35] + coeff0_1*dmats0[1][35] + coeff0_2*dmats0[2][35] + coeff0_3*dmats0[3][35] + coeff0_4*dmats0[4][35] + coeff0_5*dmats0[5][35] + coeff0_6*dmats0[6][35] + coeff0_7*dmats0[7][35] + coeff0_8*dmats0[8][35] + coeff0_9*dmats0[9][35] + coeff0_10*dmats0[10][35] + coeff0_11*dmats0[11][35] + coeff0_12*dmats0[12][35] + coeff0_13*dmats0[13][35] + coeff0_14*dmats0[14][35] + coeff0_15*dmats0[15][35] + coeff0_16*dmats0[16][35] + coeff0_17*dmats0[17][35] + coeff0_18*dmats0[18][35] + coeff0_19*dmats0[19][35] + coeff0_20*dmats0[20][35] + coeff0_21*dmats0[21][35] + coeff0_22*dmats0[22][35] + coeff0_23*dmats0[23][35] + coeff0_24*dmats0[24][35] + coeff0_25*dmats0[25][35] + coeff0_26*dmats0[26][35] + coeff0_27*dmats0[27][35] + coeff0_28*dmats0[28][35] + coeff0_29*dmats0[29][35] + coeff0_30*dmats0[30][35] + coeff0_31*dmats0[31][35] + coeff0_32*dmats0[32][35] + coeff0_33*dmats0[33][35] + coeff0_34*dmats0[34][35] + coeff0_35*dmats0[35][35] + coeff0_36*dmats0[36][35] + coeff0_37*dmats0[37][35] + coeff0_38*dmats0[38][35] + coeff0_39*dmats0[39][35] + coeff0_40*dmats0[40][35] + coeff0_41*dmats0[41][35] + coeff0_42*dmats0[42][35] + coeff0_43*dmats0[43][35] + coeff0_44*dmats0[44][35] + coeff0_45*dmats0[45][35] + coeff0_46*dmats0[46][35] + coeff0_47*dmats0[47][35] + coeff0_48*dmats0[48][35] + coeff0_49*dmats0[49][35] + coeff0_50*dmats0[50][35] + coeff0_51*dmats0[51][35] + coeff0_52*dmats0[52][35] + coeff0_53*dmats0[53][35] + coeff0_54*dmats0[54][35] + coeff0_55*dmats0[55][35];
          new_coeff0_36 = coeff0_0*dmats0[0][36] + coeff0_1*dmats0[1][36] + coeff0_2*dmats0[2][36] + coeff0_3*dmats0[3][36] + coeff0_4*dmats0[4][36] + coeff0_5*dmats0[5][36] + coeff0_6*dmats0[6][36] + coeff0_7*dmats0[7][36] + coeff0_8*dmats0[8][36] + coeff0_9*dmats0[9][36] + coeff0_10*dmats0[10][36] + coeff0_11*dmats0[11][36] + coeff0_12*dmats0[12][36] + coeff0_13*dmats0[13][36] + coeff0_14*dmats0[14][36] + coeff0_15*dmats0[15][36] + coeff0_16*dmats0[16][36] + coeff0_17*dmats0[17][36] + coeff0_18*dmats0[18][36] + coeff0_19*dmats0[19][36] + coeff0_20*dmats0[20][36] + coeff0_21*dmats0[21][36] + coeff0_22*dmats0[22][36] + coeff0_23*dmats0[23][36] + coeff0_24*dmats0[24][36] + coeff0_25*dmats0[25][36] + coeff0_26*dmats0[26][36] + coeff0_27*dmats0[27][36] + coeff0_28*dmats0[28][36] + coeff0_29*dmats0[29][36] + coeff0_30*dmats0[30][36] + coeff0_31*dmats0[31][36] + coeff0_32*dmats0[32][36] + coeff0_33*dmats0[33][36] + coeff0_34*dmats0[34][36] + coeff0_35*dmats0[35][36] + coeff0_36*dmats0[36][36] + coeff0_37*dmats0[37][36] + coeff0_38*dmats0[38][36] + coeff0_39*dmats0[39][36] + coeff0_40*dmats0[40][36] + coeff0_41*dmats0[41][36] + coeff0_42*dmats0[42][36] + coeff0_43*dmats0[43][36] + coeff0_44*dmats0[44][36] + coeff0_45*dmats0[45][36] + coeff0_46*dmats0[46][36] + coeff0_47*dmats0[47][36] + coeff0_48*dmats0[48][36] + coeff0_49*dmats0[49][36] + coeff0_50*dmats0[50][36] + coeff0_51*dmats0[51][36] + coeff0_52*dmats0[52][36] + coeff0_53*dmats0[53][36] + coeff0_54*dmats0[54][36] + coeff0_55*dmats0[55][36];
          new_coeff0_37 = coeff0_0*dmats0[0][37] + coeff0_1*dmats0[1][37] + coeff0_2*dmats0[2][37] + coeff0_3*dmats0[3][37] + coeff0_4*dmats0[4][37] + coeff0_5*dmats0[5][37] + coeff0_6*dmats0[6][37] + coeff0_7*dmats0[7][37] + coeff0_8*dmats0[8][37] + coeff0_9*dmats0[9][37] + coeff0_10*dmats0[10][37] + coeff0_11*dmats0[11][37] + coeff0_12*dmats0[12][37] + coeff0_13*dmats0[13][37] + coeff0_14*dmats0[14][37] + coeff0_15*dmats0[15][37] + coeff0_16*dmats0[16][37] + coeff0_17*dmats0[17][37] + coeff0_18*dmats0[18][37] + coeff0_19*dmats0[19][37] + coeff0_20*dmats0[20][37] + coeff0_21*dmats0[21][37] + coeff0_22*dmats0[22][37] + coeff0_23*dmats0[23][37] + coeff0_24*dmats0[24][37] + coeff0_25*dmats0[25][37] + coeff0_26*dmats0[26][37] + coeff0_27*dmats0[27][37] + coeff0_28*dmats0[28][37] + coeff0_29*dmats0[29][37] + coeff0_30*dmats0[30][37] + coeff0_31*dmats0[31][37] + coeff0_32*dmats0[32][37] + coeff0_33*dmats0[33][37] + coeff0_34*dmats0[34][37] + coeff0_35*dmats0[35][37] + coeff0_36*dmats0[36][37] + coeff0_37*dmats0[37][37] + coeff0_38*dmats0[38][37] + coeff0_39*dmats0[39][37] + coeff0_40*dmats0[40][37] + coeff0_41*dmats0[41][37] + coeff0_42*dmats0[42][37] + coeff0_43*dmats0[43][37] + coeff0_44*dmats0[44][37] + coeff0_45*dmats0[45][37] + coeff0_46*dmats0[46][37] + coeff0_47*dmats0[47][37] + coeff0_48*dmats0[48][37] + coeff0_49*dmats0[49][37] + coeff0_50*dmats0[50][37] + coeff0_51*dmats0[51][37] + coeff0_52*dmats0[52][37] + coeff0_53*dmats0[53][37] + coeff0_54*dmats0[54][37] + coeff0_55*dmats0[55][37];
          new_coeff0_38 = coeff0_0*dmats0[0][38] + coeff0_1*dmats0[1][38] + coeff0_2*dmats0[2][38] + coeff0_3*dmats0[3][38] + coeff0_4*dmats0[4][38] + coeff0_5*dmats0[5][38] + coeff0_6*dmats0[6][38] + coeff0_7*dmats0[7][38] + coeff0_8*dmats0[8][38] + coeff0_9*dmats0[9][38] + coeff0_10*dmats0[10][38] + coeff0_11*dmats0[11][38] + coeff0_12*dmats0[12][38] + coeff0_13*dmats0[13][38] + coeff0_14*dmats0[14][38] + coeff0_15*dmats0[15][38] + coeff0_16*dmats0[16][38] + coeff0_17*dmats0[17][38] + coeff0_18*dmats0[18][38] + coeff0_19*dmats0[19][38] + coeff0_20*dmats0[20][38] + coeff0_21*dmats0[21][38] + coeff0_22*dmats0[22][38] + coeff0_23*dmats0[23][38] + coeff0_24*dmats0[24][38] + coeff0_25*dmats0[25][38] + coeff0_26*dmats0[26][38] + coeff0_27*dmats0[27][38] + coeff0_28*dmats0[28][38] + coeff0_29*dmats0[29][38] + coeff0_30*dmats0[30][38] + coeff0_31*dmats0[31][38] + coeff0_32*dmats0[32][38] + coeff0_33*dmats0[33][38] + coeff0_34*dmats0[34][38] + coeff0_35*dmats0[35][38] + coeff0_36*dmats0[36][38] + coeff0_37*dmats0[37][38] + coeff0_38*dmats0[38][38] + coeff0_39*dmats0[39][38] + coeff0_40*dmats0[40][38] + coeff0_41*dmats0[41][38] + coeff0_42*dmats0[42][38] + coeff0_43*dmats0[43][38] + coeff0_44*dmats0[44][38] + coeff0_45*dmats0[45][38] + coeff0_46*dmats0[46][38] + coeff0_47*dmats0[47][38] + coeff0_48*dmats0[48][38] + coeff0_49*dmats0[49][38] + coeff0_50*dmats0[50][38] + coeff0_51*dmats0[51][38] + coeff0_52*dmats0[52][38] + coeff0_53*dmats0[53][38] + coeff0_54*dmats0[54][38] + coeff0_55*dmats0[55][38];
          new_coeff0_39 = coeff0_0*dmats0[0][39] + coeff0_1*dmats0[1][39] + coeff0_2*dmats0[2][39] + coeff0_3*dmats0[3][39] + coeff0_4*dmats0[4][39] + coeff0_5*dmats0[5][39] + coeff0_6*dmats0[6][39] + coeff0_7*dmats0[7][39] + coeff0_8*dmats0[8][39] + coeff0_9*dmats0[9][39] + coeff0_10*dmats0[10][39] + coeff0_11*dmats0[11][39] + coeff0_12*dmats0[12][39] + coeff0_13*dmats0[13][39] + coeff0_14*dmats0[14][39] + coeff0_15*dmats0[15][39] + coeff0_16*dmats0[16][39] + coeff0_17*dmats0[17][39] + coeff0_18*dmats0[18][39] + coeff0_19*dmats0[19][39] + coeff0_20*dmats0[20][39] + coeff0_21*dmats0[21][39] + coeff0_22*dmats0[22][39] + coeff0_23*dmats0[23][39] + coeff0_24*dmats0[24][39] + coeff0_25*dmats0[25][39] + coeff0_26*dmats0[26][39] + coeff0_27*dmats0[27][39] + coeff0_28*dmats0[28][39] + coeff0_29*dmats0[29][39] + coeff0_30*dmats0[30][39] + coeff0_31*dmats0[31][39] + coeff0_32*dmats0[32][39] + coeff0_33*dmats0[33][39] + coeff0_34*dmats0[34][39] + coeff0_35*dmats0[35][39] + coeff0_36*dmats0[36][39] + coeff0_37*dmats0[37][39] + coeff0_38*dmats0[38][39] + coeff0_39*dmats0[39][39] + coeff0_40*dmats0[40][39] + coeff0_41*dmats0[41][39] + coeff0_42*dmats0[42][39] + coeff0_43*dmats0[43][39] + coeff0_44*dmats0[44][39] + coeff0_45*dmats0[45][39] + coeff0_46*dmats0[46][39] + coeff0_47*dmats0[47][39] + coeff0_48*dmats0[48][39] + coeff0_49*dmats0[49][39] + coeff0_50*dmats0[50][39] + coeff0_51*dmats0[51][39] + coeff0_52*dmats0[52][39] + coeff0_53*dmats0[53][39] + coeff0_54*dmats0[54][39] + coeff0_55*dmats0[55][39];
          new_coeff0_40 = coeff0_0*dmats0[0][40] + coeff0_1*dmats0[1][40] + coeff0_2*dmats0[2][40] + coeff0_3*dmats0[3][40] + coeff0_4*dmats0[4][40] + coeff0_5*dmats0[5][40] + coeff0_6*dmats0[6][40] + coeff0_7*dmats0[7][40] + coeff0_8*dmats0[8][40] + coeff0_9*dmats0[9][40] + coeff0_10*dmats0[10][40] + coeff0_11*dmats0[11][40] + coeff0_12*dmats0[12][40] + coeff0_13*dmats0[13][40] + coeff0_14*dmats0[14][40] + coeff0_15*dmats0[15][40] + coeff0_16*dmats0[16][40] + coeff0_17*dmats0[17][40] + coeff0_18*dmats0[18][40] + coeff0_19*dmats0[19][40] + coeff0_20*dmats0[20][40] + coeff0_21*dmats0[21][40] + coeff0_22*dmats0[22][40] + coeff0_23*dmats0[23][40] + coeff0_24*dmats0[24][40] + coeff0_25*dmats0[25][40] + coeff0_26*dmats0[26][40] + coeff0_27*dmats0[27][40] + coeff0_28*dmats0[28][40] + coeff0_29*dmats0[29][40] + coeff0_30*dmats0[30][40] + coeff0_31*dmats0[31][40] + coeff0_32*dmats0[32][40] + coeff0_33*dmats0[33][40] + coeff0_34*dmats0[34][40] + coeff0_35*dmats0[35][40] + coeff0_36*dmats0[36][40] + coeff0_37*dmats0[37][40] + coeff0_38*dmats0[38][40] + coeff0_39*dmats0[39][40] + coeff0_40*dmats0[40][40] + coeff0_41*dmats0[41][40] + coeff0_42*dmats0[42][40] + coeff0_43*dmats0[43][40] + coeff0_44*dmats0[44][40] + coeff0_45*dmats0[45][40] + coeff0_46*dmats0[46][40] + coeff0_47*dmats0[47][40] + coeff0_48*dmats0[48][40] + coeff0_49*dmats0[49][40] + coeff0_50*dmats0[50][40] + coeff0_51*dmats0[51][40] + coeff0_52*dmats0[52][40] + coeff0_53*dmats0[53][40] + coeff0_54*dmats0[54][40] + coeff0_55*dmats0[55][40];
          new_coeff0_41 = coeff0_0*dmats0[0][41] + coeff0_1*dmats0[1][41] + coeff0_2*dmats0[2][41] + coeff0_3*dmats0[3][41] + coeff0_4*dmats0[4][41] + coeff0_5*dmats0[5][41] + coeff0_6*dmats0[6][41] + coeff0_7*dmats0[7][41] + coeff0_8*dmats0[8][41] + coeff0_9*dmats0[9][41] + coeff0_10*dmats0[10][41] + coeff0_11*dmats0[11][41] + coeff0_12*dmats0[12][41] + coeff0_13*dmats0[13][41] + coeff0_14*dmats0[14][41] + coeff0_15*dmats0[15][41] + coeff0_16*dmats0[16][41] + coeff0_17*dmats0[17][41] + coeff0_18*dmats0[18][41] + coeff0_19*dmats0[19][41] + coeff0_20*dmats0[20][41] + coeff0_21*dmats0[21][41] + coeff0_22*dmats0[22][41] + coeff0_23*dmats0[23][41] + coeff0_24*dmats0[24][41] + coeff0_25*dmats0[25][41] + coeff0_26*dmats0[26][41] + coeff0_27*dmats0[27][41] + coeff0_28*dmats0[28][41] + coeff0_29*dmats0[29][41] + coeff0_30*dmats0[30][41] + coeff0_31*dmats0[31][41] + coeff0_32*dmats0[32][41] + coeff0_33*dmats0[33][41] + coeff0_34*dmats0[34][41] + coeff0_35*dmats0[35][41] + coeff0_36*dmats0[36][41] + coeff0_37*dmats0[37][41] + coeff0_38*dmats0[38][41] + coeff0_39*dmats0[39][41] + coeff0_40*dmats0[40][41] + coeff0_41*dmats0[41][41] + coeff0_42*dmats0[42][41] + coeff0_43*dmats0[43][41] + coeff0_44*dmats0[44][41] + coeff0_45*dmats0[45][41] + coeff0_46*dmats0[46][41] + coeff0_47*dmats0[47][41] + coeff0_48*dmats0[48][41] + coeff0_49*dmats0[49][41] + coeff0_50*dmats0[50][41] + coeff0_51*dmats0[51][41] + coeff0_52*dmats0[52][41] + coeff0_53*dmats0[53][41] + coeff0_54*dmats0[54][41] + coeff0_55*dmats0[55][41];
          new_coeff0_42 = coeff0_0*dmats0[0][42] + coeff0_1*dmats0[1][42] + coeff0_2*dmats0[2][42] + coeff0_3*dmats0[3][42] + coeff0_4*dmats0[4][42] + coeff0_5*dmats0[5][42] + coeff0_6*dmats0[6][42] + coeff0_7*dmats0[7][42] + coeff0_8*dmats0[8][42] + coeff0_9*dmats0[9][42] + coeff0_10*dmats0[10][42] + coeff0_11*dmats0[11][42] + coeff0_12*dmats0[12][42] + coeff0_13*dmats0[13][42] + coeff0_14*dmats0[14][42] + coeff0_15*dmats0[15][42] + coeff0_16*dmats0[16][42] + coeff0_17*dmats0[17][42] + coeff0_18*dmats0[18][42] + coeff0_19*dmats0[19][42] + coeff0_20*dmats0[20][42] + coeff0_21*dmats0[21][42] + coeff0_22*dmats0[22][42] + coeff0_23*dmats0[23][42] + coeff0_24*dmats0[24][42] + coeff0_25*dmats0[25][42] + coeff0_26*dmats0[26][42] + coeff0_27*dmats0[27][42] + coeff0_28*dmats0[28][42] + coeff0_29*dmats0[29][42] + coeff0_30*dmats0[30][42] + coeff0_31*dmats0[31][42] + coeff0_32*dmats0[32][42] + coeff0_33*dmats0[33][42] + coeff0_34*dmats0[34][42] + coeff0_35*dmats0[35][42] + coeff0_36*dmats0[36][42] + coeff0_37*dmats0[37][42] + coeff0_38*dmats0[38][42] + coeff0_39*dmats0[39][42] + coeff0_40*dmats0[40][42] + coeff0_41*dmats0[41][42] + coeff0_42*dmats0[42][42] + coeff0_43*dmats0[43][42] + coeff0_44*dmats0[44][42] + coeff0_45*dmats0[45][42] + coeff0_46*dmats0[46][42] + coeff0_47*dmats0[47][42] + coeff0_48*dmats0[48][42] + coeff0_49*dmats0[49][42] + coeff0_50*dmats0[50][42] + coeff0_51*dmats0[51][42] + coeff0_52*dmats0[52][42] + coeff0_53*dmats0[53][42] + coeff0_54*dmats0[54][42] + coeff0_55*dmats0[55][42];
          new_coeff0_43 = coeff0_0*dmats0[0][43] + coeff0_1*dmats0[1][43] + coeff0_2*dmats0[2][43] + coeff0_3*dmats0[3][43] + coeff0_4*dmats0[4][43] + coeff0_5*dmats0[5][43] + coeff0_6*dmats0[6][43] + coeff0_7*dmats0[7][43] + coeff0_8*dmats0[8][43] + coeff0_9*dmats0[9][43] + coeff0_10*dmats0[10][43] + coeff0_11*dmats0[11][43] + coeff0_12*dmats0[12][43] + coeff0_13*dmats0[13][43] + coeff0_14*dmats0[14][43] + coeff0_15*dmats0[15][43] + coeff0_16*dmats0[16][43] + coeff0_17*dmats0[17][43] + coeff0_18*dmats0[18][43] + coeff0_19*dmats0[19][43] + coeff0_20*dmats0[20][43] + coeff0_21*dmats0[21][43] + coeff0_22*dmats0[22][43] + coeff0_23*dmats0[23][43] + coeff0_24*dmats0[24][43] + coeff0_25*dmats0[25][43] + coeff0_26*dmats0[26][43] + coeff0_27*dmats0[27][43] + coeff0_28*dmats0[28][43] + coeff0_29*dmats0[29][43] + coeff0_30*dmats0[30][43] + coeff0_31*dmats0[31][43] + coeff0_32*dmats0[32][43] + coeff0_33*dmats0[33][43] + coeff0_34*dmats0[34][43] + coeff0_35*dmats0[35][43] + coeff0_36*dmats0[36][43] + coeff0_37*dmats0[37][43] + coeff0_38*dmats0[38][43] + coeff0_39*dmats0[39][43] + coeff0_40*dmats0[40][43] + coeff0_41*dmats0[41][43] + coeff0_42*dmats0[42][43] + coeff0_43*dmats0[43][43] + coeff0_44*dmats0[44][43] + coeff0_45*dmats0[45][43] + coeff0_46*dmats0[46][43] + coeff0_47*dmats0[47][43] + coeff0_48*dmats0[48][43] + coeff0_49*dmats0[49][43] + coeff0_50*dmats0[50][43] + coeff0_51*dmats0[51][43] + coeff0_52*dmats0[52][43] + coeff0_53*dmats0[53][43] + coeff0_54*dmats0[54][43] + coeff0_55*dmats0[55][43];
          new_coeff0_44 = coeff0_0*dmats0[0][44] + coeff0_1*dmats0[1][44] + coeff0_2*dmats0[2][44] + coeff0_3*dmats0[3][44] + coeff0_4*dmats0[4][44] + coeff0_5*dmats0[5][44] + coeff0_6*dmats0[6][44] + coeff0_7*dmats0[7][44] + coeff0_8*dmats0[8][44] + coeff0_9*dmats0[9][44] + coeff0_10*dmats0[10][44] + coeff0_11*dmats0[11][44] + coeff0_12*dmats0[12][44] + coeff0_13*dmats0[13][44] + coeff0_14*dmats0[14][44] + coeff0_15*dmats0[15][44] + coeff0_16*dmats0[16][44] + coeff0_17*dmats0[17][44] + coeff0_18*dmats0[18][44] + coeff0_19*dmats0[19][44] + coeff0_20*dmats0[20][44] + coeff0_21*dmats0[21][44] + coeff0_22*dmats0[22][44] + coeff0_23*dmats0[23][44] + coeff0_24*dmats0[24][44] + coeff0_25*dmats0[25][44] + coeff0_26*dmats0[26][44] + coeff0_27*dmats0[27][44] + coeff0_28*dmats0[28][44] + coeff0_29*dmats0[29][44] + coeff0_30*dmats0[30][44] + coeff0_31*dmats0[31][44] + coeff0_32*dmats0[32][44] + coeff0_33*dmats0[33][44] + coeff0_34*dmats0[34][44] + coeff0_35*dmats0[35][44] + coeff0_36*dmats0[36][44] + coeff0_37*dmats0[37][44] + coeff0_38*dmats0[38][44] + coeff0_39*dmats0[39][44] + coeff0_40*dmats0[40][44] + coeff0_41*dmats0[41][44] + coeff0_42*dmats0[42][44] + coeff0_43*dmats0[43][44] + coeff0_44*dmats0[44][44] + coeff0_45*dmats0[45][44] + coeff0_46*dmats0[46][44] + coeff0_47*dmats0[47][44] + coeff0_48*dmats0[48][44] + coeff0_49*dmats0[49][44] + coeff0_50*dmats0[50][44] + coeff0_51*dmats0[51][44] + coeff0_52*dmats0[52][44] + coeff0_53*dmats0[53][44] + coeff0_54*dmats0[54][44] + coeff0_55*dmats0[55][44];
          new_coeff0_45 = coeff0_0*dmats0[0][45] + coeff0_1*dmats0[1][45] + coeff0_2*dmats0[2][45] + coeff0_3*dmats0[3][45] + coeff0_4*dmats0[4][45] + coeff0_5*dmats0[5][45] + coeff0_6*dmats0[6][45] + coeff0_7*dmats0[7][45] + coeff0_8*dmats0[8][45] + coeff0_9*dmats0[9][45] + coeff0_10*dmats0[10][45] + coeff0_11*dmats0[11][45] + coeff0_12*dmats0[12][45] + coeff0_13*dmats0[13][45] + coeff0_14*dmats0[14][45] + coeff0_15*dmats0[15][45] + coeff0_16*dmats0[16][45] + coeff0_17*dmats0[17][45] + coeff0_18*dmats0[18][45] + coeff0_19*dmats0[19][45] + coeff0_20*dmats0[20][45] + coeff0_21*dmats0[21][45] + coeff0_22*dmats0[22][45] + coeff0_23*dmats0[23][45] + coeff0_24*dmats0[24][45] + coeff0_25*dmats0[25][45] + coeff0_26*dmats0[26][45] + coeff0_27*dmats0[27][45] + coeff0_28*dmats0[28][45] + coeff0_29*dmats0[29][45] + coeff0_30*dmats0[30][45] + coeff0_31*dmats0[31][45] + coeff0_32*dmats0[32][45] + coeff0_33*dmats0[33][45] + coeff0_34*dmats0[34][45] + coeff0_35*dmats0[35][45] + coeff0_36*dmats0[36][45] + coeff0_37*dmats0[37][45] + coeff0_38*dmats0[38][45] + coeff0_39*dmats0[39][45] + coeff0_40*dmats0[40][45] + coeff0_41*dmats0[41][45] + coeff0_42*dmats0[42][45] + coeff0_43*dmats0[43][45] + coeff0_44*dmats0[44][45] + coeff0_45*dmats0[45][45] + coeff0_46*dmats0[46][45] + coeff0_47*dmats0[47][45] + coeff0_48*dmats0[48][45] + coeff0_49*dmats0[49][45] + coeff0_50*dmats0[50][45] + coeff0_51*dmats0[51][45] + coeff0_52*dmats0[52][45] + coeff0_53*dmats0[53][45] + coeff0_54*dmats0[54][45] + coeff0_55*dmats0[55][45];
          new_coeff0_46 = coeff0_0*dmats0[0][46] + coeff0_1*dmats0[1][46] + coeff0_2*dmats0[2][46] + coeff0_3*dmats0[3][46] + coeff0_4*dmats0[4][46] + coeff0_5*dmats0[5][46] + coeff0_6*dmats0[6][46] + coeff0_7*dmats0[7][46] + coeff0_8*dmats0[8][46] + coeff0_9*dmats0[9][46] + coeff0_10*dmats0[10][46] + coeff0_11*dmats0[11][46] + coeff0_12*dmats0[12][46] + coeff0_13*dmats0[13][46] + coeff0_14*dmats0[14][46] + coeff0_15*dmats0[15][46] + coeff0_16*dmats0[16][46] + coeff0_17*dmats0[17][46] + coeff0_18*dmats0[18][46] + coeff0_19*dmats0[19][46] + coeff0_20*dmats0[20][46] + coeff0_21*dmats0[21][46] + coeff0_22*dmats0[22][46] + coeff0_23*dmats0[23][46] + coeff0_24*dmats0[24][46] + coeff0_25*dmats0[25][46] + coeff0_26*dmats0[26][46] + coeff0_27*dmats0[27][46] + coeff0_28*dmats0[28][46] + coeff0_29*dmats0[29][46] + coeff0_30*dmats0[30][46] + coeff0_31*dmats0[31][46] + coeff0_32*dmats0[32][46] + coeff0_33*dmats0[33][46] + coeff0_34*dmats0[34][46] + coeff0_35*dmats0[35][46] + coeff0_36*dmats0[36][46] + coeff0_37*dmats0[37][46] + coeff0_38*dmats0[38][46] + coeff0_39*dmats0[39][46] + coeff0_40*dmats0[40][46] + coeff0_41*dmats0[41][46] + coeff0_42*dmats0[42][46] + coeff0_43*dmats0[43][46] + coeff0_44*dmats0[44][46] + coeff0_45*dmats0[45][46] + coeff0_46*dmats0[46][46] + coeff0_47*dmats0[47][46] + coeff0_48*dmats0[48][46] + coeff0_49*dmats0[49][46] + coeff0_50*dmats0[50][46] + coeff0_51*dmats0[51][46] + coeff0_52*dmats0[52][46] + coeff0_53*dmats0[53][46] + coeff0_54*dmats0[54][46] + coeff0_55*dmats0[55][46];
          new_coeff0_47 = coeff0_0*dmats0[0][47] + coeff0_1*dmats0[1][47] + coeff0_2*dmats0[2][47] + coeff0_3*dmats0[3][47] + coeff0_4*dmats0[4][47] + coeff0_5*dmats0[5][47] + coeff0_6*dmats0[6][47] + coeff0_7*dmats0[7][47] + coeff0_8*dmats0[8][47] + coeff0_9*dmats0[9][47] + coeff0_10*dmats0[10][47] + coeff0_11*dmats0[11][47] + coeff0_12*dmats0[12][47] + coeff0_13*dmats0[13][47] + coeff0_14*dmats0[14][47] + coeff0_15*dmats0[15][47] + coeff0_16*dmats0[16][47] + coeff0_17*dmats0[17][47] + coeff0_18*dmats0[18][47] + coeff0_19*dmats0[19][47] + coeff0_20*dmats0[20][47] + coeff0_21*dmats0[21][47] + coeff0_22*dmats0[22][47] + coeff0_23*dmats0[23][47] + coeff0_24*dmats0[24][47] + coeff0_25*dmats0[25][47] + coeff0_26*dmats0[26][47] + coeff0_27*dmats0[27][47] + coeff0_28*dmats0[28][47] + coeff0_29*dmats0[29][47] + coeff0_30*dmats0[30][47] + coeff0_31*dmats0[31][47] + coeff0_32*dmats0[32][47] + coeff0_33*dmats0[33][47] + coeff0_34*dmats0[34][47] + coeff0_35*dmats0[35][47] + coeff0_36*dmats0[36][47] + coeff0_37*dmats0[37][47] + coeff0_38*dmats0[38][47] + coeff0_39*dmats0[39][47] + coeff0_40*dmats0[40][47] + coeff0_41*dmats0[41][47] + coeff0_42*dmats0[42][47] + coeff0_43*dmats0[43][47] + coeff0_44*dmats0[44][47] + coeff0_45*dmats0[45][47] + coeff0_46*dmats0[46][47] + coeff0_47*dmats0[47][47] + coeff0_48*dmats0[48][47] + coeff0_49*dmats0[49][47] + coeff0_50*dmats0[50][47] + coeff0_51*dmats0[51][47] + coeff0_52*dmats0[52][47] + coeff0_53*dmats0[53][47] + coeff0_54*dmats0[54][47] + coeff0_55*dmats0[55][47];
          new_coeff0_48 = coeff0_0*dmats0[0][48] + coeff0_1*dmats0[1][48] + coeff0_2*dmats0[2][48] + coeff0_3*dmats0[3][48] + coeff0_4*dmats0[4][48] + coeff0_5*dmats0[5][48] + coeff0_6*dmats0[6][48] + coeff0_7*dmats0[7][48] + coeff0_8*dmats0[8][48] + coeff0_9*dmats0[9][48] + coeff0_10*dmats0[10][48] + coeff0_11*dmats0[11][48] + coeff0_12*dmats0[12][48] + coeff0_13*dmats0[13][48] + coeff0_14*dmats0[14][48] + coeff0_15*dmats0[15][48] + coeff0_16*dmats0[16][48] + coeff0_17*dmats0[17][48] + coeff0_18*dmats0[18][48] + coeff0_19*dmats0[19][48] + coeff0_20*dmats0[20][48] + coeff0_21*dmats0[21][48] + coeff0_22*dmats0[22][48] + coeff0_23*dmats0[23][48] + coeff0_24*dmats0[24][48] + coeff0_25*dmats0[25][48] + coeff0_26*dmats0[26][48] + coeff0_27*dmats0[27][48] + coeff0_28*dmats0[28][48] + coeff0_29*dmats0[29][48] + coeff0_30*dmats0[30][48] + coeff0_31*dmats0[31][48] + coeff0_32*dmats0[32][48] + coeff0_33*dmats0[33][48] + coeff0_34*dmats0[34][48] + coeff0_35*dmats0[35][48] + coeff0_36*dmats0[36][48] + coeff0_37*dmats0[37][48] + coeff0_38*dmats0[38][48] + coeff0_39*dmats0[39][48] + coeff0_40*dmats0[40][48] + coeff0_41*dmats0[41][48] + coeff0_42*dmats0[42][48] + coeff0_43*dmats0[43][48] + coeff0_44*dmats0[44][48] + coeff0_45*dmats0[45][48] + coeff0_46*dmats0[46][48] + coeff0_47*dmats0[47][48] + coeff0_48*dmats0[48][48] + coeff0_49*dmats0[49][48] + coeff0_50*dmats0[50][48] + coeff0_51*dmats0[51][48] + coeff0_52*dmats0[52][48] + coeff0_53*dmats0[53][48] + coeff0_54*dmats0[54][48] + coeff0_55*dmats0[55][48];
          new_coeff0_49 = coeff0_0*dmats0[0][49] + coeff0_1*dmats0[1][49] + coeff0_2*dmats0[2][49] + coeff0_3*dmats0[3][49] + coeff0_4*dmats0[4][49] + coeff0_5*dmats0[5][49] + coeff0_6*dmats0[6][49] + coeff0_7*dmats0[7][49] + coeff0_8*dmats0[8][49] + coeff0_9*dmats0[9][49] + coeff0_10*dmats0[10][49] + coeff0_11*dmats0[11][49] + coeff0_12*dmats0[12][49] + coeff0_13*dmats0[13][49] + coeff0_14*dmats0[14][49] + coeff0_15*dmats0[15][49] + coeff0_16*dmats0[16][49] + coeff0_17*dmats0[17][49] + coeff0_18*dmats0[18][49] + coeff0_19*dmats0[19][49] + coeff0_20*dmats0[20][49] + coeff0_21*dmats0[21][49] + coeff0_22*dmats0[22][49] + coeff0_23*dmats0[23][49] + coeff0_24*dmats0[24][49] + coeff0_25*dmats0[25][49] + coeff0_26*dmats0[26][49] + coeff0_27*dmats0[27][49] + coeff0_28*dmats0[28][49] + coeff0_29*dmats0[29][49] + coeff0_30*dmats0[30][49] + coeff0_31*dmats0[31][49] + coeff0_32*dmats0[32][49] + coeff0_33*dmats0[33][49] + coeff0_34*dmats0[34][49] + coeff0_35*dmats0[35][49] + coeff0_36*dmats0[36][49] + coeff0_37*dmats0[37][49] + coeff0_38*dmats0[38][49] + coeff0_39*dmats0[39][49] + coeff0_40*dmats0[40][49] + coeff0_41*dmats0[41][49] + coeff0_42*dmats0[42][49] + coeff0_43*dmats0[43][49] + coeff0_44*dmats0[44][49] + coeff0_45*dmats0[45][49] + coeff0_46*dmats0[46][49] + coeff0_47*dmats0[47][49] + coeff0_48*dmats0[48][49] + coeff0_49*dmats0[49][49] + coeff0_50*dmats0[50][49] + coeff0_51*dmats0[51][49] + coeff0_52*dmats0[52][49] + coeff0_53*dmats0[53][49] + coeff0_54*dmats0[54][49] + coeff0_55*dmats0[55][49];
          new_coeff0_50 = coeff0_0*dmats0[0][50] + coeff0_1*dmats0[1][50] + coeff0_2*dmats0[2][50] + coeff0_3*dmats0[3][50] + coeff0_4*dmats0[4][50] + coeff0_5*dmats0[5][50] + coeff0_6*dmats0[6][50] + coeff0_7*dmats0[7][50] + coeff0_8*dmats0[8][50] + coeff0_9*dmats0[9][50] + coeff0_10*dmats0[10][50] + coeff0_11*dmats0[11][50] + coeff0_12*dmats0[12][50] + coeff0_13*dmats0[13][50] + coeff0_14*dmats0[14][50] + coeff0_15*dmats0[15][50] + coeff0_16*dmats0[16][50] + coeff0_17*dmats0[17][50] + coeff0_18*dmats0[18][50] + coeff0_19*dmats0[19][50] + coeff0_20*dmats0[20][50] + coeff0_21*dmats0[21][50] + coeff0_22*dmats0[22][50] + coeff0_23*dmats0[23][50] + coeff0_24*dmats0[24][50] + coeff0_25*dmats0[25][50] + coeff0_26*dmats0[26][50] + coeff0_27*dmats0[27][50] + coeff0_28*dmats0[28][50] + coeff0_29*dmats0[29][50] + coeff0_30*dmats0[30][50] + coeff0_31*dmats0[31][50] + coeff0_32*dmats0[32][50] + coeff0_33*dmats0[33][50] + coeff0_34*dmats0[34][50] + coeff0_35*dmats0[35][50] + coeff0_36*dmats0[36][50] + coeff0_37*dmats0[37][50] + coeff0_38*dmats0[38][50] + coeff0_39*dmats0[39][50] + coeff0_40*dmats0[40][50] + coeff0_41*dmats0[41][50] + coeff0_42*dmats0[42][50] + coeff0_43*dmats0[43][50] + coeff0_44*dmats0[44][50] + coeff0_45*dmats0[45][50] + coeff0_46*dmats0[46][50] + coeff0_47*dmats0[47][50] + coeff0_48*dmats0[48][50] + coeff0_49*dmats0[49][50] + coeff0_50*dmats0[50][50] + coeff0_51*dmats0[51][50] + coeff0_52*dmats0[52][50] + coeff0_53*dmats0[53][50] + coeff0_54*dmats0[54][50] + coeff0_55*dmats0[55][50];
          new_coeff0_51 = coeff0_0*dmats0[0][51] + coeff0_1*dmats0[1][51] + coeff0_2*dmats0[2][51] + coeff0_3*dmats0[3][51] + coeff0_4*dmats0[4][51] + coeff0_5*dmats0[5][51] + coeff0_6*dmats0[6][51] + coeff0_7*dmats0[7][51] + coeff0_8*dmats0[8][51] + coeff0_9*dmats0[9][51] + coeff0_10*dmats0[10][51] + coeff0_11*dmats0[11][51] + coeff0_12*dmats0[12][51] + coeff0_13*dmats0[13][51] + coeff0_14*dmats0[14][51] + coeff0_15*dmats0[15][51] + coeff0_16*dmats0[16][51] + coeff0_17*dmats0[17][51] + coeff0_18*dmats0[18][51] + coeff0_19*dmats0[19][51] + coeff0_20*dmats0[20][51] + coeff0_21*dmats0[21][51] + coeff0_22*dmats0[22][51] + coeff0_23*dmats0[23][51] + coeff0_24*dmats0[24][51] + coeff0_25*dmats0[25][51] + coeff0_26*dmats0[26][51] + coeff0_27*dmats0[27][51] + coeff0_28*dmats0[28][51] + coeff0_29*dmats0[29][51] + coeff0_30*dmats0[30][51] + coeff0_31*dmats0[31][51] + coeff0_32*dmats0[32][51] + coeff0_33*dmats0[33][51] + coeff0_34*dmats0[34][51] + coeff0_35*dmats0[35][51] + coeff0_36*dmats0[36][51] + coeff0_37*dmats0[37][51] + coeff0_38*dmats0[38][51] + coeff0_39*dmats0[39][51] + coeff0_40*dmats0[40][51] + coeff0_41*dmats0[41][51] + coeff0_42*dmats0[42][51] + coeff0_43*dmats0[43][51] + coeff0_44*dmats0[44][51] + coeff0_45*dmats0[45][51] + coeff0_46*dmats0[46][51] + coeff0_47*dmats0[47][51] + coeff0_48*dmats0[48][51] + coeff0_49*dmats0[49][51] + coeff0_50*dmats0[50][51] + coeff0_51*dmats0[51][51] + coeff0_52*dmats0[52][51] + coeff0_53*dmats0[53][51] + coeff0_54*dmats0[54][51] + coeff0_55*dmats0[55][51];
          new_coeff0_52 = coeff0_0*dmats0[0][52] + coeff0_1*dmats0[1][52] + coeff0_2*dmats0[2][52] + coeff0_3*dmats0[3][52] + coeff0_4*dmats0[4][52] + coeff0_5*dmats0[5][52] + coeff0_6*dmats0[6][52] + coeff0_7*dmats0[7][52] + coeff0_8*dmats0[8][52] + coeff0_9*dmats0[9][52] + coeff0_10*dmats0[10][52] + coeff0_11*dmats0[11][52] + coeff0_12*dmats0[12][52] + coeff0_13*dmats0[13][52] + coeff0_14*dmats0[14][52] + coeff0_15*dmats0[15][52] + coeff0_16*dmats0[16][52] + coeff0_17*dmats0[17][52] + coeff0_18*dmats0[18][52] + coeff0_19*dmats0[19][52] + coeff0_20*dmats0[20][52] + coeff0_21*dmats0[21][52] + coeff0_22*dmats0[22][52] + coeff0_23*dmats0[23][52] + coeff0_24*dmats0[24][52] + coeff0_25*dmats0[25][52] + coeff0_26*dmats0[26][52] + coeff0_27*dmats0[27][52] + coeff0_28*dmats0[28][52] + coeff0_29*dmats0[29][52] + coeff0_30*dmats0[30][52] + coeff0_31*dmats0[31][52] + coeff0_32*dmats0[32][52] + coeff0_33*dmats0[33][52] + coeff0_34*dmats0[34][52] + coeff0_35*dmats0[35][52] + coeff0_36*dmats0[36][52] + coeff0_37*dmats0[37][52] + coeff0_38*dmats0[38][52] + coeff0_39*dmats0[39][52] + coeff0_40*dmats0[40][52] + coeff0_41*dmats0[41][52] + coeff0_42*dmats0[42][52] + coeff0_43*dmats0[43][52] + coeff0_44*dmats0[44][52] + coeff0_45*dmats0[45][52] + coeff0_46*dmats0[46][52] + coeff0_47*dmats0[47][52] + coeff0_48*dmats0[48][52] + coeff0_49*dmats0[49][52] + coeff0_50*dmats0[50][52] + coeff0_51*dmats0[51][52] + coeff0_52*dmats0[52][52] + coeff0_53*dmats0[53][52] + coeff0_54*dmats0[54][52] + coeff0_55*dmats0[55][52];
          new_coeff0_53 = coeff0_0*dmats0[0][53] + coeff0_1*dmats0[1][53] + coeff0_2*dmats0[2][53] + coeff0_3*dmats0[3][53] + coeff0_4*dmats0[4][53] + coeff0_5*dmats0[5][53] + coeff0_6*dmats0[6][53] + coeff0_7*dmats0[7][53] + coeff0_8*dmats0[8][53] + coeff0_9*dmats0[9][53] + coeff0_10*dmats0[10][53] + coeff0_11*dmats0[11][53] + coeff0_12*dmats0[12][53] + coeff0_13*dmats0[13][53] + coeff0_14*dmats0[14][53] + coeff0_15*dmats0[15][53] + coeff0_16*dmats0[16][53] + coeff0_17*dmats0[17][53] + coeff0_18*dmats0[18][53] + coeff0_19*dmats0[19][53] + coeff0_20*dmats0[20][53] + coeff0_21*dmats0[21][53] + coeff0_22*dmats0[22][53] + coeff0_23*dmats0[23][53] + coeff0_24*dmats0[24][53] + coeff0_25*dmats0[25][53] + coeff0_26*dmats0[26][53] + coeff0_27*dmats0[27][53] + coeff0_28*dmats0[28][53] + coeff0_29*dmats0[29][53] + coeff0_30*dmats0[30][53] + coeff0_31*dmats0[31][53] + coeff0_32*dmats0[32][53] + coeff0_33*dmats0[33][53] + coeff0_34*dmats0[34][53] + coeff0_35*dmats0[35][53] + coeff0_36*dmats0[36][53] + coeff0_37*dmats0[37][53] + coeff0_38*dmats0[38][53] + coeff0_39*dmats0[39][53] + coeff0_40*dmats0[40][53] + coeff0_41*dmats0[41][53] + coeff0_42*dmats0[42][53] + coeff0_43*dmats0[43][53] + coeff0_44*dmats0[44][53] + coeff0_45*dmats0[45][53] + coeff0_46*dmats0[46][53] + coeff0_47*dmats0[47][53] + coeff0_48*dmats0[48][53] + coeff0_49*dmats0[49][53] + coeff0_50*dmats0[50][53] + coeff0_51*dmats0[51][53] + coeff0_52*dmats0[52][53] + coeff0_53*dmats0[53][53] + coeff0_54*dmats0[54][53] + coeff0_55*dmats0[55][53];
          new_coeff0_54 = coeff0_0*dmats0[0][54] + coeff0_1*dmats0[1][54] + coeff0_2*dmats0[2][54] + coeff0_3*dmats0[3][54] + coeff0_4*dmats0[4][54] + coeff0_5*dmats0[5][54] + coeff0_6*dmats0[6][54] + coeff0_7*dmats0[7][54] + coeff0_8*dmats0[8][54] + coeff0_9*dmats0[9][54] + coeff0_10*dmats0[10][54] + coeff0_11*dmats0[11][54] + coeff0_12*dmats0[12][54] + coeff0_13*dmats0[13][54] + coeff0_14*dmats0[14][54] + coeff0_15*dmats0[15][54] + coeff0_16*dmats0[16][54] + coeff0_17*dmats0[17][54] + coeff0_18*dmats0[18][54] + coeff0_19*dmats0[19][54] + coeff0_20*dmats0[20][54] + coeff0_21*dmats0[21][54] + coeff0_22*dmats0[22][54] + coeff0_23*dmats0[23][54] + coeff0_24*dmats0[24][54] + coeff0_25*dmats0[25][54] + coeff0_26*dmats0[26][54] + coeff0_27*dmats0[27][54] + coeff0_28*dmats0[28][54] + coeff0_29*dmats0[29][54] + coeff0_30*dmats0[30][54] + coeff0_31*dmats0[31][54] + coeff0_32*dmats0[32][54] + coeff0_33*dmats0[33][54] + coeff0_34*dmats0[34][54] + coeff0_35*dmats0[35][54] + coeff0_36*dmats0[36][54] + coeff0_37*dmats0[37][54] + coeff0_38*dmats0[38][54] + coeff0_39*dmats0[39][54] + coeff0_40*dmats0[40][54] + coeff0_41*dmats0[41][54] + coeff0_42*dmats0[42][54] + coeff0_43*dmats0[43][54] + coeff0_44*dmats0[44][54] + coeff0_45*dmats0[45][54] + coeff0_46*dmats0[46][54] + coeff0_47*dmats0[47][54] + coeff0_48*dmats0[48][54] + coeff0_49*dmats0[49][54] + coeff0_50*dmats0[50][54] + coeff0_51*dmats0[51][54] + coeff0_52*dmats0[52][54] + coeff0_53*dmats0[53][54] + coeff0_54*dmats0[54][54] + coeff0_55*dmats0[55][54];
          new_coeff0_55 = coeff0_0*dmats0[0][55] + coeff0_1*dmats0[1][55] + coeff0_2*dmats0[2][55] + coeff0_3*dmats0[3][55] + coeff0_4*dmats0[4][55] + coeff0_5*dmats0[5][55] + coeff0_6*dmats0[6][55] + coeff0_7*dmats0[7][55] + coeff0_8*dmats0[8][55] + coeff0_9*dmats0[9][55] + coeff0_10*dmats0[10][55] + coeff0_11*dmats0[11][55] + coeff0_12*dmats0[12][55] + coeff0_13*dmats0[13][55] + coeff0_14*dmats0[14][55] + coeff0_15*dmats0[15][55] + coeff0_16*dmats0[16][55] + coeff0_17*dmats0[17][55] + coeff0_18*dmats0[18][55] + coeff0_19*dmats0[19][55] + coeff0_20*dmats0[20][55] + coeff0_21*dmats0[21][55] + coeff0_22*dmats0[22][55] + coeff0_23*dmats0[23][55] + coeff0_24*dmats0[24][55] + coeff0_25*dmats0[25][55] + coeff0_26*dmats0[26][55] + coeff0_27*dmats0[27][55] + coeff0_28*dmats0[28][55] + coeff0_29*dmats0[29][55] + coeff0_30*dmats0[30][55] + coeff0_31*dmats0[31][55] + coeff0_32*dmats0[32][55] + coeff0_33*dmats0[33][55] + coeff0_34*dmats0[34][55] + coeff0_35*dmats0[35][55] + coeff0_36*dmats0[36][55] + coeff0_37*dmats0[37][55] + coeff0_38*dmats0[38][55] + coeff0_39*dmats0[39][55] + coeff0_40*dmats0[40][55] + coeff0_41*dmats0[41][55] + coeff0_42*dmats0[42][55] + coeff0_43*dmats0[43][55] + coeff0_44*dmats0[44][55] + coeff0_45*dmats0[45][55] + coeff0_46*dmats0[46][55] + coeff0_47*dmats0[47][55] + coeff0_48*dmats0[48][55] + coeff0_49*dmats0[49][55] + coeff0_50*dmats0[50][55] + coeff0_51*dmats0[51][55] + coeff0_52*dmats0[52][55] + coeff0_53*dmats0[53][55] + coeff0_54*dmats0[54][55] + coeff0_55*dmats0[55][55];
        }
        if(combinations[deriv_num][j] == 1)
        {
          new_coeff0_0 = coeff0_0*dmats1[0][0] + coeff0_1*dmats1[1][0] + coeff0_2*dmats1[2][0] + coeff0_3*dmats1[3][0] + coeff0_4*dmats1[4][0] + coeff0_5*dmats1[5][0] + coeff0_6*dmats1[6][0] + coeff0_7*dmats1[7][0] + coeff0_8*dmats1[8][0] + coeff0_9*dmats1[9][0] + coeff0_10*dmats1[10][0] + coeff0_11*dmats1[11][0] + coeff0_12*dmats1[12][0] + coeff0_13*dmats1[13][0] + coeff0_14*dmats1[14][0] + coeff0_15*dmats1[15][0] + coeff0_16*dmats1[16][0] + coeff0_17*dmats1[17][0] + coeff0_18*dmats1[18][0] + coeff0_19*dmats1[19][0] + coeff0_20*dmats1[20][0] + coeff0_21*dmats1[21][0] + coeff0_22*dmats1[22][0] + coeff0_23*dmats1[23][0] + coeff0_24*dmats1[24][0] + coeff0_25*dmats1[25][0] + coeff0_26*dmats1[26][0] + coeff0_27*dmats1[27][0] + coeff0_28*dmats1[28][0] + coeff0_29*dmats1[29][0] + coeff0_30*dmats1[30][0] + coeff0_31*dmats1[31][0] + coeff0_32*dmats1[32][0] + coeff0_33*dmats1[33][0] + coeff0_34*dmats1[34][0] + coeff0_35*dmats1[35][0] + coeff0_36*dmats1[36][0] + coeff0_37*dmats1[37][0] + coeff0_38*dmats1[38][0] + coeff0_39*dmats1[39][0] + coeff0_40*dmats1[40][0] + coeff0_41*dmats1[41][0] + coeff0_42*dmats1[42][0] + coeff0_43*dmats1[43][0] + coeff0_44*dmats1[44][0] + coeff0_45*dmats1[45][0] + coeff0_46*dmats1[46][0] + coeff0_47*dmats1[47][0] + coeff0_48*dmats1[48][0] + coeff0_49*dmats1[49][0] + coeff0_50*dmats1[50][0] + coeff0_51*dmats1[51][0] + coeff0_52*dmats1[52][0] + coeff0_53*dmats1[53][0] + coeff0_54*dmats1[54][0] + coeff0_55*dmats1[55][0];
          new_coeff0_1 = coeff0_0*dmats1[0][1] + coeff0_1*dmats1[1][1] + coeff0_2*dmats1[2][1] + coeff0_3*dmats1[3][1] + coeff0_4*dmats1[4][1] + coeff0_5*dmats1[5][1] + coeff0_6*dmats1[6][1] + coeff0_7*dmats1[7][1] + coeff0_8*dmats1[8][1] + coeff0_9*dmats1[9][1] + coeff0_10*dmats1[10][1] + coeff0_11*dmats1[11][1] + coeff0_12*dmats1[12][1] + coeff0_13*dmats1[13][1] + coeff0_14*dmats1[14][1] + coeff0_15*dmats1[15][1] + coeff0_16*dmats1[16][1] + coeff0_17*dmats1[17][1] + coeff0_18*dmats1[18][1] + coeff0_19*dmats1[19][1] + coeff0_20*dmats1[20][1] + coeff0_21*dmats1[21][1] + coeff0_22*dmats1[22][1] + coeff0_23*dmats1[23][1] + coeff0_24*dmats1[24][1] + coeff0_25*dmats1[25][1] + coeff0_26*dmats1[26][1] + coeff0_27*dmats1[27][1] + coeff0_28*dmats1[28][1] + coeff0_29*dmats1[29][1] + coeff0_30*dmats1[30][1] + coeff0_31*dmats1[31][1] + coeff0_32*dmats1[32][1] + coeff0_33*dmats1[33][1] + coeff0_34*dmats1[34][1] + coeff0_35*dmats1[35][1] + coeff0_36*dmats1[36][1] + coeff0_37*dmats1[37][1] + coeff0_38*dmats1[38][1] + coeff0_39*dmats1[39][1] + coeff0_40*dmats1[40][1] + coeff0_41*dmats1[41][1] + coeff0_42*dmats1[42][1] + coeff0_43*dmats1[43][1] + coeff0_44*dmats1[44][1] + coeff0_45*dmats1[45][1] + coeff0_46*dmats1[46][1] + coeff0_47*dmats1[47][1] + coeff0_48*dmats1[48][1] + coeff0_49*dmats1[49][1] + coeff0_50*dmats1[50][1] + coeff0_51*dmats1[51][1] + coeff0_52*dmats1[52][1] + coeff0_53*dmats1[53][1] + coeff0_54*dmats1[54][1] + coeff0_55*dmats1[55][1];
          new_coeff0_2 = coeff0_0*dmats1[0][2] + coeff0_1*dmats1[1][2] + coeff0_2*dmats1[2][2] + coeff0_3*dmats1[3][2] + coeff0_4*dmats1[4][2] + coeff0_5*dmats1[5][2] + coeff0_6*dmats1[6][2] + coeff0_7*dmats1[7][2] + coeff0_8*dmats1[8][2] + coeff0_9*dmats1[9][2] + coeff0_10*dmats1[10][2] + coeff0_11*dmats1[11][2] + coeff0_12*dmats1[12][2] + coeff0_13*dmats1[13][2] + coeff0_14*dmats1[14][2] + coeff0_15*dmats1[15][2] + coeff0_16*dmats1[16][2] + coeff0_17*dmats1[17][2] + coeff0_18*dmats1[18][2] + coeff0_19*dmats1[19][2] + coeff0_20*dmats1[20][2] + coeff0_21*dmats1[21][2] + coeff0_22*dmats1[22][2] + coeff0_23*dmats1[23][2] + coeff0_24*dmats1[24][2] + coeff0_25*dmats1[25][2] + coeff0_26*dmats1[26][2] + coeff0_27*dmats1[27][2] + coeff0_28*dmats1[28][2] + coeff0_29*dmats1[29][2] + coeff0_30*dmats1[30][2] + coeff0_31*dmats1[31][2] + coeff0_32*dmats1[32][2] + coeff0_33*dmats1[33][2] + coeff0_34*dmats1[34][2] + coeff0_35*dmats1[35][2] + coeff0_36*dmats1[36][2] + coeff0_37*dmats1[37][2] + coeff0_38*dmats1[38][2] + coeff0_39*dmats1[39][2] + coeff0_40*dmats1[40][2] + coeff0_41*dmats1[41][2] + coeff0_42*dmats1[42][2] + coeff0_43*dmats1[43][2] + coeff0_44*dmats1[44][2] + coeff0_45*dmats1[45][2] + coeff0_46*dmats1[46][2] + coeff0_47*dmats1[47][2] + coeff0_48*dmats1[48][2] + coeff0_49*dmats1[49][2] + coeff0_50*dmats1[50][2] + coeff0_51*dmats1[51][2] + coeff0_52*dmats1[52][2] + coeff0_53*dmats1[53][2] + coeff0_54*dmats1[54][2] + coeff0_55*dmats1[55][2];
          new_coeff0_3 = coeff0_0*dmats1[0][3] + coeff0_1*dmats1[1][3] + coeff0_2*dmats1[2][3] + coeff0_3*dmats1[3][3] + coeff0_4*dmats1[4][3] + coeff0_5*dmats1[5][3] + coeff0_6*dmats1[6][3] + coeff0_7*dmats1[7][3] + coeff0_8*dmats1[8][3] + coeff0_9*dmats1[9][3] + coeff0_10*dmats1[10][3] + coeff0_11*dmats1[11][3] + coeff0_12*dmats1[12][3] + coeff0_13*dmats1[13][3] + coeff0_14*dmats1[14][3] + coeff0_15*dmats1[15][3] + coeff0_16*dmats1[16][3] + coeff0_17*dmats1[17][3] + coeff0_18*dmats1[18][3] + coeff0_19*dmats1[19][3] + coeff0_20*dmats1[20][3] + coeff0_21*dmats1[21][3] + coeff0_22*dmats1[22][3] + coeff0_23*dmats1[23][3] + coeff0_24*dmats1[24][3] + coeff0_25*dmats1[25][3] + coeff0_26*dmats1[26][3] + coeff0_27*dmats1[27][3] + coeff0_28*dmats1[28][3] + coeff0_29*dmats1[29][3] + coeff0_30*dmats1[30][3] + coeff0_31*dmats1[31][3] + coeff0_32*dmats1[32][3] + coeff0_33*dmats1[33][3] + coeff0_34*dmats1[34][3] + coeff0_35*dmats1[35][3] + coeff0_36*dmats1[36][3] + coeff0_37*dmats1[37][3] + coeff0_38*dmats1[38][3] + coeff0_39*dmats1[39][3] + coeff0_40*dmats1[40][3] + coeff0_41*dmats1[41][3] + coeff0_42*dmats1[42][3] + coeff0_43*dmats1[43][3] + coeff0_44*dmats1[44][3] + coeff0_45*dmats1[45][3] + coeff0_46*dmats1[46][3] + coeff0_47*dmats1[47][3] + coeff0_48*dmats1[48][3] + coeff0_49*dmats1[49][3] + coeff0_50*dmats1[50][3] + coeff0_51*dmats1[51][3] + coeff0_52*dmats1[52][3] + coeff0_53*dmats1[53][3] + coeff0_54*dmats1[54][3] + coeff0_55*dmats1[55][3];
          new_coeff0_4 = coeff0_0*dmats1[0][4] + coeff0_1*dmats1[1][4] + coeff0_2*dmats1[2][4] + coeff0_3*dmats1[3][4] + coeff0_4*dmats1[4][4] + coeff0_5*dmats1[5][4] + coeff0_6*dmats1[6][4] + coeff0_7*dmats1[7][4] + coeff0_8*dmats1[8][4] + coeff0_9*dmats1[9][4] + coeff0_10*dmats1[10][4] + coeff0_11*dmats1[11][4] + coeff0_12*dmats1[12][4] + coeff0_13*dmats1[13][4] + coeff0_14*dmats1[14][4] + coeff0_15*dmats1[15][4] + coeff0_16*dmats1[16][4] + coeff0_17*dmats1[17][4] + coeff0_18*dmats1[18][4] + coeff0_19*dmats1[19][4] + coeff0_20*dmats1[20][4] + coeff0_21*dmats1[21][4] + coeff0_22*dmats1[22][4] + coeff0_23*dmats1[23][4] + coeff0_24*dmats1[24][4] + coeff0_25*dmats1[25][4] + coeff0_26*dmats1[26][4] + coeff0_27*dmats1[27][4] + coeff0_28*dmats1[28][4] + coeff0_29*dmats1[29][4] + coeff0_30*dmats1[30][4] + coeff0_31*dmats1[31][4] + coeff0_32*dmats1[32][4] + coeff0_33*dmats1[33][4] + coeff0_34*dmats1[34][4] + coeff0_35*dmats1[35][4] + coeff0_36*dmats1[36][4] + coeff0_37*dmats1[37][4] + coeff0_38*dmats1[38][4] + coeff0_39*dmats1[39][4] + coeff0_40*dmats1[40][4] + coeff0_41*dmats1[41][4] + coeff0_42*dmats1[42][4] + coeff0_43*dmats1[43][4] + coeff0_44*dmats1[44][4] + coeff0_45*dmats1[45][4] + coeff0_46*dmats1[46][4] + coeff0_47*dmats1[47][4] + coeff0_48*dmats1[48][4] + coeff0_49*dmats1[49][4] + coeff0_50*dmats1[50][4] + coeff0_51*dmats1[51][4] + coeff0_52*dmats1[52][4] + coeff0_53*dmats1[53][4] + coeff0_54*dmats1[54][4] + coeff0_55*dmats1[55][4];
          new_coeff0_5 = coeff0_0*dmats1[0][5] + coeff0_1*dmats1[1][5] + coeff0_2*dmats1[2][5] + coeff0_3*dmats1[3][5] + coeff0_4*dmats1[4][5] + coeff0_5*dmats1[5][5] + coeff0_6*dmats1[6][5] + coeff0_7*dmats1[7][5] + coeff0_8*dmats1[8][5] + coeff0_9*dmats1[9][5] + coeff0_10*dmats1[10][5] + coeff0_11*dmats1[11][5] + coeff0_12*dmats1[12][5] + coeff0_13*dmats1[13][5] + coeff0_14*dmats1[14][5] + coeff0_15*dmats1[15][5] + coeff0_16*dmats1[16][5] + coeff0_17*dmats1[17][5] + coeff0_18*dmats1[18][5] + coeff0_19*dmats1[19][5] + coeff0_20*dmats1[20][5] + coeff0_21*dmats1[21][5] + coeff0_22*dmats1[22][5] + coeff0_23*dmats1[23][5] + coeff0_24*dmats1[24][5] + coeff0_25*dmats1[25][5] + coeff0_26*dmats1[26][5] + coeff0_27*dmats1[27][5] + coeff0_28*dmats1[28][5] + coeff0_29*dmats1[29][5] + coeff0_30*dmats1[30][5] + coeff0_31*dmats1[31][5] + coeff0_32*dmats1[32][5] + coeff0_33*dmats1[33][5] + coeff0_34*dmats1[34][5] + coeff0_35*dmats1[35][5] + coeff0_36*dmats1[36][5] + coeff0_37*dmats1[37][5] + coeff0_38*dmats1[38][5] + coeff0_39*dmats1[39][5] + coeff0_40*dmats1[40][5] + coeff0_41*dmats1[41][5] + coeff0_42*dmats1[42][5] + coeff0_43*dmats1[43][5] + coeff0_44*dmats1[44][5] + coeff0_45*dmats1[45][5] + coeff0_46*dmats1[46][5] + coeff0_47*dmats1[47][5] + coeff0_48*dmats1[48][5] + coeff0_49*dmats1[49][5] + coeff0_50*dmats1[50][5] + coeff0_51*dmats1[51][5] + coeff0_52*dmats1[52][5] + coeff0_53*dmats1[53][5] + coeff0_54*dmats1[54][5] + coeff0_55*dmats1[55][5];
          new_coeff0_6 = coeff0_0*dmats1[0][6] + coeff0_1*dmats1[1][6] + coeff0_2*dmats1[2][6] + coeff0_3*dmats1[3][6] + coeff0_4*dmats1[4][6] + coeff0_5*dmats1[5][6] + coeff0_6*dmats1[6][6] + coeff0_7*dmats1[7][6] + coeff0_8*dmats1[8][6] + coeff0_9*dmats1[9][6] + coeff0_10*dmats1[10][6] + coeff0_11*dmats1[11][6] + coeff0_12*dmats1[12][6] + coeff0_13*dmats1[13][6] + coeff0_14*dmats1[14][6] + coeff0_15*dmats1[15][6] + coeff0_16*dmats1[16][6] + coeff0_17*dmats1[17][6] + coeff0_18*dmats1[18][6] + coeff0_19*dmats1[19][6] + coeff0_20*dmats1[20][6] + coeff0_21*dmats1[21][6] + coeff0_22*dmats1[22][6] + coeff0_23*dmats1[23][6] + coeff0_24*dmats1[24][6] + coeff0_25*dmats1[25][6] + coeff0_26*dmats1[26][6] + coeff0_27*dmats1[27][6] + coeff0_28*dmats1[28][6] + coeff0_29*dmats1[29][6] + coeff0_30*dmats1[30][6] + coeff0_31*dmats1[31][6] + coeff0_32*dmats1[32][6] + coeff0_33*dmats1[33][6] + coeff0_34*dmats1[34][6] + coeff0_35*dmats1[35][6] + coeff0_36*dmats1[36][6] + coeff0_37*dmats1[37][6] + coeff0_38*dmats1[38][6] + coeff0_39*dmats1[39][6] + coeff0_40*dmats1[40][6] + coeff0_41*dmats1[41][6] + coeff0_42*dmats1[42][6] + coeff0_43*dmats1[43][6] + coeff0_44*dmats1[44][6] + coeff0_45*dmats1[45][6] + coeff0_46*dmats1[46][6] + coeff0_47*dmats1[47][6] + coeff0_48*dmats1[48][6] + coeff0_49*dmats1[49][6] + coeff0_50*dmats1[50][6] + coeff0_51*dmats1[51][6] + coeff0_52*dmats1[52][6] + coeff0_53*dmats1[53][6] + coeff0_54*dmats1[54][6] + coeff0_55*dmats1[55][6];
          new_coeff0_7 = coeff0_0*dmats1[0][7] + coeff0_1*dmats1[1][7] + coeff0_2*dmats1[2][7] + coeff0_3*dmats1[3][7] + coeff0_4*dmats1[4][7] + coeff0_5*dmats1[5][7] + coeff0_6*dmats1[6][7] + coeff0_7*dmats1[7][7] + coeff0_8*dmats1[8][7] + coeff0_9*dmats1[9][7] + coeff0_10*dmats1[10][7] + coeff0_11*dmats1[11][7] + coeff0_12*dmats1[12][7] + coeff0_13*dmats1[13][7] + coeff0_14*dmats1[14][7] + coeff0_15*dmats1[15][7] + coeff0_16*dmats1[16][7] + coeff0_17*dmats1[17][7] + coeff0_18*dmats1[18][7] + coeff0_19*dmats1[19][7] + coeff0_20*dmats1[20][7] + coeff0_21*dmats1[21][7] + coeff0_22*dmats1[22][7] + coeff0_23*dmats1[23][7] + coeff0_24*dmats1[24][7] + coeff0_25*dmats1[25][7] + coeff0_26*dmats1[26][7] + coeff0_27*dmats1[27][7] + coeff0_28*dmats1[28][7] + coeff0_29*dmats1[29][7] + coeff0_30*dmats1[30][7] + coeff0_31*dmats1[31][7] + coeff0_32*dmats1[32][7] + coeff0_33*dmats1[33][7] + coeff0_34*dmats1[34][7] + coeff0_35*dmats1[35][7] + coeff0_36*dmats1[36][7] + coeff0_37*dmats1[37][7] + coeff0_38*dmats1[38][7] + coeff0_39*dmats1[39][7] + coeff0_40*dmats1[40][7] + coeff0_41*dmats1[41][7] + coeff0_42*dmats1[42][7] + coeff0_43*dmats1[43][7] + coeff0_44*dmats1[44][7] + coeff0_45*dmats1[45][7] + coeff0_46*dmats1[46][7] + coeff0_47*dmats1[47][7] + coeff0_48*dmats1[48][7] + coeff0_49*dmats1[49][7] + coeff0_50*dmats1[50][7] + coeff0_51*dmats1[51][7] + coeff0_52*dmats1[52][7] + coeff0_53*dmats1[53][7] + coeff0_54*dmats1[54][7] + coeff0_55*dmats1[55][7];
          new_coeff0_8 = coeff0_0*dmats1[0][8] + coeff0_1*dmats1[1][8] + coeff0_2*dmats1[2][8] + coeff0_3*dmats1[3][8] + coeff0_4*dmats1[4][8] + coeff0_5*dmats1[5][8] + coeff0_6*dmats1[6][8] + coeff0_7*dmats1[7][8] + coeff0_8*dmats1[8][8] + coeff0_9*dmats1[9][8] + coeff0_10*dmats1[10][8] + coeff0_11*dmats1[11][8] + coeff0_12*dmats1[12][8] + coeff0_13*dmats1[13][8] + coeff0_14*dmats1[14][8] + coeff0_15*dmats1[15][8] + coeff0_16*dmats1[16][8] + coeff0_17*dmats1[17][8] + coeff0_18*dmats1[18][8] + coeff0_19*dmats1[19][8] + coeff0_20*dmats1[20][8] + coeff0_21*dmats1[21][8] + coeff0_22*dmats1[22][8] + coeff0_23*dmats1[23][8] + coeff0_24*dmats1[24][8] + coeff0_25*dmats1[25][8] + coeff0_26*dmats1[26][8] + coeff0_27*dmats1[27][8] + coeff0_28*dmats1[28][8] + coeff0_29*dmats1[29][8] + coeff0_30*dmats1[30][8] + coeff0_31*dmats1[31][8] + coeff0_32*dmats1[32][8] + coeff0_33*dmats1[33][8] + coeff0_34*dmats1[34][8] + coeff0_35*dmats1[35][8] + coeff0_36*dmats1[36][8] + coeff0_37*dmats1[37][8] + coeff0_38*dmats1[38][8] + coeff0_39*dmats1[39][8] + coeff0_40*dmats1[40][8] + coeff0_41*dmats1[41][8] + coeff0_42*dmats1[42][8] + coeff0_43*dmats1[43][8] + coeff0_44*dmats1[44][8] + coeff0_45*dmats1[45][8] + coeff0_46*dmats1[46][8] + coeff0_47*dmats1[47][8] + coeff0_48*dmats1[48][8] + coeff0_49*dmats1[49][8] + coeff0_50*dmats1[50][8] + coeff0_51*dmats1[51][8] + coeff0_52*dmats1[52][8] + coeff0_53*dmats1[53][8] + coeff0_54*dmats1[54][8] + coeff0_55*dmats1[55][8];
          new_coeff0_9 = coeff0_0*dmats1[0][9] + coeff0_1*dmats1[1][9] + coeff0_2*dmats1[2][9] + coeff0_3*dmats1[3][9] + coeff0_4*dmats1[4][9] + coeff0_5*dmats1[5][9] + coeff0_6*dmats1[6][9] + coeff0_7*dmats1[7][9] + coeff0_8*dmats1[8][9] + coeff0_9*dmats1[9][9] + coeff0_10*dmats1[10][9] + coeff0_11*dmats1[11][9] + coeff0_12*dmats1[12][9] + coeff0_13*dmats1[13][9] + coeff0_14*dmats1[14][9] + coeff0_15*dmats1[15][9] + coeff0_16*dmats1[16][9] + coeff0_17*dmats1[17][9] + coeff0_18*dmats1[18][9] + coeff0_19*dmats1[19][9] + coeff0_20*dmats1[20][9] + coeff0_21*dmats1[21][9] + coeff0_22*dmats1[22][9] + coeff0_23*dmats1[23][9] + coeff0_24*dmats1[24][9] + coeff0_25*dmats1[25][9] + coeff0_26*dmats1[26][9] + coeff0_27*dmats1[27][9] + coeff0_28*dmats1[28][9] + coeff0_29*dmats1[29][9] + coeff0_30*dmats1[30][9] + coeff0_31*dmats1[31][9] + coeff0_32*dmats1[32][9] + coeff0_33*dmats1[33][9] + coeff0_34*dmats1[34][9] + coeff0_35*dmats1[35][9] + coeff0_36*dmats1[36][9] + coeff0_37*dmats1[37][9] + coeff0_38*dmats1[38][9] + coeff0_39*dmats1[39][9] + coeff0_40*dmats1[40][9] + coeff0_41*dmats1[41][9] + coeff0_42*dmats1[42][9] + coeff0_43*dmats1[43][9] + coeff0_44*dmats1[44][9] + coeff0_45*dmats1[45][9] + coeff0_46*dmats1[46][9] + coeff0_47*dmats1[47][9] + coeff0_48*dmats1[48][9] + coeff0_49*dmats1[49][9] + coeff0_50*dmats1[50][9] + coeff0_51*dmats1[51][9] + coeff0_52*dmats1[52][9] + coeff0_53*dmats1[53][9] + coeff0_54*dmats1[54][9] + coeff0_55*dmats1[55][9];
          new_coeff0_10 = coeff0_0*dmats1[0][10] + coeff0_1*dmats1[1][10] + coeff0_2*dmats1[2][10] + coeff0_3*dmats1[3][10] + coeff0_4*dmats1[4][10] + coeff0_5*dmats1[5][10] + coeff0_6*dmats1[6][10] + coeff0_7*dmats1[7][10] + coeff0_8*dmats1[8][10] + coeff0_9*dmats1[9][10] + coeff0_10*dmats1[10][10] + coeff0_11*dmats1[11][10] + coeff0_12*dmats1[12][10] + coeff0_13*dmats1[13][10] + coeff0_14*dmats1[14][10] + coeff0_15*dmats1[15][10] + coeff0_16*dmats1[16][10] + coeff0_17*dmats1[17][10] + coeff0_18*dmats1[18][10] + coeff0_19*dmats1[19][10] + coeff0_20*dmats1[20][10] + coeff0_21*dmats1[21][10] + coeff0_22*dmats1[22][10] + coeff0_23*dmats1[23][10] + coeff0_24*dmats1[24][10] + coeff0_25*dmats1[25][10] + coeff0_26*dmats1[26][10] + coeff0_27*dmats1[27][10] + coeff0_28*dmats1[28][10] + coeff0_29*dmats1[29][10] + coeff0_30*dmats1[30][10] + coeff0_31*dmats1[31][10] + coeff0_32*dmats1[32][10] + coeff0_33*dmats1[33][10] + coeff0_34*dmats1[34][10] + coeff0_35*dmats1[35][10] + coeff0_36*dmats1[36][10] + coeff0_37*dmats1[37][10] + coeff0_38*dmats1[38][10] + coeff0_39*dmats1[39][10] + coeff0_40*dmats1[40][10] + coeff0_41*dmats1[41][10] + coeff0_42*dmats1[42][10] + coeff0_43*dmats1[43][10] + coeff0_44*dmats1[44][10] + coeff0_45*dmats1[45][10] + coeff0_46*dmats1[46][10] + coeff0_47*dmats1[47][10] + coeff0_48*dmats1[48][10] + coeff0_49*dmats1[49][10] + coeff0_50*dmats1[50][10] + coeff0_51*dmats1[51][10] + coeff0_52*dmats1[52][10] + coeff0_53*dmats1[53][10] + coeff0_54*dmats1[54][10] + coeff0_55*dmats1[55][10];
          new_coeff0_11 = coeff0_0*dmats1[0][11] + coeff0_1*dmats1[1][11] + coeff0_2*dmats1[2][11] + coeff0_3*dmats1[3][11] + coeff0_4*dmats1[4][11] + coeff0_5*dmats1[5][11] + coeff0_6*dmats1[6][11] + coeff0_7*dmats1[7][11] + coeff0_8*dmats1[8][11] + coeff0_9*dmats1[9][11] + coeff0_10*dmats1[10][11] + coeff0_11*dmats1[11][11] + coeff0_12*dmats1[12][11] + coeff0_13*dmats1[13][11] + coeff0_14*dmats1[14][11] + coeff0_15*dmats1[15][11] + coeff0_16*dmats1[16][11] + coeff0_17*dmats1[17][11] + coeff0_18*dmats1[18][11] + coeff0_19*dmats1[19][11] + coeff0_20*dmats1[20][11] + coeff0_21*dmats1[21][11] + coeff0_22*dmats1[22][11] + coeff0_23*dmats1[23][11] + coeff0_24*dmats1[24][11] + coeff0_25*dmats1[25][11] + coeff0_26*dmats1[26][11] + coeff0_27*dmats1[27][11] + coeff0_28*dmats1[28][11] + coeff0_29*dmats1[29][11] + coeff0_30*dmats1[30][11] + coeff0_31*dmats1[31][11] + coeff0_32*dmats1[32][11] + coeff0_33*dmats1[33][11] + coeff0_34*dmats1[34][11] + coeff0_35*dmats1[35][11] + coeff0_36*dmats1[36][11] + coeff0_37*dmats1[37][11] + coeff0_38*dmats1[38][11] + coeff0_39*dmats1[39][11] + coeff0_40*dmats1[40][11] + coeff0_41*dmats1[41][11] + coeff0_42*dmats1[42][11] + coeff0_43*dmats1[43][11] + coeff0_44*dmats1[44][11] + coeff0_45*dmats1[45][11] + coeff0_46*dmats1[46][11] + coeff0_47*dmats1[47][11] + coeff0_48*dmats1[48][11] + coeff0_49*dmats1[49][11] + coeff0_50*dmats1[50][11] + coeff0_51*dmats1[51][11] + coeff0_52*dmats1[52][11] + coeff0_53*dmats1[53][11] + coeff0_54*dmats1[54][11] + coeff0_55*dmats1[55][11];
          new_coeff0_12 = coeff0_0*dmats1[0][12] + coeff0_1*dmats1[1][12] + coeff0_2*dmats1[2][12] + coeff0_3*dmats1[3][12] + coeff0_4*dmats1[4][12] + coeff0_5*dmats1[5][12] + coeff0_6*dmats1[6][12] + coeff0_7*dmats1[7][12] + coeff0_8*dmats1[8][12] + coeff0_9*dmats1[9][12] + coeff0_10*dmats1[10][12] + coeff0_11*dmats1[11][12] + coeff0_12*dmats1[12][12] + coeff0_13*dmats1[13][12] + coeff0_14*dmats1[14][12] + coeff0_15*dmats1[15][12] + coeff0_16*dmats1[16][12] + coeff0_17*dmats1[17][12] + coeff0_18*dmats1[18][12] + coeff0_19*dmats1[19][12] + coeff0_20*dmats1[20][12] + coeff0_21*dmats1[21][12] + coeff0_22*dmats1[22][12] + coeff0_23*dmats1[23][12] + coeff0_24*dmats1[24][12] + coeff0_25*dmats1[25][12] + coeff0_26*dmats1[26][12] + coeff0_27*dmats1[27][12] + coeff0_28*dmats1[28][12] + coeff0_29*dmats1[29][12] + coeff0_30*dmats1[30][12] + coeff0_31*dmats1[31][12] + coeff0_32*dmats1[32][12] + coeff0_33*dmats1[33][12] + coeff0_34*dmats1[34][12] + coeff0_35*dmats1[35][12] + coeff0_36*dmats1[36][12] + coeff0_37*dmats1[37][12] + coeff0_38*dmats1[38][12] + coeff0_39*dmats1[39][12] + coeff0_40*dmats1[40][12] + coeff0_41*dmats1[41][12] + coeff0_42*dmats1[42][12] + coeff0_43*dmats1[43][12] + coeff0_44*dmats1[44][12] + coeff0_45*dmats1[45][12] + coeff0_46*dmats1[46][12] + coeff0_47*dmats1[47][12] + coeff0_48*dmats1[48][12] + coeff0_49*dmats1[49][12] + coeff0_50*dmats1[50][12] + coeff0_51*dmats1[51][12] + coeff0_52*dmats1[52][12] + coeff0_53*dmats1[53][12] + coeff0_54*dmats1[54][12] + coeff0_55*dmats1[55][12];
          new_coeff0_13 = coeff0_0*dmats1[0][13] + coeff0_1*dmats1[1][13] + coeff0_2*dmats1[2][13] + coeff0_3*dmats1[3][13] + coeff0_4*dmats1[4][13] + coeff0_5*dmats1[5][13] + coeff0_6*dmats1[6][13] + coeff0_7*dmats1[7][13] + coeff0_8*dmats1[8][13] + coeff0_9*dmats1[9][13] + coeff0_10*dmats1[10][13] + coeff0_11*dmats1[11][13] + coeff0_12*dmats1[12][13] + coeff0_13*dmats1[13][13] + coeff0_14*dmats1[14][13] + coeff0_15*dmats1[15][13] + coeff0_16*dmats1[16][13] + coeff0_17*dmats1[17][13] + coeff0_18*dmats1[18][13] + coeff0_19*dmats1[19][13] + coeff0_20*dmats1[20][13] + coeff0_21*dmats1[21][13] + coeff0_22*dmats1[22][13] + coeff0_23*dmats1[23][13] + coeff0_24*dmats1[24][13] + coeff0_25*dmats1[25][13] + coeff0_26*dmats1[26][13] + coeff0_27*dmats1[27][13] + coeff0_28*dmats1[28][13] + coeff0_29*dmats1[29][13] + coeff0_30*dmats1[30][13] + coeff0_31*dmats1[31][13] + coeff0_32*dmats1[32][13] + coeff0_33*dmats1[33][13] + coeff0_34*dmats1[34][13] + coeff0_35*dmats1[35][13] + coeff0_36*dmats1[36][13] + coeff0_37*dmats1[37][13] + coeff0_38*dmats1[38][13] + coeff0_39*dmats1[39][13] + coeff0_40*dmats1[40][13] + coeff0_41*dmats1[41][13] + coeff0_42*dmats1[42][13] + coeff0_43*dmats1[43][13] + coeff0_44*dmats1[44][13] + coeff0_45*dmats1[45][13] + coeff0_46*dmats1[46][13] + coeff0_47*dmats1[47][13] + coeff0_48*dmats1[48][13] + coeff0_49*dmats1[49][13] + coeff0_50*dmats1[50][13] + coeff0_51*dmats1[51][13] + coeff0_52*dmats1[52][13] + coeff0_53*dmats1[53][13] + coeff0_54*dmats1[54][13] + coeff0_55*dmats1[55][13];
          new_coeff0_14 = coeff0_0*dmats1[0][14] + coeff0_1*dmats1[1][14] + coeff0_2*dmats1[2][14] + coeff0_3*dmats1[3][14] + coeff0_4*dmats1[4][14] + coeff0_5*dmats1[5][14] + coeff0_6*dmats1[6][14] + coeff0_7*dmats1[7][14] + coeff0_8*dmats1[8][14] + coeff0_9*dmats1[9][14] + coeff0_10*dmats1[10][14] + coeff0_11*dmats1[11][14] + coeff0_12*dmats1[12][14] + coeff0_13*dmats1[13][14] + coeff0_14*dmats1[14][14] + coeff0_15*dmats1[15][14] + coeff0_16*dmats1[16][14] + coeff0_17*dmats1[17][14] + coeff0_18*dmats1[18][14] + coeff0_19*dmats1[19][14] + coeff0_20*dmats1[20][14] + coeff0_21*dmats1[21][14] + coeff0_22*dmats1[22][14] + coeff0_23*dmats1[23][14] + coeff0_24*dmats1[24][14] + coeff0_25*dmats1[25][14] + coeff0_26*dmats1[26][14] + coeff0_27*dmats1[27][14] + coeff0_28*dmats1[28][14] + coeff0_29*dmats1[29][14] + coeff0_30*dmats1[30][14] + coeff0_31*dmats1[31][14] + coeff0_32*dmats1[32][14] + coeff0_33*dmats1[33][14] + coeff0_34*dmats1[34][14] + coeff0_35*dmats1[35][14] + coeff0_36*dmats1[36][14] + coeff0_37*dmats1[37][14] + coeff0_38*dmats1[38][14] + coeff0_39*dmats1[39][14] + coeff0_40*dmats1[40][14] + coeff0_41*dmats1[41][14] + coeff0_42*dmats1[42][14] + coeff0_43*dmats1[43][14] + coeff0_44*dmats1[44][14] + coeff0_45*dmats1[45][14] + coeff0_46*dmats1[46][14] + coeff0_47*dmats1[47][14] + coeff0_48*dmats1[48][14] + coeff0_49*dmats1[49][14] + coeff0_50*dmats1[50][14] + coeff0_51*dmats1[51][14] + coeff0_52*dmats1[52][14] + coeff0_53*dmats1[53][14] + coeff0_54*dmats1[54][14] + coeff0_55*dmats1[55][14];
          new_coeff0_15 = coeff0_0*dmats1[0][15] + coeff0_1*dmats1[1][15] + coeff0_2*dmats1[2][15] + coeff0_3*dmats1[3][15] + coeff0_4*dmats1[4][15] + coeff0_5*dmats1[5][15] + coeff0_6*dmats1[6][15] + coeff0_7*dmats1[7][15] + coeff0_8*dmats1[8][15] + coeff0_9*dmats1[9][15] + coeff0_10*dmats1[10][15] + coeff0_11*dmats1[11][15] + coeff0_12*dmats1[12][15] + coeff0_13*dmats1[13][15] + coeff0_14*dmats1[14][15] + coeff0_15*dmats1[15][15] + coeff0_16*dmats1[16][15] + coeff0_17*dmats1[17][15] + coeff0_18*dmats1[18][15] + coeff0_19*dmats1[19][15] + coeff0_20*dmats1[20][15] + coeff0_21*dmats1[21][15] + coeff0_22*dmats1[22][15] + coeff0_23*dmats1[23][15] + coeff0_24*dmats1[24][15] + coeff0_25*dmats1[25][15] + coeff0_26*dmats1[26][15] + coeff0_27*dmats1[27][15] + coeff0_28*dmats1[28][15] + coeff0_29*dmats1[29][15] + coeff0_30*dmats1[30][15] + coeff0_31*dmats1[31][15] + coeff0_32*dmats1[32][15] + coeff0_33*dmats1[33][15] + coeff0_34*dmats1[34][15] + coeff0_35*dmats1[35][15] + coeff0_36*dmats1[36][15] + coeff0_37*dmats1[37][15] + coeff0_38*dmats1[38][15] + coeff0_39*dmats1[39][15] + coeff0_40*dmats1[40][15] + coeff0_41*dmats1[41][15] + coeff0_42*dmats1[42][15] + coeff0_43*dmats1[43][15] + coeff0_44*dmats1[44][15] + coeff0_45*dmats1[45][15] + coeff0_46*dmats1[46][15] + coeff0_47*dmats1[47][15] + coeff0_48*dmats1[48][15] + coeff0_49*dmats1[49][15] + coeff0_50*dmats1[50][15] + coeff0_51*dmats1[51][15] + coeff0_52*dmats1[52][15] + coeff0_53*dmats1[53][15] + coeff0_54*dmats1[54][15] + coeff0_55*dmats1[55][15];
          new_coeff0_16 = coeff0_0*dmats1[0][16] + coeff0_1*dmats1[1][16] + coeff0_2*dmats1[2][16] + coeff0_3*dmats1[3][16] + coeff0_4*dmats1[4][16] + coeff0_5*dmats1[5][16] + coeff0_6*dmats1[6][16] + coeff0_7*dmats1[7][16] + coeff0_8*dmats1[8][16] + coeff0_9*dmats1[9][16] + coeff0_10*dmats1[10][16] + coeff0_11*dmats1[11][16] + coeff0_12*dmats1[12][16] + coeff0_13*dmats1[13][16] + coeff0_14*dmats1[14][16] + coeff0_15*dmats1[15][16] + coeff0_16*dmats1[16][16] + coeff0_17*dmats1[17][16] + coeff0_18*dmats1[18][16] + coeff0_19*dmats1[19][16] + coeff0_20*dmats1[20][16] + coeff0_21*dmats1[21][16] + coeff0_22*dmats1[22][16] + coeff0_23*dmats1[23][16] + coeff0_24*dmats1[24][16] + coeff0_25*dmats1[25][16] + coeff0_26*dmats1[26][16] + coeff0_27*dmats1[27][16] + coeff0_28*dmats1[28][16] + coeff0_29*dmats1[29][16] + coeff0_30*dmats1[30][16] + coeff0_31*dmats1[31][16] + coeff0_32*dmats1[32][16] + coeff0_33*dmats1[33][16] + coeff0_34*dmats1[34][16] + coeff0_35*dmats1[35][16] + coeff0_36*dmats1[36][16] + coeff0_37*dmats1[37][16] + coeff0_38*dmats1[38][16] + coeff0_39*dmats1[39][16] + coeff0_40*dmats1[40][16] + coeff0_41*dmats1[41][16] + coeff0_42*dmats1[42][16] + coeff0_43*dmats1[43][16] + coeff0_44*dmats1[44][16] + coeff0_45*dmats1[45][16] + coeff0_46*dmats1[46][16] + coeff0_47*dmats1[47][16] + coeff0_48*dmats1[48][16] + coeff0_49*dmats1[49][16] + coeff0_50*dmats1[50][16] + coeff0_51*dmats1[51][16] + coeff0_52*dmats1[52][16] + coeff0_53*dmats1[53][16] + coeff0_54*dmats1[54][16] + coeff0_55*dmats1[55][16];
          new_coeff0_17 = coeff0_0*dmats1[0][17] + coeff0_1*dmats1[1][17] + coeff0_2*dmats1[2][17] + coeff0_3*dmats1[3][17] + coeff0_4*dmats1[4][17] + coeff0_5*dmats1[5][17] + coeff0_6*dmats1[6][17] + coeff0_7*dmats1[7][17] + coeff0_8*dmats1[8][17] + coeff0_9*dmats1[9][17] + coeff0_10*dmats1[10][17] + coeff0_11*dmats1[11][17] + coeff0_12*dmats1[12][17] + coeff0_13*dmats1[13][17] + coeff0_14*dmats1[14][17] + coeff0_15*dmats1[15][17] + coeff0_16*dmats1[16][17] + coeff0_17*dmats1[17][17] + coeff0_18*dmats1[18][17] + coeff0_19*dmats1[19][17] + coeff0_20*dmats1[20][17] + coeff0_21*dmats1[21][17] + coeff0_22*dmats1[22][17] + coeff0_23*dmats1[23][17] + coeff0_24*dmats1[24][17] + coeff0_25*dmats1[25][17] + coeff0_26*dmats1[26][17] + coeff0_27*dmats1[27][17] + coeff0_28*dmats1[28][17] + coeff0_29*dmats1[29][17] + coeff0_30*dmats1[30][17] + coeff0_31*dmats1[31][17] + coeff0_32*dmats1[32][17] + coeff0_33*dmats1[33][17] + coeff0_34*dmats1[34][17] + coeff0_35*dmats1[35][17] + coeff0_36*dmats1[36][17] + coeff0_37*dmats1[37][17] + coeff0_38*dmats1[38][17] + coeff0_39*dmats1[39][17] + coeff0_40*dmats1[40][17] + coeff0_41*dmats1[41][17] + coeff0_42*dmats1[42][17] + coeff0_43*dmats1[43][17] + coeff0_44*dmats1[44][17] + coeff0_45*dmats1[45][17] + coeff0_46*dmats1[46][17] + coeff0_47*dmats1[47][17] + coeff0_48*dmats1[48][17] + coeff0_49*dmats1[49][17] + coeff0_50*dmats1[50][17] + coeff0_51*dmats1[51][17] + coeff0_52*dmats1[52][17] + coeff0_53*dmats1[53][17] + coeff0_54*dmats1[54][17] + coeff0_55*dmats1[55][17];
          new_coeff0_18 = coeff0_0*dmats1[0][18] + coeff0_1*dmats1[1][18] + coeff0_2*dmats1[2][18] + coeff0_3*dmats1[3][18] + coeff0_4*dmats1[4][18] + coeff0_5*dmats1[5][18] + coeff0_6*dmats1[6][18] + coeff0_7*dmats1[7][18] + coeff0_8*dmats1[8][18] + coeff0_9*dmats1[9][18] + coeff0_10*dmats1[10][18] + coeff0_11*dmats1[11][18] + coeff0_12*dmats1[12][18] + coeff0_13*dmats1[13][18] + coeff0_14*dmats1[14][18] + coeff0_15*dmats1[15][18] + coeff0_16*dmats1[16][18] + coeff0_17*dmats1[17][18] + coeff0_18*dmats1[18][18] + coeff0_19*dmats1[19][18] + coeff0_20*dmats1[20][18] + coeff0_21*dmats1[21][18] + coeff0_22*dmats1[22][18] + coeff0_23*dmats1[23][18] + coeff0_24*dmats1[24][18] + coeff0_25*dmats1[25][18] + coeff0_26*dmats1[26][18] + coeff0_27*dmats1[27][18] + coeff0_28*dmats1[28][18] + coeff0_29*dmats1[29][18] + coeff0_30*dmats1[30][18] + coeff0_31*dmats1[31][18] + coeff0_32*dmats1[32][18] + coeff0_33*dmats1[33][18] + coeff0_34*dmats1[34][18] + coeff0_35*dmats1[35][18] + coeff0_36*dmats1[36][18] + coeff0_37*dmats1[37][18] + coeff0_38*dmats1[38][18] + coeff0_39*dmats1[39][18] + coeff0_40*dmats1[40][18] + coeff0_41*dmats1[41][18] + coeff0_42*dmats1[42][18] + coeff0_43*dmats1[43][18] + coeff0_44*dmats1[44][18] + coeff0_45*dmats1[45][18] + coeff0_46*dmats1[46][18] + coeff0_47*dmats1[47][18] + coeff0_48*dmats1[48][18] + coeff0_49*dmats1[49][18] + coeff0_50*dmats1[50][18] + coeff0_51*dmats1[51][18] + coeff0_52*dmats1[52][18] + coeff0_53*dmats1[53][18] + coeff0_54*dmats1[54][18] + coeff0_55*dmats1[55][18];
          new_coeff0_19 = coeff0_0*dmats1[0][19] + coeff0_1*dmats1[1][19] + coeff0_2*dmats1[2][19] + coeff0_3*dmats1[3][19] + coeff0_4*dmats1[4][19] + coeff0_5*dmats1[5][19] + coeff0_6*dmats1[6][19] + coeff0_7*dmats1[7][19] + coeff0_8*dmats1[8][19] + coeff0_9*dmats1[9][19] + coeff0_10*dmats1[10][19] + coeff0_11*dmats1[11][19] + coeff0_12*dmats1[12][19] + coeff0_13*dmats1[13][19] + coeff0_14*dmats1[14][19] + coeff0_15*dmats1[15][19] + coeff0_16*dmats1[16][19] + coeff0_17*dmats1[17][19] + coeff0_18*dmats1[18][19] + coeff0_19*dmats1[19][19] + coeff0_20*dmats1[20][19] + coeff0_21*dmats1[21][19] + coeff0_22*dmats1[22][19] + coeff0_23*dmats1[23][19] + coeff0_24*dmats1[24][19] + coeff0_25*dmats1[25][19] + coeff0_26*dmats1[26][19] + coeff0_27*dmats1[27][19] + coeff0_28*dmats1[28][19] + coeff0_29*dmats1[29][19] + coeff0_30*dmats1[30][19] + coeff0_31*dmats1[31][19] + coeff0_32*dmats1[32][19] + coeff0_33*dmats1[33][19] + coeff0_34*dmats1[34][19] + coeff0_35*dmats1[35][19] + coeff0_36*dmats1[36][19] + coeff0_37*dmats1[37][19] + coeff0_38*dmats1[38][19] + coeff0_39*dmats1[39][19] + coeff0_40*dmats1[40][19] + coeff0_41*dmats1[41][19] + coeff0_42*dmats1[42][19] + coeff0_43*dmats1[43][19] + coeff0_44*dmats1[44][19] + coeff0_45*dmats1[45][19] + coeff0_46*dmats1[46][19] + coeff0_47*dmats1[47][19] + coeff0_48*dmats1[48][19] + coeff0_49*dmats1[49][19] + coeff0_50*dmats1[50][19] + coeff0_51*dmats1[51][19] + coeff0_52*dmats1[52][19] + coeff0_53*dmats1[53][19] + coeff0_54*dmats1[54][19] + coeff0_55*dmats1[55][19];
          new_coeff0_20 = coeff0_0*dmats1[0][20] + coeff0_1*dmats1[1][20] + coeff0_2*dmats1[2][20] + coeff0_3*dmats1[3][20] + coeff0_4*dmats1[4][20] + coeff0_5*dmats1[5][20] + coeff0_6*dmats1[6][20] + coeff0_7*dmats1[7][20] + coeff0_8*dmats1[8][20] + coeff0_9*dmats1[9][20] + coeff0_10*dmats1[10][20] + coeff0_11*dmats1[11][20] + coeff0_12*dmats1[12][20] + coeff0_13*dmats1[13][20] + coeff0_14*dmats1[14][20] + coeff0_15*dmats1[15][20] + coeff0_16*dmats1[16][20] + coeff0_17*dmats1[17][20] + coeff0_18*dmats1[18][20] + coeff0_19*dmats1[19][20] + coeff0_20*dmats1[20][20] + coeff0_21*dmats1[21][20] + coeff0_22*dmats1[22][20] + coeff0_23*dmats1[23][20] + coeff0_24*dmats1[24][20] + coeff0_25*dmats1[25][20] + coeff0_26*dmats1[26][20] + coeff0_27*dmats1[27][20] + coeff0_28*dmats1[28][20] + coeff0_29*dmats1[29][20] + coeff0_30*dmats1[30][20] + coeff0_31*dmats1[31][20] + coeff0_32*dmats1[32][20] + coeff0_33*dmats1[33][20] + coeff0_34*dmats1[34][20] + coeff0_35*dmats1[35][20] + coeff0_36*dmats1[36][20] + coeff0_37*dmats1[37][20] + coeff0_38*dmats1[38][20] + coeff0_39*dmats1[39][20] + coeff0_40*dmats1[40][20] + coeff0_41*dmats1[41][20] + coeff0_42*dmats1[42][20] + coeff0_43*dmats1[43][20] + coeff0_44*dmats1[44][20] + coeff0_45*dmats1[45][20] + coeff0_46*dmats1[46][20] + coeff0_47*dmats1[47][20] + coeff0_48*dmats1[48][20] + coeff0_49*dmats1[49][20] + coeff0_50*dmats1[50][20] + coeff0_51*dmats1[51][20] + coeff0_52*dmats1[52][20] + coeff0_53*dmats1[53][20] + coeff0_54*dmats1[54][20] + coeff0_55*dmats1[55][20];
          new_coeff0_21 = coeff0_0*dmats1[0][21] + coeff0_1*dmats1[1][21] + coeff0_2*dmats1[2][21] + coeff0_3*dmats1[3][21] + coeff0_4*dmats1[4][21] + coeff0_5*dmats1[5][21] + coeff0_6*dmats1[6][21] + coeff0_7*dmats1[7][21] + coeff0_8*dmats1[8][21] + coeff0_9*dmats1[9][21] + coeff0_10*dmats1[10][21] + coeff0_11*dmats1[11][21] + coeff0_12*dmats1[12][21] + coeff0_13*dmats1[13][21] + coeff0_14*dmats1[14][21] + coeff0_15*dmats1[15][21] + coeff0_16*dmats1[16][21] + coeff0_17*dmats1[17][21] + coeff0_18*dmats1[18][21] + coeff0_19*dmats1[19][21] + coeff0_20*dmats1[20][21] + coeff0_21*dmats1[21][21] + coeff0_22*dmats1[22][21] + coeff0_23*dmats1[23][21] + coeff0_24*dmats1[24][21] + coeff0_25*dmats1[25][21] + coeff0_26*dmats1[26][21] + coeff0_27*dmats1[27][21] + coeff0_28*dmats1[28][21] + coeff0_29*dmats1[29][21] + coeff0_30*dmats1[30][21] + coeff0_31*dmats1[31][21] + coeff0_32*dmats1[32][21] + coeff0_33*dmats1[33][21] + coeff0_34*dmats1[34][21] + coeff0_35*dmats1[35][21] + coeff0_36*dmats1[36][21] + coeff0_37*dmats1[37][21] + coeff0_38*dmats1[38][21] + coeff0_39*dmats1[39][21] + coeff0_40*dmats1[40][21] + coeff0_41*dmats1[41][21] + coeff0_42*dmats1[42][21] + coeff0_43*dmats1[43][21] + coeff0_44*dmats1[44][21] + coeff0_45*dmats1[45][21] + coeff0_46*dmats1[46][21] + coeff0_47*dmats1[47][21] + coeff0_48*dmats1[48][21] + coeff0_49*dmats1[49][21] + coeff0_50*dmats1[50][21] + coeff0_51*dmats1[51][21] + coeff0_52*dmats1[52][21] + coeff0_53*dmats1[53][21] + coeff0_54*dmats1[54][21] + coeff0_55*dmats1[55][21];
          new_coeff0_22 = coeff0_0*dmats1[0][22] + coeff0_1*dmats1[1][22] + coeff0_2*dmats1[2][22] + coeff0_3*dmats1[3][22] + coeff0_4*dmats1[4][22] + coeff0_5*dmats1[5][22] + coeff0_6*dmats1[6][22] + coeff0_7*dmats1[7][22] + coeff0_8*dmats1[8][22] + coeff0_9*dmats1[9][22] + coeff0_10*dmats1[10][22] + coeff0_11*dmats1[11][22] + coeff0_12*dmats1[12][22] + coeff0_13*dmats1[13][22] + coeff0_14*dmats1[14][22] + coeff0_15*dmats1[15][22] + coeff0_16*dmats1[16][22] + coeff0_17*dmats1[17][22] + coeff0_18*dmats1[18][22] + coeff0_19*dmats1[19][22] + coeff0_20*dmats1[20][22] + coeff0_21*dmats1[21][22] + coeff0_22*dmats1[22][22] + coeff0_23*dmats1[23][22] + coeff0_24*dmats1[24][22] + coeff0_25*dmats1[25][22] + coeff0_26*dmats1[26][22] + coeff0_27*dmats1[27][22] + coeff0_28*dmats1[28][22] + coeff0_29*dmats1[29][22] + coeff0_30*dmats1[30][22] + coeff0_31*dmats1[31][22] + coeff0_32*dmats1[32][22] + coeff0_33*dmats1[33][22] + coeff0_34*dmats1[34][22] + coeff0_35*dmats1[35][22] + coeff0_36*dmats1[36][22] + coeff0_37*dmats1[37][22] + coeff0_38*dmats1[38][22] + coeff0_39*dmats1[39][22] + coeff0_40*dmats1[40][22] + coeff0_41*dmats1[41][22] + coeff0_42*dmats1[42][22] + coeff0_43*dmats1[43][22] + coeff0_44*dmats1[44][22] + coeff0_45*dmats1[45][22] + coeff0_46*dmats1[46][22] + coeff0_47*dmats1[47][22] + coeff0_48*dmats1[48][22] + coeff0_49*dmats1[49][22] + coeff0_50*dmats1[50][22] + coeff0_51*dmats1[51][22] + coeff0_52*dmats1[52][22] + coeff0_53*dmats1[53][22] + coeff0_54*dmats1[54][22] + coeff0_55*dmats1[55][22];
          new_coeff0_23 = coeff0_0*dmats1[0][23] + coeff0_1*dmats1[1][23] + coeff0_2*dmats1[2][23] + coeff0_3*dmats1[3][23] + coeff0_4*dmats1[4][23] + coeff0_5*dmats1[5][23] + coeff0_6*dmats1[6][23] + coeff0_7*dmats1[7][23] + coeff0_8*dmats1[8][23] + coeff0_9*dmats1[9][23] + coeff0_10*dmats1[10][23] + coeff0_11*dmats1[11][23] + coeff0_12*dmats1[12][23] + coeff0_13*dmats1[13][23] + coeff0_14*dmats1[14][23] + coeff0_15*dmats1[15][23] + coeff0_16*dmats1[16][23] + coeff0_17*dmats1[17][23] + coeff0_18*dmats1[18][23] + coeff0_19*dmats1[19][23] + coeff0_20*dmats1[20][23] + coeff0_21*dmats1[21][23] + coeff0_22*dmats1[22][23] + coeff0_23*dmats1[23][23] + coeff0_24*dmats1[24][23] + coeff0_25*dmats1[25][23] + coeff0_26*dmats1[26][23] + coeff0_27*dmats1[27][23] + coeff0_28*dmats1[28][23] + coeff0_29*dmats1[29][23] + coeff0_30*dmats1[30][23] + coeff0_31*dmats1[31][23] + coeff0_32*dmats1[32][23] + coeff0_33*dmats1[33][23] + coeff0_34*dmats1[34][23] + coeff0_35*dmats1[35][23] + coeff0_36*dmats1[36][23] + coeff0_37*dmats1[37][23] + coeff0_38*dmats1[38][23] + coeff0_39*dmats1[39][23] + coeff0_40*dmats1[40][23] + coeff0_41*dmats1[41][23] + coeff0_42*dmats1[42][23] + coeff0_43*dmats1[43][23] + coeff0_44*dmats1[44][23] + coeff0_45*dmats1[45][23] + coeff0_46*dmats1[46][23] + coeff0_47*dmats1[47][23] + coeff0_48*dmats1[48][23] + coeff0_49*dmats1[49][23] + coeff0_50*dmats1[50][23] + coeff0_51*dmats1[51][23] + coeff0_52*dmats1[52][23] + coeff0_53*dmats1[53][23] + coeff0_54*dmats1[54][23] + coeff0_55*dmats1[55][23];
          new_coeff0_24 = coeff0_0*dmats1[0][24] + coeff0_1*dmats1[1][24] + coeff0_2*dmats1[2][24] + coeff0_3*dmats1[3][24] + coeff0_4*dmats1[4][24] + coeff0_5*dmats1[5][24] + coeff0_6*dmats1[6][24] + coeff0_7*dmats1[7][24] + coeff0_8*dmats1[8][24] + coeff0_9*dmats1[9][24] + coeff0_10*dmats1[10][24] + coeff0_11*dmats1[11][24] + coeff0_12*dmats1[12][24] + coeff0_13*dmats1[13][24] + coeff0_14*dmats1[14][24] + coeff0_15*dmats1[15][24] + coeff0_16*dmats1[16][24] + coeff0_17*dmats1[17][24] + coeff0_18*dmats1[18][24] + coeff0_19*dmats1[19][24] + coeff0_20*dmats1[20][24] + coeff0_21*dmats1[21][24] + coeff0_22*dmats1[22][24] + coeff0_23*dmats1[23][24] + coeff0_24*dmats1[24][24] + coeff0_25*dmats1[25][24] + coeff0_26*dmats1[26][24] + coeff0_27*dmats1[27][24] + coeff0_28*dmats1[28][24] + coeff0_29*dmats1[29][24] + coeff0_30*dmats1[30][24] + coeff0_31*dmats1[31][24] + coeff0_32*dmats1[32][24] + coeff0_33*dmats1[33][24] + coeff0_34*dmats1[34][24] + coeff0_35*dmats1[35][24] + coeff0_36*dmats1[36][24] + coeff0_37*dmats1[37][24] + coeff0_38*dmats1[38][24] + coeff0_39*dmats1[39][24] + coeff0_40*dmats1[40][24] + coeff0_41*dmats1[41][24] + coeff0_42*dmats1[42][24] + coeff0_43*dmats1[43][24] + coeff0_44*dmats1[44][24] + coeff0_45*dmats1[45][24] + coeff0_46*dmats1[46][24] + coeff0_47*dmats1[47][24] + coeff0_48*dmats1[48][24] + coeff0_49*dmats1[49][24] + coeff0_50*dmats1[50][24] + coeff0_51*dmats1[51][24] + coeff0_52*dmats1[52][24] + coeff0_53*dmats1[53][24] + coeff0_54*dmats1[54][24] + coeff0_55*dmats1[55][24];
          new_coeff0_25 = coeff0_0*dmats1[0][25] + coeff0_1*dmats1[1][25] + coeff0_2*dmats1[2][25] + coeff0_3*dmats1[3][25] + coeff0_4*dmats1[4][25] + coeff0_5*dmats1[5][25] + coeff0_6*dmats1[6][25] + coeff0_7*dmats1[7][25] + coeff0_8*dmats1[8][25] + coeff0_9*dmats1[9][25] + coeff0_10*dmats1[10][25] + coeff0_11*dmats1[11][25] + coeff0_12*dmats1[12][25] + coeff0_13*dmats1[13][25] + coeff0_14*dmats1[14][25] + coeff0_15*dmats1[15][25] + coeff0_16*dmats1[16][25] + coeff0_17*dmats1[17][25] + coeff0_18*dmats1[18][25] + coeff0_19*dmats1[19][25] + coeff0_20*dmats1[20][25] + coeff0_21*dmats1[21][25] + coeff0_22*dmats1[22][25] + coeff0_23*dmats1[23][25] + coeff0_24*dmats1[24][25] + coeff0_25*dmats1[25][25] + coeff0_26*dmats1[26][25] + coeff0_27*dmats1[27][25] + coeff0_28*dmats1[28][25] + coeff0_29*dmats1[29][25] + coeff0_30*dmats1[30][25] + coeff0_31*dmats1[31][25] + coeff0_32*dmats1[32][25] + coeff0_33*dmats1[33][25] + coeff0_34*dmats1[34][25] + coeff0_35*dmats1[35][25] + coeff0_36*dmats1[36][25] + coeff0_37*dmats1[37][25] + coeff0_38*dmats1[38][25] + coeff0_39*dmats1[39][25] + coeff0_40*dmats1[40][25] + coeff0_41*dmats1[41][25] + coeff0_42*dmats1[42][25] + coeff0_43*dmats1[43][25] + coeff0_44*dmats1[44][25] + coeff0_45*dmats1[45][25] + coeff0_46*dmats1[46][25] + coeff0_47*dmats1[47][25] + coeff0_48*dmats1[48][25] + coeff0_49*dmats1[49][25] + coeff0_50*dmats1[50][25] + coeff0_51*dmats1[51][25] + coeff0_52*dmats1[52][25] + coeff0_53*dmats1[53][25] + coeff0_54*dmats1[54][25] + coeff0_55*dmats1[55][25];
          new_coeff0_26 = coeff0_0*dmats1[0][26] + coeff0_1*dmats1[1][26] + coeff0_2*dmats1[2][26] + coeff0_3*dmats1[3][26] + coeff0_4*dmats1[4][26] + coeff0_5*dmats1[5][26] + coeff0_6*dmats1[6][26] + coeff0_7*dmats1[7][26] + coeff0_8*dmats1[8][26] + coeff0_9*dmats1[9][26] + coeff0_10*dmats1[10][26] + coeff0_11*dmats1[11][26] + coeff0_12*dmats1[12][26] + coeff0_13*dmats1[13][26] + coeff0_14*dmats1[14][26] + coeff0_15*dmats1[15][26] + coeff0_16*dmats1[16][26] + coeff0_17*dmats1[17][26] + coeff0_18*dmats1[18][26] + coeff0_19*dmats1[19][26] + coeff0_20*dmats1[20][26] + coeff0_21*dmats1[21][26] + coeff0_22*dmats1[22][26] + coeff0_23*dmats1[23][26] + coeff0_24*dmats1[24][26] + coeff0_25*dmats1[25][26] + coeff0_26*dmats1[26][26] + coeff0_27*dmats1[27][26] + coeff0_28*dmats1[28][26] + coeff0_29*dmats1[29][26] + coeff0_30*dmats1[30][26] + coeff0_31*dmats1[31][26] + coeff0_32*dmats1[32][26] + coeff0_33*dmats1[33][26] + coeff0_34*dmats1[34][26] + coeff0_35*dmats1[35][26] + coeff0_36*dmats1[36][26] + coeff0_37*dmats1[37][26] + coeff0_38*dmats1[38][26] + coeff0_39*dmats1[39][26] + coeff0_40*dmats1[40][26] + coeff0_41*dmats1[41][26] + coeff0_42*dmats1[42][26] + coeff0_43*dmats1[43][26] + coeff0_44*dmats1[44][26] + coeff0_45*dmats1[45][26] + coeff0_46*dmats1[46][26] + coeff0_47*dmats1[47][26] + coeff0_48*dmats1[48][26] + coeff0_49*dmats1[49][26] + coeff0_50*dmats1[50][26] + coeff0_51*dmats1[51][26] + coeff0_52*dmats1[52][26] + coeff0_53*dmats1[53][26] + coeff0_54*dmats1[54][26] + coeff0_55*dmats1[55][26];
          new_coeff0_27 = coeff0_0*dmats1[0][27] + coeff0_1*dmats1[1][27] + coeff0_2*dmats1[2][27] + coeff0_3*dmats1[3][27] + coeff0_4*dmats1[4][27] + coeff0_5*dmats1[5][27] + coeff0_6*dmats1[6][27] + coeff0_7*dmats1[7][27] + coeff0_8*dmats1[8][27] + coeff0_9*dmats1[9][27] + coeff0_10*dmats1[10][27] + coeff0_11*dmats1[11][27] + coeff0_12*dmats1[12][27] + coeff0_13*dmats1[13][27] + coeff0_14*dmats1[14][27] + coeff0_15*dmats1[15][27] + coeff0_16*dmats1[16][27] + coeff0_17*dmats1[17][27] + coeff0_18*dmats1[18][27] + coeff0_19*dmats1[19][27] + coeff0_20*dmats1[20][27] + coeff0_21*dmats1[21][27] + coeff0_22*dmats1[22][27] + coeff0_23*dmats1[23][27] + coeff0_24*dmats1[24][27] + coeff0_25*dmats1[25][27] + coeff0_26*dmats1[26][27] + coeff0_27*dmats1[27][27] + coeff0_28*dmats1[28][27] + coeff0_29*dmats1[29][27] + coeff0_30*dmats1[30][27] + coeff0_31*dmats1[31][27] + coeff0_32*dmats1[32][27] + coeff0_33*dmats1[33][27] + coeff0_34*dmats1[34][27] + coeff0_35*dmats1[35][27] + coeff0_36*dmats1[36][27] + coeff0_37*dmats1[37][27] + coeff0_38*dmats1[38][27] + coeff0_39*dmats1[39][27] + coeff0_40*dmats1[40][27] + coeff0_41*dmats1[41][27] + coeff0_42*dmats1[42][27] + coeff0_43*dmats1[43][27] + coeff0_44*dmats1[44][27] + coeff0_45*dmats1[45][27] + coeff0_46*dmats1[46][27] + coeff0_47*dmats1[47][27] + coeff0_48*dmats1[48][27] + coeff0_49*dmats1[49][27] + coeff0_50*dmats1[50][27] + coeff0_51*dmats1[51][27] + coeff0_52*dmats1[52][27] + coeff0_53*dmats1[53][27] + coeff0_54*dmats1[54][27] + coeff0_55*dmats1[55][27];
          new_coeff0_28 = coeff0_0*dmats1[0][28] + coeff0_1*dmats1[1][28] + coeff0_2*dmats1[2][28] + coeff0_3*dmats1[3][28] + coeff0_4*dmats1[4][28] + coeff0_5*dmats1[5][28] + coeff0_6*dmats1[6][28] + coeff0_7*dmats1[7][28] + coeff0_8*dmats1[8][28] + coeff0_9*dmats1[9][28] + coeff0_10*dmats1[10][28] + coeff0_11*dmats1[11][28] + coeff0_12*dmats1[12][28] + coeff0_13*dmats1[13][28] + coeff0_14*dmats1[14][28] + coeff0_15*dmats1[15][28] + coeff0_16*dmats1[16][28] + coeff0_17*dmats1[17][28] + coeff0_18*dmats1[18][28] + coeff0_19*dmats1[19][28] + coeff0_20*dmats1[20][28] + coeff0_21*dmats1[21][28] + coeff0_22*dmats1[22][28] + coeff0_23*dmats1[23][28] + coeff0_24*dmats1[24][28] + coeff0_25*dmats1[25][28] + coeff0_26*dmats1[26][28] + coeff0_27*dmats1[27][28] + coeff0_28*dmats1[28][28] + coeff0_29*dmats1[29][28] + coeff0_30*dmats1[30][28] + coeff0_31*dmats1[31][28] + coeff0_32*dmats1[32][28] + coeff0_33*dmats1[33][28] + coeff0_34*dmats1[34][28] + coeff0_35*dmats1[35][28] + coeff0_36*dmats1[36][28] + coeff0_37*dmats1[37][28] + coeff0_38*dmats1[38][28] + coeff0_39*dmats1[39][28] + coeff0_40*dmats1[40][28] + coeff0_41*dmats1[41][28] + coeff0_42*dmats1[42][28] + coeff0_43*dmats1[43][28] + coeff0_44*dmats1[44][28] + coeff0_45*dmats1[45][28] + coeff0_46*dmats1[46][28] + coeff0_47*dmats1[47][28] + coeff0_48*dmats1[48][28] + coeff0_49*dmats1[49][28] + coeff0_50*dmats1[50][28] + coeff0_51*dmats1[51][28] + coeff0_52*dmats1[52][28] + coeff0_53*dmats1[53][28] + coeff0_54*dmats1[54][28] + coeff0_55*dmats1[55][28];
          new_coeff0_29 = coeff0_0*dmats1[0][29] + coeff0_1*dmats1[1][29] + coeff0_2*dmats1[2][29] + coeff0_3*dmats1[3][29] + coeff0_4*dmats1[4][29] + coeff0_5*dmats1[5][29] + coeff0_6*dmats1[6][29] + coeff0_7*dmats1[7][29] + coeff0_8*dmats1[8][29] + coeff0_9*dmats1[9][29] + coeff0_10*dmats1[10][29] + coeff0_11*dmats1[11][29] + coeff0_12*dmats1[12][29] + coeff0_13*dmats1[13][29] + coeff0_14*dmats1[14][29] + coeff0_15*dmats1[15][29] + coeff0_16*dmats1[16][29] + coeff0_17*dmats1[17][29] + coeff0_18*dmats1[18][29] + coeff0_19*dmats1[19][29] + coeff0_20*dmats1[20][29] + coeff0_21*dmats1[21][29] + coeff0_22*dmats1[22][29] + coeff0_23*dmats1[23][29] + coeff0_24*dmats1[24][29] + coeff0_25*dmats1[25][29] + coeff0_26*dmats1[26][29] + coeff0_27*dmats1[27][29] + coeff0_28*dmats1[28][29] + coeff0_29*dmats1[29][29] + coeff0_30*dmats1[30][29] + coeff0_31*dmats1[31][29] + coeff0_32*dmats1[32][29] + coeff0_33*dmats1[33][29] + coeff0_34*dmats1[34][29] + coeff0_35*dmats1[35][29] + coeff0_36*dmats1[36][29] + coeff0_37*dmats1[37][29] + coeff0_38*dmats1[38][29] + coeff0_39*dmats1[39][29] + coeff0_40*dmats1[40][29] + coeff0_41*dmats1[41][29] + coeff0_42*dmats1[42][29] + coeff0_43*dmats1[43][29] + coeff0_44*dmats1[44][29] + coeff0_45*dmats1[45][29] + coeff0_46*dmats1[46][29] + coeff0_47*dmats1[47][29] + coeff0_48*dmats1[48][29] + coeff0_49*dmats1[49][29] + coeff0_50*dmats1[50][29] + coeff0_51*dmats1[51][29] + coeff0_52*dmats1[52][29] + coeff0_53*dmats1[53][29] + coeff0_54*dmats1[54][29] + coeff0_55*dmats1[55][29];
          new_coeff0_30 = coeff0_0*dmats1[0][30] + coeff0_1*dmats1[1][30] + coeff0_2*dmats1[2][30] + coeff0_3*dmats1[3][30] + coeff0_4*dmats1[4][30] + coeff0_5*dmats1[5][30] + coeff0_6*dmats1[6][30] + coeff0_7*dmats1[7][30] + coeff0_8*dmats1[8][30] + coeff0_9*dmats1[9][30] + coeff0_10*dmats1[10][30] + coeff0_11*dmats1[11][30] + coeff0_12*dmats1[12][30] + coeff0_13*dmats1[13][30] + coeff0_14*dmats1[14][30] + coeff0_15*dmats1[15][30] + coeff0_16*dmats1[16][30] + coeff0_17*dmats1[17][30] + coeff0_18*dmats1[18][30] + coeff0_19*dmats1[19][30] + coeff0_20*dmats1[20][30] + coeff0_21*dmats1[21][30] + coeff0_22*dmats1[22][30] + coeff0_23*dmats1[23][30] + coeff0_24*dmats1[24][30] + coeff0_25*dmats1[25][30] + coeff0_26*dmats1[26][30] + coeff0_27*dmats1[27][30] + coeff0_28*dmats1[28][30] + coeff0_29*dmats1[29][30] + coeff0_30*dmats1[30][30] + coeff0_31*dmats1[31][30] + coeff0_32*dmats1[32][30] + coeff0_33*dmats1[33][30] + coeff0_34*dmats1[34][30] + coeff0_35*dmats1[35][30] + coeff0_36*dmats1[36][30] + coeff0_37*dmats1[37][30] + coeff0_38*dmats1[38][30] + coeff0_39*dmats1[39][30] + coeff0_40*dmats1[40][30] + coeff0_41*dmats1[41][30] + coeff0_42*dmats1[42][30] + coeff0_43*dmats1[43][30] + coeff0_44*dmats1[44][30] + coeff0_45*dmats1[45][30] + coeff0_46*dmats1[46][30] + coeff0_47*dmats1[47][30] + coeff0_48*dmats1[48][30] + coeff0_49*dmats1[49][30] + coeff0_50*dmats1[50][30] + coeff0_51*dmats1[51][30] + coeff0_52*dmats1[52][30] + coeff0_53*dmats1[53][30] + coeff0_54*dmats1[54][30] + coeff0_55*dmats1[55][30];
          new_coeff0_31 = coeff0_0*dmats1[0][31] + coeff0_1*dmats1[1][31] + coeff0_2*dmats1[2][31] + coeff0_3*dmats1[3][31] + coeff0_4*dmats1[4][31] + coeff0_5*dmats1[5][31] + coeff0_6*dmats1[6][31] + coeff0_7*dmats1[7][31] + coeff0_8*dmats1[8][31] + coeff0_9*dmats1[9][31] + coeff0_10*dmats1[10][31] + coeff0_11*dmats1[11][31] + coeff0_12*dmats1[12][31] + coeff0_13*dmats1[13][31] + coeff0_14*dmats1[14][31] + coeff0_15*dmats1[15][31] + coeff0_16*dmats1[16][31] + coeff0_17*dmats1[17][31] + coeff0_18*dmats1[18][31] + coeff0_19*dmats1[19][31] + coeff0_20*dmats1[20][31] + coeff0_21*dmats1[21][31] + coeff0_22*dmats1[22][31] + coeff0_23*dmats1[23][31] + coeff0_24*dmats1[24][31] + coeff0_25*dmats1[25][31] + coeff0_26*dmats1[26][31] + coeff0_27*dmats1[27][31] + coeff0_28*dmats1[28][31] + coeff0_29*dmats1[29][31] + coeff0_30*dmats1[30][31] + coeff0_31*dmats1[31][31] + coeff0_32*dmats1[32][31] + coeff0_33*dmats1[33][31] + coeff0_34*dmats1[34][31] + coeff0_35*dmats1[35][31] + coeff0_36*dmats1[36][31] + coeff0_37*dmats1[37][31] + coeff0_38*dmats1[38][31] + coeff0_39*dmats1[39][31] + coeff0_40*dmats1[40][31] + coeff0_41*dmats1[41][31] + coeff0_42*dmats1[42][31] + coeff0_43*dmats1[43][31] + coeff0_44*dmats1[44][31] + coeff0_45*dmats1[45][31] + coeff0_46*dmats1[46][31] + coeff0_47*dmats1[47][31] + coeff0_48*dmats1[48][31] + coeff0_49*dmats1[49][31] + coeff0_50*dmats1[50][31] + coeff0_51*dmats1[51][31] + coeff0_52*dmats1[52][31] + coeff0_53*dmats1[53][31] + coeff0_54*dmats1[54][31] + coeff0_55*dmats1[55][31];
          new_coeff0_32 = coeff0_0*dmats1[0][32] + coeff0_1*dmats1[1][32] + coeff0_2*dmats1[2][32] + coeff0_3*dmats1[3][32] + coeff0_4*dmats1[4][32] + coeff0_5*dmats1[5][32] + coeff0_6*dmats1[6][32] + coeff0_7*dmats1[7][32] + coeff0_8*dmats1[8][32] + coeff0_9*dmats1[9][32] + coeff0_10*dmats1[10][32] + coeff0_11*dmats1[11][32] + coeff0_12*dmats1[12][32] + coeff0_13*dmats1[13][32] + coeff0_14*dmats1[14][32] + coeff0_15*dmats1[15][32] + coeff0_16*dmats1[16][32] + coeff0_17*dmats1[17][32] + coeff0_18*dmats1[18][32] + coeff0_19*dmats1[19][32] + coeff0_20*dmats1[20][32] + coeff0_21*dmats1[21][32] + coeff0_22*dmats1[22][32] + coeff0_23*dmats1[23][32] + coeff0_24*dmats1[24][32] + coeff0_25*dmats1[25][32] + coeff0_26*dmats1[26][32] + coeff0_27*dmats1[27][32] + coeff0_28*dmats1[28][32] + coeff0_29*dmats1[29][32] + coeff0_30*dmats1[30][32] + coeff0_31*dmats1[31][32] + coeff0_32*dmats1[32][32] + coeff0_33*dmats1[33][32] + coeff0_34*dmats1[34][32] + coeff0_35*dmats1[35][32] + coeff0_36*dmats1[36][32] + coeff0_37*dmats1[37][32] + coeff0_38*dmats1[38][32] + coeff0_39*dmats1[39][32] + coeff0_40*dmats1[40][32] + coeff0_41*dmats1[41][32] + coeff0_42*dmats1[42][32] + coeff0_43*dmats1[43][32] + coeff0_44*dmats1[44][32] + coeff0_45*dmats1[45][32] + coeff0_46*dmats1[46][32] + coeff0_47*dmats1[47][32] + coeff0_48*dmats1[48][32] + coeff0_49*dmats1[49][32] + coeff0_50*dmats1[50][32] + coeff0_51*dmats1[51][32] + coeff0_52*dmats1[52][32] + coeff0_53*dmats1[53][32] + coeff0_54*dmats1[54][32] + coeff0_55*dmats1[55][32];
          new_coeff0_33 = coeff0_0*dmats1[0][33] + coeff0_1*dmats1[1][33] + coeff0_2*dmats1[2][33] + coeff0_3*dmats1[3][33] + coeff0_4*dmats1[4][33] + coeff0_5*dmats1[5][33] + coeff0_6*dmats1[6][33] + coeff0_7*dmats1[7][33] + coeff0_8*dmats1[8][33] + coeff0_9*dmats1[9][33] + coeff0_10*dmats1[10][33] + coeff0_11*dmats1[11][33] + coeff0_12*dmats1[12][33] + coeff0_13*dmats1[13][33] + coeff0_14*dmats1[14][33] + coeff0_15*dmats1[15][33] + coeff0_16*dmats1[16][33] + coeff0_17*dmats1[17][33] + coeff0_18*dmats1[18][33] + coeff0_19*dmats1[19][33] + coeff0_20*dmats1[20][33] + coeff0_21*dmats1[21][33] + coeff0_22*dmats1[22][33] + coeff0_23*dmats1[23][33] + coeff0_24*dmats1[24][33] + coeff0_25*dmats1[25][33] + coeff0_26*dmats1[26][33] + coeff0_27*dmats1[27][33] + coeff0_28*dmats1[28][33] + coeff0_29*dmats1[29][33] + coeff0_30*dmats1[30][33] + coeff0_31*dmats1[31][33] + coeff0_32*dmats1[32][33] + coeff0_33*dmats1[33][33] + coeff0_34*dmats1[34][33] + coeff0_35*dmats1[35][33] + coeff0_36*dmats1[36][33] + coeff0_37*dmats1[37][33] + coeff0_38*dmats1[38][33] + coeff0_39*dmats1[39][33] + coeff0_40*dmats1[40][33] + coeff0_41*dmats1[41][33] + coeff0_42*dmats1[42][33] + coeff0_43*dmats1[43][33] + coeff0_44*dmats1[44][33] + coeff0_45*dmats1[45][33] + coeff0_46*dmats1[46][33] + coeff0_47*dmats1[47][33] + coeff0_48*dmats1[48][33] + coeff0_49*dmats1[49][33] + coeff0_50*dmats1[50][33] + coeff0_51*dmats1[51][33] + coeff0_52*dmats1[52][33] + coeff0_53*dmats1[53][33] + coeff0_54*dmats1[54][33] + coeff0_55*dmats1[55][33];
          new_coeff0_34 = coeff0_0*dmats1[0][34] + coeff0_1*dmats1[1][34] + coeff0_2*dmats1[2][34] + coeff0_3*dmats1[3][34] + coeff0_4*dmats1[4][34] + coeff0_5*dmats1[5][34] + coeff0_6*dmats1[6][34] + coeff0_7*dmats1[7][34] + coeff0_8*dmats1[8][34] + coeff0_9*dmats1[9][34] + coeff0_10*dmats1[10][34] + coeff0_11*dmats1[11][34] + coeff0_12*dmats1[12][34] + coeff0_13*dmats1[13][34] + coeff0_14*dmats1[14][34] + coeff0_15*dmats1[15][34] + coeff0_16*dmats1[16][34] + coeff0_17*dmats1[17][34] + coeff0_18*dmats1[18][34] + coeff0_19*dmats1[19][34] + coeff0_20*dmats1[20][34] + coeff0_21*dmats1[21][34] + coeff0_22*dmats1[22][34] + coeff0_23*dmats1[23][34] + coeff0_24*dmats1[24][34] + coeff0_25*dmats1[25][34] + coeff0_26*dmats1[26][34] + coeff0_27*dmats1[27][34] + coeff0_28*dmats1[28][34] + coeff0_29*dmats1[29][34] + coeff0_30*dmats1[30][34] + coeff0_31*dmats1[31][34] + coeff0_32*dmats1[32][34] + coeff0_33*dmats1[33][34] + coeff0_34*dmats1[34][34] + coeff0_35*dmats1[35][34] + coeff0_36*dmats1[36][34] + coeff0_37*dmats1[37][34] + coeff0_38*dmats1[38][34] + coeff0_39*dmats1[39][34] + coeff0_40*dmats1[40][34] + coeff0_41*dmats1[41][34] + coeff0_42*dmats1[42][34] + coeff0_43*dmats1[43][34] + coeff0_44*dmats1[44][34] + coeff0_45*dmats1[45][34] + coeff0_46*dmats1[46][34] + coeff0_47*dmats1[47][34] + coeff0_48*dmats1[48][34] + coeff0_49*dmats1[49][34] + coeff0_50*dmats1[50][34] + coeff0_51*dmats1[51][34] + coeff0_52*dmats1[52][34] + coeff0_53*dmats1[53][34] + coeff0_54*dmats1[54][34] + coeff0_55*dmats1[55][34];
          new_coeff0_35 = coeff0_0*dmats1[0][35] + coeff0_1*dmats1[1][35] + coeff0_2*dmats1[2][35] + coeff0_3*dmats1[3][35] + coeff0_4*dmats1[4][35] + coeff0_5*dmats1[5][35] + coeff0_6*dmats1[6][35] + coeff0_7*dmats1[7][35] + coeff0_8*dmats1[8][35] + coeff0_9*dmats1[9][35] + coeff0_10*dmats1[10][35] + coeff0_11*dmats1[11][35] + coeff0_12*dmats1[12][35] + coeff0_13*dmats1[13][35] + coeff0_14*dmats1[14][35] + coeff0_15*dmats1[15][35] + coeff0_16*dmats1[16][35] + coeff0_17*dmats1[17][35] + coeff0_18*dmats1[18][35] + coeff0_19*dmats1[19][35] + coeff0_20*dmats1[20][35] + coeff0_21*dmats1[21][35] + coeff0_22*dmats1[22][35] + coeff0_23*dmats1[23][35] + coeff0_24*dmats1[24][35] + coeff0_25*dmats1[25][35] + coeff0_26*dmats1[26][35] + coeff0_27*dmats1[27][35] + coeff0_28*dmats1[28][35] + coeff0_29*dmats1[29][35] + coeff0_30*dmats1[30][35] + coeff0_31*dmats1[31][35] + coeff0_32*dmats1[32][35] + coeff0_33*dmats1[33][35] + coeff0_34*dmats1[34][35] + coeff0_35*dmats1[35][35] + coeff0_36*dmats1[36][35] + coeff0_37*dmats1[37][35] + coeff0_38*dmats1[38][35] + coeff0_39*dmats1[39][35] + coeff0_40*dmats1[40][35] + coeff0_41*dmats1[41][35] + coeff0_42*dmats1[42][35] + coeff0_43*dmats1[43][35] + coeff0_44*dmats1[44][35] + coeff0_45*dmats1[45][35] + coeff0_46*dmats1[46][35] + coeff0_47*dmats1[47][35] + coeff0_48*dmats1[48][35] + coeff0_49*dmats1[49][35] + coeff0_50*dmats1[50][35] + coeff0_51*dmats1[51][35] + coeff0_52*dmats1[52][35] + coeff0_53*dmats1[53][35] + coeff0_54*dmats1[54][35] + coeff0_55*dmats1[55][35];
          new_coeff0_36 = coeff0_0*dmats1[0][36] + coeff0_1*dmats1[1][36] + coeff0_2*dmats1[2][36] + coeff0_3*dmats1[3][36] + coeff0_4*dmats1[4][36] + coeff0_5*dmats1[5][36] + coeff0_6*dmats1[6][36] + coeff0_7*dmats1[7][36] + coeff0_8*dmats1[8][36] + coeff0_9*dmats1[9][36] + coeff0_10*dmats1[10][36] + coeff0_11*dmats1[11][36] + coeff0_12*dmats1[12][36] + coeff0_13*dmats1[13][36] + coeff0_14*dmats1[14][36] + coeff0_15*dmats1[15][36] + coeff0_16*dmats1[16][36] + coeff0_17*dmats1[17][36] + coeff0_18*dmats1[18][36] + coeff0_19*dmats1[19][36] + coeff0_20*dmats1[20][36] + coeff0_21*dmats1[21][36] + coeff0_22*dmats1[22][36] + coeff0_23*dmats1[23][36] + coeff0_24*dmats1[24][36] + coeff0_25*dmats1[25][36] + coeff0_26*dmats1[26][36] + coeff0_27*dmats1[27][36] + coeff0_28*dmats1[28][36] + coeff0_29*dmats1[29][36] + coeff0_30*dmats1[30][36] + coeff0_31*dmats1[31][36] + coeff0_32*dmats1[32][36] + coeff0_33*dmats1[33][36] + coeff0_34*dmats1[34][36] + coeff0_35*dmats1[35][36] + coeff0_36*dmats1[36][36] + coeff0_37*dmats1[37][36] + coeff0_38*dmats1[38][36] + coeff0_39*dmats1[39][36] + coeff0_40*dmats1[40][36] + coeff0_41*dmats1[41][36] + coeff0_42*dmats1[42][36] + coeff0_43*dmats1[43][36] + coeff0_44*dmats1[44][36] + coeff0_45*dmats1[45][36] + coeff0_46*dmats1[46][36] + coeff0_47*dmats1[47][36] + coeff0_48*dmats1[48][36] + coeff0_49*dmats1[49][36] + coeff0_50*dmats1[50][36] + coeff0_51*dmats1[51][36] + coeff0_52*dmats1[52][36] + coeff0_53*dmats1[53][36] + coeff0_54*dmats1[54][36] + coeff0_55*dmats1[55][36];
          new_coeff0_37 = coeff0_0*dmats1[0][37] + coeff0_1*dmats1[1][37] + coeff0_2*dmats1[2][37] + coeff0_3*dmats1[3][37] + coeff0_4*dmats1[4][37] + coeff0_5*dmats1[5][37] + coeff0_6*dmats1[6][37] + coeff0_7*dmats1[7][37] + coeff0_8*dmats1[8][37] + coeff0_9*dmats1[9][37] + coeff0_10*dmats1[10][37] + coeff0_11*dmats1[11][37] + coeff0_12*dmats1[12][37] + coeff0_13*dmats1[13][37] + coeff0_14*dmats1[14][37] + coeff0_15*dmats1[15][37] + coeff0_16*dmats1[16][37] + coeff0_17*dmats1[17][37] + coeff0_18*dmats1[18][37] + coeff0_19*dmats1[19][37] + coeff0_20*dmats1[20][37] + coeff0_21*dmats1[21][37] + coeff0_22*dmats1[22][37] + coeff0_23*dmats1[23][37] + coeff0_24*dmats1[24][37] + coeff0_25*dmats1[25][37] + coeff0_26*dmats1[26][37] + coeff0_27*dmats1[27][37] + coeff0_28*dmats1[28][37] + coeff0_29*dmats1[29][37] + coeff0_30*dmats1[30][37] + coeff0_31*dmats1[31][37] + coeff0_32*dmats1[32][37] + coeff0_33*dmats1[33][37] + coeff0_34*dmats1[34][37] + coeff0_35*dmats1[35][37] + coeff0_36*dmats1[36][37] + coeff0_37*dmats1[37][37] + coeff0_38*dmats1[38][37] + coeff0_39*dmats1[39][37] + coeff0_40*dmats1[40][37] + coeff0_41*dmats1[41][37] + coeff0_42*dmats1[42][37] + coeff0_43*dmats1[43][37] + coeff0_44*dmats1[44][37] + coeff0_45*dmats1[45][37] + coeff0_46*dmats1[46][37] + coeff0_47*dmats1[47][37] + coeff0_48*dmats1[48][37] + coeff0_49*dmats1[49][37] + coeff0_50*dmats1[50][37] + coeff0_51*dmats1[51][37] + coeff0_52*dmats1[52][37] + coeff0_53*dmats1[53][37] + coeff0_54*dmats1[54][37] + coeff0_55*dmats1[55][37];
          new_coeff0_38 = coeff0_0*dmats1[0][38] + coeff0_1*dmats1[1][38] + coeff0_2*dmats1[2][38] + coeff0_3*dmats1[3][38] + coeff0_4*dmats1[4][38] + coeff0_5*dmats1[5][38] + coeff0_6*dmats1[6][38] + coeff0_7*dmats1[7][38] + coeff0_8*dmats1[8][38] + coeff0_9*dmats1[9][38] + coeff0_10*dmats1[10][38] + coeff0_11*dmats1[11][38] + coeff0_12*dmats1[12][38] + coeff0_13*dmats1[13][38] + coeff0_14*dmats1[14][38] + coeff0_15*dmats1[15][38] + coeff0_16*dmats1[16][38] + coeff0_17*dmats1[17][38] + coeff0_18*dmats1[18][38] + coeff0_19*dmats1[19][38] + coeff0_20*dmats1[20][38] + coeff0_21*dmats1[21][38] + coeff0_22*dmats1[22][38] + coeff0_23*dmats1[23][38] + coeff0_24*dmats1[24][38] + coeff0_25*dmats1[25][38] + coeff0_26*dmats1[26][38] + coeff0_27*dmats1[27][38] + coeff0_28*dmats1[28][38] + coeff0_29*dmats1[29][38] + coeff0_30*dmats1[30][38] + coeff0_31*dmats1[31][38] + coeff0_32*dmats1[32][38] + coeff0_33*dmats1[33][38] + coeff0_34*dmats1[34][38] + coeff0_35*dmats1[35][38] + coeff0_36*dmats1[36][38] + coeff0_37*dmats1[37][38] + coeff0_38*dmats1[38][38] + coeff0_39*dmats1[39][38] + coeff0_40*dmats1[40][38] + coeff0_41*dmats1[41][38] + coeff0_42*dmats1[42][38] + coeff0_43*dmats1[43][38] + coeff0_44*dmats1[44][38] + coeff0_45*dmats1[45][38] + coeff0_46*dmats1[46][38] + coeff0_47*dmats1[47][38] + coeff0_48*dmats1[48][38] + coeff0_49*dmats1[49][38] + coeff0_50*dmats1[50][38] + coeff0_51*dmats1[51][38] + coeff0_52*dmats1[52][38] + coeff0_53*dmats1[53][38] + coeff0_54*dmats1[54][38] + coeff0_55*dmats1[55][38];
          new_coeff0_39 = coeff0_0*dmats1[0][39] + coeff0_1*dmats1[1][39] + coeff0_2*dmats1[2][39] + coeff0_3*dmats1[3][39] + coeff0_4*dmats1[4][39] + coeff0_5*dmats1[5][39] + coeff0_6*dmats1[6][39] + coeff0_7*dmats1[7][39] + coeff0_8*dmats1[8][39] + coeff0_9*dmats1[9][39] + coeff0_10*dmats1[10][39] + coeff0_11*dmats1[11][39] + coeff0_12*dmats1[12][39] + coeff0_13*dmats1[13][39] + coeff0_14*dmats1[14][39] + coeff0_15*dmats1[15][39] + coeff0_16*dmats1[16][39] + coeff0_17*dmats1[17][39] + coeff0_18*dmats1[18][39] + coeff0_19*dmats1[19][39] + coeff0_20*dmats1[20][39] + coeff0_21*dmats1[21][39] + coeff0_22*dmats1[22][39] + coeff0_23*dmats1[23][39] + coeff0_24*dmats1[24][39] + coeff0_25*dmats1[25][39] + coeff0_26*dmats1[26][39] + coeff0_27*dmats1[27][39] + coeff0_28*dmats1[28][39] + coeff0_29*dmats1[29][39] + coeff0_30*dmats1[30][39] + coeff0_31*dmats1[31][39] + coeff0_32*dmats1[32][39] + coeff0_33*dmats1[33][39] + coeff0_34*dmats1[34][39] + coeff0_35*dmats1[35][39] + coeff0_36*dmats1[36][39] + coeff0_37*dmats1[37][39] + coeff0_38*dmats1[38][39] + coeff0_39*dmats1[39][39] + coeff0_40*dmats1[40][39] + coeff0_41*dmats1[41][39] + coeff0_42*dmats1[42][39] + coeff0_43*dmats1[43][39] + coeff0_44*dmats1[44][39] + coeff0_45*dmats1[45][39] + coeff0_46*dmats1[46][39] + coeff0_47*dmats1[47][39] + coeff0_48*dmats1[48][39] + coeff0_49*dmats1[49][39] + coeff0_50*dmats1[50][39] + coeff0_51*dmats1[51][39] + coeff0_52*dmats1[52][39] + coeff0_53*dmats1[53][39] + coeff0_54*dmats1[54][39] + coeff0_55*dmats1[55][39];
          new_coeff0_40 = coeff0_0*dmats1[0][40] + coeff0_1*dmats1[1][40] + coeff0_2*dmats1[2][40] + coeff0_3*dmats1[3][40] + coeff0_4*dmats1[4][40] + coeff0_5*dmats1[5][40] + coeff0_6*dmats1[6][40] + coeff0_7*dmats1[7][40] + coeff0_8*dmats1[8][40] + coeff0_9*dmats1[9][40] + coeff0_10*dmats1[10][40] + coeff0_11*dmats1[11][40] + coeff0_12*dmats1[12][40] + coeff0_13*dmats1[13][40] + coeff0_14*dmats1[14][40] + coeff0_15*dmats1[15][40] + coeff0_16*dmats1[16][40] + coeff0_17*dmats1[17][40] + coeff0_18*dmats1[18][40] + coeff0_19*dmats1[19][40] + coeff0_20*dmats1[20][40] + coeff0_21*dmats1[21][40] + coeff0_22*dmats1[22][40] + coeff0_23*dmats1[23][40] + coeff0_24*dmats1[24][40] + coeff0_25*dmats1[25][40] + coeff0_26*dmats1[26][40] + coeff0_27*dmats1[27][40] + coeff0_28*dmats1[28][40] + coeff0_29*dmats1[29][40] + coeff0_30*dmats1[30][40] + coeff0_31*dmats1[31][40] + coeff0_32*dmats1[32][40] + coeff0_33*dmats1[33][40] + coeff0_34*dmats1[34][40] + coeff0_35*dmats1[35][40] + coeff0_36*dmats1[36][40] + coeff0_37*dmats1[37][40] + coeff0_38*dmats1[38][40] + coeff0_39*dmats1[39][40] + coeff0_40*dmats1[40][40] + coeff0_41*dmats1[41][40] + coeff0_42*dmats1[42][40] + coeff0_43*dmats1[43][40] + coeff0_44*dmats1[44][40] + coeff0_45*dmats1[45][40] + coeff0_46*dmats1[46][40] + coeff0_47*dmats1[47][40] + coeff0_48*dmats1[48][40] + coeff0_49*dmats1[49][40] + coeff0_50*dmats1[50][40] + coeff0_51*dmats1[51][40] + coeff0_52*dmats1[52][40] + coeff0_53*dmats1[53][40] + coeff0_54*dmats1[54][40] + coeff0_55*dmats1[55][40];
          new_coeff0_41 = coeff0_0*dmats1[0][41] + coeff0_1*dmats1[1][41] + coeff0_2*dmats1[2][41] + coeff0_3*dmats1[3][41] + coeff0_4*dmats1[4][41] + coeff0_5*dmats1[5][41] + coeff0_6*dmats1[6][41] + coeff0_7*dmats1[7][41] + coeff0_8*dmats1[8][41] + coeff0_9*dmats1[9][41] + coeff0_10*dmats1[10][41] + coeff0_11*dmats1[11][41] + coeff0_12*dmats1[12][41] + coeff0_13*dmats1[13][41] + coeff0_14*dmats1[14][41] + coeff0_15*dmats1[15][41] + coeff0_16*dmats1[16][41] + coeff0_17*dmats1[17][41] + coeff0_18*dmats1[18][41] + coeff0_19*dmats1[19][41] + coeff0_20*dmats1[20][41] + coeff0_21*dmats1[21][41] + coeff0_22*dmats1[22][41] + coeff0_23*dmats1[23][41] + coeff0_24*dmats1[24][41] + coeff0_25*dmats1[25][41] + coeff0_26*dmats1[26][41] + coeff0_27*dmats1[27][41] + coeff0_28*dmats1[28][41] + coeff0_29*dmats1[29][41] + coeff0_30*dmats1[30][41] + coeff0_31*dmats1[31][41] + coeff0_32*dmats1[32][41] + coeff0_33*dmats1[33][41] + coeff0_34*dmats1[34][41] + coeff0_35*dmats1[35][41] + coeff0_36*dmats1[36][41] + coeff0_37*dmats1[37][41] + coeff0_38*dmats1[38][41] + coeff0_39*dmats1[39][41] + coeff0_40*dmats1[40][41] + coeff0_41*dmats1[41][41] + coeff0_42*dmats1[42][41] + coeff0_43*dmats1[43][41] + coeff0_44*dmats1[44][41] + coeff0_45*dmats1[45][41] + coeff0_46*dmats1[46][41] + coeff0_47*dmats1[47][41] + coeff0_48*dmats1[48][41] + coeff0_49*dmats1[49][41] + coeff0_50*dmats1[50][41] + coeff0_51*dmats1[51][41] + coeff0_52*dmats1[52][41] + coeff0_53*dmats1[53][41] + coeff0_54*dmats1[54][41] + coeff0_55*dmats1[55][41];
          new_coeff0_42 = coeff0_0*dmats1[0][42] + coeff0_1*dmats1[1][42] + coeff0_2*dmats1[2][42] + coeff0_3*dmats1[3][42] + coeff0_4*dmats1[4][42] + coeff0_5*dmats1[5][42] + coeff0_6*dmats1[6][42] + coeff0_7*dmats1[7][42] + coeff0_8*dmats1[8][42] + coeff0_9*dmats1[9][42] + coeff0_10*dmats1[10][42] + coeff0_11*dmats1[11][42] + coeff0_12*dmats1[12][42] + coeff0_13*dmats1[13][42] + coeff0_14*dmats1[14][42] + coeff0_15*dmats1[15][42] + coeff0_16*dmats1[16][42] + coeff0_17*dmats1[17][42] + coeff0_18*dmats1[18][42] + coeff0_19*dmats1[19][42] + coeff0_20*dmats1[20][42] + coeff0_21*dmats1[21][42] + coeff0_22*dmats1[22][42] + coeff0_23*dmats1[23][42] + coeff0_24*dmats1[24][42] + coeff0_25*dmats1[25][42] + coeff0_26*dmats1[26][42] + coeff0_27*dmats1[27][42] + coeff0_28*dmats1[28][42] + coeff0_29*dmats1[29][42] + coeff0_30*dmats1[30][42] + coeff0_31*dmats1[31][42] + coeff0_32*dmats1[32][42] + coeff0_33*dmats1[33][42] + coeff0_34*dmats1[34][42] + coeff0_35*dmats1[35][42] + coeff0_36*dmats1[36][42] + coeff0_37*dmats1[37][42] + coeff0_38*dmats1[38][42] + coeff0_39*dmats1[39][42] + coeff0_40*dmats1[40][42] + coeff0_41*dmats1[41][42] + coeff0_42*dmats1[42][42] + coeff0_43*dmats1[43][42] + coeff0_44*dmats1[44][42] + coeff0_45*dmats1[45][42] + coeff0_46*dmats1[46][42] + coeff0_47*dmats1[47][42] + coeff0_48*dmats1[48][42] + coeff0_49*dmats1[49][42] + coeff0_50*dmats1[50][42] + coeff0_51*dmats1[51][42] + coeff0_52*dmats1[52][42] + coeff0_53*dmats1[53][42] + coeff0_54*dmats1[54][42] + coeff0_55*dmats1[55][42];
          new_coeff0_43 = coeff0_0*dmats1[0][43] + coeff0_1*dmats1[1][43] + coeff0_2*dmats1[2][43] + coeff0_3*dmats1[3][43] + coeff0_4*dmats1[4][43] + coeff0_5*dmats1[5][43] + coeff0_6*dmats1[6][43] + coeff0_7*dmats1[7][43] + coeff0_8*dmats1[8][43] + coeff0_9*dmats1[9][43] + coeff0_10*dmats1[10][43] + coeff0_11*dmats1[11][43] + coeff0_12*dmats1[12][43] + coeff0_13*dmats1[13][43] + coeff0_14*dmats1[14][43] + coeff0_15*dmats1[15][43] + coeff0_16*dmats1[16][43] + coeff0_17*dmats1[17][43] + coeff0_18*dmats1[18][43] + coeff0_19*dmats1[19][43] + coeff0_20*dmats1[20][43] + coeff0_21*dmats1[21][43] + coeff0_22*dmats1[22][43] + coeff0_23*dmats1[23][43] + coeff0_24*dmats1[24][43] + coeff0_25*dmats1[25][43] + coeff0_26*dmats1[26][43] + coeff0_27*dmats1[27][43] + coeff0_28*dmats1[28][43] + coeff0_29*dmats1[29][43] + coeff0_30*dmats1[30][43] + coeff0_31*dmats1[31][43] + coeff0_32*dmats1[32][43] + coeff0_33*dmats1[33][43] + coeff0_34*dmats1[34][43] + coeff0_35*dmats1[35][43] + coeff0_36*dmats1[36][43] + coeff0_37*dmats1[37][43] + coeff0_38*dmats1[38][43] + coeff0_39*dmats1[39][43] + coeff0_40*dmats1[40][43] + coeff0_41*dmats1[41][43] + coeff0_42*dmats1[42][43] + coeff0_43*dmats1[43][43] + coeff0_44*dmats1[44][43] + coeff0_45*dmats1[45][43] + coeff0_46*dmats1[46][43] + coeff0_47*dmats1[47][43] + coeff0_48*dmats1[48][43] + coeff0_49*dmats1[49][43] + coeff0_50*dmats1[50][43] + coeff0_51*dmats1[51][43] + coeff0_52*dmats1[52][43] + coeff0_53*dmats1[53][43] + coeff0_54*dmats1[54][43] + coeff0_55*dmats1[55][43];
          new_coeff0_44 = coeff0_0*dmats1[0][44] + coeff0_1*dmats1[1][44] + coeff0_2*dmats1[2][44] + coeff0_3*dmats1[3][44] + coeff0_4*dmats1[4][44] + coeff0_5*dmats1[5][44] + coeff0_6*dmats1[6][44] + coeff0_7*dmats1[7][44] + coeff0_8*dmats1[8][44] + coeff0_9*dmats1[9][44] + coeff0_10*dmats1[10][44] + coeff0_11*dmats1[11][44] + coeff0_12*dmats1[12][44] + coeff0_13*dmats1[13][44] + coeff0_14*dmats1[14][44] + coeff0_15*dmats1[15][44] + coeff0_16*dmats1[16][44] + coeff0_17*dmats1[17][44] + coeff0_18*dmats1[18][44] + coeff0_19*dmats1[19][44] + coeff0_20*dmats1[20][44] + coeff0_21*dmats1[21][44] + coeff0_22*dmats1[22][44] + coeff0_23*dmats1[23][44] + coeff0_24*dmats1[24][44] + coeff0_25*dmats1[25][44] + coeff0_26*dmats1[26][44] + coeff0_27*dmats1[27][44] + coeff0_28*dmats1[28][44] + coeff0_29*dmats1[29][44] + coeff0_30*dmats1[30][44] + coeff0_31*dmats1[31][44] + coeff0_32*dmats1[32][44] + coeff0_33*dmats1[33][44] + coeff0_34*dmats1[34][44] + coeff0_35*dmats1[35][44] + coeff0_36*dmats1[36][44] + coeff0_37*dmats1[37][44] + coeff0_38*dmats1[38][44] + coeff0_39*dmats1[39][44] + coeff0_40*dmats1[40][44] + coeff0_41*dmats1[41][44] + coeff0_42*dmats1[42][44] + coeff0_43*dmats1[43][44] + coeff0_44*dmats1[44][44] + coeff0_45*dmats1[45][44] + coeff0_46*dmats1[46][44] + coeff0_47*dmats1[47][44] + coeff0_48*dmats1[48][44] + coeff0_49*dmats1[49][44] + coeff0_50*dmats1[50][44] + coeff0_51*dmats1[51][44] + coeff0_52*dmats1[52][44] + coeff0_53*dmats1[53][44] + coeff0_54*dmats1[54][44] + coeff0_55*dmats1[55][44];
          new_coeff0_45 = coeff0_0*dmats1[0][45] + coeff0_1*dmats1[1][45] + coeff0_2*dmats1[2][45] + coeff0_3*dmats1[3][45] + coeff0_4*dmats1[4][45] + coeff0_5*dmats1[5][45] + coeff0_6*dmats1[6][45] + coeff0_7*dmats1[7][45] + coeff0_8*dmats1[8][45] + coeff0_9*dmats1[9][45] + coeff0_10*dmats1[10][45] + coeff0_11*dmats1[11][45] + coeff0_12*dmats1[12][45] + coeff0_13*dmats1[13][45] + coeff0_14*dmats1[14][45] + coeff0_15*dmats1[15][45] + coeff0_16*dmats1[16][45] + coeff0_17*dmats1[17][45] + coeff0_18*dmats1[18][45] + coeff0_19*dmats1[19][45] + coeff0_20*dmats1[20][45] + coeff0_21*dmats1[21][45] + coeff0_22*dmats1[22][45] + coeff0_23*dmats1[23][45] + coeff0_24*dmats1[24][45] + coeff0_25*dmats1[25][45] + coeff0_26*dmats1[26][45] + coeff0_27*dmats1[27][45] + coeff0_28*dmats1[28][45] + coeff0_29*dmats1[29][45] + coeff0_30*dmats1[30][45] + coeff0_31*dmats1[31][45] + coeff0_32*dmats1[32][45] + coeff0_33*dmats1[33][45] + coeff0_34*dmats1[34][45] + coeff0_35*dmats1[35][45] + coeff0_36*dmats1[36][45] + coeff0_37*dmats1[37][45] + coeff0_38*dmats1[38][45] + coeff0_39*dmats1[39][45] + coeff0_40*dmats1[40][45] + coeff0_41*dmats1[41][45] + coeff0_42*dmats1[42][45] + coeff0_43*dmats1[43][45] + coeff0_44*dmats1[44][45] + coeff0_45*dmats1[45][45] + coeff0_46*dmats1[46][45] + coeff0_47*dmats1[47][45] + coeff0_48*dmats1[48][45] + coeff0_49*dmats1[49][45] + coeff0_50*dmats1[50][45] + coeff0_51*dmats1[51][45] + coeff0_52*dmats1[52][45] + coeff0_53*dmats1[53][45] + coeff0_54*dmats1[54][45] + coeff0_55*dmats1[55][45];
          new_coeff0_46 = coeff0_0*dmats1[0][46] + coeff0_1*dmats1[1][46] + coeff0_2*dmats1[2][46] + coeff0_3*dmats1[3][46] + coeff0_4*dmats1[4][46] + coeff0_5*dmats1[5][46] + coeff0_6*dmats1[6][46] + coeff0_7*dmats1[7][46] + coeff0_8*dmats1[8][46] + coeff0_9*dmats1[9][46] + coeff0_10*dmats1[10][46] + coeff0_11*dmats1[11][46] + coeff0_12*dmats1[12][46] + coeff0_13*dmats1[13][46] + coeff0_14*dmats1[14][46] + coeff0_15*dmats1[15][46] + coeff0_16*dmats1[16][46] + coeff0_17*dmats1[17][46] + coeff0_18*dmats1[18][46] + coeff0_19*dmats1[19][46] + coeff0_20*dmats1[20][46] + coeff0_21*dmats1[21][46] + coeff0_22*dmats1[22][46] + coeff0_23*dmats1[23][46] + coeff0_24*dmats1[24][46] + coeff0_25*dmats1[25][46] + coeff0_26*dmats1[26][46] + coeff0_27*dmats1[27][46] + coeff0_28*dmats1[28][46] + coeff0_29*dmats1[29][46] + coeff0_30*dmats1[30][46] + coeff0_31*dmats1[31][46] + coeff0_32*dmats1[32][46] + coeff0_33*dmats1[33][46] + coeff0_34*dmats1[34][46] + coeff0_35*dmats1[35][46] + coeff0_36*dmats1[36][46] + coeff0_37*dmats1[37][46] + coeff0_38*dmats1[38][46] + coeff0_39*dmats1[39][46] + coeff0_40*dmats1[40][46] + coeff0_41*dmats1[41][46] + coeff0_42*dmats1[42][46] + coeff0_43*dmats1[43][46] + coeff0_44*dmats1[44][46] + coeff0_45*dmats1[45][46] + coeff0_46*dmats1[46][46] + coeff0_47*dmats1[47][46] + coeff0_48*dmats1[48][46] + coeff0_49*dmats1[49][46] + coeff0_50*dmats1[50][46] + coeff0_51*dmats1[51][46] + coeff0_52*dmats1[52][46] + coeff0_53*dmats1[53][46] + coeff0_54*dmats1[54][46] + coeff0_55*dmats1[55][46];
          new_coeff0_47 = coeff0_0*dmats1[0][47] + coeff0_1*dmats1[1][47] + coeff0_2*dmats1[2][47] + coeff0_3*dmats1[3][47] + coeff0_4*dmats1[4][47] + coeff0_5*dmats1[5][47] + coeff0_6*dmats1[6][47] + coeff0_7*dmats1[7][47] + coeff0_8*dmats1[8][47] + coeff0_9*dmats1[9][47] + coeff0_10*dmats1[10][47] + coeff0_11*dmats1[11][47] + coeff0_12*dmats1[12][47] + coeff0_13*dmats1[13][47] + coeff0_14*dmats1[14][47] + coeff0_15*dmats1[15][47] + coeff0_16*dmats1[16][47] + coeff0_17*dmats1[17][47] + coeff0_18*dmats1[18][47] + coeff0_19*dmats1[19][47] + coeff0_20*dmats1[20][47] + coeff0_21*dmats1[21][47] + coeff0_22*dmats1[22][47] + coeff0_23*dmats1[23][47] + coeff0_24*dmats1[24][47] + coeff0_25*dmats1[25][47] + coeff0_26*dmats1[26][47] + coeff0_27*dmats1[27][47] + coeff0_28*dmats1[28][47] + coeff0_29*dmats1[29][47] + coeff0_30*dmats1[30][47] + coeff0_31*dmats1[31][47] + coeff0_32*dmats1[32][47] + coeff0_33*dmats1[33][47] + coeff0_34*dmats1[34][47] + coeff0_35*dmats1[35][47] + coeff0_36*dmats1[36][47] + coeff0_37*dmats1[37][47] + coeff0_38*dmats1[38][47] + coeff0_39*dmats1[39][47] + coeff0_40*dmats1[40][47] + coeff0_41*dmats1[41][47] + coeff0_42*dmats1[42][47] + coeff0_43*dmats1[43][47] + coeff0_44*dmats1[44][47] + coeff0_45*dmats1[45][47] + coeff0_46*dmats1[46][47] + coeff0_47*dmats1[47][47] + coeff0_48*dmats1[48][47] + coeff0_49*dmats1[49][47] + coeff0_50*dmats1[50][47] + coeff0_51*dmats1[51][47] + coeff0_52*dmats1[52][47] + coeff0_53*dmats1[53][47] + coeff0_54*dmats1[54][47] + coeff0_55*dmats1[55][47];
          new_coeff0_48 = coeff0_0*dmats1[0][48] + coeff0_1*dmats1[1][48] + coeff0_2*dmats1[2][48] + coeff0_3*dmats1[3][48] + coeff0_4*dmats1[4][48] + coeff0_5*dmats1[5][48] + coeff0_6*dmats1[6][48] + coeff0_7*dmats1[7][48] + coeff0_8*dmats1[8][48] + coeff0_9*dmats1[9][48] + coeff0_10*dmats1[10][48] + coeff0_11*dmats1[11][48] + coeff0_12*dmats1[12][48] + coeff0_13*dmats1[13][48] + coeff0_14*dmats1[14][48] + coeff0_15*dmats1[15][48] + coeff0_16*dmats1[16][48] + coeff0_17*dmats1[17][48] + coeff0_18*dmats1[18][48] + coeff0_19*dmats1[19][48] + coeff0_20*dmats1[20][48] + coeff0_21*dmats1[21][48] + coeff0_22*dmats1[22][48] + coeff0_23*dmats1[23][48] + coeff0_24*dmats1[24][48] + coeff0_25*dmats1[25][48] + coeff0_26*dmats1[26][48] + coeff0_27*dmats1[27][48] + coeff0_28*dmats1[28][48] + coeff0_29*dmats1[29][48] + coeff0_30*dmats1[30][48] + coeff0_31*dmats1[31][48] + coeff0_32*dmats1[32][48] + coeff0_33*dmats1[33][48] + coeff0_34*dmats1[34][48] + coeff0_35*dmats1[35][48] + coeff0_36*dmats1[36][48] + coeff0_37*dmats1[37][48] + coeff0_38*dmats1[38][48] + coeff0_39*dmats1[39][48] + coeff0_40*dmats1[40][48] + coeff0_41*dmats1[41][48] + coeff0_42*dmats1[42][48] + coeff0_43*dmats1[43][48] + coeff0_44*dmats1[44][48] + coeff0_45*dmats1[45][48] + coeff0_46*dmats1[46][48] + coeff0_47*dmats1[47][48] + coeff0_48*dmats1[48][48] + coeff0_49*dmats1[49][48] + coeff0_50*dmats1[50][48] + coeff0_51*dmats1[51][48] + coeff0_52*dmats1[52][48] + coeff0_53*dmats1[53][48] + coeff0_54*dmats1[54][48] + coeff0_55*dmats1[55][48];
          new_coeff0_49 = coeff0_0*dmats1[0][49] + coeff0_1*dmats1[1][49] + coeff0_2*dmats1[2][49] + coeff0_3*dmats1[3][49] + coeff0_4*dmats1[4][49] + coeff0_5*dmats1[5][49] + coeff0_6*dmats1[6][49] + coeff0_7*dmats1[7][49] + coeff0_8*dmats1[8][49] + coeff0_9*dmats1[9][49] + coeff0_10*dmats1[10][49] + coeff0_11*dmats1[11][49] + coeff0_12*dmats1[12][49] + coeff0_13*dmats1[13][49] + coeff0_14*dmats1[14][49] + coeff0_15*dmats1[15][49] + coeff0_16*dmats1[16][49] + coeff0_17*dmats1[17][49] + coeff0_18*dmats1[18][49] + coeff0_19*dmats1[19][49] + coeff0_20*dmats1[20][49] + coeff0_21*dmats1[21][49] + coeff0_22*dmats1[22][49] + coeff0_23*dmats1[23][49] + coeff0_24*dmats1[24][49] + coeff0_25*dmats1[25][49] + coeff0_26*dmats1[26][49] + coeff0_27*dmats1[27][49] + coeff0_28*dmats1[28][49] + coeff0_29*dmats1[29][49] + coeff0_30*dmats1[30][49] + coeff0_31*dmats1[31][49] + coeff0_32*dmats1[32][49] + coeff0_33*dmats1[33][49] + coeff0_34*dmats1[34][49] + coeff0_35*dmats1[35][49] + coeff0_36*dmats1[36][49] + coeff0_37*dmats1[37][49] + coeff0_38*dmats1[38][49] + coeff0_39*dmats1[39][49] + coeff0_40*dmats1[40][49] + coeff0_41*dmats1[41][49] + coeff0_42*dmats1[42][49] + coeff0_43*dmats1[43][49] + coeff0_44*dmats1[44][49] + coeff0_45*dmats1[45][49] + coeff0_46*dmats1[46][49] + coeff0_47*dmats1[47][49] + coeff0_48*dmats1[48][49] + coeff0_49*dmats1[49][49] + coeff0_50*dmats1[50][49] + coeff0_51*dmats1[51][49] + coeff0_52*dmats1[52][49] + coeff0_53*dmats1[53][49] + coeff0_54*dmats1[54][49] + coeff0_55*dmats1[55][49];
          new_coeff0_50 = coeff0_0*dmats1[0][50] + coeff0_1*dmats1[1][50] + coeff0_2*dmats1[2][50] + coeff0_3*dmats1[3][50] + coeff0_4*dmats1[4][50] + coeff0_5*dmats1[5][50] + coeff0_6*dmats1[6][50] + coeff0_7*dmats1[7][50] + coeff0_8*dmats1[8][50] + coeff0_9*dmats1[9][50] + coeff0_10*dmats1[10][50] + coeff0_11*dmats1[11][50] + coeff0_12*dmats1[12][50] + coeff0_13*dmats1[13][50] + coeff0_14*dmats1[14][50] + coeff0_15*dmats1[15][50] + coeff0_16*dmats1[16][50] + coeff0_17*dmats1[17][50] + coeff0_18*dmats1[18][50] + coeff0_19*dmats1[19][50] + coeff0_20*dmats1[20][50] + coeff0_21*dmats1[21][50] + coeff0_22*dmats1[22][50] + coeff0_23*dmats1[23][50] + coeff0_24*dmats1[24][50] + coeff0_25*dmats1[25][50] + coeff0_26*dmats1[26][50] + coeff0_27*dmats1[27][50] + coeff0_28*dmats1[28][50] + coeff0_29*dmats1[29][50] + coeff0_30*dmats1[30][50] + coeff0_31*dmats1[31][50] + coeff0_32*dmats1[32][50] + coeff0_33*dmats1[33][50] + coeff0_34*dmats1[34][50] + coeff0_35*dmats1[35][50] + coeff0_36*dmats1[36][50] + coeff0_37*dmats1[37][50] + coeff0_38*dmats1[38][50] + coeff0_39*dmats1[39][50] + coeff0_40*dmats1[40][50] + coeff0_41*dmats1[41][50] + coeff0_42*dmats1[42][50] + coeff0_43*dmats1[43][50] + coeff0_44*dmats1[44][50] + coeff0_45*dmats1[45][50] + coeff0_46*dmats1[46][50] + coeff0_47*dmats1[47][50] + coeff0_48*dmats1[48][50] + coeff0_49*dmats1[49][50] + coeff0_50*dmats1[50][50] + coeff0_51*dmats1[51][50] + coeff0_52*dmats1[52][50] + coeff0_53*dmats1[53][50] + coeff0_54*dmats1[54][50] + coeff0_55*dmats1[55][50];
          new_coeff0_51 = coeff0_0*dmats1[0][51] + coeff0_1*dmats1[1][51] + coeff0_2*dmats1[2][51] + coeff0_3*dmats1[3][51] + coeff0_4*dmats1[4][51] + coeff0_5*dmats1[5][51] + coeff0_6*dmats1[6][51] + coeff0_7*dmats1[7][51] + coeff0_8*dmats1[8][51] + coeff0_9*dmats1[9][51] + coeff0_10*dmats1[10][51] + coeff0_11*dmats1[11][51] + coeff0_12*dmats1[12][51] + coeff0_13*dmats1[13][51] + coeff0_14*dmats1[14][51] + coeff0_15*dmats1[15][51] + coeff0_16*dmats1[16][51] + coeff0_17*dmats1[17][51] + coeff0_18*dmats1[18][51] + coeff0_19*dmats1[19][51] + coeff0_20*dmats1[20][51] + coeff0_21*dmats1[21][51] + coeff0_22*dmats1[22][51] + coeff0_23*dmats1[23][51] + coeff0_24*dmats1[24][51] + coeff0_25*dmats1[25][51] + coeff0_26*dmats1[26][51] + coeff0_27*dmats1[27][51] + coeff0_28*dmats1[28][51] + coeff0_29*dmats1[29][51] + coeff0_30*dmats1[30][51] + coeff0_31*dmats1[31][51] + coeff0_32*dmats1[32][51] + coeff0_33*dmats1[33][51] + coeff0_34*dmats1[34][51] + coeff0_35*dmats1[35][51] + coeff0_36*dmats1[36][51] + coeff0_37*dmats1[37][51] + coeff0_38*dmats1[38][51] + coeff0_39*dmats1[39][51] + coeff0_40*dmats1[40][51] + coeff0_41*dmats1[41][51] + coeff0_42*dmats1[42][51] + coeff0_43*dmats1[43][51] + coeff0_44*dmats1[44][51] + coeff0_45*dmats1[45][51] + coeff0_46*dmats1[46][51] + coeff0_47*dmats1[47][51] + coeff0_48*dmats1[48][51] + coeff0_49*dmats1[49][51] + coeff0_50*dmats1[50][51] + coeff0_51*dmats1[51][51] + coeff0_52*dmats1[52][51] + coeff0_53*dmats1[53][51] + coeff0_54*dmats1[54][51] + coeff0_55*dmats1[55][51];
          new_coeff0_52 = coeff0_0*dmats1[0][52] + coeff0_1*dmats1[1][52] + coeff0_2*dmats1[2][52] + coeff0_3*dmats1[3][52] + coeff0_4*dmats1[4][52] + coeff0_5*dmats1[5][52] + coeff0_6*dmats1[6][52] + coeff0_7*dmats1[7][52] + coeff0_8*dmats1[8][52] + coeff0_9*dmats1[9][52] + coeff0_10*dmats1[10][52] + coeff0_11*dmats1[11][52] + coeff0_12*dmats1[12][52] + coeff0_13*dmats1[13][52] + coeff0_14*dmats1[14][52] + coeff0_15*dmats1[15][52] + coeff0_16*dmats1[16][52] + coeff0_17*dmats1[17][52] + coeff0_18*dmats1[18][52] + coeff0_19*dmats1[19][52] + coeff0_20*dmats1[20][52] + coeff0_21*dmats1[21][52] + coeff0_22*dmats1[22][52] + coeff0_23*dmats1[23][52] + coeff0_24*dmats1[24][52] + coeff0_25*dmats1[25][52] + coeff0_26*dmats1[26][52] + coeff0_27*dmats1[27][52] + coeff0_28*dmats1[28][52] + coeff0_29*dmats1[29][52] + coeff0_30*dmats1[30][52] + coeff0_31*dmats1[31][52] + coeff0_32*dmats1[32][52] + coeff0_33*dmats1[33][52] + coeff0_34*dmats1[34][52] + coeff0_35*dmats1[35][52] + coeff0_36*dmats1[36][52] + coeff0_37*dmats1[37][52] + coeff0_38*dmats1[38][52] + coeff0_39*dmats1[39][52] + coeff0_40*dmats1[40][52] + coeff0_41*dmats1[41][52] + coeff0_42*dmats1[42][52] + coeff0_43*dmats1[43][52] + coeff0_44*dmats1[44][52] + coeff0_45*dmats1[45][52] + coeff0_46*dmats1[46][52] + coeff0_47*dmats1[47][52] + coeff0_48*dmats1[48][52] + coeff0_49*dmats1[49][52] + coeff0_50*dmats1[50][52] + coeff0_51*dmats1[51][52] + coeff0_52*dmats1[52][52] + coeff0_53*dmats1[53][52] + coeff0_54*dmats1[54][52] + coeff0_55*dmats1[55][52];
          new_coeff0_53 = coeff0_0*dmats1[0][53] + coeff0_1*dmats1[1][53] + coeff0_2*dmats1[2][53] + coeff0_3*dmats1[3][53] + coeff0_4*dmats1[4][53] + coeff0_5*dmats1[5][53] + coeff0_6*dmats1[6][53] + coeff0_7*dmats1[7][53] + coeff0_8*dmats1[8][53] + coeff0_9*dmats1[9][53] + coeff0_10*dmats1[10][53] + coeff0_11*dmats1[11][53] + coeff0_12*dmats1[12][53] + coeff0_13*dmats1[13][53] + coeff0_14*dmats1[14][53] + coeff0_15*dmats1[15][53] + coeff0_16*dmats1[16][53] + coeff0_17*dmats1[17][53] + coeff0_18*dmats1[18][53] + coeff0_19*dmats1[19][53] + coeff0_20*dmats1[20][53] + coeff0_21*dmats1[21][53] + coeff0_22*dmats1[22][53] + coeff0_23*dmats1[23][53] + coeff0_24*dmats1[24][53] + coeff0_25*dmats1[25][53] + coeff0_26*dmats1[26][53] + coeff0_27*dmats1[27][53] + coeff0_28*dmats1[28][53] + coeff0_29*dmats1[29][53] + coeff0_30*dmats1[30][53] + coeff0_31*dmats1[31][53] + coeff0_32*dmats1[32][53] + coeff0_33*dmats1[33][53] + coeff0_34*dmats1[34][53] + coeff0_35*dmats1[35][53] + coeff0_36*dmats1[36][53] + coeff0_37*dmats1[37][53] + coeff0_38*dmats1[38][53] + coeff0_39*dmats1[39][53] + coeff0_40*dmats1[40][53] + coeff0_41*dmats1[41][53] + coeff0_42*dmats1[42][53] + coeff0_43*dmats1[43][53] + coeff0_44*dmats1[44][53] + coeff0_45*dmats1[45][53] + coeff0_46*dmats1[46][53] + coeff0_47*dmats1[47][53] + coeff0_48*dmats1[48][53] + coeff0_49*dmats1[49][53] + coeff0_50*dmats1[50][53] + coeff0_51*dmats1[51][53] + coeff0_52*dmats1[52][53] + coeff0_53*dmats1[53][53] + coeff0_54*dmats1[54][53] + coeff0_55*dmats1[55][53];
          new_coeff0_54 = coeff0_0*dmats1[0][54] + coeff0_1*dmats1[1][54] + coeff0_2*dmats1[2][54] + coeff0_3*dmats1[3][54] + coeff0_4*dmats1[4][54] + coeff0_5*dmats1[5][54] + coeff0_6*dmats1[6][54] + coeff0_7*dmats1[7][54] + coeff0_8*dmats1[8][54] + coeff0_9*dmats1[9][54] + coeff0_10*dmats1[10][54] + coeff0_11*dmats1[11][54] + coeff0_12*dmats1[12][54] + coeff0_13*dmats1[13][54] + coeff0_14*dmats1[14][54] + coeff0_15*dmats1[15][54] + coeff0_16*dmats1[16][54] + coeff0_17*dmats1[17][54] + coeff0_18*dmats1[18][54] + coeff0_19*dmats1[19][54] + coeff0_20*dmats1[20][54] + coeff0_21*dmats1[21][54] + coeff0_22*dmats1[22][54] + coeff0_23*dmats1[23][54] + coeff0_24*dmats1[24][54] + coeff0_25*dmats1[25][54] + coeff0_26*dmats1[26][54] + coeff0_27*dmats1[27][54] + coeff0_28*dmats1[28][54] + coeff0_29*dmats1[29][54] + coeff0_30*dmats1[30][54] + coeff0_31*dmats1[31][54] + coeff0_32*dmats1[32][54] + coeff0_33*dmats1[33][54] + coeff0_34*dmats1[34][54] + coeff0_35*dmats1[35][54] + coeff0_36*dmats1[36][54] + coeff0_37*dmats1[37][54] + coeff0_38*dmats1[38][54] + coeff0_39*dmats1[39][54] + coeff0_40*dmats1[40][54] + coeff0_41*dmats1[41][54] + coeff0_42*dmats1[42][54] + coeff0_43*dmats1[43][54] + coeff0_44*dmats1[44][54] + coeff0_45*dmats1[45][54] + coeff0_46*dmats1[46][54] + coeff0_47*dmats1[47][54] + coeff0_48*dmats1[48][54] + coeff0_49*dmats1[49][54] + coeff0_50*dmats1[50][54] + coeff0_51*dmats1[51][54] + coeff0_52*dmats1[52][54] + coeff0_53*dmats1[53][54] + coeff0_54*dmats1[54][54] + coeff0_55*dmats1[55][54];
          new_coeff0_55 = coeff0_0*dmats1[0][55] + coeff0_1*dmats1[1][55] + coeff0_2*dmats1[2][55] + coeff0_3*dmats1[3][55] + coeff0_4*dmats1[4][55] + coeff0_5*dmats1[5][55] + coeff0_6*dmats1[6][55] + coeff0_7*dmats1[7][55] + coeff0_8*dmats1[8][55] + coeff0_9*dmats1[9][55] + coeff0_10*dmats1[10][55] + coeff0_11*dmats1[11][55] + coeff0_12*dmats1[12][55] + coeff0_13*dmats1[13][55] + coeff0_14*dmats1[14][55] + coeff0_15*dmats1[15][55] + coeff0_16*dmats1[16][55] + coeff0_17*dmats1[17][55] + coeff0_18*dmats1[18][55] + coeff0_19*dmats1[19][55] + coeff0_20*dmats1[20][55] + coeff0_21*dmats1[21][55] + coeff0_22*dmats1[22][55] + coeff0_23*dmats1[23][55] + coeff0_24*dmats1[24][55] + coeff0_25*dmats1[25][55] + coeff0_26*dmats1[26][55] + coeff0_27*dmats1[27][55] + coeff0_28*dmats1[28][55] + coeff0_29*dmats1[29][55] + coeff0_30*dmats1[30][55] + coeff0_31*dmats1[31][55] + coeff0_32*dmats1[32][55] + coeff0_33*dmats1[33][55] + coeff0_34*dmats1[34][55] + coeff0_35*dmats1[35][55] + coeff0_36*dmats1[36][55] + coeff0_37*dmats1[37][55] + coeff0_38*dmats1[38][55] + coeff0_39*dmats1[39][55] + coeff0_40*dmats1[40][55] + coeff0_41*dmats1[41][55] + coeff0_42*dmats1[42][55] + coeff0_43*dmats1[43][55] + coeff0_44*dmats1[44][55] + coeff0_45*dmats1[45][55] + coeff0_46*dmats1[46][55] + coeff0_47*dmats1[47][55] + coeff0_48*dmats1[48][55] + coeff0_49*dmats1[49][55] + coeff0_50*dmats1[50][55] + coeff0_51*dmats1[51][55] + coeff0_52*dmats1[52][55] + coeff0_53*dmats1[53][55] + coeff0_54*dmats1[54][55] + coeff0_55*dmats1[55][55];
        }
        if(combinations[deriv_num][j] == 2)
        {
          new_coeff0_0 = coeff0_0*dmats2[0][0] + coeff0_1*dmats2[1][0] + coeff0_2*dmats2[2][0] + coeff0_3*dmats2[3][0] + coeff0_4*dmats2[4][0] + coeff0_5*dmats2[5][0] + coeff0_6*dmats2[6][0] + coeff0_7*dmats2[7][0] + coeff0_8*dmats2[8][0] + coeff0_9*dmats2[9][0] + coeff0_10*dmats2[10][0] + coeff0_11*dmats2[11][0] + coeff0_12*dmats2[12][0] + coeff0_13*dmats2[13][0] + coeff0_14*dmats2[14][0] + coeff0_15*dmats2[15][0] + coeff0_16*dmats2[16][0] + coeff0_17*dmats2[17][0] + coeff0_18*dmats2[18][0] + coeff0_19*dmats2[19][0] + coeff0_20*dmats2[20][0] + coeff0_21*dmats2[21][0] + coeff0_22*dmats2[22][0] + coeff0_23*dmats2[23][0] + coeff0_24*dmats2[24][0] + coeff0_25*dmats2[25][0] + coeff0_26*dmats2[26][0] + coeff0_27*dmats2[27][0] + coeff0_28*dmats2[28][0] + coeff0_29*dmats2[29][0] + coeff0_30*dmats2[30][0] + coeff0_31*dmats2[31][0] + coeff0_32*dmats2[32][0] + coeff0_33*dmats2[33][0] + coeff0_34*dmats2[34][0] + coeff0_35*dmats2[35][0] + coeff0_36*dmats2[36][0] + coeff0_37*dmats2[37][0] + coeff0_38*dmats2[38][0] + coeff0_39*dmats2[39][0] + coeff0_40*dmats2[40][0] + coeff0_41*dmats2[41][0] + coeff0_42*dmats2[42][0] + coeff0_43*dmats2[43][0] + coeff0_44*dmats2[44][0] + coeff0_45*dmats2[45][0] + coeff0_46*dmats2[46][0] + coeff0_47*dmats2[47][0] + coeff0_48*dmats2[48][0] + coeff0_49*dmats2[49][0] + coeff0_50*dmats2[50][0] + coeff0_51*dmats2[51][0] + coeff0_52*dmats2[52][0] + coeff0_53*dmats2[53][0] + coeff0_54*dmats2[54][0] + coeff0_55*dmats2[55][0];
          new_coeff0_1 = coeff0_0*dmats2[0][1] + coeff0_1*dmats2[1][1] + coeff0_2*dmats2[2][1] + coeff0_3*dmats2[3][1] + coeff0_4*dmats2[4][1] + coeff0_5*dmats2[5][1] + coeff0_6*dmats2[6][1] + coeff0_7*dmats2[7][1] + coeff0_8*dmats2[8][1] + coeff0_9*dmats2[9][1] + coeff0_10*dmats2[10][1] + coeff0_11*dmats2[11][1] + coeff0_12*dmats2[12][1] + coeff0_13*dmats2[13][1] + coeff0_14*dmats2[14][1] + coeff0_15*dmats2[15][1] + coeff0_16*dmats2[16][1] + coeff0_17*dmats2[17][1] + coeff0_18*dmats2[18][1] + coeff0_19*dmats2[19][1] + coeff0_20*dmats2[20][1] + coeff0_21*dmats2[21][1] + coeff0_22*dmats2[22][1] + coeff0_23*dmats2[23][1] + coeff0_24*dmats2[24][1] + coeff0_25*dmats2[25][1] + coeff0_26*dmats2[26][1] + coeff0_27*dmats2[27][1] + coeff0_28*dmats2[28][1] + coeff0_29*dmats2[29][1] + coeff0_30*dmats2[30][1] + coeff0_31*dmats2[31][1] + coeff0_32*dmats2[32][1] + coeff0_33*dmats2[33][1] + coeff0_34*dmats2[34][1] + coeff0_35*dmats2[35][1] + coeff0_36*dmats2[36][1] + coeff0_37*dmats2[37][1] + coeff0_38*dmats2[38][1] + coeff0_39*dmats2[39][1] + coeff0_40*dmats2[40][1] + coeff0_41*dmats2[41][1] + coeff0_42*dmats2[42][1] + coeff0_43*dmats2[43][1] + coeff0_44*dmats2[44][1] + coeff0_45*dmats2[45][1] + coeff0_46*dmats2[46][1] + coeff0_47*dmats2[47][1] + coeff0_48*dmats2[48][1] + coeff0_49*dmats2[49][1] + coeff0_50*dmats2[50][1] + coeff0_51*dmats2[51][1] + coeff0_52*dmats2[52][1] + coeff0_53*dmats2[53][1] + coeff0_54*dmats2[54][1] + coeff0_55*dmats2[55][1];
          new_coeff0_2 = coeff0_0*dmats2[0][2] + coeff0_1*dmats2[1][2] + coeff0_2*dmats2[2][2] + coeff0_3*dmats2[3][2] + coeff0_4*dmats2[4][2] + coeff0_5*dmats2[5][2] + coeff0_6*dmats2[6][2] + coeff0_7*dmats2[7][2] + coeff0_8*dmats2[8][2] + coeff0_9*dmats2[9][2] + coeff0_10*dmats2[10][2] + coeff0_11*dmats2[11][2] + coeff0_12*dmats2[12][2] + coeff0_13*dmats2[13][2] + coeff0_14*dmats2[14][2] + coeff0_15*dmats2[15][2] + coeff0_16*dmats2[16][2] + coeff0_17*dmats2[17][2] + coeff0_18*dmats2[18][2] + coeff0_19*dmats2[19][2] + coeff0_20*dmats2[20][2] + coeff0_21*dmats2[21][2] + coeff0_22*dmats2[22][2] + coeff0_23*dmats2[23][2] + coeff0_24*dmats2[24][2] + coeff0_25*dmats2[25][2] + coeff0_26*dmats2[26][2] + coeff0_27*dmats2[27][2] + coeff0_28*dmats2[28][2] + coeff0_29*dmats2[29][2] + coeff0_30*dmats2[30][2] + coeff0_31*dmats2[31][2] + coeff0_32*dmats2[32][2] + coeff0_33*dmats2[33][2] + coeff0_34*dmats2[34][2] + coeff0_35*dmats2[35][2] + coeff0_36*dmats2[36][2] + coeff0_37*dmats2[37][2] + coeff0_38*dmats2[38][2] + coeff0_39*dmats2[39][2] + coeff0_40*dmats2[40][2] + coeff0_41*dmats2[41][2] + coeff0_42*dmats2[42][2] + coeff0_43*dmats2[43][2] + coeff0_44*dmats2[44][2] + coeff0_45*dmats2[45][2] + coeff0_46*dmats2[46][2] + coeff0_47*dmats2[47][2] + coeff0_48*dmats2[48][2] + coeff0_49*dmats2[49][2] + coeff0_50*dmats2[50][2] + coeff0_51*dmats2[51][2] + coeff0_52*dmats2[52][2] + coeff0_53*dmats2[53][2] + coeff0_54*dmats2[54][2] + coeff0_55*dmats2[55][2];
          new_coeff0_3 = coeff0_0*dmats2[0][3] + coeff0_1*dmats2[1][3] + coeff0_2*dmats2[2][3] + coeff0_3*dmats2[3][3] + coeff0_4*dmats2[4][3] + coeff0_5*dmats2[5][3] + coeff0_6*dmats2[6][3] + coeff0_7*dmats2[7][3] + coeff0_8*dmats2[8][3] + coeff0_9*dmats2[9][3] + coeff0_10*dmats2[10][3] + coeff0_11*dmats2[11][3] + coeff0_12*dmats2[12][3] + coeff0_13*dmats2[13][3] + coeff0_14*dmats2[14][3] + coeff0_15*dmats2[15][3] + coeff0_16*dmats2[16][3] + coeff0_17*dmats2[17][3] + coeff0_18*dmats2[18][3] + coeff0_19*dmats2[19][3] + coeff0_20*dmats2[20][3] + coeff0_21*dmats2[21][3] + coeff0_22*dmats2[22][3] + coeff0_23*dmats2[23][3] + coeff0_24*dmats2[24][3] + coeff0_25*dmats2[25][3] + coeff0_26*dmats2[26][3] + coeff0_27*dmats2[27][3] + coeff0_28*dmats2[28][3] + coeff0_29*dmats2[29][3] + coeff0_30*dmats2[30][3] + coeff0_31*dmats2[31][3] + coeff0_32*dmats2[32][3] + coeff0_33*dmats2[33][3] + coeff0_34*dmats2[34][3] + coeff0_35*dmats2[35][3] + coeff0_36*dmats2[36][3] + coeff0_37*dmats2[37][3] + coeff0_38*dmats2[38][3] + coeff0_39*dmats2[39][3] + coeff0_40*dmats2[40][3] + coeff0_41*dmats2[41][3] + coeff0_42*dmats2[42][3] + coeff0_43*dmats2[43][3] + coeff0_44*dmats2[44][3] + coeff0_45*dmats2[45][3] + coeff0_46*dmats2[46][3] + coeff0_47*dmats2[47][3] + coeff0_48*dmats2[48][3] + coeff0_49*dmats2[49][3] + coeff0_50*dmats2[50][3] + coeff0_51*dmats2[51][3] + coeff0_52*dmats2[52][3] + coeff0_53*dmats2[53][3] + coeff0_54*dmats2[54][3] + coeff0_55*dmats2[55][3];
          new_coeff0_4 = coeff0_0*dmats2[0][4] + coeff0_1*dmats2[1][4] + coeff0_2*dmats2[2][4] + coeff0_3*dmats2[3][4] + coeff0_4*dmats2[4][4] + coeff0_5*dmats2[5][4] + coeff0_6*dmats2[6][4] + coeff0_7*dmats2[7][4] + coeff0_8*dmats2[8][4] + coeff0_9*dmats2[9][4] + coeff0_10*dmats2[10][4] + coeff0_11*dmats2[11][4] + coeff0_12*dmats2[12][4] + coeff0_13*dmats2[13][4] + coeff0_14*dmats2[14][4] + coeff0_15*dmats2[15][4] + coeff0_16*dmats2[16][4] + coeff0_17*dmats2[17][4] + coeff0_18*dmats2[18][4] + coeff0_19*dmats2[19][4] + coeff0_20*dmats2[20][4] + coeff0_21*dmats2[21][4] + coeff0_22*dmats2[22][4] + coeff0_23*dmats2[23][4] + coeff0_24*dmats2[24][4] + coeff0_25*dmats2[25][4] + coeff0_26*dmats2[26][4] + coeff0_27*dmats2[27][4] + coeff0_28*dmats2[28][4] + coeff0_29*dmats2[29][4] + coeff0_30*dmats2[30][4] + coeff0_31*dmats2[31][4] + coeff0_32*dmats2[32][4] + coeff0_33*dmats2[33][4] + coeff0_34*dmats2[34][4] + coeff0_35*dmats2[35][4] + coeff0_36*dmats2[36][4] + coeff0_37*dmats2[37][4] + coeff0_38*dmats2[38][4] + coeff0_39*dmats2[39][4] + coeff0_40*dmats2[40][4] + coeff0_41*dmats2[41][4] + coeff0_42*dmats2[42][4] + coeff0_43*dmats2[43][4] + coeff0_44*dmats2[44][4] + coeff0_45*dmats2[45][4] + coeff0_46*dmats2[46][4] + coeff0_47*dmats2[47][4] + coeff0_48*dmats2[48][4] + coeff0_49*dmats2[49][4] + coeff0_50*dmats2[50][4] + coeff0_51*dmats2[51][4] + coeff0_52*dmats2[52][4] + coeff0_53*dmats2[53][4] + coeff0_54*dmats2[54][4] + coeff0_55*dmats2[55][4];
          new_coeff0_5 = coeff0_0*dmats2[0][5] + coeff0_1*dmats2[1][5] + coeff0_2*dmats2[2][5] + coeff0_3*dmats2[3][5] + coeff0_4*dmats2[4][5] + coeff0_5*dmats2[5][5] + coeff0_6*dmats2[6][5] + coeff0_7*dmats2[7][5] + coeff0_8*dmats2[8][5] + coeff0_9*dmats2[9][5] + coeff0_10*dmats2[10][5] + coeff0_11*dmats2[11][5] + coeff0_12*dmats2[12][5] + coeff0_13*dmats2[13][5] + coeff0_14*dmats2[14][5] + coeff0_15*dmats2[15][5] + coeff0_16*dmats2[16][5] + coeff0_17*dmats2[17][5] + coeff0_18*dmats2[18][5] + coeff0_19*dmats2[19][5] + coeff0_20*dmats2[20][5] + coeff0_21*dmats2[21][5] + coeff0_22*dmats2[22][5] + coeff0_23*dmats2[23][5] + coeff0_24*dmats2[24][5] + coeff0_25*dmats2[25][5] + coeff0_26*dmats2[26][5] + coeff0_27*dmats2[27][5] + coeff0_28*dmats2[28][5] + coeff0_29*dmats2[29][5] + coeff0_30*dmats2[30][5] + coeff0_31*dmats2[31][5] + coeff0_32*dmats2[32][5] + coeff0_33*dmats2[33][5] + coeff0_34*dmats2[34][5] + coeff0_35*dmats2[35][5] + coeff0_36*dmats2[36][5] + coeff0_37*dmats2[37][5] + coeff0_38*dmats2[38][5] + coeff0_39*dmats2[39][5] + coeff0_40*dmats2[40][5] + coeff0_41*dmats2[41][5] + coeff0_42*dmats2[42][5] + coeff0_43*dmats2[43][5] + coeff0_44*dmats2[44][5] + coeff0_45*dmats2[45][5] + coeff0_46*dmats2[46][5] + coeff0_47*dmats2[47][5] + coeff0_48*dmats2[48][5] + coeff0_49*dmats2[49][5] + coeff0_50*dmats2[50][5] + coeff0_51*dmats2[51][5] + coeff0_52*dmats2[52][5] + coeff0_53*dmats2[53][5] + coeff0_54*dmats2[54][5] + coeff0_55*dmats2[55][5];
          new_coeff0_6 = coeff0_0*dmats2[0][6] + coeff0_1*dmats2[1][6] + coeff0_2*dmats2[2][6] + coeff0_3*dmats2[3][6] + coeff0_4*dmats2[4][6] + coeff0_5*dmats2[5][6] + coeff0_6*dmats2[6][6] + coeff0_7*dmats2[7][6] + coeff0_8*dmats2[8][6] + coeff0_9*dmats2[9][6] + coeff0_10*dmats2[10][6] + coeff0_11*dmats2[11][6] + coeff0_12*dmats2[12][6] + coeff0_13*dmats2[13][6] + coeff0_14*dmats2[14][6] + coeff0_15*dmats2[15][6] + coeff0_16*dmats2[16][6] + coeff0_17*dmats2[17][6] + coeff0_18*dmats2[18][6] + coeff0_19*dmats2[19][6] + coeff0_20*dmats2[20][6] + coeff0_21*dmats2[21][6] + coeff0_22*dmats2[22][6] + coeff0_23*dmats2[23][6] + coeff0_24*dmats2[24][6] + coeff0_25*dmats2[25][6] + coeff0_26*dmats2[26][6] + coeff0_27*dmats2[27][6] + coeff0_28*dmats2[28][6] + coeff0_29*dmats2[29][6] + coeff0_30*dmats2[30][6] + coeff0_31*dmats2[31][6] + coeff0_32*dmats2[32][6] + coeff0_33*dmats2[33][6] + coeff0_34*dmats2[34][6] + coeff0_35*dmats2[35][6] + coeff0_36*dmats2[36][6] + coeff0_37*dmats2[37][6] + coeff0_38*dmats2[38][6] + coeff0_39*dmats2[39][6] + coeff0_40*dmats2[40][6] + coeff0_41*dmats2[41][6] + coeff0_42*dmats2[42][6] + coeff0_43*dmats2[43][6] + coeff0_44*dmats2[44][6] + coeff0_45*dmats2[45][6] + coeff0_46*dmats2[46][6] + coeff0_47*dmats2[47][6] + coeff0_48*dmats2[48][6] + coeff0_49*dmats2[49][6] + coeff0_50*dmats2[50][6] + coeff0_51*dmats2[51][6] + coeff0_52*dmats2[52][6] + coeff0_53*dmats2[53][6] + coeff0_54*dmats2[54][6] + coeff0_55*dmats2[55][6];
          new_coeff0_7 = coeff0_0*dmats2[0][7] + coeff0_1*dmats2[1][7] + coeff0_2*dmats2[2][7] + coeff0_3*dmats2[3][7] + coeff0_4*dmats2[4][7] + coeff0_5*dmats2[5][7] + coeff0_6*dmats2[6][7] + coeff0_7*dmats2[7][7] + coeff0_8*dmats2[8][7] + coeff0_9*dmats2[9][7] + coeff0_10*dmats2[10][7] + coeff0_11*dmats2[11][7] + coeff0_12*dmats2[12][7] + coeff0_13*dmats2[13][7] + coeff0_14*dmats2[14][7] + coeff0_15*dmats2[15][7] + coeff0_16*dmats2[16][7] + coeff0_17*dmats2[17][7] + coeff0_18*dmats2[18][7] + coeff0_19*dmats2[19][7] + coeff0_20*dmats2[20][7] + coeff0_21*dmats2[21][7] + coeff0_22*dmats2[22][7] + coeff0_23*dmats2[23][7] + coeff0_24*dmats2[24][7] + coeff0_25*dmats2[25][7] + coeff0_26*dmats2[26][7] + coeff0_27*dmats2[27][7] + coeff0_28*dmats2[28][7] + coeff0_29*dmats2[29][7] + coeff0_30*dmats2[30][7] + coeff0_31*dmats2[31][7] + coeff0_32*dmats2[32][7] + coeff0_33*dmats2[33][7] + coeff0_34*dmats2[34][7] + coeff0_35*dmats2[35][7] + coeff0_36*dmats2[36][7] + coeff0_37*dmats2[37][7] + coeff0_38*dmats2[38][7] + coeff0_39*dmats2[39][7] + coeff0_40*dmats2[40][7] + coeff0_41*dmats2[41][7] + coeff0_42*dmats2[42][7] + coeff0_43*dmats2[43][7] + coeff0_44*dmats2[44][7] + coeff0_45*dmats2[45][7] + coeff0_46*dmats2[46][7] + coeff0_47*dmats2[47][7] + coeff0_48*dmats2[48][7] + coeff0_49*dmats2[49][7] + coeff0_50*dmats2[50][7] + coeff0_51*dmats2[51][7] + coeff0_52*dmats2[52][7] + coeff0_53*dmats2[53][7] + coeff0_54*dmats2[54][7] + coeff0_55*dmats2[55][7];
          new_coeff0_8 = coeff0_0*dmats2[0][8] + coeff0_1*dmats2[1][8] + coeff0_2*dmats2[2][8] + coeff0_3*dmats2[3][8] + coeff0_4*dmats2[4][8] + coeff0_5*dmats2[5][8] + coeff0_6*dmats2[6][8] + coeff0_7*dmats2[7][8] + coeff0_8*dmats2[8][8] + coeff0_9*dmats2[9][8] + coeff0_10*dmats2[10][8] + coeff0_11*dmats2[11][8] + coeff0_12*dmats2[12][8] + coeff0_13*dmats2[13][8] + coeff0_14*dmats2[14][8] + coeff0_15*dmats2[15][8] + coeff0_16*dmats2[16][8] + coeff0_17*dmats2[17][8] + coeff0_18*dmats2[18][8] + coeff0_19*dmats2[19][8] + coeff0_20*dmats2[20][8] + coeff0_21*dmats2[21][8] + coeff0_22*dmats2[22][8] + coeff0_23*dmats2[23][8] + coeff0_24*dmats2[24][8] + coeff0_25*dmats2[25][8] + coeff0_26*dmats2[26][8] + coeff0_27*dmats2[27][8] + coeff0_28*dmats2[28][8] + coeff0_29*dmats2[29][8] + coeff0_30*dmats2[30][8] + coeff0_31*dmats2[31][8] + coeff0_32*dmats2[32][8] + coeff0_33*dmats2[33][8] + coeff0_34*dmats2[34][8] + coeff0_35*dmats2[35][8] + coeff0_36*dmats2[36][8] + coeff0_37*dmats2[37][8] + coeff0_38*dmats2[38][8] + coeff0_39*dmats2[39][8] + coeff0_40*dmats2[40][8] + coeff0_41*dmats2[41][8] + coeff0_42*dmats2[42][8] + coeff0_43*dmats2[43][8] + coeff0_44*dmats2[44][8] + coeff0_45*dmats2[45][8] + coeff0_46*dmats2[46][8] + coeff0_47*dmats2[47][8] + coeff0_48*dmats2[48][8] + coeff0_49*dmats2[49][8] + coeff0_50*dmats2[50][8] + coeff0_51*dmats2[51][8] + coeff0_52*dmats2[52][8] + coeff0_53*dmats2[53][8] + coeff0_54*dmats2[54][8] + coeff0_55*dmats2[55][8];
          new_coeff0_9 = coeff0_0*dmats2[0][9] + coeff0_1*dmats2[1][9] + coeff0_2*dmats2[2][9] + coeff0_3*dmats2[3][9] + coeff0_4*dmats2[4][9] + coeff0_5*dmats2[5][9] + coeff0_6*dmats2[6][9] + coeff0_7*dmats2[7][9] + coeff0_8*dmats2[8][9] + coeff0_9*dmats2[9][9] + coeff0_10*dmats2[10][9] + coeff0_11*dmats2[11][9] + coeff0_12*dmats2[12][9] + coeff0_13*dmats2[13][9] + coeff0_14*dmats2[14][9] + coeff0_15*dmats2[15][9] + coeff0_16*dmats2[16][9] + coeff0_17*dmats2[17][9] + coeff0_18*dmats2[18][9] + coeff0_19*dmats2[19][9] + coeff0_20*dmats2[20][9] + coeff0_21*dmats2[21][9] + coeff0_22*dmats2[22][9] + coeff0_23*dmats2[23][9] + coeff0_24*dmats2[24][9] + coeff0_25*dmats2[25][9] + coeff0_26*dmats2[26][9] + coeff0_27*dmats2[27][9] + coeff0_28*dmats2[28][9] + coeff0_29*dmats2[29][9] + coeff0_30*dmats2[30][9] + coeff0_31*dmats2[31][9] + coeff0_32*dmats2[32][9] + coeff0_33*dmats2[33][9] + coeff0_34*dmats2[34][9] + coeff0_35*dmats2[35][9] + coeff0_36*dmats2[36][9] + coeff0_37*dmats2[37][9] + coeff0_38*dmats2[38][9] + coeff0_39*dmats2[39][9] + coeff0_40*dmats2[40][9] + coeff0_41*dmats2[41][9] + coeff0_42*dmats2[42][9] + coeff0_43*dmats2[43][9] + coeff0_44*dmats2[44][9] + coeff0_45*dmats2[45][9] + coeff0_46*dmats2[46][9] + coeff0_47*dmats2[47][9] + coeff0_48*dmats2[48][9] + coeff0_49*dmats2[49][9] + coeff0_50*dmats2[50][9] + coeff0_51*dmats2[51][9] + coeff0_52*dmats2[52][9] + coeff0_53*dmats2[53][9] + coeff0_54*dmats2[54][9] + coeff0_55*dmats2[55][9];
          new_coeff0_10 = coeff0_0*dmats2[0][10] + coeff0_1*dmats2[1][10] + coeff0_2*dmats2[2][10] + coeff0_3*dmats2[3][10] + coeff0_4*dmats2[4][10] + coeff0_5*dmats2[5][10] + coeff0_6*dmats2[6][10] + coeff0_7*dmats2[7][10] + coeff0_8*dmats2[8][10] + coeff0_9*dmats2[9][10] + coeff0_10*dmats2[10][10] + coeff0_11*dmats2[11][10] + coeff0_12*dmats2[12][10] + coeff0_13*dmats2[13][10] + coeff0_14*dmats2[14][10] + coeff0_15*dmats2[15][10] + coeff0_16*dmats2[16][10] + coeff0_17*dmats2[17][10] + coeff0_18*dmats2[18][10] + coeff0_19*dmats2[19][10] + coeff0_20*dmats2[20][10] + coeff0_21*dmats2[21][10] + coeff0_22*dmats2[22][10] + coeff0_23*dmats2[23][10] + coeff0_24*dmats2[24][10] + coeff0_25*dmats2[25][10] + coeff0_26*dmats2[26][10] + coeff0_27*dmats2[27][10] + coeff0_28*dmats2[28][10] + coeff0_29*dmats2[29][10] + coeff0_30*dmats2[30][10] + coeff0_31*dmats2[31][10] + coeff0_32*dmats2[32][10] + coeff0_33*dmats2[33][10] + coeff0_34*dmats2[34][10] + coeff0_35*dmats2[35][10] + coeff0_36*dmats2[36][10] + coeff0_37*dmats2[37][10] + coeff0_38*dmats2[38][10] + coeff0_39*dmats2[39][10] + coeff0_40*dmats2[40][10] + coeff0_41*dmats2[41][10] + coeff0_42*dmats2[42][10] + coeff0_43*dmats2[43][10] + coeff0_44*dmats2[44][10] + coeff0_45*dmats2[45][10] + coeff0_46*dmats2[46][10] + coeff0_47*dmats2[47][10] + coeff0_48*dmats2[48][10] + coeff0_49*dmats2[49][10] + coeff0_50*dmats2[50][10] + coeff0_51*dmats2[51][10] + coeff0_52*dmats2[52][10] + coeff0_53*dmats2[53][10] + coeff0_54*dmats2[54][10] + coeff0_55*dmats2[55][10];
          new_coeff0_11 = coeff0_0*dmats2[0][11] + coeff0_1*dmats2[1][11] + coeff0_2*dmats2[2][11] + coeff0_3*dmats2[3][11] + coeff0_4*dmats2[4][11] + coeff0_5*dmats2[5][11] + coeff0_6*dmats2[6][11] + coeff0_7*dmats2[7][11] + coeff0_8*dmats2[8][11] + coeff0_9*dmats2[9][11] + coeff0_10*dmats2[10][11] + coeff0_11*dmats2[11][11] + coeff0_12*dmats2[12][11] + coeff0_13*dmats2[13][11] + coeff0_14*dmats2[14][11] + coeff0_15*dmats2[15][11] + coeff0_16*dmats2[16][11] + coeff0_17*dmats2[17][11] + coeff0_18*dmats2[18][11] + coeff0_19*dmats2[19][11] + coeff0_20*dmats2[20][11] + coeff0_21*dmats2[21][11] + coeff0_22*dmats2[22][11] + coeff0_23*dmats2[23][11] + coeff0_24*dmats2[24][11] + coeff0_25*dmats2[25][11] + coeff0_26*dmats2[26][11] + coeff0_27*dmats2[27][11] + coeff0_28*dmats2[28][11] + coeff0_29*dmats2[29][11] + coeff0_30*dmats2[30][11] + coeff0_31*dmats2[31][11] + coeff0_32*dmats2[32][11] + coeff0_33*dmats2[33][11] + coeff0_34*dmats2[34][11] + coeff0_35*dmats2[35][11] + coeff0_36*dmats2[36][11] + coeff0_37*dmats2[37][11] + coeff0_38*dmats2[38][11] + coeff0_39*dmats2[39][11] + coeff0_40*dmats2[40][11] + coeff0_41*dmats2[41][11] + coeff0_42*dmats2[42][11] + coeff0_43*dmats2[43][11] + coeff0_44*dmats2[44][11] + coeff0_45*dmats2[45][11] + coeff0_46*dmats2[46][11] + coeff0_47*dmats2[47][11] + coeff0_48*dmats2[48][11] + coeff0_49*dmats2[49][11] + coeff0_50*dmats2[50][11] + coeff0_51*dmats2[51][11] + coeff0_52*dmats2[52][11] + coeff0_53*dmats2[53][11] + coeff0_54*dmats2[54][11] + coeff0_55*dmats2[55][11];
          new_coeff0_12 = coeff0_0*dmats2[0][12] + coeff0_1*dmats2[1][12] + coeff0_2*dmats2[2][12] + coeff0_3*dmats2[3][12] + coeff0_4*dmats2[4][12] + coeff0_5*dmats2[5][12] + coeff0_6*dmats2[6][12] + coeff0_7*dmats2[7][12] + coeff0_8*dmats2[8][12] + coeff0_9*dmats2[9][12] + coeff0_10*dmats2[10][12] + coeff0_11*dmats2[11][12] + coeff0_12*dmats2[12][12] + coeff0_13*dmats2[13][12] + coeff0_14*dmats2[14][12] + coeff0_15*dmats2[15][12] + coeff0_16*dmats2[16][12] + coeff0_17*dmats2[17][12] + coeff0_18*dmats2[18][12] + coeff0_19*dmats2[19][12] + coeff0_20*dmats2[20][12] + coeff0_21*dmats2[21][12] + coeff0_22*dmats2[22][12] + coeff0_23*dmats2[23][12] + coeff0_24*dmats2[24][12] + coeff0_25*dmats2[25][12] + coeff0_26*dmats2[26][12] + coeff0_27*dmats2[27][12] + coeff0_28*dmats2[28][12] + coeff0_29*dmats2[29][12] + coeff0_30*dmats2[30][12] + coeff0_31*dmats2[31][12] + coeff0_32*dmats2[32][12] + coeff0_33*dmats2[33][12] + coeff0_34*dmats2[34][12] + coeff0_35*dmats2[35][12] + coeff0_36*dmats2[36][12] + coeff0_37*dmats2[37][12] + coeff0_38*dmats2[38][12] + coeff0_39*dmats2[39][12] + coeff0_40*dmats2[40][12] + coeff0_41*dmats2[41][12] + coeff0_42*dmats2[42][12] + coeff0_43*dmats2[43][12] + coeff0_44*dmats2[44][12] + coeff0_45*dmats2[45][12] + coeff0_46*dmats2[46][12] + coeff0_47*dmats2[47][12] + coeff0_48*dmats2[48][12] + coeff0_49*dmats2[49][12] + coeff0_50*dmats2[50][12] + coeff0_51*dmats2[51][12] + coeff0_52*dmats2[52][12] + coeff0_53*dmats2[53][12] + coeff0_54*dmats2[54][12] + coeff0_55*dmats2[55][12];
          new_coeff0_13 = coeff0_0*dmats2[0][13] + coeff0_1*dmats2[1][13] + coeff0_2*dmats2[2][13] + coeff0_3*dmats2[3][13] + coeff0_4*dmats2[4][13] + coeff0_5*dmats2[5][13] + coeff0_6*dmats2[6][13] + coeff0_7*dmats2[7][13] + coeff0_8*dmats2[8][13] + coeff0_9*dmats2[9][13] + coeff0_10*dmats2[10][13] + coeff0_11*dmats2[11][13] + coeff0_12*dmats2[12][13] + coeff0_13*dmats2[13][13] + coeff0_14*dmats2[14][13] + coeff0_15*dmats2[15][13] + coeff0_16*dmats2[16][13] + coeff0_17*dmats2[17][13] + coeff0_18*dmats2[18][13] + coeff0_19*dmats2[19][13] + coeff0_20*dmats2[20][13] + coeff0_21*dmats2[21][13] + coeff0_22*dmats2[22][13] + coeff0_23*dmats2[23][13] + coeff0_24*dmats2[24][13] + coeff0_25*dmats2[25][13] + coeff0_26*dmats2[26][13] + coeff0_27*dmats2[27][13] + coeff0_28*dmats2[28][13] + coeff0_29*dmats2[29][13] + coeff0_30*dmats2[30][13] + coeff0_31*dmats2[31][13] + coeff0_32*dmats2[32][13] + coeff0_33*dmats2[33][13] + coeff0_34*dmats2[34][13] + coeff0_35*dmats2[35][13] + coeff0_36*dmats2[36][13] + coeff0_37*dmats2[37][13] + coeff0_38*dmats2[38][13] + coeff0_39*dmats2[39][13] + coeff0_40*dmats2[40][13] + coeff0_41*dmats2[41][13] + coeff0_42*dmats2[42][13] + coeff0_43*dmats2[43][13] + coeff0_44*dmats2[44][13] + coeff0_45*dmats2[45][13] + coeff0_46*dmats2[46][13] + coeff0_47*dmats2[47][13] + coeff0_48*dmats2[48][13] + coeff0_49*dmats2[49][13] + coeff0_50*dmats2[50][13] + coeff0_51*dmats2[51][13] + coeff0_52*dmats2[52][13] + coeff0_53*dmats2[53][13] + coeff0_54*dmats2[54][13] + coeff0_55*dmats2[55][13];
          new_coeff0_14 = coeff0_0*dmats2[0][14] + coeff0_1*dmats2[1][14] + coeff0_2*dmats2[2][14] + coeff0_3*dmats2[3][14] + coeff0_4*dmats2[4][14] + coeff0_5*dmats2[5][14] + coeff0_6*dmats2[6][14] + coeff0_7*dmats2[7][14] + coeff0_8*dmats2[8][14] + coeff0_9*dmats2[9][14] + coeff0_10*dmats2[10][14] + coeff0_11*dmats2[11][14] + coeff0_12*dmats2[12][14] + coeff0_13*dmats2[13][14] + coeff0_14*dmats2[14][14] + coeff0_15*dmats2[15][14] + coeff0_16*dmats2[16][14] + coeff0_17*dmats2[17][14] + coeff0_18*dmats2[18][14] + coeff0_19*dmats2[19][14] + coeff0_20*dmats2[20][14] + coeff0_21*dmats2[21][14] + coeff0_22*dmats2[22][14] + coeff0_23*dmats2[23][14] + coeff0_24*dmats2[24][14] + coeff0_25*dmats2[25][14] + coeff0_26*dmats2[26][14] + coeff0_27*dmats2[27][14] + coeff0_28*dmats2[28][14] + coeff0_29*dmats2[29][14] + coeff0_30*dmats2[30][14] + coeff0_31*dmats2[31][14] + coeff0_32*dmats2[32][14] + coeff0_33*dmats2[33][14] + coeff0_34*dmats2[34][14] + coeff0_35*dmats2[35][14] + coeff0_36*dmats2[36][14] + coeff0_37*dmats2[37][14] + coeff0_38*dmats2[38][14] + coeff0_39*dmats2[39][14] + coeff0_40*dmats2[40][14] + coeff0_41*dmats2[41][14] + coeff0_42*dmats2[42][14] + coeff0_43*dmats2[43][14] + coeff0_44*dmats2[44][14] + coeff0_45*dmats2[45][14] + coeff0_46*dmats2[46][14] + coeff0_47*dmats2[47][14] + coeff0_48*dmats2[48][14] + coeff0_49*dmats2[49][14] + coeff0_50*dmats2[50][14] + coeff0_51*dmats2[51][14] + coeff0_52*dmats2[52][14] + coeff0_53*dmats2[53][14] + coeff0_54*dmats2[54][14] + coeff0_55*dmats2[55][14];
          new_coeff0_15 = coeff0_0*dmats2[0][15] + coeff0_1*dmats2[1][15] + coeff0_2*dmats2[2][15] + coeff0_3*dmats2[3][15] + coeff0_4*dmats2[4][15] + coeff0_5*dmats2[5][15] + coeff0_6*dmats2[6][15] + coeff0_7*dmats2[7][15] + coeff0_8*dmats2[8][15] + coeff0_9*dmats2[9][15] + coeff0_10*dmats2[10][15] + coeff0_11*dmats2[11][15] + coeff0_12*dmats2[12][15] + coeff0_13*dmats2[13][15] + coeff0_14*dmats2[14][15] + coeff0_15*dmats2[15][15] + coeff0_16*dmats2[16][15] + coeff0_17*dmats2[17][15] + coeff0_18*dmats2[18][15] + coeff0_19*dmats2[19][15] + coeff0_20*dmats2[20][15] + coeff0_21*dmats2[21][15] + coeff0_22*dmats2[22][15] + coeff0_23*dmats2[23][15] + coeff0_24*dmats2[24][15] + coeff0_25*dmats2[25][15] + coeff0_26*dmats2[26][15] + coeff0_27*dmats2[27][15] + coeff0_28*dmats2[28][15] + coeff0_29*dmats2[29][15] + coeff0_30*dmats2[30][15] + coeff0_31*dmats2[31][15] + coeff0_32*dmats2[32][15] + coeff0_33*dmats2[33][15] + coeff0_34*dmats2[34][15] + coeff0_35*dmats2[35][15] + coeff0_36*dmats2[36][15] + coeff0_37*dmats2[37][15] + coeff0_38*dmats2[38][15] + coeff0_39*dmats2[39][15] + coeff0_40*dmats2[40][15] + coeff0_41*dmats2[41][15] + coeff0_42*dmats2[42][15] + coeff0_43*dmats2[43][15] + coeff0_44*dmats2[44][15] + coeff0_45*dmats2[45][15] + coeff0_46*dmats2[46][15] + coeff0_47*dmats2[47][15] + coeff0_48*dmats2[48][15] + coeff0_49*dmats2[49][15] + coeff0_50*dmats2[50][15] + coeff0_51*dmats2[51][15] + coeff0_52*dmats2[52][15] + coeff0_53*dmats2[53][15] + coeff0_54*dmats2[54][15] + coeff0_55*dmats2[55][15];
          new_coeff0_16 = coeff0_0*dmats2[0][16] + coeff0_1*dmats2[1][16] + coeff0_2*dmats2[2][16] + coeff0_3*dmats2[3][16] + coeff0_4*dmats2[4][16] + coeff0_5*dmats2[5][16] + coeff0_6*dmats2[6][16] + coeff0_7*dmats2[7][16] + coeff0_8*dmats2[8][16] + coeff0_9*dmats2[9][16] + coeff0_10*dmats2[10][16] + coeff0_11*dmats2[11][16] + coeff0_12*dmats2[12][16] + coeff0_13*dmats2[13][16] + coeff0_14*dmats2[14][16] + coeff0_15*dmats2[15][16] + coeff0_16*dmats2[16][16] + coeff0_17*dmats2[17][16] + coeff0_18*dmats2[18][16] + coeff0_19*dmats2[19][16] + coeff0_20*dmats2[20][16] + coeff0_21*dmats2[21][16] + coeff0_22*dmats2[22][16] + coeff0_23*dmats2[23][16] + coeff0_24*dmats2[24][16] + coeff0_25*dmats2[25][16] + coeff0_26*dmats2[26][16] + coeff0_27*dmats2[27][16] + coeff0_28*dmats2[28][16] + coeff0_29*dmats2[29][16] + coeff0_30*dmats2[30][16] + coeff0_31*dmats2[31][16] + coeff0_32*dmats2[32][16] + coeff0_33*dmats2[33][16] + coeff0_34*dmats2[34][16] + coeff0_35*dmats2[35][16] + coeff0_36*dmats2[36][16] + coeff0_37*dmats2[37][16] + coeff0_38*dmats2[38][16] + coeff0_39*dmats2[39][16] + coeff0_40*dmats2[40][16] + coeff0_41*dmats2[41][16] + coeff0_42*dmats2[42][16] + coeff0_43*dmats2[43][16] + coeff0_44*dmats2[44][16] + coeff0_45*dmats2[45][16] + coeff0_46*dmats2[46][16] + coeff0_47*dmats2[47][16] + coeff0_48*dmats2[48][16] + coeff0_49*dmats2[49][16] + coeff0_50*dmats2[50][16] + coeff0_51*dmats2[51][16] + coeff0_52*dmats2[52][16] + coeff0_53*dmats2[53][16] + coeff0_54*dmats2[54][16] + coeff0_55*dmats2[55][16];
          new_coeff0_17 = coeff0_0*dmats2[0][17] + coeff0_1*dmats2[1][17] + coeff0_2*dmats2[2][17] + coeff0_3*dmats2[3][17] + coeff0_4*dmats2[4][17] + coeff0_5*dmats2[5][17] + coeff0_6*dmats2[6][17] + coeff0_7*dmats2[7][17] + coeff0_8*dmats2[8][17] + coeff0_9*dmats2[9][17] + coeff0_10*dmats2[10][17] + coeff0_11*dmats2[11][17] + coeff0_12*dmats2[12][17] + coeff0_13*dmats2[13][17] + coeff0_14*dmats2[14][17] + coeff0_15*dmats2[15][17] + coeff0_16*dmats2[16][17] + coeff0_17*dmats2[17][17] + coeff0_18*dmats2[18][17] + coeff0_19*dmats2[19][17] + coeff0_20*dmats2[20][17] + coeff0_21*dmats2[21][17] + coeff0_22*dmats2[22][17] + coeff0_23*dmats2[23][17] + coeff0_24*dmats2[24][17] + coeff0_25*dmats2[25][17] + coeff0_26*dmats2[26][17] + coeff0_27*dmats2[27][17] + coeff0_28*dmats2[28][17] + coeff0_29*dmats2[29][17] + coeff0_30*dmats2[30][17] + coeff0_31*dmats2[31][17] + coeff0_32*dmats2[32][17] + coeff0_33*dmats2[33][17] + coeff0_34*dmats2[34][17] + coeff0_35*dmats2[35][17] + coeff0_36*dmats2[36][17] + coeff0_37*dmats2[37][17] + coeff0_38*dmats2[38][17] + coeff0_39*dmats2[39][17] + coeff0_40*dmats2[40][17] + coeff0_41*dmats2[41][17] + coeff0_42*dmats2[42][17] + coeff0_43*dmats2[43][17] + coeff0_44*dmats2[44][17] + coeff0_45*dmats2[45][17] + coeff0_46*dmats2[46][17] + coeff0_47*dmats2[47][17] + coeff0_48*dmats2[48][17] + coeff0_49*dmats2[49][17] + coeff0_50*dmats2[50][17] + coeff0_51*dmats2[51][17] + coeff0_52*dmats2[52][17] + coeff0_53*dmats2[53][17] + coeff0_54*dmats2[54][17] + coeff0_55*dmats2[55][17];
          new_coeff0_18 = coeff0_0*dmats2[0][18] + coeff0_1*dmats2[1][18] + coeff0_2*dmats2[2][18] + coeff0_3*dmats2[3][18] + coeff0_4*dmats2[4][18] + coeff0_5*dmats2[5][18] + coeff0_6*dmats2[6][18] + coeff0_7*dmats2[7][18] + coeff0_8*dmats2[8][18] + coeff0_9*dmats2[9][18] + coeff0_10*dmats2[10][18] + coeff0_11*dmats2[11][18] + coeff0_12*dmats2[12][18] + coeff0_13*dmats2[13][18] + coeff0_14*dmats2[14][18] + coeff0_15*dmats2[15][18] + coeff0_16*dmats2[16][18] + coeff0_17*dmats2[17][18] + coeff0_18*dmats2[18][18] + coeff0_19*dmats2[19][18] + coeff0_20*dmats2[20][18] + coeff0_21*dmats2[21][18] + coeff0_22*dmats2[22][18] + coeff0_23*dmats2[23][18] + coeff0_24*dmats2[24][18] + coeff0_25*dmats2[25][18] + coeff0_26*dmats2[26][18] + coeff0_27*dmats2[27][18] + coeff0_28*dmats2[28][18] + coeff0_29*dmats2[29][18] + coeff0_30*dmats2[30][18] + coeff0_31*dmats2[31][18] + coeff0_32*dmats2[32][18] + coeff0_33*dmats2[33][18] + coeff0_34*dmats2[34][18] + coeff0_35*dmats2[35][18] + coeff0_36*dmats2[36][18] + coeff0_37*dmats2[37][18] + coeff0_38*dmats2[38][18] + coeff0_39*dmats2[39][18] + coeff0_40*dmats2[40][18] + coeff0_41*dmats2[41][18] + coeff0_42*dmats2[42][18] + coeff0_43*dmats2[43][18] + coeff0_44*dmats2[44][18] + coeff0_45*dmats2[45][18] + coeff0_46*dmats2[46][18] + coeff0_47*dmats2[47][18] + coeff0_48*dmats2[48][18] + coeff0_49*dmats2[49][18] + coeff0_50*dmats2[50][18] + coeff0_51*dmats2[51][18] + coeff0_52*dmats2[52][18] + coeff0_53*dmats2[53][18] + coeff0_54*dmats2[54][18] + coeff0_55*dmats2[55][18];
          new_coeff0_19 = coeff0_0*dmats2[0][19] + coeff0_1*dmats2[1][19] + coeff0_2*dmats2[2][19] + coeff0_3*dmats2[3][19] + coeff0_4*dmats2[4][19] + coeff0_5*dmats2[5][19] + coeff0_6*dmats2[6][19] + coeff0_7*dmats2[7][19] + coeff0_8*dmats2[8][19] + coeff0_9*dmats2[9][19] + coeff0_10*dmats2[10][19] + coeff0_11*dmats2[11][19] + coeff0_12*dmats2[12][19] + coeff0_13*dmats2[13][19] + coeff0_14*dmats2[14][19] + coeff0_15*dmats2[15][19] + coeff0_16*dmats2[16][19] + coeff0_17*dmats2[17][19] + coeff0_18*dmats2[18][19] + coeff0_19*dmats2[19][19] + coeff0_20*dmats2[20][19] + coeff0_21*dmats2[21][19] + coeff0_22*dmats2[22][19] + coeff0_23*dmats2[23][19] + coeff0_24*dmats2[24][19] + coeff0_25*dmats2[25][19] + coeff0_26*dmats2[26][19] + coeff0_27*dmats2[27][19] + coeff0_28*dmats2[28][19] + coeff0_29*dmats2[29][19] + coeff0_30*dmats2[30][19] + coeff0_31*dmats2[31][19] + coeff0_32*dmats2[32][19] + coeff0_33*dmats2[33][19] + coeff0_34*dmats2[34][19] + coeff0_35*dmats2[35][19] + coeff0_36*dmats2[36][19] + coeff0_37*dmats2[37][19] + coeff0_38*dmats2[38][19] + coeff0_39*dmats2[39][19] + coeff0_40*dmats2[40][19] + coeff0_41*dmats2[41][19] + coeff0_42*dmats2[42][19] + coeff0_43*dmats2[43][19] + coeff0_44*dmats2[44][19] + coeff0_45*dmats2[45][19] + coeff0_46*dmats2[46][19] + coeff0_47*dmats2[47][19] + coeff0_48*dmats2[48][19] + coeff0_49*dmats2[49][19] + coeff0_50*dmats2[50][19] + coeff0_51*dmats2[51][19] + coeff0_52*dmats2[52][19] + coeff0_53*dmats2[53][19] + coeff0_54*dmats2[54][19] + coeff0_55*dmats2[55][19];
          new_coeff0_20 = coeff0_0*dmats2[0][20] + coeff0_1*dmats2[1][20] + coeff0_2*dmats2[2][20] + coeff0_3*dmats2[3][20] + coeff0_4*dmats2[4][20] + coeff0_5*dmats2[5][20] + coeff0_6*dmats2[6][20] + coeff0_7*dmats2[7][20] + coeff0_8*dmats2[8][20] + coeff0_9*dmats2[9][20] + coeff0_10*dmats2[10][20] + coeff0_11*dmats2[11][20] + coeff0_12*dmats2[12][20] + coeff0_13*dmats2[13][20] + coeff0_14*dmats2[14][20] + coeff0_15*dmats2[15][20] + coeff0_16*dmats2[16][20] + coeff0_17*dmats2[17][20] + coeff0_18*dmats2[18][20] + coeff0_19*dmats2[19][20] + coeff0_20*dmats2[20][20] + coeff0_21*dmats2[21][20] + coeff0_22*dmats2[22][20] + coeff0_23*dmats2[23][20] + coeff0_24*dmats2[24][20] + coeff0_25*dmats2[25][20] + coeff0_26*dmats2[26][20] + coeff0_27*dmats2[27][20] + coeff0_28*dmats2[28][20] + coeff0_29*dmats2[29][20] + coeff0_30*dmats2[30][20] + coeff0_31*dmats2[31][20] + coeff0_32*dmats2[32][20] + coeff0_33*dmats2[33][20] + coeff0_34*dmats2[34][20] + coeff0_35*dmats2[35][20] + coeff0_36*dmats2[36][20] + coeff0_37*dmats2[37][20] + coeff0_38*dmats2[38][20] + coeff0_39*dmats2[39][20] + coeff0_40*dmats2[40][20] + coeff0_41*dmats2[41][20] + coeff0_42*dmats2[42][20] + coeff0_43*dmats2[43][20] + coeff0_44*dmats2[44][20] + coeff0_45*dmats2[45][20] + coeff0_46*dmats2[46][20] + coeff0_47*dmats2[47][20] + coeff0_48*dmats2[48][20] + coeff0_49*dmats2[49][20] + coeff0_50*dmats2[50][20] + coeff0_51*dmats2[51][20] + coeff0_52*dmats2[52][20] + coeff0_53*dmats2[53][20] + coeff0_54*dmats2[54][20] + coeff0_55*dmats2[55][20];
          new_coeff0_21 = coeff0_0*dmats2[0][21] + coeff0_1*dmats2[1][21] + coeff0_2*dmats2[2][21] + coeff0_3*dmats2[3][21] + coeff0_4*dmats2[4][21] + coeff0_5*dmats2[5][21] + coeff0_6*dmats2[6][21] + coeff0_7*dmats2[7][21] + coeff0_8*dmats2[8][21] + coeff0_9*dmats2[9][21] + coeff0_10*dmats2[10][21] + coeff0_11*dmats2[11][21] + coeff0_12*dmats2[12][21] + coeff0_13*dmats2[13][21] + coeff0_14*dmats2[14][21] + coeff0_15*dmats2[15][21] + coeff0_16*dmats2[16][21] + coeff0_17*dmats2[17][21] + coeff0_18*dmats2[18][21] + coeff0_19*dmats2[19][21] + coeff0_20*dmats2[20][21] + coeff0_21*dmats2[21][21] + coeff0_22*dmats2[22][21] + coeff0_23*dmats2[23][21] + coeff0_24*dmats2[24][21] + coeff0_25*dmats2[25][21] + coeff0_26*dmats2[26][21] + coeff0_27*dmats2[27][21] + coeff0_28*dmats2[28][21] + coeff0_29*dmats2[29][21] + coeff0_30*dmats2[30][21] + coeff0_31*dmats2[31][21] + coeff0_32*dmats2[32][21] + coeff0_33*dmats2[33][21] + coeff0_34*dmats2[34][21] + coeff0_35*dmats2[35][21] + coeff0_36*dmats2[36][21] + coeff0_37*dmats2[37][21] + coeff0_38*dmats2[38][21] + coeff0_39*dmats2[39][21] + coeff0_40*dmats2[40][21] + coeff0_41*dmats2[41][21] + coeff0_42*dmats2[42][21] + coeff0_43*dmats2[43][21] + coeff0_44*dmats2[44][21] + coeff0_45*dmats2[45][21] + coeff0_46*dmats2[46][21] + coeff0_47*dmats2[47][21] + coeff0_48*dmats2[48][21] + coeff0_49*dmats2[49][21] + coeff0_50*dmats2[50][21] + coeff0_51*dmats2[51][21] + coeff0_52*dmats2[52][21] + coeff0_53*dmats2[53][21] + coeff0_54*dmats2[54][21] + coeff0_55*dmats2[55][21];
          new_coeff0_22 = coeff0_0*dmats2[0][22] + coeff0_1*dmats2[1][22] + coeff0_2*dmats2[2][22] + coeff0_3*dmats2[3][22] + coeff0_4*dmats2[4][22] + coeff0_5*dmats2[5][22] + coeff0_6*dmats2[6][22] + coeff0_7*dmats2[7][22] + coeff0_8*dmats2[8][22] + coeff0_9*dmats2[9][22] + coeff0_10*dmats2[10][22] + coeff0_11*dmats2[11][22] + coeff0_12*dmats2[12][22] + coeff0_13*dmats2[13][22] + coeff0_14*dmats2[14][22] + coeff0_15*dmats2[15][22] + coeff0_16*dmats2[16][22] + coeff0_17*dmats2[17][22] + coeff0_18*dmats2[18][22] + coeff0_19*dmats2[19][22] + coeff0_20*dmats2[20][22] + coeff0_21*dmats2[21][22] + coeff0_22*dmats2[22][22] + coeff0_23*dmats2[23][22] + coeff0_24*dmats2[24][22] + coeff0_25*dmats2[25][22] + coeff0_26*dmats2[26][22] + coeff0_27*dmats2[27][22] + coeff0_28*dmats2[28][22] + coeff0_29*dmats2[29][22] + coeff0_30*dmats2[30][22] + coeff0_31*dmats2[31][22] + coeff0_32*dmats2[32][22] + coeff0_33*dmats2[33][22] + coeff0_34*dmats2[34][22] + coeff0_35*dmats2[35][22] + coeff0_36*dmats2[36][22] + coeff0_37*dmats2[37][22] + coeff0_38*dmats2[38][22] + coeff0_39*dmats2[39][22] + coeff0_40*dmats2[40][22] + coeff0_41*dmats2[41][22] + coeff0_42*dmats2[42][22] + coeff0_43*dmats2[43][22] + coeff0_44*dmats2[44][22] + coeff0_45*dmats2[45][22] + coeff0_46*dmats2[46][22] + coeff0_47*dmats2[47][22] + coeff0_48*dmats2[48][22] + coeff0_49*dmats2[49][22] + coeff0_50*dmats2[50][22] + coeff0_51*dmats2[51][22] + coeff0_52*dmats2[52][22] + coeff0_53*dmats2[53][22] + coeff0_54*dmats2[54][22] + coeff0_55*dmats2[55][22];
          new_coeff0_23 = coeff0_0*dmats2[0][23] + coeff0_1*dmats2[1][23] + coeff0_2*dmats2[2][23] + coeff0_3*dmats2[3][23] + coeff0_4*dmats2[4][23] + coeff0_5*dmats2[5][23] + coeff0_6*dmats2[6][23] + coeff0_7*dmats2[7][23] + coeff0_8*dmats2[8][23] + coeff0_9*dmats2[9][23] + coeff0_10*dmats2[10][23] + coeff0_11*dmats2[11][23] + coeff0_12*dmats2[12][23] + coeff0_13*dmats2[13][23] + coeff0_14*dmats2[14][23] + coeff0_15*dmats2[15][23] + coeff0_16*dmats2[16][23] + coeff0_17*dmats2[17][23] + coeff0_18*dmats2[18][23] + coeff0_19*dmats2[19][23] + coeff0_20*dmats2[20][23] + coeff0_21*dmats2[21][23] + coeff0_22*dmats2[22][23] + coeff0_23*dmats2[23][23] + coeff0_24*dmats2[24][23] + coeff0_25*dmats2[25][23] + coeff0_26*dmats2[26][23] + coeff0_27*dmats2[27][23] + coeff0_28*dmats2[28][23] + coeff0_29*dmats2[29][23] + coeff0_30*dmats2[30][23] + coeff0_31*dmats2[31][23] + coeff0_32*dmats2[32][23] + coeff0_33*dmats2[33][23] + coeff0_34*dmats2[34][23] + coeff0_35*dmats2[35][23] + coeff0_36*dmats2[36][23] + coeff0_37*dmats2[37][23] + coeff0_38*dmats2[38][23] + coeff0_39*dmats2[39][23] + coeff0_40*dmats2[40][23] + coeff0_41*dmats2[41][23] + coeff0_42*dmats2[42][23] + coeff0_43*dmats2[43][23] + coeff0_44*dmats2[44][23] + coeff0_45*dmats2[45][23] + coeff0_46*dmats2[46][23] + coeff0_47*dmats2[47][23] + coeff0_48*dmats2[48][23] + coeff0_49*dmats2[49][23] + coeff0_50*dmats2[50][23] + coeff0_51*dmats2[51][23] + coeff0_52*dmats2[52][23] + coeff0_53*dmats2[53][23] + coeff0_54*dmats2[54][23] + coeff0_55*dmats2[55][23];
          new_coeff0_24 = coeff0_0*dmats2[0][24] + coeff0_1*dmats2[1][24] + coeff0_2*dmats2[2][24] + coeff0_3*dmats2[3][24] + coeff0_4*dmats2[4][24] + coeff0_5*dmats2[5][24] + coeff0_6*dmats2[6][24] + coeff0_7*dmats2[7][24] + coeff0_8*dmats2[8][24] + coeff0_9*dmats2[9][24] + coeff0_10*dmats2[10][24] + coeff0_11*dmats2[11][24] + coeff0_12*dmats2[12][24] + coeff0_13*dmats2[13][24] + coeff0_14*dmats2[14][24] + coeff0_15*dmats2[15][24] + coeff0_16*dmats2[16][24] + coeff0_17*dmats2[17][24] + coeff0_18*dmats2[18][24] + coeff0_19*dmats2[19][24] + coeff0_20*dmats2[20][24] + coeff0_21*dmats2[21][24] + coeff0_22*dmats2[22][24] + coeff0_23*dmats2[23][24] + coeff0_24*dmats2[24][24] + coeff0_25*dmats2[25][24] + coeff0_26*dmats2[26][24] + coeff0_27*dmats2[27][24] + coeff0_28*dmats2[28][24] + coeff0_29*dmats2[29][24] + coeff0_30*dmats2[30][24] + coeff0_31*dmats2[31][24] + coeff0_32*dmats2[32][24] + coeff0_33*dmats2[33][24] + coeff0_34*dmats2[34][24] + coeff0_35*dmats2[35][24] + coeff0_36*dmats2[36][24] + coeff0_37*dmats2[37][24] + coeff0_38*dmats2[38][24] + coeff0_39*dmats2[39][24] + coeff0_40*dmats2[40][24] + coeff0_41*dmats2[41][24] + coeff0_42*dmats2[42][24] + coeff0_43*dmats2[43][24] + coeff0_44*dmats2[44][24] + coeff0_45*dmats2[45][24] + coeff0_46*dmats2[46][24] + coeff0_47*dmats2[47][24] + coeff0_48*dmats2[48][24] + coeff0_49*dmats2[49][24] + coeff0_50*dmats2[50][24] + coeff0_51*dmats2[51][24] + coeff0_52*dmats2[52][24] + coeff0_53*dmats2[53][24] + coeff0_54*dmats2[54][24] + coeff0_55*dmats2[55][24];
          new_coeff0_25 = coeff0_0*dmats2[0][25] + coeff0_1*dmats2[1][25] + coeff0_2*dmats2[2][25] + coeff0_3*dmats2[3][25] + coeff0_4*dmats2[4][25] + coeff0_5*dmats2[5][25] + coeff0_6*dmats2[6][25] + coeff0_7*dmats2[7][25] + coeff0_8*dmats2[8][25] + coeff0_9*dmats2[9][25] + coeff0_10*dmats2[10][25] + coeff0_11*dmats2[11][25] + coeff0_12*dmats2[12][25] + coeff0_13*dmats2[13][25] + coeff0_14*dmats2[14][25] + coeff0_15*dmats2[15][25] + coeff0_16*dmats2[16][25] + coeff0_17*dmats2[17][25] + coeff0_18*dmats2[18][25] + coeff0_19*dmats2[19][25] + coeff0_20*dmats2[20][25] + coeff0_21*dmats2[21][25] + coeff0_22*dmats2[22][25] + coeff0_23*dmats2[23][25] + coeff0_24*dmats2[24][25] + coeff0_25*dmats2[25][25] + coeff0_26*dmats2[26][25] + coeff0_27*dmats2[27][25] + coeff0_28*dmats2[28][25] + coeff0_29*dmats2[29][25] + coeff0_30*dmats2[30][25] + coeff0_31*dmats2[31][25] + coeff0_32*dmats2[32][25] + coeff0_33*dmats2[33][25] + coeff0_34*dmats2[34][25] + coeff0_35*dmats2[35][25] + coeff0_36*dmats2[36][25] + coeff0_37*dmats2[37][25] + coeff0_38*dmats2[38][25] + coeff0_39*dmats2[39][25] + coeff0_40*dmats2[40][25] + coeff0_41*dmats2[41][25] + coeff0_42*dmats2[42][25] + coeff0_43*dmats2[43][25] + coeff0_44*dmats2[44][25] + coeff0_45*dmats2[45][25] + coeff0_46*dmats2[46][25] + coeff0_47*dmats2[47][25] + coeff0_48*dmats2[48][25] + coeff0_49*dmats2[49][25] + coeff0_50*dmats2[50][25] + coeff0_51*dmats2[51][25] + coeff0_52*dmats2[52][25] + coeff0_53*dmats2[53][25] + coeff0_54*dmats2[54][25] + coeff0_55*dmats2[55][25];
          new_coeff0_26 = coeff0_0*dmats2[0][26] + coeff0_1*dmats2[1][26] + coeff0_2*dmats2[2][26] + coeff0_3*dmats2[3][26] + coeff0_4*dmats2[4][26] + coeff0_5*dmats2[5][26] + coeff0_6*dmats2[6][26] + coeff0_7*dmats2[7][26] + coeff0_8*dmats2[8][26] + coeff0_9*dmats2[9][26] + coeff0_10*dmats2[10][26] + coeff0_11*dmats2[11][26] + coeff0_12*dmats2[12][26] + coeff0_13*dmats2[13][26] + coeff0_14*dmats2[14][26] + coeff0_15*dmats2[15][26] + coeff0_16*dmats2[16][26] + coeff0_17*dmats2[17][26] + coeff0_18*dmats2[18][26] + coeff0_19*dmats2[19][26] + coeff0_20*dmats2[20][26] + coeff0_21*dmats2[21][26] + coeff0_22*dmats2[22][26] + coeff0_23*dmats2[23][26] + coeff0_24*dmats2[24][26] + coeff0_25*dmats2[25][26] + coeff0_26*dmats2[26][26] + coeff0_27*dmats2[27][26] + coeff0_28*dmats2[28][26] + coeff0_29*dmats2[29][26] + coeff0_30*dmats2[30][26] + coeff0_31*dmats2[31][26] + coeff0_32*dmats2[32][26] + coeff0_33*dmats2[33][26] + coeff0_34*dmats2[34][26] + coeff0_35*dmats2[35][26] + coeff0_36*dmats2[36][26] + coeff0_37*dmats2[37][26] + coeff0_38*dmats2[38][26] + coeff0_39*dmats2[39][26] + coeff0_40*dmats2[40][26] + coeff0_41*dmats2[41][26] + coeff0_42*dmats2[42][26] + coeff0_43*dmats2[43][26] + coeff0_44*dmats2[44][26] + coeff0_45*dmats2[45][26] + coeff0_46*dmats2[46][26] + coeff0_47*dmats2[47][26] + coeff0_48*dmats2[48][26] + coeff0_49*dmats2[49][26] + coeff0_50*dmats2[50][26] + coeff0_51*dmats2[51][26] + coeff0_52*dmats2[52][26] + coeff0_53*dmats2[53][26] + coeff0_54*dmats2[54][26] + coeff0_55*dmats2[55][26];
          new_coeff0_27 = coeff0_0*dmats2[0][27] + coeff0_1*dmats2[1][27] + coeff0_2*dmats2[2][27] + coeff0_3*dmats2[3][27] + coeff0_4*dmats2[4][27] + coeff0_5*dmats2[5][27] + coeff0_6*dmats2[6][27] + coeff0_7*dmats2[7][27] + coeff0_8*dmats2[8][27] + coeff0_9*dmats2[9][27] + coeff0_10*dmats2[10][27] + coeff0_11*dmats2[11][27] + coeff0_12*dmats2[12][27] + coeff0_13*dmats2[13][27] + coeff0_14*dmats2[14][27] + coeff0_15*dmats2[15][27] + coeff0_16*dmats2[16][27] + coeff0_17*dmats2[17][27] + coeff0_18*dmats2[18][27] + coeff0_19*dmats2[19][27] + coeff0_20*dmats2[20][27] + coeff0_21*dmats2[21][27] + coeff0_22*dmats2[22][27] + coeff0_23*dmats2[23][27] + coeff0_24*dmats2[24][27] + coeff0_25*dmats2[25][27] + coeff0_26*dmats2[26][27] + coeff0_27*dmats2[27][27] + coeff0_28*dmats2[28][27] + coeff0_29*dmats2[29][27] + coeff0_30*dmats2[30][27] + coeff0_31*dmats2[31][27] + coeff0_32*dmats2[32][27] + coeff0_33*dmats2[33][27] + coeff0_34*dmats2[34][27] + coeff0_35*dmats2[35][27] + coeff0_36*dmats2[36][27] + coeff0_37*dmats2[37][27] + coeff0_38*dmats2[38][27] + coeff0_39*dmats2[39][27] + coeff0_40*dmats2[40][27] + coeff0_41*dmats2[41][27] + coeff0_42*dmats2[42][27] + coeff0_43*dmats2[43][27] + coeff0_44*dmats2[44][27] + coeff0_45*dmats2[45][27] + coeff0_46*dmats2[46][27] + coeff0_47*dmats2[47][27] + coeff0_48*dmats2[48][27] + coeff0_49*dmats2[49][27] + coeff0_50*dmats2[50][27] + coeff0_51*dmats2[51][27] + coeff0_52*dmats2[52][27] + coeff0_53*dmats2[53][27] + coeff0_54*dmats2[54][27] + coeff0_55*dmats2[55][27];
          new_coeff0_28 = coeff0_0*dmats2[0][28] + coeff0_1*dmats2[1][28] + coeff0_2*dmats2[2][28] + coeff0_3*dmats2[3][28] + coeff0_4*dmats2[4][28] + coeff0_5*dmats2[5][28] + coeff0_6*dmats2[6][28] + coeff0_7*dmats2[7][28] + coeff0_8*dmats2[8][28] + coeff0_9*dmats2[9][28] + coeff0_10*dmats2[10][28] + coeff0_11*dmats2[11][28] + coeff0_12*dmats2[12][28] + coeff0_13*dmats2[13][28] + coeff0_14*dmats2[14][28] + coeff0_15*dmats2[15][28] + coeff0_16*dmats2[16][28] + coeff0_17*dmats2[17][28] + coeff0_18*dmats2[18][28] + coeff0_19*dmats2[19][28] + coeff0_20*dmats2[20][28] + coeff0_21*dmats2[21][28] + coeff0_22*dmats2[22][28] + coeff0_23*dmats2[23][28] + coeff0_24*dmats2[24][28] + coeff0_25*dmats2[25][28] + coeff0_26*dmats2[26][28] + coeff0_27*dmats2[27][28] + coeff0_28*dmats2[28][28] + coeff0_29*dmats2[29][28] + coeff0_30*dmats2[30][28] + coeff0_31*dmats2[31][28] + coeff0_32*dmats2[32][28] + coeff0_33*dmats2[33][28] + coeff0_34*dmats2[34][28] + coeff0_35*dmats2[35][28] + coeff0_36*dmats2[36][28] + coeff0_37*dmats2[37][28] + coeff0_38*dmats2[38][28] + coeff0_39*dmats2[39][28] + coeff0_40*dmats2[40][28] + coeff0_41*dmats2[41][28] + coeff0_42*dmats2[42][28] + coeff0_43*dmats2[43][28] + coeff0_44*dmats2[44][28] + coeff0_45*dmats2[45][28] + coeff0_46*dmats2[46][28] + coeff0_47*dmats2[47][28] + coeff0_48*dmats2[48][28] + coeff0_49*dmats2[49][28] + coeff0_50*dmats2[50][28] + coeff0_51*dmats2[51][28] + coeff0_52*dmats2[52][28] + coeff0_53*dmats2[53][28] + coeff0_54*dmats2[54][28] + coeff0_55*dmats2[55][28];
          new_coeff0_29 = coeff0_0*dmats2[0][29] + coeff0_1*dmats2[1][29] + coeff0_2*dmats2[2][29] + coeff0_3*dmats2[3][29] + coeff0_4*dmats2[4][29] + coeff0_5*dmats2[5][29] + coeff0_6*dmats2[6][29] + coeff0_7*dmats2[7][29] + coeff0_8*dmats2[8][29] + coeff0_9*dmats2[9][29] + coeff0_10*dmats2[10][29] + coeff0_11*dmats2[11][29] + coeff0_12*dmats2[12][29] + coeff0_13*dmats2[13][29] + coeff0_14*dmats2[14][29] + coeff0_15*dmats2[15][29] + coeff0_16*dmats2[16][29] + coeff0_17*dmats2[17][29] + coeff0_18*dmats2[18][29] + coeff0_19*dmats2[19][29] + coeff0_20*dmats2[20][29] + coeff0_21*dmats2[21][29] + coeff0_22*dmats2[22][29] + coeff0_23*dmats2[23][29] + coeff0_24*dmats2[24][29] + coeff0_25*dmats2[25][29] + coeff0_26*dmats2[26][29] + coeff0_27*dmats2[27][29] + coeff0_28*dmats2[28][29] + coeff0_29*dmats2[29][29] + coeff0_30*dmats2[30][29] + coeff0_31*dmats2[31][29] + coeff0_32*dmats2[32][29] + coeff0_33*dmats2[33][29] + coeff0_34*dmats2[34][29] + coeff0_35*dmats2[35][29] + coeff0_36*dmats2[36][29] + coeff0_37*dmats2[37][29] + coeff0_38*dmats2[38][29] + coeff0_39*dmats2[39][29] + coeff0_40*dmats2[40][29] + coeff0_41*dmats2[41][29] + coeff0_42*dmats2[42][29] + coeff0_43*dmats2[43][29] + coeff0_44*dmats2[44][29] + coeff0_45*dmats2[45][29] + coeff0_46*dmats2[46][29] + coeff0_47*dmats2[47][29] + coeff0_48*dmats2[48][29] + coeff0_49*dmats2[49][29] + coeff0_50*dmats2[50][29] + coeff0_51*dmats2[51][29] + coeff0_52*dmats2[52][29] + coeff0_53*dmats2[53][29] + coeff0_54*dmats2[54][29] + coeff0_55*dmats2[55][29];
          new_coeff0_30 = coeff0_0*dmats2[0][30] + coeff0_1*dmats2[1][30] + coeff0_2*dmats2[2][30] + coeff0_3*dmats2[3][30] + coeff0_4*dmats2[4][30] + coeff0_5*dmats2[5][30] + coeff0_6*dmats2[6][30] + coeff0_7*dmats2[7][30] + coeff0_8*dmats2[8][30] + coeff0_9*dmats2[9][30] + coeff0_10*dmats2[10][30] + coeff0_11*dmats2[11][30] + coeff0_12*dmats2[12][30] + coeff0_13*dmats2[13][30] + coeff0_14*dmats2[14][30] + coeff0_15*dmats2[15][30] + coeff0_16*dmats2[16][30] + coeff0_17*dmats2[17][30] + coeff0_18*dmats2[18][30] + coeff0_19*dmats2[19][30] + coeff0_20*dmats2[20][30] + coeff0_21*dmats2[21][30] + coeff0_22*dmats2[22][30] + coeff0_23*dmats2[23][30] + coeff0_24*dmats2[24][30] + coeff0_25*dmats2[25][30] + coeff0_26*dmats2[26][30] + coeff0_27*dmats2[27][30] + coeff0_28*dmats2[28][30] + coeff0_29*dmats2[29][30] + coeff0_30*dmats2[30][30] + coeff0_31*dmats2[31][30] + coeff0_32*dmats2[32][30] + coeff0_33*dmats2[33][30] + coeff0_34*dmats2[34][30] + coeff0_35*dmats2[35][30] + coeff0_36*dmats2[36][30] + coeff0_37*dmats2[37][30] + coeff0_38*dmats2[38][30] + coeff0_39*dmats2[39][30] + coeff0_40*dmats2[40][30] + coeff0_41*dmats2[41][30] + coeff0_42*dmats2[42][30] + coeff0_43*dmats2[43][30] + coeff0_44*dmats2[44][30] + coeff0_45*dmats2[45][30] + coeff0_46*dmats2[46][30] + coeff0_47*dmats2[47][30] + coeff0_48*dmats2[48][30] + coeff0_49*dmats2[49][30] + coeff0_50*dmats2[50][30] + coeff0_51*dmats2[51][30] + coeff0_52*dmats2[52][30] + coeff0_53*dmats2[53][30] + coeff0_54*dmats2[54][30] + coeff0_55*dmats2[55][30];
          new_coeff0_31 = coeff0_0*dmats2[0][31] + coeff0_1*dmats2[1][31] + coeff0_2*dmats2[2][31] + coeff0_3*dmats2[3][31] + coeff0_4*dmats2[4][31] + coeff0_5*dmats2[5][31] + coeff0_6*dmats2[6][31] + coeff0_7*dmats2[7][31] + coeff0_8*dmats2[8][31] + coeff0_9*dmats2[9][31] + coeff0_10*dmats2[10][31] + coeff0_11*dmats2[11][31] + coeff0_12*dmats2[12][31] + coeff0_13*dmats2[13][31] + coeff0_14*dmats2[14][31] + coeff0_15*dmats2[15][31] + coeff0_16*dmats2[16][31] + coeff0_17*dmats2[17][31] + coeff0_18*dmats2[18][31] + coeff0_19*dmats2[19][31] + coeff0_20*dmats2[20][31] + coeff0_21*dmats2[21][31] + coeff0_22*dmats2[22][31] + coeff0_23*dmats2[23][31] + coeff0_24*dmats2[24][31] + coeff0_25*dmats2[25][31] + coeff0_26*dmats2[26][31] + coeff0_27*dmats2[27][31] + coeff0_28*dmats2[28][31] + coeff0_29*dmats2[29][31] + coeff0_30*dmats2[30][31] + coeff0_31*dmats2[31][31] + coeff0_32*dmats2[32][31] + coeff0_33*dmats2[33][31] + coeff0_34*dmats2[34][31] + coeff0_35*dmats2[35][31] + coeff0_36*dmats2[36][31] + coeff0_37*dmats2[37][31] + coeff0_38*dmats2[38][31] + coeff0_39*dmats2[39][31] + coeff0_40*dmats2[40][31] + coeff0_41*dmats2[41][31] + coeff0_42*dmats2[42][31] + coeff0_43*dmats2[43][31] + coeff0_44*dmats2[44][31] + coeff0_45*dmats2[45][31] + coeff0_46*dmats2[46][31] + coeff0_47*dmats2[47][31] + coeff0_48*dmats2[48][31] + coeff0_49*dmats2[49][31] + coeff0_50*dmats2[50][31] + coeff0_51*dmats2[51][31] + coeff0_52*dmats2[52][31] + coeff0_53*dmats2[53][31] + coeff0_54*dmats2[54][31] + coeff0_55*dmats2[55][31];
          new_coeff0_32 = coeff0_0*dmats2[0][32] + coeff0_1*dmats2[1][32] + coeff0_2*dmats2[2][32] + coeff0_3*dmats2[3][32] + coeff0_4*dmats2[4][32] + coeff0_5*dmats2[5][32] + coeff0_6*dmats2[6][32] + coeff0_7*dmats2[7][32] + coeff0_8*dmats2[8][32] + coeff0_9*dmats2[9][32] + coeff0_10*dmats2[10][32] + coeff0_11*dmats2[11][32] + coeff0_12*dmats2[12][32] + coeff0_13*dmats2[13][32] + coeff0_14*dmats2[14][32] + coeff0_15*dmats2[15][32] + coeff0_16*dmats2[16][32] + coeff0_17*dmats2[17][32] + coeff0_18*dmats2[18][32] + coeff0_19*dmats2[19][32] + coeff0_20*dmats2[20][32] + coeff0_21*dmats2[21][32] + coeff0_22*dmats2[22][32] + coeff0_23*dmats2[23][32] + coeff0_24*dmats2[24][32] + coeff0_25*dmats2[25][32] + coeff0_26*dmats2[26][32] + coeff0_27*dmats2[27][32] + coeff0_28*dmats2[28][32] + coeff0_29*dmats2[29][32] + coeff0_30*dmats2[30][32] + coeff0_31*dmats2[31][32] + coeff0_32*dmats2[32][32] + coeff0_33*dmats2[33][32] + coeff0_34*dmats2[34][32] + coeff0_35*dmats2[35][32] + coeff0_36*dmats2[36][32] + coeff0_37*dmats2[37][32] + coeff0_38*dmats2[38][32] + coeff0_39*dmats2[39][32] + coeff0_40*dmats2[40][32] + coeff0_41*dmats2[41][32] + coeff0_42*dmats2[42][32] + coeff0_43*dmats2[43][32] + coeff0_44*dmats2[44][32] + coeff0_45*dmats2[45][32] + coeff0_46*dmats2[46][32] + coeff0_47*dmats2[47][32] + coeff0_48*dmats2[48][32] + coeff0_49*dmats2[49][32] + coeff0_50*dmats2[50][32] + coeff0_51*dmats2[51][32] + coeff0_52*dmats2[52][32] + coeff0_53*dmats2[53][32] + coeff0_54*dmats2[54][32] + coeff0_55*dmats2[55][32];
          new_coeff0_33 = coeff0_0*dmats2[0][33] + coeff0_1*dmats2[1][33] + coeff0_2*dmats2[2][33] + coeff0_3*dmats2[3][33] + coeff0_4*dmats2[4][33] + coeff0_5*dmats2[5][33] + coeff0_6*dmats2[6][33] + coeff0_7*dmats2[7][33] + coeff0_8*dmats2[8][33] + coeff0_9*dmats2[9][33] + coeff0_10*dmats2[10][33] + coeff0_11*dmats2[11][33] + coeff0_12*dmats2[12][33] + coeff0_13*dmats2[13][33] + coeff0_14*dmats2[14][33] + coeff0_15*dmats2[15][33] + coeff0_16*dmats2[16][33] + coeff0_17*dmats2[17][33] + coeff0_18*dmats2[18][33] + coeff0_19*dmats2[19][33] + coeff0_20*dmats2[20][33] + coeff0_21*dmats2[21][33] + coeff0_22*dmats2[22][33] + coeff0_23*dmats2[23][33] + coeff0_24*dmats2[24][33] + coeff0_25*dmats2[25][33] + coeff0_26*dmats2[26][33] + coeff0_27*dmats2[27][33] + coeff0_28*dmats2[28][33] + coeff0_29*dmats2[29][33] + coeff0_30*dmats2[30][33] + coeff0_31*dmats2[31][33] + coeff0_32*dmats2[32][33] + coeff0_33*dmats2[33][33] + coeff0_34*dmats2[34][33] + coeff0_35*dmats2[35][33] + coeff0_36*dmats2[36][33] + coeff0_37*dmats2[37][33] + coeff0_38*dmats2[38][33] + coeff0_39*dmats2[39][33] + coeff0_40*dmats2[40][33] + coeff0_41*dmats2[41][33] + coeff0_42*dmats2[42][33] + coeff0_43*dmats2[43][33] + coeff0_44*dmats2[44][33] + coeff0_45*dmats2[45][33] + coeff0_46*dmats2[46][33] + coeff0_47*dmats2[47][33] + coeff0_48*dmats2[48][33] + coeff0_49*dmats2[49][33] + coeff0_50*dmats2[50][33] + coeff0_51*dmats2[51][33] + coeff0_52*dmats2[52][33] + coeff0_53*dmats2[53][33] + coeff0_54*dmats2[54][33] + coeff0_55*dmats2[55][33];
          new_coeff0_34 = coeff0_0*dmats2[0][34] + coeff0_1*dmats2[1][34] + coeff0_2*dmats2[2][34] + coeff0_3*dmats2[3][34] + coeff0_4*dmats2[4][34] + coeff0_5*dmats2[5][34] + coeff0_6*dmats2[6][34] + coeff0_7*dmats2[7][34] + coeff0_8*dmats2[8][34] + coeff0_9*dmats2[9][34] + coeff0_10*dmats2[10][34] + coeff0_11*dmats2[11][34] + coeff0_12*dmats2[12][34] + coeff0_13*dmats2[13][34] + coeff0_14*dmats2[14][34] + coeff0_15*dmats2[15][34] + coeff0_16*dmats2[16][34] + coeff0_17*dmats2[17][34] + coeff0_18*dmats2[18][34] + coeff0_19*dmats2[19][34] + coeff0_20*dmats2[20][34] + coeff0_21*dmats2[21][34] + coeff0_22*dmats2[22][34] + coeff0_23*dmats2[23][34] + coeff0_24*dmats2[24][34] + coeff0_25*dmats2[25][34] + coeff0_26*dmats2[26][34] + coeff0_27*dmats2[27][34] + coeff0_28*dmats2[28][34] + coeff0_29*dmats2[29][34] + coeff0_30*dmats2[30][34] + coeff0_31*dmats2[31][34] + coeff0_32*dmats2[32][34] + coeff0_33*dmats2[33][34] + coeff0_34*dmats2[34][34] + coeff0_35*dmats2[35][34] + coeff0_36*dmats2[36][34] + coeff0_37*dmats2[37][34] + coeff0_38*dmats2[38][34] + coeff0_39*dmats2[39][34] + coeff0_40*dmats2[40][34] + coeff0_41*dmats2[41][34] + coeff0_42*dmats2[42][34] + coeff0_43*dmats2[43][34] + coeff0_44*dmats2[44][34] + coeff0_45*dmats2[45][34] + coeff0_46*dmats2[46][34] + coeff0_47*dmats2[47][34] + coeff0_48*dmats2[48][34] + coeff0_49*dmats2[49][34] + coeff0_50*dmats2[50][34] + coeff0_51*dmats2[51][34] + coeff0_52*dmats2[52][34] + coeff0_53*dmats2[53][34] + coeff0_54*dmats2[54][34] + coeff0_55*dmats2[55][34];
          new_coeff0_35 = coeff0_0*dmats2[0][35] + coeff0_1*dmats2[1][35] + coeff0_2*dmats2[2][35] + coeff0_3*dmats2[3][35] + coeff0_4*dmats2[4][35] + coeff0_5*dmats2[5][35] + coeff0_6*dmats2[6][35] + coeff0_7*dmats2[7][35] + coeff0_8*dmats2[8][35] + coeff0_9*dmats2[9][35] + coeff0_10*dmats2[10][35] + coeff0_11*dmats2[11][35] + coeff0_12*dmats2[12][35] + coeff0_13*dmats2[13][35] + coeff0_14*dmats2[14][35] + coeff0_15*dmats2[15][35] + coeff0_16*dmats2[16][35] + coeff0_17*dmats2[17][35] + coeff0_18*dmats2[18][35] + coeff0_19*dmats2[19][35] + coeff0_20*dmats2[20][35] + coeff0_21*dmats2[21][35] + coeff0_22*dmats2[22][35] + coeff0_23*dmats2[23][35] + coeff0_24*dmats2[24][35] + coeff0_25*dmats2[25][35] + coeff0_26*dmats2[26][35] + coeff0_27*dmats2[27][35] + coeff0_28*dmats2[28][35] + coeff0_29*dmats2[29][35] + coeff0_30*dmats2[30][35] + coeff0_31*dmats2[31][35] + coeff0_32*dmats2[32][35] + coeff0_33*dmats2[33][35] + coeff0_34*dmats2[34][35] + coeff0_35*dmats2[35][35] + coeff0_36*dmats2[36][35] + coeff0_37*dmats2[37][35] + coeff0_38*dmats2[38][35] + coeff0_39*dmats2[39][35] + coeff0_40*dmats2[40][35] + coeff0_41*dmats2[41][35] + coeff0_42*dmats2[42][35] + coeff0_43*dmats2[43][35] + coeff0_44*dmats2[44][35] + coeff0_45*dmats2[45][35] + coeff0_46*dmats2[46][35] + coeff0_47*dmats2[47][35] + coeff0_48*dmats2[48][35] + coeff0_49*dmats2[49][35] + coeff0_50*dmats2[50][35] + coeff0_51*dmats2[51][35] + coeff0_52*dmats2[52][35] + coeff0_53*dmats2[53][35] + coeff0_54*dmats2[54][35] + coeff0_55*dmats2[55][35];
          new_coeff0_36 = coeff0_0*dmats2[0][36] + coeff0_1*dmats2[1][36] + coeff0_2*dmats2[2][36] + coeff0_3*dmats2[3][36] + coeff0_4*dmats2[4][36] + coeff0_5*dmats2[5][36] + coeff0_6*dmats2[6][36] + coeff0_7*dmats2[7][36] + coeff0_8*dmats2[8][36] + coeff0_9*dmats2[9][36] + coeff0_10*dmats2[10][36] + coeff0_11*dmats2[11][36] + coeff0_12*dmats2[12][36] + coeff0_13*dmats2[13][36] + coeff0_14*dmats2[14][36] + coeff0_15*dmats2[15][36] + coeff0_16*dmats2[16][36] + coeff0_17*dmats2[17][36] + coeff0_18*dmats2[18][36] + coeff0_19*dmats2[19][36] + coeff0_20*dmats2[20][36] + coeff0_21*dmats2[21][36] + coeff0_22*dmats2[22][36] + coeff0_23*dmats2[23][36] + coeff0_24*dmats2[24][36] + coeff0_25*dmats2[25][36] + coeff0_26*dmats2[26][36] + coeff0_27*dmats2[27][36] + coeff0_28*dmats2[28][36] + coeff0_29*dmats2[29][36] + coeff0_30*dmats2[30][36] + coeff0_31*dmats2[31][36] + coeff0_32*dmats2[32][36] + coeff0_33*dmats2[33][36] + coeff0_34*dmats2[34][36] + coeff0_35*dmats2[35][36] + coeff0_36*dmats2[36][36] + coeff0_37*dmats2[37][36] + coeff0_38*dmats2[38][36] + coeff0_39*dmats2[39][36] + coeff0_40*dmats2[40][36] + coeff0_41*dmats2[41][36] + coeff0_42*dmats2[42][36] + coeff0_43*dmats2[43][36] + coeff0_44*dmats2[44][36] + coeff0_45*dmats2[45][36] + coeff0_46*dmats2[46][36] + coeff0_47*dmats2[47][36] + coeff0_48*dmats2[48][36] + coeff0_49*dmats2[49][36] + coeff0_50*dmats2[50][36] + coeff0_51*dmats2[51][36] + coeff0_52*dmats2[52][36] + coeff0_53*dmats2[53][36] + coeff0_54*dmats2[54][36] + coeff0_55*dmats2[55][36];
          new_coeff0_37 = coeff0_0*dmats2[0][37] + coeff0_1*dmats2[1][37] + coeff0_2*dmats2[2][37] + coeff0_3*dmats2[3][37] + coeff0_4*dmats2[4][37] + coeff0_5*dmats2[5][37] + coeff0_6*dmats2[6][37] + coeff0_7*dmats2[7][37] + coeff0_8*dmats2[8][37] + coeff0_9*dmats2[9][37] + coeff0_10*dmats2[10][37] + coeff0_11*dmats2[11][37] + coeff0_12*dmats2[12][37] + coeff0_13*dmats2[13][37] + coeff0_14*dmats2[14][37] + coeff0_15*dmats2[15][37] + coeff0_16*dmats2[16][37] + coeff0_17*dmats2[17][37] + coeff0_18*dmats2[18][37] + coeff0_19*dmats2[19][37] + coeff0_20*dmats2[20][37] + coeff0_21*dmats2[21][37] + coeff0_22*dmats2[22][37] + coeff0_23*dmats2[23][37] + coeff0_24*dmats2[24][37] + coeff0_25*dmats2[25][37] + coeff0_26*dmats2[26][37] + coeff0_27*dmats2[27][37] + coeff0_28*dmats2[28][37] + coeff0_29*dmats2[29][37] + coeff0_30*dmats2[30][37] + coeff0_31*dmats2[31][37] + coeff0_32*dmats2[32][37] + coeff0_33*dmats2[33][37] + coeff0_34*dmats2[34][37] + coeff0_35*dmats2[35][37] + coeff0_36*dmats2[36][37] + coeff0_37*dmats2[37][37] + coeff0_38*dmats2[38][37] + coeff0_39*dmats2[39][37] + coeff0_40*dmats2[40][37] + coeff0_41*dmats2[41][37] + coeff0_42*dmats2[42][37] + coeff0_43*dmats2[43][37] + coeff0_44*dmats2[44][37] + coeff0_45*dmats2[45][37] + coeff0_46*dmats2[46][37] + coeff0_47*dmats2[47][37] + coeff0_48*dmats2[48][37] + coeff0_49*dmats2[49][37] + coeff0_50*dmats2[50][37] + coeff0_51*dmats2[51][37] + coeff0_52*dmats2[52][37] + coeff0_53*dmats2[53][37] + coeff0_54*dmats2[54][37] + coeff0_55*dmats2[55][37];
          new_coeff0_38 = coeff0_0*dmats2[0][38] + coeff0_1*dmats2[1][38] + coeff0_2*dmats2[2][38] + coeff0_3*dmats2[3][38] + coeff0_4*dmats2[4][38] + coeff0_5*dmats2[5][38] + coeff0_6*dmats2[6][38] + coeff0_7*dmats2[7][38] + coeff0_8*dmats2[8][38] + coeff0_9*dmats2[9][38] + coeff0_10*dmats2[10][38] + coeff0_11*dmats2[11][38] + coeff0_12*dmats2[12][38] + coeff0_13*dmats2[13][38] + coeff0_14*dmats2[14][38] + coeff0_15*dmats2[15][38] + coeff0_16*dmats2[16][38] + coeff0_17*dmats2[17][38] + coeff0_18*dmats2[18][38] + coeff0_19*dmats2[19][38] + coeff0_20*dmats2[20][38] + coeff0_21*dmats2[21][38] + coeff0_22*dmats2[22][38] + coeff0_23*dmats2[23][38] + coeff0_24*dmats2[24][38] + coeff0_25*dmats2[25][38] + coeff0_26*dmats2[26][38] + coeff0_27*dmats2[27][38] + coeff0_28*dmats2[28][38] + coeff0_29*dmats2[29][38] + coeff0_30*dmats2[30][38] + coeff0_31*dmats2[31][38] + coeff0_32*dmats2[32][38] + coeff0_33*dmats2[33][38] + coeff0_34*dmats2[34][38] + coeff0_35*dmats2[35][38] + coeff0_36*dmats2[36][38] + coeff0_37*dmats2[37][38] + coeff0_38*dmats2[38][38] + coeff0_39*dmats2[39][38] + coeff0_40*dmats2[40][38] + coeff0_41*dmats2[41][38] + coeff0_42*dmats2[42][38] + coeff0_43*dmats2[43][38] + coeff0_44*dmats2[44][38] + coeff0_45*dmats2[45][38] + coeff0_46*dmats2[46][38] + coeff0_47*dmats2[47][38] + coeff0_48*dmats2[48][38] + coeff0_49*dmats2[49][38] + coeff0_50*dmats2[50][38] + coeff0_51*dmats2[51][38] + coeff0_52*dmats2[52][38] + coeff0_53*dmats2[53][38] + coeff0_54*dmats2[54][38] + coeff0_55*dmats2[55][38];
          new_coeff0_39 = coeff0_0*dmats2[0][39] + coeff0_1*dmats2[1][39] + coeff0_2*dmats2[2][39] + coeff0_3*dmats2[3][39] + coeff0_4*dmats2[4][39] + coeff0_5*dmats2[5][39] + coeff0_6*dmats2[6][39] + coeff0_7*dmats2[7][39] + coeff0_8*dmats2[8][39] + coeff0_9*dmats2[9][39] + coeff0_10*dmats2[10][39] + coeff0_11*dmats2[11][39] + coeff0_12*dmats2[12][39] + coeff0_13*dmats2[13][39] + coeff0_14*dmats2[14][39] + coeff0_15*dmats2[15][39] + coeff0_16*dmats2[16][39] + coeff0_17*dmats2[17][39] + coeff0_18*dmats2[18][39] + coeff0_19*dmats2[19][39] + coeff0_20*dmats2[20][39] + coeff0_21*dmats2[21][39] + coeff0_22*dmats2[22][39] + coeff0_23*dmats2[23][39] + coeff0_24*dmats2[24][39] + coeff0_25*dmats2[25][39] + coeff0_26*dmats2[26][39] + coeff0_27*dmats2[27][39] + coeff0_28*dmats2[28][39] + coeff0_29*dmats2[29][39] + coeff0_30*dmats2[30][39] + coeff0_31*dmats2[31][39] + coeff0_32*dmats2[32][39] + coeff0_33*dmats2[33][39] + coeff0_34*dmats2[34][39] + coeff0_35*dmats2[35][39] + coeff0_36*dmats2[36][39] + coeff0_37*dmats2[37][39] + coeff0_38*dmats2[38][39] + coeff0_39*dmats2[39][39] + coeff0_40*dmats2[40][39] + coeff0_41*dmats2[41][39] + coeff0_42*dmats2[42][39] + coeff0_43*dmats2[43][39] + coeff0_44*dmats2[44][39] + coeff0_45*dmats2[45][39] + coeff0_46*dmats2[46][39] + coeff0_47*dmats2[47][39] + coeff0_48*dmats2[48][39] + coeff0_49*dmats2[49][39] + coeff0_50*dmats2[50][39] + coeff0_51*dmats2[51][39] + coeff0_52*dmats2[52][39] + coeff0_53*dmats2[53][39] + coeff0_54*dmats2[54][39] + coeff0_55*dmats2[55][39];
          new_coeff0_40 = coeff0_0*dmats2[0][40] + coeff0_1*dmats2[1][40] + coeff0_2*dmats2[2][40] + coeff0_3*dmats2[3][40] + coeff0_4*dmats2[4][40] + coeff0_5*dmats2[5][40] + coeff0_6*dmats2[6][40] + coeff0_7*dmats2[7][40] + coeff0_8*dmats2[8][40] + coeff0_9*dmats2[9][40] + coeff0_10*dmats2[10][40] + coeff0_11*dmats2[11][40] + coeff0_12*dmats2[12][40] + coeff0_13*dmats2[13][40] + coeff0_14*dmats2[14][40] + coeff0_15*dmats2[15][40] + coeff0_16*dmats2[16][40] + coeff0_17*dmats2[17][40] + coeff0_18*dmats2[18][40] + coeff0_19*dmats2[19][40] + coeff0_20*dmats2[20][40] + coeff0_21*dmats2[21][40] + coeff0_22*dmats2[22][40] + coeff0_23*dmats2[23][40] + coeff0_24*dmats2[24][40] + coeff0_25*dmats2[25][40] + coeff0_26*dmats2[26][40] + coeff0_27*dmats2[27][40] + coeff0_28*dmats2[28][40] + coeff0_29*dmats2[29][40] + coeff0_30*dmats2[30][40] + coeff0_31*dmats2[31][40] + coeff0_32*dmats2[32][40] + coeff0_33*dmats2[33][40] + coeff0_34*dmats2[34][40] + coeff0_35*dmats2[35][40] + coeff0_36*dmats2[36][40] + coeff0_37*dmats2[37][40] + coeff0_38*dmats2[38][40] + coeff0_39*dmats2[39][40] + coeff0_40*dmats2[40][40] + coeff0_41*dmats2[41][40] + coeff0_42*dmats2[42][40] + coeff0_43*dmats2[43][40] + coeff0_44*dmats2[44][40] + coeff0_45*dmats2[45][40] + coeff0_46*dmats2[46][40] + coeff0_47*dmats2[47][40] + coeff0_48*dmats2[48][40] + coeff0_49*dmats2[49][40] + coeff0_50*dmats2[50][40] + coeff0_51*dmats2[51][40] + coeff0_52*dmats2[52][40] + coeff0_53*dmats2[53][40] + coeff0_54*dmats2[54][40] + coeff0_55*dmats2[55][40];
          new_coeff0_41 = coeff0_0*dmats2[0][41] + coeff0_1*dmats2[1][41] + coeff0_2*dmats2[2][41] + coeff0_3*dmats2[3][41] + coeff0_4*dmats2[4][41] + coeff0_5*dmats2[5][41] + coeff0_6*dmats2[6][41] + coeff0_7*dmats2[7][41] + coeff0_8*dmats2[8][41] + coeff0_9*dmats2[9][41] + coeff0_10*dmats2[10][41] + coeff0_11*dmats2[11][41] + coeff0_12*dmats2[12][41] + coeff0_13*dmats2[13][41] + coeff0_14*dmats2[14][41] + coeff0_15*dmats2[15][41] + coeff0_16*dmats2[16][41] + coeff0_17*dmats2[17][41] + coeff0_18*dmats2[18][41] + coeff0_19*dmats2[19][41] + coeff0_20*dmats2[20][41] + coeff0_21*dmats2[21][41] + coeff0_22*dmats2[22][41] + coeff0_23*dmats2[23][41] + coeff0_24*dmats2[24][41] + coeff0_25*dmats2[25][41] + coeff0_26*dmats2[26][41] + coeff0_27*dmats2[27][41] + coeff0_28*dmats2[28][41] + coeff0_29*dmats2[29][41] + coeff0_30*dmats2[30][41] + coeff0_31*dmats2[31][41] + coeff0_32*dmats2[32][41] + coeff0_33*dmats2[33][41] + coeff0_34*dmats2[34][41] + coeff0_35*dmats2[35][41] + coeff0_36*dmats2[36][41] + coeff0_37*dmats2[37][41] + coeff0_38*dmats2[38][41] + coeff0_39*dmats2[39][41] + coeff0_40*dmats2[40][41] + coeff0_41*dmats2[41][41] + coeff0_42*dmats2[42][41] + coeff0_43*dmats2[43][41] + coeff0_44*dmats2[44][41] + coeff0_45*dmats2[45][41] + coeff0_46*dmats2[46][41] + coeff0_47*dmats2[47][41] + coeff0_48*dmats2[48][41] + coeff0_49*dmats2[49][41] + coeff0_50*dmats2[50][41] + coeff0_51*dmats2[51][41] + coeff0_52*dmats2[52][41] + coeff0_53*dmats2[53][41] + coeff0_54*dmats2[54][41] + coeff0_55*dmats2[55][41];
          new_coeff0_42 = coeff0_0*dmats2[0][42] + coeff0_1*dmats2[1][42] + coeff0_2*dmats2[2][42] + coeff0_3*dmats2[3][42] + coeff0_4*dmats2[4][42] + coeff0_5*dmats2[5][42] + coeff0_6*dmats2[6][42] + coeff0_7*dmats2[7][42] + coeff0_8*dmats2[8][42] + coeff0_9*dmats2[9][42] + coeff0_10*dmats2[10][42] + coeff0_11*dmats2[11][42] + coeff0_12*dmats2[12][42] + coeff0_13*dmats2[13][42] + coeff0_14*dmats2[14][42] + coeff0_15*dmats2[15][42] + coeff0_16*dmats2[16][42] + coeff0_17*dmats2[17][42] + coeff0_18*dmats2[18][42] + coeff0_19*dmats2[19][42] + coeff0_20*dmats2[20][42] + coeff0_21*dmats2[21][42] + coeff0_22*dmats2[22][42] + coeff0_23*dmats2[23][42] + coeff0_24*dmats2[24][42] + coeff0_25*dmats2[25][42] + coeff0_26*dmats2[26][42] + coeff0_27*dmats2[27][42] + coeff0_28*dmats2[28][42] + coeff0_29*dmats2[29][42] + coeff0_30*dmats2[30][42] + coeff0_31*dmats2[31][42] + coeff0_32*dmats2[32][42] + coeff0_33*dmats2[33][42] + coeff0_34*dmats2[34][42] + coeff0_35*dmats2[35][42] + coeff0_36*dmats2[36][42] + coeff0_37*dmats2[37][42] + coeff0_38*dmats2[38][42] + coeff0_39*dmats2[39][42] + coeff0_40*dmats2[40][42] + coeff0_41*dmats2[41][42] + coeff0_42*dmats2[42][42] + coeff0_43*dmats2[43][42] + coeff0_44*dmats2[44][42] + coeff0_45*dmats2[45][42] + coeff0_46*dmats2[46][42] + coeff0_47*dmats2[47][42] + coeff0_48*dmats2[48][42] + coeff0_49*dmats2[49][42] + coeff0_50*dmats2[50][42] + coeff0_51*dmats2[51][42] + coeff0_52*dmats2[52][42] + coeff0_53*dmats2[53][42] + coeff0_54*dmats2[54][42] + coeff0_55*dmats2[55][42];
          new_coeff0_43 = coeff0_0*dmats2[0][43] + coeff0_1*dmats2[1][43] + coeff0_2*dmats2[2][43] + coeff0_3*dmats2[3][43] + coeff0_4*dmats2[4][43] + coeff0_5*dmats2[5][43] + coeff0_6*dmats2[6][43] + coeff0_7*dmats2[7][43] + coeff0_8*dmats2[8][43] + coeff0_9*dmats2[9][43] + coeff0_10*dmats2[10][43] + coeff0_11*dmats2[11][43] + coeff0_12*dmats2[12][43] + coeff0_13*dmats2[13][43] + coeff0_14*dmats2[14][43] + coeff0_15*dmats2[15][43] + coeff0_16*dmats2[16][43] + coeff0_17*dmats2[17][43] + coeff0_18*dmats2[18][43] + coeff0_19*dmats2[19][43] + coeff0_20*dmats2[20][43] + coeff0_21*dmats2[21][43] + coeff0_22*dmats2[22][43] + coeff0_23*dmats2[23][43] + coeff0_24*dmats2[24][43] + coeff0_25*dmats2[25][43] + coeff0_26*dmats2[26][43] + coeff0_27*dmats2[27][43] + coeff0_28*dmats2[28][43] + coeff0_29*dmats2[29][43] + coeff0_30*dmats2[30][43] + coeff0_31*dmats2[31][43] + coeff0_32*dmats2[32][43] + coeff0_33*dmats2[33][43] + coeff0_34*dmats2[34][43] + coeff0_35*dmats2[35][43] + coeff0_36*dmats2[36][43] + coeff0_37*dmats2[37][43] + coeff0_38*dmats2[38][43] + coeff0_39*dmats2[39][43] + coeff0_40*dmats2[40][43] + coeff0_41*dmats2[41][43] + coeff0_42*dmats2[42][43] + coeff0_43*dmats2[43][43] + coeff0_44*dmats2[44][43] + coeff0_45*dmats2[45][43] + coeff0_46*dmats2[46][43] + coeff0_47*dmats2[47][43] + coeff0_48*dmats2[48][43] + coeff0_49*dmats2[49][43] + coeff0_50*dmats2[50][43] + coeff0_51*dmats2[51][43] + coeff0_52*dmats2[52][43] + coeff0_53*dmats2[53][43] + coeff0_54*dmats2[54][43] + coeff0_55*dmats2[55][43];
          new_coeff0_44 = coeff0_0*dmats2[0][44] + coeff0_1*dmats2[1][44] + coeff0_2*dmats2[2][44] + coeff0_3*dmats2[3][44] + coeff0_4*dmats2[4][44] + coeff0_5*dmats2[5][44] + coeff0_6*dmats2[6][44] + coeff0_7*dmats2[7][44] + coeff0_8*dmats2[8][44] + coeff0_9*dmats2[9][44] + coeff0_10*dmats2[10][44] + coeff0_11*dmats2[11][44] + coeff0_12*dmats2[12][44] + coeff0_13*dmats2[13][44] + coeff0_14*dmats2[14][44] + coeff0_15*dmats2[15][44] + coeff0_16*dmats2[16][44] + coeff0_17*dmats2[17][44] + coeff0_18*dmats2[18][44] + coeff0_19*dmats2[19][44] + coeff0_20*dmats2[20][44] + coeff0_21*dmats2[21][44] + coeff0_22*dmats2[22][44] + coeff0_23*dmats2[23][44] + coeff0_24*dmats2[24][44] + coeff0_25*dmats2[25][44] + coeff0_26*dmats2[26][44] + coeff0_27*dmats2[27][44] + coeff0_28*dmats2[28][44] + coeff0_29*dmats2[29][44] + coeff0_30*dmats2[30][44] + coeff0_31*dmats2[31][44] + coeff0_32*dmats2[32][44] + coeff0_33*dmats2[33][44] + coeff0_34*dmats2[34][44] + coeff0_35*dmats2[35][44] + coeff0_36*dmats2[36][44] + coeff0_37*dmats2[37][44] + coeff0_38*dmats2[38][44] + coeff0_39*dmats2[39][44] + coeff0_40*dmats2[40][44] + coeff0_41*dmats2[41][44] + coeff0_42*dmats2[42][44] + coeff0_43*dmats2[43][44] + coeff0_44*dmats2[44][44] + coeff0_45*dmats2[45][44] + coeff0_46*dmats2[46][44] + coeff0_47*dmats2[47][44] + coeff0_48*dmats2[48][44] + coeff0_49*dmats2[49][44] + coeff0_50*dmats2[50][44] + coeff0_51*dmats2[51][44] + coeff0_52*dmats2[52][44] + coeff0_53*dmats2[53][44] + coeff0_54*dmats2[54][44] + coeff0_55*dmats2[55][44];
          new_coeff0_45 = coeff0_0*dmats2[0][45] + coeff0_1*dmats2[1][45] + coeff0_2*dmats2[2][45] + coeff0_3*dmats2[3][45] + coeff0_4*dmats2[4][45] + coeff0_5*dmats2[5][45] + coeff0_6*dmats2[6][45] + coeff0_7*dmats2[7][45] + coeff0_8*dmats2[8][45] + coeff0_9*dmats2[9][45] + coeff0_10*dmats2[10][45] + coeff0_11*dmats2[11][45] + coeff0_12*dmats2[12][45] + coeff0_13*dmats2[13][45] + coeff0_14*dmats2[14][45] + coeff0_15*dmats2[15][45] + coeff0_16*dmats2[16][45] + coeff0_17*dmats2[17][45] + coeff0_18*dmats2[18][45] + coeff0_19*dmats2[19][45] + coeff0_20*dmats2[20][45] + coeff0_21*dmats2[21][45] + coeff0_22*dmats2[22][45] + coeff0_23*dmats2[23][45] + coeff0_24*dmats2[24][45] + coeff0_25*dmats2[25][45] + coeff0_26*dmats2[26][45] + coeff0_27*dmats2[27][45] + coeff0_28*dmats2[28][45] + coeff0_29*dmats2[29][45] + coeff0_30*dmats2[30][45] + coeff0_31*dmats2[31][45] + coeff0_32*dmats2[32][45] + coeff0_33*dmats2[33][45] + coeff0_34*dmats2[34][45] + coeff0_35*dmats2[35][45] + coeff0_36*dmats2[36][45] + coeff0_37*dmats2[37][45] + coeff0_38*dmats2[38][45] + coeff0_39*dmats2[39][45] + coeff0_40*dmats2[40][45] + coeff0_41*dmats2[41][45] + coeff0_42*dmats2[42][45] + coeff0_43*dmats2[43][45] + coeff0_44*dmats2[44][45] + coeff0_45*dmats2[45][45] + coeff0_46*dmats2[46][45] + coeff0_47*dmats2[47][45] + coeff0_48*dmats2[48][45] + coeff0_49*dmats2[49][45] + coeff0_50*dmats2[50][45] + coeff0_51*dmats2[51][45] + coeff0_52*dmats2[52][45] + coeff0_53*dmats2[53][45] + coeff0_54*dmats2[54][45] + coeff0_55*dmats2[55][45];
          new_coeff0_46 = coeff0_0*dmats2[0][46] + coeff0_1*dmats2[1][46] + coeff0_2*dmats2[2][46] + coeff0_3*dmats2[3][46] + coeff0_4*dmats2[4][46] + coeff0_5*dmats2[5][46] + coeff0_6*dmats2[6][46] + coeff0_7*dmats2[7][46] + coeff0_8*dmats2[8][46] + coeff0_9*dmats2[9][46] + coeff0_10*dmats2[10][46] + coeff0_11*dmats2[11][46] + coeff0_12*dmats2[12][46] + coeff0_13*dmats2[13][46] + coeff0_14*dmats2[14][46] + coeff0_15*dmats2[15][46] + coeff0_16*dmats2[16][46] + coeff0_17*dmats2[17][46] + coeff0_18*dmats2[18][46] + coeff0_19*dmats2[19][46] + coeff0_20*dmats2[20][46] + coeff0_21*dmats2[21][46] + coeff0_22*dmats2[22][46] + coeff0_23*dmats2[23][46] + coeff0_24*dmats2[24][46] + coeff0_25*dmats2[25][46] + coeff0_26*dmats2[26][46] + coeff0_27*dmats2[27][46] + coeff0_28*dmats2[28][46] + coeff0_29*dmats2[29][46] + coeff0_30*dmats2[30][46] + coeff0_31*dmats2[31][46] + coeff0_32*dmats2[32][46] + coeff0_33*dmats2[33][46] + coeff0_34*dmats2[34][46] + coeff0_35*dmats2[35][46] + coeff0_36*dmats2[36][46] + coeff0_37*dmats2[37][46] + coeff0_38*dmats2[38][46] + coeff0_39*dmats2[39][46] + coeff0_40*dmats2[40][46] + coeff0_41*dmats2[41][46] + coeff0_42*dmats2[42][46] + coeff0_43*dmats2[43][46] + coeff0_44*dmats2[44][46] + coeff0_45*dmats2[45][46] + coeff0_46*dmats2[46][46] + coeff0_47*dmats2[47][46] + coeff0_48*dmats2[48][46] + coeff0_49*dmats2[49][46] + coeff0_50*dmats2[50][46] + coeff0_51*dmats2[51][46] + coeff0_52*dmats2[52][46] + coeff0_53*dmats2[53][46] + coeff0_54*dmats2[54][46] + coeff0_55*dmats2[55][46];
          new_coeff0_47 = coeff0_0*dmats2[0][47] + coeff0_1*dmats2[1][47] + coeff0_2*dmats2[2][47] + coeff0_3*dmats2[3][47] + coeff0_4*dmats2[4][47] + coeff0_5*dmats2[5][47] + coeff0_6*dmats2[6][47] + coeff0_7*dmats2[7][47] + coeff0_8*dmats2[8][47] + coeff0_9*dmats2[9][47] + coeff0_10*dmats2[10][47] + coeff0_11*dmats2[11][47] + coeff0_12*dmats2[12][47] + coeff0_13*dmats2[13][47] + coeff0_14*dmats2[14][47] + coeff0_15*dmats2[15][47] + coeff0_16*dmats2[16][47] + coeff0_17*dmats2[17][47] + coeff0_18*dmats2[18][47] + coeff0_19*dmats2[19][47] + coeff0_20*dmats2[20][47] + coeff0_21*dmats2[21][47] + coeff0_22*dmats2[22][47] + coeff0_23*dmats2[23][47] + coeff0_24*dmats2[24][47] + coeff0_25*dmats2[25][47] + coeff0_26*dmats2[26][47] + coeff0_27*dmats2[27][47] + coeff0_28*dmats2[28][47] + coeff0_29*dmats2[29][47] + coeff0_30*dmats2[30][47] + coeff0_31*dmats2[31][47] + coeff0_32*dmats2[32][47] + coeff0_33*dmats2[33][47] + coeff0_34*dmats2[34][47] + coeff0_35*dmats2[35][47] + coeff0_36*dmats2[36][47] + coeff0_37*dmats2[37][47] + coeff0_38*dmats2[38][47] + coeff0_39*dmats2[39][47] + coeff0_40*dmats2[40][47] + coeff0_41*dmats2[41][47] + coeff0_42*dmats2[42][47] + coeff0_43*dmats2[43][47] + coeff0_44*dmats2[44][47] + coeff0_45*dmats2[45][47] + coeff0_46*dmats2[46][47] + coeff0_47*dmats2[47][47] + coeff0_48*dmats2[48][47] + coeff0_49*dmats2[49][47] + coeff0_50*dmats2[50][47] + coeff0_51*dmats2[51][47] + coeff0_52*dmats2[52][47] + coeff0_53*dmats2[53][47] + coeff0_54*dmats2[54][47] + coeff0_55*dmats2[55][47];
          new_coeff0_48 = coeff0_0*dmats2[0][48] + coeff0_1*dmats2[1][48] + coeff0_2*dmats2[2][48] + coeff0_3*dmats2[3][48] + coeff0_4*dmats2[4][48] + coeff0_5*dmats2[5][48] + coeff0_6*dmats2[6][48] + coeff0_7*dmats2[7][48] + coeff0_8*dmats2[8][48] + coeff0_9*dmats2[9][48] + coeff0_10*dmats2[10][48] + coeff0_11*dmats2[11][48] + coeff0_12*dmats2[12][48] + coeff0_13*dmats2[13][48] + coeff0_14*dmats2[14][48] + coeff0_15*dmats2[15][48] + coeff0_16*dmats2[16][48] + coeff0_17*dmats2[17][48] + coeff0_18*dmats2[18][48] + coeff0_19*dmats2[19][48] + coeff0_20*dmats2[20][48] + coeff0_21*dmats2[21][48] + coeff0_22*dmats2[22][48] + coeff0_23*dmats2[23][48] + coeff0_24*dmats2[24][48] + coeff0_25*dmats2[25][48] + coeff0_26*dmats2[26][48] + coeff0_27*dmats2[27][48] + coeff0_28*dmats2[28][48] + coeff0_29*dmats2[29][48] + coeff0_30*dmats2[30][48] + coeff0_31*dmats2[31][48] + coeff0_32*dmats2[32][48] + coeff0_33*dmats2[33][48] + coeff0_34*dmats2[34][48] + coeff0_35*dmats2[35][48] + coeff0_36*dmats2[36][48] + coeff0_37*dmats2[37][48] + coeff0_38*dmats2[38][48] + coeff0_39*dmats2[39][48] + coeff0_40*dmats2[40][48] + coeff0_41*dmats2[41][48] + coeff0_42*dmats2[42][48] + coeff0_43*dmats2[43][48] + coeff0_44*dmats2[44][48] + coeff0_45*dmats2[45][48] + coeff0_46*dmats2[46][48] + coeff0_47*dmats2[47][48] + coeff0_48*dmats2[48][48] + coeff0_49*dmats2[49][48] + coeff0_50*dmats2[50][48] + coeff0_51*dmats2[51][48] + coeff0_52*dmats2[52][48] + coeff0_53*dmats2[53][48] + coeff0_54*dmats2[54][48] + coeff0_55*dmats2[55][48];
          new_coeff0_49 = coeff0_0*dmats2[0][49] + coeff0_1*dmats2[1][49] + coeff0_2*dmats2[2][49] + coeff0_3*dmats2[3][49] + coeff0_4*dmats2[4][49] + coeff0_5*dmats2[5][49] + coeff0_6*dmats2[6][49] + coeff0_7*dmats2[7][49] + coeff0_8*dmats2[8][49] + coeff0_9*dmats2[9][49] + coeff0_10*dmats2[10][49] + coeff0_11*dmats2[11][49] + coeff0_12*dmats2[12][49] + coeff0_13*dmats2[13][49] + coeff0_14*dmats2[14][49] + coeff0_15*dmats2[15][49] + coeff0_16*dmats2[16][49] + coeff0_17*dmats2[17][49] + coeff0_18*dmats2[18][49] + coeff0_19*dmats2[19][49] + coeff0_20*dmats2[20][49] + coeff0_21*dmats2[21][49] + coeff0_22*dmats2[22][49] + coeff0_23*dmats2[23][49] + coeff0_24*dmats2[24][49] + coeff0_25*dmats2[25][49] + coeff0_26*dmats2[26][49] + coeff0_27*dmats2[27][49] + coeff0_28*dmats2[28][49] + coeff0_29*dmats2[29][49] + coeff0_30*dmats2[30][49] + coeff0_31*dmats2[31][49] + coeff0_32*dmats2[32][49] + coeff0_33*dmats2[33][49] + coeff0_34*dmats2[34][49] + coeff0_35*dmats2[35][49] + coeff0_36*dmats2[36][49] + coeff0_37*dmats2[37][49] + coeff0_38*dmats2[38][49] + coeff0_39*dmats2[39][49] + coeff0_40*dmats2[40][49] + coeff0_41*dmats2[41][49] + coeff0_42*dmats2[42][49] + coeff0_43*dmats2[43][49] + coeff0_44*dmats2[44][49] + coeff0_45*dmats2[45][49] + coeff0_46*dmats2[46][49] + coeff0_47*dmats2[47][49] + coeff0_48*dmats2[48][49] + coeff0_49*dmats2[49][49] + coeff0_50*dmats2[50][49] + coeff0_51*dmats2[51][49] + coeff0_52*dmats2[52][49] + coeff0_53*dmats2[53][49] + coeff0_54*dmats2[54][49] + coeff0_55*dmats2[55][49];
          new_coeff0_50 = coeff0_0*dmats2[0][50] + coeff0_1*dmats2[1][50] + coeff0_2*dmats2[2][50] + coeff0_3*dmats2[3][50] + coeff0_4*dmats2[4][50] + coeff0_5*dmats2[5][50] + coeff0_6*dmats2[6][50] + coeff0_7*dmats2[7][50] + coeff0_8*dmats2[8][50] + coeff0_9*dmats2[9][50] + coeff0_10*dmats2[10][50] + coeff0_11*dmats2[11][50] + coeff0_12*dmats2[12][50] + coeff0_13*dmats2[13][50] + coeff0_14*dmats2[14][50] + coeff0_15*dmats2[15][50] + coeff0_16*dmats2[16][50] + coeff0_17*dmats2[17][50] + coeff0_18*dmats2[18][50] + coeff0_19*dmats2[19][50] + coeff0_20*dmats2[20][50] + coeff0_21*dmats2[21][50] + coeff0_22*dmats2[22][50] + coeff0_23*dmats2[23][50] + coeff0_24*dmats2[24][50] + coeff0_25*dmats2[25][50] + coeff0_26*dmats2[26][50] + coeff0_27*dmats2[27][50] + coeff0_28*dmats2[28][50] + coeff0_29*dmats2[29][50] + coeff0_30*dmats2[30][50] + coeff0_31*dmats2[31][50] + coeff0_32*dmats2[32][50] + coeff0_33*dmats2[33][50] + coeff0_34*dmats2[34][50] + coeff0_35*dmats2[35][50] + coeff0_36*dmats2[36][50] + coeff0_37*dmats2[37][50] + coeff0_38*dmats2[38][50] + coeff0_39*dmats2[39][50] + coeff0_40*dmats2[40][50] + coeff0_41*dmats2[41][50] + coeff0_42*dmats2[42][50] + coeff0_43*dmats2[43][50] + coeff0_44*dmats2[44][50] + coeff0_45*dmats2[45][50] + coeff0_46*dmats2[46][50] + coeff0_47*dmats2[47][50] + coeff0_48*dmats2[48][50] + coeff0_49*dmats2[49][50] + coeff0_50*dmats2[50][50] + coeff0_51*dmats2[51][50] + coeff0_52*dmats2[52][50] + coeff0_53*dmats2[53][50] + coeff0_54*dmats2[54][50] + coeff0_55*dmats2[55][50];
          new_coeff0_51 = coeff0_0*dmats2[0][51] + coeff0_1*dmats2[1][51] + coeff0_2*dmats2[2][51] + coeff0_3*dmats2[3][51] + coeff0_4*dmats2[4][51] + coeff0_5*dmats2[5][51] + coeff0_6*dmats2[6][51] + coeff0_7*dmats2[7][51] + coeff0_8*dmats2[8][51] + coeff0_9*dmats2[9][51] + coeff0_10*dmats2[10][51] + coeff0_11*dmats2[11][51] + coeff0_12*dmats2[12][51] + coeff0_13*dmats2[13][51] + coeff0_14*dmats2[14][51] + coeff0_15*dmats2[15][51] + coeff0_16*dmats2[16][51] + coeff0_17*dmats2[17][51] + coeff0_18*dmats2[18][51] + coeff0_19*dmats2[19][51] + coeff0_20*dmats2[20][51] + coeff0_21*dmats2[21][51] + coeff0_22*dmats2[22][51] + coeff0_23*dmats2[23][51] + coeff0_24*dmats2[24][51] + coeff0_25*dmats2[25][51] + coeff0_26*dmats2[26][51] + coeff0_27*dmats2[27][51] + coeff0_28*dmats2[28][51] + coeff0_29*dmats2[29][51] + coeff0_30*dmats2[30][51] + coeff0_31*dmats2[31][51] + coeff0_32*dmats2[32][51] + coeff0_33*dmats2[33][51] + coeff0_34*dmats2[34][51] + coeff0_35*dmats2[35][51] + coeff0_36*dmats2[36][51] + coeff0_37*dmats2[37][51] + coeff0_38*dmats2[38][51] + coeff0_39*dmats2[39][51] + coeff0_40*dmats2[40][51] + coeff0_41*dmats2[41][51] + coeff0_42*dmats2[42][51] + coeff0_43*dmats2[43][51] + coeff0_44*dmats2[44][51] + coeff0_45*dmats2[45][51] + coeff0_46*dmats2[46][51] + coeff0_47*dmats2[47][51] + coeff0_48*dmats2[48][51] + coeff0_49*dmats2[49][51] + coeff0_50*dmats2[50][51] + coeff0_51*dmats2[51][51] + coeff0_52*dmats2[52][51] + coeff0_53*dmats2[53][51] + coeff0_54*dmats2[54][51] + coeff0_55*dmats2[55][51];
          new_coeff0_52 = coeff0_0*dmats2[0][52] + coeff0_1*dmats2[1][52] + coeff0_2*dmats2[2][52] + coeff0_3*dmats2[3][52] + coeff0_4*dmats2[4][52] + coeff0_5*dmats2[5][52] + coeff0_6*dmats2[6][52] + coeff0_7*dmats2[7][52] + coeff0_8*dmats2[8][52] + coeff0_9*dmats2[9][52] + coeff0_10*dmats2[10][52] + coeff0_11*dmats2[11][52] + coeff0_12*dmats2[12][52] + coeff0_13*dmats2[13][52] + coeff0_14*dmats2[14][52] + coeff0_15*dmats2[15][52] + coeff0_16*dmats2[16][52] + coeff0_17*dmats2[17][52] + coeff0_18*dmats2[18][52] + coeff0_19*dmats2[19][52] + coeff0_20*dmats2[20][52] + coeff0_21*dmats2[21][52] + coeff0_22*dmats2[22][52] + coeff0_23*dmats2[23][52] + coeff0_24*dmats2[24][52] + coeff0_25*dmats2[25][52] + coeff0_26*dmats2[26][52] + coeff0_27*dmats2[27][52] + coeff0_28*dmats2[28][52] + coeff0_29*dmats2[29][52] + coeff0_30*dmats2[30][52] + coeff0_31*dmats2[31][52] + coeff0_32*dmats2[32][52] + coeff0_33*dmats2[33][52] + coeff0_34*dmats2[34][52] + coeff0_35*dmats2[35][52] + coeff0_36*dmats2[36][52] + coeff0_37*dmats2[37][52] + coeff0_38*dmats2[38][52] + coeff0_39*dmats2[39][52] + coeff0_40*dmats2[40][52] + coeff0_41*dmats2[41][52] + coeff0_42*dmats2[42][52] + coeff0_43*dmats2[43][52] + coeff0_44*dmats2[44][52] + coeff0_45*dmats2[45][52] + coeff0_46*dmats2[46][52] + coeff0_47*dmats2[47][52] + coeff0_48*dmats2[48][52] + coeff0_49*dmats2[49][52] + coeff0_50*dmats2[50][52] + coeff0_51*dmats2[51][52] + coeff0_52*dmats2[52][52] + coeff0_53*dmats2[53][52] + coeff0_54*dmats2[54][52] + coeff0_55*dmats2[55][52];
          new_coeff0_53 = coeff0_0*dmats2[0][53] + coeff0_1*dmats2[1][53] + coeff0_2*dmats2[2][53] + coeff0_3*dmats2[3][53] + coeff0_4*dmats2[4][53] + coeff0_5*dmats2[5][53] + coeff0_6*dmats2[6][53] + coeff0_7*dmats2[7][53] + coeff0_8*dmats2[8][53] + coeff0_9*dmats2[9][53] + coeff0_10*dmats2[10][53] + coeff0_11*dmats2[11][53] + coeff0_12*dmats2[12][53] + coeff0_13*dmats2[13][53] + coeff0_14*dmats2[14][53] + coeff0_15*dmats2[15][53] + coeff0_16*dmats2[16][53] + coeff0_17*dmats2[17][53] + coeff0_18*dmats2[18][53] + coeff0_19*dmats2[19][53] + coeff0_20*dmats2[20][53] + coeff0_21*dmats2[21][53] + coeff0_22*dmats2[22][53] + coeff0_23*dmats2[23][53] + coeff0_24*dmats2[24][53] + coeff0_25*dmats2[25][53] + coeff0_26*dmats2[26][53] + coeff0_27*dmats2[27][53] + coeff0_28*dmats2[28][53] + coeff0_29*dmats2[29][53] + coeff0_30*dmats2[30][53] + coeff0_31*dmats2[31][53] + coeff0_32*dmats2[32][53] + coeff0_33*dmats2[33][53] + coeff0_34*dmats2[34][53] + coeff0_35*dmats2[35][53] + coeff0_36*dmats2[36][53] + coeff0_37*dmats2[37][53] + coeff0_38*dmats2[38][53] + coeff0_39*dmats2[39][53] + coeff0_40*dmats2[40][53] + coeff0_41*dmats2[41][53] + coeff0_42*dmats2[42][53] + coeff0_43*dmats2[43][53] + coeff0_44*dmats2[44][53] + coeff0_45*dmats2[45][53] + coeff0_46*dmats2[46][53] + coeff0_47*dmats2[47][53] + coeff0_48*dmats2[48][53] + coeff0_49*dmats2[49][53] + coeff0_50*dmats2[50][53] + coeff0_51*dmats2[51][53] + coeff0_52*dmats2[52][53] + coeff0_53*dmats2[53][53] + coeff0_54*dmats2[54][53] + coeff0_55*dmats2[55][53];
          new_coeff0_54 = coeff0_0*dmats2[0][54] + coeff0_1*dmats2[1][54] + coeff0_2*dmats2[2][54] + coeff0_3*dmats2[3][54] + coeff0_4*dmats2[4][54] + coeff0_5*dmats2[5][54] + coeff0_6*dmats2[6][54] + coeff0_7*dmats2[7][54] + coeff0_8*dmats2[8][54] + coeff0_9*dmats2[9][54] + coeff0_10*dmats2[10][54] + coeff0_11*dmats2[11][54] + coeff0_12*dmats2[12][54] + coeff0_13*dmats2[13][54] + coeff0_14*dmats2[14][54] + coeff0_15*dmats2[15][54] + coeff0_16*dmats2[16][54] + coeff0_17*dmats2[17][54] + coeff0_18*dmats2[18][54] + coeff0_19*dmats2[19][54] + coeff0_20*dmats2[20][54] + coeff0_21*dmats2[21][54] + coeff0_22*dmats2[22][54] + coeff0_23*dmats2[23][54] + coeff0_24*dmats2[24][54] + coeff0_25*dmats2[25][54] + coeff0_26*dmats2[26][54] + coeff0_27*dmats2[27][54] + coeff0_28*dmats2[28][54] + coeff0_29*dmats2[29][54] + coeff0_30*dmats2[30][54] + coeff0_31*dmats2[31][54] + coeff0_32*dmats2[32][54] + coeff0_33*dmats2[33][54] + coeff0_34*dmats2[34][54] + coeff0_35*dmats2[35][54] + coeff0_36*dmats2[36][54] + coeff0_37*dmats2[37][54] + coeff0_38*dmats2[38][54] + coeff0_39*dmats2[39][54] + coeff0_40*dmats2[40][54] + coeff0_41*dmats2[41][54] + coeff0_42*dmats2[42][54] + coeff0_43*dmats2[43][54] + coeff0_44*dmats2[44][54] + coeff0_45*dmats2[45][54] + coeff0_46*dmats2[46][54] + coeff0_47*dmats2[47][54] + coeff0_48*dmats2[48][54] + coeff0_49*dmats2[49][54] + coeff0_50*dmats2[50][54] + coeff0_51*dmats2[51][54] + coeff0_52*dmats2[52][54] + coeff0_53*dmats2[53][54] + coeff0_54*dmats2[54][54] + coeff0_55*dmats2[55][54];
          new_coeff0_55 = coeff0_0*dmats2[0][55] + coeff0_1*dmats2[1][55] + coeff0_2*dmats2[2][55] + coeff0_3*dmats2[3][55] + coeff0_4*dmats2[4][55] + coeff0_5*dmats2[5][55] + coeff0_6*dmats2[6][55] + coeff0_7*dmats2[7][55] + coeff0_8*dmats2[8][55] + coeff0_9*dmats2[9][55] + coeff0_10*dmats2[10][55] + coeff0_11*dmats2[11][55] + coeff0_12*dmats2[12][55] + coeff0_13*dmats2[13][55] + coeff0_14*dmats2[14][55] + coeff0_15*dmats2[15][55] + coeff0_16*dmats2[16][55] + coeff0_17*dmats2[17][55] + coeff0_18*dmats2[18][55] + coeff0_19*dmats2[19][55] + coeff0_20*dmats2[20][55] + coeff0_21*dmats2[21][55] + coeff0_22*dmats2[22][55] + coeff0_23*dmats2[23][55] + coeff0_24*dmats2[24][55] + coeff0_25*dmats2[25][55] + coeff0_26*dmats2[26][55] + coeff0_27*dmats2[27][55] + coeff0_28*dmats2[28][55] + coeff0_29*dmats2[29][55] + coeff0_30*dmats2[30][55] + coeff0_31*dmats2[31][55] + coeff0_32*dmats2[32][55] + coeff0_33*dmats2[33][55] + coeff0_34*dmats2[34][55] + coeff0_35*dmats2[35][55] + coeff0_36*dmats2[36][55] + coeff0_37*dmats2[37][55] + coeff0_38*dmats2[38][55] + coeff0_39*dmats2[39][55] + coeff0_40*dmats2[40][55] + coeff0_41*dmats2[41][55] + coeff0_42*dmats2[42][55] + coeff0_43*dmats2[43][55] + coeff0_44*dmats2[44][55] + coeff0_45*dmats2[45][55] + coeff0_46*dmats2[46][55] + coeff0_47*dmats2[47][55] + coeff0_48*dmats2[48][55] + coeff0_49*dmats2[49][55] + coeff0_50*dmats2[50][55] + coeff0_51*dmats2[51][55] + coeff0_52*dmats2[52][55] + coeff0_53*dmats2[53][55] + coeff0_54*dmats2[54][55] + coeff0_55*dmats2[55][55];
        }
    
      }
      // Compute derivatives on reference element as dot product of coefficients and basisvalues
      derivatives[deriv_num] = new_coeff0_0*basisvalue0 + new_coeff0_1*basisvalue1 + new_coeff0_2*basisvalue2 + new_coeff0_3*basisvalue3 + new_coeff0_4*basisvalue4 + new_coeff0_5*basisvalue5 + new_coeff0_6*basisvalue6 + new_coeff0_7*basisvalue7 + new_coeff0_8*basisvalue8 + new_coeff0_9*basisvalue9 + new_coeff0_10*basisvalue10 + new_coeff0_11*basisvalue11 + new_coeff0_12*basisvalue12 + new_coeff0_13*basisvalue13 + new_coeff0_14*basisvalue14 + new_coeff0_15*basisvalue15 + new_coeff0_16*basisvalue16 + new_coeff0_17*basisvalue17 + new_coeff0_18*basisvalue18 + new_coeff0_19*basisvalue19 + new_coeff0_20*basisvalue20 + new_coeff0_21*basisvalue21 + new_coeff0_22*basisvalue22 + new_coeff0_23*basisvalue23 + new_coeff0_24*basisvalue24 + new_coeff0_25*basisvalue25 + new_coeff0_26*basisvalue26 + new_coeff0_27*basisvalue27 + new_coeff0_28*basisvalue28 + new_coeff0_29*basisvalue29 + new_coeff0_30*basisvalue30 + new_coeff0_31*basisvalue31 + new_coeff0_32*basisvalue32 + new_coeff0_33*basisvalue33 + new_coeff0_34*basisvalue34 + new_coeff0_35*basisvalue35 + new_coeff0_36*basisvalue36 + new_coeff0_37*basisvalue37 + new_coeff0_38*basisvalue38 + new_coeff0_39*basisvalue39 + new_coeff0_40*basisvalue40 + new_coeff0_41*basisvalue41 + new_coeff0_42*basisvalue42 + new_coeff0_43*basisvalue43 + new_coeff0_44*basisvalue44 + new_coeff0_45*basisvalue45 + new_coeff0_46*basisvalue46 + new_coeff0_47*basisvalue47 + new_coeff0_48*basisvalue48 + new_coeff0_49*basisvalue49 + new_coeff0_50*basisvalue50 + new_coeff0_51*basisvalue51 + new_coeff0_52*basisvalue52 + new_coeff0_53*basisvalue53 + new_coeff0_54*basisvalue54 + new_coeff0_55*basisvalue55;
    }
    
    // Transform derivatives back to physical element
    for (unsigned int row = 0; row < num_derivatives; row++)
    {
      for (unsigned int col = 0; col < num_derivatives; col++)
      {
        values[row] += transform[row][col]*derivatives[col];
      }
    }
    // Delete pointer to array of derivatives on FIAT element
    delete [] derivatives;
    
    // Delete pointer to array of combinations of derivatives and transform
    for (unsigned int row = 0; row < num_derivatives; row++)
    {
      delete [] combinations[row];
      delete [] transform[row];
    }
    
    delete [] combinations;
    delete [] transform;
  }

  /// Evaluate order n derivatives of all basis functions at given point in cell
  virtual void evaluate_basis_derivatives_all(unsigned int n,
                                              double* values,
                                              const double* coordinates,
                                              const ufc::cell& c) const
  {
    throw std::runtime_error("The vectorised version of evaluate_basis_derivatives() is not yet implemented.");
  }

  /// Evaluate linear functional for dof i on the function f
  virtual double evaluate_dof(unsigned int i,
                              const ufc::function& f,
                              const ufc::cell& c) const
  {
    // The reference points, direction and weights:
    const static double X[56][1][3] = {{{0, 0, 0}}, {{1, 0, 0}}, {{0, 1, 0}}, {{0, 0, 1}}, {{0, 0.8, 0.2}}, {{0, 0.6, 0.4}}, {{0, 0.4, 0.6}}, {{0, 0.2, 0.8}}, {{0.8, 0, 0.2}}, {{0.6, 0, 0.4}}, {{0.4, 0, 0.6}}, {{0.2, 0, 0.8}}, {{0.8, 0.2, 0}}, {{0.6, 0.4, 0}}, {{0.4, 0.6, 0}}, {{0.2, 0.8, 0}}, {{0, 0, 0.2}}, {{0, 0, 0.4}}, {{0, 0, 0.6}}, {{0, 0, 0.8}}, {{0, 0.2, 0}}, {{0, 0.4, 0}}, {{0, 0.6, 0}}, {{0, 0.8, 0}}, {{0.2, 0, 0}}, {{0.4, 0, 0}}, {{0.6, 0, 0}}, {{0.8, 0, 0}}, {{0.6, 0.2, 0.2}}, {{0.4, 0.4, 0.2}}, {{0.2, 0.6, 0.2}}, {{0.4, 0.2, 0.4}}, {{0.2, 0.4, 0.4}}, {{0.2, 0.2, 0.6}}, {{0, 0.2, 0.2}}, {{0, 0.4, 0.2}}, {{0, 0.6, 0.2}}, {{0, 0.2, 0.4}}, {{0, 0.4, 0.4}}, {{0, 0.2, 0.6}}, {{0.2, 0, 0.2}}, {{0.4, 0, 0.2}}, {{0.6, 0, 0.2}}, {{0.2, 0, 0.4}}, {{0.4, 0, 0.4}}, {{0.2, 0, 0.6}}, {{0.2, 0.2, 0}}, {{0.4, 0.2, 0}}, {{0.6, 0.2, 0}}, {{0.2, 0.4, 0}}, {{0.4, 0.4, 0}}, {{0.2, 0.6, 0}}, {{0.2, 0.2, 0.2}}, {{0.4, 0.2, 0.2}}, {{0.2, 0.4, 0.2}}, {{0.2, 0.2, 0.4}}};
    const static double W[56][1] = {{1}, {1}, {1}, {1}, {1}, {1}, {1}, {1}, {1}, {1}, {1}, {1}, {1}, {1}, {1}, {1}, {1}, {1}, {1}, {1}, {1}, {1}, {1}, {1}, {1}, {1}, {1}, {1}, {1}, {1}, {1}, {1}, {1}, {1}, {1}, {1}, {1}, {1}, {1}, {1}, {1}, {1}, {1}, {1}, {1}, {1}, {1}, {1}, {1}, {1}, {1}, {1}, {1}, {1}, {1}, {1}};
    const static double D[56][1][1] = {{{1}}, {{1}}, {{1}}, {{1}}, {{1}}, {{1}}, {{1}}, {{1}}, {{1}}, {{1}}, {{1}}, {{1}}, {{1}}, {{1}}, {{1}}, {{1}}, {{1}}, {{1}}, {{1}}, {{1}}, {{1}}, {{1}}, {{1}}, {{1}}, {{1}}, {{1}}, {{1}}, {{1}}, {{1}}, {{1}}, {{1}}, {{1}}, {{1}}, {{1}}, {{1}}, {{1}}, {{1}}, {{1}}, {{1}}, {{1}}, {{1}}, {{1}}, {{1}}, {{1}}, {{1}}, {{1}}, {{1}}, {{1}}, {{1}}, {{1}}, {{1}}, {{1}}, {{1}}, {{1}}, {{1}}, {{1}}};
    
    const double * const * x = c.coordinates;
    double result = 0.0;
    // Iterate over the points:
    // Evaluate basis functions for affine mapping
    const double w0 = 1.0 - X[i][0][0] - X[i][0][1] - X[i][0][2];
    const double w1 = X[i][0][0];
    const double w2 = X[i][0][1];
    const double w3 = X[i][0][2];
    
    // Compute affine mapping y = F(X)
    double y[3];
    y[0] = w0*x[0][0] + w1*x[1][0] + w2*x[2][0] + w3*x[3][0];
    y[1] = w0*x[0][1] + w1*x[1][1] + w2*x[2][1] + w3*x[3][1];
    y[2] = w0*x[0][2] + w1*x[1][2] + w2*x[2][2] + w3*x[3][2];
    
    // Evaluate function at physical points
    double values[1];
    f.evaluate(values, y, c);
    
    // Map function values using appropriate mapping
    // Affine map: Do nothing
    
    // Note that we do not map the weights (yet).
    
    // Take directional components
    for(int k = 0; k < 1; k++)
      result += values[k]*D[i][0][k];
    // Multiply by weights 
    result *= W[i][0];
    
    return result;
  }

  /// Evaluate linear functionals for all dofs on the function f
  virtual void evaluate_dofs(double* values,
                             const ufc::function& f,
                             const ufc::cell& c) const
  {
    throw std::runtime_error("Not implemented (introduced in UFC v1.1).");
  }

  /// Interpolate vertex values from dof values
  virtual void interpolate_vertex_values(double* vertex_values,
                                         const double* dof_values,
                                         const ufc::cell& c) const
  {
    // Evaluate at vertices and use affine mapping
    vertex_values[0] = dof_values[0];
    vertex_values[1] = dof_values[1];
    vertex_values[2] = dof_values[2];
    vertex_values[3] = dof_values[3];
  }

  /// Return the number of sub elements (for a mixed element)
  virtual unsigned int num_sub_elements() const
  {
    return 1;
  }

  /// Create a new finite element for sub element i (for a mixed element)
  virtual ufc::finite_element* create_sub_element(unsigned int i) const
  {
    return new UFC_Poisson3D_5BilinearForm_finite_element_1();
  }

};

/// This class defines the interface for a local-to-global mapping of
/// degrees of freedom (dofs).

class UFC_Poisson3D_5BilinearForm_dof_map_0: public ufc::dof_map
{
private:

  unsigned int __global_dimension;

public:

  /// Constructor
  UFC_Poisson3D_5BilinearForm_dof_map_0() : ufc::dof_map()
  {
    __global_dimension = 0;
  }

  /// Destructor
  virtual ~UFC_Poisson3D_5BilinearForm_dof_map_0()
  {
    // Do nothing
  }

  /// Return a string identifying the dof map
  virtual const char* signature() const
  {
    return "FFC dof map for Lagrange finite element of degree 5 on a tetrahedron";
  }

  /// Return true iff mesh entities of topological dimension d are needed
  virtual bool needs_mesh_entities(unsigned int d) const
  {
    switch ( d )
    {
    case 0:
      return true;
      break;
    case 1:
      return true;
      break;
    case 2:
      return true;
      break;
    case 3:
      return true;
      break;
    }
    return false;
  }

  /// Initialize dof map for mesh (return true iff init_cell() is needed)
  virtual bool init_mesh(const ufc::mesh& m)
  {
    __global_dimension = m.num_entities[0] + 4*m.num_entities[1] + 6*m.num_entities[2] + 4*m.num_entities[3];
    return false;
  }

  /// Initialize dof map for given cell
  virtual void init_cell(const ufc::mesh& m,
                         const ufc::cell& c)
  {
    // Do nothing
  }

  /// Finish initialization of dof map for cells
  virtual void init_cell_finalize()
  {
    // Do nothing
  }

  /// Return the dimension of the global finite element function space
  virtual unsigned int global_dimension() const
  {
    return __global_dimension;
  }

  /// Return the dimension of the local finite element function space
  virtual unsigned int local_dimension() const
  {
    return 56;
  }

  // Return the geometric dimension of the coordinates this dof map provides
  virtual unsigned int geometric_dimension() const
  {
    return 3;
  }

  /// Return the number of dofs on each cell facet
  virtual unsigned int num_facet_dofs() const
  {
    return 21;
  }

  /// Return the number of dofs associated with each cell entity of dimension d
  virtual unsigned int num_entity_dofs(unsigned int d) const
  {
    throw std::runtime_error("Not implemented (introduced in UFC v1.1).");
  }

  /// Tabulate the local-to-global mapping of dofs on a cell
  virtual void tabulate_dofs(unsigned int* dofs,
                             const ufc::mesh& m,
                             const ufc::cell& c) const
  {
    dofs[0] = c.entity_indices[0][0];
    dofs[1] = c.entity_indices[0][1];
    dofs[2] = c.entity_indices[0][2];
    dofs[3] = c.entity_indices[0][3];
    unsigned int offset = m.num_entities[0];
    dofs[4] = offset + 4*c.entity_indices[1][0];
    dofs[5] = offset + 4*c.entity_indices[1][0] + 1;
    dofs[6] = offset + 4*c.entity_indices[1][0] + 2;
    dofs[7] = offset + 4*c.entity_indices[1][0] + 3;
    dofs[8] = offset + 4*c.entity_indices[1][1];
    dofs[9] = offset + 4*c.entity_indices[1][1] + 1;
    dofs[10] = offset + 4*c.entity_indices[1][1] + 2;
    dofs[11] = offset + 4*c.entity_indices[1][1] + 3;
    dofs[12] = offset + 4*c.entity_indices[1][2];
    dofs[13] = offset + 4*c.entity_indices[1][2] + 1;
    dofs[14] = offset + 4*c.entity_indices[1][2] + 2;
    dofs[15] = offset + 4*c.entity_indices[1][2] + 3;
    dofs[16] = offset + 4*c.entity_indices[1][3];
    dofs[17] = offset + 4*c.entity_indices[1][3] + 1;
    dofs[18] = offset + 4*c.entity_indices[1][3] + 2;
    dofs[19] = offset + 4*c.entity_indices[1][3] + 3;
    dofs[20] = offset + 4*c.entity_indices[1][4];
    dofs[21] = offset + 4*c.entity_indices[1][4] + 1;
    dofs[22] = offset + 4*c.entity_indices[1][4] + 2;
    dofs[23] = offset + 4*c.entity_indices[1][4] + 3;
    dofs[24] = offset + 4*c.entity_indices[1][5];
    dofs[25] = offset + 4*c.entity_indices[1][5] + 1;
    dofs[26] = offset + 4*c.entity_indices[1][5] + 2;
    dofs[27] = offset + 4*c.entity_indices[1][5] + 3;
    offset = offset + 4*m.num_entities[1];
    dofs[28] = offset + 6*c.entity_indices[2][0];
    dofs[29] = offset + 6*c.entity_indices[2][0] + 1;
    dofs[30] = offset + 6*c.entity_indices[2][0] + 2;
    dofs[31] = offset + 6*c.entity_indices[2][0] + 3;
    dofs[32] = offset + 6*c.entity_indices[2][0] + 4;
    dofs[33] = offset + 6*c.entity_indices[2][0] + 5;
    dofs[34] = offset + 6*c.entity_indices[2][1];
    dofs[35] = offset + 6*c.entity_indices[2][1] + 1;
    dofs[36] = offset + 6*c.entity_indices[2][1] + 2;
    dofs[37] = offset + 6*c.entity_indices[2][1] + 3;
    dofs[38] = offset + 6*c.entity_indices[2][1] + 4;
    dofs[39] = offset + 6*c.entity_indices[2][1] + 5;
    dofs[40] = offset + 6*c.entity_indices[2][2];
    dofs[41] = offset + 6*c.entity_indices[2][2] + 1;
    dofs[42] = offset + 6*c.entity_indices[2][2] + 2;
    dofs[43] = offset + 6*c.entity_indices[2][2] + 3;
    dofs[44] = offset + 6*c.entity_indices[2][2] + 4;
    dofs[45] = offset + 6*c.entity_indices[2][2] + 5;
    dofs[46] = offset + 6*c.entity_indices[2][3];
    dofs[47] = offset + 6*c.entity_indices[2][3] + 1;
    dofs[48] = offset + 6*c.entity_indices[2][3] + 2;
    dofs[49] = offset + 6*c.entity_indices[2][3] + 3;
    dofs[50] = offset + 6*c.entity_indices[2][3] + 4;
    dofs[51] = offset + 6*c.entity_indices[2][3] + 5;
    offset = offset + 6*m.num_entities[2];
    dofs[52] = offset + 4*c.entity_indices[3][0];
    dofs[53] = offset + 4*c.entity_indices[3][0] + 1;
    dofs[54] = offset + 4*c.entity_indices[3][0] + 2;
    dofs[55] = offset + 4*c.entity_indices[3][0] + 3;
  }

  /// Tabulate the local-to-local mapping from facet dofs to cell dofs
  virtual void tabulate_facet_dofs(unsigned int* dofs,
                                   unsigned int facet) const
  {
    switch ( facet )
    {
    case 0:
      dofs[0] = 1;
      dofs[1] = 2;
      dofs[2] = 3;
      dofs[3] = 4;
      dofs[4] = 5;
      dofs[5] = 6;
      dofs[6] = 7;
      dofs[7] = 8;
      dofs[8] = 9;
      dofs[9] = 10;
      dofs[10] = 11;
      dofs[11] = 12;
      dofs[12] = 13;
      dofs[13] = 14;
      dofs[14] = 15;
      dofs[15] = 28;
      dofs[16] = 29;
      dofs[17] = 30;
      dofs[18] = 31;
      dofs[19] = 32;
      dofs[20] = 33;
      break;
    case 1:
      dofs[0] = 0;
      dofs[1] = 2;
      dofs[2] = 3;
      dofs[3] = 4;
      dofs[4] = 5;
      dofs[5] = 6;
      dofs[6] = 7;
      dofs[7] = 16;
      dofs[8] = 17;
      dofs[9] = 18;
      dofs[10] = 19;
      dofs[11] = 20;
      dofs[12] = 21;
      dofs[13] = 22;
      dofs[14] = 23;
      dofs[15] = 34;
      dofs[16] = 35;
      dofs[17] = 36;
      dofs[18] = 37;
      dofs[19] = 38;
      dofs[20] = 39;
      break;
    case 2:
      dofs[0] = 0;
      dofs[1] = 1;
      dofs[2] = 3;
      dofs[3] = 8;
      dofs[4] = 9;
      dofs[5] = 10;
      dofs[6] = 11;
      dofs[7] = 16;
      dofs[8] = 17;
      dofs[9] = 18;
      dofs[10] = 19;
      dofs[11] = 24;
      dofs[12] = 25;
      dofs[13] = 26;
      dofs[14] = 27;
      dofs[15] = 40;
      dofs[16] = 41;
      dofs[17] = 42;
      dofs[18] = 43;
      dofs[19] = 44;
      dofs[20] = 45;
      break;
    case 3:
      dofs[0] = 0;
      dofs[1] = 1;
      dofs[2] = 2;
      dofs[3] = 12;
      dofs[4] = 13;
      dofs[5] = 14;
      dofs[6] = 15;
      dofs[7] = 20;
      dofs[8] = 21;
      dofs[9] = 22;
      dofs[10] = 23;
      dofs[11] = 24;
      dofs[12] = 25;
      dofs[13] = 26;
      dofs[14] = 27;
      dofs[15] = 46;
      dofs[16] = 47;
      dofs[17] = 48;
      dofs[18] = 49;
      dofs[19] = 50;
      dofs[20] = 51;
      break;
    }
  }

  /// Tabulate the local-to-local mapping of dofs on entity (d, i)
  virtual void tabulate_entity_dofs(unsigned int* dofs,
                                    unsigned int d, unsigned int i) const
  {
    throw std::runtime_error("Not implemented (introduced in UFC v1.1).");
  }

  /// Tabulate the coordinates of all dofs on a cell
  virtual void tabulate_coordinates(double** coordinates,
                                    const ufc::cell& c) const
  {
    const double * const * x = c.coordinates;
    coordinates[0][0] = x[0][0];
    coordinates[0][1] = x[0][1];
    coordinates[0][2] = x[0][2];
    coordinates[1][0] = x[1][0];
    coordinates[1][1] = x[1][1];
    coordinates[1][2] = x[1][2];
    coordinates[2][0] = x[2][0];
    coordinates[2][1] = x[2][1];
    coordinates[2][2] = x[2][2];
    coordinates[3][0] = x[3][0];
    coordinates[3][1] = x[3][1];
    coordinates[3][2] = x[3][2];
    coordinates[4][0] = 0.8*x[2][0] + 0.2*x[3][0];
    coordinates[4][1] = 0.8*x[2][1] + 0.2*x[3][1];
    coordinates[4][2] = 0.8*x[2][2] + 0.2*x[3][2];
    coordinates[5][0] = 0.6*x[2][0] + 0.4*x[3][0];
    coordinates[5][1] = 0.6*x[2][1] + 0.4*x[3][1];
    coordinates[5][2] = 0.6*x[2][2] + 0.4*x[3][2];
    coordinates[6][0] = 0.4*x[2][0] + 0.6*x[3][0];
    coordinates[6][1] = 0.4*x[2][1] + 0.6*x[3][1];
    coordinates[6][2] = 0.4*x[2][2] + 0.6*x[3][2];
    coordinates[7][0] = 0.2*x[2][0] + 0.8*x[3][0];
    coordinates[7][1] = 0.2*x[2][1] + 0.8*x[3][1];
    coordinates[7][2] = 0.2*x[2][2] + 0.8*x[3][2];
    coordinates[8][0] = 0.8*x[1][0] + 0.2*x[3][0];
    coordinates[8][1] = 0.8*x[1][1] + 0.2*x[3][1];
    coordinates[8][2] = 0.8*x[1][2] + 0.2*x[3][2];
    coordinates[9][0] = 0.6*x[1][0] + 0.4*x[3][0];
    coordinates[9][1] = 0.6*x[1][1] + 0.4*x[3][1];
    coordinates[9][2] = 0.6*x[1][2] + 0.4*x[3][2];
    coordinates[10][0] = 0.4*x[1][0] + 0.6*x[3][0];
    coordinates[10][1] = 0.4*x[1][1] + 0.6*x[3][1];
    coordinates[10][2] = 0.4*x[1][2] + 0.6*x[3][2];
    coordinates[11][0] = 0.2*x[1][0] + 0.8*x[3][0];
    coordinates[11][1] = 0.2*x[1][1] + 0.8*x[3][1];
    coordinates[11][2] = 0.2*x[1][2] + 0.8*x[3][2];
    coordinates[12][0] = 0.8*x[1][0] + 0.2*x[2][0];
    coordinates[12][1] = 0.8*x[1][1] + 0.2*x[2][1];
    coordinates[12][2] = 0.8*x[1][2] + 0.2*x[2][2];
    coordinates[13][0] = 0.6*x[1][0] + 0.4*x[2][0];
    coordinates[13][1] = 0.6*x[1][1] + 0.4*x[2][1];
    coordinates[13][2] = 0.6*x[1][2] + 0.4*x[2][2];
    coordinates[14][0] = 0.4*x[1][0] + 0.6*x[2][0];
    coordinates[14][1] = 0.4*x[1][1] + 0.6*x[2][1];
    coordinates[14][2] = 0.4*x[1][2] + 0.6*x[2][2];
    coordinates[15][0] = 0.2*x[1][0] + 0.8*x[2][0];
    coordinates[15][1] = 0.2*x[1][1] + 0.8*x[2][1];
    coordinates[15][2] = 0.2*x[1][2] + 0.8*x[2][2];
    coordinates[16][0] = 0.8*x[0][0] + 0.2*x[3][0];
    coordinates[16][1] = 0.8*x[0][1] + 0.2*x[3][1];
    coordinates[16][2] = 0.8*x[0][2] + 0.2*x[3][2];
    coordinates[17][0] = 0.6*x[0][0] + 0.4*x[3][0];
    coordinates[17][1] = 0.6*x[0][1] + 0.4*x[3][1];
    coordinates[17][2] = 0.6*x[0][2] + 0.4*x[3][2];
    coordinates[18][0] = 0.4*x[0][0] + 0.6*x[3][0];
    coordinates[18][1] = 0.4*x[0][1] + 0.6*x[3][1];
    coordinates[18][2] = 0.4*x[0][2] + 0.6*x[3][2];
    coordinates[19][0] = 0.2*x[0][0] + 0.8*x[3][0];
    coordinates[19][1] = 0.2*x[0][1] + 0.8*x[3][1];
    coordinates[19][2] = 0.2*x[0][2] + 0.8*x[3][2];
    coordinates[20][0] = 0.8*x[0][0] + 0.2*x[2][0];
    coordinates[20][1] = 0.8*x[0][1] + 0.2*x[2][1];
    coordinates[20][2] = 0.8*x[0][2] + 0.2*x[2][2];
    coordinates[21][0] = 0.6*x[0][0] + 0.4*x[2][0];
    coordinates[21][1] = 0.6*x[0][1] + 0.4*x[2][1];
    coordinates[21][2] = 0.6*x[0][2] + 0.4*x[2][2];
    coordinates[22][0] = 0.4*x[0][0] + 0.6*x[2][0];
    coordinates[22][1] = 0.4*x[0][1] + 0.6*x[2][1];
    coordinates[22][2] = 0.4*x[0][2] + 0.6*x[2][2];
    coordinates[23][0] = 0.2*x[0][0] + 0.8*x[2][0];
    coordinates[23][1] = 0.2*x[0][1] + 0.8*x[2][1];
    coordinates[23][2] = 0.2*x[0][2] + 0.8*x[2][2];
    coordinates[24][0] = 0.8*x[0][0] + 0.2*x[1][0];
    coordinates[24][1] = 0.8*x[0][1] + 0.2*x[1][1];
    coordinates[24][2] = 0.8*x[0][2] + 0.2*x[1][2];
    coordinates[25][0] = 0.6*x[0][0] + 0.4*x[1][0];
    coordinates[25][1] = 0.6*x[0][1] + 0.4*x[1][1];
    coordinates[25][2] = 0.6*x[0][2] + 0.4*x[1][2];
    coordinates[26][0] = 0.4*x[0][0] + 0.6*x[1][0];
    coordinates[26][1] = 0.4*x[0][1] + 0.6*x[1][1];
    coordinates[26][2] = 0.4*x[0][2] + 0.6*x[1][2];
    coordinates[27][0] = 0.2*x[0][0] + 0.8*x[1][0];
    coordinates[27][1] = 0.2*x[0][1] + 0.8*x[1][1];
    coordinates[27][2] = 0.2*x[0][2] + 0.8*x[1][2];
    coordinates[28][0] = 0.6*x[1][0] + 0.2*x[2][0] + 0.2*x[3][0];
    coordinates[28][1] = 0.6*x[1][1] + 0.2*x[2][1] + 0.2*x[3][1];
    coordinates[28][2] = 0.6*x[1][2] + 0.2*x[2][2] + 0.2*x[3][2];
    coordinates[29][0] = 0.4*x[1][0] + 0.4*x[2][0] + 0.2*x[3][0];
    coordinates[29][1] = 0.4*x[1][1] + 0.4*x[2][1] + 0.2*x[3][1];
    coordinates[29][2] = 0.4*x[1][2] + 0.4*x[2][2] + 0.2*x[3][2];
    coordinates[30][0] = 0.2*x[1][0] + 0.6*x[2][0] + 0.2*x[3][0];
    coordinates[30][1] = 0.2*x[1][1] + 0.6*x[2][1] + 0.2*x[3][1];
    coordinates[30][2] = 0.2*x[1][2] + 0.6*x[2][2] + 0.2*x[3][2];
    coordinates[31][0] = 0.4*x[1][0] + 0.2*x[2][0] + 0.4*x[3][0];
    coordinates[31][1] = 0.4*x[1][1] + 0.2*x[2][1] + 0.4*x[3][1];
    coordinates[31][2] = 0.4*x[1][2] + 0.2*x[2][2] + 0.4*x[3][2];
    coordinates[32][0] = 0.2*x[1][0] + 0.4*x[2][0] + 0.4*x[3][0];
    coordinates[32][1] = 0.2*x[1][1] + 0.4*x[2][1] + 0.4*x[3][1];
    coordinates[32][2] = 0.2*x[1][2] + 0.4*x[2][2] + 0.4*x[3][2];
    coordinates[33][0] = 0.2*x[1][0] + 0.2*x[2][0] + 0.6*x[3][0];
    coordinates[33][1] = 0.2*x[1][1] + 0.2*x[2][1] + 0.6*x[3][1];
    coordinates[33][2] = 0.2*x[1][2] + 0.2*x[2][2] + 0.6*x[3][2];
    coordinates[34][0] = 0.6*x[0][0] + 0.2*x[2][0] + 0.2*x[3][0];
    coordinates[34][1] = 0.6*x[0][1] + 0.2*x[2][1] + 0.2*x[3][1];
    coordinates[34][2] = 0.6*x[0][2] + 0.2*x[2][2] + 0.2*x[3][2];
    coordinates[35][0] = 0.4*x[0][0] + 0.4*x[2][0] + 0.2*x[3][0];
    coordinates[35][1] = 0.4*x[0][1] + 0.4*x[2][1] + 0.2*x[3][1];
    coordinates[35][2] = 0.4*x[0][2] + 0.4*x[2][2] + 0.2*x[3][2];
    coordinates[36][0] = 0.2*x[0][0] + 0.6*x[2][0] + 0.2*x[3][0];
    coordinates[36][1] = 0.2*x[0][1] + 0.6*x[2][1] + 0.2*x[3][1];
    coordinates[36][2] = 0.2*x[0][2] + 0.6*x[2][2] + 0.2*x[3][2];
    coordinates[37][0] = 0.4*x[0][0] + 0.2*x[2][0] + 0.4*x[3][0];
    coordinates[37][1] = 0.4*x[0][1] + 0.2*x[2][1] + 0.4*x[3][1];
    coordinates[37][2] = 0.4*x[0][2] + 0.2*x[2][2] + 0.4*x[3][2];
    coordinates[38][0] = 0.2*x[0][0] + 0.4*x[2][0] + 0.4*x[3][0];
    coordinates[38][1] = 0.2*x[0][1] + 0.4*x[2][1] + 0.4*x[3][1];
    coordinates[38][2] = 0.2*x[0][2] + 0.4*x[2][2] + 0.4*x[3][2];
    coordinates[39][0] = 0.2*x[0][0] + 0.2*x[2][0] + 0.6*x[3][0];
    coordinates[39][1] = 0.2*x[0][1] + 0.2*x[2][1] + 0.6*x[3][1];
    coordinates[39][2] = 0.2*x[0][2] + 0.2*x[2][2] + 0.6*x[3][2];
    coordinates[40][0] = 0.6*x[0][0] + 0.2*x[1][0] + 0.2*x[3][0];
    coordinates[40][1] = 0.6*x[0][1] + 0.2*x[1][1] + 0.2*x[3][1];
    coordinates[40][2] = 0.6*x[0][2] + 0.2*x[1][2] + 0.2*x[3][2];
    coordinates[41][0] = 0.4*x[0][0] + 0.4*x[1][0] + 0.2*x[3][0];
    coordinates[41][1] = 0.4*x[0][1] + 0.4*x[1][1] + 0.2*x[3][1];
    coordinates[41][2] = 0.4*x[0][2] + 0.4*x[1][2] + 0.2*x[3][2];
    coordinates[42][0] = 0.2*x[0][0] + 0.6*x[1][0] + 0.2*x[3][0];
    coordinates[42][1] = 0.2*x[0][1] + 0.6*x[1][1] + 0.2*x[3][1];
    coordinates[42][2] = 0.2*x[0][2] + 0.6*x[1][2] + 0.2*x[3][2];
    coordinates[43][0] = 0.4*x[0][0] + 0.2*x[1][0] + 0.4*x[3][0];
    coordinates[43][1] = 0.4*x[0][1] + 0.2*x[1][1] + 0.4*x[3][1];
    coordinates[43][2] = 0.4*x[0][2] + 0.2*x[1][2] + 0.4*x[3][2];
    coordinates[44][0] = 0.2*x[0][0] + 0.4*x[1][0] + 0.4*x[3][0];
    coordinates[44][1] = 0.2*x[0][1] + 0.4*x[1][1] + 0.4*x[3][1];
    coordinates[44][2] = 0.2*x[0][2] + 0.4*x[1][2] + 0.4*x[3][2];
    coordinates[45][0] = 0.2*x[0][0] + 0.2*x[1][0] + 0.6*x[3][0];
    coordinates[45][1] = 0.2*x[0][1] + 0.2*x[1][1] + 0.6*x[3][1];
    coordinates[45][2] = 0.2*x[0][2] + 0.2*x[1][2] + 0.6*x[3][2];
    coordinates[46][0] = 0.6*x[0][0] + 0.2*x[1][0] + 0.2*x[2][0];
    coordinates[46][1] = 0.6*x[0][1] + 0.2*x[1][1] + 0.2*x[2][1];
    coordinates[46][2] = 0.6*x[0][2] + 0.2*x[1][2] + 0.2*x[2][2];
    coordinates[47][0] = 0.4*x[0][0] + 0.4*x[1][0] + 0.2*x[2][0];
    coordinates[47][1] = 0.4*x[0][1] + 0.4*x[1][1] + 0.2*x[2][1];
    coordinates[47][2] = 0.4*x[0][2] + 0.4*x[1][2] + 0.2*x[2][2];
    coordinates[48][0] = 0.2*x[0][0] + 0.6*x[1][0] + 0.2*x[2][0];
    coordinates[48][1] = 0.2*x[0][1] + 0.6*x[1][1] + 0.2*x[2][1];
    coordinates[48][2] = 0.2*x[0][2] + 0.6*x[1][2] + 0.2*x[2][2];
    coordinates[49][0] = 0.4*x[0][0] + 0.2*x[1][0] + 0.4*x[2][0];
    coordinates[49][1] = 0.4*x[0][1] + 0.2*x[1][1] + 0.4*x[2][1];
    coordinates[49][2] = 0.4*x[0][2] + 0.2*x[1][2] + 0.4*x[2][2];
    coordinates[50][0] = 0.2*x[0][0] + 0.4*x[1][0] + 0.4*x[2][0];
    coordinates[50][1] = 0.2*x[0][1] + 0.4*x[1][1] + 0.4*x[2][1];
    coordinates[50][2] = 0.2*x[0][2] + 0.4*x[1][2] + 0.4*x[2][2];
    coordinates[51][0] = 0.2*x[0][0] + 0.2*x[1][0] + 0.6*x[2][0];
    coordinates[51][1] = 0.2*x[0][1] + 0.2*x[1][1] + 0.6*x[2][1];
    coordinates[51][2] = 0.2*x[0][2] + 0.2*x[1][2] + 0.6*x[2][2];
    coordinates[52][0] = 0.4*x[0][0] + 0.2*x[1][0] + 0.2*x[2][0] + 0.2*x[3][0];
    coordinates[52][1] = 0.4*x[0][1] + 0.2*x[1][1] + 0.2*x[2][1] + 0.2*x[3][1];
    coordinates[52][2] = 0.4*x[0][2] + 0.2*x[1][2] + 0.2*x[2][2] + 0.2*x[3][2];
    coordinates[53][0] = 0.2*x[0][0] + 0.4*x[1][0] + 0.2*x[2][0] + 0.2*x[3][0];
    coordinates[53][1] = 0.2*x[0][1] + 0.4*x[1][1] + 0.2*x[2][1] + 0.2*x[3][1];
    coordinates[53][2] = 0.2*x[0][2] + 0.4*x[1][2] + 0.2*x[2][2] + 0.2*x[3][2];
    coordinates[54][0] = 0.2*x[0][0] + 0.2*x[1][0] + 0.4*x[2][0] + 0.2*x[3][0];
    coordinates[54][1] = 0.2*x[0][1] + 0.2*x[1][1] + 0.4*x[2][1] + 0.2*x[3][1];
    coordinates[54][2] = 0.2*x[0][2] + 0.2*x[1][2] + 0.4*x[2][2] + 0.2*x[3][2];
    coordinates[55][0] = 0.2*x[0][0] + 0.2*x[1][0] + 0.2*x[2][0] + 0.4*x[3][0];
    coordinates[55][1] = 0.2*x[0][1] + 0.2*x[1][1] + 0.2*x[2][1] + 0.4*x[3][1];
    coordinates[55][2] = 0.2*x[0][2] + 0.2*x[1][2] + 0.2*x[2][2] + 0.4*x[3][2];
  }

  /// Return the number of sub dof maps (for a mixed element)
  virtual unsigned int num_sub_dof_maps() const
  {
    return 1;
  }

  /// Create a new dof_map for sub dof map i (for a mixed element)
  virtual ufc::dof_map* create_sub_dof_map(unsigned int i) const
  {
    return new UFC_Poisson3D_5BilinearForm_dof_map_0();
  }

};

/// This class defines the interface for a local-to-global mapping of
/// degrees of freedom (dofs).

class UFC_Poisson3D_5BilinearForm_dof_map_1: public ufc::dof_map
{
private:

  unsigned int __global_dimension;

public:

  /// Constructor
  UFC_Poisson3D_5BilinearForm_dof_map_1() : ufc::dof_map()
  {
    __global_dimension = 0;
  }

  /// Destructor
  virtual ~UFC_Poisson3D_5BilinearForm_dof_map_1()
  {
    // Do nothing
  }

  /// Return a string identifying the dof map
  virtual const char* signature() const
  {
    return "FFC dof map for Lagrange finite element of degree 5 on a tetrahedron";
  }

  /// Return true iff mesh entities of topological dimension d are needed
  virtual bool needs_mesh_entities(unsigned int d) const
  {
    switch ( d )
    {
    case 0:
      return true;
      break;
    case 1:
      return true;
      break;
    case 2:
      return true;
      break;
    case 3:
      return true;
      break;
    }
    return false;
  }

  /// Initialize dof map for mesh (return true iff init_cell() is needed)
  virtual bool init_mesh(const ufc::mesh& m)
  {
    __global_dimension = m.num_entities[0] + 4*m.num_entities[1] + 6*m.num_entities[2] + 4*m.num_entities[3];
    return false;
  }

  /// Initialize dof map for given cell
  virtual void init_cell(const ufc::mesh& m,
                         const ufc::cell& c)
  {
    // Do nothing
  }

  /// Finish initialization of dof map for cells
  virtual void init_cell_finalize()
  {
    // Do nothing
  }

  /// Return the dimension of the global finite element function space
  virtual unsigned int global_dimension() const
  {
    return __global_dimension;
  }

  /// Return the dimension of the local finite element function space
  virtual unsigned int local_dimension() const
  {
    return 56;
  }

  // Return the geometric dimension of the coordinates this dof map provides
  virtual unsigned int geometric_dimension() const
  {
    return 3;
  }

  /// Return the number of dofs on each cell facet
  virtual unsigned int num_facet_dofs() const
  {
    return 21;
  }

  /// Return the number of dofs associated with each cell entity of dimension d
  virtual unsigned int num_entity_dofs(unsigned int d) const
  {
    throw std::runtime_error("Not implemented (introduced in UFC v1.1).");
  }

  /// Tabulate the local-to-global mapping of dofs on a cell
  virtual void tabulate_dofs(unsigned int* dofs,
                             const ufc::mesh& m,
                             const ufc::cell& c) const
  {
    dofs[0] = c.entity_indices[0][0];
    dofs[1] = c.entity_indices[0][1];
    dofs[2] = c.entity_indices[0][2];
    dofs[3] = c.entity_indices[0][3];
    unsigned int offset = m.num_entities[0];
    dofs[4] = offset + 4*c.entity_indices[1][0];
    dofs[5] = offset + 4*c.entity_indices[1][0] + 1;
    dofs[6] = offset + 4*c.entity_indices[1][0] + 2;
    dofs[7] = offset + 4*c.entity_indices[1][0] + 3;
    dofs[8] = offset + 4*c.entity_indices[1][1];
    dofs[9] = offset + 4*c.entity_indices[1][1] + 1;
    dofs[10] = offset + 4*c.entity_indices[1][1] + 2;
    dofs[11] = offset + 4*c.entity_indices[1][1] + 3;
    dofs[12] = offset + 4*c.entity_indices[1][2];
    dofs[13] = offset + 4*c.entity_indices[1][2] + 1;
    dofs[14] = offset + 4*c.entity_indices[1][2] + 2;
    dofs[15] = offset + 4*c.entity_indices[1][2] + 3;
    dofs[16] = offset + 4*c.entity_indices[1][3];
    dofs[17] = offset + 4*c.entity_indices[1][3] + 1;
    dofs[18] = offset + 4*c.entity_indices[1][3] + 2;
    dofs[19] = offset + 4*c.entity_indices[1][3] + 3;
    dofs[20] = offset + 4*c.entity_indices[1][4];
    dofs[21] = offset + 4*c.entity_indices[1][4] + 1;
    dofs[22] = offset + 4*c.entity_indices[1][4] + 2;
    dofs[23] = offset + 4*c.entity_indices[1][4] + 3;
    dofs[24] = offset + 4*c.entity_indices[1][5];
    dofs[25] = offset + 4*c.entity_indices[1][5] + 1;
    dofs[26] = offset + 4*c.entity_indices[1][5] + 2;
    dofs[27] = offset + 4*c.entity_indices[1][5] + 3;
    offset = offset + 4*m.num_entities[1];
    dofs[28] = offset + 6*c.entity_indices[2][0];
    dofs[29] = offset + 6*c.entity_indices[2][0] + 1;
    dofs[30] = offset + 6*c.entity_indices[2][0] + 2;
    dofs[31] = offset + 6*c.entity_indices[2][0] + 3;
    dofs[32] = offset + 6*c.entity_indices[2][0] + 4;
    dofs[33] = offset + 6*c.entity_indices[2][0] + 5;
    dofs[34] = offset + 6*c.entity_indices[2][1];
    dofs[35] = offset + 6*c.entity_indices[2][1] + 1;
    dofs[36] = offset + 6*c.entity_indices[2][1] + 2;
    dofs[37] = offset + 6*c.entity_indices[2][1] + 3;
    dofs[38] = offset + 6*c.entity_indices[2][1] + 4;
    dofs[39] = offset + 6*c.entity_indices[2][1] + 5;
    dofs[40] = offset + 6*c.entity_indices[2][2];
    dofs[41] = offset + 6*c.entity_indices[2][2] + 1;
    dofs[42] = offset + 6*c.entity_indices[2][2] + 2;
    dofs[43] = offset + 6*c.entity_indices[2][2] + 3;
    dofs[44] = offset + 6*c.entity_indices[2][2] + 4;
    dofs[45] = offset + 6*c.entity_indices[2][2] + 5;
    dofs[46] = offset + 6*c.entity_indices[2][3];
    dofs[47] = offset + 6*c.entity_indices[2][3] + 1;
    dofs[48] = offset + 6*c.entity_indices[2][3] + 2;
    dofs[49] = offset + 6*c.entity_indices[2][3] + 3;
    dofs[50] = offset + 6*c.entity_indices[2][3] + 4;
    dofs[51] = offset + 6*c.entity_indices[2][3] + 5;
    offset = offset + 6*m.num_entities[2];
    dofs[52] = offset + 4*c.entity_indices[3][0];
    dofs[53] = offset + 4*c.entity_indices[3][0] + 1;
    dofs[54] = offset + 4*c.entity_indices[3][0] + 2;
    dofs[55] = offset + 4*c.entity_indices[3][0] + 3;
  }

  /// Tabulate the local-to-local mapping from facet dofs to cell dofs
  virtual void tabulate_facet_dofs(unsigned int* dofs,
                                   unsigned int facet) const
  {
    switch ( facet )
    {
    case 0:
      dofs[0] = 1;
      dofs[1] = 2;
      dofs[2] = 3;
      dofs[3] = 4;
      dofs[4] = 5;
      dofs[5] = 6;
      dofs[6] = 7;
      dofs[7] = 8;
      dofs[8] = 9;
      dofs[9] = 10;
      dofs[10] = 11;
      dofs[11] = 12;
      dofs[12] = 13;
      dofs[13] = 14;
      dofs[14] = 15;
      dofs[15] = 28;
      dofs[16] = 29;
      dofs[17] = 30;
      dofs[18] = 31;
      dofs[19] = 32;
      dofs[20] = 33;
      break;
    case 1:
      dofs[0] = 0;
      dofs[1] = 2;
      dofs[2] = 3;
      dofs[3] = 4;
      dofs[4] = 5;
      dofs[5] = 6;
      dofs[6] = 7;
      dofs[7] = 16;
      dofs[8] = 17;
      dofs[9] = 18;
      dofs[10] = 19;
      dofs[11] = 20;
      dofs[12] = 21;
      dofs[13] = 22;
      dofs[14] = 23;
      dofs[15] = 34;
      dofs[16] = 35;
      dofs[17] = 36;
      dofs[18] = 37;
      dofs[19] = 38;
      dofs[20] = 39;
      break;
    case 2:
      dofs[0] = 0;
      dofs[1] = 1;
      dofs[2] = 3;
      dofs[3] = 8;
      dofs[4] = 9;
      dofs[5] = 10;
      dofs[6] = 11;
      dofs[7] = 16;
      dofs[8] = 17;
      dofs[9] = 18;
      dofs[10] = 19;
      dofs[11] = 24;
      dofs[12] = 25;
      dofs[13] = 26;
      dofs[14] = 27;
      dofs[15] = 40;
      dofs[16] = 41;
      dofs[17] = 42;
      dofs[18] = 43;
      dofs[19] = 44;
      dofs[20] = 45;
      break;
    case 3:
      dofs[0] = 0;
      dofs[1] = 1;
      dofs[2] = 2;
      dofs[3] = 12;
      dofs[4] = 13;
      dofs[5] = 14;
      dofs[6] = 15;
      dofs[7] = 20;
      dofs[8] = 21;
      dofs[9] = 22;
      dofs[10] = 23;
      dofs[11] = 24;
      dofs[12] = 25;
      dofs[13] = 26;
      dofs[14] = 27;
      dofs[15] = 46;
      dofs[16] = 47;
      dofs[17] = 48;
      dofs[18] = 49;
      dofs[19] = 50;
      dofs[20] = 51;
      break;
    }
  }

  /// Tabulate the local-to-local mapping of dofs on entity (d, i)
  virtual void tabulate_entity_dofs(unsigned int* dofs,
                                    unsigned int d, unsigned int i) const
  {
    throw std::runtime_error("Not implemented (introduced in UFC v1.1).");
  }

  /// Tabulate the coordinates of all dofs on a cell
  virtual void tabulate_coordinates(double** coordinates,
                                    const ufc::cell& c) const
  {
    const double * const * x = c.coordinates;
    coordinates[0][0] = x[0][0];
    coordinates[0][1] = x[0][1];
    coordinates[0][2] = x[0][2];
    coordinates[1][0] = x[1][0];
    coordinates[1][1] = x[1][1];
    coordinates[1][2] = x[1][2];
    coordinates[2][0] = x[2][0];
    coordinates[2][1] = x[2][1];
    coordinates[2][2] = x[2][2];
    coordinates[3][0] = x[3][0];
    coordinates[3][1] = x[3][1];
    coordinates[3][2] = x[3][2];
    coordinates[4][0] = 0.8*x[2][0] + 0.2*x[3][0];
    coordinates[4][1] = 0.8*x[2][1] + 0.2*x[3][1];
    coordinates[4][2] = 0.8*x[2][2] + 0.2*x[3][2];
    coordinates[5][0] = 0.6*x[2][0] + 0.4*x[3][0];
    coordinates[5][1] = 0.6*x[2][1] + 0.4*x[3][1];
    coordinates[5][2] = 0.6*x[2][2] + 0.4*x[3][2];
    coordinates[6][0] = 0.4*x[2][0] + 0.6*x[3][0];
    coordinates[6][1] = 0.4*x[2][1] + 0.6*x[3][1];
    coordinates[6][2] = 0.4*x[2][2] + 0.6*x[3][2];
    coordinates[7][0] = 0.2*x[2][0] + 0.8*x[3][0];
    coordinates[7][1] = 0.2*x[2][1] + 0.8*x[3][1];
    coordinates[7][2] = 0.2*x[2][2] + 0.8*x[3][2];
    coordinates[8][0] = 0.8*x[1][0] + 0.2*x[3][0];
    coordinates[8][1] = 0.8*x[1][1] + 0.2*x[3][1];
    coordinates[8][2] = 0.8*x[1][2] + 0.2*x[3][2];
    coordinates[9][0] = 0.6*x[1][0] + 0.4*x[3][0];
    coordinates[9][1] = 0.6*x[1][1] + 0.4*x[3][1];
    coordinates[9][2] = 0.6*x[1][2] + 0.4*x[3][2];
    coordinates[10][0] = 0.4*x[1][0] + 0.6*x[3][0];
    coordinates[10][1] = 0.4*x[1][1] + 0.6*x[3][1];
    coordinates[10][2] = 0.4*x[1][2] + 0.6*x[3][2];
    coordinates[11][0] = 0.2*x[1][0] + 0.8*x[3][0];
    coordinates[11][1] = 0.2*x[1][1] + 0.8*x[3][1];
    coordinates[11][2] = 0.2*x[1][2] + 0.8*x[3][2];
    coordinates[12][0] = 0.8*x[1][0] + 0.2*x[2][0];
    coordinates[12][1] = 0.8*x[1][1] + 0.2*x[2][1];
    coordinates[12][2] = 0.8*x[1][2] + 0.2*x[2][2];
    coordinates[13][0] = 0.6*x[1][0] + 0.4*x[2][0];
    coordinates[13][1] = 0.6*x[1][1] + 0.4*x[2][1];
    coordinates[13][2] = 0.6*x[1][2] + 0.4*x[2][2];
    coordinates[14][0] = 0.4*x[1][0] + 0.6*x[2][0];
    coordinates[14][1] = 0.4*x[1][1] + 0.6*x[2][1];
    coordinates[14][2] = 0.4*x[1][2] + 0.6*x[2][2];
    coordinates[15][0] = 0.2*x[1][0] + 0.8*x[2][0];
    coordinates[15][1] = 0.2*x[1][1] + 0.8*x[2][1];
    coordinates[15][2] = 0.2*x[1][2] + 0.8*x[2][2];
    coordinates[16][0] = 0.8*x[0][0] + 0.2*x[3][0];
    coordinates[16][1] = 0.8*x[0][1] + 0.2*x[3][1];
    coordinates[16][2] = 0.8*x[0][2] + 0.2*x[3][2];
    coordinates[17][0] = 0.6*x[0][0] + 0.4*x[3][0];
    coordinates[17][1] = 0.6*x[0][1] + 0.4*x[3][1];
    coordinates[17][2] = 0.6*x[0][2] + 0.4*x[3][2];
    coordinates[18][0] = 0.4*x[0][0] + 0.6*x[3][0];
    coordinates[18][1] = 0.4*x[0][1] + 0.6*x[3][1];
    coordinates[18][2] = 0.4*x[0][2] + 0.6*x[3][2];
    coordinates[19][0] = 0.2*x[0][0] + 0.8*x[3][0];
    coordinates[19][1] = 0.2*x[0][1] + 0.8*x[3][1];
    coordinates[19][2] = 0.2*x[0][2] + 0.8*x[3][2];
    coordinates[20][0] = 0.8*x[0][0] + 0.2*x[2][0];
    coordinates[20][1] = 0.8*x[0][1] + 0.2*x[2][1];
    coordinates[20][2] = 0.8*x[0][2] + 0.2*x[2][2];
    coordinates[21][0] = 0.6*x[0][0] + 0.4*x[2][0];
    coordinates[21][1] = 0.6*x[0][1] + 0.4*x[2][1];
    coordinates[21][2] = 0.6*x[0][2] + 0.4*x[2][2];
    coordinates[22][0] = 0.4*x[0][0] + 0.6*x[2][0];
    coordinates[22][1] = 0.4*x[0][1] + 0.6*x[2][1];
    coordinates[22][2] = 0.4*x[0][2] + 0.6*x[2][2];
    coordinates[23][0] = 0.2*x[0][0] + 0.8*x[2][0];
    coordinates[23][1] = 0.2*x[0][1] + 0.8*x[2][1];
    coordinates[23][2] = 0.2*x[0][2] + 0.8*x[2][2];
    coordinates[24][0] = 0.8*x[0][0] + 0.2*x[1][0];
    coordinates[24][1] = 0.8*x[0][1] + 0.2*x[1][1];
    coordinates[24][2] = 0.8*x[0][2] + 0.2*x[1][2];
    coordinates[25][0] = 0.6*x[0][0] + 0.4*x[1][0];
    coordinates[25][1] = 0.6*x[0][1] + 0.4*x[1][1];
    coordinates[25][2] = 0.6*x[0][2] + 0.4*x[1][2];
    coordinates[26][0] = 0.4*x[0][0] + 0.6*x[1][0];
    coordinates[26][1] = 0.4*x[0][1] + 0.6*x[1][1];
    coordinates[26][2] = 0.4*x[0][2] + 0.6*x[1][2];
    coordinates[27][0] = 0.2*x[0][0] + 0.8*x[1][0];
    coordinates[27][1] = 0.2*x[0][1] + 0.8*x[1][1];
    coordinates[27][2] = 0.2*x[0][2] + 0.8*x[1][2];
    coordinates[28][0] = 0.6*x[1][0] + 0.2*x[2][0] + 0.2*x[3][0];
    coordinates[28][1] = 0.6*x[1][1] + 0.2*x[2][1] + 0.2*x[3][1];
    coordinates[28][2] = 0.6*x[1][2] + 0.2*x[2][2] + 0.2*x[3][2];
    coordinates[29][0] = 0.4*x[1][0] + 0.4*x[2][0] + 0.2*x[3][0];
    coordinates[29][1] = 0.4*x[1][1] + 0.4*x[2][1] + 0.2*x[3][1];
    coordinates[29][2] = 0.4*x[1][2] + 0.4*x[2][2] + 0.2*x[3][2];
    coordinates[30][0] = 0.2*x[1][0] + 0.6*x[2][0] + 0.2*x[3][0];
    coordinates[30][1] = 0.2*x[1][1] + 0.6*x[2][1] + 0.2*x[3][1];
    coordinates[30][2] = 0.2*x[1][2] + 0.6*x[2][2] + 0.2*x[3][2];
    coordinates[31][0] = 0.4*x[1][0] + 0.2*x[2][0] + 0.4*x[3][0];
    coordinates[31][1] = 0.4*x[1][1] + 0.2*x[2][1] + 0.4*x[3][1];
    coordinates[31][2] = 0.4*x[1][2] + 0.2*x[2][2] + 0.4*x[3][2];
    coordinates[32][0] = 0.2*x[1][0] + 0.4*x[2][0] + 0.4*x[3][0];
    coordinates[32][1] = 0.2*x[1][1] + 0.4*x[2][1] + 0.4*x[3][1];
    coordinates[32][2] = 0.2*x[1][2] + 0.4*x[2][2] + 0.4*x[3][2];
    coordinates[33][0] = 0.2*x[1][0] + 0.2*x[2][0] + 0.6*x[3][0];
    coordinates[33][1] = 0.2*x[1][1] + 0.2*x[2][1] + 0.6*x[3][1];
    coordinates[33][2] = 0.2*x[1][2] + 0.2*x[2][2] + 0.6*x[3][2];
    coordinates[34][0] = 0.6*x[0][0] + 0.2*x[2][0] + 0.2*x[3][0];
    coordinates[34][1] = 0.6*x[0][1] + 0.2*x[2][1] + 0.2*x[3][1];
    coordinates[34][2] = 0.6*x[0][2] + 0.2*x[2][2] + 0.2*x[3][2];
    coordinates[35][0] = 0.4*x[0][0] + 0.4*x[2][0] + 0.2*x[3][0];
    coordinates[35][1] = 0.4*x[0][1] + 0.4*x[2][1] + 0.2*x[3][1];
    coordinates[35][2] = 0.4*x[0][2] + 0.4*x[2][2] + 0.2*x[3][2];
    coordinates[36][0] = 0.2*x[0][0] + 0.6*x[2][0] + 0.2*x[3][0];
    coordinates[36][1] = 0.2*x[0][1] + 0.6*x[2][1] + 0.2*x[3][1];
    coordinates[36][2] = 0.2*x[0][2] + 0.6*x[2][2] + 0.2*x[3][2];
    coordinates[37][0] = 0.4*x[0][0] + 0.2*x[2][0] + 0.4*x[3][0];
    coordinates[37][1] = 0.4*x[0][1] + 0.2*x[2][1] + 0.4*x[3][1];
    coordinates[37][2] = 0.4*x[0][2] + 0.2*x[2][2] + 0.4*x[3][2];
    coordinates[38][0] = 0.2*x[0][0] + 0.4*x[2][0] + 0.4*x[3][0];
    coordinates[38][1] = 0.2*x[0][1] + 0.4*x[2][1] + 0.4*x[3][1];
    coordinates[38][2] = 0.2*x[0][2] + 0.4*x[2][2] + 0.4*x[3][2];
    coordinates[39][0] = 0.2*x[0][0] + 0.2*x[2][0] + 0.6*x[3][0];
    coordinates[39][1] = 0.2*x[0][1] + 0.2*x[2][1] + 0.6*x[3][1];
    coordinates[39][2] = 0.2*x[0][2] + 0.2*x[2][2] + 0.6*x[3][2];
    coordinates[40][0] = 0.6*x[0][0] + 0.2*x[1][0] + 0.2*x[3][0];
    coordinates[40][1] = 0.6*x[0][1] + 0.2*x[1][1] + 0.2*x[3][1];
    coordinates[40][2] = 0.6*x[0][2] + 0.2*x[1][2] + 0.2*x[3][2];
    coordinates[41][0] = 0.4*x[0][0] + 0.4*x[1][0] + 0.2*x[3][0];
    coordinates[41][1] = 0.4*x[0][1] + 0.4*x[1][1] + 0.2*x[3][1];
    coordinates[41][2] = 0.4*x[0][2] + 0.4*x[1][2] + 0.2*x[3][2];
    coordinates[42][0] = 0.2*x[0][0] + 0.6*x[1][0] + 0.2*x[3][0];
    coordinates[42][1] = 0.2*x[0][1] + 0.6*x[1][1] + 0.2*x[3][1];
    coordinates[42][2] = 0.2*x[0][2] + 0.6*x[1][2] + 0.2*x[3][2];
    coordinates[43][0] = 0.4*x[0][0] + 0.2*x[1][0] + 0.4*x[3][0];
    coordinates[43][1] = 0.4*x[0][1] + 0.2*x[1][1] + 0.4*x[3][1];
    coordinates[43][2] = 0.4*x[0][2] + 0.2*x[1][2] + 0.4*x[3][2];
    coordinates[44][0] = 0.2*x[0][0] + 0.4*x[1][0] + 0.4*x[3][0];
    coordinates[44][1] = 0.2*x[0][1] + 0.4*x[1][1] + 0.4*x[3][1];
    coordinates[44][2] = 0.2*x[0][2] + 0.4*x[1][2] + 0.4*x[3][2];
    coordinates[45][0] = 0.2*x[0][0] + 0.2*x[1][0] + 0.6*x[3][0];
    coordinates[45][1] = 0.2*x[0][1] + 0.2*x[1][1] + 0.6*x[3][1];
    coordinates[45][2] = 0.2*x[0][2] + 0.2*x[1][2] + 0.6*x[3][2];
    coordinates[46][0] = 0.6*x[0][0] + 0.2*x[1][0] + 0.2*x[2][0];
    coordinates[46][1] = 0.6*x[0][1] + 0.2*x[1][1] + 0.2*x[2][1];
    coordinates[46][2] = 0.6*x[0][2] + 0.2*x[1][2] + 0.2*x[2][2];
    coordinates[47][0] = 0.4*x[0][0] + 0.4*x[1][0] + 0.2*x[2][0];
    coordinates[47][1] = 0.4*x[0][1] + 0.4*x[1][1] + 0.2*x[2][1];
    coordinates[47][2] = 0.4*x[0][2] + 0.4*x[1][2] + 0.2*x[2][2];
    coordinates[48][0] = 0.2*x[0][0] + 0.6*x[1][0] + 0.2*x[2][0];
    coordinates[48][1] = 0.2*x[0][1] + 0.6*x[1][1] + 0.2*x[2][1];
    coordinates[48][2] = 0.2*x[0][2] + 0.6*x[1][2] + 0.2*x[2][2];
    coordinates[49][0] = 0.4*x[0][0] + 0.2*x[1][0] + 0.4*x[2][0];
    coordinates[49][1] = 0.4*x[0][1] + 0.2*x[1][1] + 0.4*x[2][1];
    coordinates[49][2] = 0.4*x[0][2] + 0.2*x[1][2] + 0.4*x[2][2];
    coordinates[50][0] = 0.2*x[0][0] + 0.4*x[1][0] + 0.4*x[2][0];
    coordinates[50][1] = 0.2*x[0][1] + 0.4*x[1][1] + 0.4*x[2][1];
    coordinates[50][2] = 0.2*x[0][2] + 0.4*x[1][2] + 0.4*x[2][2];
    coordinates[51][0] = 0.2*x[0][0] + 0.2*x[1][0] + 0.6*x[2][0];
    coordinates[51][1] = 0.2*x[0][1] + 0.2*x[1][1] + 0.6*x[2][1];
    coordinates[51][2] = 0.2*x[0][2] + 0.2*x[1][2] + 0.6*x[2][2];
    coordinates[52][0] = 0.4*x[0][0] + 0.2*x[1][0] + 0.2*x[2][0] + 0.2*x[3][0];
    coordinates[52][1] = 0.4*x[0][1] + 0.2*x[1][1] + 0.2*x[2][1] + 0.2*x[3][1];
    coordinates[52][2] = 0.4*x[0][2] + 0.2*x[1][2] + 0.2*x[2][2] + 0.2*x[3][2];
    coordinates[53][0] = 0.2*x[0][0] + 0.4*x[1][0] + 0.2*x[2][0] + 0.2*x[3][0];
    coordinates[53][1] = 0.2*x[0][1] + 0.4*x[1][1] + 0.2*x[2][1] + 0.2*x[3][1];
    coordinates[53][2] = 0.2*x[0][2] + 0.4*x[1][2] + 0.2*x[2][2] + 0.2*x[3][2];
    coordinates[54][0] = 0.2*x[0][0] + 0.2*x[1][0] + 0.4*x[2][0] + 0.2*x[3][0];
    coordinates[54][1] = 0.2*x[0][1] + 0.2*x[1][1] + 0.4*x[2][1] + 0.2*x[3][1];
    coordinates[54][2] = 0.2*x[0][2] + 0.2*x[1][2] + 0.4*x[2][2] + 0.2*x[3][2];
    coordinates[55][0] = 0.2*x[0][0] + 0.2*x[1][0] + 0.2*x[2][0] + 0.4*x[3][0];
    coordinates[55][1] = 0.2*x[0][1] + 0.2*x[1][1] + 0.2*x[2][1] + 0.4*x[3][1];
    coordinates[55][2] = 0.2*x[0][2] + 0.2*x[1][2] + 0.2*x[2][2] + 0.4*x[3][2];
  }

  /// Return the number of sub dof maps (for a mixed element)
  virtual unsigned int num_sub_dof_maps() const
  {
    return 1;
  }

  /// Create a new dof_map for sub dof map i (for a mixed element)
  virtual ufc::dof_map* create_sub_dof_map(unsigned int i) const
  {
    return new UFC_Poisson3D_5BilinearForm_dof_map_1();
  }

};

/// This class defines the interface for the tabulation of the cell
/// tensor corresponding to the local contribution to a form from
/// the integral over a cell.

class UFC_Poisson3D_5BilinearForm_cell_integral_0: public ufc::cell_integral
{
public:

  /// Constructor
  UFC_Poisson3D_5BilinearForm_cell_integral_0() : ufc::cell_integral()
  {
    // Do nothing
  }

  /// Destructor
  virtual ~UFC_Poisson3D_5BilinearForm_cell_integral_0()
  {
    // Do nothing
  }

  /// Tabulate the tensor for the contribution from a local cell
  virtual void tabulate_tensor(double* A,
                               const double * const * w,
                               const ufc::cell& c) const
  {
    // Extract vertex coordinates
    const double * const * x = c.coordinates;
    
    // Compute Jacobian of affine map from reference cell
    const double J_00 = x[1][0] - x[0][0];
    const double J_01 = x[2][0] - x[0][0];
    const double J_02 = x[3][0] - x[0][0];
    const double J_10 = x[1][1] - x[0][1];
    const double J_11 = x[2][1] - x[0][1];
    const double J_12 = x[3][1] - x[0][1];
    const double J_20 = x[1][2] - x[0][2];
    const double J_21 = x[2][2] - x[0][2];
    const double J_22 = x[3][2] - x[0][2];
      
    // Compute sub determinants
    const double d_00 = J_11*J_22 - J_12*J_21;
    const double d_01 = J_12*J_20 - J_10*J_22;
    const double d_02 = J_10*J_21 - J_11*J_20;
    
    const double d_10 = J_02*J_21 - J_01*J_22;
    const double d_11 = J_00*J_22 - J_02*J_20;
    const double d_12 = J_01*J_20 - J_00*J_21;
    
    const double d_20 = J_01*J_12 - J_02*J_11;
    const double d_21 = J_02*J_10 - J_00*J_12;
    const double d_22 = J_00*J_11 - J_01*J_10;
      
    // Compute determinant of Jacobian
    double detJ = J_00*d_00 + J_10*d_10 + J_20*d_20;
      
    // Compute inverse of Jacobian
    const double Jinv_00 = d_00 / detJ;
    const double Jinv_01 = d_10 / detJ;
    const double Jinv_02 = d_20 / detJ;
    const double Jinv_10 = d_01 / detJ;
    const double Jinv_11 = d_11 / detJ;
    const double Jinv_12 = d_21 / detJ;
    const double Jinv_20 = d_02 / detJ;
    const double Jinv_21 = d_12 / detJ;
    const double Jinv_22 = d_22 / detJ;
    
    // Set scale factor
    const double det = std::abs(detJ);
    
    // Compute geometry tensors
    const double G0_0_0 = det*(Jinv_00*Jinv_00 + Jinv_01*Jinv_01 + Jinv_02*Jinv_02);
    const double G0_0_1 = det*(Jinv_00*Jinv_10 + Jinv_01*Jinv_11 + Jinv_02*Jinv_12);
    const double G0_0_2 = det*(Jinv_00*Jinv_20 + Jinv_01*Jinv_21 + Jinv_02*Jinv_22);
    const double G0_1_0 = det*(Jinv_10*Jinv_00 + Jinv_11*Jinv_01 + Jinv_12*Jinv_02);
    const double G0_1_1 = det*(Jinv_10*Jinv_10 + Jinv_11*Jinv_11 + Jinv_12*Jinv_12);
    const double G0_1_2 = det*(Jinv_10*Jinv_20 + Jinv_11*Jinv_21 + Jinv_12*Jinv_22);
    const double G0_2_0 = det*(Jinv_20*Jinv_00 + Jinv_21*Jinv_01 + Jinv_22*Jinv_02);
    const double G0_2_1 = det*(Jinv_20*Jinv_10 + Jinv_21*Jinv_11 + Jinv_22*Jinv_12);
    const double G0_2_2 = det*(Jinv_20*Jinv_20 + Jinv_21*Jinv_21 + Jinv_22*Jinv_22);
    
    // Compute element tensor
    A[0] = 0.0262652567340064*G0_0_0 + 0.0262652567340064*G0_0_1 + 0.0262652567340064*G0_0_2 + 0.0262652567340064*G0_1_0 + 0.0262652567340064*G0_1_1 + 0.0262652567340064*G0_1_2 + 0.0262652567340064*G0_2_0 + 0.0262652567340064*G0_2_1 + 0.0262652567340064*G0_2_2;
    A[1] = -0.00277890512265508*G0_0_0 - 0.00277890512265509*G0_1_0 - 0.0027789051226551*G0_2_0;
    A[2] = -0.00277890512265506*G0_0_1 - 0.00277890512265506*G0_1_1 - 0.00277890512265506*G0_2_1;
    A[3] = -0.00277890512265513*G0_0_2 - 0.00277890512265514*G0_1_2 - 0.00277890512265513*G0_2_2;
    A[4] = -0.00480034930989788*G0_0_1 - 0.00480034930989781*G0_0_2 - 0.00480034930989788*G0_1_1 - 0.00480034930989782*G0_1_2 - 0.00480034930989788*G0_2_1 - 0.00480034930989782*G0_2_2;
    A[5] = -0.00336507226698201*G0_0_1 - 0.00336507226698203*G0_0_2 - 0.003365072266982*G0_1_1 - 0.00336507226698201*G0_1_2 - 0.003365072266982*G0_2_1 - 0.00336507226698202*G0_2_2;
    A[6] = -0.00336507226698185*G0_0_1 - 0.0033650722669819*G0_0_2 - 0.00336507226698184*G0_1_1 - 0.00336507226698188*G0_1_2 - 0.00336507226698183*G0_2_1 - 0.00336507226698186*G0_2_2;
    A[7] = -0.00480034930989796*G0_0_1 - 0.00480034930989792*G0_0_2 - 0.00480034930989799*G0_1_1 - 0.00480034930989793*G0_1_2 - 0.004800349309898*G0_2_1 - 0.00480034930989797*G0_2_2;
    A[8] = -0.00480034930989783*G0_0_0 - 0.00480034930989781*G0_0_2 - 0.00480034930989786*G0_1_0 - 0.00480034930989784*G0_1_2 - 0.00480034930989786*G0_2_0 - 0.00480034930989787*G0_2_2;
    A[9] = -0.00336507226698195*G0_0_0 - 0.00336507226698199*G0_0_2 - 0.00336507226698198*G0_1_0 - 0.00336507226698199*G0_1_2 - 0.00336507226698197*G0_2_0 - 0.00336507226698196*G0_2_2;
    A[10] = -0.00336507226698197*G0_0_0 - 0.00336507226698191*G0_0_2 - 0.00336507226698195*G0_1_0 - 0.00336507226698193*G0_1_2 - 0.00336507226698193*G0_2_0 - 0.0033650722669819*G0_2_2;
    A[11] = -0.00480034930989796*G0_0_0 - 0.00480034930989795*G0_0_2 - 0.00480034930989798*G0_1_0 - 0.00480034930989796*G0_1_2 - 0.004800349309898*G0_2_0 - 0.00480034930989799*G0_2_2;
    A[12] = -0.00480034930989784*G0_0_0 - 0.00480034930989785*G0_0_1 - 0.00480034930989785*G0_1_0 - 0.00480034930989788*G0_1_1 - 0.00480034930989787*G0_2_0 - 0.00480034930989791*G0_2_1;
    A[13] = -0.00336507226698189*G0_0_0 - 0.00336507226698191*G0_0_1 - 0.00336507226698187*G0_1_0 - 0.00336507226698185*G0_1_1 - 0.00336507226698185*G0_2_0 - 0.00336507226698183*G0_2_1;
    A[14] = -0.00336507226698199*G0_0_0 - 0.00336507226698195*G0_0_1 - 0.003365072266982*G0_1_0 - 0.00336507226698199*G0_1_1 - 0.00336507226698199*G0_2_0 - 0.00336507226698198*G0_2_1;
    A[15] = -0.00480034930989779*G0_0_0 - 0.00480034930989785*G0_0_1 - 0.00480034930989781*G0_1_0 - 0.00480034930989785*G0_1_1 - 0.0048003493098978*G0_2_0 - 0.00480034930989784*G0_2_1;
    A[16] = -0.0178705039816149*G0_0_0 - 0.0178705039816148*G0_0_1 - 0.0551276864210886*G0_0_2 - 0.0178705039816149*G0_1_0 - 0.0178705039816149*G0_1_1 - 0.0551276864210886*G0_1_2 - 0.0178705039816149*G0_2_0 - 0.0178705039816149*G0_2_1 - 0.0551276864210886*G0_2_2;
    A[17] = 0.0214691349673988*G0_0_0 + 0.0214691349673988*G0_0_1 + 0.0551276864210887*G0_0_2 + 0.0214691349673987*G0_1_0 + 0.0214691349673987*G0_1_1 + 0.0551276864210887*G0_1_2 + 0.0214691349673988*G0_2_0 + 0.0214691349673988*G0_2_1 + 0.0551276864210887*G0_2_2;
    A[18] = -0.0155544887532733*G0_0_0 - 0.0155544887532733*G0_0_1 - 0.0388177200243169*G0_0_2 - 0.0155544887532733*G0_1_0 - 0.0155544887532733*G0_1_1 - 0.0388177200243168*G0_1_2 - 0.0155544887532734*G0_2_0 - 0.0155544887532734*G0_2_1 - 0.0388177200243168*G0_2_2;
    A[19] = 0.00480034930989794*G0_0_0 + 0.00480034930989795*G0_0_1 + 0.0153313684129655*G0_0_2 + 0.00480034930989797*G0_1_0 + 0.00480034930989797*G0_1_1 + 0.0153313684129655*G0_1_2 + 0.00480034930989798*G0_2_0 + 0.00480034930989799*G0_2_1 + 0.0153313684129655*G0_2_2;
    A[20] = -0.0178705039816148*G0_0_0 - 0.0551276864210885*G0_0_1 - 0.0178705039816148*G0_0_2 - 0.0178705039816148*G0_1_0 - 0.0551276864210885*G0_1_1 - 0.0178705039816148*G0_1_2 - 0.0178705039816148*G0_2_0 - 0.0551276864210885*G0_2_1 - 0.0178705039816148*G0_2_2;
    A[21] = 0.0214691349673985*G0_0_0 + 0.0551276864210885*G0_0_1 + 0.0214691349673985*G0_0_2 + 0.0214691349673985*G0_1_0 + 0.0551276864210885*G0_1_1 + 0.0214691349673985*G0_1_2 + 0.0214691349673984*G0_2_0 + 0.0551276864210885*G0_2_1 + 0.0214691349673985*G0_2_2;
    A[22] = -0.0155544887532731*G0_0_0 - 0.0388177200243167*G0_0_1 - 0.0155544887532731*G0_0_2 - 0.0155544887532731*G0_1_0 - 0.0388177200243166*G0_1_1 - 0.0155544887532731*G0_1_2 - 0.0155544887532731*G0_2_0 - 0.0388177200243167*G0_2_1 - 0.0155544887532731*G0_2_2;
    A[23] = 0.00480034930989781*G0_0_0 + 0.0153313684129654*G0_0_1 + 0.00480034930989781*G0_0_2 + 0.00480034930989783*G0_1_0 + 0.0153313684129653*G0_1_1 + 0.00480034930989783*G0_1_2 + 0.00480034930989782*G0_2_0 + 0.0153313684129653*G0_2_1 + 0.00480034930989782*G0_2_2;
    A[24] = -0.0551276864210884*G0_0_0 - 0.0178705039816147*G0_0_1 - 0.0178705039816147*G0_0_2 - 0.0551276864210884*G0_1_0 - 0.0178705039816147*G0_1_1 - 0.0178705039816147*G0_1_2 - 0.0551276864210884*G0_2_0 - 0.0178705039816147*G0_2_1 - 0.0178705039816147*G0_2_2;
    A[25] = 0.0551276864210883*G0_0_0 + 0.0214691349673984*G0_0_1 + 0.0214691349673984*G0_0_2 + 0.0551276864210883*G0_1_0 + 0.0214691349673984*G0_1_1 + 0.0214691349673984*G0_1_2 + 0.0551276864210883*G0_2_0 + 0.0214691349673984*G0_2_1 + 0.0214691349673984*G0_2_2;
    A[26] = -0.0388177200243166*G0_0_0 - 0.0155544887532731*G0_0_1 - 0.0155544887532731*G0_0_2 - 0.0388177200243166*G0_1_0 - 0.0155544887532731*G0_1_1 - 0.0155544887532731*G0_1_2 - 0.0388177200243166*G0_2_0 - 0.0155544887532731*G0_2_1 - 0.0155544887532731*G0_2_2;
    A[27] = 0.0153313684129654*G0_0_0 + 0.00480034930989783*G0_0_1 + 0.00480034930989784*G0_0_2 + 0.0153313684129654*G0_1_0 + 0.00480034930989786*G0_1_1 + 0.00480034930989788*G0_1_2 + 0.0153313684129654*G0_2_0 + 0.00480034930989789*G0_2_1 + 0.0048003493098979*G0_2_2;
    A[28] = -0.0137421252872641*G0_0_0 - 0.013742125287264*G0_0_1 - 0.013742125287264*G0_0_2 - 0.0137421252872641*G0_1_0 - 0.0137421252872641*G0_1_1 - 0.0137421252872641*G0_1_2 - 0.0137421252872641*G0_2_0 - 0.0137421252872641*G0_2_1 - 0.0137421252872641*G0_2_2;
    A[29] = -0.00946499969937453*G0_0_0 - 0.00946499969937467*G0_0_1 - 0.0094649996993745*G0_0_2 - 0.00946499969937456*G0_1_0 - 0.00946499969937462*G0_1_1 - 0.00946499969937448*G0_1_2 - 0.00946499969937457*G0_2_0 - 0.00946499969937463*G0_2_1 - 0.00946499969937443*G0_2_2;
    A[30] = -0.013742125287264*G0_0_0 - 0.0137421252872639*G0_0_1 - 0.013742125287264*G0_0_2 - 0.0137421252872641*G0_1_0 - 0.013742125287264*G0_1_1 - 0.013742125287264*G0_1_2 - 0.0137421252872641*G0_2_0 - 0.013742125287264*G0_2_1 - 0.013742125287264*G0_2_2;
    A[31] = -0.00946499969937459*G0_0_0 - 0.0094649996993745*G0_0_1 - 0.0094649996993747*G0_0_2 - 0.00946499969937462*G0_1_0 - 0.00946499969937453*G0_1_1 - 0.00946499969937473*G0_1_2 - 0.00946499969937462*G0_2_0 - 0.0094649996993745*G0_2_1 - 0.00946499969937478*G0_2_2;
    A[32] = -0.00946499969937453*G0_0_0 - 0.00946499969937451*G0_0_1 - 0.00946499969937456*G0_0_2 - 0.00946499969937445*G0_1_0 - 0.00946499969937447*G0_1_1 - 0.00946499969937454*G0_1_2 - 0.00946499969937447*G0_2_0 - 0.00946499969937451*G0_2_1 - 0.00946499969937461*G0_2_2;
    A[33] = -0.0137421252872641*G0_0_0 - 0.0137421252872642*G0_0_1 - 0.013742125287264*G0_0_2 - 0.0137421252872641*G0_1_0 - 0.0137421252872642*G0_1_1 - 0.013742125287264*G0_1_2 - 0.0137421252872641*G0_2_0 - 0.0137421252872642*G0_2_1 - 0.0137421252872639*G0_2_2;
    A[34] = 0.047993054720485*G0_0_0 + 0.0335985307773499*G0_0_1 + 0.03359853077735*G0_0_2 + 0.047993054720485*G0_1_0 + 0.0335985307773499*G0_1_1 + 0.0335985307773499*G0_1_2 + 0.047993054720485*G0_2_0 + 0.0335985307773499*G0_2_1 + 0.03359853077735*G0_2_2;
    A[35] = -0.0345719095719094*G0_0_0 - 0.0298472385060926*G0_0_1 - 0.0298472385060924*G0_0_2 - 0.0345719095719094*G0_1_0 - 0.0298472385060926*G0_1_1 - 0.0298472385060924*G0_1_2 - 0.0345719095719093*G0_2_0 - 0.0298472385060925*G0_2_1 - 0.0298472385060923*G0_2_2;
    A[36] = 0.013742125287264*G0_0_0 + 0.0189195610202554*G0_0_1 + 0.0189195610202551*G0_0_2 + 0.0137421252872641*G0_1_0 + 0.0189195610202554*G0_1_1 + 0.0189195610202551*G0_1_2 + 0.0137421252872641*G0_2_0 + 0.0189195610202554*G0_2_1 + 0.0189195610202551*G0_2_2;
    A[37] = -0.0345719095719091*G0_0_0 - 0.0298472385060924*G0_0_1 - 0.0298472385060926*G0_0_2 - 0.0345719095719091*G0_1_0 - 0.0298472385060924*G0_1_1 - 0.0298472385060925*G0_1_2 - 0.0345719095719092*G0_2_0 - 0.0298472385060924*G0_2_1 - 0.0298472385060926*G0_2_2;
    A[38] = 0.00946499969937456*G0_0_0 + 0.0117431758056756*G0_0_1 + 0.0117431758056758*G0_0_2 + 0.00946499969937447*G0_1_0 + 0.0117431758056756*G0_1_1 + 0.0117431758056758*G0_1_2 + 0.00946499969937449*G0_2_0 + 0.0117431758056756*G0_2_1 + 0.0117431758056757*G0_2_2;
    A[39] = 0.0137421252872641*G0_0_0 + 0.0189195610202553*G0_0_1 + 0.0189195610202553*G0_0_2 + 0.0137421252872642*G0_1_0 + 0.0189195610202552*G0_1_1 + 0.0189195610202553*G0_1_2 + 0.0137421252872642*G0_2_0 + 0.0189195610202553*G0_2_1 + 0.0189195610202554*G0_2_2;
    A[40] = 0.0335985307773497*G0_0_0 + 0.0479930547204848*G0_0_1 + 0.0335985307773496*G0_0_2 + 0.0335985307773498*G0_1_0 + 0.0479930547204849*G0_1_1 + 0.0335985307773497*G0_1_2 + 0.0335985307773498*G0_2_0 + 0.0479930547204849*G0_2_1 + 0.0335985307773498*G0_2_2;
    A[41] = -0.0298472385060919*G0_0_0 - 0.034571909571909*G0_0_1 - 0.029847238506092*G0_0_2 - 0.029847238506092*G0_1_0 - 0.034571909571909*G0_1_1 - 0.0298472385060921*G0_1_2 - 0.029847238506092*G0_2_0 - 0.0345719095719089*G0_2_1 - 0.029847238506092*G0_2_2;
    A[42] = 0.018919561020255*G0_0_0 + 0.0137421252872641*G0_0_1 + 0.0189195610202549*G0_0_2 + 0.018919561020255*G0_1_0 + 0.0137421252872642*G0_1_1 + 0.0189195610202549*G0_1_2 + 0.018919561020255*G0_2_0 + 0.0137421252872642*G0_2_1 + 0.0189195610202549*G0_2_2;
    A[43] = -0.0298472385060926*G0_0_0 - 0.0345719095719093*G0_0_1 - 0.0298472385060924*G0_0_2 - 0.0298472385060925*G0_1_0 - 0.0345719095719093*G0_1_1 - 0.0298472385060925*G0_1_2 - 0.0298472385060926*G0_2_0 - 0.0345719095719093*G0_2_1 - 0.0298472385060926*G0_2_2;
    A[44] = 0.0117431758056757*G0_0_0 + 0.00946499969937446*G0_0_1 + 0.0117431758056757*G0_0_2 + 0.0117431758056757*G0_1_0 + 0.0094649996993745*G0_1_1 + 0.0117431758056757*G0_1_2 + 0.0117431758056757*G0_2_0 + 0.00946499969937446*G0_2_1 + 0.0117431758056756*G0_2_2;
    A[45] = 0.0189195610202553*G0_0_0 + 0.0137421252872642*G0_0_1 + 0.0189195610202553*G0_0_2 + 0.0189195610202553*G0_1_0 + 0.0137421252872642*G0_1_1 + 0.0189195610202553*G0_1_2 + 0.0189195610202553*G0_2_0 + 0.0137421252872642*G0_2_1 + 0.0189195610202554*G0_2_2;
    A[46] = 0.0335985307773496*G0_0_0 + 0.0335985307773495*G0_0_1 + 0.0479930547204847*G0_0_2 + 0.0335985307773497*G0_1_0 + 0.0335985307773496*G0_1_1 + 0.0479930547204848*G0_1_2 + 0.0335985307773497*G0_2_0 + 0.0335985307773496*G0_2_1 + 0.0479930547204847*G0_2_2;
    A[47] = -0.029847238506092*G0_0_0 - 0.0298472385060921*G0_0_1 - 0.034571909571909*G0_0_2 - 0.029847238506092*G0_1_0 - 0.0298472385060921*G0_1_1 - 0.034571909571909*G0_1_2 - 0.0298472385060921*G0_2_0 - 0.0298472385060921*G0_2_1 - 0.034571909571909*G0_2_2;
    A[48] = 0.018919561020255*G0_0_0 + 0.018919561020255*G0_0_1 + 0.0137421252872642*G0_0_2 + 0.018919561020255*G0_1_0 + 0.018919561020255*G0_1_1 + 0.0137421252872642*G0_1_2 + 0.0189195610202551*G0_2_0 + 0.018919561020255*G0_2_1 + 0.0137421252872642*G0_2_2;
    A[49] = -0.0298472385060923*G0_0_0 - 0.0298472385060922*G0_0_1 - 0.0345719095719093*G0_0_2 - 0.0298472385060923*G0_1_0 - 0.0298472385060923*G0_1_1 - 0.0345719095719093*G0_1_2 - 0.0298472385060923*G0_2_0 - 0.0298472385060923*G0_2_1 - 0.0345719095719092*G0_2_2;
    A[50] = 0.0117431758056757*G0_0_0 + 0.0117431758056756*G0_0_1 + 0.00946499969937444*G0_0_2 + 0.0117431758056757*G0_1_0 + 0.0117431758056756*G0_1_1 + 0.00946499969937442*G0_1_2 + 0.0117431758056757*G0_2_0 + 0.0117431758056756*G0_2_1 + 0.00946499969937437*G0_2_2;
    A[51] = 0.0189195610202551*G0_0_0 + 0.0189195610202552*G0_0_1 + 0.013742125287264*G0_0_2 + 0.0189195610202551*G0_1_0 + 0.0189195610202553*G0_1_1 + 0.0137421252872641*G0_1_2 + 0.0189195610202551*G0_2_0 + 0.0189195610202553*G0_2_1 + 0.0137421252872641*G0_2_2;
    A[52] = -0.0782878387045049*G0_0_0 - 0.0782878387045047*G0_0_1 - 0.0782878387045045*G0_0_2 - 0.0782878387045048*G0_1_0 - 0.0782878387045048*G0_1_1 - 0.0782878387045046*G0_1_2 - 0.0782878387045048*G0_2_0 - 0.0782878387045047*G0_2_1 - 0.0782878387045045*G0_2_2;
    A[53] = 0.0440369092712839*G0_0_0 + 0.0440369092712836*G0_0_1 + 0.0440369092712836*G0_0_2 + 0.0440369092712839*G0_1_0 + 0.0440369092712835*G0_1_1 + 0.0440369092712835*G0_1_2 + 0.0440369092712838*G0_2_0 + 0.0440369092712835*G0_2_1 + 0.0440369092712836*G0_2_2;
    A[54] = 0.0440369092712839*G0_0_0 + 0.0440369092712838*G0_0_1 + 0.0440369092712839*G0_0_2 + 0.0440369092712839*G0_1_0 + 0.0440369092712839*G0_1_1 + 0.0440369092712839*G0_1_2 + 0.0440369092712839*G0_2_0 + 0.0440369092712838*G0_2_1 + 0.0440369092712839*G0_2_2;
    A[55] = 0.0440369092712837*G0_0_0 + 0.0440369092712839*G0_0_1 + 0.0440369092712839*G0_0_2 + 0.0440369092712838*G0_1_0 + 0.0440369092712839*G0_1_1 + 0.0440369092712839*G0_1_2 + 0.0440369092712838*G0_2_0 + 0.0440369092712839*G0_2_1 + 0.0440369092712838*G0_2_2;
    A[56] = -0.00277890512265508*G0_0_0 - 0.00277890512265509*G0_0_1 - 0.0027789051226551*G0_0_2;
    A[57] = 0.0262652567340065*G0_0_0;
    A[58] = 0.00277890512265513*G0_0_1;
    A[59] = 0.00277890512265514*G0_0_2;
    A[60] = 0.00480034930989771*G0_0_1 + 0.00480034930989782*G0_0_2;
    A[61] = 0.00336507226698213*G0_0_1 + 0.00336507226698192*G0_0_2;
    A[62] = 0.00336507226698184*G0_0_1 + 0.00336507226698196*G0_0_2;
    A[63] = 0.00480034930989798*G0_0_1 + 0.00480034930989792*G0_0_2;
    A[64] = -0.0178705039816151*G0_0_0 + 0.0372571824394737*G0_0_2;
    A[65] = 0.0214691349673988*G0_0_0 - 0.0336585514536901*G0_0_2;
    A[66] = -0.0155544887532735*G0_0_0 + 0.0232632312710435*G0_0_2;
    A[67] = 0.00480034930989798*G0_0_0 - 0.0105310191030676*G0_0_2;
    A[68] = -0.017870503981615*G0_0_0 + 0.0372571824394737*G0_0_1;
    A[69] = 0.0214691349673986*G0_0_0 - 0.0336585514536901*G0_0_1;
    A[70] = -0.0155544887532734*G0_0_0 + 0.0232632312710436*G0_0_1;
    A[71] = 0.00480034930989781*G0_0_0 - 0.0105310191030677*G0_0_1;
    A[72] = -0.00480034930989787*G0_0_0 - 0.00480034930989784*G0_0_1;
    A[73] = -0.00336507226698203*G0_0_0 - 0.00336507226698207*G0_0_1;
    A[74] = -0.0033650722669819*G0_0_0 - 0.00336507226698188*G0_0_1;
    A[75] = -0.00480034930989798*G0_0_0 - 0.00480034930989797*G0_0_1;
    A[76] = -0.00480034930989794*G0_0_0 - 0.00480034930989794*G0_0_2;
    A[77] = -0.00336507226698186*G0_0_0 - 0.00336507226698188*G0_0_2;
    A[78] = -0.00336507226698191*G0_0_0 - 0.00336507226698194*G0_0_2;
    A[79] = -0.00480034930989785*G0_0_0 - 0.00480034930989782*G0_0_2;
    A[80] = 0.0153313684129654*G0_0_0 + 0.0105310191030675*G0_0_1 + 0.0105310191030675*G0_0_2;
    A[81] = -0.0388177200243168*G0_0_0 - 0.0232632312710434*G0_0_1 - 0.0232632312710435*G0_0_2;
    A[82] = 0.0551276864210887*G0_0_0 + 0.0336585514536898*G0_0_1 + 0.0336585514536898*G0_0_2;
    A[83] = -0.0551276864210887*G0_0_0 - 0.0372571824394737*G0_0_1 - 0.0372571824394737*G0_0_2;
    A[84] = 0.0479930547204846*G0_0_0 + 0.0143945239431345*G0_0_1 + 0.0143945239431346*G0_0_2;
    A[85] = -0.0345719095719089*G0_0_0 - 0.00472467106581677*G0_0_1 - 0.00472467106581691*G0_0_2;
    A[86] = 0.0137421252872637*G0_0_0 - 0.00517743573299127*G0_0_1 - 0.00517743573299154*G0_0_2;
    A[87] = -0.0345719095719095*G0_0_0 - 0.00472467106581712*G0_0_1 - 0.00472467106581699*G0_0_2;
    A[88] = 0.00946499969937495*G0_0_0 - 0.00227817610630087*G0_0_1 - 0.00227817610630055*G0_0_2;
    A[89] = 0.0137421252872639*G0_0_0 - 0.00517743573299143*G0_0_1 - 0.0051774357329916*G0_0_2;
    A[90] = -0.013742125287264*G0_0_0;
    A[91] = -0.00946499969937473*G0_0_0;
    A[92] = -0.0137421252872637*G0_0_0;
    A[93] = -0.00946499969937443*G0_0_0;
    A[94] = -0.00946499969937494*G0_0_0;
    A[95] = -0.013742125287264*G0_0_0;
    A[96] = 0.0189195610202553*G0_0_0 + 0.00517743573299118*G0_0_1;
    A[97] = -0.0298472385060925*G0_0_0 + 0.00472467106581688*G0_0_1;
    A[98] = 0.0335985307773502*G0_0_0 - 0.0143945239431346*G0_0_1;
    A[99] = 0.011743175805676*G0_0_0 + 0.00227817610630144*G0_0_1;
    A[100] = -0.0298472385060927*G0_0_0 + 0.00472467106581712*G0_0_1;
    A[101] = 0.0189195610202554*G0_0_0 + 0.00517743573299142*G0_0_1;
    A[102] = 0.0189195610202554*G0_0_0 + 0.00517743573299127*G0_0_2;
    A[103] = -0.0298472385060925*G0_0_0 + 0.00472467106581677*G0_0_2;
    A[104] = 0.03359853077735*G0_0_0 - 0.0143945239431347*G0_0_2;
    A[105] = 0.0117431758056756*G0_0_0 + 0.00227817610630104*G0_0_2;
    A[106] = -0.0298472385060922*G0_0_0 + 0.00472467106581689*G0_0_2;
    A[107] = 0.0189195610202553*G0_0_0 + 0.00517743573299156*G0_0_2;
    A[108] = 0.044036909271284*G0_0_0;
    A[109] = -0.0782878387045046*G0_0_0;
    A[110] = 0.0440369092712837*G0_0_0;
    A[111] = 0.044036909271284*G0_0_0;
    A[112] = -0.00277890512265506*G0_1_0 - 0.00277890512265506*G0_1_1 - 0.00277890512265506*G0_1_2;
    A[113] = 0.00277890512265513*G0_1_0;
    A[114] = 0.0262652567340065*G0_1_1;
    A[115] = 0.00277890512265513*G0_1_2;
    A[116] = -0.017870503981615*G0_1_1 + 0.0372571824394737*G0_1_2;
    A[117] = 0.0214691349673987*G0_1_1 - 0.0336585514536901*G0_1_2;
    A[118] = -0.0155544887532734*G0_1_1 + 0.0232632312710437*G0_1_2;
    A[119] = 0.00480034930989799*G0_1_1 - 0.0105310191030676*G0_1_2;
    A[120] = 0.0048003493098979*G0_1_0 + 0.00480034930989801*G0_1_2;
    A[121] = 0.00336507226698197*G0_1_0 + 0.00336507226698185*G0_1_2;
    A[122] = 0.00336507226698181*G0_1_0 + 0.00336507226698191*G0_1_2;
    A[123] = 0.00480034930989801*G0_1_0 + 0.00480034930989791*G0_1_2;
    A[124] = -0.0105310191030675*G0_1_0 + 0.00480034930989802*G0_1_1;
    A[125] = 0.0232632312710434*G0_1_0 - 0.0155544887532734*G0_1_1;
    A[126] = -0.03365855145369*G0_1_0 + 0.0214691349673987*G0_1_1;
    A[127] = 0.0372571824394737*G0_1_0 - 0.017870503981615*G0_1_1;
    A[128] = -0.00480034930989791*G0_1_0 - 0.00480034930989788*G0_1_1;
    A[129] = -0.00336507226698199*G0_1_0 - 0.00336507226698202*G0_1_1;
    A[130] = -0.003365072266982*G0_1_0 - 0.00336507226698196*G0_1_1;
    A[131] = -0.004800349309898*G0_1_0 - 0.00480034930989803*G0_1_1;
    A[132] = 0.0105310191030676*G0_1_0 + 0.0153313684129654*G0_1_1 + 0.0105310191030676*G0_1_2;
    A[133] = -0.0232632312710435*G0_1_0 - 0.0388177200243168*G0_1_1 - 0.0232632312710435*G0_1_2;
    A[134] = 0.0336585514536898*G0_1_0 + 0.0551276864210886*G0_1_1 + 0.0336585514536899*G0_1_2;
    A[135] = -0.0372571824394737*G0_1_0 - 0.0551276864210887*G0_1_1 - 0.0372571824394737*G0_1_2;
    A[136] = -0.00480034930989781*G0_1_1 - 0.00480034930989785*G0_1_2;
    A[137] = -0.00336507226698204*G0_1_1 - 0.003365072266982*G0_1_2;
    A[138] = -0.00336507226698195*G0_1_1 - 0.00336507226698194*G0_1_2;
    A[139] = -0.004800349309898*G0_1_1 - 0.00480034930989802*G0_1_2;
    A[140] = -0.00517743573299117*G0_1_0 + 0.0137421252872642*G0_1_1 - 0.00517743573299102*G0_1_2;
    A[141] = -0.00472467106581674*G0_1_0 - 0.0345719095719092*G0_1_1 - 0.00472467106581695*G0_1_2;
    A[142] = 0.0143945239431347*G0_1_0 + 0.0479930547204847*G0_1_1 + 0.0143945239431348*G0_1_2;
    A[143] = -0.00227817610630117*G0_1_0 + 0.00946499969937431*G0_1_1 - 0.00227817610630119*G0_1_2;
    A[144] = -0.0047246710658168*G0_1_0 - 0.0345719095719091*G0_1_1 - 0.00472467106581681*G0_1_2;
    A[145] = -0.00517743573299099*G0_1_0 + 0.0137421252872642*G0_1_1 - 0.00517743573299102*G0_1_2;
    A[146] = 0.00517743573299116*G0_1_0 + 0.0189195610202553*G0_1_1;
    A[147] = 0.00472467106581679*G0_1_0 - 0.0298472385060925*G0_1_1;
    A[148] = -0.0143945239431347*G0_1_0 + 0.0335985307773502*G0_1_1;
    A[149] = 0.00227817610630115*G0_1_0 + 0.0117431758056758*G0_1_1;
    A[150] = 0.0047246710658168*G0_1_0 - 0.0298472385060926*G0_1_1;
    A[151] = 0.005177435732991*G0_1_0 + 0.0189195610202554*G0_1_1;
    A[152] = -0.0137421252872642*G0_1_1;
    A[153] = -0.00946499969937454*G0_1_1;
    A[154] = -0.0137421252872643*G0_1_1;
    A[155] = -0.00946499969937459*G0_1_1;
    A[156] = -0.00946499969937429*G0_1_1;
    A[157] = -0.0137421252872642*G0_1_1;
    A[158] = 0.0189195610202552*G0_1_1 + 0.00517743573299122*G0_1_2;
    A[159] = 0.0117431758056759*G0_1_1 + 0.0022781761063012*G0_1_2;
    A[160] = 0.0189195610202554*G0_1_1 + 0.00517743573299104*G0_1_2;
    A[161] = -0.0298472385060927*G0_1_1 + 0.00472467106581664*G0_1_2;
    A[162] = -0.0298472385060925*G0_1_1 + 0.00472467106581695*G0_1_2;
    A[163] = 0.0335985307773502*G0_1_1 - 0.0143945239431348*G0_1_2;
    A[164] = 0.0440369092712842*G0_1_1;
    A[165] = 0.0440369092712839*G0_1_1;
    A[166] = -0.0782878387045047*G0_1_1;
    A[167] = 0.0440369092712835*G0_1_1;
    A[168] = -0.00277890512265513*G0_2_0 - 0.00277890512265514*G0_2_1 - 0.00277890512265513*G0_2_2;
    A[169] = 0.00277890512265514*G0_2_0;
    A[170] = 0.00277890512265513*G0_2_1;
    A[171] = 0.0262652567340067*G0_2_2;
    A[172] = -0.0105310191030678*G0_2_1 + 0.00480034930989789*G0_2_2;
    A[173] = 0.0232632312710439*G0_2_1 - 0.0155544887532733*G0_2_2;
    A[174] = -0.0336585514536905*G0_2_1 + 0.0214691349673987*G0_2_2;
    A[175] = 0.0372571824394742*G0_2_1 - 0.0178705039816148*G0_2_2;
    A[176] = -0.0105310191030678*G0_2_0 + 0.00480034930989798*G0_2_2;
    A[177] = 0.023263231271044*G0_2_0 - 0.0155544887532735*G0_2_2;
    A[178] = -0.0336585514536904*G0_2_0 + 0.0214691349673989*G0_2_2;
    A[179] = 0.0372571824394742*G0_2_0 - 0.0178705039816149*G0_2_2;
    A[180] = 0.00480034930989791*G0_2_0 + 0.00480034930989797*G0_2_1;
    A[181] = 0.00336507226698193*G0_2_0 + 0.00336507226698188*G0_2_1;
    A[182] = 0.00336507226698199*G0_2_0 + 0.00336507226698204*G0_2_1;
    A[183] = 0.00480034930989787*G0_2_0 + 0.0048003493098978*G0_2_1;
    A[184] = 0.0105310191030678*G0_2_0 + 0.0105310191030679*G0_2_1 + 0.0153313684129657*G0_2_2;
    A[185] = -0.0232632312710439*G0_2_0 - 0.023263231271044*G0_2_1 - 0.0388177200243173*G0_2_2;
    A[186] = 0.0336585514536905*G0_2_0 + 0.0336585514536904*G0_2_1 + 0.0551276864210891*G0_2_2;
    A[187] = -0.0372571824394742*G0_2_0 - 0.0372571824394742*G0_2_1 - 0.0551276864210891*G0_2_2;
    A[188] = -0.00480034930989783*G0_2_0 - 0.00480034930989785*G0_2_2;
    A[189] = -0.00336507226698201*G0_2_0 - 0.00336507226698199*G0_2_2;
    A[190] = -0.00336507226698191*G0_2_0 - 0.0033650722669819*G0_2_2;
    A[191] = -0.00480034930989786*G0_2_0 - 0.00480034930989786*G0_2_2;
    A[192] = -0.0048003493098978*G0_2_1 - 0.00480034930989781*G0_2_2;
    A[193] = -0.00336507226698199*G0_2_1 - 0.00336507226698197*G0_2_2;
    A[194] = -0.00336507226698197*G0_2_1 - 0.00336507226698196*G0_2_2;
    A[195] = -0.00480034930989794*G0_2_1 - 0.00480034930989794*G0_2_2;
    A[196] = -0.00517743573299109*G0_2_0 - 0.00517743573299115*G0_2_1 + 0.0137421252872641*G0_2_2;
    A[197] = -0.00227817610630099*G0_2_0 - 0.00227817610630108*G0_2_1 + 0.00946499969937458*G0_2_2;
    A[198] = -0.0051774357329912*G0_2_0 - 0.005177435732991*G0_2_1 + 0.0137421252872641*G0_2_2;
    A[199] = -0.00472467106581704*G0_2_0 - 0.00472467106581676*G0_2_1 - 0.0345719095719091*G0_2_2;
    A[200] = -0.00472467106581685*G0_2_0 - 0.00472467106581711*G0_2_1 - 0.0345719095719092*G0_2_2;
    A[201] = 0.0143945239431351*G0_2_0 + 0.0143945239431351*G0_2_1 + 0.0479930547204847*G0_2_2;
    A[202] = 0.00517743573299101*G0_2_0 + 0.0189195610202551*G0_2_2;
    A[203] = 0.00227817610630117*G0_2_0 + 0.0117431758056758*G0_2_2;
    A[204] = 0.00517743573299117*G0_2_0 + 0.0189195610202552*G0_2_2;
    A[205] = 0.00472467106581727*G0_2_0 - 0.029847238506092*G0_2_2;
    A[206] = 0.00472467106581687*G0_2_0 - 0.0298472385060923*G0_2_2;
    A[207] = -0.0143945239431351*G0_2_0 + 0.0335985307773495*G0_2_2;
    A[208] = 0.00517743573299099*G0_2_1 + 0.0189195610202551*G0_2_2;
    A[209] = 0.00227817610630123*G0_2_1 + 0.0117431758056757*G0_2_2;
    A[210] = 0.00517743573299109*G0_2_1 + 0.0189195610202553*G0_2_2;
    A[211] = 0.00472467106581737*G0_2_1 - 0.029847238506092*G0_2_2;
    A[212] = 0.00472467106581677*G0_2_1 - 0.0298472385060926*G0_2_2;
    A[213] = -0.0143945239431352*G0_2_1 + 0.0335985307773495*G0_2_2;
    A[214] = -0.0137421252872641*G0_2_2;
    A[215] = -0.0094649996993747*G0_2_2;
    A[216] = -0.0137421252872641*G0_2_2;
    A[217] = -0.00946499969937468*G0_2_2;
    A[218] = -0.00946499969937467*G0_2_2;
    A[219] = -0.0137421252872641*G0_2_2;
    A[220] = 0.0440369092712843*G0_2_2;
    A[221] = 0.0440369092712839*G0_2_2;
    A[222] = 0.0440369092712839*G0_2_2;
    A[223] = -0.0782878387045049*G0_2_2;
    A[224] = -0.00480034930989788*G0_1_0 - 0.00480034930989788*G0_1_1 - 0.00480034930989788*G0_1_2 - 0.00480034930989781*G0_2_0 - 0.00480034930989782*G0_2_1 - 0.00480034930989781*G0_2_2;
    A[225] = 0.00480034930989771*G0_1_0 + 0.00480034930989782*G0_2_0;
    A[226] = -0.017870503981615*G0_1_1 + 0.0372571824394737*G0_2_1;
    A[227] = -0.0105310191030678*G0_1_2 + 0.00480034930989789*G0_2_2;
    A[228] = 0.179096479009671*G0_1_1 + 0.0699110399631219*G0_1_2 + 0.0699110399631219*G0_2_1 + 0.139822079926245*G0_2_2;
    A[229] = -0.147063705006412*G0_1_1 + 0.0263830016434183*G0_1_2 - 0.0387263842124946*G0_2_1 - 0.108637424175618*G0_2_2;
    A[230] = 0.100717304493345*G0_1_1 - 0.0246345732456843*G0_1_2 + 0.0189782768992836*G0_2_1 + 0.0788945694544649*G0_2_2;
    A[231] = -0.0328352243499816*G0_1_1 + 0.0209159009072198*G0_1_2 - 0.00537576492437589*G0_2_1 - 0.0328352243499812*G0_2_2;
    A[232] = -0.0107515298487513*G0_1_0 - 0.00537576492437592*G0_1_2 - 0.00537576492437577*G0_2_0 - 0.00537576492437583*G0_2_2;
    A[233] = -0.0139743792087546*G0_1_0 - 0.011684459926647*G0_1_2 - 0.00228991928210695*G0_2_0 - 0.00228991928210671*G0_2_2;
    A[234] = -0.0111690649885085*G0_1_0 - 0.0088791457064023*G0_1_2 - 0.00228991928210673*G0_2_0 - 0.00228991928210687*G0_2_2;
    A[235] = -0.0328352243499816*G0_1_0 - 0.0274594594256052*G0_1_2 - 0.00537576492437587*G0_2_0 - 0.00537576492437588*G0_2_2;
    A[236] = -0.0274594594256046*G0_1_0 - 0.0053757649243759*G0_1_1 - 0.0328352243499806*G0_2_0 - 0.00537576492437581*G0_2_1;
    A[237] = 0.0599162925551805*G0_1_0 + 0.0283728175438238*G0_1_1 + 0.078894569454464*G0_2_0 + 0.0189782768992834*G0_2_1;
    A[238] = -0.069911039963122*G0_1_0 - 0.0574176057031946*G0_1_1 - 0.108637424175617*G0_2_0 - 0.0387263842124945*G0_2_1;
    A[239] = 0.069911039963122*G0_1_0 + 0.0895482395048353*G0_1_1 + 0.139822079926245*G0_2_0 + 0.0699110399631223*G0_2_1;
    A[240] = 0.0107515298487517*G0_1_0 + 0.0107515298487517*G0_1_1 + 0.00537576492437594*G0_1_2 + 0.00537576492437579*G0_2_0 + 0.00537576492437584*G0_2_1;
    A[241] = 0.0139743792087547*G0_1_0 + 0.0139743792087547*G0_1_1 + 0.002289919282107*G0_1_2 + 0.00228991928210708*G0_2_0 + 0.00228991928210699*G0_2_1;
    A[242] = 0.0111690649885089*G0_1_0 + 0.0111690649885089*G0_1_1 + 0.00228991928210651*G0_1_2 + 0.00228991928210654*G0_2_0 + 0.00228991928210658*G0_2_1;
    A[243] = 0.0328352243499816*G0_1_0 + 0.0328352243499817*G0_1_1 + 0.00537576492437623*G0_1_2 + 0.00537576492437588*G0_2_0 + 0.0053757649243759*G0_2_1;
    A[244] = 0.0274594594256046*G0_1_0 + 0.022083694501229*G0_1_1 + 0.0274594594256046*G0_1_2 + 0.0328352243499807*G0_2_0 + 0.0274594594256048*G0_2_1 + 0.0328352243499807*G0_2_2;
    A[245] = -0.0599162925551803*G0_1_0 - 0.0315434750113566*G0_1_1 - 0.0599162925551804*G0_1_2 - 0.078894569454464*G0_2_0 - 0.0599162925551806*G0_2_1 - 0.0788945694544642*G0_2_2;
    A[246] = 0.0699110399631223*G0_1_0 + 0.0124934342599272*G0_1_1 + 0.0699110399631223*G0_1_2 + 0.108637424175617*G0_2_0 + 0.0699110399631223*G0_2_1 + 0.108637424175617*G0_2_2;
    A[247] = -0.069911039963122*G0_1_0 + 0.0196371995417132*G0_1_1 - 0.069911039963122*G0_1_2 - 0.139822079926245*G0_2_0 - 0.0699110399631225*G0_2_1 - 0.139822079926245*G0_2_2;
    A[248] = 0.00537576492437593*G0_1_1 + 0.00537576492437597*G0_1_2 + 0.00537576492437606*G0_2_1 + 0.00537576492437603*G0_2_2;
    A[249] = 0.00228991928210655*G0_1_1 + 0.00228991928210649*G0_1_2 + 0.00228991928210661*G0_2_1 + 0.00228991928210662*G0_2_2;
    A[250] = 0.00228991928210687*G0_1_1 + 0.00228991928210684*G0_1_2 + 0.00228991928210689*G0_2_1 + 0.0022899192821069*G0_2_2;
    A[251] = 0.00537576492437586*G0_1_1 + 0.00537576492437586*G0_1_2 + 0.00537576492437583*G0_2_1 + 0.00537576492437577*G0_2_2;
    A[252] = 0.0425363923627802*G0_1_0 - 0.0187890812890807*G0_1_1 + 0.0212681961813905*G0_1_2 + 0.0118736555368497*G0_2_0 - 0.00939454064454018*G0_2_1 + 0.0118736555368502*G0_2_2;
    A[253] = -0.0845508658008646*G0_1_0 + 0.0439716694056964*G0_1_1 - 0.042275432900432*G0_1_2 - 0.0429604514890966*G0_2_0 - 0.000685018588664287*G0_2_1 - 0.0429604514890967*G0_2_2;
    A[254] = 0.19258808321308*G0_1_0 - 0.0644569872000428*G0_1_1 + 0.0962940416065399*G0_1_2 + 0.124738623002509*G0_2_0 + 0.028444581395969*G0_2_1 + 0.12473862300251*G0_2_2;
    A[255] = 0.0142875305635734*G0_1_0 - 0.0292600797158079*G0_1_1 + 0.00844856259352766*G0_1_2 + 0.00254435475789676*G0_2_0 - 0.00329461321214826*G0_2_1 + 0.00254435475789598*G0_2_2;
    A[256] = -0.0493213383838381*G0_1_0 + 0.127315597693201*G0_1_1 - 0.00704590548340609*G0_1_2 - 0.0429604514890967*G0_2_0 - 0.000685018588664125*G0_2_1 - 0.0429604514890974*G0_2_2;
    A[257] = 0.0477555816097472*G0_1_0 - 0.0641307878721069*G0_1_1 + 0.0264873854283582*G0_1_2 + 0.0118736555368503*G0_2_0 - 0.00939454064454004*G0_2_1 + 0.011873655536851*G0_2_2;
    A[258] = -0.0425363923627799*G0_1_0 - 0.0613254736518614*G0_1_1 - 0.0212681961813895*G0_1_2 - 0.0118736555368494*G0_2_0 - 0.0212681961813902*G0_2_1;
    A[259] = 0.0845508658008643*G0_1_0 + 0.128522535206561*G0_1_1 + 0.042275432900432*G0_1_2 + 0.0429604514890963*G0_2_0 + 0.0422754329004323*G0_2_1;
    A[260] = -0.19258808321308*G0_1_0 - 0.257045070413123*G0_1_1 - 0.0962940416065404*G0_1_2 - 0.124738623002509*G0_2_0 - 0.09629404160654*G0_2_1;
    A[261] = -0.0142875305635724*G0_1_0 - 0.0435476102793813*G0_1_1 - 0.0058389679700452*G0_1_2 - 0.00254435475789624*G0_2_0 - 0.00583896797004454*G0_2_1;
    A[262] = 0.0493213383838381*G0_1_0 + 0.176636936077039*G0_1_1 + 0.0422754329004329*G0_1_2 + 0.0429604514890967*G0_2_0 + 0.0422754329004323*G0_2_1;
    A[263] = -0.0477555816097472*G0_1_0 - 0.111886369481854*G0_1_1 - 0.0212681961813899*G0_1_2 - 0.0118736555368503*G0_2_0 - 0.0212681961813901*G0_2_1;
    A[264] = 0.0187890812890812*G0_1_1 + 0.00939454064454083*G0_1_2 + 0.00939454064454066*G0_2_1;
    A[265] = 0.00658922642429559*G0_1_1 + 0.0032946132121482*G0_1_2 + 0.00329461321214772*G0_2_1;
    A[266] = 0.0187890812890809*G0_1_1 + 0.00939454064454028*G0_1_2 + 0.00939454064454016*G0_2_1;
    A[267] = 0.029260079715809*G0_1_1 + 0.00329461321214747*G0_1_2 + 0.0032946132121484*G0_2_1;
    A[268] = 0.0292600797158079*G0_1_1 + 0.00329461321214762*G0_1_2 + 0.00329461321214829*G0_2_1;
    A[269] = 0.0641307878721066*G0_1_1 + 0.00939454064454104*G0_1_2 + 0.00939454064454004*G0_2_1;
    A[270] = -0.0306627368259309*G0_1_1 - 0.0212681961813905*G0_1_2 - 0.0212681961813905*G0_2_1 - 0.0118736555368498*G0_2_2;
    A[271] = -0.00913358118219204*G0_1_1 - 0.00583896797004444*G0_1_2 - 0.00583896797004399*G0_2_1 - 0.00254435475789631*G0_2_2;
    A[272] = -0.0306627368259308*G0_1_1 - 0.0212681961813904*G0_1_2 - 0.0212681961813904*G0_2_1 - 0.0118736555368501*G0_2_2;
    A[273] = 0.0642612676032812*G0_1_1 + 0.0422754329004328*G0_1_2 + 0.0422754329004324*G0_2_1 + 0.0429604514890965*G0_2_2;
    A[274] = 0.0642612676032802*G0_1_1 + 0.0422754329004323*G0_1_2 + 0.042275432900432*G0_2_1 + 0.0429604514890969*G0_2_2;
    A[275] = -0.128522535206561*G0_1_1 - 0.0962940416065399*G0_1_2 - 0.0962940416065401*G0_2_1 - 0.12473862300251*G0_2_2;
    A[276] = -0.0505608958299931*G0_1_1 - 0.00260959462348333*G0_1_2 - 0.00260959462348382*G0_2_1;
    A[277] = -0.0505608958299922*G0_1_1 - 0.00260959462348356*G0_1_2 - 0.00260959462348341*G0_2_1;
    A[278] = 0.0962288017409547*G0_1_1 - 0.0352295274170272*G0_1_2 - 0.0352295274170257*G0_2_1;
    A[279] = -0.15657567740901*G0_1_1 - 0.00260959462348417*G0_1_2 - 0.00260959462348456*G0_2_1;
    A[280] = -0.00336507226698201*G0_1_0 - 0.003365072266982*G0_1_1 - 0.003365072266982*G0_1_2 - 0.00336507226698203*G0_2_0 - 0.00336507226698201*G0_2_1 - 0.00336507226698202*G0_2_2;
    A[281] = 0.00336507226698213*G0_1_0 + 0.00336507226698192*G0_2_0;
    A[282] = 0.0214691349673987*G0_1_1 - 0.0336585514536901*G0_2_1;
    A[283] = 0.0232632312710439*G0_1_2 - 0.0155544887532733*G0_2_2;
    A[284] = -0.147063705006412*G0_1_1 - 0.0387263842124946*G0_1_2 + 0.0263830016434183*G0_2_1 - 0.108637424175618*G0_2_2;
    A[285] = 0.304226541205706*G0_1_1 + 0.0781312630270955*G0_1_2 + 0.0781312630270955*G0_2_1 + 0.281001149056703*G0_2_2;
    A[286] = -0.204187731314467*G0_1_1 + 0.0153052724667309*G0_1_2 - 0.0364951808094168*G0_2_1 - 0.204187731314466*G0_2_2;
    A[287] = 0.0788945694544659*G0_1_1 - 0.0246345732456837*G0_1_2 + 0.0189782768992835*G0_2_1 + 0.100717304493345*G0_2_2;
    A[288] = -0.0139743792087548*G0_1_0 - 0.00228991928210677*G0_1_2 - 0.0116844599266471*G0_2_0 - 0.00228991928210676*G0_2_2;
    A[289] = 0.0263047138047148*G0_1_0 + 0.0131523569023568*G0_1_2 + 0.0131523569023569*G0_2_0 + 0.013217596767944*G0_2_2;
    A[290] = 0.00523223722008297*G0_1_0 + 0.00578025209101563*G0_1_2 - 0.000548014870931329*G0_2_0 - 0.00048277500534406*G0_2_2;
    A[291] = 0.0788945694544659*G0_1_0 + 0.0599162925551812*G0_1_2 + 0.0189782768992834*G0_2_0 + 0.0283728175438239*G0_2_2;
    A[292] = -0.00887914570640286*G0_1_0 - 0.00228991928210689*G0_1_1 - 0.0111690649885094*G0_2_0 - 0.00228991928210686*G0_2_1;
    A[293] = 0.00578025209101597*G0_1_0 - 0.000482775005344236*G0_1_1 + 0.00523223722008435*G0_2_0 - 0.00054801487093142*G0_2_1;
    A[294] = 0.00480165410720929*G0_1_0 + 0.00506261356955741*G0_1_1 + 0.00960330821441927*G0_2_0 + 0.00480165410720954*G0_2_1;
    A[295] = -0.0387263842124947*G0_1_0 - 0.0574176057031952*G0_1_1 - 0.108637424175618*G0_2_0 - 0.0699110399631227*G0_2_1;
    A[296] = 0.0139743792087547*G0_1_0 + 0.0139743792087547*G0_1_1 + 0.0116844599266477*G0_1_2 + 0.0116844599266476*G0_2_0 + 0.0116844599266475*G0_2_1 + 0.00939454064454094*G0_2_2;
    A[297] = -0.0263047138047148*G0_1_0 - 0.0263047138047148*G0_1_1 - 0.0131523569023576*G0_1_2 - 0.0131523569023572*G0_2_0 - 0.0131523569023572*G0_2_1 + 6.52398655864274e-05*G0_2_2;
    A[298] = -0.00523223722008331*G0_1_0 - 0.00523223722008316*G0_1_1 + 0.000548014870932251*G0_1_2 + 0.000548014870931637*G0_2_0 + 0.000548014870931732*G0_2_1 + 6.52398655876953e-05*G0_2_2;
    A[299] = -0.0788945694544658*G0_1_0 - 0.0788945694544659*G0_1_1 - 0.0189782768992843*G0_1_2 - 0.0189782768992834*G0_2_0 - 0.0189782768992835*G0_2_1 + 0.00939454064454039*G0_2_2;
    A[300] = 0.00887914570640293*G0_1_0 + 0.00658922642429573*G0_1_1 + 0.00887914570640286*G0_1_2 + 0.0111690649885094*G0_2_0 + 0.00887914570640258*G0_2_1 + 0.0111690649885094*G0_2_2;
    A[301] = -0.0057802520910159*G0_1_0 - 0.00626302709635985*G0_1_1 - 0.00578025209101586*G0_1_2 - 0.00523223722008423*G0_2_0 - 0.00578025209101556*G0_2_1 - 0.00523223722008417*G0_2_2;
    A[302] = -0.00480165410720964*G0_1_0 + 0.0002609594623476*G0_1_1 - 0.00480165410720962*G0_1_2 - 0.00960330821441909*G0_2_0 - 0.00480165410721001*G0_2_1 - 0.00960330821441906*G0_2_2;
    A[303] = 0.0387263842124947*G0_1_0 - 0.0186912214907001*G0_1_1 + 0.0387263842124946*G0_1_2 + 0.108637424175618*G0_2_0 + 0.0387263842124952*G0_2_1 + 0.108637424175618*G0_2_2;
    A[304] = 0.00228991928210721*G0_1_1 + 0.00228991928210718*G0_1_2 + 0.00228991928210695*G0_2_1 + 0.00228991928210698*G0_2_2;
    A[305] = 0.00178757231708625*G0_1_1 + 0.00178757231708628*G0_1_2 + 0.00178757231708599*G0_2_1 + 0.00178757231708594*G0_2_2;
    A[306] = 0.00178757231708605*G0_1_1 + 0.00178757231708613*G0_1_2 + 0.00178757231708597*G0_2_1 + 0.00178757231708599*G0_2_2;
    A[307] = 0.00228991928210687*G0_1_1 + 0.00228991928210684*G0_1_2 + 0.00228991928210681*G0_2_1 + 0.00228991928210682*G0_2_2;
    A[308] = 0.0403182369328203*G0_1_0 + 0.0148746893538559*G0_1_1 + 0.00123955744615451*G0_1_2 + 0.0540186087061081*G0_2_0 + 0.0149399292194433*G0_2_1 + 0.00117431758056757*G0_2_2;
    A[309] = -0.0808321934624015*G0_1_0 - 0.0436454700777622*G0_1_1 - 0.00649136662591609*G0_1_2 - 0.108232937008978*G0_2_0 - 0.0338921101724931*G0_2_1 - 0.00352295274170327*G0_2_2;
    A[310] = 0.0109602974186317*G0_1_0 + 0.132306447410614*G0_1_1 - 0.0623693115012546*G0_1_2 + 0.146659217839773*G0_2_0 + 0.0733296089198877*G0_2_1 - 0.12473862300251*G0_2_2;
    A[311] = -0.0377738821749255*G0_1_0 + 0.0189848008858414*G0_1_1 - 0.0188869410874622*G0_1_2 - 0.00939454064454051*G0_2_0 + 0.00949240044292173*G0_2_1 - 0.0155923278753139*G0_2_2;
    A[312] = 0.167731694424402*G0_1_0 - 0.166948816037357*G0_1_1 + 0.0838658472122013*G0_1_2 + 0.0777006799142204*G0_2_0 - 0.00616516729798032*G0_2_1 + 0.182410664181497*G0_2_2;
    A[313] = -0.0823327103709038*G0_1_0 + 0.127087258163646*G0_1_1 - 0.0463855444324205*G0_1_2 + 0.00182671623643726*G0_2_0 + 0.0377738821749228*G0_2_1 - 0.0510175748891035*G0_2_2;
    A[314] = -0.0403182369328212*G0_1_0 - 0.0254435475789651*G0_1_1 - 0.0390786794866666*G0_1_2 - 0.0540186087061088*G0_2_0 - 0.0390786794866656*G0_2_1 - 0.052844291125541*G0_2_2;
    A[315] = 0.0808321934624015*G0_1_0 + 0.0371867233846405*G0_1_1 + 0.0743408268364856*G0_1_2 + 0.108232937008978*G0_2_0 + 0.0743408268364859*G0_2_1 + 0.104709984267274*G0_2_2;
    A[316] = -0.0109602974186318*G0_1_0 + 0.121346149991982*G0_1_1 - 0.0733296089198861*G0_1_2 - 0.146659217839773*G0_2_0 - 0.0733296089198862*G0_2_1 - 0.271397840842284*G0_2_2;
    A[317] = 0.0377738821749239*G0_1_0 + 0.0567586830607668*G0_1_1 + 0.0188869410874636*G0_1_2 + 0.00939454064453991*G0_2_0 + 0.018886941087462*G0_2_1 - 0.00619778723077264*G0_2_2;
    A[318] = -0.167731694424401*G0_1_0 - 0.334680510461758*G0_1_1 - 0.0838658472122007*G0_1_2 - 0.0777006799142203*G0_2_0 - 0.0838658472122003*G0_2_1 + 0.104709984267276*G0_2_2;
    A[319] = 0.0823327103709037*G0_1_0 + 0.20941996853455*G0_1_1 + 0.0359471659384839*G0_1_2 - 0.00182671623643738*G0_2_0 + 0.0359471659384851*G0_2_1 - 0.0528442911255411*G0_2_2;
    A[320] = -0.0148746893538567*G0_1_1 - 0.0149399292194444*G0_1_2 - 0.0149399292194439*G0_2_1 - 0.0150051690850309*G0_2_2;
    A[321] = 0.00450155072550967*G0_1_1 + 0.00192457603481917*G0_1_2 + 0.00192457603481958*G0_2_1 - 0.000652398655870243*G0_2_2;
    A[322] = -0.0148746893538561*G0_1_1 - 0.014939929219443*G0_1_2 - 0.0149399292194433*G0_2_1 - 0.01500516908503*G0_2_2;
    A[323] = -0.0189848008858419*G0_1_1 - 0.00949240044291928*G0_1_2 - 0.00949240044292166*G0_2_1 - 0.000652398655870409*G0_2_2;
    A[324] = -0.0189848008858415*G0_1_1 - 0.00949240044292091*G0_1_2 - 0.00949240044292178*G0_2_1 - 0.000652398655871725*G0_2_2;
    A[325] = -0.127087258163646*G0_1_1 - 0.0377738821749251*G0_1_2 - 0.0377738821749227*G0_2_1 - 0.0150051690850297*G0_2_2;
    A[326] = -0.00130479731174253*G0_1_1 - 0.00123955744615478*G0_1_2 - 0.00123955744615509*G0_2_1 - 0.00117431758056822*G0_2_2;
    A[327] = -0.0156575677409007*G0_1_1 - 0.0130805930502108*G0_1_2 - 0.0130805930502106*G0_2_1 - 0.010503618359521*G0_2_2;
    A[328] = -0.0013047973117417*G0_1_1 - 0.0012395574461545*G0_1_2 - 0.00123955744615445*G0_2_1 - 0.00117431758056762*G0_2_2;
    A[329] = 0.0088073818542571*G0_1_1 + 0.00649136662591503*G0_1_2 + 0.00649136662591578*G0_2_1 + 0.0035229527417038*G0_2_2;
    A[330] = 0.00880738185425698*G0_1_1 + 0.00649136662591592*G0_1_2 + 0.00649136662591499*G0_2_1 + 0.00352295274170322*G0_2_2;
    A[331] = 0.0476251018785733*G0_1_1 + 0.0623693115012546*G0_1_2 + 0.0623693115012549*G0_2_1 + 0.12473862300251*G0_2_2;
    A[332] = 0.0391439193522537*G0_1_1 + 0.0319675341376736*G0_1_2 + 0.0319675341376744*G0_2_1 + 0.0260959462348357*G0_2_2;
    A[333] = 0.0391439193522526*G0_1_1 + 0.0319675341376731*G0_1_2 + 0.0319675341376732*G0_2_1 + 0.026095946234835*G0_2_2;
    A[334] = -0.156575677409011*G0_1_1 - 0.0903572138381159*G0_1_2 - 0.0903572138381182*G0_2_1 - 0.1859336169232*G0_2_2;
    A[335] = 0.185933616923199*G0_1_1 + 0.0156575677409014*G0_1_2 + 0.0156575677409021*G0_2_1 + 0.0260959462348356*G0_2_2;
    A[336] = -0.00336507226698186*G0_1_0 - 0.00336507226698184*G0_1_1 - 0.00336507226698183*G0_1_2 - 0.0033650722669819*G0_2_0 - 0.00336507226698188*G0_2_1 - 0.00336507226698186*G0_2_2;
    A[337] = 0.00336507226698184*G0_1_0 + 0.00336507226698196*G0_2_0;
    A[338] = -0.0155544887532734*G0_1_1 + 0.0232632312710437*G0_2_1;
    A[339] = -0.0336585514536905*G0_1_2 + 0.0214691349673987*G0_2_2;
    A[340] = 0.100717304493345*G0_1_1 + 0.0189782768992836*G0_1_2 - 0.0246345732456843*G0_2_1 + 0.0788945694544649*G0_2_2;
    A[341] = -0.204187731314467*G0_1_1 - 0.0364951808094168*G0_1_2 + 0.0153052724667309*G0_2_1 - 0.204187731314466*G0_2_2;
    A[342] = 0.281001149056703*G0_1_1 + 0.0781312630270956*G0_1_2 + 0.0781312630270956*G0_2_1 + 0.304226541205706*G0_2_2;
    A[343] = -0.108637424175619*G0_1_1 + 0.0263830016434176*G0_1_2 - 0.0387263842124946*G0_2_1 - 0.147063705006412*G0_2_2;
    A[344] = -0.0111690649885088*G0_1_0 - 0.00228991928210675*G0_1_2 - 0.00887914570640276*G0_2_0 - 0.00228991928210689*G0_2_2;
    A[345] = 0.005232237220083*G0_1_0 - 0.000548014870931652*G0_1_2 + 0.00578025209101584*G0_2_0 - 0.000482775005344332*G0_2_2;
    A[346] = 0.00960330821442051*G0_1_0 + 0.00480165410720968*G0_1_2 + 0.00480165410720933*G0_2_0 + 0.00506261356955739*G0_2_2;
    A[347] = -0.108637424175619*G0_1_0 - 0.0699110399631231*G0_1_2 - 0.0387263842124946*G0_2_0 - 0.057417605703195*G0_2_2;
    A[348] = -0.0116844599266473*G0_1_0 - 0.0022899192821067*G0_1_1 - 0.0139743792087543*G0_2_0 - 0.00228991928210682*G0_2_1;
    A[349] = 0.0131523569023569*G0_1_0 + 0.0132175967679436*G0_1_1 + 0.0263047138047138*G0_2_0 + 0.0131523569023566*G0_2_1;
    A[350] = -0.000548014870931448*G0_1_0 - 0.00048277500534389*G0_1_1 + 0.00523223722008417*G0_2_0 + 0.00578025209101611*G0_2_1;
    A[351] = 0.0189782768992836*G0_1_0 + 0.0283728175438239*G0_1_1 + 0.0788945694544649*G0_2_0 + 0.059916292555181*G0_2_1;
    A[352] = 0.0111690649885085*G0_1_0 + 0.0111690649885086*G0_1_1 + 0.00887914570640205*G0_1_2 + 0.00887914570640246*G0_2_0 + 0.00887914570640257*G0_2_1 + 0.0065892264242955*G0_2_2;
    A[353] = -0.00523223722008294*G0_1_0 - 0.0052322372200831*G0_1_1 - 0.00578025209101493*G0_1_2 - 0.00578025209101549*G0_2_0 - 0.00578025209101575*G0_2_1 - 0.00626302709635972*G0_2_2;
    A[354] = -0.00960330821442066*G0_1_0 - 0.00960330821442069*G0_1_1 - 0.00480165410721068*G0_1_2 - 0.00480165410720957*G0_2_0 - 0.00480165410720957*G0_2_1 + 0.000260959462347569*G0_2_2;
    A[355] = 0.108637424175619*G0_1_0 + 0.108637424175619*G0_1_1 + 0.0387263842124958*G0_1_2 + 0.0387263842124946*G0_2_0 + 0.0387263842124946*G0_2_1 - 0.0186912214907003*G0_2_2;
    A[356] = 0.0116844599266469*G0_1_0 + 0.00939454064454049*G0_1_1 + 0.011684459926647*G0_1_2 + 0.0139743792087541*G0_2_0 + 0.0116844599266472*G0_2_1 + 0.0139743792087541*G0_2_2;
    A[357] = -0.0131523569023565*G0_1_0 + 6.52398655869548e-05*G0_1_1 - 0.0131523569023566*G0_1_2 - 0.0263047138047136*G0_2_0 - 0.0131523569023568*G0_2_1 - 0.0263047138047136*G0_2_2;
    A[358] = 0.000548014870931397*G0_1_0 + 6.52398655874139e-05*G0_1_1 + 0.000548014870931399*G0_1_2 - 0.00523223722008419*G0_2_0 + 0.000548014870931552*G0_2_1 - 0.00523223722008423*G0_2_2;
    A[359] = -0.0189782768992836*G0_1_0 + 0.00939454064454025*G0_1_1 - 0.0189782768992835*G0_1_2 - 0.078894569454465*G0_2_0 - 0.0189782768992838*G0_2_1 - 0.0788945694544648*G0_2_2;
    A[360] = 0.00228991928210637*G0_1_1 + 0.0022899192821064*G0_1_2 + 0.0022899192821069*G0_2_1 + 0.00228991928210689*G0_2_2;
    A[361] = 0.0017875723170862*G0_1_1 + 0.00178757231708612*G0_1_2 + 0.00178757231708644*G0_2_1 + 0.00178757231708643*G0_2_2;
    A[362] = 0.00178757231708616*G0_1_1 + 0.0017875723170861*G0_1_2 + 0.00178757231708639*G0_2_1 + 0.00178757231708639*G0_2_2;
    A[363] = 0.0022899192821067*G0_1_1 + 0.00228991928210673*G0_1_2 + 0.00228991928210686*G0_2_1 + 0.00228991928210684*G0_2_2;
    A[364] = 0.0540186087061073*G0_1_0 + 0.00117431758056727*G0_1_1 + 0.0149399292194429*G0_1_2 + 0.0403182369328195*G0_2_0 + 0.00123955744615429*G0_2_1 + 0.0148746893538556*G0_2_2;
    A[365] = -0.00939454064453957*G0_1_0 - 0.0155923278753132*G0_1_1 + 0.00949240044292197*G0_1_2 - 0.0377738821749228*G0_2_0 - 0.0188869410874612*G0_2_1 + 0.0189848008858427*G0_2_2;
    A[366] = 0.00182671623643747*G0_1_0 - 0.0510175748891028*G0_1_1 + 0.0377738821749231*G0_1_2 - 0.082332710370905*G0_2_0 - 0.0463855444324198*G0_2_1 + 0.127087258163646*G0_2_2;
    A[367] = -0.108232937008976*G0_1_0 - 0.00352295274170267*G0_1_1 - 0.0338921101724922*G0_1_2 - 0.0808321934624016*G0_2_0 - 0.00649136662591593*G0_2_1 - 0.0436454700777619*G0_2_2;
    A[368] = 0.0777006799142207*G0_1_0 + 0.182410664181496*G0_1_1 - 0.00616516729798013*G0_1_2 + 0.167731694424402*G0_2_0 + 0.0838658472122008*G0_2_1 - 0.166948816037357*G0_2_2;
    A[369] = 0.146659217839771*G0_1_0 - 0.124738623002511*G0_1_1 + 0.0733296089198869*G0_1_2 + 0.0109602974186313*G0_2_0 - 0.062369311501255*G0_2_1 + 0.132306447410614*G0_2_2;
    A[370] = -0.0540186087061072*G0_1_0 - 0.0528442911255396*G0_1_1 - 0.0390786794866641*G0_1_2 - 0.0403182369328193*G0_2_0 - 0.0390786794866651*G0_2_1 - 0.025443547578964*G0_2_2;
    A[371] = 0.00939454064453967*G0_1_0 - 0.00619778723077444*G0_1_1 + 0.0188869410874612*G0_1_2 + 0.0377738821749226*G0_2_0 + 0.0188869410874617*G0_2_1 + 0.0567586830607655*G0_2_2;
    A[372] = -0.00182671623643763*G0_1_0 - 0.05284429112554*G0_1_1 + 0.0359471659384857*G0_1_2 + 0.0823327103709049*G0_2_0 + 0.0359471659384851*G0_2_1 + 0.209419968534551*G0_2_2;
    A[373] = 0.108232937008977*G0_1_0 + 0.104709984267274*G0_1_1 + 0.0743408268364842*G0_1_2 + 0.080832193462401*G0_2_0 + 0.0743408268364852*G0_2_1 + 0.0371867233846396*G0_2_2;
    A[374] = -0.0777006799142206*G0_1_0 + 0.104709984267276*G0_1_1 - 0.0838658472122004*G0_1_2 - 0.167731694424402*G0_2_0 - 0.0838658472122003*G0_2_1 - 0.334680510461758*G0_2_2;
    A[375] = -0.146659217839771*G0_1_0 - 0.271397840842283*G0_1_1 - 0.0733296089198849*G0_1_2 - 0.0109602974186314*G0_2_0 - 0.0733296089198862*G0_2_1 + 0.121346149991982*G0_2_2;
    A[376] = -0.00117431758056675*G0_1_1 - 0.00123955744615341*G0_1_2 - 0.00123955744615388*G0_2_1 - 0.00130479731174173*G0_2_2;
    A[377] = -0.0105036183595218*G0_1_1 - 0.013080593050211*G0_1_2 - 0.0130805930502113*G0_2_1 - 0.0156575677409012*G0_2_2;
    A[378] = -0.00117431758056702*G0_1_1 - 0.00123955744615432*G0_1_2 - 0.00123955744615431*G0_2_1 - 0.00130479731174208*G0_2_2;
    A[379] = 0.0035229527417019*G0_1_1 + 0.0064913666259134*G0_1_2 + 0.00649136662591566*G0_2_1 + 0.00880738185425708*G0_2_2;
    A[380] = 0.00352295274170278*G0_1_1 + 0.00649136662591492*G0_1_2 + 0.00649136662591604*G0_2_1 + 0.00880738185425744*G0_2_2;
    A[381] = 0.124738623002511*G0_1_1 + 0.062369311501257*G0_1_2 + 0.0623693115012549*G0_2_1 + 0.0476251018785729*G0_2_2;
    A[382] = -0.0150051690850296*G0_1_1 - 0.014939929219443*G0_1_2 - 0.0149399292194432*G0_2_1 - 0.0148746893538552*G0_2_2;
    A[383] = -0.000652398655870716*G0_1_1 + 0.00192457603481878*G0_1_2 + 0.00192457603481865*G0_2_1 + 0.00450155072550872*G0_2_2;
    A[384] = -0.0150051690850299*G0_1_1 - 0.0149399292194428*G0_1_2 - 0.0149399292194431*G0_2_1 - 0.0148746893538554*G0_2_2;
    A[385] = -0.000652398655871191*G0_1_1 - 0.00949240044292085*G0_1_2 - 0.00949240044292108*G0_2_1 - 0.0189848008858429*G0_2_2;
    A[386] = -0.000652398655871445*G0_1_1 - 0.00949240044292177*G0_1_2 - 0.00949240044292108*G0_2_1 - 0.0189848008858428*G0_2_2;
    A[387] = -0.0150051690850295*G0_1_1 - 0.037773882174923*G0_1_2 - 0.0377738821749236*G0_2_1 - 0.127087258163646*G0_2_2;
    A[388] = 0.0260959462348343*G0_1_1 + 0.0319675341376722*G0_1_2 + 0.031967534137671*G0_2_1 + 0.0391439193522509*G0_2_2;
    A[389] = 0.0260959462348352*G0_1_1 + 0.0319675341376731*G0_1_2 + 0.0319675341376728*G0_2_1 + 0.0391439193522529*G0_2_2;
    A[390] = 0.0260959462348353*G0_1_1 + 0.0156575677409005*G0_1_2 + 0.0156575677409026*G0_2_1 + 0.185933616923199*G0_2_2;
    A[391] = -0.185933616923198*G0_1_1 - 0.0903572138381159*G0_1_2 - 0.0903572138381166*G0_2_1 - 0.156575677409009*G0_2_2;
    A[392] = -0.00480034930989796*G0_1_0 - 0.00480034930989799*G0_1_1 - 0.004800349309898*G0_1_2 - 0.00480034930989792*G0_2_0 - 0.00480034930989793*G0_2_1 - 0.00480034930989797*G0_2_2;
    A[393] = 0.00480034930989798*G0_1_0 + 0.00480034930989792*G0_2_0;
    A[394] = 0.00480034930989799*G0_1_1 - 0.0105310191030676*G0_2_1;
    A[395] = 0.0372571824394742*G0_1_2 - 0.0178705039816148*G0_2_2;
    A[396] = -0.0328352243499816*G0_1_1 - 0.00537576492437589*G0_1_2 + 0.0209159009072198*G0_2_1 - 0.0328352243499812*G0_2_2;
    A[397] = 0.0788945694544659*G0_1_1 + 0.0189782768992835*G0_1_2 - 0.0246345732456837*G0_2_1 + 0.100717304493345*G0_2_2;
    A[398] = -0.108637424175619*G0_1_1 - 0.0387263842124946*G0_1_2 + 0.0263830016434176*G0_2_1 - 0.147063705006412*G0_2_2;
    A[399] = 0.139822079926247*G0_1_1 + 0.069911039963123*G0_1_2 + 0.069911039963123*G0_2_1 + 0.179096479009672*G0_2_2;
    A[400] = -0.0328352243499814*G0_1_0 - 0.00537576492437586*G0_1_2 - 0.0274594594256051*G0_2_0 - 0.00537576492437595*G0_2_2;
    A[401] = 0.0788945694544658*G0_1_0 + 0.0189782768992839*G0_1_2 + 0.0599162925551813*G0_2_0 + 0.0283728175438244*G0_2_2;
    A[402] = -0.108637424175619*G0_1_0 - 0.0387263842124951*G0_1_2 - 0.0699110399631229*G0_2_0 - 0.0574176057031956*G0_2_2;
    A[403] = 0.139822079926247*G0_1_0 + 0.0699110399631231*G0_1_2 + 0.069911039963123*G0_2_0 + 0.089548239504836*G0_2_2;
    A[404] = -0.00537576492437596*G0_1_0 - 0.00537576492437589*G0_1_1 - 0.010751529848752*G0_2_0 - 0.00537576492437598*G0_2_1;
    A[405] = -0.00228991928210684*G0_1_0 - 0.00228991928210687*G0_1_1 - 0.0139743792087541*G0_2_0 - 0.0116844599266474*G0_2_1;
    A[406] = -0.0022899192821067*G0_1_0 - 0.00228991928210666*G0_1_1 - 0.0111690649885092*G0_2_0 - 0.0088791457064024*G0_2_1;
    A[407] = -0.00537576492437598*G0_1_0 - 0.00537576492437618*G0_1_1 - 0.0328352243499813*G0_2_0 - 0.0274594594256053*G0_2_1;
    A[408] = 0.0328352243499816*G0_1_0 + 0.0328352243499816*G0_1_1 + 0.0274594594256056*G0_1_2 + 0.0274594594256051*G0_2_0 + 0.0274594594256051*G0_2_1 + 0.0220836945012294*G0_2_2;
    A[409] = -0.0788945694544659*G0_1_0 - 0.0788945694544658*G0_1_1 - 0.0599162925551822*G0_1_2 - 0.0599162925551814*G0_2_0 - 0.0599162925551813*G0_2_1 - 0.0315434750113574*G0_2_2;
    A[410] = 0.108637424175619*G0_1_0 + 0.108637424175619*G0_1_1 + 0.0699110399631239*G0_1_2 + 0.0699110399631227*G0_2_0 + 0.0699110399631227*G0_2_1 + 0.012493434259928*G0_2_2;
    A[411] = -0.139822079926247*G0_1_0 - 0.139822079926247*G0_1_1 - 0.0699110399631236*G0_1_2 - 0.0699110399631228*G0_2_0 - 0.0699110399631229*G0_2_1 + 0.019637199541713*G0_2_2;
    A[412] = 0.00537576492437616*G0_1_0 + 0.00537576492437611*G0_1_2 + 0.0107515298487519*G0_2_0 + 0.00537576492437611*G0_2_1 + 0.0107515298487519*G0_2_2;
    A[413] = 0.00228991928210671*G0_1_0 + 0.00228991928210678*G0_1_2 + 0.0139743792087542*G0_2_0 + 0.00228991928210681*G0_2_1 + 0.0139743792087542*G0_2_2;
    A[414] = 0.00228991928210667*G0_1_0 + 0.00228991928210666*G0_1_2 + 0.011169064988509*G0_2_0 + 0.00228991928210679*G0_2_1 + 0.011169064988509*G0_2_2;
    A[415] = 0.00537576492437602*G0_1_0 + 0.00537576492437599*G0_1_2 + 0.0328352243499813*G0_2_0 + 0.00537576492437598*G0_2_1 + 0.0328352243499813*G0_2_2;
    A[416] = 0.00537576492437622*G0_1_1 + 0.00537576492437615*G0_1_2 + 0.00537576492437589*G0_2_1 + 0.00537576492437586*G0_2_2;
    A[417] = 0.00228991928210666*G0_1_1 + 0.00228991928210675*G0_1_2 + 0.00228991928210661*G0_2_1 + 0.00228991928210664*G0_2_2;
    A[418] = 0.00228991928210675*G0_1_1 + 0.00228991928210677*G0_1_2 + 0.00228991928210661*G0_2_1 + 0.0022899192821066*G0_2_2;
    A[419] = 0.00537576492437595*G0_1_1 + 0.00537576492437592*G0_1_2 + 0.00537576492437598*G0_2_1 + 0.00537576492437598*G0_2_2;
    A[420] = 0.0118736555368506*G0_1_0 + 0.0118736555368505*G0_1_1 - 0.00939454064454046*G0_1_2 + 0.0425363923627812*G0_2_0 + 0.0212681961813907*G0_2_1 - 0.0187890812890812*G0_2_2;
    A[421] = 0.00254435475789634*G0_1_0 + 0.00254435475789633*G0_1_1 - 0.00329461321214806*G0_1_2 + 0.0142875305635722*G0_2_0 + 0.008448562593528*G0_2_1 - 0.0292600797158084*G0_2_2;
    A[422] = 0.0118736555368499*G0_1_0 + 0.0118736555368503*G0_1_1 - 0.00939454064454054*G0_1_2 + 0.0477555816097477*G0_2_0 + 0.0264873854283576*G0_2_1 - 0.0641307878721071*G0_2_2;
    A[423] = -0.0429604514890975*G0_1_0 - 0.0429604514890971*G0_1_1 - 0.000685018588663708*G0_1_2 - 0.084550865800865*G0_2_0 - 0.0422754329004327*G0_2_1 + 0.0439716694056974*G0_2_2;
    A[424] = -0.0429604514890966*G0_1_0 - 0.0429604514890973*G0_1_1 - 0.000685018588664004*G0_1_2 - 0.0493213383838379*G0_2_0 - 0.00704590548340546*G0_2_1 + 0.127315597693201*G0_2_2;
    A[425] = 0.124738623002511*G0_1_0 + 0.124738623002511*G0_1_1 + 0.0284445813959692*G0_1_2 + 0.192588083213081*G0_2_0 + 0.0962940416065408*G0_2_1 - 0.0644569872000432*G0_2_2;
    A[426] = -0.011873655536851*G0_1_0 - 0.0212681961813914*G0_1_2 - 0.0425363923627817*G0_2_0 - 0.0212681961813908*G0_2_1 - 0.0613254736518623*G0_2_2;
    A[427] = -0.00254435475789556*G0_1_0 - 0.00583896797004387*G0_1_2 - 0.0142875305635713*G0_2_0 - 0.00583896797004431*G0_2_1 - 0.0435476102793803*G0_2_2;
    A[428] = -0.01187365553685*G0_1_0 - 0.0212681961813906*G0_1_2 - 0.0477555816097479*G0_2_0 - 0.0212681961813901*G0_2_1 - 0.111886369481855*G0_2_2;
    A[429] = 0.0429604514890981*G0_1_0 + 0.042275432900434*G0_1_2 + 0.0845508658008656*G0_2_0 + 0.0422754329004333*G0_2_1 + 0.128522535206562*G0_2_2;
    A[430] = 0.0429604514890967*G0_1_0 + 0.0422754329004321*G0_1_2 + 0.049321338383838*G0_2_0 + 0.0422754329004319*G0_2_1 + 0.176636936077038*G0_2_2;
    A[431] = -0.124738623002511*G0_1_0 - 0.0962940416065417*G0_1_2 - 0.192588083213081*G0_2_0 - 0.0962940416065404*G0_2_1 - 0.257045070413124*G0_2_2;
    A[432] = -0.0118736555368508*G0_1_1 - 0.0212681961813914*G0_1_2 - 0.0212681961813911*G0_2_1 - 0.030662736825931*G0_2_2;
    A[433] = -0.00254435475789577*G0_1_1 - 0.00583896797004432*G0_1_2 - 0.00583896797004405*G0_2_1 - 0.00913358118219214*G0_2_2;
    A[434] = -0.0118736555368506*G0_1_1 - 0.0212681961813907*G0_1_2 - 0.0212681961813907*G0_2_1 - 0.0306627368259309*G0_2_2;
    A[435] = 0.0429604514890982*G0_1_1 + 0.0422754329004344*G0_1_2 + 0.0422754329004328*G0_2_1 + 0.0642612676032813*G0_2_2;
    A[436] = 0.042960451489097*G0_1_1 + 0.0422754329004329*G0_1_2 + 0.0422754329004326*G0_2_1 + 0.0642612676032811*G0_2_2;
    A[437] = -0.124738623002511*G0_1_1 - 0.0962940416065424*G0_1_2 - 0.0962940416065408*G0_2_1 - 0.128522535206562*G0_2_2;
    A[438] = 0.00939454064454026*G0_1_2 + 0.00939454064454065*G0_2_1 + 0.0187890812890805*G0_2_2;
    A[439] = 0.00329461321214806*G0_1_2 + 0.00329461321214837*G0_2_1 + 0.00658922642429618*G0_2_2;
    A[440] = 0.00939454064454025*G0_1_2 + 0.00939454064454077*G0_2_1 + 0.0187890812890809*G0_2_2;
    A[441] = 0.00329461321214784*G0_1_2 + 0.00329461321214769*G0_2_1 + 0.029260079715809*G0_2_2;
    A[442] = 0.00329461321214807*G0_1_2 + 0.00329461321214739*G0_2_1 + 0.0292600797158087*G0_2_2;
    A[443] = 0.00939454064454043*G0_1_2 + 0.00939454064454098*G0_2_1 + 0.0641307878721069*G0_2_2;
    A[444] = -0.00260959462348228*G0_1_2 - 0.00260959462348162*G0_2_1 - 0.050560895829991*G0_2_2;
    A[445] = -0.00260959462348401*G0_1_2 - 0.00260959462348395*G0_2_1 - 0.0505608958299933*G0_2_2;
    A[446] = -0.00260959462348353*G0_1_2 - 0.0026095946234847*G0_2_1 - 0.15657567740901*G0_2_2;
    A[447] = -0.0352295274170277*G0_1_2 - 0.0352295274170273*G0_2_1 + 0.096228801740953*G0_2_2;
    A[448] = -0.00480034930989783*G0_0_0 - 0.00480034930989786*G0_0_1 - 0.00480034930989786*G0_0_2 - 0.00480034930989781*G0_2_0 - 0.00480034930989784*G0_2_1 - 0.00480034930989787*G0_2_2;
    A[449] = -0.0178705039816151*G0_0_0 + 0.0372571824394737*G0_2_0;
    A[450] = 0.0048003493098979*G0_0_1 + 0.00480034930989801*G0_2_1;
    A[451] = -0.0105310191030678*G0_0_2 + 0.00480034930989797*G0_2_2;
    A[452] = -0.0107515298487513*G0_0_1 - 0.00537576492437577*G0_0_2 - 0.00537576492437592*G0_2_1 - 0.00537576492437583*G0_2_2;
    A[453] = -0.0139743792087548*G0_0_1 - 0.0116844599266471*G0_0_2 - 0.00228991928210677*G0_2_1 - 0.00228991928210676*G0_2_2;
    A[454] = -0.0111690649885088*G0_0_1 - 0.00887914570640275*G0_0_2 - 0.00228991928210675*G0_2_1 - 0.00228991928210689*G0_2_2;
    A[455] = -0.0328352243499814*G0_0_1 - 0.0274594594256051*G0_0_2 - 0.00537576492437586*G0_2_1 - 0.00537576492437595*G0_2_2;
    A[456] = 0.179096479009672*G0_0_0 + 0.069911039963122*G0_0_2 + 0.069911039963122*G0_2_0 + 0.139822079926245*G0_2_2;
    A[457] = -0.147063705006412*G0_0_0 + 0.0263830016434186*G0_0_2 - 0.0387263842124947*G0_2_0 - 0.108637424175618*G0_2_2;
    A[458] = 0.100717304493346*G0_0_0 - 0.0246345732456841*G0_0_2 + 0.0189782768992834*G0_2_0 + 0.0788945694544647*G0_2_2;
    A[459] = -0.0328352243499814*G0_0_0 + 0.0209159009072202*G0_0_2 - 0.00537576492437583*G0_2_0 - 0.032835224349981*G0_2_2;
    A[460] = 0.0895482395048354*G0_0_0 + 0.0699110399631222*G0_0_1 + 0.0699110399631222*G0_2_0 + 0.139822079926245*G0_2_1;
    A[461] = -0.0574176057031947*G0_0_0 - 0.0699110399631223*G0_0_1 - 0.0387263842124945*G0_2_0 - 0.108637424175617*G0_2_1;
    A[462] = 0.0283728175438239*G0_0_0 + 0.0599162925551808*G0_0_1 + 0.0189782768992832*G0_2_0 + 0.0788945694544644*G0_2_1;
    A[463] = -0.00537576492437576*G0_0_0 - 0.0274594594256048*G0_0_1 - 0.00537576492437586*G0_2_0 - 0.0328352243499811*G0_2_1;
    A[464] = 0.0107515298487519*G0_0_0 + 0.0107515298487518*G0_0_1 + 0.00537576492437562*G0_0_2 + 0.00537576492437608*G0_2_0 + 0.00537576492437608*G0_2_1;
    A[465] = 0.013974379208754*G0_0_0 + 0.0139743792087541*G0_0_1 + 0.0022899192821073*G0_0_2 + 0.00228991928210639*G0_2_0 + 0.00228991928210645*G0_2_1;
    A[466] = 0.0111690649885093*G0_0_0 + 0.0111690649885093*G0_0_1 + 0.00228991928210628*G0_0_2 + 0.00228991928210707*G0_2_0 + 0.00228991928210706*G0_2_1;
    A[467] = 0.0328352243499814*G0_0_0 + 0.0328352243499814*G0_0_1 + 0.0053757649243762*G0_0_2 + 0.00537576492437585*G0_2_0 + 0.00537576492437586*G0_2_1;
    A[468] = 0.00537576492437622*G0_0_0 + 0.0053757649243762*G0_0_2 + 0.00537576492437611*G0_2_0 + 0.00537576492437607*G0_2_2;
    A[469] = 0.0022899192821065*G0_0_0 + 0.00228991928210649*G0_0_2 + 0.00228991928210669*G0_2_0 + 0.0022899192821067*G0_2_2;
    A[470] = 0.00228991928210686*G0_0_0 + 0.00228991928210685*G0_0_2 + 0.00228991928210704*G0_2_0 + 0.00228991928210695*G0_2_2;
    A[471] = 0.00537576492437586*G0_0_0 + 0.00537576492437581*G0_0_2 + 0.00537576492437588*G0_2_0 + 0.00537576492437587*G0_2_2;
    A[472] = 0.0220836945012289*G0_0_0 + 0.0274594594256051*G0_0_1 + 0.0274594594256051*G0_0_2 + 0.0274594594256047*G0_2_0 + 0.0328352243499809*G0_2_1 + 0.0328352243499809*G0_2_2;
    A[473] = -0.0315434750113565*G0_0_0 - 0.0599162925551808*G0_0_1 - 0.0599162925551808*G0_0_2 - 0.0599162925551806*G0_2_0 - 0.0788945694544644*G0_2_1 - 0.0788945694544644*G0_2_2;
    A[474] = 0.0124934342599271*G0_0_0 + 0.0699110399631227*G0_0_1 + 0.0699110399631226*G0_0_2 + 0.0699110399631224*G0_2_0 + 0.108637424175617*G0_2_1 + 0.108637424175617*G0_2_2;
    A[475] = 0.0196371995417134*G0_0_0 - 0.0699110399631221*G0_0_1 - 0.0699110399631221*G0_0_2 - 0.0699110399631224*G0_2_0 - 0.139822079926245*G0_2_1 - 0.139822079926245*G0_2_2;
    A[476] = -0.0644569872000426*G0_0_0 + 0.19258808321308*G0_0_1 + 0.0962940416065399*G0_0_2 + 0.0284445813959693*G0_2_0 + 0.124738623002509*G0_2_1 + 0.124738623002509*G0_2_2;
    A[477] = 0.0439716694056962*G0_0_0 - 0.0845508658008648*G0_0_1 - 0.0422754329004319*G0_0_2 - 0.000685018588664062*G0_2_0 - 0.0429604514890962*G0_2_1 - 0.0429604514890963*G0_2_2;
    A[478] = -0.0187890812890806*G0_0_0 + 0.0425363923627805*G0_0_1 + 0.0212681961813904*G0_0_2 - 0.00939454064454086*G0_2_0 + 0.0118736555368495*G0_2_1 + 0.011873655536849*G0_2_2;
    A[479] = 0.127315597693202*G0_0_0 - 0.0493213383838367*G0_0_1 - 0.00704590548340527*G0_0_2 - 0.000685018588664553*G0_2_0 - 0.0429604514890966*G0_2_1 - 0.0429604514890971*G0_2_2;
    A[480] = -0.0292600797158094*G0_0_0 + 0.0142875305635721*G0_0_1 + 0.00844856259352676*G0_0_2 - 0.00329461321214772*G0_2_0 + 0.00254435475789672*G0_2_1 + 0.00254435475789699*G0_2_2;
    A[481] = -0.0641307878721065*G0_0_0 + 0.0477555816097479*G0_0_1 + 0.0264873854283586*G0_0_2 - 0.0093945406445406*G0_2_0 + 0.0118736555368495*G0_2_1 + 0.0118736555368497*G0_2_2;
    A[482] = 0.0187890812890813*G0_0_0 + 0.0093945406445404*G0_0_2 + 0.00939454064454125*G0_2_0;
    A[483] = 0.00658922642429623*G0_0_0 + 0.00329461321214831*G0_0_2 + 0.00329461321214795*G0_2_0;
    A[484] = 0.0187890812890807*G0_0_0 + 0.00939454064454034*G0_0_2 + 0.00939454064454091*G0_2_0;
    A[485] = 0.0292600797158082*G0_0_0 + 0.00329461321214774*G0_0_2 + 0.00329461321214817*G0_2_0;
    A[486] = 0.0292600797158093*G0_0_0 + 0.00329461321214768*G0_0_2 + 0.00329461321214766*G0_2_0;
    A[487] = 0.0641307878721066*G0_0_0 + 0.00939454064454101*G0_0_2 + 0.00939454064454059*G0_2_0;
    A[488] = -0.0613254736518621*G0_0_0 - 0.0425363923627813*G0_0_1 - 0.0212681961813909*G0_0_2 - 0.0212681961813908*G0_2_0 - 0.0118736555368502*G0_2_1;
    A[489] = 0.128522535206562*G0_0_0 + 0.0845508658008658*G0_0_1 + 0.0422754329004331*G0_0_2 + 0.0422754329004329*G0_2_0 + 0.0429604514890974*G0_2_1;
    A[490] = -0.257045070413124*G0_0_0 - 0.192588083213081*G0_0_1 - 0.0962940416065413*G0_0_2 - 0.0962940416065403*G0_2_0 - 0.124738623002509*G0_2_1;
    A[491] = -0.0435476102793809*G0_0_0 - 0.0142875305635728*G0_0_1 - 0.00583896797004448*G0_0_2 - 0.00583896797004381*G0_2_0 - 0.00254435475789614*G0_2_1;
    A[492] = 0.176636936077039*G0_0_0 + 0.0493213383838371*G0_0_1 + 0.0422754329004321*G0_0_2 + 0.0422754329004321*G0_2_0 + 0.0429604514890969*G0_2_1;
    A[493] = -0.111886369481855*G0_0_0 - 0.047755581609748*G0_0_1 - 0.0212681961813899*G0_0_2 - 0.0212681961813906*G0_2_0 - 0.0118736555368496*G0_2_1;
    A[494] = -0.0306627368259311*G0_0_0 - 0.0212681961813903*G0_0_2 - 0.0212681961813905*G0_2_0 - 0.0118736555368496*G0_2_2;
    A[495] = 0.064261267603281*G0_0_0 + 0.0422754329004334*G0_0_2 + 0.0422754329004325*G0_2_0 + 0.0429604514890973*G0_2_2;
    A[496] = -0.128522535206562*G0_0_0 - 0.0962940416065403*G0_0_2 - 0.0962940416065403*G0_2_0 - 0.12473862300251*G0_2_2;
    A[497] = -0.00913358118219201*G0_0_0 - 0.00583896797004412*G0_0_2 - 0.00583896797004447*G0_2_0 - 0.00254435475789649*G0_2_2;
    A[498] = 0.0642612676032803*G0_0_0 + 0.0422754329004319*G0_0_2 + 0.0422754329004322*G0_2_0 + 0.0429604514890963*G0_2_2;
    A[499] = -0.0306627368259308*G0_0_0 - 0.0212681961813904*G0_0_2 - 0.0212681961813902*G0_2_0 - 0.011873655536849*G0_2_2;
    A[500] = -0.0505608958299928*G0_0_0 - 0.00260959462348407*G0_0_2 - 0.00260959462348402*G0_2_0;
    A[501] = 0.096228801740954*G0_0_0 - 0.0352295274170274*G0_0_2 - 0.0352295274170265*G0_2_0;
    A[502] = -0.0505608958299925*G0_0_0 - 0.00260959462348265*G0_0_2 - 0.00260959462348395*G0_2_0;
    A[503] = -0.15657567740901*G0_0_0 - 0.0026095946234842*G0_0_2 - 0.00260959462348361*G0_2_0;
    A[504] = -0.00336507226698195*G0_0_0 - 0.00336507226698198*G0_0_1 - 0.00336507226698197*G0_0_2 - 0.00336507226698199*G0_2_0 - 0.00336507226698199*G0_2_1 - 0.00336507226698196*G0_2_2;
    A[505] = 0.0214691349673988*G0_0_0 - 0.0336585514536901*G0_2_0;
    A[506] = 0.00336507226698197*G0_0_1 + 0.00336507226698185*G0_2_1;
    A[507] = 0.023263231271044*G0_0_2 - 0.0155544887532735*G0_2_2;
    A[508] = -0.0139743792087546*G0_0_1 - 0.00228991928210695*G0_0_2 - 0.011684459926647*G0_2_1 - 0.00228991928210671*G0_2_2;
    A[509] = 0.0263047138047148*G0_0_1 + 0.0131523569023569*G0_0_2 + 0.0131523569023568*G0_2_1 + 0.013217596767944*G0_2_2;
    A[510] = 0.005232237220083*G0_0_1 + 0.00578025209101583*G0_0_2 - 0.000548014870931651*G0_2_1 - 0.000482775005344332*G0_2_2;
    A[511] = 0.0788945694544658*G0_0_1 + 0.0599162925551813*G0_0_2 + 0.0189782768992839*G0_2_1 + 0.0283728175438244*G0_2_2;
    A[512] = -0.147063705006412*G0_0_0 - 0.0387263842124947*G0_0_2 + 0.0263830016434186*G0_2_0 - 0.108637424175618*G0_2_2;
    A[513] = 0.304226541205707*G0_0_0 + 0.0781312630270959*G0_0_2 + 0.0781312630270959*G0_2_0 + 0.281001149056704*G0_2_2;
    A[514] = -0.204187731314467*G0_0_0 + 0.0153052724667301*G0_0_2 - 0.0364951808094169*G0_2_0 - 0.204187731314467*G0_2_2;
    A[515] = 0.0788945694544659*G0_0_0 - 0.0246345732456839*G0_0_2 + 0.0189782768992839*G0_2_0 + 0.100717304493346*G0_2_2;
    A[516] = -0.0574176057031954*G0_0_0 - 0.0387263842124949*G0_0_1 - 0.0699110399631227*G0_2_0 - 0.108637424175618*G0_2_1;
    A[517] = 0.00506261356955749*G0_0_0 + 0.00480165410720932*G0_0_1 + 0.00480165410720925*G0_2_0 + 0.00960330821441903*G0_2_1;
    A[518] = -0.000482775005344231*G0_0_0 + 0.00578025209101599*G0_0_1 - 0.000548014870931198*G0_2_0 + 0.00523223722008446*G0_2_1;
    A[519] = -0.00228991928210703*G0_0_0 - 0.00887914570640285*G0_0_1 - 0.00228991928210674*G0_2_0 - 0.0111690649885091*G0_2_1;
    A[520] = 0.013974379208754*G0_0_0 + 0.0139743792087541*G0_0_1 + 0.0116844599266477*G0_0_2 + 0.0116844599266472*G0_2_0 + 0.0116844599266471*G0_2_1 + 0.00939454064454058*G0_2_2;
    A[521] = -0.0263047138047135*G0_0_0 - 0.0263047138047136*G0_0_1 - 0.0131523569023575*G0_0_2 - 0.0131523569023564*G0_2_0 - 0.0131523569023564*G0_2_1 + 6.52398655873845e-05*G0_2_2;
    A[522] = -0.00523223722008429*G0_0_0 - 0.00523223722008423*G0_0_1 + 0.000548014870932288*G0_0_2 + 0.000548014870931164*G0_2_0 + 0.000548014870931203*G0_2_1 + 6.5239865586628e-05*G0_2_2;
    A[523] = -0.0788945694544657*G0_0_0 - 0.0788945694544658*G0_0_1 - 0.0189782768992844*G0_0_2 - 0.0189782768992838*G0_2_0 - 0.0189782768992839*G0_2_1 + 0.00939454064454086*G0_2_2;
    A[524] = 0.00228991928210649*G0_0_0 + 0.00228991928210654*G0_0_2 + 0.00228991928210691*G0_2_0 + 0.00228991928210697*G0_2_2;
    A[525] = 0.00178757231708631*G0_0_0 + 0.00178757231708641*G0_0_2 + 0.00178757231708591*G0_2_0 + 0.00178757231708596*G0_2_2;
    A[526] = 0.00178757231708598*G0_0_0 + 0.00178757231708591*G0_0_2 + 0.00178757231708616*G0_2_0 + 0.00178757231708612*G0_2_2;
    A[527] = 0.00228991928210695*G0_0_0 + 0.00228991928210698*G0_0_2 + 0.00228991928210672*G0_2_0 + 0.00228991928210674*G0_2_2;
    A[528] = 0.00658922642429577*G0_0_0 + 0.00887914570640223*G0_0_1 + 0.00887914570640221*G0_0_2 + 0.0088791457064026*G0_2_0 + 0.0111690649885093*G0_2_1 + 0.0111690649885093*G0_2_2;
    A[529] = -0.00626302709636026*G0_0_0 - 0.00578025209101568*G0_0_1 - 0.00578025209101567*G0_0_2 - 0.00578025209101572*G0_2_0 - 0.00523223722008425*G0_2_1 - 0.00523223722008427*G0_2_2;
    A[530] = 0.000260959462348173*G0_0_0 - 0.0048016541072099*G0_0_1 - 0.00480165410720983*G0_0_2 - 0.00480165410720981*G0_2_0 - 0.00960330821441904*G0_2_1 - 0.00960330821441894*G0_2_2;
    A[531] = -0.0186912214907005*G0_0_0 + 0.0387263842124949*G0_0_1 + 0.0387263842124948*G0_0_2 + 0.0387263842124951*G0_2_0 + 0.108637424175618*G0_2_1 + 0.108637424175618*G0_2_2;
    A[532] = 0.132306447410614*G0_0_0 + 0.0109602974186318*G0_0_1 - 0.0623693115012551*G0_0_2 + 0.0733296089198867*G0_2_0 + 0.146659217839774*G0_2_1 - 0.124738623002511*G0_2_2;
    A[533] = -0.0436454700777619*G0_0_0 - 0.0808321934624021*G0_0_1 - 0.00649136662591639*G0_0_2 - 0.0338921101724932*G0_2_0 - 0.10823293700898*G0_2_1 - 0.00352295274170381*G0_2_2;
    A[534] = 0.0148746893538559*G0_0_0 + 0.0403182369328205*G0_0_1 + 0.00123955744615476*G0_0_2 + 0.0149399292194438*G0_2_0 + 0.0540186087061086*G0_2_1 + 0.00117431758056869*G0_2_2;
    A[535] = -0.166948816037357*G0_0_0 + 0.1677316944244*G0_0_1 + 0.0838658472122012*G0_0_2 - 0.00616516729797945*G0_2_0 + 0.0777006799142209*G0_2_1 + 0.182410664181497*G0_2_2;
    A[536] = 0.0189848008858427*G0_0_0 - 0.037773882174924*G0_0_1 - 0.0188869410874614*G0_0_2 + 0.00949240044292081*G0_2_0 - 0.00939454064454063*G0_2_1 - 0.0155923278753151*G0_2_2;
    A[537] = 0.127087258163646*G0_0_0 - 0.0823327103709042*G0_0_1 - 0.0463855444324203*G0_0_2 + 0.037773882174924*G0_2_0 + 0.00182671623643879*G0_2_1 - 0.0510175748891019*G0_2_2;
    A[538] = -0.0148746893538566*G0_0_0 - 0.0149399292194431*G0_0_2 - 0.0149399292194443*G0_2_0 - 0.0150051690850309*G0_2_2;
    A[539] = 0.0045015507255098*G0_0_0 + 0.00192457603481959*G0_0_2 + 0.00192457603482031*G0_2_0 - 0.000652398655869762*G0_2_2;
    A[540] = -0.014874689353856*G0_0_0 - 0.014939929219443*G0_0_2 - 0.0149399292194438*G0_2_0 - 0.0150051690850302*G0_2_2;
    A[541] = -0.0189848008858423*G0_0_0 - 0.00949240044292119*G0_0_2 - 0.00949240044292199*G0_2_0 - 0.000652398655870933*G0_2_2;
    A[542] = -0.0189848008858425*G0_0_0 - 0.00949240044292156*G0_0_2 - 0.00949240044292072*G0_2_0 - 0.000652398655871752*G0_2_2;
    A[543] = -0.127087258163646*G0_0_0 - 0.0377738821749237*G0_0_2 - 0.0377738821749241*G0_2_0 - 0.0150051690850298*G0_2_2;
    A[544] = -0.0254435475789635*G0_0_0 - 0.0403182369328196*G0_0_1 - 0.0390786794866646*G0_0_2 - 0.0390786794866649*G0_2_0 - 0.0540186087061081*G0_2_1 - 0.0528442911255407*G0_2_2;
    A[545] = 0.0371867233846389*G0_0_0 + 0.0808321934624002*G0_0_1 + 0.0743408268364849*G0_0_2 + 0.0743408268364856*G0_2_0 + 0.108232937008978*G0_2_1 + 0.104709984267274*G0_2_2;
    A[546] = 0.121346149991983*G0_0_0 - 0.0109602974186315*G0_0_1 - 0.0733296089198859*G0_0_2 - 0.0733296089198865*G0_2_0 - 0.146659217839774*G0_2_1 - 0.271397840842284*G0_2_2;
    A[547] = 0.0567586830607652*G0_0_0 + 0.0377738821749233*G0_0_1 + 0.0188869410874615*G0_0_2 + 0.0188869410874606*G0_2_0 + 0.00939454064453939*G0_2_1 - 0.00619778723077343*G0_2_2;
    A[548] = -0.334680510461758*G0_0_0 - 0.167731694424401*G0_0_1 - 0.0838658472121996*G0_0_2 - 0.0838658472122*G0_2_0 - 0.077700679914221*G0_2_1 + 0.104709984267277*G0_2_2;
    A[549] = 0.209419968534551*G0_0_0 + 0.0823327103709041*G0_0_1 + 0.0359471659384848*G0_0_2 + 0.0359471659384856*G0_2_0 - 0.00182671623643881*G0_2_1 - 0.0528442911255408*G0_2_2;
    A[550] = -0.00130479731174143*G0_0_0 - 0.00123955744615513*G0_0_2 - 0.00123955744615477*G0_2_0 - 0.00117431758056812*G0_2_2;
    A[551] = 0.00880738185425659*G0_0_0 + 0.00649136662591484*G0_0_2 + 0.00649136662591527*G0_2_0 + 0.00352295274170294*G0_2_2;
    A[552] = 0.0476251018785738*G0_0_0 + 0.0623693115012552*G0_0_2 + 0.0623693115012552*G0_2_0 + 0.124738623002511*G0_2_2;
    A[553] = -0.0156575677409007*G0_0_0 - 0.0130805930502105*G0_0_2 - 0.0130805930502103*G0_2_0 - 0.0105036183595202*G0_2_2;
    A[554] = 0.00880738185425678*G0_0_0 + 0.00649136662591629*G0_0_2 + 0.00649136662591524*G0_2_0 + 0.00352295274170391*G0_2_2;
    A[555] = -0.00130479731174163*G0_0_0 - 0.00123955744615466*G0_0_2 - 0.00123955744615497*G0_2_0 - 0.00117431758056866*G0_2_2;
    A[556] = 0.0391439193522538*G0_0_0 + 0.0319675341376743*G0_0_2 + 0.0319675341376746*G0_2_0 + 0.0260959462348355*G0_2_2;
    A[557] = -0.156575677409011*G0_0_0 - 0.090357213838116*G0_0_2 - 0.0903572138381172*G0_2_0 - 0.1859336169232*G0_2_2;
    A[558] = 0.0391439193522521*G0_0_0 + 0.0319675341376719*G0_0_2 + 0.0319675341376728*G0_2_0 + 0.0260959462348351*G0_2_2;
    A[559] = 0.1859336169232*G0_0_0 + 0.015657567740901*G0_0_2 + 0.0156575677409017*G0_2_0 + 0.0260959462348346*G0_2_2;
    A[560] = -0.00336507226698197*G0_0_0 - 0.00336507226698195*G0_0_1 - 0.00336507226698193*G0_0_2 - 0.00336507226698191*G0_2_0 - 0.00336507226698193*G0_2_1 - 0.00336507226698191*G0_2_2;
    A[561] = -0.0155544887532735*G0_0_0 + 0.0232632312710435*G0_2_0;
    A[562] = 0.00336507226698181*G0_0_1 + 0.00336507226698192*G0_2_1;
    A[563] = -0.0336585514536904*G0_0_2 + 0.0214691349673989*G0_2_2;
    A[564] = -0.0111690649885085*G0_0_1 - 0.00228991928210673*G0_0_2 - 0.0088791457064023*G0_2_1 - 0.00228991928210687*G0_2_2;
    A[565] = 0.00523223722008297*G0_0_1 - 0.00054801487093133*G0_0_2 + 0.00578025209101563*G0_2_1 - 0.000482775005344061*G0_2_2;
    A[566] = 0.00960330821442051*G0_0_1 + 0.00480165410720933*G0_0_2 + 0.00480165410720968*G0_2_1 + 0.00506261356955739*G0_2_2;
    A[567] = -0.108637424175619*G0_0_1 - 0.0699110399631229*G0_0_2 - 0.0387263842124951*G0_2_1 - 0.0574176057031956*G0_2_2;
    A[568] = 0.100717304493346*G0_0_0 + 0.0189782768992834*G0_0_2 - 0.0246345732456841*G0_2_0 + 0.0788945694544647*G0_2_2;
    A[569] = -0.204187731314467*G0_0_0 - 0.0364951808094169*G0_0_2 + 0.0153052724667301*G0_2_0 - 0.204187731314467*G0_2_2;
    A[570] = 0.281001149056704*G0_0_0 + 0.0781312630270961*G0_0_2 + 0.0781312630270961*G0_2_0 + 0.304226541205706*G0_2_2;
    A[571] = -0.108637424175619*G0_0_0 + 0.0263830016434179*G0_0_2 - 0.0387263842124951*G0_2_0 - 0.147063705006413*G0_2_2;
    A[572] = 0.0283728175438241*G0_0_0 + 0.0189782768992835*G0_0_1 + 0.059916292555181*G0_2_0 + 0.0788945694544647*G0_2_1;
    A[573] = -0.000482775005344225*G0_0_0 - 0.000548014870931285*G0_0_1 + 0.00578025209101614*G0_2_0 + 0.00523223722008474*G0_2_1;
    A[574] = 0.0132175967679437*G0_0_0 + 0.0131523569023563*G0_0_1 + 0.0131523569023565*G0_2_0 + 0.0263047138047131*G0_2_1;
    A[575] = -0.00228991928210664*G0_0_0 - 0.0116844599266468*G0_0_1 - 0.00228991928210682*G0_2_0 - 0.013974379208754*G0_2_1;
    A[576] = 0.011169064988509*G0_0_0 + 0.0111690649885089*G0_0_1 + 0.00887914570640211*G0_0_2 + 0.0088791457064024*G0_2_0 + 0.00887914570640246*G0_2_1 + 0.00658922642429564*G0_2_2;
    A[577] = -0.00523223722008402*G0_0_0 - 0.0052322372200839*G0_0_1 - 0.00578025209101505*G0_0_2 - 0.0057802520910159*G0_2_0 - 0.00578025209101589*G0_2_1 - 0.00626302709636029*G0_2_2;
    A[578] = -0.00960330821441942*G0_0_0 - 0.00960330821441941*G0_0_1 - 0.00480165410721048*G0_0_2 - 0.00480165410720927*G0_2_0 - 0.00480165410720929*G0_2_1 + 0.000260959462348509*G0_2_2;
    A[579] = 0.108637424175619*G0_0_0 + 0.108637424175619*G0_0_1 + 0.0387263842124955*G0_0_2 + 0.038726384212495*G0_2_0 + 0.038726384212495*G0_2_1 - 0.0186912214907009*G0_2_2;
    A[580] = 0.00228991928210679*G0_0_0 + 0.00228991928210675*G0_0_2 + 0.00228991928210652*G0_2_0 + 0.00228991928210647*G0_2_2;
    A[581] = 0.00178757231708615*G0_0_0 + 0.00178757231708613*G0_0_2 + 0.00178757231708639*G0_2_0 + 0.0017875723170864*G0_2_2;
    A[582] = 0.00178757231708627*G0_0_0 + 0.00178757231708629*G0_0_2 + 0.00178757231708608*G0_2_0 + 0.00178757231708609*G0_2_2;
    A[583] = 0.00228991928210665*G0_0_0 + 0.00228991928210664*G0_0_2 + 0.00228991928210683*G0_2_0 + 0.00228991928210681*G0_2_2;
    A[584] = 0.00939454064454053*G0_0_0 + 0.0116844599266473*G0_0_1 + 0.0116844599266473*G0_0_2 + 0.0116844599266472*G0_2_0 + 0.0139743792087539*G0_2_1 + 0.0139743792087539*G0_2_2;
    A[585] = 6.52398655871084e-05*G0_0_0 - 0.0131523569023567*G0_0_1 - 0.0131523569023568*G0_0_2 - 0.0131523569023568*G0_2_0 - 0.0263047138047136*G0_2_1 - 0.0263047138047135*G0_2_2;
    A[586] = 6.52398655870678e-05*G0_0_0 + 0.000548014870931594*G0_0_1 + 0.000548014870931523*G0_0_2 + 0.000548014870931451*G0_2_0 - 0.00523223722008477*G0_2_1 - 0.00523223722008489*G0_2_2;
    A[587] = 0.0093945406445407*G0_0_0 - 0.0189782768992835*G0_0_1 - 0.0189782768992835*G0_0_2 - 0.0189782768992835*G0_2_0 - 0.0788945694544646*G0_2_1 - 0.0788945694544645*G0_2_2;
    A[588] = -0.0510175748891032*G0_0_0 + 0.0018267162364373*G0_0_1 + 0.037773882174923*G0_0_2 - 0.0463855444324198*G0_2_0 - 0.0823327103709054*G0_2_1 + 0.127087258163646*G0_2_2;
    A[589] = -0.0155923278753144*G0_0_0 - 0.00939454064454023*G0_0_1 + 0.00949240044292128*G0_0_2 - 0.0188869410874618*G0_2_0 - 0.0377738821749229*G0_2_1 + 0.0189848008858422*G0_2_2;
    A[590] = 0.00117431758056806*G0_0_0 + 0.0540186087061073*G0_0_1 + 0.0149399292194431*G0_0_2 + 0.00123955744615497*G0_2_0 + 0.0403182369328196*G0_2_1 + 0.0148746893538555*G0_2_2;
    A[591] = 0.182410664181498*G0_0_0 + 0.0777006799142218*G0_0_1 - 0.00616516729797999*G0_0_2 + 0.0838658472122023*G0_2_0 + 0.167731694424402*G0_2_1 - 0.166948816037357*G0_2_2;
    A[592] = -0.00352295274170334*G0_0_0 - 0.108232937008977*G0_0_1 - 0.0338921101724927*G0_0_2 - 0.00649136662591548*G0_2_0 - 0.0808321934624007*G0_2_1 - 0.043645470077761*G0_2_2;
    A[593] = -0.12473862300251*G0_0_0 + 0.146659217839773*G0_0_1 + 0.0733296089198884*G0_0_2 - 0.0623693115012553*G0_2_0 + 0.0109602974186311*G0_2_1 + 0.132306447410614*G0_2_2;
    A[594] = -0.00117431758056713*G0_0_0 - 0.00123955744615425*G0_0_2 - 0.00123955744615405*G0_2_0 - 0.00130479731174103*G0_2_2;
    A[595] = -0.0105036183595213*G0_0_0 - 0.0130805930502111*G0_0_2 - 0.0130805930502116*G0_2_0 - 0.0156575677409014*G0_2_2;
    A[596] = -0.00117431758056782*G0_0_0 - 0.00123955744615469*G0_0_2 - 0.00123955744615484*G0_2_0 - 0.00130479731174189*G0_2_2;
    A[597] = 0.00352295274170217*G0_0_0 + 0.00649136662591436*G0_0_2 + 0.00649136662591553*G0_2_0 + 0.00880738185425647*G0_2_2;
    A[598] = 0.00352295274170308*G0_0_0 + 0.00649136662591534*G0_0_2 + 0.00649136662591537*G0_2_0 + 0.0088073818542575*G0_2_2;
    A[599] = 0.124738623002511*G0_0_0 + 0.0623693115012562*G0_0_2 + 0.0623693115012555*G0_2_0 + 0.0476251018785739*G0_2_2;
    A[600] = -0.0528442911255403*G0_0_0 - 0.0540186087061081*G0_0_1 - 0.0390786794866651*G0_0_2 - 0.039078679486665*G0_2_0 - 0.0403182369328198*G0_2_1 - 0.0254435475789636*G0_2_2;
    A[601] = -0.00619778723077301*G0_0_0 + 0.0093945406445408*G0_0_1 + 0.0188869410874621*G0_0_2 + 0.0188869410874617*G0_2_0 + 0.0377738821749232*G0_2_1 + 0.0567586830607663*G0_2_2;
    A[602] = -0.0528442911255414*G0_0_0 - 0.00182671623643766*G0_0_1 + 0.0359471659384851*G0_0_2 + 0.0359471659384849*G0_2_0 + 0.082332710370905*G0_2_1 + 0.209419968534551*G0_2_2;
    A[603] = 0.104709984267274*G0_0_0 + 0.108232937008977*G0_0_1 + 0.0743408268364848*G0_0_2 + 0.0743408268364853*G0_2_0 + 0.0808321934624008*G0_2_1 + 0.0371867233846387*G0_2_2;
    A[604] = 0.104709984267277*G0_0_0 - 0.0777006799142214*G0_0_1 - 0.0838658472122014*G0_0_2 - 0.0838658472121996*G0_2_0 - 0.167731694424402*G0_2_1 - 0.334680510461759*G0_2_2;
    A[605] = -0.271397840842284*G0_0_0 - 0.146659217839773*G0_0_1 - 0.0733296089198847*G0_0_2 - 0.0733296089198866*G0_2_0 - 0.010960297418631*G0_2_1 + 0.121346149991984*G0_2_2;
    A[606] = -0.0150051690850302*G0_0_0 - 0.0149399292194428*G0_0_2 - 0.0149399292194428*G0_2_0 - 0.0148746893538557*G0_2_2;
    A[607] = -0.000652398655870569*G0_0_0 - 0.00949240044292084*G0_0_2 - 0.00949240044292121*G0_2_0 - 0.0189848008858425*G0_2_2;
    A[608] = -0.0150051690850302*G0_0_0 - 0.0377738821749231*G0_0_2 - 0.0377738821749235*G0_2_0 - 0.127087258163646*G0_2_2;
    A[609] = -0.000652398655870683*G0_0_0 + 0.00192457603481923*G0_0_2 + 0.00192457603481892*G0_2_0 + 0.00450155072550878*G0_2_2;
    A[610] = -0.00065239865587109*G0_0_0 - 0.00949240044292119*G0_0_2 - 0.00949240044292143*G0_2_0 - 0.0189848008858424*G0_2_2;
    A[611] = -0.01500516908503*G0_0_0 - 0.0149399292194431*G0_0_2 - 0.0149399292194426*G0_2_0 - 0.0148746893538555*G0_2_2;
    A[612] = 0.0260959462348343*G0_0_0 + 0.0319675341376719*G0_0_2 + 0.0319675341376719*G0_2_0 + 0.0391439193522521*G0_2_2;
    A[613] = 0.0260959462348363*G0_0_0 + 0.0156575677409009*G0_0_2 + 0.0156575677409022*G0_2_0 + 0.185933616923199*G0_2_2;
    A[614] = 0.0260959462348357*G0_0_0 + 0.0319675341376733*G0_0_2 + 0.0319675341376734*G0_2_0 + 0.0391439193522523*G0_2_2;
    A[615] = -0.185933616923201*G0_0_0 - 0.0903572138381173*G0_0_2 - 0.0903572138381181*G0_2_0 - 0.156575677409011*G0_2_2;
    A[616] = -0.00480034930989796*G0_0_0 - 0.00480034930989798*G0_0_1 - 0.004800349309898*G0_0_2 - 0.00480034930989795*G0_2_0 - 0.00480034930989796*G0_2_1 - 0.00480034930989799*G0_2_2;
    A[617] = 0.00480034930989798*G0_0_0 - 0.0105310191030676*G0_2_0;
    A[618] = 0.00480034930989801*G0_0_1 + 0.00480034930989791*G0_2_1;
    A[619] = 0.0372571824394742*G0_0_2 - 0.0178705039816149*G0_2_2;
    A[620] = -0.0328352243499816*G0_0_1 - 0.00537576492437587*G0_0_2 - 0.0274594594256052*G0_2_1 - 0.00537576492437588*G0_2_2;
    A[621] = 0.0788945694544659*G0_0_1 + 0.0189782768992834*G0_0_2 + 0.0599162925551812*G0_2_1 + 0.0283728175438238*G0_2_2;
    A[622] = -0.108637424175619*G0_0_1 - 0.0387263842124946*G0_0_2 - 0.0699110399631231*G0_2_1 - 0.057417605703195*G0_2_2;
    A[623] = 0.139822079926247*G0_0_1 + 0.069911039963123*G0_0_2 + 0.0699110399631231*G0_2_1 + 0.089548239504836*G0_2_2;
    A[624] = -0.0328352243499814*G0_0_0 - 0.00537576492437583*G0_0_2 + 0.0209159009072202*G0_2_0 - 0.032835224349981*G0_2_2;
    A[625] = 0.0788945694544659*G0_0_0 + 0.0189782768992839*G0_0_2 - 0.0246345732456839*G0_2_0 + 0.100717304493346*G0_2_2;
    A[626] = -0.108637424175619*G0_0_0 - 0.0387263842124951*G0_0_2 + 0.0263830016434179*G0_2_0 - 0.147063705006413*G0_2_2;
    A[627] = 0.139822079926247*G0_0_0 + 0.0699110399631231*G0_0_2 + 0.0699110399631231*G0_2_0 + 0.179096479009672*G0_2_2;
    A[628] = -0.00537576492437593*G0_0_0 - 0.00537576492437587*G0_0_1 - 0.027459459425605*G0_2_0 - 0.032835224349981*G0_2_1;
    A[629] = -0.00228991928210688*G0_0_0 - 0.00228991928210692*G0_0_1 - 0.00887914570640271*G0_2_0 - 0.0111690649885095*G0_2_1;
    A[630] = -0.00228991928210666*G0_0_0 - 0.00228991928210658*G0_0_1 - 0.0116844599266472*G0_2_0 - 0.013974379208754*G0_2_1;
    A[631] = -0.00537576492437597*G0_0_0 - 0.00537576492437621*G0_0_1 - 0.00537576492437593*G0_2_0 - 0.010751529848752*G0_2_1;
    A[632] = 0.0328352243499816*G0_0_0 + 0.0328352243499816*G0_0_1 + 0.0274594594256056*G0_0_2 + 0.0274594594256052*G0_2_0 + 0.0274594594256052*G0_2_1 + 0.0220836945012293*G0_2_2;
    A[633] = -0.0788945694544659*G0_0_0 - 0.0788945694544657*G0_0_1 - 0.0599162925551821*G0_0_2 - 0.0599162925551815*G0_2_0 - 0.0599162925551814*G0_2_1 - 0.0315434750113572*G0_2_2;
    A[634] = 0.108637424175619*G0_0_0 + 0.108637424175619*G0_0_1 + 0.0699110399631239*G0_0_2 + 0.069911039963123*G0_2_0 + 0.069911039963123*G0_2_1 + 0.0124934342599276*G0_2_2;
    A[635] = -0.139822079926247*G0_0_0 - 0.139822079926247*G0_0_1 - 0.0699110399631235*G0_0_2 - 0.069911039963123*G0_2_0 - 0.0699110399631231*G0_2_1 + 0.0196371995417131*G0_2_2;
    A[636] = 0.00537576492437617*G0_0_0 + 0.00537576492437612*G0_0_2 + 0.0053757649243762*G0_2_0 + 0.00537576492437617*G0_2_2;
    A[637] = 0.00228991928210667*G0_0_0 + 0.00228991928210674*G0_0_2 + 0.00228991928210663*G0_2_0 + 0.00228991928210667*G0_2_2;
    A[638] = 0.00228991928210667*G0_0_0 + 0.00228991928210666*G0_0_2 + 0.00228991928210666*G0_2_0 + 0.00228991928210666*G0_2_2;
    A[639] = 0.00537576492437601*G0_0_0 + 0.00537576492437598*G0_0_2 + 0.00537576492437596*G0_2_0 + 0.00537576492437594*G0_2_2;
    A[640] = 0.00537576492437622*G0_0_1 + 0.00537576492437616*G0_0_2 + 0.00537576492437613*G0_2_0 + 0.0107515298487521*G0_2_1 + 0.0107515298487521*G0_2_2;
    A[641] = 0.00228991928210664*G0_0_1 + 0.00228991928210672*G0_0_2 + 0.00228991928210676*G0_2_0 + 0.013974379208754*G0_2_1 + 0.013974379208754*G0_2_2;
    A[642] = 0.00228991928210677*G0_0_1 + 0.00228991928210679*G0_0_2 + 0.00228991928210677*G0_2_0 + 0.0111690649885095*G0_2_1 + 0.0111690649885095*G0_2_2;
    A[643] = 0.00537576492437592*G0_0_1 + 0.0053757649243759*G0_0_2 + 0.00537576492437586*G0_2_0 + 0.0328352243499811*G0_2_1 + 0.032835224349981*G0_2_2;
    A[644] = 0.0118736555368505*G0_0_0 + 0.0118736555368506*G0_0_1 - 0.00939454064454044*G0_0_2 + 0.0264873854283576*G0_2_0 + 0.0477555816097482*G0_2_1 - 0.0641307878721071*G0_2_2;
    A[645] = 0.0025443547578964*G0_0_0 + 0.00254435475789618*G0_0_1 - 0.00329461321214805*G0_0_2 + 0.00844856259352771*G0_2_0 + 0.014287530563572*G0_2_1 - 0.0292600797158086*G0_2_2;
    A[646] = 0.0118736555368499*G0_0_0 + 0.0118736555368504*G0_0_1 - 0.00939454064454046*G0_0_2 + 0.0212681961813902*G0_2_0 + 0.0425363923627808*G0_2_1 - 0.018789081289081*G0_2_2;
    A[647] = -0.0429604514890976*G0_0_0 - 0.0429604514890971*G0_0_1 - 0.000685018588663805*G0_0_2 - 0.00704590548340579*G0_2_0 - 0.0493213383838385*G0_2_1 + 0.127315597693201*G0_2_2;
    A[648] = -0.0429604514890967*G0_0_0 - 0.0429604514890974*G0_0_1 - 0.000685018588664115*G0_0_2 - 0.0422754329004325*G0_2_0 - 0.0845508658008651*G0_2_1 + 0.0439716694056965*G0_2_2;
    A[649] = 0.124738623002511*G0_0_0 + 0.124738623002511*G0_0_1 + 0.0284445813959692*G0_0_2 + 0.096294041606541*G0_2_0 + 0.192588083213082*G0_2_1 - 0.0644569872000427*G0_2_2;
    A[650] = -0.0118736555368511*G0_0_0 - 0.0212681961813914*G0_0_2 - 0.0212681961813914*G0_2_0 - 0.0306627368259318*G0_2_2;
    A[651] = -0.00254435475789556*G0_0_0 - 0.00583896797004381*G0_0_2 - 0.00583896797004335*G0_2_0 - 0.00913358118219155*G0_2_2;
    A[652] = -0.01187365553685*G0_0_0 - 0.0212681961813905*G0_0_2 - 0.0212681961813903*G0_2_0 - 0.0306627368259308*G0_2_2;
    A[653] = 0.0429604514890981*G0_0_0 + 0.042275432900434*G0_0_2 + 0.042275432900433*G0_2_0 + 0.0642612676032816*G0_2_2;
    A[654] = 0.0429604514890968*G0_0_0 + 0.042275432900432*G0_0_2 + 0.0422754329004325*G0_2_0 + 0.06426126760328*G0_2_2;
    A[655] = -0.124738623002511*G0_0_0 - 0.0962940416065417*G0_0_2 - 0.096294041606541*G0_2_0 - 0.128522535206562*G0_2_2;
    A[656] = -0.0118736555368508*G0_0_1 - 0.0212681961813913*G0_0_2 - 0.0212681961813908*G0_2_0 - 0.0425363923627815*G0_2_1 - 0.0613254736518626*G0_2_2;
    A[657] = -0.00254435475789574*G0_0_1 - 0.00583896797004429*G0_0_2 - 0.00583896797004409*G0_2_0 - 0.0142875305635715*G0_2_1 - 0.0435476102793807*G0_2_2;
    A[658] = -0.0118736555368507*G0_0_1 - 0.0212681961813908*G0_0_2 - 0.0212681961813905*G0_2_0 - 0.0477555816097483*G0_2_1 - 0.111886369481855*G0_2_2;
    A[659] = 0.0429604514890981*G0_0_1 + 0.0422754329004342*G0_0_2 + 0.0422754329004331*G0_2_0 + 0.0845508658008655*G0_2_1 + 0.128522535206563*G0_2_2;
    A[660] = 0.042960451489097*G0_0_1 + 0.0422754329004329*G0_0_2 + 0.0422754329004324*G0_2_0 + 0.0493213383838383*G0_2_1 + 0.176636936077039*G0_2_2;
    A[661] = -0.124738623002511*G0_0_1 - 0.0962940416065423*G0_0_2 - 0.0962940416065409*G0_2_0 - 0.192588083213082*G0_2_1 - 0.257045070413125*G0_2_2;
    A[662] = 0.0093945406445402*G0_0_2 + 0.0093945406445404*G0_2_0 + 0.0187890812890807*G0_2_2;
    A[663] = 0.00329461321214808*G0_0_2 + 0.00329461321214809*G0_2_0 + 0.0292600797158091*G0_2_2;
    A[664] = 0.00939454064454023*G0_0_2 + 0.0093945406445404*G0_2_0 + 0.0641307878721069*G0_2_2;
    A[665] = 0.00329461321214782*G0_0_2 + 0.00329461321214812*G0_2_0 + 0.00658922642429606*G0_2_2;
    A[666] = 0.00329461321214806*G0_0_2 + 0.00329461321214802*G0_2_0 + 0.0292600797158088*G0_2_2;
    A[667] = 0.00939454064454035*G0_0_2 + 0.00939454064454047*G0_2_0 + 0.0187890812890809*G0_2_2;
    A[668] = -0.0026095946234822*G0_0_2 - 0.00260959462348186*G0_2_0 - 0.0505608958299915*G0_2_2;
    A[669] = -0.00260959462348394*G0_0_2 - 0.00260959462348446*G0_2_0 - 0.15657567740901*G0_2_2;
    A[670] = -0.00260959462348339*G0_0_2 - 0.00260959462348422*G0_2_0 - 0.0505608958299925*G0_2_2;
    A[671] = -0.0352295274170277*G0_0_2 - 0.0352295274170271*G0_2_0 + 0.0962288017409531*G0_2_2;
    A[672] = -0.00480034930989784*G0_0_0 - 0.00480034930989785*G0_0_1 - 0.00480034930989787*G0_0_2 - 0.00480034930989785*G0_1_0 - 0.00480034930989788*G0_1_1 - 0.00480034930989791*G0_1_2;
    A[673] = -0.017870503981615*G0_0_0 + 0.0372571824394737*G0_1_0;
    A[674] = -0.0105310191030675*G0_0_1 + 0.00480034930989802*G0_1_1;
    A[675] = 0.00480034930989791*G0_0_2 + 0.00480034930989797*G0_1_2;
    A[676] = -0.0274594594256046*G0_0_1 - 0.0328352243499806*G0_0_2 - 0.0053757649243759*G0_1_1 - 0.00537576492437581*G0_1_2;
    A[677] = -0.00887914570640287*G0_0_1 - 0.0111690649885094*G0_0_2 - 0.00228991928210689*G0_1_1 - 0.00228991928210686*G0_1_2;
    A[678] = -0.0116844599266473*G0_0_1 - 0.0139743792087543*G0_0_2 - 0.0022899192821067*G0_1_1 - 0.00228991928210683*G0_1_2;
    A[679] = -0.00537576492437596*G0_0_1 - 0.010751529848752*G0_0_2 - 0.00537576492437589*G0_1_1 - 0.00537576492437598*G0_1_2;
    A[680] = 0.0895482395048354*G0_0_0 + 0.0699110399631222*G0_0_2 + 0.0699110399631222*G0_1_0 + 0.139822079926245*G0_1_2;
    A[681] = -0.0574176057031953*G0_0_0 - 0.0699110399631227*G0_0_2 - 0.0387263842124949*G0_1_0 - 0.108637424175618*G0_1_2;
    A[682] = 0.0283728175438241*G0_0_0 + 0.059916292555181*G0_0_2 + 0.0189782768992835*G0_1_0 + 0.0788945694544647*G0_1_2;
    A[683] = -0.00537576492437593*G0_0_0 - 0.027459459425605*G0_0_2 - 0.00537576492437587*G0_1_0 - 0.032835224349981*G0_1_2;
    A[684] = 0.179096479009671*G0_0_0 + 0.0699110399631223*G0_0_1 + 0.0699110399631223*G0_1_0 + 0.139822079926245*G0_1_1;
    A[685] = -0.147063705006411*G0_0_0 + 0.0263830016434185*G0_0_1 - 0.0387263842124946*G0_1_0 - 0.108637424175618*G0_1_1;
    A[686] = 0.100717304493345*G0_0_0 - 0.0246345732456843*G0_0_1 + 0.0189782768992833*G0_1_0 + 0.0788945694544645*G0_1_1;
    A[687] = -0.0328352243499806*G0_0_0 + 0.0209159009072202*G0_0_1 - 0.00537576492437585*G0_1_0 - 0.0328352243499811*G0_1_1;
    A[688] = 0.00537576492437596*G0_0_0 + 0.00537576492437589*G0_0_1 + 0.00537576492437607*G0_1_0 + 0.00537576492437607*G0_1_1;
    A[689] = 0.00228991928210652*G0_0_0 + 0.00228991928210665*G0_0_1 + 0.00228991928210636*G0_1_0 + 0.00228991928210642*G0_1_1;
    A[690] = 0.00228991928210691*G0_0_0 + 0.00228991928210686*G0_0_1 + 0.00228991928210709*G0_1_0 + 0.00228991928210708*G0_1_1;
    A[691] = 0.00537576492437596*G0_0_0 + 0.00537576492437597*G0_0_1 + 0.00537576492437588*G0_1_0 + 0.00537576492437589*G0_1_1;
    A[692] = 0.0107515298487521*G0_0_0 + 0.00537576492437599*G0_0_1 + 0.010751529848752*G0_0_2 + 0.00537576492437613*G0_1_0 + 0.00537576492437609*G0_1_2;
    A[693] = 0.0139743792087539*G0_0_0 + 0.00228991928210647*G0_0_1 + 0.013974379208754*G0_0_2 + 0.0022899192821067*G0_1_0 + 0.00228991928210672*G0_1_2;
    A[694] = 0.0111690649885096*G0_0_0 + 0.00228991928210722*G0_0_1 + 0.0111690649885095*G0_0_2 + 0.00228991928210704*G0_1_0 + 0.00228991928210696*G0_1_2;
    A[695] = 0.0328352243499807*G0_0_0 + 0.00537576492437565*G0_0_1 + 0.0328352243499807*G0_0_2 + 0.00537576492437586*G0_1_0 + 0.00537576492437585*G0_1_2;
    A[696] = 0.0220836945012289*G0_0_0 + 0.0274594594256049*G0_0_1 + 0.0274594594256049*G0_0_2 + 0.0274594594256049*G0_1_0 + 0.032835224349981*G0_1_1 + 0.0328352243499809*G0_1_2;
    A[697] = -0.0315434750113564*G0_0_0 - 0.0599162925551806*G0_0_1 - 0.0599162925551806*G0_0_2 - 0.0599162925551807*G0_1_0 - 0.0788945694544645*G0_1_1 - 0.0788945694544645*G0_1_2;
    A[698] = 0.0124934342599271*G0_0_0 + 0.0699110399631226*G0_0_1 + 0.0699110399631225*G0_0_2 + 0.0699110399631226*G0_1_0 + 0.108637424175617*G0_1_1 + 0.108637424175617*G0_1_2;
    A[699] = 0.0196371995417133*G0_0_0 - 0.0699110399631222*G0_0_1 - 0.0699110399631222*G0_0_2 - 0.0699110399631226*G0_1_0 - 0.139822079926245*G0_1_1 - 0.139822079926245*G0_1_2;
    A[700] = -0.0644569872000423*G0_0_0 + 0.0962940416065395*G0_0_1 + 0.192588083213081*G0_0_2 + 0.0284445813959694*G0_1_0 + 0.124738623002509*G0_1_1 + 0.12473862300251*G0_1_2;
    A[701] = 0.1273155976932*G0_0_0 - 0.00704590548340472*G0_0_1 - 0.049321338383837*G0_0_2 - 0.000685018588664006*G0_1_0 - 0.0429604514890962*G0_1_1 - 0.0429604514890964*G0_1_2;
    A[702] = -0.0641307878721065*G0_0_0 + 0.026487385428357*G0_0_1 + 0.0477555816097474*G0_0_2 - 0.00939454064454091*G0_1_0 + 0.0118736555368495*G0_1_1 + 0.0118736555368492*G0_1_2;
    A[703] = 0.0439716694056969*G0_0_0 - 0.0422754329004319*G0_0_1 - 0.084550865800865*G0_0_2 - 0.000685018588664432*G0_1_0 - 0.0429604514890968*G0_1_1 - 0.042960451489097*G0_1_2;
    A[704] = -0.0292600797158093*G0_0_0 + 0.0084485625935274*G0_0_1 + 0.0142875305635715*G0_0_2 - 0.00329461321214779*G0_1_0 + 0.00254435475789686*G0_1_1 + 0.00254435475789687*G0_1_2;
    A[705] = -0.0187890812890811*G0_0_0 + 0.0212681961813902*G0_0_1 + 0.042536392362781*G0_0_2 - 0.00939454064454057*G0_1_0 + 0.0118736555368495*G0_1_1 + 0.0118736555368498*G0_1_2;
    A[706] = 0.0187890812890818*G0_0_0 + 0.00939454064454131*G0_0_1 + 0.0093945406445413*G0_1_0;
    A[707] = 0.0292600797158088*G0_0_0 + 0.00329461321214709*G0_0_1 + 0.00329461321214792*G0_1_0;
    A[708] = 0.0641307878721065*G0_0_0 + 0.00939454064454053*G0_0_1 + 0.00939454064454097*G0_1_0;
    A[709] = 0.00658922642429567*G0_0_0 + 0.00329461321214803*G0_0_1 + 0.00329461321214812*G0_1_0;
    A[710] = 0.0292600797158092*G0_0_0 + 0.00329461321214814*G0_0_1 + 0.00329461321214773*G0_1_0;
    A[711] = 0.0187890812890812*G0_0_0 + 0.0093945406445404*G0_0_1 + 0.00939454064454056*G0_1_0;
    A[712] = -0.030662736825931*G0_0_0 - 0.0212681961813903*G0_0_1 - 0.0212681961813908*G0_1_0 - 0.0118736555368503*G0_1_1;
    A[713] = 0.064261267603281*G0_0_0 + 0.0422754329004328*G0_0_1 + 0.042275432900433*G0_1_0 + 0.0429604514890975*G0_1_1;
    A[714] = -0.128522535206561*G0_0_0 - 0.0962940416065398*G0_0_1 - 0.0962940416065406*G0_1_0 - 0.12473862300251*G0_1_1;
    A[715] = -0.0091335811821923*G0_0_0 - 0.00583896797004482*G0_0_1 - 0.00583896797004379*G0_1_0 - 0.00254435475789616*G0_1_1;
    A[716] = 0.0642612676032809*G0_0_0 + 0.0422754329004321*G0_0_1 + 0.0422754329004323*G0_1_0 + 0.042960451489097*G0_1_1;
    A[717] = -0.0306627368259311*G0_0_0 - 0.0212681961813903*G0_0_1 - 0.0212681961813907*G0_1_0 - 0.0118736555368496*G0_1_1;
    A[718] = -0.0613254736518619*G0_0_0 - 0.02126819618139*G0_0_1 - 0.0425363923627803*G0_0_2 - 0.0212681961813905*G0_1_0 - 0.0118736555368496*G0_1_2;
    A[719] = 0.128522535206562*G0_0_0 + 0.0422754329004328*G0_0_1 + 0.0845508658008656*G0_0_2 + 0.0422754329004326*G0_1_0 + 0.0429604514890974*G0_1_2;
    A[720] = -0.257045070413123*G0_0_0 - 0.0962940416065414*G0_0_1 - 0.192588083213081*G0_0_2 - 0.0962940416065405*G0_1_0 - 0.12473862300251*G0_1_2;
    A[721] = -0.0435476102793807*G0_0_0 - 0.00583896797004494*G0_0_1 - 0.014287530563572*G0_0_2 - 0.00583896797004445*G0_1_0 - 0.00254435475789644*G0_1_2;
    A[722] = 0.176636936077038*G0_0_0 + 0.0422754329004326*G0_0_1 + 0.049321338383837*G0_0_2 + 0.0422754329004323*G0_1_0 + 0.0429604514890963*G0_1_2;
    A[723] = -0.111886369481854*G0_0_0 - 0.0212681961813904*G0_0_1 - 0.0477555816097475*G0_0_2 - 0.0212681961813903*G0_1_0 - 0.0118736555368492*G0_1_2;
    A[724] = -0.050560895829993*G0_0_0 - 0.00260959462348319*G0_0_1 - 0.00260959462348398*G0_1_0;
    A[725] = 0.0962288017409536*G0_0_0 - 0.0352295274170275*G0_0_1 - 0.0352295274170266*G0_1_0;
    A[726] = -0.156575677409009*G0_0_0 - 0.00260959462348404*G0_0_1 - 0.00260959462348398*G0_1_0;
    A[727] = -0.0505608958299926*G0_0_0 - 0.00260959462348252*G0_0_1 - 0.00260959462348368*G0_1_0;
    A[728] = -0.00336507226698189*G0_0_0 - 0.00336507226698188*G0_0_1 - 0.00336507226698185*G0_0_2 - 0.00336507226698191*G0_1_0 - 0.00336507226698186*G0_1_1 - 0.00336507226698183*G0_1_2;
    A[729] = 0.0214691349673986*G0_0_0 - 0.0336585514536901*G0_1_0;
    A[730] = 0.0232632312710434*G0_0_1 - 0.0155544887532734*G0_1_1;
    A[731] = 0.00336507226698193*G0_0_2 + 0.00336507226698188*G0_1_2;
    A[732] = 0.0599162925551804*G0_0_1 + 0.078894569454464*G0_0_2 + 0.0283728175438238*G0_1_1 + 0.0189782768992834*G0_1_2;
    A[733] = 0.00578025209101597*G0_0_1 + 0.00523223722008435*G0_0_2 - 0.000482775005344235*G0_1_1 - 0.000548014870931417*G0_1_2;
    A[734] = 0.0131523569023569*G0_0_1 + 0.0263047138047138*G0_0_2 + 0.0132175967679436*G0_1_1 + 0.0131523569023566*G0_1_2;
    A[735] = -0.00228991928210684*G0_0_1 - 0.0139743792087541*G0_0_2 - 0.00228991928210687*G0_1_1 - 0.0116844599266474*G0_1_2;
    A[736] = -0.0574176057031947*G0_0_0 - 0.0387263842124945*G0_0_2 - 0.0699110399631223*G0_1_0 - 0.108637424175617*G0_1_2;
    A[737] = 0.00506261356955749*G0_0_0 + 0.00480165410720924*G0_0_2 + 0.00480165410720932*G0_1_0 + 0.00960330821441903*G0_1_2;
    A[738] = -0.000482775005344224*G0_0_0 + 0.00578025209101614*G0_0_2 - 0.000548014870931284*G0_1_0 + 0.00523223722008475*G0_1_2;
    A[739] = -0.00228991928210688*G0_0_0 - 0.00887914570640272*G0_0_2 - 0.00228991928210692*G0_1_0 - 0.0111690649885095*G0_1_2;
    A[740] = -0.147063705006411*G0_0_0 - 0.0387263842124946*G0_0_1 + 0.0263830016434185*G0_1_0 - 0.108637424175618*G0_1_1;
    A[741] = 0.304226541205704*G0_0_0 + 0.0781312630270952*G0_0_1 + 0.0781312630270952*G0_1_0 + 0.281001149056702*G0_1_1;
    A[742] = -0.204187731314465*G0_0_0 + 0.0153052724667309*G0_0_1 - 0.0364951808094162*G0_1_0 - 0.204187731314465*G0_1_1;
    A[743] = 0.0788945694544641*G0_0_0 - 0.0246345732456842*G0_0_1 + 0.0189782768992834*G0_1_0 + 0.100717304493345*G0_1_1;
    A[744] = 0.00228991928210677*G0_0_0 + 0.00228991928210685*G0_0_1 + 0.00228991928210656*G0_1_0 + 0.00228991928210657*G0_1_1;
    A[745] = 0.0017875723170864*G0_0_0 + 0.00178757231708626*G0_0_1 + 0.00178757231708656*G0_1_0 + 0.00178757231708645*G0_1_1;
    A[746] = 0.00178757231708606*G0_0_0 + 0.00178757231708617*G0_0_1 + 0.00178757231708572*G0_1_0 + 0.0017875723170858*G0_1_1;
    A[747] = 0.00228991928210685*G0_0_0 + 0.00228991928210685*G0_0_1 + 0.00228991928210689*G0_1_0 + 0.0022899192821069*G0_1_1;
    A[748] = 0.0139743792087538*G0_0_0 + 0.011684459926647*G0_0_1 + 0.0139743792087538*G0_0_2 + 0.011684459926647*G0_1_0 + 0.00939454064454031*G0_1_1 + 0.011684459926647*G0_1_2;
    A[749] = -0.026304713804713*G0_0_0 - 0.0131523569023561*G0_0_1 - 0.0263047138047132*G0_0_2 - 0.0131523569023563*G0_1_0 + 6.52398655877307e-05*G0_1_1 - 0.0131523569023563*G0_1_2;
    A[750] = -0.00523223722008506*G0_0_0 + 0.000548014870930781*G0_0_1 - 0.00523223722008487*G0_0_2 + 0.000548014870931044*G0_1_0 + 6.52398655863469e-05*G0_1_1 + 0.000548014870931197*G0_1_2;
    A[751] = -0.0788945694544642*G0_0_0 - 0.0189782768992833*G0_0_1 - 0.0788945694544641*G0_0_2 - 0.0189782768992835*G0_1_0 + 0.00939454064454084*G0_1_1 - 0.0189782768992835*G0_1_2;
    A[752] = 0.00658922642429582*G0_0_0 + 0.00887914570640262*G0_0_1 + 0.00887914570640258*G0_0_2 + 0.00887914570640253*G0_1_0 + 0.0111690649885091*G0_1_1 + 0.0111690649885091*G0_1_2;
    A[753] = -0.00626302709636057*G0_0_0 - 0.005780252091016*G0_0_1 - 0.00578025209101596*G0_0_2 - 0.00578025209101585*G0_1_0 - 0.00523223722008417*G0_1_1 - 0.00523223722008415*G0_1_2;
    A[754] = 0.000260959462348416*G0_0_0 - 0.00480165410720971*G0_0_1 - 0.00480165410720971*G0_0_2 - 0.0048016541072098*G0_1_0 - 0.00960330821441933*G0_1_1 - 0.00960330821441933*G0_1_2;
    A[755] = -0.0186912214907003*G0_0_0 + 0.0387263842124946*G0_0_1 + 0.0387263842124945*G0_0_2 + 0.0387263842124951*G0_1_0 + 0.108637424175618*G0_1_1 + 0.108637424175618*G0_1_2;
    A[756] = 0.132306447410612*G0_0_0 - 0.0623693115012545*G0_0_1 + 0.0109602974186314*G0_0_2 + 0.0733296089198855*G0_1_0 - 0.12473862300251*G0_1_1 + 0.146659217839773*G0_1_2;
    A[757] = -0.166948816037355*G0_0_0 + 0.0838658472121995*G0_0_1 + 0.1677316944244*G0_0_2 - 0.00616516729797966*G0_1_0 + 0.182410664181495*G0_1_1 + 0.0777006799142207*G0_1_2;
    A[758] = 0.127087258163645*G0_0_0 - 0.0463855444324186*G0_0_1 - 0.0823327103709038*G0_0_2 + 0.0377738821749235*G0_1_0 - 0.0510175748891017*G0_1_1 + 0.00182671623643848*G0_1_2;
    A[759] = -0.0436454700777611*G0_0_0 - 0.00649136662591603*G0_0_1 - 0.080832193462402*G0_0_2 - 0.0338921101724917*G0_1_0 - 0.00352295274170303*G0_1_1 - 0.108232937008979*G0_1_2;
    A[760] = 0.0189848008858429*G0_0_0 - 0.0188869410874609*G0_0_1 - 0.0377738821749227*G0_0_2 + 0.00949240044292089*G0_1_0 - 0.0155923278753145*G0_1_1 - 0.00939454064454062*G0_1_2;
    A[761] = 0.0148746893538559*G0_0_0 + 0.00123955744615484*G0_0_1 + 0.0403182369328202*G0_0_2 + 0.0149399292194425*G0_1_0 + 0.00117431758056772*G0_1_1 + 0.0540186087061081*G0_1_2;
    A[762] = -0.0148746893538566*G0_0_0 - 0.0149399292194436*G0_0_1 - 0.0149399292194436*G0_1_0 - 0.0150051690850304*G0_1_1;
    A[763] = -0.0189848008858434*G0_0_0 - 0.00949240044292086*G0_0_1 - 0.00949240044292182*G0_1_0 - 0.000652398655870405*G0_1_1;
    A[764] = -0.127087258163645*G0_0_0 - 0.037773882174923*G0_0_1 - 0.0377738821749235*G0_1_0 - 0.0150051690850297*G0_1_1;
    A[765] = 0.00450155072550928*G0_0_0 + 0.00192457603481906*G0_0_1 + 0.00192457603481866*G0_1_0 - 0.000652398655871104*G0_1_1;
    A[766] = -0.0189848008858428*G0_0_0 - 0.00949240044292121*G0_0_1 - 0.00949240044292094*G0_1_0 - 0.000652398655871075*G0_1_1;
    A[767] = -0.014874689353856*G0_0_0 - 0.0149399292194431*G0_0_1 - 0.0149399292194425*G0_1_0 - 0.0150051690850295*G0_1_1;
    A[768] = -0.00130479731174201*G0_0_0 - 0.00123955744615494*G0_0_1 - 0.00123955744615449*G0_1_0 - 0.00117431758056745*G0_1_1;
    A[769] = 0.00880738185425693*G0_0_0 + 0.00649136662591516*G0_0_1 + 0.00649136662591492*G0_1_0 + 0.00352295274170224*G0_1_1;
    A[770] = 0.0476251018785729*G0_0_0 + 0.0623693115012548*G0_0_1 + 0.0623693115012553*G0_1_0 + 0.124738623002511*G0_1_1;
    A[771] = -0.0156575677409007*G0_0_0 - 0.0130805930502105*G0_0_1 - 0.0130805930502111*G0_1_0 - 0.0105036183595214*G0_1_1;
    A[772] = 0.00880738185425703*G0_0_0 + 0.00649136662591594*G0_0_1 + 0.00649136662591527*G0_1_0 + 0.00352295274170293*G0_1_1;
    A[773] = -0.00130479731174182*G0_0_0 - 0.00123955744615474*G0_0_1 - 0.00123955744615439*G0_1_0 - 0.00117431758056763*G0_1_1;
    A[774] = -0.0254435475789639*G0_0_0 - 0.0390786794866657*G0_0_1 - 0.0403182369328203*G0_0_2 - 0.0390786794866652*G0_1_0 - 0.0528442911255407*G0_1_1 - 0.0540186087061083*G0_1_2;
    A[775] = 0.03718672338464*G0_0_0 + 0.0743408268364861*G0_0_1 + 0.080832193462401*G0_0_2 + 0.0743408268364863*G0_1_0 + 0.104709984267275*G0_1_1 + 0.108232937008978*G0_1_2;
    A[776] = 0.121346149991981*G0_0_0 - 0.0733296089198854*G0_0_1 - 0.010960297418631*G0_0_2 - 0.0733296089198867*G0_1_0 - 0.271397840842283*G0_1_1 - 0.146659217839772*G0_1_2;
    A[777] = 0.0567586830607654*G0_0_0 + 0.0188869410874619*G0_0_1 + 0.0377738821749228*G0_0_2 + 0.0188869410874614*G0_1_0 - 0.00619778723077303*G0_1_1 + 0.00939454064454036*G0_1_2;
    A[778] = -0.334680510461756*G0_0_0 - 0.0838658472122012*G0_0_1 - 0.1677316944244*G0_0_2 - 0.0838658472122003*G0_1_0 + 0.104709984267275*G0_1_1 - 0.0777006799142206*G0_1_2;
    A[779] = 0.209419968534549*G0_0_0 + 0.0359471659384855*G0_0_1 + 0.0823327103709037*G0_0_2 + 0.0359471659384851*G0_1_0 - 0.0528442911255405*G0_1_1 - 0.00182671623643865*G0_1_2;
    A[780] = 0.0391439193522531*G0_0_0 + 0.0319675341376723*G0_0_1 + 0.0319675341376736*G0_1_0 + 0.0260959462348346*G0_1_1;
    A[781] = -0.156575677409008*G0_0_0 - 0.0903572138381153*G0_0_1 - 0.0903572138381152*G0_1_0 - 0.185933616923198*G0_1_1;
    A[782] = 0.185933616923199*G0_0_0 + 0.0156575677409019*G0_0_1 + 0.0156575677409014*G0_1_0 + 0.0260959462348351*G0_1_1;
    A[783] = 0.0391439193522517*G0_0_0 + 0.0319675341376712*G0_0_1 + 0.0319675341376731*G0_1_0 + 0.0260959462348357*G0_1_1;
    A[784] = -0.00336507226698199*G0_0_0 - 0.003365072266982*G0_0_1 - 0.00336507226698199*G0_0_2 - 0.00336507226698195*G0_1_0 - 0.00336507226698199*G0_1_1 - 0.00336507226698198*G0_1_2;
    A[785] = -0.0155544887532734*G0_0_0 + 0.0232632312710436*G0_1_0;
    A[786] = -0.03365855145369*G0_0_1 + 0.0214691349673987*G0_1_1;
    A[787] = 0.00336507226698199*G0_0_2 + 0.00336507226698204*G0_1_2;
    A[788] = -0.069911039963122*G0_0_1 - 0.108637424175617*G0_0_2 - 0.0574176057031946*G0_1_1 - 0.0387263842124945*G0_1_2;
    A[789] = 0.00480165410720929*G0_0_1 + 0.00960330821441925*G0_0_2 + 0.00506261356955741*G0_1_1 + 0.00480165410720954*G0_1_2;
    A[790] = -0.00054801487093145*G0_0_1 + 0.00523223722008417*G0_0_2 - 0.000482775005343889*G0_1_1 + 0.00578025209101611*G0_1_2;
    A[791] = -0.0022899192821067*G0_0_1 - 0.0111690649885092*G0_0_2 - 0.00228991928210666*G0_1_1 - 0.0088791457064024*G0_1_2;
    A[792] = 0.0283728175438239*G0_0_0 + 0.0189782768992832*G0_0_2 + 0.0599162925551808*G0_1_0 + 0.0788945694544644*G0_1_2;
    A[793] = -0.000482775005344233*G0_0_0 - 0.000548014870931196*G0_0_2 + 0.005780252091016*G0_1_0 + 0.00523223722008447*G0_1_2;
    A[794] = 0.0132175967679437*G0_0_0 + 0.0131523569023565*G0_0_2 + 0.0131523569023563*G0_1_0 + 0.0263047138047131*G0_1_2;
    A[795] = -0.00228991928210666*G0_0_0 - 0.0116844599266472*G0_0_2 - 0.00228991928210658*G0_1_0 - 0.013974379208754*G0_1_2;
    A[796] = 0.100717304493345*G0_0_0 + 0.0189782768992833*G0_0_1 - 0.0246345732456843*G0_1_0 + 0.0788945694544645*G0_1_1;
    A[797] = -0.204187731314465*G0_0_0 - 0.0364951808094162*G0_0_1 + 0.0153052724667309*G0_1_0 - 0.204187731314465*G0_1_1;
    A[798] = 0.281001149056701*G0_0_0 + 0.0781312630270951*G0_0_1 + 0.0781312630270951*G0_1_0 + 0.304226541205705*G0_1_1;
    A[799] = -0.108637424175617*G0_0_0 + 0.0263830016434184*G0_0_1 - 0.0387263842124945*G0_1_0 - 0.147063705006411*G0_1_1;
    A[800] = 0.00228991928210667*G0_0_0 + 0.00228991928210661*G0_0_1 + 0.00228991928210686*G0_1_0 + 0.00228991928210687*G0_1_1;
    A[801] = 0.001787572317086*G0_0_0 + 0.00178757231708613*G0_0_1 + 0.00178757231708586*G0_1_0 + 0.001787572317086*G0_1_1;
    A[802] = 0.00178757231708638*G0_0_0 + 0.00178757231708631*G0_0_1 + 0.00178757231708654*G0_1_0 + 0.00178757231708648*G0_1_1;
    A[803] = 0.00228991928210666*G0_0_0 + 0.00228991928210667*G0_0_1 + 0.00228991928210659*G0_1_0 + 0.00228991928210659*G0_1_1;
    A[804] = 0.0111690649885094*G0_0_0 + 0.00887914570640275*G0_0_1 + 0.0111690649885093*G0_0_2 + 0.00887914570640275*G0_1_0 + 0.00658922642429584*G0_1_1 + 0.00887914570640274*G0_1_2;
    A[805] = -0.00523223722008482*G0_0_0 - 0.00578025209101645*G0_0_1 - 0.0052322372200845*G0_0_2 - 0.00578025209101649*G0_1_0 - 0.00626302709636073*G0_1_1 - 0.00578025209101641*G0_1_2;
    A[806] = -0.00960330821441854*G0_0_0 - 0.00480165410720897*G0_0_1 - 0.00960330821441881*G0_0_2 - 0.0048016541072092*G0_1_0 + 0.000260959462348779*G0_1_1 - 0.00480165410720937*G0_1_2;
    A[807] = 0.108637424175617*G0_0_0 + 0.0387263842124947*G0_0_1 + 0.108637424175617*G0_0_2 + 0.0387263842124945*G0_1_0 - 0.0186912214907006*G0_1_1 + 0.0387263842124946*G0_1_2;
    A[808] = 0.00939454064454038*G0_0_0 + 0.011684459926647*G0_0_1 + 0.011684459926647*G0_0_2 + 0.0116844599266472*G0_1_0 + 0.0139743792087542*G0_1_1 + 0.0139743792087542*G0_1_2;
    A[809] = 6.52398655875047e-05*G0_0_0 - 0.0131523569023565*G0_0_1 - 0.0131523569023565*G0_0_2 - 0.0131523569023568*G0_1_0 - 0.0263047138047136*G0_1_1 - 0.0263047138047135*G0_1_2;
    A[810] = 6.52398655867894e-05*G0_0_0 + 0.000548014870931483*G0_0_1 + 0.000548014870931463*G0_0_2 + 0.000548014870931659*G0_1_0 - 0.0052322372200843*G0_1_1 - 0.00523223722008431*G0_1_2;
    A[811] = 0.00939454064454068*G0_0_0 - 0.0189782768992831*G0_0_1 - 0.0189782768992831*G0_0_2 - 0.0189782768992837*G0_1_0 - 0.0788945694544643*G0_1_1 - 0.0788945694544643*G0_1_2;
    A[812] = -0.0510175748891017*G0_0_0 + 0.0377738821749227*G0_0_1 + 0.0018267162364378*G0_0_2 - 0.0463855444324188*G0_1_0 + 0.127087258163645*G0_1_1 - 0.0823327103709044*G0_1_2;
    A[813] = 0.182410664181495*G0_0_0 - 0.00616516729797893*G0_0_1 + 0.0777006799142211*G0_0_2 + 0.0838658472122006*G0_1_0 - 0.166948816037356*G0_1_1 + 0.167731694424401*G0_1_2;
    A[814] = -0.12473862300251*G0_0_0 + 0.0733296089198856*G0_0_1 + 0.146659217839772*G0_0_2 - 0.0623693115012548*G0_1_0 + 0.132306447410612*G0_1_1 + 0.0109602974186309*G0_1_2;
    A[815] = -0.0155923278753137*G0_0_0 + 0.00949240044292181*G0_0_1 - 0.00939454064453967*G0_0_2 - 0.0188869410874617*G0_1_0 + 0.0189848008858422*G0_1_1 - 0.037773882174923*G0_1_2;
    A[816] = -0.00352295274170305*G0_0_0 - 0.0338921101724919*G0_0_1 - 0.108232937008978*G0_0_2 - 0.00649136662591492*G0_1_0 - 0.04364547007776*G0_1_1 - 0.0808321934624007*G0_1_2;
    A[817] = 0.00117431758056757*G0_0_0 + 0.0149399292194427*G0_0_1 + 0.054018608706108*G0_0_2 + 0.00123955744615544*G0_1_0 + 0.0148746893538557*G0_1_1 + 0.0403182369328203*G0_1_2;
    A[818] = -0.00117431758056706*G0_0_0 - 0.00123955744615395*G0_0_1 - 0.00123955744615476*G0_1_0 - 0.00130479731174177*G0_1_1;
    A[819] = 0.00352295274170322*G0_0_0 + 0.00649136662591485*G0_0_1 + 0.00649136662591566*G0_1_0 + 0.00880738185425674*G0_1_1;
    A[820] = 0.12473862300251*G0_0_0 + 0.0623693115012548*G0_0_1 + 0.0623693115012548*G0_1_0 + 0.047625101878573*G0_1_1;
    A[821] = -0.0105036183595216*G0_0_0 - 0.0130805930502114*G0_0_1 - 0.0130805930502109*G0_1_0 - 0.0156575677409008*G0_1_1;
    A[822] = 0.00352295274170293*G0_0_0 + 0.0064913666259158*G0_0_1 + 0.0064913666259149*G0_1_0 + 0.00880738185425734*G0_1_1;
    A[823] = -0.00117431758056754*G0_0_0 - 0.00123955744615478*G0_0_1 - 0.00123955744615543*G0_1_0 - 0.00130479731174248*G0_1_1;
    A[824] = -0.0150051690850294*G0_0_0 - 0.0149399292194425*G0_0_1 - 0.0149399292194427*G0_1_0 - 0.0148746893538558*G0_1_1;
    A[825] = -0.000652398655871144*G0_0_0 - 0.00949240044292105*G0_0_1 - 0.00949240044292112*G0_1_0 - 0.0189848008858418*G0_1_1;
    A[826] = -0.0150051690850298*G0_0_0 - 0.037773882174923*G0_0_1 - 0.0377738821749238*G0_1_0 - 0.127087258163646*G0_1_1;
    A[827] = -0.000652398655871334*G0_0_0 + 0.00192457603481818*G0_0_1 + 0.00192457603481869*G0_1_0 + 0.00450155072550843*G0_1_1;
    A[828] = -0.000652398655870629*G0_0_0 - 0.00949240044292172*G0_0_1 - 0.00949240044292058*G0_1_0 - 0.0189848008858421*G0_1_1;
    A[829] = -0.01500516908503*G0_0_0 - 0.0149399292194428*G0_0_1 - 0.0149399292194429*G0_1_0 - 0.0148746893538557*G0_1_1;
    A[830] = -0.0528442911255402*G0_0_0 - 0.0390786794866642*G0_0_1 - 0.0540186087061073*G0_0_2 - 0.0390786794866648*G0_1_0 - 0.0254435475789638*G0_1_1 - 0.0403182369328196*G0_1_2;
    A[831] = -0.00619778723077408*G0_0_0 + 0.018886941087461*G0_0_1 + 0.0093945406445403*G0_0_2 + 0.018886941087461*G0_1_0 + 0.0567586830607658*G0_1_1 + 0.0377738821749239*G0_1_2;
    A[832] = -0.0528442911255397*G0_0_0 + 0.0359471659384845*G0_0_1 - 0.00182671623643819*G0_0_2 + 0.0359471659384853*G0_1_0 + 0.20941996853455*G0_1_1 + 0.082332710370904*G0_1_2;
    A[833] = 0.104709984267275*G0_0_0 + 0.0743408268364853*G0_0_1 + 0.108232937008978*G0_0_2 + 0.0743408268364859*G0_1_0 + 0.0371867233846391*G0_1_1 + 0.0808321934624006*G0_1_2;
    A[834] = 0.104709984267274*G0_0_0 - 0.0838658472121995*G0_0_1 - 0.0777006799142212*G0_0_2 - 0.0838658472122004*G0_1_0 - 0.334680510461757*G0_1_1 - 0.167731694424401*G0_1_2;
    A[835] = -0.271397840842283*G0_0_0 - 0.0733296089198866*G0_0_1 - 0.146659217839772*G0_0_2 - 0.0733296089198859*G0_1_0 + 0.121346149991982*G0_1_1 - 0.0109602974186309*G0_1_2;
    A[836] = 0.0260959462348338*G0_0_0 + 0.0319675341376721*G0_0_1 + 0.0319675341376724*G0_1_0 + 0.0391439193522521*G0_1_1;
    A[837] = 0.0260959462348348*G0_0_0 + 0.0156575677409006*G0_0_1 + 0.015657567740901*G0_1_0 + 0.185933616923198*G0_1_1;
    A[838] = -0.185933616923199*G0_0_0 - 0.0903572138381157*G0_0_1 - 0.0903572138381161*G0_1_0 - 0.156575677409009*G0_1_1;
    A[839] = 0.0260959462348355*G0_0_0 + 0.0319675341376738*G0_0_1 + 0.0319675341376725*G0_1_0 + 0.0391439193522515*G0_1_1;
    A[840] = -0.00480034930989779*G0_0_0 - 0.00480034930989781*G0_0_1 - 0.0048003493098978*G0_0_2 - 0.00480034930989785*G0_1_0 - 0.00480034930989785*G0_1_1 - 0.00480034930989784*G0_1_2;
    A[841] = 0.00480034930989781*G0_0_0 - 0.0105310191030677*G0_1_0;
    A[842] = 0.0372571824394737*G0_0_1 - 0.017870503981615*G0_1_1;
    A[843] = 0.00480034930989787*G0_0_2 + 0.0048003493098978*G0_1_2;
    A[844] = 0.069911039963122*G0_0_1 + 0.139822079926245*G0_0_2 + 0.0895482395048353*G0_1_1 + 0.0699110399631223*G0_1_2;
    A[845] = -0.0387263842124947*G0_0_1 - 0.108637424175618*G0_0_2 - 0.0574176057031952*G0_1_1 - 0.0699110399631227*G0_1_2;
    A[846] = 0.0189782768992835*G0_0_1 + 0.0788945694544649*G0_0_2 + 0.0283728175438239*G0_1_1 + 0.059916292555181*G0_1_2;
    A[847] = -0.00537576492437598*G0_0_1 - 0.0328352243499813*G0_0_2 - 0.00537576492437618*G0_1_1 - 0.0274594594256053*G0_1_2;
    A[848] = -0.00537576492437576*G0_0_0 - 0.00537576492437586*G0_0_2 - 0.0274594594256048*G0_1_0 - 0.0328352243499811*G0_1_2;
    A[849] = -0.00228991928210703*G0_0_0 - 0.00228991928210674*G0_0_2 - 0.00887914570640286*G0_1_0 - 0.0111690649885091*G0_1_2;
    A[850] = -0.00228991928210664*G0_0_0 - 0.00228991928210682*G0_0_2 - 0.0116844599266468*G0_1_0 - 0.013974379208754*G0_1_2;
    A[851] = -0.00537576492437597*G0_0_0 - 0.00537576492437593*G0_0_2 - 0.00537576492437621*G0_1_0 - 0.010751529848752*G0_1_2;
    A[852] = -0.0328352243499806*G0_0_0 - 0.00537576492437585*G0_0_1 + 0.0209159009072202*G0_1_0 - 0.0328352243499811*G0_1_1;
    A[853] = 0.0788945694544641*G0_0_0 + 0.0189782768992834*G0_0_1 - 0.0246345732456842*G0_1_0 + 0.100717304493345*G0_1_1;
    A[854] = -0.108637424175617*G0_0_0 - 0.0387263842124945*G0_0_1 + 0.0263830016434184*G0_1_0 - 0.147063705006411*G0_1_1;
    A[855] = 0.139822079926245*G0_0_0 + 0.0699110399631223*G0_0_1 + 0.0699110399631223*G0_1_0 + 0.179096479009671*G0_1_1;
    A[856] = 0.00537576492437577*G0_0_0 + 0.00537576492437583*G0_0_1 + 0.00537576492437571*G0_1_0 + 0.0053757649243757*G0_1_1;
    A[857] = 0.00228991928210714*G0_0_0 + 0.00228991928210705*G0_0_1 + 0.0022899192821073*G0_1_0 + 0.00228991928210722*G0_1_1;
    A[858] = 0.00228991928210647*G0_0_0 + 0.0022899192821065*G0_0_1 + 0.00228991928210646*G0_1_0 + 0.00228991928210646*G0_1_1;
    A[859] = 0.00537576492437597*G0_0_0 + 0.005375764924376*G0_0_1 + 0.00537576492437622*G0_1_0 + 0.00537576492437627*G0_1_1;
    A[860] = 0.0328352243499807*G0_0_0 + 0.0274594594256048*G0_0_1 + 0.0328352243499808*G0_0_2 + 0.0274594594256047*G0_1_0 + 0.0220836945012289*G0_1_1 + 0.0274594594256046*G0_1_2;
    A[861] = -0.0788945694544641*G0_0_0 - 0.0599162925551807*G0_0_1 - 0.0788945694544643*G0_0_2 - 0.0599162925551804*G0_1_0 - 0.0315434750113566*G0_1_1 - 0.0599162925551804*G0_1_2;
    A[862] = 0.108637424175617*G0_0_0 + 0.0699110399631224*G0_0_1 + 0.108637424175617*G0_0_2 + 0.0699110399631224*G0_1_0 + 0.0124934342599272*G0_1_1 + 0.0699110399631224*G0_1_2;
    A[863] = -0.139822079926245*G0_0_0 - 0.0699110399631225*G0_0_1 - 0.139822079926245*G0_0_2 - 0.0699110399631223*G0_1_0 + 0.0196371995417133*G0_1_1 - 0.0699110399631223*G0_1_2;
    A[864] = 0.00537576492437607*G0_0_1 + 0.00537576492437604*G0_0_2 + 0.00537576492437599*G0_1_0 + 0.0107515298487518*G0_1_1 + 0.0107515298487518*G0_1_2;
    A[865] = 0.00228991928210665*G0_0_1 + 0.00228991928210666*G0_0_2 + 0.00228991928210664*G0_1_0 + 0.0139743792087542*G0_1_1 + 0.0139743792087541*G0_1_2;
    A[866] = 0.00228991928210685*G0_0_1 + 0.00228991928210687*G0_0_2 + 0.0022899192821068*G0_1_0 + 0.0111690649885094*G0_1_1 + 0.0111690649885094*G0_1_2;
    A[867] = 0.00537576492437586*G0_0_1 + 0.00537576492437581*G0_0_2 + 0.00537576492437612*G0_1_0 + 0.0328352243499811*G0_1_1 + 0.0328352243499811*G0_1_2;
    A[868] = 0.0118736555368494*G0_0_0 - 0.00939454064454034*G0_0_1 + 0.0118736555368501*G0_0_2 + 0.026487385428357*G0_1_0 - 0.0641307878721067*G0_1_1 + 0.0477555816097477*G0_1_2;
    A[869] = -0.0429604514890966*G0_0_0 - 0.000685018588664262*G0_0_1 - 0.0429604514890967*G0_0_2 - 0.00704590548340551*G0_1_0 + 0.127315597693201*G0_1_1 - 0.0493213383838374*G0_1_2;
    A[870] = 0.124738623002509*G0_0_0 + 0.028444581395969*G0_0_1 + 0.12473862300251*G0_0_2 + 0.0962940416065395*G0_1_0 - 0.0644569872000432*G0_1_1 + 0.192588083213081*G0_1_2;
    A[871] = 0.00254435475789698*G0_0_0 - 0.00329461321214813*G0_0_1 + 0.00254435475789607*G0_0_2 + 0.00844856259352856*G0_1_0 - 0.0292600797158081*G0_1_1 + 0.0142875305635721*G0_1_2;
    A[872] = -0.0429604514890969*G0_0_0 - 0.000685018588664207*G0_0_1 - 0.0429604514890975*G0_0_2 - 0.0422754329004326*G0_1_0 + 0.0439716694056962*G0_1_1 - 0.084550865800866*G0_1_2;
    A[873] = 0.0118736555368501*G0_0_0 - 0.00939454064454017*G0_0_1 + 0.0118736555368509*G0_0_2 + 0.0212681961813898*G0_1_0 - 0.018789081289081*G0_1_1 + 0.0425363923627813*G0_1_2;
    A[874] = -0.0118736555368494*G0_0_0 - 0.0212681961813902*G0_0_1 - 0.0212681961813894*G0_1_0 - 0.0306627368259303*G0_1_1;
    A[875] = 0.0429604514890963*G0_0_0 + 0.0422754329004324*G0_0_1 + 0.0422754329004318*G0_1_0 + 0.0642612676032806*G0_1_1;
    A[876] = -0.124738623002509*G0_0_0 - 0.0962940416065402*G0_0_1 - 0.0962940416065395*G0_1_0 - 0.128522535206561*G0_1_1;
    A[877] = -0.00254435475789634*G0_0_0 - 0.00583896797004456*G0_0_1 - 0.00583896797004461*G0_1_0 - 0.00913358118219294*G0_1_1;
    A[878] = 0.0429604514890969*G0_0_0 + 0.0422754329004323*G0_0_1 + 0.0422754329004326*G0_1_0 + 0.0642612676032809*G0_1_1;
    A[879] = -0.01187365553685*G0_0_0 - 0.0212681961813901*G0_0_1 - 0.0212681961813898*G0_1_0 - 0.0306627368259305*G0_1_1;
    A[880] = 0.00939454064454068*G0_0_1 + 0.00939454064454085*G0_1_0 + 0.0187890812890817*G0_1_1;
    A[881] = 0.00329461321214776*G0_0_1 + 0.00329461321214778*G0_1_0 + 0.0292600797158083*G0_1_1;
    A[882] = 0.00939454064454032*G0_0_1 + 0.00939454064454041*G0_1_0 + 0.0641307878721069*G0_1_1;
    A[883] = 0.00329461321214832*G0_0_1 + 0.00329461321214779*G0_1_0 + 0.00658922642429623*G0_1_1;
    A[884] = 0.00329461321214817*G0_0_1 + 0.0032946132121478*G0_1_0 + 0.0292600797158081*G0_1_1;
    A[885] = 0.00939454064454017*G0_0_1 + 0.00939454064454031*G0_1_0 + 0.0187890812890809*G0_1_1;
    A[886] = -0.0212681961813906*G0_0_1 - 0.0118736555368498*G0_0_2 - 0.02126819618139*G0_1_0 - 0.0613254736518615*G0_1_1 - 0.0425363923627802*G0_1_2;
    A[887] = -0.00583896797004401*G0_0_1 - 0.00254435475789632*G0_0_2 - 0.00583896797004434*G0_1_0 - 0.043547610279381*G0_1_1 - 0.0142875305635725*G0_1_2;
    A[888] = -0.0212681961813904*G0_0_1 - 0.01187365553685*G0_0_2 - 0.0212681961813905*G0_1_0 - 0.111886369481855*G0_1_1 - 0.0477555816097476*G0_1_2;
    A[889] = 0.0422754329004325*G0_0_1 + 0.0429604514890965*G0_0_2 + 0.0422754329004323*G0_1_0 + 0.128522535206562*G0_1_1 + 0.0845508658008649*G0_1_2;
    A[890] = 0.042275432900432*G0_0_1 + 0.042960451489097*G0_0_2 + 0.0422754329004323*G0_1_0 + 0.176636936077038*G0_1_1 + 0.0493213383838377*G0_1_2;
    A[891] = -0.0962940416065401*G0_0_1 - 0.12473862300251*G0_0_2 - 0.0962940416065408*G0_1_0 - 0.257045070413123*G0_1_1 - 0.192588083213081*G0_1_2;
    A[892] = -0.00260959462348385*G0_0_1 - 0.00260959462348476*G0_1_0 - 0.0505608958299937*G0_1_1;
    A[893] = -0.00260959462348347*G0_0_1 - 0.00260959462348286*G0_1_0 - 0.156575677409009*G0_1_1;
    A[894] = -0.0352295274170257*G0_0_1 - 0.0352295274170262*G0_1_0 + 0.0962288017409552*G0_1_1;
    A[895] = -0.00260959462348441*G0_0_1 - 0.00260959462348394*G0_1_0 - 0.0505608958299925*G0_1_1;
    A[896] = -0.0178705039816149*G0_0_0 - 0.0178705039816149*G0_0_1 - 0.0178705039816149*G0_0_2 - 0.0178705039816148*G0_1_0 - 0.0178705039816149*G0_1_1 - 0.0178705039816149*G0_1_2 - 0.0551276864210886*G0_2_0 - 0.0551276864210886*G0_2_1 - 0.0551276864210886*G0_2_2;
    A[897] = -0.00480034930989787*G0_0_0 - 0.00480034930989784*G0_1_0;
    A[898] = -0.00480034930989791*G0_0_1 - 0.00480034930989787*G0_1_1;
    A[899] = 0.0105310191030678*G0_0_2 + 0.0105310191030679*G0_1_2 + 0.0153313684129657*G0_2_2;
    A[900] = 0.0107515298487517*G0_0_1 + 0.00537576492437579*G0_0_2 + 0.0107515298487517*G0_1_1 + 0.00537576492437584*G0_1_2 + 0.00537576492437594*G0_2_1;
    A[901] = 0.0139743792087547*G0_0_1 + 0.0116844599266476*G0_0_2 + 0.0139743792087547*G0_1_1 + 0.0116844599266475*G0_1_2 + 0.0116844599266477*G0_2_1 + 0.00939454064454094*G0_2_2;
    A[902] = 0.0111690649885085*G0_0_1 + 0.00887914570640245*G0_0_2 + 0.0111690649885086*G0_1_1 + 0.00887914570640257*G0_1_2 + 0.00887914570640205*G0_2_1 + 0.0065892264242955*G0_2_2;
    A[903] = 0.0328352243499816*G0_0_1 + 0.0274594594256051*G0_0_2 + 0.0328352243499816*G0_1_1 + 0.0274594594256051*G0_1_2 + 0.0274594594256056*G0_2_1 + 0.0220836945012294*G0_2_2;
    A[904] = 0.0107515298487519*G0_0_0 + 0.00537576492437608*G0_0_2 + 0.0107515298487518*G0_1_0 + 0.00537576492437608*G0_1_2 + 0.00537576492437562*G0_2_0;
    A[905] = 0.013974379208754*G0_0_0 + 0.0116844599266472*G0_0_2 + 0.0139743792087541*G0_1_0 + 0.0116844599266471*G0_1_2 + 0.0116844599266477*G0_2_0 + 0.00939454064454058*G0_2_2;
    A[906] = 0.011169064988509*G0_0_0 + 0.0088791457064024*G0_0_2 + 0.0111690649885089*G0_1_0 + 0.00887914570640246*G0_1_2 + 0.00887914570640211*G0_2_0 + 0.00658922642429563*G0_2_2;
    A[907] = 0.0328352243499816*G0_0_0 + 0.0274594594256052*G0_0_2 + 0.0328352243499816*G0_1_0 + 0.0274594594256052*G0_1_2 + 0.0274594594256056*G0_2_0 + 0.0220836945012293*G0_2_2;
    A[908] = 0.00537576492437596*G0_0_0 + 0.00537576492437607*G0_0_1 + 0.00537576492437589*G0_1_0 + 0.00537576492437607*G0_1_1;
    A[909] = 0.00228991928210677*G0_0_0 + 0.00228991928210656*G0_0_1 + 0.00228991928210685*G0_1_0 + 0.00228991928210657*G0_1_1;
    A[910] = 0.00228991928210667*G0_0_0 + 0.00228991928210686*G0_0_1 + 0.00228991928210661*G0_1_0 + 0.00228991928210687*G0_1_1;
    A[911] = 0.00537576492437577*G0_0_0 + 0.00537576492437571*G0_0_1 + 0.00537576492437583*G0_1_0 + 0.0053757649243757*G0_1_1;
    A[912] = 0.179096479009672*G0_0_0 + 0.179096479009672*G0_0_1 + 0.109185439046549*G0_0_2 + 0.179096479009672*G0_1_0 + 0.179096479009672*G0_1_1 + 0.109185439046549*G0_1_2 + 0.109185439046549*G0_2_0 + 0.109185439046549*G0_2_1 + 0.179096479009672*G0_2_2;
    A[913] = -0.147063705006412*G0_0_0 - 0.147063705006412*G0_0_1 - 0.173446706649831*G0_0_2 - 0.147063705006413*G0_1_0 - 0.147063705006412*G0_1_1 - 0.173446706649831*G0_1_2 - 0.108337320793918*G0_2_0 - 0.108337320793918*G0_2_1 - 0.243357746612953*G0_2_2;
    A[914] = 0.100717304493346*G0_0_0 + 0.100717304493345*G0_0_1 + 0.12535187773903*G0_0_2 + 0.100717304493346*G0_1_0 + 0.100717304493345*G0_1_1 + 0.12535187773903*G0_1_2 + 0.0817390275940619*G0_2_0 + 0.0817390275940619*G0_2_1 + 0.185268170294211*G0_2_2;
    A[915] = -0.0328352243499816*G0_0_0 - 0.0328352243499815*G0_0_1 - 0.0537511252572017*G0_0_2 - 0.0328352243499816*G0_1_0 - 0.0328352243499815*G0_1_1 - 0.0537511252572017*G0_1_2 - 0.0274594594256056*G0_2_0 - 0.0274594594256056*G0_2_1 - 0.0812105846828067*G0_2_2;
    A[916] = 0.0895482395048359*G0_0_0 + 0.0196371995417132*G0_0_1 + 0.0895482395048359*G0_0_2 + 0.089548239504836*G0_1_0 + 0.0196371995417132*G0_1_1 + 0.089548239504836*G0_1_2 + 0.0196371995417131*G0_2_0 + 0.0895482395048357*G0_2_1 + 0.0196371995417131*G0_2_2;
    A[917] = -0.0574176057031954*G0_0_0 + 0.0124934342599274*G0_0_1 - 0.0574176057031953*G0_0_2 - 0.0574176057031954*G0_1_0 + 0.0124934342599273*G0_1_1 - 0.0574176057031954*G0_1_2 - 0.0186912214907002*G0_2_0 - 0.0574176057031951*G0_2_1 - 0.0186912214907002*G0_2_2;
    A[918] = 0.0283728175438239*G0_0_0 - 0.0315434750113568*G0_0_1 + 0.0283728175438239*G0_0_2 + 0.0283728175438239*G0_1_0 - 0.0315434750113567*G0_1_1 + 0.0283728175438239*G0_1_2 + 0.0093945406445401*G0_2_0 + 0.0283728175438238*G0_2_1 + 0.00939454064454014*G0_2_2;
    A[919] = -0.0053757649243758*G0_0_0 + 0.0220836945012291*G0_0_1 - 0.00537576492437583*G0_0_2 - 0.00537576492437585*G0_1_0 + 0.022083694501229*G0_1_1 - 0.00537576492437589*G0_1_2 - 0.00537576492437567*G0_2_1;
    A[920] = 0.0196371995417131*G0_0_0 + 0.089548239504836*G0_0_1 + 0.089548239504836*G0_0_2 + 0.0196371995417131*G0_1_0 + 0.089548239504836*G0_1_1 + 0.089548239504836*G0_1_2 + 0.0895482395048356*G0_2_0 + 0.0196371995417128*G0_2_1 + 0.0196371995417128*G0_2_2;
    A[921] = 0.0124934342599273*G0_0_0 - 0.0574176057031956*G0_0_1 - 0.0574176057031956*G0_0_2 + 0.0124934342599272*G0_1_0 - 0.0574176057031957*G0_1_1 - 0.0574176057031957*G0_1_2 - 0.0574176057031948*G0_2_0 - 0.0186912214907001*G0_2_1 - 0.0186912214907001*G0_2_2;
    A[922] = -0.0315434750113566*G0_0_0 + 0.0283728175438244*G0_0_1 + 0.0283728175438243*G0_0_2 - 0.0315434750113566*G0_1_0 + 0.0283728175438243*G0_1_1 + 0.0283728175438242*G0_1_2 + 0.0283728175438238*G0_2_0 + 0.00939454064454022*G0_2_1 + 0.0093945406445402*G0_2_2;
    A[923] = 0.022083694501229*G0_0_0 - 0.00537576492437597*G0_0_1 - 0.00537576492437592*G0_0_2 + 0.022083694501229*G0_1_0 - 0.00537576492437597*G0_1_1 - 0.00537576492437592*G0_1_2 - 0.00537576492437586*G0_2_0;
    A[924] = 0.0187890812890804*G0_0_0 + 0.0187890812890805*G0_0_1 + 0.00939454064453988*G0_0_2 + 0.0187890812890806*G0_1_0 + 0.0187890812890805*G0_1_1 + 0.00939454064453991*G0_1_2 + 0.0093945406445409*G0_2_0 + 0.00939454064454038*G0_2_1;
    A[925] = 0.00658922642429553*G0_0_0 + 0.0065892264242958*G0_0_1 + 0.00329461321214782*G0_0_2 + 0.00658922642429545*G0_1_0 + 0.00658922642429585*G0_1_1 + 0.00329461321214783*G0_1_2 + 0.00329461321214762*G0_2_0 + 0.00329461321214824*G0_2_1;
    A[926] = 0.0187890812890809*G0_0_0 + 0.0187890812890808*G0_0_1 + 0.00939454064453985*G0_0_2 + 0.018789081289081*G0_1_0 + 0.0187890812890807*G0_1_1 + 0.00939454064453989*G0_1_2 + 0.00939454064454069*G0_2_0 + 0.00939454064454039*G0_2_1;
    A[927] = 0.029260079715809*G0_0_0 + 0.0292600797158081*G0_0_1 + 0.0259654665036604*G0_0_2 + 0.0292600797158088*G0_1_0 + 0.0292600797158081*G0_1_1 + 0.0259654665036604*G0_1_2 + 0.0259654665036604*G0_2_0 + 0.0259654665036604*G0_2_1 + 0.0226708532915133*G0_2_2;
    A[928] = 0.0292600797158087*G0_0_0 + 0.0292600797158089*G0_0_1 + 0.025965466503661*G0_0_2 + 0.0292600797158088*G0_1_0 + 0.0292600797158089*G0_1_1 + 0.025965466503661*G0_1_2 + 0.0259654665036605*G0_2_0 + 0.0259654665036604*G0_2_1 + 0.0226708532915128*G0_2_2;
    A[929] = 0.0641307878721069*G0_0_0 + 0.0641307878721072*G0_0_1 + 0.0547362472275664*G0_0_2 + 0.064130787872107*G0_1_0 + 0.0641307878721072*G0_1_1 + 0.0547362472275664*G0_1_2 + 0.0547362472275667*G0_2_0 + 0.0547362472275667*G0_2_1 + 0.0453417065830254*G0_2_2;
    A[930] = -0.0644569872000422*G0_0_0 - 0.257045070413124*G0_0_1 - 0.160751028806583*G0_0_2 - 0.0644569872000422*G0_1_0 - 0.257045070413124*G0_1_1 - 0.160751028806583*G0_1_2 - 0.0929015685960128*G0_2_0 - 0.160751028806583*G0_2_1 - 0.0644569872000424*G0_2_2;
    A[931] = 0.0439716694056967*G0_0_0 + 0.128522535206562*G0_0_1 + 0.0862471023061297*G0_0_2 + 0.0439716694056968*G0_1_0 + 0.128522535206562*G0_1_1 + 0.0862471023061297*G0_1_2 + 0.0446566879943616*G0_2_0 + 0.0862471023061298*G0_2_1 + 0.0439716694056971*G0_2_2;
    A[932] = -0.018789081289081*G0_0_0 - 0.0613254736518618*G0_0_1 - 0.0400572774704716*G0_0_2 - 0.0187890812890811*G0_1_0 - 0.0613254736518617*G0_1_1 - 0.0400572774704715*G0_1_2 - 0.00939454064454072*G0_2_0 - 0.0400572774704719*G0_2_1 - 0.0187890812890811*G0_2_2;
    A[933] = 0.127315597693201*G0_0_0 + 0.176636936077039*G0_0_1 + 0.134361503176606*G0_0_2 + 0.127315597693201*G0_1_0 + 0.176636936077039*G0_1_1 + 0.134361503176606*G0_1_2 + 0.128000616281866*G0_2_0 + 0.134361503176607*G0_2_1 + 0.0920860702761741*G0_2_2;
    A[934] = -0.0292600797158088*G0_0_0 - 0.0435476102793812*G0_0_1 - 0.0377086423093366*G0_0_2 - 0.0292600797158088*G0_1_0 - 0.0435476102793811*G0_1_1 - 0.0377086423093368*G0_1_2 - 0.0259654665036606*G0_2_0 - 0.0377086423093367*G0_2_1 - 0.0318696743392923*G0_2_2;
    A[935] = -0.064130787872107*G0_0_0 - 0.111886369481854*G0_0_1 - 0.0906181733004641*G0_0_2 - 0.064130787872107*G0_1_0 - 0.111886369481854*G0_1_1 - 0.0906181733004641*G0_1_2 - 0.0547362472275668*G0_2_0 - 0.0906181733004642*G0_2_1 - 0.0693499771190741*G0_2_2;
    A[936] = -0.257045070413124*G0_0_0 - 0.0644569872000419*G0_0_1 - 0.160751028806583*G0_0_2 - 0.257045070413124*G0_1_0 - 0.064456987200042*G0_1_1 - 0.160751028806583*G0_1_2 - 0.160751028806583*G0_2_0 - 0.0929015685960123*G0_2_1 - 0.0644569872000416*G0_2_2;
    A[937] = 0.128522535206562*G0_0_0 + 0.0439716694056966*G0_0_1 + 0.0862471023061295*G0_0_2 + 0.128522535206562*G0_1_0 + 0.0439716694056966*G0_1_1 + 0.0862471023061297*G0_1_2 + 0.0862471023061284*G0_2_0 + 0.0446566879943606*G0_2_1 + 0.0439716694056961*G0_2_2;
    A[938] = -0.0613254736518621*G0_0_0 - 0.0187890812890808*G0_0_1 - 0.0400572774704718*G0_0_2 - 0.0613254736518619*G0_1_0 - 0.0187890812890808*G0_1_1 - 0.0400572774704717*G0_1_2 - 0.0400572774704708*G0_2_0 - 0.00939454064454056*G0_2_1 - 0.0187890812890806*G0_2_2;
    A[939] = 0.176636936077039*G0_0_0 + 0.1273155976932*G0_0_1 + 0.134361503176606*G0_0_2 + 0.176636936077039*G0_1_0 + 0.127315597693201*G0_1_1 + 0.134361503176606*G0_1_2 + 0.134361503176607*G0_2_0 + 0.128000616281866*G0_2_1 + 0.0920860702761738*G0_2_2;
    A[940] = -0.0435476102793801*G0_0_0 - 0.0292600797158079*G0_0_1 - 0.0377086423093359*G0_0_2 - 0.0435476102793804*G0_1_0 - 0.0292600797158079*G0_1_1 - 0.037708642309336*G0_1_2 - 0.0377086423093366*G0_2_0 - 0.0259654665036604*G0_2_1 - 0.0318696743392917*G0_2_2;
    A[941] = -0.111886369481855*G0_0_0 - 0.0641307878721073*G0_0_1 - 0.0906181733004643*G0_0_2 - 0.111886369481855*G0_1_0 - 0.0641307878721072*G0_1_1 - 0.0906181733004643*G0_1_2 - 0.0906181733004641*G0_2_0 - 0.0547362472275667*G0_2_1 - 0.0693499771190741*G0_2_2;
    A[942] = -0.128522535206562*G0_0_0 - 0.128522535206562*G0_0_1 - 0.0322284936000211*G0_0_2 - 0.128522535206562*G0_1_0 - 0.128522535206562*G0_1_1 - 0.0322284936000211*G0_1_2 - 0.0322284936000209*G0_2_0 - 0.0322284936000205*G0_2_1 - 0.0606730749959915*G0_2_2;
    A[943] = 0.0642612676032814*G0_0_0 + 0.0642612676032815*G0_0_1 + 0.0219858347028484*G0_0_2 + 0.0642612676032814*G0_1_0 + 0.0642612676032816*G0_1_1 + 0.0219858347028485*G0_1_2 + 0.0219858347028482*G0_2_0 + 0.0219858347028481*G0_2_1 + 0.0226708532915126*G0_2_2;
    A[944] = -0.0306627368259313*G0_0_0 - 0.0306627368259312*G0_0_1 - 0.00939454064454026*G0_0_2 - 0.0306627368259312*G0_1_0 - 0.0306627368259311*G0_1_1 - 0.00939454064454029*G0_1_2 - 0.00939454064454012*G0_2_0 - 0.00939454064454003*G0_2_1;
    A[945] = 0.064261267603281*G0_0_0 + 0.0642612676032806*G0_0_1 + 0.021985834702848*G0_0_2 + 0.0642612676032811*G0_1_0 + 0.0642612676032805*G0_1_1 + 0.021985834702848*G0_1_2 + 0.0219858347028485*G0_2_0 + 0.0219858347028483*G0_2_1 + 0.0226708532915129*G0_2_2;
    A[946] = -0.00913358118219242*G0_0_0 - 0.0091335811821925*G0_0_1 - 0.0032946132121476*G0_0_2 - 0.00913358118219248*G0_1_0 - 0.00913358118219258*G0_1_1 - 0.0032946132121476*G0_1_2 - 0.00329461321214821*G0_2_0 - 0.00329461321214807*G0_2_1;
    A[947] = -0.0306627368259306*G0_0_0 - 0.0306627368259304*G0_0_1 - 0.00939454064453991*G0_0_2 - 0.0306627368259306*G0_1_0 - 0.0306627368259304*G0_1_1 - 0.00939454064453994*G0_1_2 - 0.00939454064454018*G0_2_0 - 0.00939454064454035*G0_2_1;
    A[948] = 0.0962288017409537*G0_0_0 + 0.0962288017409536*G0_0_1 + 0.131458329157981*G0_0_2 + 0.0962288017409539*G0_1_0 + 0.096228801740954*G0_1_1 + 0.131458329157982*G0_1_2 + 0.131458329157981*G0_2_0 + 0.131458329157981*G0_2_1 + 0.166687856575009*G0_2_2;
    A[949] = -0.0505608958299918*G0_0_0 - 0.050560895829992*G0_0_1 - 0.0479513012065085*G0_0_2 - 0.0505608958299922*G0_1_0 - 0.0505608958299921*G0_1_1 - 0.0479513012065086*G0_1_2 - 0.0479513012065095*G0_2_0 - 0.0479513012065086*G0_2_1 - 0.0453417065830255*G0_2_2;
    A[950] = -0.0505608958299923*G0_0_0 - 0.0505608958299926*G0_0_1 - 0.0479513012065092*G0_0_2 - 0.0505608958299923*G0_1_0 - 0.0505608958299928*G0_1_1 - 0.0479513012065091*G0_1_2 - 0.0479513012065092*G0_2_0 - 0.0479513012065095*G0_2_1 - 0.0453417065830258*G0_2_2;
    A[951] = -0.15657567740901*G0_0_0 - 0.156575677409009*G0_0_1 - 0.153966082785527*G0_0_2 - 0.15657567740901*G0_1_0 - 0.15657567740901*G0_1_1 - 0.153966082785527*G0_1_2 - 0.153966082785526*G0_2_0 - 0.153966082785526*G0_2_1 - 0.151356488162043*G0_2_2;
    A[952] = 0.0214691349673988*G0_0_0 + 0.0214691349673987*G0_0_1 + 0.0214691349673988*G0_0_2 + 0.0214691349673988*G0_1_0 + 0.0214691349673987*G0_1_1 + 0.0214691349673988*G0_1_2 + 0.0551276864210887*G0_2_0 + 0.0551276864210887*G0_2_1 + 0.0551276864210887*G0_2_2;
    A[953] = -0.00336507226698203*G0_0_0 - 0.00336507226698207*G0_1_0;
    A[954] = -0.00336507226698199*G0_0_1 - 0.00336507226698202*G0_1_1;
    A[955] = -0.0232632312710439*G0_0_2 - 0.023263231271044*G0_1_2 - 0.0388177200243173*G0_2_2;
    A[956] = 0.0139743792087547*G0_0_1 + 0.00228991928210708*G0_0_2 + 0.0139743792087547*G0_1_1 + 0.00228991928210699*G0_1_2 + 0.002289919282107*G0_2_1;
    A[957] = -0.0263047138047148*G0_0_1 - 0.0131523569023572*G0_0_2 - 0.0263047138047148*G0_1_1 - 0.0131523569023572*G0_1_2 - 0.0131523569023576*G0_2_1 + 6.5239865586427e-05*G0_2_2;
    A[958] = -0.00523223722008294*G0_0_1 - 0.00578025209101549*G0_0_2 - 0.0052322372200831*G0_1_1 - 0.00578025209101575*G0_1_2 - 0.00578025209101493*G0_2_1 - 0.00626302709635972*G0_2_2;
    A[959] = -0.0788945694544659*G0_0_1 - 0.0599162925551814*G0_0_2 - 0.0788945694544658*G0_1_1 - 0.0599162925551813*G0_1_2 - 0.0599162925551822*G0_2_1 - 0.0315434750113574*G0_2_2;
    A[960] = 0.013974379208754*G0_0_0 + 0.00228991928210639*G0_0_2 + 0.0139743792087541*G0_1_0 + 0.00228991928210645*G0_1_2 + 0.00228991928210729*G0_2_0;
    A[961] = -0.0263047138047135*G0_0_0 - 0.0131523569023564*G0_0_2 - 0.0263047138047136*G0_1_0 - 0.0131523569023564*G0_1_2 - 0.0131523569023575*G0_2_0 + 6.52398655873854e-05*G0_2_2;
    A[962] = -0.00523223722008402*G0_0_0 - 0.0057802520910159*G0_0_2 - 0.00523223722008391*G0_1_0 - 0.00578025209101589*G0_1_2 - 0.00578025209101504*G0_2_0 - 0.00626302709636029*G0_2_2;
    A[963] = -0.0788945694544659*G0_0_0 - 0.0599162925551815*G0_0_2 - 0.0788945694544657*G0_1_0 - 0.0599162925551814*G0_1_2 - 0.0599162925551821*G0_2_0 - 0.0315434750113572*G0_2_2;
    A[964] = 0.00228991928210652*G0_0_0 + 0.00228991928210636*G0_0_1 + 0.00228991928210665*G0_1_0 + 0.00228991928210642*G0_1_1;
    A[965] = 0.00178757231708641*G0_0_0 + 0.00178757231708656*G0_0_1 + 0.00178757231708626*G0_1_0 + 0.00178757231708645*G0_1_1;
    A[966] = 0.001787572317086*G0_0_0 + 0.00178757231708586*G0_0_1 + 0.00178757231708613*G0_1_0 + 0.001787572317086*G0_1_1;
    A[967] = 0.00228991928210714*G0_0_0 + 0.0022899192821073*G0_0_1 + 0.00228991928210705*G0_1_0 + 0.00228991928210722*G0_1_1;
    A[968] = -0.147063705006412*G0_0_0 - 0.147063705006413*G0_0_1 - 0.108337320793918*G0_0_2 - 0.147063705006412*G0_1_0 - 0.147063705006412*G0_1_1 - 0.108337320793918*G0_1_2 - 0.173446706649831*G0_2_0 - 0.173446706649831*G0_2_1 - 0.243357746612953*G0_2_2;
    A[969] = 0.304226541205706*G0_0_0 + 0.304226541205706*G0_0_1 + 0.226095278178611*G0_0_2 + 0.304226541205706*G0_1_0 + 0.304226541205706*G0_1_1 + 0.226095278178611*G0_1_2 + 0.226095278178611*G0_2_0 + 0.226095278178611*G0_2_1 + 0.428965164208218*G0_2_2;
    A[970] = -0.204187731314467*G0_0_0 - 0.204187731314467*G0_0_1 - 0.219493003781198*G0_0_2 - 0.204187731314467*G0_1_0 - 0.204187731314466*G0_1_1 - 0.219493003781198*G0_1_2 - 0.16769255050505*G0_2_0 - 0.16769255050505*G0_2_1 - 0.387185554286247*G0_2_2;
    A[971] = 0.0788945694544659*G0_0_0 + 0.0788945694544658*G0_0_1 + 0.10352914270015*G0_0_2 + 0.0788945694544657*G0_1_0 + 0.0788945694544657*G0_1_1 + 0.10352914270015*G0_1_2 + 0.0599162925551821*G0_2_0 + 0.0599162925551821*G0_2_1 + 0.185268170294211*G0_2_2;
    A[972] = -0.0574176057031954*G0_0_0 - 0.0186912214907006*G0_0_1 - 0.0574176057031955*G0_0_2 - 0.0574176057031954*G0_1_0 - 0.0186912214907006*G0_1_1 - 0.0574176057031955*G0_1_2 + 0.0124934342599273*G0_2_0 - 0.0574176057031954*G0_2_1 + 0.0124934342599272*G0_2_2;
    A[973] = 0.00506261356955768*G0_0_0 + 0.000260959462348176*G0_0_1 + 0.00506261356955765*G0_0_2 + 0.00506261356955771*G0_1_0 + 0.000260959462348207*G0_1_1 + 0.00506261356955768*G0_1_2 + 0.000260959462348008*G0_2_0 + 0.00506261356955765*G0_2_1 + 0.000260959462348042*G0_2_2;
    A[974] = -0.000482775005343907*G0_0_0 - 0.00626302709636026*G0_0_1 - 0.000482775005343882*G0_0_2 - 0.000482775005343835*G0_1_0 - 0.00626302709636031*G0_1_1 - 0.00048277500534381*G0_1_2 + 6.52398655877462e-05*G0_2_0 - 0.000482775005343997*G0_2_1 + 6.52398655876894e-05*G0_2_2;
    A[975] = -0.00228991928210713*G0_0_0 + 0.00658922642429579*G0_0_1 - 0.00228991928210714*G0_0_2 - 0.00228991928210704*G0_1_0 + 0.00658922642429585*G0_1_1 - 0.00228991928210705*G0_1_2 - 0.00228991928210713*G0_2_1;
    A[976] = -0.0186912214907006*G0_0_0 - 0.0574176057031956*G0_0_1 - 0.0574176057031956*G0_0_2 - 0.0186912214907006*G0_1_0 - 0.0574176057031955*G0_1_1 - 0.0574176057031955*G0_1_2 - 0.0574176057031952*G0_2_0 + 0.0124934342599275*G0_2_1 + 0.0124934342599275*G0_2_2;
    A[977] = 0.000260959462348427*G0_0_0 + 0.00506261356955825*G0_0_1 + 0.0050626135695582*G0_0_2 + 0.000260959462348396*G0_1_0 + 0.00506261356955823*G0_1_1 + 0.00506261356955818*G0_1_2 + 0.00506261356955751*G0_2_0 + 0.000260959462348209*G0_2_1 + 0.000260959462348219*G0_2_2;
    A[978] = -0.00626302709636053*G0_0_0 - 0.000482775005344564*G0_0_1 - 0.000482775005344508*G0_0_2 - 0.00626302709636053*G0_1_0 - 0.000482775005344437*G0_1_1 - 0.000482775005344379*G0_1_2 - 0.000482775005344219*G0_2_0 + 6.52398655871231e-05*G0_2_1 + 6.52398655871654e-05*G0_2_2;
    A[979] = 0.00658922642429596*G0_0_0 - 0.00228991928210654*G0_0_1 - 0.00228991928210653*G0_0_2 + 0.006589226424296*G0_1_0 - 0.00228991928210659*G0_1_1 - 0.00228991928210659*G0_1_2 - 0.00228991928210667*G0_2_0;
    A[980] = -0.0148746893538554*G0_0_0 - 0.0148746893538558*G0_0_1 + 6.52398655872089e-05*G0_0_2 - 0.0148746893538558*G0_1_0 - 0.0148746893538559*G0_1_1 + 6.52398655870688e-05*G0_1_2 + 6.52398655864832e-05*G0_2_0 + 6.52398655873015e-05*G0_2_1;
    A[981] = 0.00450155072550815*G0_0_0 + 0.00450155072550854*G0_0_1 + 0.00257697469068971*G0_0_2 + 0.00450155072550838*G0_1_0 + 0.00450155072550843*G0_1_1 + 0.00257697469068982*G0_1_2 + 0.00257697469068991*G0_2_0 + 0.00257697469068901*G0_2_1;
    A[982] = -0.0148746893538562*G0_0_0 - 0.0148746893538569*G0_0_1 + 6.52398655872219e-05*G0_0_2 - 0.0148746893538563*G0_1_0 - 0.0148746893538569*G0_1_1 + 6.52398655871178e-05*G0_1_2 + 6.52398655869301e-05*G0_2_0 + 6.52398655871525e-05*G0_2_1;
    A[983] = -0.0189848008858427*G0_0_0 - 0.0189848008858416*G0_0_1 - 0.00949240044292108*G0_0_2 - 0.0189848008858425*G0_1_0 - 0.0189848008858416*G0_1_1 - 0.00949240044292113*G0_1_2 - 0.00949240044292053*G0_2_0 - 0.00949240044292059*G0_2_1 - 0.000652398655871575*G0_2_2;
    A[984] = -0.0189848008858423*G0_0_0 - 0.0189848008858419*G0_0_1 - 0.00949240044292219*G0_0_2 - 0.0189848008858426*G0_1_0 - 0.0189848008858421*G0_1_1 - 0.00949240044292226*G0_1_2 - 0.00949240044292069*G0_2_0 - 0.00949240044292048*G0_2_1 - 0.000652398655870941*G0_2_2;
    A[985] = -0.127087258163646*G0_0_0 - 0.127087258163647*G0_0_1 - 0.0893133759887223*G0_0_2 - 0.127087258163646*G0_1_0 - 0.127087258163647*G0_1_1 - 0.0893133759887222*G0_1_2 - 0.0893133759887233*G0_2_0 - 0.0893133759887232*G0_2_1 - 0.0665446628988283*G0_2_2;
    A[986] = 0.132306447410613*G0_0_0 + 0.121346149991983*G0_0_1 + 0.194675758911869*G0_0_2 + 0.132306447410614*G0_1_0 + 0.121346149991983*G0_1_1 + 0.194675758911869*G0_1_2 + 0.0589768384907276*G0_2_0 + 0.194675758911869*G0_2_1 - 0.00339247301052804*G0_2_2;
    A[987] = -0.0436454700777618*G0_0_0 + 0.0371867233846401*G0_0_1 - 0.0371541034518464*G0_0_2 - 0.0436454700777619*G0_1_0 + 0.0371867233846402*G0_1_1 - 0.0371541034518465*G0_1_2 - 0.00975335990527012*G0_2_0 - 0.0371541034518463*G0_2_1 - 0.00678494602105759*G0_2_2;
    A[988] = 0.0148746893538562*G0_0_0 - 0.0254435475789646*G0_0_1 + 0.0136351319077013*G0_0_2 + 0.0148746893538564*G0_1_0 - 0.0254435475789648*G0_1_1 + 0.0136351319077012*G0_1_2 - 6.52398655869063e-05*G0_2_0 + 0.0136351319077013*G0_2_1 - 0.000130479731173979*G0_2_2;
    A[989] = -0.166948816037357*G0_0_0 - 0.334680510461758*G0_0_1 - 0.250814663249558*G0_0_2 - 0.166948816037357*G0_1_0 - 0.334680510461759*G0_1_1 - 0.250814663249557*G0_1_2 - 0.160783648739378*G0_2_0 - 0.250814663249558*G0_2_1 - 0.0622388317700823*G0_2_2;
    A[990] = 0.0189848008858423*G0_0_0 + 0.0567586830607668*G0_0_1 + 0.037871741973304*G0_0_2 + 0.0189848008858426*G0_1_0 + 0.056758683060767*G0_1_1 + 0.0378717419733044*G0_1_2 + 0.00949240044292073*G0_2_0 + 0.0378717419733049*G0_2_1 + 0.0127870136550692*G0_2_2;
    A[991] = 0.127087258163647*G0_0_0 + 0.20941996853455*G0_0_1 + 0.173472802596066*G0_0_2 + 0.127087258163647*G0_1_0 + 0.20941996853455*G0_1_1 + 0.173472802596065*G0_1_2 + 0.0893133759887234*G0_2_0 + 0.173472802596065*G0_2_1 + 0.0846813455320404*G0_2_2;
    A[992] = 0.121346149991983*G0_0_0 + 0.132306447410613*G0_0_1 + 0.194675758911869*G0_0_2 + 0.121346149991983*G0_1_0 + 0.132306447410613*G0_1_1 + 0.194675758911869*G0_1_2 + 0.194675758911869*G0_2_0 + 0.0589768384907273*G0_2_1 - 0.00339247301052873*G0_2_2;
    A[993] = 0.0371867233846397*G0_0_0 - 0.0436454700777613*G0_0_1 - 0.0371541034518462*G0_0_2 + 0.0371867233846398*G0_1_0 - 0.0436454700777612*G0_1_1 - 0.0371541034518463*G0_1_2 - 0.0371541034518455*G0_2_0 - 0.00975335990526898*G0_2_1 - 0.00678494602105647*G0_2_2;
    A[994] = -0.0254435475789637*G0_0_0 + 0.0148746893538563*G0_0_1 + 0.0136351319077013*G0_0_2 - 0.025443547578964*G0_1_0 + 0.0148746893538563*G0_1_1 + 0.0136351319077012*G0_1_2 + 0.0136351319077004*G0_2_0 - 6.52398655870324e-05*G0_2_1 - 0.00013047973117454*G0_2_2;
    A[995] = -0.334680510461758*G0_0_0 - 0.166948816037357*G0_0_1 - 0.250814663249558*G0_0_2 - 0.334680510461758*G0_1_0 - 0.166948816037357*G0_1_1 - 0.250814663249558*G0_1_2 - 0.250814663249558*G0_2_0 - 0.160783648739378*G0_2_1 - 0.0622388317700821*G0_2_2;
    A[996] = 0.0567586830607648*G0_0_0 + 0.0189848008858414*G0_0_1 + 0.0378717419733035*G0_0_2 + 0.0567586830607651*G0_1_0 + 0.0189848008858413*G0_1_1 + 0.0378717419733035*G0_1_2 + 0.0378717419733046*G0_2_0 + 0.00949240044292059*G0_2_1 + 0.0127870136550684*G0_2_2;
    A[997] = 0.209419968534551*G0_0_0 + 0.127087258163648*G0_0_1 + 0.173472802596066*G0_0_2 + 0.209419968534551*G0_1_0 + 0.127087258163647*G0_1_1 + 0.173472802596066*G0_1_2 + 0.173472802596065*G0_2_0 + 0.0893133759887233*G0_2_1 + 0.0846813455320404*G0_2_2;
    A[998] = 0.0476251018785736*G0_0_0 + 0.0476251018785737*G0_0_1 - 0.0147442096226816*G0_0_2 + 0.0476251018785736*G0_1_0 + 0.0476251018785736*G0_1_1 - 0.0147442096226815*G0_1_2 - 0.0147442096226816*G0_2_0 - 0.0147442096226823*G0_2_1 + 0.0476251018785742*G0_2_2;
    A[999] = 0.00880738185425672*G0_0_0 + 0.00880738185425634*G0_0_1 + 0.0023160152283416*G0_0_2 + 0.00880738185425683*G0_1_0 + 0.00880738185425631*G0_1_1 + 0.00231601522834169*G0_1_2 + 0.00231601522834151*G0_2_0 + 0.00231601522834185*G0_2_1 - 0.000652398655870623*G0_2_2;
    A[1000] = -0.0013047973117415*G0_0_0 - 0.00130479731174184*G0_0_1 - 6.52398655868565e-05*G0_0_2 - 0.00130479731174174*G0_1_0 - 0.00130479731174186*G0_1_1 - 6.52398655867172e-05*G0_1_2 - 6.52398655873204e-05*G0_2_0 - 6.52398655874506e-05*G0_2_1;
    A[1001] = 0.00880738185425689*G0_0_0 + 0.00880738185425743*G0_0_1 + 0.00231601522834183*G0_0_2 + 0.00880738185425682*G0_1_0 + 0.00880738185425751*G0_1_1 + 0.00231601522834179*G0_1_2 + 0.00231601522834159*G0_2_0 + 0.00231601522834202*G0_2_1 - 0.000652398655870845*G0_2_2;
    A[1002] = -0.0156575677409011*G0_0_0 - 0.0156575677409005*G0_0_1 - 0.00257697469068986*G0_0_2 - 0.0156575677409009*G0_1_0 - 0.0156575677409005*G0_1_1 - 0.00257697469068998*G0_1_2 - 0.00257697469068979*G0_2_0 - 0.00257697469068988*G0_2_1;
    A[1003] = -0.00130479731174203*G0_0_0 - 0.00130479731174271*G0_0_1 - 6.52398655871863e-05*G0_0_2 - 0.00130479731174222*G0_1_0 - 0.00130479731174275*G0_1_1 - 6.5239865587082e-05*G0_1_2 - 6.52398655874711e-05*G0_2_0 - 6.52398655876297e-05*G0_2_1;
    A[1004] = -0.15657567740901*G0_0_0 - 0.15657567740901*G0_0_1 - 0.066218463570894*G0_0_2 - 0.15657567740901*G0_1_0 - 0.156575677409011*G0_1_1 - 0.0662184635708944*G0_1_2 - 0.0662184635708945*G0_2_0 - 0.0662184635708949*G0_2_1 - 0.161794866655978*G0_2_2;
    A[1005] = 0.0391439193522519*G0_0_0 + 0.039143919352252*G0_0_1 + 0.00717638521457903*G0_0_2 + 0.0391439193522525*G0_1_0 + 0.0391439193522521*G0_1_1 + 0.007176385214579*G0_1_2 + 0.00717638521458039*G0_2_0 + 0.00717638521457961*G0_2_1 + 0.00130479731174192*G0_2_2;
    A[1006] = 0.0391439193522535*G0_0_0 + 0.0391439193522542*G0_0_1 + 0.00717638521458032*G0_0_2 + 0.0391439193522534*G0_1_0 + 0.0391439193522545*G0_1_1 + 0.00717638521458043*G0_1_2 + 0.00717638521458018*G0_2_0 + 0.00717638521458061*G0_2_1 + 0.0013047973117418*G0_2_2;
    A[1007] = 0.1859336169232*G0_0_0 + 0.185933616923199*G0_0_1 + 0.170276049182299*G0_0_2 + 0.1859336169232*G0_1_0 + 0.185933616923199*G0_1_1 + 0.170276049182299*G0_1_2 + 0.170276049182299*G0_2_0 + 0.170276049182298*G0_2_1 + 0.180714427676233*G0_2_2;
    A[1008] = -0.0155544887532733*G0_0_0 - 0.0155544887532733*G0_0_1 - 0.0155544887532734*G0_0_2 - 0.0155544887532733*G0_1_0 - 0.0155544887532733*G0_1_1 - 0.0155544887532734*G0_1_2 - 0.0388177200243169*G0_2_0 - 0.0388177200243168*G0_2_1 - 0.0388177200243168*G0_2_2;
    A[1009] = -0.0033650722669819*G0_0_0 - 0.00336507226698188*G0_1_0;
    A[1010] = -0.003365072266982*G0_0_1 - 0.00336507226698196*G0_1_1;
    A[1011] = 0.0336585514536905*G0_0_2 + 0.0336585514536905*G0_1_2 + 0.0551276864210891*G0_2_2;
    A[1012] = 0.0111690649885089*G0_0_1 + 0.00228991928210655*G0_0_2 + 0.0111690649885089*G0_1_1 + 0.00228991928210658*G0_1_2 + 0.00228991928210651*G0_2_1;
    A[1013] = -0.00523223722008332*G0_0_1 + 0.000548014870931637*G0_0_2 - 0.00523223722008315*G0_1_1 + 0.000548014870931731*G0_1_2 + 0.000548014870932251*G0_2_1 + 6.52398655876989e-05*G0_2_2;
    A[1014] = -0.00960330821442065*G0_0_1 - 0.00480165410720958*G0_0_2 - 0.00960330821442069*G0_1_1 - 0.00480165410720958*G0_1_2 - 0.00480165410721068*G0_2_1 + 0.000260959462347572*G0_2_2;
    A[1015] = 0.108637424175619*G0_0_1 + 0.0699110399631227*G0_0_2 + 0.108637424175619*G0_1_1 + 0.0699110399631227*G0_1_2 + 0.0699110399631239*G0_2_1 + 0.012493434259928*G0_2_2;
    A[1016] = 0.0111690649885093*G0_0_0 + 0.00228991928210707*G0_0_2 + 0.0111690649885093*G0_1_0 + 0.00228991928210705*G0_1_2 + 0.00228991928210628*G0_2_0;
    A[1017] = -0.00523223722008429*G0_0_0 + 0.000548014870931164*G0_0_2 - 0.00523223722008422*G0_1_0 + 0.000548014870931202*G0_1_2 + 0.000548014870932286*G0_2_0 + 6.52398655866298e-05*G0_2_2;
    A[1018] = -0.00960330821441941*G0_0_0 - 0.00480165410720926*G0_0_2 - 0.0096033082144194*G0_1_0 - 0.00480165410720928*G0_1_2 - 0.00480165410721048*G0_2_0 + 0.000260959462348507*G0_2_2;
    A[1019] = 0.108637424175619*G0_0_0 + 0.069911039963123*G0_0_2 + 0.108637424175619*G0_1_0 + 0.069911039963123*G0_1_2 + 0.0699110399631239*G0_2_0 + 0.0124934342599276*G0_2_2;
    A[1020] = 0.00228991928210691*G0_0_0 + 0.0022899192821071*G0_0_1 + 0.00228991928210686*G0_1_0 + 0.00228991928210708*G0_1_1;
    A[1021] = 0.00178757231708606*G0_0_0 + 0.00178757231708572*G0_0_1 + 0.00178757231708617*G0_1_0 + 0.0017875723170858*G0_1_1;
    A[1022] = 0.00178757231708638*G0_0_0 + 0.00178757231708654*G0_0_1 + 0.00178757231708631*G0_1_0 + 0.00178757231708648*G0_1_1;
    A[1023] = 0.00228991928210646*G0_0_0 + 0.00228991928210646*G0_0_1 + 0.0022899192821065*G0_1_0 + 0.00228991928210647*G0_1_1;
    A[1024] = 0.100717304493346*G0_0_0 + 0.100717304493346*G0_0_1 + 0.0817390275940619*G0_0_2 + 0.100717304493345*G0_1_0 + 0.100717304493345*G0_1_1 + 0.0817390275940619*G0_1_2 + 0.12535187773903*G0_2_0 + 0.12535187773903*G0_2_1 + 0.185268170294211*G0_2_2;
    A[1025] = -0.204187731314467*G0_0_0 - 0.204187731314467*G0_0_1 - 0.16769255050505*G0_0_2 - 0.204187731314467*G0_1_0 - 0.204187731314466*G0_1_1 - 0.16769255050505*G0_1_2 - 0.219493003781198*G0_2_0 - 0.219493003781198*G0_2_1 - 0.387185554286247*G0_2_2;
    A[1026] = 0.281001149056703*G0_0_0 + 0.281001149056703*G0_0_1 + 0.202869886029608*G0_0_2 + 0.281001149056703*G0_1_0 + 0.281001149056703*G0_1_1 + 0.202869886029607*G0_1_2 + 0.202869886029608*G0_2_0 + 0.202869886029608*G0_2_1 + 0.428965164208218*G0_2_2;
    A[1027] = -0.108637424175619*G0_0_0 - 0.108637424175619*G0_0_1 - 0.135020425819037*G0_0_2 - 0.108637424175619*G0_1_0 - 0.108637424175619*G0_1_1 - 0.135020425819037*G0_1_2 - 0.0699110399631239*G0_2_0 - 0.0699110399631239*G0_2_1 - 0.243357746612954*G0_2_2;
    A[1028] = 0.0283728175438242*G0_0_0 + 0.00939454064454053*G0_0_1 + 0.0283728175438242*G0_0_2 + 0.0283728175438242*G0_1_0 + 0.00939454064454054*G0_1_1 + 0.0283728175438242*G0_1_2 - 0.0315434750113568*G0_2_0 + 0.0283728175438241*G0_2_1 - 0.0315434750113567*G0_2_2;
    A[1029] = -0.000482775005344318*G0_0_0 + 6.52398655870969e-05*G0_0_1 - 0.000482775005344274*G0_0_2 - 0.00048277500534437*G0_1_0 + 6.52398655870594e-05*G0_1_1 - 0.000482775005344325*G0_1_2 - 0.00626302709635996*G0_2_0 - 0.000482775005344125*G0_2_1 - 0.00626302709636001*G0_2_2;
    A[1030] = 0.0132175967679434*G0_0_0 + 6.52398655870793e-05*G0_0_1 + 0.0132175967679434*G0_0_2 + 0.0132175967679434*G0_1_0 + 6.5239865587117e-05*G0_1_1 + 0.0132175967679434*G0_1_2 + 6.52398655864602e-05*G0_2_0 + 0.0132175967679435*G0_2_1 + 6.52398655865357e-05*G0_2_2;
    A[1031] = -0.00228991928210643*G0_0_0 + 0.00939454064454068*G0_0_1 - 0.00228991928210644*G0_0_2 - 0.00228991928210647*G0_1_0 + 0.00939454064454064*G0_1_1 - 0.00228991928210648*G0_1_2 - 0.00228991928210642*G0_2_1;
    A[1032] = 0.00939454064454049*G0_0_0 + 0.0283728175438242*G0_0_1 + 0.0283728175438242*G0_0_2 + 0.00939454064454049*G0_1_0 + 0.0283728175438242*G0_1_1 + 0.0283728175438242*G0_1_2 + 0.028372817543824*G0_2_0 - 0.0315434750113569*G0_2_1 - 0.0315434750113569*G0_2_2;
    A[1033] = 6.52398655872099e-05*G0_0_0 - 0.000482775005344599*G0_0_1 - 0.000482775005344535*G0_0_2 + 6.52398655872006e-05*G0_1_0 - 0.000482775005344587*G0_1_1 - 0.000482775005344523*G0_1_2 - 0.000482775005344036*G0_2_0 - 0.00626302709636027*G0_2_1 - 0.00626302709636026*G0_2_2;
    A[1034] = 6.52398655869515e-05*G0_0_0 + 0.0132175967679438*G0_0_1 + 0.0132175967679437*G0_0_2 + 6.52398655869777e-05*G0_1_0 + 0.0132175967679438*G0_1_1 + 0.0132175967679437*G0_1_2 + 0.0132175967679437*G0_2_0 + 6.52398655871112e-05*G0_2_1 + 6.52398655870794e-05*G0_2_2;
    A[1035] = 0.00939454064454054*G0_0_0 - 0.00228991928210695*G0_0_1 - 0.00228991928210694*G0_0_2 + 0.00939454064454051*G0_1_0 - 0.00228991928210693*G0_1_1 - 0.00228991928210692*G0_1_2 - 0.00228991928210693*G0_2_0;
    A[1036] = -0.00117431758056781*G0_0_0 - 0.00117431758056765*G0_0_1 + 6.52398655868674e-05*G0_0_2 - 0.00117431758056752*G0_1_0 - 0.00117431758056751*G0_1_1 + 6.52398655870448e-05*G0_1_2 + 6.52398655875194e-05*G0_2_0 + 6.52398655869191e-05*G0_2_1;
    A[1037] = -0.0105036183595209*G0_0_0 - 0.0105036183595203*G0_0_1 + 0.00257697469069018*G0_0_2 - 0.0105036183595211*G0_1_0 - 0.0105036183595203*G0_1_1 + 0.00257697469069009*G0_1_2 + 0.00257697469069036*G0_2_0 + 0.00257697469069084*G0_2_1;
    A[1038] = -0.0011743175805677*G0_0_0 - 0.00117431758056794*G0_0_1 + 6.52398655867355e-05*G0_0_2 - 0.0011743175805674*G0_1_0 - 0.00117431758056764*G0_1_1 + 6.52398655868849e-05*G0_1_2 + 6.52398655872666e-05*G0_2_0 + 6.52398655872692e-05*G0_2_1;
    A[1039] = 0.00352295274170372*G0_0_0 + 0.00352295274170261*G0_0_1 - 0.00296841388421187*G0_0_2 + 0.00352295274170368*G0_1_0 + 0.00352295274170265*G0_1_1 - 0.0029684138842118*G0_1_2 - 0.00296841388421318*G0_2_0 - 0.00296841388421289*G0_2_1 - 0.000652398655870194*G0_2_2;
    A[1040] = 0.00352295274170253*G0_0_0 + 0.00352295274170252*G0_0_1 - 0.00296841388421246*G0_0_2 + 0.00352295274170259*G0_1_0 + 0.0035229527417024*G0_1_1 - 0.00296841388421235*G0_1_2 - 0.00296841388421322*G0_2_0 - 0.00296841388421347*G0_2_1 - 0.000652398655870845*G0_2_2;
    A[1041] = 0.124738623002511*G0_0_0 + 0.124738623002512*G0_0_1 + 0.0623693115012556*G0_0_2 + 0.124738623002511*G0_1_0 + 0.124738623002512*G0_1_1 + 0.0623693115012555*G0_1_2 + 0.0623693115012562*G0_2_0 + 0.0623693115012562*G0_2_1 + 0.0476251018785728*G0_2_2;
    A[1042] = -0.0510175748891024*G0_0_0 - 0.0528442911255408*G0_0_1 - 0.0887914570640262*G0_0_2 - 0.0510175748891026*G0_1_0 - 0.0528442911255407*G0_1_1 - 0.0887914570640262*G0_1_2 - 0.00463203045668346*G0_2_0 - 0.0887914570640258*G0_2_1 + 0.0846813455320387*G0_2_2;
    A[1043] = -0.0155923278753133*G0_0_0 - 0.00619778723077327*G0_0_1 - 0.0250847283182346*G0_0_2 - 0.0155923278753133*G0_1_0 - 0.00619778723077334*G0_1_1 - 0.0250847283182345*G0_1_2 + 0.00329461321214816*G0_2_0 - 0.0250847283182353*G0_2_1 + 0.0127870136550693*G0_2_2;
    A[1044] = 0.00117431758056761*G0_0_0 - 0.0528442911255409*G0_0_1 - 0.0137656116388754*G0_0_2 + 0.00117431758056729*G0_1_0 - 0.0528442911255408*G0_1_1 - 0.0137656116388754*G0_1_2 - 6.52398655872584e-05*G0_2_0 - 0.0137656116388754*G0_2_1 - 0.000130479731174338*G0_2_2;
    A[1045] = 0.182410664181496*G0_0_0 + 0.104709984267275*G0_0_1 + 0.188575831479476*G0_0_2 + 0.182410664181496*G0_1_0 + 0.104709984267275*G0_1_1 + 0.188575831479476*G0_1_2 + 0.0985448169692961*G0_2_0 + 0.188575831479477*G0_2_1 - 0.0622388317700804*G0_2_2;
    A[1046] = -0.00352295274170253*G0_0_0 + 0.104709984267275*G0_0_1 + 0.0303691574307888*G0_0_2 - 0.00352295274170259*G0_1_0 + 0.104709984267275*G0_1_1 + 0.0303691574307887*G0_1_2 + 0.00296841388421318*G0_2_0 + 0.0303691574307887*G0_2_1 - 0.00678494602105679*G0_2_2;
    A[1047] = -0.124738623002511*G0_0_0 - 0.271397840842282*G0_0_1 - 0.198068231922397*G0_0_2 - 0.124738623002511*G0_1_0 - 0.271397840842282*G0_1_1 - 0.198068231922397*G0_1_2 - 0.0623693115012562*G0_2_0 - 0.198068231922397*G0_2_1 - 0.00339247301052977*G0_2_2;
    A[1048] = -0.0528442911255412*G0_0_0 - 0.0510175748891026*G0_0_1 - 0.0887914570640261*G0_0_2 - 0.0528442911255411*G0_1_0 - 0.0510175748891028*G0_1_1 - 0.0887914570640261*G0_1_2 - 0.088791457064026*G0_2_0 - 0.00463203045668354*G0_2_1 + 0.0846813455320393*G0_2_2;
    A[1049] = -0.00619778723077312*G0_0_0 - 0.015592327875314*G0_0_1 - 0.0250847283182349*G0_0_2 - 0.00619778723077321*G0_1_0 - 0.015592327875314*G0_1_1 - 0.0250847283182349*G0_1_2 - 0.0250847283182356*G0_2_0 + 0.00329461321214736*G0_2_1 + 0.0127870136550685*G0_2_2;
    A[1050] = -0.0528442911255407*G0_0_0 + 0.00117431758056729*G0_0_1 - 0.0137656116388753*G0_0_2 - 0.0528442911255406*G0_1_0 + 0.00117431758056713*G0_1_1 - 0.0137656116388753*G0_1_2 - 0.0137656116388747*G0_2_0 - 6.52398655871623e-05*G0_2_1 - 0.000130479731173768*G0_2_2;
    A[1051] = 0.104709984267276*G0_0_0 + 0.182410664181496*G0_0_1 + 0.188575831479477*G0_0_2 + 0.104709984267276*G0_1_0 + 0.182410664181496*G0_1_1 + 0.188575831479476*G0_1_2 + 0.188575831479477*G0_2_0 + 0.0985448169692962*G0_2_1 - 0.0622388317700803*G0_2_2;
    A[1052] = 0.104709984267275*G0_0_0 - 0.00352295274170245*G0_0_1 + 0.0303691574307893*G0_0_2 + 0.104709984267275*G0_1_0 - 0.0035229527417025*G0_1_1 + 0.0303691574307894*G0_1_2 + 0.0303691574307886*G0_2_0 + 0.0029684138842129*G0_2_1 - 0.00678494602105659*G0_2_2;
    A[1053] = -0.271397840842284*G0_0_0 - 0.124738623002512*G0_0_1 - 0.198068231922398*G0_0_2 - 0.271397840842284*G0_1_0 - 0.124738623002512*G0_1_1 - 0.198068231922397*G0_1_2 - 0.198068231922397*G0_2_0 - 0.0623693115012563*G0_2_1 - 0.0033924730105298*G0_2_2;
    A[1054] = -0.0150051690850301*G0_0_0 - 0.0150051690850299*G0_0_1 + 0.0227687130898933*G0_0_2 - 0.01500516908503*G0_1_0 - 0.0150051690850299*G0_1_1 + 0.0227687130898931*G0_1_2 + 0.0227687130898935*G0_2_0 + 0.0227687130898942*G0_2_1 - 0.066544662898829*G0_2_2;
    A[1055] = -0.000652398655870928*G0_0_0 - 0.000652398655870829*G0_0_1 + 0.00884000178704984*G0_0_2 - 0.000652398655870979*G0_1_0 - 0.000652398655870805*G0_1_1 + 0.00884000178704985*G0_1_2 + 0.00884000178705014*G0_2_0 + 0.00884000178704979*G0_2_1 - 0.00065239865587147*G0_2_2;
    A[1056] = -0.0150051690850301*G0_0_0 - 0.0150051690850298*G0_0_1 - 6.52398655872633e-05*G0_0_2 - 0.01500516908503*G0_1_0 - 0.0150051690850299*G0_1_1 - 6.52398655874431e-05*G0_1_2 - 6.52398655868381e-05*G0_2_0 - 6.52398655866698e-05*G0_2_1;
    A[1057] = -0.000652398655870982*G0_0_0 - 0.000652398655871107*G0_0_1 + 0.00884000178704998*G0_0_2 - 0.000652398655870901*G0_1_0 - 0.000652398655871132*G0_1_1 + 0.00884000178705*G0_1_2 + 0.00884000178704997*G0_2_0 + 0.00884000178704964*G0_2_1 - 0.000652398655871214*G0_2_2;
    A[1058] = -0.000652398655870737*G0_0_0 - 0.000652398655870875*G0_0_1 - 0.00257697469068993*G0_0_2 - 0.000652398655870861*G0_1_0 - 0.000652398655870842*G0_1_1 - 0.00257697469068984*G0_1_2 - 0.00257697469068969*G0_2_0 - 0.00257697469068981*G0_2_1;
    A[1059] = -0.0150051690850299*G0_0_0 - 0.0150051690850299*G0_0_1 - 6.5239865586848e-05*G0_0_2 - 0.0150051690850298*G0_1_0 - 0.0150051690850298*G0_1_1 - 6.52398655869972e-05*G0_1_2 - 6.52398655868502e-05*G0_2_0 - 6.52398655865258e-05*G0_2_1;
    A[1060] = 0.0260959462348338*G0_0_0 + 0.0260959462348342*G0_0_1 + 0.0104383784939332*G0_0_2 + 0.026095946234834*G0_1_0 + 0.0260959462348344*G0_1_1 + 0.0104383784939333*G0_1_2 + 0.010438378493934*G0_2_0 + 0.0104383784939345*G0_2_1 + 0.180714427676232*G0_2_2;
    A[1061] = 0.0260959462348364*G0_0_0 + 0.0260959462348349*G0_0_1 - 0.00587158790283747*G0_0_2 + 0.0260959462348362*G0_1_0 + 0.026095946234835*G0_1_1 - 0.00587158790283755*G0_1_2 - 0.00587158790283798*G0_2_0 - 0.0058715879028379*G0_2_1 + 0.00130479731174188*G0_2_2;
    A[1062] = 0.0260959462348343*G0_0_0 + 0.0260959462348365*G0_0_1 - 0.00587158790283751*G0_0_2 + 0.0260959462348344*G0_1_0 + 0.0260959462348361*G0_1_1 - 0.00587158790283765*G0_1_2 - 0.0058715879028385*G0_2_0 - 0.00587158790283832*G0_2_1 + 0.00130479731174207*G0_2_2;
    A[1063] = -0.1859336169232*G0_0_0 - 0.185933616923199*G0_0_1 - 0.0955764030850827*G0_0_2 - 0.1859336169232*G0_1_0 - 0.185933616923199*G0_1_1 - 0.0955764030850827*G0_1_2 - 0.0955764030850831*G0_2_0 - 0.0955764030850828*G0_2_1 - 0.161794866655976*G0_2_2;
    A[1064] = 0.00480034930989794*G0_0_0 + 0.00480034930989797*G0_0_1 + 0.00480034930989798*G0_0_2 + 0.00480034930989795*G0_1_0 + 0.00480034930989797*G0_1_1 + 0.00480034930989799*G0_1_2 + 0.0153313684129655*G0_2_0 + 0.0153313684129655*G0_2_1 + 0.0153313684129655*G0_2_2;
    A[1065] = -0.00480034930989798*G0_0_0 - 0.00480034930989797*G0_1_0;
    A[1066] = -0.004800349309898*G0_0_1 - 0.00480034930989803*G0_1_1;
    A[1067] = -0.0372571824394742*G0_0_2 - 0.0372571824394742*G0_1_2 - 0.0551276864210891*G0_2_2;
    A[1068] = 0.0328352243499816*G0_0_1 + 0.00537576492437588*G0_0_2 + 0.0328352243499817*G0_1_1 + 0.0053757649243759*G0_1_2 + 0.00537576492437622*G0_2_1;
    A[1069] = -0.0788945694544658*G0_0_1 - 0.0189782768992834*G0_0_2 - 0.0788945694544659*G0_1_1 - 0.0189782768992835*G0_1_2 - 0.0189782768992843*G0_2_1 + 0.00939454064454039*G0_2_2;
    A[1070] = 0.108637424175619*G0_0_1 + 0.0387263842124946*G0_0_2 + 0.108637424175619*G0_1_1 + 0.0387263842124946*G0_1_2 + 0.0387263842124958*G0_2_1 - 0.0186912214907003*G0_2_2;
    A[1071] = -0.139822079926247*G0_0_1 - 0.0699110399631228*G0_0_2 - 0.139822079926247*G0_1_1 - 0.0699110399631229*G0_1_2 - 0.0699110399631236*G0_2_1 + 0.019637199541713*G0_2_2;
    A[1072] = 0.0328352243499814*G0_0_0 + 0.00537576492437585*G0_0_2 + 0.0328352243499814*G0_1_0 + 0.00537576492437586*G0_1_2 + 0.0053757649243762*G0_2_0;
    A[1073] = -0.0788945694544657*G0_0_0 - 0.0189782768992838*G0_0_2 - 0.0788945694544658*G0_1_0 - 0.0189782768992839*G0_1_2 - 0.0189782768992844*G0_2_0 + 0.00939454064454086*G0_2_2;
    A[1074] = 0.108637424175619*G0_0_0 + 0.038726384212495*G0_0_2 + 0.108637424175619*G0_1_0 + 0.038726384212495*G0_1_2 + 0.0387263842124955*G0_2_0 - 0.0186912214907009*G0_2_2;
    A[1075] = -0.139822079926247*G0_0_0 - 0.069911039963123*G0_0_2 - 0.139822079926247*G0_1_0 - 0.0699110399631231*G0_1_2 - 0.0699110399631235*G0_2_0 + 0.0196371995417131*G0_2_2;
    A[1076] = 0.00537576492437596*G0_0_0 + 0.00537576492437588*G0_0_1 + 0.00537576492437597*G0_1_0 + 0.00537576492437589*G0_1_1;
    A[1077] = 0.00228991928210685*G0_0_0 + 0.00228991928210689*G0_0_1 + 0.00228991928210685*G0_1_0 + 0.0022899192821069*G0_1_1;
    A[1078] = 0.00228991928210666*G0_0_0 + 0.00228991928210659*G0_0_1 + 0.00228991928210667*G0_1_0 + 0.00228991928210659*G0_1_1;
    A[1079] = 0.00537576492437597*G0_0_0 + 0.00537576492437622*G0_0_1 + 0.005375764924376*G0_1_0 + 0.00537576492437627*G0_1_1;
    A[1080] = -0.0328352243499816*G0_0_0 - 0.0328352243499816*G0_0_1 - 0.0274594594256056*G0_0_2 - 0.0328352243499815*G0_1_0 - 0.0328352243499815*G0_1_1 - 0.0274594594256056*G0_1_2 - 0.0537511252572017*G0_2_0 - 0.0537511252572017*G0_2_1 - 0.0812105846828067*G0_2_2;
    A[1081] = 0.0788945694544659*G0_0_0 + 0.0788945694544657*G0_0_1 + 0.0599162925551821*G0_0_2 + 0.0788945694544658*G0_1_0 + 0.0788945694544657*G0_1_1 + 0.0599162925551821*G0_1_2 + 0.10352914270015*G0_2_0 + 0.10352914270015*G0_2_1 + 0.185268170294211*G0_2_2;
    A[1082] = -0.108637424175619*G0_0_0 - 0.108637424175619*G0_0_1 - 0.0699110399631239*G0_0_2 - 0.108637424175619*G0_1_0 - 0.108637424175619*G0_1_1 - 0.0699110399631239*G0_1_2 - 0.135020425819037*G0_2_0 - 0.135020425819037*G0_2_1 - 0.243357746612954*G0_2_2;
    A[1083] = 0.139822079926246*G0_0_0 + 0.139822079926246*G0_0_1 + 0.0699110399631235*G0_0_2 + 0.139822079926246*G0_1_0 + 0.139822079926246*G0_1_1 + 0.0699110399631235*G0_1_2 + 0.0699110399631235*G0_2_0 + 0.0699110399631235*G0_2_1 + 0.179096479009673*G0_2_2;
    A[1084] = -0.00537576492437617*G0_0_0 - 0.00537576492437612*G0_0_2 - 0.00537576492437613*G0_1_0 - 0.00537576492437608*G0_1_2 + 0.022083694501229*G0_2_0 - 0.00537576492437597*G0_2_1 + 0.022083694501229*G0_2_2;
    A[1085] = -0.0022899192821067*G0_0_0 - 0.00228991928210677*G0_0_2 - 0.00228991928210675*G0_1_0 - 0.00228991928210682*G0_1_2 + 0.00658922642429582*G0_2_0 - 0.00228991928210695*G0_2_1 + 0.00658922642429582*G0_2_2;
    A[1086] = -0.00228991928210666*G0_0_0 - 0.00228991928210665*G0_0_2 - 0.00228991928210663*G0_1_0 - 0.00228991928210662*G0_1_2 + 0.00939454064454076*G0_2_0 - 0.0022899192821065*G0_2_1 + 0.00939454064454071*G0_2_2;
    A[1087] = -0.00537576492437601*G0_0_0 - 0.00537576492437598*G0_0_2 - 0.00537576492437604*G0_1_0 - 0.00537576492437601*G0_1_2 - 0.00537576492437618*G0_2_1;
    A[1088] = -0.00537576492437623*G0_0_1 - 0.00537576492437616*G0_0_2 - 0.00537576492437618*G0_1_1 - 0.00537576492437611*G0_1_2 - 0.00537576492437601*G0_2_0 + 0.0220836945012289*G0_2_1 + 0.0220836945012289*G0_2_2;
    A[1089] = -0.00228991928210666*G0_0_1 - 0.00228991928210675*G0_0_2 - 0.00228991928210667*G0_1_1 - 0.00228991928210676*G0_1_2 - 0.00228991928210682*G0_2_0 + 0.00658922642429592*G0_2_1 + 0.00658922642429589*G0_2_2;
    A[1090] = -0.00228991928210673*G0_0_1 - 0.00228991928210675*G0_0_2 - 0.00228991928210671*G0_1_1 - 0.00228991928210673*G0_1_2 - 0.00228991928210672*G0_2_0 + 0.00939454064454044*G0_2_1 + 0.00939454064454044*G0_2_2;
    A[1091] = -0.00537576492437594*G0_0_1 - 0.00537576492437591*G0_0_2 - 0.00537576492437595*G0_1_1 - 0.00537576492437592*G0_1_2 - 0.00537576492437587*G0_2_0;
    A[1092] = -0.0118736555368505*G0_0_0 - 0.0118736555368505*G0_0_1 + 0.00939454064454048*G0_0_2 - 0.0118736555368505*G0_1_0 - 0.0118736555368505*G0_1_1 + 0.00939454064454048*G0_1_2 + 0.00939454064454013*G0_2_0 + 0.00939454064454029*G0_2_1;
    A[1093] = -0.00254435475789634*G0_0_0 - 0.00254435475789621*G0_0_1 + 0.00329461321214803*G0_0_2 - 0.0025443547578963*G0_1_0 - 0.00254435475789618*G0_1_1 + 0.00329461321214807*G0_1_2 + 0.00329461321214764*G0_2_0 + 0.00329461321214759*G0_2_1;
    A[1094] = -0.0118736555368498*G0_0_0 - 0.0118736555368504*G0_0_1 + 0.0093945406445405*G0_0_2 - 0.0118736555368499*G0_1_0 - 0.0118736555368505*G0_1_1 + 0.0093945406445405*G0_1_2 + 0.00939454064454037*G0_2_0 + 0.00939454064454014*G0_2_1;
    A[1095] = 0.0429604514890975*G0_0_0 + 0.042960451489097*G0_0_1 + 0.000685018588663803*G0_0_2 + 0.0429604514890976*G0_1_0 + 0.0429604514890971*G0_1_1 + 0.000685018588663782*G0_1_2 + 0.000685018588664673*G0_2_0 + 0.00068501858866422*G0_2_1 + 0.0226708532915123*G0_2_2;
    A[1096] = 0.0429604514890967*G0_0_0 + 0.0429604514890974*G0_0_1 + 0.000685018588664167*G0_0_2 + 0.0429604514890966*G0_1_0 + 0.0429604514890976*G0_1_1 + 0.00068501858866406*G0_1_2 + 0.000685018588664732*G0_2_0 + 0.000685018588665172*G0_2_1 + 0.0226708532915129*G0_2_2;
    A[1097] = -0.124738623002511*G0_0_0 - 0.124738623002511*G0_0_1 - 0.0284445813959692*G0_0_2 - 0.124738623002511*G0_1_0 - 0.124738623002511*G0_1_1 - 0.028444581395969*G0_1_2 - 0.0284445813959705*G0_2_0 - 0.0284445813959706*G0_2_1 - 0.0606730749959909*G0_2_2;
    A[1098] = 0.0118736555368509*G0_0_0 + 0.0212681961813913*G0_0_2 + 0.0118736555368509*G0_1_0 + 0.0212681961813913*G0_1_2 - 0.0146137298915071*G0_2_0 + 0.0212681961813909*G0_2_1 - 0.0693499771190732*G0_2_2;
    A[1099] = 0.00254435475789565*G0_0_0 + 0.00583896797004389*G0_0_2 + 0.00254435475789555*G0_1_0 + 0.00583896797004391*G0_1_2 - 0.00590420783563172*G0_2_0 + 0.00583896797004409*G0_2_1 - 0.0318696743392925*G0_2_2;
    A[1100] = 0.01187365553685*G0_0_0 + 0.0212681961813905*G0_0_2 + 0.0118736555368501*G0_1_0 + 0.0212681961813905*G0_1_2 - 0.00939454064454035*G0_2_0 + 0.0212681961813906*G0_2_1 - 0.018789081289081*G0_2_2;
    A[1101] = -0.042960451489098*G0_0_0 - 0.0422754329004339*G0_0_2 - 0.042960451489098*G0_1_0 - 0.0422754329004339*G0_1_2 - 0.0359145460056923*G0_2_0 - 0.0422754329004335*G0_2_1 + 0.0920860702761729*G0_2_2;
    A[1102] = -0.0429604514890968*G0_0_0 - 0.042275432900432*G0_0_2 - 0.0429604514890967*G0_1_0 - 0.0422754329004321*G0_1_2 - 0.000685018588664752*G0_2_0 - 0.0422754329004321*G0_2_1 + 0.0439716694056969*G0_2_2;
    A[1103] = 0.124738623002511*G0_0_0 + 0.0962940416065416*G0_0_2 + 0.124738623002511*G0_1_0 + 0.0962940416065417*G0_1_2 + 0.0284445813959705*G0_2_0 + 0.096294041606541*G0_2_1 - 0.0644569872000415*G0_2_2;
    A[1104] = 0.0118736555368507*G0_0_1 + 0.0212681961813913*G0_0_2 + 0.0118736555368509*G0_1_1 + 0.0212681961813913*G0_1_2 + 0.021268196181391*G0_2_0 - 0.014613729891507*G0_2_1 - 0.0693499771190733*G0_2_2;
    A[1105] = 0.00254435475789585*G0_0_1 + 0.00583896797004434*G0_0_2 + 0.00254435475789593*G0_1_1 + 0.00583896797004438*G0_1_2 + 0.00583896797004426*G0_2_0 - 0.00590420783563142*G0_2_1 - 0.031869674339292*G0_2_2;
    A[1106] = 0.0118736555368506*G0_0_1 + 0.0212681961813907*G0_0_2 + 0.0118736555368506*G0_1_1 + 0.0212681961813907*G0_1_2 + 0.0212681961813903*G0_2_0 - 0.00939454064454015*G0_2_1 - 0.0187890812890812*G0_2_2;
    A[1107] = -0.0429604514890982*G0_0_1 - 0.0422754329004342*G0_0_2 - 0.0429604514890981*G0_1_1 - 0.0422754329004343*G0_1_2 - 0.0422754329004335*G0_2_0 - 0.0359145460056925*G0_2_1 + 0.0920860702761727*G0_2_2;
    A[1108] = -0.0429604514890969*G0_0_1 - 0.0422754329004328*G0_0_2 - 0.042960451489097*G0_1_1 - 0.0422754329004329*G0_1_2 - 0.0422754329004319*G0_2_0 - 0.000685018588664243*G0_2_1 + 0.0439716694056969*G0_2_2;
    A[1109] = 0.124738623002511*G0_0_1 + 0.0962940416065423*G0_0_2 + 0.124738623002511*G0_1_1 + 0.0962940416065424*G0_1_2 + 0.0962940416065412*G0_2_0 + 0.0284445813959708*G0_2_1 - 0.0644569872000414*G0_2_2;
    A[1110] = -0.00939454064454034*G0_0_2 - 0.00939454064454026*G0_1_2 - 0.00939454064454043*G0_2_0 - 0.00939454064454051*G0_2_1 + 0.045341706583026*G0_2_2;
    A[1111] = -0.00329461321214802*G0_0_2 - 0.00329461321214797*G0_1_2 - 0.00329461321214789*G0_2_0 - 0.00329461321214752*G0_2_1 + 0.0226708532915132*G0_2_2;
    A[1112] = -0.00939454064454026*G0_0_2 - 0.00939454064454027*G0_1_2 - 0.0093945406445406*G0_2_0 - 0.00939454064454078*G0_2_1;
    A[1113] = -0.00329461321214781*G0_0_2 - 0.00329461321214786*G0_1_2 - 0.00329461321214771*G0_2_0 - 0.00329461321214786*G0_2_1 + 0.0226708532915131*G0_2_2;
    A[1114] = -0.00329461321214804*G0_0_2 - 0.00329461321214807*G0_1_2 - 0.00329461321214825*G0_2_0 - 0.00329461321214806*G0_2_1;
    A[1115] = -0.0093945406445404*G0_0_2 - 0.0093945406445404*G0_1_2 - 0.00939454064454051*G0_2_0 - 0.00939454064454072*G0_2_1;
    A[1116] = 0.00260959462348229*G0_0_2 + 0.0026095946234823*G0_1_2 + 0.00260959462348312*G0_2_0 + 0.00260959462348305*G0_2_1 - 0.151356488162043*G0_2_2;
    A[1117] = 0.00260959462348388*G0_0_2 + 0.00260959462348385*G0_1_2 + 0.00260959462348382*G0_2_0 + 0.00260959462348376*G0_2_1 - 0.0453417065830256*G0_2_2;
    A[1118] = 0.00260959462348334*G0_0_2 + 0.00260959462348349*G0_1_2 + 0.00260959462348407*G0_2_0 + 0.0026095946234838*G0_2_1 - 0.0453417065830262*G0_2_2;
    A[1119] = 0.0352295274170277*G0_0_2 + 0.0352295274170275*G0_1_2 + 0.0352295274170277*G0_2_0 + 0.0352295274170273*G0_2_1 + 0.166687856575008*G0_2_2;
    A[1120] = -0.0178705039816148*G0_0_0 - 0.0178705039816148*G0_0_1 - 0.0178705039816148*G0_0_2 - 0.0551276864210885*G0_1_0 - 0.0551276864210885*G0_1_1 - 0.0551276864210885*G0_1_2 - 0.0178705039816148*G0_2_0 - 0.0178705039816148*G0_2_1 - 0.0178705039816148*G0_2_2;
    A[1121] = -0.00480034930989794*G0_0_0 - 0.00480034930989794*G0_2_0;
    A[1122] = 0.0105310191030676*G0_0_1 + 0.0153313684129654*G0_1_1 + 0.0105310191030676*G0_2_1;
    A[1123] = -0.00480034930989783*G0_0_2 - 0.00480034930989785*G0_2_2;
    A[1124] = 0.0274594594256046*G0_0_1 + 0.0328352243499807*G0_0_2 + 0.022083694501229*G0_1_1 + 0.0274594594256048*G0_1_2 + 0.0274594594256046*G0_2_1 + 0.0328352243499807*G0_2_2;
    A[1125] = 0.00887914570640293*G0_0_1 + 0.0111690649885094*G0_0_2 + 0.00658922642429573*G0_1_1 + 0.00887914570640258*G0_1_2 + 0.00887914570640287*G0_2_1 + 0.0111690649885094*G0_2_2;
    A[1126] = 0.0116844599266469*G0_0_1 + 0.0139743792087541*G0_0_2 + 0.00939454064454049*G0_1_1 + 0.0116844599266472*G0_1_2 + 0.011684459926647*G0_2_1 + 0.0139743792087541*G0_2_2;
    A[1127] = 0.00537576492437616*G0_0_1 + 0.0107515298487519*G0_0_2 + 0.00537576492437611*G0_1_2 + 0.00537576492437611*G0_2_1 + 0.0107515298487519*G0_2_2;
    A[1128] = 0.00537576492437622*G0_0_0 + 0.00537576492437611*G0_0_2 + 0.0053757649243762*G0_2_0 + 0.00537576492437607*G0_2_2;
    A[1129] = 0.00228991928210649*G0_0_0 + 0.00228991928210691*G0_0_2 + 0.00228991928210654*G0_2_0 + 0.00228991928210697*G0_2_2;
    A[1130] = 0.00228991928210679*G0_0_0 + 0.00228991928210652*G0_0_2 + 0.00228991928210675*G0_2_0 + 0.00228991928210647*G0_2_2;
    A[1131] = 0.00537576492437617*G0_0_0 + 0.0053757649243762*G0_0_2 + 0.00537576492437612*G0_2_0 + 0.00537576492437618*G0_2_2;
    A[1132] = 0.0107515298487521*G0_0_0 + 0.00537576492437613*G0_0_1 + 0.00537576492437599*G0_1_0 + 0.010751529848752*G0_2_0 + 0.00537576492437609*G0_2_1;
    A[1133] = 0.0139743792087538*G0_0_0 + 0.011684459926647*G0_0_1 + 0.011684459926647*G0_1_0 + 0.00939454064454031*G0_1_1 + 0.0139743792087538*G0_2_0 + 0.011684459926647*G0_2_1;
    A[1134] = 0.0111690649885094*G0_0_0 + 0.00887914570640274*G0_0_1 + 0.00887914570640275*G0_1_0 + 0.00658922642429584*G0_1_1 + 0.0111690649885093*G0_2_0 + 0.00887914570640274*G0_2_1;
    A[1135] = 0.0328352243499807*G0_0_0 + 0.0274594594256047*G0_0_1 + 0.0274594594256048*G0_1_0 + 0.0220836945012289*G0_1_1 + 0.0328352243499808*G0_2_0 + 0.0274594594256046*G0_2_1;
    A[1136] = 0.0895482395048359*G0_0_0 + 0.089548239504836*G0_0_1 + 0.0196371995417131*G0_0_2 + 0.0196371995417131*G0_1_0 + 0.0196371995417132*G0_1_1 + 0.0895482395048357*G0_1_2 + 0.0895482395048359*G0_2_0 + 0.089548239504836*G0_2_1 + 0.0196371995417131*G0_2_2;
    A[1137] = -0.0574176057031954*G0_0_0 - 0.0574176057031954*G0_0_1 + 0.0124934342599273*G0_0_2 - 0.0186912214907006*G0_1_0 - 0.0186912214907006*G0_1_1 - 0.0574176057031954*G0_1_2 - 0.0574176057031954*G0_2_0 - 0.0574176057031955*G0_2_1 + 0.0124934342599272*G0_2_2;
    A[1138] = 0.0283728175438242*G0_0_0 + 0.0283728175438242*G0_0_1 - 0.0315434750113568*G0_0_2 + 0.00939454064454053*G0_1_0 + 0.00939454064454054*G0_1_1 + 0.0283728175438241*G0_1_2 + 0.0283728175438242*G0_2_0 + 0.0283728175438242*G0_2_1 - 0.0315434750113567*G0_2_2;
    A[1139] = -0.00537576492437617*G0_0_0 - 0.00537576492437613*G0_0_1 + 0.022083694501229*G0_0_2 - 0.00537576492437597*G0_1_2 - 0.00537576492437612*G0_2_0 - 0.00537576492437608*G0_2_1 + 0.022083694501229*G0_2_2;
    A[1140] = 0.179096479009671*G0_0_0 + 0.109185439046549*G0_0_1 + 0.179096479009671*G0_0_2 + 0.109185439046549*G0_1_0 + 0.179096479009671*G0_1_1 + 0.109185439046549*G0_1_2 + 0.179096479009671*G0_2_0 + 0.109185439046549*G0_2_1 + 0.179096479009671*G0_2_2;
    A[1141] = -0.147063705006411*G0_0_0 - 0.17344670664983*G0_0_1 - 0.147063705006411*G0_0_2 - 0.108337320793916*G0_1_0 - 0.243357746612952*G0_1_1 - 0.108337320793916*G0_1_2 - 0.147063705006411*G0_2_0 - 0.17344670664983*G0_2_1 - 0.147063705006411*G0_2_2;
    A[1142] = 0.100717304493345*G0_0_0 + 0.125351877739029*G0_0_1 + 0.100717304493345*G0_0_2 + 0.0817390275940609*G0_1_0 + 0.185268170294209*G0_1_1 + 0.0817390275940609*G0_1_2 + 0.100717304493345*G0_2_0 + 0.125351877739029*G0_2_1 + 0.100717304493345*G0_2_2;
    A[1143] = -0.0328352243499808*G0_0_0 - 0.053751125257201*G0_0_1 - 0.0328352243499809*G0_0_2 - 0.0274594594256048*G0_1_0 - 0.0812105846828057*G0_1_1 - 0.0274594594256049*G0_1_2 - 0.0328352243499808*G0_2_0 - 0.0537511252572011*G0_2_1 - 0.0328352243499809*G0_2_2;
    A[1144] = 0.0196371995417131*G0_0_0 + 0.0895482395048359*G0_0_1 + 0.0895482395048359*G0_0_2 + 0.0895482395048354*G0_1_0 + 0.0196371995417128*G0_1_1 + 0.0196371995417128*G0_1_2 + 0.0196371995417131*G0_2_0 + 0.0895482395048359*G0_2_1 + 0.0895482395048358*G0_2_2;
    A[1145] = 0.0124934342599272*G0_0_0 - 0.0574176057031958*G0_0_1 - 0.0574176057031957*G0_0_2 - 0.0574176057031948*G0_1_0 - 0.0186912214907002*G0_1_1 - 0.0186912214907002*G0_1_2 + 0.0124934342599272*G0_2_0 - 0.0574176057031957*G0_2_1 - 0.0574176057031957*G0_2_2;
    A[1146] = -0.0315434750113565*G0_0_0 + 0.0283728175438243*G0_0_1 + 0.0283728175438242*G0_0_2 + 0.0283728175438238*G0_1_0 + 0.0093945406445402*G0_1_1 + 0.00939454064454018*G0_1_2 - 0.0315434750113565*G0_2_0 + 0.0283728175438243*G0_2_1 + 0.0283728175438242*G0_2_2;
    A[1147] = 0.0220836945012291*G0_0_0 - 0.00537576492437599*G0_0_1 - 0.00537576492437596*G0_0_2 - 0.00537576492437577*G0_1_0 + 0.0220836945012291*G0_2_0 - 0.00537576492437596*G0_2_1 - 0.00537576492437592*G0_2_2;
    A[1148] = 0.0187890812890807*G0_0_0 + 0.00939454064454069*G0_0_1 + 0.0187890812890809*G0_0_2 + 0.00939454064454086*G0_1_0 + 0.00939454064454062*G0_1_2 + 0.0187890812890806*G0_2_0 + 0.00939454064454062*G0_2_1 + 0.0187890812890807*G0_2_2;
    A[1149] = 0.0292600797158082*G0_0_0 + 0.0259654665036603*G0_0_1 + 0.0292600797158083*G0_0_2 + 0.0259654665036606*G0_1_0 + 0.0226708532915129*G0_1_1 + 0.0259654665036604*G0_1_2 + 0.0292600797158083*G0_2_0 + 0.0259654665036603*G0_2_1 + 0.0292600797158083*G0_2_2;
    A[1150] = 0.0641307878721061*G0_0_0 + 0.0547362472275656*G0_0_1 + 0.064130787872106*G0_0_2 + 0.0547362472275657*G0_1_0 + 0.045341706583025*G0_1_1 + 0.0547362472275658*G0_1_2 + 0.064130787872106*G0_2_0 + 0.0547362472275655*G0_2_1 + 0.0641307878721058*G0_2_2;
    A[1151] = 0.00658922642429619*G0_0_0 + 0.00329461321214765*G0_0_1 + 0.00658922642429546*G0_0_2 + 0.00329461321214786*G0_1_0 + 0.00329461321214832*G0_1_2 + 0.0065892264242963*G0_2_0 + 0.00329461321214775*G0_2_1 + 0.00658922642429568*G0_2_2;
    A[1152] = 0.0292600797158088*G0_0_0 + 0.0259654665036609*G0_0_1 + 0.0292600797158087*G0_0_2 + 0.0259654665036604*G0_1_0 + 0.0226708532915127*G0_1_1 + 0.0259654665036605*G0_1_2 + 0.0292600797158089*G0_2_0 + 0.0259654665036611*G0_2_1 + 0.029260079715809*G0_2_2;
    A[1153] = 0.018789081289081*G0_0_0 + 0.0093945406445409*G0_0_1 + 0.0187890812890814*G0_0_2 + 0.00939454064454068*G0_1_0 + 0.0093945406445404*G0_1_2 + 0.0187890812890809*G0_2_0 + 0.00939454064454071*G0_2_1 + 0.0187890812890813*G0_2_2;
    A[1154] = -0.0644569872000415*G0_0_0 - 0.160751028806583*G0_0_1 - 0.257045070413123*G0_0_2 - 0.0929015685960121*G0_1_0 - 0.064456987200042*G0_1_1 - 0.160751028806583*G0_1_2 - 0.0644569872000416*G0_2_0 - 0.160751028806583*G0_2_1 - 0.257045070413123*G0_2_2;
    A[1155] = 0.1273155976932*G0_0_0 + 0.134361503176606*G0_0_1 + 0.176636936077038*G0_0_2 + 0.128000616281865*G0_1_0 + 0.0920860702761738*G0_1_1 + 0.134361503176606*G0_1_2 + 0.1273155976932*G0_2_0 + 0.134361503176606*G0_2_1 + 0.176636936077038*G0_2_2;
    A[1156] = -0.0641307878721062*G0_0_0 - 0.0906181733004636*G0_0_1 - 0.111886369481854*G0_0_2 - 0.0547362472275657*G0_1_0 - 0.0693499771190739*G0_1_1 - 0.0906181733004634*G0_1_2 - 0.0641307878721061*G0_2_0 - 0.0906181733004637*G0_2_1 - 0.111886369481854*G0_2_2;
    A[1157] = 0.0439716694056962*G0_0_0 + 0.0862471023061288*G0_0_1 + 0.128522535206562*G0_0_2 + 0.0446566879943608*G0_1_0 + 0.0439716694056964*G0_1_1 + 0.0862471023061294*G0_1_2 + 0.0439716694056964*G0_2_0 + 0.086247102306129*G0_2_1 + 0.128522535206562*G0_2_2;
    A[1158] = -0.0292600797158088*G0_0_0 - 0.0377086423093362*G0_0_1 - 0.0435476102793807*G0_0_2 - 0.0259654665036605*G0_1_0 - 0.0318696743392916*G0_1_1 - 0.0377086423093365*G0_1_2 - 0.029260079715809*G0_2_0 - 0.0377086423093363*G0_2_1 - 0.0435476102793807*G0_2_2;
    A[1159] = -0.0187890812890811*G0_0_0 - 0.0400572774704712*G0_0_1 - 0.0613254736518617*G0_0_2 - 0.00939454064454081*G0_1_0 - 0.0187890812890812*G0_1_1 - 0.0400572774704716*G0_1_2 - 0.018789081289081*G0_2_0 - 0.0400572774704713*G0_2_1 - 0.0613254736518617*G0_2_2;
    A[1160] = -0.128522535206562*G0_0_0 - 0.0322284936000208*G0_0_1 - 0.128522535206562*G0_0_2 - 0.032228493600021*G0_1_0 - 0.0606730749959911*G0_1_1 - 0.0322284936000206*G0_1_2 - 0.128522535206562*G0_2_0 - 0.0322284936000209*G0_2_1 - 0.128522535206562*G0_2_2;
    A[1161] = 0.0642612676032811*G0_0_0 + 0.0219858347028482*G0_0_1 + 0.0642612676032814*G0_0_2 + 0.0219858347028478*G0_1_0 + 0.0226708532915123*G0_1_1 + 0.021985834702848*G0_1_2 + 0.0642612676032811*G0_2_0 + 0.0219858347028483*G0_2_1 + 0.0642612676032814*G0_2_2;
    A[1162] = -0.0306627368259313*G0_0_0 - 0.00939454064454086*G0_0_1 - 0.0306627368259314*G0_0_2 - 0.00939454064454008*G0_1_0 - 0.00939454064454025*G0_1_2 - 0.0306627368259313*G0_2_0 - 0.0093945406445408*G0_2_1 - 0.0306627368259314*G0_2_2;
    A[1163] = 0.0642612676032808*G0_0_0 + 0.0219858347028483*G0_0_1 + 0.0642612676032808*G0_0_2 + 0.0219858347028487*G0_1_0 + 0.0226708532915131*G0_1_1 + 0.0219858347028484*G0_1_2 + 0.0642612676032809*G0_2_0 + 0.0219858347028484*G0_2_1 + 0.0642612676032808*G0_2_2;
    A[1164] = -0.00913358118219183*G0_0_0 - 0.00329461321214749*G0_0_1 - 0.00913358118219189*G0_0_2 - 0.00329461321214826*G0_1_0 - 0.00329461321214785*G0_1_2 - 0.00913358118219198*G0_2_0 - 0.00329461321214759*G0_2_1 - 0.00913358118219192*G0_2_2;
    A[1165] = -0.0306627368259311*G0_0_0 - 0.00939454064454092*G0_0_1 - 0.0306627368259311*G0_0_2 - 0.00939454064454056*G0_1_0 - 0.00939454064454068*G0_1_2 - 0.030662736825931*G0_2_0 - 0.00939454064454073*G0_2_1 - 0.0306627368259311*G0_2_2;
    A[1166] = -0.257045070413123*G0_0_0 - 0.160751028806582*G0_0_1 - 0.0644569872000414*G0_0_2 - 0.160751028806582*G0_1_0 - 0.0644569872000412*G0_1_1 - 0.0929015685960118*G0_1_2 - 0.257045070413123*G0_2_0 - 0.160751028806582*G0_2_1 - 0.0644569872000415*G0_2_2;
    A[1167] = 0.128522535206562*G0_0_0 + 0.0862471023061296*G0_0_1 + 0.0439716694056963*G0_0_2 + 0.0862471023061286*G0_1_0 + 0.0439716694056961*G0_1_1 + 0.0446566879943604*G0_1_2 + 0.128522535206562*G0_2_0 + 0.0862471023061296*G0_2_1 + 0.0439716694056963*G0_2_2;
    A[1168] = -0.0613254736518621*G0_0_0 - 0.0400572774704716*G0_0_1 - 0.0187890812890812*G0_0_2 - 0.040057277470471*G0_1_0 - 0.0187890812890806*G0_1_1 - 0.00939454064454076*G0_1_2 - 0.0613254736518621*G0_2_0 - 0.0400572774704716*G0_2_1 - 0.0187890812890811*G0_2_2;
    A[1169] = 0.176636936077038*G0_0_0 + 0.134361503176605*G0_0_1 + 0.1273155976932*G0_0_2 + 0.134361503176606*G0_1_0 + 0.0920860702761732*G0_1_1 + 0.128000616281865*G0_1_2 + 0.176636936077038*G0_2_0 + 0.134361503176605*G0_2_1 + 0.1273155976932*G0_2_2;
    A[1170] = -0.0435476102793806*G0_0_0 - 0.0377086423093364*G0_0_1 - 0.029260079715808*G0_0_2 - 0.0377086423093364*G0_1_0 - 0.0318696743392918*G0_1_1 - 0.0259654665036603*G0_1_2 - 0.0435476102793806*G0_2_0 - 0.0377086423093364*G0_2_1 - 0.029260079715808*G0_2_2;
    A[1171] = -0.111886369481854*G0_0_0 - 0.0906181733004635*G0_0_1 - 0.0641307878721061*G0_0_2 - 0.0906181733004637*G0_1_0 - 0.0693499771190736*G0_1_1 - 0.054736247227566*G0_1_2 - 0.111886369481854*G0_2_0 - 0.0906181733004635*G0_2_1 - 0.0641307878721059*G0_2_2;
    A[1172] = 0.0962288017409527*G0_0_0 + 0.13145832915798*G0_0_1 + 0.0962288017409528*G0_0_2 + 0.131458329157981*G0_1_0 + 0.166687856575007*G0_1_1 + 0.13145832915798*G0_1_2 + 0.0962288017409529*G0_2_0 + 0.13145832915798*G0_2_1 + 0.0962288017409532*G0_2_2;
    A[1173] = -0.0505608958299918*G0_0_0 - 0.0479513012065082*G0_0_1 - 0.0505608958299918*G0_0_2 - 0.0479513012065093*G0_1_0 - 0.0453417065830252*G0_1_1 - 0.0479513012065087*G0_1_2 - 0.0505608958299919*G0_2_0 - 0.0479513012065084*G0_2_1 - 0.050560895829992*G0_2_2;
    A[1174] = -0.156575677409008*G0_0_0 - 0.153966082785525*G0_0_1 - 0.156575677409009*G0_0_2 - 0.153966082785526*G0_1_0 - 0.151356488162041*G0_1_1 - 0.153966082785526*G0_1_2 - 0.156575677409008*G0_2_0 - 0.153966082785525*G0_2_1 - 0.156575677409009*G0_2_2;
    A[1175] = -0.0505608958299924*G0_0_0 - 0.0479513012065093*G0_0_1 - 0.0505608958299928*G0_0_2 - 0.0479513012065087*G0_1_0 - 0.045341706583026*G0_1_1 - 0.0479513012065089*G0_1_2 - 0.0505608958299927*G0_2_0 - 0.0479513012065097*G0_2_1 - 0.050560895829993*G0_2_2;
    A[1176] = 0.0214691349673985*G0_0_0 + 0.0214691349673985*G0_0_1 + 0.0214691349673984*G0_0_2 + 0.0551276864210885*G0_1_0 + 0.0551276864210885*G0_1_1 + 0.0551276864210885*G0_1_2 + 0.0214691349673985*G0_2_0 + 0.0214691349673985*G0_2_1 + 0.0214691349673985*G0_2_2;
    A[1177] = -0.00336507226698186*G0_0_0 - 0.00336507226698188*G0_2_0;
    A[1178] = -0.0232632312710435*G0_0_1 - 0.0388177200243168*G0_1_1 - 0.0232632312710435*G0_2_1;
    A[1179] = -0.00336507226698201*G0_0_2 - 0.00336507226698199*G0_2_2;
    A[1180] = -0.0599162925551803*G0_0_1 - 0.078894569454464*G0_0_2 - 0.0315434750113566*G0_1_1 - 0.0599162925551806*G0_1_2 - 0.0599162925551804*G0_2_1 - 0.0788945694544642*G0_2_2;
    A[1181] = -0.0057802520910159*G0_0_1 - 0.00523223722008424*G0_0_2 - 0.00626302709635985*G0_1_1 - 0.00578025209101555*G0_1_2 - 0.00578025209101586*G0_2_1 - 0.00523223722008417*G0_2_2;
    A[1182] = -0.0131523569023565*G0_0_1 - 0.0263047138047136*G0_0_2 + 6.52398655869549e-05*G0_1_1 - 0.0131523569023568*G0_1_2 - 0.0131523569023566*G0_2_1 - 0.0263047138047136*G0_2_2;
    A[1183] = 0.00228991928210671*G0_0_1 + 0.0139743792087542*G0_0_2 + 0.00228991928210681*G0_1_2 + 0.00228991928210678*G0_2_1 + 0.0139743792087542*G0_2_2;
    A[1184] = 0.00228991928210651*G0_0_0 + 0.00228991928210669*G0_0_2 + 0.00228991928210649*G0_2_0 + 0.0022899192821067*G0_2_2;
    A[1185] = 0.00178757231708631*G0_0_0 + 0.00178757231708591*G0_0_2 + 0.00178757231708641*G0_2_0 + 0.00178757231708596*G0_2_2;
    A[1186] = 0.00178757231708615*G0_0_0 + 0.00178757231708639*G0_0_2 + 0.00178757231708613*G0_2_0 + 0.0017875723170864*G0_2_2;
    A[1187] = 0.00228991928210667*G0_0_0 + 0.00228991928210663*G0_0_2 + 0.00228991928210674*G0_2_0 + 0.00228991928210667*G0_2_2;
    A[1188] = 0.0139743792087539*G0_0_0 + 0.0022899192821067*G0_0_1 + 0.00228991928210647*G0_1_0 + 0.013974379208754*G0_2_0 + 0.00228991928210672*G0_2_1;
    A[1189] = -0.026304713804713*G0_0_0 - 0.0131523569023563*G0_0_1 - 0.0131523569023561*G0_1_0 + 6.52398655877318e-05*G0_1_1 - 0.0263047138047132*G0_2_0 - 0.0131523569023563*G0_2_1;
    A[1190] = -0.00523223722008482*G0_0_0 - 0.00578025209101648*G0_0_1 - 0.00578025209101644*G0_1_0 - 0.00626302709636073*G0_1_1 - 0.00523223722008449*G0_2_0 - 0.0057802520910164*G0_2_1;
    A[1191] = -0.0788945694544641*G0_0_0 - 0.0599162925551804*G0_0_1 - 0.0599162925551807*G0_1_0 - 0.0315434750113566*G0_1_1 - 0.0788945694544643*G0_2_0 - 0.0599162925551804*G0_2_1;
    A[1192] = -0.0574176057031954*G0_0_0 - 0.0574176057031954*G0_0_1 - 0.0186912214907002*G0_0_2 + 0.0124934342599274*G0_1_0 + 0.0124934342599273*G0_1_1 - 0.0574176057031951*G0_1_2 - 0.0574176057031953*G0_2_0 - 0.0574176057031954*G0_2_1 - 0.0186912214907002*G0_2_2;
    A[1193] = 0.00506261356955768*G0_0_0 + 0.00506261356955771*G0_0_1 + 0.000260959462348013*G0_0_2 + 0.000260959462348172*G0_1_0 + 0.000260959462348206*G0_1_1 + 0.00506261356955765*G0_1_2 + 0.00506261356955765*G0_2_0 + 0.00506261356955768*G0_2_1 + 0.000260959462348044*G0_2_2;
    A[1194] = -0.000482775005344317*G0_0_0 - 0.00048277500534437*G0_0_1 - 0.00626302709635996*G0_0_2 + 6.52398655870962e-05*G0_1_0 + 6.52398655870605e-05*G0_1_1 - 0.000482775005344124*G0_1_2 - 0.000482775005344271*G0_2_0 - 0.000482775005344325*G0_2_1 - 0.00626302709636001*G0_2_2;
    A[1195] = -0.0022899192821067*G0_0_0 - 0.00228991928210675*G0_0_1 + 0.00658922642429581*G0_0_2 - 0.00228991928210695*G0_1_2 - 0.00228991928210677*G0_2_0 - 0.00228991928210682*G0_2_1 + 0.00658922642429582*G0_2_2;
    A[1196] = -0.147063705006411*G0_0_0 - 0.108337320793916*G0_0_1 - 0.147063705006411*G0_0_2 - 0.17344670664983*G0_1_0 - 0.243357746612952*G0_1_1 - 0.17344670664983*G0_1_2 - 0.147063705006411*G0_2_0 - 0.108337320793916*G0_2_1 - 0.147063705006411*G0_2_2;
    A[1197] = 0.304226541205704*G0_0_0 + 0.226095278178609*G0_0_1 + 0.304226541205704*G0_0_2 + 0.226095278178609*G0_1_0 + 0.428965164208215*G0_1_1 + 0.226095278178609*G0_1_2 + 0.304226541205704*G0_2_0 + 0.226095278178609*G0_2_1 + 0.304226541205704*G0_2_2;
    A[1198] = -0.204187731314465*G0_0_0 - 0.219493003781196*G0_0_1 - 0.204187731314465*G0_0_2 - 0.167692550505048*G0_1_0 - 0.387185554286245*G0_1_1 - 0.167692550505048*G0_1_2 - 0.204187731314465*G0_2_0 - 0.219493003781196*G0_2_1 - 0.204187731314465*G0_2_2;
    A[1199] = 0.0788945694544643*G0_0_0 + 0.103529142700148*G0_0_1 + 0.0788945694544643*G0_0_2 + 0.0599162925551808*G0_1_0 + 0.18526817029421*G0_1_1 + 0.0599162925551808*G0_1_2 + 0.0788945694544644*G0_2_0 + 0.103529142700149*G0_2_1 + 0.0788945694544645*G0_2_2;
    A[1200] = -0.0186912214907002*G0_0_0 - 0.0574176057031955*G0_0_1 - 0.0574176057031956*G0_0_2 - 0.057417605703195*G0_1_0 + 0.0124934342599275*G0_1_1 + 0.0124934342599276*G0_1_2 - 0.0186912214907003*G0_2_0 - 0.0574176057031955*G0_2_1 - 0.0574176057031956*G0_2_2;
    A[1201] = 0.000260959462348455*G0_0_0 + 0.00506261356955835*G0_0_1 + 0.00506261356955834*G0_0_2 + 0.00506261356955743*G0_1_0 + 0.000260959462348345*G0_1_1 + 0.000260959462348253*G0_1_2 + 0.00026095946234851*G0_2_0 + 0.00506261356955827*G0_2_1 + 0.00506261356955826*G0_2_2;
    A[1202] = -0.00626302709636054*G0_0_0 - 0.000482775005344432*G0_0_1 - 0.000482775005344464*G0_0_2 - 0.00048277500534411*G0_1_0 + 6.52398655872696e-05*G0_1_1 + 6.52398655873361e-05*G0_1_2 - 0.00626302709636061*G0_2_0 - 0.000482775005344427*G0_2_1 - 0.000482775005344458*G0_2_2;
    A[1203] = 0.00658922642429574*G0_0_0 - 0.00228991928210681*G0_0_1 - 0.00228991928210675*G0_0_2 - 0.00228991928210694*G0_1_0 + 0.00658922642429578*G0_2_0 - 0.00228991928210683*G0_2_1 - 0.00228991928210676*G0_2_2;
    A[1204] = -0.0148746893538551*G0_0_0 + 6.52398655869713e-05*G0_0_1 - 0.0148746893538555*G0_0_2 + 6.52398655870026e-05*G0_1_0 + 6.52398655868226e-05*G0_1_2 - 0.0148746893538549*G0_2_0 + 6.52398655871299e-05*G0_2_1 - 0.0148746893538554*G0_2_2;
    A[1205] = -0.0189848008858424*G0_0_0 - 0.00949240044292075*G0_0_1 - 0.0189848008858423*G0_0_2 - 0.00949240044292153*G0_1_0 - 0.000652398655871126*G0_1_1 - 0.00949240044292106*G0_1_2 - 0.0189848008858424*G0_2_0 - 0.00949240044292079*G0_2_1 - 0.0189848008858423*G0_2_2;
    A[1206] = -0.127087258163644*G0_0_0 - 0.0893133759887212*G0_0_1 - 0.127087258163645*G0_0_2 - 0.0893133759887213*G0_1_0 - 0.0665446628988278*G0_1_1 - 0.0893133759887221*G0_1_2 - 0.127087258163644*G0_2_0 - 0.0893133759887211*G0_2_1 - 0.127087258163645*G0_2_2;
    A[1207] = 0.00450155072550844*G0_0_0 + 0.00257697469069006*G0_0_1 + 0.00450155072550928*G0_0_2 + 0.00257697469068961*G0_1_0 + 0.00257697469068942*G0_1_2 + 0.00450155072550824*G0_2_0 + 0.0025769746906901*G0_2_1 + 0.00450155072550927*G0_2_2;
    A[1208] = -0.0189848008858421*G0_0_0 - 0.00949240044292141*G0_0_1 - 0.0189848008858417*G0_0_2 - 0.0094924004429205*G0_1_0 - 0.000652398655870894*G0_1_1 - 0.00949240044292051*G0_1_2 - 0.0189848008858423*G0_2_0 - 0.00949240044292179*G0_2_1 - 0.0189848008858421*G0_2_2;
    A[1209] = -0.0148746893538557*G0_0_0 + 6.52398655868822e-05*G0_0_1 - 0.0148746893538565*G0_0_2 + 6.52398655871571e-05*G0_1_0 + 6.52398655871641e-05*G0_1_2 - 0.0148746893538557*G0_2_0 + 6.52398655870058e-05*G0_2_1 - 0.0148746893538564*G0_2_2;
    A[1210] = 0.132306447410612*G0_0_0 + 0.194675758911868*G0_0_1 + 0.121346149991981*G0_0_2 + 0.0589768384907262*G0_1_0 - 0.00339247301052899*G0_1_1 + 0.194675758911868*G0_1_2 + 0.132306447410612*G0_2_0 + 0.194675758911868*G0_2_1 + 0.121346149991981*G0_2_2;
    A[1211] = -0.166948816037355*G0_0_0 - 0.250814663249556*G0_0_1 - 0.334680510461757*G0_0_2 - 0.160783648739376*G0_1_0 - 0.0622388317700811*G0_1_1 - 0.250814663249556*G0_1_2 - 0.166948816037355*G0_2_0 - 0.250814663249556*G0_2_1 - 0.334680510461757*G0_2_2;
    A[1212] = 0.127087258163644*G0_0_0 + 0.173472802596064*G0_0_1 + 0.209419968534549*G0_0_2 + 0.0893133759887213*G0_1_0 + 0.0846813455320394*G0_1_1 + 0.173472802596064*G0_1_2 + 0.127087258163644*G0_2_0 + 0.173472802596064*G0_2_1 + 0.209419968534549*G0_2_2;
    A[1213] = -0.0436454700777608*G0_0_0 - 0.0371541034518452*G0_0_1 + 0.0371867233846405*G0_0_2 - 0.00975335990526901*G0_1_0 - 0.00678494602105624*G0_1_1 - 0.0371541034518461*G0_1_2 - 0.0436454700777609*G0_2_0 - 0.0371541034518454*G0_2_1 + 0.0371867233846405*G0_2_2;
    A[1214] = 0.0189848008858421*G0_0_0 + 0.0378717419733033*G0_0_1 + 0.0567586830607657*G0_0_2 + 0.00949240044292057*G0_1_0 + 0.0127870136550679*G0_1_1 + 0.0378717419733042*G0_1_2 + 0.0189848008858424*G0_2_0 + 0.0378717419733034*G0_2_1 + 0.0567586830607657*G0_2_2;
    A[1215] = 0.0148746893538558*G0_0_0 + 0.0136351319077011*G0_0_1 - 0.0254435475789643*G0_0_2 - 6.52398655870746e-05*G0_1_0 - 0.000130479731173964*G0_1_1 + 0.0136351319077012*G0_1_2 + 0.0148746893538558*G0_2_0 + 0.0136351319077011*G0_2_1 - 0.0254435475789643*G0_2_2;
    A[1216] = 0.0476251018785738*G0_0_0 - 0.0147442096226821*G0_0_1 + 0.0476251018785738*G0_0_2 - 0.0147442096226817*G0_1_0 + 0.0476251018785737*G0_1_1 - 0.0147442096226822*G0_1_2 + 0.0476251018785737*G0_2_0 - 0.0147442096226821*G0_2_1 + 0.0476251018785737*G0_2_2;
    A[1217] = 0.00880738185425675*G0_0_0 + 0.00231601522834183*G0_0_1 + 0.00880738185425623*G0_0_2 + 0.00231601522834195*G0_1_0 - 0.000652398655870384*G0_1_1 + 0.00231601522834185*G0_1_2 + 0.00880738185425676*G0_2_0 + 0.00231601522834177*G0_2_1 + 0.00880738185425628*G0_2_2;
    A[1218] = -0.00130479731174163*G0_0_0 - 6.5239865586931e-05*G0_0_1 - 0.00130479731174157*G0_0_2 - 6.52398655871916e-05*G0_1_0 - 6.52398655874389e-05*G0_1_2 - 0.00130479731174158*G0_2_0 - 6.52398655870902e-05*G0_2_1 - 0.00130479731174162*G0_2_2;
    A[1219] = 0.00880738185425666*G0_0_0 + 0.002316015228341*G0_0_1 + 0.00880738185425675*G0_0_2 + 0.00231601522834132*G0_1_0 - 0.000652398655871401*G0_1_1 + 0.00231601522834145*G0_1_2 + 0.00880738185425673*G0_2_0 + 0.00231601522834104*G0_2_1 + 0.00880738185425683*G0_2_2;
    A[1220] = -0.0156575677409007*G0_0_0 - 0.0025769746906901*G0_0_1 - 0.015657567740901*G0_0_2 - 0.00257697469068982*G0_1_0 - 0.00257697469069006*G0_1_2 - 0.0156575677409009*G0_2_0 - 0.00257697469069013*G0_2_1 - 0.015657567740901*G0_2_2;
    A[1221] = -0.00130479731174169*G0_0_0 - 6.52398655868603e-05*G0_0_1 - 0.00130479731174152*G0_0_2 - 6.52398655867784e-05*G0_1_0 - 6.52398655867649e-05*G0_1_2 - 0.00130479731174172*G0_2_0 - 6.5239865586984e-05*G0_2_1 - 0.00130479731174163*G0_2_2;
    A[1222] = 0.121346149991982*G0_0_0 + 0.194675758911868*G0_0_1 + 0.132306447410612*G0_0_2 + 0.194675758911868*G0_1_0 - 0.00339247301052967*G0_1_1 + 0.0589768384907262*G0_1_2 + 0.121346149991981*G0_2_0 + 0.194675758911868*G0_2_1 + 0.132306447410612*G0_2_2;
    A[1223] = 0.03718672338464*G0_0_0 - 0.0371541034518467*G0_0_1 - 0.0436454700777609*G0_0_2 - 0.0371541034518454*G0_1_0 - 0.00678494602105651*G0_1_1 - 0.00975335990526861*G0_1_2 + 0.0371867233846401*G0_2_0 - 0.0371541034518465*G0_2_1 - 0.0436454700777609*G0_2_2;
    A[1224] = -0.025443547578964*G0_0_0 + 0.013635131907701*G0_0_1 + 0.0148746893538556*G0_0_2 + 0.0136351319077008*G0_1_0 - 0.000130479731174879*G0_1_1 - 6.52398655867489e-05*G0_1_2 - 0.0254435475789641*G0_2_0 + 0.013635131907701*G0_2_1 + 0.0148746893538555*G0_2_2;
    A[1225] = -0.334680510461757*G0_0_0 - 0.250814663249556*G0_0_1 - 0.166948816037355*G0_0_2 - 0.250814663249557*G0_1_0 - 0.0622388317700808*G0_1_1 - 0.160783648739377*G0_1_2 - 0.334680510461757*G0_2_0 - 0.250814663249556*G0_2_1 - 0.166948816037355*G0_2_2;
    A[1226] = 0.0567586830607657*G0_0_0 + 0.0378717419733046*G0_0_1 + 0.0189848008858421*G0_0_2 + 0.0378717419733041*G0_1_0 + 0.012787013655069*G0_1_1 + 0.00949240044292086*G0_1_2 + 0.0567586830607658*G0_2_0 + 0.0378717419733045*G0_2_1 + 0.018984800885842*G0_2_2;
    A[1227] = 0.20941996853455*G0_0_0 + 0.173472802596064*G0_0_1 + 0.127087258163645*G0_0_2 + 0.173472802596065*G0_1_0 + 0.0846813455320394*G0_1_1 + 0.0893133759887223*G0_1_2 + 0.209419968534549*G0_2_0 + 0.173472802596064*G0_2_1 + 0.127087258163645*G0_2_2;
    A[1228] = -0.156575677409008*G0_0_0 - 0.066218463570892*G0_0_1 - 0.156575677409009*G0_0_2 - 0.0662184635708929*G0_1_0 - 0.161794866655976*G0_1_1 - 0.0662184635708927*G0_1_2 - 0.156575677409008*G0_2_0 - 0.066218463570892*G0_2_1 - 0.156575677409009*G0_2_2;
    A[1229] = 0.0391439193522516*G0_0_0 + 0.0071763852145789*G0_0_1 + 0.0391439193522519*G0_0_2 + 0.00717638521457971*G0_1_0 + 0.00130479731174148*G0_1_1 + 0.00717638521457939*G0_1_2 + 0.0391439193522514*G0_2_0 + 0.00717638521457899*G0_2_1 + 0.0391439193522519*G0_2_2;
    A[1230] = 0.185933616923197*G0_0_0 + 0.170276049182295*G0_0_1 + 0.185933616923197*G0_0_2 + 0.170276049182298*G0_1_0 + 0.18071442767623*G0_1_1 + 0.170276049182297*G0_1_2 + 0.185933616923197*G0_2_0 + 0.170276049182295*G0_2_1 + 0.185933616923198*G0_2_2;
    A[1231] = 0.0391439193522523*G0_0_0 + 0.00717638521458048*G0_0_1 + 0.0391439193522529*G0_0_2 + 0.00717638521457944*G0_1_0 + 0.00130479731174266*G0_1_1 + 0.00717638521457929*G0_1_2 + 0.0391439193522526*G0_2_0 + 0.00717638521458085*G0_2_1 + 0.039143919352253*G0_2_2;
    A[1232] = -0.0155544887532731*G0_0_0 - 0.0155544887532731*G0_0_1 - 0.0155544887532731*G0_0_2 - 0.0388177200243167*G0_1_0 - 0.0388177200243166*G0_1_1 - 0.0388177200243167*G0_1_2 - 0.0155544887532731*G0_2_0 - 0.0155544887532731*G0_2_1 - 0.0155544887532731*G0_2_2;
    A[1233] = -0.00336507226698191*G0_0_0 - 0.00336507226698194*G0_2_0;
    A[1234] = 0.0336585514536898*G0_0_1 + 0.0551276864210886*G0_1_1 + 0.0336585514536899*G0_2_1;
    A[1235] = -0.00336507226698191*G0_0_2 - 0.0033650722669819*G0_2_2;
    A[1236] = 0.0699110399631223*G0_0_1 + 0.108637424175617*G0_0_2 + 0.0124934342599272*G0_1_1 + 0.0699110399631223*G0_1_2 + 0.0699110399631223*G0_2_1 + 0.108637424175617*G0_2_2;
    A[1237] = -0.00480165410720964*G0_0_1 - 0.00960330821441908*G0_0_2 + 0.000260959462347601*G0_1_1 - 0.00480165410721002*G0_1_2 - 0.00480165410720962*G0_2_1 - 0.00960330821441907*G0_2_2;
    A[1238] = 0.000548014870931396*G0_0_1 - 0.00523223722008419*G0_0_2 + 6.52398655874122e-05*G0_1_1 + 0.000548014870931549*G0_1_2 + 0.000548014870931399*G0_2_1 - 0.00523223722008424*G0_2_2;
    A[1239] = 0.00228991928210667*G0_0_1 + 0.011169064988509*G0_0_2 + 0.0022899192821068*G0_1_2 + 0.00228991928210666*G0_2_1 + 0.011169064988509*G0_2_2;
    A[1240] = 0.00228991928210686*G0_0_0 + 0.00228991928210704*G0_0_2 + 0.00228991928210685*G0_2_0 + 0.00228991928210695*G0_2_2;
    A[1241] = 0.00178757231708598*G0_0_0 + 0.00178757231708616*G0_0_2 + 0.00178757231708591*G0_2_0 + 0.00178757231708612*G0_2_2;
    A[1242] = 0.00178757231708627*G0_0_0 + 0.00178757231708608*G0_0_2 + 0.00178757231708629*G0_2_0 + 0.00178757231708609*G0_2_2;
    A[1243] = 0.00228991928210667*G0_0_0 + 0.00228991928210666*G0_0_2 + 0.00228991928210666*G0_2_0 + 0.00228991928210666*G0_2_2;
    A[1244] = 0.0111690649885096*G0_0_0 + 0.00228991928210704*G0_0_1 + 0.00228991928210722*G0_1_0 + 0.0111690649885095*G0_2_0 + 0.00228991928210696*G0_2_1;
    A[1245] = -0.00523223722008506*G0_0_0 + 0.000548014870931042*G0_0_1 + 0.000548014870930781*G0_1_0 + 6.52398655863467e-05*G0_1_1 - 0.00523223722008488*G0_2_0 + 0.000548014870931199*G0_2_1;
    A[1246] = -0.00960330821441854*G0_0_0 - 0.0048016541072092*G0_0_1 - 0.00480165410720897*G0_1_0 + 0.000260959462348779*G0_1_1 - 0.00960330821441881*G0_2_0 - 0.00480165410720937*G0_2_1;
    A[1247] = 0.108637424175617*G0_0_0 + 0.0699110399631224*G0_0_1 + 0.0699110399631224*G0_1_0 + 0.0124934342599272*G0_1_1 + 0.108637424175617*G0_2_0 + 0.0699110399631224*G0_2_1;
    A[1248] = 0.0283728175438239*G0_0_0 + 0.0283728175438239*G0_0_1 + 0.0093945406445401*G0_0_2 - 0.0315434750113568*G0_1_0 - 0.0315434750113567*G0_1_1 + 0.0283728175438238*G0_1_2 + 0.0283728175438239*G0_2_0 + 0.0283728175438239*G0_2_1 + 0.00939454064454014*G0_2_2;
    A[1249] = -0.000482775005343908*G0_0_0 - 0.000482775005343837*G0_0_1 + 6.52398655877445e-05*G0_0_2 - 0.00626302709636026*G0_1_0 - 0.00626302709636031*G0_1_1 - 0.000482775005343987*G0_1_2 - 0.000482775005343882*G0_2_0 - 0.000482775005343811*G0_2_1 + 6.52398655876895e-05*G0_2_2;
    A[1250] = 0.0132175967679434*G0_0_0 + 0.0132175967679434*G0_0_1 + 6.52398655864602e-05*G0_0_2 + 6.52398655870784e-05*G0_1_0 + 6.52398655871169e-05*G0_1_1 + 0.0132175967679435*G0_1_2 + 0.0132175967679434*G0_2_0 + 0.0132175967679434*G0_2_1 + 6.52398655865358e-05*G0_2_2;
    A[1251] = -0.00228991928210666*G0_0_0 - 0.00228991928210663*G0_0_1 + 0.00939454064454075*G0_0_2 - 0.0022899192821065*G0_1_2 - 0.00228991928210665*G0_2_0 - 0.00228991928210662*G0_2_1 + 0.00939454064454071*G0_2_2;
    A[1252] = 0.100717304493345*G0_0_0 + 0.081739027594061*G0_0_1 + 0.100717304493345*G0_0_2 + 0.125351877739029*G0_1_0 + 0.185268170294209*G0_1_1 + 0.125351877739029*G0_1_2 + 0.100717304493345*G0_2_0 + 0.0817390275940609*G0_2_1 + 0.100717304493345*G0_2_2;
    A[1253] = -0.204187731314465*G0_0_0 - 0.167692550505048*G0_0_1 - 0.204187731314465*G0_0_2 - 0.219493003781196*G0_1_0 - 0.387185554286245*G0_1_1 - 0.219493003781196*G0_1_2 - 0.204187731314465*G0_2_0 - 0.167692550505048*G0_2_1 - 0.204187731314465*G0_2_2;
    A[1254] = 0.281001149056702*G0_0_0 + 0.202869886029606*G0_0_1 + 0.281001149056701*G0_0_2 + 0.202869886029606*G0_1_0 + 0.428965164208215*G0_1_1 + 0.202869886029606*G0_1_2 + 0.281001149056701*G0_2_0 + 0.202869886029606*G0_2_1 + 0.281001149056701*G0_2_2;
    A[1255] = -0.108637424175617*G0_0_0 - 0.135020425819035*G0_0_1 - 0.108637424175617*G0_0_2 - 0.0699110399631225*G0_1_0 - 0.243357746612952*G0_1_1 - 0.0699110399631225*G0_1_2 - 0.108637424175617*G0_2_0 - 0.135020425819035*G0_2_1 - 0.108637424175617*G0_2_2;
    A[1256] = 0.00939454064454029*G0_0_0 + 0.0283728175438242*G0_0_1 + 0.0283728175438242*G0_0_2 + 0.0283728175438237*G0_1_0 - 0.0315434750113568*G0_1_1 - 0.0315434750113569*G0_1_2 + 0.0093945406445403*G0_2_0 + 0.0283728175438242*G0_2_1 + 0.0283728175438242*G0_2_2;
    A[1257] = 6.52398655870093e-05*G0_0_0 - 0.000482775005344684*G0_0_1 - 0.000482775005344697*G0_0_2 - 0.000482775005343964*G0_1_0 - 0.00626302709636052*G0_1_1 - 0.00626302709636041*G0_1_2 + 6.5239865586997e-05*G0_2_0 - 0.000482775005344614*G0_2_1 - 0.000482775005344631*G0_2_2;
    A[1258] = 6.523986558741e-05*G0_0_0 + 0.0132175967679439*G0_0_1 + 0.0132175967679439*G0_0_2 + 0.0132175967679436*G0_1_0 + 6.52398655870797e-05*G0_1_1 + 6.52398655870195e-05*G0_1_2 + 6.52398655874015e-05*G0_2_0 + 0.0132175967679439*G0_2_1 + 0.0132175967679439*G0_2_2;
    A[1259] = 0.00939454064454034*G0_0_0 - 0.00228991928210697*G0_0_1 - 0.00228991928210698*G0_0_2 - 0.00228991928210675*G0_1_0 + 0.00939454064454038*G0_2_0 - 0.00228991928210688*G0_2_1 - 0.00228991928210689*G0_2_2;
    A[1260] = -0.00117431758056722*G0_0_0 + 6.5239865587492e-05*G0_0_1 - 0.00117431758056742*G0_0_2 + 6.5239865587368e-05*G0_1_0 + 6.52398655877239e-05*G0_1_2 - 0.00117431758056742*G0_2_0 + 6.52398655872554e-05*G0_2_1 - 0.00117431758056748*G0_2_2;
    A[1261] = 0.0035229527417022*G0_0_0 - 0.0029684138842121*G0_0_1 + 0.00352295274170277*G0_0_2 - 0.00296841388421188*G0_1_0 - 0.000652398655869983*G0_1_1 - 0.00296841388421237*G0_1_2 + 0.00352295274170219*G0_2_0 - 0.00296841388421197*G0_2_1 + 0.00352295274170276*G0_2_2;
    A[1262] = 0.124738623002509*G0_0_0 + 0.0623693115012533*G0_0_1 + 0.12473862300251*G0_0_2 + 0.0623693115012543*G0_1_0 + 0.0476251018785723*G0_1_1 + 0.0623693115012553*G0_1_2 + 0.124738623002509*G0_2_0 + 0.0623693115012533*G0_2_1 + 0.12473862300251*G0_2_2;
    A[1263] = -0.0105036183595201*G0_0_0 + 0.00257697469068985*G0_0_1 - 0.0105036183595208*G0_0_2 + 0.00257697469069048*G0_1_0 + 0.00257697469069048*G0_1_2 - 0.0105036183595199*G0_2_0 + 0.00257697469068987*G0_2_1 - 0.0105036183595208*G0_2_2;
    A[1264] = 0.00352295274170262*G0_0_0 - 0.00296841388421169*G0_0_1 + 0.0035229527417016*G0_0_2 - 0.00296841388421336*G0_1_0 - 0.000652398655870477*G0_1_1 - 0.00296841388421328*G0_1_2 + 0.00352295274170264*G0_2_0 - 0.00296841388421157*G0_2_1 + 0.00352295274170183*G0_2_2;
    A[1265] = -0.0011743175805675*G0_0_0 + 6.52398655873313e-05*G0_0_1 - 0.00117431758056626*G0_0_2 + 6.52398655874807e-05*G0_1_0 + 6.52398655875164e-05*G0_1_2 - 0.00117431758056752*G0_2_0 + 6.5239865587347e-05*G0_2_1 - 0.0011743175805662*G0_2_2;
    A[1266] = -0.051017574889101*G0_0_0 - 0.088791457064025*G0_0_1 - 0.0528442911255398*G0_0_2 - 0.00463203045668258*G0_1_0 + 0.0846813455320397*G0_1_1 - 0.0887914570640254*G0_1_2 - 0.051017574889101*G0_2_0 - 0.0887914570640251*G0_2_1 - 0.0528442911255397*G0_2_2;
    A[1267] = 0.182410664181495*G0_0_0 + 0.188575831479475*G0_0_1 + 0.104709984267275*G0_0_2 + 0.0985448169692951*G0_1_0 - 0.0622388317700814*G0_1_1 + 0.188575831479475*G0_1_2 + 0.182410664181495*G0_2_0 + 0.188575831479475*G0_2_1 + 0.104709984267275*G0_2_2;
    A[1268] = -0.124738623002509*G0_0_0 - 0.198068231922397*G0_0_1 - 0.271397840842282*G0_0_2 - 0.0623693115012543*G0_1_0 - 0.00339247301052873*G0_1_1 - 0.198068231922396*G0_1_2 - 0.124738623002509*G0_2_0 - 0.198068231922397*G0_2_1 - 0.271397840842282*G0_2_2;
    A[1269] = -0.0155923278753146*G0_0_0 - 0.0250847283182362*G0_0_1 - 0.00619778723077475*G0_0_2 + 0.00329461321214751*G0_1_0 + 0.0127870136550685*G0_1_1 - 0.0250847283182353*G0_1_2 - 0.0155923278753145*G0_2_0 - 0.0250847283182361*G0_2_1 - 0.0061977872307749*G0_2_2;
    A[1270] = -0.00352295274170271*G0_0_0 + 0.0303691574307899*G0_0_1 + 0.104709984267275*G0_0_2 + 0.00296841388421327*G0_1_0 - 0.00678494602105585*G0_1_1 + 0.0303691574307889*G0_1_2 - 0.00352295274170276*G0_2_0 + 0.0303691574307897*G0_2_1 + 0.104709984267274*G0_2_2;
    A[1271] = 0.00117431758056744*G0_0_0 - 0.0137656116388749*G0_0_1 - 0.0528442911255396*G0_0_2 - 6.52398655875198e-05*G0_1_0 - 0.000130479731174503*G0_1_1 - 0.0137656116388754*G0_1_2 + 0.00117431758056746*G0_2_0 - 0.0137656116388749*G0_2_1 - 0.0528442911255395*G0_2_2;
    A[1272] = -0.0150051690850297*G0_0_0 + 0.0227687130898942*G0_0_1 - 0.0150051690850296*G0_0_2 + 0.0227687130898935*G0_1_0 - 0.066544662898829*G0_1_1 + 0.0227687130898938*G0_1_2 - 0.0150051690850296*G0_2_0 + 0.0227687130898942*G0_2_1 - 0.0150051690850296*G0_2_2;
    A[1273] = -0.000652398655871658*G0_0_0 + 0.00884000178704938*G0_0_1 - 0.000652398655871128*G0_0_2 + 0.00884000178704992*G0_1_0 - 0.000652398655871407*G0_1_1 + 0.00884000178704991*G0_1_2 - 0.000652398655871599*G0_2_0 + 0.00884000178704939*G0_2_1 - 0.000652398655871186*G0_2_2;
    A[1274] = -0.0150051690850295*G0_0_0 - 6.52398655875743e-05*G0_0_1 - 0.01500516908503*G0_0_2 - 6.5239865586998e-05*G0_1_0 - 6.52398655863809e-05*G0_1_2 - 0.0150051690850296*G0_2_0 - 6.52398655873352e-05*G0_2_1 - 0.01500516908503*G0_2_2;
    A[1275] = -0.000652398655871188*G0_0_0 + 0.00884000178705037*G0_0_1 - 0.000652398655871392*G0_0_2 + 0.0088400017870505*G0_1_0 - 0.000652398655870508*G0_1_1 + 0.0088400017870505*G0_1_2 - 0.000652398655871318*G0_2_0 + 0.00884000178705031*G0_2_1 - 0.000652398655871492*G0_2_2;
    A[1276] = -0.000652398655870887*G0_0_0 - 0.00257697469068983*G0_0_1 - 0.000652398655870235*G0_0_2 - 0.00257697469068974*G0_1_0 - 0.00257697469068972*G0_1_2 - 0.000652398655870705*G0_2_0 - 0.00257697469068985*G0_2_1 - 0.000652398655870253*G0_2_2;
    A[1277] = -0.0150051690850298*G0_0_0 - 6.52398655873267e-05*G0_0_1 - 0.0150051690850298*G0_0_2 - 6.52398655873246e-05*G0_1_0 - 6.52398655874151e-05*G0_1_2 - 0.0150051690850298*G0_2_0 - 6.52398655873427e-05*G0_2_1 - 0.0150051690850297*G0_2_2;
    A[1278] = -0.05284429112554*G0_0_0 - 0.0887914570640253*G0_0_1 - 0.0510175748891016*G0_0_2 - 0.0887914570640254*G0_1_0 + 0.08468134553204*G0_1_1 - 0.00463203045668271*G0_1_2 - 0.0528442911255399*G0_2_0 - 0.0887914570640254*G0_2_1 - 0.0510175748891017*G0_2_2;
    A[1279] = -0.00619778723077399*G0_0_0 - 0.025084728318235*G0_0_1 - 0.0155923278753147*G0_0_2 - 0.0250847283182357*G0_1_0 + 0.0127870136550688*G0_1_1 + 0.00329461321214725*G0_1_2 - 0.00619778723077392*G0_2_0 - 0.025084728318235*G0_2_1 - 0.0155923278753146*G0_2_2;
    A[1280] = -0.0528442911255403*G0_0_0 - 0.0137656116388752*G0_0_1 + 0.0011743175805672*G0_0_2 - 0.0137656116388752*G0_1_0 - 0.000130479731173505*G0_1_1 - 6.5239865587723e-05*G0_1_2 - 0.0528442911255404*G0_2_0 - 0.0137656116388754*G0_2_1 + 0.00117431758056726*G0_2_2;
    A[1281] = 0.104709984267275*G0_0_0 + 0.188575831479475*G0_0_1 + 0.182410664181495*G0_0_2 + 0.188575831479476*G0_1_0 - 0.0622388317700817*G0_1_1 + 0.0985448169692955*G0_1_2 + 0.104709984267275*G0_2_0 + 0.188575831479475*G0_2_1 + 0.182410664181495*G0_2_2;
    A[1282] = 0.104709984267275*G0_0_0 + 0.0303691574307891*G0_0_1 - 0.00352295274170231*G0_0_2 + 0.0303691574307891*G0_1_0 - 0.0067849460210571*G0_1_1 + 0.00296841388421254*G0_1_2 + 0.104709984267275*G0_2_0 + 0.0303691574307892*G0_2_1 - 0.00352295274170231*G0_2_2;
    A[1283] = -0.271397840842283*G0_0_0 - 0.198068231922397*G0_0_1 - 0.12473862300251*G0_0_2 - 0.198068231922397*G0_1_0 - 0.00339247301052863*G0_1_1 - 0.0623693115012554*G0_1_2 - 0.271397840842283*G0_2_0 - 0.198068231922396*G0_2_1 - 0.12473862300251*G0_2_2;
    A[1284] = 0.0260959462348334*G0_0_0 + 0.0104383784939324*G0_0_1 + 0.0260959462348346*G0_0_2 + 0.0104383784939333*G0_1_0 + 0.180714427676231*G0_1_1 + 0.0104383784939331*G0_1_2 + 0.0260959462348333*G0_2_0 + 0.0104383784939322*G0_2_1 + 0.0260959462348347*G0_2_2;
    A[1285] = 0.0260959462348348*G0_0_0 - 0.00587158790283715*G0_0_1 + 0.0260959462348352*G0_0_2 - 0.00587158790283804*G0_1_0 + 0.00130479731174145*G0_1_1 - 0.0058715879028379*G0_1_2 + 0.0260959462348352*G0_2_0 - 0.00587158790283729*G0_2_1 + 0.0260959462348352*G0_2_2;
    A[1286] = -0.185933616923197*G0_0_0 - 0.0955764030850799*G0_0_1 - 0.185933616923197*G0_0_2 - 0.0955764030850831*G0_1_0 - 0.161794866655974*G0_1_1 - 0.0955764030850822*G0_1_2 - 0.185933616923197*G0_2_0 - 0.0955764030850797*G0_2_1 - 0.185933616923197*G0_2_2;
    A[1287] = 0.0260959462348348*G0_0_0 - 0.00587158790283889*G0_0_1 + 0.0260959462348334*G0_0_2 - 0.00587158790283802*G0_1_0 + 0.0013047973117405*G0_1_1 - 0.00587158790283794*G0_1_2 + 0.0260959462348344*G0_2_0 - 0.00587158790283893*G0_2_1 + 0.0260959462348333*G0_2_2;
    A[1288] = 0.00480034930989781*G0_0_0 + 0.00480034930989783*G0_0_1 + 0.00480034930989782*G0_0_2 + 0.0153313684129654*G0_1_0 + 0.0153313684129653*G0_1_1 + 0.0153313684129653*G0_1_2 + 0.00480034930989781*G0_2_0 + 0.00480034930989783*G0_2_1 + 0.00480034930989782*G0_2_2;
    A[1289] = -0.00480034930989785*G0_0_0 - 0.00480034930989782*G0_2_0;
    A[1290] = -0.0372571824394737*G0_0_1 - 0.0551276864210887*G0_1_1 - 0.0372571824394737*G0_2_1;
    A[1291] = -0.00480034930989786*G0_0_2 - 0.00480034930989786*G0_2_2;
    A[1292] = -0.069911039963122*G0_0_1 - 0.139822079926245*G0_0_2 + 0.0196371995417132*G0_1_1 - 0.0699110399631225*G0_1_2 - 0.069911039963122*G0_2_1 - 0.139822079926245*G0_2_2;
    A[1293] = 0.0387263842124947*G0_0_1 + 0.108637424175618*G0_0_2 - 0.0186912214907001*G0_1_1 + 0.0387263842124952*G0_1_2 + 0.0387263842124946*G0_2_1 + 0.108637424175618*G0_2_2;
    A[1294] = -0.0189782768992836*G0_0_1 - 0.078894569454465*G0_0_2 + 0.00939454064454025*G0_1_1 - 0.0189782768992838*G0_1_2 - 0.0189782768992835*G0_2_1 - 0.0788945694544648*G0_2_2;
    A[1295] = 0.00537576492437602*G0_0_1 + 0.0328352243499813*G0_0_2 + 0.00537576492437598*G0_1_2 + 0.00537576492437599*G0_2_1 + 0.0328352243499813*G0_2_2;
    A[1296] = 0.00537576492437586*G0_0_0 + 0.00537576492437588*G0_0_2 + 0.00537576492437581*G0_2_0 + 0.00537576492437587*G0_2_2;
    A[1297] = 0.00228991928210695*G0_0_0 + 0.00228991928210672*G0_0_2 + 0.00228991928210698*G0_2_0 + 0.00228991928210674*G0_2_2;
    A[1298] = 0.00228991928210665*G0_0_0 + 0.00228991928210683*G0_0_2 + 0.00228991928210664*G0_2_0 + 0.00228991928210681*G0_2_2;
    A[1299] = 0.00537576492437601*G0_0_0 + 0.00537576492437596*G0_0_2 + 0.00537576492437598*G0_2_0 + 0.00537576492437594*G0_2_2;
    A[1300] = 0.0328352243499807*G0_0_0 + 0.00537576492437586*G0_0_1 + 0.00537576492437565*G0_1_0 + 0.0328352243499807*G0_2_0 + 0.00537576492437586*G0_2_1;
    A[1301] = -0.0788945694544642*G0_0_0 - 0.0189782768992835*G0_0_1 - 0.0189782768992833*G0_1_0 + 0.00939454064454084*G0_1_1 - 0.0788945694544641*G0_2_0 - 0.0189782768992835*G0_2_1;
    A[1302] = 0.108637424175617*G0_0_0 + 0.0387263842124945*G0_0_1 + 0.0387263842124947*G0_1_0 - 0.0186912214907006*G0_1_1 + 0.108637424175617*G0_2_0 + 0.0387263842124946*G0_2_1;
    A[1303] = -0.139822079926245*G0_0_0 - 0.0699110399631223*G0_0_1 - 0.0699110399631225*G0_1_0 + 0.0196371995417133*G0_1_1 - 0.139822079926245*G0_2_0 - 0.0699110399631223*G0_2_1;
    A[1304] = -0.0053757649243758*G0_0_0 - 0.00537576492437585*G0_0_1 + 0.0220836945012291*G0_1_0 + 0.022083694501229*G0_1_1 - 0.00537576492437566*G0_1_2 - 0.00537576492437583*G0_2_0 - 0.00537576492437589*G0_2_1;
    A[1305] = -0.00228991928210713*G0_0_0 - 0.00228991928210704*G0_0_1 + 0.00658922642429579*G0_1_0 + 0.00658922642429585*G0_1_1 - 0.00228991928210713*G0_1_2 - 0.00228991928210714*G0_2_0 - 0.00228991928210705*G0_2_1;
    A[1306] = -0.00228991928210642*G0_0_0 - 0.00228991928210647*G0_0_1 + 0.00939454064454068*G0_1_0 + 0.00939454064454064*G0_1_1 - 0.00228991928210641*G0_1_2 - 0.00228991928210644*G0_2_0 - 0.00228991928210648*G0_2_1;
    A[1307] = -0.00537576492437601*G0_0_0 - 0.00537576492437604*G0_0_1 - 0.00537576492437618*G0_1_2 - 0.00537576492437598*G0_2_0 - 0.00537576492437601*G0_2_1;
    A[1308] = -0.0328352243499808*G0_0_0 - 0.0274594594256048*G0_0_1 - 0.0328352243499808*G0_0_2 - 0.053751125257201*G0_1_0 - 0.0812105846828057*G0_1_1 - 0.0537511252572011*G0_1_2 - 0.0328352243499809*G0_2_0 - 0.0274594594256049*G0_2_1 - 0.0328352243499809*G0_2_2;
    A[1309] = 0.0788945694544643*G0_0_0 + 0.0599162925551808*G0_0_1 + 0.0788945694544644*G0_0_2 + 0.103529142700148*G0_1_0 + 0.18526817029421*G0_1_1 + 0.103529142700149*G0_1_2 + 0.0788945694544643*G0_2_0 + 0.0599162925551808*G0_2_1 + 0.0788945694544645*G0_2_2;
    A[1310] = -0.108637424175617*G0_0_0 - 0.0699110399631225*G0_0_1 - 0.108637424175617*G0_0_2 - 0.135020425819035*G0_1_0 - 0.243357746612952*G0_1_1 - 0.135020425819035*G0_1_2 - 0.108637424175617*G0_2_0 - 0.0699110399631225*G0_2_1 - 0.108637424175617*G0_2_2;
    A[1311] = 0.139822079926245*G0_0_0 + 0.0699110399631226*G0_0_1 + 0.139822079926245*G0_0_2 + 0.0699110399631226*G0_1_0 + 0.179096479009672*G0_1_1 + 0.0699110399631226*G0_1_2 + 0.139822079926245*G0_2_0 + 0.0699110399631226*G0_2_1 + 0.139822079926245*G0_2_2;
    A[1312] = -0.00537576492437613*G0_0_1 - 0.0053757649243761*G0_0_2 - 0.00537576492437572*G0_1_0 + 0.0220836945012289*G0_1_1 + 0.022083694501229*G0_1_2 - 0.00537576492437616*G0_2_1 - 0.00537576492437612*G0_2_2;
    A[1313] = -0.00228991928210661*G0_0_1 - 0.00228991928210663*G0_0_2 - 0.00228991928210692*G0_1_0 + 0.00658922642429602*G0_1_1 + 0.00658922642429594*G0_1_2 - 0.00228991928210659*G0_2_1 - 0.00228991928210661*G0_2_2;
    A[1314] = -0.00228991928210685*G0_0_1 - 0.00228991928210686*G0_0_2 - 0.00228991928210674*G0_1_0 + 0.00939454064454051*G0_1_1 + 0.00939454064454053*G0_1_2 - 0.00228991928210691*G0_2_1 - 0.00228991928210692*G0_2_2;
    A[1315] = -0.00537576492437588*G0_0_1 - 0.00537576492437582*G0_0_2 - 0.00537576492437587*G0_1_0 - 0.00537576492437587*G0_2_1 - 0.00537576492437581*G0_2_2;
    A[1316] = -0.0118736555368497*G0_0_0 + 0.0093945406445403*G0_0_1 - 0.0118736555368502*G0_0_2 + 0.00939454064454012*G0_1_0 + 0.00939454064453983*G0_1_2 - 0.0118736555368496*G0_2_0 + 0.00939454064454033*G0_2_1 - 0.0118736555368502*G0_2_2;
    A[1317] = 0.0429604514890966*G0_0_0 + 0.000685018588664183*G0_0_1 + 0.0429604514890966*G0_0_2 + 0.000685018588663985*G0_1_0 + 0.022670853291512*G0_1_1 + 0.000685018588664335*G0_1_2 + 0.0429604514890966*G0_2_0 + 0.000685018588664234*G0_2_1 + 0.0429604514890967*G0_2_2;
    A[1318] = -0.124738623002509*G0_0_0 - 0.0284445813959691*G0_0_1 - 0.12473862300251*G0_0_2 - 0.0284445813959695*G0_1_0 - 0.0606730749959904*G0_1_1 - 0.0284445813959701*G0_1_2 - 0.12473862300251*G0_2_0 - 0.0284445813959691*G0_2_1 - 0.12473862300251*G0_2_2;
    A[1319] = -0.00254435475789676*G0_0_0 + 0.00329461321214815*G0_0_1 - 0.00254435475789605*G0_0_2 + 0.00329461321214788*G0_1_0 + 0.00329461321214788*G0_1_2 - 0.00254435475789679*G0_2_0 + 0.00329461321214817*G0_2_1 - 0.00254435475789596*G0_2_2;
    A[1320] = 0.0429604514890967*G0_0_0 + 0.000685018588664178*G0_0_1 + 0.0429604514890974*G0_0_2 + 0.00068501858866479*G0_1_0 + 0.0226708532915124*G0_1_1 + 0.000685018588664711*G0_1_2 + 0.0429604514890966*G0_2_0 + 0.00068501858866418*G0_2_1 + 0.0429604514890972*G0_2_2;
    A[1321] = -0.0118736555368501*G0_0_0 + 0.00939454064454023*G0_0_1 - 0.0118736555368508*G0_0_2 + 0.00939454064454002*G0_1_0 + 0.00939454064454009*G0_1_2 - 0.01187365553685*G0_2_0 + 0.00939454064454018*G0_2_1 - 0.0118736555368508*G0_2_2;
    A[1322] = 0.0118736555368495*G0_0_0 + 0.0212681961813903*G0_0_1 - 0.0146137298915077*G0_1_0 - 0.0693499771190738*G0_1_1 + 0.0212681961813904*G0_1_2 + 0.0118736555368495*G0_2_0 + 0.0212681961813904*G0_2_1;
    A[1323] = -0.0429604514890963*G0_0_0 - 0.0422754329004325*G0_0_1 - 0.0359145460056913*G0_1_0 + 0.0920860702761738*G0_1_1 - 0.0422754329004327*G0_1_2 - 0.0429604514890964*G0_2_0 - 0.0422754329004326*G0_2_1;
    A[1324] = 0.12473862300251*G0_0_0 + 0.0962940416065402*G0_0_1 + 0.0284445813959695*G0_1_0 - 0.0644569872000424*G0_1_1 + 0.09629404160654*G0_1_2 + 0.12473862300251*G0_2_0 + 0.0962940416065402*G0_2_1;
    A[1325] = 0.00254435475789641*G0_0_0 + 0.00583896797004456*G0_0_1 - 0.00590420783563136*G0_1_0 - 0.0318696743392921*G0_1_1 + 0.00583896797004449*G0_1_2 + 0.00254435475789631*G0_2_0 + 0.00583896797004453*G0_2_1;
    A[1326] = -0.0429604514890967*G0_0_0 - 0.0422754329004323*G0_0_1 - 0.000685018588664754*G0_1_0 + 0.0439716694056964*G0_1_1 - 0.0422754329004321*G0_1_2 - 0.0429604514890966*G0_2_0 - 0.0422754329004322*G0_2_1;
    A[1327] = 0.01187365553685*G0_0_0 + 0.0212681961813901*G0_0_1 - 0.00939454064454003*G0_1_0 - 0.018789081289081*G0_1_1 + 0.0212681961813903*G0_1_2 + 0.01187365553685*G0_2_0 + 0.02126819618139*G0_2_1;
    A[1328] = -0.00939454064454065*G0_0_1 - 0.00939454064454058*G0_1_0 + 0.0453417065830257*G0_1_1 - 0.0093945406445406*G0_1_2 - 0.00939454064454068*G0_2_1;
    A[1329] = -0.00329461321214785*G0_0_1 - 0.00329461321214759*G0_1_0 + 0.0226708532915131*G0_1_1 - 0.00329461321214766*G0_1_2 - 0.00329461321214787*G0_2_1;
    A[1330] = -0.00939454064454028*G0_0_1 - 0.00939454064454079*G0_1_0 - 0.00939454064454116*G0_1_2 - 0.00939454064454032*G0_2_1;
    A[1331] = -0.00329461321214839*G0_0_1 - 0.00329461321214808*G0_1_0 + 0.0226708532915127*G0_1_1 - 0.00329461321214799*G0_1_2 - 0.00329461321214837*G0_2_1;
    A[1332] = -0.00329461321214819*G0_0_1 - 0.00329461321214797*G0_1_0 - 0.00329461321214815*G0_1_2 - 0.00329461321214821*G0_2_1;
    A[1333] = -0.00939454064454024*G0_0_1 - 0.00939454064454076*G0_1_0 - 0.00939454064454043*G0_1_2 - 0.00939454064454018*G0_2_1;
    A[1334] = 0.0212681961813907*G0_0_1 + 0.0118736555368499*G0_0_2 + 0.0212681961813904*G0_1_0 - 0.0693499771190737*G0_1_1 - 0.0146137298915075*G0_1_2 + 0.0212681961813908*G0_2_1 + 0.01187365553685*G0_2_2;
    A[1335] = 0.0058389679700439*G0_0_1 + 0.00254435475789619*G0_0_2 + 0.00583896797004449*G0_1_0 - 0.0318696743392923*G0_1_1 - 0.00590420783563136*G0_1_2 + 0.00583896797004392*G0_2_1 + 0.00254435475789619*G0_2_2;
    A[1336] = 0.0212681961813904*G0_0_1 + 0.0118736555368501*G0_0_2 + 0.0212681961813905*G0_1_0 - 0.0187890812890813*G0_1_1 - 0.00939454064453986*G0_1_2 + 0.0212681961813905*G0_2_1 + 0.0118736555368501*G0_2_2;
    A[1337] = -0.0422754329004326*G0_0_1 - 0.0429604514890966*G0_0_2 - 0.0422754329004326*G0_1_0 + 0.092086070276174*G0_1_1 - 0.0359145460056914*G0_1_2 - 0.0422754329004327*G0_2_1 - 0.0429604514890966*G0_2_2;
    A[1338] = -0.0422754329004319*G0_0_1 - 0.0429604514890969*G0_0_2 - 0.0422754329004325*G0_1_0 + 0.0439716694056969*G0_1_1 - 0.000685018588664422*G0_1_2 - 0.042275432900432*G0_2_1 - 0.0429604514890969*G0_2_2;
    A[1339] = 0.0962940416065403*G0_0_1 + 0.12473862300251*G0_0_2 + 0.0962940416065406*G0_1_0 - 0.0644569872000426*G0_1_1 + 0.02844458139597*G0_1_2 + 0.0962940416065402*G0_2_1 + 0.12473862300251*G0_2_2;
    A[1340] = 0.00260959462348369*G0_0_1 + 0.00260959462348337*G0_1_0 - 0.151356488162043*G0_1_1 + 0.00260959462348332*G0_1_2 + 0.00260959462348374*G0_2_1;
    A[1341] = 0.00260959462348364*G0_0_1 + 0.00260959462348414*G0_1_0 - 0.0453417065830252*G0_1_1 + 0.00260959462348362*G0_1_2 + 0.0026095946234836*G0_2_1;
    A[1342] = 0.0352295274170259*G0_0_1 + 0.0352295274170272*G0_1_0 + 0.166687856575007*G0_1_1 + 0.0352295274170266*G0_1_2 + 0.0352295274170259*G0_2_1;
    A[1343] = 0.00260959462348449*G0_0_1 + 0.00260959462348351*G0_1_0 - 0.0453417065830247*G0_1_1 + 0.00260959462348413*G0_1_2 + 0.00260959462348447*G0_2_1;
    A[1344] = -0.0551276864210884*G0_0_0 - 0.0551276864210884*G0_0_1 - 0.0551276864210884*G0_0_2 - 0.0178705039816147*G0_1_0 - 0.0178705039816147*G0_1_1 - 0.0178705039816147*G0_1_2 - 0.0178705039816147*G0_2_0 - 0.0178705039816147*G0_2_1 - 0.0178705039816147*G0_2_2;
    A[1345] = 0.0153313684129654*G0_0_0 + 0.0105310191030675*G0_1_0 + 0.0105310191030675*G0_2_0;
    A[1346] = -0.00480034930989781*G0_1_1 - 0.00480034930989785*G0_2_1;
    A[1347] = -0.0048003493098978*G0_1_2 - 0.00480034930989781*G0_2_2;
    A[1348] = 0.00537576492437593*G0_1_1 + 0.00537576492437606*G0_1_2 + 0.00537576492437597*G0_2_1 + 0.00537576492437603*G0_2_2;
    A[1349] = 0.00228991928210721*G0_1_1 + 0.00228991928210695*G0_1_2 + 0.00228991928210718*G0_2_1 + 0.00228991928210698*G0_2_2;
    A[1350] = 0.00228991928210637*G0_1_1 + 0.0022899192821069*G0_1_2 + 0.0022899192821064*G0_2_1 + 0.00228991928210689*G0_2_2;
    A[1351] = 0.00537576492437622*G0_1_1 + 0.0053757649243759*G0_1_2 + 0.00537576492437615*G0_2_1 + 0.00537576492437586*G0_2_2;
    A[1352] = 0.0220836945012289*G0_0_0 + 0.0274594594256047*G0_0_2 + 0.0274594594256051*G0_1_0 + 0.0328352243499809*G0_1_2 + 0.0274594594256051*G0_2_0 + 0.0328352243499809*G0_2_2;
    A[1353] = 0.00658922642429577*G0_0_0 + 0.0088791457064026*G0_0_2 + 0.00887914570640223*G0_1_0 + 0.0111690649885093*G0_1_2 + 0.00887914570640221*G0_2_0 + 0.0111690649885093*G0_2_2;
    A[1354] = 0.00939454064454053*G0_0_0 + 0.0116844599266472*G0_0_2 + 0.0116844599266473*G0_1_0 + 0.0139743792087539*G0_1_2 + 0.0116844599266473*G0_2_0 + 0.0139743792087539*G0_2_2;
    A[1355] = 0.00537576492437613*G0_0_2 + 0.00537576492437622*G0_1_0 + 0.0107515298487521*G0_1_2 + 0.00537576492437616*G0_2_0 + 0.0107515298487521*G0_2_2;
    A[1356] = 0.0220836945012289*G0_0_0 + 0.0274594594256049*G0_0_1 + 0.0274594594256049*G0_1_0 + 0.032835224349981*G0_1_1 + 0.0274594594256049*G0_2_0 + 0.0328352243499809*G0_2_1;
    A[1357] = 0.00658922642429582*G0_0_0 + 0.00887914570640253*G0_0_1 + 0.00887914570640262*G0_1_0 + 0.0111690649885091*G0_1_1 + 0.00887914570640258*G0_2_0 + 0.0111690649885091*G0_2_1;
    A[1358] = 0.00939454064454038*G0_0_0 + 0.0116844599266472*G0_0_1 + 0.011684459926647*G0_1_0 + 0.0139743792087542*G0_1_1 + 0.011684459926647*G0_2_0 + 0.0139743792087542*G0_2_1;
    A[1359] = 0.00537576492437599*G0_0_1 + 0.00537576492437607*G0_1_0 + 0.0107515298487518*G0_1_1 + 0.00537576492437604*G0_2_0 + 0.0107515298487518*G0_2_1;
    A[1360] = 0.0196371995417131*G0_0_0 + 0.0196371995417131*G0_0_1 + 0.0895482395048356*G0_0_2 + 0.089548239504836*G0_1_0 + 0.089548239504836*G0_1_1 + 0.0196371995417128*G0_1_2 + 0.089548239504836*G0_2_0 + 0.089548239504836*G0_2_1 + 0.0196371995417128*G0_2_2;
    A[1361] = -0.0186912214907006*G0_0_0 - 0.0186912214907006*G0_0_1 - 0.0574176057031952*G0_0_2 - 0.0574176057031956*G0_1_0 - 0.0574176057031955*G0_1_1 + 0.0124934342599275*G0_1_2 - 0.0574176057031956*G0_2_0 - 0.0574176057031955*G0_2_1 + 0.0124934342599275*G0_2_2;
    A[1362] = 0.00939454064454049*G0_0_0 + 0.00939454064454049*G0_0_1 + 0.028372817543824*G0_0_2 + 0.0283728175438242*G0_1_0 + 0.0283728175438242*G0_1_1 - 0.0315434750113569*G0_1_2 + 0.0283728175438242*G0_2_0 + 0.0283728175438242*G0_2_1 - 0.0315434750113569*G0_2_2;
    A[1363] = -0.00537576492437601*G0_0_2 - 0.00537576492437623*G0_1_0 - 0.00537576492437618*G0_1_1 + 0.0220836945012289*G0_1_2 - 0.00537576492437616*G0_2_0 - 0.00537576492437611*G0_2_1 + 0.0220836945012289*G0_2_2;
    A[1364] = 0.0196371995417131*G0_0_0 + 0.0895482395048354*G0_0_1 + 0.0196371995417131*G0_0_2 + 0.0895482395048359*G0_1_0 + 0.0196371995417128*G0_1_1 + 0.0895482395048359*G0_1_2 + 0.0895482395048359*G0_2_0 + 0.0196371995417128*G0_2_1 + 0.0895482395048358*G0_2_2;
    A[1365] = -0.0186912214907002*G0_0_0 - 0.057417605703195*G0_0_1 - 0.0186912214907003*G0_0_2 - 0.0574176057031955*G0_1_0 + 0.0124934342599275*G0_1_1 - 0.0574176057031955*G0_1_2 - 0.0574176057031956*G0_2_0 + 0.0124934342599276*G0_2_1 - 0.0574176057031956*G0_2_2;
    A[1366] = 0.00939454064454029*G0_0_0 + 0.0283728175438237*G0_0_1 + 0.0093945406445403*G0_0_2 + 0.0283728175438242*G0_1_0 - 0.0315434750113568*G0_1_1 + 0.0283728175438242*G0_1_2 + 0.0283728175438242*G0_2_0 - 0.0315434750113569*G0_2_1 + 0.0283728175438242*G0_2_2;
    A[1367] = -0.00537576492437572*G0_0_1 - 0.00537576492437613*G0_1_0 + 0.0220836945012289*G0_1_1 - 0.00537576492437616*G0_1_2 - 0.0053757649243761*G0_2_0 + 0.022083694501229*G0_2_1 - 0.00537576492437612*G0_2_2;
    A[1368] = 0.179096479009671*G0_0_0 + 0.109185439046549*G0_0_1 + 0.109185439046549*G0_0_2 + 0.109185439046549*G0_1_0 + 0.179096479009672*G0_1_1 + 0.179096479009672*G0_1_2 + 0.109185439046549*G0_2_0 + 0.179096479009672*G0_2_1 + 0.179096479009672*G0_2_2;
    A[1369] = -0.243357746612952*G0_0_0 - 0.108337320793916*G0_0_1 - 0.108337320793916*G0_0_2 - 0.173446706649829*G0_1_0 - 0.147063705006412*G0_1_1 - 0.147063705006412*G0_1_2 - 0.173446706649829*G0_2_0 - 0.147063705006412*G0_2_1 - 0.147063705006412*G0_2_2;
    A[1370] = 0.185268170294209*G0_0_0 + 0.0817390275940609*G0_0_1 + 0.081739027594061*G0_0_2 + 0.125351877739029*G0_1_0 + 0.100717304493345*G0_1_1 + 0.100717304493345*G0_1_2 + 0.125351877739029*G0_2_0 + 0.100717304493345*G0_2_1 + 0.100717304493345*G0_2_2;
    A[1371] = -0.0812105846828056*G0_0_0 - 0.0274594594256048*G0_0_1 - 0.0274594594256048*G0_0_2 - 0.0537511252572008*G0_1_0 - 0.0328352243499808*G0_1_1 - 0.0328352243499808*G0_1_2 - 0.0537511252572008*G0_2_0 - 0.0328352243499808*G0_2_1 - 0.0328352243499808*G0_2_2;
    A[1372] = 0.0453417065830255*G0_0_0 + 0.0547362472275659*G0_0_1 + 0.054736247227566*G0_0_2 + 0.0547362472275653*G0_1_0 + 0.0641307878721062*G0_1_1 + 0.0641307878721061*G0_1_2 + 0.0547362472275652*G0_2_0 + 0.0641307878721061*G0_2_1 + 0.064130787872106*G0_2_2;
    A[1373] = 0.0226708532915124*G0_0_0 + 0.0259654665036607*G0_0_1 + 0.0259654665036603*G0_0_2 + 0.0259654665036602*G0_1_0 + 0.0292600797158081*G0_1_1 + 0.0292600797158081*G0_1_2 + 0.0259654665036603*G0_2_0 + 0.0292600797158081*G0_2_1 + 0.0292600797158081*G0_2_2;
    A[1374] = 0.00939454064454047*G0_0_1 + 0.00939454064454042*G0_0_2 + 0.00939454064454073*G0_1_0 + 0.018789081289081*G0_1_1 + 0.0187890812890807*G0_1_2 + 0.00939454064454068*G0_2_0 + 0.018789081289081*G0_2_1 + 0.0187890812890807*G0_2_2;
    A[1375] = 0.022670853291513*G0_0_0 + 0.0259654665036605*G0_0_1 + 0.0259654665036609*G0_0_2 + 0.0259654665036614*G0_1_0 + 0.0292600797158082*G0_1_1 + 0.0292600797158082*G0_1_2 + 0.0259654665036614*G0_2_0 + 0.0292600797158083*G0_2_1 + 0.0292600797158083*G0_2_2;
    A[1376] = 0.00329461321214802*G0_0_1 + 0.00329461321214821*G0_0_2 + 0.00329461321214846*G0_1_0 + 0.00658922642429665*G0_1_1 + 0.00658922642429651*G0_1_2 + 0.00329461321214857*G0_2_0 + 0.00658922642429675*G0_2_1 + 0.00658922642429664*G0_2_2;
    A[1377] = 0.00939454064454078*G0_0_1 + 0.0093945406445405*G0_0_2 + 0.00939454064454059*G0_1_0 + 0.0187890812890814*G0_1_1 + 0.0187890812890817*G0_1_2 + 0.00939454064454041*G0_2_0 + 0.0187890812890813*G0_2_1 + 0.0187890812890816*G0_2_2;
    A[1378] = -0.0606730749959914*G0_0_0 - 0.0322284936000212*G0_0_1 - 0.0322284936000212*G0_0_2 - 0.0322284936000212*G0_1_0 - 0.128522535206562*G0_1_1 - 0.128522535206562*G0_1_2 - 0.0322284936000212*G0_2_0 - 0.128522535206562*G0_2_1 - 0.128522535206562*G0_2_2;
    A[1379] = 0.0226708532915131*G0_0_0 + 0.0219858347028489*G0_0_1 + 0.0219858347028485*G0_0_2 + 0.0219858347028483*G0_1_0 + 0.0642612676032809*G0_1_1 + 0.0642612676032812*G0_1_2 + 0.0219858347028483*G0_2_0 + 0.0642612676032811*G0_2_1 + 0.0642612676032813*G0_2_2;
    A[1380] = -0.00939454064454099*G0_0_1 - 0.00939454064454051*G0_0_2 - 0.00939454064454073*G0_1_0 - 0.0306627368259309*G0_1_1 - 0.030662736825931*G0_1_2 - 0.00939454064454068*G0_2_0 - 0.030662736825931*G0_2_1 - 0.0306627368259311*G0_2_2;
    A[1381] = 0.0226708532915123*G0_0_0 + 0.0219858347028482*G0_0_1 + 0.0219858347028485*G0_0_2 + 0.021985834702848*G0_1_0 + 0.0642612676032809*G0_1_1 + 0.0642612676032808*G0_1_2 + 0.0219858347028481*G0_2_0 + 0.0642612676032808*G0_2_1 + 0.0642612676032807*G0_2_2;
    A[1382] = -0.00329461321214782*G0_0_1 - 0.00329461321214809*G0_0_2 - 0.00329461321214857*G0_1_0 - 0.00913358118219244*G0_1_1 - 0.00913358118219257*G0_1_2 - 0.00329461321214867*G0_2_0 - 0.00913358118219245*G0_2_1 - 0.00913358118219258*G0_2_2;
    A[1383] = -0.00939454064454038*G0_0_1 - 0.00939454064454037*G0_0_2 - 0.00939454064454065*G0_1_0 - 0.0306627368259307*G0_1_1 - 0.0306627368259305*G0_1_2 - 0.00939454064454048*G0_2_0 - 0.0306627368259307*G0_2_1 - 0.0306627368259304*G0_2_2;
    A[1384] = -0.0644569872000419*G0_0_0 - 0.0929015685960123*G0_0_1 - 0.160751028806582*G0_0_2 - 0.160751028806583*G0_1_0 - 0.0644569872000422*G0_1_1 - 0.257045070413124*G0_1_2 - 0.160751028806583*G0_2_0 - 0.064456987200042*G0_2_1 - 0.257045070413124*G0_2_2;
    A[1385] = 0.0920860702761729*G0_0_0 + 0.128000616281865*G0_0_1 + 0.134361503176605*G0_0_2 + 0.134361503176606*G0_1_0 + 0.127315597693201*G0_1_1 + 0.176636936077039*G0_1_2 + 0.134361503176606*G0_2_0 + 0.127315597693201*G0_2_1 + 0.176636936077039*G0_2_2;
    A[1386] = -0.0693499771190732*G0_0_0 - 0.0547362472275661*G0_0_1 - 0.0906181733004633*G0_0_2 - 0.0906181733004644*G0_1_0 - 0.0641307878721065*G0_1_1 - 0.111886369481855*G0_1_2 - 0.0906181733004643*G0_2_0 - 0.0641307878721064*G0_2_1 - 0.111886369481854*G0_2_2;
    A[1387] = 0.0439716694056968*G0_0_0 + 0.044656687994361*G0_0_1 + 0.0862471023061293*G0_0_2 + 0.0862471023061291*G0_1_0 + 0.0439716694056963*G0_1_1 + 0.128522535206562*G0_1_2 + 0.0862471023061292*G0_2_0 + 0.0439716694056964*G0_2_1 + 0.128522535206562*G0_2_2;
    A[1388] = -0.031869674339292*G0_0_0 - 0.0259654665036605*G0_0_1 - 0.0377086423093364*G0_0_2 - 0.0377086423093357*G0_1_0 - 0.029260079715808*G0_1_1 - 0.0435476102793806*G0_1_2 - 0.0377086423093358*G0_2_0 - 0.0292600797158081*G0_2_1 - 0.0435476102793806*G0_2_2;
    A[1389] = -0.0187890812890811*G0_0_0 - 0.00939454064454078*G0_0_1 - 0.0400572774704714*G0_0_2 - 0.0400572774704715*G0_1_0 - 0.0187890812890815*G0_1_1 - 0.0613254736518618*G0_1_2 - 0.0400572774704716*G0_2_0 - 0.0187890812890813*G0_2_1 - 0.0613254736518618*G0_2_2;
    A[1390] = -0.0644569872000418*G0_0_0 - 0.160751028806582*G0_0_1 - 0.0929015685960122*G0_0_2 - 0.160751028806583*G0_1_0 - 0.257045070413124*G0_1_1 - 0.064456987200042*G0_1_2 - 0.160751028806583*G0_2_0 - 0.257045070413124*G0_2_1 - 0.0644569872000421*G0_2_2;
    A[1391] = 0.0920860702761732*G0_0_0 + 0.134361503176606*G0_0_1 + 0.128000616281865*G0_0_2 + 0.134361503176606*G0_1_0 + 0.176636936077039*G0_1_1 + 0.127315597693201*G0_1_2 + 0.134361503176606*G0_2_0 + 0.176636936077039*G0_2_1 + 0.127315597693201*G0_2_2;
    A[1392] = -0.0693499771190734*G0_0_0 - 0.0906181733004635*G0_0_1 - 0.0547362472275663*G0_0_2 - 0.0906181733004642*G0_1_0 - 0.111886369481854*G0_1_1 - 0.0641307878721066*G0_1_2 - 0.0906181733004642*G0_2_0 - 0.111886369481854*G0_2_1 - 0.0641307878721065*G0_2_2;
    A[1393] = 0.0439716694056965*G0_0_0 + 0.0862471023061292*G0_0_1 + 0.0446566879943612*G0_0_2 + 0.0862471023061292*G0_1_0 + 0.128522535206562*G0_1_1 + 0.0439716694056962*G0_1_2 + 0.0862471023061292*G0_2_0 + 0.128522535206562*G0_2_1 + 0.0439716694056963*G0_2_2;
    A[1394] = -0.0318696743392921*G0_0_0 - 0.0377086423093363*G0_0_1 - 0.0259654665036602*G0_0_2 - 0.0377086423093363*G0_1_0 - 0.0435476102793808*G0_1_1 - 0.029260079715808*G0_1_2 - 0.0377086423093363*G0_2_0 - 0.0435476102793809*G0_2_1 - 0.029260079715808*G0_2_2;
    A[1395] = -0.0187890812890806*G0_0_0 - 0.0400572774704716*G0_0_1 - 0.00939454064454053*G0_0_2 - 0.0400572774704712*G0_1_0 - 0.0613254736518616*G0_1_1 - 0.0187890812890807*G0_1_2 - 0.0400572774704713*G0_2_0 - 0.0613254736518617*G0_2_1 - 0.0187890812890807*G0_2_2;
    A[1396] = 0.166687856575008*G0_0_0 + 0.131458329157981*G0_0_1 + 0.13145832915798*G0_0_2 + 0.13145832915798*G0_1_0 + 0.0962288017409534*G0_1_1 + 0.0962288017409534*G0_1_2 + 0.13145832915798*G0_2_0 + 0.0962288017409532*G0_2_1 + 0.0962288017409536*G0_2_2;
    A[1397] = -0.151356488162042*G0_0_0 - 0.153966082785525*G0_0_1 - 0.153966082785525*G0_0_2 - 0.153966082785524*G0_1_0 - 0.156575677409008*G0_1_1 - 0.156575677409008*G0_1_2 - 0.153966082785524*G0_2_0 - 0.156575677409008*G0_2_1 - 0.156575677409008*G0_2_2;
    A[1398] = -0.0453417065830256*G0_0_0 - 0.047951301206509*G0_0_1 - 0.0479513012065095*G0_0_2 - 0.0479513012065087*G0_1_0 - 0.0505608958299924*G0_1_1 - 0.050560895829993*G0_1_2 - 0.0479513012065088*G0_2_0 - 0.0505608958299923*G0_2_1 - 0.0505608958299932*G0_2_2;
    A[1399] = -0.0453417065830253*G0_0_0 - 0.0479513012065091*G0_0_1 - 0.0479513012065088*G0_0_2 - 0.0479513012065093*G0_1_0 - 0.050560895829993*G0_1_1 - 0.0505608958299934*G0_1_2 - 0.0479513012065095*G0_2_0 - 0.0505608958299932*G0_2_1 - 0.0505608958299933*G0_2_2;
    A[1400] = 0.0551276864210883*G0_0_0 + 0.0551276864210883*G0_0_1 + 0.0551276864210883*G0_0_2 + 0.0214691349673984*G0_1_0 + 0.0214691349673984*G0_1_1 + 0.0214691349673984*G0_1_2 + 0.0214691349673984*G0_2_0 + 0.0214691349673984*G0_2_1 + 0.0214691349673984*G0_2_2;
    A[1401] = -0.0388177200243168*G0_0_0 - 0.0232632312710434*G0_1_0 - 0.0232632312710435*G0_2_0;
    A[1402] = -0.00336507226698204*G0_1_1 - 0.003365072266982*G0_2_1;
    A[1403] = -0.00336507226698199*G0_1_2 - 0.00336507226698197*G0_2_2;
    A[1404] = 0.00228991928210655*G0_1_1 + 0.00228991928210661*G0_1_2 + 0.00228991928210649*G0_2_1 + 0.00228991928210662*G0_2_2;
    A[1405] = 0.00178757231708625*G0_1_1 + 0.00178757231708599*G0_1_2 + 0.00178757231708628*G0_2_1 + 0.00178757231708594*G0_2_2;
    A[1406] = 0.0017875723170862*G0_1_1 + 0.00178757231708644*G0_1_2 + 0.00178757231708612*G0_2_1 + 0.00178757231708643*G0_2_2;
    A[1407] = 0.00228991928210666*G0_1_1 + 0.00228991928210661*G0_1_2 + 0.00228991928210675*G0_2_1 + 0.00228991928210664*G0_2_2;
    A[1408] = -0.0315434750113565*G0_0_0 - 0.0599162925551806*G0_0_2 - 0.0599162925551808*G0_1_0 - 0.0788945694544644*G0_1_2 - 0.0599162925551808*G0_2_0 - 0.0788945694544644*G0_2_2;
    A[1409] = -0.00626302709636026*G0_0_0 - 0.00578025209101572*G0_0_2 - 0.00578025209101569*G0_1_0 - 0.00523223722008425*G0_1_2 - 0.00578025209101567*G0_2_0 - 0.00523223722008428*G0_2_2;
    A[1410] = 6.52398655871073e-05*G0_0_0 - 0.0131523569023568*G0_0_2 - 0.0131523569023568*G0_1_0 - 0.0263047138047135*G0_1_2 - 0.0131523569023568*G0_2_0 - 0.0263047138047135*G0_2_2;
    A[1411] = 0.00228991928210676*G0_0_2 + 0.00228991928210664*G0_1_0 + 0.013974379208754*G0_1_2 + 0.00228991928210672*G0_2_0 + 0.013974379208754*G0_2_2;
    A[1412] = -0.0315434750113564*G0_0_0 - 0.0599162925551807*G0_0_1 - 0.0599162925551806*G0_1_0 - 0.0788945694544645*G0_1_1 - 0.0599162925551806*G0_2_0 - 0.0788945694544645*G0_2_1;
    A[1413] = -0.00626302709636057*G0_0_0 - 0.00578025209101585*G0_0_1 - 0.005780252091016*G0_1_0 - 0.00523223722008418*G0_1_1 - 0.00578025209101596*G0_2_0 - 0.00523223722008415*G0_2_1;
    A[1414] = 6.52398655875029e-05*G0_0_0 - 0.0131523569023568*G0_0_1 - 0.0131523569023565*G0_1_0 - 0.0263047138047136*G0_1_1 - 0.0131523569023565*G0_2_0 - 0.0263047138047135*G0_2_1;
    A[1415] = 0.00228991928210664*G0_0_1 + 0.00228991928210665*G0_1_0 + 0.0139743792087542*G0_1_1 + 0.00228991928210666*G0_2_0 + 0.0139743792087541*G0_2_1;
    A[1416] = 0.0124934342599273*G0_0_0 + 0.0124934342599272*G0_0_1 - 0.0574176057031948*G0_0_2 - 0.0574176057031956*G0_1_0 - 0.0574176057031957*G0_1_1 - 0.0186912214907001*G0_1_2 - 0.0574176057031956*G0_2_0 - 0.0574176057031957*G0_2_1 - 0.0186912214907001*G0_2_2;
    A[1417] = 0.000260959462348424*G0_0_0 + 0.000260959462348397*G0_0_1 + 0.00506261356955749*G0_0_2 + 0.00506261356955825*G0_1_0 + 0.00506261356955823*G0_1_1 + 0.000260959462348211*G0_1_2 + 0.0050626135695582*G0_2_0 + 0.00506261356955818*G0_2_1 + 0.000260959462348218*G0_2_2;
    A[1418] = 6.52398655872093e-05*G0_0_0 + 6.52398655872013e-05*G0_0_1 - 0.000482775005344054*G0_0_2 - 0.000482775005344597*G0_1_0 - 0.000482775005344587*G0_1_1 - 0.00626302709636027*G0_1_2 - 0.000482775005344534*G0_2_0 - 0.000482775005344523*G0_2_1 - 0.00626302709636025*G0_2_2;
    A[1419] = -0.00228991928210683*G0_0_2 - 0.00228991928210666*G0_1_0 - 0.00228991928210667*G0_1_1 + 0.00658922642429592*G0_1_2 - 0.00228991928210675*G0_2_0 - 0.00228991928210676*G0_2_1 + 0.00658922642429589*G0_2_2;
    A[1420] = 0.0124934342599272*G0_0_0 - 0.0574176057031948*G0_0_1 + 0.0124934342599272*G0_0_2 - 0.0574176057031958*G0_1_0 - 0.0186912214907002*G0_1_1 - 0.0574176057031957*G0_1_2 - 0.0574176057031957*G0_2_0 - 0.0186912214907002*G0_2_1 - 0.0574176057031957*G0_2_2;
    A[1421] = 0.000260959462348446*G0_0_0 + 0.00506261356955745*G0_0_1 + 0.00026095946234851*G0_0_2 + 0.00506261356955835*G0_1_0 + 0.000260959462348345*G0_1_1 + 0.00506261356955827*G0_1_2 + 0.00506261356955834*G0_2_0 + 0.000260959462348255*G0_2_1 + 0.00506261356955826*G0_2_2;
    A[1422] = 6.52398655870086e-05*G0_0_0 - 0.000482775005343964*G0_0_1 + 6.52398655869985e-05*G0_0_2 - 0.000482775005344682*G0_1_0 - 0.00626302709636052*G0_1_1 - 0.000482775005344613*G0_1_2 - 0.000482775005344697*G0_2_0 - 0.00626302709636041*G0_2_1 - 0.000482775005344631*G0_2_2;
    A[1423] = -0.00228991928210692*G0_0_1 - 0.00228991928210661*G0_1_0 + 0.00658922642429602*G0_1_1 - 0.00228991928210659*G0_1_2 - 0.00228991928210663*G0_2_0 + 0.00658922642429594*G0_2_1 - 0.00228991928210661*G0_2_2;
    A[1424] = -0.243357746612952*G0_0_0 - 0.173446706649829*G0_0_1 - 0.173446706649829*G0_0_2 - 0.108337320793916*G0_1_0 - 0.147063705006412*G0_1_1 - 0.147063705006412*G0_1_2 - 0.108337320793916*G0_2_0 - 0.147063705006412*G0_2_1 - 0.147063705006412*G0_2_2;
    A[1425] = 0.428965164208214*G0_0_0 + 0.226095278178608*G0_0_1 + 0.226095278178609*G0_0_2 + 0.226095278178608*G0_1_0 + 0.304226541205705*G0_1_1 + 0.304226541205705*G0_1_2 + 0.226095278178609*G0_2_0 + 0.304226541205705*G0_2_1 + 0.304226541205705*G0_2_2;
    A[1426] = -0.387185554286244*G0_0_0 - 0.167692550505048*G0_0_1 - 0.167692550505048*G0_0_2 - 0.219493003781196*G0_1_0 - 0.204187731314465*G0_1_1 - 0.204187731314465*G0_1_2 - 0.219493003781196*G0_2_0 - 0.204187731314465*G0_2_1 - 0.204187731314465*G0_2_2;
    A[1427] = 0.18526817029421*G0_0_0 + 0.0599162925551807*G0_0_1 + 0.0599162925551807*G0_0_2 + 0.103529142700148*G0_1_0 + 0.0788945694544644*G0_1_1 + 0.0788945694544643*G0_1_2 + 0.103529142700148*G0_2_0 + 0.0788945694544644*G0_2_1 + 0.0788945694544643*G0_2_2;
    A[1428] = -0.0665446628988284*G0_0_0 - 0.0893133759887217*G0_0_1 - 0.089313375988722*G0_0_2 - 0.0893133759887213*G0_1_0 - 0.127087258163645*G0_1_1 - 0.127087258163645*G0_1_2 - 0.0893133759887213*G0_2_0 - 0.127087258163645*G0_2_1 - 0.127087258163645*G0_2_2;
    A[1429] = -0.000652398655870627*G0_0_0 - 0.0094924004429211*G0_0_1 - 0.00949240044292079*G0_0_2 - 0.00949240044292085*G0_1_0 - 0.0189848008858419*G0_1_1 - 0.0189848008858421*G0_1_2 - 0.00949240044292081*G0_2_0 - 0.0189848008858419*G0_2_1 - 0.018984800885842*G0_2_2;
    A[1430] = 6.52398655871768e-05*G0_0_1 + 6.52398655870528e-05*G0_0_2 + 6.52398655870471e-05*G0_1_0 - 0.0148746893538557*G0_1_1 - 0.0148746893538553*G0_1_2 + 6.52398655870095e-05*G0_2_0 - 0.0148746893538556*G0_2_1 - 0.0148746893538553*G0_2_2;
    A[1431] = -0.000652398655871045*G0_0_0 - 0.00949240044292083*G0_0_1 - 0.00949240044292106*G0_0_2 - 0.00949240044292143*G0_1_0 - 0.0189848008858423*G0_1_1 - 0.0189848008858421*G0_1_2 - 0.0094924004429214*G0_2_0 - 0.0189848008858423*G0_2_1 - 0.0189848008858421*G0_2_2;
    A[1432] = 0.00257697469068943*G0_0_1 + 0.0025769746906894*G0_0_2 + 0.00257697469068997*G0_1_0 + 0.00450155072550873*G0_1_1 + 0.00450155072550862*G0_1_2 + 0.00257697469068985*G0_2_0 + 0.00450155072550861*G0_2_1 + 0.00450155072550854*G0_2_2;
    A[1433] = 6.52398655871023e-05*G0_0_1 + 6.52398655870878e-05*G0_0_2 + 6.52398655870967e-05*G0_1_0 - 0.0148746893538558*G0_1_1 - 0.0148746893538559*G0_1_2 + 6.52398655871974e-05*G0_2_0 - 0.0148746893538556*G0_2_1 - 0.0148746893538558*G0_2_2;
    A[1434] = 0.0476251018785743*G0_0_0 - 0.0147442096226813*G0_0_1 - 0.0147442096226814*G0_0_2 - 0.0147442096226818*G0_1_0 + 0.047625101878574*G0_1_1 + 0.0476251018785742*G0_1_2 - 0.0147442096226817*G0_2_0 + 0.0476251018785741*G0_2_1 + 0.0476251018785741*G0_2_2;
    A[1435] = -0.000652398655871199*G0_0_0 + 0.00231601522834077*G0_0_1 + 0.00231601522834138*G0_0_2 + 0.00231601522834147*G0_1_0 + 0.00880738185425661*G0_1_1 + 0.00880738185425616*G0_1_2 + 0.00231601522834145*G0_2_0 + 0.00880738185425646*G0_2_1 + 0.00880738185425615*G0_2_2;
    A[1436] = -6.52398655864318e-05*G0_0_1 - 6.5239865587018e-05*G0_0_2 - 6.52398655870775e-05*G0_1_0 - 0.00130479731174158*G0_1_1 - 0.00130479731174142*G0_1_2 - 6.523986558704e-05*G0_2_0 - 0.00130479731174148*G0_2_1 - 0.00130479731174136*G0_2_2;
    A[1437] = -0.000652398655870426*G0_0_0 + 0.00231601522834171*G0_0_1 + 0.00231601522834131*G0_0_2 + 0.00231601522834161*G0_1_0 + 0.0088073818542564*G0_1_1 + 0.00880738185425657*G0_1_2 + 0.00231601522834155*G0_2_0 + 0.00880738185425635*G0_2_1 + 0.00880738185425656*G0_2_2;
    A[1438] = -0.0025769746906901*G0_0_1 - 0.00257697469068979*G0_0_2 - 0.00257697469068986*G0_1_0 - 0.0156575677409009*G0_1_1 - 0.0156575677409009*G0_1_2 - 0.00257697469068974*G0_2_0 - 0.0156575677409009*G0_2_1 - 0.0156575677409009*G0_2_2;
    A[1439] = -6.52398655871701e-05*G0_0_1 - 6.52398655872695e-05*G0_0_2 - 6.52398655870363e-05*G0_1_0 - 0.0013047973117415*G0_1_1 - 0.00130479731174177*G0_1_2 - 6.52398655871378e-05*G0_2_0 - 0.00130479731174149*G0_2_1 - 0.0013047973117418*G0_2_2;
    A[1440] = -0.00339247301052869*G0_0_0 + 0.0589768384907272*G0_0_1 + 0.194675758911868*G0_0_2 + 0.194675758911869*G0_1_0 + 0.132306447410613*G0_1_1 + 0.121346149991982*G0_1_2 + 0.194675758911869*G0_2_0 + 0.132306447410613*G0_2_1 + 0.121346149991982*G0_2_2;
    A[1441] = -0.0622388317700807*G0_0_0 - 0.160783648739376*G0_0_1 - 0.250814663249556*G0_0_2 - 0.250814663249557*G0_1_0 - 0.166948816037356*G0_1_1 - 0.334680510461758*G0_1_2 - 0.250814663249557*G0_2_0 - 0.166948816037356*G0_2_1 - 0.334680510461758*G0_2_2;
    A[1442] = 0.0846813455320387*G0_0_0 + 0.089313375988722*G0_0_1 + 0.173472802596064*G0_0_2 + 0.173472802596065*G0_1_0 + 0.127087258163645*G0_1_1 + 0.20941996853455*G0_1_2 + 0.173472802596065*G0_2_0 + 0.127087258163645*G0_2_1 + 0.20941996853455*G0_2_2;
    A[1443] = -0.00678494602105722*G0_0_0 - 0.00975335990526936*G0_0_1 - 0.0371541034518463*G0_0_2 - 0.0371541034518463*G0_1_0 - 0.043645470077761*G0_1_1 + 0.0371867233846399*G0_1_2 - 0.0371541034518463*G0_2_0 - 0.0436454700777611*G0_2_1 + 0.03718672338464*G0_2_2;
    A[1444] = 0.012787013655069*G0_0_0 + 0.00949240044292077*G0_0_1 + 0.0378717419733044*G0_0_2 + 0.0378717419733037*G0_1_0 + 0.018984800885842*G0_1_1 + 0.0567586830607662*G0_1_2 + 0.0378717419733038*G0_2_0 + 0.018984800885842*G0_2_1 + 0.0567586830607661*G0_2_2;
    A[1445] = -0.000130479731174283*G0_0_0 - 6.52398655870828e-05*G0_0_1 + 0.013635131907701*G0_0_2 + 0.0136351319077014*G0_1_0 + 0.0148746893538558*G0_1_1 - 0.0254435475789644*G0_1_2 + 0.0136351319077013*G0_2_0 + 0.0148746893538556*G0_2_1 - 0.0254435475789644*G0_2_2;
    A[1446] = -0.00339247301052861*G0_0_0 + 0.194675758911868*G0_0_1 + 0.0589768384907274*G0_0_2 + 0.194675758911868*G0_1_0 + 0.121346149991982*G0_1_1 + 0.132306447410613*G0_1_2 + 0.194675758911868*G0_2_0 + 0.121346149991982*G0_2_1 + 0.132306447410613*G0_2_2;
    A[1447] = -0.0622388317700808*G0_0_0 - 0.250814663249556*G0_0_1 - 0.160783648739376*G0_0_2 - 0.250814663249557*G0_1_0 - 0.334680510461758*G0_1_1 - 0.166948816037357*G0_1_2 - 0.250814663249557*G0_2_0 - 0.334680510461758*G0_2_1 - 0.166948816037356*G0_2_2;
    A[1448] = 0.0846813455320388*G0_0_0 + 0.173472802596064*G0_0_1 + 0.0893133759887225*G0_0_2 + 0.173472802596065*G0_1_0 + 0.20941996853455*G0_1_1 + 0.127087258163645*G0_1_2 + 0.173472802596065*G0_2_0 + 0.20941996853455*G0_2_1 + 0.127087258163645*G0_2_2;
    A[1449] = -0.00678494602105701*G0_0_0 - 0.0371541034518466*G0_0_1 - 0.00975335990526975*G0_0_2 - 0.0371541034518464*G0_1_0 + 0.03718672338464*G0_1_1 - 0.0436454700777612*G0_1_2 - 0.0371541034518463*G0_2_0 + 0.0371867233846398*G0_2_1 - 0.0436454700777611*G0_2_2;
    A[1450] = 0.0127870136550692*G0_0_0 + 0.0378717419733042*G0_0_1 + 0.0094924004429207*G0_0_2 + 0.0378717419733041*G0_1_0 + 0.0567586830607663*G0_1_1 + 0.018984800885842*G0_1_2 + 0.037871741973304*G0_2_0 + 0.0567586830607661*G0_2_1 + 0.0189848008858419*G0_2_2;
    A[1451] = -0.000130479731174546*G0_0_0 + 0.0136351319077016*G0_0_1 - 6.52398655869473e-05*G0_0_2 + 0.0136351319077012*G0_1_0 - 0.0254435475789642*G0_1_1 + 0.0148746893538555*G0_1_2 + 0.0136351319077013*G0_2_0 - 0.0254435475789641*G0_2_1 + 0.0148746893538555*G0_2_2;
    A[1452] = -0.161794866655976*G0_0_0 - 0.0662184635708935*G0_0_1 - 0.0662184635708933*G0_0_2 - 0.0662184635708939*G0_1_0 - 0.15657567740901*G0_1_1 - 0.156575677409009*G0_1_2 - 0.0662184635708939*G0_2_0 - 0.15657567740901*G0_2_1 - 0.15657567740901*G0_2_2;
    A[1453] = 0.18071442767623*G0_0_0 + 0.170276049182297*G0_0_1 + 0.170276049182296*G0_0_2 + 0.170276049182297*G0_1_0 + 0.185933616923198*G0_1_1 + 0.185933616923198*G0_1_2 + 0.170276049182297*G0_2_0 + 0.185933616923198*G0_2_1 + 0.185933616923198*G0_2_2;
    A[1454] = 0.00130479731174195*G0_0_0 + 0.00717638521457943*G0_0_1 + 0.00717638521458054*G0_0_2 + 0.00717638521457962*G0_1_0 + 0.0391439193522531*G0_1_1 + 0.0391439193522529*G0_1_2 + 0.0071763852145796*G0_2_0 + 0.039143919352253*G0_2_1 + 0.0391439193522529*G0_2_2;
    A[1455] = 0.00130479731174142*G0_0_0 + 0.00717638521457993*G0_0_1 + 0.00717638521457916*G0_0_2 + 0.00717638521457974*G0_1_0 + 0.0391439193522525*G0_1_1 + 0.039143919352253*G0_1_2 + 0.00717638521457977*G0_2_0 + 0.0391439193522527*G0_2_1 + 0.039143919352253*G0_2_2;
    A[1456] = -0.0388177200243166*G0_0_0 - 0.0388177200243166*G0_0_1 - 0.0388177200243166*G0_0_2 - 0.0155544887532731*G0_1_0 - 0.0155544887532731*G0_1_1 - 0.0155544887532731*G0_1_2 - 0.0155544887532731*G0_2_0 - 0.0155544887532731*G0_2_1 - 0.0155544887532731*G0_2_2;
    A[1457] = 0.0551276864210887*G0_0_0 + 0.0336585514536898*G0_1_0 + 0.0336585514536898*G0_2_0;
    A[1458] = -0.00336507226698195*G0_1_1 - 0.00336507226698194*G0_2_1;
    A[1459] = -0.00336507226698197*G0_1_2 - 0.00336507226698196*G0_2_2;
    A[1460] = 0.00228991928210687*G0_1_1 + 0.00228991928210689*G0_1_2 + 0.00228991928210684*G0_2_1 + 0.0022899192821069*G0_2_2;
    A[1461] = 0.00178757231708605*G0_1_1 + 0.00178757231708598*G0_1_2 + 0.00178757231708613*G0_2_1 + 0.00178757231708599*G0_2_2;
    A[1462] = 0.00178757231708616*G0_1_1 + 0.00178757231708639*G0_1_2 + 0.0017875723170861*G0_2_1 + 0.00178757231708639*G0_2_2;
    A[1463] = 0.00228991928210675*G0_1_1 + 0.00228991928210661*G0_1_2 + 0.00228991928210677*G0_2_1 + 0.0022899192821066*G0_2_2;
    A[1464] = 0.0124934342599271*G0_0_0 + 0.0699110399631224*G0_0_2 + 0.0699110399631227*G0_1_0 + 0.108637424175617*G0_1_2 + 0.0699110399631226*G0_2_0 + 0.108637424175617*G0_2_2;
    A[1465] = 0.000260959462348169*G0_0_0 - 0.00480165410720981*G0_0_2 - 0.0048016541072099*G0_1_0 - 0.00960330821441905*G0_1_2 - 0.00480165410720983*G0_2_0 - 0.00960330821441892*G0_2_2;
    A[1466] = 6.52398655870681e-05*G0_0_0 + 0.000548014870931453*G0_0_2 + 0.000548014870931593*G0_1_0 - 0.00523223722008478*G0_1_2 + 0.000548014870931519*G0_2_0 - 0.00523223722008489*G0_2_2;
    A[1467] = 0.00228991928210677*G0_0_2 + 0.00228991928210677*G0_1_0 + 0.0111690649885095*G0_1_2 + 0.00228991928210679*G0_2_0 + 0.0111690649885095*G0_2_2;
    A[1468] = 0.0124934342599271*G0_0_0 + 0.0699110399631226*G0_0_1 + 0.0699110399631226*G0_1_0 + 0.108637424175617*G0_1_1 + 0.0699110399631225*G0_2_0 + 0.108637424175617*G0_2_1;
    A[1469] = 0.000260959462348415*G0_0_0 - 0.00480165410720982*G0_0_1 - 0.00480165410720972*G0_1_0 - 0.00960330821441934*G0_1_1 - 0.00480165410720971*G0_2_0 - 0.00960330821441933*G0_2_1;
    A[1470] = 6.52398655867894e-05*G0_0_0 + 0.000548014870931649*G0_0_1 + 0.000548014870931483*G0_1_0 - 0.0052322372200843*G0_1_1 + 0.00054801487093146*G0_2_0 - 0.00523223722008432*G0_2_1;
    A[1471] = 0.0022899192821068*G0_0_1 + 0.00228991928210685*G0_1_0 + 0.0111690649885094*G0_1_1 + 0.00228991928210687*G0_2_0 + 0.0111690649885094*G0_2_1;
    A[1472] = -0.0315434750113566*G0_0_0 - 0.0315434750113566*G0_0_1 + 0.0283728175438238*G0_0_2 + 0.0283728175438244*G0_1_0 + 0.0283728175438243*G0_1_1 + 0.00939454064454022*G0_1_2 + 0.0283728175438243*G0_2_0 + 0.0283728175438242*G0_2_1 + 0.0093945406445402*G0_2_2;
    A[1473] = -0.00626302709636053*G0_0_0 - 0.00626302709636054*G0_0_1 - 0.000482775005344217*G0_0_2 - 0.000482775005344565*G0_1_0 - 0.000482775005344437*G0_1_1 + 6.52398655871233e-05*G0_1_2 - 0.000482775005344508*G0_2_0 - 0.00048277500534438*G0_2_1 + 6.52398655871639e-05*G0_2_2;
    A[1474] = 6.52398655869487e-05*G0_0_0 + 6.52398655869789e-05*G0_0_1 + 0.0132175967679437*G0_0_2 + 0.0132175967679438*G0_1_0 + 0.0132175967679438*G0_1_1 + 6.52398655871079e-05*G0_1_2 + 0.0132175967679437*G0_2_0 + 0.0132175967679437*G0_2_1 + 6.52398655870792e-05*G0_2_2;
    A[1475] = -0.00228991928210672*G0_0_2 - 0.00228991928210673*G0_1_0 - 0.00228991928210671*G0_1_1 + 0.00939454064454044*G0_1_2 - 0.00228991928210675*G0_2_0 - 0.00228991928210673*G0_2_1 + 0.00939454064454044*G0_2_2;
    A[1476] = -0.0315434750113565*G0_0_0 + 0.0283728175438238*G0_0_1 - 0.0315434750113565*G0_0_2 + 0.0283728175438243*G0_1_0 + 0.0093945406445402*G0_1_1 + 0.0283728175438243*G0_1_2 + 0.0283728175438242*G0_2_0 + 0.00939454064454018*G0_2_1 + 0.0283728175438242*G0_2_2;
    A[1477] = -0.00626302709636054*G0_0_0 - 0.000482775005344118*G0_0_1 - 0.00626302709636061*G0_0_2 - 0.000482775005344434*G0_1_0 + 6.52398655872716e-05*G0_1_1 - 0.000482775005344428*G0_1_2 - 0.000482775005344465*G0_2_0 + 6.52398655873376e-05*G0_2_1 - 0.000482775005344459*G0_2_2;
    A[1478] = 6.5239865587414e-05*G0_0_0 + 0.0132175967679436*G0_0_1 + 6.52398655874007e-05*G0_0_2 + 0.0132175967679439*G0_1_0 + 6.523986558708e-05*G0_1_1 + 0.0132175967679439*G0_1_2 + 0.0132175967679439*G0_2_0 + 6.52398655870196e-05*G0_2_1 + 0.0132175967679439*G0_2_2;
    A[1479] = -0.00228991928210675*G0_0_1 - 0.00228991928210685*G0_1_0 + 0.00939454064454051*G0_1_1 - 0.00228991928210691*G0_1_2 - 0.00228991928210686*G0_2_0 + 0.00939454064454053*G0_2_1 - 0.00228991928210692*G0_2_2;
    A[1480] = 0.185268170294209*G0_0_0 + 0.125351877739029*G0_0_1 + 0.125351877739029*G0_0_2 + 0.0817390275940609*G0_1_0 + 0.100717304493345*G0_1_1 + 0.100717304493345*G0_1_2 + 0.081739027594061*G0_2_0 + 0.100717304493345*G0_2_1 + 0.100717304493345*G0_2_2;
    A[1481] = -0.387185554286244*G0_0_0 - 0.219493003781196*G0_0_1 - 0.219493003781196*G0_0_2 - 0.167692550505048*G0_1_0 - 0.204187731314465*G0_1_1 - 0.204187731314465*G0_1_2 - 0.167692550505048*G0_2_0 - 0.204187731314465*G0_2_1 - 0.204187731314465*G0_2_2;
    A[1482] = 0.428965164208214*G0_0_0 + 0.202869886029606*G0_0_1 + 0.202869886029606*G0_0_2 + 0.202869886029606*G0_1_0 + 0.281001149056702*G0_1_1 + 0.281001149056702*G0_1_2 + 0.202869886029606*G0_2_0 + 0.281001149056702*G0_2_1 + 0.281001149056702*G0_2_2;
    A[1483] = -0.243357746612952*G0_0_0 - 0.0699110399631225*G0_0_1 - 0.0699110399631226*G0_0_2 - 0.135020425819035*G0_1_0 - 0.108637424175617*G0_1_1 - 0.108637424175617*G0_1_2 - 0.135020425819035*G0_2_0 - 0.108637424175617*G0_2_1 - 0.108637424175617*G0_2_2;
    A[1484] = 0.0476251018785731*G0_0_0 + 0.0623693115012549*G0_0_1 + 0.062369311501255*G0_0_2 + 0.0623693115012546*G0_1_0 + 0.12473862300251*G0_1_1 + 0.12473862300251*G0_1_2 + 0.0623693115012545*G0_2_0 + 0.12473862300251*G0_2_1 + 0.12473862300251*G0_2_2;
    A[1485] = -0.000652398655870607*G0_0_0 - 0.00296841388421235*G0_0_1 - 0.00296841388421261*G0_0_2 - 0.00296841388421233*G0_1_0 + 0.00352295274170266*G0_1_1 + 0.0035229527417026*G0_1_2 - 0.00296841388421231*G0_2_0 + 0.00352295274170263*G0_2_1 + 0.00352295274170257*G0_2_2;
    A[1486] = 6.52398655868777e-05*G0_0_1 + 6.52398655869362e-05*G0_0_2 + 6.52398655867471e-05*G0_1_0 - 0.00117431758056806*G0_1_1 - 0.00117431758056787*G0_1_2 + 6.52398655868334e-05*G0_2_0 - 0.00117431758056793*G0_2_1 - 0.00117431758056782*G0_2_2;
    A[1487] = -0.000652398655871013*G0_0_0 - 0.00296841388421324*G0_0_1 - 0.00296841388421277*G0_0_2 - 0.00296841388421222*G0_1_0 + 0.00352295274170233*G0_1_1 + 0.00352295274170277*G0_1_2 - 0.00296841388421228*G0_2_0 + 0.00352295274170233*G0_2_1 + 0.00352295274170283*G0_2_2;
    A[1488] = 0.0025769746906907*G0_0_1 + 0.00257697469069072*G0_0_2 + 0.00257697469069013*G0_1_0 - 0.0105036183595202*G0_1_1 - 0.0105036183595207*G0_1_2 + 0.0025769746906901*G0_2_0 - 0.0105036183595203*G0_2_1 - 0.0105036183595207*G0_2_2;
    A[1489] = 6.52398655869304e-05*G0_0_1 + 6.52398655869288e-05*G0_0_2 + 6.52398655869559e-05*G0_1_0 - 0.00117431758056774*G0_1_1 - 0.00117431758056754*G0_1_2 + 6.52398655869568e-05*G0_2_0 - 0.00117431758056776*G0_2_1 - 0.0011743175805676*G0_2_2;
    A[1490] = -0.0665446628988294*G0_0_0 + 0.0227687130898929*G0_0_1 + 0.0227687130898931*G0_0_2 + 0.0227687130898939*G0_1_0 - 0.0150051690850299*G0_1_1 - 0.01500516908503*G0_1_2 + 0.0227687130898938*G0_2_0 - 0.01500516908503*G0_2_1 - 0.0150051690850299*G0_2_2;
    A[1491] = -0.000652398655870483*G0_0_0 + 0.00884000178705102*G0_0_1 + 0.00884000178705049*G0_0_2 + 0.0088400017870505*G0_1_0 - 0.000652398655870916*G0_1_1 - 0.000652398655870801*G0_1_2 + 0.00884000178705047*G0_2_0 - 0.000652398655870853*G0_2_1 - 0.000652398655870727*G0_2_2;
    A[1492] = -6.5239865587568e-05*G0_0_1 - 6.52398655872416e-05*G0_0_2 - 6.52398655867435e-05*G0_1_0 - 0.0150051690850305*G0_1_1 - 0.0150051690850298*G0_1_2 - 6.52398655868283e-05*G0_2_0 - 0.0150051690850304*G0_2_1 - 0.0150051690850298*G0_2_2;
    A[1493] = -0.000652398655871267*G0_0_0 + 0.00884000178705042*G0_0_1 + 0.00884000178705054*G0_0_2 + 0.00884000178704962*G0_1_0 - 0.000652398655871273*G0_1_1 - 0.000652398655871262*G0_1_2 + 0.0088400017870496*G0_2_0 - 0.000652398655871263*G0_2_1 - 0.000652398655871366*G0_2_2;
    A[1494] = -0.00257697469068978*G0_0_1 - 0.00257697469068999*G0_0_2 - 0.00257697469069019*G0_1_0 - 0.000652398655870288*G0_1_1 - 0.000652398655871226*G0_1_2 - 0.00257697469069015*G0_2_0 - 0.000652398655870441*G0_2_1 - 0.000652398655871265*G0_2_2;
    A[1495] = -6.52398655871149e-05*G0_0_1 - 6.52398655869747e-05*G0_0_2 - 6.52398655870494e-05*G0_1_0 - 0.0150051690850301*G0_1_1 - 0.0150051690850295*G0_1_2 - 6.52398655870542e-05*G0_2_0 - 0.0150051690850299*G0_2_1 - 0.0150051690850294*G0_2_2;
    A[1496] = 0.0846813455320391*G0_0_0 - 0.00463203045668354*G0_0_1 - 0.0887914570640255*G0_0_2 - 0.0887914570640263*G0_1_0 - 0.0510175748891024*G0_1_1 - 0.0528442911255407*G0_1_2 - 0.0887914570640262*G0_2_0 - 0.0510175748891024*G0_2_1 - 0.0528442911255407*G0_2_2;
    A[1497] = -0.0622388317700816*G0_0_0 + 0.0985448169692949*G0_0_1 + 0.188575831479475*G0_0_2 + 0.188575831479476*G0_1_0 + 0.182410664181496*G0_1_1 + 0.104709984267275*G0_1_2 + 0.188575831479476*G0_2_0 + 0.182410664181496*G0_2_1 + 0.104709984267275*G0_2_2;
    A[1498] = -0.00339247301052818*G0_0_0 - 0.0623693115012551*G0_0_1 - 0.198068231922396*G0_0_2 - 0.198068231922397*G0_1_0 - 0.12473862300251*G0_1_1 - 0.271397840842283*G0_1_2 - 0.198068231922397*G0_2_0 - 0.12473862300251*G0_2_1 - 0.271397840842283*G0_2_2;
    A[1499] = 0.0127870136550696*G0_0_0 + 0.00329461321214797*G0_0_1 - 0.0250847283182348*G0_0_2 - 0.0250847283182347*G0_1_0 - 0.015592327875314*G0_1_1 - 0.00619778723077325*G0_1_2 - 0.0250847283182347*G0_2_0 - 0.0155923278753141*G0_2_1 - 0.00619778723077339*G0_2_2;
    A[1500] = -0.00678494602105722*G0_0_0 + 0.00296841388421322*G0_0_1 + 0.0303691574307889*G0_0_2 + 0.0303691574307891*G0_1_0 - 0.00352295274170222*G0_1_1 + 0.104709984267275*G0_1_2 + 0.030369157430789*G0_2_0 - 0.00352295274170223*G0_2_1 + 0.104709984267275*G0_2_2;
    A[1501] = -0.000130479731174056*G0_0_0 - 6.52398655869629e-05*G0_0_1 - 0.0137656116388753*G0_0_2 - 0.0137656116388755*G0_1_0 + 0.00117431758056771*G0_1_1 - 0.0528442911255406*G0_1_2 - 0.0137656116388753*G0_2_0 + 0.00117431758056771*G0_2_1 - 0.0528442911255405*G0_2_2;
    A[1502] = 0.084681345532039*G0_0_0 - 0.0887914570640257*G0_0_1 - 0.00463203045668371*G0_0_2 - 0.0887914570640259*G0_1_0 - 0.0528442911255407*G0_1_1 - 0.0510175748891023*G0_1_2 - 0.0887914570640258*G0_2_0 - 0.0528442911255409*G0_2_1 - 0.0510175748891024*G0_2_2;
    A[1503] = -0.0622388317700814*G0_0_0 + 0.188575831479475*G0_0_1 + 0.0985448169692949*G0_0_2 + 0.188575831479476*G0_1_0 + 0.104709984267276*G0_1_1 + 0.182410664181496*G0_1_2 + 0.188575831479476*G0_2_0 + 0.104709984267276*G0_2_1 + 0.182410664181496*G0_2_2;
    A[1504] = -0.00339247301052822*G0_0_0 - 0.198068231922396*G0_0_1 - 0.0623693115012554*G0_0_2 - 0.198068231922397*G0_1_0 - 0.271397840842283*G0_1_1 - 0.124738623002511*G0_1_2 - 0.198068231922397*G0_2_0 - 0.271397840842283*G0_2_1 - 0.124738623002511*G0_2_2;
    A[1505] = 0.0127870136550691*G0_0_0 - 0.0250847283182344*G0_0_1 + 0.00329461321214829*G0_0_2 - 0.0250847283182351*G0_1_0 - 0.00619778723077332*G0_1_1 - 0.0155923278753137*G0_1_2 - 0.0250847283182351*G0_2_0 - 0.00619778723077327*G0_2_1 - 0.0155923278753138*G0_2_2;
    A[1506] = -0.00678494602105699*G0_0_0 + 0.0303691574307889*G0_0_1 + 0.00296841388421273*G0_0_2 + 0.0303691574307892*G0_1_0 + 0.104709984267275*G0_1_1 - 0.00352295274170247*G0_1_2 + 0.0303691574307892*G0_2_0 + 0.104709984267275*G0_2_1 - 0.00352295274170247*G0_2_2;
    A[1507] = -0.000130479731174092*G0_0_0 - 0.0137656116388758*G0_0_1 - 6.52398655870423e-05*G0_0_2 - 0.0137656116388754*G0_1_0 - 0.0528442911255407*G0_1_1 + 0.00117431758056779*G0_1_2 - 0.0137656116388754*G0_2_0 - 0.0528442911255407*G0_2_1 + 0.00117431758056774*G0_2_2;
    A[1508] = 0.180714427676232*G0_0_0 + 0.0104383784939341*G0_0_1 + 0.010438378493934*G0_0_2 + 0.0104383784939329*G0_1_0 + 0.0260959462348349*G0_1_1 + 0.0260959462348344*G0_1_2 + 0.0104383784939331*G0_2_0 + 0.0260959462348349*G0_2_1 + 0.0260959462348346*G0_2_2;
    A[1509] = -0.161794866655975*G0_0_0 - 0.095576403085082*G0_0_1 - 0.0955764030850813*G0_0_2 - 0.0955764030850811*G0_1_0 - 0.185933616923198*G0_1_1 - 0.185933616923197*G0_1_2 - 0.0955764030850812*G0_2_0 - 0.185933616923198*G0_2_1 - 0.185933616923197*G0_2_2;
    A[1510] = 0.00130479731174105*G0_0_0 - 0.00587158790283775*G0_0_1 - 0.00587158790283919*G0_0_2 - 0.00587158790283838*G0_1_0 + 0.0260959462348354*G0_1_1 + 0.0260959462348342*G0_1_2 - 0.00587158790283838*G0_2_0 + 0.0260959462348353*G0_2_1 + 0.0260959462348343*G0_2_2;
    A[1511] = 0.00130479731174228*G0_0_0 - 0.00587158790283863*G0_0_1 - 0.00587158790283757*G0_0_2 - 0.00587158790283729*G0_1_0 + 0.0260959462348342*G0_1_1 + 0.0260959462348349*G0_1_2 - 0.00587158790283722*G0_2_0 + 0.0260959462348344*G0_2_1 + 0.0260959462348349*G0_2_2;
    A[1512] = 0.0153313684129654*G0_0_0 + 0.0153313684129654*G0_0_1 + 0.0153313684129654*G0_0_2 + 0.00480034930989783*G0_1_0 + 0.00480034930989786*G0_1_1 + 0.00480034930989789*G0_1_2 + 0.00480034930989784*G0_2_0 + 0.00480034930989788*G0_2_1 + 0.0048003493098979*G0_2_2;
    A[1513] = -0.0551276864210887*G0_0_0 - 0.0372571824394737*G0_1_0 - 0.0372571824394737*G0_2_0;
    A[1514] = -0.004800349309898*G0_1_1 - 0.00480034930989802*G0_2_1;
    A[1515] = -0.00480034930989794*G0_1_2 - 0.00480034930989794*G0_2_2;
    A[1516] = 0.00537576492437586*G0_1_1 + 0.00537576492437583*G0_1_2 + 0.00537576492437586*G0_2_1 + 0.00537576492437577*G0_2_2;
    A[1517] = 0.00228991928210687*G0_1_1 + 0.00228991928210681*G0_1_2 + 0.00228991928210683*G0_2_1 + 0.00228991928210682*G0_2_2;
    A[1518] = 0.0022899192821067*G0_1_1 + 0.00228991928210686*G0_1_2 + 0.00228991928210673*G0_2_1 + 0.00228991928210684*G0_2_2;
    A[1519] = 0.00537576492437595*G0_1_1 + 0.00537576492437598*G0_1_2 + 0.00537576492437592*G0_2_1 + 0.00537576492437598*G0_2_2;
    A[1520] = 0.0196371995417134*G0_0_0 - 0.0699110399631224*G0_0_2 - 0.0699110399631221*G0_1_0 - 0.139822079926245*G0_1_2 - 0.0699110399631221*G0_2_0 - 0.139822079926245*G0_2_2;
    A[1521] = -0.0186912214907005*G0_0_0 + 0.0387263842124951*G0_0_2 + 0.0387263842124949*G0_1_0 + 0.108637424175618*G0_1_2 + 0.0387263842124948*G0_2_0 + 0.108637424175618*G0_2_2;
    A[1522] = 0.0093945406445407*G0_0_0 - 0.0189782768992834*G0_0_2 - 0.0189782768992835*G0_1_0 - 0.0788945694544646*G0_1_2 - 0.0189782768992835*G0_2_0 - 0.0788945694544645*G0_2_2;
    A[1523] = 0.00537576492437586*G0_0_2 + 0.00537576492437592*G0_1_0 + 0.0328352243499811*G0_1_2 + 0.0053757649243759*G0_2_0 + 0.032835224349981*G0_2_2;
    A[1524] = 0.0196371995417133*G0_0_0 - 0.0699110399631226*G0_0_1 - 0.0699110399631222*G0_1_0 - 0.139822079926245*G0_1_1 - 0.0699110399631222*G0_2_0 - 0.139822079926245*G0_2_1;
    A[1525] = -0.0186912214907003*G0_0_0 + 0.0387263842124951*G0_0_1 + 0.0387263842124946*G0_1_0 + 0.108637424175618*G0_1_1 + 0.0387263842124945*G0_2_0 + 0.108637424175618*G0_2_1;
    A[1526] = 0.00939454064454068*G0_0_0 - 0.0189782768992837*G0_0_1 - 0.0189782768992831*G0_1_0 - 0.0788945694544643*G0_1_1 - 0.0189782768992831*G0_2_0 - 0.0788945694544643*G0_2_1;
    A[1527] = 0.00537576492437612*G0_0_1 + 0.00537576492437586*G0_1_0 + 0.0328352243499811*G0_1_1 + 0.00537576492437581*G0_2_0 + 0.0328352243499811*G0_2_1;
    A[1528] = 0.022083694501229*G0_0_0 + 0.022083694501229*G0_0_1 - 0.00537576492437586*G0_0_2 - 0.00537576492437597*G0_1_0 - 0.00537576492437597*G0_1_1 - 0.00537576492437592*G0_2_0 - 0.00537576492437592*G0_2_1;
    A[1529] = 0.00658922642429596*G0_0_0 + 0.006589226424296*G0_0_1 - 0.00228991928210667*G0_0_2 - 0.00228991928210654*G0_1_0 - 0.00228991928210659*G0_1_1 - 0.00228991928210653*G0_2_0 - 0.00228991928210659*G0_2_1;
    A[1530] = 0.00939454064454054*G0_0_0 + 0.00939454064454051*G0_0_1 - 0.00228991928210693*G0_0_2 - 0.00228991928210695*G0_1_0 - 0.00228991928210693*G0_1_1 - 0.00228991928210694*G0_2_0 - 0.00228991928210692*G0_2_1;
    A[1531] = -0.00537576492437587*G0_0_2 - 0.00537576492437594*G0_1_0 - 0.00537576492437595*G0_1_1 - 0.00537576492437591*G0_2_0 - 0.00537576492437592*G0_2_1;
    A[1532] = 0.0220836945012291*G0_0_0 - 0.00537576492437577*G0_0_1 + 0.0220836945012291*G0_0_2 - 0.00537576492437599*G0_1_0 - 0.00537576492437596*G0_1_2 - 0.00537576492437596*G0_2_0 - 0.00537576492437592*G0_2_2;
    A[1533] = 0.00658922642429574*G0_0_0 - 0.00228991928210694*G0_0_1 + 0.00658922642429578*G0_0_2 - 0.00228991928210681*G0_1_0 - 0.00228991928210683*G0_1_2 - 0.00228991928210675*G0_2_0 - 0.00228991928210676*G0_2_2;
    A[1534] = 0.00939454064454034*G0_0_0 - 0.00228991928210676*G0_0_1 + 0.00939454064454038*G0_0_2 - 0.00228991928210697*G0_1_0 - 0.00228991928210688*G0_1_2 - 0.00228991928210698*G0_2_0 - 0.00228991928210689*G0_2_2;
    A[1535] = -0.00537576492437587*G0_0_1 - 0.00537576492437588*G0_1_0 - 0.00537576492437587*G0_1_2 - 0.00537576492437582*G0_2_0 - 0.00537576492437581*G0_2_2;
    A[1536] = -0.0812105846828056*G0_0_0 - 0.0537511252572008*G0_0_1 - 0.0537511252572008*G0_0_2 - 0.0274594594256048*G0_1_0 - 0.0328352243499808*G0_1_1 - 0.0328352243499808*G0_1_2 - 0.0274594594256048*G0_2_0 - 0.0328352243499808*G0_2_1 - 0.0328352243499808*G0_2_2;
    A[1537] = 0.18526817029421*G0_0_0 + 0.103529142700148*G0_0_1 + 0.103529142700148*G0_0_2 + 0.0599162925551807*G0_1_0 + 0.0788945694544644*G0_1_1 + 0.0788945694544644*G0_1_2 + 0.0599162925551807*G0_2_0 + 0.0788945694544643*G0_2_1 + 0.0788945694544643*G0_2_2;
    A[1538] = -0.243357746612952*G0_0_0 - 0.135020425819035*G0_0_1 - 0.135020425819035*G0_0_2 - 0.0699110399631225*G0_1_0 - 0.108637424175617*G0_1_1 - 0.108637424175617*G0_1_2 - 0.0699110399631226*G0_2_0 - 0.108637424175617*G0_2_1 - 0.108637424175617*G0_2_2;
    A[1539] = 0.179096479009671*G0_0_0 + 0.0699110399631225*G0_0_1 + 0.0699110399631225*G0_0_2 + 0.0699110399631225*G0_1_0 + 0.139822079926245*G0_1_1 + 0.139822079926245*G0_1_2 + 0.0699110399631225*G0_2_0 + 0.139822079926245*G0_2_1 + 0.139822079926245*G0_2_2;
    A[1540] = -0.0606730749959905*G0_0_0 - 0.0284445813959695*G0_0_1 - 0.0284445813959695*G0_0_2 - 0.0284445813959692*G0_1_0 - 0.124738623002509*G0_1_1 - 0.124738623002509*G0_1_2 - 0.0284445813959693*G0_2_0 - 0.124738623002509*G0_2_1 - 0.124738623002509*G0_2_2;
    A[1541] = 0.0226708532915123*G0_0_0 + 0.000685018588664283*G0_0_1 + 0.00068501858866457*G0_0_2 + 0.0006850185886641*G0_1_0 + 0.0429604514890962*G0_1_1 + 0.0429604514890966*G0_1_2 + 0.000685018588664102*G0_2_0 + 0.0429604514890962*G0_2_1 + 0.0429604514890966*G0_2_2;
    A[1542] = 0.00939454064454061*G0_0_1 + 0.00939454064454097*G0_0_2 + 0.00939454064454093*G0_1_0 - 0.0118736555368493*G0_1_1 - 0.011873655536849*G0_1_2 + 0.00939454064454077*G0_2_0 - 0.0118736555368495*G0_2_1 - 0.0118736555368491*G0_2_2;
    A[1543] = 0.0226708532915131*G0_0_0 + 0.000685018588665162*G0_0_1 + 0.000685018588664678*G0_0_2 + 0.000685018588664406*G0_1_0 + 0.0429604514890968*G0_1_1 + 0.042960451489097*G0_1_2 + 0.00068501858866443*G0_2_0 + 0.0429604514890968*G0_2_1 + 0.0429604514890969*G0_2_2;
    A[1544] = 0.00329461321214718*G0_0_1 + 0.00329461321214678*G0_0_2 + 0.00329461321214764*G0_1_0 - 0.00254435475789705*G0_1_1 - 0.00254435475789716*G0_1_2 + 0.00329461321214771*G0_2_0 - 0.00254435475789693*G0_2_1 - 0.00254435475789711*G0_2_2;
    A[1545] = 0.00939454064454059*G0_0_1 + 0.00939454064454087*G0_0_2 + 0.00939454064454069*G0_1_0 - 0.0118736555368494*G0_1_1 - 0.0118736555368497*G0_1_2 + 0.00939454064454066*G0_2_0 - 0.0118736555368494*G0_2_1 - 0.0118736555368496*G0_2_2;
    A[1546] = 0.0453417065830257*G0_0_0 - 0.00939454064454051*G0_0_1 - 0.0093945406445405*G0_0_2 - 0.00939454064454139*G0_1_0 - 0.0093945406445412*G0_2_0;
    A[1547] = 0.0226708532915127*G0_0_0 - 0.0032946132121479*G0_0_1 - 0.00329461321214775*G0_0_2 - 0.00329461321214785*G0_1_0 - 0.00329461321214787*G0_2_0;
    A[1548] = -0.00939454064454072*G0_0_1 - 0.0093945406445404*G0_0_2 - 0.00939454064454098*G0_1_0 - 0.00939454064454082*G0_2_0;
    A[1549] = 0.0226708532915129*G0_0_0 - 0.00329461321214798*G0_0_1 - 0.00329461321214794*G0_0_2 - 0.00329461321214813*G0_1_0 - 0.00329461321214816*G0_2_0;
    A[1550] = -0.00329461321214782*G0_0_1 - 0.00329461321214808*G0_0_2 - 0.00329461321214758*G0_1_0 - 0.00329461321214765*G0_2_0;
    A[1551] = -0.00939454064454061*G0_0_1 - 0.00939454064454057*G0_0_2 - 0.00939454064454068*G0_1_0 - 0.00939454064454064*G0_2_0;
    A[1552] = -0.0693499771190738*G0_0_0 - 0.0146137298915074*G0_0_1 + 0.0212681961813902*G0_0_2 + 0.0212681961813906*G0_1_0 + 0.0118736555368501*G0_1_1 + 0.0212681961813905*G0_2_0 + 0.0118736555368501*G0_2_1;
    A[1553] = 0.0920860702761739*G0_0_0 - 0.0359145460056913*G0_0_1 - 0.0422754329004325*G0_0_2 - 0.0422754329004328*G0_1_0 - 0.0429604514890974*G0_1_1 - 0.0422754329004327*G0_2_0 - 0.0429604514890974*G0_2_1;
    A[1554] = -0.0644569872000426*G0_0_0 + 0.0284445813959697*G0_0_1 + 0.09629404160654*G0_0_2 + 0.0962940416065404*G0_1_0 + 0.124738623002509*G0_1_1 + 0.0962940416065403*G0_2_0 + 0.12473862300251*G0_2_1;
    A[1555] = -0.0318696743392926*G0_0_0 - 0.00590420783563173*G0_0_1 + 0.00583896797004429*G0_0_2 + 0.00583896797004386*G0_1_0 + 0.00254435475789609*G0_1_1 + 0.00583896797004389*G0_2_0 + 0.00254435475789614*G0_2_1;
    A[1556] = 0.043971669405697*G0_0_0 - 0.000685018588665131*G0_0_1 - 0.0422754329004324*G0_0_2 - 0.0422754329004322*G0_1_0 - 0.0429604514890971*G0_1_1 - 0.0422754329004322*G0_2_0 - 0.042960451489097*G0_2_1;
    A[1557] = -0.0187890812890812*G0_0_0 - 0.00939454064454057*G0_0_1 + 0.0212681961813905*G0_0_2 + 0.0212681961813905*G0_1_0 + 0.0118736555368495*G0_1_1 + 0.0212681961813904*G0_2_0 + 0.0118736555368495*G0_2_1;
    A[1558] = -0.0693499771190738*G0_0_0 + 0.0212681961813903*G0_0_1 - 0.0146137298915074*G0_0_2 + 0.0212681961813903*G0_1_0 + 0.0118736555368494*G0_1_2 + 0.0212681961813902*G0_2_0 + 0.0118736555368496*G0_2_2;
    A[1559] = 0.0920860702761739*G0_0_0 - 0.0422754329004325*G0_0_1 - 0.0359145460056911*G0_0_2 - 0.0422754329004324*G0_1_0 - 0.0429604514890972*G0_1_2 - 0.0422754329004324*G0_2_0 - 0.0429604514890974*G0_2_2;
    A[1560] = -0.0644569872000424*G0_0_0 + 0.0962940416065401*G0_0_1 + 0.0284445813959698*G0_0_2 + 0.0962940416065403*G0_1_0 + 0.12473862300251*G0_1_2 + 0.0962940416065403*G0_2_0 + 0.12473862300251*G0_2_2;
    A[1561] = -0.0318696743392919*G0_0_0 + 0.00583896797004427*G0_0_1 - 0.00590420783563149*G0_0_2 + 0.00583896797004465*G0_1_0 + 0.00254435475789663*G0_1_2 + 0.00583896797004462*G0_2_0 + 0.00254435475789664*G0_2_2;
    A[1562] = 0.0439716694056964*G0_0_0 - 0.0422754329004324*G0_0_1 - 0.000685018588664589*G0_0_2 - 0.0422754329004323*G0_1_0 - 0.0429604514890965*G0_1_2 - 0.0422754329004324*G0_2_0 - 0.0429604514890965*G0_2_2;
    A[1563] = -0.0187890812890811*G0_0_0 + 0.0212681961813903*G0_0_1 - 0.00939454064454096*G0_0_2 + 0.0212681961813901*G0_1_0 + 0.011873655536849*G0_1_2 + 0.0212681961813901*G0_2_0 + 0.0118736555368491*G0_2_2;
    A[1564] = -0.151356488162042*G0_0_0 + 0.00260959462348343*G0_0_1 + 0.00260959462348332*G0_0_2 + 0.00260959462348423*G0_1_0 + 0.00260959462348391*G0_2_0;
    A[1565] = 0.166687856575007*G0_0_0 + 0.0352295274170266*G0_0_1 + 0.0352295274170259*G0_0_2 + 0.0352295274170263*G0_1_0 + 0.0352295274170265*G0_2_0;
    A[1566] = -0.045341706583025*G0_0_0 + 0.00260959462348357*G0_0_1 + 0.00260959462348476*G0_0_2 + 0.00260959462348381*G0_1_0 + 0.00260959462348384*G0_2_0;
    A[1567] = -0.045341706583026*G0_0_0 + 0.00260959462348448*G0_0_1 + 0.00260959462348354*G0_0_2 + 0.00260959462348371*G0_1_0 + 0.00260959462348372*G0_2_0;
    A[1568] = -0.0137421252872641*G0_0_0 - 0.0137421252872641*G0_0_1 - 0.0137421252872641*G0_0_2 - 0.013742125287264*G0_1_0 - 0.0137421252872641*G0_1_1 - 0.0137421252872641*G0_1_2 - 0.013742125287264*G0_2_0 - 0.0137421252872641*G0_2_1 - 0.0137421252872641*G0_2_2;
    A[1569] = 0.0479930547204846*G0_0_0 + 0.0143945239431345*G0_1_0 + 0.0143945239431346*G0_2_0;
    A[1570] = -0.00517743573299117*G0_0_1 + 0.0137421252872642*G0_1_1 - 0.00517743573299102*G0_2_1;
    A[1571] = -0.00517743573299109*G0_0_2 - 0.00517743573299115*G0_1_2 + 0.0137421252872641*G0_2_2;
    A[1572] = 0.0425363923627802*G0_0_1 + 0.0118736555368497*G0_0_2 - 0.0187890812890807*G0_1_1 - 0.00939454064454018*G0_1_2 + 0.0212681961813905*G0_2_1 + 0.0118736555368502*G0_2_2;
    A[1573] = 0.0403182369328203*G0_0_1 + 0.0540186087061081*G0_0_2 + 0.0148746893538559*G0_1_1 + 0.0149399292194434*G0_1_2 + 0.00123955744615451*G0_2_1 + 0.00117431758056757*G0_2_2;
    A[1574] = 0.0540186087061074*G0_0_1 + 0.0403182369328195*G0_0_2 + 0.00117431758056727*G0_1_1 + 0.00123955744615429*G0_1_2 + 0.0149399292194429*G0_2_1 + 0.0148746893538556*G0_2_2;
    A[1575] = 0.0118736555368506*G0_0_1 + 0.0425363923627812*G0_0_2 + 0.0118736555368505*G0_1_1 + 0.0212681961813907*G0_1_2 - 0.00939454064454046*G0_2_1 - 0.0187890812890812*G0_2_2;
    A[1576] = -0.0644569872000426*G0_0_0 + 0.0284445813959693*G0_0_2 + 0.19258808321308*G0_1_0 + 0.124738623002509*G0_1_2 + 0.0962940416065399*G0_2_0 + 0.124738623002509*G0_2_2;
    A[1577] = 0.132306447410614*G0_0_0 + 0.0733296089198867*G0_0_2 + 0.0109602974186318*G0_1_0 + 0.146659217839774*G0_1_2 - 0.0623693115012551*G0_2_0 - 0.124738623002511*G0_2_2;
    A[1578] = -0.0510175748891032*G0_0_0 - 0.0463855444324198*G0_0_2 + 0.0018267162364373*G0_1_0 - 0.0823327103709054*G0_1_2 + 0.037773882174923*G0_2_0 + 0.127087258163646*G0_2_2;
    A[1579] = 0.0118736555368505*G0_0_0 + 0.0264873854283576*G0_0_2 + 0.0118736555368506*G0_1_0 + 0.0477555816097482*G0_1_2 - 0.00939454064454044*G0_2_0 - 0.0641307878721071*G0_2_2;
    A[1580] = -0.0644569872000423*G0_0_0 + 0.0284445813959694*G0_0_1 + 0.0962940416065395*G0_1_0 + 0.124738623002509*G0_1_1 + 0.192588083213081*G0_2_0 + 0.12473862300251*G0_2_1;
    A[1581] = 0.132306447410612*G0_0_0 + 0.0733296089198855*G0_0_1 - 0.0623693115012545*G0_1_0 - 0.12473862300251*G0_1_1 + 0.0109602974186314*G0_2_0 + 0.146659217839773*G0_2_1;
    A[1582] = -0.0510175748891017*G0_0_0 - 0.0463855444324188*G0_0_1 + 0.0377738821749227*G0_1_0 + 0.127087258163645*G0_1_1 + 0.0018267162364378*G0_2_0 - 0.0823327103709045*G0_2_1;
    A[1583] = 0.0118736555368494*G0_0_0 + 0.026487385428357*G0_0_1 - 0.00939454064454034*G0_1_0 - 0.0641307878721067*G0_1_1 + 0.0118736555368501*G0_2_0 + 0.0477555816097477*G0_2_1;
    A[1584] = 0.0187890812890804*G0_0_0 + 0.0187890812890806*G0_0_1 + 0.0093945406445409*G0_0_2 + 0.0187890812890805*G0_1_0 + 0.0187890812890805*G0_1_1 + 0.00939454064454038*G0_1_2 + 0.00939454064453988*G0_2_0 + 0.00939454064453991*G0_2_1;
    A[1585] = -0.0148746893538554*G0_0_0 - 0.0148746893538558*G0_0_1 + 6.52398655864871e-05*G0_0_2 - 0.0148746893538558*G0_1_0 - 0.0148746893538559*G0_1_1 + 6.52398655873023e-05*G0_1_2 + 6.5239865587209e-05*G0_2_0 + 6.52398655870697e-05*G0_2_1;
    A[1586] = -0.00117431758056781*G0_0_0 - 0.00117431758056752*G0_0_1 + 6.52398655875197e-05*G0_0_2 - 0.00117431758056765*G0_1_0 - 0.00117431758056751*G0_1_1 + 6.52398655869191e-05*G0_1_2 + 6.52398655868674e-05*G0_2_0 + 6.52398655870448e-05*G0_2_1;
    A[1587] = -0.0118736555368505*G0_0_0 - 0.0118736555368505*G0_0_1 + 0.00939454064454013*G0_0_2 - 0.0118736555368505*G0_1_0 - 0.0118736555368505*G0_1_1 + 0.00939454064454028*G0_1_2 + 0.00939454064454048*G0_2_0 + 0.00939454064454048*G0_2_1;
    A[1588] = 0.0187890812890807*G0_0_0 + 0.00939454064454086*G0_0_1 + 0.0187890812890806*G0_0_2 + 0.00939454064454069*G0_1_0 + 0.00939454064454062*G0_1_2 + 0.0187890812890809*G0_2_0 + 0.00939454064454062*G0_2_1 + 0.0187890812890807*G0_2_2;
    A[1589] = -0.0148746893538551*G0_0_0 + 6.52398655869952e-05*G0_0_1 - 0.0148746893538549*G0_0_2 + 6.52398655869721e-05*G0_1_0 + 6.52398655871293e-05*G0_1_2 - 0.0148746893538555*G0_2_0 + 6.52398655868222e-05*G0_2_1 - 0.0148746893538554*G0_2_2;
    A[1590] = -0.00117431758056722*G0_0_0 + 6.52398655873685e-05*G0_0_1 - 0.00117431758056742*G0_0_2 + 6.52398655874923e-05*G0_1_0 + 6.52398655872552e-05*G0_1_2 - 0.00117431758056742*G0_2_0 + 6.52398655877259e-05*G0_2_1 - 0.00117431758056748*G0_2_2;
    A[1591] = -0.0118736555368497*G0_0_0 + 0.00939454064454012*G0_0_1 - 0.0118736555368496*G0_0_2 + 0.0093945406445403*G0_1_0 + 0.00939454064454033*G0_1_2 - 0.0118736555368502*G0_2_0 + 0.00939454064453983*G0_2_1 - 0.0118736555368502*G0_2_2;
    A[1592] = 0.0453417065830255*G0_0_0 + 0.0547362472275653*G0_0_1 + 0.0547362472275652*G0_0_2 + 0.0547362472275659*G0_1_0 + 0.0641307878721062*G0_1_1 + 0.0641307878721061*G0_1_2 + 0.054736247227566*G0_2_0 + 0.0641307878721061*G0_2_1 + 0.064130787872106*G0_2_2;
    A[1593] = -0.0665446628988284*G0_0_0 - 0.0893133759887213*G0_0_1 - 0.0893133759887213*G0_0_2 - 0.0893133759887217*G0_1_0 - 0.127087258163645*G0_1_1 - 0.127087258163645*G0_1_2 - 0.089313375988722*G0_2_0 - 0.127087258163645*G0_2_1 - 0.127087258163645*G0_2_2;
    A[1594] = 0.047625101878573*G0_0_0 + 0.0623693115012545*G0_0_1 + 0.0623693115012545*G0_0_2 + 0.0623693115012549*G0_1_0 + 0.12473862300251*G0_1_1 + 0.12473862300251*G0_1_2 + 0.062369311501255*G0_2_0 + 0.12473862300251*G0_2_1 + 0.12473862300251*G0_2_2;
    A[1595] = -0.0606730749959905*G0_0_0 - 0.0284445813959692*G0_0_1 - 0.0284445813959693*G0_0_2 - 0.0284445813959695*G0_1_0 - 0.124738623002509*G0_1_1 - 0.124738623002509*G0_1_2 - 0.0284445813959695*G0_2_0 - 0.124738623002509*G0_2_1 - 0.124738623002509*G0_2_2;
    A[1596] = 0.647179466623903*G0_0_0 + 0.271397840842282*G0_0_1 + 0.271397840842281*G0_0_2 + 0.271397840842282*G0_1_0 + 0.542795681684563*G0_1_1 + 0.27139784084228*G0_1_2 + 0.271397840842281*G0_2_0 + 0.27139784084228*G0_2_1 + 0.542795681684564*G0_2_2;
    A[1597] = -0.268135847562928*G0_0_0 + 0.180714427676229*G0_0_1 + 0.0225077536275443*G0_0_2 - 0.0737210481134101*G0_1_0 - 0.209419968534552*G0_1_1 - 0.104709984267275*G0_1_2 - 0.0459941052388965*G0_2_0 + 0.0812236326559228*G0_2_1 - 0.0234863516113507*G0_2_2;
    A[1598] = 0.10568858225108*G0_0_0 - 0.0743734467692791*G0_0_1 + 0.000652398655870297*G0_0_2 + 0.0300103381700608*G0_1_0 + 0.105688582251081*G0_1_1 + 0.0528442911255401*G0_1_2 + 0.0528442911255407*G0_2_0 + 0.0006523986558711*G0_2_1 + 0.0534966897814106*G0_2_2;
    A[1599] = -0.268135847562929*G0_0_0 + 0.0225077536275443*G0_0_1 + 0.180714427676231*G0_0_2 - 0.045994105238897*G0_1_0 - 0.0234863516113505*G0_1_1 + 0.0812236326559233*G0_1_2 - 0.0737210481134091*G0_2_0 - 0.104709984267275*G0_2_1 - 0.209419968534552*G0_2_2;
    A[1600] = 0.113517366121532*G0_0_0 - 0.0851380245911491*G0_0_1 - 0.0851380245911487*G0_0_2 + 0.0127217737894814*G0_1_0 - 0.0137003717732888*G0_1_1 - 0.0198981590040628*G0_1_2 + 0.0127217737894816*G0_2_0 - 0.019898159004062*G0_2_1 - 0.0137003717732884*G0_2_2;
    A[1601] = 0.105688582251081*G0_0_0 + 0.000652398655870142*G0_0_1 - 0.0743734467692806*G0_0_2 + 0.052844291125541*G0_1_0 + 0.0534966897814109*G0_1_1 + 0.000652398655871044*G0_1_2 + 0.0300103381700597*G0_2_0 + 0.0528442911255397*G0_2_1 + 0.105688582251081*G0_2_2;
    A[1602] = -0.0600206763401199*G0_0_0 - 0.0300103381700593*G0_0_1 - 0.0300103381700597*G0_0_2 - 0.0300103381700602*G0_1_0 - 0.0150051690850304*G0_1_2 - 0.0300103381700597*G0_2_0 - 0.0150051690850285*G0_2_1;
    A[1603] = -0.025443547578964*G0_0_0 - 0.00130479731174218*G0_0_1 - 0.0127217737894834*G0_0_2 - 0.0013047973117402*G0_1_0 - 0.000652398655870298*G0_1_2 - 0.0127217737894824*G0_2_0 - 0.000652398655872611*G0_2_1;
    A[1604] = -0.10568858225108*G0_0_0 - 0.0300103381700595*G0_0_1 - 0.0528442911255406*G0_0_2 - 0.0300103381700607*G0_1_0 - 0.0150051690850307*G0_1_2 - 0.0528442911255407*G0_2_0 - 0.0150051690850292*G0_2_1;
    A[1605] = -0.0254435475789649*G0_0_0 - 0.0127217737894827*G0_0_1 - 0.00130479731174155*G0_0_2 - 0.0127217737894827*G0_1_0 - 0.000652398655870801*G0_1_2 - 0.00130479731174197*G0_2_0 - 0.000652398655870551*G0_2_1;
    A[1606] = -0.113517366121532*G0_0_0 - 0.0127217737894818*G0_0_1 - 0.0127217737894813*G0_0_2 - 0.0127217737894815*G0_1_0 - 0.000652398655871505*G0_1_2 - 0.0127217737894818*G0_2_0 - 0.000652398655870996*G0_2_1;
    A[1607] = -0.105688582251081*G0_0_0 - 0.0528442911255399*G0_0_1 - 0.0300103381700602*G0_0_2 - 0.0528442911255412*G0_1_0 - 0.0150051690850296*G0_1_2 - 0.0300103381700598*G0_2_0 - 0.01500516908503*G0_2_1;
    A[1608] = -0.0456679059109604*G0_0_0 - 0.0756782440810209*G0_0_1 - 0.0378391220405099*G0_0_2 - 0.0756782440810204*G0_1_0 - 0.105688582251081*G0_1_1 - 0.0528442911255409*G0_1_2 - 0.03783912204051*G0_2_0 - 0.0528442911255408*G0_2_1;
    A[1609] = 0.039143919352252*G0_0_0 + 0.135698920421142*G0_0_1 + 0.0678494602105707*G0_0_2 + 0.135698920421142*G0_1_0 + 0.209419968534551*G0_1_1 + 0.104709984267275*G0_1_2 + 0.0678494602105708*G0_2_0 + 0.104709984267276*G0_2_1;
    A[1610] = 0.0521918924696704*G0_0_0 - 0.271397840842281*G0_0_1 - 0.135698920421141*G0_0_2 - 0.271397840842282*G0_1_0 - 0.542795681684564*G0_1_1 - 0.271397840842284*G0_1_2 - 0.135698920421141*G0_2_0 - 0.27139784084228*G0_2_1;
    A[1611] = 0.039143919352252*G0_0_0 + 0.0264221455627699*G0_0_1 - 0.00554538857490288*G0_0_2 + 0.0264221455627692*G0_1_0 + 0.0137003717732867*G0_1_1 - 0.00619778723077384*G0_1_2 - 0.00554538857490251*G0_2_0 - 0.00619778723077406*G0_2_1;
    A[1612] = -0.15657567740901*G0_0_0 - 0.022507753627544*G0_0_1 + 0.0678494602105707*G0_0_2 - 0.022507753627545*G0_1_0 + 0.0234863516113512*G0_1_1 + 0.104709984267275*G0_1_2 + 0.0678494602105705*G0_2_0 + 0.104709984267275*G0_2_1;
    A[1613] = 0.0521918924696708*G0_0_0 - 0.000652398655870227*G0_0_1 - 0.0378391220405101*G0_0_2 - 0.000652398655869872*G0_1_0 - 0.0534966897814112*G0_1_1 - 0.0528442911255399*G0_1_2 - 0.03783912204051*G0_2_0 - 0.0528442911255398*G0_2_1;
    A[1614] = -0.0456679059109606*G0_0_0 - 0.0378391220405096*G0_0_1 - 0.0756782440810203*G0_0_2 - 0.0378391220405101*G0_1_0 - 0.0528442911255399*G0_1_2 - 0.0756782440810206*G0_2_0 - 0.0528442911255397*G0_2_1 - 0.10568858225108*G0_2_2;
    A[1615] = 0.0391439193522524*G0_0_0 + 0.0678494602105703*G0_0_1 + 0.135698920421141*G0_0_2 + 0.0678494602105706*G0_1_0 + 0.104709984267276*G0_1_2 + 0.135698920421142*G0_2_0 + 0.104709984267276*G0_2_1 + 0.209419968534551*G0_2_2;
    A[1616] = 0.0521918924696694*G0_0_0 - 0.13569892042114*G0_0_1 - 0.271397840842281*G0_0_2 - 0.135698920421141*G0_1_0 - 0.271397840842281*G0_1_2 - 0.271397840842282*G0_2_0 - 0.271397840842283*G0_2_1 - 0.542795681684564*G0_2_2;
    A[1617] = 0.0391439193522515*G0_0_0 - 0.00554538857490279*G0_0_1 + 0.0264221455627707*G0_0_2 - 0.00554538857490254*G0_1_0 - 0.00619778723077275*G0_1_2 + 0.0264221455627694*G0_2_0 - 0.00619778723077465*G0_2_1 + 0.013700371773288*G0_2_2;
    A[1618] = -0.156575677409008*G0_0_0 + 0.0678494602105698*G0_0_1 - 0.0225077536275444*G0_0_2 + 0.0678494602105701*G0_1_0 + 0.104709984267275*G0_1_2 - 0.0225077536275453*G0_2_0 + 0.104709984267274*G0_2_1 + 0.0234863516113506*G0_2_2;
    A[1619] = 0.0521918924696689*G0_0_0 - 0.0378391220405102*G0_0_1 - 0.000652398655870307*G0_0_2 - 0.0378391220405102*G0_1_0 - 0.0528442911255401*G0_1_2 - 0.000652398655870397*G0_2_0 - 0.0528442911255398*G0_2_1 - 0.0534966897814108*G0_2_2;
    A[1620] = 0.195719596761263*G0_0_0 + 0.0750258454251516*G0_0_1 + 0.0750258454251508*G0_0_2 + 0.0750258454251521*G0_1_0 + 0.0260959462348353*G0_1_2 + 0.0750258454251514*G0_2_0 + 0.026095946234836*G0_2_1;
    A[1621] = -0.782878387045046*G0_0_0 - 0.316413348097371*G0_0_1 - 0.316413348097371*G0_0_2 - 0.316413348097373*G0_1_0 - 0.185933616923198*G0_1_2 - 0.316413348097372*G0_2_0 - 0.185933616923198*G0_2_1;
    A[1622] = 0.293579395141892*G0_0_0 + 0.0750258454251484*G0_0_1 + 0.0587158790283775*G0_0_2 + 0.0750258454251501*G0_1_0 + 0.0260959462348354*G0_1_2 + 0.0587158790283787*G0_2_0 + 0.0260959462348331*G0_2_1;
    A[1623] = 0.293579395141895*G0_0_0 + 0.0587158790283793*G0_0_1 + 0.0750258454251503*G0_0_2 + 0.0587158790283802*G0_1_0 + 0.0260959462348334*G0_1_2 + 0.0750258454251514*G0_2_0 + 0.026095946234836*G0_2_1;
    A[1624] = -0.00946499969937453*G0_0_0 - 0.00946499969937457*G0_0_1 - 0.00946499969937457*G0_0_2 - 0.00946499969937467*G0_1_0 - 0.00946499969937463*G0_1_1 - 0.00946499969937464*G0_1_2 - 0.0094649996993745*G0_2_0 - 0.00946499969937448*G0_2_1 - 0.00946499969937443*G0_2_2;
    A[1625] = -0.0345719095719089*G0_0_0 - 0.00472467106581677*G0_1_0 - 0.00472467106581692*G0_2_0;
    A[1626] = -0.00472467106581674*G0_0_1 - 0.0345719095719092*G0_1_1 - 0.00472467106581695*G0_2_1;
    A[1627] = -0.00227817610630099*G0_0_2 - 0.00227817610630108*G0_1_2 + 0.00946499969937458*G0_2_2;
    A[1628] = -0.0845508658008646*G0_0_1 - 0.0429604514890966*G0_0_2 + 0.0439716694056964*G0_1_1 - 0.000685018588664286*G0_1_2 - 0.042275432900432*G0_2_1 - 0.0429604514890967*G0_2_2;
    A[1629] = -0.0808321934624015*G0_0_1 - 0.108232937008978*G0_0_2 - 0.0436454700777622*G0_1_1 - 0.0338921101724931*G0_1_2 - 0.00649136662591609*G0_2_1 - 0.00352295274170328*G0_2_2;
    A[1630] = -0.00939454064453957*G0_0_1 - 0.0377738821749228*G0_0_2 - 0.0155923278753132*G0_1_1 - 0.0188869410874612*G0_1_2 + 0.00949240044292197*G0_2_1 + 0.0189848008858427*G0_2_2;
    A[1631] = 0.00254435475789634*G0_0_1 + 0.0142875305635722*G0_0_2 + 0.00254435475789633*G0_1_1 + 0.00844856259352801*G0_1_2 - 0.00329461321214806*G0_2_1 - 0.0292600797158084*G0_2_2;
    A[1632] = 0.0439716694056963*G0_0_0 - 0.000685018588664062*G0_0_2 - 0.0845508658008648*G0_1_0 - 0.0429604514890962*G0_1_2 - 0.0422754329004319*G0_2_0 - 0.0429604514890963*G0_2_2;
    A[1633] = -0.0436454700777619*G0_0_0 - 0.0338921101724932*G0_0_2 - 0.0808321934624021*G0_1_0 - 0.10823293700898*G0_1_2 - 0.00649136662591639*G0_2_0 - 0.00352295274170382*G0_2_2;
    A[1634] = -0.0155923278753144*G0_0_0 - 0.0188869410874618*G0_0_2 - 0.00939454064454023*G0_1_0 - 0.0377738821749229*G0_1_2 + 0.00949240044292128*G0_2_0 + 0.0189848008858422*G0_2_2;
    A[1635] = 0.0025443547578964*G0_0_0 + 0.0084485625935277*G0_0_2 + 0.00254435475789618*G0_1_0 + 0.014287530563572*G0_1_2 - 0.00329461321214805*G0_2_0 - 0.0292600797158086*G0_2_2;
    A[1636] = 0.1273155976932*G0_0_0 - 0.000685018588664003*G0_0_1 - 0.00704590548340472*G0_1_0 - 0.0429604514890962*G0_1_1 - 0.049321338383837*G0_2_0 - 0.0429604514890964*G0_2_1;
    A[1637] = -0.166948816037355*G0_0_0 - 0.00616516729797966*G0_0_1 + 0.0838658472121995*G0_1_0 + 0.182410664181495*G0_1_1 + 0.1677316944244*G0_2_0 + 0.0777006799142207*G0_2_1;
    A[1638] = 0.182410664181495*G0_0_0 + 0.0838658472122006*G0_0_1 - 0.00616516729797893*G0_1_0 - 0.166948816037356*G0_1_1 + 0.0777006799142211*G0_2_0 + 0.167731694424401*G0_2_1;
    A[1639] = -0.0429604514890966*G0_0_0 - 0.00704590548340551*G0_0_1 - 0.000685018588664261*G0_1_0 + 0.127315597693201*G0_1_1 - 0.0429604514890967*G0_2_0 - 0.0493213383838374*G0_2_1;
    A[1640] = 0.00658922642429553*G0_0_0 + 0.00658922642429545*G0_0_1 + 0.00329461321214762*G0_0_2 + 0.0065892264242958*G0_1_0 + 0.00658922642429586*G0_1_1 + 0.00329461321214824*G0_1_2 + 0.00329461321214782*G0_2_0 + 0.00329461321214783*G0_2_1;
    A[1641] = 0.00450155072550815*G0_0_0 + 0.00450155072550838*G0_0_1 + 0.00257697469068991*G0_0_2 + 0.00450155072550855*G0_1_0 + 0.00450155072550843*G0_1_1 + 0.00257697469068901*G0_1_2 + 0.00257697469068971*G0_2_0 + 0.00257697469068982*G0_2_1;
    A[1642] = -0.0105036183595209*G0_0_0 - 0.010503618359521*G0_0_1 + 0.00257697469069036*G0_0_2 - 0.0105036183595203*G0_1_0 - 0.0105036183595203*G0_1_1 + 0.00257697469069084*G0_1_2 + 0.00257697469069018*G0_2_0 + 0.00257697469069009*G0_2_1;
    A[1643] = -0.00254435475789634*G0_0_0 - 0.0025443547578963*G0_0_1 + 0.00329461321214764*G0_0_2 - 0.00254435475789621*G0_1_0 - 0.00254435475789618*G0_1_1 + 0.00329461321214759*G0_1_2 + 0.00329461321214803*G0_2_0 + 0.00329461321214807*G0_2_1;
    A[1644] = 0.0292600797158082*G0_0_0 + 0.0259654665036606*G0_0_1 + 0.0292600797158083*G0_0_2 + 0.0259654665036603*G0_1_0 + 0.0226708532915129*G0_1_1 + 0.0259654665036603*G0_1_2 + 0.0292600797158083*G0_2_0 + 0.0259654665036604*G0_2_1 + 0.0292600797158083*G0_2_2;
    A[1645] = -0.0189848008858424*G0_0_0 - 0.00949240044292153*G0_0_1 - 0.0189848008858424*G0_0_2 - 0.00949240044292075*G0_1_0 - 0.000652398655871126*G0_1_1 - 0.00949240044292079*G0_1_2 - 0.0189848008858423*G0_2_0 - 0.00949240044292106*G0_2_1 - 0.0189848008858422*G0_2_2;
    A[1646] = 0.00352295274170221*G0_0_0 - 0.00296841388421188*G0_0_1 + 0.00352295274170219*G0_0_2 - 0.0029684138842121*G0_1_0 - 0.000652398655869983*G0_1_1 - 0.00296841388421196*G0_1_2 + 0.00352295274170277*G0_2_0 - 0.00296841388421237*G0_2_1 + 0.00352295274170277*G0_2_2;
    A[1647] = 0.0429604514890966*G0_0_0 + 0.000685018588663983*G0_0_1 + 0.0429604514890966*G0_0_2 + 0.000685018588664183*G0_1_0 + 0.022670853291512*G0_1_1 + 0.000685018588664235*G0_1_2 + 0.0429604514890966*G0_2_0 + 0.000685018588664334*G0_2_1 + 0.0429604514890967*G0_2_2;
    A[1648] = 0.0226708532915124*G0_0_0 + 0.0259654665036602*G0_0_1 + 0.0259654665036603*G0_0_2 + 0.0259654665036607*G0_1_0 + 0.0292600797158081*G0_1_1 + 0.0292600797158081*G0_1_2 + 0.0259654665036603*G0_2_0 + 0.0292600797158081*G0_2_1 + 0.0292600797158081*G0_2_2;
    A[1649] = -0.000652398655870625*G0_0_0 - 0.00949240044292086*G0_0_1 - 0.00949240044292081*G0_0_2 - 0.0094924004429211*G0_1_0 - 0.0189848008858419*G0_1_1 - 0.0189848008858419*G0_1_2 - 0.00949240044292079*G0_2_0 - 0.0189848008858421*G0_2_1 - 0.018984800885842*G0_2_2;
    A[1650] = -0.000652398655870607*G0_0_0 - 0.00296841388421233*G0_0_1 - 0.00296841388421231*G0_0_2 - 0.00296841388421236*G0_1_0 + 0.00352295274170267*G0_1_1 + 0.00352295274170263*G0_1_2 - 0.00296841388421261*G0_2_0 + 0.0035229527417026*G0_2_1 + 0.00352295274170256*G0_2_2;
    A[1651] = 0.0226708532915123*G0_0_0 + 0.000685018588664098*G0_0_1 + 0.0006850185886641*G0_0_2 + 0.000685018588664284*G0_1_0 + 0.0429604514890962*G0_1_1 + 0.0429604514890962*G0_1_2 + 0.00068501858866457*G0_2_0 + 0.0429604514890966*G0_2_1 + 0.0429604514890966*G0_2_2;
    A[1652] = -0.268135847562928*G0_0_0 - 0.0737210481134101*G0_0_1 - 0.0459941052388966*G0_0_2 + 0.180714427676229*G0_1_0 - 0.209419968534552*G0_1_1 + 0.0812236326559228*G0_1_2 + 0.0225077536275443*G0_2_0 - 0.104709984267275*G0_2_1 - 0.0234863516113507*G0_2_2;
    A[1653] = 0.726119703984283*G0_0_0 + 0.270093043530545*G0_0_1 + 0.270093043530543*G0_0_2 + 0.270093043530545*G0_1_0 + 0.726119703984283*G0_1_1 + 0.270093043530543*G0_1_2 + 0.270093043530543*G0_2_0 + 0.270093043530543*G0_2_1 + 0.540186087061082*G0_2_2;
    A[1654] = -0.209419968534551*G0_0_0 + 0.18071442767623*G0_0_1 + 0.0812236326559224*G0_0_2 - 0.0737210481134108*G0_1_0 - 0.26813584756293*G0_1_1 - 0.045994105238897*G0_1_2 - 0.104709984267275*G0_2_0 + 0.0225077536275445*G0_2_1 - 0.0234863516113512*G0_2_2;
    A[1655] = 0.0655660649150212*G0_0_0 - 0.0748627457611845*G0_0_1 + 0.0278900425384796*G0_0_2 + 0.0278900425384795*G0_1_0 - 0.149725491522367*G0_1_1 + 0.177126235068942*G0_1_2 - 0.0748627457611822*G0_2_0 - 0.0283793415303847*G0_2_1 - 0.149725491522365*G0_2_2;
    A[1656] = -0.149725491522365*G0_0_0 + 0.0278900425384811*G0_0_1 + 0.177126235068943*G0_0_2 - 0.0748627457611826*G0_1_0 + 0.065566064915024*G0_1_1 + 0.0278900425384809*G0_1_2 - 0.0283793415303826*G0_2_0 - 0.0748627457611803*G0_2_1 - 0.149725491522363*G0_2_2;
    A[1657] = -0.0137003717732872*G0_0_0 - 0.0198981590040619*G0_0_1 - 0.085138024591149*G0_0_2 - 0.0198981590040616*G0_1_0 - 0.0137003717732885*G0_1_1 - 0.0851380245911497*G0_1_2 + 0.0127217737894826*G0_2_0 + 0.012721773789482*G0_2_1 + 0.113517366121532*G0_2_2;
    A[1658] = -0.0254435475789638*G0_0_0 - 0.0241387502672214*G0_0_1 - 0.0127217737894818*G0_0_2 - 0.024138750267221*G0_1_0 - 0.0228339529554797*G0_1_1 - 0.0120693751336102*G0_1_2 - 0.0127217737894823*G0_2_0 - 0.0120693751336111*G0_2_1;
    A[1659] = 0.0567586830607671*G0_0_0 + 0.028379341530382*G0_0_1 + 0.0283793415303844*G0_0_2 + 0.0283793415303821*G0_1_0 - 0.00489298991903275*G0_1_1 + 0.0141896707651902*G0_1_2 + 0.028379341530383*G0_2_0 + 0.0141896707651917*G0_2_1;
    A[1660] = 0.20941996853455*G0_0_0 + 0.0737210481134092*G0_0_1 + 0.104709984267276*G0_0_2 + 0.0737210481134108*G0_1_0 - 0.0228339529554798*G0_1_1 + 0.0368605240567052*G0_1_2 + 0.104709984267275*G0_2_0 + 0.0368605240567039*G0_2_1;
    A[1661] = -0.0166361657247074*G0_0_0 + 0.00146789697571057*G0_0_1 - 0.0181040627004175*G0_0_2 + 0.00146789697571009*G0_1_0 + 0.0195719596761263*G0_1_1 - 0.010275278829966*G0_1_2 - 0.0181040627004172*G0_2_0 - 0.0102752788299666*G0_2_1;
    A[1662] = 0.149725491522365*G0_0_0 + 0.0748627457611823*G0_0_1 + 0.028379341530381*G0_0_2 + 0.0748627457611823*G0_1_0 + 0.0195719596761273*G0_1_1 + 0.0141896707651921*G0_1_2 + 0.0283793415303824*G0_2_0 + 0.0141896707651927*G0_2_1;
    A[1663] = 0.0137003717732872*G0_0_0 + 0.0198981590040605*G0_0_1 - 0.0127217737894814*G0_0_2 + 0.0198981590040615*G0_1_0 + 0.0260959462348333*G0_1_1 - 0.0120693751336121*G0_1_2 - 0.0127217737894826*G0_2_0 - 0.0120693751336121*G0_2_1;
    A[1664] = -0.0228339529554787*G0_0_0 - 0.0241387502672216*G0_0_1 - 0.0120693751336105*G0_0_2 - 0.0241387502672215*G0_1_0 - 0.0254435475789631*G0_1_1 - 0.0127217737894812*G0_1_2 - 0.0120693751336101*G0_2_0 - 0.0127217737894814*G0_2_1;
    A[1665] = -0.00489298991903267*G0_0_0 + 0.0283793415303826*G0_0_1 + 0.0141896707651906*G0_0_2 + 0.0283793415303823*G0_1_0 + 0.0567586830607657*G0_1_1 + 0.0283793415303828*G0_1_2 + 0.0141896707651906*G0_2_0 + 0.0283793415303824*G0_2_1;
    A[1666] = -0.02283395295548*G0_0_0 + 0.0737210481134099*G0_0_1 + 0.0368605240567055*G0_0_2 + 0.0737210481134084*G0_1_0 + 0.209419968534552*G0_1_1 + 0.104709984267277*G0_1_2 + 0.0368605240567037*G0_2_0 + 0.104709984267275*G0_2_1;
    A[1667] = 0.019571959676126*G0_0_0 + 0.00146789697570811*G0_0_1 - 0.0102752788299668*G0_0_2 + 0.00146789697571025*G0_1_0 - 0.0166361657247058*G0_1_1 - 0.0181040627004163*G0_1_2 - 0.0102752788299667*G0_2_0 - 0.0181040627004176*G0_2_1;
    A[1668] = 0.0195719596761274*G0_0_0 + 0.0748627457611844*G0_0_1 + 0.0141896707651922*G0_0_2 + 0.0748627457611833*G0_1_0 + 0.149725491522367*G0_1_1 + 0.0283793415303825*G0_1_2 + 0.0141896707651933*G0_2_0 + 0.0283793415303845*G0_2_1;
    A[1669] = 0.0260959462348353*G0_0_0 + 0.0198981590040618*G0_0_1 - 0.0120693751336107*G0_0_2 + 0.0198981590040606*G0_1_0 + 0.0137003717732885*G0_1_1 - 0.0127217737894826*G0_1_2 - 0.0120693751336114*G0_2_0 - 0.012721773789482*G0_2_1;
    A[1670] = -0.0880738185425658*G0_0_0 - 0.0887262171984369*G0_0_1 - 0.100795592332048*G0_0_2 - 0.0887262171984371*G0_1_0 - 0.0880738185425662*G0_1_1 - 0.100795592332048*G0_1_2 - 0.100795592332048*G0_2_0 - 0.100795592332049*G0_2_1 - 0.113517366121531*G0_2_2;
    A[1671] = 0.019571959676124*G0_0_0 + 0.0606730749959903*G0_0_1 + 0.0748627457611826*G0_0_2 + 0.0606730749959897*G0_1_0 + 0.0929668084615985*G0_1_1 + 0.121346149991982*G0_1_2 + 0.0748627457611808*G0_2_0 + 0.121346149991982*G0_2_1 + 0.149725491522364*G0_2_2;
    A[1672] = -0.0880738185425668*G0_0_0 + 0.00913358118219171*G0_0_1 + 0.0459941052388959*G0_0_2 + 0.00913358118219161*G0_1_0 - 0.185933616923198*G0_1_1 - 0.0812236326559229*G0_1_2 + 0.045994105238896*G0_2_0 - 0.0812236326559235*G0_2_1 + 0.0234863516113506*G0_2_2;
    A[1673] = 0.0929668084615984*G0_0_0 + 0.0606730749959886*G0_0_1 + 0.121346149991981*G0_0_2 + 0.0606730749959897*G0_1_0 + 0.0195719596761246*G0_1_1 + 0.0748627457611818*G0_1_2 + 0.121346149991981*G0_2_0 + 0.0748627457611814*G0_2_1 + 0.149725491522365*G0_2_2;
    A[1674] = 0.0929668084615988*G0_0_0 - 0.13504652176527*G0_0_1 - 0.270093043530543*G0_0_2 - 0.135046521765269*G0_1_0 + 0.0929668084615991*G0_1_1 - 0.270093043530542*G0_1_2 - 0.27009304353054*G0_2_0 - 0.27009304353054*G0_2_1 - 0.540186087061082*G0_2_2;
    A[1675] = -0.185933616923197*G0_0_0 + 0.00913358118219361*G0_0_1 - 0.0812236326559228*G0_0_2 + 0.0091335811821911*G0_1_0 - 0.0880738185425672*G0_1_1 + 0.0459941052388966*G0_1_2 - 0.0812236326559238*G0_2_0 + 0.0459941052388965*G0_2_1 + 0.0234863516113509*G0_2_2;
    A[1676] = 0.146789697570945*G0_0_0 + 0.141896707651915*G0_0_1 + 0.0929668084615972*G0_0_2 + 0.141896707651913*G0_1_0 + 0.146789697570945*G0_1_1 + 0.092966808461597*G0_1_2 + 0.0929668084615982*G0_2_0 + 0.0929668084615978*G0_2_1;
    A[1677] = 0.146789697570946*G0_0_0 - 0.298472385060927*G0_0_1 - 0.102752788299662*G0_0_2 - 0.298472385060921*G0_1_0 - 0.78287838704505*G0_1_1 - 0.298472385060923*G0_1_2 - 0.10275278829966*G0_2_0 - 0.298472385060925*G0_2_1;
    A[1678] = -0.78287838704505*G0_0_0 - 0.298472385060924*G0_0_1 - 0.298472385060924*G0_0_2 - 0.298472385060927*G0_1_0 + 0.146789697570948*G0_1_1 - 0.102752788299663*G0_1_2 - 0.298472385060926*G0_2_0 - 0.102752788299661*G0_2_1;
    A[1679] = -0.0489298991903142*G0_0_0 - 0.0293579395141891*G0_0_1 + 0.0929668084616001*G0_0_2 - 0.0293579395141896*G0_1_0 - 0.048929899190318*G0_1_1 + 0.0929668084616006*G0_1_2 + 0.0929668084615992*G0_2_0 + 0.0929668084615978*G0_2_1;
    A[1680] = -0.013742125287264*G0_0_0 - 0.0137421252872641*G0_0_1 - 0.0137421252872641*G0_0_2 - 0.0137421252872639*G0_1_0 - 0.013742125287264*G0_1_1 - 0.013742125287264*G0_1_2 - 0.013742125287264*G0_2_0 - 0.013742125287264*G0_2_1 - 0.013742125287264*G0_2_2;
    A[1681] = 0.0137421252872637*G0_0_0 - 0.00517743573299127*G0_1_0 - 0.00517743573299154*G0_2_0;
    A[1682] = 0.0143945239431347*G0_0_1 + 0.0479930547204847*G0_1_1 + 0.0143945239431348*G0_2_1;
    A[1683] = -0.0051774357329912*G0_0_2 - 0.005177435732991*G0_1_2 + 0.0137421252872641*G0_2_2;
    A[1684] = 0.19258808321308*G0_0_1 + 0.124738623002509*G0_0_2 - 0.0644569872000428*G0_1_1 + 0.028444581395969*G0_1_2 + 0.0962940416065399*G0_2_1 + 0.12473862300251*G0_2_2;
    A[1685] = 0.0109602974186317*G0_0_1 + 0.146659217839773*G0_0_2 + 0.132306447410614*G0_1_1 + 0.0733296089198877*G0_1_2 - 0.0623693115012546*G0_2_1 - 0.12473862300251*G0_2_2;
    A[1686] = 0.00182671623643747*G0_0_1 - 0.082332710370905*G0_0_2 - 0.0510175748891028*G0_1_1 - 0.0463855444324198*G0_1_2 + 0.0377738821749231*G0_2_1 + 0.127087258163646*G0_2_2;
    A[1687] = 0.0118736555368499*G0_0_1 + 0.0477555816097477*G0_0_2 + 0.0118736555368503*G0_1_1 + 0.0264873854283576*G0_1_2 - 0.00939454064454054*G0_2_1 - 0.0641307878721071*G0_2_2;
    A[1688] = -0.0187890812890806*G0_0_0 - 0.00939454064454086*G0_0_2 + 0.0425363923627805*G0_1_0 + 0.0118736555368495*G0_1_2 + 0.0212681961813904*G0_2_0 + 0.011873655536849*G0_2_2;
    A[1689] = 0.0148746893538559*G0_0_0 + 0.0149399292194438*G0_0_2 + 0.0403182369328205*G0_1_0 + 0.0540186087061086*G0_1_2 + 0.00123955744615476*G0_2_0 + 0.00117431758056869*G0_2_2;
    A[1690] = 0.00117431758056805*G0_0_0 + 0.00123955744615497*G0_0_2 + 0.0540186087061073*G0_1_0 + 0.0403182369328196*G0_1_2 + 0.0149399292194431*G0_2_0 + 0.0148746893538555*G0_2_2;
    A[1691] = 0.0118736555368499*G0_0_0 + 0.0212681961813902*G0_0_2 + 0.0118736555368504*G0_1_0 + 0.0425363923627808*G0_1_2 - 0.00939454064454046*G0_2_0 - 0.018789081289081*G0_2_2;
    A[1692] = -0.0641307878721065*G0_0_0 - 0.00939454064454091*G0_0_1 + 0.026487385428357*G0_1_0 + 0.0118736555368495*G0_1_1 + 0.0477555816097474*G0_2_0 + 0.0118736555368492*G0_2_1;
    A[1693] = 0.127087258163645*G0_0_0 + 0.0377738821749235*G0_0_1 - 0.0463855444324186*G0_1_0 - 0.0510175748891017*G0_1_1 - 0.0823327103709038*G0_2_0 + 0.00182671623643847*G0_2_1;
    A[1694] = -0.12473862300251*G0_0_0 - 0.0623693115012548*G0_0_1 + 0.0733296089198856*G0_1_0 + 0.132306447410612*G0_1_1 + 0.146659217839772*G0_2_0 + 0.0109602974186309*G0_2_1;
    A[1695] = 0.124738623002509*G0_0_0 + 0.0962940416065395*G0_0_1 + 0.028444581395969*G0_1_0 - 0.0644569872000432*G0_1_1 + 0.12473862300251*G0_2_0 + 0.192588083213081*G0_2_1;
    A[1696] = 0.0187890812890809*G0_0_0 + 0.018789081289081*G0_0_1 + 0.00939454064454069*G0_0_2 + 0.0187890812890808*G0_1_0 + 0.0187890812890807*G0_1_1 + 0.00939454064454039*G0_1_2 + 0.00939454064453985*G0_2_0 + 0.00939454064453988*G0_2_1;
    A[1697] = -0.0148746893538562*G0_0_0 - 0.0148746893538563*G0_0_1 + 6.52398655869318e-05*G0_0_2 - 0.0148746893538569*G0_1_0 - 0.0148746893538569*G0_1_1 + 6.52398655871517e-05*G0_1_2 + 6.52398655872217e-05*G0_2_0 + 6.52398655871175e-05*G0_2_1;
    A[1698] = -0.0011743175805677*G0_0_0 - 0.0011743175805674*G0_0_1 + 6.52398655872663e-05*G0_0_2 - 0.00117431758056794*G0_1_0 - 0.00117431758056765*G0_1_1 + 6.52398655872653e-05*G0_1_2 + 6.52398655867353e-05*G0_2_0 + 6.52398655868861e-05*G0_2_1;
    A[1699] = -0.0118736555368498*G0_0_0 - 0.0118736555368499*G0_0_1 + 0.00939454064454037*G0_0_2 - 0.0118736555368504*G0_1_0 - 0.0118736555368505*G0_1_1 + 0.00939454064454015*G0_1_2 + 0.0093945406445405*G0_2_0 + 0.0093945406445405*G0_2_1;
    A[1700] = 0.0641307878721061*G0_0_0 + 0.0547362472275657*G0_0_1 + 0.064130787872106*G0_0_2 + 0.0547362472275656*G0_1_0 + 0.045341706583025*G0_1_1 + 0.0547362472275655*G0_1_2 + 0.064130787872106*G0_2_0 + 0.0547362472275658*G0_2_1 + 0.0641307878721058*G0_2_2;
    A[1701] = -0.127087258163644*G0_0_0 - 0.0893133759887213*G0_0_1 - 0.127087258163644*G0_0_2 - 0.0893133759887212*G0_1_0 - 0.0665446628988278*G0_1_1 - 0.0893133759887211*G0_1_2 - 0.127087258163645*G0_2_0 - 0.0893133759887221*G0_2_1 - 0.127087258163645*G0_2_2;
    A[1702] = 0.124738623002509*G0_0_0 + 0.0623693115012543*G0_0_1 + 0.124738623002509*G0_0_2 + 0.0623693115012533*G0_1_0 + 0.0476251018785723*G0_1_1 + 0.0623693115012533*G0_1_2 + 0.12473862300251*G0_2_0 + 0.0623693115012553*G0_2_1 + 0.12473862300251*G0_2_2;
    A[1703] = -0.12473862300251*G0_0_0 - 0.0284445813959695*G0_0_1 - 0.12473862300251*G0_0_2 - 0.0284445813959691*G0_1_0 - 0.0606730749959904*G0_1_1 - 0.0284445813959691*G0_1_2 - 0.12473862300251*G0_2_0 - 0.0284445813959701*G0_2_1 - 0.12473862300251*G0_2_2;
    A[1704] = 0.00939454064454073*G0_0_1 + 0.00939454064454068*G0_0_2 + 0.00939454064454048*G0_1_0 + 0.018789081289081*G0_1_1 + 0.018789081289081*G0_1_2 + 0.00939454064454042*G0_2_0 + 0.0187890812890807*G0_2_1 + 0.0187890812890807*G0_2_2;
    A[1705] = 6.52398655870471e-05*G0_0_1 + 6.52398655870106e-05*G0_0_2 + 6.5239865587182e-05*G0_1_0 - 0.0148746893538557*G0_1_1 - 0.0148746893538556*G0_1_2 + 6.52398655870498e-05*G0_2_0 - 0.0148746893538553*G0_2_1 - 0.0148746893538553*G0_2_2;
    A[1706] = 6.52398655867464e-05*G0_0_1 + 6.52398655868333e-05*G0_0_2 + 6.52398655868792e-05*G0_1_0 - 0.00117431758056806*G0_1_1 - 0.00117431758056792*G0_1_2 + 6.52398655869416e-05*G0_2_0 - 0.00117431758056787*G0_2_1 - 0.00117431758056782*G0_2_2;
    A[1707] = 0.00939454064454092*G0_0_1 + 0.00939454064454077*G0_0_2 + 0.00939454064454061*G0_1_0 - 0.0118736555368493*G0_1_1 - 0.0118736555368495*G0_1_2 + 0.00939454064454097*G0_2_0 - 0.011873655536849*G0_2_1 - 0.0118736555368491*G0_2_2;
    A[1708] = 0.10568858225108*G0_0_0 + 0.0300103381700608*G0_0_1 + 0.0528442911255407*G0_0_2 - 0.0743734467692791*G0_1_0 + 0.105688582251081*G0_1_1 + 0.000652398655871092*G0_1_2 + 0.000652398655870297*G0_2_0 + 0.0528442911255401*G0_2_1 + 0.0534966897814106*G0_2_2;
    A[1709] = -0.209419968534551*G0_0_0 - 0.0737210481134108*G0_0_1 - 0.104709984267275*G0_0_2 + 0.18071442767623*G0_1_0 - 0.26813584756293*G0_1_1 + 0.0225077536275445*G0_1_2 + 0.0812236326559224*G0_2_0 - 0.045994105238897*G0_2_1 - 0.0234863516113512*G0_2_2;
    A[1710] = 0.542795681684564*G0_0_0 + 0.271397840842283*G0_0_1 + 0.271397840842281*G0_0_2 + 0.271397840842283*G0_1_0 + 0.647179466623907*G0_1_1 + 0.271397840842281*G0_1_2 + 0.271397840842281*G0_2_0 + 0.271397840842281*G0_2_1 + 0.542795681684564*G0_2_2;
    A[1711] = -0.0137003717732874*G0_0_0 + 0.0127217737894828*G0_0_1 - 0.0198981590040628*G0_0_2 - 0.0851380245911501*G0_1_0 + 0.113517366121531*G0_1_1 - 0.0851380245911502*G0_1_2 - 0.0198981590040605*G0_2_0 + 0.0127217737894832*G0_2_1 - 0.0137003717732883*G0_2_2;
    A[1712] = -0.0234863516113515*G0_0_0 - 0.0459941052388979*G0_0_1 + 0.0812236326559232*G0_0_2 + 0.0225077536275456*G0_1_0 - 0.268135847562929*G0_1_1 + 0.180714427676232*G0_1_2 - 0.104709984267275*G0_2_0 - 0.0737210481134094*G0_2_1 - 0.209419968534552*G0_2_2;
    A[1713] = 0.0534966897814102*G0_0_0 + 0.0528442911255408*G0_0_1 + 0.00065239865587115*G0_0_2 + 0.000652398655870844*G0_1_0 + 0.105688582251081*G0_1_1 - 0.0743734467692795*G0_1_2 + 0.0528442911255401*G0_2_0 + 0.0300103381700602*G0_2_1 + 0.105688582251082*G0_2_2;
    A[1714] = -0.105688582251081*G0_0_0 - 0.0756782440810207*G0_0_1 - 0.0528442911255404*G0_0_2 - 0.0756782440810227*G0_1_0 - 0.0456679059109606*G0_1_1 - 0.0378391220405115*G0_1_2 - 0.0528442911255396*G0_2_0 - 0.0378391220405089*G0_2_1;
    A[1715] = 0.209419968534549*G0_0_0 + 0.13569892042114*G0_0_1 + 0.104709984267274*G0_0_2 + 0.135698920421142*G0_1_0 + 0.0391439193522513*G0_1_1 + 0.0678494602105712*G0_1_2 + 0.104709984267274*G0_2_0 + 0.0678494602105691*G0_2_1;
    A[1716] = -0.542795681684565*G0_0_0 - 0.271397840842281*G0_0_1 - 0.271397840842282*G0_0_2 - 0.271397840842284*G0_1_0 + 0.0521918924696718*G0_1_1 - 0.135698920421141*G0_1_2 - 0.271397840842282*G0_2_0 - 0.13569892042114*G0_2_1;
    A[1717] = 0.0137003717732877*G0_0_0 + 0.0264221455627707*G0_0_1 - 0.00619778723077294*G0_0_2 + 0.026422145562769*G0_1_0 + 0.0391439193522539*G0_1_1 - 0.00554538857490213*G0_1_2 - 0.00619778723077398*G0_2_0 - 0.00554538857490254*G0_2_1;
    A[1718] = 0.0234863516113517*G0_0_0 - 0.0225077536275459*G0_0_1 + 0.104709984267276*G0_0_2 - 0.0225077536275455*G0_1_0 - 0.156575677409011*G0_1_1 + 0.0678494602105695*G0_1_2 + 0.104709984267275*G0_2_0 + 0.06784946021057*G0_2_1;
    A[1719] = -0.0534966897814102*G0_0_0 - 0.000652398655869992*G0_0_1 - 0.0528442911255404*G0_0_2 - 0.000652398655870816*G0_1_0 + 0.0521918924696709*G0_1_1 - 0.0378391220405096*G0_1_2 - 0.05284429112554*G0_2_0 - 0.0378391220405097*G0_2_1;
    A[1720] = -0.0300103381700616*G0_0_1 - 0.0150051690850313*G0_0_2 - 0.0300103381700601*G0_1_0 - 0.0600206763401223*G0_1_1 - 0.0300103381700614*G0_1_2 - 0.0150051690850291*G0_2_0 - 0.0300103381700602*G0_2_1;
    A[1721] = -0.00130479731174052*G0_0_1 - 0.000652398655870034*G0_0_2 - 0.00130479731174138*G0_1_0 - 0.0254435475789627*G0_1_1 - 0.0127217737894817*G0_1_2 - 0.000652398655870843*G0_2_0 - 0.0127217737894811*G0_2_1;
    A[1722] = -0.0300103381700608*G0_0_1 - 0.0150051690850304*G0_0_2 - 0.0300103381700598*G0_1_0 - 0.105688582251082*G0_1_1 - 0.0528442911255406*G0_1_2 - 0.0150051690850303*G0_2_0 - 0.0528442911255406*G0_2_1;
    A[1723] = -0.0127217737894818*G0_0_1 - 0.000652398655869783*G0_0_2 - 0.0127217737894813*G0_1_0 - 0.0254435475789664*G0_1_1 - 0.00130479731174133*G0_1_2 - 0.000652398655870841*G0_2_0 - 0.00130479731174224*G0_2_1;
    A[1724] = -0.0127217737894827*G0_0_1 - 0.000652398655871785*G0_0_2 - 0.0127217737894823*G0_1_0 - 0.113517366121531*G0_1_1 - 0.0127217737894823*G0_1_2 - 0.000652398655871077*G0_2_0 - 0.012721773789483*G0_2_1;
    A[1725] = -0.0528442911255406*G0_0_1 - 0.01500516908503*G0_0_2 - 0.0528442911255394*G0_1_0 - 0.105688582251081*G0_1_1 - 0.0300103381700591*G0_1_2 - 0.0150051690850299*G0_2_0 - 0.0300103381700601*G0_2_1;
    A[1726] = -0.0378391220405109*G0_0_1 - 0.052844291125541*G0_0_2 - 0.0378391220405102*G0_1_0 - 0.0456679059109607*G0_1_1 - 0.0756782440810215*G0_1_2 - 0.0528442911255392*G0_2_0 - 0.0756782440810198*G0_2_1 - 0.10568858225108*G0_2_2;
    A[1727] = -0.00554538857490167*G0_0_1 - 0.0061977872307727*G0_0_2 - 0.0055453885749022*G0_1_0 + 0.0391439193522529*G0_1_1 + 0.0264221455627712*G0_1_2 - 0.00619778723077379*G0_2_0 + 0.0264221455627698*G0_2_1 + 0.0137003717732886*G0_2_2;
    A[1728] = -0.0378391220405105*G0_0_1 - 0.0528442911255406*G0_0_2 - 0.0378391220405103*G0_1_0 + 0.0521918924696694*G0_1_1 - 0.000652398655871355*G0_1_2 - 0.0528442911255403*G0_2_0 - 0.000652398655871143*G0_2_1 - 0.0534966897814108*G0_2_2;
    A[1729] = 0.0678494602105709*G0_0_1 + 0.104709984267275*G0_0_2 + 0.0678494602105701*G0_1_0 + 0.0391439193522509*G0_1_1 + 0.13569892042114*G0_1_2 + 0.104709984267274*G0_2_0 + 0.13569892042114*G0_2_1 + 0.209419968534549*G0_2_2;
    A[1730] = 0.0678494602105696*G0_0_1 + 0.104709984267276*G0_0_2 + 0.0678494602105698*G0_1_0 - 0.156575677409009*G0_1_1 - 0.0225077536275443*G0_1_2 + 0.104709984267275*G0_2_0 - 0.0225077536275451*G0_2_1 + 0.0234863516113515*G0_2_2;
    A[1731] = -0.13569892042114*G0_0_1 - 0.271397840842281*G0_0_2 - 0.135698920421139*G0_1_0 + 0.0521918924696718*G0_1_1 - 0.271397840842281*G0_1_2 - 0.271397840842282*G0_2_0 - 0.271397840842281*G0_2_1 - 0.542795681684564*G0_2_2;
    A[1732] = 0.0750258454251526*G0_0_1 + 0.026095946234837*G0_0_2 + 0.0750258454251539*G0_1_0 + 0.195719596761266*G0_1_1 + 0.0750258454251521*G0_1_2 + 0.0260959462348345*G0_2_0 + 0.0750258454251503*G0_2_1;
    A[1733] = 0.0750258454251513*G0_0_1 + 0.0260959462348351*G0_0_2 + 0.0750258454251477*G0_1_0 + 0.293579395141893*G0_1_1 + 0.058715879028379*G0_1_2 + 0.0260959462348347*G0_2_0 + 0.0587158790283791*G0_2_1;
    A[1734] = -0.316413348097374*G0_0_1 - 0.185933616923198*G0_0_2 - 0.316413348097372*G0_1_0 - 0.78287838704505*G0_1_1 - 0.316413348097373*G0_1_2 - 0.185933616923196*G0_2_0 - 0.316413348097372*G0_2_1;
    A[1735] = 0.0587158790283791*G0_0_1 + 0.0260959462348332*G0_0_2 + 0.0587158790283808*G0_1_0 + 0.293579395141895*G0_1_1 + 0.0750258454251494*G0_1_2 + 0.0260959462348343*G0_2_0 + 0.0750258454251511*G0_2_1;
    A[1736] = -0.00946499969937459*G0_0_0 - 0.00946499969937462*G0_0_1 - 0.00946499969937462*G0_0_2 - 0.0094649996993745*G0_1_0 - 0.00946499969937453*G0_1_1 - 0.0094649996993745*G0_1_2 - 0.0094649996993747*G0_2_0 - 0.00946499969937473*G0_2_1 - 0.00946499969937478*G0_2_2;
    A[1737] = -0.0345719095719095*G0_0_0 - 0.00472467106581712*G0_1_0 - 0.00472467106581699*G0_2_0;
    A[1738] = -0.00227817610630117*G0_0_1 + 0.00946499969937432*G0_1_1 - 0.00227817610630119*G0_2_1;
    A[1739] = -0.00472467106581704*G0_0_2 - 0.00472467106581676*G0_1_2 - 0.0345719095719091*G0_2_2;
    A[1740] = 0.0142875305635734*G0_0_1 + 0.00254435475789676*G0_0_2 - 0.0292600797158079*G0_1_1 - 0.00329461321214826*G0_1_2 + 0.00844856259352766*G0_2_1 + 0.00254435475789598*G0_2_2;
    A[1741] = -0.0377738821749255*G0_0_1 - 0.00939454064454051*G0_0_2 + 0.0189848008858415*G0_1_1 + 0.00949240044292173*G0_1_2 - 0.0188869410874622*G0_2_1 - 0.0155923278753139*G0_2_2;
    A[1742] = -0.108232937008976*G0_0_1 - 0.0808321934624016*G0_0_2 - 0.00352295274170268*G0_1_1 - 0.00649136662591593*G0_1_2 - 0.0338921101724922*G0_2_1 - 0.0436454700777619*G0_2_2;
    A[1743] = -0.0429604514890975*G0_0_1 - 0.084550865800865*G0_0_2 - 0.0429604514890971*G0_1_1 - 0.0422754329004327*G0_1_2 - 0.000685018588663708*G0_2_1 + 0.0439716694056974*G0_2_2;
    A[1744] = 0.127315597693202*G0_0_0 - 0.000685018588664552*G0_0_2 - 0.0493213383838367*G0_1_0 - 0.0429604514890966*G0_1_2 - 0.00704590548340528*G0_2_0 - 0.0429604514890971*G0_2_2;
    A[1745] = -0.166948816037357*G0_0_0 - 0.00616516729797946*G0_0_2 + 0.1677316944244*G0_1_0 + 0.0777006799142209*G0_1_2 + 0.0838658472122012*G0_2_0 + 0.182410664181497*G0_2_2;
    A[1746] = 0.182410664181498*G0_0_0 + 0.0838658472122023*G0_0_2 + 0.0777006799142218*G0_1_0 + 0.167731694424402*G0_1_2 - 0.00616516729797999*G0_2_0 - 0.166948816037357*G0_2_2;
    A[1747] = -0.0429604514890976*G0_0_0 - 0.00704590548340579*G0_0_2 - 0.0429604514890971*G0_1_0 - 0.0493213383838385*G0_1_2 - 0.000685018588663805*G0_2_0 + 0.127315597693201*G0_2_2;
    A[1748] = 0.0439716694056969*G0_0_0 - 0.000685018588664433*G0_0_1 - 0.0422754329004319*G0_1_0 - 0.0429604514890968*G0_1_1 - 0.084550865800865*G0_2_0 - 0.042960451489097*G0_2_1;
    A[1749] = -0.0436454700777611*G0_0_0 - 0.0338921101724917*G0_0_1 - 0.00649136662591603*G0_1_0 - 0.00352295274170304*G0_1_1 - 0.080832193462402*G0_2_0 - 0.108232937008978*G0_2_1;
    A[1750] = -0.0155923278753137*G0_0_0 - 0.0188869410874617*G0_0_1 + 0.00949240044292181*G0_1_0 + 0.0189848008858422*G0_1_1 - 0.00939454064453967*G0_2_0 - 0.037773882174923*G0_2_1;
    A[1751] = 0.00254435475789698*G0_0_0 + 0.00844856259352856*G0_0_1 - 0.00329461321214813*G0_1_0 - 0.0292600797158081*G0_1_1 + 0.00254435475789607*G0_2_0 + 0.0142875305635721*G0_2_1;
    A[1752] = 0.029260079715809*G0_0_0 + 0.0292600797158088*G0_0_1 + 0.0259654665036604*G0_0_2 + 0.0292600797158081*G0_1_0 + 0.0292600797158081*G0_1_1 + 0.0259654665036604*G0_1_2 + 0.0259654665036604*G0_2_0 + 0.0259654665036604*G0_2_1 + 0.0226708532915133*G0_2_2;
    A[1753] = -0.0189848008858427*G0_0_0 - 0.0189848008858425*G0_0_1 - 0.00949240044292052*G0_0_2 - 0.0189848008858416*G0_1_0 - 0.0189848008858416*G0_1_1 - 0.00949240044292059*G0_1_2 - 0.00949240044292108*G0_2_0 - 0.00949240044292113*G0_2_1 - 0.000652398655871572*G0_2_2;
    A[1754] = 0.00352295274170373*G0_0_0 + 0.00352295274170369*G0_0_1 - 0.00296841388421318*G0_0_2 + 0.00352295274170261*G0_1_0 + 0.00352295274170265*G0_1_1 - 0.00296841388421289*G0_1_2 - 0.00296841388421187*G0_2_0 - 0.0029684138842118*G0_2_1 - 0.000652398655870196*G0_2_2;
    A[1755] = 0.0429604514890976*G0_0_0 + 0.0429604514890976*G0_0_1 + 0.00068501858866467*G0_0_2 + 0.042960451489097*G0_1_0 + 0.0429604514890971*G0_1_1 + 0.000685018588664221*G0_1_2 + 0.000685018588663804*G0_2_0 + 0.000685018588663782*G0_2_1 + 0.0226708532915123*G0_2_2;
    A[1756] = 0.00658922642429619*G0_0_0 + 0.00329461321214786*G0_0_1 + 0.0065892264242963*G0_0_2 + 0.00329461321214765*G0_1_0 + 0.00329461321214775*G0_1_2 + 0.00658922642429546*G0_2_0 + 0.00329461321214832*G0_2_1 + 0.00658922642429569*G0_2_2;
    A[1757] = 0.00450155072550844*G0_0_0 + 0.0025769746906896*G0_0_1 + 0.00450155072550824*G0_0_2 + 0.00257697469069006*G0_1_0 + 0.0025769746906901*G0_1_2 + 0.00450155072550928*G0_2_0 + 0.00257697469068942*G0_2_1 + 0.00450155072550927*G0_2_2;
    A[1758] = -0.0105036183595201*G0_0_0 + 0.00257697469069048*G0_0_1 - 0.0105036183595199*G0_0_2 + 0.00257697469068985*G0_1_0 + 0.00257697469068987*G0_1_2 - 0.0105036183595208*G0_2_0 + 0.00257697469069048*G0_2_1 - 0.0105036183595208*G0_2_2;
    A[1759] = -0.00254435475789676*G0_0_0 + 0.00329461321214789*G0_0_1 - 0.00254435475789679*G0_0_2 + 0.00329461321214815*G0_1_0 + 0.00329461321214817*G0_1_2 - 0.00254435475789605*G0_2_0 + 0.00329461321214788*G0_2_1 - 0.00254435475789596*G0_2_2;
    A[1760] = 0.022670853291513*G0_0_0 + 0.0259654665036614*G0_0_1 + 0.0259654665036614*G0_0_2 + 0.0259654665036605*G0_1_0 + 0.0292600797158082*G0_1_1 + 0.0292600797158083*G0_1_2 + 0.0259654665036609*G0_2_0 + 0.0292600797158082*G0_2_1 + 0.0292600797158083*G0_2_2;
    A[1761] = -0.00065239865587105*G0_0_0 - 0.00949240044292143*G0_0_1 - 0.0094924004429214*G0_0_2 - 0.00949240044292083*G0_1_0 - 0.0189848008858423*G0_1_1 - 0.0189848008858423*G0_1_2 - 0.00949240044292106*G0_2_0 - 0.0189848008858421*G0_2_1 - 0.0189848008858421*G0_2_2;
    A[1762] = -0.000652398655871014*G0_0_0 - 0.00296841388421222*G0_0_1 - 0.00296841388421228*G0_0_2 - 0.00296841388421324*G0_1_0 + 0.00352295274170233*G0_1_1 + 0.00352295274170233*G0_1_2 - 0.00296841388421276*G0_2_0 + 0.00352295274170277*G0_2_1 + 0.00352295274170285*G0_2_2;
    A[1763] = 0.0226708532915131*G0_0_0 + 0.000685018588664407*G0_0_1 + 0.000685018588664431*G0_0_2 + 0.000685018588665166*G0_1_0 + 0.0429604514890968*G0_1_1 + 0.0429604514890968*G0_1_2 + 0.000685018588664687*G0_2_0 + 0.042960451489097*G0_2_1 + 0.0429604514890969*G0_2_2;
    A[1764] = -0.268135847562929*G0_0_0 - 0.045994105238897*G0_0_1 - 0.0737210481134091*G0_0_2 + 0.0225077536275443*G0_1_0 - 0.0234863516113505*G0_1_1 - 0.104709984267275*G0_1_2 + 0.180714427676231*G0_2_0 + 0.0812236326559233*G0_2_1 - 0.209419968534552*G0_2_2;
    A[1765] = 0.0655660649150212*G0_0_0 + 0.0278900425384795*G0_0_1 - 0.0748627457611822*G0_0_2 - 0.0748627457611845*G0_1_0 - 0.149725491522367*G0_1_1 - 0.0283793415303847*G0_1_2 + 0.0278900425384796*G0_2_0 + 0.177126235068942*G0_2_1 - 0.149725491522365*G0_2_2;
    A[1766] = -0.0137003717732874*G0_0_0 - 0.0851380245911501*G0_0_1 - 0.0198981590040605*G0_0_2 + 0.0127217737894828*G0_1_0 + 0.113517366121531*G0_1_1 + 0.0127217737894832*G0_1_2 - 0.0198981590040629*G0_2_0 - 0.0851380245911502*G0_2_1 - 0.0137003717732883*G0_2_2;
    A[1767] = 0.726119703984287*G0_0_0 + 0.270093043530546*G0_0_1 + 0.270093043530543*G0_0_2 + 0.270093043530546*G0_1_0 + 0.540186087061085*G0_1_1 + 0.270093043530543*G0_1_2 + 0.270093043530543*G0_2_0 + 0.270093043530543*G0_2_1 + 0.726119703984283*G0_2_2;
    A[1768] = -0.149725491522367*G0_0_0 + 0.177126235068943*G0_0_1 + 0.0278900425384769*G0_0_2 - 0.0283793415303845*G0_1_0 - 0.149725491522365*G0_1_1 - 0.0748627457611848*G0_1_2 - 0.0748627457611842*G0_2_0 + 0.0278900425384793*G0_2_1 + 0.0655660649150202*G0_2_2;
    A[1769] = -0.209419968534548*G0_0_0 + 0.0812236326559257*G0_0_1 + 0.180714427676237*G0_0_2 - 0.104709984267275*G0_1_0 - 0.02348635161135*G0_1_1 + 0.0225077536275484*G0_1_2 - 0.0737210481134076*G0_2_0 - 0.0459941052388954*G0_2_1 - 0.268135847562928*G0_2_2;
    A[1770] = -0.0254435475789636*G0_0_0 - 0.0127217737894818*G0_0_1 - 0.0241387502672224*G0_0_2 - 0.012721773789482*G0_1_0 - 0.0120693751336108*G0_1_2 - 0.024138750267221*G0_2_0 - 0.0120693751336103*G0_2_1 - 0.0228339529554796*G0_2_2;
    A[1771] = -0.0166361657247073*G0_0_0 - 0.0181040627004163*G0_0_1 + 0.00146789697570951*G0_0_2 - 0.0181040627004161*G0_1_0 - 0.0102752788299658*G0_1_2 + 0.00146789697571015*G0_2_0 - 0.0102752788299659*G0_2_1 + 0.0195719596761263*G0_2_2;
    A[1772] = 0.0137003717732877*G0_0_0 - 0.0127217737894841*G0_0_1 + 0.0198981590040605*G0_0_2 - 0.0127217737894826*G0_1_0 - 0.0120693751336114*G0_1_2 + 0.0198981590040631*G0_2_0 - 0.0120693751336121*G0_2_1 + 0.0260959462348348*G0_2_2;
    A[1773] = 0.0567586830607662*G0_0_0 + 0.0283793415303822*G0_0_1 + 0.0283793415303831*G0_0_2 + 0.0283793415303814*G0_1_0 + 0.0141896707651901*G0_1_2 + 0.0283793415303824*G0_2_0 + 0.01418967076519*G0_2_1 - 0.00489298991903299*G0_2_2;
    A[1774] = 0.149725491522367*G0_0_0 + 0.0283793415303859*G0_0_1 + 0.0748627457611837*G0_0_2 + 0.0283793415303844*G0_1_0 + 0.0141896707651916*G0_1_2 + 0.0748627457611841*G0_2_0 + 0.0141896707651933*G0_2_1 + 0.0195719596761263*G0_2_2;
    A[1775] = 0.209419968534549*G0_0_0 + 0.104709984267273*G0_0_1 + 0.0737210481134082*G0_0_2 + 0.104709984267275*G0_1_0 + 0.0368605240567056*G0_1_2 + 0.0737210481134076*G0_2_0 + 0.0368605240567039*G0_2_1 - 0.0228339529554804*G0_2_2;
    A[1776] = -0.0880738185425679*G0_0_0 - 0.10079559233205*G0_0_1 - 0.0887262171984396*G0_0_2 - 0.100795592332049*G0_1_0 - 0.113517366121531*G0_1_1 - 0.100795592332049*G0_1_2 - 0.0887262171984379*G0_2_0 - 0.100795592332049*G0_2_1 - 0.0880738185425671*G0_2_2;
    A[1777] = 0.019571959676127*G0_0_0 + 0.0748627457611841*G0_0_1 + 0.0606730749959919*G0_0_2 + 0.0748627457611826*G0_1_0 + 0.149725491522365*G0_1_1 + 0.121346149991982*G0_1_2 + 0.0606730749959912*G0_2_0 + 0.121346149991983*G0_2_1 + 0.092966808461599*G0_2_2;
    A[1778] = -0.0880738185425698*G0_0_0 + 0.0459941052388964*G0_0_1 + 0.00913358118219177*G0_0_2 + 0.0459941052388951*G0_1_0 + 0.0234863516113503*G0_1_1 - 0.0812236326559233*G0_1_2 + 0.0091335811821915*G0_2_0 - 0.0812236326559236*G0_2_1 - 0.185933616923199*G0_2_2;
    A[1779] = 0.092966808461598*G0_0_0 + 0.121346149991982*G0_0_1 + 0.0606730749959909*G0_0_2 + 0.12134614999198*G0_1_0 + 0.149725491522363*G0_1_1 + 0.0748627457611805*G0_1_2 + 0.0606730749959896*G0_2_0 + 0.0748627457611824*G0_2_1 + 0.0195719596761254*G0_2_2;
    A[1780] = 0.0929668084616017*G0_0_0 - 0.270093043530545*G0_0_1 - 0.135046521765273*G0_0_2 - 0.270093043530539*G0_1_0 - 0.540186087061084*G0_1_1 - 0.270093043530542*G0_1_2 - 0.13504652176527*G0_2_0 - 0.270093043530542*G0_2_1 + 0.0929668084615994*G0_2_2;
    A[1781] = -0.185933616923202*G0_0_0 - 0.0812236326559258*G0_0_1 + 0.00913358118219323*G0_0_2 - 0.0812236326559241*G0_1_0 + 0.0234863516113502*G0_1_1 + 0.0459941052388989*G0_1_2 + 0.00913358118219186*G0_2_0 + 0.0459941052388953*G0_2_1 - 0.0880738185425676*G0_2_2;
    A[1782] = -0.0228339529554802*G0_0_0 - 0.0120693751336114*G0_0_1 - 0.0241387502672215*G0_0_2 - 0.0120693751336111*G0_1_0 - 0.0127217737894824*G0_1_2 - 0.0241387502672215*G0_2_0 - 0.012721773789481*G0_2_1 - 0.0254435475789632*G0_2_2;
    A[1783] = -0.00489298991903193*G0_0_0 + 0.0141896707651915*G0_0_1 + 0.0283793415303838*G0_0_2 + 0.0141896707651918*G0_1_0 + 0.0283793415303828*G0_1_2 + 0.0283793415303829*G0_2_0 + 0.0283793415303823*G0_2_1 + 0.0567586830607663*G0_2_2;
    A[1784] = -0.0228339529554811*G0_0_0 + 0.0368605240567038*G0_0_1 + 0.0737210481134086*G0_0_2 + 0.0368605240567035*G0_1_0 + 0.104709984267275*G0_1_2 + 0.0737210481134081*G0_2_0 + 0.104709984267276*G0_2_1 + 0.209419968534551*G0_2_2;
    A[1785] = 0.0195719596761264*G0_0_0 - 0.0102752788299658*G0_0_1 + 0.00146789697570992*G0_0_2 - 0.0102752788299656*G0_1_0 - 0.0181040627004156*G0_1_2 + 0.00146789697570943*G0_2_0 - 0.0181040627004168*G0_2_1 - 0.0166361657247067*G0_2_2;
    A[1786] = 0.0195719596761267*G0_0_0 + 0.0141896707651917*G0_0_1 + 0.0748627457611823*G0_0_2 + 0.0141896707651917*G0_1_0 + 0.0283793415303847*G0_1_2 + 0.0748627457611834*G0_2_0 + 0.0283793415303825*G0_2_1 + 0.149725491522365*G0_2_2;
    A[1787] = 0.0260959462348335*G0_0_0 - 0.0120693751336127*G0_0_1 + 0.0198981590040604*G0_0_2 - 0.0120693751336118*G0_1_0 - 0.012721773789483*G0_1_2 + 0.0198981590040604*G0_2_0 - 0.0127217737894824*G0_2_1 + 0.0137003717732884*G0_2_2;
    A[1788] = 0.146789697570946*G0_0_0 + 0.0929668084615975*G0_0_1 + 0.141896707651913*G0_0_2 + 0.0929668084615992*G0_1_0 + 0.092966808461599*G0_1_2 + 0.141896707651914*G0_2_0 + 0.092966808461599*G0_2_1 + 0.146789697570946*G0_2_2;
    A[1789] = 0.146789697570947*G0_0_0 - 0.102752788299663*G0_0_1 - 0.298472385060926*G0_0_2 - 0.102752788299661*G0_1_0 - 0.298472385060925*G0_1_2 - 0.298472385060923*G0_2_0 - 0.298472385060925*G0_2_1 - 0.782878387045049*G0_2_2;
    A[1790] = -0.0489298991903142*G0_0_0 + 0.0929668084616022*G0_0_1 - 0.029357939514187*G0_0_2 + 0.0929668084616004*G0_1_0 + 0.0929668084616*G0_1_2 - 0.0293579395141901*G0_2_0 + 0.0929668084616006*G0_2_1 - 0.0489298991903142*G0_2_2;
    A[1791] = -0.782878387045054*G0_0_0 - 0.298472385060925*G0_0_1 - 0.298472385060928*G0_0_2 - 0.298472385060927*G0_1_0 - 0.102752788299665*G0_1_2 - 0.298472385060925*G0_2_0 - 0.102752788299662*G0_2_1 + 0.146789697570945*G0_2_2;
    A[1792] = -0.00946499969937453*G0_0_0 - 0.00946499969937445*G0_0_1 - 0.00946499969937447*G0_0_2 - 0.00946499969937451*G0_1_0 - 0.00946499969937447*G0_1_1 - 0.00946499969937451*G0_1_2 - 0.00946499969937455*G0_2_0 - 0.00946499969937454*G0_2_1 - 0.00946499969937461*G0_2_2;
    A[1793] = 0.00946499969937495*G0_0_0 - 0.00227817610630087*G0_1_0 - 0.00227817610630055*G0_2_0;
    A[1794] = -0.00472467106581679*G0_0_1 - 0.0345719095719091*G0_1_1 - 0.00472467106581681*G0_2_1;
    A[1795] = -0.00472467106581684*G0_0_2 - 0.00472467106581711*G0_1_2 - 0.0345719095719092*G0_2_2;
    A[1796] = -0.0493213383838381*G0_0_1 - 0.0429604514890967*G0_0_2 + 0.127315597693201*G0_1_1 - 0.000685018588664125*G0_1_2 - 0.00704590548340609*G0_2_1 - 0.0429604514890974*G0_2_2;
    A[1797] = 0.167731694424402*G0_0_1 + 0.0777006799142204*G0_0_2 - 0.166948816037357*G0_1_1 - 0.00616516729798032*G0_1_2 + 0.0838658472122013*G0_2_1 + 0.182410664181497*G0_2_2;
    A[1798] = 0.0777006799142207*G0_0_1 + 0.167731694424402*G0_0_2 + 0.182410664181496*G0_1_1 + 0.0838658472122008*G0_1_2 - 0.00616516729798013*G0_2_1 - 0.166948816037357*G0_2_2;
    A[1799] = -0.0429604514890966*G0_0_1 - 0.0493213383838379*G0_0_2 - 0.0429604514890973*G0_1_1 - 0.00704590548340548*G0_1_2 - 0.000685018588664004*G0_2_1 + 0.127315597693201*G0_2_2;
    A[1800] = -0.0292600797158094*G0_0_0 - 0.00329461321214772*G0_0_2 + 0.0142875305635721*G0_1_0 + 0.00254435475789672*G0_1_2 + 0.00844856259352676*G0_2_0 + 0.00254435475789699*G0_2_2;
    A[1801] = 0.0189848008858427*G0_0_0 + 0.00949240044292081*G0_0_2 - 0.037773882174924*G0_1_0 - 0.00939454064454063*G0_1_2 - 0.0188869410874614*G0_2_0 - 0.0155923278753151*G0_2_2;
    A[1802] = -0.00352295274170334*G0_0_0 - 0.00649136662591548*G0_0_2 - 0.108232937008977*G0_1_0 - 0.0808321934624007*G0_1_2 - 0.0338921101724927*G0_2_0 - 0.043645470077761*G0_2_2;
    A[1803] = -0.0429604514890967*G0_0_0 - 0.0422754329004325*G0_0_2 - 0.0429604514890974*G0_1_0 - 0.0845508658008651*G0_1_2 - 0.000685018588664114*G0_2_0 + 0.0439716694056965*G0_2_2;
    A[1804] = -0.0292600797158093*G0_0_0 - 0.00329461321214779*G0_0_1 + 0.0084485625935274*G0_1_0 + 0.00254435475789686*G0_1_1 + 0.0142875305635715*G0_2_0 + 0.00254435475789687*G0_2_1;
    A[1805] = 0.0189848008858429*G0_0_0 + 0.00949240044292089*G0_0_1 - 0.018886941087461*G0_1_0 - 0.0155923278753145*G0_1_1 - 0.0377738821749227*G0_2_0 - 0.00939454064454061*G0_2_1;
    A[1806] = -0.00352295274170306*G0_0_0 - 0.00649136662591493*G0_0_1 - 0.0338921101724919*G0_1_0 - 0.04364547007776*G0_1_1 - 0.108232937008978*G0_2_0 - 0.0808321934624007*G0_2_1;
    A[1807] = -0.0429604514890969*G0_0_0 - 0.0422754329004326*G0_0_1 - 0.000685018588664207*G0_1_0 + 0.0439716694056962*G0_1_1 - 0.0429604514890975*G0_2_0 - 0.084550865800866*G0_2_1;
    A[1808] = 0.0292600797158087*G0_0_0 + 0.0292600797158088*G0_0_1 + 0.0259654665036605*G0_0_2 + 0.0292600797158089*G0_1_0 + 0.0292600797158089*G0_1_1 + 0.0259654665036604*G0_1_2 + 0.025965466503661*G0_2_0 + 0.025965466503661*G0_2_1 + 0.0226708532915128*G0_2_2;
    A[1809] = -0.0189848008858423*G0_0_0 - 0.0189848008858426*G0_0_1 - 0.00949240044292069*G0_0_2 - 0.0189848008858419*G0_1_0 - 0.0189848008858421*G0_1_1 - 0.00949240044292049*G0_1_2 - 0.00949240044292219*G0_2_0 - 0.00949240044292226*G0_2_1 - 0.000652398655870942*G0_2_2;
    A[1810] = 0.00352295274170252*G0_0_0 + 0.00352295274170258*G0_0_1 - 0.00296841388421322*G0_0_2 + 0.00352295274170252*G0_1_0 + 0.00352295274170239*G0_1_1 - 0.00296841388421347*G0_1_2 - 0.00296841388421246*G0_2_0 - 0.00296841388421235*G0_2_1 - 0.000652398655870846*G0_2_2;
    A[1811] = 0.0429604514890967*G0_0_0 + 0.0429604514890966*G0_0_1 + 0.000685018588664732*G0_0_2 + 0.0429604514890974*G0_1_0 + 0.0429604514890976*G0_1_1 + 0.000685018588665179*G0_1_2 + 0.000685018588664166*G0_2_0 + 0.000685018588664061*G0_2_1 + 0.0226708532915129*G0_2_2;
    A[1812] = 0.0292600797158088*G0_0_0 + 0.0259654665036604*G0_0_1 + 0.0292600797158089*G0_0_2 + 0.0259654665036609*G0_1_0 + 0.0226708532915127*G0_1_1 + 0.0259654665036611*G0_1_2 + 0.0292600797158087*G0_2_0 + 0.0259654665036605*G0_2_1 + 0.029260079715809*G0_2_2;
    A[1813] = -0.0189848008858421*G0_0_0 - 0.0094924004429205*G0_0_1 - 0.0189848008858423*G0_0_2 - 0.0094924004429214*G0_1_0 - 0.000652398655870895*G0_1_1 - 0.00949240044292179*G0_1_2 - 0.0189848008858417*G0_2_0 - 0.0094924004429205*G0_2_1 - 0.0189848008858421*G0_2_2;
    A[1814] = 0.00352295274170262*G0_0_0 - 0.00296841388421336*G0_0_1 + 0.00352295274170264*G0_0_2 - 0.00296841388421169*G0_1_0 - 0.000652398655870477*G0_1_1 - 0.00296841388421157*G0_1_2 + 0.00352295274170159*G0_2_0 - 0.00296841388421328*G0_2_1 + 0.00352295274170182*G0_2_2;
    A[1815] = 0.0429604514890967*G0_0_0 + 0.000685018588664785*G0_0_1 + 0.0429604514890966*G0_0_2 + 0.000685018588664178*G0_1_0 + 0.0226708532915124*G0_1_1 + 0.000685018588664179*G0_1_2 + 0.0429604514890974*G0_2_0 + 0.000685018588664701*G0_2_1 + 0.0429604514890972*G0_2_2;
    A[1816] = 0.00329461321214846*G0_0_1 + 0.00329461321214856*G0_0_2 + 0.00329461321214802*G0_1_0 + 0.00658922642429666*G0_1_1 + 0.00658922642429675*G0_1_2 + 0.0032946132121482*G0_2_0 + 0.00658922642429651*G0_2_1 + 0.00658922642429664*G0_2_2;
    A[1817] = 0.00257697469068997*G0_0_1 + 0.00257697469068985*G0_0_2 + 0.00257697469068942*G0_1_0 + 0.00450155072550873*G0_1_1 + 0.00450155072550861*G0_1_2 + 0.0025769746906894*G0_2_0 + 0.00450155072550862*G0_2_1 + 0.00450155072550854*G0_2_2;
    A[1818] = 0.00257697469069013*G0_0_1 + 0.0025769746906901*G0_0_2 + 0.0025769746906907*G0_1_0 - 0.0105036183595202*G0_1_1 - 0.0105036183595203*G0_1_2 + 0.00257697469069072*G0_2_0 - 0.0105036183595207*G0_2_1 - 0.0105036183595207*G0_2_2;
    A[1819] = 0.00329461321214764*G0_0_1 + 0.00329461321214771*G0_0_2 + 0.00329461321214717*G0_1_0 - 0.00254435475789705*G0_1_1 - 0.00254435475789693*G0_1_2 + 0.00329461321214677*G0_2_0 - 0.00254435475789716*G0_2_1 - 0.00254435475789711*G0_2_2;
    A[1820] = 0.113517366121532*G0_0_0 + 0.0127217737894814*G0_0_1 + 0.0127217737894816*G0_0_2 - 0.0851380245911491*G0_1_0 - 0.0137003717732888*G0_1_1 - 0.019898159004062*G0_1_2 - 0.0851380245911487*G0_2_0 - 0.0198981590040628*G0_2_1 - 0.0137003717732884*G0_2_2;
    A[1821] = -0.149725491522365*G0_0_0 - 0.0748627457611826*G0_0_1 - 0.0283793415303826*G0_0_2 + 0.0278900425384811*G0_1_0 + 0.065566064915024*G0_1_1 - 0.0748627457611803*G0_1_2 + 0.177126235068943*G0_2_0 + 0.0278900425384809*G0_2_1 - 0.149725491522363*G0_2_2;
    A[1822] = -0.0234863516113515*G0_0_0 + 0.0225077536275456*G0_0_1 - 0.104709984267275*G0_0_2 - 0.0459941052388979*G0_1_0 - 0.268135847562929*G0_1_1 - 0.0737210481134094*G0_1_2 + 0.0812236326559232*G0_2_0 + 0.180714427676232*G0_2_1 - 0.209419968534552*G0_2_2;
    A[1823] = -0.149725491522367*G0_0_0 - 0.0283793415303845*G0_0_1 - 0.0748627457611842*G0_0_2 + 0.177126235068943*G0_1_0 - 0.149725491522365*G0_1_1 + 0.0278900425384793*G0_1_2 + 0.0278900425384769*G0_2_0 - 0.0748627457611848*G0_2_1 + 0.0655660649150203*G0_2_2;
    A[1824] = 0.540186087061083*G0_0_0 + 0.270093043530541*G0_0_1 + 0.270093043530542*G0_0_2 + 0.270093043530541*G0_1_0 + 0.726119703984281*G0_1_1 + 0.270093043530541*G0_1_2 + 0.270093043530542*G0_2_0 + 0.270093043530541*G0_2_1 + 0.726119703984283*G0_2_2;
    A[1825] = -0.0234863516113516*G0_0_0 - 0.104709984267276*G0_0_1 + 0.0225077536275445*G0_0_2 + 0.0812236326559237*G0_1_0 - 0.20941996853455*G0_1_1 + 0.180714427676233*G0_1_2 - 0.0459941052388966*G0_2_0 - 0.0737210481134091*G0_2_1 - 0.26813584756293*G0_2_2;
    A[1826] = -0.113517366121532*G0_0_0 - 0.10079559233205*G0_0_1 - 0.100795592332051*G0_0_2 - 0.100795592332048*G0_1_0 - 0.088073818542568*G0_1_1 - 0.0887262171984384*G0_1_2 - 0.10079559233205*G0_2_0 - 0.088726217198439*G0_2_1 - 0.0880738185425689*G0_2_2;
    A[1827] = 0.149725491522365*G0_0_0 + 0.0748627457611825*G0_0_1 + 0.121346149991981*G0_0_2 + 0.0748627457611815*G0_1_0 + 0.0195719596761258*G0_1_1 + 0.0606730749959905*G0_1_2 + 0.121346149991982*G0_2_0 + 0.0606730749959902*G0_2_1 + 0.0929668084615995*G0_2_2;
    A[1828] = 0.0234863516113511*G0_0_0 + 0.0459941052388968*G0_0_1 - 0.0812236326559231*G0_0_2 + 0.0459941052388975*G0_1_0 - 0.0880738185425679*G0_1_1 + 0.0091335811821918*G0_1_2 - 0.0812236326559236*G0_2_0 + 0.00913358118219386*G0_2_1 - 0.185933616923199*G0_2_2;
    A[1829] = 0.149725491522364*G0_0_0 + 0.121346149991982*G0_0_1 + 0.074862745761183*G0_0_2 + 0.121346149991982*G0_1_0 + 0.0929668084615988*G0_1_1 + 0.0606730749959906*G0_1_2 + 0.0748627457611819*G0_2_0 + 0.0606730749959928*G0_2_1 + 0.0195719596761268*G0_2_2;
    A[1830] = -0.540186087061083*G0_0_0 - 0.270093043530541*G0_0_1 - 0.270093043530541*G0_0_2 - 0.270093043530541*G0_1_0 + 0.0929668084616004*G0_1_1 - 0.13504652176527*G0_1_2 - 0.270093043530541*G0_2_0 - 0.135046521765272*G0_2_1 + 0.0929668084615992*G0_2_2;
    A[1831] = 0.0234863516113512*G0_0_0 - 0.0812236326559236*G0_0_1 + 0.0459941052388966*G0_0_2 - 0.0812236326559241*G0_1_0 - 0.1859336169232*G0_1_1 + 0.00913358118219207*G0_1_2 + 0.0459941052388963*G0_2_0 + 0.00913358118219217*G0_2_1 - 0.0880738185425679*G0_2_2;
    A[1832] = -0.0127217737894818*G0_0_1 - 0.0120693751336123*G0_0_2 - 0.0127217737894821*G0_1_0 - 0.0254435475789624*G0_1_1 - 0.0241387502672229*G0_1_2 - 0.0120693751336118*G0_2_0 - 0.0241387502672227*G0_2_1 - 0.0228339529554821*G0_2_2;
    A[1833] = -0.0181040627004174*G0_0_1 - 0.0102752788299664*G0_0_2 - 0.0181040627004169*G0_1_0 - 0.0166361657247076*G0_1_1 + 0.00146789697570931*G0_1_2 - 0.0102752788299664*G0_2_0 + 0.00146789697570921*G0_2_1 + 0.0195719596761261*G0_2_2;
    A[1834] = -0.0127217737894812*G0_0_1 - 0.0120693751336109*G0_0_2 - 0.0127217737894822*G0_1_0 + 0.0137003717732891*G0_1_1 + 0.0198981590040614*G0_1_2 - 0.0120693751336098*G0_2_0 + 0.019898159004063*G0_2_1 + 0.0260959462348362*G0_2_2;
    A[1835] = 0.0283793415303824*G0_0_1 + 0.0141896707651925*G0_0_2 + 0.0283793415303829*G0_1_0 + 0.0567586830607667*G0_1_1 + 0.0283793415303833*G0_1_2 + 0.0141896707651926*G0_2_0 + 0.028379341530382*G0_2_1 - 0.0048929899190303*G0_2_2;
    A[1836] = 0.0283793415303846*G0_0_1 + 0.0141896707651918*G0_0_2 + 0.0283793415303832*G0_1_0 + 0.149725491522365*G0_1_1 + 0.0748627457611824*G0_1_2 + 0.0141896707651911*G0_2_0 + 0.0748627457611848*G0_2_1 + 0.0195719596761261*G0_2_2;
    A[1837] = 0.104709984267275*G0_0_1 + 0.0368605240567039*G0_0_2 + 0.104709984267274*G0_1_0 + 0.209419968534549*G0_1_1 + 0.0737210481134082*G0_1_2 + 0.036860524056705*G0_2_0 + 0.073721048113409*G0_2_1 - 0.0228339529554808*G0_2_2;
    A[1838] = -0.0120693751336124*G0_0_1 - 0.0127217737894819*G0_0_2 - 0.0120693751336107*G0_1_0 - 0.0228339529554813*G0_1_1 - 0.024138750267221*G0_1_2 - 0.0127217737894824*G0_2_0 - 0.0241387502672231*G0_2_1 - 0.0254435475789638*G0_2_2;
    A[1839] = -0.0102752788299664*G0_0_1 - 0.0181040627004174*G0_0_2 - 0.0102752788299674*G0_1_0 + 0.0195719596761255*G0_1_1 + 0.00146789697570916*G0_1_2 - 0.0181040627004167*G0_2_0 + 0.00146789697571017*G0_2_1 - 0.0166361657247068*G0_2_2;
    A[1840] = -0.0120693751336109*G0_0_1 - 0.0127217737894814*G0_0_2 - 0.0120693751336103*G0_1_0 + 0.0260959462348349*G0_1_1 + 0.0198981590040619*G0_1_2 - 0.0127217737894813*G0_2_0 + 0.0198981590040616*G0_2_1 + 0.0137003717732883*G0_2_2;
    A[1841] = 0.0141896707651923*G0_0_1 + 0.0283793415303831*G0_0_2 + 0.0141896707651917*G0_1_0 - 0.00489298991903103*G0_1_1 + 0.0283793415303831*G0_1_2 + 0.028379341530383*G0_2_0 + 0.0283793415303825*G0_2_1 + 0.0567586830607657*G0_2_2;
    A[1842] = 0.0141896707651914*G0_0_1 + 0.0283793415303828*G0_0_2 + 0.0141896707651909*G0_1_0 + 0.0195719596761259*G0_1_1 + 0.0748627457611807*G0_1_2 + 0.0283793415303816*G0_2_0 + 0.0748627457611818*G0_2_1 + 0.149725491522364*G0_2_2;
    A[1843] = 0.0368605240567041*G0_0_1 + 0.104709984267275*G0_0_2 + 0.0368605240567035*G0_1_0 - 0.0228339529554807*G0_1_1 + 0.0737210481134091*G0_1_2 + 0.104709984267276*G0_2_0 + 0.0737210481134101*G0_2_1 + 0.209419968534551*G0_2_2;
    A[1844] = 0.0929668084615995*G0_0_1 + 0.0929668084615987*G0_0_2 + 0.0929668084615975*G0_1_0 + 0.146789697570944*G0_1_1 + 0.141896707651913*G0_1_2 + 0.0929668084616013*G0_2_0 + 0.141896707651917*G0_2_1 + 0.146789697570947*G0_2_2;
    A[1845] = 0.0929668084615999*G0_0_1 + 0.0929668084616013*G0_0_2 + 0.0929668084615997*G0_1_0 - 0.0489298991903167*G0_1_1 - 0.0293579395141886*G0_1_2 + 0.0929668084615972*G0_2_0 - 0.0293579395141905*G0_2_1 - 0.0489298991903136*G0_2_2;
    A[1846] = -0.102752788299663*G0_0_1 - 0.298472385060925*G0_0_2 - 0.102752788299663*G0_1_0 + 0.146789697570948*G0_1_1 - 0.298472385060924*G0_1_2 - 0.298472385060925*G0_2_0 - 0.298472385060927*G0_2_1 - 0.782878387045048*G0_2_2;
    A[1847] = -0.298472385060923*G0_0_1 - 0.102752788299662*G0_0_2 - 0.298472385060924*G0_1_0 - 0.782878387045047*G0_1_1 - 0.298472385060925*G0_1_2 - 0.102752788299659*G0_2_0 - 0.298472385060922*G0_2_1 + 0.146789697570946*G0_2_2;
    A[1848] = -0.0137421252872641*G0_0_0 - 0.0137421252872641*G0_0_1 - 0.0137421252872641*G0_0_2 - 0.0137421252872642*G0_1_0 - 0.0137421252872642*G0_1_1 - 0.0137421252872642*G0_1_2 - 0.013742125287264*G0_2_0 - 0.013742125287264*G0_2_1 - 0.0137421252872639*G0_2_2;
    A[1849] = 0.0137421252872639*G0_0_0 - 0.00517743573299143*G0_1_0 - 0.0051774357329916*G0_2_0;
    A[1850] = -0.00517743573299099*G0_0_1 + 0.0137421252872642*G0_1_1 - 0.00517743573299102*G0_2_1;
    A[1851] = 0.0143945239431351*G0_0_2 + 0.0143945239431351*G0_1_2 + 0.0479930547204847*G0_2_2;
    A[1852] = 0.0477555816097472*G0_0_1 + 0.0118736555368503*G0_0_2 - 0.0641307878721069*G0_1_1 - 0.00939454064454004*G0_1_2 + 0.0264873854283582*G0_2_1 + 0.011873655536851*G0_2_2;
    A[1853] = -0.0823327103709038*G0_0_1 + 0.00182671623643726*G0_0_2 + 0.127087258163646*G0_1_1 + 0.0377738821749228*G0_1_2 - 0.0463855444324205*G0_2_1 - 0.0510175748891035*G0_2_2;
    A[1854] = 0.146659217839771*G0_0_1 + 0.0109602974186313*G0_0_2 - 0.124738623002511*G0_1_1 - 0.062369311501255*G0_1_2 + 0.0733296089198869*G0_2_1 + 0.132306447410614*G0_2_2;
    A[1855] = 0.124738623002511*G0_0_1 + 0.192588083213081*G0_0_2 + 0.124738623002511*G0_1_1 + 0.0962940416065408*G0_1_2 + 0.0284445813959692*G0_2_1 - 0.0644569872000432*G0_2_2;
    A[1856] = -0.0641307878721065*G0_0_0 - 0.0093945406445406*G0_0_2 + 0.0477555816097479*G0_1_0 + 0.0118736555368495*G0_1_2 + 0.0264873854283586*G0_2_0 + 0.0118736555368497*G0_2_2;
    A[1857] = 0.127087258163646*G0_0_0 + 0.037773882174924*G0_0_2 - 0.0823327103709042*G0_1_0 + 0.00182671623643879*G0_1_2 - 0.0463855444324203*G0_2_0 - 0.0510175748891019*G0_2_2;
    A[1858] = -0.12473862300251*G0_0_0 - 0.0623693115012553*G0_0_2 + 0.146659217839773*G0_1_0 + 0.0109602974186311*G0_1_2 + 0.0733296089198884*G0_2_0 + 0.132306447410614*G0_2_2;
    A[1859] = 0.124738623002511*G0_0_0 + 0.096294041606541*G0_0_2 + 0.124738623002511*G0_1_0 + 0.192588083213082*G0_1_2 + 0.0284445813959692*G0_2_0 - 0.0644569872000427*G0_2_2;
    A[1860] = -0.0187890812890811*G0_0_0 - 0.00939454064454058*G0_0_1 + 0.0212681961813902*G0_1_0 + 0.0118736555368495*G0_1_1 + 0.042536392362781*G0_2_0 + 0.0118736555368498*G0_2_1;
    A[1861] = 0.0148746893538559*G0_0_0 + 0.0149399292194425*G0_0_1 + 0.00123955744615484*G0_1_0 + 0.00117431758056771*G0_1_1 + 0.0403182369328202*G0_2_0 + 0.0540186087061081*G0_2_1;
    A[1862] = 0.00117431758056757*G0_0_0 + 0.00123955744615544*G0_0_1 + 0.0149399292194427*G0_1_0 + 0.0148746893538557*G0_1_1 + 0.054018608706108*G0_2_0 + 0.0403182369328203*G0_2_1;
    A[1863] = 0.0118736555368501*G0_0_0 + 0.0212681961813898*G0_0_1 - 0.00939454064454017*G0_1_0 - 0.018789081289081*G0_1_1 + 0.0118736555368509*G0_2_0 + 0.0425363923627813*G0_2_1;
    A[1864] = 0.0641307878721069*G0_0_0 + 0.064130787872107*G0_0_1 + 0.0547362472275667*G0_0_2 + 0.0641307878721072*G0_1_0 + 0.0641307878721072*G0_1_1 + 0.0547362472275667*G0_1_2 + 0.0547362472275664*G0_2_0 + 0.0547362472275664*G0_2_1 + 0.0453417065830253*G0_2_2;
    A[1865] = -0.127087258163646*G0_0_0 - 0.127087258163646*G0_0_1 - 0.0893133759887233*G0_0_2 - 0.127087258163647*G0_1_0 - 0.127087258163647*G0_1_1 - 0.0893133759887232*G0_1_2 - 0.0893133759887223*G0_2_0 - 0.0893133759887222*G0_2_1 - 0.0665446628988283*G0_2_2;
    A[1866] = 0.124738623002511*G0_0_0 + 0.124738623002511*G0_0_1 + 0.0623693115012562*G0_0_2 + 0.124738623002512*G0_1_0 + 0.124738623002512*G0_1_1 + 0.0623693115012562*G0_1_2 + 0.0623693115012556*G0_2_0 + 0.0623693115012555*G0_2_1 + 0.0476251018785728*G0_2_2;
    A[1867] = -0.124738623002511*G0_0_0 - 0.124738623002511*G0_0_1 - 0.0284445813959705*G0_0_2 - 0.124738623002511*G0_1_0 - 0.124738623002511*G0_1_1 - 0.0284445813959706*G0_1_2 - 0.0284445813959692*G0_2_0 - 0.028444581395969*G0_2_1 - 0.0606730749959909*G0_2_2;
    A[1868] = 0.018789081289081*G0_0_0 + 0.00939454064454068*G0_0_1 + 0.0187890812890809*G0_0_2 + 0.0093945406445409*G0_1_0 + 0.00939454064454071*G0_1_2 + 0.0187890812890814*G0_2_0 + 0.00939454064454039*G0_2_1 + 0.0187890812890813*G0_2_2;
    A[1869] = -0.0148746893538557*G0_0_0 + 6.52398655871534e-05*G0_0_1 - 0.0148746893538557*G0_0_2 + 6.52398655868821e-05*G0_1_0 + 6.52398655870059e-05*G0_1_2 - 0.0148746893538565*G0_2_0 + 6.52398655871583e-05*G0_2_1 - 0.0148746893538564*G0_2_2;
    A[1870] = -0.0011743175805675*G0_0_0 + 6.52398655874842e-05*G0_0_1 - 0.00117431758056751*G0_0_2 + 6.52398655873305e-05*G0_1_0 + 6.52398655873479e-05*G0_1_2 - 0.00117431758056626*G0_2_0 + 6.52398655875217e-05*G0_2_1 - 0.0011743175805662*G0_2_2;
    A[1871] = -0.0118736555368501*G0_0_0 + 0.00939454064454002*G0_0_1 - 0.01187365553685*G0_0_2 + 0.00939454064454023*G0_1_0 + 0.00939454064454018*G0_1_2 - 0.0118736555368508*G0_2_0 + 0.00939454064454009*G0_2_1 - 0.0118736555368508*G0_2_2;
    A[1872] = 0.00939454064454059*G0_0_1 + 0.00939454064454041*G0_0_2 + 0.00939454064454078*G0_1_0 + 0.0187890812890814*G0_1_1 + 0.0187890812890812*G0_1_2 + 0.00939454064454049*G0_2_0 + 0.0187890812890817*G0_2_1 + 0.0187890812890816*G0_2_2;
    A[1873] = 6.52398655870967e-05*G0_0_1 + 6.52398655871972e-05*G0_0_2 + 6.52398655871062e-05*G0_1_0 - 0.0148746893538558*G0_1_1 - 0.0148746893538556*G0_1_2 + 6.523986558709e-05*G0_2_0 - 0.0148746893538559*G0_2_1 - 0.0148746893538558*G0_2_2;
    A[1874] = 6.52398655869567e-05*G0_0_1 + 6.52398655869571e-05*G0_0_2 + 6.52398655869303e-05*G0_1_0 - 0.00117431758056774*G0_1_1 - 0.00117431758056775*G0_1_2 + 6.52398655869323e-05*G0_2_0 - 0.00117431758056753*G0_2_1 - 0.0011743175805676*G0_2_2;
    A[1875] = 0.00939454064454069*G0_0_1 + 0.00939454064454066*G0_0_2 + 0.00939454064454059*G0_1_0 - 0.0118736555368494*G0_1_1 - 0.0118736555368494*G0_1_2 + 0.00939454064454087*G0_2_0 - 0.0118736555368497*G0_2_1 - 0.0118736555368496*G0_2_2;
    A[1876] = 0.105688582251081*G0_0_0 + 0.052844291125541*G0_0_1 + 0.0300103381700596*G0_0_2 + 0.000652398655870142*G0_1_0 + 0.0534966897814109*G0_1_1 + 0.0528442911255397*G0_1_2 - 0.0743734467692806*G0_2_0 + 0.000652398655871044*G0_2_1 + 0.105688582251081*G0_2_2;
    A[1877] = -0.0137003717732872*G0_0_0 - 0.0198981590040616*G0_0_1 + 0.0127217737894826*G0_0_2 - 0.0198981590040619*G0_1_0 - 0.0137003717732885*G0_1_1 + 0.012721773789482*G0_1_2 - 0.085138024591149*G0_2_0 - 0.0851380245911497*G0_2_1 + 0.113517366121532*G0_2_2;
    A[1878] = 0.0534966897814102*G0_0_0 + 0.000652398655870844*G0_0_1 + 0.0528442911255401*G0_0_2 + 0.0528442911255408*G0_1_0 + 0.105688582251081*G0_1_1 + 0.0300103381700602*G0_1_2 + 0.000652398655871146*G0_2_0 - 0.0743734467692795*G0_2_1 + 0.105688582251082*G0_2_2;
    A[1879] = -0.209419968534548*G0_0_0 - 0.104709984267275*G0_0_1 - 0.0737210481134076*G0_0_2 + 0.0812236326559257*G0_1_0 - 0.02348635161135*G0_1_1 - 0.0459941052388954*G0_1_2 + 0.180714427676237*G0_2_0 + 0.0225077536275484*G0_2_1 - 0.268135847562928*G0_2_2;
    A[1880] = -0.0234863516113516*G0_0_0 + 0.0812236326559237*G0_0_1 - 0.0459941052388966*G0_0_2 - 0.104709984267276*G0_1_0 - 0.20941996853455*G0_1_1 - 0.073721048113409*G0_1_2 + 0.0225077536275444*G0_2_0 + 0.180714427676233*G0_2_1 - 0.26813584756293*G0_2_2;
    A[1881] = 0.542795681684565*G0_0_0 + 0.271397840842284*G0_0_1 + 0.271397840842283*G0_0_2 + 0.271397840842284*G0_1_0 + 0.542795681684568*G0_1_1 + 0.271397840842284*G0_1_2 + 0.271397840842283*G0_2_0 + 0.271397840842284*G0_2_1 + 0.64717946662391*G0_2_2;
    A[1882] = -0.105688582251082*G0_0_0 - 0.0528442911255411*G0_0_1 - 0.0756782440810213*G0_0_2 - 0.0528442911255425*G0_1_0 - 0.0378391220405117*G0_1_2 - 0.0756782440810218*G0_2_0 - 0.037839122040511*G0_2_1 - 0.0456679059109612*G0_2_2;
    A[1883] = 0.0137003717732888*G0_0_0 - 0.00619778723077358*G0_0_1 + 0.0264221455627701*G0_0_2 - 0.006197787230772*G0_1_0 - 0.00554538857490194*G0_1_2 + 0.0264221455627702*G0_2_0 - 0.00554538857490211*G0_2_1 + 0.0391439193522518*G0_2_2;
    A[1884] = -0.0534966897814107*G0_0_0 - 0.0528442911255402*G0_0_1 - 0.000652398655870259*G0_0_2 - 0.0528442911255409*G0_1_0 - 0.0378391220405107*G0_1_2 - 0.000652398655871378*G0_2_0 - 0.037839122040512*G0_2_1 + 0.0521918924696696*G0_2_2;
    A[1885] = 0.209419968534552*G0_0_0 + 0.104709984267276*G0_0_1 + 0.135698920421142*G0_0_2 + 0.104709984267277*G0_1_0 + 0.0678494602105727*G0_1_2 + 0.135698920421142*G0_2_0 + 0.0678494602105703*G0_2_1 + 0.039143919352253*G0_2_2;
    A[1886] = 0.0234863516113517*G0_0_0 + 0.104709984267275*G0_0_1 - 0.0225077536275453*G0_0_2 + 0.104709984267275*G0_1_0 + 0.0678494602105703*G0_1_2 - 0.0225077536275444*G0_2_0 + 0.0678494602105726*G0_2_1 - 0.156575677409009*G0_2_2;
    A[1887] = -0.542795681684566*G0_0_0 - 0.271397840842283*G0_0_1 - 0.271397840842283*G0_0_2 - 0.271397840842283*G0_1_0 - 0.135698920421142*G0_1_2 - 0.271397840842283*G0_2_0 - 0.135698920421143*G0_2_1 + 0.05219189246967*G0_2_2;
    A[1888] = -0.0528442911255412*G0_0_1 - 0.0378391220405107*G0_0_2 - 0.0528442911255413*G0_1_0 - 0.105688582251083*G0_1_1 - 0.0756782440810225*G0_1_2 - 0.0378391220405109*G0_2_0 - 0.0756782440810213*G0_2_1 - 0.0456679059109618*G0_2_2;
    A[1889] = -0.00619778723077224*G0_0_1 - 0.00554538857490229*G0_0_2 - 0.00619778723077282*G0_1_0 + 0.0137003717732903*G0_1_1 + 0.0264221455627708*G0_1_2 - 0.0055453885749021*G0_2_0 + 0.0264221455627718*G0_2_1 + 0.0391439193522531*G0_2_2;
    A[1890] = -0.052844291125541*G0_0_1 - 0.0378391220405108*G0_0_2 - 0.052844291125541*G0_1_0 - 0.0534966897814115*G0_1_1 - 0.000652398655871234*G0_1_2 - 0.037839122040512*G0_2_0 - 0.000652398655871365*G0_2_1 + 0.0521918924696692*G0_2_2;
    A[1891] = 0.104709984267277*G0_0_1 + 0.0678494602105719*G0_0_2 + 0.104709984267276*G0_1_0 + 0.209419968534552*G0_1_1 + 0.135698920421144*G0_1_2 + 0.0678494602105709*G0_2_0 + 0.135698920421143*G0_2_1 + 0.0391439193522529*G0_2_2;
    A[1892] = 0.104709984267274*G0_0_1 + 0.0678494602105705*G0_0_2 + 0.104709984267275*G0_1_0 + 0.0234863516113502*G0_1_1 - 0.0225077536275459*G0_1_2 + 0.0678494602105717*G0_2_0 - 0.0225077536275479*G0_2_1 - 0.156575677409011*G0_2_2;
    A[1893] = -0.271397840842284*G0_0_1 - 0.135698920421142*G0_0_2 - 0.271397840842285*G0_1_0 - 0.542795681684567*G0_1_1 - 0.271397840842284*G0_1_2 - 0.135698920421144*G0_2_0 - 0.271397840842284*G0_2_1 + 0.0521918924696704*G0_2_2;
    A[1894] = -0.0150051690850298*G0_0_1 - 0.0300103381700603*G0_0_2 - 0.0150051690850298*G0_1_0 - 0.0300103381700604*G0_1_2 - 0.0300103381700595*G0_2_0 - 0.0300103381700607*G0_2_1 - 0.06002067634012*G0_2_2;
    A[1895] = -0.000652398655870691*G0_0_1 - 0.00130479731174134*G0_0_2 - 0.000652398655871041*G0_1_0 - 0.0127217737894809*G0_1_2 - 0.00130479731174264*G0_2_0 - 0.0127217737894824*G0_2_1 - 0.0254435475789641*G0_2_2;
    A[1896] = -0.0150051690850297*G0_0_1 - 0.0300103381700599*G0_0_2 - 0.0150051690850301*G0_1_0 - 0.0528442911255401*G0_1_2 - 0.03001033817006*G0_2_0 - 0.0528442911255408*G0_2_1 - 0.105688582251081*G0_2_2;
    A[1897] = -0.000652398655871402*G0_0_1 - 0.0127217737894813*G0_0_2 - 0.000652398655870802*G0_1_0 - 0.00130479731174112*G0_1_2 - 0.0127217737894821*G0_2_0 - 0.00130479731174118*G0_2_1 - 0.0254435475789632*G0_2_2;
    A[1898] = -0.000652398655871725*G0_0_1 - 0.0127217737894823*G0_0_2 - 0.000652398655870679*G0_1_0 - 0.0127217737894819*G0_1_2 - 0.0127217737894814*G0_2_0 - 0.0127217737894818*G0_2_1 - 0.113517366121532*G0_2_2;
    A[1899] = -0.0150051690850295*G0_0_1 - 0.0528442911255403*G0_0_2 - 0.0150051690850303*G0_1_0 - 0.0300103381700604*G0_1_2 - 0.052844291125542*G0_2_0 - 0.0300103381700614*G0_2_1 - 0.105688582251083*G0_2_2;
    A[1900] = 0.0260959462348375*G0_0_1 + 0.075025845425153*G0_0_2 + 0.0260959462348394*G0_1_0 + 0.0750258454251533*G0_1_2 + 0.0750258454251523*G0_2_0 + 0.0750258454251504*G0_2_1 + 0.195719596761262*G0_2_2;
    A[1901] = 0.0260959462348341*G0_0_1 + 0.0750258454251493*G0_0_2 + 0.0260959462348327*G0_1_0 + 0.0587158790283769*G0_1_2 + 0.07502584542515*G0_2_0 + 0.0587158790283784*G0_2_1 + 0.293579395141892*G0_2_2;
    A[1902] = 0.0260959462348333*G0_0_1 + 0.0587158790283781*G0_0_2 + 0.0260959462348343*G0_1_0 + 0.0750258454251507*G0_1_2 + 0.058715879028379*G0_2_0 + 0.0750258454251507*G0_2_1 + 0.293579395141894*G0_2_2;
    A[1903] = -0.185933616923201*G0_0_1 - 0.316413348097376*G0_0_2 - 0.185933616923203*G0_1_0 - 0.316413348097377*G0_1_2 - 0.316413348097379*G0_2_0 - 0.316413348097376*G0_2_1 - 0.782878387045052*G0_2_2;
    A[1904] = 0.047993054720485*G0_0_0 + 0.047993054720485*G0_0_1 + 0.047993054720485*G0_0_2 + 0.0335985307773499*G0_1_0 + 0.0335985307773499*G0_1_1 + 0.0335985307773499*G0_1_2 + 0.03359853077735*G0_2_0 + 0.03359853077735*G0_2_1 + 0.03359853077735*G0_2_2;
    A[1905] = -0.013742125287264*G0_0_0;
    A[1906] = 0.00517743573299116*G0_0_1 + 0.0189195610202553*G0_1_1;
    A[1907] = 0.00517743573299101*G0_0_2 + 0.0189195610202551*G0_2_2;
    A[1908] = -0.0425363923627799*G0_0_1 - 0.0118736555368494*G0_0_2 - 0.0613254736518614*G0_1_1 - 0.0212681961813902*G0_1_2 - 0.0212681961813895*G0_2_1;
    A[1909] = -0.0403182369328212*G0_0_1 - 0.0540186087061088*G0_0_2 - 0.0254435475789651*G0_1_1 - 0.0390786794866656*G0_1_2 - 0.0390786794866666*G0_2_1 - 0.052844291125541*G0_2_2;
    A[1910] = -0.0540186087061072*G0_0_1 - 0.0403182369328193*G0_0_2 - 0.0528442911255396*G0_1_1 - 0.0390786794866651*G0_1_2 - 0.0390786794866641*G0_2_1 - 0.0254435475789639*G0_2_2;
    A[1911] = -0.011873655536851*G0_0_1 - 0.0425363923627817*G0_0_2 - 0.0212681961813908*G0_1_2 - 0.0212681961813914*G0_2_1 - 0.0613254736518623*G0_2_2;
    A[1912] = 0.0187890812890813*G0_0_0 + 0.00939454064454125*G0_0_2 + 0.00939454064454041*G0_2_0;
    A[1913] = -0.0148746893538566*G0_0_0 - 0.0149399292194443*G0_0_2 - 0.0149399292194431*G0_2_0 - 0.0150051690850309*G0_2_2;
    A[1914] = -0.00117431758056713*G0_0_0 - 0.00123955744615404*G0_0_2 - 0.00123955744615425*G0_2_0 - 0.00130479731174103*G0_2_2;
    A[1915] = -0.0118736555368511*G0_0_0 - 0.0212681961813914*G0_0_2 - 0.0212681961813914*G0_2_0 - 0.0306627368259318*G0_2_2;
    A[1916] = 0.0187890812890818*G0_0_0 + 0.00939454064454129*G0_0_1 + 0.0093945406445413*G0_1_0;
    A[1917] = -0.0148746893538566*G0_0_0 - 0.0149399292194436*G0_0_1 - 0.0149399292194436*G0_1_0 - 0.0150051690850304*G0_1_1;
    A[1918] = -0.00117431758056705*G0_0_0 - 0.00123955744615476*G0_0_1 - 0.00123955744615395*G0_1_0 - 0.00130479731174177*G0_1_1;
    A[1919] = -0.0118736555368494*G0_0_0 - 0.0212681961813894*G0_0_1 - 0.0212681961813902*G0_1_0 - 0.0306627368259303*G0_1_1;
    A[1920] = -0.0644569872000422*G0_0_0 - 0.0644569872000422*G0_0_1 - 0.0929015685960128*G0_0_2 - 0.257045070413124*G0_1_0 - 0.257045070413124*G0_1_1 - 0.160751028806583*G0_1_2 - 0.160751028806583*G0_2_0 - 0.160751028806583*G0_2_1 - 0.0644569872000424*G0_2_2;
    A[1921] = 0.132306447410613*G0_0_0 + 0.132306447410614*G0_0_1 + 0.0589768384907276*G0_0_2 + 0.121346149991983*G0_1_0 + 0.121346149991983*G0_1_1 + 0.194675758911869*G0_1_2 + 0.194675758911869*G0_2_0 + 0.194675758911869*G0_2_1 - 0.00339247301052801*G0_2_2;
    A[1922] = -0.0510175748891024*G0_0_0 - 0.0510175748891026*G0_0_1 - 0.00463203045668345*G0_0_2 - 0.0528442911255408*G0_1_0 - 0.0528442911255407*G0_1_1 - 0.0887914570640258*G0_1_2 - 0.0887914570640262*G0_2_0 - 0.0887914570640262*G0_2_1 + 0.0846813455320387*G0_2_2;
    A[1923] = 0.0118736555368509*G0_0_0 + 0.0118736555368509*G0_0_1 - 0.0146137298915071*G0_0_2 + 0.0212681961813909*G0_1_2 + 0.0212681961813913*G0_2_0 + 0.0212681961813913*G0_2_1 - 0.0693499771190732*G0_2_2;
    A[1924] = -0.0644569872000415*G0_0_0 - 0.0929015685960121*G0_0_1 - 0.0644569872000416*G0_0_2 - 0.160751028806583*G0_1_0 - 0.064456987200042*G0_1_1 - 0.160751028806583*G0_1_2 - 0.257045070413123*G0_2_0 - 0.160751028806583*G0_2_1 - 0.257045070413123*G0_2_2;
    A[1925] = 0.132306447410612*G0_0_0 + 0.0589768384907262*G0_0_1 + 0.132306447410612*G0_0_2 + 0.194675758911868*G0_1_0 - 0.00339247301052899*G0_1_1 + 0.194675758911868*G0_1_2 + 0.121346149991981*G0_2_0 + 0.194675758911868*G0_2_1 + 0.121346149991981*G0_2_2;
    A[1926] = -0.051017574889101*G0_0_0 - 0.00463203045668258*G0_0_1 - 0.051017574889101*G0_0_2 - 0.088791457064025*G0_1_0 + 0.0846813455320397*G0_1_1 - 0.0887914570640251*G0_1_2 - 0.0528442911255398*G0_2_0 - 0.0887914570640255*G0_2_1 - 0.0528442911255397*G0_2_2;
    A[1927] = 0.0118736555368495*G0_0_0 - 0.0146137298915077*G0_0_1 + 0.0118736555368495*G0_0_2 + 0.0212681961813903*G0_1_0 - 0.0693499771190738*G0_1_1 + 0.0212681961813904*G0_1_2 + 0.0212681961813904*G0_2_1;
    A[1928] = -0.0606730749959914*G0_0_0 - 0.0322284936000212*G0_0_1 - 0.0322284936000212*G0_0_2 - 0.0322284936000212*G0_1_0 - 0.128522535206562*G0_1_1 - 0.128522535206562*G0_1_2 - 0.0322284936000212*G0_2_0 - 0.128522535206562*G0_2_1 - 0.128522535206562*G0_2_2;
    A[1929] = 0.0476251018785743*G0_0_0 - 0.0147442096226818*G0_0_1 - 0.0147442096226817*G0_0_2 - 0.0147442096226813*G0_1_0 + 0.047625101878574*G0_1_1 + 0.0476251018785741*G0_1_2 - 0.0147442096226814*G0_2_0 + 0.0476251018785742*G0_2_1 + 0.0476251018785741*G0_2_2;
    A[1930] = -0.0665446628988294*G0_0_0 + 0.0227687130898939*G0_0_1 + 0.0227687130898938*G0_0_2 + 0.0227687130898929*G0_1_0 - 0.0150051690850299*G0_1_1 - 0.01500516908503*G0_1_2 + 0.0227687130898931*G0_2_0 - 0.01500516908503*G0_2_1 - 0.0150051690850299*G0_2_2;
    A[1931] = 0.0453417065830257*G0_0_0 - 0.00939454064454139*G0_0_1 - 0.0093945406445412*G0_0_2 - 0.00939454064454052*G0_1_0 - 0.00939454064454051*G0_2_0;
    A[1932] = -0.0600206763401199*G0_0_0 - 0.0300103381700602*G0_0_1 - 0.0300103381700597*G0_0_2 - 0.0300103381700593*G0_1_0 - 0.0150051690850285*G0_1_2 - 0.0300103381700597*G0_2_0 - 0.0150051690850304*G0_2_1;
    A[1933] = -0.0254435475789638*G0_0_0 - 0.024138750267221*G0_0_1 - 0.0127217737894823*G0_0_2 - 0.0241387502672213*G0_1_0 - 0.0228339529554797*G0_1_1 - 0.0120693751336111*G0_1_2 - 0.0127217737894818*G0_2_0 - 0.0120693751336102*G0_2_1;
    A[1934] = -0.105688582251081*G0_0_0 - 0.0756782440810227*G0_0_1 - 0.0528442911255396*G0_0_2 - 0.0756782440810207*G0_1_0 - 0.0456679059109606*G0_1_1 - 0.0378391220405089*G0_1_2 - 0.0528442911255404*G0_2_0 - 0.0378391220405115*G0_2_1;
    A[1935] = -0.0254435475789636*G0_0_0 - 0.012721773789482*G0_0_1 - 0.024138750267221*G0_0_2 - 0.0127217737894818*G0_1_0 - 0.0120693751336103*G0_1_2 - 0.0241387502672224*G0_2_0 - 0.0120693751336108*G0_2_1 - 0.0228339529554796*G0_2_2;
    A[1936] = -0.113517366121532*G0_0_0 - 0.100795592332048*G0_0_1 - 0.10079559233205*G0_0_2 - 0.10079559233205*G0_1_0 - 0.088073818542568*G0_1_1 - 0.088726217198439*G0_1_2 - 0.100795592332051*G0_2_0 - 0.0887262171984384*G0_2_1 - 0.0880738185425689*G0_2_2;
    A[1937] = -0.105688582251082*G0_0_0 - 0.0528442911255425*G0_0_1 - 0.0756782440810218*G0_0_2 - 0.0528442911255411*G0_1_0 - 0.037839122040511*G0_1_2 - 0.0756782440810213*G0_2_0 - 0.0378391220405117*G0_2_1 - 0.0456679059109612*G0_2_2;
    A[1938] = 0.647179466623905*G0_0_0 + 0.375781625781622*G0_0_1 + 0.375781625781622*G0_0_2 + 0.375781625781622*G0_1_0 + 0.647179466623905*G0_1_1 + 0.375781625781623*G0_1_2 + 0.375781625781622*G0_2_0 + 0.375781625781623*G0_2_1 + 0.647179466623906*G0_2_2;
    A[1939] = -0.268135847562927*G0_0_0 - 0.448850275239159*G0_0_1 - 0.290643601190473*G0_0_2 - 0.194414799449519*G0_1_0 - 0.584549195660301*G0_1_1 - 0.32163253734434*G0_1_2 - 0.222141742324032*G0_2_0 - 0.32163253734434*G0_2_1 - 0.268135847562928*G0_2_2;
    A[1940] = 0.105688582251081*G0_0_0 + 0.180062029020359*G0_0_1 + 0.10503618359521*G0_0_2 + 0.0756782440810208*G0_1_0 + 0.255740273101381*G0_1_1 + 0.127870136550691*G0_1_2 + 0.0528442911255404*G0_2_0 + 0.12787013655069*G0_2_1 + 0.105688582251081*G0_2_2;
    A[1941] = -0.268135847562929*G0_0_0 - 0.290643601190474*G0_0_1 - 0.448850275239161*G0_0_2 - 0.222141742324032*G0_1_0 - 0.268135847562929*G0_1_1 - 0.32163253734434*G0_1_2 - 0.19441479944952*G0_2_0 - 0.32163253734434*G0_2_1 - 0.584549195660302*G0_2_2;
    A[1942] = 0.113517366121532*G0_0_0 + 0.198655390712682*G0_0_1 + 0.19865539071268*G0_0_2 + 0.10079559233205*G0_1_0 + 0.172233245149911*G0_1_1 + 0.166035457919137*G0_1_2 + 0.100795592332051*G0_2_0 + 0.166035457919138*G0_2_1 + 0.17223324514991*G0_2_2;
    A[1943] = 0.105688582251083*G0_0_0 + 0.10503618359521*G0_0_1 + 0.180062029020362*G0_0_2 + 0.0528442911255413*G0_1_0 + 0.105688582251081*G0_1_1 + 0.127870136550691*G0_1_2 + 0.0756782440810217*G0_2_0 + 0.127870136550691*G0_2_1 + 0.255740273101382*G0_2_2;
    A[1944] = 0.0521918924696691*G0_0_0 + 0.323589733311953*G0_0_1 + 0.187890812890812*G0_0_2 + 0.323589733311953*G0_1_0 + 0.0521918924696695*G0_1_1 + 0.187890812890812*G0_1_2 + 0.187890812890812*G0_2_0 + 0.187890812890811*G0_2_1 + 0.323589733311953*G0_2_2;
    A[1945] = 0.0391439193522531*G0_0_0 - 0.0965550010688891*G0_0_1 - 0.0287055408583187*G0_0_2 - 0.0965550010688889*G0_1_0 - 0.0228339529554804*G0_1_1 - 0.059694477012186*G0_1_2 - 0.0287055408583185*G0_2_0 - 0.0596944770121848*G0_2_1 - 0.09655500106889*G0_2_2;
    A[1946] = -0.0456679059109613*G0_0_0 + 0.0300103381700604*G0_0_1 - 0.00782878387044991*G0_0_2 + 0.0300103381700596*G0_1_0 + 0.0150051690850312*G0_1_2 - 0.00782878387045011*G0_2_0 + 0.0150051690850308*G0_2_1 + 0.0300103381700612*G0_2_2;
    A[1947] = -0.156575677409009*G0_0_0 - 0.134067923781465*G0_0_1 - 0.22442513761958*G0_0_2 - 0.134067923781465*G0_1_0 - 0.0880738185425684*G0_1_1 - 0.09720739972476*G0_1_2 - 0.22442513761958*G0_2_0 - 0.0972073997247602*G0_2_1 - 0.292274597830151*G0_2_2;
    A[1948] = 0.039143919352252*G0_0_0 + 0.0127217737894816*G0_0_1 + 0.0446893079271548*G0_0_2 + 0.0127217737894826*G0_1_0 + 0.0120693751336106*G0_1_2 + 0.044689307927154*G0_2_0 + 0.0120693751336104*G0_2_1 + 0.0502346965020565*G0_2_2;
    A[1949] = 0.0521918924696701*G0_0_0 + 0.0528442911255427*G0_0_1 + 0.0900310145101808*G0_0_2 + 0.05284429112554*G0_1_0 + 0.037839122040511*G0_1_2 + 0.0900310145101808*G0_2_0 + 0.0378391220405119*G0_2_1 + 0.127870136550692*G0_2_2;
    A[1950] = 0.0521918924696686*G0_0_0 + 0.187890812890811*G0_0_1 + 0.323589733311951*G0_0_2 + 0.187890812890811*G0_1_0 + 0.323589733311953*G0_1_1 + 0.187890812890811*G0_1_2 + 0.323589733311953*G0_2_0 + 0.187890812890811*G0_2_1 + 0.0521918924696693*G0_2_2;
    A[1951] = 0.0391439193522533*G0_0_0 - 0.0287055408583189*G0_0_1 - 0.0965550010688893*G0_0_2 - 0.028705540858319*G0_1_0 - 0.0965550010688904*G0_1_1 - 0.0596944770121849*G0_1_2 - 0.0965550010688892*G0_2_0 - 0.0596944770121858*G0_2_1 - 0.0228339529554804*G0_2_2;
    A[1952] = -0.0456679059109619*G0_0_0 - 0.00782878387045032*G0_0_1 + 0.0300103381700599*G0_0_2 - 0.00782878387045068*G0_1_0 + 0.0300103381700606*G0_1_1 + 0.0150051690850288*G0_1_2 + 0.0300103381700599*G0_2_0 + 0.0150051690850301*G0_2_1;
    A[1953] = -0.156575677409008*G0_0_0 - 0.224425137619579*G0_0_1 - 0.134067923781463*G0_0_2 - 0.22442513761958*G0_1_0 - 0.29227459783015*G0_1_1 - 0.0972073997247588*G0_1_2 - 0.134067923781464*G0_2_0 - 0.0972073997247591*G0_2_1 - 0.0880738185425673*G0_2_2;
    A[1954] = 0.0391439193522531*G0_0_0 + 0.0446893079271557*G0_0_1 + 0.0127217737894822*G0_0_2 + 0.0446893079271556*G0_1_0 + 0.050234696502058*G0_1_1 + 0.0120693751336108*G0_1_2 + 0.0127217737894828*G0_2_0 + 0.012069375133612*G0_2_1;
    A[1955] = 0.0521918924696679*G0_0_0 + 0.0900310145101783*G0_0_1 + 0.0528442911255396*G0_0_2 + 0.0900310145101789*G0_1_0 + 0.127870136550689*G0_1_1 + 0.037839122040509*G0_1_2 + 0.0528442911255396*G0_2_0 + 0.0378391220405091*G0_2_1;
    A[1956] = -0.782878387045048*G0_0_0 - 0.466465038947674*G0_0_1 - 0.466465038947673*G0_0_2 - 0.466465038947674*G0_1_0 - 0.1500516908503*G0_1_1 - 0.3359853077735*G0_1_2 - 0.466465038947674*G0_2_0 - 0.3359853077735*G0_2_1 - 0.150051690850301*G0_2_2;
    A[1957] = 0.195719596761263*G0_0_0 + 0.12069375133611*G0_0_1 + 0.12069375133611*G0_0_2 + 0.120693751336111*G0_1_0 + 0.0456679059109598*G0_1_1 + 0.0717638521457954*G0_1_2 + 0.120693751336112*G0_2_0 + 0.0717638521457943*G0_2_1 + 0.0456679059109599*G0_2_2;
    A[1958] = 0.293579395141891*G0_0_0 + 0.218553549716744*G0_0_1 + 0.234863516113514*G0_0_2 + 0.21855354971674*G0_1_0 + 0.143527704291592*G0_1_1 + 0.185933616923198*G0_1_2 + 0.234863516113513*G0_2_0 + 0.1859336169232*G0_2_1 + 0.176147637085136*G0_2_2;
    A[1959] = 0.293579395141892*G0_0_0 + 0.234863516113513*G0_0_1 + 0.218553549716744*G0_0_2 + 0.234863516113514*G0_1_0 + 0.176147637085136*G0_1_1 + 0.185933616923201*G0_1_2 + 0.218553549716742*G0_2_0 + 0.185933616923199*G0_2_1 + 0.143527704291593*G0_2_2;
    A[1960] = -0.0345719095719094*G0_0_0 - 0.0345719095719094*G0_0_1 - 0.0345719095719093*G0_0_2 - 0.0298472385060926*G0_1_0 - 0.0298472385060926*G0_1_1 - 0.0298472385060925*G0_1_2 - 0.0298472385060924*G0_2_0 - 0.0298472385060924*G0_2_1 - 0.0298472385060923*G0_2_2;
    A[1961] = -0.00946499969937473*G0_0_0;
    A[1962] = 0.00472467106581679*G0_0_1 - 0.0298472385060925*G0_1_1;
    A[1963] = 0.00227817610630117*G0_0_2 + 0.0117431758056758*G0_2_2;
    A[1964] = 0.0845508658008643*G0_0_1 + 0.0429604514890963*G0_0_2 + 0.128522535206561*G0_1_1 + 0.0422754329004323*G0_1_2 + 0.042275432900432*G0_2_1;
    A[1965] = 0.0808321934624015*G0_0_1 + 0.108232937008978*G0_0_2 + 0.0371867233846405*G0_1_1 + 0.0743408268364859*G0_1_2 + 0.0743408268364856*G0_2_1 + 0.104709984267274*G0_2_2;
    A[1966] = 0.00939454064453967*G0_0_1 + 0.0377738821749225*G0_0_2 - 0.00619778723077443*G0_1_1 + 0.0188869410874617*G0_1_2 + 0.0188869410874612*G0_2_1 + 0.0567586830607655*G0_2_2;
    A[1967] = -0.00254435475789556*G0_0_1 - 0.0142875305635713*G0_0_2 - 0.0058389679700443*G0_1_2 - 0.00583896797004387*G0_2_1 - 0.0435476102793803*G0_2_2;
    A[1968] = 0.00658922642429623*G0_0_0 + 0.00329461321214795*G0_0_2 + 0.00329461321214831*G0_2_0;
    A[1969] = 0.00450155072550981*G0_0_0 + 0.0019245760348203*G0_0_2 + 0.00192457603481959*G0_2_0 - 0.000652398655869762*G0_2_2;
    A[1970] = -0.0105036183595213*G0_0_0 - 0.0130805930502116*G0_0_2 - 0.0130805930502111*G0_2_0 - 0.0156575677409014*G0_2_2;
    A[1971] = -0.00254435475789556*G0_0_0 - 0.00583896797004335*G0_0_2 - 0.00583896797004381*G0_2_0 - 0.00913358118219155*G0_2_2;
    A[1972] = 0.0292600797158088*G0_0_0 + 0.00329461321214792*G0_0_1 + 0.00329461321214709*G0_1_0;
    A[1973] = -0.0189848008858434*G0_0_0 - 0.00949240044292182*G0_0_1 - 0.00949240044292086*G0_1_0 - 0.000652398655870405*G0_1_1;
    A[1974] = 0.00352295274170321*G0_0_0 + 0.00649136662591566*G0_0_1 + 0.00649136662591484*G0_1_0 + 0.00880738185425674*G0_1_1;
    A[1975] = 0.0429604514890963*G0_0_0 + 0.0422754329004318*G0_0_1 + 0.0422754329004324*G0_1_0 + 0.0642612676032806*G0_1_1;
    A[1976] = 0.0439716694056967*G0_0_0 + 0.0439716694056968*G0_0_1 + 0.0446566879943616*G0_0_2 + 0.128522535206562*G0_1_0 + 0.128522535206562*G0_1_1 + 0.0862471023061298*G0_1_2 + 0.0862471023061297*G0_2_0 + 0.0862471023061297*G0_2_1 + 0.0439716694056971*G0_2_2;
    A[1977] = -0.0436454700777618*G0_0_0 - 0.0436454700777619*G0_0_1 - 0.00975335990527012*G0_0_2 + 0.0371867233846401*G0_1_0 + 0.0371867233846402*G0_1_1 - 0.0371541034518463*G0_1_2 - 0.0371541034518464*G0_2_0 - 0.0371541034518465*G0_2_1 - 0.00678494602105759*G0_2_2;
    A[1978] = -0.0155923278753133*G0_0_0 - 0.0155923278753133*G0_0_1 + 0.00329461321214817*G0_0_2 - 0.00619778723077327*G0_1_0 - 0.00619778723077334*G0_1_1 - 0.0250847283182353*G0_1_2 - 0.0250847283182346*G0_2_0 - 0.0250847283182345*G0_2_1 + 0.0127870136550693*G0_2_2;
    A[1979] = 0.00254435475789565*G0_0_0 + 0.00254435475789555*G0_0_1 - 0.00590420783563171*G0_0_2 + 0.00583896797004409*G0_1_2 + 0.00583896797004389*G0_2_0 + 0.00583896797004391*G0_2_1 - 0.0318696743392925*G0_2_2;
    A[1980] = 0.1273155976932*G0_0_0 + 0.128000616281865*G0_0_1 + 0.1273155976932*G0_0_2 + 0.134361503176606*G0_1_0 + 0.0920860702761738*G0_1_1 + 0.134361503176606*G0_1_2 + 0.176636936077038*G0_2_0 + 0.134361503176606*G0_2_1 + 0.176636936077038*G0_2_2;
    A[1981] = -0.166948816037355*G0_0_0 - 0.160783648739376*G0_0_1 - 0.166948816037355*G0_0_2 - 0.250814663249556*G0_1_0 - 0.0622388317700811*G0_1_1 - 0.250814663249556*G0_1_2 - 0.334680510461757*G0_2_0 - 0.250814663249556*G0_2_1 - 0.334680510461757*G0_2_2;
    A[1982] = 0.182410664181495*G0_0_0 + 0.0985448169692951*G0_0_1 + 0.182410664181495*G0_0_2 + 0.188575831479475*G0_1_0 - 0.0622388317700814*G0_1_1 + 0.188575831479475*G0_1_2 + 0.104709984267275*G0_2_0 + 0.188575831479475*G0_2_1 + 0.104709984267275*G0_2_2;
    A[1983] = -0.0429604514890963*G0_0_0 - 0.0359145460056913*G0_0_1 - 0.0429604514890964*G0_0_2 - 0.0422754329004325*G0_1_0 + 0.0920860702761738*G0_1_1 - 0.0422754329004326*G0_1_2 - 0.0422754329004327*G0_2_1;
    A[1984] = 0.0226708532915131*G0_0_0 + 0.0219858347028483*G0_0_1 + 0.0219858347028483*G0_0_2 + 0.0219858347028489*G0_1_0 + 0.0642612676032809*G0_1_1 + 0.0642612676032811*G0_1_2 + 0.0219858347028485*G0_2_0 + 0.0642612676032812*G0_2_1 + 0.0642612676032813*G0_2_2;
    A[1985] = -0.000652398655871197*G0_0_0 + 0.00231601522834148*G0_0_1 + 0.00231601522834145*G0_0_2 + 0.00231601522834077*G0_1_0 + 0.00880738185425661*G0_1_1 + 0.00880738185425646*G0_1_2 + 0.00231601522834138*G0_2_0 + 0.00880738185425616*G0_2_1 + 0.00880738185425615*G0_2_2;
    A[1986] = -0.000652398655870491*G0_0_0 + 0.0088400017870505*G0_0_1 + 0.00884000178705047*G0_0_2 + 0.00884000178705101*G0_1_0 - 0.000652398655870916*G0_1_1 - 0.000652398655870853*G0_1_2 + 0.00884000178705049*G0_2_0 - 0.000652398655870802*G0_2_1 - 0.000652398655870729*G0_2_2;
    A[1987] = 0.0226708532915127*G0_0_0 - 0.00329461321214785*G0_0_1 - 0.00329461321214787*G0_0_2 - 0.0032946132121479*G0_1_0 - 0.00329461321214775*G0_2_0;
    A[1988] = -0.025443547578964*G0_0_0 - 0.0013047973117402*G0_0_1 - 0.0127217737894824*G0_0_2 - 0.00130479731174218*G0_1_0 - 0.000652398655872604*G0_1_2 - 0.0127217737894834*G0_2_0 - 0.000652398655870298*G0_2_1;
    A[1989] = 0.0567586830607671*G0_0_0 + 0.0283793415303821*G0_0_1 + 0.028379341530383*G0_0_2 + 0.028379341530382*G0_1_0 - 0.00489298991903275*G0_1_1 + 0.0141896707651917*G0_1_2 + 0.0283793415303844*G0_2_0 + 0.0141896707651902*G0_2_1;
    A[1990] = 0.209419968534549*G0_0_0 + 0.135698920421142*G0_0_1 + 0.104709984267274*G0_0_2 + 0.13569892042114*G0_1_0 + 0.0391439193522513*G0_1_1 + 0.0678494602105691*G0_1_2 + 0.104709984267274*G0_2_0 + 0.0678494602105712*G0_2_1;
    A[1991] = -0.0166361657247073*G0_0_0 - 0.0181040627004161*G0_0_1 + 0.00146789697571015*G0_0_2 - 0.0181040627004163*G0_1_0 - 0.0102752788299659*G0_1_2 + 0.00146789697570951*G0_2_0 - 0.0102752788299658*G0_2_1 + 0.0195719596761263*G0_2_2;
    A[1992] = 0.149725491522365*G0_0_0 + 0.0748627457611815*G0_0_1 + 0.121346149991982*G0_0_2 + 0.0748627457611825*G0_1_0 + 0.0195719596761258*G0_1_1 + 0.0606730749959902*G0_1_2 + 0.121346149991981*G0_2_0 + 0.0606730749959905*G0_2_1 + 0.0929668084615995*G0_2_2;
    A[1993] = 0.0137003717732888*G0_0_0 - 0.006197787230772*G0_0_1 + 0.0264221455627702*G0_0_2 - 0.00619778723077359*G0_1_0 - 0.00554538857490211*G0_1_2 + 0.0264221455627701*G0_2_0 - 0.00554538857490194*G0_2_1 + 0.0391439193522518*G0_2_2;
    A[1994] = -0.268135847562927*G0_0_0 - 0.194414799449519*G0_0_1 - 0.222141742324032*G0_0_2 - 0.448850275239159*G0_1_0 - 0.584549195660301*G0_1_1 - 0.32163253734434*G0_1_2 - 0.290643601190473*G0_2_0 - 0.32163253734434*G0_2_1 - 0.268135847562928*G0_2_2;
    A[1995] = 0.72611970398428*G0_0_0 + 0.456026660453739*G0_0_1 + 0.456026660453738*G0_0_2 + 0.456026660453739*G0_1_0 + 0.912053320907479*G0_1_1 + 0.456026660453739*G0_1_2 + 0.456026660453738*G0_2_0 + 0.456026660453739*G0_2_1 + 0.726119703984279*G0_2_2;
    A[1996] = -0.209419968534549*G0_0_0 - 0.390134396210781*G0_0_1 - 0.290643601190473*G0_0_2 - 0.13569892042114*G0_1_0 - 0.584549195660302*G0_1_1 - 0.262916658315961*G0_1_2 - 0.104709984267274*G0_2_0 - 0.262916658315961*G0_2_1 - 0.209419968534549*G0_2_2;
    A[1997] = 0.0655660649150217*G0_0_0 + 0.140428810676204*G0_0_1 + 0.0376760223765426*G0_0_2 + 0.0376760223765422*G0_1_0 - 0.0371867233846415*G0_1_1 + 0.186912214907004*G0_1_2 + 0.140428810676205*G0_2_0 + 0.186912214907004*G0_2_1 - 0.0371867233846397*G0_2_2;
    A[1998] = -0.149725491522365*G0_0_0 - 0.177615534060844*G0_0_1 - 0.326851726591306*G0_0_2 - 0.0748627457611826*G0_1_0 - 0.0371867233846387*G0_1_1 - 0.224098938291644*G0_1_2 - 0.121346149991981*G0_2_0 - 0.224098938291643*G0_2_1 - 0.448197876583288*G0_2_2;
    A[1999] = -0.0137003717732892*G0_0_0 + 0.00619778723077312*G0_0_1 + 0.0714376528178597*G0_0_2 + 0.00619778723077322*G0_1_0 + 0.0123955744615471*G0_1_1 + 0.00619778723077351*G0_1_2 - 0.0264221455627705*G0_2_0 + 0.00619778723077282*G0_2_1 + 0.172233245149909*G0_2_2;
    A[2000] = -0.022833952955481*G0_0_0 - 0.0965550010688896*G0_0_1 - 0.0596944770121852*G0_0_2 - 0.0965550010688892*G0_1_0 + 0.0391439193522526*G0_1_1 - 0.0287055408583187*G0_1_2 - 0.059694477012186*G0_2_0 - 0.028705540858319*G0_2_1 - 0.0965550010688899*G0_2_2;
    A[2001] = -0.00489298991903146*G0_0_0 - 0.0332723314494145*G0_0_1 - 0.0190826606842226*G0_0_2 - 0.0332723314494144*G0_1_0 - 0.00489298991903157*G0_1_1 - 0.0190826606842221*G0_1_2 - 0.019082660684222*G0_2_0 - 0.0190826606842223*G0_2_1 - 0.0332723314494128*G0_2_2;
    A[2002] = -0.0228339529554807*G0_0_0 + 0.0013047973117399*G0_0_1 - 0.0107645778218708*G0_0_2 + 0.00130479731174214*G0_1_0 + 0.000652398655868842*G0_1_2 - 0.01076457782187*G0_2_0 + 0.000652398655870246*G0_2_1 + 0.00130479731174057*G0_2_2;
    A[2003] = 0.0195719596761268*G0_0_0 - 0.0552907860850554*G0_0_1 + 0.00538228891093558*G0_0_2 - 0.0552907860850568*G0_1_0 + 0.0195719596761273*G0_1_1 - 0.0411011153198649*G0_1_2 + 0.0053822889109357*G0_2_0 - 0.0411011153198631*G0_2_1 - 0.00880738185425596*G0_2_2;
    A[2004] = 0.0195719596761253*G0_0_0 + 0.0181040627004162*G0_0_1 + 0.0298472385060928*G0_0_2 + 0.0181040627004154*G0_1_0 + 0.0102752788299668*G0_1_2 + 0.0298472385060912*G0_2_0 + 0.0102752788299658*G0_2_1 + 0.0401225173360581*G0_2_2;
    A[2005] = 0.0260959462348342*G0_0_0 + 0.0061977872307722*G0_0_1 + 0.0381653213684446*G0_0_2 + 0.00619778723077352*G0_1_0 + 0.00554538857490168*G0_1_2 + 0.0381653213684453*G0_2_0 + 0.00554538857490197*G0_2_1 + 0.0502346965020559*G0_2_2;
    A[2006] = -0.0880738185425669*G0_0_0 - 0.0972073997247587*G0_0_1 - 0.134067923781463*G0_0_2 - 0.0972073997247594*G0_1_0 - 0.29227459783015*G0_1_1 - 0.224425137619579*G0_1_2 - 0.134067923781464*G0_2_0 - 0.22442513761958*G0_2_1 - 0.156575677409009*G0_2_2;
    A[2007] = 0.0195719596761252*G0_0_0 - 0.0411011153198663*G0_0_1 - 0.0552907860850584*G0_0_2 - 0.0411011153198651*G0_1_0 - 0.00880738185425641*G0_1_1 + 0.00538228891093423*G0_1_2 - 0.0552907860850563*G0_2_0 + 0.00538228891093508*G0_2_1 + 0.0195719596761254*G0_2_2;
    A[2008] = -0.0880738185425672*G0_0_0 + 0.00065239865587114*G0_0_1 + 0.0127217737894821*G0_0_2 + 0.00065239865587156*G0_1_0 + 0.00130479731174116*G0_1_1 + 0.000652398655872398*G0_1_2 + 0.0127217737894823*G0_2_0 + 0.000652398655871241*G0_2_1;
    A[2009] = 0.0929668084615978*G0_0_0 + 0.228013330226869*G0_0_1 + 0.363059851992139*G0_0_2 + 0.228013330226869*G0_1_0 + 0.45602666045374*G0_1_1 + 0.228013330226869*G0_1_2 + 0.363059851992139*G0_2_0 + 0.228013330226869*G0_2_1 + 0.092966808461599*G0_2_2;
    A[2010] = 0.0929668084616002*G0_0_0 + 0.0322937334656092*G0_0_1 - 0.0283793415303828*G0_0_2 + 0.0322937334656076*G0_1_0 - 0.0088073818542569*G0_1_1 - 0.0141896707651913*G0_1_2 - 0.0283793415303827*G0_2_0 - 0.014189670765192*G0_2_1;
    A[2011] = -0.185933616923198*G0_0_0 - 0.19506719810539*G0_0_1 - 0.104709984267274*G0_0_2 - 0.19506719810539*G0_1_0 - 0.292274597830151*G0_1_1 - 0.0678494602105692*G0_1_2 - 0.104709984267275*G0_2_0 - 0.0678494602105702*G0_2_1;
    A[2012] = 0.146789697570944*G0_0_0 + 0.445262082631868*G0_0_1 + 0.249542485870607*G0_0_2 + 0.44526208263187*G0_1_0 - 0.0391439193522544*G0_1_1 + 0.249542485870609*G0_1_2 + 0.249542485870608*G0_2_0 + 0.249542485870607*G0_2_1 + 0.352295274170271*G0_2_2;
    A[2013] = 0.146789697570948*G0_0_0 + 0.00489298991903281*G0_0_1 + 0.0538228891093484*G0_0_2 + 0.00489298991903148*G0_1_0 + 0.00978597983806421*G0_1_1 + 0.00489298991903122*G0_1_2 + 0.053822889109348*G0_2_0 + 0.00489298991903201*G0_2_1 - 0.0391439193522516*G0_2_2;
    A[2014] = -0.782878387045046*G0_0_0 - 0.484406001984121*G0_0_1 - 0.484406001984122*G0_0_2 - 0.48440600198412*G0_1_0 - 0.0391439193522499*G0_1_1 - 0.28868640522286*G0_1_2 - 0.484406001984122*G0_2_0 - 0.288686405222859*G0_2_1 - 0.185933616923198*G0_2_2;
    A[2015] = -0.0489298991903142*G0_0_0 - 0.0195719596761265*G0_0_1 - 0.141896707651914*G0_0_2 - 0.0195719596761257*G0_1_0 - 0.0391439193522525*G0_1_1 - 0.0195719596761269*G0_1_2 - 0.141896707651914*G0_2_0 - 0.0195719596761272*G0_2_1 - 0.234863516113513*G0_2_2;
    A[2016] = 0.013742125287264*G0_0_0 + 0.0137421252872641*G0_0_1 + 0.0137421252872641*G0_0_2 + 0.0189195610202554*G0_1_0 + 0.0189195610202554*G0_1_1 + 0.0189195610202554*G0_1_2 + 0.0189195610202551*G0_2_0 + 0.0189195610202551*G0_2_1 + 0.0189195610202551*G0_2_2;
    A[2017] = -0.0137421252872637*G0_0_0;
    A[2018] = -0.0143945239431347*G0_0_1 + 0.0335985307773502*G0_1_1;
    A[2019] = 0.00517743573299117*G0_0_2 + 0.0189195610202552*G0_2_2;
    A[2020] = -0.19258808321308*G0_0_1 - 0.124738623002509*G0_0_2 - 0.257045070413123*G0_1_1 - 0.09629404160654*G0_1_2 - 0.0962940416065404*G0_2_1;
    A[2021] = -0.0109602974186318*G0_0_1 - 0.146659217839773*G0_0_2 + 0.121346149991982*G0_1_1 - 0.0733296089198862*G0_1_2 - 0.0733296089198861*G0_2_1 - 0.271397840842284*G0_2_2;
    A[2022] = -0.00182671623643763*G0_0_1 + 0.0823327103709049*G0_0_2 - 0.05284429112554*G0_1_1 + 0.0359471659384851*G0_1_2 + 0.0359471659384857*G0_2_1 + 0.209419968534551*G0_2_2;
    A[2023] = -0.01187365553685*G0_0_1 - 0.0477555816097479*G0_0_2 - 0.0212681961813901*G0_1_2 - 0.0212681961813906*G0_2_1 - 0.111886369481855*G0_2_2;
    A[2024] = 0.0187890812890807*G0_0_0 + 0.00939454064454091*G0_0_2 + 0.00939454064454034*G0_2_0;
    A[2025] = -0.014874689353856*G0_0_0 - 0.0149399292194438*G0_0_2 - 0.014939929219443*G0_2_0 - 0.0150051690850302*G0_2_2;
    A[2026] = -0.00117431758056783*G0_0_0 - 0.00123955744615484*G0_0_2 - 0.00123955744615469*G0_2_0 - 0.00130479731174189*G0_2_2;
    A[2027] = -0.01187365553685*G0_0_0 - 0.0212681961813903*G0_0_2 - 0.0212681961813905*G0_2_0 - 0.0306627368259308*G0_2_2;
    A[2028] = 0.0641307878721065*G0_0_0 + 0.00939454064454097*G0_0_1 + 0.00939454064454053*G0_1_0;
    A[2029] = -0.127087258163645*G0_0_0 - 0.0377738821749235*G0_0_1 - 0.0377738821749231*G0_1_0 - 0.0150051690850297*G0_1_1;
    A[2030] = 0.12473862300251*G0_0_0 + 0.0623693115012548*G0_0_1 + 0.0623693115012548*G0_1_0 + 0.047625101878573*G0_1_1;
    A[2031] = -0.124738623002509*G0_0_0 - 0.0962940416065395*G0_0_1 - 0.0962940416065402*G0_1_0 - 0.128522535206561*G0_1_1;
    A[2032] = -0.018789081289081*G0_0_0 - 0.018789081289081*G0_0_1 - 0.00939454064454072*G0_0_2 - 0.0613254736518618*G0_1_0 - 0.0613254736518617*G0_1_1 - 0.0400572774704719*G0_1_2 - 0.0400572774704716*G0_2_0 - 0.0400572774704715*G0_2_1 - 0.0187890812890811*G0_2_2;
    A[2033] = 0.0148746893538562*G0_0_0 + 0.0148746893538564*G0_0_1 - 6.52398655869121e-05*G0_0_2 - 0.0254435475789646*G0_1_0 - 0.0254435475789647*G0_1_1 + 0.0136351319077013*G0_1_2 + 0.0136351319077013*G0_2_0 + 0.0136351319077012*G0_2_1 - 0.000130479731173976*G0_2_2;
    A[2034] = 0.00117431758056761*G0_0_0 + 0.0011743175805673*G0_0_1 - 6.52398655872633e-05*G0_0_2 - 0.0528442911255409*G0_1_0 - 0.0528442911255408*G0_1_1 - 0.0137656116388754*G0_1_2 - 0.0137656116388754*G0_2_0 - 0.0137656116388754*G0_2_1 - 0.000130479731174334*G0_2_2;
    A[2035] = 0.01187365553685*G0_0_0 + 0.0118736555368501*G0_0_1 - 0.00939454064454035*G0_0_2 + 0.0212681961813906*G0_1_2 + 0.0212681961813905*G0_2_0 + 0.0212681961813905*G0_2_1 - 0.018789081289081*G0_2_2;
    A[2036] = -0.0641307878721062*G0_0_0 - 0.0547362472275658*G0_0_1 - 0.0641307878721061*G0_0_2 - 0.0906181733004636*G0_1_0 - 0.0693499771190739*G0_1_1 - 0.0906181733004637*G0_1_2 - 0.111886369481854*G0_2_0 - 0.0906181733004634*G0_2_1 - 0.111886369481854*G0_2_2;
    A[2037] = 0.127087258163644*G0_0_0 + 0.0893133759887213*G0_0_1 + 0.127087258163644*G0_0_2 + 0.173472802596064*G0_1_0 + 0.0846813455320394*G0_1_1 + 0.173472802596064*G0_1_2 + 0.209419968534549*G0_2_0 + 0.173472802596064*G0_2_1 + 0.209419968534549*G0_2_2;
    A[2038] = -0.124738623002509*G0_0_0 - 0.0623693115012543*G0_0_1 - 0.12473862300251*G0_0_2 - 0.198068231922397*G0_1_0 - 0.00339247301052873*G0_1_1 - 0.198068231922397*G0_1_2 - 0.271397840842282*G0_2_0 - 0.198068231922396*G0_2_1 - 0.271397840842282*G0_2_2;
    A[2039] = 0.12473862300251*G0_0_0 + 0.0284445813959695*G0_0_1 + 0.12473862300251*G0_0_2 + 0.0962940416065402*G0_1_0 - 0.0644569872000424*G0_1_1 + 0.0962940416065402*G0_1_2 + 0.09629404160654*G0_2_1;
    A[2040] = -0.00939454064454073*G0_0_1 - 0.00939454064454068*G0_0_2 - 0.00939454064454099*G0_1_0 - 0.0306627368259309*G0_1_1 - 0.030662736825931*G0_1_2 - 0.00939454064454051*G0_2_0 - 0.030662736825931*G0_2_1 - 0.0306627368259311*G0_2_2;
    A[2041] = -6.52398655870776e-05*G0_0_1 - 6.523986558704e-05*G0_0_2 - 6.52398655864237e-05*G0_1_0 - 0.00130479731174158*G0_1_1 - 0.00130479731174149*G0_1_2 - 6.52398655870178e-05*G0_2_0 - 0.00130479731174142*G0_2_1 - 0.00130479731174136*G0_2_2;
    A[2042] = -6.52398655867431e-05*G0_0_1 - 6.52398655868288e-05*G0_0_2 - 6.52398655875671e-05*G0_1_0 - 0.0150051690850305*G0_1_1 - 0.0150051690850304*G0_1_2 - 6.52398655872397e-05*G0_2_0 - 0.0150051690850298*G0_2_1 - 0.0150051690850298*G0_2_2;
    A[2043] = -0.00939454064454098*G0_0_1 - 0.00939454064454082*G0_0_2 - 0.00939454064454072*G0_1_0 - 0.0093945406445404*G0_2_0;
    A[2044] = -0.10568858225108*G0_0_0 - 0.0300103381700607*G0_0_1 - 0.0528442911255407*G0_0_2 - 0.0300103381700595*G0_1_0 - 0.0150051690850292*G0_1_2 - 0.0528442911255406*G0_2_0 - 0.0150051690850307*G0_2_1;
    A[2045] = 0.20941996853455*G0_0_0 + 0.0737210481134108*G0_0_1 + 0.104709984267275*G0_0_2 + 0.0737210481134092*G0_1_0 - 0.0228339529554798*G0_1_1 + 0.0368605240567039*G0_1_2 + 0.104709984267276*G0_2_0 + 0.0368605240567052*G0_2_1;
    A[2046] = -0.542795681684565*G0_0_0 - 0.271397840842284*G0_0_1 - 0.271397840842282*G0_0_2 - 0.271397840842281*G0_1_0 + 0.0521918924696718*G0_1_1 - 0.13569892042114*G0_1_2 - 0.271397840842282*G0_2_0 - 0.135698920421141*G0_2_1;
    A[2047] = 0.0137003717732877*G0_0_0 - 0.0127217737894826*G0_0_1 + 0.0198981590040631*G0_0_2 - 0.0127217737894841*G0_1_0 - 0.0120693751336121*G0_1_2 + 0.0198981590040605*G0_2_0 - 0.0120693751336114*G0_2_1 + 0.0260959462348348*G0_2_2;
    A[2048] = 0.0234863516113511*G0_0_0 + 0.0459941052388975*G0_0_1 - 0.0812236326559236*G0_0_2 + 0.0459941052388968*G0_1_0 - 0.0880738185425679*G0_1_1 + 0.00913358118219386*G0_1_2 - 0.0812236326559231*G0_2_0 + 0.0091335811821918*G0_2_1 - 0.185933616923199*G0_2_2;
    A[2049] = -0.0534966897814107*G0_0_0 - 0.0528442911255409*G0_0_1 - 0.000652398655871372*G0_0_2 - 0.0528442911255403*G0_1_0 - 0.037839122040512*G0_1_2 - 0.000652398655870257*G0_2_0 - 0.0378391220405107*G0_2_1 + 0.0521918924696696*G0_2_2;
    A[2050] = 0.105688582251081*G0_0_0 + 0.0756782440810208*G0_0_1 + 0.0528442911255404*G0_0_2 + 0.180062029020359*G0_1_0 + 0.255740273101381*G0_1_1 + 0.12787013655069*G0_1_2 + 0.10503618359521*G0_2_0 + 0.127870136550691*G0_2_1 + 0.105688582251081*G0_2_2;
    A[2051] = -0.209419968534549*G0_0_0 - 0.13569892042114*G0_0_1 - 0.104709984267274*G0_0_2 - 0.390134396210781*G0_1_0 - 0.584549195660302*G0_1_1 - 0.262916658315961*G0_1_2 - 0.290643601190473*G0_2_0 - 0.262916658315961*G0_2_1 - 0.209419968534549*G0_2_2;
    A[2052] = 0.542795681684565*G0_0_0 + 0.271397840842281*G0_0_1 + 0.271397840842283*G0_0_2 + 0.271397840842281*G0_1_0 + 0.647179466623906*G0_1_1 + 0.271397840842283*G0_1_2 + 0.271397840842283*G0_2_0 + 0.271397840842283*G0_2_1 + 0.542795681684566*G0_2_2;
    A[2053] = -0.0137003717732881*G0_0_0 - 0.0264221455627708*G0_0_1 + 0.00619778723077291*G0_0_2 + 0.0714376528178609*G0_1_0 + 0.172233245149912*G0_1_1 + 0.00619778723077459*G0_1_2 + 0.00619778723077432*G0_2_0 + 0.00619778723077413*G0_2_1 + 0.0123955744615471*G0_2_2;
    A[2054] = -0.0234863516113513*G0_0_0 + 0.0225077536275461*G0_0_1 - 0.104709984267276*G0_0_2 - 0.0459941052388967*G0_1_0 - 0.26813584756293*G0_1_1 + 0.0534966897814111*G0_1_2 + 0.0812236326559232*G0_2_0 + 0.0534966897814106*G0_2_1 - 0.209419968534551*G0_2_2;
    A[2055] = 0.0534966897814107*G0_0_0 + 0.000652398655870244*G0_0_1 + 0.0528442911255406*G0_0_2 + 0.0528442911255404*G0_1_0 + 0.105688582251081*G0_1_1 - 0.0221815542996102*G0_1_2 + 0.00065239865587043*G0_2_0 - 0.0221815542996101*G0_2_1 + 0.105688582251081*G0_2_2;
    A[2056] = 0.0300103381700615*G0_0_1 + 0.0150051690850313*G0_0_2 + 0.0300103381700604*G0_1_0 - 0.0456679059109617*G0_1_1 - 0.00782878387045077*G0_1_2 + 0.0150051690850307*G0_2_0 - 0.00782878387045038*G0_2_1 + 0.0300103381700604*G0_2_2;
    A[2057] = 0.00130479731174067*G0_0_1 + 0.000652398655870089*G0_0_2 + 0.00130479731174124*G0_1_0 - 0.0228339529554801*G0_1_1 - 0.0107645778218695*G0_1_2 + 0.00065239865587122*G0_2_0 - 0.0107645778218688*G0_2_1 + 0.00130479731174145*G0_2_2;
    A[2058] = 0.0300103381700607*G0_0_1 + 0.0150051690850305*G0_0_2 + 0.0300103381700609*G0_1_0 + 0.0150051690850317*G0_1_2 + 0.0150051690850295*G0_2_0 + 0.0150051690850307*G0_2_1 + 0.03001033817006*G0_2_2;
    A[2059] = 0.0127217737894818*G0_0_1 + 0.000652398655869844*G0_0_2 + 0.0127217737894826*G0_1_0 - 0.088073818542569*G0_1_1 + 0.000652398655870892*G0_1_2 + 0.00065239865587065*G0_2_0 + 0.000652398655870983*G0_2_1 + 0.0013047973117417*G0_2_2;
    A[2060] = 0.0127217737894825*G0_0_1 + 0.000652398655871625*G0_0_2 + 0.012721773789482*G0_1_0 + 0.0120693751336106*G0_1_2 + 0.000652398655870885*G0_2_0 + 0.0120693751336114*G0_2_1 + 0.00130479731174186*G0_2_2;
    A[2061] = 0.0528442911255407*G0_0_1 + 0.01500516908503*G0_0_2 + 0.0528442911255416*G0_1_0 + 0.0378391220405111*G0_1_2 + 0.0150051690850303*G0_2_0 + 0.0378391220405105*G0_2_1 + 0.0300103381700599*G0_2_2;
    A[2062] = 0.0378391220405109*G0_0_1 + 0.052844291125541*G0_0_2 + 0.03783912204051*G0_1_0 + 0.12787013655069*G0_1_1 + 0.0900310145101796*G0_1_2 + 0.0528442911255403*G0_2_0 + 0.0900310145101802*G0_2_1 + 0.0521918924696696*G0_2_2;
    A[2063] = 0.00554538857490177*G0_0_1 + 0.0061977872307729*G0_0_2 + 0.00554538857490241*G0_1_0 + 0.0502346965020575*G0_1_1 + 0.0381653213684467*G0_1_2 + 0.00619778723077369*G0_2_0 + 0.0381653213684458*G0_2_1 + 0.026095946234836*G0_2_2;
    A[2064] = 0.0378391220405105*G0_0_1 + 0.0528442911255405*G0_0_2 + 0.0378391220405108*G0_1_0 + 0.0300103381700604*G0_1_1 + 0.0150051690850293*G0_1_2 + 0.0528442911255401*G0_2_0 + 0.0150051690850295*G0_2_1;
    A[2065] = -0.067849460210571*G0_0_1 - 0.104709984267275*G0_0_2 - 0.0678494602105705*G0_1_0 - 0.292274597830151*G0_1_1 - 0.195067198105391*G0_1_2 - 0.104709984267275*G0_2_0 - 0.195067198105391*G0_2_1 - 0.185933616923199*G0_2_2;
    A[2066] = -0.0678494602105697*G0_0_1 - 0.104709984267276*G0_0_2 - 0.0678494602105707*G0_1_0 - 0.0965550010688891*G0_1_1 - 0.0368605240567044*G0_1_2 - 0.104709984267275*G0_2_0 - 0.0368605240567042*G0_2_1;
    A[2067] = 0.13569892042114*G0_0_1 + 0.271397840842281*G0_0_2 + 0.135698920421142*G0_1_0 + 0.323589733311953*G0_1_1 + 0.13569892042114*G0_1_2 + 0.271397840842283*G0_2_0 + 0.135698920421141*G0_2_1;
    A[2068] = -0.0750258454251525*G0_0_1 - 0.0260959462348368*G0_0_2 - 0.0750258454251476*G0_1_0 + 0.143527704291595*G0_1_1 - 0.0424059126316058*G0_1_2 - 0.0260959462348348*G0_2_0 - 0.0424059126316064*G0_2_1 - 0.0521918924696701*G0_2_2;
    A[2069] = -0.0750258454251515*G0_0_1 - 0.0260959462348355*G0_0_2 - 0.0750258454251517*G0_1_0 + 0.0456679059109602*G0_1_1 - 0.0260959462348343*G0_1_2 - 0.0260959462348344*G0_2_0 - 0.0260959462348362*G0_2_1 - 0.0521918924696705*G0_2_2;
    A[2070] = 0.316413348097374*G0_0_1 + 0.185933616923199*G0_0_2 + 0.316413348097371*G0_1_0 - 0.150051690850305*G0_1_1 + 0.185933616923198*G0_1_2 + 0.185933616923197*G0_2_0 + 0.185933616923197*G0_2_1 + 0.371867233846397*G0_2_2;
    A[2071] = -0.0587158790283787*G0_0_1 - 0.0260959462348331*G0_0_2 - 0.0587158790283769*G0_1_0 + 0.176147637085137*G0_1_1 - 0.00978597983806206*G0_1_2 - 0.026095946234835*G0_2_0 - 0.00978597983806276*G0_2_1 - 0.0521918924696688*G0_2_2;
    A[2072] = -0.0345719095719091*G0_0_0 - 0.0345719095719091*G0_0_1 - 0.0345719095719092*G0_0_2 - 0.0298472385060924*G0_1_0 - 0.0298472385060924*G0_1_1 - 0.0298472385060924*G0_1_2 - 0.0298472385060926*G0_2_0 - 0.0298472385060925*G0_2_1 - 0.0298472385060926*G0_2_2;
    A[2073] = -0.00946499969937444*G0_0_0;
    A[2074] = 0.00227817610630115*G0_0_1 + 0.0117431758056758*G0_1_1;
    A[2075] = 0.00472467106581727*G0_0_2 - 0.029847238506092*G0_2_2;
    A[2076] = -0.0142875305635724*G0_0_1 - 0.00254435475789624*G0_0_2 - 0.0435476102793813*G0_1_1 - 0.00583896797004454*G0_1_2 - 0.0058389679700452*G0_2_1;
    A[2077] = 0.0377738821749239*G0_0_1 + 0.00939454064453991*G0_0_2 + 0.0567586830607668*G0_1_1 + 0.018886941087462*G0_1_2 + 0.0188869410874636*G0_2_1 - 0.00619778723077263*G0_2_2;
    A[2078] = 0.108232937008977*G0_0_1 + 0.080832193462401*G0_0_2 + 0.104709984267274*G0_1_1 + 0.0743408268364852*G0_1_2 + 0.0743408268364842*G0_2_1 + 0.0371867233846396*G0_2_2;
    A[2079] = 0.0429604514890981*G0_0_1 + 0.0845508658008656*G0_0_2 + 0.0422754329004333*G0_1_2 + 0.042275432900434*G0_2_1 + 0.128522535206562*G0_2_2;
    A[2080] = 0.0292600797158082*G0_0_0 + 0.00329461321214817*G0_0_2 + 0.00329461321214774*G0_2_0;
    A[2081] = -0.0189848008858423*G0_0_0 - 0.00949240044292199*G0_0_2 - 0.00949240044292119*G0_2_0 - 0.000652398655870933*G0_2_2;
    A[2082] = 0.00352295274170217*G0_0_0 + 0.00649136662591553*G0_0_2 + 0.00649136662591436*G0_2_0 + 0.00880738185425648*G0_2_2;
    A[2083] = 0.0429604514890981*G0_0_0 + 0.042275432900433*G0_0_2 + 0.042275432900434*G0_2_0 + 0.0642612676032816*G0_2_2;
    A[2084] = 0.00658922642429568*G0_0_0 + 0.00329461321214813*G0_0_1 + 0.00329461321214804*G0_1_0;
    A[2085] = 0.00450155072550928*G0_0_0 + 0.00192457603481866*G0_0_1 + 0.00192457603481906*G0_1_0 - 0.000652398655871102*G0_1_1;
    A[2086] = -0.0105036183595217*G0_0_0 - 0.0130805930502109*G0_0_1 - 0.0130805930502114*G0_1_0 - 0.0156575677409008*G0_1_1;
    A[2087] = -0.00254435475789634*G0_0_0 - 0.00583896797004461*G0_0_1 - 0.00583896797004455*G0_1_0 - 0.00913358118219294*G0_1_1;
    A[2088] = 0.127315597693201*G0_0_0 + 0.127315597693201*G0_0_1 + 0.128000616281866*G0_0_2 + 0.176636936077039*G0_1_0 + 0.176636936077039*G0_1_1 + 0.134361503176607*G0_1_2 + 0.134361503176606*G0_2_0 + 0.134361503176606*G0_2_1 + 0.0920860702761741*G0_2_2;
    A[2089] = -0.166948816037357*G0_0_0 - 0.166948816037357*G0_0_1 - 0.160783648739378*G0_0_2 - 0.334680510461758*G0_1_0 - 0.334680510461759*G0_1_1 - 0.250814663249558*G0_1_2 - 0.250814663249558*G0_2_0 - 0.250814663249557*G0_2_1 - 0.0622388317700823*G0_2_2;
    A[2090] = 0.182410664181496*G0_0_0 + 0.182410664181496*G0_0_1 + 0.0985448169692961*G0_0_2 + 0.104709984267275*G0_1_0 + 0.104709984267275*G0_1_1 + 0.188575831479477*G0_1_2 + 0.188575831479476*G0_2_0 + 0.188575831479476*G0_2_1 - 0.0622388317700804*G0_2_2;
    A[2091] = -0.042960451489098*G0_0_0 - 0.042960451489098*G0_0_1 - 0.0359145460056923*G0_0_2 - 0.0422754329004334*G0_1_2 - 0.0422754329004339*G0_2_0 - 0.0422754329004339*G0_2_1 + 0.0920860702761729*G0_2_2;
    A[2092] = 0.0439716694056962*G0_0_0 + 0.0446566879943608*G0_0_1 + 0.0439716694056964*G0_0_2 + 0.0862471023061288*G0_1_0 + 0.0439716694056964*G0_1_1 + 0.086247102306129*G0_1_2 + 0.128522535206562*G0_2_0 + 0.0862471023061294*G0_2_1 + 0.128522535206562*G0_2_2;
    A[2093] = -0.0436454700777608*G0_0_0 - 0.00975335990526901*G0_0_1 - 0.0436454700777609*G0_0_2 - 0.0371541034518452*G0_1_0 - 0.00678494602105624*G0_1_1 - 0.0371541034518454*G0_1_2 + 0.0371867233846405*G0_2_0 - 0.0371541034518461*G0_2_1 + 0.0371867233846405*G0_2_2;
    A[2094] = -0.0155923278753146*G0_0_0 + 0.0032946132121475*G0_0_1 - 0.0155923278753145*G0_0_2 - 0.0250847283182362*G0_1_0 + 0.0127870136550685*G0_1_1 - 0.0250847283182361*G0_1_2 - 0.00619778723077475*G0_2_0 - 0.0250847283182353*G0_2_1 - 0.0061977872307749*G0_2_2;
    A[2095] = 0.00254435475789641*G0_0_0 - 0.00590420783563136*G0_0_1 + 0.00254435475789631*G0_0_2 + 0.00583896797004456*G0_1_0 - 0.0318696743392922*G0_1_1 + 0.00583896797004453*G0_1_2 + 0.00583896797004449*G0_2_1;
    A[2096] = 0.0226708532915123*G0_0_0 + 0.021985834702848*G0_0_1 + 0.0219858347028481*G0_0_2 + 0.0219858347028482*G0_1_0 + 0.0642612676032809*G0_1_1 + 0.0642612676032808*G0_1_2 + 0.0219858347028485*G0_2_0 + 0.0642612676032808*G0_2_1 + 0.0642612676032807*G0_2_2;
    A[2097] = -0.000652398655870424*G0_0_0 + 0.00231601522834161*G0_0_1 + 0.00231601522834155*G0_0_2 + 0.00231601522834172*G0_1_0 + 0.0088073818542564*G0_1_1 + 0.00880738185425635*G0_1_2 + 0.00231601522834131*G0_2_0 + 0.00880738185425657*G0_2_1 + 0.00880738185425656*G0_2_2;
    A[2098] = -0.000652398655871268*G0_0_0 + 0.00884000178704962*G0_0_1 + 0.0088400017870496*G0_0_2 + 0.00884000178705042*G0_1_0 - 0.000652398655871272*G0_1_1 - 0.000652398655871263*G0_1_2 + 0.00884000178705055*G0_2_0 - 0.00065239865587126*G0_2_1 - 0.000652398655871365*G0_2_2;
    A[2099] = 0.0226708532915129*G0_0_0 - 0.00329461321214813*G0_0_1 - 0.00329461321214816*G0_0_2 - 0.00329461321214798*G0_1_0 - 0.00329461321214795*G0_2_0;
    A[2100] = -0.0254435475789649*G0_0_0 - 0.0127217737894827*G0_0_1 - 0.00130479731174197*G0_0_2 - 0.0127217737894827*G0_1_0 - 0.000652398655870552*G0_1_2 - 0.00130479731174156*G0_2_0 - 0.000652398655870804*G0_2_1;
    A[2101] = -0.0166361657247074*G0_0_0 + 0.0014678969757101*G0_0_1 - 0.0181040627004172*G0_0_2 + 0.00146789697571057*G0_1_0 + 0.0195719596761263*G0_1_1 - 0.0102752788299666*G0_1_2 - 0.0181040627004175*G0_2_0 - 0.010275278829966*G0_2_1;
    A[2102] = 0.0137003717732877*G0_0_0 + 0.026422145562769*G0_0_1 - 0.00619778723077398*G0_0_2 + 0.0264221455627707*G0_1_0 + 0.0391439193522539*G0_1_1 - 0.00554538857490254*G0_1_2 - 0.00619778723077294*G0_2_0 - 0.00554538857490213*G0_2_1;
    A[2103] = 0.0567586830607662*G0_0_0 + 0.0283793415303814*G0_0_1 + 0.0283793415303824*G0_0_2 + 0.0283793415303822*G0_1_0 + 0.01418967076519*G0_1_2 + 0.0283793415303831*G0_2_0 + 0.0141896707651901*G0_2_1 - 0.00489298991903298*G0_2_2;
    A[2104] = 0.149725491522364*G0_0_0 + 0.121346149991982*G0_0_1 + 0.0748627457611819*G0_0_2 + 0.121346149991982*G0_1_0 + 0.0929668084615988*G0_1_1 + 0.0606730749959928*G0_1_2 + 0.074862745761183*G0_2_0 + 0.0606730749959906*G0_2_1 + 0.0195719596761268*G0_2_2;
    A[2105] = 0.209419968534552*G0_0_0 + 0.104709984267277*G0_0_1 + 0.135698920421142*G0_0_2 + 0.104709984267276*G0_1_0 + 0.0678494602105703*G0_1_2 + 0.135698920421142*G0_2_0 + 0.0678494602105727*G0_2_1 + 0.039143919352253*G0_2_2;
    A[2106] = -0.268135847562929*G0_0_0 - 0.222141742324032*G0_0_1 - 0.19441479944952*G0_0_2 - 0.290643601190474*G0_1_0 - 0.268135847562929*G0_1_1 - 0.32163253734434*G0_1_2 - 0.448850275239161*G0_2_0 - 0.32163253734434*G0_2_1 - 0.584549195660302*G0_2_2;
    A[2107] = 0.0655660649150217*G0_0_0 + 0.0376760223765422*G0_0_1 + 0.140428810676205*G0_0_2 + 0.140428810676204*G0_1_0 - 0.0371867233846415*G0_1_1 + 0.186912214907004*G0_1_2 + 0.0376760223765426*G0_2_0 + 0.186912214907004*G0_2_1 - 0.0371867233846397*G0_2_2;
    A[2108] = -0.0137003717732881*G0_0_0 + 0.0714376528178609*G0_0_1 + 0.00619778723077431*G0_0_2 - 0.0264221455627708*G0_1_0 + 0.172233245149912*G0_1_1 + 0.00619778723077414*G0_1_2 + 0.00619778723077292*G0_2_0 + 0.00619778723077459*G0_2_1 + 0.0123955744615471*G0_2_2;
    A[2109] = 0.72611970398428*G0_0_0 + 0.45602666045374*G0_0_1 + 0.45602666045374*G0_0_2 + 0.45602666045374*G0_1_0 + 0.726119703984282*G0_1_1 + 0.45602666045374*G0_1_2 + 0.45602666045374*G0_2_0 + 0.45602666045374*G0_2_1 + 0.91205332090748*G0_2_2;
    A[2110] = -0.149725491522364*G0_0_0 - 0.326851726591307*G0_0_1 - 0.177615534060845*G0_0_2 - 0.121346149991982*G0_1_0 - 0.448197876583291*G0_1_1 - 0.224098938291644*G0_1_2 - 0.074862745761183*G0_2_0 - 0.224098938291646*G0_2_1 - 0.0371867233846402*G0_2_2;
    A[2111] = -0.209419968534552*G0_0_0 - 0.290643601190473*G0_0_1 - 0.390134396210782*G0_0_2 - 0.104709984267276*G0_1_0 - 0.20941996853455*G0_1_1 - 0.262916658315963*G0_1_2 - 0.135698920421143*G0_2_0 - 0.26291665831596*G0_2_1 - 0.584549195660302*G0_2_2;
    A[2112] = -0.0880738185425679*G0_0_0 - 0.134067923781464*G0_0_1 - 0.0972073997247596*G0_0_2 - 0.134067923781465*G0_1_0 - 0.15657567740901*G0_1_1 - 0.22442513761958*G0_1_2 - 0.0972073997247597*G0_2_0 - 0.22442513761958*G0_2_1 - 0.292274597830151*G0_2_2;
    A[2113] = 0.0195719596761256*G0_0_0 - 0.0552907860850571*G0_0_1 - 0.0411011153198644*G0_0_2 - 0.0552907860850569*G0_1_0 + 0.0195719596761258*G0_1_1 + 0.0053822889109356*G0_1_2 - 0.0411011153198655*G0_2_0 + 0.00538228891093417*G0_2_1 - 0.00880738185425645*G0_2_2;
    A[2114] = -0.0880738185425669*G0_0_0 + 0.0127217737894824*G0_0_1 + 0.000652398655872312*G0_0_2 + 0.0127217737894823*G0_1_0 + 0.000652398655871527*G0_1_2 + 0.000652398655871002*G0_2_0 + 0.000652398655870317*G0_2_1 + 0.00130479731174188*G0_2_2;
    A[2115] = 0.0929668084616*G0_0_0 + 0.36305985199214*G0_0_1 + 0.228013330226869*G0_0_2 + 0.363059851992141*G0_1_0 + 0.0929668084615992*G0_1_1 + 0.22801333022687*G0_1_2 + 0.228013330226869*G0_2_0 + 0.228013330226869*G0_2_1 + 0.45602666045374*G0_2_2;
    A[2116] = 0.0929668084615996*G0_0_0 - 0.0283793415303813*G0_0_1 + 0.0322937334656076*G0_0_2 - 0.0283793415303823*G0_1_0 - 0.0141896707651913*G0_1_2 + 0.0322937334656098*G0_2_0 - 0.0141896707651897*G0_2_1 - 0.00880738185425602*G0_2_2;
    A[2117] = -0.185933616923198*G0_0_0 - 0.104709984267277*G0_0_1 - 0.19506719810539*G0_0_2 - 0.104709984267275*G0_1_0 - 0.0678494602105709*G0_1_2 - 0.19506719810539*G0_2_0 - 0.0678494602105729*G0_2_1 - 0.292274597830151*G0_2_2;
    A[2118] = -0.0228339529554803*G0_0_0 - 0.0596944770121847*G0_0_1 - 0.0965550010688896*G0_0_2 - 0.0596944770121847*G0_1_0 - 0.096555001068889*G0_1_1 - 0.0287055408583185*G0_1_2 - 0.0965550010688891*G0_2_0 - 0.0287055408583183*G0_2_1 + 0.0391439193522524*G0_2_2;
    A[2119] = -0.00489298991903154*G0_0_0 - 0.0190826606842223*G0_0_1 - 0.0332723314494153*G0_0_2 - 0.019082660684223*G0_1_0 - 0.0332723314494134*G0_1_1 - 0.0190826606842235*G0_1_2 - 0.0332723314494145*G0_2_0 - 0.0190826606842218*G0_2_1 - 0.00489298991903122*G0_2_2;
    A[2120] = -0.0228339529554801*G0_0_0 - 0.0107645778218685*G0_0_1 + 0.00130479731174165*G0_0_2 - 0.0107645778218692*G0_1_0 + 0.00130479731174223*G0_1_1 + 0.000652398655870322*G0_1_2 + 0.00130479731174185*G0_2_0 + 0.000652398655870583*G0_2_1;
    A[2121] = 0.019571959676126*G0_0_0 + 0.00538228891093442*G0_0_1 - 0.0552907860850571*G0_0_2 + 0.00538228891093423*G0_1_0 - 0.00880738185425819*G0_1_1 - 0.0411011153198658*G0_1_2 - 0.0552907860850562*G0_2_0 - 0.0411011153198659*G0_2_1 + 0.0195719596761255*G0_2_2;
    A[2122] = 0.0195719596761256*G0_0_0 + 0.029847238506092*G0_0_1 + 0.0181040627004174*G0_0_2 + 0.0298472385060921*G0_1_0 + 0.0401225173360578*G0_1_1 + 0.0102752788299666*G0_1_2 + 0.0181040627004153*G0_2_0 + 0.0102752788299655*G0_2_1;
    A[2123] = 0.026095946234836*G0_0_0 + 0.0381653213684467*G0_0_1 + 0.00619778723077401*G0_0_2 + 0.0381653213684474*G0_1_0 + 0.0502346965020593*G0_1_1 + 0.0055453885749027*G0_1_2 + 0.00619778723077503*G0_2_0 + 0.00554538857490385*G0_2_1;
    A[2124] = 0.146789697570945*G0_0_0 + 0.249542485870607*G0_0_1 + 0.44526208263187*G0_0_2 + 0.249542485870609*G0_1_0 + 0.352295274170272*G0_1_1 + 0.249542485870609*G0_1_2 + 0.445262082631871*G0_2_0 + 0.249542485870609*G0_2_1 - 0.0391439193522532*G0_2_2;
    A[2125] = 0.146789697570949*G0_0_0 + 0.0538228891093475*G0_0_1 + 0.00489298991903355*G0_0_2 + 0.0538228891093476*G0_1_0 - 0.0391439193522511*G0_1_1 + 0.00489298991903382*G0_1_2 + 0.00489298991903189*G0_2_0 + 0.00489298991903289*G0_2_1 + 0.00978597983806455*G0_2_2;
    A[2126] = -0.0489298991903143*G0_0_0 - 0.141896707651912*G0_0_1 - 0.019571959676125*G0_0_2 - 0.141896707651915*G0_1_0 - 0.234863516113516*G0_1_1 - 0.0195719596761271*G0_1_2 - 0.0195719596761249*G0_2_0 - 0.0195719596761268*G0_2_1 - 0.0391439193522526*G0_2_2;
    A[2127] = -0.782878387045047*G0_0_0 - 0.484406001984122*G0_0_1 - 0.484406001984123*G0_0_2 - 0.484406001984123*G0_1_0 - 0.185933616923198*G0_1_1 - 0.288686405222862*G0_1_2 - 0.484406001984123*G0_2_0 - 0.28868640522286*G0_2_1 - 0.0391439193522532*G0_2_2;
    A[2128] = 0.00946499969937455*G0_0_0 + 0.00946499969937447*G0_0_1 + 0.00946499969937449*G0_0_2 + 0.0117431758056756*G0_1_0 + 0.0117431758056756*G0_1_1 + 0.0117431758056756*G0_1_2 + 0.0117431758056758*G0_2_0 + 0.0117431758056758*G0_2_1 + 0.0117431758056757*G0_2_2;
    A[2129] = -0.00946499969937494*G0_0_0;
    A[2130] = 0.0047246710658168*G0_0_1 - 0.0298472385060926*G0_1_1;
    A[2131] = 0.00472467106581687*G0_0_2 - 0.0298472385060923*G0_2_2;
    A[2132] = 0.0493213383838381*G0_0_1 + 0.0429604514890967*G0_0_2 + 0.176636936077039*G0_1_1 + 0.0422754329004322*G0_1_2 + 0.0422754329004329*G0_2_1;
    A[2133] = -0.167731694424401*G0_0_1 - 0.0777006799142204*G0_0_2 - 0.334680510461758*G0_1_1 - 0.0838658472122003*G0_1_2 - 0.0838658472122007*G0_2_1 + 0.104709984267276*G0_2_2;
    A[2134] = -0.0777006799142206*G0_0_1 - 0.167731694424402*G0_0_2 + 0.104709984267276*G0_1_1 - 0.0838658472122003*G0_1_2 - 0.0838658472122004*G0_2_1 - 0.334680510461758*G0_2_2;
    A[2135] = 0.0429604514890967*G0_0_1 + 0.049321338383838*G0_0_2 + 0.0422754329004319*G0_1_2 + 0.0422754329004321*G0_2_1 + 0.176636936077038*G0_2_2;
    A[2136] = 0.0292600797158093*G0_0_0 + 0.00329461321214766*G0_0_2 + 0.00329461321214769*G0_2_0;
    A[2137] = -0.0189848008858425*G0_0_0 - 0.00949240044292072*G0_0_2 - 0.00949240044292156*G0_2_0 - 0.000652398655871752*G0_2_2;
    A[2138] = 0.00352295274170308*G0_0_0 + 0.00649136662591537*G0_0_2 + 0.00649136662591535*G0_2_0 + 0.0088073818542575*G0_2_2;
    A[2139] = 0.0429604514890968*G0_0_0 + 0.0422754329004325*G0_0_2 + 0.042275432900432*G0_2_0 + 0.06426126760328*G0_2_2;
    A[2140] = 0.0292600797158092*G0_0_0 + 0.00329461321214773*G0_0_1 + 0.00329461321214814*G0_1_0;
    A[2141] = -0.0189848008858428*G0_0_0 - 0.00949240044292094*G0_0_1 - 0.00949240044292122*G0_1_0 - 0.000652398655871076*G0_1_1;
    A[2142] = 0.00352295274170293*G0_0_0 + 0.0064913666259149*G0_0_1 + 0.00649136662591579*G0_1_0 + 0.00880738185425734*G0_1_1;
    A[2143] = 0.0429604514890969*G0_0_0 + 0.0422754329004326*G0_0_1 + 0.0422754329004323*G0_1_0 + 0.0642612676032809*G0_1_1;
    A[2144] = -0.0292600797158088*G0_0_0 - 0.0292600797158088*G0_0_1 - 0.0259654665036605*G0_0_2 - 0.0435476102793812*G0_1_0 - 0.0435476102793811*G0_1_1 - 0.0377086423093367*G0_1_2 - 0.0377086423093366*G0_2_0 - 0.0377086423093368*G0_2_1 - 0.0318696743392923*G0_2_2;
    A[2145] = 0.0189848008858423*G0_0_0 + 0.0189848008858426*G0_0_1 + 0.00949240044292073*G0_0_2 + 0.0567586830607668*G0_1_0 + 0.056758683060767*G0_1_1 + 0.0378717419733049*G0_1_2 + 0.037871741973304*G0_2_0 + 0.0378717419733044*G0_2_1 + 0.0127870136550692*G0_2_2;
    A[2146] = -0.00352295274170253*G0_0_0 - 0.00352295274170259*G0_0_1 + 0.00296841388421318*G0_0_2 + 0.104709984267275*G0_1_0 + 0.104709984267275*G0_1_1 + 0.0303691574307887*G0_1_2 + 0.0303691574307888*G0_2_0 + 0.0303691574307887*G0_2_1 - 0.00678494602105678*G0_2_2;
    A[2147] = -0.0429604514890968*G0_0_0 - 0.0429604514890967*G0_0_1 - 0.000685018588664749*G0_0_2 - 0.0422754329004321*G0_1_2 - 0.042275432900432*G0_2_0 - 0.0422754329004321*G0_2_1 + 0.0439716694056969*G0_2_2;
    A[2148] = -0.0292600797158088*G0_0_0 - 0.0259654665036605*G0_0_1 - 0.029260079715809*G0_0_2 - 0.0377086423093362*G0_1_0 - 0.0318696743392916*G0_1_1 - 0.0377086423093363*G0_1_2 - 0.0435476102793807*G0_2_0 - 0.0377086423093365*G0_2_1 - 0.0435476102793807*G0_2_2;
    A[2149] = 0.0189848008858421*G0_0_0 + 0.00949240044292058*G0_0_1 + 0.0189848008858424*G0_0_2 + 0.0378717419733033*G0_1_0 + 0.0127870136550679*G0_1_1 + 0.0378717419733034*G0_1_2 + 0.0567586830607657*G0_2_0 + 0.0378717419733042*G0_2_1 + 0.0567586830607657*G0_2_2;
    A[2150] = -0.00352295274170272*G0_0_0 + 0.00296841388421327*G0_0_1 - 0.00352295274170276*G0_0_2 + 0.0303691574307899*G0_1_0 - 0.00678494602105585*G0_1_1 + 0.0303691574307897*G0_1_2 + 0.104709984267275*G0_2_0 + 0.0303691574307889*G0_2_1 + 0.104709984267274*G0_2_2;
    A[2151] = -0.0429604514890967*G0_0_0 - 0.000685018588664751*G0_0_1 - 0.0429604514890966*G0_0_2 - 0.0422754329004323*G0_1_0 + 0.0439716694056964*G0_1_1 - 0.0422754329004322*G0_1_2 - 0.0422754329004322*G0_2_1;
    A[2152] = -0.00329461321214857*G0_0_1 - 0.00329461321214867*G0_0_2 - 0.00329461321214781*G0_1_0 - 0.00913358118219244*G0_1_1 - 0.00913358118219244*G0_1_2 - 0.00329461321214809*G0_2_0 - 0.00913358118219258*G0_2_1 - 0.00913358118219258*G0_2_2;
    A[2153] = -0.00257697469068986*G0_0_1 - 0.00257697469068974*G0_0_2 - 0.0025769746906901*G0_1_0 - 0.0156575677409009*G0_1_1 - 0.0156575677409009*G0_1_2 - 0.00257697469068979*G0_2_0 - 0.0156575677409009*G0_2_1 - 0.0156575677409009*G0_2_2;
    A[2154] = -0.00257697469069019*G0_0_1 - 0.00257697469069015*G0_0_2 - 0.00257697469068978*G0_1_0 - 0.000652398655870287*G0_1_1 - 0.000652398655870442*G0_1_2 - 0.00257697469068999*G0_2_0 - 0.000652398655871222*G0_2_1 - 0.000652398655871264*G0_2_2;
    A[2155] = -0.00329461321214758*G0_0_1 - 0.00329461321214765*G0_0_2 - 0.00329461321214782*G0_1_0 - 0.00329461321214808*G0_2_0;
    A[2156] = -0.113517366121532*G0_0_0 - 0.0127217737894815*G0_0_1 - 0.0127217737894818*G0_0_2 - 0.0127217737894818*G0_1_0 - 0.000652398655870996*G0_1_2 - 0.0127217737894813*G0_2_0 - 0.000652398655871503*G0_2_1;
    A[2157] = 0.149725491522365*G0_0_0 + 0.0748627457611823*G0_0_1 + 0.0283793415303824*G0_0_2 + 0.0748627457611823*G0_1_0 + 0.0195719596761273*G0_1_1 + 0.0141896707651927*G0_1_2 + 0.028379341530381*G0_2_0 + 0.0141896707651921*G0_2_1;
    A[2158] = 0.0234863516113517*G0_0_0 - 0.0225077536275455*G0_0_1 + 0.104709984267275*G0_0_2 - 0.022507753627546*G0_1_0 - 0.156575677409011*G0_1_1 + 0.06784946021057*G0_1_2 + 0.104709984267276*G0_2_0 + 0.0678494602105696*G0_2_1;
    A[2159] = 0.149725491522367*G0_0_0 + 0.0283793415303844*G0_0_1 + 0.0748627457611841*G0_0_2 + 0.0283793415303859*G0_1_0 + 0.0141896707651933*G0_1_2 + 0.0748627457611837*G0_2_0 + 0.0141896707651916*G0_2_1 + 0.0195719596761263*G0_2_2;
    A[2160] = -0.540186087061083*G0_0_0 - 0.270093043530541*G0_0_1 - 0.270093043530541*G0_0_2 - 0.270093043530541*G0_1_0 + 0.0929668084616004*G0_1_1 - 0.135046521765272*G0_1_2 - 0.270093043530541*G0_2_0 - 0.13504652176527*G0_2_1 + 0.0929668084615992*G0_2_2;
    A[2161] = 0.0234863516113517*G0_0_0 + 0.104709984267275*G0_0_1 - 0.0225077536275444*G0_0_2 + 0.104709984267275*G0_1_0 + 0.0678494602105726*G0_1_2 - 0.0225077536275453*G0_2_0 + 0.0678494602105703*G0_2_1 - 0.156575677409009*G0_2_2;
    A[2162] = 0.113517366121532*G0_0_0 + 0.10079559233205*G0_0_1 + 0.100795592332051*G0_0_2 + 0.198655390712682*G0_1_0 + 0.172233245149911*G0_1_1 + 0.166035457919138*G0_1_2 + 0.19865539071268*G0_2_0 + 0.166035457919137*G0_2_1 + 0.17223324514991*G0_2_2;
    A[2163] = -0.149725491522365*G0_0_0 - 0.0748627457611826*G0_0_1 - 0.121346149991981*G0_0_2 - 0.177615534060844*G0_1_0 - 0.0371867233846387*G0_1_1 - 0.224098938291643*G0_1_2 - 0.326851726591306*G0_2_0 - 0.224098938291644*G0_2_1 - 0.448197876583288*G0_2_2;
    A[2164] = -0.0234863516113513*G0_0_0 - 0.0459941052388967*G0_0_1 + 0.0812236326559232*G0_0_2 + 0.0225077536275461*G0_1_0 - 0.26813584756293*G0_1_1 + 0.0534966897814106*G0_1_2 - 0.104709984267276*G0_2_0 + 0.0534966897814111*G0_2_1 - 0.209419968534551*G0_2_2;
    A[2165] = -0.149725491522364*G0_0_0 - 0.121346149991982*G0_0_1 - 0.074862745761183*G0_0_2 - 0.326851726591307*G0_1_0 - 0.448197876583291*G0_1_1 - 0.224098938291646*G0_1_2 - 0.177615534060845*G0_2_0 - 0.224098938291644*G0_2_1 - 0.0371867233846402*G0_2_2;
    A[2166] = 0.540186087061082*G0_0_0 + 0.270093043530541*G0_0_1 + 0.270093043530541*G0_0_2 + 0.270093043530541*G0_1_0 + 0.726119703984283*G0_1_1 + 0.27009304353054*G0_1_2 + 0.270093043530541*G0_2_0 + 0.27009304353054*G0_2_1 + 0.72611970398428*G0_2_2;
    A[2167] = -0.0234863516113513*G0_0_0 + 0.0812236326559236*G0_0_1 - 0.0459941052388967*G0_0_2 - 0.104709984267275*G0_1_0 - 0.209419968534551*G0_1_1 + 0.0534966897814129*G0_1_2 + 0.0225077536275456*G0_2_0 + 0.053496689781412*G0_2_1 - 0.268135847562928*G0_2_2;
    A[2168] = 0.0127217737894818*G0_0_1 + 0.0120693751336124*G0_0_2 + 0.0127217737894827*G0_1_0 + 0.0391439193522532*G0_1_1 + 0.0446893079271557*G0_1_2 + 0.0120693751336117*G0_2_0 + 0.0446893079271549*G0_2_1 + 0.0502346965020578*G0_2_2;
    A[2169] = 0.0181040627004173*G0_0_1 + 0.0102752788299663*G0_0_2 + 0.0181040627004169*G0_1_0 + 0.0195719596761257*G0_1_1 + 0.0298472385060915*G0_1_2 + 0.0102752788299654*G0_2_0 + 0.0298472385060918*G0_2_1 + 0.040122517336058*G0_2_2;
    A[2170] = 0.0127217737894813*G0_0_1 + 0.0120693751336108*G0_0_2 + 0.012721773789481*G0_1_0 + 0.00065239865587015*G0_1_2 + 0.0120693751336119*G0_2_0 + 0.000652398655871538*G0_2_1 + 0.00130479731174308*G0_2_2;
    A[2171] = -0.0283793415303825*G0_0_1 - 0.0141896707651925*G0_0_2 - 0.0283793415303843*G0_1_0 + 0.0929668084615997*G0_1_1 + 0.0322937334656075*G0_1_2 - 0.0141896707651921*G0_2_0 + 0.0322937334656068*G0_2_1 - 0.00880738185425785*G0_2_2;
    A[2172] = -0.0283793415303845*G0_0_1 - 0.0141896707651917*G0_0_2 - 0.0283793415303818*G0_1_0 - 0.0141896707651911*G0_1_2 - 0.0141896707651905*G0_2_0 - 0.0141896707651917*G0_2_1 - 0.0332723314494147*G0_2_2;
    A[2173] = -0.104709984267275*G0_0_1 - 0.0368605240567038*G0_0_2 - 0.104709984267276*G0_1_0 - 0.0678494602105702*G0_1_2 - 0.0368605240567039*G0_2_0 - 0.0678494602105703*G0_2_1 - 0.0965550010688874*G0_2_2;
    A[2174] = 0.0120693751336126*G0_0_1 + 0.012721773789482*G0_0_2 + 0.0120693751336115*G0_1_0 + 0.0502346965020578*G0_1_1 + 0.044689307927155*G0_1_2 + 0.0127217737894822*G0_2_0 + 0.044689307927155*G0_2_1 + 0.0391439193522518*G0_2_2;
    A[2175] = 0.0102752788299662*G0_0_1 + 0.0181040627004172*G0_0_2 + 0.0102752788299658*G0_1_0 + 0.0401225173360579*G0_1_1 + 0.0298472385060926*G0_1_2 + 0.0181040627004165*G0_2_0 + 0.0298472385060929*G0_2_1 + 0.0195719596761265*G0_2_2;
    A[2176] = 0.012069375133611*G0_0_1 + 0.0127217737894816*G0_0_2 + 0.0120693751336108*G0_1_0 + 0.00130479731174124*G0_1_1 + 0.000652398655870857*G0_1_2 + 0.012721773789482*G0_2_0 + 0.000652398655870891*G0_2_1;
    A[2177] = -0.0141896707651924*G0_0_1 - 0.0283793415303833*G0_0_2 - 0.0141896707651907*G0_1_0 - 0.00880738185425632*G0_1_1 + 0.032293733465609*G0_1_2 - 0.0283793415303822*G0_2_0 + 0.0322937334656082*G0_2_1 + 0.0929668084615997*G0_2_2;
    A[2178] = -0.0141896707651913*G0_0_1 - 0.0283793415303826*G0_0_2 - 0.0141896707651916*G0_1_0 - 0.0332723314494142*G0_1_1 - 0.0141896707651924*G0_1_2 - 0.0283793415303834*G0_2_0 - 0.0141896707651916*G0_2_1;
    A[2179] = -0.036860524056704*G0_0_1 - 0.104709984267275*G0_0_2 - 0.0368605240567056*G0_1_0 - 0.09655500106889*G0_1_1 - 0.0678494602105701*G0_1_2 - 0.104709984267275*G0_2_0 - 0.0678494602105708*G0_2_1;
    A[2180] = -0.0929668084615997*G0_0_1 - 0.092966808461599*G0_0_2 - 0.0929668084616015*G0_1_0 - 0.234863516113515*G0_1_1 - 0.215291556437389*G0_1_2 - 0.0929668084615981*G0_2_0 - 0.215291556437386*G0_2_1 - 0.234863516113512*G0_2_2;
    A[2181] = -0.0929668084615995*G0_0_1 - 0.0929668084616009*G0_0_2 - 0.0929668084615983*G0_1_0 - 0.0391439193522531*G0_1_1 - 0.044036909271286*G0_1_2 - 0.0929668084616007*G0_2_0 - 0.0440369092712841*G0_2_1 - 0.039143919352253*G0_2_2;
    A[2182] = 0.102752788299663*G0_0_1 + 0.298472385060925*G0_0_2 + 0.102752788299662*G0_1_0 + 0.352295274170274*G0_1_1 + 0.102752788299662*G0_1_2 + 0.298472385060925*G0_2_0 + 0.102752788299662*G0_2_1 - 0.185933616923199*G0_2_2;
    A[2183] = 0.298472385060923*G0_0_1 + 0.102752788299662*G0_0_2 + 0.298472385060921*G0_1_0 - 0.1859336169232*G0_1_1 + 0.102752788299661*G0_1_2 + 0.102752788299661*G0_2_0 + 0.102752788299663*G0_2_1 + 0.352295274170269*G0_2_2;
    A[2184] = 0.0137421252872641*G0_0_0 + 0.0137421252872642*G0_0_1 + 0.0137421252872642*G0_0_2 + 0.0189195610202553*G0_1_0 + 0.0189195610202552*G0_1_1 + 0.0189195610202553*G0_1_2 + 0.0189195610202553*G0_2_0 + 0.0189195610202553*G0_2_1 + 0.0189195610202554*G0_2_2;
    A[2185] = -0.013742125287264*G0_0_0;
    A[2186] = 0.005177435732991*G0_0_1 + 0.0189195610202554*G0_1_1;
    A[2187] = -0.0143945239431351*G0_0_2 + 0.0335985307773495*G0_2_2;
    A[2188] = -0.0477555816097472*G0_0_1 - 0.0118736555368503*G0_0_2 - 0.111886369481854*G0_1_1 - 0.0212681961813901*G0_1_2 - 0.0212681961813899*G0_2_1;
    A[2189] = 0.0823327103709037*G0_0_1 - 0.00182671623643738*G0_0_2 + 0.20941996853455*G0_1_1 + 0.0359471659384851*G0_1_2 + 0.0359471659384839*G0_2_1 - 0.0528442911255411*G0_2_2;
    A[2190] = -0.146659217839771*G0_0_1 - 0.0109602974186314*G0_0_2 - 0.271397840842283*G0_1_1 - 0.0733296089198862*G0_1_2 - 0.0733296089198849*G0_2_1 + 0.121346149991982*G0_2_2;
    A[2191] = -0.124738623002511*G0_0_1 - 0.192588083213081*G0_0_2 - 0.0962940416065404*G0_1_2 - 0.0962940416065417*G0_2_1 - 0.257045070413124*G0_2_2;
    A[2192] = 0.0641307878721066*G0_0_0 + 0.00939454064454059*G0_0_2 + 0.00939454064454102*G0_2_0;
    A[2193] = -0.127087258163646*G0_0_0 - 0.0377738821749241*G0_0_2 - 0.0377738821749237*G0_2_0 - 0.0150051690850298*G0_2_2;
    A[2194] = 0.124738623002511*G0_0_0 + 0.0623693115012555*G0_0_2 + 0.0623693115012562*G0_2_0 + 0.0476251018785739*G0_2_2;
    A[2195] = -0.124738623002511*G0_0_0 - 0.096294041606541*G0_0_2 - 0.0962940416065417*G0_2_0 - 0.128522535206562*G0_2_2;
    A[2196] = 0.0187890812890812*G0_0_0 + 0.00939454064454056*G0_0_1 + 0.0093945406445404*G0_1_0;
    A[2197] = -0.014874689353856*G0_0_0 - 0.0149399292194425*G0_0_1 - 0.0149399292194431*G0_1_0 - 0.0150051690850295*G0_1_1;
    A[2198] = -0.00117431758056755*G0_0_0 - 0.00123955744615543*G0_0_1 - 0.00123955744615479*G0_1_0 - 0.00130479731174248*G0_1_1;
    A[2199] = -0.01187365553685*G0_0_0 - 0.0212681961813898*G0_0_1 - 0.0212681961813901*G0_1_0 - 0.0306627368259305*G0_1_1;
    A[2200] = -0.064130787872107*G0_0_0 - 0.064130787872107*G0_0_1 - 0.0547362472275668*G0_0_2 - 0.111886369481854*G0_1_0 - 0.111886369481854*G0_1_1 - 0.0906181733004642*G0_1_2 - 0.0906181733004641*G0_2_0 - 0.0906181733004641*G0_2_1 - 0.0693499771190741*G0_2_2;
    A[2201] = 0.127087258163647*G0_0_0 + 0.127087258163647*G0_0_1 + 0.0893133759887234*G0_0_2 + 0.20941996853455*G0_1_0 + 0.20941996853455*G0_1_1 + 0.173472802596065*G0_1_2 + 0.173472802596066*G0_2_0 + 0.173472802596065*G0_2_1 + 0.0846813455320404*G0_2_2;
    A[2202] = -0.124738623002511*G0_0_0 - 0.124738623002511*G0_0_1 - 0.0623693115012562*G0_0_2 - 0.271397840842282*G0_1_0 - 0.271397840842282*G0_1_1 - 0.198068231922397*G0_1_2 - 0.198068231922397*G0_2_0 - 0.198068231922397*G0_2_1 - 0.00339247301052976*G0_2_2;
    A[2203] = 0.124738623002511*G0_0_0 + 0.124738623002511*G0_0_1 + 0.0284445813959705*G0_0_2 + 0.096294041606541*G0_1_2 + 0.0962940416065416*G0_2_0 + 0.0962940416065417*G0_2_1 - 0.0644569872000414*G0_2_2;
    A[2204] = -0.0187890812890811*G0_0_0 - 0.00939454064454081*G0_0_1 - 0.018789081289081*G0_0_2 - 0.0400572774704712*G0_1_0 - 0.0187890812890812*G0_1_1 - 0.0400572774704713*G0_1_2 - 0.0613254736518617*G0_2_0 - 0.0400572774704716*G0_2_1 - 0.0613254736518617*G0_2_2;
    A[2205] = 0.0148746893538558*G0_0_0 - 6.52398655870772e-05*G0_0_1 + 0.0148746893538558*G0_0_2 + 0.0136351319077011*G0_1_0 - 0.000130479731173962*G0_1_1 + 0.0136351319077011*G0_1_2 - 0.0254435475789643*G0_2_0 + 0.0136351319077012*G0_2_1 - 0.0254435475789643*G0_2_2;
    A[2206] = 0.00117431758056744*G0_0_0 - 6.52398655875225e-05*G0_0_1 + 0.00117431758056746*G0_0_2 - 0.0137656116388749*G0_1_0 - 0.000130479731174499*G0_1_1 - 0.0137656116388749*G0_1_2 - 0.0528442911255396*G0_2_0 - 0.0137656116388754*G0_2_1 - 0.0528442911255395*G0_2_2;
    A[2207] = 0.01187365553685*G0_0_0 - 0.00939454064454003*G0_0_1 + 0.01187365553685*G0_0_2 + 0.0212681961813901*G0_1_0 - 0.018789081289081*G0_1_1 + 0.0212681961813901*G0_1_2 + 0.0212681961813903*G0_2_1;
    A[2208] = -0.00939454064454065*G0_0_1 - 0.00939454064454048*G0_0_2 - 0.00939454064454038*G0_1_0 - 0.0306627368259307*G0_1_1 - 0.0306627368259308*G0_1_2 - 0.00939454064454037*G0_2_0 - 0.0306627368259305*G0_2_1 - 0.0306627368259305*G0_2_2;
    A[2209] = -6.52398655870367e-05*G0_0_1 - 6.52398655871376e-05*G0_0_2 - 6.52398655871685e-05*G0_1_0 - 0.0013047973117415*G0_1_1 - 0.00130479731174149*G0_1_2 - 6.52398655872706e-05*G0_2_0 - 0.00130479731174177*G0_2_1 - 0.0013047973117418*G0_2_2;
    A[2210] = -6.52398655870494e-05*G0_0_1 - 6.52398655870549e-05*G0_0_2 - 6.52398655871173e-05*G0_1_0 - 0.0150051690850301*G0_1_1 - 0.0150051690850299*G0_1_2 - 6.52398655869791e-05*G0_2_0 - 0.0150051690850295*G0_2_1 - 0.0150051690850294*G0_2_2;
    A[2211] = -0.00939454064454068*G0_0_1 - 0.00939454064454064*G0_0_2 - 0.00939454064454061*G0_1_0 - 0.00939454064454057*G0_2_0;
    A[2212] = -0.105688582251081*G0_0_0 - 0.0528442911255412*G0_0_1 - 0.0300103381700598*G0_0_2 - 0.0528442911255399*G0_1_0 - 0.01500516908503*G0_1_2 - 0.0300103381700601*G0_2_0 - 0.0150051690850296*G0_2_1;
    A[2213] = 0.0137003717732872*G0_0_0 + 0.0198981590040615*G0_0_1 - 0.0127217737894826*G0_0_2 + 0.0198981590040605*G0_1_0 + 0.0260959462348333*G0_1_1 - 0.0120693751336121*G0_1_2 - 0.0127217737894814*G0_2_0 - 0.0120693751336121*G0_2_1;
    A[2214] = -0.0534966897814102*G0_0_0 - 0.000652398655870817*G0_0_1 - 0.05284429112554*G0_0_2 - 0.000652398655869986*G0_1_0 + 0.0521918924696709*G0_1_1 - 0.0378391220405097*G0_1_2 - 0.0528442911255404*G0_2_0 - 0.0378391220405096*G0_2_1;
    A[2215] = 0.209419968534549*G0_0_0 + 0.104709984267275*G0_0_1 + 0.0737210481134076*G0_0_2 + 0.104709984267273*G0_1_0 + 0.0368605240567039*G0_1_2 + 0.0737210481134082*G0_2_0 + 0.0368605240567056*G0_2_1 - 0.0228339529554804*G0_2_2;
    A[2216] = 0.0234863516113512*G0_0_0 - 0.0812236326559241*G0_0_1 + 0.0459941052388963*G0_0_2 - 0.0812236326559236*G0_1_0 - 0.1859336169232*G0_1_1 + 0.00913358118219216*G0_1_2 + 0.0459941052388966*G0_2_0 + 0.00913358118219207*G0_2_1 - 0.0880738185425679*G0_2_2;
    A[2217] = -0.542795681684566*G0_0_0 - 0.271397840842283*G0_0_1 - 0.271397840842283*G0_0_2 - 0.271397840842283*G0_1_0 - 0.135698920421143*G0_1_2 - 0.271397840842283*G0_2_0 - 0.135698920421142*G0_2_1 + 0.05219189246967*G0_2_2;
    A[2218] = 0.105688582251083*G0_0_0 + 0.0528442911255413*G0_0_1 + 0.0756782440810217*G0_0_2 + 0.10503618359521*G0_1_0 + 0.105688582251081*G0_1_1 + 0.127870136550691*G0_1_2 + 0.180062029020362*G0_2_0 + 0.127870136550691*G0_2_1 + 0.255740273101382*G0_2_2;
    A[2219] = -0.0137003717732892*G0_0_0 + 0.00619778723077322*G0_0_1 - 0.0264221455627705*G0_0_2 + 0.00619778723077312*G0_1_0 + 0.0123955744615471*G0_1_1 + 0.00619778723077281*G0_1_2 + 0.0714376528178597*G0_2_0 + 0.0061977872307735*G0_2_1 + 0.172233245149909*G0_2_2;
    A[2220] = 0.0534966897814107*G0_0_0 + 0.0528442911255404*G0_0_1 + 0.000652398655870428*G0_0_2 + 0.00065239865587025*G0_1_0 + 0.105688582251081*G0_1_1 - 0.0221815542996101*G0_1_2 + 0.0528442911255406*G0_2_0 - 0.0221815542996102*G0_2_1 + 0.105688582251081*G0_2_2;
    A[2221] = -0.209419968534552*G0_0_0 - 0.104709984267276*G0_0_1 - 0.135698920421143*G0_0_2 - 0.290643601190473*G0_1_0 - 0.20941996853455*G0_1_1 - 0.26291665831596*G0_1_2 - 0.390134396210782*G0_2_0 - 0.262916658315963*G0_2_1 - 0.584549195660302*G0_2_2;
    A[2222] = -0.0234863516113513*G0_0_0 - 0.104709984267275*G0_0_1 + 0.0225077536275456*G0_0_2 + 0.0812236326559236*G0_1_0 - 0.209419968534551*G0_1_1 + 0.053496689781412*G0_1_2 - 0.0459941052388967*G0_2_0 + 0.0534966897814129*G0_2_1 - 0.268135847562928*G0_2_2;
    A[2223] = 0.542795681684566*G0_0_0 + 0.271397840842283*G0_0_1 + 0.271397840842284*G0_0_2 + 0.271397840842283*G0_1_0 + 0.542795681684565*G0_1_1 + 0.271397840842282*G0_1_2 + 0.271397840842284*G0_2_0 + 0.271397840842282*G0_2_1 + 0.647179466623905*G0_2_2;
    A[2224] = 0.0528442911255414*G0_0_1 + 0.0378391220405108*G0_0_2 + 0.0528442911255407*G0_1_0 + 0.0521918924696697*G0_1_1 + 0.0900310145101798*G0_1_2 + 0.0378391220405103*G0_2_0 + 0.0900310145101803*G0_2_1 + 0.12787013655069*G0_2_2;
    A[2225] = 0.0061977872307719*G0_0_1 + 0.00554538857490217*G0_0_2 + 0.00619778723077332*G0_1_0 + 0.0260959462348356*G0_1_1 + 0.0381653213684457*G0_1_2 + 0.00554538857490315*G0_2_0 + 0.0381653213684466*G0_2_1 + 0.0502346965020572*G0_2_2;
    A[2226] = 0.0528442911255412*G0_0_1 + 0.037839122040511*G0_0_2 + 0.052844291125541*G0_1_0 + 0.0150051690850298*G0_1_2 + 0.0378391220405098*G0_2_0 + 0.0150051690850299*G0_2_1 + 0.0300103381700594*G0_2_2;
    A[2227] = -0.104709984267277*G0_0_1 - 0.0678494602105719*G0_0_2 - 0.104709984267275*G0_1_0 - 0.185933616923198*G0_1_1 - 0.19506719810539*G0_1_2 - 0.0678494602105706*G0_2_0 - 0.19506719810539*G0_2_1 - 0.29227459783015*G0_2_2;
    A[2228] = -0.104709984267275*G0_0_1 - 0.0678494602105706*G0_0_2 - 0.104709984267276*G0_1_0 - 0.0368605240567044*G0_1_2 - 0.0678494602105716*G0_2_0 - 0.0368605240567056*G0_2_1 - 0.0965550010688899*G0_2_2;
    A[2229] = 0.271397840842284*G0_0_1 + 0.135698920421142*G0_0_2 + 0.271397840842283*G0_1_0 + 0.135698920421141*G0_1_2 + 0.135698920421141*G0_2_0 + 0.135698920421143*G0_2_1 + 0.323589733311952*G0_2_2;
    A[2230] = 0.0150051690850299*G0_0_1 + 0.0300103381700606*G0_0_2 + 0.01500516908503*G0_1_0 + 0.0300103381700599*G0_1_1 - 0.00782878387044984*G0_1_2 + 0.0300103381700598*G0_2_0 - 0.00782878387045079*G0_2_1 - 0.0456679059109605*G0_2_2;
    A[2231] = 0.000652398655870594*G0_0_1 + 0.00130479731174101*G0_0_2 + 0.000652398655870986*G0_1_0 + 0.00130479731174139*G0_1_1 - 0.0107645778218688*G0_1_2 + 0.0013047973117419*G0_2_0 - 0.0107645778218704*G0_2_1 - 0.0228339529554808*G0_2_2;
    A[2232] = 0.0150051690850298*G0_0_1 + 0.03001033817006*G0_0_2 + 0.0150051690850298*G0_1_0 + 0.0300103381700599*G0_1_1 + 0.0150051690850303*G0_1_2 + 0.0300103381700599*G0_2_0 + 0.0150051690850299*G0_2_1;
    A[2233] = 0.000652398655871233*G0_0_1 + 0.0127217737894811*G0_0_2 + 0.000652398655870388*G0_1_0 + 0.00130479731174199*G0_1_1 + 0.000652398655870605*G0_1_2 + 0.0127217737894815*G0_2_0 + 0.000652398655871434*G0_2_1 - 0.0880738185425676*G0_2_2;
    A[2234] = 0.00065239865587176*G0_0_1 + 0.0127217737894824*G0_0_2 + 0.000652398655871631*G0_1_0 + 0.0013047973117422*G0_1_1 + 0.0120693751336116*G0_1_2 + 0.012721773789483*G0_2_0 + 0.0120693751336118*G0_2_1;
    A[2235] = 0.0150051690850295*G0_0_1 + 0.0528442911255402*G0_0_2 + 0.0150051690850298*G0_1_0 + 0.0300103381700596*G0_1_1 + 0.0378391220405098*G0_1_2 + 0.0528442911255397*G0_2_0 + 0.0378391220405097*G0_2_1;
    A[2236] = -0.0260959462348382*G0_0_1 - 0.0750258454251536*G0_0_2 - 0.0260959462348332*G0_1_0 - 0.0521918924696692*G0_1_1 - 0.0424059126316062*G0_1_2 - 0.0750258454251507*G0_2_0 - 0.0424059126316075*G0_2_1 + 0.143527704291591*G0_2_2;
    A[2237] = -0.0260959462348337*G0_0_1 - 0.0750258454251489*G0_0_2 - 0.0260959462348371*G0_1_0 - 0.0521918924696695*G0_1_1 - 0.0260959462348353*G0_1_2 - 0.0750258454251505*G0_2_0 - 0.0260959462348351*G0_2_1 + 0.0456679059109607*G0_2_2;
    A[2238] = -0.0260959462348329*G0_0_1 - 0.0587158790283775*G0_0_2 - 0.0260959462348339*G0_1_0 - 0.0521918924696716*G0_1_1 - 0.00978597983806234*G0_1_2 - 0.0587158790283787*G0_2_0 - 0.00978597983806335*G0_2_1 + 0.176147637085136*G0_2_2;
    A[2239] = 0.185933616923201*G0_0_1 + 0.316413348097377*G0_0_2 + 0.1859336169232*G0_1_0 + 0.371867233846398*G0_1_1 + 0.185933616923199*G0_1_2 + 0.316413348097374*G0_2_0 + 0.185933616923198*G0_2_1 - 0.150051690850299*G0_2_2;
    A[2240] = 0.0335985307773497*G0_0_0 + 0.0335985307773498*G0_0_1 + 0.0335985307773498*G0_0_2 + 0.0479930547204848*G0_1_0 + 0.0479930547204849*G0_1_1 + 0.0479930547204849*G0_1_2 + 0.0335985307773496*G0_2_0 + 0.0335985307773497*G0_2_1 + 0.0335985307773498*G0_2_2;
    A[2241] = 0.0189195610202553*G0_0_0 + 0.00517743573299117*G0_1_0;
    A[2242] = -0.0137421252872642*G0_1_1;
    A[2243] = 0.005177435732991*G0_1_2 + 0.0189195610202551*G0_2_2;
    A[2244] = 0.0187890812890812*G0_1_1 + 0.00939454064454066*G0_1_2 + 0.00939454064454083*G0_2_1;
    A[2245] = -0.0148746893538567*G0_1_1 - 0.0149399292194439*G0_1_2 - 0.0149399292194444*G0_2_1 - 0.0150051690850309*G0_2_2;
    A[2246] = -0.00117431758056676*G0_1_1 - 0.00123955744615388*G0_1_2 - 0.00123955744615341*G0_2_1 - 0.00130479731174173*G0_2_2;
    A[2247] = -0.0118736555368508*G0_1_1 - 0.0212681961813911*G0_1_2 - 0.0212681961813914*G0_2_1 - 0.030662736825931*G0_2_2;
    A[2248] = -0.0613254736518621*G0_0_0 - 0.0212681961813908*G0_0_2 - 0.0425363923627813*G0_1_0 - 0.0118736555368502*G0_1_2 - 0.0212681961813909*G0_2_0;
    A[2249] = -0.0254435475789635*G0_0_0 - 0.0390786794866649*G0_0_2 - 0.0403182369328196*G0_1_0 - 0.0540186087061081*G0_1_2 - 0.0390786794866646*G0_2_0 - 0.0528442911255407*G0_2_2;
    A[2250] = -0.0528442911255403*G0_0_0 - 0.039078679486665*G0_0_2 - 0.0540186087061081*G0_1_0 - 0.0403182369328198*G0_1_2 - 0.0390786794866651*G0_2_0 - 0.0254435475789636*G0_2_2;
    A[2251] = -0.0212681961813908*G0_0_2 - 0.0118736555368508*G0_1_0 - 0.0425363923627815*G0_1_2 - 0.0212681961813913*G0_2_0 - 0.0613254736518626*G0_2_2;
    A[2252] = -0.030662736825931*G0_0_0 - 0.0212681961813908*G0_0_1 - 0.0212681961813903*G0_1_0 - 0.0118736555368503*G0_1_1;
    A[2253] = -0.00130479731174201*G0_0_0 - 0.00123955744615449*G0_0_1 - 0.00123955744615494*G0_1_0 - 0.00117431758056745*G0_1_1;
    A[2254] = -0.0150051690850294*G0_0_0 - 0.0149399292194427*G0_0_1 - 0.0149399292194425*G0_1_0 - 0.0148746893538558*G0_1_1;
    A[2255] = 0.00939454064454084*G0_0_1 + 0.00939454064454069*G0_1_0 + 0.0187890812890817*G0_1_1;
    A[2256] = -0.257045070413124*G0_0_0 - 0.257045070413124*G0_0_1 - 0.160751028806583*G0_0_2 - 0.0644569872000419*G0_1_0 - 0.064456987200042*G0_1_1 - 0.0929015685960123*G0_1_2 - 0.160751028806583*G0_2_0 - 0.160751028806583*G0_2_1 - 0.0644569872000416*G0_2_2;
    A[2257] = 0.121346149991983*G0_0_0 + 0.121346149991983*G0_0_1 + 0.194675758911869*G0_0_2 + 0.132306447410613*G0_1_0 + 0.132306447410613*G0_1_1 + 0.0589768384907273*G0_1_2 + 0.194675758911869*G0_2_0 + 0.194675758911869*G0_2_1 - 0.00339247301052875*G0_2_2;
    A[2258] = -0.0528442911255412*G0_0_0 - 0.0528442911255411*G0_0_1 - 0.088791457064026*G0_0_2 - 0.0510175748891026*G0_1_0 - 0.0510175748891028*G0_1_1 - 0.00463203045668354*G0_1_2 - 0.0887914570640261*G0_2_0 - 0.0887914570640261*G0_2_1 + 0.0846813455320392*G0_2_2;
    A[2259] = 0.021268196181391*G0_0_2 + 0.0118736555368507*G0_1_0 + 0.0118736555368509*G0_1_1 - 0.014613729891507*G0_1_2 + 0.0212681961813913*G0_2_0 + 0.0212681961813913*G0_2_1 - 0.0693499771190733*G0_2_2;
    A[2260] = -0.128522535206562*G0_0_0 - 0.032228493600021*G0_0_1 - 0.128522535206562*G0_0_2 - 0.0322284936000208*G0_1_0 - 0.0606730749959911*G0_1_1 - 0.0322284936000209*G0_1_2 - 0.128522535206562*G0_2_0 - 0.0322284936000206*G0_2_1 - 0.128522535206562*G0_2_2;
    A[2261] = 0.0476251018785738*G0_0_0 - 0.0147442096226817*G0_0_1 + 0.0476251018785737*G0_0_2 - 0.0147442096226821*G0_1_0 + 0.0476251018785737*G0_1_1 - 0.0147442096226821*G0_1_2 + 0.0476251018785738*G0_2_0 - 0.0147442096226822*G0_2_1 + 0.0476251018785737*G0_2_2;
    A[2262] = -0.0150051690850297*G0_0_0 + 0.0227687130898935*G0_0_1 - 0.0150051690850297*G0_0_2 + 0.0227687130898942*G0_1_0 - 0.066544662898829*G0_1_1 + 0.0227687130898943*G0_1_2 - 0.0150051690850296*G0_2_0 + 0.0227687130898938*G0_2_1 - 0.0150051690850296*G0_2_2;
    A[2263] = -0.00939454064454058*G0_0_1 - 0.00939454064454066*G0_1_0 + 0.0453417065830257*G0_1_1 - 0.00939454064454068*G0_1_2 - 0.0093945406445406*G0_2_1;
    A[2264] = -0.0644569872000419*G0_0_0 - 0.160751028806583*G0_0_1 - 0.160751028806583*G0_0_2 - 0.0929015685960123*G0_1_0 - 0.0644569872000422*G0_1_1 - 0.064456987200042*G0_1_2 - 0.160751028806582*G0_2_0 - 0.257045070413124*G0_2_1 - 0.257045070413124*G0_2_2;
    A[2265] = -0.00339247301052869*G0_0_0 + 0.194675758911869*G0_0_1 + 0.194675758911869*G0_0_2 + 0.0589768384907272*G0_1_0 + 0.132306447410613*G0_1_1 + 0.132306447410613*G0_1_2 + 0.194675758911868*G0_2_0 + 0.121346149991982*G0_2_1 + 0.121346149991982*G0_2_2;
    A[2266] = 0.0846813455320391*G0_0_0 - 0.0887914570640263*G0_0_1 - 0.0887914570640262*G0_0_2 - 0.00463203045668354*G0_1_0 - 0.0510175748891024*G0_1_1 - 0.0510175748891024*G0_1_2 - 0.0887914570640255*G0_2_0 - 0.0528442911255407*G0_2_1 - 0.0528442911255407*G0_2_2;
    A[2267] = -0.0693499771190738*G0_0_0 + 0.0212681961813906*G0_0_1 + 0.0212681961813905*G0_0_2 - 0.0146137298915074*G0_1_0 + 0.0118736555368501*G0_1_1 + 0.0118736555368501*G0_1_2 + 0.0212681961813902*G0_2_0;
    A[2268] = -0.0456679059109604*G0_0_0 - 0.0756782440810204*G0_0_1 - 0.03783912204051*G0_0_2 - 0.0756782440810209*G0_1_0 - 0.105688582251082*G0_1_1 - 0.0528442911255408*G0_1_2 - 0.03783912204051*G0_2_0 - 0.0528442911255409*G0_2_1;
    A[2269] = -0.0228339529554787*G0_0_0 - 0.0241387502672215*G0_0_1 - 0.0120693751336101*G0_0_2 - 0.0241387502672216*G0_1_0 - 0.0254435475789631*G0_1_1 - 0.0127217737894814*G0_1_2 - 0.0120693751336105*G0_2_0 - 0.0127217737894812*G0_2_1;
    A[2270] = -0.0300103381700601*G0_0_1 - 0.0150051690850291*G0_0_2 - 0.0300103381700616*G0_1_0 - 0.0600206763401223*G0_1_1 - 0.0300103381700602*G0_1_2 - 0.0150051690850313*G0_2_0 - 0.0300103381700614*G0_2_1;
    A[2271] = -0.0880738185425679*G0_0_0 - 0.100795592332049*G0_0_1 - 0.088726217198438*G0_0_2 - 0.10079559233205*G0_1_0 - 0.113517366121531*G0_1_1 - 0.100795592332049*G0_1_2 - 0.0887262171984396*G0_2_0 - 0.100795592332049*G0_2_1 - 0.0880738185425671*G0_2_2;
    A[2272] = -0.0127217737894821*G0_0_1 - 0.0120693751336118*G0_0_2 - 0.0127217737894818*G0_1_0 - 0.0254435475789624*G0_1_1 - 0.0241387502672227*G0_1_2 - 0.0120693751336123*G0_2_0 - 0.0241387502672229*G0_2_1 - 0.0228339529554821*G0_2_2;
    A[2273] = -0.0528442911255413*G0_0_1 - 0.0378391220405109*G0_0_2 - 0.0528442911255412*G0_1_0 - 0.105688582251083*G0_1_1 - 0.0756782440810213*G0_1_2 - 0.0378391220405107*G0_2_0 - 0.0756782440810225*G0_2_1 - 0.0456679059109618*G0_2_2;
    A[2274] = 0.0521918924696691*G0_0_0 + 0.323589733311953*G0_0_1 + 0.187890812890812*G0_0_2 + 0.323589733311953*G0_1_0 + 0.0521918924696695*G0_1_1 + 0.187890812890811*G0_1_2 + 0.187890812890812*G0_2_0 + 0.187890812890812*G0_2_1 + 0.323589733311953*G0_2_2;
    A[2275] = -0.022833952955481*G0_0_0 - 0.0965550010688892*G0_0_1 - 0.059694477012186*G0_0_2 - 0.0965550010688896*G0_1_0 + 0.0391439193522527*G0_1_1 - 0.0287055408583189*G0_1_2 - 0.0596944770121852*G0_2_0 - 0.0287055408583187*G0_2_1 - 0.0965550010688899*G0_2_2;
    A[2276] = 0.0300103381700604*G0_0_1 + 0.0150051690850307*G0_0_2 + 0.0300103381700615*G0_1_0 - 0.0456679059109617*G0_1_1 - 0.00782878387045039*G0_1_2 + 0.0150051690850313*G0_2_0 - 0.00782878387045077*G0_2_1 + 0.0300103381700604*G0_2_2;
    A[2277] = -0.0880738185425679*G0_0_0 - 0.134067923781465*G0_0_1 - 0.0972073997247597*G0_0_2 - 0.134067923781464*G0_1_0 - 0.15657567740901*G0_1_1 - 0.22442513761958*G0_1_2 - 0.0972073997247596*G0_2_0 - 0.22442513761958*G0_2_1 - 0.292274597830151*G0_2_2;
    A[2278] = 0.0127217737894827*G0_0_1 + 0.0120693751336117*G0_0_2 + 0.0127217737894818*G0_1_0 + 0.0391439193522532*G0_1_1 + 0.0446893079271549*G0_1_2 + 0.0120693751336124*G0_2_0 + 0.0446893079271557*G0_2_1 + 0.0502346965020578*G0_2_2;
    A[2279] = 0.0528442911255407*G0_0_1 + 0.0378391220405103*G0_0_2 + 0.0528442911255414*G0_1_0 + 0.0521918924696697*G0_1_1 + 0.0900310145101803*G0_1_2 + 0.0378391220405108*G0_2_0 + 0.0900310145101798*G0_2_1 + 0.12787013655069*G0_2_2;
    A[2280] = 0.647179466623905*G0_0_0 + 0.375781625781621*G0_0_1 + 0.375781625781622*G0_0_2 + 0.375781625781621*G0_1_0 + 0.647179466623904*G0_1_1 + 0.375781625781622*G0_1_2 + 0.375781625781622*G0_2_0 + 0.375781625781622*G0_2_1 + 0.647179466623906*G0_2_2;
    A[2281] = -0.584549195660302*G0_0_0 - 0.194414799449519*G0_0_1 - 0.32163253734434*G0_0_2 - 0.448850275239159*G0_1_0 - 0.268135847562928*G0_1_1 - 0.290643601190473*G0_1_2 - 0.32163253734434*G0_2_0 - 0.222141742324032*G0_2_1 - 0.268135847562928*G0_2_2;
    A[2282] = 0.255740273101383*G0_0_0 + 0.0756782440810208*G0_0_1 + 0.127870136550691*G0_0_2 + 0.180062029020361*G0_1_0 + 0.105688582251082*G0_1_1 + 0.10503618359521*G0_1_2 + 0.127870136550692*G0_2_0 + 0.0528442911255414*G0_2_1 + 0.105688582251082*G0_2_2;
    A[2283] = -0.268135847562928*G0_0_0 - 0.222141742324031*G0_0_1 - 0.321632537344339*G0_0_2 - 0.290643601190472*G0_1_0 - 0.268135847562927*G0_1_1 - 0.44885027523916*G0_1_2 - 0.321632537344339*G0_2_0 - 0.194414799449519*G0_2_1 - 0.584549195660302*G0_2_2;
    A[2284] = 0.172233245149908*G0_0_0 + 0.100795592332049*G0_0_1 + 0.166035457919136*G0_0_2 + 0.198655390712678*G0_1_0 + 0.113517366121531*G0_1_1 + 0.19865539071268*G0_1_2 + 0.166035457919135*G0_2_0 + 0.100795592332049*G0_2_1 + 0.172233245149909*G0_2_2;
    A[2285] = 0.105688582251082*G0_0_0 + 0.0528442911255412*G0_0_1 + 0.127870136550691*G0_0_2 + 0.105036183595211*G0_1_0 + 0.105688582251083*G0_1_1 + 0.18006202902036*G0_1_2 + 0.127870136550691*G0_2_0 + 0.0756782440810225*G0_2_1 + 0.255740273101382*G0_2_2;
    A[2286] = 0.323589733311953*G0_0_0 + 0.187890812890811*G0_0_1 + 0.18789081289081*G0_0_2 + 0.18789081289081*G0_1_0 + 0.0521918924696691*G0_1_1 + 0.323589733311951*G0_1_2 + 0.187890812890811*G0_2_0 + 0.323589733311953*G0_2_1 + 0.0521918924696697*G0_2_2;
    A[2287] = -0.292274597830152*G0_0_0 - 0.224425137619581*G0_0_1 - 0.0972073997247608*G0_0_2 - 0.224425137619579*G0_1_0 - 0.15657567740901*G0_1_1 - 0.134067923781465*G0_1_2 - 0.09720739972476*G0_2_0 - 0.134067923781465*G0_2_1 - 0.0880738185425683*G0_2_2;
    A[2288] = 0.127870136550692*G0_0_0 + 0.0900310145101812*G0_0_1 + 0.0378391220405106*G0_0_2 + 0.0900310145101806*G0_1_0 + 0.05219189246967*G0_1_1 + 0.0528442911255413*G0_1_2 + 0.037839122040511*G0_2_0 + 0.0528442911255408*G0_2_1;
    A[2289] = -0.0965550010688891*G0_0_0 - 0.0287055408583174*G0_0_1 - 0.0596944770121839*G0_0_2 - 0.0287055408583178*G0_1_0 + 0.0391439193522539*G0_1_1 - 0.0965550010688882*G0_1_2 - 0.0596944770121848*G0_2_0 - 0.0965550010688888*G0_2_1 - 0.0228339529554799*G0_2_2;
    A[2290] = 0.0502346965020573*G0_0_0 + 0.0446893079271547*G0_0_1 + 0.01206937513361*G0_0_2 + 0.0446893079271547*G0_1_0 + 0.0391439193522527*G0_1_1 + 0.0127217737894814*G0_1_2 + 0.0120693751336113*G0_2_0 + 0.0127217737894824*G0_2_1;
    A[2291] = 0.0300103381700592*G0_0_0 - 0.00782878387045145*G0_0_1 + 0.0150051690850292*G0_0_2 - 0.00782878387045164*G0_1_0 - 0.0456679059109627*G0_1_1 + 0.0300103381700603*G0_1_2 + 0.0150051690850291*G0_2_0 + 0.0300103381700592*G0_2_1;
    A[2292] = -0.1500516908503*G0_0_0 - 0.466465038947674*G0_0_1 - 0.335985307773499*G0_0_2 - 0.466465038947674*G0_1_0 - 0.782878387045047*G0_1_1 - 0.466465038947672*G0_1_2 - 0.335985307773499*G0_2_0 - 0.466465038947673*G0_2_1 - 0.1500516908503*G0_2_2;
    A[2293] = 0.14352770429159*G0_0_0 + 0.218553549716741*G0_0_1 + 0.185933616923198*G0_0_2 + 0.218553549716743*G0_1_0 + 0.293579395141891*G0_1_1 + 0.234863516113513*G0_1_2 + 0.185933616923197*G0_2_0 + 0.234863516113512*G0_2_1 + 0.176147637085134*G0_2_2;
    A[2294] = 0.04566790591096*G0_0_0 + 0.120693751336113*G0_0_1 + 0.0717638521457961*G0_0_2 + 0.120693751336111*G0_1_0 + 0.195719596761266*G0_1_1 + 0.120693751336111*G0_1_2 + 0.0717638521457957*G0_2_0 + 0.120693751336113*G0_2_1 + 0.0456679059109622*G0_2_2;
    A[2295] = 0.176147637085136*G0_0_0 + 0.234863516113513*G0_0_1 + 0.1859336169232*G0_0_2 + 0.234863516113514*G0_1_0 + 0.293579395141891*G0_1_1 + 0.218553549716743*G0_1_2 + 0.185933616923199*G0_2_0 + 0.218553549716742*G0_2_1 + 0.143527704291593*G0_2_2;
    A[2296] = -0.0298472385060919*G0_0_0 - 0.029847238506092*G0_0_1 - 0.029847238506092*G0_0_2 - 0.034571909571909*G0_1_0 - 0.034571909571909*G0_1_1 - 0.0345719095719089*G0_1_2 - 0.029847238506092*G0_2_0 - 0.0298472385060921*G0_2_1 - 0.029847238506092*G0_2_2;
    A[2297] = -0.0298472385060925*G0_0_0 + 0.00472467106581688*G0_1_0;
    A[2298] = -0.00946499969937454*G0_1_1;
    A[2299] = 0.00227817610630123*G0_1_2 + 0.0117431758056757*G0_2_2;
    A[2300] = 0.00658922642429559*G0_1_1 + 0.00329461321214772*G0_1_2 + 0.0032946132121482*G0_2_1;
    A[2301] = 0.00450155072550967*G0_1_1 + 0.00192457603481958*G0_1_2 + 0.00192457603481917*G0_2_1 - 0.000652398655870245*G0_2_2;
    A[2302] = -0.0105036183595218*G0_1_1 - 0.0130805930502113*G0_1_2 - 0.013080593050211*G0_2_1 - 0.0156575677409012*G0_2_2;
    A[2303] = -0.00254435475789577*G0_1_1 - 0.00583896797004405*G0_1_2 - 0.00583896797004432*G0_2_1 - 0.00913358118219214*G0_2_2;
    A[2304] = 0.128522535206562*G0_0_0 + 0.0422754329004329*G0_0_2 + 0.0845508658008658*G0_1_0 + 0.0429604514890974*G0_1_2 + 0.0422754329004331*G0_2_0;
    A[2305] = 0.0371867233846389*G0_0_0 + 0.0743408268364856*G0_0_2 + 0.0808321934624002*G0_1_0 + 0.108232937008978*G0_1_2 + 0.0743408268364849*G0_2_0 + 0.104709984267274*G0_2_2;
    A[2306] = -0.00619778723077301*G0_0_0 + 0.0188869410874617*G0_0_2 + 0.0093945406445408*G0_1_0 + 0.0377738821749232*G0_1_2 + 0.0188869410874621*G0_2_0 + 0.0567586830607663*G0_2_2;
    A[2307] = -0.00583896797004408*G0_0_2 - 0.00254435475789574*G0_1_0 - 0.0142875305635715*G0_1_2 - 0.00583896797004429*G0_2_0 - 0.0435476102793807*G0_2_2;
    A[2308] = 0.0642612676032811*G0_0_0 + 0.042275432900433*G0_0_1 + 0.0422754329004328*G0_1_0 + 0.0429604514890975*G0_1_1;
    A[2309] = 0.00880738185425693*G0_0_0 + 0.00649136662591492*G0_0_1 + 0.00649136662591517*G0_1_0 + 0.00352295274170224*G0_1_1;
    A[2310] = -0.000652398655871144*G0_0_0 - 0.00949240044292112*G0_0_1 - 0.00949240044292105*G0_1_0 - 0.0189848008858418*G0_1_1;
    A[2311] = 0.00329461321214779*G0_0_1 + 0.00329461321214776*G0_1_0 + 0.0292600797158083*G0_1_1;
    A[2312] = 0.128522535206562*G0_0_0 + 0.128522535206562*G0_0_1 + 0.0862471023061284*G0_0_2 + 0.0439716694056966*G0_1_0 + 0.0439716694056966*G0_1_1 + 0.0446566879943606*G0_1_2 + 0.0862471023061295*G0_2_0 + 0.0862471023061297*G0_2_1 + 0.0439716694056961*G0_2_2;
    A[2313] = 0.0371867233846397*G0_0_0 + 0.0371867233846398*G0_0_1 - 0.0371541034518455*G0_0_2 - 0.0436454700777613*G0_1_0 - 0.0436454700777612*G0_1_1 - 0.00975335990526898*G0_1_2 - 0.0371541034518462*G0_2_0 - 0.0371541034518463*G0_2_1 - 0.00678494602105647*G0_2_2;
    A[2314] = -0.00619778723077312*G0_0_0 - 0.00619778723077321*G0_0_1 - 0.0250847283182356*G0_0_2 - 0.015592327875314*G0_1_0 - 0.015592327875314*G0_1_1 + 0.00329461321214736*G0_1_2 - 0.0250847283182349*G0_2_0 - 0.0250847283182349*G0_2_1 + 0.0127870136550685*G0_2_2;
    A[2315] = 0.00583896797004425*G0_0_2 + 0.00254435475789585*G0_1_0 + 0.00254435475789593*G0_1_1 - 0.00590420783563142*G0_1_2 + 0.00583896797004434*G0_2_0 + 0.00583896797004438*G0_2_1 - 0.031869674339292*G0_2_2;
    A[2316] = 0.0642612676032811*G0_0_0 + 0.0219858347028478*G0_0_1 + 0.0642612676032811*G0_0_2 + 0.0219858347028482*G0_1_0 + 0.0226708532915123*G0_1_1 + 0.0219858347028483*G0_1_2 + 0.0642612676032814*G0_2_0 + 0.021985834702848*G0_2_1 + 0.0642612676032814*G0_2_2;
    A[2317] = 0.00880738185425675*G0_0_0 + 0.00231601522834194*G0_0_1 + 0.00880738185425677*G0_0_2 + 0.00231601522834183*G0_1_0 - 0.000652398655870379*G0_1_1 + 0.00231601522834177*G0_1_2 + 0.00880738185425623*G0_2_0 + 0.00231601522834185*G0_2_1 + 0.00880738185425628*G0_2_2;
    A[2318] = -0.000652398655871658*G0_0_0 + 0.00884000178704991*G0_0_1 - 0.000652398655871598*G0_0_2 + 0.00884000178704937*G0_1_0 - 0.00065239865587141*G0_1_1 + 0.0088400017870494*G0_1_2 - 0.000652398655871128*G0_2_0 + 0.00884000178704991*G0_2_1 - 0.000652398655871186*G0_2_2;
    A[2319] = -0.00329461321214758*G0_0_1 - 0.00329461321214785*G0_1_0 + 0.0226708532915131*G0_1_1 - 0.00329461321214787*G0_1_2 - 0.00329461321214765*G0_2_1;
    A[2320] = 0.0920860702761729*G0_0_0 + 0.134361503176606*G0_0_1 + 0.134361503176606*G0_0_2 + 0.128000616281865*G0_1_0 + 0.127315597693201*G0_1_1 + 0.127315597693201*G0_1_2 + 0.134361503176605*G0_2_0 + 0.176636936077039*G0_2_1 + 0.176636936077039*G0_2_2;
    A[2321] = -0.0622388317700807*G0_0_0 - 0.250814663249557*G0_0_1 - 0.250814663249557*G0_0_2 - 0.160783648739376*G0_1_0 - 0.166948816037356*G0_1_1 - 0.166948816037356*G0_1_2 - 0.250814663249556*G0_2_0 - 0.334680510461758*G0_2_1 - 0.334680510461758*G0_2_2;
    A[2322] = -0.0622388317700815*G0_0_0 + 0.188575831479476*G0_0_1 + 0.188575831479476*G0_0_2 + 0.0985448169692949*G0_1_0 + 0.182410664181496*G0_1_1 + 0.182410664181496*G0_1_2 + 0.188575831479475*G0_2_0 + 0.104709984267275*G0_2_1 + 0.104709984267275*G0_2_2;
    A[2323] = 0.0920860702761739*G0_0_0 - 0.0422754329004328*G0_0_1 - 0.0422754329004327*G0_0_2 - 0.0359145460056913*G0_1_0 - 0.0429604514890974*G0_1_1 - 0.0429604514890974*G0_1_2 - 0.0422754329004325*G0_2_0;
    A[2324] = 0.039143919352252*G0_0_0 + 0.135698920421142*G0_0_1 + 0.0678494602105708*G0_0_2 + 0.135698920421142*G0_1_0 + 0.209419968534551*G0_1_1 + 0.104709984267276*G0_1_2 + 0.0678494602105707*G0_2_0 + 0.104709984267275*G0_2_1;
    A[2325] = -0.00489298991903267*G0_0_0 + 0.0283793415303823*G0_0_1 + 0.0141896707651906*G0_0_2 + 0.0283793415303826*G0_1_0 + 0.0567586830607657*G0_1_1 + 0.0283793415303824*G0_1_2 + 0.0141896707651906*G0_2_0 + 0.0283793415303828*G0_2_1;
    A[2326] = -0.00130479731174137*G0_0_1 - 0.000652398655870843*G0_0_2 - 0.00130479731174052*G0_1_0 - 0.0254435475789627*G0_1_1 - 0.0127217737894811*G0_1_2 - 0.000652398655870033*G0_2_0 - 0.0127217737894817*G0_2_1;
    A[2327] = 0.019571959676127*G0_0_0 + 0.0748627457611826*G0_0_1 + 0.0606730749959912*G0_0_2 + 0.0748627457611841*G0_1_0 + 0.149725491522365*G0_1_1 + 0.121346149991983*G0_1_2 + 0.0606730749959919*G0_2_0 + 0.121346149991982*G0_2_1 + 0.092966808461599*G0_2_2;
    A[2328] = -0.0181040627004169*G0_0_1 - 0.0102752788299664*G0_0_2 - 0.0181040627004174*G0_1_0 - 0.0166361657247076*G0_1_1 + 0.00146789697570921*G0_1_2 - 0.0102752788299664*G0_2_0 + 0.00146789697570931*G0_2_1 + 0.0195719596761261*G0_2_2;
    A[2329] = -0.00619778723077281*G0_0_1 - 0.0055453885749021*G0_0_2 - 0.00619778723077224*G0_1_0 + 0.0137003717732903*G0_1_1 + 0.0264221455627718*G0_1_2 - 0.00554538857490229*G0_2_0 + 0.0264221455627708*G0_2_1 + 0.039143919352253*G0_2_2;
    A[2330] = 0.0391439193522531*G0_0_0 - 0.0965550010688889*G0_0_1 - 0.0287055408583186*G0_0_2 - 0.0965550010688891*G0_1_0 - 0.0228339529554804*G0_1_1 - 0.0596944770121848*G0_1_2 - 0.0287055408583187*G0_2_0 - 0.059694477012186*G0_2_1 - 0.09655500106889*G0_2_2;
    A[2331] = -0.00489298991903145*G0_0_0 - 0.0332723314494144*G0_0_1 - 0.019082660684222*G0_0_2 - 0.0332723314494145*G0_1_0 - 0.00489298991903157*G0_1_1 - 0.0190826606842223*G0_1_2 - 0.0190826606842226*G0_2_0 - 0.0190826606842221*G0_2_1 - 0.0332723314494128*G0_2_2;
    A[2332] = 0.00130479731174123*G0_0_1 + 0.00065239865587122*G0_0_2 + 0.00130479731174067*G0_1_0 - 0.0228339529554801*G0_1_1 - 0.0107645778218688*G0_1_2 + 0.000652398655870092*G0_2_0 - 0.0107645778218695*G0_2_1 + 0.00130479731174145*G0_2_2;
    A[2333] = 0.0195719596761256*G0_0_0 - 0.0552907860850569*G0_0_1 - 0.0411011153198655*G0_0_2 - 0.0552907860850571*G0_1_0 + 0.0195719596761257*G0_1_1 + 0.00538228891093417*G0_1_2 - 0.0411011153198644*G0_2_0 + 0.0053822889109356*G0_2_1 - 0.00880738185425646*G0_2_2;
    A[2334] = 0.0181040627004169*G0_0_1 + 0.0102752788299654*G0_0_2 + 0.0181040627004173*G0_1_0 + 0.0195719596761257*G0_1_1 + 0.0298472385060918*G0_1_2 + 0.0102752788299663*G0_2_0 + 0.0298472385060915*G0_2_1 + 0.040122517336058*G0_2_2;
    A[2335] = 0.00619778723077332*G0_0_1 + 0.00554538857490315*G0_0_2 + 0.0061977872307719*G0_1_0 + 0.0260959462348356*G0_1_1 + 0.0381653213684466*G0_1_2 + 0.00554538857490216*G0_2_0 + 0.0381653213684457*G0_2_1 + 0.0502346965020572*G0_2_2;
    A[2336] = -0.584549195660302*G0_0_0 - 0.448850275239159*G0_0_1 - 0.32163253734434*G0_0_2 - 0.194414799449519*G0_1_0 - 0.268135847562928*G0_1_1 - 0.222141742324032*G0_1_2 - 0.32163253734434*G0_2_0 - 0.290643601190473*G0_2_1 - 0.268135847562928*G0_2_2;
    A[2337] = 0.912053320907479*G0_0_0 + 0.456026660453739*G0_0_1 + 0.456026660453739*G0_0_2 + 0.456026660453739*G0_1_0 + 0.726119703984279*G0_1_1 + 0.456026660453738*G0_1_2 + 0.456026660453739*G0_2_0 + 0.456026660453738*G0_2_1 + 0.726119703984279*G0_2_2;
    A[2338] = -0.584549195660302*G0_0_0 - 0.135698920421142*G0_0_1 - 0.262916658315962*G0_0_2 - 0.390134396210782*G0_1_0 - 0.209419968534551*G0_1_1 - 0.290643601190473*G0_1_2 - 0.262916658315962*G0_2_0 - 0.104709984267275*G0_2_1 - 0.209419968534549*G0_2_2;
    A[2339] = -0.0371867233846407*G0_0_0 + 0.0376760223765414*G0_0_1 + 0.186912214907004*G0_0_2 + 0.140428810676203*G0_1_0 + 0.0655660649150206*G0_1_1 + 0.0376760223765421*G0_1_2 + 0.186912214907004*G0_2_0 + 0.140428810676204*G0_2_1 - 0.0371867233846403*G0_2_2;
    A[2340] = -0.0371867233846378*G0_0_0 - 0.0748627457611822*G0_0_1 - 0.224098938291644*G0_0_2 - 0.177615534060842*G0_1_0 - 0.149725491522364*G0_1_1 - 0.326851726591306*G0_1_2 - 0.224098938291642*G0_2_0 - 0.121346149991982*G0_2_1 - 0.448197876583289*G0_2_2;
    A[2341] = 0.012395574461546*G0_0_0 + 0.0061977872307729*G0_0_1 + 0.0061977872307737*G0_0_2 + 0.00619778723077308*G0_1_0 - 0.0137003717732904*G0_1_1 + 0.0714376528178611*G0_1_2 + 0.0061977872307728*G0_2_0 - 0.0264221455627708*G0_2_1 + 0.172233245149911*G0_2_2;
    A[2342] = -0.292274597830151*G0_0_0 - 0.09720739972476*G0_0_1 - 0.224425137619579*G0_0_2 - 0.0972073997247596*G0_1_0 - 0.0880738185425679*G0_1_1 - 0.134067923781464*G0_1_2 - 0.224425137619581*G0_2_0 - 0.134067923781465*G0_2_1 - 0.156575677409009*G0_2_2;
    A[2343] = 0.45602666045374*G0_0_0 + 0.228013330226871*G0_0_1 + 0.228013330226871*G0_0_2 + 0.228013330226869*G0_1_0 + 0.0929668084615999*G0_1_1 + 0.36305985199214*G0_1_2 + 0.22801333022687*G0_2_0 + 0.363059851992141*G0_2_1 + 0.0929668084615997*G0_2_2;
    A[2344] = -0.292274597830152*G0_0_0 - 0.195067198105391*G0_0_1 - 0.0678494602105715*G0_0_2 - 0.195067198105391*G0_1_0 - 0.185933616923198*G0_1_1 - 0.104709984267277*G0_1_2 - 0.0678494602105713*G0_2_0 - 0.104709984267275*G0_2_1;
    A[2345] = -0.0088073818542567*G0_0_0 - 0.0411011153198659*G0_0_1 + 0.00538228891093404*G0_0_2 - 0.0411011153198649*G0_1_0 + 0.0195719596761256*G0_1_1 - 0.0552907860850567*G0_1_2 + 0.00538228891093563*G0_2_0 - 0.0552907860850563*G0_2_1 + 0.0195719596761261*G0_2_2;
    A[2346] = -0.00880738185425705*G0_0_0 + 0.032293733465608*G0_0_1 - 0.0141896707651905*G0_0_2 + 0.0322937334656079*G0_1_0 + 0.0929668084615987*G0_1_1 - 0.0283793415303822*G0_1_2 - 0.014189670765192*G0_2_0 - 0.0283793415303833*G0_2_1;
    A[2347] = 0.00130479731174266*G0_0_0 + 0.000652398655871512*G0_0_1 + 0.000652398655870713*G0_0_2 + 0.000652398655871813*G0_1_0 - 0.0880738185425668*G0_1_1 + 0.0127217737894811*G0_1_2 + 0.000652398655871214*G0_2_0 + 0.0127217737894823*G0_2_1;
    A[2348] = -0.0391439193522543*G0_0_0 + 0.44526208263187*G0_0_1 + 0.249542485870607*G0_0_2 + 0.445262082631869*G0_1_0 + 0.146789697570947*G0_1_1 + 0.249542485870607*G0_1_2 + 0.249542485870609*G0_2_0 + 0.24954248587061*G0_2_1 + 0.352295274170271*G0_2_2;
    A[2349] = -0.0391439193522506*G0_0_0 - 0.484406001984121*G0_0_1 - 0.288686405222861*G0_0_2 - 0.484406001984121*G0_1_0 - 0.782878387045045*G0_1_1 - 0.484406001984122*G0_1_2 - 0.288686405222861*G0_2_0 - 0.484406001984121*G0_2_1 - 0.185933616923198*G0_2_2;
    A[2350] = 0.0097859798380637*G0_0_0 + 0.00489298991903083*G0_0_1 + 0.00489298991903194*G0_0_2 + 0.00489298991903148*G0_1_0 + 0.146789697570944*G0_1_1 + 0.0538228891093468*G0_1_2 + 0.00489298991903169*G0_2_0 + 0.0538228891093444*G0_2_1 - 0.0391439193522527*G0_2_2;
    A[2351] = -0.0391439193522524*G0_0_0 - 0.019571959676125*G0_0_1 - 0.0195719596761259*G0_0_2 - 0.0195719596761268*G0_1_0 - 0.0489298991903129*G0_1_1 - 0.141896707651915*G0_1_2 - 0.0195719596761275*G0_2_0 - 0.141896707651914*G0_2_1 - 0.234863516113515*G0_2_2;
    A[2352] = 0.018919561020255*G0_0_0 + 0.018919561020255*G0_0_1 + 0.018919561020255*G0_0_2 + 0.0137421252872641*G0_1_0 + 0.0137421252872642*G0_1_1 + 0.0137421252872642*G0_1_2 + 0.0189195610202549*G0_2_0 + 0.0189195610202549*G0_2_1 + 0.0189195610202549*G0_2_2;
    A[2353] = 0.0335985307773502*G0_0_0 - 0.0143945239431346*G0_1_0;
    A[2354] = -0.0137421252872643*G0_1_1;
    A[2355] = 0.00517743573299109*G0_1_2 + 0.0189195610202553*G0_2_2;
    A[2356] = 0.0187890812890809*G0_1_1 + 0.00939454064454016*G0_1_2 + 0.00939454064454028*G0_2_1;
    A[2357] = -0.0148746893538561*G0_1_1 - 0.0149399292194433*G0_1_2 - 0.014939929219443*G0_2_1 - 0.01500516908503*G0_2_2;
    A[2358] = -0.00117431758056702*G0_1_1 - 0.00123955744615431*G0_1_2 - 0.00123955744615432*G0_2_1 - 0.00130479731174208*G0_2_2;
    A[2359] = -0.0118736555368506*G0_1_1 - 0.0212681961813907*G0_1_2 - 0.0212681961813907*G0_2_1 - 0.0306627368259309*G0_2_2;
    A[2360] = -0.257045070413124*G0_0_0 - 0.0962940416065403*G0_0_2 - 0.192588083213081*G0_1_0 - 0.124738623002509*G0_1_2 - 0.0962940416065413*G0_2_0;
    A[2361] = 0.121346149991983*G0_0_0 - 0.0733296089198865*G0_0_2 - 0.0109602974186315*G0_1_0 - 0.146659217839774*G0_1_2 - 0.0733296089198858*G0_2_0 - 0.271397840842284*G0_2_2;
    A[2362] = -0.0528442911255414*G0_0_0 + 0.0359471659384849*G0_0_2 - 0.00182671623643766*G0_1_0 + 0.082332710370905*G0_1_2 + 0.0359471659384851*G0_2_0 + 0.209419968534551*G0_2_2;
    A[2363] = -0.0212681961813905*G0_0_2 - 0.0118736555368507*G0_1_0 - 0.0477555816097483*G0_1_2 - 0.0212681961813908*G0_2_0 - 0.111886369481855*G0_2_2;
    A[2364] = -0.128522535206561*G0_0_0 - 0.0962940416065406*G0_0_1 - 0.0962940416065398*G0_1_0 - 0.12473862300251*G0_1_1;
    A[2365] = 0.0476251018785729*G0_0_0 + 0.0623693115012553*G0_0_1 + 0.0623693115012548*G0_1_0 + 0.124738623002511*G0_1_1;
    A[2366] = -0.0150051690850298*G0_0_0 - 0.0377738821749238*G0_0_1 - 0.037773882174923*G0_1_0 - 0.127087258163646*G0_1_1;
    A[2367] = 0.00939454064454043*G0_0_1 + 0.00939454064454033*G0_1_0 + 0.0641307878721069*G0_1_1;
    A[2368] = -0.0613254736518621*G0_0_0 - 0.0613254736518619*G0_0_1 - 0.0400572774704708*G0_0_2 - 0.0187890812890808*G0_1_0 - 0.0187890812890808*G0_1_1 - 0.00939454064454056*G0_1_2 - 0.0400572774704718*G0_2_0 - 0.0400572774704717*G0_2_1 - 0.0187890812890806*G0_2_2;
    A[2369] = -0.0254435475789637*G0_0_0 - 0.025443547578964*G0_0_1 + 0.0136351319077004*G0_0_2 + 0.0148746893538563*G0_1_0 + 0.0148746893538563*G0_1_1 - 6.52398655870331e-05*G0_1_2 + 0.0136351319077013*G0_2_0 + 0.0136351319077012*G0_2_1 - 0.000130479731174542*G0_2_2;
    A[2370] = -0.0528442911255407*G0_0_0 - 0.0528442911255406*G0_0_1 - 0.0137656116388747*G0_0_2 + 0.00117431758056729*G0_1_0 + 0.00117431758056714*G0_1_1 - 6.52398655871634e-05*G0_1_2 - 0.0137656116388753*G0_2_0 - 0.0137656116388753*G0_2_1 - 0.000130479731173769*G0_2_2;
    A[2371] = 0.0212681961813903*G0_0_2 + 0.0118736555368506*G0_1_0 + 0.0118736555368506*G0_1_1 - 0.00939454064454015*G0_1_2 + 0.0212681961813907*G0_2_0 + 0.0212681961813907*G0_2_1 - 0.0187890812890812*G0_2_2;
    A[2372] = -0.0306627368259313*G0_0_0 - 0.00939454064454008*G0_0_1 - 0.0306627368259313*G0_0_2 - 0.00939454064454086*G0_1_0 - 0.0093945406445408*G0_1_2 - 0.0306627368259314*G0_2_0 - 0.00939454064454025*G0_2_1 - 0.0306627368259314*G0_2_2;
    A[2373] = -0.00130479731174163*G0_0_0 - 6.52398655871973e-05*G0_0_1 - 0.00130479731174158*G0_0_2 - 6.52398655869314e-05*G0_1_0 - 6.52398655870906e-05*G0_1_2 - 0.00130479731174157*G0_2_0 - 6.52398655874395e-05*G0_2_1 - 0.00130479731174162*G0_2_2;
    A[2374] = -0.0150051690850295*G0_0_0 - 6.52398655869952e-05*G0_0_1 - 0.0150051690850296*G0_0_2 - 6.52398655875744e-05*G0_1_0 - 6.52398655873355e-05*G0_1_2 - 0.01500516908503*G0_2_0 - 6.52398655863811e-05*G0_2_1 - 0.01500516908503*G0_2_2;
    A[2375] = -0.00939454064454079*G0_0_1 - 0.00939454064454028*G0_1_0 - 0.00939454064454032*G0_1_2 - 0.00939454064454116*G0_2_1;
    A[2376] = -0.0693499771190732*G0_0_0 - 0.0906181733004644*G0_0_1 - 0.0906181733004643*G0_0_2 - 0.0547362472275661*G0_1_0 - 0.0641307878721065*G0_1_1 - 0.0641307878721064*G0_1_2 - 0.0906181733004633*G0_2_0 - 0.111886369481855*G0_2_1 - 0.111886369481854*G0_2_2;
    A[2377] = 0.0846813455320387*G0_0_0 + 0.173472802596065*G0_0_1 + 0.173472802596065*G0_0_2 + 0.089313375988722*G0_1_0 + 0.127087258163645*G0_1_1 + 0.127087258163645*G0_1_2 + 0.173472802596064*G0_2_0 + 0.20941996853455*G0_2_1 + 0.20941996853455*G0_2_2;
    A[2378] = -0.00339247301052818*G0_0_0 - 0.198068231922397*G0_0_1 - 0.198068231922397*G0_0_2 - 0.0623693115012551*G0_1_0 - 0.12473862300251*G0_1_1 - 0.12473862300251*G0_1_2 - 0.198068231922396*G0_2_0 - 0.271397840842283*G0_2_1 - 0.271397840842283*G0_2_2;
    A[2379] = -0.0644569872000426*G0_0_0 + 0.0962940416065403*G0_0_1 + 0.0962940416065403*G0_0_2 + 0.0284445813959697*G0_1_0 + 0.124738623002509*G0_1_1 + 0.12473862300251*G0_1_2 + 0.09629404160654*G0_2_0;
    A[2380] = 0.0521918924696703*G0_0_0 - 0.271397840842282*G0_0_1 - 0.135698920421141*G0_0_2 - 0.271397840842282*G0_1_0 - 0.542795681684564*G0_1_1 - 0.27139784084228*G0_1_2 - 0.135698920421141*G0_2_0 - 0.271397840842284*G0_2_1;
    A[2381] = -0.02283395295548*G0_0_0 + 0.0737210481134084*G0_0_1 + 0.0368605240567037*G0_0_2 + 0.0737210481134099*G0_1_0 + 0.209419968534552*G0_1_1 + 0.104709984267275*G0_1_2 + 0.0368605240567055*G0_2_0 + 0.104709984267277*G0_2_1;
    A[2382] = -0.0300103381700598*G0_0_1 - 0.0150051690850303*G0_0_2 - 0.0300103381700608*G0_1_0 - 0.105688582251082*G0_1_1 - 0.0528442911255406*G0_1_2 - 0.0150051690850304*G0_2_0 - 0.0528442911255406*G0_2_1;
    A[2383] = -0.0880738185425698*G0_0_0 + 0.0459941052388951*G0_0_1 + 0.0091335811821915*G0_0_2 + 0.0459941052388964*G0_1_0 + 0.0234863516113503*G0_1_1 - 0.0812236326559236*G0_1_2 + 0.00913358118219177*G0_2_0 - 0.0812236326559233*G0_2_1 - 0.185933616923199*G0_2_2;
    A[2384] = -0.0127217737894822*G0_0_1 - 0.0120693751336098*G0_0_2 - 0.0127217737894812*G0_1_0 + 0.0137003717732891*G0_1_1 + 0.019898159004063*G0_1_2 - 0.0120693751336109*G0_2_0 + 0.0198981590040614*G0_2_1 + 0.0260959462348362*G0_2_2;
    A[2385] = -0.052844291125541*G0_0_1 - 0.037839122040512*G0_0_2 - 0.052844291125541*G0_1_0 - 0.0534966897814115*G0_1_1 - 0.000652398655871373*G0_1_2 - 0.0378391220405108*G0_2_0 - 0.000652398655871233*G0_2_1 + 0.0521918924696692*G0_2_2;
    A[2386] = -0.0456679059109613*G0_0_0 + 0.0300103381700596*G0_0_1 - 0.00782878387045012*G0_0_2 + 0.0300103381700604*G0_1_0 + 0.0150051690850308*G0_1_2 - 0.00782878387044992*G0_2_0 + 0.0150051690850312*G0_2_1 + 0.0300103381700612*G0_2_2;
    A[2387] = -0.0228339529554807*G0_0_0 + 0.00130479731174216*G0_0_1 - 0.01076457782187*G0_0_2 + 0.0013047973117399*G0_1_0 + 0.000652398655870247*G0_1_2 - 0.0107645778218708*G0_2_0 + 0.000652398655868843*G0_2_1 + 0.00130479731174057*G0_2_2;
    A[2388] = 0.0300103381700609*G0_0_1 + 0.0150051690850295*G0_0_2 + 0.0300103381700607*G0_1_0 + 0.0150051690850307*G0_1_2 + 0.0150051690850305*G0_2_0 + 0.0150051690850317*G0_2_1 + 0.03001033817006*G0_2_2;
    A[2389] = -0.0880738185425669*G0_0_0 + 0.0127217737894823*G0_0_1 + 0.000652398655870998*G0_0_2 + 0.0127217737894824*G0_1_0 + 0.000652398655870314*G0_1_2 + 0.000652398655872312*G0_2_0 + 0.000652398655871526*G0_2_1 + 0.00130479731174188*G0_2_2;
    A[2390] = 0.012721773789481*G0_0_1 + 0.0120693751336119*G0_0_2 + 0.0127217737894813*G0_1_0 + 0.000652398655871539*G0_1_2 + 0.0120693751336108*G0_2_0 + 0.000652398655870152*G0_2_1 + 0.00130479731174308*G0_2_2;
    A[2391] = 0.052844291125541*G0_0_1 + 0.0378391220405098*G0_0_2 + 0.0528442911255412*G0_1_0 + 0.0150051690850298*G0_1_2 + 0.037839122040511*G0_2_0 + 0.0150051690850298*G0_2_1 + 0.0300103381700594*G0_2_2;
    A[2392] = 0.255740273101383*G0_0_0 + 0.180062029020361*G0_0_1 + 0.127870136550692*G0_0_2 + 0.0756782440810208*G0_1_0 + 0.105688582251082*G0_1_1 + 0.0528442911255413*G0_1_2 + 0.127870136550691*G0_2_0 + 0.10503618359521*G0_2_1 + 0.105688582251082*G0_2_2;
    A[2393] = -0.584549195660302*G0_0_0 - 0.390134396210782*G0_0_1 - 0.262916658315962*G0_0_2 - 0.135698920421142*G0_1_0 - 0.209419968534551*G0_1_1 - 0.104709984267275*G0_1_2 - 0.262916658315962*G0_2_0 - 0.290643601190473*G0_2_1 - 0.209419968534549*G0_2_2;
    A[2394] = 0.647179466623906*G0_0_0 + 0.271397840842283*G0_0_1 + 0.271397840842284*G0_0_2 + 0.271397840842283*G0_1_0 + 0.542795681684565*G0_1_1 + 0.271397840842284*G0_1_2 + 0.271397840842283*G0_2_0 + 0.271397840842284*G0_2_1 + 0.542795681684567*G0_2_2;
    A[2395] = 0.17223324514991*G0_0_0 + 0.0714376528178615*G0_0_1 + 0.00619778723077301*G0_0_2 - 0.0264221455627695*G0_1_0 - 0.0137003717732868*G0_1_1 + 0.00619778723077343*G0_1_2 + 0.00619778723077393*G0_2_0 + 0.00619778723077477*G0_2_1 + 0.0123955744615464*G0_2_2;
    A[2396] = -0.26813584756293*G0_0_0 - 0.0459941052388955*G0_0_1 + 0.0534966897814119*G0_0_2 + 0.0225077536275446*G0_1_0 - 0.0234863516113509*G0_1_1 - 0.104709984267275*G0_1_2 + 0.0534966897814105*G0_2_0 + 0.0812236326559232*G0_2_1 - 0.209419968534551*G0_2_2;
    A[2397] = 0.105688582251082*G0_0_0 + 0.0528442911255411*G0_0_1 - 0.0221815542996101*G0_0_2 + 0.00065239865587058*G0_1_0 + 0.0534966897814117*G0_1_1 + 0.0528442911255401*G0_1_2 - 0.0221815542996097*G0_2_0 + 0.000652398655871296*G0_2_1 + 0.105688582251082*G0_2_2;
    A[2398] = 0.127870136550691*G0_0_0 + 0.0378391220405109*G0_0_1 + 0.0900310145101805*G0_0_2 + 0.0378391220405103*G0_1_0 + 0.0528442911255399*G0_1_2 + 0.0900310145101809*G0_2_0 + 0.0528442911255415*G0_2_1 + 0.05219189246967*G0_2_2;
    A[2399] = -0.292274597830151*G0_0_0 - 0.0678494602105721*G0_0_1 - 0.195067198105392*G0_0_2 - 0.0678494602105706*G0_1_0 - 0.104709984267276*G0_1_2 - 0.195067198105391*G0_2_0 - 0.104709984267275*G0_2_1 - 0.185933616923198*G0_2_2;
    A[2400] = 0.323589733311953*G0_0_0 + 0.135698920421142*G0_0_1 + 0.135698920421142*G0_0_2 + 0.135698920421141*G0_1_0 + 0.271397840842281*G0_1_2 + 0.135698920421142*G0_2_0 + 0.271397840842283*G0_2_1;
    A[2401] = 0.0502346965020569*G0_0_0 + 0.00554538857490281*G0_0_1 + 0.038165321368446*G0_0_2 + 0.00554538857490241*G0_1_0 + 0.00619778723077237*G0_1_2 + 0.0381653213684456*G0_2_0 + 0.00619778723077203*G0_2_1 + 0.026095946234834*G0_2_2;
    A[2402] = -0.0965550010688881*G0_0_0 - 0.0678494602105703*G0_0_1 - 0.0368605240567038*G0_0_2 - 0.0678494602105703*G0_1_0 - 0.104709984267275*G0_1_2 - 0.0368605240567045*G0_2_0 - 0.104709984267275*G0_2_1;
    A[2403] = 0.0300103381700595*G0_0_0 + 0.0378391220405107*G0_0_1 + 0.0150051690850304*G0_0_2 + 0.0378391220405105*G0_1_0 + 0.0528442911255406*G0_1_2 + 0.0150051690850304*G0_2_0 + 0.0528442911255418*G0_2_1;
    A[2404] = 0.143527704291593*G0_0_0 - 0.0750258454251506*G0_0_1 - 0.0424059126316054*G0_0_2 - 0.0750258454251527*G0_1_0 - 0.0260959462348356*G0_1_2 - 0.0424059126316068*G0_2_0 - 0.0260959462348354*G0_2_1 - 0.0521918924696692*G0_2_2;
    A[2405] = -0.150051690850302*G0_0_0 + 0.316413348097373*G0_0_1 + 0.185933616923199*G0_0_2 + 0.316413348097374*G0_1_0 + 0.185933616923198*G0_1_2 + 0.185933616923198*G0_2_0 + 0.185933616923196*G0_2_1 + 0.371867233846397*G0_2_2;
    A[2406] = 0.045667905910961*G0_0_0 - 0.0750258454251507*G0_0_1 - 0.026095946234836*G0_0_2 - 0.0750258454251496*G0_1_0 - 0.0260959462348352*G0_1_2 - 0.0260959462348344*G0_2_0 - 0.0260959462348344*G0_2_1 - 0.0521918924696698*G0_2_2;
    A[2407] = 0.176147637085137*G0_0_0 - 0.0587158790283794*G0_0_1 - 0.00978597983806268*G0_0_2 - 0.0587158790283792*G0_1_0 - 0.0260959462348329*G0_1_2 - 0.00978597983806447*G0_2_0 - 0.0260959462348363*G0_2_1 - 0.0521918924696696*G0_2_2;
    A[2408] = -0.0298472385060926*G0_0_0 - 0.0298472385060925*G0_0_1 - 0.0298472385060926*G0_0_2 - 0.0345719095719093*G0_1_0 - 0.0345719095719093*G0_1_1 - 0.0345719095719093*G0_1_2 - 0.0298472385060924*G0_2_0 - 0.0298472385060925*G0_2_1 - 0.0298472385060926*G0_2_2;
    A[2409] = 0.011743175805676*G0_0_0 + 0.00227817610630144*G0_1_0;
    A[2410] = -0.00946499969937459*G0_1_1;
    A[2411] = 0.00472467106581737*G0_1_2 - 0.029847238506092*G0_2_2;
    A[2412] = 0.029260079715809*G0_1_1 + 0.0032946132121484*G0_1_2 + 0.00329461321214747*G0_2_1;
    A[2413] = -0.0189848008858419*G0_1_1 - 0.00949240044292167*G0_1_2 - 0.00949240044291928*G0_2_1 - 0.000652398655870409*G0_2_2;
    A[2414] = 0.0035229527417019*G0_1_1 + 0.00649136662591566*G0_1_2 + 0.0064913666259134*G0_2_1 + 0.00880738185425708*G0_2_2;
    A[2415] = 0.0429604514890982*G0_1_1 + 0.0422754329004328*G0_1_2 + 0.0422754329004343*G0_2_1 + 0.0642612676032813*G0_2_2;
    A[2416] = -0.0435476102793809*G0_0_0 - 0.00583896797004381*G0_0_2 - 0.0142875305635728*G0_1_0 - 0.00254435475789614*G0_1_2 - 0.00583896797004447*G0_2_0;
    A[2417] = 0.0567586830607652*G0_0_0 + 0.0188869410874606*G0_0_2 + 0.0377738821749233*G0_1_0 + 0.00939454064453938*G0_1_2 + 0.0188869410874615*G0_2_0 - 0.00619778723077344*G0_2_2;
    A[2418] = 0.104709984267274*G0_0_0 + 0.0743408268364853*G0_0_2 + 0.108232937008977*G0_1_0 + 0.0808321934624008*G0_1_2 + 0.0743408268364848*G0_2_0 + 0.0371867233846387*G0_2_2;
    A[2419] = 0.0422754329004331*G0_0_2 + 0.0429604514890981*G0_1_0 + 0.0845508658008655*G0_1_2 + 0.0422754329004342*G0_2_0 + 0.128522535206563*G0_2_2;
    A[2420] = -0.0091335811821923*G0_0_0 - 0.00583896797004379*G0_0_1 - 0.00583896797004482*G0_1_0 - 0.00254435475789615*G0_1_1;
    A[2421] = -0.0156575677409007*G0_0_0 - 0.0130805930502111*G0_0_1 - 0.0130805930502105*G0_1_0 - 0.0105036183595214*G0_1_1;
    A[2422] = -0.000652398655871334*G0_0_0 + 0.00192457603481868*G0_0_1 + 0.00192457603481818*G0_1_0 + 0.00450155072550843*G0_1_1;
    A[2423] = 0.00329461321214779*G0_0_1 + 0.00329461321214832*G0_1_0 + 0.00658922642429624*G0_1_1;
    A[2424] = 0.176636936077039*G0_0_0 + 0.176636936077039*G0_0_1 + 0.134361503176607*G0_0_2 + 0.1273155976932*G0_1_0 + 0.127315597693201*G0_1_1 + 0.128000616281866*G0_1_2 + 0.134361503176606*G0_2_0 + 0.134361503176606*G0_2_1 + 0.0920860702761738*G0_2_2;
    A[2425] = -0.334680510461758*G0_0_0 - 0.334680510461758*G0_0_1 - 0.250814663249558*G0_0_2 - 0.166948816037357*G0_1_0 - 0.166948816037357*G0_1_1 - 0.160783648739378*G0_1_2 - 0.250814663249558*G0_2_0 - 0.250814663249558*G0_2_1 - 0.0622388317700821*G0_2_2;
    A[2426] = 0.104709984267276*G0_0_0 + 0.104709984267276*G0_0_1 + 0.188575831479477*G0_0_2 + 0.182410664181496*G0_1_0 + 0.182410664181496*G0_1_1 + 0.0985448169692962*G0_1_2 + 0.188575831479477*G0_2_0 + 0.188575831479476*G0_2_1 - 0.0622388317700803*G0_2_2;
    A[2427] = -0.0422754329004335*G0_0_2 - 0.0429604514890982*G0_1_0 - 0.0429604514890981*G0_1_1 - 0.0359145460056925*G0_1_2 - 0.0422754329004342*G0_2_0 - 0.0422754329004343*G0_2_1 + 0.0920860702761727*G0_2_2;
    A[2428] = 0.0642612676032808*G0_0_0 + 0.0219858347028487*G0_0_1 + 0.0642612676032809*G0_0_2 + 0.0219858347028483*G0_1_0 + 0.0226708532915131*G0_1_1 + 0.0219858347028484*G0_1_2 + 0.0642612676032808*G0_2_0 + 0.0219858347028484*G0_2_1 + 0.0642612676032808*G0_2_2;
    A[2429] = 0.00880738185425666*G0_0_0 + 0.00231601522834132*G0_0_1 + 0.00880738185425673*G0_0_2 + 0.002316015228341*G0_1_0 - 0.0006523986558714*G0_1_1 + 0.00231601522834104*G0_1_2 + 0.00880738185425675*G0_2_0 + 0.00231601522834145*G0_2_1 + 0.00880738185425683*G0_2_2;
    A[2430] = -0.000652398655871187*G0_0_0 + 0.0088400017870505*G0_0_1 - 0.00065239865587132*G0_0_2 + 0.00884000178705037*G0_1_0 - 0.00065239865587051*G0_1_1 + 0.00884000178705031*G0_1_2 - 0.000652398655871387*G0_2_0 + 0.00884000178705052*G0_2_1 - 0.00065239865587149*G0_2_2;
    A[2431] = -0.00329461321214808*G0_0_1 - 0.00329461321214839*G0_1_0 + 0.0226708532915127*G0_1_1 - 0.00329461321214837*G0_1_2 - 0.003294613212148*G0_2_1;
    A[2432] = 0.0439716694056968*G0_0_0 + 0.0862471023061291*G0_0_1 + 0.0862471023061292*G0_0_2 + 0.044656687994361*G0_1_0 + 0.0439716694056963*G0_1_1 + 0.0439716694056964*G0_1_2 + 0.0862471023061293*G0_2_0 + 0.128522535206562*G0_2_1 + 0.128522535206562*G0_2_2;
    A[2433] = -0.00678494602105722*G0_0_0 - 0.0371541034518463*G0_0_1 - 0.0371541034518463*G0_0_2 - 0.00975335990526936*G0_1_0 - 0.043645470077761*G0_1_1 - 0.0436454700777611*G0_1_2 - 0.0371541034518464*G0_2_0 + 0.03718672338464*G0_2_1 + 0.03718672338464*G0_2_2;
    A[2434] = 0.0127870136550696*G0_0_0 - 0.0250847283182347*G0_0_1 - 0.0250847283182347*G0_0_2 + 0.00329461321214797*G0_1_0 - 0.015592327875314*G0_1_1 - 0.0155923278753141*G0_1_2 - 0.0250847283182348*G0_2_0 - 0.00619778723077325*G0_2_1 - 0.00619778723077339*G0_2_2;
    A[2435] = -0.0318696743392926*G0_0_0 + 0.00583896797004386*G0_0_1 + 0.00583896797004389*G0_0_2 - 0.00590420783563172*G0_1_0 + 0.00254435475789609*G0_1_1 + 0.00254435475789614*G0_1_2 + 0.00583896797004428*G0_2_0;
    A[2436] = 0.039143919352252*G0_0_0 + 0.0264221455627692*G0_0_1 - 0.00554538857490251*G0_0_2 + 0.0264221455627699*G0_1_0 + 0.0137003717732867*G0_1_1 - 0.00619778723077406*G0_1_2 - 0.00554538857490289*G0_2_0 - 0.00619778723077383*G0_2_1;
    A[2437] = 0.019571959676126*G0_0_0 + 0.00146789697571023*G0_0_1 - 0.0102752788299667*G0_0_2 + 0.0014678969757081*G0_1_0 - 0.0166361657247058*G0_1_1 - 0.0181040627004176*G0_1_2 - 0.0102752788299668*G0_2_0 - 0.0181040627004163*G0_2_1;
    A[2438] = -0.0127217737894813*G0_0_1 - 0.000652398655870838*G0_0_2 - 0.0127217737894818*G0_1_0 - 0.0254435475789664*G0_1_1 - 0.00130479731174224*G0_1_2 - 0.000652398655869783*G0_2_0 - 0.00130479731174132*G0_2_1;
    A[2439] = 0.092966808461598*G0_0_0 + 0.12134614999198*G0_0_1 + 0.0606730749959896*G0_0_2 + 0.121346149991982*G0_1_0 + 0.149725491522363*G0_1_1 + 0.0748627457611824*G0_1_2 + 0.0606730749959909*G0_2_0 + 0.0748627457611805*G0_2_1 + 0.0195719596761254*G0_2_2;
    A[2440] = 0.0283793415303829*G0_0_1 + 0.0141896707651926*G0_0_2 + 0.0283793415303824*G0_1_0 + 0.0567586830607666*G0_1_1 + 0.028379341530382*G0_1_2 + 0.0141896707651925*G0_2_0 + 0.0283793415303833*G0_2_1 - 0.00489298991903029*G0_2_2;
    A[2441] = 0.104709984267276*G0_0_1 + 0.0678494602105709*G0_0_2 + 0.104709984267277*G0_1_0 + 0.209419968534552*G0_1_1 + 0.135698920421143*G0_1_2 + 0.0678494602105719*G0_2_0 + 0.135698920421144*G0_2_1 + 0.0391439193522529*G0_2_2;
    A[2442] = -0.156575677409009*G0_0_0 - 0.134067923781465*G0_0_1 - 0.22442513761958*G0_0_2 - 0.134067923781465*G0_1_0 - 0.0880738185425684*G0_1_1 - 0.0972073997247602*G0_1_2 - 0.22442513761958*G0_2_0 - 0.09720739972476*G0_2_1 - 0.292274597830151*G0_2_2;
    A[2443] = 0.0195719596761268*G0_0_0 - 0.0552907860850567*G0_0_1 + 0.00538228891093569*G0_0_2 - 0.0552907860850554*G0_1_0 + 0.0195719596761273*G0_1_1 - 0.0411011153198631*G0_1_2 + 0.00538228891093557*G0_2_0 - 0.0411011153198649*G0_2_1 - 0.00880738185425596*G0_2_2;
    A[2444] = 0.0127217737894826*G0_0_1 + 0.000652398655870649*G0_0_2 + 0.0127217737894818*G0_1_0 - 0.088073818542569*G0_1_1 + 0.000652398655870983*G0_1_2 + 0.000652398655869837*G0_2_0 + 0.00065239865587089*G0_2_1 + 0.0013047973117417*G0_2_2;
    A[2445] = 0.0929668084616*G0_0_0 + 0.363059851992141*G0_0_1 + 0.228013330226869*G0_0_2 + 0.36305985199214*G0_1_0 + 0.0929668084615992*G0_1_1 + 0.228013330226869*G0_1_2 + 0.228013330226869*G0_2_0 + 0.22801333022687*G0_2_1 + 0.45602666045374*G0_2_2;
    A[2446] = -0.0283793415303843*G0_0_1 - 0.0141896707651921*G0_0_2 - 0.0283793415303825*G0_1_0 + 0.0929668084615997*G0_1_1 + 0.0322937334656068*G0_1_2 - 0.0141896707651925*G0_2_0 + 0.0322937334656075*G0_2_1 - 0.00880738185425785*G0_2_2;
    A[2447] = -0.104709984267275*G0_0_1 - 0.0678494602105706*G0_0_2 - 0.104709984267277*G0_1_0 - 0.185933616923198*G0_1_1 - 0.19506719810539*G0_1_2 - 0.0678494602105719*G0_2_0 - 0.19506719810539*G0_2_1 - 0.29227459783015*G0_2_2;
    A[2448] = -0.268135847562928*G0_0_0 - 0.290643601190472*G0_0_1 - 0.321632537344339*G0_0_2 - 0.222141742324031*G0_1_0 - 0.268135847562927*G0_1_1 - 0.194414799449519*G0_1_2 - 0.321632537344339*G0_2_0 - 0.44885027523916*G0_2_1 - 0.584549195660302*G0_2_2;
    A[2449] = -0.0371867233846407*G0_0_0 + 0.140428810676203*G0_0_1 + 0.186912214907004*G0_0_2 + 0.0376760223765414*G0_1_0 + 0.0655660649150206*G0_1_1 + 0.140428810676204*G0_1_2 + 0.186912214907004*G0_2_0 + 0.0376760223765421*G0_2_1 - 0.0371867233846403*G0_2_2;
    A[2450] = 0.17223324514991*G0_0_0 - 0.0264221455627695*G0_0_1 + 0.00619778723077393*G0_0_2 + 0.0714376528178615*G0_1_0 - 0.0137003717732868*G0_1_1 + 0.00619778723077477*G0_1_2 + 0.00619778723077303*G0_2_0 + 0.00619778723077344*G0_2_1 + 0.0123955744615464*G0_2_2;
    A[2451] = 0.726119703984279*G0_0_0 + 0.456026660453739*G0_0_1 + 0.456026660453739*G0_0_2 + 0.456026660453739*G0_1_0 + 0.726119703984279*G0_1_1 + 0.456026660453739*G0_1_2 + 0.456026660453739*G0_2_0 + 0.456026660453739*G0_2_1 + 0.912053320907481*G0_2_2;
    A[2452] = -0.448197876583287*G0_0_0 - 0.121346149991979*G0_0_1 - 0.224098938291643*G0_0_2 - 0.326851726591305*G0_1_0 - 0.149725491522363*G0_1_1 - 0.177615534060844*G0_1_2 - 0.224098938291642*G0_2_0 - 0.0748627457611802*G0_2_1 - 0.0371867233846378*G0_2_2;
    A[2453] = -0.20941996853455*G0_0_0 - 0.104709984267276*G0_0_1 - 0.262916658315962*G0_0_2 - 0.290643601190473*G0_1_0 - 0.209419968534552*G0_1_1 - 0.390134396210781*G0_1_2 - 0.262916658315962*G0_2_0 - 0.135698920421144*G0_2_1 - 0.584549195660303*G0_2_2;
    A[2454] = -0.0965550010688889*G0_0_0 - 0.0596944770121848*G0_0_1 - 0.0287055408583181*G0_0_2 - 0.0596944770121845*G0_1_0 - 0.0228339529554807*G0_1_1 - 0.0965550010688892*G0_1_2 - 0.0287055408583177*G0_2_0 - 0.0965550010688891*G0_2_1 + 0.0391439193522528*G0_2_2;
    A[2455] = -0.00880738185425664*G0_0_0 + 0.00538228891093508*G0_0_1 - 0.0411011153198651*G0_0_2 + 0.00538228891093441*G0_1_0 + 0.0195719596761263*G0_1_1 - 0.055290786085057*G0_1_2 - 0.0411011153198651*G0_2_0 - 0.055290786085056*G0_2_1 + 0.0195719596761267*G0_2_2;
    A[2456] = 0.050234696502057*G0_0_0 + 0.0381653213684459*G0_0_1 + 0.00554538857490228*G0_0_2 + 0.0381653213684466*G0_1_0 + 0.0260959462348354*G0_1_1 + 0.00619778723077373*G0_1_2 + 0.00554538857490237*G0_2_0 + 0.00619778723077303*G0_2_1;
    A[2457] = -0.0332723314494141*G0_0_0 - 0.0190826606842234*G0_0_1 - 0.0190826606842231*G0_0_2 - 0.0190826606842221*G0_1_0 - 0.00489298991903076*G0_1_1 - 0.0332723314494143*G0_1_2 - 0.019082660684223*G0_2_0 - 0.0332723314494148*G0_2_1 - 0.00489298991903192*G0_2_2;
    A[2458] = 0.0401225173360582*G0_0_0 + 0.0298472385060922*G0_0_1 + 0.0102752788299668*G0_0_2 + 0.0298472385060916*G0_1_0 + 0.0195719596761264*G0_1_1 + 0.0181040627004179*G0_1_2 + 0.0102752788299658*G0_2_0 + 0.0181040627004165*G0_2_1;
    A[2459] = 0.00130479731174259*G0_0_0 - 0.0107645778218685*G0_0_1 + 0.000652398655870919*G0_0_2 - 0.0107645778218686*G0_1_0 - 0.0228339529554811*G0_1_1 + 0.00130479731174219*G0_1_2 + 0.000652398655872044*G0_2_0 + 0.0013047973117423*G0_2_1;
    A[2460] = 0.352295274170271*G0_0_0 + 0.24954248587061*G0_0_1 + 0.249542485870609*G0_0_2 + 0.249542485870608*G0_1_0 + 0.146789697570946*G0_1_1 + 0.44526208263187*G0_1_2 + 0.249542485870607*G0_2_0 + 0.44526208263187*G0_2_1 - 0.0391439193522548*G0_2_2;
    A[2461] = -0.234863516113514*G0_0_0 - 0.141896707651913*G0_0_1 - 0.0195719596761241*G0_0_2 - 0.141896707651913*G0_1_0 - 0.0489298991903147*G0_1_1 - 0.0195719596761248*G0_1_2 - 0.0195719596761237*G0_2_0 - 0.0195719596761248*G0_2_1 - 0.0391439193522512*G0_2_2;
    A[2462] = -0.0391439193522528*G0_0_0 + 0.0538228891093434*G0_0_1 + 0.00489298991903028*G0_0_2 + 0.0538228891093472*G0_1_0 + 0.146789697570948*G0_1_1 + 0.00489298991903198*G0_1_2 + 0.00489298991903137*G0_2_0 + 0.00489298991903138*G0_2_1 + 0.00978597983806218*G0_2_2;
    A[2463] = -0.185933616923198*G0_0_0 - 0.484406001984122*G0_0_1 - 0.288686405222863*G0_0_2 - 0.484406001984123*G0_1_0 - 0.782878387045047*G0_1_1 - 0.484406001984125*G0_1_2 - 0.288686405222861*G0_2_0 - 0.484406001984123*G0_2_1 - 0.0391439193522521*G0_2_2;
    A[2464] = 0.0117431758056757*G0_0_0 + 0.0117431758056757*G0_0_1 + 0.0117431758056757*G0_0_2 + 0.00946499969937447*G0_1_0 + 0.0094649996993745*G0_1_1 + 0.00946499969937446*G0_1_2 + 0.0117431758056757*G0_2_0 + 0.0117431758056757*G0_2_1 + 0.0117431758056756*G0_2_2;
    A[2465] = -0.0298472385060927*G0_0_0 + 0.00472467106581712*G0_1_0;
    A[2466] = -0.00946499969937429*G0_1_1;
    A[2467] = 0.00472467106581677*G0_1_2 - 0.0298472385060925*G0_2_2;
    A[2468] = 0.0292600797158079*G0_1_1 + 0.00329461321214829*G0_1_2 + 0.00329461321214761*G0_2_1;
    A[2469] = -0.0189848008858415*G0_1_1 - 0.00949240044292178*G0_1_2 - 0.00949240044292091*G0_2_1 - 0.000652398655871724*G0_2_2;
    A[2470] = 0.00352295274170277*G0_1_1 + 0.00649136662591604*G0_1_2 + 0.00649136662591492*G0_2_1 + 0.00880738185425745*G0_2_2;
    A[2471] = 0.042960451489097*G0_1_1 + 0.0422754329004326*G0_1_2 + 0.0422754329004329*G0_2_1 + 0.0642612676032811*G0_2_2;
    A[2472] = 0.176636936077039*G0_0_0 + 0.0422754329004321*G0_0_2 + 0.0493213383838371*G0_1_0 + 0.0429604514890969*G0_1_2 + 0.0422754329004321*G0_2_0;
    A[2473] = -0.334680510461758*G0_0_0 - 0.0838658472122*G0_0_2 - 0.167731694424401*G0_1_0 - 0.077700679914221*G0_1_2 - 0.0838658472121996*G0_2_0 + 0.104709984267277*G0_2_2;
    A[2474] = 0.104709984267277*G0_0_0 - 0.0838658472121996*G0_0_2 - 0.0777006799142214*G0_1_0 - 0.167731694424402*G0_1_2 - 0.0838658472122014*G0_2_0 - 0.334680510461759*G0_2_2;
    A[2475] = 0.0422754329004324*G0_0_2 + 0.042960451489097*G0_1_0 + 0.0493213383838383*G0_1_2 + 0.0422754329004329*G0_2_0 + 0.176636936077039*G0_2_2;
    A[2476] = 0.0642612676032809*G0_0_0 + 0.0422754329004323*G0_0_1 + 0.0422754329004321*G0_1_0 + 0.042960451489097*G0_1_1;
    A[2477] = 0.00880738185425703*G0_0_0 + 0.00649136662591528*G0_0_1 + 0.00649136662591593*G0_1_0 + 0.00352295274170293*G0_1_1;
    A[2478] = -0.000652398655870629*G0_0_0 - 0.00949240044292058*G0_0_1 - 0.00949240044292172*G0_1_0 - 0.0189848008858421*G0_1_1;
    A[2479] = 0.00329461321214781*G0_0_1 + 0.00329461321214817*G0_1_0 + 0.0292600797158081*G0_1_1;
    A[2480] = -0.0435476102793801*G0_0_0 - 0.0435476102793804*G0_0_1 - 0.0377086423093366*G0_0_2 - 0.0292600797158079*G0_1_0 - 0.0292600797158079*G0_1_1 - 0.0259654665036604*G0_1_2 - 0.0377086423093359*G0_2_0 - 0.037708642309336*G0_2_1 - 0.0318696743392917*G0_2_2;
    A[2481] = 0.0567586830607648*G0_0_0 + 0.0567586830607651*G0_0_1 + 0.0378717419733046*G0_0_2 + 0.0189848008858414*G0_1_0 + 0.0189848008858413*G0_1_1 + 0.00949240044292058*G0_1_2 + 0.0378717419733035*G0_2_0 + 0.0378717419733035*G0_2_1 + 0.0127870136550684*G0_2_2;
    A[2482] = 0.104709984267275*G0_0_0 + 0.104709984267275*G0_0_1 + 0.0303691574307886*G0_0_2 - 0.00352295274170245*G0_1_0 - 0.00352295274170251*G0_1_1 + 0.0029684138842129*G0_1_2 + 0.0303691574307893*G0_2_0 + 0.0303691574307894*G0_2_1 - 0.00678494602105659*G0_2_2;
    A[2483] = -0.0422754329004319*G0_0_2 - 0.0429604514890969*G0_1_0 - 0.042960451489097*G0_1_1 - 0.000685018588664247*G0_1_2 - 0.0422754329004328*G0_2_0 - 0.0422754329004329*G0_2_1 + 0.0439716694056969*G0_2_2;
    A[2484] = -0.00913358118219183*G0_0_0 - 0.00329461321214826*G0_0_1 - 0.00913358118219198*G0_0_2 - 0.00329461321214749*G0_1_0 - 0.0032946132121476*G0_1_2 - 0.00913358118219189*G0_2_0 - 0.00329461321214785*G0_2_1 - 0.00913358118219192*G0_2_2;
    A[2485] = -0.0156575677409007*G0_0_0 - 0.00257697469068982*G0_0_1 - 0.0156575677409009*G0_0_2 - 0.0025769746906901*G0_1_0 - 0.00257697469069013*G0_1_2 - 0.015657567740901*G0_2_0 - 0.00257697469069006*G0_2_1 - 0.015657567740901*G0_2_2;
    A[2486] = -0.000652398655870884*G0_0_0 - 0.00257697469068973*G0_0_1 - 0.000652398655870706*G0_0_2 - 0.00257697469068983*G0_1_0 - 0.00257697469068985*G0_1_2 - 0.000652398655870234*G0_2_0 - 0.00257697469068972*G0_2_1 - 0.000652398655870252*G0_2_2;
    A[2487] = -0.00329461321214797*G0_0_1 - 0.00329461321214819*G0_1_0 - 0.00329461321214821*G0_1_2 - 0.00329461321214815*G0_2_1;
    A[2488] = -0.031869674339292*G0_0_0 - 0.0377086423093357*G0_0_1 - 0.0377086423093358*G0_0_2 - 0.0259654665036605*G0_1_0 - 0.029260079715808*G0_1_1 - 0.0292600797158081*G0_1_2 - 0.0377086423093364*G0_2_0 - 0.0435476102793806*G0_2_1 - 0.0435476102793806*G0_2_2;
    A[2489] = 0.012787013655069*G0_0_0 + 0.0378717419733037*G0_0_1 + 0.0378717419733038*G0_0_2 + 0.00949240044292077*G0_1_0 + 0.018984800885842*G0_1_1 + 0.018984800885842*G0_1_2 + 0.0378717419733044*G0_2_0 + 0.0567586830607662*G0_2_1 + 0.0567586830607661*G0_2_2;
    A[2490] = -0.00678494602105722*G0_0_0 + 0.0303691574307891*G0_0_1 + 0.030369157430789*G0_0_2 + 0.00296841388421322*G0_1_0 - 0.00352295274170222*G0_1_1 - 0.00352295274170222*G0_1_2 + 0.0303691574307889*G0_2_0 + 0.104709984267275*G0_2_1 + 0.104709984267275*G0_2_2;
    A[2491] = 0.043971669405697*G0_0_0 - 0.0422754329004322*G0_0_1 - 0.0422754329004322*G0_0_2 - 0.000685018588665141*G0_1_0 - 0.0429604514890971*G0_1_1 - 0.042960451489097*G0_1_2 - 0.0422754329004324*G0_2_0;
    A[2492] = -0.15657567740901*G0_0_0 - 0.022507753627545*G0_0_1 + 0.0678494602105704*G0_0_2 - 0.022507753627544*G0_1_0 + 0.0234863516113512*G0_1_1 + 0.104709984267275*G0_1_2 + 0.0678494602105707*G0_2_0 + 0.104709984267275*G0_2_1;
    A[2493] = 0.0195719596761274*G0_0_0 + 0.0748627457611833*G0_0_1 + 0.0141896707651933*G0_0_2 + 0.0748627457611844*G0_1_0 + 0.149725491522367*G0_1_1 + 0.0283793415303845*G0_1_2 + 0.0141896707651922*G0_2_0 + 0.0283793415303825*G0_2_1;
    A[2494] = -0.0127217737894823*G0_0_1 - 0.000652398655871077*G0_0_2 - 0.0127217737894827*G0_1_0 - 0.113517366121531*G0_1_1 - 0.012721773789483*G0_1_2 - 0.000652398655871787*G0_2_0 - 0.0127217737894823*G0_2_1;
    A[2495] = 0.0929668084616017*G0_0_0 - 0.270093043530538*G0_0_1 - 0.13504652176527*G0_0_2 - 0.270093043530545*G0_1_0 - 0.540186087061084*G0_1_1 - 0.270093043530542*G0_1_2 - 0.135046521765273*G0_2_0 - 0.270093043530542*G0_2_1 + 0.0929668084615994*G0_2_2;
    A[2496] = 0.0283793415303832*G0_0_1 + 0.0141896707651911*G0_0_2 + 0.0283793415303846*G0_1_0 + 0.149725491522365*G0_1_1 + 0.0748627457611848*G0_1_2 + 0.0141896707651918*G0_2_0 + 0.0748627457611824*G0_2_1 + 0.0195719596761261*G0_2_2;
    A[2497] = 0.104709984267275*G0_0_1 + 0.0678494602105717*G0_0_2 + 0.104709984267274*G0_1_0 + 0.0234863516113502*G0_1_1 - 0.0225077536275479*G0_1_2 + 0.0678494602105705*G0_2_0 - 0.0225077536275459*G0_2_1 - 0.156575677409011*G0_2_2;
    A[2498] = 0.039143919352252*G0_0_0 + 0.0127217737894826*G0_0_1 + 0.044689307927154*G0_0_2 + 0.0127217737894816*G0_1_0 + 0.0120693751336104*G0_1_2 + 0.0446893079271548*G0_2_0 + 0.0120693751336106*G0_2_1 + 0.0502346965020565*G0_2_2;
    A[2499] = 0.0195719596761253*G0_0_0 + 0.0181040627004154*G0_0_1 + 0.0298472385060912*G0_0_2 + 0.0181040627004162*G0_1_0 + 0.0102752788299658*G0_1_2 + 0.0298472385060928*G0_2_0 + 0.0102752788299668*G0_2_1 + 0.0401225173360581*G0_2_2;
    A[2500] = 0.012721773789482*G0_0_1 + 0.000652398655870886*G0_0_2 + 0.0127217737894825*G0_1_0 + 0.0120693751336114*G0_1_2 + 0.000652398655871628*G0_2_0 + 0.0120693751336106*G0_2_1 + 0.00130479731174186*G0_2_2;
    A[2501] = 0.0929668084615996*G0_0_0 - 0.0283793415303823*G0_0_1 + 0.0322937334656098*G0_0_2 - 0.0283793415303813*G0_1_0 - 0.0141896707651897*G0_1_2 + 0.0322937334656076*G0_2_0 - 0.0141896707651913*G0_2_1 - 0.00880738185425602*G0_2_2;
    A[2502] = -0.0283793415303818*G0_0_1 - 0.0141896707651905*G0_0_2 - 0.0283793415303845*G0_1_0 - 0.0141896707651917*G0_1_2 - 0.0141896707651917*G0_2_0 - 0.0141896707651911*G0_2_1 - 0.0332723314494147*G0_2_2;
    A[2503] = -0.104709984267276*G0_0_1 - 0.0678494602105716*G0_0_2 - 0.104709984267275*G0_1_0 - 0.0368605240567056*G0_1_2 - 0.0678494602105706*G0_2_0 - 0.0368605240567044*G0_2_1 - 0.0965550010688899*G0_2_2;
    A[2504] = 0.172233245149908*G0_0_0 + 0.198655390712678*G0_0_1 + 0.166035457919135*G0_0_2 + 0.100795592332049*G0_1_0 + 0.113517366121531*G0_1_1 + 0.100795592332049*G0_1_2 + 0.166035457919136*G0_2_0 + 0.19865539071268*G0_2_1 + 0.172233245149909*G0_2_2;
    A[2505] = -0.0371867233846378*G0_0_0 - 0.177615534060842*G0_0_1 - 0.224098938291642*G0_0_2 - 0.0748627457611822*G0_1_0 - 0.149725491522364*G0_1_1 - 0.121346149991982*G0_1_2 - 0.224098938291644*G0_2_0 - 0.326851726591306*G0_2_1 - 0.448197876583289*G0_2_2;
    A[2506] = -0.26813584756293*G0_0_0 + 0.0225077536275446*G0_0_1 + 0.0534966897814105*G0_0_2 - 0.0459941052388955*G0_1_0 - 0.0234863516113509*G0_1_1 + 0.0812236326559232*G0_1_2 + 0.0534966897814119*G0_2_0 - 0.104709984267275*G0_2_1 - 0.209419968534551*G0_2_2;
    A[2507] = -0.448197876583287*G0_0_0 - 0.326851726591305*G0_0_1 - 0.224098938291642*G0_0_2 - 0.121346149991979*G0_1_0 - 0.149725491522363*G0_1_1 - 0.0748627457611802*G0_1_2 - 0.224098938291643*G0_2_0 - 0.177615534060844*G0_2_1 - 0.0371867233846378*G0_2_2;
    A[2508] = 0.726119703984279*G0_0_0 + 0.270093043530538*G0_0_1 + 0.270093043530539*G0_0_2 + 0.270093043530538*G0_1_0 + 0.540186087061083*G0_1_1 + 0.270093043530542*G0_1_2 + 0.270093043530539*G0_2_0 + 0.270093043530542*G0_2_1 + 0.726119703984282*G0_2_2;
    A[2509] = -0.209419968534552*G0_0_0 - 0.104709984267275*G0_0_1 + 0.0534966897814105*G0_0_2 + 0.0812236326559236*G0_1_0 - 0.0234863516113504*G0_1_1 - 0.0459941052388989*G0_1_2 + 0.0534966897814119*G0_2_0 + 0.0225077536275458*G0_2_1 - 0.268135847562931*G0_2_2;
    A[2510] = 0.0502346965020567*G0_0_0 + 0.0120693751336111*G0_0_1 + 0.0446893079271546*G0_0_2 + 0.0120693751336108*G0_1_0 + 0.0127217737894821*G0_1_2 + 0.0446893079271547*G0_2_0 + 0.0127217737894819*G0_2_1 + 0.0391439193522527*G0_2_2;
    A[2511] = -0.00880738185425671*G0_0_0 - 0.0141896707651911*G0_0_1 + 0.0322937334656087*G0_0_2 - 0.0141896707651916*G0_1_0 - 0.0283793415303826*G0_1_2 + 0.0322937334656083*G0_2_0 - 0.0283793415303833*G0_2_1 + 0.0929668084615993*G0_2_2;
    A[2512] = -0.096555001068889*G0_0_0 - 0.0368605240567048*G0_0_1 - 0.0678494602105707*G0_0_2 - 0.0368605240567038*G0_1_0 - 0.104709984267275*G0_1_2 - 0.0678494602105708*G0_2_0 - 0.104709984267276*G0_2_1;
    A[2513] = 0.0401225173360579*G0_0_0 + 0.0102752788299657*G0_0_1 + 0.0298472385060919*G0_0_2 + 0.0102752788299657*G0_1_0 + 0.0181040627004158*G0_1_2 + 0.0298472385060918*G0_2_0 + 0.018104062700417*G0_2_1 + 0.0195719596761263*G0_2_2;
    A[2514] = -0.0332723314494142*G0_0_0 - 0.0141896707651915*G0_0_1 - 0.0141896707651931*G0_0_2 - 0.0141896707651917*G0_1_0 - 0.0283793415303845*G0_1_2 - 0.0141896707651907*G0_2_0 - 0.0283793415303825*G0_2_1;
    A[2515] = 0.00130479731174125*G0_0_0 + 0.012069375133611*G0_0_1 + 0.000652398655870861*G0_0_2 + 0.0120693751336117*G0_1_0 + 0.0127217737894828*G0_1_2 + 0.000652398655870208*G0_2_0 + 0.0127217737894814*G0_2_1;
    A[2516] = -0.234863516113514*G0_0_0 - 0.0929668084616005*G0_0_1 - 0.215291556437389*G0_0_2 - 0.0929668084615985*G0_1_0 - 0.0929668084615986*G0_1_2 - 0.215291556437389*G0_2_0 - 0.0929668084616003*G0_2_1 - 0.234863516113515*G0_2_2;
    A[2517] = 0.352295274170272*G0_0_0 + 0.10275278829966*G0_0_1 + 0.102752788299661*G0_0_2 + 0.102752788299661*G0_1_0 + 0.298472385060925*G0_1_2 + 0.102752788299663*G0_2_0 + 0.298472385060924*G0_2_1 - 0.185933616923198*G0_2_2;
    A[2518] = -0.0391439193522524*G0_0_0 - 0.0929668084615958*G0_0_1 - 0.0440369092712827*G0_0_2 - 0.0929668084616003*G0_1_0 - 0.0929668084616002*G0_1_2 - 0.0440369092712849*G0_2_0 - 0.0929668084615974*G0_2_1 - 0.0391439193522523*G0_2_2;
    A[2519] = -0.185933616923201*G0_0_0 + 0.298472385060922*G0_0_1 + 0.102752788299663*G0_0_2 + 0.298472385060926*G0_1_0 + 0.102752788299664*G0_1_2 + 0.102752788299666*G0_2_0 + 0.102752788299662*G0_2_1 + 0.352295274170274*G0_2_2;
    A[2520] = 0.0189195610202553*G0_0_0 + 0.0189195610202553*G0_0_1 + 0.0189195610202553*G0_0_2 + 0.0137421252872642*G0_1_0 + 0.0137421252872642*G0_1_1 + 0.0137421252872642*G0_1_2 + 0.0189195610202553*G0_2_0 + 0.0189195610202553*G0_2_1 + 0.0189195610202554*G0_2_2;
    A[2521] = 0.0189195610202554*G0_0_0 + 0.00517743573299142*G0_1_0;
    A[2522] = -0.0137421252872642*G0_1_1;
    A[2523] = -0.0143945239431352*G0_1_2 + 0.0335985307773495*G0_2_2;
    A[2524] = 0.0641307878721066*G0_1_1 + 0.00939454064454004*G0_1_2 + 0.00939454064454105*G0_2_1;
    A[2525] = -0.127087258163646*G0_1_1 - 0.0377738821749227*G0_1_2 - 0.0377738821749251*G0_2_1 - 0.0150051690850297*G0_2_2;
    A[2526] = 0.124738623002511*G0_1_1 + 0.0623693115012549*G0_1_2 + 0.062369311501257*G0_2_1 + 0.0476251018785729*G0_2_2;
    A[2527] = -0.124738623002511*G0_1_1 - 0.0962940416065408*G0_1_2 - 0.0962940416065424*G0_2_1 - 0.128522535206562*G0_2_2;
    A[2528] = -0.111886369481855*G0_0_0 - 0.0212681961813906*G0_0_2 - 0.047755581609748*G0_1_0 - 0.0118736555368495*G0_1_2 - 0.0212681961813899*G0_2_0;
    A[2529] = 0.209419968534551*G0_0_0 + 0.0359471659384856*G0_0_2 + 0.0823327103709041*G0_1_0 - 0.00182671623643881*G0_1_2 + 0.0359471659384848*G0_2_0 - 0.0528442911255408*G0_2_2;
    A[2530] = -0.271397840842284*G0_0_0 - 0.0733296089198866*G0_0_2 - 0.146659217839773*G0_1_0 - 0.010960297418631*G0_1_2 - 0.0733296089198847*G0_2_0 + 0.121346149991984*G0_2_2;
    A[2531] = -0.0962940416065409*G0_0_2 - 0.124738623002511*G0_1_0 - 0.192588083213082*G0_1_2 - 0.0962940416065423*G0_2_0 - 0.257045070413125*G0_2_2;
    A[2532] = -0.0306627368259311*G0_0_0 - 0.0212681961813907*G0_0_1 - 0.0212681961813903*G0_1_0 - 0.0118736555368496*G0_1_1;
    A[2533] = -0.00130479731174182*G0_0_0 - 0.00123955744615439*G0_0_1 - 0.00123955744615474*G0_1_0 - 0.00117431758056763*G0_1_1;
    A[2534] = -0.01500516908503*G0_0_0 - 0.0149399292194429*G0_0_1 - 0.0149399292194428*G0_1_0 - 0.0148746893538557*G0_1_1;
    A[2535] = 0.00939454064454031*G0_0_1 + 0.00939454064454017*G0_1_0 + 0.0187890812890809*G0_1_1;
    A[2536] = -0.111886369481855*G0_0_0 - 0.111886369481855*G0_0_1 - 0.0906181733004641*G0_0_2 - 0.0641307878721073*G0_1_0 - 0.0641307878721073*G0_1_1 - 0.0547362472275667*G0_1_2 - 0.0906181733004643*G0_2_0 - 0.0906181733004643*G0_2_1 - 0.0693499771190741*G0_2_2;
    A[2537] = 0.209419968534551*G0_0_0 + 0.20941996853455*G0_0_1 + 0.173472802596065*G0_0_2 + 0.127087258163648*G0_1_0 + 0.127087258163647*G0_1_1 + 0.0893133759887233*G0_1_2 + 0.173472802596066*G0_2_0 + 0.173472802596066*G0_2_1 + 0.0846813455320404*G0_2_2;
    A[2538] = -0.271397840842284*G0_0_0 - 0.271397840842284*G0_0_1 - 0.198068231922397*G0_0_2 - 0.124738623002512*G0_1_0 - 0.124738623002512*G0_1_1 - 0.0623693115012563*G0_1_2 - 0.198068231922398*G0_2_0 - 0.198068231922397*G0_2_1 - 0.00339247301052978*G0_2_2;
    A[2539] = 0.0962940416065412*G0_0_2 + 0.124738623002511*G0_1_0 + 0.124738623002511*G0_1_1 + 0.0284445813959708*G0_1_2 + 0.0962940416065423*G0_2_0 + 0.0962940416065424*G0_2_1 - 0.0644569872000414*G0_2_2;
    A[2540] = -0.0306627368259311*G0_0_0 - 0.00939454064454056*G0_0_1 - 0.030662736825931*G0_0_2 - 0.00939454064454092*G0_1_0 - 0.00939454064454073*G0_1_2 - 0.0306627368259311*G0_2_0 - 0.00939454064454068*G0_2_1 - 0.0306627368259311*G0_2_2;
    A[2541] = -0.00130479731174169*G0_0_0 - 6.52398655867775e-05*G0_0_1 - 0.00130479731174172*G0_0_2 - 6.52398655868603e-05*G0_1_0 - 6.52398655869856e-05*G0_1_2 - 0.00130479731174152*G0_2_0 - 6.52398655867656e-05*G0_2_1 - 0.00130479731174163*G0_2_2;
    A[2542] = -0.0150051690850298*G0_0_0 - 6.52398655873263e-05*G0_0_1 - 0.0150051690850298*G0_0_2 - 6.52398655873268e-05*G0_1_0 - 6.52398655873427e-05*G0_1_2 - 0.0150051690850298*G0_2_0 - 6.52398655874169e-05*G0_2_1 - 0.0150051690850297*G0_2_2;
    A[2543] = -0.00939454064454075*G0_0_1 - 0.00939454064454024*G0_1_0 - 0.00939454064454018*G0_1_2 - 0.00939454064454044*G0_2_1;
    A[2544] = -0.0187890812890811*G0_0_0 - 0.0400572774704715*G0_0_1 - 0.0400572774704716*G0_0_2 - 0.00939454064454079*G0_1_0 - 0.0187890812890815*G0_1_1 - 0.0187890812890813*G0_1_2 - 0.0400572774704714*G0_2_0 - 0.0613254736518618*G0_2_1 - 0.0613254736518618*G0_2_2;
    A[2545] = -0.000130479731174288*G0_0_0 + 0.0136351319077014*G0_0_1 + 0.0136351319077013*G0_0_2 - 6.52398655870825e-05*G0_1_0 + 0.0148746893538558*G0_1_1 + 0.0148746893538556*G0_1_2 + 0.013635131907701*G0_2_0 - 0.0254435475789644*G0_2_1 - 0.0254435475789644*G0_2_2;
    A[2546] = -0.000130479731174058*G0_0_0 - 0.0137656116388755*G0_0_1 - 0.0137656116388753*G0_0_2 - 6.52398655869601e-05*G0_1_0 + 0.00117431758056771*G0_1_1 + 0.00117431758056771*G0_1_2 - 0.0137656116388753*G0_2_0 - 0.0528442911255406*G0_2_1 - 0.0528442911255405*G0_2_2;
    A[2547] = -0.0187890812890812*G0_0_0 + 0.0212681961813905*G0_0_1 + 0.0212681961813904*G0_0_2 - 0.00939454064454057*G0_1_0 + 0.0118736555368495*G0_1_1 + 0.0118736555368495*G0_1_2 + 0.0212681961813905*G0_2_0;
    A[2548] = 0.0521918924696708*G0_0_0 - 0.000652398655869865*G0_0_1 - 0.03783912204051*G0_0_2 - 0.000652398655870228*G0_1_0 - 0.0534966897814111*G0_1_1 - 0.0528442911255398*G0_1_2 - 0.0378391220405101*G0_2_0 - 0.0528442911255399*G0_2_1;
    A[2549] = 0.0260959462348353*G0_0_0 + 0.0198981590040606*G0_0_1 - 0.0120693751336114*G0_0_2 + 0.0198981590040618*G0_1_0 + 0.0137003717732885*G0_1_1 - 0.012721773789482*G0_1_2 - 0.0120693751336107*G0_2_0 - 0.0127217737894827*G0_2_1;
    A[2550] = -0.0528442911255395*G0_0_1 - 0.0150051690850299*G0_0_2 - 0.0528442911255406*G0_1_0 - 0.105688582251081*G0_1_1 - 0.0300103381700601*G0_1_2 - 0.01500516908503*G0_2_0 - 0.0300103381700591*G0_2_1;
    A[2551] = -0.185933616923202*G0_0_0 - 0.0812236326559242*G0_0_1 + 0.00913358118219186*G0_0_2 - 0.0812236326559258*G0_1_0 + 0.0234863516113502*G0_1_1 + 0.0459941052388953*G0_1_2 + 0.00913358118219322*G0_2_0 + 0.0459941052388989*G0_2_1 - 0.0880738185425676*G0_2_2;
    A[2552] = 0.104709984267274*G0_0_1 + 0.036860524056705*G0_0_2 + 0.104709984267275*G0_1_0 + 0.209419968534549*G0_1_1 + 0.073721048113409*G0_1_2 + 0.0368605240567039*G0_2_0 + 0.0737210481134081*G0_2_1 - 0.0228339529554808*G0_2_2;
    A[2553] = -0.271397840842285*G0_0_1 - 0.135698920421144*G0_0_2 - 0.271397840842284*G0_1_0 - 0.542795681684567*G0_1_1 - 0.271397840842284*G0_1_2 - 0.135698920421142*G0_2_0 - 0.271397840842284*G0_2_1 + 0.0521918924696704*G0_2_2;
    A[2554] = 0.0521918924696701*G0_0_0 + 0.05284429112554*G0_0_1 + 0.0900310145101808*G0_0_2 + 0.0528442911255427*G0_1_0 + 0.0378391220405119*G0_1_2 + 0.0900310145101808*G0_2_0 + 0.037839122040511*G0_2_1 + 0.127870136550692*G0_2_2;
    A[2555] = 0.0260959462348342*G0_0_0 + 0.00619778723077353*G0_0_1 + 0.0381653213684453*G0_0_2 + 0.0061977872307722*G0_1_0 + 0.00554538857490196*G0_1_2 + 0.0381653213684446*G0_2_0 + 0.00554538857490169*G0_2_1 + 0.0502346965020559*G0_2_2;
    A[2556] = 0.0528442911255416*G0_0_1 + 0.0150051690850303*G0_0_2 + 0.0528442911255407*G0_1_0 + 0.0378391220405105*G0_1_2 + 0.01500516908503*G0_2_0 + 0.0378391220405111*G0_2_1 + 0.0300103381700598*G0_2_2;
    A[2557] = -0.185933616923198*G0_0_0 - 0.104709984267275*G0_0_1 - 0.19506719810539*G0_0_2 - 0.104709984267277*G0_1_0 - 0.0678494602105729*G0_1_2 - 0.19506719810539*G0_2_0 - 0.0678494602105709*G0_2_1 - 0.292274597830151*G0_2_2;
    A[2558] = -0.104709984267276*G0_0_1 - 0.0368605240567039*G0_0_2 - 0.104709984267275*G0_1_0 - 0.0678494602105703*G0_1_2 - 0.0368605240567038*G0_2_0 - 0.0678494602105702*G0_2_1 - 0.0965550010688874*G0_2_2;
    A[2559] = 0.271397840842283*G0_0_1 + 0.135698920421141*G0_0_2 + 0.271397840842284*G0_1_0 + 0.135698920421142*G0_1_2 + 0.135698920421142*G0_2_0 + 0.135698920421141*G0_2_1 + 0.323589733311952*G0_2_2;
    A[2560] = 0.105688582251082*G0_0_0 + 0.105036183595211*G0_0_1 + 0.127870136550691*G0_0_2 + 0.0528442911255412*G0_1_0 + 0.105688582251083*G0_1_1 + 0.0756782440810225*G0_1_2 + 0.127870136550691*G0_2_0 + 0.18006202902036*G0_2_1 + 0.255740273101382*G0_2_2;
    A[2561] = 0.012395574461546*G0_0_0 + 0.00619778723077308*G0_0_1 + 0.0061977872307728*G0_0_2 + 0.00619778723077291*G0_1_0 - 0.0137003717732904*G0_1_1 - 0.0264221455627708*G0_1_2 + 0.0061977872307737*G0_2_0 + 0.0714376528178611*G0_2_1 + 0.172233245149911*G0_2_2;
    A[2562] = 0.105688582251082*G0_0_0 + 0.000652398655870582*G0_0_1 - 0.0221815542996097*G0_0_2 + 0.0528442911255411*G0_1_0 + 0.0534966897814117*G0_1_1 + 0.0006523986558713*G0_1_2 - 0.0221815542996101*G0_2_0 + 0.0528442911255402*G0_2_1 + 0.105688582251082*G0_2_2;
    A[2563] = -0.20941996853455*G0_0_0 - 0.290643601190473*G0_0_1 - 0.262916658315962*G0_0_2 - 0.104709984267276*G0_1_0 - 0.209419968534552*G0_1_1 - 0.135698920421144*G0_1_2 - 0.262916658315962*G0_2_0 - 0.390134396210781*G0_2_1 - 0.584549195660303*G0_2_2;
    A[2564] = -0.209419968534552*G0_0_0 + 0.0812236326559236*G0_0_1 + 0.0534966897814119*G0_0_2 - 0.104709984267275*G0_1_0 - 0.0234863516113504*G0_1_1 + 0.0225077536275458*G0_1_2 + 0.0534966897814106*G0_2_0 - 0.0459941052388989*G0_2_1 - 0.268135847562931*G0_2_2;
    A[2565] = 0.542795681684567*G0_0_0 + 0.271397840842285*G0_0_1 + 0.271397840842282*G0_0_2 + 0.271397840842285*G0_1_0 + 0.542795681684567*G0_1_1 + 0.271397840842284*G0_1_2 + 0.271397840842282*G0_2_0 + 0.271397840842284*G0_2_1 + 0.647179466623907*G0_2_2;
    A[2566] = 0.0300103381700604*G0_0_0 + 0.0150051690850299*G0_0_1 - 0.00782878387045024*G0_0_2 + 0.0150051690850298*G0_1_0 + 0.0300103381700603*G0_1_2 - 0.00782878387045082*G0_2_0 + 0.03001033817006*G0_2_1 - 0.0456679059109614*G0_2_2;
    A[2567] = 0.00130479731174142*G0_0_0 + 0.00065239865587048*G0_0_1 + 0.000652398655870939*G0_0_2 + 0.000652398655871106*G0_1_0 + 0.0127217737894808*G0_1_2 + 0.000652398655870965*G0_2_0 + 0.0127217737894816*G0_2_1 - 0.088073818542568*G0_2_2;
    A[2568] = 0.0300103381700604*G0_0_0 + 0.0150051690850303*G0_0_1 + 0.0378391220405105*G0_0_2 + 0.0150051690850301*G0_1_0 + 0.0528442911255402*G0_1_2 + 0.0378391220405106*G0_2_0 + 0.0528442911255409*G0_2_1;
    A[2569] = 0.00130479731174153*G0_0_0 + 0.000652398655870862*G0_0_1 - 0.0107645778218694*G0_0_2 + 0.00065239865587076*G0_1_0 + 0.00130479731174125*G0_1_2 - 0.0107645778218696*G0_2_0 + 0.00130479731174177*G0_2_1 - 0.0228339529554804*G0_2_2;
    A[2570] = 0.00130479731174181*G0_0_0 + 0.000652398655870665*G0_0_1 + 0.0120693751336111*G0_0_2 + 0.000652398655870599*G0_1_0 + 0.0127217737894818*G0_1_2 + 0.0120693751336112*G0_2_0 + 0.012721773789482*G0_2_1;
    A[2571] = 0.0300103381700601*G0_0_0 + 0.0150051690850305*G0_0_1 + 0.01500516908503*G0_0_2 + 0.0150051690850303*G0_1_0 + 0.0300103381700604*G0_1_2 + 0.0150051690850295*G0_2_0 + 0.0300103381700602*G0_2_1;
    A[2572] = -0.0521918924696685*G0_0_0 - 0.0260959462348341*G0_0_1 - 0.0424059126316054*G0_0_2 - 0.0260959462348396*G0_1_0 - 0.075025845425153*G0_1_2 - 0.0424059126316058*G0_2_0 - 0.0750258454251499*G0_2_1 + 0.143527704291593*G0_2_2;
    A[2573] = -0.0521918924696727*G0_0_0 - 0.0260959462348348*G0_0_1 - 0.00978597983806334*G0_0_2 - 0.0260959462348326*G0_1_0 - 0.0587158790283768*G0_1_2 - 0.00978597983806496*G0_2_0 - 0.0587158790283791*G0_2_1 + 0.176147637085135*G0_2_2;
    A[2574] = -0.0521918924696696*G0_0_0 - 0.0260959462348371*G0_0_1 - 0.0260959462348354*G0_0_2 - 0.0260959462348344*G0_1_0 - 0.0750258454251508*G0_1_2 - 0.0260959462348341*G0_2_0 - 0.0750258454251515*G0_2_1 + 0.0456679059109613*G0_2_2;
    A[2575] = 0.3718672338464*G0_0_0 + 0.185933616923199*G0_0_1 + 0.1859336169232*G0_0_2 + 0.185933616923203*G0_1_0 + 0.316413348097377*G0_1_2 + 0.185933616923198*G0_2_0 + 0.316413348097373*G0_2_1 - 0.150051690850301*G0_2_2;
    A[2576] = 0.0335985307773496*G0_0_0 + 0.0335985307773497*G0_0_1 + 0.0335985307773497*G0_0_2 + 0.0335985307773495*G0_1_0 + 0.0335985307773496*G0_1_1 + 0.0335985307773496*G0_1_2 + 0.0479930547204848*G0_2_0 + 0.0479930547204848*G0_2_1 + 0.0479930547204847*G0_2_2;
    A[2577] = 0.0189195610202554*G0_0_0 + 0.00517743573299127*G0_2_0;
    A[2578] = 0.0189195610202552*G0_1_1 + 0.00517743573299122*G0_2_1;
    A[2579] = -0.0137421252872641*G0_2_2;
    A[2580] = -0.0306627368259309*G0_1_1 - 0.0212681961813905*G0_1_2 - 0.0212681961813905*G0_2_1 - 0.0118736555368498*G0_2_2;
    A[2581] = -0.00130479731174253*G0_1_1 - 0.00123955744615509*G0_1_2 - 0.00123955744615478*G0_2_1 - 0.00117431758056822*G0_2_2;
    A[2582] = -0.0150051690850296*G0_1_1 - 0.0149399292194432*G0_1_2 - 0.014939929219443*G0_2_1 - 0.0148746893538552*G0_2_2;
    A[2583] = 0.00939454064454065*G0_1_2 + 0.00939454064454026*G0_2_1 + 0.0187890812890805*G0_2_2;
    A[2584] = -0.0306627368259311*G0_0_0 - 0.0212681961813905*G0_0_2 - 0.0212681961813903*G0_2_0 - 0.0118736555368496*G0_2_2;
    A[2585] = -0.00130479731174143*G0_0_0 - 0.00123955744615477*G0_0_2 - 0.00123955744615513*G0_2_0 - 0.00117431758056812*G0_2_2;
    A[2586] = -0.0150051690850302*G0_0_0 - 0.0149399292194428*G0_0_2 - 0.0149399292194428*G0_2_0 - 0.0148746893538557*G0_2_2;
    A[2587] = 0.00939454064454039*G0_0_2 + 0.0093945406445402*G0_2_0 + 0.0187890812890807*G0_2_2;
    A[2588] = -0.0613254736518619*G0_0_0 - 0.0212681961813905*G0_0_1 - 0.02126819618139*G0_1_0 - 0.0425363923627803*G0_2_0 - 0.0118736555368496*G0_2_1;
    A[2589] = -0.0254435475789639*G0_0_0 - 0.0390786794866652*G0_0_1 - 0.0390786794866657*G0_1_0 - 0.0528442911255407*G0_1_1 - 0.0403182369328203*G0_2_0 - 0.0540186087061083*G0_2_1;
    A[2590] = -0.0528442911255402*G0_0_0 - 0.0390786794866648*G0_0_1 - 0.0390786794866642*G0_1_0 - 0.0254435475789638*G0_1_1 - 0.0540186087061073*G0_2_0 - 0.0403182369328196*G0_2_1;
    A[2591] = -0.02126819618139*G0_0_1 - 0.0212681961813906*G0_1_0 - 0.0613254736518615*G0_1_1 - 0.0118736555368498*G0_2_0 - 0.0425363923627802*G0_2_1;
    A[2592] = -0.128522535206562*G0_0_0 - 0.128522535206562*G0_0_1 - 0.0322284936000209*G0_0_2 - 0.128522535206562*G0_1_0 - 0.128522535206562*G0_1_1 - 0.0322284936000205*G0_1_2 - 0.0322284936000211*G0_2_0 - 0.0322284936000211*G0_2_1 - 0.0606730749959915*G0_2_2;
    A[2593] = 0.0476251018785736*G0_0_0 + 0.0476251018785736*G0_0_1 - 0.0147442096226816*G0_0_2 + 0.0476251018785737*G0_1_0 + 0.0476251018785736*G0_1_1 - 0.0147442096226823*G0_1_2 - 0.0147442096226816*G0_2_0 - 0.0147442096226814*G0_2_1 + 0.0476251018785742*G0_2_2;
    A[2594] = -0.0150051690850301*G0_0_0 - 0.01500516908503*G0_0_1 + 0.0227687130898935*G0_0_2 - 0.0150051690850299*G0_1_0 - 0.0150051690850299*G0_1_1 + 0.0227687130898942*G0_1_2 + 0.0227687130898933*G0_2_0 + 0.0227687130898931*G0_2_1 - 0.066544662898829*G0_2_2;
    A[2595] = -0.00939454064454043*G0_0_2 - 0.0093945406445405*G0_1_2 - 0.00939454064454034*G0_2_0 - 0.00939454064454026*G0_2_1 + 0.045341706583026*G0_2_2;
    A[2596] = -0.257045070413123*G0_0_0 - 0.160751028806582*G0_0_1 - 0.257045070413123*G0_0_2 - 0.160751028806582*G0_1_0 - 0.0644569872000412*G0_1_1 - 0.160751028806582*G0_1_2 - 0.0644569872000414*G0_2_0 - 0.0929015685960118*G0_2_1 - 0.0644569872000415*G0_2_2;
    A[2597] = 0.121346149991982*G0_0_0 + 0.194675758911868*G0_0_1 + 0.121346149991981*G0_0_2 + 0.194675758911868*G0_1_0 - 0.00339247301052964*G0_1_1 + 0.194675758911868*G0_1_2 + 0.132306447410612*G0_2_0 + 0.0589768384907262*G0_2_1 + 0.132306447410612*G0_2_2;
    A[2598] = -0.05284429112554*G0_0_0 - 0.0887914570640253*G0_0_1 - 0.0528442911255399*G0_0_2 - 0.0887914570640253*G0_1_0 + 0.0846813455320401*G0_1_1 - 0.0887914570640254*G0_1_2 - 0.0510175748891016*G0_2_0 - 0.00463203045668273*G0_2_1 - 0.0510175748891017*G0_2_2;
    A[2599] = 0.0212681961813904*G0_0_1 + 0.0212681961813907*G0_1_0 - 0.0693499771190737*G0_1_1 + 0.0212681961813908*G0_1_2 + 0.0118736555368499*G0_2_0 - 0.0146137298915075*G0_2_1 + 0.01187365553685*G0_2_2;
    A[2600] = -0.0644569872000418*G0_0_0 - 0.160751028806583*G0_0_1 - 0.160751028806583*G0_0_2 - 0.160751028806582*G0_1_0 - 0.257045070413124*G0_1_1 - 0.257045070413124*G0_1_2 - 0.0929015685960122*G0_2_0 - 0.064456987200042*G0_2_1 - 0.0644569872000421*G0_2_2;
    A[2601] = -0.00339247301052863*G0_0_0 + 0.194675758911868*G0_0_1 + 0.194675758911868*G0_0_2 + 0.194675758911868*G0_1_0 + 0.121346149991982*G0_1_1 + 0.121346149991982*G0_1_2 + 0.0589768384907274*G0_2_0 + 0.132306447410613*G0_2_1 + 0.132306447410613*G0_2_2;
    A[2602] = 0.0846813455320391*G0_0_0 - 0.0887914570640259*G0_0_1 - 0.0887914570640258*G0_0_2 - 0.0887914570640257*G0_1_0 - 0.0528442911255407*G0_1_1 - 0.0528442911255409*G0_1_2 - 0.00463203045668371*G0_2_0 - 0.0510175748891023*G0_2_1 - 0.0510175748891024*G0_2_2;
    A[2603] = -0.0693499771190738*G0_0_0 + 0.0212681961813903*G0_0_1 + 0.0212681961813902*G0_0_2 + 0.0212681961813903*G0_1_0 - 0.0146137298915074*G0_2_0 + 0.0118736555368494*G0_2_1 + 0.0118736555368496*G0_2_2;
    A[2604] = -0.0456679059109606*G0_0_0 - 0.0378391220405101*G0_0_1 - 0.0756782440810206*G0_0_2 - 0.0378391220405096*G0_1_0 - 0.0528442911255397*G0_1_2 - 0.0756782440810203*G0_2_0 - 0.0528442911255399*G0_2_1 - 0.10568858225108*G0_2_2;
    A[2605] = -0.0880738185425657*G0_0_0 - 0.0887262171984371*G0_0_1 - 0.100795592332048*G0_0_2 - 0.0887262171984369*G0_1_0 - 0.0880738185425661*G0_1_1 - 0.100795592332049*G0_1_2 - 0.100795592332048*G0_2_0 - 0.100795592332048*G0_2_1 - 0.113517366121531*G0_2_2;
    A[2606] = -0.0378391220405102*G0_0_1 - 0.0528442911255392*G0_0_2 - 0.0378391220405109*G0_1_0 - 0.0456679059109607*G0_1_1 - 0.0756782440810198*G0_1_2 - 0.052844291125541*G0_2_0 - 0.0756782440810215*G0_2_1 - 0.10568858225108*G0_2_2;
    A[2607] = -0.0228339529554802*G0_0_0 - 0.0120693751336111*G0_0_1 - 0.0241387502672215*G0_0_2 - 0.0120693751336114*G0_1_0 - 0.012721773789481*G0_1_2 - 0.0241387502672214*G0_2_0 - 0.0127217737894824*G0_2_1 - 0.0254435475789632*G0_2_2;
    A[2608] = -0.0120693751336106*G0_0_1 - 0.0127217737894824*G0_0_2 - 0.0120693751336124*G0_1_0 - 0.0228339529554813*G0_1_1 - 0.0241387502672231*G0_1_2 - 0.0127217737894819*G0_2_0 - 0.024138750267221*G0_2_1 - 0.0254435475789638*G0_2_2;
    A[2609] = -0.0150051690850298*G0_0_1 - 0.0300103381700595*G0_0_2 - 0.0150051690850298*G0_1_0 - 0.0300103381700607*G0_1_2 - 0.0300103381700603*G0_2_0 - 0.0300103381700604*G0_2_1 - 0.06002067634012*G0_2_2;
    A[2610] = 0.0521918924696686*G0_0_0 + 0.187890812890811*G0_0_1 + 0.323589733311953*G0_0_2 + 0.187890812890811*G0_1_0 + 0.323589733311953*G0_1_1 + 0.187890812890811*G0_1_2 + 0.323589733311951*G0_2_0 + 0.187890812890811*G0_2_1 + 0.0521918924696693*G0_2_2;
    A[2611] = -0.0880738185425669*G0_0_0 - 0.0972073997247594*G0_0_1 - 0.134067923781464*G0_0_2 - 0.0972073997247587*G0_1_0 - 0.29227459783015*G0_1_1 - 0.22442513761958*G0_1_2 - 0.134067923781463*G0_2_0 - 0.224425137619579*G0_2_1 - 0.156575677409009*G0_2_2;
    A[2612] = 0.03783912204051*G0_0_1 + 0.0528442911255403*G0_0_2 + 0.0378391220405109*G0_1_0 + 0.12787013655069*G0_1_1 + 0.0900310145101802*G0_1_2 + 0.052844291125541*G0_2_0 + 0.0900310145101796*G0_2_1 + 0.0521918924696696*G0_2_2;
    A[2613] = -0.0228339529554803*G0_0_0 - 0.0596944770121847*G0_0_1 - 0.0965550010688891*G0_0_2 - 0.0596944770121847*G0_1_0 - 0.096555001068889*G0_1_1 - 0.0287055408583183*G0_1_2 - 0.0965550010688895*G0_2_0 - 0.0287055408583185*G0_2_1 + 0.0391439193522524*G0_2_2;
    A[2614] = 0.0120693751336115*G0_0_1 + 0.0127217737894822*G0_0_2 + 0.0120693751336126*G0_1_0 + 0.0502346965020578*G0_1_1 + 0.044689307927155*G0_1_2 + 0.012721773789482*G0_2_0 + 0.044689307927155*G0_2_1 + 0.0391439193522518*G0_2_2;
    A[2615] = 0.01500516908503*G0_0_1 + 0.0300103381700598*G0_0_2 + 0.0150051690850299*G0_1_0 + 0.0300103381700599*G0_1_1 - 0.0078287838704508*G0_1_2 + 0.0300103381700606*G0_2_0 - 0.00782878387044984*G0_2_1 - 0.0456679059109605*G0_2_2;
    A[2616] = 0.323589733311953*G0_0_0 + 0.18789081289081*G0_0_1 + 0.187890812890811*G0_0_2 + 0.187890812890811*G0_1_0 + 0.0521918924696691*G0_1_1 + 0.323589733311953*G0_1_2 + 0.18789081289081*G0_2_0 + 0.323589733311951*G0_2_1 + 0.0521918924696697*G0_2_2;
    A[2617] = -0.292274597830151*G0_0_0 - 0.0972073997247596*G0_0_1 - 0.224425137619581*G0_0_2 - 0.09720739972476*G0_1_0 - 0.0880738185425679*G0_1_1 - 0.134067923781465*G0_1_2 - 0.224425137619579*G0_2_0 - 0.134067923781464*G0_2_1 - 0.156575677409009*G0_2_2;
    A[2618] = 0.127870136550691*G0_0_0 + 0.0378391220405103*G0_0_1 + 0.0900310145101809*G0_0_2 + 0.0378391220405109*G0_1_0 + 0.0528442911255415*G0_1_2 + 0.0900310145101805*G0_2_0 + 0.0528442911255399*G0_2_1 + 0.05219189246967*G0_2_2;
    A[2619] = -0.0965550010688889*G0_0_0 - 0.0596944770121845*G0_0_1 - 0.0287055408583177*G0_0_2 - 0.0596944770121848*G0_1_0 - 0.0228339529554807*G0_1_1 - 0.0965550010688891*G0_1_2 - 0.0287055408583181*G0_2_0 - 0.0965550010688892*G0_2_1 + 0.0391439193522528*G0_2_2;
    A[2620] = 0.0502346965020567*G0_0_0 + 0.0120693751336108*G0_0_1 + 0.0446893079271547*G0_0_2 + 0.0120693751336111*G0_1_0 + 0.0127217737894819*G0_1_2 + 0.0446893079271546*G0_2_0 + 0.0127217737894821*G0_2_1 + 0.0391439193522527*G0_2_2;
    A[2621] = 0.0300103381700604*G0_0_0 + 0.0150051690850298*G0_0_1 - 0.00782878387045082*G0_0_2 + 0.0150051690850299*G0_1_0 + 0.03001033817006*G0_1_2 - 0.00782878387045024*G0_2_0 + 0.0300103381700603*G0_2_1 - 0.0456679059109614*G0_2_2;
    A[2622] = 0.647179466623905*G0_0_0 + 0.375781625781621*G0_0_1 + 0.37578162578162*G0_0_2 + 0.375781625781621*G0_1_0 + 0.647179466623904*G0_1_1 + 0.375781625781621*G0_1_2 + 0.37578162578162*G0_2_0 + 0.375781625781621*G0_2_1 + 0.647179466623902*G0_2_2;
    A[2623] = -0.584549195660301*G0_0_0 - 0.32163253734434*G0_0_1 - 0.19441479944952*G0_0_2 - 0.32163253734434*G0_1_0 - 0.268135847562929*G0_1_1 - 0.222141742324032*G0_1_2 - 0.448850275239159*G0_2_0 - 0.290643601190473*G0_2_1 - 0.268135847562929*G0_2_2;
    A[2624] = 0.255740273101382*G0_0_0 + 0.127870136550692*G0_0_1 + 0.0756782440810212*G0_0_2 + 0.127870136550691*G0_1_0 + 0.105688582251082*G0_1_1 + 0.0528442911255402*G0_1_2 + 0.180062029020361*G0_2_0 + 0.105036183595211*G0_2_1 + 0.105688582251081*G0_2_2;
    A[2625] = -0.268135847562927*G0_0_0 - 0.321632537344338*G0_0_1 - 0.22214174232403*G0_0_2 - 0.321632537344339*G0_1_0 - 0.5845491956603*G0_1_1 - 0.194414799449518*G0_1_2 - 0.290643601190472*G0_2_0 - 0.448850275239158*G0_2_1 - 0.268135847562926*G0_2_2;
    A[2626] = 0.17223324514991*G0_0_0 + 0.166035457919136*G0_0_1 + 0.100795592332048*G0_0_2 + 0.166035457919137*G0_1_0 + 0.17223324514991*G0_1_1 + 0.100795592332048*G0_1_2 + 0.19865539071268*G0_2_0 + 0.19865539071268*G0_2_1 + 0.113517366121531*G0_2_2;
    A[2627] = 0.10568858225108*G0_0_0 + 0.12787013655069*G0_0_1 + 0.0528442911255395*G0_0_2 + 0.12787013655069*G0_1_0 + 0.255740273101381*G0_1_1 + 0.0756782440810199*G0_1_2 + 0.105036183595209*G0_2_0 + 0.180062029020359*G0_2_1 + 0.10568858225108*G0_2_2;
    A[2628] = -0.1500516908503*G0_0_0 - 0.335985307773498*G0_0_1 - 0.466465038947673*G0_0_2 - 0.335985307773498*G0_1_0 - 0.150051690850299*G0_1_1 - 0.466465038947673*G0_1_2 - 0.466465038947673*G0_2_0 - 0.466465038947672*G0_2_1 - 0.782878387045045*G0_2_2;
    A[2629] = 0.143527704291591*G0_0_0 + 0.185933616923197*G0_0_1 + 0.218553549716741*G0_0_2 + 0.185933616923197*G0_1_0 + 0.176147637085134*G0_1_1 + 0.234863516113512*G0_1_2 + 0.218553549716742*G0_2_0 + 0.234863516113512*G0_2_1 + 0.293579395141892*G0_2_2;
    A[2630] = 0.176147637085133*G0_0_0 + 0.185933616923199*G0_0_1 + 0.234863516113513*G0_0_2 + 0.185933616923196*G0_1_0 + 0.14352770429159*G0_1_1 + 0.218553549716741*G0_1_2 + 0.234863516113511*G0_2_0 + 0.218553549716743*G0_2_1 + 0.293579395141891*G0_2_2;
    A[2631] = 0.0456679059109605*G0_0_0 + 0.0717638521457955*G0_0_1 + 0.120693751336112*G0_0_2 + 0.071763852145796*G0_1_0 + 0.0456679059109619*G0_1_1 + 0.120693751336113*G0_1_2 + 0.120693751336111*G0_2_0 + 0.120693751336111*G0_2_1 + 0.195719596761263*G0_2_2;
    A[2632] = -0.029847238506092*G0_0_0 - 0.029847238506092*G0_0_1 - 0.0298472385060921*G0_0_2 - 0.0298472385060921*G0_1_0 - 0.0298472385060921*G0_1_1 - 0.0298472385060921*G0_1_2 - 0.034571909571909*G0_2_0 - 0.034571909571909*G0_2_1 - 0.034571909571909*G0_2_2;
    A[2633] = -0.0298472385060925*G0_0_0 + 0.00472467106581676*G0_2_0;
    A[2634] = 0.0117431758056759*G0_1_1 + 0.0022781761063012*G0_2_1;
    A[2635] = -0.0094649996993747*G0_2_2;
    A[2636] = -0.00913358118219204*G0_1_1 - 0.00583896797004399*G0_1_2 - 0.00583896797004444*G0_2_1 - 0.00254435475789631*G0_2_2;
    A[2637] = -0.0156575677409007*G0_1_1 - 0.0130805930502106*G0_1_2 - 0.0130805930502108*G0_2_1 - 0.010503618359521*G0_2_2;
    A[2638] = -0.000652398655870717*G0_1_1 + 0.00192457603481865*G0_1_2 + 0.00192457603481878*G0_2_1 + 0.00450155072550872*G0_2_2;
    A[2639] = 0.00329461321214837*G0_1_2 + 0.00329461321214806*G0_2_1 + 0.00658922642429618*G0_2_2;
    A[2640] = 0.064261267603281*G0_0_0 + 0.0422754329004326*G0_0_2 + 0.0422754329004334*G0_2_0 + 0.0429604514890973*G0_2_2;
    A[2641] = 0.00880738185425659*G0_0_0 + 0.00649136662591527*G0_0_2 + 0.00649136662591483*G0_2_0 + 0.00352295274170293*G0_2_2;
    A[2642] = -0.000652398655870567*G0_0_0 - 0.00949240044292122*G0_0_2 - 0.00949240044292084*G0_2_0 - 0.0189848008858425*G0_2_2;
    A[2643] = 0.00329461321214809*G0_0_2 + 0.00329461321214808*G0_2_0 + 0.0292600797158091*G0_2_2;
    A[2644] = 0.128522535206562*G0_0_0 + 0.0422754329004326*G0_0_1 + 0.0422754329004328*G0_1_0 + 0.0845508658008656*G0_2_0 + 0.0429604514890974*G0_2_1;
    A[2645] = 0.03718672338464*G0_0_0 + 0.0743408268364863*G0_0_1 + 0.0743408268364861*G0_1_0 + 0.104709984267275*G0_1_1 + 0.0808321934624009*G0_2_0 + 0.108232937008978*G0_2_1;
    A[2646] = -0.00619778723077409*G0_0_0 + 0.018886941087461*G0_0_1 + 0.018886941087461*G0_1_0 + 0.0567586830607658*G0_1_1 + 0.00939454064454031*G0_2_0 + 0.0377738821749239*G0_2_1;
    A[2647] = -0.00583896797004434*G0_0_1 - 0.00583896797004401*G0_1_0 - 0.0435476102793809*G0_1_1 - 0.00254435475789632*G0_2_0 - 0.0142875305635725*G0_2_1;
    A[2648] = 0.0642612676032814*G0_0_0 + 0.0642612676032814*G0_0_1 + 0.0219858347028482*G0_0_2 + 0.0642612676032815*G0_1_0 + 0.0642612676032816*G0_1_1 + 0.0219858347028481*G0_1_2 + 0.0219858347028484*G0_2_0 + 0.0219858347028485*G0_2_1 + 0.0226708532915126*G0_2_2;
    A[2649] = 0.00880738185425672*G0_0_0 + 0.00880738185425683*G0_0_1 + 0.00231601522834151*G0_0_2 + 0.00880738185425634*G0_1_0 + 0.00880738185425631*G0_1_1 + 0.00231601522834185*G0_1_2 + 0.0023160152283416*G0_2_0 + 0.00231601522834168*G0_2_1 - 0.00065239865587062*G0_2_2;
    A[2650] = -0.000652398655870928*G0_0_0 - 0.000652398655870979*G0_0_1 + 0.00884000178705013*G0_0_2 - 0.00065239865587083*G0_1_0 - 0.000652398655870805*G0_1_1 + 0.00884000178704979*G0_1_2 + 0.00884000178704984*G0_2_0 + 0.00884000178704985*G0_2_1 - 0.000652398655871472*G0_2_2;
    A[2651] = -0.00329461321214789*G0_0_2 - 0.00329461321214752*G0_1_2 - 0.00329461321214802*G0_2_0 - 0.00329461321214797*G0_2_1 + 0.0226708532915132*G0_2_2;
    A[2652] = 0.128522535206562*G0_0_0 + 0.0862471023061286*G0_0_1 + 0.128522535206562*G0_0_2 + 0.0862471023061296*G0_1_0 + 0.0439716694056961*G0_1_1 + 0.0862471023061296*G0_1_2 + 0.0439716694056963*G0_2_0 + 0.0446566879943604*G0_2_1 + 0.0439716694056963*G0_2_2;
    A[2653] = 0.03718672338464*G0_0_0 - 0.0371541034518454*G0_0_1 + 0.0371867233846401*G0_0_2 - 0.0371541034518467*G0_1_0 - 0.00678494602105651*G0_1_1 - 0.0371541034518465*G0_1_2 - 0.0436454700777609*G0_2_0 - 0.00975335990526861*G0_2_1 - 0.0436454700777609*G0_2_2;
    A[2654] = -0.00619778723077399*G0_0_0 - 0.0250847283182357*G0_0_1 - 0.00619778723077392*G0_0_2 - 0.025084728318235*G0_1_0 + 0.0127870136550688*G0_1_1 - 0.025084728318235*G0_1_2 - 0.0155923278753147*G0_2_0 + 0.00329461321214724*G0_2_1 - 0.0155923278753146*G0_2_2;
    A[2655] = 0.00583896797004448*G0_0_1 + 0.0058389679700439*G0_1_0 - 0.0318696743392923*G0_1_1 + 0.00583896797004392*G0_1_2 + 0.00254435475789619*G0_2_0 - 0.00590420783563135*G0_2_1 + 0.00254435475789619*G0_2_2;
    A[2656] = 0.0920860702761732*G0_0_0 + 0.134361503176606*G0_0_1 + 0.134361503176606*G0_0_2 + 0.134361503176606*G0_1_0 + 0.176636936077039*G0_1_1 + 0.176636936077039*G0_1_2 + 0.128000616281865*G0_2_0 + 0.127315597693201*G0_2_1 + 0.127315597693201*G0_2_2;
    A[2657] = -0.0622388317700808*G0_0_0 - 0.250814663249557*G0_0_1 - 0.250814663249557*G0_0_2 - 0.250814663249556*G0_1_0 - 0.334680510461758*G0_1_1 - 0.334680510461758*G0_1_2 - 0.160783648739376*G0_2_0 - 0.166948816037357*G0_2_1 - 0.166948816037356*G0_2_2;
    A[2658] = -0.0622388317700814*G0_0_0 + 0.188575831479476*G0_0_1 + 0.188575831479476*G0_0_2 + 0.188575831479475*G0_1_0 + 0.104709984267276*G0_1_1 + 0.104709984267276*G0_1_2 + 0.0985448169692949*G0_2_0 + 0.182410664181496*G0_2_1 + 0.182410664181496*G0_2_2;
    A[2659] = 0.0920860702761739*G0_0_0 - 0.0422754329004324*G0_0_1 - 0.0422754329004324*G0_0_2 - 0.0422754329004325*G0_1_0 - 0.0359145460056911*G0_2_0 - 0.0429604514890972*G0_2_1 - 0.0429604514890974*G0_2_2;
    A[2660] = 0.0391439193522525*G0_0_0 + 0.0678494602105706*G0_0_1 + 0.135698920421142*G0_0_2 + 0.0678494602105703*G0_1_0 + 0.104709984267276*G0_1_2 + 0.135698920421141*G0_2_0 + 0.104709984267276*G0_2_1 + 0.209419968534551*G0_2_2;
    A[2661] = 0.019571959676124*G0_0_0 + 0.0606730749959897*G0_0_1 + 0.0748627457611808*G0_0_2 + 0.0606730749959903*G0_1_0 + 0.0929668084615984*G0_1_1 + 0.121346149991981*G0_1_2 + 0.0748627457611826*G0_2_0 + 0.121346149991982*G0_2_1 + 0.149725491522364*G0_2_2;
    A[2662] = -0.00554538857490221*G0_0_1 - 0.00619778723077379*G0_0_2 - 0.00554538857490167*G0_1_0 + 0.0391439193522529*G0_1_1 + 0.0264221455627698*G0_1_2 - 0.0061977872307727*G0_2_0 + 0.0264221455627712*G0_2_1 + 0.0137003717732886*G0_2_2;
    A[2663] = -0.00489298991903192*G0_0_0 + 0.0141896707651918*G0_0_1 + 0.0283793415303829*G0_0_2 + 0.0141896707651915*G0_1_0 + 0.0283793415303823*G0_1_2 + 0.0283793415303838*G0_2_0 + 0.0283793415303828*G0_2_1 + 0.0567586830607663*G0_2_2;
    A[2664] = -0.0102752788299674*G0_0_1 - 0.0181040627004167*G0_0_2 - 0.0102752788299664*G0_1_0 + 0.0195719596761255*G0_1_1 + 0.00146789697571018*G0_1_2 - 0.0181040627004174*G0_2_0 + 0.00146789697570916*G0_2_1 - 0.0166361657247068*G0_2_2;
    A[2665] = -0.000652398655871039*G0_0_1 - 0.00130479731174263*G0_0_2 - 0.000652398655870691*G0_1_0 - 0.0127217737894824*G0_1_2 - 0.00130479731174134*G0_2_0 - 0.0127217737894809*G0_2_1 - 0.0254435475789641*G0_2_2;
    A[2666] = 0.0391439193522533*G0_0_0 - 0.028705540858319*G0_0_1 - 0.0965550010688892*G0_0_2 - 0.0287055408583189*G0_1_0 - 0.0965550010688904*G0_1_1 - 0.0596944770121858*G0_1_2 - 0.0965550010688892*G0_2_0 - 0.0596944770121849*G0_2_1 - 0.0228339529554804*G0_2_2;
    A[2667] = 0.0195719596761252*G0_0_0 - 0.0411011153198651*G0_0_1 - 0.0552907860850563*G0_0_2 - 0.0411011153198663*G0_1_0 - 0.00880738185425641*G0_1_1 + 0.00538228891093508*G0_1_2 - 0.0552907860850584*G0_2_0 + 0.00538228891093423*G0_2_1 + 0.0195719596761254*G0_2_2;
    A[2668] = 0.00554538857490241*G0_0_1 + 0.00619778723077369*G0_0_2 + 0.00554538857490177*G0_1_0 + 0.0502346965020575*G0_1_1 + 0.0381653213684458*G0_1_2 + 0.0061977872307729*G0_2_0 + 0.0381653213684467*G0_2_1 + 0.026095946234836*G0_2_2;
    A[2669] = -0.00489298991903154*G0_0_0 - 0.019082660684223*G0_0_1 - 0.0332723314494145*G0_0_2 - 0.0190826606842223*G0_1_0 - 0.0332723314494134*G0_1_1 - 0.0190826606842218*G0_1_2 - 0.0332723314494153*G0_2_0 - 0.0190826606842235*G0_2_1 - 0.00489298991903122*G0_2_2;
    A[2670] = 0.0102752788299658*G0_0_1 + 0.0181040627004165*G0_0_2 + 0.0102752788299662*G0_1_0 + 0.0401225173360579*G0_1_1 + 0.0298472385060929*G0_1_2 + 0.0181040627004171*G0_2_0 + 0.0298472385060926*G0_2_1 + 0.0195719596761265*G0_2_2;
    A[2671] = 0.000652398655870987*G0_0_1 + 0.00130479731174189*G0_0_2 + 0.000652398655870594*G0_1_0 + 0.00130479731174139*G0_1_1 - 0.0107645778218704*G0_1_2 + 0.00130479731174102*G0_2_0 - 0.0107645778218688*G0_2_1 - 0.0228339529554807*G0_2_2;
    A[2672] = -0.292274597830152*G0_0_0 - 0.224425137619579*G0_0_1 - 0.09720739972476*G0_0_2 - 0.224425137619581*G0_1_0 - 0.15657567740901*G0_1_1 - 0.134067923781465*G0_1_2 - 0.0972073997247608*G0_2_0 - 0.134067923781465*G0_2_1 - 0.0880738185425683*G0_2_2;
    A[2673] = 0.45602666045374*G0_0_0 + 0.228013330226869*G0_0_1 + 0.22801333022687*G0_0_2 + 0.228013330226871*G0_1_0 + 0.0929668084615999*G0_1_1 + 0.363059851992141*G0_1_2 + 0.228013330226871*G0_2_0 + 0.36305985199214*G0_2_1 + 0.0929668084615997*G0_2_2;
    A[2674] = -0.292274597830151*G0_0_0 - 0.0678494602105706*G0_0_1 - 0.195067198105391*G0_0_2 - 0.0678494602105721*G0_1_0 - 0.104709984267275*G0_1_2 - 0.195067198105392*G0_2_0 - 0.104709984267276*G0_2_1 - 0.185933616923198*G0_2_2;
    A[2675] = -0.00880738185425664*G0_0_0 + 0.00538228891093441*G0_0_1 - 0.0411011153198651*G0_0_2 + 0.00538228891093508*G0_1_0 + 0.0195719596761263*G0_1_1 - 0.0552907860850559*G0_1_2 - 0.0411011153198651*G0_2_0 - 0.055290786085057*G0_2_1 + 0.0195719596761267*G0_2_2;
    A[2676] = -0.00880738185425671*G0_0_0 - 0.0141896707651916*G0_0_1 + 0.0322937334656082*G0_0_2 - 0.0141896707651911*G0_1_0 - 0.0283793415303833*G0_1_2 + 0.0322937334656087*G0_2_0 - 0.0283793415303826*G0_2_1 + 0.0929668084615992*G0_2_2;
    A[2677] = 0.00130479731174142*G0_0_0 + 0.000652398655871102*G0_0_1 + 0.000652398655870965*G0_0_2 + 0.000652398655870481*G0_1_0 + 0.0127217737894816*G0_1_2 + 0.000652398655870943*G0_2_0 + 0.0127217737894808*G0_2_1 - 0.088073818542568*G0_2_2;
    A[2678] = -0.584549195660301*G0_0_0 - 0.32163253734434*G0_0_1 - 0.448850275239159*G0_0_2 - 0.32163253734434*G0_1_0 - 0.268135847562929*G0_1_1 - 0.290643601190473*G0_1_2 - 0.19441479944952*G0_2_0 - 0.222141742324032*G0_2_1 - 0.268135847562929*G0_2_2;
    A[2679] = 0.91205332090748*G0_0_0 + 0.456026660453741*G0_0_1 + 0.45602666045374*G0_0_2 + 0.456026660453741*G0_1_0 + 0.726119703984282*G0_1_1 + 0.456026660453741*G0_1_2 + 0.45602666045374*G0_2_0 + 0.456026660453741*G0_2_1 + 0.726119703984281*G0_2_2;
    A[2680] = -0.584549195660302*G0_0_0 - 0.262916658315963*G0_0_1 - 0.135698920421142*G0_0_2 - 0.262916658315963*G0_1_0 - 0.209419968534551*G0_1_1 - 0.104709984267276*G0_1_2 - 0.390134396210783*G0_2_0 - 0.290643601190474*G0_2_1 - 0.209419968534552*G0_2_2;
    A[2681] = -0.0371867233846409*G0_0_0 + 0.186912214907003*G0_0_1 + 0.0376760223765412*G0_0_2 + 0.186912214907004*G0_1_0 - 0.0371867233846407*G0_1_1 + 0.140428810676203*G0_1_2 + 0.140428810676204*G0_2_0 + 0.0376760223765413*G0_2_1 + 0.0655660649150205*G0_2_2;
    A[2682] = -0.0371867233846395*G0_0_0 - 0.224098938291644*G0_0_1 - 0.0748627457611809*G0_0_2 - 0.224098938291644*G0_1_0 - 0.448197876583289*G0_1_1 - 0.121346149991981*G0_1_2 - 0.177615534060844*G0_2_0 - 0.326851726591307*G0_2_1 - 0.149725491522364*G0_2_2;
    A[2683] = 0.0123955744615478*G0_0_0 + 0.00619778723077414*G0_0_1 + 0.00619778723077358*G0_0_2 + 0.00619778723077388*G0_1_0 + 0.172233245149911*G0_1_1 - 0.0264221455627701*G0_1_2 + 0.00619778723077445*G0_2_0 + 0.0714376528178619*G0_2_1 - 0.0137003717732887*G0_2_2;
    A[2684] = -0.0391439193522533*G0_0_0 + 0.249542485870608*G0_0_1 + 0.44526208263187*G0_0_2 + 0.249542485870609*G0_1_0 + 0.35229527417027*G0_1_1 + 0.249542485870608*G0_1_2 + 0.445262082631869*G0_2_0 + 0.249542485870607*G0_2_1 + 0.146789697570944*G0_2_2;
    A[2685] = -0.039143919352252*G0_0_0 - 0.288686405222859*G0_0_1 - 0.484406001984122*G0_0_2 - 0.28868640522286*G0_1_0 - 0.185933616923198*G0_1_1 - 0.484406001984123*G0_1_2 - 0.48440600198412*G0_2_0 - 0.484406001984122*G0_2_1 - 0.782878387045047*G0_2_2;
    A[2686] = -0.0391439193522496*G0_0_0 - 0.0195719596761264*G0_0_1 - 0.0195719596761249*G0_0_2 - 0.0195719596761243*G0_1_0 - 0.234863516113513*G0_1_1 - 0.141896707651914*G0_1_2 - 0.0195719596761246*G0_2_0 - 0.141896707651913*G0_2_1 - 0.0489298991903144*G0_2_2;
    A[2687] = 0.00978597983806363*G0_0_0 + 0.0048929899190326*G0_0_1 + 0.00489298991903173*G0_0_2 + 0.00489298991903077*G0_1_0 - 0.0391439193522523*G0_1_1 + 0.0538228891093477*G0_1_2 + 0.00489298991903156*G0_2_0 + 0.0538228891093476*G0_2_1 + 0.146789697570948*G0_2_2;
    A[2688] = 0.018919561020255*G0_0_0 + 0.018919561020255*G0_0_1 + 0.0189195610202551*G0_0_2 + 0.018919561020255*G0_1_0 + 0.018919561020255*G0_1_1 + 0.018919561020255*G0_1_2 + 0.0137421252872641*G0_2_0 + 0.0137421252872642*G0_2_1 + 0.0137421252872642*G0_2_2;
    A[2689] = 0.03359853077735*G0_0_0 - 0.0143945239431347*G0_2_0;
    A[2690] = 0.0189195610202554*G0_1_1 + 0.00517743573299104*G0_2_1;
    A[2691] = -0.0137421252872641*G0_2_2;
    A[2692] = -0.0306627368259308*G0_1_1 - 0.0212681961813904*G0_1_2 - 0.0212681961813904*G0_2_1 - 0.0118736555368501*G0_2_2;
    A[2693] = -0.0013047973117417*G0_1_1 - 0.00123955744615445*G0_1_2 - 0.00123955744615449*G0_2_1 - 0.00117431758056762*G0_2_2;
    A[2694] = -0.0150051690850299*G0_1_1 - 0.0149399292194431*G0_1_2 - 0.0149399292194428*G0_2_1 - 0.0148746893538554*G0_2_2;
    A[2695] = 0.00939454064454077*G0_1_2 + 0.00939454064454025*G0_2_1 + 0.018789081289081*G0_2_2;
    A[2696] = -0.128522535206561*G0_0_0 - 0.0962940416065403*G0_0_2 - 0.0962940416065403*G0_2_0 - 0.12473862300251*G0_2_2;
    A[2697] = 0.0476251018785737*G0_0_0 + 0.0623693115012552*G0_0_2 + 0.0623693115012552*G0_2_0 + 0.124738623002511*G0_2_2;
    A[2698] = -0.0150051690850302*G0_0_0 - 0.0377738821749235*G0_0_2 - 0.0377738821749231*G0_2_0 - 0.127087258163646*G0_2_2;
    A[2699] = 0.0093945406445404*G0_0_2 + 0.00939454064454023*G0_2_0 + 0.0641307878721069*G0_2_2;
    A[2700] = -0.257045070413123*G0_0_0 - 0.0962940416065405*G0_0_1 - 0.0962940416065414*G0_1_0 - 0.192588083213081*G0_2_0 - 0.12473862300251*G0_2_1;
    A[2701] = 0.121346149991981*G0_0_0 - 0.0733296089198867*G0_0_1 - 0.0733296089198854*G0_1_0 - 0.271397840842283*G0_1_1 - 0.010960297418631*G0_2_0 - 0.146659217839772*G0_2_1;
    A[2702] = -0.0528442911255397*G0_0_0 + 0.0359471659384853*G0_0_1 + 0.0359471659384845*G0_1_0 + 0.20941996853455*G0_1_1 - 0.00182671623643819*G0_2_0 + 0.082332710370904*G0_2_1;
    A[2703] = -0.0212681961813905*G0_0_1 - 0.0212681961813904*G0_1_0 - 0.111886369481855*G0_1_1 - 0.01187365553685*G0_2_0 - 0.0477555816097476*G0_2_1;
    A[2704] = -0.0306627368259313*G0_0_0 - 0.0306627368259312*G0_0_1 - 0.00939454064454013*G0_0_2 - 0.0306627368259312*G0_1_0 - 0.0306627368259311*G0_1_1 - 0.00939454064454003*G0_1_2 - 0.00939454064454026*G0_2_0 - 0.00939454064454029*G0_2_1;
    A[2705] = -0.0013047973117415*G0_0_0 - 0.00130479731174174*G0_0_1 - 6.52398655873191e-05*G0_0_2 - 0.00130479731174184*G0_1_0 - 0.00130479731174185*G0_1_1 - 6.52398655874507e-05*G0_1_2 - 6.52398655868569e-05*G0_2_0 - 6.52398655867167e-05*G0_2_1;
    A[2706] = -0.0150051690850301*G0_0_0 - 0.01500516908503*G0_0_1 - 6.52398655868374e-05*G0_0_2 - 0.0150051690850298*G0_1_0 - 0.0150051690850299*G0_1_1 - 6.52398655866687e-05*G0_1_2 - 6.52398655872633e-05*G0_2_0 - 6.5239865587443e-05*G0_2_1;
    A[2707] = -0.0093945406445406*G0_0_2 - 0.00939454064454078*G0_1_2 - 0.00939454064454026*G0_2_0 - 0.00939454064454027*G0_2_1;
    A[2708] = -0.0613254736518621*G0_0_0 - 0.040057277470471*G0_0_1 - 0.0613254736518621*G0_0_2 - 0.0400572774704716*G0_1_0 - 0.0187890812890806*G0_1_1 - 0.0400572774704716*G0_1_2 - 0.0187890812890812*G0_2_0 - 0.00939454064454076*G0_2_1 - 0.0187890812890811*G0_2_2;
    A[2709] = -0.025443547578964*G0_0_0 + 0.0136351319077008*G0_0_1 - 0.0254435475789641*G0_0_2 + 0.013635131907701*G0_1_0 - 0.000130479731174878*G0_1_1 + 0.013635131907701*G0_1_2 + 0.0148746893538556*G0_2_0 - 6.52398655867473e-05*G0_2_1 + 0.0148746893538555*G0_2_2;
    A[2710] = -0.0528442911255403*G0_0_0 - 0.0137656116388752*G0_0_1 - 0.0528442911255404*G0_0_2 - 0.0137656116388752*G0_1_0 - 0.000130479731173502*G0_1_1 - 0.0137656116388754*G0_1_2 + 0.0011743175805672*G0_2_0 - 6.52398655877235e-05*G0_2_1 + 0.00117431758056725*G0_2_2;
    A[2711] = 0.0212681961813905*G0_0_1 + 0.0212681961813904*G0_1_0 - 0.0187890812890813*G0_1_1 + 0.0212681961813905*G0_1_2 + 0.0118736555368501*G0_2_0 - 0.00939454064453986*G0_2_1 + 0.0118736555368501*G0_2_2;
    A[2712] = -0.0693499771190734*G0_0_0 - 0.0906181733004642*G0_0_1 - 0.0906181733004642*G0_0_2 - 0.0906181733004635*G0_1_0 - 0.111886369481854*G0_1_1 - 0.111886369481854*G0_1_2 - 0.0547362472275663*G0_2_0 - 0.0641307878721066*G0_2_1 - 0.0641307878721065*G0_2_2;
    A[2713] = 0.0846813455320388*G0_0_0 + 0.173472802596065*G0_0_1 + 0.173472802596065*G0_0_2 + 0.173472802596064*G0_1_0 + 0.20941996853455*G0_1_1 + 0.20941996853455*G0_1_2 + 0.0893133759887225*G0_2_0 + 0.127087258163645*G0_2_1 + 0.127087258163645*G0_2_2;
    A[2714] = -0.00339247301052821*G0_0_0 - 0.198068231922397*G0_0_1 - 0.198068231922397*G0_0_2 - 0.198068231922396*G0_1_0 - 0.271397840842283*G0_1_1 - 0.271397840842283*G0_1_2 - 0.0623693115012554*G0_2_0 - 0.124738623002511*G0_2_1 - 0.124738623002511*G0_2_2;
    A[2715] = -0.0644569872000424*G0_0_0 + 0.0962940416065403*G0_0_1 + 0.0962940416065403*G0_0_2 + 0.0962940416065401*G0_1_0 + 0.0284445813959698*G0_2_0 + 0.12473862300251*G0_2_1 + 0.12473862300251*G0_2_2;
    A[2716] = 0.0521918924696694*G0_0_0 - 0.135698920421141*G0_0_1 - 0.271397840842282*G0_0_2 - 0.13569892042114*G0_1_0 - 0.271397840842283*G0_1_2 - 0.271397840842281*G0_2_0 - 0.271397840842281*G0_2_1 - 0.542795681684564*G0_2_2;
    A[2717] = -0.0880738185425669*G0_0_0 + 0.0091335811821916*G0_0_1 + 0.045994105238896*G0_0_2 + 0.00913358118219171*G0_1_0 - 0.185933616923198*G0_1_1 - 0.0812236326559235*G0_1_2 + 0.0459941052388959*G0_2_0 - 0.0812236326559229*G0_2_1 + 0.0234863516113506*G0_2_2;
    A[2718] = -0.0378391220405103*G0_0_1 - 0.0528442911255403*G0_0_2 - 0.0378391220405105*G0_1_0 + 0.0521918924696693*G0_1_1 - 0.000652398655871141*G0_1_2 - 0.0528442911255406*G0_2_0 - 0.000652398655871359*G0_2_1 - 0.0534966897814108*G0_2_2;
    A[2719] = -0.0228339529554811*G0_0_0 + 0.0368605240567035*G0_0_1 + 0.073721048113408*G0_0_2 + 0.0368605240567038*G0_1_0 + 0.104709984267276*G0_1_2 + 0.0737210481134086*G0_2_0 + 0.104709984267275*G0_2_1 + 0.209419968534551*G0_2_2;
    A[2720] = -0.0120693751336103*G0_0_1 - 0.0127217737894813*G0_0_2 - 0.0120693751336109*G0_1_0 + 0.0260959462348349*G0_1_1 + 0.0198981590040616*G0_1_2 - 0.0127217737894814*G0_2_0 + 0.0198981590040619*G0_2_1 + 0.0137003717732883*G0_2_2;
    A[2721] = -0.0150051690850301*G0_0_1 - 0.0300103381700601*G0_0_2 - 0.0150051690850297*G0_1_0 - 0.0528442911255408*G0_1_2 - 0.0300103381700599*G0_2_0 - 0.0528442911255401*G0_2_1 - 0.105688582251081*G0_2_2;
    A[2722] = -0.0456679059109619*G0_0_0 - 0.00782878387045068*G0_0_1 + 0.0300103381700599*G0_0_2 - 0.00782878387045032*G0_1_0 + 0.0300103381700606*G0_1_1 + 0.0150051690850301*G0_1_2 + 0.0300103381700599*G0_2_0 + 0.0150051690850288*G0_2_1;
    A[2723] = -0.0880738185425672*G0_0_0 + 0.000652398655871544*G0_0_1 + 0.0127217737894823*G0_0_2 + 0.00065239865587114*G0_1_0 + 0.00130479731174116*G0_1_1 + 0.000652398655871238*G0_1_2 + 0.012721773789482*G0_2_0 + 0.0006523986558724*G0_2_1;
    A[2724] = 0.0378391220405108*G0_0_1 + 0.0528442911255401*G0_0_2 + 0.0378391220405105*G0_1_0 + 0.0300103381700604*G0_1_1 + 0.0150051690850295*G0_1_2 + 0.0528442911255405*G0_2_0 + 0.0150051690850293*G0_2_1;
    A[2725] = -0.02283395295548*G0_0_0 - 0.0107645778218692*G0_0_1 + 0.00130479731174184*G0_0_2 - 0.0107645778218686*G0_1_0 + 0.00130479731174223*G0_1_1 + 0.000652398655870583*G0_1_2 + 0.00130479731174166*G0_2_0 + 0.000652398655870323*G0_2_1;
    A[2726] = 0.0120693751336109*G0_0_1 + 0.012721773789482*G0_0_2 + 0.012069375133611*G0_1_0 + 0.00130479731174124*G0_1_1 + 0.000652398655870891*G0_1_2 + 0.0127217737894816*G0_2_0 + 0.00065239865587086*G0_2_1;
    A[2727] = 0.0150051690850298*G0_0_1 + 0.0300103381700599*G0_0_2 + 0.0150051690850298*G0_1_0 + 0.0300103381700599*G0_1_1 + 0.0150051690850299*G0_1_2 + 0.03001033817006*G0_2_0 + 0.0150051690850303*G0_2_1;
    A[2728] = 0.127870136550692*G0_0_0 + 0.0900310145101806*G0_0_1 + 0.037839122040511*G0_0_2 + 0.0900310145101812*G0_1_0 + 0.05219189246967*G0_1_1 + 0.0528442911255408*G0_1_2 + 0.0378391220405106*G0_2_0 + 0.0528442911255413*G0_2_1;
    A[2729] = -0.292274597830152*G0_0_0 - 0.195067198105391*G0_0_1 - 0.0678494602105713*G0_0_2 - 0.195067198105391*G0_1_0 - 0.185933616923198*G0_1_1 - 0.104709984267275*G0_1_2 - 0.0678494602105715*G0_2_0 - 0.104709984267277*G0_2_1;
    A[2730] = 0.323589733311953*G0_0_0 + 0.135698920421141*G0_0_1 + 0.135698920421142*G0_0_2 + 0.135698920421142*G0_1_0 + 0.271397840842283*G0_1_2 + 0.135698920421142*G0_2_0 + 0.271397840842281*G0_2_1;
    A[2731] = 0.050234696502057*G0_0_0 + 0.0381653213684466*G0_0_1 + 0.00554538857490237*G0_0_2 + 0.0381653213684459*G0_1_0 + 0.0260959462348354*G0_1_1 + 0.00619778723077303*G0_1_2 + 0.00554538857490228*G0_2_0 + 0.00619778723077373*G0_2_1;
    A[2732] = -0.096555001068889*G0_0_0 - 0.0368605240567038*G0_0_1 - 0.0678494602105707*G0_0_2 - 0.0368605240567048*G0_1_0 - 0.104709984267276*G0_1_2 - 0.0678494602105706*G0_2_0 - 0.104709984267275*G0_2_1;
    A[2733] = 0.0300103381700604*G0_0_0 + 0.0150051690850301*G0_0_1 + 0.0378391220405106*G0_0_2 + 0.0150051690850303*G0_1_0 + 0.0528442911255409*G0_1_2 + 0.0378391220405105*G0_2_0 + 0.0528442911255402*G0_2_1;
    A[2734] = 0.255740273101382*G0_0_0 + 0.127870136550691*G0_0_1 + 0.180062029020361*G0_0_2 + 0.127870136550692*G0_1_0 + 0.105688582251082*G0_1_1 + 0.105036183595211*G0_1_2 + 0.0756782440810212*G0_2_0 + 0.0528442911255402*G0_2_1 + 0.105688582251081*G0_2_2;
    A[2735] = -0.584549195660302*G0_0_0 - 0.262916658315963*G0_0_1 - 0.390134396210783*G0_0_2 - 0.262916658315963*G0_1_0 - 0.209419968534551*G0_1_1 - 0.290643601190474*G0_1_2 - 0.135698920421142*G0_2_0 - 0.104709984267276*G0_2_1 - 0.209419968534552*G0_2_2;
    A[2736] = 0.647179466623905*G0_0_0 + 0.271397840842284*G0_0_1 + 0.271397840842283*G0_0_2 + 0.271397840842284*G0_1_0 + 0.542795681684566*G0_1_1 + 0.271397840842283*G0_1_2 + 0.271397840842283*G0_2_0 + 0.271397840842283*G0_2_1 + 0.542795681684565*G0_2_2;
    A[2737] = 0.17223324514991*G0_0_0 + 0.00619778723077415*G0_0_1 + 0.0714376528178609*G0_0_2 + 0.00619778723077353*G0_1_0 + 0.0123955744615459*G0_1_1 + 0.00619778723077313*G0_1_2 - 0.0264221455627697*G0_2_0 + 0.00619778723077422*G0_2_1 - 0.0137003717732885*G0_2_2;
    A[2738] = -0.268135847562927*G0_0_0 + 0.0534966897814116*G0_0_1 - 0.0459941052388961*G0_0_2 + 0.053496689781411*G0_1_0 - 0.209419968534549*G0_1_1 + 0.0812236326559232*G0_1_2 + 0.0225077536275451*G0_2_0 - 0.104709984267274*G0_2_1 - 0.0234863516113506*G0_2_2;
    A[2739] = 0.10568858225108*G0_0_0 - 0.02218155429961*G0_0_1 + 0.0528442911255404*G0_0_2 - 0.0221815542996092*G0_1_0 + 0.105688582251082*G0_1_1 + 0.000652398655871392*G0_1_2 + 0.000652398655871002*G0_2_0 + 0.0528442911255402*G0_2_1 + 0.053496689781411*G0_2_2;
    A[2740] = 0.143527704291593*G0_0_0 - 0.042405912631607*G0_0_1 - 0.0750258454251495*G0_0_2 - 0.0424059126316063*G0_1_0 - 0.0521918924696695*G0_1_1 - 0.026095946234834*G0_1_2 - 0.0750258454251521*G0_2_0 - 0.0260959462348368*G0_2_1;
    A[2741] = -0.1500516908503*G0_0_0 + 0.185933616923199*G0_0_1 + 0.316413348097373*G0_0_2 + 0.185933616923196*G0_1_0 + 0.371867233846396*G0_1_1 + 0.185933616923198*G0_1_2 + 0.316413348097372*G0_2_0 + 0.185933616923199*G0_2_1;
    A[2742] = 0.176147637085134*G0_0_0 - 0.0097859798380631*G0_0_1 - 0.0587158790283793*G0_0_2 - 0.00978597983806256*G0_1_0 - 0.0521918924696702*G0_1_1 - 0.0260959462348342*G0_1_2 - 0.0587158790283778*G0_2_0 - 0.0260959462348325*G0_2_1;
    A[2743] = 0.0456679059109609*G0_0_0 - 0.0260959462348361*G0_0_1 - 0.0750258454251507*G0_0_2 - 0.0260959462348355*G0_1_0 - 0.0521918924696701*G0_1_1 - 0.0260959462348359*G0_1_2 - 0.0750258454251506*G0_2_0 - 0.0260959462348356*G0_2_1;
    A[2744] = -0.0298472385060923*G0_0_0 - 0.0298472385060923*G0_0_1 - 0.0298472385060923*G0_0_2 - 0.0298472385060922*G0_1_0 - 0.0298472385060923*G0_1_1 - 0.0298472385060923*G0_1_2 - 0.0345719095719092*G0_2_0 - 0.0345719095719093*G0_2_1 - 0.0345719095719092*G0_2_2;
    A[2745] = 0.0117431758056756*G0_0_0 + 0.00227817610630104*G0_2_0;
    A[2746] = -0.0298472385060927*G0_1_1 + 0.00472467106581664*G0_2_1;
    A[2747] = -0.00946499969937468*G0_2_2;
    A[2748] = 0.0642612676032812*G0_1_1 + 0.0422754329004324*G0_1_2 + 0.0422754329004328*G0_2_1 + 0.0429604514890965*G0_2_2;
    A[2749] = 0.0088073818542571*G0_1_1 + 0.00649136662591578*G0_1_2 + 0.00649136662591503*G0_2_1 + 0.0035229527417038*G0_2_2;
    A[2750] = -0.000652398655871191*G0_1_1 - 0.0094924004429211*G0_1_2 - 0.00949240044292086*G0_2_1 - 0.0189848008858429*G0_2_2;
    A[2751] = 0.00329461321214769*G0_1_2 + 0.00329461321214784*G0_2_1 + 0.029260079715809*G0_2_2;
    A[2752] = -0.00913358118219201*G0_0_0 - 0.00583896797004447*G0_0_2 - 0.00583896797004412*G0_2_0 - 0.00254435475789649*G0_2_2;
    A[2753] = -0.0156575677409007*G0_0_0 - 0.0130805930502103*G0_0_2 - 0.0130805930502105*G0_2_0 - 0.0105036183595202*G0_2_2;
    A[2754] = -0.000652398655870683*G0_0_0 + 0.00192457603481892*G0_0_2 + 0.00192457603481923*G0_2_0 + 0.00450155072550878*G0_2_2;
    A[2755] = 0.00329461321214812*G0_0_2 + 0.00329461321214781*G0_2_0 + 0.00658922642429607*G0_2_2;
    A[2756] = -0.0435476102793807*G0_0_0 - 0.00583896797004445*G0_0_1 - 0.00583896797004495*G0_1_0 - 0.014287530563572*G0_2_0 - 0.00254435475789644*G0_2_1;
    A[2757] = 0.0567586830607654*G0_0_0 + 0.0188869410874614*G0_0_1 + 0.0188869410874619*G0_1_0 - 0.00619778723077302*G0_1_1 + 0.0377738821749228*G0_2_0 + 0.00939454064454036*G0_2_1;
    A[2758] = 0.104709984267275*G0_0_0 + 0.0743408268364859*G0_0_1 + 0.0743408268364853*G0_1_0 + 0.0371867233846391*G0_1_1 + 0.108232937008978*G0_2_0 + 0.0808321934624006*G0_2_1;
    A[2759] = 0.0422754329004323*G0_0_1 + 0.0422754329004325*G0_1_0 + 0.128522535206562*G0_1_1 + 0.0429604514890965*G0_2_0 + 0.0845508658008649*G0_2_1;
    A[2760] = 0.064261267603281*G0_0_0 + 0.0642612676032811*G0_0_1 + 0.0219858347028485*G0_0_2 + 0.0642612676032806*G0_1_0 + 0.0642612676032805*G0_1_1 + 0.0219858347028483*G0_1_2 + 0.021985834702848*G0_2_0 + 0.021985834702848*G0_2_1 + 0.0226708532915129*G0_2_2;
    A[2761] = 0.00880738185425689*G0_0_0 + 0.00880738185425682*G0_0_1 + 0.00231601522834159*G0_0_2 + 0.00880738185425743*G0_1_0 + 0.0088073818542575*G0_1_1 + 0.00231601522834202*G0_1_2 + 0.00231601522834183*G0_2_0 + 0.00231601522834179*G0_2_1 - 0.000652398655870843*G0_2_2;
    A[2762] = -0.000652398655870984*G0_0_0 - 0.000652398655870904*G0_0_1 + 0.00884000178704997*G0_0_2 - 0.000652398655871111*G0_1_0 - 0.00065239865587113*G0_1_1 + 0.00884000178704963*G0_1_2 + 0.00884000178704998*G0_2_0 + 0.00884000178705*G0_2_1 - 0.000652398655871218*G0_2_2;
    A[2763] = -0.00329461321214771*G0_0_2 - 0.00329461321214785*G0_1_2 - 0.00329461321214781*G0_2_0 - 0.00329461321214787*G0_2_1 + 0.0226708532915131*G0_2_2;
    A[2764] = 0.176636936077038*G0_0_0 + 0.134361503176606*G0_0_1 + 0.176636936077038*G0_0_2 + 0.134361503176605*G0_1_0 + 0.0920860702761732*G0_1_1 + 0.134361503176605*G0_1_2 + 0.1273155976932*G0_2_0 + 0.128000616281865*G0_2_1 + 0.1273155976932*G0_2_2;
    A[2765] = -0.334680510461757*G0_0_0 - 0.250814663249557*G0_0_1 - 0.334680510461757*G0_0_2 - 0.250814663249556*G0_1_0 - 0.0622388317700808*G0_1_1 - 0.250814663249556*G0_1_2 - 0.166948816037355*G0_2_0 - 0.160783648739377*G0_2_1 - 0.166948816037355*G0_2_2;
    A[2766] = 0.104709984267275*G0_0_0 + 0.188575831479476*G0_0_1 + 0.104709984267275*G0_0_2 + 0.188575831479475*G0_1_0 - 0.0622388317700817*G0_1_1 + 0.188575831479475*G0_1_2 + 0.182410664181495*G0_2_0 + 0.0985448169692955*G0_2_1 + 0.182410664181495*G0_2_2;
    A[2767] = -0.0422754329004326*G0_0_1 - 0.0422754329004326*G0_1_0 + 0.092086070276174*G0_1_1 - 0.0422754329004327*G0_1_2 - 0.0429604514890966*G0_2_0 - 0.0359145460056914*G0_2_1 - 0.0429604514890966*G0_2_2;
    A[2768] = 0.0439716694056965*G0_0_0 + 0.0862471023061292*G0_0_1 + 0.0862471023061292*G0_0_2 + 0.0862471023061292*G0_1_0 + 0.128522535206562*G0_1_1 + 0.128522535206562*G0_1_2 + 0.0446566879943612*G0_2_0 + 0.0439716694056962*G0_2_1 + 0.0439716694056963*G0_2_2;
    A[2769] = -0.00678494602105702*G0_0_0 - 0.0371541034518464*G0_0_1 - 0.0371541034518463*G0_0_2 - 0.0371541034518466*G0_1_0 + 0.03718672338464*G0_1_1 + 0.0371867233846398*G0_1_2 - 0.00975335990526975*G0_2_0 - 0.0436454700777612*G0_2_1 - 0.0436454700777611*G0_2_2;
    A[2770] = 0.0127870136550691*G0_0_0 - 0.0250847283182351*G0_0_1 - 0.0250847283182351*G0_0_2 - 0.0250847283182344*G0_1_0 - 0.00619778723077332*G0_1_1 - 0.00619778723077327*G0_1_2 + 0.00329461321214829*G0_2_0 - 0.0155923278753137*G0_2_1 - 0.0155923278753138*G0_2_2;
    A[2771] = -0.0318696743392919*G0_0_0 + 0.00583896797004465*G0_0_1 + 0.00583896797004462*G0_0_2 + 0.00583896797004427*G0_1_0 - 0.0059042078356315*G0_2_0 + 0.00254435475789663*G0_2_1 + 0.00254435475789664*G0_2_2;
    A[2772] = 0.0391439193522515*G0_0_0 - 0.00554538857490254*G0_0_1 + 0.0264221455627694*G0_0_2 - 0.00554538857490279*G0_1_0 - 0.00619778723077465*G0_1_2 + 0.0264221455627707*G0_2_0 - 0.00619778723077275*G0_2_1 + 0.0137003717732879*G0_2_2;
    A[2773] = 0.0929668084615984*G0_0_0 + 0.0606730749959897*G0_0_1 + 0.121346149991981*G0_0_2 + 0.0606730749959887*G0_1_0 + 0.0195719596761246*G0_1_1 + 0.0748627457611814*G0_1_2 + 0.121346149991981*G0_2_0 + 0.0748627457611818*G0_2_1 + 0.149725491522365*G0_2_2;
    A[2774] = 0.0678494602105702*G0_0_1 + 0.104709984267274*G0_0_2 + 0.0678494602105709*G0_1_0 + 0.039143919352251*G0_1_1 + 0.13569892042114*G0_1_2 + 0.104709984267275*G0_2_0 + 0.13569892042114*G0_2_1 + 0.209419968534549*G0_2_2;
    A[2775] = 0.0195719596761264*G0_0_0 - 0.0102752788299656*G0_0_1 + 0.00146789697570942*G0_0_2 - 0.0102752788299658*G0_1_0 - 0.0181040627004168*G0_1_2 + 0.00146789697570992*G0_2_0 - 0.0181040627004157*G0_2_1 - 0.0166361657247067*G0_2_2;
    A[2776] = 0.0141896707651917*G0_0_1 + 0.028379341530383*G0_0_2 + 0.0141896707651923*G0_1_0 - 0.00489298991903103*G0_1_1 + 0.0283793415303825*G0_1_2 + 0.0283793415303831*G0_2_0 + 0.0283793415303831*G0_2_1 + 0.0567586830607657*G0_2_2;
    A[2777] = -0.000652398655870804*G0_0_1 - 0.0127217737894821*G0_0_2 - 0.000652398655871401*G0_1_0 - 0.00130479731174116*G0_1_2 - 0.0127217737894813*G0_2_0 - 0.00130479731174112*G0_2_1 - 0.0254435475789632*G0_2_2;
    A[2778] = -0.156575677409008*G0_0_0 - 0.22442513761958*G0_0_1 - 0.134067923781464*G0_0_2 - 0.224425137619578*G0_1_0 - 0.29227459783015*G0_1_1 - 0.0972073997247591*G0_1_2 - 0.134067923781463*G0_2_0 - 0.0972073997247588*G0_2_1 - 0.0880738185425673*G0_2_2;
    A[2779] = 0.0929668084615978*G0_0_0 + 0.228013330226869*G0_0_1 + 0.363059851992139*G0_0_2 + 0.228013330226869*G0_1_0 + 0.45602666045374*G0_1_1 + 0.228013330226869*G0_1_2 + 0.363059851992139*G0_2_0 + 0.228013330226869*G0_2_1 + 0.092966808461599*G0_2_2;
    A[2780] = -0.0678494602105705*G0_0_1 - 0.104709984267275*G0_0_2 - 0.067849460210571*G0_1_0 - 0.292274597830151*G0_1_1 - 0.195067198105391*G0_1_2 - 0.104709984267275*G0_2_0 - 0.195067198105391*G0_2_1 - 0.185933616923199*G0_2_2;
    A[2781] = 0.019571959676126*G0_0_0 + 0.00538228891093423*G0_0_1 - 0.0552907860850562*G0_0_2 + 0.00538228891093442*G0_1_0 - 0.00880738185425819*G0_1_1 - 0.0411011153198659*G0_1_2 - 0.0552907860850571*G0_2_0 - 0.0411011153198658*G0_2_1 + 0.0195719596761255*G0_2_2;
    A[2782] = -0.0141896707651907*G0_0_1 - 0.0283793415303822*G0_0_2 - 0.0141896707651924*G0_1_0 - 0.00880738185425631*G0_1_1 + 0.0322937334656082*G0_1_2 - 0.0283793415303833*G0_2_0 + 0.032293733465609*G0_2_1 + 0.0929668084615997*G0_2_2;
    A[2783] = 0.000652398655870389*G0_0_1 + 0.0127217737894815*G0_0_2 + 0.000652398655871233*G0_1_0 + 0.00130479731174199*G0_1_1 + 0.000652398655871427*G0_1_2 + 0.0127217737894811*G0_2_0 + 0.000652398655870609*G0_2_1 - 0.0880738185425676*G0_2_2;
    A[2784] = -0.0965550010688891*G0_0_0 - 0.0287055408583178*G0_0_1 - 0.0596944770121848*G0_0_2 - 0.0287055408583174*G0_1_0 + 0.0391439193522539*G0_1_1 - 0.0965550010688888*G0_1_2 - 0.0596944770121839*G0_2_0 - 0.0965550010688883*G0_2_1 - 0.0228339529554799*G0_2_2;
    A[2785] = -0.0088073818542567*G0_0_0 - 0.0411011153198649*G0_0_1 + 0.00538228891093564*G0_0_2 - 0.0411011153198659*G0_1_0 + 0.0195719596761256*G0_1_1 - 0.0552907860850563*G0_1_2 + 0.00538228891093405*G0_2_0 - 0.0552907860850567*G0_2_1 + 0.0195719596761261*G0_2_2;
    A[2786] = 0.0502346965020569*G0_0_0 + 0.00554538857490241*G0_0_1 + 0.0381653213684456*G0_0_2 + 0.00554538857490283*G0_1_0 + 0.00619778723077204*G0_1_2 + 0.038165321368446*G0_2_0 + 0.00619778723077237*G0_2_1 + 0.026095946234834*G0_2_2;
    A[2787] = -0.0332723314494141*G0_0_0 - 0.0190826606842221*G0_0_1 - 0.019082660684223*G0_0_2 - 0.0190826606842234*G0_1_0 - 0.00489298991903076*G0_1_1 - 0.0332723314494148*G0_1_2 - 0.0190826606842231*G0_2_0 - 0.0332723314494143*G0_2_1 - 0.00489298991903192*G0_2_2;
    A[2788] = 0.0401225173360579*G0_0_0 + 0.0102752788299657*G0_0_1 + 0.0298472385060918*G0_0_2 + 0.0102752788299657*G0_1_0 + 0.018104062700417*G0_1_2 + 0.0298472385060919*G0_2_0 + 0.0181040627004158*G0_2_1 + 0.0195719596761263*G0_2_2;
    A[2789] = 0.00130479731174153*G0_0_0 + 0.000652398655870761*G0_0_1 - 0.0107645778218696*G0_0_2 + 0.000652398655870862*G0_1_0 + 0.00130479731174177*G0_1_2 - 0.0107645778218694*G0_2_0 + 0.00130479731174124*G0_2_1 - 0.0228339529554804*G0_2_2;
    A[2790] = -0.268135847562927*G0_0_0 - 0.321632537344339*G0_0_1 - 0.290643601190472*G0_0_2 - 0.321632537344338*G0_1_0 - 0.5845491956603*G0_1_1 - 0.448850275239158*G0_1_2 - 0.22214174232403*G0_2_0 - 0.194414799449518*G0_2_1 - 0.268135847562926*G0_2_2;
    A[2791] = -0.0371867233846409*G0_0_0 + 0.186912214907004*G0_0_1 + 0.140428810676204*G0_0_2 + 0.186912214907003*G0_1_0 - 0.0371867233846407*G0_1_1 + 0.0376760223765413*G0_1_2 + 0.0376760223765412*G0_2_0 + 0.140428810676203*G0_2_1 + 0.0655660649150205*G0_2_2;
    A[2792] = 0.17223324514991*G0_0_0 + 0.00619778723077352*G0_0_1 - 0.0264221455627697*G0_0_2 + 0.00619778723077416*G0_1_0 + 0.0123955744615459*G0_1_1 + 0.00619778723077422*G0_1_2 + 0.0714376528178609*G0_2_0 + 0.00619778723077313*G0_2_1 - 0.0137003717732885*G0_2_2;
    A[2793] = 0.72611970398428*G0_0_0 + 0.456026660453738*G0_0_1 + 0.456026660453738*G0_0_2 + 0.456026660453738*G0_1_0 + 0.912053320907479*G0_1_1 + 0.456026660453738*G0_1_2 + 0.456026660453738*G0_2_0 + 0.456026660453738*G0_2_1 + 0.726119703984278*G0_2_2;
    A[2794] = -0.448197876583288*G0_0_0 - 0.224098938291644*G0_0_1 - 0.121346149991981*G0_0_2 - 0.224098938291644*G0_1_0 - 0.0371867233846381*G0_1_1 - 0.0748627457611811*G0_1_2 - 0.326851726591305*G0_2_0 - 0.177615534060843*G0_2_1 - 0.149725491522364*G0_2_2;
    A[2795] = -0.20941996853455*G0_0_0 - 0.262916658315961*G0_0_1 - 0.104709984267274*G0_0_2 - 0.262916658315961*G0_1_0 - 0.584549195660302*G0_1_1 - 0.13569892042114*G0_1_2 - 0.290643601190473*G0_2_0 - 0.390134396210781*G0_2_1 - 0.209419968534549*G0_2_2;
    A[2796] = 0.352295274170269*G0_0_0 + 0.249542485870605*G0_0_1 + 0.249542485870607*G0_0_2 + 0.249542485870606*G0_1_0 - 0.0391439193522564*G0_1_1 + 0.445262082631869*G0_1_2 + 0.249542485870605*G0_2_0 + 0.445262082631866*G0_2_1 + 0.146789697570943*G0_2_2;
    A[2797] = -0.234863516113512*G0_0_0 - 0.0195719596761247*G0_0_1 - 0.141896707651914*G0_0_2 - 0.0195719596761246*G0_1_0 - 0.0391439193522504*G0_1_1 - 0.0195719596761247*G0_1_2 - 0.141896707651912*G0_2_0 - 0.0195719596761245*G0_2_1 - 0.0489298991903139*G0_2_2;
    A[2798] = -0.185933616923196*G0_0_0 - 0.288686405222858*G0_0_1 - 0.484406001984121*G0_0_2 - 0.288686405222857*G0_1_0 - 0.0391439193522487*G0_1_1 - 0.484406001984121*G0_1_2 - 0.48440600198412*G0_2_0 - 0.484406001984118*G0_2_1 - 0.782878387045044*G0_2_2;
    A[2799] = -0.0391439193522526*G0_0_0 + 0.00489298991903025*G0_0_1 + 0.0538228891093478*G0_0_2 + 0.0048929899190314*G0_1_0 + 0.00978597983806221*G0_1_1 + 0.00489298991903042*G0_1_2 + 0.0538228891093471*G0_2_0 + 0.00489298991903056*G0_2_1 + 0.146789697570946*G0_2_2;
    A[2800] = 0.0117431758056757*G0_0_0 + 0.0117431758056757*G0_0_1 + 0.0117431758056757*G0_0_2 + 0.0117431758056756*G0_1_0 + 0.0117431758056756*G0_1_1 + 0.0117431758056756*G0_1_2 + 0.00946499969937445*G0_2_0 + 0.00946499969937442*G0_2_1 + 0.00946499969937438*G0_2_2;
    A[2801] = -0.0298472385060922*G0_0_0 + 0.00472467106581689*G0_2_0;
    A[2802] = -0.0298472385060926*G0_1_1 + 0.00472467106581695*G0_2_1;
    A[2803] = -0.00946499969937467*G0_2_2;
    A[2804] = 0.0642612676032802*G0_1_1 + 0.042275432900432*G0_1_2 + 0.0422754329004323*G0_2_1 + 0.0429604514890969*G0_2_2;
    A[2805] = 0.00880738185425698*G0_1_1 + 0.00649136662591499*G0_1_2 + 0.00649136662591591*G0_2_1 + 0.00352295274170322*G0_2_2;
    A[2806] = -0.000652398655871443*G0_1_1 - 0.00949240044292108*G0_1_2 - 0.00949240044292177*G0_2_1 - 0.0189848008858428*G0_2_2;
    A[2807] = 0.00329461321214739*G0_1_2 + 0.00329461321214807*G0_2_1 + 0.0292600797158087*G0_2_2;
    A[2808] = 0.0642612676032803*G0_0_0 + 0.0422754329004322*G0_0_2 + 0.0422754329004319*G0_2_0 + 0.0429604514890963*G0_2_2;
    A[2809] = 0.00880738185425678*G0_0_0 + 0.00649136662591524*G0_0_2 + 0.00649136662591629*G0_2_0 + 0.00352295274170391*G0_2_2;
    A[2810] = -0.000652398655871091*G0_0_0 - 0.00949240044292143*G0_0_2 - 0.00949240044292119*G0_2_0 - 0.0189848008858424*G0_2_2;
    A[2811] = 0.00329461321214802*G0_0_2 + 0.00329461321214806*G0_2_0 + 0.0292600797158088*G0_2_2;
    A[2812] = 0.176636936077038*G0_0_0 + 0.0422754329004323*G0_0_1 + 0.0422754329004326*G0_1_0 + 0.049321338383837*G0_2_0 + 0.0429604514890963*G0_2_1;
    A[2813] = -0.334680510461756*G0_0_0 - 0.0838658472122003*G0_0_1 - 0.0838658472122012*G0_1_0 + 0.104709984267275*G0_1_1 - 0.1677316944244*G0_2_0 - 0.0777006799142205*G0_2_1;
    A[2814] = 0.104709984267274*G0_0_0 - 0.0838658472122004*G0_0_1 - 0.0838658472121995*G0_1_0 - 0.334680510461757*G0_1_1 - 0.0777006799142212*G0_2_0 - 0.167731694424401*G0_2_1;
    A[2815] = 0.0422754329004323*G0_0_1 + 0.042275432900432*G0_1_0 + 0.176636936077038*G0_1_1 + 0.042960451489097*G0_2_0 + 0.0493213383838377*G0_2_1;
    A[2816] = -0.00913358118219242*G0_0_0 - 0.00913358118219248*G0_0_1 - 0.00329461321214821*G0_0_2 - 0.00913358118219251*G0_1_0 - 0.00913358118219258*G0_1_1 - 0.00329461321214807*G0_1_2 - 0.0032946132121476*G0_2_0 - 0.00329461321214761*G0_2_1;
    A[2817] = -0.0156575677409011*G0_0_0 - 0.0156575677409009*G0_0_1 - 0.00257697469068979*G0_0_2 - 0.0156575677409005*G0_1_0 - 0.0156575677409005*G0_1_1 - 0.00257697469068988*G0_1_2 - 0.00257697469068986*G0_2_0 - 0.00257697469068998*G0_2_1;
    A[2818] = -0.000652398655870737*G0_0_0 - 0.000652398655870862*G0_0_1 - 0.00257697469068969*G0_0_2 - 0.000652398655870874*G0_1_0 - 0.000652398655870843*G0_1_1 - 0.0025769746906898*G0_1_2 - 0.00257697469068993*G0_2_0 - 0.00257697469068984*G0_2_1;
    A[2819] = -0.00329461321214825*G0_0_2 - 0.00329461321214806*G0_1_2 - 0.00329461321214804*G0_2_0 - 0.00329461321214807*G0_2_1;
    A[2820] = -0.0435476102793806*G0_0_0 - 0.0377086423093364*G0_0_1 - 0.0435476102793806*G0_0_2 - 0.0377086423093364*G0_1_0 - 0.0318696743392918*G0_1_1 - 0.0377086423093364*G0_1_2 - 0.029260079715808*G0_2_0 - 0.0259654665036603*G0_2_1 - 0.029260079715808*G0_2_2;
    A[2821] = 0.0567586830607657*G0_0_0 + 0.0378717419733041*G0_0_1 + 0.0567586830607658*G0_0_2 + 0.0378717419733046*G0_1_0 + 0.0127870136550689*G0_1_1 + 0.0378717419733045*G0_1_2 + 0.0189848008858421*G0_2_0 + 0.00949240044292086*G0_2_1 + 0.018984800885842*G0_2_2;
    A[2822] = 0.104709984267275*G0_0_0 + 0.0303691574307891*G0_0_1 + 0.104709984267275*G0_0_2 + 0.0303691574307891*G0_1_0 - 0.00678494602105711*G0_1_1 + 0.0303691574307892*G0_1_2 - 0.0035229527417023*G0_2_0 + 0.00296841388421254*G0_2_1 - 0.0035229527417023*G0_2_2;
    A[2823] = -0.0422754329004325*G0_0_1 - 0.0422754329004319*G0_1_0 + 0.0439716694056969*G0_1_1 - 0.042275432900432*G0_1_2 - 0.0429604514890969*G0_2_0 - 0.000685018588664421*G0_2_1 - 0.0429604514890969*G0_2_2;
    A[2824] = -0.0318696743392921*G0_0_0 - 0.0377086423093363*G0_0_1 - 0.0377086423093363*G0_0_2 - 0.0377086423093363*G0_1_0 - 0.0435476102793808*G0_1_1 - 0.0435476102793809*G0_1_2 - 0.0259654665036602*G0_2_0 - 0.029260079715808*G0_2_1 - 0.029260079715808*G0_2_2;
    A[2825] = 0.0127870136550692*G0_0_0 + 0.0378717419733041*G0_0_1 + 0.037871741973304*G0_0_2 + 0.0378717419733042*G0_1_0 + 0.0567586830607662*G0_1_1 + 0.0567586830607661*G0_1_2 + 0.00949240044292071*G0_2_0 + 0.018984800885842*G0_2_1 + 0.0189848008858419*G0_2_2;
    A[2826] = -0.00678494602105698*G0_0_0 + 0.0303691574307892*G0_0_1 + 0.0303691574307892*G0_0_2 + 0.0303691574307889*G0_1_0 + 0.104709984267275*G0_1_1 + 0.104709984267275*G0_1_2 + 0.00296841388421273*G0_2_0 - 0.00352295274170247*G0_2_1 - 0.00352295274170245*G0_2_2;
    A[2827] = 0.0439716694056964*G0_0_0 - 0.0422754329004323*G0_0_1 - 0.0422754329004324*G0_0_2 - 0.0422754329004324*G0_1_0 - 0.000685018588664586*G0_2_0 - 0.0429604514890965*G0_2_1 - 0.0429604514890965*G0_2_2;
    A[2828] = -0.156575677409008*G0_0_0 + 0.0678494602105701*G0_0_1 - 0.0225077536275453*G0_0_2 + 0.0678494602105698*G0_1_0 + 0.104709984267274*G0_1_2 - 0.0225077536275444*G0_2_0 + 0.104709984267275*G0_2_1 + 0.0234863516113506*G0_2_2;
    A[2829] = 0.0929668084615988*G0_0_0 - 0.135046521765269*G0_0_1 - 0.27009304353054*G0_0_2 - 0.13504652176527*G0_1_0 + 0.092966808461599*G0_1_1 - 0.27009304353054*G0_1_2 - 0.270093043530543*G0_2_0 - 0.270093043530542*G0_2_1 - 0.540186087061082*G0_2_2;
    A[2830] = 0.0678494602105698*G0_0_1 + 0.104709984267275*G0_0_2 + 0.0678494602105695*G0_1_0 - 0.15657567740901*G0_1_1 - 0.0225077536275451*G0_1_2 + 0.104709984267276*G0_2_0 - 0.0225077536275443*G0_2_1 + 0.0234863516113515*G0_2_2;
    A[2831] = 0.0195719596761267*G0_0_0 + 0.0141896707651917*G0_0_1 + 0.0748627457611834*G0_0_2 + 0.0141896707651917*G0_1_0 + 0.0283793415303825*G0_1_2 + 0.0748627457611823*G0_2_0 + 0.0283793415303847*G0_2_1 + 0.149725491522365*G0_2_2;
    A[2832] = 0.0141896707651909*G0_0_1 + 0.0283793415303816*G0_0_2 + 0.0141896707651914*G0_1_0 + 0.0195719596761259*G0_1_1 + 0.0748627457611818*G0_1_2 + 0.0283793415303828*G0_2_0 + 0.0748627457611807*G0_2_1 + 0.149725491522364*G0_2_2;
    A[2833] = -0.000652398655870679*G0_0_1 - 0.0127217737894814*G0_0_2 - 0.000652398655871725*G0_1_0 - 0.0127217737894818*G0_1_2 - 0.0127217737894823*G0_2_0 - 0.0127217737894819*G0_2_1 - 0.113517366121532*G0_2_2;
    A[2834] = 0.0391439193522531*G0_0_0 + 0.0446893079271556*G0_0_1 + 0.0127217737894828*G0_0_2 + 0.0446893079271557*G0_1_0 + 0.050234696502058*G0_1_1 + 0.012069375133612*G0_1_2 + 0.0127217737894822*G0_2_0 + 0.0120693751336108*G0_2_1;
    A[2835] = 0.0929668084616003*G0_0_0 + 0.0322937334656076*G0_0_1 - 0.0283793415303827*G0_0_2 + 0.0322937334656092*G0_1_0 - 0.0088073818542569*G0_1_1 - 0.014189670765192*G0_1_2 - 0.0283793415303828*G0_2_0 - 0.0141896707651913*G0_2_1;
    A[2836] = -0.0678494602105707*G0_0_1 - 0.104709984267275*G0_0_2 - 0.0678494602105697*G0_1_0 - 0.0965550010688891*G0_1_1 - 0.0368605240567042*G0_1_2 - 0.104709984267276*G0_2_0 - 0.0368605240567044*G0_2_1;
    A[2837] = 0.0195719596761256*G0_0_0 + 0.0298472385060921*G0_0_1 + 0.0181040627004153*G0_0_2 + 0.029847238506092*G0_1_0 + 0.0401225173360578*G0_1_1 + 0.0102752788299655*G0_1_2 + 0.0181040627004174*G0_2_0 + 0.0102752788299666*G0_2_1;
    A[2838] = -0.0141896707651916*G0_0_1 - 0.0283793415303834*G0_0_2 - 0.0141896707651913*G0_1_0 - 0.0332723314494142*G0_1_1 - 0.0141896707651916*G0_1_2 - 0.0283793415303826*G0_2_0 - 0.0141896707651924*G0_2_1;
    A[2839] = 0.000652398655871632*G0_0_1 + 0.012721773789483*G0_0_2 + 0.00065239865587176*G0_1_0 + 0.0013047973117422*G0_1_1 + 0.0120693751336118*G0_1_2 + 0.0127217737894824*G0_2_0 + 0.0120693751336116*G0_2_1;
    A[2840] = 0.0502346965020573*G0_0_0 + 0.0446893079271547*G0_0_1 + 0.0120693751336113*G0_0_2 + 0.0446893079271547*G0_1_0 + 0.0391439193522527*G0_1_1 + 0.0127217737894824*G0_1_2 + 0.01206937513361*G0_2_0 + 0.0127217737894814*G0_2_1;
    A[2841] = -0.00880738185425706*G0_0_0 + 0.0322937334656079*G0_0_1 - 0.014189670765192*G0_0_2 + 0.032293733465608*G0_1_0 + 0.0929668084615987*G0_1_1 - 0.0283793415303833*G0_1_2 - 0.0141896707651905*G0_2_0 - 0.0283793415303822*G0_2_1;
    A[2842] = -0.0965550010688881*G0_0_0 - 0.0678494602105703*G0_0_1 - 0.0368605240567045*G0_0_2 - 0.0678494602105703*G0_1_0 - 0.104709984267275*G0_1_2 - 0.0368605240567038*G0_2_0 - 0.104709984267275*G0_2_1;
    A[2843] = 0.0401225173360582*G0_0_0 + 0.0298472385060916*G0_0_1 + 0.0102752788299658*G0_0_2 + 0.0298472385060922*G0_1_0 + 0.0195719596761264*G0_1_1 + 0.0181040627004165*G0_1_2 + 0.0102752788299668*G0_2_0 + 0.0181040627004179*G0_2_1;
    A[2844] = -0.0332723314494142*G0_0_0 - 0.0141896707651917*G0_0_1 - 0.0141896707651907*G0_0_2 - 0.0141896707651915*G0_1_0 - 0.0283793415303825*G0_1_2 - 0.0141896707651931*G0_2_0 - 0.0283793415303845*G0_2_1;
    A[2845] = 0.00130479731174181*G0_0_0 + 0.000652398655870599*G0_0_1 + 0.0120693751336112*G0_0_2 + 0.000652398655870664*G0_1_0 + 0.012721773789482*G0_1_2 + 0.0120693751336111*G0_2_0 + 0.0127217737894818*G0_2_1;
    A[2846] = 0.17223324514991*G0_0_0 + 0.166035457919137*G0_0_1 + 0.19865539071268*G0_0_2 + 0.166035457919136*G0_1_0 + 0.17223324514991*G0_1_1 + 0.19865539071268*G0_1_2 + 0.100795592332048*G0_2_0 + 0.100795592332048*G0_2_1 + 0.113517366121531*G0_2_2;
    A[2847] = -0.0371867233846395*G0_0_0 - 0.224098938291644*G0_0_1 - 0.177615534060844*G0_0_2 - 0.224098938291644*G0_1_0 - 0.448197876583289*G0_1_1 - 0.326851726591307*G0_1_2 - 0.0748627457611809*G0_2_0 - 0.121346149991981*G0_2_1 - 0.149725491522364*G0_2_2;
    A[2848] = -0.268135847562927*G0_0_0 + 0.053496689781411*G0_0_1 + 0.0225077536275451*G0_0_2 + 0.0534966897814116*G0_1_0 - 0.209419968534549*G0_1_1 - 0.104709984267274*G0_1_2 - 0.0459941052388961*G0_2_0 + 0.0812236326559232*G0_2_1 - 0.0234863516113505*G0_2_2;
    A[2849] = -0.448197876583288*G0_0_0 - 0.224098938291644*G0_0_1 - 0.326851726591305*G0_0_2 - 0.224098938291644*G0_1_0 - 0.0371867233846381*G0_1_1 - 0.177615534060843*G0_1_2 - 0.121346149991981*G0_2_0 - 0.0748627457611811*G0_2_1 - 0.149725491522364*G0_2_2;
    A[2850] = 0.726119703984278*G0_0_0 + 0.270093043530541*G0_0_1 + 0.27009304353054*G0_0_2 + 0.270093043530541*G0_1_0 + 0.72611970398428*G0_1_1 + 0.27009304353054*G0_1_2 + 0.27009304353054*G0_2_0 + 0.27009304353054*G0_2_1 + 0.540186087061082*G0_2_2;
    A[2851] = -0.209419968534549*G0_0_0 + 0.0534966897814113*G0_0_1 - 0.104709984267274*G0_0_2 + 0.0534966897814105*G0_1_0 - 0.268135847562929*G0_1_1 + 0.0225077536275453*G0_1_2 + 0.0812236326559234*G0_2_0 - 0.045994105238897*G0_2_1 - 0.0234863516113512*G0_2_2;
    A[2852] = -0.234863516113514*G0_0_0 - 0.215291556437385*G0_0_1 - 0.0929668084615982*G0_0_2 - 0.215291556437388*G0_1_0 - 0.234863516113513*G0_1_1 - 0.0929668084615986*G0_1_2 - 0.092966808461598*G0_2_0 - 0.0929668084615979*G0_2_1;
    A[2853] = 0.352295274170268*G0_0_0 + 0.102752788299661*G0_0_1 + 0.102752788299662*G0_0_2 + 0.102752788299662*G0_1_0 - 0.185933616923198*G0_1_1 + 0.298472385060925*G0_1_2 + 0.10275278829966*G0_2_0 + 0.298472385060925*G0_2_1;
    A[2854] = -0.185933616923199*G0_0_0 + 0.10275278829966*G0_0_1 + 0.298472385060924*G0_0_2 + 0.102752788299661*G0_1_0 + 0.35229527417027*G0_1_1 + 0.102752788299661*G0_1_2 + 0.298472385060926*G0_2_0 + 0.102752788299661*G0_2_1;
    A[2855] = -0.0391439193522521*G0_0_0 - 0.0440369092712822*G0_0_1 - 0.0929668084616008*G0_0_2 - 0.0440369092712834*G0_1_0 - 0.0391439193522519*G0_1_1 - 0.0929668084616003*G0_1_2 - 0.0929668084615995*G0_2_0 - 0.0929668084615985*G0_2_1;
    A[2856] = 0.0189195610202551*G0_0_0 + 0.0189195610202551*G0_0_1 + 0.0189195610202551*G0_0_2 + 0.0189195610202552*G0_1_0 + 0.0189195610202553*G0_1_1 + 0.0189195610202553*G0_1_2 + 0.013742125287264*G0_2_0 + 0.0137421252872641*G0_2_1 + 0.0137421252872641*G0_2_2;
    A[2857] = 0.0189195610202553*G0_0_0 + 0.00517743573299156*G0_2_0;
    A[2858] = 0.0335985307773502*G0_1_1 - 0.0143945239431348*G0_2_1;
    A[2859] = -0.0137421252872641*G0_2_2;
    A[2860] = -0.128522535206561*G0_1_1 - 0.0962940416065401*G0_1_2 - 0.0962940416065399*G0_2_1 - 0.12473862300251*G0_2_2;
    A[2861] = 0.0476251018785733*G0_1_1 + 0.0623693115012549*G0_1_2 + 0.0623693115012546*G0_2_1 + 0.12473862300251*G0_2_2;
    A[2862] = -0.0150051690850295*G0_1_1 - 0.0377738821749236*G0_1_2 - 0.037773882174923*G0_2_1 - 0.127087258163646*G0_2_2;
    A[2863] = 0.00939454064454098*G0_1_2 + 0.00939454064454043*G0_2_1 + 0.0641307878721069*G0_2_2;
    A[2864] = -0.0306627368259308*G0_0_0 - 0.0212681961813902*G0_0_2 - 0.0212681961813904*G0_2_0 - 0.011873655536849*G0_2_2;
    A[2865] = -0.00130479731174163*G0_0_0 - 0.00123955744615496*G0_0_2 - 0.00123955744615466*G0_2_0 - 0.00117431758056867*G0_2_2;
    A[2866] = -0.01500516908503*G0_0_0 - 0.0149399292194426*G0_0_2 - 0.0149399292194431*G0_2_0 - 0.0148746893538554*G0_2_2;
    A[2867] = 0.00939454064454046*G0_0_2 + 0.00939454064454035*G0_2_0 + 0.0187890812890809*G0_2_2;
    A[2868] = -0.111886369481854*G0_0_0 - 0.0212681961813903*G0_0_1 - 0.0212681961813904*G0_1_0 - 0.0477555816097475*G0_2_0 - 0.0118736555368492*G0_2_1;
    A[2869] = 0.209419968534549*G0_0_0 + 0.0359471659384851*G0_0_1 + 0.0359471659384855*G0_1_0 - 0.0528442911255405*G0_1_1 + 0.0823327103709037*G0_2_0 - 0.00182671623643866*G0_2_1;
    A[2870] = -0.271397840842283*G0_0_0 - 0.0733296089198859*G0_0_1 - 0.0733296089198866*G0_1_0 + 0.121346149991982*G0_1_1 - 0.146659217839772*G0_2_0 - 0.0109602974186309*G0_2_1;
    A[2871] = -0.0962940416065408*G0_0_1 - 0.0962940416065401*G0_1_0 - 0.257045070413123*G0_1_1 - 0.12473862300251*G0_2_0 - 0.19258808321308*G0_2_1;
    A[2872] = -0.0306627368259306*G0_0_0 - 0.0306627368259306*G0_0_1 - 0.00939454064454018*G0_0_2 - 0.0306627368259304*G0_1_0 - 0.0306627368259304*G0_1_1 - 0.00939454064454035*G0_1_2 - 0.00939454064453991*G0_2_0 - 0.00939454064453994*G0_2_1;
    A[2873] = -0.00130479731174203*G0_0_0 - 0.00130479731174223*G0_0_1 - 6.52398655874693e-05*G0_0_2 - 0.00130479731174271*G0_1_0 - 0.00130479731174275*G0_1_1 - 6.52398655876294e-05*G0_1_2 - 6.52398655871863e-05*G0_2_0 - 6.52398655870822e-05*G0_2_1;
    A[2874] = -0.0150051690850299*G0_0_0 - 0.0150051690850298*G0_0_1 - 6.52398655868515e-05*G0_0_2 - 0.0150051690850299*G0_1_0 - 0.0150051690850298*G0_1_1 - 6.52398655865302e-05*G0_1_2 - 6.52398655868465e-05*G0_2_0 - 6.52398655869971e-05*G0_2_1;
    A[2875] = -0.00939454064454051*G0_0_2 - 0.00939454064454072*G0_1_2 - 0.0093945406445404*G0_2_0 - 0.0093945406445404*G0_2_1;
    A[2876] = -0.111886369481854*G0_0_0 - 0.0906181733004637*G0_0_1 - 0.111886369481854*G0_0_2 - 0.0906181733004635*G0_1_0 - 0.0693499771190736*G0_1_1 - 0.0906181733004635*G0_1_2 - 0.0641307878721061*G0_2_0 - 0.054736247227566*G0_2_1 - 0.0641307878721059*G0_2_2;
    A[2877] = 0.20941996853455*G0_0_0 + 0.173472802596065*G0_0_1 + 0.20941996853455*G0_0_2 + 0.173472802596064*G0_1_0 + 0.0846813455320394*G0_1_1 + 0.173472802596064*G0_1_2 + 0.127087258163645*G0_2_0 + 0.0893133759887223*G0_2_1 + 0.127087258163645*G0_2_2;
    A[2878] = -0.271397840842283*G0_0_0 - 0.198068231922397*G0_0_1 - 0.271397840842283*G0_0_2 - 0.198068231922397*G0_1_0 - 0.00339247301052863*G0_1_1 - 0.198068231922396*G0_1_2 - 0.12473862300251*G0_2_0 - 0.0623693115012554*G0_2_1 - 0.12473862300251*G0_2_2;
    A[2879] = 0.0962940416065406*G0_0_1 + 0.0962940416065403*G0_1_0 - 0.0644569872000426*G0_1_1 + 0.0962940416065402*G0_1_2 + 0.12473862300251*G0_2_0 + 0.02844458139597*G0_2_1 + 0.12473862300251*G0_2_2;
    A[2880] = -0.0187890812890806*G0_0_0 - 0.0400572774704712*G0_0_1 - 0.0400572774704713*G0_0_2 - 0.0400572774704716*G0_1_0 - 0.0613254736518616*G0_1_1 - 0.0613254736518617*G0_1_2 - 0.00939454064454053*G0_2_0 - 0.0187890812890807*G0_2_1 - 0.0187890812890807*G0_2_2;
    A[2881] = -0.000130479731174547*G0_0_0 + 0.0136351319077012*G0_0_1 + 0.0136351319077013*G0_0_2 + 0.0136351319077016*G0_1_0 - 0.0254435475789642*G0_1_1 - 0.0254435475789641*G0_1_2 - 6.52398655869443e-05*G0_2_0 + 0.0148746893538555*G0_2_1 + 0.0148746893538555*G0_2_2;
    A[2882] = -0.000130479731174093*G0_0_0 - 0.0137656116388754*G0_0_1 - 0.0137656116388754*G0_0_2 - 0.0137656116388758*G0_1_0 - 0.0528442911255407*G0_1_1 - 0.0528442911255407*G0_1_2 - 6.52398655870422e-05*G0_2_0 + 0.00117431758056779*G0_2_1 + 0.00117431758056774*G0_2_2;
    A[2883] = -0.0187890812890811*G0_0_0 + 0.0212681961813901*G0_0_1 + 0.0212681961813901*G0_0_2 + 0.0212681961813903*G0_1_0 - 0.00939454064454096*G0_2_0 + 0.011873655536849*G0_2_1 + 0.0118736555368491*G0_2_2;
    A[2884] = 0.0521918924696689*G0_0_0 - 0.0378391220405102*G0_0_1 - 0.000652398655870395*G0_0_2 - 0.0378391220405102*G0_1_0 - 0.0528442911255398*G0_1_2 - 0.000652398655870307*G0_2_0 - 0.0528442911255401*G0_2_1 - 0.0534966897814108*G0_2_2;
    A[2885] = -0.185933616923197*G0_0_0 + 0.00913358118219109*G0_0_1 - 0.0812236326559238*G0_0_2 + 0.0091335811821936*G0_1_0 - 0.0880738185425671*G0_1_1 + 0.0459941052388965*G0_1_2 - 0.0812236326559228*G0_2_0 + 0.0459941052388966*G0_2_1 + 0.0234863516113509*G0_2_2;
    A[2886] = -0.135698920421139*G0_0_1 - 0.271397840842282*G0_0_2 - 0.13569892042114*G0_1_0 + 0.0521918924696718*G0_1_1 - 0.271397840842281*G0_1_2 - 0.271397840842281*G0_2_0 - 0.271397840842281*G0_2_1 - 0.542795681684564*G0_2_2;
    A[2887] = 0.0260959462348335*G0_0_0 - 0.0120693751336118*G0_0_1 + 0.0198981590040604*G0_0_2 - 0.0120693751336127*G0_1_0 - 0.0127217737894824*G0_1_2 + 0.0198981590040604*G0_2_0 - 0.012721773789483*G0_2_1 + 0.0137003717732884*G0_2_2;
    A[2888] = 0.0368605240567035*G0_0_1 + 0.104709984267276*G0_0_2 + 0.0368605240567041*G0_1_0 - 0.0228339529554807*G0_1_1 + 0.0737210481134101*G0_1_2 + 0.104709984267275*G0_2_0 + 0.0737210481134091*G0_2_1 + 0.209419968534551*G0_2_2;
    A[2889] = -0.0150051690850302*G0_0_1 - 0.052844291125542*G0_0_2 - 0.0150051690850295*G0_1_0 - 0.0300103381700614*G0_1_2 - 0.0528442911255403*G0_2_0 - 0.0300103381700604*G0_2_1 - 0.105688582251083*G0_2_2;
    A[2890] = 0.0521918924696679*G0_0_0 + 0.0900310145101789*G0_0_1 + 0.0528442911255396*G0_0_2 + 0.0900310145101782*G0_1_0 + 0.127870136550689*G0_1_1 + 0.0378391220405092*G0_1_2 + 0.0528442911255396*G0_2_0 + 0.037839122040509*G0_2_1;
    A[2891] = -0.185933616923198*G0_0_0 - 0.19506719810539*G0_0_1 - 0.104709984267275*G0_0_2 - 0.19506719810539*G0_1_0 - 0.292274597830151*G0_1_1 - 0.0678494602105702*G0_1_2 - 0.104709984267274*G0_2_0 - 0.0678494602105692*G0_2_1;
    A[2892] = 0.135698920421142*G0_0_1 + 0.271397840842283*G0_0_2 + 0.13569892042114*G0_1_0 + 0.323589733311953*G0_1_1 + 0.135698920421141*G0_1_2 + 0.271397840842281*G0_2_0 + 0.13569892042114*G0_2_1;
    A[2893] = 0.026095946234836*G0_0_0 + 0.0381653213684474*G0_0_1 + 0.00619778723077503*G0_0_2 + 0.0381653213684467*G0_1_0 + 0.0502346965020592*G0_1_1 + 0.00554538857490386*G0_1_2 + 0.00619778723077401*G0_2_0 + 0.00554538857490269*G0_2_1;
    A[2894] = -0.0368605240567056*G0_0_1 - 0.104709984267275*G0_0_2 - 0.036860524056704*G0_1_0 - 0.09655500106889*G0_1_1 - 0.0678494602105708*G0_1_2 - 0.104709984267275*G0_2_0 - 0.0678494602105701*G0_2_1;
    A[2895] = 0.0150051690850298*G0_0_1 + 0.0528442911255397*G0_0_2 + 0.0150051690850296*G0_1_0 + 0.0300103381700596*G0_1_1 + 0.0378391220405097*G0_1_2 + 0.0528442911255402*G0_2_0 + 0.0378391220405098*G0_2_1;
    A[2896] = 0.0300103381700592*G0_0_0 - 0.00782878387045164*G0_0_1 + 0.0150051690850291*G0_0_2 - 0.00782878387045146*G0_1_0 - 0.0456679059109626*G0_1_1 + 0.0300103381700592*G0_1_2 + 0.0150051690850292*G0_2_0 + 0.0300103381700603*G0_2_1;
    A[2897] = 0.00130479731174266*G0_0_0 + 0.000652398655871811*G0_0_1 + 0.000652398655871212*G0_0_2 + 0.000652398655871498*G0_1_0 - 0.0880738185425668*G0_1_1 + 0.0127217737894823*G0_1_2 + 0.000652398655870716*G0_2_0 + 0.0127217737894811*G0_2_1;
    A[2898] = 0.0300103381700595*G0_0_0 + 0.0378391220405106*G0_0_1 + 0.0150051690850305*G0_0_2 + 0.0378391220405107*G0_1_0 + 0.0528442911255418*G0_1_2 + 0.0150051690850304*G0_2_0 + 0.0528442911255406*G0_2_1;
    A[2899] = 0.00130479731174259*G0_0_0 - 0.0107645778218686*G0_0_1 + 0.000652398655872042*G0_0_2 - 0.0107645778218685*G0_1_0 - 0.0228339529554811*G0_1_1 + 0.00130479731174229*G0_1_2 + 0.000652398655870918*G0_2_0 + 0.00130479731174218*G0_2_1;
    A[2900] = 0.00130479731174125*G0_0_0 + 0.0120693751336117*G0_0_1 + 0.000652398655870208*G0_0_2 + 0.012069375133611*G0_1_0 + 0.0127217737894814*G0_1_2 + 0.000652398655870863*G0_2_0 + 0.0127217737894828*G0_2_1;
    A[2901] = 0.03001033817006*G0_0_0 + 0.0150051690850303*G0_0_1 + 0.0150051690850295*G0_0_2 + 0.0150051690850305*G0_1_0 + 0.0300103381700602*G0_1_2 + 0.01500516908503*G0_2_0 + 0.0300103381700604*G0_2_1;
    A[2902] = 0.10568858225108*G0_0_0 + 0.12787013655069*G0_0_1 + 0.105036183595209*G0_0_2 + 0.12787013655069*G0_1_0 + 0.255740273101381*G0_1_1 + 0.180062029020359*G0_1_2 + 0.0528442911255395*G0_2_0 + 0.0756782440810199*G0_2_1 + 0.10568858225108*G0_2_2;
    A[2903] = 0.0123955744615478*G0_0_0 + 0.00619778723077388*G0_0_1 + 0.00619778723077444*G0_0_2 + 0.00619778723077414*G0_1_0 + 0.172233245149911*G0_1_1 + 0.0714376528178619*G0_1_2 + 0.00619778723077358*G0_2_0 - 0.0264221455627701*G0_2_1 - 0.0137003717732887*G0_2_2;
    A[2904] = 0.10568858225108*G0_0_0 - 0.0221815542996092*G0_0_1 + 0.000652398655871002*G0_0_2 - 0.02218155429961*G0_1_0 + 0.105688582251082*G0_1_1 + 0.0528442911255402*G0_1_2 + 0.0528442911255404*G0_2_0 + 0.000652398655871393*G0_2_1 + 0.053496689781411*G0_2_2;
    A[2905] = -0.20941996853455*G0_0_0 - 0.262916658315961*G0_0_1 - 0.290643601190473*G0_0_2 - 0.262916658315961*G0_1_0 - 0.584549195660302*G0_1_1 - 0.390134396210781*G0_1_2 - 0.104709984267274*G0_2_0 - 0.13569892042114*G0_2_1 - 0.209419968534549*G0_2_2;
    A[2906] = -0.209419968534549*G0_0_0 + 0.0534966897814105*G0_0_1 + 0.0812236326559234*G0_0_2 + 0.0534966897814113*G0_1_0 - 0.268135847562929*G0_1_1 - 0.0459941052388971*G0_1_2 - 0.104709984267274*G0_2_0 + 0.0225077536275453*G0_2_1 - 0.0234863516113512*G0_2_2;
    A[2907] = 0.542795681684565*G0_0_0 + 0.271397840842283*G0_0_1 + 0.271397840842282*G0_0_2 + 0.271397840842283*G0_1_0 + 0.647179466623906*G0_1_1 + 0.271397840842281*G0_1_2 + 0.271397840842282*G0_2_0 + 0.271397840842281*G0_2_1 + 0.542795681684563*G0_2_2;
    A[2908] = -0.0521918924696666*G0_0_0 - 0.0424059126316043*G0_0_1 - 0.0260959462348329*G0_0_2 - 0.0424059126316029*G0_1_0 + 0.143527704291596*G0_1_1 - 0.0750258454251483*G0_1_2 - 0.0260959462348345*G0_2_0 - 0.0750258454251504*G0_2_1;
    A[2909] = -0.0521918924696701*G0_0_0 - 0.00978597983806367*G0_0_1 - 0.026095946234835*G0_0_2 - 0.00978597983806513*G0_1_0 + 0.176147637085134*G0_1_1 - 0.0587158790283794*G0_1_2 - 0.0260959462348348*G0_2_0 - 0.0587158790283786*G0_2_1;
    A[2910] = 0.371867233846395*G0_0_0 + 0.185933616923196*G0_0_1 + 0.185933616923197*G0_0_2 + 0.185933616923196*G0_1_0 - 0.150051690850305*G0_1_1 + 0.316413348097372*G0_1_2 + 0.185933616923197*G0_2_0 + 0.316413348097372*G0_2_1;
    A[2911] = -0.0521918924696696*G0_0_0 - 0.0260959462348348*G0_0_1 - 0.0260959462348343*G0_0_2 - 0.026095946234834*G0_1_0 + 0.0456679059109615*G0_1_1 - 0.0750258454251498*G0_1_2 - 0.0260959462348343*G0_2_0 - 0.0750258454251507*G0_2_1;
    A[2912] = -0.0782878387045049*G0_0_0 - 0.0782878387045048*G0_0_1 - 0.0782878387045048*G0_0_2 - 0.0782878387045047*G0_1_0 - 0.0782878387045048*G0_1_1 - 0.0782878387045047*G0_1_2 - 0.0782878387045045*G0_2_0 - 0.0782878387045046*G0_2_1 - 0.0782878387045045*G0_2_2;
    A[2913] = 0.044036909271284*G0_0_0;
    A[2914] = 0.0440369092712842*G0_1_1;
    A[2915] = 0.0440369092712843*G0_2_2;
    A[2916] = -0.050560895829993*G0_1_1 - 0.00260959462348382*G0_1_2 - 0.00260959462348333*G0_2_1;
    A[2917] = 0.0391439193522537*G0_1_1 + 0.0319675341376744*G0_1_2 + 0.0319675341376736*G0_2_1 + 0.0260959462348357*G0_2_2;
    A[2918] = 0.0260959462348343*G0_1_1 + 0.031967534137671*G0_1_2 + 0.0319675341376722*G0_2_1 + 0.0391439193522509*G0_2_2;
    A[2919] = -0.00260959462348162*G0_1_2 - 0.00260959462348228*G0_2_1 - 0.050560895829991*G0_2_2;
    A[2920] = -0.0505608958299928*G0_0_0 - 0.00260959462348402*G0_0_2 - 0.00260959462348407*G0_2_0;
    A[2921] = 0.0391439193522538*G0_0_0 + 0.0319675341376746*G0_0_2 + 0.0319675341376743*G0_2_0 + 0.0260959462348355*G0_2_2;
    A[2922] = 0.0260959462348343*G0_0_0 + 0.0319675341376719*G0_0_2 + 0.0319675341376719*G0_2_0 + 0.0391439193522521*G0_2_2;
    A[2923] = -0.00260959462348186*G0_0_2 - 0.00260959462348221*G0_2_0 - 0.0505608958299915*G0_2_2;
    A[2924] = -0.050560895829993*G0_0_0 - 0.00260959462348398*G0_0_1 - 0.00260959462348318*G0_1_0;
    A[2925] = 0.0391439193522531*G0_0_0 + 0.0319675341376736*G0_0_1 + 0.0319675341376723*G0_1_0 + 0.0260959462348346*G0_1_1;
    A[2926] = 0.0260959462348338*G0_0_0 + 0.0319675341376724*G0_0_1 + 0.031967534137672*G0_1_0 + 0.0391439193522521*G0_1_1;
    A[2927] = -0.00260959462348476*G0_0_1 - 0.00260959462348385*G0_1_0 - 0.0505608958299937*G0_1_1;
    A[2928] = 0.0962288017409537*G0_0_0 + 0.0962288017409539*G0_0_1 + 0.131458329157981*G0_0_2 + 0.0962288017409536*G0_1_0 + 0.096228801740954*G0_1_1 + 0.131458329157981*G0_1_2 + 0.131458329157981*G0_2_0 + 0.131458329157982*G0_2_1 + 0.166687856575009*G0_2_2;
    A[2929] = -0.15657567740901*G0_0_0 - 0.15657567740901*G0_0_1 - 0.0662184635708945*G0_0_2 - 0.15657567740901*G0_1_0 - 0.156575677409011*G0_1_1 - 0.0662184635708949*G0_1_2 - 0.066218463570894*G0_2_0 - 0.0662184635708944*G0_2_1 - 0.161794866655978*G0_2_2;
    A[2930] = 0.0260959462348338*G0_0_0 + 0.026095946234834*G0_0_1 + 0.010438378493934*G0_0_2 + 0.0260959462348342*G0_1_0 + 0.0260959462348344*G0_1_1 + 0.0104383784939345*G0_1_2 + 0.0104383784939332*G0_2_0 + 0.0104383784939333*G0_2_1 + 0.180714427676232*G0_2_2;
    A[2931] = 0.00260959462348311*G0_0_2 + 0.00260959462348304*G0_1_2 + 0.00260959462348229*G0_2_0 + 0.0026095946234823*G0_2_1 - 0.151356488162043*G0_2_2;
    A[2932] = 0.0962288017409527*G0_0_0 + 0.131458329157981*G0_0_1 + 0.0962288017409529*G0_0_2 + 0.13145832915798*G0_1_0 + 0.166687856575007*G0_1_1 + 0.13145832915798*G0_1_2 + 0.0962288017409528*G0_2_0 + 0.13145832915798*G0_2_1 + 0.0962288017409532*G0_2_2;
    A[2933] = -0.156575677409008*G0_0_0 - 0.0662184635708929*G0_0_1 - 0.156575677409008*G0_0_2 - 0.066218463570892*G0_1_0 - 0.161794866655976*G0_1_1 - 0.066218463570892*G0_1_2 - 0.156575677409009*G0_2_0 - 0.0662184635708927*G0_2_1 - 0.156575677409009*G0_2_2;
    A[2934] = 0.0260959462348334*G0_0_0 + 0.0104383784939333*G0_0_1 + 0.0260959462348333*G0_0_2 + 0.0104383784939324*G0_1_0 + 0.180714427676231*G0_1_1 + 0.0104383784939322*G0_1_2 + 0.0260959462348346*G0_2_0 + 0.0104383784939331*G0_2_1 + 0.0260959462348347*G0_2_2;
    A[2935] = 0.00260959462348337*G0_0_1 + 0.00260959462348369*G0_1_0 - 0.151356488162043*G0_1_1 + 0.00260959462348374*G0_1_2 + 0.00260959462348332*G0_2_1;
    A[2936] = 0.166687856575008*G0_0_0 + 0.13145832915798*G0_0_1 + 0.13145832915798*G0_0_2 + 0.131458329157981*G0_1_0 + 0.0962288017409534*G0_1_1 + 0.0962288017409532*G0_1_2 + 0.13145832915798*G0_2_0 + 0.0962288017409534*G0_2_1 + 0.0962288017409536*G0_2_2;
    A[2937] = -0.161794866655976*G0_0_0 - 0.0662184635708939*G0_0_1 - 0.066218463570894*G0_0_2 - 0.0662184635708935*G0_1_0 - 0.15657567740901*G0_1_1 - 0.15657567740901*G0_1_2 - 0.0662184635708933*G0_2_0 - 0.156575677409009*G0_2_1 - 0.15657567740901*G0_2_2;
    A[2938] = 0.180714427676232*G0_0_0 + 0.0104383784939329*G0_0_1 + 0.0104383784939331*G0_0_2 + 0.0104383784939341*G0_1_0 + 0.0260959462348349*G0_1_1 + 0.0260959462348349*G0_1_2 + 0.010438378493934*G0_2_0 + 0.0260959462348344*G0_2_1 + 0.0260959462348346*G0_2_2;
    A[2939] = -0.151356488162042*G0_0_0 + 0.00260959462348423*G0_0_1 + 0.00260959462348392*G0_0_2 + 0.00260959462348342*G0_1_0 + 0.00260959462348333*G0_2_0;
    A[2940] = 0.195719596761263*G0_0_0 + 0.0750258454251521*G0_0_1 + 0.0750258454251514*G0_0_2 + 0.0750258454251516*G0_1_0 + 0.026095946234836*G0_1_2 + 0.0750258454251508*G0_2_0 + 0.0260959462348353*G0_2_1;
    A[2941] = 0.146789697570945*G0_0_0 + 0.141896707651913*G0_0_1 + 0.0929668084615983*G0_0_2 + 0.141896707651915*G0_1_0 + 0.146789697570945*G0_1_1 + 0.0929668084615978*G0_1_2 + 0.0929668084615972*G0_2_0 + 0.092966808461597*G0_2_1;
    A[2942] = 0.0750258454251539*G0_0_1 + 0.0260959462348345*G0_0_2 + 0.0750258454251526*G0_1_0 + 0.195719596761266*G0_1_1 + 0.0750258454251503*G0_1_2 + 0.026095946234837*G0_2_0 + 0.0750258454251521*G0_2_1;
    A[2943] = 0.146789697570946*G0_0_0 + 0.0929668084615992*G0_0_1 + 0.141896707651914*G0_0_2 + 0.0929668084615975*G0_1_0 + 0.092966808461599*G0_1_2 + 0.141896707651913*G0_2_0 + 0.092966808461599*G0_2_1 + 0.146789697570946*G0_2_2;
    A[2944] = 0.0929668084615975*G0_0_1 + 0.0929668084616013*G0_0_2 + 0.0929668084615995*G0_1_0 + 0.146789697570944*G0_1_1 + 0.141896707651917*G0_1_2 + 0.0929668084615987*G0_2_0 + 0.141896707651913*G0_2_1 + 0.146789697570947*G0_2_2;
    A[2945] = 0.0260959462348394*G0_0_1 + 0.0750258454251523*G0_0_2 + 0.0260959462348375*G0_1_0 + 0.0750258454251504*G0_1_2 + 0.075025845425153*G0_2_0 + 0.0750258454251533*G0_2_1 + 0.195719596761262*G0_2_2;
    A[2946] = -0.782878387045048*G0_0_0 - 0.466465038947674*G0_0_1 - 0.466465038947674*G0_0_2 - 0.466465038947674*G0_1_0 - 0.1500516908503*G0_1_1 - 0.3359853077735*G0_1_2 - 0.466465038947673*G0_2_0 - 0.3359853077735*G0_2_1 - 0.150051690850301*G0_2_2;
    A[2947] = 0.146789697570944*G0_0_0 + 0.44526208263187*G0_0_1 + 0.249542485870608*G0_0_2 + 0.445262082631868*G0_1_0 - 0.0391439193522544*G0_1_1 + 0.249542485870607*G0_1_2 + 0.249542485870607*G0_2_0 + 0.249542485870609*G0_2_1 + 0.352295274170271*G0_2_2;
    A[2948] = -0.0750258454251476*G0_0_1 - 0.0260959462348348*G0_0_2 - 0.0750258454251525*G0_1_0 + 0.143527704291595*G0_1_1 - 0.0424059126316064*G0_1_2 - 0.0260959462348368*G0_2_0 - 0.0424059126316058*G0_2_1 - 0.0521918924696701*G0_2_2;
    A[2949] = 0.146789697570945*G0_0_0 + 0.249542485870609*G0_0_1 + 0.445262082631871*G0_0_2 + 0.249542485870607*G0_1_0 + 0.352295274170272*G0_1_1 + 0.249542485870609*G0_1_2 + 0.44526208263187*G0_2_0 + 0.249542485870609*G0_2_1 - 0.0391439193522532*G0_2_2;
    A[2950] = -0.0929668084616015*G0_0_1 - 0.0929668084615981*G0_0_2 - 0.0929668084615997*G0_1_0 - 0.234863516113515*G0_1_1 - 0.215291556437386*G0_1_2 - 0.092966808461599*G0_2_0 - 0.215291556437389*G0_2_1 - 0.234863516113512*G0_2_2;
    A[2951] = -0.0260959462348332*G0_0_1 - 0.0750258454251507*G0_0_2 - 0.0260959462348382*G0_1_0 - 0.0521918924696692*G0_1_1 - 0.0424059126316075*G0_1_2 - 0.0750258454251536*G0_2_0 - 0.0424059126316062*G0_2_1 + 0.143527704291591*G0_2_2;
    A[2952] = -0.1500516908503*G0_0_0 - 0.466465038947674*G0_0_1 - 0.335985307773499*G0_0_2 - 0.466465038947673*G0_1_0 - 0.782878387045047*G0_1_1 - 0.466465038947673*G0_1_2 - 0.335985307773499*G0_2_0 - 0.466465038947672*G0_2_1 - 0.1500516908503*G0_2_2;
    A[2953] = -0.0391439193522543*G0_0_0 + 0.445262082631869*G0_0_1 + 0.249542485870609*G0_0_2 + 0.44526208263187*G0_1_0 + 0.146789697570947*G0_1_1 + 0.24954248587061*G0_1_2 + 0.249542485870607*G0_2_0 + 0.249542485870607*G0_2_1 + 0.352295274170271*G0_2_2;
    A[2954] = 0.143527704291593*G0_0_0 - 0.0750258454251527*G0_0_1 - 0.0424059126316068*G0_0_2 - 0.0750258454251506*G0_1_0 - 0.0260959462348354*G0_1_2 - 0.0424059126316054*G0_2_0 - 0.0260959462348356*G0_2_1 - 0.0521918924696692*G0_2_2;
    A[2955] = 0.352295274170271*G0_0_0 + 0.249542485870608*G0_0_1 + 0.249542485870607*G0_0_2 + 0.24954248587061*G0_1_0 + 0.146789697570946*G0_1_1 + 0.44526208263187*G0_1_2 + 0.249542485870609*G0_2_0 + 0.44526208263187*G0_2_1 - 0.0391439193522547*G0_2_2;
    A[2956] = -0.234863516113514*G0_0_0 - 0.0929668084615985*G0_0_1 - 0.215291556437389*G0_0_2 - 0.0929668084616005*G0_1_0 - 0.0929668084616003*G0_1_2 - 0.215291556437389*G0_2_0 - 0.0929668084615986*G0_2_1 - 0.234863516113515*G0_2_2;
    A[2957] = -0.0521918924696685*G0_0_0 - 0.0260959462348396*G0_0_1 - 0.0424059126316058*G0_0_2 - 0.0260959462348341*G0_1_0 - 0.07502584542515*G0_1_2 - 0.0424059126316054*G0_2_0 - 0.075025845425153*G0_2_1 + 0.143527704291593*G0_2_2;
    A[2958] = -0.1500516908503*G0_0_0 - 0.335985307773498*G0_0_1 - 0.466465038947673*G0_0_2 - 0.335985307773498*G0_1_0 - 0.150051690850299*G0_1_1 - 0.466465038947672*G0_1_2 - 0.466465038947673*G0_2_0 - 0.466465038947673*G0_2_1 - 0.782878387045046*G0_2_2;
    A[2959] = -0.0391439193522533*G0_0_0 + 0.249542485870609*G0_0_1 + 0.445262082631869*G0_0_2 + 0.249542485870608*G0_1_0 + 0.35229527417027*G0_1_1 + 0.249542485870607*G0_1_2 + 0.44526208263187*G0_2_0 + 0.249542485870608*G0_2_1 + 0.146789697570944*G0_2_2;
    A[2960] = 0.143527704291593*G0_0_0 - 0.0424059126316063*G0_0_1 - 0.0750258454251521*G0_0_2 - 0.042405912631607*G0_1_0 - 0.0521918924696695*G0_1_1 - 0.0260959462348368*G0_1_2 - 0.0750258454251495*G0_2_0 - 0.026095946234834*G0_2_1;
    A[2961] = 0.352295274170269*G0_0_0 + 0.249542485870606*G0_0_1 + 0.249542485870605*G0_0_2 + 0.249542485870605*G0_1_0 - 0.0391439193522564*G0_1_1 + 0.445262082631866*G0_1_2 + 0.249542485870607*G0_2_0 + 0.445262082631869*G0_2_1 + 0.146789697570943*G0_2_2;
    A[2962] = -0.234863516113514*G0_0_0 - 0.215291556437388*G0_0_1 - 0.092966808461598*G0_0_2 - 0.215291556437386*G0_1_0 - 0.234863516113513*G0_1_1 - 0.0929668084615979*G0_1_2 - 0.0929668084615982*G0_2_0 - 0.0929668084615986*G0_2_1;
    A[2963] = -0.0521918924696666*G0_0_0 - 0.0424059126316029*G0_0_1 - 0.0260959462348345*G0_0_2 - 0.0424059126316043*G0_1_0 + 0.143527704291596*G0_1_1 - 0.0750258454251504*G0_1_2 - 0.0260959462348329*G0_2_0 - 0.0750258454251483*G0_2_1;
    A[2964] = 1.76147637085136*G0_0_0 + 0.978597983806307*G0_0_1 + 0.978597983806308*G0_0_2 + 0.978597983806307*G0_1_0 + 1.76147637085135*G0_1_1 + 0.978597983806306*G0_1_2 + 0.978597983806308*G0_2_0 + 0.978597983806306*G0_2_1 + 1.76147637085136*G0_2_2;
    A[2965] = -1.17431758056757*G0_0_0 - 0.587158790283782*G0_0_1 - 0.587158790283782*G0_0_2 - 0.587158790283784*G0_1_0 - 0.293579395141891*G0_1_1 - 0.342509294332205*G0_1_2 - 0.587158790283785*G0_2_0 - 0.342509294332204*G0_2_1 - 0.293579395141889*G0_2_2;
    A[2966] = -0.293579395141889*G0_0_0 - 0.587158790283787*G0_0_1 - 0.342509294332207*G0_0_2 - 0.587158790283783*G0_1_0 - 1.17431758056757*G0_1_1 - 0.587158790283783*G0_1_2 - 0.342509294332204*G0_2_0 - 0.587158790283786*G0_2_1 - 0.293579395141891*G0_2_2;
    A[2967] = -0.293579395141892*G0_0_0 - 0.342509294332206*G0_0_1 - 0.587158790283787*G0_0_2 - 0.342509294332205*G0_1_0 - 0.293579395141892*G0_1_1 - 0.587158790283785*G0_1_2 - 0.587158790283783*G0_2_0 - 0.587158790283784*G0_2_1 - 1.17431758056757*G0_2_2;
    A[2968] = 0.0440369092712839*G0_0_0 + 0.0440369092712839*G0_0_1 + 0.0440369092712838*G0_0_2 + 0.0440369092712836*G0_1_0 + 0.0440369092712835*G0_1_1 + 0.0440369092712835*G0_1_2 + 0.0440369092712836*G0_2_0 + 0.0440369092712835*G0_2_1 + 0.0440369092712836*G0_2_2;
    A[2969] = -0.0782878387045046*G0_0_0;
    A[2970] = 0.0440369092712839*G0_1_1;
    A[2971] = 0.0440369092712839*G0_2_2;
    A[2972] = -0.0505608958299922*G0_1_1 - 0.00260959462348341*G0_1_2 - 0.00260959462348356*G0_2_1;
    A[2973] = 0.0391439193522526*G0_1_1 + 0.0319675341376732*G0_1_2 + 0.0319675341376731*G0_2_1 + 0.026095946234835*G0_2_2;
    A[2974] = 0.0260959462348352*G0_1_1 + 0.0319675341376728*G0_1_2 + 0.0319675341376731*G0_2_1 + 0.0391439193522529*G0_2_2;
    A[2975] = -0.00260959462348395*G0_1_2 - 0.00260959462348401*G0_2_1 - 0.0505608958299933*G0_2_2;
    A[2976] = 0.096228801740954*G0_0_0 - 0.0352295274170265*G0_0_2 - 0.0352295274170274*G0_2_0;
    A[2977] = -0.156575677409011*G0_0_0 - 0.0903572138381172*G0_0_2 - 0.090357213838116*G0_2_0 - 0.1859336169232*G0_2_2;
    A[2978] = 0.0260959462348363*G0_0_0 + 0.0156575677409022*G0_0_2 + 0.0156575677409009*G0_2_0 + 0.185933616923199*G0_2_2;
    A[2979] = -0.00260959462348445*G0_0_2 - 0.00260959462348394*G0_2_0 - 0.15657567740901*G0_2_2;
    A[2980] = 0.0962288017409536*G0_0_0 - 0.0352295274170266*G0_0_1 - 0.0352295274170275*G0_1_0;
    A[2981] = -0.156575677409008*G0_0_0 - 0.0903572138381152*G0_0_1 - 0.0903572138381153*G0_1_0 - 0.185933616923198*G0_1_1;
    A[2982] = 0.0260959462348348*G0_0_0 + 0.015657567740901*G0_0_1 + 0.0156575677409006*G0_1_0 + 0.185933616923198*G0_1_1;
    A[2983] = -0.00260959462348284*G0_0_1 - 0.00260959462348346*G0_1_0 - 0.156575677409009*G0_1_1;
    A[2984] = -0.0505608958299918*G0_0_0 - 0.0505608958299922*G0_0_1 - 0.0479513012065095*G0_0_2 - 0.050560895829992*G0_1_0 - 0.0505608958299921*G0_1_1 - 0.0479513012065086*G0_1_2 - 0.0479513012065085*G0_2_0 - 0.0479513012065086*G0_2_1 - 0.0453417065830255*G0_2_2;
    A[2985] = 0.0391439193522519*G0_0_0 + 0.0391439193522525*G0_0_1 + 0.00717638521458038*G0_0_2 + 0.039143919352252*G0_1_0 + 0.0391439193522521*G0_1_1 + 0.0071763852145796*G0_1_2 + 0.00717638521457903*G0_2_0 + 0.007176385214579*G0_2_1 + 0.00130479731174192*G0_2_2;
    A[2986] = 0.0260959462348364*G0_0_0 + 0.0260959462348362*G0_0_1 - 0.00587158790283798*G0_0_2 + 0.0260959462348349*G0_1_0 + 0.026095946234835*G0_1_1 - 0.0058715879028379*G0_1_2 - 0.00587158790283747*G0_2_0 - 0.00587158790283755*G0_2_1 + 0.00130479731174188*G0_2_2;
    A[2987] = 0.00260959462348383*G0_0_2 + 0.00260959462348376*G0_1_2 + 0.00260959462348388*G0_2_0 + 0.00260959462348385*G0_2_1 - 0.0453417065830256*G0_2_2;
    A[2988] = -0.0505608958299918*G0_0_0 - 0.0479513012065093*G0_0_1 - 0.0505608958299919*G0_0_2 - 0.0479513012065082*G0_1_0 - 0.0453417065830252*G0_1_1 - 0.0479513012065084*G0_1_2 - 0.0505608958299918*G0_2_0 - 0.0479513012065087*G0_2_1 - 0.050560895829992*G0_2_2;
    A[2989] = 0.0391439193522517*G0_0_0 + 0.00717638521457971*G0_0_1 + 0.0391439193522514*G0_0_2 + 0.0071763852145789*G0_1_0 + 0.00130479731174149*G0_1_1 + 0.00717638521457898*G0_1_2 + 0.0391439193522519*G0_2_0 + 0.00717638521457939*G0_2_1 + 0.0391439193522519*G0_2_2;
    A[2990] = 0.0260959462348348*G0_0_0 - 0.00587158790283805*G0_0_1 + 0.0260959462348352*G0_0_2 - 0.00587158790283714*G0_1_0 + 0.00130479731174145*G0_1_1 - 0.00587158790283729*G0_1_2 + 0.0260959462348352*G0_2_0 - 0.0058715879028379*G0_2_1 + 0.0260959462348352*G0_2_2;
    A[2991] = 0.00260959462348414*G0_0_1 + 0.00260959462348364*G0_1_0 - 0.0453417065830252*G0_1_1 + 0.0026095946234836*G0_1_2 + 0.00260959462348363*G0_2_1;
    A[2992] = -0.151356488162042*G0_0_0 - 0.153966082785524*G0_0_1 - 0.153966082785524*G0_0_2 - 0.153966082785525*G0_1_0 - 0.156575677409008*G0_1_1 - 0.156575677409008*G0_1_2 - 0.153966082785525*G0_2_0 - 0.156575677409008*G0_2_1 - 0.156575677409008*G0_2_2;
    A[2993] = 0.18071442767623*G0_0_0 + 0.170276049182297*G0_0_1 + 0.170276049182297*G0_0_2 + 0.170276049182297*G0_1_0 + 0.185933616923198*G0_1_1 + 0.185933616923198*G0_1_2 + 0.170276049182296*G0_2_0 + 0.185933616923198*G0_2_1 + 0.185933616923198*G0_2_2;
    A[2994] = -0.161794866655975*G0_0_0 - 0.0955764030850811*G0_0_1 - 0.0955764030850812*G0_0_2 - 0.095576403085082*G0_1_0 - 0.185933616923198*G0_1_1 - 0.185933616923198*G0_1_2 - 0.0955764030850813*G0_2_0 - 0.185933616923197*G0_2_1 - 0.185933616923197*G0_2_2;
    A[2995] = 0.166687856575007*G0_0_0 + 0.0352295274170263*G0_0_1 + 0.0352295274170265*G0_0_2 + 0.0352295274170265*G0_1_0 + 0.0352295274170259*G0_2_0;
    A[2996] = -0.782878387045046*G0_0_0 - 0.316413348097373*G0_0_1 - 0.316413348097372*G0_0_2 - 0.316413348097371*G0_1_0 - 0.185933616923198*G0_1_2 - 0.316413348097371*G0_2_0 - 0.185933616923198*G0_2_1;
    A[2997] = 0.146789697570946*G0_0_0 - 0.298472385060921*G0_0_1 - 0.10275278829966*G0_0_2 - 0.298472385060927*G0_1_0 - 0.78287838704505*G0_1_1 - 0.298472385060925*G0_1_2 - 0.102752788299662*G0_2_0 - 0.298472385060923*G0_2_1;
    A[2998] = 0.0750258454251478*G0_0_1 + 0.0260959462348347*G0_0_2 + 0.0750258454251513*G0_1_0 + 0.293579395141893*G0_1_1 + 0.0587158790283791*G0_1_2 + 0.0260959462348351*G0_2_0 + 0.058715879028379*G0_2_1;
    A[2999] = 0.146789697570947*G0_0_0 - 0.102752788299661*G0_0_1 - 0.298472385060923*G0_0_2 - 0.102752788299663*G0_1_0 - 0.298472385060925*G0_1_2 - 0.298472385060926*G0_2_0 - 0.298472385060925*G0_2_1 - 0.782878387045049*G0_2_2;
    A[3000] = 0.0929668084615997*G0_0_1 + 0.0929668084615972*G0_0_2 + 0.0929668084615999*G0_1_0 - 0.0489298991903167*G0_1_1 - 0.0293579395141905*G0_1_2 + 0.0929668084616013*G0_2_0 - 0.0293579395141886*G0_2_1 - 0.0489298991903136*G0_2_2;
    A[3001] = 0.0260959462348327*G0_0_1 + 0.07502584542515*G0_0_2 + 0.0260959462348341*G0_1_0 + 0.0587158790283784*G0_1_2 + 0.0750258454251493*G0_2_0 + 0.0587158790283769*G0_2_1 + 0.293579395141892*G0_2_2;
    A[3002] = 0.195719596761263*G0_0_0 + 0.120693751336111*G0_0_1 + 0.120693751336112*G0_0_2 + 0.12069375133611*G0_1_0 + 0.0456679059109598*G0_1_1 + 0.0717638521457943*G0_1_2 + 0.12069375133611*G0_2_0 + 0.0717638521457954*G0_2_1 + 0.0456679059109599*G0_2_2;
    A[3003] = 0.146789697570948*G0_0_0 + 0.00489298991903147*G0_0_1 + 0.053822889109348*G0_0_2 + 0.0048929899190328*G0_1_0 + 0.00978597983806421*G0_1_1 + 0.00489298991903201*G0_1_2 + 0.0538228891093484*G0_2_0 + 0.00489298991903122*G0_2_1 - 0.0391439193522516*G0_2_2;
    A[3004] = -0.0750258454251517*G0_0_1 - 0.0260959462348344*G0_0_2 - 0.0750258454251515*G0_1_0 + 0.0456679059109602*G0_1_1 - 0.0260959462348362*G0_1_2 - 0.0260959462348355*G0_2_0 - 0.0260959462348343*G0_2_1 - 0.0521918924696705*G0_2_2;
    A[3005] = 0.146789697570949*G0_0_0 + 0.0538228891093476*G0_0_1 + 0.00489298991903193*G0_0_2 + 0.0538228891093475*G0_1_0 - 0.0391439193522511*G0_1_1 + 0.00489298991903289*G0_1_2 + 0.00489298991903355*G0_2_0 + 0.00489298991903381*G0_2_1 + 0.00978597983806456*G0_2_2;
    A[3006] = -0.0929668084615983*G0_0_1 - 0.0929668084616007*G0_0_2 - 0.0929668084615995*G0_1_0 - 0.0391439193522531*G0_1_1 - 0.0440369092712841*G0_1_2 - 0.0929668084616009*G0_2_0 - 0.044036909271286*G0_2_1 - 0.039143919352253*G0_2_2;
    A[3007] = -0.0260959462348371*G0_0_1 - 0.0750258454251506*G0_0_2 - 0.0260959462348336*G0_1_0 - 0.0521918924696695*G0_1_1 - 0.0260959462348351*G0_1_2 - 0.0750258454251489*G0_2_0 - 0.0260959462348353*G0_2_1 + 0.0456679059109607*G0_2_2;
    A[3008] = 0.14352770429159*G0_0_0 + 0.218553549716743*G0_0_1 + 0.185933616923197*G0_0_2 + 0.218553549716741*G0_1_0 + 0.293579395141891*G0_1_1 + 0.234863516113512*G0_1_2 + 0.185933616923198*G0_2_0 + 0.234863516113513*G0_2_1 + 0.176147637085134*G0_2_2;
    A[3009] = -0.0391439193522506*G0_0_0 - 0.484406001984121*G0_0_1 - 0.288686405222861*G0_0_2 - 0.484406001984121*G0_1_0 - 0.782878387045045*G0_1_1 - 0.484406001984121*G0_1_2 - 0.288686405222861*G0_2_0 - 0.484406001984122*G0_2_1 - 0.185933616923198*G0_2_2;
    A[3010] = -0.150051690850302*G0_0_0 + 0.316413348097374*G0_0_1 + 0.185933616923198*G0_0_2 + 0.316413348097373*G0_1_0 + 0.185933616923196*G0_1_2 + 0.185933616923199*G0_2_0 + 0.185933616923198*G0_2_1 + 0.371867233846397*G0_2_2;
    A[3011] = -0.234863516113514*G0_0_0 - 0.141896707651913*G0_0_1 - 0.0195719596761236*G0_0_2 - 0.141896707651913*G0_1_0 - 0.0489298991903147*G0_1_1 - 0.0195719596761248*G0_1_2 - 0.0195719596761241*G0_2_0 - 0.0195719596761247*G0_2_1 - 0.0391439193522512*G0_2_2;
    A[3012] = 0.352295274170272*G0_0_0 + 0.102752788299661*G0_0_1 + 0.102752788299663*G0_0_2 + 0.10275278829966*G0_1_0 + 0.298472385060924*G0_1_2 + 0.102752788299661*G0_2_0 + 0.298472385060925*G0_2_1 - 0.185933616923198*G0_2_2;
    A[3013] = -0.0521918924696727*G0_0_0 - 0.0260959462348326*G0_0_1 - 0.00978597983806497*G0_0_2 - 0.0260959462348348*G0_1_0 - 0.0587158790283791*G0_1_2 - 0.00978597983806334*G0_2_0 - 0.0587158790283767*G0_2_1 + 0.176147637085135*G0_2_2;
    A[3014] = 0.143527704291592*G0_0_0 + 0.185933616923197*G0_0_1 + 0.218553549716742*G0_0_2 + 0.185933616923197*G0_1_0 + 0.176147637085134*G0_1_1 + 0.234863516113512*G0_1_2 + 0.218553549716741*G0_2_0 + 0.234863516113512*G0_2_1 + 0.293579395141892*G0_2_2;
    A[3015] = -0.039143919352252*G0_0_0 - 0.28868640522286*G0_0_1 - 0.48440600198412*G0_0_2 - 0.288686405222859*G0_1_0 - 0.185933616923198*G0_1_1 - 0.484406001984122*G0_1_2 - 0.484406001984122*G0_2_0 - 0.484406001984123*G0_2_1 - 0.782878387045047*G0_2_2;
    A[3016] = -0.1500516908503*G0_0_0 + 0.185933616923196*G0_0_1 + 0.316413348097372*G0_0_2 + 0.185933616923199*G0_1_0 + 0.371867233846396*G0_1_1 + 0.185933616923199*G0_1_2 + 0.316413348097373*G0_2_0 + 0.185933616923198*G0_2_1;
    A[3017] = -0.234863516113512*G0_0_0 - 0.0195719596761246*G0_0_1 - 0.141896707651912*G0_0_2 - 0.0195719596761247*G0_1_0 - 0.0391439193522503*G0_1_1 - 0.0195719596761245*G0_1_2 - 0.141896707651914*G0_2_0 - 0.0195719596761247*G0_2_1 - 0.0489298991903139*G0_2_2;
    A[3018] = 0.352295274170268*G0_0_0 + 0.102752788299662*G0_0_1 + 0.10275278829966*G0_0_2 + 0.102752788299661*G0_1_0 - 0.185933616923198*G0_1_1 + 0.298472385060925*G0_1_2 + 0.102752788299662*G0_2_0 + 0.298472385060925*G0_2_1;
    A[3019] = -0.0521918924696701*G0_0_0 - 0.00978597983806514*G0_0_1 - 0.0260959462348348*G0_0_2 - 0.00978597983806366*G0_1_0 + 0.176147637085134*G0_1_1 - 0.0587158790283786*G0_1_2 - 0.026095946234835*G0_2_0 - 0.0587158790283794*G0_2_1;
    A[3020] = -1.17431758056757*G0_0_0 - 0.587158790283784*G0_0_1 - 0.587158790283785*G0_0_2 - 0.587158790283782*G0_1_0 - 0.293579395141891*G0_1_1 - 0.342509294332204*G0_1_2 - 0.587158790283782*G0_2_0 - 0.342509294332205*G0_2_1 - 0.293579395141889*G0_2_2;
    A[3021] = 1.76147637085135*G0_0_0 + 0.782878387045043*G0_0_1 + 0.782878387045042*G0_0_2 + 0.782878387045043*G0_1_0 + 1.5657567740901*G0_1_1 + 0.782878387045045*G0_1_2 + 0.782878387045042*G0_2_0 + 0.782878387045045*G0_2_1 + 1.56575677409009*G0_2_2;
    A[3022] = -0.293579395141893*G0_0_0 + 0.293579395141894*G0_0_1 + 0.0489298991903164*G0_0_2 + 0.293579395141895*G0_1_0 - 0.293579395141894*G0_1_1 + 0.0489298991903164*G0_1_2 + 0.0489298991903145*G0_2_0 + 0.0489298991903122*G0_2_1 + 0.0978597983806289*G0_2_2;
    A[3023] = -0.293579395141897*G0_0_0 + 0.0489298991903141*G0_0_1 + 0.293579395141893*G0_0_2 + 0.0489298991903156*G0_1_0 + 0.097859798380631*G0_1_1 + 0.0489298991903142*G0_1_2 + 0.293579395141892*G0_2_0 + 0.0489298991903137*G0_2_1 - 0.293579395141894*G0_2_2;
    A[3024] = 0.0440369092712839*G0_0_0 + 0.0440369092712839*G0_0_1 + 0.0440369092712839*G0_0_2 + 0.0440369092712838*G0_1_0 + 0.0440369092712839*G0_1_1 + 0.0440369092712839*G0_1_2 + 0.0440369092712839*G0_2_0 + 0.0440369092712839*G0_2_1 + 0.0440369092712839*G0_2_2;
    A[3025] = 0.0440369092712837*G0_0_0;
    A[3026] = -0.0782878387045047*G0_1_1;
    A[3027] = 0.0440369092712839*G0_2_2;
    A[3028] = 0.0962288017409547*G0_1_1 - 0.0352295274170257*G0_1_2 - 0.0352295274170272*G0_2_1;
    A[3029] = -0.156575677409011*G0_1_1 - 0.0903572138381182*G0_1_2 - 0.090357213838116*G0_2_1 - 0.1859336169232*G0_2_2;
    A[3030] = 0.0260959462348353*G0_1_1 + 0.0156575677409026*G0_1_2 + 0.0156575677409005*G0_2_1 + 0.185933616923199*G0_2_2;
    A[3031] = -0.00260959462348471*G0_1_2 - 0.00260959462348353*G0_2_1 - 0.15657567740901*G0_2_2;
    A[3032] = -0.0505608958299925*G0_0_0 - 0.00260959462348395*G0_0_2 - 0.00260959462348265*G0_2_0;
    A[3033] = 0.0391439193522521*G0_0_0 + 0.0319675341376728*G0_0_2 + 0.0319675341376719*G0_2_0 + 0.0260959462348351*G0_2_2;
    A[3034] = 0.0260959462348357*G0_0_0 + 0.0319675341376734*G0_0_2 + 0.0319675341376733*G0_2_0 + 0.0391439193522523*G0_2_2;
    A[3035] = -0.00260959462348422*G0_0_2 - 0.00260959462348339*G0_2_0 - 0.0505608958299925*G0_2_2;
    A[3036] = -0.156575677409009*G0_0_0 - 0.00260959462348398*G0_0_1 - 0.00260959462348405*G0_1_0;
    A[3037] = 0.185933616923199*G0_0_0 + 0.0156575677409014*G0_0_1 + 0.0156575677409019*G0_1_0 + 0.0260959462348351*G0_1_1;
    A[3038] = -0.185933616923199*G0_0_0 - 0.0903572138381161*G0_0_1 - 0.0903572138381157*G0_1_0 - 0.156575677409009*G0_1_1;
    A[3039] = -0.0352295274170262*G0_0_1 - 0.0352295274170257*G0_1_0 + 0.0962288017409552*G0_1_1;
    A[3040] = -0.0505608958299923*G0_0_0 - 0.0505608958299923*G0_0_1 - 0.0479513012065092*G0_0_2 - 0.0505608958299926*G0_1_0 - 0.0505608958299928*G0_1_1 - 0.0479513012065095*G0_1_2 - 0.0479513012065092*G0_2_0 - 0.0479513012065091*G0_2_1 - 0.0453417065830258*G0_2_2;
    A[3041] = 0.0391439193522535*G0_0_0 + 0.0391439193522533*G0_0_1 + 0.00717638521458018*G0_0_2 + 0.0391439193522541*G0_1_0 + 0.0391439193522545*G0_1_1 + 0.00717638521458061*G0_1_2 + 0.00717638521458032*G0_2_0 + 0.00717638521458043*G0_2_1 + 0.0013047973117418*G0_2_2;
    A[3042] = 0.0260959462348343*G0_0_0 + 0.0260959462348344*G0_0_1 - 0.0058715879028385*G0_0_2 + 0.0260959462348365*G0_1_0 + 0.0260959462348361*G0_1_1 - 0.00587158790283832*G0_1_2 - 0.00587158790283751*G0_2_0 - 0.00587158790283765*G0_2_1 + 0.00130479731174208*G0_2_2;
    A[3043] = 0.00260959462348407*G0_0_2 + 0.0026095946234838*G0_1_2 + 0.00260959462348334*G0_2_0 + 0.00260959462348349*G0_2_1 - 0.0453417065830262*G0_2_2;
    A[3044] = -0.156575677409008*G0_0_0 - 0.153966082785526*G0_0_1 - 0.156575677409008*G0_0_2 - 0.153966082785525*G0_1_0 - 0.151356488162041*G0_1_1 - 0.153966082785525*G0_1_2 - 0.156575677409009*G0_2_0 - 0.153966082785526*G0_2_1 - 0.156575677409009*G0_2_2;
    A[3045] = 0.185933616923197*G0_0_0 + 0.170276049182298*G0_0_1 + 0.185933616923197*G0_0_2 + 0.170276049182295*G0_1_0 + 0.18071442767623*G0_1_1 + 0.170276049182295*G0_1_2 + 0.185933616923197*G0_2_0 + 0.170276049182297*G0_2_1 + 0.185933616923198*G0_2_2;
    A[3046] = -0.185933616923197*G0_0_0 - 0.0955764030850831*G0_0_1 - 0.185933616923197*G0_0_2 - 0.0955764030850799*G0_1_0 - 0.161794866655974*G0_1_1 - 0.0955764030850797*G0_1_2 - 0.185933616923197*G0_2_0 - 0.0955764030850822*G0_2_1 - 0.185933616923197*G0_2_2;
    A[3047] = 0.0352295274170272*G0_0_1 + 0.0352295274170259*G0_1_0 + 0.166687856575007*G0_1_1 + 0.0352295274170259*G0_1_2 + 0.0352295274170266*G0_2_1;
    A[3048] = -0.0453417065830256*G0_0_0 - 0.0479513012065087*G0_0_1 - 0.0479513012065088*G0_0_2 - 0.047951301206509*G0_1_0 - 0.0505608958299924*G0_1_1 - 0.0505608958299923*G0_1_2 - 0.0479513012065095*G0_2_0 - 0.050560895829993*G0_2_1 - 0.0505608958299932*G0_2_2;
    A[3049] = 0.00130479731174195*G0_0_0 + 0.00717638521457962*G0_0_1 + 0.00717638521457961*G0_0_2 + 0.00717638521457943*G0_1_0 + 0.0391439193522531*G0_1_1 + 0.039143919352253*G0_1_2 + 0.00717638521458053*G0_2_0 + 0.0391439193522529*G0_2_1 + 0.0391439193522529*G0_2_2;
    A[3050] = 0.00130479731174105*G0_0_0 - 0.00587158790283838*G0_0_1 - 0.00587158790283837*G0_0_2 - 0.00587158790283776*G0_1_0 + 0.0260959462348354*G0_1_1 + 0.0260959462348353*G0_1_2 - 0.00587158790283919*G0_2_0 + 0.0260959462348342*G0_2_1 + 0.0260959462348343*G0_2_2;
    A[3051] = -0.045341706583025*G0_0_0 + 0.00260959462348381*G0_0_1 + 0.00260959462348383*G0_0_2 + 0.00260959462348357*G0_1_0 + 0.00260959462348476*G0_2_0;
    A[3052] = 0.293579395141892*G0_0_0 + 0.0750258454251501*G0_0_1 + 0.0587158790283787*G0_0_2 + 0.0750258454251484*G0_1_0 + 0.0260959462348331*G0_1_2 + 0.0587158790283775*G0_2_0 + 0.0260959462348354*G0_2_1;
    A[3053] = -0.78287838704505*G0_0_0 - 0.298472385060927*G0_0_1 - 0.298472385060926*G0_0_2 - 0.298472385060924*G0_1_0 + 0.146789697570948*G0_1_1 - 0.102752788299661*G0_1_2 - 0.298472385060924*G0_2_0 - 0.102752788299663*G0_2_1;
    A[3054] = -0.316413348097372*G0_0_1 - 0.185933616923196*G0_0_2 - 0.316413348097374*G0_1_0 - 0.78287838704505*G0_1_1 - 0.316413348097372*G0_1_2 - 0.185933616923198*G0_2_0 - 0.316413348097372*G0_2_1;
    A[3055] = -0.0489298991903142*G0_0_0 + 0.0929668084616004*G0_0_1 - 0.0293579395141901*G0_0_2 + 0.0929668084616023*G0_1_0 + 0.0929668084616006*G0_1_2 - 0.029357939514187*G0_2_0 + 0.0929668084616*G0_2_1 - 0.0489298991903142*G0_2_2;
    A[3056] = -0.102752788299663*G0_0_1 - 0.298472385060925*G0_0_2 - 0.102752788299663*G0_1_0 + 0.146789697570948*G0_1_1 - 0.298472385060927*G0_1_2 - 0.298472385060925*G0_2_0 - 0.298472385060924*G0_2_1 - 0.782878387045048*G0_2_2;
    A[3057] = 0.0260959462348342*G0_0_1 + 0.058715879028379*G0_0_2 + 0.0260959462348333*G0_1_0 + 0.0750258454251507*G0_1_2 + 0.0587158790283781*G0_2_0 + 0.0750258454251507*G0_2_1 + 0.293579395141894*G0_2_2;
    A[3058] = 0.293579395141891*G0_0_0 + 0.21855354971674*G0_0_1 + 0.234863516113513*G0_0_2 + 0.218553549716744*G0_1_0 + 0.143527704291592*G0_1_1 + 0.1859336169232*G0_1_2 + 0.234863516113514*G0_2_0 + 0.185933616923198*G0_2_1 + 0.176147637085136*G0_2_2;
    A[3059] = -0.782878387045047*G0_0_0 - 0.48440600198412*G0_0_1 - 0.484406001984122*G0_0_2 - 0.484406001984121*G0_1_0 - 0.0391439193522498*G0_1_1 - 0.288686405222859*G0_1_2 - 0.484406001984122*G0_2_0 - 0.28868640522286*G0_2_1 - 0.185933616923198*G0_2_2;
    A[3060] = 0.316413348097371*G0_0_1 + 0.185933616923197*G0_0_2 + 0.316413348097374*G0_1_0 - 0.150051690850305*G0_1_1 + 0.185933616923197*G0_1_2 + 0.185933616923199*G0_2_0 + 0.185933616923198*G0_2_1 + 0.371867233846397*G0_2_2;
    A[3061] = -0.0489298991903143*G0_0_0 - 0.141896707651914*G0_0_1 - 0.0195719596761249*G0_0_2 - 0.141896707651912*G0_1_0 - 0.234863516113516*G0_1_1 - 0.0195719596761268*G0_1_2 - 0.019571959676125*G0_2_0 - 0.0195719596761271*G0_2_1 - 0.0391439193522526*G0_2_2;
    A[3062] = 0.102752788299662*G0_0_1 + 0.298472385060925*G0_0_2 + 0.102752788299663*G0_1_0 + 0.352295274170274*G0_1_1 + 0.102752788299662*G0_1_2 + 0.298472385060925*G0_2_0 + 0.102752788299662*G0_2_1 - 0.185933616923199*G0_2_2;
    A[3063] = -0.0260959462348339*G0_0_1 - 0.0587158790283787*G0_0_2 - 0.0260959462348329*G0_1_0 - 0.0521918924696716*G0_1_1 - 0.00978597983806335*G0_1_2 - 0.0587158790283775*G0_2_0 - 0.00978597983806234*G0_2_1 + 0.176147637085136*G0_2_2;
    A[3064] = 0.04566790591096*G0_0_0 + 0.120693751336111*G0_0_1 + 0.0717638521457957*G0_0_2 + 0.120693751336113*G0_1_0 + 0.195719596761266*G0_1_1 + 0.120693751336113*G0_1_2 + 0.0717638521457961*G0_2_0 + 0.120693751336111*G0_2_1 + 0.0456679059109622*G0_2_2;
    A[3065] = 0.0097859798380637*G0_0_0 + 0.00489298991903146*G0_0_1 + 0.00489298991903167*G0_0_2 + 0.0048929899190308*G0_1_0 + 0.146789697570944*G0_1_1 + 0.0538228891093444*G0_1_2 + 0.00489298991903195*G0_2_0 + 0.0538228891093468*G0_2_1 - 0.0391439193522527*G0_2_2;
    A[3066] = 0.0456679059109611*G0_0_0 - 0.0750258454251496*G0_0_1 - 0.0260959462348344*G0_0_2 - 0.0750258454251507*G0_1_0 - 0.0260959462348344*G0_1_2 - 0.026095946234836*G0_2_0 - 0.0260959462348352*G0_2_1 - 0.0521918924696698*G0_2_2;
    A[3067] = -0.0391439193522528*G0_0_0 + 0.0538228891093472*G0_0_1 + 0.00489298991903137*G0_0_2 + 0.0538228891093433*G0_1_0 + 0.146789697570948*G0_1_1 + 0.00489298991903136*G0_1_2 + 0.00489298991903027*G0_2_0 + 0.00489298991903198*G0_2_1 + 0.00978597983806217*G0_2_2;
    A[3068] = -0.0391439193522524*G0_0_0 - 0.0929668084616003*G0_0_1 - 0.0440369092712849*G0_0_2 - 0.0929668084615958*G0_1_0 - 0.0929668084615975*G0_1_2 - 0.0440369092712827*G0_2_0 - 0.0929668084616002*G0_2_1 - 0.0391439193522523*G0_2_2;
    A[3069] = -0.0521918924696696*G0_0_0 - 0.0260959462348344*G0_0_1 - 0.0260959462348341*G0_0_2 - 0.0260959462348371*G0_1_0 - 0.0750258454251515*G0_1_2 - 0.0260959462348354*G0_2_0 - 0.0750258454251508*G0_2_1 + 0.0456679059109613*G0_2_2;
    A[3070] = 0.176147637085133*G0_0_0 + 0.185933616923196*G0_0_1 + 0.234863516113511*G0_0_2 + 0.185933616923198*G0_1_0 + 0.14352770429159*G0_1_1 + 0.218553549716743*G0_1_2 + 0.234863516113513*G0_2_0 + 0.218553549716741*G0_2_1 + 0.293579395141891*G0_2_2;
    A[3071] = -0.0391439193522496*G0_0_0 - 0.0195719596761243*G0_0_1 - 0.0195719596761246*G0_0_2 - 0.0195719596761264*G0_1_0 - 0.234863516113513*G0_1_1 - 0.141896707651913*G0_1_2 - 0.0195719596761249*G0_2_0 - 0.141896707651914*G0_2_1 - 0.0489298991903144*G0_2_2;
    A[3072] = 0.176147637085134*G0_0_0 - 0.00978597983806256*G0_0_1 - 0.0587158790283778*G0_0_2 - 0.0097859798380631*G0_1_0 - 0.0521918924696702*G0_1_1 - 0.0260959462348325*G0_1_2 - 0.0587158790283793*G0_2_0 - 0.0260959462348342*G0_2_1;
    A[3073] = -0.185933616923196*G0_0_0 - 0.288686405222857*G0_0_1 - 0.48440600198412*G0_0_2 - 0.288686405222858*G0_1_0 - 0.0391439193522487*G0_1_1 - 0.484406001984118*G0_1_2 - 0.484406001984121*G0_2_0 - 0.484406001984121*G0_2_1 - 0.782878387045044*G0_2_2;
    A[3074] = -0.185933616923199*G0_0_0 + 0.102752788299661*G0_0_1 + 0.298472385060926*G0_0_2 + 0.10275278829966*G0_1_0 + 0.35229527417027*G0_1_1 + 0.102752788299661*G0_1_2 + 0.298472385060924*G0_2_0 + 0.102752788299661*G0_2_1;
    A[3075] = 0.371867233846395*G0_0_0 + 0.185933616923196*G0_0_1 + 0.185933616923197*G0_0_2 + 0.185933616923195*G0_1_0 - 0.150051690850305*G0_1_1 + 0.316413348097372*G0_1_2 + 0.185933616923197*G0_2_0 + 0.316413348097372*G0_2_1;
    A[3076] = -0.293579395141889*G0_0_0 - 0.587158790283783*G0_0_1 - 0.342509294332204*G0_0_2 - 0.587158790283787*G0_1_0 - 1.17431758056757*G0_1_1 - 0.587158790283786*G0_1_2 - 0.342509294332207*G0_2_0 - 0.587158790283783*G0_2_1 - 0.293579395141891*G0_2_2;
    A[3077] = -0.293579395141893*G0_0_0 + 0.293579395141895*G0_0_1 + 0.0489298991903145*G0_0_2 + 0.293579395141894*G0_1_0 - 0.293579395141894*G0_1_1 + 0.0489298991903121*G0_1_2 + 0.0489298991903164*G0_2_0 + 0.0489298991903164*G0_2_1 + 0.0978597983806289*G0_2_2;
    A[3078] = 1.5657567740901*G0_0_0 + 0.782878387045045*G0_0_1 + 0.782878387045046*G0_0_2 + 0.782878387045045*G0_1_0 + 1.76147637085136*G0_1_1 + 0.782878387045045*G0_1_2 + 0.782878387045046*G0_2_0 + 0.782878387045045*G0_2_1 + 1.56575677409009*G0_2_2;
    A[3079] = 0.0978597983806286*G0_0_0 + 0.0489298991903156*G0_0_1 + 0.0489298991903138*G0_0_2 + 0.0489298991903101*G0_1_0 - 0.293579395141894*G0_1_1 + 0.293579395141892*G0_1_2 + 0.0489298991903121*G0_2_0 + 0.293579395141893*G0_2_1 - 0.293579395141893*G0_2_2;
    A[3080] = 0.0440369092712837*G0_0_0 + 0.0440369092712838*G0_0_1 + 0.0440369092712838*G0_0_2 + 0.0440369092712839*G0_1_0 + 0.0440369092712839*G0_1_1 + 0.0440369092712839*G0_1_2 + 0.0440369092712839*G0_2_0 + 0.0440369092712839*G0_2_1 + 0.0440369092712838*G0_2_2;
    A[3081] = 0.044036909271284*G0_0_0;
    A[3082] = 0.0440369092712835*G0_1_1;
    A[3083] = -0.0782878387045049*G0_2_2;
    A[3084] = -0.15657567740901*G0_1_1 - 0.00260959462348456*G0_1_2 - 0.00260959462348416*G0_2_1;
    A[3085] = 0.185933616923199*G0_1_1 + 0.0156575677409021*G0_1_2 + 0.0156575677409014*G0_2_1 + 0.0260959462348356*G0_2_2;
    A[3086] = -0.185933616923198*G0_1_1 - 0.0903572138381166*G0_1_2 - 0.0903572138381159*G0_2_1 - 0.156575677409009*G0_2_2;
    A[3087] = -0.0352295274170273*G0_1_2 - 0.0352295274170277*G0_2_1 + 0.096228801740953*G0_2_2;
    A[3088] = -0.15657567740901*G0_0_0 - 0.00260959462348361*G0_0_2 - 0.0026095946234842*G0_2_0;
    A[3089] = 0.1859336169232*G0_0_0 + 0.0156575677409017*G0_0_2 + 0.015657567740901*G0_2_0 + 0.0260959462348346*G0_2_2;
    A[3090] = -0.185933616923201*G0_0_0 - 0.0903572138381181*G0_0_2 - 0.0903572138381173*G0_2_0 - 0.156575677409011*G0_2_2;
    A[3091] = -0.0352295274170271*G0_0_2 - 0.0352295274170277*G0_2_0 + 0.096228801740953*G0_2_2;
    A[3092] = -0.0505608958299926*G0_0_0 - 0.00260959462348368*G0_0_1 - 0.00260959462348252*G0_1_0;
    A[3093] = 0.0391439193522517*G0_0_0 + 0.0319675341376731*G0_0_1 + 0.0319675341376712*G0_1_0 + 0.0260959462348357*G0_1_1;
    A[3094] = 0.0260959462348355*G0_0_0 + 0.0319675341376725*G0_0_1 + 0.0319675341376738*G0_1_0 + 0.0391439193522515*G0_1_1;
    A[3095] = -0.00260959462348394*G0_0_1 - 0.00260959462348441*G0_1_0 - 0.0505608958299925*G0_1_1;
    A[3096] = -0.15657567740901*G0_0_0 - 0.15657567740901*G0_0_1 - 0.153966082785526*G0_0_2 - 0.156575677409009*G0_1_0 - 0.15657567740901*G0_1_1 - 0.153966082785526*G0_1_2 - 0.153966082785527*G0_2_0 - 0.153966082785527*G0_2_1 - 0.151356488162043*G0_2_2;
    A[3097] = 0.1859336169232*G0_0_0 + 0.1859336169232*G0_0_1 + 0.170276049182299*G0_0_2 + 0.185933616923199*G0_1_0 + 0.185933616923199*G0_1_1 + 0.170276049182298*G0_1_2 + 0.170276049182299*G0_2_0 + 0.170276049182299*G0_2_1 + 0.180714427676233*G0_2_2;
    A[3098] = -0.1859336169232*G0_0_0 - 0.1859336169232*G0_0_1 - 0.0955764030850832*G0_0_2 - 0.185933616923199*G0_1_0 - 0.185933616923199*G0_1_1 - 0.0955764030850828*G0_1_2 - 0.0955764030850827*G0_2_0 - 0.0955764030850827*G0_2_1 - 0.161794866655976*G0_2_2;
    A[3099] = 0.0352295274170277*G0_0_2 + 0.0352295274170273*G0_1_2 + 0.0352295274170277*G0_2_0 + 0.0352295274170275*G0_2_1 + 0.166687856575008*G0_2_2;
    A[3100] = -0.0505608958299924*G0_0_0 - 0.0479513012065087*G0_0_1 - 0.0505608958299927*G0_0_2 - 0.0479513012065093*G0_1_0 - 0.045341706583026*G0_1_1 - 0.0479513012065097*G0_1_2 - 0.0505608958299928*G0_2_0 - 0.0479513012065089*G0_2_1 - 0.050560895829993*G0_2_2;
    A[3101] = 0.0391439193522523*G0_0_0 + 0.00717638521457944*G0_0_1 + 0.0391439193522526*G0_0_2 + 0.00717638521458048*G0_1_0 + 0.00130479731174266*G0_1_1 + 0.00717638521458085*G0_1_2 + 0.0391439193522529*G0_2_0 + 0.00717638521457929*G0_2_1 + 0.0391439193522529*G0_2_2;
    A[3102] = 0.0260959462348348*G0_0_0 - 0.00587158790283801*G0_0_1 + 0.0260959462348344*G0_0_2 - 0.00587158790283888*G0_1_0 + 0.0013047973117405*G0_1_1 - 0.00587158790283893*G0_1_2 + 0.0260959462348334*G0_2_0 - 0.00587158790283794*G0_2_1 + 0.0260959462348333*G0_2_2;
    A[3103] = 0.00260959462348351*G0_0_1 + 0.00260959462348449*G0_1_0 - 0.0453417065830247*G0_1_1 + 0.00260959462348447*G0_1_2 + 0.00260959462348413*G0_2_1;
    A[3104] = -0.0453417065830253*G0_0_0 - 0.0479513012065093*G0_0_1 - 0.0479513012065095*G0_0_2 - 0.0479513012065091*G0_1_0 - 0.050560895829993*G0_1_1 - 0.0505608958299932*G0_1_2 - 0.0479513012065088*G0_2_0 - 0.0505608958299934*G0_2_1 - 0.0505608958299933*G0_2_2;
    A[3105] = 0.00130479731174142*G0_0_0 + 0.00717638521457974*G0_0_1 + 0.00717638521457977*G0_0_2 + 0.00717638521457993*G0_1_0 + 0.0391439193522525*G0_1_1 + 0.0391439193522527*G0_1_2 + 0.00717638521457915*G0_2_0 + 0.039143919352253*G0_2_1 + 0.039143919352253*G0_2_2;
    A[3106] = 0.00130479731174228*G0_0_0 - 0.00587158790283728*G0_0_1 - 0.00587158790283722*G0_0_2 - 0.00587158790283863*G0_1_0 + 0.0260959462348342*G0_1_1 + 0.0260959462348344*G0_1_2 - 0.00587158790283757*G0_2_0 + 0.0260959462348349*G0_2_1 + 0.0260959462348349*G0_2_2;
    A[3107] = -0.045341706583026*G0_0_0 + 0.00260959462348371*G0_0_1 + 0.00260959462348372*G0_0_2 + 0.00260959462348448*G0_1_0 + 0.00260959462348354*G0_2_0;
    A[3108] = 0.293579395141895*G0_0_0 + 0.0587158790283802*G0_0_1 + 0.0750258454251514*G0_0_2 + 0.0587158790283793*G0_1_0 + 0.026095946234836*G0_1_2 + 0.0750258454251503*G0_2_0 + 0.0260959462348334*G0_2_1;
    A[3109] = -0.0489298991903142*G0_0_0 - 0.0293579395141895*G0_0_1 + 0.0929668084615992*G0_0_2 - 0.0293579395141891*G0_1_0 - 0.048929899190318*G0_1_1 + 0.0929668084615978*G0_1_2 + 0.0929668084616001*G0_2_0 + 0.0929668084616006*G0_2_1;
    A[3110] = 0.0587158790283808*G0_0_1 + 0.0260959462348343*G0_0_2 + 0.0587158790283791*G0_1_0 + 0.293579395141895*G0_1_1 + 0.0750258454251511*G0_1_2 + 0.0260959462348332*G0_2_0 + 0.0750258454251494*G0_2_1;
    A[3111] = -0.782878387045054*G0_0_0 - 0.298472385060927*G0_0_1 - 0.298472385060925*G0_0_2 - 0.298472385060925*G0_1_0 - 0.102752788299662*G0_1_2 - 0.298472385060928*G0_2_0 - 0.102752788299665*G0_2_1 + 0.146789697570945*G0_2_2;
    A[3112] = -0.298472385060924*G0_0_1 - 0.102752788299659*G0_0_2 - 0.298472385060923*G0_1_0 - 0.782878387045047*G0_1_1 - 0.298472385060922*G0_1_2 - 0.102752788299662*G0_2_0 - 0.298472385060925*G0_2_1 + 0.146789697570946*G0_2_2;
    A[3113] = -0.185933616923203*G0_0_1 - 0.316413348097379*G0_0_2 - 0.185933616923201*G0_1_0 - 0.316413348097376*G0_1_2 - 0.316413348097376*G0_2_0 - 0.316413348097377*G0_2_1 - 0.782878387045052*G0_2_2;
    A[3114] = 0.293579395141892*G0_0_0 + 0.234863516113514*G0_0_1 + 0.218553549716742*G0_0_2 + 0.234863516113513*G0_1_0 + 0.176147637085136*G0_1_1 + 0.185933616923199*G0_1_2 + 0.218553549716744*G0_2_0 + 0.1859336169232*G0_2_1 + 0.143527704291593*G0_2_2;
    A[3115] = -0.0489298991903142*G0_0_0 - 0.0195719596761257*G0_0_1 - 0.141896707651914*G0_0_2 - 0.0195719596761265*G0_1_0 - 0.0391439193522525*G0_1_1 - 0.0195719596761272*G0_1_2 - 0.141896707651914*G0_2_0 - 0.0195719596761269*G0_2_1 - 0.234863516113513*G0_2_2;
    A[3116] = -0.0587158790283769*G0_0_1 - 0.026095946234835*G0_0_2 - 0.0587158790283787*G0_1_0 + 0.176147637085137*G0_1_1 - 0.00978597983806275*G0_1_2 - 0.0260959462348331*G0_2_0 - 0.00978597983806207*G0_2_1 - 0.0521918924696687*G0_2_2;
    A[3117] = -0.782878387045047*G0_0_0 - 0.484406001984123*G0_0_1 - 0.484406001984123*G0_0_2 - 0.484406001984122*G0_1_0 - 0.185933616923198*G0_1_1 - 0.28868640522286*G0_1_2 - 0.484406001984123*G0_2_0 - 0.288686405222862*G0_2_1 - 0.0391439193522531*G0_2_2;
    A[3118] = 0.298472385060921*G0_0_1 + 0.102752788299661*G0_0_2 + 0.298472385060923*G0_1_0 - 0.1859336169232*G0_1_1 + 0.102752788299663*G0_1_2 + 0.102752788299662*G0_2_0 + 0.102752788299661*G0_2_1 + 0.352295274170269*G0_2_2;
    A[3119] = 0.1859336169232*G0_0_1 + 0.316413348097374*G0_0_2 + 0.185933616923201*G0_1_0 + 0.371867233846398*G0_1_1 + 0.185933616923198*G0_1_2 + 0.316413348097377*G0_2_0 + 0.185933616923199*G0_2_1 - 0.150051690850299*G0_2_2;
    A[3120] = 0.176147637085136*G0_0_0 + 0.234863516113514*G0_0_1 + 0.185933616923199*G0_0_2 + 0.234863516113513*G0_1_0 + 0.293579395141891*G0_1_1 + 0.218553549716742*G0_1_2 + 0.1859336169232*G0_2_0 + 0.218553549716743*G0_2_1 + 0.143527704291593*G0_2_2;
    A[3121] = -0.0391439193522524*G0_0_0 - 0.0195719596761268*G0_0_1 - 0.0195719596761275*G0_0_2 - 0.019571959676125*G0_1_0 - 0.0489298991903129*G0_1_1 - 0.141896707651914*G0_1_2 - 0.0195719596761258*G0_2_0 - 0.141896707651915*G0_2_1 - 0.234863516113515*G0_2_2;
    A[3122] = 0.176147637085137*G0_0_0 - 0.0587158790283792*G0_0_1 - 0.00978597983806447*G0_0_2 - 0.0587158790283794*G0_1_0 - 0.0260959462348363*G0_1_2 - 0.00978597983806269*G0_2_0 - 0.0260959462348329*G0_2_1 - 0.0521918924696696*G0_2_2;
    A[3123] = -0.185933616923198*G0_0_0 - 0.484406001984123*G0_0_1 - 0.288686405222861*G0_0_2 - 0.484406001984122*G0_1_0 - 0.782878387045047*G0_1_1 - 0.484406001984123*G0_1_2 - 0.288686405222863*G0_2_0 - 0.484406001984125*G0_2_1 - 0.0391439193522521*G0_2_2;
    A[3124] = -0.185933616923201*G0_0_0 + 0.298472385060926*G0_0_1 + 0.102752788299666*G0_0_2 + 0.298472385060922*G0_1_0 + 0.102752788299662*G0_1_2 + 0.102752788299663*G0_2_0 + 0.102752788299664*G0_2_1 + 0.352295274170274*G0_2_2;
    A[3125] = 0.3718672338464*G0_0_0 + 0.185933616923203*G0_0_1 + 0.185933616923198*G0_0_2 + 0.185933616923199*G0_1_0 + 0.316413348097373*G0_1_2 + 0.1859336169232*G0_2_0 + 0.316413348097377*G0_2_1 - 0.150051690850301*G0_2_2;
    A[3126] = 0.0456679059109605*G0_0_0 + 0.071763852145796*G0_0_1 + 0.120693751336111*G0_0_2 + 0.0717638521457955*G0_1_0 + 0.0456679059109619*G0_1_1 + 0.120693751336111*G0_1_2 + 0.120693751336112*G0_2_0 + 0.120693751336113*G0_2_1 + 0.195719596761263*G0_2_2;
    A[3127] = 0.00978597983806363*G0_0_0 + 0.00489298991903078*G0_0_1 + 0.00489298991903156*G0_0_2 + 0.0048929899190326*G0_1_0 - 0.0391439193522523*G0_1_1 + 0.0538228891093476*G0_1_2 + 0.00489298991903174*G0_2_0 + 0.0538228891093478*G0_2_1 + 0.146789697570948*G0_2_2;
    A[3128] = 0.0456679059109609*G0_0_0 - 0.0260959462348355*G0_0_1 - 0.0750258454251506*G0_0_2 - 0.0260959462348361*G0_1_0 - 0.0521918924696701*G0_1_1 - 0.0260959462348356*G0_1_2 - 0.0750258454251507*G0_2_0 - 0.0260959462348359*G0_2_1;
    A[3129] = -0.0391439193522526*G0_0_0 + 0.0048929899190314*G0_0_1 + 0.0538228891093471*G0_0_2 + 0.00489298991903024*G0_1_0 + 0.00978597983806222*G0_1_1 + 0.00489298991903056*G0_1_2 + 0.0538228891093478*G0_2_0 + 0.00489298991903043*G0_2_1 + 0.146789697570946*G0_2_2;
    A[3130] = -0.0391439193522521*G0_0_0 - 0.0440369092712834*G0_0_1 - 0.0929668084615995*G0_0_2 - 0.0440369092712822*G0_1_0 - 0.0391439193522519*G0_1_1 - 0.0929668084615985*G0_1_2 - 0.0929668084616009*G0_2_0 - 0.0929668084616003*G0_2_1;
    A[3131] = -0.0521918924696696*G0_0_0 - 0.026095946234834*G0_0_1 - 0.0260959462348343*G0_0_2 - 0.0260959462348348*G0_1_0 + 0.0456679059109614*G0_1_1 - 0.0750258454251507*G0_1_2 - 0.0260959462348343*G0_2_0 - 0.0750258454251498*G0_2_1;
    A[3132] = -0.293579395141892*G0_0_0 - 0.342509294332205*G0_0_1 - 0.587158790283783*G0_0_2 - 0.342509294332206*G0_1_0 - 0.293579395141892*G0_1_1 - 0.587158790283784*G0_1_2 - 0.587158790283787*G0_2_0 - 0.587158790283785*G0_2_1 - 1.17431758056757*G0_2_2;
    A[3133] = -0.293579395141897*G0_0_0 + 0.0489298991903156*G0_0_1 + 0.293579395141892*G0_0_2 + 0.048929899190314*G0_1_0 + 0.0978597983806311*G0_1_1 + 0.0489298991903137*G0_1_2 + 0.293579395141893*G0_2_0 + 0.0489298991903142*G0_2_1 - 0.293579395141894*G0_2_2;
    A[3134] = 0.0978597983806286*G0_0_0 + 0.0489298991903101*G0_0_1 + 0.048929899190312*G0_0_2 + 0.0489298991903156*G0_1_0 - 0.293579395141894*G0_1_1 + 0.293579395141893*G0_1_2 + 0.0489298991903138*G0_2_0 + 0.293579395141892*G0_2_1 - 0.293579395141893*G0_2_2;
    A[3135] = 1.5657567740901*G0_0_0 + 0.782878387045048*G0_0_1 + 0.782878387045052*G0_0_2 + 0.782878387045048*G0_1_0 + 1.56575677409009*G0_1_1 + 0.782878387045051*G0_1_2 + 0.782878387045052*G0_2_0 + 0.782878387045051*G0_2_1 + 1.76147637085136*G0_2_2;
  }

};

/// This class defines the interface for the assembly of the global
/// tensor corresponding to a form with r + n arguments, that is, a
/// mapping
///
///     a : V1 x V2 x ... Vr x W1 x W2 x ... x Wn -> R
///
/// with arguments v1, v2, ..., vr, w1, w2, ..., wn. The rank r
/// global tensor A is defined by
///
///     A = a(V1, V2, ..., Vr, w1, w2, ..., wn),
///
/// where each argument Vj represents the application to the
/// sequence of basis functions of Vj and w1, w2, ..., wn are given
/// fixed functions (coefficients).

class UFC_Poisson3D_5BilinearForm: public ufc::form
{
public:

  /// Constructor
  UFC_Poisson3D_5BilinearForm() : ufc::form()
  {
    // Do nothing
  }

  /// Destructor
  virtual ~UFC_Poisson3D_5BilinearForm()
  {
    // Do nothing
  }

  /// Return a string identifying the form
  virtual const char* signature() const
  {
    return "(dXa0[0, 1, 2]/dxb0[0, 1, 2])(dXa1[0, 1, 2]/dxb0[0, 1, 2]) | ((d/dXa0[0, 1, 2])vi0[0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 19, 20, 21, 22, 23, 24, 25, 26, 27, 28, 29, 30, 31, 32, 33, 34, 35, 36, 37, 38, 39, 40, 41, 42, 43, 44, 45, 46, 47, 48, 49, 50, 51, 52, 53, 54, 55])*((d/dXa1[0, 1, 2])vi1[0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 19, 20, 21, 22, 23, 24, 25, 26, 27, 28, 29, 30, 31, 32, 33, 34, 35, 36, 37, 38, 39, 40, 41, 42, 43, 44, 45, 46, 47, 48, 49, 50, 51, 52, 53, 54, 55])*dX(0)";
  }

  /// Return the rank of the global tensor (r)
  virtual unsigned int rank() const
  {
    return 2;
  }

  /// Return the number of coefficients (n)
  virtual unsigned int num_coefficients() const
  {
    return 0;
  }

  /// Return the number of cell integrals
  virtual unsigned int num_cell_integrals() const
  {
    return 1;
  }
  
  /// Return the number of exterior facet integrals
  virtual unsigned int num_exterior_facet_integrals() const
  {
    return 0;
  }
  
  /// Return the number of interior facet integrals
  virtual unsigned int num_interior_facet_integrals() const
  {
    return 0;
  }
    
  /// Create a new finite element for argument function i
  virtual ufc::finite_element* create_finite_element(unsigned int i) const
  {
    switch ( i )
    {
    case 0:
      return new UFC_Poisson3D_5BilinearForm_finite_element_0();
      break;
    case 1:
      return new UFC_Poisson3D_5BilinearForm_finite_element_1();
      break;
    }
    return 0;
  }
  
  /// Create a new dof map for argument function i
  virtual ufc::dof_map* create_dof_map(unsigned int i) const
  {
    switch ( i )
    {
    case 0:
      return new UFC_Poisson3D_5BilinearForm_dof_map_0();
      break;
    case 1:
      return new UFC_Poisson3D_5BilinearForm_dof_map_1();
      break;
    }
    return 0;
  }

  /// Create a new cell integral on sub domain i
  virtual ufc::cell_integral* create_cell_integral(unsigned int i) const
  {
    return new UFC_Poisson3D_5BilinearForm_cell_integral_0();
  }

  /// Create a new exterior facet integral on sub domain i
  virtual ufc::exterior_facet_integral* create_exterior_facet_integral(unsigned int i) const
  {
    return 0;
  }

  /// Create a new interior facet integral on sub domain i
  virtual ufc::interior_facet_integral* create_interior_facet_integral(unsigned int i) const
  {
    return 0;
  }

};

/// This class defines the interface for a finite element.

class UFC_Poisson3D_5LinearForm_finite_element_0: public ufc::finite_element
{
public:

  /// Constructor
  UFC_Poisson3D_5LinearForm_finite_element_0() : ufc::finite_element()
  {
    // Do nothing
  }

  /// Destructor
  virtual ~UFC_Poisson3D_5LinearForm_finite_element_0()
  {
    // Do nothing
  }

  /// Return a string identifying the finite element
  virtual const char* signature() const
  {
    return "Lagrange finite element of degree 5 on a tetrahedron";
  }

  /// Return the cell shape
  virtual ufc::shape cell_shape() const
  {
    return ufc::tetrahedron;
  }

  /// Return the dimension of the finite element function space
  virtual unsigned int space_dimension() const
  {
    return 56;
  }

  /// Return the rank of the value space
  virtual unsigned int value_rank() const
  {
    return 0;
  }

  /// Return the dimension of the value space for axis i
  virtual unsigned int value_dimension(unsigned int i) const
  {
    return 1;
  }

  /// Evaluate basis function i at given point in cell
  virtual void evaluate_basis(unsigned int i,
                              double* values,
                              const double* coordinates,
                              const ufc::cell& c) const
  {
    // Extract vertex coordinates
    const double * const * element_coordinates = c.coordinates;
    
    // Compute Jacobian of affine map from reference cell
    const double J_00 = element_coordinates[1][0] - element_coordinates[0][0];
    const double J_01 = element_coordinates[2][0] - element_coordinates[0][0];
    const double J_02 = element_coordinates[3][0] - element_coordinates[0][0];
    const double J_10 = element_coordinates[1][1] - element_coordinates[0][1];
    const double J_11 = element_coordinates[2][1] - element_coordinates[0][1];
    const double J_12 = element_coordinates[3][1] - element_coordinates[0][1];
    const double J_20 = element_coordinates[1][2] - element_coordinates[0][2];
    const double J_21 = element_coordinates[2][2] - element_coordinates[0][2];
    const double J_22 = element_coordinates[3][2] - element_coordinates[0][2];
      
    // Compute sub determinants
    const double d00 = J_11*J_22 - J_12*J_21;
    const double d01 = J_12*J_20 - J_10*J_22;
    const double d02 = J_10*J_21 - J_11*J_20;
    
    const double d10 = J_02*J_21 - J_01*J_22;
    const double d11 = J_00*J_22 - J_02*J_20;
    const double d12 = J_01*J_20 - J_00*J_21;
    
    const double d20 = J_01*J_12 - J_02*J_11;
    const double d21 = J_02*J_10 - J_00*J_12;
    const double d22 = J_00*J_11 - J_01*J_10;
      
    // Compute determinant of Jacobian
    double detJ = J_00*d00 + J_10*d10 + J_20*d20;
    
    // Compute inverse of Jacobian
    
    // Compute constants
    const double C0 = d00*(element_coordinates[0][0] - element_coordinates[2][0] - element_coordinates[3][0]) \
                    + d10*(element_coordinates[0][1] - element_coordinates[2][1] - element_coordinates[3][1]) \
                    + d20*(element_coordinates[0][2] - element_coordinates[2][2] - element_coordinates[3][2]);
    
    const double C1 = d01*(element_coordinates[0][0] - element_coordinates[1][0] - element_coordinates[3][0]) \
                    + d11*(element_coordinates[0][1] - element_coordinates[1][1] - element_coordinates[3][1]) \
                    + d21*(element_coordinates[0][2] - element_coordinates[1][2] - element_coordinates[3][2]);
    
    const double C2 = d02*(element_coordinates[0][0] - element_coordinates[1][0] - element_coordinates[2][0]) \
                    + d12*(element_coordinates[0][1] - element_coordinates[1][1] - element_coordinates[2][1]) \
                    + d22*(element_coordinates[0][2] - element_coordinates[1][2] - element_coordinates[2][2]);
    
    // Get coordinates and map to the UFC reference element
    double x = (C0 + d00*coordinates[0] + d10*coordinates[1] + d20*coordinates[2]) / detJ;
    double y = (C1 + d01*coordinates[0] + d11*coordinates[1] + d21*coordinates[2]) / detJ;
    double z = (C2 + d02*coordinates[0] + d12*coordinates[1] + d22*coordinates[2]) / detJ;
    
    // Map coordinates to the reference cube
    if (std::abs(y + z - 1.0) < 1e-14)
      x = 1.0;
    else
      x = -2.0 * x/(y + z - 1.0) - 1.0;
    if (std::abs(z - 1.0) < 1e-14)
      y = -1.0;
    else
      y = 2.0 * y/(1.0 - z) - 1.0;
    z = 2.0 * z - 1.0;
    
    // Reset values
    *values = 0;
    
    // Map degree of freedom to element degree of freedom
    const unsigned int dof = i;
    
    // Generate scalings
    const double scalings_y_0 = 1;
    const double scalings_y_1 = scalings_y_0*(0.5 - 0.5*y);
    const double scalings_y_2 = scalings_y_1*(0.5 - 0.5*y);
    const double scalings_y_3 = scalings_y_2*(0.5 - 0.5*y);
    const double scalings_y_4 = scalings_y_3*(0.5 - 0.5*y);
    const double scalings_y_5 = scalings_y_4*(0.5 - 0.5*y);
    const double scalings_z_0 = 1;
    const double scalings_z_1 = scalings_z_0*(0.5 - 0.5*z);
    const double scalings_z_2 = scalings_z_1*(0.5 - 0.5*z);
    const double scalings_z_3 = scalings_z_2*(0.5 - 0.5*z);
    const double scalings_z_4 = scalings_z_3*(0.5 - 0.5*z);
    const double scalings_z_5 = scalings_z_4*(0.5 - 0.5*z);
    
    // Compute psitilde_a
    const double psitilde_a_0 = 1;
    const double psitilde_a_1 = x;
    const double psitilde_a_2 = 1.5*x*psitilde_a_1 - 0.5*psitilde_a_0;
    const double psitilde_a_3 = 1.66666666666667*x*psitilde_a_2 - 0.666666666666667*psitilde_a_1;
    const double psitilde_a_4 = 1.75*x*psitilde_a_3 - 0.75*psitilde_a_2;
    const double psitilde_a_5 = 1.8*x*psitilde_a_4 - 0.8*psitilde_a_3;
    
    // Compute psitilde_bs
    const double psitilde_bs_0_0 = 1;
    const double psitilde_bs_0_1 = 1.5*y + 0.5;
    const double psitilde_bs_0_2 = 0.111111111111111*psitilde_bs_0_1 + 1.66666666666667*y*psitilde_bs_0_1 - 0.555555555555556*psitilde_bs_0_0;
    const double psitilde_bs_0_3 = 0.05*psitilde_bs_0_2 + 1.75*y*psitilde_bs_0_2 - 0.7*psitilde_bs_0_1;
    const double psitilde_bs_0_4 = 0.0285714285714286*psitilde_bs_0_3 + 1.8*y*psitilde_bs_0_3 - 0.771428571428571*psitilde_bs_0_2;
    const double psitilde_bs_0_5 = 0.0185185185185185*psitilde_bs_0_4 + 1.83333333333333*y*psitilde_bs_0_4 - 0.814814814814815*psitilde_bs_0_3;
    const double psitilde_bs_1_0 = 1;
    const double psitilde_bs_1_1 = 2.5*y + 1.5;
    const double psitilde_bs_1_2 = 0.54*psitilde_bs_1_1 + 2.1*y*psitilde_bs_1_1 - 0.56*psitilde_bs_1_0;
    const double psitilde_bs_1_3 = 0.285714285714286*psitilde_bs_1_2 + 2*y*psitilde_bs_1_2 - 0.714285714285714*psitilde_bs_1_1;
    const double psitilde_bs_1_4 = 0.178571428571429*psitilde_bs_1_3 + 1.96428571428571*y*psitilde_bs_1_3 - 0.785714285714286*psitilde_bs_1_2;
    const double psitilde_bs_2_0 = 1;
    const double psitilde_bs_2_1 = 3.5*y + 2.5;
    const double psitilde_bs_2_2 = 1.02040816326531*psitilde_bs_2_1 + 2.57142857142857*y*psitilde_bs_2_1 - 0.551020408163265*psitilde_bs_2_0;
    const double psitilde_bs_2_3 = 0.578703703703704*psitilde_bs_2_2 + 2.29166666666667*y*psitilde_bs_2_2 - 0.712962962962963*psitilde_bs_2_1;
    const double psitilde_bs_3_0 = 1;
    const double psitilde_bs_3_1 = 4.5*y + 3.5;
    const double psitilde_bs_3_2 = 1.51234567901235*psitilde_bs_3_1 + 3.05555555555556*y*psitilde_bs_3_1 - 0.54320987654321*psitilde_bs_3_0;
    const double psitilde_bs_4_0 = 1;
    const double psitilde_bs_4_1 = 5.5*y + 4.5;
    const double psitilde_bs_5_0 = 1;
    
    // Compute psitilde_cs
    const double psitilde_cs_00_0 = 1;
    const double psitilde_cs_00_1 = 2*z + 1;
    const double psitilde_cs_00_2 = 0.3125*psitilde_cs_00_1 + 1.875*z*psitilde_cs_00_1 - 0.5625*psitilde_cs_00_0;
    const double psitilde_cs_00_3 = 0.155555555555556*psitilde_cs_00_2 + 1.86666666666667*z*psitilde_cs_00_2 - 0.711111111111111*psitilde_cs_00_1;
    const double psitilde_cs_00_4 = 0.09375*psitilde_cs_00_3 + 1.875*z*psitilde_cs_00_3 - 0.78125*psitilde_cs_00_2;
    const double psitilde_cs_00_5 = 0.0628571428571429*psitilde_cs_00_4 + 1.88571428571429*z*psitilde_cs_00_4 - 0.822857142857143*psitilde_cs_00_3;
    const double psitilde_cs_01_0 = 1;
    const double psitilde_cs_01_1 = 3*z + 2;
    const double psitilde_cs_01_2 = 0.777777777777778*psitilde_cs_01_1 + 2.33333333333333*z*psitilde_cs_01_1 - 0.555555555555556*psitilde_cs_01_0;
    const double psitilde_cs_01_3 = 0.428571428571429*psitilde_cs_01_2 + 2.14285714285714*z*psitilde_cs_01_2 - 0.714285714285714*psitilde_cs_01_1;
    const double psitilde_cs_01_4 = 0.275*psitilde_cs_01_3 + 2.0625*z*psitilde_cs_01_3 - 0.7875*psitilde_cs_01_2;
    const double psitilde_cs_02_0 = 1;
    const double psitilde_cs_02_1 = 4*z + 3;
    const double psitilde_cs_02_2 = 1.265625*psitilde_cs_02_1 + 2.8125*z*psitilde_cs_02_1 - 0.546875*psitilde_cs_02_0;
    const double psitilde_cs_02_3 = 0.733333333333333*psitilde_cs_02_2 + 2.44444444444444*z*psitilde_cs_02_2 - 0.711111111111111*psitilde_cs_02_1;
    const double psitilde_cs_03_0 = 1;
    const double psitilde_cs_03_1 = 5*z + 4;
    const double psitilde_cs_03_2 = 1.76*psitilde_cs_03_1 + 3.3*z*psitilde_cs_03_1 - 0.54*psitilde_cs_03_0;
    const double psitilde_cs_04_0 = 1;
    const double psitilde_cs_04_1 = 6*z + 5;
    const double psitilde_cs_05_0 = 1;
    const double psitilde_cs_10_0 = 1;
    const double psitilde_cs_10_1 = 3*z + 2;
    const double psitilde_cs_10_2 = 0.777777777777778*psitilde_cs_10_1 + 2.33333333333333*z*psitilde_cs_10_1 - 0.555555555555556*psitilde_cs_10_0;
    const double psitilde_cs_10_3 = 0.428571428571429*psitilde_cs_10_2 + 2.14285714285714*z*psitilde_cs_10_2 - 0.714285714285714*psitilde_cs_10_1;
    const double psitilde_cs_10_4 = 0.275*psitilde_cs_10_3 + 2.0625*z*psitilde_cs_10_3 - 0.7875*psitilde_cs_10_2;
    const double psitilde_cs_11_0 = 1;
    const double psitilde_cs_11_1 = 4*z + 3;
    const double psitilde_cs_11_2 = 1.265625*psitilde_cs_11_1 + 2.8125*z*psitilde_cs_11_1 - 0.546875*psitilde_cs_11_0;
    const double psitilde_cs_11_3 = 0.733333333333333*psitilde_cs_11_2 + 2.44444444444444*z*psitilde_cs_11_2 - 0.711111111111111*psitilde_cs_11_1;
    const double psitilde_cs_12_0 = 1;
    const double psitilde_cs_12_1 = 5*z + 4;
    const double psitilde_cs_12_2 = 1.76*psitilde_cs_12_1 + 3.3*z*psitilde_cs_12_1 - 0.54*psitilde_cs_12_0;
    const double psitilde_cs_13_0 = 1;
    const double psitilde_cs_13_1 = 6*z + 5;
    const double psitilde_cs_14_0 = 1;
    const double psitilde_cs_20_0 = 1;
    const double psitilde_cs_20_1 = 4*z + 3;
    const double psitilde_cs_20_2 = 1.265625*psitilde_cs_20_1 + 2.8125*z*psitilde_cs_20_1 - 0.546875*psitilde_cs_20_0;
    const double psitilde_cs_20_3 = 0.733333333333333*psitilde_cs_20_2 + 2.44444444444444*z*psitilde_cs_20_2 - 0.711111111111111*psitilde_cs_20_1;
    const double psitilde_cs_21_0 = 1;
    const double psitilde_cs_21_1 = 5*z + 4;
    const double psitilde_cs_21_2 = 1.76*psitilde_cs_21_1 + 3.3*z*psitilde_cs_21_1 - 0.54*psitilde_cs_21_0;
    const double psitilde_cs_22_0 = 1;
    const double psitilde_cs_22_1 = 6*z + 5;
    const double psitilde_cs_23_0 = 1;
    const double psitilde_cs_30_0 = 1;
    const double psitilde_cs_30_1 = 5*z + 4;
    const double psitilde_cs_30_2 = 1.76*psitilde_cs_30_1 + 3.3*z*psitilde_cs_30_1 - 0.54*psitilde_cs_30_0;
    const double psitilde_cs_31_0 = 1;
    const double psitilde_cs_31_1 = 6*z + 5;
    const double psitilde_cs_32_0 = 1;
    const double psitilde_cs_40_0 = 1;
    const double psitilde_cs_40_1 = 6*z + 5;
    const double psitilde_cs_41_0 = 1;
    const double psitilde_cs_50_0 = 1;
    
    // Compute basisvalues
    const double basisvalue0 = 0.866025403784439*psitilde_a_0*scalings_y_0*psitilde_bs_0_0*scalings_z_0*psitilde_cs_00_0;
    const double basisvalue1 = 2.73861278752583*psitilde_a_1*scalings_y_1*psitilde_bs_1_0*scalings_z_1*psitilde_cs_10_0;
    const double basisvalue2 = 1.58113883008419*psitilde_a_0*scalings_y_0*psitilde_bs_0_1*scalings_z_1*psitilde_cs_01_0;
    const double basisvalue3 = 1.11803398874989*psitilde_a_0*scalings_y_0*psitilde_bs_0_0*scalings_z_0*psitilde_cs_00_1;
    const double basisvalue4 = 5.1234753829798*psitilde_a_2*scalings_y_2*psitilde_bs_2_0*scalings_z_2*psitilde_cs_20_0;
    const double basisvalue5 = 3.96862696659689*psitilde_a_1*scalings_y_1*psitilde_bs_1_1*scalings_z_2*psitilde_cs_11_0;
    const double basisvalue6 = 2.29128784747792*psitilde_a_0*scalings_y_0*psitilde_bs_0_2*scalings_z_2*psitilde_cs_02_0;
    const double basisvalue7 = 3.24037034920393*psitilde_a_1*scalings_y_1*psitilde_bs_1_0*scalings_z_1*psitilde_cs_10_1;
    const double basisvalue8 = 1.87082869338697*psitilde_a_0*scalings_y_0*psitilde_bs_0_1*scalings_z_1*psitilde_cs_01_1;
    const double basisvalue9 = 1.3228756555323*psitilde_a_0*scalings_y_0*psitilde_bs_0_0*scalings_z_0*psitilde_cs_00_2;
    const double basisvalue10 = 7.93725393319377*psitilde_a_3*scalings_y_3*psitilde_bs_3_0*scalings_z_3*psitilde_cs_30_0;
    const double basisvalue11 = 6.70820393249937*psitilde_a_2*scalings_y_2*psitilde_bs_2_1*scalings_z_3*psitilde_cs_21_0;
    const double basisvalue12 = 5.19615242270663*psitilde_a_1*scalings_y_1*psitilde_bs_1_2*scalings_z_3*psitilde_cs_12_0;
    const double basisvalue13 = 3*psitilde_a_0*scalings_y_0*psitilde_bs_0_3*scalings_z_3*psitilde_cs_03_0;
    const double basisvalue14 = 5.80947501931113*psitilde_a_2*scalings_y_2*psitilde_bs_2_0*scalings_z_2*psitilde_cs_20_1;
    const double basisvalue15 = 4.5*psitilde_a_1*scalings_y_1*psitilde_bs_1_1*scalings_z_2*psitilde_cs_11_1;
    const double basisvalue16 = 2.59807621135332*psitilde_a_0*scalings_y_0*psitilde_bs_0_2*scalings_z_2*psitilde_cs_02_1;
    const double basisvalue17 = 3.67423461417477*psitilde_a_1*scalings_y_1*psitilde_bs_1_0*scalings_z_1*psitilde_cs_10_2;
    const double basisvalue18 = 2.12132034355964*psitilde_a_0*scalings_y_0*psitilde_bs_0_1*scalings_z_1*psitilde_cs_01_2;
    const double basisvalue19 = 1.5*psitilde_a_0*scalings_y_0*psitilde_bs_0_0*scalings_z_0*psitilde_cs_00_3;
    const double basisvalue20 = 11.1242977306435*psitilde_a_4*scalings_y_4*psitilde_bs_4_0*scalings_z_4*psitilde_cs_40_0;
    const double basisvalue21 = 9.81070843517429*psitilde_a_3*scalings_y_3*psitilde_bs_3_1*scalings_z_4*psitilde_cs_31_0;
    const double basisvalue22 = 8.2915619758885*psitilde_a_2*scalings_y_2*psitilde_bs_2_2*scalings_z_4*psitilde_cs_22_0;
    const double basisvalue23 = 6.42261628933256*psitilde_a_1*scalings_y_1*psitilde_bs_1_3*scalings_z_4*psitilde_cs_13_0;
    const double basisvalue24 = 3.70809924354783*psitilde_a_0*scalings_y_0*psitilde_bs_0_4*scalings_z_4*psitilde_cs_04_0;
    const double basisvalue25 = 8.77496438739212*psitilde_a_3*scalings_y_3*psitilde_bs_3_0*scalings_z_3*psitilde_cs_30_1;
    const double basisvalue26 = 7.41619848709566*psitilde_a_2*scalings_y_2*psitilde_bs_2_1*scalings_z_3*psitilde_cs_21_1;
    const double basisvalue27 = 5.74456264653803*psitilde_a_1*scalings_y_1*psitilde_bs_1_2*scalings_z_3*psitilde_cs_12_1;
    const double basisvalue28 = 3.3166247903554*psitilde_a_0*scalings_y_0*psitilde_bs_0_3*scalings_z_3*psitilde_cs_03_1;
    const double basisvalue29 = 6.42261628933256*psitilde_a_2*scalings_y_2*psitilde_bs_2_0*scalings_z_2*psitilde_cs_20_2;
    const double basisvalue30 = 4.9749371855331*psitilde_a_1*scalings_y_1*psitilde_bs_1_1*scalings_z_2*psitilde_cs_11_2;
    const double basisvalue31 = 2.87228132326901*psitilde_a_0*scalings_y_0*psitilde_bs_0_2*scalings_z_2*psitilde_cs_02_2;
    const double basisvalue32 = 4.06201920231798*psitilde_a_1*scalings_y_1*psitilde_bs_1_0*scalings_z_1*psitilde_cs_10_3;
    const double basisvalue33 = 2.34520787991171*psitilde_a_0*scalings_y_0*psitilde_bs_0_1*scalings_z_1*psitilde_cs_01_3;
    const double basisvalue34 = 1.6583123951777*psitilde_a_0*scalings_y_0*psitilde_bs_0_0*scalings_z_0*psitilde_cs_00_4;
    const double basisvalue35 = 14.6458185158768*psitilde_a_5*scalings_y_5*psitilde_bs_5_0*scalings_z_5*psitilde_cs_50_0;
    const double basisvalue36 = 13.2476412994918*psitilde_a_4*scalings_y_4*psitilde_bs_4_1*scalings_z_5*psitilde_cs_41_0;
    const double basisvalue37 = 11.6833214455479*psitilde_a_3*scalings_y_3*psitilde_bs_3_2*scalings_z_5*psitilde_cs_32_0;
    const double basisvalue38 = 9.87420882906575*psitilde_a_2*scalings_y_2*psitilde_bs_2_3*scalings_z_5*psitilde_cs_23_0;
    const double basisvalue39 = 7.64852927038918*psitilde_a_1*scalings_y_1*psitilde_bs_1_4*scalings_z_5*psitilde_cs_14_0;
    const double basisvalue40 = 4.41588043316392*psitilde_a_0*scalings_y_0*psitilde_bs_0_5*scalings_z_5*psitilde_cs_05_0;
    const double basisvalue41 = 12.0933866224478*psitilde_a_4*scalings_y_4*psitilde_bs_4_0*scalings_z_4*psitilde_cs_40_1;
    const double basisvalue42 = 10.6653645038508*psitilde_a_3*scalings_y_3*psitilde_bs_3_1*scalings_z_4*psitilde_cs_31_1;
    const double basisvalue43 = 9.01387818865997*psitilde_a_2*scalings_y_2*psitilde_bs_2_2*scalings_z_4*psitilde_cs_22_1;
    const double basisvalue44 = 6.98212002188447*psitilde_a_1*scalings_y_1*psitilde_bs_1_3*scalings_z_4*psitilde_cs_13_1;
    const double basisvalue45 = 4.03112887414927*psitilde_a_0*scalings_y_0*psitilde_bs_0_4*scalings_z_4*psitilde_cs_04_1;
    const double basisvalue46 = 9.53939201416946*psitilde_a_3*scalings_y_3*psitilde_bs_3_0*scalings_z_3*psitilde_cs_30_2;
    const double basisvalue47 = 8.06225774829855*psitilde_a_2*scalings_y_2*psitilde_bs_2_1*scalings_z_3*psitilde_cs_21_2;
    const double basisvalue48 = 6.2449979983984*psitilde_a_1*scalings_y_1*psitilde_bs_1_2*scalings_z_3*psitilde_cs_12_2;
    const double basisvalue49 = 3.60555127546399*psitilde_a_0*scalings_y_0*psitilde_bs_0_3*scalings_z_3*psitilde_cs_03_2;
    const double basisvalue50 = 6.98212002188447*psitilde_a_2*scalings_y_2*psitilde_bs_2_0*scalings_z_2*psitilde_cs_20_3;
    const double basisvalue51 = 5.40832691319598*psitilde_a_1*scalings_y_1*psitilde_bs_1_1*scalings_z_2*psitilde_cs_11_3;
    const double basisvalue52 = 3.1224989991992*psitilde_a_0*scalings_y_0*psitilde_bs_0_2*scalings_z_2*psitilde_cs_02_3;
    const double basisvalue53 = 4.41588043316392*psitilde_a_1*scalings_y_1*psitilde_bs_1_0*scalings_z_1*psitilde_cs_10_4;
    const double basisvalue54 = 2.54950975679639*psitilde_a_0*scalings_y_0*psitilde_bs_0_1*scalings_z_1*psitilde_cs_01_4;
    const double basisvalue55 = 1.80277563773199*psitilde_a_0*scalings_y_0*psitilde_bs_0_0*scalings_z_0*psitilde_cs_00_5;
    
    // Table(s) of coefficients
    const static double coefficients0[56][56] = \
    {{0.00945067404923504, 0.00609788120679252, 0.003520613356228, 0.00248944957812473, 0.00705946233325038, 0.00546823600997147, 0.00315708753234945, 0.00446479600584757, 0.00257775117585287, 0.00182274533665719, -0.00298267418725718, -0.0025208197798293, -0.00195261860521383, -0.0011273448773449, -0.00218309396769446, -0.00169101731601731, -0.000976309302606924, -0.00138070985681772, -0.000797153207506492, -0.000563672438672437, 0.00278687112460006, 0.00245778931055954, 0.00207721109304565, 0.00160900079398463, 0.000928957041533348, 0.00219831358911339, 0.00185791408306671, 0.00143913406048031, 0.000830884437218262, 0.00160900079398463, 0.00124632665582739, 0.000719567030240153, 0.00101762145320216, 0.000587524019939396, 0.000415442218609115, -0.00705593524717797, -0.0063823335708908, -0.00562868913761196, -0.00475711057319694, -0.00368484200521179, -0.00212744452363026, -0.00582624677716564, -0.00513826668309063, -0.00434262794914383, -0.00336378514516178, -0.00194208225905521, -0.00459580543596519, -0.00388416451811043, -0.00300866089851429, -0.00173705117965752, -0.00336378514516179, -0.00260557676948628, -0.00150433044925714, -0.00212744452363026, -0.00122828066840392, -0.000868525589828755},
    {0.00945067404923484, -0.0060978812067926, 0.00352061335622807, 0.00248944957812473, 0.00705946233325039, -0.00546823600997146, 0.00315708753234942, -0.00446479600584756, 0.00257775117585286, 0.00182274533665715, 0.00298267418725716, -0.00252081977982931, 0.00195261860521383, -0.00112734487734489, -0.00218309396769446, 0.00169101731601731, -0.000976309302606908, 0.00138070985681772, -0.000797153207506487, -0.000563672438672423, 0.00278687112460003, -0.00245778931055954, 0.00207721109304565, -0.00160900079398463, 0.000928957041533354, -0.00219831358911339, 0.00185791408306669, -0.00143913406048031, 0.000830884437218255, 0.00160900079398463, -0.0012463266558274, 0.000719567030240159, -0.00101762145320216, 0.000587524019939409, 0.000415442218609113, 0.00705593524717798, -0.00638233357089081, 0.00562868913761196, -0.00475711057319694, 0.00368484200521178, -0.00212744452363027, -0.00582624677716566, 0.00513826668309062, -0.00434262794914383, 0.00336378514516178, -0.00194208225905522, 0.0045958054359652, -0.00388416451811043, 0.00300866089851429, -0.00173705117965753, -0.00336378514516178, 0.00260557676948629, -0.00150433044925715, 0.00212744452363026, -0.00122828066840393, -0.000868525589828756},
    {0.00945067404923496, 0, -0.00704122671245609, 0.00248944957812476, 0, 0, 0.00947126259704828, 0, -0.00515550235170569, 0.00182274533665715, 0, 0, 0, 0.0045093795093795, 0, 0, -0.00292892790782073, 0, 0.00159430641501296, -0.000563672438672431, 0, 0, 0, 0, 0.00464478520766673, 0, 0, 0, -0.00332353774887306, 0, 0, 0.00215870109072046, 0, -0.00117504803987882, 0.000415442218609132, 0, 0, 0, 0, 0, 0.0127646671417816, 0, 0, 0, 0, -0.00971041129527609, 0, 0, 0, 0.0069482047186301, 0, 0, -0.00451299134777144, 0, 0.00245656133680787, -0.000868525589828766},
    {0.00945067404923495, 0, 0, -0.00746834873437432, 0, 0, 0, 0, 0, 0.0109364720199429, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0.00563672438672439, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0.00623163327913698, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0.0182390373864041},
    {-0.0100234421734309, 0, 0.0374718748311398, -0.0289614183593723, 0, 0, 0.0284137877911449, 0, 0.00515550235170565, -0.00619733414463432, 0, 0, 0, 0.0315656565656566, 0, 0, -0.0113902751970807, 0, 0.00265717735835495, -0.000187890812890785, 0, 0, 0, 0, 0.0232239260383336, 0, 0, 0, -0.00166176887443649, 0, 0, -0.00647610327216139, 0, 0.00646276421933346, -0.00290809553026393, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0.0213629048496073, 0, 0, 0, -0.0277928188745204, 0, 0, 0.0243701532779658, 0, -0.0157219925555703, 0.00607967912880134},
    {0.0100234421734308, 0, -0.0287574853355261, 0.05360943398437, 0, 0, -0.0189425251940965, 0, 0.0154665070551172, 0.0153110608279201, 0, 0, 0, -0.00450937950937953, 0, 0, 0.0370997534990628, 0, -0.0281660799985623, 0.0108976671476671, 0, 0, 0, 0, 0, 0, 0, 0, 0.0299118397398575, 0, 0, -0.00431740218144094, 0, -0.0123380044187275, 0.00872428659079179, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0.0312669212338355, 0, 0, -0.0487403065559315, 0, 0.0412702304583721, -0.018239037386404},
    {0.0100234421734309, 0, 0.0409576306293854, -0.0449826285156208, 0, 0, 0.0189425251940966, 0, -0.00515550235170581, -0.021143845905223, 0, 0, 0, 0, 0, 0, 0.00455611007883228, 0, 0.0356061766019561, -0.0319414381914382, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0.0302218152700864, 0, 0.00411266813957589, -0.0145404776513196, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0.0379091273212801, 0, -0.0550269739444961, 0.0303983956440067},
    {-0.0100234421734309, 0, -0.0148144621425437, 0.0449826285156208, 0, 0, 0, 0, 0.0206220094068229, 0.021143845905223, 0, 0, 0, 0, 0, 0, 0, 0, 0.0106287094334197, 0.0319414381914382, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0.0205633406978791, 0.0145404776513196, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0.03439185871531, -0.0303983956440067},
    {-0.0100234421734309, 0.0324515955311978, -0.01873593741557, -0.0289614183593723, 0.0211783869997511, -0.0164047080299144, 0.00947126259704827, 0.00446479600584751, -0.00257775117585286, -0.00619733414463429, 0.0208787193108001, -0.0176457384588051, 0.0136683302364968, -0.00789141414141412, -0.00848980987436741, 0.00657617845117844, -0.00379675839902691, 0.0023011830946962, -0.00132858867917746, -0.000187890812890819, 0.0139343556230002, -0.0122889465527977, 0.0103860554652283, -0.00804500396992315, 0.00464478520766672, -0.00109915679455667, 0.000928957041533349, -0.000719567030240153, 0.000415442218609154, -0.00482700238195389, 0.00373897996748218, -0.00215870109072046, 0.00559691799261187, -0.00323138210966673, -0.00290809553026394, 0, 0, 0, 0, 0, 0, 0.0128177429097644, -0.0113041867027994, 0.00955378148811641, -0.00740032731935592, 0.00427258096992148, -0.0183832217438608, 0.0155366580724417, -0.0120346435940572, 0.00694820471863012, 0.0181644397838736, -0.014070114555226, 0.00812338442598859, -0.0136156449512337, 0.00786099627778517, 0.00607967912880136},
    {0.0100234421734311, -0.024904712849524, 0.014378742667763, 0.0536094339843701, -0.0141189246665008, 0.0109364720199429, -0.00631417506469887, 0.0133943880175428, -0.00773325352755854, 0.01531106082792, -0.00298267418725714, 0.0025208197798293, -0.00195261860521383, 0.00112734487734485, 0.0276525235907967, -0.0214195526695527, 0.0123665844996876, -0.0243925408037797, 0.0140830399992812, 0.0108976671476671, 0, 0, 0, 0, 0, 0.0197848223020205, -0.0167212267476003, 0.0129522065443228, -0.0074779599349644, -0.00321800158796925, 0.00249265331165481, -0.00143913406048031, -0.0106850252586226, 0.00616900220936375, 0.00872428659079183, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0.0206811244618434, -0.017478740331497, 0.0135389740433143, -0.00781673030845889, -0.0363288795677472, 0.028140229110452, -0.0162467688519772, 0.0357410679969885, -0.0206351152291861, -0.0182390373864041},
    {0.0100234421734311, 0.0354703486038675, -0.0204788153146928, -0.0449826285156207, 0.0141189246665009, -0.0109364720199429, 0.00631417506469889, -0.0044647960058475, 0.00257775117585279, -0.021143845905223, 0, 0, 0, 0, 0.00339592394974698, -0.00263047138047136, 0.00151870335961076, 0.0308358534689291, -0.0178030883009781, -0.0319414381914382, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0.0225260111157848, -0.0174485731815835, 0.0100739384233621, 0.00356167508620756, -0.00205633406978792, -0.0145404776513196, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0.028255795219359, -0.0218868448636849, 0.01263637577376, -0.047654757329318, 0.0275134869722481, 0.0303983956440068},
    {-0.0100234421734311, -0.0128297005588457, 0.00740723107127181, 0.0449826285156207, 0, 0, 0, 0.0178591840233903, -0.0103110047034114, 0.0211438459052229, 0, 0, 0, 0, 0, 0, 0, 0.0092047323787848, -0.00531435471670985, 0.0319414381914382, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0.0178083754310377, -0.0102816703489396, 0.0145404776513196, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0.0297842233308237, -0.017195929357655, -0.0303983956440068},
    {-0.0100234421734311, 0.0324515955311977, -0.0335503995581136, -0.00801060507812428, 0.0211783869997512, 0.00109364720199428, -0.00568275755822898, -0.0169662248222208, 0.00567105258687634, 0.0083846285486229, 0.0208787193108001, -0.0109235523792603, 0.00499002532443531, -0.00187890812890811, -0.0162519217595033, 0.00958243145743145, -0.00466458889023307, 0.0108922666482287, -0.00558007245254534, -0.00469727032227035, 0.0139343556230002, -0.00245778931055952, -0.00415442218609131, 0.0064360031759385, -0.00464478520766676, -0.0120907247401236, 0.00464478520766674, -0.000719567030240154, -0.000415442218609135, 0.00965400476390777, -0.00498530662330959, 0.00215870109072046, -0.00661453944581401, 0.0032313821096667, 0.00290809553026394, 0, 0.0127646671417816, -0.0202632808954031, 0.0228341307513453, -0.020635115229186, 0.0127646671417816, -0.00116524935543312, -0.00719357335632687, 0.0112908326677739, -0.0114368694935501, 0.00737991258440983, 0.00183832217438608, 0.00310733161448834, -0.00481385743762287, 0.00347410235931505, -0.00201827108709706, -0.000521115353897253, 0.000902598269554298, 0.00170195561890421, -0.000491312267361565, -0.000868525589828764},
    {0.0100234421734313, -0.0249047128495239, 0.0553363732971484, -0.00431340273437463, -0.0141189246665008, 0.0153110608279201, 0.00883984509057839, 0.00803663281052565, -0.0159820572902878, 0.000729098134662871, -0.00298267418725714, 0.0277290175781224, -0.0301571095694136, 0.0191648629148629, -0.00145539597846295, -0.0116492303992304, 0.00976309302606917, 0.00352848074520082, 0.00132858867917745, -0.00263047138047135, 0, 0.0196623144844763, -0.00727023882565982, -0.00724050357293083, 0.00928957041533347, -0.00219831358911338, -0.0148633126645336, 0.0129522065443228, -0.00664707549774609, 0.00402250198496158, 0.00685479660705067, -0.00575653624192123, -0.00457929653940971, -0.000293762009969677, 0.0029080955302639, 0, 0, 0.0202632808954031, -0.0399597288148543, 0.0464290092656686, -0.031911667854454, 0, -0.00411061334647249, -0.00303983956440069, 0.00908221989193683, -0.00776832903622088, 0.000459580543596524, 0.00427258096992149, -0.00270779480866288, 0.000868525589828788, -0.00100913554354855, -0.00234501909253767, 0.00180519653910856, 0.00127646671417817, 0.000245656133680783, -0.000868525589828766},
    {0.0100234421734309, 0.0354703486038674, -0.0492363006502187, -0.00431340273437457, 0.0141189246665008, -0.00656188321196574, -0.016416855168217, -0.00982255121286468, 0.0149509568199466, 0.00072909813466289, 0, 0.00336109303977242, 0.0282044909641998, -0.0353234728234728, -0.000485131992821028, -0.0124007936507936, 0.00889526253486298, 0.00291483191994853, 0.00239145962251946, -0.0026304713804714, 0, 0, 0.0218107164769794, -0.00402250198496158, -0.00928957041533346, 0, -0.00557374224920008, -0.0143913406048031, 0.0141250354327105, 0.000804500396992295, 0.00934744991870546, -0.00287826812096058, -0.00254405363300538, -0.00381890612960614, 0.00290809553026392, 0, 0, 0, 0.0266398192099029, -0.0515877880729651, 0.0425488904726054, 0, 0, -0.00911951869320201, 0.0084094628629044, -0.0038841645181104, 0, 0.00233049871086625, 0.00300866089851431, -0.00347410235931508, -0.000336378514516169, -0.00286613444643491, 0.00120346435940571, 0.000850977809452102, 0.000982624534723146, -0.000868525589828757},
    {-0.0100234421734311, -0.0128297005588456, 0.0448791059024117, -0.00801060507812423, 0, 0.0174983552319087, 0.0132597676358676, -0.00357183680467805, -0.0175287079957994, 0.00838462854862291, 0, 0, 0.00867830491206144, 0.0225468975468975, 0, -0.00300625300625305, -0.019200749617936, 0.000613648825252313, 0.0122230158484327, -0.00469727032227032, 0, 0, 0, 0.0160900079398463, 0.00464478520766673, 0, 0, -0.00719567030240156, -0.0108014976838374, 0, 0.00249265331165479, 0.0107935054536023, -0.000508810726601082, -0.00734405024924251, 0.00290809553026393, 0, 0, 0, 0, 0.0257938940364825, -0.031911667854454, 0, 0, 0, -0.0134551405806471, 0.0135945758133865, 0, 0, 0.00601732179702858, -0.00347410235931504, 0, -0.00208446141558903, -0.000902598269554288, 0.000425488904726059, 0.0017195929357655, -0.000868525589828767},
    {-0.010023442173431, -0.0324515955311979, -0.0187359374155701, -0.0289614183593722, 0.0211783869997511, 0.0164047080299144, 0.00947126259704825, -0.0044647960058475, -0.00257775117585286, -0.00619733414463431, -0.0208787193108001, -0.0176457384588052, -0.0136683302364968, -0.00789141414141413, -0.00848980987436743, -0.00657617845117846, -0.00379675839902691, -0.00230118309469614, -0.00132858867917743, -0.000187890812890788, 0.0139343556230002, 0.0122889465527977, 0.0103860554652283, 0.00804500396992312, 0.00464478520766673, 0.00109915679455666, 0.00092895704153335, 0.000719567030240155, 0.000415442218609149, -0.0048270023819539, -0.00373897996748222, -0.00215870109072046, -0.00559691799261192, -0.00323138210966673, -0.00290809553026393, 0, 0, 0, 0, 0, 0, 0.0128177429097644, 0.0113041867027994, 0.00955378148811642, 0.00740032731935589, 0.00427258096992148, 0.0183832217438608, 0.0155366580724417, 0.0120346435940572, 0.00694820471863011, 0.0181644397838736, 0.014070114555226, 0.00812338442598858, 0.0136156449512337, 0.00786099627778515, 0.00607967912880134},
    {0.0100234421734311, 0.024904712849524, 0.014378742667763, 0.05360943398437, -0.0141189246665008, -0.0109364720199429, -0.00631417506469881, -0.0133943880175428, -0.00773325352755853, 0.01531106082792, 0.00298267418725717, 0.0025208197798293, 0.00195261860521382, 0.00112734487734485, 0.0276525235907967, 0.0214195526695527, 0.0123665844996876, 0.0243925408037797, 0.0140830399992811, 0.0108976671476671, 0, 0, 0, 0, 0, -0.0197848223020205, -0.0167212267476003, -0.0129522065443227, -0.0074779599349644, -0.00321800158796927, -0.00249265331165477, -0.00143913406048032, 0.0106850252586227, 0.00616900220936373, 0.00872428659079179, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, -0.0206811244618434, -0.017478740331497, -0.0135389740433143, -0.00781673030845888, -0.0363288795677472, -0.028140229110452, -0.0162467688519772, -0.0357410679969885, -0.020635115229186, -0.018239037386404},
    {0.0100234421734307, -0.0354703486038675, -0.0204788153146927, -0.0449826285156207, 0.0141189246665008, 0.0109364720199429, 0.00631417506469886, 0.00446479600584751, 0.00257775117585285, -0.0211438459052229, 0, 0, 0, 0, 0.00339592394974697, 0.00263047138047138, 0.00151870335961076, -0.0308358534689291, -0.017803088300978, -0.0319414381914382, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0.0225260111157848, 0.0174485731815835, 0.0100739384233621, -0.00356167508620757, -0.00205633406978789, -0.0145404776513196, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0.0282557952193589, 0.0218868448636849, 0.01263637577376, 0.047654757329318, 0.027513486972248, 0.0303983956440067},
    {-0.010023442173431, 0.0128297005588457, 0.00740723107127182, 0.0449826285156208, 0, 0, 0, -0.0178591840233902, -0.0103110047034114, 0.021143845905223, 0, 0, 0, 0, 0, 0, 0, -0.00920473237878481, -0.00531435471670985, 0.0319414381914381, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, -0.0178083754310377, -0.0102816703489396, 0.0145404776513196, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, -0.0297842233308237, -0.017195929357655, -0.0303983956440068},
    {-0.0100234421734306, -0.0324515955311977, -0.0335503995581136, -0.00801060507812422, 0.0211783869997512, -0.00109364720199428, -0.00568275755822895, 0.0169662248222208, 0.00567105258687631, 0.00838462854862288, -0.0208787193108001, -0.0109235523792604, -0.00499002532443533, -0.00187890812890813, -0.0162519217595033, -0.00958243145743147, -0.00466458889023305, -0.0108922666482287, -0.00558007245254534, -0.00469727032227036, 0.0139343556230002, 0.00245778931055952, -0.00415442218609133, -0.00643600317593853, -0.00464478520766674, 0.0120907247401236, 0.00464478520766673, 0.000719567030240151, -0.000415442218609145, 0.00965400476390776, 0.00498530662330958, 0.00215870109072046, 0.006614539445814, 0.00323138210966669, 0.00290809553026394, 0, 0.0127646671417816, 0.0202632808954031, 0.0228341307513453, 0.020635115229186, 0.0127646671417816, -0.00116524935543314, 0.00719357335632687, 0.0112908326677739, 0.01143686949355, 0.00737991258440983, -0.00183832217438609, 0.00310733161448835, 0.00481385743762286, 0.00347410235931504, -0.00201827108709706, 0.00052111535389726, 0.000902598269554298, -0.00170195561890421, -0.000491312267361563, -0.000868525589828763},
    {0.0100234421734315, 0.024904712849524, 0.0553363732971485, -0.00431340273437464, -0.0141189246665008, -0.0153110608279201, 0.00883984509057836, -0.00803663281052558, -0.0159820572902878, 0.000729098134662864, 0.00298267418725715, 0.0277290175781224, 0.0301571095694136, 0.0191648629148629, -0.00145539597846298, 0.0116492303992304, 0.00976309302606916, -0.00352848074520084, 0.00132858867917744, -0.00263047138047133, 0, -0.0196623144844763, -0.0072702388256598, 0.00724050357293084, 0.0092895704153335, 0.00219831358911338, -0.0148633126645336, -0.0129522065443228, -0.0066470754977461, 0.00402250198496159, -0.00685479660705068, -0.00575653624192122, 0.0045792965394097, -0.000293762009969664, 0.00290809553026391, 0, 0, -0.0202632808954031, -0.0399597288148543, -0.0464290092656686, -0.031911667854454, 0, 0.00411061334647249, -0.00303983956440066, -0.0090822198919368, -0.00776832903622086, -0.000459580543596519, 0.0042725809699215, 0.00270779480866287, 0.000868525589828796, -0.00100913554354854, 0.00234501909253766, 0.00180519653910856, -0.00127646671417816, 0.00024565613368078, -0.00086852558982877},
    {0.0100234421734312, -0.0354703486038675, -0.0492363006502186, -0.0043134027343746, 0.0141189246665008, 0.00656188321196572, -0.016416855168217, 0.0098225512128646, 0.0149509568199467, 0.000729098134662864, 0, 0.0033610930397724, -0.0282044909641998, -0.0353234728234728, -0.000485131992820996, 0.0124007936507936, 0.00889526253486298, -0.00291483191994851, 0.00239145962251946, -0.00263047138047141, 0, 0, 0.0218107164769794, 0.00402250198496156, -0.00928957041533348, 0, -0.0055737422492001, 0.0143913406048031, 0.0141250354327105, 0.00080450039699229, -0.00934744991870546, -0.00287826812096059, 0.00254405363300538, -0.00381890612960615, 0.00290809553026392, 0, 0, 0, 0.0266398192099029, 0.0515877880729651, 0.0425488904726054, 0, 0, -0.00911951869320204, -0.00840946286290444, -0.00388416451811041, 0, 0.00233049871086624, -0.0030086608985143, -0.00347410235931509, -0.000336378514516169, 0.00286613444643491, 0.00120346435940572, -0.000850977809452107, 0.000982624534723155, -0.000868525589828754},
    {-0.010023442173431, 0.0128297005588457, 0.0448791059024117, -0.00801060507812423, 0, -0.0174983552319087, 0.0132597676358676, 0.0035718368046781, -0.0175287079957995, 0.00838462854862291, 0, 0, -0.00867830491206147, 0.0225468975468975, 0, 0.00300625300625302, -0.019200749617936, -0.000613648825252327, 0.0122230158484327, -0.00469727032227032, 0, 0, 0, -0.0160900079398463, 0.00464478520766674, 0, 0, 0.00719567030240152, -0.0108014976838374, 0, -0.00249265331165479, 0.0107935054536023, 0.000508810726601078, -0.00734405024924251, 0.00290809553026392, 0, 0, 0, 0, -0.0257938940364826, -0.031911667854454, 0, 0, 0, 0.0134551405806471, 0.0135945758133865, 0, 0, -0.00601732179702859, -0.00347410235931504, 0, 0.00208446141558904, -0.000902598269554294, -0.000425488904726048, 0.0017195929357655, -0.000868525589828768},
    {-0.0100234421734308, -0.0452812960900435, -0.0113287063442981, -0.00801060507812425, -0.00141189246665006, 0.0164047080299144, 0.0145226026488073, 0.0133943880175427, 0.0118576554089232, 0.0083846285486229, -0.00894802256177148, -0.0142846454190327, -0.014536160727703, -0.00939454064454062, -0.0123708658169354, -0.0125886844636845, -0.00813591085505763, -0.0102786178229763, -0.00664294339588729, -0.00469727032227033, -0.00836061337380013, 0.00245778931055953, 0.00830884437218264, 0.00965400476390776, 0.00650269929073342, 0.00219831358911336, 0.00743165633226678, 0.00863480436288185, 0.00581619106052787, 0.00643600317593851, 0.00747795993496438, 0.00503696921168107, 0.00610572871921291, 0.00411266813957582, 0.00290809553026393, 0.0352796762358899, 0.0191470007126724, 0.00788016479265675, 0.000951422114639387, -0.00221090520312706, -0.00212744452363027, 0.017478740331497, 0.00719357335632688, 0.000868525589828766, -0.00201827108709706, -0.00194208225905523, 0.00643412761035128, 0.00077683290362208, -0.00180519653910858, -0.00173705117965754, 0.000672757029032357, -0.00156334606169177, -0.00150433044925715, -0.00127646671417815, -0.00122828066840394, -0.000868525589828763},
    {0.010023442173431, 0.0603750614533912, -0.00610007264692973, -0.0043134027343746, -0.00564756986660029, -0.0218729440398858, 0.00126283501293976, -0.0178591840233903, 0.00103110047034112, 0.00072909813466285, 0.0417574386216002, 0.0168054651988621, -0.000867830491206138, -0.00526094276094278, 0.0145539597846298, -0.000751563251563223, -0.00455611007883228, -0.000613648825252315, -0.00372004830169692, -0.00263047138047137, 0.00557374224920009, -0.0196623144844763, -0.0103860554652283, 0.00321800158796927, 0.00650269929073345, -0.0175865087129071, -0.00928957041533348, 0.00287826812096062, 0.00581619106052783, -0.00804500396992314, 0.0024926533116548, 0.00503696921168107, 0.00203524290640432, 0.00411266813957582, 0.00290809553026392, -0.0705593524717798, -0.0127646671417816, 0.00450295131008957, 0.00380568845855756, -0.000736968401042374, -0.00212744452363027, -0.0116524935543313, 0.0041106133464725, 0.00347410235931506, -0.000672757029032359, -0.0019420822590552, 0.00367664434877216, 0.00310733161448835, -0.000601732179702866, -0.00173705117965752, 0.00269102811612943, -0.000521115353897279, -0.00150433044925714, -0.000425488904726059, -0.00122828066840393, -0.000868525589828762},
    {0.010023442173431, -0.0603750614533913, -0.00610007264692972, -0.00431340273437459, -0.00564756986660034, 0.0218729440398858, 0.00126283501293975, 0.0178591840233902, 0.00103110047034114, 0.000729098134662857, -0.0417574386216002, 0.016805465198862, 0.000867830491206142, -0.00526094276094274, 0.0145539597846298, 0.000751563251563242, -0.00455611007883226, 0.000613648825252326, -0.0037200483016969, -0.00263047138047136, 0.0055737422492001, 0.0196623144844763, -0.0103860554652283, -0.00321800158796928, 0.00650269929073344, 0.0175865087129071, -0.00928957041533347, -0.00287826812096062, 0.00581619106052785, -0.00804500396992314, -0.0024926533116548, 0.00503696921168109, -0.00203524290640433, 0.00411266813957582, 0.00290809553026392, 0.0705593524717798, -0.0127646671417816, -0.00450295131008957, 0.00380568845855755, 0.000736968401042381, -0.00212744452363028, -0.0116524935543313, -0.00411061334647249, 0.00347410235931505, 0.00067275702903236, -0.00194208225905522, -0.00367664434877216, 0.00310733161448835, 0.000601732179702862, -0.00173705117965754, 0.00269102811612941, 0.000521115353897267, -0.00150433044925715, 0.000425488904726055, -0.00122828066840394, -0.00086852558982877},
    {-0.0100234421734311, 0.0452812960900435, -0.0113287063442981, -0.00801060507812424, -0.00141189246665007, -0.0164047080299144, 0.0145226026488074, -0.0133943880175427, 0.0118576554089232, 0.00838462854862294, 0.00894802256177145, -0.0142846454190327, 0.014536160727703, -0.00939454064454065, -0.0123708658169354, 0.0125886844636845, -0.00813591085505765, 0.0102786178229763, -0.00664294339588732, -0.00469727032227034, -0.00836061337380011, -0.00245778931055953, 0.00830884437218264, -0.00965400476390776, 0.00650269929073342, -0.00219831358911338, 0.00743165633226677, -0.00863480436288184, 0.00581619106052786, 0.00643600317593851, -0.00747795993496437, 0.00503696921168106, -0.0061057287192129, 0.00411266813957581, 0.00290809553026391, -0.0352796762358899, 0.0191470007126724, -0.00788016479265674, 0.000951422114639392, 0.00221090520312706, -0.00212744452363026, 0.017478740331497, -0.00719357335632687, 0.000868525589828777, 0.00201827108709706, -0.00194208225905521, -0.00643412761035126, 0.000776832903622074, 0.00180519653910858, -0.00173705117965752, 0.00067275702903235, 0.00156334606169178, -0.00150433044925714, 0.00127646671417815, -0.00122828066840393, -0.000868525589828756},
    {0.0787556170769578, 0.0679219441350652, -0.00435719474780692, -0.00308100195312471, 0.0705946233325039, -0.0328094160598287, 0.0126283501293978, -0.0267887760350854, 0.03608851646194, -0.00729098134662863, 0.0298267418725716, 0.0151249186789758, -0.0325436434202305, 0.0248015873015872, 0.0130985638061668, -0.0229226791726792, 0.0169226945785199, -0.0248527774227189, 0.0164744996218005, 0.0142797017797018, 0, 0.0245778931055953, -0.0363511941282991, 0.0362025178646542, -0.0232239260383337, 0.0219831358911338, -0.00743165633226679, 0, 0.00166176887443652, -0.0345935170706695, 0.0130864298861876, -0.00359783515120077, 0.0335815079556711, -0.014688100498485, -0.0178640154001927, 0, 0, 0, 0, 0, 0, 0, 0.0226083734055987, -0.0334382352084074, 0.0333014729371016, -0.0213629048496074, -0.00459580543596519, -0.0194208225905522, 0.0270779480866286, -0.0191075629762328, 0.0090822198919368, 0.00703505727761299, -0.00812338442598857, -0.0102117337134253, 0.00196524906944628, 0.00607967912880137},
    {-0.0214788046573525, 0.00377344134083702, 0.00217859737390337, -0.021567013671873, 0, 0.0984282481794862, -0.0852413633734345, -0.0312535720409329, -0.0180442582309699, 0.0309866707231716, 0, 0.0453747560369276, 0, -0.0202922077922077, -0.0116431678277039, 0.0236742424242425, -0.0335199527228374, -0.00138070985681772, -0.000797153207506455, 0.00394570707070704, 0, 0, 0.0363511941282991, -0.0603375297744235, 0.0464478520766673, 0, 0.0241528830798671, 0, -0.0108014976838374, -0.00724050357293081, -0.0392592896585629, 0.0259044130886455, 0.0188259968842399, 0.0108691943688789, -0.0178640154001926, 0, 0, 0, 0, 0, 0, 0, 0, 0.0334382352084074, -0.0555024548951693, 0.0427258096992148, 0, -0.0155366580724417, 0, 0.00694820471863008, 0.00302740663064561, 0.0164151336477636, -0.0108311792346514, -0.00680782247561684, -0.00393049813889257, 0.00607967912880131},
    {0.0787556170769581, 0.0301875307266957, 0.0610007264692977, -0.00308100195312485, 0, 0.0218729440398858, 0.0757701007763862, 0.0178591840233902, -0.0412440188136458, -0.00729098134662867, 0, 0, 0.0520698294723689, -0.00901875901875909, 0, -0.0127765752765753, 0.0286384062098029, 0.00184094647575695, -0.0297603864135753, 0.0142797017797018, 0, 0, 0, 0.0402250198496157, -0.0464478520766674, 0, 0, 0.014391340604803, 0.0182794576188018, 0, -0.0137095932141014, -0.0345392174515274, 0.00407048581280862, 0.036426489236243, -0.0178640154001927, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0.0370016365967796, -0.0427258096992148, 0, 0, -0.030086608985143, 0.0243187165152054, 0, 0.014070114555226, 0, -0.00340391123780844, -0.00982624534723146, 0.00607967912880138},
    {-0.0214788046573519, 0.00377344134083692, -0.0196073763651313, 0.00924300585937408, 0, -0.0109364720199429, 0.00947126259704824, 0.102690308134494, -0.0695992817480272, -0.060150596109686, 0, -0.00504163955965862, 0.0065087286840461, -0.00450937950937948, 0.0465726713108155, 0.0214195526695527, -0.0289638426440052, -0.00782402252196701, -0.0263060558477138, 0.000563672438672498, 0, 0, 0, 0, 0, 0, 0.0334424534952005, -0.0431740218144092, 0.0299118397398575, 0.0289620142917233, -0.00249265331165477, -0.00431740218144094, -0.0605484764655283, 0.0185070066280912, 0.0461140862656136, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0.0349574806629939, -0.0451299134777144, 0.0312669212338355, -0.0121096265225824, -0.028140229110452, 0.0270779480866286, 0.023827378664659, 0, -0.0182390373864041},
    {-0.0214788046573522, -0.0150937653633478, 0.0130715842434209, 0.00924300585937409, 0, -0.0109364720199429, 0.00947126259704823, -0.00892959201169517, 0.123732056440937, -0.060150596109686, 0, 0, -0.00650872868404615, 0.00676406926406924, 0, 0.0574945887445888, 0.0126920209338899, -0.026693723898476, 0.00637722566005184, 0.000563672438672442, 0, 0, 0, 0, 0, 0, 0, 0.0431740218144092, -0.0448677596097863, 0, 0.0199412264932383, 0.0215870109072046, -0.0142467003448301, -0.0616900220936373, 0.0461140862656136, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0.0451299134777144, -0.0469003818507533, 0, -0.0375203054806026, 0.0162467688519772, 0.0119136893323295, 0.0206351152291861, -0.0182390373864041},
    {0.078755617076958, 0.0301875307266957, 0.0174287789912279, 0.0585390371093694, 0, 0.0218729440398858, -0.0189425251940966, 0.0178591840233902, 0.0103110047034113, 0.083846285486229, 0, 0, 0, 0, 0, 0.00526094276094275, -0.00455611007883227, 0.0533874477969518, 0.0308232573569172, -0.0319414381914382, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0.0348971463631671, -0.0302218152700865, 0.0284934006896603, 0.0164506725583033, -0.0643935438844155, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0.0437736897273697, -0.0379091273212801, -0.023827378664659, -0.0137567434861241, 0.0303983956440068},
    {0.0787556170769583, -0.0679219441350652, -0.00435719474780694, -0.00308100195312467, 0.0705946233325039, 0.0328094160598287, 0.0126283501293977, 0.0267887760350853, 0.03608851646194, -0.00729098134662864, -0.0298267418725716, 0.0151249186789759, 0.0325436434202305, 0.0248015873015873, 0.0130985638061668, 0.0229226791726792, 0.0169226945785199, 0.0248527774227189, 0.0164744996218005, 0.0142797017797018, 0, -0.0245778931055954, -0.036351194128299, -0.0362025178646541, -0.0232239260383336, -0.0219831358911338, -0.00743165633226678, 0, 0.00166176887443654, -0.0345935170706695, -0.0130864298861876, -0.00359783515120077, -0.0335815079556711, -0.014688100498485, -0.0178640154001927, 0, 0, 0, 0, 0, 0, 0, -0.0226083734055987, -0.0334382352084074, -0.0333014729371016, -0.0213629048496074, 0.00459580543596522, -0.0194208225905522, -0.0270779480866286, -0.0191075629762328, 0.0090822198919368, -0.007035057277613, -0.00812338442598858, 0.0102117337134253, 0.00196524906944627, 0.00607967912880136},
    {-0.0214788046573522, -0.00377344134083698, 0.00217859737390352, -0.0215670136718729, 0, -0.0984282481794863, -0.0852413633734345, 0.031253572040933, -0.0180442582309699, 0.0309866707231716, 0, 0.0453747560369276, 0, -0.0202922077922078, -0.0116431678277039, -0.0236742424242425, -0.0335199527228375, 0.00138070985681774, -0.000797153207506485, 0.00394570707070704, 0, 0, 0.0363511941282991, 0.0603375297744236, 0.0464478520766674, 0, 0.024152883079867, 0, -0.0108014976838374, -0.0072405035729308, 0.0392592896585629, 0.0259044130886455, -0.0188259968842399, 0.0108691943688789, -0.0178640154001927, 0, 0, 0, 0, 0, 0, 0, 0, 0.0334382352084074, 0.0555024548951693, 0.0427258096992148, 0, -0.0155366580724417, 0, 0.0069482047186301, 0.0030274066306456, -0.0164151336477636, -0.0108311792346515, 0.00680782247561683, -0.00393049813889256, 0.00607967912880134},
    {0.078755617076958, -0.0301875307266958, 0.0610007264692976, -0.00308100195312472, 0, -0.0218729440398859, 0.0757701007763862, -0.0178591840233903, -0.0412440188136458, -0.00729098134662865, 0, 0, -0.0520698294723688, -0.00901875901875904, 0, 0.0127765752765752, 0.0286384062098029, -0.00184094647575696, -0.0297603864135752, 0.0142797017797018, 0, 0, 0, -0.0402250198496157, -0.0464478520766673, 0, 0, -0.0143913406048031, 0.0182794576188018, 0, 0.0137095932141014, -0.0345392174515273, -0.00407048581280864, 0.036426489236243, -0.0178640154001927, 0, 0, 0, 0, 0, 0, 0, 0, 0, -0.0370016365967795, -0.0427258096992148, 0, 0, 0.0300866089851429, 0.0243187165152054, 0, -0.014070114555226, 0, 0.00340391123780844, -0.00982624534723146, 0.00607967912880137},
    {-0.0214788046573522, -0.00377344134083677, -0.0196073763651313, 0.00924300585937403, 0, 0.0109364720199429, 0.00947126259704829, -0.102690308134494, -0.0695992817480271, -0.0601505961096861, 0, -0.00504163955965865, -0.00650872868404618, -0.00450937950937947, 0.0465726713108155, -0.0214195526695527, -0.0289638426440052, 0.00782402252196705, -0.0263060558477139, 0.000563672438672498, 0, 0, 0, 0, 0, 0, 0.0334424534952005, 0.0431740218144092, 0.0299118397398575, 0.0289620142917233, 0.00249265331165484, -0.00431740218144093, 0.0605484764655283, 0.0185070066280911, 0.0461140862656136, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0.0349574806629939, 0.0451299134777144, 0.0312669212338355, -0.0121096265225824, 0.028140229110452, 0.0270779480866286, -0.023827378664659, 0, -0.018239037386404},
    {-0.0214788046573521, 0.0150937653633479, 0.0130715842434208, 0.00924300585937423, 0, 0.010936472019943, 0.00947126259704825, 0.00892959201169522, 0.123732056440937, -0.060150596109686, 0, 0, 0.00650872868404613, 0.00676406926406924, 0, -0.0574945887445887, 0.0126920209338899, 0.0266937238984759, 0.00637722566005184, 0.000563672438672452, 0, 0, 0, 0, 0, 0, 0, -0.0431740218144092, -0.0448677596097863, 0, -0.0199412264932383, 0.0215870109072046, 0.0142467003448301, -0.0616900220936373, 0.0461140862656136, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, -0.0451299134777144, -0.0469003818507533, 0, 0.0375203054806026, 0.0162467688519772, -0.0119136893323295, 0.0206351152291861, -0.0182390373864041},
    {0.0787556170769579, -0.0301875307266956, 0.017428778991228, 0.0585390371093696, 0, -0.0218729440398858, -0.0189425251940965, -0.0178591840233903, 0.0103110047034114, 0.083846285486229, 0, 0, 0, 0, 0, -0.00526094276094277, -0.00455611007883227, -0.0533874477969518, 0.0308232573569172, -0.0319414381914383, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, -0.0348971463631671, -0.0302218152700865, -0.0284934006896603, 0.0164506725583033, -0.0643935438844154, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, -0.0437736897273697, -0.0379091273212801, 0.023827378664659, -0.0137567434861241, 0.0303983956440068},
    {0.078755617076958, -0.0377344134083697, -0.0566435317214906, -0.00308100195312479, 0.0423567739995023, 0.0546823600997145, 0.0378850503881931, 0.0446479600584756, 0.00515550235170578, -0.00729098134662856, 0.0417574386216002, -0.00504163955965862, -0.0247331689993752, -0.0202922077922078, 0.0295930515620807, 0.0101461038961039, 0.00216957622801541, 0.0266937238984759, 0.0132858867917747, 0.0142797017797018, -0.0557374224920008, -0.0245778931055953, -0.00519302773261415, 0.00402250198496156, 0.00464478520766674, -0.00219831358911332, -0.0130053985814669, -0.0158304746652834, -0.0108014976838375, -0.0168945083368386, -0.026796023100289, -0.0194283098164841, -0.0295110221428625, -0.0217383887377579, -0.0178640154001927, 0, 0, 0, 0, 0, 0, -0.0512709716390577, -0.0226083734055987, -0.00477689074405822, 0.00370016365967794, 0.00427258096992147, -0.0367664434877216, -0.00776832903622086, 0.00601732179702859, 0.00694820471863011, -0.00908221989193683, 0.00703505727761299, 0.00812338442598859, 0.00680782247561683, 0.00786099627778515, 0.00607967912880135},
    {-0.0214788046573522, 0, -0.00435719474780689, -0.0215670136718729, -0.127070321998507, 0, 0.0284137877911447, 0, 0.0360885164619399, 0.0309866707231715, 0, 0.0453747560369275, 0, -0.0202922077922078, -0.0422064833754266, 0, -0.00618329224984384, 0, 0.0015943064150129, 0.00394570707070703, 0.0836061337380012, 0, -0.0103860554652283, 0, 0.00464478520766673, 0, 0.024152883079867, 0, -0.0108014976838374, 0.043443021437585, 0, -0.0194283098164841, 0, -0.0217383887377578, -0.0178640154001926, 0, 0, 0, 0, 0, 0, 0.0769064574585866, 0, -0.00955378148811639, 0, 0.00427258096992146, 0, -0.0155366580724417, 0, 0.00694820471863015, -0.0181644397838736, 0, 0.00812338442598858, 0, 0.00786099627778515, 0.00607967912880134},
    {0.0787556170769581, 0.0377344134083697, -0.0566435317214905, -0.00308100195312477, 0.0423567739995024, -0.0546823600997146, 0.0378850503881931, -0.0446479600584756, 0.00515550235170579, -0.00729098134662863, -0.0417574386216002, -0.00504163955965864, 0.0247331689993752, -0.0202922077922078, 0.0295930515620807, -0.0101461038961039, 0.00216957622801542, -0.0266937238984759, 0.0132858867917747, 0.0142797017797018, -0.0557374224920008, 0.0245778931055954, -0.00519302773261416, -0.00402250198496156, 0.00464478520766676, 0.00219831358911337, -0.0130053985814669, 0.0158304746652834, -0.0108014976838375, -0.0168945083368386, 0.026796023100289, -0.0194283098164841, 0.0295110221428625, -0.0217383887377579, -0.0178640154001927, 0, 0, 0, 0, 0, 0, -0.0512709716390577, 0.0226083734055987, -0.00477689074405822, -0.00370016365967795, 0.00427258096992149, 0.0367664434877216, -0.00776832903622086, -0.00601732179702859, 0.00694820471863009, -0.00908221989193678, -0.00703505727761301, 0.00812338442598859, -0.00680782247561686, 0.00786099627778517, 0.00607967912880135},
    {-0.021478804657352, -0.0188672067041847, 0.00653579212171052, 0.00924300585937401, 0.0141189246665008, 0, -0.00315708753234946, -0.111619900146189, -0.05413277469291, -0.0601505961096861, -0.00894802256177153, -0.00252081977982931, 0.000650872868404653, 0.00112734487734485, -0.0276525235907967, 0.0360750360750361, 0.0374251899332651, -0.0188697013765088, 0.019928830187662, 0.000563672438672417, 0, 0, 0, 0, 0, 0.0593544669060614, 0.0167212267476003, -0.00431740218144091, -0.0074779599349644, 0.00321800158796925, 0.0224338798048931, 0.018708742786244, 0.0463017761206981, 0.0431830154655461, 0.0461140862656137, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0.0620433733855302, 0.017478740331497, -0.00451299134777146, -0.0078167303084589, 0.0363288795677472, -0.00938007637015066, -0.0162467688519772, -0.0119136893323295, -0.020635115229186, -0.018239037386404},
    {-0.021478804657352, 0.0188672067041848, 0.00653579212171056, 0.00924300585937413, 0.0141189246665008, 0, -0.00315708753234944, 0.111619900146189, -0.05413277469291, -0.060150596109686, 0.00894802256177151, -0.00252081977982929, -0.000650872868404627, 0.00112734487734488, -0.0276525235907967, -0.0360750360750361, 0.0374251899332651, 0.0188697013765088, 0.0199288301876619, 0.000563672438672459, 0, 0, 0, 0, 0, -0.0593544669060614, 0.0167212267476003, 0.00431740218144093, -0.0074779599349644, 0.00321800158796927, -0.0224338798048931, 0.018708742786244, -0.046301776120698, 0.0431830154655461, 0.0461140862656136, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, -0.0620433733855302, 0.017478740331497, 0.00451299134777145, -0.00781673030845891, 0.0363288795677472, 0.00938007637015065, -0.0162467688519772, 0.0119136893323295, -0.020635115229186, -0.0182390373864041},
    {0.0787556170769582, 0, -0.0348575579824558, 0.0585390371093696, -0.0282378493330016, 0, 0.00631417506469889, 0, -0.0206220094068229, 0.083846285486229, 0, 0, 0, 0, -0.00679184789949391, 0, 0.00151870335961076, 0, -0.0616465147138344, -0.0319414381914382, 0, 0, 0, 0, 0, 0, 0, 0, 0, -0.0450520222315696, 0, 0.0100739384233621, 0, -0.0329013451166065, -0.0643935438844154, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, -0.0565115904387179, 0, 0.01263637577376, 0, 0.027513486972248, 0.0303983956440067},
    {0.0787556170769578, -0.0377344134083695, -0.0217859737390347, -0.0523770332031201, 0.0423567739995023, 0.0546823600997146, 0, 0.0446479600584757, 0.0257775117585286, 0.0291639253865144, 0.0417574386216002, 0.0285692908380655, 0.0273366604729936, 0.0187890812890813, -0.00921750786359888, -0.00789141414141413, -0.00347132196482459, -0.0248527774227189, -0.0143487577351166, -0.015031265031265, -0.0557374224920008, -0.00491557862111907, -0.0197335053839337, -0.0394205194526234, -0.0325134964536672, -0.0241814494802472, -0.00928957041533347, -0.0158304746652834, -0.0141250354327105, -0.00241350119097692, -0.000623163327913714, -0.00215870109072046, 0.00712335017241511, 0.00411266813957583, 0.00540074884191872, 0, -0.0510586685671264, -0.0405265617908061, -0.0114170653756727, 0.010317557614593, 0.0127646671417816, 0.00466099742173251, -0.0143871467126538, -0.00564541633388696, 0.00571843474677503, 0.00737991258440982, 0.00367664434877214, -0.00155366580724416, 0.00240692871881144, 0.00347410235931507, 0.00100913554354852, 0.000260557676948644, 0.000902598269554298, -0.000850977809452118, -0.000491312267361587, -0.000868525589828769},
    {-0.0214788046573518, 0, -0.0217859737390347, 0.00308100195312481, -0.127070321998507, 0, 0.0473563129852414, 0, 0.0257775117585285, 0.0127592173566, 0, -0.0352914769176103, 0, 0.00225468975468975, 0.0509388592462044, 0, -0.00943765659186686, 0, -0.0143487577351166, -0.0129644660894661, 0.0836061337380013, 0, 0.0477758551400502, 0, -0.0325134964536671, 0, 0.00928957041533347, 0, -0.0141250354327104, -0.0144810071458617, 0, -0.00215870109072047, 0, 0.00411266813957582, 0.00540074884191876, 0, 0.0765880028506897, 0, -0.0228341307513453, 0, 0.0127646671417816, -0.00699149613259876, 0, -0.0112908326677739, 0, 0.00737991258440979, 0, -0.00310733161448836, 0, 0.00347410235931506, 0.00201827108709705, 0, 0.000902598269554284, 0, -0.000491312267361592, -0.000868525589828792},
    {0.0787556170769577, 0.0377344134083696, -0.0217859737390349, -0.05237703320312, 0.0423567739995023, -0.0546823600997146, 0, -0.0446479600584756, 0.0257775117585286, 0.0291639253865144, -0.0417574386216002, 0.0285692908380655, -0.0273366604729936, 0.0187890812890813, -0.0092175078635989, 0.00789141414141413, -0.00347132196482458, 0.0248527774227189, -0.0143487577351166, -0.015031265031265, -0.0557374224920009, 0.00491557862111903, -0.0197335053839338, 0.0394205194526234, -0.0325134964536671, 0.0241814494802472, -0.00928957041533348, 0.0158304746652834, -0.0141250354327105, -0.00241350119097695, 0.000623163327913703, -0.00215870109072045, -0.00712335017241511, 0.00411266813957582, 0.00540074884191871, 0, -0.0510586685671264, 0.0405265617908061, -0.0114170653756726, -0.010317557614593, 0.0127646671417816, 0.0046609974217325, 0.0143871467126537, -0.00564541633388698, -0.00571843474677501, 0.00737991258440984, -0.00367664434877218, -0.00155366580724417, -0.00240692871881142, 0.00347410235931505, 0.00100913554354854, -0.000260557676948633, 0.000902598269554291, 0.000850977809452119, -0.000491312267361577, -0.000868525589828757},
    {-0.0214788046573522, -0.0188672067041847, 0.0108929868695173, 0.00308100195312474, 0.0141189246665008, -0.109364720199429, -0.0789271883087357, 0.0223239800292378, -0.0128887558792642, 0.0127592173566001, -0.00894802256177151, -0.0277290175781224, -0.00585785581564148, 0.0191648629148629, 0.00145539597846299, 0.0383297258297258, 0.0348216984596467, -0.0124263887113595, 0.00717437886755833, -0.0129644660894661, 0, 0.0589869434534289, 0.00727023882565978, 0.0506835250105158, 0.0650269929073344, -0.00659494076734013, 0.0148633126645336, -0.00431740218144089, -0.00166176887443655, -0.0040225019849616, -0.00810112326287806, -0.0115130724838424, 0.00356167508620753, -0.00205633406978792, 0.00540074884191869, 0, 0, 0.0607898426862092, 0.0399597288148543, -0.0154763364218895, -0.031911667854454, 0, -0.0123318400394175, 0.00303983956440066, -0.00302740663064559, -0.00776832903622087, 0.00137874163078958, -0.0042725809699215, 0.000902598269554286, 0.000868525589828745, 0.00100913554354854, 0.000781673030845884, 0.00180519653910853, -0.000425488904726043, 0.0002456561336808, -0.000868525589828759},
    {-0.0214788046573523, 0.0188672067041848, 0.0108929868695173, 0.00308100195312473, 0.0141189246665008, 0.109364720199429, -0.0789271883087356, -0.0223239800292378, -0.0128887558792643, 0.0127592173566, 0.00894802256177147, -0.0277290175781224, 0.00585785581564147, 0.0191648629148629, 0.00145539597846298, -0.0383297258297258, 0.0348216984596467, 0.0124263887113595, 0.00717437886755829, -0.0129644660894661, 0, -0.0589869434534289, 0.0072702388256598, -0.0506835250105158, 0.0650269929073343, 0.00659494076734013, 0.0148633126645336, 0.0043174021814409, -0.00166176887443651, -0.00402250198496155, 0.00810112326287803, -0.0115130724838425, -0.00356167508620755, -0.0020563340697879, 0.00540074884191873, 0, 0, -0.0607898426862092, 0.0399597288148542, 0.0154763364218895, -0.031911667854454, 0, 0.0123318400394175, 0.00303983956440069, 0.0030274066306456, -0.00776832903622087, -0.00137874163078958, -0.00427258096992146, -0.000902598269554311, 0.000868525589828782, 0.00100913554354855, -0.000781673030845884, 0.00180519653910857, 0.000425488904726043, 0.000245656133680809, -0.000868525589828767},
    {0.0787556170769581, 0, 0.0435719474780695, -0.0523770332031201, -0.0282378493330016, 0, 0.0631417506469885, 0, -0.0515550235170572, 0.0291639253865144, 0, -0.00672218607954481, 0, -0.0601250601250601, 0.000970263985641982, 0, -0.0125835421224891, 0, 0.0286975154702333, -0.015031265031265, 0, 0, -0.0436214329539588, 0, -0.0650269929073343, 0, 0.0111474844984002, 0, 0.031573608614294, -0.00160900079398462, 0, -0.0028782681209606, 0, -0.00822533627915164, 0.00540074884191871, 0, 0, 0, -0.0532796384198057, 0, 0.0425488904726053, 0, 0, 0.018239037386404, 0, -0.00388416451811043, 0, -0.00466099742173252, 0, -0.00347410235931507, 0.00067275702903234, 0, 0.00120346435940573, 0, 0.000982624534723144, -0.000868525589828764},
    {0.10739402328676, -0.188672067041848, -0.108929868695174, -0.0770250488281177, -0.0705946233325038, -0.0546823600997147, -0.0315708753234941, -0.0446479600584757, -0.0257775117585286, -0.0182274533665716, 0.0894802256177148, -0.0151249186789758, 0.058578558156415, 0.0744047619047618, -0.0130985638061668, -0.0281836219336219, 0.00976309302606916, 0.069035492840886, 0.0398576603753241, 0.0597492784992785, 0, 0.0737336793167861, 0.036351194128299, -0.0120675059548847, -0.0232239260383337, 0.0659494076734016, 0.00743165633226678, 0.0431740218144092, 0.0465295284842228, 0.0345935170706695, 0.0155790831978425, 0.0251848460584054, -0.0254405363300539, -0.0146881004984851, -0.0328199352701214, 0, 0, 0, 0, 0, 0, 0, 0.0678251202167962, 0.0334382352084074, -0.0111004909790339, -0.0213629048496074, -0.0137874163078956, 0.0194208225905522, -0.0090259826955429, -0.0191075629762328, -0.00908221989193678, -0.00234501909253769, -0.00812338442598858, 0.00340391123780841, 0.00196524906944627, 0.00607967912880134},
    {0.107394023286761, 0.188672067041848, -0.108929868695174, -0.0770250488281178, -0.0705946233325039, 0.0546823600997145, -0.0315708753234942, 0.0446479600584757, -0.0257775117585287, -0.0182274533665715, -0.0894802256177147, -0.0151249186789758, -0.0585785581564149, 0.0744047619047619, -0.0130985638061668, 0.0281836219336219, 0.00976309302606916, -0.0690354928408859, 0.039857660375324, 0.0597492784992785, 0, -0.0737336793167861, 0.036351194128299, 0.0120675059548847, -0.0232239260383338, -0.0659494076734015, 0.00743165633226678, -0.0431740218144092, 0.0465295284842228, 0.0345935170706695, -0.0155790831978424, 0.0251848460584053, 0.0254405363300539, -0.0146881004984851, -0.0328199352701214, 0, 0, 0, 0, 0, 0, 0, -0.0678251202167962, 0.0334382352084074, 0.0111004909790339, -0.0213629048496074, 0.0137874163078956, 0.0194208225905522, 0.00902598269554287, -0.0191075629762328, -0.0090822198919368, 0.00234501909253767, -0.00812338442598857, -0.0034039112378084, 0.00196524906944627, 0.00607967912880135},
    {0.107394023286761, 0, 0.217859737390348, -0.0770250488281178, 0, 0, -0.094712625970483, 0, 0.0515550235170572, -0.0182274533665716, 0, -0.0907495120738551, 0, -0.0946969696969696, 0.0232863356554077, 0, -0.0227805503941613, 0, -0.0797153207506479, 0.0597492784992786, 0, 0, -0.0727023882565981, 0, 0.0464478520766674, 0, -0.0483057661597341, 0, -0.0781031370985168, 0.0144810071458616, 0, 0.0431740218144092, 0, 0.0293762009969701, -0.0328199352701215, 0, 0, 0, 0, 0, 0, 0, 0, -0.0668764704168148, 0, 0.0427258096992148, 0, 0.0310733161448835, 0, 0.00694820471863007, -0.00605481326129118, 0, -0.0108311792346515, 0, -0.00393049813889257, 0.00607967912880135},
    {0.107394023286761, 0, 0, 0.231075146484353, 0, 0, 0, 0, 0, -0.109364720199429, 0, 0.0100832791193172, 0, -0.00450937950937963, -0.093145342621631, 0, -0.104139658944738, 0, 0, -0.09244227994228, 0, 0, 0, 0, 0, 0, -0.066884906990401, 0, 0.0299118397398574, -0.0579240285834467, 0, -0.0647610327216138, 0, 0, 0.0835038859404354, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, -0.0699149613259879, 0, 0.0312669212338356, 0.0242192530451648, 0, 0.0270779480866286, 0, 0, -0.018239037386404}};
    
    // Extract relevant coefficients
    const double coeff0_0 = coefficients0[dof][0];
    const double coeff0_1 = coefficients0[dof][1];
    const double coeff0_2 = coefficients0[dof][2];
    const double coeff0_3 = coefficients0[dof][3];
    const double coeff0_4 = coefficients0[dof][4];
    const double coeff0_5 = coefficients0[dof][5];
    const double coeff0_6 = coefficients0[dof][6];
    const double coeff0_7 = coefficients0[dof][7];
    const double coeff0_8 = coefficients0[dof][8];
    const double coeff0_9 = coefficients0[dof][9];
    const double coeff0_10 = coefficients0[dof][10];
    const double coeff0_11 = coefficients0[dof][11];
    const double coeff0_12 = coefficients0[dof][12];
    const double coeff0_13 = coefficients0[dof][13];
    const double coeff0_14 = coefficients0[dof][14];
    const double coeff0_15 = coefficients0[dof][15];
    const double coeff0_16 = coefficients0[dof][16];
    const double coeff0_17 = coefficients0[dof][17];
    const double coeff0_18 = coefficients0[dof][18];
    const double coeff0_19 = coefficients0[dof][19];
    const double coeff0_20 = coefficients0[dof][20];
    const double coeff0_21 = coefficients0[dof][21];
    const double coeff0_22 = coefficients0[dof][22];
    const double coeff0_23 = coefficients0[dof][23];
    const double coeff0_24 = coefficients0[dof][24];
    const double coeff0_25 = coefficients0[dof][25];
    const double coeff0_26 = coefficients0[dof][26];
    const double coeff0_27 = coefficients0[dof][27];
    const double coeff0_28 = coefficients0[dof][28];
    const double coeff0_29 = coefficients0[dof][29];
    const double coeff0_30 = coefficients0[dof][30];
    const double coeff0_31 = coefficients0[dof][31];
    const double coeff0_32 = coefficients0[dof][32];
    const double coeff0_33 = coefficients0[dof][33];
    const double coeff0_34 = coefficients0[dof][34];
    const double coeff0_35 = coefficients0[dof][35];
    const double coeff0_36 = coefficients0[dof][36];
    const double coeff0_37 = coefficients0[dof][37];
    const double coeff0_38 = coefficients0[dof][38];
    const double coeff0_39 = coefficients0[dof][39];
    const double coeff0_40 = coefficients0[dof][40];
    const double coeff0_41 = coefficients0[dof][41];
    const double coeff0_42 = coefficients0[dof][42];
    const double coeff0_43 = coefficients0[dof][43];
    const double coeff0_44 = coefficients0[dof][44];
    const double coeff0_45 = coefficients0[dof][45];
    const double coeff0_46 = coefficients0[dof][46];
    const double coeff0_47 = coefficients0[dof][47];
    const double coeff0_48 = coefficients0[dof][48];
    const double coeff0_49 = coefficients0[dof][49];
    const double coeff0_50 = coefficients0[dof][50];
    const double coeff0_51 = coefficients0[dof][51];
    const double coeff0_52 = coefficients0[dof][52];
    const double coeff0_53 = coefficients0[dof][53];
    const double coeff0_54 = coefficients0[dof][54];
    const double coeff0_55 = coefficients0[dof][55];
    
    // Compute value(s)
    *values = coeff0_0*basisvalue0 + coeff0_1*basisvalue1 + coeff0_2*basisvalue2 + coeff0_3*basisvalue3 + coeff0_4*basisvalue4 + coeff0_5*basisvalue5 + coeff0_6*basisvalue6 + coeff0_7*basisvalue7 + coeff0_8*basisvalue8 + coeff0_9*basisvalue9 + coeff0_10*basisvalue10 + coeff0_11*basisvalue11 + coeff0_12*basisvalue12 + coeff0_13*basisvalue13 + coeff0_14*basisvalue14 + coeff0_15*basisvalue15 + coeff0_16*basisvalue16 + coeff0_17*basisvalue17 + coeff0_18*basisvalue18 + coeff0_19*basisvalue19 + coeff0_20*basisvalue20 + coeff0_21*basisvalue21 + coeff0_22*basisvalue22 + coeff0_23*basisvalue23 + coeff0_24*basisvalue24 + coeff0_25*basisvalue25 + coeff0_26*basisvalue26 + coeff0_27*basisvalue27 + coeff0_28*basisvalue28 + coeff0_29*basisvalue29 + coeff0_30*basisvalue30 + coeff0_31*basisvalue31 + coeff0_32*basisvalue32 + coeff0_33*basisvalue33 + coeff0_34*basisvalue34 + coeff0_35*basisvalue35 + coeff0_36*basisvalue36 + coeff0_37*basisvalue37 + coeff0_38*basisvalue38 + coeff0_39*basisvalue39 + coeff0_40*basisvalue40 + coeff0_41*basisvalue41 + coeff0_42*basisvalue42 + coeff0_43*basisvalue43 + coeff0_44*basisvalue44 + coeff0_45*basisvalue45 + coeff0_46*basisvalue46 + coeff0_47*basisvalue47 + coeff0_48*basisvalue48 + coeff0_49*basisvalue49 + coeff0_50*basisvalue50 + coeff0_51*basisvalue51 + coeff0_52*basisvalue52 + coeff0_53*basisvalue53 + coeff0_54*basisvalue54 + coeff0_55*basisvalue55;
  }

  /// Evaluate all basis functions at given point in cell
  virtual void evaluate_basis_all(double* values,
                                  const double* coordinates,
                                  const ufc::cell& c) const
  {
    throw std::runtime_error("The vectorised version of evaluate_basis() is not yet implemented.");
  }

  /// Evaluate order n derivatives of basis function i at given point in cell
  virtual void evaluate_basis_derivatives(unsigned int i,
                                          unsigned int n,
                                          double* values,
                                          const double* coordinates,
                                          const ufc::cell& c) const
  {
    // Extract vertex coordinates
    const double * const * element_coordinates = c.coordinates;
    
    // Compute Jacobian of affine map from reference cell
    const double J_00 = element_coordinates[1][0] - element_coordinates[0][0];
    const double J_01 = element_coordinates[2][0] - element_coordinates[0][0];
    const double J_02 = element_coordinates[3][0] - element_coordinates[0][0];
    const double J_10 = element_coordinates[1][1] - element_coordinates[0][1];
    const double J_11 = element_coordinates[2][1] - element_coordinates[0][1];
    const double J_12 = element_coordinates[3][1] - element_coordinates[0][1];
    const double J_20 = element_coordinates[1][2] - element_coordinates[0][2];
    const double J_21 = element_coordinates[2][2] - element_coordinates[0][2];
    const double J_22 = element_coordinates[3][2] - element_coordinates[0][2];
      
    // Compute sub determinants
    const double d00 = J_11*J_22 - J_12*J_21;
    const double d01 = J_12*J_20 - J_10*J_22;
    const double d02 = J_10*J_21 - J_11*J_20;
    
    const double d10 = J_02*J_21 - J_01*J_22;
    const double d11 = J_00*J_22 - J_02*J_20;
    const double d12 = J_01*J_20 - J_00*J_21;
    
    const double d20 = J_01*J_12 - J_02*J_11;
    const double d21 = J_02*J_10 - J_00*J_12;
    const double d22 = J_00*J_11 - J_01*J_10;
      
    // Compute determinant of Jacobian
    double detJ = J_00*d00 + J_10*d10 + J_20*d20;
    
    // Compute inverse of Jacobian
    
    // Compute constants
    const double C0 = d00*(element_coordinates[0][0] - element_coordinates[2][0] - element_coordinates[3][0]) \
                    + d10*(element_coordinates[0][1] - element_coordinates[2][1] - element_coordinates[3][1]) \
                    + d20*(element_coordinates[0][2] - element_coordinates[2][2] - element_coordinates[3][2]);
    
    const double C1 = d01*(element_coordinates[0][0] - element_coordinates[1][0] - element_coordinates[3][0]) \
                    + d11*(element_coordinates[0][1] - element_coordinates[1][1] - element_coordinates[3][1]) \
                    + d21*(element_coordinates[0][2] - element_coordinates[1][2] - element_coordinates[3][2]);
    
    const double C2 = d02*(element_coordinates[0][0] - element_coordinates[1][0] - element_coordinates[2][0]) \
                    + d12*(element_coordinates[0][1] - element_coordinates[1][1] - element_coordinates[2][1]) \
                    + d22*(element_coordinates[0][2] - element_coordinates[1][2] - element_coordinates[2][2]);
    
    // Get coordinates and map to the UFC reference element
    double x = (C0 + d00*coordinates[0] + d10*coordinates[1] + d20*coordinates[2]) / detJ;
    double y = (C1 + d01*coordinates[0] + d11*coordinates[1] + d21*coordinates[2]) / detJ;
    double z = (C2 + d02*coordinates[0] + d12*coordinates[1] + d22*coordinates[2]) / detJ;
    
    // Map coordinates to the reference cube
    if (std::abs(y + z - 1.0) < 1e-14)
      x = 1.0;
    else
      x = -2.0 * x/(y + z - 1.0) - 1.0;
    if (std::abs(z - 1.0) < 1e-14)
      y = -1.0;
    else
      y = 2.0 * y/(1.0 - z) - 1.0;
    z = 2.0 * z - 1.0;
    
    // Compute number of derivatives
    unsigned int num_derivatives = 1;
    
    for (unsigned int j = 0; j < n; j++)
      num_derivatives *= 3;
    
    
    // Declare pointer to two dimensional array that holds combinations of derivatives and initialise
    unsigned int **combinations = new unsigned int *[num_derivatives];
        
    for (unsigned int j = 0; j < num_derivatives; j++)
    {
      combinations[j] = new unsigned int [n];
      for (unsigned int k = 0; k < n; k++)
        combinations[j][k] = 0;
    }
        
    // Generate combinations of derivatives
    for (unsigned int row = 1; row < num_derivatives; row++)
    {
      for (unsigned int num = 0; num < row; num++)
      {
        for (unsigned int col = n-1; col+1 > 0; col--)
        {
          if (combinations[row][col] + 1 > 2)
            combinations[row][col] = 0;
          else
          {
            combinations[row][col] += 1;
            break;
          }
        }
      }
    }
    
    // Compute inverse of Jacobian
    const double Jinv[3][3] ={{d00 / detJ, d10 / detJ, d20 / detJ}, {d01 / detJ, d11 / detJ, d21 / detJ}, {d02 / detJ, d12 / detJ, d22 / detJ}};
    
    // Declare transformation matrix
    // Declare pointer to two dimensional array and initialise
    double **transform = new double *[num_derivatives];
        
    for (unsigned int j = 0; j < num_derivatives; j++)
    {
      transform[j] = new double [num_derivatives];
      for (unsigned int k = 0; k < num_derivatives; k++)
        transform[j][k] = 1;
    }
    
    // Construct transformation matrix
    for (unsigned int row = 0; row < num_derivatives; row++)
    {
      for (unsigned int col = 0; col < num_derivatives; col++)
      {
        for (unsigned int k = 0; k < n; k++)
          transform[row][col] *= Jinv[combinations[col][k]][combinations[row][k]];
      }
    }
    
    // Reset values
    for (unsigned int j = 0; j < 1*num_derivatives; j++)
      values[j] = 0;
    
    // Map degree of freedom to element degree of freedom
    const unsigned int dof = i;
    
    // Generate scalings
    const double scalings_y_0 = 1;
    const double scalings_y_1 = scalings_y_0*(0.5 - 0.5*y);
    const double scalings_y_2 = scalings_y_1*(0.5 - 0.5*y);
    const double scalings_y_3 = scalings_y_2*(0.5 - 0.5*y);
    const double scalings_y_4 = scalings_y_3*(0.5 - 0.5*y);
    const double scalings_y_5 = scalings_y_4*(0.5 - 0.5*y);
    const double scalings_z_0 = 1;
    const double scalings_z_1 = scalings_z_0*(0.5 - 0.5*z);
    const double scalings_z_2 = scalings_z_1*(0.5 - 0.5*z);
    const double scalings_z_3 = scalings_z_2*(0.5 - 0.5*z);
    const double scalings_z_4 = scalings_z_3*(0.5 - 0.5*z);
    const double scalings_z_5 = scalings_z_4*(0.5 - 0.5*z);
    
    // Compute psitilde_a
    const double psitilde_a_0 = 1;
    const double psitilde_a_1 = x;
    const double psitilde_a_2 = 1.5*x*psitilde_a_1 - 0.5*psitilde_a_0;
    const double psitilde_a_3 = 1.66666666666667*x*psitilde_a_2 - 0.666666666666667*psitilde_a_1;
    const double psitilde_a_4 = 1.75*x*psitilde_a_3 - 0.75*psitilde_a_2;
    const double psitilde_a_5 = 1.8*x*psitilde_a_4 - 0.8*psitilde_a_3;
    
    // Compute psitilde_bs
    const double psitilde_bs_0_0 = 1;
    const double psitilde_bs_0_1 = 1.5*y + 0.5;
    const double psitilde_bs_0_2 = 0.111111111111111*psitilde_bs_0_1 + 1.66666666666667*y*psitilde_bs_0_1 - 0.555555555555556*psitilde_bs_0_0;
    const double psitilde_bs_0_3 = 0.05*psitilde_bs_0_2 + 1.75*y*psitilde_bs_0_2 - 0.7*psitilde_bs_0_1;
    const double psitilde_bs_0_4 = 0.0285714285714286*psitilde_bs_0_3 + 1.8*y*psitilde_bs_0_3 - 0.771428571428571*psitilde_bs_0_2;
    const double psitilde_bs_0_5 = 0.0185185185185185*psitilde_bs_0_4 + 1.83333333333333*y*psitilde_bs_0_4 - 0.814814814814815*psitilde_bs_0_3;
    const double psitilde_bs_1_0 = 1;
    const double psitilde_bs_1_1 = 2.5*y + 1.5;
    const double psitilde_bs_1_2 = 0.54*psitilde_bs_1_1 + 2.1*y*psitilde_bs_1_1 - 0.56*psitilde_bs_1_0;
    const double psitilde_bs_1_3 = 0.285714285714286*psitilde_bs_1_2 + 2*y*psitilde_bs_1_2 - 0.714285714285714*psitilde_bs_1_1;
    const double psitilde_bs_1_4 = 0.178571428571429*psitilde_bs_1_3 + 1.96428571428571*y*psitilde_bs_1_3 - 0.785714285714286*psitilde_bs_1_2;
    const double psitilde_bs_2_0 = 1;
    const double psitilde_bs_2_1 = 3.5*y + 2.5;
    const double psitilde_bs_2_2 = 1.02040816326531*psitilde_bs_2_1 + 2.57142857142857*y*psitilde_bs_2_1 - 0.551020408163265*psitilde_bs_2_0;
    const double psitilde_bs_2_3 = 0.578703703703704*psitilde_bs_2_2 + 2.29166666666667*y*psitilde_bs_2_2 - 0.712962962962963*psitilde_bs_2_1;
    const double psitilde_bs_3_0 = 1;
    const double psitilde_bs_3_1 = 4.5*y + 3.5;
    const double psitilde_bs_3_2 = 1.51234567901235*psitilde_bs_3_1 + 3.05555555555556*y*psitilde_bs_3_1 - 0.54320987654321*psitilde_bs_3_0;
    const double psitilde_bs_4_0 = 1;
    const double psitilde_bs_4_1 = 5.5*y + 4.5;
    const double psitilde_bs_5_0 = 1;
    
    // Compute psitilde_cs
    const double psitilde_cs_00_0 = 1;
    const double psitilde_cs_00_1 = 2*z + 1;
    const double psitilde_cs_00_2 = 0.3125*psitilde_cs_00_1 + 1.875*z*psitilde_cs_00_1 - 0.5625*psitilde_cs_00_0;
    const double psitilde_cs_00_3 = 0.155555555555556*psitilde_cs_00_2 + 1.86666666666667*z*psitilde_cs_00_2 - 0.711111111111111*psitilde_cs_00_1;
    const double psitilde_cs_00_4 = 0.09375*psitilde_cs_00_3 + 1.875*z*psitilde_cs_00_3 - 0.78125*psitilde_cs_00_2;
    const double psitilde_cs_00_5 = 0.0628571428571429*psitilde_cs_00_4 + 1.88571428571429*z*psitilde_cs_00_4 - 0.822857142857143*psitilde_cs_00_3;
    const double psitilde_cs_01_0 = 1;
    const double psitilde_cs_01_1 = 3*z + 2;
    const double psitilde_cs_01_2 = 0.777777777777778*psitilde_cs_01_1 + 2.33333333333333*z*psitilde_cs_01_1 - 0.555555555555556*psitilde_cs_01_0;
    const double psitilde_cs_01_3 = 0.428571428571429*psitilde_cs_01_2 + 2.14285714285714*z*psitilde_cs_01_2 - 0.714285714285714*psitilde_cs_01_1;
    const double psitilde_cs_01_4 = 0.275*psitilde_cs_01_3 + 2.0625*z*psitilde_cs_01_3 - 0.7875*psitilde_cs_01_2;
    const double psitilde_cs_02_0 = 1;
    const double psitilde_cs_02_1 = 4*z + 3;
    const double psitilde_cs_02_2 = 1.265625*psitilde_cs_02_1 + 2.8125*z*psitilde_cs_02_1 - 0.546875*psitilde_cs_02_0;
    const double psitilde_cs_02_3 = 0.733333333333333*psitilde_cs_02_2 + 2.44444444444444*z*psitilde_cs_02_2 - 0.711111111111111*psitilde_cs_02_1;
    const double psitilde_cs_03_0 = 1;
    const double psitilde_cs_03_1 = 5*z + 4;
    const double psitilde_cs_03_2 = 1.76*psitilde_cs_03_1 + 3.3*z*psitilde_cs_03_1 - 0.54*psitilde_cs_03_0;
    const double psitilde_cs_04_0 = 1;
    const double psitilde_cs_04_1 = 6*z + 5;
    const double psitilde_cs_05_0 = 1;
    const double psitilde_cs_10_0 = 1;
    const double psitilde_cs_10_1 = 3*z + 2;
    const double psitilde_cs_10_2 = 0.777777777777778*psitilde_cs_10_1 + 2.33333333333333*z*psitilde_cs_10_1 - 0.555555555555556*psitilde_cs_10_0;
    const double psitilde_cs_10_3 = 0.428571428571429*psitilde_cs_10_2 + 2.14285714285714*z*psitilde_cs_10_2 - 0.714285714285714*psitilde_cs_10_1;
    const double psitilde_cs_10_4 = 0.275*psitilde_cs_10_3 + 2.0625*z*psitilde_cs_10_3 - 0.7875*psitilde_cs_10_2;
    const double psitilde_cs_11_0 = 1;
    const double psitilde_cs_11_1 = 4*z + 3;
    const double psitilde_cs_11_2 = 1.265625*psitilde_cs_11_1 + 2.8125*z*psitilde_cs_11_1 - 0.546875*psitilde_cs_11_0;
    const double psitilde_cs_11_3 = 0.733333333333333*psitilde_cs_11_2 + 2.44444444444444*z*psitilde_cs_11_2 - 0.711111111111111*psitilde_cs_11_1;
    const double psitilde_cs_12_0 = 1;
    const double psitilde_cs_12_1 = 5*z + 4;
    const double psitilde_cs_12_2 = 1.76*psitilde_cs_12_1 + 3.3*z*psitilde_cs_12_1 - 0.54*psitilde_cs_12_0;
    const double psitilde_cs_13_0 = 1;
    const double psitilde_cs_13_1 = 6*z + 5;
    const double psitilde_cs_14_0 = 1;
    const double psitilde_cs_20_0 = 1;
    const double psitilde_cs_20_1 = 4*z + 3;
    const double psitilde_cs_20_2 = 1.265625*psitilde_cs_20_1 + 2.8125*z*psitilde_cs_20_1 - 0.546875*psitilde_cs_20_0;
    const double psitilde_cs_20_3 = 0.733333333333333*psitilde_cs_20_2 + 2.44444444444444*z*psitilde_cs_20_2 - 0.711111111111111*psitilde_cs_20_1;
    const double psitilde_cs_21_0 = 1;
    const double psitilde_cs_21_1 = 5*z + 4;
    const double psitilde_cs_21_2 = 1.76*psitilde_cs_21_1 + 3.3*z*psitilde_cs_21_1 - 0.54*psitilde_cs_21_0;
    const double psitilde_cs_22_0 = 1;
    const double psitilde_cs_22_1 = 6*z + 5;
    const double psitilde_cs_23_0 = 1;
    const double psitilde_cs_30_0 = 1;
    const double psitilde_cs_30_1 = 5*z + 4;
    const double psitilde_cs_30_2 = 1.76*psitilde_cs_30_1 + 3.3*z*psitilde_cs_30_1 - 0.54*psitilde_cs_30_0;
    const double psitilde_cs_31_0 = 1;
    const double psitilde_cs_31_1 = 6*z + 5;
    const double psitilde_cs_32_0 = 1;
    const double psitilde_cs_40_0 = 1;
    const double psitilde_cs_40_1 = 6*z + 5;
    const double psitilde_cs_41_0 = 1;
    const double psitilde_cs_50_0 = 1;
    
    // Compute basisvalues
    const double basisvalue0 = 0.866025403784439*psitilde_a_0*scalings_y_0*psitilde_bs_0_0*scalings_z_0*psitilde_cs_00_0;
    const double basisvalue1 = 2.73861278752583*psitilde_a_1*scalings_y_1*psitilde_bs_1_0*scalings_z_1*psitilde_cs_10_0;
    const double basisvalue2 = 1.58113883008419*psitilde_a_0*scalings_y_0*psitilde_bs_0_1*scalings_z_1*psitilde_cs_01_0;
    const double basisvalue3 = 1.11803398874989*psitilde_a_0*scalings_y_0*psitilde_bs_0_0*scalings_z_0*psitilde_cs_00_1;
    const double basisvalue4 = 5.1234753829798*psitilde_a_2*scalings_y_2*psitilde_bs_2_0*scalings_z_2*psitilde_cs_20_0;
    const double basisvalue5 = 3.96862696659689*psitilde_a_1*scalings_y_1*psitilde_bs_1_1*scalings_z_2*psitilde_cs_11_0;
    const double basisvalue6 = 2.29128784747792*psitilde_a_0*scalings_y_0*psitilde_bs_0_2*scalings_z_2*psitilde_cs_02_0;
    const double basisvalue7 = 3.24037034920393*psitilde_a_1*scalings_y_1*psitilde_bs_1_0*scalings_z_1*psitilde_cs_10_1;
    const double basisvalue8 = 1.87082869338697*psitilde_a_0*scalings_y_0*psitilde_bs_0_1*scalings_z_1*psitilde_cs_01_1;
    const double basisvalue9 = 1.3228756555323*psitilde_a_0*scalings_y_0*psitilde_bs_0_0*scalings_z_0*psitilde_cs_00_2;
    const double basisvalue10 = 7.93725393319377*psitilde_a_3*scalings_y_3*psitilde_bs_3_0*scalings_z_3*psitilde_cs_30_0;
    const double basisvalue11 = 6.70820393249937*psitilde_a_2*scalings_y_2*psitilde_bs_2_1*scalings_z_3*psitilde_cs_21_0;
    const double basisvalue12 = 5.19615242270663*psitilde_a_1*scalings_y_1*psitilde_bs_1_2*scalings_z_3*psitilde_cs_12_0;
    const double basisvalue13 = 3*psitilde_a_0*scalings_y_0*psitilde_bs_0_3*scalings_z_3*psitilde_cs_03_0;
    const double basisvalue14 = 5.80947501931113*psitilde_a_2*scalings_y_2*psitilde_bs_2_0*scalings_z_2*psitilde_cs_20_1;
    const double basisvalue15 = 4.5*psitilde_a_1*scalings_y_1*psitilde_bs_1_1*scalings_z_2*psitilde_cs_11_1;
    const double basisvalue16 = 2.59807621135332*psitilde_a_0*scalings_y_0*psitilde_bs_0_2*scalings_z_2*psitilde_cs_02_1;
    const double basisvalue17 = 3.67423461417477*psitilde_a_1*scalings_y_1*psitilde_bs_1_0*scalings_z_1*psitilde_cs_10_2;
    const double basisvalue18 = 2.12132034355964*psitilde_a_0*scalings_y_0*psitilde_bs_0_1*scalings_z_1*psitilde_cs_01_2;
    const double basisvalue19 = 1.5*psitilde_a_0*scalings_y_0*psitilde_bs_0_0*scalings_z_0*psitilde_cs_00_3;
    const double basisvalue20 = 11.1242977306435*psitilde_a_4*scalings_y_4*psitilde_bs_4_0*scalings_z_4*psitilde_cs_40_0;
    const double basisvalue21 = 9.81070843517429*psitilde_a_3*scalings_y_3*psitilde_bs_3_1*scalings_z_4*psitilde_cs_31_0;
    const double basisvalue22 = 8.2915619758885*psitilde_a_2*scalings_y_2*psitilde_bs_2_2*scalings_z_4*psitilde_cs_22_0;
    const double basisvalue23 = 6.42261628933256*psitilde_a_1*scalings_y_1*psitilde_bs_1_3*scalings_z_4*psitilde_cs_13_0;
    const double basisvalue24 = 3.70809924354783*psitilde_a_0*scalings_y_0*psitilde_bs_0_4*scalings_z_4*psitilde_cs_04_0;
    const double basisvalue25 = 8.77496438739212*psitilde_a_3*scalings_y_3*psitilde_bs_3_0*scalings_z_3*psitilde_cs_30_1;
    const double basisvalue26 = 7.41619848709566*psitilde_a_2*scalings_y_2*psitilde_bs_2_1*scalings_z_3*psitilde_cs_21_1;
    const double basisvalue27 = 5.74456264653803*psitilde_a_1*scalings_y_1*psitilde_bs_1_2*scalings_z_3*psitilde_cs_12_1;
    const double basisvalue28 = 3.3166247903554*psitilde_a_0*scalings_y_0*psitilde_bs_0_3*scalings_z_3*psitilde_cs_03_1;
    const double basisvalue29 = 6.42261628933256*psitilde_a_2*scalings_y_2*psitilde_bs_2_0*scalings_z_2*psitilde_cs_20_2;
    const double basisvalue30 = 4.9749371855331*psitilde_a_1*scalings_y_1*psitilde_bs_1_1*scalings_z_2*psitilde_cs_11_2;
    const double basisvalue31 = 2.87228132326901*psitilde_a_0*scalings_y_0*psitilde_bs_0_2*scalings_z_2*psitilde_cs_02_2;
    const double basisvalue32 = 4.06201920231798*psitilde_a_1*scalings_y_1*psitilde_bs_1_0*scalings_z_1*psitilde_cs_10_3;
    const double basisvalue33 = 2.34520787991171*psitilde_a_0*scalings_y_0*psitilde_bs_0_1*scalings_z_1*psitilde_cs_01_3;
    const double basisvalue34 = 1.6583123951777*psitilde_a_0*scalings_y_0*psitilde_bs_0_0*scalings_z_0*psitilde_cs_00_4;
    const double basisvalue35 = 14.6458185158768*psitilde_a_5*scalings_y_5*psitilde_bs_5_0*scalings_z_5*psitilde_cs_50_0;
    const double basisvalue36 = 13.2476412994918*psitilde_a_4*scalings_y_4*psitilde_bs_4_1*scalings_z_5*psitilde_cs_41_0;
    const double basisvalue37 = 11.6833214455479*psitilde_a_3*scalings_y_3*psitilde_bs_3_2*scalings_z_5*psitilde_cs_32_0;
    const double basisvalue38 = 9.87420882906575*psitilde_a_2*scalings_y_2*psitilde_bs_2_3*scalings_z_5*psitilde_cs_23_0;
    const double basisvalue39 = 7.64852927038918*psitilde_a_1*scalings_y_1*psitilde_bs_1_4*scalings_z_5*psitilde_cs_14_0;
    const double basisvalue40 = 4.41588043316392*psitilde_a_0*scalings_y_0*psitilde_bs_0_5*scalings_z_5*psitilde_cs_05_0;
    const double basisvalue41 = 12.0933866224478*psitilde_a_4*scalings_y_4*psitilde_bs_4_0*scalings_z_4*psitilde_cs_40_1;
    const double basisvalue42 = 10.6653645038508*psitilde_a_3*scalings_y_3*psitilde_bs_3_1*scalings_z_4*psitilde_cs_31_1;
    const double basisvalue43 = 9.01387818865997*psitilde_a_2*scalings_y_2*psitilde_bs_2_2*scalings_z_4*psitilde_cs_22_1;
    const double basisvalue44 = 6.98212002188447*psitilde_a_1*scalings_y_1*psitilde_bs_1_3*scalings_z_4*psitilde_cs_13_1;
    const double basisvalue45 = 4.03112887414927*psitilde_a_0*scalings_y_0*psitilde_bs_0_4*scalings_z_4*psitilde_cs_04_1;
    const double basisvalue46 = 9.53939201416946*psitilde_a_3*scalings_y_3*psitilde_bs_3_0*scalings_z_3*psitilde_cs_30_2;
    const double basisvalue47 = 8.06225774829855*psitilde_a_2*scalings_y_2*psitilde_bs_2_1*scalings_z_3*psitilde_cs_21_2;
    const double basisvalue48 = 6.2449979983984*psitilde_a_1*scalings_y_1*psitilde_bs_1_2*scalings_z_3*psitilde_cs_12_2;
    const double basisvalue49 = 3.60555127546399*psitilde_a_0*scalings_y_0*psitilde_bs_0_3*scalings_z_3*psitilde_cs_03_2;
    const double basisvalue50 = 6.98212002188447*psitilde_a_2*scalings_y_2*psitilde_bs_2_0*scalings_z_2*psitilde_cs_20_3;
    const double basisvalue51 = 5.40832691319598*psitilde_a_1*scalings_y_1*psitilde_bs_1_1*scalings_z_2*psitilde_cs_11_3;
    const double basisvalue52 = 3.1224989991992*psitilde_a_0*scalings_y_0*psitilde_bs_0_2*scalings_z_2*psitilde_cs_02_3;
    const double basisvalue53 = 4.41588043316392*psitilde_a_1*scalings_y_1*psitilde_bs_1_0*scalings_z_1*psitilde_cs_10_4;
    const double basisvalue54 = 2.54950975679639*psitilde_a_0*scalings_y_0*psitilde_bs_0_1*scalings_z_1*psitilde_cs_01_4;
    const double basisvalue55 = 1.80277563773199*psitilde_a_0*scalings_y_0*psitilde_bs_0_0*scalings_z_0*psitilde_cs_00_5;
    
    // Table(s) of coefficients
    const static double coefficients0[56][56] = \
    {{0.00945067404923504, 0.00609788120679252, 0.003520613356228, 0.00248944957812473, 0.00705946233325038, 0.00546823600997147, 0.00315708753234945, 0.00446479600584757, 0.00257775117585287, 0.00182274533665719, -0.00298267418725718, -0.0025208197798293, -0.00195261860521383, -0.0011273448773449, -0.00218309396769446, -0.00169101731601731, -0.000976309302606924, -0.00138070985681772, -0.000797153207506492, -0.000563672438672437, 0.00278687112460006, 0.00245778931055954, 0.00207721109304565, 0.00160900079398463, 0.000928957041533348, 0.00219831358911339, 0.00185791408306671, 0.00143913406048031, 0.000830884437218262, 0.00160900079398463, 0.00124632665582739, 0.000719567030240153, 0.00101762145320216, 0.000587524019939396, 0.000415442218609115, -0.00705593524717797, -0.0063823335708908, -0.00562868913761196, -0.00475711057319694, -0.00368484200521179, -0.00212744452363026, -0.00582624677716564, -0.00513826668309063, -0.00434262794914383, -0.00336378514516178, -0.00194208225905521, -0.00459580543596519, -0.00388416451811043, -0.00300866089851429, -0.00173705117965752, -0.00336378514516179, -0.00260557676948628, -0.00150433044925714, -0.00212744452363026, -0.00122828066840392, -0.000868525589828755},
    {0.00945067404923484, -0.0060978812067926, 0.00352061335622807, 0.00248944957812473, 0.00705946233325039, -0.00546823600997146, 0.00315708753234942, -0.00446479600584756, 0.00257775117585286, 0.00182274533665715, 0.00298267418725716, -0.00252081977982931, 0.00195261860521383, -0.00112734487734489, -0.00218309396769446, 0.00169101731601731, -0.000976309302606908, 0.00138070985681772, -0.000797153207506487, -0.000563672438672423, 0.00278687112460003, -0.00245778931055954, 0.00207721109304565, -0.00160900079398463, 0.000928957041533354, -0.00219831358911339, 0.00185791408306669, -0.00143913406048031, 0.000830884437218255, 0.00160900079398463, -0.0012463266558274, 0.000719567030240159, -0.00101762145320216, 0.000587524019939409, 0.000415442218609113, 0.00705593524717798, -0.00638233357089081, 0.00562868913761196, -0.00475711057319694, 0.00368484200521178, -0.00212744452363027, -0.00582624677716566, 0.00513826668309062, -0.00434262794914383, 0.00336378514516178, -0.00194208225905522, 0.0045958054359652, -0.00388416451811043, 0.00300866089851429, -0.00173705117965753, -0.00336378514516178, 0.00260557676948629, -0.00150433044925715, 0.00212744452363026, -0.00122828066840393, -0.000868525589828756},
    {0.00945067404923496, 0, -0.00704122671245609, 0.00248944957812476, 0, 0, 0.00947126259704828, 0, -0.00515550235170569, 0.00182274533665715, 0, 0, 0, 0.0045093795093795, 0, 0, -0.00292892790782073, 0, 0.00159430641501296, -0.000563672438672431, 0, 0, 0, 0, 0.00464478520766673, 0, 0, 0, -0.00332353774887306, 0, 0, 0.00215870109072046, 0, -0.00117504803987882, 0.000415442218609132, 0, 0, 0, 0, 0, 0.0127646671417816, 0, 0, 0, 0, -0.00971041129527609, 0, 0, 0, 0.0069482047186301, 0, 0, -0.00451299134777144, 0, 0.00245656133680787, -0.000868525589828766},
    {0.00945067404923495, 0, 0, -0.00746834873437432, 0, 0, 0, 0, 0, 0.0109364720199429, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0.00563672438672439, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0.00623163327913698, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0.0182390373864041},
    {-0.0100234421734309, 0, 0.0374718748311398, -0.0289614183593723, 0, 0, 0.0284137877911449, 0, 0.00515550235170565, -0.00619733414463432, 0, 0, 0, 0.0315656565656566, 0, 0, -0.0113902751970807, 0, 0.00265717735835495, -0.000187890812890785, 0, 0, 0, 0, 0.0232239260383336, 0, 0, 0, -0.00166176887443649, 0, 0, -0.00647610327216139, 0, 0.00646276421933346, -0.00290809553026393, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0.0213629048496073, 0, 0, 0, -0.0277928188745204, 0, 0, 0.0243701532779658, 0, -0.0157219925555703, 0.00607967912880134},
    {0.0100234421734308, 0, -0.0287574853355261, 0.05360943398437, 0, 0, -0.0189425251940965, 0, 0.0154665070551172, 0.0153110608279201, 0, 0, 0, -0.00450937950937953, 0, 0, 0.0370997534990628, 0, -0.0281660799985623, 0.0108976671476671, 0, 0, 0, 0, 0, 0, 0, 0, 0.0299118397398575, 0, 0, -0.00431740218144094, 0, -0.0123380044187275, 0.00872428659079179, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0.0312669212338355, 0, 0, -0.0487403065559315, 0, 0.0412702304583721, -0.018239037386404},
    {0.0100234421734309, 0, 0.0409576306293854, -0.0449826285156208, 0, 0, 0.0189425251940966, 0, -0.00515550235170581, -0.021143845905223, 0, 0, 0, 0, 0, 0, 0.00455611007883228, 0, 0.0356061766019561, -0.0319414381914382, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0.0302218152700864, 0, 0.00411266813957589, -0.0145404776513196, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0.0379091273212801, 0, -0.0550269739444961, 0.0303983956440067},
    {-0.0100234421734309, 0, -0.0148144621425437, 0.0449826285156208, 0, 0, 0, 0, 0.0206220094068229, 0.021143845905223, 0, 0, 0, 0, 0, 0, 0, 0, 0.0106287094334197, 0.0319414381914382, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0.0205633406978791, 0.0145404776513196, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0.03439185871531, -0.0303983956440067},
    {-0.0100234421734309, 0.0324515955311978, -0.01873593741557, -0.0289614183593723, 0.0211783869997511, -0.0164047080299144, 0.00947126259704827, 0.00446479600584751, -0.00257775117585286, -0.00619733414463429, 0.0208787193108001, -0.0176457384588051, 0.0136683302364968, -0.00789141414141412, -0.00848980987436741, 0.00657617845117844, -0.00379675839902691, 0.0023011830946962, -0.00132858867917746, -0.000187890812890819, 0.0139343556230002, -0.0122889465527977, 0.0103860554652283, -0.00804500396992315, 0.00464478520766672, -0.00109915679455667, 0.000928957041533349, -0.000719567030240153, 0.000415442218609154, -0.00482700238195389, 0.00373897996748218, -0.00215870109072046, 0.00559691799261187, -0.00323138210966673, -0.00290809553026394, 0, 0, 0, 0, 0, 0, 0.0128177429097644, -0.0113041867027994, 0.00955378148811641, -0.00740032731935592, 0.00427258096992148, -0.0183832217438608, 0.0155366580724417, -0.0120346435940572, 0.00694820471863012, 0.0181644397838736, -0.014070114555226, 0.00812338442598859, -0.0136156449512337, 0.00786099627778517, 0.00607967912880136},
    {0.0100234421734311, -0.024904712849524, 0.014378742667763, 0.0536094339843701, -0.0141189246665008, 0.0109364720199429, -0.00631417506469887, 0.0133943880175428, -0.00773325352755854, 0.01531106082792, -0.00298267418725714, 0.0025208197798293, -0.00195261860521383, 0.00112734487734485, 0.0276525235907967, -0.0214195526695527, 0.0123665844996876, -0.0243925408037797, 0.0140830399992812, 0.0108976671476671, 0, 0, 0, 0, 0, 0.0197848223020205, -0.0167212267476003, 0.0129522065443228, -0.0074779599349644, -0.00321800158796925, 0.00249265331165481, -0.00143913406048031, -0.0106850252586226, 0.00616900220936375, 0.00872428659079183, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0.0206811244618434, -0.017478740331497, 0.0135389740433143, -0.00781673030845889, -0.0363288795677472, 0.028140229110452, -0.0162467688519772, 0.0357410679969885, -0.0206351152291861, -0.0182390373864041},
    {0.0100234421734311, 0.0354703486038675, -0.0204788153146928, -0.0449826285156207, 0.0141189246665009, -0.0109364720199429, 0.00631417506469889, -0.0044647960058475, 0.00257775117585279, -0.021143845905223, 0, 0, 0, 0, 0.00339592394974698, -0.00263047138047136, 0.00151870335961076, 0.0308358534689291, -0.0178030883009781, -0.0319414381914382, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0.0225260111157848, -0.0174485731815835, 0.0100739384233621, 0.00356167508620756, -0.00205633406978792, -0.0145404776513196, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0.028255795219359, -0.0218868448636849, 0.01263637577376, -0.047654757329318, 0.0275134869722481, 0.0303983956440068},
    {-0.0100234421734311, -0.0128297005588457, 0.00740723107127181, 0.0449826285156207, 0, 0, 0, 0.0178591840233903, -0.0103110047034114, 0.0211438459052229, 0, 0, 0, 0, 0, 0, 0, 0.0092047323787848, -0.00531435471670985, 0.0319414381914382, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0.0178083754310377, -0.0102816703489396, 0.0145404776513196, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0.0297842233308237, -0.017195929357655, -0.0303983956440068},
    {-0.0100234421734311, 0.0324515955311977, -0.0335503995581136, -0.00801060507812428, 0.0211783869997512, 0.00109364720199428, -0.00568275755822898, -0.0169662248222208, 0.00567105258687634, 0.0083846285486229, 0.0208787193108001, -0.0109235523792603, 0.00499002532443531, -0.00187890812890811, -0.0162519217595033, 0.00958243145743145, -0.00466458889023307, 0.0108922666482287, -0.00558007245254534, -0.00469727032227035, 0.0139343556230002, -0.00245778931055952, -0.00415442218609131, 0.0064360031759385, -0.00464478520766676, -0.0120907247401236, 0.00464478520766674, -0.000719567030240154, -0.000415442218609135, 0.00965400476390777, -0.00498530662330959, 0.00215870109072046, -0.00661453944581401, 0.0032313821096667, 0.00290809553026394, 0, 0.0127646671417816, -0.0202632808954031, 0.0228341307513453, -0.020635115229186, 0.0127646671417816, -0.00116524935543312, -0.00719357335632687, 0.0112908326677739, -0.0114368694935501, 0.00737991258440983, 0.00183832217438608, 0.00310733161448834, -0.00481385743762287, 0.00347410235931505, -0.00201827108709706, -0.000521115353897253, 0.000902598269554298, 0.00170195561890421, -0.000491312267361565, -0.000868525589828764},
    {0.0100234421734313, -0.0249047128495239, 0.0553363732971484, -0.00431340273437463, -0.0141189246665008, 0.0153110608279201, 0.00883984509057839, 0.00803663281052565, -0.0159820572902878, 0.000729098134662871, -0.00298267418725714, 0.0277290175781224, -0.0301571095694136, 0.0191648629148629, -0.00145539597846295, -0.0116492303992304, 0.00976309302606917, 0.00352848074520082, 0.00132858867917745, -0.00263047138047135, 0, 0.0196623144844763, -0.00727023882565982, -0.00724050357293083, 0.00928957041533347, -0.00219831358911338, -0.0148633126645336, 0.0129522065443228, -0.00664707549774609, 0.00402250198496158, 0.00685479660705067, -0.00575653624192123, -0.00457929653940971, -0.000293762009969677, 0.0029080955302639, 0, 0, 0.0202632808954031, -0.0399597288148543, 0.0464290092656686, -0.031911667854454, 0, -0.00411061334647249, -0.00303983956440069, 0.00908221989193683, -0.00776832903622088, 0.000459580543596524, 0.00427258096992149, -0.00270779480866288, 0.000868525589828788, -0.00100913554354855, -0.00234501909253767, 0.00180519653910856, 0.00127646671417817, 0.000245656133680783, -0.000868525589828766},
    {0.0100234421734309, 0.0354703486038674, -0.0492363006502187, -0.00431340273437457, 0.0141189246665008, -0.00656188321196574, -0.016416855168217, -0.00982255121286468, 0.0149509568199466, 0.00072909813466289, 0, 0.00336109303977242, 0.0282044909641998, -0.0353234728234728, -0.000485131992821028, -0.0124007936507936, 0.00889526253486298, 0.00291483191994853, 0.00239145962251946, -0.0026304713804714, 0, 0, 0.0218107164769794, -0.00402250198496158, -0.00928957041533346, 0, -0.00557374224920008, -0.0143913406048031, 0.0141250354327105, 0.000804500396992295, 0.00934744991870546, -0.00287826812096058, -0.00254405363300538, -0.00381890612960614, 0.00290809553026392, 0, 0, 0, 0.0266398192099029, -0.0515877880729651, 0.0425488904726054, 0, 0, -0.00911951869320201, 0.0084094628629044, -0.0038841645181104, 0, 0.00233049871086625, 0.00300866089851431, -0.00347410235931508, -0.000336378514516169, -0.00286613444643491, 0.00120346435940571, 0.000850977809452102, 0.000982624534723146, -0.000868525589828757},
    {-0.0100234421734311, -0.0128297005588456, 0.0448791059024117, -0.00801060507812423, 0, 0.0174983552319087, 0.0132597676358676, -0.00357183680467805, -0.0175287079957994, 0.00838462854862291, 0, 0, 0.00867830491206144, 0.0225468975468975, 0, -0.00300625300625305, -0.019200749617936, 0.000613648825252313, 0.0122230158484327, -0.00469727032227032, 0, 0, 0, 0.0160900079398463, 0.00464478520766673, 0, 0, -0.00719567030240156, -0.0108014976838374, 0, 0.00249265331165479, 0.0107935054536023, -0.000508810726601082, -0.00734405024924251, 0.00290809553026393, 0, 0, 0, 0, 0.0257938940364825, -0.031911667854454, 0, 0, 0, -0.0134551405806471, 0.0135945758133865, 0, 0, 0.00601732179702858, -0.00347410235931504, 0, -0.00208446141558903, -0.000902598269554288, 0.000425488904726059, 0.0017195929357655, -0.000868525589828767},
    {-0.010023442173431, -0.0324515955311979, -0.0187359374155701, -0.0289614183593722, 0.0211783869997511, 0.0164047080299144, 0.00947126259704825, -0.0044647960058475, -0.00257775117585286, -0.00619733414463431, -0.0208787193108001, -0.0176457384588052, -0.0136683302364968, -0.00789141414141413, -0.00848980987436743, -0.00657617845117846, -0.00379675839902691, -0.00230118309469614, -0.00132858867917743, -0.000187890812890788, 0.0139343556230002, 0.0122889465527977, 0.0103860554652283, 0.00804500396992312, 0.00464478520766673, 0.00109915679455666, 0.00092895704153335, 0.000719567030240155, 0.000415442218609149, -0.0048270023819539, -0.00373897996748222, -0.00215870109072046, -0.00559691799261192, -0.00323138210966673, -0.00290809553026393, 0, 0, 0, 0, 0, 0, 0.0128177429097644, 0.0113041867027994, 0.00955378148811642, 0.00740032731935589, 0.00427258096992148, 0.0183832217438608, 0.0155366580724417, 0.0120346435940572, 0.00694820471863011, 0.0181644397838736, 0.014070114555226, 0.00812338442598858, 0.0136156449512337, 0.00786099627778515, 0.00607967912880134},
    {0.0100234421734311, 0.024904712849524, 0.014378742667763, 0.05360943398437, -0.0141189246665008, -0.0109364720199429, -0.00631417506469881, -0.0133943880175428, -0.00773325352755853, 0.01531106082792, 0.00298267418725717, 0.0025208197798293, 0.00195261860521382, 0.00112734487734485, 0.0276525235907967, 0.0214195526695527, 0.0123665844996876, 0.0243925408037797, 0.0140830399992811, 0.0108976671476671, 0, 0, 0, 0, 0, -0.0197848223020205, -0.0167212267476003, -0.0129522065443227, -0.0074779599349644, -0.00321800158796927, -0.00249265331165477, -0.00143913406048032, 0.0106850252586227, 0.00616900220936373, 0.00872428659079179, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, -0.0206811244618434, -0.017478740331497, -0.0135389740433143, -0.00781673030845888, -0.0363288795677472, -0.028140229110452, -0.0162467688519772, -0.0357410679969885, -0.020635115229186, -0.018239037386404},
    {0.0100234421734307, -0.0354703486038675, -0.0204788153146927, -0.0449826285156207, 0.0141189246665008, 0.0109364720199429, 0.00631417506469886, 0.00446479600584751, 0.00257775117585285, -0.0211438459052229, 0, 0, 0, 0, 0.00339592394974697, 0.00263047138047138, 0.00151870335961076, -0.0308358534689291, -0.017803088300978, -0.0319414381914382, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0.0225260111157848, 0.0174485731815835, 0.0100739384233621, -0.00356167508620757, -0.00205633406978789, -0.0145404776513196, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0.0282557952193589, 0.0218868448636849, 0.01263637577376, 0.047654757329318, 0.027513486972248, 0.0303983956440067},
    {-0.010023442173431, 0.0128297005588457, 0.00740723107127182, 0.0449826285156208, 0, 0, 0, -0.0178591840233902, -0.0103110047034114, 0.021143845905223, 0, 0, 0, 0, 0, 0, 0, -0.00920473237878481, -0.00531435471670985, 0.0319414381914381, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, -0.0178083754310377, -0.0102816703489396, 0.0145404776513196, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, -0.0297842233308237, -0.017195929357655, -0.0303983956440068},
    {-0.0100234421734306, -0.0324515955311977, -0.0335503995581136, -0.00801060507812422, 0.0211783869997512, -0.00109364720199428, -0.00568275755822895, 0.0169662248222208, 0.00567105258687631, 0.00838462854862288, -0.0208787193108001, -0.0109235523792604, -0.00499002532443533, -0.00187890812890813, -0.0162519217595033, -0.00958243145743147, -0.00466458889023305, -0.0108922666482287, -0.00558007245254534, -0.00469727032227036, 0.0139343556230002, 0.00245778931055952, -0.00415442218609133, -0.00643600317593853, -0.00464478520766674, 0.0120907247401236, 0.00464478520766673, 0.000719567030240151, -0.000415442218609145, 0.00965400476390776, 0.00498530662330958, 0.00215870109072046, 0.006614539445814, 0.00323138210966669, 0.00290809553026394, 0, 0.0127646671417816, 0.0202632808954031, 0.0228341307513453, 0.020635115229186, 0.0127646671417816, -0.00116524935543314, 0.00719357335632687, 0.0112908326677739, 0.01143686949355, 0.00737991258440983, -0.00183832217438609, 0.00310733161448835, 0.00481385743762286, 0.00347410235931504, -0.00201827108709706, 0.00052111535389726, 0.000902598269554298, -0.00170195561890421, -0.000491312267361563, -0.000868525589828763},
    {0.0100234421734315, 0.024904712849524, 0.0553363732971485, -0.00431340273437464, -0.0141189246665008, -0.0153110608279201, 0.00883984509057836, -0.00803663281052558, -0.0159820572902878, 0.000729098134662864, 0.00298267418725715, 0.0277290175781224, 0.0301571095694136, 0.0191648629148629, -0.00145539597846298, 0.0116492303992304, 0.00976309302606916, -0.00352848074520084, 0.00132858867917744, -0.00263047138047133, 0, -0.0196623144844763, -0.0072702388256598, 0.00724050357293084, 0.0092895704153335, 0.00219831358911338, -0.0148633126645336, -0.0129522065443228, -0.0066470754977461, 0.00402250198496159, -0.00685479660705068, -0.00575653624192122, 0.0045792965394097, -0.000293762009969664, 0.00290809553026391, 0, 0, -0.0202632808954031, -0.0399597288148543, -0.0464290092656686, -0.031911667854454, 0, 0.00411061334647249, -0.00303983956440066, -0.0090822198919368, -0.00776832903622086, -0.000459580543596519, 0.0042725809699215, 0.00270779480866287, 0.000868525589828796, -0.00100913554354854, 0.00234501909253766, 0.00180519653910856, -0.00127646671417816, 0.00024565613368078, -0.00086852558982877},
    {0.0100234421734312, -0.0354703486038675, -0.0492363006502186, -0.0043134027343746, 0.0141189246665008, 0.00656188321196572, -0.016416855168217, 0.0098225512128646, 0.0149509568199467, 0.000729098134662864, 0, 0.0033610930397724, -0.0282044909641998, -0.0353234728234728, -0.000485131992820996, 0.0124007936507936, 0.00889526253486298, -0.00291483191994851, 0.00239145962251946, -0.00263047138047141, 0, 0, 0.0218107164769794, 0.00402250198496156, -0.00928957041533348, 0, -0.0055737422492001, 0.0143913406048031, 0.0141250354327105, 0.00080450039699229, -0.00934744991870546, -0.00287826812096059, 0.00254405363300538, -0.00381890612960615, 0.00290809553026392, 0, 0, 0, 0.0266398192099029, 0.0515877880729651, 0.0425488904726054, 0, 0, -0.00911951869320204, -0.00840946286290444, -0.00388416451811041, 0, 0.00233049871086624, -0.0030086608985143, -0.00347410235931509, -0.000336378514516169, 0.00286613444643491, 0.00120346435940572, -0.000850977809452107, 0.000982624534723155, -0.000868525589828754},
    {-0.010023442173431, 0.0128297005588457, 0.0448791059024117, -0.00801060507812423, 0, -0.0174983552319087, 0.0132597676358676, 0.0035718368046781, -0.0175287079957995, 0.00838462854862291, 0, 0, -0.00867830491206147, 0.0225468975468975, 0, 0.00300625300625302, -0.019200749617936, -0.000613648825252327, 0.0122230158484327, -0.00469727032227032, 0, 0, 0, -0.0160900079398463, 0.00464478520766674, 0, 0, 0.00719567030240152, -0.0108014976838374, 0, -0.00249265331165479, 0.0107935054536023, 0.000508810726601078, -0.00734405024924251, 0.00290809553026392, 0, 0, 0, 0, -0.0257938940364826, -0.031911667854454, 0, 0, 0, 0.0134551405806471, 0.0135945758133865, 0, 0, -0.00601732179702859, -0.00347410235931504, 0, 0.00208446141558904, -0.000902598269554294, -0.000425488904726048, 0.0017195929357655, -0.000868525589828768},
    {-0.0100234421734308, -0.0452812960900435, -0.0113287063442981, -0.00801060507812425, -0.00141189246665006, 0.0164047080299144, 0.0145226026488073, 0.0133943880175427, 0.0118576554089232, 0.0083846285486229, -0.00894802256177148, -0.0142846454190327, -0.014536160727703, -0.00939454064454062, -0.0123708658169354, -0.0125886844636845, -0.00813591085505763, -0.0102786178229763, -0.00664294339588729, -0.00469727032227033, -0.00836061337380013, 0.00245778931055953, 0.00830884437218264, 0.00965400476390776, 0.00650269929073342, 0.00219831358911336, 0.00743165633226678, 0.00863480436288185, 0.00581619106052787, 0.00643600317593851, 0.00747795993496438, 0.00503696921168107, 0.00610572871921291, 0.00411266813957582, 0.00290809553026393, 0.0352796762358899, 0.0191470007126724, 0.00788016479265675, 0.000951422114639387, -0.00221090520312706, -0.00212744452363027, 0.017478740331497, 0.00719357335632688, 0.000868525589828766, -0.00201827108709706, -0.00194208225905523, 0.00643412761035128, 0.00077683290362208, -0.00180519653910858, -0.00173705117965754, 0.000672757029032357, -0.00156334606169177, -0.00150433044925715, -0.00127646671417815, -0.00122828066840394, -0.000868525589828763},
    {0.010023442173431, 0.0603750614533912, -0.00610007264692973, -0.0043134027343746, -0.00564756986660029, -0.0218729440398858, 0.00126283501293976, -0.0178591840233903, 0.00103110047034112, 0.00072909813466285, 0.0417574386216002, 0.0168054651988621, -0.000867830491206138, -0.00526094276094278, 0.0145539597846298, -0.000751563251563223, -0.00455611007883228, -0.000613648825252315, -0.00372004830169692, -0.00263047138047137, 0.00557374224920009, -0.0196623144844763, -0.0103860554652283, 0.00321800158796927, 0.00650269929073345, -0.0175865087129071, -0.00928957041533348, 0.00287826812096062, 0.00581619106052783, -0.00804500396992314, 0.0024926533116548, 0.00503696921168107, 0.00203524290640432, 0.00411266813957582, 0.00290809553026392, -0.0705593524717798, -0.0127646671417816, 0.00450295131008957, 0.00380568845855756, -0.000736968401042374, -0.00212744452363027, -0.0116524935543313, 0.0041106133464725, 0.00347410235931506, -0.000672757029032359, -0.0019420822590552, 0.00367664434877216, 0.00310733161448835, -0.000601732179702866, -0.00173705117965752, 0.00269102811612943, -0.000521115353897279, -0.00150433044925714, -0.000425488904726059, -0.00122828066840393, -0.000868525589828762},
    {0.010023442173431, -0.0603750614533913, -0.00610007264692972, -0.00431340273437459, -0.00564756986660034, 0.0218729440398858, 0.00126283501293975, 0.0178591840233902, 0.00103110047034114, 0.000729098134662857, -0.0417574386216002, 0.016805465198862, 0.000867830491206142, -0.00526094276094274, 0.0145539597846298, 0.000751563251563242, -0.00455611007883226, 0.000613648825252326, -0.0037200483016969, -0.00263047138047136, 0.0055737422492001, 0.0196623144844763, -0.0103860554652283, -0.00321800158796928, 0.00650269929073344, 0.0175865087129071, -0.00928957041533347, -0.00287826812096062, 0.00581619106052785, -0.00804500396992314, -0.0024926533116548, 0.00503696921168109, -0.00203524290640433, 0.00411266813957582, 0.00290809553026392, 0.0705593524717798, -0.0127646671417816, -0.00450295131008957, 0.00380568845855755, 0.000736968401042381, -0.00212744452363028, -0.0116524935543313, -0.00411061334647249, 0.00347410235931505, 0.00067275702903236, -0.00194208225905522, -0.00367664434877216, 0.00310733161448835, 0.000601732179702862, -0.00173705117965754, 0.00269102811612941, 0.000521115353897267, -0.00150433044925715, 0.000425488904726055, -0.00122828066840394, -0.00086852558982877},
    {-0.0100234421734311, 0.0452812960900435, -0.0113287063442981, -0.00801060507812424, -0.00141189246665007, -0.0164047080299144, 0.0145226026488074, -0.0133943880175427, 0.0118576554089232, 0.00838462854862294, 0.00894802256177145, -0.0142846454190327, 0.014536160727703, -0.00939454064454065, -0.0123708658169354, 0.0125886844636845, -0.00813591085505765, 0.0102786178229763, -0.00664294339588732, -0.00469727032227034, -0.00836061337380011, -0.00245778931055953, 0.00830884437218264, -0.00965400476390776, 0.00650269929073342, -0.00219831358911338, 0.00743165633226677, -0.00863480436288184, 0.00581619106052786, 0.00643600317593851, -0.00747795993496437, 0.00503696921168106, -0.0061057287192129, 0.00411266813957581, 0.00290809553026391, -0.0352796762358899, 0.0191470007126724, -0.00788016479265674, 0.000951422114639392, 0.00221090520312706, -0.00212744452363026, 0.017478740331497, -0.00719357335632687, 0.000868525589828777, 0.00201827108709706, -0.00194208225905521, -0.00643412761035126, 0.000776832903622074, 0.00180519653910858, -0.00173705117965752, 0.00067275702903235, 0.00156334606169178, -0.00150433044925714, 0.00127646671417815, -0.00122828066840393, -0.000868525589828756},
    {0.0787556170769578, 0.0679219441350652, -0.00435719474780692, -0.00308100195312471, 0.0705946233325039, -0.0328094160598287, 0.0126283501293978, -0.0267887760350854, 0.03608851646194, -0.00729098134662863, 0.0298267418725716, 0.0151249186789758, -0.0325436434202305, 0.0248015873015872, 0.0130985638061668, -0.0229226791726792, 0.0169226945785199, -0.0248527774227189, 0.0164744996218005, 0.0142797017797018, 0, 0.0245778931055953, -0.0363511941282991, 0.0362025178646542, -0.0232239260383337, 0.0219831358911338, -0.00743165633226679, 0, 0.00166176887443652, -0.0345935170706695, 0.0130864298861876, -0.00359783515120077, 0.0335815079556711, -0.014688100498485, -0.0178640154001927, 0, 0, 0, 0, 0, 0, 0, 0.0226083734055987, -0.0334382352084074, 0.0333014729371016, -0.0213629048496074, -0.00459580543596519, -0.0194208225905522, 0.0270779480866286, -0.0191075629762328, 0.0090822198919368, 0.00703505727761299, -0.00812338442598857, -0.0102117337134253, 0.00196524906944628, 0.00607967912880137},
    {-0.0214788046573525, 0.00377344134083702, 0.00217859737390337, -0.021567013671873, 0, 0.0984282481794862, -0.0852413633734345, -0.0312535720409329, -0.0180442582309699, 0.0309866707231716, 0, 0.0453747560369276, 0, -0.0202922077922077, -0.0116431678277039, 0.0236742424242425, -0.0335199527228374, -0.00138070985681772, -0.000797153207506455, 0.00394570707070704, 0, 0, 0.0363511941282991, -0.0603375297744235, 0.0464478520766673, 0, 0.0241528830798671, 0, -0.0108014976838374, -0.00724050357293081, -0.0392592896585629, 0.0259044130886455, 0.0188259968842399, 0.0108691943688789, -0.0178640154001926, 0, 0, 0, 0, 0, 0, 0, 0, 0.0334382352084074, -0.0555024548951693, 0.0427258096992148, 0, -0.0155366580724417, 0, 0.00694820471863008, 0.00302740663064561, 0.0164151336477636, -0.0108311792346514, -0.00680782247561684, -0.00393049813889257, 0.00607967912880131},
    {0.0787556170769581, 0.0301875307266957, 0.0610007264692977, -0.00308100195312485, 0, 0.0218729440398858, 0.0757701007763862, 0.0178591840233902, -0.0412440188136458, -0.00729098134662867, 0, 0, 0.0520698294723689, -0.00901875901875909, 0, -0.0127765752765753, 0.0286384062098029, 0.00184094647575695, -0.0297603864135753, 0.0142797017797018, 0, 0, 0, 0.0402250198496157, -0.0464478520766674, 0, 0, 0.014391340604803, 0.0182794576188018, 0, -0.0137095932141014, -0.0345392174515274, 0.00407048581280862, 0.036426489236243, -0.0178640154001927, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0.0370016365967796, -0.0427258096992148, 0, 0, -0.030086608985143, 0.0243187165152054, 0, 0.014070114555226, 0, -0.00340391123780844, -0.00982624534723146, 0.00607967912880138},
    {-0.0214788046573519, 0.00377344134083692, -0.0196073763651313, 0.00924300585937408, 0, -0.0109364720199429, 0.00947126259704824, 0.102690308134494, -0.0695992817480272, -0.060150596109686, 0, -0.00504163955965862, 0.0065087286840461, -0.00450937950937948, 0.0465726713108155, 0.0214195526695527, -0.0289638426440052, -0.00782402252196701, -0.0263060558477138, 0.000563672438672498, 0, 0, 0, 0, 0, 0, 0.0334424534952005, -0.0431740218144092, 0.0299118397398575, 0.0289620142917233, -0.00249265331165477, -0.00431740218144094, -0.0605484764655283, 0.0185070066280912, 0.0461140862656136, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0.0349574806629939, -0.0451299134777144, 0.0312669212338355, -0.0121096265225824, -0.028140229110452, 0.0270779480866286, 0.023827378664659, 0, -0.0182390373864041},
    {-0.0214788046573522, -0.0150937653633478, 0.0130715842434209, 0.00924300585937409, 0, -0.0109364720199429, 0.00947126259704823, -0.00892959201169517, 0.123732056440937, -0.060150596109686, 0, 0, -0.00650872868404615, 0.00676406926406924, 0, 0.0574945887445888, 0.0126920209338899, -0.026693723898476, 0.00637722566005184, 0.000563672438672442, 0, 0, 0, 0, 0, 0, 0, 0.0431740218144092, -0.0448677596097863, 0, 0.0199412264932383, 0.0215870109072046, -0.0142467003448301, -0.0616900220936373, 0.0461140862656136, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0.0451299134777144, -0.0469003818507533, 0, -0.0375203054806026, 0.0162467688519772, 0.0119136893323295, 0.0206351152291861, -0.0182390373864041},
    {0.078755617076958, 0.0301875307266957, 0.0174287789912279, 0.0585390371093694, 0, 0.0218729440398858, -0.0189425251940966, 0.0178591840233902, 0.0103110047034113, 0.083846285486229, 0, 0, 0, 0, 0, 0.00526094276094275, -0.00455611007883227, 0.0533874477969518, 0.0308232573569172, -0.0319414381914382, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0.0348971463631671, -0.0302218152700865, 0.0284934006896603, 0.0164506725583033, -0.0643935438844155, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0.0437736897273697, -0.0379091273212801, -0.023827378664659, -0.0137567434861241, 0.0303983956440068},
    {0.0787556170769583, -0.0679219441350652, -0.00435719474780694, -0.00308100195312467, 0.0705946233325039, 0.0328094160598287, 0.0126283501293977, 0.0267887760350853, 0.03608851646194, -0.00729098134662864, -0.0298267418725716, 0.0151249186789759, 0.0325436434202305, 0.0248015873015873, 0.0130985638061668, 0.0229226791726792, 0.0169226945785199, 0.0248527774227189, 0.0164744996218005, 0.0142797017797018, 0, -0.0245778931055954, -0.036351194128299, -0.0362025178646541, -0.0232239260383336, -0.0219831358911338, -0.00743165633226678, 0, 0.00166176887443654, -0.0345935170706695, -0.0130864298861876, -0.00359783515120077, -0.0335815079556711, -0.014688100498485, -0.0178640154001927, 0, 0, 0, 0, 0, 0, 0, -0.0226083734055987, -0.0334382352084074, -0.0333014729371016, -0.0213629048496074, 0.00459580543596522, -0.0194208225905522, -0.0270779480866286, -0.0191075629762328, 0.0090822198919368, -0.007035057277613, -0.00812338442598858, 0.0102117337134253, 0.00196524906944627, 0.00607967912880136},
    {-0.0214788046573522, -0.00377344134083698, 0.00217859737390352, -0.0215670136718729, 0, -0.0984282481794863, -0.0852413633734345, 0.031253572040933, -0.0180442582309699, 0.0309866707231716, 0, 0.0453747560369276, 0, -0.0202922077922078, -0.0116431678277039, -0.0236742424242425, -0.0335199527228375, 0.00138070985681774, -0.000797153207506485, 0.00394570707070704, 0, 0, 0.0363511941282991, 0.0603375297744236, 0.0464478520766674, 0, 0.024152883079867, 0, -0.0108014976838374, -0.0072405035729308, 0.0392592896585629, 0.0259044130886455, -0.0188259968842399, 0.0108691943688789, -0.0178640154001927, 0, 0, 0, 0, 0, 0, 0, 0, 0.0334382352084074, 0.0555024548951693, 0.0427258096992148, 0, -0.0155366580724417, 0, 0.0069482047186301, 0.0030274066306456, -0.0164151336477636, -0.0108311792346515, 0.00680782247561683, -0.00393049813889256, 0.00607967912880134},
    {0.078755617076958, -0.0301875307266958, 0.0610007264692976, -0.00308100195312472, 0, -0.0218729440398859, 0.0757701007763862, -0.0178591840233903, -0.0412440188136458, -0.00729098134662865, 0, 0, -0.0520698294723688, -0.00901875901875904, 0, 0.0127765752765752, 0.0286384062098029, -0.00184094647575696, -0.0297603864135752, 0.0142797017797018, 0, 0, 0, -0.0402250198496157, -0.0464478520766673, 0, 0, -0.0143913406048031, 0.0182794576188018, 0, 0.0137095932141014, -0.0345392174515273, -0.00407048581280864, 0.036426489236243, -0.0178640154001927, 0, 0, 0, 0, 0, 0, 0, 0, 0, -0.0370016365967795, -0.0427258096992148, 0, 0, 0.0300866089851429, 0.0243187165152054, 0, -0.014070114555226, 0, 0.00340391123780844, -0.00982624534723146, 0.00607967912880137},
    {-0.0214788046573522, -0.00377344134083677, -0.0196073763651313, 0.00924300585937403, 0, 0.0109364720199429, 0.00947126259704829, -0.102690308134494, -0.0695992817480271, -0.0601505961096861, 0, -0.00504163955965865, -0.00650872868404618, -0.00450937950937947, 0.0465726713108155, -0.0214195526695527, -0.0289638426440052, 0.00782402252196705, -0.0263060558477139, 0.000563672438672498, 0, 0, 0, 0, 0, 0, 0.0334424534952005, 0.0431740218144092, 0.0299118397398575, 0.0289620142917233, 0.00249265331165484, -0.00431740218144093, 0.0605484764655283, 0.0185070066280911, 0.0461140862656136, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0.0349574806629939, 0.0451299134777144, 0.0312669212338355, -0.0121096265225824, 0.028140229110452, 0.0270779480866286, -0.023827378664659, 0, -0.018239037386404},
    {-0.0214788046573521, 0.0150937653633479, 0.0130715842434208, 0.00924300585937423, 0, 0.010936472019943, 0.00947126259704825, 0.00892959201169522, 0.123732056440937, -0.060150596109686, 0, 0, 0.00650872868404613, 0.00676406926406924, 0, -0.0574945887445887, 0.0126920209338899, 0.0266937238984759, 0.00637722566005184, 0.000563672438672452, 0, 0, 0, 0, 0, 0, 0, -0.0431740218144092, -0.0448677596097863, 0, -0.0199412264932383, 0.0215870109072046, 0.0142467003448301, -0.0616900220936373, 0.0461140862656136, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, -0.0451299134777144, -0.0469003818507533, 0, 0.0375203054806026, 0.0162467688519772, -0.0119136893323295, 0.0206351152291861, -0.0182390373864041},
    {0.0787556170769579, -0.0301875307266956, 0.017428778991228, 0.0585390371093696, 0, -0.0218729440398858, -0.0189425251940965, -0.0178591840233903, 0.0103110047034114, 0.083846285486229, 0, 0, 0, 0, 0, -0.00526094276094277, -0.00455611007883227, -0.0533874477969518, 0.0308232573569172, -0.0319414381914383, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, -0.0348971463631671, -0.0302218152700865, -0.0284934006896603, 0.0164506725583033, -0.0643935438844154, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, -0.0437736897273697, -0.0379091273212801, 0.023827378664659, -0.0137567434861241, 0.0303983956440068},
    {0.078755617076958, -0.0377344134083697, -0.0566435317214906, -0.00308100195312479, 0.0423567739995023, 0.0546823600997145, 0.0378850503881931, 0.0446479600584756, 0.00515550235170578, -0.00729098134662856, 0.0417574386216002, -0.00504163955965862, -0.0247331689993752, -0.0202922077922078, 0.0295930515620807, 0.0101461038961039, 0.00216957622801541, 0.0266937238984759, 0.0132858867917747, 0.0142797017797018, -0.0557374224920008, -0.0245778931055953, -0.00519302773261415, 0.00402250198496156, 0.00464478520766674, -0.00219831358911332, -0.0130053985814669, -0.0158304746652834, -0.0108014976838375, -0.0168945083368386, -0.026796023100289, -0.0194283098164841, -0.0295110221428625, -0.0217383887377579, -0.0178640154001927, 0, 0, 0, 0, 0, 0, -0.0512709716390577, -0.0226083734055987, -0.00477689074405822, 0.00370016365967794, 0.00427258096992147, -0.0367664434877216, -0.00776832903622086, 0.00601732179702859, 0.00694820471863011, -0.00908221989193683, 0.00703505727761299, 0.00812338442598859, 0.00680782247561683, 0.00786099627778515, 0.00607967912880135},
    {-0.0214788046573522, 0, -0.00435719474780689, -0.0215670136718729, -0.127070321998507, 0, 0.0284137877911447, 0, 0.0360885164619399, 0.0309866707231715, 0, 0.0453747560369275, 0, -0.0202922077922078, -0.0422064833754266, 0, -0.00618329224984384, 0, 0.0015943064150129, 0.00394570707070703, 0.0836061337380012, 0, -0.0103860554652283, 0, 0.00464478520766673, 0, 0.024152883079867, 0, -0.0108014976838374, 0.043443021437585, 0, -0.0194283098164841, 0, -0.0217383887377578, -0.0178640154001926, 0, 0, 0, 0, 0, 0, 0.0769064574585866, 0, -0.00955378148811639, 0, 0.00427258096992146, 0, -0.0155366580724417, 0, 0.00694820471863015, -0.0181644397838736, 0, 0.00812338442598858, 0, 0.00786099627778515, 0.00607967912880134},
    {0.0787556170769581, 0.0377344134083697, -0.0566435317214905, -0.00308100195312477, 0.0423567739995024, -0.0546823600997146, 0.0378850503881931, -0.0446479600584756, 0.00515550235170579, -0.00729098134662863, -0.0417574386216002, -0.00504163955965864, 0.0247331689993752, -0.0202922077922078, 0.0295930515620807, -0.0101461038961039, 0.00216957622801542, -0.0266937238984759, 0.0132858867917747, 0.0142797017797018, -0.0557374224920008, 0.0245778931055954, -0.00519302773261416, -0.00402250198496156, 0.00464478520766676, 0.00219831358911337, -0.0130053985814669, 0.0158304746652834, -0.0108014976838375, -0.0168945083368386, 0.026796023100289, -0.0194283098164841, 0.0295110221428625, -0.0217383887377579, -0.0178640154001927, 0, 0, 0, 0, 0, 0, -0.0512709716390577, 0.0226083734055987, -0.00477689074405822, -0.00370016365967795, 0.00427258096992149, 0.0367664434877216, -0.00776832903622086, -0.00601732179702859, 0.00694820471863009, -0.00908221989193678, -0.00703505727761301, 0.00812338442598859, -0.00680782247561686, 0.00786099627778517, 0.00607967912880135},
    {-0.021478804657352, -0.0188672067041847, 0.00653579212171052, 0.00924300585937401, 0.0141189246665008, 0, -0.00315708753234946, -0.111619900146189, -0.05413277469291, -0.0601505961096861, -0.00894802256177153, -0.00252081977982931, 0.000650872868404653, 0.00112734487734485, -0.0276525235907967, 0.0360750360750361, 0.0374251899332651, -0.0188697013765088, 0.019928830187662, 0.000563672438672417, 0, 0, 0, 0, 0, 0.0593544669060614, 0.0167212267476003, -0.00431740218144091, -0.0074779599349644, 0.00321800158796925, 0.0224338798048931, 0.018708742786244, 0.0463017761206981, 0.0431830154655461, 0.0461140862656137, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0.0620433733855302, 0.017478740331497, -0.00451299134777146, -0.0078167303084589, 0.0363288795677472, -0.00938007637015066, -0.0162467688519772, -0.0119136893323295, -0.020635115229186, -0.018239037386404},
    {-0.021478804657352, 0.0188672067041848, 0.00653579212171056, 0.00924300585937413, 0.0141189246665008, 0, -0.00315708753234944, 0.111619900146189, -0.05413277469291, -0.060150596109686, 0.00894802256177151, -0.00252081977982929, -0.000650872868404627, 0.00112734487734488, -0.0276525235907967, -0.0360750360750361, 0.0374251899332651, 0.0188697013765088, 0.0199288301876619, 0.000563672438672459, 0, 0, 0, 0, 0, -0.0593544669060614, 0.0167212267476003, 0.00431740218144093, -0.0074779599349644, 0.00321800158796927, -0.0224338798048931, 0.018708742786244, -0.046301776120698, 0.0431830154655461, 0.0461140862656136, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, -0.0620433733855302, 0.017478740331497, 0.00451299134777145, -0.00781673030845891, 0.0363288795677472, 0.00938007637015065, -0.0162467688519772, 0.0119136893323295, -0.020635115229186, -0.0182390373864041},
    {0.0787556170769582, 0, -0.0348575579824558, 0.0585390371093696, -0.0282378493330016, 0, 0.00631417506469889, 0, -0.0206220094068229, 0.083846285486229, 0, 0, 0, 0, -0.00679184789949391, 0, 0.00151870335961076, 0, -0.0616465147138344, -0.0319414381914382, 0, 0, 0, 0, 0, 0, 0, 0, 0, -0.0450520222315696, 0, 0.0100739384233621, 0, -0.0329013451166065, -0.0643935438844154, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, -0.0565115904387179, 0, 0.01263637577376, 0, 0.027513486972248, 0.0303983956440067},
    {0.0787556170769578, -0.0377344134083695, -0.0217859737390347, -0.0523770332031201, 0.0423567739995023, 0.0546823600997146, 0, 0.0446479600584757, 0.0257775117585286, 0.0291639253865144, 0.0417574386216002, 0.0285692908380655, 0.0273366604729936, 0.0187890812890813, -0.00921750786359888, -0.00789141414141413, -0.00347132196482459, -0.0248527774227189, -0.0143487577351166, -0.015031265031265, -0.0557374224920008, -0.00491557862111907, -0.0197335053839337, -0.0394205194526234, -0.0325134964536672, -0.0241814494802472, -0.00928957041533347, -0.0158304746652834, -0.0141250354327105, -0.00241350119097692, -0.000623163327913714, -0.00215870109072046, 0.00712335017241511, 0.00411266813957583, 0.00540074884191872, 0, -0.0510586685671264, -0.0405265617908061, -0.0114170653756727, 0.010317557614593, 0.0127646671417816, 0.00466099742173251, -0.0143871467126538, -0.00564541633388696, 0.00571843474677503, 0.00737991258440982, 0.00367664434877214, -0.00155366580724416, 0.00240692871881144, 0.00347410235931507, 0.00100913554354852, 0.000260557676948644, 0.000902598269554298, -0.000850977809452118, -0.000491312267361587, -0.000868525589828769},
    {-0.0214788046573518, 0, -0.0217859737390347, 0.00308100195312481, -0.127070321998507, 0, 0.0473563129852414, 0, 0.0257775117585285, 0.0127592173566, 0, -0.0352914769176103, 0, 0.00225468975468975, 0.0509388592462044, 0, -0.00943765659186686, 0, -0.0143487577351166, -0.0129644660894661, 0.0836061337380013, 0, 0.0477758551400502, 0, -0.0325134964536671, 0, 0.00928957041533347, 0, -0.0141250354327104, -0.0144810071458617, 0, -0.00215870109072047, 0, 0.00411266813957582, 0.00540074884191876, 0, 0.0765880028506897, 0, -0.0228341307513453, 0, 0.0127646671417816, -0.00699149613259876, 0, -0.0112908326677739, 0, 0.00737991258440979, 0, -0.00310733161448836, 0, 0.00347410235931506, 0.00201827108709705, 0, 0.000902598269554284, 0, -0.000491312267361592, -0.000868525589828792},
    {0.0787556170769577, 0.0377344134083696, -0.0217859737390349, -0.05237703320312, 0.0423567739995023, -0.0546823600997146, 0, -0.0446479600584756, 0.0257775117585286, 0.0291639253865144, -0.0417574386216002, 0.0285692908380655, -0.0273366604729936, 0.0187890812890813, -0.0092175078635989, 0.00789141414141413, -0.00347132196482458, 0.0248527774227189, -0.0143487577351166, -0.015031265031265, -0.0557374224920009, 0.00491557862111903, -0.0197335053839338, 0.0394205194526234, -0.0325134964536671, 0.0241814494802472, -0.00928957041533348, 0.0158304746652834, -0.0141250354327105, -0.00241350119097695, 0.000623163327913703, -0.00215870109072045, -0.00712335017241511, 0.00411266813957582, 0.00540074884191871, 0, -0.0510586685671264, 0.0405265617908061, -0.0114170653756726, -0.010317557614593, 0.0127646671417816, 0.0046609974217325, 0.0143871467126537, -0.00564541633388698, -0.00571843474677501, 0.00737991258440984, -0.00367664434877218, -0.00155366580724417, -0.00240692871881142, 0.00347410235931505, 0.00100913554354854, -0.000260557676948633, 0.000902598269554291, 0.000850977809452119, -0.000491312267361577, -0.000868525589828757},
    {-0.0214788046573522, -0.0188672067041847, 0.0108929868695173, 0.00308100195312474, 0.0141189246665008, -0.109364720199429, -0.0789271883087357, 0.0223239800292378, -0.0128887558792642, 0.0127592173566001, -0.00894802256177151, -0.0277290175781224, -0.00585785581564148, 0.0191648629148629, 0.00145539597846299, 0.0383297258297258, 0.0348216984596467, -0.0124263887113595, 0.00717437886755833, -0.0129644660894661, 0, 0.0589869434534289, 0.00727023882565978, 0.0506835250105158, 0.0650269929073344, -0.00659494076734013, 0.0148633126645336, -0.00431740218144089, -0.00166176887443655, -0.0040225019849616, -0.00810112326287806, -0.0115130724838424, 0.00356167508620753, -0.00205633406978792, 0.00540074884191869, 0, 0, 0.0607898426862092, 0.0399597288148543, -0.0154763364218895, -0.031911667854454, 0, -0.0123318400394175, 0.00303983956440066, -0.00302740663064559, -0.00776832903622087, 0.00137874163078958, -0.0042725809699215, 0.000902598269554286, 0.000868525589828745, 0.00100913554354854, 0.000781673030845884, 0.00180519653910853, -0.000425488904726043, 0.0002456561336808, -0.000868525589828759},
    {-0.0214788046573523, 0.0188672067041848, 0.0108929868695173, 0.00308100195312473, 0.0141189246665008, 0.109364720199429, -0.0789271883087356, -0.0223239800292378, -0.0128887558792643, 0.0127592173566, 0.00894802256177147, -0.0277290175781224, 0.00585785581564147, 0.0191648629148629, 0.00145539597846298, -0.0383297258297258, 0.0348216984596467, 0.0124263887113595, 0.00717437886755829, -0.0129644660894661, 0, -0.0589869434534289, 0.0072702388256598, -0.0506835250105158, 0.0650269929073343, 0.00659494076734013, 0.0148633126645336, 0.0043174021814409, -0.00166176887443651, -0.00402250198496155, 0.00810112326287803, -0.0115130724838425, -0.00356167508620755, -0.0020563340697879, 0.00540074884191873, 0, 0, -0.0607898426862092, 0.0399597288148542, 0.0154763364218895, -0.031911667854454, 0, 0.0123318400394175, 0.00303983956440069, 0.0030274066306456, -0.00776832903622087, -0.00137874163078958, -0.00427258096992146, -0.000902598269554311, 0.000868525589828782, 0.00100913554354855, -0.000781673030845884, 0.00180519653910857, 0.000425488904726043, 0.000245656133680809, -0.000868525589828767},
    {0.0787556170769581, 0, 0.0435719474780695, -0.0523770332031201, -0.0282378493330016, 0, 0.0631417506469885, 0, -0.0515550235170572, 0.0291639253865144, 0, -0.00672218607954481, 0, -0.0601250601250601, 0.000970263985641982, 0, -0.0125835421224891, 0, 0.0286975154702333, -0.015031265031265, 0, 0, -0.0436214329539588, 0, -0.0650269929073343, 0, 0.0111474844984002, 0, 0.031573608614294, -0.00160900079398462, 0, -0.0028782681209606, 0, -0.00822533627915164, 0.00540074884191871, 0, 0, 0, -0.0532796384198057, 0, 0.0425488904726053, 0, 0, 0.018239037386404, 0, -0.00388416451811043, 0, -0.00466099742173252, 0, -0.00347410235931507, 0.00067275702903234, 0, 0.00120346435940573, 0, 0.000982624534723144, -0.000868525589828764},
    {0.10739402328676, -0.188672067041848, -0.108929868695174, -0.0770250488281177, -0.0705946233325038, -0.0546823600997147, -0.0315708753234941, -0.0446479600584757, -0.0257775117585286, -0.0182274533665716, 0.0894802256177148, -0.0151249186789758, 0.058578558156415, 0.0744047619047618, -0.0130985638061668, -0.0281836219336219, 0.00976309302606916, 0.069035492840886, 0.0398576603753241, 0.0597492784992785, 0, 0.0737336793167861, 0.036351194128299, -0.0120675059548847, -0.0232239260383337, 0.0659494076734016, 0.00743165633226678, 0.0431740218144092, 0.0465295284842228, 0.0345935170706695, 0.0155790831978425, 0.0251848460584054, -0.0254405363300539, -0.0146881004984851, -0.0328199352701214, 0, 0, 0, 0, 0, 0, 0, 0.0678251202167962, 0.0334382352084074, -0.0111004909790339, -0.0213629048496074, -0.0137874163078956, 0.0194208225905522, -0.0090259826955429, -0.0191075629762328, -0.00908221989193678, -0.00234501909253769, -0.00812338442598858, 0.00340391123780841, 0.00196524906944627, 0.00607967912880134},
    {0.107394023286761, 0.188672067041848, -0.108929868695174, -0.0770250488281178, -0.0705946233325039, 0.0546823600997145, -0.0315708753234942, 0.0446479600584757, -0.0257775117585287, -0.0182274533665715, -0.0894802256177147, -0.0151249186789758, -0.0585785581564149, 0.0744047619047619, -0.0130985638061668, 0.0281836219336219, 0.00976309302606916, -0.0690354928408859, 0.039857660375324, 0.0597492784992785, 0, -0.0737336793167861, 0.036351194128299, 0.0120675059548847, -0.0232239260383338, -0.0659494076734015, 0.00743165633226678, -0.0431740218144092, 0.0465295284842228, 0.0345935170706695, -0.0155790831978424, 0.0251848460584053, 0.0254405363300539, -0.0146881004984851, -0.0328199352701214, 0, 0, 0, 0, 0, 0, 0, -0.0678251202167962, 0.0334382352084074, 0.0111004909790339, -0.0213629048496074, 0.0137874163078956, 0.0194208225905522, 0.00902598269554287, -0.0191075629762328, -0.0090822198919368, 0.00234501909253767, -0.00812338442598857, -0.0034039112378084, 0.00196524906944627, 0.00607967912880135},
    {0.107394023286761, 0, 0.217859737390348, -0.0770250488281178, 0, 0, -0.094712625970483, 0, 0.0515550235170572, -0.0182274533665716, 0, -0.0907495120738551, 0, -0.0946969696969696, 0.0232863356554077, 0, -0.0227805503941613, 0, -0.0797153207506479, 0.0597492784992786, 0, 0, -0.0727023882565981, 0, 0.0464478520766674, 0, -0.0483057661597341, 0, -0.0781031370985168, 0.0144810071458616, 0, 0.0431740218144092, 0, 0.0293762009969701, -0.0328199352701215, 0, 0, 0, 0, 0, 0, 0, 0, -0.0668764704168148, 0, 0.0427258096992148, 0, 0.0310733161448835, 0, 0.00694820471863007, -0.00605481326129118, 0, -0.0108311792346515, 0, -0.00393049813889257, 0.00607967912880135},
    {0.107394023286761, 0, 0, 0.231075146484353, 0, 0, 0, 0, 0, -0.109364720199429, 0, 0.0100832791193172, 0, -0.00450937950937963, -0.093145342621631, 0, -0.104139658944738, 0, 0, -0.09244227994228, 0, 0, 0, 0, 0, 0, -0.066884906990401, 0, 0.0299118397398574, -0.0579240285834467, 0, -0.0647610327216138, 0, 0, 0.0835038859404354, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, -0.0699149613259879, 0, 0.0312669212338356, 0.0242192530451648, 0, 0.0270779480866286, 0, 0, -0.018239037386404}};
    
    // Interesting (new) part
    // Tables of derivatives of the polynomial base (transpose)
    const static double dmats0[56][56] = \
    {{0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {6.32455532033671, 0, 0, 0, 0, 2.16505338810566e-14, 0, 2.05845987409731e-14, -1.22512335797976e-14, 0, 0, 0, 0, 0, -1.08130730953582e-14, 0, 1.22483177886604e-14, 0, 1.21041660596838e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {-1.33727360005496e-14, 11.2249721603218, 0, 0, 0, 2.8506046608589e-14, 0, 0, 0, 1.00951115474313e-14, 0, -1.20188905885721e-14, 1.28545373789999e-14, 0, 0, -1.03476262582923e-14, -1.93999491268998e-14, 0, 1.33226762955019e-14, 0, 0, 0, 0, 0, 1.05444038416428e-14, 0, 0, 0, -1.00620217474173e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1.11065094110678e-14, 0, 0, -1.90621990730363e-14, 0, 0, 1.25288297745861e-14, 0, 0, 0, 0, 0, 0, 0},
    {4.58257569495582, -1.0968053520467e-14, 8.36660026534075, -1.18321595661994, 0, 0, -1.51387532776558e-14, -1.08221538572235e-14, -1.3598597955937e-14, 0, 0, 1.9215300364784e-14, -1.53957147424772e-14, 0, 0, 1.50088321802921e-14, 0, 0, 0, -1.15463194561016e-14, 0, 0, 0, 0, 0, 0, 0, -1.00882470358913e-14, 0, 0, 0, 0, 0, 0, 0, 0, 1.03041185216246e-14, 0, 0, 0, 0, 0, 0, 0, 1.67984122495906e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {3.74165738677394, 2.85608062261247e-14, 0, 8.69482604771366, 0, 1.32677370521961e-14, -1.0621406876971e-14, 4.08006961549745e-14, -2.2818496382273e-14, -2.1316282072803e-14, 0, 0, 0, 0, 0, -1.21576837027352e-14, 0, 1.41223042954593e-14, 1.22383070012588e-14, -1.4210854715202e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, -1.00375396573897e-14, -1.10000775654917e-14, 0, -1.24344978758018e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, -1.27154464053485e-14, 0, -1.33226762955019e-14},
    {0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {5.49909083394704, -2.12464983341981e-14, -3.34664010613631, -2.36643191323983, 15.4919333848297, 0, 0.692820323027548, 0, 0.565685424949223, 0.400000000000019, -2.45617804595055e-14, 0, -2.50285594161414e-14, -1.12752625494543e-14, 0, -2.06682744450517e-14, 0, -1.87825158914691e-14, 0, 0, 0, 1.67613011937654e-14, 0, 0, 0, 1.28202275780632e-14, 0, 1.30009839200864e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, -1.30459382160744e-14, 0, 0, 0, 0, -1.95146235819213e-14, 1.32110885228657e-14, 1.12246955104063e-14, 0, -1.24476276034392e-14, 0, 0, 0, 1.14879018579048e-14, 0, 0, 0, 0, 0},
    {1.79012076899773e-14, 4.89897948556635, 0, 0, 0, 14.1985914794391, -1.56346770721582e-14, -0.828078671210841, 1.85962356624714e-14, 1.44765989981636e-14, 0, -3.14433996145488e-14, 0, -1.75618818945982e-14, -1.31955099699921e-14, 1.85934501523347e-14, -2.7953864447905e-14, 1.28785870856518e-14, 0, 0, 0, 0, 1.3985009962376e-14, 0, 1.27337992225105e-14, 0, 1.07957902261976e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, -1.64679520687763e-14, 0, -1.14163879478899e-14, 0, 0, -1.9487535909084e-14, 1.99307532306351e-14, -2.46558752706505e-14, 0, 0, 0, 0, 0, 1.11086317443674e-14, 0, 0, 0, 0},
    {3.60000000000003, -1.17043800208557e-14, 8.76356092008264, -1.54919333848296, 0, 0, 9.52470471983252, -1.08725201100446e-14, -1.48131215963609, 0.261861468283195, 0, 0, -2.19166032081878e-14, -1.15727480921906e-14, 1.19782412678522e-14, 0, 2.00140531387981e-14, -1.08284340452512e-14, -2.62934736398987e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1.25371390349952e-14, 0, 0, 0, 0, 0, -1.02110492017178e-14, 0, 0, 0, 0, 0, 0},
    {0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {-2.24915431235802e-14, 4.24264068711927, -1.64313007644523e-14, 0, 0, 0, 0, 14.3427433120127, 0, 1.59620734906468e-14, 0, 0, 2.06494753825246e-14, 1.24642807480246e-14, 1.31854538806711e-14, 0, -2.70659809367255e-14, 0, 0, -1.23240404410758e-14, 0, 0, 0, 0, 0, 0, 0, 0, -1.22125558154546e-14, 1.05003199257295e-14, 1.39537903134404e-14, 1.71734205981969e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, -2.01310662662892e-14, 0, 0, 2.30351452905449e-14, 0, 0, 0, 0, 0, 1.24344978758018e-14, 0},
    {3.11769145362398, 0, 3.1622776601684, 4.91934955049952, 0, 0, 0, 0, 10.690449676497, -2.41897262725906, 0, 0, 0, 1.08320679531897e-14, 0, 0, 0, 1.61780317385165e-14, 0, -3.01980662698043e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, -1.01163085491832e-14, 0, -1.4210854715202e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1.90254351599586e-14, 0, 0, 0, 1.4669610635169e-14, 0, 0, 0, 1.20972007395846e-14, 0, 0, 0},
    {0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {2.54558441227159, 0, 0, 7.66811580507233, 0, 0, 0, 2.00435576705123e-14, 0, 10.3691851174526, 0, 0, 0, 0, 0, 0, -1.4355744756654e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, -1.31237403589518e-14, 0, 1.19255832274785e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, -1.21989789544854e-14, 0, 0, 0, 1.08631431175004e-14, 0, 0, 0, 0, 0, 0},
    {0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {-3.7592379574637e-14, 11.60576914948, 1.02695629777827e-14, 0, 0, -4.20458932936044, 1.40755215458979e-14, -3.43303281162799, 0, 0, 19.6214168703486, 2.18203847862408e-14, 0.611677741841185, 1.46622784386203e-14, 0, 0.529728463363973, 1.12642251693005e-14, 0.4325214791568, 0, 0, 0, -1.06703789507439e-14, 0, 0, 0, -1.60649692254054e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1.42508255988556e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {4.53137212479104, -2.04494956755329e-14, 3.54562104171167, -2.50713268211203, 4.78713553878169, 0, -5.56626745075489, 0, -1.04880884817014, 0.741619848709575, -1.23815345194237e-14, 18.8034951158403, -1.86020833228553e-14, 1.68183573174416, -0.603122387046451, -1.92746782669555e-14, 0.701283781217653, 0, 0.132137494528672, -0.0934353184302359, 0, 1.50506010183261e-14, 0, 0, 0, 1.11149238093515e-14, 0, 1.50741577923534e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, -1.5968916436806e-14, 0, 0, 0, 0, -2.36522650283687e-14, 1.26860679795035e-14, 2.21112666692775e-14, -1.04107559329747e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {3.86209880120533e-14, 2.5951288749407, 1.92068583260152e-14, 0, 0, 10.0285307284481, -1.22612314755764e-14, -0.767649473578743, 0, 0, 0, 0, 16.4130361329658, 0, 0, -1.2634761106116, 3.26893402596665e-14, 0.0967147429123276, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1.32878527298855e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, -1.0672045061505e-14, -1.16637535122366e-14, 0, -1.46741880066399e-14, 0, 1.32138836425176e-14, 0, 0, 0, -1.00831659427246e-14, 0, 0, 0, 0, 0, 0, 0},
    {2.96647939483827, 0, 8.12403840463593, -1.64130361329658, 0, 0, 11.7728501222092, 0, -2.40312296813959, 0.485504156227621, 0, 0, -1.71460988515286e-14, 10.2761860629321, 1.14956049951209e-14, -4.33332801490326e-14, -1.48323969741912, -1.03124879870092e-14, 0.302765035409724, -0.0611677741841183, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, -1.45169999216629e-14, 1.16636407607716e-14, -2.16593088290316e-14, 0, 0, 0, 0, -1.3706959798735e-14, 0, 0, 0, 0, 0, 0},
    {0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {4.05298244095222, -1.17512299762369e-14, -1.58564993434419, 4.48489528465113, 4.28174419288836, 0, 0.191485421551272, 0, -3.75233260785876, -4.311612227462, 0, 0, -1.13529814871025e-14, -1.07967706476489e-14, 18.8807171866293, 0, 0.844371341865059, -1.30185990386282e-14, 1.18187368057055, 1.04463861754669, 0, 1.00351859283595e-14, 0, 0, 0, 1.08982318008379e-14, 1.16650614658273e-14, 1.78540699708776e-14, 0, 1.48568586111212e-14, 1.2997651195193e-14, 0, 1.19564095870861e-14, 0, 0, 0, 0, 0, 0, 0, 0, -1.90031310112405e-14, -2.21313572206035e-14, 0, 2.31096376853888e-14, 1.25147787012743e-14, -1.31451704746157e-14, 0, 0, 0, 2.34657400816165e-14, 0, 0, 0, 1.19833135941395e-14, 0},
    {1.03681424251779e-14, 2.321153829896, -1.04360964314765e-14, 0, 0, 3.9242833740697, 0, 5.49285249860475, 0, 1.4249489099847e-14, 0, 0, 2.63684141763982e-14, 1.12010640489931e-14, 0, 17.3044631365566, -5.0266260224736e-14, -1.73008591662709, 0, 0, 0, 0, 0, 0, 1.32329702395699e-14, 0, 1.22894929912907e-14, -1.21668635551694e-14, -1.35421297848238e-14, 1.0253781445641e-14, 0, 1.86415806813024e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 2.41268605947696e-14, -3.37685959257577e-14, 0, 0, 3.2419745025707e-14, 1.21885930822672e-14, 0, 1.77958086184916e-14, 0, 1.4210854715202e-14, 0, 1.40814441510978e-14},
    {2.65329983228435, -2.32098024377698e-14, 4.15220619990512, 2.93605316083775, 0, -1.72381743916861e-14, 2.63248931621763, -4.00364084983131e-14, 9.82591326180784, -2.8226127713774, 0, 0, -1.33474731826653e-14, -1.06959103616322e-14, 1.26654039377864e-14, 1.8991099966876e-14, 11.6081867662439, -1.19927215290095e-14, -3.0948717731947, 0.683876505540241, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1.61148409137901e-14, 1.47100077738277e-14, 0, 0, 0, 0, 0, 0, 0, 1.3422167174583e-14, 0, -2.15269669668842e-14, 0},
    {0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {4.0410983709019e-14, 2.01017818278147, 0, 1.34362847627492e-14, 0, -2.17732253774783e-14, 0, 8.91928088068604, -1.37667655053519e-14, 1.38253977761529e-14, -1.02564879604251e-14, 0, 0, 0, -1.34085149628415e-14, 0, 0, 16.8558564884489, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1.06581410364015e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, -1.48981943277725e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1.77635683940025e-14, 0, 0},
    {2.29782505861523, 1.22923050669652e-14, 1.49829835452879, 5.50917601898536, 0, 0, 0, 1.92682707206214e-14, 6.64803945320939, 4.13676892548485, 0, 0, 1.46016900962614e-14, 2.50030852312528e-14, 0, -1.88410641965856e-14, -3.22541831300375e-14, -1.10413917361675e-14, 12.5636136423842, -3.55352656109507, 0, 0, 0, 0, 0, 0, 0, 0, -2.00805899834693e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, -1.13841034187579e-14, -2.03312427847762e-14, 0, 0, 0, 1.35488987469781e-14, 0, 0, 0, 0, -1.41455019594718e-14, 0},
    {0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {1.87616630392938, -2.83080569710049e-14, 0, 6.22830929985772, 0, 0, 0, -1.88930188353776e-14, 0, 11.0541524195339, 0, 1.07055222640856e-14, 0, 0, 0, 1.48136724622646e-14, 1.71418057121951e-14, -2.40309133366388e-14, 0, 11.6057691494799, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1.22846189228214e-14, 1.32604072649903e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1.33226762955019e-14},
    {0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {4.83186700722507, 0, -4.41087092453692, -3.11895674167863, 16.6749979177077, -1.13866638474088e-14, 2.13065307820926, -2.61564103865851e-14, 1.73967095350101, 1.23013312825381, 0, -4.36653943835008, 1.40983965277648e-14, -0.557935943461962, -3.78153408023779, 2.08111552908862e-14, -0.483186700722529, 1.18208720943676e-14, -0.394520289089675, -0.278967971730977, 23.6981011897578, 0, 0.490653381462656, 0, 0.0626933893908963, -1.13263908790947e-14, 0.438853725736244, 0, 0.0560746721671562, 0.380058475033041, 0, 0.0485620906056454, 0, 0.0396507809421992, 0.0280373360835894, 0, 0, 0, 0, 0, 0, 1.56357143454011e-14, 0, 0, 0, 0, 1.0335662317795e-14, 1.60193852711159e-14, 0, 0, 0, 1.11786220690221e-14, -1.25974506359966e-14, 0, -1.04950117655873e-14, 0},
    {-2.48352929483769e-14, 6.91050664139874, 0, -1.11694286112743e-14, 0, 10.5706241650195, 1.37569619238853e-14, -2.72554057547698, -1.5404344466674e-14, 0, 4.67332857821917, 1.98201416293709e-14, -7.86705867811461, 0, 0, -2.39719223514615, 0, 0.61809450436526, 0, 0, 0, 23.10555488766, 0, 1.62065746295907, 0, -0.469687194424188, 0, 0.790669146636838, 1.33915447641953e-14, 0, 0.240926884676765, 0, -0.0621208350291002, 0, 0, 0, 1.5285431037408e-14, -1.09568490725226e-14, 0, 0, 0, 0, 0, -1.60712929692335e-14, 0, 1.01513003014644e-14, 0, 0, -1.13906592977489e-14, 0, 0, 0, 0, 0, 0, 0},
    {3.85449644663769, 0, 5.98171817966647, -2.48806675764062, 1.90029237516522, 0, -0.339934634239515, 0, -2.35922146606158, 0.981306762925306, -1.25696577846526e-14, 9.9522670305624, 1.59597845699183e-14, -6.45364422706461, -0.430945803685653, 1.41669885314633e-14, 0.0770899289327383, 0, 0.535021138879222, -0.222539456105695, 0, 0, 21.5273297444417, 0, 2.75066129614998, 0, -1.00024047132728, 0, 0.648615649442928, 0.0433116829031283, 0, -0.00774782937530501, 0, -0.053771647653677, 0.0223660568773448, 0, 0, 0, 0, 0, -1.57026876352916e-14, 0, 0, 0, 0, 0, 0, 1.17596913617043e-14, 0, 0, 0, 0, -1.17877079382012e-14, 0, 0, 0},
    {0, 1.54523626091315, 0, 0, 0, 6.71778024562961, 1.24434270987286e-14, -0.609449400220049, -2.44890913103646e-14, 0, 0, 1.86912392652466e-14, 14.6593983227032, 0, 1.30441346035566e-14, -1.52344936219876, 0, 0.138210132827974, 0, 0, 0, 0, 0, 18.119501277094, 0, 0, 0, -1.47332496632642, 0, 0, 0.153112421864233, 0, -0.0138906409944119, 0, 0, 0, 0, 0, 1.22183321677886e-14, 0, 0, 0, -1.08769537630915e-14, 0, -1.31873092152821e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {2.52336024752224, 0, 7.37120708415863, -1.62882203585592, 0, 0, 12.0171306297064, 0, -2.90724328050878, 0.642416074439641, 0, 0, 1.07383829658909e-14, 13.9858820944259, 0, -2.32400745727179e-14, -2.72522906732405, 0, 0.659300804656265, -0.145686271816941, 0, 0, 0, 0, 10.8043830863937, 0, 0, 0, -1.40563403846548, 0, 0, 0.273895826790425, 0, -0.0662622240310308, 0.0146420212340205, 0, 0, 0, 0, 0, -2.09114976580726e-14, 0, 0, 0, 0, -2.35127496508513e-14, 0, 0, 0, -1.14219479838847e-14, 0, 0, 0, 0, 0, 0},
    {0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0, 6.30840061880561, 0, 0, 0, -1.52362350055011, 0, 11.1963004093827, -2.55351295663786e-14, 0, 4.26614580154032, 0, 0.132992762321598, 0, 2.56609067489954e-14, -4.72217953828914, 1.17310134406447e-14, -6.67684597013019, 0, 1.60758884410055e-14, 0, 0, 0, 0, 0, 23.1532444221696, 0, 0.72177887856059, 1.28783366735277e-14, -1.53368636742207e-14, 1.1112512795529, -1.22624479997289e-14, 1.19087439227729, 0, 0, 0, 0, 0, 0, 0, 0, 2.28291451277892e-14, 0, 0, 0, 1.34418865877235e-14, 2.02478204822475e-14, 0, -1.70814325121815e-14, -1.2090896650498e-14, 0, 0, 0, -2.48689957516035e-14, 0, 0},
    {3.518657752745, 0, 1.92724822331884, 2.72554057547703, 1.73472166622177, 0, -2.01705689521107, 0, 3.42052627529744, -4.03112887414927, 0, 4.08831086321546, -1.50711171565645e-14, 0.365669640132009, 5.37643791520789, -1.10002031399277e-14, -6.25148194071022, 0, -2.03981013744553, 1.8080332206528, 0, 1.12311783689995e-14, -1.2495355609558e-14, 0, 0, 0, 22.1880978975596, 2.59644251000699e-14, 1.98456380761453, -1.2652152389098, 0, 1.47113578579291, 0, 0.363818136386349, -0.294009452979948, 0, 0, 0, 0, 0, 0, 0, -2.14091784397362e-14, 3.96979729470341e-14, 2.95273442389967e-14, 0, -1.02898405303999e-14, -1.55944604137083e-14, 0, -1.2756671985334e-14, 2.67341313377078e-14, 0, 0, 0, 0, 0},
    {5.00588290781391e-14, 1.4106012612951, 1.86517468137026e-14, 1.57048767420779e-14, 1.14431516135412e-14, 3.63405410063598, 0, 2.50356888118883, 0, 0, 0, 0, 3.56857028479905, 0, 0, 11.263055412878, 3.38127349244425e-14, -1.49298814645067, 0, 2.47950570155456e-14, 0, 0, 0, 0, 0, 0, 0, 19.3673596462205, 0, 0, -2.65048896124153, 0, 0.266287609379583, 0, 0, 0, 0, 0, 0, 0, 0, 0, -1.24805331057479e-14, 0, 2.67117428927039e-14, 0, 0, 0, -2.83074697144157e-14, 0, 0, 0, 0, 0, 0, 0},
    {2.30350221379958, 0, 4.4158804331639, 1.78428514239955, 0, 0, 4.2661458015403, 1.92100483859941e-14, 7.83741028656788, -2.63899331455873, 0, 0, 0, 2.23427840700302, 1.30548943777412e-14, -2.38078955158898e-14, 13.2221027072096, -1.23603801475692e-14, -4.67379217909115, 1.18363558466232, 0, 0, 0, 0, 0, 0, -1.58444179603095e-14, 0, 12.1258851598178, 0, 1.08652403506101e-14, -3.11150358274815, 0, 0.833612074594099, -0.192474367616158, 0, 0, 0, 0, 0, 0, 0, 0, 1.49231173587483e-14, -1.98974312579798e-14, -1.74186427463722e-14, 0, 0, 0, 0, 0, 0, 0, 0, -2.0574377947513e-14, 0},
    {0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {3.14718316987773, 0, -0.861891607371334, 6.09449400220044, 1.55158222708543, 0, 0.0693888666488801, 0, -3.05941170815564, 1.80277563773202, 0, 0, 0, 0, 9.03122156537873, 2.80112084875395e-14, 0.403888506800968, -1.0607742830904e-14, -3.67461994579463, -5.49650682721189, 0, 0, 0, 0, 0, 0, 1.10205726024203e-14, 0, 1.65064789881475e-14, 21.7841260588564, 0, 0.974215733960566, 1.13989530288554e-14, 1.78974858569577, 1.80791908794933, 0, 0, 0, 0, 0, 0, -1.4994119558265e-14, 0, -2.17157199142971e-14, 0, 1.16195530358102e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 2.22044604925031e-14},
    {0, 1.26168012376113, 1.95399252334028e-14, 0, -1.17816950171735e-14, 1.42204860051341, 1.74453095862365e-14, 4.47852016375308, -2.08721928629529e-14, 1.0744670813999e-14, -1.34464719541228e-14, 3.78456620442729e-14, 0, 2.80676208590537e-14, 0, 8.27725128825316, 0, 5.37909280973871, 1.24344978758017e-14, -1.95594693216656e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 19.9654813226339, 0, -2.61992366301004, 1.06581410364015e-14, 0, 0, 0, 0, 1.3455473305103e-14, 0, 1.46585553191507e-14, 0, -1.27103118437955e-14, 0, -2.2308537695773e-14, 0, 0, 1.05662688412451e-14, 2.42300772936765e-14, 0, 0, 0, 1.19298522441569e-14, 2.66453525910038e-14, 1.50990331349021e-14, 0},
    {2.06031501455086, -3.07345773477637e-14, 2.25696201807219, 3.98978286964828, 0, -1.17529718472799e-14, 0.953939201416948, -4.35480723877723e-14, 8.01142041980431, 1.18019368870419, 0, 0, 0, 1.31328347545057e-14, 0, 1.03323004232013e-14, 5.5525489642145, -2.26382415357737e-14, 9.62241374388481, -3.59830837050572, 0, 0, 0, 0, 0, 0, 1.02408289475646e-14, 0, 0, 0, 0, 13.3932520322736, 0, -4.6866619250806, 1.18356086726904, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1.27725097955809e-14, -1.27832588361236e-14, 0, 0, -1.04546715347572e-14, 1.45339591718248e-14, 0, 0, 0, 0, -1.59580006645491e-14, 1.4210854715202e-14},
    {0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {3.93184363124296e-14, 1.09264703862703, 1.28785870856518e-14, 0, 0, 1.22575380004522e-14, -1.14864973879958e-14, 5.38682254607081, 0, 0, 0, -1.44121099608086e-14, 0, -1.75698029368367e-14, -1.80160878442005e-14, 2.57528022630009e-14, 2.69054202848798e-14, 13.4377817958113, 0, 0, 0, 0, 1.06823510383984e-14, 0, 0, 0, 0, 0, 1.89056142369167e-14, 0, -1.31345967659526e-14, -1.51823905359486e-14, 18.907670401189, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, -1.07391608525697e-14, 0, 1.46384409866762e-14, 0, 0, -1.63612658717074e-14, 0, 0, 0, 0, 0, 0, 0},
    {1.78428514239956, -1.14226462513753e-14, 0.814411017927918, 5.06770487035906, 0, 1.13885635970811e-14, 0, 1.47667974881489e-14, 4.01510046524763, 6.58672305740271, 0, 2.01242961116133e-14, 0, -1.5623829602503e-14, 0, 0, 0, -3.27887259670989e-14, 10.0159311874144, 2.83293314500738, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1.60502464055099e-14, 0, 1.06885013954928e-14, 14.0929454377398, -4.55552790213387, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, -2.19027884045748e-14, -1.24344978758018e-14},
    {0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {1.4568627181694, 0, 0, 5.07816454066245, -1.07582259106397e-14, 0, 0, -2.20867693944166e-14, 0, 10.0142755247554, 0, 0, 0, 0, 0, 1.02210862188786e-14, 3.51018877774324e-14, 0, 0, 13.8784813613724, 0, 0, 0, 0, 0, 0, 0, 0, 1.67232492370418e-14, 0, 0, 0, 0, 0, 12.5535587278944, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1.55927253999833e-14, 1.90609040202255e-14, 0, 0, 0, -1.79253699586741e-14, 0, 0, 1.08479262389015e-14, 0, 0, 3.10862446895044e-14},
    {0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0}};
    
    const static double dmats1[56][56] = \
    {{0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {3.16227766016836, 0, 0, 0, 0, 1.08252669405283e-14, 0, 1.02922993704865e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {5.47722557505162, 0, 0, 0, 0, 1.92068165630412e-14, 0, 1.80834223416197e-14, -1.14152204219919e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {2.95803989154978, 5.6124860801609, -1.08012344973465, -0.763762615825971, 0, 2.58464835607474e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, -1.40850097772758e-14, 0, 0, 1.16248598628335e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {2.2912878474779, 7.24568837309471, 4.18330013267037, -0.591607978309959, 0, 2.03703972918374e-14, 0, 0, 0, 1.08801856413265e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1.16401775975232e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {-2.64575131106454, 0, 9.66091783079294, 0.683130051063963, 0, -1.49625671831933e-14, 0, -3.2502596093762e-14, 0, 0, 0, 1.34670915888159e-14, 0, 0, 0, 2.90440444432224e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {1.87082869338697, 1.42804031130623e-14, 0, 4.34741302385683, 0, 0, 0, 2.04003480774873e-14, -1.14092481911365e-14, -1.06581410364015e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {3.24037034920393, 2.38080669966159e-14, 0, 7.52994023880667, 0, 1.13478631598507e-14, 0, 3.45590001304086e-14, -1.69864122767649e-14, -1.4210854715202e-14, 0, 0, 0, 0, 0, -1.32400798627181e-14, 0, 1.22473609554424e-14, 1.33269883629769e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {2.74954541697351, 5.79655069847575, -1.67332005306816, -1.18321595661991, 7.74596669241483, -1.19999999999998, 0.346410161513775, -0.979795897113269, 0.282842712474611, 0.200000000000014, -1.0696429695597e-14, -1.26705171178379e-14, -1.2948158065601e-14, 0, 0, -1.45874734860756e-14, 0, -1.16672151462161e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, -1.07102102118129e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {2.32379000772449, 2.44948974278316, 2.82842712474617, -0.999999999999999, 9.16515138991168, 7.09929573971955, -2.04939015319193, -0.414039335605429, -0.478091443733758, 0.169030850945723, 0, -1.28646947787458e-14, -1.8460920120059e-14, -1.81736908030475e-14, 0, 1.37951452580958e-14, -1.77890663211312e-14, 0, 0, 0, 0, 1.08892803896054e-14, 0, 0, 0, 0, 1.05640513109647e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, -1.22044403129704e-14, 0, 0, 0, 2.27334526137921e-14, -1.76454983335969e-14, -1.05574887489127e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {1.80000000000004, -5.69209978830308, 4.38178046004131, -0.774596669241497, 1.24189930203385e-14, 10.998181667894, 4.76235235991625, 0.962140470884722, -0.740656079818038, 0.130930734141603, 0, -1.20430917197879e-14, -1.1981685754404e-14, -2.88797787541631e-14, -1.3779450272627e-14, 0, 0, 0, -2.22505656218757e-14, -1.14908083048704e-14, 0, 0, 0, 0, 0, 0, 0, 0, 1.22673594366302e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, -1.12284484669662e-14, 0, 0, 0, 0, 0, 0, 0, 0, -1.1387566167118e-14, 0, 1.48982410846523e-14, 0, 0, 0, 0},
    {5.19615242270664, 0, -3.16227766016842, -2.23606797749978, 0, 0, 13.7477270848675, 1.78325020862166e-14, 0.534522483824845, 0.377964473009226, 0, -2.80136022320966e-14, -1.36437984192736e-14, -2.9552592280819e-14, 1.3605842011612e-14, -4.49783579557011e-14, 2.03714412277317e-14, -1.61597054173618e-14, -2.47182474765566e-14, 0, 0, 0, 0, 0, 0, 0, 0, 1.7014378114654e-14, 0, 0, 0, -1.14827159581245e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, -1.57668143745762e-14, 2.14083883902157e-14, -2.68877125810315e-14, 0, 0, 0, -1.36900091929644e-14, -2.0687516582275e-14, 0, 0, 0, 0, 0, 0},
    {2.01246117974979, 2.12132034355965, -0.408248290463867, 3.17542648054294, 0, 1.00616573615107e-14, 0, 7.17137165600639, -1.38013111868472, -1.56144011671766, 0, 0, 1.10848947143279e-14, 0, 0, -1.41970084791766e-14, -1.32242728950118e-14, 1.59365405521834e-14, 0, -1.4210854715202e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, -1.214689797411e-14, 0, 0, 1.24931114721564e-14, 0, 0, 0, 0, 0, 0, 0},
    {1.55884572681198, 2.73861278752583, 1.58113883008418, 2.45967477524978, 0, 0, 0, 9.25820099772551, 5.34522483824847, -1.20948631362952, 0, 0, 1.84632408273169e-14, 0, 0, 0, -1.50085060064379e-14, 1.49723986219342e-14, 0, -1.33226762955019e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, -1.46277209591922e-14, 0, 0, 2.16306109608827e-14, 0, 0, 0, 1.38795388762322e-14, 0, 0, 0},
    {-1.79999999999998, -2.72689985024876e-14, 3.65148371670112, -2.84018778721877, 0, -2.17150305878754e-14, 0, -5.90005603264198e-14, 12.3442679969674, 1.39659449751037, 0, 0, 0, 0, 0, 2.68747109317333e-14, 0, 0, -1.90845800418516e-14, 0, 0, 0, 0, 0, 0, -1.23984917003451e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1.36024520914242e-14, 0, 0, 0, 0, 0, 1.0365125502094e-14, 0, 0, 1.00866023900427e-14, -1.51206012498905e-14, 0},
    {1.27279220613579, 0, 0, 3.83405790253616, 0, 0, 0, 1.00217788352562e-14, 0, 5.18459255872629, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {2.20454076850487, 0, 0, 6.6407830863536, 0, 0, 0, 1.6954568423569e-14, -1.04207941733572e-14, 8.97997772825745, 0, 0, 0, 0, 0, 0, -1.07399115598352e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, -1.06281916397066e-14, 0, 1.0771947239792e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {2.56904651573302, 5.80288457473997, -2.01017818278147, -1.42141062443802, 8.14215222512715, -2.10229466468021, 0.728256234341424, -1.71651640581399, 0.594618725379061, 0.420458932936053, 9.8107084351743, -1.18450885369836, 0.30583887092058, -0.105945692672794, -1.02581475831037, 0.26486423168198, -0.0917516612761728, 0.216260739578395, -0.0749149177264333, -0.0529728463363882, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {2.26568606239551, 3.07059789431495, 1.77281052085584, -1.25356634105602, 2.39356776939086, 6.3037687140313, -2.78313372537745, -0.908295106229251, -0.524404424085067, 0.370809924354793, 11.1242977306435, 9.40174755792013, -2.42752078113808, 0.840917865872078, -0.301561193523236, -0.794200206656967, 0.350641890608821, 0.114434427054281, 0.0660687472643356, -0.0467176592151211, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1.01015959200066e-14, 0, 0, 0, 0, 0, 0, 0, 1.84814770714563e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {1.91485421551268, 1.29756443747034, 3.74574588632198, -1.05945692672795, -8.09173593712686, 5.01426536422406, 1.80936716113936, -0.383824736789375, -1.10800657553489, 0.313391585263996, 0, 12.7134831207354, 8.20651806648289, -2.84282124887607, 1.01946290306867, -0.631738055305808, -0.227958835180056, 0.0483573714561744, 0.139595707137592, -0.039483628456617, 0, 0, 0, 0, 0, 0, 0, 1.2257456641157e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, -1.29080203530852e-14, 1.16595669138782e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {1.48323969741914, 7.03562363973513, 4.062019202318, -0.820651806648271, 0, -6.79705818718658, 5.88642506110458, -2.08116553882674, -1.20156148406981, 0.242752078113807, 0, 0, 14.8323969741913, 5.13809303146607, 1.63714487212955e-14, 0.856348838577624, -0.741619848709526, 0.262202212042521, 0.151382517704875, -0.030583887092048, 0, 0, 0, 0, 0, 0, -1.18952346076914e-14, 1.30853433842222e-14, -1.16925429925565e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, -1.88029658615293e-14, 1.91748164115319e-14, -1.45977163230575e-14, 1.09655150319724e-14, 0, 0, -1.89024735595586e-14, 0, 0, 0, 0, 0, 0, 0},
    {-3.42539535431071, 0, 9.38083151964686, 1.89521416591735, 0, 0, -3.39852909359329, 0, -2.77488738510232, -0.560611910581379, 0, 0, 0, 17.7988763690296, 0, -1.94078776694002e-14, 0.42817441928881, 0, 0.349602949390047, 0.0706304617818554, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, -1.20061774357534e-14, 0, 0, 0, 0, 0, -1.00079856173773e-14, 0, -2.26674794489762e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {2.02649122047609, 2.74642624930236, -0.792824967172097, 2.24244764232558, 2.14087209644418, -0.331662479035527, 0.0957427107756351, 6.49923072370878, -1.87616630392938, -2.15580611373099, 0, 0, 0, 0, 9.44035859331468, -1.46249406456536, 0.422185670932516, -2.04706526287666, 0.590936840285292, 0.52231930877335, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1.66277980866635e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1.68132514445692e-14, 0, 0, -1.30320642762071e-14, 1.5317530203921e-14, 0},
    {1.71269767715538, 1.16057691494799, 1.34011878852097, 1.89521416591739, 2.5331140255951, 1.96214168703485, -0.566421515598877, 2.74642624930236, 3.17129986868837, -1.82198870938948, 0, 0, 1.26698604015865e-14, 0, 11.1699829238239, 8.6522315682783, -2.49768411251824, -0.86504295831355, -0.998865569685873, 0.441440386136653, 0, 0, 0, 0, 0, 0, 1.98384517908788e-14, 0, 0, 1.09458095686894e-14, 0, 1.6523812455151e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, -1.03226253428986e-14, 3.45055971392177e-14, -1.0963026379453e-14, 0, 0, 1.98669435916771e-14, 0, 2.24143595655932e-14, 1.21246267628695e-14, 0, 1.40825657142122e-14, 0, 0},
    {1.32664991614221, -2.69693703815181, 2.07610309995257, 1.46802658041887, 0, 3.03973683071411, 1.31624465810881, -6.38211787508104, 4.91295663090393, -1.4113063856887, 0, 0, 0, 0, -1.12046668488404e-14, 13.4039795085888, 5.80409338312195, 2.01017818278149, -1.54743588659735, 0.341938252770126, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1.99188479486906e-14, 0, 0, 0, 0, 0, 0, 1.9439663850877e-14, 0, 2.04972151503089e-14, -2.09775353099935e-14, 0},
    {3.82970843102538, -1.35375539472083e-14, -1.49829835452878, 4.23782770691181, 0, 0, 3.79967103839267, 1.23996592074953e-14, -3.54562104171168, -4.07409060843206, 0, 0, -1.7600753095073e-14, -1.0822450122319e-14, 1.7784123181026e-14, -1.50271421719475e-14, 16.7549743857359, -2.83790269242822e-14, 1.11676565710081, 0.987090711415297, 0, 0, 0, 0, 0, 1.14492917376685e-14, -1.371929590441e-14, 2.1952750214123e-14, 0, 0, 1.18235696338831e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, -1.24727047960437e-14, 2.54018340570979e-14, -1.39065024647839e-14, 0, 0, 0, -2.41394818807004e-14, 0, 0, 0, 0, 0, 0, 0},
    {1.48323969741916, 1.00508909139075, -0.193429485824665, 3.55615782880926, 0, -1.09316735690911e-14, 0, 4.45964044034304, -0.85825820290701, 2.67027285925186, 0, 0, 0, 0, 0, 0, 0, 8.42792824422444, -1.62195554683794, -2.29379153190449, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {1.14891252930764, 1.29756443747036, 0.749149177264399, 2.75458800949268, 0, -1.56850849701631e-14, 0, 5.75737105184055, 3.32401972660469, 2.06838446274243, 0, 0, 0, 1.55284895654593e-14, 0, 0, -1.28699742148281e-14, 10.8804085776374, 6.28180682119208, -1.77676328054755, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, -1.17679879087453e-14, 0, 0, 0, 0, 0, 0, 0, 0, 1.29340213107974e-14, 0, 0, 0, 0},
    {-1.32664991614215, -2.47030321919216e-14, 1.73008591662716, -3.18072425757422, 0, 0, 1.13119318909818e-14, -3.04199753195453e-14, 7.6764947357874, -2.38836465270395, 0, 0, 0, 2.05269241282193e-14, 0, -1.45662197301565e-14, -1.85676576002613e-14, -2.8126806197496e-14, 14.5072114368499, 2.05162951662072, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, -1.09052206632751e-14, 0, 0, 0, 0, 0, 0, 0, 0, -1.073521706443e-14, 0, 0, 0, 0, 0, 0, 0, 1.02157480437484e-14, 0, 0, 0, 0, 1.06581410364015e-14},
    {0.938083151964688, -1.67075414421403e-14, 0, 3.11415464992886, 0, 0, 0, 0, 0, 5.52707620976693, 0, 0, 0, 0, 0, 0, 0, -1.44579473224948e-14, 0, 5.80288457473998, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {1.6248076809272, -2.71174079765265e-14, 0, 5.39387407630366, 0, 0, 0, -1.76761006483537e-14, 0, 9.57317681262153, 0, 0, 0, 0, 0, 1.62541936715167e-14, 1.29794840192237e-14, -2.09070339378817e-14, 0, 10.0508909139074, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1.07896695788578e-14, 1.01165706770544e-14, 0, 1.24344978758018e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1.24344978758018e-14},
    {0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {2.41593350361252, 5.72988941019469, -2.20543546226845, -1.55947837083931, 8.33749895885385, -2.7677995385711, 1.06532653910463, -2.25989885993664, 0.869835476750505, 0.615066564126907, 10.333118277332, -2.18326971917503, 0.724780051083774, -0.278967971730974, -1.8907670401189, 0.627677936394735, -0.241593350361259, 0.512496888990078, -0.197260144544833, -0.139483985865487, 11.8490505948789, -1.16109782023229, 0.245326690731328, -0.0814411017928031, 0.0313466946954498, -1.03851746182649, 0.219426862868121, -0.0728431359084709, 0.0280373360835783, 0.190029237516519, -0.063084006188059, 0.024281045302823, -0.0515078753637739, 0.019825390471097, 0.0140186680417963, 0, 0, 1.00108896213852e-14, 0, 0, 0, 0, 0, 0, 0, 0, 1.21150877964365e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {2.18529407725402, 3.45525332069937, 0.997445717412081, -1.41060126129511, 3.23209352764251, 5.28531208250976, -2.89087233497829, -1.36277028773851, -0.393397896234712, 0.556348640264183, 2.3366642891096, 9.02784807228871, -3.93352933905729, 1.64018416088948, -0.73296991613515, -1.19859611757306, 0.655588223176202, 0.309047252182647, 0.0892142571199726, -0.12616801237611, 13.0996183150502, 11.55277744383, -2.44096975264614, 0.810328731479527, -0.311895674167859, -0.234843597212102, -0.907332870306517, 0.395334573318419, -0.164844710568308, 0.0736662483163133, 0.120463442338377, -0.0658890955530689, -0.031060417514551, -0.00896637020659282, 0.012680362351413, 0, 2.00524187044473e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1.35703519068716e-14, 0, 0, 0, 0, 0, 0, 0, -1.01417984314538e-14, 0},
    {1.92724822331887, 1.82834820066011, 2.99085908983326, -1.24403337882031, 0.950146187582608, 5.74064456311309, -0.169967317119755, -0.721110255092798, -1.1796107330308, 0.490653381462651, -10.3037210107158, 4.97613351528121, 5.78174466995658, -3.22682211353229, -0.215472901842825, -1.30185581821821, 0.0385449644663842, 0.163532434528608, 0.267510569439612, -0.111269728052846, 0, 14.555131461167, 10.7636648722209, -3.57321383950879, 1.37533064807499, 1.03556292536504, -0.500120235663637, -0.581087203147965, 0.324307824721464, 0.0216558414515625, 0.13084143273245, -0.00387391468765269, -0.0164356280722631, -0.0268858238268406, 0.0111830284386733, 0, 0, 0, 0, 0, 0, 0, -1.24894083304515e-14, 0, 0, 0, 0, 0, -1.79685973736065e-14, 0, 0, 0, 0, 0, 0, 0},
    {1.62882203585591, 0.77261813045657, 3.86595114186567, -1.05140010313427, 9.1544290607646, 3.35889012281481, 3.87851223317102, -0.304724700110015, -1.52475169285617, 0.414677792940093, -1.50838999194959e-14, -9.46260092820841, 7.3296991613516, 0.564240504518021, -2.07602937339133, -0.761724681099385, -0.879563899362177, 0.0691050664139764, 0.345781182036188, -0.0940400840863524, 0, 0, 16.3745228938128, 9.05975063854702, -3.48709964663735, 0, 0.951027176355628, -0.736662483163206, -0.0567083043941531, 0.208648802584714, 0.076556210932118, 0.0883994979795754, -0.00694532049720325, -0.0347523163751444, 0.00945138406569365, 0, -1.34204874150947e-14, 0, 0, 0, 0, 0, 0, 0, -2.42374492692268e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {1.26168012376109, -6.78263087840205, 3.68560354207928, -0.814411017927976, 0, 10.0216907612581, 6.00856531485322, 2.67510569439611, -1.4536216402544, 0.321208037219824, 0, 0, -7.57008074256671, 6.99294104721291, 0, -2.27270584034422, -1.36261453366206, -0.606656948415834, 0.329650402328115, -0.0728431359084831, 0, 0, 0, 18.7137404500718, 5.40219154319688, 0, 0, 0.760821741084502, -0.702817019232737, 0, 0.228415531250642, 0.136947913395211, 0.0609713174047674, -0.033131112015512, 0.00732101061700773, 0, 0, 0, 0, 0, -1.30324950839724e-14, 0, 0, 0, -1.36428396125789e-14, -2.70392224640622e-14, 0, 0, 0, -1.78921270332478e-14, 0, 0, 0, 0, 0, 0},
    {5.09901951359278, 1.47405589080242e-14, -4.65474668125635, -3.29140294302192, 0, -1.75121952403549e-14, 13.490737563232, -1.88659898491929e-14, 1.83585684909538, 1.29814682728313, 1.74929961328223e-14, 0, 3.18637490243895e-14, -3.53270434653118, 0, 2.70955022774169e-14, -3.05941170815572, 0, -0.416333199893208, -0.294392028877594, 0, 0, -1.00417365518154e-14, -2.77992935601067e-14, 21.8326971917504, 0, 0, 0, 0.355050145839428, 0, 0, 0.307482445914326, 0, 0.0418430609640593, 0.0295875121532926, 0, 0, 1.53618763757974e-14, -1.4536552951721e-14, 0, -2.34517773777192e-14, 0, 0, 0, 1.75720087957737e-14, -2.91662766570223e-14, 0, 1.76059530106093e-14, 0, 0, 0, 0, 1.07470318048245e-14, 0, 0, 0},
    {1.99489143482416, 3.15420030940281, -1.09264703862702, 1.54523626091316, 2.95048422176041, -0.761811750275054, 0.263899331455877, 5.59815020469135, -1.93925611658553, -2.28543525082516, 2.13307290077015, -0.257539376818855, 0.0664963811607952, -0.0230350221380054, 9.14446135479766, -2.36108976914457, 0.817905488277913, -3.33842298506509, 1.1564636454577, 1.02505848514083, 0, 0, 0, 0, 0, 11.5766222110848, -1.39771878815471, 0.3608894392803, -0.125015768949695, -2.15192884958005, 0.555625639776457, -0.192474367616166, 0.595437196138648, -0.206265495285699, -0.166687691932932, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1.62916528370584e-14, 1.08185730701625e-14, 0, 0, 0, -1.29677564833166e-14, 0, 0, 0, 0, 0, 0},
    {1.75932887637252, 1.66904592079257, 0.963624111659422, 1.36277028773852, 0.867360833110887, 2.28430636201791, -1.00852844760554, 2.96226264871973, 1.71026313764871, -2.01556443707464, 2.41867732448957, 2.04415543160773, -0.527798662911753, 0.182834820065996, 2.68821895760395, 7.07977053255619, -3.12574097035512, -1.76652739792481, -1.01990506872276, 0.904016610326408, 0, 0, 0, 0, 0, 13.1266557397187, 11.0940489487798, -2.8644711213756, 0.992281903807275, -0.632607619454911, -1.66605356688629, 0.735567892896454, 0.315075748468089, 0.18190906819317, -0.147004726489971, 0, 0, 0, 0, 0, 0, 0, 0, 0, 3.50533057667082e-14, 0, 0, 0, 0, 0, 1.4911169345421e-14, 0, 0, 0, -1.04402756430621e-14, 0},
    {1.48690428533297, 0.70530063064754, 2.03602754481989, 1.1517511068998, -2.93221479395414, 1.81702705031798, 0.655663160391202, 1.2517844405944, 3.61359041872289, -1.70346285967313, 0, 2.76420265655949, 1.78428514239953, -0.618094504365263, -9.08783875864323, 5.63152770643898, 2.03210252328839, -0.746494073225332, -2.15494277062554, 0.764033484562606, 0, 0, 0, 0, 0, 0, 15.0018922739643, 9.68367982311024, -3.35452509157131, 2.13860408462396, -1.32524448062077, -0.478206411017787, 0.133143804689794, 0.384353057392899, -0.124241670058204, 0, 0, 0, 0, 0, 0, 0, -1.2184958691979e-14, 4.08565188434445e-14, 2.21302457056835e-14, 0, 0, 0, -1.0903058801995e-14, 0, 0, 0, 0, 0, 0, 0},
    {1.15175110689978, 3.82426463519456, 2.20794021658196, 0.892142571199804, 0, -2.46306042692148, 2.13307290077015, 6.78739640804928, 3.91870514328392, -1.31949665727937, 0, 0, 3.22490309931938, 1.11713920350151, 2.5966242421383e-14, -7.6337845572604, 6.61105135360486, -4.04762275910199, -2.33689608954557, 0.591817792331191, 0, 0, 0, 0, 0, 0, -2.33944627292341e-14, 17.5022076529583, 6.06294257990891, 0, 1.79642743108414, -1.55575179137408, 0.721929233499939, 0.416806037297049, -0.0962371838080711, 0, 0, 0, 0, 0, 0, 0, -1.06500367370775e-14, 2.55596393426003e-14, -1.78278849535737e-14, 1.24660473149391e-14, 0, 0, -3.33471800635493e-14, 0, 0, 0, 0, -2.74739380889122e-14, 0, 0},
    {-2.65985524643224, 3.04548590059106e-14, 5.09901951359276, -2.06031501455087, 0, 1.27314974860562e-14, -1.23153021346075, 4.54600263645348e-14, 9.04986187739903, 3.04724700110024, 0, 0, 1.37797484823791e-14, 3.8698837191833, 0, -2.28548550248364e-14, -3.81689227863023, 1.32945293178207e-14, -5.39683034546927, -1.36674464685441, 0, 0, 0, 1.36272283216478e-14, 1.00072372011083e-14, 0, 0, -1.24367890909937e-14, 21.00264918355, 0, 0, 0.898213715542077, 0, 0.962572311333257, 0.22225025591058, 0, 0, 0, 0, 0, 0, 0, 1.43237887055327e-14, -2.25958210828995e-14, -2.54351954443607e-14, -4.42292868735092e-14, 0, 0, 1.18670146344888e-14, 0, 0, 0, 0, 0, -2.23325753829243e-14, 0},
    {1.57359158493888, 1.49284005458436, -0.43094580368567, 3.04724700110021, 0.775791113542718, -0.120185042515478, 0.0346944333244453, 5.29905651979672, -1.52970585407783, 0.901387818866015, 0, 0, 1.54345206564601e-14, 0, 4.51561078268936, -0.69955541437241, 0.201944253400477, 6.36462844462229, -1.83730997289732, -2.74825341360596, 0, 0, 0, 0, 0, 0, 0, 0, 0, 10.8920630294282, -1.68739114875268, 0.487107866980291, -3.0999354831996, 0.894874292847886, 0.903959543974661, 0, 0, 0, 0, 0, 0, 0, 0, -1.35315801444742e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {1.32992762321608, 0.630840061880569, 0.728431359084703, 2.57539376818858, 0.917928424547677, 0.711024300256696, -0.205255035576777, 2.23926008187653, 2.58567482211402, 0.761811750275079, 0, 1.50972141321262e-14, 1.92432056571699e-14, 2.94270416460811e-14, 5.34294273196304, 4.13862564412658, -1.1947183148558, 2.68954640486935, 3.10562068169861, -2.32269806558124, 0, 0, 0, 0, 0, 0, 0, 0, 0, 12.8876627769294, 9.98274066131697, -2.88176900403077, -1.30996183150502, -1.51261363209512, 0.763985254692642, 0, 0, 0, 0, 0, 1.07852658462479e-14, 0, -1.13464763436474e-14, -1.56492117088446e-14, -1.2428153133211e-14, 0, 0, 0, 1.57186842569255e-14, 1.23293837842049e-14, 0, 0, 0, 1.39814305552841e-14, 0, 1.99840144432528e-14},
    {1.03015750727542, -1.46593983227033, 1.12848100903611, 1.99489143482414, 0, 1.10151410945721, 0.476969600708479, -5.20357020296097, 4.00571020990215, 0.590096844352093, 0, 1.61268443608728e-14, 0, 2.17744761210991e-14, 0, 6.41153127835569, 2.77627448210725, -6.24994106094659, 4.81120687194241, -1.79915418525286, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 15.4651953323153, 6.69662601613678, 3.04407621455179, -2.3433309625403, 0.59178043363452, 0, 0, 0, 1.05482318770498e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {2.9738085706659, -2.31628665433557e-14, -0.814411017927956, 5.75875553449895, 0, 0, 1.37689263682152, -1.87511781470446e-14, -2.89087233497829, 1.70346285967313, 0, 1.53767244373726e-14, -2.60164507598573e-14, -3.0606105648312e-14, 0, 3.91555026057662e-14, 8.01441409794458, 0, -3.47218947830366, -5.19371076584692, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 19.3314941653941, 0, 1.69115345252876, 1.70832296330022, 0, 0, 0, 0, 0, -1.47710207017114e-14, 0, 0, 0, 2.39374069194048e-14, 0, -1.16737795196999e-14, 0, -2.35947835773664e-14, 0, 0, 0, -2.14656473740582e-14, 0, 0, 0},
    {1.15175110689981, 0.546323519313508, -0.105140010313419, 3.27118942773249, 0, 0, 0, 2.69341127303541, -0.518347241175128, 4.25171145123352, 0, 0, 0, 0, 0, 1.89132326876168e-14, 1.07703482219709e-14, 6.71889089790563, -1.29305115618717, 1.82865048192212, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 9.45383520059452, -1.81939143264587, -2.94058061635795, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0.892142571199805, 0.705300630647555, 0.40720550896397, 2.53385243517954, 0, 1.39813392926345e-14, 0, 3.47717900165116, 2.00755023262382, 3.29336152870135, 0, 0, 0, -1.4230588162391e-14, 0, 2.12522836138748e-14, 1.52618648424287e-14, 8.67405085085769, 5.00796559370718, 1.4164665725037, 0, 0, 0, 0, 0, 0, 0, 0, 1.13147703328239e-14, 0, 0, 0, 12.2048487632307, 7.0464727188699, -2.27776395106694, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1.14999053942145e-14, 0, 0},
    {-1.03015750727539, 0, 0.940400840863377, -2.92584077107543, 0, 0, 0, 1.11293406173924e-14, 4.63623866886823, -3.80284633026899, 0, 0, 0, -2.94156668790987e-14, 0, 0, 0, -1.33922800433549e-14, 11.5654011344769, -1.63559471386628, 0, 0, 0, 0, 0, 0, 0, 0, 1.04604926852949e-14, 0, 0, -1.05559372926886e-14, 0, 16.2731316843076, 2.63013526059784, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, -1.36105120406189e-14, 0, 0, -1.64162100411904e-14, 0, 0, -1.43624641143248e-14, 0, 0, -1.06581410364015e-14},
    {0.728431359084702, 0, 0, 2.53908227033122, 0, 0, 0, -1.12654293021334e-14, 0, 5.00713776237769, 0, 0, 0, 0, 0, 0, 1.75509438887162e-14, 0, 0, 6.9392406806862, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 6.27677936394721, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1.55431223447522e-14},
    {1.26168012376115, 0, 0, 4.39781949681102, 0, 0, 0, -1.94061175819238e-14, 0, 8.6726170049349, 0, 0, 0, 0, 0, 1.01179304274323e-14, 2.87309305235289e-14, 0, -1.04523229052332e-14, 12.0191174248973, 0, 0, 0, 0, 0, 0, 0, 0, 1.49130416320287e-14, 0, 0, 0, 0, 0, 10.8717007662564, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1.39042309332655e-14, 1.68148173406132e-14, 0, 0, 0, -1.39259589347659e-14, 0, 0, 0, 0, 0, 2.57571741713036e-14},
    {0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0}};
    
    const static double dmats2[56][56] = \
    {{0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {3.16227766016836, 0, 0, 0, 0, 1.11583338479159e-14, 0, 1.02922993704865e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {1.82574185835054, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {5.16397779494319, 0, 0, 0, 0, 1.73479189776187e-14, 0, 1.65773933526039e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {2.95803989154978, 5.6124860801609, -1.08012344973464, -0.763762615825969, 0, 2.58464835607474e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, -1.45290989871259e-14, 0, 0, 1.07366814431334e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {2.29128784747791, 1.44913767461894, 4.18330013267037, -0.591607978309966, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1.02908502069211e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {1.32287565553229, 0, 3.86436713231718, -0.341565025531991, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {1.87082869338696, 7.09929573971954, 0, 4.34741302385683, 0, 2.31412979990705e-14, 0, 1.79578574233119e-14, -1.09651589812864e-14, 0, 0, 0, 1.0275456662804e-14, 1.11253951863901e-14, 0, 0, -1.23516279655211e-14, 0, 1.67772945370309e-14, -1.06581410364015e-14, 0, 0, 0, 0, 0, 0, 0, 0, -1.04810894589995e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, -1.45492836924348e-14, 0, 0, 0, 0, 0, 0, 0, -1.07986153011749e-14, 0, 0},
    {1.08012344973466, 0, 7.09929573971953, 2.50998007960221, 0, 0, 0, 0, 0, 0, 0, 1.10981825134126e-14, 0, 0, 0, 1.49126177831961e-14, 0, 0, -1.09527631538099e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1.11294528477952e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {-3.81881307912981, 2.63805095952449e-14, 0, 8.87411967464941, 0, -1.08587953265362e-14, 0, 1.60284200435428e-14, 0, -2.30926389122033e-14, 0, 0, 2.0594465710441e-14, 1.14859138885232e-14, 1.07233040380981e-14, 0, -1.40594294599945e-14, 0, 0, -2.1316282072803e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, -1.15463194561016e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {2.74954541697351, 5.79655069847576, -1.67332005306816, -1.18321595661991, 7.74596669241483, -1.19999999999999, 0.346410161513776, -0.979795897113272, 0.282842712474611, 0.200000000000011, 0, -1.42248293523131e-14, -1.16991571628977e-14, 0, 0, -1.34772504614504e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, -1.05991879093504e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {2.32379000772445, 2.44948974278317, 2.82842712474618, -1.00000000000001, 1.30930734141596, 7.09929573971955, -2.04939015319193, -0.414039335605422, -0.478091443733756, 0.169030850945715, 0, 0, -1.38137027369175e-14, 0, -1.23135650746216e-14, 1.4971941277217e-14, -1.68198040008164e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, -1.41869149998573e-14, 1.99142275743244e-14, -1.52384110702395e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {1.80000000000002, 0.632455532033669, 4.38178046004131, -0.774596669241481, 0, 3.14233761939829, 4.76235235991625, -0.106904496764978, -0.740656079818043, 0.130930734141599, 0, 0, 0, -1.70268155420246e-14, 0, 0, 0, 0, -1.65329170450561e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {1.03923048454135, 0, 3.16227766016836, -0.447213595499958, 0, 0, 5.89188303637179, 0, -0.53452248382485, 0.0755928946018462, 0, 0, 0, -1.1578427571356e-14, 0, 0, 0, 0, -1.31656139489963e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {2.01246117974984, 2.12132034355964, -0.40824829046387, 3.17542648054295, 9.07114735222145, 0, 0, 7.17137165600638, -1.38013111868473, -1.56144011671764, -1.28131039656276e-14, -1.05722626181333e-14, 0, 0, 0, -1.43560570277939e-14, -2.27050760178409e-14, 0, 0, 0, 0, 0, 0, 0, 0, 1.20645452546378e-14, 0, 0, -1.39975315007857e-14, 0, 1.33944298105448e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, -1.79370968861081e-14, 0, 0, 0, 0, 0, 0, 0, 0, 1.10225378253926e-14, 0},
    {1.558845726812, 0.547722557505167, 1.58113883008418, 2.45967477524977, 0, 9.07114735222146, -1.85760141295822e-14, 1.85164019954509, 5.34522483824848, -1.20948631362952, 0, -1.79269712993067e-14, 0, 0, -1.21834179081657e-14, 0, -1.08231757343816e-14, 1.58605770416344e-14, 0, -1.77635683940025e-14, 0, 0, 0, 0, 0, 0, 1.20070279998216e-14, 0, 0, 0, 0, 0, 0, 0, -1.11022302462516e-14, 0, 0, 0, 0, 0, 0, 0, 0, -1.5994754107584e-14, 1.5836245903059e-14, -1.59336158038543e-14, 0, 0, 1.24090681928578e-14, 0, 0, 0, 0, 0, 0, 0},
    {0.900000000000037, 0, 1.46059348668045, 1.42009389360939, 0, -1.21399527118824e-14, 9.07114735222146, 0, 4.93770719878694, -0.698297248755176, 0, -1.69782446707204e-14, 0, 0, 1.38222655524042e-14, -1.41725217571765e-14, 0, 0, -1.92181403448241e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1.52483803956785e-14, -1.04387107607342e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {1.27279220613579, -6.2609903369994, 0, 3.83405790253616, 0, -2.66966858276427e-14, 0, 10.5830052442584, -1.29525898283173e-14, 5.18459255872629, 0, 1.15955623710069e-14, 2.12136539532159e-14, 0, 0, 0, -1.1187497199729e-14, 0, 0, -2.1316282072803e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1.1791882435372e-14, 1.34567031689284e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1.11941676856894e-14, 0, 0, 1.28246579327207e-14, 0, 0, 0, 0},
    {0.734846922834948, 0, -6.26099033699939, 2.21359436211787, 0, 0, 0, 0, 10.5830052442584, 2.99332590941916, 0, -1.26338784464357e-14, 1.51937191134705e-14, 2.01241989119956e-14, 0, 0, -1.07773281838078e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {5.71576766497725, -3.5255606739263e-14, -2.0846313271379e-14, -4.69574275274955, 0, 0, 1.14621342066822e-14, -1.32075361813795e-14, 0, 12.6996062931101, 0, 1.1579866837032e-14, -1.17571210377054e-14, 0, 0, 1.11781775592062e-14, 0, -2.03916735577721e-14, 0, 1.24344978758018e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1.77352530249445e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1.34841270690221e-14, 0, 0, 0, 0, 0, 1.33226762955019e-14},
    {2.56904651573302, 5.80288457473997, -2.01017818278147, -1.42141062443802, 8.14215222512715, -2.10229466468021, 0.728256234341424, -1.71651640581399, 0.59461872537906, 0.420458932936052, 9.8107084351743, -1.18450885369835, 0.305838870920579, -0.105945692672794, -1.02581475831037, 0.264864231681979, -0.0917516612761747, 0.216260739578394, -0.0749149177264344, -0.0529728463363901, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {2.26568606239552, 3.07059789431494, 1.77281052085583, -1.25356634105602, 2.39356776939085, 6.30376871403133, -2.78313372537745, -0.908295106229254, -0.524404424085065, 0.370809924354795, 1.23603308118261, 9.40174755792012, -2.42752078113806, 0.84091786587207, -0.301561193523234, -0.794200206656967, 0.350641890608815, 0.114434427054271, 0.0660687472643394, -0.0467176592151202, 0, 0, 0, 0, 1.20766335966975e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, -1.26955258141599e-14, 0, 0, 0, 0, 0, 1.65501530620379e-14, -1.82242579771102e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {1.91485421551271, 1.29756443747034, 3.74574588632197, -1.05945692672795, 0.404586796856347, 5.01426536422407, 1.80936716113936, -0.383824736789378, -1.1080065755349, 0.313391585264007, 0, 2.82521847127454, 8.20651806648288, -2.84282124887606, -0.0509731451534317, -0.631738055305793, -0.227958835180061, 0.0483573714561631, 0.139595707137598, -0.0394836284566152, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {1.48323969741914, 0.335029697130245, 4.06201920231797, -0.820651806648286, 0, 1.69926454679664, 5.88642506110457, -0.0991031208965069, -1.20156148406979, 0.24275207811381, 0, 0, 4.94413232473043, 5.13809303146604, 0, -0.214087209644449, -0.741619848709543, 0.0124858196210661, 0.151382517704864, -0.0305838870920592, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, -1.236469996079e-14, 0, -1.11803154979493e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0.85634883857767, 0, 2.68023757704195, -0.473803541479343, 0, 0, 5.09779364038993, 1.03628894560728e-14, -0.792824967172095, 0.140152977645353, 0, 0, 0, 7.9106117195687, 0, -3.2024651826757e-14, -0.642261628933259, 0, 0.0998865569685812, -0.0176576154454647, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, -1.41874267782629e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {2.02649122047608, 2.74642624930236, -0.79282496717209, 2.24244764232558, 2.14087209644418, -0.331662479035553, 0.0957427107756432, 6.49923072370878, -1.87616630392938, -2.15580611373099, 11.0554159678513, 0, 0, 0, 9.44035859331468, -1.46249406456537, 0.42218567093253, -2.04706526287665, 0.590936840285286, 0.522319308773345, 0, 0, 0, 0, 0, 0, 0, 1.07417485753862e-14, 0, 0, 1.36630425513329e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, -1.23701834881004e-14, 0, 1.02417490692638e-14, 0, 0, 0, 0, 0, 1.80291351901461e-14, 0, 0, -1.01316066243738e-14, 0, 0},
    {1.71269767715536, 1.16057691494801, 1.34011878852098, 1.89521416591738, 0.361873432227874, 1.96214168703485, -0.56642151559888, 2.74642624930238, 3.17129986868838, -1.82198870938951, 0, 11.0554159678513, 1.45760542423516e-14, 1.47166563795188e-14, 1.59571184626056, 8.65223156827826, -2.49768411251825, -0.865042958313534, -0.998865569685866, 0.44144038613664, 0, 0, 0, 0, 1.29892244953406e-14, 0, 1.19356062430186e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 2.55080325568313e-14, -1.99643912911818e-14, 0, 0, 2.52250943240554e-14, 0, 0, 0, 0, 0, 0, 0},
    {1.32664991614219, 0.299659670905743, 2.07610309995258, 1.46802658041888, 0, 0.868496237346862, 1.31624465810882, 0.709124208342315, 4.91295663090392, -1.4113063856887, 0, 1.04153151561804e-14, 11.0554159678513, 0, 0, 3.82970843102535, 5.80409338312197, -0.223353131420174, -1.54743588659735, 0.341938252770125, 0, 0, 0, -1.22971310029744e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, -1.3965710429727e-14, 1.20613126934675e-14, 1.38979281389624e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, -1.63145986065679e-14, 0},
    {0.765941686205059, -1.0369529147852e-14, 1.49829835452879, 0.847565541382363, 0, 0, 1.62843044502544, 0, 3.54562104171167, -0.814818121686403, 0, 0, 0, 11.0554159678513, 0, -2.54103636481183e-14, 7.18070330817254, 0, -1.11676565710084, 0.197418142283058, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, -1.21467689198221e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {1.48323969741914, 1.00508909139075, -0.193429485824672, 3.55615782880927, -8.46157280212813, 0, 1.04737699623147e-14, 4.45964044034302, -0.858258202906999, 2.67027285925188, 0, 0, 1.43424333847387e-14, 0, 12.4373429638327, 2.02429672213922e-14, 1.33492152068286e-14, 8.42792824422444, -1.62195554683795, -2.29379153190448, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1.03917274103107e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, -1.08180915476963e-14, 0, 0, 0, 1.39785916269036e-14, 0, 0, 0, 0, 0, 1.03080706885695e-14, 0, 0, 0, 0},
    {1.14891252930762, 0.259512887494088, 0.749149177264413, 2.75458800949269, 0, -8.46157280212813, 1.97736713455776e-14, 1.15147421036812, 3.32401972660468, 2.06838446274242, 0, 0, 1.76595565092679e-14, 3.53657180389623e-14, 0, 12.4373429638327, -2.12998980019174e-14, 2.1760817155275, 6.28180682119209, -1.77676328054753, 0, 0, 0, 0, 0, 0, 0, 0, -2.02381918388612e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1.29298824184789e-14, 0, 0, 0, 0, -1.20899537342362e-14, 0, 0, 1.49778288260513e-14, 1.57856876457792e-14, 0, 0, 0, 0, 0, 0},
    {0.663324958071077, -2.03917269773753e-14, 0.692034366650878, 1.59036212878712, 0, 0, -8.46157280212812, -1.91967284198321e-14, 3.07059789431496, 1.194182326352, 0, 1.63277888554017e-14, 0, 2.18186738788674e-14, 0, 2.30908333865784e-14, 12.4373429638327, -2.11081152198351e-14, 5.80288457473997, -1.02581475831036, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1.31364686077023e-14, 0, -1.38856086370832e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1.88129856179495e-14, 0, 0, 0, 0, 0, 0},
    {0.938083151964734, 7.62808987244123, 0, 3.11415464992886, 0, 0, 0, -7.52139804633612, 0, 5.52707620976692, 0, 0, -1.777231263474e-14, 0, -1.62387587846106e-14, 1.69100848543788e-14, 2.98390950438128e-14, 14.2141062443803, 0, 5.80288457473998, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1.39576755950212e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1.84182164821129e-14, 0, 1.05827350607878e-14, 0, 0, 0, 1.04237510536707e-14, 0, 1.26392642221179e-14, 0, 0},
    {0.541602560309088, -1.13099822653614e-14, 7.62808987244125, 1.79795802543456, 0, 0, 0, 0, -7.5213980463361, 3.19105893754052, 0, 1.03213108228181e-14, 0, 0, 0, -1.02355531327919e-14, 0, -3.31702746937809e-14, 14.2141062443803, 3.35029697130246, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, -1.166062958575e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {-5.36159180343545, -1.34228950103456e-14, 1.18705474084944e-14, 8.89943818451483, 0, 0, 0, -2.89831441233684e-14, 0, -5.26497863243526, 0, 0, 0, 0, -1.29782943052458e-14, 0, 2.98133166946067e-14, 0, 0, 16.583123951777, 0, 0, 0, 0, 0, 0, 0, 0, 1.67376776564845e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1.61338757506847e-14, 1.94557000035003e-14, 0, 0, 0, -1.11267253048686e-14, 0, 0, 0, 0, 1.32728374914348e-14, 1.68753899743024e-14},
    {2.41593350361252, 5.72988941019469, -2.20543546226845, -1.55947837083931, 8.33749895885385, -2.7677995385711, 1.06532653910463, -2.25989885993664, 0.869835476750504, 0.615066564126906, 10.333118277332, -2.18326971917503, 0.724780051083775, -0.278967971730974, -1.8907670401189, 0.627677936394735, -0.241593350361257, 0.51249688899008, -0.197260144544834, -0.139483985865487, 11.8490505948789, -1.16109782023229, 0.245326690731328, -0.0814411017928033, 0.0313466946954489, -1.03851746182649, 0.219426862868122, -0.0728431359084709, 0.0280373360835781, 0.190029237516519, -0.063084006188059, 0.0242810453028233, -0.0515078753637743, 0.0198253904710968, 0.014018668041796, 0, 0, 1.04549788312352e-14, 0, 0, 0, 0, 0, 0, 0, 0, 1.07828201668863e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {2.18529407725403, 3.45525332069937, 0.997445717412063, -1.41060126129511, 3.23209352764252, 5.28531208250976, -2.89087233497829, -1.36277028773849, -0.393397896234727, 0.556348640264195, 2.33666428910958, 9.02784807228869, -3.93352933905731, 1.64018416088945, -0.732969916135171, -1.19859611757308, 0.655588223176205, 0.309047252182632, 0.0892142571199684, -0.126168012376111, 1.19087439227729, 11.55277744383, -2.44096975264613, 0.810328731479535, -0.311895674167857, -0.234843597212088, -0.907332870306508, 0.395334573318422, -0.164844710568306, 0.0736662483163242, 0.120463442338383, -0.0658890955530685, -0.0310604175145507, -0.00896637020658553, 0.0126803623514123, 0, 0, 0, 0, 0, 0, 0, -1.66112853067542e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {1.92724822331887, 1.82834820066012, 2.99085908983324, -1.24403337882031, 0.950146187582616, 5.74064456311309, -0.169967317119763, -0.721110255092806, -1.17961073303079, 0.490653381462656, 0.294392028877589, 4.9761335152812, 5.78174466995658, -3.2268221135323, -0.215472901842837, -1.30185581821819, 0.0385449644663795, 0.163532434528623, 0.267510569439614, -0.11126972805285, 0, 2.64638753839399, 10.7636648722209, -3.57321383950879, 1.37533064807499, -0.0295875121532857, -0.500120235663636, -0.581087203147967, 0.324307824721465, 0.0216558414515637, 0.130841432732446, -0.00387391468765088, -0.0164356280722617, -0.0268858238268399, 0.0111830284386723, 0, 0, 0, 0, 0, -1.09166603315028e-14, 0, 0, 0, 1.42437445527895e-14, 0, 0, 0, -1.02438819301777e-14, 0, 0, 0, 0, 0, 0, 0},
    {1.62882203585591, 0.772618130456562, 3.86595114186567, -1.05140010313427, 0.160604018609904, 3.35889012281481, 3.87851223317101, -0.304724700110016, -1.52475169285617, 0.414677792940092, 0, 1.13551211138501, 7.32969916135159, 0.564240504518027, -0.0364215679542207, -0.761724681099395, -0.879563899362167, 0.0691050664139829, 0.345781182036178, -0.0940400840863483, 0, 0, 4.46577897103986, 9.05975063854702, -3.48709964663734, 0, -0.114123261162682, -0.736662483163202, -0.0567083043941542, 0.0036605053085017, 0.0765562109321201, 0.0883994979795774, -0.00694532049720645, -0.0347523163751393, 0.00945138406569343, 0, 0, 0, 0, 0, 0, 0, 0, 1.06468345288818e-14, -1.93401297016719e-14, 1.26134404858539e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {1.26168012376111, 0.199489143482416, 3.68560354207931, -0.814411017927961, 0, 1.02786571910339, 6.00856531485322, -0.0786795792469436, -1.4536216402544, 0.321208037219821, 0, 0, 3.0280322970267, 6.99294104721294, 0, -0.23309803490712, -1.36261453366203, 0.0178428514239939, 0.32965040232813, -0.0728431359084719, 0, 0, 0, 6.80499652729883, 5.40219154319687, 0, 0, -0.304328696433803, -0.702817019232747, 0, 0.0234272339744286, 0.136947913395211, -0.00179327404131641, -0.0331311120155151, 0.00732101061701076, 0, 0, 0, 0, 0, 0, 0, 0, 0, -1.10019056033674e-14, -1.37033343408735e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0.728431359084681, 0, 2.32737334062814, -0.470200420431712, 0, 0, 4.49691252107734, 0, -0.917928424547679, 0.185449546754743, 0, 0, 1.80505771145776e-14, 7.06540869306227, 0, 0, -1.01980390271859, 0, 0.208166599946618, -0.042056004125377, 0, 0, 0, 0, 9.92395326897748, 0, 0, 0, -0.710100291678884, 0, 0, 0.102494148638114, 0, -0.0209215304820281, 0.00422678745047274, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, -2.50163969655578e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {1.99489143482414, 3.15420030940281, -1.09264703862701, 1.54523626091315, 2.9504842217604, -0.761811750275062, 0.263899331455876, 5.59815020469134, -1.93925611658551, -2.28543525082518, 2.13307290077017, -0.257539376818833, 0.0664963811608233, -0.0230350221379902, 9.14446135479768, -2.36108976914455, 0.817905488277909, -3.33842298506508, 1.1564636454577, 1.02505848514083, 13.0453753561106, -1.2103323911658e-14, 0, -1.46406519642548e-14, 0, 11.5766222110848, -1.39771878815472, 0.360889439280293, -0.125015768949695, -2.15192884958006, 0.555625639776451, -0.192474367616164, 0.595437196138645, -0.206265495285706, -0.166687691932928, 0, 0, 1.33850600318309e-14, 0, 0, 0, 2.24412658009199e-14, 0, 0, 0, 1.66496943714949e-14, 1.3113069716231e-14, 0, -1.15323982020285e-14, 0, 0, 0, -1.02677636674246e-14, 0, 0, 0},
    {1.7593288763725, 1.66904592079256, 0.96362411165943, 1.36277028773851, 0.86736083311088, 2.28430636201791, -1.00852844760552, 2.96226264871973, 1.71026313764871, -2.01556443707463, 0.268741924943271, 2.04415543160776, -0.527798662911752, 0.18283482006601, 2.68821895760395, 7.07977053255616, -3.12574097035513, -1.76652739792481, -1.01990506872277, 0.904016610326402, 0, 13.0453753561106, 0, 0, 0, 1.4585173044132, 11.0940489487798, -2.86447112137559, 0.992281903807266, -0.632607619454896, -1.66605356688627, 0.735567892896463, 0.315075748468093, 0.181909068193174, -0.147004726489972, 0, 0, -1.03074285928854e-14, 0, 0, 0, 0, -1.72927567727304e-14, 1.04819480446677e-14, 1.76046313417165e-14, 0, 0, 0, 0, 0, 1.43579679437173e-14, 0, 0, 0, 0, 0},
    {1.48690428533297, 0.705300630647553, 2.03602754481992, 1.1517511068998, 0.14661073969771, 1.81702705031798, 0.65566316039122, 1.2517844405944, 3.61359041872288, -1.70346285967312, 0, 0.614267257013221, 1.78428514239955, -0.618094504365253, 0.45439193793218, 5.631527706439, 2.03210252328837, -0.746494073225325, -2.15494277062552, 0.764033484562605, 0, 0, 13.0453753561106, 0, 0, 0, 3.33375383865873, 9.68367982311024, -3.3545250915713, -0.1069302042312, -1.32524448062077, -0.478206411017785, 0.133143804689794, 0.384353057392897, -0.124241670058195, 0, 0, 0, 0, 0, 0, 0, 0, 1.35190501684936e-14, 2.13313974056602e-14, 0, 0, 0, -1.39901730746776e-14, 0, 0, 0, 0, 0, 0, 0},
    {1.15175110689978, 0.18210783977117, 2.20794021658194, 0.892142571199777, 0, 0.615765106730376, 2.13307290077017, 0.323209352764267, 3.91870514328392, -1.31949665727937, -1.1268544360518e-14, 0, 1.07496769977313, 1.11713920350151, 2.42487345110409e-14, 1.90844613931506, 6.61105135360481, -0.192743940909627, -2.33689608954558, 0.591817792331174, 0, 0, 0, 13.0453753561106, 0, 0, -1.3756522835902e-14, 5.83406921765278, 6.06294257990891, 0, -0.449106857771022, -1.55575179137408, 0.0343775825476135, 0.416806037297048, -0.0962371838080746, 0, 0, 0, 0, 0, 0, 0, 0, 1.42803357596901e-14, 0, 0, 0, 0, -1.21198190345942e-14, 0, 0, 0, 0, 0, -1.3839902652557e-14, 0},
    {0.664963811608031, 1.90472468148085e-14, 1.45686271816934, 0.515078753637701, 0, 0, 1.8472953201911, 0, 2.58567482211402, -0.761811750275038, 0, 1.08037159720826e-14, 2.26487951659612e-14, 1.71994831963699, 0, 1.52639255690414e-14, 5.72533841794521, 0, -1.54195152727693, 0.3416861617136, 0, 0, -1.00168576142402e-14, 0, 13.0453753561106, 0, 0, 0, 9.33451074824442, 0, 0, -1.34732057331308, 0, 0.275020660380927, -0.0555625639776386, 0, 0, 0, 0, 0, -2.03433658742105e-14, 0, 0, 0, 0, -3.84929350411527e-14, 0, 1.299574464293e-14, 0, 0, 0, 0, 0, 0, 0, 0},
    {1.57359158493892, 1.49284005458436, -0.43094580368567, 3.04724700110022, 0.775791113542717, -0.120185042515459, 0.0346944333244422, 5.29905651979673, -1.52970585407784, 0.901387818866001, -10.576283741361, 0, 0, -1.14454236590933e-14, 4.51561078268938, -0.699555414372402, 0.201944253400485, 6.36462844462229, -1.83730997289732, -2.74825341360594, 0, 0, 0, -1.03903584730172e-14, 0, 14.3499128917217, 0, 0, 1.48731679939335e-14, 10.8920630294282, -1.68739114875269, 0.487107866980279, -3.0999354831996, 0.894874292847882, 0.903959543974665, 0, 0, 0, 0, 0, 0, 0, 0, -1.69818485661453e-14, 1.01890667576586e-14, 1.15185313992138e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1.11022302462516e-14},
    {1.3299276232161, 0.630840061880575, 0.728431359084684, 2.57539376818859, 0.131132632078234, 0.711024300256703, -0.205255035576781, 2.23926008187655, 2.585674822114, 0.761811750275056, 0, -10.576283741361, 1.57303341744021e-14, 1.70725378740663e-14, 0.763277533137587, 4.13862564412658, -1.19471831485577, 2.68954640486935, 3.10562068169859, -2.32269806558125, 0, 0, 0, 0, 0, 0, 14.3499128917217, 0, 0, 1.84109468241849, 9.98274066131698, -2.88176900403078, -1.30996183150502, -1.51261363209512, 0.763985254692618, 0, 0, 0, 0, 0, 1.04560213685676e-14, 0, 0, 1.2776556039649e-14, -1.21474404387607e-14, 0, 0, 0, 1.05952301335811e-14, 0, 0, 1.33062123785088e-14, 1.01731376449221e-14, 1.65118907931275e-14, 0, 0},
    {1.03015750727544, 0.162882203585583, 1.12848100903609, 1.99489143482414, 0, 0.314718316987774, 0.476969600708478, 0.578174466995633, 4.00571020990216, 0.590096844352089, 0, 0, -10.5762837413611, 0, 0, 1.83186607953021, 2.77627448210726, 0.694437895660724, 4.8112068719424, -1.79915418525285, 0, 0, 0, 0, 0, 0, 0, 14.3499128917217, 0, 0, 4.41862723780437, 6.69662601613678, -0.338230690505753, -2.3433309625403, 0.591780433634519, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, -1.48270629766312e-14, 0, 0, 0, 0, 0, -1.15317140110751e-14, 0},
    {0.59476171413317, 0, 0.814411017927954, 1.15175110689979, 0, 0, 0.590096844352077, 1.28927856428244e-14, 2.89087233497831, 0.340692571934635, 0, 0, 0, -10.576283741361, 0, 0, 3.4347488991191, -1.66893313716044e-14, 3.47218947830366, -1.03874215316937, 0, 0, 0, 0, 0, 0, 0, 0, 14.3499128917217, 0, 0, 8.2849260708832, 0, -1.69115345252879, 0.341664592660046, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, -1.19245108709972e-14, -1.0049703503871e-14, 0, -1.01285720807823e-14, 1.46934267294508e-14, 0, 0, 0, 0, -1.33537813474028e-14, 1.11022302462516e-14},
    {1.15175110689978, 0.546323519313492, -0.105140010313412, 3.27118942773249, 9.65295620481433, 0, 0, 2.69341127303539, -0.518347241175101, 4.25171145123355, 0, 0, -1.03926949302858e-14, 0, -9.915266007526, 2.32678428089351e-14, -1.46605852864906e-14, 6.71889089790561, -1.29305115618716, 1.82865048192215, 0, 0, 1.09752665623958e-14, 0, 0, 0, 0, 0, 0, 15.9443476574685, -1.29058000479168e-14, 0, 9.45383520059453, -1.81939143264587, -2.94058061635793, 0, 0, 0, 0, 0, 0, 0, 0, -1.22633667590031e-14, 0, 0, 0, 1.71762874424691e-14, 0, 0, 0, 0, 0, 0, 0, 2.15938378289593e-14},
    {0.89214257119978, 0.141060126129492, 0.407205508963968, 2.53385243517952, 0, 9.65295620481434, 0, 0.695435800330236, 2.00755023262381, 3.29336152870136, 0, 1.78864735610198e-14, -1.45456143897546e-14, -1.59407140440659e-14, -1.27256502403865e-14, -9.91526600752597, 1.74986266317689e-14, 1.73481017017152, 5.00796559370718, 1.41646657250367, 0, 0, 0, 0, 0, 0, 0, 0, 1.67839306981372e-14, 0, 15.9443476574685, 0, 2.44096975264614, 7.04647271886991, -2.27776395106694, 0, 0, 0, 0, 0, 0, 0, 0, -1.25723596525151e-14, -1.13140881999668e-14, 1.41799567998805e-14, 0, 1.21347241236521e-14, 0, 0, 0, 0, 0, 0, 0, -1.24344978758018e-14},
    {0.515078753637724, -2.3619485326075e-14, 0.376160336345333, 1.4629203855377, 0, 1.01804649671178e-14, 9.65295620481433, -2.11069547491802e-14, 1.8544954675473, 1.90142316513451, 0, 0, -2.14622752906569e-14, -4.34603909418763e-14, -1.24631731336805e-14, 0, -9.91526600752596, -1.51138454398741e-14, 4.62616045379076, 0.817797356933143, 0, 0, 0, 0, -1.03207533752511e-14, 0, 0, 0, 2.46817376593445e-14, 0, 0, 15.9443476574685, 1.00089905485244e-14, 6.50925267372302, -1.3150676302989, 0, 0, 0, 0, 0, -1.6390734838769e-14, 0, 0, -1.2109391798167e-14, 0, 1.09260746948048e-14, 0, 0, 0, -1.16429829249248e-14, 0, 0, 0, 0, 0, 1.24344978758018e-14},
    {0.728431359084676, -7.94708263760856, 0, 2.53908227033122, 0, 0, -1.27415633137685e-14, 10.2207771580387, 0, 5.0071377623777, 2.28612268350337e-14, -1.8719154526289e-14, 1.31033584296945e-14, -2.53050332009669e-14, 0, 2.83491039062371e-14, 1.64090158394923e-14, -8.49879943502228, 0, 6.93924068068621, 0, -1.52898531740193e-14, 0, 0, 0, -1.42660700700925e-14, 0, 0, 1.84038304706445e-14, 0, -1.43708740249419e-14, 0, 17.9373911146521, 0, 6.27677936394721, 0, 0, 0, 0, 0, 0, 0, 1.74801884942513e-14, 0, 1.54335530678735e-14, 0, 0, 0, 0, -1.82587310754697e-14, 0, 0, 0, -1.41280724493113e-14, 0, 1.68753899743024e-14},
    {0.420560041253715, 0, -7.94708263760862, 1.46593983227031, 0, 0, 0, 0, 10.2207771580387, 2.89087233497828, 0, 0, 0, -4.72854335057725e-14, 0, 2.90947883710161e-14, 2.94685265414151e-14, 0, -8.49879943502229, 4.00637247496575, 0, 0, 0, -1.04474568447982e-14, -1.49316504487054e-14, 0, 0, 0, 2.10839214785391e-14, 0, 0, -1.06464761849486e-14, 0, 17.9373911146521, 3.62390025541881, 0, 0, 0, 0, 0, -1.23540168908514e-14, 0, 0, 0, 2.09495610608972e-14, 1.91267628119894e-14, 0, 0, 0, -2.42410393446135e-14, 0, 0, 0, 0, 0, 0},
    {6.542378855465, 2.78160733626274e-14, 0, -6.91050664139872, 0, 0, -1.56111299636533e-14, 1.45917324702114e-14, 0, 12.2649325896465, 0, -1.97411729296339e-14, 0, 0, 0, 0, 1.23535778391715e-14, 2.43644083996432e-14, -1.70040757261543e-14, -5.66586629001476, 0, 0, 0, 0, 0, 0, 1.69114983860161e-14, 0, 0, 0, 0, 0, 0, 0, 20.4998755596024, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, -1.54091873433635e-14, 0, -2.20095446006121e-14, 0, 0, 1.1817028848966e-14, -1.05566456925328e-14, -1.23638867682084e-14, 2.1316282072803e-14}};
    
    // Compute reference derivatives
    // Declare pointer to array of derivatives on FIAT element
    double *derivatives = new double [num_derivatives];
    
    // Declare coefficients
    double coeff0_0 = 0;
    double coeff0_1 = 0;
    double coeff0_2 = 0;
    double coeff0_3 = 0;
    double coeff0_4 = 0;
    double coeff0_5 = 0;
    double coeff0_6 = 0;
    double coeff0_7 = 0;
    double coeff0_8 = 0;
    double coeff0_9 = 0;
    double coeff0_10 = 0;
    double coeff0_11 = 0;
    double coeff0_12 = 0;
    double coeff0_13 = 0;
    double coeff0_14 = 0;
    double coeff0_15 = 0;
    double coeff0_16 = 0;
    double coeff0_17 = 0;
    double coeff0_18 = 0;
    double coeff0_19 = 0;
    double coeff0_20 = 0;
    double coeff0_21 = 0;
    double coeff0_22 = 0;
    double coeff0_23 = 0;
    double coeff0_24 = 0;
    double coeff0_25 = 0;
    double coeff0_26 = 0;
    double coeff0_27 = 0;
    double coeff0_28 = 0;
    double coeff0_29 = 0;
    double coeff0_30 = 0;
    double coeff0_31 = 0;
    double coeff0_32 = 0;
    double coeff0_33 = 0;
    double coeff0_34 = 0;
    double coeff0_35 = 0;
    double coeff0_36 = 0;
    double coeff0_37 = 0;
    double coeff0_38 = 0;
    double coeff0_39 = 0;
    double coeff0_40 = 0;
    double coeff0_41 = 0;
    double coeff0_42 = 0;
    double coeff0_43 = 0;
    double coeff0_44 = 0;
    double coeff0_45 = 0;
    double coeff0_46 = 0;
    double coeff0_47 = 0;
    double coeff0_48 = 0;
    double coeff0_49 = 0;
    double coeff0_50 = 0;
    double coeff0_51 = 0;
    double coeff0_52 = 0;
    double coeff0_53 = 0;
    double coeff0_54 = 0;
    double coeff0_55 = 0;
    
    // Declare new coefficients
    double new_coeff0_0 = 0;
    double new_coeff0_1 = 0;
    double new_coeff0_2 = 0;
    double new_coeff0_3 = 0;
    double new_coeff0_4 = 0;
    double new_coeff0_5 = 0;
    double new_coeff0_6 = 0;
    double new_coeff0_7 = 0;
    double new_coeff0_8 = 0;
    double new_coeff0_9 = 0;
    double new_coeff0_10 = 0;
    double new_coeff0_11 = 0;
    double new_coeff0_12 = 0;
    double new_coeff0_13 = 0;
    double new_coeff0_14 = 0;
    double new_coeff0_15 = 0;
    double new_coeff0_16 = 0;
    double new_coeff0_17 = 0;
    double new_coeff0_18 = 0;
    double new_coeff0_19 = 0;
    double new_coeff0_20 = 0;
    double new_coeff0_21 = 0;
    double new_coeff0_22 = 0;
    double new_coeff0_23 = 0;
    double new_coeff0_24 = 0;
    double new_coeff0_25 = 0;
    double new_coeff0_26 = 0;
    double new_coeff0_27 = 0;
    double new_coeff0_28 = 0;
    double new_coeff0_29 = 0;
    double new_coeff0_30 = 0;
    double new_coeff0_31 = 0;
    double new_coeff0_32 = 0;
    double new_coeff0_33 = 0;
    double new_coeff0_34 = 0;
    double new_coeff0_35 = 0;
    double new_coeff0_36 = 0;
    double new_coeff0_37 = 0;
    double new_coeff0_38 = 0;
    double new_coeff0_39 = 0;
    double new_coeff0_40 = 0;
    double new_coeff0_41 = 0;
    double new_coeff0_42 = 0;
    double new_coeff0_43 = 0;
    double new_coeff0_44 = 0;
    double new_coeff0_45 = 0;
    double new_coeff0_46 = 0;
    double new_coeff0_47 = 0;
    double new_coeff0_48 = 0;
    double new_coeff0_49 = 0;
    double new_coeff0_50 = 0;
    double new_coeff0_51 = 0;
    double new_coeff0_52 = 0;
    double new_coeff0_53 = 0;
    double new_coeff0_54 = 0;
    double new_coeff0_55 = 0;
    
    // Loop possible derivatives
    for (unsigned int deriv_num = 0; deriv_num < num_derivatives; deriv_num++)
    {
      // Get values from coefficients array
      new_coeff0_0 = coefficients0[dof][0];
      new_coeff0_1 = coefficients0[dof][1];
      new_coeff0_2 = coefficients0[dof][2];
      new_coeff0_3 = coefficients0[dof][3];
      new_coeff0_4 = coefficients0[dof][4];
      new_coeff0_5 = coefficients0[dof][5];
      new_coeff0_6 = coefficients0[dof][6];
      new_coeff0_7 = coefficients0[dof][7];
      new_coeff0_8 = coefficients0[dof][8];
      new_coeff0_9 = coefficients0[dof][9];
      new_coeff0_10 = coefficients0[dof][10];
      new_coeff0_11 = coefficients0[dof][11];
      new_coeff0_12 = coefficients0[dof][12];
      new_coeff0_13 = coefficients0[dof][13];
      new_coeff0_14 = coefficients0[dof][14];
      new_coeff0_15 = coefficients0[dof][15];
      new_coeff0_16 = coefficients0[dof][16];
      new_coeff0_17 = coefficients0[dof][17];
      new_coeff0_18 = coefficients0[dof][18];
      new_coeff0_19 = coefficients0[dof][19];
      new_coeff0_20 = coefficients0[dof][20];
      new_coeff0_21 = coefficients0[dof][21];
      new_coeff0_22 = coefficients0[dof][22];
      new_coeff0_23 = coefficients0[dof][23];
      new_coeff0_24 = coefficients0[dof][24];
      new_coeff0_25 = coefficients0[dof][25];
      new_coeff0_26 = coefficients0[dof][26];
      new_coeff0_27 = coefficients0[dof][27];
      new_coeff0_28 = coefficients0[dof][28];
      new_coeff0_29 = coefficients0[dof][29];
      new_coeff0_30 = coefficients0[dof][30];
      new_coeff0_31 = coefficients0[dof][31];
      new_coeff0_32 = coefficients0[dof][32];
      new_coeff0_33 = coefficients0[dof][33];
      new_coeff0_34 = coefficients0[dof][34];
      new_coeff0_35 = coefficients0[dof][35];
      new_coeff0_36 = coefficients0[dof][36];
      new_coeff0_37 = coefficients0[dof][37];
      new_coeff0_38 = coefficients0[dof][38];
      new_coeff0_39 = coefficients0[dof][39];
      new_coeff0_40 = coefficients0[dof][40];
      new_coeff0_41 = coefficients0[dof][41];
      new_coeff0_42 = coefficients0[dof][42];
      new_coeff0_43 = coefficients0[dof][43];
      new_coeff0_44 = coefficients0[dof][44];
      new_coeff0_45 = coefficients0[dof][45];
      new_coeff0_46 = coefficients0[dof][46];
      new_coeff0_47 = coefficients0[dof][47];
      new_coeff0_48 = coefficients0[dof][48];
      new_coeff0_49 = coefficients0[dof][49];
      new_coeff0_50 = coefficients0[dof][50];
      new_coeff0_51 = coefficients0[dof][51];
      new_coeff0_52 = coefficients0[dof][52];
      new_coeff0_53 = coefficients0[dof][53];
      new_coeff0_54 = coefficients0[dof][54];
      new_coeff0_55 = coefficients0[dof][55];
    
      // Loop derivative order
      for (unsigned int j = 0; j < n; j++)
      {
        // Update old coefficients
        coeff0_0 = new_coeff0_0;
        coeff0_1 = new_coeff0_1;
        coeff0_2 = new_coeff0_2;
        coeff0_3 = new_coeff0_3;
        coeff0_4 = new_coeff0_4;
        coeff0_5 = new_coeff0_5;
        coeff0_6 = new_coeff0_6;
        coeff0_7 = new_coeff0_7;
        coeff0_8 = new_coeff0_8;
        coeff0_9 = new_coeff0_9;
        coeff0_10 = new_coeff0_10;
        coeff0_11 = new_coeff0_11;
        coeff0_12 = new_coeff0_12;
        coeff0_13 = new_coeff0_13;
        coeff0_14 = new_coeff0_14;
        coeff0_15 = new_coeff0_15;
        coeff0_16 = new_coeff0_16;
        coeff0_17 = new_coeff0_17;
        coeff0_18 = new_coeff0_18;
        coeff0_19 = new_coeff0_19;
        coeff0_20 = new_coeff0_20;
        coeff0_21 = new_coeff0_21;
        coeff0_22 = new_coeff0_22;
        coeff0_23 = new_coeff0_23;
        coeff0_24 = new_coeff0_24;
        coeff0_25 = new_coeff0_25;
        coeff0_26 = new_coeff0_26;
        coeff0_27 = new_coeff0_27;
        coeff0_28 = new_coeff0_28;
        coeff0_29 = new_coeff0_29;
        coeff0_30 = new_coeff0_30;
        coeff0_31 = new_coeff0_31;
        coeff0_32 = new_coeff0_32;
        coeff0_33 = new_coeff0_33;
        coeff0_34 = new_coeff0_34;
        coeff0_35 = new_coeff0_35;
        coeff0_36 = new_coeff0_36;
        coeff0_37 = new_coeff0_37;
        coeff0_38 = new_coeff0_38;
        coeff0_39 = new_coeff0_39;
        coeff0_40 = new_coeff0_40;
        coeff0_41 = new_coeff0_41;
        coeff0_42 = new_coeff0_42;
        coeff0_43 = new_coeff0_43;
        coeff0_44 = new_coeff0_44;
        coeff0_45 = new_coeff0_45;
        coeff0_46 = new_coeff0_46;
        coeff0_47 = new_coeff0_47;
        coeff0_48 = new_coeff0_48;
        coeff0_49 = new_coeff0_49;
        coeff0_50 = new_coeff0_50;
        coeff0_51 = new_coeff0_51;
        coeff0_52 = new_coeff0_52;
        coeff0_53 = new_coeff0_53;
        coeff0_54 = new_coeff0_54;
        coeff0_55 = new_coeff0_55;
    
        if(combinations[deriv_num][j] == 0)
        {
          new_coeff0_0 = coeff0_0*dmats0[0][0] + coeff0_1*dmats0[1][0] + coeff0_2*dmats0[2][0] + coeff0_3*dmats0[3][0] + coeff0_4*dmats0[4][0] + coeff0_5*dmats0[5][0] + coeff0_6*dmats0[6][0] + coeff0_7*dmats0[7][0] + coeff0_8*dmats0[8][0] + coeff0_9*dmats0[9][0] + coeff0_10*dmats0[10][0] + coeff0_11*dmats0[11][0] + coeff0_12*dmats0[12][0] + coeff0_13*dmats0[13][0] + coeff0_14*dmats0[14][0] + coeff0_15*dmats0[15][0] + coeff0_16*dmats0[16][0] + coeff0_17*dmats0[17][0] + coeff0_18*dmats0[18][0] + coeff0_19*dmats0[19][0] + coeff0_20*dmats0[20][0] + coeff0_21*dmats0[21][0] + coeff0_22*dmats0[22][0] + coeff0_23*dmats0[23][0] + coeff0_24*dmats0[24][0] + coeff0_25*dmats0[25][0] + coeff0_26*dmats0[26][0] + coeff0_27*dmats0[27][0] + coeff0_28*dmats0[28][0] + coeff0_29*dmats0[29][0] + coeff0_30*dmats0[30][0] + coeff0_31*dmats0[31][0] + coeff0_32*dmats0[32][0] + coeff0_33*dmats0[33][0] + coeff0_34*dmats0[34][0] + coeff0_35*dmats0[35][0] + coeff0_36*dmats0[36][0] + coeff0_37*dmats0[37][0] + coeff0_38*dmats0[38][0] + coeff0_39*dmats0[39][0] + coeff0_40*dmats0[40][0] + coeff0_41*dmats0[41][0] + coeff0_42*dmats0[42][0] + coeff0_43*dmats0[43][0] + coeff0_44*dmats0[44][0] + coeff0_45*dmats0[45][0] + coeff0_46*dmats0[46][0] + coeff0_47*dmats0[47][0] + coeff0_48*dmats0[48][0] + coeff0_49*dmats0[49][0] + coeff0_50*dmats0[50][0] + coeff0_51*dmats0[51][0] + coeff0_52*dmats0[52][0] + coeff0_53*dmats0[53][0] + coeff0_54*dmats0[54][0] + coeff0_55*dmats0[55][0];
          new_coeff0_1 = coeff0_0*dmats0[0][1] + coeff0_1*dmats0[1][1] + coeff0_2*dmats0[2][1] + coeff0_3*dmats0[3][1] + coeff0_4*dmats0[4][1] + coeff0_5*dmats0[5][1] + coeff0_6*dmats0[6][1] + coeff0_7*dmats0[7][1] + coeff0_8*dmats0[8][1] + coeff0_9*dmats0[9][1] + coeff0_10*dmats0[10][1] + coeff0_11*dmats0[11][1] + coeff0_12*dmats0[12][1] + coeff0_13*dmats0[13][1] + coeff0_14*dmats0[14][1] + coeff0_15*dmats0[15][1] + coeff0_16*dmats0[16][1] + coeff0_17*dmats0[17][1] + coeff0_18*dmats0[18][1] + coeff0_19*dmats0[19][1] + coeff0_20*dmats0[20][1] + coeff0_21*dmats0[21][1] + coeff0_22*dmats0[22][1] + coeff0_23*dmats0[23][1] + coeff0_24*dmats0[24][1] + coeff0_25*dmats0[25][1] + coeff0_26*dmats0[26][1] + coeff0_27*dmats0[27][1] + coeff0_28*dmats0[28][1] + coeff0_29*dmats0[29][1] + coeff0_30*dmats0[30][1] + coeff0_31*dmats0[31][1] + coeff0_32*dmats0[32][1] + coeff0_33*dmats0[33][1] + coeff0_34*dmats0[34][1] + coeff0_35*dmats0[35][1] + coeff0_36*dmats0[36][1] + coeff0_37*dmats0[37][1] + coeff0_38*dmats0[38][1] + coeff0_39*dmats0[39][1] + coeff0_40*dmats0[40][1] + coeff0_41*dmats0[41][1] + coeff0_42*dmats0[42][1] + coeff0_43*dmats0[43][1] + coeff0_44*dmats0[44][1] + coeff0_45*dmats0[45][1] + coeff0_46*dmats0[46][1] + coeff0_47*dmats0[47][1] + coeff0_48*dmats0[48][1] + coeff0_49*dmats0[49][1] + coeff0_50*dmats0[50][1] + coeff0_51*dmats0[51][1] + coeff0_52*dmats0[52][1] + coeff0_53*dmats0[53][1] + coeff0_54*dmats0[54][1] + coeff0_55*dmats0[55][1];
          new_coeff0_2 = coeff0_0*dmats0[0][2] + coeff0_1*dmats0[1][2] + coeff0_2*dmats0[2][2] + coeff0_3*dmats0[3][2] + coeff0_4*dmats0[4][2] + coeff0_5*dmats0[5][2] + coeff0_6*dmats0[6][2] + coeff0_7*dmats0[7][2] + coeff0_8*dmats0[8][2] + coeff0_9*dmats0[9][2] + coeff0_10*dmats0[10][2] + coeff0_11*dmats0[11][2] + coeff0_12*dmats0[12][2] + coeff0_13*dmats0[13][2] + coeff0_14*dmats0[14][2] + coeff0_15*dmats0[15][2] + coeff0_16*dmats0[16][2] + coeff0_17*dmats0[17][2] + coeff0_18*dmats0[18][2] + coeff0_19*dmats0[19][2] + coeff0_20*dmats0[20][2] + coeff0_21*dmats0[21][2] + coeff0_22*dmats0[22][2] + coeff0_23*dmats0[23][2] + coeff0_24*dmats0[24][2] + coeff0_25*dmats0[25][2] + coeff0_26*dmats0[26][2] + coeff0_27*dmats0[27][2] + coeff0_28*dmats0[28][2] + coeff0_29*dmats0[29][2] + coeff0_30*dmats0[30][2] + coeff0_31*dmats0[31][2] + coeff0_32*dmats0[32][2] + coeff0_33*dmats0[33][2] + coeff0_34*dmats0[34][2] + coeff0_35*dmats0[35][2] + coeff0_36*dmats0[36][2] + coeff0_37*dmats0[37][2] + coeff0_38*dmats0[38][2] + coeff0_39*dmats0[39][2] + coeff0_40*dmats0[40][2] + coeff0_41*dmats0[41][2] + coeff0_42*dmats0[42][2] + coeff0_43*dmats0[43][2] + coeff0_44*dmats0[44][2] + coeff0_45*dmats0[45][2] + coeff0_46*dmats0[46][2] + coeff0_47*dmats0[47][2] + coeff0_48*dmats0[48][2] + coeff0_49*dmats0[49][2] + coeff0_50*dmats0[50][2] + coeff0_51*dmats0[51][2] + coeff0_52*dmats0[52][2] + coeff0_53*dmats0[53][2] + coeff0_54*dmats0[54][2] + coeff0_55*dmats0[55][2];
          new_coeff0_3 = coeff0_0*dmats0[0][3] + coeff0_1*dmats0[1][3] + coeff0_2*dmats0[2][3] + coeff0_3*dmats0[3][3] + coeff0_4*dmats0[4][3] + coeff0_5*dmats0[5][3] + coeff0_6*dmats0[6][3] + coeff0_7*dmats0[7][3] + coeff0_8*dmats0[8][3] + coeff0_9*dmats0[9][3] + coeff0_10*dmats0[10][3] + coeff0_11*dmats0[11][3] + coeff0_12*dmats0[12][3] + coeff0_13*dmats0[13][3] + coeff0_14*dmats0[14][3] + coeff0_15*dmats0[15][3] + coeff0_16*dmats0[16][3] + coeff0_17*dmats0[17][3] + coeff0_18*dmats0[18][3] + coeff0_19*dmats0[19][3] + coeff0_20*dmats0[20][3] + coeff0_21*dmats0[21][3] + coeff0_22*dmats0[22][3] + coeff0_23*dmats0[23][3] + coeff0_24*dmats0[24][3] + coeff0_25*dmats0[25][3] + coeff0_26*dmats0[26][3] + coeff0_27*dmats0[27][3] + coeff0_28*dmats0[28][3] + coeff0_29*dmats0[29][3] + coeff0_30*dmats0[30][3] + coeff0_31*dmats0[31][3] + coeff0_32*dmats0[32][3] + coeff0_33*dmats0[33][3] + coeff0_34*dmats0[34][3] + coeff0_35*dmats0[35][3] + coeff0_36*dmats0[36][3] + coeff0_37*dmats0[37][3] + coeff0_38*dmats0[38][3] + coeff0_39*dmats0[39][3] + coeff0_40*dmats0[40][3] + coeff0_41*dmats0[41][3] + coeff0_42*dmats0[42][3] + coeff0_43*dmats0[43][3] + coeff0_44*dmats0[44][3] + coeff0_45*dmats0[45][3] + coeff0_46*dmats0[46][3] + coeff0_47*dmats0[47][3] + coeff0_48*dmats0[48][3] + coeff0_49*dmats0[49][3] + coeff0_50*dmats0[50][3] + coeff0_51*dmats0[51][3] + coeff0_52*dmats0[52][3] + coeff0_53*dmats0[53][3] + coeff0_54*dmats0[54][3] + coeff0_55*dmats0[55][3];
          new_coeff0_4 = coeff0_0*dmats0[0][4] + coeff0_1*dmats0[1][4] + coeff0_2*dmats0[2][4] + coeff0_3*dmats0[3][4] + coeff0_4*dmats0[4][4] + coeff0_5*dmats0[5][4] + coeff0_6*dmats0[6][4] + coeff0_7*dmats0[7][4] + coeff0_8*dmats0[8][4] + coeff0_9*dmats0[9][4] + coeff0_10*dmats0[10][4] + coeff0_11*dmats0[11][4] + coeff0_12*dmats0[12][4] + coeff0_13*dmats0[13][4] + coeff0_14*dmats0[14][4] + coeff0_15*dmats0[15][4] + coeff0_16*dmats0[16][4] + coeff0_17*dmats0[17][4] + coeff0_18*dmats0[18][4] + coeff0_19*dmats0[19][4] + coeff0_20*dmats0[20][4] + coeff0_21*dmats0[21][4] + coeff0_22*dmats0[22][4] + coeff0_23*dmats0[23][4] + coeff0_24*dmats0[24][4] + coeff0_25*dmats0[25][4] + coeff0_26*dmats0[26][4] + coeff0_27*dmats0[27][4] + coeff0_28*dmats0[28][4] + coeff0_29*dmats0[29][4] + coeff0_30*dmats0[30][4] + coeff0_31*dmats0[31][4] + coeff0_32*dmats0[32][4] + coeff0_33*dmats0[33][4] + coeff0_34*dmats0[34][4] + coeff0_35*dmats0[35][4] + coeff0_36*dmats0[36][4] + coeff0_37*dmats0[37][4] + coeff0_38*dmats0[38][4] + coeff0_39*dmats0[39][4] + coeff0_40*dmats0[40][4] + coeff0_41*dmats0[41][4] + coeff0_42*dmats0[42][4] + coeff0_43*dmats0[43][4] + coeff0_44*dmats0[44][4] + coeff0_45*dmats0[45][4] + coeff0_46*dmats0[46][4] + coeff0_47*dmats0[47][4] + coeff0_48*dmats0[48][4] + coeff0_49*dmats0[49][4] + coeff0_50*dmats0[50][4] + coeff0_51*dmats0[51][4] + coeff0_52*dmats0[52][4] + coeff0_53*dmats0[53][4] + coeff0_54*dmats0[54][4] + coeff0_55*dmats0[55][4];
          new_coeff0_5 = coeff0_0*dmats0[0][5] + coeff0_1*dmats0[1][5] + coeff0_2*dmats0[2][5] + coeff0_3*dmats0[3][5] + coeff0_4*dmats0[4][5] + coeff0_5*dmats0[5][5] + coeff0_6*dmats0[6][5] + coeff0_7*dmats0[7][5] + coeff0_8*dmats0[8][5] + coeff0_9*dmats0[9][5] + coeff0_10*dmats0[10][5] + coeff0_11*dmats0[11][5] + coeff0_12*dmats0[12][5] + coeff0_13*dmats0[13][5] + coeff0_14*dmats0[14][5] + coeff0_15*dmats0[15][5] + coeff0_16*dmats0[16][5] + coeff0_17*dmats0[17][5] + coeff0_18*dmats0[18][5] + coeff0_19*dmats0[19][5] + coeff0_20*dmats0[20][5] + coeff0_21*dmats0[21][5] + coeff0_22*dmats0[22][5] + coeff0_23*dmats0[23][5] + coeff0_24*dmats0[24][5] + coeff0_25*dmats0[25][5] + coeff0_26*dmats0[26][5] + coeff0_27*dmats0[27][5] + coeff0_28*dmats0[28][5] + coeff0_29*dmats0[29][5] + coeff0_30*dmats0[30][5] + coeff0_31*dmats0[31][5] + coeff0_32*dmats0[32][5] + coeff0_33*dmats0[33][5] + coeff0_34*dmats0[34][5] + coeff0_35*dmats0[35][5] + coeff0_36*dmats0[36][5] + coeff0_37*dmats0[37][5] + coeff0_38*dmats0[38][5] + coeff0_39*dmats0[39][5] + coeff0_40*dmats0[40][5] + coeff0_41*dmats0[41][5] + coeff0_42*dmats0[42][5] + coeff0_43*dmats0[43][5] + coeff0_44*dmats0[44][5] + coeff0_45*dmats0[45][5] + coeff0_46*dmats0[46][5] + coeff0_47*dmats0[47][5] + coeff0_48*dmats0[48][5] + coeff0_49*dmats0[49][5] + coeff0_50*dmats0[50][5] + coeff0_51*dmats0[51][5] + coeff0_52*dmats0[52][5] + coeff0_53*dmats0[53][5] + coeff0_54*dmats0[54][5] + coeff0_55*dmats0[55][5];
          new_coeff0_6 = coeff0_0*dmats0[0][6] + coeff0_1*dmats0[1][6] + coeff0_2*dmats0[2][6] + coeff0_3*dmats0[3][6] + coeff0_4*dmats0[4][6] + coeff0_5*dmats0[5][6] + coeff0_6*dmats0[6][6] + coeff0_7*dmats0[7][6] + coeff0_8*dmats0[8][6] + coeff0_9*dmats0[9][6] + coeff0_10*dmats0[10][6] + coeff0_11*dmats0[11][6] + coeff0_12*dmats0[12][6] + coeff0_13*dmats0[13][6] + coeff0_14*dmats0[14][6] + coeff0_15*dmats0[15][6] + coeff0_16*dmats0[16][6] + coeff0_17*dmats0[17][6] + coeff0_18*dmats0[18][6] + coeff0_19*dmats0[19][6] + coeff0_20*dmats0[20][6] + coeff0_21*dmats0[21][6] + coeff0_22*dmats0[22][6] + coeff0_23*dmats0[23][6] + coeff0_24*dmats0[24][6] + coeff0_25*dmats0[25][6] + coeff0_26*dmats0[26][6] + coeff0_27*dmats0[27][6] + coeff0_28*dmats0[28][6] + coeff0_29*dmats0[29][6] + coeff0_30*dmats0[30][6] + coeff0_31*dmats0[31][6] + coeff0_32*dmats0[32][6] + coeff0_33*dmats0[33][6] + coeff0_34*dmats0[34][6] + coeff0_35*dmats0[35][6] + coeff0_36*dmats0[36][6] + coeff0_37*dmats0[37][6] + coeff0_38*dmats0[38][6] + coeff0_39*dmats0[39][6] + coeff0_40*dmats0[40][6] + coeff0_41*dmats0[41][6] + coeff0_42*dmats0[42][6] + coeff0_43*dmats0[43][6] + coeff0_44*dmats0[44][6] + coeff0_45*dmats0[45][6] + coeff0_46*dmats0[46][6] + coeff0_47*dmats0[47][6] + coeff0_48*dmats0[48][6] + coeff0_49*dmats0[49][6] + coeff0_50*dmats0[50][6] + coeff0_51*dmats0[51][6] + coeff0_52*dmats0[52][6] + coeff0_53*dmats0[53][6] + coeff0_54*dmats0[54][6] + coeff0_55*dmats0[55][6];
          new_coeff0_7 = coeff0_0*dmats0[0][7] + coeff0_1*dmats0[1][7] + coeff0_2*dmats0[2][7] + coeff0_3*dmats0[3][7] + coeff0_4*dmats0[4][7] + coeff0_5*dmats0[5][7] + coeff0_6*dmats0[6][7] + coeff0_7*dmats0[7][7] + coeff0_8*dmats0[8][7] + coeff0_9*dmats0[9][7] + coeff0_10*dmats0[10][7] + coeff0_11*dmats0[11][7] + coeff0_12*dmats0[12][7] + coeff0_13*dmats0[13][7] + coeff0_14*dmats0[14][7] + coeff0_15*dmats0[15][7] + coeff0_16*dmats0[16][7] + coeff0_17*dmats0[17][7] + coeff0_18*dmats0[18][7] + coeff0_19*dmats0[19][7] + coeff0_20*dmats0[20][7] + coeff0_21*dmats0[21][7] + coeff0_22*dmats0[22][7] + coeff0_23*dmats0[23][7] + coeff0_24*dmats0[24][7] + coeff0_25*dmats0[25][7] + coeff0_26*dmats0[26][7] + coeff0_27*dmats0[27][7] + coeff0_28*dmats0[28][7] + coeff0_29*dmats0[29][7] + coeff0_30*dmats0[30][7] + coeff0_31*dmats0[31][7] + coeff0_32*dmats0[32][7] + coeff0_33*dmats0[33][7] + coeff0_34*dmats0[34][7] + coeff0_35*dmats0[35][7] + coeff0_36*dmats0[36][7] + coeff0_37*dmats0[37][7] + coeff0_38*dmats0[38][7] + coeff0_39*dmats0[39][7] + coeff0_40*dmats0[40][7] + coeff0_41*dmats0[41][7] + coeff0_42*dmats0[42][7] + coeff0_43*dmats0[43][7] + coeff0_44*dmats0[44][7] + coeff0_45*dmats0[45][7] + coeff0_46*dmats0[46][7] + coeff0_47*dmats0[47][7] + coeff0_48*dmats0[48][7] + coeff0_49*dmats0[49][7] + coeff0_50*dmats0[50][7] + coeff0_51*dmats0[51][7] + coeff0_52*dmats0[52][7] + coeff0_53*dmats0[53][7] + coeff0_54*dmats0[54][7] + coeff0_55*dmats0[55][7];
          new_coeff0_8 = coeff0_0*dmats0[0][8] + coeff0_1*dmats0[1][8] + coeff0_2*dmats0[2][8] + coeff0_3*dmats0[3][8] + coeff0_4*dmats0[4][8] + coeff0_5*dmats0[5][8] + coeff0_6*dmats0[6][8] + coeff0_7*dmats0[7][8] + coeff0_8*dmats0[8][8] + coeff0_9*dmats0[9][8] + coeff0_10*dmats0[10][8] + coeff0_11*dmats0[11][8] + coeff0_12*dmats0[12][8] + coeff0_13*dmats0[13][8] + coeff0_14*dmats0[14][8] + coeff0_15*dmats0[15][8] + coeff0_16*dmats0[16][8] + coeff0_17*dmats0[17][8] + coeff0_18*dmats0[18][8] + coeff0_19*dmats0[19][8] + coeff0_20*dmats0[20][8] + coeff0_21*dmats0[21][8] + coeff0_22*dmats0[22][8] + coeff0_23*dmats0[23][8] + coeff0_24*dmats0[24][8] + coeff0_25*dmats0[25][8] + coeff0_26*dmats0[26][8] + coeff0_27*dmats0[27][8] + coeff0_28*dmats0[28][8] + coeff0_29*dmats0[29][8] + coeff0_30*dmats0[30][8] + coeff0_31*dmats0[31][8] + coeff0_32*dmats0[32][8] + coeff0_33*dmats0[33][8] + coeff0_34*dmats0[34][8] + coeff0_35*dmats0[35][8] + coeff0_36*dmats0[36][8] + coeff0_37*dmats0[37][8] + coeff0_38*dmats0[38][8] + coeff0_39*dmats0[39][8] + coeff0_40*dmats0[40][8] + coeff0_41*dmats0[41][8] + coeff0_42*dmats0[42][8] + coeff0_43*dmats0[43][8] + coeff0_44*dmats0[44][8] + coeff0_45*dmats0[45][8] + coeff0_46*dmats0[46][8] + coeff0_47*dmats0[47][8] + coeff0_48*dmats0[48][8] + coeff0_49*dmats0[49][8] + coeff0_50*dmats0[50][8] + coeff0_51*dmats0[51][8] + coeff0_52*dmats0[52][8] + coeff0_53*dmats0[53][8] + coeff0_54*dmats0[54][8] + coeff0_55*dmats0[55][8];
          new_coeff0_9 = coeff0_0*dmats0[0][9] + coeff0_1*dmats0[1][9] + coeff0_2*dmats0[2][9] + coeff0_3*dmats0[3][9] + coeff0_4*dmats0[4][9] + coeff0_5*dmats0[5][9] + coeff0_6*dmats0[6][9] + coeff0_7*dmats0[7][9] + coeff0_8*dmats0[8][9] + coeff0_9*dmats0[9][9] + coeff0_10*dmats0[10][9] + coeff0_11*dmats0[11][9] + coeff0_12*dmats0[12][9] + coeff0_13*dmats0[13][9] + coeff0_14*dmats0[14][9] + coeff0_15*dmats0[15][9] + coeff0_16*dmats0[16][9] + coeff0_17*dmats0[17][9] + coeff0_18*dmats0[18][9] + coeff0_19*dmats0[19][9] + coeff0_20*dmats0[20][9] + coeff0_21*dmats0[21][9] + coeff0_22*dmats0[22][9] + coeff0_23*dmats0[23][9] + coeff0_24*dmats0[24][9] + coeff0_25*dmats0[25][9] + coeff0_26*dmats0[26][9] + coeff0_27*dmats0[27][9] + coeff0_28*dmats0[28][9] + coeff0_29*dmats0[29][9] + coeff0_30*dmats0[30][9] + coeff0_31*dmats0[31][9] + coeff0_32*dmats0[32][9] + coeff0_33*dmats0[33][9] + coeff0_34*dmats0[34][9] + coeff0_35*dmats0[35][9] + coeff0_36*dmats0[36][9] + coeff0_37*dmats0[37][9] + coeff0_38*dmats0[38][9] + coeff0_39*dmats0[39][9] + coeff0_40*dmats0[40][9] + coeff0_41*dmats0[41][9] + coeff0_42*dmats0[42][9] + coeff0_43*dmats0[43][9] + coeff0_44*dmats0[44][9] + coeff0_45*dmats0[45][9] + coeff0_46*dmats0[46][9] + coeff0_47*dmats0[47][9] + coeff0_48*dmats0[48][9] + coeff0_49*dmats0[49][9] + coeff0_50*dmats0[50][9] + coeff0_51*dmats0[51][9] + coeff0_52*dmats0[52][9] + coeff0_53*dmats0[53][9] + coeff0_54*dmats0[54][9] + coeff0_55*dmats0[55][9];
          new_coeff0_10 = coeff0_0*dmats0[0][10] + coeff0_1*dmats0[1][10] + coeff0_2*dmats0[2][10] + coeff0_3*dmats0[3][10] + coeff0_4*dmats0[4][10] + coeff0_5*dmats0[5][10] + coeff0_6*dmats0[6][10] + coeff0_7*dmats0[7][10] + coeff0_8*dmats0[8][10] + coeff0_9*dmats0[9][10] + coeff0_10*dmats0[10][10] + coeff0_11*dmats0[11][10] + coeff0_12*dmats0[12][10] + coeff0_13*dmats0[13][10] + coeff0_14*dmats0[14][10] + coeff0_15*dmats0[15][10] + coeff0_16*dmats0[16][10] + coeff0_17*dmats0[17][10] + coeff0_18*dmats0[18][10] + coeff0_19*dmats0[19][10] + coeff0_20*dmats0[20][10] + coeff0_21*dmats0[21][10] + coeff0_22*dmats0[22][10] + coeff0_23*dmats0[23][10] + coeff0_24*dmats0[24][10] + coeff0_25*dmats0[25][10] + coeff0_26*dmats0[26][10] + coeff0_27*dmats0[27][10] + coeff0_28*dmats0[28][10] + coeff0_29*dmats0[29][10] + coeff0_30*dmats0[30][10] + coeff0_31*dmats0[31][10] + coeff0_32*dmats0[32][10] + coeff0_33*dmats0[33][10] + coeff0_34*dmats0[34][10] + coeff0_35*dmats0[35][10] + coeff0_36*dmats0[36][10] + coeff0_37*dmats0[37][10] + coeff0_38*dmats0[38][10] + coeff0_39*dmats0[39][10] + coeff0_40*dmats0[40][10] + coeff0_41*dmats0[41][10] + coeff0_42*dmats0[42][10] + coeff0_43*dmats0[43][10] + coeff0_44*dmats0[44][10] + coeff0_45*dmats0[45][10] + coeff0_46*dmats0[46][10] + coeff0_47*dmats0[47][10] + coeff0_48*dmats0[48][10] + coeff0_49*dmats0[49][10] + coeff0_50*dmats0[50][10] + coeff0_51*dmats0[51][10] + coeff0_52*dmats0[52][10] + coeff0_53*dmats0[53][10] + coeff0_54*dmats0[54][10] + coeff0_55*dmats0[55][10];
          new_coeff0_11 = coeff0_0*dmats0[0][11] + coeff0_1*dmats0[1][11] + coeff0_2*dmats0[2][11] + coeff0_3*dmats0[3][11] + coeff0_4*dmats0[4][11] + coeff0_5*dmats0[5][11] + coeff0_6*dmats0[6][11] + coeff0_7*dmats0[7][11] + coeff0_8*dmats0[8][11] + coeff0_9*dmats0[9][11] + coeff0_10*dmats0[10][11] + coeff0_11*dmats0[11][11] + coeff0_12*dmats0[12][11] + coeff0_13*dmats0[13][11] + coeff0_14*dmats0[14][11] + coeff0_15*dmats0[15][11] + coeff0_16*dmats0[16][11] + coeff0_17*dmats0[17][11] + coeff0_18*dmats0[18][11] + coeff0_19*dmats0[19][11] + coeff0_20*dmats0[20][11] + coeff0_21*dmats0[21][11] + coeff0_22*dmats0[22][11] + coeff0_23*dmats0[23][11] + coeff0_24*dmats0[24][11] + coeff0_25*dmats0[25][11] + coeff0_26*dmats0[26][11] + coeff0_27*dmats0[27][11] + coeff0_28*dmats0[28][11] + coeff0_29*dmats0[29][11] + coeff0_30*dmats0[30][11] + coeff0_31*dmats0[31][11] + coeff0_32*dmats0[32][11] + coeff0_33*dmats0[33][11] + coeff0_34*dmats0[34][11] + coeff0_35*dmats0[35][11] + coeff0_36*dmats0[36][11] + coeff0_37*dmats0[37][11] + coeff0_38*dmats0[38][11] + coeff0_39*dmats0[39][11] + coeff0_40*dmats0[40][11] + coeff0_41*dmats0[41][11] + coeff0_42*dmats0[42][11] + coeff0_43*dmats0[43][11] + coeff0_44*dmats0[44][11] + coeff0_45*dmats0[45][11] + coeff0_46*dmats0[46][11] + coeff0_47*dmats0[47][11] + coeff0_48*dmats0[48][11] + coeff0_49*dmats0[49][11] + coeff0_50*dmats0[50][11] + coeff0_51*dmats0[51][11] + coeff0_52*dmats0[52][11] + coeff0_53*dmats0[53][11] + coeff0_54*dmats0[54][11] + coeff0_55*dmats0[55][11];
          new_coeff0_12 = coeff0_0*dmats0[0][12] + coeff0_1*dmats0[1][12] + coeff0_2*dmats0[2][12] + coeff0_3*dmats0[3][12] + coeff0_4*dmats0[4][12] + coeff0_5*dmats0[5][12] + coeff0_6*dmats0[6][12] + coeff0_7*dmats0[7][12] + coeff0_8*dmats0[8][12] + coeff0_9*dmats0[9][12] + coeff0_10*dmats0[10][12] + coeff0_11*dmats0[11][12] + coeff0_12*dmats0[12][12] + coeff0_13*dmats0[13][12] + coeff0_14*dmats0[14][12] + coeff0_15*dmats0[15][12] + coeff0_16*dmats0[16][12] + coeff0_17*dmats0[17][12] + coeff0_18*dmats0[18][12] + coeff0_19*dmats0[19][12] + coeff0_20*dmats0[20][12] + coeff0_21*dmats0[21][12] + coeff0_22*dmats0[22][12] + coeff0_23*dmats0[23][12] + coeff0_24*dmats0[24][12] + coeff0_25*dmats0[25][12] + coeff0_26*dmats0[26][12] + coeff0_27*dmats0[27][12] + coeff0_28*dmats0[28][12] + coeff0_29*dmats0[29][12] + coeff0_30*dmats0[30][12] + coeff0_31*dmats0[31][12] + coeff0_32*dmats0[32][12] + coeff0_33*dmats0[33][12] + coeff0_34*dmats0[34][12] + coeff0_35*dmats0[35][12] + coeff0_36*dmats0[36][12] + coeff0_37*dmats0[37][12] + coeff0_38*dmats0[38][12] + coeff0_39*dmats0[39][12] + coeff0_40*dmats0[40][12] + coeff0_41*dmats0[41][12] + coeff0_42*dmats0[42][12] + coeff0_43*dmats0[43][12] + coeff0_44*dmats0[44][12] + coeff0_45*dmats0[45][12] + coeff0_46*dmats0[46][12] + coeff0_47*dmats0[47][12] + coeff0_48*dmats0[48][12] + coeff0_49*dmats0[49][12] + coeff0_50*dmats0[50][12] + coeff0_51*dmats0[51][12] + coeff0_52*dmats0[52][12] + coeff0_53*dmats0[53][12] + coeff0_54*dmats0[54][12] + coeff0_55*dmats0[55][12];
          new_coeff0_13 = coeff0_0*dmats0[0][13] + coeff0_1*dmats0[1][13] + coeff0_2*dmats0[2][13] + coeff0_3*dmats0[3][13] + coeff0_4*dmats0[4][13] + coeff0_5*dmats0[5][13] + coeff0_6*dmats0[6][13] + coeff0_7*dmats0[7][13] + coeff0_8*dmats0[8][13] + coeff0_9*dmats0[9][13] + coeff0_10*dmats0[10][13] + coeff0_11*dmats0[11][13] + coeff0_12*dmats0[12][13] + coeff0_13*dmats0[13][13] + coeff0_14*dmats0[14][13] + coeff0_15*dmats0[15][13] + coeff0_16*dmats0[16][13] + coeff0_17*dmats0[17][13] + coeff0_18*dmats0[18][13] + coeff0_19*dmats0[19][13] + coeff0_20*dmats0[20][13] + coeff0_21*dmats0[21][13] + coeff0_22*dmats0[22][13] + coeff0_23*dmats0[23][13] + coeff0_24*dmats0[24][13] + coeff0_25*dmats0[25][13] + coeff0_26*dmats0[26][13] + coeff0_27*dmats0[27][13] + coeff0_28*dmats0[28][13] + coeff0_29*dmats0[29][13] + coeff0_30*dmats0[30][13] + coeff0_31*dmats0[31][13] + coeff0_32*dmats0[32][13] + coeff0_33*dmats0[33][13] + coeff0_34*dmats0[34][13] + coeff0_35*dmats0[35][13] + coeff0_36*dmats0[36][13] + coeff0_37*dmats0[37][13] + coeff0_38*dmats0[38][13] + coeff0_39*dmats0[39][13] + coeff0_40*dmats0[40][13] + coeff0_41*dmats0[41][13] + coeff0_42*dmats0[42][13] + coeff0_43*dmats0[43][13] + coeff0_44*dmats0[44][13] + coeff0_45*dmats0[45][13] + coeff0_46*dmats0[46][13] + coeff0_47*dmats0[47][13] + coeff0_48*dmats0[48][13] + coeff0_49*dmats0[49][13] + coeff0_50*dmats0[50][13] + coeff0_51*dmats0[51][13] + coeff0_52*dmats0[52][13] + coeff0_53*dmats0[53][13] + coeff0_54*dmats0[54][13] + coeff0_55*dmats0[55][13];
          new_coeff0_14 = coeff0_0*dmats0[0][14] + coeff0_1*dmats0[1][14] + coeff0_2*dmats0[2][14] + coeff0_3*dmats0[3][14] + coeff0_4*dmats0[4][14] + coeff0_5*dmats0[5][14] + coeff0_6*dmats0[6][14] + coeff0_7*dmats0[7][14] + coeff0_8*dmats0[8][14] + coeff0_9*dmats0[9][14] + coeff0_10*dmats0[10][14] + coeff0_11*dmats0[11][14] + coeff0_12*dmats0[12][14] + coeff0_13*dmats0[13][14] + coeff0_14*dmats0[14][14] + coeff0_15*dmats0[15][14] + coeff0_16*dmats0[16][14] + coeff0_17*dmats0[17][14] + coeff0_18*dmats0[18][14] + coeff0_19*dmats0[19][14] + coeff0_20*dmats0[20][14] + coeff0_21*dmats0[21][14] + coeff0_22*dmats0[22][14] + coeff0_23*dmats0[23][14] + coeff0_24*dmats0[24][14] + coeff0_25*dmats0[25][14] + coeff0_26*dmats0[26][14] + coeff0_27*dmats0[27][14] + coeff0_28*dmats0[28][14] + coeff0_29*dmats0[29][14] + coeff0_30*dmats0[30][14] + coeff0_31*dmats0[31][14] + coeff0_32*dmats0[32][14] + coeff0_33*dmats0[33][14] + coeff0_34*dmats0[34][14] + coeff0_35*dmats0[35][14] + coeff0_36*dmats0[36][14] + coeff0_37*dmats0[37][14] + coeff0_38*dmats0[38][14] + coeff0_39*dmats0[39][14] + coeff0_40*dmats0[40][14] + coeff0_41*dmats0[41][14] + coeff0_42*dmats0[42][14] + coeff0_43*dmats0[43][14] + coeff0_44*dmats0[44][14] + coeff0_45*dmats0[45][14] + coeff0_46*dmats0[46][14] + coeff0_47*dmats0[47][14] + coeff0_48*dmats0[48][14] + coeff0_49*dmats0[49][14] + coeff0_50*dmats0[50][14] + coeff0_51*dmats0[51][14] + coeff0_52*dmats0[52][14] + coeff0_53*dmats0[53][14] + coeff0_54*dmats0[54][14] + coeff0_55*dmats0[55][14];
          new_coeff0_15 = coeff0_0*dmats0[0][15] + coeff0_1*dmats0[1][15] + coeff0_2*dmats0[2][15] + coeff0_3*dmats0[3][15] + coeff0_4*dmats0[4][15] + coeff0_5*dmats0[5][15] + coeff0_6*dmats0[6][15] + coeff0_7*dmats0[7][15] + coeff0_8*dmats0[8][15] + coeff0_9*dmats0[9][15] + coeff0_10*dmats0[10][15] + coeff0_11*dmats0[11][15] + coeff0_12*dmats0[12][15] + coeff0_13*dmats0[13][15] + coeff0_14*dmats0[14][15] + coeff0_15*dmats0[15][15] + coeff0_16*dmats0[16][15] + coeff0_17*dmats0[17][15] + coeff0_18*dmats0[18][15] + coeff0_19*dmats0[19][15] + coeff0_20*dmats0[20][15] + coeff0_21*dmats0[21][15] + coeff0_22*dmats0[22][15] + coeff0_23*dmats0[23][15] + coeff0_24*dmats0[24][15] + coeff0_25*dmats0[25][15] + coeff0_26*dmats0[26][15] + coeff0_27*dmats0[27][15] + coeff0_28*dmats0[28][15] + coeff0_29*dmats0[29][15] + coeff0_30*dmats0[30][15] + coeff0_31*dmats0[31][15] + coeff0_32*dmats0[32][15] + coeff0_33*dmats0[33][15] + coeff0_34*dmats0[34][15] + coeff0_35*dmats0[35][15] + coeff0_36*dmats0[36][15] + coeff0_37*dmats0[37][15] + coeff0_38*dmats0[38][15] + coeff0_39*dmats0[39][15] + coeff0_40*dmats0[40][15] + coeff0_41*dmats0[41][15] + coeff0_42*dmats0[42][15] + coeff0_43*dmats0[43][15] + coeff0_44*dmats0[44][15] + coeff0_45*dmats0[45][15] + coeff0_46*dmats0[46][15] + coeff0_47*dmats0[47][15] + coeff0_48*dmats0[48][15] + coeff0_49*dmats0[49][15] + coeff0_50*dmats0[50][15] + coeff0_51*dmats0[51][15] + coeff0_52*dmats0[52][15] + coeff0_53*dmats0[53][15] + coeff0_54*dmats0[54][15] + coeff0_55*dmats0[55][15];
          new_coeff0_16 = coeff0_0*dmats0[0][16] + coeff0_1*dmats0[1][16] + coeff0_2*dmats0[2][16] + coeff0_3*dmats0[3][16] + coeff0_4*dmats0[4][16] + coeff0_5*dmats0[5][16] + coeff0_6*dmats0[6][16] + coeff0_7*dmats0[7][16] + coeff0_8*dmats0[8][16] + coeff0_9*dmats0[9][16] + coeff0_10*dmats0[10][16] + coeff0_11*dmats0[11][16] + coeff0_12*dmats0[12][16] + coeff0_13*dmats0[13][16] + coeff0_14*dmats0[14][16] + coeff0_15*dmats0[15][16] + coeff0_16*dmats0[16][16] + coeff0_17*dmats0[17][16] + coeff0_18*dmats0[18][16] + coeff0_19*dmats0[19][16] + coeff0_20*dmats0[20][16] + coeff0_21*dmats0[21][16] + coeff0_22*dmats0[22][16] + coeff0_23*dmats0[23][16] + coeff0_24*dmats0[24][16] + coeff0_25*dmats0[25][16] + coeff0_26*dmats0[26][16] + coeff0_27*dmats0[27][16] + coeff0_28*dmats0[28][16] + coeff0_29*dmats0[29][16] + coeff0_30*dmats0[30][16] + coeff0_31*dmats0[31][16] + coeff0_32*dmats0[32][16] + coeff0_33*dmats0[33][16] + coeff0_34*dmats0[34][16] + coeff0_35*dmats0[35][16] + coeff0_36*dmats0[36][16] + coeff0_37*dmats0[37][16] + coeff0_38*dmats0[38][16] + coeff0_39*dmats0[39][16] + coeff0_40*dmats0[40][16] + coeff0_41*dmats0[41][16] + coeff0_42*dmats0[42][16] + coeff0_43*dmats0[43][16] + coeff0_44*dmats0[44][16] + coeff0_45*dmats0[45][16] + coeff0_46*dmats0[46][16] + coeff0_47*dmats0[47][16] + coeff0_48*dmats0[48][16] + coeff0_49*dmats0[49][16] + coeff0_50*dmats0[50][16] + coeff0_51*dmats0[51][16] + coeff0_52*dmats0[52][16] + coeff0_53*dmats0[53][16] + coeff0_54*dmats0[54][16] + coeff0_55*dmats0[55][16];
          new_coeff0_17 = coeff0_0*dmats0[0][17] + coeff0_1*dmats0[1][17] + coeff0_2*dmats0[2][17] + coeff0_3*dmats0[3][17] + coeff0_4*dmats0[4][17] + coeff0_5*dmats0[5][17] + coeff0_6*dmats0[6][17] + coeff0_7*dmats0[7][17] + coeff0_8*dmats0[8][17] + coeff0_9*dmats0[9][17] + coeff0_10*dmats0[10][17] + coeff0_11*dmats0[11][17] + coeff0_12*dmats0[12][17] + coeff0_13*dmats0[13][17] + coeff0_14*dmats0[14][17] + coeff0_15*dmats0[15][17] + coeff0_16*dmats0[16][17] + coeff0_17*dmats0[17][17] + coeff0_18*dmats0[18][17] + coeff0_19*dmats0[19][17] + coeff0_20*dmats0[20][17] + coeff0_21*dmats0[21][17] + coeff0_22*dmats0[22][17] + coeff0_23*dmats0[23][17] + coeff0_24*dmats0[24][17] + coeff0_25*dmats0[25][17] + coeff0_26*dmats0[26][17] + coeff0_27*dmats0[27][17] + coeff0_28*dmats0[28][17] + coeff0_29*dmats0[29][17] + coeff0_30*dmats0[30][17] + coeff0_31*dmats0[31][17] + coeff0_32*dmats0[32][17] + coeff0_33*dmats0[33][17] + coeff0_34*dmats0[34][17] + coeff0_35*dmats0[35][17] + coeff0_36*dmats0[36][17] + coeff0_37*dmats0[37][17] + coeff0_38*dmats0[38][17] + coeff0_39*dmats0[39][17] + coeff0_40*dmats0[40][17] + coeff0_41*dmats0[41][17] + coeff0_42*dmats0[42][17] + coeff0_43*dmats0[43][17] + coeff0_44*dmats0[44][17] + coeff0_45*dmats0[45][17] + coeff0_46*dmats0[46][17] + coeff0_47*dmats0[47][17] + coeff0_48*dmats0[48][17] + coeff0_49*dmats0[49][17] + coeff0_50*dmats0[50][17] + coeff0_51*dmats0[51][17] + coeff0_52*dmats0[52][17] + coeff0_53*dmats0[53][17] + coeff0_54*dmats0[54][17] + coeff0_55*dmats0[55][17];
          new_coeff0_18 = coeff0_0*dmats0[0][18] + coeff0_1*dmats0[1][18] + coeff0_2*dmats0[2][18] + coeff0_3*dmats0[3][18] + coeff0_4*dmats0[4][18] + coeff0_5*dmats0[5][18] + coeff0_6*dmats0[6][18] + coeff0_7*dmats0[7][18] + coeff0_8*dmats0[8][18] + coeff0_9*dmats0[9][18] + coeff0_10*dmats0[10][18] + coeff0_11*dmats0[11][18] + coeff0_12*dmats0[12][18] + coeff0_13*dmats0[13][18] + coeff0_14*dmats0[14][18] + coeff0_15*dmats0[15][18] + coeff0_16*dmats0[16][18] + coeff0_17*dmats0[17][18] + coeff0_18*dmats0[18][18] + coeff0_19*dmats0[19][18] + coeff0_20*dmats0[20][18] + coeff0_21*dmats0[21][18] + coeff0_22*dmats0[22][18] + coeff0_23*dmats0[23][18] + coeff0_24*dmats0[24][18] + coeff0_25*dmats0[25][18] + coeff0_26*dmats0[26][18] + coeff0_27*dmats0[27][18] + coeff0_28*dmats0[28][18] + coeff0_29*dmats0[29][18] + coeff0_30*dmats0[30][18] + coeff0_31*dmats0[31][18] + coeff0_32*dmats0[32][18] + coeff0_33*dmats0[33][18] + coeff0_34*dmats0[34][18] + coeff0_35*dmats0[35][18] + coeff0_36*dmats0[36][18] + coeff0_37*dmats0[37][18] + coeff0_38*dmats0[38][18] + coeff0_39*dmats0[39][18] + coeff0_40*dmats0[40][18] + coeff0_41*dmats0[41][18] + coeff0_42*dmats0[42][18] + coeff0_43*dmats0[43][18] + coeff0_44*dmats0[44][18] + coeff0_45*dmats0[45][18] + coeff0_46*dmats0[46][18] + coeff0_47*dmats0[47][18] + coeff0_48*dmats0[48][18] + coeff0_49*dmats0[49][18] + coeff0_50*dmats0[50][18] + coeff0_51*dmats0[51][18] + coeff0_52*dmats0[52][18] + coeff0_53*dmats0[53][18] + coeff0_54*dmats0[54][18] + coeff0_55*dmats0[55][18];
          new_coeff0_19 = coeff0_0*dmats0[0][19] + coeff0_1*dmats0[1][19] + coeff0_2*dmats0[2][19] + coeff0_3*dmats0[3][19] + coeff0_4*dmats0[4][19] + coeff0_5*dmats0[5][19] + coeff0_6*dmats0[6][19] + coeff0_7*dmats0[7][19] + coeff0_8*dmats0[8][19] + coeff0_9*dmats0[9][19] + coeff0_10*dmats0[10][19] + coeff0_11*dmats0[11][19] + coeff0_12*dmats0[12][19] + coeff0_13*dmats0[13][19] + coeff0_14*dmats0[14][19] + coeff0_15*dmats0[15][19] + coeff0_16*dmats0[16][19] + coeff0_17*dmats0[17][19] + coeff0_18*dmats0[18][19] + coeff0_19*dmats0[19][19] + coeff0_20*dmats0[20][19] + coeff0_21*dmats0[21][19] + coeff0_22*dmats0[22][19] + coeff0_23*dmats0[23][19] + coeff0_24*dmats0[24][19] + coeff0_25*dmats0[25][19] + coeff0_26*dmats0[26][19] + coeff0_27*dmats0[27][19] + coeff0_28*dmats0[28][19] + coeff0_29*dmats0[29][19] + coeff0_30*dmats0[30][19] + coeff0_31*dmats0[31][19] + coeff0_32*dmats0[32][19] + coeff0_33*dmats0[33][19] + coeff0_34*dmats0[34][19] + coeff0_35*dmats0[35][19] + coeff0_36*dmats0[36][19] + coeff0_37*dmats0[37][19] + coeff0_38*dmats0[38][19] + coeff0_39*dmats0[39][19] + coeff0_40*dmats0[40][19] + coeff0_41*dmats0[41][19] + coeff0_42*dmats0[42][19] + coeff0_43*dmats0[43][19] + coeff0_44*dmats0[44][19] + coeff0_45*dmats0[45][19] + coeff0_46*dmats0[46][19] + coeff0_47*dmats0[47][19] + coeff0_48*dmats0[48][19] + coeff0_49*dmats0[49][19] + coeff0_50*dmats0[50][19] + coeff0_51*dmats0[51][19] + coeff0_52*dmats0[52][19] + coeff0_53*dmats0[53][19] + coeff0_54*dmats0[54][19] + coeff0_55*dmats0[55][19];
          new_coeff0_20 = coeff0_0*dmats0[0][20] + coeff0_1*dmats0[1][20] + coeff0_2*dmats0[2][20] + coeff0_3*dmats0[3][20] + coeff0_4*dmats0[4][20] + coeff0_5*dmats0[5][20] + coeff0_6*dmats0[6][20] + coeff0_7*dmats0[7][20] + coeff0_8*dmats0[8][20] + coeff0_9*dmats0[9][20] + coeff0_10*dmats0[10][20] + coeff0_11*dmats0[11][20] + coeff0_12*dmats0[12][20] + coeff0_13*dmats0[13][20] + coeff0_14*dmats0[14][20] + coeff0_15*dmats0[15][20] + coeff0_16*dmats0[16][20] + coeff0_17*dmats0[17][20] + coeff0_18*dmats0[18][20] + coeff0_19*dmats0[19][20] + coeff0_20*dmats0[20][20] + coeff0_21*dmats0[21][20] + coeff0_22*dmats0[22][20] + coeff0_23*dmats0[23][20] + coeff0_24*dmats0[24][20] + coeff0_25*dmats0[25][20] + coeff0_26*dmats0[26][20] + coeff0_27*dmats0[27][20] + coeff0_28*dmats0[28][20] + coeff0_29*dmats0[29][20] + coeff0_30*dmats0[30][20] + coeff0_31*dmats0[31][20] + coeff0_32*dmats0[32][20] + coeff0_33*dmats0[33][20] + coeff0_34*dmats0[34][20] + coeff0_35*dmats0[35][20] + coeff0_36*dmats0[36][20] + coeff0_37*dmats0[37][20] + coeff0_38*dmats0[38][20] + coeff0_39*dmats0[39][20] + coeff0_40*dmats0[40][20] + coeff0_41*dmats0[41][20] + coeff0_42*dmats0[42][20] + coeff0_43*dmats0[43][20] + coeff0_44*dmats0[44][20] + coeff0_45*dmats0[45][20] + coeff0_46*dmats0[46][20] + coeff0_47*dmats0[47][20] + coeff0_48*dmats0[48][20] + coeff0_49*dmats0[49][20] + coeff0_50*dmats0[50][20] + coeff0_51*dmats0[51][20] + coeff0_52*dmats0[52][20] + coeff0_53*dmats0[53][20] + coeff0_54*dmats0[54][20] + coeff0_55*dmats0[55][20];
          new_coeff0_21 = coeff0_0*dmats0[0][21] + coeff0_1*dmats0[1][21] + coeff0_2*dmats0[2][21] + coeff0_3*dmats0[3][21] + coeff0_4*dmats0[4][21] + coeff0_5*dmats0[5][21] + coeff0_6*dmats0[6][21] + coeff0_7*dmats0[7][21] + coeff0_8*dmats0[8][21] + coeff0_9*dmats0[9][21] + coeff0_10*dmats0[10][21] + coeff0_11*dmats0[11][21] + coeff0_12*dmats0[12][21] + coeff0_13*dmats0[13][21] + coeff0_14*dmats0[14][21] + coeff0_15*dmats0[15][21] + coeff0_16*dmats0[16][21] + coeff0_17*dmats0[17][21] + coeff0_18*dmats0[18][21] + coeff0_19*dmats0[19][21] + coeff0_20*dmats0[20][21] + coeff0_21*dmats0[21][21] + coeff0_22*dmats0[22][21] + coeff0_23*dmats0[23][21] + coeff0_24*dmats0[24][21] + coeff0_25*dmats0[25][21] + coeff0_26*dmats0[26][21] + coeff0_27*dmats0[27][21] + coeff0_28*dmats0[28][21] + coeff0_29*dmats0[29][21] + coeff0_30*dmats0[30][21] + coeff0_31*dmats0[31][21] + coeff0_32*dmats0[32][21] + coeff0_33*dmats0[33][21] + coeff0_34*dmats0[34][21] + coeff0_35*dmats0[35][21] + coeff0_36*dmats0[36][21] + coeff0_37*dmats0[37][21] + coeff0_38*dmats0[38][21] + coeff0_39*dmats0[39][21] + coeff0_40*dmats0[40][21] + coeff0_41*dmats0[41][21] + coeff0_42*dmats0[42][21] + coeff0_43*dmats0[43][21] + coeff0_44*dmats0[44][21] + coeff0_45*dmats0[45][21] + coeff0_46*dmats0[46][21] + coeff0_47*dmats0[47][21] + coeff0_48*dmats0[48][21] + coeff0_49*dmats0[49][21] + coeff0_50*dmats0[50][21] + coeff0_51*dmats0[51][21] + coeff0_52*dmats0[52][21] + coeff0_53*dmats0[53][21] + coeff0_54*dmats0[54][21] + coeff0_55*dmats0[55][21];
          new_coeff0_22 = coeff0_0*dmats0[0][22] + coeff0_1*dmats0[1][22] + coeff0_2*dmats0[2][22] + coeff0_3*dmats0[3][22] + coeff0_4*dmats0[4][22] + coeff0_5*dmats0[5][22] + coeff0_6*dmats0[6][22] + coeff0_7*dmats0[7][22] + coeff0_8*dmats0[8][22] + coeff0_9*dmats0[9][22] + coeff0_10*dmats0[10][22] + coeff0_11*dmats0[11][22] + coeff0_12*dmats0[12][22] + coeff0_13*dmats0[13][22] + coeff0_14*dmats0[14][22] + coeff0_15*dmats0[15][22] + coeff0_16*dmats0[16][22] + coeff0_17*dmats0[17][22] + coeff0_18*dmats0[18][22] + coeff0_19*dmats0[19][22] + coeff0_20*dmats0[20][22] + coeff0_21*dmats0[21][22] + coeff0_22*dmats0[22][22] + coeff0_23*dmats0[23][22] + coeff0_24*dmats0[24][22] + coeff0_25*dmats0[25][22] + coeff0_26*dmats0[26][22] + coeff0_27*dmats0[27][22] + coeff0_28*dmats0[28][22] + coeff0_29*dmats0[29][22] + coeff0_30*dmats0[30][22] + coeff0_31*dmats0[31][22] + coeff0_32*dmats0[32][22] + coeff0_33*dmats0[33][22] + coeff0_34*dmats0[34][22] + coeff0_35*dmats0[35][22] + coeff0_36*dmats0[36][22] + coeff0_37*dmats0[37][22] + coeff0_38*dmats0[38][22] + coeff0_39*dmats0[39][22] + coeff0_40*dmats0[40][22] + coeff0_41*dmats0[41][22] + coeff0_42*dmats0[42][22] + coeff0_43*dmats0[43][22] + coeff0_44*dmats0[44][22] + coeff0_45*dmats0[45][22] + coeff0_46*dmats0[46][22] + coeff0_47*dmats0[47][22] + coeff0_48*dmats0[48][22] + coeff0_49*dmats0[49][22] + coeff0_50*dmats0[50][22] + coeff0_51*dmats0[51][22] + coeff0_52*dmats0[52][22] + coeff0_53*dmats0[53][22] + coeff0_54*dmats0[54][22] + coeff0_55*dmats0[55][22];
          new_coeff0_23 = coeff0_0*dmats0[0][23] + coeff0_1*dmats0[1][23] + coeff0_2*dmats0[2][23] + coeff0_3*dmats0[3][23] + coeff0_4*dmats0[4][23] + coeff0_5*dmats0[5][23] + coeff0_6*dmats0[6][23] + coeff0_7*dmats0[7][23] + coeff0_8*dmats0[8][23] + coeff0_9*dmats0[9][23] + coeff0_10*dmats0[10][23] + coeff0_11*dmats0[11][23] + coeff0_12*dmats0[12][23] + coeff0_13*dmats0[13][23] + coeff0_14*dmats0[14][23] + coeff0_15*dmats0[15][23] + coeff0_16*dmats0[16][23] + coeff0_17*dmats0[17][23] + coeff0_18*dmats0[18][23] + coeff0_19*dmats0[19][23] + coeff0_20*dmats0[20][23] + coeff0_21*dmats0[21][23] + coeff0_22*dmats0[22][23] + coeff0_23*dmats0[23][23] + coeff0_24*dmats0[24][23] + coeff0_25*dmats0[25][23] + coeff0_26*dmats0[26][23] + coeff0_27*dmats0[27][23] + coeff0_28*dmats0[28][23] + coeff0_29*dmats0[29][23] + coeff0_30*dmats0[30][23] + coeff0_31*dmats0[31][23] + coeff0_32*dmats0[32][23] + coeff0_33*dmats0[33][23] + coeff0_34*dmats0[34][23] + coeff0_35*dmats0[35][23] + coeff0_36*dmats0[36][23] + coeff0_37*dmats0[37][23] + coeff0_38*dmats0[38][23] + coeff0_39*dmats0[39][23] + coeff0_40*dmats0[40][23] + coeff0_41*dmats0[41][23] + coeff0_42*dmats0[42][23] + coeff0_43*dmats0[43][23] + coeff0_44*dmats0[44][23] + coeff0_45*dmats0[45][23] + coeff0_46*dmats0[46][23] + coeff0_47*dmats0[47][23] + coeff0_48*dmats0[48][23] + coeff0_49*dmats0[49][23] + coeff0_50*dmats0[50][23] + coeff0_51*dmats0[51][23] + coeff0_52*dmats0[52][23] + coeff0_53*dmats0[53][23] + coeff0_54*dmats0[54][23] + coeff0_55*dmats0[55][23];
          new_coeff0_24 = coeff0_0*dmats0[0][24] + coeff0_1*dmats0[1][24] + coeff0_2*dmats0[2][24] + coeff0_3*dmats0[3][24] + coeff0_4*dmats0[4][24] + coeff0_5*dmats0[5][24] + coeff0_6*dmats0[6][24] + coeff0_7*dmats0[7][24] + coeff0_8*dmats0[8][24] + coeff0_9*dmats0[9][24] + coeff0_10*dmats0[10][24] + coeff0_11*dmats0[11][24] + coeff0_12*dmats0[12][24] + coeff0_13*dmats0[13][24] + coeff0_14*dmats0[14][24] + coeff0_15*dmats0[15][24] + coeff0_16*dmats0[16][24] + coeff0_17*dmats0[17][24] + coeff0_18*dmats0[18][24] + coeff0_19*dmats0[19][24] + coeff0_20*dmats0[20][24] + coeff0_21*dmats0[21][24] + coeff0_22*dmats0[22][24] + coeff0_23*dmats0[23][24] + coeff0_24*dmats0[24][24] + coeff0_25*dmats0[25][24] + coeff0_26*dmats0[26][24] + coeff0_27*dmats0[27][24] + coeff0_28*dmats0[28][24] + coeff0_29*dmats0[29][24] + coeff0_30*dmats0[30][24] + coeff0_31*dmats0[31][24] + coeff0_32*dmats0[32][24] + coeff0_33*dmats0[33][24] + coeff0_34*dmats0[34][24] + coeff0_35*dmats0[35][24] + coeff0_36*dmats0[36][24] + coeff0_37*dmats0[37][24] + coeff0_38*dmats0[38][24] + coeff0_39*dmats0[39][24] + coeff0_40*dmats0[40][24] + coeff0_41*dmats0[41][24] + coeff0_42*dmats0[42][24] + coeff0_43*dmats0[43][24] + coeff0_44*dmats0[44][24] + coeff0_45*dmats0[45][24] + coeff0_46*dmats0[46][24] + coeff0_47*dmats0[47][24] + coeff0_48*dmats0[48][24] + coeff0_49*dmats0[49][24] + coeff0_50*dmats0[50][24] + coeff0_51*dmats0[51][24] + coeff0_52*dmats0[52][24] + coeff0_53*dmats0[53][24] + coeff0_54*dmats0[54][24] + coeff0_55*dmats0[55][24];
          new_coeff0_25 = coeff0_0*dmats0[0][25] + coeff0_1*dmats0[1][25] + coeff0_2*dmats0[2][25] + coeff0_3*dmats0[3][25] + coeff0_4*dmats0[4][25] + coeff0_5*dmats0[5][25] + coeff0_6*dmats0[6][25] + coeff0_7*dmats0[7][25] + coeff0_8*dmats0[8][25] + coeff0_9*dmats0[9][25] + coeff0_10*dmats0[10][25] + coeff0_11*dmats0[11][25] + coeff0_12*dmats0[12][25] + coeff0_13*dmats0[13][25] + coeff0_14*dmats0[14][25] + coeff0_15*dmats0[15][25] + coeff0_16*dmats0[16][25] + coeff0_17*dmats0[17][25] + coeff0_18*dmats0[18][25] + coeff0_19*dmats0[19][25] + coeff0_20*dmats0[20][25] + coeff0_21*dmats0[21][25] + coeff0_22*dmats0[22][25] + coeff0_23*dmats0[23][25] + coeff0_24*dmats0[24][25] + coeff0_25*dmats0[25][25] + coeff0_26*dmats0[26][25] + coeff0_27*dmats0[27][25] + coeff0_28*dmats0[28][25] + coeff0_29*dmats0[29][25] + coeff0_30*dmats0[30][25] + coeff0_31*dmats0[31][25] + coeff0_32*dmats0[32][25] + coeff0_33*dmats0[33][25] + coeff0_34*dmats0[34][25] + coeff0_35*dmats0[35][25] + coeff0_36*dmats0[36][25] + coeff0_37*dmats0[37][25] + coeff0_38*dmats0[38][25] + coeff0_39*dmats0[39][25] + coeff0_40*dmats0[40][25] + coeff0_41*dmats0[41][25] + coeff0_42*dmats0[42][25] + coeff0_43*dmats0[43][25] + coeff0_44*dmats0[44][25] + coeff0_45*dmats0[45][25] + coeff0_46*dmats0[46][25] + coeff0_47*dmats0[47][25] + coeff0_48*dmats0[48][25] + coeff0_49*dmats0[49][25] + coeff0_50*dmats0[50][25] + coeff0_51*dmats0[51][25] + coeff0_52*dmats0[52][25] + coeff0_53*dmats0[53][25] + coeff0_54*dmats0[54][25] + coeff0_55*dmats0[55][25];
          new_coeff0_26 = coeff0_0*dmats0[0][26] + coeff0_1*dmats0[1][26] + coeff0_2*dmats0[2][26] + coeff0_3*dmats0[3][26] + coeff0_4*dmats0[4][26] + coeff0_5*dmats0[5][26] + coeff0_6*dmats0[6][26] + coeff0_7*dmats0[7][26] + coeff0_8*dmats0[8][26] + coeff0_9*dmats0[9][26] + coeff0_10*dmats0[10][26] + coeff0_11*dmats0[11][26] + coeff0_12*dmats0[12][26] + coeff0_13*dmats0[13][26] + coeff0_14*dmats0[14][26] + coeff0_15*dmats0[15][26] + coeff0_16*dmats0[16][26] + coeff0_17*dmats0[17][26] + coeff0_18*dmats0[18][26] + coeff0_19*dmats0[19][26] + coeff0_20*dmats0[20][26] + coeff0_21*dmats0[21][26] + coeff0_22*dmats0[22][26] + coeff0_23*dmats0[23][26] + coeff0_24*dmats0[24][26] + coeff0_25*dmats0[25][26] + coeff0_26*dmats0[26][26] + coeff0_27*dmats0[27][26] + coeff0_28*dmats0[28][26] + coeff0_29*dmats0[29][26] + coeff0_30*dmats0[30][26] + coeff0_31*dmats0[31][26] + coeff0_32*dmats0[32][26] + coeff0_33*dmats0[33][26] + coeff0_34*dmats0[34][26] + coeff0_35*dmats0[35][26] + coeff0_36*dmats0[36][26] + coeff0_37*dmats0[37][26] + coeff0_38*dmats0[38][26] + coeff0_39*dmats0[39][26] + coeff0_40*dmats0[40][26] + coeff0_41*dmats0[41][26] + coeff0_42*dmats0[42][26] + coeff0_43*dmats0[43][26] + coeff0_44*dmats0[44][26] + coeff0_45*dmats0[45][26] + coeff0_46*dmats0[46][26] + coeff0_47*dmats0[47][26] + coeff0_48*dmats0[48][26] + coeff0_49*dmats0[49][26] + coeff0_50*dmats0[50][26] + coeff0_51*dmats0[51][26] + coeff0_52*dmats0[52][26] + coeff0_53*dmats0[53][26] + coeff0_54*dmats0[54][26] + coeff0_55*dmats0[55][26];
          new_coeff0_27 = coeff0_0*dmats0[0][27] + coeff0_1*dmats0[1][27] + coeff0_2*dmats0[2][27] + coeff0_3*dmats0[3][27] + coeff0_4*dmats0[4][27] + coeff0_5*dmats0[5][27] + coeff0_6*dmats0[6][27] + coeff0_7*dmats0[7][27] + coeff0_8*dmats0[8][27] + coeff0_9*dmats0[9][27] + coeff0_10*dmats0[10][27] + coeff0_11*dmats0[11][27] + coeff0_12*dmats0[12][27] + coeff0_13*dmats0[13][27] + coeff0_14*dmats0[14][27] + coeff0_15*dmats0[15][27] + coeff0_16*dmats0[16][27] + coeff0_17*dmats0[17][27] + coeff0_18*dmats0[18][27] + coeff0_19*dmats0[19][27] + coeff0_20*dmats0[20][27] + coeff0_21*dmats0[21][27] + coeff0_22*dmats0[22][27] + coeff0_23*dmats0[23][27] + coeff0_24*dmats0[24][27] + coeff0_25*dmats0[25][27] + coeff0_26*dmats0[26][27] + coeff0_27*dmats0[27][27] + coeff0_28*dmats0[28][27] + coeff0_29*dmats0[29][27] + coeff0_30*dmats0[30][27] + coeff0_31*dmats0[31][27] + coeff0_32*dmats0[32][27] + coeff0_33*dmats0[33][27] + coeff0_34*dmats0[34][27] + coeff0_35*dmats0[35][27] + coeff0_36*dmats0[36][27] + coeff0_37*dmats0[37][27] + coeff0_38*dmats0[38][27] + coeff0_39*dmats0[39][27] + coeff0_40*dmats0[40][27] + coeff0_41*dmats0[41][27] + coeff0_42*dmats0[42][27] + coeff0_43*dmats0[43][27] + coeff0_44*dmats0[44][27] + coeff0_45*dmats0[45][27] + coeff0_46*dmats0[46][27] + coeff0_47*dmats0[47][27] + coeff0_48*dmats0[48][27] + coeff0_49*dmats0[49][27] + coeff0_50*dmats0[50][27] + coeff0_51*dmats0[51][27] + coeff0_52*dmats0[52][27] + coeff0_53*dmats0[53][27] + coeff0_54*dmats0[54][27] + coeff0_55*dmats0[55][27];
          new_coeff0_28 = coeff0_0*dmats0[0][28] + coeff0_1*dmats0[1][28] + coeff0_2*dmats0[2][28] + coeff0_3*dmats0[3][28] + coeff0_4*dmats0[4][28] + coeff0_5*dmats0[5][28] + coeff0_6*dmats0[6][28] + coeff0_7*dmats0[7][28] + coeff0_8*dmats0[8][28] + coeff0_9*dmats0[9][28] + coeff0_10*dmats0[10][28] + coeff0_11*dmats0[11][28] + coeff0_12*dmats0[12][28] + coeff0_13*dmats0[13][28] + coeff0_14*dmats0[14][28] + coeff0_15*dmats0[15][28] + coeff0_16*dmats0[16][28] + coeff0_17*dmats0[17][28] + coeff0_18*dmats0[18][28] + coeff0_19*dmats0[19][28] + coeff0_20*dmats0[20][28] + coeff0_21*dmats0[21][28] + coeff0_22*dmats0[22][28] + coeff0_23*dmats0[23][28] + coeff0_24*dmats0[24][28] + coeff0_25*dmats0[25][28] + coeff0_26*dmats0[26][28] + coeff0_27*dmats0[27][28] + coeff0_28*dmats0[28][28] + coeff0_29*dmats0[29][28] + coeff0_30*dmats0[30][28] + coeff0_31*dmats0[31][28] + coeff0_32*dmats0[32][28] + coeff0_33*dmats0[33][28] + coeff0_34*dmats0[34][28] + coeff0_35*dmats0[35][28] + coeff0_36*dmats0[36][28] + coeff0_37*dmats0[37][28] + coeff0_38*dmats0[38][28] + coeff0_39*dmats0[39][28] + coeff0_40*dmats0[40][28] + coeff0_41*dmats0[41][28] + coeff0_42*dmats0[42][28] + coeff0_43*dmats0[43][28] + coeff0_44*dmats0[44][28] + coeff0_45*dmats0[45][28] + coeff0_46*dmats0[46][28] + coeff0_47*dmats0[47][28] + coeff0_48*dmats0[48][28] + coeff0_49*dmats0[49][28] + coeff0_50*dmats0[50][28] + coeff0_51*dmats0[51][28] + coeff0_52*dmats0[52][28] + coeff0_53*dmats0[53][28] + coeff0_54*dmats0[54][28] + coeff0_55*dmats0[55][28];
          new_coeff0_29 = coeff0_0*dmats0[0][29] + coeff0_1*dmats0[1][29] + coeff0_2*dmats0[2][29] + coeff0_3*dmats0[3][29] + coeff0_4*dmats0[4][29] + coeff0_5*dmats0[5][29] + coeff0_6*dmats0[6][29] + coeff0_7*dmats0[7][29] + coeff0_8*dmats0[8][29] + coeff0_9*dmats0[9][29] + coeff0_10*dmats0[10][29] + coeff0_11*dmats0[11][29] + coeff0_12*dmats0[12][29] + coeff0_13*dmats0[13][29] + coeff0_14*dmats0[14][29] + coeff0_15*dmats0[15][29] + coeff0_16*dmats0[16][29] + coeff0_17*dmats0[17][29] + coeff0_18*dmats0[18][29] + coeff0_19*dmats0[19][29] + coeff0_20*dmats0[20][29] + coeff0_21*dmats0[21][29] + coeff0_22*dmats0[22][29] + coeff0_23*dmats0[23][29] + coeff0_24*dmats0[24][29] + coeff0_25*dmats0[25][29] + coeff0_26*dmats0[26][29] + coeff0_27*dmats0[27][29] + coeff0_28*dmats0[28][29] + coeff0_29*dmats0[29][29] + coeff0_30*dmats0[30][29] + coeff0_31*dmats0[31][29] + coeff0_32*dmats0[32][29] + coeff0_33*dmats0[33][29] + coeff0_34*dmats0[34][29] + coeff0_35*dmats0[35][29] + coeff0_36*dmats0[36][29] + coeff0_37*dmats0[37][29] + coeff0_38*dmats0[38][29] + coeff0_39*dmats0[39][29] + coeff0_40*dmats0[40][29] + coeff0_41*dmats0[41][29] + coeff0_42*dmats0[42][29] + coeff0_43*dmats0[43][29] + coeff0_44*dmats0[44][29] + coeff0_45*dmats0[45][29] + coeff0_46*dmats0[46][29] + coeff0_47*dmats0[47][29] + coeff0_48*dmats0[48][29] + coeff0_49*dmats0[49][29] + coeff0_50*dmats0[50][29] + coeff0_51*dmats0[51][29] + coeff0_52*dmats0[52][29] + coeff0_53*dmats0[53][29] + coeff0_54*dmats0[54][29] + coeff0_55*dmats0[55][29];
          new_coeff0_30 = coeff0_0*dmats0[0][30] + coeff0_1*dmats0[1][30] + coeff0_2*dmats0[2][30] + coeff0_3*dmats0[3][30] + coeff0_4*dmats0[4][30] + coeff0_5*dmats0[5][30] + coeff0_6*dmats0[6][30] + coeff0_7*dmats0[7][30] + coeff0_8*dmats0[8][30] + coeff0_9*dmats0[9][30] + coeff0_10*dmats0[10][30] + coeff0_11*dmats0[11][30] + coeff0_12*dmats0[12][30] + coeff0_13*dmats0[13][30] + coeff0_14*dmats0[14][30] + coeff0_15*dmats0[15][30] + coeff0_16*dmats0[16][30] + coeff0_17*dmats0[17][30] + coeff0_18*dmats0[18][30] + coeff0_19*dmats0[19][30] + coeff0_20*dmats0[20][30] + coeff0_21*dmats0[21][30] + coeff0_22*dmats0[22][30] + coeff0_23*dmats0[23][30] + coeff0_24*dmats0[24][30] + coeff0_25*dmats0[25][30] + coeff0_26*dmats0[26][30] + coeff0_27*dmats0[27][30] + coeff0_28*dmats0[28][30] + coeff0_29*dmats0[29][30] + coeff0_30*dmats0[30][30] + coeff0_31*dmats0[31][30] + coeff0_32*dmats0[32][30] + coeff0_33*dmats0[33][30] + coeff0_34*dmats0[34][30] + coeff0_35*dmats0[35][30] + coeff0_36*dmats0[36][30] + coeff0_37*dmats0[37][30] + coeff0_38*dmats0[38][30] + coeff0_39*dmats0[39][30] + coeff0_40*dmats0[40][30] + coeff0_41*dmats0[41][30] + coeff0_42*dmats0[42][30] + coeff0_43*dmats0[43][30] + coeff0_44*dmats0[44][30] + coeff0_45*dmats0[45][30] + coeff0_46*dmats0[46][30] + coeff0_47*dmats0[47][30] + coeff0_48*dmats0[48][30] + coeff0_49*dmats0[49][30] + coeff0_50*dmats0[50][30] + coeff0_51*dmats0[51][30] + coeff0_52*dmats0[52][30] + coeff0_53*dmats0[53][30] + coeff0_54*dmats0[54][30] + coeff0_55*dmats0[55][30];
          new_coeff0_31 = coeff0_0*dmats0[0][31] + coeff0_1*dmats0[1][31] + coeff0_2*dmats0[2][31] + coeff0_3*dmats0[3][31] + coeff0_4*dmats0[4][31] + coeff0_5*dmats0[5][31] + coeff0_6*dmats0[6][31] + coeff0_7*dmats0[7][31] + coeff0_8*dmats0[8][31] + coeff0_9*dmats0[9][31] + coeff0_10*dmats0[10][31] + coeff0_11*dmats0[11][31] + coeff0_12*dmats0[12][31] + coeff0_13*dmats0[13][31] + coeff0_14*dmats0[14][31] + coeff0_15*dmats0[15][31] + coeff0_16*dmats0[16][31] + coeff0_17*dmats0[17][31] + coeff0_18*dmats0[18][31] + coeff0_19*dmats0[19][31] + coeff0_20*dmats0[20][31] + coeff0_21*dmats0[21][31] + coeff0_22*dmats0[22][31] + coeff0_23*dmats0[23][31] + coeff0_24*dmats0[24][31] + coeff0_25*dmats0[25][31] + coeff0_26*dmats0[26][31] + coeff0_27*dmats0[27][31] + coeff0_28*dmats0[28][31] + coeff0_29*dmats0[29][31] + coeff0_30*dmats0[30][31] + coeff0_31*dmats0[31][31] + coeff0_32*dmats0[32][31] + coeff0_33*dmats0[33][31] + coeff0_34*dmats0[34][31] + coeff0_35*dmats0[35][31] + coeff0_36*dmats0[36][31] + coeff0_37*dmats0[37][31] + coeff0_38*dmats0[38][31] + coeff0_39*dmats0[39][31] + coeff0_40*dmats0[40][31] + coeff0_41*dmats0[41][31] + coeff0_42*dmats0[42][31] + coeff0_43*dmats0[43][31] + coeff0_44*dmats0[44][31] + coeff0_45*dmats0[45][31] + coeff0_46*dmats0[46][31] + coeff0_47*dmats0[47][31] + coeff0_48*dmats0[48][31] + coeff0_49*dmats0[49][31] + coeff0_50*dmats0[50][31] + coeff0_51*dmats0[51][31] + coeff0_52*dmats0[52][31] + coeff0_53*dmats0[53][31] + coeff0_54*dmats0[54][31] + coeff0_55*dmats0[55][31];
          new_coeff0_32 = coeff0_0*dmats0[0][32] + coeff0_1*dmats0[1][32] + coeff0_2*dmats0[2][32] + coeff0_3*dmats0[3][32] + coeff0_4*dmats0[4][32] + coeff0_5*dmats0[5][32] + coeff0_6*dmats0[6][32] + coeff0_7*dmats0[7][32] + coeff0_8*dmats0[8][32] + coeff0_9*dmats0[9][32] + coeff0_10*dmats0[10][32] + coeff0_11*dmats0[11][32] + coeff0_12*dmats0[12][32] + coeff0_13*dmats0[13][32] + coeff0_14*dmats0[14][32] + coeff0_15*dmats0[15][32] + coeff0_16*dmats0[16][32] + coeff0_17*dmats0[17][32] + coeff0_18*dmats0[18][32] + coeff0_19*dmats0[19][32] + coeff0_20*dmats0[20][32] + coeff0_21*dmats0[21][32] + coeff0_22*dmats0[22][32] + coeff0_23*dmats0[23][32] + coeff0_24*dmats0[24][32] + coeff0_25*dmats0[25][32] + coeff0_26*dmats0[26][32] + coeff0_27*dmats0[27][32] + coeff0_28*dmats0[28][32] + coeff0_29*dmats0[29][32] + coeff0_30*dmats0[30][32] + coeff0_31*dmats0[31][32] + coeff0_32*dmats0[32][32] + coeff0_33*dmats0[33][32] + coeff0_34*dmats0[34][32] + coeff0_35*dmats0[35][32] + coeff0_36*dmats0[36][32] + coeff0_37*dmats0[37][32] + coeff0_38*dmats0[38][32] + coeff0_39*dmats0[39][32] + coeff0_40*dmats0[40][32] + coeff0_41*dmats0[41][32] + coeff0_42*dmats0[42][32] + coeff0_43*dmats0[43][32] + coeff0_44*dmats0[44][32] + coeff0_45*dmats0[45][32] + coeff0_46*dmats0[46][32] + coeff0_47*dmats0[47][32] + coeff0_48*dmats0[48][32] + coeff0_49*dmats0[49][32] + coeff0_50*dmats0[50][32] + coeff0_51*dmats0[51][32] + coeff0_52*dmats0[52][32] + coeff0_53*dmats0[53][32] + coeff0_54*dmats0[54][32] + coeff0_55*dmats0[55][32];
          new_coeff0_33 = coeff0_0*dmats0[0][33] + coeff0_1*dmats0[1][33] + coeff0_2*dmats0[2][33] + coeff0_3*dmats0[3][33] + coeff0_4*dmats0[4][33] + coeff0_5*dmats0[5][33] + coeff0_6*dmats0[6][33] + coeff0_7*dmats0[7][33] + coeff0_8*dmats0[8][33] + coeff0_9*dmats0[9][33] + coeff0_10*dmats0[10][33] + coeff0_11*dmats0[11][33] + coeff0_12*dmats0[12][33] + coeff0_13*dmats0[13][33] + coeff0_14*dmats0[14][33] + coeff0_15*dmats0[15][33] + coeff0_16*dmats0[16][33] + coeff0_17*dmats0[17][33] + coeff0_18*dmats0[18][33] + coeff0_19*dmats0[19][33] + coeff0_20*dmats0[20][33] + coeff0_21*dmats0[21][33] + coeff0_22*dmats0[22][33] + coeff0_23*dmats0[23][33] + coeff0_24*dmats0[24][33] + coeff0_25*dmats0[25][33] + coeff0_26*dmats0[26][33] + coeff0_27*dmats0[27][33] + coeff0_28*dmats0[28][33] + coeff0_29*dmats0[29][33] + coeff0_30*dmats0[30][33] + coeff0_31*dmats0[31][33] + coeff0_32*dmats0[32][33] + coeff0_33*dmats0[33][33] + coeff0_34*dmats0[34][33] + coeff0_35*dmats0[35][33] + coeff0_36*dmats0[36][33] + coeff0_37*dmats0[37][33] + coeff0_38*dmats0[38][33] + coeff0_39*dmats0[39][33] + coeff0_40*dmats0[40][33] + coeff0_41*dmats0[41][33] + coeff0_42*dmats0[42][33] + coeff0_43*dmats0[43][33] + coeff0_44*dmats0[44][33] + coeff0_45*dmats0[45][33] + coeff0_46*dmats0[46][33] + coeff0_47*dmats0[47][33] + coeff0_48*dmats0[48][33] + coeff0_49*dmats0[49][33] + coeff0_50*dmats0[50][33] + coeff0_51*dmats0[51][33] + coeff0_52*dmats0[52][33] + coeff0_53*dmats0[53][33] + coeff0_54*dmats0[54][33] + coeff0_55*dmats0[55][33];
          new_coeff0_34 = coeff0_0*dmats0[0][34] + coeff0_1*dmats0[1][34] + coeff0_2*dmats0[2][34] + coeff0_3*dmats0[3][34] + coeff0_4*dmats0[4][34] + coeff0_5*dmats0[5][34] + coeff0_6*dmats0[6][34] + coeff0_7*dmats0[7][34] + coeff0_8*dmats0[8][34] + coeff0_9*dmats0[9][34] + coeff0_10*dmats0[10][34] + coeff0_11*dmats0[11][34] + coeff0_12*dmats0[12][34] + coeff0_13*dmats0[13][34] + coeff0_14*dmats0[14][34] + coeff0_15*dmats0[15][34] + coeff0_16*dmats0[16][34] + coeff0_17*dmats0[17][34] + coeff0_18*dmats0[18][34] + coeff0_19*dmats0[19][34] + coeff0_20*dmats0[20][34] + coeff0_21*dmats0[21][34] + coeff0_22*dmats0[22][34] + coeff0_23*dmats0[23][34] + coeff0_24*dmats0[24][34] + coeff0_25*dmats0[25][34] + coeff0_26*dmats0[26][34] + coeff0_27*dmats0[27][34] + coeff0_28*dmats0[28][34] + coeff0_29*dmats0[29][34] + coeff0_30*dmats0[30][34] + coeff0_31*dmats0[31][34] + coeff0_32*dmats0[32][34] + coeff0_33*dmats0[33][34] + coeff0_34*dmats0[34][34] + coeff0_35*dmats0[35][34] + coeff0_36*dmats0[36][34] + coeff0_37*dmats0[37][34] + coeff0_38*dmats0[38][34] + coeff0_39*dmats0[39][34] + coeff0_40*dmats0[40][34] + coeff0_41*dmats0[41][34] + coeff0_42*dmats0[42][34] + coeff0_43*dmats0[43][34] + coeff0_44*dmats0[44][34] + coeff0_45*dmats0[45][34] + coeff0_46*dmats0[46][34] + coeff0_47*dmats0[47][34] + coeff0_48*dmats0[48][34] + coeff0_49*dmats0[49][34] + coeff0_50*dmats0[50][34] + coeff0_51*dmats0[51][34] + coeff0_52*dmats0[52][34] + coeff0_53*dmats0[53][34] + coeff0_54*dmats0[54][34] + coeff0_55*dmats0[55][34];
          new_coeff0_35 = coeff0_0*dmats0[0][35] + coeff0_1*dmats0[1][35] + coeff0_2*dmats0[2][35] + coeff0_3*dmats0[3][35] + coeff0_4*dmats0[4][35] + coeff0_5*dmats0[5][35] + coeff0_6*dmats0[6][35] + coeff0_7*dmats0[7][35] + coeff0_8*dmats0[8][35] + coeff0_9*dmats0[9][35] + coeff0_10*dmats0[10][35] + coeff0_11*dmats0[11][35] + coeff0_12*dmats0[12][35] + coeff0_13*dmats0[13][35] + coeff0_14*dmats0[14][35] + coeff0_15*dmats0[15][35] + coeff0_16*dmats0[16][35] + coeff0_17*dmats0[17][35] + coeff0_18*dmats0[18][35] + coeff0_19*dmats0[19][35] + coeff0_20*dmats0[20][35] + coeff0_21*dmats0[21][35] + coeff0_22*dmats0[22][35] + coeff0_23*dmats0[23][35] + coeff0_24*dmats0[24][35] + coeff0_25*dmats0[25][35] + coeff0_26*dmats0[26][35] + coeff0_27*dmats0[27][35] + coeff0_28*dmats0[28][35] + coeff0_29*dmats0[29][35] + coeff0_30*dmats0[30][35] + coeff0_31*dmats0[31][35] + coeff0_32*dmats0[32][35] + coeff0_33*dmats0[33][35] + coeff0_34*dmats0[34][35] + coeff0_35*dmats0[35][35] + coeff0_36*dmats0[36][35] + coeff0_37*dmats0[37][35] + coeff0_38*dmats0[38][35] + coeff0_39*dmats0[39][35] + coeff0_40*dmats0[40][35] + coeff0_41*dmats0[41][35] + coeff0_42*dmats0[42][35] + coeff0_43*dmats0[43][35] + coeff0_44*dmats0[44][35] + coeff0_45*dmats0[45][35] + coeff0_46*dmats0[46][35] + coeff0_47*dmats0[47][35] + coeff0_48*dmats0[48][35] + coeff0_49*dmats0[49][35] + coeff0_50*dmats0[50][35] + coeff0_51*dmats0[51][35] + coeff0_52*dmats0[52][35] + coeff0_53*dmats0[53][35] + coeff0_54*dmats0[54][35] + coeff0_55*dmats0[55][35];
          new_coeff0_36 = coeff0_0*dmats0[0][36] + coeff0_1*dmats0[1][36] + coeff0_2*dmats0[2][36] + coeff0_3*dmats0[3][36] + coeff0_4*dmats0[4][36] + coeff0_5*dmats0[5][36] + coeff0_6*dmats0[6][36] + coeff0_7*dmats0[7][36] + coeff0_8*dmats0[8][36] + coeff0_9*dmats0[9][36] + coeff0_10*dmats0[10][36] + coeff0_11*dmats0[11][36] + coeff0_12*dmats0[12][36] + coeff0_13*dmats0[13][36] + coeff0_14*dmats0[14][36] + coeff0_15*dmats0[15][36] + coeff0_16*dmats0[16][36] + coeff0_17*dmats0[17][36] + coeff0_18*dmats0[18][36] + coeff0_19*dmats0[19][36] + coeff0_20*dmats0[20][36] + coeff0_21*dmats0[21][36] + coeff0_22*dmats0[22][36] + coeff0_23*dmats0[23][36] + coeff0_24*dmats0[24][36] + coeff0_25*dmats0[25][36] + coeff0_26*dmats0[26][36] + coeff0_27*dmats0[27][36] + coeff0_28*dmats0[28][36] + coeff0_29*dmats0[29][36] + coeff0_30*dmats0[30][36] + coeff0_31*dmats0[31][36] + coeff0_32*dmats0[32][36] + coeff0_33*dmats0[33][36] + coeff0_34*dmats0[34][36] + coeff0_35*dmats0[35][36] + coeff0_36*dmats0[36][36] + coeff0_37*dmats0[37][36] + coeff0_38*dmats0[38][36] + coeff0_39*dmats0[39][36] + coeff0_40*dmats0[40][36] + coeff0_41*dmats0[41][36] + coeff0_42*dmats0[42][36] + coeff0_43*dmats0[43][36] + coeff0_44*dmats0[44][36] + coeff0_45*dmats0[45][36] + coeff0_46*dmats0[46][36] + coeff0_47*dmats0[47][36] + coeff0_48*dmats0[48][36] + coeff0_49*dmats0[49][36] + coeff0_50*dmats0[50][36] + coeff0_51*dmats0[51][36] + coeff0_52*dmats0[52][36] + coeff0_53*dmats0[53][36] + coeff0_54*dmats0[54][36] + coeff0_55*dmats0[55][36];
          new_coeff0_37 = coeff0_0*dmats0[0][37] + coeff0_1*dmats0[1][37] + coeff0_2*dmats0[2][37] + coeff0_3*dmats0[3][37] + coeff0_4*dmats0[4][37] + coeff0_5*dmats0[5][37] + coeff0_6*dmats0[6][37] + coeff0_7*dmats0[7][37] + coeff0_8*dmats0[8][37] + coeff0_9*dmats0[9][37] + coeff0_10*dmats0[10][37] + coeff0_11*dmats0[11][37] + coeff0_12*dmats0[12][37] + coeff0_13*dmats0[13][37] + coeff0_14*dmats0[14][37] + coeff0_15*dmats0[15][37] + coeff0_16*dmats0[16][37] + coeff0_17*dmats0[17][37] + coeff0_18*dmats0[18][37] + coeff0_19*dmats0[19][37] + coeff0_20*dmats0[20][37] + coeff0_21*dmats0[21][37] + coeff0_22*dmats0[22][37] + coeff0_23*dmats0[23][37] + coeff0_24*dmats0[24][37] + coeff0_25*dmats0[25][37] + coeff0_26*dmats0[26][37] + coeff0_27*dmats0[27][37] + coeff0_28*dmats0[28][37] + coeff0_29*dmats0[29][37] + coeff0_30*dmats0[30][37] + coeff0_31*dmats0[31][37] + coeff0_32*dmats0[32][37] + coeff0_33*dmats0[33][37] + coeff0_34*dmats0[34][37] + coeff0_35*dmats0[35][37] + coeff0_36*dmats0[36][37] + coeff0_37*dmats0[37][37] + coeff0_38*dmats0[38][37] + coeff0_39*dmats0[39][37] + coeff0_40*dmats0[40][37] + coeff0_41*dmats0[41][37] + coeff0_42*dmats0[42][37] + coeff0_43*dmats0[43][37] + coeff0_44*dmats0[44][37] + coeff0_45*dmats0[45][37] + coeff0_46*dmats0[46][37] + coeff0_47*dmats0[47][37] + coeff0_48*dmats0[48][37] + coeff0_49*dmats0[49][37] + coeff0_50*dmats0[50][37] + coeff0_51*dmats0[51][37] + coeff0_52*dmats0[52][37] + coeff0_53*dmats0[53][37] + coeff0_54*dmats0[54][37] + coeff0_55*dmats0[55][37];
          new_coeff0_38 = coeff0_0*dmats0[0][38] + coeff0_1*dmats0[1][38] + coeff0_2*dmats0[2][38] + coeff0_3*dmats0[3][38] + coeff0_4*dmats0[4][38] + coeff0_5*dmats0[5][38] + coeff0_6*dmats0[6][38] + coeff0_7*dmats0[7][38] + coeff0_8*dmats0[8][38] + coeff0_9*dmats0[9][38] + coeff0_10*dmats0[10][38] + coeff0_11*dmats0[11][38] + coeff0_12*dmats0[12][38] + coeff0_13*dmats0[13][38] + coeff0_14*dmats0[14][38] + coeff0_15*dmats0[15][38] + coeff0_16*dmats0[16][38] + coeff0_17*dmats0[17][38] + coeff0_18*dmats0[18][38] + coeff0_19*dmats0[19][38] + coeff0_20*dmats0[20][38] + coeff0_21*dmats0[21][38] + coeff0_22*dmats0[22][38] + coeff0_23*dmats0[23][38] + coeff0_24*dmats0[24][38] + coeff0_25*dmats0[25][38] + coeff0_26*dmats0[26][38] + coeff0_27*dmats0[27][38] + coeff0_28*dmats0[28][38] + coeff0_29*dmats0[29][38] + coeff0_30*dmats0[30][38] + coeff0_31*dmats0[31][38] + coeff0_32*dmats0[32][38] + coeff0_33*dmats0[33][38] + coeff0_34*dmats0[34][38] + coeff0_35*dmats0[35][38] + coeff0_36*dmats0[36][38] + coeff0_37*dmats0[37][38] + coeff0_38*dmats0[38][38] + coeff0_39*dmats0[39][38] + coeff0_40*dmats0[40][38] + coeff0_41*dmats0[41][38] + coeff0_42*dmats0[42][38] + coeff0_43*dmats0[43][38] + coeff0_44*dmats0[44][38] + coeff0_45*dmats0[45][38] + coeff0_46*dmats0[46][38] + coeff0_47*dmats0[47][38] + coeff0_48*dmats0[48][38] + coeff0_49*dmats0[49][38] + coeff0_50*dmats0[50][38] + coeff0_51*dmats0[51][38] + coeff0_52*dmats0[52][38] + coeff0_53*dmats0[53][38] + coeff0_54*dmats0[54][38] + coeff0_55*dmats0[55][38];
          new_coeff0_39 = coeff0_0*dmats0[0][39] + coeff0_1*dmats0[1][39] + coeff0_2*dmats0[2][39] + coeff0_3*dmats0[3][39] + coeff0_4*dmats0[4][39] + coeff0_5*dmats0[5][39] + coeff0_6*dmats0[6][39] + coeff0_7*dmats0[7][39] + coeff0_8*dmats0[8][39] + coeff0_9*dmats0[9][39] + coeff0_10*dmats0[10][39] + coeff0_11*dmats0[11][39] + coeff0_12*dmats0[12][39] + coeff0_13*dmats0[13][39] + coeff0_14*dmats0[14][39] + coeff0_15*dmats0[15][39] + coeff0_16*dmats0[16][39] + coeff0_17*dmats0[17][39] + coeff0_18*dmats0[18][39] + coeff0_19*dmats0[19][39] + coeff0_20*dmats0[20][39] + coeff0_21*dmats0[21][39] + coeff0_22*dmats0[22][39] + coeff0_23*dmats0[23][39] + coeff0_24*dmats0[24][39] + coeff0_25*dmats0[25][39] + coeff0_26*dmats0[26][39] + coeff0_27*dmats0[27][39] + coeff0_28*dmats0[28][39] + coeff0_29*dmats0[29][39] + coeff0_30*dmats0[30][39] + coeff0_31*dmats0[31][39] + coeff0_32*dmats0[32][39] + coeff0_33*dmats0[33][39] + coeff0_34*dmats0[34][39] + coeff0_35*dmats0[35][39] + coeff0_36*dmats0[36][39] + coeff0_37*dmats0[37][39] + coeff0_38*dmats0[38][39] + coeff0_39*dmats0[39][39] + coeff0_40*dmats0[40][39] + coeff0_41*dmats0[41][39] + coeff0_42*dmats0[42][39] + coeff0_43*dmats0[43][39] + coeff0_44*dmats0[44][39] + coeff0_45*dmats0[45][39] + coeff0_46*dmats0[46][39] + coeff0_47*dmats0[47][39] + coeff0_48*dmats0[48][39] + coeff0_49*dmats0[49][39] + coeff0_50*dmats0[50][39] + coeff0_51*dmats0[51][39] + coeff0_52*dmats0[52][39] + coeff0_53*dmats0[53][39] + coeff0_54*dmats0[54][39] + coeff0_55*dmats0[55][39];
          new_coeff0_40 = coeff0_0*dmats0[0][40] + coeff0_1*dmats0[1][40] + coeff0_2*dmats0[2][40] + coeff0_3*dmats0[3][40] + coeff0_4*dmats0[4][40] + coeff0_5*dmats0[5][40] + coeff0_6*dmats0[6][40] + coeff0_7*dmats0[7][40] + coeff0_8*dmats0[8][40] + coeff0_9*dmats0[9][40] + coeff0_10*dmats0[10][40] + coeff0_11*dmats0[11][40] + coeff0_12*dmats0[12][40] + coeff0_13*dmats0[13][40] + coeff0_14*dmats0[14][40] + coeff0_15*dmats0[15][40] + coeff0_16*dmats0[16][40] + coeff0_17*dmats0[17][40] + coeff0_18*dmats0[18][40] + coeff0_19*dmats0[19][40] + coeff0_20*dmats0[20][40] + coeff0_21*dmats0[21][40] + coeff0_22*dmats0[22][40] + coeff0_23*dmats0[23][40] + coeff0_24*dmats0[24][40] + coeff0_25*dmats0[25][40] + coeff0_26*dmats0[26][40] + coeff0_27*dmats0[27][40] + coeff0_28*dmats0[28][40] + coeff0_29*dmats0[29][40] + coeff0_30*dmats0[30][40] + coeff0_31*dmats0[31][40] + coeff0_32*dmats0[32][40] + coeff0_33*dmats0[33][40] + coeff0_34*dmats0[34][40] + coeff0_35*dmats0[35][40] + coeff0_36*dmats0[36][40] + coeff0_37*dmats0[37][40] + coeff0_38*dmats0[38][40] + coeff0_39*dmats0[39][40] + coeff0_40*dmats0[40][40] + coeff0_41*dmats0[41][40] + coeff0_42*dmats0[42][40] + coeff0_43*dmats0[43][40] + coeff0_44*dmats0[44][40] + coeff0_45*dmats0[45][40] + coeff0_46*dmats0[46][40] + coeff0_47*dmats0[47][40] + coeff0_48*dmats0[48][40] + coeff0_49*dmats0[49][40] + coeff0_50*dmats0[50][40] + coeff0_51*dmats0[51][40] + coeff0_52*dmats0[52][40] + coeff0_53*dmats0[53][40] + coeff0_54*dmats0[54][40] + coeff0_55*dmats0[55][40];
          new_coeff0_41 = coeff0_0*dmats0[0][41] + coeff0_1*dmats0[1][41] + coeff0_2*dmats0[2][41] + coeff0_3*dmats0[3][41] + coeff0_4*dmats0[4][41] + coeff0_5*dmats0[5][41] + coeff0_6*dmats0[6][41] + coeff0_7*dmats0[7][41] + coeff0_8*dmats0[8][41] + coeff0_9*dmats0[9][41] + coeff0_10*dmats0[10][41] + coeff0_11*dmats0[11][41] + coeff0_12*dmats0[12][41] + coeff0_13*dmats0[13][41] + coeff0_14*dmats0[14][41] + coeff0_15*dmats0[15][41] + coeff0_16*dmats0[16][41] + coeff0_17*dmats0[17][41] + coeff0_18*dmats0[18][41] + coeff0_19*dmats0[19][41] + coeff0_20*dmats0[20][41] + coeff0_21*dmats0[21][41] + coeff0_22*dmats0[22][41] + coeff0_23*dmats0[23][41] + coeff0_24*dmats0[24][41] + coeff0_25*dmats0[25][41] + coeff0_26*dmats0[26][41] + coeff0_27*dmats0[27][41] + coeff0_28*dmats0[28][41] + coeff0_29*dmats0[29][41] + coeff0_30*dmats0[30][41] + coeff0_31*dmats0[31][41] + coeff0_32*dmats0[32][41] + coeff0_33*dmats0[33][41] + coeff0_34*dmats0[34][41] + coeff0_35*dmats0[35][41] + coeff0_36*dmats0[36][41] + coeff0_37*dmats0[37][41] + coeff0_38*dmats0[38][41] + coeff0_39*dmats0[39][41] + coeff0_40*dmats0[40][41] + coeff0_41*dmats0[41][41] + coeff0_42*dmats0[42][41] + coeff0_43*dmats0[43][41] + coeff0_44*dmats0[44][41] + coeff0_45*dmats0[45][41] + coeff0_46*dmats0[46][41] + coeff0_47*dmats0[47][41] + coeff0_48*dmats0[48][41] + coeff0_49*dmats0[49][41] + coeff0_50*dmats0[50][41] + coeff0_51*dmats0[51][41] + coeff0_52*dmats0[52][41] + coeff0_53*dmats0[53][41] + coeff0_54*dmats0[54][41] + coeff0_55*dmats0[55][41];
          new_coeff0_42 = coeff0_0*dmats0[0][42] + coeff0_1*dmats0[1][42] + coeff0_2*dmats0[2][42] + coeff0_3*dmats0[3][42] + coeff0_4*dmats0[4][42] + coeff0_5*dmats0[5][42] + coeff0_6*dmats0[6][42] + coeff0_7*dmats0[7][42] + coeff0_8*dmats0[8][42] + coeff0_9*dmats0[9][42] + coeff0_10*dmats0[10][42] + coeff0_11*dmats0[11][42] + coeff0_12*dmats0[12][42] + coeff0_13*dmats0[13][42] + coeff0_14*dmats0[14][42] + coeff0_15*dmats0[15][42] + coeff0_16*dmats0[16][42] + coeff0_17*dmats0[17][42] + coeff0_18*dmats0[18][42] + coeff0_19*dmats0[19][42] + coeff0_20*dmats0[20][42] + coeff0_21*dmats0[21][42] + coeff0_22*dmats0[22][42] + coeff0_23*dmats0[23][42] + coeff0_24*dmats0[24][42] + coeff0_25*dmats0[25][42] + coeff0_26*dmats0[26][42] + coeff0_27*dmats0[27][42] + coeff0_28*dmats0[28][42] + coeff0_29*dmats0[29][42] + coeff0_30*dmats0[30][42] + coeff0_31*dmats0[31][42] + coeff0_32*dmats0[32][42] + coeff0_33*dmats0[33][42] + coeff0_34*dmats0[34][42] + coeff0_35*dmats0[35][42] + coeff0_36*dmats0[36][42] + coeff0_37*dmats0[37][42] + coeff0_38*dmats0[38][42] + coeff0_39*dmats0[39][42] + coeff0_40*dmats0[40][42] + coeff0_41*dmats0[41][42] + coeff0_42*dmats0[42][42] + coeff0_43*dmats0[43][42] + coeff0_44*dmats0[44][42] + coeff0_45*dmats0[45][42] + coeff0_46*dmats0[46][42] + coeff0_47*dmats0[47][42] + coeff0_48*dmats0[48][42] + coeff0_49*dmats0[49][42] + coeff0_50*dmats0[50][42] + coeff0_51*dmats0[51][42] + coeff0_52*dmats0[52][42] + coeff0_53*dmats0[53][42] + coeff0_54*dmats0[54][42] + coeff0_55*dmats0[55][42];
          new_coeff0_43 = coeff0_0*dmats0[0][43] + coeff0_1*dmats0[1][43] + coeff0_2*dmats0[2][43] + coeff0_3*dmats0[3][43] + coeff0_4*dmats0[4][43] + coeff0_5*dmats0[5][43] + coeff0_6*dmats0[6][43] + coeff0_7*dmats0[7][43] + coeff0_8*dmats0[8][43] + coeff0_9*dmats0[9][43] + coeff0_10*dmats0[10][43] + coeff0_11*dmats0[11][43] + coeff0_12*dmats0[12][43] + coeff0_13*dmats0[13][43] + coeff0_14*dmats0[14][43] + coeff0_15*dmats0[15][43] + coeff0_16*dmats0[16][43] + coeff0_17*dmats0[17][43] + coeff0_18*dmats0[18][43] + coeff0_19*dmats0[19][43] + coeff0_20*dmats0[20][43] + coeff0_21*dmats0[21][43] + coeff0_22*dmats0[22][43] + coeff0_23*dmats0[23][43] + coeff0_24*dmats0[24][43] + coeff0_25*dmats0[25][43] + coeff0_26*dmats0[26][43] + coeff0_27*dmats0[27][43] + coeff0_28*dmats0[28][43] + coeff0_29*dmats0[29][43] + coeff0_30*dmats0[30][43] + coeff0_31*dmats0[31][43] + coeff0_32*dmats0[32][43] + coeff0_33*dmats0[33][43] + coeff0_34*dmats0[34][43] + coeff0_35*dmats0[35][43] + coeff0_36*dmats0[36][43] + coeff0_37*dmats0[37][43] + coeff0_38*dmats0[38][43] + coeff0_39*dmats0[39][43] + coeff0_40*dmats0[40][43] + coeff0_41*dmats0[41][43] + coeff0_42*dmats0[42][43] + coeff0_43*dmats0[43][43] + coeff0_44*dmats0[44][43] + coeff0_45*dmats0[45][43] + coeff0_46*dmats0[46][43] + coeff0_47*dmats0[47][43] + coeff0_48*dmats0[48][43] + coeff0_49*dmats0[49][43] + coeff0_50*dmats0[50][43] + coeff0_51*dmats0[51][43] + coeff0_52*dmats0[52][43] + coeff0_53*dmats0[53][43] + coeff0_54*dmats0[54][43] + coeff0_55*dmats0[55][43];
          new_coeff0_44 = coeff0_0*dmats0[0][44] + coeff0_1*dmats0[1][44] + coeff0_2*dmats0[2][44] + coeff0_3*dmats0[3][44] + coeff0_4*dmats0[4][44] + coeff0_5*dmats0[5][44] + coeff0_6*dmats0[6][44] + coeff0_7*dmats0[7][44] + coeff0_8*dmats0[8][44] + coeff0_9*dmats0[9][44] + coeff0_10*dmats0[10][44] + coeff0_11*dmats0[11][44] + coeff0_12*dmats0[12][44] + coeff0_13*dmats0[13][44] + coeff0_14*dmats0[14][44] + coeff0_15*dmats0[15][44] + coeff0_16*dmats0[16][44] + coeff0_17*dmats0[17][44] + coeff0_18*dmats0[18][44] + coeff0_19*dmats0[19][44] + coeff0_20*dmats0[20][44] + coeff0_21*dmats0[21][44] + coeff0_22*dmats0[22][44] + coeff0_23*dmats0[23][44] + coeff0_24*dmats0[24][44] + coeff0_25*dmats0[25][44] + coeff0_26*dmats0[26][44] + coeff0_27*dmats0[27][44] + coeff0_28*dmats0[28][44] + coeff0_29*dmats0[29][44] + coeff0_30*dmats0[30][44] + coeff0_31*dmats0[31][44] + coeff0_32*dmats0[32][44] + coeff0_33*dmats0[33][44] + coeff0_34*dmats0[34][44] + coeff0_35*dmats0[35][44] + coeff0_36*dmats0[36][44] + coeff0_37*dmats0[37][44] + coeff0_38*dmats0[38][44] + coeff0_39*dmats0[39][44] + coeff0_40*dmats0[40][44] + coeff0_41*dmats0[41][44] + coeff0_42*dmats0[42][44] + coeff0_43*dmats0[43][44] + coeff0_44*dmats0[44][44] + coeff0_45*dmats0[45][44] + coeff0_46*dmats0[46][44] + coeff0_47*dmats0[47][44] + coeff0_48*dmats0[48][44] + coeff0_49*dmats0[49][44] + coeff0_50*dmats0[50][44] + coeff0_51*dmats0[51][44] + coeff0_52*dmats0[52][44] + coeff0_53*dmats0[53][44] + coeff0_54*dmats0[54][44] + coeff0_55*dmats0[55][44];
          new_coeff0_45 = coeff0_0*dmats0[0][45] + coeff0_1*dmats0[1][45] + coeff0_2*dmats0[2][45] + coeff0_3*dmats0[3][45] + coeff0_4*dmats0[4][45] + coeff0_5*dmats0[5][45] + coeff0_6*dmats0[6][45] + coeff0_7*dmats0[7][45] + coeff0_8*dmats0[8][45] + coeff0_9*dmats0[9][45] + coeff0_10*dmats0[10][45] + coeff0_11*dmats0[11][45] + coeff0_12*dmats0[12][45] + coeff0_13*dmats0[13][45] + coeff0_14*dmats0[14][45] + coeff0_15*dmats0[15][45] + coeff0_16*dmats0[16][45] + coeff0_17*dmats0[17][45] + coeff0_18*dmats0[18][45] + coeff0_19*dmats0[19][45] + coeff0_20*dmats0[20][45] + coeff0_21*dmats0[21][45] + coeff0_22*dmats0[22][45] + coeff0_23*dmats0[23][45] + coeff0_24*dmats0[24][45] + coeff0_25*dmats0[25][45] + coeff0_26*dmats0[26][45] + coeff0_27*dmats0[27][45] + coeff0_28*dmats0[28][45] + coeff0_29*dmats0[29][45] + coeff0_30*dmats0[30][45] + coeff0_31*dmats0[31][45] + coeff0_32*dmats0[32][45] + coeff0_33*dmats0[33][45] + coeff0_34*dmats0[34][45] + coeff0_35*dmats0[35][45] + coeff0_36*dmats0[36][45] + coeff0_37*dmats0[37][45] + coeff0_38*dmats0[38][45] + coeff0_39*dmats0[39][45] + coeff0_40*dmats0[40][45] + coeff0_41*dmats0[41][45] + coeff0_42*dmats0[42][45] + coeff0_43*dmats0[43][45] + coeff0_44*dmats0[44][45] + coeff0_45*dmats0[45][45] + coeff0_46*dmats0[46][45] + coeff0_47*dmats0[47][45] + coeff0_48*dmats0[48][45] + coeff0_49*dmats0[49][45] + coeff0_50*dmats0[50][45] + coeff0_51*dmats0[51][45] + coeff0_52*dmats0[52][45] + coeff0_53*dmats0[53][45] + coeff0_54*dmats0[54][45] + coeff0_55*dmats0[55][45];
          new_coeff0_46 = coeff0_0*dmats0[0][46] + coeff0_1*dmats0[1][46] + coeff0_2*dmats0[2][46] + coeff0_3*dmats0[3][46] + coeff0_4*dmats0[4][46] + coeff0_5*dmats0[5][46] + coeff0_6*dmats0[6][46] + coeff0_7*dmats0[7][46] + coeff0_8*dmats0[8][46] + coeff0_9*dmats0[9][46] + coeff0_10*dmats0[10][46] + coeff0_11*dmats0[11][46] + coeff0_12*dmats0[12][46] + coeff0_13*dmats0[13][46] + coeff0_14*dmats0[14][46] + coeff0_15*dmats0[15][46] + coeff0_16*dmats0[16][46] + coeff0_17*dmats0[17][46] + coeff0_18*dmats0[18][46] + coeff0_19*dmats0[19][46] + coeff0_20*dmats0[20][46] + coeff0_21*dmats0[21][46] + coeff0_22*dmats0[22][46] + coeff0_23*dmats0[23][46] + coeff0_24*dmats0[24][46] + coeff0_25*dmats0[25][46] + coeff0_26*dmats0[26][46] + coeff0_27*dmats0[27][46] + coeff0_28*dmats0[28][46] + coeff0_29*dmats0[29][46] + coeff0_30*dmats0[30][46] + coeff0_31*dmats0[31][46] + coeff0_32*dmats0[32][46] + coeff0_33*dmats0[33][46] + coeff0_34*dmats0[34][46] + coeff0_35*dmats0[35][46] + coeff0_36*dmats0[36][46] + coeff0_37*dmats0[37][46] + coeff0_38*dmats0[38][46] + coeff0_39*dmats0[39][46] + coeff0_40*dmats0[40][46] + coeff0_41*dmats0[41][46] + coeff0_42*dmats0[42][46] + coeff0_43*dmats0[43][46] + coeff0_44*dmats0[44][46] + coeff0_45*dmats0[45][46] + coeff0_46*dmats0[46][46] + coeff0_47*dmats0[47][46] + coeff0_48*dmats0[48][46] + coeff0_49*dmats0[49][46] + coeff0_50*dmats0[50][46] + coeff0_51*dmats0[51][46] + coeff0_52*dmats0[52][46] + coeff0_53*dmats0[53][46] + coeff0_54*dmats0[54][46] + coeff0_55*dmats0[55][46];
          new_coeff0_47 = coeff0_0*dmats0[0][47] + coeff0_1*dmats0[1][47] + coeff0_2*dmats0[2][47] + coeff0_3*dmats0[3][47] + coeff0_4*dmats0[4][47] + coeff0_5*dmats0[5][47] + coeff0_6*dmats0[6][47] + coeff0_7*dmats0[7][47] + coeff0_8*dmats0[8][47] + coeff0_9*dmats0[9][47] + coeff0_10*dmats0[10][47] + coeff0_11*dmats0[11][47] + coeff0_12*dmats0[12][47] + coeff0_13*dmats0[13][47] + coeff0_14*dmats0[14][47] + coeff0_15*dmats0[15][47] + coeff0_16*dmats0[16][47] + coeff0_17*dmats0[17][47] + coeff0_18*dmats0[18][47] + coeff0_19*dmats0[19][47] + coeff0_20*dmats0[20][47] + coeff0_21*dmats0[21][47] + coeff0_22*dmats0[22][47] + coeff0_23*dmats0[23][47] + coeff0_24*dmats0[24][47] + coeff0_25*dmats0[25][47] + coeff0_26*dmats0[26][47] + coeff0_27*dmats0[27][47] + coeff0_28*dmats0[28][47] + coeff0_29*dmats0[29][47] + coeff0_30*dmats0[30][47] + coeff0_31*dmats0[31][47] + coeff0_32*dmats0[32][47] + coeff0_33*dmats0[33][47] + coeff0_34*dmats0[34][47] + coeff0_35*dmats0[35][47] + coeff0_36*dmats0[36][47] + coeff0_37*dmats0[37][47] + coeff0_38*dmats0[38][47] + coeff0_39*dmats0[39][47] + coeff0_40*dmats0[40][47] + coeff0_41*dmats0[41][47] + coeff0_42*dmats0[42][47] + coeff0_43*dmats0[43][47] + coeff0_44*dmats0[44][47] + coeff0_45*dmats0[45][47] + coeff0_46*dmats0[46][47] + coeff0_47*dmats0[47][47] + coeff0_48*dmats0[48][47] + coeff0_49*dmats0[49][47] + coeff0_50*dmats0[50][47] + coeff0_51*dmats0[51][47] + coeff0_52*dmats0[52][47] + coeff0_53*dmats0[53][47] + coeff0_54*dmats0[54][47] + coeff0_55*dmats0[55][47];
          new_coeff0_48 = coeff0_0*dmats0[0][48] + coeff0_1*dmats0[1][48] + coeff0_2*dmats0[2][48] + coeff0_3*dmats0[3][48] + coeff0_4*dmats0[4][48] + coeff0_5*dmats0[5][48] + coeff0_6*dmats0[6][48] + coeff0_7*dmats0[7][48] + coeff0_8*dmats0[8][48] + coeff0_9*dmats0[9][48] + coeff0_10*dmats0[10][48] + coeff0_11*dmats0[11][48] + coeff0_12*dmats0[12][48] + coeff0_13*dmats0[13][48] + coeff0_14*dmats0[14][48] + coeff0_15*dmats0[15][48] + coeff0_16*dmats0[16][48] + coeff0_17*dmats0[17][48] + coeff0_18*dmats0[18][48] + coeff0_19*dmats0[19][48] + coeff0_20*dmats0[20][48] + coeff0_21*dmats0[21][48] + coeff0_22*dmats0[22][48] + coeff0_23*dmats0[23][48] + coeff0_24*dmats0[24][48] + coeff0_25*dmats0[25][48] + coeff0_26*dmats0[26][48] + coeff0_27*dmats0[27][48] + coeff0_28*dmats0[28][48] + coeff0_29*dmats0[29][48] + coeff0_30*dmats0[30][48] + coeff0_31*dmats0[31][48] + coeff0_32*dmats0[32][48] + coeff0_33*dmats0[33][48] + coeff0_34*dmats0[34][48] + coeff0_35*dmats0[35][48] + coeff0_36*dmats0[36][48] + coeff0_37*dmats0[37][48] + coeff0_38*dmats0[38][48] + coeff0_39*dmats0[39][48] + coeff0_40*dmats0[40][48] + coeff0_41*dmats0[41][48] + coeff0_42*dmats0[42][48] + coeff0_43*dmats0[43][48] + coeff0_44*dmats0[44][48] + coeff0_45*dmats0[45][48] + coeff0_46*dmats0[46][48] + coeff0_47*dmats0[47][48] + coeff0_48*dmats0[48][48] + coeff0_49*dmats0[49][48] + coeff0_50*dmats0[50][48] + coeff0_51*dmats0[51][48] + coeff0_52*dmats0[52][48] + coeff0_53*dmats0[53][48] + coeff0_54*dmats0[54][48] + coeff0_55*dmats0[55][48];
          new_coeff0_49 = coeff0_0*dmats0[0][49] + coeff0_1*dmats0[1][49] + coeff0_2*dmats0[2][49] + coeff0_3*dmats0[3][49] + coeff0_4*dmats0[4][49] + coeff0_5*dmats0[5][49] + coeff0_6*dmats0[6][49] + coeff0_7*dmats0[7][49] + coeff0_8*dmats0[8][49] + coeff0_9*dmats0[9][49] + coeff0_10*dmats0[10][49] + coeff0_11*dmats0[11][49] + coeff0_12*dmats0[12][49] + coeff0_13*dmats0[13][49] + coeff0_14*dmats0[14][49] + coeff0_15*dmats0[15][49] + coeff0_16*dmats0[16][49] + coeff0_17*dmats0[17][49] + coeff0_18*dmats0[18][49] + coeff0_19*dmats0[19][49] + coeff0_20*dmats0[20][49] + coeff0_21*dmats0[21][49] + coeff0_22*dmats0[22][49] + coeff0_23*dmats0[23][49] + coeff0_24*dmats0[24][49] + coeff0_25*dmats0[25][49] + coeff0_26*dmats0[26][49] + coeff0_27*dmats0[27][49] + coeff0_28*dmats0[28][49] + coeff0_29*dmats0[29][49] + coeff0_30*dmats0[30][49] + coeff0_31*dmats0[31][49] + coeff0_32*dmats0[32][49] + coeff0_33*dmats0[33][49] + coeff0_34*dmats0[34][49] + coeff0_35*dmats0[35][49] + coeff0_36*dmats0[36][49] + coeff0_37*dmats0[37][49] + coeff0_38*dmats0[38][49] + coeff0_39*dmats0[39][49] + coeff0_40*dmats0[40][49] + coeff0_41*dmats0[41][49] + coeff0_42*dmats0[42][49] + coeff0_43*dmats0[43][49] + coeff0_44*dmats0[44][49] + coeff0_45*dmats0[45][49] + coeff0_46*dmats0[46][49] + coeff0_47*dmats0[47][49] + coeff0_48*dmats0[48][49] + coeff0_49*dmats0[49][49] + coeff0_50*dmats0[50][49] + coeff0_51*dmats0[51][49] + coeff0_52*dmats0[52][49] + coeff0_53*dmats0[53][49] + coeff0_54*dmats0[54][49] + coeff0_55*dmats0[55][49];
          new_coeff0_50 = coeff0_0*dmats0[0][50] + coeff0_1*dmats0[1][50] + coeff0_2*dmats0[2][50] + coeff0_3*dmats0[3][50] + coeff0_4*dmats0[4][50] + coeff0_5*dmats0[5][50] + coeff0_6*dmats0[6][50] + coeff0_7*dmats0[7][50] + coeff0_8*dmats0[8][50] + coeff0_9*dmats0[9][50] + coeff0_10*dmats0[10][50] + coeff0_11*dmats0[11][50] + coeff0_12*dmats0[12][50] + coeff0_13*dmats0[13][50] + coeff0_14*dmats0[14][50] + coeff0_15*dmats0[15][50] + coeff0_16*dmats0[16][50] + coeff0_17*dmats0[17][50] + coeff0_18*dmats0[18][50] + coeff0_19*dmats0[19][50] + coeff0_20*dmats0[20][50] + coeff0_21*dmats0[21][50] + coeff0_22*dmats0[22][50] + coeff0_23*dmats0[23][50] + coeff0_24*dmats0[24][50] + coeff0_25*dmats0[25][50] + coeff0_26*dmats0[26][50] + coeff0_27*dmats0[27][50] + coeff0_28*dmats0[28][50] + coeff0_29*dmats0[29][50] + coeff0_30*dmats0[30][50] + coeff0_31*dmats0[31][50] + coeff0_32*dmats0[32][50] + coeff0_33*dmats0[33][50] + coeff0_34*dmats0[34][50] + coeff0_35*dmats0[35][50] + coeff0_36*dmats0[36][50] + coeff0_37*dmats0[37][50] + coeff0_38*dmats0[38][50] + coeff0_39*dmats0[39][50] + coeff0_40*dmats0[40][50] + coeff0_41*dmats0[41][50] + coeff0_42*dmats0[42][50] + coeff0_43*dmats0[43][50] + coeff0_44*dmats0[44][50] + coeff0_45*dmats0[45][50] + coeff0_46*dmats0[46][50] + coeff0_47*dmats0[47][50] + coeff0_48*dmats0[48][50] + coeff0_49*dmats0[49][50] + coeff0_50*dmats0[50][50] + coeff0_51*dmats0[51][50] + coeff0_52*dmats0[52][50] + coeff0_53*dmats0[53][50] + coeff0_54*dmats0[54][50] + coeff0_55*dmats0[55][50];
          new_coeff0_51 = coeff0_0*dmats0[0][51] + coeff0_1*dmats0[1][51] + coeff0_2*dmats0[2][51] + coeff0_3*dmats0[3][51] + coeff0_4*dmats0[4][51] + coeff0_5*dmats0[5][51] + coeff0_6*dmats0[6][51] + coeff0_7*dmats0[7][51] + coeff0_8*dmats0[8][51] + coeff0_9*dmats0[9][51] + coeff0_10*dmats0[10][51] + coeff0_11*dmats0[11][51] + coeff0_12*dmats0[12][51] + coeff0_13*dmats0[13][51] + coeff0_14*dmats0[14][51] + coeff0_15*dmats0[15][51] + coeff0_16*dmats0[16][51] + coeff0_17*dmats0[17][51] + coeff0_18*dmats0[18][51] + coeff0_19*dmats0[19][51] + coeff0_20*dmats0[20][51] + coeff0_21*dmats0[21][51] + coeff0_22*dmats0[22][51] + coeff0_23*dmats0[23][51] + coeff0_24*dmats0[24][51] + coeff0_25*dmats0[25][51] + coeff0_26*dmats0[26][51] + coeff0_27*dmats0[27][51] + coeff0_28*dmats0[28][51] + coeff0_29*dmats0[29][51] + coeff0_30*dmats0[30][51] + coeff0_31*dmats0[31][51] + coeff0_32*dmats0[32][51] + coeff0_33*dmats0[33][51] + coeff0_34*dmats0[34][51] + coeff0_35*dmats0[35][51] + coeff0_36*dmats0[36][51] + coeff0_37*dmats0[37][51] + coeff0_38*dmats0[38][51] + coeff0_39*dmats0[39][51] + coeff0_40*dmats0[40][51] + coeff0_41*dmats0[41][51] + coeff0_42*dmats0[42][51] + coeff0_43*dmats0[43][51] + coeff0_44*dmats0[44][51] + coeff0_45*dmats0[45][51] + coeff0_46*dmats0[46][51] + coeff0_47*dmats0[47][51] + coeff0_48*dmats0[48][51] + coeff0_49*dmats0[49][51] + coeff0_50*dmats0[50][51] + coeff0_51*dmats0[51][51] + coeff0_52*dmats0[52][51] + coeff0_53*dmats0[53][51] + coeff0_54*dmats0[54][51] + coeff0_55*dmats0[55][51];
          new_coeff0_52 = coeff0_0*dmats0[0][52] + coeff0_1*dmats0[1][52] + coeff0_2*dmats0[2][52] + coeff0_3*dmats0[3][52] + coeff0_4*dmats0[4][52] + coeff0_5*dmats0[5][52] + coeff0_6*dmats0[6][52] + coeff0_7*dmats0[7][52] + coeff0_8*dmats0[8][52] + coeff0_9*dmats0[9][52] + coeff0_10*dmats0[10][52] + coeff0_11*dmats0[11][52] + coeff0_12*dmats0[12][52] + coeff0_13*dmats0[13][52] + coeff0_14*dmats0[14][52] + coeff0_15*dmats0[15][52] + coeff0_16*dmats0[16][52] + coeff0_17*dmats0[17][52] + coeff0_18*dmats0[18][52] + coeff0_19*dmats0[19][52] + coeff0_20*dmats0[20][52] + coeff0_21*dmats0[21][52] + coeff0_22*dmats0[22][52] + coeff0_23*dmats0[23][52] + coeff0_24*dmats0[24][52] + coeff0_25*dmats0[25][52] + coeff0_26*dmats0[26][52] + coeff0_27*dmats0[27][52] + coeff0_28*dmats0[28][52] + coeff0_29*dmats0[29][52] + coeff0_30*dmats0[30][52] + coeff0_31*dmats0[31][52] + coeff0_32*dmats0[32][52] + coeff0_33*dmats0[33][52] + coeff0_34*dmats0[34][52] + coeff0_35*dmats0[35][52] + coeff0_36*dmats0[36][52] + coeff0_37*dmats0[37][52] + coeff0_38*dmats0[38][52] + coeff0_39*dmats0[39][52] + coeff0_40*dmats0[40][52] + coeff0_41*dmats0[41][52] + coeff0_42*dmats0[42][52] + coeff0_43*dmats0[43][52] + coeff0_44*dmats0[44][52] + coeff0_45*dmats0[45][52] + coeff0_46*dmats0[46][52] + coeff0_47*dmats0[47][52] + coeff0_48*dmats0[48][52] + coeff0_49*dmats0[49][52] + coeff0_50*dmats0[50][52] + coeff0_51*dmats0[51][52] + coeff0_52*dmats0[52][52] + coeff0_53*dmats0[53][52] + coeff0_54*dmats0[54][52] + coeff0_55*dmats0[55][52];
          new_coeff0_53 = coeff0_0*dmats0[0][53] + coeff0_1*dmats0[1][53] + coeff0_2*dmats0[2][53] + coeff0_3*dmats0[3][53] + coeff0_4*dmats0[4][53] + coeff0_5*dmats0[5][53] + coeff0_6*dmats0[6][53] + coeff0_7*dmats0[7][53] + coeff0_8*dmats0[8][53] + coeff0_9*dmats0[9][53] + coeff0_10*dmats0[10][53] + coeff0_11*dmats0[11][53] + coeff0_12*dmats0[12][53] + coeff0_13*dmats0[13][53] + coeff0_14*dmats0[14][53] + coeff0_15*dmats0[15][53] + coeff0_16*dmats0[16][53] + coeff0_17*dmats0[17][53] + coeff0_18*dmats0[18][53] + coeff0_19*dmats0[19][53] + coeff0_20*dmats0[20][53] + coeff0_21*dmats0[21][53] + coeff0_22*dmats0[22][53] + coeff0_23*dmats0[23][53] + coeff0_24*dmats0[24][53] + coeff0_25*dmats0[25][53] + coeff0_26*dmats0[26][53] + coeff0_27*dmats0[27][53] + coeff0_28*dmats0[28][53] + coeff0_29*dmats0[29][53] + coeff0_30*dmats0[30][53] + coeff0_31*dmats0[31][53] + coeff0_32*dmats0[32][53] + coeff0_33*dmats0[33][53] + coeff0_34*dmats0[34][53] + coeff0_35*dmats0[35][53] + coeff0_36*dmats0[36][53] + coeff0_37*dmats0[37][53] + coeff0_38*dmats0[38][53] + coeff0_39*dmats0[39][53] + coeff0_40*dmats0[40][53] + coeff0_41*dmats0[41][53] + coeff0_42*dmats0[42][53] + coeff0_43*dmats0[43][53] + coeff0_44*dmats0[44][53] + coeff0_45*dmats0[45][53] + coeff0_46*dmats0[46][53] + coeff0_47*dmats0[47][53] + coeff0_48*dmats0[48][53] + coeff0_49*dmats0[49][53] + coeff0_50*dmats0[50][53] + coeff0_51*dmats0[51][53] + coeff0_52*dmats0[52][53] + coeff0_53*dmats0[53][53] + coeff0_54*dmats0[54][53] + coeff0_55*dmats0[55][53];
          new_coeff0_54 = coeff0_0*dmats0[0][54] + coeff0_1*dmats0[1][54] + coeff0_2*dmats0[2][54] + coeff0_3*dmats0[3][54] + coeff0_4*dmats0[4][54] + coeff0_5*dmats0[5][54] + coeff0_6*dmats0[6][54] + coeff0_7*dmats0[7][54] + coeff0_8*dmats0[8][54] + coeff0_9*dmats0[9][54] + coeff0_10*dmats0[10][54] + coeff0_11*dmats0[11][54] + coeff0_12*dmats0[12][54] + coeff0_13*dmats0[13][54] + coeff0_14*dmats0[14][54] + coeff0_15*dmats0[15][54] + coeff0_16*dmats0[16][54] + coeff0_17*dmats0[17][54] + coeff0_18*dmats0[18][54] + coeff0_19*dmats0[19][54] + coeff0_20*dmats0[20][54] + coeff0_21*dmats0[21][54] + coeff0_22*dmats0[22][54] + coeff0_23*dmats0[23][54] + coeff0_24*dmats0[24][54] + coeff0_25*dmats0[25][54] + coeff0_26*dmats0[26][54] + coeff0_27*dmats0[27][54] + coeff0_28*dmats0[28][54] + coeff0_29*dmats0[29][54] + coeff0_30*dmats0[30][54] + coeff0_31*dmats0[31][54] + coeff0_32*dmats0[32][54] + coeff0_33*dmats0[33][54] + coeff0_34*dmats0[34][54] + coeff0_35*dmats0[35][54] + coeff0_36*dmats0[36][54] + coeff0_37*dmats0[37][54] + coeff0_38*dmats0[38][54] + coeff0_39*dmats0[39][54] + coeff0_40*dmats0[40][54] + coeff0_41*dmats0[41][54] + coeff0_42*dmats0[42][54] + coeff0_43*dmats0[43][54] + coeff0_44*dmats0[44][54] + coeff0_45*dmats0[45][54] + coeff0_46*dmats0[46][54] + coeff0_47*dmats0[47][54] + coeff0_48*dmats0[48][54] + coeff0_49*dmats0[49][54] + coeff0_50*dmats0[50][54] + coeff0_51*dmats0[51][54] + coeff0_52*dmats0[52][54] + coeff0_53*dmats0[53][54] + coeff0_54*dmats0[54][54] + coeff0_55*dmats0[55][54];
          new_coeff0_55 = coeff0_0*dmats0[0][55] + coeff0_1*dmats0[1][55] + coeff0_2*dmats0[2][55] + coeff0_3*dmats0[3][55] + coeff0_4*dmats0[4][55] + coeff0_5*dmats0[5][55] + coeff0_6*dmats0[6][55] + coeff0_7*dmats0[7][55] + coeff0_8*dmats0[8][55] + coeff0_9*dmats0[9][55] + coeff0_10*dmats0[10][55] + coeff0_11*dmats0[11][55] + coeff0_12*dmats0[12][55] + coeff0_13*dmats0[13][55] + coeff0_14*dmats0[14][55] + coeff0_15*dmats0[15][55] + coeff0_16*dmats0[16][55] + coeff0_17*dmats0[17][55] + coeff0_18*dmats0[18][55] + coeff0_19*dmats0[19][55] + coeff0_20*dmats0[20][55] + coeff0_21*dmats0[21][55] + coeff0_22*dmats0[22][55] + coeff0_23*dmats0[23][55] + coeff0_24*dmats0[24][55] + coeff0_25*dmats0[25][55] + coeff0_26*dmats0[26][55] + coeff0_27*dmats0[27][55] + coeff0_28*dmats0[28][55] + coeff0_29*dmats0[29][55] + coeff0_30*dmats0[30][55] + coeff0_31*dmats0[31][55] + coeff0_32*dmats0[32][55] + coeff0_33*dmats0[33][55] + coeff0_34*dmats0[34][55] + coeff0_35*dmats0[35][55] + coeff0_36*dmats0[36][55] + coeff0_37*dmats0[37][55] + coeff0_38*dmats0[38][55] + coeff0_39*dmats0[39][55] + coeff0_40*dmats0[40][55] + coeff0_41*dmats0[41][55] + coeff0_42*dmats0[42][55] + coeff0_43*dmats0[43][55] + coeff0_44*dmats0[44][55] + coeff0_45*dmats0[45][55] + coeff0_46*dmats0[46][55] + coeff0_47*dmats0[47][55] + coeff0_48*dmats0[48][55] + coeff0_49*dmats0[49][55] + coeff0_50*dmats0[50][55] + coeff0_51*dmats0[51][55] + coeff0_52*dmats0[52][55] + coeff0_53*dmats0[53][55] + coeff0_54*dmats0[54][55] + coeff0_55*dmats0[55][55];
        }
        if(combinations[deriv_num][j] == 1)
        {
          new_coeff0_0 = coeff0_0*dmats1[0][0] + coeff0_1*dmats1[1][0] + coeff0_2*dmats1[2][0] + coeff0_3*dmats1[3][0] + coeff0_4*dmats1[4][0] + coeff0_5*dmats1[5][0] + coeff0_6*dmats1[6][0] + coeff0_7*dmats1[7][0] + coeff0_8*dmats1[8][0] + coeff0_9*dmats1[9][0] + coeff0_10*dmats1[10][0] + coeff0_11*dmats1[11][0] + coeff0_12*dmats1[12][0] + coeff0_13*dmats1[13][0] + coeff0_14*dmats1[14][0] + coeff0_15*dmats1[15][0] + coeff0_16*dmats1[16][0] + coeff0_17*dmats1[17][0] + coeff0_18*dmats1[18][0] + coeff0_19*dmats1[19][0] + coeff0_20*dmats1[20][0] + coeff0_21*dmats1[21][0] + coeff0_22*dmats1[22][0] + coeff0_23*dmats1[23][0] + coeff0_24*dmats1[24][0] + coeff0_25*dmats1[25][0] + coeff0_26*dmats1[26][0] + coeff0_27*dmats1[27][0] + coeff0_28*dmats1[28][0] + coeff0_29*dmats1[29][0] + coeff0_30*dmats1[30][0] + coeff0_31*dmats1[31][0] + coeff0_32*dmats1[32][0] + coeff0_33*dmats1[33][0] + coeff0_34*dmats1[34][0] + coeff0_35*dmats1[35][0] + coeff0_36*dmats1[36][0] + coeff0_37*dmats1[37][0] + coeff0_38*dmats1[38][0] + coeff0_39*dmats1[39][0] + coeff0_40*dmats1[40][0] + coeff0_41*dmats1[41][0] + coeff0_42*dmats1[42][0] + coeff0_43*dmats1[43][0] + coeff0_44*dmats1[44][0] + coeff0_45*dmats1[45][0] + coeff0_46*dmats1[46][0] + coeff0_47*dmats1[47][0] + coeff0_48*dmats1[48][0] + coeff0_49*dmats1[49][0] + coeff0_50*dmats1[50][0] + coeff0_51*dmats1[51][0] + coeff0_52*dmats1[52][0] + coeff0_53*dmats1[53][0] + coeff0_54*dmats1[54][0] + coeff0_55*dmats1[55][0];
          new_coeff0_1 = coeff0_0*dmats1[0][1] + coeff0_1*dmats1[1][1] + coeff0_2*dmats1[2][1] + coeff0_3*dmats1[3][1] + coeff0_4*dmats1[4][1] + coeff0_5*dmats1[5][1] + coeff0_6*dmats1[6][1] + coeff0_7*dmats1[7][1] + coeff0_8*dmats1[8][1] + coeff0_9*dmats1[9][1] + coeff0_10*dmats1[10][1] + coeff0_11*dmats1[11][1] + coeff0_12*dmats1[12][1] + coeff0_13*dmats1[13][1] + coeff0_14*dmats1[14][1] + coeff0_15*dmats1[15][1] + coeff0_16*dmats1[16][1] + coeff0_17*dmats1[17][1] + coeff0_18*dmats1[18][1] + coeff0_19*dmats1[19][1] + coeff0_20*dmats1[20][1] + coeff0_21*dmats1[21][1] + coeff0_22*dmats1[22][1] + coeff0_23*dmats1[23][1] + coeff0_24*dmats1[24][1] + coeff0_25*dmats1[25][1] + coeff0_26*dmats1[26][1] + coeff0_27*dmats1[27][1] + coeff0_28*dmats1[28][1] + coeff0_29*dmats1[29][1] + coeff0_30*dmats1[30][1] + coeff0_31*dmats1[31][1] + coeff0_32*dmats1[32][1] + coeff0_33*dmats1[33][1] + coeff0_34*dmats1[34][1] + coeff0_35*dmats1[35][1] + coeff0_36*dmats1[36][1] + coeff0_37*dmats1[37][1] + coeff0_38*dmats1[38][1] + coeff0_39*dmats1[39][1] + coeff0_40*dmats1[40][1] + coeff0_41*dmats1[41][1] + coeff0_42*dmats1[42][1] + coeff0_43*dmats1[43][1] + coeff0_44*dmats1[44][1] + coeff0_45*dmats1[45][1] + coeff0_46*dmats1[46][1] + coeff0_47*dmats1[47][1] + coeff0_48*dmats1[48][1] + coeff0_49*dmats1[49][1] + coeff0_50*dmats1[50][1] + coeff0_51*dmats1[51][1] + coeff0_52*dmats1[52][1] + coeff0_53*dmats1[53][1] + coeff0_54*dmats1[54][1] + coeff0_55*dmats1[55][1];
          new_coeff0_2 = coeff0_0*dmats1[0][2] + coeff0_1*dmats1[1][2] + coeff0_2*dmats1[2][2] + coeff0_3*dmats1[3][2] + coeff0_4*dmats1[4][2] + coeff0_5*dmats1[5][2] + coeff0_6*dmats1[6][2] + coeff0_7*dmats1[7][2] + coeff0_8*dmats1[8][2] + coeff0_9*dmats1[9][2] + coeff0_10*dmats1[10][2] + coeff0_11*dmats1[11][2] + coeff0_12*dmats1[12][2] + coeff0_13*dmats1[13][2] + coeff0_14*dmats1[14][2] + coeff0_15*dmats1[15][2] + coeff0_16*dmats1[16][2] + coeff0_17*dmats1[17][2] + coeff0_18*dmats1[18][2] + coeff0_19*dmats1[19][2] + coeff0_20*dmats1[20][2] + coeff0_21*dmats1[21][2] + coeff0_22*dmats1[22][2] + coeff0_23*dmats1[23][2] + coeff0_24*dmats1[24][2] + coeff0_25*dmats1[25][2] + coeff0_26*dmats1[26][2] + coeff0_27*dmats1[27][2] + coeff0_28*dmats1[28][2] + coeff0_29*dmats1[29][2] + coeff0_30*dmats1[30][2] + coeff0_31*dmats1[31][2] + coeff0_32*dmats1[32][2] + coeff0_33*dmats1[33][2] + coeff0_34*dmats1[34][2] + coeff0_35*dmats1[35][2] + coeff0_36*dmats1[36][2] + coeff0_37*dmats1[37][2] + coeff0_38*dmats1[38][2] + coeff0_39*dmats1[39][2] + coeff0_40*dmats1[40][2] + coeff0_41*dmats1[41][2] + coeff0_42*dmats1[42][2] + coeff0_43*dmats1[43][2] + coeff0_44*dmats1[44][2] + coeff0_45*dmats1[45][2] + coeff0_46*dmats1[46][2] + coeff0_47*dmats1[47][2] + coeff0_48*dmats1[48][2] + coeff0_49*dmats1[49][2] + coeff0_50*dmats1[50][2] + coeff0_51*dmats1[51][2] + coeff0_52*dmats1[52][2] + coeff0_53*dmats1[53][2] + coeff0_54*dmats1[54][2] + coeff0_55*dmats1[55][2];
          new_coeff0_3 = coeff0_0*dmats1[0][3] + coeff0_1*dmats1[1][3] + coeff0_2*dmats1[2][3] + coeff0_3*dmats1[3][3] + coeff0_4*dmats1[4][3] + coeff0_5*dmats1[5][3] + coeff0_6*dmats1[6][3] + coeff0_7*dmats1[7][3] + coeff0_8*dmats1[8][3] + coeff0_9*dmats1[9][3] + coeff0_10*dmats1[10][3] + coeff0_11*dmats1[11][3] + coeff0_12*dmats1[12][3] + coeff0_13*dmats1[13][3] + coeff0_14*dmats1[14][3] + coeff0_15*dmats1[15][3] + coeff0_16*dmats1[16][3] + coeff0_17*dmats1[17][3] + coeff0_18*dmats1[18][3] + coeff0_19*dmats1[19][3] + coeff0_20*dmats1[20][3] + coeff0_21*dmats1[21][3] + coeff0_22*dmats1[22][3] + coeff0_23*dmats1[23][3] + coeff0_24*dmats1[24][3] + coeff0_25*dmats1[25][3] + coeff0_26*dmats1[26][3] + coeff0_27*dmats1[27][3] + coeff0_28*dmats1[28][3] + coeff0_29*dmats1[29][3] + coeff0_30*dmats1[30][3] + coeff0_31*dmats1[31][3] + coeff0_32*dmats1[32][3] + coeff0_33*dmats1[33][3] + coeff0_34*dmats1[34][3] + coeff0_35*dmats1[35][3] + coeff0_36*dmats1[36][3] + coeff0_37*dmats1[37][3] + coeff0_38*dmats1[38][3] + coeff0_39*dmats1[39][3] + coeff0_40*dmats1[40][3] + coeff0_41*dmats1[41][3] + coeff0_42*dmats1[42][3] + coeff0_43*dmats1[43][3] + coeff0_44*dmats1[44][3] + coeff0_45*dmats1[45][3] + coeff0_46*dmats1[46][3] + coeff0_47*dmats1[47][3] + coeff0_48*dmats1[48][3] + coeff0_49*dmats1[49][3] + coeff0_50*dmats1[50][3] + coeff0_51*dmats1[51][3] + coeff0_52*dmats1[52][3] + coeff0_53*dmats1[53][3] + coeff0_54*dmats1[54][3] + coeff0_55*dmats1[55][3];
          new_coeff0_4 = coeff0_0*dmats1[0][4] + coeff0_1*dmats1[1][4] + coeff0_2*dmats1[2][4] + coeff0_3*dmats1[3][4] + coeff0_4*dmats1[4][4] + coeff0_5*dmats1[5][4] + coeff0_6*dmats1[6][4] + coeff0_7*dmats1[7][4] + coeff0_8*dmats1[8][4] + coeff0_9*dmats1[9][4] + coeff0_10*dmats1[10][4] + coeff0_11*dmats1[11][4] + coeff0_12*dmats1[12][4] + coeff0_13*dmats1[13][4] + coeff0_14*dmats1[14][4] + coeff0_15*dmats1[15][4] + coeff0_16*dmats1[16][4] + coeff0_17*dmats1[17][4] + coeff0_18*dmats1[18][4] + coeff0_19*dmats1[19][4] + coeff0_20*dmats1[20][4] + coeff0_21*dmats1[21][4] + coeff0_22*dmats1[22][4] + coeff0_23*dmats1[23][4] + coeff0_24*dmats1[24][4] + coeff0_25*dmats1[25][4] + coeff0_26*dmats1[26][4] + coeff0_27*dmats1[27][4] + coeff0_28*dmats1[28][4] + coeff0_29*dmats1[29][4] + coeff0_30*dmats1[30][4] + coeff0_31*dmats1[31][4] + coeff0_32*dmats1[32][4] + coeff0_33*dmats1[33][4] + coeff0_34*dmats1[34][4] + coeff0_35*dmats1[35][4] + coeff0_36*dmats1[36][4] + coeff0_37*dmats1[37][4] + coeff0_38*dmats1[38][4] + coeff0_39*dmats1[39][4] + coeff0_40*dmats1[40][4] + coeff0_41*dmats1[41][4] + coeff0_42*dmats1[42][4] + coeff0_43*dmats1[43][4] + coeff0_44*dmats1[44][4] + coeff0_45*dmats1[45][4] + coeff0_46*dmats1[46][4] + coeff0_47*dmats1[47][4] + coeff0_48*dmats1[48][4] + coeff0_49*dmats1[49][4] + coeff0_50*dmats1[50][4] + coeff0_51*dmats1[51][4] + coeff0_52*dmats1[52][4] + coeff0_53*dmats1[53][4] + coeff0_54*dmats1[54][4] + coeff0_55*dmats1[55][4];
          new_coeff0_5 = coeff0_0*dmats1[0][5] + coeff0_1*dmats1[1][5] + coeff0_2*dmats1[2][5] + coeff0_3*dmats1[3][5] + coeff0_4*dmats1[4][5] + coeff0_5*dmats1[5][5] + coeff0_6*dmats1[6][5] + coeff0_7*dmats1[7][5] + coeff0_8*dmats1[8][5] + coeff0_9*dmats1[9][5] + coeff0_10*dmats1[10][5] + coeff0_11*dmats1[11][5] + coeff0_12*dmats1[12][5] + coeff0_13*dmats1[13][5] + coeff0_14*dmats1[14][5] + coeff0_15*dmats1[15][5] + coeff0_16*dmats1[16][5] + coeff0_17*dmats1[17][5] + coeff0_18*dmats1[18][5] + coeff0_19*dmats1[19][5] + coeff0_20*dmats1[20][5] + coeff0_21*dmats1[21][5] + coeff0_22*dmats1[22][5] + coeff0_23*dmats1[23][5] + coeff0_24*dmats1[24][5] + coeff0_25*dmats1[25][5] + coeff0_26*dmats1[26][5] + coeff0_27*dmats1[27][5] + coeff0_28*dmats1[28][5] + coeff0_29*dmats1[29][5] + coeff0_30*dmats1[30][5] + coeff0_31*dmats1[31][5] + coeff0_32*dmats1[32][5] + coeff0_33*dmats1[33][5] + coeff0_34*dmats1[34][5] + coeff0_35*dmats1[35][5] + coeff0_36*dmats1[36][5] + coeff0_37*dmats1[37][5] + coeff0_38*dmats1[38][5] + coeff0_39*dmats1[39][5] + coeff0_40*dmats1[40][5] + coeff0_41*dmats1[41][5] + coeff0_42*dmats1[42][5] + coeff0_43*dmats1[43][5] + coeff0_44*dmats1[44][5] + coeff0_45*dmats1[45][5] + coeff0_46*dmats1[46][5] + coeff0_47*dmats1[47][5] + coeff0_48*dmats1[48][5] + coeff0_49*dmats1[49][5] + coeff0_50*dmats1[50][5] + coeff0_51*dmats1[51][5] + coeff0_52*dmats1[52][5] + coeff0_53*dmats1[53][5] + coeff0_54*dmats1[54][5] + coeff0_55*dmats1[55][5];
          new_coeff0_6 = coeff0_0*dmats1[0][6] + coeff0_1*dmats1[1][6] + coeff0_2*dmats1[2][6] + coeff0_3*dmats1[3][6] + coeff0_4*dmats1[4][6] + coeff0_5*dmats1[5][6] + coeff0_6*dmats1[6][6] + coeff0_7*dmats1[7][6] + coeff0_8*dmats1[8][6] + coeff0_9*dmats1[9][6] + coeff0_10*dmats1[10][6] + coeff0_11*dmats1[11][6] + coeff0_12*dmats1[12][6] + coeff0_13*dmats1[13][6] + coeff0_14*dmats1[14][6] + coeff0_15*dmats1[15][6] + coeff0_16*dmats1[16][6] + coeff0_17*dmats1[17][6] + coeff0_18*dmats1[18][6] + coeff0_19*dmats1[19][6] + coeff0_20*dmats1[20][6] + coeff0_21*dmats1[21][6] + coeff0_22*dmats1[22][6] + coeff0_23*dmats1[23][6] + coeff0_24*dmats1[24][6] + coeff0_25*dmats1[25][6] + coeff0_26*dmats1[26][6] + coeff0_27*dmats1[27][6] + coeff0_28*dmats1[28][6] + coeff0_29*dmats1[29][6] + coeff0_30*dmats1[30][6] + coeff0_31*dmats1[31][6] + coeff0_32*dmats1[32][6] + coeff0_33*dmats1[33][6] + coeff0_34*dmats1[34][6] + coeff0_35*dmats1[35][6] + coeff0_36*dmats1[36][6] + coeff0_37*dmats1[37][6] + coeff0_38*dmats1[38][6] + coeff0_39*dmats1[39][6] + coeff0_40*dmats1[40][6] + coeff0_41*dmats1[41][6] + coeff0_42*dmats1[42][6] + coeff0_43*dmats1[43][6] + coeff0_44*dmats1[44][6] + coeff0_45*dmats1[45][6] + coeff0_46*dmats1[46][6] + coeff0_47*dmats1[47][6] + coeff0_48*dmats1[48][6] + coeff0_49*dmats1[49][6] + coeff0_50*dmats1[50][6] + coeff0_51*dmats1[51][6] + coeff0_52*dmats1[52][6] + coeff0_53*dmats1[53][6] + coeff0_54*dmats1[54][6] + coeff0_55*dmats1[55][6];
          new_coeff0_7 = coeff0_0*dmats1[0][7] + coeff0_1*dmats1[1][7] + coeff0_2*dmats1[2][7] + coeff0_3*dmats1[3][7] + coeff0_4*dmats1[4][7] + coeff0_5*dmats1[5][7] + coeff0_6*dmats1[6][7] + coeff0_7*dmats1[7][7] + coeff0_8*dmats1[8][7] + coeff0_9*dmats1[9][7] + coeff0_10*dmats1[10][7] + coeff0_11*dmats1[11][7] + coeff0_12*dmats1[12][7] + coeff0_13*dmats1[13][7] + coeff0_14*dmats1[14][7] + coeff0_15*dmats1[15][7] + coeff0_16*dmats1[16][7] + coeff0_17*dmats1[17][7] + coeff0_18*dmats1[18][7] + coeff0_19*dmats1[19][7] + coeff0_20*dmats1[20][7] + coeff0_21*dmats1[21][7] + coeff0_22*dmats1[22][7] + coeff0_23*dmats1[23][7] + coeff0_24*dmats1[24][7] + coeff0_25*dmats1[25][7] + coeff0_26*dmats1[26][7] + coeff0_27*dmats1[27][7] + coeff0_28*dmats1[28][7] + coeff0_29*dmats1[29][7] + coeff0_30*dmats1[30][7] + coeff0_31*dmats1[31][7] + coeff0_32*dmats1[32][7] + coeff0_33*dmats1[33][7] + coeff0_34*dmats1[34][7] + coeff0_35*dmats1[35][7] + coeff0_36*dmats1[36][7] + coeff0_37*dmats1[37][7] + coeff0_38*dmats1[38][7] + coeff0_39*dmats1[39][7] + coeff0_40*dmats1[40][7] + coeff0_41*dmats1[41][7] + coeff0_42*dmats1[42][7] + coeff0_43*dmats1[43][7] + coeff0_44*dmats1[44][7] + coeff0_45*dmats1[45][7] + coeff0_46*dmats1[46][7] + coeff0_47*dmats1[47][7] + coeff0_48*dmats1[48][7] + coeff0_49*dmats1[49][7] + coeff0_50*dmats1[50][7] + coeff0_51*dmats1[51][7] + coeff0_52*dmats1[52][7] + coeff0_53*dmats1[53][7] + coeff0_54*dmats1[54][7] + coeff0_55*dmats1[55][7];
          new_coeff0_8 = coeff0_0*dmats1[0][8] + coeff0_1*dmats1[1][8] + coeff0_2*dmats1[2][8] + coeff0_3*dmats1[3][8] + coeff0_4*dmats1[4][8] + coeff0_5*dmats1[5][8] + coeff0_6*dmats1[6][8] + coeff0_7*dmats1[7][8] + coeff0_8*dmats1[8][8] + coeff0_9*dmats1[9][8] + coeff0_10*dmats1[10][8] + coeff0_11*dmats1[11][8] + coeff0_12*dmats1[12][8] + coeff0_13*dmats1[13][8] + coeff0_14*dmats1[14][8] + coeff0_15*dmats1[15][8] + coeff0_16*dmats1[16][8] + coeff0_17*dmats1[17][8] + coeff0_18*dmats1[18][8] + coeff0_19*dmats1[19][8] + coeff0_20*dmats1[20][8] + coeff0_21*dmats1[21][8] + coeff0_22*dmats1[22][8] + coeff0_23*dmats1[23][8] + coeff0_24*dmats1[24][8] + coeff0_25*dmats1[25][8] + coeff0_26*dmats1[26][8] + coeff0_27*dmats1[27][8] + coeff0_28*dmats1[28][8] + coeff0_29*dmats1[29][8] + coeff0_30*dmats1[30][8] + coeff0_31*dmats1[31][8] + coeff0_32*dmats1[32][8] + coeff0_33*dmats1[33][8] + coeff0_34*dmats1[34][8] + coeff0_35*dmats1[35][8] + coeff0_36*dmats1[36][8] + coeff0_37*dmats1[37][8] + coeff0_38*dmats1[38][8] + coeff0_39*dmats1[39][8] + coeff0_40*dmats1[40][8] + coeff0_41*dmats1[41][8] + coeff0_42*dmats1[42][8] + coeff0_43*dmats1[43][8] + coeff0_44*dmats1[44][8] + coeff0_45*dmats1[45][8] + coeff0_46*dmats1[46][8] + coeff0_47*dmats1[47][8] + coeff0_48*dmats1[48][8] + coeff0_49*dmats1[49][8] + coeff0_50*dmats1[50][8] + coeff0_51*dmats1[51][8] + coeff0_52*dmats1[52][8] + coeff0_53*dmats1[53][8] + coeff0_54*dmats1[54][8] + coeff0_55*dmats1[55][8];
          new_coeff0_9 = coeff0_0*dmats1[0][9] + coeff0_1*dmats1[1][9] + coeff0_2*dmats1[2][9] + coeff0_3*dmats1[3][9] + coeff0_4*dmats1[4][9] + coeff0_5*dmats1[5][9] + coeff0_6*dmats1[6][9] + coeff0_7*dmats1[7][9] + coeff0_8*dmats1[8][9] + coeff0_9*dmats1[9][9] + coeff0_10*dmats1[10][9] + coeff0_11*dmats1[11][9] + coeff0_12*dmats1[12][9] + coeff0_13*dmats1[13][9] + coeff0_14*dmats1[14][9] + coeff0_15*dmats1[15][9] + coeff0_16*dmats1[16][9] + coeff0_17*dmats1[17][9] + coeff0_18*dmats1[18][9] + coeff0_19*dmats1[19][9] + coeff0_20*dmats1[20][9] + coeff0_21*dmats1[21][9] + coeff0_22*dmats1[22][9] + coeff0_23*dmats1[23][9] + coeff0_24*dmats1[24][9] + coeff0_25*dmats1[25][9] + coeff0_26*dmats1[26][9] + coeff0_27*dmats1[27][9] + coeff0_28*dmats1[28][9] + coeff0_29*dmats1[29][9] + coeff0_30*dmats1[30][9] + coeff0_31*dmats1[31][9] + coeff0_32*dmats1[32][9] + coeff0_33*dmats1[33][9] + coeff0_34*dmats1[34][9] + coeff0_35*dmats1[35][9] + coeff0_36*dmats1[36][9] + coeff0_37*dmats1[37][9] + coeff0_38*dmats1[38][9] + coeff0_39*dmats1[39][9] + coeff0_40*dmats1[40][9] + coeff0_41*dmats1[41][9] + coeff0_42*dmats1[42][9] + coeff0_43*dmats1[43][9] + coeff0_44*dmats1[44][9] + coeff0_45*dmats1[45][9] + coeff0_46*dmats1[46][9] + coeff0_47*dmats1[47][9] + coeff0_48*dmats1[48][9] + coeff0_49*dmats1[49][9] + coeff0_50*dmats1[50][9] + coeff0_51*dmats1[51][9] + coeff0_52*dmats1[52][9] + coeff0_53*dmats1[53][9] + coeff0_54*dmats1[54][9] + coeff0_55*dmats1[55][9];
          new_coeff0_10 = coeff0_0*dmats1[0][10] + coeff0_1*dmats1[1][10] + coeff0_2*dmats1[2][10] + coeff0_3*dmats1[3][10] + coeff0_4*dmats1[4][10] + coeff0_5*dmats1[5][10] + coeff0_6*dmats1[6][10] + coeff0_7*dmats1[7][10] + coeff0_8*dmats1[8][10] + coeff0_9*dmats1[9][10] + coeff0_10*dmats1[10][10] + coeff0_11*dmats1[11][10] + coeff0_12*dmats1[12][10] + coeff0_13*dmats1[13][10] + coeff0_14*dmats1[14][10] + coeff0_15*dmats1[15][10] + coeff0_16*dmats1[16][10] + coeff0_17*dmats1[17][10] + coeff0_18*dmats1[18][10] + coeff0_19*dmats1[19][10] + coeff0_20*dmats1[20][10] + coeff0_21*dmats1[21][10] + coeff0_22*dmats1[22][10] + coeff0_23*dmats1[23][10] + coeff0_24*dmats1[24][10] + coeff0_25*dmats1[25][10] + coeff0_26*dmats1[26][10] + coeff0_27*dmats1[27][10] + coeff0_28*dmats1[28][10] + coeff0_29*dmats1[29][10] + coeff0_30*dmats1[30][10] + coeff0_31*dmats1[31][10] + coeff0_32*dmats1[32][10] + coeff0_33*dmats1[33][10] + coeff0_34*dmats1[34][10] + coeff0_35*dmats1[35][10] + coeff0_36*dmats1[36][10] + coeff0_37*dmats1[37][10] + coeff0_38*dmats1[38][10] + coeff0_39*dmats1[39][10] + coeff0_40*dmats1[40][10] + coeff0_41*dmats1[41][10] + coeff0_42*dmats1[42][10] + coeff0_43*dmats1[43][10] + coeff0_44*dmats1[44][10] + coeff0_45*dmats1[45][10] + coeff0_46*dmats1[46][10] + coeff0_47*dmats1[47][10] + coeff0_48*dmats1[48][10] + coeff0_49*dmats1[49][10] + coeff0_50*dmats1[50][10] + coeff0_51*dmats1[51][10] + coeff0_52*dmats1[52][10] + coeff0_53*dmats1[53][10] + coeff0_54*dmats1[54][10] + coeff0_55*dmats1[55][10];
          new_coeff0_11 = coeff0_0*dmats1[0][11] + coeff0_1*dmats1[1][11] + coeff0_2*dmats1[2][11] + coeff0_3*dmats1[3][11] + coeff0_4*dmats1[4][11] + coeff0_5*dmats1[5][11] + coeff0_6*dmats1[6][11] + coeff0_7*dmats1[7][11] + coeff0_8*dmats1[8][11] + coeff0_9*dmats1[9][11] + coeff0_10*dmats1[10][11] + coeff0_11*dmats1[11][11] + coeff0_12*dmats1[12][11] + coeff0_13*dmats1[13][11] + coeff0_14*dmats1[14][11] + coeff0_15*dmats1[15][11] + coeff0_16*dmats1[16][11] + coeff0_17*dmats1[17][11] + coeff0_18*dmats1[18][11] + coeff0_19*dmats1[19][11] + coeff0_20*dmats1[20][11] + coeff0_21*dmats1[21][11] + coeff0_22*dmats1[22][11] + coeff0_23*dmats1[23][11] + coeff0_24*dmats1[24][11] + coeff0_25*dmats1[25][11] + coeff0_26*dmats1[26][11] + coeff0_27*dmats1[27][11] + coeff0_28*dmats1[28][11] + coeff0_29*dmats1[29][11] + coeff0_30*dmats1[30][11] + coeff0_31*dmats1[31][11] + coeff0_32*dmats1[32][11] + coeff0_33*dmats1[33][11] + coeff0_34*dmats1[34][11] + coeff0_35*dmats1[35][11] + coeff0_36*dmats1[36][11] + coeff0_37*dmats1[37][11] + coeff0_38*dmats1[38][11] + coeff0_39*dmats1[39][11] + coeff0_40*dmats1[40][11] + coeff0_41*dmats1[41][11] + coeff0_42*dmats1[42][11] + coeff0_43*dmats1[43][11] + coeff0_44*dmats1[44][11] + coeff0_45*dmats1[45][11] + coeff0_46*dmats1[46][11] + coeff0_47*dmats1[47][11] + coeff0_48*dmats1[48][11] + coeff0_49*dmats1[49][11] + coeff0_50*dmats1[50][11] + coeff0_51*dmats1[51][11] + coeff0_52*dmats1[52][11] + coeff0_53*dmats1[53][11] + coeff0_54*dmats1[54][11] + coeff0_55*dmats1[55][11];
          new_coeff0_12 = coeff0_0*dmats1[0][12] + coeff0_1*dmats1[1][12] + coeff0_2*dmats1[2][12] + coeff0_3*dmats1[3][12] + coeff0_4*dmats1[4][12] + coeff0_5*dmats1[5][12] + coeff0_6*dmats1[6][12] + coeff0_7*dmats1[7][12] + coeff0_8*dmats1[8][12] + coeff0_9*dmats1[9][12] + coeff0_10*dmats1[10][12] + coeff0_11*dmats1[11][12] + coeff0_12*dmats1[12][12] + coeff0_13*dmats1[13][12] + coeff0_14*dmats1[14][12] + coeff0_15*dmats1[15][12] + coeff0_16*dmats1[16][12] + coeff0_17*dmats1[17][12] + coeff0_18*dmats1[18][12] + coeff0_19*dmats1[19][12] + coeff0_20*dmats1[20][12] + coeff0_21*dmats1[21][12] + coeff0_22*dmats1[22][12] + coeff0_23*dmats1[23][12] + coeff0_24*dmats1[24][12] + coeff0_25*dmats1[25][12] + coeff0_26*dmats1[26][12] + coeff0_27*dmats1[27][12] + coeff0_28*dmats1[28][12] + coeff0_29*dmats1[29][12] + coeff0_30*dmats1[30][12] + coeff0_31*dmats1[31][12] + coeff0_32*dmats1[32][12] + coeff0_33*dmats1[33][12] + coeff0_34*dmats1[34][12] + coeff0_35*dmats1[35][12] + coeff0_36*dmats1[36][12] + coeff0_37*dmats1[37][12] + coeff0_38*dmats1[38][12] + coeff0_39*dmats1[39][12] + coeff0_40*dmats1[40][12] + coeff0_41*dmats1[41][12] + coeff0_42*dmats1[42][12] + coeff0_43*dmats1[43][12] + coeff0_44*dmats1[44][12] + coeff0_45*dmats1[45][12] + coeff0_46*dmats1[46][12] + coeff0_47*dmats1[47][12] + coeff0_48*dmats1[48][12] + coeff0_49*dmats1[49][12] + coeff0_50*dmats1[50][12] + coeff0_51*dmats1[51][12] + coeff0_52*dmats1[52][12] + coeff0_53*dmats1[53][12] + coeff0_54*dmats1[54][12] + coeff0_55*dmats1[55][12];
          new_coeff0_13 = coeff0_0*dmats1[0][13] + coeff0_1*dmats1[1][13] + coeff0_2*dmats1[2][13] + coeff0_3*dmats1[3][13] + coeff0_4*dmats1[4][13] + coeff0_5*dmats1[5][13] + coeff0_6*dmats1[6][13] + coeff0_7*dmats1[7][13] + coeff0_8*dmats1[8][13] + coeff0_9*dmats1[9][13] + coeff0_10*dmats1[10][13] + coeff0_11*dmats1[11][13] + coeff0_12*dmats1[12][13] + coeff0_13*dmats1[13][13] + coeff0_14*dmats1[14][13] + coeff0_15*dmats1[15][13] + coeff0_16*dmats1[16][13] + coeff0_17*dmats1[17][13] + coeff0_18*dmats1[18][13] + coeff0_19*dmats1[19][13] + coeff0_20*dmats1[20][13] + coeff0_21*dmats1[21][13] + coeff0_22*dmats1[22][13] + coeff0_23*dmats1[23][13] + coeff0_24*dmats1[24][13] + coeff0_25*dmats1[25][13] + coeff0_26*dmats1[26][13] + coeff0_27*dmats1[27][13] + coeff0_28*dmats1[28][13] + coeff0_29*dmats1[29][13] + coeff0_30*dmats1[30][13] + coeff0_31*dmats1[31][13] + coeff0_32*dmats1[32][13] + coeff0_33*dmats1[33][13] + coeff0_34*dmats1[34][13] + coeff0_35*dmats1[35][13] + coeff0_36*dmats1[36][13] + coeff0_37*dmats1[37][13] + coeff0_38*dmats1[38][13] + coeff0_39*dmats1[39][13] + coeff0_40*dmats1[40][13] + coeff0_41*dmats1[41][13] + coeff0_42*dmats1[42][13] + coeff0_43*dmats1[43][13] + coeff0_44*dmats1[44][13] + coeff0_45*dmats1[45][13] + coeff0_46*dmats1[46][13] + coeff0_47*dmats1[47][13] + coeff0_48*dmats1[48][13] + coeff0_49*dmats1[49][13] + coeff0_50*dmats1[50][13] + coeff0_51*dmats1[51][13] + coeff0_52*dmats1[52][13] + coeff0_53*dmats1[53][13] + coeff0_54*dmats1[54][13] + coeff0_55*dmats1[55][13];
          new_coeff0_14 = coeff0_0*dmats1[0][14] + coeff0_1*dmats1[1][14] + coeff0_2*dmats1[2][14] + coeff0_3*dmats1[3][14] + coeff0_4*dmats1[4][14] + coeff0_5*dmats1[5][14] + coeff0_6*dmats1[6][14] + coeff0_7*dmats1[7][14] + coeff0_8*dmats1[8][14] + coeff0_9*dmats1[9][14] + coeff0_10*dmats1[10][14] + coeff0_11*dmats1[11][14] + coeff0_12*dmats1[12][14] + coeff0_13*dmats1[13][14] + coeff0_14*dmats1[14][14] + coeff0_15*dmats1[15][14] + coeff0_16*dmats1[16][14] + coeff0_17*dmats1[17][14] + coeff0_18*dmats1[18][14] + coeff0_19*dmats1[19][14] + coeff0_20*dmats1[20][14] + coeff0_21*dmats1[21][14] + coeff0_22*dmats1[22][14] + coeff0_23*dmats1[23][14] + coeff0_24*dmats1[24][14] + coeff0_25*dmats1[25][14] + coeff0_26*dmats1[26][14] + coeff0_27*dmats1[27][14] + coeff0_28*dmats1[28][14] + coeff0_29*dmats1[29][14] + coeff0_30*dmats1[30][14] + coeff0_31*dmats1[31][14] + coeff0_32*dmats1[32][14] + coeff0_33*dmats1[33][14] + coeff0_34*dmats1[34][14] + coeff0_35*dmats1[35][14] + coeff0_36*dmats1[36][14] + coeff0_37*dmats1[37][14] + coeff0_38*dmats1[38][14] + coeff0_39*dmats1[39][14] + coeff0_40*dmats1[40][14] + coeff0_41*dmats1[41][14] + coeff0_42*dmats1[42][14] + coeff0_43*dmats1[43][14] + coeff0_44*dmats1[44][14] + coeff0_45*dmats1[45][14] + coeff0_46*dmats1[46][14] + coeff0_47*dmats1[47][14] + coeff0_48*dmats1[48][14] + coeff0_49*dmats1[49][14] + coeff0_50*dmats1[50][14] + coeff0_51*dmats1[51][14] + coeff0_52*dmats1[52][14] + coeff0_53*dmats1[53][14] + coeff0_54*dmats1[54][14] + coeff0_55*dmats1[55][14];
          new_coeff0_15 = coeff0_0*dmats1[0][15] + coeff0_1*dmats1[1][15] + coeff0_2*dmats1[2][15] + coeff0_3*dmats1[3][15] + coeff0_4*dmats1[4][15] + coeff0_5*dmats1[5][15] + coeff0_6*dmats1[6][15] + coeff0_7*dmats1[7][15] + coeff0_8*dmats1[8][15] + coeff0_9*dmats1[9][15] + coeff0_10*dmats1[10][15] + coeff0_11*dmats1[11][15] + coeff0_12*dmats1[12][15] + coeff0_13*dmats1[13][15] + coeff0_14*dmats1[14][15] + coeff0_15*dmats1[15][15] + coeff0_16*dmats1[16][15] + coeff0_17*dmats1[17][15] + coeff0_18*dmats1[18][15] + coeff0_19*dmats1[19][15] + coeff0_20*dmats1[20][15] + coeff0_21*dmats1[21][15] + coeff0_22*dmats1[22][15] + coeff0_23*dmats1[23][15] + coeff0_24*dmats1[24][15] + coeff0_25*dmats1[25][15] + coeff0_26*dmats1[26][15] + coeff0_27*dmats1[27][15] + coeff0_28*dmats1[28][15] + coeff0_29*dmats1[29][15] + coeff0_30*dmats1[30][15] + coeff0_31*dmats1[31][15] + coeff0_32*dmats1[32][15] + coeff0_33*dmats1[33][15] + coeff0_34*dmats1[34][15] + coeff0_35*dmats1[35][15] + coeff0_36*dmats1[36][15] + coeff0_37*dmats1[37][15] + coeff0_38*dmats1[38][15] + coeff0_39*dmats1[39][15] + coeff0_40*dmats1[40][15] + coeff0_41*dmats1[41][15] + coeff0_42*dmats1[42][15] + coeff0_43*dmats1[43][15] + coeff0_44*dmats1[44][15] + coeff0_45*dmats1[45][15] + coeff0_46*dmats1[46][15] + coeff0_47*dmats1[47][15] + coeff0_48*dmats1[48][15] + coeff0_49*dmats1[49][15] + coeff0_50*dmats1[50][15] + coeff0_51*dmats1[51][15] + coeff0_52*dmats1[52][15] + coeff0_53*dmats1[53][15] + coeff0_54*dmats1[54][15] + coeff0_55*dmats1[55][15];
          new_coeff0_16 = coeff0_0*dmats1[0][16] + coeff0_1*dmats1[1][16] + coeff0_2*dmats1[2][16] + coeff0_3*dmats1[3][16] + coeff0_4*dmats1[4][16] + coeff0_5*dmats1[5][16] + coeff0_6*dmats1[6][16] + coeff0_7*dmats1[7][16] + coeff0_8*dmats1[8][16] + coeff0_9*dmats1[9][16] + coeff0_10*dmats1[10][16] + coeff0_11*dmats1[11][16] + coeff0_12*dmats1[12][16] + coeff0_13*dmats1[13][16] + coeff0_14*dmats1[14][16] + coeff0_15*dmats1[15][16] + coeff0_16*dmats1[16][16] + coeff0_17*dmats1[17][16] + coeff0_18*dmats1[18][16] + coeff0_19*dmats1[19][16] + coeff0_20*dmats1[20][16] + coeff0_21*dmats1[21][16] + coeff0_22*dmats1[22][16] + coeff0_23*dmats1[23][16] + coeff0_24*dmats1[24][16] + coeff0_25*dmats1[25][16] + coeff0_26*dmats1[26][16] + coeff0_27*dmats1[27][16] + coeff0_28*dmats1[28][16] + coeff0_29*dmats1[29][16] + coeff0_30*dmats1[30][16] + coeff0_31*dmats1[31][16] + coeff0_32*dmats1[32][16] + coeff0_33*dmats1[33][16] + coeff0_34*dmats1[34][16] + coeff0_35*dmats1[35][16] + coeff0_36*dmats1[36][16] + coeff0_37*dmats1[37][16] + coeff0_38*dmats1[38][16] + coeff0_39*dmats1[39][16] + coeff0_40*dmats1[40][16] + coeff0_41*dmats1[41][16] + coeff0_42*dmats1[42][16] + coeff0_43*dmats1[43][16] + coeff0_44*dmats1[44][16] + coeff0_45*dmats1[45][16] + coeff0_46*dmats1[46][16] + coeff0_47*dmats1[47][16] + coeff0_48*dmats1[48][16] + coeff0_49*dmats1[49][16] + coeff0_50*dmats1[50][16] + coeff0_51*dmats1[51][16] + coeff0_52*dmats1[52][16] + coeff0_53*dmats1[53][16] + coeff0_54*dmats1[54][16] + coeff0_55*dmats1[55][16];
          new_coeff0_17 = coeff0_0*dmats1[0][17] + coeff0_1*dmats1[1][17] + coeff0_2*dmats1[2][17] + coeff0_3*dmats1[3][17] + coeff0_4*dmats1[4][17] + coeff0_5*dmats1[5][17] + coeff0_6*dmats1[6][17] + coeff0_7*dmats1[7][17] + coeff0_8*dmats1[8][17] + coeff0_9*dmats1[9][17] + coeff0_10*dmats1[10][17] + coeff0_11*dmats1[11][17] + coeff0_12*dmats1[12][17] + coeff0_13*dmats1[13][17] + coeff0_14*dmats1[14][17] + coeff0_15*dmats1[15][17] + coeff0_16*dmats1[16][17] + coeff0_17*dmats1[17][17] + coeff0_18*dmats1[18][17] + coeff0_19*dmats1[19][17] + coeff0_20*dmats1[20][17] + coeff0_21*dmats1[21][17] + coeff0_22*dmats1[22][17] + coeff0_23*dmats1[23][17] + coeff0_24*dmats1[24][17] + coeff0_25*dmats1[25][17] + coeff0_26*dmats1[26][17] + coeff0_27*dmats1[27][17] + coeff0_28*dmats1[28][17] + coeff0_29*dmats1[29][17] + coeff0_30*dmats1[30][17] + coeff0_31*dmats1[31][17] + coeff0_32*dmats1[32][17] + coeff0_33*dmats1[33][17] + coeff0_34*dmats1[34][17] + coeff0_35*dmats1[35][17] + coeff0_36*dmats1[36][17] + coeff0_37*dmats1[37][17] + coeff0_38*dmats1[38][17] + coeff0_39*dmats1[39][17] + coeff0_40*dmats1[40][17] + coeff0_41*dmats1[41][17] + coeff0_42*dmats1[42][17] + coeff0_43*dmats1[43][17] + coeff0_44*dmats1[44][17] + coeff0_45*dmats1[45][17] + coeff0_46*dmats1[46][17] + coeff0_47*dmats1[47][17] + coeff0_48*dmats1[48][17] + coeff0_49*dmats1[49][17] + coeff0_50*dmats1[50][17] + coeff0_51*dmats1[51][17] + coeff0_52*dmats1[52][17] + coeff0_53*dmats1[53][17] + coeff0_54*dmats1[54][17] + coeff0_55*dmats1[55][17];
          new_coeff0_18 = coeff0_0*dmats1[0][18] + coeff0_1*dmats1[1][18] + coeff0_2*dmats1[2][18] + coeff0_3*dmats1[3][18] + coeff0_4*dmats1[4][18] + coeff0_5*dmats1[5][18] + coeff0_6*dmats1[6][18] + coeff0_7*dmats1[7][18] + coeff0_8*dmats1[8][18] + coeff0_9*dmats1[9][18] + coeff0_10*dmats1[10][18] + coeff0_11*dmats1[11][18] + coeff0_12*dmats1[12][18] + coeff0_13*dmats1[13][18] + coeff0_14*dmats1[14][18] + coeff0_15*dmats1[15][18] + coeff0_16*dmats1[16][18] + coeff0_17*dmats1[17][18] + coeff0_18*dmats1[18][18] + coeff0_19*dmats1[19][18] + coeff0_20*dmats1[20][18] + coeff0_21*dmats1[21][18] + coeff0_22*dmats1[22][18] + coeff0_23*dmats1[23][18] + coeff0_24*dmats1[24][18] + coeff0_25*dmats1[25][18] + coeff0_26*dmats1[26][18] + coeff0_27*dmats1[27][18] + coeff0_28*dmats1[28][18] + coeff0_29*dmats1[29][18] + coeff0_30*dmats1[30][18] + coeff0_31*dmats1[31][18] + coeff0_32*dmats1[32][18] + coeff0_33*dmats1[33][18] + coeff0_34*dmats1[34][18] + coeff0_35*dmats1[35][18] + coeff0_36*dmats1[36][18] + coeff0_37*dmats1[37][18] + coeff0_38*dmats1[38][18] + coeff0_39*dmats1[39][18] + coeff0_40*dmats1[40][18] + coeff0_41*dmats1[41][18] + coeff0_42*dmats1[42][18] + coeff0_43*dmats1[43][18] + coeff0_44*dmats1[44][18] + coeff0_45*dmats1[45][18] + coeff0_46*dmats1[46][18] + coeff0_47*dmats1[47][18] + coeff0_48*dmats1[48][18] + coeff0_49*dmats1[49][18] + coeff0_50*dmats1[50][18] + coeff0_51*dmats1[51][18] + coeff0_52*dmats1[52][18] + coeff0_53*dmats1[53][18] + coeff0_54*dmats1[54][18] + coeff0_55*dmats1[55][18];
          new_coeff0_19 = coeff0_0*dmats1[0][19] + coeff0_1*dmats1[1][19] + coeff0_2*dmats1[2][19] + coeff0_3*dmats1[3][19] + coeff0_4*dmats1[4][19] + coeff0_5*dmats1[5][19] + coeff0_6*dmats1[6][19] + coeff0_7*dmats1[7][19] + coeff0_8*dmats1[8][19] + coeff0_9*dmats1[9][19] + coeff0_10*dmats1[10][19] + coeff0_11*dmats1[11][19] + coeff0_12*dmats1[12][19] + coeff0_13*dmats1[13][19] + coeff0_14*dmats1[14][19] + coeff0_15*dmats1[15][19] + coeff0_16*dmats1[16][19] + coeff0_17*dmats1[17][19] + coeff0_18*dmats1[18][19] + coeff0_19*dmats1[19][19] + coeff0_20*dmats1[20][19] + coeff0_21*dmats1[21][19] + coeff0_22*dmats1[22][19] + coeff0_23*dmats1[23][19] + coeff0_24*dmats1[24][19] + coeff0_25*dmats1[25][19] + coeff0_26*dmats1[26][19] + coeff0_27*dmats1[27][19] + coeff0_28*dmats1[28][19] + coeff0_29*dmats1[29][19] + coeff0_30*dmats1[30][19] + coeff0_31*dmats1[31][19] + coeff0_32*dmats1[32][19] + coeff0_33*dmats1[33][19] + coeff0_34*dmats1[34][19] + coeff0_35*dmats1[35][19] + coeff0_36*dmats1[36][19] + coeff0_37*dmats1[37][19] + coeff0_38*dmats1[38][19] + coeff0_39*dmats1[39][19] + coeff0_40*dmats1[40][19] + coeff0_41*dmats1[41][19] + coeff0_42*dmats1[42][19] + coeff0_43*dmats1[43][19] + coeff0_44*dmats1[44][19] + coeff0_45*dmats1[45][19] + coeff0_46*dmats1[46][19] + coeff0_47*dmats1[47][19] + coeff0_48*dmats1[48][19] + coeff0_49*dmats1[49][19] + coeff0_50*dmats1[50][19] + coeff0_51*dmats1[51][19] + coeff0_52*dmats1[52][19] + coeff0_53*dmats1[53][19] + coeff0_54*dmats1[54][19] + coeff0_55*dmats1[55][19];
          new_coeff0_20 = coeff0_0*dmats1[0][20] + coeff0_1*dmats1[1][20] + coeff0_2*dmats1[2][20] + coeff0_3*dmats1[3][20] + coeff0_4*dmats1[4][20] + coeff0_5*dmats1[5][20] + coeff0_6*dmats1[6][20] + coeff0_7*dmats1[7][20] + coeff0_8*dmats1[8][20] + coeff0_9*dmats1[9][20] + coeff0_10*dmats1[10][20] + coeff0_11*dmats1[11][20] + coeff0_12*dmats1[12][20] + coeff0_13*dmats1[13][20] + coeff0_14*dmats1[14][20] + coeff0_15*dmats1[15][20] + coeff0_16*dmats1[16][20] + coeff0_17*dmats1[17][20] + coeff0_18*dmats1[18][20] + coeff0_19*dmats1[19][20] + coeff0_20*dmats1[20][20] + coeff0_21*dmats1[21][20] + coeff0_22*dmats1[22][20] + coeff0_23*dmats1[23][20] + coeff0_24*dmats1[24][20] + coeff0_25*dmats1[25][20] + coeff0_26*dmats1[26][20] + coeff0_27*dmats1[27][20] + coeff0_28*dmats1[28][20] + coeff0_29*dmats1[29][20] + coeff0_30*dmats1[30][20] + coeff0_31*dmats1[31][20] + coeff0_32*dmats1[32][20] + coeff0_33*dmats1[33][20] + coeff0_34*dmats1[34][20] + coeff0_35*dmats1[35][20] + coeff0_36*dmats1[36][20] + coeff0_37*dmats1[37][20] + coeff0_38*dmats1[38][20] + coeff0_39*dmats1[39][20] + coeff0_40*dmats1[40][20] + coeff0_41*dmats1[41][20] + coeff0_42*dmats1[42][20] + coeff0_43*dmats1[43][20] + coeff0_44*dmats1[44][20] + coeff0_45*dmats1[45][20] + coeff0_46*dmats1[46][20] + coeff0_47*dmats1[47][20] + coeff0_48*dmats1[48][20] + coeff0_49*dmats1[49][20] + coeff0_50*dmats1[50][20] + coeff0_51*dmats1[51][20] + coeff0_52*dmats1[52][20] + coeff0_53*dmats1[53][20] + coeff0_54*dmats1[54][20] + coeff0_55*dmats1[55][20];
          new_coeff0_21 = coeff0_0*dmats1[0][21] + coeff0_1*dmats1[1][21] + coeff0_2*dmats1[2][21] + coeff0_3*dmats1[3][21] + coeff0_4*dmats1[4][21] + coeff0_5*dmats1[5][21] + coeff0_6*dmats1[6][21] + coeff0_7*dmats1[7][21] + coeff0_8*dmats1[8][21] + coeff0_9*dmats1[9][21] + coeff0_10*dmats1[10][21] + coeff0_11*dmats1[11][21] + coeff0_12*dmats1[12][21] + coeff0_13*dmats1[13][21] + coeff0_14*dmats1[14][21] + coeff0_15*dmats1[15][21] + coeff0_16*dmats1[16][21] + coeff0_17*dmats1[17][21] + coeff0_18*dmats1[18][21] + coeff0_19*dmats1[19][21] + coeff0_20*dmats1[20][21] + coeff0_21*dmats1[21][21] + coeff0_22*dmats1[22][21] + coeff0_23*dmats1[23][21] + coeff0_24*dmats1[24][21] + coeff0_25*dmats1[25][21] + coeff0_26*dmats1[26][21] + coeff0_27*dmats1[27][21] + coeff0_28*dmats1[28][21] + coeff0_29*dmats1[29][21] + coeff0_30*dmats1[30][21] + coeff0_31*dmats1[31][21] + coeff0_32*dmats1[32][21] + coeff0_33*dmats1[33][21] + coeff0_34*dmats1[34][21] + coeff0_35*dmats1[35][21] + coeff0_36*dmats1[36][21] + coeff0_37*dmats1[37][21] + coeff0_38*dmats1[38][21] + coeff0_39*dmats1[39][21] + coeff0_40*dmats1[40][21] + coeff0_41*dmats1[41][21] + coeff0_42*dmats1[42][21] + coeff0_43*dmats1[43][21] + coeff0_44*dmats1[44][21] + coeff0_45*dmats1[45][21] + coeff0_46*dmats1[46][21] + coeff0_47*dmats1[47][21] + coeff0_48*dmats1[48][21] + coeff0_49*dmats1[49][21] + coeff0_50*dmats1[50][21] + coeff0_51*dmats1[51][21] + coeff0_52*dmats1[52][21] + coeff0_53*dmats1[53][21] + coeff0_54*dmats1[54][21] + coeff0_55*dmats1[55][21];
          new_coeff0_22 = coeff0_0*dmats1[0][22] + coeff0_1*dmats1[1][22] + coeff0_2*dmats1[2][22] + coeff0_3*dmats1[3][22] + coeff0_4*dmats1[4][22] + coeff0_5*dmats1[5][22] + coeff0_6*dmats1[6][22] + coeff0_7*dmats1[7][22] + coeff0_8*dmats1[8][22] + coeff0_9*dmats1[9][22] + coeff0_10*dmats1[10][22] + coeff0_11*dmats1[11][22] + coeff0_12*dmats1[12][22] + coeff0_13*dmats1[13][22] + coeff0_14*dmats1[14][22] + coeff0_15*dmats1[15][22] + coeff0_16*dmats1[16][22] + coeff0_17*dmats1[17][22] + coeff0_18*dmats1[18][22] + coeff0_19*dmats1[19][22] + coeff0_20*dmats1[20][22] + coeff0_21*dmats1[21][22] + coeff0_22*dmats1[22][22] + coeff0_23*dmats1[23][22] + coeff0_24*dmats1[24][22] + coeff0_25*dmats1[25][22] + coeff0_26*dmats1[26][22] + coeff0_27*dmats1[27][22] + coeff0_28*dmats1[28][22] + coeff0_29*dmats1[29][22] + coeff0_30*dmats1[30][22] + coeff0_31*dmats1[31][22] + coeff0_32*dmats1[32][22] + coeff0_33*dmats1[33][22] + coeff0_34*dmats1[34][22] + coeff0_35*dmats1[35][22] + coeff0_36*dmats1[36][22] + coeff0_37*dmats1[37][22] + coeff0_38*dmats1[38][22] + coeff0_39*dmats1[39][22] + coeff0_40*dmats1[40][22] + coeff0_41*dmats1[41][22] + coeff0_42*dmats1[42][22] + coeff0_43*dmats1[43][22] + coeff0_44*dmats1[44][22] + coeff0_45*dmats1[45][22] + coeff0_46*dmats1[46][22] + coeff0_47*dmats1[47][22] + coeff0_48*dmats1[48][22] + coeff0_49*dmats1[49][22] + coeff0_50*dmats1[50][22] + coeff0_51*dmats1[51][22] + coeff0_52*dmats1[52][22] + coeff0_53*dmats1[53][22] + coeff0_54*dmats1[54][22] + coeff0_55*dmats1[55][22];
          new_coeff0_23 = coeff0_0*dmats1[0][23] + coeff0_1*dmats1[1][23] + coeff0_2*dmats1[2][23] + coeff0_3*dmats1[3][23] + coeff0_4*dmats1[4][23] + coeff0_5*dmats1[5][23] + coeff0_6*dmats1[6][23] + coeff0_7*dmats1[7][23] + coeff0_8*dmats1[8][23] + coeff0_9*dmats1[9][23] + coeff0_10*dmats1[10][23] + coeff0_11*dmats1[11][23] + coeff0_12*dmats1[12][23] + coeff0_13*dmats1[13][23] + coeff0_14*dmats1[14][23] + coeff0_15*dmats1[15][23] + coeff0_16*dmats1[16][23] + coeff0_17*dmats1[17][23] + coeff0_18*dmats1[18][23] + coeff0_19*dmats1[19][23] + coeff0_20*dmats1[20][23] + coeff0_21*dmats1[21][23] + coeff0_22*dmats1[22][23] + coeff0_23*dmats1[23][23] + coeff0_24*dmats1[24][23] + coeff0_25*dmats1[25][23] + coeff0_26*dmats1[26][23] + coeff0_27*dmats1[27][23] + coeff0_28*dmats1[28][23] + coeff0_29*dmats1[29][23] + coeff0_30*dmats1[30][23] + coeff0_31*dmats1[31][23] + coeff0_32*dmats1[32][23] + coeff0_33*dmats1[33][23] + coeff0_34*dmats1[34][23] + coeff0_35*dmats1[35][23] + coeff0_36*dmats1[36][23] + coeff0_37*dmats1[37][23] + coeff0_38*dmats1[38][23] + coeff0_39*dmats1[39][23] + coeff0_40*dmats1[40][23] + coeff0_41*dmats1[41][23] + coeff0_42*dmats1[42][23] + coeff0_43*dmats1[43][23] + coeff0_44*dmats1[44][23] + coeff0_45*dmats1[45][23] + coeff0_46*dmats1[46][23] + coeff0_47*dmats1[47][23] + coeff0_48*dmats1[48][23] + coeff0_49*dmats1[49][23] + coeff0_50*dmats1[50][23] + coeff0_51*dmats1[51][23] + coeff0_52*dmats1[52][23] + coeff0_53*dmats1[53][23] + coeff0_54*dmats1[54][23] + coeff0_55*dmats1[55][23];
          new_coeff0_24 = coeff0_0*dmats1[0][24] + coeff0_1*dmats1[1][24] + coeff0_2*dmats1[2][24] + coeff0_3*dmats1[3][24] + coeff0_4*dmats1[4][24] + coeff0_5*dmats1[5][24] + coeff0_6*dmats1[6][24] + coeff0_7*dmats1[7][24] + coeff0_8*dmats1[8][24] + coeff0_9*dmats1[9][24] + coeff0_10*dmats1[10][24] + coeff0_11*dmats1[11][24] + coeff0_12*dmats1[12][24] + coeff0_13*dmats1[13][24] + coeff0_14*dmats1[14][24] + coeff0_15*dmats1[15][24] + coeff0_16*dmats1[16][24] + coeff0_17*dmats1[17][24] + coeff0_18*dmats1[18][24] + coeff0_19*dmats1[19][24] + coeff0_20*dmats1[20][24] + coeff0_21*dmats1[21][24] + coeff0_22*dmats1[22][24] + coeff0_23*dmats1[23][24] + coeff0_24*dmats1[24][24] + coeff0_25*dmats1[25][24] + coeff0_26*dmats1[26][24] + coeff0_27*dmats1[27][24] + coeff0_28*dmats1[28][24] + coeff0_29*dmats1[29][24] + coeff0_30*dmats1[30][24] + coeff0_31*dmats1[31][24] + coeff0_32*dmats1[32][24] + coeff0_33*dmats1[33][24] + coeff0_34*dmats1[34][24] + coeff0_35*dmats1[35][24] + coeff0_36*dmats1[36][24] + coeff0_37*dmats1[37][24] + coeff0_38*dmats1[38][24] + coeff0_39*dmats1[39][24] + coeff0_40*dmats1[40][24] + coeff0_41*dmats1[41][24] + coeff0_42*dmats1[42][24] + coeff0_43*dmats1[43][24] + coeff0_44*dmats1[44][24] + coeff0_45*dmats1[45][24] + coeff0_46*dmats1[46][24] + coeff0_47*dmats1[47][24] + coeff0_48*dmats1[48][24] + coeff0_49*dmats1[49][24] + coeff0_50*dmats1[50][24] + coeff0_51*dmats1[51][24] + coeff0_52*dmats1[52][24] + coeff0_53*dmats1[53][24] + coeff0_54*dmats1[54][24] + coeff0_55*dmats1[55][24];
          new_coeff0_25 = coeff0_0*dmats1[0][25] + coeff0_1*dmats1[1][25] + coeff0_2*dmats1[2][25] + coeff0_3*dmats1[3][25] + coeff0_4*dmats1[4][25] + coeff0_5*dmats1[5][25] + coeff0_6*dmats1[6][25] + coeff0_7*dmats1[7][25] + coeff0_8*dmats1[8][25] + coeff0_9*dmats1[9][25] + coeff0_10*dmats1[10][25] + coeff0_11*dmats1[11][25] + coeff0_12*dmats1[12][25] + coeff0_13*dmats1[13][25] + coeff0_14*dmats1[14][25] + coeff0_15*dmats1[15][25] + coeff0_16*dmats1[16][25] + coeff0_17*dmats1[17][25] + coeff0_18*dmats1[18][25] + coeff0_19*dmats1[19][25] + coeff0_20*dmats1[20][25] + coeff0_21*dmats1[21][25] + coeff0_22*dmats1[22][25] + coeff0_23*dmats1[23][25] + coeff0_24*dmats1[24][25] + coeff0_25*dmats1[25][25] + coeff0_26*dmats1[26][25] + coeff0_27*dmats1[27][25] + coeff0_28*dmats1[28][25] + coeff0_29*dmats1[29][25] + coeff0_30*dmats1[30][25] + coeff0_31*dmats1[31][25] + coeff0_32*dmats1[32][25] + coeff0_33*dmats1[33][25] + coeff0_34*dmats1[34][25] + coeff0_35*dmats1[35][25] + coeff0_36*dmats1[36][25] + coeff0_37*dmats1[37][25] + coeff0_38*dmats1[38][25] + coeff0_39*dmats1[39][25] + coeff0_40*dmats1[40][25] + coeff0_41*dmats1[41][25] + coeff0_42*dmats1[42][25] + coeff0_43*dmats1[43][25] + coeff0_44*dmats1[44][25] + coeff0_45*dmats1[45][25] + coeff0_46*dmats1[46][25] + coeff0_47*dmats1[47][25] + coeff0_48*dmats1[48][25] + coeff0_49*dmats1[49][25] + coeff0_50*dmats1[50][25] + coeff0_51*dmats1[51][25] + coeff0_52*dmats1[52][25] + coeff0_53*dmats1[53][25] + coeff0_54*dmats1[54][25] + coeff0_55*dmats1[55][25];
          new_coeff0_26 = coeff0_0*dmats1[0][26] + coeff0_1*dmats1[1][26] + coeff0_2*dmats1[2][26] + coeff0_3*dmats1[3][26] + coeff0_4*dmats1[4][26] + coeff0_5*dmats1[5][26] + coeff0_6*dmats1[6][26] + coeff0_7*dmats1[7][26] + coeff0_8*dmats1[8][26] + coeff0_9*dmats1[9][26] + coeff0_10*dmats1[10][26] + coeff0_11*dmats1[11][26] + coeff0_12*dmats1[12][26] + coeff0_13*dmats1[13][26] + coeff0_14*dmats1[14][26] + coeff0_15*dmats1[15][26] + coeff0_16*dmats1[16][26] + coeff0_17*dmats1[17][26] + coeff0_18*dmats1[18][26] + coeff0_19*dmats1[19][26] + coeff0_20*dmats1[20][26] + coeff0_21*dmats1[21][26] + coeff0_22*dmats1[22][26] + coeff0_23*dmats1[23][26] + coeff0_24*dmats1[24][26] + coeff0_25*dmats1[25][26] + coeff0_26*dmats1[26][26] + coeff0_27*dmats1[27][26] + coeff0_28*dmats1[28][26] + coeff0_29*dmats1[29][26] + coeff0_30*dmats1[30][26] + coeff0_31*dmats1[31][26] + coeff0_32*dmats1[32][26] + coeff0_33*dmats1[33][26] + coeff0_34*dmats1[34][26] + coeff0_35*dmats1[35][26] + coeff0_36*dmats1[36][26] + coeff0_37*dmats1[37][26] + coeff0_38*dmats1[38][26] + coeff0_39*dmats1[39][26] + coeff0_40*dmats1[40][26] + coeff0_41*dmats1[41][26] + coeff0_42*dmats1[42][26] + coeff0_43*dmats1[43][26] + coeff0_44*dmats1[44][26] + coeff0_45*dmats1[45][26] + coeff0_46*dmats1[46][26] + coeff0_47*dmats1[47][26] + coeff0_48*dmats1[48][26] + coeff0_49*dmats1[49][26] + coeff0_50*dmats1[50][26] + coeff0_51*dmats1[51][26] + coeff0_52*dmats1[52][26] + coeff0_53*dmats1[53][26] + coeff0_54*dmats1[54][26] + coeff0_55*dmats1[55][26];
          new_coeff0_27 = coeff0_0*dmats1[0][27] + coeff0_1*dmats1[1][27] + coeff0_2*dmats1[2][27] + coeff0_3*dmats1[3][27] + coeff0_4*dmats1[4][27] + coeff0_5*dmats1[5][27] + coeff0_6*dmats1[6][27] + coeff0_7*dmats1[7][27] + coeff0_8*dmats1[8][27] + coeff0_9*dmats1[9][27] + coeff0_10*dmats1[10][27] + coeff0_11*dmats1[11][27] + coeff0_12*dmats1[12][27] + coeff0_13*dmats1[13][27] + coeff0_14*dmats1[14][27] + coeff0_15*dmats1[15][27] + coeff0_16*dmats1[16][27] + coeff0_17*dmats1[17][27] + coeff0_18*dmats1[18][27] + coeff0_19*dmats1[19][27] + coeff0_20*dmats1[20][27] + coeff0_21*dmats1[21][27] + coeff0_22*dmats1[22][27] + coeff0_23*dmats1[23][27] + coeff0_24*dmats1[24][27] + coeff0_25*dmats1[25][27] + coeff0_26*dmats1[26][27] + coeff0_27*dmats1[27][27] + coeff0_28*dmats1[28][27] + coeff0_29*dmats1[29][27] + coeff0_30*dmats1[30][27] + coeff0_31*dmats1[31][27] + coeff0_32*dmats1[32][27] + coeff0_33*dmats1[33][27] + coeff0_34*dmats1[34][27] + coeff0_35*dmats1[35][27] + coeff0_36*dmats1[36][27] + coeff0_37*dmats1[37][27] + coeff0_38*dmats1[38][27] + coeff0_39*dmats1[39][27] + coeff0_40*dmats1[40][27] + coeff0_41*dmats1[41][27] + coeff0_42*dmats1[42][27] + coeff0_43*dmats1[43][27] + coeff0_44*dmats1[44][27] + coeff0_45*dmats1[45][27] + coeff0_46*dmats1[46][27] + coeff0_47*dmats1[47][27] + coeff0_48*dmats1[48][27] + coeff0_49*dmats1[49][27] + coeff0_50*dmats1[50][27] + coeff0_51*dmats1[51][27] + coeff0_52*dmats1[52][27] + coeff0_53*dmats1[53][27] + coeff0_54*dmats1[54][27] + coeff0_55*dmats1[55][27];
          new_coeff0_28 = coeff0_0*dmats1[0][28] + coeff0_1*dmats1[1][28] + coeff0_2*dmats1[2][28] + coeff0_3*dmats1[3][28] + coeff0_4*dmats1[4][28] + coeff0_5*dmats1[5][28] + coeff0_6*dmats1[6][28] + coeff0_7*dmats1[7][28] + coeff0_8*dmats1[8][28] + coeff0_9*dmats1[9][28] + coeff0_10*dmats1[10][28] + coeff0_11*dmats1[11][28] + coeff0_12*dmats1[12][28] + coeff0_13*dmats1[13][28] + coeff0_14*dmats1[14][28] + coeff0_15*dmats1[15][28] + coeff0_16*dmats1[16][28] + coeff0_17*dmats1[17][28] + coeff0_18*dmats1[18][28] + coeff0_19*dmats1[19][28] + coeff0_20*dmats1[20][28] + coeff0_21*dmats1[21][28] + coeff0_22*dmats1[22][28] + coeff0_23*dmats1[23][28] + coeff0_24*dmats1[24][28] + coeff0_25*dmats1[25][28] + coeff0_26*dmats1[26][28] + coeff0_27*dmats1[27][28] + coeff0_28*dmats1[28][28] + coeff0_29*dmats1[29][28] + coeff0_30*dmats1[30][28] + coeff0_31*dmats1[31][28] + coeff0_32*dmats1[32][28] + coeff0_33*dmats1[33][28] + coeff0_34*dmats1[34][28] + coeff0_35*dmats1[35][28] + coeff0_36*dmats1[36][28] + coeff0_37*dmats1[37][28] + coeff0_38*dmats1[38][28] + coeff0_39*dmats1[39][28] + coeff0_40*dmats1[40][28] + coeff0_41*dmats1[41][28] + coeff0_42*dmats1[42][28] + coeff0_43*dmats1[43][28] + coeff0_44*dmats1[44][28] + coeff0_45*dmats1[45][28] + coeff0_46*dmats1[46][28] + coeff0_47*dmats1[47][28] + coeff0_48*dmats1[48][28] + coeff0_49*dmats1[49][28] + coeff0_50*dmats1[50][28] + coeff0_51*dmats1[51][28] + coeff0_52*dmats1[52][28] + coeff0_53*dmats1[53][28] + coeff0_54*dmats1[54][28] + coeff0_55*dmats1[55][28];
          new_coeff0_29 = coeff0_0*dmats1[0][29] + coeff0_1*dmats1[1][29] + coeff0_2*dmats1[2][29] + coeff0_3*dmats1[3][29] + coeff0_4*dmats1[4][29] + coeff0_5*dmats1[5][29] + coeff0_6*dmats1[6][29] + coeff0_7*dmats1[7][29] + coeff0_8*dmats1[8][29] + coeff0_9*dmats1[9][29] + coeff0_10*dmats1[10][29] + coeff0_11*dmats1[11][29] + coeff0_12*dmats1[12][29] + coeff0_13*dmats1[13][29] + coeff0_14*dmats1[14][29] + coeff0_15*dmats1[15][29] + coeff0_16*dmats1[16][29] + coeff0_17*dmats1[17][29] + coeff0_18*dmats1[18][29] + coeff0_19*dmats1[19][29] + coeff0_20*dmats1[20][29] + coeff0_21*dmats1[21][29] + coeff0_22*dmats1[22][29] + coeff0_23*dmats1[23][29] + coeff0_24*dmats1[24][29] + coeff0_25*dmats1[25][29] + coeff0_26*dmats1[26][29] + coeff0_27*dmats1[27][29] + coeff0_28*dmats1[28][29] + coeff0_29*dmats1[29][29] + coeff0_30*dmats1[30][29] + coeff0_31*dmats1[31][29] + coeff0_32*dmats1[32][29] + coeff0_33*dmats1[33][29] + coeff0_34*dmats1[34][29] + coeff0_35*dmats1[35][29] + coeff0_36*dmats1[36][29] + coeff0_37*dmats1[37][29] + coeff0_38*dmats1[38][29] + coeff0_39*dmats1[39][29] + coeff0_40*dmats1[40][29] + coeff0_41*dmats1[41][29] + coeff0_42*dmats1[42][29] + coeff0_43*dmats1[43][29] + coeff0_44*dmats1[44][29] + coeff0_45*dmats1[45][29] + coeff0_46*dmats1[46][29] + coeff0_47*dmats1[47][29] + coeff0_48*dmats1[48][29] + coeff0_49*dmats1[49][29] + coeff0_50*dmats1[50][29] + coeff0_51*dmats1[51][29] + coeff0_52*dmats1[52][29] + coeff0_53*dmats1[53][29] + coeff0_54*dmats1[54][29] + coeff0_55*dmats1[55][29];
          new_coeff0_30 = coeff0_0*dmats1[0][30] + coeff0_1*dmats1[1][30] + coeff0_2*dmats1[2][30] + coeff0_3*dmats1[3][30] + coeff0_4*dmats1[4][30] + coeff0_5*dmats1[5][30] + coeff0_6*dmats1[6][30] + coeff0_7*dmats1[7][30] + coeff0_8*dmats1[8][30] + coeff0_9*dmats1[9][30] + coeff0_10*dmats1[10][30] + coeff0_11*dmats1[11][30] + coeff0_12*dmats1[12][30] + coeff0_13*dmats1[13][30] + coeff0_14*dmats1[14][30] + coeff0_15*dmats1[15][30] + coeff0_16*dmats1[16][30] + coeff0_17*dmats1[17][30] + coeff0_18*dmats1[18][30] + coeff0_19*dmats1[19][30] + coeff0_20*dmats1[20][30] + coeff0_21*dmats1[21][30] + coeff0_22*dmats1[22][30] + coeff0_23*dmats1[23][30] + coeff0_24*dmats1[24][30] + coeff0_25*dmats1[25][30] + coeff0_26*dmats1[26][30] + coeff0_27*dmats1[27][30] + coeff0_28*dmats1[28][30] + coeff0_29*dmats1[29][30] + coeff0_30*dmats1[30][30] + coeff0_31*dmats1[31][30] + coeff0_32*dmats1[32][30] + coeff0_33*dmats1[33][30] + coeff0_34*dmats1[34][30] + coeff0_35*dmats1[35][30] + coeff0_36*dmats1[36][30] + coeff0_37*dmats1[37][30] + coeff0_38*dmats1[38][30] + coeff0_39*dmats1[39][30] + coeff0_40*dmats1[40][30] + coeff0_41*dmats1[41][30] + coeff0_42*dmats1[42][30] + coeff0_43*dmats1[43][30] + coeff0_44*dmats1[44][30] + coeff0_45*dmats1[45][30] + coeff0_46*dmats1[46][30] + coeff0_47*dmats1[47][30] + coeff0_48*dmats1[48][30] + coeff0_49*dmats1[49][30] + coeff0_50*dmats1[50][30] + coeff0_51*dmats1[51][30] + coeff0_52*dmats1[52][30] + coeff0_53*dmats1[53][30] + coeff0_54*dmats1[54][30] + coeff0_55*dmats1[55][30];
          new_coeff0_31 = coeff0_0*dmats1[0][31] + coeff0_1*dmats1[1][31] + coeff0_2*dmats1[2][31] + coeff0_3*dmats1[3][31] + coeff0_4*dmats1[4][31] + coeff0_5*dmats1[5][31] + coeff0_6*dmats1[6][31] + coeff0_7*dmats1[7][31] + coeff0_8*dmats1[8][31] + coeff0_9*dmats1[9][31] + coeff0_10*dmats1[10][31] + coeff0_11*dmats1[11][31] + coeff0_12*dmats1[12][31] + coeff0_13*dmats1[13][31] + coeff0_14*dmats1[14][31] + coeff0_15*dmats1[15][31] + coeff0_16*dmats1[16][31] + coeff0_17*dmats1[17][31] + coeff0_18*dmats1[18][31] + coeff0_19*dmats1[19][31] + coeff0_20*dmats1[20][31] + coeff0_21*dmats1[21][31] + coeff0_22*dmats1[22][31] + coeff0_23*dmats1[23][31] + coeff0_24*dmats1[24][31] + coeff0_25*dmats1[25][31] + coeff0_26*dmats1[26][31] + coeff0_27*dmats1[27][31] + coeff0_28*dmats1[28][31] + coeff0_29*dmats1[29][31] + coeff0_30*dmats1[30][31] + coeff0_31*dmats1[31][31] + coeff0_32*dmats1[32][31] + coeff0_33*dmats1[33][31] + coeff0_34*dmats1[34][31] + coeff0_35*dmats1[35][31] + coeff0_36*dmats1[36][31] + coeff0_37*dmats1[37][31] + coeff0_38*dmats1[38][31] + coeff0_39*dmats1[39][31] + coeff0_40*dmats1[40][31] + coeff0_41*dmats1[41][31] + coeff0_42*dmats1[42][31] + coeff0_43*dmats1[43][31] + coeff0_44*dmats1[44][31] + coeff0_45*dmats1[45][31] + coeff0_46*dmats1[46][31] + coeff0_47*dmats1[47][31] + coeff0_48*dmats1[48][31] + coeff0_49*dmats1[49][31] + coeff0_50*dmats1[50][31] + coeff0_51*dmats1[51][31] + coeff0_52*dmats1[52][31] + coeff0_53*dmats1[53][31] + coeff0_54*dmats1[54][31] + coeff0_55*dmats1[55][31];
          new_coeff0_32 = coeff0_0*dmats1[0][32] + coeff0_1*dmats1[1][32] + coeff0_2*dmats1[2][32] + coeff0_3*dmats1[3][32] + coeff0_4*dmats1[4][32] + coeff0_5*dmats1[5][32] + coeff0_6*dmats1[6][32] + coeff0_7*dmats1[7][32] + coeff0_8*dmats1[8][32] + coeff0_9*dmats1[9][32] + coeff0_10*dmats1[10][32] + coeff0_11*dmats1[11][32] + coeff0_12*dmats1[12][32] + coeff0_13*dmats1[13][32] + coeff0_14*dmats1[14][32] + coeff0_15*dmats1[15][32] + coeff0_16*dmats1[16][32] + coeff0_17*dmats1[17][32] + coeff0_18*dmats1[18][32] + coeff0_19*dmats1[19][32] + coeff0_20*dmats1[20][32] + coeff0_21*dmats1[21][32] + coeff0_22*dmats1[22][32] + coeff0_23*dmats1[23][32] + coeff0_24*dmats1[24][32] + coeff0_25*dmats1[25][32] + coeff0_26*dmats1[26][32] + coeff0_27*dmats1[27][32] + coeff0_28*dmats1[28][32] + coeff0_29*dmats1[29][32] + coeff0_30*dmats1[30][32] + coeff0_31*dmats1[31][32] + coeff0_32*dmats1[32][32] + coeff0_33*dmats1[33][32] + coeff0_34*dmats1[34][32] + coeff0_35*dmats1[35][32] + coeff0_36*dmats1[36][32] + coeff0_37*dmats1[37][32] + coeff0_38*dmats1[38][32] + coeff0_39*dmats1[39][32] + coeff0_40*dmats1[40][32] + coeff0_41*dmats1[41][32] + coeff0_42*dmats1[42][32] + coeff0_43*dmats1[43][32] + coeff0_44*dmats1[44][32] + coeff0_45*dmats1[45][32] + coeff0_46*dmats1[46][32] + coeff0_47*dmats1[47][32] + coeff0_48*dmats1[48][32] + coeff0_49*dmats1[49][32] + coeff0_50*dmats1[50][32] + coeff0_51*dmats1[51][32] + coeff0_52*dmats1[52][32] + coeff0_53*dmats1[53][32] + coeff0_54*dmats1[54][32] + coeff0_55*dmats1[55][32];
          new_coeff0_33 = coeff0_0*dmats1[0][33] + coeff0_1*dmats1[1][33] + coeff0_2*dmats1[2][33] + coeff0_3*dmats1[3][33] + coeff0_4*dmats1[4][33] + coeff0_5*dmats1[5][33] + coeff0_6*dmats1[6][33] + coeff0_7*dmats1[7][33] + coeff0_8*dmats1[8][33] + coeff0_9*dmats1[9][33] + coeff0_10*dmats1[10][33] + coeff0_11*dmats1[11][33] + coeff0_12*dmats1[12][33] + coeff0_13*dmats1[13][33] + coeff0_14*dmats1[14][33] + coeff0_15*dmats1[15][33] + coeff0_16*dmats1[16][33] + coeff0_17*dmats1[17][33] + coeff0_18*dmats1[18][33] + coeff0_19*dmats1[19][33] + coeff0_20*dmats1[20][33] + coeff0_21*dmats1[21][33] + coeff0_22*dmats1[22][33] + coeff0_23*dmats1[23][33] + coeff0_24*dmats1[24][33] + coeff0_25*dmats1[25][33] + coeff0_26*dmats1[26][33] + coeff0_27*dmats1[27][33] + coeff0_28*dmats1[28][33] + coeff0_29*dmats1[29][33] + coeff0_30*dmats1[30][33] + coeff0_31*dmats1[31][33] + coeff0_32*dmats1[32][33] + coeff0_33*dmats1[33][33] + coeff0_34*dmats1[34][33] + coeff0_35*dmats1[35][33] + coeff0_36*dmats1[36][33] + coeff0_37*dmats1[37][33] + coeff0_38*dmats1[38][33] + coeff0_39*dmats1[39][33] + coeff0_40*dmats1[40][33] + coeff0_41*dmats1[41][33] + coeff0_42*dmats1[42][33] + coeff0_43*dmats1[43][33] + coeff0_44*dmats1[44][33] + coeff0_45*dmats1[45][33] + coeff0_46*dmats1[46][33] + coeff0_47*dmats1[47][33] + coeff0_48*dmats1[48][33] + coeff0_49*dmats1[49][33] + coeff0_50*dmats1[50][33] + coeff0_51*dmats1[51][33] + coeff0_52*dmats1[52][33] + coeff0_53*dmats1[53][33] + coeff0_54*dmats1[54][33] + coeff0_55*dmats1[55][33];
          new_coeff0_34 = coeff0_0*dmats1[0][34] + coeff0_1*dmats1[1][34] + coeff0_2*dmats1[2][34] + coeff0_3*dmats1[3][34] + coeff0_4*dmats1[4][34] + coeff0_5*dmats1[5][34] + coeff0_6*dmats1[6][34] + coeff0_7*dmats1[7][34] + coeff0_8*dmats1[8][34] + coeff0_9*dmats1[9][34] + coeff0_10*dmats1[10][34] + coeff0_11*dmats1[11][34] + coeff0_12*dmats1[12][34] + coeff0_13*dmats1[13][34] + coeff0_14*dmats1[14][34] + coeff0_15*dmats1[15][34] + coeff0_16*dmats1[16][34] + coeff0_17*dmats1[17][34] + coeff0_18*dmats1[18][34] + coeff0_19*dmats1[19][34] + coeff0_20*dmats1[20][34] + coeff0_21*dmats1[21][34] + coeff0_22*dmats1[22][34] + coeff0_23*dmats1[23][34] + coeff0_24*dmats1[24][34] + coeff0_25*dmats1[25][34] + coeff0_26*dmats1[26][34] + coeff0_27*dmats1[27][34] + coeff0_28*dmats1[28][34] + coeff0_29*dmats1[29][34] + coeff0_30*dmats1[30][34] + coeff0_31*dmats1[31][34] + coeff0_32*dmats1[32][34] + coeff0_33*dmats1[33][34] + coeff0_34*dmats1[34][34] + coeff0_35*dmats1[35][34] + coeff0_36*dmats1[36][34] + coeff0_37*dmats1[37][34] + coeff0_38*dmats1[38][34] + coeff0_39*dmats1[39][34] + coeff0_40*dmats1[40][34] + coeff0_41*dmats1[41][34] + coeff0_42*dmats1[42][34] + coeff0_43*dmats1[43][34] + coeff0_44*dmats1[44][34] + coeff0_45*dmats1[45][34] + coeff0_46*dmats1[46][34] + coeff0_47*dmats1[47][34] + coeff0_48*dmats1[48][34] + coeff0_49*dmats1[49][34] + coeff0_50*dmats1[50][34] + coeff0_51*dmats1[51][34] + coeff0_52*dmats1[52][34] + coeff0_53*dmats1[53][34] + coeff0_54*dmats1[54][34] + coeff0_55*dmats1[55][34];
          new_coeff0_35 = coeff0_0*dmats1[0][35] + coeff0_1*dmats1[1][35] + coeff0_2*dmats1[2][35] + coeff0_3*dmats1[3][35] + coeff0_4*dmats1[4][35] + coeff0_5*dmats1[5][35] + coeff0_6*dmats1[6][35] + coeff0_7*dmats1[7][35] + coeff0_8*dmats1[8][35] + coeff0_9*dmats1[9][35] + coeff0_10*dmats1[10][35] + coeff0_11*dmats1[11][35] + coeff0_12*dmats1[12][35] + coeff0_13*dmats1[13][35] + coeff0_14*dmats1[14][35] + coeff0_15*dmats1[15][35] + coeff0_16*dmats1[16][35] + coeff0_17*dmats1[17][35] + coeff0_18*dmats1[18][35] + coeff0_19*dmats1[19][35] + coeff0_20*dmats1[20][35] + coeff0_21*dmats1[21][35] + coeff0_22*dmats1[22][35] + coeff0_23*dmats1[23][35] + coeff0_24*dmats1[24][35] + coeff0_25*dmats1[25][35] + coeff0_26*dmats1[26][35] + coeff0_27*dmats1[27][35] + coeff0_28*dmats1[28][35] + coeff0_29*dmats1[29][35] + coeff0_30*dmats1[30][35] + coeff0_31*dmats1[31][35] + coeff0_32*dmats1[32][35] + coeff0_33*dmats1[33][35] + coeff0_34*dmats1[34][35] + coeff0_35*dmats1[35][35] + coeff0_36*dmats1[36][35] + coeff0_37*dmats1[37][35] + coeff0_38*dmats1[38][35] + coeff0_39*dmats1[39][35] + coeff0_40*dmats1[40][35] + coeff0_41*dmats1[41][35] + coeff0_42*dmats1[42][35] + coeff0_43*dmats1[43][35] + coeff0_44*dmats1[44][35] + coeff0_45*dmats1[45][35] + coeff0_46*dmats1[46][35] + coeff0_47*dmats1[47][35] + coeff0_48*dmats1[48][35] + coeff0_49*dmats1[49][35] + coeff0_50*dmats1[50][35] + coeff0_51*dmats1[51][35] + coeff0_52*dmats1[52][35] + coeff0_53*dmats1[53][35] + coeff0_54*dmats1[54][35] + coeff0_55*dmats1[55][35];
          new_coeff0_36 = coeff0_0*dmats1[0][36] + coeff0_1*dmats1[1][36] + coeff0_2*dmats1[2][36] + coeff0_3*dmats1[3][36] + coeff0_4*dmats1[4][36] + coeff0_5*dmats1[5][36] + coeff0_6*dmats1[6][36] + coeff0_7*dmats1[7][36] + coeff0_8*dmats1[8][36] + coeff0_9*dmats1[9][36] + coeff0_10*dmats1[10][36] + coeff0_11*dmats1[11][36] + coeff0_12*dmats1[12][36] + coeff0_13*dmats1[13][36] + coeff0_14*dmats1[14][36] + coeff0_15*dmats1[15][36] + coeff0_16*dmats1[16][36] + coeff0_17*dmats1[17][36] + coeff0_18*dmats1[18][36] + coeff0_19*dmats1[19][36] + coeff0_20*dmats1[20][36] + coeff0_21*dmats1[21][36] + coeff0_22*dmats1[22][36] + coeff0_23*dmats1[23][36] + coeff0_24*dmats1[24][36] + coeff0_25*dmats1[25][36] + coeff0_26*dmats1[26][36] + coeff0_27*dmats1[27][36] + coeff0_28*dmats1[28][36] + coeff0_29*dmats1[29][36] + coeff0_30*dmats1[30][36] + coeff0_31*dmats1[31][36] + coeff0_32*dmats1[32][36] + coeff0_33*dmats1[33][36] + coeff0_34*dmats1[34][36] + coeff0_35*dmats1[35][36] + coeff0_36*dmats1[36][36] + coeff0_37*dmats1[37][36] + coeff0_38*dmats1[38][36] + coeff0_39*dmats1[39][36] + coeff0_40*dmats1[40][36] + coeff0_41*dmats1[41][36] + coeff0_42*dmats1[42][36] + coeff0_43*dmats1[43][36] + coeff0_44*dmats1[44][36] + coeff0_45*dmats1[45][36] + coeff0_46*dmats1[46][36] + coeff0_47*dmats1[47][36] + coeff0_48*dmats1[48][36] + coeff0_49*dmats1[49][36] + coeff0_50*dmats1[50][36] + coeff0_51*dmats1[51][36] + coeff0_52*dmats1[52][36] + coeff0_53*dmats1[53][36] + coeff0_54*dmats1[54][36] + coeff0_55*dmats1[55][36];
          new_coeff0_37 = coeff0_0*dmats1[0][37] + coeff0_1*dmats1[1][37] + coeff0_2*dmats1[2][37] + coeff0_3*dmats1[3][37] + coeff0_4*dmats1[4][37] + coeff0_5*dmats1[5][37] + coeff0_6*dmats1[6][37] + coeff0_7*dmats1[7][37] + coeff0_8*dmats1[8][37] + coeff0_9*dmats1[9][37] + coeff0_10*dmats1[10][37] + coeff0_11*dmats1[11][37] + coeff0_12*dmats1[12][37] + coeff0_13*dmats1[13][37] + coeff0_14*dmats1[14][37] + coeff0_15*dmats1[15][37] + coeff0_16*dmats1[16][37] + coeff0_17*dmats1[17][37] + coeff0_18*dmats1[18][37] + coeff0_19*dmats1[19][37] + coeff0_20*dmats1[20][37] + coeff0_21*dmats1[21][37] + coeff0_22*dmats1[22][37] + coeff0_23*dmats1[23][37] + coeff0_24*dmats1[24][37] + coeff0_25*dmats1[25][37] + coeff0_26*dmats1[26][37] + coeff0_27*dmats1[27][37] + coeff0_28*dmats1[28][37] + coeff0_29*dmats1[29][37] + coeff0_30*dmats1[30][37] + coeff0_31*dmats1[31][37] + coeff0_32*dmats1[32][37] + coeff0_33*dmats1[33][37] + coeff0_34*dmats1[34][37] + coeff0_35*dmats1[35][37] + coeff0_36*dmats1[36][37] + coeff0_37*dmats1[37][37] + coeff0_38*dmats1[38][37] + coeff0_39*dmats1[39][37] + coeff0_40*dmats1[40][37] + coeff0_41*dmats1[41][37] + coeff0_42*dmats1[42][37] + coeff0_43*dmats1[43][37] + coeff0_44*dmats1[44][37] + coeff0_45*dmats1[45][37] + coeff0_46*dmats1[46][37] + coeff0_47*dmats1[47][37] + coeff0_48*dmats1[48][37] + coeff0_49*dmats1[49][37] + coeff0_50*dmats1[50][37] + coeff0_51*dmats1[51][37] + coeff0_52*dmats1[52][37] + coeff0_53*dmats1[53][37] + coeff0_54*dmats1[54][37] + coeff0_55*dmats1[55][37];
          new_coeff0_38 = coeff0_0*dmats1[0][38] + coeff0_1*dmats1[1][38] + coeff0_2*dmats1[2][38] + coeff0_3*dmats1[3][38] + coeff0_4*dmats1[4][38] + coeff0_5*dmats1[5][38] + coeff0_6*dmats1[6][38] + coeff0_7*dmats1[7][38] + coeff0_8*dmats1[8][38] + coeff0_9*dmats1[9][38] + coeff0_10*dmats1[10][38] + coeff0_11*dmats1[11][38] + coeff0_12*dmats1[12][38] + coeff0_13*dmats1[13][38] + coeff0_14*dmats1[14][38] + coeff0_15*dmats1[15][38] + coeff0_16*dmats1[16][38] + coeff0_17*dmats1[17][38] + coeff0_18*dmats1[18][38] + coeff0_19*dmats1[19][38] + coeff0_20*dmats1[20][38] + coeff0_21*dmats1[21][38] + coeff0_22*dmats1[22][38] + coeff0_23*dmats1[23][38] + coeff0_24*dmats1[24][38] + coeff0_25*dmats1[25][38] + coeff0_26*dmats1[26][38] + coeff0_27*dmats1[27][38] + coeff0_28*dmats1[28][38] + coeff0_29*dmats1[29][38] + coeff0_30*dmats1[30][38] + coeff0_31*dmats1[31][38] + coeff0_32*dmats1[32][38] + coeff0_33*dmats1[33][38] + coeff0_34*dmats1[34][38] + coeff0_35*dmats1[35][38] + coeff0_36*dmats1[36][38] + coeff0_37*dmats1[37][38] + coeff0_38*dmats1[38][38] + coeff0_39*dmats1[39][38] + coeff0_40*dmats1[40][38] + coeff0_41*dmats1[41][38] + coeff0_42*dmats1[42][38] + coeff0_43*dmats1[43][38] + coeff0_44*dmats1[44][38] + coeff0_45*dmats1[45][38] + coeff0_46*dmats1[46][38] + coeff0_47*dmats1[47][38] + coeff0_48*dmats1[48][38] + coeff0_49*dmats1[49][38] + coeff0_50*dmats1[50][38] + coeff0_51*dmats1[51][38] + coeff0_52*dmats1[52][38] + coeff0_53*dmats1[53][38] + coeff0_54*dmats1[54][38] + coeff0_55*dmats1[55][38];
          new_coeff0_39 = coeff0_0*dmats1[0][39] + coeff0_1*dmats1[1][39] + coeff0_2*dmats1[2][39] + coeff0_3*dmats1[3][39] + coeff0_4*dmats1[4][39] + coeff0_5*dmats1[5][39] + coeff0_6*dmats1[6][39] + coeff0_7*dmats1[7][39] + coeff0_8*dmats1[8][39] + coeff0_9*dmats1[9][39] + coeff0_10*dmats1[10][39] + coeff0_11*dmats1[11][39] + coeff0_12*dmats1[12][39] + coeff0_13*dmats1[13][39] + coeff0_14*dmats1[14][39] + coeff0_15*dmats1[15][39] + coeff0_16*dmats1[16][39] + coeff0_17*dmats1[17][39] + coeff0_18*dmats1[18][39] + coeff0_19*dmats1[19][39] + coeff0_20*dmats1[20][39] + coeff0_21*dmats1[21][39] + coeff0_22*dmats1[22][39] + coeff0_23*dmats1[23][39] + coeff0_24*dmats1[24][39] + coeff0_25*dmats1[25][39] + coeff0_26*dmats1[26][39] + coeff0_27*dmats1[27][39] + coeff0_28*dmats1[28][39] + coeff0_29*dmats1[29][39] + coeff0_30*dmats1[30][39] + coeff0_31*dmats1[31][39] + coeff0_32*dmats1[32][39] + coeff0_33*dmats1[33][39] + coeff0_34*dmats1[34][39] + coeff0_35*dmats1[35][39] + coeff0_36*dmats1[36][39] + coeff0_37*dmats1[37][39] + coeff0_38*dmats1[38][39] + coeff0_39*dmats1[39][39] + coeff0_40*dmats1[40][39] + coeff0_41*dmats1[41][39] + coeff0_42*dmats1[42][39] + coeff0_43*dmats1[43][39] + coeff0_44*dmats1[44][39] + coeff0_45*dmats1[45][39] + coeff0_46*dmats1[46][39] + coeff0_47*dmats1[47][39] + coeff0_48*dmats1[48][39] + coeff0_49*dmats1[49][39] + coeff0_50*dmats1[50][39] + coeff0_51*dmats1[51][39] + coeff0_52*dmats1[52][39] + coeff0_53*dmats1[53][39] + coeff0_54*dmats1[54][39] + coeff0_55*dmats1[55][39];
          new_coeff0_40 = coeff0_0*dmats1[0][40] + coeff0_1*dmats1[1][40] + coeff0_2*dmats1[2][40] + coeff0_3*dmats1[3][40] + coeff0_4*dmats1[4][40] + coeff0_5*dmats1[5][40] + coeff0_6*dmats1[6][40] + coeff0_7*dmats1[7][40] + coeff0_8*dmats1[8][40] + coeff0_9*dmats1[9][40] + coeff0_10*dmats1[10][40] + coeff0_11*dmats1[11][40] + coeff0_12*dmats1[12][40] + coeff0_13*dmats1[13][40] + coeff0_14*dmats1[14][40] + coeff0_15*dmats1[15][40] + coeff0_16*dmats1[16][40] + coeff0_17*dmats1[17][40] + coeff0_18*dmats1[18][40] + coeff0_19*dmats1[19][40] + coeff0_20*dmats1[20][40] + coeff0_21*dmats1[21][40] + coeff0_22*dmats1[22][40] + coeff0_23*dmats1[23][40] + coeff0_24*dmats1[24][40] + coeff0_25*dmats1[25][40] + coeff0_26*dmats1[26][40] + coeff0_27*dmats1[27][40] + coeff0_28*dmats1[28][40] + coeff0_29*dmats1[29][40] + coeff0_30*dmats1[30][40] + coeff0_31*dmats1[31][40] + coeff0_32*dmats1[32][40] + coeff0_33*dmats1[33][40] + coeff0_34*dmats1[34][40] + coeff0_35*dmats1[35][40] + coeff0_36*dmats1[36][40] + coeff0_37*dmats1[37][40] + coeff0_38*dmats1[38][40] + coeff0_39*dmats1[39][40] + coeff0_40*dmats1[40][40] + coeff0_41*dmats1[41][40] + coeff0_42*dmats1[42][40] + coeff0_43*dmats1[43][40] + coeff0_44*dmats1[44][40] + coeff0_45*dmats1[45][40] + coeff0_46*dmats1[46][40] + coeff0_47*dmats1[47][40] + coeff0_48*dmats1[48][40] + coeff0_49*dmats1[49][40] + coeff0_50*dmats1[50][40] + coeff0_51*dmats1[51][40] + coeff0_52*dmats1[52][40] + coeff0_53*dmats1[53][40] + coeff0_54*dmats1[54][40] + coeff0_55*dmats1[55][40];
          new_coeff0_41 = coeff0_0*dmats1[0][41] + coeff0_1*dmats1[1][41] + coeff0_2*dmats1[2][41] + coeff0_3*dmats1[3][41] + coeff0_4*dmats1[4][41] + coeff0_5*dmats1[5][41] + coeff0_6*dmats1[6][41] + coeff0_7*dmats1[7][41] + coeff0_8*dmats1[8][41] + coeff0_9*dmats1[9][41] + coeff0_10*dmats1[10][41] + coeff0_11*dmats1[11][41] + coeff0_12*dmats1[12][41] + coeff0_13*dmats1[13][41] + coeff0_14*dmats1[14][41] + coeff0_15*dmats1[15][41] + coeff0_16*dmats1[16][41] + coeff0_17*dmats1[17][41] + coeff0_18*dmats1[18][41] + coeff0_19*dmats1[19][41] + coeff0_20*dmats1[20][41] + coeff0_21*dmats1[21][41] + coeff0_22*dmats1[22][41] + coeff0_23*dmats1[23][41] + coeff0_24*dmats1[24][41] + coeff0_25*dmats1[25][41] + coeff0_26*dmats1[26][41] + coeff0_27*dmats1[27][41] + coeff0_28*dmats1[28][41] + coeff0_29*dmats1[29][41] + coeff0_30*dmats1[30][41] + coeff0_31*dmats1[31][41] + coeff0_32*dmats1[32][41] + coeff0_33*dmats1[33][41] + coeff0_34*dmats1[34][41] + coeff0_35*dmats1[35][41] + coeff0_36*dmats1[36][41] + coeff0_37*dmats1[37][41] + coeff0_38*dmats1[38][41] + coeff0_39*dmats1[39][41] + coeff0_40*dmats1[40][41] + coeff0_41*dmats1[41][41] + coeff0_42*dmats1[42][41] + coeff0_43*dmats1[43][41] + coeff0_44*dmats1[44][41] + coeff0_45*dmats1[45][41] + coeff0_46*dmats1[46][41] + coeff0_47*dmats1[47][41] + coeff0_48*dmats1[48][41] + coeff0_49*dmats1[49][41] + coeff0_50*dmats1[50][41] + coeff0_51*dmats1[51][41] + coeff0_52*dmats1[52][41] + coeff0_53*dmats1[53][41] + coeff0_54*dmats1[54][41] + coeff0_55*dmats1[55][41];
          new_coeff0_42 = coeff0_0*dmats1[0][42] + coeff0_1*dmats1[1][42] + coeff0_2*dmats1[2][42] + coeff0_3*dmats1[3][42] + coeff0_4*dmats1[4][42] + coeff0_5*dmats1[5][42] + coeff0_6*dmats1[6][42] + coeff0_7*dmats1[7][42] + coeff0_8*dmats1[8][42] + coeff0_9*dmats1[9][42] + coeff0_10*dmats1[10][42] + coeff0_11*dmats1[11][42] + coeff0_12*dmats1[12][42] + coeff0_13*dmats1[13][42] + coeff0_14*dmats1[14][42] + coeff0_15*dmats1[15][42] + coeff0_16*dmats1[16][42] + coeff0_17*dmats1[17][42] + coeff0_18*dmats1[18][42] + coeff0_19*dmats1[19][42] + coeff0_20*dmats1[20][42] + coeff0_21*dmats1[21][42] + coeff0_22*dmats1[22][42] + coeff0_23*dmats1[23][42] + coeff0_24*dmats1[24][42] + coeff0_25*dmats1[25][42] + coeff0_26*dmats1[26][42] + coeff0_27*dmats1[27][42] + coeff0_28*dmats1[28][42] + coeff0_29*dmats1[29][42] + coeff0_30*dmats1[30][42] + coeff0_31*dmats1[31][42] + coeff0_32*dmats1[32][42] + coeff0_33*dmats1[33][42] + coeff0_34*dmats1[34][42] + coeff0_35*dmats1[35][42] + coeff0_36*dmats1[36][42] + coeff0_37*dmats1[37][42] + coeff0_38*dmats1[38][42] + coeff0_39*dmats1[39][42] + coeff0_40*dmats1[40][42] + coeff0_41*dmats1[41][42] + coeff0_42*dmats1[42][42] + coeff0_43*dmats1[43][42] + coeff0_44*dmats1[44][42] + coeff0_45*dmats1[45][42] + coeff0_46*dmats1[46][42] + coeff0_47*dmats1[47][42] + coeff0_48*dmats1[48][42] + coeff0_49*dmats1[49][42] + coeff0_50*dmats1[50][42] + coeff0_51*dmats1[51][42] + coeff0_52*dmats1[52][42] + coeff0_53*dmats1[53][42] + coeff0_54*dmats1[54][42] + coeff0_55*dmats1[55][42];
          new_coeff0_43 = coeff0_0*dmats1[0][43] + coeff0_1*dmats1[1][43] + coeff0_2*dmats1[2][43] + coeff0_3*dmats1[3][43] + coeff0_4*dmats1[4][43] + coeff0_5*dmats1[5][43] + coeff0_6*dmats1[6][43] + coeff0_7*dmats1[7][43] + coeff0_8*dmats1[8][43] + coeff0_9*dmats1[9][43] + coeff0_10*dmats1[10][43] + coeff0_11*dmats1[11][43] + coeff0_12*dmats1[12][43] + coeff0_13*dmats1[13][43] + coeff0_14*dmats1[14][43] + coeff0_15*dmats1[15][43] + coeff0_16*dmats1[16][43] + coeff0_17*dmats1[17][43] + coeff0_18*dmats1[18][43] + coeff0_19*dmats1[19][43] + coeff0_20*dmats1[20][43] + coeff0_21*dmats1[21][43] + coeff0_22*dmats1[22][43] + coeff0_23*dmats1[23][43] + coeff0_24*dmats1[24][43] + coeff0_25*dmats1[25][43] + coeff0_26*dmats1[26][43] + coeff0_27*dmats1[27][43] + coeff0_28*dmats1[28][43] + coeff0_29*dmats1[29][43] + coeff0_30*dmats1[30][43] + coeff0_31*dmats1[31][43] + coeff0_32*dmats1[32][43] + coeff0_33*dmats1[33][43] + coeff0_34*dmats1[34][43] + coeff0_35*dmats1[35][43] + coeff0_36*dmats1[36][43] + coeff0_37*dmats1[37][43] + coeff0_38*dmats1[38][43] + coeff0_39*dmats1[39][43] + coeff0_40*dmats1[40][43] + coeff0_41*dmats1[41][43] + coeff0_42*dmats1[42][43] + coeff0_43*dmats1[43][43] + coeff0_44*dmats1[44][43] + coeff0_45*dmats1[45][43] + coeff0_46*dmats1[46][43] + coeff0_47*dmats1[47][43] + coeff0_48*dmats1[48][43] + coeff0_49*dmats1[49][43] + coeff0_50*dmats1[50][43] + coeff0_51*dmats1[51][43] + coeff0_52*dmats1[52][43] + coeff0_53*dmats1[53][43] + coeff0_54*dmats1[54][43] + coeff0_55*dmats1[55][43];
          new_coeff0_44 = coeff0_0*dmats1[0][44] + coeff0_1*dmats1[1][44] + coeff0_2*dmats1[2][44] + coeff0_3*dmats1[3][44] + coeff0_4*dmats1[4][44] + coeff0_5*dmats1[5][44] + coeff0_6*dmats1[6][44] + coeff0_7*dmats1[7][44] + coeff0_8*dmats1[8][44] + coeff0_9*dmats1[9][44] + coeff0_10*dmats1[10][44] + coeff0_11*dmats1[11][44] + coeff0_12*dmats1[12][44] + coeff0_13*dmats1[13][44] + coeff0_14*dmats1[14][44] + coeff0_15*dmats1[15][44] + coeff0_16*dmats1[16][44] + coeff0_17*dmats1[17][44] + coeff0_18*dmats1[18][44] + coeff0_19*dmats1[19][44] + coeff0_20*dmats1[20][44] + coeff0_21*dmats1[21][44] + coeff0_22*dmats1[22][44] + coeff0_23*dmats1[23][44] + coeff0_24*dmats1[24][44] + coeff0_25*dmats1[25][44] + coeff0_26*dmats1[26][44] + coeff0_27*dmats1[27][44] + coeff0_28*dmats1[28][44] + coeff0_29*dmats1[29][44] + coeff0_30*dmats1[30][44] + coeff0_31*dmats1[31][44] + coeff0_32*dmats1[32][44] + coeff0_33*dmats1[33][44] + coeff0_34*dmats1[34][44] + coeff0_35*dmats1[35][44] + coeff0_36*dmats1[36][44] + coeff0_37*dmats1[37][44] + coeff0_38*dmats1[38][44] + coeff0_39*dmats1[39][44] + coeff0_40*dmats1[40][44] + coeff0_41*dmats1[41][44] + coeff0_42*dmats1[42][44] + coeff0_43*dmats1[43][44] + coeff0_44*dmats1[44][44] + coeff0_45*dmats1[45][44] + coeff0_46*dmats1[46][44] + coeff0_47*dmats1[47][44] + coeff0_48*dmats1[48][44] + coeff0_49*dmats1[49][44] + coeff0_50*dmats1[50][44] + coeff0_51*dmats1[51][44] + coeff0_52*dmats1[52][44] + coeff0_53*dmats1[53][44] + coeff0_54*dmats1[54][44] + coeff0_55*dmats1[55][44];
          new_coeff0_45 = coeff0_0*dmats1[0][45] + coeff0_1*dmats1[1][45] + coeff0_2*dmats1[2][45] + coeff0_3*dmats1[3][45] + coeff0_4*dmats1[4][45] + coeff0_5*dmats1[5][45] + coeff0_6*dmats1[6][45] + coeff0_7*dmats1[7][45] + coeff0_8*dmats1[8][45] + coeff0_9*dmats1[9][45] + coeff0_10*dmats1[10][45] + coeff0_11*dmats1[11][45] + coeff0_12*dmats1[12][45] + coeff0_13*dmats1[13][45] + coeff0_14*dmats1[14][45] + coeff0_15*dmats1[15][45] + coeff0_16*dmats1[16][45] + coeff0_17*dmats1[17][45] + coeff0_18*dmats1[18][45] + coeff0_19*dmats1[19][45] + coeff0_20*dmats1[20][45] + coeff0_21*dmats1[21][45] + coeff0_22*dmats1[22][45] + coeff0_23*dmats1[23][45] + coeff0_24*dmats1[24][45] + coeff0_25*dmats1[25][45] + coeff0_26*dmats1[26][45] + coeff0_27*dmats1[27][45] + coeff0_28*dmats1[28][45] + coeff0_29*dmats1[29][45] + coeff0_30*dmats1[30][45] + coeff0_31*dmats1[31][45] + coeff0_32*dmats1[32][45] + coeff0_33*dmats1[33][45] + coeff0_34*dmats1[34][45] + coeff0_35*dmats1[35][45] + coeff0_36*dmats1[36][45] + coeff0_37*dmats1[37][45] + coeff0_38*dmats1[38][45] + coeff0_39*dmats1[39][45] + coeff0_40*dmats1[40][45] + coeff0_41*dmats1[41][45] + coeff0_42*dmats1[42][45] + coeff0_43*dmats1[43][45] + coeff0_44*dmats1[44][45] + coeff0_45*dmats1[45][45] + coeff0_46*dmats1[46][45] + coeff0_47*dmats1[47][45] + coeff0_48*dmats1[48][45] + coeff0_49*dmats1[49][45] + coeff0_50*dmats1[50][45] + coeff0_51*dmats1[51][45] + coeff0_52*dmats1[52][45] + coeff0_53*dmats1[53][45] + coeff0_54*dmats1[54][45] + coeff0_55*dmats1[55][45];
          new_coeff0_46 = coeff0_0*dmats1[0][46] + coeff0_1*dmats1[1][46] + coeff0_2*dmats1[2][46] + coeff0_3*dmats1[3][46] + coeff0_4*dmats1[4][46] + coeff0_5*dmats1[5][46] + coeff0_6*dmats1[6][46] + coeff0_7*dmats1[7][46] + coeff0_8*dmats1[8][46] + coeff0_9*dmats1[9][46] + coeff0_10*dmats1[10][46] + coeff0_11*dmats1[11][46] + coeff0_12*dmats1[12][46] + coeff0_13*dmats1[13][46] + coeff0_14*dmats1[14][46] + coeff0_15*dmats1[15][46] + coeff0_16*dmats1[16][46] + coeff0_17*dmats1[17][46] + coeff0_18*dmats1[18][46] + coeff0_19*dmats1[19][46] + coeff0_20*dmats1[20][46] + coeff0_21*dmats1[21][46] + coeff0_22*dmats1[22][46] + coeff0_23*dmats1[23][46] + coeff0_24*dmats1[24][46] + coeff0_25*dmats1[25][46] + coeff0_26*dmats1[26][46] + coeff0_27*dmats1[27][46] + coeff0_28*dmats1[28][46] + coeff0_29*dmats1[29][46] + coeff0_30*dmats1[30][46] + coeff0_31*dmats1[31][46] + coeff0_32*dmats1[32][46] + coeff0_33*dmats1[33][46] + coeff0_34*dmats1[34][46] + coeff0_35*dmats1[35][46] + coeff0_36*dmats1[36][46] + coeff0_37*dmats1[37][46] + coeff0_38*dmats1[38][46] + coeff0_39*dmats1[39][46] + coeff0_40*dmats1[40][46] + coeff0_41*dmats1[41][46] + coeff0_42*dmats1[42][46] + coeff0_43*dmats1[43][46] + coeff0_44*dmats1[44][46] + coeff0_45*dmats1[45][46] + coeff0_46*dmats1[46][46] + coeff0_47*dmats1[47][46] + coeff0_48*dmats1[48][46] + coeff0_49*dmats1[49][46] + coeff0_50*dmats1[50][46] + coeff0_51*dmats1[51][46] + coeff0_52*dmats1[52][46] + coeff0_53*dmats1[53][46] + coeff0_54*dmats1[54][46] + coeff0_55*dmats1[55][46];
          new_coeff0_47 = coeff0_0*dmats1[0][47] + coeff0_1*dmats1[1][47] + coeff0_2*dmats1[2][47] + coeff0_3*dmats1[3][47] + coeff0_4*dmats1[4][47] + coeff0_5*dmats1[5][47] + coeff0_6*dmats1[6][47] + coeff0_7*dmats1[7][47] + coeff0_8*dmats1[8][47] + coeff0_9*dmats1[9][47] + coeff0_10*dmats1[10][47] + coeff0_11*dmats1[11][47] + coeff0_12*dmats1[12][47] + coeff0_13*dmats1[13][47] + coeff0_14*dmats1[14][47] + coeff0_15*dmats1[15][47] + coeff0_16*dmats1[16][47] + coeff0_17*dmats1[17][47] + coeff0_18*dmats1[18][47] + coeff0_19*dmats1[19][47] + coeff0_20*dmats1[20][47] + coeff0_21*dmats1[21][47] + coeff0_22*dmats1[22][47] + coeff0_23*dmats1[23][47] + coeff0_24*dmats1[24][47] + coeff0_25*dmats1[25][47] + coeff0_26*dmats1[26][47] + coeff0_27*dmats1[27][47] + coeff0_28*dmats1[28][47] + coeff0_29*dmats1[29][47] + coeff0_30*dmats1[30][47] + coeff0_31*dmats1[31][47] + coeff0_32*dmats1[32][47] + coeff0_33*dmats1[33][47] + coeff0_34*dmats1[34][47] + coeff0_35*dmats1[35][47] + coeff0_36*dmats1[36][47] + coeff0_37*dmats1[37][47] + coeff0_38*dmats1[38][47] + coeff0_39*dmats1[39][47] + coeff0_40*dmats1[40][47] + coeff0_41*dmats1[41][47] + coeff0_42*dmats1[42][47] + coeff0_43*dmats1[43][47] + coeff0_44*dmats1[44][47] + coeff0_45*dmats1[45][47] + coeff0_46*dmats1[46][47] + coeff0_47*dmats1[47][47] + coeff0_48*dmats1[48][47] + coeff0_49*dmats1[49][47] + coeff0_50*dmats1[50][47] + coeff0_51*dmats1[51][47] + coeff0_52*dmats1[52][47] + coeff0_53*dmats1[53][47] + coeff0_54*dmats1[54][47] + coeff0_55*dmats1[55][47];
          new_coeff0_48 = coeff0_0*dmats1[0][48] + coeff0_1*dmats1[1][48] + coeff0_2*dmats1[2][48] + coeff0_3*dmats1[3][48] + coeff0_4*dmats1[4][48] + coeff0_5*dmats1[5][48] + coeff0_6*dmats1[6][48] + coeff0_7*dmats1[7][48] + coeff0_8*dmats1[8][48] + coeff0_9*dmats1[9][48] + coeff0_10*dmats1[10][48] + coeff0_11*dmats1[11][48] + coeff0_12*dmats1[12][48] + coeff0_13*dmats1[13][48] + coeff0_14*dmats1[14][48] + coeff0_15*dmats1[15][48] + coeff0_16*dmats1[16][48] + coeff0_17*dmats1[17][48] + coeff0_18*dmats1[18][48] + coeff0_19*dmats1[19][48] + coeff0_20*dmats1[20][48] + coeff0_21*dmats1[21][48] + coeff0_22*dmats1[22][48] + coeff0_23*dmats1[23][48] + coeff0_24*dmats1[24][48] + coeff0_25*dmats1[25][48] + coeff0_26*dmats1[26][48] + coeff0_27*dmats1[27][48] + coeff0_28*dmats1[28][48] + coeff0_29*dmats1[29][48] + coeff0_30*dmats1[30][48] + coeff0_31*dmats1[31][48] + coeff0_32*dmats1[32][48] + coeff0_33*dmats1[33][48] + coeff0_34*dmats1[34][48] + coeff0_35*dmats1[35][48] + coeff0_36*dmats1[36][48] + coeff0_37*dmats1[37][48] + coeff0_38*dmats1[38][48] + coeff0_39*dmats1[39][48] + coeff0_40*dmats1[40][48] + coeff0_41*dmats1[41][48] + coeff0_42*dmats1[42][48] + coeff0_43*dmats1[43][48] + coeff0_44*dmats1[44][48] + coeff0_45*dmats1[45][48] + coeff0_46*dmats1[46][48] + coeff0_47*dmats1[47][48] + coeff0_48*dmats1[48][48] + coeff0_49*dmats1[49][48] + coeff0_50*dmats1[50][48] + coeff0_51*dmats1[51][48] + coeff0_52*dmats1[52][48] + coeff0_53*dmats1[53][48] + coeff0_54*dmats1[54][48] + coeff0_55*dmats1[55][48];
          new_coeff0_49 = coeff0_0*dmats1[0][49] + coeff0_1*dmats1[1][49] + coeff0_2*dmats1[2][49] + coeff0_3*dmats1[3][49] + coeff0_4*dmats1[4][49] + coeff0_5*dmats1[5][49] + coeff0_6*dmats1[6][49] + coeff0_7*dmats1[7][49] + coeff0_8*dmats1[8][49] + coeff0_9*dmats1[9][49] + coeff0_10*dmats1[10][49] + coeff0_11*dmats1[11][49] + coeff0_12*dmats1[12][49] + coeff0_13*dmats1[13][49] + coeff0_14*dmats1[14][49] + coeff0_15*dmats1[15][49] + coeff0_16*dmats1[16][49] + coeff0_17*dmats1[17][49] + coeff0_18*dmats1[18][49] + coeff0_19*dmats1[19][49] + coeff0_20*dmats1[20][49] + coeff0_21*dmats1[21][49] + coeff0_22*dmats1[22][49] + coeff0_23*dmats1[23][49] + coeff0_24*dmats1[24][49] + coeff0_25*dmats1[25][49] + coeff0_26*dmats1[26][49] + coeff0_27*dmats1[27][49] + coeff0_28*dmats1[28][49] + coeff0_29*dmats1[29][49] + coeff0_30*dmats1[30][49] + coeff0_31*dmats1[31][49] + coeff0_32*dmats1[32][49] + coeff0_33*dmats1[33][49] + coeff0_34*dmats1[34][49] + coeff0_35*dmats1[35][49] + coeff0_36*dmats1[36][49] + coeff0_37*dmats1[37][49] + coeff0_38*dmats1[38][49] + coeff0_39*dmats1[39][49] + coeff0_40*dmats1[40][49] + coeff0_41*dmats1[41][49] + coeff0_42*dmats1[42][49] + coeff0_43*dmats1[43][49] + coeff0_44*dmats1[44][49] + coeff0_45*dmats1[45][49] + coeff0_46*dmats1[46][49] + coeff0_47*dmats1[47][49] + coeff0_48*dmats1[48][49] + coeff0_49*dmats1[49][49] + coeff0_50*dmats1[50][49] + coeff0_51*dmats1[51][49] + coeff0_52*dmats1[52][49] + coeff0_53*dmats1[53][49] + coeff0_54*dmats1[54][49] + coeff0_55*dmats1[55][49];
          new_coeff0_50 = coeff0_0*dmats1[0][50] + coeff0_1*dmats1[1][50] + coeff0_2*dmats1[2][50] + coeff0_3*dmats1[3][50] + coeff0_4*dmats1[4][50] + coeff0_5*dmats1[5][50] + coeff0_6*dmats1[6][50] + coeff0_7*dmats1[7][50] + coeff0_8*dmats1[8][50] + coeff0_9*dmats1[9][50] + coeff0_10*dmats1[10][50] + coeff0_11*dmats1[11][50] + coeff0_12*dmats1[12][50] + coeff0_13*dmats1[13][50] + coeff0_14*dmats1[14][50] + coeff0_15*dmats1[15][50] + coeff0_16*dmats1[16][50] + coeff0_17*dmats1[17][50] + coeff0_18*dmats1[18][50] + coeff0_19*dmats1[19][50] + coeff0_20*dmats1[20][50] + coeff0_21*dmats1[21][50] + coeff0_22*dmats1[22][50] + coeff0_23*dmats1[23][50] + coeff0_24*dmats1[24][50] + coeff0_25*dmats1[25][50] + coeff0_26*dmats1[26][50] + coeff0_27*dmats1[27][50] + coeff0_28*dmats1[28][50] + coeff0_29*dmats1[29][50] + coeff0_30*dmats1[30][50] + coeff0_31*dmats1[31][50] + coeff0_32*dmats1[32][50] + coeff0_33*dmats1[33][50] + coeff0_34*dmats1[34][50] + coeff0_35*dmats1[35][50] + coeff0_36*dmats1[36][50] + coeff0_37*dmats1[37][50] + coeff0_38*dmats1[38][50] + coeff0_39*dmats1[39][50] + coeff0_40*dmats1[40][50] + coeff0_41*dmats1[41][50] + coeff0_42*dmats1[42][50] + coeff0_43*dmats1[43][50] + coeff0_44*dmats1[44][50] + coeff0_45*dmats1[45][50] + coeff0_46*dmats1[46][50] + coeff0_47*dmats1[47][50] + coeff0_48*dmats1[48][50] + coeff0_49*dmats1[49][50] + coeff0_50*dmats1[50][50] + coeff0_51*dmats1[51][50] + coeff0_52*dmats1[52][50] + coeff0_53*dmats1[53][50] + coeff0_54*dmats1[54][50] + coeff0_55*dmats1[55][50];
          new_coeff0_51 = coeff0_0*dmats1[0][51] + coeff0_1*dmats1[1][51] + coeff0_2*dmats1[2][51] + coeff0_3*dmats1[3][51] + coeff0_4*dmats1[4][51] + coeff0_5*dmats1[5][51] + coeff0_6*dmats1[6][51] + coeff0_7*dmats1[7][51] + coeff0_8*dmats1[8][51] + coeff0_9*dmats1[9][51] + coeff0_10*dmats1[10][51] + coeff0_11*dmats1[11][51] + coeff0_12*dmats1[12][51] + coeff0_13*dmats1[13][51] + coeff0_14*dmats1[14][51] + coeff0_15*dmats1[15][51] + coeff0_16*dmats1[16][51] + coeff0_17*dmats1[17][51] + coeff0_18*dmats1[18][51] + coeff0_19*dmats1[19][51] + coeff0_20*dmats1[20][51] + coeff0_21*dmats1[21][51] + coeff0_22*dmats1[22][51] + coeff0_23*dmats1[23][51] + coeff0_24*dmats1[24][51] + coeff0_25*dmats1[25][51] + coeff0_26*dmats1[26][51] + coeff0_27*dmats1[27][51] + coeff0_28*dmats1[28][51] + coeff0_29*dmats1[29][51] + coeff0_30*dmats1[30][51] + coeff0_31*dmats1[31][51] + coeff0_32*dmats1[32][51] + coeff0_33*dmats1[33][51] + coeff0_34*dmats1[34][51] + coeff0_35*dmats1[35][51] + coeff0_36*dmats1[36][51] + coeff0_37*dmats1[37][51] + coeff0_38*dmats1[38][51] + coeff0_39*dmats1[39][51] + coeff0_40*dmats1[40][51] + coeff0_41*dmats1[41][51] + coeff0_42*dmats1[42][51] + coeff0_43*dmats1[43][51] + coeff0_44*dmats1[44][51] + coeff0_45*dmats1[45][51] + coeff0_46*dmats1[46][51] + coeff0_47*dmats1[47][51] + coeff0_48*dmats1[48][51] + coeff0_49*dmats1[49][51] + coeff0_50*dmats1[50][51] + coeff0_51*dmats1[51][51] + coeff0_52*dmats1[52][51] + coeff0_53*dmats1[53][51] + coeff0_54*dmats1[54][51] + coeff0_55*dmats1[55][51];
          new_coeff0_52 = coeff0_0*dmats1[0][52] + coeff0_1*dmats1[1][52] + coeff0_2*dmats1[2][52] + coeff0_3*dmats1[3][52] + coeff0_4*dmats1[4][52] + coeff0_5*dmats1[5][52] + coeff0_6*dmats1[6][52] + coeff0_7*dmats1[7][52] + coeff0_8*dmats1[8][52] + coeff0_9*dmats1[9][52] + coeff0_10*dmats1[10][52] + coeff0_11*dmats1[11][52] + coeff0_12*dmats1[12][52] + coeff0_13*dmats1[13][52] + coeff0_14*dmats1[14][52] + coeff0_15*dmats1[15][52] + coeff0_16*dmats1[16][52] + coeff0_17*dmats1[17][52] + coeff0_18*dmats1[18][52] + coeff0_19*dmats1[19][52] + coeff0_20*dmats1[20][52] + coeff0_21*dmats1[21][52] + coeff0_22*dmats1[22][52] + coeff0_23*dmats1[23][52] + coeff0_24*dmats1[24][52] + coeff0_25*dmats1[25][52] + coeff0_26*dmats1[26][52] + coeff0_27*dmats1[27][52] + coeff0_28*dmats1[28][52] + coeff0_29*dmats1[29][52] + coeff0_30*dmats1[30][52] + coeff0_31*dmats1[31][52] + coeff0_32*dmats1[32][52] + coeff0_33*dmats1[33][52] + coeff0_34*dmats1[34][52] + coeff0_35*dmats1[35][52] + coeff0_36*dmats1[36][52] + coeff0_37*dmats1[37][52] + coeff0_38*dmats1[38][52] + coeff0_39*dmats1[39][52] + coeff0_40*dmats1[40][52] + coeff0_41*dmats1[41][52] + coeff0_42*dmats1[42][52] + coeff0_43*dmats1[43][52] + coeff0_44*dmats1[44][52] + coeff0_45*dmats1[45][52] + coeff0_46*dmats1[46][52] + coeff0_47*dmats1[47][52] + coeff0_48*dmats1[48][52] + coeff0_49*dmats1[49][52] + coeff0_50*dmats1[50][52] + coeff0_51*dmats1[51][52] + coeff0_52*dmats1[52][52] + coeff0_53*dmats1[53][52] + coeff0_54*dmats1[54][52] + coeff0_55*dmats1[55][52];
          new_coeff0_53 = coeff0_0*dmats1[0][53] + coeff0_1*dmats1[1][53] + coeff0_2*dmats1[2][53] + coeff0_3*dmats1[3][53] + coeff0_4*dmats1[4][53] + coeff0_5*dmats1[5][53] + coeff0_6*dmats1[6][53] + coeff0_7*dmats1[7][53] + coeff0_8*dmats1[8][53] + coeff0_9*dmats1[9][53] + coeff0_10*dmats1[10][53] + coeff0_11*dmats1[11][53] + coeff0_12*dmats1[12][53] + coeff0_13*dmats1[13][53] + coeff0_14*dmats1[14][53] + coeff0_15*dmats1[15][53] + coeff0_16*dmats1[16][53] + coeff0_17*dmats1[17][53] + coeff0_18*dmats1[18][53] + coeff0_19*dmats1[19][53] + coeff0_20*dmats1[20][53] + coeff0_21*dmats1[21][53] + coeff0_22*dmats1[22][53] + coeff0_23*dmats1[23][53] + coeff0_24*dmats1[24][53] + coeff0_25*dmats1[25][53] + coeff0_26*dmats1[26][53] + coeff0_27*dmats1[27][53] + coeff0_28*dmats1[28][53] + coeff0_29*dmats1[29][53] + coeff0_30*dmats1[30][53] + coeff0_31*dmats1[31][53] + coeff0_32*dmats1[32][53] + coeff0_33*dmats1[33][53] + coeff0_34*dmats1[34][53] + coeff0_35*dmats1[35][53] + coeff0_36*dmats1[36][53] + coeff0_37*dmats1[37][53] + coeff0_38*dmats1[38][53] + coeff0_39*dmats1[39][53] + coeff0_40*dmats1[40][53] + coeff0_41*dmats1[41][53] + coeff0_42*dmats1[42][53] + coeff0_43*dmats1[43][53] + coeff0_44*dmats1[44][53] + coeff0_45*dmats1[45][53] + coeff0_46*dmats1[46][53] + coeff0_47*dmats1[47][53] + coeff0_48*dmats1[48][53] + coeff0_49*dmats1[49][53] + coeff0_50*dmats1[50][53] + coeff0_51*dmats1[51][53] + coeff0_52*dmats1[52][53] + coeff0_53*dmats1[53][53] + coeff0_54*dmats1[54][53] + coeff0_55*dmats1[55][53];
          new_coeff0_54 = coeff0_0*dmats1[0][54] + coeff0_1*dmats1[1][54] + coeff0_2*dmats1[2][54] + coeff0_3*dmats1[3][54] + coeff0_4*dmats1[4][54] + coeff0_5*dmats1[5][54] + coeff0_6*dmats1[6][54] + coeff0_7*dmats1[7][54] + coeff0_8*dmats1[8][54] + coeff0_9*dmats1[9][54] + coeff0_10*dmats1[10][54] + coeff0_11*dmats1[11][54] + coeff0_12*dmats1[12][54] + coeff0_13*dmats1[13][54] + coeff0_14*dmats1[14][54] + coeff0_15*dmats1[15][54] + coeff0_16*dmats1[16][54] + coeff0_17*dmats1[17][54] + coeff0_18*dmats1[18][54] + coeff0_19*dmats1[19][54] + coeff0_20*dmats1[20][54] + coeff0_21*dmats1[21][54] + coeff0_22*dmats1[22][54] + coeff0_23*dmats1[23][54] + coeff0_24*dmats1[24][54] + coeff0_25*dmats1[25][54] + coeff0_26*dmats1[26][54] + coeff0_27*dmats1[27][54] + coeff0_28*dmats1[28][54] + coeff0_29*dmats1[29][54] + coeff0_30*dmats1[30][54] + coeff0_31*dmats1[31][54] + coeff0_32*dmats1[32][54] + coeff0_33*dmats1[33][54] + coeff0_34*dmats1[34][54] + coeff0_35*dmats1[35][54] + coeff0_36*dmats1[36][54] + coeff0_37*dmats1[37][54] + coeff0_38*dmats1[38][54] + coeff0_39*dmats1[39][54] + coeff0_40*dmats1[40][54] + coeff0_41*dmats1[41][54] + coeff0_42*dmats1[42][54] + coeff0_43*dmats1[43][54] + coeff0_44*dmats1[44][54] + coeff0_45*dmats1[45][54] + coeff0_46*dmats1[46][54] + coeff0_47*dmats1[47][54] + coeff0_48*dmats1[48][54] + coeff0_49*dmats1[49][54] + coeff0_50*dmats1[50][54] + coeff0_51*dmats1[51][54] + coeff0_52*dmats1[52][54] + coeff0_53*dmats1[53][54] + coeff0_54*dmats1[54][54] + coeff0_55*dmats1[55][54];
          new_coeff0_55 = coeff0_0*dmats1[0][55] + coeff0_1*dmats1[1][55] + coeff0_2*dmats1[2][55] + coeff0_3*dmats1[3][55] + coeff0_4*dmats1[4][55] + coeff0_5*dmats1[5][55] + coeff0_6*dmats1[6][55] + coeff0_7*dmats1[7][55] + coeff0_8*dmats1[8][55] + coeff0_9*dmats1[9][55] + coeff0_10*dmats1[10][55] + coeff0_11*dmats1[11][55] + coeff0_12*dmats1[12][55] + coeff0_13*dmats1[13][55] + coeff0_14*dmats1[14][55] + coeff0_15*dmats1[15][55] + coeff0_16*dmats1[16][55] + coeff0_17*dmats1[17][55] + coeff0_18*dmats1[18][55] + coeff0_19*dmats1[19][55] + coeff0_20*dmats1[20][55] + coeff0_21*dmats1[21][55] + coeff0_22*dmats1[22][55] + coeff0_23*dmats1[23][55] + coeff0_24*dmats1[24][55] + coeff0_25*dmats1[25][55] + coeff0_26*dmats1[26][55] + coeff0_27*dmats1[27][55] + coeff0_28*dmats1[28][55] + coeff0_29*dmats1[29][55] + coeff0_30*dmats1[30][55] + coeff0_31*dmats1[31][55] + coeff0_32*dmats1[32][55] + coeff0_33*dmats1[33][55] + coeff0_34*dmats1[34][55] + coeff0_35*dmats1[35][55] + coeff0_36*dmats1[36][55] + coeff0_37*dmats1[37][55] + coeff0_38*dmats1[38][55] + coeff0_39*dmats1[39][55] + coeff0_40*dmats1[40][55] + coeff0_41*dmats1[41][55] + coeff0_42*dmats1[42][55] + coeff0_43*dmats1[43][55] + coeff0_44*dmats1[44][55] + coeff0_45*dmats1[45][55] + coeff0_46*dmats1[46][55] + coeff0_47*dmats1[47][55] + coeff0_48*dmats1[48][55] + coeff0_49*dmats1[49][55] + coeff0_50*dmats1[50][55] + coeff0_51*dmats1[51][55] + coeff0_52*dmats1[52][55] + coeff0_53*dmats1[53][55] + coeff0_54*dmats1[54][55] + coeff0_55*dmats1[55][55];
        }
        if(combinations[deriv_num][j] == 2)
        {
          new_coeff0_0 = coeff0_0*dmats2[0][0] + coeff0_1*dmats2[1][0] + coeff0_2*dmats2[2][0] + coeff0_3*dmats2[3][0] + coeff0_4*dmats2[4][0] + coeff0_5*dmats2[5][0] + coeff0_6*dmats2[6][0] + coeff0_7*dmats2[7][0] + coeff0_8*dmats2[8][0] + coeff0_9*dmats2[9][0] + coeff0_10*dmats2[10][0] + coeff0_11*dmats2[11][0] + coeff0_12*dmats2[12][0] + coeff0_13*dmats2[13][0] + coeff0_14*dmats2[14][0] + coeff0_15*dmats2[15][0] + coeff0_16*dmats2[16][0] + coeff0_17*dmats2[17][0] + coeff0_18*dmats2[18][0] + coeff0_19*dmats2[19][0] + coeff0_20*dmats2[20][0] + coeff0_21*dmats2[21][0] + coeff0_22*dmats2[22][0] + coeff0_23*dmats2[23][0] + coeff0_24*dmats2[24][0] + coeff0_25*dmats2[25][0] + coeff0_26*dmats2[26][0] + coeff0_27*dmats2[27][0] + coeff0_28*dmats2[28][0] + coeff0_29*dmats2[29][0] + coeff0_30*dmats2[30][0] + coeff0_31*dmats2[31][0] + coeff0_32*dmats2[32][0] + coeff0_33*dmats2[33][0] + coeff0_34*dmats2[34][0] + coeff0_35*dmats2[35][0] + coeff0_36*dmats2[36][0] + coeff0_37*dmats2[37][0] + coeff0_38*dmats2[38][0] + coeff0_39*dmats2[39][0] + coeff0_40*dmats2[40][0] + coeff0_41*dmats2[41][0] + coeff0_42*dmats2[42][0] + coeff0_43*dmats2[43][0] + coeff0_44*dmats2[44][0] + coeff0_45*dmats2[45][0] + coeff0_46*dmats2[46][0] + coeff0_47*dmats2[47][0] + coeff0_48*dmats2[48][0] + coeff0_49*dmats2[49][0] + coeff0_50*dmats2[50][0] + coeff0_51*dmats2[51][0] + coeff0_52*dmats2[52][0] + coeff0_53*dmats2[53][0] + coeff0_54*dmats2[54][0] + coeff0_55*dmats2[55][0];
          new_coeff0_1 = coeff0_0*dmats2[0][1] + coeff0_1*dmats2[1][1] + coeff0_2*dmats2[2][1] + coeff0_3*dmats2[3][1] + coeff0_4*dmats2[4][1] + coeff0_5*dmats2[5][1] + coeff0_6*dmats2[6][1] + coeff0_7*dmats2[7][1] + coeff0_8*dmats2[8][1] + coeff0_9*dmats2[9][1] + coeff0_10*dmats2[10][1] + coeff0_11*dmats2[11][1] + coeff0_12*dmats2[12][1] + coeff0_13*dmats2[13][1] + coeff0_14*dmats2[14][1] + coeff0_15*dmats2[15][1] + coeff0_16*dmats2[16][1] + coeff0_17*dmats2[17][1] + coeff0_18*dmats2[18][1] + coeff0_19*dmats2[19][1] + coeff0_20*dmats2[20][1] + coeff0_21*dmats2[21][1] + coeff0_22*dmats2[22][1] + coeff0_23*dmats2[23][1] + coeff0_24*dmats2[24][1] + coeff0_25*dmats2[25][1] + coeff0_26*dmats2[26][1] + coeff0_27*dmats2[27][1] + coeff0_28*dmats2[28][1] + coeff0_29*dmats2[29][1] + coeff0_30*dmats2[30][1] + coeff0_31*dmats2[31][1] + coeff0_32*dmats2[32][1] + coeff0_33*dmats2[33][1] + coeff0_34*dmats2[34][1] + coeff0_35*dmats2[35][1] + coeff0_36*dmats2[36][1] + coeff0_37*dmats2[37][1] + coeff0_38*dmats2[38][1] + coeff0_39*dmats2[39][1] + coeff0_40*dmats2[40][1] + coeff0_41*dmats2[41][1] + coeff0_42*dmats2[42][1] + coeff0_43*dmats2[43][1] + coeff0_44*dmats2[44][1] + coeff0_45*dmats2[45][1] + coeff0_46*dmats2[46][1] + coeff0_47*dmats2[47][1] + coeff0_48*dmats2[48][1] + coeff0_49*dmats2[49][1] + coeff0_50*dmats2[50][1] + coeff0_51*dmats2[51][1] + coeff0_52*dmats2[52][1] + coeff0_53*dmats2[53][1] + coeff0_54*dmats2[54][1] + coeff0_55*dmats2[55][1];
          new_coeff0_2 = coeff0_0*dmats2[0][2] + coeff0_1*dmats2[1][2] + coeff0_2*dmats2[2][2] + coeff0_3*dmats2[3][2] + coeff0_4*dmats2[4][2] + coeff0_5*dmats2[5][2] + coeff0_6*dmats2[6][2] + coeff0_7*dmats2[7][2] + coeff0_8*dmats2[8][2] + coeff0_9*dmats2[9][2] + coeff0_10*dmats2[10][2] + coeff0_11*dmats2[11][2] + coeff0_12*dmats2[12][2] + coeff0_13*dmats2[13][2] + coeff0_14*dmats2[14][2] + coeff0_15*dmats2[15][2] + coeff0_16*dmats2[16][2] + coeff0_17*dmats2[17][2] + coeff0_18*dmats2[18][2] + coeff0_19*dmats2[19][2] + coeff0_20*dmats2[20][2] + coeff0_21*dmats2[21][2] + coeff0_22*dmats2[22][2] + coeff0_23*dmats2[23][2] + coeff0_24*dmats2[24][2] + coeff0_25*dmats2[25][2] + coeff0_26*dmats2[26][2] + coeff0_27*dmats2[27][2] + coeff0_28*dmats2[28][2] + coeff0_29*dmats2[29][2] + coeff0_30*dmats2[30][2] + coeff0_31*dmats2[31][2] + coeff0_32*dmats2[32][2] + coeff0_33*dmats2[33][2] + coeff0_34*dmats2[34][2] + coeff0_35*dmats2[35][2] + coeff0_36*dmats2[36][2] + coeff0_37*dmats2[37][2] + coeff0_38*dmats2[38][2] + coeff0_39*dmats2[39][2] + coeff0_40*dmats2[40][2] + coeff0_41*dmats2[41][2] + coeff0_42*dmats2[42][2] + coeff0_43*dmats2[43][2] + coeff0_44*dmats2[44][2] + coeff0_45*dmats2[45][2] + coeff0_46*dmats2[46][2] + coeff0_47*dmats2[47][2] + coeff0_48*dmats2[48][2] + coeff0_49*dmats2[49][2] + coeff0_50*dmats2[50][2] + coeff0_51*dmats2[51][2] + coeff0_52*dmats2[52][2] + coeff0_53*dmats2[53][2] + coeff0_54*dmats2[54][2] + coeff0_55*dmats2[55][2];
          new_coeff0_3 = coeff0_0*dmats2[0][3] + coeff0_1*dmats2[1][3] + coeff0_2*dmats2[2][3] + coeff0_3*dmats2[3][3] + coeff0_4*dmats2[4][3] + coeff0_5*dmats2[5][3] + coeff0_6*dmats2[6][3] + coeff0_7*dmats2[7][3] + coeff0_8*dmats2[8][3] + coeff0_9*dmats2[9][3] + coeff0_10*dmats2[10][3] + coeff0_11*dmats2[11][3] + coeff0_12*dmats2[12][3] + coeff0_13*dmats2[13][3] + coeff0_14*dmats2[14][3] + coeff0_15*dmats2[15][3] + coeff0_16*dmats2[16][3] + coeff0_17*dmats2[17][3] + coeff0_18*dmats2[18][3] + coeff0_19*dmats2[19][3] + coeff0_20*dmats2[20][3] + coeff0_21*dmats2[21][3] + coeff0_22*dmats2[22][3] + coeff0_23*dmats2[23][3] + coeff0_24*dmats2[24][3] + coeff0_25*dmats2[25][3] + coeff0_26*dmats2[26][3] + coeff0_27*dmats2[27][3] + coeff0_28*dmats2[28][3] + coeff0_29*dmats2[29][3] + coeff0_30*dmats2[30][3] + coeff0_31*dmats2[31][3] + coeff0_32*dmats2[32][3] + coeff0_33*dmats2[33][3] + coeff0_34*dmats2[34][3] + coeff0_35*dmats2[35][3] + coeff0_36*dmats2[36][3] + coeff0_37*dmats2[37][3] + coeff0_38*dmats2[38][3] + coeff0_39*dmats2[39][3] + coeff0_40*dmats2[40][3] + coeff0_41*dmats2[41][3] + coeff0_42*dmats2[42][3] + coeff0_43*dmats2[43][3] + coeff0_44*dmats2[44][3] + coeff0_45*dmats2[45][3] + coeff0_46*dmats2[46][3] + coeff0_47*dmats2[47][3] + coeff0_48*dmats2[48][3] + coeff0_49*dmats2[49][3] + coeff0_50*dmats2[50][3] + coeff0_51*dmats2[51][3] + coeff0_52*dmats2[52][3] + coeff0_53*dmats2[53][3] + coeff0_54*dmats2[54][3] + coeff0_55*dmats2[55][3];
          new_coeff0_4 = coeff0_0*dmats2[0][4] + coeff0_1*dmats2[1][4] + coeff0_2*dmats2[2][4] + coeff0_3*dmats2[3][4] + coeff0_4*dmats2[4][4] + coeff0_5*dmats2[5][4] + coeff0_6*dmats2[6][4] + coeff0_7*dmats2[7][4] + coeff0_8*dmats2[8][4] + coeff0_9*dmats2[9][4] + coeff0_10*dmats2[10][4] + coeff0_11*dmats2[11][4] + coeff0_12*dmats2[12][4] + coeff0_13*dmats2[13][4] + coeff0_14*dmats2[14][4] + coeff0_15*dmats2[15][4] + coeff0_16*dmats2[16][4] + coeff0_17*dmats2[17][4] + coeff0_18*dmats2[18][4] + coeff0_19*dmats2[19][4] + coeff0_20*dmats2[20][4] + coeff0_21*dmats2[21][4] + coeff0_22*dmats2[22][4] + coeff0_23*dmats2[23][4] + coeff0_24*dmats2[24][4] + coeff0_25*dmats2[25][4] + coeff0_26*dmats2[26][4] + coeff0_27*dmats2[27][4] + coeff0_28*dmats2[28][4] + coeff0_29*dmats2[29][4] + coeff0_30*dmats2[30][4] + coeff0_31*dmats2[31][4] + coeff0_32*dmats2[32][4] + coeff0_33*dmats2[33][4] + coeff0_34*dmats2[34][4] + coeff0_35*dmats2[35][4] + coeff0_36*dmats2[36][4] + coeff0_37*dmats2[37][4] + coeff0_38*dmats2[38][4] + coeff0_39*dmats2[39][4] + coeff0_40*dmats2[40][4] + coeff0_41*dmats2[41][4] + coeff0_42*dmats2[42][4] + coeff0_43*dmats2[43][4] + coeff0_44*dmats2[44][4] + coeff0_45*dmats2[45][4] + coeff0_46*dmats2[46][4] + coeff0_47*dmats2[47][4] + coeff0_48*dmats2[48][4] + coeff0_49*dmats2[49][4] + coeff0_50*dmats2[50][4] + coeff0_51*dmats2[51][4] + coeff0_52*dmats2[52][4] + coeff0_53*dmats2[53][4] + coeff0_54*dmats2[54][4] + coeff0_55*dmats2[55][4];
          new_coeff0_5 = coeff0_0*dmats2[0][5] + coeff0_1*dmats2[1][5] + coeff0_2*dmats2[2][5] + coeff0_3*dmats2[3][5] + coeff0_4*dmats2[4][5] + coeff0_5*dmats2[5][5] + coeff0_6*dmats2[6][5] + coeff0_7*dmats2[7][5] + coeff0_8*dmats2[8][5] + coeff0_9*dmats2[9][5] + coeff0_10*dmats2[10][5] + coeff0_11*dmats2[11][5] + coeff0_12*dmats2[12][5] + coeff0_13*dmats2[13][5] + coeff0_14*dmats2[14][5] + coeff0_15*dmats2[15][5] + coeff0_16*dmats2[16][5] + coeff0_17*dmats2[17][5] + coeff0_18*dmats2[18][5] + coeff0_19*dmats2[19][5] + coeff0_20*dmats2[20][5] + coeff0_21*dmats2[21][5] + coeff0_22*dmats2[22][5] + coeff0_23*dmats2[23][5] + coeff0_24*dmats2[24][5] + coeff0_25*dmats2[25][5] + coeff0_26*dmats2[26][5] + coeff0_27*dmats2[27][5] + coeff0_28*dmats2[28][5] + coeff0_29*dmats2[29][5] + coeff0_30*dmats2[30][5] + coeff0_31*dmats2[31][5] + coeff0_32*dmats2[32][5] + coeff0_33*dmats2[33][5] + coeff0_34*dmats2[34][5] + coeff0_35*dmats2[35][5] + coeff0_36*dmats2[36][5] + coeff0_37*dmats2[37][5] + coeff0_38*dmats2[38][5] + coeff0_39*dmats2[39][5] + coeff0_40*dmats2[40][5] + coeff0_41*dmats2[41][5] + coeff0_42*dmats2[42][5] + coeff0_43*dmats2[43][5] + coeff0_44*dmats2[44][5] + coeff0_45*dmats2[45][5] + coeff0_46*dmats2[46][5] + coeff0_47*dmats2[47][5] + coeff0_48*dmats2[48][5] + coeff0_49*dmats2[49][5] + coeff0_50*dmats2[50][5] + coeff0_51*dmats2[51][5] + coeff0_52*dmats2[52][5] + coeff0_53*dmats2[53][5] + coeff0_54*dmats2[54][5] + coeff0_55*dmats2[55][5];
          new_coeff0_6 = coeff0_0*dmats2[0][6] + coeff0_1*dmats2[1][6] + coeff0_2*dmats2[2][6] + coeff0_3*dmats2[3][6] + coeff0_4*dmats2[4][6] + coeff0_5*dmats2[5][6] + coeff0_6*dmats2[6][6] + coeff0_7*dmats2[7][6] + coeff0_8*dmats2[8][6] + coeff0_9*dmats2[9][6] + coeff0_10*dmats2[10][6] + coeff0_11*dmats2[11][6] + coeff0_12*dmats2[12][6] + coeff0_13*dmats2[13][6] + coeff0_14*dmats2[14][6] + coeff0_15*dmats2[15][6] + coeff0_16*dmats2[16][6] + coeff0_17*dmats2[17][6] + coeff0_18*dmats2[18][6] + coeff0_19*dmats2[19][6] + coeff0_20*dmats2[20][6] + coeff0_21*dmats2[21][6] + coeff0_22*dmats2[22][6] + coeff0_23*dmats2[23][6] + coeff0_24*dmats2[24][6] + coeff0_25*dmats2[25][6] + coeff0_26*dmats2[26][6] + coeff0_27*dmats2[27][6] + coeff0_28*dmats2[28][6] + coeff0_29*dmats2[29][6] + coeff0_30*dmats2[30][6] + coeff0_31*dmats2[31][6] + coeff0_32*dmats2[32][6] + coeff0_33*dmats2[33][6] + coeff0_34*dmats2[34][6] + coeff0_35*dmats2[35][6] + coeff0_36*dmats2[36][6] + coeff0_37*dmats2[37][6] + coeff0_38*dmats2[38][6] + coeff0_39*dmats2[39][6] + coeff0_40*dmats2[40][6] + coeff0_41*dmats2[41][6] + coeff0_42*dmats2[42][6] + coeff0_43*dmats2[43][6] + coeff0_44*dmats2[44][6] + coeff0_45*dmats2[45][6] + coeff0_46*dmats2[46][6] + coeff0_47*dmats2[47][6] + coeff0_48*dmats2[48][6] + coeff0_49*dmats2[49][6] + coeff0_50*dmats2[50][6] + coeff0_51*dmats2[51][6] + coeff0_52*dmats2[52][6] + coeff0_53*dmats2[53][6] + coeff0_54*dmats2[54][6] + coeff0_55*dmats2[55][6];
          new_coeff0_7 = coeff0_0*dmats2[0][7] + coeff0_1*dmats2[1][7] + coeff0_2*dmats2[2][7] + coeff0_3*dmats2[3][7] + coeff0_4*dmats2[4][7] + coeff0_5*dmats2[5][7] + coeff0_6*dmats2[6][7] + coeff0_7*dmats2[7][7] + coeff0_8*dmats2[8][7] + coeff0_9*dmats2[9][7] + coeff0_10*dmats2[10][7] + coeff0_11*dmats2[11][7] + coeff0_12*dmats2[12][7] + coeff0_13*dmats2[13][7] + coeff0_14*dmats2[14][7] + coeff0_15*dmats2[15][7] + coeff0_16*dmats2[16][7] + coeff0_17*dmats2[17][7] + coeff0_18*dmats2[18][7] + coeff0_19*dmats2[19][7] + coeff0_20*dmats2[20][7] + coeff0_21*dmats2[21][7] + coeff0_22*dmats2[22][7] + coeff0_23*dmats2[23][7] + coeff0_24*dmats2[24][7] + coeff0_25*dmats2[25][7] + coeff0_26*dmats2[26][7] + coeff0_27*dmats2[27][7] + coeff0_28*dmats2[28][7] + coeff0_29*dmats2[29][7] + coeff0_30*dmats2[30][7] + coeff0_31*dmats2[31][7] + coeff0_32*dmats2[32][7] + coeff0_33*dmats2[33][7] + coeff0_34*dmats2[34][7] + coeff0_35*dmats2[35][7] + coeff0_36*dmats2[36][7] + coeff0_37*dmats2[37][7] + coeff0_38*dmats2[38][7] + coeff0_39*dmats2[39][7] + coeff0_40*dmats2[40][7] + coeff0_41*dmats2[41][7] + coeff0_42*dmats2[42][7] + coeff0_43*dmats2[43][7] + coeff0_44*dmats2[44][7] + coeff0_45*dmats2[45][7] + coeff0_46*dmats2[46][7] + coeff0_47*dmats2[47][7] + coeff0_48*dmats2[48][7] + coeff0_49*dmats2[49][7] + coeff0_50*dmats2[50][7] + coeff0_51*dmats2[51][7] + coeff0_52*dmats2[52][7] + coeff0_53*dmats2[53][7] + coeff0_54*dmats2[54][7] + coeff0_55*dmats2[55][7];
          new_coeff0_8 = coeff0_0*dmats2[0][8] + coeff0_1*dmats2[1][8] + coeff0_2*dmats2[2][8] + coeff0_3*dmats2[3][8] + coeff0_4*dmats2[4][8] + coeff0_5*dmats2[5][8] + coeff0_6*dmats2[6][8] + coeff0_7*dmats2[7][8] + coeff0_8*dmats2[8][8] + coeff0_9*dmats2[9][8] + coeff0_10*dmats2[10][8] + coeff0_11*dmats2[11][8] + coeff0_12*dmats2[12][8] + coeff0_13*dmats2[13][8] + coeff0_14*dmats2[14][8] + coeff0_15*dmats2[15][8] + coeff0_16*dmats2[16][8] + coeff0_17*dmats2[17][8] + coeff0_18*dmats2[18][8] + coeff0_19*dmats2[19][8] + coeff0_20*dmats2[20][8] + coeff0_21*dmats2[21][8] + coeff0_22*dmats2[22][8] + coeff0_23*dmats2[23][8] + coeff0_24*dmats2[24][8] + coeff0_25*dmats2[25][8] + coeff0_26*dmats2[26][8] + coeff0_27*dmats2[27][8] + coeff0_28*dmats2[28][8] + coeff0_29*dmats2[29][8] + coeff0_30*dmats2[30][8] + coeff0_31*dmats2[31][8] + coeff0_32*dmats2[32][8] + coeff0_33*dmats2[33][8] + coeff0_34*dmats2[34][8] + coeff0_35*dmats2[35][8] + coeff0_36*dmats2[36][8] + coeff0_37*dmats2[37][8] + coeff0_38*dmats2[38][8] + coeff0_39*dmats2[39][8] + coeff0_40*dmats2[40][8] + coeff0_41*dmats2[41][8] + coeff0_42*dmats2[42][8] + coeff0_43*dmats2[43][8] + coeff0_44*dmats2[44][8] + coeff0_45*dmats2[45][8] + coeff0_46*dmats2[46][8] + coeff0_47*dmats2[47][8] + coeff0_48*dmats2[48][8] + coeff0_49*dmats2[49][8] + coeff0_50*dmats2[50][8] + coeff0_51*dmats2[51][8] + coeff0_52*dmats2[52][8] + coeff0_53*dmats2[53][8] + coeff0_54*dmats2[54][8] + coeff0_55*dmats2[55][8];
          new_coeff0_9 = coeff0_0*dmats2[0][9] + coeff0_1*dmats2[1][9] + coeff0_2*dmats2[2][9] + coeff0_3*dmats2[3][9] + coeff0_4*dmats2[4][9] + coeff0_5*dmats2[5][9] + coeff0_6*dmats2[6][9] + coeff0_7*dmats2[7][9] + coeff0_8*dmats2[8][9] + coeff0_9*dmats2[9][9] + coeff0_10*dmats2[10][9] + coeff0_11*dmats2[11][9] + coeff0_12*dmats2[12][9] + coeff0_13*dmats2[13][9] + coeff0_14*dmats2[14][9] + coeff0_15*dmats2[15][9] + coeff0_16*dmats2[16][9] + coeff0_17*dmats2[17][9] + coeff0_18*dmats2[18][9] + coeff0_19*dmats2[19][9] + coeff0_20*dmats2[20][9] + coeff0_21*dmats2[21][9] + coeff0_22*dmats2[22][9] + coeff0_23*dmats2[23][9] + coeff0_24*dmats2[24][9] + coeff0_25*dmats2[25][9] + coeff0_26*dmats2[26][9] + coeff0_27*dmats2[27][9] + coeff0_28*dmats2[28][9] + coeff0_29*dmats2[29][9] + coeff0_30*dmats2[30][9] + coeff0_31*dmats2[31][9] + coeff0_32*dmats2[32][9] + coeff0_33*dmats2[33][9] + coeff0_34*dmats2[34][9] + coeff0_35*dmats2[35][9] + coeff0_36*dmats2[36][9] + coeff0_37*dmats2[37][9] + coeff0_38*dmats2[38][9] + coeff0_39*dmats2[39][9] + coeff0_40*dmats2[40][9] + coeff0_41*dmats2[41][9] + coeff0_42*dmats2[42][9] + coeff0_43*dmats2[43][9] + coeff0_44*dmats2[44][9] + coeff0_45*dmats2[45][9] + coeff0_46*dmats2[46][9] + coeff0_47*dmats2[47][9] + coeff0_48*dmats2[48][9] + coeff0_49*dmats2[49][9] + coeff0_50*dmats2[50][9] + coeff0_51*dmats2[51][9] + coeff0_52*dmats2[52][9] + coeff0_53*dmats2[53][9] + coeff0_54*dmats2[54][9] + coeff0_55*dmats2[55][9];
          new_coeff0_10 = coeff0_0*dmats2[0][10] + coeff0_1*dmats2[1][10] + coeff0_2*dmats2[2][10] + coeff0_3*dmats2[3][10] + coeff0_4*dmats2[4][10] + coeff0_5*dmats2[5][10] + coeff0_6*dmats2[6][10] + coeff0_7*dmats2[7][10] + coeff0_8*dmats2[8][10] + coeff0_9*dmats2[9][10] + coeff0_10*dmats2[10][10] + coeff0_11*dmats2[11][10] + coeff0_12*dmats2[12][10] + coeff0_13*dmats2[13][10] + coeff0_14*dmats2[14][10] + coeff0_15*dmats2[15][10] + coeff0_16*dmats2[16][10] + coeff0_17*dmats2[17][10] + coeff0_18*dmats2[18][10] + coeff0_19*dmats2[19][10] + coeff0_20*dmats2[20][10] + coeff0_21*dmats2[21][10] + coeff0_22*dmats2[22][10] + coeff0_23*dmats2[23][10] + coeff0_24*dmats2[24][10] + coeff0_25*dmats2[25][10] + coeff0_26*dmats2[26][10] + coeff0_27*dmats2[27][10] + coeff0_28*dmats2[28][10] + coeff0_29*dmats2[29][10] + coeff0_30*dmats2[30][10] + coeff0_31*dmats2[31][10] + coeff0_32*dmats2[32][10] + coeff0_33*dmats2[33][10] + coeff0_34*dmats2[34][10] + coeff0_35*dmats2[35][10] + coeff0_36*dmats2[36][10] + coeff0_37*dmats2[37][10] + coeff0_38*dmats2[38][10] + coeff0_39*dmats2[39][10] + coeff0_40*dmats2[40][10] + coeff0_41*dmats2[41][10] + coeff0_42*dmats2[42][10] + coeff0_43*dmats2[43][10] + coeff0_44*dmats2[44][10] + coeff0_45*dmats2[45][10] + coeff0_46*dmats2[46][10] + coeff0_47*dmats2[47][10] + coeff0_48*dmats2[48][10] + coeff0_49*dmats2[49][10] + coeff0_50*dmats2[50][10] + coeff0_51*dmats2[51][10] + coeff0_52*dmats2[52][10] + coeff0_53*dmats2[53][10] + coeff0_54*dmats2[54][10] + coeff0_55*dmats2[55][10];
          new_coeff0_11 = coeff0_0*dmats2[0][11] + coeff0_1*dmats2[1][11] + coeff0_2*dmats2[2][11] + coeff0_3*dmats2[3][11] + coeff0_4*dmats2[4][11] + coeff0_5*dmats2[5][11] + coeff0_6*dmats2[6][11] + coeff0_7*dmats2[7][11] + coeff0_8*dmats2[8][11] + coeff0_9*dmats2[9][11] + coeff0_10*dmats2[10][11] + coeff0_11*dmats2[11][11] + coeff0_12*dmats2[12][11] + coeff0_13*dmats2[13][11] + coeff0_14*dmats2[14][11] + coeff0_15*dmats2[15][11] + coeff0_16*dmats2[16][11] + coeff0_17*dmats2[17][11] + coeff0_18*dmats2[18][11] + coeff0_19*dmats2[19][11] + coeff0_20*dmats2[20][11] + coeff0_21*dmats2[21][11] + coeff0_22*dmats2[22][11] + coeff0_23*dmats2[23][11] + coeff0_24*dmats2[24][11] + coeff0_25*dmats2[25][11] + coeff0_26*dmats2[26][11] + coeff0_27*dmats2[27][11] + coeff0_28*dmats2[28][11] + coeff0_29*dmats2[29][11] + coeff0_30*dmats2[30][11] + coeff0_31*dmats2[31][11] + coeff0_32*dmats2[32][11] + coeff0_33*dmats2[33][11] + coeff0_34*dmats2[34][11] + coeff0_35*dmats2[35][11] + coeff0_36*dmats2[36][11] + coeff0_37*dmats2[37][11] + coeff0_38*dmats2[38][11] + coeff0_39*dmats2[39][11] + coeff0_40*dmats2[40][11] + coeff0_41*dmats2[41][11] + coeff0_42*dmats2[42][11] + coeff0_43*dmats2[43][11] + coeff0_44*dmats2[44][11] + coeff0_45*dmats2[45][11] + coeff0_46*dmats2[46][11] + coeff0_47*dmats2[47][11] + coeff0_48*dmats2[48][11] + coeff0_49*dmats2[49][11] + coeff0_50*dmats2[50][11] + coeff0_51*dmats2[51][11] + coeff0_52*dmats2[52][11] + coeff0_53*dmats2[53][11] + coeff0_54*dmats2[54][11] + coeff0_55*dmats2[55][11];
          new_coeff0_12 = coeff0_0*dmats2[0][12] + coeff0_1*dmats2[1][12] + coeff0_2*dmats2[2][12] + coeff0_3*dmats2[3][12] + coeff0_4*dmats2[4][12] + coeff0_5*dmats2[5][12] + coeff0_6*dmats2[6][12] + coeff0_7*dmats2[7][12] + coeff0_8*dmats2[8][12] + coeff0_9*dmats2[9][12] + coeff0_10*dmats2[10][12] + coeff0_11*dmats2[11][12] + coeff0_12*dmats2[12][12] + coeff0_13*dmats2[13][12] + coeff0_14*dmats2[14][12] + coeff0_15*dmats2[15][12] + coeff0_16*dmats2[16][12] + coeff0_17*dmats2[17][12] + coeff0_18*dmats2[18][12] + coeff0_19*dmats2[19][12] + coeff0_20*dmats2[20][12] + coeff0_21*dmats2[21][12] + coeff0_22*dmats2[22][12] + coeff0_23*dmats2[23][12] + coeff0_24*dmats2[24][12] + coeff0_25*dmats2[25][12] + coeff0_26*dmats2[26][12] + coeff0_27*dmats2[27][12] + coeff0_28*dmats2[28][12] + coeff0_29*dmats2[29][12] + coeff0_30*dmats2[30][12] + coeff0_31*dmats2[31][12] + coeff0_32*dmats2[32][12] + coeff0_33*dmats2[33][12] + coeff0_34*dmats2[34][12] + coeff0_35*dmats2[35][12] + coeff0_36*dmats2[36][12] + coeff0_37*dmats2[37][12] + coeff0_38*dmats2[38][12] + coeff0_39*dmats2[39][12] + coeff0_40*dmats2[40][12] + coeff0_41*dmats2[41][12] + coeff0_42*dmats2[42][12] + coeff0_43*dmats2[43][12] + coeff0_44*dmats2[44][12] + coeff0_45*dmats2[45][12] + coeff0_46*dmats2[46][12] + coeff0_47*dmats2[47][12] + coeff0_48*dmats2[48][12] + coeff0_49*dmats2[49][12] + coeff0_50*dmats2[50][12] + coeff0_51*dmats2[51][12] + coeff0_52*dmats2[52][12] + coeff0_53*dmats2[53][12] + coeff0_54*dmats2[54][12] + coeff0_55*dmats2[55][12];
          new_coeff0_13 = coeff0_0*dmats2[0][13] + coeff0_1*dmats2[1][13] + coeff0_2*dmats2[2][13] + coeff0_3*dmats2[3][13] + coeff0_4*dmats2[4][13] + coeff0_5*dmats2[5][13] + coeff0_6*dmats2[6][13] + coeff0_7*dmats2[7][13] + coeff0_8*dmats2[8][13] + coeff0_9*dmats2[9][13] + coeff0_10*dmats2[10][13] + coeff0_11*dmats2[11][13] + coeff0_12*dmats2[12][13] + coeff0_13*dmats2[13][13] + coeff0_14*dmats2[14][13] + coeff0_15*dmats2[15][13] + coeff0_16*dmats2[16][13] + coeff0_17*dmats2[17][13] + coeff0_18*dmats2[18][13] + coeff0_19*dmats2[19][13] + coeff0_20*dmats2[20][13] + coeff0_21*dmats2[21][13] + coeff0_22*dmats2[22][13] + coeff0_23*dmats2[23][13] + coeff0_24*dmats2[24][13] + coeff0_25*dmats2[25][13] + coeff0_26*dmats2[26][13] + coeff0_27*dmats2[27][13] + coeff0_28*dmats2[28][13] + coeff0_29*dmats2[29][13] + coeff0_30*dmats2[30][13] + coeff0_31*dmats2[31][13] + coeff0_32*dmats2[32][13] + coeff0_33*dmats2[33][13] + coeff0_34*dmats2[34][13] + coeff0_35*dmats2[35][13] + coeff0_36*dmats2[36][13] + coeff0_37*dmats2[37][13] + coeff0_38*dmats2[38][13] + coeff0_39*dmats2[39][13] + coeff0_40*dmats2[40][13] + coeff0_41*dmats2[41][13] + coeff0_42*dmats2[42][13] + coeff0_43*dmats2[43][13] + coeff0_44*dmats2[44][13] + coeff0_45*dmats2[45][13] + coeff0_46*dmats2[46][13] + coeff0_47*dmats2[47][13] + coeff0_48*dmats2[48][13] + coeff0_49*dmats2[49][13] + coeff0_50*dmats2[50][13] + coeff0_51*dmats2[51][13] + coeff0_52*dmats2[52][13] + coeff0_53*dmats2[53][13] + coeff0_54*dmats2[54][13] + coeff0_55*dmats2[55][13];
          new_coeff0_14 = coeff0_0*dmats2[0][14] + coeff0_1*dmats2[1][14] + coeff0_2*dmats2[2][14] + coeff0_3*dmats2[3][14] + coeff0_4*dmats2[4][14] + coeff0_5*dmats2[5][14] + coeff0_6*dmats2[6][14] + coeff0_7*dmats2[7][14] + coeff0_8*dmats2[8][14] + coeff0_9*dmats2[9][14] + coeff0_10*dmats2[10][14] + coeff0_11*dmats2[11][14] + coeff0_12*dmats2[12][14] + coeff0_13*dmats2[13][14] + coeff0_14*dmats2[14][14] + coeff0_15*dmats2[15][14] + coeff0_16*dmats2[16][14] + coeff0_17*dmats2[17][14] + coeff0_18*dmats2[18][14] + coeff0_19*dmats2[19][14] + coeff0_20*dmats2[20][14] + coeff0_21*dmats2[21][14] + coeff0_22*dmats2[22][14] + coeff0_23*dmats2[23][14] + coeff0_24*dmats2[24][14] + coeff0_25*dmats2[25][14] + coeff0_26*dmats2[26][14] + coeff0_27*dmats2[27][14] + coeff0_28*dmats2[28][14] + coeff0_29*dmats2[29][14] + coeff0_30*dmats2[30][14] + coeff0_31*dmats2[31][14] + coeff0_32*dmats2[32][14] + coeff0_33*dmats2[33][14] + coeff0_34*dmats2[34][14] + coeff0_35*dmats2[35][14] + coeff0_36*dmats2[36][14] + coeff0_37*dmats2[37][14] + coeff0_38*dmats2[38][14] + coeff0_39*dmats2[39][14] + coeff0_40*dmats2[40][14] + coeff0_41*dmats2[41][14] + coeff0_42*dmats2[42][14] + coeff0_43*dmats2[43][14] + coeff0_44*dmats2[44][14] + coeff0_45*dmats2[45][14] + coeff0_46*dmats2[46][14] + coeff0_47*dmats2[47][14] + coeff0_48*dmats2[48][14] + coeff0_49*dmats2[49][14] + coeff0_50*dmats2[50][14] + coeff0_51*dmats2[51][14] + coeff0_52*dmats2[52][14] + coeff0_53*dmats2[53][14] + coeff0_54*dmats2[54][14] + coeff0_55*dmats2[55][14];
          new_coeff0_15 = coeff0_0*dmats2[0][15] + coeff0_1*dmats2[1][15] + coeff0_2*dmats2[2][15] + coeff0_3*dmats2[3][15] + coeff0_4*dmats2[4][15] + coeff0_5*dmats2[5][15] + coeff0_6*dmats2[6][15] + coeff0_7*dmats2[7][15] + coeff0_8*dmats2[8][15] + coeff0_9*dmats2[9][15] + coeff0_10*dmats2[10][15] + coeff0_11*dmats2[11][15] + coeff0_12*dmats2[12][15] + coeff0_13*dmats2[13][15] + coeff0_14*dmats2[14][15] + coeff0_15*dmats2[15][15] + coeff0_16*dmats2[16][15] + coeff0_17*dmats2[17][15] + coeff0_18*dmats2[18][15] + coeff0_19*dmats2[19][15] + coeff0_20*dmats2[20][15] + coeff0_21*dmats2[21][15] + coeff0_22*dmats2[22][15] + coeff0_23*dmats2[23][15] + coeff0_24*dmats2[24][15] + coeff0_25*dmats2[25][15] + coeff0_26*dmats2[26][15] + coeff0_27*dmats2[27][15] + coeff0_28*dmats2[28][15] + coeff0_29*dmats2[29][15] + coeff0_30*dmats2[30][15] + coeff0_31*dmats2[31][15] + coeff0_32*dmats2[32][15] + coeff0_33*dmats2[33][15] + coeff0_34*dmats2[34][15] + coeff0_35*dmats2[35][15] + coeff0_36*dmats2[36][15] + coeff0_37*dmats2[37][15] + coeff0_38*dmats2[38][15] + coeff0_39*dmats2[39][15] + coeff0_40*dmats2[40][15] + coeff0_41*dmats2[41][15] + coeff0_42*dmats2[42][15] + coeff0_43*dmats2[43][15] + coeff0_44*dmats2[44][15] + coeff0_45*dmats2[45][15] + coeff0_46*dmats2[46][15] + coeff0_47*dmats2[47][15] + coeff0_48*dmats2[48][15] + coeff0_49*dmats2[49][15] + coeff0_50*dmats2[50][15] + coeff0_51*dmats2[51][15] + coeff0_52*dmats2[52][15] + coeff0_53*dmats2[53][15] + coeff0_54*dmats2[54][15] + coeff0_55*dmats2[55][15];
          new_coeff0_16 = coeff0_0*dmats2[0][16] + coeff0_1*dmats2[1][16] + coeff0_2*dmats2[2][16] + coeff0_3*dmats2[3][16] + coeff0_4*dmats2[4][16] + coeff0_5*dmats2[5][16] + coeff0_6*dmats2[6][16] + coeff0_7*dmats2[7][16] + coeff0_8*dmats2[8][16] + coeff0_9*dmats2[9][16] + coeff0_10*dmats2[10][16] + coeff0_11*dmats2[11][16] + coeff0_12*dmats2[12][16] + coeff0_13*dmats2[13][16] + coeff0_14*dmats2[14][16] + coeff0_15*dmats2[15][16] + coeff0_16*dmats2[16][16] + coeff0_17*dmats2[17][16] + coeff0_18*dmats2[18][16] + coeff0_19*dmats2[19][16] + coeff0_20*dmats2[20][16] + coeff0_21*dmats2[21][16] + coeff0_22*dmats2[22][16] + coeff0_23*dmats2[23][16] + coeff0_24*dmats2[24][16] + coeff0_25*dmats2[25][16] + coeff0_26*dmats2[26][16] + coeff0_27*dmats2[27][16] + coeff0_28*dmats2[28][16] + coeff0_29*dmats2[29][16] + coeff0_30*dmats2[30][16] + coeff0_31*dmats2[31][16] + coeff0_32*dmats2[32][16] + coeff0_33*dmats2[33][16] + coeff0_34*dmats2[34][16] + coeff0_35*dmats2[35][16] + coeff0_36*dmats2[36][16] + coeff0_37*dmats2[37][16] + coeff0_38*dmats2[38][16] + coeff0_39*dmats2[39][16] + coeff0_40*dmats2[40][16] + coeff0_41*dmats2[41][16] + coeff0_42*dmats2[42][16] + coeff0_43*dmats2[43][16] + coeff0_44*dmats2[44][16] + coeff0_45*dmats2[45][16] + coeff0_46*dmats2[46][16] + coeff0_47*dmats2[47][16] + coeff0_48*dmats2[48][16] + coeff0_49*dmats2[49][16] + coeff0_50*dmats2[50][16] + coeff0_51*dmats2[51][16] + coeff0_52*dmats2[52][16] + coeff0_53*dmats2[53][16] + coeff0_54*dmats2[54][16] + coeff0_55*dmats2[55][16];
          new_coeff0_17 = coeff0_0*dmats2[0][17] + coeff0_1*dmats2[1][17] + coeff0_2*dmats2[2][17] + coeff0_3*dmats2[3][17] + coeff0_4*dmats2[4][17] + coeff0_5*dmats2[5][17] + coeff0_6*dmats2[6][17] + coeff0_7*dmats2[7][17] + coeff0_8*dmats2[8][17] + coeff0_9*dmats2[9][17] + coeff0_10*dmats2[10][17] + coeff0_11*dmats2[11][17] + coeff0_12*dmats2[12][17] + coeff0_13*dmats2[13][17] + coeff0_14*dmats2[14][17] + coeff0_15*dmats2[15][17] + coeff0_16*dmats2[16][17] + coeff0_17*dmats2[17][17] + coeff0_18*dmats2[18][17] + coeff0_19*dmats2[19][17] + coeff0_20*dmats2[20][17] + coeff0_21*dmats2[21][17] + coeff0_22*dmats2[22][17] + coeff0_23*dmats2[23][17] + coeff0_24*dmats2[24][17] + coeff0_25*dmats2[25][17] + coeff0_26*dmats2[26][17] + coeff0_27*dmats2[27][17] + coeff0_28*dmats2[28][17] + coeff0_29*dmats2[29][17] + coeff0_30*dmats2[30][17] + coeff0_31*dmats2[31][17] + coeff0_32*dmats2[32][17] + coeff0_33*dmats2[33][17] + coeff0_34*dmats2[34][17] + coeff0_35*dmats2[35][17] + coeff0_36*dmats2[36][17] + coeff0_37*dmats2[37][17] + coeff0_38*dmats2[38][17] + coeff0_39*dmats2[39][17] + coeff0_40*dmats2[40][17] + coeff0_41*dmats2[41][17] + coeff0_42*dmats2[42][17] + coeff0_43*dmats2[43][17] + coeff0_44*dmats2[44][17] + coeff0_45*dmats2[45][17] + coeff0_46*dmats2[46][17] + coeff0_47*dmats2[47][17] + coeff0_48*dmats2[48][17] + coeff0_49*dmats2[49][17] + coeff0_50*dmats2[50][17] + coeff0_51*dmats2[51][17] + coeff0_52*dmats2[52][17] + coeff0_53*dmats2[53][17] + coeff0_54*dmats2[54][17] + coeff0_55*dmats2[55][17];
          new_coeff0_18 = coeff0_0*dmats2[0][18] + coeff0_1*dmats2[1][18] + coeff0_2*dmats2[2][18] + coeff0_3*dmats2[3][18] + coeff0_4*dmats2[4][18] + coeff0_5*dmats2[5][18] + coeff0_6*dmats2[6][18] + coeff0_7*dmats2[7][18] + coeff0_8*dmats2[8][18] + coeff0_9*dmats2[9][18] + coeff0_10*dmats2[10][18] + coeff0_11*dmats2[11][18] + coeff0_12*dmats2[12][18] + coeff0_13*dmats2[13][18] + coeff0_14*dmats2[14][18] + coeff0_15*dmats2[15][18] + coeff0_16*dmats2[16][18] + coeff0_17*dmats2[17][18] + coeff0_18*dmats2[18][18] + coeff0_19*dmats2[19][18] + coeff0_20*dmats2[20][18] + coeff0_21*dmats2[21][18] + coeff0_22*dmats2[22][18] + coeff0_23*dmats2[23][18] + coeff0_24*dmats2[24][18] + coeff0_25*dmats2[25][18] + coeff0_26*dmats2[26][18] + coeff0_27*dmats2[27][18] + coeff0_28*dmats2[28][18] + coeff0_29*dmats2[29][18] + coeff0_30*dmats2[30][18] + coeff0_31*dmats2[31][18] + coeff0_32*dmats2[32][18] + coeff0_33*dmats2[33][18] + coeff0_34*dmats2[34][18] + coeff0_35*dmats2[35][18] + coeff0_36*dmats2[36][18] + coeff0_37*dmats2[37][18] + coeff0_38*dmats2[38][18] + coeff0_39*dmats2[39][18] + coeff0_40*dmats2[40][18] + coeff0_41*dmats2[41][18] + coeff0_42*dmats2[42][18] + coeff0_43*dmats2[43][18] + coeff0_44*dmats2[44][18] + coeff0_45*dmats2[45][18] + coeff0_46*dmats2[46][18] + coeff0_47*dmats2[47][18] + coeff0_48*dmats2[48][18] + coeff0_49*dmats2[49][18] + coeff0_50*dmats2[50][18] + coeff0_51*dmats2[51][18] + coeff0_52*dmats2[52][18] + coeff0_53*dmats2[53][18] + coeff0_54*dmats2[54][18] + coeff0_55*dmats2[55][18];
          new_coeff0_19 = coeff0_0*dmats2[0][19] + coeff0_1*dmats2[1][19] + coeff0_2*dmats2[2][19] + coeff0_3*dmats2[3][19] + coeff0_4*dmats2[4][19] + coeff0_5*dmats2[5][19] + coeff0_6*dmats2[6][19] + coeff0_7*dmats2[7][19] + coeff0_8*dmats2[8][19] + coeff0_9*dmats2[9][19] + coeff0_10*dmats2[10][19] + coeff0_11*dmats2[11][19] + coeff0_12*dmats2[12][19] + coeff0_13*dmats2[13][19] + coeff0_14*dmats2[14][19] + coeff0_15*dmats2[15][19] + coeff0_16*dmats2[16][19] + coeff0_17*dmats2[17][19] + coeff0_18*dmats2[18][19] + coeff0_19*dmats2[19][19] + coeff0_20*dmats2[20][19] + coeff0_21*dmats2[21][19] + coeff0_22*dmats2[22][19] + coeff0_23*dmats2[23][19] + coeff0_24*dmats2[24][19] + coeff0_25*dmats2[25][19] + coeff0_26*dmats2[26][19] + coeff0_27*dmats2[27][19] + coeff0_28*dmats2[28][19] + coeff0_29*dmats2[29][19] + coeff0_30*dmats2[30][19] + coeff0_31*dmats2[31][19] + coeff0_32*dmats2[32][19] + coeff0_33*dmats2[33][19] + coeff0_34*dmats2[34][19] + coeff0_35*dmats2[35][19] + coeff0_36*dmats2[36][19] + coeff0_37*dmats2[37][19] + coeff0_38*dmats2[38][19] + coeff0_39*dmats2[39][19] + coeff0_40*dmats2[40][19] + coeff0_41*dmats2[41][19] + coeff0_42*dmats2[42][19] + coeff0_43*dmats2[43][19] + coeff0_44*dmats2[44][19] + coeff0_45*dmats2[45][19] + coeff0_46*dmats2[46][19] + coeff0_47*dmats2[47][19] + coeff0_48*dmats2[48][19] + coeff0_49*dmats2[49][19] + coeff0_50*dmats2[50][19] + coeff0_51*dmats2[51][19] + coeff0_52*dmats2[52][19] + coeff0_53*dmats2[53][19] + coeff0_54*dmats2[54][19] + coeff0_55*dmats2[55][19];
          new_coeff0_20 = coeff0_0*dmats2[0][20] + coeff0_1*dmats2[1][20] + coeff0_2*dmats2[2][20] + coeff0_3*dmats2[3][20] + coeff0_4*dmats2[4][20] + coeff0_5*dmats2[5][20] + coeff0_6*dmats2[6][20] + coeff0_7*dmats2[7][20] + coeff0_8*dmats2[8][20] + coeff0_9*dmats2[9][20] + coeff0_10*dmats2[10][20] + coeff0_11*dmats2[11][20] + coeff0_12*dmats2[12][20] + coeff0_13*dmats2[13][20] + coeff0_14*dmats2[14][20] + coeff0_15*dmats2[15][20] + coeff0_16*dmats2[16][20] + coeff0_17*dmats2[17][20] + coeff0_18*dmats2[18][20] + coeff0_19*dmats2[19][20] + coeff0_20*dmats2[20][20] + coeff0_21*dmats2[21][20] + coeff0_22*dmats2[22][20] + coeff0_23*dmats2[23][20] + coeff0_24*dmats2[24][20] + coeff0_25*dmats2[25][20] + coeff0_26*dmats2[26][20] + coeff0_27*dmats2[27][20] + coeff0_28*dmats2[28][20] + coeff0_29*dmats2[29][20] + coeff0_30*dmats2[30][20] + coeff0_31*dmats2[31][20] + coeff0_32*dmats2[32][20] + coeff0_33*dmats2[33][20] + coeff0_34*dmats2[34][20] + coeff0_35*dmats2[35][20] + coeff0_36*dmats2[36][20] + coeff0_37*dmats2[37][20] + coeff0_38*dmats2[38][20] + coeff0_39*dmats2[39][20] + coeff0_40*dmats2[40][20] + coeff0_41*dmats2[41][20] + coeff0_42*dmats2[42][20] + coeff0_43*dmats2[43][20] + coeff0_44*dmats2[44][20] + coeff0_45*dmats2[45][20] + coeff0_46*dmats2[46][20] + coeff0_47*dmats2[47][20] + coeff0_48*dmats2[48][20] + coeff0_49*dmats2[49][20] + coeff0_50*dmats2[50][20] + coeff0_51*dmats2[51][20] + coeff0_52*dmats2[52][20] + coeff0_53*dmats2[53][20] + coeff0_54*dmats2[54][20] + coeff0_55*dmats2[55][20];
          new_coeff0_21 = coeff0_0*dmats2[0][21] + coeff0_1*dmats2[1][21] + coeff0_2*dmats2[2][21] + coeff0_3*dmats2[3][21] + coeff0_4*dmats2[4][21] + coeff0_5*dmats2[5][21] + coeff0_6*dmats2[6][21] + coeff0_7*dmats2[7][21] + coeff0_8*dmats2[8][21] + coeff0_9*dmats2[9][21] + coeff0_10*dmats2[10][21] + coeff0_11*dmats2[11][21] + coeff0_12*dmats2[12][21] + coeff0_13*dmats2[13][21] + coeff0_14*dmats2[14][21] + coeff0_15*dmats2[15][21] + coeff0_16*dmats2[16][21] + coeff0_17*dmats2[17][21] + coeff0_18*dmats2[18][21] + coeff0_19*dmats2[19][21] + coeff0_20*dmats2[20][21] + coeff0_21*dmats2[21][21] + coeff0_22*dmats2[22][21] + coeff0_23*dmats2[23][21] + coeff0_24*dmats2[24][21] + coeff0_25*dmats2[25][21] + coeff0_26*dmats2[26][21] + coeff0_27*dmats2[27][21] + coeff0_28*dmats2[28][21] + coeff0_29*dmats2[29][21] + coeff0_30*dmats2[30][21] + coeff0_31*dmats2[31][21] + coeff0_32*dmats2[32][21] + coeff0_33*dmats2[33][21] + coeff0_34*dmats2[34][21] + coeff0_35*dmats2[35][21] + coeff0_36*dmats2[36][21] + coeff0_37*dmats2[37][21] + coeff0_38*dmats2[38][21] + coeff0_39*dmats2[39][21] + coeff0_40*dmats2[40][21] + coeff0_41*dmats2[41][21] + coeff0_42*dmats2[42][21] + coeff0_43*dmats2[43][21] + coeff0_44*dmats2[44][21] + coeff0_45*dmats2[45][21] + coeff0_46*dmats2[46][21] + coeff0_47*dmats2[47][21] + coeff0_48*dmats2[48][21] + coeff0_49*dmats2[49][21] + coeff0_50*dmats2[50][21] + coeff0_51*dmats2[51][21] + coeff0_52*dmats2[52][21] + coeff0_53*dmats2[53][21] + coeff0_54*dmats2[54][21] + coeff0_55*dmats2[55][21];
          new_coeff0_22 = coeff0_0*dmats2[0][22] + coeff0_1*dmats2[1][22] + coeff0_2*dmats2[2][22] + coeff0_3*dmats2[3][22] + coeff0_4*dmats2[4][22] + coeff0_5*dmats2[5][22] + coeff0_6*dmats2[6][22] + coeff0_7*dmats2[7][22] + coeff0_8*dmats2[8][22] + coeff0_9*dmats2[9][22] + coeff0_10*dmats2[10][22] + coeff0_11*dmats2[11][22] + coeff0_12*dmats2[12][22] + coeff0_13*dmats2[13][22] + coeff0_14*dmats2[14][22] + coeff0_15*dmats2[15][22] + coeff0_16*dmats2[16][22] + coeff0_17*dmats2[17][22] + coeff0_18*dmats2[18][22] + coeff0_19*dmats2[19][22] + coeff0_20*dmats2[20][22] + coeff0_21*dmats2[21][22] + coeff0_22*dmats2[22][22] + coeff0_23*dmats2[23][22] + coeff0_24*dmats2[24][22] + coeff0_25*dmats2[25][22] + coeff0_26*dmats2[26][22] + coeff0_27*dmats2[27][22] + coeff0_28*dmats2[28][22] + coeff0_29*dmats2[29][22] + coeff0_30*dmats2[30][22] + coeff0_31*dmats2[31][22] + coeff0_32*dmats2[32][22] + coeff0_33*dmats2[33][22] + coeff0_34*dmats2[34][22] + coeff0_35*dmats2[35][22] + coeff0_36*dmats2[36][22] + coeff0_37*dmats2[37][22] + coeff0_38*dmats2[38][22] + coeff0_39*dmats2[39][22] + coeff0_40*dmats2[40][22] + coeff0_41*dmats2[41][22] + coeff0_42*dmats2[42][22] + coeff0_43*dmats2[43][22] + coeff0_44*dmats2[44][22] + coeff0_45*dmats2[45][22] + coeff0_46*dmats2[46][22] + coeff0_47*dmats2[47][22] + coeff0_48*dmats2[48][22] + coeff0_49*dmats2[49][22] + coeff0_50*dmats2[50][22] + coeff0_51*dmats2[51][22] + coeff0_52*dmats2[52][22] + coeff0_53*dmats2[53][22] + coeff0_54*dmats2[54][22] + coeff0_55*dmats2[55][22];
          new_coeff0_23 = coeff0_0*dmats2[0][23] + coeff0_1*dmats2[1][23] + coeff0_2*dmats2[2][23] + coeff0_3*dmats2[3][23] + coeff0_4*dmats2[4][23] + coeff0_5*dmats2[5][23] + coeff0_6*dmats2[6][23] + coeff0_7*dmats2[7][23] + coeff0_8*dmats2[8][23] + coeff0_9*dmats2[9][23] + coeff0_10*dmats2[10][23] + coeff0_11*dmats2[11][23] + coeff0_12*dmats2[12][23] + coeff0_13*dmats2[13][23] + coeff0_14*dmats2[14][23] + coeff0_15*dmats2[15][23] + coeff0_16*dmats2[16][23] + coeff0_17*dmats2[17][23] + coeff0_18*dmats2[18][23] + coeff0_19*dmats2[19][23] + coeff0_20*dmats2[20][23] + coeff0_21*dmats2[21][23] + coeff0_22*dmats2[22][23] + coeff0_23*dmats2[23][23] + coeff0_24*dmats2[24][23] + coeff0_25*dmats2[25][23] + coeff0_26*dmats2[26][23] + coeff0_27*dmats2[27][23] + coeff0_28*dmats2[28][23] + coeff0_29*dmats2[29][23] + coeff0_30*dmats2[30][23] + coeff0_31*dmats2[31][23] + coeff0_32*dmats2[32][23] + coeff0_33*dmats2[33][23] + coeff0_34*dmats2[34][23] + coeff0_35*dmats2[35][23] + coeff0_36*dmats2[36][23] + coeff0_37*dmats2[37][23] + coeff0_38*dmats2[38][23] + coeff0_39*dmats2[39][23] + coeff0_40*dmats2[40][23] + coeff0_41*dmats2[41][23] + coeff0_42*dmats2[42][23] + coeff0_43*dmats2[43][23] + coeff0_44*dmats2[44][23] + coeff0_45*dmats2[45][23] + coeff0_46*dmats2[46][23] + coeff0_47*dmats2[47][23] + coeff0_48*dmats2[48][23] + coeff0_49*dmats2[49][23] + coeff0_50*dmats2[50][23] + coeff0_51*dmats2[51][23] + coeff0_52*dmats2[52][23] + coeff0_53*dmats2[53][23] + coeff0_54*dmats2[54][23] + coeff0_55*dmats2[55][23];
          new_coeff0_24 = coeff0_0*dmats2[0][24] + coeff0_1*dmats2[1][24] + coeff0_2*dmats2[2][24] + coeff0_3*dmats2[3][24] + coeff0_4*dmats2[4][24] + coeff0_5*dmats2[5][24] + coeff0_6*dmats2[6][24] + coeff0_7*dmats2[7][24] + coeff0_8*dmats2[8][24] + coeff0_9*dmats2[9][24] + coeff0_10*dmats2[10][24] + coeff0_11*dmats2[11][24] + coeff0_12*dmats2[12][24] + coeff0_13*dmats2[13][24] + coeff0_14*dmats2[14][24] + coeff0_15*dmats2[15][24] + coeff0_16*dmats2[16][24] + coeff0_17*dmats2[17][24] + coeff0_18*dmats2[18][24] + coeff0_19*dmats2[19][24] + coeff0_20*dmats2[20][24] + coeff0_21*dmats2[21][24] + coeff0_22*dmats2[22][24] + coeff0_23*dmats2[23][24] + coeff0_24*dmats2[24][24] + coeff0_25*dmats2[25][24] + coeff0_26*dmats2[26][24] + coeff0_27*dmats2[27][24] + coeff0_28*dmats2[28][24] + coeff0_29*dmats2[29][24] + coeff0_30*dmats2[30][24] + coeff0_31*dmats2[31][24] + coeff0_32*dmats2[32][24] + coeff0_33*dmats2[33][24] + coeff0_34*dmats2[34][24] + coeff0_35*dmats2[35][24] + coeff0_36*dmats2[36][24] + coeff0_37*dmats2[37][24] + coeff0_38*dmats2[38][24] + coeff0_39*dmats2[39][24] + coeff0_40*dmats2[40][24] + coeff0_41*dmats2[41][24] + coeff0_42*dmats2[42][24] + coeff0_43*dmats2[43][24] + coeff0_44*dmats2[44][24] + coeff0_45*dmats2[45][24] + coeff0_46*dmats2[46][24] + coeff0_47*dmats2[47][24] + coeff0_48*dmats2[48][24] + coeff0_49*dmats2[49][24] + coeff0_50*dmats2[50][24] + coeff0_51*dmats2[51][24] + coeff0_52*dmats2[52][24] + coeff0_53*dmats2[53][24] + coeff0_54*dmats2[54][24] + coeff0_55*dmats2[55][24];
          new_coeff0_25 = coeff0_0*dmats2[0][25] + coeff0_1*dmats2[1][25] + coeff0_2*dmats2[2][25] + coeff0_3*dmats2[3][25] + coeff0_4*dmats2[4][25] + coeff0_5*dmats2[5][25] + coeff0_6*dmats2[6][25] + coeff0_7*dmats2[7][25] + coeff0_8*dmats2[8][25] + coeff0_9*dmats2[9][25] + coeff0_10*dmats2[10][25] + coeff0_11*dmats2[11][25] + coeff0_12*dmats2[12][25] + coeff0_13*dmats2[13][25] + coeff0_14*dmats2[14][25] + coeff0_15*dmats2[15][25] + coeff0_16*dmats2[16][25] + coeff0_17*dmats2[17][25] + coeff0_18*dmats2[18][25] + coeff0_19*dmats2[19][25] + coeff0_20*dmats2[20][25] + coeff0_21*dmats2[21][25] + coeff0_22*dmats2[22][25] + coeff0_23*dmats2[23][25] + coeff0_24*dmats2[24][25] + coeff0_25*dmats2[25][25] + coeff0_26*dmats2[26][25] + coeff0_27*dmats2[27][25] + coeff0_28*dmats2[28][25] + coeff0_29*dmats2[29][25] + coeff0_30*dmats2[30][25] + coeff0_31*dmats2[31][25] + coeff0_32*dmats2[32][25] + coeff0_33*dmats2[33][25] + coeff0_34*dmats2[34][25] + coeff0_35*dmats2[35][25] + coeff0_36*dmats2[36][25] + coeff0_37*dmats2[37][25] + coeff0_38*dmats2[38][25] + coeff0_39*dmats2[39][25] + coeff0_40*dmats2[40][25] + coeff0_41*dmats2[41][25] + coeff0_42*dmats2[42][25] + coeff0_43*dmats2[43][25] + coeff0_44*dmats2[44][25] + coeff0_45*dmats2[45][25] + coeff0_46*dmats2[46][25] + coeff0_47*dmats2[47][25] + coeff0_48*dmats2[48][25] + coeff0_49*dmats2[49][25] + coeff0_50*dmats2[50][25] + coeff0_51*dmats2[51][25] + coeff0_52*dmats2[52][25] + coeff0_53*dmats2[53][25] + coeff0_54*dmats2[54][25] + coeff0_55*dmats2[55][25];
          new_coeff0_26 = coeff0_0*dmats2[0][26] + coeff0_1*dmats2[1][26] + coeff0_2*dmats2[2][26] + coeff0_3*dmats2[3][26] + coeff0_4*dmats2[4][26] + coeff0_5*dmats2[5][26] + coeff0_6*dmats2[6][26] + coeff0_7*dmats2[7][26] + coeff0_8*dmats2[8][26] + coeff0_9*dmats2[9][26] + coeff0_10*dmats2[10][26] + coeff0_11*dmats2[11][26] + coeff0_12*dmats2[12][26] + coeff0_13*dmats2[13][26] + coeff0_14*dmats2[14][26] + coeff0_15*dmats2[15][26] + coeff0_16*dmats2[16][26] + coeff0_17*dmats2[17][26] + coeff0_18*dmats2[18][26] + coeff0_19*dmats2[19][26] + coeff0_20*dmats2[20][26] + coeff0_21*dmats2[21][26] + coeff0_22*dmats2[22][26] + coeff0_23*dmats2[23][26] + coeff0_24*dmats2[24][26] + coeff0_25*dmats2[25][26] + coeff0_26*dmats2[26][26] + coeff0_27*dmats2[27][26] + coeff0_28*dmats2[28][26] + coeff0_29*dmats2[29][26] + coeff0_30*dmats2[30][26] + coeff0_31*dmats2[31][26] + coeff0_32*dmats2[32][26] + coeff0_33*dmats2[33][26] + coeff0_34*dmats2[34][26] + coeff0_35*dmats2[35][26] + coeff0_36*dmats2[36][26] + coeff0_37*dmats2[37][26] + coeff0_38*dmats2[38][26] + coeff0_39*dmats2[39][26] + coeff0_40*dmats2[40][26] + coeff0_41*dmats2[41][26] + coeff0_42*dmats2[42][26] + coeff0_43*dmats2[43][26] + coeff0_44*dmats2[44][26] + coeff0_45*dmats2[45][26] + coeff0_46*dmats2[46][26] + coeff0_47*dmats2[47][26] + coeff0_48*dmats2[48][26] + coeff0_49*dmats2[49][26] + coeff0_50*dmats2[50][26] + coeff0_51*dmats2[51][26] + coeff0_52*dmats2[52][26] + coeff0_53*dmats2[53][26] + coeff0_54*dmats2[54][26] + coeff0_55*dmats2[55][26];
          new_coeff0_27 = coeff0_0*dmats2[0][27] + coeff0_1*dmats2[1][27] + coeff0_2*dmats2[2][27] + coeff0_3*dmats2[3][27] + coeff0_4*dmats2[4][27] + coeff0_5*dmats2[5][27] + coeff0_6*dmats2[6][27] + coeff0_7*dmats2[7][27] + coeff0_8*dmats2[8][27] + coeff0_9*dmats2[9][27] + coeff0_10*dmats2[10][27] + coeff0_11*dmats2[11][27] + coeff0_12*dmats2[12][27] + coeff0_13*dmats2[13][27] + coeff0_14*dmats2[14][27] + coeff0_15*dmats2[15][27] + coeff0_16*dmats2[16][27] + coeff0_17*dmats2[17][27] + coeff0_18*dmats2[18][27] + coeff0_19*dmats2[19][27] + coeff0_20*dmats2[20][27] + coeff0_21*dmats2[21][27] + coeff0_22*dmats2[22][27] + coeff0_23*dmats2[23][27] + coeff0_24*dmats2[24][27] + coeff0_25*dmats2[25][27] + coeff0_26*dmats2[26][27] + coeff0_27*dmats2[27][27] + coeff0_28*dmats2[28][27] + coeff0_29*dmats2[29][27] + coeff0_30*dmats2[30][27] + coeff0_31*dmats2[31][27] + coeff0_32*dmats2[32][27] + coeff0_33*dmats2[33][27] + coeff0_34*dmats2[34][27] + coeff0_35*dmats2[35][27] + coeff0_36*dmats2[36][27] + coeff0_37*dmats2[37][27] + coeff0_38*dmats2[38][27] + coeff0_39*dmats2[39][27] + coeff0_40*dmats2[40][27] + coeff0_41*dmats2[41][27] + coeff0_42*dmats2[42][27] + coeff0_43*dmats2[43][27] + coeff0_44*dmats2[44][27] + coeff0_45*dmats2[45][27] + coeff0_46*dmats2[46][27] + coeff0_47*dmats2[47][27] + coeff0_48*dmats2[48][27] + coeff0_49*dmats2[49][27] + coeff0_50*dmats2[50][27] + coeff0_51*dmats2[51][27] + coeff0_52*dmats2[52][27] + coeff0_53*dmats2[53][27] + coeff0_54*dmats2[54][27] + coeff0_55*dmats2[55][27];
          new_coeff0_28 = coeff0_0*dmats2[0][28] + coeff0_1*dmats2[1][28] + coeff0_2*dmats2[2][28] + coeff0_3*dmats2[3][28] + coeff0_4*dmats2[4][28] + coeff0_5*dmats2[5][28] + coeff0_6*dmats2[6][28] + coeff0_7*dmats2[7][28] + coeff0_8*dmats2[8][28] + coeff0_9*dmats2[9][28] + coeff0_10*dmats2[10][28] + coeff0_11*dmats2[11][28] + coeff0_12*dmats2[12][28] + coeff0_13*dmats2[13][28] + coeff0_14*dmats2[14][28] + coeff0_15*dmats2[15][28] + coeff0_16*dmats2[16][28] + coeff0_17*dmats2[17][28] + coeff0_18*dmats2[18][28] + coeff0_19*dmats2[19][28] + coeff0_20*dmats2[20][28] + coeff0_21*dmats2[21][28] + coeff0_22*dmats2[22][28] + coeff0_23*dmats2[23][28] + coeff0_24*dmats2[24][28] + coeff0_25*dmats2[25][28] + coeff0_26*dmats2[26][28] + coeff0_27*dmats2[27][28] + coeff0_28*dmats2[28][28] + coeff0_29*dmats2[29][28] + coeff0_30*dmats2[30][28] + coeff0_31*dmats2[31][28] + coeff0_32*dmats2[32][28] + coeff0_33*dmats2[33][28] + coeff0_34*dmats2[34][28] + coeff0_35*dmats2[35][28] + coeff0_36*dmats2[36][28] + coeff0_37*dmats2[37][28] + coeff0_38*dmats2[38][28] + coeff0_39*dmats2[39][28] + coeff0_40*dmats2[40][28] + coeff0_41*dmats2[41][28] + coeff0_42*dmats2[42][28] + coeff0_43*dmats2[43][28] + coeff0_44*dmats2[44][28] + coeff0_45*dmats2[45][28] + coeff0_46*dmats2[46][28] + coeff0_47*dmats2[47][28] + coeff0_48*dmats2[48][28] + coeff0_49*dmats2[49][28] + coeff0_50*dmats2[50][28] + coeff0_51*dmats2[51][28] + coeff0_52*dmats2[52][28] + coeff0_53*dmats2[53][28] + coeff0_54*dmats2[54][28] + coeff0_55*dmats2[55][28];
          new_coeff0_29 = coeff0_0*dmats2[0][29] + coeff0_1*dmats2[1][29] + coeff0_2*dmats2[2][29] + coeff0_3*dmats2[3][29] + coeff0_4*dmats2[4][29] + coeff0_5*dmats2[5][29] + coeff0_6*dmats2[6][29] + coeff0_7*dmats2[7][29] + coeff0_8*dmats2[8][29] + coeff0_9*dmats2[9][29] + coeff0_10*dmats2[10][29] + coeff0_11*dmats2[11][29] + coeff0_12*dmats2[12][29] + coeff0_13*dmats2[13][29] + coeff0_14*dmats2[14][29] + coeff0_15*dmats2[15][29] + coeff0_16*dmats2[16][29] + coeff0_17*dmats2[17][29] + coeff0_18*dmats2[18][29] + coeff0_19*dmats2[19][29] + coeff0_20*dmats2[20][29] + coeff0_21*dmats2[21][29] + coeff0_22*dmats2[22][29] + coeff0_23*dmats2[23][29] + coeff0_24*dmats2[24][29] + coeff0_25*dmats2[25][29] + coeff0_26*dmats2[26][29] + coeff0_27*dmats2[27][29] + coeff0_28*dmats2[28][29] + coeff0_29*dmats2[29][29] + coeff0_30*dmats2[30][29] + coeff0_31*dmats2[31][29] + coeff0_32*dmats2[32][29] + coeff0_33*dmats2[33][29] + coeff0_34*dmats2[34][29] + coeff0_35*dmats2[35][29] + coeff0_36*dmats2[36][29] + coeff0_37*dmats2[37][29] + coeff0_38*dmats2[38][29] + coeff0_39*dmats2[39][29] + coeff0_40*dmats2[40][29] + coeff0_41*dmats2[41][29] + coeff0_42*dmats2[42][29] + coeff0_43*dmats2[43][29] + coeff0_44*dmats2[44][29] + coeff0_45*dmats2[45][29] + coeff0_46*dmats2[46][29] + coeff0_47*dmats2[47][29] + coeff0_48*dmats2[48][29] + coeff0_49*dmats2[49][29] + coeff0_50*dmats2[50][29] + coeff0_51*dmats2[51][29] + coeff0_52*dmats2[52][29] + coeff0_53*dmats2[53][29] + coeff0_54*dmats2[54][29] + coeff0_55*dmats2[55][29];
          new_coeff0_30 = coeff0_0*dmats2[0][30] + coeff0_1*dmats2[1][30] + coeff0_2*dmats2[2][30] + coeff0_3*dmats2[3][30] + coeff0_4*dmats2[4][30] + coeff0_5*dmats2[5][30] + coeff0_6*dmats2[6][30] + coeff0_7*dmats2[7][30] + coeff0_8*dmats2[8][30] + coeff0_9*dmats2[9][30] + coeff0_10*dmats2[10][30] + coeff0_11*dmats2[11][30] + coeff0_12*dmats2[12][30] + coeff0_13*dmats2[13][30] + coeff0_14*dmats2[14][30] + coeff0_15*dmats2[15][30] + coeff0_16*dmats2[16][30] + coeff0_17*dmats2[17][30] + coeff0_18*dmats2[18][30] + coeff0_19*dmats2[19][30] + coeff0_20*dmats2[20][30] + coeff0_21*dmats2[21][30] + coeff0_22*dmats2[22][30] + coeff0_23*dmats2[23][30] + coeff0_24*dmats2[24][30] + coeff0_25*dmats2[25][30] + coeff0_26*dmats2[26][30] + coeff0_27*dmats2[27][30] + coeff0_28*dmats2[28][30] + coeff0_29*dmats2[29][30] + coeff0_30*dmats2[30][30] + coeff0_31*dmats2[31][30] + coeff0_32*dmats2[32][30] + coeff0_33*dmats2[33][30] + coeff0_34*dmats2[34][30] + coeff0_35*dmats2[35][30] + coeff0_36*dmats2[36][30] + coeff0_37*dmats2[37][30] + coeff0_38*dmats2[38][30] + coeff0_39*dmats2[39][30] + coeff0_40*dmats2[40][30] + coeff0_41*dmats2[41][30] + coeff0_42*dmats2[42][30] + coeff0_43*dmats2[43][30] + coeff0_44*dmats2[44][30] + coeff0_45*dmats2[45][30] + coeff0_46*dmats2[46][30] + coeff0_47*dmats2[47][30] + coeff0_48*dmats2[48][30] + coeff0_49*dmats2[49][30] + coeff0_50*dmats2[50][30] + coeff0_51*dmats2[51][30] + coeff0_52*dmats2[52][30] + coeff0_53*dmats2[53][30] + coeff0_54*dmats2[54][30] + coeff0_55*dmats2[55][30];
          new_coeff0_31 = coeff0_0*dmats2[0][31] + coeff0_1*dmats2[1][31] + coeff0_2*dmats2[2][31] + coeff0_3*dmats2[3][31] + coeff0_4*dmats2[4][31] + coeff0_5*dmats2[5][31] + coeff0_6*dmats2[6][31] + coeff0_7*dmats2[7][31] + coeff0_8*dmats2[8][31] + coeff0_9*dmats2[9][31] + coeff0_10*dmats2[10][31] + coeff0_11*dmats2[11][31] + coeff0_12*dmats2[12][31] + coeff0_13*dmats2[13][31] + coeff0_14*dmats2[14][31] + coeff0_15*dmats2[15][31] + coeff0_16*dmats2[16][31] + coeff0_17*dmats2[17][31] + coeff0_18*dmats2[18][31] + coeff0_19*dmats2[19][31] + coeff0_20*dmats2[20][31] + coeff0_21*dmats2[21][31] + coeff0_22*dmats2[22][31] + coeff0_23*dmats2[23][31] + coeff0_24*dmats2[24][31] + coeff0_25*dmats2[25][31] + coeff0_26*dmats2[26][31] + coeff0_27*dmats2[27][31] + coeff0_28*dmats2[28][31] + coeff0_29*dmats2[29][31] + coeff0_30*dmats2[30][31] + coeff0_31*dmats2[31][31] + coeff0_32*dmats2[32][31] + coeff0_33*dmats2[33][31] + coeff0_34*dmats2[34][31] + coeff0_35*dmats2[35][31] + coeff0_36*dmats2[36][31] + coeff0_37*dmats2[37][31] + coeff0_38*dmats2[38][31] + coeff0_39*dmats2[39][31] + coeff0_40*dmats2[40][31] + coeff0_41*dmats2[41][31] + coeff0_42*dmats2[42][31] + coeff0_43*dmats2[43][31] + coeff0_44*dmats2[44][31] + coeff0_45*dmats2[45][31] + coeff0_46*dmats2[46][31] + coeff0_47*dmats2[47][31] + coeff0_48*dmats2[48][31] + coeff0_49*dmats2[49][31] + coeff0_50*dmats2[50][31] + coeff0_51*dmats2[51][31] + coeff0_52*dmats2[52][31] + coeff0_53*dmats2[53][31] + coeff0_54*dmats2[54][31] + coeff0_55*dmats2[55][31];
          new_coeff0_32 = coeff0_0*dmats2[0][32] + coeff0_1*dmats2[1][32] + coeff0_2*dmats2[2][32] + coeff0_3*dmats2[3][32] + coeff0_4*dmats2[4][32] + coeff0_5*dmats2[5][32] + coeff0_6*dmats2[6][32] + coeff0_7*dmats2[7][32] + coeff0_8*dmats2[8][32] + coeff0_9*dmats2[9][32] + coeff0_10*dmats2[10][32] + coeff0_11*dmats2[11][32] + coeff0_12*dmats2[12][32] + coeff0_13*dmats2[13][32] + coeff0_14*dmats2[14][32] + coeff0_15*dmats2[15][32] + coeff0_16*dmats2[16][32] + coeff0_17*dmats2[17][32] + coeff0_18*dmats2[18][32] + coeff0_19*dmats2[19][32] + coeff0_20*dmats2[20][32] + coeff0_21*dmats2[21][32] + coeff0_22*dmats2[22][32] + coeff0_23*dmats2[23][32] + coeff0_24*dmats2[24][32] + coeff0_25*dmats2[25][32] + coeff0_26*dmats2[26][32] + coeff0_27*dmats2[27][32] + coeff0_28*dmats2[28][32] + coeff0_29*dmats2[29][32] + coeff0_30*dmats2[30][32] + coeff0_31*dmats2[31][32] + coeff0_32*dmats2[32][32] + coeff0_33*dmats2[33][32] + coeff0_34*dmats2[34][32] + coeff0_35*dmats2[35][32] + coeff0_36*dmats2[36][32] + coeff0_37*dmats2[37][32] + coeff0_38*dmats2[38][32] + coeff0_39*dmats2[39][32] + coeff0_40*dmats2[40][32] + coeff0_41*dmats2[41][32] + coeff0_42*dmats2[42][32] + coeff0_43*dmats2[43][32] + coeff0_44*dmats2[44][32] + coeff0_45*dmats2[45][32] + coeff0_46*dmats2[46][32] + coeff0_47*dmats2[47][32] + coeff0_48*dmats2[48][32] + coeff0_49*dmats2[49][32] + coeff0_50*dmats2[50][32] + coeff0_51*dmats2[51][32] + coeff0_52*dmats2[52][32] + coeff0_53*dmats2[53][32] + coeff0_54*dmats2[54][32] + coeff0_55*dmats2[55][32];
          new_coeff0_33 = coeff0_0*dmats2[0][33] + coeff0_1*dmats2[1][33] + coeff0_2*dmats2[2][33] + coeff0_3*dmats2[3][33] + coeff0_4*dmats2[4][33] + coeff0_5*dmats2[5][33] + coeff0_6*dmats2[6][33] + coeff0_7*dmats2[7][33] + coeff0_8*dmats2[8][33] + coeff0_9*dmats2[9][33] + coeff0_10*dmats2[10][33] + coeff0_11*dmats2[11][33] + coeff0_12*dmats2[12][33] + coeff0_13*dmats2[13][33] + coeff0_14*dmats2[14][33] + coeff0_15*dmats2[15][33] + coeff0_16*dmats2[16][33] + coeff0_17*dmats2[17][33] + coeff0_18*dmats2[18][33] + coeff0_19*dmats2[19][33] + coeff0_20*dmats2[20][33] + coeff0_21*dmats2[21][33] + coeff0_22*dmats2[22][33] + coeff0_23*dmats2[23][33] + coeff0_24*dmats2[24][33] + coeff0_25*dmats2[25][33] + coeff0_26*dmats2[26][33] + coeff0_27*dmats2[27][33] + coeff0_28*dmats2[28][33] + coeff0_29*dmats2[29][33] + coeff0_30*dmats2[30][33] + coeff0_31*dmats2[31][33] + coeff0_32*dmats2[32][33] + coeff0_33*dmats2[33][33] + coeff0_34*dmats2[34][33] + coeff0_35*dmats2[35][33] + coeff0_36*dmats2[36][33] + coeff0_37*dmats2[37][33] + coeff0_38*dmats2[38][33] + coeff0_39*dmats2[39][33] + coeff0_40*dmats2[40][33] + coeff0_41*dmats2[41][33] + coeff0_42*dmats2[42][33] + coeff0_43*dmats2[43][33] + coeff0_44*dmats2[44][33] + coeff0_45*dmats2[45][33] + coeff0_46*dmats2[46][33] + coeff0_47*dmats2[47][33] + coeff0_48*dmats2[48][33] + coeff0_49*dmats2[49][33] + coeff0_50*dmats2[50][33] + coeff0_51*dmats2[51][33] + coeff0_52*dmats2[52][33] + coeff0_53*dmats2[53][33] + coeff0_54*dmats2[54][33] + coeff0_55*dmats2[55][33];
          new_coeff0_34 = coeff0_0*dmats2[0][34] + coeff0_1*dmats2[1][34] + coeff0_2*dmats2[2][34] + coeff0_3*dmats2[3][34] + coeff0_4*dmats2[4][34] + coeff0_5*dmats2[5][34] + coeff0_6*dmats2[6][34] + coeff0_7*dmats2[7][34] + coeff0_8*dmats2[8][34] + coeff0_9*dmats2[9][34] + coeff0_10*dmats2[10][34] + coeff0_11*dmats2[11][34] + coeff0_12*dmats2[12][34] + coeff0_13*dmats2[13][34] + coeff0_14*dmats2[14][34] + coeff0_15*dmats2[15][34] + coeff0_16*dmats2[16][34] + coeff0_17*dmats2[17][34] + coeff0_18*dmats2[18][34] + coeff0_19*dmats2[19][34] + coeff0_20*dmats2[20][34] + coeff0_21*dmats2[21][34] + coeff0_22*dmats2[22][34] + coeff0_23*dmats2[23][34] + coeff0_24*dmats2[24][34] + coeff0_25*dmats2[25][34] + coeff0_26*dmats2[26][34] + coeff0_27*dmats2[27][34] + coeff0_28*dmats2[28][34] + coeff0_29*dmats2[29][34] + coeff0_30*dmats2[30][34] + coeff0_31*dmats2[31][34] + coeff0_32*dmats2[32][34] + coeff0_33*dmats2[33][34] + coeff0_34*dmats2[34][34] + coeff0_35*dmats2[35][34] + coeff0_36*dmats2[36][34] + coeff0_37*dmats2[37][34] + coeff0_38*dmats2[38][34] + coeff0_39*dmats2[39][34] + coeff0_40*dmats2[40][34] + coeff0_41*dmats2[41][34] + coeff0_42*dmats2[42][34] + coeff0_43*dmats2[43][34] + coeff0_44*dmats2[44][34] + coeff0_45*dmats2[45][34] + coeff0_46*dmats2[46][34] + coeff0_47*dmats2[47][34] + coeff0_48*dmats2[48][34] + coeff0_49*dmats2[49][34] + coeff0_50*dmats2[50][34] + coeff0_51*dmats2[51][34] + coeff0_52*dmats2[52][34] + coeff0_53*dmats2[53][34] + coeff0_54*dmats2[54][34] + coeff0_55*dmats2[55][34];
          new_coeff0_35 = coeff0_0*dmats2[0][35] + coeff0_1*dmats2[1][35] + coeff0_2*dmats2[2][35] + coeff0_3*dmats2[3][35] + coeff0_4*dmats2[4][35] + coeff0_5*dmats2[5][35] + coeff0_6*dmats2[6][35] + coeff0_7*dmats2[7][35] + coeff0_8*dmats2[8][35] + coeff0_9*dmats2[9][35] + coeff0_10*dmats2[10][35] + coeff0_11*dmats2[11][35] + coeff0_12*dmats2[12][35] + coeff0_13*dmats2[13][35] + coeff0_14*dmats2[14][35] + coeff0_15*dmats2[15][35] + coeff0_16*dmats2[16][35] + coeff0_17*dmats2[17][35] + coeff0_18*dmats2[18][35] + coeff0_19*dmats2[19][35] + coeff0_20*dmats2[20][35] + coeff0_21*dmats2[21][35] + coeff0_22*dmats2[22][35] + coeff0_23*dmats2[23][35] + coeff0_24*dmats2[24][35] + coeff0_25*dmats2[25][35] + coeff0_26*dmats2[26][35] + coeff0_27*dmats2[27][35] + coeff0_28*dmats2[28][35] + coeff0_29*dmats2[29][35] + coeff0_30*dmats2[30][35] + coeff0_31*dmats2[31][35] + coeff0_32*dmats2[32][35] + coeff0_33*dmats2[33][35] + coeff0_34*dmats2[34][35] + coeff0_35*dmats2[35][35] + coeff0_36*dmats2[36][35] + coeff0_37*dmats2[37][35] + coeff0_38*dmats2[38][35] + coeff0_39*dmats2[39][35] + coeff0_40*dmats2[40][35] + coeff0_41*dmats2[41][35] + coeff0_42*dmats2[42][35] + coeff0_43*dmats2[43][35] + coeff0_44*dmats2[44][35] + coeff0_45*dmats2[45][35] + coeff0_46*dmats2[46][35] + coeff0_47*dmats2[47][35] + coeff0_48*dmats2[48][35] + coeff0_49*dmats2[49][35] + coeff0_50*dmats2[50][35] + coeff0_51*dmats2[51][35] + coeff0_52*dmats2[52][35] + coeff0_53*dmats2[53][35] + coeff0_54*dmats2[54][35] + coeff0_55*dmats2[55][35];
          new_coeff0_36 = coeff0_0*dmats2[0][36] + coeff0_1*dmats2[1][36] + coeff0_2*dmats2[2][36] + coeff0_3*dmats2[3][36] + coeff0_4*dmats2[4][36] + coeff0_5*dmats2[5][36] + coeff0_6*dmats2[6][36] + coeff0_7*dmats2[7][36] + coeff0_8*dmats2[8][36] + coeff0_9*dmats2[9][36] + coeff0_10*dmats2[10][36] + coeff0_11*dmats2[11][36] + coeff0_12*dmats2[12][36] + coeff0_13*dmats2[13][36] + coeff0_14*dmats2[14][36] + coeff0_15*dmats2[15][36] + coeff0_16*dmats2[16][36] + coeff0_17*dmats2[17][36] + coeff0_18*dmats2[18][36] + coeff0_19*dmats2[19][36] + coeff0_20*dmats2[20][36] + coeff0_21*dmats2[21][36] + coeff0_22*dmats2[22][36] + coeff0_23*dmats2[23][36] + coeff0_24*dmats2[24][36] + coeff0_25*dmats2[25][36] + coeff0_26*dmats2[26][36] + coeff0_27*dmats2[27][36] + coeff0_28*dmats2[28][36] + coeff0_29*dmats2[29][36] + coeff0_30*dmats2[30][36] + coeff0_31*dmats2[31][36] + coeff0_32*dmats2[32][36] + coeff0_33*dmats2[33][36] + coeff0_34*dmats2[34][36] + coeff0_35*dmats2[35][36] + coeff0_36*dmats2[36][36] + coeff0_37*dmats2[37][36] + coeff0_38*dmats2[38][36] + coeff0_39*dmats2[39][36] + coeff0_40*dmats2[40][36] + coeff0_41*dmats2[41][36] + coeff0_42*dmats2[42][36] + coeff0_43*dmats2[43][36] + coeff0_44*dmats2[44][36] + coeff0_45*dmats2[45][36] + coeff0_46*dmats2[46][36] + coeff0_47*dmats2[47][36] + coeff0_48*dmats2[48][36] + coeff0_49*dmats2[49][36] + coeff0_50*dmats2[50][36] + coeff0_51*dmats2[51][36] + coeff0_52*dmats2[52][36] + coeff0_53*dmats2[53][36] + coeff0_54*dmats2[54][36] + coeff0_55*dmats2[55][36];
          new_coeff0_37 = coeff0_0*dmats2[0][37] + coeff0_1*dmats2[1][37] + coeff0_2*dmats2[2][37] + coeff0_3*dmats2[3][37] + coeff0_4*dmats2[4][37] + coeff0_5*dmats2[5][37] + coeff0_6*dmats2[6][37] + coeff0_7*dmats2[7][37] + coeff0_8*dmats2[8][37] + coeff0_9*dmats2[9][37] + coeff0_10*dmats2[10][37] + coeff0_11*dmats2[11][37] + coeff0_12*dmats2[12][37] + coeff0_13*dmats2[13][37] + coeff0_14*dmats2[14][37] + coeff0_15*dmats2[15][37] + coeff0_16*dmats2[16][37] + coeff0_17*dmats2[17][37] + coeff0_18*dmats2[18][37] + coeff0_19*dmats2[19][37] + coeff0_20*dmats2[20][37] + coeff0_21*dmats2[21][37] + coeff0_22*dmats2[22][37] + coeff0_23*dmats2[23][37] + coeff0_24*dmats2[24][37] + coeff0_25*dmats2[25][37] + coeff0_26*dmats2[26][37] + coeff0_27*dmats2[27][37] + coeff0_28*dmats2[28][37] + coeff0_29*dmats2[29][37] + coeff0_30*dmats2[30][37] + coeff0_31*dmats2[31][37] + coeff0_32*dmats2[32][37] + coeff0_33*dmats2[33][37] + coeff0_34*dmats2[34][37] + coeff0_35*dmats2[35][37] + coeff0_36*dmats2[36][37] + coeff0_37*dmats2[37][37] + coeff0_38*dmats2[38][37] + coeff0_39*dmats2[39][37] + coeff0_40*dmats2[40][37] + coeff0_41*dmats2[41][37] + coeff0_42*dmats2[42][37] + coeff0_43*dmats2[43][37] + coeff0_44*dmats2[44][37] + coeff0_45*dmats2[45][37] + coeff0_46*dmats2[46][37] + coeff0_47*dmats2[47][37] + coeff0_48*dmats2[48][37] + coeff0_49*dmats2[49][37] + coeff0_50*dmats2[50][37] + coeff0_51*dmats2[51][37] + coeff0_52*dmats2[52][37] + coeff0_53*dmats2[53][37] + coeff0_54*dmats2[54][37] + coeff0_55*dmats2[55][37];
          new_coeff0_38 = coeff0_0*dmats2[0][38] + coeff0_1*dmats2[1][38] + coeff0_2*dmats2[2][38] + coeff0_3*dmats2[3][38] + coeff0_4*dmats2[4][38] + coeff0_5*dmats2[5][38] + coeff0_6*dmats2[6][38] + coeff0_7*dmats2[7][38] + coeff0_8*dmats2[8][38] + coeff0_9*dmats2[9][38] + coeff0_10*dmats2[10][38] + coeff0_11*dmats2[11][38] + coeff0_12*dmats2[12][38] + coeff0_13*dmats2[13][38] + coeff0_14*dmats2[14][38] + coeff0_15*dmats2[15][38] + coeff0_16*dmats2[16][38] + coeff0_17*dmats2[17][38] + coeff0_18*dmats2[18][38] + coeff0_19*dmats2[19][38] + coeff0_20*dmats2[20][38] + coeff0_21*dmats2[21][38] + coeff0_22*dmats2[22][38] + coeff0_23*dmats2[23][38] + coeff0_24*dmats2[24][38] + coeff0_25*dmats2[25][38] + coeff0_26*dmats2[26][38] + coeff0_27*dmats2[27][38] + coeff0_28*dmats2[28][38] + coeff0_29*dmats2[29][38] + coeff0_30*dmats2[30][38] + coeff0_31*dmats2[31][38] + coeff0_32*dmats2[32][38] + coeff0_33*dmats2[33][38] + coeff0_34*dmats2[34][38] + coeff0_35*dmats2[35][38] + coeff0_36*dmats2[36][38] + coeff0_37*dmats2[37][38] + coeff0_38*dmats2[38][38] + coeff0_39*dmats2[39][38] + coeff0_40*dmats2[40][38] + coeff0_41*dmats2[41][38] + coeff0_42*dmats2[42][38] + coeff0_43*dmats2[43][38] + coeff0_44*dmats2[44][38] + coeff0_45*dmats2[45][38] + coeff0_46*dmats2[46][38] + coeff0_47*dmats2[47][38] + coeff0_48*dmats2[48][38] + coeff0_49*dmats2[49][38] + coeff0_50*dmats2[50][38] + coeff0_51*dmats2[51][38] + coeff0_52*dmats2[52][38] + coeff0_53*dmats2[53][38] + coeff0_54*dmats2[54][38] + coeff0_55*dmats2[55][38];
          new_coeff0_39 = coeff0_0*dmats2[0][39] + coeff0_1*dmats2[1][39] + coeff0_2*dmats2[2][39] + coeff0_3*dmats2[3][39] + coeff0_4*dmats2[4][39] + coeff0_5*dmats2[5][39] + coeff0_6*dmats2[6][39] + coeff0_7*dmats2[7][39] + coeff0_8*dmats2[8][39] + coeff0_9*dmats2[9][39] + coeff0_10*dmats2[10][39] + coeff0_11*dmats2[11][39] + coeff0_12*dmats2[12][39] + coeff0_13*dmats2[13][39] + coeff0_14*dmats2[14][39] + coeff0_15*dmats2[15][39] + coeff0_16*dmats2[16][39] + coeff0_17*dmats2[17][39] + coeff0_18*dmats2[18][39] + coeff0_19*dmats2[19][39] + coeff0_20*dmats2[20][39] + coeff0_21*dmats2[21][39] + coeff0_22*dmats2[22][39] + coeff0_23*dmats2[23][39] + coeff0_24*dmats2[24][39] + coeff0_25*dmats2[25][39] + coeff0_26*dmats2[26][39] + coeff0_27*dmats2[27][39] + coeff0_28*dmats2[28][39] + coeff0_29*dmats2[29][39] + coeff0_30*dmats2[30][39] + coeff0_31*dmats2[31][39] + coeff0_32*dmats2[32][39] + coeff0_33*dmats2[33][39] + coeff0_34*dmats2[34][39] + coeff0_35*dmats2[35][39] + coeff0_36*dmats2[36][39] + coeff0_37*dmats2[37][39] + coeff0_38*dmats2[38][39] + coeff0_39*dmats2[39][39] + coeff0_40*dmats2[40][39] + coeff0_41*dmats2[41][39] + coeff0_42*dmats2[42][39] + coeff0_43*dmats2[43][39] + coeff0_44*dmats2[44][39] + coeff0_45*dmats2[45][39] + coeff0_46*dmats2[46][39] + coeff0_47*dmats2[47][39] + coeff0_48*dmats2[48][39] + coeff0_49*dmats2[49][39] + coeff0_50*dmats2[50][39] + coeff0_51*dmats2[51][39] + coeff0_52*dmats2[52][39] + coeff0_53*dmats2[53][39] + coeff0_54*dmats2[54][39] + coeff0_55*dmats2[55][39];
          new_coeff0_40 = coeff0_0*dmats2[0][40] + coeff0_1*dmats2[1][40] + coeff0_2*dmats2[2][40] + coeff0_3*dmats2[3][40] + coeff0_4*dmats2[4][40] + coeff0_5*dmats2[5][40] + coeff0_6*dmats2[6][40] + coeff0_7*dmats2[7][40] + coeff0_8*dmats2[8][40] + coeff0_9*dmats2[9][40] + coeff0_10*dmats2[10][40] + coeff0_11*dmats2[11][40] + coeff0_12*dmats2[12][40] + coeff0_13*dmats2[13][40] + coeff0_14*dmats2[14][40] + coeff0_15*dmats2[15][40] + coeff0_16*dmats2[16][40] + coeff0_17*dmats2[17][40] + coeff0_18*dmats2[18][40] + coeff0_19*dmats2[19][40] + coeff0_20*dmats2[20][40] + coeff0_21*dmats2[21][40] + coeff0_22*dmats2[22][40] + coeff0_23*dmats2[23][40] + coeff0_24*dmats2[24][40] + coeff0_25*dmats2[25][40] + coeff0_26*dmats2[26][40] + coeff0_27*dmats2[27][40] + coeff0_28*dmats2[28][40] + coeff0_29*dmats2[29][40] + coeff0_30*dmats2[30][40] + coeff0_31*dmats2[31][40] + coeff0_32*dmats2[32][40] + coeff0_33*dmats2[33][40] + coeff0_34*dmats2[34][40] + coeff0_35*dmats2[35][40] + coeff0_36*dmats2[36][40] + coeff0_37*dmats2[37][40] + coeff0_38*dmats2[38][40] + coeff0_39*dmats2[39][40] + coeff0_40*dmats2[40][40] + coeff0_41*dmats2[41][40] + coeff0_42*dmats2[42][40] + coeff0_43*dmats2[43][40] + coeff0_44*dmats2[44][40] + coeff0_45*dmats2[45][40] + coeff0_46*dmats2[46][40] + coeff0_47*dmats2[47][40] + coeff0_48*dmats2[48][40] + coeff0_49*dmats2[49][40] + coeff0_50*dmats2[50][40] + coeff0_51*dmats2[51][40] + coeff0_52*dmats2[52][40] + coeff0_53*dmats2[53][40] + coeff0_54*dmats2[54][40] + coeff0_55*dmats2[55][40];
          new_coeff0_41 = coeff0_0*dmats2[0][41] + coeff0_1*dmats2[1][41] + coeff0_2*dmats2[2][41] + coeff0_3*dmats2[3][41] + coeff0_4*dmats2[4][41] + coeff0_5*dmats2[5][41] + coeff0_6*dmats2[6][41] + coeff0_7*dmats2[7][41] + coeff0_8*dmats2[8][41] + coeff0_9*dmats2[9][41] + coeff0_10*dmats2[10][41] + coeff0_11*dmats2[11][41] + coeff0_12*dmats2[12][41] + coeff0_13*dmats2[13][41] + coeff0_14*dmats2[14][41] + coeff0_15*dmats2[15][41] + coeff0_16*dmats2[16][41] + coeff0_17*dmats2[17][41] + coeff0_18*dmats2[18][41] + coeff0_19*dmats2[19][41] + coeff0_20*dmats2[20][41] + coeff0_21*dmats2[21][41] + coeff0_22*dmats2[22][41] + coeff0_23*dmats2[23][41] + coeff0_24*dmats2[24][41] + coeff0_25*dmats2[25][41] + coeff0_26*dmats2[26][41] + coeff0_27*dmats2[27][41] + coeff0_28*dmats2[28][41] + coeff0_29*dmats2[29][41] + coeff0_30*dmats2[30][41] + coeff0_31*dmats2[31][41] + coeff0_32*dmats2[32][41] + coeff0_33*dmats2[33][41] + coeff0_34*dmats2[34][41] + coeff0_35*dmats2[35][41] + coeff0_36*dmats2[36][41] + coeff0_37*dmats2[37][41] + coeff0_38*dmats2[38][41] + coeff0_39*dmats2[39][41] + coeff0_40*dmats2[40][41] + coeff0_41*dmats2[41][41] + coeff0_42*dmats2[42][41] + coeff0_43*dmats2[43][41] + coeff0_44*dmats2[44][41] + coeff0_45*dmats2[45][41] + coeff0_46*dmats2[46][41] + coeff0_47*dmats2[47][41] + coeff0_48*dmats2[48][41] + coeff0_49*dmats2[49][41] + coeff0_50*dmats2[50][41] + coeff0_51*dmats2[51][41] + coeff0_52*dmats2[52][41] + coeff0_53*dmats2[53][41] + coeff0_54*dmats2[54][41] + coeff0_55*dmats2[55][41];
          new_coeff0_42 = coeff0_0*dmats2[0][42] + coeff0_1*dmats2[1][42] + coeff0_2*dmats2[2][42] + coeff0_3*dmats2[3][42] + coeff0_4*dmats2[4][42] + coeff0_5*dmats2[5][42] + coeff0_6*dmats2[6][42] + coeff0_7*dmats2[7][42] + coeff0_8*dmats2[8][42] + coeff0_9*dmats2[9][42] + coeff0_10*dmats2[10][42] + coeff0_11*dmats2[11][42] + coeff0_12*dmats2[12][42] + coeff0_13*dmats2[13][42] + coeff0_14*dmats2[14][42] + coeff0_15*dmats2[15][42] + coeff0_16*dmats2[16][42] + coeff0_17*dmats2[17][42] + coeff0_18*dmats2[18][42] + coeff0_19*dmats2[19][42] + coeff0_20*dmats2[20][42] + coeff0_21*dmats2[21][42] + coeff0_22*dmats2[22][42] + coeff0_23*dmats2[23][42] + coeff0_24*dmats2[24][42] + coeff0_25*dmats2[25][42] + coeff0_26*dmats2[26][42] + coeff0_27*dmats2[27][42] + coeff0_28*dmats2[28][42] + coeff0_29*dmats2[29][42] + coeff0_30*dmats2[30][42] + coeff0_31*dmats2[31][42] + coeff0_32*dmats2[32][42] + coeff0_33*dmats2[33][42] + coeff0_34*dmats2[34][42] + coeff0_35*dmats2[35][42] + coeff0_36*dmats2[36][42] + coeff0_37*dmats2[37][42] + coeff0_38*dmats2[38][42] + coeff0_39*dmats2[39][42] + coeff0_40*dmats2[40][42] + coeff0_41*dmats2[41][42] + coeff0_42*dmats2[42][42] + coeff0_43*dmats2[43][42] + coeff0_44*dmats2[44][42] + coeff0_45*dmats2[45][42] + coeff0_46*dmats2[46][42] + coeff0_47*dmats2[47][42] + coeff0_48*dmats2[48][42] + coeff0_49*dmats2[49][42] + coeff0_50*dmats2[50][42] + coeff0_51*dmats2[51][42] + coeff0_52*dmats2[52][42] + coeff0_53*dmats2[53][42] + coeff0_54*dmats2[54][42] + coeff0_55*dmats2[55][42];
          new_coeff0_43 = coeff0_0*dmats2[0][43] + coeff0_1*dmats2[1][43] + coeff0_2*dmats2[2][43] + coeff0_3*dmats2[3][43] + coeff0_4*dmats2[4][43] + coeff0_5*dmats2[5][43] + coeff0_6*dmats2[6][43] + coeff0_7*dmats2[7][43] + coeff0_8*dmats2[8][43] + coeff0_9*dmats2[9][43] + coeff0_10*dmats2[10][43] + coeff0_11*dmats2[11][43] + coeff0_12*dmats2[12][43] + coeff0_13*dmats2[13][43] + coeff0_14*dmats2[14][43] + coeff0_15*dmats2[15][43] + coeff0_16*dmats2[16][43] + coeff0_17*dmats2[17][43] + coeff0_18*dmats2[18][43] + coeff0_19*dmats2[19][43] + coeff0_20*dmats2[20][43] + coeff0_21*dmats2[21][43] + coeff0_22*dmats2[22][43] + coeff0_23*dmats2[23][43] + coeff0_24*dmats2[24][43] + coeff0_25*dmats2[25][43] + coeff0_26*dmats2[26][43] + coeff0_27*dmats2[27][43] + coeff0_28*dmats2[28][43] + coeff0_29*dmats2[29][43] + coeff0_30*dmats2[30][43] + coeff0_31*dmats2[31][43] + coeff0_32*dmats2[32][43] + coeff0_33*dmats2[33][43] + coeff0_34*dmats2[34][43] + coeff0_35*dmats2[35][43] + coeff0_36*dmats2[36][43] + coeff0_37*dmats2[37][43] + coeff0_38*dmats2[38][43] + coeff0_39*dmats2[39][43] + coeff0_40*dmats2[40][43] + coeff0_41*dmats2[41][43] + coeff0_42*dmats2[42][43] + coeff0_43*dmats2[43][43] + coeff0_44*dmats2[44][43] + coeff0_45*dmats2[45][43] + coeff0_46*dmats2[46][43] + coeff0_47*dmats2[47][43] + coeff0_48*dmats2[48][43] + coeff0_49*dmats2[49][43] + coeff0_50*dmats2[50][43] + coeff0_51*dmats2[51][43] + coeff0_52*dmats2[52][43] + coeff0_53*dmats2[53][43] + coeff0_54*dmats2[54][43] + coeff0_55*dmats2[55][43];
          new_coeff0_44 = coeff0_0*dmats2[0][44] + coeff0_1*dmats2[1][44] + coeff0_2*dmats2[2][44] + coeff0_3*dmats2[3][44] + coeff0_4*dmats2[4][44] + coeff0_5*dmats2[5][44] + coeff0_6*dmats2[6][44] + coeff0_7*dmats2[7][44] + coeff0_8*dmats2[8][44] + coeff0_9*dmats2[9][44] + coeff0_10*dmats2[10][44] + coeff0_11*dmats2[11][44] + coeff0_12*dmats2[12][44] + coeff0_13*dmats2[13][44] + coeff0_14*dmats2[14][44] + coeff0_15*dmats2[15][44] + coeff0_16*dmats2[16][44] + coeff0_17*dmats2[17][44] + coeff0_18*dmats2[18][44] + coeff0_19*dmats2[19][44] + coeff0_20*dmats2[20][44] + coeff0_21*dmats2[21][44] + coeff0_22*dmats2[22][44] + coeff0_23*dmats2[23][44] + coeff0_24*dmats2[24][44] + coeff0_25*dmats2[25][44] + coeff0_26*dmats2[26][44] + coeff0_27*dmats2[27][44] + coeff0_28*dmats2[28][44] + coeff0_29*dmats2[29][44] + coeff0_30*dmats2[30][44] + coeff0_31*dmats2[31][44] + coeff0_32*dmats2[32][44] + coeff0_33*dmats2[33][44] + coeff0_34*dmats2[34][44] + coeff0_35*dmats2[35][44] + coeff0_36*dmats2[36][44] + coeff0_37*dmats2[37][44] + coeff0_38*dmats2[38][44] + coeff0_39*dmats2[39][44] + coeff0_40*dmats2[40][44] + coeff0_41*dmats2[41][44] + coeff0_42*dmats2[42][44] + coeff0_43*dmats2[43][44] + coeff0_44*dmats2[44][44] + coeff0_45*dmats2[45][44] + coeff0_46*dmats2[46][44] + coeff0_47*dmats2[47][44] + coeff0_48*dmats2[48][44] + coeff0_49*dmats2[49][44] + coeff0_50*dmats2[50][44] + coeff0_51*dmats2[51][44] + coeff0_52*dmats2[52][44] + coeff0_53*dmats2[53][44] + coeff0_54*dmats2[54][44] + coeff0_55*dmats2[55][44];
          new_coeff0_45 = coeff0_0*dmats2[0][45] + coeff0_1*dmats2[1][45] + coeff0_2*dmats2[2][45] + coeff0_3*dmats2[3][45] + coeff0_4*dmats2[4][45] + coeff0_5*dmats2[5][45] + coeff0_6*dmats2[6][45] + coeff0_7*dmats2[7][45] + coeff0_8*dmats2[8][45] + coeff0_9*dmats2[9][45] + coeff0_10*dmats2[10][45] + coeff0_11*dmats2[11][45] + coeff0_12*dmats2[12][45] + coeff0_13*dmats2[13][45] + coeff0_14*dmats2[14][45] + coeff0_15*dmats2[15][45] + coeff0_16*dmats2[16][45] + coeff0_17*dmats2[17][45] + coeff0_18*dmats2[18][45] + coeff0_19*dmats2[19][45] + coeff0_20*dmats2[20][45] + coeff0_21*dmats2[21][45] + coeff0_22*dmats2[22][45] + coeff0_23*dmats2[23][45] + coeff0_24*dmats2[24][45] + coeff0_25*dmats2[25][45] + coeff0_26*dmats2[26][45] + coeff0_27*dmats2[27][45] + coeff0_28*dmats2[28][45] + coeff0_29*dmats2[29][45] + coeff0_30*dmats2[30][45] + coeff0_31*dmats2[31][45] + coeff0_32*dmats2[32][45] + coeff0_33*dmats2[33][45] + coeff0_34*dmats2[34][45] + coeff0_35*dmats2[35][45] + coeff0_36*dmats2[36][45] + coeff0_37*dmats2[37][45] + coeff0_38*dmats2[38][45] + coeff0_39*dmats2[39][45] + coeff0_40*dmats2[40][45] + coeff0_41*dmats2[41][45] + coeff0_42*dmats2[42][45] + coeff0_43*dmats2[43][45] + coeff0_44*dmats2[44][45] + coeff0_45*dmats2[45][45] + coeff0_46*dmats2[46][45] + coeff0_47*dmats2[47][45] + coeff0_48*dmats2[48][45] + coeff0_49*dmats2[49][45] + coeff0_50*dmats2[50][45] + coeff0_51*dmats2[51][45] + coeff0_52*dmats2[52][45] + coeff0_53*dmats2[53][45] + coeff0_54*dmats2[54][45] + coeff0_55*dmats2[55][45];
          new_coeff0_46 = coeff0_0*dmats2[0][46] + coeff0_1*dmats2[1][46] + coeff0_2*dmats2[2][46] + coeff0_3*dmats2[3][46] + coeff0_4*dmats2[4][46] + coeff0_5*dmats2[5][46] + coeff0_6*dmats2[6][46] + coeff0_7*dmats2[7][46] + coeff0_8*dmats2[8][46] + coeff0_9*dmats2[9][46] + coeff0_10*dmats2[10][46] + coeff0_11*dmats2[11][46] + coeff0_12*dmats2[12][46] + coeff0_13*dmats2[13][46] + coeff0_14*dmats2[14][46] + coeff0_15*dmats2[15][46] + coeff0_16*dmats2[16][46] + coeff0_17*dmats2[17][46] + coeff0_18*dmats2[18][46] + coeff0_19*dmats2[19][46] + coeff0_20*dmats2[20][46] + coeff0_21*dmats2[21][46] + coeff0_22*dmats2[22][46] + coeff0_23*dmats2[23][46] + coeff0_24*dmats2[24][46] + coeff0_25*dmats2[25][46] + coeff0_26*dmats2[26][46] + coeff0_27*dmats2[27][46] + coeff0_28*dmats2[28][46] + coeff0_29*dmats2[29][46] + coeff0_30*dmats2[30][46] + coeff0_31*dmats2[31][46] + coeff0_32*dmats2[32][46] + coeff0_33*dmats2[33][46] + coeff0_34*dmats2[34][46] + coeff0_35*dmats2[35][46] + coeff0_36*dmats2[36][46] + coeff0_37*dmats2[37][46] + coeff0_38*dmats2[38][46] + coeff0_39*dmats2[39][46] + coeff0_40*dmats2[40][46] + coeff0_41*dmats2[41][46] + coeff0_42*dmats2[42][46] + coeff0_43*dmats2[43][46] + coeff0_44*dmats2[44][46] + coeff0_45*dmats2[45][46] + coeff0_46*dmats2[46][46] + coeff0_47*dmats2[47][46] + coeff0_48*dmats2[48][46] + coeff0_49*dmats2[49][46] + coeff0_50*dmats2[50][46] + coeff0_51*dmats2[51][46] + coeff0_52*dmats2[52][46] + coeff0_53*dmats2[53][46] + coeff0_54*dmats2[54][46] + coeff0_55*dmats2[55][46];
          new_coeff0_47 = coeff0_0*dmats2[0][47] + coeff0_1*dmats2[1][47] + coeff0_2*dmats2[2][47] + coeff0_3*dmats2[3][47] + coeff0_4*dmats2[4][47] + coeff0_5*dmats2[5][47] + coeff0_6*dmats2[6][47] + coeff0_7*dmats2[7][47] + coeff0_8*dmats2[8][47] + coeff0_9*dmats2[9][47] + coeff0_10*dmats2[10][47] + coeff0_11*dmats2[11][47] + coeff0_12*dmats2[12][47] + coeff0_13*dmats2[13][47] + coeff0_14*dmats2[14][47] + coeff0_15*dmats2[15][47] + coeff0_16*dmats2[16][47] + coeff0_17*dmats2[17][47] + coeff0_18*dmats2[18][47] + coeff0_19*dmats2[19][47] + coeff0_20*dmats2[20][47] + coeff0_21*dmats2[21][47] + coeff0_22*dmats2[22][47] + coeff0_23*dmats2[23][47] + coeff0_24*dmats2[24][47] + coeff0_25*dmats2[25][47] + coeff0_26*dmats2[26][47] + coeff0_27*dmats2[27][47] + coeff0_28*dmats2[28][47] + coeff0_29*dmats2[29][47] + coeff0_30*dmats2[30][47] + coeff0_31*dmats2[31][47] + coeff0_32*dmats2[32][47] + coeff0_33*dmats2[33][47] + coeff0_34*dmats2[34][47] + coeff0_35*dmats2[35][47] + coeff0_36*dmats2[36][47] + coeff0_37*dmats2[37][47] + coeff0_38*dmats2[38][47] + coeff0_39*dmats2[39][47] + coeff0_40*dmats2[40][47] + coeff0_41*dmats2[41][47] + coeff0_42*dmats2[42][47] + coeff0_43*dmats2[43][47] + coeff0_44*dmats2[44][47] + coeff0_45*dmats2[45][47] + coeff0_46*dmats2[46][47] + coeff0_47*dmats2[47][47] + coeff0_48*dmats2[48][47] + coeff0_49*dmats2[49][47] + coeff0_50*dmats2[50][47] + coeff0_51*dmats2[51][47] + coeff0_52*dmats2[52][47] + coeff0_53*dmats2[53][47] + coeff0_54*dmats2[54][47] + coeff0_55*dmats2[55][47];
          new_coeff0_48 = coeff0_0*dmats2[0][48] + coeff0_1*dmats2[1][48] + coeff0_2*dmats2[2][48] + coeff0_3*dmats2[3][48] + coeff0_4*dmats2[4][48] + coeff0_5*dmats2[5][48] + coeff0_6*dmats2[6][48] + coeff0_7*dmats2[7][48] + coeff0_8*dmats2[8][48] + coeff0_9*dmats2[9][48] + coeff0_10*dmats2[10][48] + coeff0_11*dmats2[11][48] + coeff0_12*dmats2[12][48] + coeff0_13*dmats2[13][48] + coeff0_14*dmats2[14][48] + coeff0_15*dmats2[15][48] + coeff0_16*dmats2[16][48] + coeff0_17*dmats2[17][48] + coeff0_18*dmats2[18][48] + coeff0_19*dmats2[19][48] + coeff0_20*dmats2[20][48] + coeff0_21*dmats2[21][48] + coeff0_22*dmats2[22][48] + coeff0_23*dmats2[23][48] + coeff0_24*dmats2[24][48] + coeff0_25*dmats2[25][48] + coeff0_26*dmats2[26][48] + coeff0_27*dmats2[27][48] + coeff0_28*dmats2[28][48] + coeff0_29*dmats2[29][48] + coeff0_30*dmats2[30][48] + coeff0_31*dmats2[31][48] + coeff0_32*dmats2[32][48] + coeff0_33*dmats2[33][48] + coeff0_34*dmats2[34][48] + coeff0_35*dmats2[35][48] + coeff0_36*dmats2[36][48] + coeff0_37*dmats2[37][48] + coeff0_38*dmats2[38][48] + coeff0_39*dmats2[39][48] + coeff0_40*dmats2[40][48] + coeff0_41*dmats2[41][48] + coeff0_42*dmats2[42][48] + coeff0_43*dmats2[43][48] + coeff0_44*dmats2[44][48] + coeff0_45*dmats2[45][48] + coeff0_46*dmats2[46][48] + coeff0_47*dmats2[47][48] + coeff0_48*dmats2[48][48] + coeff0_49*dmats2[49][48] + coeff0_50*dmats2[50][48] + coeff0_51*dmats2[51][48] + coeff0_52*dmats2[52][48] + coeff0_53*dmats2[53][48] + coeff0_54*dmats2[54][48] + coeff0_55*dmats2[55][48];
          new_coeff0_49 = coeff0_0*dmats2[0][49] + coeff0_1*dmats2[1][49] + coeff0_2*dmats2[2][49] + coeff0_3*dmats2[3][49] + coeff0_4*dmats2[4][49] + coeff0_5*dmats2[5][49] + coeff0_6*dmats2[6][49] + coeff0_7*dmats2[7][49] + coeff0_8*dmats2[8][49] + coeff0_9*dmats2[9][49] + coeff0_10*dmats2[10][49] + coeff0_11*dmats2[11][49] + coeff0_12*dmats2[12][49] + coeff0_13*dmats2[13][49] + coeff0_14*dmats2[14][49] + coeff0_15*dmats2[15][49] + coeff0_16*dmats2[16][49] + coeff0_17*dmats2[17][49] + coeff0_18*dmats2[18][49] + coeff0_19*dmats2[19][49] + coeff0_20*dmats2[20][49] + coeff0_21*dmats2[21][49] + coeff0_22*dmats2[22][49] + coeff0_23*dmats2[23][49] + coeff0_24*dmats2[24][49] + coeff0_25*dmats2[25][49] + coeff0_26*dmats2[26][49] + coeff0_27*dmats2[27][49] + coeff0_28*dmats2[28][49] + coeff0_29*dmats2[29][49] + coeff0_30*dmats2[30][49] + coeff0_31*dmats2[31][49] + coeff0_32*dmats2[32][49] + coeff0_33*dmats2[33][49] + coeff0_34*dmats2[34][49] + coeff0_35*dmats2[35][49] + coeff0_36*dmats2[36][49] + coeff0_37*dmats2[37][49] + coeff0_38*dmats2[38][49] + coeff0_39*dmats2[39][49] + coeff0_40*dmats2[40][49] + coeff0_41*dmats2[41][49] + coeff0_42*dmats2[42][49] + coeff0_43*dmats2[43][49] + coeff0_44*dmats2[44][49] + coeff0_45*dmats2[45][49] + coeff0_46*dmats2[46][49] + coeff0_47*dmats2[47][49] + coeff0_48*dmats2[48][49] + coeff0_49*dmats2[49][49] + coeff0_50*dmats2[50][49] + coeff0_51*dmats2[51][49] + coeff0_52*dmats2[52][49] + coeff0_53*dmats2[53][49] + coeff0_54*dmats2[54][49] + coeff0_55*dmats2[55][49];
          new_coeff0_50 = coeff0_0*dmats2[0][50] + coeff0_1*dmats2[1][50] + coeff0_2*dmats2[2][50] + coeff0_3*dmats2[3][50] + coeff0_4*dmats2[4][50] + coeff0_5*dmats2[5][50] + coeff0_6*dmats2[6][50] + coeff0_7*dmats2[7][50] + coeff0_8*dmats2[8][50] + coeff0_9*dmats2[9][50] + coeff0_10*dmats2[10][50] + coeff0_11*dmats2[11][50] + coeff0_12*dmats2[12][50] + coeff0_13*dmats2[13][50] + coeff0_14*dmats2[14][50] + coeff0_15*dmats2[15][50] + coeff0_16*dmats2[16][50] + coeff0_17*dmats2[17][50] + coeff0_18*dmats2[18][50] + coeff0_19*dmats2[19][50] + coeff0_20*dmats2[20][50] + coeff0_21*dmats2[21][50] + coeff0_22*dmats2[22][50] + coeff0_23*dmats2[23][50] + coeff0_24*dmats2[24][50] + coeff0_25*dmats2[25][50] + coeff0_26*dmats2[26][50] + coeff0_27*dmats2[27][50] + coeff0_28*dmats2[28][50] + coeff0_29*dmats2[29][50] + coeff0_30*dmats2[30][50] + coeff0_31*dmats2[31][50] + coeff0_32*dmats2[32][50] + coeff0_33*dmats2[33][50] + coeff0_34*dmats2[34][50] + coeff0_35*dmats2[35][50] + coeff0_36*dmats2[36][50] + coeff0_37*dmats2[37][50] + coeff0_38*dmats2[38][50] + coeff0_39*dmats2[39][50] + coeff0_40*dmats2[40][50] + coeff0_41*dmats2[41][50] + coeff0_42*dmats2[42][50] + coeff0_43*dmats2[43][50] + coeff0_44*dmats2[44][50] + coeff0_45*dmats2[45][50] + coeff0_46*dmats2[46][50] + coeff0_47*dmats2[47][50] + coeff0_48*dmats2[48][50] + coeff0_49*dmats2[49][50] + coeff0_50*dmats2[50][50] + coeff0_51*dmats2[51][50] + coeff0_52*dmats2[52][50] + coeff0_53*dmats2[53][50] + coeff0_54*dmats2[54][50] + coeff0_55*dmats2[55][50];
          new_coeff0_51 = coeff0_0*dmats2[0][51] + coeff0_1*dmats2[1][51] + coeff0_2*dmats2[2][51] + coeff0_3*dmats2[3][51] + coeff0_4*dmats2[4][51] + coeff0_5*dmats2[5][51] + coeff0_6*dmats2[6][51] + coeff0_7*dmats2[7][51] + coeff0_8*dmats2[8][51] + coeff0_9*dmats2[9][51] + coeff0_10*dmats2[10][51] + coeff0_11*dmats2[11][51] + coeff0_12*dmats2[12][51] + coeff0_13*dmats2[13][51] + coeff0_14*dmats2[14][51] + coeff0_15*dmats2[15][51] + coeff0_16*dmats2[16][51] + coeff0_17*dmats2[17][51] + coeff0_18*dmats2[18][51] + coeff0_19*dmats2[19][51] + coeff0_20*dmats2[20][51] + coeff0_21*dmats2[21][51] + coeff0_22*dmats2[22][51] + coeff0_23*dmats2[23][51] + coeff0_24*dmats2[24][51] + coeff0_25*dmats2[25][51] + coeff0_26*dmats2[26][51] + coeff0_27*dmats2[27][51] + coeff0_28*dmats2[28][51] + coeff0_29*dmats2[29][51] + coeff0_30*dmats2[30][51] + coeff0_31*dmats2[31][51] + coeff0_32*dmats2[32][51] + coeff0_33*dmats2[33][51] + coeff0_34*dmats2[34][51] + coeff0_35*dmats2[35][51] + coeff0_36*dmats2[36][51] + coeff0_37*dmats2[37][51] + coeff0_38*dmats2[38][51] + coeff0_39*dmats2[39][51] + coeff0_40*dmats2[40][51] + coeff0_41*dmats2[41][51] + coeff0_42*dmats2[42][51] + coeff0_43*dmats2[43][51] + coeff0_44*dmats2[44][51] + coeff0_45*dmats2[45][51] + coeff0_46*dmats2[46][51] + coeff0_47*dmats2[47][51] + coeff0_48*dmats2[48][51] + coeff0_49*dmats2[49][51] + coeff0_50*dmats2[50][51] + coeff0_51*dmats2[51][51] + coeff0_52*dmats2[52][51] + coeff0_53*dmats2[53][51] + coeff0_54*dmats2[54][51] + coeff0_55*dmats2[55][51];
          new_coeff0_52 = coeff0_0*dmats2[0][52] + coeff0_1*dmats2[1][52] + coeff0_2*dmats2[2][52] + coeff0_3*dmats2[3][52] + coeff0_4*dmats2[4][52] + coeff0_5*dmats2[5][52] + coeff0_6*dmats2[6][52] + coeff0_7*dmats2[7][52] + coeff0_8*dmats2[8][52] + coeff0_9*dmats2[9][52] + coeff0_10*dmats2[10][52] + coeff0_11*dmats2[11][52] + coeff0_12*dmats2[12][52] + coeff0_13*dmats2[13][52] + coeff0_14*dmats2[14][52] + coeff0_15*dmats2[15][52] + coeff0_16*dmats2[16][52] + coeff0_17*dmats2[17][52] + coeff0_18*dmats2[18][52] + coeff0_19*dmats2[19][52] + coeff0_20*dmats2[20][52] + coeff0_21*dmats2[21][52] + coeff0_22*dmats2[22][52] + coeff0_23*dmats2[23][52] + coeff0_24*dmats2[24][52] + coeff0_25*dmats2[25][52] + coeff0_26*dmats2[26][52] + coeff0_27*dmats2[27][52] + coeff0_28*dmats2[28][52] + coeff0_29*dmats2[29][52] + coeff0_30*dmats2[30][52] + coeff0_31*dmats2[31][52] + coeff0_32*dmats2[32][52] + coeff0_33*dmats2[33][52] + coeff0_34*dmats2[34][52] + coeff0_35*dmats2[35][52] + coeff0_36*dmats2[36][52] + coeff0_37*dmats2[37][52] + coeff0_38*dmats2[38][52] + coeff0_39*dmats2[39][52] + coeff0_40*dmats2[40][52] + coeff0_41*dmats2[41][52] + coeff0_42*dmats2[42][52] + coeff0_43*dmats2[43][52] + coeff0_44*dmats2[44][52] + coeff0_45*dmats2[45][52] + coeff0_46*dmats2[46][52] + coeff0_47*dmats2[47][52] + coeff0_48*dmats2[48][52] + coeff0_49*dmats2[49][52] + coeff0_50*dmats2[50][52] + coeff0_51*dmats2[51][52] + coeff0_52*dmats2[52][52] + coeff0_53*dmats2[53][52] + coeff0_54*dmats2[54][52] + coeff0_55*dmats2[55][52];
          new_coeff0_53 = coeff0_0*dmats2[0][53] + coeff0_1*dmats2[1][53] + coeff0_2*dmats2[2][53] + coeff0_3*dmats2[3][53] + coeff0_4*dmats2[4][53] + coeff0_5*dmats2[5][53] + coeff0_6*dmats2[6][53] + coeff0_7*dmats2[7][53] + coeff0_8*dmats2[8][53] + coeff0_9*dmats2[9][53] + coeff0_10*dmats2[10][53] + coeff0_11*dmats2[11][53] + coeff0_12*dmats2[12][53] + coeff0_13*dmats2[13][53] + coeff0_14*dmats2[14][53] + coeff0_15*dmats2[15][53] + coeff0_16*dmats2[16][53] + coeff0_17*dmats2[17][53] + coeff0_18*dmats2[18][53] + coeff0_19*dmats2[19][53] + coeff0_20*dmats2[20][53] + coeff0_21*dmats2[21][53] + coeff0_22*dmats2[22][53] + coeff0_23*dmats2[23][53] + coeff0_24*dmats2[24][53] + coeff0_25*dmats2[25][53] + coeff0_26*dmats2[26][53] + coeff0_27*dmats2[27][53] + coeff0_28*dmats2[28][53] + coeff0_29*dmats2[29][53] + coeff0_30*dmats2[30][53] + coeff0_31*dmats2[31][53] + coeff0_32*dmats2[32][53] + coeff0_33*dmats2[33][53] + coeff0_34*dmats2[34][53] + coeff0_35*dmats2[35][53] + coeff0_36*dmats2[36][53] + coeff0_37*dmats2[37][53] + coeff0_38*dmats2[38][53] + coeff0_39*dmats2[39][53] + coeff0_40*dmats2[40][53] + coeff0_41*dmats2[41][53] + coeff0_42*dmats2[42][53] + coeff0_43*dmats2[43][53] + coeff0_44*dmats2[44][53] + coeff0_45*dmats2[45][53] + coeff0_46*dmats2[46][53] + coeff0_47*dmats2[47][53] + coeff0_48*dmats2[48][53] + coeff0_49*dmats2[49][53] + coeff0_50*dmats2[50][53] + coeff0_51*dmats2[51][53] + coeff0_52*dmats2[52][53] + coeff0_53*dmats2[53][53] + coeff0_54*dmats2[54][53] + coeff0_55*dmats2[55][53];
          new_coeff0_54 = coeff0_0*dmats2[0][54] + coeff0_1*dmats2[1][54] + coeff0_2*dmats2[2][54] + coeff0_3*dmats2[3][54] + coeff0_4*dmats2[4][54] + coeff0_5*dmats2[5][54] + coeff0_6*dmats2[6][54] + coeff0_7*dmats2[7][54] + coeff0_8*dmats2[8][54] + coeff0_9*dmats2[9][54] + coeff0_10*dmats2[10][54] + coeff0_11*dmats2[11][54] + coeff0_12*dmats2[12][54] + coeff0_13*dmats2[13][54] + coeff0_14*dmats2[14][54] + coeff0_15*dmats2[15][54] + coeff0_16*dmats2[16][54] + coeff0_17*dmats2[17][54] + coeff0_18*dmats2[18][54] + coeff0_19*dmats2[19][54] + coeff0_20*dmats2[20][54] + coeff0_21*dmats2[21][54] + coeff0_22*dmats2[22][54] + coeff0_23*dmats2[23][54] + coeff0_24*dmats2[24][54] + coeff0_25*dmats2[25][54] + coeff0_26*dmats2[26][54] + coeff0_27*dmats2[27][54] + coeff0_28*dmats2[28][54] + coeff0_29*dmats2[29][54] + coeff0_30*dmats2[30][54] + coeff0_31*dmats2[31][54] + coeff0_32*dmats2[32][54] + coeff0_33*dmats2[33][54] + coeff0_34*dmats2[34][54] + coeff0_35*dmats2[35][54] + coeff0_36*dmats2[36][54] + coeff0_37*dmats2[37][54] + coeff0_38*dmats2[38][54] + coeff0_39*dmats2[39][54] + coeff0_40*dmats2[40][54] + coeff0_41*dmats2[41][54] + coeff0_42*dmats2[42][54] + coeff0_43*dmats2[43][54] + coeff0_44*dmats2[44][54] + coeff0_45*dmats2[45][54] + coeff0_46*dmats2[46][54] + coeff0_47*dmats2[47][54] + coeff0_48*dmats2[48][54] + coeff0_49*dmats2[49][54] + coeff0_50*dmats2[50][54] + coeff0_51*dmats2[51][54] + coeff0_52*dmats2[52][54] + coeff0_53*dmats2[53][54] + coeff0_54*dmats2[54][54] + coeff0_55*dmats2[55][54];
          new_coeff0_55 = coeff0_0*dmats2[0][55] + coeff0_1*dmats2[1][55] + coeff0_2*dmats2[2][55] + coeff0_3*dmats2[3][55] + coeff0_4*dmats2[4][55] + coeff0_5*dmats2[5][55] + coeff0_6*dmats2[6][55] + coeff0_7*dmats2[7][55] + coeff0_8*dmats2[8][55] + coeff0_9*dmats2[9][55] + coeff0_10*dmats2[10][55] + coeff0_11*dmats2[11][55] + coeff0_12*dmats2[12][55] + coeff0_13*dmats2[13][55] + coeff0_14*dmats2[14][55] + coeff0_15*dmats2[15][55] + coeff0_16*dmats2[16][55] + coeff0_17*dmats2[17][55] + coeff0_18*dmats2[18][55] + coeff0_19*dmats2[19][55] + coeff0_20*dmats2[20][55] + coeff0_21*dmats2[21][55] + coeff0_22*dmats2[22][55] + coeff0_23*dmats2[23][55] + coeff0_24*dmats2[24][55] + coeff0_25*dmats2[25][55] + coeff0_26*dmats2[26][55] + coeff0_27*dmats2[27][55] + coeff0_28*dmats2[28][55] + coeff0_29*dmats2[29][55] + coeff0_30*dmats2[30][55] + coeff0_31*dmats2[31][55] + coeff0_32*dmats2[32][55] + coeff0_33*dmats2[33][55] + coeff0_34*dmats2[34][55] + coeff0_35*dmats2[35][55] + coeff0_36*dmats2[36][55] + coeff0_37*dmats2[37][55] + coeff0_38*dmats2[38][55] + coeff0_39*dmats2[39][55] + coeff0_40*dmats2[40][55] + coeff0_41*dmats2[41][55] + coeff0_42*dmats2[42][55] + coeff0_43*dmats2[43][55] + coeff0_44*dmats2[44][55] + coeff0_45*dmats2[45][55] + coeff0_46*dmats2[46][55] + coeff0_47*dmats2[47][55] + coeff0_48*dmats2[48][55] + coeff0_49*dmats2[49][55] + coeff0_50*dmats2[50][55] + coeff0_51*dmats2[51][55] + coeff0_52*dmats2[52][55] + coeff0_53*dmats2[53][55] + coeff0_54*dmats2[54][55] + coeff0_55*dmats2[55][55];
        }
    
      }
      // Compute derivatives on reference element as dot product of coefficients and basisvalues
      derivatives[deriv_num] = new_coeff0_0*basisvalue0 + new_coeff0_1*basisvalue1 + new_coeff0_2*basisvalue2 + new_coeff0_3*basisvalue3 + new_coeff0_4*basisvalue4 + new_coeff0_5*basisvalue5 + new_coeff0_6*basisvalue6 + new_coeff0_7*basisvalue7 + new_coeff0_8*basisvalue8 + new_coeff0_9*basisvalue9 + new_coeff0_10*basisvalue10 + new_coeff0_11*basisvalue11 + new_coeff0_12*basisvalue12 + new_coeff0_13*basisvalue13 + new_coeff0_14*basisvalue14 + new_coeff0_15*basisvalue15 + new_coeff0_16*basisvalue16 + new_coeff0_17*basisvalue17 + new_coeff0_18*basisvalue18 + new_coeff0_19*basisvalue19 + new_coeff0_20*basisvalue20 + new_coeff0_21*basisvalue21 + new_coeff0_22*basisvalue22 + new_coeff0_23*basisvalue23 + new_coeff0_24*basisvalue24 + new_coeff0_25*basisvalue25 + new_coeff0_26*basisvalue26 + new_coeff0_27*basisvalue27 + new_coeff0_28*basisvalue28 + new_coeff0_29*basisvalue29 + new_coeff0_30*basisvalue30 + new_coeff0_31*basisvalue31 + new_coeff0_32*basisvalue32 + new_coeff0_33*basisvalue33 + new_coeff0_34*basisvalue34 + new_coeff0_35*basisvalue35 + new_coeff0_36*basisvalue36 + new_coeff0_37*basisvalue37 + new_coeff0_38*basisvalue38 + new_coeff0_39*basisvalue39 + new_coeff0_40*basisvalue40 + new_coeff0_41*basisvalue41 + new_coeff0_42*basisvalue42 + new_coeff0_43*basisvalue43 + new_coeff0_44*basisvalue44 + new_coeff0_45*basisvalue45 + new_coeff0_46*basisvalue46 + new_coeff0_47*basisvalue47 + new_coeff0_48*basisvalue48 + new_coeff0_49*basisvalue49 + new_coeff0_50*basisvalue50 + new_coeff0_51*basisvalue51 + new_coeff0_52*basisvalue52 + new_coeff0_53*basisvalue53 + new_coeff0_54*basisvalue54 + new_coeff0_55*basisvalue55;
    }
    
    // Transform derivatives back to physical element
    for (unsigned int row = 0; row < num_derivatives; row++)
    {
      for (unsigned int col = 0; col < num_derivatives; col++)
      {
        values[row] += transform[row][col]*derivatives[col];
      }
    }
    // Delete pointer to array of derivatives on FIAT element
    delete [] derivatives;
    
    // Delete pointer to array of combinations of derivatives and transform
    for (unsigned int row = 0; row < num_derivatives; row++)
    {
      delete [] combinations[row];
      delete [] transform[row];
    }
    
    delete [] combinations;
    delete [] transform;
  }

  /// Evaluate order n derivatives of all basis functions at given point in cell
  virtual void evaluate_basis_derivatives_all(unsigned int n,
                                              double* values,
                                              const double* coordinates,
                                              const ufc::cell& c) const
  {
    throw std::runtime_error("The vectorised version of evaluate_basis_derivatives() is not yet implemented.");
  }

  /// Evaluate linear functional for dof i on the function f
  virtual double evaluate_dof(unsigned int i,
                              const ufc::function& f,
                              const ufc::cell& c) const
  {
    // The reference points, direction and weights:
    const static double X[56][1][3] = {{{0, 0, 0}}, {{1, 0, 0}}, {{0, 1, 0}}, {{0, 0, 1}}, {{0, 0.8, 0.2}}, {{0, 0.6, 0.4}}, {{0, 0.4, 0.6}}, {{0, 0.2, 0.8}}, {{0.8, 0, 0.2}}, {{0.6, 0, 0.4}}, {{0.4, 0, 0.6}}, {{0.2, 0, 0.8}}, {{0.8, 0.2, 0}}, {{0.6, 0.4, 0}}, {{0.4, 0.6, 0}}, {{0.2, 0.8, 0}}, {{0, 0, 0.2}}, {{0, 0, 0.4}}, {{0, 0, 0.6}}, {{0, 0, 0.8}}, {{0, 0.2, 0}}, {{0, 0.4, 0}}, {{0, 0.6, 0}}, {{0, 0.8, 0}}, {{0.2, 0, 0}}, {{0.4, 0, 0}}, {{0.6, 0, 0}}, {{0.8, 0, 0}}, {{0.6, 0.2, 0.2}}, {{0.4, 0.4, 0.2}}, {{0.2, 0.6, 0.2}}, {{0.4, 0.2, 0.4}}, {{0.2, 0.4, 0.4}}, {{0.2, 0.2, 0.6}}, {{0, 0.2, 0.2}}, {{0, 0.4, 0.2}}, {{0, 0.6, 0.2}}, {{0, 0.2, 0.4}}, {{0, 0.4, 0.4}}, {{0, 0.2, 0.6}}, {{0.2, 0, 0.2}}, {{0.4, 0, 0.2}}, {{0.6, 0, 0.2}}, {{0.2, 0, 0.4}}, {{0.4, 0, 0.4}}, {{0.2, 0, 0.6}}, {{0.2, 0.2, 0}}, {{0.4, 0.2, 0}}, {{0.6, 0.2, 0}}, {{0.2, 0.4, 0}}, {{0.4, 0.4, 0}}, {{0.2, 0.6, 0}}, {{0.2, 0.2, 0.2}}, {{0.4, 0.2, 0.2}}, {{0.2, 0.4, 0.2}}, {{0.2, 0.2, 0.4}}};
    const static double W[56][1] = {{1}, {1}, {1}, {1}, {1}, {1}, {1}, {1}, {1}, {1}, {1}, {1}, {1}, {1}, {1}, {1}, {1}, {1}, {1}, {1}, {1}, {1}, {1}, {1}, {1}, {1}, {1}, {1}, {1}, {1}, {1}, {1}, {1}, {1}, {1}, {1}, {1}, {1}, {1}, {1}, {1}, {1}, {1}, {1}, {1}, {1}, {1}, {1}, {1}, {1}, {1}, {1}, {1}, {1}, {1}, {1}};
    const static double D[56][1][1] = {{{1}}, {{1}}, {{1}}, {{1}}, {{1}}, {{1}}, {{1}}, {{1}}, {{1}}, {{1}}, {{1}}, {{1}}, {{1}}, {{1}}, {{1}}, {{1}}, {{1}}, {{1}}, {{1}}, {{1}}, {{1}}, {{1}}, {{1}}, {{1}}, {{1}}, {{1}}, {{1}}, {{1}}, {{1}}, {{1}}, {{1}}, {{1}}, {{1}}, {{1}}, {{1}}, {{1}}, {{1}}, {{1}}, {{1}}, {{1}}, {{1}}, {{1}}, {{1}}, {{1}}, {{1}}, {{1}}, {{1}}, {{1}}, {{1}}, {{1}}, {{1}}, {{1}}, {{1}}, {{1}}, {{1}}, {{1}}};
    
    const double * const * x = c.coordinates;
    double result = 0.0;
    // Iterate over the points:
    // Evaluate basis functions for affine mapping
    const double w0 = 1.0 - X[i][0][0] - X[i][0][1] - X[i][0][2];
    const double w1 = X[i][0][0];
    const double w2 = X[i][0][1];
    const double w3 = X[i][0][2];
    
    // Compute affine mapping y = F(X)
    double y[3];
    y[0] = w0*x[0][0] + w1*x[1][0] + w2*x[2][0] + w3*x[3][0];
    y[1] = w0*x[0][1] + w1*x[1][1] + w2*x[2][1] + w3*x[3][1];
    y[2] = w0*x[0][2] + w1*x[1][2] + w2*x[2][2] + w3*x[3][2];
    
    // Evaluate function at physical points
    double values[1];
    f.evaluate(values, y, c);
    
    // Map function values using appropriate mapping
    // Affine map: Do nothing
    
    // Note that we do not map the weights (yet).
    
    // Take directional components
    for(int k = 0; k < 1; k++)
      result += values[k]*D[i][0][k];
    // Multiply by weights 
    result *= W[i][0];
    
    return result;
  }

  /// Evaluate linear functionals for all dofs on the function f
  virtual void evaluate_dofs(double* values,
                             const ufc::function& f,
                             const ufc::cell& c) const
  {
    throw std::runtime_error("Not implemented (introduced in UFC v1.1).");
  }

  /// Interpolate vertex values from dof values
  virtual void interpolate_vertex_values(double* vertex_values,
                                         const double* dof_values,
                                         const ufc::cell& c) const
  {
    // Evaluate at vertices and use affine mapping
    vertex_values[0] = dof_values[0];
    vertex_values[1] = dof_values[1];
    vertex_values[2] = dof_values[2];
    vertex_values[3] = dof_values[3];
  }

  /// Return the number of sub elements (for a mixed element)
  virtual unsigned int num_sub_elements() const
  {
    return 1;
  }

  /// Create a new finite element for sub element i (for a mixed element)
  virtual ufc::finite_element* create_sub_element(unsigned int i) const
  {
    return new UFC_Poisson3D_5LinearForm_finite_element_0();
  }

};

/// This class defines the interface for a finite element.

class UFC_Poisson3D_5LinearForm_finite_element_1: public ufc::finite_element
{
public:

  /// Constructor
  UFC_Poisson3D_5LinearForm_finite_element_1() : ufc::finite_element()
  {
    // Do nothing
  }

  /// Destructor
  virtual ~UFC_Poisson3D_5LinearForm_finite_element_1()
  {
    // Do nothing
  }

  /// Return a string identifying the finite element
  virtual const char* signature() const
  {
    return "Lagrange finite element of degree 5 on a tetrahedron";
  }

  /// Return the cell shape
  virtual ufc::shape cell_shape() const
  {
    return ufc::tetrahedron;
  }

  /// Return the dimension of the finite element function space
  virtual unsigned int space_dimension() const
  {
    return 56;
  }

  /// Return the rank of the value space
  virtual unsigned int value_rank() const
  {
    return 0;
  }

  /// Return the dimension of the value space for axis i
  virtual unsigned int value_dimension(unsigned int i) const
  {
    return 1;
  }

  /// Evaluate basis function i at given point in cell
  virtual void evaluate_basis(unsigned int i,
                              double* values,
                              const double* coordinates,
                              const ufc::cell& c) const
  {
    // Extract vertex coordinates
    const double * const * element_coordinates = c.coordinates;
    
    // Compute Jacobian of affine map from reference cell
    const double J_00 = element_coordinates[1][0] - element_coordinates[0][0];
    const double J_01 = element_coordinates[2][0] - element_coordinates[0][0];
    const double J_02 = element_coordinates[3][0] - element_coordinates[0][0];
    const double J_10 = element_coordinates[1][1] - element_coordinates[0][1];
    const double J_11 = element_coordinates[2][1] - element_coordinates[0][1];
    const double J_12 = element_coordinates[3][1] - element_coordinates[0][1];
    const double J_20 = element_coordinates[1][2] - element_coordinates[0][2];
    const double J_21 = element_coordinates[2][2] - element_coordinates[0][2];
    const double J_22 = element_coordinates[3][2] - element_coordinates[0][2];
      
    // Compute sub determinants
    const double d00 = J_11*J_22 - J_12*J_21;
    const double d01 = J_12*J_20 - J_10*J_22;
    const double d02 = J_10*J_21 - J_11*J_20;
    
    const double d10 = J_02*J_21 - J_01*J_22;
    const double d11 = J_00*J_22 - J_02*J_20;
    const double d12 = J_01*J_20 - J_00*J_21;
    
    const double d20 = J_01*J_12 - J_02*J_11;
    const double d21 = J_02*J_10 - J_00*J_12;
    const double d22 = J_00*J_11 - J_01*J_10;
      
    // Compute determinant of Jacobian
    double detJ = J_00*d00 + J_10*d10 + J_20*d20;
    
    // Compute inverse of Jacobian
    
    // Compute constants
    const double C0 = d00*(element_coordinates[0][0] - element_coordinates[2][0] - element_coordinates[3][0]) \
                    + d10*(element_coordinates[0][1] - element_coordinates[2][1] - element_coordinates[3][1]) \
                    + d20*(element_coordinates[0][2] - element_coordinates[2][2] - element_coordinates[3][2]);
    
    const double C1 = d01*(element_coordinates[0][0] - element_coordinates[1][0] - element_coordinates[3][0]) \
                    + d11*(element_coordinates[0][1] - element_coordinates[1][1] - element_coordinates[3][1]) \
                    + d21*(element_coordinates[0][2] - element_coordinates[1][2] - element_coordinates[3][2]);
    
    const double C2 = d02*(element_coordinates[0][0] - element_coordinates[1][0] - element_coordinates[2][0]) \
                    + d12*(element_coordinates[0][1] - element_coordinates[1][1] - element_coordinates[2][1]) \
                    + d22*(element_coordinates[0][2] - element_coordinates[1][2] - element_coordinates[2][2]);
    
    // Get coordinates and map to the UFC reference element
    double x = (C0 + d00*coordinates[0] + d10*coordinates[1] + d20*coordinates[2]) / detJ;
    double y = (C1 + d01*coordinates[0] + d11*coordinates[1] + d21*coordinates[2]) / detJ;
    double z = (C2 + d02*coordinates[0] + d12*coordinates[1] + d22*coordinates[2]) / detJ;
    
    // Map coordinates to the reference cube
    if (std::abs(y + z - 1.0) < 1e-14)
      x = 1.0;
    else
      x = -2.0 * x/(y + z - 1.0) - 1.0;
    if (std::abs(z - 1.0) < 1e-14)
      y = -1.0;
    else
      y = 2.0 * y/(1.0 - z) - 1.0;
    z = 2.0 * z - 1.0;
    
    // Reset values
    *values = 0;
    
    // Map degree of freedom to element degree of freedom
    const unsigned int dof = i;
    
    // Generate scalings
    const double scalings_y_0 = 1;
    const double scalings_y_1 = scalings_y_0*(0.5 - 0.5*y);
    const double scalings_y_2 = scalings_y_1*(0.5 - 0.5*y);
    const double scalings_y_3 = scalings_y_2*(0.5 - 0.5*y);
    const double scalings_y_4 = scalings_y_3*(0.5 - 0.5*y);
    const double scalings_y_5 = scalings_y_4*(0.5 - 0.5*y);
    const double scalings_z_0 = 1;
    const double scalings_z_1 = scalings_z_0*(0.5 - 0.5*z);
    const double scalings_z_2 = scalings_z_1*(0.5 - 0.5*z);
    const double scalings_z_3 = scalings_z_2*(0.5 - 0.5*z);
    const double scalings_z_4 = scalings_z_3*(0.5 - 0.5*z);
    const double scalings_z_5 = scalings_z_4*(0.5 - 0.5*z);
    
    // Compute psitilde_a
    const double psitilde_a_0 = 1;
    const double psitilde_a_1 = x;
    const double psitilde_a_2 = 1.5*x*psitilde_a_1 - 0.5*psitilde_a_0;
    const double psitilde_a_3 = 1.66666666666667*x*psitilde_a_2 - 0.666666666666667*psitilde_a_1;
    const double psitilde_a_4 = 1.75*x*psitilde_a_3 - 0.75*psitilde_a_2;
    const double psitilde_a_5 = 1.8*x*psitilde_a_4 - 0.8*psitilde_a_3;
    
    // Compute psitilde_bs
    const double psitilde_bs_0_0 = 1;
    const double psitilde_bs_0_1 = 1.5*y + 0.5;
    const double psitilde_bs_0_2 = 0.111111111111111*psitilde_bs_0_1 + 1.66666666666667*y*psitilde_bs_0_1 - 0.555555555555556*psitilde_bs_0_0;
    const double psitilde_bs_0_3 = 0.05*psitilde_bs_0_2 + 1.75*y*psitilde_bs_0_2 - 0.7*psitilde_bs_0_1;
    const double psitilde_bs_0_4 = 0.0285714285714286*psitilde_bs_0_3 + 1.8*y*psitilde_bs_0_3 - 0.771428571428571*psitilde_bs_0_2;
    const double psitilde_bs_0_5 = 0.0185185185185185*psitilde_bs_0_4 + 1.83333333333333*y*psitilde_bs_0_4 - 0.814814814814815*psitilde_bs_0_3;
    const double psitilde_bs_1_0 = 1;
    const double psitilde_bs_1_1 = 2.5*y + 1.5;
    const double psitilde_bs_1_2 = 0.54*psitilde_bs_1_1 + 2.1*y*psitilde_bs_1_1 - 0.56*psitilde_bs_1_0;
    const double psitilde_bs_1_3 = 0.285714285714286*psitilde_bs_1_2 + 2*y*psitilde_bs_1_2 - 0.714285714285714*psitilde_bs_1_1;
    const double psitilde_bs_1_4 = 0.178571428571429*psitilde_bs_1_3 + 1.96428571428571*y*psitilde_bs_1_3 - 0.785714285714286*psitilde_bs_1_2;
    const double psitilde_bs_2_0 = 1;
    const double psitilde_bs_2_1 = 3.5*y + 2.5;
    const double psitilde_bs_2_2 = 1.02040816326531*psitilde_bs_2_1 + 2.57142857142857*y*psitilde_bs_2_1 - 0.551020408163265*psitilde_bs_2_0;
    const double psitilde_bs_2_3 = 0.578703703703704*psitilde_bs_2_2 + 2.29166666666667*y*psitilde_bs_2_2 - 0.712962962962963*psitilde_bs_2_1;
    const double psitilde_bs_3_0 = 1;
    const double psitilde_bs_3_1 = 4.5*y + 3.5;
    const double psitilde_bs_3_2 = 1.51234567901235*psitilde_bs_3_1 + 3.05555555555556*y*psitilde_bs_3_1 - 0.54320987654321*psitilde_bs_3_0;
    const double psitilde_bs_4_0 = 1;
    const double psitilde_bs_4_1 = 5.5*y + 4.5;
    const double psitilde_bs_5_0 = 1;
    
    // Compute psitilde_cs
    const double psitilde_cs_00_0 = 1;
    const double psitilde_cs_00_1 = 2*z + 1;
    const double psitilde_cs_00_2 = 0.3125*psitilde_cs_00_1 + 1.875*z*psitilde_cs_00_1 - 0.5625*psitilde_cs_00_0;
    const double psitilde_cs_00_3 = 0.155555555555556*psitilde_cs_00_2 + 1.86666666666667*z*psitilde_cs_00_2 - 0.711111111111111*psitilde_cs_00_1;
    const double psitilde_cs_00_4 = 0.09375*psitilde_cs_00_3 + 1.875*z*psitilde_cs_00_3 - 0.78125*psitilde_cs_00_2;
    const double psitilde_cs_00_5 = 0.0628571428571429*psitilde_cs_00_4 + 1.88571428571429*z*psitilde_cs_00_4 - 0.822857142857143*psitilde_cs_00_3;
    const double psitilde_cs_01_0 = 1;
    const double psitilde_cs_01_1 = 3*z + 2;
    const double psitilde_cs_01_2 = 0.777777777777778*psitilde_cs_01_1 + 2.33333333333333*z*psitilde_cs_01_1 - 0.555555555555556*psitilde_cs_01_0;
    const double psitilde_cs_01_3 = 0.428571428571429*psitilde_cs_01_2 + 2.14285714285714*z*psitilde_cs_01_2 - 0.714285714285714*psitilde_cs_01_1;
    const double psitilde_cs_01_4 = 0.275*psitilde_cs_01_3 + 2.0625*z*psitilde_cs_01_3 - 0.7875*psitilde_cs_01_2;
    const double psitilde_cs_02_0 = 1;
    const double psitilde_cs_02_1 = 4*z + 3;
    const double psitilde_cs_02_2 = 1.265625*psitilde_cs_02_1 + 2.8125*z*psitilde_cs_02_1 - 0.546875*psitilde_cs_02_0;
    const double psitilde_cs_02_3 = 0.733333333333333*psitilde_cs_02_2 + 2.44444444444444*z*psitilde_cs_02_2 - 0.711111111111111*psitilde_cs_02_1;
    const double psitilde_cs_03_0 = 1;
    const double psitilde_cs_03_1 = 5*z + 4;
    const double psitilde_cs_03_2 = 1.76*psitilde_cs_03_1 + 3.3*z*psitilde_cs_03_1 - 0.54*psitilde_cs_03_0;
    const double psitilde_cs_04_0 = 1;
    const double psitilde_cs_04_1 = 6*z + 5;
    const double psitilde_cs_05_0 = 1;
    const double psitilde_cs_10_0 = 1;
    const double psitilde_cs_10_1 = 3*z + 2;
    const double psitilde_cs_10_2 = 0.777777777777778*psitilde_cs_10_1 + 2.33333333333333*z*psitilde_cs_10_1 - 0.555555555555556*psitilde_cs_10_0;
    const double psitilde_cs_10_3 = 0.428571428571429*psitilde_cs_10_2 + 2.14285714285714*z*psitilde_cs_10_2 - 0.714285714285714*psitilde_cs_10_1;
    const double psitilde_cs_10_4 = 0.275*psitilde_cs_10_3 + 2.0625*z*psitilde_cs_10_3 - 0.7875*psitilde_cs_10_2;
    const double psitilde_cs_11_0 = 1;
    const double psitilde_cs_11_1 = 4*z + 3;
    const double psitilde_cs_11_2 = 1.265625*psitilde_cs_11_1 + 2.8125*z*psitilde_cs_11_1 - 0.546875*psitilde_cs_11_0;
    const double psitilde_cs_11_3 = 0.733333333333333*psitilde_cs_11_2 + 2.44444444444444*z*psitilde_cs_11_2 - 0.711111111111111*psitilde_cs_11_1;
    const double psitilde_cs_12_0 = 1;
    const double psitilde_cs_12_1 = 5*z + 4;
    const double psitilde_cs_12_2 = 1.76*psitilde_cs_12_1 + 3.3*z*psitilde_cs_12_1 - 0.54*psitilde_cs_12_0;
    const double psitilde_cs_13_0 = 1;
    const double psitilde_cs_13_1 = 6*z + 5;
    const double psitilde_cs_14_0 = 1;
    const double psitilde_cs_20_0 = 1;
    const double psitilde_cs_20_1 = 4*z + 3;
    const double psitilde_cs_20_2 = 1.265625*psitilde_cs_20_1 + 2.8125*z*psitilde_cs_20_1 - 0.546875*psitilde_cs_20_0;
    const double psitilde_cs_20_3 = 0.733333333333333*psitilde_cs_20_2 + 2.44444444444444*z*psitilde_cs_20_2 - 0.711111111111111*psitilde_cs_20_1;
    const double psitilde_cs_21_0 = 1;
    const double psitilde_cs_21_1 = 5*z + 4;
    const double psitilde_cs_21_2 = 1.76*psitilde_cs_21_1 + 3.3*z*psitilde_cs_21_1 - 0.54*psitilde_cs_21_0;
    const double psitilde_cs_22_0 = 1;
    const double psitilde_cs_22_1 = 6*z + 5;
    const double psitilde_cs_23_0 = 1;
    const double psitilde_cs_30_0 = 1;
    const double psitilde_cs_30_1 = 5*z + 4;
    const double psitilde_cs_30_2 = 1.76*psitilde_cs_30_1 + 3.3*z*psitilde_cs_30_1 - 0.54*psitilde_cs_30_0;
    const double psitilde_cs_31_0 = 1;
    const double psitilde_cs_31_1 = 6*z + 5;
    const double psitilde_cs_32_0 = 1;
    const double psitilde_cs_40_0 = 1;
    const double psitilde_cs_40_1 = 6*z + 5;
    const double psitilde_cs_41_0 = 1;
    const double psitilde_cs_50_0 = 1;
    
    // Compute basisvalues
    const double basisvalue0 = 0.866025403784439*psitilde_a_0*scalings_y_0*psitilde_bs_0_0*scalings_z_0*psitilde_cs_00_0;
    const double basisvalue1 = 2.73861278752583*psitilde_a_1*scalings_y_1*psitilde_bs_1_0*scalings_z_1*psitilde_cs_10_0;
    const double basisvalue2 = 1.58113883008419*psitilde_a_0*scalings_y_0*psitilde_bs_0_1*scalings_z_1*psitilde_cs_01_0;
    const double basisvalue3 = 1.11803398874989*psitilde_a_0*scalings_y_0*psitilde_bs_0_0*scalings_z_0*psitilde_cs_00_1;
    const double basisvalue4 = 5.1234753829798*psitilde_a_2*scalings_y_2*psitilde_bs_2_0*scalings_z_2*psitilde_cs_20_0;
    const double basisvalue5 = 3.96862696659689*psitilde_a_1*scalings_y_1*psitilde_bs_1_1*scalings_z_2*psitilde_cs_11_0;
    const double basisvalue6 = 2.29128784747792*psitilde_a_0*scalings_y_0*psitilde_bs_0_2*scalings_z_2*psitilde_cs_02_0;
    const double basisvalue7 = 3.24037034920393*psitilde_a_1*scalings_y_1*psitilde_bs_1_0*scalings_z_1*psitilde_cs_10_1;
    const double basisvalue8 = 1.87082869338697*psitilde_a_0*scalings_y_0*psitilde_bs_0_1*scalings_z_1*psitilde_cs_01_1;
    const double basisvalue9 = 1.3228756555323*psitilde_a_0*scalings_y_0*psitilde_bs_0_0*scalings_z_0*psitilde_cs_00_2;
    const double basisvalue10 = 7.93725393319377*psitilde_a_3*scalings_y_3*psitilde_bs_3_0*scalings_z_3*psitilde_cs_30_0;
    const double basisvalue11 = 6.70820393249937*psitilde_a_2*scalings_y_2*psitilde_bs_2_1*scalings_z_3*psitilde_cs_21_0;
    const double basisvalue12 = 5.19615242270663*psitilde_a_1*scalings_y_1*psitilde_bs_1_2*scalings_z_3*psitilde_cs_12_0;
    const double basisvalue13 = 3*psitilde_a_0*scalings_y_0*psitilde_bs_0_3*scalings_z_3*psitilde_cs_03_0;
    const double basisvalue14 = 5.80947501931113*psitilde_a_2*scalings_y_2*psitilde_bs_2_0*scalings_z_2*psitilde_cs_20_1;
    const double basisvalue15 = 4.5*psitilde_a_1*scalings_y_1*psitilde_bs_1_1*scalings_z_2*psitilde_cs_11_1;
    const double basisvalue16 = 2.59807621135332*psitilde_a_0*scalings_y_0*psitilde_bs_0_2*scalings_z_2*psitilde_cs_02_1;
    const double basisvalue17 = 3.67423461417477*psitilde_a_1*scalings_y_1*psitilde_bs_1_0*scalings_z_1*psitilde_cs_10_2;
    const double basisvalue18 = 2.12132034355964*psitilde_a_0*scalings_y_0*psitilde_bs_0_1*scalings_z_1*psitilde_cs_01_2;
    const double basisvalue19 = 1.5*psitilde_a_0*scalings_y_0*psitilde_bs_0_0*scalings_z_0*psitilde_cs_00_3;
    const double basisvalue20 = 11.1242977306435*psitilde_a_4*scalings_y_4*psitilde_bs_4_0*scalings_z_4*psitilde_cs_40_0;
    const double basisvalue21 = 9.81070843517429*psitilde_a_3*scalings_y_3*psitilde_bs_3_1*scalings_z_4*psitilde_cs_31_0;
    const double basisvalue22 = 8.2915619758885*psitilde_a_2*scalings_y_2*psitilde_bs_2_2*scalings_z_4*psitilde_cs_22_0;
    const double basisvalue23 = 6.42261628933256*psitilde_a_1*scalings_y_1*psitilde_bs_1_3*scalings_z_4*psitilde_cs_13_0;
    const double basisvalue24 = 3.70809924354783*psitilde_a_0*scalings_y_0*psitilde_bs_0_4*scalings_z_4*psitilde_cs_04_0;
    const double basisvalue25 = 8.77496438739212*psitilde_a_3*scalings_y_3*psitilde_bs_3_0*scalings_z_3*psitilde_cs_30_1;
    const double basisvalue26 = 7.41619848709566*psitilde_a_2*scalings_y_2*psitilde_bs_2_1*scalings_z_3*psitilde_cs_21_1;
    const double basisvalue27 = 5.74456264653803*psitilde_a_1*scalings_y_1*psitilde_bs_1_2*scalings_z_3*psitilde_cs_12_1;
    const double basisvalue28 = 3.3166247903554*psitilde_a_0*scalings_y_0*psitilde_bs_0_3*scalings_z_3*psitilde_cs_03_1;
    const double basisvalue29 = 6.42261628933256*psitilde_a_2*scalings_y_2*psitilde_bs_2_0*scalings_z_2*psitilde_cs_20_2;
    const double basisvalue30 = 4.9749371855331*psitilde_a_1*scalings_y_1*psitilde_bs_1_1*scalings_z_2*psitilde_cs_11_2;
    const double basisvalue31 = 2.87228132326901*psitilde_a_0*scalings_y_0*psitilde_bs_0_2*scalings_z_2*psitilde_cs_02_2;
    const double basisvalue32 = 4.06201920231798*psitilde_a_1*scalings_y_1*psitilde_bs_1_0*scalings_z_1*psitilde_cs_10_3;
    const double basisvalue33 = 2.34520787991171*psitilde_a_0*scalings_y_0*psitilde_bs_0_1*scalings_z_1*psitilde_cs_01_3;
    const double basisvalue34 = 1.6583123951777*psitilde_a_0*scalings_y_0*psitilde_bs_0_0*scalings_z_0*psitilde_cs_00_4;
    const double basisvalue35 = 14.6458185158768*psitilde_a_5*scalings_y_5*psitilde_bs_5_0*scalings_z_5*psitilde_cs_50_0;
    const double basisvalue36 = 13.2476412994918*psitilde_a_4*scalings_y_4*psitilde_bs_4_1*scalings_z_5*psitilde_cs_41_0;
    const double basisvalue37 = 11.6833214455479*psitilde_a_3*scalings_y_3*psitilde_bs_3_2*scalings_z_5*psitilde_cs_32_0;
    const double basisvalue38 = 9.87420882906575*psitilde_a_2*scalings_y_2*psitilde_bs_2_3*scalings_z_5*psitilde_cs_23_0;
    const double basisvalue39 = 7.64852927038918*psitilde_a_1*scalings_y_1*psitilde_bs_1_4*scalings_z_5*psitilde_cs_14_0;
    const double basisvalue40 = 4.41588043316392*psitilde_a_0*scalings_y_0*psitilde_bs_0_5*scalings_z_5*psitilde_cs_05_0;
    const double basisvalue41 = 12.0933866224478*psitilde_a_4*scalings_y_4*psitilde_bs_4_0*scalings_z_4*psitilde_cs_40_1;
    const double basisvalue42 = 10.6653645038508*psitilde_a_3*scalings_y_3*psitilde_bs_3_1*scalings_z_4*psitilde_cs_31_1;
    const double basisvalue43 = 9.01387818865997*psitilde_a_2*scalings_y_2*psitilde_bs_2_2*scalings_z_4*psitilde_cs_22_1;
    const double basisvalue44 = 6.98212002188447*psitilde_a_1*scalings_y_1*psitilde_bs_1_3*scalings_z_4*psitilde_cs_13_1;
    const double basisvalue45 = 4.03112887414927*psitilde_a_0*scalings_y_0*psitilde_bs_0_4*scalings_z_4*psitilde_cs_04_1;
    const double basisvalue46 = 9.53939201416946*psitilde_a_3*scalings_y_3*psitilde_bs_3_0*scalings_z_3*psitilde_cs_30_2;
    const double basisvalue47 = 8.06225774829855*psitilde_a_2*scalings_y_2*psitilde_bs_2_1*scalings_z_3*psitilde_cs_21_2;
    const double basisvalue48 = 6.2449979983984*psitilde_a_1*scalings_y_1*psitilde_bs_1_2*scalings_z_3*psitilde_cs_12_2;
    const double basisvalue49 = 3.60555127546399*psitilde_a_0*scalings_y_0*psitilde_bs_0_3*scalings_z_3*psitilde_cs_03_2;
    const double basisvalue50 = 6.98212002188447*psitilde_a_2*scalings_y_2*psitilde_bs_2_0*scalings_z_2*psitilde_cs_20_3;
    const double basisvalue51 = 5.40832691319598*psitilde_a_1*scalings_y_1*psitilde_bs_1_1*scalings_z_2*psitilde_cs_11_3;
    const double basisvalue52 = 3.1224989991992*psitilde_a_0*scalings_y_0*psitilde_bs_0_2*scalings_z_2*psitilde_cs_02_3;
    const double basisvalue53 = 4.41588043316392*psitilde_a_1*scalings_y_1*psitilde_bs_1_0*scalings_z_1*psitilde_cs_10_4;
    const double basisvalue54 = 2.54950975679639*psitilde_a_0*scalings_y_0*psitilde_bs_0_1*scalings_z_1*psitilde_cs_01_4;
    const double basisvalue55 = 1.80277563773199*psitilde_a_0*scalings_y_0*psitilde_bs_0_0*scalings_z_0*psitilde_cs_00_5;
    
    // Table(s) of coefficients
    const static double coefficients0[56][56] = \
    {{0.00945067404923504, 0.00609788120679252, 0.003520613356228, 0.00248944957812473, 0.00705946233325038, 0.00546823600997147, 0.00315708753234945, 0.00446479600584757, 0.00257775117585287, 0.00182274533665719, -0.00298267418725718, -0.0025208197798293, -0.00195261860521383, -0.0011273448773449, -0.00218309396769446, -0.00169101731601731, -0.000976309302606924, -0.00138070985681772, -0.000797153207506492, -0.000563672438672437, 0.00278687112460006, 0.00245778931055954, 0.00207721109304565, 0.00160900079398463, 0.000928957041533348, 0.00219831358911339, 0.00185791408306671, 0.00143913406048031, 0.000830884437218262, 0.00160900079398463, 0.00124632665582739, 0.000719567030240153, 0.00101762145320216, 0.000587524019939396, 0.000415442218609115, -0.00705593524717797, -0.0063823335708908, -0.00562868913761196, -0.00475711057319694, -0.00368484200521179, -0.00212744452363026, -0.00582624677716564, -0.00513826668309063, -0.00434262794914383, -0.00336378514516178, -0.00194208225905521, -0.00459580543596519, -0.00388416451811043, -0.00300866089851429, -0.00173705117965752, -0.00336378514516179, -0.00260557676948628, -0.00150433044925714, -0.00212744452363026, -0.00122828066840392, -0.000868525589828755},
    {0.00945067404923484, -0.0060978812067926, 0.00352061335622807, 0.00248944957812473, 0.00705946233325039, -0.00546823600997146, 0.00315708753234942, -0.00446479600584756, 0.00257775117585286, 0.00182274533665715, 0.00298267418725716, -0.00252081977982931, 0.00195261860521383, -0.00112734487734489, -0.00218309396769446, 0.00169101731601731, -0.000976309302606908, 0.00138070985681772, -0.000797153207506487, -0.000563672438672423, 0.00278687112460003, -0.00245778931055954, 0.00207721109304565, -0.00160900079398463, 0.000928957041533354, -0.00219831358911339, 0.00185791408306669, -0.00143913406048031, 0.000830884437218255, 0.00160900079398463, -0.0012463266558274, 0.000719567030240159, -0.00101762145320216, 0.000587524019939409, 0.000415442218609113, 0.00705593524717798, -0.00638233357089081, 0.00562868913761196, -0.00475711057319694, 0.00368484200521178, -0.00212744452363027, -0.00582624677716566, 0.00513826668309062, -0.00434262794914383, 0.00336378514516178, -0.00194208225905522, 0.0045958054359652, -0.00388416451811043, 0.00300866089851429, -0.00173705117965753, -0.00336378514516178, 0.00260557676948629, -0.00150433044925715, 0.00212744452363026, -0.00122828066840393, -0.000868525589828756},
    {0.00945067404923496, 0, -0.00704122671245609, 0.00248944957812476, 0, 0, 0.00947126259704828, 0, -0.00515550235170569, 0.00182274533665715, 0, 0, 0, 0.0045093795093795, 0, 0, -0.00292892790782073, 0, 0.00159430641501296, -0.000563672438672431, 0, 0, 0, 0, 0.00464478520766673, 0, 0, 0, -0.00332353774887306, 0, 0, 0.00215870109072046, 0, -0.00117504803987882, 0.000415442218609132, 0, 0, 0, 0, 0, 0.0127646671417816, 0, 0, 0, 0, -0.00971041129527609, 0, 0, 0, 0.0069482047186301, 0, 0, -0.00451299134777144, 0, 0.00245656133680787, -0.000868525589828766},
    {0.00945067404923495, 0, 0, -0.00746834873437432, 0, 0, 0, 0, 0, 0.0109364720199429, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0.00563672438672439, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0.00623163327913698, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0.0182390373864041},
    {-0.0100234421734309, 0, 0.0374718748311398, -0.0289614183593723, 0, 0, 0.0284137877911449, 0, 0.00515550235170565, -0.00619733414463432, 0, 0, 0, 0.0315656565656566, 0, 0, -0.0113902751970807, 0, 0.00265717735835495, -0.000187890812890785, 0, 0, 0, 0, 0.0232239260383336, 0, 0, 0, -0.00166176887443649, 0, 0, -0.00647610327216139, 0, 0.00646276421933346, -0.00290809553026393, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0.0213629048496073, 0, 0, 0, -0.0277928188745204, 0, 0, 0.0243701532779658, 0, -0.0157219925555703, 0.00607967912880134},
    {0.0100234421734308, 0, -0.0287574853355261, 0.05360943398437, 0, 0, -0.0189425251940965, 0, 0.0154665070551172, 0.0153110608279201, 0, 0, 0, -0.00450937950937953, 0, 0, 0.0370997534990628, 0, -0.0281660799985623, 0.0108976671476671, 0, 0, 0, 0, 0, 0, 0, 0, 0.0299118397398575, 0, 0, -0.00431740218144094, 0, -0.0123380044187275, 0.00872428659079179, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0.0312669212338355, 0, 0, -0.0487403065559315, 0, 0.0412702304583721, -0.018239037386404},
    {0.0100234421734309, 0, 0.0409576306293854, -0.0449826285156208, 0, 0, 0.0189425251940966, 0, -0.00515550235170581, -0.021143845905223, 0, 0, 0, 0, 0, 0, 0.00455611007883228, 0, 0.0356061766019561, -0.0319414381914382, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0.0302218152700864, 0, 0.00411266813957589, -0.0145404776513196, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0.0379091273212801, 0, -0.0550269739444961, 0.0303983956440067},
    {-0.0100234421734309, 0, -0.0148144621425437, 0.0449826285156208, 0, 0, 0, 0, 0.0206220094068229, 0.021143845905223, 0, 0, 0, 0, 0, 0, 0, 0, 0.0106287094334197, 0.0319414381914382, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0.0205633406978791, 0.0145404776513196, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0.03439185871531, -0.0303983956440067},
    {-0.0100234421734309, 0.0324515955311978, -0.01873593741557, -0.0289614183593723, 0.0211783869997511, -0.0164047080299144, 0.00947126259704827, 0.00446479600584751, -0.00257775117585286, -0.00619733414463429, 0.0208787193108001, -0.0176457384588051, 0.0136683302364968, -0.00789141414141412, -0.00848980987436741, 0.00657617845117844, -0.00379675839902691, 0.0023011830946962, -0.00132858867917746, -0.000187890812890819, 0.0139343556230002, -0.0122889465527977, 0.0103860554652283, -0.00804500396992315, 0.00464478520766672, -0.00109915679455667, 0.000928957041533349, -0.000719567030240153, 0.000415442218609154, -0.00482700238195389, 0.00373897996748218, -0.00215870109072046, 0.00559691799261187, -0.00323138210966673, -0.00290809553026394, 0, 0, 0, 0, 0, 0, 0.0128177429097644, -0.0113041867027994, 0.00955378148811641, -0.00740032731935592, 0.00427258096992148, -0.0183832217438608, 0.0155366580724417, -0.0120346435940572, 0.00694820471863012, 0.0181644397838736, -0.014070114555226, 0.00812338442598859, -0.0136156449512337, 0.00786099627778517, 0.00607967912880136},
    {0.0100234421734311, -0.024904712849524, 0.014378742667763, 0.0536094339843701, -0.0141189246665008, 0.0109364720199429, -0.00631417506469887, 0.0133943880175428, -0.00773325352755854, 0.01531106082792, -0.00298267418725714, 0.0025208197798293, -0.00195261860521383, 0.00112734487734485, 0.0276525235907967, -0.0214195526695527, 0.0123665844996876, -0.0243925408037797, 0.0140830399992812, 0.0108976671476671, 0, 0, 0, 0, 0, 0.0197848223020205, -0.0167212267476003, 0.0129522065443228, -0.0074779599349644, -0.00321800158796925, 0.00249265331165481, -0.00143913406048031, -0.0106850252586226, 0.00616900220936375, 0.00872428659079183, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0.0206811244618434, -0.017478740331497, 0.0135389740433143, -0.00781673030845889, -0.0363288795677472, 0.028140229110452, -0.0162467688519772, 0.0357410679969885, -0.0206351152291861, -0.0182390373864041},
    {0.0100234421734311, 0.0354703486038675, -0.0204788153146928, -0.0449826285156207, 0.0141189246665009, -0.0109364720199429, 0.00631417506469889, -0.0044647960058475, 0.00257775117585279, -0.021143845905223, 0, 0, 0, 0, 0.00339592394974698, -0.00263047138047136, 0.00151870335961076, 0.0308358534689291, -0.0178030883009781, -0.0319414381914382, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0.0225260111157848, -0.0174485731815835, 0.0100739384233621, 0.00356167508620756, -0.00205633406978792, -0.0145404776513196, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0.028255795219359, -0.0218868448636849, 0.01263637577376, -0.047654757329318, 0.0275134869722481, 0.0303983956440068},
    {-0.0100234421734311, -0.0128297005588457, 0.00740723107127181, 0.0449826285156207, 0, 0, 0, 0.0178591840233903, -0.0103110047034114, 0.0211438459052229, 0, 0, 0, 0, 0, 0, 0, 0.0092047323787848, -0.00531435471670985, 0.0319414381914382, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0.0178083754310377, -0.0102816703489396, 0.0145404776513196, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0.0297842233308237, -0.017195929357655, -0.0303983956440068},
    {-0.0100234421734311, 0.0324515955311977, -0.0335503995581136, -0.00801060507812428, 0.0211783869997512, 0.00109364720199428, -0.00568275755822898, -0.0169662248222208, 0.00567105258687634, 0.0083846285486229, 0.0208787193108001, -0.0109235523792603, 0.00499002532443531, -0.00187890812890811, -0.0162519217595033, 0.00958243145743145, -0.00466458889023307, 0.0108922666482287, -0.00558007245254534, -0.00469727032227035, 0.0139343556230002, -0.00245778931055952, -0.00415442218609131, 0.0064360031759385, -0.00464478520766676, -0.0120907247401236, 0.00464478520766674, -0.000719567030240154, -0.000415442218609135, 0.00965400476390777, -0.00498530662330959, 0.00215870109072046, -0.00661453944581401, 0.0032313821096667, 0.00290809553026394, 0, 0.0127646671417816, -0.0202632808954031, 0.0228341307513453, -0.020635115229186, 0.0127646671417816, -0.00116524935543312, -0.00719357335632687, 0.0112908326677739, -0.0114368694935501, 0.00737991258440983, 0.00183832217438608, 0.00310733161448834, -0.00481385743762287, 0.00347410235931505, -0.00201827108709706, -0.000521115353897253, 0.000902598269554298, 0.00170195561890421, -0.000491312267361565, -0.000868525589828764},
    {0.0100234421734313, -0.0249047128495239, 0.0553363732971484, -0.00431340273437463, -0.0141189246665008, 0.0153110608279201, 0.00883984509057839, 0.00803663281052565, -0.0159820572902878, 0.000729098134662871, -0.00298267418725714, 0.0277290175781224, -0.0301571095694136, 0.0191648629148629, -0.00145539597846295, -0.0116492303992304, 0.00976309302606917, 0.00352848074520082, 0.00132858867917745, -0.00263047138047135, 0, 0.0196623144844763, -0.00727023882565982, -0.00724050357293083, 0.00928957041533347, -0.00219831358911338, -0.0148633126645336, 0.0129522065443228, -0.00664707549774609, 0.00402250198496158, 0.00685479660705067, -0.00575653624192123, -0.00457929653940971, -0.000293762009969677, 0.0029080955302639, 0, 0, 0.0202632808954031, -0.0399597288148543, 0.0464290092656686, -0.031911667854454, 0, -0.00411061334647249, -0.00303983956440069, 0.00908221989193683, -0.00776832903622088, 0.000459580543596524, 0.00427258096992149, -0.00270779480866288, 0.000868525589828788, -0.00100913554354855, -0.00234501909253767, 0.00180519653910856, 0.00127646671417817, 0.000245656133680783, -0.000868525589828766},
    {0.0100234421734309, 0.0354703486038674, -0.0492363006502187, -0.00431340273437457, 0.0141189246665008, -0.00656188321196574, -0.016416855168217, -0.00982255121286468, 0.0149509568199466, 0.00072909813466289, 0, 0.00336109303977242, 0.0282044909641998, -0.0353234728234728, -0.000485131992821028, -0.0124007936507936, 0.00889526253486298, 0.00291483191994853, 0.00239145962251946, -0.0026304713804714, 0, 0, 0.0218107164769794, -0.00402250198496158, -0.00928957041533346, 0, -0.00557374224920008, -0.0143913406048031, 0.0141250354327105, 0.000804500396992295, 0.00934744991870546, -0.00287826812096058, -0.00254405363300538, -0.00381890612960614, 0.00290809553026392, 0, 0, 0, 0.0266398192099029, -0.0515877880729651, 0.0425488904726054, 0, 0, -0.00911951869320201, 0.0084094628629044, -0.0038841645181104, 0, 0.00233049871086625, 0.00300866089851431, -0.00347410235931508, -0.000336378514516169, -0.00286613444643491, 0.00120346435940571, 0.000850977809452102, 0.000982624534723146, -0.000868525589828757},
    {-0.0100234421734311, -0.0128297005588456, 0.0448791059024117, -0.00801060507812423, 0, 0.0174983552319087, 0.0132597676358676, -0.00357183680467805, -0.0175287079957994, 0.00838462854862291, 0, 0, 0.00867830491206144, 0.0225468975468975, 0, -0.00300625300625305, -0.019200749617936, 0.000613648825252313, 0.0122230158484327, -0.00469727032227032, 0, 0, 0, 0.0160900079398463, 0.00464478520766673, 0, 0, -0.00719567030240156, -0.0108014976838374, 0, 0.00249265331165479, 0.0107935054536023, -0.000508810726601082, -0.00734405024924251, 0.00290809553026393, 0, 0, 0, 0, 0.0257938940364825, -0.031911667854454, 0, 0, 0, -0.0134551405806471, 0.0135945758133865, 0, 0, 0.00601732179702858, -0.00347410235931504, 0, -0.00208446141558903, -0.000902598269554288, 0.000425488904726059, 0.0017195929357655, -0.000868525589828767},
    {-0.010023442173431, -0.0324515955311979, -0.0187359374155701, -0.0289614183593722, 0.0211783869997511, 0.0164047080299144, 0.00947126259704825, -0.0044647960058475, -0.00257775117585286, -0.00619733414463431, -0.0208787193108001, -0.0176457384588052, -0.0136683302364968, -0.00789141414141413, -0.00848980987436743, -0.00657617845117846, -0.00379675839902691, -0.00230118309469614, -0.00132858867917743, -0.000187890812890788, 0.0139343556230002, 0.0122889465527977, 0.0103860554652283, 0.00804500396992312, 0.00464478520766673, 0.00109915679455666, 0.00092895704153335, 0.000719567030240155, 0.000415442218609149, -0.0048270023819539, -0.00373897996748222, -0.00215870109072046, -0.00559691799261192, -0.00323138210966673, -0.00290809553026393, 0, 0, 0, 0, 0, 0, 0.0128177429097644, 0.0113041867027994, 0.00955378148811642, 0.00740032731935589, 0.00427258096992148, 0.0183832217438608, 0.0155366580724417, 0.0120346435940572, 0.00694820471863011, 0.0181644397838736, 0.014070114555226, 0.00812338442598858, 0.0136156449512337, 0.00786099627778515, 0.00607967912880134},
    {0.0100234421734311, 0.024904712849524, 0.014378742667763, 0.05360943398437, -0.0141189246665008, -0.0109364720199429, -0.00631417506469881, -0.0133943880175428, -0.00773325352755853, 0.01531106082792, 0.00298267418725717, 0.0025208197798293, 0.00195261860521382, 0.00112734487734485, 0.0276525235907967, 0.0214195526695527, 0.0123665844996876, 0.0243925408037797, 0.0140830399992811, 0.0108976671476671, 0, 0, 0, 0, 0, -0.0197848223020205, -0.0167212267476003, -0.0129522065443227, -0.0074779599349644, -0.00321800158796927, -0.00249265331165477, -0.00143913406048032, 0.0106850252586227, 0.00616900220936373, 0.00872428659079179, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, -0.0206811244618434, -0.017478740331497, -0.0135389740433143, -0.00781673030845888, -0.0363288795677472, -0.028140229110452, -0.0162467688519772, -0.0357410679969885, -0.020635115229186, -0.018239037386404},
    {0.0100234421734307, -0.0354703486038675, -0.0204788153146927, -0.0449826285156207, 0.0141189246665008, 0.0109364720199429, 0.00631417506469886, 0.00446479600584751, 0.00257775117585285, -0.0211438459052229, 0, 0, 0, 0, 0.00339592394974697, 0.00263047138047138, 0.00151870335961076, -0.0308358534689291, -0.017803088300978, -0.0319414381914382, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0.0225260111157848, 0.0174485731815835, 0.0100739384233621, -0.00356167508620757, -0.00205633406978789, -0.0145404776513196, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0.0282557952193589, 0.0218868448636849, 0.01263637577376, 0.047654757329318, 0.027513486972248, 0.0303983956440067},
    {-0.010023442173431, 0.0128297005588457, 0.00740723107127182, 0.0449826285156208, 0, 0, 0, -0.0178591840233902, -0.0103110047034114, 0.021143845905223, 0, 0, 0, 0, 0, 0, 0, -0.00920473237878481, -0.00531435471670985, 0.0319414381914381, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, -0.0178083754310377, -0.0102816703489396, 0.0145404776513196, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, -0.0297842233308237, -0.017195929357655, -0.0303983956440068},
    {-0.0100234421734306, -0.0324515955311977, -0.0335503995581136, -0.00801060507812422, 0.0211783869997512, -0.00109364720199428, -0.00568275755822895, 0.0169662248222208, 0.00567105258687631, 0.00838462854862288, -0.0208787193108001, -0.0109235523792604, -0.00499002532443533, -0.00187890812890813, -0.0162519217595033, -0.00958243145743147, -0.00466458889023305, -0.0108922666482287, -0.00558007245254534, -0.00469727032227036, 0.0139343556230002, 0.00245778931055952, -0.00415442218609133, -0.00643600317593853, -0.00464478520766674, 0.0120907247401236, 0.00464478520766673, 0.000719567030240151, -0.000415442218609145, 0.00965400476390776, 0.00498530662330958, 0.00215870109072046, 0.006614539445814, 0.00323138210966669, 0.00290809553026394, 0, 0.0127646671417816, 0.0202632808954031, 0.0228341307513453, 0.020635115229186, 0.0127646671417816, -0.00116524935543314, 0.00719357335632687, 0.0112908326677739, 0.01143686949355, 0.00737991258440983, -0.00183832217438609, 0.00310733161448835, 0.00481385743762286, 0.00347410235931504, -0.00201827108709706, 0.00052111535389726, 0.000902598269554298, -0.00170195561890421, -0.000491312267361563, -0.000868525589828763},
    {0.0100234421734315, 0.024904712849524, 0.0553363732971485, -0.00431340273437464, -0.0141189246665008, -0.0153110608279201, 0.00883984509057836, -0.00803663281052558, -0.0159820572902878, 0.000729098134662864, 0.00298267418725715, 0.0277290175781224, 0.0301571095694136, 0.0191648629148629, -0.00145539597846298, 0.0116492303992304, 0.00976309302606916, -0.00352848074520084, 0.00132858867917744, -0.00263047138047133, 0, -0.0196623144844763, -0.0072702388256598, 0.00724050357293084, 0.0092895704153335, 0.00219831358911338, -0.0148633126645336, -0.0129522065443228, -0.0066470754977461, 0.00402250198496159, -0.00685479660705068, -0.00575653624192122, 0.0045792965394097, -0.000293762009969664, 0.00290809553026391, 0, 0, -0.0202632808954031, -0.0399597288148543, -0.0464290092656686, -0.031911667854454, 0, 0.00411061334647249, -0.00303983956440066, -0.0090822198919368, -0.00776832903622086, -0.000459580543596519, 0.0042725809699215, 0.00270779480866287, 0.000868525589828796, -0.00100913554354854, 0.00234501909253766, 0.00180519653910856, -0.00127646671417816, 0.00024565613368078, -0.00086852558982877},
    {0.0100234421734312, -0.0354703486038675, -0.0492363006502186, -0.0043134027343746, 0.0141189246665008, 0.00656188321196572, -0.016416855168217, 0.0098225512128646, 0.0149509568199467, 0.000729098134662864, 0, 0.0033610930397724, -0.0282044909641998, -0.0353234728234728, -0.000485131992820996, 0.0124007936507936, 0.00889526253486298, -0.00291483191994851, 0.00239145962251946, -0.00263047138047141, 0, 0, 0.0218107164769794, 0.00402250198496156, -0.00928957041533348, 0, -0.0055737422492001, 0.0143913406048031, 0.0141250354327105, 0.00080450039699229, -0.00934744991870546, -0.00287826812096059, 0.00254405363300538, -0.00381890612960615, 0.00290809553026392, 0, 0, 0, 0.0266398192099029, 0.0515877880729651, 0.0425488904726054, 0, 0, -0.00911951869320204, -0.00840946286290444, -0.00388416451811041, 0, 0.00233049871086624, -0.0030086608985143, -0.00347410235931509, -0.000336378514516169, 0.00286613444643491, 0.00120346435940572, -0.000850977809452107, 0.000982624534723155, -0.000868525589828754},
    {-0.010023442173431, 0.0128297005588457, 0.0448791059024117, -0.00801060507812423, 0, -0.0174983552319087, 0.0132597676358676, 0.0035718368046781, -0.0175287079957995, 0.00838462854862291, 0, 0, -0.00867830491206147, 0.0225468975468975, 0, 0.00300625300625302, -0.019200749617936, -0.000613648825252327, 0.0122230158484327, -0.00469727032227032, 0, 0, 0, -0.0160900079398463, 0.00464478520766674, 0, 0, 0.00719567030240152, -0.0108014976838374, 0, -0.00249265331165479, 0.0107935054536023, 0.000508810726601078, -0.00734405024924251, 0.00290809553026392, 0, 0, 0, 0, -0.0257938940364826, -0.031911667854454, 0, 0, 0, 0.0134551405806471, 0.0135945758133865, 0, 0, -0.00601732179702859, -0.00347410235931504, 0, 0.00208446141558904, -0.000902598269554294, -0.000425488904726048, 0.0017195929357655, -0.000868525589828768},
    {-0.0100234421734308, -0.0452812960900435, -0.0113287063442981, -0.00801060507812425, -0.00141189246665006, 0.0164047080299144, 0.0145226026488073, 0.0133943880175427, 0.0118576554089232, 0.0083846285486229, -0.00894802256177148, -0.0142846454190327, -0.014536160727703, -0.00939454064454062, -0.0123708658169354, -0.0125886844636845, -0.00813591085505763, -0.0102786178229763, -0.00664294339588729, -0.00469727032227033, -0.00836061337380013, 0.00245778931055953, 0.00830884437218264, 0.00965400476390776, 0.00650269929073342, 0.00219831358911336, 0.00743165633226678, 0.00863480436288185, 0.00581619106052787, 0.00643600317593851, 0.00747795993496438, 0.00503696921168107, 0.00610572871921291, 0.00411266813957582, 0.00290809553026393, 0.0352796762358899, 0.0191470007126724, 0.00788016479265675, 0.000951422114639387, -0.00221090520312706, -0.00212744452363027, 0.017478740331497, 0.00719357335632688, 0.000868525589828766, -0.00201827108709706, -0.00194208225905523, 0.00643412761035128, 0.00077683290362208, -0.00180519653910858, -0.00173705117965754, 0.000672757029032357, -0.00156334606169177, -0.00150433044925715, -0.00127646671417815, -0.00122828066840394, -0.000868525589828763},
    {0.010023442173431, 0.0603750614533912, -0.00610007264692973, -0.0043134027343746, -0.00564756986660029, -0.0218729440398858, 0.00126283501293976, -0.0178591840233903, 0.00103110047034112, 0.00072909813466285, 0.0417574386216002, 0.0168054651988621, -0.000867830491206138, -0.00526094276094278, 0.0145539597846298, -0.000751563251563223, -0.00455611007883228, -0.000613648825252315, -0.00372004830169692, -0.00263047138047137, 0.00557374224920009, -0.0196623144844763, -0.0103860554652283, 0.00321800158796927, 0.00650269929073345, -0.0175865087129071, -0.00928957041533348, 0.00287826812096062, 0.00581619106052783, -0.00804500396992314, 0.0024926533116548, 0.00503696921168107, 0.00203524290640432, 0.00411266813957582, 0.00290809553026392, -0.0705593524717798, -0.0127646671417816, 0.00450295131008957, 0.00380568845855756, -0.000736968401042374, -0.00212744452363027, -0.0116524935543313, 0.0041106133464725, 0.00347410235931506, -0.000672757029032359, -0.0019420822590552, 0.00367664434877216, 0.00310733161448835, -0.000601732179702866, -0.00173705117965752, 0.00269102811612943, -0.000521115353897279, -0.00150433044925714, -0.000425488904726059, -0.00122828066840393, -0.000868525589828762},
    {0.010023442173431, -0.0603750614533913, -0.00610007264692972, -0.00431340273437459, -0.00564756986660034, 0.0218729440398858, 0.00126283501293975, 0.0178591840233902, 0.00103110047034114, 0.000729098134662857, -0.0417574386216002, 0.016805465198862, 0.000867830491206142, -0.00526094276094274, 0.0145539597846298, 0.000751563251563242, -0.00455611007883226, 0.000613648825252326, -0.0037200483016969, -0.00263047138047136, 0.0055737422492001, 0.0196623144844763, -0.0103860554652283, -0.00321800158796928, 0.00650269929073344, 0.0175865087129071, -0.00928957041533347, -0.00287826812096062, 0.00581619106052785, -0.00804500396992314, -0.0024926533116548, 0.00503696921168109, -0.00203524290640433, 0.00411266813957582, 0.00290809553026392, 0.0705593524717798, -0.0127646671417816, -0.00450295131008957, 0.00380568845855755, 0.000736968401042381, -0.00212744452363028, -0.0116524935543313, -0.00411061334647249, 0.00347410235931505, 0.00067275702903236, -0.00194208225905522, -0.00367664434877216, 0.00310733161448835, 0.000601732179702862, -0.00173705117965754, 0.00269102811612941, 0.000521115353897267, -0.00150433044925715, 0.000425488904726055, -0.00122828066840394, -0.00086852558982877},
    {-0.0100234421734311, 0.0452812960900435, -0.0113287063442981, -0.00801060507812424, -0.00141189246665007, -0.0164047080299144, 0.0145226026488074, -0.0133943880175427, 0.0118576554089232, 0.00838462854862294, 0.00894802256177145, -0.0142846454190327, 0.014536160727703, -0.00939454064454065, -0.0123708658169354, 0.0125886844636845, -0.00813591085505765, 0.0102786178229763, -0.00664294339588732, -0.00469727032227034, -0.00836061337380011, -0.00245778931055953, 0.00830884437218264, -0.00965400476390776, 0.00650269929073342, -0.00219831358911338, 0.00743165633226677, -0.00863480436288184, 0.00581619106052786, 0.00643600317593851, -0.00747795993496437, 0.00503696921168106, -0.0061057287192129, 0.00411266813957581, 0.00290809553026391, -0.0352796762358899, 0.0191470007126724, -0.00788016479265674, 0.000951422114639392, 0.00221090520312706, -0.00212744452363026, 0.017478740331497, -0.00719357335632687, 0.000868525589828777, 0.00201827108709706, -0.00194208225905521, -0.00643412761035126, 0.000776832903622074, 0.00180519653910858, -0.00173705117965752, 0.00067275702903235, 0.00156334606169178, -0.00150433044925714, 0.00127646671417815, -0.00122828066840393, -0.000868525589828756},
    {0.0787556170769578, 0.0679219441350652, -0.00435719474780692, -0.00308100195312471, 0.0705946233325039, -0.0328094160598287, 0.0126283501293978, -0.0267887760350854, 0.03608851646194, -0.00729098134662863, 0.0298267418725716, 0.0151249186789758, -0.0325436434202305, 0.0248015873015872, 0.0130985638061668, -0.0229226791726792, 0.0169226945785199, -0.0248527774227189, 0.0164744996218005, 0.0142797017797018, 0, 0.0245778931055953, -0.0363511941282991, 0.0362025178646542, -0.0232239260383337, 0.0219831358911338, -0.00743165633226679, 0, 0.00166176887443652, -0.0345935170706695, 0.0130864298861876, -0.00359783515120077, 0.0335815079556711, -0.014688100498485, -0.0178640154001927, 0, 0, 0, 0, 0, 0, 0, 0.0226083734055987, -0.0334382352084074, 0.0333014729371016, -0.0213629048496074, -0.00459580543596519, -0.0194208225905522, 0.0270779480866286, -0.0191075629762328, 0.0090822198919368, 0.00703505727761299, -0.00812338442598857, -0.0102117337134253, 0.00196524906944628, 0.00607967912880137},
    {-0.0214788046573525, 0.00377344134083702, 0.00217859737390337, -0.021567013671873, 0, 0.0984282481794862, -0.0852413633734345, -0.0312535720409329, -0.0180442582309699, 0.0309866707231716, 0, 0.0453747560369276, 0, -0.0202922077922077, -0.0116431678277039, 0.0236742424242425, -0.0335199527228374, -0.00138070985681772, -0.000797153207506455, 0.00394570707070704, 0, 0, 0.0363511941282991, -0.0603375297744235, 0.0464478520766673, 0, 0.0241528830798671, 0, -0.0108014976838374, -0.00724050357293081, -0.0392592896585629, 0.0259044130886455, 0.0188259968842399, 0.0108691943688789, -0.0178640154001926, 0, 0, 0, 0, 0, 0, 0, 0, 0.0334382352084074, -0.0555024548951693, 0.0427258096992148, 0, -0.0155366580724417, 0, 0.00694820471863008, 0.00302740663064561, 0.0164151336477636, -0.0108311792346514, -0.00680782247561684, -0.00393049813889257, 0.00607967912880131},
    {0.0787556170769581, 0.0301875307266957, 0.0610007264692977, -0.00308100195312485, 0, 0.0218729440398858, 0.0757701007763862, 0.0178591840233902, -0.0412440188136458, -0.00729098134662867, 0, 0, 0.0520698294723689, -0.00901875901875909, 0, -0.0127765752765753, 0.0286384062098029, 0.00184094647575695, -0.0297603864135753, 0.0142797017797018, 0, 0, 0, 0.0402250198496157, -0.0464478520766674, 0, 0, 0.014391340604803, 0.0182794576188018, 0, -0.0137095932141014, -0.0345392174515274, 0.00407048581280862, 0.036426489236243, -0.0178640154001927, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0.0370016365967796, -0.0427258096992148, 0, 0, -0.030086608985143, 0.0243187165152054, 0, 0.014070114555226, 0, -0.00340391123780844, -0.00982624534723146, 0.00607967912880138},
    {-0.0214788046573519, 0.00377344134083692, -0.0196073763651313, 0.00924300585937408, 0, -0.0109364720199429, 0.00947126259704824, 0.102690308134494, -0.0695992817480272, -0.060150596109686, 0, -0.00504163955965862, 0.0065087286840461, -0.00450937950937948, 0.0465726713108155, 0.0214195526695527, -0.0289638426440052, -0.00782402252196701, -0.0263060558477138, 0.000563672438672498, 0, 0, 0, 0, 0, 0, 0.0334424534952005, -0.0431740218144092, 0.0299118397398575, 0.0289620142917233, -0.00249265331165477, -0.00431740218144094, -0.0605484764655283, 0.0185070066280912, 0.0461140862656136, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0.0349574806629939, -0.0451299134777144, 0.0312669212338355, -0.0121096265225824, -0.028140229110452, 0.0270779480866286, 0.023827378664659, 0, -0.0182390373864041},
    {-0.0214788046573522, -0.0150937653633478, 0.0130715842434209, 0.00924300585937409, 0, -0.0109364720199429, 0.00947126259704823, -0.00892959201169517, 0.123732056440937, -0.060150596109686, 0, 0, -0.00650872868404615, 0.00676406926406924, 0, 0.0574945887445888, 0.0126920209338899, -0.026693723898476, 0.00637722566005184, 0.000563672438672442, 0, 0, 0, 0, 0, 0, 0, 0.0431740218144092, -0.0448677596097863, 0, 0.0199412264932383, 0.0215870109072046, -0.0142467003448301, -0.0616900220936373, 0.0461140862656136, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0.0451299134777144, -0.0469003818507533, 0, -0.0375203054806026, 0.0162467688519772, 0.0119136893323295, 0.0206351152291861, -0.0182390373864041},
    {0.078755617076958, 0.0301875307266957, 0.0174287789912279, 0.0585390371093694, 0, 0.0218729440398858, -0.0189425251940966, 0.0178591840233902, 0.0103110047034113, 0.083846285486229, 0, 0, 0, 0, 0, 0.00526094276094275, -0.00455611007883227, 0.0533874477969518, 0.0308232573569172, -0.0319414381914382, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0.0348971463631671, -0.0302218152700865, 0.0284934006896603, 0.0164506725583033, -0.0643935438844155, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0.0437736897273697, -0.0379091273212801, -0.023827378664659, -0.0137567434861241, 0.0303983956440068},
    {0.0787556170769583, -0.0679219441350652, -0.00435719474780694, -0.00308100195312467, 0.0705946233325039, 0.0328094160598287, 0.0126283501293977, 0.0267887760350853, 0.03608851646194, -0.00729098134662864, -0.0298267418725716, 0.0151249186789759, 0.0325436434202305, 0.0248015873015873, 0.0130985638061668, 0.0229226791726792, 0.0169226945785199, 0.0248527774227189, 0.0164744996218005, 0.0142797017797018, 0, -0.0245778931055954, -0.036351194128299, -0.0362025178646541, -0.0232239260383336, -0.0219831358911338, -0.00743165633226678, 0, 0.00166176887443654, -0.0345935170706695, -0.0130864298861876, -0.00359783515120077, -0.0335815079556711, -0.014688100498485, -0.0178640154001927, 0, 0, 0, 0, 0, 0, 0, -0.0226083734055987, -0.0334382352084074, -0.0333014729371016, -0.0213629048496074, 0.00459580543596522, -0.0194208225905522, -0.0270779480866286, -0.0191075629762328, 0.0090822198919368, -0.007035057277613, -0.00812338442598858, 0.0102117337134253, 0.00196524906944627, 0.00607967912880136},
    {-0.0214788046573522, -0.00377344134083698, 0.00217859737390352, -0.0215670136718729, 0, -0.0984282481794863, -0.0852413633734345, 0.031253572040933, -0.0180442582309699, 0.0309866707231716, 0, 0.0453747560369276, 0, -0.0202922077922078, -0.0116431678277039, -0.0236742424242425, -0.0335199527228375, 0.00138070985681774, -0.000797153207506485, 0.00394570707070704, 0, 0, 0.0363511941282991, 0.0603375297744236, 0.0464478520766674, 0, 0.024152883079867, 0, -0.0108014976838374, -0.0072405035729308, 0.0392592896585629, 0.0259044130886455, -0.0188259968842399, 0.0108691943688789, -0.0178640154001927, 0, 0, 0, 0, 0, 0, 0, 0, 0.0334382352084074, 0.0555024548951693, 0.0427258096992148, 0, -0.0155366580724417, 0, 0.0069482047186301, 0.0030274066306456, -0.0164151336477636, -0.0108311792346515, 0.00680782247561683, -0.00393049813889256, 0.00607967912880134},
    {0.078755617076958, -0.0301875307266958, 0.0610007264692976, -0.00308100195312472, 0, -0.0218729440398859, 0.0757701007763862, -0.0178591840233903, -0.0412440188136458, -0.00729098134662865, 0, 0, -0.0520698294723688, -0.00901875901875904, 0, 0.0127765752765752, 0.0286384062098029, -0.00184094647575696, -0.0297603864135752, 0.0142797017797018, 0, 0, 0, -0.0402250198496157, -0.0464478520766673, 0, 0, -0.0143913406048031, 0.0182794576188018, 0, 0.0137095932141014, -0.0345392174515273, -0.00407048581280864, 0.036426489236243, -0.0178640154001927, 0, 0, 0, 0, 0, 0, 0, 0, 0, -0.0370016365967795, -0.0427258096992148, 0, 0, 0.0300866089851429, 0.0243187165152054, 0, -0.014070114555226, 0, 0.00340391123780844, -0.00982624534723146, 0.00607967912880137},
    {-0.0214788046573522, -0.00377344134083677, -0.0196073763651313, 0.00924300585937403, 0, 0.0109364720199429, 0.00947126259704829, -0.102690308134494, -0.0695992817480271, -0.0601505961096861, 0, -0.00504163955965865, -0.00650872868404618, -0.00450937950937947, 0.0465726713108155, -0.0214195526695527, -0.0289638426440052, 0.00782402252196705, -0.0263060558477139, 0.000563672438672498, 0, 0, 0, 0, 0, 0, 0.0334424534952005, 0.0431740218144092, 0.0299118397398575, 0.0289620142917233, 0.00249265331165484, -0.00431740218144093, 0.0605484764655283, 0.0185070066280911, 0.0461140862656136, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0.0349574806629939, 0.0451299134777144, 0.0312669212338355, -0.0121096265225824, 0.028140229110452, 0.0270779480866286, -0.023827378664659, 0, -0.018239037386404},
    {-0.0214788046573521, 0.0150937653633479, 0.0130715842434208, 0.00924300585937423, 0, 0.010936472019943, 0.00947126259704825, 0.00892959201169522, 0.123732056440937, -0.060150596109686, 0, 0, 0.00650872868404613, 0.00676406926406924, 0, -0.0574945887445887, 0.0126920209338899, 0.0266937238984759, 0.00637722566005184, 0.000563672438672452, 0, 0, 0, 0, 0, 0, 0, -0.0431740218144092, -0.0448677596097863, 0, -0.0199412264932383, 0.0215870109072046, 0.0142467003448301, -0.0616900220936373, 0.0461140862656136, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, -0.0451299134777144, -0.0469003818507533, 0, 0.0375203054806026, 0.0162467688519772, -0.0119136893323295, 0.0206351152291861, -0.0182390373864041},
    {0.0787556170769579, -0.0301875307266956, 0.017428778991228, 0.0585390371093696, 0, -0.0218729440398858, -0.0189425251940965, -0.0178591840233903, 0.0103110047034114, 0.083846285486229, 0, 0, 0, 0, 0, -0.00526094276094277, -0.00455611007883227, -0.0533874477969518, 0.0308232573569172, -0.0319414381914383, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, -0.0348971463631671, -0.0302218152700865, -0.0284934006896603, 0.0164506725583033, -0.0643935438844154, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, -0.0437736897273697, -0.0379091273212801, 0.023827378664659, -0.0137567434861241, 0.0303983956440068},
    {0.078755617076958, -0.0377344134083697, -0.0566435317214906, -0.00308100195312479, 0.0423567739995023, 0.0546823600997145, 0.0378850503881931, 0.0446479600584756, 0.00515550235170578, -0.00729098134662856, 0.0417574386216002, -0.00504163955965862, -0.0247331689993752, -0.0202922077922078, 0.0295930515620807, 0.0101461038961039, 0.00216957622801541, 0.0266937238984759, 0.0132858867917747, 0.0142797017797018, -0.0557374224920008, -0.0245778931055953, -0.00519302773261415, 0.00402250198496156, 0.00464478520766674, -0.00219831358911332, -0.0130053985814669, -0.0158304746652834, -0.0108014976838375, -0.0168945083368386, -0.026796023100289, -0.0194283098164841, -0.0295110221428625, -0.0217383887377579, -0.0178640154001927, 0, 0, 0, 0, 0, 0, -0.0512709716390577, -0.0226083734055987, -0.00477689074405822, 0.00370016365967794, 0.00427258096992147, -0.0367664434877216, -0.00776832903622086, 0.00601732179702859, 0.00694820471863011, -0.00908221989193683, 0.00703505727761299, 0.00812338442598859, 0.00680782247561683, 0.00786099627778515, 0.00607967912880135},
    {-0.0214788046573522, 0, -0.00435719474780689, -0.0215670136718729, -0.127070321998507, 0, 0.0284137877911447, 0, 0.0360885164619399, 0.0309866707231715, 0, 0.0453747560369275, 0, -0.0202922077922078, -0.0422064833754266, 0, -0.00618329224984384, 0, 0.0015943064150129, 0.00394570707070703, 0.0836061337380012, 0, -0.0103860554652283, 0, 0.00464478520766673, 0, 0.024152883079867, 0, -0.0108014976838374, 0.043443021437585, 0, -0.0194283098164841, 0, -0.0217383887377578, -0.0178640154001926, 0, 0, 0, 0, 0, 0, 0.0769064574585866, 0, -0.00955378148811639, 0, 0.00427258096992146, 0, -0.0155366580724417, 0, 0.00694820471863015, -0.0181644397838736, 0, 0.00812338442598858, 0, 0.00786099627778515, 0.00607967912880134},
    {0.0787556170769581, 0.0377344134083697, -0.0566435317214905, -0.00308100195312477, 0.0423567739995024, -0.0546823600997146, 0.0378850503881931, -0.0446479600584756, 0.00515550235170579, -0.00729098134662863, -0.0417574386216002, -0.00504163955965864, 0.0247331689993752, -0.0202922077922078, 0.0295930515620807, -0.0101461038961039, 0.00216957622801542, -0.0266937238984759, 0.0132858867917747, 0.0142797017797018, -0.0557374224920008, 0.0245778931055954, -0.00519302773261416, -0.00402250198496156, 0.00464478520766676, 0.00219831358911337, -0.0130053985814669, 0.0158304746652834, -0.0108014976838375, -0.0168945083368386, 0.026796023100289, -0.0194283098164841, 0.0295110221428625, -0.0217383887377579, -0.0178640154001927, 0, 0, 0, 0, 0, 0, -0.0512709716390577, 0.0226083734055987, -0.00477689074405822, -0.00370016365967795, 0.00427258096992149, 0.0367664434877216, -0.00776832903622086, -0.00601732179702859, 0.00694820471863009, -0.00908221989193678, -0.00703505727761301, 0.00812338442598859, -0.00680782247561686, 0.00786099627778517, 0.00607967912880135},
    {-0.021478804657352, -0.0188672067041847, 0.00653579212171052, 0.00924300585937401, 0.0141189246665008, 0, -0.00315708753234946, -0.111619900146189, -0.05413277469291, -0.0601505961096861, -0.00894802256177153, -0.00252081977982931, 0.000650872868404653, 0.00112734487734485, -0.0276525235907967, 0.0360750360750361, 0.0374251899332651, -0.0188697013765088, 0.019928830187662, 0.000563672438672417, 0, 0, 0, 0, 0, 0.0593544669060614, 0.0167212267476003, -0.00431740218144091, -0.0074779599349644, 0.00321800158796925, 0.0224338798048931, 0.018708742786244, 0.0463017761206981, 0.0431830154655461, 0.0461140862656137, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0.0620433733855302, 0.017478740331497, -0.00451299134777146, -0.0078167303084589, 0.0363288795677472, -0.00938007637015066, -0.0162467688519772, -0.0119136893323295, -0.020635115229186, -0.018239037386404},
    {-0.021478804657352, 0.0188672067041848, 0.00653579212171056, 0.00924300585937413, 0.0141189246665008, 0, -0.00315708753234944, 0.111619900146189, -0.05413277469291, -0.060150596109686, 0.00894802256177151, -0.00252081977982929, -0.000650872868404627, 0.00112734487734488, -0.0276525235907967, -0.0360750360750361, 0.0374251899332651, 0.0188697013765088, 0.0199288301876619, 0.000563672438672459, 0, 0, 0, 0, 0, -0.0593544669060614, 0.0167212267476003, 0.00431740218144093, -0.0074779599349644, 0.00321800158796927, -0.0224338798048931, 0.018708742786244, -0.046301776120698, 0.0431830154655461, 0.0461140862656136, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, -0.0620433733855302, 0.017478740331497, 0.00451299134777145, -0.00781673030845891, 0.0363288795677472, 0.00938007637015065, -0.0162467688519772, 0.0119136893323295, -0.020635115229186, -0.0182390373864041},
    {0.0787556170769582, 0, -0.0348575579824558, 0.0585390371093696, -0.0282378493330016, 0, 0.00631417506469889, 0, -0.0206220094068229, 0.083846285486229, 0, 0, 0, 0, -0.00679184789949391, 0, 0.00151870335961076, 0, -0.0616465147138344, -0.0319414381914382, 0, 0, 0, 0, 0, 0, 0, 0, 0, -0.0450520222315696, 0, 0.0100739384233621, 0, -0.0329013451166065, -0.0643935438844154, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, -0.0565115904387179, 0, 0.01263637577376, 0, 0.027513486972248, 0.0303983956440067},
    {0.0787556170769578, -0.0377344134083695, -0.0217859737390347, -0.0523770332031201, 0.0423567739995023, 0.0546823600997146, 0, 0.0446479600584757, 0.0257775117585286, 0.0291639253865144, 0.0417574386216002, 0.0285692908380655, 0.0273366604729936, 0.0187890812890813, -0.00921750786359888, -0.00789141414141413, -0.00347132196482459, -0.0248527774227189, -0.0143487577351166, -0.015031265031265, -0.0557374224920008, -0.00491557862111907, -0.0197335053839337, -0.0394205194526234, -0.0325134964536672, -0.0241814494802472, -0.00928957041533347, -0.0158304746652834, -0.0141250354327105, -0.00241350119097692, -0.000623163327913714, -0.00215870109072046, 0.00712335017241511, 0.00411266813957583, 0.00540074884191872, 0, -0.0510586685671264, -0.0405265617908061, -0.0114170653756727, 0.010317557614593, 0.0127646671417816, 0.00466099742173251, -0.0143871467126538, -0.00564541633388696, 0.00571843474677503, 0.00737991258440982, 0.00367664434877214, -0.00155366580724416, 0.00240692871881144, 0.00347410235931507, 0.00100913554354852, 0.000260557676948644, 0.000902598269554298, -0.000850977809452118, -0.000491312267361587, -0.000868525589828769},
    {-0.0214788046573518, 0, -0.0217859737390347, 0.00308100195312481, -0.127070321998507, 0, 0.0473563129852414, 0, 0.0257775117585285, 0.0127592173566, 0, -0.0352914769176103, 0, 0.00225468975468975, 0.0509388592462044, 0, -0.00943765659186686, 0, -0.0143487577351166, -0.0129644660894661, 0.0836061337380013, 0, 0.0477758551400502, 0, -0.0325134964536671, 0, 0.00928957041533347, 0, -0.0141250354327104, -0.0144810071458617, 0, -0.00215870109072047, 0, 0.00411266813957582, 0.00540074884191876, 0, 0.0765880028506897, 0, -0.0228341307513453, 0, 0.0127646671417816, -0.00699149613259876, 0, -0.0112908326677739, 0, 0.00737991258440979, 0, -0.00310733161448836, 0, 0.00347410235931506, 0.00201827108709705, 0, 0.000902598269554284, 0, -0.000491312267361592, -0.000868525589828792},
    {0.0787556170769577, 0.0377344134083696, -0.0217859737390349, -0.05237703320312, 0.0423567739995023, -0.0546823600997146, 0, -0.0446479600584756, 0.0257775117585286, 0.0291639253865144, -0.0417574386216002, 0.0285692908380655, -0.0273366604729936, 0.0187890812890813, -0.0092175078635989, 0.00789141414141413, -0.00347132196482458, 0.0248527774227189, -0.0143487577351166, -0.015031265031265, -0.0557374224920009, 0.00491557862111903, -0.0197335053839338, 0.0394205194526234, -0.0325134964536671, 0.0241814494802472, -0.00928957041533348, 0.0158304746652834, -0.0141250354327105, -0.00241350119097695, 0.000623163327913703, -0.00215870109072045, -0.00712335017241511, 0.00411266813957582, 0.00540074884191871, 0, -0.0510586685671264, 0.0405265617908061, -0.0114170653756726, -0.010317557614593, 0.0127646671417816, 0.0046609974217325, 0.0143871467126537, -0.00564541633388698, -0.00571843474677501, 0.00737991258440984, -0.00367664434877218, -0.00155366580724417, -0.00240692871881142, 0.00347410235931505, 0.00100913554354854, -0.000260557676948633, 0.000902598269554291, 0.000850977809452119, -0.000491312267361577, -0.000868525589828757},
    {-0.0214788046573522, -0.0188672067041847, 0.0108929868695173, 0.00308100195312474, 0.0141189246665008, -0.109364720199429, -0.0789271883087357, 0.0223239800292378, -0.0128887558792642, 0.0127592173566001, -0.00894802256177151, -0.0277290175781224, -0.00585785581564148, 0.0191648629148629, 0.00145539597846299, 0.0383297258297258, 0.0348216984596467, -0.0124263887113595, 0.00717437886755833, -0.0129644660894661, 0, 0.0589869434534289, 0.00727023882565978, 0.0506835250105158, 0.0650269929073344, -0.00659494076734013, 0.0148633126645336, -0.00431740218144089, -0.00166176887443655, -0.0040225019849616, -0.00810112326287806, -0.0115130724838424, 0.00356167508620753, -0.00205633406978792, 0.00540074884191869, 0, 0, 0.0607898426862092, 0.0399597288148543, -0.0154763364218895, -0.031911667854454, 0, -0.0123318400394175, 0.00303983956440066, -0.00302740663064559, -0.00776832903622087, 0.00137874163078958, -0.0042725809699215, 0.000902598269554286, 0.000868525589828745, 0.00100913554354854, 0.000781673030845884, 0.00180519653910853, -0.000425488904726043, 0.0002456561336808, -0.000868525589828759},
    {-0.0214788046573523, 0.0188672067041848, 0.0108929868695173, 0.00308100195312473, 0.0141189246665008, 0.109364720199429, -0.0789271883087356, -0.0223239800292378, -0.0128887558792643, 0.0127592173566, 0.00894802256177147, -0.0277290175781224, 0.00585785581564147, 0.0191648629148629, 0.00145539597846298, -0.0383297258297258, 0.0348216984596467, 0.0124263887113595, 0.00717437886755829, -0.0129644660894661, 0, -0.0589869434534289, 0.0072702388256598, -0.0506835250105158, 0.0650269929073343, 0.00659494076734013, 0.0148633126645336, 0.0043174021814409, -0.00166176887443651, -0.00402250198496155, 0.00810112326287803, -0.0115130724838425, -0.00356167508620755, -0.0020563340697879, 0.00540074884191873, 0, 0, -0.0607898426862092, 0.0399597288148542, 0.0154763364218895, -0.031911667854454, 0, 0.0123318400394175, 0.00303983956440069, 0.0030274066306456, -0.00776832903622087, -0.00137874163078958, -0.00427258096992146, -0.000902598269554311, 0.000868525589828782, 0.00100913554354855, -0.000781673030845884, 0.00180519653910857, 0.000425488904726043, 0.000245656133680809, -0.000868525589828767},
    {0.0787556170769581, 0, 0.0435719474780695, -0.0523770332031201, -0.0282378493330016, 0, 0.0631417506469885, 0, -0.0515550235170572, 0.0291639253865144, 0, -0.00672218607954481, 0, -0.0601250601250601, 0.000970263985641982, 0, -0.0125835421224891, 0, 0.0286975154702333, -0.015031265031265, 0, 0, -0.0436214329539588, 0, -0.0650269929073343, 0, 0.0111474844984002, 0, 0.031573608614294, -0.00160900079398462, 0, -0.0028782681209606, 0, -0.00822533627915164, 0.00540074884191871, 0, 0, 0, -0.0532796384198057, 0, 0.0425488904726053, 0, 0, 0.018239037386404, 0, -0.00388416451811043, 0, -0.00466099742173252, 0, -0.00347410235931507, 0.00067275702903234, 0, 0.00120346435940573, 0, 0.000982624534723144, -0.000868525589828764},
    {0.10739402328676, -0.188672067041848, -0.108929868695174, -0.0770250488281177, -0.0705946233325038, -0.0546823600997147, -0.0315708753234941, -0.0446479600584757, -0.0257775117585286, -0.0182274533665716, 0.0894802256177148, -0.0151249186789758, 0.058578558156415, 0.0744047619047618, -0.0130985638061668, -0.0281836219336219, 0.00976309302606916, 0.069035492840886, 0.0398576603753241, 0.0597492784992785, 0, 0.0737336793167861, 0.036351194128299, -0.0120675059548847, -0.0232239260383337, 0.0659494076734016, 0.00743165633226678, 0.0431740218144092, 0.0465295284842228, 0.0345935170706695, 0.0155790831978425, 0.0251848460584054, -0.0254405363300539, -0.0146881004984851, -0.0328199352701214, 0, 0, 0, 0, 0, 0, 0, 0.0678251202167962, 0.0334382352084074, -0.0111004909790339, -0.0213629048496074, -0.0137874163078956, 0.0194208225905522, -0.0090259826955429, -0.0191075629762328, -0.00908221989193678, -0.00234501909253769, -0.00812338442598858, 0.00340391123780841, 0.00196524906944627, 0.00607967912880134},
    {0.107394023286761, 0.188672067041848, -0.108929868695174, -0.0770250488281178, -0.0705946233325039, 0.0546823600997145, -0.0315708753234942, 0.0446479600584757, -0.0257775117585287, -0.0182274533665715, -0.0894802256177147, -0.0151249186789758, -0.0585785581564149, 0.0744047619047619, -0.0130985638061668, 0.0281836219336219, 0.00976309302606916, -0.0690354928408859, 0.039857660375324, 0.0597492784992785, 0, -0.0737336793167861, 0.036351194128299, 0.0120675059548847, -0.0232239260383338, -0.0659494076734015, 0.00743165633226678, -0.0431740218144092, 0.0465295284842228, 0.0345935170706695, -0.0155790831978424, 0.0251848460584053, 0.0254405363300539, -0.0146881004984851, -0.0328199352701214, 0, 0, 0, 0, 0, 0, 0, -0.0678251202167962, 0.0334382352084074, 0.0111004909790339, -0.0213629048496074, 0.0137874163078956, 0.0194208225905522, 0.00902598269554287, -0.0191075629762328, -0.0090822198919368, 0.00234501909253767, -0.00812338442598857, -0.0034039112378084, 0.00196524906944627, 0.00607967912880135},
    {0.107394023286761, 0, 0.217859737390348, -0.0770250488281178, 0, 0, -0.094712625970483, 0, 0.0515550235170572, -0.0182274533665716, 0, -0.0907495120738551, 0, -0.0946969696969696, 0.0232863356554077, 0, -0.0227805503941613, 0, -0.0797153207506479, 0.0597492784992786, 0, 0, -0.0727023882565981, 0, 0.0464478520766674, 0, -0.0483057661597341, 0, -0.0781031370985168, 0.0144810071458616, 0, 0.0431740218144092, 0, 0.0293762009969701, -0.0328199352701215, 0, 0, 0, 0, 0, 0, 0, 0, -0.0668764704168148, 0, 0.0427258096992148, 0, 0.0310733161448835, 0, 0.00694820471863007, -0.00605481326129118, 0, -0.0108311792346515, 0, -0.00393049813889257, 0.00607967912880135},
    {0.107394023286761, 0, 0, 0.231075146484353, 0, 0, 0, 0, 0, -0.109364720199429, 0, 0.0100832791193172, 0, -0.00450937950937963, -0.093145342621631, 0, -0.104139658944738, 0, 0, -0.09244227994228, 0, 0, 0, 0, 0, 0, -0.066884906990401, 0, 0.0299118397398574, -0.0579240285834467, 0, -0.0647610327216138, 0, 0, 0.0835038859404354, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, -0.0699149613259879, 0, 0.0312669212338356, 0.0242192530451648, 0, 0.0270779480866286, 0, 0, -0.018239037386404}};
    
    // Extract relevant coefficients
    const double coeff0_0 = coefficients0[dof][0];
    const double coeff0_1 = coefficients0[dof][1];
    const double coeff0_2 = coefficients0[dof][2];
    const double coeff0_3 = coefficients0[dof][3];
    const double coeff0_4 = coefficients0[dof][4];
    const double coeff0_5 = coefficients0[dof][5];
    const double coeff0_6 = coefficients0[dof][6];
    const double coeff0_7 = coefficients0[dof][7];
    const double coeff0_8 = coefficients0[dof][8];
    const double coeff0_9 = coefficients0[dof][9];
    const double coeff0_10 = coefficients0[dof][10];
    const double coeff0_11 = coefficients0[dof][11];
    const double coeff0_12 = coefficients0[dof][12];
    const double coeff0_13 = coefficients0[dof][13];
    const double coeff0_14 = coefficients0[dof][14];
    const double coeff0_15 = coefficients0[dof][15];
    const double coeff0_16 = coefficients0[dof][16];
    const double coeff0_17 = coefficients0[dof][17];
    const double coeff0_18 = coefficients0[dof][18];
    const double coeff0_19 = coefficients0[dof][19];
    const double coeff0_20 = coefficients0[dof][20];
    const double coeff0_21 = coefficients0[dof][21];
    const double coeff0_22 = coefficients0[dof][22];
    const double coeff0_23 = coefficients0[dof][23];
    const double coeff0_24 = coefficients0[dof][24];
    const double coeff0_25 = coefficients0[dof][25];
    const double coeff0_26 = coefficients0[dof][26];
    const double coeff0_27 = coefficients0[dof][27];
    const double coeff0_28 = coefficients0[dof][28];
    const double coeff0_29 = coefficients0[dof][29];
    const double coeff0_30 = coefficients0[dof][30];
    const double coeff0_31 = coefficients0[dof][31];
    const double coeff0_32 = coefficients0[dof][32];
    const double coeff0_33 = coefficients0[dof][33];
    const double coeff0_34 = coefficients0[dof][34];
    const double coeff0_35 = coefficients0[dof][35];
    const double coeff0_36 = coefficients0[dof][36];
    const double coeff0_37 = coefficients0[dof][37];
    const double coeff0_38 = coefficients0[dof][38];
    const double coeff0_39 = coefficients0[dof][39];
    const double coeff0_40 = coefficients0[dof][40];
    const double coeff0_41 = coefficients0[dof][41];
    const double coeff0_42 = coefficients0[dof][42];
    const double coeff0_43 = coefficients0[dof][43];
    const double coeff0_44 = coefficients0[dof][44];
    const double coeff0_45 = coefficients0[dof][45];
    const double coeff0_46 = coefficients0[dof][46];
    const double coeff0_47 = coefficients0[dof][47];
    const double coeff0_48 = coefficients0[dof][48];
    const double coeff0_49 = coefficients0[dof][49];
    const double coeff0_50 = coefficients0[dof][50];
    const double coeff0_51 = coefficients0[dof][51];
    const double coeff0_52 = coefficients0[dof][52];
    const double coeff0_53 = coefficients0[dof][53];
    const double coeff0_54 = coefficients0[dof][54];
    const double coeff0_55 = coefficients0[dof][55];
    
    // Compute value(s)
    *values = coeff0_0*basisvalue0 + coeff0_1*basisvalue1 + coeff0_2*basisvalue2 + coeff0_3*basisvalue3 + coeff0_4*basisvalue4 + coeff0_5*basisvalue5 + coeff0_6*basisvalue6 + coeff0_7*basisvalue7 + coeff0_8*basisvalue8 + coeff0_9*basisvalue9 + coeff0_10*basisvalue10 + coeff0_11*basisvalue11 + coeff0_12*basisvalue12 + coeff0_13*basisvalue13 + coeff0_14*basisvalue14 + coeff0_15*basisvalue15 + coeff0_16*basisvalue16 + coeff0_17*basisvalue17 + coeff0_18*basisvalue18 + coeff0_19*basisvalue19 + coeff0_20*basisvalue20 + coeff0_21*basisvalue21 + coeff0_22*basisvalue22 + coeff0_23*basisvalue23 + coeff0_24*basisvalue24 + coeff0_25*basisvalue25 + coeff0_26*basisvalue26 + coeff0_27*basisvalue27 + coeff0_28*basisvalue28 + coeff0_29*basisvalue29 + coeff0_30*basisvalue30 + coeff0_31*basisvalue31 + coeff0_32*basisvalue32 + coeff0_33*basisvalue33 + coeff0_34*basisvalue34 + coeff0_35*basisvalue35 + coeff0_36*basisvalue36 + coeff0_37*basisvalue37 + coeff0_38*basisvalue38 + coeff0_39*basisvalue39 + coeff0_40*basisvalue40 + coeff0_41*basisvalue41 + coeff0_42*basisvalue42 + coeff0_43*basisvalue43 + coeff0_44*basisvalue44 + coeff0_45*basisvalue45 + coeff0_46*basisvalue46 + coeff0_47*basisvalue47 + coeff0_48*basisvalue48 + coeff0_49*basisvalue49 + coeff0_50*basisvalue50 + coeff0_51*basisvalue51 + coeff0_52*basisvalue52 + coeff0_53*basisvalue53 + coeff0_54*basisvalue54 + coeff0_55*basisvalue55;
  }

  /// Evaluate all basis functions at given point in cell
  virtual void evaluate_basis_all(double* values,
                                  const double* coordinates,
                                  const ufc::cell& c) const
  {
    throw std::runtime_error("The vectorised version of evaluate_basis() is not yet implemented.");
  }

  /// Evaluate order n derivatives of basis function i at given point in cell
  virtual void evaluate_basis_derivatives(unsigned int i,
                                          unsigned int n,
                                          double* values,
                                          const double* coordinates,
                                          const ufc::cell& c) const
  {
    // Extract vertex coordinates
    const double * const * element_coordinates = c.coordinates;
    
    // Compute Jacobian of affine map from reference cell
    const double J_00 = element_coordinates[1][0] - element_coordinates[0][0];
    const double J_01 = element_coordinates[2][0] - element_coordinates[0][0];
    const double J_02 = element_coordinates[3][0] - element_coordinates[0][0];
    const double J_10 = element_coordinates[1][1] - element_coordinates[0][1];
    const double J_11 = element_coordinates[2][1] - element_coordinates[0][1];
    const double J_12 = element_coordinates[3][1] - element_coordinates[0][1];
    const double J_20 = element_coordinates[1][2] - element_coordinates[0][2];
    const double J_21 = element_coordinates[2][2] - element_coordinates[0][2];
    const double J_22 = element_coordinates[3][2] - element_coordinates[0][2];
      
    // Compute sub determinants
    const double d00 = J_11*J_22 - J_12*J_21;
    const double d01 = J_12*J_20 - J_10*J_22;
    const double d02 = J_10*J_21 - J_11*J_20;
    
    const double d10 = J_02*J_21 - J_01*J_22;
    const double d11 = J_00*J_22 - J_02*J_20;
    const double d12 = J_01*J_20 - J_00*J_21;
    
    const double d20 = J_01*J_12 - J_02*J_11;
    const double d21 = J_02*J_10 - J_00*J_12;
    const double d22 = J_00*J_11 - J_01*J_10;
      
    // Compute determinant of Jacobian
    double detJ = J_00*d00 + J_10*d10 + J_20*d20;
    
    // Compute inverse of Jacobian
    
    // Compute constants
    const double C0 = d00*(element_coordinates[0][0] - element_coordinates[2][0] - element_coordinates[3][0]) \
                    + d10*(element_coordinates[0][1] - element_coordinates[2][1] - element_coordinates[3][1]) \
                    + d20*(element_coordinates[0][2] - element_coordinates[2][2] - element_coordinates[3][2]);
    
    const double C1 = d01*(element_coordinates[0][0] - element_coordinates[1][0] - element_coordinates[3][0]) \
                    + d11*(element_coordinates[0][1] - element_coordinates[1][1] - element_coordinates[3][1]) \
                    + d21*(element_coordinates[0][2] - element_coordinates[1][2] - element_coordinates[3][2]);
    
    const double C2 = d02*(element_coordinates[0][0] - element_coordinates[1][0] - element_coordinates[2][0]) \
                    + d12*(element_coordinates[0][1] - element_coordinates[1][1] - element_coordinates[2][1]) \
                    + d22*(element_coordinates[0][2] - element_coordinates[1][2] - element_coordinates[2][2]);
    
    // Get coordinates and map to the UFC reference element
    double x = (C0 + d00*coordinates[0] + d10*coordinates[1] + d20*coordinates[2]) / detJ;
    double y = (C1 + d01*coordinates[0] + d11*coordinates[1] + d21*coordinates[2]) / detJ;
    double z = (C2 + d02*coordinates[0] + d12*coordinates[1] + d22*coordinates[2]) / detJ;
    
    // Map coordinates to the reference cube
    if (std::abs(y + z - 1.0) < 1e-14)
      x = 1.0;
    else
      x = -2.0 * x/(y + z - 1.0) - 1.0;
    if (std::abs(z - 1.0) < 1e-14)
      y = -1.0;
    else
      y = 2.0 * y/(1.0 - z) - 1.0;
    z = 2.0 * z - 1.0;
    
    // Compute number of derivatives
    unsigned int num_derivatives = 1;
    
    for (unsigned int j = 0; j < n; j++)
      num_derivatives *= 3;
    
    
    // Declare pointer to two dimensional array that holds combinations of derivatives and initialise
    unsigned int **combinations = new unsigned int *[num_derivatives];
        
    for (unsigned int j = 0; j < num_derivatives; j++)
    {
      combinations[j] = new unsigned int [n];
      for (unsigned int k = 0; k < n; k++)
        combinations[j][k] = 0;
    }
        
    // Generate combinations of derivatives
    for (unsigned int row = 1; row < num_derivatives; row++)
    {
      for (unsigned int num = 0; num < row; num++)
      {
        for (unsigned int col = n-1; col+1 > 0; col--)
        {
          if (combinations[row][col] + 1 > 2)
            combinations[row][col] = 0;
          else
          {
            combinations[row][col] += 1;
            break;
          }
        }
      }
    }
    
    // Compute inverse of Jacobian
    const double Jinv[3][3] ={{d00 / detJ, d10 / detJ, d20 / detJ}, {d01 / detJ, d11 / detJ, d21 / detJ}, {d02 / detJ, d12 / detJ, d22 / detJ}};
    
    // Declare transformation matrix
    // Declare pointer to two dimensional array and initialise
    double **transform = new double *[num_derivatives];
        
    for (unsigned int j = 0; j < num_derivatives; j++)
    {
      transform[j] = new double [num_derivatives];
      for (unsigned int k = 0; k < num_derivatives; k++)
        transform[j][k] = 1;
    }
    
    // Construct transformation matrix
    for (unsigned int row = 0; row < num_derivatives; row++)
    {
      for (unsigned int col = 0; col < num_derivatives; col++)
      {
        for (unsigned int k = 0; k < n; k++)
          transform[row][col] *= Jinv[combinations[col][k]][combinations[row][k]];
      }
    }
    
    // Reset values
    for (unsigned int j = 0; j < 1*num_derivatives; j++)
      values[j] = 0;
    
    // Map degree of freedom to element degree of freedom
    const unsigned int dof = i;
    
    // Generate scalings
    const double scalings_y_0 = 1;
    const double scalings_y_1 = scalings_y_0*(0.5 - 0.5*y);
    const double scalings_y_2 = scalings_y_1*(0.5 - 0.5*y);
    const double scalings_y_3 = scalings_y_2*(0.5 - 0.5*y);
    const double scalings_y_4 = scalings_y_3*(0.5 - 0.5*y);
    const double scalings_y_5 = scalings_y_4*(0.5 - 0.5*y);
    const double scalings_z_0 = 1;
    const double scalings_z_1 = scalings_z_0*(0.5 - 0.5*z);
    const double scalings_z_2 = scalings_z_1*(0.5 - 0.5*z);
    const double scalings_z_3 = scalings_z_2*(0.5 - 0.5*z);
    const double scalings_z_4 = scalings_z_3*(0.5 - 0.5*z);
    const double scalings_z_5 = scalings_z_4*(0.5 - 0.5*z);
    
    // Compute psitilde_a
    const double psitilde_a_0 = 1;
    const double psitilde_a_1 = x;
    const double psitilde_a_2 = 1.5*x*psitilde_a_1 - 0.5*psitilde_a_0;
    const double psitilde_a_3 = 1.66666666666667*x*psitilde_a_2 - 0.666666666666667*psitilde_a_1;
    const double psitilde_a_4 = 1.75*x*psitilde_a_3 - 0.75*psitilde_a_2;
    const double psitilde_a_5 = 1.8*x*psitilde_a_4 - 0.8*psitilde_a_3;
    
    // Compute psitilde_bs
    const double psitilde_bs_0_0 = 1;
    const double psitilde_bs_0_1 = 1.5*y + 0.5;
    const double psitilde_bs_0_2 = 0.111111111111111*psitilde_bs_0_1 + 1.66666666666667*y*psitilde_bs_0_1 - 0.555555555555556*psitilde_bs_0_0;
    const double psitilde_bs_0_3 = 0.05*psitilde_bs_0_2 + 1.75*y*psitilde_bs_0_2 - 0.7*psitilde_bs_0_1;
    const double psitilde_bs_0_4 = 0.0285714285714286*psitilde_bs_0_3 + 1.8*y*psitilde_bs_0_3 - 0.771428571428571*psitilde_bs_0_2;
    const double psitilde_bs_0_5 = 0.0185185185185185*psitilde_bs_0_4 + 1.83333333333333*y*psitilde_bs_0_4 - 0.814814814814815*psitilde_bs_0_3;
    const double psitilde_bs_1_0 = 1;
    const double psitilde_bs_1_1 = 2.5*y + 1.5;
    const double psitilde_bs_1_2 = 0.54*psitilde_bs_1_1 + 2.1*y*psitilde_bs_1_1 - 0.56*psitilde_bs_1_0;
    const double psitilde_bs_1_3 = 0.285714285714286*psitilde_bs_1_2 + 2*y*psitilde_bs_1_2 - 0.714285714285714*psitilde_bs_1_1;
    const double psitilde_bs_1_4 = 0.178571428571429*psitilde_bs_1_3 + 1.96428571428571*y*psitilde_bs_1_3 - 0.785714285714286*psitilde_bs_1_2;
    const double psitilde_bs_2_0 = 1;
    const double psitilde_bs_2_1 = 3.5*y + 2.5;
    const double psitilde_bs_2_2 = 1.02040816326531*psitilde_bs_2_1 + 2.57142857142857*y*psitilde_bs_2_1 - 0.551020408163265*psitilde_bs_2_0;
    const double psitilde_bs_2_3 = 0.578703703703704*psitilde_bs_2_2 + 2.29166666666667*y*psitilde_bs_2_2 - 0.712962962962963*psitilde_bs_2_1;
    const double psitilde_bs_3_0 = 1;
    const double psitilde_bs_3_1 = 4.5*y + 3.5;
    const double psitilde_bs_3_2 = 1.51234567901235*psitilde_bs_3_1 + 3.05555555555556*y*psitilde_bs_3_1 - 0.54320987654321*psitilde_bs_3_0;
    const double psitilde_bs_4_0 = 1;
    const double psitilde_bs_4_1 = 5.5*y + 4.5;
    const double psitilde_bs_5_0 = 1;
    
    // Compute psitilde_cs
    const double psitilde_cs_00_0 = 1;
    const double psitilde_cs_00_1 = 2*z + 1;
    const double psitilde_cs_00_2 = 0.3125*psitilde_cs_00_1 + 1.875*z*psitilde_cs_00_1 - 0.5625*psitilde_cs_00_0;
    const double psitilde_cs_00_3 = 0.155555555555556*psitilde_cs_00_2 + 1.86666666666667*z*psitilde_cs_00_2 - 0.711111111111111*psitilde_cs_00_1;
    const double psitilde_cs_00_4 = 0.09375*psitilde_cs_00_3 + 1.875*z*psitilde_cs_00_3 - 0.78125*psitilde_cs_00_2;
    const double psitilde_cs_00_5 = 0.0628571428571429*psitilde_cs_00_4 + 1.88571428571429*z*psitilde_cs_00_4 - 0.822857142857143*psitilde_cs_00_3;
    const double psitilde_cs_01_0 = 1;
    const double psitilde_cs_01_1 = 3*z + 2;
    const double psitilde_cs_01_2 = 0.777777777777778*psitilde_cs_01_1 + 2.33333333333333*z*psitilde_cs_01_1 - 0.555555555555556*psitilde_cs_01_0;
    const double psitilde_cs_01_3 = 0.428571428571429*psitilde_cs_01_2 + 2.14285714285714*z*psitilde_cs_01_2 - 0.714285714285714*psitilde_cs_01_1;
    const double psitilde_cs_01_4 = 0.275*psitilde_cs_01_3 + 2.0625*z*psitilde_cs_01_3 - 0.7875*psitilde_cs_01_2;
    const double psitilde_cs_02_0 = 1;
    const double psitilde_cs_02_1 = 4*z + 3;
    const double psitilde_cs_02_2 = 1.265625*psitilde_cs_02_1 + 2.8125*z*psitilde_cs_02_1 - 0.546875*psitilde_cs_02_0;
    const double psitilde_cs_02_3 = 0.733333333333333*psitilde_cs_02_2 + 2.44444444444444*z*psitilde_cs_02_2 - 0.711111111111111*psitilde_cs_02_1;
    const double psitilde_cs_03_0 = 1;
    const double psitilde_cs_03_1 = 5*z + 4;
    const double psitilde_cs_03_2 = 1.76*psitilde_cs_03_1 + 3.3*z*psitilde_cs_03_1 - 0.54*psitilde_cs_03_0;
    const double psitilde_cs_04_0 = 1;
    const double psitilde_cs_04_1 = 6*z + 5;
    const double psitilde_cs_05_0 = 1;
    const double psitilde_cs_10_0 = 1;
    const double psitilde_cs_10_1 = 3*z + 2;
    const double psitilde_cs_10_2 = 0.777777777777778*psitilde_cs_10_1 + 2.33333333333333*z*psitilde_cs_10_1 - 0.555555555555556*psitilde_cs_10_0;
    const double psitilde_cs_10_3 = 0.428571428571429*psitilde_cs_10_2 + 2.14285714285714*z*psitilde_cs_10_2 - 0.714285714285714*psitilde_cs_10_1;
    const double psitilde_cs_10_4 = 0.275*psitilde_cs_10_3 + 2.0625*z*psitilde_cs_10_3 - 0.7875*psitilde_cs_10_2;
    const double psitilde_cs_11_0 = 1;
    const double psitilde_cs_11_1 = 4*z + 3;
    const double psitilde_cs_11_2 = 1.265625*psitilde_cs_11_1 + 2.8125*z*psitilde_cs_11_1 - 0.546875*psitilde_cs_11_0;
    const double psitilde_cs_11_3 = 0.733333333333333*psitilde_cs_11_2 + 2.44444444444444*z*psitilde_cs_11_2 - 0.711111111111111*psitilde_cs_11_1;
    const double psitilde_cs_12_0 = 1;
    const double psitilde_cs_12_1 = 5*z + 4;
    const double psitilde_cs_12_2 = 1.76*psitilde_cs_12_1 + 3.3*z*psitilde_cs_12_1 - 0.54*psitilde_cs_12_0;
    const double psitilde_cs_13_0 = 1;
    const double psitilde_cs_13_1 = 6*z + 5;
    const double psitilde_cs_14_0 = 1;
    const double psitilde_cs_20_0 = 1;
    const double psitilde_cs_20_1 = 4*z + 3;
    const double psitilde_cs_20_2 = 1.265625*psitilde_cs_20_1 + 2.8125*z*psitilde_cs_20_1 - 0.546875*psitilde_cs_20_0;
    const double psitilde_cs_20_3 = 0.733333333333333*psitilde_cs_20_2 + 2.44444444444444*z*psitilde_cs_20_2 - 0.711111111111111*psitilde_cs_20_1;
    const double psitilde_cs_21_0 = 1;
    const double psitilde_cs_21_1 = 5*z + 4;
    const double psitilde_cs_21_2 = 1.76*psitilde_cs_21_1 + 3.3*z*psitilde_cs_21_1 - 0.54*psitilde_cs_21_0;
    const double psitilde_cs_22_0 = 1;
    const double psitilde_cs_22_1 = 6*z + 5;
    const double psitilde_cs_23_0 = 1;
    const double psitilde_cs_30_0 = 1;
    const double psitilde_cs_30_1 = 5*z + 4;
    const double psitilde_cs_30_2 = 1.76*psitilde_cs_30_1 + 3.3*z*psitilde_cs_30_1 - 0.54*psitilde_cs_30_0;
    const double psitilde_cs_31_0 = 1;
    const double psitilde_cs_31_1 = 6*z + 5;
    const double psitilde_cs_32_0 = 1;
    const double psitilde_cs_40_0 = 1;
    const double psitilde_cs_40_1 = 6*z + 5;
    const double psitilde_cs_41_0 = 1;
    const double psitilde_cs_50_0 = 1;
    
    // Compute basisvalues
    const double basisvalue0 = 0.866025403784439*psitilde_a_0*scalings_y_0*psitilde_bs_0_0*scalings_z_0*psitilde_cs_00_0;
    const double basisvalue1 = 2.73861278752583*psitilde_a_1*scalings_y_1*psitilde_bs_1_0*scalings_z_1*psitilde_cs_10_0;
    const double basisvalue2 = 1.58113883008419*psitilde_a_0*scalings_y_0*psitilde_bs_0_1*scalings_z_1*psitilde_cs_01_0;
    const double basisvalue3 = 1.11803398874989*psitilde_a_0*scalings_y_0*psitilde_bs_0_0*scalings_z_0*psitilde_cs_00_1;
    const double basisvalue4 = 5.1234753829798*psitilde_a_2*scalings_y_2*psitilde_bs_2_0*scalings_z_2*psitilde_cs_20_0;
    const double basisvalue5 = 3.96862696659689*psitilde_a_1*scalings_y_1*psitilde_bs_1_1*scalings_z_2*psitilde_cs_11_0;
    const double basisvalue6 = 2.29128784747792*psitilde_a_0*scalings_y_0*psitilde_bs_0_2*scalings_z_2*psitilde_cs_02_0;
    const double basisvalue7 = 3.24037034920393*psitilde_a_1*scalings_y_1*psitilde_bs_1_0*scalings_z_1*psitilde_cs_10_1;
    const double basisvalue8 = 1.87082869338697*psitilde_a_0*scalings_y_0*psitilde_bs_0_1*scalings_z_1*psitilde_cs_01_1;
    const double basisvalue9 = 1.3228756555323*psitilde_a_0*scalings_y_0*psitilde_bs_0_0*scalings_z_0*psitilde_cs_00_2;
    const double basisvalue10 = 7.93725393319377*psitilde_a_3*scalings_y_3*psitilde_bs_3_0*scalings_z_3*psitilde_cs_30_0;
    const double basisvalue11 = 6.70820393249937*psitilde_a_2*scalings_y_2*psitilde_bs_2_1*scalings_z_3*psitilde_cs_21_0;
    const double basisvalue12 = 5.19615242270663*psitilde_a_1*scalings_y_1*psitilde_bs_1_2*scalings_z_3*psitilde_cs_12_0;
    const double basisvalue13 = 3*psitilde_a_0*scalings_y_0*psitilde_bs_0_3*scalings_z_3*psitilde_cs_03_0;
    const double basisvalue14 = 5.80947501931113*psitilde_a_2*scalings_y_2*psitilde_bs_2_0*scalings_z_2*psitilde_cs_20_1;
    const double basisvalue15 = 4.5*psitilde_a_1*scalings_y_1*psitilde_bs_1_1*scalings_z_2*psitilde_cs_11_1;
    const double basisvalue16 = 2.59807621135332*psitilde_a_0*scalings_y_0*psitilde_bs_0_2*scalings_z_2*psitilde_cs_02_1;
    const double basisvalue17 = 3.67423461417477*psitilde_a_1*scalings_y_1*psitilde_bs_1_0*scalings_z_1*psitilde_cs_10_2;
    const double basisvalue18 = 2.12132034355964*psitilde_a_0*scalings_y_0*psitilde_bs_0_1*scalings_z_1*psitilde_cs_01_2;
    const double basisvalue19 = 1.5*psitilde_a_0*scalings_y_0*psitilde_bs_0_0*scalings_z_0*psitilde_cs_00_3;
    const double basisvalue20 = 11.1242977306435*psitilde_a_4*scalings_y_4*psitilde_bs_4_0*scalings_z_4*psitilde_cs_40_0;
    const double basisvalue21 = 9.81070843517429*psitilde_a_3*scalings_y_3*psitilde_bs_3_1*scalings_z_4*psitilde_cs_31_0;
    const double basisvalue22 = 8.2915619758885*psitilde_a_2*scalings_y_2*psitilde_bs_2_2*scalings_z_4*psitilde_cs_22_0;
    const double basisvalue23 = 6.42261628933256*psitilde_a_1*scalings_y_1*psitilde_bs_1_3*scalings_z_4*psitilde_cs_13_0;
    const double basisvalue24 = 3.70809924354783*psitilde_a_0*scalings_y_0*psitilde_bs_0_4*scalings_z_4*psitilde_cs_04_0;
    const double basisvalue25 = 8.77496438739212*psitilde_a_3*scalings_y_3*psitilde_bs_3_0*scalings_z_3*psitilde_cs_30_1;
    const double basisvalue26 = 7.41619848709566*psitilde_a_2*scalings_y_2*psitilde_bs_2_1*scalings_z_3*psitilde_cs_21_1;
    const double basisvalue27 = 5.74456264653803*psitilde_a_1*scalings_y_1*psitilde_bs_1_2*scalings_z_3*psitilde_cs_12_1;
    const double basisvalue28 = 3.3166247903554*psitilde_a_0*scalings_y_0*psitilde_bs_0_3*scalings_z_3*psitilde_cs_03_1;
    const double basisvalue29 = 6.42261628933256*psitilde_a_2*scalings_y_2*psitilde_bs_2_0*scalings_z_2*psitilde_cs_20_2;
    const double basisvalue30 = 4.9749371855331*psitilde_a_1*scalings_y_1*psitilde_bs_1_1*scalings_z_2*psitilde_cs_11_2;
    const double basisvalue31 = 2.87228132326901*psitilde_a_0*scalings_y_0*psitilde_bs_0_2*scalings_z_2*psitilde_cs_02_2;
    const double basisvalue32 = 4.06201920231798*psitilde_a_1*scalings_y_1*psitilde_bs_1_0*scalings_z_1*psitilde_cs_10_3;
    const double basisvalue33 = 2.34520787991171*psitilde_a_0*scalings_y_0*psitilde_bs_0_1*scalings_z_1*psitilde_cs_01_3;
    const double basisvalue34 = 1.6583123951777*psitilde_a_0*scalings_y_0*psitilde_bs_0_0*scalings_z_0*psitilde_cs_00_4;
    const double basisvalue35 = 14.6458185158768*psitilde_a_5*scalings_y_5*psitilde_bs_5_0*scalings_z_5*psitilde_cs_50_0;
    const double basisvalue36 = 13.2476412994918*psitilde_a_4*scalings_y_4*psitilde_bs_4_1*scalings_z_5*psitilde_cs_41_0;
    const double basisvalue37 = 11.6833214455479*psitilde_a_3*scalings_y_3*psitilde_bs_3_2*scalings_z_5*psitilde_cs_32_0;
    const double basisvalue38 = 9.87420882906575*psitilde_a_2*scalings_y_2*psitilde_bs_2_3*scalings_z_5*psitilde_cs_23_0;
    const double basisvalue39 = 7.64852927038918*psitilde_a_1*scalings_y_1*psitilde_bs_1_4*scalings_z_5*psitilde_cs_14_0;
    const double basisvalue40 = 4.41588043316392*psitilde_a_0*scalings_y_0*psitilde_bs_0_5*scalings_z_5*psitilde_cs_05_0;
    const double basisvalue41 = 12.0933866224478*psitilde_a_4*scalings_y_4*psitilde_bs_4_0*scalings_z_4*psitilde_cs_40_1;
    const double basisvalue42 = 10.6653645038508*psitilde_a_3*scalings_y_3*psitilde_bs_3_1*scalings_z_4*psitilde_cs_31_1;
    const double basisvalue43 = 9.01387818865997*psitilde_a_2*scalings_y_2*psitilde_bs_2_2*scalings_z_4*psitilde_cs_22_1;
    const double basisvalue44 = 6.98212002188447*psitilde_a_1*scalings_y_1*psitilde_bs_1_3*scalings_z_4*psitilde_cs_13_1;
    const double basisvalue45 = 4.03112887414927*psitilde_a_0*scalings_y_0*psitilde_bs_0_4*scalings_z_4*psitilde_cs_04_1;
    const double basisvalue46 = 9.53939201416946*psitilde_a_3*scalings_y_3*psitilde_bs_3_0*scalings_z_3*psitilde_cs_30_2;
    const double basisvalue47 = 8.06225774829855*psitilde_a_2*scalings_y_2*psitilde_bs_2_1*scalings_z_3*psitilde_cs_21_2;
    const double basisvalue48 = 6.2449979983984*psitilde_a_1*scalings_y_1*psitilde_bs_1_2*scalings_z_3*psitilde_cs_12_2;
    const double basisvalue49 = 3.60555127546399*psitilde_a_0*scalings_y_0*psitilde_bs_0_3*scalings_z_3*psitilde_cs_03_2;
    const double basisvalue50 = 6.98212002188447*psitilde_a_2*scalings_y_2*psitilde_bs_2_0*scalings_z_2*psitilde_cs_20_3;
    const double basisvalue51 = 5.40832691319598*psitilde_a_1*scalings_y_1*psitilde_bs_1_1*scalings_z_2*psitilde_cs_11_3;
    const double basisvalue52 = 3.1224989991992*psitilde_a_0*scalings_y_0*psitilde_bs_0_2*scalings_z_2*psitilde_cs_02_3;
    const double basisvalue53 = 4.41588043316392*psitilde_a_1*scalings_y_1*psitilde_bs_1_0*scalings_z_1*psitilde_cs_10_4;
    const double basisvalue54 = 2.54950975679639*psitilde_a_0*scalings_y_0*psitilde_bs_0_1*scalings_z_1*psitilde_cs_01_4;
    const double basisvalue55 = 1.80277563773199*psitilde_a_0*scalings_y_0*psitilde_bs_0_0*scalings_z_0*psitilde_cs_00_5;
    
    // Table(s) of coefficients
    const static double coefficients0[56][56] = \
    {{0.00945067404923504, 0.00609788120679252, 0.003520613356228, 0.00248944957812473, 0.00705946233325038, 0.00546823600997147, 0.00315708753234945, 0.00446479600584757, 0.00257775117585287, 0.00182274533665719, -0.00298267418725718, -0.0025208197798293, -0.00195261860521383, -0.0011273448773449, -0.00218309396769446, -0.00169101731601731, -0.000976309302606924, -0.00138070985681772, -0.000797153207506492, -0.000563672438672437, 0.00278687112460006, 0.00245778931055954, 0.00207721109304565, 0.00160900079398463, 0.000928957041533348, 0.00219831358911339, 0.00185791408306671, 0.00143913406048031, 0.000830884437218262, 0.00160900079398463, 0.00124632665582739, 0.000719567030240153, 0.00101762145320216, 0.000587524019939396, 0.000415442218609115, -0.00705593524717797, -0.0063823335708908, -0.00562868913761196, -0.00475711057319694, -0.00368484200521179, -0.00212744452363026, -0.00582624677716564, -0.00513826668309063, -0.00434262794914383, -0.00336378514516178, -0.00194208225905521, -0.00459580543596519, -0.00388416451811043, -0.00300866089851429, -0.00173705117965752, -0.00336378514516179, -0.00260557676948628, -0.00150433044925714, -0.00212744452363026, -0.00122828066840392, -0.000868525589828755},
    {0.00945067404923484, -0.0060978812067926, 0.00352061335622807, 0.00248944957812473, 0.00705946233325039, -0.00546823600997146, 0.00315708753234942, -0.00446479600584756, 0.00257775117585286, 0.00182274533665715, 0.00298267418725716, -0.00252081977982931, 0.00195261860521383, -0.00112734487734489, -0.00218309396769446, 0.00169101731601731, -0.000976309302606908, 0.00138070985681772, -0.000797153207506487, -0.000563672438672423, 0.00278687112460003, -0.00245778931055954, 0.00207721109304565, -0.00160900079398463, 0.000928957041533354, -0.00219831358911339, 0.00185791408306669, -0.00143913406048031, 0.000830884437218255, 0.00160900079398463, -0.0012463266558274, 0.000719567030240159, -0.00101762145320216, 0.000587524019939409, 0.000415442218609113, 0.00705593524717798, -0.00638233357089081, 0.00562868913761196, -0.00475711057319694, 0.00368484200521178, -0.00212744452363027, -0.00582624677716566, 0.00513826668309062, -0.00434262794914383, 0.00336378514516178, -0.00194208225905522, 0.0045958054359652, -0.00388416451811043, 0.00300866089851429, -0.00173705117965753, -0.00336378514516178, 0.00260557676948629, -0.00150433044925715, 0.00212744452363026, -0.00122828066840393, -0.000868525589828756},
    {0.00945067404923496, 0, -0.00704122671245609, 0.00248944957812476, 0, 0, 0.00947126259704828, 0, -0.00515550235170569, 0.00182274533665715, 0, 0, 0, 0.0045093795093795, 0, 0, -0.00292892790782073, 0, 0.00159430641501296, -0.000563672438672431, 0, 0, 0, 0, 0.00464478520766673, 0, 0, 0, -0.00332353774887306, 0, 0, 0.00215870109072046, 0, -0.00117504803987882, 0.000415442218609132, 0, 0, 0, 0, 0, 0.0127646671417816, 0, 0, 0, 0, -0.00971041129527609, 0, 0, 0, 0.0069482047186301, 0, 0, -0.00451299134777144, 0, 0.00245656133680787, -0.000868525589828766},
    {0.00945067404923495, 0, 0, -0.00746834873437432, 0, 0, 0, 0, 0, 0.0109364720199429, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0.00563672438672439, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0.00623163327913698, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0.0182390373864041},
    {-0.0100234421734309, 0, 0.0374718748311398, -0.0289614183593723, 0, 0, 0.0284137877911449, 0, 0.00515550235170565, -0.00619733414463432, 0, 0, 0, 0.0315656565656566, 0, 0, -0.0113902751970807, 0, 0.00265717735835495, -0.000187890812890785, 0, 0, 0, 0, 0.0232239260383336, 0, 0, 0, -0.00166176887443649, 0, 0, -0.00647610327216139, 0, 0.00646276421933346, -0.00290809553026393, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0.0213629048496073, 0, 0, 0, -0.0277928188745204, 0, 0, 0.0243701532779658, 0, -0.0157219925555703, 0.00607967912880134},
    {0.0100234421734308, 0, -0.0287574853355261, 0.05360943398437, 0, 0, -0.0189425251940965, 0, 0.0154665070551172, 0.0153110608279201, 0, 0, 0, -0.00450937950937953, 0, 0, 0.0370997534990628, 0, -0.0281660799985623, 0.0108976671476671, 0, 0, 0, 0, 0, 0, 0, 0, 0.0299118397398575, 0, 0, -0.00431740218144094, 0, -0.0123380044187275, 0.00872428659079179, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0.0312669212338355, 0, 0, -0.0487403065559315, 0, 0.0412702304583721, -0.018239037386404},
    {0.0100234421734309, 0, 0.0409576306293854, -0.0449826285156208, 0, 0, 0.0189425251940966, 0, -0.00515550235170581, -0.021143845905223, 0, 0, 0, 0, 0, 0, 0.00455611007883228, 0, 0.0356061766019561, -0.0319414381914382, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0.0302218152700864, 0, 0.00411266813957589, -0.0145404776513196, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0.0379091273212801, 0, -0.0550269739444961, 0.0303983956440067},
    {-0.0100234421734309, 0, -0.0148144621425437, 0.0449826285156208, 0, 0, 0, 0, 0.0206220094068229, 0.021143845905223, 0, 0, 0, 0, 0, 0, 0, 0, 0.0106287094334197, 0.0319414381914382, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0.0205633406978791, 0.0145404776513196, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0.03439185871531, -0.0303983956440067},
    {-0.0100234421734309, 0.0324515955311978, -0.01873593741557, -0.0289614183593723, 0.0211783869997511, -0.0164047080299144, 0.00947126259704827, 0.00446479600584751, -0.00257775117585286, -0.00619733414463429, 0.0208787193108001, -0.0176457384588051, 0.0136683302364968, -0.00789141414141412, -0.00848980987436741, 0.00657617845117844, -0.00379675839902691, 0.0023011830946962, -0.00132858867917746, -0.000187890812890819, 0.0139343556230002, -0.0122889465527977, 0.0103860554652283, -0.00804500396992315, 0.00464478520766672, -0.00109915679455667, 0.000928957041533349, -0.000719567030240153, 0.000415442218609154, -0.00482700238195389, 0.00373897996748218, -0.00215870109072046, 0.00559691799261187, -0.00323138210966673, -0.00290809553026394, 0, 0, 0, 0, 0, 0, 0.0128177429097644, -0.0113041867027994, 0.00955378148811641, -0.00740032731935592, 0.00427258096992148, -0.0183832217438608, 0.0155366580724417, -0.0120346435940572, 0.00694820471863012, 0.0181644397838736, -0.014070114555226, 0.00812338442598859, -0.0136156449512337, 0.00786099627778517, 0.00607967912880136},
    {0.0100234421734311, -0.024904712849524, 0.014378742667763, 0.0536094339843701, -0.0141189246665008, 0.0109364720199429, -0.00631417506469887, 0.0133943880175428, -0.00773325352755854, 0.01531106082792, -0.00298267418725714, 0.0025208197798293, -0.00195261860521383, 0.00112734487734485, 0.0276525235907967, -0.0214195526695527, 0.0123665844996876, -0.0243925408037797, 0.0140830399992812, 0.0108976671476671, 0, 0, 0, 0, 0, 0.0197848223020205, -0.0167212267476003, 0.0129522065443228, -0.0074779599349644, -0.00321800158796925, 0.00249265331165481, -0.00143913406048031, -0.0106850252586226, 0.00616900220936375, 0.00872428659079183, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0.0206811244618434, -0.017478740331497, 0.0135389740433143, -0.00781673030845889, -0.0363288795677472, 0.028140229110452, -0.0162467688519772, 0.0357410679969885, -0.0206351152291861, -0.0182390373864041},
    {0.0100234421734311, 0.0354703486038675, -0.0204788153146928, -0.0449826285156207, 0.0141189246665009, -0.0109364720199429, 0.00631417506469889, -0.0044647960058475, 0.00257775117585279, -0.021143845905223, 0, 0, 0, 0, 0.00339592394974698, -0.00263047138047136, 0.00151870335961076, 0.0308358534689291, -0.0178030883009781, -0.0319414381914382, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0.0225260111157848, -0.0174485731815835, 0.0100739384233621, 0.00356167508620756, -0.00205633406978792, -0.0145404776513196, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0.028255795219359, -0.0218868448636849, 0.01263637577376, -0.047654757329318, 0.0275134869722481, 0.0303983956440068},
    {-0.0100234421734311, -0.0128297005588457, 0.00740723107127181, 0.0449826285156207, 0, 0, 0, 0.0178591840233903, -0.0103110047034114, 0.0211438459052229, 0, 0, 0, 0, 0, 0, 0, 0.0092047323787848, -0.00531435471670985, 0.0319414381914382, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0.0178083754310377, -0.0102816703489396, 0.0145404776513196, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0.0297842233308237, -0.017195929357655, -0.0303983956440068},
    {-0.0100234421734311, 0.0324515955311977, -0.0335503995581136, -0.00801060507812428, 0.0211783869997512, 0.00109364720199428, -0.00568275755822898, -0.0169662248222208, 0.00567105258687634, 0.0083846285486229, 0.0208787193108001, -0.0109235523792603, 0.00499002532443531, -0.00187890812890811, -0.0162519217595033, 0.00958243145743145, -0.00466458889023307, 0.0108922666482287, -0.00558007245254534, -0.00469727032227035, 0.0139343556230002, -0.00245778931055952, -0.00415442218609131, 0.0064360031759385, -0.00464478520766676, -0.0120907247401236, 0.00464478520766674, -0.000719567030240154, -0.000415442218609135, 0.00965400476390777, -0.00498530662330959, 0.00215870109072046, -0.00661453944581401, 0.0032313821096667, 0.00290809553026394, 0, 0.0127646671417816, -0.0202632808954031, 0.0228341307513453, -0.020635115229186, 0.0127646671417816, -0.00116524935543312, -0.00719357335632687, 0.0112908326677739, -0.0114368694935501, 0.00737991258440983, 0.00183832217438608, 0.00310733161448834, -0.00481385743762287, 0.00347410235931505, -0.00201827108709706, -0.000521115353897253, 0.000902598269554298, 0.00170195561890421, -0.000491312267361565, -0.000868525589828764},
    {0.0100234421734313, -0.0249047128495239, 0.0553363732971484, -0.00431340273437463, -0.0141189246665008, 0.0153110608279201, 0.00883984509057839, 0.00803663281052565, -0.0159820572902878, 0.000729098134662871, -0.00298267418725714, 0.0277290175781224, -0.0301571095694136, 0.0191648629148629, -0.00145539597846295, -0.0116492303992304, 0.00976309302606917, 0.00352848074520082, 0.00132858867917745, -0.00263047138047135, 0, 0.0196623144844763, -0.00727023882565982, -0.00724050357293083, 0.00928957041533347, -0.00219831358911338, -0.0148633126645336, 0.0129522065443228, -0.00664707549774609, 0.00402250198496158, 0.00685479660705067, -0.00575653624192123, -0.00457929653940971, -0.000293762009969677, 0.0029080955302639, 0, 0, 0.0202632808954031, -0.0399597288148543, 0.0464290092656686, -0.031911667854454, 0, -0.00411061334647249, -0.00303983956440069, 0.00908221989193683, -0.00776832903622088, 0.000459580543596524, 0.00427258096992149, -0.00270779480866288, 0.000868525589828788, -0.00100913554354855, -0.00234501909253767, 0.00180519653910856, 0.00127646671417817, 0.000245656133680783, -0.000868525589828766},
    {0.0100234421734309, 0.0354703486038674, -0.0492363006502187, -0.00431340273437457, 0.0141189246665008, -0.00656188321196574, -0.016416855168217, -0.00982255121286468, 0.0149509568199466, 0.00072909813466289, 0, 0.00336109303977242, 0.0282044909641998, -0.0353234728234728, -0.000485131992821028, -0.0124007936507936, 0.00889526253486298, 0.00291483191994853, 0.00239145962251946, -0.0026304713804714, 0, 0, 0.0218107164769794, -0.00402250198496158, -0.00928957041533346, 0, -0.00557374224920008, -0.0143913406048031, 0.0141250354327105, 0.000804500396992295, 0.00934744991870546, -0.00287826812096058, -0.00254405363300538, -0.00381890612960614, 0.00290809553026392, 0, 0, 0, 0.0266398192099029, -0.0515877880729651, 0.0425488904726054, 0, 0, -0.00911951869320201, 0.0084094628629044, -0.0038841645181104, 0, 0.00233049871086625, 0.00300866089851431, -0.00347410235931508, -0.000336378514516169, -0.00286613444643491, 0.00120346435940571, 0.000850977809452102, 0.000982624534723146, -0.000868525589828757},
    {-0.0100234421734311, -0.0128297005588456, 0.0448791059024117, -0.00801060507812423, 0, 0.0174983552319087, 0.0132597676358676, -0.00357183680467805, -0.0175287079957994, 0.00838462854862291, 0, 0, 0.00867830491206144, 0.0225468975468975, 0, -0.00300625300625305, -0.019200749617936, 0.000613648825252313, 0.0122230158484327, -0.00469727032227032, 0, 0, 0, 0.0160900079398463, 0.00464478520766673, 0, 0, -0.00719567030240156, -0.0108014976838374, 0, 0.00249265331165479, 0.0107935054536023, -0.000508810726601082, -0.00734405024924251, 0.00290809553026393, 0, 0, 0, 0, 0.0257938940364825, -0.031911667854454, 0, 0, 0, -0.0134551405806471, 0.0135945758133865, 0, 0, 0.00601732179702858, -0.00347410235931504, 0, -0.00208446141558903, -0.000902598269554288, 0.000425488904726059, 0.0017195929357655, -0.000868525589828767},
    {-0.010023442173431, -0.0324515955311979, -0.0187359374155701, -0.0289614183593722, 0.0211783869997511, 0.0164047080299144, 0.00947126259704825, -0.0044647960058475, -0.00257775117585286, -0.00619733414463431, -0.0208787193108001, -0.0176457384588052, -0.0136683302364968, -0.00789141414141413, -0.00848980987436743, -0.00657617845117846, -0.00379675839902691, -0.00230118309469614, -0.00132858867917743, -0.000187890812890788, 0.0139343556230002, 0.0122889465527977, 0.0103860554652283, 0.00804500396992312, 0.00464478520766673, 0.00109915679455666, 0.00092895704153335, 0.000719567030240155, 0.000415442218609149, -0.0048270023819539, -0.00373897996748222, -0.00215870109072046, -0.00559691799261192, -0.00323138210966673, -0.00290809553026393, 0, 0, 0, 0, 0, 0, 0.0128177429097644, 0.0113041867027994, 0.00955378148811642, 0.00740032731935589, 0.00427258096992148, 0.0183832217438608, 0.0155366580724417, 0.0120346435940572, 0.00694820471863011, 0.0181644397838736, 0.014070114555226, 0.00812338442598858, 0.0136156449512337, 0.00786099627778515, 0.00607967912880134},
    {0.0100234421734311, 0.024904712849524, 0.014378742667763, 0.05360943398437, -0.0141189246665008, -0.0109364720199429, -0.00631417506469881, -0.0133943880175428, -0.00773325352755853, 0.01531106082792, 0.00298267418725717, 0.0025208197798293, 0.00195261860521382, 0.00112734487734485, 0.0276525235907967, 0.0214195526695527, 0.0123665844996876, 0.0243925408037797, 0.0140830399992811, 0.0108976671476671, 0, 0, 0, 0, 0, -0.0197848223020205, -0.0167212267476003, -0.0129522065443227, -0.0074779599349644, -0.00321800158796927, -0.00249265331165477, -0.00143913406048032, 0.0106850252586227, 0.00616900220936373, 0.00872428659079179, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, -0.0206811244618434, -0.017478740331497, -0.0135389740433143, -0.00781673030845888, -0.0363288795677472, -0.028140229110452, -0.0162467688519772, -0.0357410679969885, -0.020635115229186, -0.018239037386404},
    {0.0100234421734307, -0.0354703486038675, -0.0204788153146927, -0.0449826285156207, 0.0141189246665008, 0.0109364720199429, 0.00631417506469886, 0.00446479600584751, 0.00257775117585285, -0.0211438459052229, 0, 0, 0, 0, 0.00339592394974697, 0.00263047138047138, 0.00151870335961076, -0.0308358534689291, -0.017803088300978, -0.0319414381914382, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0.0225260111157848, 0.0174485731815835, 0.0100739384233621, -0.00356167508620757, -0.00205633406978789, -0.0145404776513196, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0.0282557952193589, 0.0218868448636849, 0.01263637577376, 0.047654757329318, 0.027513486972248, 0.0303983956440067},
    {-0.010023442173431, 0.0128297005588457, 0.00740723107127182, 0.0449826285156208, 0, 0, 0, -0.0178591840233902, -0.0103110047034114, 0.021143845905223, 0, 0, 0, 0, 0, 0, 0, -0.00920473237878481, -0.00531435471670985, 0.0319414381914381, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, -0.0178083754310377, -0.0102816703489396, 0.0145404776513196, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, -0.0297842233308237, -0.017195929357655, -0.0303983956440068},
    {-0.0100234421734306, -0.0324515955311977, -0.0335503995581136, -0.00801060507812422, 0.0211783869997512, -0.00109364720199428, -0.00568275755822895, 0.0169662248222208, 0.00567105258687631, 0.00838462854862288, -0.0208787193108001, -0.0109235523792604, -0.00499002532443533, -0.00187890812890813, -0.0162519217595033, -0.00958243145743147, -0.00466458889023305, -0.0108922666482287, -0.00558007245254534, -0.00469727032227036, 0.0139343556230002, 0.00245778931055952, -0.00415442218609133, -0.00643600317593853, -0.00464478520766674, 0.0120907247401236, 0.00464478520766673, 0.000719567030240151, -0.000415442218609145, 0.00965400476390776, 0.00498530662330958, 0.00215870109072046, 0.006614539445814, 0.00323138210966669, 0.00290809553026394, 0, 0.0127646671417816, 0.0202632808954031, 0.0228341307513453, 0.020635115229186, 0.0127646671417816, -0.00116524935543314, 0.00719357335632687, 0.0112908326677739, 0.01143686949355, 0.00737991258440983, -0.00183832217438609, 0.00310733161448835, 0.00481385743762286, 0.00347410235931504, -0.00201827108709706, 0.00052111535389726, 0.000902598269554298, -0.00170195561890421, -0.000491312267361563, -0.000868525589828763},
    {0.0100234421734315, 0.024904712849524, 0.0553363732971485, -0.00431340273437464, -0.0141189246665008, -0.0153110608279201, 0.00883984509057836, -0.00803663281052558, -0.0159820572902878, 0.000729098134662864, 0.00298267418725715, 0.0277290175781224, 0.0301571095694136, 0.0191648629148629, -0.00145539597846298, 0.0116492303992304, 0.00976309302606916, -0.00352848074520084, 0.00132858867917744, -0.00263047138047133, 0, -0.0196623144844763, -0.0072702388256598, 0.00724050357293084, 0.0092895704153335, 0.00219831358911338, -0.0148633126645336, -0.0129522065443228, -0.0066470754977461, 0.00402250198496159, -0.00685479660705068, -0.00575653624192122, 0.0045792965394097, -0.000293762009969664, 0.00290809553026391, 0, 0, -0.0202632808954031, -0.0399597288148543, -0.0464290092656686, -0.031911667854454, 0, 0.00411061334647249, -0.00303983956440066, -0.0090822198919368, -0.00776832903622086, -0.000459580543596519, 0.0042725809699215, 0.00270779480866287, 0.000868525589828796, -0.00100913554354854, 0.00234501909253766, 0.00180519653910856, -0.00127646671417816, 0.00024565613368078, -0.00086852558982877},
    {0.0100234421734312, -0.0354703486038675, -0.0492363006502186, -0.0043134027343746, 0.0141189246665008, 0.00656188321196572, -0.016416855168217, 0.0098225512128646, 0.0149509568199467, 0.000729098134662864, 0, 0.0033610930397724, -0.0282044909641998, -0.0353234728234728, -0.000485131992820996, 0.0124007936507936, 0.00889526253486298, -0.00291483191994851, 0.00239145962251946, -0.00263047138047141, 0, 0, 0.0218107164769794, 0.00402250198496156, -0.00928957041533348, 0, -0.0055737422492001, 0.0143913406048031, 0.0141250354327105, 0.00080450039699229, -0.00934744991870546, -0.00287826812096059, 0.00254405363300538, -0.00381890612960615, 0.00290809553026392, 0, 0, 0, 0.0266398192099029, 0.0515877880729651, 0.0425488904726054, 0, 0, -0.00911951869320204, -0.00840946286290444, -0.00388416451811041, 0, 0.00233049871086624, -0.0030086608985143, -0.00347410235931509, -0.000336378514516169, 0.00286613444643491, 0.00120346435940572, -0.000850977809452107, 0.000982624534723155, -0.000868525589828754},
    {-0.010023442173431, 0.0128297005588457, 0.0448791059024117, -0.00801060507812423, 0, -0.0174983552319087, 0.0132597676358676, 0.0035718368046781, -0.0175287079957995, 0.00838462854862291, 0, 0, -0.00867830491206147, 0.0225468975468975, 0, 0.00300625300625302, -0.019200749617936, -0.000613648825252327, 0.0122230158484327, -0.00469727032227032, 0, 0, 0, -0.0160900079398463, 0.00464478520766674, 0, 0, 0.00719567030240152, -0.0108014976838374, 0, -0.00249265331165479, 0.0107935054536023, 0.000508810726601078, -0.00734405024924251, 0.00290809553026392, 0, 0, 0, 0, -0.0257938940364826, -0.031911667854454, 0, 0, 0, 0.0134551405806471, 0.0135945758133865, 0, 0, -0.00601732179702859, -0.00347410235931504, 0, 0.00208446141558904, -0.000902598269554294, -0.000425488904726048, 0.0017195929357655, -0.000868525589828768},
    {-0.0100234421734308, -0.0452812960900435, -0.0113287063442981, -0.00801060507812425, -0.00141189246665006, 0.0164047080299144, 0.0145226026488073, 0.0133943880175427, 0.0118576554089232, 0.0083846285486229, -0.00894802256177148, -0.0142846454190327, -0.014536160727703, -0.00939454064454062, -0.0123708658169354, -0.0125886844636845, -0.00813591085505763, -0.0102786178229763, -0.00664294339588729, -0.00469727032227033, -0.00836061337380013, 0.00245778931055953, 0.00830884437218264, 0.00965400476390776, 0.00650269929073342, 0.00219831358911336, 0.00743165633226678, 0.00863480436288185, 0.00581619106052787, 0.00643600317593851, 0.00747795993496438, 0.00503696921168107, 0.00610572871921291, 0.00411266813957582, 0.00290809553026393, 0.0352796762358899, 0.0191470007126724, 0.00788016479265675, 0.000951422114639387, -0.00221090520312706, -0.00212744452363027, 0.017478740331497, 0.00719357335632688, 0.000868525589828766, -0.00201827108709706, -0.00194208225905523, 0.00643412761035128, 0.00077683290362208, -0.00180519653910858, -0.00173705117965754, 0.000672757029032357, -0.00156334606169177, -0.00150433044925715, -0.00127646671417815, -0.00122828066840394, -0.000868525589828763},
    {0.010023442173431, 0.0603750614533912, -0.00610007264692973, -0.0043134027343746, -0.00564756986660029, -0.0218729440398858, 0.00126283501293976, -0.0178591840233903, 0.00103110047034112, 0.00072909813466285, 0.0417574386216002, 0.0168054651988621, -0.000867830491206138, -0.00526094276094278, 0.0145539597846298, -0.000751563251563223, -0.00455611007883228, -0.000613648825252315, -0.00372004830169692, -0.00263047138047137, 0.00557374224920009, -0.0196623144844763, -0.0103860554652283, 0.00321800158796927, 0.00650269929073345, -0.0175865087129071, -0.00928957041533348, 0.00287826812096062, 0.00581619106052783, -0.00804500396992314, 0.0024926533116548, 0.00503696921168107, 0.00203524290640432, 0.00411266813957582, 0.00290809553026392, -0.0705593524717798, -0.0127646671417816, 0.00450295131008957, 0.00380568845855756, -0.000736968401042374, -0.00212744452363027, -0.0116524935543313, 0.0041106133464725, 0.00347410235931506, -0.000672757029032359, -0.0019420822590552, 0.00367664434877216, 0.00310733161448835, -0.000601732179702866, -0.00173705117965752, 0.00269102811612943, -0.000521115353897279, -0.00150433044925714, -0.000425488904726059, -0.00122828066840393, -0.000868525589828762},
    {0.010023442173431, -0.0603750614533913, -0.00610007264692972, -0.00431340273437459, -0.00564756986660034, 0.0218729440398858, 0.00126283501293975, 0.0178591840233902, 0.00103110047034114, 0.000729098134662857, -0.0417574386216002, 0.016805465198862, 0.000867830491206142, -0.00526094276094274, 0.0145539597846298, 0.000751563251563242, -0.00455611007883226, 0.000613648825252326, -0.0037200483016969, -0.00263047138047136, 0.0055737422492001, 0.0196623144844763, -0.0103860554652283, -0.00321800158796928, 0.00650269929073344, 0.0175865087129071, -0.00928957041533347, -0.00287826812096062, 0.00581619106052785, -0.00804500396992314, -0.0024926533116548, 0.00503696921168109, -0.00203524290640433, 0.00411266813957582, 0.00290809553026392, 0.0705593524717798, -0.0127646671417816, -0.00450295131008957, 0.00380568845855755, 0.000736968401042381, -0.00212744452363028, -0.0116524935543313, -0.00411061334647249, 0.00347410235931505, 0.00067275702903236, -0.00194208225905522, -0.00367664434877216, 0.00310733161448835, 0.000601732179702862, -0.00173705117965754, 0.00269102811612941, 0.000521115353897267, -0.00150433044925715, 0.000425488904726055, -0.00122828066840394, -0.00086852558982877},
    {-0.0100234421734311, 0.0452812960900435, -0.0113287063442981, -0.00801060507812424, -0.00141189246665007, -0.0164047080299144, 0.0145226026488074, -0.0133943880175427, 0.0118576554089232, 0.00838462854862294, 0.00894802256177145, -0.0142846454190327, 0.014536160727703, -0.00939454064454065, -0.0123708658169354, 0.0125886844636845, -0.00813591085505765, 0.0102786178229763, -0.00664294339588732, -0.00469727032227034, -0.00836061337380011, -0.00245778931055953, 0.00830884437218264, -0.00965400476390776, 0.00650269929073342, -0.00219831358911338, 0.00743165633226677, -0.00863480436288184, 0.00581619106052786, 0.00643600317593851, -0.00747795993496437, 0.00503696921168106, -0.0061057287192129, 0.00411266813957581, 0.00290809553026391, -0.0352796762358899, 0.0191470007126724, -0.00788016479265674, 0.000951422114639392, 0.00221090520312706, -0.00212744452363026, 0.017478740331497, -0.00719357335632687, 0.000868525589828777, 0.00201827108709706, -0.00194208225905521, -0.00643412761035126, 0.000776832903622074, 0.00180519653910858, -0.00173705117965752, 0.00067275702903235, 0.00156334606169178, -0.00150433044925714, 0.00127646671417815, -0.00122828066840393, -0.000868525589828756},
    {0.0787556170769578, 0.0679219441350652, -0.00435719474780692, -0.00308100195312471, 0.0705946233325039, -0.0328094160598287, 0.0126283501293978, -0.0267887760350854, 0.03608851646194, -0.00729098134662863, 0.0298267418725716, 0.0151249186789758, -0.0325436434202305, 0.0248015873015872, 0.0130985638061668, -0.0229226791726792, 0.0169226945785199, -0.0248527774227189, 0.0164744996218005, 0.0142797017797018, 0, 0.0245778931055953, -0.0363511941282991, 0.0362025178646542, -0.0232239260383337, 0.0219831358911338, -0.00743165633226679, 0, 0.00166176887443652, -0.0345935170706695, 0.0130864298861876, -0.00359783515120077, 0.0335815079556711, -0.014688100498485, -0.0178640154001927, 0, 0, 0, 0, 0, 0, 0, 0.0226083734055987, -0.0334382352084074, 0.0333014729371016, -0.0213629048496074, -0.00459580543596519, -0.0194208225905522, 0.0270779480866286, -0.0191075629762328, 0.0090822198919368, 0.00703505727761299, -0.00812338442598857, -0.0102117337134253, 0.00196524906944628, 0.00607967912880137},
    {-0.0214788046573525, 0.00377344134083702, 0.00217859737390337, -0.021567013671873, 0, 0.0984282481794862, -0.0852413633734345, -0.0312535720409329, -0.0180442582309699, 0.0309866707231716, 0, 0.0453747560369276, 0, -0.0202922077922077, -0.0116431678277039, 0.0236742424242425, -0.0335199527228374, -0.00138070985681772, -0.000797153207506455, 0.00394570707070704, 0, 0, 0.0363511941282991, -0.0603375297744235, 0.0464478520766673, 0, 0.0241528830798671, 0, -0.0108014976838374, -0.00724050357293081, -0.0392592896585629, 0.0259044130886455, 0.0188259968842399, 0.0108691943688789, -0.0178640154001926, 0, 0, 0, 0, 0, 0, 0, 0, 0.0334382352084074, -0.0555024548951693, 0.0427258096992148, 0, -0.0155366580724417, 0, 0.00694820471863008, 0.00302740663064561, 0.0164151336477636, -0.0108311792346514, -0.00680782247561684, -0.00393049813889257, 0.00607967912880131},
    {0.0787556170769581, 0.0301875307266957, 0.0610007264692977, -0.00308100195312485, 0, 0.0218729440398858, 0.0757701007763862, 0.0178591840233902, -0.0412440188136458, -0.00729098134662867, 0, 0, 0.0520698294723689, -0.00901875901875909, 0, -0.0127765752765753, 0.0286384062098029, 0.00184094647575695, -0.0297603864135753, 0.0142797017797018, 0, 0, 0, 0.0402250198496157, -0.0464478520766674, 0, 0, 0.014391340604803, 0.0182794576188018, 0, -0.0137095932141014, -0.0345392174515274, 0.00407048581280862, 0.036426489236243, -0.0178640154001927, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0.0370016365967796, -0.0427258096992148, 0, 0, -0.030086608985143, 0.0243187165152054, 0, 0.014070114555226, 0, -0.00340391123780844, -0.00982624534723146, 0.00607967912880138},
    {-0.0214788046573519, 0.00377344134083692, -0.0196073763651313, 0.00924300585937408, 0, -0.0109364720199429, 0.00947126259704824, 0.102690308134494, -0.0695992817480272, -0.060150596109686, 0, -0.00504163955965862, 0.0065087286840461, -0.00450937950937948, 0.0465726713108155, 0.0214195526695527, -0.0289638426440052, -0.00782402252196701, -0.0263060558477138, 0.000563672438672498, 0, 0, 0, 0, 0, 0, 0.0334424534952005, -0.0431740218144092, 0.0299118397398575, 0.0289620142917233, -0.00249265331165477, -0.00431740218144094, -0.0605484764655283, 0.0185070066280912, 0.0461140862656136, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0.0349574806629939, -0.0451299134777144, 0.0312669212338355, -0.0121096265225824, -0.028140229110452, 0.0270779480866286, 0.023827378664659, 0, -0.0182390373864041},
    {-0.0214788046573522, -0.0150937653633478, 0.0130715842434209, 0.00924300585937409, 0, -0.0109364720199429, 0.00947126259704823, -0.00892959201169517, 0.123732056440937, -0.060150596109686, 0, 0, -0.00650872868404615, 0.00676406926406924, 0, 0.0574945887445888, 0.0126920209338899, -0.026693723898476, 0.00637722566005184, 0.000563672438672442, 0, 0, 0, 0, 0, 0, 0, 0.0431740218144092, -0.0448677596097863, 0, 0.0199412264932383, 0.0215870109072046, -0.0142467003448301, -0.0616900220936373, 0.0461140862656136, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0.0451299134777144, -0.0469003818507533, 0, -0.0375203054806026, 0.0162467688519772, 0.0119136893323295, 0.0206351152291861, -0.0182390373864041},
    {0.078755617076958, 0.0301875307266957, 0.0174287789912279, 0.0585390371093694, 0, 0.0218729440398858, -0.0189425251940966, 0.0178591840233902, 0.0103110047034113, 0.083846285486229, 0, 0, 0, 0, 0, 0.00526094276094275, -0.00455611007883227, 0.0533874477969518, 0.0308232573569172, -0.0319414381914382, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0.0348971463631671, -0.0302218152700865, 0.0284934006896603, 0.0164506725583033, -0.0643935438844155, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0.0437736897273697, -0.0379091273212801, -0.023827378664659, -0.0137567434861241, 0.0303983956440068},
    {0.0787556170769583, -0.0679219441350652, -0.00435719474780694, -0.00308100195312467, 0.0705946233325039, 0.0328094160598287, 0.0126283501293977, 0.0267887760350853, 0.03608851646194, -0.00729098134662864, -0.0298267418725716, 0.0151249186789759, 0.0325436434202305, 0.0248015873015873, 0.0130985638061668, 0.0229226791726792, 0.0169226945785199, 0.0248527774227189, 0.0164744996218005, 0.0142797017797018, 0, -0.0245778931055954, -0.036351194128299, -0.0362025178646541, -0.0232239260383336, -0.0219831358911338, -0.00743165633226678, 0, 0.00166176887443654, -0.0345935170706695, -0.0130864298861876, -0.00359783515120077, -0.0335815079556711, -0.014688100498485, -0.0178640154001927, 0, 0, 0, 0, 0, 0, 0, -0.0226083734055987, -0.0334382352084074, -0.0333014729371016, -0.0213629048496074, 0.00459580543596522, -0.0194208225905522, -0.0270779480866286, -0.0191075629762328, 0.0090822198919368, -0.007035057277613, -0.00812338442598858, 0.0102117337134253, 0.00196524906944627, 0.00607967912880136},
    {-0.0214788046573522, -0.00377344134083698, 0.00217859737390352, -0.0215670136718729, 0, -0.0984282481794863, -0.0852413633734345, 0.031253572040933, -0.0180442582309699, 0.0309866707231716, 0, 0.0453747560369276, 0, -0.0202922077922078, -0.0116431678277039, -0.0236742424242425, -0.0335199527228375, 0.00138070985681774, -0.000797153207506485, 0.00394570707070704, 0, 0, 0.0363511941282991, 0.0603375297744236, 0.0464478520766674, 0, 0.024152883079867, 0, -0.0108014976838374, -0.0072405035729308, 0.0392592896585629, 0.0259044130886455, -0.0188259968842399, 0.0108691943688789, -0.0178640154001927, 0, 0, 0, 0, 0, 0, 0, 0, 0.0334382352084074, 0.0555024548951693, 0.0427258096992148, 0, -0.0155366580724417, 0, 0.0069482047186301, 0.0030274066306456, -0.0164151336477636, -0.0108311792346515, 0.00680782247561683, -0.00393049813889256, 0.00607967912880134},
    {0.078755617076958, -0.0301875307266958, 0.0610007264692976, -0.00308100195312472, 0, -0.0218729440398859, 0.0757701007763862, -0.0178591840233903, -0.0412440188136458, -0.00729098134662865, 0, 0, -0.0520698294723688, -0.00901875901875904, 0, 0.0127765752765752, 0.0286384062098029, -0.00184094647575696, -0.0297603864135752, 0.0142797017797018, 0, 0, 0, -0.0402250198496157, -0.0464478520766673, 0, 0, -0.0143913406048031, 0.0182794576188018, 0, 0.0137095932141014, -0.0345392174515273, -0.00407048581280864, 0.036426489236243, -0.0178640154001927, 0, 0, 0, 0, 0, 0, 0, 0, 0, -0.0370016365967795, -0.0427258096992148, 0, 0, 0.0300866089851429, 0.0243187165152054, 0, -0.014070114555226, 0, 0.00340391123780844, -0.00982624534723146, 0.00607967912880137},
    {-0.0214788046573522, -0.00377344134083677, -0.0196073763651313, 0.00924300585937403, 0, 0.0109364720199429, 0.00947126259704829, -0.102690308134494, -0.0695992817480271, -0.0601505961096861, 0, -0.00504163955965865, -0.00650872868404618, -0.00450937950937947, 0.0465726713108155, -0.0214195526695527, -0.0289638426440052, 0.00782402252196705, -0.0263060558477139, 0.000563672438672498, 0, 0, 0, 0, 0, 0, 0.0334424534952005, 0.0431740218144092, 0.0299118397398575, 0.0289620142917233, 0.00249265331165484, -0.00431740218144093, 0.0605484764655283, 0.0185070066280911, 0.0461140862656136, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0.0349574806629939, 0.0451299134777144, 0.0312669212338355, -0.0121096265225824, 0.028140229110452, 0.0270779480866286, -0.023827378664659, 0, -0.018239037386404},
    {-0.0214788046573521, 0.0150937653633479, 0.0130715842434208, 0.00924300585937423, 0, 0.010936472019943, 0.00947126259704825, 0.00892959201169522, 0.123732056440937, -0.060150596109686, 0, 0, 0.00650872868404613, 0.00676406926406924, 0, -0.0574945887445887, 0.0126920209338899, 0.0266937238984759, 0.00637722566005184, 0.000563672438672452, 0, 0, 0, 0, 0, 0, 0, -0.0431740218144092, -0.0448677596097863, 0, -0.0199412264932383, 0.0215870109072046, 0.0142467003448301, -0.0616900220936373, 0.0461140862656136, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, -0.0451299134777144, -0.0469003818507533, 0, 0.0375203054806026, 0.0162467688519772, -0.0119136893323295, 0.0206351152291861, -0.0182390373864041},
    {0.0787556170769579, -0.0301875307266956, 0.017428778991228, 0.0585390371093696, 0, -0.0218729440398858, -0.0189425251940965, -0.0178591840233903, 0.0103110047034114, 0.083846285486229, 0, 0, 0, 0, 0, -0.00526094276094277, -0.00455611007883227, -0.0533874477969518, 0.0308232573569172, -0.0319414381914383, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, -0.0348971463631671, -0.0302218152700865, -0.0284934006896603, 0.0164506725583033, -0.0643935438844154, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, -0.0437736897273697, -0.0379091273212801, 0.023827378664659, -0.0137567434861241, 0.0303983956440068},
    {0.078755617076958, -0.0377344134083697, -0.0566435317214906, -0.00308100195312479, 0.0423567739995023, 0.0546823600997145, 0.0378850503881931, 0.0446479600584756, 0.00515550235170578, -0.00729098134662856, 0.0417574386216002, -0.00504163955965862, -0.0247331689993752, -0.0202922077922078, 0.0295930515620807, 0.0101461038961039, 0.00216957622801541, 0.0266937238984759, 0.0132858867917747, 0.0142797017797018, -0.0557374224920008, -0.0245778931055953, -0.00519302773261415, 0.00402250198496156, 0.00464478520766674, -0.00219831358911332, -0.0130053985814669, -0.0158304746652834, -0.0108014976838375, -0.0168945083368386, -0.026796023100289, -0.0194283098164841, -0.0295110221428625, -0.0217383887377579, -0.0178640154001927, 0, 0, 0, 0, 0, 0, -0.0512709716390577, -0.0226083734055987, -0.00477689074405822, 0.00370016365967794, 0.00427258096992147, -0.0367664434877216, -0.00776832903622086, 0.00601732179702859, 0.00694820471863011, -0.00908221989193683, 0.00703505727761299, 0.00812338442598859, 0.00680782247561683, 0.00786099627778515, 0.00607967912880135},
    {-0.0214788046573522, 0, -0.00435719474780689, -0.0215670136718729, -0.127070321998507, 0, 0.0284137877911447, 0, 0.0360885164619399, 0.0309866707231715, 0, 0.0453747560369275, 0, -0.0202922077922078, -0.0422064833754266, 0, -0.00618329224984384, 0, 0.0015943064150129, 0.00394570707070703, 0.0836061337380012, 0, -0.0103860554652283, 0, 0.00464478520766673, 0, 0.024152883079867, 0, -0.0108014976838374, 0.043443021437585, 0, -0.0194283098164841, 0, -0.0217383887377578, -0.0178640154001926, 0, 0, 0, 0, 0, 0, 0.0769064574585866, 0, -0.00955378148811639, 0, 0.00427258096992146, 0, -0.0155366580724417, 0, 0.00694820471863015, -0.0181644397838736, 0, 0.00812338442598858, 0, 0.00786099627778515, 0.00607967912880134},
    {0.0787556170769581, 0.0377344134083697, -0.0566435317214905, -0.00308100195312477, 0.0423567739995024, -0.0546823600997146, 0.0378850503881931, -0.0446479600584756, 0.00515550235170579, -0.00729098134662863, -0.0417574386216002, -0.00504163955965864, 0.0247331689993752, -0.0202922077922078, 0.0295930515620807, -0.0101461038961039, 0.00216957622801542, -0.0266937238984759, 0.0132858867917747, 0.0142797017797018, -0.0557374224920008, 0.0245778931055954, -0.00519302773261416, -0.00402250198496156, 0.00464478520766676, 0.00219831358911337, -0.0130053985814669, 0.0158304746652834, -0.0108014976838375, -0.0168945083368386, 0.026796023100289, -0.0194283098164841, 0.0295110221428625, -0.0217383887377579, -0.0178640154001927, 0, 0, 0, 0, 0, 0, -0.0512709716390577, 0.0226083734055987, -0.00477689074405822, -0.00370016365967795, 0.00427258096992149, 0.0367664434877216, -0.00776832903622086, -0.00601732179702859, 0.00694820471863009, -0.00908221989193678, -0.00703505727761301, 0.00812338442598859, -0.00680782247561686, 0.00786099627778517, 0.00607967912880135},
    {-0.021478804657352, -0.0188672067041847, 0.00653579212171052, 0.00924300585937401, 0.0141189246665008, 0, -0.00315708753234946, -0.111619900146189, -0.05413277469291, -0.0601505961096861, -0.00894802256177153, -0.00252081977982931, 0.000650872868404653, 0.00112734487734485, -0.0276525235907967, 0.0360750360750361, 0.0374251899332651, -0.0188697013765088, 0.019928830187662, 0.000563672438672417, 0, 0, 0, 0, 0, 0.0593544669060614, 0.0167212267476003, -0.00431740218144091, -0.0074779599349644, 0.00321800158796925, 0.0224338798048931, 0.018708742786244, 0.0463017761206981, 0.0431830154655461, 0.0461140862656137, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0.0620433733855302, 0.017478740331497, -0.00451299134777146, -0.0078167303084589, 0.0363288795677472, -0.00938007637015066, -0.0162467688519772, -0.0119136893323295, -0.020635115229186, -0.018239037386404},
    {-0.021478804657352, 0.0188672067041848, 0.00653579212171056, 0.00924300585937413, 0.0141189246665008, 0, -0.00315708753234944, 0.111619900146189, -0.05413277469291, -0.060150596109686, 0.00894802256177151, -0.00252081977982929, -0.000650872868404627, 0.00112734487734488, -0.0276525235907967, -0.0360750360750361, 0.0374251899332651, 0.0188697013765088, 0.0199288301876619, 0.000563672438672459, 0, 0, 0, 0, 0, -0.0593544669060614, 0.0167212267476003, 0.00431740218144093, -0.0074779599349644, 0.00321800158796927, -0.0224338798048931, 0.018708742786244, -0.046301776120698, 0.0431830154655461, 0.0461140862656136, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, -0.0620433733855302, 0.017478740331497, 0.00451299134777145, -0.00781673030845891, 0.0363288795677472, 0.00938007637015065, -0.0162467688519772, 0.0119136893323295, -0.020635115229186, -0.0182390373864041},
    {0.0787556170769582, 0, -0.0348575579824558, 0.0585390371093696, -0.0282378493330016, 0, 0.00631417506469889, 0, -0.0206220094068229, 0.083846285486229, 0, 0, 0, 0, -0.00679184789949391, 0, 0.00151870335961076, 0, -0.0616465147138344, -0.0319414381914382, 0, 0, 0, 0, 0, 0, 0, 0, 0, -0.0450520222315696, 0, 0.0100739384233621, 0, -0.0329013451166065, -0.0643935438844154, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, -0.0565115904387179, 0, 0.01263637577376, 0, 0.027513486972248, 0.0303983956440067},
    {0.0787556170769578, -0.0377344134083695, -0.0217859737390347, -0.0523770332031201, 0.0423567739995023, 0.0546823600997146, 0, 0.0446479600584757, 0.0257775117585286, 0.0291639253865144, 0.0417574386216002, 0.0285692908380655, 0.0273366604729936, 0.0187890812890813, -0.00921750786359888, -0.00789141414141413, -0.00347132196482459, -0.0248527774227189, -0.0143487577351166, -0.015031265031265, -0.0557374224920008, -0.00491557862111907, -0.0197335053839337, -0.0394205194526234, -0.0325134964536672, -0.0241814494802472, -0.00928957041533347, -0.0158304746652834, -0.0141250354327105, -0.00241350119097692, -0.000623163327913714, -0.00215870109072046, 0.00712335017241511, 0.00411266813957583, 0.00540074884191872, 0, -0.0510586685671264, -0.0405265617908061, -0.0114170653756727, 0.010317557614593, 0.0127646671417816, 0.00466099742173251, -0.0143871467126538, -0.00564541633388696, 0.00571843474677503, 0.00737991258440982, 0.00367664434877214, -0.00155366580724416, 0.00240692871881144, 0.00347410235931507, 0.00100913554354852, 0.000260557676948644, 0.000902598269554298, -0.000850977809452118, -0.000491312267361587, -0.000868525589828769},
    {-0.0214788046573518, 0, -0.0217859737390347, 0.00308100195312481, -0.127070321998507, 0, 0.0473563129852414, 0, 0.0257775117585285, 0.0127592173566, 0, -0.0352914769176103, 0, 0.00225468975468975, 0.0509388592462044, 0, -0.00943765659186686, 0, -0.0143487577351166, -0.0129644660894661, 0.0836061337380013, 0, 0.0477758551400502, 0, -0.0325134964536671, 0, 0.00928957041533347, 0, -0.0141250354327104, -0.0144810071458617, 0, -0.00215870109072047, 0, 0.00411266813957582, 0.00540074884191876, 0, 0.0765880028506897, 0, -0.0228341307513453, 0, 0.0127646671417816, -0.00699149613259876, 0, -0.0112908326677739, 0, 0.00737991258440979, 0, -0.00310733161448836, 0, 0.00347410235931506, 0.00201827108709705, 0, 0.000902598269554284, 0, -0.000491312267361592, -0.000868525589828792},
    {0.0787556170769577, 0.0377344134083696, -0.0217859737390349, -0.05237703320312, 0.0423567739995023, -0.0546823600997146, 0, -0.0446479600584756, 0.0257775117585286, 0.0291639253865144, -0.0417574386216002, 0.0285692908380655, -0.0273366604729936, 0.0187890812890813, -0.0092175078635989, 0.00789141414141413, -0.00347132196482458, 0.0248527774227189, -0.0143487577351166, -0.015031265031265, -0.0557374224920009, 0.00491557862111903, -0.0197335053839338, 0.0394205194526234, -0.0325134964536671, 0.0241814494802472, -0.00928957041533348, 0.0158304746652834, -0.0141250354327105, -0.00241350119097695, 0.000623163327913703, -0.00215870109072045, -0.00712335017241511, 0.00411266813957582, 0.00540074884191871, 0, -0.0510586685671264, 0.0405265617908061, -0.0114170653756726, -0.010317557614593, 0.0127646671417816, 0.0046609974217325, 0.0143871467126537, -0.00564541633388698, -0.00571843474677501, 0.00737991258440984, -0.00367664434877218, -0.00155366580724417, -0.00240692871881142, 0.00347410235931505, 0.00100913554354854, -0.000260557676948633, 0.000902598269554291, 0.000850977809452119, -0.000491312267361577, -0.000868525589828757},
    {-0.0214788046573522, -0.0188672067041847, 0.0108929868695173, 0.00308100195312474, 0.0141189246665008, -0.109364720199429, -0.0789271883087357, 0.0223239800292378, -0.0128887558792642, 0.0127592173566001, -0.00894802256177151, -0.0277290175781224, -0.00585785581564148, 0.0191648629148629, 0.00145539597846299, 0.0383297258297258, 0.0348216984596467, -0.0124263887113595, 0.00717437886755833, -0.0129644660894661, 0, 0.0589869434534289, 0.00727023882565978, 0.0506835250105158, 0.0650269929073344, -0.00659494076734013, 0.0148633126645336, -0.00431740218144089, -0.00166176887443655, -0.0040225019849616, -0.00810112326287806, -0.0115130724838424, 0.00356167508620753, -0.00205633406978792, 0.00540074884191869, 0, 0, 0.0607898426862092, 0.0399597288148543, -0.0154763364218895, -0.031911667854454, 0, -0.0123318400394175, 0.00303983956440066, -0.00302740663064559, -0.00776832903622087, 0.00137874163078958, -0.0042725809699215, 0.000902598269554286, 0.000868525589828745, 0.00100913554354854, 0.000781673030845884, 0.00180519653910853, -0.000425488904726043, 0.0002456561336808, -0.000868525589828759},
    {-0.0214788046573523, 0.0188672067041848, 0.0108929868695173, 0.00308100195312473, 0.0141189246665008, 0.109364720199429, -0.0789271883087356, -0.0223239800292378, -0.0128887558792643, 0.0127592173566, 0.00894802256177147, -0.0277290175781224, 0.00585785581564147, 0.0191648629148629, 0.00145539597846298, -0.0383297258297258, 0.0348216984596467, 0.0124263887113595, 0.00717437886755829, -0.0129644660894661, 0, -0.0589869434534289, 0.0072702388256598, -0.0506835250105158, 0.0650269929073343, 0.00659494076734013, 0.0148633126645336, 0.0043174021814409, -0.00166176887443651, -0.00402250198496155, 0.00810112326287803, -0.0115130724838425, -0.00356167508620755, -0.0020563340697879, 0.00540074884191873, 0, 0, -0.0607898426862092, 0.0399597288148542, 0.0154763364218895, -0.031911667854454, 0, 0.0123318400394175, 0.00303983956440069, 0.0030274066306456, -0.00776832903622087, -0.00137874163078958, -0.00427258096992146, -0.000902598269554311, 0.000868525589828782, 0.00100913554354855, -0.000781673030845884, 0.00180519653910857, 0.000425488904726043, 0.000245656133680809, -0.000868525589828767},
    {0.0787556170769581, 0, 0.0435719474780695, -0.0523770332031201, -0.0282378493330016, 0, 0.0631417506469885, 0, -0.0515550235170572, 0.0291639253865144, 0, -0.00672218607954481, 0, -0.0601250601250601, 0.000970263985641982, 0, -0.0125835421224891, 0, 0.0286975154702333, -0.015031265031265, 0, 0, -0.0436214329539588, 0, -0.0650269929073343, 0, 0.0111474844984002, 0, 0.031573608614294, -0.00160900079398462, 0, -0.0028782681209606, 0, -0.00822533627915164, 0.00540074884191871, 0, 0, 0, -0.0532796384198057, 0, 0.0425488904726053, 0, 0, 0.018239037386404, 0, -0.00388416451811043, 0, -0.00466099742173252, 0, -0.00347410235931507, 0.00067275702903234, 0, 0.00120346435940573, 0, 0.000982624534723144, -0.000868525589828764},
    {0.10739402328676, -0.188672067041848, -0.108929868695174, -0.0770250488281177, -0.0705946233325038, -0.0546823600997147, -0.0315708753234941, -0.0446479600584757, -0.0257775117585286, -0.0182274533665716, 0.0894802256177148, -0.0151249186789758, 0.058578558156415, 0.0744047619047618, -0.0130985638061668, -0.0281836219336219, 0.00976309302606916, 0.069035492840886, 0.0398576603753241, 0.0597492784992785, 0, 0.0737336793167861, 0.036351194128299, -0.0120675059548847, -0.0232239260383337, 0.0659494076734016, 0.00743165633226678, 0.0431740218144092, 0.0465295284842228, 0.0345935170706695, 0.0155790831978425, 0.0251848460584054, -0.0254405363300539, -0.0146881004984851, -0.0328199352701214, 0, 0, 0, 0, 0, 0, 0, 0.0678251202167962, 0.0334382352084074, -0.0111004909790339, -0.0213629048496074, -0.0137874163078956, 0.0194208225905522, -0.0090259826955429, -0.0191075629762328, -0.00908221989193678, -0.00234501909253769, -0.00812338442598858, 0.00340391123780841, 0.00196524906944627, 0.00607967912880134},
    {0.107394023286761, 0.188672067041848, -0.108929868695174, -0.0770250488281178, -0.0705946233325039, 0.0546823600997145, -0.0315708753234942, 0.0446479600584757, -0.0257775117585287, -0.0182274533665715, -0.0894802256177147, -0.0151249186789758, -0.0585785581564149, 0.0744047619047619, -0.0130985638061668, 0.0281836219336219, 0.00976309302606916, -0.0690354928408859, 0.039857660375324, 0.0597492784992785, 0, -0.0737336793167861, 0.036351194128299, 0.0120675059548847, -0.0232239260383338, -0.0659494076734015, 0.00743165633226678, -0.0431740218144092, 0.0465295284842228, 0.0345935170706695, -0.0155790831978424, 0.0251848460584053, 0.0254405363300539, -0.0146881004984851, -0.0328199352701214, 0, 0, 0, 0, 0, 0, 0, -0.0678251202167962, 0.0334382352084074, 0.0111004909790339, -0.0213629048496074, 0.0137874163078956, 0.0194208225905522, 0.00902598269554287, -0.0191075629762328, -0.0090822198919368, 0.00234501909253767, -0.00812338442598857, -0.0034039112378084, 0.00196524906944627, 0.00607967912880135},
    {0.107394023286761, 0, 0.217859737390348, -0.0770250488281178, 0, 0, -0.094712625970483, 0, 0.0515550235170572, -0.0182274533665716, 0, -0.0907495120738551, 0, -0.0946969696969696, 0.0232863356554077, 0, -0.0227805503941613, 0, -0.0797153207506479, 0.0597492784992786, 0, 0, -0.0727023882565981, 0, 0.0464478520766674, 0, -0.0483057661597341, 0, -0.0781031370985168, 0.0144810071458616, 0, 0.0431740218144092, 0, 0.0293762009969701, -0.0328199352701215, 0, 0, 0, 0, 0, 0, 0, 0, -0.0668764704168148, 0, 0.0427258096992148, 0, 0.0310733161448835, 0, 0.00694820471863007, -0.00605481326129118, 0, -0.0108311792346515, 0, -0.00393049813889257, 0.00607967912880135},
    {0.107394023286761, 0, 0, 0.231075146484353, 0, 0, 0, 0, 0, -0.109364720199429, 0, 0.0100832791193172, 0, -0.00450937950937963, -0.093145342621631, 0, -0.104139658944738, 0, 0, -0.09244227994228, 0, 0, 0, 0, 0, 0, -0.066884906990401, 0, 0.0299118397398574, -0.0579240285834467, 0, -0.0647610327216138, 0, 0, 0.0835038859404354, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, -0.0699149613259879, 0, 0.0312669212338356, 0.0242192530451648, 0, 0.0270779480866286, 0, 0, -0.018239037386404}};
    
    // Interesting (new) part
    // Tables of derivatives of the polynomial base (transpose)
    const static double dmats0[56][56] = \
    {{0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {6.32455532033671, 0, 0, 0, 0, 2.16505338810566e-14, 0, 2.05845987409731e-14, -1.22512335797976e-14, 0, 0, 0, 0, 0, -1.08130730953582e-14, 0, 1.22483177886604e-14, 0, 1.21041660596838e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {-1.33727360005496e-14, 11.2249721603218, 0, 0, 0, 2.8506046608589e-14, 0, 0, 0, 1.00951115474313e-14, 0, -1.20188905885721e-14, 1.28545373789999e-14, 0, 0, -1.03476262582923e-14, -1.93999491268998e-14, 0, 1.33226762955019e-14, 0, 0, 0, 0, 0, 1.05444038416428e-14, 0, 0, 0, -1.00620217474173e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1.11065094110678e-14, 0, 0, -1.90621990730363e-14, 0, 0, 1.25288297745861e-14, 0, 0, 0, 0, 0, 0, 0},
    {4.58257569495582, -1.0968053520467e-14, 8.36660026534075, -1.18321595661994, 0, 0, -1.51387532776558e-14, -1.08221538572235e-14, -1.3598597955937e-14, 0, 0, 1.9215300364784e-14, -1.53957147424772e-14, 0, 0, 1.50088321802921e-14, 0, 0, 0, -1.15463194561016e-14, 0, 0, 0, 0, 0, 0, 0, -1.00882470358913e-14, 0, 0, 0, 0, 0, 0, 0, 0, 1.03041185216246e-14, 0, 0, 0, 0, 0, 0, 0, 1.67984122495906e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {3.74165738677394, 2.85608062261247e-14, 0, 8.69482604771366, 0, 1.32677370521961e-14, -1.0621406876971e-14, 4.08006961549745e-14, -2.2818496382273e-14, -2.1316282072803e-14, 0, 0, 0, 0, 0, -1.21576837027352e-14, 0, 1.41223042954593e-14, 1.22383070012588e-14, -1.4210854715202e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, -1.00375396573897e-14, -1.10000775654917e-14, 0, -1.24344978758018e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, -1.27154464053485e-14, 0, -1.33226762955019e-14},
    {0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {5.49909083394704, -2.12464983341981e-14, -3.34664010613631, -2.36643191323983, 15.4919333848297, 0, 0.692820323027548, 0, 0.565685424949223, 0.400000000000019, -2.45617804595055e-14, 0, -2.50285594161414e-14, -1.12752625494543e-14, 0, -2.06682744450517e-14, 0, -1.87825158914691e-14, 0, 0, 0, 1.67613011937654e-14, 0, 0, 0, 1.28202275780632e-14, 0, 1.30009839200864e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, -1.30459382160744e-14, 0, 0, 0, 0, -1.95146235819213e-14, 1.32110885228657e-14, 1.12246955104063e-14, 0, -1.24476276034392e-14, 0, 0, 0, 1.14879018579048e-14, 0, 0, 0, 0, 0},
    {1.79012076899773e-14, 4.89897948556635, 0, 0, 0, 14.1985914794391, -1.56346770721582e-14, -0.828078671210841, 1.85962356624714e-14, 1.44765989981636e-14, 0, -3.14433996145488e-14, 0, -1.75618818945982e-14, -1.31955099699921e-14, 1.85934501523347e-14, -2.7953864447905e-14, 1.28785870856518e-14, 0, 0, 0, 0, 1.3985009962376e-14, 0, 1.27337992225105e-14, 0, 1.07957902261976e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, -1.64679520687763e-14, 0, -1.14163879478899e-14, 0, 0, -1.9487535909084e-14, 1.99307532306351e-14, -2.46558752706505e-14, 0, 0, 0, 0, 0, 1.11086317443674e-14, 0, 0, 0, 0},
    {3.60000000000003, -1.17043800208557e-14, 8.76356092008264, -1.54919333848296, 0, 0, 9.52470471983252, -1.08725201100446e-14, -1.48131215963609, 0.261861468283195, 0, 0, -2.19166032081878e-14, -1.15727480921906e-14, 1.19782412678522e-14, 0, 2.00140531387981e-14, -1.08284340452512e-14, -2.62934736398987e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1.25371390349952e-14, 0, 0, 0, 0, 0, -1.02110492017178e-14, 0, 0, 0, 0, 0, 0},
    {0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {-2.24915431235802e-14, 4.24264068711927, -1.64313007644523e-14, 0, 0, 0, 0, 14.3427433120127, 0, 1.59620734906468e-14, 0, 0, 2.06494753825246e-14, 1.24642807480246e-14, 1.31854538806711e-14, 0, -2.70659809367255e-14, 0, 0, -1.23240404410758e-14, 0, 0, 0, 0, 0, 0, 0, 0, -1.22125558154546e-14, 1.05003199257295e-14, 1.39537903134404e-14, 1.71734205981969e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, -2.01310662662892e-14, 0, 0, 2.30351452905449e-14, 0, 0, 0, 0, 0, 1.24344978758018e-14, 0},
    {3.11769145362398, 0, 3.1622776601684, 4.91934955049952, 0, 0, 0, 0, 10.690449676497, -2.41897262725906, 0, 0, 0, 1.08320679531897e-14, 0, 0, 0, 1.61780317385165e-14, 0, -3.01980662698043e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, -1.01163085491832e-14, 0, -1.4210854715202e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1.90254351599586e-14, 0, 0, 0, 1.4669610635169e-14, 0, 0, 0, 1.20972007395846e-14, 0, 0, 0},
    {0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {2.54558441227159, 0, 0, 7.66811580507233, 0, 0, 0, 2.00435576705123e-14, 0, 10.3691851174526, 0, 0, 0, 0, 0, 0, -1.4355744756654e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, -1.31237403589518e-14, 0, 1.19255832274785e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, -1.21989789544854e-14, 0, 0, 0, 1.08631431175004e-14, 0, 0, 0, 0, 0, 0},
    {0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {-3.7592379574637e-14, 11.60576914948, 1.02695629777827e-14, 0, 0, -4.20458932936044, 1.40755215458979e-14, -3.43303281162799, 0, 0, 19.6214168703486, 2.18203847862408e-14, 0.611677741841185, 1.46622784386203e-14, 0, 0.529728463363973, 1.12642251693005e-14, 0.4325214791568, 0, 0, 0, -1.06703789507439e-14, 0, 0, 0, -1.60649692254054e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1.42508255988556e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {4.53137212479104, -2.04494956755329e-14, 3.54562104171167, -2.50713268211203, 4.78713553878169, 0, -5.56626745075489, 0, -1.04880884817014, 0.741619848709575, -1.23815345194237e-14, 18.8034951158403, -1.86020833228553e-14, 1.68183573174416, -0.603122387046451, -1.92746782669555e-14, 0.701283781217653, 0, 0.132137494528672, -0.0934353184302359, 0, 1.50506010183261e-14, 0, 0, 0, 1.11149238093515e-14, 0, 1.50741577923534e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, -1.5968916436806e-14, 0, 0, 0, 0, -2.36522650283687e-14, 1.26860679795035e-14, 2.21112666692775e-14, -1.04107559329747e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {3.86209880120533e-14, 2.5951288749407, 1.92068583260152e-14, 0, 0, 10.0285307284481, -1.22612314755764e-14, -0.767649473578743, 0, 0, 0, 0, 16.4130361329658, 0, 0, -1.2634761106116, 3.26893402596665e-14, 0.0967147429123276, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1.32878527298855e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, -1.0672045061505e-14, -1.16637535122366e-14, 0, -1.46741880066399e-14, 0, 1.32138836425176e-14, 0, 0, 0, -1.00831659427246e-14, 0, 0, 0, 0, 0, 0, 0},
    {2.96647939483827, 0, 8.12403840463593, -1.64130361329658, 0, 0, 11.7728501222092, 0, -2.40312296813959, 0.485504156227621, 0, 0, -1.71460988515286e-14, 10.2761860629321, 1.14956049951209e-14, -4.33332801490326e-14, -1.48323969741912, -1.03124879870092e-14, 0.302765035409724, -0.0611677741841183, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, -1.45169999216629e-14, 1.16636407607716e-14, -2.16593088290316e-14, 0, 0, 0, 0, -1.3706959798735e-14, 0, 0, 0, 0, 0, 0},
    {0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {4.05298244095222, -1.17512299762369e-14, -1.58564993434419, 4.48489528465113, 4.28174419288836, 0, 0.191485421551272, 0, -3.75233260785876, -4.311612227462, 0, 0, -1.13529814871025e-14, -1.07967706476489e-14, 18.8807171866293, 0, 0.844371341865059, -1.30185990386282e-14, 1.18187368057055, 1.04463861754669, 0, 1.00351859283595e-14, 0, 0, 0, 1.08982318008379e-14, 1.16650614658273e-14, 1.78540699708776e-14, 0, 1.48568586111212e-14, 1.2997651195193e-14, 0, 1.19564095870861e-14, 0, 0, 0, 0, 0, 0, 0, 0, -1.90031310112405e-14, -2.21313572206035e-14, 0, 2.31096376853888e-14, 1.25147787012743e-14, -1.31451704746157e-14, 0, 0, 0, 2.34657400816165e-14, 0, 0, 0, 1.19833135941395e-14, 0},
    {1.03681424251779e-14, 2.321153829896, -1.04360964314765e-14, 0, 0, 3.9242833740697, 0, 5.49285249860475, 0, 1.4249489099847e-14, 0, 0, 2.63684141763982e-14, 1.12010640489931e-14, 0, 17.3044631365566, -5.0266260224736e-14, -1.73008591662709, 0, 0, 0, 0, 0, 0, 1.32329702395699e-14, 0, 1.22894929912907e-14, -1.21668635551694e-14, -1.35421297848238e-14, 1.0253781445641e-14, 0, 1.86415806813024e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 2.41268605947696e-14, -3.37685959257577e-14, 0, 0, 3.2419745025707e-14, 1.21885930822672e-14, 0, 1.77958086184916e-14, 0, 1.4210854715202e-14, 0, 1.40814441510978e-14},
    {2.65329983228435, -2.32098024377698e-14, 4.15220619990512, 2.93605316083775, 0, -1.72381743916861e-14, 2.63248931621763, -4.00364084983131e-14, 9.82591326180784, -2.8226127713774, 0, 0, -1.33474731826653e-14, -1.06959103616322e-14, 1.26654039377864e-14, 1.8991099966876e-14, 11.6081867662439, -1.19927215290095e-14, -3.0948717731947, 0.683876505540241, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1.61148409137901e-14, 1.47100077738277e-14, 0, 0, 0, 0, 0, 0, 0, 1.3422167174583e-14, 0, -2.15269669668842e-14, 0},
    {0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {4.0410983709019e-14, 2.01017818278147, 0, 1.34362847627492e-14, 0, -2.17732253774783e-14, 0, 8.91928088068604, -1.37667655053519e-14, 1.38253977761529e-14, -1.02564879604251e-14, 0, 0, 0, -1.34085149628415e-14, 0, 0, 16.8558564884489, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1.06581410364015e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, -1.48981943277725e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1.77635683940025e-14, 0, 0},
    {2.29782505861523, 1.22923050669652e-14, 1.49829835452879, 5.50917601898536, 0, 0, 0, 1.92682707206214e-14, 6.64803945320939, 4.13676892548485, 0, 0, 1.46016900962614e-14, 2.50030852312528e-14, 0, -1.88410641965856e-14, -3.22541831300375e-14, -1.10413917361675e-14, 12.5636136423842, -3.55352656109507, 0, 0, 0, 0, 0, 0, 0, 0, -2.00805899834693e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, -1.13841034187579e-14, -2.03312427847762e-14, 0, 0, 0, 1.35488987469781e-14, 0, 0, 0, 0, -1.41455019594718e-14, 0},
    {0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {1.87616630392938, -2.83080569710049e-14, 0, 6.22830929985772, 0, 0, 0, -1.88930188353776e-14, 0, 11.0541524195339, 0, 1.07055222640856e-14, 0, 0, 0, 1.48136724622646e-14, 1.71418057121951e-14, -2.40309133366388e-14, 0, 11.6057691494799, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1.22846189228214e-14, 1.32604072649903e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1.33226762955019e-14},
    {0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {4.83186700722507, 0, -4.41087092453692, -3.11895674167863, 16.6749979177077, -1.13866638474088e-14, 2.13065307820926, -2.61564103865851e-14, 1.73967095350101, 1.23013312825381, 0, -4.36653943835008, 1.40983965277648e-14, -0.557935943461962, -3.78153408023779, 2.08111552908862e-14, -0.483186700722529, 1.18208720943676e-14, -0.394520289089675, -0.278967971730977, 23.6981011897578, 0, 0.490653381462656, 0, 0.0626933893908963, -1.13263908790947e-14, 0.438853725736244, 0, 0.0560746721671562, 0.380058475033041, 0, 0.0485620906056454, 0, 0.0396507809421992, 0.0280373360835894, 0, 0, 0, 0, 0, 0, 1.56357143454011e-14, 0, 0, 0, 0, 1.0335662317795e-14, 1.60193852711159e-14, 0, 0, 0, 1.11786220690221e-14, -1.25974506359966e-14, 0, -1.04950117655873e-14, 0},
    {-2.48352929483769e-14, 6.91050664139874, 0, -1.11694286112743e-14, 0, 10.5706241650195, 1.37569619238853e-14, -2.72554057547698, -1.5404344466674e-14, 0, 4.67332857821917, 1.98201416293709e-14, -7.86705867811461, 0, 0, -2.39719223514615, 0, 0.61809450436526, 0, 0, 0, 23.10555488766, 0, 1.62065746295907, 0, -0.469687194424188, 0, 0.790669146636838, 1.33915447641953e-14, 0, 0.240926884676765, 0, -0.0621208350291002, 0, 0, 0, 1.5285431037408e-14, -1.09568490725226e-14, 0, 0, 0, 0, 0, -1.60712929692335e-14, 0, 1.01513003014644e-14, 0, 0, -1.13906592977489e-14, 0, 0, 0, 0, 0, 0, 0},
    {3.85449644663769, 0, 5.98171817966647, -2.48806675764062, 1.90029237516522, 0, -0.339934634239515, 0, -2.35922146606158, 0.981306762925306, -1.25696577846526e-14, 9.9522670305624, 1.59597845699183e-14, -6.45364422706461, -0.430945803685653, 1.41669885314633e-14, 0.0770899289327383, 0, 0.535021138879222, -0.222539456105695, 0, 0, 21.5273297444417, 0, 2.75066129614998, 0, -1.00024047132728, 0, 0.648615649442928, 0.0433116829031283, 0, -0.00774782937530501, 0, -0.053771647653677, 0.0223660568773448, 0, 0, 0, 0, 0, -1.57026876352916e-14, 0, 0, 0, 0, 0, 0, 1.17596913617043e-14, 0, 0, 0, 0, -1.17877079382012e-14, 0, 0, 0},
    {0, 1.54523626091315, 0, 0, 0, 6.71778024562961, 1.24434270987286e-14, -0.609449400220049, -2.44890913103646e-14, 0, 0, 1.86912392652466e-14, 14.6593983227032, 0, 1.30441346035566e-14, -1.52344936219876, 0, 0.138210132827974, 0, 0, 0, 0, 0, 18.119501277094, 0, 0, 0, -1.47332496632642, 0, 0, 0.153112421864233, 0, -0.0138906409944119, 0, 0, 0, 0, 0, 1.22183321677886e-14, 0, 0, 0, -1.08769537630915e-14, 0, -1.31873092152821e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {2.52336024752224, 0, 7.37120708415863, -1.62882203585592, 0, 0, 12.0171306297064, 0, -2.90724328050878, 0.642416074439641, 0, 0, 1.07383829658909e-14, 13.9858820944259, 0, -2.32400745727179e-14, -2.72522906732405, 0, 0.659300804656265, -0.145686271816941, 0, 0, 0, 0, 10.8043830863937, 0, 0, 0, -1.40563403846548, 0, 0, 0.273895826790425, 0, -0.0662622240310308, 0.0146420212340205, 0, 0, 0, 0, 0, -2.09114976580726e-14, 0, 0, 0, 0, -2.35127496508513e-14, 0, 0, 0, -1.14219479838847e-14, 0, 0, 0, 0, 0, 0},
    {0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0, 6.30840061880561, 0, 0, 0, -1.52362350055011, 0, 11.1963004093827, -2.55351295663786e-14, 0, 4.26614580154032, 0, 0.132992762321598, 0, 2.56609067489954e-14, -4.72217953828914, 1.17310134406447e-14, -6.67684597013019, 0, 1.60758884410055e-14, 0, 0, 0, 0, 0, 23.1532444221696, 0, 0.72177887856059, 1.28783366735277e-14, -1.53368636742207e-14, 1.1112512795529, -1.22624479997289e-14, 1.19087439227729, 0, 0, 0, 0, 0, 0, 0, 0, 2.28291451277892e-14, 0, 0, 0, 1.34418865877235e-14, 2.02478204822475e-14, 0, -1.70814325121815e-14, -1.2090896650498e-14, 0, 0, 0, -2.48689957516035e-14, 0, 0},
    {3.518657752745, 0, 1.92724822331884, 2.72554057547703, 1.73472166622177, 0, -2.01705689521107, 0, 3.42052627529744, -4.03112887414927, 0, 4.08831086321546, -1.50711171565645e-14, 0.365669640132009, 5.37643791520789, -1.10002031399277e-14, -6.25148194071022, 0, -2.03981013744553, 1.8080332206528, 0, 1.12311783689995e-14, -1.2495355609558e-14, 0, 0, 0, 22.1880978975596, 2.59644251000699e-14, 1.98456380761453, -1.2652152389098, 0, 1.47113578579291, 0, 0.363818136386349, -0.294009452979948, 0, 0, 0, 0, 0, 0, 0, -2.14091784397362e-14, 3.96979729470341e-14, 2.95273442389967e-14, 0, -1.02898405303999e-14, -1.55944604137083e-14, 0, -1.2756671985334e-14, 2.67341313377078e-14, 0, 0, 0, 0, 0},
    {5.00588290781391e-14, 1.4106012612951, 1.86517468137026e-14, 1.57048767420779e-14, 1.14431516135412e-14, 3.63405410063598, 0, 2.50356888118883, 0, 0, 0, 0, 3.56857028479905, 0, 0, 11.263055412878, 3.38127349244425e-14, -1.49298814645067, 0, 2.47950570155456e-14, 0, 0, 0, 0, 0, 0, 0, 19.3673596462205, 0, 0, -2.65048896124153, 0, 0.266287609379583, 0, 0, 0, 0, 0, 0, 0, 0, 0, -1.24805331057479e-14, 0, 2.67117428927039e-14, 0, 0, 0, -2.83074697144157e-14, 0, 0, 0, 0, 0, 0, 0},
    {2.30350221379958, 0, 4.4158804331639, 1.78428514239955, 0, 0, 4.2661458015403, 1.92100483859941e-14, 7.83741028656788, -2.63899331455873, 0, 0, 0, 2.23427840700302, 1.30548943777412e-14, -2.38078955158898e-14, 13.2221027072096, -1.23603801475692e-14, -4.67379217909115, 1.18363558466232, 0, 0, 0, 0, 0, 0, -1.58444179603095e-14, 0, 12.1258851598178, 0, 1.08652403506101e-14, -3.11150358274815, 0, 0.833612074594099, -0.192474367616158, 0, 0, 0, 0, 0, 0, 0, 0, 1.49231173587483e-14, -1.98974312579798e-14, -1.74186427463722e-14, 0, 0, 0, 0, 0, 0, 0, 0, -2.0574377947513e-14, 0},
    {0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {3.14718316987773, 0, -0.861891607371334, 6.09449400220044, 1.55158222708543, 0, 0.0693888666488801, 0, -3.05941170815564, 1.80277563773202, 0, 0, 0, 0, 9.03122156537873, 2.80112084875395e-14, 0.403888506800968, -1.0607742830904e-14, -3.67461994579463, -5.49650682721189, 0, 0, 0, 0, 0, 0, 1.10205726024203e-14, 0, 1.65064789881475e-14, 21.7841260588564, 0, 0.974215733960566, 1.13989530288554e-14, 1.78974858569577, 1.80791908794933, 0, 0, 0, 0, 0, 0, -1.4994119558265e-14, 0, -2.17157199142971e-14, 0, 1.16195530358102e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 2.22044604925031e-14},
    {0, 1.26168012376113, 1.95399252334028e-14, 0, -1.17816950171735e-14, 1.42204860051341, 1.74453095862365e-14, 4.47852016375308, -2.08721928629529e-14, 1.0744670813999e-14, -1.34464719541228e-14, 3.78456620442729e-14, 0, 2.80676208590537e-14, 0, 8.27725128825316, 0, 5.37909280973871, 1.24344978758017e-14, -1.95594693216656e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 19.9654813226339, 0, -2.61992366301004, 1.06581410364015e-14, 0, 0, 0, 0, 1.3455473305103e-14, 0, 1.46585553191507e-14, 0, -1.27103118437955e-14, 0, -2.2308537695773e-14, 0, 0, 1.05662688412451e-14, 2.42300772936765e-14, 0, 0, 0, 1.19298522441569e-14, 2.66453525910038e-14, 1.50990331349021e-14, 0},
    {2.06031501455086, -3.07345773477637e-14, 2.25696201807219, 3.98978286964828, 0, -1.17529718472799e-14, 0.953939201416948, -4.35480723877723e-14, 8.01142041980431, 1.18019368870419, 0, 0, 0, 1.31328347545057e-14, 0, 1.03323004232013e-14, 5.5525489642145, -2.26382415357737e-14, 9.62241374388481, -3.59830837050572, 0, 0, 0, 0, 0, 0, 1.02408289475646e-14, 0, 0, 0, 0, 13.3932520322736, 0, -4.6866619250806, 1.18356086726904, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1.27725097955809e-14, -1.27832588361236e-14, 0, 0, -1.04546715347572e-14, 1.45339591718248e-14, 0, 0, 0, 0, -1.59580006645491e-14, 1.4210854715202e-14},
    {0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {3.93184363124296e-14, 1.09264703862703, 1.28785870856518e-14, 0, 0, 1.22575380004522e-14, -1.14864973879958e-14, 5.38682254607081, 0, 0, 0, -1.44121099608086e-14, 0, -1.75698029368367e-14, -1.80160878442005e-14, 2.57528022630009e-14, 2.69054202848798e-14, 13.4377817958113, 0, 0, 0, 0, 1.06823510383984e-14, 0, 0, 0, 0, 0, 1.89056142369167e-14, 0, -1.31345967659526e-14, -1.51823905359486e-14, 18.907670401189, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, -1.07391608525697e-14, 0, 1.46384409866762e-14, 0, 0, -1.63612658717074e-14, 0, 0, 0, 0, 0, 0, 0},
    {1.78428514239956, -1.14226462513753e-14, 0.814411017927918, 5.06770487035906, 0, 1.13885635970811e-14, 0, 1.47667974881489e-14, 4.01510046524763, 6.58672305740271, 0, 2.01242961116133e-14, 0, -1.5623829602503e-14, 0, 0, 0, -3.27887259670989e-14, 10.0159311874144, 2.83293314500738, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1.60502464055099e-14, 0, 1.06885013954928e-14, 14.0929454377398, -4.55552790213387, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, -2.19027884045748e-14, -1.24344978758018e-14},
    {0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {1.4568627181694, 0, 0, 5.07816454066245, -1.07582259106397e-14, 0, 0, -2.20867693944166e-14, 0, 10.0142755247554, 0, 0, 0, 0, 0, 1.02210862188786e-14, 3.51018877774324e-14, 0, 0, 13.8784813613724, 0, 0, 0, 0, 0, 0, 0, 0, 1.67232492370418e-14, 0, 0, 0, 0, 0, 12.5535587278944, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1.55927253999833e-14, 1.90609040202255e-14, 0, 0, 0, -1.79253699586741e-14, 0, 0, 1.08479262389015e-14, 0, 0, 3.10862446895044e-14},
    {0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0}};
    
    const static double dmats1[56][56] = \
    {{0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {3.16227766016836, 0, 0, 0, 0, 1.08252669405283e-14, 0, 1.02922993704865e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {5.47722557505162, 0, 0, 0, 0, 1.92068165630412e-14, 0, 1.80834223416197e-14, -1.14152204219919e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {2.95803989154978, 5.6124860801609, -1.08012344973465, -0.763762615825971, 0, 2.58464835607474e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, -1.40850097772758e-14, 0, 0, 1.16248598628335e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {2.2912878474779, 7.24568837309471, 4.18330013267037, -0.591607978309959, 0, 2.03703972918374e-14, 0, 0, 0, 1.08801856413265e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1.16401775975232e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {-2.64575131106454, 0, 9.66091783079294, 0.683130051063963, 0, -1.49625671831933e-14, 0, -3.2502596093762e-14, 0, 0, 0, 1.34670915888159e-14, 0, 0, 0, 2.90440444432224e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {1.87082869338697, 1.42804031130623e-14, 0, 4.34741302385683, 0, 0, 0, 2.04003480774873e-14, -1.14092481911365e-14, -1.06581410364015e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {3.24037034920393, 2.38080669966159e-14, 0, 7.52994023880667, 0, 1.13478631598507e-14, 0, 3.45590001304086e-14, -1.69864122767649e-14, -1.4210854715202e-14, 0, 0, 0, 0, 0, -1.32400798627181e-14, 0, 1.22473609554424e-14, 1.33269883629769e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {2.74954541697351, 5.79655069847575, -1.67332005306816, -1.18321595661991, 7.74596669241483, -1.19999999999998, 0.346410161513775, -0.979795897113269, 0.282842712474611, 0.200000000000014, -1.0696429695597e-14, -1.26705171178379e-14, -1.2948158065601e-14, 0, 0, -1.45874734860756e-14, 0, -1.16672151462161e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, -1.07102102118129e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {2.32379000772449, 2.44948974278316, 2.82842712474617, -0.999999999999999, 9.16515138991168, 7.09929573971955, -2.04939015319193, -0.414039335605429, -0.478091443733758, 0.169030850945723, 0, -1.28646947787458e-14, -1.8460920120059e-14, -1.81736908030475e-14, 0, 1.37951452580958e-14, -1.77890663211312e-14, 0, 0, 0, 0, 1.08892803896054e-14, 0, 0, 0, 0, 1.05640513109647e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, -1.22044403129704e-14, 0, 0, 0, 2.27334526137921e-14, -1.76454983335969e-14, -1.05574887489127e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {1.80000000000004, -5.69209978830308, 4.38178046004131, -0.774596669241497, 1.24189930203385e-14, 10.998181667894, 4.76235235991625, 0.962140470884722, -0.740656079818038, 0.130930734141603, 0, -1.20430917197879e-14, -1.1981685754404e-14, -2.88797787541631e-14, -1.3779450272627e-14, 0, 0, 0, -2.22505656218757e-14, -1.14908083048704e-14, 0, 0, 0, 0, 0, 0, 0, 0, 1.22673594366302e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, -1.12284484669662e-14, 0, 0, 0, 0, 0, 0, 0, 0, -1.1387566167118e-14, 0, 1.48982410846523e-14, 0, 0, 0, 0},
    {5.19615242270664, 0, -3.16227766016842, -2.23606797749978, 0, 0, 13.7477270848675, 1.78325020862166e-14, 0.534522483824845, 0.377964473009226, 0, -2.80136022320966e-14, -1.36437984192736e-14, -2.9552592280819e-14, 1.3605842011612e-14, -4.49783579557011e-14, 2.03714412277317e-14, -1.61597054173618e-14, -2.47182474765566e-14, 0, 0, 0, 0, 0, 0, 0, 0, 1.7014378114654e-14, 0, 0, 0, -1.14827159581245e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, -1.57668143745762e-14, 2.14083883902157e-14, -2.68877125810315e-14, 0, 0, 0, -1.36900091929644e-14, -2.0687516582275e-14, 0, 0, 0, 0, 0, 0},
    {2.01246117974979, 2.12132034355965, -0.408248290463867, 3.17542648054294, 0, 1.00616573615107e-14, 0, 7.17137165600639, -1.38013111868472, -1.56144011671766, 0, 0, 1.10848947143279e-14, 0, 0, -1.41970084791766e-14, -1.32242728950118e-14, 1.59365405521834e-14, 0, -1.4210854715202e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, -1.214689797411e-14, 0, 0, 1.24931114721564e-14, 0, 0, 0, 0, 0, 0, 0},
    {1.55884572681198, 2.73861278752583, 1.58113883008418, 2.45967477524978, 0, 0, 0, 9.25820099772551, 5.34522483824847, -1.20948631362952, 0, 0, 1.84632408273169e-14, 0, 0, 0, -1.50085060064379e-14, 1.49723986219342e-14, 0, -1.33226762955019e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, -1.46277209591922e-14, 0, 0, 2.16306109608827e-14, 0, 0, 0, 1.38795388762322e-14, 0, 0, 0},
    {-1.79999999999998, -2.72689985024876e-14, 3.65148371670112, -2.84018778721877, 0, -2.17150305878754e-14, 0, -5.90005603264198e-14, 12.3442679969674, 1.39659449751037, 0, 0, 0, 0, 0, 2.68747109317333e-14, 0, 0, -1.90845800418516e-14, 0, 0, 0, 0, 0, 0, -1.23984917003451e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1.36024520914242e-14, 0, 0, 0, 0, 0, 1.0365125502094e-14, 0, 0, 1.00866023900427e-14, -1.51206012498905e-14, 0},
    {1.27279220613579, 0, 0, 3.83405790253616, 0, 0, 0, 1.00217788352562e-14, 0, 5.18459255872629, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {2.20454076850487, 0, 0, 6.6407830863536, 0, 0, 0, 1.6954568423569e-14, -1.04207941733572e-14, 8.97997772825745, 0, 0, 0, 0, 0, 0, -1.07399115598352e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, -1.06281916397066e-14, 0, 1.0771947239792e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {2.56904651573302, 5.80288457473997, -2.01017818278147, -1.42141062443802, 8.14215222512715, -2.10229466468021, 0.728256234341424, -1.71651640581399, 0.594618725379061, 0.420458932936053, 9.8107084351743, -1.18450885369836, 0.30583887092058, -0.105945692672794, -1.02581475831037, 0.26486423168198, -0.0917516612761728, 0.216260739578395, -0.0749149177264333, -0.0529728463363882, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {2.26568606239551, 3.07059789431495, 1.77281052085584, -1.25356634105602, 2.39356776939086, 6.3037687140313, -2.78313372537745, -0.908295106229251, -0.524404424085067, 0.370809924354793, 11.1242977306435, 9.40174755792013, -2.42752078113808, 0.840917865872078, -0.301561193523236, -0.794200206656967, 0.350641890608821, 0.114434427054281, 0.0660687472643356, -0.0467176592151211, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1.01015959200066e-14, 0, 0, 0, 0, 0, 0, 0, 1.84814770714563e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {1.91485421551268, 1.29756443747034, 3.74574588632198, -1.05945692672795, -8.09173593712686, 5.01426536422406, 1.80936716113936, -0.383824736789375, -1.10800657553489, 0.313391585263996, 0, 12.7134831207354, 8.20651806648289, -2.84282124887607, 1.01946290306867, -0.631738055305808, -0.227958835180056, 0.0483573714561744, 0.139595707137592, -0.039483628456617, 0, 0, 0, 0, 0, 0, 0, 1.2257456641157e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, -1.29080203530852e-14, 1.16595669138782e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {1.48323969741914, 7.03562363973513, 4.062019202318, -0.820651806648271, 0, -6.79705818718658, 5.88642506110458, -2.08116553882674, -1.20156148406981, 0.242752078113807, 0, 0, 14.8323969741913, 5.13809303146607, 1.63714487212955e-14, 0.856348838577624, -0.741619848709526, 0.262202212042521, 0.151382517704875, -0.030583887092048, 0, 0, 0, 0, 0, 0, -1.18952346076914e-14, 1.30853433842222e-14, -1.16925429925565e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, -1.88029658615293e-14, 1.91748164115319e-14, -1.45977163230575e-14, 1.09655150319724e-14, 0, 0, -1.89024735595586e-14, 0, 0, 0, 0, 0, 0, 0},
    {-3.42539535431071, 0, 9.38083151964686, 1.89521416591735, 0, 0, -3.39852909359329, 0, -2.77488738510232, -0.560611910581379, 0, 0, 0, 17.7988763690296, 0, -1.94078776694002e-14, 0.42817441928881, 0, 0.349602949390047, 0.0706304617818554, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, -1.20061774357534e-14, 0, 0, 0, 0, 0, -1.00079856173773e-14, 0, -2.26674794489762e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {2.02649122047609, 2.74642624930236, -0.792824967172097, 2.24244764232558, 2.14087209644418, -0.331662479035527, 0.0957427107756351, 6.49923072370878, -1.87616630392938, -2.15580611373099, 0, 0, 0, 0, 9.44035859331468, -1.46249406456536, 0.422185670932516, -2.04706526287666, 0.590936840285292, 0.52231930877335, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1.66277980866635e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1.68132514445692e-14, 0, 0, -1.30320642762071e-14, 1.5317530203921e-14, 0},
    {1.71269767715538, 1.16057691494799, 1.34011878852097, 1.89521416591739, 2.5331140255951, 1.96214168703485, -0.566421515598877, 2.74642624930236, 3.17129986868837, -1.82198870938948, 0, 0, 1.26698604015865e-14, 0, 11.1699829238239, 8.6522315682783, -2.49768411251824, -0.86504295831355, -0.998865569685873, 0.441440386136653, 0, 0, 0, 0, 0, 0, 1.98384517908788e-14, 0, 0, 1.09458095686894e-14, 0, 1.6523812455151e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, -1.03226253428986e-14, 3.45055971392177e-14, -1.0963026379453e-14, 0, 0, 1.98669435916771e-14, 0, 2.24143595655932e-14, 1.21246267628695e-14, 0, 1.40825657142122e-14, 0, 0},
    {1.32664991614221, -2.69693703815181, 2.07610309995257, 1.46802658041887, 0, 3.03973683071411, 1.31624465810881, -6.38211787508104, 4.91295663090393, -1.4113063856887, 0, 0, 0, 0, -1.12046668488404e-14, 13.4039795085888, 5.80409338312195, 2.01017818278149, -1.54743588659735, 0.341938252770126, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1.99188479486906e-14, 0, 0, 0, 0, 0, 0, 1.9439663850877e-14, 0, 2.04972151503089e-14, -2.09775353099935e-14, 0},
    {3.82970843102538, -1.35375539472083e-14, -1.49829835452878, 4.23782770691181, 0, 0, 3.79967103839267, 1.23996592074953e-14, -3.54562104171168, -4.07409060843206, 0, 0, -1.7600753095073e-14, -1.0822450122319e-14, 1.7784123181026e-14, -1.50271421719475e-14, 16.7549743857359, -2.83790269242822e-14, 1.11676565710081, 0.987090711415297, 0, 0, 0, 0, 0, 1.14492917376685e-14, -1.371929590441e-14, 2.1952750214123e-14, 0, 0, 1.18235696338831e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, -1.24727047960437e-14, 2.54018340570979e-14, -1.39065024647839e-14, 0, 0, 0, -2.41394818807004e-14, 0, 0, 0, 0, 0, 0, 0},
    {1.48323969741916, 1.00508909139075, -0.193429485824665, 3.55615782880926, 0, -1.09316735690911e-14, 0, 4.45964044034304, -0.85825820290701, 2.67027285925186, 0, 0, 0, 0, 0, 0, 0, 8.42792824422444, -1.62195554683794, -2.29379153190449, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {1.14891252930764, 1.29756443747036, 0.749149177264399, 2.75458800949268, 0, -1.56850849701631e-14, 0, 5.75737105184055, 3.32401972660469, 2.06838446274243, 0, 0, 0, 1.55284895654593e-14, 0, 0, -1.28699742148281e-14, 10.8804085776374, 6.28180682119208, -1.77676328054755, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, -1.17679879087453e-14, 0, 0, 0, 0, 0, 0, 0, 0, 1.29340213107974e-14, 0, 0, 0, 0},
    {-1.32664991614215, -2.47030321919216e-14, 1.73008591662716, -3.18072425757422, 0, 0, 1.13119318909818e-14, -3.04199753195453e-14, 7.6764947357874, -2.38836465270395, 0, 0, 0, 2.05269241282193e-14, 0, -1.45662197301565e-14, -1.85676576002613e-14, -2.8126806197496e-14, 14.5072114368499, 2.05162951662072, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, -1.09052206632751e-14, 0, 0, 0, 0, 0, 0, 0, 0, -1.073521706443e-14, 0, 0, 0, 0, 0, 0, 0, 1.02157480437484e-14, 0, 0, 0, 0, 1.06581410364015e-14},
    {0.938083151964688, -1.67075414421403e-14, 0, 3.11415464992886, 0, 0, 0, 0, 0, 5.52707620976693, 0, 0, 0, 0, 0, 0, 0, -1.44579473224948e-14, 0, 5.80288457473998, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {1.6248076809272, -2.71174079765265e-14, 0, 5.39387407630366, 0, 0, 0, -1.76761006483537e-14, 0, 9.57317681262153, 0, 0, 0, 0, 0, 1.62541936715167e-14, 1.29794840192237e-14, -2.09070339378817e-14, 0, 10.0508909139074, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1.07896695788578e-14, 1.01165706770544e-14, 0, 1.24344978758018e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1.24344978758018e-14},
    {0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {2.41593350361252, 5.72988941019469, -2.20543546226845, -1.55947837083931, 8.33749895885385, -2.7677995385711, 1.06532653910463, -2.25989885993664, 0.869835476750505, 0.615066564126907, 10.333118277332, -2.18326971917503, 0.724780051083774, -0.278967971730974, -1.8907670401189, 0.627677936394735, -0.241593350361259, 0.512496888990078, -0.197260144544833, -0.139483985865487, 11.8490505948789, -1.16109782023229, 0.245326690731328, -0.0814411017928031, 0.0313466946954498, -1.03851746182649, 0.219426862868121, -0.0728431359084709, 0.0280373360835783, 0.190029237516519, -0.063084006188059, 0.024281045302823, -0.0515078753637739, 0.019825390471097, 0.0140186680417963, 0, 0, 1.00108896213852e-14, 0, 0, 0, 0, 0, 0, 0, 0, 1.21150877964365e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {2.18529407725402, 3.45525332069937, 0.997445717412081, -1.41060126129511, 3.23209352764251, 5.28531208250976, -2.89087233497829, -1.36277028773851, -0.393397896234712, 0.556348640264183, 2.3366642891096, 9.02784807228871, -3.93352933905729, 1.64018416088948, -0.73296991613515, -1.19859611757306, 0.655588223176202, 0.309047252182647, 0.0892142571199726, -0.12616801237611, 13.0996183150502, 11.55277744383, -2.44096975264614, 0.810328731479527, -0.311895674167859, -0.234843597212102, -0.907332870306517, 0.395334573318419, -0.164844710568308, 0.0736662483163133, 0.120463442338377, -0.0658890955530689, -0.031060417514551, -0.00896637020659282, 0.012680362351413, 0, 2.00524187044473e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1.35703519068716e-14, 0, 0, 0, 0, 0, 0, 0, -1.01417984314538e-14, 0},
    {1.92724822331887, 1.82834820066011, 2.99085908983326, -1.24403337882031, 0.950146187582608, 5.74064456311309, -0.169967317119755, -0.721110255092798, -1.1796107330308, 0.490653381462651, -10.3037210107158, 4.97613351528121, 5.78174466995658, -3.22682211353229, -0.215472901842825, -1.30185581821821, 0.0385449644663842, 0.163532434528608, 0.267510569439612, -0.111269728052846, 0, 14.555131461167, 10.7636648722209, -3.57321383950879, 1.37533064807499, 1.03556292536504, -0.500120235663637, -0.581087203147965, 0.324307824721464, 0.0216558414515625, 0.13084143273245, -0.00387391468765269, -0.0164356280722631, -0.0268858238268406, 0.0111830284386733, 0, 0, 0, 0, 0, 0, 0, -1.24894083304515e-14, 0, 0, 0, 0, 0, -1.79685973736065e-14, 0, 0, 0, 0, 0, 0, 0},
    {1.62882203585591, 0.77261813045657, 3.86595114186567, -1.05140010313427, 9.1544290607646, 3.35889012281481, 3.87851223317102, -0.304724700110015, -1.52475169285617, 0.414677792940093, -1.50838999194959e-14, -9.46260092820841, 7.3296991613516, 0.564240504518021, -2.07602937339133, -0.761724681099385, -0.879563899362177, 0.0691050664139764, 0.345781182036188, -0.0940400840863524, 0, 0, 16.3745228938128, 9.05975063854702, -3.48709964663735, 0, 0.951027176355628, -0.736662483163206, -0.0567083043941531, 0.208648802584714, 0.076556210932118, 0.0883994979795754, -0.00694532049720325, -0.0347523163751444, 0.00945138406569365, 0, -1.34204874150947e-14, 0, 0, 0, 0, 0, 0, 0, -2.42374492692268e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {1.26168012376109, -6.78263087840205, 3.68560354207928, -0.814411017927976, 0, 10.0216907612581, 6.00856531485322, 2.67510569439611, -1.4536216402544, 0.321208037219824, 0, 0, -7.57008074256671, 6.99294104721291, 0, -2.27270584034422, -1.36261453366206, -0.606656948415834, 0.329650402328115, -0.0728431359084831, 0, 0, 0, 18.7137404500718, 5.40219154319688, 0, 0, 0.760821741084502, -0.702817019232737, 0, 0.228415531250642, 0.136947913395211, 0.0609713174047674, -0.033131112015512, 0.00732101061700773, 0, 0, 0, 0, 0, -1.30324950839724e-14, 0, 0, 0, -1.36428396125789e-14, -2.70392224640622e-14, 0, 0, 0, -1.78921270332478e-14, 0, 0, 0, 0, 0, 0},
    {5.09901951359278, 1.47405589080242e-14, -4.65474668125635, -3.29140294302192, 0, -1.75121952403549e-14, 13.490737563232, -1.88659898491929e-14, 1.83585684909538, 1.29814682728313, 1.74929961328223e-14, 0, 3.18637490243895e-14, -3.53270434653118, 0, 2.70955022774169e-14, -3.05941170815572, 0, -0.416333199893208, -0.294392028877594, 0, 0, -1.00417365518154e-14, -2.77992935601067e-14, 21.8326971917504, 0, 0, 0, 0.355050145839428, 0, 0, 0.307482445914326, 0, 0.0418430609640593, 0.0295875121532926, 0, 0, 1.53618763757974e-14, -1.4536552951721e-14, 0, -2.34517773777192e-14, 0, 0, 0, 1.75720087957737e-14, -2.91662766570223e-14, 0, 1.76059530106093e-14, 0, 0, 0, 0, 1.07470318048245e-14, 0, 0, 0},
    {1.99489143482416, 3.15420030940281, -1.09264703862702, 1.54523626091316, 2.95048422176041, -0.761811750275054, 0.263899331455877, 5.59815020469135, -1.93925611658553, -2.28543525082516, 2.13307290077015, -0.257539376818855, 0.0664963811607952, -0.0230350221380054, 9.14446135479766, -2.36108976914457, 0.817905488277913, -3.33842298506509, 1.1564636454577, 1.02505848514083, 0, 0, 0, 0, 0, 11.5766222110848, -1.39771878815471, 0.3608894392803, -0.125015768949695, -2.15192884958005, 0.555625639776457, -0.192474367616166, 0.595437196138648, -0.206265495285699, -0.166687691932932, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1.62916528370584e-14, 1.08185730701625e-14, 0, 0, 0, -1.29677564833166e-14, 0, 0, 0, 0, 0, 0},
    {1.75932887637252, 1.66904592079257, 0.963624111659422, 1.36277028773852, 0.867360833110887, 2.28430636201791, -1.00852844760554, 2.96226264871973, 1.71026313764871, -2.01556443707464, 2.41867732448957, 2.04415543160773, -0.527798662911753, 0.182834820065996, 2.68821895760395, 7.07977053255619, -3.12574097035512, -1.76652739792481, -1.01990506872276, 0.904016610326408, 0, 0, 0, 0, 0, 13.1266557397187, 11.0940489487798, -2.8644711213756, 0.992281903807275, -0.632607619454911, -1.66605356688629, 0.735567892896454, 0.315075748468089, 0.18190906819317, -0.147004726489971, 0, 0, 0, 0, 0, 0, 0, 0, 0, 3.50533057667082e-14, 0, 0, 0, 0, 0, 1.4911169345421e-14, 0, 0, 0, -1.04402756430621e-14, 0},
    {1.48690428533297, 0.70530063064754, 2.03602754481989, 1.1517511068998, -2.93221479395414, 1.81702705031798, 0.655663160391202, 1.2517844405944, 3.61359041872289, -1.70346285967313, 0, 2.76420265655949, 1.78428514239953, -0.618094504365263, -9.08783875864323, 5.63152770643898, 2.03210252328839, -0.746494073225332, -2.15494277062554, 0.764033484562606, 0, 0, 0, 0, 0, 0, 15.0018922739643, 9.68367982311024, -3.35452509157131, 2.13860408462396, -1.32524448062077, -0.478206411017787, 0.133143804689794, 0.384353057392899, -0.124241670058204, 0, 0, 0, 0, 0, 0, 0, -1.2184958691979e-14, 4.08565188434445e-14, 2.21302457056835e-14, 0, 0, 0, -1.0903058801995e-14, 0, 0, 0, 0, 0, 0, 0},
    {1.15175110689978, 3.82426463519456, 2.20794021658196, 0.892142571199804, 0, -2.46306042692148, 2.13307290077015, 6.78739640804928, 3.91870514328392, -1.31949665727937, 0, 0, 3.22490309931938, 1.11713920350151, 2.5966242421383e-14, -7.6337845572604, 6.61105135360486, -4.04762275910199, -2.33689608954557, 0.591817792331191, 0, 0, 0, 0, 0, 0, -2.33944627292341e-14, 17.5022076529583, 6.06294257990891, 0, 1.79642743108414, -1.55575179137408, 0.721929233499939, 0.416806037297049, -0.0962371838080711, 0, 0, 0, 0, 0, 0, 0, -1.06500367370775e-14, 2.55596393426003e-14, -1.78278849535737e-14, 1.24660473149391e-14, 0, 0, -3.33471800635493e-14, 0, 0, 0, 0, -2.74739380889122e-14, 0, 0},
    {-2.65985524643224, 3.04548590059106e-14, 5.09901951359276, -2.06031501455087, 0, 1.27314974860562e-14, -1.23153021346075, 4.54600263645348e-14, 9.04986187739903, 3.04724700110024, 0, 0, 1.37797484823791e-14, 3.8698837191833, 0, -2.28548550248364e-14, -3.81689227863023, 1.32945293178207e-14, -5.39683034546927, -1.36674464685441, 0, 0, 0, 1.36272283216478e-14, 1.00072372011083e-14, 0, 0, -1.24367890909937e-14, 21.00264918355, 0, 0, 0.898213715542077, 0, 0.962572311333257, 0.22225025591058, 0, 0, 0, 0, 0, 0, 0, 1.43237887055327e-14, -2.25958210828995e-14, -2.54351954443607e-14, -4.42292868735092e-14, 0, 0, 1.18670146344888e-14, 0, 0, 0, 0, 0, -2.23325753829243e-14, 0},
    {1.57359158493888, 1.49284005458436, -0.43094580368567, 3.04724700110021, 0.775791113542718, -0.120185042515478, 0.0346944333244453, 5.29905651979672, -1.52970585407783, 0.901387818866015, 0, 0, 1.54345206564601e-14, 0, 4.51561078268936, -0.69955541437241, 0.201944253400477, 6.36462844462229, -1.83730997289732, -2.74825341360596, 0, 0, 0, 0, 0, 0, 0, 0, 0, 10.8920630294282, -1.68739114875268, 0.487107866980291, -3.0999354831996, 0.894874292847886, 0.903959543974661, 0, 0, 0, 0, 0, 0, 0, 0, -1.35315801444742e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {1.32992762321608, 0.630840061880569, 0.728431359084703, 2.57539376818858, 0.917928424547677, 0.711024300256696, -0.205255035576777, 2.23926008187653, 2.58567482211402, 0.761811750275079, 0, 1.50972141321262e-14, 1.92432056571699e-14, 2.94270416460811e-14, 5.34294273196304, 4.13862564412658, -1.1947183148558, 2.68954640486935, 3.10562068169861, -2.32269806558124, 0, 0, 0, 0, 0, 0, 0, 0, 0, 12.8876627769294, 9.98274066131697, -2.88176900403077, -1.30996183150502, -1.51261363209512, 0.763985254692642, 0, 0, 0, 0, 0, 1.07852658462479e-14, 0, -1.13464763436474e-14, -1.56492117088446e-14, -1.2428153133211e-14, 0, 0, 0, 1.57186842569255e-14, 1.23293837842049e-14, 0, 0, 0, 1.39814305552841e-14, 0, 1.99840144432528e-14},
    {1.03015750727542, -1.46593983227033, 1.12848100903611, 1.99489143482414, 0, 1.10151410945721, 0.476969600708479, -5.20357020296097, 4.00571020990215, 0.590096844352093, 0, 1.61268443608728e-14, 0, 2.17744761210991e-14, 0, 6.41153127835569, 2.77627448210725, -6.24994106094659, 4.81120687194241, -1.79915418525286, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 15.4651953323153, 6.69662601613678, 3.04407621455179, -2.3433309625403, 0.59178043363452, 0, 0, 0, 1.05482318770498e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {2.9738085706659, -2.31628665433557e-14, -0.814411017927956, 5.75875553449895, 0, 0, 1.37689263682152, -1.87511781470446e-14, -2.89087233497829, 1.70346285967313, 0, 1.53767244373726e-14, -2.60164507598573e-14, -3.0606105648312e-14, 0, 3.91555026057662e-14, 8.01441409794458, 0, -3.47218947830366, -5.19371076584692, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 19.3314941653941, 0, 1.69115345252876, 1.70832296330022, 0, 0, 0, 0, 0, -1.47710207017114e-14, 0, 0, 0, 2.39374069194048e-14, 0, -1.16737795196999e-14, 0, -2.35947835773664e-14, 0, 0, 0, -2.14656473740582e-14, 0, 0, 0},
    {1.15175110689981, 0.546323519313508, -0.105140010313419, 3.27118942773249, 0, 0, 0, 2.69341127303541, -0.518347241175128, 4.25171145123352, 0, 0, 0, 0, 0, 1.89132326876168e-14, 1.07703482219709e-14, 6.71889089790563, -1.29305115618717, 1.82865048192212, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 9.45383520059452, -1.81939143264587, -2.94058061635795, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0.892142571199805, 0.705300630647555, 0.40720550896397, 2.53385243517954, 0, 1.39813392926345e-14, 0, 3.47717900165116, 2.00755023262382, 3.29336152870135, 0, 0, 0, -1.4230588162391e-14, 0, 2.12522836138748e-14, 1.52618648424287e-14, 8.67405085085769, 5.00796559370718, 1.4164665725037, 0, 0, 0, 0, 0, 0, 0, 0, 1.13147703328239e-14, 0, 0, 0, 12.2048487632307, 7.0464727188699, -2.27776395106694, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1.14999053942145e-14, 0, 0},
    {-1.03015750727539, 0, 0.940400840863377, -2.92584077107543, 0, 0, 0, 1.11293406173924e-14, 4.63623866886823, -3.80284633026899, 0, 0, 0, -2.94156668790987e-14, 0, 0, 0, -1.33922800433549e-14, 11.5654011344769, -1.63559471386628, 0, 0, 0, 0, 0, 0, 0, 0, 1.04604926852949e-14, 0, 0, -1.05559372926886e-14, 0, 16.2731316843076, 2.63013526059784, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, -1.36105120406189e-14, 0, 0, -1.64162100411904e-14, 0, 0, -1.43624641143248e-14, 0, 0, -1.06581410364015e-14},
    {0.728431359084702, 0, 0, 2.53908227033122, 0, 0, 0, -1.12654293021334e-14, 0, 5.00713776237769, 0, 0, 0, 0, 0, 0, 1.75509438887162e-14, 0, 0, 6.9392406806862, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 6.27677936394721, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1.55431223447522e-14},
    {1.26168012376115, 0, 0, 4.39781949681102, 0, 0, 0, -1.94061175819238e-14, 0, 8.6726170049349, 0, 0, 0, 0, 0, 1.01179304274323e-14, 2.87309305235289e-14, 0, -1.04523229052332e-14, 12.0191174248973, 0, 0, 0, 0, 0, 0, 0, 0, 1.49130416320287e-14, 0, 0, 0, 0, 0, 10.8717007662564, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1.39042309332655e-14, 1.68148173406132e-14, 0, 0, 0, -1.39259589347659e-14, 0, 0, 0, 0, 0, 2.57571741713036e-14},
    {0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0}};
    
    const static double dmats2[56][56] = \
    {{0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {3.16227766016836, 0, 0, 0, 0, 1.11583338479159e-14, 0, 1.02922993704865e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {1.82574185835054, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {5.16397779494319, 0, 0, 0, 0, 1.73479189776187e-14, 0, 1.65773933526039e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {2.95803989154978, 5.6124860801609, -1.08012344973464, -0.763762615825969, 0, 2.58464835607474e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, -1.45290989871259e-14, 0, 0, 1.07366814431334e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {2.29128784747791, 1.44913767461894, 4.18330013267037, -0.591607978309966, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1.02908502069211e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {1.32287565553229, 0, 3.86436713231718, -0.341565025531991, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {1.87082869338696, 7.09929573971954, 0, 4.34741302385683, 0, 2.31412979990705e-14, 0, 1.79578574233119e-14, -1.09651589812864e-14, 0, 0, 0, 1.0275456662804e-14, 1.11253951863901e-14, 0, 0, -1.23516279655211e-14, 0, 1.67772945370309e-14, -1.06581410364015e-14, 0, 0, 0, 0, 0, 0, 0, 0, -1.04810894589995e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, -1.45492836924348e-14, 0, 0, 0, 0, 0, 0, 0, -1.07986153011749e-14, 0, 0},
    {1.08012344973466, 0, 7.09929573971953, 2.50998007960221, 0, 0, 0, 0, 0, 0, 0, 1.10981825134126e-14, 0, 0, 0, 1.49126177831961e-14, 0, 0, -1.09527631538099e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1.11294528477952e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {-3.81881307912981, 2.63805095952449e-14, 0, 8.87411967464941, 0, -1.08587953265362e-14, 0, 1.60284200435428e-14, 0, -2.30926389122033e-14, 0, 0, 2.0594465710441e-14, 1.14859138885232e-14, 1.07233040380981e-14, 0, -1.40594294599945e-14, 0, 0, -2.1316282072803e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, -1.15463194561016e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {2.74954541697351, 5.79655069847576, -1.67332005306816, -1.18321595661991, 7.74596669241483, -1.19999999999999, 0.346410161513776, -0.979795897113272, 0.282842712474611, 0.200000000000011, 0, -1.42248293523131e-14, -1.16991571628977e-14, 0, 0, -1.34772504614504e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, -1.05991879093504e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {2.32379000772445, 2.44948974278317, 2.82842712474618, -1.00000000000001, 1.30930734141596, 7.09929573971955, -2.04939015319193, -0.414039335605422, -0.478091443733756, 0.169030850945715, 0, 0, -1.38137027369175e-14, 0, -1.23135650746216e-14, 1.4971941277217e-14, -1.68198040008164e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, -1.41869149998573e-14, 1.99142275743244e-14, -1.52384110702395e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {1.80000000000002, 0.632455532033669, 4.38178046004131, -0.774596669241481, 0, 3.14233761939829, 4.76235235991625, -0.106904496764978, -0.740656079818043, 0.130930734141599, 0, 0, 0, -1.70268155420246e-14, 0, 0, 0, 0, -1.65329170450561e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {1.03923048454135, 0, 3.16227766016836, -0.447213595499958, 0, 0, 5.89188303637179, 0, -0.53452248382485, 0.0755928946018462, 0, 0, 0, -1.1578427571356e-14, 0, 0, 0, 0, -1.31656139489963e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {2.01246117974984, 2.12132034355964, -0.40824829046387, 3.17542648054295, 9.07114735222145, 0, 0, 7.17137165600638, -1.38013111868473, -1.56144011671764, -1.28131039656276e-14, -1.05722626181333e-14, 0, 0, 0, -1.43560570277939e-14, -2.27050760178409e-14, 0, 0, 0, 0, 0, 0, 0, 0, 1.20645452546378e-14, 0, 0, -1.39975315007857e-14, 0, 1.33944298105448e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, -1.79370968861081e-14, 0, 0, 0, 0, 0, 0, 0, 0, 1.10225378253926e-14, 0},
    {1.558845726812, 0.547722557505167, 1.58113883008418, 2.45967477524977, 0, 9.07114735222146, -1.85760141295822e-14, 1.85164019954509, 5.34522483824848, -1.20948631362952, 0, -1.79269712993067e-14, 0, 0, -1.21834179081657e-14, 0, -1.08231757343816e-14, 1.58605770416344e-14, 0, -1.77635683940025e-14, 0, 0, 0, 0, 0, 0, 1.20070279998216e-14, 0, 0, 0, 0, 0, 0, 0, -1.11022302462516e-14, 0, 0, 0, 0, 0, 0, 0, 0, -1.5994754107584e-14, 1.5836245903059e-14, -1.59336158038543e-14, 0, 0, 1.24090681928578e-14, 0, 0, 0, 0, 0, 0, 0},
    {0.900000000000037, 0, 1.46059348668045, 1.42009389360939, 0, -1.21399527118824e-14, 9.07114735222146, 0, 4.93770719878694, -0.698297248755176, 0, -1.69782446707204e-14, 0, 0, 1.38222655524042e-14, -1.41725217571765e-14, 0, 0, -1.92181403448241e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1.52483803956785e-14, -1.04387107607342e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {1.27279220613579, -6.2609903369994, 0, 3.83405790253616, 0, -2.66966858276427e-14, 0, 10.5830052442584, -1.29525898283173e-14, 5.18459255872629, 0, 1.15955623710069e-14, 2.12136539532159e-14, 0, 0, 0, -1.1187497199729e-14, 0, 0, -2.1316282072803e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1.1791882435372e-14, 1.34567031689284e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1.11941676856894e-14, 0, 0, 1.28246579327207e-14, 0, 0, 0, 0},
    {0.734846922834948, 0, -6.26099033699939, 2.21359436211787, 0, 0, 0, 0, 10.5830052442584, 2.99332590941916, 0, -1.26338784464357e-14, 1.51937191134705e-14, 2.01241989119956e-14, 0, 0, -1.07773281838078e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {5.71576766497725, -3.5255606739263e-14, -2.0846313271379e-14, -4.69574275274955, 0, 0, 1.14621342066822e-14, -1.32075361813795e-14, 0, 12.6996062931101, 0, 1.1579866837032e-14, -1.17571210377054e-14, 0, 0, 1.11781775592062e-14, 0, -2.03916735577721e-14, 0, 1.24344978758018e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1.77352530249445e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1.34841270690221e-14, 0, 0, 0, 0, 0, 1.33226762955019e-14},
    {2.56904651573302, 5.80288457473997, -2.01017818278147, -1.42141062443802, 8.14215222512715, -2.10229466468021, 0.728256234341424, -1.71651640581399, 0.59461872537906, 0.420458932936052, 9.8107084351743, -1.18450885369835, 0.305838870920579, -0.105945692672794, -1.02581475831037, 0.264864231681979, -0.0917516612761747, 0.216260739578394, -0.0749149177264344, -0.0529728463363901, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {2.26568606239552, 3.07059789431494, 1.77281052085583, -1.25356634105602, 2.39356776939085, 6.30376871403133, -2.78313372537745, -0.908295106229254, -0.524404424085065, 0.370809924354795, 1.23603308118261, 9.40174755792012, -2.42752078113806, 0.84091786587207, -0.301561193523234, -0.794200206656967, 0.350641890608815, 0.114434427054271, 0.0660687472643394, -0.0467176592151202, 0, 0, 0, 0, 1.20766335966975e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, -1.26955258141599e-14, 0, 0, 0, 0, 0, 1.65501530620379e-14, -1.82242579771102e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {1.91485421551271, 1.29756443747034, 3.74574588632197, -1.05945692672795, 0.404586796856347, 5.01426536422407, 1.80936716113936, -0.383824736789378, -1.1080065755349, 0.313391585264007, 0, 2.82521847127454, 8.20651806648288, -2.84282124887606, -0.0509731451534317, -0.631738055305793, -0.227958835180061, 0.0483573714561631, 0.139595707137598, -0.0394836284566152, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {1.48323969741914, 0.335029697130245, 4.06201920231797, -0.820651806648286, 0, 1.69926454679664, 5.88642506110457, -0.0991031208965069, -1.20156148406979, 0.24275207811381, 0, 0, 4.94413232473043, 5.13809303146604, 0, -0.214087209644449, -0.741619848709543, 0.0124858196210661, 0.151382517704864, -0.0305838870920592, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, -1.236469996079e-14, 0, -1.11803154979493e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0.85634883857767, 0, 2.68023757704195, -0.473803541479343, 0, 0, 5.09779364038993, 1.03628894560728e-14, -0.792824967172095, 0.140152977645353, 0, 0, 0, 7.9106117195687, 0, -3.2024651826757e-14, -0.642261628933259, 0, 0.0998865569685812, -0.0176576154454647, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, -1.41874267782629e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {2.02649122047608, 2.74642624930236, -0.79282496717209, 2.24244764232558, 2.14087209644418, -0.331662479035553, 0.0957427107756432, 6.49923072370878, -1.87616630392938, -2.15580611373099, 11.0554159678513, 0, 0, 0, 9.44035859331468, -1.46249406456537, 0.42218567093253, -2.04706526287665, 0.590936840285286, 0.522319308773345, 0, 0, 0, 0, 0, 0, 0, 1.07417485753862e-14, 0, 0, 1.36630425513329e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, -1.23701834881004e-14, 0, 1.02417490692638e-14, 0, 0, 0, 0, 0, 1.80291351901461e-14, 0, 0, -1.01316066243738e-14, 0, 0},
    {1.71269767715536, 1.16057691494801, 1.34011878852098, 1.89521416591738, 0.361873432227874, 1.96214168703485, -0.56642151559888, 2.74642624930238, 3.17129986868838, -1.82198870938951, 0, 11.0554159678513, 1.45760542423516e-14, 1.47166563795188e-14, 1.59571184626056, 8.65223156827826, -2.49768411251825, -0.865042958313534, -0.998865569685866, 0.44144038613664, 0, 0, 0, 0, 1.29892244953406e-14, 0, 1.19356062430186e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 2.55080325568313e-14, -1.99643912911818e-14, 0, 0, 2.52250943240554e-14, 0, 0, 0, 0, 0, 0, 0},
    {1.32664991614219, 0.299659670905743, 2.07610309995258, 1.46802658041888, 0, 0.868496237346862, 1.31624465810882, 0.709124208342315, 4.91295663090392, -1.4113063856887, 0, 1.04153151561804e-14, 11.0554159678513, 0, 0, 3.82970843102535, 5.80409338312197, -0.223353131420174, -1.54743588659735, 0.341938252770125, 0, 0, 0, -1.22971310029744e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, -1.3965710429727e-14, 1.20613126934675e-14, 1.38979281389624e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, -1.63145986065679e-14, 0},
    {0.765941686205059, -1.0369529147852e-14, 1.49829835452879, 0.847565541382363, 0, 0, 1.62843044502544, 0, 3.54562104171167, -0.814818121686403, 0, 0, 0, 11.0554159678513, 0, -2.54103636481183e-14, 7.18070330817254, 0, -1.11676565710084, 0.197418142283058, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, -1.21467689198221e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {1.48323969741914, 1.00508909139075, -0.193429485824672, 3.55615782880927, -8.46157280212813, 0, 1.04737699623147e-14, 4.45964044034302, -0.858258202906999, 2.67027285925188, 0, 0, 1.43424333847387e-14, 0, 12.4373429638327, 2.02429672213922e-14, 1.33492152068286e-14, 8.42792824422444, -1.62195554683795, -2.29379153190448, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1.03917274103107e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, -1.08180915476963e-14, 0, 0, 0, 1.39785916269036e-14, 0, 0, 0, 0, 0, 1.03080706885695e-14, 0, 0, 0, 0},
    {1.14891252930762, 0.259512887494088, 0.749149177264413, 2.75458800949269, 0, -8.46157280212813, 1.97736713455776e-14, 1.15147421036812, 3.32401972660468, 2.06838446274242, 0, 0, 1.76595565092679e-14, 3.53657180389623e-14, 0, 12.4373429638327, -2.12998980019174e-14, 2.1760817155275, 6.28180682119209, -1.77676328054753, 0, 0, 0, 0, 0, 0, 0, 0, -2.02381918388612e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1.29298824184789e-14, 0, 0, 0, 0, -1.20899537342362e-14, 0, 0, 1.49778288260513e-14, 1.57856876457792e-14, 0, 0, 0, 0, 0, 0},
    {0.663324958071077, -2.03917269773753e-14, 0.692034366650878, 1.59036212878712, 0, 0, -8.46157280212812, -1.91967284198321e-14, 3.07059789431496, 1.194182326352, 0, 1.63277888554017e-14, 0, 2.18186738788674e-14, 0, 2.30908333865784e-14, 12.4373429638327, -2.11081152198351e-14, 5.80288457473997, -1.02581475831036, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1.31364686077023e-14, 0, -1.38856086370832e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1.88129856179495e-14, 0, 0, 0, 0, 0, 0},
    {0.938083151964734, 7.62808987244123, 0, 3.11415464992886, 0, 0, 0, -7.52139804633612, 0, 5.52707620976692, 0, 0, -1.777231263474e-14, 0, -1.62387587846106e-14, 1.69100848543788e-14, 2.98390950438128e-14, 14.2141062443803, 0, 5.80288457473998, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1.39576755950212e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1.84182164821129e-14, 0, 1.05827350607878e-14, 0, 0, 0, 1.04237510536707e-14, 0, 1.26392642221179e-14, 0, 0},
    {0.541602560309088, -1.13099822653614e-14, 7.62808987244125, 1.79795802543456, 0, 0, 0, 0, -7.5213980463361, 3.19105893754052, 0, 1.03213108228181e-14, 0, 0, 0, -1.02355531327919e-14, 0, -3.31702746937809e-14, 14.2141062443803, 3.35029697130246, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, -1.166062958575e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {-5.36159180343545, -1.34228950103456e-14, 1.18705474084944e-14, 8.89943818451483, 0, 0, 0, -2.89831441233684e-14, 0, -5.26497863243526, 0, 0, 0, 0, -1.29782943052458e-14, 0, 2.98133166946067e-14, 0, 0, 16.583123951777, 0, 0, 0, 0, 0, 0, 0, 0, 1.67376776564845e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1.61338757506847e-14, 1.94557000035003e-14, 0, 0, 0, -1.11267253048686e-14, 0, 0, 0, 0, 1.32728374914348e-14, 1.68753899743024e-14},
    {2.41593350361252, 5.72988941019469, -2.20543546226845, -1.55947837083931, 8.33749895885385, -2.7677995385711, 1.06532653910463, -2.25989885993664, 0.869835476750504, 0.615066564126906, 10.333118277332, -2.18326971917503, 0.724780051083775, -0.278967971730974, -1.8907670401189, 0.627677936394735, -0.241593350361257, 0.51249688899008, -0.197260144544834, -0.139483985865487, 11.8490505948789, -1.16109782023229, 0.245326690731328, -0.0814411017928033, 0.0313466946954489, -1.03851746182649, 0.219426862868122, -0.0728431359084709, 0.0280373360835781, 0.190029237516519, -0.063084006188059, 0.0242810453028233, -0.0515078753637743, 0.0198253904710968, 0.014018668041796, 0, 0, 1.04549788312352e-14, 0, 0, 0, 0, 0, 0, 0, 0, 1.07828201668863e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {2.18529407725403, 3.45525332069937, 0.997445717412063, -1.41060126129511, 3.23209352764252, 5.28531208250976, -2.89087233497829, -1.36277028773849, -0.393397896234727, 0.556348640264195, 2.33666428910958, 9.02784807228869, -3.93352933905731, 1.64018416088945, -0.732969916135171, -1.19859611757308, 0.655588223176205, 0.309047252182632, 0.0892142571199684, -0.126168012376111, 1.19087439227729, 11.55277744383, -2.44096975264613, 0.810328731479535, -0.311895674167857, -0.234843597212088, -0.907332870306508, 0.395334573318422, -0.164844710568306, 0.0736662483163242, 0.120463442338383, -0.0658890955530685, -0.0310604175145507, -0.00896637020658553, 0.0126803623514123, 0, 0, 0, 0, 0, 0, 0, -1.66112853067542e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {1.92724822331887, 1.82834820066012, 2.99085908983324, -1.24403337882031, 0.950146187582616, 5.74064456311309, -0.169967317119763, -0.721110255092806, -1.17961073303079, 0.490653381462656, 0.294392028877589, 4.9761335152812, 5.78174466995658, -3.2268221135323, -0.215472901842837, -1.30185581821819, 0.0385449644663795, 0.163532434528623, 0.267510569439614, -0.11126972805285, 0, 2.64638753839399, 10.7636648722209, -3.57321383950879, 1.37533064807499, -0.0295875121532857, -0.500120235663636, -0.581087203147967, 0.324307824721465, 0.0216558414515637, 0.130841432732446, -0.00387391468765088, -0.0164356280722617, -0.0268858238268399, 0.0111830284386723, 0, 0, 0, 0, 0, -1.09166603315028e-14, 0, 0, 0, 1.42437445527895e-14, 0, 0, 0, -1.02438819301777e-14, 0, 0, 0, 0, 0, 0, 0},
    {1.62882203585591, 0.772618130456562, 3.86595114186567, -1.05140010313427, 0.160604018609904, 3.35889012281481, 3.87851223317101, -0.304724700110016, -1.52475169285617, 0.414677792940092, 0, 1.13551211138501, 7.32969916135159, 0.564240504518027, -0.0364215679542207, -0.761724681099395, -0.879563899362167, 0.0691050664139829, 0.345781182036178, -0.0940400840863483, 0, 0, 4.46577897103986, 9.05975063854702, -3.48709964663734, 0, -0.114123261162682, -0.736662483163202, -0.0567083043941542, 0.0036605053085017, 0.0765562109321201, 0.0883994979795774, -0.00694532049720645, -0.0347523163751393, 0.00945138406569343, 0, 0, 0, 0, 0, 0, 0, 0, 1.06468345288818e-14, -1.93401297016719e-14, 1.26134404858539e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {1.26168012376111, 0.199489143482416, 3.68560354207931, -0.814411017927961, 0, 1.02786571910339, 6.00856531485322, -0.0786795792469436, -1.4536216402544, 0.321208037219821, 0, 0, 3.0280322970267, 6.99294104721294, 0, -0.23309803490712, -1.36261453366203, 0.0178428514239939, 0.32965040232813, -0.0728431359084719, 0, 0, 0, 6.80499652729883, 5.40219154319687, 0, 0, -0.304328696433803, -0.702817019232747, 0, 0.0234272339744286, 0.136947913395211, -0.00179327404131641, -0.0331311120155151, 0.00732101061701076, 0, 0, 0, 0, 0, 0, 0, 0, 0, -1.10019056033674e-14, -1.37033343408735e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0.728431359084681, 0, 2.32737334062814, -0.470200420431712, 0, 0, 4.49691252107734, 0, -0.917928424547679, 0.185449546754743, 0, 0, 1.80505771145776e-14, 7.06540869306227, 0, 0, -1.01980390271859, 0, 0.208166599946618, -0.042056004125377, 0, 0, 0, 0, 9.92395326897748, 0, 0, 0, -0.710100291678884, 0, 0, 0.102494148638114, 0, -0.0209215304820281, 0.00422678745047274, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, -2.50163969655578e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {1.99489143482414, 3.15420030940281, -1.09264703862701, 1.54523626091315, 2.9504842217604, -0.761811750275062, 0.263899331455876, 5.59815020469134, -1.93925611658551, -2.28543525082518, 2.13307290077017, -0.257539376818833, 0.0664963811608233, -0.0230350221379902, 9.14446135479768, -2.36108976914455, 0.817905488277909, -3.33842298506508, 1.1564636454577, 1.02505848514083, 13.0453753561106, -1.2103323911658e-14, 0, -1.46406519642548e-14, 0, 11.5766222110848, -1.39771878815472, 0.360889439280293, -0.125015768949695, -2.15192884958006, 0.555625639776451, -0.192474367616164, 0.595437196138645, -0.206265495285706, -0.166687691932928, 0, 0, 1.33850600318309e-14, 0, 0, 0, 2.24412658009199e-14, 0, 0, 0, 1.66496943714949e-14, 1.3113069716231e-14, 0, -1.15323982020285e-14, 0, 0, 0, -1.02677636674246e-14, 0, 0, 0},
    {1.7593288763725, 1.66904592079256, 0.96362411165943, 1.36277028773851, 0.86736083311088, 2.28430636201791, -1.00852844760552, 2.96226264871973, 1.71026313764871, -2.01556443707463, 0.268741924943271, 2.04415543160776, -0.527798662911752, 0.18283482006601, 2.68821895760395, 7.07977053255616, -3.12574097035513, -1.76652739792481, -1.01990506872277, 0.904016610326402, 0, 13.0453753561106, 0, 0, 0, 1.4585173044132, 11.0940489487798, -2.86447112137559, 0.992281903807266, -0.632607619454896, -1.66605356688627, 0.735567892896463, 0.315075748468093, 0.181909068193174, -0.147004726489972, 0, 0, -1.03074285928854e-14, 0, 0, 0, 0, -1.72927567727304e-14, 1.04819480446677e-14, 1.76046313417165e-14, 0, 0, 0, 0, 0, 1.43579679437173e-14, 0, 0, 0, 0, 0},
    {1.48690428533297, 0.705300630647553, 2.03602754481992, 1.1517511068998, 0.14661073969771, 1.81702705031798, 0.65566316039122, 1.2517844405944, 3.61359041872288, -1.70346285967312, 0, 0.614267257013221, 1.78428514239955, -0.618094504365253, 0.45439193793218, 5.631527706439, 2.03210252328837, -0.746494073225325, -2.15494277062552, 0.764033484562605, 0, 0, 13.0453753561106, 0, 0, 0, 3.33375383865873, 9.68367982311024, -3.3545250915713, -0.1069302042312, -1.32524448062077, -0.478206411017785, 0.133143804689794, 0.384353057392897, -0.124241670058195, 0, 0, 0, 0, 0, 0, 0, 0, 1.35190501684936e-14, 2.13313974056602e-14, 0, 0, 0, -1.39901730746776e-14, 0, 0, 0, 0, 0, 0, 0},
    {1.15175110689978, 0.18210783977117, 2.20794021658194, 0.892142571199777, 0, 0.615765106730376, 2.13307290077017, 0.323209352764267, 3.91870514328392, -1.31949665727937, -1.1268544360518e-14, 0, 1.07496769977313, 1.11713920350151, 2.42487345110409e-14, 1.90844613931506, 6.61105135360481, -0.192743940909627, -2.33689608954558, 0.591817792331174, 0, 0, 0, 13.0453753561106, 0, 0, -1.3756522835902e-14, 5.83406921765278, 6.06294257990891, 0, -0.449106857771022, -1.55575179137408, 0.0343775825476135, 0.416806037297048, -0.0962371838080746, 0, 0, 0, 0, 0, 0, 0, 0, 1.42803357596901e-14, 0, 0, 0, 0, -1.21198190345942e-14, 0, 0, 0, 0, 0, -1.3839902652557e-14, 0},
    {0.664963811608031, 1.90472468148085e-14, 1.45686271816934, 0.515078753637701, 0, 0, 1.8472953201911, 0, 2.58567482211402, -0.761811750275038, 0, 1.08037159720826e-14, 2.26487951659612e-14, 1.71994831963699, 0, 1.52639255690414e-14, 5.72533841794521, 0, -1.54195152727693, 0.3416861617136, 0, 0, -1.00168576142402e-14, 0, 13.0453753561106, 0, 0, 0, 9.33451074824442, 0, 0, -1.34732057331308, 0, 0.275020660380927, -0.0555625639776386, 0, 0, 0, 0, 0, -2.03433658742105e-14, 0, 0, 0, 0, -3.84929350411527e-14, 0, 1.299574464293e-14, 0, 0, 0, 0, 0, 0, 0, 0},
    {1.57359158493892, 1.49284005458436, -0.43094580368567, 3.04724700110022, 0.775791113542717, -0.120185042515459, 0.0346944333244422, 5.29905651979673, -1.52970585407784, 0.901387818866001, -10.576283741361, 0, 0, -1.14454236590933e-14, 4.51561078268938, -0.699555414372402, 0.201944253400485, 6.36462844462229, -1.83730997289732, -2.74825341360594, 0, 0, 0, -1.03903584730172e-14, 0, 14.3499128917217, 0, 0, 1.48731679939335e-14, 10.8920630294282, -1.68739114875269, 0.487107866980279, -3.0999354831996, 0.894874292847882, 0.903959543974665, 0, 0, 0, 0, 0, 0, 0, 0, -1.69818485661453e-14, 1.01890667576586e-14, 1.15185313992138e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1.11022302462516e-14},
    {1.3299276232161, 0.630840061880575, 0.728431359084684, 2.57539376818859, 0.131132632078234, 0.711024300256703, -0.205255035576781, 2.23926008187655, 2.585674822114, 0.761811750275056, 0, -10.576283741361, 1.57303341744021e-14, 1.70725378740663e-14, 0.763277533137587, 4.13862564412658, -1.19471831485577, 2.68954640486935, 3.10562068169859, -2.32269806558125, 0, 0, 0, 0, 0, 0, 14.3499128917217, 0, 0, 1.84109468241849, 9.98274066131698, -2.88176900403078, -1.30996183150502, -1.51261363209512, 0.763985254692618, 0, 0, 0, 0, 0, 1.04560213685676e-14, 0, 0, 1.2776556039649e-14, -1.21474404387607e-14, 0, 0, 0, 1.05952301335811e-14, 0, 0, 1.33062123785088e-14, 1.01731376449221e-14, 1.65118907931275e-14, 0, 0},
    {1.03015750727544, 0.162882203585583, 1.12848100903609, 1.99489143482414, 0, 0.314718316987774, 0.476969600708478, 0.578174466995633, 4.00571020990216, 0.590096844352089, 0, 0, -10.5762837413611, 0, 0, 1.83186607953021, 2.77627448210726, 0.694437895660724, 4.8112068719424, -1.79915418525285, 0, 0, 0, 0, 0, 0, 0, 14.3499128917217, 0, 0, 4.41862723780437, 6.69662601613678, -0.338230690505753, -2.3433309625403, 0.591780433634519, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, -1.48270629766312e-14, 0, 0, 0, 0, 0, -1.15317140110751e-14, 0},
    {0.59476171413317, 0, 0.814411017927954, 1.15175110689979, 0, 0, 0.590096844352077, 1.28927856428244e-14, 2.89087233497831, 0.340692571934635, 0, 0, 0, -10.576283741361, 0, 0, 3.4347488991191, -1.66893313716044e-14, 3.47218947830366, -1.03874215316937, 0, 0, 0, 0, 0, 0, 0, 0, 14.3499128917217, 0, 0, 8.2849260708832, 0, -1.69115345252879, 0.341664592660046, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, -1.19245108709972e-14, -1.0049703503871e-14, 0, -1.01285720807823e-14, 1.46934267294508e-14, 0, 0, 0, 0, -1.33537813474028e-14, 1.11022302462516e-14},
    {1.15175110689978, 0.546323519313492, -0.105140010313412, 3.27118942773249, 9.65295620481433, 0, 0, 2.69341127303539, -0.518347241175101, 4.25171145123355, 0, 0, -1.03926949302858e-14, 0, -9.915266007526, 2.32678428089351e-14, -1.46605852864906e-14, 6.71889089790561, -1.29305115618716, 1.82865048192215, 0, 0, 1.09752665623958e-14, 0, 0, 0, 0, 0, 0, 15.9443476574685, -1.29058000479168e-14, 0, 9.45383520059453, -1.81939143264587, -2.94058061635793, 0, 0, 0, 0, 0, 0, 0, 0, -1.22633667590031e-14, 0, 0, 0, 1.71762874424691e-14, 0, 0, 0, 0, 0, 0, 0, 2.15938378289593e-14},
    {0.89214257119978, 0.141060126129492, 0.407205508963968, 2.53385243517952, 0, 9.65295620481434, 0, 0.695435800330236, 2.00755023262381, 3.29336152870136, 0, 1.78864735610198e-14, -1.45456143897546e-14, -1.59407140440659e-14, -1.27256502403865e-14, -9.91526600752597, 1.74986266317689e-14, 1.73481017017152, 5.00796559370718, 1.41646657250367, 0, 0, 0, 0, 0, 0, 0, 0, 1.67839306981372e-14, 0, 15.9443476574685, 0, 2.44096975264614, 7.04647271886991, -2.27776395106694, 0, 0, 0, 0, 0, 0, 0, 0, -1.25723596525151e-14, -1.13140881999668e-14, 1.41799567998805e-14, 0, 1.21347241236521e-14, 0, 0, 0, 0, 0, 0, 0, -1.24344978758018e-14},
    {0.515078753637724, -2.3619485326075e-14, 0.376160336345333, 1.4629203855377, 0, 1.01804649671178e-14, 9.65295620481433, -2.11069547491802e-14, 1.8544954675473, 1.90142316513451, 0, 0, -2.14622752906569e-14, -4.34603909418763e-14, -1.24631731336805e-14, 0, -9.91526600752596, -1.51138454398741e-14, 4.62616045379076, 0.817797356933143, 0, 0, 0, 0, -1.03207533752511e-14, 0, 0, 0, 2.46817376593445e-14, 0, 0, 15.9443476574685, 1.00089905485244e-14, 6.50925267372302, -1.3150676302989, 0, 0, 0, 0, 0, -1.6390734838769e-14, 0, 0, -1.2109391798167e-14, 0, 1.09260746948048e-14, 0, 0, 0, -1.16429829249248e-14, 0, 0, 0, 0, 0, 1.24344978758018e-14},
    {0.728431359084676, -7.94708263760856, 0, 2.53908227033122, 0, 0, -1.27415633137685e-14, 10.2207771580387, 0, 5.0071377623777, 2.28612268350337e-14, -1.8719154526289e-14, 1.31033584296945e-14, -2.53050332009669e-14, 0, 2.83491039062371e-14, 1.64090158394923e-14, -8.49879943502228, 0, 6.93924068068621, 0, -1.52898531740193e-14, 0, 0, 0, -1.42660700700925e-14, 0, 0, 1.84038304706445e-14, 0, -1.43708740249419e-14, 0, 17.9373911146521, 0, 6.27677936394721, 0, 0, 0, 0, 0, 0, 0, 1.74801884942513e-14, 0, 1.54335530678735e-14, 0, 0, 0, 0, -1.82587310754697e-14, 0, 0, 0, -1.41280724493113e-14, 0, 1.68753899743024e-14},
    {0.420560041253715, 0, -7.94708263760862, 1.46593983227031, 0, 0, 0, 0, 10.2207771580387, 2.89087233497828, 0, 0, 0, -4.72854335057725e-14, 0, 2.90947883710161e-14, 2.94685265414151e-14, 0, -8.49879943502229, 4.00637247496575, 0, 0, 0, -1.04474568447982e-14, -1.49316504487054e-14, 0, 0, 0, 2.10839214785391e-14, 0, 0, -1.06464761849486e-14, 0, 17.9373911146521, 3.62390025541881, 0, 0, 0, 0, 0, -1.23540168908514e-14, 0, 0, 0, 2.09495610608972e-14, 1.91267628119894e-14, 0, 0, 0, -2.42410393446135e-14, 0, 0, 0, 0, 0, 0},
    {6.542378855465, 2.78160733626274e-14, 0, -6.91050664139872, 0, 0, -1.56111299636533e-14, 1.45917324702114e-14, 0, 12.2649325896465, 0, -1.97411729296339e-14, 0, 0, 0, 0, 1.23535778391715e-14, 2.43644083996432e-14, -1.70040757261543e-14, -5.66586629001476, 0, 0, 0, 0, 0, 0, 1.69114983860161e-14, 0, 0, 0, 0, 0, 0, 0, 20.4998755596024, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, -1.54091873433635e-14, 0, -2.20095446006121e-14, 0, 0, 1.1817028848966e-14, -1.05566456925328e-14, -1.23638867682084e-14, 2.1316282072803e-14}};
    
    // Compute reference derivatives
    // Declare pointer to array of derivatives on FIAT element
    double *derivatives = new double [num_derivatives];
    
    // Declare coefficients
    double coeff0_0 = 0;
    double coeff0_1 = 0;
    double coeff0_2 = 0;
    double coeff0_3 = 0;
    double coeff0_4 = 0;
    double coeff0_5 = 0;
    double coeff0_6 = 0;
    double coeff0_7 = 0;
    double coeff0_8 = 0;
    double coeff0_9 = 0;
    double coeff0_10 = 0;
    double coeff0_11 = 0;
    double coeff0_12 = 0;
    double coeff0_13 = 0;
    double coeff0_14 = 0;
    double coeff0_15 = 0;
    double coeff0_16 = 0;
    double coeff0_17 = 0;
    double coeff0_18 = 0;
    double coeff0_19 = 0;
    double coeff0_20 = 0;
    double coeff0_21 = 0;
    double coeff0_22 = 0;
    double coeff0_23 = 0;
    double coeff0_24 = 0;
    double coeff0_25 = 0;
    double coeff0_26 = 0;
    double coeff0_27 = 0;
    double coeff0_28 = 0;
    double coeff0_29 = 0;
    double coeff0_30 = 0;
    double coeff0_31 = 0;
    double coeff0_32 = 0;
    double coeff0_33 = 0;
    double coeff0_34 = 0;
    double coeff0_35 = 0;
    double coeff0_36 = 0;
    double coeff0_37 = 0;
    double coeff0_38 = 0;
    double coeff0_39 = 0;
    double coeff0_40 = 0;
    double coeff0_41 = 0;
    double coeff0_42 = 0;
    double coeff0_43 = 0;
    double coeff0_44 = 0;
    double coeff0_45 = 0;
    double coeff0_46 = 0;
    double coeff0_47 = 0;
    double coeff0_48 = 0;
    double coeff0_49 = 0;
    double coeff0_50 = 0;
    double coeff0_51 = 0;
    double coeff0_52 = 0;
    double coeff0_53 = 0;
    double coeff0_54 = 0;
    double coeff0_55 = 0;
    
    // Declare new coefficients
    double new_coeff0_0 = 0;
    double new_coeff0_1 = 0;
    double new_coeff0_2 = 0;
    double new_coeff0_3 = 0;
    double new_coeff0_4 = 0;
    double new_coeff0_5 = 0;
    double new_coeff0_6 = 0;
    double new_coeff0_7 = 0;
    double new_coeff0_8 = 0;
    double new_coeff0_9 = 0;
    double new_coeff0_10 = 0;
    double new_coeff0_11 = 0;
    double new_coeff0_12 = 0;
    double new_coeff0_13 = 0;
    double new_coeff0_14 = 0;
    double new_coeff0_15 = 0;
    double new_coeff0_16 = 0;
    double new_coeff0_17 = 0;
    double new_coeff0_18 = 0;
    double new_coeff0_19 = 0;
    double new_coeff0_20 = 0;
    double new_coeff0_21 = 0;
    double new_coeff0_22 = 0;
    double new_coeff0_23 = 0;
    double new_coeff0_24 = 0;
    double new_coeff0_25 = 0;
    double new_coeff0_26 = 0;
    double new_coeff0_27 = 0;
    double new_coeff0_28 = 0;
    double new_coeff0_29 = 0;
    double new_coeff0_30 = 0;
    double new_coeff0_31 = 0;
    double new_coeff0_32 = 0;
    double new_coeff0_33 = 0;
    double new_coeff0_34 = 0;
    double new_coeff0_35 = 0;
    double new_coeff0_36 = 0;
    double new_coeff0_37 = 0;
    double new_coeff0_38 = 0;
    double new_coeff0_39 = 0;
    double new_coeff0_40 = 0;
    double new_coeff0_41 = 0;
    double new_coeff0_42 = 0;
    double new_coeff0_43 = 0;
    double new_coeff0_44 = 0;
    double new_coeff0_45 = 0;
    double new_coeff0_46 = 0;
    double new_coeff0_47 = 0;
    double new_coeff0_48 = 0;
    double new_coeff0_49 = 0;
    double new_coeff0_50 = 0;
    double new_coeff0_51 = 0;
    double new_coeff0_52 = 0;
    double new_coeff0_53 = 0;
    double new_coeff0_54 = 0;
    double new_coeff0_55 = 0;
    
    // Loop possible derivatives
    for (unsigned int deriv_num = 0; deriv_num < num_derivatives; deriv_num++)
    {
      // Get values from coefficients array
      new_coeff0_0 = coefficients0[dof][0];
      new_coeff0_1 = coefficients0[dof][1];
      new_coeff0_2 = coefficients0[dof][2];
      new_coeff0_3 = coefficients0[dof][3];
      new_coeff0_4 = coefficients0[dof][4];
      new_coeff0_5 = coefficients0[dof][5];
      new_coeff0_6 = coefficients0[dof][6];
      new_coeff0_7 = coefficients0[dof][7];
      new_coeff0_8 = coefficients0[dof][8];
      new_coeff0_9 = coefficients0[dof][9];
      new_coeff0_10 = coefficients0[dof][10];
      new_coeff0_11 = coefficients0[dof][11];
      new_coeff0_12 = coefficients0[dof][12];
      new_coeff0_13 = coefficients0[dof][13];
      new_coeff0_14 = coefficients0[dof][14];
      new_coeff0_15 = coefficients0[dof][15];
      new_coeff0_16 = coefficients0[dof][16];
      new_coeff0_17 = coefficients0[dof][17];
      new_coeff0_18 = coefficients0[dof][18];
      new_coeff0_19 = coefficients0[dof][19];
      new_coeff0_20 = coefficients0[dof][20];
      new_coeff0_21 = coefficients0[dof][21];
      new_coeff0_22 = coefficients0[dof][22];
      new_coeff0_23 = coefficients0[dof][23];
      new_coeff0_24 = coefficients0[dof][24];
      new_coeff0_25 = coefficients0[dof][25];
      new_coeff0_26 = coefficients0[dof][26];
      new_coeff0_27 = coefficients0[dof][27];
      new_coeff0_28 = coefficients0[dof][28];
      new_coeff0_29 = coefficients0[dof][29];
      new_coeff0_30 = coefficients0[dof][30];
      new_coeff0_31 = coefficients0[dof][31];
      new_coeff0_32 = coefficients0[dof][32];
      new_coeff0_33 = coefficients0[dof][33];
      new_coeff0_34 = coefficients0[dof][34];
      new_coeff0_35 = coefficients0[dof][35];
      new_coeff0_36 = coefficients0[dof][36];
      new_coeff0_37 = coefficients0[dof][37];
      new_coeff0_38 = coefficients0[dof][38];
      new_coeff0_39 = coefficients0[dof][39];
      new_coeff0_40 = coefficients0[dof][40];
      new_coeff0_41 = coefficients0[dof][41];
      new_coeff0_42 = coefficients0[dof][42];
      new_coeff0_43 = coefficients0[dof][43];
      new_coeff0_44 = coefficients0[dof][44];
      new_coeff0_45 = coefficients0[dof][45];
      new_coeff0_46 = coefficients0[dof][46];
      new_coeff0_47 = coefficients0[dof][47];
      new_coeff0_48 = coefficients0[dof][48];
      new_coeff0_49 = coefficients0[dof][49];
      new_coeff0_50 = coefficients0[dof][50];
      new_coeff0_51 = coefficients0[dof][51];
      new_coeff0_52 = coefficients0[dof][52];
      new_coeff0_53 = coefficients0[dof][53];
      new_coeff0_54 = coefficients0[dof][54];
      new_coeff0_55 = coefficients0[dof][55];
    
      // Loop derivative order
      for (unsigned int j = 0; j < n; j++)
      {
        // Update old coefficients
        coeff0_0 = new_coeff0_0;
        coeff0_1 = new_coeff0_1;
        coeff0_2 = new_coeff0_2;
        coeff0_3 = new_coeff0_3;
        coeff0_4 = new_coeff0_4;
        coeff0_5 = new_coeff0_5;
        coeff0_6 = new_coeff0_6;
        coeff0_7 = new_coeff0_7;
        coeff0_8 = new_coeff0_8;
        coeff0_9 = new_coeff0_9;
        coeff0_10 = new_coeff0_10;
        coeff0_11 = new_coeff0_11;
        coeff0_12 = new_coeff0_12;
        coeff0_13 = new_coeff0_13;
        coeff0_14 = new_coeff0_14;
        coeff0_15 = new_coeff0_15;
        coeff0_16 = new_coeff0_16;
        coeff0_17 = new_coeff0_17;
        coeff0_18 = new_coeff0_18;
        coeff0_19 = new_coeff0_19;
        coeff0_20 = new_coeff0_20;
        coeff0_21 = new_coeff0_21;
        coeff0_22 = new_coeff0_22;
        coeff0_23 = new_coeff0_23;
        coeff0_24 = new_coeff0_24;
        coeff0_25 = new_coeff0_25;
        coeff0_26 = new_coeff0_26;
        coeff0_27 = new_coeff0_27;
        coeff0_28 = new_coeff0_28;
        coeff0_29 = new_coeff0_29;
        coeff0_30 = new_coeff0_30;
        coeff0_31 = new_coeff0_31;
        coeff0_32 = new_coeff0_32;
        coeff0_33 = new_coeff0_33;
        coeff0_34 = new_coeff0_34;
        coeff0_35 = new_coeff0_35;
        coeff0_36 = new_coeff0_36;
        coeff0_37 = new_coeff0_37;
        coeff0_38 = new_coeff0_38;
        coeff0_39 = new_coeff0_39;
        coeff0_40 = new_coeff0_40;
        coeff0_41 = new_coeff0_41;
        coeff0_42 = new_coeff0_42;
        coeff0_43 = new_coeff0_43;
        coeff0_44 = new_coeff0_44;
        coeff0_45 = new_coeff0_45;
        coeff0_46 = new_coeff0_46;
        coeff0_47 = new_coeff0_47;
        coeff0_48 = new_coeff0_48;
        coeff0_49 = new_coeff0_49;
        coeff0_50 = new_coeff0_50;
        coeff0_51 = new_coeff0_51;
        coeff0_52 = new_coeff0_52;
        coeff0_53 = new_coeff0_53;
        coeff0_54 = new_coeff0_54;
        coeff0_55 = new_coeff0_55;
    
        if(combinations[deriv_num][j] == 0)
        {
          new_coeff0_0 = coeff0_0*dmats0[0][0] + coeff0_1*dmats0[1][0] + coeff0_2*dmats0[2][0] + coeff0_3*dmats0[3][0] + coeff0_4*dmats0[4][0] + coeff0_5*dmats0[5][0] + coeff0_6*dmats0[6][0] + coeff0_7*dmats0[7][0] + coeff0_8*dmats0[8][0] + coeff0_9*dmats0[9][0] + coeff0_10*dmats0[10][0] + coeff0_11*dmats0[11][0] + coeff0_12*dmats0[12][0] + coeff0_13*dmats0[13][0] + coeff0_14*dmats0[14][0] + coeff0_15*dmats0[15][0] + coeff0_16*dmats0[16][0] + coeff0_17*dmats0[17][0] + coeff0_18*dmats0[18][0] + coeff0_19*dmats0[19][0] + coeff0_20*dmats0[20][0] + coeff0_21*dmats0[21][0] + coeff0_22*dmats0[22][0] + coeff0_23*dmats0[23][0] + coeff0_24*dmats0[24][0] + coeff0_25*dmats0[25][0] + coeff0_26*dmats0[26][0] + coeff0_27*dmats0[27][0] + coeff0_28*dmats0[28][0] + coeff0_29*dmats0[29][0] + coeff0_30*dmats0[30][0] + coeff0_31*dmats0[31][0] + coeff0_32*dmats0[32][0] + coeff0_33*dmats0[33][0] + coeff0_34*dmats0[34][0] + coeff0_35*dmats0[35][0] + coeff0_36*dmats0[36][0] + coeff0_37*dmats0[37][0] + coeff0_38*dmats0[38][0] + coeff0_39*dmats0[39][0] + coeff0_40*dmats0[40][0] + coeff0_41*dmats0[41][0] + coeff0_42*dmats0[42][0] + coeff0_43*dmats0[43][0] + coeff0_44*dmats0[44][0] + coeff0_45*dmats0[45][0] + coeff0_46*dmats0[46][0] + coeff0_47*dmats0[47][0] + coeff0_48*dmats0[48][0] + coeff0_49*dmats0[49][0] + coeff0_50*dmats0[50][0] + coeff0_51*dmats0[51][0] + coeff0_52*dmats0[52][0] + coeff0_53*dmats0[53][0] + coeff0_54*dmats0[54][0] + coeff0_55*dmats0[55][0];
          new_coeff0_1 = coeff0_0*dmats0[0][1] + coeff0_1*dmats0[1][1] + coeff0_2*dmats0[2][1] + coeff0_3*dmats0[3][1] + coeff0_4*dmats0[4][1] + coeff0_5*dmats0[5][1] + coeff0_6*dmats0[6][1] + coeff0_7*dmats0[7][1] + coeff0_8*dmats0[8][1] + coeff0_9*dmats0[9][1] + coeff0_10*dmats0[10][1] + coeff0_11*dmats0[11][1] + coeff0_12*dmats0[12][1] + coeff0_13*dmats0[13][1] + coeff0_14*dmats0[14][1] + coeff0_15*dmats0[15][1] + coeff0_16*dmats0[16][1] + coeff0_17*dmats0[17][1] + coeff0_18*dmats0[18][1] + coeff0_19*dmats0[19][1] + coeff0_20*dmats0[20][1] + coeff0_21*dmats0[21][1] + coeff0_22*dmats0[22][1] + coeff0_23*dmats0[23][1] + coeff0_24*dmats0[24][1] + coeff0_25*dmats0[25][1] + coeff0_26*dmats0[26][1] + coeff0_27*dmats0[27][1] + coeff0_28*dmats0[28][1] + coeff0_29*dmats0[29][1] + coeff0_30*dmats0[30][1] + coeff0_31*dmats0[31][1] + coeff0_32*dmats0[32][1] + coeff0_33*dmats0[33][1] + coeff0_34*dmats0[34][1] + coeff0_35*dmats0[35][1] + coeff0_36*dmats0[36][1] + coeff0_37*dmats0[37][1] + coeff0_38*dmats0[38][1] + coeff0_39*dmats0[39][1] + coeff0_40*dmats0[40][1] + coeff0_41*dmats0[41][1] + coeff0_42*dmats0[42][1] + coeff0_43*dmats0[43][1] + coeff0_44*dmats0[44][1] + coeff0_45*dmats0[45][1] + coeff0_46*dmats0[46][1] + coeff0_47*dmats0[47][1] + coeff0_48*dmats0[48][1] + coeff0_49*dmats0[49][1] + coeff0_50*dmats0[50][1] + coeff0_51*dmats0[51][1] + coeff0_52*dmats0[52][1] + coeff0_53*dmats0[53][1] + coeff0_54*dmats0[54][1] + coeff0_55*dmats0[55][1];
          new_coeff0_2 = coeff0_0*dmats0[0][2] + coeff0_1*dmats0[1][2] + coeff0_2*dmats0[2][2] + coeff0_3*dmats0[3][2] + coeff0_4*dmats0[4][2] + coeff0_5*dmats0[5][2] + coeff0_6*dmats0[6][2] + coeff0_7*dmats0[7][2] + coeff0_8*dmats0[8][2] + coeff0_9*dmats0[9][2] + coeff0_10*dmats0[10][2] + coeff0_11*dmats0[11][2] + coeff0_12*dmats0[12][2] + coeff0_13*dmats0[13][2] + coeff0_14*dmats0[14][2] + coeff0_15*dmats0[15][2] + coeff0_16*dmats0[16][2] + coeff0_17*dmats0[17][2] + coeff0_18*dmats0[18][2] + coeff0_19*dmats0[19][2] + coeff0_20*dmats0[20][2] + coeff0_21*dmats0[21][2] + coeff0_22*dmats0[22][2] + coeff0_23*dmats0[23][2] + coeff0_24*dmats0[24][2] + coeff0_25*dmats0[25][2] + coeff0_26*dmats0[26][2] + coeff0_27*dmats0[27][2] + coeff0_28*dmats0[28][2] + coeff0_29*dmats0[29][2] + coeff0_30*dmats0[30][2] + coeff0_31*dmats0[31][2] + coeff0_32*dmats0[32][2] + coeff0_33*dmats0[33][2] + coeff0_34*dmats0[34][2] + coeff0_35*dmats0[35][2] + coeff0_36*dmats0[36][2] + coeff0_37*dmats0[37][2] + coeff0_38*dmats0[38][2] + coeff0_39*dmats0[39][2] + coeff0_40*dmats0[40][2] + coeff0_41*dmats0[41][2] + coeff0_42*dmats0[42][2] + coeff0_43*dmats0[43][2] + coeff0_44*dmats0[44][2] + coeff0_45*dmats0[45][2] + coeff0_46*dmats0[46][2] + coeff0_47*dmats0[47][2] + coeff0_48*dmats0[48][2] + coeff0_49*dmats0[49][2] + coeff0_50*dmats0[50][2] + coeff0_51*dmats0[51][2] + coeff0_52*dmats0[52][2] + coeff0_53*dmats0[53][2] + coeff0_54*dmats0[54][2] + coeff0_55*dmats0[55][2];
          new_coeff0_3 = coeff0_0*dmats0[0][3] + coeff0_1*dmats0[1][3] + coeff0_2*dmats0[2][3] + coeff0_3*dmats0[3][3] + coeff0_4*dmats0[4][3] + coeff0_5*dmats0[5][3] + coeff0_6*dmats0[6][3] + coeff0_7*dmats0[7][3] + coeff0_8*dmats0[8][3] + coeff0_9*dmats0[9][3] + coeff0_10*dmats0[10][3] + coeff0_11*dmats0[11][3] + coeff0_12*dmats0[12][3] + coeff0_13*dmats0[13][3] + coeff0_14*dmats0[14][3] + coeff0_15*dmats0[15][3] + coeff0_16*dmats0[16][3] + coeff0_17*dmats0[17][3] + coeff0_18*dmats0[18][3] + coeff0_19*dmats0[19][3] + coeff0_20*dmats0[20][3] + coeff0_21*dmats0[21][3] + coeff0_22*dmats0[22][3] + coeff0_23*dmats0[23][3] + coeff0_24*dmats0[24][3] + coeff0_25*dmats0[25][3] + coeff0_26*dmats0[26][3] + coeff0_27*dmats0[27][3] + coeff0_28*dmats0[28][3] + coeff0_29*dmats0[29][3] + coeff0_30*dmats0[30][3] + coeff0_31*dmats0[31][3] + coeff0_32*dmats0[32][3] + coeff0_33*dmats0[33][3] + coeff0_34*dmats0[34][3] + coeff0_35*dmats0[35][3] + coeff0_36*dmats0[36][3] + coeff0_37*dmats0[37][3] + coeff0_38*dmats0[38][3] + coeff0_39*dmats0[39][3] + coeff0_40*dmats0[40][3] + coeff0_41*dmats0[41][3] + coeff0_42*dmats0[42][3] + coeff0_43*dmats0[43][3] + coeff0_44*dmats0[44][3] + coeff0_45*dmats0[45][3] + coeff0_46*dmats0[46][3] + coeff0_47*dmats0[47][3] + coeff0_48*dmats0[48][3] + coeff0_49*dmats0[49][3] + coeff0_50*dmats0[50][3] + coeff0_51*dmats0[51][3] + coeff0_52*dmats0[52][3] + coeff0_53*dmats0[53][3] + coeff0_54*dmats0[54][3] + coeff0_55*dmats0[55][3];
          new_coeff0_4 = coeff0_0*dmats0[0][4] + coeff0_1*dmats0[1][4] + coeff0_2*dmats0[2][4] + coeff0_3*dmats0[3][4] + coeff0_4*dmats0[4][4] + coeff0_5*dmats0[5][4] + coeff0_6*dmats0[6][4] + coeff0_7*dmats0[7][4] + coeff0_8*dmats0[8][4] + coeff0_9*dmats0[9][4] + coeff0_10*dmats0[10][4] + coeff0_11*dmats0[11][4] + coeff0_12*dmats0[12][4] + coeff0_13*dmats0[13][4] + coeff0_14*dmats0[14][4] + coeff0_15*dmats0[15][4] + coeff0_16*dmats0[16][4] + coeff0_17*dmats0[17][4] + coeff0_18*dmats0[18][4] + coeff0_19*dmats0[19][4] + coeff0_20*dmats0[20][4] + coeff0_21*dmats0[21][4] + coeff0_22*dmats0[22][4] + coeff0_23*dmats0[23][4] + coeff0_24*dmats0[24][4] + coeff0_25*dmats0[25][4] + coeff0_26*dmats0[26][4] + coeff0_27*dmats0[27][4] + coeff0_28*dmats0[28][4] + coeff0_29*dmats0[29][4] + coeff0_30*dmats0[30][4] + coeff0_31*dmats0[31][4] + coeff0_32*dmats0[32][4] + coeff0_33*dmats0[33][4] + coeff0_34*dmats0[34][4] + coeff0_35*dmats0[35][4] + coeff0_36*dmats0[36][4] + coeff0_37*dmats0[37][4] + coeff0_38*dmats0[38][4] + coeff0_39*dmats0[39][4] + coeff0_40*dmats0[40][4] + coeff0_41*dmats0[41][4] + coeff0_42*dmats0[42][4] + coeff0_43*dmats0[43][4] + coeff0_44*dmats0[44][4] + coeff0_45*dmats0[45][4] + coeff0_46*dmats0[46][4] + coeff0_47*dmats0[47][4] + coeff0_48*dmats0[48][4] + coeff0_49*dmats0[49][4] + coeff0_50*dmats0[50][4] + coeff0_51*dmats0[51][4] + coeff0_52*dmats0[52][4] + coeff0_53*dmats0[53][4] + coeff0_54*dmats0[54][4] + coeff0_55*dmats0[55][4];
          new_coeff0_5 = coeff0_0*dmats0[0][5] + coeff0_1*dmats0[1][5] + coeff0_2*dmats0[2][5] + coeff0_3*dmats0[3][5] + coeff0_4*dmats0[4][5] + coeff0_5*dmats0[5][5] + coeff0_6*dmats0[6][5] + coeff0_7*dmats0[7][5] + coeff0_8*dmats0[8][5] + coeff0_9*dmats0[9][5] + coeff0_10*dmats0[10][5] + coeff0_11*dmats0[11][5] + coeff0_12*dmats0[12][5] + coeff0_13*dmats0[13][5] + coeff0_14*dmats0[14][5] + coeff0_15*dmats0[15][5] + coeff0_16*dmats0[16][5] + coeff0_17*dmats0[17][5] + coeff0_18*dmats0[18][5] + coeff0_19*dmats0[19][5] + coeff0_20*dmats0[20][5] + coeff0_21*dmats0[21][5] + coeff0_22*dmats0[22][5] + coeff0_23*dmats0[23][5] + coeff0_24*dmats0[24][5] + coeff0_25*dmats0[25][5] + coeff0_26*dmats0[26][5] + coeff0_27*dmats0[27][5] + coeff0_28*dmats0[28][5] + coeff0_29*dmats0[29][5] + coeff0_30*dmats0[30][5] + coeff0_31*dmats0[31][5] + coeff0_32*dmats0[32][5] + coeff0_33*dmats0[33][5] + coeff0_34*dmats0[34][5] + coeff0_35*dmats0[35][5] + coeff0_36*dmats0[36][5] + coeff0_37*dmats0[37][5] + coeff0_38*dmats0[38][5] + coeff0_39*dmats0[39][5] + coeff0_40*dmats0[40][5] + coeff0_41*dmats0[41][5] + coeff0_42*dmats0[42][5] + coeff0_43*dmats0[43][5] + coeff0_44*dmats0[44][5] + coeff0_45*dmats0[45][5] + coeff0_46*dmats0[46][5] + coeff0_47*dmats0[47][5] + coeff0_48*dmats0[48][5] + coeff0_49*dmats0[49][5] + coeff0_50*dmats0[50][5] + coeff0_51*dmats0[51][5] + coeff0_52*dmats0[52][5] + coeff0_53*dmats0[53][5] + coeff0_54*dmats0[54][5] + coeff0_55*dmats0[55][5];
          new_coeff0_6 = coeff0_0*dmats0[0][6] + coeff0_1*dmats0[1][6] + coeff0_2*dmats0[2][6] + coeff0_3*dmats0[3][6] + coeff0_4*dmats0[4][6] + coeff0_5*dmats0[5][6] + coeff0_6*dmats0[6][6] + coeff0_7*dmats0[7][6] + coeff0_8*dmats0[8][6] + coeff0_9*dmats0[9][6] + coeff0_10*dmats0[10][6] + coeff0_11*dmats0[11][6] + coeff0_12*dmats0[12][6] + coeff0_13*dmats0[13][6] + coeff0_14*dmats0[14][6] + coeff0_15*dmats0[15][6] + coeff0_16*dmats0[16][6] + coeff0_17*dmats0[17][6] + coeff0_18*dmats0[18][6] + coeff0_19*dmats0[19][6] + coeff0_20*dmats0[20][6] + coeff0_21*dmats0[21][6] + coeff0_22*dmats0[22][6] + coeff0_23*dmats0[23][6] + coeff0_24*dmats0[24][6] + coeff0_25*dmats0[25][6] + coeff0_26*dmats0[26][6] + coeff0_27*dmats0[27][6] + coeff0_28*dmats0[28][6] + coeff0_29*dmats0[29][6] + coeff0_30*dmats0[30][6] + coeff0_31*dmats0[31][6] + coeff0_32*dmats0[32][6] + coeff0_33*dmats0[33][6] + coeff0_34*dmats0[34][6] + coeff0_35*dmats0[35][6] + coeff0_36*dmats0[36][6] + coeff0_37*dmats0[37][6] + coeff0_38*dmats0[38][6] + coeff0_39*dmats0[39][6] + coeff0_40*dmats0[40][6] + coeff0_41*dmats0[41][6] + coeff0_42*dmats0[42][6] + coeff0_43*dmats0[43][6] + coeff0_44*dmats0[44][6] + coeff0_45*dmats0[45][6] + coeff0_46*dmats0[46][6] + coeff0_47*dmats0[47][6] + coeff0_48*dmats0[48][6] + coeff0_49*dmats0[49][6] + coeff0_50*dmats0[50][6] + coeff0_51*dmats0[51][6] + coeff0_52*dmats0[52][6] + coeff0_53*dmats0[53][6] + coeff0_54*dmats0[54][6] + coeff0_55*dmats0[55][6];
          new_coeff0_7 = coeff0_0*dmats0[0][7] + coeff0_1*dmats0[1][7] + coeff0_2*dmats0[2][7] + coeff0_3*dmats0[3][7] + coeff0_4*dmats0[4][7] + coeff0_5*dmats0[5][7] + coeff0_6*dmats0[6][7] + coeff0_7*dmats0[7][7] + coeff0_8*dmats0[8][7] + coeff0_9*dmats0[9][7] + coeff0_10*dmats0[10][7] + coeff0_11*dmats0[11][7] + coeff0_12*dmats0[12][7] + coeff0_13*dmats0[13][7] + coeff0_14*dmats0[14][7] + coeff0_15*dmats0[15][7] + coeff0_16*dmats0[16][7] + coeff0_17*dmats0[17][7] + coeff0_18*dmats0[18][7] + coeff0_19*dmats0[19][7] + coeff0_20*dmats0[20][7] + coeff0_21*dmats0[21][7] + coeff0_22*dmats0[22][7] + coeff0_23*dmats0[23][7] + coeff0_24*dmats0[24][7] + coeff0_25*dmats0[25][7] + coeff0_26*dmats0[26][7] + coeff0_27*dmats0[27][7] + coeff0_28*dmats0[28][7] + coeff0_29*dmats0[29][7] + coeff0_30*dmats0[30][7] + coeff0_31*dmats0[31][7] + coeff0_32*dmats0[32][7] + coeff0_33*dmats0[33][7] + coeff0_34*dmats0[34][7] + coeff0_35*dmats0[35][7] + coeff0_36*dmats0[36][7] + coeff0_37*dmats0[37][7] + coeff0_38*dmats0[38][7] + coeff0_39*dmats0[39][7] + coeff0_40*dmats0[40][7] + coeff0_41*dmats0[41][7] + coeff0_42*dmats0[42][7] + coeff0_43*dmats0[43][7] + coeff0_44*dmats0[44][7] + coeff0_45*dmats0[45][7] + coeff0_46*dmats0[46][7] + coeff0_47*dmats0[47][7] + coeff0_48*dmats0[48][7] + coeff0_49*dmats0[49][7] + coeff0_50*dmats0[50][7] + coeff0_51*dmats0[51][7] + coeff0_52*dmats0[52][7] + coeff0_53*dmats0[53][7] + coeff0_54*dmats0[54][7] + coeff0_55*dmats0[55][7];
          new_coeff0_8 = coeff0_0*dmats0[0][8] + coeff0_1*dmats0[1][8] + coeff0_2*dmats0[2][8] + coeff0_3*dmats0[3][8] + coeff0_4*dmats0[4][8] + coeff0_5*dmats0[5][8] + coeff0_6*dmats0[6][8] + coeff0_7*dmats0[7][8] + coeff0_8*dmats0[8][8] + coeff0_9*dmats0[9][8] + coeff0_10*dmats0[10][8] + coeff0_11*dmats0[11][8] + coeff0_12*dmats0[12][8] + coeff0_13*dmats0[13][8] + coeff0_14*dmats0[14][8] + coeff0_15*dmats0[15][8] + coeff0_16*dmats0[16][8] + coeff0_17*dmats0[17][8] + coeff0_18*dmats0[18][8] + coeff0_19*dmats0[19][8] + coeff0_20*dmats0[20][8] + coeff0_21*dmats0[21][8] + coeff0_22*dmats0[22][8] + coeff0_23*dmats0[23][8] + coeff0_24*dmats0[24][8] + coeff0_25*dmats0[25][8] + coeff0_26*dmats0[26][8] + coeff0_27*dmats0[27][8] + coeff0_28*dmats0[28][8] + coeff0_29*dmats0[29][8] + coeff0_30*dmats0[30][8] + coeff0_31*dmats0[31][8] + coeff0_32*dmats0[32][8] + coeff0_33*dmats0[33][8] + coeff0_34*dmats0[34][8] + coeff0_35*dmats0[35][8] + coeff0_36*dmats0[36][8] + coeff0_37*dmats0[37][8] + coeff0_38*dmats0[38][8] + coeff0_39*dmats0[39][8] + coeff0_40*dmats0[40][8] + coeff0_41*dmats0[41][8] + coeff0_42*dmats0[42][8] + coeff0_43*dmats0[43][8] + coeff0_44*dmats0[44][8] + coeff0_45*dmats0[45][8] + coeff0_46*dmats0[46][8] + coeff0_47*dmats0[47][8] + coeff0_48*dmats0[48][8] + coeff0_49*dmats0[49][8] + coeff0_50*dmats0[50][8] + coeff0_51*dmats0[51][8] + coeff0_52*dmats0[52][8] + coeff0_53*dmats0[53][8] + coeff0_54*dmats0[54][8] + coeff0_55*dmats0[55][8];
          new_coeff0_9 = coeff0_0*dmats0[0][9] + coeff0_1*dmats0[1][9] + coeff0_2*dmats0[2][9] + coeff0_3*dmats0[3][9] + coeff0_4*dmats0[4][9] + coeff0_5*dmats0[5][9] + coeff0_6*dmats0[6][9] + coeff0_7*dmats0[7][9] + coeff0_8*dmats0[8][9] + coeff0_9*dmats0[9][9] + coeff0_10*dmats0[10][9] + coeff0_11*dmats0[11][9] + coeff0_12*dmats0[12][9] + coeff0_13*dmats0[13][9] + coeff0_14*dmats0[14][9] + coeff0_15*dmats0[15][9] + coeff0_16*dmats0[16][9] + coeff0_17*dmats0[17][9] + coeff0_18*dmats0[18][9] + coeff0_19*dmats0[19][9] + coeff0_20*dmats0[20][9] + coeff0_21*dmats0[21][9] + coeff0_22*dmats0[22][9] + coeff0_23*dmats0[23][9] + coeff0_24*dmats0[24][9] + coeff0_25*dmats0[25][9] + coeff0_26*dmats0[26][9] + coeff0_27*dmats0[27][9] + coeff0_28*dmats0[28][9] + coeff0_29*dmats0[29][9] + coeff0_30*dmats0[30][9] + coeff0_31*dmats0[31][9] + coeff0_32*dmats0[32][9] + coeff0_33*dmats0[33][9] + coeff0_34*dmats0[34][9] + coeff0_35*dmats0[35][9] + coeff0_36*dmats0[36][9] + coeff0_37*dmats0[37][9] + coeff0_38*dmats0[38][9] + coeff0_39*dmats0[39][9] + coeff0_40*dmats0[40][9] + coeff0_41*dmats0[41][9] + coeff0_42*dmats0[42][9] + coeff0_43*dmats0[43][9] + coeff0_44*dmats0[44][9] + coeff0_45*dmats0[45][9] + coeff0_46*dmats0[46][9] + coeff0_47*dmats0[47][9] + coeff0_48*dmats0[48][9] + coeff0_49*dmats0[49][9] + coeff0_50*dmats0[50][9] + coeff0_51*dmats0[51][9] + coeff0_52*dmats0[52][9] + coeff0_53*dmats0[53][9] + coeff0_54*dmats0[54][9] + coeff0_55*dmats0[55][9];
          new_coeff0_10 = coeff0_0*dmats0[0][10] + coeff0_1*dmats0[1][10] + coeff0_2*dmats0[2][10] + coeff0_3*dmats0[3][10] + coeff0_4*dmats0[4][10] + coeff0_5*dmats0[5][10] + coeff0_6*dmats0[6][10] + coeff0_7*dmats0[7][10] + coeff0_8*dmats0[8][10] + coeff0_9*dmats0[9][10] + coeff0_10*dmats0[10][10] + coeff0_11*dmats0[11][10] + coeff0_12*dmats0[12][10] + coeff0_13*dmats0[13][10] + coeff0_14*dmats0[14][10] + coeff0_15*dmats0[15][10] + coeff0_16*dmats0[16][10] + coeff0_17*dmats0[17][10] + coeff0_18*dmats0[18][10] + coeff0_19*dmats0[19][10] + coeff0_20*dmats0[20][10] + coeff0_21*dmats0[21][10] + coeff0_22*dmats0[22][10] + coeff0_23*dmats0[23][10] + coeff0_24*dmats0[24][10] + coeff0_25*dmats0[25][10] + coeff0_26*dmats0[26][10] + coeff0_27*dmats0[27][10] + coeff0_28*dmats0[28][10] + coeff0_29*dmats0[29][10] + coeff0_30*dmats0[30][10] + coeff0_31*dmats0[31][10] + coeff0_32*dmats0[32][10] + coeff0_33*dmats0[33][10] + coeff0_34*dmats0[34][10] + coeff0_35*dmats0[35][10] + coeff0_36*dmats0[36][10] + coeff0_37*dmats0[37][10] + coeff0_38*dmats0[38][10] + coeff0_39*dmats0[39][10] + coeff0_40*dmats0[40][10] + coeff0_41*dmats0[41][10] + coeff0_42*dmats0[42][10] + coeff0_43*dmats0[43][10] + coeff0_44*dmats0[44][10] + coeff0_45*dmats0[45][10] + coeff0_46*dmats0[46][10] + coeff0_47*dmats0[47][10] + coeff0_48*dmats0[48][10] + coeff0_49*dmats0[49][10] + coeff0_50*dmats0[50][10] + coeff0_51*dmats0[51][10] + coeff0_52*dmats0[52][10] + coeff0_53*dmats0[53][10] + coeff0_54*dmats0[54][10] + coeff0_55*dmats0[55][10];
          new_coeff0_11 = coeff0_0*dmats0[0][11] + coeff0_1*dmats0[1][11] + coeff0_2*dmats0[2][11] + coeff0_3*dmats0[3][11] + coeff0_4*dmats0[4][11] + coeff0_5*dmats0[5][11] + coeff0_6*dmats0[6][11] + coeff0_7*dmats0[7][11] + coeff0_8*dmats0[8][11] + coeff0_9*dmats0[9][11] + coeff0_10*dmats0[10][11] + coeff0_11*dmats0[11][11] + coeff0_12*dmats0[12][11] + coeff0_13*dmats0[13][11] + coeff0_14*dmats0[14][11] + coeff0_15*dmats0[15][11] + coeff0_16*dmats0[16][11] + coeff0_17*dmats0[17][11] + coeff0_18*dmats0[18][11] + coeff0_19*dmats0[19][11] + coeff0_20*dmats0[20][11] + coeff0_21*dmats0[21][11] + coeff0_22*dmats0[22][11] + coeff0_23*dmats0[23][11] + coeff0_24*dmats0[24][11] + coeff0_25*dmats0[25][11] + coeff0_26*dmats0[26][11] + coeff0_27*dmats0[27][11] + coeff0_28*dmats0[28][11] + coeff0_29*dmats0[29][11] + coeff0_30*dmats0[30][11] + coeff0_31*dmats0[31][11] + coeff0_32*dmats0[32][11] + coeff0_33*dmats0[33][11] + coeff0_34*dmats0[34][11] + coeff0_35*dmats0[35][11] + coeff0_36*dmats0[36][11] + coeff0_37*dmats0[37][11] + coeff0_38*dmats0[38][11] + coeff0_39*dmats0[39][11] + coeff0_40*dmats0[40][11] + coeff0_41*dmats0[41][11] + coeff0_42*dmats0[42][11] + coeff0_43*dmats0[43][11] + coeff0_44*dmats0[44][11] + coeff0_45*dmats0[45][11] + coeff0_46*dmats0[46][11] + coeff0_47*dmats0[47][11] + coeff0_48*dmats0[48][11] + coeff0_49*dmats0[49][11] + coeff0_50*dmats0[50][11] + coeff0_51*dmats0[51][11] + coeff0_52*dmats0[52][11] + coeff0_53*dmats0[53][11] + coeff0_54*dmats0[54][11] + coeff0_55*dmats0[55][11];
          new_coeff0_12 = coeff0_0*dmats0[0][12] + coeff0_1*dmats0[1][12] + coeff0_2*dmats0[2][12] + coeff0_3*dmats0[3][12] + coeff0_4*dmats0[4][12] + coeff0_5*dmats0[5][12] + coeff0_6*dmats0[6][12] + coeff0_7*dmats0[7][12] + coeff0_8*dmats0[8][12] + coeff0_9*dmats0[9][12] + coeff0_10*dmats0[10][12] + coeff0_11*dmats0[11][12] + coeff0_12*dmats0[12][12] + coeff0_13*dmats0[13][12] + coeff0_14*dmats0[14][12] + coeff0_15*dmats0[15][12] + coeff0_16*dmats0[16][12] + coeff0_17*dmats0[17][12] + coeff0_18*dmats0[18][12] + coeff0_19*dmats0[19][12] + coeff0_20*dmats0[20][12] + coeff0_21*dmats0[21][12] + coeff0_22*dmats0[22][12] + coeff0_23*dmats0[23][12] + coeff0_24*dmats0[24][12] + coeff0_25*dmats0[25][12] + coeff0_26*dmats0[26][12] + coeff0_27*dmats0[27][12] + coeff0_28*dmats0[28][12] + coeff0_29*dmats0[29][12] + coeff0_30*dmats0[30][12] + coeff0_31*dmats0[31][12] + coeff0_32*dmats0[32][12] + coeff0_33*dmats0[33][12] + coeff0_34*dmats0[34][12] + coeff0_35*dmats0[35][12] + coeff0_36*dmats0[36][12] + coeff0_37*dmats0[37][12] + coeff0_38*dmats0[38][12] + coeff0_39*dmats0[39][12] + coeff0_40*dmats0[40][12] + coeff0_41*dmats0[41][12] + coeff0_42*dmats0[42][12] + coeff0_43*dmats0[43][12] + coeff0_44*dmats0[44][12] + coeff0_45*dmats0[45][12] + coeff0_46*dmats0[46][12] + coeff0_47*dmats0[47][12] + coeff0_48*dmats0[48][12] + coeff0_49*dmats0[49][12] + coeff0_50*dmats0[50][12] + coeff0_51*dmats0[51][12] + coeff0_52*dmats0[52][12] + coeff0_53*dmats0[53][12] + coeff0_54*dmats0[54][12] + coeff0_55*dmats0[55][12];
          new_coeff0_13 = coeff0_0*dmats0[0][13] + coeff0_1*dmats0[1][13] + coeff0_2*dmats0[2][13] + coeff0_3*dmats0[3][13] + coeff0_4*dmats0[4][13] + coeff0_5*dmats0[5][13] + coeff0_6*dmats0[6][13] + coeff0_7*dmats0[7][13] + coeff0_8*dmats0[8][13] + coeff0_9*dmats0[9][13] + coeff0_10*dmats0[10][13] + coeff0_11*dmats0[11][13] + coeff0_12*dmats0[12][13] + coeff0_13*dmats0[13][13] + coeff0_14*dmats0[14][13] + coeff0_15*dmats0[15][13] + coeff0_16*dmats0[16][13] + coeff0_17*dmats0[17][13] + coeff0_18*dmats0[18][13] + coeff0_19*dmats0[19][13] + coeff0_20*dmats0[20][13] + coeff0_21*dmats0[21][13] + coeff0_22*dmats0[22][13] + coeff0_23*dmats0[23][13] + coeff0_24*dmats0[24][13] + coeff0_25*dmats0[25][13] + coeff0_26*dmats0[26][13] + coeff0_27*dmats0[27][13] + coeff0_28*dmats0[28][13] + coeff0_29*dmats0[29][13] + coeff0_30*dmats0[30][13] + coeff0_31*dmats0[31][13] + coeff0_32*dmats0[32][13] + coeff0_33*dmats0[33][13] + coeff0_34*dmats0[34][13] + coeff0_35*dmats0[35][13] + coeff0_36*dmats0[36][13] + coeff0_37*dmats0[37][13] + coeff0_38*dmats0[38][13] + coeff0_39*dmats0[39][13] + coeff0_40*dmats0[40][13] + coeff0_41*dmats0[41][13] + coeff0_42*dmats0[42][13] + coeff0_43*dmats0[43][13] + coeff0_44*dmats0[44][13] + coeff0_45*dmats0[45][13] + coeff0_46*dmats0[46][13] + coeff0_47*dmats0[47][13] + coeff0_48*dmats0[48][13] + coeff0_49*dmats0[49][13] + coeff0_50*dmats0[50][13] + coeff0_51*dmats0[51][13] + coeff0_52*dmats0[52][13] + coeff0_53*dmats0[53][13] + coeff0_54*dmats0[54][13] + coeff0_55*dmats0[55][13];
          new_coeff0_14 = coeff0_0*dmats0[0][14] + coeff0_1*dmats0[1][14] + coeff0_2*dmats0[2][14] + coeff0_3*dmats0[3][14] + coeff0_4*dmats0[4][14] + coeff0_5*dmats0[5][14] + coeff0_6*dmats0[6][14] + coeff0_7*dmats0[7][14] + coeff0_8*dmats0[8][14] + coeff0_9*dmats0[9][14] + coeff0_10*dmats0[10][14] + coeff0_11*dmats0[11][14] + coeff0_12*dmats0[12][14] + coeff0_13*dmats0[13][14] + coeff0_14*dmats0[14][14] + coeff0_15*dmats0[15][14] + coeff0_16*dmats0[16][14] + coeff0_17*dmats0[17][14] + coeff0_18*dmats0[18][14] + coeff0_19*dmats0[19][14] + coeff0_20*dmats0[20][14] + coeff0_21*dmats0[21][14] + coeff0_22*dmats0[22][14] + coeff0_23*dmats0[23][14] + coeff0_24*dmats0[24][14] + coeff0_25*dmats0[25][14] + coeff0_26*dmats0[26][14] + coeff0_27*dmats0[27][14] + coeff0_28*dmats0[28][14] + coeff0_29*dmats0[29][14] + coeff0_30*dmats0[30][14] + coeff0_31*dmats0[31][14] + coeff0_32*dmats0[32][14] + coeff0_33*dmats0[33][14] + coeff0_34*dmats0[34][14] + coeff0_35*dmats0[35][14] + coeff0_36*dmats0[36][14] + coeff0_37*dmats0[37][14] + coeff0_38*dmats0[38][14] + coeff0_39*dmats0[39][14] + coeff0_40*dmats0[40][14] + coeff0_41*dmats0[41][14] + coeff0_42*dmats0[42][14] + coeff0_43*dmats0[43][14] + coeff0_44*dmats0[44][14] + coeff0_45*dmats0[45][14] + coeff0_46*dmats0[46][14] + coeff0_47*dmats0[47][14] + coeff0_48*dmats0[48][14] + coeff0_49*dmats0[49][14] + coeff0_50*dmats0[50][14] + coeff0_51*dmats0[51][14] + coeff0_52*dmats0[52][14] + coeff0_53*dmats0[53][14] + coeff0_54*dmats0[54][14] + coeff0_55*dmats0[55][14];
          new_coeff0_15 = coeff0_0*dmats0[0][15] + coeff0_1*dmats0[1][15] + coeff0_2*dmats0[2][15] + coeff0_3*dmats0[3][15] + coeff0_4*dmats0[4][15] + coeff0_5*dmats0[5][15] + coeff0_6*dmats0[6][15] + coeff0_7*dmats0[7][15] + coeff0_8*dmats0[8][15] + coeff0_9*dmats0[9][15] + coeff0_10*dmats0[10][15] + coeff0_11*dmats0[11][15] + coeff0_12*dmats0[12][15] + coeff0_13*dmats0[13][15] + coeff0_14*dmats0[14][15] + coeff0_15*dmats0[15][15] + coeff0_16*dmats0[16][15] + coeff0_17*dmats0[17][15] + coeff0_18*dmats0[18][15] + coeff0_19*dmats0[19][15] + coeff0_20*dmats0[20][15] + coeff0_21*dmats0[21][15] + coeff0_22*dmats0[22][15] + coeff0_23*dmats0[23][15] + coeff0_24*dmats0[24][15] + coeff0_25*dmats0[25][15] + coeff0_26*dmats0[26][15] + coeff0_27*dmats0[27][15] + coeff0_28*dmats0[28][15] + coeff0_29*dmats0[29][15] + coeff0_30*dmats0[30][15] + coeff0_31*dmats0[31][15] + coeff0_32*dmats0[32][15] + coeff0_33*dmats0[33][15] + coeff0_34*dmats0[34][15] + coeff0_35*dmats0[35][15] + coeff0_36*dmats0[36][15] + coeff0_37*dmats0[37][15] + coeff0_38*dmats0[38][15] + coeff0_39*dmats0[39][15] + coeff0_40*dmats0[40][15] + coeff0_41*dmats0[41][15] + coeff0_42*dmats0[42][15] + coeff0_43*dmats0[43][15] + coeff0_44*dmats0[44][15] + coeff0_45*dmats0[45][15] + coeff0_46*dmats0[46][15] + coeff0_47*dmats0[47][15] + coeff0_48*dmats0[48][15] + coeff0_49*dmats0[49][15] + coeff0_50*dmats0[50][15] + coeff0_51*dmats0[51][15] + coeff0_52*dmats0[52][15] + coeff0_53*dmats0[53][15] + coeff0_54*dmats0[54][15] + coeff0_55*dmats0[55][15];
          new_coeff0_16 = coeff0_0*dmats0[0][16] + coeff0_1*dmats0[1][16] + coeff0_2*dmats0[2][16] + coeff0_3*dmats0[3][16] + coeff0_4*dmats0[4][16] + coeff0_5*dmats0[5][16] + coeff0_6*dmats0[6][16] + coeff0_7*dmats0[7][16] + coeff0_8*dmats0[8][16] + coeff0_9*dmats0[9][16] + coeff0_10*dmats0[10][16] + coeff0_11*dmats0[11][16] + coeff0_12*dmats0[12][16] + coeff0_13*dmats0[13][16] + coeff0_14*dmats0[14][16] + coeff0_15*dmats0[15][16] + coeff0_16*dmats0[16][16] + coeff0_17*dmats0[17][16] + coeff0_18*dmats0[18][16] + coeff0_19*dmats0[19][16] + coeff0_20*dmats0[20][16] + coeff0_21*dmats0[21][16] + coeff0_22*dmats0[22][16] + coeff0_23*dmats0[23][16] + coeff0_24*dmats0[24][16] + coeff0_25*dmats0[25][16] + coeff0_26*dmats0[26][16] + coeff0_27*dmats0[27][16] + coeff0_28*dmats0[28][16] + coeff0_29*dmats0[29][16] + coeff0_30*dmats0[30][16] + coeff0_31*dmats0[31][16] + coeff0_32*dmats0[32][16] + coeff0_33*dmats0[33][16] + coeff0_34*dmats0[34][16] + coeff0_35*dmats0[35][16] + coeff0_36*dmats0[36][16] + coeff0_37*dmats0[37][16] + coeff0_38*dmats0[38][16] + coeff0_39*dmats0[39][16] + coeff0_40*dmats0[40][16] + coeff0_41*dmats0[41][16] + coeff0_42*dmats0[42][16] + coeff0_43*dmats0[43][16] + coeff0_44*dmats0[44][16] + coeff0_45*dmats0[45][16] + coeff0_46*dmats0[46][16] + coeff0_47*dmats0[47][16] + coeff0_48*dmats0[48][16] + coeff0_49*dmats0[49][16] + coeff0_50*dmats0[50][16] + coeff0_51*dmats0[51][16] + coeff0_52*dmats0[52][16] + coeff0_53*dmats0[53][16] + coeff0_54*dmats0[54][16] + coeff0_55*dmats0[55][16];
          new_coeff0_17 = coeff0_0*dmats0[0][17] + coeff0_1*dmats0[1][17] + coeff0_2*dmats0[2][17] + coeff0_3*dmats0[3][17] + coeff0_4*dmats0[4][17] + coeff0_5*dmats0[5][17] + coeff0_6*dmats0[6][17] + coeff0_7*dmats0[7][17] + coeff0_8*dmats0[8][17] + coeff0_9*dmats0[9][17] + coeff0_10*dmats0[10][17] + coeff0_11*dmats0[11][17] + coeff0_12*dmats0[12][17] + coeff0_13*dmats0[13][17] + coeff0_14*dmats0[14][17] + coeff0_15*dmats0[15][17] + coeff0_16*dmats0[16][17] + coeff0_17*dmats0[17][17] + coeff0_18*dmats0[18][17] + coeff0_19*dmats0[19][17] + coeff0_20*dmats0[20][17] + coeff0_21*dmats0[21][17] + coeff0_22*dmats0[22][17] + coeff0_23*dmats0[23][17] + coeff0_24*dmats0[24][17] + coeff0_25*dmats0[25][17] + coeff0_26*dmats0[26][17] + coeff0_27*dmats0[27][17] + coeff0_28*dmats0[28][17] + coeff0_29*dmats0[29][17] + coeff0_30*dmats0[30][17] + coeff0_31*dmats0[31][17] + coeff0_32*dmats0[32][17] + coeff0_33*dmats0[33][17] + coeff0_34*dmats0[34][17] + coeff0_35*dmats0[35][17] + coeff0_36*dmats0[36][17] + coeff0_37*dmats0[37][17] + coeff0_38*dmats0[38][17] + coeff0_39*dmats0[39][17] + coeff0_40*dmats0[40][17] + coeff0_41*dmats0[41][17] + coeff0_42*dmats0[42][17] + coeff0_43*dmats0[43][17] + coeff0_44*dmats0[44][17] + coeff0_45*dmats0[45][17] + coeff0_46*dmats0[46][17] + coeff0_47*dmats0[47][17] + coeff0_48*dmats0[48][17] + coeff0_49*dmats0[49][17] + coeff0_50*dmats0[50][17] + coeff0_51*dmats0[51][17] + coeff0_52*dmats0[52][17] + coeff0_53*dmats0[53][17] + coeff0_54*dmats0[54][17] + coeff0_55*dmats0[55][17];
          new_coeff0_18 = coeff0_0*dmats0[0][18] + coeff0_1*dmats0[1][18] + coeff0_2*dmats0[2][18] + coeff0_3*dmats0[3][18] + coeff0_4*dmats0[4][18] + coeff0_5*dmats0[5][18] + coeff0_6*dmats0[6][18] + coeff0_7*dmats0[7][18] + coeff0_8*dmats0[8][18] + coeff0_9*dmats0[9][18] + coeff0_10*dmats0[10][18] + coeff0_11*dmats0[11][18] + coeff0_12*dmats0[12][18] + coeff0_13*dmats0[13][18] + coeff0_14*dmats0[14][18] + coeff0_15*dmats0[15][18] + coeff0_16*dmats0[16][18] + coeff0_17*dmats0[17][18] + coeff0_18*dmats0[18][18] + coeff0_19*dmats0[19][18] + coeff0_20*dmats0[20][18] + coeff0_21*dmats0[21][18] + coeff0_22*dmats0[22][18] + coeff0_23*dmats0[23][18] + coeff0_24*dmats0[24][18] + coeff0_25*dmats0[25][18] + coeff0_26*dmats0[26][18] + coeff0_27*dmats0[27][18] + coeff0_28*dmats0[28][18] + coeff0_29*dmats0[29][18] + coeff0_30*dmats0[30][18] + coeff0_31*dmats0[31][18] + coeff0_32*dmats0[32][18] + coeff0_33*dmats0[33][18] + coeff0_34*dmats0[34][18] + coeff0_35*dmats0[35][18] + coeff0_36*dmats0[36][18] + coeff0_37*dmats0[37][18] + coeff0_38*dmats0[38][18] + coeff0_39*dmats0[39][18] + coeff0_40*dmats0[40][18] + coeff0_41*dmats0[41][18] + coeff0_42*dmats0[42][18] + coeff0_43*dmats0[43][18] + coeff0_44*dmats0[44][18] + coeff0_45*dmats0[45][18] + coeff0_46*dmats0[46][18] + coeff0_47*dmats0[47][18] + coeff0_48*dmats0[48][18] + coeff0_49*dmats0[49][18] + coeff0_50*dmats0[50][18] + coeff0_51*dmats0[51][18] + coeff0_52*dmats0[52][18] + coeff0_53*dmats0[53][18] + coeff0_54*dmats0[54][18] + coeff0_55*dmats0[55][18];
          new_coeff0_19 = coeff0_0*dmats0[0][19] + coeff0_1*dmats0[1][19] + coeff0_2*dmats0[2][19] + coeff0_3*dmats0[3][19] + coeff0_4*dmats0[4][19] + coeff0_5*dmats0[5][19] + coeff0_6*dmats0[6][19] + coeff0_7*dmats0[7][19] + coeff0_8*dmats0[8][19] + coeff0_9*dmats0[9][19] + coeff0_10*dmats0[10][19] + coeff0_11*dmats0[11][19] + coeff0_12*dmats0[12][19] + coeff0_13*dmats0[13][19] + coeff0_14*dmats0[14][19] + coeff0_15*dmats0[15][19] + coeff0_16*dmats0[16][19] + coeff0_17*dmats0[17][19] + coeff0_18*dmats0[18][19] + coeff0_19*dmats0[19][19] + coeff0_20*dmats0[20][19] + coeff0_21*dmats0[21][19] + coeff0_22*dmats0[22][19] + coeff0_23*dmats0[23][19] + coeff0_24*dmats0[24][19] + coeff0_25*dmats0[25][19] + coeff0_26*dmats0[26][19] + coeff0_27*dmats0[27][19] + coeff0_28*dmats0[28][19] + coeff0_29*dmats0[29][19] + coeff0_30*dmats0[30][19] + coeff0_31*dmats0[31][19] + coeff0_32*dmats0[32][19] + coeff0_33*dmats0[33][19] + coeff0_34*dmats0[34][19] + coeff0_35*dmats0[35][19] + coeff0_36*dmats0[36][19] + coeff0_37*dmats0[37][19] + coeff0_38*dmats0[38][19] + coeff0_39*dmats0[39][19] + coeff0_40*dmats0[40][19] + coeff0_41*dmats0[41][19] + coeff0_42*dmats0[42][19] + coeff0_43*dmats0[43][19] + coeff0_44*dmats0[44][19] + coeff0_45*dmats0[45][19] + coeff0_46*dmats0[46][19] + coeff0_47*dmats0[47][19] + coeff0_48*dmats0[48][19] + coeff0_49*dmats0[49][19] + coeff0_50*dmats0[50][19] + coeff0_51*dmats0[51][19] + coeff0_52*dmats0[52][19] + coeff0_53*dmats0[53][19] + coeff0_54*dmats0[54][19] + coeff0_55*dmats0[55][19];
          new_coeff0_20 = coeff0_0*dmats0[0][20] + coeff0_1*dmats0[1][20] + coeff0_2*dmats0[2][20] + coeff0_3*dmats0[3][20] + coeff0_4*dmats0[4][20] + coeff0_5*dmats0[5][20] + coeff0_6*dmats0[6][20] + coeff0_7*dmats0[7][20] + coeff0_8*dmats0[8][20] + coeff0_9*dmats0[9][20] + coeff0_10*dmats0[10][20] + coeff0_11*dmats0[11][20] + coeff0_12*dmats0[12][20] + coeff0_13*dmats0[13][20] + coeff0_14*dmats0[14][20] + coeff0_15*dmats0[15][20] + coeff0_16*dmats0[16][20] + coeff0_17*dmats0[17][20] + coeff0_18*dmats0[18][20] + coeff0_19*dmats0[19][20] + coeff0_20*dmats0[20][20] + coeff0_21*dmats0[21][20] + coeff0_22*dmats0[22][20] + coeff0_23*dmats0[23][20] + coeff0_24*dmats0[24][20] + coeff0_25*dmats0[25][20] + coeff0_26*dmats0[26][20] + coeff0_27*dmats0[27][20] + coeff0_28*dmats0[28][20] + coeff0_29*dmats0[29][20] + coeff0_30*dmats0[30][20] + coeff0_31*dmats0[31][20] + coeff0_32*dmats0[32][20] + coeff0_33*dmats0[33][20] + coeff0_34*dmats0[34][20] + coeff0_35*dmats0[35][20] + coeff0_36*dmats0[36][20] + coeff0_37*dmats0[37][20] + coeff0_38*dmats0[38][20] + coeff0_39*dmats0[39][20] + coeff0_40*dmats0[40][20] + coeff0_41*dmats0[41][20] + coeff0_42*dmats0[42][20] + coeff0_43*dmats0[43][20] + coeff0_44*dmats0[44][20] + coeff0_45*dmats0[45][20] + coeff0_46*dmats0[46][20] + coeff0_47*dmats0[47][20] + coeff0_48*dmats0[48][20] + coeff0_49*dmats0[49][20] + coeff0_50*dmats0[50][20] + coeff0_51*dmats0[51][20] + coeff0_52*dmats0[52][20] + coeff0_53*dmats0[53][20] + coeff0_54*dmats0[54][20] + coeff0_55*dmats0[55][20];
          new_coeff0_21 = coeff0_0*dmats0[0][21] + coeff0_1*dmats0[1][21] + coeff0_2*dmats0[2][21] + coeff0_3*dmats0[3][21] + coeff0_4*dmats0[4][21] + coeff0_5*dmats0[5][21] + coeff0_6*dmats0[6][21] + coeff0_7*dmats0[7][21] + coeff0_8*dmats0[8][21] + coeff0_9*dmats0[9][21] + coeff0_10*dmats0[10][21] + coeff0_11*dmats0[11][21] + coeff0_12*dmats0[12][21] + coeff0_13*dmats0[13][21] + coeff0_14*dmats0[14][21] + coeff0_15*dmats0[15][21] + coeff0_16*dmats0[16][21] + coeff0_17*dmats0[17][21] + coeff0_18*dmats0[18][21] + coeff0_19*dmats0[19][21] + coeff0_20*dmats0[20][21] + coeff0_21*dmats0[21][21] + coeff0_22*dmats0[22][21] + coeff0_23*dmats0[23][21] + coeff0_24*dmats0[24][21] + coeff0_25*dmats0[25][21] + coeff0_26*dmats0[26][21] + coeff0_27*dmats0[27][21] + coeff0_28*dmats0[28][21] + coeff0_29*dmats0[29][21] + coeff0_30*dmats0[30][21] + coeff0_31*dmats0[31][21] + coeff0_32*dmats0[32][21] + coeff0_33*dmats0[33][21] + coeff0_34*dmats0[34][21] + coeff0_35*dmats0[35][21] + coeff0_36*dmats0[36][21] + coeff0_37*dmats0[37][21] + coeff0_38*dmats0[38][21] + coeff0_39*dmats0[39][21] + coeff0_40*dmats0[40][21] + coeff0_41*dmats0[41][21] + coeff0_42*dmats0[42][21] + coeff0_43*dmats0[43][21] + coeff0_44*dmats0[44][21] + coeff0_45*dmats0[45][21] + coeff0_46*dmats0[46][21] + coeff0_47*dmats0[47][21] + coeff0_48*dmats0[48][21] + coeff0_49*dmats0[49][21] + coeff0_50*dmats0[50][21] + coeff0_51*dmats0[51][21] + coeff0_52*dmats0[52][21] + coeff0_53*dmats0[53][21] + coeff0_54*dmats0[54][21] + coeff0_55*dmats0[55][21];
          new_coeff0_22 = coeff0_0*dmats0[0][22] + coeff0_1*dmats0[1][22] + coeff0_2*dmats0[2][22] + coeff0_3*dmats0[3][22] + coeff0_4*dmats0[4][22] + coeff0_5*dmats0[5][22] + coeff0_6*dmats0[6][22] + coeff0_7*dmats0[7][22] + coeff0_8*dmats0[8][22] + coeff0_9*dmats0[9][22] + coeff0_10*dmats0[10][22] + coeff0_11*dmats0[11][22] + coeff0_12*dmats0[12][22] + coeff0_13*dmats0[13][22] + coeff0_14*dmats0[14][22] + coeff0_15*dmats0[15][22] + coeff0_16*dmats0[16][22] + coeff0_17*dmats0[17][22] + coeff0_18*dmats0[18][22] + coeff0_19*dmats0[19][22] + coeff0_20*dmats0[20][22] + coeff0_21*dmats0[21][22] + coeff0_22*dmats0[22][22] + coeff0_23*dmats0[23][22] + coeff0_24*dmats0[24][22] + coeff0_25*dmats0[25][22] + coeff0_26*dmats0[26][22] + coeff0_27*dmats0[27][22] + coeff0_28*dmats0[28][22] + coeff0_29*dmats0[29][22] + coeff0_30*dmats0[30][22] + coeff0_31*dmats0[31][22] + coeff0_32*dmats0[32][22] + coeff0_33*dmats0[33][22] + coeff0_34*dmats0[34][22] + coeff0_35*dmats0[35][22] + coeff0_36*dmats0[36][22] + coeff0_37*dmats0[37][22] + coeff0_38*dmats0[38][22] + coeff0_39*dmats0[39][22] + coeff0_40*dmats0[40][22] + coeff0_41*dmats0[41][22] + coeff0_42*dmats0[42][22] + coeff0_43*dmats0[43][22] + coeff0_44*dmats0[44][22] + coeff0_45*dmats0[45][22] + coeff0_46*dmats0[46][22] + coeff0_47*dmats0[47][22] + coeff0_48*dmats0[48][22] + coeff0_49*dmats0[49][22] + coeff0_50*dmats0[50][22] + coeff0_51*dmats0[51][22] + coeff0_52*dmats0[52][22] + coeff0_53*dmats0[53][22] + coeff0_54*dmats0[54][22] + coeff0_55*dmats0[55][22];
          new_coeff0_23 = coeff0_0*dmats0[0][23] + coeff0_1*dmats0[1][23] + coeff0_2*dmats0[2][23] + coeff0_3*dmats0[3][23] + coeff0_4*dmats0[4][23] + coeff0_5*dmats0[5][23] + coeff0_6*dmats0[6][23] + coeff0_7*dmats0[7][23] + coeff0_8*dmats0[8][23] + coeff0_9*dmats0[9][23] + coeff0_10*dmats0[10][23] + coeff0_11*dmats0[11][23] + coeff0_12*dmats0[12][23] + coeff0_13*dmats0[13][23] + coeff0_14*dmats0[14][23] + coeff0_15*dmats0[15][23] + coeff0_16*dmats0[16][23] + coeff0_17*dmats0[17][23] + coeff0_18*dmats0[18][23] + coeff0_19*dmats0[19][23] + coeff0_20*dmats0[20][23] + coeff0_21*dmats0[21][23] + coeff0_22*dmats0[22][23] + coeff0_23*dmats0[23][23] + coeff0_24*dmats0[24][23] + coeff0_25*dmats0[25][23] + coeff0_26*dmats0[26][23] + coeff0_27*dmats0[27][23] + coeff0_28*dmats0[28][23] + coeff0_29*dmats0[29][23] + coeff0_30*dmats0[30][23] + coeff0_31*dmats0[31][23] + coeff0_32*dmats0[32][23] + coeff0_33*dmats0[33][23] + coeff0_34*dmats0[34][23] + coeff0_35*dmats0[35][23] + coeff0_36*dmats0[36][23] + coeff0_37*dmats0[37][23] + coeff0_38*dmats0[38][23] + coeff0_39*dmats0[39][23] + coeff0_40*dmats0[40][23] + coeff0_41*dmats0[41][23] + coeff0_42*dmats0[42][23] + coeff0_43*dmats0[43][23] + coeff0_44*dmats0[44][23] + coeff0_45*dmats0[45][23] + coeff0_46*dmats0[46][23] + coeff0_47*dmats0[47][23] + coeff0_48*dmats0[48][23] + coeff0_49*dmats0[49][23] + coeff0_50*dmats0[50][23] + coeff0_51*dmats0[51][23] + coeff0_52*dmats0[52][23] + coeff0_53*dmats0[53][23] + coeff0_54*dmats0[54][23] + coeff0_55*dmats0[55][23];
          new_coeff0_24 = coeff0_0*dmats0[0][24] + coeff0_1*dmats0[1][24] + coeff0_2*dmats0[2][24] + coeff0_3*dmats0[3][24] + coeff0_4*dmats0[4][24] + coeff0_5*dmats0[5][24] + coeff0_6*dmats0[6][24] + coeff0_7*dmats0[7][24] + coeff0_8*dmats0[8][24] + coeff0_9*dmats0[9][24] + coeff0_10*dmats0[10][24] + coeff0_11*dmats0[11][24] + coeff0_12*dmats0[12][24] + coeff0_13*dmats0[13][24] + coeff0_14*dmats0[14][24] + coeff0_15*dmats0[15][24] + coeff0_16*dmats0[16][24] + coeff0_17*dmats0[17][24] + coeff0_18*dmats0[18][24] + coeff0_19*dmats0[19][24] + coeff0_20*dmats0[20][24] + coeff0_21*dmats0[21][24] + coeff0_22*dmats0[22][24] + coeff0_23*dmats0[23][24] + coeff0_24*dmats0[24][24] + coeff0_25*dmats0[25][24] + coeff0_26*dmats0[26][24] + coeff0_27*dmats0[27][24] + coeff0_28*dmats0[28][24] + coeff0_29*dmats0[29][24] + coeff0_30*dmats0[30][24] + coeff0_31*dmats0[31][24] + coeff0_32*dmats0[32][24] + coeff0_33*dmats0[33][24] + coeff0_34*dmats0[34][24] + coeff0_35*dmats0[35][24] + coeff0_36*dmats0[36][24] + coeff0_37*dmats0[37][24] + coeff0_38*dmats0[38][24] + coeff0_39*dmats0[39][24] + coeff0_40*dmats0[40][24] + coeff0_41*dmats0[41][24] + coeff0_42*dmats0[42][24] + coeff0_43*dmats0[43][24] + coeff0_44*dmats0[44][24] + coeff0_45*dmats0[45][24] + coeff0_46*dmats0[46][24] + coeff0_47*dmats0[47][24] + coeff0_48*dmats0[48][24] + coeff0_49*dmats0[49][24] + coeff0_50*dmats0[50][24] + coeff0_51*dmats0[51][24] + coeff0_52*dmats0[52][24] + coeff0_53*dmats0[53][24] + coeff0_54*dmats0[54][24] + coeff0_55*dmats0[55][24];
          new_coeff0_25 = coeff0_0*dmats0[0][25] + coeff0_1*dmats0[1][25] + coeff0_2*dmats0[2][25] + coeff0_3*dmats0[3][25] + coeff0_4*dmats0[4][25] + coeff0_5*dmats0[5][25] + coeff0_6*dmats0[6][25] + coeff0_7*dmats0[7][25] + coeff0_8*dmats0[8][25] + coeff0_9*dmats0[9][25] + coeff0_10*dmats0[10][25] + coeff0_11*dmats0[11][25] + coeff0_12*dmats0[12][25] + coeff0_13*dmats0[13][25] + coeff0_14*dmats0[14][25] + coeff0_15*dmats0[15][25] + coeff0_16*dmats0[16][25] + coeff0_17*dmats0[17][25] + coeff0_18*dmats0[18][25] + coeff0_19*dmats0[19][25] + coeff0_20*dmats0[20][25] + coeff0_21*dmats0[21][25] + coeff0_22*dmats0[22][25] + coeff0_23*dmats0[23][25] + coeff0_24*dmats0[24][25] + coeff0_25*dmats0[25][25] + coeff0_26*dmats0[26][25] + coeff0_27*dmats0[27][25] + coeff0_28*dmats0[28][25] + coeff0_29*dmats0[29][25] + coeff0_30*dmats0[30][25] + coeff0_31*dmats0[31][25] + coeff0_32*dmats0[32][25] + coeff0_33*dmats0[33][25] + coeff0_34*dmats0[34][25] + coeff0_35*dmats0[35][25] + coeff0_36*dmats0[36][25] + coeff0_37*dmats0[37][25] + coeff0_38*dmats0[38][25] + coeff0_39*dmats0[39][25] + coeff0_40*dmats0[40][25] + coeff0_41*dmats0[41][25] + coeff0_42*dmats0[42][25] + coeff0_43*dmats0[43][25] + coeff0_44*dmats0[44][25] + coeff0_45*dmats0[45][25] + coeff0_46*dmats0[46][25] + coeff0_47*dmats0[47][25] + coeff0_48*dmats0[48][25] + coeff0_49*dmats0[49][25] + coeff0_50*dmats0[50][25] + coeff0_51*dmats0[51][25] + coeff0_52*dmats0[52][25] + coeff0_53*dmats0[53][25] + coeff0_54*dmats0[54][25] + coeff0_55*dmats0[55][25];
          new_coeff0_26 = coeff0_0*dmats0[0][26] + coeff0_1*dmats0[1][26] + coeff0_2*dmats0[2][26] + coeff0_3*dmats0[3][26] + coeff0_4*dmats0[4][26] + coeff0_5*dmats0[5][26] + coeff0_6*dmats0[6][26] + coeff0_7*dmats0[7][26] + coeff0_8*dmats0[8][26] + coeff0_9*dmats0[9][26] + coeff0_10*dmats0[10][26] + coeff0_11*dmats0[11][26] + coeff0_12*dmats0[12][26] + coeff0_13*dmats0[13][26] + coeff0_14*dmats0[14][26] + coeff0_15*dmats0[15][26] + coeff0_16*dmats0[16][26] + coeff0_17*dmats0[17][26] + coeff0_18*dmats0[18][26] + coeff0_19*dmats0[19][26] + coeff0_20*dmats0[20][26] + coeff0_21*dmats0[21][26] + coeff0_22*dmats0[22][26] + coeff0_23*dmats0[23][26] + coeff0_24*dmats0[24][26] + coeff0_25*dmats0[25][26] + coeff0_26*dmats0[26][26] + coeff0_27*dmats0[27][26] + coeff0_28*dmats0[28][26] + coeff0_29*dmats0[29][26] + coeff0_30*dmats0[30][26] + coeff0_31*dmats0[31][26] + coeff0_32*dmats0[32][26] + coeff0_33*dmats0[33][26] + coeff0_34*dmats0[34][26] + coeff0_35*dmats0[35][26] + coeff0_36*dmats0[36][26] + coeff0_37*dmats0[37][26] + coeff0_38*dmats0[38][26] + coeff0_39*dmats0[39][26] + coeff0_40*dmats0[40][26] + coeff0_41*dmats0[41][26] + coeff0_42*dmats0[42][26] + coeff0_43*dmats0[43][26] + coeff0_44*dmats0[44][26] + coeff0_45*dmats0[45][26] + coeff0_46*dmats0[46][26] + coeff0_47*dmats0[47][26] + coeff0_48*dmats0[48][26] + coeff0_49*dmats0[49][26] + coeff0_50*dmats0[50][26] + coeff0_51*dmats0[51][26] + coeff0_52*dmats0[52][26] + coeff0_53*dmats0[53][26] + coeff0_54*dmats0[54][26] + coeff0_55*dmats0[55][26];
          new_coeff0_27 = coeff0_0*dmats0[0][27] + coeff0_1*dmats0[1][27] + coeff0_2*dmats0[2][27] + coeff0_3*dmats0[3][27] + coeff0_4*dmats0[4][27] + coeff0_5*dmats0[5][27] + coeff0_6*dmats0[6][27] + coeff0_7*dmats0[7][27] + coeff0_8*dmats0[8][27] + coeff0_9*dmats0[9][27] + coeff0_10*dmats0[10][27] + coeff0_11*dmats0[11][27] + coeff0_12*dmats0[12][27] + coeff0_13*dmats0[13][27] + coeff0_14*dmats0[14][27] + coeff0_15*dmats0[15][27] + coeff0_16*dmats0[16][27] + coeff0_17*dmats0[17][27] + coeff0_18*dmats0[18][27] + coeff0_19*dmats0[19][27] + coeff0_20*dmats0[20][27] + coeff0_21*dmats0[21][27] + coeff0_22*dmats0[22][27] + coeff0_23*dmats0[23][27] + coeff0_24*dmats0[24][27] + coeff0_25*dmats0[25][27] + coeff0_26*dmats0[26][27] + coeff0_27*dmats0[27][27] + coeff0_28*dmats0[28][27] + coeff0_29*dmats0[29][27] + coeff0_30*dmats0[30][27] + coeff0_31*dmats0[31][27] + coeff0_32*dmats0[32][27] + coeff0_33*dmats0[33][27] + coeff0_34*dmats0[34][27] + coeff0_35*dmats0[35][27] + coeff0_36*dmats0[36][27] + coeff0_37*dmats0[37][27] + coeff0_38*dmats0[38][27] + coeff0_39*dmats0[39][27] + coeff0_40*dmats0[40][27] + coeff0_41*dmats0[41][27] + coeff0_42*dmats0[42][27] + coeff0_43*dmats0[43][27] + coeff0_44*dmats0[44][27] + coeff0_45*dmats0[45][27] + coeff0_46*dmats0[46][27] + coeff0_47*dmats0[47][27] + coeff0_48*dmats0[48][27] + coeff0_49*dmats0[49][27] + coeff0_50*dmats0[50][27] + coeff0_51*dmats0[51][27] + coeff0_52*dmats0[52][27] + coeff0_53*dmats0[53][27] + coeff0_54*dmats0[54][27] + coeff0_55*dmats0[55][27];
          new_coeff0_28 = coeff0_0*dmats0[0][28] + coeff0_1*dmats0[1][28] + coeff0_2*dmats0[2][28] + coeff0_3*dmats0[3][28] + coeff0_4*dmats0[4][28] + coeff0_5*dmats0[5][28] + coeff0_6*dmats0[6][28] + coeff0_7*dmats0[7][28] + coeff0_8*dmats0[8][28] + coeff0_9*dmats0[9][28] + coeff0_10*dmats0[10][28] + coeff0_11*dmats0[11][28] + coeff0_12*dmats0[12][28] + coeff0_13*dmats0[13][28] + coeff0_14*dmats0[14][28] + coeff0_15*dmats0[15][28] + coeff0_16*dmats0[16][28] + coeff0_17*dmats0[17][28] + coeff0_18*dmats0[18][28] + coeff0_19*dmats0[19][28] + coeff0_20*dmats0[20][28] + coeff0_21*dmats0[21][28] + coeff0_22*dmats0[22][28] + coeff0_23*dmats0[23][28] + coeff0_24*dmats0[24][28] + coeff0_25*dmats0[25][28] + coeff0_26*dmats0[26][28] + coeff0_27*dmats0[27][28] + coeff0_28*dmats0[28][28] + coeff0_29*dmats0[29][28] + coeff0_30*dmats0[30][28] + coeff0_31*dmats0[31][28] + coeff0_32*dmats0[32][28] + coeff0_33*dmats0[33][28] + coeff0_34*dmats0[34][28] + coeff0_35*dmats0[35][28] + coeff0_36*dmats0[36][28] + coeff0_37*dmats0[37][28] + coeff0_38*dmats0[38][28] + coeff0_39*dmats0[39][28] + coeff0_40*dmats0[40][28] + coeff0_41*dmats0[41][28] + coeff0_42*dmats0[42][28] + coeff0_43*dmats0[43][28] + coeff0_44*dmats0[44][28] + coeff0_45*dmats0[45][28] + coeff0_46*dmats0[46][28] + coeff0_47*dmats0[47][28] + coeff0_48*dmats0[48][28] + coeff0_49*dmats0[49][28] + coeff0_50*dmats0[50][28] + coeff0_51*dmats0[51][28] + coeff0_52*dmats0[52][28] + coeff0_53*dmats0[53][28] + coeff0_54*dmats0[54][28] + coeff0_55*dmats0[55][28];
          new_coeff0_29 = coeff0_0*dmats0[0][29] + coeff0_1*dmats0[1][29] + coeff0_2*dmats0[2][29] + coeff0_3*dmats0[3][29] + coeff0_4*dmats0[4][29] + coeff0_5*dmats0[5][29] + coeff0_6*dmats0[6][29] + coeff0_7*dmats0[7][29] + coeff0_8*dmats0[8][29] + coeff0_9*dmats0[9][29] + coeff0_10*dmats0[10][29] + coeff0_11*dmats0[11][29] + coeff0_12*dmats0[12][29] + coeff0_13*dmats0[13][29] + coeff0_14*dmats0[14][29] + coeff0_15*dmats0[15][29] + coeff0_16*dmats0[16][29] + coeff0_17*dmats0[17][29] + coeff0_18*dmats0[18][29] + coeff0_19*dmats0[19][29] + coeff0_20*dmats0[20][29] + coeff0_21*dmats0[21][29] + coeff0_22*dmats0[22][29] + coeff0_23*dmats0[23][29] + coeff0_24*dmats0[24][29] + coeff0_25*dmats0[25][29] + coeff0_26*dmats0[26][29] + coeff0_27*dmats0[27][29] + coeff0_28*dmats0[28][29] + coeff0_29*dmats0[29][29] + coeff0_30*dmats0[30][29] + coeff0_31*dmats0[31][29] + coeff0_32*dmats0[32][29] + coeff0_33*dmats0[33][29] + coeff0_34*dmats0[34][29] + coeff0_35*dmats0[35][29] + coeff0_36*dmats0[36][29] + coeff0_37*dmats0[37][29] + coeff0_38*dmats0[38][29] + coeff0_39*dmats0[39][29] + coeff0_40*dmats0[40][29] + coeff0_41*dmats0[41][29] + coeff0_42*dmats0[42][29] + coeff0_43*dmats0[43][29] + coeff0_44*dmats0[44][29] + coeff0_45*dmats0[45][29] + coeff0_46*dmats0[46][29] + coeff0_47*dmats0[47][29] + coeff0_48*dmats0[48][29] + coeff0_49*dmats0[49][29] + coeff0_50*dmats0[50][29] + coeff0_51*dmats0[51][29] + coeff0_52*dmats0[52][29] + coeff0_53*dmats0[53][29] + coeff0_54*dmats0[54][29] + coeff0_55*dmats0[55][29];
          new_coeff0_30 = coeff0_0*dmats0[0][30] + coeff0_1*dmats0[1][30] + coeff0_2*dmats0[2][30] + coeff0_3*dmats0[3][30] + coeff0_4*dmats0[4][30] + coeff0_5*dmats0[5][30] + coeff0_6*dmats0[6][30] + coeff0_7*dmats0[7][30] + coeff0_8*dmats0[8][30] + coeff0_9*dmats0[9][30] + coeff0_10*dmats0[10][30] + coeff0_11*dmats0[11][30] + coeff0_12*dmats0[12][30] + coeff0_13*dmats0[13][30] + coeff0_14*dmats0[14][30] + coeff0_15*dmats0[15][30] + coeff0_16*dmats0[16][30] + coeff0_17*dmats0[17][30] + coeff0_18*dmats0[18][30] + coeff0_19*dmats0[19][30] + coeff0_20*dmats0[20][30] + coeff0_21*dmats0[21][30] + coeff0_22*dmats0[22][30] + coeff0_23*dmats0[23][30] + coeff0_24*dmats0[24][30] + coeff0_25*dmats0[25][30] + coeff0_26*dmats0[26][30] + coeff0_27*dmats0[27][30] + coeff0_28*dmats0[28][30] + coeff0_29*dmats0[29][30] + coeff0_30*dmats0[30][30] + coeff0_31*dmats0[31][30] + coeff0_32*dmats0[32][30] + coeff0_33*dmats0[33][30] + coeff0_34*dmats0[34][30] + coeff0_35*dmats0[35][30] + coeff0_36*dmats0[36][30] + coeff0_37*dmats0[37][30] + coeff0_38*dmats0[38][30] + coeff0_39*dmats0[39][30] + coeff0_40*dmats0[40][30] + coeff0_41*dmats0[41][30] + coeff0_42*dmats0[42][30] + coeff0_43*dmats0[43][30] + coeff0_44*dmats0[44][30] + coeff0_45*dmats0[45][30] + coeff0_46*dmats0[46][30] + coeff0_47*dmats0[47][30] + coeff0_48*dmats0[48][30] + coeff0_49*dmats0[49][30] + coeff0_50*dmats0[50][30] + coeff0_51*dmats0[51][30] + coeff0_52*dmats0[52][30] + coeff0_53*dmats0[53][30] + coeff0_54*dmats0[54][30] + coeff0_55*dmats0[55][30];
          new_coeff0_31 = coeff0_0*dmats0[0][31] + coeff0_1*dmats0[1][31] + coeff0_2*dmats0[2][31] + coeff0_3*dmats0[3][31] + coeff0_4*dmats0[4][31] + coeff0_5*dmats0[5][31] + coeff0_6*dmats0[6][31] + coeff0_7*dmats0[7][31] + coeff0_8*dmats0[8][31] + coeff0_9*dmats0[9][31] + coeff0_10*dmats0[10][31] + coeff0_11*dmats0[11][31] + coeff0_12*dmats0[12][31] + coeff0_13*dmats0[13][31] + coeff0_14*dmats0[14][31] + coeff0_15*dmats0[15][31] + coeff0_16*dmats0[16][31] + coeff0_17*dmats0[17][31] + coeff0_18*dmats0[18][31] + coeff0_19*dmats0[19][31] + coeff0_20*dmats0[20][31] + coeff0_21*dmats0[21][31] + coeff0_22*dmats0[22][31] + coeff0_23*dmats0[23][31] + coeff0_24*dmats0[24][31] + coeff0_25*dmats0[25][31] + coeff0_26*dmats0[26][31] + coeff0_27*dmats0[27][31] + coeff0_28*dmats0[28][31] + coeff0_29*dmats0[29][31] + coeff0_30*dmats0[30][31] + coeff0_31*dmats0[31][31] + coeff0_32*dmats0[32][31] + coeff0_33*dmats0[33][31] + coeff0_34*dmats0[34][31] + coeff0_35*dmats0[35][31] + coeff0_36*dmats0[36][31] + coeff0_37*dmats0[37][31] + coeff0_38*dmats0[38][31] + coeff0_39*dmats0[39][31] + coeff0_40*dmats0[40][31] + coeff0_41*dmats0[41][31] + coeff0_42*dmats0[42][31] + coeff0_43*dmats0[43][31] + coeff0_44*dmats0[44][31] + coeff0_45*dmats0[45][31] + coeff0_46*dmats0[46][31] + coeff0_47*dmats0[47][31] + coeff0_48*dmats0[48][31] + coeff0_49*dmats0[49][31] + coeff0_50*dmats0[50][31] + coeff0_51*dmats0[51][31] + coeff0_52*dmats0[52][31] + coeff0_53*dmats0[53][31] + coeff0_54*dmats0[54][31] + coeff0_55*dmats0[55][31];
          new_coeff0_32 = coeff0_0*dmats0[0][32] + coeff0_1*dmats0[1][32] + coeff0_2*dmats0[2][32] + coeff0_3*dmats0[3][32] + coeff0_4*dmats0[4][32] + coeff0_5*dmats0[5][32] + coeff0_6*dmats0[6][32] + coeff0_7*dmats0[7][32] + coeff0_8*dmats0[8][32] + coeff0_9*dmats0[9][32] + coeff0_10*dmats0[10][32] + coeff0_11*dmats0[11][32] + coeff0_12*dmats0[12][32] + coeff0_13*dmats0[13][32] + coeff0_14*dmats0[14][32] + coeff0_15*dmats0[15][32] + coeff0_16*dmats0[16][32] + coeff0_17*dmats0[17][32] + coeff0_18*dmats0[18][32] + coeff0_19*dmats0[19][32] + coeff0_20*dmats0[20][32] + coeff0_21*dmats0[21][32] + coeff0_22*dmats0[22][32] + coeff0_23*dmats0[23][32] + coeff0_24*dmats0[24][32] + coeff0_25*dmats0[25][32] + coeff0_26*dmats0[26][32] + coeff0_27*dmats0[27][32] + coeff0_28*dmats0[28][32] + coeff0_29*dmats0[29][32] + coeff0_30*dmats0[30][32] + coeff0_31*dmats0[31][32] + coeff0_32*dmats0[32][32] + coeff0_33*dmats0[33][32] + coeff0_34*dmats0[34][32] + coeff0_35*dmats0[35][32] + coeff0_36*dmats0[36][32] + coeff0_37*dmats0[37][32] + coeff0_38*dmats0[38][32] + coeff0_39*dmats0[39][32] + coeff0_40*dmats0[40][32] + coeff0_41*dmats0[41][32] + coeff0_42*dmats0[42][32] + coeff0_43*dmats0[43][32] + coeff0_44*dmats0[44][32] + coeff0_45*dmats0[45][32] + coeff0_46*dmats0[46][32] + coeff0_47*dmats0[47][32] + coeff0_48*dmats0[48][32] + coeff0_49*dmats0[49][32] + coeff0_50*dmats0[50][32] + coeff0_51*dmats0[51][32] + coeff0_52*dmats0[52][32] + coeff0_53*dmats0[53][32] + coeff0_54*dmats0[54][32] + coeff0_55*dmats0[55][32];
          new_coeff0_33 = coeff0_0*dmats0[0][33] + coeff0_1*dmats0[1][33] + coeff0_2*dmats0[2][33] + coeff0_3*dmats0[3][33] + coeff0_4*dmats0[4][33] + coeff0_5*dmats0[5][33] + coeff0_6*dmats0[6][33] + coeff0_7*dmats0[7][33] + coeff0_8*dmats0[8][33] + coeff0_9*dmats0[9][33] + coeff0_10*dmats0[10][33] + coeff0_11*dmats0[11][33] + coeff0_12*dmats0[12][33] + coeff0_13*dmats0[13][33] + coeff0_14*dmats0[14][33] + coeff0_15*dmats0[15][33] + coeff0_16*dmats0[16][33] + coeff0_17*dmats0[17][33] + coeff0_18*dmats0[18][33] + coeff0_19*dmats0[19][33] + coeff0_20*dmats0[20][33] + coeff0_21*dmats0[21][33] + coeff0_22*dmats0[22][33] + coeff0_23*dmats0[23][33] + coeff0_24*dmats0[24][33] + coeff0_25*dmats0[25][33] + coeff0_26*dmats0[26][33] + coeff0_27*dmats0[27][33] + coeff0_28*dmats0[28][33] + coeff0_29*dmats0[29][33] + coeff0_30*dmats0[30][33] + coeff0_31*dmats0[31][33] + coeff0_32*dmats0[32][33] + coeff0_33*dmats0[33][33] + coeff0_34*dmats0[34][33] + coeff0_35*dmats0[35][33] + coeff0_36*dmats0[36][33] + coeff0_37*dmats0[37][33] + coeff0_38*dmats0[38][33] + coeff0_39*dmats0[39][33] + coeff0_40*dmats0[40][33] + coeff0_41*dmats0[41][33] + coeff0_42*dmats0[42][33] + coeff0_43*dmats0[43][33] + coeff0_44*dmats0[44][33] + coeff0_45*dmats0[45][33] + coeff0_46*dmats0[46][33] + coeff0_47*dmats0[47][33] + coeff0_48*dmats0[48][33] + coeff0_49*dmats0[49][33] + coeff0_50*dmats0[50][33] + coeff0_51*dmats0[51][33] + coeff0_52*dmats0[52][33] + coeff0_53*dmats0[53][33] + coeff0_54*dmats0[54][33] + coeff0_55*dmats0[55][33];
          new_coeff0_34 = coeff0_0*dmats0[0][34] + coeff0_1*dmats0[1][34] + coeff0_2*dmats0[2][34] + coeff0_3*dmats0[3][34] + coeff0_4*dmats0[4][34] + coeff0_5*dmats0[5][34] + coeff0_6*dmats0[6][34] + coeff0_7*dmats0[7][34] + coeff0_8*dmats0[8][34] + coeff0_9*dmats0[9][34] + coeff0_10*dmats0[10][34] + coeff0_11*dmats0[11][34] + coeff0_12*dmats0[12][34] + coeff0_13*dmats0[13][34] + coeff0_14*dmats0[14][34] + coeff0_15*dmats0[15][34] + coeff0_16*dmats0[16][34] + coeff0_17*dmats0[17][34] + coeff0_18*dmats0[18][34] + coeff0_19*dmats0[19][34] + coeff0_20*dmats0[20][34] + coeff0_21*dmats0[21][34] + coeff0_22*dmats0[22][34] + coeff0_23*dmats0[23][34] + coeff0_24*dmats0[24][34] + coeff0_25*dmats0[25][34] + coeff0_26*dmats0[26][34] + coeff0_27*dmats0[27][34] + coeff0_28*dmats0[28][34] + coeff0_29*dmats0[29][34] + coeff0_30*dmats0[30][34] + coeff0_31*dmats0[31][34] + coeff0_32*dmats0[32][34] + coeff0_33*dmats0[33][34] + coeff0_34*dmats0[34][34] + coeff0_35*dmats0[35][34] + coeff0_36*dmats0[36][34] + coeff0_37*dmats0[37][34] + coeff0_38*dmats0[38][34] + coeff0_39*dmats0[39][34] + coeff0_40*dmats0[40][34] + coeff0_41*dmats0[41][34] + coeff0_42*dmats0[42][34] + coeff0_43*dmats0[43][34] + coeff0_44*dmats0[44][34] + coeff0_45*dmats0[45][34] + coeff0_46*dmats0[46][34] + coeff0_47*dmats0[47][34] + coeff0_48*dmats0[48][34] + coeff0_49*dmats0[49][34] + coeff0_50*dmats0[50][34] + coeff0_51*dmats0[51][34] + coeff0_52*dmats0[52][34] + coeff0_53*dmats0[53][34] + coeff0_54*dmats0[54][34] + coeff0_55*dmats0[55][34];
          new_coeff0_35 = coeff0_0*dmats0[0][35] + coeff0_1*dmats0[1][35] + coeff0_2*dmats0[2][35] + coeff0_3*dmats0[3][35] + coeff0_4*dmats0[4][35] + coeff0_5*dmats0[5][35] + coeff0_6*dmats0[6][35] + coeff0_7*dmats0[7][35] + coeff0_8*dmats0[8][35] + coeff0_9*dmats0[9][35] + coeff0_10*dmats0[10][35] + coeff0_11*dmats0[11][35] + coeff0_12*dmats0[12][35] + coeff0_13*dmats0[13][35] + coeff0_14*dmats0[14][35] + coeff0_15*dmats0[15][35] + coeff0_16*dmats0[16][35] + coeff0_17*dmats0[17][35] + coeff0_18*dmats0[18][35] + coeff0_19*dmats0[19][35] + coeff0_20*dmats0[20][35] + coeff0_21*dmats0[21][35] + coeff0_22*dmats0[22][35] + coeff0_23*dmats0[23][35] + coeff0_24*dmats0[24][35] + coeff0_25*dmats0[25][35] + coeff0_26*dmats0[26][35] + coeff0_27*dmats0[27][35] + coeff0_28*dmats0[28][35] + coeff0_29*dmats0[29][35] + coeff0_30*dmats0[30][35] + coeff0_31*dmats0[31][35] + coeff0_32*dmats0[32][35] + coeff0_33*dmats0[33][35] + coeff0_34*dmats0[34][35] + coeff0_35*dmats0[35][35] + coeff0_36*dmats0[36][35] + coeff0_37*dmats0[37][35] + coeff0_38*dmats0[38][35] + coeff0_39*dmats0[39][35] + coeff0_40*dmats0[40][35] + coeff0_41*dmats0[41][35] + coeff0_42*dmats0[42][35] + coeff0_43*dmats0[43][35] + coeff0_44*dmats0[44][35] + coeff0_45*dmats0[45][35] + coeff0_46*dmats0[46][35] + coeff0_47*dmats0[47][35] + coeff0_48*dmats0[48][35] + coeff0_49*dmats0[49][35] + coeff0_50*dmats0[50][35] + coeff0_51*dmats0[51][35] + coeff0_52*dmats0[52][35] + coeff0_53*dmats0[53][35] + coeff0_54*dmats0[54][35] + coeff0_55*dmats0[55][35];
          new_coeff0_36 = coeff0_0*dmats0[0][36] + coeff0_1*dmats0[1][36] + coeff0_2*dmats0[2][36] + coeff0_3*dmats0[3][36] + coeff0_4*dmats0[4][36] + coeff0_5*dmats0[5][36] + coeff0_6*dmats0[6][36] + coeff0_7*dmats0[7][36] + coeff0_8*dmats0[8][36] + coeff0_9*dmats0[9][36] + coeff0_10*dmats0[10][36] + coeff0_11*dmats0[11][36] + coeff0_12*dmats0[12][36] + coeff0_13*dmats0[13][36] + coeff0_14*dmats0[14][36] + coeff0_15*dmats0[15][36] + coeff0_16*dmats0[16][36] + coeff0_17*dmats0[17][36] + coeff0_18*dmats0[18][36] + coeff0_19*dmats0[19][36] + coeff0_20*dmats0[20][36] + coeff0_21*dmats0[21][36] + coeff0_22*dmats0[22][36] + coeff0_23*dmats0[23][36] + coeff0_24*dmats0[24][36] + coeff0_25*dmats0[25][36] + coeff0_26*dmats0[26][36] + coeff0_27*dmats0[27][36] + coeff0_28*dmats0[28][36] + coeff0_29*dmats0[29][36] + coeff0_30*dmats0[30][36] + coeff0_31*dmats0[31][36] + coeff0_32*dmats0[32][36] + coeff0_33*dmats0[33][36] + coeff0_34*dmats0[34][36] + coeff0_35*dmats0[35][36] + coeff0_36*dmats0[36][36] + coeff0_37*dmats0[37][36] + coeff0_38*dmats0[38][36] + coeff0_39*dmats0[39][36] + coeff0_40*dmats0[40][36] + coeff0_41*dmats0[41][36] + coeff0_42*dmats0[42][36] + coeff0_43*dmats0[43][36] + coeff0_44*dmats0[44][36] + coeff0_45*dmats0[45][36] + coeff0_46*dmats0[46][36] + coeff0_47*dmats0[47][36] + coeff0_48*dmats0[48][36] + coeff0_49*dmats0[49][36] + coeff0_50*dmats0[50][36] + coeff0_51*dmats0[51][36] + coeff0_52*dmats0[52][36] + coeff0_53*dmats0[53][36] + coeff0_54*dmats0[54][36] + coeff0_55*dmats0[55][36];
          new_coeff0_37 = coeff0_0*dmats0[0][37] + coeff0_1*dmats0[1][37] + coeff0_2*dmats0[2][37] + coeff0_3*dmats0[3][37] + coeff0_4*dmats0[4][37] + coeff0_5*dmats0[5][37] + coeff0_6*dmats0[6][37] + coeff0_7*dmats0[7][37] + coeff0_8*dmats0[8][37] + coeff0_9*dmats0[9][37] + coeff0_10*dmats0[10][37] + coeff0_11*dmats0[11][37] + coeff0_12*dmats0[12][37] + coeff0_13*dmats0[13][37] + coeff0_14*dmats0[14][37] + coeff0_15*dmats0[15][37] + coeff0_16*dmats0[16][37] + coeff0_17*dmats0[17][37] + coeff0_18*dmats0[18][37] + coeff0_19*dmats0[19][37] + coeff0_20*dmats0[20][37] + coeff0_21*dmats0[21][37] + coeff0_22*dmats0[22][37] + coeff0_23*dmats0[23][37] + coeff0_24*dmats0[24][37] + coeff0_25*dmats0[25][37] + coeff0_26*dmats0[26][37] + coeff0_27*dmats0[27][37] + coeff0_28*dmats0[28][37] + coeff0_29*dmats0[29][37] + coeff0_30*dmats0[30][37] + coeff0_31*dmats0[31][37] + coeff0_32*dmats0[32][37] + coeff0_33*dmats0[33][37] + coeff0_34*dmats0[34][37] + coeff0_35*dmats0[35][37] + coeff0_36*dmats0[36][37] + coeff0_37*dmats0[37][37] + coeff0_38*dmats0[38][37] + coeff0_39*dmats0[39][37] + coeff0_40*dmats0[40][37] + coeff0_41*dmats0[41][37] + coeff0_42*dmats0[42][37] + coeff0_43*dmats0[43][37] + coeff0_44*dmats0[44][37] + coeff0_45*dmats0[45][37] + coeff0_46*dmats0[46][37] + coeff0_47*dmats0[47][37] + coeff0_48*dmats0[48][37] + coeff0_49*dmats0[49][37] + coeff0_50*dmats0[50][37] + coeff0_51*dmats0[51][37] + coeff0_52*dmats0[52][37] + coeff0_53*dmats0[53][37] + coeff0_54*dmats0[54][37] + coeff0_55*dmats0[55][37];
          new_coeff0_38 = coeff0_0*dmats0[0][38] + coeff0_1*dmats0[1][38] + coeff0_2*dmats0[2][38] + coeff0_3*dmats0[3][38] + coeff0_4*dmats0[4][38] + coeff0_5*dmats0[5][38] + coeff0_6*dmats0[6][38] + coeff0_7*dmats0[7][38] + coeff0_8*dmats0[8][38] + coeff0_9*dmats0[9][38] + coeff0_10*dmats0[10][38] + coeff0_11*dmats0[11][38] + coeff0_12*dmats0[12][38] + coeff0_13*dmats0[13][38] + coeff0_14*dmats0[14][38] + coeff0_15*dmats0[15][38] + coeff0_16*dmats0[16][38] + coeff0_17*dmats0[17][38] + coeff0_18*dmats0[18][38] + coeff0_19*dmats0[19][38] + coeff0_20*dmats0[20][38] + coeff0_21*dmats0[21][38] + coeff0_22*dmats0[22][38] + coeff0_23*dmats0[23][38] + coeff0_24*dmats0[24][38] + coeff0_25*dmats0[25][38] + coeff0_26*dmats0[26][38] + coeff0_27*dmats0[27][38] + coeff0_28*dmats0[28][38] + coeff0_29*dmats0[29][38] + coeff0_30*dmats0[30][38] + coeff0_31*dmats0[31][38] + coeff0_32*dmats0[32][38] + coeff0_33*dmats0[33][38] + coeff0_34*dmats0[34][38] + coeff0_35*dmats0[35][38] + coeff0_36*dmats0[36][38] + coeff0_37*dmats0[37][38] + coeff0_38*dmats0[38][38] + coeff0_39*dmats0[39][38] + coeff0_40*dmats0[40][38] + coeff0_41*dmats0[41][38] + coeff0_42*dmats0[42][38] + coeff0_43*dmats0[43][38] + coeff0_44*dmats0[44][38] + coeff0_45*dmats0[45][38] + coeff0_46*dmats0[46][38] + coeff0_47*dmats0[47][38] + coeff0_48*dmats0[48][38] + coeff0_49*dmats0[49][38] + coeff0_50*dmats0[50][38] + coeff0_51*dmats0[51][38] + coeff0_52*dmats0[52][38] + coeff0_53*dmats0[53][38] + coeff0_54*dmats0[54][38] + coeff0_55*dmats0[55][38];
          new_coeff0_39 = coeff0_0*dmats0[0][39] + coeff0_1*dmats0[1][39] + coeff0_2*dmats0[2][39] + coeff0_3*dmats0[3][39] + coeff0_4*dmats0[4][39] + coeff0_5*dmats0[5][39] + coeff0_6*dmats0[6][39] + coeff0_7*dmats0[7][39] + coeff0_8*dmats0[8][39] + coeff0_9*dmats0[9][39] + coeff0_10*dmats0[10][39] + coeff0_11*dmats0[11][39] + coeff0_12*dmats0[12][39] + coeff0_13*dmats0[13][39] + coeff0_14*dmats0[14][39] + coeff0_15*dmats0[15][39] + coeff0_16*dmats0[16][39] + coeff0_17*dmats0[17][39] + coeff0_18*dmats0[18][39] + coeff0_19*dmats0[19][39] + coeff0_20*dmats0[20][39] + coeff0_21*dmats0[21][39] + coeff0_22*dmats0[22][39] + coeff0_23*dmats0[23][39] + coeff0_24*dmats0[24][39] + coeff0_25*dmats0[25][39] + coeff0_26*dmats0[26][39] + coeff0_27*dmats0[27][39] + coeff0_28*dmats0[28][39] + coeff0_29*dmats0[29][39] + coeff0_30*dmats0[30][39] + coeff0_31*dmats0[31][39] + coeff0_32*dmats0[32][39] + coeff0_33*dmats0[33][39] + coeff0_34*dmats0[34][39] + coeff0_35*dmats0[35][39] + coeff0_36*dmats0[36][39] + coeff0_37*dmats0[37][39] + coeff0_38*dmats0[38][39] + coeff0_39*dmats0[39][39] + coeff0_40*dmats0[40][39] + coeff0_41*dmats0[41][39] + coeff0_42*dmats0[42][39] + coeff0_43*dmats0[43][39] + coeff0_44*dmats0[44][39] + coeff0_45*dmats0[45][39] + coeff0_46*dmats0[46][39] + coeff0_47*dmats0[47][39] + coeff0_48*dmats0[48][39] + coeff0_49*dmats0[49][39] + coeff0_50*dmats0[50][39] + coeff0_51*dmats0[51][39] + coeff0_52*dmats0[52][39] + coeff0_53*dmats0[53][39] + coeff0_54*dmats0[54][39] + coeff0_55*dmats0[55][39];
          new_coeff0_40 = coeff0_0*dmats0[0][40] + coeff0_1*dmats0[1][40] + coeff0_2*dmats0[2][40] + coeff0_3*dmats0[3][40] + coeff0_4*dmats0[4][40] + coeff0_5*dmats0[5][40] + coeff0_6*dmats0[6][40] + coeff0_7*dmats0[7][40] + coeff0_8*dmats0[8][40] + coeff0_9*dmats0[9][40] + coeff0_10*dmats0[10][40] + coeff0_11*dmats0[11][40] + coeff0_12*dmats0[12][40] + coeff0_13*dmats0[13][40] + coeff0_14*dmats0[14][40] + coeff0_15*dmats0[15][40] + coeff0_16*dmats0[16][40] + coeff0_17*dmats0[17][40] + coeff0_18*dmats0[18][40] + coeff0_19*dmats0[19][40] + coeff0_20*dmats0[20][40] + coeff0_21*dmats0[21][40] + coeff0_22*dmats0[22][40] + coeff0_23*dmats0[23][40] + coeff0_24*dmats0[24][40] + coeff0_25*dmats0[25][40] + coeff0_26*dmats0[26][40] + coeff0_27*dmats0[27][40] + coeff0_28*dmats0[28][40] + coeff0_29*dmats0[29][40] + coeff0_30*dmats0[30][40] + coeff0_31*dmats0[31][40] + coeff0_32*dmats0[32][40] + coeff0_33*dmats0[33][40] + coeff0_34*dmats0[34][40] + coeff0_35*dmats0[35][40] + coeff0_36*dmats0[36][40] + coeff0_37*dmats0[37][40] + coeff0_38*dmats0[38][40] + coeff0_39*dmats0[39][40] + coeff0_40*dmats0[40][40] + coeff0_41*dmats0[41][40] + coeff0_42*dmats0[42][40] + coeff0_43*dmats0[43][40] + coeff0_44*dmats0[44][40] + coeff0_45*dmats0[45][40] + coeff0_46*dmats0[46][40] + coeff0_47*dmats0[47][40] + coeff0_48*dmats0[48][40] + coeff0_49*dmats0[49][40] + coeff0_50*dmats0[50][40] + coeff0_51*dmats0[51][40] + coeff0_52*dmats0[52][40] + coeff0_53*dmats0[53][40] + coeff0_54*dmats0[54][40] + coeff0_55*dmats0[55][40];
          new_coeff0_41 = coeff0_0*dmats0[0][41] + coeff0_1*dmats0[1][41] + coeff0_2*dmats0[2][41] + coeff0_3*dmats0[3][41] + coeff0_4*dmats0[4][41] + coeff0_5*dmats0[5][41] + coeff0_6*dmats0[6][41] + coeff0_7*dmats0[7][41] + coeff0_8*dmats0[8][41] + coeff0_9*dmats0[9][41] + coeff0_10*dmats0[10][41] + coeff0_11*dmats0[11][41] + coeff0_12*dmats0[12][41] + coeff0_13*dmats0[13][41] + coeff0_14*dmats0[14][41] + coeff0_15*dmats0[15][41] + coeff0_16*dmats0[16][41] + coeff0_17*dmats0[17][41] + coeff0_18*dmats0[18][41] + coeff0_19*dmats0[19][41] + coeff0_20*dmats0[20][41] + coeff0_21*dmats0[21][41] + coeff0_22*dmats0[22][41] + coeff0_23*dmats0[23][41] + coeff0_24*dmats0[24][41] + coeff0_25*dmats0[25][41] + coeff0_26*dmats0[26][41] + coeff0_27*dmats0[27][41] + coeff0_28*dmats0[28][41] + coeff0_29*dmats0[29][41] + coeff0_30*dmats0[30][41] + coeff0_31*dmats0[31][41] + coeff0_32*dmats0[32][41] + coeff0_33*dmats0[33][41] + coeff0_34*dmats0[34][41] + coeff0_35*dmats0[35][41] + coeff0_36*dmats0[36][41] + coeff0_37*dmats0[37][41] + coeff0_38*dmats0[38][41] + coeff0_39*dmats0[39][41] + coeff0_40*dmats0[40][41] + coeff0_41*dmats0[41][41] + coeff0_42*dmats0[42][41] + coeff0_43*dmats0[43][41] + coeff0_44*dmats0[44][41] + coeff0_45*dmats0[45][41] + coeff0_46*dmats0[46][41] + coeff0_47*dmats0[47][41] + coeff0_48*dmats0[48][41] + coeff0_49*dmats0[49][41] + coeff0_50*dmats0[50][41] + coeff0_51*dmats0[51][41] + coeff0_52*dmats0[52][41] + coeff0_53*dmats0[53][41] + coeff0_54*dmats0[54][41] + coeff0_55*dmats0[55][41];
          new_coeff0_42 = coeff0_0*dmats0[0][42] + coeff0_1*dmats0[1][42] + coeff0_2*dmats0[2][42] + coeff0_3*dmats0[3][42] + coeff0_4*dmats0[4][42] + coeff0_5*dmats0[5][42] + coeff0_6*dmats0[6][42] + coeff0_7*dmats0[7][42] + coeff0_8*dmats0[8][42] + coeff0_9*dmats0[9][42] + coeff0_10*dmats0[10][42] + coeff0_11*dmats0[11][42] + coeff0_12*dmats0[12][42] + coeff0_13*dmats0[13][42] + coeff0_14*dmats0[14][42] + coeff0_15*dmats0[15][42] + coeff0_16*dmats0[16][42] + coeff0_17*dmats0[17][42] + coeff0_18*dmats0[18][42] + coeff0_19*dmats0[19][42] + coeff0_20*dmats0[20][42] + coeff0_21*dmats0[21][42] + coeff0_22*dmats0[22][42] + coeff0_23*dmats0[23][42] + coeff0_24*dmats0[24][42] + coeff0_25*dmats0[25][42] + coeff0_26*dmats0[26][42] + coeff0_27*dmats0[27][42] + coeff0_28*dmats0[28][42] + coeff0_29*dmats0[29][42] + coeff0_30*dmats0[30][42] + coeff0_31*dmats0[31][42] + coeff0_32*dmats0[32][42] + coeff0_33*dmats0[33][42] + coeff0_34*dmats0[34][42] + coeff0_35*dmats0[35][42] + coeff0_36*dmats0[36][42] + coeff0_37*dmats0[37][42] + coeff0_38*dmats0[38][42] + coeff0_39*dmats0[39][42] + coeff0_40*dmats0[40][42] + coeff0_41*dmats0[41][42] + coeff0_42*dmats0[42][42] + coeff0_43*dmats0[43][42] + coeff0_44*dmats0[44][42] + coeff0_45*dmats0[45][42] + coeff0_46*dmats0[46][42] + coeff0_47*dmats0[47][42] + coeff0_48*dmats0[48][42] + coeff0_49*dmats0[49][42] + coeff0_50*dmats0[50][42] + coeff0_51*dmats0[51][42] + coeff0_52*dmats0[52][42] + coeff0_53*dmats0[53][42] + coeff0_54*dmats0[54][42] + coeff0_55*dmats0[55][42];
          new_coeff0_43 = coeff0_0*dmats0[0][43] + coeff0_1*dmats0[1][43] + coeff0_2*dmats0[2][43] + coeff0_3*dmats0[3][43] + coeff0_4*dmats0[4][43] + coeff0_5*dmats0[5][43] + coeff0_6*dmats0[6][43] + coeff0_7*dmats0[7][43] + coeff0_8*dmats0[8][43] + coeff0_9*dmats0[9][43] + coeff0_10*dmats0[10][43] + coeff0_11*dmats0[11][43] + coeff0_12*dmats0[12][43] + coeff0_13*dmats0[13][43] + coeff0_14*dmats0[14][43] + coeff0_15*dmats0[15][43] + coeff0_16*dmats0[16][43] + coeff0_17*dmats0[17][43] + coeff0_18*dmats0[18][43] + coeff0_19*dmats0[19][43] + coeff0_20*dmats0[20][43] + coeff0_21*dmats0[21][43] + coeff0_22*dmats0[22][43] + coeff0_23*dmats0[23][43] + coeff0_24*dmats0[24][43] + coeff0_25*dmats0[25][43] + coeff0_26*dmats0[26][43] + coeff0_27*dmats0[27][43] + coeff0_28*dmats0[28][43] + coeff0_29*dmats0[29][43] + coeff0_30*dmats0[30][43] + coeff0_31*dmats0[31][43] + coeff0_32*dmats0[32][43] + coeff0_33*dmats0[33][43] + coeff0_34*dmats0[34][43] + coeff0_35*dmats0[35][43] + coeff0_36*dmats0[36][43] + coeff0_37*dmats0[37][43] + coeff0_38*dmats0[38][43] + coeff0_39*dmats0[39][43] + coeff0_40*dmats0[40][43] + coeff0_41*dmats0[41][43] + coeff0_42*dmats0[42][43] + coeff0_43*dmats0[43][43] + coeff0_44*dmats0[44][43] + coeff0_45*dmats0[45][43] + coeff0_46*dmats0[46][43] + coeff0_47*dmats0[47][43] + coeff0_48*dmats0[48][43] + coeff0_49*dmats0[49][43] + coeff0_50*dmats0[50][43] + coeff0_51*dmats0[51][43] + coeff0_52*dmats0[52][43] + coeff0_53*dmats0[53][43] + coeff0_54*dmats0[54][43] + coeff0_55*dmats0[55][43];
          new_coeff0_44 = coeff0_0*dmats0[0][44] + coeff0_1*dmats0[1][44] + coeff0_2*dmats0[2][44] + coeff0_3*dmats0[3][44] + coeff0_4*dmats0[4][44] + coeff0_5*dmats0[5][44] + coeff0_6*dmats0[6][44] + coeff0_7*dmats0[7][44] + coeff0_8*dmats0[8][44] + coeff0_9*dmats0[9][44] + coeff0_10*dmats0[10][44] + coeff0_11*dmats0[11][44] + coeff0_12*dmats0[12][44] + coeff0_13*dmats0[13][44] + coeff0_14*dmats0[14][44] + coeff0_15*dmats0[15][44] + coeff0_16*dmats0[16][44] + coeff0_17*dmats0[17][44] + coeff0_18*dmats0[18][44] + coeff0_19*dmats0[19][44] + coeff0_20*dmats0[20][44] + coeff0_21*dmats0[21][44] + coeff0_22*dmats0[22][44] + coeff0_23*dmats0[23][44] + coeff0_24*dmats0[24][44] + coeff0_25*dmats0[25][44] + coeff0_26*dmats0[26][44] + coeff0_27*dmats0[27][44] + coeff0_28*dmats0[28][44] + coeff0_29*dmats0[29][44] + coeff0_30*dmats0[30][44] + coeff0_31*dmats0[31][44] + coeff0_32*dmats0[32][44] + coeff0_33*dmats0[33][44] + coeff0_34*dmats0[34][44] + coeff0_35*dmats0[35][44] + coeff0_36*dmats0[36][44] + coeff0_37*dmats0[37][44] + coeff0_38*dmats0[38][44] + coeff0_39*dmats0[39][44] + coeff0_40*dmats0[40][44] + coeff0_41*dmats0[41][44] + coeff0_42*dmats0[42][44] + coeff0_43*dmats0[43][44] + coeff0_44*dmats0[44][44] + coeff0_45*dmats0[45][44] + coeff0_46*dmats0[46][44] + coeff0_47*dmats0[47][44] + coeff0_48*dmats0[48][44] + coeff0_49*dmats0[49][44] + coeff0_50*dmats0[50][44] + coeff0_51*dmats0[51][44] + coeff0_52*dmats0[52][44] + coeff0_53*dmats0[53][44] + coeff0_54*dmats0[54][44] + coeff0_55*dmats0[55][44];
          new_coeff0_45 = coeff0_0*dmats0[0][45] + coeff0_1*dmats0[1][45] + coeff0_2*dmats0[2][45] + coeff0_3*dmats0[3][45] + coeff0_4*dmats0[4][45] + coeff0_5*dmats0[5][45] + coeff0_6*dmats0[6][45] + coeff0_7*dmats0[7][45] + coeff0_8*dmats0[8][45] + coeff0_9*dmats0[9][45] + coeff0_10*dmats0[10][45] + coeff0_11*dmats0[11][45] + coeff0_12*dmats0[12][45] + coeff0_13*dmats0[13][45] + coeff0_14*dmats0[14][45] + coeff0_15*dmats0[15][45] + coeff0_16*dmats0[16][45] + coeff0_17*dmats0[17][45] + coeff0_18*dmats0[18][45] + coeff0_19*dmats0[19][45] + coeff0_20*dmats0[20][45] + coeff0_21*dmats0[21][45] + coeff0_22*dmats0[22][45] + coeff0_23*dmats0[23][45] + coeff0_24*dmats0[24][45] + coeff0_25*dmats0[25][45] + coeff0_26*dmats0[26][45] + coeff0_27*dmats0[27][45] + coeff0_28*dmats0[28][45] + coeff0_29*dmats0[29][45] + coeff0_30*dmats0[30][45] + coeff0_31*dmats0[31][45] + coeff0_32*dmats0[32][45] + coeff0_33*dmats0[33][45] + coeff0_34*dmats0[34][45] + coeff0_35*dmats0[35][45] + coeff0_36*dmats0[36][45] + coeff0_37*dmats0[37][45] + coeff0_38*dmats0[38][45] + coeff0_39*dmats0[39][45] + coeff0_40*dmats0[40][45] + coeff0_41*dmats0[41][45] + coeff0_42*dmats0[42][45] + coeff0_43*dmats0[43][45] + coeff0_44*dmats0[44][45] + coeff0_45*dmats0[45][45] + coeff0_46*dmats0[46][45] + coeff0_47*dmats0[47][45] + coeff0_48*dmats0[48][45] + coeff0_49*dmats0[49][45] + coeff0_50*dmats0[50][45] + coeff0_51*dmats0[51][45] + coeff0_52*dmats0[52][45] + coeff0_53*dmats0[53][45] + coeff0_54*dmats0[54][45] + coeff0_55*dmats0[55][45];
          new_coeff0_46 = coeff0_0*dmats0[0][46] + coeff0_1*dmats0[1][46] + coeff0_2*dmats0[2][46] + coeff0_3*dmats0[3][46] + coeff0_4*dmats0[4][46] + coeff0_5*dmats0[5][46] + coeff0_6*dmats0[6][46] + coeff0_7*dmats0[7][46] + coeff0_8*dmats0[8][46] + coeff0_9*dmats0[9][46] + coeff0_10*dmats0[10][46] + coeff0_11*dmats0[11][46] + coeff0_12*dmats0[12][46] + coeff0_13*dmats0[13][46] + coeff0_14*dmats0[14][46] + coeff0_15*dmats0[15][46] + coeff0_16*dmats0[16][46] + coeff0_17*dmats0[17][46] + coeff0_18*dmats0[18][46] + coeff0_19*dmats0[19][46] + coeff0_20*dmats0[20][46] + coeff0_21*dmats0[21][46] + coeff0_22*dmats0[22][46] + coeff0_23*dmats0[23][46] + coeff0_24*dmats0[24][46] + coeff0_25*dmats0[25][46] + coeff0_26*dmats0[26][46] + coeff0_27*dmats0[27][46] + coeff0_28*dmats0[28][46] + coeff0_29*dmats0[29][46] + coeff0_30*dmats0[30][46] + coeff0_31*dmats0[31][46] + coeff0_32*dmats0[32][46] + coeff0_33*dmats0[33][46] + coeff0_34*dmats0[34][46] + coeff0_35*dmats0[35][46] + coeff0_36*dmats0[36][46] + coeff0_37*dmats0[37][46] + coeff0_38*dmats0[38][46] + coeff0_39*dmats0[39][46] + coeff0_40*dmats0[40][46] + coeff0_41*dmats0[41][46] + coeff0_42*dmats0[42][46] + coeff0_43*dmats0[43][46] + coeff0_44*dmats0[44][46] + coeff0_45*dmats0[45][46] + coeff0_46*dmats0[46][46] + coeff0_47*dmats0[47][46] + coeff0_48*dmats0[48][46] + coeff0_49*dmats0[49][46] + coeff0_50*dmats0[50][46] + coeff0_51*dmats0[51][46] + coeff0_52*dmats0[52][46] + coeff0_53*dmats0[53][46] + coeff0_54*dmats0[54][46] + coeff0_55*dmats0[55][46];
          new_coeff0_47 = coeff0_0*dmats0[0][47] + coeff0_1*dmats0[1][47] + coeff0_2*dmats0[2][47] + coeff0_3*dmats0[3][47] + coeff0_4*dmats0[4][47] + coeff0_5*dmats0[5][47] + coeff0_6*dmats0[6][47] + coeff0_7*dmats0[7][47] + coeff0_8*dmats0[8][47] + coeff0_9*dmats0[9][47] + coeff0_10*dmats0[10][47] + coeff0_11*dmats0[11][47] + coeff0_12*dmats0[12][47] + coeff0_13*dmats0[13][47] + coeff0_14*dmats0[14][47] + coeff0_15*dmats0[15][47] + coeff0_16*dmats0[16][47] + coeff0_17*dmats0[17][47] + coeff0_18*dmats0[18][47] + coeff0_19*dmats0[19][47] + coeff0_20*dmats0[20][47] + coeff0_21*dmats0[21][47] + coeff0_22*dmats0[22][47] + coeff0_23*dmats0[23][47] + coeff0_24*dmats0[24][47] + coeff0_25*dmats0[25][47] + coeff0_26*dmats0[26][47] + coeff0_27*dmats0[27][47] + coeff0_28*dmats0[28][47] + coeff0_29*dmats0[29][47] + coeff0_30*dmats0[30][47] + coeff0_31*dmats0[31][47] + coeff0_32*dmats0[32][47] + coeff0_33*dmats0[33][47] + coeff0_34*dmats0[34][47] + coeff0_35*dmats0[35][47] + coeff0_36*dmats0[36][47] + coeff0_37*dmats0[37][47] + coeff0_38*dmats0[38][47] + coeff0_39*dmats0[39][47] + coeff0_40*dmats0[40][47] + coeff0_41*dmats0[41][47] + coeff0_42*dmats0[42][47] + coeff0_43*dmats0[43][47] + coeff0_44*dmats0[44][47] + coeff0_45*dmats0[45][47] + coeff0_46*dmats0[46][47] + coeff0_47*dmats0[47][47] + coeff0_48*dmats0[48][47] + coeff0_49*dmats0[49][47] + coeff0_50*dmats0[50][47] + coeff0_51*dmats0[51][47] + coeff0_52*dmats0[52][47] + coeff0_53*dmats0[53][47] + coeff0_54*dmats0[54][47] + coeff0_55*dmats0[55][47];
          new_coeff0_48 = coeff0_0*dmats0[0][48] + coeff0_1*dmats0[1][48] + coeff0_2*dmats0[2][48] + coeff0_3*dmats0[3][48] + coeff0_4*dmats0[4][48] + coeff0_5*dmats0[5][48] + coeff0_6*dmats0[6][48] + coeff0_7*dmats0[7][48] + coeff0_8*dmats0[8][48] + coeff0_9*dmats0[9][48] + coeff0_10*dmats0[10][48] + coeff0_11*dmats0[11][48] + coeff0_12*dmats0[12][48] + coeff0_13*dmats0[13][48] + coeff0_14*dmats0[14][48] + coeff0_15*dmats0[15][48] + coeff0_16*dmats0[16][48] + coeff0_17*dmats0[17][48] + coeff0_18*dmats0[18][48] + coeff0_19*dmats0[19][48] + coeff0_20*dmats0[20][48] + coeff0_21*dmats0[21][48] + coeff0_22*dmats0[22][48] + coeff0_23*dmats0[23][48] + coeff0_24*dmats0[24][48] + coeff0_25*dmats0[25][48] + coeff0_26*dmats0[26][48] + coeff0_27*dmats0[27][48] + coeff0_28*dmats0[28][48] + coeff0_29*dmats0[29][48] + coeff0_30*dmats0[30][48] + coeff0_31*dmats0[31][48] + coeff0_32*dmats0[32][48] + coeff0_33*dmats0[33][48] + coeff0_34*dmats0[34][48] + coeff0_35*dmats0[35][48] + coeff0_36*dmats0[36][48] + coeff0_37*dmats0[37][48] + coeff0_38*dmats0[38][48] + coeff0_39*dmats0[39][48] + coeff0_40*dmats0[40][48] + coeff0_41*dmats0[41][48] + coeff0_42*dmats0[42][48] + coeff0_43*dmats0[43][48] + coeff0_44*dmats0[44][48] + coeff0_45*dmats0[45][48] + coeff0_46*dmats0[46][48] + coeff0_47*dmats0[47][48] + coeff0_48*dmats0[48][48] + coeff0_49*dmats0[49][48] + coeff0_50*dmats0[50][48] + coeff0_51*dmats0[51][48] + coeff0_52*dmats0[52][48] + coeff0_53*dmats0[53][48] + coeff0_54*dmats0[54][48] + coeff0_55*dmats0[55][48];
          new_coeff0_49 = coeff0_0*dmats0[0][49] + coeff0_1*dmats0[1][49] + coeff0_2*dmats0[2][49] + coeff0_3*dmats0[3][49] + coeff0_4*dmats0[4][49] + coeff0_5*dmats0[5][49] + coeff0_6*dmats0[6][49] + coeff0_7*dmats0[7][49] + coeff0_8*dmats0[8][49] + coeff0_9*dmats0[9][49] + coeff0_10*dmats0[10][49] + coeff0_11*dmats0[11][49] + coeff0_12*dmats0[12][49] + coeff0_13*dmats0[13][49] + coeff0_14*dmats0[14][49] + coeff0_15*dmats0[15][49] + coeff0_16*dmats0[16][49] + coeff0_17*dmats0[17][49] + coeff0_18*dmats0[18][49] + coeff0_19*dmats0[19][49] + coeff0_20*dmats0[20][49] + coeff0_21*dmats0[21][49] + coeff0_22*dmats0[22][49] + coeff0_23*dmats0[23][49] + coeff0_24*dmats0[24][49] + coeff0_25*dmats0[25][49] + coeff0_26*dmats0[26][49] + coeff0_27*dmats0[27][49] + coeff0_28*dmats0[28][49] + coeff0_29*dmats0[29][49] + coeff0_30*dmats0[30][49] + coeff0_31*dmats0[31][49] + coeff0_32*dmats0[32][49] + coeff0_33*dmats0[33][49] + coeff0_34*dmats0[34][49] + coeff0_35*dmats0[35][49] + coeff0_36*dmats0[36][49] + coeff0_37*dmats0[37][49] + coeff0_38*dmats0[38][49] + coeff0_39*dmats0[39][49] + coeff0_40*dmats0[40][49] + coeff0_41*dmats0[41][49] + coeff0_42*dmats0[42][49] + coeff0_43*dmats0[43][49] + coeff0_44*dmats0[44][49] + coeff0_45*dmats0[45][49] + coeff0_46*dmats0[46][49] + coeff0_47*dmats0[47][49] + coeff0_48*dmats0[48][49] + coeff0_49*dmats0[49][49] + coeff0_50*dmats0[50][49] + coeff0_51*dmats0[51][49] + coeff0_52*dmats0[52][49] + coeff0_53*dmats0[53][49] + coeff0_54*dmats0[54][49] + coeff0_55*dmats0[55][49];
          new_coeff0_50 = coeff0_0*dmats0[0][50] + coeff0_1*dmats0[1][50] + coeff0_2*dmats0[2][50] + coeff0_3*dmats0[3][50] + coeff0_4*dmats0[4][50] + coeff0_5*dmats0[5][50] + coeff0_6*dmats0[6][50] + coeff0_7*dmats0[7][50] + coeff0_8*dmats0[8][50] + coeff0_9*dmats0[9][50] + coeff0_10*dmats0[10][50] + coeff0_11*dmats0[11][50] + coeff0_12*dmats0[12][50] + coeff0_13*dmats0[13][50] + coeff0_14*dmats0[14][50] + coeff0_15*dmats0[15][50] + coeff0_16*dmats0[16][50] + coeff0_17*dmats0[17][50] + coeff0_18*dmats0[18][50] + coeff0_19*dmats0[19][50] + coeff0_20*dmats0[20][50] + coeff0_21*dmats0[21][50] + coeff0_22*dmats0[22][50] + coeff0_23*dmats0[23][50] + coeff0_24*dmats0[24][50] + coeff0_25*dmats0[25][50] + coeff0_26*dmats0[26][50] + coeff0_27*dmats0[27][50] + coeff0_28*dmats0[28][50] + coeff0_29*dmats0[29][50] + coeff0_30*dmats0[30][50] + coeff0_31*dmats0[31][50] + coeff0_32*dmats0[32][50] + coeff0_33*dmats0[33][50] + coeff0_34*dmats0[34][50] + coeff0_35*dmats0[35][50] + coeff0_36*dmats0[36][50] + coeff0_37*dmats0[37][50] + coeff0_38*dmats0[38][50] + coeff0_39*dmats0[39][50] + coeff0_40*dmats0[40][50] + coeff0_41*dmats0[41][50] + coeff0_42*dmats0[42][50] + coeff0_43*dmats0[43][50] + coeff0_44*dmats0[44][50] + coeff0_45*dmats0[45][50] + coeff0_46*dmats0[46][50] + coeff0_47*dmats0[47][50] + coeff0_48*dmats0[48][50] + coeff0_49*dmats0[49][50] + coeff0_50*dmats0[50][50] + coeff0_51*dmats0[51][50] + coeff0_52*dmats0[52][50] + coeff0_53*dmats0[53][50] + coeff0_54*dmats0[54][50] + coeff0_55*dmats0[55][50];
          new_coeff0_51 = coeff0_0*dmats0[0][51] + coeff0_1*dmats0[1][51] + coeff0_2*dmats0[2][51] + coeff0_3*dmats0[3][51] + coeff0_4*dmats0[4][51] + coeff0_5*dmats0[5][51] + coeff0_6*dmats0[6][51] + coeff0_7*dmats0[7][51] + coeff0_8*dmats0[8][51] + coeff0_9*dmats0[9][51] + coeff0_10*dmats0[10][51] + coeff0_11*dmats0[11][51] + coeff0_12*dmats0[12][51] + coeff0_13*dmats0[13][51] + coeff0_14*dmats0[14][51] + coeff0_15*dmats0[15][51] + coeff0_16*dmats0[16][51] + coeff0_17*dmats0[17][51] + coeff0_18*dmats0[18][51] + coeff0_19*dmats0[19][51] + coeff0_20*dmats0[20][51] + coeff0_21*dmats0[21][51] + coeff0_22*dmats0[22][51] + coeff0_23*dmats0[23][51] + coeff0_24*dmats0[24][51] + coeff0_25*dmats0[25][51] + coeff0_26*dmats0[26][51] + coeff0_27*dmats0[27][51] + coeff0_28*dmats0[28][51] + coeff0_29*dmats0[29][51] + coeff0_30*dmats0[30][51] + coeff0_31*dmats0[31][51] + coeff0_32*dmats0[32][51] + coeff0_33*dmats0[33][51] + coeff0_34*dmats0[34][51] + coeff0_35*dmats0[35][51] + coeff0_36*dmats0[36][51] + coeff0_37*dmats0[37][51] + coeff0_38*dmats0[38][51] + coeff0_39*dmats0[39][51] + coeff0_40*dmats0[40][51] + coeff0_41*dmats0[41][51] + coeff0_42*dmats0[42][51] + coeff0_43*dmats0[43][51] + coeff0_44*dmats0[44][51] + coeff0_45*dmats0[45][51] + coeff0_46*dmats0[46][51] + coeff0_47*dmats0[47][51] + coeff0_48*dmats0[48][51] + coeff0_49*dmats0[49][51] + coeff0_50*dmats0[50][51] + coeff0_51*dmats0[51][51] + coeff0_52*dmats0[52][51] + coeff0_53*dmats0[53][51] + coeff0_54*dmats0[54][51] + coeff0_55*dmats0[55][51];
          new_coeff0_52 = coeff0_0*dmats0[0][52] + coeff0_1*dmats0[1][52] + coeff0_2*dmats0[2][52] + coeff0_3*dmats0[3][52] + coeff0_4*dmats0[4][52] + coeff0_5*dmats0[5][52] + coeff0_6*dmats0[6][52] + coeff0_7*dmats0[7][52] + coeff0_8*dmats0[8][52] + coeff0_9*dmats0[9][52] + coeff0_10*dmats0[10][52] + coeff0_11*dmats0[11][52] + coeff0_12*dmats0[12][52] + coeff0_13*dmats0[13][52] + coeff0_14*dmats0[14][52] + coeff0_15*dmats0[15][52] + coeff0_16*dmats0[16][52] + coeff0_17*dmats0[17][52] + coeff0_18*dmats0[18][52] + coeff0_19*dmats0[19][52] + coeff0_20*dmats0[20][52] + coeff0_21*dmats0[21][52] + coeff0_22*dmats0[22][52] + coeff0_23*dmats0[23][52] + coeff0_24*dmats0[24][52] + coeff0_25*dmats0[25][52] + coeff0_26*dmats0[26][52] + coeff0_27*dmats0[27][52] + coeff0_28*dmats0[28][52] + coeff0_29*dmats0[29][52] + coeff0_30*dmats0[30][52] + coeff0_31*dmats0[31][52] + coeff0_32*dmats0[32][52] + coeff0_33*dmats0[33][52] + coeff0_34*dmats0[34][52] + coeff0_35*dmats0[35][52] + coeff0_36*dmats0[36][52] + coeff0_37*dmats0[37][52] + coeff0_38*dmats0[38][52] + coeff0_39*dmats0[39][52] + coeff0_40*dmats0[40][52] + coeff0_41*dmats0[41][52] + coeff0_42*dmats0[42][52] + coeff0_43*dmats0[43][52] + coeff0_44*dmats0[44][52] + coeff0_45*dmats0[45][52] + coeff0_46*dmats0[46][52] + coeff0_47*dmats0[47][52] + coeff0_48*dmats0[48][52] + coeff0_49*dmats0[49][52] + coeff0_50*dmats0[50][52] + coeff0_51*dmats0[51][52] + coeff0_52*dmats0[52][52] + coeff0_53*dmats0[53][52] + coeff0_54*dmats0[54][52] + coeff0_55*dmats0[55][52];
          new_coeff0_53 = coeff0_0*dmats0[0][53] + coeff0_1*dmats0[1][53] + coeff0_2*dmats0[2][53] + coeff0_3*dmats0[3][53] + coeff0_4*dmats0[4][53] + coeff0_5*dmats0[5][53] + coeff0_6*dmats0[6][53] + coeff0_7*dmats0[7][53] + coeff0_8*dmats0[8][53] + coeff0_9*dmats0[9][53] + coeff0_10*dmats0[10][53] + coeff0_11*dmats0[11][53] + coeff0_12*dmats0[12][53] + coeff0_13*dmats0[13][53] + coeff0_14*dmats0[14][53] + coeff0_15*dmats0[15][53] + coeff0_16*dmats0[16][53] + coeff0_17*dmats0[17][53] + coeff0_18*dmats0[18][53] + coeff0_19*dmats0[19][53] + coeff0_20*dmats0[20][53] + coeff0_21*dmats0[21][53] + coeff0_22*dmats0[22][53] + coeff0_23*dmats0[23][53] + coeff0_24*dmats0[24][53] + coeff0_25*dmats0[25][53] + coeff0_26*dmats0[26][53] + coeff0_27*dmats0[27][53] + coeff0_28*dmats0[28][53] + coeff0_29*dmats0[29][53] + coeff0_30*dmats0[30][53] + coeff0_31*dmats0[31][53] + coeff0_32*dmats0[32][53] + coeff0_33*dmats0[33][53] + coeff0_34*dmats0[34][53] + coeff0_35*dmats0[35][53] + coeff0_36*dmats0[36][53] + coeff0_37*dmats0[37][53] + coeff0_38*dmats0[38][53] + coeff0_39*dmats0[39][53] + coeff0_40*dmats0[40][53] + coeff0_41*dmats0[41][53] + coeff0_42*dmats0[42][53] + coeff0_43*dmats0[43][53] + coeff0_44*dmats0[44][53] + coeff0_45*dmats0[45][53] + coeff0_46*dmats0[46][53] + coeff0_47*dmats0[47][53] + coeff0_48*dmats0[48][53] + coeff0_49*dmats0[49][53] + coeff0_50*dmats0[50][53] + coeff0_51*dmats0[51][53] + coeff0_52*dmats0[52][53] + coeff0_53*dmats0[53][53] + coeff0_54*dmats0[54][53] + coeff0_55*dmats0[55][53];
          new_coeff0_54 = coeff0_0*dmats0[0][54] + coeff0_1*dmats0[1][54] + coeff0_2*dmats0[2][54] + coeff0_3*dmats0[3][54] + coeff0_4*dmats0[4][54] + coeff0_5*dmats0[5][54] + coeff0_6*dmats0[6][54] + coeff0_7*dmats0[7][54] + coeff0_8*dmats0[8][54] + coeff0_9*dmats0[9][54] + coeff0_10*dmats0[10][54] + coeff0_11*dmats0[11][54] + coeff0_12*dmats0[12][54] + coeff0_13*dmats0[13][54] + coeff0_14*dmats0[14][54] + coeff0_15*dmats0[15][54] + coeff0_16*dmats0[16][54] + coeff0_17*dmats0[17][54] + coeff0_18*dmats0[18][54] + coeff0_19*dmats0[19][54] + coeff0_20*dmats0[20][54] + coeff0_21*dmats0[21][54] + coeff0_22*dmats0[22][54] + coeff0_23*dmats0[23][54] + coeff0_24*dmats0[24][54] + coeff0_25*dmats0[25][54] + coeff0_26*dmats0[26][54] + coeff0_27*dmats0[27][54] + coeff0_28*dmats0[28][54] + coeff0_29*dmats0[29][54] + coeff0_30*dmats0[30][54] + coeff0_31*dmats0[31][54] + coeff0_32*dmats0[32][54] + coeff0_33*dmats0[33][54] + coeff0_34*dmats0[34][54] + coeff0_35*dmats0[35][54] + coeff0_36*dmats0[36][54] + coeff0_37*dmats0[37][54] + coeff0_38*dmats0[38][54] + coeff0_39*dmats0[39][54] + coeff0_40*dmats0[40][54] + coeff0_41*dmats0[41][54] + coeff0_42*dmats0[42][54] + coeff0_43*dmats0[43][54] + coeff0_44*dmats0[44][54] + coeff0_45*dmats0[45][54] + coeff0_46*dmats0[46][54] + coeff0_47*dmats0[47][54] + coeff0_48*dmats0[48][54] + coeff0_49*dmats0[49][54] + coeff0_50*dmats0[50][54] + coeff0_51*dmats0[51][54] + coeff0_52*dmats0[52][54] + coeff0_53*dmats0[53][54] + coeff0_54*dmats0[54][54] + coeff0_55*dmats0[55][54];
          new_coeff0_55 = coeff0_0*dmats0[0][55] + coeff0_1*dmats0[1][55] + coeff0_2*dmats0[2][55] + coeff0_3*dmats0[3][55] + coeff0_4*dmats0[4][55] + coeff0_5*dmats0[5][55] + coeff0_6*dmats0[6][55] + coeff0_7*dmats0[7][55] + coeff0_8*dmats0[8][55] + coeff0_9*dmats0[9][55] + coeff0_10*dmats0[10][55] + coeff0_11*dmats0[11][55] + coeff0_12*dmats0[12][55] + coeff0_13*dmats0[13][55] + coeff0_14*dmats0[14][55] + coeff0_15*dmats0[15][55] + coeff0_16*dmats0[16][55] + coeff0_17*dmats0[17][55] + coeff0_18*dmats0[18][55] + coeff0_19*dmats0[19][55] + coeff0_20*dmats0[20][55] + coeff0_21*dmats0[21][55] + coeff0_22*dmats0[22][55] + coeff0_23*dmats0[23][55] + coeff0_24*dmats0[24][55] + coeff0_25*dmats0[25][55] + coeff0_26*dmats0[26][55] + coeff0_27*dmats0[27][55] + coeff0_28*dmats0[28][55] + coeff0_29*dmats0[29][55] + coeff0_30*dmats0[30][55] + coeff0_31*dmats0[31][55] + coeff0_32*dmats0[32][55] + coeff0_33*dmats0[33][55] + coeff0_34*dmats0[34][55] + coeff0_35*dmats0[35][55] + coeff0_36*dmats0[36][55] + coeff0_37*dmats0[37][55] + coeff0_38*dmats0[38][55] + coeff0_39*dmats0[39][55] + coeff0_40*dmats0[40][55] + coeff0_41*dmats0[41][55] + coeff0_42*dmats0[42][55] + coeff0_43*dmats0[43][55] + coeff0_44*dmats0[44][55] + coeff0_45*dmats0[45][55] + coeff0_46*dmats0[46][55] + coeff0_47*dmats0[47][55] + coeff0_48*dmats0[48][55] + coeff0_49*dmats0[49][55] + coeff0_50*dmats0[50][55] + coeff0_51*dmats0[51][55] + coeff0_52*dmats0[52][55] + coeff0_53*dmats0[53][55] + coeff0_54*dmats0[54][55] + coeff0_55*dmats0[55][55];
        }
        if(combinations[deriv_num][j] == 1)
        {
          new_coeff0_0 = coeff0_0*dmats1[0][0] + coeff0_1*dmats1[1][0] + coeff0_2*dmats1[2][0] + coeff0_3*dmats1[3][0] + coeff0_4*dmats1[4][0] + coeff0_5*dmats1[5][0] + coeff0_6*dmats1[6][0] + coeff0_7*dmats1[7][0] + coeff0_8*dmats1[8][0] + coeff0_9*dmats1[9][0] + coeff0_10*dmats1[10][0] + coeff0_11*dmats1[11][0] + coeff0_12*dmats1[12][0] + coeff0_13*dmats1[13][0] + coeff0_14*dmats1[14][0] + coeff0_15*dmats1[15][0] + coeff0_16*dmats1[16][0] + coeff0_17*dmats1[17][0] + coeff0_18*dmats1[18][0] + coeff0_19*dmats1[19][0] + coeff0_20*dmats1[20][0] + coeff0_21*dmats1[21][0] + coeff0_22*dmats1[22][0] + coeff0_23*dmats1[23][0] + coeff0_24*dmats1[24][0] + coeff0_25*dmats1[25][0] + coeff0_26*dmats1[26][0] + coeff0_27*dmats1[27][0] + coeff0_28*dmats1[28][0] + coeff0_29*dmats1[29][0] + coeff0_30*dmats1[30][0] + coeff0_31*dmats1[31][0] + coeff0_32*dmats1[32][0] + coeff0_33*dmats1[33][0] + coeff0_34*dmats1[34][0] + coeff0_35*dmats1[35][0] + coeff0_36*dmats1[36][0] + coeff0_37*dmats1[37][0] + coeff0_38*dmats1[38][0] + coeff0_39*dmats1[39][0] + coeff0_40*dmats1[40][0] + coeff0_41*dmats1[41][0] + coeff0_42*dmats1[42][0] + coeff0_43*dmats1[43][0] + coeff0_44*dmats1[44][0] + coeff0_45*dmats1[45][0] + coeff0_46*dmats1[46][0] + coeff0_47*dmats1[47][0] + coeff0_48*dmats1[48][0] + coeff0_49*dmats1[49][0] + coeff0_50*dmats1[50][0] + coeff0_51*dmats1[51][0] + coeff0_52*dmats1[52][0] + coeff0_53*dmats1[53][0] + coeff0_54*dmats1[54][0] + coeff0_55*dmats1[55][0];
          new_coeff0_1 = coeff0_0*dmats1[0][1] + coeff0_1*dmats1[1][1] + coeff0_2*dmats1[2][1] + coeff0_3*dmats1[3][1] + coeff0_4*dmats1[4][1] + coeff0_5*dmats1[5][1] + coeff0_6*dmats1[6][1] + coeff0_7*dmats1[7][1] + coeff0_8*dmats1[8][1] + coeff0_9*dmats1[9][1] + coeff0_10*dmats1[10][1] + coeff0_11*dmats1[11][1] + coeff0_12*dmats1[12][1] + coeff0_13*dmats1[13][1] + coeff0_14*dmats1[14][1] + coeff0_15*dmats1[15][1] + coeff0_16*dmats1[16][1] + coeff0_17*dmats1[17][1] + coeff0_18*dmats1[18][1] + coeff0_19*dmats1[19][1] + coeff0_20*dmats1[20][1] + coeff0_21*dmats1[21][1] + coeff0_22*dmats1[22][1] + coeff0_23*dmats1[23][1] + coeff0_24*dmats1[24][1] + coeff0_25*dmats1[25][1] + coeff0_26*dmats1[26][1] + coeff0_27*dmats1[27][1] + coeff0_28*dmats1[28][1] + coeff0_29*dmats1[29][1] + coeff0_30*dmats1[30][1] + coeff0_31*dmats1[31][1] + coeff0_32*dmats1[32][1] + coeff0_33*dmats1[33][1] + coeff0_34*dmats1[34][1] + coeff0_35*dmats1[35][1] + coeff0_36*dmats1[36][1] + coeff0_37*dmats1[37][1] + coeff0_38*dmats1[38][1] + coeff0_39*dmats1[39][1] + coeff0_40*dmats1[40][1] + coeff0_41*dmats1[41][1] + coeff0_42*dmats1[42][1] + coeff0_43*dmats1[43][1] + coeff0_44*dmats1[44][1] + coeff0_45*dmats1[45][1] + coeff0_46*dmats1[46][1] + coeff0_47*dmats1[47][1] + coeff0_48*dmats1[48][1] + coeff0_49*dmats1[49][1] + coeff0_50*dmats1[50][1] + coeff0_51*dmats1[51][1] + coeff0_52*dmats1[52][1] + coeff0_53*dmats1[53][1] + coeff0_54*dmats1[54][1] + coeff0_55*dmats1[55][1];
          new_coeff0_2 = coeff0_0*dmats1[0][2] + coeff0_1*dmats1[1][2] + coeff0_2*dmats1[2][2] + coeff0_3*dmats1[3][2] + coeff0_4*dmats1[4][2] + coeff0_5*dmats1[5][2] + coeff0_6*dmats1[6][2] + coeff0_7*dmats1[7][2] + coeff0_8*dmats1[8][2] + coeff0_9*dmats1[9][2] + coeff0_10*dmats1[10][2] + coeff0_11*dmats1[11][2] + coeff0_12*dmats1[12][2] + coeff0_13*dmats1[13][2] + coeff0_14*dmats1[14][2] + coeff0_15*dmats1[15][2] + coeff0_16*dmats1[16][2] + coeff0_17*dmats1[17][2] + coeff0_18*dmats1[18][2] + coeff0_19*dmats1[19][2] + coeff0_20*dmats1[20][2] + coeff0_21*dmats1[21][2] + coeff0_22*dmats1[22][2] + coeff0_23*dmats1[23][2] + coeff0_24*dmats1[24][2] + coeff0_25*dmats1[25][2] + coeff0_26*dmats1[26][2] + coeff0_27*dmats1[27][2] + coeff0_28*dmats1[28][2] + coeff0_29*dmats1[29][2] + coeff0_30*dmats1[30][2] + coeff0_31*dmats1[31][2] + coeff0_32*dmats1[32][2] + coeff0_33*dmats1[33][2] + coeff0_34*dmats1[34][2] + coeff0_35*dmats1[35][2] + coeff0_36*dmats1[36][2] + coeff0_37*dmats1[37][2] + coeff0_38*dmats1[38][2] + coeff0_39*dmats1[39][2] + coeff0_40*dmats1[40][2] + coeff0_41*dmats1[41][2] + coeff0_42*dmats1[42][2] + coeff0_43*dmats1[43][2] + coeff0_44*dmats1[44][2] + coeff0_45*dmats1[45][2] + coeff0_46*dmats1[46][2] + coeff0_47*dmats1[47][2] + coeff0_48*dmats1[48][2] + coeff0_49*dmats1[49][2] + coeff0_50*dmats1[50][2] + coeff0_51*dmats1[51][2] + coeff0_52*dmats1[52][2] + coeff0_53*dmats1[53][2] + coeff0_54*dmats1[54][2] + coeff0_55*dmats1[55][2];
          new_coeff0_3 = coeff0_0*dmats1[0][3] + coeff0_1*dmats1[1][3] + coeff0_2*dmats1[2][3] + coeff0_3*dmats1[3][3] + coeff0_4*dmats1[4][3] + coeff0_5*dmats1[5][3] + coeff0_6*dmats1[6][3] + coeff0_7*dmats1[7][3] + coeff0_8*dmats1[8][3] + coeff0_9*dmats1[9][3] + coeff0_10*dmats1[10][3] + coeff0_11*dmats1[11][3] + coeff0_12*dmats1[12][3] + coeff0_13*dmats1[13][3] + coeff0_14*dmats1[14][3] + coeff0_15*dmats1[15][3] + coeff0_16*dmats1[16][3] + coeff0_17*dmats1[17][3] + coeff0_18*dmats1[18][3] + coeff0_19*dmats1[19][3] + coeff0_20*dmats1[20][3] + coeff0_21*dmats1[21][3] + coeff0_22*dmats1[22][3] + coeff0_23*dmats1[23][3] + coeff0_24*dmats1[24][3] + coeff0_25*dmats1[25][3] + coeff0_26*dmats1[26][3] + coeff0_27*dmats1[27][3] + coeff0_28*dmats1[28][3] + coeff0_29*dmats1[29][3] + coeff0_30*dmats1[30][3] + coeff0_31*dmats1[31][3] + coeff0_32*dmats1[32][3] + coeff0_33*dmats1[33][3] + coeff0_34*dmats1[34][3] + coeff0_35*dmats1[35][3] + coeff0_36*dmats1[36][3] + coeff0_37*dmats1[37][3] + coeff0_38*dmats1[38][3] + coeff0_39*dmats1[39][3] + coeff0_40*dmats1[40][3] + coeff0_41*dmats1[41][3] + coeff0_42*dmats1[42][3] + coeff0_43*dmats1[43][3] + coeff0_44*dmats1[44][3] + coeff0_45*dmats1[45][3] + coeff0_46*dmats1[46][3] + coeff0_47*dmats1[47][3] + coeff0_48*dmats1[48][3] + coeff0_49*dmats1[49][3] + coeff0_50*dmats1[50][3] + coeff0_51*dmats1[51][3] + coeff0_52*dmats1[52][3] + coeff0_53*dmats1[53][3] + coeff0_54*dmats1[54][3] + coeff0_55*dmats1[55][3];
          new_coeff0_4 = coeff0_0*dmats1[0][4] + coeff0_1*dmats1[1][4] + coeff0_2*dmats1[2][4] + coeff0_3*dmats1[3][4] + coeff0_4*dmats1[4][4] + coeff0_5*dmats1[5][4] + coeff0_6*dmats1[6][4] + coeff0_7*dmats1[7][4] + coeff0_8*dmats1[8][4] + coeff0_9*dmats1[9][4] + coeff0_10*dmats1[10][4] + coeff0_11*dmats1[11][4] + coeff0_12*dmats1[12][4] + coeff0_13*dmats1[13][4] + coeff0_14*dmats1[14][4] + coeff0_15*dmats1[15][4] + coeff0_16*dmats1[16][4] + coeff0_17*dmats1[17][4] + coeff0_18*dmats1[18][4] + coeff0_19*dmats1[19][4] + coeff0_20*dmats1[20][4] + coeff0_21*dmats1[21][4] + coeff0_22*dmats1[22][4] + coeff0_23*dmats1[23][4] + coeff0_24*dmats1[24][4] + coeff0_25*dmats1[25][4] + coeff0_26*dmats1[26][4] + coeff0_27*dmats1[27][4] + coeff0_28*dmats1[28][4] + coeff0_29*dmats1[29][4] + coeff0_30*dmats1[30][4] + coeff0_31*dmats1[31][4] + coeff0_32*dmats1[32][4] + coeff0_33*dmats1[33][4] + coeff0_34*dmats1[34][4] + coeff0_35*dmats1[35][4] + coeff0_36*dmats1[36][4] + coeff0_37*dmats1[37][4] + coeff0_38*dmats1[38][4] + coeff0_39*dmats1[39][4] + coeff0_40*dmats1[40][4] + coeff0_41*dmats1[41][4] + coeff0_42*dmats1[42][4] + coeff0_43*dmats1[43][4] + coeff0_44*dmats1[44][4] + coeff0_45*dmats1[45][4] + coeff0_46*dmats1[46][4] + coeff0_47*dmats1[47][4] + coeff0_48*dmats1[48][4] + coeff0_49*dmats1[49][4] + coeff0_50*dmats1[50][4] + coeff0_51*dmats1[51][4] + coeff0_52*dmats1[52][4] + coeff0_53*dmats1[53][4] + coeff0_54*dmats1[54][4] + coeff0_55*dmats1[55][4];
          new_coeff0_5 = coeff0_0*dmats1[0][5] + coeff0_1*dmats1[1][5] + coeff0_2*dmats1[2][5] + coeff0_3*dmats1[3][5] + coeff0_4*dmats1[4][5] + coeff0_5*dmats1[5][5] + coeff0_6*dmats1[6][5] + coeff0_7*dmats1[7][5] + coeff0_8*dmats1[8][5] + coeff0_9*dmats1[9][5] + coeff0_10*dmats1[10][5] + coeff0_11*dmats1[11][5] + coeff0_12*dmats1[12][5] + coeff0_13*dmats1[13][5] + coeff0_14*dmats1[14][5] + coeff0_15*dmats1[15][5] + coeff0_16*dmats1[16][5] + coeff0_17*dmats1[17][5] + coeff0_18*dmats1[18][5] + coeff0_19*dmats1[19][5] + coeff0_20*dmats1[20][5] + coeff0_21*dmats1[21][5] + coeff0_22*dmats1[22][5] + coeff0_23*dmats1[23][5] + coeff0_24*dmats1[24][5] + coeff0_25*dmats1[25][5] + coeff0_26*dmats1[26][5] + coeff0_27*dmats1[27][5] + coeff0_28*dmats1[28][5] + coeff0_29*dmats1[29][5] + coeff0_30*dmats1[30][5] + coeff0_31*dmats1[31][5] + coeff0_32*dmats1[32][5] + coeff0_33*dmats1[33][5] + coeff0_34*dmats1[34][5] + coeff0_35*dmats1[35][5] + coeff0_36*dmats1[36][5] + coeff0_37*dmats1[37][5] + coeff0_38*dmats1[38][5] + coeff0_39*dmats1[39][5] + coeff0_40*dmats1[40][5] + coeff0_41*dmats1[41][5] + coeff0_42*dmats1[42][5] + coeff0_43*dmats1[43][5] + coeff0_44*dmats1[44][5] + coeff0_45*dmats1[45][5] + coeff0_46*dmats1[46][5] + coeff0_47*dmats1[47][5] + coeff0_48*dmats1[48][5] + coeff0_49*dmats1[49][5] + coeff0_50*dmats1[50][5] + coeff0_51*dmats1[51][5] + coeff0_52*dmats1[52][5] + coeff0_53*dmats1[53][5] + coeff0_54*dmats1[54][5] + coeff0_55*dmats1[55][5];
          new_coeff0_6 = coeff0_0*dmats1[0][6] + coeff0_1*dmats1[1][6] + coeff0_2*dmats1[2][6] + coeff0_3*dmats1[3][6] + coeff0_4*dmats1[4][6] + coeff0_5*dmats1[5][6] + coeff0_6*dmats1[6][6] + coeff0_7*dmats1[7][6] + coeff0_8*dmats1[8][6] + coeff0_9*dmats1[9][6] + coeff0_10*dmats1[10][6] + coeff0_11*dmats1[11][6] + coeff0_12*dmats1[12][6] + coeff0_13*dmats1[13][6] + coeff0_14*dmats1[14][6] + coeff0_15*dmats1[15][6] + coeff0_16*dmats1[16][6] + coeff0_17*dmats1[17][6] + coeff0_18*dmats1[18][6] + coeff0_19*dmats1[19][6] + coeff0_20*dmats1[20][6] + coeff0_21*dmats1[21][6] + coeff0_22*dmats1[22][6] + coeff0_23*dmats1[23][6] + coeff0_24*dmats1[24][6] + coeff0_25*dmats1[25][6] + coeff0_26*dmats1[26][6] + coeff0_27*dmats1[27][6] + coeff0_28*dmats1[28][6] + coeff0_29*dmats1[29][6] + coeff0_30*dmats1[30][6] + coeff0_31*dmats1[31][6] + coeff0_32*dmats1[32][6] + coeff0_33*dmats1[33][6] + coeff0_34*dmats1[34][6] + coeff0_35*dmats1[35][6] + coeff0_36*dmats1[36][6] + coeff0_37*dmats1[37][6] + coeff0_38*dmats1[38][6] + coeff0_39*dmats1[39][6] + coeff0_40*dmats1[40][6] + coeff0_41*dmats1[41][6] + coeff0_42*dmats1[42][6] + coeff0_43*dmats1[43][6] + coeff0_44*dmats1[44][6] + coeff0_45*dmats1[45][6] + coeff0_46*dmats1[46][6] + coeff0_47*dmats1[47][6] + coeff0_48*dmats1[48][6] + coeff0_49*dmats1[49][6] + coeff0_50*dmats1[50][6] + coeff0_51*dmats1[51][6] + coeff0_52*dmats1[52][6] + coeff0_53*dmats1[53][6] + coeff0_54*dmats1[54][6] + coeff0_55*dmats1[55][6];
          new_coeff0_7 = coeff0_0*dmats1[0][7] + coeff0_1*dmats1[1][7] + coeff0_2*dmats1[2][7] + coeff0_3*dmats1[3][7] + coeff0_4*dmats1[4][7] + coeff0_5*dmats1[5][7] + coeff0_6*dmats1[6][7] + coeff0_7*dmats1[7][7] + coeff0_8*dmats1[8][7] + coeff0_9*dmats1[9][7] + coeff0_10*dmats1[10][7] + coeff0_11*dmats1[11][7] + coeff0_12*dmats1[12][7] + coeff0_13*dmats1[13][7] + coeff0_14*dmats1[14][7] + coeff0_15*dmats1[15][7] + coeff0_16*dmats1[16][7] + coeff0_17*dmats1[17][7] + coeff0_18*dmats1[18][7] + coeff0_19*dmats1[19][7] + coeff0_20*dmats1[20][7] + coeff0_21*dmats1[21][7] + coeff0_22*dmats1[22][7] + coeff0_23*dmats1[23][7] + coeff0_24*dmats1[24][7] + coeff0_25*dmats1[25][7] + coeff0_26*dmats1[26][7] + coeff0_27*dmats1[27][7] + coeff0_28*dmats1[28][7] + coeff0_29*dmats1[29][7] + coeff0_30*dmats1[30][7] + coeff0_31*dmats1[31][7] + coeff0_32*dmats1[32][7] + coeff0_33*dmats1[33][7] + coeff0_34*dmats1[34][7] + coeff0_35*dmats1[35][7] + coeff0_36*dmats1[36][7] + coeff0_37*dmats1[37][7] + coeff0_38*dmats1[38][7] + coeff0_39*dmats1[39][7] + coeff0_40*dmats1[40][7] + coeff0_41*dmats1[41][7] + coeff0_42*dmats1[42][7] + coeff0_43*dmats1[43][7] + coeff0_44*dmats1[44][7] + coeff0_45*dmats1[45][7] + coeff0_46*dmats1[46][7] + coeff0_47*dmats1[47][7] + coeff0_48*dmats1[48][7] + coeff0_49*dmats1[49][7] + coeff0_50*dmats1[50][7] + coeff0_51*dmats1[51][7] + coeff0_52*dmats1[52][7] + coeff0_53*dmats1[53][7] + coeff0_54*dmats1[54][7] + coeff0_55*dmats1[55][7];
          new_coeff0_8 = coeff0_0*dmats1[0][8] + coeff0_1*dmats1[1][8] + coeff0_2*dmats1[2][8] + coeff0_3*dmats1[3][8] + coeff0_4*dmats1[4][8] + coeff0_5*dmats1[5][8] + coeff0_6*dmats1[6][8] + coeff0_7*dmats1[7][8] + coeff0_8*dmats1[8][8] + coeff0_9*dmats1[9][8] + coeff0_10*dmats1[10][8] + coeff0_11*dmats1[11][8] + coeff0_12*dmats1[12][8] + coeff0_13*dmats1[13][8] + coeff0_14*dmats1[14][8] + coeff0_15*dmats1[15][8] + coeff0_16*dmats1[16][8] + coeff0_17*dmats1[17][8] + coeff0_18*dmats1[18][8] + coeff0_19*dmats1[19][8] + coeff0_20*dmats1[20][8] + coeff0_21*dmats1[21][8] + coeff0_22*dmats1[22][8] + coeff0_23*dmats1[23][8] + coeff0_24*dmats1[24][8] + coeff0_25*dmats1[25][8] + coeff0_26*dmats1[26][8] + coeff0_27*dmats1[27][8] + coeff0_28*dmats1[28][8] + coeff0_29*dmats1[29][8] + coeff0_30*dmats1[30][8] + coeff0_31*dmats1[31][8] + coeff0_32*dmats1[32][8] + coeff0_33*dmats1[33][8] + coeff0_34*dmats1[34][8] + coeff0_35*dmats1[35][8] + coeff0_36*dmats1[36][8] + coeff0_37*dmats1[37][8] + coeff0_38*dmats1[38][8] + coeff0_39*dmats1[39][8] + coeff0_40*dmats1[40][8] + coeff0_41*dmats1[41][8] + coeff0_42*dmats1[42][8] + coeff0_43*dmats1[43][8] + coeff0_44*dmats1[44][8] + coeff0_45*dmats1[45][8] + coeff0_46*dmats1[46][8] + coeff0_47*dmats1[47][8] + coeff0_48*dmats1[48][8] + coeff0_49*dmats1[49][8] + coeff0_50*dmats1[50][8] + coeff0_51*dmats1[51][8] + coeff0_52*dmats1[52][8] + coeff0_53*dmats1[53][8] + coeff0_54*dmats1[54][8] + coeff0_55*dmats1[55][8];
          new_coeff0_9 = coeff0_0*dmats1[0][9] + coeff0_1*dmats1[1][9] + coeff0_2*dmats1[2][9] + coeff0_3*dmats1[3][9] + coeff0_4*dmats1[4][9] + coeff0_5*dmats1[5][9] + coeff0_6*dmats1[6][9] + coeff0_7*dmats1[7][9] + coeff0_8*dmats1[8][9] + coeff0_9*dmats1[9][9] + coeff0_10*dmats1[10][9] + coeff0_11*dmats1[11][9] + coeff0_12*dmats1[12][9] + coeff0_13*dmats1[13][9] + coeff0_14*dmats1[14][9] + coeff0_15*dmats1[15][9] + coeff0_16*dmats1[16][9] + coeff0_17*dmats1[17][9] + coeff0_18*dmats1[18][9] + coeff0_19*dmats1[19][9] + coeff0_20*dmats1[20][9] + coeff0_21*dmats1[21][9] + coeff0_22*dmats1[22][9] + coeff0_23*dmats1[23][9] + coeff0_24*dmats1[24][9] + coeff0_25*dmats1[25][9] + coeff0_26*dmats1[26][9] + coeff0_27*dmats1[27][9] + coeff0_28*dmats1[28][9] + coeff0_29*dmats1[29][9] + coeff0_30*dmats1[30][9] + coeff0_31*dmats1[31][9] + coeff0_32*dmats1[32][9] + coeff0_33*dmats1[33][9] + coeff0_34*dmats1[34][9] + coeff0_35*dmats1[35][9] + coeff0_36*dmats1[36][9] + coeff0_37*dmats1[37][9] + coeff0_38*dmats1[38][9] + coeff0_39*dmats1[39][9] + coeff0_40*dmats1[40][9] + coeff0_41*dmats1[41][9] + coeff0_42*dmats1[42][9] + coeff0_43*dmats1[43][9] + coeff0_44*dmats1[44][9] + coeff0_45*dmats1[45][9] + coeff0_46*dmats1[46][9] + coeff0_47*dmats1[47][9] + coeff0_48*dmats1[48][9] + coeff0_49*dmats1[49][9] + coeff0_50*dmats1[50][9] + coeff0_51*dmats1[51][9] + coeff0_52*dmats1[52][9] + coeff0_53*dmats1[53][9] + coeff0_54*dmats1[54][9] + coeff0_55*dmats1[55][9];
          new_coeff0_10 = coeff0_0*dmats1[0][10] + coeff0_1*dmats1[1][10] + coeff0_2*dmats1[2][10] + coeff0_3*dmats1[3][10] + coeff0_4*dmats1[4][10] + coeff0_5*dmats1[5][10] + coeff0_6*dmats1[6][10] + coeff0_7*dmats1[7][10] + coeff0_8*dmats1[8][10] + coeff0_9*dmats1[9][10] + coeff0_10*dmats1[10][10] + coeff0_11*dmats1[11][10] + coeff0_12*dmats1[12][10] + coeff0_13*dmats1[13][10] + coeff0_14*dmats1[14][10] + coeff0_15*dmats1[15][10] + coeff0_16*dmats1[16][10] + coeff0_17*dmats1[17][10] + coeff0_18*dmats1[18][10] + coeff0_19*dmats1[19][10] + coeff0_20*dmats1[20][10] + coeff0_21*dmats1[21][10] + coeff0_22*dmats1[22][10] + coeff0_23*dmats1[23][10] + coeff0_24*dmats1[24][10] + coeff0_25*dmats1[25][10] + coeff0_26*dmats1[26][10] + coeff0_27*dmats1[27][10] + coeff0_28*dmats1[28][10] + coeff0_29*dmats1[29][10] + coeff0_30*dmats1[30][10] + coeff0_31*dmats1[31][10] + coeff0_32*dmats1[32][10] + coeff0_33*dmats1[33][10] + coeff0_34*dmats1[34][10] + coeff0_35*dmats1[35][10] + coeff0_36*dmats1[36][10] + coeff0_37*dmats1[37][10] + coeff0_38*dmats1[38][10] + coeff0_39*dmats1[39][10] + coeff0_40*dmats1[40][10] + coeff0_41*dmats1[41][10] + coeff0_42*dmats1[42][10] + coeff0_43*dmats1[43][10] + coeff0_44*dmats1[44][10] + coeff0_45*dmats1[45][10] + coeff0_46*dmats1[46][10] + coeff0_47*dmats1[47][10] + coeff0_48*dmats1[48][10] + coeff0_49*dmats1[49][10] + coeff0_50*dmats1[50][10] + coeff0_51*dmats1[51][10] + coeff0_52*dmats1[52][10] + coeff0_53*dmats1[53][10] + coeff0_54*dmats1[54][10] + coeff0_55*dmats1[55][10];
          new_coeff0_11 = coeff0_0*dmats1[0][11] + coeff0_1*dmats1[1][11] + coeff0_2*dmats1[2][11] + coeff0_3*dmats1[3][11] + coeff0_4*dmats1[4][11] + coeff0_5*dmats1[5][11] + coeff0_6*dmats1[6][11] + coeff0_7*dmats1[7][11] + coeff0_8*dmats1[8][11] + coeff0_9*dmats1[9][11] + coeff0_10*dmats1[10][11] + coeff0_11*dmats1[11][11] + coeff0_12*dmats1[12][11] + coeff0_13*dmats1[13][11] + coeff0_14*dmats1[14][11] + coeff0_15*dmats1[15][11] + coeff0_16*dmats1[16][11] + coeff0_17*dmats1[17][11] + coeff0_18*dmats1[18][11] + coeff0_19*dmats1[19][11] + coeff0_20*dmats1[20][11] + coeff0_21*dmats1[21][11] + coeff0_22*dmats1[22][11] + coeff0_23*dmats1[23][11] + coeff0_24*dmats1[24][11] + coeff0_25*dmats1[25][11] + coeff0_26*dmats1[26][11] + coeff0_27*dmats1[27][11] + coeff0_28*dmats1[28][11] + coeff0_29*dmats1[29][11] + coeff0_30*dmats1[30][11] + coeff0_31*dmats1[31][11] + coeff0_32*dmats1[32][11] + coeff0_33*dmats1[33][11] + coeff0_34*dmats1[34][11] + coeff0_35*dmats1[35][11] + coeff0_36*dmats1[36][11] + coeff0_37*dmats1[37][11] + coeff0_38*dmats1[38][11] + coeff0_39*dmats1[39][11] + coeff0_40*dmats1[40][11] + coeff0_41*dmats1[41][11] + coeff0_42*dmats1[42][11] + coeff0_43*dmats1[43][11] + coeff0_44*dmats1[44][11] + coeff0_45*dmats1[45][11] + coeff0_46*dmats1[46][11] + coeff0_47*dmats1[47][11] + coeff0_48*dmats1[48][11] + coeff0_49*dmats1[49][11] + coeff0_50*dmats1[50][11] + coeff0_51*dmats1[51][11] + coeff0_52*dmats1[52][11] + coeff0_53*dmats1[53][11] + coeff0_54*dmats1[54][11] + coeff0_55*dmats1[55][11];
          new_coeff0_12 = coeff0_0*dmats1[0][12] + coeff0_1*dmats1[1][12] + coeff0_2*dmats1[2][12] + coeff0_3*dmats1[3][12] + coeff0_4*dmats1[4][12] + coeff0_5*dmats1[5][12] + coeff0_6*dmats1[6][12] + coeff0_7*dmats1[7][12] + coeff0_8*dmats1[8][12] + coeff0_9*dmats1[9][12] + coeff0_10*dmats1[10][12] + coeff0_11*dmats1[11][12] + coeff0_12*dmats1[12][12] + coeff0_13*dmats1[13][12] + coeff0_14*dmats1[14][12] + coeff0_15*dmats1[15][12] + coeff0_16*dmats1[16][12] + coeff0_17*dmats1[17][12] + coeff0_18*dmats1[18][12] + coeff0_19*dmats1[19][12] + coeff0_20*dmats1[20][12] + coeff0_21*dmats1[21][12] + coeff0_22*dmats1[22][12] + coeff0_23*dmats1[23][12] + coeff0_24*dmats1[24][12] + coeff0_25*dmats1[25][12] + coeff0_26*dmats1[26][12] + coeff0_27*dmats1[27][12] + coeff0_28*dmats1[28][12] + coeff0_29*dmats1[29][12] + coeff0_30*dmats1[30][12] + coeff0_31*dmats1[31][12] + coeff0_32*dmats1[32][12] + coeff0_33*dmats1[33][12] + coeff0_34*dmats1[34][12] + coeff0_35*dmats1[35][12] + coeff0_36*dmats1[36][12] + coeff0_37*dmats1[37][12] + coeff0_38*dmats1[38][12] + coeff0_39*dmats1[39][12] + coeff0_40*dmats1[40][12] + coeff0_41*dmats1[41][12] + coeff0_42*dmats1[42][12] + coeff0_43*dmats1[43][12] + coeff0_44*dmats1[44][12] + coeff0_45*dmats1[45][12] + coeff0_46*dmats1[46][12] + coeff0_47*dmats1[47][12] + coeff0_48*dmats1[48][12] + coeff0_49*dmats1[49][12] + coeff0_50*dmats1[50][12] + coeff0_51*dmats1[51][12] + coeff0_52*dmats1[52][12] + coeff0_53*dmats1[53][12] + coeff0_54*dmats1[54][12] + coeff0_55*dmats1[55][12];
          new_coeff0_13 = coeff0_0*dmats1[0][13] + coeff0_1*dmats1[1][13] + coeff0_2*dmats1[2][13] + coeff0_3*dmats1[3][13] + coeff0_4*dmats1[4][13] + coeff0_5*dmats1[5][13] + coeff0_6*dmats1[6][13] + coeff0_7*dmats1[7][13] + coeff0_8*dmats1[8][13] + coeff0_9*dmats1[9][13] + coeff0_10*dmats1[10][13] + coeff0_11*dmats1[11][13] + coeff0_12*dmats1[12][13] + coeff0_13*dmats1[13][13] + coeff0_14*dmats1[14][13] + coeff0_15*dmats1[15][13] + coeff0_16*dmats1[16][13] + coeff0_17*dmats1[17][13] + coeff0_18*dmats1[18][13] + coeff0_19*dmats1[19][13] + coeff0_20*dmats1[20][13] + coeff0_21*dmats1[21][13] + coeff0_22*dmats1[22][13] + coeff0_23*dmats1[23][13] + coeff0_24*dmats1[24][13] + coeff0_25*dmats1[25][13] + coeff0_26*dmats1[26][13] + coeff0_27*dmats1[27][13] + coeff0_28*dmats1[28][13] + coeff0_29*dmats1[29][13] + coeff0_30*dmats1[30][13] + coeff0_31*dmats1[31][13] + coeff0_32*dmats1[32][13] + coeff0_33*dmats1[33][13] + coeff0_34*dmats1[34][13] + coeff0_35*dmats1[35][13] + coeff0_36*dmats1[36][13] + coeff0_37*dmats1[37][13] + coeff0_38*dmats1[38][13] + coeff0_39*dmats1[39][13] + coeff0_40*dmats1[40][13] + coeff0_41*dmats1[41][13] + coeff0_42*dmats1[42][13] + coeff0_43*dmats1[43][13] + coeff0_44*dmats1[44][13] + coeff0_45*dmats1[45][13] + coeff0_46*dmats1[46][13] + coeff0_47*dmats1[47][13] + coeff0_48*dmats1[48][13] + coeff0_49*dmats1[49][13] + coeff0_50*dmats1[50][13] + coeff0_51*dmats1[51][13] + coeff0_52*dmats1[52][13] + coeff0_53*dmats1[53][13] + coeff0_54*dmats1[54][13] + coeff0_55*dmats1[55][13];
          new_coeff0_14 = coeff0_0*dmats1[0][14] + coeff0_1*dmats1[1][14] + coeff0_2*dmats1[2][14] + coeff0_3*dmats1[3][14] + coeff0_4*dmats1[4][14] + coeff0_5*dmats1[5][14] + coeff0_6*dmats1[6][14] + coeff0_7*dmats1[7][14] + coeff0_8*dmats1[8][14] + coeff0_9*dmats1[9][14] + coeff0_10*dmats1[10][14] + coeff0_11*dmats1[11][14] + coeff0_12*dmats1[12][14] + coeff0_13*dmats1[13][14] + coeff0_14*dmats1[14][14] + coeff0_15*dmats1[15][14] + coeff0_16*dmats1[16][14] + coeff0_17*dmats1[17][14] + coeff0_18*dmats1[18][14] + coeff0_19*dmats1[19][14] + coeff0_20*dmats1[20][14] + coeff0_21*dmats1[21][14] + coeff0_22*dmats1[22][14] + coeff0_23*dmats1[23][14] + coeff0_24*dmats1[24][14] + coeff0_25*dmats1[25][14] + coeff0_26*dmats1[26][14] + coeff0_27*dmats1[27][14] + coeff0_28*dmats1[28][14] + coeff0_29*dmats1[29][14] + coeff0_30*dmats1[30][14] + coeff0_31*dmats1[31][14] + coeff0_32*dmats1[32][14] + coeff0_33*dmats1[33][14] + coeff0_34*dmats1[34][14] + coeff0_35*dmats1[35][14] + coeff0_36*dmats1[36][14] + coeff0_37*dmats1[37][14] + coeff0_38*dmats1[38][14] + coeff0_39*dmats1[39][14] + coeff0_40*dmats1[40][14] + coeff0_41*dmats1[41][14] + coeff0_42*dmats1[42][14] + coeff0_43*dmats1[43][14] + coeff0_44*dmats1[44][14] + coeff0_45*dmats1[45][14] + coeff0_46*dmats1[46][14] + coeff0_47*dmats1[47][14] + coeff0_48*dmats1[48][14] + coeff0_49*dmats1[49][14] + coeff0_50*dmats1[50][14] + coeff0_51*dmats1[51][14] + coeff0_52*dmats1[52][14] + coeff0_53*dmats1[53][14] + coeff0_54*dmats1[54][14] + coeff0_55*dmats1[55][14];
          new_coeff0_15 = coeff0_0*dmats1[0][15] + coeff0_1*dmats1[1][15] + coeff0_2*dmats1[2][15] + coeff0_3*dmats1[3][15] + coeff0_4*dmats1[4][15] + coeff0_5*dmats1[5][15] + coeff0_6*dmats1[6][15] + coeff0_7*dmats1[7][15] + coeff0_8*dmats1[8][15] + coeff0_9*dmats1[9][15] + coeff0_10*dmats1[10][15] + coeff0_11*dmats1[11][15] + coeff0_12*dmats1[12][15] + coeff0_13*dmats1[13][15] + coeff0_14*dmats1[14][15] + coeff0_15*dmats1[15][15] + coeff0_16*dmats1[16][15] + coeff0_17*dmats1[17][15] + coeff0_18*dmats1[18][15] + coeff0_19*dmats1[19][15] + coeff0_20*dmats1[20][15] + coeff0_21*dmats1[21][15] + coeff0_22*dmats1[22][15] + coeff0_23*dmats1[23][15] + coeff0_24*dmats1[24][15] + coeff0_25*dmats1[25][15] + coeff0_26*dmats1[26][15] + coeff0_27*dmats1[27][15] + coeff0_28*dmats1[28][15] + coeff0_29*dmats1[29][15] + coeff0_30*dmats1[30][15] + coeff0_31*dmats1[31][15] + coeff0_32*dmats1[32][15] + coeff0_33*dmats1[33][15] + coeff0_34*dmats1[34][15] + coeff0_35*dmats1[35][15] + coeff0_36*dmats1[36][15] + coeff0_37*dmats1[37][15] + coeff0_38*dmats1[38][15] + coeff0_39*dmats1[39][15] + coeff0_40*dmats1[40][15] + coeff0_41*dmats1[41][15] + coeff0_42*dmats1[42][15] + coeff0_43*dmats1[43][15] + coeff0_44*dmats1[44][15] + coeff0_45*dmats1[45][15] + coeff0_46*dmats1[46][15] + coeff0_47*dmats1[47][15] + coeff0_48*dmats1[48][15] + coeff0_49*dmats1[49][15] + coeff0_50*dmats1[50][15] + coeff0_51*dmats1[51][15] + coeff0_52*dmats1[52][15] + coeff0_53*dmats1[53][15] + coeff0_54*dmats1[54][15] + coeff0_55*dmats1[55][15];
          new_coeff0_16 = coeff0_0*dmats1[0][16] + coeff0_1*dmats1[1][16] + coeff0_2*dmats1[2][16] + coeff0_3*dmats1[3][16] + coeff0_4*dmats1[4][16] + coeff0_5*dmats1[5][16] + coeff0_6*dmats1[6][16] + coeff0_7*dmats1[7][16] + coeff0_8*dmats1[8][16] + coeff0_9*dmats1[9][16] + coeff0_10*dmats1[10][16] + coeff0_11*dmats1[11][16] + coeff0_12*dmats1[12][16] + coeff0_13*dmats1[13][16] + coeff0_14*dmats1[14][16] + coeff0_15*dmats1[15][16] + coeff0_16*dmats1[16][16] + coeff0_17*dmats1[17][16] + coeff0_18*dmats1[18][16] + coeff0_19*dmats1[19][16] + coeff0_20*dmats1[20][16] + coeff0_21*dmats1[21][16] + coeff0_22*dmats1[22][16] + coeff0_23*dmats1[23][16] + coeff0_24*dmats1[24][16] + coeff0_25*dmats1[25][16] + coeff0_26*dmats1[26][16] + coeff0_27*dmats1[27][16] + coeff0_28*dmats1[28][16] + coeff0_29*dmats1[29][16] + coeff0_30*dmats1[30][16] + coeff0_31*dmats1[31][16] + coeff0_32*dmats1[32][16] + coeff0_33*dmats1[33][16] + coeff0_34*dmats1[34][16] + coeff0_35*dmats1[35][16] + coeff0_36*dmats1[36][16] + coeff0_37*dmats1[37][16] + coeff0_38*dmats1[38][16] + coeff0_39*dmats1[39][16] + coeff0_40*dmats1[40][16] + coeff0_41*dmats1[41][16] + coeff0_42*dmats1[42][16] + coeff0_43*dmats1[43][16] + coeff0_44*dmats1[44][16] + coeff0_45*dmats1[45][16] + coeff0_46*dmats1[46][16] + coeff0_47*dmats1[47][16] + coeff0_48*dmats1[48][16] + coeff0_49*dmats1[49][16] + coeff0_50*dmats1[50][16] + coeff0_51*dmats1[51][16] + coeff0_52*dmats1[52][16] + coeff0_53*dmats1[53][16] + coeff0_54*dmats1[54][16] + coeff0_55*dmats1[55][16];
          new_coeff0_17 = coeff0_0*dmats1[0][17] + coeff0_1*dmats1[1][17] + coeff0_2*dmats1[2][17] + coeff0_3*dmats1[3][17] + coeff0_4*dmats1[4][17] + coeff0_5*dmats1[5][17] + coeff0_6*dmats1[6][17] + coeff0_7*dmats1[7][17] + coeff0_8*dmats1[8][17] + coeff0_9*dmats1[9][17] + coeff0_10*dmats1[10][17] + coeff0_11*dmats1[11][17] + coeff0_12*dmats1[12][17] + coeff0_13*dmats1[13][17] + coeff0_14*dmats1[14][17] + coeff0_15*dmats1[15][17] + coeff0_16*dmats1[16][17] + coeff0_17*dmats1[17][17] + coeff0_18*dmats1[18][17] + coeff0_19*dmats1[19][17] + coeff0_20*dmats1[20][17] + coeff0_21*dmats1[21][17] + coeff0_22*dmats1[22][17] + coeff0_23*dmats1[23][17] + coeff0_24*dmats1[24][17] + coeff0_25*dmats1[25][17] + coeff0_26*dmats1[26][17] + coeff0_27*dmats1[27][17] + coeff0_28*dmats1[28][17] + coeff0_29*dmats1[29][17] + coeff0_30*dmats1[30][17] + coeff0_31*dmats1[31][17] + coeff0_32*dmats1[32][17] + coeff0_33*dmats1[33][17] + coeff0_34*dmats1[34][17] + coeff0_35*dmats1[35][17] + coeff0_36*dmats1[36][17] + coeff0_37*dmats1[37][17] + coeff0_38*dmats1[38][17] + coeff0_39*dmats1[39][17] + coeff0_40*dmats1[40][17] + coeff0_41*dmats1[41][17] + coeff0_42*dmats1[42][17] + coeff0_43*dmats1[43][17] + coeff0_44*dmats1[44][17] + coeff0_45*dmats1[45][17] + coeff0_46*dmats1[46][17] + coeff0_47*dmats1[47][17] + coeff0_48*dmats1[48][17] + coeff0_49*dmats1[49][17] + coeff0_50*dmats1[50][17] + coeff0_51*dmats1[51][17] + coeff0_52*dmats1[52][17] + coeff0_53*dmats1[53][17] + coeff0_54*dmats1[54][17] + coeff0_55*dmats1[55][17];
          new_coeff0_18 = coeff0_0*dmats1[0][18] + coeff0_1*dmats1[1][18] + coeff0_2*dmats1[2][18] + coeff0_3*dmats1[3][18] + coeff0_4*dmats1[4][18] + coeff0_5*dmats1[5][18] + coeff0_6*dmats1[6][18] + coeff0_7*dmats1[7][18] + coeff0_8*dmats1[8][18] + coeff0_9*dmats1[9][18] + coeff0_10*dmats1[10][18] + coeff0_11*dmats1[11][18] + coeff0_12*dmats1[12][18] + coeff0_13*dmats1[13][18] + coeff0_14*dmats1[14][18] + coeff0_15*dmats1[15][18] + coeff0_16*dmats1[16][18] + coeff0_17*dmats1[17][18] + coeff0_18*dmats1[18][18] + coeff0_19*dmats1[19][18] + coeff0_20*dmats1[20][18] + coeff0_21*dmats1[21][18] + coeff0_22*dmats1[22][18] + coeff0_23*dmats1[23][18] + coeff0_24*dmats1[24][18] + coeff0_25*dmats1[25][18] + coeff0_26*dmats1[26][18] + coeff0_27*dmats1[27][18] + coeff0_28*dmats1[28][18] + coeff0_29*dmats1[29][18] + coeff0_30*dmats1[30][18] + coeff0_31*dmats1[31][18] + coeff0_32*dmats1[32][18] + coeff0_33*dmats1[33][18] + coeff0_34*dmats1[34][18] + coeff0_35*dmats1[35][18] + coeff0_36*dmats1[36][18] + coeff0_37*dmats1[37][18] + coeff0_38*dmats1[38][18] + coeff0_39*dmats1[39][18] + coeff0_40*dmats1[40][18] + coeff0_41*dmats1[41][18] + coeff0_42*dmats1[42][18] + coeff0_43*dmats1[43][18] + coeff0_44*dmats1[44][18] + coeff0_45*dmats1[45][18] + coeff0_46*dmats1[46][18] + coeff0_47*dmats1[47][18] + coeff0_48*dmats1[48][18] + coeff0_49*dmats1[49][18] + coeff0_50*dmats1[50][18] + coeff0_51*dmats1[51][18] + coeff0_52*dmats1[52][18] + coeff0_53*dmats1[53][18] + coeff0_54*dmats1[54][18] + coeff0_55*dmats1[55][18];
          new_coeff0_19 = coeff0_0*dmats1[0][19] + coeff0_1*dmats1[1][19] + coeff0_2*dmats1[2][19] + coeff0_3*dmats1[3][19] + coeff0_4*dmats1[4][19] + coeff0_5*dmats1[5][19] + coeff0_6*dmats1[6][19] + coeff0_7*dmats1[7][19] + coeff0_8*dmats1[8][19] + coeff0_9*dmats1[9][19] + coeff0_10*dmats1[10][19] + coeff0_11*dmats1[11][19] + coeff0_12*dmats1[12][19] + coeff0_13*dmats1[13][19] + coeff0_14*dmats1[14][19] + coeff0_15*dmats1[15][19] + coeff0_16*dmats1[16][19] + coeff0_17*dmats1[17][19] + coeff0_18*dmats1[18][19] + coeff0_19*dmats1[19][19] + coeff0_20*dmats1[20][19] + coeff0_21*dmats1[21][19] + coeff0_22*dmats1[22][19] + coeff0_23*dmats1[23][19] + coeff0_24*dmats1[24][19] + coeff0_25*dmats1[25][19] + coeff0_26*dmats1[26][19] + coeff0_27*dmats1[27][19] + coeff0_28*dmats1[28][19] + coeff0_29*dmats1[29][19] + coeff0_30*dmats1[30][19] + coeff0_31*dmats1[31][19] + coeff0_32*dmats1[32][19] + coeff0_33*dmats1[33][19] + coeff0_34*dmats1[34][19] + coeff0_35*dmats1[35][19] + coeff0_36*dmats1[36][19] + coeff0_37*dmats1[37][19] + coeff0_38*dmats1[38][19] + coeff0_39*dmats1[39][19] + coeff0_40*dmats1[40][19] + coeff0_41*dmats1[41][19] + coeff0_42*dmats1[42][19] + coeff0_43*dmats1[43][19] + coeff0_44*dmats1[44][19] + coeff0_45*dmats1[45][19] + coeff0_46*dmats1[46][19] + coeff0_47*dmats1[47][19] + coeff0_48*dmats1[48][19] + coeff0_49*dmats1[49][19] + coeff0_50*dmats1[50][19] + coeff0_51*dmats1[51][19] + coeff0_52*dmats1[52][19] + coeff0_53*dmats1[53][19] + coeff0_54*dmats1[54][19] + coeff0_55*dmats1[55][19];
          new_coeff0_20 = coeff0_0*dmats1[0][20] + coeff0_1*dmats1[1][20] + coeff0_2*dmats1[2][20] + coeff0_3*dmats1[3][20] + coeff0_4*dmats1[4][20] + coeff0_5*dmats1[5][20] + coeff0_6*dmats1[6][20] + coeff0_7*dmats1[7][20] + coeff0_8*dmats1[8][20] + coeff0_9*dmats1[9][20] + coeff0_10*dmats1[10][20] + coeff0_11*dmats1[11][20] + coeff0_12*dmats1[12][20] + coeff0_13*dmats1[13][20] + coeff0_14*dmats1[14][20] + coeff0_15*dmats1[15][20] + coeff0_16*dmats1[16][20] + coeff0_17*dmats1[17][20] + coeff0_18*dmats1[18][20] + coeff0_19*dmats1[19][20] + coeff0_20*dmats1[20][20] + coeff0_21*dmats1[21][20] + coeff0_22*dmats1[22][20] + coeff0_23*dmats1[23][20] + coeff0_24*dmats1[24][20] + coeff0_25*dmats1[25][20] + coeff0_26*dmats1[26][20] + coeff0_27*dmats1[27][20] + coeff0_28*dmats1[28][20] + coeff0_29*dmats1[29][20] + coeff0_30*dmats1[30][20] + coeff0_31*dmats1[31][20] + coeff0_32*dmats1[32][20] + coeff0_33*dmats1[33][20] + coeff0_34*dmats1[34][20] + coeff0_35*dmats1[35][20] + coeff0_36*dmats1[36][20] + coeff0_37*dmats1[37][20] + coeff0_38*dmats1[38][20] + coeff0_39*dmats1[39][20] + coeff0_40*dmats1[40][20] + coeff0_41*dmats1[41][20] + coeff0_42*dmats1[42][20] + coeff0_43*dmats1[43][20] + coeff0_44*dmats1[44][20] + coeff0_45*dmats1[45][20] + coeff0_46*dmats1[46][20] + coeff0_47*dmats1[47][20] + coeff0_48*dmats1[48][20] + coeff0_49*dmats1[49][20] + coeff0_50*dmats1[50][20] + coeff0_51*dmats1[51][20] + coeff0_52*dmats1[52][20] + coeff0_53*dmats1[53][20] + coeff0_54*dmats1[54][20] + coeff0_55*dmats1[55][20];
          new_coeff0_21 = coeff0_0*dmats1[0][21] + coeff0_1*dmats1[1][21] + coeff0_2*dmats1[2][21] + coeff0_3*dmats1[3][21] + coeff0_4*dmats1[4][21] + coeff0_5*dmats1[5][21] + coeff0_6*dmats1[6][21] + coeff0_7*dmats1[7][21] + coeff0_8*dmats1[8][21] + coeff0_9*dmats1[9][21] + coeff0_10*dmats1[10][21] + coeff0_11*dmats1[11][21] + coeff0_12*dmats1[12][21] + coeff0_13*dmats1[13][21] + coeff0_14*dmats1[14][21] + coeff0_15*dmats1[15][21] + coeff0_16*dmats1[16][21] + coeff0_17*dmats1[17][21] + coeff0_18*dmats1[18][21] + coeff0_19*dmats1[19][21] + coeff0_20*dmats1[20][21] + coeff0_21*dmats1[21][21] + coeff0_22*dmats1[22][21] + coeff0_23*dmats1[23][21] + coeff0_24*dmats1[24][21] + coeff0_25*dmats1[25][21] + coeff0_26*dmats1[26][21] + coeff0_27*dmats1[27][21] + coeff0_28*dmats1[28][21] + coeff0_29*dmats1[29][21] + coeff0_30*dmats1[30][21] + coeff0_31*dmats1[31][21] + coeff0_32*dmats1[32][21] + coeff0_33*dmats1[33][21] + coeff0_34*dmats1[34][21] + coeff0_35*dmats1[35][21] + coeff0_36*dmats1[36][21] + coeff0_37*dmats1[37][21] + coeff0_38*dmats1[38][21] + coeff0_39*dmats1[39][21] + coeff0_40*dmats1[40][21] + coeff0_41*dmats1[41][21] + coeff0_42*dmats1[42][21] + coeff0_43*dmats1[43][21] + coeff0_44*dmats1[44][21] + coeff0_45*dmats1[45][21] + coeff0_46*dmats1[46][21] + coeff0_47*dmats1[47][21] + coeff0_48*dmats1[48][21] + coeff0_49*dmats1[49][21] + coeff0_50*dmats1[50][21] + coeff0_51*dmats1[51][21] + coeff0_52*dmats1[52][21] + coeff0_53*dmats1[53][21] + coeff0_54*dmats1[54][21] + coeff0_55*dmats1[55][21];
          new_coeff0_22 = coeff0_0*dmats1[0][22] + coeff0_1*dmats1[1][22] + coeff0_2*dmats1[2][22] + coeff0_3*dmats1[3][22] + coeff0_4*dmats1[4][22] + coeff0_5*dmats1[5][22] + coeff0_6*dmats1[6][22] + coeff0_7*dmats1[7][22] + coeff0_8*dmats1[8][22] + coeff0_9*dmats1[9][22] + coeff0_10*dmats1[10][22] + coeff0_11*dmats1[11][22] + coeff0_12*dmats1[12][22] + coeff0_13*dmats1[13][22] + coeff0_14*dmats1[14][22] + coeff0_15*dmats1[15][22] + coeff0_16*dmats1[16][22] + coeff0_17*dmats1[17][22] + coeff0_18*dmats1[18][22] + coeff0_19*dmats1[19][22] + coeff0_20*dmats1[20][22] + coeff0_21*dmats1[21][22] + coeff0_22*dmats1[22][22] + coeff0_23*dmats1[23][22] + coeff0_24*dmats1[24][22] + coeff0_25*dmats1[25][22] + coeff0_26*dmats1[26][22] + coeff0_27*dmats1[27][22] + coeff0_28*dmats1[28][22] + coeff0_29*dmats1[29][22] + coeff0_30*dmats1[30][22] + coeff0_31*dmats1[31][22] + coeff0_32*dmats1[32][22] + coeff0_33*dmats1[33][22] + coeff0_34*dmats1[34][22] + coeff0_35*dmats1[35][22] + coeff0_36*dmats1[36][22] + coeff0_37*dmats1[37][22] + coeff0_38*dmats1[38][22] + coeff0_39*dmats1[39][22] + coeff0_40*dmats1[40][22] + coeff0_41*dmats1[41][22] + coeff0_42*dmats1[42][22] + coeff0_43*dmats1[43][22] + coeff0_44*dmats1[44][22] + coeff0_45*dmats1[45][22] + coeff0_46*dmats1[46][22] + coeff0_47*dmats1[47][22] + coeff0_48*dmats1[48][22] + coeff0_49*dmats1[49][22] + coeff0_50*dmats1[50][22] + coeff0_51*dmats1[51][22] + coeff0_52*dmats1[52][22] + coeff0_53*dmats1[53][22] + coeff0_54*dmats1[54][22] + coeff0_55*dmats1[55][22];
          new_coeff0_23 = coeff0_0*dmats1[0][23] + coeff0_1*dmats1[1][23] + coeff0_2*dmats1[2][23] + coeff0_3*dmats1[3][23] + coeff0_4*dmats1[4][23] + coeff0_5*dmats1[5][23] + coeff0_6*dmats1[6][23] + coeff0_7*dmats1[7][23] + coeff0_8*dmats1[8][23] + coeff0_9*dmats1[9][23] + coeff0_10*dmats1[10][23] + coeff0_11*dmats1[11][23] + coeff0_12*dmats1[12][23] + coeff0_13*dmats1[13][23] + coeff0_14*dmats1[14][23] + coeff0_15*dmats1[15][23] + coeff0_16*dmats1[16][23] + coeff0_17*dmats1[17][23] + coeff0_18*dmats1[18][23] + coeff0_19*dmats1[19][23] + coeff0_20*dmats1[20][23] + coeff0_21*dmats1[21][23] + coeff0_22*dmats1[22][23] + coeff0_23*dmats1[23][23] + coeff0_24*dmats1[24][23] + coeff0_25*dmats1[25][23] + coeff0_26*dmats1[26][23] + coeff0_27*dmats1[27][23] + coeff0_28*dmats1[28][23] + coeff0_29*dmats1[29][23] + coeff0_30*dmats1[30][23] + coeff0_31*dmats1[31][23] + coeff0_32*dmats1[32][23] + coeff0_33*dmats1[33][23] + coeff0_34*dmats1[34][23] + coeff0_35*dmats1[35][23] + coeff0_36*dmats1[36][23] + coeff0_37*dmats1[37][23] + coeff0_38*dmats1[38][23] + coeff0_39*dmats1[39][23] + coeff0_40*dmats1[40][23] + coeff0_41*dmats1[41][23] + coeff0_42*dmats1[42][23] + coeff0_43*dmats1[43][23] + coeff0_44*dmats1[44][23] + coeff0_45*dmats1[45][23] + coeff0_46*dmats1[46][23] + coeff0_47*dmats1[47][23] + coeff0_48*dmats1[48][23] + coeff0_49*dmats1[49][23] + coeff0_50*dmats1[50][23] + coeff0_51*dmats1[51][23] + coeff0_52*dmats1[52][23] + coeff0_53*dmats1[53][23] + coeff0_54*dmats1[54][23] + coeff0_55*dmats1[55][23];
          new_coeff0_24 = coeff0_0*dmats1[0][24] + coeff0_1*dmats1[1][24] + coeff0_2*dmats1[2][24] + coeff0_3*dmats1[3][24] + coeff0_4*dmats1[4][24] + coeff0_5*dmats1[5][24] + coeff0_6*dmats1[6][24] + coeff0_7*dmats1[7][24] + coeff0_8*dmats1[8][24] + coeff0_9*dmats1[9][24] + coeff0_10*dmats1[10][24] + coeff0_11*dmats1[11][24] + coeff0_12*dmats1[12][24] + coeff0_13*dmats1[13][24] + coeff0_14*dmats1[14][24] + coeff0_15*dmats1[15][24] + coeff0_16*dmats1[16][24] + coeff0_17*dmats1[17][24] + coeff0_18*dmats1[18][24] + coeff0_19*dmats1[19][24] + coeff0_20*dmats1[20][24] + coeff0_21*dmats1[21][24] + coeff0_22*dmats1[22][24] + coeff0_23*dmats1[23][24] + coeff0_24*dmats1[24][24] + coeff0_25*dmats1[25][24] + coeff0_26*dmats1[26][24] + coeff0_27*dmats1[27][24] + coeff0_28*dmats1[28][24] + coeff0_29*dmats1[29][24] + coeff0_30*dmats1[30][24] + coeff0_31*dmats1[31][24] + coeff0_32*dmats1[32][24] + coeff0_33*dmats1[33][24] + coeff0_34*dmats1[34][24] + coeff0_35*dmats1[35][24] + coeff0_36*dmats1[36][24] + coeff0_37*dmats1[37][24] + coeff0_38*dmats1[38][24] + coeff0_39*dmats1[39][24] + coeff0_40*dmats1[40][24] + coeff0_41*dmats1[41][24] + coeff0_42*dmats1[42][24] + coeff0_43*dmats1[43][24] + coeff0_44*dmats1[44][24] + coeff0_45*dmats1[45][24] + coeff0_46*dmats1[46][24] + coeff0_47*dmats1[47][24] + coeff0_48*dmats1[48][24] + coeff0_49*dmats1[49][24] + coeff0_50*dmats1[50][24] + coeff0_51*dmats1[51][24] + coeff0_52*dmats1[52][24] + coeff0_53*dmats1[53][24] + coeff0_54*dmats1[54][24] + coeff0_55*dmats1[55][24];
          new_coeff0_25 = coeff0_0*dmats1[0][25] + coeff0_1*dmats1[1][25] + coeff0_2*dmats1[2][25] + coeff0_3*dmats1[3][25] + coeff0_4*dmats1[4][25] + coeff0_5*dmats1[5][25] + coeff0_6*dmats1[6][25] + coeff0_7*dmats1[7][25] + coeff0_8*dmats1[8][25] + coeff0_9*dmats1[9][25] + coeff0_10*dmats1[10][25] + coeff0_11*dmats1[11][25] + coeff0_12*dmats1[12][25] + coeff0_13*dmats1[13][25] + coeff0_14*dmats1[14][25] + coeff0_15*dmats1[15][25] + coeff0_16*dmats1[16][25] + coeff0_17*dmats1[17][25] + coeff0_18*dmats1[18][25] + coeff0_19*dmats1[19][25] + coeff0_20*dmats1[20][25] + coeff0_21*dmats1[21][25] + coeff0_22*dmats1[22][25] + coeff0_23*dmats1[23][25] + coeff0_24*dmats1[24][25] + coeff0_25*dmats1[25][25] + coeff0_26*dmats1[26][25] + coeff0_27*dmats1[27][25] + coeff0_28*dmats1[28][25] + coeff0_29*dmats1[29][25] + coeff0_30*dmats1[30][25] + coeff0_31*dmats1[31][25] + coeff0_32*dmats1[32][25] + coeff0_33*dmats1[33][25] + coeff0_34*dmats1[34][25] + coeff0_35*dmats1[35][25] + coeff0_36*dmats1[36][25] + coeff0_37*dmats1[37][25] + coeff0_38*dmats1[38][25] + coeff0_39*dmats1[39][25] + coeff0_40*dmats1[40][25] + coeff0_41*dmats1[41][25] + coeff0_42*dmats1[42][25] + coeff0_43*dmats1[43][25] + coeff0_44*dmats1[44][25] + coeff0_45*dmats1[45][25] + coeff0_46*dmats1[46][25] + coeff0_47*dmats1[47][25] + coeff0_48*dmats1[48][25] + coeff0_49*dmats1[49][25] + coeff0_50*dmats1[50][25] + coeff0_51*dmats1[51][25] + coeff0_52*dmats1[52][25] + coeff0_53*dmats1[53][25] + coeff0_54*dmats1[54][25] + coeff0_55*dmats1[55][25];
          new_coeff0_26 = coeff0_0*dmats1[0][26] + coeff0_1*dmats1[1][26] + coeff0_2*dmats1[2][26] + coeff0_3*dmats1[3][26] + coeff0_4*dmats1[4][26] + coeff0_5*dmats1[5][26] + coeff0_6*dmats1[6][26] + coeff0_7*dmats1[7][26] + coeff0_8*dmats1[8][26] + coeff0_9*dmats1[9][26] + coeff0_10*dmats1[10][26] + coeff0_11*dmats1[11][26] + coeff0_12*dmats1[12][26] + coeff0_13*dmats1[13][26] + coeff0_14*dmats1[14][26] + coeff0_15*dmats1[15][26] + coeff0_16*dmats1[16][26] + coeff0_17*dmats1[17][26] + coeff0_18*dmats1[18][26] + coeff0_19*dmats1[19][26] + coeff0_20*dmats1[20][26] + coeff0_21*dmats1[21][26] + coeff0_22*dmats1[22][26] + coeff0_23*dmats1[23][26] + coeff0_24*dmats1[24][26] + coeff0_25*dmats1[25][26] + coeff0_26*dmats1[26][26] + coeff0_27*dmats1[27][26] + coeff0_28*dmats1[28][26] + coeff0_29*dmats1[29][26] + coeff0_30*dmats1[30][26] + coeff0_31*dmats1[31][26] + coeff0_32*dmats1[32][26] + coeff0_33*dmats1[33][26] + coeff0_34*dmats1[34][26] + coeff0_35*dmats1[35][26] + coeff0_36*dmats1[36][26] + coeff0_37*dmats1[37][26] + coeff0_38*dmats1[38][26] + coeff0_39*dmats1[39][26] + coeff0_40*dmats1[40][26] + coeff0_41*dmats1[41][26] + coeff0_42*dmats1[42][26] + coeff0_43*dmats1[43][26] + coeff0_44*dmats1[44][26] + coeff0_45*dmats1[45][26] + coeff0_46*dmats1[46][26] + coeff0_47*dmats1[47][26] + coeff0_48*dmats1[48][26] + coeff0_49*dmats1[49][26] + coeff0_50*dmats1[50][26] + coeff0_51*dmats1[51][26] + coeff0_52*dmats1[52][26] + coeff0_53*dmats1[53][26] + coeff0_54*dmats1[54][26] + coeff0_55*dmats1[55][26];
          new_coeff0_27 = coeff0_0*dmats1[0][27] + coeff0_1*dmats1[1][27] + coeff0_2*dmats1[2][27] + coeff0_3*dmats1[3][27] + coeff0_4*dmats1[4][27] + coeff0_5*dmats1[5][27] + coeff0_6*dmats1[6][27] + coeff0_7*dmats1[7][27] + coeff0_8*dmats1[8][27] + coeff0_9*dmats1[9][27] + coeff0_10*dmats1[10][27] + coeff0_11*dmats1[11][27] + coeff0_12*dmats1[12][27] + coeff0_13*dmats1[13][27] + coeff0_14*dmats1[14][27] + coeff0_15*dmats1[15][27] + coeff0_16*dmats1[16][27] + coeff0_17*dmats1[17][27] + coeff0_18*dmats1[18][27] + coeff0_19*dmats1[19][27] + coeff0_20*dmats1[20][27] + coeff0_21*dmats1[21][27] + coeff0_22*dmats1[22][27] + coeff0_23*dmats1[23][27] + coeff0_24*dmats1[24][27] + coeff0_25*dmats1[25][27] + coeff0_26*dmats1[26][27] + coeff0_27*dmats1[27][27] + coeff0_28*dmats1[28][27] + coeff0_29*dmats1[29][27] + coeff0_30*dmats1[30][27] + coeff0_31*dmats1[31][27] + coeff0_32*dmats1[32][27] + coeff0_33*dmats1[33][27] + coeff0_34*dmats1[34][27] + coeff0_35*dmats1[35][27] + coeff0_36*dmats1[36][27] + coeff0_37*dmats1[37][27] + coeff0_38*dmats1[38][27] + coeff0_39*dmats1[39][27] + coeff0_40*dmats1[40][27] + coeff0_41*dmats1[41][27] + coeff0_42*dmats1[42][27] + coeff0_43*dmats1[43][27] + coeff0_44*dmats1[44][27] + coeff0_45*dmats1[45][27] + coeff0_46*dmats1[46][27] + coeff0_47*dmats1[47][27] + coeff0_48*dmats1[48][27] + coeff0_49*dmats1[49][27] + coeff0_50*dmats1[50][27] + coeff0_51*dmats1[51][27] + coeff0_52*dmats1[52][27] + coeff0_53*dmats1[53][27] + coeff0_54*dmats1[54][27] + coeff0_55*dmats1[55][27];
          new_coeff0_28 = coeff0_0*dmats1[0][28] + coeff0_1*dmats1[1][28] + coeff0_2*dmats1[2][28] + coeff0_3*dmats1[3][28] + coeff0_4*dmats1[4][28] + coeff0_5*dmats1[5][28] + coeff0_6*dmats1[6][28] + coeff0_7*dmats1[7][28] + coeff0_8*dmats1[8][28] + coeff0_9*dmats1[9][28] + coeff0_10*dmats1[10][28] + coeff0_11*dmats1[11][28] + coeff0_12*dmats1[12][28] + coeff0_13*dmats1[13][28] + coeff0_14*dmats1[14][28] + coeff0_15*dmats1[15][28] + coeff0_16*dmats1[16][28] + coeff0_17*dmats1[17][28] + coeff0_18*dmats1[18][28] + coeff0_19*dmats1[19][28] + coeff0_20*dmats1[20][28] + coeff0_21*dmats1[21][28] + coeff0_22*dmats1[22][28] + coeff0_23*dmats1[23][28] + coeff0_24*dmats1[24][28] + coeff0_25*dmats1[25][28] + coeff0_26*dmats1[26][28] + coeff0_27*dmats1[27][28] + coeff0_28*dmats1[28][28] + coeff0_29*dmats1[29][28] + coeff0_30*dmats1[30][28] + coeff0_31*dmats1[31][28] + coeff0_32*dmats1[32][28] + coeff0_33*dmats1[33][28] + coeff0_34*dmats1[34][28] + coeff0_35*dmats1[35][28] + coeff0_36*dmats1[36][28] + coeff0_37*dmats1[37][28] + coeff0_38*dmats1[38][28] + coeff0_39*dmats1[39][28] + coeff0_40*dmats1[40][28] + coeff0_41*dmats1[41][28] + coeff0_42*dmats1[42][28] + coeff0_43*dmats1[43][28] + coeff0_44*dmats1[44][28] + coeff0_45*dmats1[45][28] + coeff0_46*dmats1[46][28] + coeff0_47*dmats1[47][28] + coeff0_48*dmats1[48][28] + coeff0_49*dmats1[49][28] + coeff0_50*dmats1[50][28] + coeff0_51*dmats1[51][28] + coeff0_52*dmats1[52][28] + coeff0_53*dmats1[53][28] + coeff0_54*dmats1[54][28] + coeff0_55*dmats1[55][28];
          new_coeff0_29 = coeff0_0*dmats1[0][29] + coeff0_1*dmats1[1][29] + coeff0_2*dmats1[2][29] + coeff0_3*dmats1[3][29] + coeff0_4*dmats1[4][29] + coeff0_5*dmats1[5][29] + coeff0_6*dmats1[6][29] + coeff0_7*dmats1[7][29] + coeff0_8*dmats1[8][29] + coeff0_9*dmats1[9][29] + coeff0_10*dmats1[10][29] + coeff0_11*dmats1[11][29] + coeff0_12*dmats1[12][29] + coeff0_13*dmats1[13][29] + coeff0_14*dmats1[14][29] + coeff0_15*dmats1[15][29] + coeff0_16*dmats1[16][29] + coeff0_17*dmats1[17][29] + coeff0_18*dmats1[18][29] + coeff0_19*dmats1[19][29] + coeff0_20*dmats1[20][29] + coeff0_21*dmats1[21][29] + coeff0_22*dmats1[22][29] + coeff0_23*dmats1[23][29] + coeff0_24*dmats1[24][29] + coeff0_25*dmats1[25][29] + coeff0_26*dmats1[26][29] + coeff0_27*dmats1[27][29] + coeff0_28*dmats1[28][29] + coeff0_29*dmats1[29][29] + coeff0_30*dmats1[30][29] + coeff0_31*dmats1[31][29] + coeff0_32*dmats1[32][29] + coeff0_33*dmats1[33][29] + coeff0_34*dmats1[34][29] + coeff0_35*dmats1[35][29] + coeff0_36*dmats1[36][29] + coeff0_37*dmats1[37][29] + coeff0_38*dmats1[38][29] + coeff0_39*dmats1[39][29] + coeff0_40*dmats1[40][29] + coeff0_41*dmats1[41][29] + coeff0_42*dmats1[42][29] + coeff0_43*dmats1[43][29] + coeff0_44*dmats1[44][29] + coeff0_45*dmats1[45][29] + coeff0_46*dmats1[46][29] + coeff0_47*dmats1[47][29] + coeff0_48*dmats1[48][29] + coeff0_49*dmats1[49][29] + coeff0_50*dmats1[50][29] + coeff0_51*dmats1[51][29] + coeff0_52*dmats1[52][29] + coeff0_53*dmats1[53][29] + coeff0_54*dmats1[54][29] + coeff0_55*dmats1[55][29];
          new_coeff0_30 = coeff0_0*dmats1[0][30] + coeff0_1*dmats1[1][30] + coeff0_2*dmats1[2][30] + coeff0_3*dmats1[3][30] + coeff0_4*dmats1[4][30] + coeff0_5*dmats1[5][30] + coeff0_6*dmats1[6][30] + coeff0_7*dmats1[7][30] + coeff0_8*dmats1[8][30] + coeff0_9*dmats1[9][30] + coeff0_10*dmats1[10][30] + coeff0_11*dmats1[11][30] + coeff0_12*dmats1[12][30] + coeff0_13*dmats1[13][30] + coeff0_14*dmats1[14][30] + coeff0_15*dmats1[15][30] + coeff0_16*dmats1[16][30] + coeff0_17*dmats1[17][30] + coeff0_18*dmats1[18][30] + coeff0_19*dmats1[19][30] + coeff0_20*dmats1[20][30] + coeff0_21*dmats1[21][30] + coeff0_22*dmats1[22][30] + coeff0_23*dmats1[23][30] + coeff0_24*dmats1[24][30] + coeff0_25*dmats1[25][30] + coeff0_26*dmats1[26][30] + coeff0_27*dmats1[27][30] + coeff0_28*dmats1[28][30] + coeff0_29*dmats1[29][30] + coeff0_30*dmats1[30][30] + coeff0_31*dmats1[31][30] + coeff0_32*dmats1[32][30] + coeff0_33*dmats1[33][30] + coeff0_34*dmats1[34][30] + coeff0_35*dmats1[35][30] + coeff0_36*dmats1[36][30] + coeff0_37*dmats1[37][30] + coeff0_38*dmats1[38][30] + coeff0_39*dmats1[39][30] + coeff0_40*dmats1[40][30] + coeff0_41*dmats1[41][30] + coeff0_42*dmats1[42][30] + coeff0_43*dmats1[43][30] + coeff0_44*dmats1[44][30] + coeff0_45*dmats1[45][30] + coeff0_46*dmats1[46][30] + coeff0_47*dmats1[47][30] + coeff0_48*dmats1[48][30] + coeff0_49*dmats1[49][30] + coeff0_50*dmats1[50][30] + coeff0_51*dmats1[51][30] + coeff0_52*dmats1[52][30] + coeff0_53*dmats1[53][30] + coeff0_54*dmats1[54][30] + coeff0_55*dmats1[55][30];
          new_coeff0_31 = coeff0_0*dmats1[0][31] + coeff0_1*dmats1[1][31] + coeff0_2*dmats1[2][31] + coeff0_3*dmats1[3][31] + coeff0_4*dmats1[4][31] + coeff0_5*dmats1[5][31] + coeff0_6*dmats1[6][31] + coeff0_7*dmats1[7][31] + coeff0_8*dmats1[8][31] + coeff0_9*dmats1[9][31] + coeff0_10*dmats1[10][31] + coeff0_11*dmats1[11][31] + coeff0_12*dmats1[12][31] + coeff0_13*dmats1[13][31] + coeff0_14*dmats1[14][31] + coeff0_15*dmats1[15][31] + coeff0_16*dmats1[16][31] + coeff0_17*dmats1[17][31] + coeff0_18*dmats1[18][31] + coeff0_19*dmats1[19][31] + coeff0_20*dmats1[20][31] + coeff0_21*dmats1[21][31] + coeff0_22*dmats1[22][31] + coeff0_23*dmats1[23][31] + coeff0_24*dmats1[24][31] + coeff0_25*dmats1[25][31] + coeff0_26*dmats1[26][31] + coeff0_27*dmats1[27][31] + coeff0_28*dmats1[28][31] + coeff0_29*dmats1[29][31] + coeff0_30*dmats1[30][31] + coeff0_31*dmats1[31][31] + coeff0_32*dmats1[32][31] + coeff0_33*dmats1[33][31] + coeff0_34*dmats1[34][31] + coeff0_35*dmats1[35][31] + coeff0_36*dmats1[36][31] + coeff0_37*dmats1[37][31] + coeff0_38*dmats1[38][31] + coeff0_39*dmats1[39][31] + coeff0_40*dmats1[40][31] + coeff0_41*dmats1[41][31] + coeff0_42*dmats1[42][31] + coeff0_43*dmats1[43][31] + coeff0_44*dmats1[44][31] + coeff0_45*dmats1[45][31] + coeff0_46*dmats1[46][31] + coeff0_47*dmats1[47][31] + coeff0_48*dmats1[48][31] + coeff0_49*dmats1[49][31] + coeff0_50*dmats1[50][31] + coeff0_51*dmats1[51][31] + coeff0_52*dmats1[52][31] + coeff0_53*dmats1[53][31] + coeff0_54*dmats1[54][31] + coeff0_55*dmats1[55][31];
          new_coeff0_32 = coeff0_0*dmats1[0][32] + coeff0_1*dmats1[1][32] + coeff0_2*dmats1[2][32] + coeff0_3*dmats1[3][32] + coeff0_4*dmats1[4][32] + coeff0_5*dmats1[5][32] + coeff0_6*dmats1[6][32] + coeff0_7*dmats1[7][32] + coeff0_8*dmats1[8][32] + coeff0_9*dmats1[9][32] + coeff0_10*dmats1[10][32] + coeff0_11*dmats1[11][32] + coeff0_12*dmats1[12][32] + coeff0_13*dmats1[13][32] + coeff0_14*dmats1[14][32] + coeff0_15*dmats1[15][32] + coeff0_16*dmats1[16][32] + coeff0_17*dmats1[17][32] + coeff0_18*dmats1[18][32] + coeff0_19*dmats1[19][32] + coeff0_20*dmats1[20][32] + coeff0_21*dmats1[21][32] + coeff0_22*dmats1[22][32] + coeff0_23*dmats1[23][32] + coeff0_24*dmats1[24][32] + coeff0_25*dmats1[25][32] + coeff0_26*dmats1[26][32] + coeff0_27*dmats1[27][32] + coeff0_28*dmats1[28][32] + coeff0_29*dmats1[29][32] + coeff0_30*dmats1[30][32] + coeff0_31*dmats1[31][32] + coeff0_32*dmats1[32][32] + coeff0_33*dmats1[33][32] + coeff0_34*dmats1[34][32] + coeff0_35*dmats1[35][32] + coeff0_36*dmats1[36][32] + coeff0_37*dmats1[37][32] + coeff0_38*dmats1[38][32] + coeff0_39*dmats1[39][32] + coeff0_40*dmats1[40][32] + coeff0_41*dmats1[41][32] + coeff0_42*dmats1[42][32] + coeff0_43*dmats1[43][32] + coeff0_44*dmats1[44][32] + coeff0_45*dmats1[45][32] + coeff0_46*dmats1[46][32] + coeff0_47*dmats1[47][32] + coeff0_48*dmats1[48][32] + coeff0_49*dmats1[49][32] + coeff0_50*dmats1[50][32] + coeff0_51*dmats1[51][32] + coeff0_52*dmats1[52][32] + coeff0_53*dmats1[53][32] + coeff0_54*dmats1[54][32] + coeff0_55*dmats1[55][32];
          new_coeff0_33 = coeff0_0*dmats1[0][33] + coeff0_1*dmats1[1][33] + coeff0_2*dmats1[2][33] + coeff0_3*dmats1[3][33] + coeff0_4*dmats1[4][33] + coeff0_5*dmats1[5][33] + coeff0_6*dmats1[6][33] + coeff0_7*dmats1[7][33] + coeff0_8*dmats1[8][33] + coeff0_9*dmats1[9][33] + coeff0_10*dmats1[10][33] + coeff0_11*dmats1[11][33] + coeff0_12*dmats1[12][33] + coeff0_13*dmats1[13][33] + coeff0_14*dmats1[14][33] + coeff0_15*dmats1[15][33] + coeff0_16*dmats1[16][33] + coeff0_17*dmats1[17][33] + coeff0_18*dmats1[18][33] + coeff0_19*dmats1[19][33] + coeff0_20*dmats1[20][33] + coeff0_21*dmats1[21][33] + coeff0_22*dmats1[22][33] + coeff0_23*dmats1[23][33] + coeff0_24*dmats1[24][33] + coeff0_25*dmats1[25][33] + coeff0_26*dmats1[26][33] + coeff0_27*dmats1[27][33] + coeff0_28*dmats1[28][33] + coeff0_29*dmats1[29][33] + coeff0_30*dmats1[30][33] + coeff0_31*dmats1[31][33] + coeff0_32*dmats1[32][33] + coeff0_33*dmats1[33][33] + coeff0_34*dmats1[34][33] + coeff0_35*dmats1[35][33] + coeff0_36*dmats1[36][33] + coeff0_37*dmats1[37][33] + coeff0_38*dmats1[38][33] + coeff0_39*dmats1[39][33] + coeff0_40*dmats1[40][33] + coeff0_41*dmats1[41][33] + coeff0_42*dmats1[42][33] + coeff0_43*dmats1[43][33] + coeff0_44*dmats1[44][33] + coeff0_45*dmats1[45][33] + coeff0_46*dmats1[46][33] + coeff0_47*dmats1[47][33] + coeff0_48*dmats1[48][33] + coeff0_49*dmats1[49][33] + coeff0_50*dmats1[50][33] + coeff0_51*dmats1[51][33] + coeff0_52*dmats1[52][33] + coeff0_53*dmats1[53][33] + coeff0_54*dmats1[54][33] + coeff0_55*dmats1[55][33];
          new_coeff0_34 = coeff0_0*dmats1[0][34] + coeff0_1*dmats1[1][34] + coeff0_2*dmats1[2][34] + coeff0_3*dmats1[3][34] + coeff0_4*dmats1[4][34] + coeff0_5*dmats1[5][34] + coeff0_6*dmats1[6][34] + coeff0_7*dmats1[7][34] + coeff0_8*dmats1[8][34] + coeff0_9*dmats1[9][34] + coeff0_10*dmats1[10][34] + coeff0_11*dmats1[11][34] + coeff0_12*dmats1[12][34] + coeff0_13*dmats1[13][34] + coeff0_14*dmats1[14][34] + coeff0_15*dmats1[15][34] + coeff0_16*dmats1[16][34] + coeff0_17*dmats1[17][34] + coeff0_18*dmats1[18][34] + coeff0_19*dmats1[19][34] + coeff0_20*dmats1[20][34] + coeff0_21*dmats1[21][34] + coeff0_22*dmats1[22][34] + coeff0_23*dmats1[23][34] + coeff0_24*dmats1[24][34] + coeff0_25*dmats1[25][34] + coeff0_26*dmats1[26][34] + coeff0_27*dmats1[27][34] + coeff0_28*dmats1[28][34] + coeff0_29*dmats1[29][34] + coeff0_30*dmats1[30][34] + coeff0_31*dmats1[31][34] + coeff0_32*dmats1[32][34] + coeff0_33*dmats1[33][34] + coeff0_34*dmats1[34][34] + coeff0_35*dmats1[35][34] + coeff0_36*dmats1[36][34] + coeff0_37*dmats1[37][34] + coeff0_38*dmats1[38][34] + coeff0_39*dmats1[39][34] + coeff0_40*dmats1[40][34] + coeff0_41*dmats1[41][34] + coeff0_42*dmats1[42][34] + coeff0_43*dmats1[43][34] + coeff0_44*dmats1[44][34] + coeff0_45*dmats1[45][34] + coeff0_46*dmats1[46][34] + coeff0_47*dmats1[47][34] + coeff0_48*dmats1[48][34] + coeff0_49*dmats1[49][34] + coeff0_50*dmats1[50][34] + coeff0_51*dmats1[51][34] + coeff0_52*dmats1[52][34] + coeff0_53*dmats1[53][34] + coeff0_54*dmats1[54][34] + coeff0_55*dmats1[55][34];
          new_coeff0_35 = coeff0_0*dmats1[0][35] + coeff0_1*dmats1[1][35] + coeff0_2*dmats1[2][35] + coeff0_3*dmats1[3][35] + coeff0_4*dmats1[4][35] + coeff0_5*dmats1[5][35] + coeff0_6*dmats1[6][35] + coeff0_7*dmats1[7][35] + coeff0_8*dmats1[8][35] + coeff0_9*dmats1[9][35] + coeff0_10*dmats1[10][35] + coeff0_11*dmats1[11][35] + coeff0_12*dmats1[12][35] + coeff0_13*dmats1[13][35] + coeff0_14*dmats1[14][35] + coeff0_15*dmats1[15][35] + coeff0_16*dmats1[16][35] + coeff0_17*dmats1[17][35] + coeff0_18*dmats1[18][35] + coeff0_19*dmats1[19][35] + coeff0_20*dmats1[20][35] + coeff0_21*dmats1[21][35] + coeff0_22*dmats1[22][35] + coeff0_23*dmats1[23][35] + coeff0_24*dmats1[24][35] + coeff0_25*dmats1[25][35] + coeff0_26*dmats1[26][35] + coeff0_27*dmats1[27][35] + coeff0_28*dmats1[28][35] + coeff0_29*dmats1[29][35] + coeff0_30*dmats1[30][35] + coeff0_31*dmats1[31][35] + coeff0_32*dmats1[32][35] + coeff0_33*dmats1[33][35] + coeff0_34*dmats1[34][35] + coeff0_35*dmats1[35][35] + coeff0_36*dmats1[36][35] + coeff0_37*dmats1[37][35] + coeff0_38*dmats1[38][35] + coeff0_39*dmats1[39][35] + coeff0_40*dmats1[40][35] + coeff0_41*dmats1[41][35] + coeff0_42*dmats1[42][35] + coeff0_43*dmats1[43][35] + coeff0_44*dmats1[44][35] + coeff0_45*dmats1[45][35] + coeff0_46*dmats1[46][35] + coeff0_47*dmats1[47][35] + coeff0_48*dmats1[48][35] + coeff0_49*dmats1[49][35] + coeff0_50*dmats1[50][35] + coeff0_51*dmats1[51][35] + coeff0_52*dmats1[52][35] + coeff0_53*dmats1[53][35] + coeff0_54*dmats1[54][35] + coeff0_55*dmats1[55][35];
          new_coeff0_36 = coeff0_0*dmats1[0][36] + coeff0_1*dmats1[1][36] + coeff0_2*dmats1[2][36] + coeff0_3*dmats1[3][36] + coeff0_4*dmats1[4][36] + coeff0_5*dmats1[5][36] + coeff0_6*dmats1[6][36] + coeff0_7*dmats1[7][36] + coeff0_8*dmats1[8][36] + coeff0_9*dmats1[9][36] + coeff0_10*dmats1[10][36] + coeff0_11*dmats1[11][36] + coeff0_12*dmats1[12][36] + coeff0_13*dmats1[13][36] + coeff0_14*dmats1[14][36] + coeff0_15*dmats1[15][36] + coeff0_16*dmats1[16][36] + coeff0_17*dmats1[17][36] + coeff0_18*dmats1[18][36] + coeff0_19*dmats1[19][36] + coeff0_20*dmats1[20][36] + coeff0_21*dmats1[21][36] + coeff0_22*dmats1[22][36] + coeff0_23*dmats1[23][36] + coeff0_24*dmats1[24][36] + coeff0_25*dmats1[25][36] + coeff0_26*dmats1[26][36] + coeff0_27*dmats1[27][36] + coeff0_28*dmats1[28][36] + coeff0_29*dmats1[29][36] + coeff0_30*dmats1[30][36] + coeff0_31*dmats1[31][36] + coeff0_32*dmats1[32][36] + coeff0_33*dmats1[33][36] + coeff0_34*dmats1[34][36] + coeff0_35*dmats1[35][36] + coeff0_36*dmats1[36][36] + coeff0_37*dmats1[37][36] + coeff0_38*dmats1[38][36] + coeff0_39*dmats1[39][36] + coeff0_40*dmats1[40][36] + coeff0_41*dmats1[41][36] + coeff0_42*dmats1[42][36] + coeff0_43*dmats1[43][36] + coeff0_44*dmats1[44][36] + coeff0_45*dmats1[45][36] + coeff0_46*dmats1[46][36] + coeff0_47*dmats1[47][36] + coeff0_48*dmats1[48][36] + coeff0_49*dmats1[49][36] + coeff0_50*dmats1[50][36] + coeff0_51*dmats1[51][36] + coeff0_52*dmats1[52][36] + coeff0_53*dmats1[53][36] + coeff0_54*dmats1[54][36] + coeff0_55*dmats1[55][36];
          new_coeff0_37 = coeff0_0*dmats1[0][37] + coeff0_1*dmats1[1][37] + coeff0_2*dmats1[2][37] + coeff0_3*dmats1[3][37] + coeff0_4*dmats1[4][37] + coeff0_5*dmats1[5][37] + coeff0_6*dmats1[6][37] + coeff0_7*dmats1[7][37] + coeff0_8*dmats1[8][37] + coeff0_9*dmats1[9][37] + coeff0_10*dmats1[10][37] + coeff0_11*dmats1[11][37] + coeff0_12*dmats1[12][37] + coeff0_13*dmats1[13][37] + coeff0_14*dmats1[14][37] + coeff0_15*dmats1[15][37] + coeff0_16*dmats1[16][37] + coeff0_17*dmats1[17][37] + coeff0_18*dmats1[18][37] + coeff0_19*dmats1[19][37] + coeff0_20*dmats1[20][37] + coeff0_21*dmats1[21][37] + coeff0_22*dmats1[22][37] + coeff0_23*dmats1[23][37] + coeff0_24*dmats1[24][37] + coeff0_25*dmats1[25][37] + coeff0_26*dmats1[26][37] + coeff0_27*dmats1[27][37] + coeff0_28*dmats1[28][37] + coeff0_29*dmats1[29][37] + coeff0_30*dmats1[30][37] + coeff0_31*dmats1[31][37] + coeff0_32*dmats1[32][37] + coeff0_33*dmats1[33][37] + coeff0_34*dmats1[34][37] + coeff0_35*dmats1[35][37] + coeff0_36*dmats1[36][37] + coeff0_37*dmats1[37][37] + coeff0_38*dmats1[38][37] + coeff0_39*dmats1[39][37] + coeff0_40*dmats1[40][37] + coeff0_41*dmats1[41][37] + coeff0_42*dmats1[42][37] + coeff0_43*dmats1[43][37] + coeff0_44*dmats1[44][37] + coeff0_45*dmats1[45][37] + coeff0_46*dmats1[46][37] + coeff0_47*dmats1[47][37] + coeff0_48*dmats1[48][37] + coeff0_49*dmats1[49][37] + coeff0_50*dmats1[50][37] + coeff0_51*dmats1[51][37] + coeff0_52*dmats1[52][37] + coeff0_53*dmats1[53][37] + coeff0_54*dmats1[54][37] + coeff0_55*dmats1[55][37];
          new_coeff0_38 = coeff0_0*dmats1[0][38] + coeff0_1*dmats1[1][38] + coeff0_2*dmats1[2][38] + coeff0_3*dmats1[3][38] + coeff0_4*dmats1[4][38] + coeff0_5*dmats1[5][38] + coeff0_6*dmats1[6][38] + coeff0_7*dmats1[7][38] + coeff0_8*dmats1[8][38] + coeff0_9*dmats1[9][38] + coeff0_10*dmats1[10][38] + coeff0_11*dmats1[11][38] + coeff0_12*dmats1[12][38] + coeff0_13*dmats1[13][38] + coeff0_14*dmats1[14][38] + coeff0_15*dmats1[15][38] + coeff0_16*dmats1[16][38] + coeff0_17*dmats1[17][38] + coeff0_18*dmats1[18][38] + coeff0_19*dmats1[19][38] + coeff0_20*dmats1[20][38] + coeff0_21*dmats1[21][38] + coeff0_22*dmats1[22][38] + coeff0_23*dmats1[23][38] + coeff0_24*dmats1[24][38] + coeff0_25*dmats1[25][38] + coeff0_26*dmats1[26][38] + coeff0_27*dmats1[27][38] + coeff0_28*dmats1[28][38] + coeff0_29*dmats1[29][38] + coeff0_30*dmats1[30][38] + coeff0_31*dmats1[31][38] + coeff0_32*dmats1[32][38] + coeff0_33*dmats1[33][38] + coeff0_34*dmats1[34][38] + coeff0_35*dmats1[35][38] + coeff0_36*dmats1[36][38] + coeff0_37*dmats1[37][38] + coeff0_38*dmats1[38][38] + coeff0_39*dmats1[39][38] + coeff0_40*dmats1[40][38] + coeff0_41*dmats1[41][38] + coeff0_42*dmats1[42][38] + coeff0_43*dmats1[43][38] + coeff0_44*dmats1[44][38] + coeff0_45*dmats1[45][38] + coeff0_46*dmats1[46][38] + coeff0_47*dmats1[47][38] + coeff0_48*dmats1[48][38] + coeff0_49*dmats1[49][38] + coeff0_50*dmats1[50][38] + coeff0_51*dmats1[51][38] + coeff0_52*dmats1[52][38] + coeff0_53*dmats1[53][38] + coeff0_54*dmats1[54][38] + coeff0_55*dmats1[55][38];
          new_coeff0_39 = coeff0_0*dmats1[0][39] + coeff0_1*dmats1[1][39] + coeff0_2*dmats1[2][39] + coeff0_3*dmats1[3][39] + coeff0_4*dmats1[4][39] + coeff0_5*dmats1[5][39] + coeff0_6*dmats1[6][39] + coeff0_7*dmats1[7][39] + coeff0_8*dmats1[8][39] + coeff0_9*dmats1[9][39] + coeff0_10*dmats1[10][39] + coeff0_11*dmats1[11][39] + coeff0_12*dmats1[12][39] + coeff0_13*dmats1[13][39] + coeff0_14*dmats1[14][39] + coeff0_15*dmats1[15][39] + coeff0_16*dmats1[16][39] + coeff0_17*dmats1[17][39] + coeff0_18*dmats1[18][39] + coeff0_19*dmats1[19][39] + coeff0_20*dmats1[20][39] + coeff0_21*dmats1[21][39] + coeff0_22*dmats1[22][39] + coeff0_23*dmats1[23][39] + coeff0_24*dmats1[24][39] + coeff0_25*dmats1[25][39] + coeff0_26*dmats1[26][39] + coeff0_27*dmats1[27][39] + coeff0_28*dmats1[28][39] + coeff0_29*dmats1[29][39] + coeff0_30*dmats1[30][39] + coeff0_31*dmats1[31][39] + coeff0_32*dmats1[32][39] + coeff0_33*dmats1[33][39] + coeff0_34*dmats1[34][39] + coeff0_35*dmats1[35][39] + coeff0_36*dmats1[36][39] + coeff0_37*dmats1[37][39] + coeff0_38*dmats1[38][39] + coeff0_39*dmats1[39][39] + coeff0_40*dmats1[40][39] + coeff0_41*dmats1[41][39] + coeff0_42*dmats1[42][39] + coeff0_43*dmats1[43][39] + coeff0_44*dmats1[44][39] + coeff0_45*dmats1[45][39] + coeff0_46*dmats1[46][39] + coeff0_47*dmats1[47][39] + coeff0_48*dmats1[48][39] + coeff0_49*dmats1[49][39] + coeff0_50*dmats1[50][39] + coeff0_51*dmats1[51][39] + coeff0_52*dmats1[52][39] + coeff0_53*dmats1[53][39] + coeff0_54*dmats1[54][39] + coeff0_55*dmats1[55][39];
          new_coeff0_40 = coeff0_0*dmats1[0][40] + coeff0_1*dmats1[1][40] + coeff0_2*dmats1[2][40] + coeff0_3*dmats1[3][40] + coeff0_4*dmats1[4][40] + coeff0_5*dmats1[5][40] + coeff0_6*dmats1[6][40] + coeff0_7*dmats1[7][40] + coeff0_8*dmats1[8][40] + coeff0_9*dmats1[9][40] + coeff0_10*dmats1[10][40] + coeff0_11*dmats1[11][40] + coeff0_12*dmats1[12][40] + coeff0_13*dmats1[13][40] + coeff0_14*dmats1[14][40] + coeff0_15*dmats1[15][40] + coeff0_16*dmats1[16][40] + coeff0_17*dmats1[17][40] + coeff0_18*dmats1[18][40] + coeff0_19*dmats1[19][40] + coeff0_20*dmats1[20][40] + coeff0_21*dmats1[21][40] + coeff0_22*dmats1[22][40] + coeff0_23*dmats1[23][40] + coeff0_24*dmats1[24][40] + coeff0_25*dmats1[25][40] + coeff0_26*dmats1[26][40] + coeff0_27*dmats1[27][40] + coeff0_28*dmats1[28][40] + coeff0_29*dmats1[29][40] + coeff0_30*dmats1[30][40] + coeff0_31*dmats1[31][40] + coeff0_32*dmats1[32][40] + coeff0_33*dmats1[33][40] + coeff0_34*dmats1[34][40] + coeff0_35*dmats1[35][40] + coeff0_36*dmats1[36][40] + coeff0_37*dmats1[37][40] + coeff0_38*dmats1[38][40] + coeff0_39*dmats1[39][40] + coeff0_40*dmats1[40][40] + coeff0_41*dmats1[41][40] + coeff0_42*dmats1[42][40] + coeff0_43*dmats1[43][40] + coeff0_44*dmats1[44][40] + coeff0_45*dmats1[45][40] + coeff0_46*dmats1[46][40] + coeff0_47*dmats1[47][40] + coeff0_48*dmats1[48][40] + coeff0_49*dmats1[49][40] + coeff0_50*dmats1[50][40] + coeff0_51*dmats1[51][40] + coeff0_52*dmats1[52][40] + coeff0_53*dmats1[53][40] + coeff0_54*dmats1[54][40] + coeff0_55*dmats1[55][40];
          new_coeff0_41 = coeff0_0*dmats1[0][41] + coeff0_1*dmats1[1][41] + coeff0_2*dmats1[2][41] + coeff0_3*dmats1[3][41] + coeff0_4*dmats1[4][41] + coeff0_5*dmats1[5][41] + coeff0_6*dmats1[6][41] + coeff0_7*dmats1[7][41] + coeff0_8*dmats1[8][41] + coeff0_9*dmats1[9][41] + coeff0_10*dmats1[10][41] + coeff0_11*dmats1[11][41] + coeff0_12*dmats1[12][41] + coeff0_13*dmats1[13][41] + coeff0_14*dmats1[14][41] + coeff0_15*dmats1[15][41] + coeff0_16*dmats1[16][41] + coeff0_17*dmats1[17][41] + coeff0_18*dmats1[18][41] + coeff0_19*dmats1[19][41] + coeff0_20*dmats1[20][41] + coeff0_21*dmats1[21][41] + coeff0_22*dmats1[22][41] + coeff0_23*dmats1[23][41] + coeff0_24*dmats1[24][41] + coeff0_25*dmats1[25][41] + coeff0_26*dmats1[26][41] + coeff0_27*dmats1[27][41] + coeff0_28*dmats1[28][41] + coeff0_29*dmats1[29][41] + coeff0_30*dmats1[30][41] + coeff0_31*dmats1[31][41] + coeff0_32*dmats1[32][41] + coeff0_33*dmats1[33][41] + coeff0_34*dmats1[34][41] + coeff0_35*dmats1[35][41] + coeff0_36*dmats1[36][41] + coeff0_37*dmats1[37][41] + coeff0_38*dmats1[38][41] + coeff0_39*dmats1[39][41] + coeff0_40*dmats1[40][41] + coeff0_41*dmats1[41][41] + coeff0_42*dmats1[42][41] + coeff0_43*dmats1[43][41] + coeff0_44*dmats1[44][41] + coeff0_45*dmats1[45][41] + coeff0_46*dmats1[46][41] + coeff0_47*dmats1[47][41] + coeff0_48*dmats1[48][41] + coeff0_49*dmats1[49][41] + coeff0_50*dmats1[50][41] + coeff0_51*dmats1[51][41] + coeff0_52*dmats1[52][41] + coeff0_53*dmats1[53][41] + coeff0_54*dmats1[54][41] + coeff0_55*dmats1[55][41];
          new_coeff0_42 = coeff0_0*dmats1[0][42] + coeff0_1*dmats1[1][42] + coeff0_2*dmats1[2][42] + coeff0_3*dmats1[3][42] + coeff0_4*dmats1[4][42] + coeff0_5*dmats1[5][42] + coeff0_6*dmats1[6][42] + coeff0_7*dmats1[7][42] + coeff0_8*dmats1[8][42] + coeff0_9*dmats1[9][42] + coeff0_10*dmats1[10][42] + coeff0_11*dmats1[11][42] + coeff0_12*dmats1[12][42] + coeff0_13*dmats1[13][42] + coeff0_14*dmats1[14][42] + coeff0_15*dmats1[15][42] + coeff0_16*dmats1[16][42] + coeff0_17*dmats1[17][42] + coeff0_18*dmats1[18][42] + coeff0_19*dmats1[19][42] + coeff0_20*dmats1[20][42] + coeff0_21*dmats1[21][42] + coeff0_22*dmats1[22][42] + coeff0_23*dmats1[23][42] + coeff0_24*dmats1[24][42] + coeff0_25*dmats1[25][42] + coeff0_26*dmats1[26][42] + coeff0_27*dmats1[27][42] + coeff0_28*dmats1[28][42] + coeff0_29*dmats1[29][42] + coeff0_30*dmats1[30][42] + coeff0_31*dmats1[31][42] + coeff0_32*dmats1[32][42] + coeff0_33*dmats1[33][42] + coeff0_34*dmats1[34][42] + coeff0_35*dmats1[35][42] + coeff0_36*dmats1[36][42] + coeff0_37*dmats1[37][42] + coeff0_38*dmats1[38][42] + coeff0_39*dmats1[39][42] + coeff0_40*dmats1[40][42] + coeff0_41*dmats1[41][42] + coeff0_42*dmats1[42][42] + coeff0_43*dmats1[43][42] + coeff0_44*dmats1[44][42] + coeff0_45*dmats1[45][42] + coeff0_46*dmats1[46][42] + coeff0_47*dmats1[47][42] + coeff0_48*dmats1[48][42] + coeff0_49*dmats1[49][42] + coeff0_50*dmats1[50][42] + coeff0_51*dmats1[51][42] + coeff0_52*dmats1[52][42] + coeff0_53*dmats1[53][42] + coeff0_54*dmats1[54][42] + coeff0_55*dmats1[55][42];
          new_coeff0_43 = coeff0_0*dmats1[0][43] + coeff0_1*dmats1[1][43] + coeff0_2*dmats1[2][43] + coeff0_3*dmats1[3][43] + coeff0_4*dmats1[4][43] + coeff0_5*dmats1[5][43] + coeff0_6*dmats1[6][43] + coeff0_7*dmats1[7][43] + coeff0_8*dmats1[8][43] + coeff0_9*dmats1[9][43] + coeff0_10*dmats1[10][43] + coeff0_11*dmats1[11][43] + coeff0_12*dmats1[12][43] + coeff0_13*dmats1[13][43] + coeff0_14*dmats1[14][43] + coeff0_15*dmats1[15][43] + coeff0_16*dmats1[16][43] + coeff0_17*dmats1[17][43] + coeff0_18*dmats1[18][43] + coeff0_19*dmats1[19][43] + coeff0_20*dmats1[20][43] + coeff0_21*dmats1[21][43] + coeff0_22*dmats1[22][43] + coeff0_23*dmats1[23][43] + coeff0_24*dmats1[24][43] + coeff0_25*dmats1[25][43] + coeff0_26*dmats1[26][43] + coeff0_27*dmats1[27][43] + coeff0_28*dmats1[28][43] + coeff0_29*dmats1[29][43] + coeff0_30*dmats1[30][43] + coeff0_31*dmats1[31][43] + coeff0_32*dmats1[32][43] + coeff0_33*dmats1[33][43] + coeff0_34*dmats1[34][43] + coeff0_35*dmats1[35][43] + coeff0_36*dmats1[36][43] + coeff0_37*dmats1[37][43] + coeff0_38*dmats1[38][43] + coeff0_39*dmats1[39][43] + coeff0_40*dmats1[40][43] + coeff0_41*dmats1[41][43] + coeff0_42*dmats1[42][43] + coeff0_43*dmats1[43][43] + coeff0_44*dmats1[44][43] + coeff0_45*dmats1[45][43] + coeff0_46*dmats1[46][43] + coeff0_47*dmats1[47][43] + coeff0_48*dmats1[48][43] + coeff0_49*dmats1[49][43] + coeff0_50*dmats1[50][43] + coeff0_51*dmats1[51][43] + coeff0_52*dmats1[52][43] + coeff0_53*dmats1[53][43] + coeff0_54*dmats1[54][43] + coeff0_55*dmats1[55][43];
          new_coeff0_44 = coeff0_0*dmats1[0][44] + coeff0_1*dmats1[1][44] + coeff0_2*dmats1[2][44] + coeff0_3*dmats1[3][44] + coeff0_4*dmats1[4][44] + coeff0_5*dmats1[5][44] + coeff0_6*dmats1[6][44] + coeff0_7*dmats1[7][44] + coeff0_8*dmats1[8][44] + coeff0_9*dmats1[9][44] + coeff0_10*dmats1[10][44] + coeff0_11*dmats1[11][44] + coeff0_12*dmats1[12][44] + coeff0_13*dmats1[13][44] + coeff0_14*dmats1[14][44] + coeff0_15*dmats1[15][44] + coeff0_16*dmats1[16][44] + coeff0_17*dmats1[17][44] + coeff0_18*dmats1[18][44] + coeff0_19*dmats1[19][44] + coeff0_20*dmats1[20][44] + coeff0_21*dmats1[21][44] + coeff0_22*dmats1[22][44] + coeff0_23*dmats1[23][44] + coeff0_24*dmats1[24][44] + coeff0_25*dmats1[25][44] + coeff0_26*dmats1[26][44] + coeff0_27*dmats1[27][44] + coeff0_28*dmats1[28][44] + coeff0_29*dmats1[29][44] + coeff0_30*dmats1[30][44] + coeff0_31*dmats1[31][44] + coeff0_32*dmats1[32][44] + coeff0_33*dmats1[33][44] + coeff0_34*dmats1[34][44] + coeff0_35*dmats1[35][44] + coeff0_36*dmats1[36][44] + coeff0_37*dmats1[37][44] + coeff0_38*dmats1[38][44] + coeff0_39*dmats1[39][44] + coeff0_40*dmats1[40][44] + coeff0_41*dmats1[41][44] + coeff0_42*dmats1[42][44] + coeff0_43*dmats1[43][44] + coeff0_44*dmats1[44][44] + coeff0_45*dmats1[45][44] + coeff0_46*dmats1[46][44] + coeff0_47*dmats1[47][44] + coeff0_48*dmats1[48][44] + coeff0_49*dmats1[49][44] + coeff0_50*dmats1[50][44] + coeff0_51*dmats1[51][44] + coeff0_52*dmats1[52][44] + coeff0_53*dmats1[53][44] + coeff0_54*dmats1[54][44] + coeff0_55*dmats1[55][44];
          new_coeff0_45 = coeff0_0*dmats1[0][45] + coeff0_1*dmats1[1][45] + coeff0_2*dmats1[2][45] + coeff0_3*dmats1[3][45] + coeff0_4*dmats1[4][45] + coeff0_5*dmats1[5][45] + coeff0_6*dmats1[6][45] + coeff0_7*dmats1[7][45] + coeff0_8*dmats1[8][45] + coeff0_9*dmats1[9][45] + coeff0_10*dmats1[10][45] + coeff0_11*dmats1[11][45] + coeff0_12*dmats1[12][45] + coeff0_13*dmats1[13][45] + coeff0_14*dmats1[14][45] + coeff0_15*dmats1[15][45] + coeff0_16*dmats1[16][45] + coeff0_17*dmats1[17][45] + coeff0_18*dmats1[18][45] + coeff0_19*dmats1[19][45] + coeff0_20*dmats1[20][45] + coeff0_21*dmats1[21][45] + coeff0_22*dmats1[22][45] + coeff0_23*dmats1[23][45] + coeff0_24*dmats1[24][45] + coeff0_25*dmats1[25][45] + coeff0_26*dmats1[26][45] + coeff0_27*dmats1[27][45] + coeff0_28*dmats1[28][45] + coeff0_29*dmats1[29][45] + coeff0_30*dmats1[30][45] + coeff0_31*dmats1[31][45] + coeff0_32*dmats1[32][45] + coeff0_33*dmats1[33][45] + coeff0_34*dmats1[34][45] + coeff0_35*dmats1[35][45] + coeff0_36*dmats1[36][45] + coeff0_37*dmats1[37][45] + coeff0_38*dmats1[38][45] + coeff0_39*dmats1[39][45] + coeff0_40*dmats1[40][45] + coeff0_41*dmats1[41][45] + coeff0_42*dmats1[42][45] + coeff0_43*dmats1[43][45] + coeff0_44*dmats1[44][45] + coeff0_45*dmats1[45][45] + coeff0_46*dmats1[46][45] + coeff0_47*dmats1[47][45] + coeff0_48*dmats1[48][45] + coeff0_49*dmats1[49][45] + coeff0_50*dmats1[50][45] + coeff0_51*dmats1[51][45] + coeff0_52*dmats1[52][45] + coeff0_53*dmats1[53][45] + coeff0_54*dmats1[54][45] + coeff0_55*dmats1[55][45];
          new_coeff0_46 = coeff0_0*dmats1[0][46] + coeff0_1*dmats1[1][46] + coeff0_2*dmats1[2][46] + coeff0_3*dmats1[3][46] + coeff0_4*dmats1[4][46] + coeff0_5*dmats1[5][46] + coeff0_6*dmats1[6][46] + coeff0_7*dmats1[7][46] + coeff0_8*dmats1[8][46] + coeff0_9*dmats1[9][46] + coeff0_10*dmats1[10][46] + coeff0_11*dmats1[11][46] + coeff0_12*dmats1[12][46] + coeff0_13*dmats1[13][46] + coeff0_14*dmats1[14][46] + coeff0_15*dmats1[15][46] + coeff0_16*dmats1[16][46] + coeff0_17*dmats1[17][46] + coeff0_18*dmats1[18][46] + coeff0_19*dmats1[19][46] + coeff0_20*dmats1[20][46] + coeff0_21*dmats1[21][46] + coeff0_22*dmats1[22][46] + coeff0_23*dmats1[23][46] + coeff0_24*dmats1[24][46] + coeff0_25*dmats1[25][46] + coeff0_26*dmats1[26][46] + coeff0_27*dmats1[27][46] + coeff0_28*dmats1[28][46] + coeff0_29*dmats1[29][46] + coeff0_30*dmats1[30][46] + coeff0_31*dmats1[31][46] + coeff0_32*dmats1[32][46] + coeff0_33*dmats1[33][46] + coeff0_34*dmats1[34][46] + coeff0_35*dmats1[35][46] + coeff0_36*dmats1[36][46] + coeff0_37*dmats1[37][46] + coeff0_38*dmats1[38][46] + coeff0_39*dmats1[39][46] + coeff0_40*dmats1[40][46] + coeff0_41*dmats1[41][46] + coeff0_42*dmats1[42][46] + coeff0_43*dmats1[43][46] + coeff0_44*dmats1[44][46] + coeff0_45*dmats1[45][46] + coeff0_46*dmats1[46][46] + coeff0_47*dmats1[47][46] + coeff0_48*dmats1[48][46] + coeff0_49*dmats1[49][46] + coeff0_50*dmats1[50][46] + coeff0_51*dmats1[51][46] + coeff0_52*dmats1[52][46] + coeff0_53*dmats1[53][46] + coeff0_54*dmats1[54][46] + coeff0_55*dmats1[55][46];
          new_coeff0_47 = coeff0_0*dmats1[0][47] + coeff0_1*dmats1[1][47] + coeff0_2*dmats1[2][47] + coeff0_3*dmats1[3][47] + coeff0_4*dmats1[4][47] + coeff0_5*dmats1[5][47] + coeff0_6*dmats1[6][47] + coeff0_7*dmats1[7][47] + coeff0_8*dmats1[8][47] + coeff0_9*dmats1[9][47] + coeff0_10*dmats1[10][47] + coeff0_11*dmats1[11][47] + coeff0_12*dmats1[12][47] + coeff0_13*dmats1[13][47] + coeff0_14*dmats1[14][47] + coeff0_15*dmats1[15][47] + coeff0_16*dmats1[16][47] + coeff0_17*dmats1[17][47] + coeff0_18*dmats1[18][47] + coeff0_19*dmats1[19][47] + coeff0_20*dmats1[20][47] + coeff0_21*dmats1[21][47] + coeff0_22*dmats1[22][47] + coeff0_23*dmats1[23][47] + coeff0_24*dmats1[24][47] + coeff0_25*dmats1[25][47] + coeff0_26*dmats1[26][47] + coeff0_27*dmats1[27][47] + coeff0_28*dmats1[28][47] + coeff0_29*dmats1[29][47] + coeff0_30*dmats1[30][47] + coeff0_31*dmats1[31][47] + coeff0_32*dmats1[32][47] + coeff0_33*dmats1[33][47] + coeff0_34*dmats1[34][47] + coeff0_35*dmats1[35][47] + coeff0_36*dmats1[36][47] + coeff0_37*dmats1[37][47] + coeff0_38*dmats1[38][47] + coeff0_39*dmats1[39][47] + coeff0_40*dmats1[40][47] + coeff0_41*dmats1[41][47] + coeff0_42*dmats1[42][47] + coeff0_43*dmats1[43][47] + coeff0_44*dmats1[44][47] + coeff0_45*dmats1[45][47] + coeff0_46*dmats1[46][47] + coeff0_47*dmats1[47][47] + coeff0_48*dmats1[48][47] + coeff0_49*dmats1[49][47] + coeff0_50*dmats1[50][47] + coeff0_51*dmats1[51][47] + coeff0_52*dmats1[52][47] + coeff0_53*dmats1[53][47] + coeff0_54*dmats1[54][47] + coeff0_55*dmats1[55][47];
          new_coeff0_48 = coeff0_0*dmats1[0][48] + coeff0_1*dmats1[1][48] + coeff0_2*dmats1[2][48] + coeff0_3*dmats1[3][48] + coeff0_4*dmats1[4][48] + coeff0_5*dmats1[5][48] + coeff0_6*dmats1[6][48] + coeff0_7*dmats1[7][48] + coeff0_8*dmats1[8][48] + coeff0_9*dmats1[9][48] + coeff0_10*dmats1[10][48] + coeff0_11*dmats1[11][48] + coeff0_12*dmats1[12][48] + coeff0_13*dmats1[13][48] + coeff0_14*dmats1[14][48] + coeff0_15*dmats1[15][48] + coeff0_16*dmats1[16][48] + coeff0_17*dmats1[17][48] + coeff0_18*dmats1[18][48] + coeff0_19*dmats1[19][48] + coeff0_20*dmats1[20][48] + coeff0_21*dmats1[21][48] + coeff0_22*dmats1[22][48] + coeff0_23*dmats1[23][48] + coeff0_24*dmats1[24][48] + coeff0_25*dmats1[25][48] + coeff0_26*dmats1[26][48] + coeff0_27*dmats1[27][48] + coeff0_28*dmats1[28][48] + coeff0_29*dmats1[29][48] + coeff0_30*dmats1[30][48] + coeff0_31*dmats1[31][48] + coeff0_32*dmats1[32][48] + coeff0_33*dmats1[33][48] + coeff0_34*dmats1[34][48] + coeff0_35*dmats1[35][48] + coeff0_36*dmats1[36][48] + coeff0_37*dmats1[37][48] + coeff0_38*dmats1[38][48] + coeff0_39*dmats1[39][48] + coeff0_40*dmats1[40][48] + coeff0_41*dmats1[41][48] + coeff0_42*dmats1[42][48] + coeff0_43*dmats1[43][48] + coeff0_44*dmats1[44][48] + coeff0_45*dmats1[45][48] + coeff0_46*dmats1[46][48] + coeff0_47*dmats1[47][48] + coeff0_48*dmats1[48][48] + coeff0_49*dmats1[49][48] + coeff0_50*dmats1[50][48] + coeff0_51*dmats1[51][48] + coeff0_52*dmats1[52][48] + coeff0_53*dmats1[53][48] + coeff0_54*dmats1[54][48] + coeff0_55*dmats1[55][48];
          new_coeff0_49 = coeff0_0*dmats1[0][49] + coeff0_1*dmats1[1][49] + coeff0_2*dmats1[2][49] + coeff0_3*dmats1[3][49] + coeff0_4*dmats1[4][49] + coeff0_5*dmats1[5][49] + coeff0_6*dmats1[6][49] + coeff0_7*dmats1[7][49] + coeff0_8*dmats1[8][49] + coeff0_9*dmats1[9][49] + coeff0_10*dmats1[10][49] + coeff0_11*dmats1[11][49] + coeff0_12*dmats1[12][49] + coeff0_13*dmats1[13][49] + coeff0_14*dmats1[14][49] + coeff0_15*dmats1[15][49] + coeff0_16*dmats1[16][49] + coeff0_17*dmats1[17][49] + coeff0_18*dmats1[18][49] + coeff0_19*dmats1[19][49] + coeff0_20*dmats1[20][49] + coeff0_21*dmats1[21][49] + coeff0_22*dmats1[22][49] + coeff0_23*dmats1[23][49] + coeff0_24*dmats1[24][49] + coeff0_25*dmats1[25][49] + coeff0_26*dmats1[26][49] + coeff0_27*dmats1[27][49] + coeff0_28*dmats1[28][49] + coeff0_29*dmats1[29][49] + coeff0_30*dmats1[30][49] + coeff0_31*dmats1[31][49] + coeff0_32*dmats1[32][49] + coeff0_33*dmats1[33][49] + coeff0_34*dmats1[34][49] + coeff0_35*dmats1[35][49] + coeff0_36*dmats1[36][49] + coeff0_37*dmats1[37][49] + coeff0_38*dmats1[38][49] + coeff0_39*dmats1[39][49] + coeff0_40*dmats1[40][49] + coeff0_41*dmats1[41][49] + coeff0_42*dmats1[42][49] + coeff0_43*dmats1[43][49] + coeff0_44*dmats1[44][49] + coeff0_45*dmats1[45][49] + coeff0_46*dmats1[46][49] + coeff0_47*dmats1[47][49] + coeff0_48*dmats1[48][49] + coeff0_49*dmats1[49][49] + coeff0_50*dmats1[50][49] + coeff0_51*dmats1[51][49] + coeff0_52*dmats1[52][49] + coeff0_53*dmats1[53][49] + coeff0_54*dmats1[54][49] + coeff0_55*dmats1[55][49];
          new_coeff0_50 = coeff0_0*dmats1[0][50] + coeff0_1*dmats1[1][50] + coeff0_2*dmats1[2][50] + coeff0_3*dmats1[3][50] + coeff0_4*dmats1[4][50] + coeff0_5*dmats1[5][50] + coeff0_6*dmats1[6][50] + coeff0_7*dmats1[7][50] + coeff0_8*dmats1[8][50] + coeff0_9*dmats1[9][50] + coeff0_10*dmats1[10][50] + coeff0_11*dmats1[11][50] + coeff0_12*dmats1[12][50] + coeff0_13*dmats1[13][50] + coeff0_14*dmats1[14][50] + coeff0_15*dmats1[15][50] + coeff0_16*dmats1[16][50] + coeff0_17*dmats1[17][50] + coeff0_18*dmats1[18][50] + coeff0_19*dmats1[19][50] + coeff0_20*dmats1[20][50] + coeff0_21*dmats1[21][50] + coeff0_22*dmats1[22][50] + coeff0_23*dmats1[23][50] + coeff0_24*dmats1[24][50] + coeff0_25*dmats1[25][50] + coeff0_26*dmats1[26][50] + coeff0_27*dmats1[27][50] + coeff0_28*dmats1[28][50] + coeff0_29*dmats1[29][50] + coeff0_30*dmats1[30][50] + coeff0_31*dmats1[31][50] + coeff0_32*dmats1[32][50] + coeff0_33*dmats1[33][50] + coeff0_34*dmats1[34][50] + coeff0_35*dmats1[35][50] + coeff0_36*dmats1[36][50] + coeff0_37*dmats1[37][50] + coeff0_38*dmats1[38][50] + coeff0_39*dmats1[39][50] + coeff0_40*dmats1[40][50] + coeff0_41*dmats1[41][50] + coeff0_42*dmats1[42][50] + coeff0_43*dmats1[43][50] + coeff0_44*dmats1[44][50] + coeff0_45*dmats1[45][50] + coeff0_46*dmats1[46][50] + coeff0_47*dmats1[47][50] + coeff0_48*dmats1[48][50] + coeff0_49*dmats1[49][50] + coeff0_50*dmats1[50][50] + coeff0_51*dmats1[51][50] + coeff0_52*dmats1[52][50] + coeff0_53*dmats1[53][50] + coeff0_54*dmats1[54][50] + coeff0_55*dmats1[55][50];
          new_coeff0_51 = coeff0_0*dmats1[0][51] + coeff0_1*dmats1[1][51] + coeff0_2*dmats1[2][51] + coeff0_3*dmats1[3][51] + coeff0_4*dmats1[4][51] + coeff0_5*dmats1[5][51] + coeff0_6*dmats1[6][51] + coeff0_7*dmats1[7][51] + coeff0_8*dmats1[8][51] + coeff0_9*dmats1[9][51] + coeff0_10*dmats1[10][51] + coeff0_11*dmats1[11][51] + coeff0_12*dmats1[12][51] + coeff0_13*dmats1[13][51] + coeff0_14*dmats1[14][51] + coeff0_15*dmats1[15][51] + coeff0_16*dmats1[16][51] + coeff0_17*dmats1[17][51] + coeff0_18*dmats1[18][51] + coeff0_19*dmats1[19][51] + coeff0_20*dmats1[20][51] + coeff0_21*dmats1[21][51] + coeff0_22*dmats1[22][51] + coeff0_23*dmats1[23][51] + coeff0_24*dmats1[24][51] + coeff0_25*dmats1[25][51] + coeff0_26*dmats1[26][51] + coeff0_27*dmats1[27][51] + coeff0_28*dmats1[28][51] + coeff0_29*dmats1[29][51] + coeff0_30*dmats1[30][51] + coeff0_31*dmats1[31][51] + coeff0_32*dmats1[32][51] + coeff0_33*dmats1[33][51] + coeff0_34*dmats1[34][51] + coeff0_35*dmats1[35][51] + coeff0_36*dmats1[36][51] + coeff0_37*dmats1[37][51] + coeff0_38*dmats1[38][51] + coeff0_39*dmats1[39][51] + coeff0_40*dmats1[40][51] + coeff0_41*dmats1[41][51] + coeff0_42*dmats1[42][51] + coeff0_43*dmats1[43][51] + coeff0_44*dmats1[44][51] + coeff0_45*dmats1[45][51] + coeff0_46*dmats1[46][51] + coeff0_47*dmats1[47][51] + coeff0_48*dmats1[48][51] + coeff0_49*dmats1[49][51] + coeff0_50*dmats1[50][51] + coeff0_51*dmats1[51][51] + coeff0_52*dmats1[52][51] + coeff0_53*dmats1[53][51] + coeff0_54*dmats1[54][51] + coeff0_55*dmats1[55][51];
          new_coeff0_52 = coeff0_0*dmats1[0][52] + coeff0_1*dmats1[1][52] + coeff0_2*dmats1[2][52] + coeff0_3*dmats1[3][52] + coeff0_4*dmats1[4][52] + coeff0_5*dmats1[5][52] + coeff0_6*dmats1[6][52] + coeff0_7*dmats1[7][52] + coeff0_8*dmats1[8][52] + coeff0_9*dmats1[9][52] + coeff0_10*dmats1[10][52] + coeff0_11*dmats1[11][52] + coeff0_12*dmats1[12][52] + coeff0_13*dmats1[13][52] + coeff0_14*dmats1[14][52] + coeff0_15*dmats1[15][52] + coeff0_16*dmats1[16][52] + coeff0_17*dmats1[17][52] + coeff0_18*dmats1[18][52] + coeff0_19*dmats1[19][52] + coeff0_20*dmats1[20][52] + coeff0_21*dmats1[21][52] + coeff0_22*dmats1[22][52] + coeff0_23*dmats1[23][52] + coeff0_24*dmats1[24][52] + coeff0_25*dmats1[25][52] + coeff0_26*dmats1[26][52] + coeff0_27*dmats1[27][52] + coeff0_28*dmats1[28][52] + coeff0_29*dmats1[29][52] + coeff0_30*dmats1[30][52] + coeff0_31*dmats1[31][52] + coeff0_32*dmats1[32][52] + coeff0_33*dmats1[33][52] + coeff0_34*dmats1[34][52] + coeff0_35*dmats1[35][52] + coeff0_36*dmats1[36][52] + coeff0_37*dmats1[37][52] + coeff0_38*dmats1[38][52] + coeff0_39*dmats1[39][52] + coeff0_40*dmats1[40][52] + coeff0_41*dmats1[41][52] + coeff0_42*dmats1[42][52] + coeff0_43*dmats1[43][52] + coeff0_44*dmats1[44][52] + coeff0_45*dmats1[45][52] + coeff0_46*dmats1[46][52] + coeff0_47*dmats1[47][52] + coeff0_48*dmats1[48][52] + coeff0_49*dmats1[49][52] + coeff0_50*dmats1[50][52] + coeff0_51*dmats1[51][52] + coeff0_52*dmats1[52][52] + coeff0_53*dmats1[53][52] + coeff0_54*dmats1[54][52] + coeff0_55*dmats1[55][52];
          new_coeff0_53 = coeff0_0*dmats1[0][53] + coeff0_1*dmats1[1][53] + coeff0_2*dmats1[2][53] + coeff0_3*dmats1[3][53] + coeff0_4*dmats1[4][53] + coeff0_5*dmats1[5][53] + coeff0_6*dmats1[6][53] + coeff0_7*dmats1[7][53] + coeff0_8*dmats1[8][53] + coeff0_9*dmats1[9][53] + coeff0_10*dmats1[10][53] + coeff0_11*dmats1[11][53] + coeff0_12*dmats1[12][53] + coeff0_13*dmats1[13][53] + coeff0_14*dmats1[14][53] + coeff0_15*dmats1[15][53] + coeff0_16*dmats1[16][53] + coeff0_17*dmats1[17][53] + coeff0_18*dmats1[18][53] + coeff0_19*dmats1[19][53] + coeff0_20*dmats1[20][53] + coeff0_21*dmats1[21][53] + coeff0_22*dmats1[22][53] + coeff0_23*dmats1[23][53] + coeff0_24*dmats1[24][53] + coeff0_25*dmats1[25][53] + coeff0_26*dmats1[26][53] + coeff0_27*dmats1[27][53] + coeff0_28*dmats1[28][53] + coeff0_29*dmats1[29][53] + coeff0_30*dmats1[30][53] + coeff0_31*dmats1[31][53] + coeff0_32*dmats1[32][53] + coeff0_33*dmats1[33][53] + coeff0_34*dmats1[34][53] + coeff0_35*dmats1[35][53] + coeff0_36*dmats1[36][53] + coeff0_37*dmats1[37][53] + coeff0_38*dmats1[38][53] + coeff0_39*dmats1[39][53] + coeff0_40*dmats1[40][53] + coeff0_41*dmats1[41][53] + coeff0_42*dmats1[42][53] + coeff0_43*dmats1[43][53] + coeff0_44*dmats1[44][53] + coeff0_45*dmats1[45][53] + coeff0_46*dmats1[46][53] + coeff0_47*dmats1[47][53] + coeff0_48*dmats1[48][53] + coeff0_49*dmats1[49][53] + coeff0_50*dmats1[50][53] + coeff0_51*dmats1[51][53] + coeff0_52*dmats1[52][53] + coeff0_53*dmats1[53][53] + coeff0_54*dmats1[54][53] + coeff0_55*dmats1[55][53];
          new_coeff0_54 = coeff0_0*dmats1[0][54] + coeff0_1*dmats1[1][54] + coeff0_2*dmats1[2][54] + coeff0_3*dmats1[3][54] + coeff0_4*dmats1[4][54] + coeff0_5*dmats1[5][54] + coeff0_6*dmats1[6][54] + coeff0_7*dmats1[7][54] + coeff0_8*dmats1[8][54] + coeff0_9*dmats1[9][54] + coeff0_10*dmats1[10][54] + coeff0_11*dmats1[11][54] + coeff0_12*dmats1[12][54] + coeff0_13*dmats1[13][54] + coeff0_14*dmats1[14][54] + coeff0_15*dmats1[15][54] + coeff0_16*dmats1[16][54] + coeff0_17*dmats1[17][54] + coeff0_18*dmats1[18][54] + coeff0_19*dmats1[19][54] + coeff0_20*dmats1[20][54] + coeff0_21*dmats1[21][54] + coeff0_22*dmats1[22][54] + coeff0_23*dmats1[23][54] + coeff0_24*dmats1[24][54] + coeff0_25*dmats1[25][54] + coeff0_26*dmats1[26][54] + coeff0_27*dmats1[27][54] + coeff0_28*dmats1[28][54] + coeff0_29*dmats1[29][54] + coeff0_30*dmats1[30][54] + coeff0_31*dmats1[31][54] + coeff0_32*dmats1[32][54] + coeff0_33*dmats1[33][54] + coeff0_34*dmats1[34][54] + coeff0_35*dmats1[35][54] + coeff0_36*dmats1[36][54] + coeff0_37*dmats1[37][54] + coeff0_38*dmats1[38][54] + coeff0_39*dmats1[39][54] + coeff0_40*dmats1[40][54] + coeff0_41*dmats1[41][54] + coeff0_42*dmats1[42][54] + coeff0_43*dmats1[43][54] + coeff0_44*dmats1[44][54] + coeff0_45*dmats1[45][54] + coeff0_46*dmats1[46][54] + coeff0_47*dmats1[47][54] + coeff0_48*dmats1[48][54] + coeff0_49*dmats1[49][54] + coeff0_50*dmats1[50][54] + coeff0_51*dmats1[51][54] + coeff0_52*dmats1[52][54] + coeff0_53*dmats1[53][54] + coeff0_54*dmats1[54][54] + coeff0_55*dmats1[55][54];
          new_coeff0_55 = coeff0_0*dmats1[0][55] + coeff0_1*dmats1[1][55] + coeff0_2*dmats1[2][55] + coeff0_3*dmats1[3][55] + coeff0_4*dmats1[4][55] + coeff0_5*dmats1[5][55] + coeff0_6*dmats1[6][55] + coeff0_7*dmats1[7][55] + coeff0_8*dmats1[8][55] + coeff0_9*dmats1[9][55] + coeff0_10*dmats1[10][55] + coeff0_11*dmats1[11][55] + coeff0_12*dmats1[12][55] + coeff0_13*dmats1[13][55] + coeff0_14*dmats1[14][55] + coeff0_15*dmats1[15][55] + coeff0_16*dmats1[16][55] + coeff0_17*dmats1[17][55] + coeff0_18*dmats1[18][55] + coeff0_19*dmats1[19][55] + coeff0_20*dmats1[20][55] + coeff0_21*dmats1[21][55] + coeff0_22*dmats1[22][55] + coeff0_23*dmats1[23][55] + coeff0_24*dmats1[24][55] + coeff0_25*dmats1[25][55] + coeff0_26*dmats1[26][55] + coeff0_27*dmats1[27][55] + coeff0_28*dmats1[28][55] + coeff0_29*dmats1[29][55] + coeff0_30*dmats1[30][55] + coeff0_31*dmats1[31][55] + coeff0_32*dmats1[32][55] + coeff0_33*dmats1[33][55] + coeff0_34*dmats1[34][55] + coeff0_35*dmats1[35][55] + coeff0_36*dmats1[36][55] + coeff0_37*dmats1[37][55] + coeff0_38*dmats1[38][55] + coeff0_39*dmats1[39][55] + coeff0_40*dmats1[40][55] + coeff0_41*dmats1[41][55] + coeff0_42*dmats1[42][55] + coeff0_43*dmats1[43][55] + coeff0_44*dmats1[44][55] + coeff0_45*dmats1[45][55] + coeff0_46*dmats1[46][55] + coeff0_47*dmats1[47][55] + coeff0_48*dmats1[48][55] + coeff0_49*dmats1[49][55] + coeff0_50*dmats1[50][55] + coeff0_51*dmats1[51][55] + coeff0_52*dmats1[52][55] + coeff0_53*dmats1[53][55] + coeff0_54*dmats1[54][55] + coeff0_55*dmats1[55][55];
        }
        if(combinations[deriv_num][j] == 2)
        {
          new_coeff0_0 = coeff0_0*dmats2[0][0] + coeff0_1*dmats2[1][0] + coeff0_2*dmats2[2][0] + coeff0_3*dmats2[3][0] + coeff0_4*dmats2[4][0] + coeff0_5*dmats2[5][0] + coeff0_6*dmats2[6][0] + coeff0_7*dmats2[7][0] + coeff0_8*dmats2[8][0] + coeff0_9*dmats2[9][0] + coeff0_10*dmats2[10][0] + coeff0_11*dmats2[11][0] + coeff0_12*dmats2[12][0] + coeff0_13*dmats2[13][0] + coeff0_14*dmats2[14][0] + coeff0_15*dmats2[15][0] + coeff0_16*dmats2[16][0] + coeff0_17*dmats2[17][0] + coeff0_18*dmats2[18][0] + coeff0_19*dmats2[19][0] + coeff0_20*dmats2[20][0] + coeff0_21*dmats2[21][0] + coeff0_22*dmats2[22][0] + coeff0_23*dmats2[23][0] + coeff0_24*dmats2[24][0] + coeff0_25*dmats2[25][0] + coeff0_26*dmats2[26][0] + coeff0_27*dmats2[27][0] + coeff0_28*dmats2[28][0] + coeff0_29*dmats2[29][0] + coeff0_30*dmats2[30][0] + coeff0_31*dmats2[31][0] + coeff0_32*dmats2[32][0] + coeff0_33*dmats2[33][0] + coeff0_34*dmats2[34][0] + coeff0_35*dmats2[35][0] + coeff0_36*dmats2[36][0] + coeff0_37*dmats2[37][0] + coeff0_38*dmats2[38][0] + coeff0_39*dmats2[39][0] + coeff0_40*dmats2[40][0] + coeff0_41*dmats2[41][0] + coeff0_42*dmats2[42][0] + coeff0_43*dmats2[43][0] + coeff0_44*dmats2[44][0] + coeff0_45*dmats2[45][0] + coeff0_46*dmats2[46][0] + coeff0_47*dmats2[47][0] + coeff0_48*dmats2[48][0] + coeff0_49*dmats2[49][0] + coeff0_50*dmats2[50][0] + coeff0_51*dmats2[51][0] + coeff0_52*dmats2[52][0] + coeff0_53*dmats2[53][0] + coeff0_54*dmats2[54][0] + coeff0_55*dmats2[55][0];
          new_coeff0_1 = coeff0_0*dmats2[0][1] + coeff0_1*dmats2[1][1] + coeff0_2*dmats2[2][1] + coeff0_3*dmats2[3][1] + coeff0_4*dmats2[4][1] + coeff0_5*dmats2[5][1] + coeff0_6*dmats2[6][1] + coeff0_7*dmats2[7][1] + coeff0_8*dmats2[8][1] + coeff0_9*dmats2[9][1] + coeff0_10*dmats2[10][1] + coeff0_11*dmats2[11][1] + coeff0_12*dmats2[12][1] + coeff0_13*dmats2[13][1] + coeff0_14*dmats2[14][1] + coeff0_15*dmats2[15][1] + coeff0_16*dmats2[16][1] + coeff0_17*dmats2[17][1] + coeff0_18*dmats2[18][1] + coeff0_19*dmats2[19][1] + coeff0_20*dmats2[20][1] + coeff0_21*dmats2[21][1] + coeff0_22*dmats2[22][1] + coeff0_23*dmats2[23][1] + coeff0_24*dmats2[24][1] + coeff0_25*dmats2[25][1] + coeff0_26*dmats2[26][1] + coeff0_27*dmats2[27][1] + coeff0_28*dmats2[28][1] + coeff0_29*dmats2[29][1] + coeff0_30*dmats2[30][1] + coeff0_31*dmats2[31][1] + coeff0_32*dmats2[32][1] + coeff0_33*dmats2[33][1] + coeff0_34*dmats2[34][1] + coeff0_35*dmats2[35][1] + coeff0_36*dmats2[36][1] + coeff0_37*dmats2[37][1] + coeff0_38*dmats2[38][1] + coeff0_39*dmats2[39][1] + coeff0_40*dmats2[40][1] + coeff0_41*dmats2[41][1] + coeff0_42*dmats2[42][1] + coeff0_43*dmats2[43][1] + coeff0_44*dmats2[44][1] + coeff0_45*dmats2[45][1] + coeff0_46*dmats2[46][1] + coeff0_47*dmats2[47][1] + coeff0_48*dmats2[48][1] + coeff0_49*dmats2[49][1] + coeff0_50*dmats2[50][1] + coeff0_51*dmats2[51][1] + coeff0_52*dmats2[52][1] + coeff0_53*dmats2[53][1] + coeff0_54*dmats2[54][1] + coeff0_55*dmats2[55][1];
          new_coeff0_2 = coeff0_0*dmats2[0][2] + coeff0_1*dmats2[1][2] + coeff0_2*dmats2[2][2] + coeff0_3*dmats2[3][2] + coeff0_4*dmats2[4][2] + coeff0_5*dmats2[5][2] + coeff0_6*dmats2[6][2] + coeff0_7*dmats2[7][2] + coeff0_8*dmats2[8][2] + coeff0_9*dmats2[9][2] + coeff0_10*dmats2[10][2] + coeff0_11*dmats2[11][2] + coeff0_12*dmats2[12][2] + coeff0_13*dmats2[13][2] + coeff0_14*dmats2[14][2] + coeff0_15*dmats2[15][2] + coeff0_16*dmats2[16][2] + coeff0_17*dmats2[17][2] + coeff0_18*dmats2[18][2] + coeff0_19*dmats2[19][2] + coeff0_20*dmats2[20][2] + coeff0_21*dmats2[21][2] + coeff0_22*dmats2[22][2] + coeff0_23*dmats2[23][2] + coeff0_24*dmats2[24][2] + coeff0_25*dmats2[25][2] + coeff0_26*dmats2[26][2] + coeff0_27*dmats2[27][2] + coeff0_28*dmats2[28][2] + coeff0_29*dmats2[29][2] + coeff0_30*dmats2[30][2] + coeff0_31*dmats2[31][2] + coeff0_32*dmats2[32][2] + coeff0_33*dmats2[33][2] + coeff0_34*dmats2[34][2] + coeff0_35*dmats2[35][2] + coeff0_36*dmats2[36][2] + coeff0_37*dmats2[37][2] + coeff0_38*dmats2[38][2] + coeff0_39*dmats2[39][2] + coeff0_40*dmats2[40][2] + coeff0_41*dmats2[41][2] + coeff0_42*dmats2[42][2] + coeff0_43*dmats2[43][2] + coeff0_44*dmats2[44][2] + coeff0_45*dmats2[45][2] + coeff0_46*dmats2[46][2] + coeff0_47*dmats2[47][2] + coeff0_48*dmats2[48][2] + coeff0_49*dmats2[49][2] + coeff0_50*dmats2[50][2] + coeff0_51*dmats2[51][2] + coeff0_52*dmats2[52][2] + coeff0_53*dmats2[53][2] + coeff0_54*dmats2[54][2] + coeff0_55*dmats2[55][2];
          new_coeff0_3 = coeff0_0*dmats2[0][3] + coeff0_1*dmats2[1][3] + coeff0_2*dmats2[2][3] + coeff0_3*dmats2[3][3] + coeff0_4*dmats2[4][3] + coeff0_5*dmats2[5][3] + coeff0_6*dmats2[6][3] + coeff0_7*dmats2[7][3] + coeff0_8*dmats2[8][3] + coeff0_9*dmats2[9][3] + coeff0_10*dmats2[10][3] + coeff0_11*dmats2[11][3] + coeff0_12*dmats2[12][3] + coeff0_13*dmats2[13][3] + coeff0_14*dmats2[14][3] + coeff0_15*dmats2[15][3] + coeff0_16*dmats2[16][3] + coeff0_17*dmats2[17][3] + coeff0_18*dmats2[18][3] + coeff0_19*dmats2[19][3] + coeff0_20*dmats2[20][3] + coeff0_21*dmats2[21][3] + coeff0_22*dmats2[22][3] + coeff0_23*dmats2[23][3] + coeff0_24*dmats2[24][3] + coeff0_25*dmats2[25][3] + coeff0_26*dmats2[26][3] + coeff0_27*dmats2[27][3] + coeff0_28*dmats2[28][3] + coeff0_29*dmats2[29][3] + coeff0_30*dmats2[30][3] + coeff0_31*dmats2[31][3] + coeff0_32*dmats2[32][3] + coeff0_33*dmats2[33][3] + coeff0_34*dmats2[34][3] + coeff0_35*dmats2[35][3] + coeff0_36*dmats2[36][3] + coeff0_37*dmats2[37][3] + coeff0_38*dmats2[38][3] + coeff0_39*dmats2[39][3] + coeff0_40*dmats2[40][3] + coeff0_41*dmats2[41][3] + coeff0_42*dmats2[42][3] + coeff0_43*dmats2[43][3] + coeff0_44*dmats2[44][3] + coeff0_45*dmats2[45][3] + coeff0_46*dmats2[46][3] + coeff0_47*dmats2[47][3] + coeff0_48*dmats2[48][3] + coeff0_49*dmats2[49][3] + coeff0_50*dmats2[50][3] + coeff0_51*dmats2[51][3] + coeff0_52*dmats2[52][3] + coeff0_53*dmats2[53][3] + coeff0_54*dmats2[54][3] + coeff0_55*dmats2[55][3];
          new_coeff0_4 = coeff0_0*dmats2[0][4] + coeff0_1*dmats2[1][4] + coeff0_2*dmats2[2][4] + coeff0_3*dmats2[3][4] + coeff0_4*dmats2[4][4] + coeff0_5*dmats2[5][4] + coeff0_6*dmats2[6][4] + coeff0_7*dmats2[7][4] + coeff0_8*dmats2[8][4] + coeff0_9*dmats2[9][4] + coeff0_10*dmats2[10][4] + coeff0_11*dmats2[11][4] + coeff0_12*dmats2[12][4] + coeff0_13*dmats2[13][4] + coeff0_14*dmats2[14][4] + coeff0_15*dmats2[15][4] + coeff0_16*dmats2[16][4] + coeff0_17*dmats2[17][4] + coeff0_18*dmats2[18][4] + coeff0_19*dmats2[19][4] + coeff0_20*dmats2[20][4] + coeff0_21*dmats2[21][4] + coeff0_22*dmats2[22][4] + coeff0_23*dmats2[23][4] + coeff0_24*dmats2[24][4] + coeff0_25*dmats2[25][4] + coeff0_26*dmats2[26][4] + coeff0_27*dmats2[27][4] + coeff0_28*dmats2[28][4] + coeff0_29*dmats2[29][4] + coeff0_30*dmats2[30][4] + coeff0_31*dmats2[31][4] + coeff0_32*dmats2[32][4] + coeff0_33*dmats2[33][4] + coeff0_34*dmats2[34][4] + coeff0_35*dmats2[35][4] + coeff0_36*dmats2[36][4] + coeff0_37*dmats2[37][4] + coeff0_38*dmats2[38][4] + coeff0_39*dmats2[39][4] + coeff0_40*dmats2[40][4] + coeff0_41*dmats2[41][4] + coeff0_42*dmats2[42][4] + coeff0_43*dmats2[43][4] + coeff0_44*dmats2[44][4] + coeff0_45*dmats2[45][4] + coeff0_46*dmats2[46][4] + coeff0_47*dmats2[47][4] + coeff0_48*dmats2[48][4] + coeff0_49*dmats2[49][4] + coeff0_50*dmats2[50][4] + coeff0_51*dmats2[51][4] + coeff0_52*dmats2[52][4] + coeff0_53*dmats2[53][4] + coeff0_54*dmats2[54][4] + coeff0_55*dmats2[55][4];
          new_coeff0_5 = coeff0_0*dmats2[0][5] + coeff0_1*dmats2[1][5] + coeff0_2*dmats2[2][5] + coeff0_3*dmats2[3][5] + coeff0_4*dmats2[4][5] + coeff0_5*dmats2[5][5] + coeff0_6*dmats2[6][5] + coeff0_7*dmats2[7][5] + coeff0_8*dmats2[8][5] + coeff0_9*dmats2[9][5] + coeff0_10*dmats2[10][5] + coeff0_11*dmats2[11][5] + coeff0_12*dmats2[12][5] + coeff0_13*dmats2[13][5] + coeff0_14*dmats2[14][5] + coeff0_15*dmats2[15][5] + coeff0_16*dmats2[16][5] + coeff0_17*dmats2[17][5] + coeff0_18*dmats2[18][5] + coeff0_19*dmats2[19][5] + coeff0_20*dmats2[20][5] + coeff0_21*dmats2[21][5] + coeff0_22*dmats2[22][5] + coeff0_23*dmats2[23][5] + coeff0_24*dmats2[24][5] + coeff0_25*dmats2[25][5] + coeff0_26*dmats2[26][5] + coeff0_27*dmats2[27][5] + coeff0_28*dmats2[28][5] + coeff0_29*dmats2[29][5] + coeff0_30*dmats2[30][5] + coeff0_31*dmats2[31][5] + coeff0_32*dmats2[32][5] + coeff0_33*dmats2[33][5] + coeff0_34*dmats2[34][5] + coeff0_35*dmats2[35][5] + coeff0_36*dmats2[36][5] + coeff0_37*dmats2[37][5] + coeff0_38*dmats2[38][5] + coeff0_39*dmats2[39][5] + coeff0_40*dmats2[40][5] + coeff0_41*dmats2[41][5] + coeff0_42*dmats2[42][5] + coeff0_43*dmats2[43][5] + coeff0_44*dmats2[44][5] + coeff0_45*dmats2[45][5] + coeff0_46*dmats2[46][5] + coeff0_47*dmats2[47][5] + coeff0_48*dmats2[48][5] + coeff0_49*dmats2[49][5] + coeff0_50*dmats2[50][5] + coeff0_51*dmats2[51][5] + coeff0_52*dmats2[52][5] + coeff0_53*dmats2[53][5] + coeff0_54*dmats2[54][5] + coeff0_55*dmats2[55][5];
          new_coeff0_6 = coeff0_0*dmats2[0][6] + coeff0_1*dmats2[1][6] + coeff0_2*dmats2[2][6] + coeff0_3*dmats2[3][6] + coeff0_4*dmats2[4][6] + coeff0_5*dmats2[5][6] + coeff0_6*dmats2[6][6] + coeff0_7*dmats2[7][6] + coeff0_8*dmats2[8][6] + coeff0_9*dmats2[9][6] + coeff0_10*dmats2[10][6] + coeff0_11*dmats2[11][6] + coeff0_12*dmats2[12][6] + coeff0_13*dmats2[13][6] + coeff0_14*dmats2[14][6] + coeff0_15*dmats2[15][6] + coeff0_16*dmats2[16][6] + coeff0_17*dmats2[17][6] + coeff0_18*dmats2[18][6] + coeff0_19*dmats2[19][6] + coeff0_20*dmats2[20][6] + coeff0_21*dmats2[21][6] + coeff0_22*dmats2[22][6] + coeff0_23*dmats2[23][6] + coeff0_24*dmats2[24][6] + coeff0_25*dmats2[25][6] + coeff0_26*dmats2[26][6] + coeff0_27*dmats2[27][6] + coeff0_28*dmats2[28][6] + coeff0_29*dmats2[29][6] + coeff0_30*dmats2[30][6] + coeff0_31*dmats2[31][6] + coeff0_32*dmats2[32][6] + coeff0_33*dmats2[33][6] + coeff0_34*dmats2[34][6] + coeff0_35*dmats2[35][6] + coeff0_36*dmats2[36][6] + coeff0_37*dmats2[37][6] + coeff0_38*dmats2[38][6] + coeff0_39*dmats2[39][6] + coeff0_40*dmats2[40][6] + coeff0_41*dmats2[41][6] + coeff0_42*dmats2[42][6] + coeff0_43*dmats2[43][6] + coeff0_44*dmats2[44][6] + coeff0_45*dmats2[45][6] + coeff0_46*dmats2[46][6] + coeff0_47*dmats2[47][6] + coeff0_48*dmats2[48][6] + coeff0_49*dmats2[49][6] + coeff0_50*dmats2[50][6] + coeff0_51*dmats2[51][6] + coeff0_52*dmats2[52][6] + coeff0_53*dmats2[53][6] + coeff0_54*dmats2[54][6] + coeff0_55*dmats2[55][6];
          new_coeff0_7 = coeff0_0*dmats2[0][7] + coeff0_1*dmats2[1][7] + coeff0_2*dmats2[2][7] + coeff0_3*dmats2[3][7] + coeff0_4*dmats2[4][7] + coeff0_5*dmats2[5][7] + coeff0_6*dmats2[6][7] + coeff0_7*dmats2[7][7] + coeff0_8*dmats2[8][7] + coeff0_9*dmats2[9][7] + coeff0_10*dmats2[10][7] + coeff0_11*dmats2[11][7] + coeff0_12*dmats2[12][7] + coeff0_13*dmats2[13][7] + coeff0_14*dmats2[14][7] + coeff0_15*dmats2[15][7] + coeff0_16*dmats2[16][7] + coeff0_17*dmats2[17][7] + coeff0_18*dmats2[18][7] + coeff0_19*dmats2[19][7] + coeff0_20*dmats2[20][7] + coeff0_21*dmats2[21][7] + coeff0_22*dmats2[22][7] + coeff0_23*dmats2[23][7] + coeff0_24*dmats2[24][7] + coeff0_25*dmats2[25][7] + coeff0_26*dmats2[26][7] + coeff0_27*dmats2[27][7] + coeff0_28*dmats2[28][7] + coeff0_29*dmats2[29][7] + coeff0_30*dmats2[30][7] + coeff0_31*dmats2[31][7] + coeff0_32*dmats2[32][7] + coeff0_33*dmats2[33][7] + coeff0_34*dmats2[34][7] + coeff0_35*dmats2[35][7] + coeff0_36*dmats2[36][7] + coeff0_37*dmats2[37][7] + coeff0_38*dmats2[38][7] + coeff0_39*dmats2[39][7] + coeff0_40*dmats2[40][7] + coeff0_41*dmats2[41][7] + coeff0_42*dmats2[42][7] + coeff0_43*dmats2[43][7] + coeff0_44*dmats2[44][7] + coeff0_45*dmats2[45][7] + coeff0_46*dmats2[46][7] + coeff0_47*dmats2[47][7] + coeff0_48*dmats2[48][7] + coeff0_49*dmats2[49][7] + coeff0_50*dmats2[50][7] + coeff0_51*dmats2[51][7] + coeff0_52*dmats2[52][7] + coeff0_53*dmats2[53][7] + coeff0_54*dmats2[54][7] + coeff0_55*dmats2[55][7];
          new_coeff0_8 = coeff0_0*dmats2[0][8] + coeff0_1*dmats2[1][8] + coeff0_2*dmats2[2][8] + coeff0_3*dmats2[3][8] + coeff0_4*dmats2[4][8] + coeff0_5*dmats2[5][8] + coeff0_6*dmats2[6][8] + coeff0_7*dmats2[7][8] + coeff0_8*dmats2[8][8] + coeff0_9*dmats2[9][8] + coeff0_10*dmats2[10][8] + coeff0_11*dmats2[11][8] + coeff0_12*dmats2[12][8] + coeff0_13*dmats2[13][8] + coeff0_14*dmats2[14][8] + coeff0_15*dmats2[15][8] + coeff0_16*dmats2[16][8] + coeff0_17*dmats2[17][8] + coeff0_18*dmats2[18][8] + coeff0_19*dmats2[19][8] + coeff0_20*dmats2[20][8] + coeff0_21*dmats2[21][8] + coeff0_22*dmats2[22][8] + coeff0_23*dmats2[23][8] + coeff0_24*dmats2[24][8] + coeff0_25*dmats2[25][8] + coeff0_26*dmats2[26][8] + coeff0_27*dmats2[27][8] + coeff0_28*dmats2[28][8] + coeff0_29*dmats2[29][8] + coeff0_30*dmats2[30][8] + coeff0_31*dmats2[31][8] + coeff0_32*dmats2[32][8] + coeff0_33*dmats2[33][8] + coeff0_34*dmats2[34][8] + coeff0_35*dmats2[35][8] + coeff0_36*dmats2[36][8] + coeff0_37*dmats2[37][8] + coeff0_38*dmats2[38][8] + coeff0_39*dmats2[39][8] + coeff0_40*dmats2[40][8] + coeff0_41*dmats2[41][8] + coeff0_42*dmats2[42][8] + coeff0_43*dmats2[43][8] + coeff0_44*dmats2[44][8] + coeff0_45*dmats2[45][8] + coeff0_46*dmats2[46][8] + coeff0_47*dmats2[47][8] + coeff0_48*dmats2[48][8] + coeff0_49*dmats2[49][8] + coeff0_50*dmats2[50][8] + coeff0_51*dmats2[51][8] + coeff0_52*dmats2[52][8] + coeff0_53*dmats2[53][8] + coeff0_54*dmats2[54][8] + coeff0_55*dmats2[55][8];
          new_coeff0_9 = coeff0_0*dmats2[0][9] + coeff0_1*dmats2[1][9] + coeff0_2*dmats2[2][9] + coeff0_3*dmats2[3][9] + coeff0_4*dmats2[4][9] + coeff0_5*dmats2[5][9] + coeff0_6*dmats2[6][9] + coeff0_7*dmats2[7][9] + coeff0_8*dmats2[8][9] + coeff0_9*dmats2[9][9] + coeff0_10*dmats2[10][9] + coeff0_11*dmats2[11][9] + coeff0_12*dmats2[12][9] + coeff0_13*dmats2[13][9] + coeff0_14*dmats2[14][9] + coeff0_15*dmats2[15][9] + coeff0_16*dmats2[16][9] + coeff0_17*dmats2[17][9] + coeff0_18*dmats2[18][9] + coeff0_19*dmats2[19][9] + coeff0_20*dmats2[20][9] + coeff0_21*dmats2[21][9] + coeff0_22*dmats2[22][9] + coeff0_23*dmats2[23][9] + coeff0_24*dmats2[24][9] + coeff0_25*dmats2[25][9] + coeff0_26*dmats2[26][9] + coeff0_27*dmats2[27][9] + coeff0_28*dmats2[28][9] + coeff0_29*dmats2[29][9] + coeff0_30*dmats2[30][9] + coeff0_31*dmats2[31][9] + coeff0_32*dmats2[32][9] + coeff0_33*dmats2[33][9] + coeff0_34*dmats2[34][9] + coeff0_35*dmats2[35][9] + coeff0_36*dmats2[36][9] + coeff0_37*dmats2[37][9] + coeff0_38*dmats2[38][9] + coeff0_39*dmats2[39][9] + coeff0_40*dmats2[40][9] + coeff0_41*dmats2[41][9] + coeff0_42*dmats2[42][9] + coeff0_43*dmats2[43][9] + coeff0_44*dmats2[44][9] + coeff0_45*dmats2[45][9] + coeff0_46*dmats2[46][9] + coeff0_47*dmats2[47][9] + coeff0_48*dmats2[48][9] + coeff0_49*dmats2[49][9] + coeff0_50*dmats2[50][9] + coeff0_51*dmats2[51][9] + coeff0_52*dmats2[52][9] + coeff0_53*dmats2[53][9] + coeff0_54*dmats2[54][9] + coeff0_55*dmats2[55][9];
          new_coeff0_10 = coeff0_0*dmats2[0][10] + coeff0_1*dmats2[1][10] + coeff0_2*dmats2[2][10] + coeff0_3*dmats2[3][10] + coeff0_4*dmats2[4][10] + coeff0_5*dmats2[5][10] + coeff0_6*dmats2[6][10] + coeff0_7*dmats2[7][10] + coeff0_8*dmats2[8][10] + coeff0_9*dmats2[9][10] + coeff0_10*dmats2[10][10] + coeff0_11*dmats2[11][10] + coeff0_12*dmats2[12][10] + coeff0_13*dmats2[13][10] + coeff0_14*dmats2[14][10] + coeff0_15*dmats2[15][10] + coeff0_16*dmats2[16][10] + coeff0_17*dmats2[17][10] + coeff0_18*dmats2[18][10] + coeff0_19*dmats2[19][10] + coeff0_20*dmats2[20][10] + coeff0_21*dmats2[21][10] + coeff0_22*dmats2[22][10] + coeff0_23*dmats2[23][10] + coeff0_24*dmats2[24][10] + coeff0_25*dmats2[25][10] + coeff0_26*dmats2[26][10] + coeff0_27*dmats2[27][10] + coeff0_28*dmats2[28][10] + coeff0_29*dmats2[29][10] + coeff0_30*dmats2[30][10] + coeff0_31*dmats2[31][10] + coeff0_32*dmats2[32][10] + coeff0_33*dmats2[33][10] + coeff0_34*dmats2[34][10] + coeff0_35*dmats2[35][10] + coeff0_36*dmats2[36][10] + coeff0_37*dmats2[37][10] + coeff0_38*dmats2[38][10] + coeff0_39*dmats2[39][10] + coeff0_40*dmats2[40][10] + coeff0_41*dmats2[41][10] + coeff0_42*dmats2[42][10] + coeff0_43*dmats2[43][10] + coeff0_44*dmats2[44][10] + coeff0_45*dmats2[45][10] + coeff0_46*dmats2[46][10] + coeff0_47*dmats2[47][10] + coeff0_48*dmats2[48][10] + coeff0_49*dmats2[49][10] + coeff0_50*dmats2[50][10] + coeff0_51*dmats2[51][10] + coeff0_52*dmats2[52][10] + coeff0_53*dmats2[53][10] + coeff0_54*dmats2[54][10] + coeff0_55*dmats2[55][10];
          new_coeff0_11 = coeff0_0*dmats2[0][11] + coeff0_1*dmats2[1][11] + coeff0_2*dmats2[2][11] + coeff0_3*dmats2[3][11] + coeff0_4*dmats2[4][11] + coeff0_5*dmats2[5][11] + coeff0_6*dmats2[6][11] + coeff0_7*dmats2[7][11] + coeff0_8*dmats2[8][11] + coeff0_9*dmats2[9][11] + coeff0_10*dmats2[10][11] + coeff0_11*dmats2[11][11] + coeff0_12*dmats2[12][11] + coeff0_13*dmats2[13][11] + coeff0_14*dmats2[14][11] + coeff0_15*dmats2[15][11] + coeff0_16*dmats2[16][11] + coeff0_17*dmats2[17][11] + coeff0_18*dmats2[18][11] + coeff0_19*dmats2[19][11] + coeff0_20*dmats2[20][11] + coeff0_21*dmats2[21][11] + coeff0_22*dmats2[22][11] + coeff0_23*dmats2[23][11] + coeff0_24*dmats2[24][11] + coeff0_25*dmats2[25][11] + coeff0_26*dmats2[26][11] + coeff0_27*dmats2[27][11] + coeff0_28*dmats2[28][11] + coeff0_29*dmats2[29][11] + coeff0_30*dmats2[30][11] + coeff0_31*dmats2[31][11] + coeff0_32*dmats2[32][11] + coeff0_33*dmats2[33][11] + coeff0_34*dmats2[34][11] + coeff0_35*dmats2[35][11] + coeff0_36*dmats2[36][11] + coeff0_37*dmats2[37][11] + coeff0_38*dmats2[38][11] + coeff0_39*dmats2[39][11] + coeff0_40*dmats2[40][11] + coeff0_41*dmats2[41][11] + coeff0_42*dmats2[42][11] + coeff0_43*dmats2[43][11] + coeff0_44*dmats2[44][11] + coeff0_45*dmats2[45][11] + coeff0_46*dmats2[46][11] + coeff0_47*dmats2[47][11] + coeff0_48*dmats2[48][11] + coeff0_49*dmats2[49][11] + coeff0_50*dmats2[50][11] + coeff0_51*dmats2[51][11] + coeff0_52*dmats2[52][11] + coeff0_53*dmats2[53][11] + coeff0_54*dmats2[54][11] + coeff0_55*dmats2[55][11];
          new_coeff0_12 = coeff0_0*dmats2[0][12] + coeff0_1*dmats2[1][12] + coeff0_2*dmats2[2][12] + coeff0_3*dmats2[3][12] + coeff0_4*dmats2[4][12] + coeff0_5*dmats2[5][12] + coeff0_6*dmats2[6][12] + coeff0_7*dmats2[7][12] + coeff0_8*dmats2[8][12] + coeff0_9*dmats2[9][12] + coeff0_10*dmats2[10][12] + coeff0_11*dmats2[11][12] + coeff0_12*dmats2[12][12] + coeff0_13*dmats2[13][12] + coeff0_14*dmats2[14][12] + coeff0_15*dmats2[15][12] + coeff0_16*dmats2[16][12] + coeff0_17*dmats2[17][12] + coeff0_18*dmats2[18][12] + coeff0_19*dmats2[19][12] + coeff0_20*dmats2[20][12] + coeff0_21*dmats2[21][12] + coeff0_22*dmats2[22][12] + coeff0_23*dmats2[23][12] + coeff0_24*dmats2[24][12] + coeff0_25*dmats2[25][12] + coeff0_26*dmats2[26][12] + coeff0_27*dmats2[27][12] + coeff0_28*dmats2[28][12] + coeff0_29*dmats2[29][12] + coeff0_30*dmats2[30][12] + coeff0_31*dmats2[31][12] + coeff0_32*dmats2[32][12] + coeff0_33*dmats2[33][12] + coeff0_34*dmats2[34][12] + coeff0_35*dmats2[35][12] + coeff0_36*dmats2[36][12] + coeff0_37*dmats2[37][12] + coeff0_38*dmats2[38][12] + coeff0_39*dmats2[39][12] + coeff0_40*dmats2[40][12] + coeff0_41*dmats2[41][12] + coeff0_42*dmats2[42][12] + coeff0_43*dmats2[43][12] + coeff0_44*dmats2[44][12] + coeff0_45*dmats2[45][12] + coeff0_46*dmats2[46][12] + coeff0_47*dmats2[47][12] + coeff0_48*dmats2[48][12] + coeff0_49*dmats2[49][12] + coeff0_50*dmats2[50][12] + coeff0_51*dmats2[51][12] + coeff0_52*dmats2[52][12] + coeff0_53*dmats2[53][12] + coeff0_54*dmats2[54][12] + coeff0_55*dmats2[55][12];
          new_coeff0_13 = coeff0_0*dmats2[0][13] + coeff0_1*dmats2[1][13] + coeff0_2*dmats2[2][13] + coeff0_3*dmats2[3][13] + coeff0_4*dmats2[4][13] + coeff0_5*dmats2[5][13] + coeff0_6*dmats2[6][13] + coeff0_7*dmats2[7][13] + coeff0_8*dmats2[8][13] + coeff0_9*dmats2[9][13] + coeff0_10*dmats2[10][13] + coeff0_11*dmats2[11][13] + coeff0_12*dmats2[12][13] + coeff0_13*dmats2[13][13] + coeff0_14*dmats2[14][13] + coeff0_15*dmats2[15][13] + coeff0_16*dmats2[16][13] + coeff0_17*dmats2[17][13] + coeff0_18*dmats2[18][13] + coeff0_19*dmats2[19][13] + coeff0_20*dmats2[20][13] + coeff0_21*dmats2[21][13] + coeff0_22*dmats2[22][13] + coeff0_23*dmats2[23][13] + coeff0_24*dmats2[24][13] + coeff0_25*dmats2[25][13] + coeff0_26*dmats2[26][13] + coeff0_27*dmats2[27][13] + coeff0_28*dmats2[28][13] + coeff0_29*dmats2[29][13] + coeff0_30*dmats2[30][13] + coeff0_31*dmats2[31][13] + coeff0_32*dmats2[32][13] + coeff0_33*dmats2[33][13] + coeff0_34*dmats2[34][13] + coeff0_35*dmats2[35][13] + coeff0_36*dmats2[36][13] + coeff0_37*dmats2[37][13] + coeff0_38*dmats2[38][13] + coeff0_39*dmats2[39][13] + coeff0_40*dmats2[40][13] + coeff0_41*dmats2[41][13] + coeff0_42*dmats2[42][13] + coeff0_43*dmats2[43][13] + coeff0_44*dmats2[44][13] + coeff0_45*dmats2[45][13] + coeff0_46*dmats2[46][13] + coeff0_47*dmats2[47][13] + coeff0_48*dmats2[48][13] + coeff0_49*dmats2[49][13] + coeff0_50*dmats2[50][13] + coeff0_51*dmats2[51][13] + coeff0_52*dmats2[52][13] + coeff0_53*dmats2[53][13] + coeff0_54*dmats2[54][13] + coeff0_55*dmats2[55][13];
          new_coeff0_14 = coeff0_0*dmats2[0][14] + coeff0_1*dmats2[1][14] + coeff0_2*dmats2[2][14] + coeff0_3*dmats2[3][14] + coeff0_4*dmats2[4][14] + coeff0_5*dmats2[5][14] + coeff0_6*dmats2[6][14] + coeff0_7*dmats2[7][14] + coeff0_8*dmats2[8][14] + coeff0_9*dmats2[9][14] + coeff0_10*dmats2[10][14] + coeff0_11*dmats2[11][14] + coeff0_12*dmats2[12][14] + coeff0_13*dmats2[13][14] + coeff0_14*dmats2[14][14] + coeff0_15*dmats2[15][14] + coeff0_16*dmats2[16][14] + coeff0_17*dmats2[17][14] + coeff0_18*dmats2[18][14] + coeff0_19*dmats2[19][14] + coeff0_20*dmats2[20][14] + coeff0_21*dmats2[21][14] + coeff0_22*dmats2[22][14] + coeff0_23*dmats2[23][14] + coeff0_24*dmats2[24][14] + coeff0_25*dmats2[25][14] + coeff0_26*dmats2[26][14] + coeff0_27*dmats2[27][14] + coeff0_28*dmats2[28][14] + coeff0_29*dmats2[29][14] + coeff0_30*dmats2[30][14] + coeff0_31*dmats2[31][14] + coeff0_32*dmats2[32][14] + coeff0_33*dmats2[33][14] + coeff0_34*dmats2[34][14] + coeff0_35*dmats2[35][14] + coeff0_36*dmats2[36][14] + coeff0_37*dmats2[37][14] + coeff0_38*dmats2[38][14] + coeff0_39*dmats2[39][14] + coeff0_40*dmats2[40][14] + coeff0_41*dmats2[41][14] + coeff0_42*dmats2[42][14] + coeff0_43*dmats2[43][14] + coeff0_44*dmats2[44][14] + coeff0_45*dmats2[45][14] + coeff0_46*dmats2[46][14] + coeff0_47*dmats2[47][14] + coeff0_48*dmats2[48][14] + coeff0_49*dmats2[49][14] + coeff0_50*dmats2[50][14] + coeff0_51*dmats2[51][14] + coeff0_52*dmats2[52][14] + coeff0_53*dmats2[53][14] + coeff0_54*dmats2[54][14] + coeff0_55*dmats2[55][14];
          new_coeff0_15 = coeff0_0*dmats2[0][15] + coeff0_1*dmats2[1][15] + coeff0_2*dmats2[2][15] + coeff0_3*dmats2[3][15] + coeff0_4*dmats2[4][15] + coeff0_5*dmats2[5][15] + coeff0_6*dmats2[6][15] + coeff0_7*dmats2[7][15] + coeff0_8*dmats2[8][15] + coeff0_9*dmats2[9][15] + coeff0_10*dmats2[10][15] + coeff0_11*dmats2[11][15] + coeff0_12*dmats2[12][15] + coeff0_13*dmats2[13][15] + coeff0_14*dmats2[14][15] + coeff0_15*dmats2[15][15] + coeff0_16*dmats2[16][15] + coeff0_17*dmats2[17][15] + coeff0_18*dmats2[18][15] + coeff0_19*dmats2[19][15] + coeff0_20*dmats2[20][15] + coeff0_21*dmats2[21][15] + coeff0_22*dmats2[22][15] + coeff0_23*dmats2[23][15] + coeff0_24*dmats2[24][15] + coeff0_25*dmats2[25][15] + coeff0_26*dmats2[26][15] + coeff0_27*dmats2[27][15] + coeff0_28*dmats2[28][15] + coeff0_29*dmats2[29][15] + coeff0_30*dmats2[30][15] + coeff0_31*dmats2[31][15] + coeff0_32*dmats2[32][15] + coeff0_33*dmats2[33][15] + coeff0_34*dmats2[34][15] + coeff0_35*dmats2[35][15] + coeff0_36*dmats2[36][15] + coeff0_37*dmats2[37][15] + coeff0_38*dmats2[38][15] + coeff0_39*dmats2[39][15] + coeff0_40*dmats2[40][15] + coeff0_41*dmats2[41][15] + coeff0_42*dmats2[42][15] + coeff0_43*dmats2[43][15] + coeff0_44*dmats2[44][15] + coeff0_45*dmats2[45][15] + coeff0_46*dmats2[46][15] + coeff0_47*dmats2[47][15] + coeff0_48*dmats2[48][15] + coeff0_49*dmats2[49][15] + coeff0_50*dmats2[50][15] + coeff0_51*dmats2[51][15] + coeff0_52*dmats2[52][15] + coeff0_53*dmats2[53][15] + coeff0_54*dmats2[54][15] + coeff0_55*dmats2[55][15];
          new_coeff0_16 = coeff0_0*dmats2[0][16] + coeff0_1*dmats2[1][16] + coeff0_2*dmats2[2][16] + coeff0_3*dmats2[3][16] + coeff0_4*dmats2[4][16] + coeff0_5*dmats2[5][16] + coeff0_6*dmats2[6][16] + coeff0_7*dmats2[7][16] + coeff0_8*dmats2[8][16] + coeff0_9*dmats2[9][16] + coeff0_10*dmats2[10][16] + coeff0_11*dmats2[11][16] + coeff0_12*dmats2[12][16] + coeff0_13*dmats2[13][16] + coeff0_14*dmats2[14][16] + coeff0_15*dmats2[15][16] + coeff0_16*dmats2[16][16] + coeff0_17*dmats2[17][16] + coeff0_18*dmats2[18][16] + coeff0_19*dmats2[19][16] + coeff0_20*dmats2[20][16] + coeff0_21*dmats2[21][16] + coeff0_22*dmats2[22][16] + coeff0_23*dmats2[23][16] + coeff0_24*dmats2[24][16] + coeff0_25*dmats2[25][16] + coeff0_26*dmats2[26][16] + coeff0_27*dmats2[27][16] + coeff0_28*dmats2[28][16] + coeff0_29*dmats2[29][16] + coeff0_30*dmats2[30][16] + coeff0_31*dmats2[31][16] + coeff0_32*dmats2[32][16] + coeff0_33*dmats2[33][16] + coeff0_34*dmats2[34][16] + coeff0_35*dmats2[35][16] + coeff0_36*dmats2[36][16] + coeff0_37*dmats2[37][16] + coeff0_38*dmats2[38][16] + coeff0_39*dmats2[39][16] + coeff0_40*dmats2[40][16] + coeff0_41*dmats2[41][16] + coeff0_42*dmats2[42][16] + coeff0_43*dmats2[43][16] + coeff0_44*dmats2[44][16] + coeff0_45*dmats2[45][16] + coeff0_46*dmats2[46][16] + coeff0_47*dmats2[47][16] + coeff0_48*dmats2[48][16] + coeff0_49*dmats2[49][16] + coeff0_50*dmats2[50][16] + coeff0_51*dmats2[51][16] + coeff0_52*dmats2[52][16] + coeff0_53*dmats2[53][16] + coeff0_54*dmats2[54][16] + coeff0_55*dmats2[55][16];
          new_coeff0_17 = coeff0_0*dmats2[0][17] + coeff0_1*dmats2[1][17] + coeff0_2*dmats2[2][17] + coeff0_3*dmats2[3][17] + coeff0_4*dmats2[4][17] + coeff0_5*dmats2[5][17] + coeff0_6*dmats2[6][17] + coeff0_7*dmats2[7][17] + coeff0_8*dmats2[8][17] + coeff0_9*dmats2[9][17] + coeff0_10*dmats2[10][17] + coeff0_11*dmats2[11][17] + coeff0_12*dmats2[12][17] + coeff0_13*dmats2[13][17] + coeff0_14*dmats2[14][17] + coeff0_15*dmats2[15][17] + coeff0_16*dmats2[16][17] + coeff0_17*dmats2[17][17] + coeff0_18*dmats2[18][17] + coeff0_19*dmats2[19][17] + coeff0_20*dmats2[20][17] + coeff0_21*dmats2[21][17] + coeff0_22*dmats2[22][17] + coeff0_23*dmats2[23][17] + coeff0_24*dmats2[24][17] + coeff0_25*dmats2[25][17] + coeff0_26*dmats2[26][17] + coeff0_27*dmats2[27][17] + coeff0_28*dmats2[28][17] + coeff0_29*dmats2[29][17] + coeff0_30*dmats2[30][17] + coeff0_31*dmats2[31][17] + coeff0_32*dmats2[32][17] + coeff0_33*dmats2[33][17] + coeff0_34*dmats2[34][17] + coeff0_35*dmats2[35][17] + coeff0_36*dmats2[36][17] + coeff0_37*dmats2[37][17] + coeff0_38*dmats2[38][17] + coeff0_39*dmats2[39][17] + coeff0_40*dmats2[40][17] + coeff0_41*dmats2[41][17] + coeff0_42*dmats2[42][17] + coeff0_43*dmats2[43][17] + coeff0_44*dmats2[44][17] + coeff0_45*dmats2[45][17] + coeff0_46*dmats2[46][17] + coeff0_47*dmats2[47][17] + coeff0_48*dmats2[48][17] + coeff0_49*dmats2[49][17] + coeff0_50*dmats2[50][17] + coeff0_51*dmats2[51][17] + coeff0_52*dmats2[52][17] + coeff0_53*dmats2[53][17] + coeff0_54*dmats2[54][17] + coeff0_55*dmats2[55][17];
          new_coeff0_18 = coeff0_0*dmats2[0][18] + coeff0_1*dmats2[1][18] + coeff0_2*dmats2[2][18] + coeff0_3*dmats2[3][18] + coeff0_4*dmats2[4][18] + coeff0_5*dmats2[5][18] + coeff0_6*dmats2[6][18] + coeff0_7*dmats2[7][18] + coeff0_8*dmats2[8][18] + coeff0_9*dmats2[9][18] + coeff0_10*dmats2[10][18] + coeff0_11*dmats2[11][18] + coeff0_12*dmats2[12][18] + coeff0_13*dmats2[13][18] + coeff0_14*dmats2[14][18] + coeff0_15*dmats2[15][18] + coeff0_16*dmats2[16][18] + coeff0_17*dmats2[17][18] + coeff0_18*dmats2[18][18] + coeff0_19*dmats2[19][18] + coeff0_20*dmats2[20][18] + coeff0_21*dmats2[21][18] + coeff0_22*dmats2[22][18] + coeff0_23*dmats2[23][18] + coeff0_24*dmats2[24][18] + coeff0_25*dmats2[25][18] + coeff0_26*dmats2[26][18] + coeff0_27*dmats2[27][18] + coeff0_28*dmats2[28][18] + coeff0_29*dmats2[29][18] + coeff0_30*dmats2[30][18] + coeff0_31*dmats2[31][18] + coeff0_32*dmats2[32][18] + coeff0_33*dmats2[33][18] + coeff0_34*dmats2[34][18] + coeff0_35*dmats2[35][18] + coeff0_36*dmats2[36][18] + coeff0_37*dmats2[37][18] + coeff0_38*dmats2[38][18] + coeff0_39*dmats2[39][18] + coeff0_40*dmats2[40][18] + coeff0_41*dmats2[41][18] + coeff0_42*dmats2[42][18] + coeff0_43*dmats2[43][18] + coeff0_44*dmats2[44][18] + coeff0_45*dmats2[45][18] + coeff0_46*dmats2[46][18] + coeff0_47*dmats2[47][18] + coeff0_48*dmats2[48][18] + coeff0_49*dmats2[49][18] + coeff0_50*dmats2[50][18] + coeff0_51*dmats2[51][18] + coeff0_52*dmats2[52][18] + coeff0_53*dmats2[53][18] + coeff0_54*dmats2[54][18] + coeff0_55*dmats2[55][18];
          new_coeff0_19 = coeff0_0*dmats2[0][19] + coeff0_1*dmats2[1][19] + coeff0_2*dmats2[2][19] + coeff0_3*dmats2[3][19] + coeff0_4*dmats2[4][19] + coeff0_5*dmats2[5][19] + coeff0_6*dmats2[6][19] + coeff0_7*dmats2[7][19] + coeff0_8*dmats2[8][19] + coeff0_9*dmats2[9][19] + coeff0_10*dmats2[10][19] + coeff0_11*dmats2[11][19] + coeff0_12*dmats2[12][19] + coeff0_13*dmats2[13][19] + coeff0_14*dmats2[14][19] + coeff0_15*dmats2[15][19] + coeff0_16*dmats2[16][19] + coeff0_17*dmats2[17][19] + coeff0_18*dmats2[18][19] + coeff0_19*dmats2[19][19] + coeff0_20*dmats2[20][19] + coeff0_21*dmats2[21][19] + coeff0_22*dmats2[22][19] + coeff0_23*dmats2[23][19] + coeff0_24*dmats2[24][19] + coeff0_25*dmats2[25][19] + coeff0_26*dmats2[26][19] + coeff0_27*dmats2[27][19] + coeff0_28*dmats2[28][19] + coeff0_29*dmats2[29][19] + coeff0_30*dmats2[30][19] + coeff0_31*dmats2[31][19] + coeff0_32*dmats2[32][19] + coeff0_33*dmats2[33][19] + coeff0_34*dmats2[34][19] + coeff0_35*dmats2[35][19] + coeff0_36*dmats2[36][19] + coeff0_37*dmats2[37][19] + coeff0_38*dmats2[38][19] + coeff0_39*dmats2[39][19] + coeff0_40*dmats2[40][19] + coeff0_41*dmats2[41][19] + coeff0_42*dmats2[42][19] + coeff0_43*dmats2[43][19] + coeff0_44*dmats2[44][19] + coeff0_45*dmats2[45][19] + coeff0_46*dmats2[46][19] + coeff0_47*dmats2[47][19] + coeff0_48*dmats2[48][19] + coeff0_49*dmats2[49][19] + coeff0_50*dmats2[50][19] + coeff0_51*dmats2[51][19] + coeff0_52*dmats2[52][19] + coeff0_53*dmats2[53][19] + coeff0_54*dmats2[54][19] + coeff0_55*dmats2[55][19];
          new_coeff0_20 = coeff0_0*dmats2[0][20] + coeff0_1*dmats2[1][20] + coeff0_2*dmats2[2][20] + coeff0_3*dmats2[3][20] + coeff0_4*dmats2[4][20] + coeff0_5*dmats2[5][20] + coeff0_6*dmats2[6][20] + coeff0_7*dmats2[7][20] + coeff0_8*dmats2[8][20] + coeff0_9*dmats2[9][20] + coeff0_10*dmats2[10][20] + coeff0_11*dmats2[11][20] + coeff0_12*dmats2[12][20] + coeff0_13*dmats2[13][20] + coeff0_14*dmats2[14][20] + coeff0_15*dmats2[15][20] + coeff0_16*dmats2[16][20] + coeff0_17*dmats2[17][20] + coeff0_18*dmats2[18][20] + coeff0_19*dmats2[19][20] + coeff0_20*dmats2[20][20] + coeff0_21*dmats2[21][20] + coeff0_22*dmats2[22][20] + coeff0_23*dmats2[23][20] + coeff0_24*dmats2[24][20] + coeff0_25*dmats2[25][20] + coeff0_26*dmats2[26][20] + coeff0_27*dmats2[27][20] + coeff0_28*dmats2[28][20] + coeff0_29*dmats2[29][20] + coeff0_30*dmats2[30][20] + coeff0_31*dmats2[31][20] + coeff0_32*dmats2[32][20] + coeff0_33*dmats2[33][20] + coeff0_34*dmats2[34][20] + coeff0_35*dmats2[35][20] + coeff0_36*dmats2[36][20] + coeff0_37*dmats2[37][20] + coeff0_38*dmats2[38][20] + coeff0_39*dmats2[39][20] + coeff0_40*dmats2[40][20] + coeff0_41*dmats2[41][20] + coeff0_42*dmats2[42][20] + coeff0_43*dmats2[43][20] + coeff0_44*dmats2[44][20] + coeff0_45*dmats2[45][20] + coeff0_46*dmats2[46][20] + coeff0_47*dmats2[47][20] + coeff0_48*dmats2[48][20] + coeff0_49*dmats2[49][20] + coeff0_50*dmats2[50][20] + coeff0_51*dmats2[51][20] + coeff0_52*dmats2[52][20] + coeff0_53*dmats2[53][20] + coeff0_54*dmats2[54][20] + coeff0_55*dmats2[55][20];
          new_coeff0_21 = coeff0_0*dmats2[0][21] + coeff0_1*dmats2[1][21] + coeff0_2*dmats2[2][21] + coeff0_3*dmats2[3][21] + coeff0_4*dmats2[4][21] + coeff0_5*dmats2[5][21] + coeff0_6*dmats2[6][21] + coeff0_7*dmats2[7][21] + coeff0_8*dmats2[8][21] + coeff0_9*dmats2[9][21] + coeff0_10*dmats2[10][21] + coeff0_11*dmats2[11][21] + coeff0_12*dmats2[12][21] + coeff0_13*dmats2[13][21] + coeff0_14*dmats2[14][21] + coeff0_15*dmats2[15][21] + coeff0_16*dmats2[16][21] + coeff0_17*dmats2[17][21] + coeff0_18*dmats2[18][21] + coeff0_19*dmats2[19][21] + coeff0_20*dmats2[20][21] + coeff0_21*dmats2[21][21] + coeff0_22*dmats2[22][21] + coeff0_23*dmats2[23][21] + coeff0_24*dmats2[24][21] + coeff0_25*dmats2[25][21] + coeff0_26*dmats2[26][21] + coeff0_27*dmats2[27][21] + coeff0_28*dmats2[28][21] + coeff0_29*dmats2[29][21] + coeff0_30*dmats2[30][21] + coeff0_31*dmats2[31][21] + coeff0_32*dmats2[32][21] + coeff0_33*dmats2[33][21] + coeff0_34*dmats2[34][21] + coeff0_35*dmats2[35][21] + coeff0_36*dmats2[36][21] + coeff0_37*dmats2[37][21] + coeff0_38*dmats2[38][21] + coeff0_39*dmats2[39][21] + coeff0_40*dmats2[40][21] + coeff0_41*dmats2[41][21] + coeff0_42*dmats2[42][21] + coeff0_43*dmats2[43][21] + coeff0_44*dmats2[44][21] + coeff0_45*dmats2[45][21] + coeff0_46*dmats2[46][21] + coeff0_47*dmats2[47][21] + coeff0_48*dmats2[48][21] + coeff0_49*dmats2[49][21] + coeff0_50*dmats2[50][21] + coeff0_51*dmats2[51][21] + coeff0_52*dmats2[52][21] + coeff0_53*dmats2[53][21] + coeff0_54*dmats2[54][21] + coeff0_55*dmats2[55][21];
          new_coeff0_22 = coeff0_0*dmats2[0][22] + coeff0_1*dmats2[1][22] + coeff0_2*dmats2[2][22] + coeff0_3*dmats2[3][22] + coeff0_4*dmats2[4][22] + coeff0_5*dmats2[5][22] + coeff0_6*dmats2[6][22] + coeff0_7*dmats2[7][22] + coeff0_8*dmats2[8][22] + coeff0_9*dmats2[9][22] + coeff0_10*dmats2[10][22] + coeff0_11*dmats2[11][22] + coeff0_12*dmats2[12][22] + coeff0_13*dmats2[13][22] + coeff0_14*dmats2[14][22] + coeff0_15*dmats2[15][22] + coeff0_16*dmats2[16][22] + coeff0_17*dmats2[17][22] + coeff0_18*dmats2[18][22] + coeff0_19*dmats2[19][22] + coeff0_20*dmats2[20][22] + coeff0_21*dmats2[21][22] + coeff0_22*dmats2[22][22] + coeff0_23*dmats2[23][22] + coeff0_24*dmats2[24][22] + coeff0_25*dmats2[25][22] + coeff0_26*dmats2[26][22] + coeff0_27*dmats2[27][22] + coeff0_28*dmats2[28][22] + coeff0_29*dmats2[29][22] + coeff0_30*dmats2[30][22] + coeff0_31*dmats2[31][22] + coeff0_32*dmats2[32][22] + coeff0_33*dmats2[33][22] + coeff0_34*dmats2[34][22] + coeff0_35*dmats2[35][22] + coeff0_36*dmats2[36][22] + coeff0_37*dmats2[37][22] + coeff0_38*dmats2[38][22] + coeff0_39*dmats2[39][22] + coeff0_40*dmats2[40][22] + coeff0_41*dmats2[41][22] + coeff0_42*dmats2[42][22] + coeff0_43*dmats2[43][22] + coeff0_44*dmats2[44][22] + coeff0_45*dmats2[45][22] + coeff0_46*dmats2[46][22] + coeff0_47*dmats2[47][22] + coeff0_48*dmats2[48][22] + coeff0_49*dmats2[49][22] + coeff0_50*dmats2[50][22] + coeff0_51*dmats2[51][22] + coeff0_52*dmats2[52][22] + coeff0_53*dmats2[53][22] + coeff0_54*dmats2[54][22] + coeff0_55*dmats2[55][22];
          new_coeff0_23 = coeff0_0*dmats2[0][23] + coeff0_1*dmats2[1][23] + coeff0_2*dmats2[2][23] + coeff0_3*dmats2[3][23] + coeff0_4*dmats2[4][23] + coeff0_5*dmats2[5][23] + coeff0_6*dmats2[6][23] + coeff0_7*dmats2[7][23] + coeff0_8*dmats2[8][23] + coeff0_9*dmats2[9][23] + coeff0_10*dmats2[10][23] + coeff0_11*dmats2[11][23] + coeff0_12*dmats2[12][23] + coeff0_13*dmats2[13][23] + coeff0_14*dmats2[14][23] + coeff0_15*dmats2[15][23] + coeff0_16*dmats2[16][23] + coeff0_17*dmats2[17][23] + coeff0_18*dmats2[18][23] + coeff0_19*dmats2[19][23] + coeff0_20*dmats2[20][23] + coeff0_21*dmats2[21][23] + coeff0_22*dmats2[22][23] + coeff0_23*dmats2[23][23] + coeff0_24*dmats2[24][23] + coeff0_25*dmats2[25][23] + coeff0_26*dmats2[26][23] + coeff0_27*dmats2[27][23] + coeff0_28*dmats2[28][23] + coeff0_29*dmats2[29][23] + coeff0_30*dmats2[30][23] + coeff0_31*dmats2[31][23] + coeff0_32*dmats2[32][23] + coeff0_33*dmats2[33][23] + coeff0_34*dmats2[34][23] + coeff0_35*dmats2[35][23] + coeff0_36*dmats2[36][23] + coeff0_37*dmats2[37][23] + coeff0_38*dmats2[38][23] + coeff0_39*dmats2[39][23] + coeff0_40*dmats2[40][23] + coeff0_41*dmats2[41][23] + coeff0_42*dmats2[42][23] + coeff0_43*dmats2[43][23] + coeff0_44*dmats2[44][23] + coeff0_45*dmats2[45][23] + coeff0_46*dmats2[46][23] + coeff0_47*dmats2[47][23] + coeff0_48*dmats2[48][23] + coeff0_49*dmats2[49][23] + coeff0_50*dmats2[50][23] + coeff0_51*dmats2[51][23] + coeff0_52*dmats2[52][23] + coeff0_53*dmats2[53][23] + coeff0_54*dmats2[54][23] + coeff0_55*dmats2[55][23];
          new_coeff0_24 = coeff0_0*dmats2[0][24] + coeff0_1*dmats2[1][24] + coeff0_2*dmats2[2][24] + coeff0_3*dmats2[3][24] + coeff0_4*dmats2[4][24] + coeff0_5*dmats2[5][24] + coeff0_6*dmats2[6][24] + coeff0_7*dmats2[7][24] + coeff0_8*dmats2[8][24] + coeff0_9*dmats2[9][24] + coeff0_10*dmats2[10][24] + coeff0_11*dmats2[11][24] + coeff0_12*dmats2[12][24] + coeff0_13*dmats2[13][24] + coeff0_14*dmats2[14][24] + coeff0_15*dmats2[15][24] + coeff0_16*dmats2[16][24] + coeff0_17*dmats2[17][24] + coeff0_18*dmats2[18][24] + coeff0_19*dmats2[19][24] + coeff0_20*dmats2[20][24] + coeff0_21*dmats2[21][24] + coeff0_22*dmats2[22][24] + coeff0_23*dmats2[23][24] + coeff0_24*dmats2[24][24] + coeff0_25*dmats2[25][24] + coeff0_26*dmats2[26][24] + coeff0_27*dmats2[27][24] + coeff0_28*dmats2[28][24] + coeff0_29*dmats2[29][24] + coeff0_30*dmats2[30][24] + coeff0_31*dmats2[31][24] + coeff0_32*dmats2[32][24] + coeff0_33*dmats2[33][24] + coeff0_34*dmats2[34][24] + coeff0_35*dmats2[35][24] + coeff0_36*dmats2[36][24] + coeff0_37*dmats2[37][24] + coeff0_38*dmats2[38][24] + coeff0_39*dmats2[39][24] + coeff0_40*dmats2[40][24] + coeff0_41*dmats2[41][24] + coeff0_42*dmats2[42][24] + coeff0_43*dmats2[43][24] + coeff0_44*dmats2[44][24] + coeff0_45*dmats2[45][24] + coeff0_46*dmats2[46][24] + coeff0_47*dmats2[47][24] + coeff0_48*dmats2[48][24] + coeff0_49*dmats2[49][24] + coeff0_50*dmats2[50][24] + coeff0_51*dmats2[51][24] + coeff0_52*dmats2[52][24] + coeff0_53*dmats2[53][24] + coeff0_54*dmats2[54][24] + coeff0_55*dmats2[55][24];
          new_coeff0_25 = coeff0_0*dmats2[0][25] + coeff0_1*dmats2[1][25] + coeff0_2*dmats2[2][25] + coeff0_3*dmats2[3][25] + coeff0_4*dmats2[4][25] + coeff0_5*dmats2[5][25] + coeff0_6*dmats2[6][25] + coeff0_7*dmats2[7][25] + coeff0_8*dmats2[8][25] + coeff0_9*dmats2[9][25] + coeff0_10*dmats2[10][25] + coeff0_11*dmats2[11][25] + coeff0_12*dmats2[12][25] + coeff0_13*dmats2[13][25] + coeff0_14*dmats2[14][25] + coeff0_15*dmats2[15][25] + coeff0_16*dmats2[16][25] + coeff0_17*dmats2[17][25] + coeff0_18*dmats2[18][25] + coeff0_19*dmats2[19][25] + coeff0_20*dmats2[20][25] + coeff0_21*dmats2[21][25] + coeff0_22*dmats2[22][25] + coeff0_23*dmats2[23][25] + coeff0_24*dmats2[24][25] + coeff0_25*dmats2[25][25] + coeff0_26*dmats2[26][25] + coeff0_27*dmats2[27][25] + coeff0_28*dmats2[28][25] + coeff0_29*dmats2[29][25] + coeff0_30*dmats2[30][25] + coeff0_31*dmats2[31][25] + coeff0_32*dmats2[32][25] + coeff0_33*dmats2[33][25] + coeff0_34*dmats2[34][25] + coeff0_35*dmats2[35][25] + coeff0_36*dmats2[36][25] + coeff0_37*dmats2[37][25] + coeff0_38*dmats2[38][25] + coeff0_39*dmats2[39][25] + coeff0_40*dmats2[40][25] + coeff0_41*dmats2[41][25] + coeff0_42*dmats2[42][25] + coeff0_43*dmats2[43][25] + coeff0_44*dmats2[44][25] + coeff0_45*dmats2[45][25] + coeff0_46*dmats2[46][25] + coeff0_47*dmats2[47][25] + coeff0_48*dmats2[48][25] + coeff0_49*dmats2[49][25] + coeff0_50*dmats2[50][25] + coeff0_51*dmats2[51][25] + coeff0_52*dmats2[52][25] + coeff0_53*dmats2[53][25] + coeff0_54*dmats2[54][25] + coeff0_55*dmats2[55][25];
          new_coeff0_26 = coeff0_0*dmats2[0][26] + coeff0_1*dmats2[1][26] + coeff0_2*dmats2[2][26] + coeff0_3*dmats2[3][26] + coeff0_4*dmats2[4][26] + coeff0_5*dmats2[5][26] + coeff0_6*dmats2[6][26] + coeff0_7*dmats2[7][26] + coeff0_8*dmats2[8][26] + coeff0_9*dmats2[9][26] + coeff0_10*dmats2[10][26] + coeff0_11*dmats2[11][26] + coeff0_12*dmats2[12][26] + coeff0_13*dmats2[13][26] + coeff0_14*dmats2[14][26] + coeff0_15*dmats2[15][26] + coeff0_16*dmats2[16][26] + coeff0_17*dmats2[17][26] + coeff0_18*dmats2[18][26] + coeff0_19*dmats2[19][26] + coeff0_20*dmats2[20][26] + coeff0_21*dmats2[21][26] + coeff0_22*dmats2[22][26] + coeff0_23*dmats2[23][26] + coeff0_24*dmats2[24][26] + coeff0_25*dmats2[25][26] + coeff0_26*dmats2[26][26] + coeff0_27*dmats2[27][26] + coeff0_28*dmats2[28][26] + coeff0_29*dmats2[29][26] + coeff0_30*dmats2[30][26] + coeff0_31*dmats2[31][26] + coeff0_32*dmats2[32][26] + coeff0_33*dmats2[33][26] + coeff0_34*dmats2[34][26] + coeff0_35*dmats2[35][26] + coeff0_36*dmats2[36][26] + coeff0_37*dmats2[37][26] + coeff0_38*dmats2[38][26] + coeff0_39*dmats2[39][26] + coeff0_40*dmats2[40][26] + coeff0_41*dmats2[41][26] + coeff0_42*dmats2[42][26] + coeff0_43*dmats2[43][26] + coeff0_44*dmats2[44][26] + coeff0_45*dmats2[45][26] + coeff0_46*dmats2[46][26] + coeff0_47*dmats2[47][26] + coeff0_48*dmats2[48][26] + coeff0_49*dmats2[49][26] + coeff0_50*dmats2[50][26] + coeff0_51*dmats2[51][26] + coeff0_52*dmats2[52][26] + coeff0_53*dmats2[53][26] + coeff0_54*dmats2[54][26] + coeff0_55*dmats2[55][26];
          new_coeff0_27 = coeff0_0*dmats2[0][27] + coeff0_1*dmats2[1][27] + coeff0_2*dmats2[2][27] + coeff0_3*dmats2[3][27] + coeff0_4*dmats2[4][27] + coeff0_5*dmats2[5][27] + coeff0_6*dmats2[6][27] + coeff0_7*dmats2[7][27] + coeff0_8*dmats2[8][27] + coeff0_9*dmats2[9][27] + coeff0_10*dmats2[10][27] + coeff0_11*dmats2[11][27] + coeff0_12*dmats2[12][27] + coeff0_13*dmats2[13][27] + coeff0_14*dmats2[14][27] + coeff0_15*dmats2[15][27] + coeff0_16*dmats2[16][27] + coeff0_17*dmats2[17][27] + coeff0_18*dmats2[18][27] + coeff0_19*dmats2[19][27] + coeff0_20*dmats2[20][27] + coeff0_21*dmats2[21][27] + coeff0_22*dmats2[22][27] + coeff0_23*dmats2[23][27] + coeff0_24*dmats2[24][27] + coeff0_25*dmats2[25][27] + coeff0_26*dmats2[26][27] + coeff0_27*dmats2[27][27] + coeff0_28*dmats2[28][27] + coeff0_29*dmats2[29][27] + coeff0_30*dmats2[30][27] + coeff0_31*dmats2[31][27] + coeff0_32*dmats2[32][27] + coeff0_33*dmats2[33][27] + coeff0_34*dmats2[34][27] + coeff0_35*dmats2[35][27] + coeff0_36*dmats2[36][27] + coeff0_37*dmats2[37][27] + coeff0_38*dmats2[38][27] + coeff0_39*dmats2[39][27] + coeff0_40*dmats2[40][27] + coeff0_41*dmats2[41][27] + coeff0_42*dmats2[42][27] + coeff0_43*dmats2[43][27] + coeff0_44*dmats2[44][27] + coeff0_45*dmats2[45][27] + coeff0_46*dmats2[46][27] + coeff0_47*dmats2[47][27] + coeff0_48*dmats2[48][27] + coeff0_49*dmats2[49][27] + coeff0_50*dmats2[50][27] + coeff0_51*dmats2[51][27] + coeff0_52*dmats2[52][27] + coeff0_53*dmats2[53][27] + coeff0_54*dmats2[54][27] + coeff0_55*dmats2[55][27];
          new_coeff0_28 = coeff0_0*dmats2[0][28] + coeff0_1*dmats2[1][28] + coeff0_2*dmats2[2][28] + coeff0_3*dmats2[3][28] + coeff0_4*dmats2[4][28] + coeff0_5*dmats2[5][28] + coeff0_6*dmats2[6][28] + coeff0_7*dmats2[7][28] + coeff0_8*dmats2[8][28] + coeff0_9*dmats2[9][28] + coeff0_10*dmats2[10][28] + coeff0_11*dmats2[11][28] + coeff0_12*dmats2[12][28] + coeff0_13*dmats2[13][28] + coeff0_14*dmats2[14][28] + coeff0_15*dmats2[15][28] + coeff0_16*dmats2[16][28] + coeff0_17*dmats2[17][28] + coeff0_18*dmats2[18][28] + coeff0_19*dmats2[19][28] + coeff0_20*dmats2[20][28] + coeff0_21*dmats2[21][28] + coeff0_22*dmats2[22][28] + coeff0_23*dmats2[23][28] + coeff0_24*dmats2[24][28] + coeff0_25*dmats2[25][28] + coeff0_26*dmats2[26][28] + coeff0_27*dmats2[27][28] + coeff0_28*dmats2[28][28] + coeff0_29*dmats2[29][28] + coeff0_30*dmats2[30][28] + coeff0_31*dmats2[31][28] + coeff0_32*dmats2[32][28] + coeff0_33*dmats2[33][28] + coeff0_34*dmats2[34][28] + coeff0_35*dmats2[35][28] + coeff0_36*dmats2[36][28] + coeff0_37*dmats2[37][28] + coeff0_38*dmats2[38][28] + coeff0_39*dmats2[39][28] + coeff0_40*dmats2[40][28] + coeff0_41*dmats2[41][28] + coeff0_42*dmats2[42][28] + coeff0_43*dmats2[43][28] + coeff0_44*dmats2[44][28] + coeff0_45*dmats2[45][28] + coeff0_46*dmats2[46][28] + coeff0_47*dmats2[47][28] + coeff0_48*dmats2[48][28] + coeff0_49*dmats2[49][28] + coeff0_50*dmats2[50][28] + coeff0_51*dmats2[51][28] + coeff0_52*dmats2[52][28] + coeff0_53*dmats2[53][28] + coeff0_54*dmats2[54][28] + coeff0_55*dmats2[55][28];
          new_coeff0_29 = coeff0_0*dmats2[0][29] + coeff0_1*dmats2[1][29] + coeff0_2*dmats2[2][29] + coeff0_3*dmats2[3][29] + coeff0_4*dmats2[4][29] + coeff0_5*dmats2[5][29] + coeff0_6*dmats2[6][29] + coeff0_7*dmats2[7][29] + coeff0_8*dmats2[8][29] + coeff0_9*dmats2[9][29] + coeff0_10*dmats2[10][29] + coeff0_11*dmats2[11][29] + coeff0_12*dmats2[12][29] + coeff0_13*dmats2[13][29] + coeff0_14*dmats2[14][29] + coeff0_15*dmats2[15][29] + coeff0_16*dmats2[16][29] + coeff0_17*dmats2[17][29] + coeff0_18*dmats2[18][29] + coeff0_19*dmats2[19][29] + coeff0_20*dmats2[20][29] + coeff0_21*dmats2[21][29] + coeff0_22*dmats2[22][29] + coeff0_23*dmats2[23][29] + coeff0_24*dmats2[24][29] + coeff0_25*dmats2[25][29] + coeff0_26*dmats2[26][29] + coeff0_27*dmats2[27][29] + coeff0_28*dmats2[28][29] + coeff0_29*dmats2[29][29] + coeff0_30*dmats2[30][29] + coeff0_31*dmats2[31][29] + coeff0_32*dmats2[32][29] + coeff0_33*dmats2[33][29] + coeff0_34*dmats2[34][29] + coeff0_35*dmats2[35][29] + coeff0_36*dmats2[36][29] + coeff0_37*dmats2[37][29] + coeff0_38*dmats2[38][29] + coeff0_39*dmats2[39][29] + coeff0_40*dmats2[40][29] + coeff0_41*dmats2[41][29] + coeff0_42*dmats2[42][29] + coeff0_43*dmats2[43][29] + coeff0_44*dmats2[44][29] + coeff0_45*dmats2[45][29] + coeff0_46*dmats2[46][29] + coeff0_47*dmats2[47][29] + coeff0_48*dmats2[48][29] + coeff0_49*dmats2[49][29] + coeff0_50*dmats2[50][29] + coeff0_51*dmats2[51][29] + coeff0_52*dmats2[52][29] + coeff0_53*dmats2[53][29] + coeff0_54*dmats2[54][29] + coeff0_55*dmats2[55][29];
          new_coeff0_30 = coeff0_0*dmats2[0][30] + coeff0_1*dmats2[1][30] + coeff0_2*dmats2[2][30] + coeff0_3*dmats2[3][30] + coeff0_4*dmats2[4][30] + coeff0_5*dmats2[5][30] + coeff0_6*dmats2[6][30] + coeff0_7*dmats2[7][30] + coeff0_8*dmats2[8][30] + coeff0_9*dmats2[9][30] + coeff0_10*dmats2[10][30] + coeff0_11*dmats2[11][30] + coeff0_12*dmats2[12][30] + coeff0_13*dmats2[13][30] + coeff0_14*dmats2[14][30] + coeff0_15*dmats2[15][30] + coeff0_16*dmats2[16][30] + coeff0_17*dmats2[17][30] + coeff0_18*dmats2[18][30] + coeff0_19*dmats2[19][30] + coeff0_20*dmats2[20][30] + coeff0_21*dmats2[21][30] + coeff0_22*dmats2[22][30] + coeff0_23*dmats2[23][30] + coeff0_24*dmats2[24][30] + coeff0_25*dmats2[25][30] + coeff0_26*dmats2[26][30] + coeff0_27*dmats2[27][30] + coeff0_28*dmats2[28][30] + coeff0_29*dmats2[29][30] + coeff0_30*dmats2[30][30] + coeff0_31*dmats2[31][30] + coeff0_32*dmats2[32][30] + coeff0_33*dmats2[33][30] + coeff0_34*dmats2[34][30] + coeff0_35*dmats2[35][30] + coeff0_36*dmats2[36][30] + coeff0_37*dmats2[37][30] + coeff0_38*dmats2[38][30] + coeff0_39*dmats2[39][30] + coeff0_40*dmats2[40][30] + coeff0_41*dmats2[41][30] + coeff0_42*dmats2[42][30] + coeff0_43*dmats2[43][30] + coeff0_44*dmats2[44][30] + coeff0_45*dmats2[45][30] + coeff0_46*dmats2[46][30] + coeff0_47*dmats2[47][30] + coeff0_48*dmats2[48][30] + coeff0_49*dmats2[49][30] + coeff0_50*dmats2[50][30] + coeff0_51*dmats2[51][30] + coeff0_52*dmats2[52][30] + coeff0_53*dmats2[53][30] + coeff0_54*dmats2[54][30] + coeff0_55*dmats2[55][30];
          new_coeff0_31 = coeff0_0*dmats2[0][31] + coeff0_1*dmats2[1][31] + coeff0_2*dmats2[2][31] + coeff0_3*dmats2[3][31] + coeff0_4*dmats2[4][31] + coeff0_5*dmats2[5][31] + coeff0_6*dmats2[6][31] + coeff0_7*dmats2[7][31] + coeff0_8*dmats2[8][31] + coeff0_9*dmats2[9][31] + coeff0_10*dmats2[10][31] + coeff0_11*dmats2[11][31] + coeff0_12*dmats2[12][31] + coeff0_13*dmats2[13][31] + coeff0_14*dmats2[14][31] + coeff0_15*dmats2[15][31] + coeff0_16*dmats2[16][31] + coeff0_17*dmats2[17][31] + coeff0_18*dmats2[18][31] + coeff0_19*dmats2[19][31] + coeff0_20*dmats2[20][31] + coeff0_21*dmats2[21][31] + coeff0_22*dmats2[22][31] + coeff0_23*dmats2[23][31] + coeff0_24*dmats2[24][31] + coeff0_25*dmats2[25][31] + coeff0_26*dmats2[26][31] + coeff0_27*dmats2[27][31] + coeff0_28*dmats2[28][31] + coeff0_29*dmats2[29][31] + coeff0_30*dmats2[30][31] + coeff0_31*dmats2[31][31] + coeff0_32*dmats2[32][31] + coeff0_33*dmats2[33][31] + coeff0_34*dmats2[34][31] + coeff0_35*dmats2[35][31] + coeff0_36*dmats2[36][31] + coeff0_37*dmats2[37][31] + coeff0_38*dmats2[38][31] + coeff0_39*dmats2[39][31] + coeff0_40*dmats2[40][31] + coeff0_41*dmats2[41][31] + coeff0_42*dmats2[42][31] + coeff0_43*dmats2[43][31] + coeff0_44*dmats2[44][31] + coeff0_45*dmats2[45][31] + coeff0_46*dmats2[46][31] + coeff0_47*dmats2[47][31] + coeff0_48*dmats2[48][31] + coeff0_49*dmats2[49][31] + coeff0_50*dmats2[50][31] + coeff0_51*dmats2[51][31] + coeff0_52*dmats2[52][31] + coeff0_53*dmats2[53][31] + coeff0_54*dmats2[54][31] + coeff0_55*dmats2[55][31];
          new_coeff0_32 = coeff0_0*dmats2[0][32] + coeff0_1*dmats2[1][32] + coeff0_2*dmats2[2][32] + coeff0_3*dmats2[3][32] + coeff0_4*dmats2[4][32] + coeff0_5*dmats2[5][32] + coeff0_6*dmats2[6][32] + coeff0_7*dmats2[7][32] + coeff0_8*dmats2[8][32] + coeff0_9*dmats2[9][32] + coeff0_10*dmats2[10][32] + coeff0_11*dmats2[11][32] + coeff0_12*dmats2[12][32] + coeff0_13*dmats2[13][32] + coeff0_14*dmats2[14][32] + coeff0_15*dmats2[15][32] + coeff0_16*dmats2[16][32] + coeff0_17*dmats2[17][32] + coeff0_18*dmats2[18][32] + coeff0_19*dmats2[19][32] + coeff0_20*dmats2[20][32] + coeff0_21*dmats2[21][32] + coeff0_22*dmats2[22][32] + coeff0_23*dmats2[23][32] + coeff0_24*dmats2[24][32] + coeff0_25*dmats2[25][32] + coeff0_26*dmats2[26][32] + coeff0_27*dmats2[27][32] + coeff0_28*dmats2[28][32] + coeff0_29*dmats2[29][32] + coeff0_30*dmats2[30][32] + coeff0_31*dmats2[31][32] + coeff0_32*dmats2[32][32] + coeff0_33*dmats2[33][32] + coeff0_34*dmats2[34][32] + coeff0_35*dmats2[35][32] + coeff0_36*dmats2[36][32] + coeff0_37*dmats2[37][32] + coeff0_38*dmats2[38][32] + coeff0_39*dmats2[39][32] + coeff0_40*dmats2[40][32] + coeff0_41*dmats2[41][32] + coeff0_42*dmats2[42][32] + coeff0_43*dmats2[43][32] + coeff0_44*dmats2[44][32] + coeff0_45*dmats2[45][32] + coeff0_46*dmats2[46][32] + coeff0_47*dmats2[47][32] + coeff0_48*dmats2[48][32] + coeff0_49*dmats2[49][32] + coeff0_50*dmats2[50][32] + coeff0_51*dmats2[51][32] + coeff0_52*dmats2[52][32] + coeff0_53*dmats2[53][32] + coeff0_54*dmats2[54][32] + coeff0_55*dmats2[55][32];
          new_coeff0_33 = coeff0_0*dmats2[0][33] + coeff0_1*dmats2[1][33] + coeff0_2*dmats2[2][33] + coeff0_3*dmats2[3][33] + coeff0_4*dmats2[4][33] + coeff0_5*dmats2[5][33] + coeff0_6*dmats2[6][33] + coeff0_7*dmats2[7][33] + coeff0_8*dmats2[8][33] + coeff0_9*dmats2[9][33] + coeff0_10*dmats2[10][33] + coeff0_11*dmats2[11][33] + coeff0_12*dmats2[12][33] + coeff0_13*dmats2[13][33] + coeff0_14*dmats2[14][33] + coeff0_15*dmats2[15][33] + coeff0_16*dmats2[16][33] + coeff0_17*dmats2[17][33] + coeff0_18*dmats2[18][33] + coeff0_19*dmats2[19][33] + coeff0_20*dmats2[20][33] + coeff0_21*dmats2[21][33] + coeff0_22*dmats2[22][33] + coeff0_23*dmats2[23][33] + coeff0_24*dmats2[24][33] + coeff0_25*dmats2[25][33] + coeff0_26*dmats2[26][33] + coeff0_27*dmats2[27][33] + coeff0_28*dmats2[28][33] + coeff0_29*dmats2[29][33] + coeff0_30*dmats2[30][33] + coeff0_31*dmats2[31][33] + coeff0_32*dmats2[32][33] + coeff0_33*dmats2[33][33] + coeff0_34*dmats2[34][33] + coeff0_35*dmats2[35][33] + coeff0_36*dmats2[36][33] + coeff0_37*dmats2[37][33] + coeff0_38*dmats2[38][33] + coeff0_39*dmats2[39][33] + coeff0_40*dmats2[40][33] + coeff0_41*dmats2[41][33] + coeff0_42*dmats2[42][33] + coeff0_43*dmats2[43][33] + coeff0_44*dmats2[44][33] + coeff0_45*dmats2[45][33] + coeff0_46*dmats2[46][33] + coeff0_47*dmats2[47][33] + coeff0_48*dmats2[48][33] + coeff0_49*dmats2[49][33] + coeff0_50*dmats2[50][33] + coeff0_51*dmats2[51][33] + coeff0_52*dmats2[52][33] + coeff0_53*dmats2[53][33] + coeff0_54*dmats2[54][33] + coeff0_55*dmats2[55][33];
          new_coeff0_34 = coeff0_0*dmats2[0][34] + coeff0_1*dmats2[1][34] + coeff0_2*dmats2[2][34] + coeff0_3*dmats2[3][34] + coeff0_4*dmats2[4][34] + coeff0_5*dmats2[5][34] + coeff0_6*dmats2[6][34] + coeff0_7*dmats2[7][34] + coeff0_8*dmats2[8][34] + coeff0_9*dmats2[9][34] + coeff0_10*dmats2[10][34] + coeff0_11*dmats2[11][34] + coeff0_12*dmats2[12][34] + coeff0_13*dmats2[13][34] + coeff0_14*dmats2[14][34] + coeff0_15*dmats2[15][34] + coeff0_16*dmats2[16][34] + coeff0_17*dmats2[17][34] + coeff0_18*dmats2[18][34] + coeff0_19*dmats2[19][34] + coeff0_20*dmats2[20][34] + coeff0_21*dmats2[21][34] + coeff0_22*dmats2[22][34] + coeff0_23*dmats2[23][34] + coeff0_24*dmats2[24][34] + coeff0_25*dmats2[25][34] + coeff0_26*dmats2[26][34] + coeff0_27*dmats2[27][34] + coeff0_28*dmats2[28][34] + coeff0_29*dmats2[29][34] + coeff0_30*dmats2[30][34] + coeff0_31*dmats2[31][34] + coeff0_32*dmats2[32][34] + coeff0_33*dmats2[33][34] + coeff0_34*dmats2[34][34] + coeff0_35*dmats2[35][34] + coeff0_36*dmats2[36][34] + coeff0_37*dmats2[37][34] + coeff0_38*dmats2[38][34] + coeff0_39*dmats2[39][34] + coeff0_40*dmats2[40][34] + coeff0_41*dmats2[41][34] + coeff0_42*dmats2[42][34] + coeff0_43*dmats2[43][34] + coeff0_44*dmats2[44][34] + coeff0_45*dmats2[45][34] + coeff0_46*dmats2[46][34] + coeff0_47*dmats2[47][34] + coeff0_48*dmats2[48][34] + coeff0_49*dmats2[49][34] + coeff0_50*dmats2[50][34] + coeff0_51*dmats2[51][34] + coeff0_52*dmats2[52][34] + coeff0_53*dmats2[53][34] + coeff0_54*dmats2[54][34] + coeff0_55*dmats2[55][34];
          new_coeff0_35 = coeff0_0*dmats2[0][35] + coeff0_1*dmats2[1][35] + coeff0_2*dmats2[2][35] + coeff0_3*dmats2[3][35] + coeff0_4*dmats2[4][35] + coeff0_5*dmats2[5][35] + coeff0_6*dmats2[6][35] + coeff0_7*dmats2[7][35] + coeff0_8*dmats2[8][35] + coeff0_9*dmats2[9][35] + coeff0_10*dmats2[10][35] + coeff0_11*dmats2[11][35] + coeff0_12*dmats2[12][35] + coeff0_13*dmats2[13][35] + coeff0_14*dmats2[14][35] + coeff0_15*dmats2[15][35] + coeff0_16*dmats2[16][35] + coeff0_17*dmats2[17][35] + coeff0_18*dmats2[18][35] + coeff0_19*dmats2[19][35] + coeff0_20*dmats2[20][35] + coeff0_21*dmats2[21][35] + coeff0_22*dmats2[22][35] + coeff0_23*dmats2[23][35] + coeff0_24*dmats2[24][35] + coeff0_25*dmats2[25][35] + coeff0_26*dmats2[26][35] + coeff0_27*dmats2[27][35] + coeff0_28*dmats2[28][35] + coeff0_29*dmats2[29][35] + coeff0_30*dmats2[30][35] + coeff0_31*dmats2[31][35] + coeff0_32*dmats2[32][35] + coeff0_33*dmats2[33][35] + coeff0_34*dmats2[34][35] + coeff0_35*dmats2[35][35] + coeff0_36*dmats2[36][35] + coeff0_37*dmats2[37][35] + coeff0_38*dmats2[38][35] + coeff0_39*dmats2[39][35] + coeff0_40*dmats2[40][35] + coeff0_41*dmats2[41][35] + coeff0_42*dmats2[42][35] + coeff0_43*dmats2[43][35] + coeff0_44*dmats2[44][35] + coeff0_45*dmats2[45][35] + coeff0_46*dmats2[46][35] + coeff0_47*dmats2[47][35] + coeff0_48*dmats2[48][35] + coeff0_49*dmats2[49][35] + coeff0_50*dmats2[50][35] + coeff0_51*dmats2[51][35] + coeff0_52*dmats2[52][35] + coeff0_53*dmats2[53][35] + coeff0_54*dmats2[54][35] + coeff0_55*dmats2[55][35];
          new_coeff0_36 = coeff0_0*dmats2[0][36] + coeff0_1*dmats2[1][36] + coeff0_2*dmats2[2][36] + coeff0_3*dmats2[3][36] + coeff0_4*dmats2[4][36] + coeff0_5*dmats2[5][36] + coeff0_6*dmats2[6][36] + coeff0_7*dmats2[7][36] + coeff0_8*dmats2[8][36] + coeff0_9*dmats2[9][36] + coeff0_10*dmats2[10][36] + coeff0_11*dmats2[11][36] + coeff0_12*dmats2[12][36] + coeff0_13*dmats2[13][36] + coeff0_14*dmats2[14][36] + coeff0_15*dmats2[15][36] + coeff0_16*dmats2[16][36] + coeff0_17*dmats2[17][36] + coeff0_18*dmats2[18][36] + coeff0_19*dmats2[19][36] + coeff0_20*dmats2[20][36] + coeff0_21*dmats2[21][36] + coeff0_22*dmats2[22][36] + coeff0_23*dmats2[23][36] + coeff0_24*dmats2[24][36] + coeff0_25*dmats2[25][36] + coeff0_26*dmats2[26][36] + coeff0_27*dmats2[27][36] + coeff0_28*dmats2[28][36] + coeff0_29*dmats2[29][36] + coeff0_30*dmats2[30][36] + coeff0_31*dmats2[31][36] + coeff0_32*dmats2[32][36] + coeff0_33*dmats2[33][36] + coeff0_34*dmats2[34][36] + coeff0_35*dmats2[35][36] + coeff0_36*dmats2[36][36] + coeff0_37*dmats2[37][36] + coeff0_38*dmats2[38][36] + coeff0_39*dmats2[39][36] + coeff0_40*dmats2[40][36] + coeff0_41*dmats2[41][36] + coeff0_42*dmats2[42][36] + coeff0_43*dmats2[43][36] + coeff0_44*dmats2[44][36] + coeff0_45*dmats2[45][36] + coeff0_46*dmats2[46][36] + coeff0_47*dmats2[47][36] + coeff0_48*dmats2[48][36] + coeff0_49*dmats2[49][36] + coeff0_50*dmats2[50][36] + coeff0_51*dmats2[51][36] + coeff0_52*dmats2[52][36] + coeff0_53*dmats2[53][36] + coeff0_54*dmats2[54][36] + coeff0_55*dmats2[55][36];
          new_coeff0_37 = coeff0_0*dmats2[0][37] + coeff0_1*dmats2[1][37] + coeff0_2*dmats2[2][37] + coeff0_3*dmats2[3][37] + coeff0_4*dmats2[4][37] + coeff0_5*dmats2[5][37] + coeff0_6*dmats2[6][37] + coeff0_7*dmats2[7][37] + coeff0_8*dmats2[8][37] + coeff0_9*dmats2[9][37] + coeff0_10*dmats2[10][37] + coeff0_11*dmats2[11][37] + coeff0_12*dmats2[12][37] + coeff0_13*dmats2[13][37] + coeff0_14*dmats2[14][37] + coeff0_15*dmats2[15][37] + coeff0_16*dmats2[16][37] + coeff0_17*dmats2[17][37] + coeff0_18*dmats2[18][37] + coeff0_19*dmats2[19][37] + coeff0_20*dmats2[20][37] + coeff0_21*dmats2[21][37] + coeff0_22*dmats2[22][37] + coeff0_23*dmats2[23][37] + coeff0_24*dmats2[24][37] + coeff0_25*dmats2[25][37] + coeff0_26*dmats2[26][37] + coeff0_27*dmats2[27][37] + coeff0_28*dmats2[28][37] + coeff0_29*dmats2[29][37] + coeff0_30*dmats2[30][37] + coeff0_31*dmats2[31][37] + coeff0_32*dmats2[32][37] + coeff0_33*dmats2[33][37] + coeff0_34*dmats2[34][37] + coeff0_35*dmats2[35][37] + coeff0_36*dmats2[36][37] + coeff0_37*dmats2[37][37] + coeff0_38*dmats2[38][37] + coeff0_39*dmats2[39][37] + coeff0_40*dmats2[40][37] + coeff0_41*dmats2[41][37] + coeff0_42*dmats2[42][37] + coeff0_43*dmats2[43][37] + coeff0_44*dmats2[44][37] + coeff0_45*dmats2[45][37] + coeff0_46*dmats2[46][37] + coeff0_47*dmats2[47][37] + coeff0_48*dmats2[48][37] + coeff0_49*dmats2[49][37] + coeff0_50*dmats2[50][37] + coeff0_51*dmats2[51][37] + coeff0_52*dmats2[52][37] + coeff0_53*dmats2[53][37] + coeff0_54*dmats2[54][37] + coeff0_55*dmats2[55][37];
          new_coeff0_38 = coeff0_0*dmats2[0][38] + coeff0_1*dmats2[1][38] + coeff0_2*dmats2[2][38] + coeff0_3*dmats2[3][38] + coeff0_4*dmats2[4][38] + coeff0_5*dmats2[5][38] + coeff0_6*dmats2[6][38] + coeff0_7*dmats2[7][38] + coeff0_8*dmats2[8][38] + coeff0_9*dmats2[9][38] + coeff0_10*dmats2[10][38] + coeff0_11*dmats2[11][38] + coeff0_12*dmats2[12][38] + coeff0_13*dmats2[13][38] + coeff0_14*dmats2[14][38] + coeff0_15*dmats2[15][38] + coeff0_16*dmats2[16][38] + coeff0_17*dmats2[17][38] + coeff0_18*dmats2[18][38] + coeff0_19*dmats2[19][38] + coeff0_20*dmats2[20][38] + coeff0_21*dmats2[21][38] + coeff0_22*dmats2[22][38] + coeff0_23*dmats2[23][38] + coeff0_24*dmats2[24][38] + coeff0_25*dmats2[25][38] + coeff0_26*dmats2[26][38] + coeff0_27*dmats2[27][38] + coeff0_28*dmats2[28][38] + coeff0_29*dmats2[29][38] + coeff0_30*dmats2[30][38] + coeff0_31*dmats2[31][38] + coeff0_32*dmats2[32][38] + coeff0_33*dmats2[33][38] + coeff0_34*dmats2[34][38] + coeff0_35*dmats2[35][38] + coeff0_36*dmats2[36][38] + coeff0_37*dmats2[37][38] + coeff0_38*dmats2[38][38] + coeff0_39*dmats2[39][38] + coeff0_40*dmats2[40][38] + coeff0_41*dmats2[41][38] + coeff0_42*dmats2[42][38] + coeff0_43*dmats2[43][38] + coeff0_44*dmats2[44][38] + coeff0_45*dmats2[45][38] + coeff0_46*dmats2[46][38] + coeff0_47*dmats2[47][38] + coeff0_48*dmats2[48][38] + coeff0_49*dmats2[49][38] + coeff0_50*dmats2[50][38] + coeff0_51*dmats2[51][38] + coeff0_52*dmats2[52][38] + coeff0_53*dmats2[53][38] + coeff0_54*dmats2[54][38] + coeff0_55*dmats2[55][38];
          new_coeff0_39 = coeff0_0*dmats2[0][39] + coeff0_1*dmats2[1][39] + coeff0_2*dmats2[2][39] + coeff0_3*dmats2[3][39] + coeff0_4*dmats2[4][39] + coeff0_5*dmats2[5][39] + coeff0_6*dmats2[6][39] + coeff0_7*dmats2[7][39] + coeff0_8*dmats2[8][39] + coeff0_9*dmats2[9][39] + coeff0_10*dmats2[10][39] + coeff0_11*dmats2[11][39] + coeff0_12*dmats2[12][39] + coeff0_13*dmats2[13][39] + coeff0_14*dmats2[14][39] + coeff0_15*dmats2[15][39] + coeff0_16*dmats2[16][39] + coeff0_17*dmats2[17][39] + coeff0_18*dmats2[18][39] + coeff0_19*dmats2[19][39] + coeff0_20*dmats2[20][39] + coeff0_21*dmats2[21][39] + coeff0_22*dmats2[22][39] + coeff0_23*dmats2[23][39] + coeff0_24*dmats2[24][39] + coeff0_25*dmats2[25][39] + coeff0_26*dmats2[26][39] + coeff0_27*dmats2[27][39] + coeff0_28*dmats2[28][39] + coeff0_29*dmats2[29][39] + coeff0_30*dmats2[30][39] + coeff0_31*dmats2[31][39] + coeff0_32*dmats2[32][39] + coeff0_33*dmats2[33][39] + coeff0_34*dmats2[34][39] + coeff0_35*dmats2[35][39] + coeff0_36*dmats2[36][39] + coeff0_37*dmats2[37][39] + coeff0_38*dmats2[38][39] + coeff0_39*dmats2[39][39] + coeff0_40*dmats2[40][39] + coeff0_41*dmats2[41][39] + coeff0_42*dmats2[42][39] + coeff0_43*dmats2[43][39] + coeff0_44*dmats2[44][39] + coeff0_45*dmats2[45][39] + coeff0_46*dmats2[46][39] + coeff0_47*dmats2[47][39] + coeff0_48*dmats2[48][39] + coeff0_49*dmats2[49][39] + coeff0_50*dmats2[50][39] + coeff0_51*dmats2[51][39] + coeff0_52*dmats2[52][39] + coeff0_53*dmats2[53][39] + coeff0_54*dmats2[54][39] + coeff0_55*dmats2[55][39];
          new_coeff0_40 = coeff0_0*dmats2[0][40] + coeff0_1*dmats2[1][40] + coeff0_2*dmats2[2][40] + coeff0_3*dmats2[3][40] + coeff0_4*dmats2[4][40] + coeff0_5*dmats2[5][40] + coeff0_6*dmats2[6][40] + coeff0_7*dmats2[7][40] + coeff0_8*dmats2[8][40] + coeff0_9*dmats2[9][40] + coeff0_10*dmats2[10][40] + coeff0_11*dmats2[11][40] + coeff0_12*dmats2[12][40] + coeff0_13*dmats2[13][40] + coeff0_14*dmats2[14][40] + coeff0_15*dmats2[15][40] + coeff0_16*dmats2[16][40] + coeff0_17*dmats2[17][40] + coeff0_18*dmats2[18][40] + coeff0_19*dmats2[19][40] + coeff0_20*dmats2[20][40] + coeff0_21*dmats2[21][40] + coeff0_22*dmats2[22][40] + coeff0_23*dmats2[23][40] + coeff0_24*dmats2[24][40] + coeff0_25*dmats2[25][40] + coeff0_26*dmats2[26][40] + coeff0_27*dmats2[27][40] + coeff0_28*dmats2[28][40] + coeff0_29*dmats2[29][40] + coeff0_30*dmats2[30][40] + coeff0_31*dmats2[31][40] + coeff0_32*dmats2[32][40] + coeff0_33*dmats2[33][40] + coeff0_34*dmats2[34][40] + coeff0_35*dmats2[35][40] + coeff0_36*dmats2[36][40] + coeff0_37*dmats2[37][40] + coeff0_38*dmats2[38][40] + coeff0_39*dmats2[39][40] + coeff0_40*dmats2[40][40] + coeff0_41*dmats2[41][40] + coeff0_42*dmats2[42][40] + coeff0_43*dmats2[43][40] + coeff0_44*dmats2[44][40] + coeff0_45*dmats2[45][40] + coeff0_46*dmats2[46][40] + coeff0_47*dmats2[47][40] + coeff0_48*dmats2[48][40] + coeff0_49*dmats2[49][40] + coeff0_50*dmats2[50][40] + coeff0_51*dmats2[51][40] + coeff0_52*dmats2[52][40] + coeff0_53*dmats2[53][40] + coeff0_54*dmats2[54][40] + coeff0_55*dmats2[55][40];
          new_coeff0_41 = coeff0_0*dmats2[0][41] + coeff0_1*dmats2[1][41] + coeff0_2*dmats2[2][41] + coeff0_3*dmats2[3][41] + coeff0_4*dmats2[4][41] + coeff0_5*dmats2[5][41] + coeff0_6*dmats2[6][41] + coeff0_7*dmats2[7][41] + coeff0_8*dmats2[8][41] + coeff0_9*dmats2[9][41] + coeff0_10*dmats2[10][41] + coeff0_11*dmats2[11][41] + coeff0_12*dmats2[12][41] + coeff0_13*dmats2[13][41] + coeff0_14*dmats2[14][41] + coeff0_15*dmats2[15][41] + coeff0_16*dmats2[16][41] + coeff0_17*dmats2[17][41] + coeff0_18*dmats2[18][41] + coeff0_19*dmats2[19][41] + coeff0_20*dmats2[20][41] + coeff0_21*dmats2[21][41] + coeff0_22*dmats2[22][41] + coeff0_23*dmats2[23][41] + coeff0_24*dmats2[24][41] + coeff0_25*dmats2[25][41] + coeff0_26*dmats2[26][41] + coeff0_27*dmats2[27][41] + coeff0_28*dmats2[28][41] + coeff0_29*dmats2[29][41] + coeff0_30*dmats2[30][41] + coeff0_31*dmats2[31][41] + coeff0_32*dmats2[32][41] + coeff0_33*dmats2[33][41] + coeff0_34*dmats2[34][41] + coeff0_35*dmats2[35][41] + coeff0_36*dmats2[36][41] + coeff0_37*dmats2[37][41] + coeff0_38*dmats2[38][41] + coeff0_39*dmats2[39][41] + coeff0_40*dmats2[40][41] + coeff0_41*dmats2[41][41] + coeff0_42*dmats2[42][41] + coeff0_43*dmats2[43][41] + coeff0_44*dmats2[44][41] + coeff0_45*dmats2[45][41] + coeff0_46*dmats2[46][41] + coeff0_47*dmats2[47][41] + coeff0_48*dmats2[48][41] + coeff0_49*dmats2[49][41] + coeff0_50*dmats2[50][41] + coeff0_51*dmats2[51][41] + coeff0_52*dmats2[52][41] + coeff0_53*dmats2[53][41] + coeff0_54*dmats2[54][41] + coeff0_55*dmats2[55][41];
          new_coeff0_42 = coeff0_0*dmats2[0][42] + coeff0_1*dmats2[1][42] + coeff0_2*dmats2[2][42] + coeff0_3*dmats2[3][42] + coeff0_4*dmats2[4][42] + coeff0_5*dmats2[5][42] + coeff0_6*dmats2[6][42] + coeff0_7*dmats2[7][42] + coeff0_8*dmats2[8][42] + coeff0_9*dmats2[9][42] + coeff0_10*dmats2[10][42] + coeff0_11*dmats2[11][42] + coeff0_12*dmats2[12][42] + coeff0_13*dmats2[13][42] + coeff0_14*dmats2[14][42] + coeff0_15*dmats2[15][42] + coeff0_16*dmats2[16][42] + coeff0_17*dmats2[17][42] + coeff0_18*dmats2[18][42] + coeff0_19*dmats2[19][42] + coeff0_20*dmats2[20][42] + coeff0_21*dmats2[21][42] + coeff0_22*dmats2[22][42] + coeff0_23*dmats2[23][42] + coeff0_24*dmats2[24][42] + coeff0_25*dmats2[25][42] + coeff0_26*dmats2[26][42] + coeff0_27*dmats2[27][42] + coeff0_28*dmats2[28][42] + coeff0_29*dmats2[29][42] + coeff0_30*dmats2[30][42] + coeff0_31*dmats2[31][42] + coeff0_32*dmats2[32][42] + coeff0_33*dmats2[33][42] + coeff0_34*dmats2[34][42] + coeff0_35*dmats2[35][42] + coeff0_36*dmats2[36][42] + coeff0_37*dmats2[37][42] + coeff0_38*dmats2[38][42] + coeff0_39*dmats2[39][42] + coeff0_40*dmats2[40][42] + coeff0_41*dmats2[41][42] + coeff0_42*dmats2[42][42] + coeff0_43*dmats2[43][42] + coeff0_44*dmats2[44][42] + coeff0_45*dmats2[45][42] + coeff0_46*dmats2[46][42] + coeff0_47*dmats2[47][42] + coeff0_48*dmats2[48][42] + coeff0_49*dmats2[49][42] + coeff0_50*dmats2[50][42] + coeff0_51*dmats2[51][42] + coeff0_52*dmats2[52][42] + coeff0_53*dmats2[53][42] + coeff0_54*dmats2[54][42] + coeff0_55*dmats2[55][42];
          new_coeff0_43 = coeff0_0*dmats2[0][43] + coeff0_1*dmats2[1][43] + coeff0_2*dmats2[2][43] + coeff0_3*dmats2[3][43] + coeff0_4*dmats2[4][43] + coeff0_5*dmats2[5][43] + coeff0_6*dmats2[6][43] + coeff0_7*dmats2[7][43] + coeff0_8*dmats2[8][43] + coeff0_9*dmats2[9][43] + coeff0_10*dmats2[10][43] + coeff0_11*dmats2[11][43] + coeff0_12*dmats2[12][43] + coeff0_13*dmats2[13][43] + coeff0_14*dmats2[14][43] + coeff0_15*dmats2[15][43] + coeff0_16*dmats2[16][43] + coeff0_17*dmats2[17][43] + coeff0_18*dmats2[18][43] + coeff0_19*dmats2[19][43] + coeff0_20*dmats2[20][43] + coeff0_21*dmats2[21][43] + coeff0_22*dmats2[22][43] + coeff0_23*dmats2[23][43] + coeff0_24*dmats2[24][43] + coeff0_25*dmats2[25][43] + coeff0_26*dmats2[26][43] + coeff0_27*dmats2[27][43] + coeff0_28*dmats2[28][43] + coeff0_29*dmats2[29][43] + coeff0_30*dmats2[30][43] + coeff0_31*dmats2[31][43] + coeff0_32*dmats2[32][43] + coeff0_33*dmats2[33][43] + coeff0_34*dmats2[34][43] + coeff0_35*dmats2[35][43] + coeff0_36*dmats2[36][43] + coeff0_37*dmats2[37][43] + coeff0_38*dmats2[38][43] + coeff0_39*dmats2[39][43] + coeff0_40*dmats2[40][43] + coeff0_41*dmats2[41][43] + coeff0_42*dmats2[42][43] + coeff0_43*dmats2[43][43] + coeff0_44*dmats2[44][43] + coeff0_45*dmats2[45][43] + coeff0_46*dmats2[46][43] + coeff0_47*dmats2[47][43] + coeff0_48*dmats2[48][43] + coeff0_49*dmats2[49][43] + coeff0_50*dmats2[50][43] + coeff0_51*dmats2[51][43] + coeff0_52*dmats2[52][43] + coeff0_53*dmats2[53][43] + coeff0_54*dmats2[54][43] + coeff0_55*dmats2[55][43];
          new_coeff0_44 = coeff0_0*dmats2[0][44] + coeff0_1*dmats2[1][44] + coeff0_2*dmats2[2][44] + coeff0_3*dmats2[3][44] + coeff0_4*dmats2[4][44] + coeff0_5*dmats2[5][44] + coeff0_6*dmats2[6][44] + coeff0_7*dmats2[7][44] + coeff0_8*dmats2[8][44] + coeff0_9*dmats2[9][44] + coeff0_10*dmats2[10][44] + coeff0_11*dmats2[11][44] + coeff0_12*dmats2[12][44] + coeff0_13*dmats2[13][44] + coeff0_14*dmats2[14][44] + coeff0_15*dmats2[15][44] + coeff0_16*dmats2[16][44] + coeff0_17*dmats2[17][44] + coeff0_18*dmats2[18][44] + coeff0_19*dmats2[19][44] + coeff0_20*dmats2[20][44] + coeff0_21*dmats2[21][44] + coeff0_22*dmats2[22][44] + coeff0_23*dmats2[23][44] + coeff0_24*dmats2[24][44] + coeff0_25*dmats2[25][44] + coeff0_26*dmats2[26][44] + coeff0_27*dmats2[27][44] + coeff0_28*dmats2[28][44] + coeff0_29*dmats2[29][44] + coeff0_30*dmats2[30][44] + coeff0_31*dmats2[31][44] + coeff0_32*dmats2[32][44] + coeff0_33*dmats2[33][44] + coeff0_34*dmats2[34][44] + coeff0_35*dmats2[35][44] + coeff0_36*dmats2[36][44] + coeff0_37*dmats2[37][44] + coeff0_38*dmats2[38][44] + coeff0_39*dmats2[39][44] + coeff0_40*dmats2[40][44] + coeff0_41*dmats2[41][44] + coeff0_42*dmats2[42][44] + coeff0_43*dmats2[43][44] + coeff0_44*dmats2[44][44] + coeff0_45*dmats2[45][44] + coeff0_46*dmats2[46][44] + coeff0_47*dmats2[47][44] + coeff0_48*dmats2[48][44] + coeff0_49*dmats2[49][44] + coeff0_50*dmats2[50][44] + coeff0_51*dmats2[51][44] + coeff0_52*dmats2[52][44] + coeff0_53*dmats2[53][44] + coeff0_54*dmats2[54][44] + coeff0_55*dmats2[55][44];
          new_coeff0_45 = coeff0_0*dmats2[0][45] + coeff0_1*dmats2[1][45] + coeff0_2*dmats2[2][45] + coeff0_3*dmats2[3][45] + coeff0_4*dmats2[4][45] + coeff0_5*dmats2[5][45] + coeff0_6*dmats2[6][45] + coeff0_7*dmats2[7][45] + coeff0_8*dmats2[8][45] + coeff0_9*dmats2[9][45] + coeff0_10*dmats2[10][45] + coeff0_11*dmats2[11][45] + coeff0_12*dmats2[12][45] + coeff0_13*dmats2[13][45] + coeff0_14*dmats2[14][45] + coeff0_15*dmats2[15][45] + coeff0_16*dmats2[16][45] + coeff0_17*dmats2[17][45] + coeff0_18*dmats2[18][45] + coeff0_19*dmats2[19][45] + coeff0_20*dmats2[20][45] + coeff0_21*dmats2[21][45] + coeff0_22*dmats2[22][45] + coeff0_23*dmats2[23][45] + coeff0_24*dmats2[24][45] + coeff0_25*dmats2[25][45] + coeff0_26*dmats2[26][45] + coeff0_27*dmats2[27][45] + coeff0_28*dmats2[28][45] + coeff0_29*dmats2[29][45] + coeff0_30*dmats2[30][45] + coeff0_31*dmats2[31][45] + coeff0_32*dmats2[32][45] + coeff0_33*dmats2[33][45] + coeff0_34*dmats2[34][45] + coeff0_35*dmats2[35][45] + coeff0_36*dmats2[36][45] + coeff0_37*dmats2[37][45] + coeff0_38*dmats2[38][45] + coeff0_39*dmats2[39][45] + coeff0_40*dmats2[40][45] + coeff0_41*dmats2[41][45] + coeff0_42*dmats2[42][45] + coeff0_43*dmats2[43][45] + coeff0_44*dmats2[44][45] + coeff0_45*dmats2[45][45] + coeff0_46*dmats2[46][45] + coeff0_47*dmats2[47][45] + coeff0_48*dmats2[48][45] + coeff0_49*dmats2[49][45] + coeff0_50*dmats2[50][45] + coeff0_51*dmats2[51][45] + coeff0_52*dmats2[52][45] + coeff0_53*dmats2[53][45] + coeff0_54*dmats2[54][45] + coeff0_55*dmats2[55][45];
          new_coeff0_46 = coeff0_0*dmats2[0][46] + coeff0_1*dmats2[1][46] + coeff0_2*dmats2[2][46] + coeff0_3*dmats2[3][46] + coeff0_4*dmats2[4][46] + coeff0_5*dmats2[5][46] + coeff0_6*dmats2[6][46] + coeff0_7*dmats2[7][46] + coeff0_8*dmats2[8][46] + coeff0_9*dmats2[9][46] + coeff0_10*dmats2[10][46] + coeff0_11*dmats2[11][46] + coeff0_12*dmats2[12][46] + coeff0_13*dmats2[13][46] + coeff0_14*dmats2[14][46] + coeff0_15*dmats2[15][46] + coeff0_16*dmats2[16][46] + coeff0_17*dmats2[17][46] + coeff0_18*dmats2[18][46] + coeff0_19*dmats2[19][46] + coeff0_20*dmats2[20][46] + coeff0_21*dmats2[21][46] + coeff0_22*dmats2[22][46] + coeff0_23*dmats2[23][46] + coeff0_24*dmats2[24][46] + coeff0_25*dmats2[25][46] + coeff0_26*dmats2[26][46] + coeff0_27*dmats2[27][46] + coeff0_28*dmats2[28][46] + coeff0_29*dmats2[29][46] + coeff0_30*dmats2[30][46] + coeff0_31*dmats2[31][46] + coeff0_32*dmats2[32][46] + coeff0_33*dmats2[33][46] + coeff0_34*dmats2[34][46] + coeff0_35*dmats2[35][46] + coeff0_36*dmats2[36][46] + coeff0_37*dmats2[37][46] + coeff0_38*dmats2[38][46] + coeff0_39*dmats2[39][46] + coeff0_40*dmats2[40][46] + coeff0_41*dmats2[41][46] + coeff0_42*dmats2[42][46] + coeff0_43*dmats2[43][46] + coeff0_44*dmats2[44][46] + coeff0_45*dmats2[45][46] + coeff0_46*dmats2[46][46] + coeff0_47*dmats2[47][46] + coeff0_48*dmats2[48][46] + coeff0_49*dmats2[49][46] + coeff0_50*dmats2[50][46] + coeff0_51*dmats2[51][46] + coeff0_52*dmats2[52][46] + coeff0_53*dmats2[53][46] + coeff0_54*dmats2[54][46] + coeff0_55*dmats2[55][46];
          new_coeff0_47 = coeff0_0*dmats2[0][47] + coeff0_1*dmats2[1][47] + coeff0_2*dmats2[2][47] + coeff0_3*dmats2[3][47] + coeff0_4*dmats2[4][47] + coeff0_5*dmats2[5][47] + coeff0_6*dmats2[6][47] + coeff0_7*dmats2[7][47] + coeff0_8*dmats2[8][47] + coeff0_9*dmats2[9][47] + coeff0_10*dmats2[10][47] + coeff0_11*dmats2[11][47] + coeff0_12*dmats2[12][47] + coeff0_13*dmats2[13][47] + coeff0_14*dmats2[14][47] + coeff0_15*dmats2[15][47] + coeff0_16*dmats2[16][47] + coeff0_17*dmats2[17][47] + coeff0_18*dmats2[18][47] + coeff0_19*dmats2[19][47] + coeff0_20*dmats2[20][47] + coeff0_21*dmats2[21][47] + coeff0_22*dmats2[22][47] + coeff0_23*dmats2[23][47] + coeff0_24*dmats2[24][47] + coeff0_25*dmats2[25][47] + coeff0_26*dmats2[26][47] + coeff0_27*dmats2[27][47] + coeff0_28*dmats2[28][47] + coeff0_29*dmats2[29][47] + coeff0_30*dmats2[30][47] + coeff0_31*dmats2[31][47] + coeff0_32*dmats2[32][47] + coeff0_33*dmats2[33][47] + coeff0_34*dmats2[34][47] + coeff0_35*dmats2[35][47] + coeff0_36*dmats2[36][47] + coeff0_37*dmats2[37][47] + coeff0_38*dmats2[38][47] + coeff0_39*dmats2[39][47] + coeff0_40*dmats2[40][47] + coeff0_41*dmats2[41][47] + coeff0_42*dmats2[42][47] + coeff0_43*dmats2[43][47] + coeff0_44*dmats2[44][47] + coeff0_45*dmats2[45][47] + coeff0_46*dmats2[46][47] + coeff0_47*dmats2[47][47] + coeff0_48*dmats2[48][47] + coeff0_49*dmats2[49][47] + coeff0_50*dmats2[50][47] + coeff0_51*dmats2[51][47] + coeff0_52*dmats2[52][47] + coeff0_53*dmats2[53][47] + coeff0_54*dmats2[54][47] + coeff0_55*dmats2[55][47];
          new_coeff0_48 = coeff0_0*dmats2[0][48] + coeff0_1*dmats2[1][48] + coeff0_2*dmats2[2][48] + coeff0_3*dmats2[3][48] + coeff0_4*dmats2[4][48] + coeff0_5*dmats2[5][48] + coeff0_6*dmats2[6][48] + coeff0_7*dmats2[7][48] + coeff0_8*dmats2[8][48] + coeff0_9*dmats2[9][48] + coeff0_10*dmats2[10][48] + coeff0_11*dmats2[11][48] + coeff0_12*dmats2[12][48] + coeff0_13*dmats2[13][48] + coeff0_14*dmats2[14][48] + coeff0_15*dmats2[15][48] + coeff0_16*dmats2[16][48] + coeff0_17*dmats2[17][48] + coeff0_18*dmats2[18][48] + coeff0_19*dmats2[19][48] + coeff0_20*dmats2[20][48] + coeff0_21*dmats2[21][48] + coeff0_22*dmats2[22][48] + coeff0_23*dmats2[23][48] + coeff0_24*dmats2[24][48] + coeff0_25*dmats2[25][48] + coeff0_26*dmats2[26][48] + coeff0_27*dmats2[27][48] + coeff0_28*dmats2[28][48] + coeff0_29*dmats2[29][48] + coeff0_30*dmats2[30][48] + coeff0_31*dmats2[31][48] + coeff0_32*dmats2[32][48] + coeff0_33*dmats2[33][48] + coeff0_34*dmats2[34][48] + coeff0_35*dmats2[35][48] + coeff0_36*dmats2[36][48] + coeff0_37*dmats2[37][48] + coeff0_38*dmats2[38][48] + coeff0_39*dmats2[39][48] + coeff0_40*dmats2[40][48] + coeff0_41*dmats2[41][48] + coeff0_42*dmats2[42][48] + coeff0_43*dmats2[43][48] + coeff0_44*dmats2[44][48] + coeff0_45*dmats2[45][48] + coeff0_46*dmats2[46][48] + coeff0_47*dmats2[47][48] + coeff0_48*dmats2[48][48] + coeff0_49*dmats2[49][48] + coeff0_50*dmats2[50][48] + coeff0_51*dmats2[51][48] + coeff0_52*dmats2[52][48] + coeff0_53*dmats2[53][48] + coeff0_54*dmats2[54][48] + coeff0_55*dmats2[55][48];
          new_coeff0_49 = coeff0_0*dmats2[0][49] + coeff0_1*dmats2[1][49] + coeff0_2*dmats2[2][49] + coeff0_3*dmats2[3][49] + coeff0_4*dmats2[4][49] + coeff0_5*dmats2[5][49] + coeff0_6*dmats2[6][49] + coeff0_7*dmats2[7][49] + coeff0_8*dmats2[8][49] + coeff0_9*dmats2[9][49] + coeff0_10*dmats2[10][49] + coeff0_11*dmats2[11][49] + coeff0_12*dmats2[12][49] + coeff0_13*dmats2[13][49] + coeff0_14*dmats2[14][49] + coeff0_15*dmats2[15][49] + coeff0_16*dmats2[16][49] + coeff0_17*dmats2[17][49] + coeff0_18*dmats2[18][49] + coeff0_19*dmats2[19][49] + coeff0_20*dmats2[20][49] + coeff0_21*dmats2[21][49] + coeff0_22*dmats2[22][49] + coeff0_23*dmats2[23][49] + coeff0_24*dmats2[24][49] + coeff0_25*dmats2[25][49] + coeff0_26*dmats2[26][49] + coeff0_27*dmats2[27][49] + coeff0_28*dmats2[28][49] + coeff0_29*dmats2[29][49] + coeff0_30*dmats2[30][49] + coeff0_31*dmats2[31][49] + coeff0_32*dmats2[32][49] + coeff0_33*dmats2[33][49] + coeff0_34*dmats2[34][49] + coeff0_35*dmats2[35][49] + coeff0_36*dmats2[36][49] + coeff0_37*dmats2[37][49] + coeff0_38*dmats2[38][49] + coeff0_39*dmats2[39][49] + coeff0_40*dmats2[40][49] + coeff0_41*dmats2[41][49] + coeff0_42*dmats2[42][49] + coeff0_43*dmats2[43][49] + coeff0_44*dmats2[44][49] + coeff0_45*dmats2[45][49] + coeff0_46*dmats2[46][49] + coeff0_47*dmats2[47][49] + coeff0_48*dmats2[48][49] + coeff0_49*dmats2[49][49] + coeff0_50*dmats2[50][49] + coeff0_51*dmats2[51][49] + coeff0_52*dmats2[52][49] + coeff0_53*dmats2[53][49] + coeff0_54*dmats2[54][49] + coeff0_55*dmats2[55][49];
          new_coeff0_50 = coeff0_0*dmats2[0][50] + coeff0_1*dmats2[1][50] + coeff0_2*dmats2[2][50] + coeff0_3*dmats2[3][50] + coeff0_4*dmats2[4][50] + coeff0_5*dmats2[5][50] + coeff0_6*dmats2[6][50] + coeff0_7*dmats2[7][50] + coeff0_8*dmats2[8][50] + coeff0_9*dmats2[9][50] + coeff0_10*dmats2[10][50] + coeff0_11*dmats2[11][50] + coeff0_12*dmats2[12][50] + coeff0_13*dmats2[13][50] + coeff0_14*dmats2[14][50] + coeff0_15*dmats2[15][50] + coeff0_16*dmats2[16][50] + coeff0_17*dmats2[17][50] + coeff0_18*dmats2[18][50] + coeff0_19*dmats2[19][50] + coeff0_20*dmats2[20][50] + coeff0_21*dmats2[21][50] + coeff0_22*dmats2[22][50] + coeff0_23*dmats2[23][50] + coeff0_24*dmats2[24][50] + coeff0_25*dmats2[25][50] + coeff0_26*dmats2[26][50] + coeff0_27*dmats2[27][50] + coeff0_28*dmats2[28][50] + coeff0_29*dmats2[29][50] + coeff0_30*dmats2[30][50] + coeff0_31*dmats2[31][50] + coeff0_32*dmats2[32][50] + coeff0_33*dmats2[33][50] + coeff0_34*dmats2[34][50] + coeff0_35*dmats2[35][50] + coeff0_36*dmats2[36][50] + coeff0_37*dmats2[37][50] + coeff0_38*dmats2[38][50] + coeff0_39*dmats2[39][50] + coeff0_40*dmats2[40][50] + coeff0_41*dmats2[41][50] + coeff0_42*dmats2[42][50] + coeff0_43*dmats2[43][50] + coeff0_44*dmats2[44][50] + coeff0_45*dmats2[45][50] + coeff0_46*dmats2[46][50] + coeff0_47*dmats2[47][50] + coeff0_48*dmats2[48][50] + coeff0_49*dmats2[49][50] + coeff0_50*dmats2[50][50] + coeff0_51*dmats2[51][50] + coeff0_52*dmats2[52][50] + coeff0_53*dmats2[53][50] + coeff0_54*dmats2[54][50] + coeff0_55*dmats2[55][50];
          new_coeff0_51 = coeff0_0*dmats2[0][51] + coeff0_1*dmats2[1][51] + coeff0_2*dmats2[2][51] + coeff0_3*dmats2[3][51] + coeff0_4*dmats2[4][51] + coeff0_5*dmats2[5][51] + coeff0_6*dmats2[6][51] + coeff0_7*dmats2[7][51] + coeff0_8*dmats2[8][51] + coeff0_9*dmats2[9][51] + coeff0_10*dmats2[10][51] + coeff0_11*dmats2[11][51] + coeff0_12*dmats2[12][51] + coeff0_13*dmats2[13][51] + coeff0_14*dmats2[14][51] + coeff0_15*dmats2[15][51] + coeff0_16*dmats2[16][51] + coeff0_17*dmats2[17][51] + coeff0_18*dmats2[18][51] + coeff0_19*dmats2[19][51] + coeff0_20*dmats2[20][51] + coeff0_21*dmats2[21][51] + coeff0_22*dmats2[22][51] + coeff0_23*dmats2[23][51] + coeff0_24*dmats2[24][51] + coeff0_25*dmats2[25][51] + coeff0_26*dmats2[26][51] + coeff0_27*dmats2[27][51] + coeff0_28*dmats2[28][51] + coeff0_29*dmats2[29][51] + coeff0_30*dmats2[30][51] + coeff0_31*dmats2[31][51] + coeff0_32*dmats2[32][51] + coeff0_33*dmats2[33][51] + coeff0_34*dmats2[34][51] + coeff0_35*dmats2[35][51] + coeff0_36*dmats2[36][51] + coeff0_37*dmats2[37][51] + coeff0_38*dmats2[38][51] + coeff0_39*dmats2[39][51] + coeff0_40*dmats2[40][51] + coeff0_41*dmats2[41][51] + coeff0_42*dmats2[42][51] + coeff0_43*dmats2[43][51] + coeff0_44*dmats2[44][51] + coeff0_45*dmats2[45][51] + coeff0_46*dmats2[46][51] + coeff0_47*dmats2[47][51] + coeff0_48*dmats2[48][51] + coeff0_49*dmats2[49][51] + coeff0_50*dmats2[50][51] + coeff0_51*dmats2[51][51] + coeff0_52*dmats2[52][51] + coeff0_53*dmats2[53][51] + coeff0_54*dmats2[54][51] + coeff0_55*dmats2[55][51];
          new_coeff0_52 = coeff0_0*dmats2[0][52] + coeff0_1*dmats2[1][52] + coeff0_2*dmats2[2][52] + coeff0_3*dmats2[3][52] + coeff0_4*dmats2[4][52] + coeff0_5*dmats2[5][52] + coeff0_6*dmats2[6][52] + coeff0_7*dmats2[7][52] + coeff0_8*dmats2[8][52] + coeff0_9*dmats2[9][52] + coeff0_10*dmats2[10][52] + coeff0_11*dmats2[11][52] + coeff0_12*dmats2[12][52] + coeff0_13*dmats2[13][52] + coeff0_14*dmats2[14][52] + coeff0_15*dmats2[15][52] + coeff0_16*dmats2[16][52] + coeff0_17*dmats2[17][52] + coeff0_18*dmats2[18][52] + coeff0_19*dmats2[19][52] + coeff0_20*dmats2[20][52] + coeff0_21*dmats2[21][52] + coeff0_22*dmats2[22][52] + coeff0_23*dmats2[23][52] + coeff0_24*dmats2[24][52] + coeff0_25*dmats2[25][52] + coeff0_26*dmats2[26][52] + coeff0_27*dmats2[27][52] + coeff0_28*dmats2[28][52] + coeff0_29*dmats2[29][52] + coeff0_30*dmats2[30][52] + coeff0_31*dmats2[31][52] + coeff0_32*dmats2[32][52] + coeff0_33*dmats2[33][52] + coeff0_34*dmats2[34][52] + coeff0_35*dmats2[35][52] + coeff0_36*dmats2[36][52] + coeff0_37*dmats2[37][52] + coeff0_38*dmats2[38][52] + coeff0_39*dmats2[39][52] + coeff0_40*dmats2[40][52] + coeff0_41*dmats2[41][52] + coeff0_42*dmats2[42][52] + coeff0_43*dmats2[43][52] + coeff0_44*dmats2[44][52] + coeff0_45*dmats2[45][52] + coeff0_46*dmats2[46][52] + coeff0_47*dmats2[47][52] + coeff0_48*dmats2[48][52] + coeff0_49*dmats2[49][52] + coeff0_50*dmats2[50][52] + coeff0_51*dmats2[51][52] + coeff0_52*dmats2[52][52] + coeff0_53*dmats2[53][52] + coeff0_54*dmats2[54][52] + coeff0_55*dmats2[55][52];
          new_coeff0_53 = coeff0_0*dmats2[0][53] + coeff0_1*dmats2[1][53] + coeff0_2*dmats2[2][53] + coeff0_3*dmats2[3][53] + coeff0_4*dmats2[4][53] + coeff0_5*dmats2[5][53] + coeff0_6*dmats2[6][53] + coeff0_7*dmats2[7][53] + coeff0_8*dmats2[8][53] + coeff0_9*dmats2[9][53] + coeff0_10*dmats2[10][53] + coeff0_11*dmats2[11][53] + coeff0_12*dmats2[12][53] + coeff0_13*dmats2[13][53] + coeff0_14*dmats2[14][53] + coeff0_15*dmats2[15][53] + coeff0_16*dmats2[16][53] + coeff0_17*dmats2[17][53] + coeff0_18*dmats2[18][53] + coeff0_19*dmats2[19][53] + coeff0_20*dmats2[20][53] + coeff0_21*dmats2[21][53] + coeff0_22*dmats2[22][53] + coeff0_23*dmats2[23][53] + coeff0_24*dmats2[24][53] + coeff0_25*dmats2[25][53] + coeff0_26*dmats2[26][53] + coeff0_27*dmats2[27][53] + coeff0_28*dmats2[28][53] + coeff0_29*dmats2[29][53] + coeff0_30*dmats2[30][53] + coeff0_31*dmats2[31][53] + coeff0_32*dmats2[32][53] + coeff0_33*dmats2[33][53] + coeff0_34*dmats2[34][53] + coeff0_35*dmats2[35][53] + coeff0_36*dmats2[36][53] + coeff0_37*dmats2[37][53] + coeff0_38*dmats2[38][53] + coeff0_39*dmats2[39][53] + coeff0_40*dmats2[40][53] + coeff0_41*dmats2[41][53] + coeff0_42*dmats2[42][53] + coeff0_43*dmats2[43][53] + coeff0_44*dmats2[44][53] + coeff0_45*dmats2[45][53] + coeff0_46*dmats2[46][53] + coeff0_47*dmats2[47][53] + coeff0_48*dmats2[48][53] + coeff0_49*dmats2[49][53] + coeff0_50*dmats2[50][53] + coeff0_51*dmats2[51][53] + coeff0_52*dmats2[52][53] + coeff0_53*dmats2[53][53] + coeff0_54*dmats2[54][53] + coeff0_55*dmats2[55][53];
          new_coeff0_54 = coeff0_0*dmats2[0][54] + coeff0_1*dmats2[1][54] + coeff0_2*dmats2[2][54] + coeff0_3*dmats2[3][54] + coeff0_4*dmats2[4][54] + coeff0_5*dmats2[5][54] + coeff0_6*dmats2[6][54] + coeff0_7*dmats2[7][54] + coeff0_8*dmats2[8][54] + coeff0_9*dmats2[9][54] + coeff0_10*dmats2[10][54] + coeff0_11*dmats2[11][54] + coeff0_12*dmats2[12][54] + coeff0_13*dmats2[13][54] + coeff0_14*dmats2[14][54] + coeff0_15*dmats2[15][54] + coeff0_16*dmats2[16][54] + coeff0_17*dmats2[17][54] + coeff0_18*dmats2[18][54] + coeff0_19*dmats2[19][54] + coeff0_20*dmats2[20][54] + coeff0_21*dmats2[21][54] + coeff0_22*dmats2[22][54] + coeff0_23*dmats2[23][54] + coeff0_24*dmats2[24][54] + coeff0_25*dmats2[25][54] + coeff0_26*dmats2[26][54] + coeff0_27*dmats2[27][54] + coeff0_28*dmats2[28][54] + coeff0_29*dmats2[29][54] + coeff0_30*dmats2[30][54] + coeff0_31*dmats2[31][54] + coeff0_32*dmats2[32][54] + coeff0_33*dmats2[33][54] + coeff0_34*dmats2[34][54] + coeff0_35*dmats2[35][54] + coeff0_36*dmats2[36][54] + coeff0_37*dmats2[37][54] + coeff0_38*dmats2[38][54] + coeff0_39*dmats2[39][54] + coeff0_40*dmats2[40][54] + coeff0_41*dmats2[41][54] + coeff0_42*dmats2[42][54] + coeff0_43*dmats2[43][54] + coeff0_44*dmats2[44][54] + coeff0_45*dmats2[45][54] + coeff0_46*dmats2[46][54] + coeff0_47*dmats2[47][54] + coeff0_48*dmats2[48][54] + coeff0_49*dmats2[49][54] + coeff0_50*dmats2[50][54] + coeff0_51*dmats2[51][54] + coeff0_52*dmats2[52][54] + coeff0_53*dmats2[53][54] + coeff0_54*dmats2[54][54] + coeff0_55*dmats2[55][54];
          new_coeff0_55 = coeff0_0*dmats2[0][55] + coeff0_1*dmats2[1][55] + coeff0_2*dmats2[2][55] + coeff0_3*dmats2[3][55] + coeff0_4*dmats2[4][55] + coeff0_5*dmats2[5][55] + coeff0_6*dmats2[6][55] + coeff0_7*dmats2[7][55] + coeff0_8*dmats2[8][55] + coeff0_9*dmats2[9][55] + coeff0_10*dmats2[10][55] + coeff0_11*dmats2[11][55] + coeff0_12*dmats2[12][55] + coeff0_13*dmats2[13][55] + coeff0_14*dmats2[14][55] + coeff0_15*dmats2[15][55] + coeff0_16*dmats2[16][55] + coeff0_17*dmats2[17][55] + coeff0_18*dmats2[18][55] + coeff0_19*dmats2[19][55] + coeff0_20*dmats2[20][55] + coeff0_21*dmats2[21][55] + coeff0_22*dmats2[22][55] + coeff0_23*dmats2[23][55] + coeff0_24*dmats2[24][55] + coeff0_25*dmats2[25][55] + coeff0_26*dmats2[26][55] + coeff0_27*dmats2[27][55] + coeff0_28*dmats2[28][55] + coeff0_29*dmats2[29][55] + coeff0_30*dmats2[30][55] + coeff0_31*dmats2[31][55] + coeff0_32*dmats2[32][55] + coeff0_33*dmats2[33][55] + coeff0_34*dmats2[34][55] + coeff0_35*dmats2[35][55] + coeff0_36*dmats2[36][55] + coeff0_37*dmats2[37][55] + coeff0_38*dmats2[38][55] + coeff0_39*dmats2[39][55] + coeff0_40*dmats2[40][55] + coeff0_41*dmats2[41][55] + coeff0_42*dmats2[42][55] + coeff0_43*dmats2[43][55] + coeff0_44*dmats2[44][55] + coeff0_45*dmats2[45][55] + coeff0_46*dmats2[46][55] + coeff0_47*dmats2[47][55] + coeff0_48*dmats2[48][55] + coeff0_49*dmats2[49][55] + coeff0_50*dmats2[50][55] + coeff0_51*dmats2[51][55] + coeff0_52*dmats2[52][55] + coeff0_53*dmats2[53][55] + coeff0_54*dmats2[54][55] + coeff0_55*dmats2[55][55];
        }
    
      }
      // Compute derivatives on reference element as dot product of coefficients and basisvalues
      derivatives[deriv_num] = new_coeff0_0*basisvalue0 + new_coeff0_1*basisvalue1 + new_coeff0_2*basisvalue2 + new_coeff0_3*basisvalue3 + new_coeff0_4*basisvalue4 + new_coeff0_5*basisvalue5 + new_coeff0_6*basisvalue6 + new_coeff0_7*basisvalue7 + new_coeff0_8*basisvalue8 + new_coeff0_9*basisvalue9 + new_coeff0_10*basisvalue10 + new_coeff0_11*basisvalue11 + new_coeff0_12*basisvalue12 + new_coeff0_13*basisvalue13 + new_coeff0_14*basisvalue14 + new_coeff0_15*basisvalue15 + new_coeff0_16*basisvalue16 + new_coeff0_17*basisvalue17 + new_coeff0_18*basisvalue18 + new_coeff0_19*basisvalue19 + new_coeff0_20*basisvalue20 + new_coeff0_21*basisvalue21 + new_coeff0_22*basisvalue22 + new_coeff0_23*basisvalue23 + new_coeff0_24*basisvalue24 + new_coeff0_25*basisvalue25 + new_coeff0_26*basisvalue26 + new_coeff0_27*basisvalue27 + new_coeff0_28*basisvalue28 + new_coeff0_29*basisvalue29 + new_coeff0_30*basisvalue30 + new_coeff0_31*basisvalue31 + new_coeff0_32*basisvalue32 + new_coeff0_33*basisvalue33 + new_coeff0_34*basisvalue34 + new_coeff0_35*basisvalue35 + new_coeff0_36*basisvalue36 + new_coeff0_37*basisvalue37 + new_coeff0_38*basisvalue38 + new_coeff0_39*basisvalue39 + new_coeff0_40*basisvalue40 + new_coeff0_41*basisvalue41 + new_coeff0_42*basisvalue42 + new_coeff0_43*basisvalue43 + new_coeff0_44*basisvalue44 + new_coeff0_45*basisvalue45 + new_coeff0_46*basisvalue46 + new_coeff0_47*basisvalue47 + new_coeff0_48*basisvalue48 + new_coeff0_49*basisvalue49 + new_coeff0_50*basisvalue50 + new_coeff0_51*basisvalue51 + new_coeff0_52*basisvalue52 + new_coeff0_53*basisvalue53 + new_coeff0_54*basisvalue54 + new_coeff0_55*basisvalue55;
    }
    
    // Transform derivatives back to physical element
    for (unsigned int row = 0; row < num_derivatives; row++)
    {
      for (unsigned int col = 0; col < num_derivatives; col++)
      {
        values[row] += transform[row][col]*derivatives[col];
      }
    }
    // Delete pointer to array of derivatives on FIAT element
    delete [] derivatives;
    
    // Delete pointer to array of combinations of derivatives and transform
    for (unsigned int row = 0; row < num_derivatives; row++)
    {
      delete [] combinations[row];
      delete [] transform[row];
    }
    
    delete [] combinations;
    delete [] transform;
  }

  /// Evaluate order n derivatives of all basis functions at given point in cell
  virtual void evaluate_basis_derivatives_all(unsigned int n,
                                              double* values,
                                              const double* coordinates,
                                              const ufc::cell& c) const
  {
    throw std::runtime_error("The vectorised version of evaluate_basis_derivatives() is not yet implemented.");
  }

  /// Evaluate linear functional for dof i on the function f
  virtual double evaluate_dof(unsigned int i,
                              const ufc::function& f,
                              const ufc::cell& c) const
  {
    // The reference points, direction and weights:
    const static double X[56][1][3] = {{{0, 0, 0}}, {{1, 0, 0}}, {{0, 1, 0}}, {{0, 0, 1}}, {{0, 0.8, 0.2}}, {{0, 0.6, 0.4}}, {{0, 0.4, 0.6}}, {{0, 0.2, 0.8}}, {{0.8, 0, 0.2}}, {{0.6, 0, 0.4}}, {{0.4, 0, 0.6}}, {{0.2, 0, 0.8}}, {{0.8, 0.2, 0}}, {{0.6, 0.4, 0}}, {{0.4, 0.6, 0}}, {{0.2, 0.8, 0}}, {{0, 0, 0.2}}, {{0, 0, 0.4}}, {{0, 0, 0.6}}, {{0, 0, 0.8}}, {{0, 0.2, 0}}, {{0, 0.4, 0}}, {{0, 0.6, 0}}, {{0, 0.8, 0}}, {{0.2, 0, 0}}, {{0.4, 0, 0}}, {{0.6, 0, 0}}, {{0.8, 0, 0}}, {{0.6, 0.2, 0.2}}, {{0.4, 0.4, 0.2}}, {{0.2, 0.6, 0.2}}, {{0.4, 0.2, 0.4}}, {{0.2, 0.4, 0.4}}, {{0.2, 0.2, 0.6}}, {{0, 0.2, 0.2}}, {{0, 0.4, 0.2}}, {{0, 0.6, 0.2}}, {{0, 0.2, 0.4}}, {{0, 0.4, 0.4}}, {{0, 0.2, 0.6}}, {{0.2, 0, 0.2}}, {{0.4, 0, 0.2}}, {{0.6, 0, 0.2}}, {{0.2, 0, 0.4}}, {{0.4, 0, 0.4}}, {{0.2, 0, 0.6}}, {{0.2, 0.2, 0}}, {{0.4, 0.2, 0}}, {{0.6, 0.2, 0}}, {{0.2, 0.4, 0}}, {{0.4, 0.4, 0}}, {{0.2, 0.6, 0}}, {{0.2, 0.2, 0.2}}, {{0.4, 0.2, 0.2}}, {{0.2, 0.4, 0.2}}, {{0.2, 0.2, 0.4}}};
    const static double W[56][1] = {{1}, {1}, {1}, {1}, {1}, {1}, {1}, {1}, {1}, {1}, {1}, {1}, {1}, {1}, {1}, {1}, {1}, {1}, {1}, {1}, {1}, {1}, {1}, {1}, {1}, {1}, {1}, {1}, {1}, {1}, {1}, {1}, {1}, {1}, {1}, {1}, {1}, {1}, {1}, {1}, {1}, {1}, {1}, {1}, {1}, {1}, {1}, {1}, {1}, {1}, {1}, {1}, {1}, {1}, {1}, {1}};
    const static double D[56][1][1] = {{{1}}, {{1}}, {{1}}, {{1}}, {{1}}, {{1}}, {{1}}, {{1}}, {{1}}, {{1}}, {{1}}, {{1}}, {{1}}, {{1}}, {{1}}, {{1}}, {{1}}, {{1}}, {{1}}, {{1}}, {{1}}, {{1}}, {{1}}, {{1}}, {{1}}, {{1}}, {{1}}, {{1}}, {{1}}, {{1}}, {{1}}, {{1}}, {{1}}, {{1}}, {{1}}, {{1}}, {{1}}, {{1}}, {{1}}, {{1}}, {{1}}, {{1}}, {{1}}, {{1}}, {{1}}, {{1}}, {{1}}, {{1}}, {{1}}, {{1}}, {{1}}, {{1}}, {{1}}, {{1}}, {{1}}, {{1}}};
    
    const double * const * x = c.coordinates;
    double result = 0.0;
    // Iterate over the points:
    // Evaluate basis functions for affine mapping
    const double w0 = 1.0 - X[i][0][0] - X[i][0][1] - X[i][0][2];
    const double w1 = X[i][0][0];
    const double w2 = X[i][0][1];
    const double w3 = X[i][0][2];
    
    // Compute affine mapping y = F(X)
    double y[3];
    y[0] = w0*x[0][0] + w1*x[1][0] + w2*x[2][0] + w3*x[3][0];
    y[1] = w0*x[0][1] + w1*x[1][1] + w2*x[2][1] + w3*x[3][1];
    y[2] = w0*x[0][2] + w1*x[1][2] + w2*x[2][2] + w3*x[3][2];
    
    // Evaluate function at physical points
    double values[1];
    f.evaluate(values, y, c);
    
    // Map function values using appropriate mapping
    // Affine map: Do nothing
    
    // Note that we do not map the weights (yet).
    
    // Take directional components
    for(int k = 0; k < 1; k++)
      result += values[k]*D[i][0][k];
    // Multiply by weights 
    result *= W[i][0];
    
    return result;
  }

  /// Evaluate linear functionals for all dofs on the function f
  virtual void evaluate_dofs(double* values,
                             const ufc::function& f,
                             const ufc::cell& c) const
  {
    throw std::runtime_error("Not implemented (introduced in UFC v1.1).");
  }

  /// Interpolate vertex values from dof values
  virtual void interpolate_vertex_values(double* vertex_values,
                                         const double* dof_values,
                                         const ufc::cell& c) const
  {
    // Evaluate at vertices and use affine mapping
    vertex_values[0] = dof_values[0];
    vertex_values[1] = dof_values[1];
    vertex_values[2] = dof_values[2];
    vertex_values[3] = dof_values[3];
  }

  /// Return the number of sub elements (for a mixed element)
  virtual unsigned int num_sub_elements() const
  {
    return 1;
  }

  /// Create a new finite element for sub element i (for a mixed element)
  virtual ufc::finite_element* create_sub_element(unsigned int i) const
  {
    return new UFC_Poisson3D_5LinearForm_finite_element_1();
  }

};

/// This class defines the interface for a local-to-global mapping of
/// degrees of freedom (dofs).

class UFC_Poisson3D_5LinearForm_dof_map_0: public ufc::dof_map
{
private:

  unsigned int __global_dimension;

public:

  /// Constructor
  UFC_Poisson3D_5LinearForm_dof_map_0() : ufc::dof_map()
  {
    __global_dimension = 0;
  }

  /// Destructor
  virtual ~UFC_Poisson3D_5LinearForm_dof_map_0()
  {
    // Do nothing
  }

  /// Return a string identifying the dof map
  virtual const char* signature() const
  {
    return "FFC dof map for Lagrange finite element of degree 5 on a tetrahedron";
  }

  /// Return true iff mesh entities of topological dimension d are needed
  virtual bool needs_mesh_entities(unsigned int d) const
  {
    switch ( d )
    {
    case 0:
      return true;
      break;
    case 1:
      return true;
      break;
    case 2:
      return true;
      break;
    case 3:
      return true;
      break;
    }
    return false;
  }

  /// Initialize dof map for mesh (return true iff init_cell() is needed)
  virtual bool init_mesh(const ufc::mesh& m)
  {
    __global_dimension = m.num_entities[0] + 4*m.num_entities[1] + 6*m.num_entities[2] + 4*m.num_entities[3];
    return false;
  }

  /// Initialize dof map for given cell
  virtual void init_cell(const ufc::mesh& m,
                         const ufc::cell& c)
  {
    // Do nothing
  }

  /// Finish initialization of dof map for cells
  virtual void init_cell_finalize()
  {
    // Do nothing
  }

  /// Return the dimension of the global finite element function space
  virtual unsigned int global_dimension() const
  {
    return __global_dimension;
  }

  /// Return the dimension of the local finite element function space
  virtual unsigned int local_dimension() const
  {
    return 56;
  }

  // Return the geometric dimension of the coordinates this dof map provides
  virtual unsigned int geometric_dimension() const
  {
    return 3;
  }

  /// Return the number of dofs on each cell facet
  virtual unsigned int num_facet_dofs() const
  {
    return 21;
  }

  /// Return the number of dofs associated with each cell entity of dimension d
  virtual unsigned int num_entity_dofs(unsigned int d) const
  {
    throw std::runtime_error("Not implemented (introduced in UFC v1.1).");
  }

  /// Tabulate the local-to-global mapping of dofs on a cell
  virtual void tabulate_dofs(unsigned int* dofs,
                             const ufc::mesh& m,
                             const ufc::cell& c) const
  {
    dofs[0] = c.entity_indices[0][0];
    dofs[1] = c.entity_indices[0][1];
    dofs[2] = c.entity_indices[0][2];
    dofs[3] = c.entity_indices[0][3];
    unsigned int offset = m.num_entities[0];
    dofs[4] = offset + 4*c.entity_indices[1][0];
    dofs[5] = offset + 4*c.entity_indices[1][0] + 1;
    dofs[6] = offset + 4*c.entity_indices[1][0] + 2;
    dofs[7] = offset + 4*c.entity_indices[1][0] + 3;
    dofs[8] = offset + 4*c.entity_indices[1][1];
    dofs[9] = offset + 4*c.entity_indices[1][1] + 1;
    dofs[10] = offset + 4*c.entity_indices[1][1] + 2;
    dofs[11] = offset + 4*c.entity_indices[1][1] + 3;
    dofs[12] = offset + 4*c.entity_indices[1][2];
    dofs[13] = offset + 4*c.entity_indices[1][2] + 1;
    dofs[14] = offset + 4*c.entity_indices[1][2] + 2;
    dofs[15] = offset + 4*c.entity_indices[1][2] + 3;
    dofs[16] = offset + 4*c.entity_indices[1][3];
    dofs[17] = offset + 4*c.entity_indices[1][3] + 1;
    dofs[18] = offset + 4*c.entity_indices[1][3] + 2;
    dofs[19] = offset + 4*c.entity_indices[1][3] + 3;
    dofs[20] = offset + 4*c.entity_indices[1][4];
    dofs[21] = offset + 4*c.entity_indices[1][4] + 1;
    dofs[22] = offset + 4*c.entity_indices[1][4] + 2;
    dofs[23] = offset + 4*c.entity_indices[1][4] + 3;
    dofs[24] = offset + 4*c.entity_indices[1][5];
    dofs[25] = offset + 4*c.entity_indices[1][5] + 1;
    dofs[26] = offset + 4*c.entity_indices[1][5] + 2;
    dofs[27] = offset + 4*c.entity_indices[1][5] + 3;
    offset = offset + 4*m.num_entities[1];
    dofs[28] = offset + 6*c.entity_indices[2][0];
    dofs[29] = offset + 6*c.entity_indices[2][0] + 1;
    dofs[30] = offset + 6*c.entity_indices[2][0] + 2;
    dofs[31] = offset + 6*c.entity_indices[2][0] + 3;
    dofs[32] = offset + 6*c.entity_indices[2][0] + 4;
    dofs[33] = offset + 6*c.entity_indices[2][0] + 5;
    dofs[34] = offset + 6*c.entity_indices[2][1];
    dofs[35] = offset + 6*c.entity_indices[2][1] + 1;
    dofs[36] = offset + 6*c.entity_indices[2][1] + 2;
    dofs[37] = offset + 6*c.entity_indices[2][1] + 3;
    dofs[38] = offset + 6*c.entity_indices[2][1] + 4;
    dofs[39] = offset + 6*c.entity_indices[2][1] + 5;
    dofs[40] = offset + 6*c.entity_indices[2][2];
    dofs[41] = offset + 6*c.entity_indices[2][2] + 1;
    dofs[42] = offset + 6*c.entity_indices[2][2] + 2;
    dofs[43] = offset + 6*c.entity_indices[2][2] + 3;
    dofs[44] = offset + 6*c.entity_indices[2][2] + 4;
    dofs[45] = offset + 6*c.entity_indices[2][2] + 5;
    dofs[46] = offset + 6*c.entity_indices[2][3];
    dofs[47] = offset + 6*c.entity_indices[2][3] + 1;
    dofs[48] = offset + 6*c.entity_indices[2][3] + 2;
    dofs[49] = offset + 6*c.entity_indices[2][3] + 3;
    dofs[50] = offset + 6*c.entity_indices[2][3] + 4;
    dofs[51] = offset + 6*c.entity_indices[2][3] + 5;
    offset = offset + 6*m.num_entities[2];
    dofs[52] = offset + 4*c.entity_indices[3][0];
    dofs[53] = offset + 4*c.entity_indices[3][0] + 1;
    dofs[54] = offset + 4*c.entity_indices[3][0] + 2;
    dofs[55] = offset + 4*c.entity_indices[3][0] + 3;
  }

  /// Tabulate the local-to-local mapping from facet dofs to cell dofs
  virtual void tabulate_facet_dofs(unsigned int* dofs,
                                   unsigned int facet) const
  {
    switch ( facet )
    {
    case 0:
      dofs[0] = 1;
      dofs[1] = 2;
      dofs[2] = 3;
      dofs[3] = 4;
      dofs[4] = 5;
      dofs[5] = 6;
      dofs[6] = 7;
      dofs[7] = 8;
      dofs[8] = 9;
      dofs[9] = 10;
      dofs[10] = 11;
      dofs[11] = 12;
      dofs[12] = 13;
      dofs[13] = 14;
      dofs[14] = 15;
      dofs[15] = 28;
      dofs[16] = 29;
      dofs[17] = 30;
      dofs[18] = 31;
      dofs[19] = 32;
      dofs[20] = 33;
      break;
    case 1:
      dofs[0] = 0;
      dofs[1] = 2;
      dofs[2] = 3;
      dofs[3] = 4;
      dofs[4] = 5;
      dofs[5] = 6;
      dofs[6] = 7;
      dofs[7] = 16;
      dofs[8] = 17;
      dofs[9] = 18;
      dofs[10] = 19;
      dofs[11] = 20;
      dofs[12] = 21;
      dofs[13] = 22;
      dofs[14] = 23;
      dofs[15] = 34;
      dofs[16] = 35;
      dofs[17] = 36;
      dofs[18] = 37;
      dofs[19] = 38;
      dofs[20] = 39;
      break;
    case 2:
      dofs[0] = 0;
      dofs[1] = 1;
      dofs[2] = 3;
      dofs[3] = 8;
      dofs[4] = 9;
      dofs[5] = 10;
      dofs[6] = 11;
      dofs[7] = 16;
      dofs[8] = 17;
      dofs[9] = 18;
      dofs[10] = 19;
      dofs[11] = 24;
      dofs[12] = 25;
      dofs[13] = 26;
      dofs[14] = 27;
      dofs[15] = 40;
      dofs[16] = 41;
      dofs[17] = 42;
      dofs[18] = 43;
      dofs[19] = 44;
      dofs[20] = 45;
      break;
    case 3:
      dofs[0] = 0;
      dofs[1] = 1;
      dofs[2] = 2;
      dofs[3] = 12;
      dofs[4] = 13;
      dofs[5] = 14;
      dofs[6] = 15;
      dofs[7] = 20;
      dofs[8] = 21;
      dofs[9] = 22;
      dofs[10] = 23;
      dofs[11] = 24;
      dofs[12] = 25;
      dofs[13] = 26;
      dofs[14] = 27;
      dofs[15] = 46;
      dofs[16] = 47;
      dofs[17] = 48;
      dofs[18] = 49;
      dofs[19] = 50;
      dofs[20] = 51;
      break;
    }
  }

  /// Tabulate the local-to-local mapping of dofs on entity (d, i)
  virtual void tabulate_entity_dofs(unsigned int* dofs,
                                    unsigned int d, unsigned int i) const
  {
    throw std::runtime_error("Not implemented (introduced in UFC v1.1).");
  }

  /// Tabulate the coordinates of all dofs on a cell
  virtual void tabulate_coordinates(double** coordinates,
                                    const ufc::cell& c) const
  {
    const double * const * x = c.coordinates;
    coordinates[0][0] = x[0][0];
    coordinates[0][1] = x[0][1];
    coordinates[0][2] = x[0][2];
    coordinates[1][0] = x[1][0];
    coordinates[1][1] = x[1][1];
    coordinates[1][2] = x[1][2];
    coordinates[2][0] = x[2][0];
    coordinates[2][1] = x[2][1];
    coordinates[2][2] = x[2][2];
    coordinates[3][0] = x[3][0];
    coordinates[3][1] = x[3][1];
    coordinates[3][2] = x[3][2];
    coordinates[4][0] = 0.8*x[2][0] + 0.2*x[3][0];
    coordinates[4][1] = 0.8*x[2][1] + 0.2*x[3][1];
    coordinates[4][2] = 0.8*x[2][2] + 0.2*x[3][2];
    coordinates[5][0] = 0.6*x[2][0] + 0.4*x[3][0];
    coordinates[5][1] = 0.6*x[2][1] + 0.4*x[3][1];
    coordinates[5][2] = 0.6*x[2][2] + 0.4*x[3][2];
    coordinates[6][0] = 0.4*x[2][0] + 0.6*x[3][0];
    coordinates[6][1] = 0.4*x[2][1] + 0.6*x[3][1];
    coordinates[6][2] = 0.4*x[2][2] + 0.6*x[3][2];
    coordinates[7][0] = 0.2*x[2][0] + 0.8*x[3][0];
    coordinates[7][1] = 0.2*x[2][1] + 0.8*x[3][1];
    coordinates[7][2] = 0.2*x[2][2] + 0.8*x[3][2];
    coordinates[8][0] = 0.8*x[1][0] + 0.2*x[3][0];
    coordinates[8][1] = 0.8*x[1][1] + 0.2*x[3][1];
    coordinates[8][2] = 0.8*x[1][2] + 0.2*x[3][2];
    coordinates[9][0] = 0.6*x[1][0] + 0.4*x[3][0];
    coordinates[9][1] = 0.6*x[1][1] + 0.4*x[3][1];
    coordinates[9][2] = 0.6*x[1][2] + 0.4*x[3][2];
    coordinates[10][0] = 0.4*x[1][0] + 0.6*x[3][0];
    coordinates[10][1] = 0.4*x[1][1] + 0.6*x[3][1];
    coordinates[10][2] = 0.4*x[1][2] + 0.6*x[3][2];
    coordinates[11][0] = 0.2*x[1][0] + 0.8*x[3][0];
    coordinates[11][1] = 0.2*x[1][1] + 0.8*x[3][1];
    coordinates[11][2] = 0.2*x[1][2] + 0.8*x[3][2];
    coordinates[12][0] = 0.8*x[1][0] + 0.2*x[2][0];
    coordinates[12][1] = 0.8*x[1][1] + 0.2*x[2][1];
    coordinates[12][2] = 0.8*x[1][2] + 0.2*x[2][2];
    coordinates[13][0] = 0.6*x[1][0] + 0.4*x[2][0];
    coordinates[13][1] = 0.6*x[1][1] + 0.4*x[2][1];
    coordinates[13][2] = 0.6*x[1][2] + 0.4*x[2][2];
    coordinates[14][0] = 0.4*x[1][0] + 0.6*x[2][0];
    coordinates[14][1] = 0.4*x[1][1] + 0.6*x[2][1];
    coordinates[14][2] = 0.4*x[1][2] + 0.6*x[2][2];
    coordinates[15][0] = 0.2*x[1][0] + 0.8*x[2][0];
    coordinates[15][1] = 0.2*x[1][1] + 0.8*x[2][1];
    coordinates[15][2] = 0.2*x[1][2] + 0.8*x[2][2];
    coordinates[16][0] = 0.8*x[0][0] + 0.2*x[3][0];
    coordinates[16][1] = 0.8*x[0][1] + 0.2*x[3][1];
    coordinates[16][2] = 0.8*x[0][2] + 0.2*x[3][2];
    coordinates[17][0] = 0.6*x[0][0] + 0.4*x[3][0];
    coordinates[17][1] = 0.6*x[0][1] + 0.4*x[3][1];
    coordinates[17][2] = 0.6*x[0][2] + 0.4*x[3][2];
    coordinates[18][0] = 0.4*x[0][0] + 0.6*x[3][0];
    coordinates[18][1] = 0.4*x[0][1] + 0.6*x[3][1];
    coordinates[18][2] = 0.4*x[0][2] + 0.6*x[3][2];
    coordinates[19][0] = 0.2*x[0][0] + 0.8*x[3][0];
    coordinates[19][1] = 0.2*x[0][1] + 0.8*x[3][1];
    coordinates[19][2] = 0.2*x[0][2] + 0.8*x[3][2];
    coordinates[20][0] = 0.8*x[0][0] + 0.2*x[2][0];
    coordinates[20][1] = 0.8*x[0][1] + 0.2*x[2][1];
    coordinates[20][2] = 0.8*x[0][2] + 0.2*x[2][2];
    coordinates[21][0] = 0.6*x[0][0] + 0.4*x[2][0];
    coordinates[21][1] = 0.6*x[0][1] + 0.4*x[2][1];
    coordinates[21][2] = 0.6*x[0][2] + 0.4*x[2][2];
    coordinates[22][0] = 0.4*x[0][0] + 0.6*x[2][0];
    coordinates[22][1] = 0.4*x[0][1] + 0.6*x[2][1];
    coordinates[22][2] = 0.4*x[0][2] + 0.6*x[2][2];
    coordinates[23][0] = 0.2*x[0][0] + 0.8*x[2][0];
    coordinates[23][1] = 0.2*x[0][1] + 0.8*x[2][1];
    coordinates[23][2] = 0.2*x[0][2] + 0.8*x[2][2];
    coordinates[24][0] = 0.8*x[0][0] + 0.2*x[1][0];
    coordinates[24][1] = 0.8*x[0][1] + 0.2*x[1][1];
    coordinates[24][2] = 0.8*x[0][2] + 0.2*x[1][2];
    coordinates[25][0] = 0.6*x[0][0] + 0.4*x[1][0];
    coordinates[25][1] = 0.6*x[0][1] + 0.4*x[1][1];
    coordinates[25][2] = 0.6*x[0][2] + 0.4*x[1][2];
    coordinates[26][0] = 0.4*x[0][0] + 0.6*x[1][0];
    coordinates[26][1] = 0.4*x[0][1] + 0.6*x[1][1];
    coordinates[26][2] = 0.4*x[0][2] + 0.6*x[1][2];
    coordinates[27][0] = 0.2*x[0][0] + 0.8*x[1][0];
    coordinates[27][1] = 0.2*x[0][1] + 0.8*x[1][1];
    coordinates[27][2] = 0.2*x[0][2] + 0.8*x[1][2];
    coordinates[28][0] = 0.6*x[1][0] + 0.2*x[2][0] + 0.2*x[3][0];
    coordinates[28][1] = 0.6*x[1][1] + 0.2*x[2][1] + 0.2*x[3][1];
    coordinates[28][2] = 0.6*x[1][2] + 0.2*x[2][2] + 0.2*x[3][2];
    coordinates[29][0] = 0.4*x[1][0] + 0.4*x[2][0] + 0.2*x[3][0];
    coordinates[29][1] = 0.4*x[1][1] + 0.4*x[2][1] + 0.2*x[3][1];
    coordinates[29][2] = 0.4*x[1][2] + 0.4*x[2][2] + 0.2*x[3][2];
    coordinates[30][0] = 0.2*x[1][0] + 0.6*x[2][0] + 0.2*x[3][0];
    coordinates[30][1] = 0.2*x[1][1] + 0.6*x[2][1] + 0.2*x[3][1];
    coordinates[30][2] = 0.2*x[1][2] + 0.6*x[2][2] + 0.2*x[3][2];
    coordinates[31][0] = 0.4*x[1][0] + 0.2*x[2][0] + 0.4*x[3][0];
    coordinates[31][1] = 0.4*x[1][1] + 0.2*x[2][1] + 0.4*x[3][1];
    coordinates[31][2] = 0.4*x[1][2] + 0.2*x[2][2] + 0.4*x[3][2];
    coordinates[32][0] = 0.2*x[1][0] + 0.4*x[2][0] + 0.4*x[3][0];
    coordinates[32][1] = 0.2*x[1][1] + 0.4*x[2][1] + 0.4*x[3][1];
    coordinates[32][2] = 0.2*x[1][2] + 0.4*x[2][2] + 0.4*x[3][2];
    coordinates[33][0] = 0.2*x[1][0] + 0.2*x[2][0] + 0.6*x[3][0];
    coordinates[33][1] = 0.2*x[1][1] + 0.2*x[2][1] + 0.6*x[3][1];
    coordinates[33][2] = 0.2*x[1][2] + 0.2*x[2][2] + 0.6*x[3][2];
    coordinates[34][0] = 0.6*x[0][0] + 0.2*x[2][0] + 0.2*x[3][0];
    coordinates[34][1] = 0.6*x[0][1] + 0.2*x[2][1] + 0.2*x[3][1];
    coordinates[34][2] = 0.6*x[0][2] + 0.2*x[2][2] + 0.2*x[3][2];
    coordinates[35][0] = 0.4*x[0][0] + 0.4*x[2][0] + 0.2*x[3][0];
    coordinates[35][1] = 0.4*x[0][1] + 0.4*x[2][1] + 0.2*x[3][1];
    coordinates[35][2] = 0.4*x[0][2] + 0.4*x[2][2] + 0.2*x[3][2];
    coordinates[36][0] = 0.2*x[0][0] + 0.6*x[2][0] + 0.2*x[3][0];
    coordinates[36][1] = 0.2*x[0][1] + 0.6*x[2][1] + 0.2*x[3][1];
    coordinates[36][2] = 0.2*x[0][2] + 0.6*x[2][2] + 0.2*x[3][2];
    coordinates[37][0] = 0.4*x[0][0] + 0.2*x[2][0] + 0.4*x[3][0];
    coordinates[37][1] = 0.4*x[0][1] + 0.2*x[2][1] + 0.4*x[3][1];
    coordinates[37][2] = 0.4*x[0][2] + 0.2*x[2][2] + 0.4*x[3][2];
    coordinates[38][0] = 0.2*x[0][0] + 0.4*x[2][0] + 0.4*x[3][0];
    coordinates[38][1] = 0.2*x[0][1] + 0.4*x[2][1] + 0.4*x[3][1];
    coordinates[38][2] = 0.2*x[0][2] + 0.4*x[2][2] + 0.4*x[3][2];
    coordinates[39][0] = 0.2*x[0][0] + 0.2*x[2][0] + 0.6*x[3][0];
    coordinates[39][1] = 0.2*x[0][1] + 0.2*x[2][1] + 0.6*x[3][1];
    coordinates[39][2] = 0.2*x[0][2] + 0.2*x[2][2] + 0.6*x[3][2];
    coordinates[40][0] = 0.6*x[0][0] + 0.2*x[1][0] + 0.2*x[3][0];
    coordinates[40][1] = 0.6*x[0][1] + 0.2*x[1][1] + 0.2*x[3][1];
    coordinates[40][2] = 0.6*x[0][2] + 0.2*x[1][2] + 0.2*x[3][2];
    coordinates[41][0] = 0.4*x[0][0] + 0.4*x[1][0] + 0.2*x[3][0];
    coordinates[41][1] = 0.4*x[0][1] + 0.4*x[1][1] + 0.2*x[3][1];
    coordinates[41][2] = 0.4*x[0][2] + 0.4*x[1][2] + 0.2*x[3][2];
    coordinates[42][0] = 0.2*x[0][0] + 0.6*x[1][0] + 0.2*x[3][0];
    coordinates[42][1] = 0.2*x[0][1] + 0.6*x[1][1] + 0.2*x[3][1];
    coordinates[42][2] = 0.2*x[0][2] + 0.6*x[1][2] + 0.2*x[3][2];
    coordinates[43][0] = 0.4*x[0][0] + 0.2*x[1][0] + 0.4*x[3][0];
    coordinates[43][1] = 0.4*x[0][1] + 0.2*x[1][1] + 0.4*x[3][1];
    coordinates[43][2] = 0.4*x[0][2] + 0.2*x[1][2] + 0.4*x[3][2];
    coordinates[44][0] = 0.2*x[0][0] + 0.4*x[1][0] + 0.4*x[3][0];
    coordinates[44][1] = 0.2*x[0][1] + 0.4*x[1][1] + 0.4*x[3][1];
    coordinates[44][2] = 0.2*x[0][2] + 0.4*x[1][2] + 0.4*x[3][2];
    coordinates[45][0] = 0.2*x[0][0] + 0.2*x[1][0] + 0.6*x[3][0];
    coordinates[45][1] = 0.2*x[0][1] + 0.2*x[1][1] + 0.6*x[3][1];
    coordinates[45][2] = 0.2*x[0][2] + 0.2*x[1][2] + 0.6*x[3][2];
    coordinates[46][0] = 0.6*x[0][0] + 0.2*x[1][0] + 0.2*x[2][0];
    coordinates[46][1] = 0.6*x[0][1] + 0.2*x[1][1] + 0.2*x[2][1];
    coordinates[46][2] = 0.6*x[0][2] + 0.2*x[1][2] + 0.2*x[2][2];
    coordinates[47][0] = 0.4*x[0][0] + 0.4*x[1][0] + 0.2*x[2][0];
    coordinates[47][1] = 0.4*x[0][1] + 0.4*x[1][1] + 0.2*x[2][1];
    coordinates[47][2] = 0.4*x[0][2] + 0.4*x[1][2] + 0.2*x[2][2];
    coordinates[48][0] = 0.2*x[0][0] + 0.6*x[1][0] + 0.2*x[2][0];
    coordinates[48][1] = 0.2*x[0][1] + 0.6*x[1][1] + 0.2*x[2][1];
    coordinates[48][2] = 0.2*x[0][2] + 0.6*x[1][2] + 0.2*x[2][2];
    coordinates[49][0] = 0.4*x[0][0] + 0.2*x[1][0] + 0.4*x[2][0];
    coordinates[49][1] = 0.4*x[0][1] + 0.2*x[1][1] + 0.4*x[2][1];
    coordinates[49][2] = 0.4*x[0][2] + 0.2*x[1][2] + 0.4*x[2][2];
    coordinates[50][0] = 0.2*x[0][0] + 0.4*x[1][0] + 0.4*x[2][0];
    coordinates[50][1] = 0.2*x[0][1] + 0.4*x[1][1] + 0.4*x[2][1];
    coordinates[50][2] = 0.2*x[0][2] + 0.4*x[1][2] + 0.4*x[2][2];
    coordinates[51][0] = 0.2*x[0][0] + 0.2*x[1][0] + 0.6*x[2][0];
    coordinates[51][1] = 0.2*x[0][1] + 0.2*x[1][1] + 0.6*x[2][1];
    coordinates[51][2] = 0.2*x[0][2] + 0.2*x[1][2] + 0.6*x[2][2];
    coordinates[52][0] = 0.4*x[0][0] + 0.2*x[1][0] + 0.2*x[2][0] + 0.2*x[3][0];
    coordinates[52][1] = 0.4*x[0][1] + 0.2*x[1][1] + 0.2*x[2][1] + 0.2*x[3][1];
    coordinates[52][2] = 0.4*x[0][2] + 0.2*x[1][2] + 0.2*x[2][2] + 0.2*x[3][2];
    coordinates[53][0] = 0.2*x[0][0] + 0.4*x[1][0] + 0.2*x[2][0] + 0.2*x[3][0];
    coordinates[53][1] = 0.2*x[0][1] + 0.4*x[1][1] + 0.2*x[2][1] + 0.2*x[3][1];
    coordinates[53][2] = 0.2*x[0][2] + 0.4*x[1][2] + 0.2*x[2][2] + 0.2*x[3][2];
    coordinates[54][0] = 0.2*x[0][0] + 0.2*x[1][0] + 0.4*x[2][0] + 0.2*x[3][0];
    coordinates[54][1] = 0.2*x[0][1] + 0.2*x[1][1] + 0.4*x[2][1] + 0.2*x[3][1];
    coordinates[54][2] = 0.2*x[0][2] + 0.2*x[1][2] + 0.4*x[2][2] + 0.2*x[3][2];
    coordinates[55][0] = 0.2*x[0][0] + 0.2*x[1][0] + 0.2*x[2][0] + 0.4*x[3][0];
    coordinates[55][1] = 0.2*x[0][1] + 0.2*x[1][1] + 0.2*x[2][1] + 0.4*x[3][1];
    coordinates[55][2] = 0.2*x[0][2] + 0.2*x[1][2] + 0.2*x[2][2] + 0.4*x[3][2];
  }

  /// Return the number of sub dof maps (for a mixed element)
  virtual unsigned int num_sub_dof_maps() const
  {
    return 1;
  }

  /// Create a new dof_map for sub dof map i (for a mixed element)
  virtual ufc::dof_map* create_sub_dof_map(unsigned int i) const
  {
    return new UFC_Poisson3D_5LinearForm_dof_map_0();
  }

};

/// This class defines the interface for a local-to-global mapping of
/// degrees of freedom (dofs).

class UFC_Poisson3D_5LinearForm_dof_map_1: public ufc::dof_map
{
private:

  unsigned int __global_dimension;

public:

  /// Constructor
  UFC_Poisson3D_5LinearForm_dof_map_1() : ufc::dof_map()
  {
    __global_dimension = 0;
  }

  /// Destructor
  virtual ~UFC_Poisson3D_5LinearForm_dof_map_1()
  {
    // Do nothing
  }

  /// Return a string identifying the dof map
  virtual const char* signature() const
  {
    return "FFC dof map for Lagrange finite element of degree 5 on a tetrahedron";
  }

  /// Return true iff mesh entities of topological dimension d are needed
  virtual bool needs_mesh_entities(unsigned int d) const
  {
    switch ( d )
    {
    case 0:
      return true;
      break;
    case 1:
      return true;
      break;
    case 2:
      return true;
      break;
    case 3:
      return true;
      break;
    }
    return false;
  }

  /// Initialize dof map for mesh (return true iff init_cell() is needed)
  virtual bool init_mesh(const ufc::mesh& m)
  {
    __global_dimension = m.num_entities[0] + 4*m.num_entities[1] + 6*m.num_entities[2] + 4*m.num_entities[3];
    return false;
  }

  /// Initialize dof map for given cell
  virtual void init_cell(const ufc::mesh& m,
                         const ufc::cell& c)
  {
    // Do nothing
  }

  /// Finish initialization of dof map for cells
  virtual void init_cell_finalize()
  {
    // Do nothing
  }

  /// Return the dimension of the global finite element function space
  virtual unsigned int global_dimension() const
  {
    return __global_dimension;
  }

  /// Return the dimension of the local finite element function space
  virtual unsigned int local_dimension() const
  {
    return 56;
  }

  // Return the geometric dimension of the coordinates this dof map provides
  virtual unsigned int geometric_dimension() const
  {
    return 3;
  }

  /// Return the number of dofs on each cell facet
  virtual unsigned int num_facet_dofs() const
  {
    return 21;
  }

  /// Return the number of dofs associated with each cell entity of dimension d
  virtual unsigned int num_entity_dofs(unsigned int d) const
  {
    throw std::runtime_error("Not implemented (introduced in UFC v1.1).");
  }

  /// Tabulate the local-to-global mapping of dofs on a cell
  virtual void tabulate_dofs(unsigned int* dofs,
                             const ufc::mesh& m,
                             const ufc::cell& c) const
  {
    dofs[0] = c.entity_indices[0][0];
    dofs[1] = c.entity_indices[0][1];
    dofs[2] = c.entity_indices[0][2];
    dofs[3] = c.entity_indices[0][3];
    unsigned int offset = m.num_entities[0];
    dofs[4] = offset + 4*c.entity_indices[1][0];
    dofs[5] = offset + 4*c.entity_indices[1][0] + 1;
    dofs[6] = offset + 4*c.entity_indices[1][0] + 2;
    dofs[7] = offset + 4*c.entity_indices[1][0] + 3;
    dofs[8] = offset + 4*c.entity_indices[1][1];
    dofs[9] = offset + 4*c.entity_indices[1][1] + 1;
    dofs[10] = offset + 4*c.entity_indices[1][1] + 2;
    dofs[11] = offset + 4*c.entity_indices[1][1] + 3;
    dofs[12] = offset + 4*c.entity_indices[1][2];
    dofs[13] = offset + 4*c.entity_indices[1][2] + 1;
    dofs[14] = offset + 4*c.entity_indices[1][2] + 2;
    dofs[15] = offset + 4*c.entity_indices[1][2] + 3;
    dofs[16] = offset + 4*c.entity_indices[1][3];
    dofs[17] = offset + 4*c.entity_indices[1][3] + 1;
    dofs[18] = offset + 4*c.entity_indices[1][3] + 2;
    dofs[19] = offset + 4*c.entity_indices[1][3] + 3;
    dofs[20] = offset + 4*c.entity_indices[1][4];
    dofs[21] = offset + 4*c.entity_indices[1][4] + 1;
    dofs[22] = offset + 4*c.entity_indices[1][4] + 2;
    dofs[23] = offset + 4*c.entity_indices[1][4] + 3;
    dofs[24] = offset + 4*c.entity_indices[1][5];
    dofs[25] = offset + 4*c.entity_indices[1][5] + 1;
    dofs[26] = offset + 4*c.entity_indices[1][5] + 2;
    dofs[27] = offset + 4*c.entity_indices[1][5] + 3;
    offset = offset + 4*m.num_entities[1];
    dofs[28] = offset + 6*c.entity_indices[2][0];
    dofs[29] = offset + 6*c.entity_indices[2][0] + 1;
    dofs[30] = offset + 6*c.entity_indices[2][0] + 2;
    dofs[31] = offset + 6*c.entity_indices[2][0] + 3;
    dofs[32] = offset + 6*c.entity_indices[2][0] + 4;
    dofs[33] = offset + 6*c.entity_indices[2][0] + 5;
    dofs[34] = offset + 6*c.entity_indices[2][1];
    dofs[35] = offset + 6*c.entity_indices[2][1] + 1;
    dofs[36] = offset + 6*c.entity_indices[2][1] + 2;
    dofs[37] = offset + 6*c.entity_indices[2][1] + 3;
    dofs[38] = offset + 6*c.entity_indices[2][1] + 4;
    dofs[39] = offset + 6*c.entity_indices[2][1] + 5;
    dofs[40] = offset + 6*c.entity_indices[2][2];
    dofs[41] = offset + 6*c.entity_indices[2][2] + 1;
    dofs[42] = offset + 6*c.entity_indices[2][2] + 2;
    dofs[43] = offset + 6*c.entity_indices[2][2] + 3;
    dofs[44] = offset + 6*c.entity_indices[2][2] + 4;
    dofs[45] = offset + 6*c.entity_indices[2][2] + 5;
    dofs[46] = offset + 6*c.entity_indices[2][3];
    dofs[47] = offset + 6*c.entity_indices[2][3] + 1;
    dofs[48] = offset + 6*c.entity_indices[2][3] + 2;
    dofs[49] = offset + 6*c.entity_indices[2][3] + 3;
    dofs[50] = offset + 6*c.entity_indices[2][3] + 4;
    dofs[51] = offset + 6*c.entity_indices[2][3] + 5;
    offset = offset + 6*m.num_entities[2];
    dofs[52] = offset + 4*c.entity_indices[3][0];
    dofs[53] = offset + 4*c.entity_indices[3][0] + 1;
    dofs[54] = offset + 4*c.entity_indices[3][0] + 2;
    dofs[55] = offset + 4*c.entity_indices[3][0] + 3;
  }

  /// Tabulate the local-to-local mapping from facet dofs to cell dofs
  virtual void tabulate_facet_dofs(unsigned int* dofs,
                                   unsigned int facet) const
  {
    switch ( facet )
    {
    case 0:
      dofs[0] = 1;
      dofs[1] = 2;
      dofs[2] = 3;
      dofs[3] = 4;
      dofs[4] = 5;
      dofs[5] = 6;
      dofs[6] = 7;
      dofs[7] = 8;
      dofs[8] = 9;
      dofs[9] = 10;
      dofs[10] = 11;
      dofs[11] = 12;
      dofs[12] = 13;
      dofs[13] = 14;
      dofs[14] = 15;
      dofs[15] = 28;
      dofs[16] = 29;
      dofs[17] = 30;
      dofs[18] = 31;
      dofs[19] = 32;
      dofs[20] = 33;
      break;
    case 1:
      dofs[0] = 0;
      dofs[1] = 2;
      dofs[2] = 3;
      dofs[3] = 4;
      dofs[4] = 5;
      dofs[5] = 6;
      dofs[6] = 7;
      dofs[7] = 16;
      dofs[8] = 17;
      dofs[9] = 18;
      dofs[10] = 19;
      dofs[11] = 20;
      dofs[12] = 21;
      dofs[13] = 22;
      dofs[14] = 23;
      dofs[15] = 34;
      dofs[16] = 35;
      dofs[17] = 36;
      dofs[18] = 37;
      dofs[19] = 38;
      dofs[20] = 39;
      break;
    case 2:
      dofs[0] = 0;
      dofs[1] = 1;
      dofs[2] = 3;
      dofs[3] = 8;
      dofs[4] = 9;
      dofs[5] = 10;
      dofs[6] = 11;
      dofs[7] = 16;
      dofs[8] = 17;
      dofs[9] = 18;
      dofs[10] = 19;
      dofs[11] = 24;
      dofs[12] = 25;
      dofs[13] = 26;
      dofs[14] = 27;
      dofs[15] = 40;
      dofs[16] = 41;
      dofs[17] = 42;
      dofs[18] = 43;
      dofs[19] = 44;
      dofs[20] = 45;
      break;
    case 3:
      dofs[0] = 0;
      dofs[1] = 1;
      dofs[2] = 2;
      dofs[3] = 12;
      dofs[4] = 13;
      dofs[5] = 14;
      dofs[6] = 15;
      dofs[7] = 20;
      dofs[8] = 21;
      dofs[9] = 22;
      dofs[10] = 23;
      dofs[11] = 24;
      dofs[12] = 25;
      dofs[13] = 26;
      dofs[14] = 27;
      dofs[15] = 46;
      dofs[16] = 47;
      dofs[17] = 48;
      dofs[18] = 49;
      dofs[19] = 50;
      dofs[20] = 51;
      break;
    }
  }

  /// Tabulate the local-to-local mapping of dofs on entity (d, i)
  virtual void tabulate_entity_dofs(unsigned int* dofs,
                                    unsigned int d, unsigned int i) const
  {
    throw std::runtime_error("Not implemented (introduced in UFC v1.1).");
  }

  /// Tabulate the coordinates of all dofs on a cell
  virtual void tabulate_coordinates(double** coordinates,
                                    const ufc::cell& c) const
  {
    const double * const * x = c.coordinates;
    coordinates[0][0] = x[0][0];
    coordinates[0][1] = x[0][1];
    coordinates[0][2] = x[0][2];
    coordinates[1][0] = x[1][0];
    coordinates[1][1] = x[1][1];
    coordinates[1][2] = x[1][2];
    coordinates[2][0] = x[2][0];
    coordinates[2][1] = x[2][1];
    coordinates[2][2] = x[2][2];
    coordinates[3][0] = x[3][0];
    coordinates[3][1] = x[3][1];
    coordinates[3][2] = x[3][2];
    coordinates[4][0] = 0.8*x[2][0] + 0.2*x[3][0];
    coordinates[4][1] = 0.8*x[2][1] + 0.2*x[3][1];
    coordinates[4][2] = 0.8*x[2][2] + 0.2*x[3][2];
    coordinates[5][0] = 0.6*x[2][0] + 0.4*x[3][0];
    coordinates[5][1] = 0.6*x[2][1] + 0.4*x[3][1];
    coordinates[5][2] = 0.6*x[2][2] + 0.4*x[3][2];
    coordinates[6][0] = 0.4*x[2][0] + 0.6*x[3][0];
    coordinates[6][1] = 0.4*x[2][1] + 0.6*x[3][1];
    coordinates[6][2] = 0.4*x[2][2] + 0.6*x[3][2];
    coordinates[7][0] = 0.2*x[2][0] + 0.8*x[3][0];
    coordinates[7][1] = 0.2*x[2][1] + 0.8*x[3][1];
    coordinates[7][2] = 0.2*x[2][2] + 0.8*x[3][2];
    coordinates[8][0] = 0.8*x[1][0] + 0.2*x[3][0];
    coordinates[8][1] = 0.8*x[1][1] + 0.2*x[3][1];
    coordinates[8][2] = 0.8*x[1][2] + 0.2*x[3][2];
    coordinates[9][0] = 0.6*x[1][0] + 0.4*x[3][0];
    coordinates[9][1] = 0.6*x[1][1] + 0.4*x[3][1];
    coordinates[9][2] = 0.6*x[1][2] + 0.4*x[3][2];
    coordinates[10][0] = 0.4*x[1][0] + 0.6*x[3][0];
    coordinates[10][1] = 0.4*x[1][1] + 0.6*x[3][1];
    coordinates[10][2] = 0.4*x[1][2] + 0.6*x[3][2];
    coordinates[11][0] = 0.2*x[1][0] + 0.8*x[3][0];
    coordinates[11][1] = 0.2*x[1][1] + 0.8*x[3][1];
    coordinates[11][2] = 0.2*x[1][2] + 0.8*x[3][2];
    coordinates[12][0] = 0.8*x[1][0] + 0.2*x[2][0];
    coordinates[12][1] = 0.8*x[1][1] + 0.2*x[2][1];
    coordinates[12][2] = 0.8*x[1][2] + 0.2*x[2][2];
    coordinates[13][0] = 0.6*x[1][0] + 0.4*x[2][0];
    coordinates[13][1] = 0.6*x[1][1] + 0.4*x[2][1];
    coordinates[13][2] = 0.6*x[1][2] + 0.4*x[2][2];
    coordinates[14][0] = 0.4*x[1][0] + 0.6*x[2][0];
    coordinates[14][1] = 0.4*x[1][1] + 0.6*x[2][1];
    coordinates[14][2] = 0.4*x[1][2] + 0.6*x[2][2];
    coordinates[15][0] = 0.2*x[1][0] + 0.8*x[2][0];
    coordinates[15][1] = 0.2*x[1][1] + 0.8*x[2][1];
    coordinates[15][2] = 0.2*x[1][2] + 0.8*x[2][2];
    coordinates[16][0] = 0.8*x[0][0] + 0.2*x[3][0];
    coordinates[16][1] = 0.8*x[0][1] + 0.2*x[3][1];
    coordinates[16][2] = 0.8*x[0][2] + 0.2*x[3][2];
    coordinates[17][0] = 0.6*x[0][0] + 0.4*x[3][0];
    coordinates[17][1] = 0.6*x[0][1] + 0.4*x[3][1];
    coordinates[17][2] = 0.6*x[0][2] + 0.4*x[3][2];
    coordinates[18][0] = 0.4*x[0][0] + 0.6*x[3][0];
    coordinates[18][1] = 0.4*x[0][1] + 0.6*x[3][1];
    coordinates[18][2] = 0.4*x[0][2] + 0.6*x[3][2];
    coordinates[19][0] = 0.2*x[0][0] + 0.8*x[3][0];
    coordinates[19][1] = 0.2*x[0][1] + 0.8*x[3][1];
    coordinates[19][2] = 0.2*x[0][2] + 0.8*x[3][2];
    coordinates[20][0] = 0.8*x[0][0] + 0.2*x[2][0];
    coordinates[20][1] = 0.8*x[0][1] + 0.2*x[2][1];
    coordinates[20][2] = 0.8*x[0][2] + 0.2*x[2][2];
    coordinates[21][0] = 0.6*x[0][0] + 0.4*x[2][0];
    coordinates[21][1] = 0.6*x[0][1] + 0.4*x[2][1];
    coordinates[21][2] = 0.6*x[0][2] + 0.4*x[2][2];
    coordinates[22][0] = 0.4*x[0][0] + 0.6*x[2][0];
    coordinates[22][1] = 0.4*x[0][1] + 0.6*x[2][1];
    coordinates[22][2] = 0.4*x[0][2] + 0.6*x[2][2];
    coordinates[23][0] = 0.2*x[0][0] + 0.8*x[2][0];
    coordinates[23][1] = 0.2*x[0][1] + 0.8*x[2][1];
    coordinates[23][2] = 0.2*x[0][2] + 0.8*x[2][2];
    coordinates[24][0] = 0.8*x[0][0] + 0.2*x[1][0];
    coordinates[24][1] = 0.8*x[0][1] + 0.2*x[1][1];
    coordinates[24][2] = 0.8*x[0][2] + 0.2*x[1][2];
    coordinates[25][0] = 0.6*x[0][0] + 0.4*x[1][0];
    coordinates[25][1] = 0.6*x[0][1] + 0.4*x[1][1];
    coordinates[25][2] = 0.6*x[0][2] + 0.4*x[1][2];
    coordinates[26][0] = 0.4*x[0][0] + 0.6*x[1][0];
    coordinates[26][1] = 0.4*x[0][1] + 0.6*x[1][1];
    coordinates[26][2] = 0.4*x[0][2] + 0.6*x[1][2];
    coordinates[27][0] = 0.2*x[0][0] + 0.8*x[1][0];
    coordinates[27][1] = 0.2*x[0][1] + 0.8*x[1][1];
    coordinates[27][2] = 0.2*x[0][2] + 0.8*x[1][2];
    coordinates[28][0] = 0.6*x[1][0] + 0.2*x[2][0] + 0.2*x[3][0];
    coordinates[28][1] = 0.6*x[1][1] + 0.2*x[2][1] + 0.2*x[3][1];
    coordinates[28][2] = 0.6*x[1][2] + 0.2*x[2][2] + 0.2*x[3][2];
    coordinates[29][0] = 0.4*x[1][0] + 0.4*x[2][0] + 0.2*x[3][0];
    coordinates[29][1] = 0.4*x[1][1] + 0.4*x[2][1] + 0.2*x[3][1];
    coordinates[29][2] = 0.4*x[1][2] + 0.4*x[2][2] + 0.2*x[3][2];
    coordinates[30][0] = 0.2*x[1][0] + 0.6*x[2][0] + 0.2*x[3][0];
    coordinates[30][1] = 0.2*x[1][1] + 0.6*x[2][1] + 0.2*x[3][1];
    coordinates[30][2] = 0.2*x[1][2] + 0.6*x[2][2] + 0.2*x[3][2];
    coordinates[31][0] = 0.4*x[1][0] + 0.2*x[2][0] + 0.4*x[3][0];
    coordinates[31][1] = 0.4*x[1][1] + 0.2*x[2][1] + 0.4*x[3][1];
    coordinates[31][2] = 0.4*x[1][2] + 0.2*x[2][2] + 0.4*x[3][2];
    coordinates[32][0] = 0.2*x[1][0] + 0.4*x[2][0] + 0.4*x[3][0];
    coordinates[32][1] = 0.2*x[1][1] + 0.4*x[2][1] + 0.4*x[3][1];
    coordinates[32][2] = 0.2*x[1][2] + 0.4*x[2][2] + 0.4*x[3][2];
    coordinates[33][0] = 0.2*x[1][0] + 0.2*x[2][0] + 0.6*x[3][0];
    coordinates[33][1] = 0.2*x[1][1] + 0.2*x[2][1] + 0.6*x[3][1];
    coordinates[33][2] = 0.2*x[1][2] + 0.2*x[2][2] + 0.6*x[3][2];
    coordinates[34][0] = 0.6*x[0][0] + 0.2*x[2][0] + 0.2*x[3][0];
    coordinates[34][1] = 0.6*x[0][1] + 0.2*x[2][1] + 0.2*x[3][1];
    coordinates[34][2] = 0.6*x[0][2] + 0.2*x[2][2] + 0.2*x[3][2];
    coordinates[35][0] = 0.4*x[0][0] + 0.4*x[2][0] + 0.2*x[3][0];
    coordinates[35][1] = 0.4*x[0][1] + 0.4*x[2][1] + 0.2*x[3][1];
    coordinates[35][2] = 0.4*x[0][2] + 0.4*x[2][2] + 0.2*x[3][2];
    coordinates[36][0] = 0.2*x[0][0] + 0.6*x[2][0] + 0.2*x[3][0];
    coordinates[36][1] = 0.2*x[0][1] + 0.6*x[2][1] + 0.2*x[3][1];
    coordinates[36][2] = 0.2*x[0][2] + 0.6*x[2][2] + 0.2*x[3][2];
    coordinates[37][0] = 0.4*x[0][0] + 0.2*x[2][0] + 0.4*x[3][0];
    coordinates[37][1] = 0.4*x[0][1] + 0.2*x[2][1] + 0.4*x[3][1];
    coordinates[37][2] = 0.4*x[0][2] + 0.2*x[2][2] + 0.4*x[3][2];
    coordinates[38][0] = 0.2*x[0][0] + 0.4*x[2][0] + 0.4*x[3][0];
    coordinates[38][1] = 0.2*x[0][1] + 0.4*x[2][1] + 0.4*x[3][1];
    coordinates[38][2] = 0.2*x[0][2] + 0.4*x[2][2] + 0.4*x[3][2];
    coordinates[39][0] = 0.2*x[0][0] + 0.2*x[2][0] + 0.6*x[3][0];
    coordinates[39][1] = 0.2*x[0][1] + 0.2*x[2][1] + 0.6*x[3][1];
    coordinates[39][2] = 0.2*x[0][2] + 0.2*x[2][2] + 0.6*x[3][2];
    coordinates[40][0] = 0.6*x[0][0] + 0.2*x[1][0] + 0.2*x[3][0];
    coordinates[40][1] = 0.6*x[0][1] + 0.2*x[1][1] + 0.2*x[3][1];
    coordinates[40][2] = 0.6*x[0][2] + 0.2*x[1][2] + 0.2*x[3][2];
    coordinates[41][0] = 0.4*x[0][0] + 0.4*x[1][0] + 0.2*x[3][0];
    coordinates[41][1] = 0.4*x[0][1] + 0.4*x[1][1] + 0.2*x[3][1];
    coordinates[41][2] = 0.4*x[0][2] + 0.4*x[1][2] + 0.2*x[3][2];
    coordinates[42][0] = 0.2*x[0][0] + 0.6*x[1][0] + 0.2*x[3][0];
    coordinates[42][1] = 0.2*x[0][1] + 0.6*x[1][1] + 0.2*x[3][1];
    coordinates[42][2] = 0.2*x[0][2] + 0.6*x[1][2] + 0.2*x[3][2];
    coordinates[43][0] = 0.4*x[0][0] + 0.2*x[1][0] + 0.4*x[3][0];
    coordinates[43][1] = 0.4*x[0][1] + 0.2*x[1][1] + 0.4*x[3][1];
    coordinates[43][2] = 0.4*x[0][2] + 0.2*x[1][2] + 0.4*x[3][2];
    coordinates[44][0] = 0.2*x[0][0] + 0.4*x[1][0] + 0.4*x[3][0];
    coordinates[44][1] = 0.2*x[0][1] + 0.4*x[1][1] + 0.4*x[3][1];
    coordinates[44][2] = 0.2*x[0][2] + 0.4*x[1][2] + 0.4*x[3][2];
    coordinates[45][0] = 0.2*x[0][0] + 0.2*x[1][0] + 0.6*x[3][0];
    coordinates[45][1] = 0.2*x[0][1] + 0.2*x[1][1] + 0.6*x[3][1];
    coordinates[45][2] = 0.2*x[0][2] + 0.2*x[1][2] + 0.6*x[3][2];
    coordinates[46][0] = 0.6*x[0][0] + 0.2*x[1][0] + 0.2*x[2][0];
    coordinates[46][1] = 0.6*x[0][1] + 0.2*x[1][1] + 0.2*x[2][1];
    coordinates[46][2] = 0.6*x[0][2] + 0.2*x[1][2] + 0.2*x[2][2];
    coordinates[47][0] = 0.4*x[0][0] + 0.4*x[1][0] + 0.2*x[2][0];
    coordinates[47][1] = 0.4*x[0][1] + 0.4*x[1][1] + 0.2*x[2][1];
    coordinates[47][2] = 0.4*x[0][2] + 0.4*x[1][2] + 0.2*x[2][2];
    coordinates[48][0] = 0.2*x[0][0] + 0.6*x[1][0] + 0.2*x[2][0];
    coordinates[48][1] = 0.2*x[0][1] + 0.6*x[1][1] + 0.2*x[2][1];
    coordinates[48][2] = 0.2*x[0][2] + 0.6*x[1][2] + 0.2*x[2][2];
    coordinates[49][0] = 0.4*x[0][0] + 0.2*x[1][0] + 0.4*x[2][0];
    coordinates[49][1] = 0.4*x[0][1] + 0.2*x[1][1] + 0.4*x[2][1];
    coordinates[49][2] = 0.4*x[0][2] + 0.2*x[1][2] + 0.4*x[2][2];
    coordinates[50][0] = 0.2*x[0][0] + 0.4*x[1][0] + 0.4*x[2][0];
    coordinates[50][1] = 0.2*x[0][1] + 0.4*x[1][1] + 0.4*x[2][1];
    coordinates[50][2] = 0.2*x[0][2] + 0.4*x[1][2] + 0.4*x[2][2];
    coordinates[51][0] = 0.2*x[0][0] + 0.2*x[1][0] + 0.6*x[2][0];
    coordinates[51][1] = 0.2*x[0][1] + 0.2*x[1][1] + 0.6*x[2][1];
    coordinates[51][2] = 0.2*x[0][2] + 0.2*x[1][2] + 0.6*x[2][2];
    coordinates[52][0] = 0.4*x[0][0] + 0.2*x[1][0] + 0.2*x[2][0] + 0.2*x[3][0];
    coordinates[52][1] = 0.4*x[0][1] + 0.2*x[1][1] + 0.2*x[2][1] + 0.2*x[3][1];
    coordinates[52][2] = 0.4*x[0][2] + 0.2*x[1][2] + 0.2*x[2][2] + 0.2*x[3][2];
    coordinates[53][0] = 0.2*x[0][0] + 0.4*x[1][0] + 0.2*x[2][0] + 0.2*x[3][0];
    coordinates[53][1] = 0.2*x[0][1] + 0.4*x[1][1] + 0.2*x[2][1] + 0.2*x[3][1];
    coordinates[53][2] = 0.2*x[0][2] + 0.4*x[1][2] + 0.2*x[2][2] + 0.2*x[3][2];
    coordinates[54][0] = 0.2*x[0][0] + 0.2*x[1][0] + 0.4*x[2][0] + 0.2*x[3][0];
    coordinates[54][1] = 0.2*x[0][1] + 0.2*x[1][1] + 0.4*x[2][1] + 0.2*x[3][1];
    coordinates[54][2] = 0.2*x[0][2] + 0.2*x[1][2] + 0.4*x[2][2] + 0.2*x[3][2];
    coordinates[55][0] = 0.2*x[0][0] + 0.2*x[1][0] + 0.2*x[2][0] + 0.4*x[3][0];
    coordinates[55][1] = 0.2*x[0][1] + 0.2*x[1][1] + 0.2*x[2][1] + 0.4*x[3][1];
    coordinates[55][2] = 0.2*x[0][2] + 0.2*x[1][2] + 0.2*x[2][2] + 0.4*x[3][2];
  }

  /// Return the number of sub dof maps (for a mixed element)
  virtual unsigned int num_sub_dof_maps() const
  {
    return 1;
  }

  /// Create a new dof_map for sub dof map i (for a mixed element)
  virtual ufc::dof_map* create_sub_dof_map(unsigned int i) const
  {
    return new UFC_Poisson3D_5LinearForm_dof_map_1();
  }

};

/// This class defines the interface for the tabulation of the cell
/// tensor corresponding to the local contribution to a form from
/// the integral over a cell.

class UFC_Poisson3D_5LinearForm_cell_integral_0: public ufc::cell_integral
{
public:

  /// Constructor
  UFC_Poisson3D_5LinearForm_cell_integral_0() : ufc::cell_integral()
  {
    // Do nothing
  }

  /// Destructor
  virtual ~UFC_Poisson3D_5LinearForm_cell_integral_0()
  {
    // Do nothing
  }

  /// Tabulate the tensor for the contribution from a local cell
  virtual void tabulate_tensor(double* A,
                               const double * const * w,
                               const ufc::cell& c) const
  {
    // Extract vertex coordinates
    const double * const * x = c.coordinates;
    
    // Compute Jacobian of affine map from reference cell
    const double J_00 = x[1][0] - x[0][0];
    const double J_01 = x[2][0] - x[0][0];
    const double J_02 = x[3][0] - x[0][0];
    const double J_10 = x[1][1] - x[0][1];
    const double J_11 = x[2][1] - x[0][1];
    const double J_12 = x[3][1] - x[0][1];
    const double J_20 = x[1][2] - x[0][2];
    const double J_21 = x[2][2] - x[0][2];
    const double J_22 = x[3][2] - x[0][2];
      
    // Compute sub determinants
    const double d_00 = J_11*J_22 - J_12*J_21;
    
    const double d_10 = J_02*J_21 - J_01*J_22;
    
    const double d_20 = J_01*J_12 - J_02*J_11;
      
    // Compute determinant of Jacobian
    double detJ = J_00*d_00 + J_10*d_10 + J_20*d_20;
      
    // Compute inverse of Jacobian
    
    // Set scale factor
    const double det = std::abs(detJ);
    
    // Compute coefficients
    const double c0_0_0_0 = w[0][0];
    const double c0_0_0_1 = w[0][1];
    const double c0_0_0_2 = w[0][2];
    const double c0_0_0_3 = w[0][3];
    const double c0_0_0_4 = w[0][4];
    const double c0_0_0_5 = w[0][5];
    const double c0_0_0_6 = w[0][6];
    const double c0_0_0_7 = w[0][7];
    const double c0_0_0_8 = w[0][8];
    const double c0_0_0_9 = w[0][9];
    const double c0_0_0_10 = w[0][10];
    const double c0_0_0_11 = w[0][11];
    const double c0_0_0_12 = w[0][12];
    const double c0_0_0_13 = w[0][13];
    const double c0_0_0_14 = w[0][14];
    const double c0_0_0_15 = w[0][15];
    const double c0_0_0_16 = w[0][16];
    const double c0_0_0_17 = w[0][17];
    const double c0_0_0_18 = w[0][18];
    const double c0_0_0_19 = w[0][19];
    const double c0_0_0_20 = w[0][20];
    const double c0_0_0_21 = w[0][21];
    const double c0_0_0_22 = w[0][22];
    const double c0_0_0_23 = w[0][23];
    const double c0_0_0_24 = w[0][24];
    const double c0_0_0_25 = w[0][25];
    const double c0_0_0_26 = w[0][26];
    const double c0_0_0_27 = w[0][27];
    const double c0_0_0_28 = w[0][28];
    const double c0_0_0_29 = w[0][29];
    const double c0_0_0_30 = w[0][30];
    const double c0_0_0_31 = w[0][31];
    const double c0_0_0_32 = w[0][32];
    const double c0_0_0_33 = w[0][33];
    const double c0_0_0_34 = w[0][34];
    const double c0_0_0_35 = w[0][35];
    const double c0_0_0_36 = w[0][36];
    const double c0_0_0_37 = w[0][37];
    const double c0_0_0_38 = w[0][38];
    const double c0_0_0_39 = w[0][39];
    const double c0_0_0_40 = w[0][40];
    const double c0_0_0_41 = w[0][41];
    const double c0_0_0_42 = w[0][42];
    const double c0_0_0_43 = w[0][43];
    const double c0_0_0_44 = w[0][44];
    const double c0_0_0_45 = w[0][45];
    const double c0_0_0_46 = w[0][46];
    const double c0_0_0_47 = w[0][47];
    const double c0_0_0_48 = w[0][48];
    const double c0_0_0_49 = w[0][49];
    const double c0_0_0_50 = w[0][50];
    const double c0_0_0_51 = w[0][51];
    const double c0_0_0_52 = w[0][52];
    const double c0_0_0_53 = w[0][53];
    const double c0_0_0_54 = w[0][54];
    const double c0_0_0_55 = w[0][55];
    
    // Compute geometry tensors
    const double G0_0 = det*c0_0_0_0;
    const double G0_1 = det*c0_0_0_1;
    const double G0_2 = det*c0_0_0_2;
    const double G0_3 = det*c0_0_0_3;
    const double G0_4 = det*c0_0_0_4;
    const double G0_5 = det*c0_0_0_5;
    const double G0_6 = det*c0_0_0_6;
    const double G0_7 = det*c0_0_0_7;
    const double G0_8 = det*c0_0_0_8;
    const double G0_9 = det*c0_0_0_9;
    const double G0_10 = det*c0_0_0_10;
    const double G0_11 = det*c0_0_0_11;
    const double G0_12 = det*c0_0_0_12;
    const double G0_13 = det*c0_0_0_13;
    const double G0_14 = det*c0_0_0_14;
    const double G0_15 = det*c0_0_0_15;
    const double G0_16 = det*c0_0_0_16;
    const double G0_17 = det*c0_0_0_17;
    const double G0_18 = det*c0_0_0_18;
    const double G0_19 = det*c0_0_0_19;
    const double G0_20 = det*c0_0_0_20;
    const double G0_21 = det*c0_0_0_21;
    const double G0_22 = det*c0_0_0_22;
    const double G0_23 = det*c0_0_0_23;
    const double G0_24 = det*c0_0_0_24;
    const double G0_25 = det*c0_0_0_25;
    const double G0_26 = det*c0_0_0_26;
    const double G0_27 = det*c0_0_0_27;
    const double G0_28 = det*c0_0_0_28;
    const double G0_29 = det*c0_0_0_29;
    const double G0_30 = det*c0_0_0_30;
    const double G0_31 = det*c0_0_0_31;
    const double G0_32 = det*c0_0_0_32;
    const double G0_33 = det*c0_0_0_33;
    const double G0_34 = det*c0_0_0_34;
    const double G0_35 = det*c0_0_0_35;
    const double G0_36 = det*c0_0_0_36;
    const double G0_37 = det*c0_0_0_37;
    const double G0_38 = det*c0_0_0_38;
    const double G0_39 = det*c0_0_0_39;
    const double G0_40 = det*c0_0_0_40;
    const double G0_41 = det*c0_0_0_41;
    const double G0_42 = det*c0_0_0_42;
    const double G0_43 = det*c0_0_0_43;
    const double G0_44 = det*c0_0_0_44;
    const double G0_45 = det*c0_0_0_45;
    const double G0_46 = det*c0_0_0_46;
    const double G0_47 = det*c0_0_0_47;
    const double G0_48 = det*c0_0_0_48;
    const double G0_49 = det*c0_0_0_49;
    const double G0_50 = det*c0_0_0_50;
    const double G0_51 = det*c0_0_0_51;
    const double G0_52 = det*c0_0_0_52;
    const double G0_53 = det*c0_0_0_53;
    const double G0_54 = det*c0_0_0_54;
    const double G0_55 = det*c0_0_0_55;
    
    // Compute element tensor
    A[0] = 8.34957866207911e-05*G0_0 + 9.27851405282001e-06*G0_1 + 9.27851405282024e-06*G0_2 + 9.27851405282031e-06*G0_3 + 4.07498237359376e-06*G0_4 + 1.52962392545732e-05*G0_5 + 1.52962392545733e-05*G0_6 + 4.07498237359361e-06*G0_7 + 4.07498237359382e-06*G0_8 + 1.52962392545735e-05*G0_9 + 1.52962392545739e-05*G0_10 + 4.07498237359328e-06*G0_11 + 4.07498237359344e-06*G0_12 + 1.52962392545735e-05*G0_13 + 1.52962392545736e-05*G0_14 + 4.07498237359338e-06*G0_15 - 6.04020786312482e-05*G0_16 + 6.04020786312481e-05*G0_17 - 4.43831824040179e-05*G0_18 + 1.81868671452012e-05*G0_19 - 6.04020786312477e-05*G0_20 + 6.04020786312489e-05*G0_21 - 4.43831824040176e-05*G0_22 + 1.81868671452013e-05*G0_23 - 6.04020786312483e-05*G0_24 + 6.04020786312487e-05*G0_25 - 4.4383182404018e-05*G0_26 + 1.81868671452014e-05*G0_27 + 0.000173437673437684*G0_28 - 1.77151327324952e-05*G0_29 + 0.000173437673437684*G0_30 - 1.7715132732495e-05*G0_31 - 1.77151327324955e-05*G0_32 + 0.000173437673437684*G0_33 + 0.000149650214754391*G0_34 - 0.00012149670352796*G0_35 + 9.5952786925015e-05*G0_36 - 0.00012149670352796*G0_37 + 3.98465025201153e-05*G0_38 + 9.59527869250152e-05*G0_39 + 0.00014965021475439*G0_40 - 0.00012149670352796*G0_41 + 9.59527869250159e-05*G0_42 - 0.00012149670352796*G0_43 + 3.98465025201148e-05*G0_44 + 9.59527869250159e-05*G0_45 + 0.00014965021475439*G0_46 - 0.000121496703527959*G0_47 + 9.59527869250153e-05*G0_48 - 0.000121496703527961*G0_49 + 3.9846502520115e-05*G0_50 + 9.59527869250157e-05*G0_51 - 0.000280029576904591*G0_52 + 0.000204250963799587*G0_53 + 0.000204250963799587*G0_54 + 0.000204250963799585*G0_55;
    A[1] = 9.27851405282001e-06*G0_0 + 8.3495786620791e-05*G0_1 + 9.27851405281989e-06*G0_2 + 9.27851405282003e-06*G0_3 + 4.07498237359424e-06*G0_4 + 1.52962392545727e-05*G0_5 + 1.52962392545734e-05*G0_6 + 4.07498237359373e-06*G0_7 - 6.04020786312483e-05*G0_8 + 6.04020786312485e-05*G0_9 - 4.43831824040183e-05*G0_10 + 1.81868671452013e-05*G0_11 - 6.04020786312488e-05*G0_12 + 6.04020786312489e-05*G0_13 - 4.43831824040188e-05*G0_14 + 1.81868671452017e-05*G0_15 + 4.07498237359413e-06*G0_16 + 1.5296239254573e-05*G0_17 + 1.52962392545735e-05*G0_18 + 4.07498237359356e-06*G0_19 + 4.07498237359435e-06*G0_20 + 1.52962392545738e-05*G0_21 + 1.52962392545737e-05*G0_22 + 4.07498237359396e-06*G0_23 + 1.81868671452022e-05*G0_24 - 4.43831824040187e-05*G0_25 + 6.04020786312488e-05*G0_26 - 6.04020786312487e-05*G0_27 + 0.000149650214754388*G0_28 - 0.00012149670352796*G0_29 + 9.59527869250136e-05*G0_30 - 0.000121496703527959*G0_31 + 3.98465025201158e-05*G0_32 + 9.59527869250128e-05*G0_33 + 0.000173437673437683*G0_34 - 1.77151327324941e-05*G0_35 + 0.000173437673437682*G0_36 - 1.77151327324951e-05*G0_37 - 1.77151327324946e-05*G0_38 + 0.000173437673437682*G0_39 + 9.59527869250136e-05*G0_40 - 0.00012149670352796*G0_41 + 0.000149650214754388*G0_42 + 3.98465025201158e-05*G0_43 - 0.000121496703527959*G0_44 + 9.59527869250131e-05*G0_45 + 9.59527869250134e-05*G0_46 - 0.00012149670352796*G0_47 + 0.000149650214754388*G0_48 + 3.98465025201162e-05*G0_49 - 0.00012149670352796*G0_50 + 9.59527869250134e-05*G0_51 + 0.000204250963799585*G0_52 - 0.000280029576904594*G0_53 + 0.000204250963799587*G0_54 + 0.000204250963799583*G0_55;
    A[2] = 9.27851405282024e-06*G0_0 + 9.27851405281989e-06*G0_1 + 8.34957866207908e-05*G0_2 + 9.27851405282011e-06*G0_3 - 6.04020786312485e-05*G0_4 + 6.04020786312488e-05*G0_5 - 4.43831824040186e-05*G0_6 + 1.81868671452017e-05*G0_7 + 4.07498237359399e-06*G0_8 + 1.52962392545734e-05*G0_9 + 1.52962392545738e-05*G0_10 + 4.07498237359352e-06*G0_11 + 1.81868671452015e-05*G0_12 - 4.43831824040182e-05*G0_13 + 6.04020786312489e-05*G0_14 - 6.0402078631249e-05*G0_15 + 4.07498237359391e-06*G0_16 + 1.52962392545733e-05*G0_17 + 1.52962392545732e-05*G0_18 + 4.07498237359368e-06*G0_19 + 1.81868671452021e-05*G0_20 - 4.4383182404018e-05*G0_21 + 6.04020786312491e-05*G0_22 - 6.04020786312489e-05*G0_23 + 4.0749823735939e-06*G0_24 + 1.52962392545735e-05*G0_25 + 1.52962392545732e-05*G0_26 + 4.07498237359374e-06*G0_27 + 9.59527869250144e-05*G0_28 - 0.00012149670352796*G0_29 + 0.000149650214754389*G0_30 + 3.98465025201157e-05*G0_31 - 0.000121496703527959*G0_32 + 9.59527869250143e-05*G0_33 + 9.59527869250149e-05*G0_34 - 0.00012149670352796*G0_35 + 0.000149650214754389*G0_36 + 3.98465025201151e-05*G0_37 - 0.000121496703527959*G0_38 + 9.5952786925014e-05*G0_39 + 0.000173437673437683*G0_40 - 1.77151327324948e-05*G0_41 + 0.000173437673437683*G0_42 - 1.77151327324952e-05*G0_43 - 1.77151327324951e-05*G0_44 + 0.000173437673437683*G0_45 + 9.59527869250143e-05*G0_46 + 3.98465025201163e-05*G0_47 + 9.59527869250144e-05*G0_48 - 0.00012149670352796*G0_49 - 0.00012149670352796*G0_50 + 0.000149650214754389*G0_51 + 0.000204250963799586*G0_52 + 0.000204250963799587*G0_53 - 0.000280029576904592*G0_54 + 0.000204250963799586*G0_55;
    A[3] = 9.27851405282031e-06*G0_0 + 9.27851405282003e-06*G0_1 + 9.27851405282011e-06*G0_2 + 8.34957866207911e-05*G0_3 + 1.81868671452016e-05*G0_4 - 4.43831824040183e-05*G0_5 + 6.04020786312483e-05*G0_6 - 6.04020786312483e-05*G0_7 + 1.81868671452017e-05*G0_8 - 4.43831824040182e-05*G0_9 + 6.04020786312485e-05*G0_10 - 6.04020786312485e-05*G0_11 + 4.0749823735936e-06*G0_12 + 1.52962392545738e-05*G0_13 + 1.52962392545733e-05*G0_14 + 4.07498237359361e-06*G0_15 + 1.81868671452015e-05*G0_16 - 4.43831824040182e-05*G0_17 + 6.04020786312483e-05*G0_18 - 6.04020786312486e-05*G0_19 + 4.07498237359417e-06*G0_20 + 1.5296239254574e-05*G0_21 + 1.52962392545736e-05*G0_22 + 4.07498237359373e-06*G0_23 + 4.07498237359394e-06*G0_24 + 1.52962392545735e-05*G0_25 + 1.52962392545733e-05*G0_26 + 4.07498237359373e-06*G0_27 + 9.59527869250142e-05*G0_28 + 3.98465025201154e-05*G0_29 + 9.59527869250147e-05*G0_30 - 0.00012149670352796*G0_31 - 0.00012149670352796*G0_32 + 0.00014965021475439*G0_33 + 9.59527869250146e-05*G0_34 + 3.98465025201156e-05*G0_35 + 9.59527869250143e-05*G0_36 - 0.00012149670352796*G0_37 - 0.00012149670352796*G0_38 + 0.000149650214754389*G0_39 + 9.59527869250145e-05*G0_40 + 3.98465025201156e-05*G0_41 + 9.59527869250146e-05*G0_42 - 0.00012149670352796*G0_43 - 0.00012149670352796*G0_44 + 0.000149650214754389*G0_45 + 0.000173437673437683*G0_46 - 1.77151327324946e-05*G0_47 + 0.000173437673437683*G0_48 - 1.77151327324948e-05*G0_49 - 1.77151327324951e-05*G0_50 + 0.000173437673437683*G0_51 + 0.000204250963799585*G0_52 + 0.000204250963799586*G0_53 + 0.000204250963799586*G0_54 - 0.000280029576904592*G0_55;
    A[4] = 4.07498237359376e-06*G0_0 + 4.07498237359424e-06*G0_1 - 6.04020786312485e-05*G0_2 + 1.81868671452016e-05*G0_3 + 0.000886258481744637*G0_4 - 0.0008586570001501*G0_5 + 0.000659424487549522*G0_6 - 0.000299350614020769*G0_7 + 5.50356814824431e-05*G0_8 - 0.000178405940124699*G0_9 + 9.04994032887593e-05*G0_10 - 0.000144029549411501*G0_11 - 0.000144029549411501*G0_12 + 0.000357313725369299*G0_13 - 0.000456929981669588*G0_14 + 0.000443129240872319*G0_15 + 5.50356814824425e-05*G0_16 - 0.000178405940124699*G0_17 + 9.04994032887601e-05*G0_18 - 0.000144029549411502*G0_19 - 0.000144029549411502*G0_20 + 0.0003573137253693*G0_21 - 0.000456929981669588*G0_22 + 0.000443129240872319*G0_23 + 2.75178407412214e-05*G0_24 - 2.24993895422147e-05*G0_25 - 2.24993895422143e-05*G0_26 + 2.75178407412216e-05*G0_27 - 4.01476095920546e-05*G0_28 - 5.52029631890776e-05*G0_29 + 0.000110405926378158*G0_30 - 9.28413471816294e-05*G0_31 + 0.000308634748738929*G0_32 - 0.000341254681532476*G0_33 - 4.01476095920551e-05*G0_34 - 5.52029631890785e-05*G0_35 + 0.000110405926378158*G0_36 - 9.28413471816283e-05*G0_37 + 0.000308634748738928*G0_38 - 0.000341254681532475*G0_39 - 0.000266814322080539*G0_40 + 0.000135498182373188*G0_41 - 0.000266814322080538*G0_42 - 8.78228959826087e-06*G0_43 - 8.78228959826131e-06*G0_44 - 0.000555375266023443*G0_45 - 2.00738047960266e-05*G0_46 + 2.88560943942893e-05*G0_47 - 2.0073804796027e-05*G0_48 - 2.760148159454e-05*G0_49 - 2.76014815945394e-05*G0_50 + 5.52029631890791e-05*G0_51 - 0.000301107071940417*G0_52 - 0.000301107071940418*G0_53 + 0.000727675423856015*G0_54 - 0.000777859935846086*G0_55;
    A[5] = 1.52962392545732e-05*G0_0 + 1.52962392545727e-05*G0_1 + 6.04020786312487e-05*G0_2 - 4.43831824040183e-05*G0_3 - 0.0008586570001501*G0_4 + 0.00168419222238676*G0_5 - 0.00130178624102242*G0_6 + 0.000659424487549523*G0_7 - 0.000178405940124699*G0_8 + 0.000347778668091186*G0_9 - 0.000261963152588166*G0_10 + 0.0003573137253693*G0_11 + 9.04994032887603e-05*G0_12 - 0.000261963152588167*G0_13 + 0.000328541271828326*G0_14 - 0.000456929981669589*G0_15 - 0.000178405940124698*G0_16 + 0.000347778668091186*G0_17 - 0.000261963152588166*G0_18 + 0.000357313725369301*G0_19 + 9.04994032887611e-05*G0_20 - 0.000261963152588167*G0_21 + 0.000328541271828327*G0_22 - 0.000456929981669589*G0_23 - 2.24993895422142e-05*G0_24 + 1.12078743444487e-05*G0_25 + 1.12078743444488e-05*G0_26 - 2.24993895422145e-05*G0_27 + 0.000127970505574677*G0_28 - 0.000120442828776169*G0_29 - 0.000110405926378159*G0_30 - 7.02583167860975e-05*G0_31 - 4.51660607910606e-05*G0_32 + 0.000491808217502685*G0_33 + 0.000127970505574678*G0_34 - 0.000120442828776167*G0_35 - 0.000110405926378159*G0_36 - 7.02583167860985e-05*G0_37 - 4.51660607910592e-05*G0_38 + 0.000491808217502685*G0_39 + 0.000170627340766238*G0_40 - 0.000122952054375671*G0_41 + 0.000170627340766238*G0_42 - 0.000148044310370709*G0_43 - 0.000148044310370708*G0_44 + 0.000873210508627223*G0_45 - 0.000109569517844988*G0_46 - 2.25830303955296e-05*G0_47 - 0.000109569517844988*G0_48 + 0.000253431785549856*G0_49 + 0.000253431785549855*G0_50 - 0.000623960765743209*G0_51 + 6.27306399875844e-05*G0_52 + 6.27306399875852e-05*G0_53 - 0.000903321215821266*G0_54 + 0.00110405926378154*G0_55;
    A[6] = 1.52962392545733e-05*G0_0 + 1.52962392545734e-05*G0_1 - 4.43831824040186e-05*G0_2 + 6.04020786312483e-05*G0_3 + 0.000659424487549522*G0_4 - 0.00130178624102242*G0_5 + 0.00168419222238676*G0_6 - 0.000858657000150101*G0_7 + 9.04994032887602e-05*G0_8 - 0.000261963152588167*G0_9 + 0.000328541271828326*G0_10 - 0.000456929981669589*G0_11 - 0.000178405940124699*G0_12 + 0.000347778668091187*G0_13 - 0.000261963152588167*G0_14 + 0.0003573137253693*G0_15 + 9.04994032887594e-05*G0_16 - 0.000261963152588166*G0_17 + 0.000328541271828325*G0_18 - 0.000456929981669589*G0_19 - 0.000178405940124699*G0_20 + 0.000347778668091187*G0_21 - 0.000261963152588166*G0_22 + 0.000357313725369301*G0_23 - 2.24993895422142e-05*G0_24 + 1.12078743444489e-05*G0_25 + 1.12078743444488e-05*G0_26 - 2.24993895422146e-05*G0_27 + 0.000127970505574677*G0_28 - 7.02583167860982e-05*G0_29 + 0.000491808217502689*G0_30 - 0.000120442828776167*G0_31 - 4.51660607910653e-05*G0_32 - 0.000110405926378155*G0_33 + 0.000127970505574677*G0_34 - 7.02583167860982e-05*G0_35 + 0.000491808217502689*G0_36 - 0.000120442828776166*G0_37 - 4.51660607910665e-05*G0_38 - 0.000110405926378155*G0_39 - 0.000109569517844988*G0_40 - 2.25830303955315e-05*G0_41 - 0.000109569517844988*G0_42 + 0.000253431785549857*G0_43 + 0.000253431785549856*G0_44 - 0.000623960765743209*G0_45 + 0.000170627340766237*G0_46 - 0.000122952054375671*G0_47 + 0.000170627340766237*G0_48 - 0.000148044310370708*G0_49 - 0.000148044310370707*G0_50 + 0.000873210508627221*G0_51 + 6.27306399875863e-05*G0_52 + 6.27306399875869e-05*G0_53 + 0.00110405926378154*G0_54 - 0.000903321215821266*G0_55;
    A[7] = 4.07498237359361e-06*G0_0 + 4.07498237359374e-06*G0_1 + 1.81868671452018e-05*G0_2 - 6.04020786312483e-05*G0_3 - 0.000299350614020769*G0_4 + 0.000659424487549523*G0_5 - 0.000858657000150101*G0_6 + 0.000886258481744639*G0_7 - 0.000144029549411502*G0_8 + 0.000357313725369301*G0_9 - 0.00045692998166959*G0_10 + 0.00044312924087232*G0_11 + 5.50356814824435e-05*G0_12 - 0.0001784059401247*G0_13 + 9.04994032887604e-05*G0_14 - 0.000144029549411501*G0_15 - 0.000144029549411501*G0_16 + 0.0003573137253693*G0_17 - 0.000456929981669588*G0_18 + 0.00044312924087232*G0_19 + 5.50356814824429e-05*G0_20 - 0.0001784059401247*G0_21 + 9.04994032887599e-05*G0_22 - 0.000144029549411502*G0_23 + 2.75178407412214e-05*G0_24 - 2.24993895422149e-05*G0_25 - 2.24993895422145e-05*G0_26 + 2.75178407412216e-05*G0_27 - 4.01476095920554e-05*G0_28 - 9.28413471816296e-05*G0_29 - 0.000341254681532478*G0_30 - 5.52029631890778e-05*G0_31 + 0.000308634748738932*G0_32 + 0.000110405926378154*G0_33 - 4.01476095920556e-05*G0_34 - 9.28413471816294e-05*G0_35 - 0.000341254681532477*G0_36 - 5.52029631890782e-05*G0_37 + 0.000308634748738933*G0_38 + 0.000110405926378155*G0_39 - 2.0073804796027e-05*G0_40 + 2.88560943942898e-05*G0_41 - 2.00738047960273e-05*G0_42 - 2.760148159454e-05*G0_43 - 2.76014815945394e-05*G0_44 + 5.52029631890789e-05*G0_45 - 0.000266814322080539*G0_46 + 0.000135498182373189*G0_47 - 0.000266814322080539*G0_48 - 8.78228959826195e-06*G0_49 - 8.78228959826211e-06*G0_50 - 0.000555375266023443*G0_51 - 0.000301107071940419*G0_52 - 0.000301107071940421*G0_53 - 0.000777859935846087*G0_54 + 0.00072767542385602*G0_55;
    A[8] = 4.07498237359381e-06*G0_0 - 6.04020786312483e-05*G0_1 + 4.07498237359399e-06*G0_2 + 1.81868671452017e-05*G0_3 + 5.50356814824431e-05*G0_4 - 0.000178405940124699*G0_5 + 9.04994032887602e-05*G0_6 - 0.000144029549411502*G0_7 + 0.000886258481744639*G0_8 - 0.000858657000150102*G0_9 + 0.000659424487549523*G0_10 - 0.000299350614020769*G0_11 + 0.000443129240872319*G0_12 - 0.000456929981669589*G0_13 + 0.0003573137253693*G0_14 - 0.000144029549411502*G0_15 + 5.50356814824432e-05*G0_16 - 0.000178405940124699*G0_17 + 9.04994032887599e-05*G0_18 - 0.000144029549411502*G0_19 + 2.75178407412217e-05*G0_20 - 2.24993895422154e-05*G0_21 - 2.24993895422148e-05*G0_22 + 2.75178407412214e-05*G0_23 - 0.000144029549411502*G0_24 + 0.000357313725369301*G0_25 - 0.00045692998166959*G0_26 + 0.00044312924087232*G0_27 + 0.000110405926378156*G0_28 - 5.52029631890764e-05*G0_29 - 4.01476095920557e-05*G0_30 + 0.000308634748738929*G0_31 - 9.28413471816303e-05*G0_32 - 0.000341254681532475*G0_33 - 0.000266814322080539*G0_34 + 0.000135498182373189*G0_35 - 0.00026681432208054*G0_36 - 8.78228959826064e-06*G0_37 - 8.78228959826278e-06*G0_38 - 0.000555375266023441*G0_39 - 4.01476095920543e-05*G0_40 - 5.52029631890781e-05*G0_41 + 0.000110405926378159*G0_42 - 9.28413471816292e-05*G0_43 + 0.000308634748738928*G0_44 - 0.000341254681532475*G0_45 - 2.00738047960266e-05*G0_46 - 2.76014815945398e-05*G0_47 + 5.52029631890808e-05*G0_48 + 2.88560943942899e-05*G0_49 - 2.76014815945391e-05*G0_50 - 2.00738047960267e-05*G0_51 - 0.000301107071940416*G0_52 + 0.000727675423856018*G0_53 - 0.000301107071940422*G0_54 - 0.000777859935846085*G0_55;
    A[9] = 1.52962392545735e-05*G0_0 + 6.04020786312485e-05*G0_1 + 1.52962392545734e-05*G0_2 - 4.43831824040182e-05*G0_3 - 0.000178405940124699*G0_4 + 0.000347778668091186*G0_5 - 0.000261963152588167*G0_6 + 0.000357313725369301*G0_7 - 0.000858657000150102*G0_8 + 0.00168419222238676*G0_9 - 0.00130178624102242*G0_10 + 0.000659424487549523*G0_11 - 0.00045692998166959*G0_12 + 0.000328541271828327*G0_13 - 0.000261963152588167*G0_14 + 9.049940328876e-05*G0_15 - 0.0001784059401247*G0_16 + 0.000347778668091187*G0_17 - 0.000261963152588167*G0_18 + 0.0003573137253693*G0_19 - 2.24993895422148e-05*G0_20 + 1.12078743444496e-05*G0_21 + 1.12078743444492e-05*G0_22 - 2.24993895422145e-05*G0_23 + 9.04994032887606e-05*G0_24 - 0.000261963152588168*G0_25 + 0.000328541271828328*G0_26 - 0.000456929981669591*G0_27 - 0.000110405926378154*G0_28 - 0.000120442828776171*G0_29 + 0.000127970505574679*G0_30 - 4.51660607910617e-05*G0_31 - 7.02583167860976e-05*G0_32 + 0.000491808217502689*G0_33 + 0.00017062734076624*G0_34 - 0.000122952054375671*G0_35 + 0.000170627340766241*G0_36 - 0.000148044310370709*G0_37 - 0.000148044310370705*G0_38 + 0.000873210508627222*G0_39 + 0.00012797050557468*G0_40 - 0.000120442828776168*G0_41 - 0.000110405926378157*G0_42 - 7.02583167860998e-05*G0_43 - 4.51660607910602e-05*G0_44 + 0.000491808217502689*G0_45 - 0.000109569517844987*G0_46 + 0.000253431785549857*G0_47 - 0.000623960765743208*G0_48 - 2.25830303955313e-05*G0_49 + 0.000253431785549853*G0_50 - 0.000109569517844986*G0_51 + 6.27306399875888e-05*G0_52 - 0.000903321215821263*G0_53 + 6.27306399875907e-05*G0_54 + 0.00110405926378155*G0_55;
    A[10] = 1.52962392545739e-05*G0_0 - 4.43831824040182e-05*G0_1 + 1.52962392545738e-05*G0_2 + 6.04020786312485e-05*G0_3 + 9.04994032887593e-05*G0_4 - 0.000261963152588166*G0_5 + 0.000328541271828326*G0_6 - 0.00045692998166959*G0_7 + 0.000659424487549523*G0_8 - 0.00130178624102242*G0_9 + 0.00168419222238676*G0_10 - 0.000858657000150102*G0_11 + 0.0003573137253693*G0_12 - 0.000261963152588166*G0_13 + 0.000347778668091187*G0_14 - 0.0001784059401247*G0_15 + 9.04994032887598e-05*G0_16 - 0.000261963152588166*G0_17 + 0.000328541271828325*G0_18 - 0.000456929981669589*G0_19 - 2.24993895422137e-05*G0_20 + 1.12078743444492e-05*G0_21 + 1.12078743444495e-05*G0_22 - 2.24993895422154e-05*G0_23 - 0.0001784059401247*G0_24 + 0.000347778668091187*G0_25 - 0.000261963152588168*G0_26 + 0.000357313725369301*G0_27 + 0.000491808217502689*G0_28 - 7.02583167860984e-05*G0_29 + 0.000127970505574681*G0_30 - 4.51660607910621e-05*G0_31 - 0.00012044282877617*G0_32 - 0.000110405926378152*G0_33 - 0.000109569517844985*G0_34 - 2.25830303955327e-05*G0_35 - 0.000109569517844987*G0_36 + 0.000253431785549855*G0_37 + 0.000253431785549853*G0_38 - 0.000623960765743207*G0_39 + 0.000127970505574682*G0_40 - 7.02583167861003e-05*G0_41 + 0.000491808217502691*G0_42 - 0.000120442828776168*G0_43 - 4.51660607910624e-05*G0_44 - 0.000110405926378152*G0_45 + 0.00017062734076624*G0_46 - 0.000148044310370709*G0_47 + 0.000873210508627224*G0_48 - 0.000122952054375672*G0_49 - 0.000148044310370707*G0_50 + 0.00017062734076624*G0_51 + 6.27306399875865e-05*G0_52 + 0.00110405926378155*G0_53 + 6.27306399875857e-05*G0_54 - 0.000903321215821261*G0_55;
    A[11] = 4.07498237359328e-06*G0_0 + 1.81868671452013e-05*G0_1 + 4.07498237359352e-06*G0_2 - 6.04020786312485e-05*G0_3 - 0.000144029549411501*G0_4 + 0.0003573137253693*G0_5 - 0.000456929981669589*G0_6 + 0.00044312924087232*G0_7 - 0.000299350614020769*G0_8 + 0.000659424487549523*G0_9 - 0.000858657000150102*G0_10 + 0.00088625848174464*G0_11 - 0.000144029549411501*G0_12 + 9.04994032887592e-05*G0_13 - 0.000178405940124699*G0_14 + 5.50356814824438e-05*G0_15 - 0.000144029549411501*G0_16 + 0.000357313725369299*G0_17 - 0.000456929981669588*G0_18 + 0.00044312924087232*G0_19 + 2.75178407412214e-05*G0_20 - 2.24993895422156e-05*G0_21 - 2.24993895422151e-05*G0_22 + 2.75178407412221e-05*G0_23 + 5.50356814824437e-05*G0_24 - 0.0001784059401247*G0_25 + 9.04994032887605e-05*G0_26 - 0.000144029549411502*G0_27 - 0.000341254681532478*G0_28 - 9.28413471816294e-05*G0_29 - 4.01476095920582e-05*G0_30 + 0.000308634748738932*G0_31 - 5.52029631890765e-05*G0_32 + 0.000110405926378151*G0_33 - 2.0073804796029e-05*G0_34 + 2.88560943942911e-05*G0_35 - 2.0073804796029e-05*G0_36 - 2.76014815945388e-05*G0_37 - 2.76014815945371e-05*G0_38 + 5.5202963189075e-05*G0_39 - 4.0147609592057e-05*G0_40 - 9.28413471816293e-05*G0_41 - 0.000341254681532479*G0_42 - 5.52029631890783e-05*G0_43 + 0.000308634748738932*G0_44 + 0.000110405926378152*G0_45 - 0.00026681432208054*G0_46 - 8.78228959826182e-06*G0_47 - 0.000555375266023444*G0_48 + 0.00013549818237319*G0_49 - 8.78228959826172e-06*G0_50 - 0.000266814322080541*G0_51 - 0.00030110707194042*G0_52 - 0.000777859935846087*G0_53 - 0.000301107071940422*G0_54 + 0.000727675423856014*G0_55;
    A[12] = 4.07498237359345e-06*G0_0 - 6.04020786312488e-05*G0_1 + 1.81868671452015e-05*G0_2 + 4.0749823735936e-06*G0_3 - 0.000144029549411501*G0_4 + 9.04994032887603e-05*G0_5 - 0.000178405940124699*G0_6 + 5.50356814824435e-05*G0_7 + 0.000443129240872319*G0_8 - 0.00045692998166959*G0_9 + 0.0003573137253693*G0_10 - 0.000144029549411501*G0_11 + 0.000886258481744639*G0_12 - 0.000858657000150101*G0_13 + 0.000659424487549523*G0_14 - 0.000299350614020768*G0_15 + 2.75178407412228e-05*G0_16 - 2.24993895422158e-05*G0_17 - 2.24993895422142e-05*G0_18 + 2.75178407412216e-05*G0_19 + 5.50356814824443e-05*G0_20 - 0.000178405940124701*G0_21 + 9.04994032887602e-05*G0_22 - 0.000144029549411502*G0_23 - 0.000144029549411501*G0_24 + 0.000357313725369299*G0_25 - 0.00045692998166959*G0_26 + 0.00044312924087232*G0_27 + 0.000110405926378152*G0_28 + 0.000308634748738933*G0_29 - 0.000341254681532481*G0_30 - 5.52029631890785e-05*G0_31 - 9.28413471816276e-05*G0_32 - 4.01476095920583e-05*G0_33 - 0.00026681432208054*G0_34 - 8.78228959826216e-06*G0_35 - 0.000555375266023447*G0_36 + 0.00013549818237319*G0_37 - 8.78228959826033e-06*G0_38 - 0.000266814322080541*G0_39 - 2.00738047960288e-05*G0_40 - 2.76014815945386e-05*G0_41 + 5.5202963189077e-05*G0_42 + 2.88560943942909e-05*G0_43 - 2.76014815945403e-05*G0_44 - 2.00738047960296e-05*G0_45 - 4.01476095920564e-05*G0_46 - 5.52029631890787e-05*G0_47 + 0.000110405926378156*G0_48 - 9.28413471816282e-05*G0_49 + 0.000308634748738931*G0_50 - 0.000341254681532478*G0_51 - 0.000301107071940419*G0_52 + 0.000727675423856012*G0_53 - 0.000777859935846091*G0_54 - 0.000301107071940423*G0_55;
    A[13] = 1.52962392545735e-05*G0_0 + 6.04020786312489e-05*G0_1 - 4.43831824040182e-05*G0_2 + 1.52962392545738e-05*G0_3 + 0.000357313725369299*G0_4 - 0.000261963152588167*G0_5 + 0.000347778668091187*G0_6 - 0.0001784059401247*G0_7 - 0.000456929981669589*G0_8 + 0.000328541271828327*G0_9 - 0.000261963152588166*G0_10 + 9.04994032887592e-05*G0_11 - 0.000858657000150101*G0_12 + 0.00168419222238676*G0_13 - 0.00130178624102242*G0_14 + 0.000659424487549521*G0_15 - 2.24993895422159e-05*G0_16 + 1.12078743444494e-05*G0_17 + 1.12078743444494e-05*G0_18 - 2.24993895422154e-05*G0_19 - 0.0001784059401247*G0_20 + 0.000347778668091188*G0_21 - 0.000261963152588166*G0_22 + 0.000357313725369299*G0_23 + 9.049940328876e-05*G0_24 - 0.000261963152588166*G0_25 + 0.000328541271828327*G0_26 - 0.000456929981669589*G0_27 - 0.000110405926378151*G0_28 - 4.51660607910643e-05*G0_29 + 0.000491808217502693*G0_30 - 0.000120442828776168*G0_31 - 7.02583167861012e-05*G0_32 + 0.000127970505574682*G0_33 + 0.000170627340766241*G0_34 - 0.000148044310370707*G0_35 + 0.000873210508627225*G0_36 - 0.000122952054375672*G0_37 - 0.00014804431037071*G0_38 + 0.000170627340766241*G0_39 - 0.000109569517844985*G0_40 + 0.000253431785549855*G0_41 - 0.000623960765743204*G0_42 - 2.25830303955326e-05*G0_43 + 0.000253431785549855*G0_44 - 0.000109569517844984*G0_45 + 0.000127970505574681*G0_46 - 0.000120442828776167*G0_47 - 0.000110405926378155*G0_48 - 7.02583167861006e-05*G0_49 - 4.51660607910634e-05*G0_50 + 0.00049180821750269*G0_51 + 6.27306399875907e-05*G0_52 - 0.000903321215821257*G0_53 + 0.00110405926378155*G0_54 + 6.27306399875888e-05*G0_55;
    A[14] = 1.52962392545736e-05*G0_0 - 4.43831824040188e-05*G0_1 + 6.04020786312489e-05*G0_2 + 1.52962392545733e-05*G0_3 - 0.000456929981669588*G0_4 + 0.000328541271828326*G0_5 - 0.000261963152588167*G0_6 + 9.04994032887604e-05*G0_7 + 0.0003573137253693*G0_8 - 0.000261963152588167*G0_9 + 0.000347778668091187*G0_10 - 0.000178405940124699*G0_11 + 0.000659424487549523*G0_12 - 0.00130178624102242*G0_13 + 0.00168419222238676*G0_14 - 0.0008586570001501*G0_15 - 2.2499389542214e-05*G0_16 + 1.1207874344449e-05*G0_17 + 1.12078743444479e-05*G0_18 - 2.24993895422142e-05*G0_19 + 9.04994032887614e-05*G0_20 - 0.000261963152588167*G0_21 + 0.000328541271828326*G0_22 - 0.000456929981669589*G0_23 - 0.000178405940124699*G0_24 + 0.000347778668091187*G0_25 - 0.000261963152588168*G0_26 + 0.0003573137253693*G0_27 + 0.000491808217502686*G0_28 - 4.51660607910607e-05*G0_29 - 0.000110405926378158*G0_30 - 7.02583167860998e-05*G0_31 - 0.000120442828776167*G0_32 + 0.000127970505574678*G0_33 - 0.000109569517844987*G0_34 + 0.000253431785549855*G0_35 - 0.00062396076574321*G0_36 - 2.25830303955313e-05*G0_37 + 0.000253431785549856*G0_38 - 0.000109569517844988*G0_39 + 0.000170627340766239*G0_40 - 0.000148044310370708*G0_41 + 0.000873210508627223*G0_42 - 0.000122952054375671*G0_43 - 0.000148044310370709*G0_44 + 0.000170627340766239*G0_45 + 0.000127970505574678*G0_46 - 7.02583167860985e-05*G0_47 + 0.000491808217502688*G0_48 - 0.000120442828776166*G0_49 - 4.51660607910632e-05*G0_50 - 0.000110405926378156*G0_51 + 6.27306399875857e-05*G0_52 + 0.00110405926378154*G0_53 - 0.000903321215821266*G0_54 + 6.27306399875872e-05*G0_55;
    A[15] = 4.07498237359338e-06*G0_0 + 1.81868671452017e-05*G0_1 - 6.0402078631249e-05*G0_2 + 4.07498237359361e-06*G0_3 + 0.000443129240872319*G0_4 - 0.000456929981669589*G0_5 + 0.0003573137253693*G0_6 - 0.000144029549411501*G0_7 - 0.000144029549411502*G0_8 + 9.049940328876e-05*G0_9 - 0.0001784059401247*G0_10 + 5.50356814824438e-05*G0_11 - 0.000299350614020768*G0_12 + 0.000659424487549521*G0_13 - 0.0008586570001501*G0_14 + 0.000886258481744638*G0_15 + 2.75178407412213e-05*G0_16 - 2.24993895422151e-05*G0_17 - 2.24993895422142e-05*G0_18 + 2.75178407412217e-05*G0_19 - 0.000144029549411502*G0_20 + 0.000357313725369299*G0_21 - 0.000456929981669589*G0_22 + 0.000443129240872319*G0_23 + 5.50356814824437e-05*G0_24 - 0.000178405940124699*G0_25 + 9.04994032887604e-05*G0_26 - 0.000144029549411501*G0_27 - 0.000341254681532477*G0_28 + 0.000308634748738931*G0_29 + 0.000110405926378155*G0_30 - 9.28413471816293e-05*G0_31 - 5.52029631890785e-05*G0_32 - 4.01476095920562e-05*G0_33 - 2.007380479603e-05*G0_34 - 2.76014815945381e-05*G0_35 + 5.52029631890781e-05*G0_36 + 2.88560943942906e-05*G0_37 - 2.76014815945393e-05*G0_38 - 2.00738047960282e-05*G0_39 - 0.000266814322080541*G0_40 - 8.78228959826146e-06*G0_41 - 0.000555375266023445*G0_42 + 0.00013549818237319*G0_43 - 8.78228959826085e-06*G0_44 - 0.000266814322080541*G0_45 - 4.01476095920569e-05*G0_46 - 9.28413471816296e-05*G0_47 - 0.000341254681532478*G0_48 - 5.52029631890779e-05*G0_49 + 0.000308634748738932*G0_50 + 0.000110405926378153*G0_51 - 0.000301107071940423*G0_52 - 0.000777859935846088*G0_53 + 0.000727675423856015*G0_54 - 0.000301107071940422*G0_55;
    A[16] = -6.04020786312482e-05*G0_0 + 4.07498237359412e-06*G0_1 + 4.07498237359391e-06*G0_2 + 1.81868671452015e-05*G0_3 + 5.50356814824425e-05*G0_4 - 0.000178405940124698*G0_5 + 9.04994032887594e-05*G0_6 - 0.000144029549411501*G0_7 + 5.50356814824432e-05*G0_8 - 0.0001784059401247*G0_9 + 9.04994032887598e-05*G0_10 - 0.000144029549411501*G0_11 + 2.75178407412228e-05*G0_12 - 2.24993895422159e-05*G0_13 - 2.2499389542214e-05*G0_14 + 2.75178407412213e-05*G0_15 + 0.00088625848174464*G0_16 - 0.0008586570001501*G0_17 + 0.000659424487549522*G0_18 - 0.000299350614020769*G0_19 + 0.000443129240872319*G0_20 - 0.000456929981669591*G0_21 + 0.000357313725369301*G0_22 - 0.000144029549411503*G0_23 + 0.00044312924087232*G0_24 - 0.000456929981669589*G0_25 + 0.0003573137253693*G0_26 - 0.000144029549411502*G0_27 - 0.000266814322080541*G0_28 + 0.00013549818237319*G0_29 - 0.000266814322080541*G0_30 - 8.78228959826145e-06*G0_31 - 8.78228959826235e-06*G0_32 - 0.000555375266023443*G0_33 + 0.000110405926378156*G0_34 - 5.52029631890784e-05*G0_35 - 4.01476095920568e-05*G0_36 + 0.000308634748738929*G0_37 - 9.284134718163e-05*G0_38 - 0.000341254681532478*G0_39 + 0.000110405926378158*G0_40 - 5.52029631890771e-05*G0_41 - 4.01476095920566e-05*G0_42 + 0.000308634748738929*G0_43 - 9.28413471816294e-05*G0_44 - 0.000341254681532476*G0_45 + 5.5202963189078e-05*G0_46 - 2.76014815945387e-05*G0_47 - 2.0073804796029e-05*G0_48 - 2.7601481594539e-05*G0_49 + 2.88560943942908e-05*G0_50 - 2.00738047960286e-05*G0_51 + 0.000727675423856022*G0_52 - 0.000301107071940419*G0_53 - 0.000301107071940425*G0_54 - 0.000777859935846085*G0_55;
    A[17] = 6.04020786312481e-05*G0_0 + 1.5296239254573e-05*G0_1 + 1.52962392545733e-05*G0_2 - 4.43831824040182e-05*G0_3 - 0.000178405940124699*G0_4 + 0.000347778668091186*G0_5 - 0.000261963152588166*G0_6 + 0.0003573137253693*G0_7 - 0.000178405940124699*G0_8 + 0.000347778668091187*G0_9 - 0.000261963152588166*G0_10 + 0.000357313725369299*G0_11 - 2.24993895422158e-05*G0_12 + 1.12078743444494e-05*G0_13 + 1.1207874344449e-05*G0_14 - 2.24993895422151e-05*G0_15 - 0.0008586570001501*G0_16 + 0.00168419222238675*G0_17 - 0.00130178624102242*G0_18 + 0.000659424487549523*G0_19 - 0.000456929981669588*G0_20 + 0.000328541271828327*G0_21 - 0.000261963152588167*G0_22 + 9.04994032887602e-05*G0_23 - 0.000456929981669589*G0_24 + 0.000328541271828326*G0_25 - 0.000261963152588166*G0_26 + 9.049940328876e-05*G0_27 + 0.00017062734076624*G0_28 - 0.000122952054375673*G0_29 + 0.000170627340766239*G0_30 - 0.000148044310370707*G0_31 - 0.000148044310370706*G0_32 + 0.000873210508627221*G0_33 - 0.000110405926378154*G0_34 - 0.000120442828776169*G0_35 + 0.00012797050557468*G0_36 - 4.51660607910631e-05*G0_37 - 7.02583167860968e-05*G0_38 + 0.000491808217502689*G0_39 - 0.000110405926378155*G0_40 - 0.000120442828776169*G0_41 + 0.00012797050557468*G0_42 - 4.51660607910621e-05*G0_43 - 7.02583167860981e-05*G0_44 + 0.000491808217502688*G0_45 - 0.000623960765743205*G0_46 + 0.000253431785549856*G0_47 - 0.000109569517844986*G0_48 + 0.000253431785549854*G0_49 - 2.25830303955325e-05*G0_50 - 0.000109569517844986*G0_51 - 0.000903321215821264*G0_52 + 6.27306399875872e-05*G0_53 + 6.27306399875896e-05*G0_54 + 0.00110405926378155*G0_55;
    A[18] = -4.43831824040179e-05*G0_0 + 1.52962392545735e-05*G0_1 + 1.52962392545732e-05*G0_2 + 6.04020786312483e-05*G0_3 + 9.04994032887601e-05*G0_4 - 0.000261963152588166*G0_5 + 0.000328541271828325*G0_6 - 0.000456929981669588*G0_7 + 9.04994032887599e-05*G0_8 - 0.000261963152588167*G0_9 + 0.000328541271828325*G0_10 - 0.000456929981669588*G0_11 - 2.24993895422142e-05*G0_12 + 1.12078743444494e-05*G0_13 + 1.12078743444479e-05*G0_14 - 2.24993895422142e-05*G0_15 + 0.000659424487549523*G0_16 - 0.00130178624102242*G0_17 + 0.00168419222238675*G0_18 - 0.0008586570001501*G0_19 + 0.0003573137253693*G0_20 - 0.000261963152588166*G0_21 + 0.000347778668091187*G0_22 - 0.000178405940124699*G0_23 + 0.000357313725369301*G0_24 - 0.000261963152588167*G0_25 + 0.000347778668091186*G0_26 - 0.0001784059401247*G0_27 - 0.000109569517844989*G0_28 - 2.25830303955304e-05*G0_29 - 0.000109569517844988*G0_30 + 0.000253431785549854*G0_31 + 0.000253431785549854*G0_32 - 0.000623960765743208*G0_33 + 0.000491808217502686*G0_34 - 7.02583167860987e-05*G0_35 + 0.000127970505574678*G0_36 - 4.51660607910628e-05*G0_37 - 0.000120442828776169*G0_38 - 0.000110405926378156*G0_39 + 0.000491808217502686*G0_40 - 7.02583167860974e-05*G0_41 + 0.000127970505574676*G0_42 - 4.51660607910617e-05*G0_43 - 0.00012044282877617*G0_44 - 0.000110405926378155*G0_45 + 0.000873210508627218*G0_46 - 0.000148044310370707*G0_47 + 0.000170627340766236*G0_48 - 0.000148044310370707*G0_49 - 0.000122952054375671*G0_50 + 0.000170627340766237*G0_51 + 0.00110405926378154*G0_52 + 6.2730639987582e-05*G0_53 + 6.27306399875854e-05*G0_54 - 0.000903321215821265*G0_55;
    A[19] = 1.81868671452012e-05*G0_0 + 4.07498237359355e-06*G0_1 + 4.07498237359367e-06*G0_2 - 6.04020786312486e-05*G0_3 - 0.000144029549411502*G0_4 + 0.000357313725369301*G0_5 - 0.000456929981669589*G0_6 + 0.00044312924087232*G0_7 - 0.000144029549411502*G0_8 + 0.0003573137253693*G0_9 - 0.000456929981669589*G0_10 + 0.00044312924087232*G0_11 + 2.75178407412216e-05*G0_12 - 2.24993895422154e-05*G0_13 - 2.24993895422142e-05*G0_14 + 2.75178407412217e-05*G0_15 - 0.000299350614020769*G0_16 + 0.000659424487549522*G0_17 - 0.0008586570001501*G0_18 + 0.000886258481744639*G0_19 - 0.000144029549411502*G0_20 + 9.04994032887596e-05*G0_21 - 0.0001784059401247*G0_22 + 5.50356814824438e-05*G0_23 - 0.000144029549411502*G0_24 + 9.04994032887602e-05*G0_25 - 0.000178405940124699*G0_26 + 5.5035681482444e-05*G0_27 - 2.0073804796027e-05*G0_28 + 2.885609439429e-05*G0_29 - 2.00738047960288e-05*G0_30 - 2.76014815945391e-05*G0_31 - 2.76014815945377e-05*G0_32 + 5.52029631890781e-05*G0_33 - 0.000341254681532477*G0_34 - 9.28413471816298e-05*G0_35 - 4.01476095920572e-05*G0_36 + 0.00030863474873893*G0_37 - 5.52029631890756e-05*G0_38 + 0.000110405926378155*G0_39 - 0.000341254681532478*G0_40 - 9.284134718163e-05*G0_41 - 4.01476095920559e-05*G0_42 + 0.00030863474873893*G0_43 - 5.52029631890767e-05*G0_44 + 0.000110405926378155*G0_45 - 0.000555375266023444*G0_46 - 8.78228959826225e-06*G0_47 - 0.000266814322080539*G0_48 - 8.78228959826189e-06*G0_49 + 0.00013549818237319*G0_50 - 0.000266814322080541*G0_51 - 0.00077785993584609*G0_52 - 0.00030110707194042*G0_53 - 0.000301107071940422*G0_54 + 0.000727675423856022*G0_55;
    A[20] = -6.04020786312477e-05*G0_0 + 4.07498237359435e-06*G0_1 + 1.81868671452021e-05*G0_2 + 4.07498237359417e-06*G0_3 - 0.000144029549411502*G0_4 + 9.04994032887611e-05*G0_5 - 0.000178405940124699*G0_6 + 5.50356814824429e-05*G0_7 + 2.75178407412217e-05*G0_8 - 2.24993895422148e-05*G0_9 - 2.24993895422137e-05*G0_10 + 2.75178407412214e-05*G0_11 + 5.50356814824443e-05*G0_12 - 0.0001784059401247*G0_13 + 9.04994032887614e-05*G0_14 - 0.000144029549411502*G0_15 + 0.000443129240872319*G0_16 - 0.000456929981669588*G0_17 + 0.0003573137253693*G0_18 - 0.000144029549411502*G0_19 + 0.000886258481744636*G0_20 - 0.0008586570001501*G0_21 + 0.000659424487549521*G0_22 - 0.000299350614020769*G0_23 + 0.000443129240872318*G0_24 - 0.000456929981669587*G0_25 + 0.000357313725369299*G0_26 - 0.000144029549411501*G0_27 - 0.000266814322080535*G0_28 - 8.78228959826348e-06*G0_29 - 0.00055537526602344*G0_30 + 0.000135498182373189*G0_31 - 8.78228959826252e-06*G0_32 - 0.000266814322080536*G0_33 + 0.000110405926378159*G0_34 + 0.00030863474873893*G0_35 - 0.000341254681532474*G0_36 - 5.52029631890798e-05*G0_37 - 9.28413471816293e-05*G0_38 - 4.01476095920535e-05*G0_39 + 5.52029631890824e-05*G0_40 - 2.76014815945402e-05*G0_41 - 2.0073804796024e-05*G0_42 - 2.76014815945413e-05*G0_43 + 2.88560943942894e-05*G0_44 - 2.0073804796024e-05*G0_45 + 0.000110405926378158*G0_46 - 5.5202963189079e-05*G0_47 - 4.01476095920522e-05*G0_48 + 0.00030863474873893*G0_49 - 9.284134718163e-05*G0_50 - 0.000341254681532473*G0_51 + 0.00072767542385602*G0_52 - 0.000301107071940413*G0_53 - 0.000777859935846082*G0_54 - 0.000301107071940413*G0_55;
    A[21] = 6.04020786312489e-05*G0_0 + 1.52962392545738e-05*G0_1 - 4.4383182404018e-05*G0_2 + 1.5296239254574e-05*G0_3 + 0.0003573137253693*G0_4 - 0.000261963152588167*G0_5 + 0.000347778668091188*G0_6 - 0.0001784059401247*G0_7 - 2.24993895422154e-05*G0_8 + 1.12078743444496e-05*G0_9 + 1.12078743444492e-05*G0_10 - 2.24993895422156e-05*G0_11 - 0.000178405940124701*G0_12 + 0.000347778668091188*G0_13 - 0.000261963152588167*G0_14 + 0.000357313725369299*G0_15 - 0.000456929981669591*G0_16 + 0.000328541271828327*G0_17 - 0.000261963152588166*G0_18 + 9.04994032887596e-05*G0_19 - 0.0008586570001501*G0_20 + 0.00168419222238676*G0_21 - 0.00130178624102242*G0_22 + 0.000659424487549523*G0_23 - 0.00045692998166959*G0_24 + 0.000328541271828326*G0_25 - 0.000261963152588166*G0_26 + 9.04994032887597e-05*G0_27 + 0.000170627340766242*G0_28 - 0.000148044310370708*G0_29 + 0.000873210508627229*G0_30 - 0.000122952054375672*G0_31 - 0.00014804431037071*G0_32 + 0.000170627340766244*G0_33 - 0.000110405926378151*G0_34 - 4.51660607910633e-05*G0_35 + 0.000491808217502694*G0_36 - 0.000120442828776168*G0_37 - 7.02583167861021e-05*G0_38 + 0.000127970505574684*G0_39 - 0.000623960765743204*G0_40 + 0.000253431785549854*G0_41 - 0.000109569517844983*G0_42 + 0.000253431785549854*G0_43 - 2.25830303955307e-05*G0_44 - 0.000109569517844983*G0_45 - 0.000110405926378151*G0_46 - 0.000120442828776169*G0_47 + 0.000127970505574683*G0_48 - 4.51660607910641e-05*G0_49 - 7.02583167861006e-05*G0_50 + 0.000491808217502693*G0_51 - 0.000903321215821259*G0_52 + 6.27306399875956e-05*G0_53 + 0.00110405926378155*G0_54 + 6.27306399875918e-05*G0_55;
    A[22] = -4.43831824040176e-05*G0_0 + 1.52962392545737e-05*G0_1 + 6.04020786312491e-05*G0_2 + 1.52962392545736e-05*G0_3 - 0.000456929981669588*G0_4 + 0.000328541271828327*G0_5 - 0.000261963152588166*G0_6 + 9.04994032887599e-05*G0_7 - 2.24993895422148e-05*G0_8 + 1.12078743444492e-05*G0_9 + 1.12078743444495e-05*G0_10 - 2.24993895422151e-05*G0_11 + 9.04994032887602e-05*G0_12 - 0.000261963152588166*G0_13 + 0.000328541271828326*G0_14 - 0.000456929981669589*G0_15 + 0.000357313725369301*G0_16 - 0.000261963152588166*G0_17 + 0.000347778668091187*G0_18 - 0.0001784059401247*G0_19 + 0.000659424487549522*G0_20 - 0.00130178624102242*G0_21 + 0.00168419222238675*G0_22 - 0.000858657000150101*G0_23 + 0.0003573137253693*G0_24 - 0.000261963152588166*G0_25 + 0.000347778668091186*G0_26 - 0.0001784059401247*G0_27 - 0.000109569517844985*G0_28 + 0.000253431785549854*G0_29 - 0.000623960765743207*G0_30 - 2.25830303955334e-05*G0_31 + 0.000253431785549856*G0_32 - 0.000109569517844986*G0_33 + 0.000491808217502691*G0_34 - 4.51660607910629e-05*G0_35 - 0.000110405926378155*G0_36 - 7.02583167861009e-05*G0_37 - 0.000120442828776166*G0_38 + 0.00012797050557468*G0_39 + 0.000873210508627225*G0_40 - 0.000148044310370708*G0_41 + 0.00017062734076624*G0_42 - 0.000148044310370709*G0_43 - 0.000122952054375675*G0_44 + 0.000170627340766241*G0_45 + 0.000491808217502689*G0_46 - 7.02583167860988e-05*G0_47 + 0.000127970505574681*G0_48 - 4.51660607910635e-05*G0_49 - 0.000120442828776168*G0_50 - 0.000110405926378153*G0_51 + 0.00110405926378155*G0_52 + 6.27306399875873e-05*G0_53 - 0.000903321215821257*G0_54 + 6.27306399875912e-05*G0_55;
    A[23] = 1.81868671452013e-05*G0_0 + 4.07498237359396e-06*G0_1 - 6.04020786312489e-05*G0_2 + 4.07498237359374e-06*G0_3 + 0.000443129240872319*G0_4 - 0.000456929981669589*G0_5 + 0.000357313725369301*G0_6 - 0.000144029549411502*G0_7 + 2.75178407412214e-05*G0_8 - 2.24993895422145e-05*G0_9 - 2.24993895422154e-05*G0_10 + 2.75178407412221e-05*G0_11 - 0.000144029549411502*G0_12 + 0.000357313725369299*G0_13 - 0.000456929981669589*G0_14 + 0.000443129240872319*G0_15 - 0.000144029549411503*G0_16 + 9.04994032887602e-05*G0_17 - 0.000178405940124699*G0_18 + 5.50356814824437e-05*G0_19 - 0.000299350614020769*G0_20 + 0.000659424487549523*G0_21 - 0.000858657000150101*G0_22 + 0.000886258481744639*G0_23 - 0.000144029549411502*G0_24 + 9.04994032887598e-05*G0_25 - 0.000178405940124699*G0_26 + 5.50356814824438e-05*G0_27 - 2.00738047960278e-05*G0_28 - 2.76014815945385e-05*G0_29 + 5.52029631890796e-05*G0_30 + 2.88560943942908e-05*G0_31 - 2.76014815945395e-05*G0_32 - 2.00738047960268e-05*G0_33 - 0.000341254681532478*G0_34 + 0.000308634748738932*G0_35 + 0.000110405926378157*G0_36 - 9.28413471816293e-05*G0_37 - 5.52029631890792e-05*G0_38 - 4.01476095920548e-05*G0_39 - 0.000555375266023444*G0_40 - 8.78228959826233e-06*G0_41 - 0.00026681432208054*G0_42 - 8.78228959826174e-06*G0_43 + 0.000135498182373191*G0_44 - 0.000266814322080541*G0_45 - 0.000341254681532477*G0_46 - 9.28413471816306e-05*G0_47 - 4.01476095920563e-05*G0_48 + 0.000308634748738932*G0_49 - 5.52029631890776e-05*G0_50 + 0.000110405926378154*G0_51 - 0.000777859935846089*G0_52 - 0.000301107071940421*G0_53 + 0.000727675423856019*G0_54 - 0.000301107071940421*G0_55;
    A[24] = -6.04020786312483e-05*G0_0 + 1.81868671452022e-05*G0_1 + 4.0749823735939e-06*G0_2 + 4.07498237359394e-06*G0_3 + 2.75178407412214e-05*G0_4 - 2.24993895422142e-05*G0_5 - 2.24993895422142e-05*G0_6 + 2.75178407412214e-05*G0_7 - 0.000144029549411503*G0_8 + 9.04994032887606e-05*G0_9 - 0.0001784059401247*G0_10 + 5.50356814824436e-05*G0_11 - 0.000144029549411501*G0_12 + 9.049940328876e-05*G0_13 - 0.000178405940124699*G0_14 + 5.50356814824437e-05*G0_15 + 0.00044312924087232*G0_16 - 0.000456929981669589*G0_17 + 0.000357313725369301*G0_18 - 0.000144029549411502*G0_19 + 0.000443129240872318*G0_20 - 0.00045692998166959*G0_21 + 0.0003573137253693*G0_22 - 0.000144029549411502*G0_23 + 0.00088625848174464*G0_24 - 0.000858657000150101*G0_25 + 0.000659424487549523*G0_26 - 0.000299350614020769*G0_27 - 0.000555375266023442*G0_28 - 8.78228959826235e-06*G0_29 - 0.00026681432208054*G0_30 - 8.78228959826294e-06*G0_31 + 0.000135498182373189*G0_32 - 0.000266814322080539*G0_33 + 5.5202963189079e-05*G0_34 - 2.7601481594539e-05*G0_35 - 2.0073804796026e-05*G0_36 - 2.76014815945403e-05*G0_37 + 2.88560943942899e-05*G0_38 - 2.00738047960266e-05*G0_39 + 0.000110405926378157*G0_40 + 0.000308634748738931*G0_41 - 0.000341254681532477*G0_42 - 5.52029631890788e-05*G0_43 - 9.28413471816303e-05*G0_44 - 4.01476095920551e-05*G0_45 + 0.000110405926378156*G0_46 + 0.000308634748738931*G0_47 - 0.000341254681532477*G0_48 - 5.52029631890775e-05*G0_49 - 9.28413471816294e-05*G0_50 - 4.01476095920548e-05*G0_51 + 0.000727675423856022*G0_52 - 0.000777859935846086*G0_53 - 0.00030110707194042*G0_54 - 0.000301107071940418*G0_55;
    A[25] = 6.04020786312487e-05*G0_0 - 4.43831824040187e-05*G0_1 + 1.52962392545735e-05*G0_2 + 1.52962392545735e-05*G0_3 - 2.24993895422147e-05*G0_4 + 1.12078743444487e-05*G0_5 + 1.12078743444489e-05*G0_6 - 2.24993895422149e-05*G0_7 + 0.000357313725369301*G0_8 - 0.000261963152588168*G0_9 + 0.000347778668091188*G0_10 - 0.0001784059401247*G0_11 + 0.000357313725369299*G0_12 - 0.000261963152588166*G0_13 + 0.000347778668091187*G0_14 - 0.000178405940124699*G0_15 - 0.000456929981669589*G0_16 + 0.000328541271828326*G0_17 - 0.000261963152588167*G0_18 + 9.04994032887601e-05*G0_19 - 0.000456929981669587*G0_20 + 0.000328541271828326*G0_21 - 0.000261963152588166*G0_22 + 9.04994032887599e-05*G0_23 - 0.000858657000150101*G0_24 + 0.00168419222238676*G0_25 - 0.00130178624102242*G0_26 + 0.000659424487549523*G0_27 + 0.000873210508627222*G0_28 - 0.000148044310370707*G0_29 + 0.000170627340766239*G0_30 - 0.000148044310370708*G0_31 - 0.000122952054375671*G0_32 + 0.000170627340766239*G0_33 - 0.000623960765743204*G0_34 + 0.000253431785549855*G0_35 - 0.000109569517844987*G0_36 + 0.000253431785549856*G0_37 - 2.25830303955322e-05*G0_38 - 0.000109569517844986*G0_39 - 0.000110405926378153*G0_40 - 4.51660607910647e-05*G0_41 + 0.00049180821750269*G0_42 - 0.000120442828776168*G0_43 - 7.0258316786098e-05*G0_44 + 0.00012797050557468*G0_45 - 0.000110405926378153*G0_46 - 4.51660607910654e-05*G0_47 + 0.000491808217502689*G0_48 - 0.000120442828776167*G0_49 - 7.02583167860995e-05*G0_50 + 0.000127970505574679*G0_51 - 0.000903321215821262*G0_52 + 0.00110405926378154*G0_53 + 6.27306399875891e-05*G0_54 + 6.27306399875876e-05*G0_55;
    A[26] = -4.4383182404018e-05*G0_0 + 6.04020786312488e-05*G0_1 + 1.52962392545732e-05*G0_2 + 1.52962392545733e-05*G0_3 - 2.24993895422143e-05*G0_4 + 1.12078743444488e-05*G0_5 + 1.12078743444488e-05*G0_6 - 2.24993895422145e-05*G0_7 - 0.00045692998166959*G0_8 + 0.000328541271828328*G0_9 - 0.000261963152588168*G0_10 + 9.04994032887605e-05*G0_11 - 0.00045692998166959*G0_12 + 0.000328541271828327*G0_13 - 0.000261963152588168*G0_14 + 9.04994032887604e-05*G0_15 + 0.0003573137253693*G0_16 - 0.000261963152588166*G0_17 + 0.000347778668091186*G0_18 - 0.000178405940124699*G0_19 + 0.000357313725369299*G0_20 - 0.000261963152588166*G0_21 + 0.000347778668091186*G0_22 - 0.000178405940124699*G0_23 + 0.000659424487549523*G0_24 - 0.00130178624102242*G0_25 + 0.00168419222238676*G0_26 - 0.000858657000150102*G0_27 - 0.000623960765743209*G0_28 + 0.000253431785549855*G0_29 - 0.000109569517844987*G0_30 + 0.000253431785549856*G0_31 - 2.25830303955322e-05*G0_32 - 0.000109569517844987*G0_33 + 0.00087321050862722*G0_34 - 0.000148044310370707*G0_35 + 0.000170627340766239*G0_36 - 0.000148044310370708*G0_37 - 0.000122952054375671*G0_38 + 0.000170627340766237*G0_39 + 0.000491808217502687*G0_40 - 4.51660607910612e-05*G0_41 - 0.000110405926378158*G0_42 - 7.0258316786098e-05*G0_43 - 0.000120442828776169*G0_44 + 0.000127970505574679*G0_45 + 0.000491808217502685*G0_46 - 4.51660607910604e-05*G0_47 - 0.000110405926378158*G0_48 - 7.02583167860985e-05*G0_49 - 0.000120442828776167*G0_50 + 0.000127970505574678*G0_51 + 0.00110405926378154*G0_52 - 0.000903321215821262*G0_53 + 6.27306399875891e-05*G0_54 + 6.27306399875873e-05*G0_55;
    A[27] = 1.81868671452014e-05*G0_0 - 6.04020786312487e-05*G0_1 + 4.07498237359374e-06*G0_2 + 4.07498237359373e-06*G0_3 + 2.75178407412216e-05*G0_4 - 2.24993895422145e-05*G0_5 - 2.24993895422146e-05*G0_6 + 2.75178407412216e-05*G0_7 + 0.00044312924087232*G0_8 - 0.000456929981669591*G0_9 + 0.000357313725369301*G0_10 - 0.000144029549411502*G0_11 + 0.00044312924087232*G0_12 - 0.000456929981669589*G0_13 + 0.0003573137253693*G0_14 - 0.000144029549411501*G0_15 - 0.000144029549411502*G0_16 + 9.04994032887599e-05*G0_17 - 0.0001784059401247*G0_18 + 5.5035681482444e-05*G0_19 - 0.000144029549411501*G0_20 + 9.04994032887597e-05*G0_21 - 0.0001784059401247*G0_22 + 5.50356814824438e-05*G0_23 - 0.000299350614020769*G0_24 + 0.000659424487549523*G0_25 - 0.000858657000150102*G0_26 + 0.00088625848174464*G0_27 + 5.5202963189078e-05*G0_28 - 2.76014815945375e-05*G0_29 - 2.00738047960287e-05*G0_30 - 2.7601481594541e-05*G0_31 + 2.88560943942909e-05*G0_32 - 2.0073804796028e-05*G0_33 - 0.000555375266023446*G0_34 - 8.78228959826207e-06*G0_35 - 0.000266814322080541*G0_36 - 8.78228959826064e-06*G0_37 + 0.000135498182373189*G0_38 - 0.000266814322080539*G0_39 - 0.000341254681532479*G0_40 + 0.000308634748738932*G0_41 + 0.000110405926378156*G0_42 - 9.28413471816286e-05*G0_43 - 5.52029631890795e-05*G0_44 - 4.01476095920562e-05*G0_45 - 0.000341254681532478*G0_46 + 0.00030863474873893*G0_47 + 0.000110405926378156*G0_48 - 9.2841347181629e-05*G0_49 - 5.52029631890776e-05*G0_50 - 4.01476095920565e-05*G0_51 - 0.000777859935846089*G0_52 + 0.000727675423856015*G0_53 - 0.000301107071940423*G0_54 - 0.000301107071940422*G0_55;
    A[28] = 0.000173437673437684*G0_0 + 0.000149650214754388*G0_1 + 9.59527869250145e-05*G0_2 + 9.59527869250142e-05*G0_3 - 4.01476095920546e-05*G0_4 + 0.000127970505574677*G0_5 + 0.000127970505574677*G0_6 - 4.01476095920554e-05*G0_7 + 0.000110405926378156*G0_8 - 0.000110405926378154*G0_9 + 0.000491808217502689*G0_10 - 0.000341254681532478*G0_11 + 0.000110405926378152*G0_12 - 0.000110405926378151*G0_13 + 0.000491808217502686*G0_14 - 0.000341254681532477*G0_15 - 0.000266814322080541*G0_16 + 0.00017062734076624*G0_17 - 0.000109569517844989*G0_18 - 2.0073804796027e-05*G0_19 - 0.000266814322080535*G0_20 + 0.000170627340766242*G0_21 - 0.000109569517844985*G0_22 - 2.00738047960278e-05*G0_23 - 0.000555375266023442*G0_24 + 0.000873210508627222*G0_25 - 0.000623960765743209*G0_26 + 5.52029631890781e-05*G0_27 + 0.00455006242043302*G0_28 - 0.00180664243164253*G0_29 + 0.00103714658112812*G0_30 - 0.00180664243164252*G0_31 + 0.000514391247898218*G0_32 + 0.00103714658112812*G0_33 + 0.000618942314544204*G0_34 - 0.000213284175957798*G0_35 + 0.000694219082529304*G0_36 - 0.000213284175957796*G0_37 + 0.000577121887885806*G0_38 + 0.000694219082529303*G0_39 + 0.000694219082529307*G0_40 - 0.00125461279975176*G0_41 + 0.00227503121021652*G0_42 + 6.27306399875887e-05*G0_43 - 0.000903321215821258*G0_44 + 0.000518573290564061*G0_45 + 0.000694219082529305*G0_46 - 0.00125461279975175*G0_47 + 0.00227503121021651*G0_48 + 6.27306399875887e-05*G0_49 - 0.000903321215821266*G0_50 + 0.000518573290564061*G0_51 - 0.000351291583930501*G0_52 + 0.00140516633572197*G0_53 + 0.000777859935846093*G0_54 + 0.00077785993584609*G0_55;
    A[29] = -1.77151327324952e-05*G0_0 - 0.00012149670352796*G0_1 - 0.00012149670352796*G0_2 + 3.98465025201154e-05*G0_3 - 5.52029631890776e-05*G0_4 - 0.000120442828776169*G0_5 - 7.02583167860982e-05*G0_6 - 9.28413471816297e-05*G0_7 - 5.52029631890764e-05*G0_8 - 0.000120442828776171*G0_9 - 7.02583167860984e-05*G0_10 - 9.28413471816294e-05*G0_11 + 0.000308634748738933*G0_12 - 4.51660607910643e-05*G0_13 - 4.51660607910606e-05*G0_14 + 0.000308634748738931*G0_15 + 0.00013549818237319*G0_16 - 0.000122952054375673*G0_17 - 2.25830303955304e-05*G0_18 + 2.885609439429e-05*G0_19 - 8.78228959826348e-06*G0_20 - 0.000148044310370708*G0_21 + 0.000253431785549854*G0_22 - 2.76014815945384e-05*G0_23 - 8.78228959826232e-06*G0_24 - 0.000148044310370707*G0_25 + 0.000253431785549855*G0_26 - 2.76014815945375e-05*G0_27 - 0.00180664243164253*G0_28 + 0.00526937375895737*G0_29 - 0.00180664243164253*G0_30 - 0.000940959599813815*G0_31 - 0.00094095959981381*G0_32 + 0.000514391247898216*G0_33 - 0.000213284175957802*G0_34 + 0.000225830303955317*G0_35 - 0.00125461279975176*G0_36 + 0.000414022223918077*G0_37 - 0.00063985252787339*G0_38 + 6.2730639987582e-05*G0_39 - 0.000213284175957803*G0_40 + 0.00022583030395532*G0_41 - 0.00125461279975176*G0_42 + 0.000414022223918079*G0_43 - 0.000639852527873393*G0_44 + 6.27306399875819e-05*G0_45 + 0.000577121887885804*G0_46 - 0.000639852527873394*G0_47 - 0.000903321215821265*G0_48 - 0.000639852527873393*G0_49 + 0.00263468687947868*G0_50 - 0.000903321215821268*G0_51 - 0.000639852527873401*G0_52 + 0.000677490911865942*G0_53 + 0.000677490911865936*G0_54 - 0.00120442828776169*G0_55;
    A[30] = 0.000173437673437684*G0_0 + 9.59527869250136e-05*G0_1 + 0.000149650214754389*G0_2 + 9.59527869250147e-05*G0_3 + 0.000110405926378158*G0_4 - 0.000110405926378159*G0_5 + 0.000491808217502689*G0_6 - 0.000341254681532478*G0_7 - 4.01476095920557e-05*G0_8 + 0.000127970505574679*G0_9 + 0.000127970505574681*G0_10 - 4.01476095920582e-05*G0_11 - 0.000341254681532481*G0_12 + 0.000491808217502693*G0_13 - 0.000110405926378158*G0_14 + 0.000110405926378155*G0_15 - 0.000266814322080541*G0_16 + 0.000170627340766239*G0_17 - 0.000109569517844988*G0_18 - 2.00738047960288e-05*G0_19 - 0.00055537526602344*G0_20 + 0.000873210508627229*G0_21 - 0.000623960765743207*G0_22 + 5.52029631890796e-05*G0_23 - 0.000266814322080539*G0_24 + 0.000170627340766239*G0_25 - 0.000109569517844987*G0_26 - 2.00738047960287e-05*G0_27 + 0.00103714658112812*G0_28 - 0.00180664243164253*G0_29 + 0.00455006242043304*G0_30 + 0.000514391247898221*G0_31 - 0.00180664243164253*G0_32 + 0.00103714658112812*G0_33 + 0.000694219082529311*G0_34 - 0.00125461279975176*G0_35 + 0.00227503121021652*G0_36 + 6.27306399875872e-05*G0_37 - 0.000903321215821263*G0_38 + 0.000518573290564062*G0_39 + 0.000618942314544199*G0_40 - 0.000213284175957799*G0_41 + 0.000694219082529309*G0_42 - 0.000213284175957797*G0_43 + 0.000577121887885808*G0_44 + 0.000694219082529307*G0_45 + 0.000694219082529307*G0_46 + 6.27306399875902e-05*G0_47 + 0.00051857329056406*G0_48 - 0.00125461279975176*G0_49 - 0.000903321215821264*G0_50 + 0.00227503121021652*G0_51 - 0.000351291583930494*G0_52 + 0.000777859935846091*G0_53 + 0.00140516633572198*G0_54 + 0.000777859935846091*G0_55;
    A[31] = -1.7715132732495e-05*G0_0 - 0.000121496703527959*G0_1 + 3.98465025201157e-05*G0_2 - 0.00012149670352796*G0_3 - 9.28413471816294e-05*G0_4 - 7.02583167860975e-05*G0_5 - 0.000120442828776167*G0_6 - 5.52029631890778e-05*G0_7 + 0.000308634748738929*G0_8 - 4.51660607910617e-05*G0_9 - 4.5166060791062e-05*G0_10 + 0.000308634748738932*G0_11 - 5.52029631890785e-05*G0_12 - 0.000120442828776168*G0_13 - 7.02583167860998e-05*G0_14 - 9.28413471816293e-05*G0_15 - 8.78228959826145e-06*G0_16 - 0.000148044310370707*G0_17 + 0.000253431785549854*G0_18 - 2.76014815945391e-05*G0_19 + 0.000135498182373189*G0_20 - 0.000122952054375672*G0_21 - 2.25830303955334e-05*G0_22 + 2.88560943942908e-05*G0_23 - 8.78228959826294e-06*G0_24 - 0.000148044310370708*G0_25 + 0.000253431785549856*G0_26 - 2.7601481594541e-05*G0_27 - 0.00180664243164252*G0_28 - 0.000940959599813815*G0_29 + 0.000514391247898221*G0_30 + 0.00526937375895737*G0_31 - 0.000940959599813817*G0_32 - 0.00180664243164252*G0_33 - 0.000213284175957795*G0_34 + 0.000414022223918077*G0_35 + 6.27306399875896e-05*G0_36 + 0.000225830303955314*G0_37 - 0.000639852527873393*G0_38 - 0.00125461279975175*G0_39 + 0.000577121887885808*G0_40 - 0.000639852527873393*G0_41 - 0.000903321215821261*G0_42 - 0.000639852527873397*G0_43 + 0.00263468687947868*G0_44 - 0.00090332121582126*G0_45 - 0.000213284175957795*G0_46 + 0.000225830303955317*G0_47 - 0.00125461279975175*G0_48 + 0.000414022223918075*G0_49 - 0.000639852527873392*G0_50 + 6.27306399875895e-05*G0_51 - 0.000639852527873386*G0_52 + 0.000677490911865952*G0_53 - 0.00120442828776168*G0_54 + 0.000677490911865946*G0_55;
    A[32] = -1.77151327324954e-05*G0_0 + 3.98465025201158e-05*G0_1 - 0.000121496703527959*G0_2 - 0.00012149670352796*G0_3 + 0.000308634748738929*G0_4 - 4.51660607910606e-05*G0_5 - 4.51660607910653e-05*G0_6 + 0.000308634748738932*G0_7 - 9.28413471816303e-05*G0_8 - 7.02583167860975e-05*G0_9 - 0.00012044282877617*G0_10 - 5.52029631890765e-05*G0_11 - 9.28413471816276e-05*G0_12 - 7.02583167861012e-05*G0_13 - 0.000120442828776167*G0_14 - 5.52029631890785e-05*G0_15 - 8.78228959826238e-06*G0_16 - 0.000148044310370706*G0_17 + 0.000253431785549854*G0_18 - 2.76014815945377e-05*G0_19 - 8.78228959826252e-06*G0_20 - 0.00014804431037071*G0_21 + 0.000253431785549856*G0_22 - 2.76014815945395e-05*G0_23 + 0.000135498182373189*G0_24 - 0.000122952054375671*G0_25 - 2.25830303955322e-05*G0_26 + 2.88560943942909e-05*G0_27 + 0.000514391247898218*G0_28 - 0.00094095959981381*G0_29 - 0.00180664243164253*G0_30 - 0.000940959599813817*G0_31 + 0.00526937375895737*G0_32 - 0.00180664243164253*G0_33 + 0.000577121887885802*G0_34 - 0.000639852527873391*G0_35 - 0.000903321215821265*G0_36 - 0.000639852527873393*G0_37 + 0.00263468687947868*G0_38 - 0.000903321215821262*G0_39 - 0.000213284175957799*G0_40 + 0.000414022223918078*G0_41 + 6.27306399875886e-05*G0_42 + 0.000225830303955318*G0_43 - 0.000639852527873394*G0_44 - 0.00125461279975176*G0_45 - 0.000213284175957799*G0_46 + 0.000414022223918078*G0_47 + 6.27306399875892e-05*G0_48 + 0.000225830303955317*G0_49 - 0.000639852527873395*G0_50 - 0.00125461279975176*G0_51 - 0.000639852527873394*G0_52 - 0.00120442828776168*G0_53 + 0.000677490911865947*G0_54 + 0.00067749091186594*G0_55;
    A[33] = 0.000173437673437684*G0_0 + 9.59527869250128e-05*G0_1 + 9.59527869250143e-05*G0_2 + 0.00014965021475439*G0_3 - 0.000341254681532476*G0_4 + 0.000491808217502685*G0_5 - 0.000110405926378155*G0_6 + 0.000110405926378154*G0_7 - 0.000341254681532475*G0_8 + 0.000491808217502689*G0_9 - 0.000110405926378152*G0_10 + 0.000110405926378151*G0_11 - 4.01476095920583e-05*G0_12 + 0.000127970505574683*G0_13 + 0.000127970505574678*G0_14 - 4.01476095920562e-05*G0_15 - 0.000555375266023443*G0_16 + 0.000873210508627221*G0_17 - 0.000623960765743208*G0_18 + 5.52029631890781e-05*G0_19 - 0.000266814322080536*G0_20 + 0.000170627340766244*G0_21 - 0.000109569517844986*G0_22 - 2.00738047960268e-05*G0_23 - 0.000266814322080539*G0_24 + 0.000170627340766239*G0_25 - 0.000109569517844987*G0_26 - 2.0073804796028e-05*G0_27 + 0.00103714658112812*G0_28 + 0.000514391247898216*G0_29 + 0.00103714658112812*G0_30 - 0.00180664243164252*G0_31 - 0.00180664243164253*G0_32 + 0.00455006242043303*G0_33 + 0.000694219082529307*G0_34 + 6.27306399875882e-05*G0_35 + 0.000518573290564061*G0_36 - 0.00125461279975175*G0_37 - 0.000903321215821263*G0_38 + 0.00227503121021651*G0_39 + 0.000694219082529304*G0_40 + 6.27306399875852e-05*G0_41 + 0.000518573290564061*G0_42 - 0.00125461279975175*G0_43 - 0.00090332121582126*G0_44 + 0.00227503121021652*G0_45 + 0.0006189423145442*G0_46 - 0.000213284175957796*G0_47 + 0.000694219082529305*G0_48 - 0.000213284175957797*G0_49 + 0.000577121887885805*G0_50 + 0.000694219082529308*G0_51 - 0.000351291583930496*G0_52 + 0.000777859935846091*G0_53 + 0.000777859935846095*G0_54 + 0.00140516633572197*G0_55;
    A[34] = 0.000149650214754391*G0_0 + 0.000173437673437683*G0_1 + 9.59527869250149e-05*G0_2 + 9.59527869250146e-05*G0_3 - 4.01476095920551e-05*G0_4 + 0.000127970505574678*G0_5 + 0.000127970505574677*G0_6 - 4.01476095920556e-05*G0_7 - 0.000266814322080539*G0_8 + 0.00017062734076624*G0_9 - 0.000109569517844985*G0_10 - 2.0073804796029e-05*G0_11 - 0.00026681432208054*G0_12 + 0.00017062734076624*G0_13 - 0.000109569517844987*G0_14 - 2.007380479603e-05*G0_15 + 0.000110405926378156*G0_16 - 0.000110405926378154*G0_17 + 0.000491808217502686*G0_18 - 0.000341254681532477*G0_19 + 0.000110405926378159*G0_20 - 0.000110405926378151*G0_21 + 0.000491808217502691*G0_22 - 0.000341254681532478*G0_23 + 5.52029631890789e-05*G0_24 - 0.000623960765743204*G0_25 + 0.00087321050862722*G0_26 - 0.000555375266023445*G0_27 + 0.000618942314544203*G0_28 - 0.000213284175957802*G0_29 + 0.000694219082529311*G0_30 - 0.000213284175957795*G0_31 + 0.000577121887885802*G0_32 + 0.000694219082529307*G0_33 + 0.00455006242043303*G0_34 - 0.00180664243164253*G0_35 + 0.00103714658112812*G0_36 - 0.00180664243164252*G0_37 + 0.000514391247898217*G0_38 + 0.00103714658112812*G0_39 + 0.00227503121021652*G0_40 - 0.00125461279975176*G0_41 + 0.000694219082529312*G0_42 - 0.00090332121582126*G0_43 + 6.27306399875852e-05*G0_44 + 0.000518573290564065*G0_45 + 0.00227503121021651*G0_46 - 0.00125461279975175*G0_47 + 0.000694219082529306*G0_48 - 0.000903321215821263*G0_49 + 6.27306399875859e-05*G0_50 + 0.000518573290564063*G0_51 + 0.00140516633572197*G0_52 - 0.000351291583930484*G0_53 + 0.000777859935846097*G0_54 + 0.000777859935846097*G0_55;
    A[35] = -0.00012149670352796*G0_0 - 1.77151327324941e-05*G0_1 - 0.00012149670352796*G0_2 + 3.98465025201156e-05*G0_3 - 5.52029631890785e-05*G0_4 - 0.000120442828776167*G0_5 - 7.02583167860982e-05*G0_6 - 9.28413471816294e-05*G0_7 + 0.000135498182373189*G0_8 - 0.000122952054375671*G0_9 - 2.25830303955326e-05*G0_10 + 2.88560943942911e-05*G0_11 - 8.78228959826215e-06*G0_12 - 0.000148044310370707*G0_13 + 0.000253431785549855*G0_14 - 2.76014815945382e-05*G0_15 - 5.52029631890784e-05*G0_16 - 0.000120442828776169*G0_17 - 7.02583167860987e-05*G0_18 - 9.28413471816297e-05*G0_19 + 0.00030863474873893*G0_20 - 4.51660607910633e-05*G0_21 - 4.5166060791063e-05*G0_22 + 0.000308634748738932*G0_23 - 2.7601481594539e-05*G0_24 + 0.000253431785549855*G0_25 - 0.000148044310370707*G0_26 - 8.78228959826205e-06*G0_27 - 0.000213284175957798*G0_28 + 0.000225830303955317*G0_29 - 0.00125461279975176*G0_30 + 0.000414022223918077*G0_31 - 0.000639852527873391*G0_32 + 6.27306399875883e-05*G0_33 - 0.00180664243164253*G0_34 + 0.00526937375895737*G0_35 - 0.00180664243164253*G0_36 - 0.000940959599813814*G0_37 - 0.000940959599813813*G0_38 + 0.000514391247898218*G0_39 - 0.00125461279975176*G0_40 + 0.000225830303955317*G0_41 - 0.000213284175957799*G0_42 - 0.000639852527873395*G0_43 + 0.000414022223918078*G0_44 + 6.27306399875858e-05*G0_45 - 0.000903321215821263*G0_46 - 0.000639852527873396*G0_47 + 0.000577121887885808*G0_48 + 0.00263468687947869*G0_49 - 0.000639852527873396*G0_50 - 0.000903321215821265*G0_51 + 0.000677490911865944*G0_52 - 0.0006398525278734*G0_53 + 0.000677490911865946*G0_54 - 0.00120442828776168*G0_55;
    A[36] = 9.5952786925015e-05*G0_0 + 0.000173437673437682*G0_1 + 0.000149650214754389*G0_2 + 9.59527869250143e-05*G0_3 + 0.000110405926378158*G0_4 - 0.000110405926378159*G0_5 + 0.000491808217502689*G0_6 - 0.000341254681532477*G0_7 - 0.00026681432208054*G0_8 + 0.000170627340766241*G0_9 - 0.000109569517844987*G0_10 - 2.00738047960289e-05*G0_11 - 0.000555375266023447*G0_12 + 0.000873210508627225*G0_13 - 0.00062396076574321*G0_14 + 5.5202963189078e-05*G0_15 - 4.01476095920567e-05*G0_16 + 0.00012797050557468*G0_17 + 0.000127970505574678*G0_18 - 4.01476095920572e-05*G0_19 - 0.000341254681532474*G0_20 + 0.000491808217502694*G0_21 - 0.000110405926378155*G0_22 + 0.000110405926378157*G0_23 - 2.0073804796026e-05*G0_24 - 0.000109569517844987*G0_25 + 0.000170627340766239*G0_26 - 0.000266814322080541*G0_27 + 0.000694219082529304*G0_28 - 0.00125461279975176*G0_29 + 0.00227503121021652*G0_30 + 6.27306399875897e-05*G0_31 - 0.000903321215821265*G0_32 + 0.000518573290564061*G0_33 + 0.00103714658112812*G0_34 - 0.00180664243164253*G0_35 + 0.00455006242043303*G0_36 + 0.000514391247898218*G0_37 - 0.00180664243164253*G0_38 + 0.00103714658112812*G0_39 + 0.000694219082529305*G0_40 - 0.000213284175957798*G0_41 + 0.000618942314544198*G0_42 + 0.000577121887885806*G0_43 - 0.000213284175957797*G0_44 + 0.000694219082529306*G0_45 + 0.000518573290564058*G0_46 + 6.27306399875907e-05*G0_47 + 0.000694219082529301*G0_48 - 0.000903321215821264*G0_49 - 0.00125461279975176*G0_50 + 0.00227503121021652*G0_51 + 0.000777859935846089*G0_52 - 0.000351291583930493*G0_53 + 0.00140516633572198*G0_54 + 0.000777859935846092*G0_55;
    A[37] = -0.00012149670352796*G0_0 - 1.77151327324951e-05*G0_1 + 3.98465025201151e-05*G0_2 - 0.00012149670352796*G0_3 - 9.28413471816283e-05*G0_4 - 7.02583167860986e-05*G0_5 - 0.000120442828776166*G0_6 - 5.52029631890782e-05*G0_7 - 8.78228959826064e-06*G0_8 - 0.000148044310370709*G0_9 + 0.000253431785549855*G0_10 - 2.76014815945388e-05*G0_11 + 0.00013549818237319*G0_12 - 0.000122952054375672*G0_13 - 2.25830303955313e-05*G0_14 + 2.88560943942906e-05*G0_15 + 0.000308634748738929*G0_16 - 4.51660607910632e-05*G0_17 - 4.51660607910628e-05*G0_18 + 0.000308634748738931*G0_19 - 5.52029631890798e-05*G0_20 - 0.000120442828776168*G0_21 - 7.02583167861009e-05*G0_22 - 9.28413471816293e-05*G0_23 - 2.76014815945403e-05*G0_24 + 0.000253431785549856*G0_25 - 0.000148044310370708*G0_26 - 8.7822895982606e-06*G0_27 - 0.000213284175957796*G0_28 + 0.000414022223918077*G0_29 + 6.27306399875872e-05*G0_30 + 0.000225830303955314*G0_31 - 0.000639852527873393*G0_32 - 0.00125461279975175*G0_33 - 0.00180664243164252*G0_34 - 0.000940959599813814*G0_35 + 0.000514391247898218*G0_36 + 0.00526937375895736*G0_37 - 0.000940959599813814*G0_38 - 0.00180664243164253*G0_39 - 0.000903321215821263*G0_40 - 0.000639852527873391*G0_41 + 0.000577121887885804*G0_42 + 0.00263468687947868*G0_43 - 0.000639852527873391*G0_44 - 0.000903321215821264*G0_45 - 0.00125461279975175*G0_46 + 0.000225830303955316*G0_47 - 0.000213284175957799*G0_48 - 0.000639852527873395*G0_49 + 0.000414022223918078*G0_50 + 6.27306399875868e-05*G0_51 + 0.000677490911865942*G0_52 - 0.000639852527873386*G0_53 - 0.00120442828776168*G0_54 + 0.000677490911865941*G0_55;
    A[38] = 3.98465025201152e-05*G0_0 - 1.77151327324946e-05*G0_1 - 0.000121496703527959*G0_2 - 0.00012149670352796*G0_3 + 0.000308634748738928*G0_4 - 4.51660607910592e-05*G0_5 - 4.51660607910666e-05*G0_6 + 0.000308634748738933*G0_7 - 8.78228959826284e-06*G0_8 - 0.000148044310370705*G0_9 + 0.000253431785549853*G0_10 - 2.76014815945371e-05*G0_11 - 8.78228959826038e-06*G0_12 - 0.00014804431037071*G0_13 + 0.000253431785549856*G0_14 - 2.76014815945393e-05*G0_15 - 9.284134718163e-05*G0_16 - 7.02583167860967e-05*G0_17 - 0.000120442828776169*G0_18 - 5.52029631890756e-05*G0_19 - 9.28413471816293e-05*G0_20 - 7.02583167861021e-05*G0_21 - 0.000120442828776166*G0_22 - 5.52029631890792e-05*G0_23 + 2.88560943942899e-05*G0_24 - 2.25830303955322e-05*G0_25 - 0.000122952054375671*G0_26 + 0.000135498182373189*G0_27 + 0.000577121887885806*G0_28 - 0.00063985252787339*G0_29 - 0.000903321215821263*G0_30 - 0.000639852527873393*G0_31 + 0.00263468687947868*G0_32 - 0.000903321215821263*G0_33 + 0.000514391247898217*G0_34 - 0.000940959599813813*G0_35 - 0.00180664243164253*G0_36 - 0.000940959599813814*G0_37 + 0.00526937375895736*G0_38 - 0.00180664243164252*G0_39 + 6.27306399875896e-05*G0_40 + 0.000414022223918077*G0_41 - 0.000213284175957796*G0_42 - 0.000639852527873394*G0_43 + 0.000225830303955318*G0_44 - 0.00125461279975175*G0_45 + 6.27306399875888e-05*G0_46 + 0.000414022223918077*G0_47 - 0.000213284175957798*G0_48 - 0.000639852527873395*G0_49 + 0.000225830303955317*G0_50 - 0.00125461279975175*G0_51 - 0.00120442828776168*G0_52 - 0.000639852527873392*G0_53 + 0.000677490911865944*G0_54 + 0.000677490911865947*G0_55;
    A[39] = 9.59527869250152e-05*G0_0 + 0.000173437673437682*G0_1 + 9.5952786925014e-05*G0_2 + 0.000149650214754389*G0_3 - 0.000341254681532475*G0_4 + 0.000491808217502685*G0_5 - 0.000110405926378155*G0_6 + 0.000110405926378155*G0_7 - 0.000555375266023441*G0_8 + 0.000873210508627222*G0_9 - 0.000623960765743207*G0_10 + 5.5202963189075e-05*G0_11 - 0.000266814322080541*G0_12 + 0.000170627340766241*G0_13 - 0.000109569517844988*G0_14 - 2.00738047960282e-05*G0_15 - 0.000341254681532478*G0_16 + 0.000491808217502689*G0_17 - 0.000110405926378156*G0_18 + 0.000110405926378155*G0_19 - 4.01476095920535e-05*G0_20 + 0.000127970505574684*G0_21 + 0.00012797050557468*G0_22 - 4.01476095920548e-05*G0_23 - 2.00738047960266e-05*G0_24 - 0.000109569517844986*G0_25 + 0.000170627340766237*G0_26 - 0.000266814322080539*G0_27 + 0.000694219082529303*G0_28 + 6.2730639987582e-05*G0_29 + 0.000518573290564062*G0_30 - 0.00125461279975175*G0_31 - 0.000903321215821262*G0_32 + 0.00227503121021651*G0_33 + 0.00103714658112812*G0_34 + 0.000514391247898218*G0_35 + 0.00103714658112812*G0_36 - 0.00180664243164253*G0_37 - 0.00180664243164252*G0_38 + 0.00455006242043303*G0_39 + 0.000518573290564058*G0_40 + 6.27306399875859e-05*G0_41 + 0.000694219082529304*G0_42 - 0.000903321215821261*G0_43 - 0.00125461279975175*G0_44 + 0.00227503121021652*G0_45 + 0.000694219082529301*G0_46 - 0.000213284175957795*G0_47 + 0.000618942314544196*G0_48 + 0.000577121887885808*G0_49 - 0.000213284175957801*G0_50 + 0.000694219082529305*G0_51 + 0.000777859935846075*G0_52 - 0.000351291583930489*G0_53 + 0.000777859935846092*G0_54 + 0.00140516633572197*G0_55;
    A[40] = 0.00014965021475439*G0_0 + 9.59527869250136e-05*G0_1 + 0.000173437673437683*G0_2 + 9.59527869250145e-05*G0_3 - 0.000266814322080539*G0_4 + 0.000170627340766238*G0_5 - 0.000109569517844988*G0_6 - 2.0073804796027e-05*G0_7 - 4.01476095920544e-05*G0_8 + 0.00012797050557468*G0_9 + 0.000127970505574682*G0_10 - 4.0147609592057e-05*G0_11 - 2.00738047960288e-05*G0_12 - 0.000109569517844985*G0_13 + 0.000170627340766239*G0_14 - 0.000266814322080541*G0_15 + 0.000110405926378158*G0_16 - 0.000110405926378155*G0_17 + 0.000491808217502686*G0_18 - 0.000341254681532478*G0_19 + 5.52029631890824e-05*G0_20 - 0.000623960765743204*G0_21 + 0.000873210508627225*G0_22 - 0.000555375266023444*G0_23 + 0.000110405926378157*G0_24 - 0.000110405926378153*G0_25 + 0.000491808217502687*G0_26 - 0.000341254681532478*G0_27 + 0.000694219082529307*G0_28 - 0.000213284175957803*G0_29 + 0.000618942314544199*G0_30 + 0.000577121887885808*G0_31 - 0.000213284175957799*G0_32 + 0.000694219082529304*G0_33 + 0.00227503121021652*G0_34 - 0.00125461279975176*G0_35 + 0.000694219082529305*G0_36 - 0.000903321215821263*G0_37 + 6.27306399875896e-05*G0_38 + 0.000518573290564058*G0_39 + 0.00455006242043304*G0_40 - 0.00180664243164253*G0_41 + 0.00103714658112812*G0_42 - 0.00180664243164253*G0_43 + 0.000514391247898214*G0_44 + 0.00103714658112812*G0_45 + 0.00227503121021651*G0_46 - 0.000903321215821261*G0_47 + 0.000518573290564061*G0_48 - 0.00125461279975175*G0_49 + 6.27306399875862e-05*G0_50 + 0.000694219082529308*G0_51 + 0.00140516633572197*G0_52 + 0.00077785993584609*G0_53 - 0.000351291583930488*G0_54 + 0.000777859935846086*G0_55;
    A[41] = -0.00012149670352796*G0_0 - 0.00012149670352796*G0_1 - 1.77151327324948e-05*G0_2 + 3.98465025201156e-05*G0_3 + 0.000135498182373188*G0_4 - 0.000122952054375671*G0_5 - 2.25830303955315e-05*G0_6 + 2.88560943942898e-05*G0_7 - 5.52029631890781e-05*G0_8 - 0.000120442828776168*G0_9 - 7.02583167861003e-05*G0_10 - 9.28413471816292e-05*G0_11 - 2.76014815945386e-05*G0_12 + 0.000253431785549855*G0_13 - 0.000148044310370708*G0_14 - 8.78228959826149e-06*G0_15 - 5.52029631890771e-05*G0_16 - 0.000120442828776169*G0_17 - 7.02583167860975e-05*G0_18 - 9.28413471816301e-05*G0_19 - 2.76014815945402e-05*G0_20 + 0.000253431785549854*G0_21 - 0.000148044310370708*G0_22 - 8.7822895982624e-06*G0_23 + 0.000308634748738931*G0_24 - 4.51660607910648e-05*G0_25 - 4.51660607910612e-05*G0_26 + 0.000308634748738932*G0_27 - 0.00125461279975176*G0_28 + 0.00022583030395532*G0_29 - 0.000213284175957799*G0_30 - 0.000639852527873393*G0_31 + 0.000414022223918078*G0_32 + 6.27306399875852e-05*G0_33 - 0.00125461279975176*G0_34 + 0.000225830303955317*G0_35 - 0.000213284175957798*G0_36 - 0.000639852527873391*G0_37 + 0.000414022223918077*G0_38 + 6.2730639987586e-05*G0_39 - 0.00180664243164253*G0_40 + 0.00526937375895736*G0_41 - 0.00180664243164253*G0_42 - 0.000940959599813813*G0_43 - 0.000940959599813814*G0_44 + 0.000514391247898217*G0_45 - 0.000903321215821264*G0_46 + 0.00263468687947869*G0_47 - 0.000903321215821265*G0_48 - 0.000639852527873395*G0_49 - 0.000639852527873393*G0_50 + 0.000577121887885806*G0_51 + 0.000677490911865947*G0_52 + 0.000677490911865943*G0_53 - 0.000639852527873396*G0_54 - 0.00120442828776168*G0_55;
    A[42] = 9.59527869250159e-05*G0_0 + 0.000149650214754388*G0_1 + 0.000173437673437683*G0_2 + 9.59527869250146e-05*G0_3 - 0.000266814322080538*G0_4 + 0.000170627340766238*G0_5 - 0.000109569517844988*G0_6 - 2.00738047960273e-05*G0_7 + 0.000110405926378159*G0_8 - 0.000110405926378157*G0_9 + 0.000491808217502691*G0_10 - 0.000341254681532479*G0_11 + 5.5202963189077e-05*G0_12 - 0.000623960765743204*G0_13 + 0.000873210508627223*G0_14 - 0.000555375266023445*G0_15 - 4.01476095920566e-05*G0_16 + 0.00012797050557468*G0_17 + 0.000127970505574676*G0_18 - 4.01476095920559e-05*G0_19 - 2.0073804796024e-05*G0_20 - 0.000109569517844983*G0_21 + 0.00017062734076624*G0_22 - 0.00026681432208054*G0_23 - 0.000341254681532477*G0_24 + 0.00049180821750269*G0_25 - 0.000110405926378158*G0_26 + 0.000110405926378156*G0_27 + 0.00227503121021652*G0_28 - 0.00125461279975176*G0_29 + 0.000694219082529309*G0_30 - 0.000903321215821261*G0_31 + 6.27306399875887e-05*G0_32 + 0.000518573290564061*G0_33 + 0.000694219082529312*G0_34 - 0.000213284175957799*G0_35 + 0.000618942314544198*G0_36 + 0.000577121887885804*G0_37 - 0.000213284175957796*G0_38 + 0.000694219082529304*G0_39 + 0.00103714658112812*G0_40 - 0.00180664243164253*G0_41 + 0.00455006242043304*G0_42 + 0.000514391247898218*G0_43 - 0.00180664243164252*G0_44 + 0.00103714658112812*G0_45 + 0.00051857329056406*G0_46 - 0.000903321215821264*G0_47 + 0.00227503121021652*G0_48 + 6.27306399875895e-05*G0_49 - 0.00125461279975176*G0_50 + 0.000694219082529308*G0_51 + 0.000777859935846085*G0_52 + 0.00140516633572197*G0_53 - 0.000351291583930483*G0_54 + 0.000777859935846091*G0_55;
    A[43] = -0.00012149670352796*G0_0 + 3.98465025201158e-05*G0_1 - 1.77151327324952e-05*G0_2 - 0.00012149670352796*G0_3 - 8.7822895982608e-06*G0_4 - 0.000148044310370709*G0_5 + 0.000253431785549857*G0_6 - 2.760148159454e-05*G0_7 - 9.28413471816292e-05*G0_8 - 7.02583167860998e-05*G0_9 - 0.000120442828776168*G0_10 - 5.52029631890783e-05*G0_11 + 2.88560943942909e-05*G0_12 - 2.25830303955326e-05*G0_13 - 0.000122952054375671*G0_14 + 0.00013549818237319*G0_15 + 0.000308634748738929*G0_16 - 4.51660607910622e-05*G0_17 - 4.51660607910617e-05*G0_18 + 0.00030863474873893*G0_19 - 2.76014815945413e-05*G0_20 + 0.000253431785549854*G0_21 - 0.000148044310370709*G0_22 - 8.78228959826182e-06*G0_23 - 5.52029631890787e-05*G0_24 - 0.000120442828776168*G0_25 - 7.02583167860981e-05*G0_26 - 9.28413471816286e-05*G0_27 + 6.27306399875887e-05*G0_28 + 0.000414022223918079*G0_29 - 0.000213284175957797*G0_30 - 0.000639852527873397*G0_31 + 0.000225830303955318*G0_32 - 0.00125461279975175*G0_33 - 0.00090332121582126*G0_34 - 0.000639852527873395*G0_35 + 0.000577121887885806*G0_36 + 0.00263468687947868*G0_37 - 0.000639852527873394*G0_38 - 0.000903321215821261*G0_39 - 0.00180664243164253*G0_40 - 0.000940959599813813*G0_41 + 0.000514391247898218*G0_42 + 0.00526937375895736*G0_43 - 0.000940959599813813*G0_44 - 0.00180664243164253*G0_45 - 0.00125461279975175*G0_46 - 0.000639852527873396*G0_47 + 6.27306399875887e-05*G0_48 + 0.000225830303955316*G0_49 + 0.000414022223918078*G0_50 - 0.000213284175957798*G0_51 + 0.000677490911865946*G0_52 - 0.00120442828776168*G0_53 - 0.000639852527873388*G0_54 + 0.000677490911865945*G0_55;
    A[44] = 3.98465025201148e-05*G0_0 - 0.000121496703527959*G0_1 - 1.77151327324951e-05*G0_2 - 0.00012149670352796*G0_3 - 8.78228959826125e-06*G0_4 - 0.000148044310370708*G0_5 + 0.000253431785549856*G0_6 - 2.76014815945394e-05*G0_7 + 0.000308634748738928*G0_8 - 4.51660607910604e-05*G0_9 - 4.51660607910625e-05*G0_10 + 0.000308634748738932*G0_11 - 2.76014815945403e-05*G0_12 + 0.000253431785549855*G0_13 - 0.000148044310370709*G0_14 - 8.78228959826088e-06*G0_15 - 9.28413471816294e-05*G0_16 - 7.02583167860981e-05*G0_17 - 0.00012044282877617*G0_18 - 5.52029631890767e-05*G0_19 + 2.88560943942894e-05*G0_20 - 2.25830303955307e-05*G0_21 - 0.000122952054375675*G0_22 + 0.000135498182373191*G0_23 - 9.28413471816303e-05*G0_24 - 7.0258316786098e-05*G0_25 - 0.000120442828776169*G0_26 - 5.52029631890795e-05*G0_27 - 0.000903321215821258*G0_28 - 0.000639852527873393*G0_29 + 0.000577121887885808*G0_30 + 0.00263468687947868*G0_31 - 0.000639852527873394*G0_32 - 0.00090332121582126*G0_33 + 6.27306399875853e-05*G0_34 + 0.000414022223918078*G0_35 - 0.000213284175957797*G0_36 - 0.000639852527873391*G0_37 + 0.000225830303955318*G0_38 - 0.00125461279975175*G0_39 + 0.000514391247898215*G0_40 - 0.000940959599813814*G0_41 - 0.00180664243164252*G0_42 - 0.000940959599813813*G0_43 + 0.00526937375895736*G0_44 - 0.00180664243164253*G0_45 + 6.27306399875886e-05*G0_46 - 0.000639852527873395*G0_47 - 0.00125461279975175*G0_48 + 0.000414022223918077*G0_49 + 0.000225830303955318*G0_50 - 0.000213284175957798*G0_51 - 0.00120442828776168*G0_52 + 0.000677490911865945*G0_53 - 0.000639852527873388*G0_54 + 0.000677490911865945*G0_55;
    A[45] = 9.59527869250159e-05*G0_0 + 9.59527869250131e-05*G0_1 + 0.000173437673437683*G0_2 + 0.000149650214754389*G0_3 - 0.000555375266023443*G0_4 + 0.000873210508627223*G0_5 - 0.000623960765743209*G0_6 + 5.52029631890789e-05*G0_7 - 0.000341254681532475*G0_8 + 0.000491808217502689*G0_9 - 0.000110405926378152*G0_10 + 0.000110405926378152*G0_11 - 2.00738047960296e-05*G0_12 - 0.000109569517844984*G0_13 + 0.000170627340766239*G0_14 - 0.000266814322080541*G0_15 - 0.000341254681532476*G0_16 + 0.000491808217502688*G0_17 - 0.000110405926378155*G0_18 + 0.000110405926378155*G0_19 - 2.0073804796024e-05*G0_20 - 0.000109569517844983*G0_21 + 0.000170627340766241*G0_22 - 0.000266814322080541*G0_23 - 4.01476095920551e-05*G0_24 + 0.00012797050557468*G0_25 + 0.000127970505574679*G0_26 - 4.01476095920563e-05*G0_27 + 0.000518573290564061*G0_28 + 6.2730639987582e-05*G0_29 + 0.000694219082529307*G0_30 - 0.00090332121582126*G0_31 - 0.00125461279975176*G0_32 + 0.00227503121021652*G0_33 + 0.000518573290564065*G0_34 + 6.27306399875859e-05*G0_35 + 0.000694219082529306*G0_36 - 0.000903321215821263*G0_37 - 0.00125461279975175*G0_38 + 0.00227503121021652*G0_39 + 0.00103714658112812*G0_40 + 0.000514391247898217*G0_41 + 0.00103714658112812*G0_42 - 0.00180664243164253*G0_43 - 0.00180664243164253*G0_44 + 0.00455006242043304*G0_45 + 0.000694219082529305*G0_46 + 0.00057712188788581*G0_47 + 0.000694219082529306*G0_48 - 0.000213284175957799*G0_49 - 0.000213284175957801*G0_50 + 0.000618942314544203*G0_51 + 0.000777859935846088*G0_52 + 0.000777859935846095*G0_53 - 0.000351291583930489*G0_54 + 0.00140516633572198*G0_55;
    A[46] = 0.00014965021475439*G0_0 + 9.59527869250134e-05*G0_1 + 9.59527869250143e-05*G0_2 + 0.000173437673437683*G0_3 - 2.00738047960266e-05*G0_4 - 0.000109569517844988*G0_5 + 0.000170627340766237*G0_6 - 0.000266814322080539*G0_7 - 2.00738047960265e-05*G0_8 - 0.000109569517844987*G0_9 + 0.00017062734076624*G0_10 - 0.00026681432208054*G0_11 - 4.01476095920564e-05*G0_12 + 0.000127970505574681*G0_13 + 0.000127970505574678*G0_14 - 4.01476095920569e-05*G0_15 + 5.5202963189078e-05*G0_16 - 0.000623960765743205*G0_17 + 0.000873210508627218*G0_18 - 0.000555375266023444*G0_19 + 0.000110405926378158*G0_20 - 0.000110405926378151*G0_21 + 0.000491808217502689*G0_22 - 0.000341254681532477*G0_23 + 0.000110405926378156*G0_24 - 0.000110405926378153*G0_25 + 0.000491808217502685*G0_26 - 0.000341254681532478*G0_27 + 0.000694219082529305*G0_28 + 0.000577121887885804*G0_29 + 0.000694219082529307*G0_30 - 0.000213284175957795*G0_31 - 0.000213284175957799*G0_32 + 0.0006189423145442*G0_33 + 0.00227503121021651*G0_34 - 0.000903321215821263*G0_35 + 0.000518573290564058*G0_36 - 0.00125461279975175*G0_37 + 6.27306399875888e-05*G0_38 + 0.000694219082529301*G0_39 + 0.00227503121021652*G0_40 - 0.000903321215821264*G0_41 + 0.00051857329056406*G0_42 - 0.00125461279975175*G0_43 + 6.27306399875886e-05*G0_44 + 0.000694219082529305*G0_45 + 0.00455006242043302*G0_46 - 0.00180664243164253*G0_47 + 0.00103714658112812*G0_48 - 0.00180664243164253*G0_49 + 0.000514391247898218*G0_50 + 0.00103714658112812*G0_51 + 0.00140516633572196*G0_52 + 0.000777859935846094*G0_53 + 0.000777859935846092*G0_54 - 0.00035129158393049*G0_55;
    A[47] = -0.000121496703527959*G0_0 - 0.00012149670352796*G0_1 + 3.98465025201163e-05*G0_2 - 1.77151327324946e-05*G0_3 + 2.88560943942893e-05*G0_4 - 2.25830303955296e-05*G0_5 - 0.000122952054375671*G0_6 + 0.000135498182373189*G0_7 - 2.76014815945398e-05*G0_8 + 0.000253431785549857*G0_9 - 0.000148044310370709*G0_10 - 8.78228959826182e-06*G0_11 - 5.52029631890786e-05*G0_12 - 0.000120442828776167*G0_13 - 7.02583167860985e-05*G0_14 - 9.28413471816296e-05*G0_15 - 2.76014815945387e-05*G0_16 + 0.000253431785549856*G0_17 - 0.000148044310370707*G0_18 - 8.78228959826226e-06*G0_19 - 5.52029631890791e-05*G0_20 - 0.000120442828776169*G0_21 - 7.02583167860988e-05*G0_22 - 9.28413471816306e-05*G0_23 + 0.000308634748738931*G0_24 - 4.51660607910654e-05*G0_25 - 4.51660607910603e-05*G0_26 + 0.00030863474873893*G0_27 - 0.00125461279975175*G0_28 - 0.000639852527873395*G0_29 + 6.27306399875901e-05*G0_30 + 0.000225830303955317*G0_31 + 0.000414022223918078*G0_32 - 0.000213284175957796*G0_33 - 0.00125461279975175*G0_34 - 0.000639852527873396*G0_35 + 6.27306399875908e-05*G0_36 + 0.000225830303955316*G0_37 + 0.000414022223918077*G0_38 - 0.000213284175957795*G0_39 - 0.000903321215821261*G0_40 + 0.00263468687947869*G0_41 - 0.000903321215821264*G0_42 - 0.000639852527873396*G0_43 - 0.000639852527873395*G0_44 + 0.00057712188788581*G0_45 - 0.00180664243164253*G0_46 + 0.00526937375895737*G0_47 - 0.00180664243164253*G0_48 - 0.000940959599813817*G0_49 - 0.000940959599813816*G0_50 + 0.000514391247898223*G0_51 + 0.000677490911865954*G0_52 + 0.000677490911865951*G0_53 - 0.00120442828776168*G0_54 - 0.000639852527873387*G0_55;
    A[48] = 9.59527869250153e-05*G0_0 + 0.000149650214754388*G0_1 + 9.59527869250144e-05*G0_2 + 0.000173437673437683*G0_3 - 2.0073804796027e-05*G0_4 - 0.000109569517844988*G0_5 + 0.000170627340766237*G0_6 - 0.000266814322080539*G0_7 + 5.52029631890808e-05*G0_8 - 0.000623960765743208*G0_9 + 0.000873210508627223*G0_10 - 0.000555375266023444*G0_11 + 0.000110405926378156*G0_12 - 0.000110405926378155*G0_13 + 0.000491808217502688*G0_14 - 0.000341254681532478*G0_15 - 2.0073804796029e-05*G0_16 - 0.000109569517844986*G0_17 + 0.000170627340766235*G0_18 - 0.00026681432208054*G0_19 - 4.01476095920522e-05*G0_20 + 0.000127970505574683*G0_21 + 0.00012797050557468*G0_22 - 4.01476095920563e-05*G0_23 - 0.000341254681532476*G0_24 + 0.000491808217502689*G0_25 - 0.000110405926378159*G0_26 + 0.000110405926378156*G0_27 + 0.00227503121021651*G0_28 - 0.000903321215821265*G0_29 + 0.00051857329056406*G0_30 - 0.00125461279975175*G0_31 + 6.27306399875892e-05*G0_32 + 0.000694219082529305*G0_33 + 0.000694219082529306*G0_34 + 0.000577121887885808*G0_35 + 0.000694219082529301*G0_36 - 0.000213284175957799*G0_37 - 0.000213284175957798*G0_38 + 0.000618942314544196*G0_39 + 0.000518573290564061*G0_40 - 0.000903321215821265*G0_41 + 0.00227503121021652*G0_42 + 6.27306399875887e-05*G0_43 - 0.00125461279975175*G0_44 + 0.000694219082529306*G0_45 + 0.00103714658112812*G0_46 - 0.00180664243164253*G0_47 + 0.00455006242043302*G0_48 + 0.000514391247898221*G0_49 - 0.00180664243164253*G0_50 + 0.00103714658112812*G0_51 + 0.000777859935846078*G0_52 + 0.00140516633572197*G0_53 + 0.000777859935846089*G0_54 - 0.00035129158393049*G0_55;
    A[49] = -0.000121496703527961*G0_0 + 3.98465025201162e-05*G0_1 - 0.00012149670352796*G0_2 - 1.77151327324948e-05*G0_3 - 2.760148159454e-05*G0_4 + 0.000253431785549856*G0_5 - 0.000148044310370708*G0_6 - 8.78228959826196e-06*G0_7 + 2.88560943942899e-05*G0_8 - 2.25830303955313e-05*G0_9 - 0.000122952054375672*G0_10 + 0.00013549818237319*G0_11 - 9.28413471816282e-05*G0_12 - 7.02583167861006e-05*G0_13 - 0.000120442828776166*G0_14 - 5.52029631890779e-05*G0_15 - 2.76014815945391e-05*G0_16 + 0.000253431785549854*G0_17 - 0.000148044310370707*G0_18 - 8.7822895982619e-06*G0_19 + 0.00030863474873893*G0_20 - 4.51660607910641e-05*G0_21 - 4.51660607910635e-05*G0_22 + 0.000308634748738932*G0_23 - 5.52029631890775e-05*G0_24 - 0.000120442828776167*G0_25 - 7.02583167860984e-05*G0_26 - 9.2841347181629e-05*G0_27 + 6.27306399875887e-05*G0_28 - 0.000639852527873392*G0_29 - 0.00125461279975176*G0_30 + 0.000414022223918075*G0_31 + 0.000225830303955317*G0_32 - 0.000213284175957797*G0_33 - 0.000903321215821263*G0_34 + 0.00263468687947869*G0_35 - 0.000903321215821264*G0_36 - 0.000639852527873395*G0_37 - 0.000639852527873396*G0_38 + 0.000577121887885808*G0_39 - 0.00125461279975175*G0_40 - 0.000639852527873395*G0_41 + 6.27306399875895e-05*G0_42 + 0.000225830303955316*G0_43 + 0.000414022223918077*G0_44 - 0.000213284175957799*G0_45 - 0.00180664243164253*G0_46 - 0.000940959599813817*G0_47 + 0.000514391247898221*G0_48 + 0.00526937375895737*G0_49 - 0.000940959599813813*G0_50 - 0.00180664243164253*G0_51 + 0.000677490911865946*G0_52 - 0.00120442828776169*G0_53 + 0.000677490911865947*G0_54 - 0.000639852527873397*G0_55;
    A[50] = 3.98465025201149e-05*G0_0 - 0.00012149670352796*G0_1 - 0.00012149670352796*G0_2 - 1.77151327324951e-05*G0_3 - 2.76014815945394e-05*G0_4 + 0.000253431785549855*G0_5 - 0.000148044310370707*G0_6 - 8.78228959826211e-06*G0_7 - 2.76014815945391e-05*G0_8 + 0.000253431785549853*G0_9 - 0.000148044310370707*G0_10 - 8.78228959826175e-06*G0_11 + 0.000308634748738931*G0_12 - 4.51660607910636e-05*G0_13 - 4.51660607910632e-05*G0_14 + 0.000308634748738932*G0_15 + 2.88560943942908e-05*G0_16 - 2.25830303955325e-05*G0_17 - 0.000122952054375671*G0_18 + 0.00013549818237319*G0_19 - 9.284134718163e-05*G0_20 - 7.02583167861007e-05*G0_21 - 0.000120442828776168*G0_22 - 5.52029631890777e-05*G0_23 - 9.28413471816293e-05*G0_24 - 7.02583167860994e-05*G0_25 - 0.000120442828776167*G0_26 - 5.52029631890776e-05*G0_27 - 0.000903321215821266*G0_28 + 0.00263468687947868*G0_29 - 0.000903321215821264*G0_30 - 0.000639852527873392*G0_31 - 0.000639852527873395*G0_32 + 0.000577121887885805*G0_33 + 6.27306399875859e-05*G0_34 - 0.000639852527873396*G0_35 - 0.00125461279975176*G0_36 + 0.000414022223918078*G0_37 + 0.000225830303955317*G0_38 - 0.000213284175957801*G0_39 + 6.27306399875863e-05*G0_40 - 0.000639852527873393*G0_41 - 0.00125461279975176*G0_42 + 0.000414022223918078*G0_43 + 0.000225830303955318*G0_44 - 0.000213284175957801*G0_45 + 0.000514391247898218*G0_46 - 0.000940959599813816*G0_47 - 0.00180664243164253*G0_48 - 0.000940959599813813*G0_49 + 0.00526937375895736*G0_50 - 0.00180664243164253*G0_51 - 0.00120442828776169*G0_52 + 0.000677490911865943*G0_53 + 0.00067749091186594*G0_54 - 0.000639852527873395*G0_55;
    A[51] = 9.59527869250157e-05*G0_0 + 9.59527869250134e-05*G0_1 + 0.000149650214754389*G0_2 + 0.000173437673437683*G0_3 + 5.52029631890791e-05*G0_4 - 0.000623960765743209*G0_5 + 0.000873210508627221*G0_6 - 0.000555375266023443*G0_7 - 2.00738047960268e-05*G0_8 - 0.000109569517844986*G0_9 + 0.00017062734076624*G0_10 - 0.000266814322080541*G0_11 - 0.000341254681532478*G0_12 + 0.00049180821750269*G0_13 - 0.000110405926378156*G0_14 + 0.000110405926378153*G0_15 - 2.00738047960286e-05*G0_16 - 0.000109569517844986*G0_17 + 0.000170627340766237*G0_18 - 0.000266814322080541*G0_19 - 0.000341254681532473*G0_20 + 0.000491808217502693*G0_21 - 0.000110405926378153*G0_22 + 0.000110405926378154*G0_23 - 4.01476095920548e-05*G0_24 + 0.000127970505574679*G0_25 + 0.000127970505574678*G0_26 - 4.01476095920565e-05*G0_27 + 0.000518573290564061*G0_28 - 0.000903321215821268*G0_29 + 0.00227503121021652*G0_30 + 6.27306399875895e-05*G0_31 - 0.00125461279975176*G0_32 + 0.000694219082529308*G0_33 + 0.000518573290564063*G0_34 - 0.000903321215821265*G0_35 + 0.00227503121021652*G0_36 + 6.27306399875868e-05*G0_37 - 0.00125461279975175*G0_38 + 0.000694219082529306*G0_39 + 0.000694219082529308*G0_40 + 0.000577121887885806*G0_41 + 0.000694219082529308*G0_42 - 0.000213284175957798*G0_43 - 0.000213284175957798*G0_44 + 0.000618942314544203*G0_45 + 0.00103714658112812*G0_46 + 0.000514391247898223*G0_47 + 0.00103714658112812*G0_48 - 0.00180664243164253*G0_49 - 0.00180664243164253*G0_50 + 0.00455006242043303*G0_51 + 0.000777859935846091*G0_52 + 0.000777859935846097*G0_53 + 0.00140516633572197*G0_54 - 0.000351291583930485*G0_55;
    A[52] = -0.000280029576904591*G0_0 + 0.000204250963799585*G0_1 + 0.000204250963799586*G0_2 + 0.000204250963799585*G0_3 - 0.000301107071940417*G0_4 + 6.27306399875843e-05*G0_5 + 6.27306399875863e-05*G0_6 - 0.000301107071940419*G0_7 - 0.000301107071940416*G0_8 + 6.27306399875888e-05*G0_9 + 6.27306399875865e-05*G0_10 - 0.00030110707194042*G0_11 - 0.000301107071940419*G0_12 + 6.27306399875907e-05*G0_13 + 6.27306399875857e-05*G0_14 - 0.000301107071940423*G0_15 + 0.000727675423856022*G0_16 - 0.000903321215821264*G0_17 + 0.00110405926378154*G0_18 - 0.00077785993584609*G0_19 + 0.00072767542385602*G0_20 - 0.000903321215821259*G0_21 + 0.00110405926378155*G0_22 - 0.000777859935846089*G0_23 + 0.000727675423856022*G0_24 - 0.000903321215821262*G0_25 + 0.00110405926378154*G0_26 - 0.000777859935846089*G0_27 - 0.000351291583930501*G0_28 - 0.000639852527873401*G0_29 - 0.000351291583930494*G0_30 - 0.000639852527873386*G0_31 - 0.000639852527873394*G0_32 - 0.000351291583930496*G0_33 + 0.00140516633572197*G0_34 + 0.000677490911865944*G0_35 + 0.000777859935846089*G0_36 + 0.000677490911865943*G0_37 - 0.00120442828776168*G0_38 + 0.000777859935846075*G0_39 + 0.00140516633572197*G0_40 + 0.000677490911865947*G0_41 + 0.000777859935846084*G0_42 + 0.000677490911865946*G0_43 - 0.00120442828776168*G0_44 + 0.000777859935846088*G0_45 + 0.00140516633572196*G0_46 + 0.000677490911865954*G0_47 + 0.000777859935846078*G0_48 + 0.000677490911865946*G0_49 - 0.00120442828776169*G0_50 + 0.000777859935846091*G0_51 + 0.0165608889567232*G0_52 - 0.00225830303955316*G0_53 - 0.00225830303955316*G0_54 - 0.00225830303955315*G0_55;
    A[53] = 0.000204250963799587*G0_0 - 0.000280029576904594*G0_1 + 0.000204250963799587*G0_2 + 0.000204250963799586*G0_3 - 0.000301107071940418*G0_4 + 6.27306399875852e-05*G0_5 + 6.27306399875869e-05*G0_6 - 0.000301107071940421*G0_7 + 0.000727675423856018*G0_8 - 0.000903321215821263*G0_9 + 0.00110405926378155*G0_10 - 0.000777859935846087*G0_11 + 0.000727675423856012*G0_12 - 0.000903321215821257*G0_13 + 0.00110405926378154*G0_14 - 0.000777859935846088*G0_15 - 0.000301107071940419*G0_16 + 6.27306399875872e-05*G0_17 + 6.2730639987582e-05*G0_18 - 0.00030110707194042*G0_19 - 0.000301107071940413*G0_20 + 6.27306399875956e-05*G0_21 + 6.27306399875872e-05*G0_22 - 0.000301107071940421*G0_23 - 0.000777859935846086*G0_24 + 0.00110405926378154*G0_25 - 0.000903321215821262*G0_26 + 0.000727675423856015*G0_27 + 0.00140516633572197*G0_28 + 0.000677490911865942*G0_29 + 0.000777859935846091*G0_30 + 0.000677490911865952*G0_31 - 0.00120442828776168*G0_32 + 0.000777859935846091*G0_33 - 0.000351291583930484*G0_34 - 0.0006398525278734*G0_35 - 0.000351291583930493*G0_36 - 0.000639852527873387*G0_37 - 0.000639852527873392*G0_38 - 0.000351291583930489*G0_39 + 0.00077785993584609*G0_40 + 0.000677490911865943*G0_41 + 0.00140516633572197*G0_42 - 0.00120442828776168*G0_43 + 0.000677490911865945*G0_44 + 0.000777859935846095*G0_45 + 0.000777859935846094*G0_46 + 0.000677490911865951*G0_47 + 0.00140516633572197*G0_48 - 0.00120442828776169*G0_49 + 0.000677490911865943*G0_50 + 0.000777859935846097*G0_51 - 0.00225830303955316*G0_52 + 0.0165608889567231*G0_53 - 0.00225830303955315*G0_54 - 0.00225830303955315*G0_55;
    A[54] = 0.000204250963799587*G0_0 + 0.000204250963799587*G0_1 - 0.000280029576904592*G0_2 + 0.000204250963799586*G0_3 + 0.000727675423856015*G0_4 - 0.000903321215821266*G0_5 + 0.00110405926378154*G0_6 - 0.000777859935846087*G0_7 - 0.000301107071940422*G0_8 + 6.27306399875908e-05*G0_9 + 6.27306399875857e-05*G0_10 - 0.000301107071940422*G0_11 - 0.000777859935846091*G0_12 + 0.00110405926378155*G0_13 - 0.000903321215821266*G0_14 + 0.000727675423856015*G0_15 - 0.000301107071940425*G0_16 + 6.27306399875897e-05*G0_17 + 6.27306399875854e-05*G0_18 - 0.000301107071940422*G0_19 - 0.000777859935846082*G0_20 + 0.00110405926378155*G0_21 - 0.000903321215821257*G0_22 + 0.000727675423856019*G0_23 - 0.00030110707194042*G0_24 + 6.27306399875891e-05*G0_25 + 6.27306399875891e-05*G0_26 - 0.000301107071940423*G0_27 + 0.000777859935846093*G0_28 + 0.000677490911865936*G0_29 + 0.00140516633572198*G0_30 - 0.00120442828776168*G0_31 + 0.000677490911865947*G0_32 + 0.000777859935846095*G0_33 + 0.000777859935846097*G0_34 + 0.000677490911865946*G0_35 + 0.00140516633572198*G0_36 - 0.00120442828776168*G0_37 + 0.000677490911865944*G0_38 + 0.000777859935846093*G0_39 - 0.000351291583930488*G0_40 - 0.000639852527873396*G0_41 - 0.000351291583930483*G0_42 - 0.000639852527873388*G0_43 - 0.000639852527873388*G0_44 - 0.000351291583930489*G0_45 + 0.000777859935846092*G0_46 - 0.00120442828776168*G0_47 + 0.000777859935846089*G0_48 + 0.000677490911865947*G0_49 + 0.000677490911865941*G0_50 + 0.00140516633572197*G0_51 - 0.00225830303955316*G0_52 - 0.00225830303955315*G0_53 + 0.0165608889567232*G0_54 - 0.00225830303955314*G0_55;
    A[55] = 0.000204250963799585*G0_0 + 0.000204250963799583*G0_1 + 0.000204250963799586*G0_2 - 0.000280029576904592*G0_3 - 0.000777859935846086*G0_4 + 0.00110405926378154*G0_5 - 0.000903321215821266*G0_6 + 0.00072767542385602*G0_7 - 0.000777859935846085*G0_8 + 0.00110405926378155*G0_9 - 0.000903321215821261*G0_10 + 0.000727675423856014*G0_11 - 0.000301107071940423*G0_12 + 6.27306399875888e-05*G0_13 + 6.27306399875872e-05*G0_14 - 0.000301107071940422*G0_15 - 0.000777859935846085*G0_16 + 0.00110405926378155*G0_17 - 0.000903321215821265*G0_18 + 0.000727675423856022*G0_19 - 0.000301107071940413*G0_20 + 6.27306399875918e-05*G0_21 + 6.27306399875912e-05*G0_22 - 0.000301107071940421*G0_23 - 0.000301107071940418*G0_24 + 6.27306399875876e-05*G0_25 + 6.27306399875872e-05*G0_26 - 0.000301107071940422*G0_27 + 0.00077785993584609*G0_28 - 0.00120442828776169*G0_29 + 0.00077785993584609*G0_30 + 0.000677490911865946*G0_31 + 0.00067749091186594*G0_32 + 0.00140516633572197*G0_33 + 0.000777859935846097*G0_34 - 0.00120442828776168*G0_35 + 0.000777859935846092*G0_36 + 0.000677490911865941*G0_37 + 0.000677490911865947*G0_38 + 0.00140516633572197*G0_39 + 0.000777859935846086*G0_40 - 0.00120442828776168*G0_41 + 0.00077785993584609*G0_42 + 0.000677490911865946*G0_43 + 0.000677490911865945*G0_44 + 0.00140516633572198*G0_45 - 0.00035129158393049*G0_46 - 0.000639852527873387*G0_47 - 0.00035129158393049*G0_48 - 0.000639852527873398*G0_49 - 0.000639852527873395*G0_50 - 0.000351291583930485*G0_51 - 0.00225830303955315*G0_52 - 0.00225830303955315*G0_53 - 0.00225830303955314*G0_54 + 0.0165608889567231*G0_55;
  }

};

/// This class defines the interface for the assembly of the global
/// tensor corresponding to a form with r + n arguments, that is, a
/// mapping
///
///     a : V1 x V2 x ... Vr x W1 x W2 x ... x Wn -> R
///
/// with arguments v1, v2, ..., vr, w1, w2, ..., wn. The rank r
/// global tensor A is defined by
///
///     A = a(V1, V2, ..., Vr, w1, w2, ..., wn),
///
/// where each argument Vj represents the application to the
/// sequence of basis functions of Vj and w1, w2, ..., wn are given
/// fixed functions (coefficients).

class UFC_Poisson3D_5LinearForm: public ufc::form
{
public:

  /// Constructor
  UFC_Poisson3D_5LinearForm() : ufc::form()
  {
    // Do nothing
  }

  /// Destructor
  virtual ~UFC_Poisson3D_5LinearForm()
  {
    // Do nothing
  }

  /// Return a string identifying the form
  virtual const char* signature() const
  {
    return "w0_a0[0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 19, 20, 21, 22, 23, 24, 25, 26, 27, 28, 29, 30, 31, 32, 33, 34, 35, 36, 37, 38, 39, 40, 41, 42, 43, 44, 45, 46, 47, 48, 49, 50, 51, 52, 53, 54, 55] | vi0[0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 19, 20, 21, 22, 23, 24, 25, 26, 27, 28, 29, 30, 31, 32, 33, 34, 35, 36, 37, 38, 39, 40, 41, 42, 43, 44, 45, 46, 47, 48, 49, 50, 51, 52, 53, 54, 55]*va0[0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 19, 20, 21, 22, 23, 24, 25, 26, 27, 28, 29, 30, 31, 32, 33, 34, 35, 36, 37, 38, 39, 40, 41, 42, 43, 44, 45, 46, 47, 48, 49, 50, 51, 52, 53, 54, 55]*dX(0)";
  }

  /// Return the rank of the global tensor (r)
  virtual unsigned int rank() const
  {
    return 1;
  }

  /// Return the number of coefficients (n)
  virtual unsigned int num_coefficients() const
  {
    return 1;
  }

  /// Return the number of cell integrals
  virtual unsigned int num_cell_integrals() const
  {
    return 1;
  }
  
  /// Return the number of exterior facet integrals
  virtual unsigned int num_exterior_facet_integrals() const
  {
    return 0;
  }
  
  /// Return the number of interior facet integrals
  virtual unsigned int num_interior_facet_integrals() const
  {
    return 0;
  }
    
  /// Create a new finite element for argument function i
  virtual ufc::finite_element* create_finite_element(unsigned int i) const
  {
    switch ( i )
    {
    case 0:
      return new UFC_Poisson3D_5LinearForm_finite_element_0();
      break;
    case 1:
      return new UFC_Poisson3D_5LinearForm_finite_element_1();
      break;
    }
    return 0;
  }
  
  /// Create a new dof map for argument function i
  virtual ufc::dof_map* create_dof_map(unsigned int i) const
  {
    switch ( i )
    {
    case 0:
      return new UFC_Poisson3D_5LinearForm_dof_map_0();
      break;
    case 1:
      return new UFC_Poisson3D_5LinearForm_dof_map_1();
      break;
    }
    return 0;
  }

  /// Create a new cell integral on sub domain i
  virtual ufc::cell_integral* create_cell_integral(unsigned int i) const
  {
    return new UFC_Poisson3D_5LinearForm_cell_integral_0();
  }

  /// Create a new exterior facet integral on sub domain i
  virtual ufc::exterior_facet_integral* create_exterior_facet_integral(unsigned int i) const
  {
    return 0;
  }

  /// Create a new interior facet integral on sub domain i
  virtual ufc::interior_facet_integral* create_interior_facet_integral(unsigned int i) const
  {
    return 0;
  }

};

// DOLFIN wrappers

#include <dolfin/fem/Form.h>

class Poisson3D_5BilinearForm : public dolfin::Form
{
public:

  Poisson3D_5BilinearForm() : dolfin::Form()
  {
    // Do nothing
  }

  /// Return UFC form
  virtual const ufc::form& form() const
  {
    return __form;
  }
  
  /// Return array of coefficients
  virtual const dolfin::Array<dolfin::Function*>& coefficients() const
  {
    return __coefficients;
  }

private:

  // UFC form
  UFC_Poisson3D_5BilinearForm __form;

  /// Array of coefficients
  dolfin::Array<dolfin::Function*> __coefficients;

};

class Poisson3D_5LinearForm : public dolfin::Form
{
public:

  Poisson3D_5LinearForm(dolfin::Function& w0) : dolfin::Form()
  {
    __coefficients.push_back(&w0);
  }

  /// Return UFC form
  virtual const ufc::form& form() const
  {
    return __form;
  }
  
  /// Return array of coefficients
  virtual const dolfin::Array<dolfin::Function*>& coefficients() const
  {
    return __coefficients;
  }

private:

  // UFC form
  UFC_Poisson3D_5LinearForm __form;

  /// Array of coefficients
  dolfin::Array<dolfin::Function*> __coefficients;

};

#endif
